#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

ELF_RELOC(R_XTENSA_NONE,              0)
ELF_RELOC(R_XTENSA_32,                1)
ELF_RELOC(R_XTENSA_RTLD,              2)
ELF_RELOC(R_XTENSA_GLOB_DAT,          3)
ELF_RELOC(R_XTENSA_JMP_SLOT,          4)
ELF_RELOC(R_XTENSA_RELATIVE,          5)
ELF_RELOC(R_XTENSA_PLT,               6)
// RELOC '7' currently is not used.
ELF_RELOC(R_XTENSA_OP0,               8)
ELF_RELOC(R_XTENSA_OP1,               9)
ELF_RELOC(R_XTENSA_OP2,              10)
ELF_RELOC(R_XTENSA_ASM_EXPAND,       11)
ELF_RELOC(R_XTENSA_ASM_SIMPLIFY,     12)
ELF_RELOC(R_XTENSA_32_PCREL,         14)
ELF_RELOC(R_XTENSA_GNU_VTINHERIT,    15)
ELF_RELOC(R_XTENSA_GNU_VTENTRY,      16)
ELF_RELOC(R_XTENSA_DIFF8,            17)
ELF_RELOC(R_XTENSA_DIFF16,           18)
ELF_RELOC(R_XTENSA_DIFF32,           19)
ELF_RELOC(R_XTENSA_SLOT0_OP,         20)
ELF_RELOC(R_XTENSA_SLOT1_OP,         21)
ELF_RELOC(R_XTENSA_SLOT2_OP,         22)
ELF_RELOC(R_XTENSA_SLOT3_OP,         23)
ELF_RELOC(R_XTENSA_SLOT4_OP,         24)
ELF_RELOC(R_XTENSA_SLOT5_OP,         25)
ELF_RELOC(R_XTENSA_SLOT6_OP,         26)
ELF_RELOC(R_XTENSA_SLOT7_OP,         27)
ELF_RELOC(R_XTENSA_SLOT8_OP,         28)
ELF_RELOC(R_XTENSA_SLOT9_OP,         29)
ELF_RELOC(R_XTENSA_SLOT10_OP,        30)
ELF_RELOC(R_XTENSA_SLOT11_OP,        31)
ELF_RELOC(R_XTENSA_SLOT12_OP,        32)
ELF_RELOC(R_XTENSA_SLOT13_OP,        33)
ELF_RELOC(R_XTENSA_SLOT14_OP,        34)
ELF_RELOC(R_XTENSA_SLOT0_ALT,        35)
ELF_RELOC(R_XTENSA_SLOT1_ALT,        36)
ELF_RELOC(R_XTENSA_SLOT2_ALT,        37)
ELF_RELOC(R_XTENSA_SLOT3_ALT,        38)
ELF_RELOC(R_XTENSA_SLOT4_ALT,        39)
ELF_RELOC(R_XTENSA_SLOT5_ALT,        40)
ELF_RELOC(R_XTENSA_SLOT6_ALT,        41)
ELF_RELOC(R_XTENSA_SLOT7_ALT,        42)
ELF_RELOC(R_XTENSA_SLOT8_ALT,        43)
ELF_RELOC(R_XTENSA_SLOT9_ALT,        44)
ELF_RELOC(R_XTENSA_SLOT10_ALT,       45)
ELF_RELOC(R_XTENSA_SLOT11_ALT,       46)
ELF_RELOC(R_XTENSA_SLOT12_ALT,       47)
ELF_RELOC(R_XTENSA_SLOT13_ALT,       48)
ELF_RELOC(R_XTENSA_SLOT14_ALT,       49)
ELF_RELOC(R_XTENSA_TLSDESC_FN,       50)
ELF_RELOC(R_XTENSA_TLSDESC_ARG,      51)
ELF_RELOC(R_XTENSA_TLS_DTPOFF,       52)
ELF_RELOC(R_XTENSA_TLS_TPOFF,        53)
ELF_RELOC(R_XTENSA_TLS_FUNC,         54)
ELF_RELOC(R_XTENSA_TLS_ARG,          55)
ELF_RELOC(R_XTENSA_TLS_CALL,         56)
