/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Intrinsic Function Source Fragment                                         *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifndef LLVM_IR_INTRINSIC_ARM_ENUMS_H
#define LLVM_IR_INTRINSIC_ARM_ENUMS_H
namespace llvm::Intrinsic {
enum ARMIntrinsics : unsigned {
// Enum values for intrinsics.
    arm_cde_cx1 = 3683,                               // llvm.arm.cde.cx1 (IntrinsicsARM.td:1432)
    arm_cde_cx1a,                              // llvm.arm.cde.cx1a (IntrinsicsARM.td:1436)
    arm_cde_cx1d,                              // llvm.arm.cde.cx1d (IntrinsicsARM.td:1442)
    arm_cde_cx1da,                             // llvm.arm.cde.cx1da (IntrinsicsARM.td:1446)
    arm_cde_cx2,                               // llvm.arm.cde.cx2 (IntrinsicsARM.td:1432)
    arm_cde_cx2a,                              // llvm.arm.cde.cx2a (IntrinsicsARM.td:1436)
    arm_cde_cx2d,                              // llvm.arm.cde.cx2d (IntrinsicsARM.td:1442)
    arm_cde_cx2da,                             // llvm.arm.cde.cx2da (IntrinsicsARM.td:1446)
    arm_cde_cx3,                               // llvm.arm.cde.cx3 (IntrinsicsARM.td:1432)
    arm_cde_cx3a,                              // llvm.arm.cde.cx3a (IntrinsicsARM.td:1436)
    arm_cde_cx3d,                              // llvm.arm.cde.cx3d (IntrinsicsARM.td:1442)
    arm_cde_cx3da,                             // llvm.arm.cde.cx3da (IntrinsicsARM.td:1446)
    arm_cde_vcx1,                              // llvm.arm.cde.vcx1 (IntrinsicsARM.td:1458)
    arm_cde_vcx1a,                             // llvm.arm.cde.vcx1a (IntrinsicsARM.td:1462)
    arm_cde_vcx1q,                             // llvm.arm.cde.vcx1q (IntrinsicsARM.td:1474)
    arm_cde_vcx1q_predicated,                  // llvm.arm.cde.vcx1q.predicated (IntrinsicsARM.td:1484)
    arm_cde_vcx1qa,                            // llvm.arm.cde.vcx1qa (IntrinsicsARM.td:1478)
    arm_cde_vcx1qa_predicated,                 // llvm.arm.cde.vcx1qa.predicated (IntrinsicsARM.td:1489)
    arm_cde_vcx2,                              // llvm.arm.cde.vcx2 (IntrinsicsARM.td:1458)
    arm_cde_vcx2a,                             // llvm.arm.cde.vcx2a (IntrinsicsARM.td:1462)
    arm_cde_vcx2q,                             // llvm.arm.cde.vcx2q (IntrinsicsARM.td:1474)
    arm_cde_vcx2q_predicated,                  // llvm.arm.cde.vcx2q.predicated (IntrinsicsARM.td:1484)
    arm_cde_vcx2qa,                            // llvm.arm.cde.vcx2qa (IntrinsicsARM.td:1478)
    arm_cde_vcx2qa_predicated,                 // llvm.arm.cde.vcx2qa.predicated (IntrinsicsARM.td:1489)
    arm_cde_vcx3,                              // llvm.arm.cde.vcx3 (IntrinsicsARM.td:1458)
    arm_cde_vcx3a,                             // llvm.arm.cde.vcx3a (IntrinsicsARM.td:1462)
    arm_cde_vcx3q,                             // llvm.arm.cde.vcx3q (IntrinsicsARM.td:1474)
    arm_cde_vcx3q_predicated,                  // llvm.arm.cde.vcx3q.predicated (IntrinsicsARM.td:1484)
    arm_cde_vcx3qa,                            // llvm.arm.cde.vcx3qa (IntrinsicsARM.td:1478)
    arm_cde_vcx3qa_predicated,                 // llvm.arm.cde.vcx3qa.predicated (IntrinsicsARM.td:1489)
    arm_cdp,                                   // llvm.arm.cdp (IntrinsicsARM.td:363)
    arm_cdp2,                                  // llvm.arm.cdp2 (IntrinsicsARM.td:366)
    arm_clrex,                                 // llvm.arm.clrex (IntrinsicsARM.td:289)
    arm_cls,                                   // llvm.arm.cls (IntrinsicsARM.td:861)
    arm_cls64,                                 // llvm.arm.cls64 (IntrinsicsARM.td:863)
    arm_cmse_tt,                               // llvm.arm.cmse.tt (IntrinsicsARM.td:401)
    arm_cmse_tta,                              // llvm.arm.cmse.tta (IntrinsicsARM.td:405)
    arm_cmse_ttat,                             // llvm.arm.cmse.ttat (IntrinsicsARM.td:407)
    arm_cmse_ttt,                              // llvm.arm.cmse.ttt (IntrinsicsARM.td:403)
    arm_crc32b,                                // llvm.arm.crc32b (IntrinsicsARM.td:384)
    arm_crc32cb,                               // llvm.arm.crc32cb (IntrinsicsARM.td:386)
    arm_crc32ch,                               // llvm.arm.crc32ch (IntrinsicsARM.td:390)
    arm_crc32cw,                               // llvm.arm.crc32cw (IntrinsicsARM.td:394)
    arm_crc32h,                                // llvm.arm.crc32h (IntrinsicsARM.td:388)
    arm_crc32w,                                // llvm.arm.crc32w (IntrinsicsARM.td:392)
    arm_dbg,                                   // llvm.arm.dbg (IntrinsicsARM.td:415)
    arm_dmb,                                   // llvm.arm.dmb (IntrinsicsARM.td:303)
    arm_dsb,                                   // llvm.arm.dsb (IntrinsicsARM.td:305)
    arm_get_fpscr,                             // llvm.arm.get.fpscr (IntrinsicsARM.td:313)
    arm_gnu_eabi_mcount,                       // llvm.arm.gnu.eabi.mcount (IntrinsicsARM.td:881)
    arm_hint,                                  // llvm.arm.hint (IntrinsicsARM.td:414)
    arm_isb,                                   // llvm.arm.isb (IntrinsicsARM.td:307)
    arm_ldaex,                                 // llvm.arm.ldaex (IntrinsicsARM.td:286)
    arm_ldaexd,                                // llvm.arm.ldaexd (IntrinsicsARM.td:297)
    arm_ldc,                                   // llvm.arm.ldc (IntrinsicsARM.td:326)
    arm_ldc2,                                  // llvm.arm.ldc2 (IntrinsicsARM.td:330)
    arm_ldc2l,                                 // llvm.arm.ldc2l (IntrinsicsARM.td:332)
    arm_ldcl,                                  // llvm.arm.ldcl (IntrinsicsARM.td:328)
    arm_ldrex,                                 // llvm.arm.ldrex (IntrinsicsARM.td:283)
    arm_ldrexd,                                // llvm.arm.ldrexd (IntrinsicsARM.td:293)
    arm_mcr,                                   // llvm.arm.mcr (IntrinsicsARM.td:345)
    arm_mcr2,                                  // llvm.arm.mcr2 (IntrinsicsARM.td:348)
    arm_mcrr,                                  // llvm.arm.mcrr (IntrinsicsARM.td:371)
    arm_mcrr2,                                 // llvm.arm.mcrr2 (IntrinsicsARM.td:373)
    arm_mrc,                                   // llvm.arm.mrc (IntrinsicsARM.td:353)
    arm_mrc2,                                  // llvm.arm.mrc2 (IntrinsicsARM.td:357)
    arm_mrrc,                                  // llvm.arm.mrrc (IntrinsicsARM.td:376)
    arm_mrrc2,                                 // llvm.arm.mrrc2 (IntrinsicsARM.td:378)
    arm_mve_abd_predicated,                    // llvm.arm.mve.abd.predicated (IntrinsicsARM.td:898)
    arm_mve_abs_predicated,                    // llvm.arm.mve.abs.predicated (IntrinsicsARM.td:1347)
    arm_mve_add_predicated,                    // llvm.arm.mve.add.predicated (IntrinsicsARM.td:901)
    arm_mve_addlv,                             // llvm.arm.mve.addlv (IntrinsicsARM.td:986)
    arm_mve_addlv_predicated,                  // llvm.arm.mve.addlv.predicated (IntrinsicsARM.td:987)
    arm_mve_addv,                              // llvm.arm.mve.addv (IntrinsicsARM.td:986)
    arm_mve_addv_predicated,                   // llvm.arm.mve.addv.predicated (IntrinsicsARM.td:987)
    arm_mve_and_predicated,                    // llvm.arm.mve.and.predicated (IntrinsicsARM.td:904)
    arm_mve_asrl,                              // llvm.arm.mve.asrl (IntrinsicsARM.td:1120)
    arm_mve_bic_predicated,                    // llvm.arm.mve.bic.predicated (IntrinsicsARM.td:907)
    arm_mve_cls_predicated,                    // llvm.arm.mve.cls.predicated (IntrinsicsARM.td:1352)
    arm_mve_clz_predicated,                    // llvm.arm.mve.clz.predicated (IntrinsicsARM.td:1351)
    arm_mve_cmp_eq,                            // llvm.arm.mve.cmp.eq (IntrinsicsARM.td:1422)
    arm_mve_cmp_ge,                            // llvm.arm.mve.cmp.ge (IntrinsicsARM.td:1425)
    arm_mve_cmp_gt,                            // llvm.arm.mve.cmp.gt (IntrinsicsARM.td:1424)
    arm_mve_cmp_le,                            // llvm.arm.mve.cmp.le (IntrinsicsARM.td:1427)
    arm_mve_cmp_lt,                            // llvm.arm.mve.cmp.lt (IntrinsicsARM.td:1426)
    arm_mve_cmp_ne,                            // llvm.arm.mve.cmp.ne (IntrinsicsARM.td:1423)
    arm_mve_eor_predicated,                    // llvm.arm.mve.eor.predicated (IntrinsicsARM.td:910)
    arm_mve_fma,                               // llvm.arm.mve.fma (IntrinsicsARM.td:1386)
    arm_mve_fma_predicated,                    // llvm.arm.mve.fma.predicated (IntrinsicsARM.td:1390)
    arm_mve_hadd_predicated,                   // llvm.arm.mve.hadd.predicated (IntrinsicsARM.td:950)
    arm_mve_hsub_predicated,                   // llvm.arm.mve.hsub.predicated (IntrinsicsARM.td:959)
    arm_mve_lsll,                              // llvm.arm.mve.lsll (IntrinsicsARM.td:1119)
    arm_mve_max_predicated,                    // llvm.arm.mve.max.predicated (IntrinsicsARM.td:894)
    arm_mve_maxav,                             // llvm.arm.mve.maxav (IntrinsicsARM.td:986)
    arm_mve_maxav_predicated,                  // llvm.arm.mve.maxav.predicated (IntrinsicsARM.td:987)
    arm_mve_maxnmav,                           // llvm.arm.mve.maxnmav (IntrinsicsARM.td:986)
    arm_mve_maxnmav_predicated,                // llvm.arm.mve.maxnmav.predicated (IntrinsicsARM.td:987)
    arm_mve_maxnmv,                            // llvm.arm.mve.maxnmv (IntrinsicsARM.td:986)
    arm_mve_maxnmv_predicated,                 // llvm.arm.mve.maxnmv.predicated (IntrinsicsARM.td:987)
    arm_mve_maxv,                              // llvm.arm.mve.maxv (IntrinsicsARM.td:986)
    arm_mve_maxv_predicated,                   // llvm.arm.mve.maxv.predicated (IntrinsicsARM.td:987)
    arm_mve_min_predicated,                    // llvm.arm.mve.min.predicated (IntrinsicsARM.td:890)
    arm_mve_minav,                             // llvm.arm.mve.minav (IntrinsicsARM.td:986)
    arm_mve_minav_predicated,                  // llvm.arm.mve.minav.predicated (IntrinsicsARM.td:987)
    arm_mve_minnmav,                           // llvm.arm.mve.minnmav (IntrinsicsARM.td:986)
    arm_mve_minnmav_predicated,                // llvm.arm.mve.minnmav.predicated (IntrinsicsARM.td:987)
    arm_mve_minnmv,                            // llvm.arm.mve.minnmv (IntrinsicsARM.td:986)
    arm_mve_minnmv_predicated,                 // llvm.arm.mve.minnmv.predicated (IntrinsicsARM.td:987)
    arm_mve_minv,                              // llvm.arm.mve.minv (IntrinsicsARM.td:986)
    arm_mve_minv_predicated,                   // llvm.arm.mve.minv.predicated (IntrinsicsARM.td:987)
    arm_mve_mul_predicated,                    // llvm.arm.mve.mul.predicated (IntrinsicsARM.td:922)
    arm_mve_mulh_predicated,                   // llvm.arm.mve.mulh.predicated (IntrinsicsARM.td:925)
    arm_mve_mull_int_predicated,               // llvm.arm.mve.mull.int.predicated (IntrinsicsARM.td:939)
    arm_mve_mull_poly_predicated,              // llvm.arm.mve.mull.poly.predicated (IntrinsicsARM.td:943)
    arm_mve_mvn_predicated,                    // llvm.arm.mve.mvn.predicated (IntrinsicsARM.td:1346)
    arm_mve_neg_predicated,                    // llvm.arm.mve.neg.predicated (IntrinsicsARM.td:1348)
    arm_mve_orn_predicated,                    // llvm.arm.mve.orn.predicated (IntrinsicsARM.td:913)
    arm_mve_orr_predicated,                    // llvm.arm.mve.orr.predicated (IntrinsicsARM.td:916)
    arm_mve_pred_i2v,                          // llvm.arm.mve.pred.i2v (IntrinsicsARM.td:883)
    arm_mve_pred_v2i,                          // llvm.arm.mve.pred.v2i (IntrinsicsARM.td:885)
    arm_mve_qabs_predicated,                   // llvm.arm.mve.qabs.predicated (IntrinsicsARM.td:1349)
    arm_mve_qadd_predicated,                   // llvm.arm.mve.qadd.predicated (IntrinsicsARM.td:947)
    arm_mve_qdmulh_predicated,                 // llvm.arm.mve.qdmulh.predicated (IntrinsicsARM.td:929)
    arm_mve_qneg_predicated,                   // llvm.arm.mve.qneg.predicated (IntrinsicsARM.td:1350)
    arm_mve_qrdmulh_predicated,                // llvm.arm.mve.qrdmulh.predicated (IntrinsicsARM.td:936)
    arm_mve_qsub_predicated,                   // llvm.arm.mve.qsub.predicated (IntrinsicsARM.td:956)
    arm_mve_rhadd_predicated,                  // llvm.arm.mve.rhadd.predicated (IntrinsicsARM.td:953)
    arm_mve_rmulh_predicated,                  // llvm.arm.mve.rmulh.predicated (IntrinsicsARM.td:932)
    arm_mve_shl_imm_predicated,                // llvm.arm.mve.shl.imm.predicated (IntrinsicsARM.td:1066)
    arm_mve_shr_imm_predicated,                // llvm.arm.mve.shr.imm.predicated (IntrinsicsARM.td:1069)
    arm_mve_sqrshr,                            // llvm.arm.mve.sqrshr (IntrinsicsARM.td:1109)
    arm_mve_sqrshrl,                           // llvm.arm.mve.sqrshrl (IntrinsicsARM.td:1110)
    arm_mve_sqshl,                             // llvm.arm.mve.sqshl (IntrinsicsARM.td:1109)
    arm_mve_sqshll,                            // llvm.arm.mve.sqshll (IntrinsicsARM.td:1110)
    arm_mve_srshr,                             // llvm.arm.mve.srshr (IntrinsicsARM.td:1109)
    arm_mve_srshrl,                            // llvm.arm.mve.srshrl (IntrinsicsARM.td:1110)
    arm_mve_sub_predicated,                    // llvm.arm.mve.sub.predicated (IntrinsicsARM.td:919)
    arm_mve_uqrshl,                            // llvm.arm.mve.uqrshl (IntrinsicsARM.td:1109)
    arm_mve_uqrshll,                           // llvm.arm.mve.uqrshll (IntrinsicsARM.td:1110)
    arm_mve_uqshl,                             // llvm.arm.mve.uqshl (IntrinsicsARM.td:1109)
    arm_mve_uqshll,                            // llvm.arm.mve.uqshll (IntrinsicsARM.td:1110)
    arm_mve_urshr,                             // llvm.arm.mve.urshr (IntrinsicsARM.td:1109)
    arm_mve_urshrl,                            // llvm.arm.mve.urshrl (IntrinsicsARM.td:1110)
    arm_mve_vabav,                             // llvm.arm.mve.vabav (IntrinsicsARM.td:986)
    arm_mve_vabav_predicated,                  // llvm.arm.mve.vabav.predicated (IntrinsicsARM.td:987)
    arm_mve_vabd,                              // llvm.arm.mve.vabd (IntrinsicsARM.td:1122)
    arm_mve_vadc,                              // llvm.arm.mve.vadc (IntrinsicsARM.td:1126)
    arm_mve_vadc_predicated,                   // llvm.arm.mve.vadc.predicated (IntrinsicsARM.td:1132)
    arm_mve_vadd,                              // llvm.arm.mve.vadd (IntrinsicsARM.td:1148)
    arm_mve_vbrsr,                             // llvm.arm.mve.vbrsr (IntrinsicsARM.td:1024)
    arm_mve_vbrsr_predicated,                  // llvm.arm.mve.vbrsr.predicated (IntrinsicsARM.td:1025)
    arm_mve_vcaddq,                            // llvm.arm.mve.vcaddq (IntrinsicsARM.td:1024)
    arm_mve_vcaddq_predicated,                 // llvm.arm.mve.vcaddq.predicated (IntrinsicsARM.td:1025)
    arm_mve_vcls,                              // llvm.arm.mve.vcls (IntrinsicsARM.td:1324)
    arm_mve_vcmlaq,                            // llvm.arm.mve.vcmlaq (IntrinsicsARM.td:986)
    arm_mve_vcmlaq_predicated,                 // llvm.arm.mve.vcmlaq.predicated (IntrinsicsARM.td:987)
    arm_mve_vcmulq,                            // llvm.arm.mve.vcmulq (IntrinsicsARM.td:1024)
    arm_mve_vcmulq_predicated,                 // llvm.arm.mve.vcmulq.predicated (IntrinsicsARM.td:1025)
    arm_mve_vctp16,                            // llvm.arm.mve.vctp16 (IntrinsicsARM.td:868)
    arm_mve_vctp32,                            // llvm.arm.mve.vctp32 (IntrinsicsARM.td:870)
    arm_mve_vctp64,                            // llvm.arm.mve.vctp64 (IntrinsicsARM.td:872)
    arm_mve_vctp8,                             // llvm.arm.mve.vctp8 (IntrinsicsARM.td:866)
    arm_mve_vcvt_fix,                          // llvm.arm.mve.vcvt.fix (IntrinsicsARM.td:1024)
    arm_mve_vcvt_fix_predicated,               // llvm.arm.mve.vcvt.fix.predicated (IntrinsicsARM.td:1025)
    arm_mve_vcvt_fp_int,                       // llvm.arm.mve.vcvt.fp.int (IntrinsicsARM.td:1307)
    arm_mve_vcvt_fp_int_predicated,            // llvm.arm.mve.vcvt.fp.int.predicated (IntrinsicsARM.td:1313)
    arm_mve_vcvt_int_fp,                       // llvm.arm.mve.vcvt.int.fp (IntrinsicsARM.td:1310)
    arm_mve_vcvt_narrow,                       // llvm.arm.mve.vcvt.narrow (IntrinsicsARM.td:986)
    arm_mve_vcvt_narrow_predicated,            // llvm.arm.mve.vcvt.narrow.predicated (IntrinsicsARM.td:987)
    arm_mve_vcvt_widen,                        // llvm.arm.mve.vcvt.widen (IntrinsicsARM.td:1024)
    arm_mve_vcvt_widen_predicated,             // llvm.arm.mve.vcvt.widen.predicated (IntrinsicsARM.td:1025)
    arm_mve_vcvta,                             // llvm.arm.mve.vcvta (IntrinsicsARM.td:1024)
    arm_mve_vcvta_predicated,                  // llvm.arm.mve.vcvta.predicated (IntrinsicsARM.td:1025)
    arm_mve_vcvtm,                             // llvm.arm.mve.vcvtm (IntrinsicsARM.td:1024)
    arm_mve_vcvtm_predicated,                  // llvm.arm.mve.vcvtm.predicated (IntrinsicsARM.td:1025)
    arm_mve_vcvtn,                             // llvm.arm.mve.vcvtn (IntrinsicsARM.td:1024)
    arm_mve_vcvtn_predicated,                  // llvm.arm.mve.vcvtn.predicated (IntrinsicsARM.td:1025)
    arm_mve_vcvtp,                             // llvm.arm.mve.vcvtp (IntrinsicsARM.td:1024)
    arm_mve_vcvtp_predicated,                  // llvm.arm.mve.vcvtp.predicated (IntrinsicsARM.td:1025)
    arm_mve_vddup,                             // llvm.arm.mve.vddup (IntrinsicsARM.td:1024)
    arm_mve_vddup_predicated,                  // llvm.arm.mve.vddup.predicated (IntrinsicsARM.td:1025)
    arm_mve_vdwdup,                            // llvm.arm.mve.vdwdup (IntrinsicsARM.td:1024)
    arm_mve_vdwdup_predicated,                 // llvm.arm.mve.vdwdup.predicated (IntrinsicsARM.td:1025)
    arm_mve_vhadd,                             // llvm.arm.mve.vhadd (IntrinsicsARM.td:1167)
    arm_mve_vhsub,                             // llvm.arm.mve.vhsub (IntrinsicsARM.td:1175)
    arm_mve_vidup,                             // llvm.arm.mve.vidup (IntrinsicsARM.td:1024)
    arm_mve_vidup_predicated,                  // llvm.arm.mve.vidup.predicated (IntrinsicsARM.td:1025)
    arm_mve_viwdup,                            // llvm.arm.mve.viwdup (IntrinsicsARM.td:1024)
    arm_mve_viwdup_predicated,                 // llvm.arm.mve.viwdup.predicated (IntrinsicsARM.td:1025)
    arm_mve_vld2q,                             // llvm.arm.mve.vld2q (IntrinsicsARM.td:1220)
    arm_mve_vld4q,                             // llvm.arm.mve.vld4q (IntrinsicsARM.td:1223)
    arm_mve_vldr_gather_base,                  // llvm.arm.mve.vldr.gather.base (IntrinsicsARM.td:986)
    arm_mve_vldr_gather_base_predicated,       // llvm.arm.mve.vldr.gather.base.predicated (IntrinsicsARM.td:987)
    arm_mve_vldr_gather_base_wb,               // llvm.arm.mve.vldr.gather.base.wb (IntrinsicsARM.td:986)
    arm_mve_vldr_gather_base_wb_predicated,    // llvm.arm.mve.vldr.gather.base.wb.predicated (IntrinsicsARM.td:987)
    arm_mve_vldr_gather_offset,                // llvm.arm.mve.vldr.gather.offset (IntrinsicsARM.td:986)
    arm_mve_vldr_gather_offset_predicated,     // llvm.arm.mve.vldr.gather.offset.predicated (IntrinsicsARM.td:987)
    arm_mve_vmaxa_predicated,                  // llvm.arm.mve.vmaxa.predicated (IntrinsicsARM.td:965)
    arm_mve_vmaxnm,                            // llvm.arm.mve.vmaxnm (IntrinsicsARM.td:978)
    arm_mve_vmaxnma_predicated,                // llvm.arm.mve.vmaxnma.predicated (IntrinsicsARM.td:971)
    arm_mve_vmina_predicated,                  // llvm.arm.mve.vmina.predicated (IntrinsicsARM.td:962)
    arm_mve_vminnm,                            // llvm.arm.mve.vminnm (IntrinsicsARM.td:975)
    arm_mve_vminnma_predicated,                // llvm.arm.mve.vminnma.predicated (IntrinsicsARM.td:968)
    arm_mve_vmla_n_predicated,                 // llvm.arm.mve.vmla.n.predicated (IntrinsicsARM.td:1393)
    arm_mve_vmlas_n_predicated,                // llvm.arm.mve.vmlas.n.predicated (IntrinsicsARM.td:1397)
    arm_mve_vmldava,                           // llvm.arm.mve.vmldava (IntrinsicsARM.td:986)
    arm_mve_vmldava_predicated,                // llvm.arm.mve.vmldava.predicated (IntrinsicsARM.td:987)
    arm_mve_vmlldava,                          // llvm.arm.mve.vmlldava (IntrinsicsARM.td:986)
    arm_mve_vmlldava_predicated,               // llvm.arm.mve.vmlldava.predicated (IntrinsicsARM.td:987)
    arm_mve_vmovl_predicated,                  // llvm.arm.mve.vmovl.predicated (IntrinsicsARM.td:1370)
    arm_mve_vmovn_predicated,                  // llvm.arm.mve.vmovn.predicated (IntrinsicsARM.td:1373)
    arm_mve_vmul,                              // llvm.arm.mve.vmul (IntrinsicsARM.td:1156)
    arm_mve_vmulh,                             // llvm.arm.mve.vmulh (IntrinsicsARM.td:1160)
    arm_mve_vmull,                             // llvm.arm.mve.vmull (IntrinsicsARM.td:1186)
    arm_mve_vmull_poly,                        // llvm.arm.mve.vmull.poly (IntrinsicsARM.td:1190)
    arm_mve_vqdmlad,                           // llvm.arm.mve.vqdmlad (IntrinsicsARM.td:986)
    arm_mve_vqdmlad_predicated,                // llvm.arm.mve.vqdmlad.predicated (IntrinsicsARM.td:987)
    arm_mve_vqdmlah,                           // llvm.arm.mve.vqdmlah (IntrinsicsARM.td:986)
    arm_mve_vqdmlah_predicated,                // llvm.arm.mve.vqdmlah.predicated (IntrinsicsARM.td:987)
    arm_mve_vqdmlash,                          // llvm.arm.mve.vqdmlash (IntrinsicsARM.td:986)
    arm_mve_vqdmlash_predicated,               // llvm.arm.mve.vqdmlash.predicated (IntrinsicsARM.td:987)
    arm_mve_vqdmulh,                           // llvm.arm.mve.vqdmulh (IntrinsicsARM.td:1164)
    arm_mve_vqdmull,                           // llvm.arm.mve.vqdmull (IntrinsicsARM.td:1331)
    arm_mve_vqdmull_predicated,                // llvm.arm.mve.vqdmull.predicated (IntrinsicsARM.td:1335)
    arm_mve_vqmovn,                            // llvm.arm.mve.vqmovn (IntrinsicsARM.td:1377)
    arm_mve_vqmovn_predicated,                 // llvm.arm.mve.vqmovn.predicated (IntrinsicsARM.td:1381)
    arm_mve_vqrdmlah,                          // llvm.arm.mve.vqrdmlah (IntrinsicsARM.td:986)
    arm_mve_vqrdmlah_predicated,               // llvm.arm.mve.vqrdmlah.predicated (IntrinsicsARM.td:987)
    arm_mve_vqrdmlash,                         // llvm.arm.mve.vqrdmlash (IntrinsicsARM.td:986)
    arm_mve_vqrdmlash_predicated,              // llvm.arm.mve.vqrdmlash.predicated (IntrinsicsARM.td:987)
    arm_mve_vqrdmulh,                          // llvm.arm.mve.vqrdmulh (IntrinsicsARM.td:1183)
    arm_mve_vqshl_imm,                         // llvm.arm.mve.vqshl.imm (IntrinsicsARM.td:993)
    arm_mve_vqshl_imm_predicated,              // llvm.arm.mve.vqshl.imm.predicated (IntrinsicsARM.td:994)
    arm_mve_vqshlu_imm,                        // llvm.arm.mve.vqshlu.imm (IntrinsicsARM.td:993)
    arm_mve_vqshlu_imm_predicated,             // llvm.arm.mve.vqshlu.imm.predicated (IntrinsicsARM.td:994)
    arm_mve_vreinterpretq,                     // llvm.arm.mve.vreinterpretq (IntrinsicsARM.td:887)
    arm_mve_vrev_predicated,                   // llvm.arm.mve.vrev.predicated (IntrinsicsARM.td:1366)
    arm_mve_vrhadd,                            // llvm.arm.mve.vrhadd (IntrinsicsARM.td:1171)
    arm_mve_vrinta,                            // llvm.arm.mve.vrinta (IntrinsicsARM.td:1356)
    arm_mve_vrinta_predicated,                 // llvm.arm.mve.vrinta.predicated (IntrinsicsARM.td:1362)
    arm_mve_vrintm,                            // llvm.arm.mve.vrintm (IntrinsicsARM.td:1354)
    arm_mve_vrintm_predicated,                 // llvm.arm.mve.vrintm.predicated (IntrinsicsARM.td:1360)
    arm_mve_vrintn,                            // llvm.arm.mve.vrintn (IntrinsicsARM.td:1358)
    arm_mve_vrintn_predicated,                 // llvm.arm.mve.vrintn.predicated (IntrinsicsARM.td:1364)
    arm_mve_vrintp,                            // llvm.arm.mve.vrintp (IntrinsicsARM.td:1355)
    arm_mve_vrintp_predicated,                 // llvm.arm.mve.vrintp.predicated (IntrinsicsARM.td:1361)
    arm_mve_vrintx,                            // llvm.arm.mve.vrintx (IntrinsicsARM.td:1357)
    arm_mve_vrintx_predicated,                 // llvm.arm.mve.vrintx.predicated (IntrinsicsARM.td:1363)
    arm_mve_vrintz,                            // llvm.arm.mve.vrintz (IntrinsicsARM.td:1353)
    arm_mve_vrintz_predicated,                 // llvm.arm.mve.vrintz.predicated (IntrinsicsARM.td:1359)
    arm_mve_vrmlldavha,                        // llvm.arm.mve.vrmlldavha (IntrinsicsARM.td:986)
    arm_mve_vrmlldavha_predicated,             // llvm.arm.mve.vrmlldavha.predicated (IntrinsicsARM.td:987)
    arm_mve_vrmulh,                            // llvm.arm.mve.vrmulh (IntrinsicsARM.td:1179)
    arm_mve_vrshr_imm,                         // llvm.arm.mve.vrshr.imm (IntrinsicsARM.td:993)
    arm_mve_vrshr_imm_predicated,              // llvm.arm.mve.vrshr.imm.predicated (IntrinsicsARM.td:994)
    arm_mve_vsbc,                              // llvm.arm.mve.vsbc (IntrinsicsARM.td:1129)
    arm_mve_vsbc_predicated,                   // llvm.arm.mve.vsbc.predicated (IntrinsicsARM.td:1136)
    arm_mve_vshl_scalar,                       // llvm.arm.mve.vshl.scalar (IntrinsicsARM.td:986)
    arm_mve_vshl_scalar_predicated,            // llvm.arm.mve.vshl.scalar.predicated (IntrinsicsARM.td:987)
    arm_mve_vshl_vector,                       // llvm.arm.mve.vshl.vector (IntrinsicsARM.td:993)
    arm_mve_vshl_vector_predicated,            // llvm.arm.mve.vshl.vector.predicated (IntrinsicsARM.td:994)
    arm_mve_vshlc,                             // llvm.arm.mve.vshlc (IntrinsicsARM.td:1140)
    arm_mve_vshlc_predicated,                  // llvm.arm.mve.vshlc.predicated (IntrinsicsARM.td:1144)
    arm_mve_vshll_imm,                         // llvm.arm.mve.vshll.imm (IntrinsicsARM.td:993)
    arm_mve_vshll_imm_predicated,              // llvm.arm.mve.vshll.imm.predicated (IntrinsicsARM.td:994)
    arm_mve_vshrn,                             // llvm.arm.mve.vshrn (IntrinsicsARM.td:986)
    arm_mve_vshrn_predicated,                  // llvm.arm.mve.vshrn.predicated (IntrinsicsARM.td:987)
    arm_mve_vsli,                              // llvm.arm.mve.vsli (IntrinsicsARM.td:986)
    arm_mve_vsli_predicated,                   // llvm.arm.mve.vsli.predicated (IntrinsicsARM.td:987)
    arm_mve_vsri,                              // llvm.arm.mve.vsri (IntrinsicsARM.td:986)
    arm_mve_vsri_predicated,                   // llvm.arm.mve.vsri.predicated (IntrinsicsARM.td:987)
    arm_mve_vst2q,                             // llvm.arm.mve.vst2q (IntrinsicsARM.td:1227)
    arm_mve_vst4q,                             // llvm.arm.mve.vst4q (IntrinsicsARM.td:1230)
    arm_mve_vstr_scatter_base,                 // llvm.arm.mve.vstr.scatter.base (IntrinsicsARM.td:986)
    arm_mve_vstr_scatter_base_predicated,      // llvm.arm.mve.vstr.scatter.base.predicated (IntrinsicsARM.td:987)
    arm_mve_vstr_scatter_base_wb,              // llvm.arm.mve.vstr.scatter.base.wb (IntrinsicsARM.td:986)
    arm_mve_vstr_scatter_base_wb_predicated,   // llvm.arm.mve.vstr.scatter.base.wb.predicated (IntrinsicsARM.td:987)
    arm_mve_vstr_scatter_offset,               // llvm.arm.mve.vstr.scatter.offset (IntrinsicsARM.td:986)
    arm_mve_vstr_scatter_offset_predicated,    // llvm.arm.mve.vstr.scatter.offset.predicated (IntrinsicsARM.td:987)
    arm_mve_vsub,                              // llvm.arm.mve.vsub (IntrinsicsARM.td:1152)
    arm_neon_aesd,                             // llvm.arm.neon.aesd (IntrinsicsARM.td:804)
    arm_neon_aese,                             // llvm.arm.neon.aese (IntrinsicsARM.td:805)
    arm_neon_aesimc,                           // llvm.arm.neon.aesimc (IntrinsicsARM.td:806)
    arm_neon_aesmc,                            // llvm.arm.neon.aesmc (IntrinsicsARM.td:807)
    arm_neon_bfdot,                            // llvm.arm.neon.bfdot (IntrinsicsARM.td:848)
    arm_neon_bfmlalb,                          // llvm.arm.neon.bfmlalb (IntrinsicsARM.td:858)
    arm_neon_bfmlalt,                          // llvm.arm.neon.bfmlalt (IntrinsicsARM.td:859)
    arm_neon_bfmmla,                           // llvm.arm.neon.bfmmla (IntrinsicsARM.td:849)
    arm_neon_sdot,                             // llvm.arm.neon.sdot (IntrinsicsARM.td:829)
    arm_neon_sha1c,                            // llvm.arm.neon.sha1c (IntrinsicsARM.td:811)
    arm_neon_sha1h,                            // llvm.arm.neon.sha1h (IntrinsicsARM.td:808)
    arm_neon_sha1m,                            // llvm.arm.neon.sha1m (IntrinsicsARM.td:812)
    arm_neon_sha1p,                            // llvm.arm.neon.sha1p (IntrinsicsARM.td:813)
    arm_neon_sha1su0,                          // llvm.arm.neon.sha1su0 (IntrinsicsARM.td:814)
    arm_neon_sha1su1,                          // llvm.arm.neon.sha1su1 (IntrinsicsARM.td:809)
    arm_neon_sha256h,                          // llvm.arm.neon.sha256h (IntrinsicsARM.td:815)
    arm_neon_sha256h2,                         // llvm.arm.neon.sha256h2 (IntrinsicsARM.td:816)
    arm_neon_sha256su0,                        // llvm.arm.neon.sha256su0 (IntrinsicsARM.td:810)
    arm_neon_sha256su1,                        // llvm.arm.neon.sha256su1 (IntrinsicsARM.td:817)
    arm_neon_smmla,                            // llvm.arm.neon.smmla (IntrinsicsARM.td:838)
    arm_neon_udot,                             // llvm.arm.neon.udot (IntrinsicsARM.td:828)
    arm_neon_ummla,                            // llvm.arm.neon.ummla (IntrinsicsARM.td:837)
    arm_neon_usdot,                            // llvm.arm.neon.usdot (IntrinsicsARM.td:840)
    arm_neon_usmmla,                           // llvm.arm.neon.usmmla (IntrinsicsARM.td:839)
    arm_neon_vabds,                            // llvm.arm.neon.vabds (IntrinsicsARM.td:541)
    arm_neon_vabdu,                            // llvm.arm.neon.vabdu (IntrinsicsARM.td:542)
    arm_neon_vabs,                             // llvm.arm.neon.vabs (IntrinsicsARM.td:619)
    arm_neon_vacge,                            // llvm.arm.neon.vacge (IntrinsicsARM.td:537)
    arm_neon_vacgt,                            // llvm.arm.neon.vacgt (IntrinsicsARM.td:538)
    arm_neon_vbsl,                             // llvm.arm.neon.vbsl (IntrinsicsARM.td:784)
    arm_neon_vcadd_rot270,                     // llvm.arm.neon.vcadd.rot270 (IntrinsicsARM.td:877)
    arm_neon_vcadd_rot90,                      // llvm.arm.neon.vcadd.rot90 (IntrinsicsARM.td:876)
    arm_neon_vcls,                             // llvm.arm.neon.vcls (IntrinsicsARM.td:626)
    arm_neon_vcvtas,                           // llvm.arm.neon.vcvtas (IntrinsicsARM.td:636)
    arm_neon_vcvtau,                           // llvm.arm.neon.vcvtau (IntrinsicsARM.td:635)
    arm_neon_vcvtbfp2bf,                       // llvm.arm.neon.vcvtbfp2bf (IntrinsicsARM.td:845)
    arm_neon_vcvtfp2bf,                        // llvm.arm.neon.vcvtfp2bf (IntrinsicsARM.td:843)
    arm_neon_vcvtfp2fxs,                       // llvm.arm.neon.vcvtfp2fxs (IntrinsicsARM.td:645)
    arm_neon_vcvtfp2fxu,                       // llvm.arm.neon.vcvtfp2fxu (IntrinsicsARM.td:646)
    arm_neon_vcvtfp2hf,                        // llvm.arm.neon.vcvtfp2hf (IntrinsicsARM.td:651)
    arm_neon_vcvtfxs2fp,                       // llvm.arm.neon.vcvtfxs2fp (IntrinsicsARM.td:647)
    arm_neon_vcvtfxu2fp,                       // llvm.arm.neon.vcvtfxu2fp (IntrinsicsARM.td:648)
    arm_neon_vcvthf2fp,                        // llvm.arm.neon.vcvthf2fp (IntrinsicsARM.td:653)
    arm_neon_vcvtms,                           // llvm.arm.neon.vcvtms (IntrinsicsARM.td:642)
    arm_neon_vcvtmu,                           // llvm.arm.neon.vcvtmu (IntrinsicsARM.td:641)
    arm_neon_vcvtns,                           // llvm.arm.neon.vcvtns (IntrinsicsARM.td:638)
    arm_neon_vcvtnu,                           // llvm.arm.neon.vcvtnu (IntrinsicsARM.td:637)
    arm_neon_vcvtps,                           // llvm.arm.neon.vcvtps (IntrinsicsARM.td:640)
    arm_neon_vcvtpu,                           // llvm.arm.neon.vcvtpu (IntrinsicsARM.td:639)
    arm_neon_vhadds,                           // llvm.arm.neon.vhadds (IntrinsicsARM.td:499)
    arm_neon_vhaddu,                           // llvm.arm.neon.vhaddu (IntrinsicsARM.td:500)
    arm_neon_vhsubs,                           // llvm.arm.neon.vhsubs (IntrinsicsARM.td:532)
    arm_neon_vhsubu,                           // llvm.arm.neon.vhsubu (IntrinsicsARM.td:533)
    arm_neon_vld1,                             // llvm.arm.neon.vld1 (IntrinsicsARM.td:679)
    arm_neon_vld1x2,                           // llvm.arm.neon.vld1x2 (IntrinsicsARM.td:692)
    arm_neon_vld1x3,                           // llvm.arm.neon.vld1x3 (IntrinsicsARM.td:695)
    arm_neon_vld1x4,                           // llvm.arm.neon.vld1x4 (IntrinsicsARM.td:698)
    arm_neon_vld2,                             // llvm.arm.neon.vld2 (IntrinsicsARM.td:682)
    arm_neon_vld2dup,                          // llvm.arm.neon.vld2dup (IntrinsicsARM.td:723)
    arm_neon_vld2lane,                         // llvm.arm.neon.vld2lane (IntrinsicsARM.td:705)
    arm_neon_vld3,                             // llvm.arm.neon.vld3 (IntrinsicsARM.td:685)
    arm_neon_vld3dup,                          // llvm.arm.neon.vld3dup (IntrinsicsARM.td:726)
    arm_neon_vld3lane,                         // llvm.arm.neon.vld3lane (IntrinsicsARM.td:710)
    arm_neon_vld4,                             // llvm.arm.neon.vld4 (IntrinsicsARM.td:688)
    arm_neon_vld4dup,                          // llvm.arm.neon.vld4dup (IntrinsicsARM.td:729)
    arm_neon_vld4lane,                         // llvm.arm.neon.vld4lane (IntrinsicsARM.td:715)
    arm_neon_vmaxnm,                           // llvm.arm.neon.vmaxnm (IntrinsicsARM.td:517)
    arm_neon_vmaxs,                            // llvm.arm.neon.vmaxs (IntrinsicsARM.td:515)
    arm_neon_vmaxu,                            // llvm.arm.neon.vmaxu (IntrinsicsARM.td:516)
    arm_neon_vminnm,                           // llvm.arm.neon.vminnm (IntrinsicsARM.td:522)
    arm_neon_vmins,                            // llvm.arm.neon.vmins (IntrinsicsARM.td:520)
    arm_neon_vminu,                            // llvm.arm.neon.vminu (IntrinsicsARM.td:521)
    arm_neon_vmullp,                           // llvm.arm.neon.vmullp (IntrinsicsARM.td:511)
    arm_neon_vmulls,                           // llvm.arm.neon.vmulls (IntrinsicsARM.td:509)
    arm_neon_vmullu,                           // llvm.arm.neon.vmullu (IntrinsicsARM.td:510)
    arm_neon_vmulp,                            // llvm.arm.neon.vmulp (IntrinsicsARM.td:506)
    arm_neon_vpadals,                          // llvm.arm.neon.vpadals (IntrinsicsARM.td:559)
    arm_neon_vpadalu,                          // llvm.arm.neon.vpadalu (IntrinsicsARM.td:561)
    arm_neon_vpadd,                            // llvm.arm.neon.vpadd (IntrinsicsARM.td:545)
    arm_neon_vpaddls,                          // llvm.arm.neon.vpaddls (IntrinsicsARM.td:551)
    arm_neon_vpaddlu,                          // llvm.arm.neon.vpaddlu (IntrinsicsARM.td:553)
    arm_neon_vpmaxs,                           // llvm.arm.neon.vpmaxs (IntrinsicsARM.td:565)
    arm_neon_vpmaxu,                           // llvm.arm.neon.vpmaxu (IntrinsicsARM.td:566)
    arm_neon_vpmins,                           // llvm.arm.neon.vpmins (IntrinsicsARM.td:567)
    arm_neon_vpminu,                           // llvm.arm.neon.vpminu (IntrinsicsARM.td:568)
    arm_neon_vqabs,                            // llvm.arm.neon.vqabs (IntrinsicsARM.td:620)
    arm_neon_vqdmulh,                          // llvm.arm.neon.vqdmulh (IntrinsicsARM.td:507)
    arm_neon_vqdmull,                          // llvm.arm.neon.vqdmull (IntrinsicsARM.td:512)
    arm_neon_vqmovns,                          // llvm.arm.neon.vqmovns (IntrinsicsARM.td:657)
    arm_neon_vqmovnsu,                         // llvm.arm.neon.vqmovnsu (IntrinsicsARM.td:659)
    arm_neon_vqmovnu,                          // llvm.arm.neon.vqmovnu (IntrinsicsARM.td:658)
    arm_neon_vqneg,                            // llvm.arm.neon.vqneg (IntrinsicsARM.td:623)
    arm_neon_vqrdmlah,                         // llvm.arm.neon.vqrdmlah (IntrinsicsARM.td:819)
    arm_neon_vqrdmlsh,                         // llvm.arm.neon.vqrdmlsh (IntrinsicsARM.td:820)
    arm_neon_vqrdmulh,                         // llvm.arm.neon.vqrdmulh (IntrinsicsARM.td:508)
    arm_neon_vqrshiftns,                       // llvm.arm.neon.vqrshiftns (IntrinsicsARM.td:611)
    arm_neon_vqrshiftnsu,                      // llvm.arm.neon.vqrshiftnsu (IntrinsicsARM.td:613)
    arm_neon_vqrshiftnu,                       // llvm.arm.neon.vqrshiftnu (IntrinsicsARM.td:612)
    arm_neon_vqrshifts,                        // llvm.arm.neon.vqrshifts (IntrinsicsARM.td:609)
    arm_neon_vqrshiftu,                        // llvm.arm.neon.vqrshiftu (IntrinsicsARM.td:610)
    arm_neon_vqshiftns,                        // llvm.arm.neon.vqshiftns (IntrinsicsARM.td:604)
    arm_neon_vqshiftnsu,                       // llvm.arm.neon.vqshiftnsu (IntrinsicsARM.td:606)
    arm_neon_vqshiftnu,                        // llvm.arm.neon.vqshiftnu (IntrinsicsARM.td:605)
    arm_neon_vqshifts,                         // llvm.arm.neon.vqshifts (IntrinsicsARM.td:601)
    arm_neon_vqshiftsu,                        // llvm.arm.neon.vqshiftsu (IntrinsicsARM.td:603)
    arm_neon_vqshiftu,                         // llvm.arm.neon.vqshiftu (IntrinsicsARM.td:602)
    arm_neon_vraddhn,                          // llvm.arm.neon.vraddhn (IntrinsicsARM.td:503)
    arm_neon_vrecpe,                           // llvm.arm.neon.vrecpe (IntrinsicsARM.td:629)
    arm_neon_vrecps,                           // llvm.arm.neon.vrecps (IntrinsicsARM.td:525)
    arm_neon_vrhadds,                          // llvm.arm.neon.vrhadds (IntrinsicsARM.td:501)
    arm_neon_vrhaddu,                          // llvm.arm.neon.vrhaddu (IntrinsicsARM.td:502)
    arm_neon_vrshiftn,                         // llvm.arm.neon.vrshiftn (IntrinsicsARM.td:598)
    arm_neon_vrshifts,                         // llvm.arm.neon.vrshifts (IntrinsicsARM.td:596)
    arm_neon_vrshiftu,                         // llvm.arm.neon.vrshiftu (IntrinsicsARM.td:597)
    arm_neon_vrsqrte,                          // llvm.arm.neon.vrsqrte (IntrinsicsARM.td:632)
    arm_neon_vrsqrts,                          // llvm.arm.neon.vrsqrts (IntrinsicsARM.td:528)
    arm_neon_vrsubhn,                          // llvm.arm.neon.vrsubhn (IntrinsicsARM.td:534)
    arm_neon_vshiftins,                        // llvm.arm.neon.vshiftins (IntrinsicsARM.td:616)
    arm_neon_vshifts,                          // llvm.arm.neon.vshifts (IntrinsicsARM.td:592)
    arm_neon_vshiftu,                          // llvm.arm.neon.vshiftu (IntrinsicsARM.td:593)
    arm_neon_vst1,                             // llvm.arm.neon.vst1 (IntrinsicsARM.td:735)
    arm_neon_vst1x2,                           // llvm.arm.neon.vst1x2 (IntrinsicsARM.td:752)
    arm_neon_vst1x3,                           // llvm.arm.neon.vst1x3 (IntrinsicsARM.td:755)
    arm_neon_vst1x4,                           // llvm.arm.neon.vst1x4 (IntrinsicsARM.td:758)
    arm_neon_vst2,                             // llvm.arm.neon.vst2 (IntrinsicsARM.td:738)
    arm_neon_vst2lane,                         // llvm.arm.neon.vst2lane (IntrinsicsARM.td:767)
    arm_neon_vst3,                             // llvm.arm.neon.vst3 (IntrinsicsARM.td:741)
    arm_neon_vst3lane,                         // llvm.arm.neon.vst3lane (IntrinsicsARM.td:772)
    arm_neon_vst4,                             // llvm.arm.neon.vst4 (IntrinsicsARM.td:746)
    arm_neon_vst4lane,                         // llvm.arm.neon.vst4lane (IntrinsicsARM.td:777)
    arm_neon_vtbl1,                            // llvm.arm.neon.vtbl1 (IntrinsicsARM.td:663)
    arm_neon_vtbl2,                            // llvm.arm.neon.vtbl2 (IntrinsicsARM.td:664)
    arm_neon_vtbl3,                            // llvm.arm.neon.vtbl3 (IntrinsicsARM.td:665)
    arm_neon_vtbl4,                            // llvm.arm.neon.vtbl4 (IntrinsicsARM.td:666)
    arm_neon_vtbx1,                            // llvm.arm.neon.vtbx1 (IntrinsicsARM.td:672)
    arm_neon_vtbx2,                            // llvm.arm.neon.vtbx2 (IntrinsicsARM.td:673)
    arm_neon_vtbx3,                            // llvm.arm.neon.vtbx3 (IntrinsicsARM.td:674)
    arm_neon_vtbx4,                            // llvm.arm.neon.vtbx4 (IntrinsicsARM.td:675)
    arm_qadd,                                  // llvm.arm.qadd (IntrinsicsARM.td:47)
    arm_qadd16,                                // llvm.arm.qadd16 (IntrinsicsARM.td:159)
    arm_qadd8,                                 // llvm.arm.qadd8 (IntrinsicsARM.td:112)
    arm_qasx,                                  // llvm.arm.qasx (IntrinsicsARM.td:162)
    arm_qsax,                                  // llvm.arm.qsax (IntrinsicsARM.td:165)
    arm_qsub,                                  // llvm.arm.qsub (IntrinsicsARM.td:50)
    arm_qsub16,                                // llvm.arm.qsub16 (IntrinsicsARM.td:168)
    arm_qsub8,                                 // llvm.arm.qsub8 (IntrinsicsARM.td:115)
    arm_sadd16,                                // llvm.arm.sadd16 (IntrinsicsARM.td:172)
    arm_sadd8,                                 // llvm.arm.sadd8 (IntrinsicsARM.td:119)
    arm_sasx,                                  // llvm.arm.sasx (IntrinsicsARM.td:175)
    arm_sel,                                   // llvm.arm.sel (IntrinsicsARM.td:107)
    arm_set_fpscr,                             // llvm.arm.set.fpscr (IntrinsicsARM.td:315)
    arm_shadd16,                               // llvm.arm.shadd16 (IntrinsicsARM.td:177)
    arm_shadd8,                                // llvm.arm.shadd8 (IntrinsicsARM.td:121)
    arm_shasx,                                 // llvm.arm.shasx (IntrinsicsARM.td:180)
    arm_shsax,                                 // llvm.arm.shsax (IntrinsicsARM.td:183)
    arm_shsub16,                               // llvm.arm.shsub16 (IntrinsicsARM.td:186)
    arm_shsub8,                                // llvm.arm.shsub8 (IntrinsicsARM.td:124)
    arm_smlabb,                                // llvm.arm.smlabb (IntrinsicsARM.td:61)
    arm_smlabt,                                // llvm.arm.smlabt (IntrinsicsARM.td:65)
    arm_smlad,                                 // llvm.arm.smlad (IntrinsicsARM.td:233)
    arm_smladx,                                // llvm.arm.smladx (IntrinsicsARM.td:237)
    arm_smlald,                                // llvm.arm.smlald (IntrinsicsARM.td:241)
    arm_smlaldx,                               // llvm.arm.smlaldx (IntrinsicsARM.td:245)
    arm_smlatb,                                // llvm.arm.smlatb (IntrinsicsARM.td:69)
    arm_smlatt,                                // llvm.arm.smlatt (IntrinsicsARM.td:73)
    arm_smlawb,                                // llvm.arm.smlawb (IntrinsicsARM.td:77)
    arm_smlawt,                                // llvm.arm.smlawt (IntrinsicsARM.td:81)
    arm_smlsd,                                 // llvm.arm.smlsd (IntrinsicsARM.td:249)
    arm_smlsdx,                                // llvm.arm.smlsdx (IntrinsicsARM.td:253)
    arm_smlsld,                                // llvm.arm.smlsld (IntrinsicsARM.td:257)
    arm_smlsldx,                               // llvm.arm.smlsldx (IntrinsicsARM.td:261)
    arm_smuad,                                 // llvm.arm.smuad (IntrinsicsARM.td:265)
    arm_smuadx,                                // llvm.arm.smuadx (IntrinsicsARM.td:268)
    arm_smulbb,                                // llvm.arm.smulbb (IntrinsicsARM.td:25)
    arm_smulbt,                                // llvm.arm.smulbt (IntrinsicsARM.td:28)
    arm_smultb,                                // llvm.arm.smultb (IntrinsicsARM.td:31)
    arm_smultt,                                // llvm.arm.smultt (IntrinsicsARM.td:34)
    arm_smulwb,                                // llvm.arm.smulwb (IntrinsicsARM.td:37)
    arm_smulwt,                                // llvm.arm.smulwt (IntrinsicsARM.td:40)
    arm_smusd,                                 // llvm.arm.smusd (IntrinsicsARM.td:271)
    arm_smusdx,                                // llvm.arm.smusdx (IntrinsicsARM.td:274)
    arm_space,                                 // llvm.arm.space (IntrinsicsARM.td:22)
    arm_ssat,                                  // llvm.arm.ssat (IntrinsicsARM.td:53)
    arm_ssat16,                                // llvm.arm.ssat16 (IntrinsicsARM.td:87)
    arm_ssax,                                  // llvm.arm.ssax (IntrinsicsARM.td:190)
    arm_ssub16,                                // llvm.arm.ssub16 (IntrinsicsARM.td:193)
    arm_ssub8,                                 // llvm.arm.ssub8 (IntrinsicsARM.td:128)
    arm_stc,                                   // llvm.arm.stc (IntrinsicsARM.td:335)
    arm_stc2,                                  // llvm.arm.stc2 (IntrinsicsARM.td:339)
    arm_stc2l,                                 // llvm.arm.stc2l (IntrinsicsARM.td:341)
    arm_stcl,                                  // llvm.arm.stcl (IntrinsicsARM.td:337)
    arm_stlex,                                 // llvm.arm.stlex (IntrinsicsARM.td:287)
    arm_stlexd,                                // llvm.arm.stlexd (IntrinsicsARM.td:295)
    arm_strex,                                 // llvm.arm.strex (IntrinsicsARM.td:284)
    arm_strexd,                                // llvm.arm.strexd (IntrinsicsARM.td:291)
    arm_sxtab16,                               // llvm.arm.sxtab16 (IntrinsicsARM.td:95)
    arm_sxtb16,                                // llvm.arm.sxtb16 (IntrinsicsARM.td:98)
    arm_uadd16,                                // llvm.arm.uadd16 (IntrinsicsARM.td:196)
    arm_uadd8,                                 // llvm.arm.uadd8 (IntrinsicsARM.td:131)
    arm_uasx,                                  // llvm.arm.uasx (IntrinsicsARM.td:199)
    arm_uhadd16,                               // llvm.arm.uhadd16 (IntrinsicsARM.td:201)
    arm_uhadd8,                                // llvm.arm.uhadd8 (IntrinsicsARM.td:133)
    arm_uhasx,                                 // llvm.arm.uhasx (IntrinsicsARM.td:204)
    arm_uhsax,                                 // llvm.arm.uhsax (IntrinsicsARM.td:207)
    arm_uhsub16,                               // llvm.arm.uhsub16 (IntrinsicsARM.td:210)
    arm_uhsub8,                                // llvm.arm.uhsub8 (IntrinsicsARM.td:136)
    arm_undefined,                             // llvm.arm.undefined (IntrinsicsARM.td:421)
    arm_uqadd16,                               // llvm.arm.uqadd16 (IntrinsicsARM.td:213)
    arm_uqadd8,                                // llvm.arm.uqadd8 (IntrinsicsARM.td:139)
    arm_uqasx,                                 // llvm.arm.uqasx (IntrinsicsARM.td:216)
    arm_uqsax,                                 // llvm.arm.uqsax (IntrinsicsARM.td:219)
    arm_uqsub16,                               // llvm.arm.uqsub16 (IntrinsicsARM.td:222)
    arm_uqsub8,                                // llvm.arm.uqsub8 (IntrinsicsARM.td:142)
    arm_usad8,                                 // llvm.arm.usad8 (IntrinsicsARM.td:150)
    arm_usada8,                                // llvm.arm.usada8 (IntrinsicsARM.td:153)
    arm_usat,                                  // llvm.arm.usat (IntrinsicsARM.td:56)
    arm_usat16,                                // llvm.arm.usat16 (IntrinsicsARM.td:90)
    arm_usax,                                  // llvm.arm.usax (IntrinsicsARM.td:226)
    arm_usub16,                                // llvm.arm.usub16 (IntrinsicsARM.td:229)
    arm_usub8,                                 // llvm.arm.usub8 (IntrinsicsARM.td:146)
    arm_uxtab16,                               // llvm.arm.uxtab16 (IntrinsicsARM.td:100)
    arm_uxtb16,                                // llvm.arm.uxtb16 (IntrinsicsARM.td:103)
    arm_vcvtr,                                 // llvm.arm.vcvtr (IntrinsicsARM.td:317)
    arm_vcvtru,                                // llvm.arm.vcvtru (IntrinsicsARM.td:319)
}; // enum
} // namespace llvm::Intrinsic
#endif

