/*===- TableGen'erated file -------------------------------------*- C++ -*-===*\
|*                                                                            *|
|* Intrinsic Function Source Fragment                                         *|
|*                                                                            *|
|* Automatically generated file, do not edit!                                 *|
|*                                                                            *|
\*===----------------------------------------------------------------------===*/

#ifdef GET_INTRINSIC_IITINFO
  IIT_Done = 0,
  IIT_I1 = 1,
  IIT_I8 = 2,
  IIT_I16 = 3,
  IIT_I32 = 4,
  IIT_I64 = 5,
  IIT_F16 = 6,
  IIT_F32 = 7,
  IIT_F64 = 8,
  IIT_V2 = 9,
  IIT_V4 = 10,
  IIT_V8 = 11,
  IIT_V16 = 12,
  IIT_V32 = 13,
  IIT_PTR = 14,
  IIT_ARG = 15,
  IIT_V64 = 16,
  IIT_MMX = 17,
  IIT_TOKEN = 18,
  IIT_METADATA = 19,
  IIT_EMPTYSTRUCT = 20,
  IIT_STRUCT = 21,
  IIT_EXTEND_ARG = 22,
  IIT_TRUNC_ARG = 23,
  IIT_ANYPTR = 24,
  IIT_V1 = 25,
  IIT_VARARG = 26,
  IIT_ONE_NTH_ELTS_VEC_ARG = 27,
  IIT_SAME_VEC_WIDTH_ARG = 28,
  IIT_VEC_OF_ANYPTRS_TO_ELT = 29,
  IIT_I128 = 30,
  IIT_V512 = 31,
  IIT_V1024 = 32,
  IIT_F128 = 33,
  IIT_VEC_ELEMENT = 34,
  IIT_SCALABLE_VEC = 35,
  IIT_SUBDIVIDE2_ARG = 36,
  IIT_SUBDIVIDE4_ARG = 37,
  IIT_VEC_OF_BITCASTS_TO_INT = 38,
  IIT_V128 = 39,
  IIT_BF16 = 40,
  IIT_V256 = 41,
  IIT_AMX = 42,
  IIT_PPCF128 = 43,
  IIT_V3 = 44,
  IIT_EXTERNREF = 45,
  IIT_FUNCREF = 46,
  IIT_I2 = 47,
  IIT_I4 = 48,
  IIT_AARCH64_SVCOUNT = 49,
  IIT_V6 = 50,
  IIT_V10 = 51,
  IIT_V2048 = 52,
  IIT_V4096 = 53,
#endif

// Target mapping.
#ifdef GET_INTRINSIC_TARGET_DATA
struct IntrinsicTargetInfo {
  StringLiteral Name;
  size_t Offset;
  size_t Count;
};
static constexpr IntrinsicTargetInfo TargetInfos[] = {
  {"", 0, 520},
  {"aarch64", 520, 1705},
  {"amdgcn", 2225, 1457},
  {"arm", 3682, 504},
  {"bpf", 4186, 12},
  {"dx", 4198, 74},
  {"hexagon", 4272, 2115},
  {"loongarch", 6387, 1535},
  {"mips", 7922, 671},
  {"nvvm", 8593, 2569},
  {"ppc", 11162, 685},
  {"r600", 11847, 35},
  {"riscv", 11882, 795},
  {"s390", 12677, 258},
  {"spv", 12935, 111},
  {"ve", 13046, 1263},
  {"wasm", 14309, 74},
  {"x86", 14383, 1687},
  {"xcore", 16070, 53},
};
#endif

// Intrinsic ID to name table.
#ifdef GET_INTRINSIC_NAME_TABLE
// Note that entry #0 is the invalid intrinsic!


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char IntrinsicNameTableStorage[] =
  "\0"
  "not_intrinsic\0"
  "llvm.abs\0"
  "llvm.acos\0"
  "llvm.addressofreturnaddress\0"
  "llvm.adjust.trampoline\0"
  "llvm.alloc.token.id\0"
  "llvm.allow.runtime.check\0"
  "llvm.allow.sanitize.address\0"
  "llvm.allow.sanitize.hwaddress\0"
  "llvm.allow.sanitize.memory\0"
  "llvm.allow.sanitize.thread\0"
  "llvm.allow.ubsan.check\0"
  "llvm.annotation\0"
  "llvm.arithmetic.fence\0"
  "llvm.asan.check.memaccess\0"
  "llvm.asin\0"
  "llvm.assume\0"
  "llvm.atan\0"
  "llvm.atan2\0"
  "llvm.bitreverse\0"
  "llvm.bswap\0"
  "llvm.call.preallocated.arg\0"
  "llvm.call.preallocated.setup\0"
  "llvm.call.preallocated.teardown\0"
  "llvm.callbr.landingpad\0"
  "llvm.canonicalize\0"
  "llvm.ceil\0"
  "llvm.clear_cache\0"
  "llvm.clmul\0"
  "llvm.codeview.annotation\0"
  "llvm.convert.from.fp16\0"
  "llvm.convert.to.fp16\0"
  "llvm.copysign\0"
  "llvm.coro.align\0"
  "llvm.coro.alloc\0"
  "llvm.coro.alloca.alloc\0"
  "llvm.coro.alloca.free\0"
  "llvm.coro.alloca.get\0"
  "llvm.coro.async.context.alloc\0"
  "llvm.coro.async.context.dealloc\0"
  "llvm.coro.async.resume\0"
  "llvm.coro.async.size.replace\0"
  "llvm.coro.await.suspend.bool\0"
  "llvm.coro.await.suspend.handle\0"
  "llvm.coro.await.suspend.void\0"
  "llvm.coro.begin\0"
  "llvm.coro.begin.custom.abi\0"
  "llvm.coro.destroy\0"
  "llvm.coro.done\0"
  "llvm.coro.end\0"
  "llvm.coro.end.async\0"
  "llvm.coro.end.results\0"
  "llvm.coro.frame\0"
  "llvm.coro.free\0"
  "llvm.coro.id\0"
  "llvm.coro.id.async\0"
  "llvm.coro.id.retcon\0"
  "llvm.coro.id.retcon.once\0"
  "llvm.coro.is_in_ramp\0"
  "llvm.coro.noop\0"
  "llvm.coro.prepare.async\0"
  "llvm.coro.prepare.retcon\0"
  "llvm.coro.promise\0"
  "llvm.coro.resume\0"
  "llvm.coro.save\0"
  "llvm.coro.size\0"
  "llvm.coro.subfn.addr\0"
  "llvm.coro.suspend\0"
  "llvm.coro.suspend.async\0"
  "llvm.coro.suspend.retcon\0"
  "llvm.cos\0"
  "llvm.cosh\0"
  "llvm.ctlz\0"
  "llvm.ctpop\0"
  "llvm.cttz\0"
  "llvm.dbg.assign\0"
  "llvm.dbg.declare\0"
  "llvm.dbg.label\0"
  "llvm.dbg.value\0"
  "llvm.debugtrap\0"
  "llvm.donothing\0"
  "llvm.eh.dwarf.cfa\0"
  "llvm.eh.exceptioncode\0"
  "llvm.eh.exceptionpointer\0"
  "llvm.eh.recoverfp\0"
  "llvm.eh.return.i32\0"
  "llvm.eh.return.i64\0"
  "llvm.eh.sjlj.callsite\0"
  "llvm.eh.sjlj.functioncontext\0"
  "llvm.eh.sjlj.longjmp\0"
  "llvm.eh.sjlj.lsda\0"
  "llvm.eh.sjlj.setjmp\0"
  "llvm.eh.sjlj.setup.dispatch\0"
  "llvm.eh.typeid.for\0"
  "llvm.eh.unwind.init\0"
  "llvm.exp\0"
  "llvm.exp10\0"
  "llvm.exp2\0"
  "llvm.expect\0"
  "llvm.expect.with.probability\0"
  "llvm.experimental.constrained.acos\0"
  "llvm.experimental.constrained.asin\0"
  "llvm.experimental.constrained.atan\0"
  "llvm.experimental.constrained.atan2\0"
  "llvm.experimental.constrained.ceil\0"
  "llvm.experimental.constrained.cos\0"
  "llvm.experimental.constrained.cosh\0"
  "llvm.experimental.constrained.exp\0"
  "llvm.experimental.constrained.exp2\0"
  "llvm.experimental.constrained.fadd\0"
  "llvm.experimental.constrained.fcmp\0"
  "llvm.experimental.constrained.fcmps\0"
  "llvm.experimental.constrained.fdiv\0"
  "llvm.experimental.constrained.floor\0"
  "llvm.experimental.constrained.fma\0"
  "llvm.experimental.constrained.fmul\0"
  "llvm.experimental.constrained.fmuladd\0"
  "llvm.experimental.constrained.fpext\0"
  "llvm.experimental.constrained.fptosi\0"
  "llvm.experimental.constrained.fptoui\0"
  "llvm.experimental.constrained.fptrunc\0"
  "llvm.experimental.constrained.frem\0"
  "llvm.experimental.constrained.fsub\0"
  "llvm.experimental.constrained.ldexp\0"
  "llvm.experimental.constrained.llrint\0"
  "llvm.experimental.constrained.llround\0"
  "llvm.experimental.constrained.log\0"
  "llvm.experimental.constrained.log10\0"
  "llvm.experimental.constrained.log2\0"
  "llvm.experimental.constrained.lrint\0"
  "llvm.experimental.constrained.lround\0"
  "llvm.experimental.constrained.maximum\0"
  "llvm.experimental.constrained.maxnum\0"
  "llvm.experimental.constrained.minimum\0"
  "llvm.experimental.constrained.minnum\0"
  "llvm.experimental.constrained.nearbyint\0"
  "llvm.experimental.constrained.pow\0"
  "llvm.experimental.constrained.powi\0"
  "llvm.experimental.constrained.rint\0"
  "llvm.experimental.constrained.round\0"
  "llvm.experimental.constrained.roundeven\0"
  "llvm.experimental.constrained.sin\0"
  "llvm.experimental.constrained.sinh\0"
  "llvm.experimental.constrained.sitofp\0"
  "llvm.experimental.constrained.sqrt\0"
  "llvm.experimental.constrained.tan\0"
  "llvm.experimental.constrained.tanh\0"
  "llvm.experimental.constrained.trunc\0"
  "llvm.experimental.constrained.uitofp\0"
  "llvm.experimental.convergence.anchor\0"
  "llvm.experimental.convergence.entry\0"
  "llvm.experimental.convergence.loop\0"
  "llvm.experimental.cttz.elts\0"
  "llvm.experimental.deoptimize\0"
  "llvm.experimental.gc.get.pointer.base\0"
  "llvm.experimental.gc.get.pointer.offset\0"
  "llvm.experimental.gc.relocate\0"
  "llvm.experimental.gc.result\0"
  "llvm.experimental.gc.statepoint\0"
  "llvm.experimental.get.vector.length\0"
  "llvm.experimental.guard\0"
  "llvm.experimental.memset.pattern\0"
  "llvm.experimental.noalias.scope.decl\0"
  "llvm.experimental.patchpoint\0"
  "llvm.experimental.patchpoint.void\0"
  "llvm.experimental.stackmap\0"
  "llvm.experimental.vector.compress\0"
  "llvm.experimental.vector.extract.last.active\0"
  "llvm.experimental.vector.histogram.add\0"
  "llvm.experimental.vector.histogram.uadd.sat\0"
  "llvm.experimental.vector.histogram.umax\0"
  "llvm.experimental.vector.histogram.umin\0"
  "llvm.experimental.vector.match\0"
  "llvm.experimental.vp.reverse\0"
  "llvm.experimental.vp.splice\0"
  "llvm.experimental.vp.strided.load\0"
  "llvm.experimental.vp.strided.store\0"
  "llvm.experimental.widenable.condition\0"
  "llvm.fabs\0"
  "llvm.fake.use\0"
  "llvm.floor\0"
  "llvm.fma\0"
  "llvm.fmuladd\0"
  "llvm.fptosi.sat\0"
  "llvm.fptoui.sat\0"
  "llvm.fptrunc.round\0"
  "llvm.frameaddress\0"
  "llvm.frexp\0"
  "llvm.fshl\0"
  "llvm.fshr\0"
  "llvm.gcread\0"
  "llvm.gcroot\0"
  "llvm.gcwrite\0"
  "llvm.get.active.lane.mask\0"
  "llvm.get.dynamic.area.offset\0"
  "llvm.get.fpenv\0"
  "llvm.get.fpmode\0"
  "llvm.get.rounding\0"
  "llvm.hwasan.check.memaccess\0"
  "llvm.hwasan.check.memaccess.fixedshadow\0"
  "llvm.hwasan.check.memaccess.shortgranules\0"
  "llvm.hwasan.check.memaccess.shortgranules.fixedshadow\0"
  "llvm.icall.branch.funnel\0"
  "llvm.init.trampoline\0"
  "llvm.instrprof.callsite\0"
  "llvm.instrprof.cover\0"
  "llvm.instrprof.increment\0"
  "llvm.instrprof.increment.step\0"
  "llvm.instrprof.mcdc.parameters\0"
  "llvm.instrprof.mcdc.tvbitmap.update\0"
  "llvm.instrprof.timestamp\0"
  "llvm.instrprof.value.profile\0"
  "llvm.invariant.end\0"
  "llvm.invariant.start\0"
  "llvm.is.constant\0"
  "llvm.is.fpclass\0"
  "llvm.launder.invariant.group\0"
  "llvm.ldexp\0"
  "llvm.lifetime.end\0"
  "llvm.lifetime.start\0"
  "llvm.llrint\0"
  "llvm.llround\0"
  "llvm.load.relative\0"
  "llvm.localaddress\0"
  "llvm.localescape\0"
  "llvm.localrecover\0"
  "llvm.log\0"
  "llvm.log10\0"
  "llvm.log2\0"
  "llvm.loop.decrement\0"
  "llvm.loop.decrement.reg\0"
  "llvm.loop.dependence.raw.mask\0"
  "llvm.loop.dependence.war.mask\0"
  "llvm.lrint\0"
  "llvm.lround\0"
  "llvm.masked.compressstore\0"
  "llvm.masked.expandload\0"
  "llvm.masked.gather\0"
  "llvm.masked.load\0"
  "llvm.masked.scatter\0"
  "llvm.masked.store\0"
  "llvm.matrix.column.major.load\0"
  "llvm.matrix.column.major.store\0"
  "llvm.matrix.multiply\0"
  "llvm.matrix.transpose\0"
  "llvm.maximum\0"
  "llvm.maximumnum\0"
  "llvm.maxnum\0"
  "llvm.memcpy\0"
  "llvm.memcpy.element.unordered.atomic\0"
  "llvm.memcpy.inline\0"
  "llvm.memmove\0"
  "llvm.memmove.element.unordered.atomic\0"
  "llvm.memset\0"
  "llvm.memset.element.unordered.atomic\0"
  "llvm.memset.inline\0"
  "llvm.minimum\0"
  "llvm.minimumnum\0"
  "llvm.minnum\0"
  "llvm.modf\0"
  "llvm.nearbyint\0"
  "llvm.objc.arc.annotation.bottomup.bbend\0"
  "llvm.objc.arc.annotation.bottomup.bbstart\0"
  "llvm.objc.arc.annotation.topdown.bbend\0"
  "llvm.objc.arc.annotation.topdown.bbstart\0"
  "llvm.objc.autorelease\0"
  "llvm.objc.autoreleasePoolPop\0"
  "llvm.objc.autoreleasePoolPush\0"
  "llvm.objc.autoreleaseReturnValue\0"
  "llvm.objc.claimAutoreleasedReturnValue\0"
  "llvm.objc.clang.arc.noop.use\0"
  "llvm.objc.clang.arc.use\0"
  "llvm.objc.copyWeak\0"
  "llvm.objc.destroyWeak\0"
  "llvm.objc.initWeak\0"
  "llvm.objc.loadWeak\0"
  "llvm.objc.loadWeakRetained\0"
  "llvm.objc.moveWeak\0"
  "llvm.objc.release\0"
  "llvm.objc.retain\0"
  "llvm.objc.retain.autorelease\0"
  "llvm.objc.retainAutorelease\0"
  "llvm.objc.retainAutoreleaseReturnValue\0"
  "llvm.objc.retainAutoreleasedReturnValue\0"
  "llvm.objc.retainBlock\0"
  "llvm.objc.retainedObject\0"
  "llvm.objc.storeStrong\0"
  "llvm.objc.storeWeak\0"
  "llvm.objc.sync.enter\0"
  "llvm.objc.sync.exit\0"
  "llvm.objc.unretainedObject\0"
  "llvm.objc.unretainedPointer\0"
  "llvm.objc.unsafeClaimAutoreleasedReturnValue\0"
  "llvm.objectsize\0"
  "llvm.pcmarker\0"
  "llvm.pow\0"
  "llvm.powi\0"
  "llvm.prefetch\0"
  "llvm.preserve.array.access.index\0"
  "llvm.preserve.static.offset\0"
  "llvm.preserve.struct.access.index\0"
  "llvm.preserve.union.access.index\0"
  "llvm.protected.field.ptr\0"
  "llvm.pseudoprobe\0"
  "llvm.ptr.annotation\0"
  "llvm.ptrauth.auth\0"
  "llvm.ptrauth.blend\0"
  "llvm.ptrauth.resign\0"
  "llvm.ptrauth.sign\0"
  "llvm.ptrauth.sign.generic\0"
  "llvm.ptrauth.strip\0"
  "llvm.ptrmask\0"
  "llvm.public.type.test\0"
  "llvm.read_register\0"
  "llvm.read_volatile_register\0"
  "llvm.readcyclecounter\0"
  "llvm.readsteadycounter\0"
  "llvm.reloc.none\0"
  "llvm.reset.fpenv\0"
  "llvm.reset.fpmode\0"
  "llvm.returnaddress\0"
  "llvm.rint\0"
  "llvm.round\0"
  "llvm.roundeven\0"
  "llvm.sadd.sat\0"
  "llvm.sadd.with.overflow\0"
  "llvm.scmp\0"
  "llvm.sdiv.fix\0"
  "llvm.sdiv.fix.sat\0"
  "llvm.seh.scope.begin\0"
  "llvm.seh.scope.end\0"
  "llvm.seh.try.begin\0"
  "llvm.seh.try.end\0"
  "llvm.set.fpenv\0"
  "llvm.set.fpmode\0"
  "llvm.set.loop.iterations\0"
  "llvm.set.rounding\0"
  "llvm.sideeffect\0"
  "llvm.sin\0"
  "llvm.sincos\0"
  "llvm.sincospi\0"
  "llvm.sinh\0"
  "llvm.smax\0"
  "llvm.smin\0"
  "llvm.smul.fix\0"
  "llvm.smul.fix.sat\0"
  "llvm.smul.with.overflow\0"
  "llvm.sponentry\0"
  "llvm.sqrt\0"
  "llvm.ssa.copy\0"
  "llvm.sshl.sat\0"
  "llvm.ssub.sat\0"
  "llvm.ssub.with.overflow\0"
  "llvm.stackaddress\0"
  "llvm.stackguard\0"
  "llvm.stackprotector\0"
  "llvm.stackrestore\0"
  "llvm.stacksave\0"
  "llvm.start.loop.iterations\0"
  "llvm.stepvector\0"
  "llvm.strip.invariant.group\0"
  "llvm.swift.async.context.addr\0"
  "llvm.tan\0"
  "llvm.tanh\0"
  "llvm.test.set.loop.iterations\0"
  "llvm.test.start.loop.iterations\0"
  "llvm.thread.pointer\0"
  "llvm.threadlocal.address\0"
  "llvm.trap\0"
  "llvm.trunc\0"
  "llvm.type.checked.load\0"
  "llvm.type.checked.load.relative\0"
  "llvm.type.test\0"
  "llvm.uadd.sat\0"
  "llvm.uadd.with.overflow\0"
  "llvm.ubsantrap\0"
  "llvm.ucmp\0"
  "llvm.udiv.fix\0"
  "llvm.udiv.fix.sat\0"
  "llvm.umax\0"
  "llvm.umin\0"
  "llvm.umul.fix\0"
  "llvm.umul.fix.sat\0"
  "llvm.umul.with.overflow\0"
  "llvm.ushl.sat\0"
  "llvm.usub.sat\0"
  "llvm.usub.with.overflow\0"
  "llvm.va_copy\0"
  "llvm.va_end\0"
  "llvm.va_start\0"
  "llvm.var.annotation\0"
  "llvm.vector.deinterleave2\0"
  "llvm.vector.deinterleave3\0"
  "llvm.vector.deinterleave4\0"
  "llvm.vector.deinterleave5\0"
  "llvm.vector.deinterleave6\0"
  "llvm.vector.deinterleave7\0"
  "llvm.vector.deinterleave8\0"
  "llvm.vector.extract\0"
  "llvm.vector.insert\0"
  "llvm.vector.interleave2\0"
  "llvm.vector.interleave3\0"
  "llvm.vector.interleave4\0"
  "llvm.vector.interleave5\0"
  "llvm.vector.interleave6\0"
  "llvm.vector.interleave7\0"
  "llvm.vector.interleave8\0"
  "llvm.vector.partial.reduce.add\0"
  "llvm.vector.partial.reduce.fadd\0"
  "llvm.vector.reduce.add\0"
  "llvm.vector.reduce.and\0"
  "llvm.vector.reduce.fadd\0"
  "llvm.vector.reduce.fmax\0"
  "llvm.vector.reduce.fmaximum\0"
  "llvm.vector.reduce.fmin\0"
  "llvm.vector.reduce.fminimum\0"
  "llvm.vector.reduce.fmul\0"
  "llvm.vector.reduce.mul\0"
  "llvm.vector.reduce.or\0"
  "llvm.vector.reduce.smax\0"
  "llvm.vector.reduce.smin\0"
  "llvm.vector.reduce.umax\0"
  "llvm.vector.reduce.umin\0"
  "llvm.vector.reduce.xor\0"
  "llvm.vector.reverse\0"
  "llvm.vector.splice.left\0"
  "llvm.vector.splice.right\0"
  "llvm.vp.abs\0"
  "llvm.vp.add\0"
  "llvm.vp.and\0"
  "llvm.vp.ashr\0"
  "llvm.vp.bitreverse\0"
  "llvm.vp.bswap\0"
  "llvm.vp.ceil\0"
  "llvm.vp.copysign\0"
  "llvm.vp.ctlz\0"
  "llvm.vp.ctpop\0"
  "llvm.vp.cttz\0"
  "llvm.vp.cttz.elts\0"
  "llvm.vp.fabs\0"
  "llvm.vp.fadd\0"
  "llvm.vp.fcmp\0"
  "llvm.vp.fdiv\0"
  "llvm.vp.floor\0"
  "llvm.vp.fma\0"
  "llvm.vp.fmul\0"
  "llvm.vp.fmuladd\0"
  "llvm.vp.fneg\0"
  "llvm.vp.fpext\0"
  "llvm.vp.fptosi\0"
  "llvm.vp.fptoui\0"
  "llvm.vp.fptrunc\0"
  "llvm.vp.frem\0"
  "llvm.vp.fshl\0"
  "llvm.vp.fshr\0"
  "llvm.vp.fsub\0"
  "llvm.vp.gather\0"
  "llvm.vp.icmp\0"
  "llvm.vp.inttoptr\0"
  "llvm.vp.is.fpclass\0"
  "llvm.vp.llrint\0"
  "llvm.vp.load\0"
  "llvm.vp.load.ff\0"
  "llvm.vp.lrint\0"
  "llvm.vp.lshr\0"
  "llvm.vp.maximum\0"
  "llvm.vp.maxnum\0"
  "llvm.vp.merge\0"
  "llvm.vp.minimum\0"
  "llvm.vp.minnum\0"
  "llvm.vp.mul\0"
  "llvm.vp.nearbyint\0"
  "llvm.vp.or\0"
  "llvm.vp.ptrtoint\0"
  "llvm.vp.reduce.add\0"
  "llvm.vp.reduce.and\0"
  "llvm.vp.reduce.fadd\0"
  "llvm.vp.reduce.fmax\0"
  "llvm.vp.reduce.fmaximum\0"
  "llvm.vp.reduce.fmin\0"
  "llvm.vp.reduce.fminimum\0"
  "llvm.vp.reduce.fmul\0"
  "llvm.vp.reduce.mul\0"
  "llvm.vp.reduce.or\0"
  "llvm.vp.reduce.smax\0"
  "llvm.vp.reduce.smin\0"
  "llvm.vp.reduce.umax\0"
  "llvm.vp.reduce.umin\0"
  "llvm.vp.reduce.xor\0"
  "llvm.vp.rint\0"
  "llvm.vp.round\0"
  "llvm.vp.roundeven\0"
  "llvm.vp.roundtozero\0"
  "llvm.vp.sadd.sat\0"
  "llvm.vp.scatter\0"
  "llvm.vp.sdiv\0"
  "llvm.vp.select\0"
  "llvm.vp.sext\0"
  "llvm.vp.shl\0"
  "llvm.vp.sitofp\0"
  "llvm.vp.smax\0"
  "llvm.vp.smin\0"
  "llvm.vp.sqrt\0"
  "llvm.vp.srem\0"
  "llvm.vp.ssub.sat\0"
  "llvm.vp.store\0"
  "llvm.vp.sub\0"
  "llvm.vp.trunc\0"
  "llvm.vp.uadd.sat\0"
  "llvm.vp.udiv\0"
  "llvm.vp.uitofp\0"
  "llvm.vp.umax\0"
  "llvm.vp.umin\0"
  "llvm.vp.urem\0"
  "llvm.vp.usub.sat\0"
  "llvm.vp.xor\0"
  "llvm.vp.zext\0"
  "llvm.vscale\0"
  "llvm.write_register\0"
  "llvm.xray.customevent\0"
  "llvm.xray.typedevent\0"
  "llvm.aarch64.addg\0"
  "llvm.aarch64.break\0"
  "llvm.aarch64.chkfeat\0"
  "llvm.aarch64.clrex\0"
  "llvm.aarch64.cls\0"
  "llvm.aarch64.cls64\0"
  "llvm.aarch64.crc32b\0"
  "llvm.aarch64.crc32cb\0"
  "llvm.aarch64.crc32ch\0"
  "llvm.aarch64.crc32cw\0"
  "llvm.aarch64.crc32cx\0"
  "llvm.aarch64.crc32h\0"
  "llvm.aarch64.crc32w\0"
  "llvm.aarch64.crc32x\0"
  "llvm.aarch64.crypto.aesd\0"
  "llvm.aarch64.crypto.aese\0"
  "llvm.aarch64.crypto.aesimc\0"
  "llvm.aarch64.crypto.aesmc\0"
  "llvm.aarch64.crypto.bcaxs\0"
  "llvm.aarch64.crypto.bcaxu\0"
  "llvm.aarch64.crypto.eor3s\0"
  "llvm.aarch64.crypto.eor3u\0"
  "llvm.aarch64.crypto.rax1\0"
  "llvm.aarch64.crypto.sha1c\0"
  "llvm.aarch64.crypto.sha1h\0"
  "llvm.aarch64.crypto.sha1m\0"
  "llvm.aarch64.crypto.sha1p\0"
  "llvm.aarch64.crypto.sha1su0\0"
  "llvm.aarch64.crypto.sha1su1\0"
  "llvm.aarch64.crypto.sha256h\0"
  "llvm.aarch64.crypto.sha256h2\0"
  "llvm.aarch64.crypto.sha256su0\0"
  "llvm.aarch64.crypto.sha256su1\0"
  "llvm.aarch64.crypto.sha512h\0"
  "llvm.aarch64.crypto.sha512h2\0"
  "llvm.aarch64.crypto.sha512su0\0"
  "llvm.aarch64.crypto.sha512su1\0"
  "llvm.aarch64.crypto.sm3partw1\0"
  "llvm.aarch64.crypto.sm3partw2\0"
  "llvm.aarch64.crypto.sm3ss1\0"
  "llvm.aarch64.crypto.sm3tt1a\0"
  "llvm.aarch64.crypto.sm3tt1b\0"
  "llvm.aarch64.crypto.sm3tt2a\0"
  "llvm.aarch64.crypto.sm3tt2b\0"
  "llvm.aarch64.crypto.sm4e\0"
  "llvm.aarch64.crypto.sm4ekey\0"
  "llvm.aarch64.crypto.xar\0"
  "llvm.aarch64.dmb\0"
  "llvm.aarch64.dsb\0"
  "llvm.aarch64.fjcvtzs\0"
  "llvm.aarch64.frint32x\0"
  "llvm.aarch64.frint32z\0"
  "llvm.aarch64.frint64x\0"
  "llvm.aarch64.frint64z\0"
  "llvm.aarch64.gcspopm\0"
  "llvm.aarch64.gcsss\0"
  "llvm.aarch64.get.fpcr\0"
  "llvm.aarch64.get.fpsr\0"
  "llvm.aarch64.gmi\0"
  "llvm.aarch64.hint\0"
  "llvm.aarch64.hlt\0"
  "llvm.aarch64.irg\0"
  "llvm.aarch64.irg.sp\0"
  "llvm.aarch64.isb\0"
  "llvm.aarch64.ld64b\0"
  "llvm.aarch64.ldaxp\0"
  "llvm.aarch64.ldaxr\0"
  "llvm.aarch64.ldg\0"
  "llvm.aarch64.ldxp\0"
  "llvm.aarch64.ldxr\0"
  "llvm.aarch64.mops.memset.tag\0"
  "llvm.aarch64.neon.abs\0"
  "llvm.aarch64.neon.addhn\0"
  "llvm.aarch64.neon.addp\0"
  "llvm.aarch64.neon.bfdot\0"
  "llvm.aarch64.neon.bfmlalb\0"
  "llvm.aarch64.neon.bfmlalt\0"
  "llvm.aarch64.neon.bfmmla\0"
  "llvm.aarch64.neon.cls\0"
  "llvm.aarch64.neon.fabd\0"
  "llvm.aarch64.neon.facge\0"
  "llvm.aarch64.neon.facgt\0"
  "llvm.aarch64.neon.faddp\0"
  "llvm.aarch64.neon.faddv\0"
  "llvm.aarch64.neon.famax\0"
  "llvm.aarch64.neon.famin\0"
  "llvm.aarch64.neon.fcvtas\0"
  "llvm.aarch64.neon.fcvtau\0"
  "llvm.aarch64.neon.fcvtms\0"
  "llvm.aarch64.neon.fcvtmu\0"
  "llvm.aarch64.neon.fcvtns\0"
  "llvm.aarch64.neon.fcvtnu\0"
  "llvm.aarch64.neon.fcvtps\0"
  "llvm.aarch64.neon.fcvtpu\0"
  "llvm.aarch64.neon.fcvtxn\0"
  "llvm.aarch64.neon.fcvtzs\0"
  "llvm.aarch64.neon.fcvtzu\0"
  "llvm.aarch64.neon.fmax\0"
  "llvm.aarch64.neon.fmaxnm\0"
  "llvm.aarch64.neon.fmaxnmp\0"
  "llvm.aarch64.neon.fmaxnmv\0"
  "llvm.aarch64.neon.fmaxp\0"
  "llvm.aarch64.neon.fmaxv\0"
  "llvm.aarch64.neon.fmin\0"
  "llvm.aarch64.neon.fminnm\0"
  "llvm.aarch64.neon.fminnmp\0"
  "llvm.aarch64.neon.fminnmv\0"
  "llvm.aarch64.neon.fminp\0"
  "llvm.aarch64.neon.fminv\0"
  "llvm.aarch64.neon.fmlal\0"
  "llvm.aarch64.neon.fmlal2\0"
  "llvm.aarch64.neon.fmlsl\0"
  "llvm.aarch64.neon.fmlsl2\0"
  "llvm.aarch64.neon.fmmla\0"
  "llvm.aarch64.neon.fmulx\0"
  "llvm.aarch64.neon.fp8.cvtl1\0"
  "llvm.aarch64.neon.fp8.cvtl2\0"
  "llvm.aarch64.neon.fp8.fcvtn\0"
  "llvm.aarch64.neon.fp8.fcvtn2\0"
  "llvm.aarch64.neon.fp8.fdot2\0"
  "llvm.aarch64.neon.fp8.fdot2.lane\0"
  "llvm.aarch64.neon.fp8.fdot4\0"
  "llvm.aarch64.neon.fp8.fdot4.lane\0"
  "llvm.aarch64.neon.fp8.fmlalb\0"
  "llvm.aarch64.neon.fp8.fmlalb.lane\0"
  "llvm.aarch64.neon.fp8.fmlallbb\0"
  "llvm.aarch64.neon.fp8.fmlallbb.lane\0"
  "llvm.aarch64.neon.fp8.fmlallbt\0"
  "llvm.aarch64.neon.fp8.fmlallbt.lane\0"
  "llvm.aarch64.neon.fp8.fmlalltb\0"
  "llvm.aarch64.neon.fp8.fmlalltb.lane\0"
  "llvm.aarch64.neon.fp8.fmlalltt\0"
  "llvm.aarch64.neon.fp8.fmlalltt.lane\0"
  "llvm.aarch64.neon.fp8.fmlalt\0"
  "llvm.aarch64.neon.fp8.fmlalt.lane\0"
  "llvm.aarch64.neon.fp8.fscale\0"
  "llvm.aarch64.neon.frecpe\0"
  "llvm.aarch64.neon.frecps\0"
  "llvm.aarch64.neon.frecpx\0"
  "llvm.aarch64.neon.frint32x\0"
  "llvm.aarch64.neon.frint32z\0"
  "llvm.aarch64.neon.frint64x\0"
  "llvm.aarch64.neon.frint64z\0"
  "llvm.aarch64.neon.frsqrte\0"
  "llvm.aarch64.neon.frsqrts\0"
  "llvm.aarch64.neon.ld1x2\0"
  "llvm.aarch64.neon.ld1x3\0"
  "llvm.aarch64.neon.ld1x4\0"
  "llvm.aarch64.neon.ld2\0"
  "llvm.aarch64.neon.ld2lane\0"
  "llvm.aarch64.neon.ld2r\0"
  "llvm.aarch64.neon.ld3\0"
  "llvm.aarch64.neon.ld3lane\0"
  "llvm.aarch64.neon.ld3r\0"
  "llvm.aarch64.neon.ld4\0"
  "llvm.aarch64.neon.ld4lane\0"
  "llvm.aarch64.neon.ld4r\0"
  "llvm.aarch64.neon.pmul\0"
  "llvm.aarch64.neon.pmull\0"
  "llvm.aarch64.neon.pmull64\0"
  "llvm.aarch64.neon.raddhn\0"
  "llvm.aarch64.neon.rshrn\0"
  "llvm.aarch64.neon.rsubhn\0"
  "llvm.aarch64.neon.sabd\0"
  "llvm.aarch64.neon.saddlp\0"
  "llvm.aarch64.neon.saddlv\0"
  "llvm.aarch64.neon.saddv\0"
  "llvm.aarch64.neon.scalar.sqxtn\0"
  "llvm.aarch64.neon.scalar.sqxtun\0"
  "llvm.aarch64.neon.scalar.uqxtn\0"
  "llvm.aarch64.neon.sdot\0"
  "llvm.aarch64.neon.shadd\0"
  "llvm.aarch64.neon.shll\0"
  "llvm.aarch64.neon.shsub\0"
  "llvm.aarch64.neon.smax\0"
  "llvm.aarch64.neon.smaxp\0"
  "llvm.aarch64.neon.smaxv\0"
  "llvm.aarch64.neon.smin\0"
  "llvm.aarch64.neon.sminp\0"
  "llvm.aarch64.neon.sminv\0"
  "llvm.aarch64.neon.smmla\0"
  "llvm.aarch64.neon.smull\0"
  "llvm.aarch64.neon.sqabs\0"
  "llvm.aarch64.neon.sqadd\0"
  "llvm.aarch64.neon.sqdmulh\0"
  "llvm.aarch64.neon.sqdmulh.lane\0"
  "llvm.aarch64.neon.sqdmulh.laneq\0"
  "llvm.aarch64.neon.sqdmull\0"
  "llvm.aarch64.neon.sqdmulls.scalar\0"
  "llvm.aarch64.neon.sqneg\0"
  "llvm.aarch64.neon.sqrdmlah\0"
  "llvm.aarch64.neon.sqrdmlsh\0"
  "llvm.aarch64.neon.sqrdmulh\0"
  "llvm.aarch64.neon.sqrdmulh.lane\0"
  "llvm.aarch64.neon.sqrdmulh.laneq\0"
  "llvm.aarch64.neon.sqrshl\0"
  "llvm.aarch64.neon.sqrshrn\0"
  "llvm.aarch64.neon.sqrshrun\0"
  "llvm.aarch64.neon.sqshl\0"
  "llvm.aarch64.neon.sqshlu\0"
  "llvm.aarch64.neon.sqshrn\0"
  "llvm.aarch64.neon.sqshrun\0"
  "llvm.aarch64.neon.sqsub\0"
  "llvm.aarch64.neon.sqxtn\0"
  "llvm.aarch64.neon.sqxtun\0"
  "llvm.aarch64.neon.srhadd\0"
  "llvm.aarch64.neon.srshl\0"
  "llvm.aarch64.neon.sshl\0"
  "llvm.aarch64.neon.sshll\0"
  "llvm.aarch64.neon.st1x2\0"
  "llvm.aarch64.neon.st1x3\0"
  "llvm.aarch64.neon.st1x4\0"
  "llvm.aarch64.neon.st2\0"
  "llvm.aarch64.neon.st2lane\0"
  "llvm.aarch64.neon.st3\0"
  "llvm.aarch64.neon.st3lane\0"
  "llvm.aarch64.neon.st4\0"
  "llvm.aarch64.neon.st4lane\0"
  "llvm.aarch64.neon.subhn\0"
  "llvm.aarch64.neon.suqadd\0"
  "llvm.aarch64.neon.tbl1\0"
  "llvm.aarch64.neon.tbl2\0"
  "llvm.aarch64.neon.tbl3\0"
  "llvm.aarch64.neon.tbl4\0"
  "llvm.aarch64.neon.tbx1\0"
  "llvm.aarch64.neon.tbx2\0"
  "llvm.aarch64.neon.tbx3\0"
  "llvm.aarch64.neon.tbx4\0"
  "llvm.aarch64.neon.uabd\0"
  "llvm.aarch64.neon.uaddlp\0"
  "llvm.aarch64.neon.uaddlv\0"
  "llvm.aarch64.neon.uaddv\0"
  "llvm.aarch64.neon.udot\0"
  "llvm.aarch64.neon.uhadd\0"
  "llvm.aarch64.neon.uhsub\0"
  "llvm.aarch64.neon.umax\0"
  "llvm.aarch64.neon.umaxp\0"
  "llvm.aarch64.neon.umaxv\0"
  "llvm.aarch64.neon.umin\0"
  "llvm.aarch64.neon.uminp\0"
  "llvm.aarch64.neon.uminv\0"
  "llvm.aarch64.neon.ummla\0"
  "llvm.aarch64.neon.umull\0"
  "llvm.aarch64.neon.uqadd\0"
  "llvm.aarch64.neon.uqrshl\0"
  "llvm.aarch64.neon.uqrshrn\0"
  "llvm.aarch64.neon.uqshl\0"
  "llvm.aarch64.neon.uqshrn\0"
  "llvm.aarch64.neon.uqsub\0"
  "llvm.aarch64.neon.uqxtn\0"
  "llvm.aarch64.neon.urecpe\0"
  "llvm.aarch64.neon.urhadd\0"
  "llvm.aarch64.neon.urshl\0"
  "llvm.aarch64.neon.ursqrte\0"
  "llvm.aarch64.neon.usdot\0"
  "llvm.aarch64.neon.ushl\0"
  "llvm.aarch64.neon.ushll\0"
  "llvm.aarch64.neon.usmmla\0"
  "llvm.aarch64.neon.usqadd\0"
  "llvm.aarch64.neon.vcadd.rot270\0"
  "llvm.aarch64.neon.vcadd.rot90\0"
  "llvm.aarch64.neon.vcmla.rot0\0"
  "llvm.aarch64.neon.vcmla.rot180\0"
  "llvm.aarch64.neon.vcmla.rot270\0"
  "llvm.aarch64.neon.vcmla.rot90\0"
  "llvm.aarch64.neon.vcopy.lane\0"
  "llvm.aarch64.neon.vcvtfp2fxs\0"
  "llvm.aarch64.neon.vcvtfp2fxu\0"
  "llvm.aarch64.neon.vcvtfp2hf\0"
  "llvm.aarch64.neon.vcvtfxs2fp\0"
  "llvm.aarch64.neon.vcvtfxu2fp\0"
  "llvm.aarch64.neon.vcvthf2fp\0"
  "llvm.aarch64.neon.vluti2.lane\0"
  "llvm.aarch64.neon.vluti2.laneq\0"
  "llvm.aarch64.neon.vluti4q.lane\0"
  "llvm.aarch64.neon.vluti4q.lane.x2\0"
  "llvm.aarch64.neon.vluti4q.laneq\0"
  "llvm.aarch64.neon.vluti4q.laneq.x2\0"
  "llvm.aarch64.neon.vsli\0"
  "llvm.aarch64.neon.vsri\0"
  "llvm.aarch64.prefetch\0"
  "llvm.aarch64.range.prefetch\0"
  "llvm.aarch64.rndr\0"
  "llvm.aarch64.rndrrs\0"
  "llvm.aarch64.sdiv\0"
  "llvm.aarch64.set.fpcr\0"
  "llvm.aarch64.set.fpmr\0"
  "llvm.aarch64.set.fpsr\0"
  "llvm.aarch64.settag\0"
  "llvm.aarch64.settag.zero\0"
  "llvm.aarch64.sisd.fabd\0"
  "llvm.aarch64.sisd.fcvtxn\0"
  "llvm.aarch64.sme.add.write.single.za.vg1x2\0"
  "llvm.aarch64.sme.add.write.single.za.vg1x4\0"
  "llvm.aarch64.sme.add.write.za.vg1x2\0"
  "llvm.aarch64.sme.add.write.za.vg1x4\0"
  "llvm.aarch64.sme.add.za16.vg1x2\0"
  "llvm.aarch64.sme.add.za16.vg1x4\0"
  "llvm.aarch64.sme.add.za32.vg1x2\0"
  "llvm.aarch64.sme.add.za32.vg1x4\0"
  "llvm.aarch64.sme.add.za64.vg1x2\0"
  "llvm.aarch64.sme.add.za64.vg1x4\0"
  "llvm.aarch64.sme.addha\0"
  "llvm.aarch64.sme.addva\0"
  "llvm.aarch64.sme.bmopa.za32\0"
  "llvm.aarch64.sme.bmops.za32\0"
  "llvm.aarch64.sme.cntsd\0"
  "llvm.aarch64.sme.famax.x2\0"
  "llvm.aarch64.sme.famax.x4\0"
  "llvm.aarch64.sme.famin.x2\0"
  "llvm.aarch64.sme.famin.x4\0"
  "llvm.aarch64.sme.fdot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.fdot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.fdot.single.za32.vg1x2\0"
  "llvm.aarch64.sme.fdot.single.za32.vg1x4\0"
  "llvm.aarch64.sme.fdot.za32.vg1x2\0"
  "llvm.aarch64.sme.fdot.za32.vg1x4\0"
  "llvm.aarch64.sme.fmla.lane.vg1x2\0"
  "llvm.aarch64.sme.fmla.lane.vg1x4\0"
  "llvm.aarch64.sme.fmla.single.vg1x2\0"
  "llvm.aarch64.sme.fmla.single.vg1x4\0"
  "llvm.aarch64.sme.fmla.vg1x2\0"
  "llvm.aarch64.sme.fmla.vg1x4\0"
  "llvm.aarch64.sme.fmlal.lane.vg2x1\0"
  "llvm.aarch64.sme.fmlal.lane.vg2x2\0"
  "llvm.aarch64.sme.fmlal.lane.vg2x4\0"
  "llvm.aarch64.sme.fmlal.single.vg2x1\0"
  "llvm.aarch64.sme.fmlal.single.vg2x2\0"
  "llvm.aarch64.sme.fmlal.single.vg2x4\0"
  "llvm.aarch64.sme.fmlal.vg2x2\0"
  "llvm.aarch64.sme.fmlal.vg2x4\0"
  "llvm.aarch64.sme.fmls.lane.vg1x2\0"
  "llvm.aarch64.sme.fmls.lane.vg1x4\0"
  "llvm.aarch64.sme.fmls.single.vg1x2\0"
  "llvm.aarch64.sme.fmls.single.vg1x4\0"
  "llvm.aarch64.sme.fmls.vg1x2\0"
  "llvm.aarch64.sme.fmls.vg1x4\0"
  "llvm.aarch64.sme.fmlsl.lane.vg2x1\0"
  "llvm.aarch64.sme.fmlsl.lane.vg2x2\0"
  "llvm.aarch64.sme.fmlsl.lane.vg2x4\0"
  "llvm.aarch64.sme.fmlsl.single.vg2x1\0"
  "llvm.aarch64.sme.fmlsl.single.vg2x2\0"
  "llvm.aarch64.sme.fmlsl.single.vg2x4\0"
  "llvm.aarch64.sme.fmlsl.vg2x2\0"
  "llvm.aarch64.sme.fmlsl.vg2x4\0"
  "llvm.aarch64.sme.fp8.fdot.lane.za16.vg1x2\0"
  "llvm.aarch64.sme.fp8.fdot.lane.za16.vg1x4\0"
  "llvm.aarch64.sme.fp8.fdot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.fp8.fdot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.fp8.fdot.multi.za16.vg1x2\0"
  "llvm.aarch64.sme.fp8.fdot.multi.za16.vg1x4\0"
  "llvm.aarch64.sme.fp8.fdot.multi.za32.vg1x2\0"
  "llvm.aarch64.sme.fp8.fdot.multi.za32.vg1x4\0"
  "llvm.aarch64.sme.fp8.fdot.single.za16.vg1x2\0"
  "llvm.aarch64.sme.fp8.fdot.single.za16.vg1x4\0"
  "llvm.aarch64.sme.fp8.fdot.single.za32.vg1x2\0"
  "llvm.aarch64.sme.fp8.fdot.single.za32.vg1x4\0"
  "llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x1\0"
  "llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x2\0"
  "llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x4\0"
  "llvm.aarch64.sme.fp8.fmlal.multi.za16.vg2x2\0"
  "llvm.aarch64.sme.fp8.fmlal.multi.za16.vg2x4\0"
  "llvm.aarch64.sme.fp8.fmlal.single.za16.vg2x2\0"
  "llvm.aarch64.sme.fp8.fmlal.single.za16.vg2x4\0"
  "llvm.aarch64.sme.fp8.fmlal.za16.vg2x1\0"
  "llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x1\0"
  "llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x2\0"
  "llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x4\0"
  "llvm.aarch64.sme.fp8.fmlall.multi.za32.vg4x2\0"
  "llvm.aarch64.sme.fp8.fmlall.multi.za32.vg4x4\0"
  "llvm.aarch64.sme.fp8.fmlall.single.za32.vg4x2\0"
  "llvm.aarch64.sme.fp8.fmlall.single.za32.vg4x4\0"
  "llvm.aarch64.sme.fp8.fmlall.za32.vg4x1\0"
  "llvm.aarch64.sme.fp8.fmop4a.za16.1x1\0"
  "llvm.aarch64.sme.fp8.fmop4a.za16.1x2\0"
  "llvm.aarch64.sme.fp8.fmop4a.za16.2x1\0"
  "llvm.aarch64.sme.fp8.fmop4a.za16.2x2\0"
  "llvm.aarch64.sme.fp8.fmop4a.za32.1x1\0"
  "llvm.aarch64.sme.fp8.fmop4a.za32.1x2\0"
  "llvm.aarch64.sme.fp8.fmop4a.za32.2x1\0"
  "llvm.aarch64.sme.fp8.fmop4a.za32.2x2\0"
  "llvm.aarch64.sme.fp8.fmopa.za16\0"
  "llvm.aarch64.sme.fp8.fmopa.za32\0"
  "llvm.aarch64.sme.fp8.fvdot.lane.za16.vg1x2\0"
  "llvm.aarch64.sme.fp8.fvdotb.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.fp8.fvdott.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.fp8.scale.single.x2\0"
  "llvm.aarch64.sme.fp8.scale.single.x4\0"
  "llvm.aarch64.sme.fp8.scale.x2\0"
  "llvm.aarch64.sme.fp8.scale.x4\0"
  "llvm.aarch64.sme.ftmopa.za16\0"
  "llvm.aarch64.sme.ftmopa.za32\0"
  "llvm.aarch64.sme.fvdot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.get.tpidr2\0"
  "llvm.aarch64.sme.in.streaming.mode\0"
  "llvm.aarch64.sme.ld1b.horiz\0"
  "llvm.aarch64.sme.ld1b.vert\0"
  "llvm.aarch64.sme.ld1d.horiz\0"
  "llvm.aarch64.sme.ld1d.vert\0"
  "llvm.aarch64.sme.ld1h.horiz\0"
  "llvm.aarch64.sme.ld1h.vert\0"
  "llvm.aarch64.sme.ld1q.horiz\0"
  "llvm.aarch64.sme.ld1q.vert\0"
  "llvm.aarch64.sme.ld1w.horiz\0"
  "llvm.aarch64.sme.ld1w.vert\0"
  "llvm.aarch64.sme.ldr\0"
  "llvm.aarch64.sme.ldr.zt\0"
  "llvm.aarch64.sme.luti2.lane.zt\0"
  "llvm.aarch64.sme.luti2.lane.zt.x2\0"
  "llvm.aarch64.sme.luti2.lane.zt.x4\0"
  "llvm.aarch64.sme.luti4.lane.zt\0"
  "llvm.aarch64.sme.luti4.lane.zt.x2\0"
  "llvm.aarch64.sme.luti4.lane.zt.x4\0"
  "llvm.aarch64.sme.luti4.zt.x4\0"
  "llvm.aarch64.sme.mop4a.1x1\0"
  "llvm.aarch64.sme.mop4a.1x2\0"
  "llvm.aarch64.sme.mop4a.2x1\0"
  "llvm.aarch64.sme.mop4a.2x2\0"
  "llvm.aarch64.sme.mop4a.wide.1x1\0"
  "llvm.aarch64.sme.mop4a.wide.1x2\0"
  "llvm.aarch64.sme.mop4a.wide.2x1\0"
  "llvm.aarch64.sme.mop4a.wide.2x2\0"
  "llvm.aarch64.sme.mop4s.1x1\0"
  "llvm.aarch64.sme.mop4s.1x2\0"
  "llvm.aarch64.sme.mop4s.2x1\0"
  "llvm.aarch64.sme.mop4s.2x2\0"
  "llvm.aarch64.sme.mop4s.wide.1x1\0"
  "llvm.aarch64.sme.mop4s.wide.1x2\0"
  "llvm.aarch64.sme.mop4s.wide.2x1\0"
  "llvm.aarch64.sme.mop4s.wide.2x2\0"
  "llvm.aarch64.sme.mopa\0"
  "llvm.aarch64.sme.mopa.nonwide\0"
  "llvm.aarch64.sme.mopa.wide\0"
  "llvm.aarch64.sme.mops\0"
  "llvm.aarch64.sme.mops.nonwide\0"
  "llvm.aarch64.sme.mops.wide\0"
  "llvm.aarch64.sme.read.hor.vg2\0"
  "llvm.aarch64.sme.read.hor.vg4\0"
  "llvm.aarch64.sme.read.horiz\0"
  "llvm.aarch64.sme.read.ver.vg2\0"
  "llvm.aarch64.sme.read.ver.vg4\0"
  "llvm.aarch64.sme.read.vert\0"
  "llvm.aarch64.sme.read.vg1x2\0"
  "llvm.aarch64.sme.read.vg1x4\0"
  "llvm.aarch64.sme.readq.horiz\0"
  "llvm.aarch64.sme.readq.vert\0"
  "llvm.aarch64.sme.readz.horiz\0"
  "llvm.aarch64.sme.readz.horiz.x2\0"
  "llvm.aarch64.sme.readz.horiz.x4\0"
  "llvm.aarch64.sme.readz.q.horiz\0"
  "llvm.aarch64.sme.readz.q.vert\0"
  "llvm.aarch64.sme.readz.vert\0"
  "llvm.aarch64.sme.readz.vert.x2\0"
  "llvm.aarch64.sme.readz.vert.x4\0"
  "llvm.aarch64.sme.readz.x2\0"
  "llvm.aarch64.sme.readz.x4\0"
  "llvm.aarch64.sme.sdot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.sdot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.sdot.lane.za64.vg1x2\0"
  "llvm.aarch64.sme.sdot.lane.za64.vg1x4\0"
  "llvm.aarch64.sme.sdot.single.za32.vg1x2\0"
  "llvm.aarch64.sme.sdot.single.za32.vg1x4\0"
  "llvm.aarch64.sme.sdot.single.za64.vg1x2\0"
  "llvm.aarch64.sme.sdot.single.za64.vg1x4\0"
  "llvm.aarch64.sme.sdot.za32.vg1x2\0"
  "llvm.aarch64.sme.sdot.za32.vg1x4\0"
  "llvm.aarch64.sme.sdot.za64.vg1x2\0"
  "llvm.aarch64.sme.sdot.za64.vg1x4\0"
  "llvm.aarch64.sme.set.tpidr2\0"
  "llvm.aarch64.sme.smla.za32.lane.vg4x1\0"
  "llvm.aarch64.sme.smla.za32.lane.vg4x2\0"
  "llvm.aarch64.sme.smla.za32.lane.vg4x4\0"
  "llvm.aarch64.sme.smla.za32.single.vg4x1\0"
  "llvm.aarch64.sme.smla.za32.single.vg4x2\0"
  "llvm.aarch64.sme.smla.za32.single.vg4x4\0"
  "llvm.aarch64.sme.smla.za32.vg4x2\0"
  "llvm.aarch64.sme.smla.za32.vg4x4\0"
  "llvm.aarch64.sme.smla.za64.lane.vg4x1\0"
  "llvm.aarch64.sme.smla.za64.lane.vg4x2\0"
  "llvm.aarch64.sme.smla.za64.lane.vg4x4\0"
  "llvm.aarch64.sme.smla.za64.single.vg4x1\0"
  "llvm.aarch64.sme.smla.za64.single.vg4x2\0"
  "llvm.aarch64.sme.smla.za64.single.vg4x4\0"
  "llvm.aarch64.sme.smla.za64.vg4x2\0"
  "llvm.aarch64.sme.smla.za64.vg4x4\0"
  "llvm.aarch64.sme.smlal.lane.vg2x1\0"
  "llvm.aarch64.sme.smlal.lane.vg2x2\0"
  "llvm.aarch64.sme.smlal.lane.vg2x4\0"
  "llvm.aarch64.sme.smlal.single.vg2x1\0"
  "llvm.aarch64.sme.smlal.single.vg2x2\0"
  "llvm.aarch64.sme.smlal.single.vg2x4\0"
  "llvm.aarch64.sme.smlal.vg2x2\0"
  "llvm.aarch64.sme.smlal.vg2x4\0"
  "llvm.aarch64.sme.smls.za32.lane.vg4x1\0"
  "llvm.aarch64.sme.smls.za32.lane.vg4x2\0"
  "llvm.aarch64.sme.smls.za32.lane.vg4x4\0"
  "llvm.aarch64.sme.smls.za32.single.vg4x1\0"
  "llvm.aarch64.sme.smls.za32.single.vg4x2\0"
  "llvm.aarch64.sme.smls.za32.single.vg4x4\0"
  "llvm.aarch64.sme.smls.za32.vg4x2\0"
  "llvm.aarch64.sme.smls.za32.vg4x4\0"
  "llvm.aarch64.sme.smls.za64.lane.vg4x1\0"
  "llvm.aarch64.sme.smls.za64.lane.vg4x2\0"
  "llvm.aarch64.sme.smls.za64.lane.vg4x4\0"
  "llvm.aarch64.sme.smls.za64.single.vg4x1\0"
  "llvm.aarch64.sme.smls.za64.single.vg4x2\0"
  "llvm.aarch64.sme.smls.za64.single.vg4x4\0"
  "llvm.aarch64.sme.smls.za64.vg4x2\0"
  "llvm.aarch64.sme.smls.za64.vg4x4\0"
  "llvm.aarch64.sme.smlsl.lane.vg2x1\0"
  "llvm.aarch64.sme.smlsl.lane.vg2x2\0"
  "llvm.aarch64.sme.smlsl.lane.vg2x4\0"
  "llvm.aarch64.sme.smlsl.single.vg2x1\0"
  "llvm.aarch64.sme.smlsl.single.vg2x2\0"
  "llvm.aarch64.sme.smlsl.single.vg2x4\0"
  "llvm.aarch64.sme.smlsl.vg2x2\0"
  "llvm.aarch64.sme.smlsl.vg2x4\0"
  "llvm.aarch64.sme.smop4a.wide.1x1\0"
  "llvm.aarch64.sme.smop4a.wide.1x2\0"
  "llvm.aarch64.sme.smop4a.wide.2x1\0"
  "llvm.aarch64.sme.smop4a.wide.2x2\0"
  "llvm.aarch64.sme.smop4a.za64.wide.1x1\0"
  "llvm.aarch64.sme.smop4a.za64.wide.1x2\0"
  "llvm.aarch64.sme.smop4a.za64.wide.2x1\0"
  "llvm.aarch64.sme.smop4a.za64.wide.2x2\0"
  "llvm.aarch64.sme.smop4s.wide.1x1\0"
  "llvm.aarch64.sme.smop4s.wide.1x2\0"
  "llvm.aarch64.sme.smop4s.wide.2x1\0"
  "llvm.aarch64.sme.smop4s.wide.2x2\0"
  "llvm.aarch64.sme.smop4s.za64.wide.1x1\0"
  "llvm.aarch64.sme.smop4s.za64.wide.1x2\0"
  "llvm.aarch64.sme.smop4s.za64.wide.2x1\0"
  "llvm.aarch64.sme.smop4s.za64.wide.2x2\0"
  "llvm.aarch64.sme.smopa.wide\0"
  "llvm.aarch64.sme.smopa.za32\0"
  "llvm.aarch64.sme.smops.wide\0"
  "llvm.aarch64.sme.smops.za32\0"
  "llvm.aarch64.sme.st1b.horiz\0"
  "llvm.aarch64.sme.st1b.vert\0"
  "llvm.aarch64.sme.st1d.horiz\0"
  "llvm.aarch64.sme.st1d.vert\0"
  "llvm.aarch64.sme.st1h.horiz\0"
  "llvm.aarch64.sme.st1h.vert\0"
  "llvm.aarch64.sme.st1q.horiz\0"
  "llvm.aarch64.sme.st1q.vert\0"
  "llvm.aarch64.sme.st1w.horiz\0"
  "llvm.aarch64.sme.st1w.vert\0"
  "llvm.aarch64.sme.stmopa.za32\0"
  "llvm.aarch64.sme.str\0"
  "llvm.aarch64.sme.str.zt\0"
  "llvm.aarch64.sme.sub.write.single.za.vg1x2\0"
  "llvm.aarch64.sme.sub.write.single.za.vg1x4\0"
  "llvm.aarch64.sme.sub.write.za.vg1x2\0"
  "llvm.aarch64.sme.sub.write.za.vg1x4\0"
  "llvm.aarch64.sme.sub.za16.vg1x2\0"
  "llvm.aarch64.sme.sub.za16.vg1x4\0"
  "llvm.aarch64.sme.sub.za32.vg1x2\0"
  "llvm.aarch64.sme.sub.za32.vg1x4\0"
  "llvm.aarch64.sme.sub.za64.vg1x2\0"
  "llvm.aarch64.sme.sub.za64.vg1x4\0"
  "llvm.aarch64.sme.sudot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.sudot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.sudot.single.za32.vg1x2\0"
  "llvm.aarch64.sme.sudot.single.za32.vg1x4\0"
  "llvm.aarch64.sme.sumla.za32.lane.vg4x1\0"
  "llvm.aarch64.sme.sumla.za32.lane.vg4x2\0"
  "llvm.aarch64.sme.sumla.za32.lane.vg4x4\0"
  "llvm.aarch64.sme.sumla.za32.single.vg4x2\0"
  "llvm.aarch64.sme.sumla.za32.single.vg4x4\0"
  "llvm.aarch64.sme.sumop4a.wide.1x1\0"
  "llvm.aarch64.sme.sumop4a.wide.1x2\0"
  "llvm.aarch64.sme.sumop4a.wide.2x1\0"
  "llvm.aarch64.sme.sumop4a.wide.2x2\0"
  "llvm.aarch64.sme.sumop4a.za64.wide.1x1\0"
  "llvm.aarch64.sme.sumop4a.za64.wide.1x2\0"
  "llvm.aarch64.sme.sumop4a.za64.wide.2x1\0"
  "llvm.aarch64.sme.sumop4a.za64.wide.2x2\0"
  "llvm.aarch64.sme.sumop4s.wide.1x1\0"
  "llvm.aarch64.sme.sumop4s.wide.1x2\0"
  "llvm.aarch64.sme.sumop4s.wide.2x1\0"
  "llvm.aarch64.sme.sumop4s.wide.2x2\0"
  "llvm.aarch64.sme.sumop4s.za64.wide.1x1\0"
  "llvm.aarch64.sme.sumop4s.za64.wide.1x2\0"
  "llvm.aarch64.sme.sumop4s.za64.wide.2x1\0"
  "llvm.aarch64.sme.sumop4s.za64.wide.2x2\0"
  "llvm.aarch64.sme.sumopa.wide\0"
  "llvm.aarch64.sme.sumops.wide\0"
  "llvm.aarch64.sme.sutmopa.za32\0"
  "llvm.aarch64.sme.suvdot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.svdot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.svdot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.svdot.lane.za64.vg1x4\0"
  "llvm.aarch64.sme.udot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.udot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.udot.lane.za64.vg1x2\0"
  "llvm.aarch64.sme.udot.lane.za64.vg1x4\0"
  "llvm.aarch64.sme.udot.single.za32.vg1x2\0"
  "llvm.aarch64.sme.udot.single.za32.vg1x4\0"
  "llvm.aarch64.sme.udot.single.za64.vg1x2\0"
  "llvm.aarch64.sme.udot.single.za64.vg1x4\0"
  "llvm.aarch64.sme.udot.za32.vg1x2\0"
  "llvm.aarch64.sme.udot.za32.vg1x4\0"
  "llvm.aarch64.sme.udot.za64.vg1x2\0"
  "llvm.aarch64.sme.udot.za64.vg1x4\0"
  "llvm.aarch64.sme.umla.za32.lane.vg4x1\0"
  "llvm.aarch64.sme.umla.za32.lane.vg4x2\0"
  "llvm.aarch64.sme.umla.za32.lane.vg4x4\0"
  "llvm.aarch64.sme.umla.za32.single.vg4x1\0"
  "llvm.aarch64.sme.umla.za32.single.vg4x2\0"
  "llvm.aarch64.sme.umla.za32.single.vg4x4\0"
  "llvm.aarch64.sme.umla.za32.vg4x2\0"
  "llvm.aarch64.sme.umla.za32.vg4x4\0"
  "llvm.aarch64.sme.umla.za64.lane.vg4x1\0"
  "llvm.aarch64.sme.umla.za64.lane.vg4x2\0"
  "llvm.aarch64.sme.umla.za64.lane.vg4x4\0"
  "llvm.aarch64.sme.umla.za64.single.vg4x1\0"
  "llvm.aarch64.sme.umla.za64.single.vg4x2\0"
  "llvm.aarch64.sme.umla.za64.single.vg4x4\0"
  "llvm.aarch64.sme.umla.za64.vg4x2\0"
  "llvm.aarch64.sme.umla.za64.vg4x4\0"
  "llvm.aarch64.sme.umlal.lane.vg2x1\0"
  "llvm.aarch64.sme.umlal.lane.vg2x2\0"
  "llvm.aarch64.sme.umlal.lane.vg2x4\0"
  "llvm.aarch64.sme.umlal.single.vg2x1\0"
  "llvm.aarch64.sme.umlal.single.vg2x2\0"
  "llvm.aarch64.sme.umlal.single.vg2x4\0"
  "llvm.aarch64.sme.umlal.vg2x2\0"
  "llvm.aarch64.sme.umlal.vg2x4\0"
  "llvm.aarch64.sme.umls.za32.lane.vg4x1\0"
  "llvm.aarch64.sme.umls.za32.lane.vg4x2\0"
  "llvm.aarch64.sme.umls.za32.lane.vg4x4\0"
  "llvm.aarch64.sme.umls.za32.single.vg4x1\0"
  "llvm.aarch64.sme.umls.za32.single.vg4x2\0"
  "llvm.aarch64.sme.umls.za32.single.vg4x4\0"
  "llvm.aarch64.sme.umls.za32.vg4x2\0"
  "llvm.aarch64.sme.umls.za32.vg4x4\0"
  "llvm.aarch64.sme.umls.za64.lane.vg4x1\0"
  "llvm.aarch64.sme.umls.za64.lane.vg4x2\0"
  "llvm.aarch64.sme.umls.za64.lane.vg4x4\0"
  "llvm.aarch64.sme.umls.za64.single.vg4x1\0"
  "llvm.aarch64.sme.umls.za64.single.vg4x2\0"
  "llvm.aarch64.sme.umls.za64.single.vg4x4\0"
  "llvm.aarch64.sme.umls.za64.vg4x2\0"
  "llvm.aarch64.sme.umls.za64.vg4x4\0"
  "llvm.aarch64.sme.umlsl.lane.vg2x1\0"
  "llvm.aarch64.sme.umlsl.lane.vg2x2\0"
  "llvm.aarch64.sme.umlsl.lane.vg2x4\0"
  "llvm.aarch64.sme.umlsl.single.vg2x1\0"
  "llvm.aarch64.sme.umlsl.single.vg2x2\0"
  "llvm.aarch64.sme.umlsl.single.vg2x4\0"
  "llvm.aarch64.sme.umlsl.vg2x2\0"
  "llvm.aarch64.sme.umlsl.vg2x4\0"
  "llvm.aarch64.sme.umop4a.wide.1x1\0"
  "llvm.aarch64.sme.umop4a.wide.1x2\0"
  "llvm.aarch64.sme.umop4a.wide.2x1\0"
  "llvm.aarch64.sme.umop4a.wide.2x2\0"
  "llvm.aarch64.sme.umop4a.za64.wide.1x1\0"
  "llvm.aarch64.sme.umop4a.za64.wide.1x2\0"
  "llvm.aarch64.sme.umop4a.za64.wide.2x1\0"
  "llvm.aarch64.sme.umop4a.za64.wide.2x2\0"
  "llvm.aarch64.sme.umop4s.wide.1x1\0"
  "llvm.aarch64.sme.umop4s.wide.1x2\0"
  "llvm.aarch64.sme.umop4s.wide.2x1\0"
  "llvm.aarch64.sme.umop4s.wide.2x2\0"
  "llvm.aarch64.sme.umop4s.za64.wide.1x1\0"
  "llvm.aarch64.sme.umop4s.za64.wide.1x2\0"
  "llvm.aarch64.sme.umop4s.za64.wide.2x1\0"
  "llvm.aarch64.sme.umop4s.za64.wide.2x2\0"
  "llvm.aarch64.sme.umopa.wide\0"
  "llvm.aarch64.sme.umopa.za32\0"
  "llvm.aarch64.sme.umops.wide\0"
  "llvm.aarch64.sme.umops.za32\0"
  "llvm.aarch64.sme.usdot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.usdot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.usdot.single.za32.vg1x2\0"
  "llvm.aarch64.sme.usdot.single.za32.vg1x4\0"
  "llvm.aarch64.sme.usdot.za32.vg1x2\0"
  "llvm.aarch64.sme.usdot.za32.vg1x4\0"
  "llvm.aarch64.sme.usmla.za32.lane.vg4x1\0"
  "llvm.aarch64.sme.usmla.za32.lane.vg4x2\0"
  "llvm.aarch64.sme.usmla.za32.lane.vg4x4\0"
  "llvm.aarch64.sme.usmla.za32.single.vg4x1\0"
  "llvm.aarch64.sme.usmla.za32.single.vg4x2\0"
  "llvm.aarch64.sme.usmla.za32.single.vg4x4\0"
  "llvm.aarch64.sme.usmla.za32.vg4x2\0"
  "llvm.aarch64.sme.usmla.za32.vg4x4\0"
  "llvm.aarch64.sme.usmop4a.wide.1x1\0"
  "llvm.aarch64.sme.usmop4a.wide.1x2\0"
  "llvm.aarch64.sme.usmop4a.wide.2x1\0"
  "llvm.aarch64.sme.usmop4a.wide.2x2\0"
  "llvm.aarch64.sme.usmop4a.za64.wide.1x1\0"
  "llvm.aarch64.sme.usmop4a.za64.wide.1x2\0"
  "llvm.aarch64.sme.usmop4a.za64.wide.2x1\0"
  "llvm.aarch64.sme.usmop4a.za64.wide.2x2\0"
  "llvm.aarch64.sme.usmop4s.wide.1x1\0"
  "llvm.aarch64.sme.usmop4s.wide.1x2\0"
  "llvm.aarch64.sme.usmop4s.wide.2x1\0"
  "llvm.aarch64.sme.usmop4s.wide.2x2\0"
  "llvm.aarch64.sme.usmop4s.za64.wide.1x1\0"
  "llvm.aarch64.sme.usmop4s.za64.wide.1x2\0"
  "llvm.aarch64.sme.usmop4s.za64.wide.2x1\0"
  "llvm.aarch64.sme.usmop4s.za64.wide.2x2\0"
  "llvm.aarch64.sme.usmopa.wide\0"
  "llvm.aarch64.sme.usmops.wide\0"
  "llvm.aarch64.sme.ustmopa.za32\0"
  "llvm.aarch64.sme.usvdot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.utmopa.za32\0"
  "llvm.aarch64.sme.uvdot.lane.za32.vg1x2\0"
  "llvm.aarch64.sme.uvdot.lane.za32.vg1x4\0"
  "llvm.aarch64.sme.uvdot.lane.za64.vg1x4\0"
  "llvm.aarch64.sme.write.hor.vg2\0"
  "llvm.aarch64.sme.write.hor.vg4\0"
  "llvm.aarch64.sme.write.horiz\0"
  "llvm.aarch64.sme.write.lane.zt\0"
  "llvm.aarch64.sme.write.ver.vg2\0"
  "llvm.aarch64.sme.write.ver.vg4\0"
  "llvm.aarch64.sme.write.vert\0"
  "llvm.aarch64.sme.write.vg1x2\0"
  "llvm.aarch64.sme.write.vg1x4\0"
  "llvm.aarch64.sme.write.zt\0"
  "llvm.aarch64.sme.writeq.horiz\0"
  "llvm.aarch64.sme.writeq.vert\0"
  "llvm.aarch64.sme.za.disable\0"
  "llvm.aarch64.sme.za.enable\0"
  "llvm.aarch64.sme.zero\0"
  "llvm.aarch64.sme.zero.za64.vg1x2\0"
  "llvm.aarch64.sme.zero.za64.vg1x4\0"
  "llvm.aarch64.sme.zero.za64.vg2x1\0"
  "llvm.aarch64.sme.zero.za64.vg2x2\0"
  "llvm.aarch64.sme.zero.za64.vg2x4\0"
  "llvm.aarch64.sme.zero.za64.vg4x1\0"
  "llvm.aarch64.sme.zero.za64.vg4x2\0"
  "llvm.aarch64.sme.zero.za64.vg4x4\0"
  "llvm.aarch64.sme.zero.zt\0"
  "llvm.aarch64.space\0"
  "llvm.aarch64.st64b\0"
  "llvm.aarch64.st64bv\0"
  "llvm.aarch64.st64bv0\0"
  "llvm.aarch64.stg\0"
  "llvm.aarch64.stgp\0"
  "llvm.aarch64.stlxp\0"
  "llvm.aarch64.stlxr\0"
  "llvm.aarch64.stxp\0"
  "llvm.aarch64.stxr\0"
  "llvm.aarch64.subp\0"
  "llvm.aarch64.sve.abs\0"
  "llvm.aarch64.sve.adclb\0"
  "llvm.aarch64.sve.adclt\0"
  "llvm.aarch64.sve.add\0"
  "llvm.aarch64.sve.add.single.x2\0"
  "llvm.aarch64.sve.add.single.x4\0"
  "llvm.aarch64.sve.add.u\0"
  "llvm.aarch64.sve.addhnb\0"
  "llvm.aarch64.sve.addhnt\0"
  "llvm.aarch64.sve.addp\0"
  "llvm.aarch64.sve.addqv\0"
  "llvm.aarch64.sve.adrb\0"
  "llvm.aarch64.sve.adrd\0"
  "llvm.aarch64.sve.adrh\0"
  "llvm.aarch64.sve.adrw\0"
  "llvm.aarch64.sve.aesd\0"
  "llvm.aarch64.sve.aesd.lane.x2\0"
  "llvm.aarch64.sve.aesd.lane.x4\0"
  "llvm.aarch64.sve.aesdimc.lane.x2\0"
  "llvm.aarch64.sve.aesdimc.lane.x4\0"
  "llvm.aarch64.sve.aese\0"
  "llvm.aarch64.sve.aese.lane.x2\0"
  "llvm.aarch64.sve.aese.lane.x4\0"
  "llvm.aarch64.sve.aesemc.lane.x2\0"
  "llvm.aarch64.sve.aesemc.lane.x4\0"
  "llvm.aarch64.sve.aesimc\0"
  "llvm.aarch64.sve.aesmc\0"
  "llvm.aarch64.sve.and\0"
  "llvm.aarch64.sve.and.u\0"
  "llvm.aarch64.sve.and.z\0"
  "llvm.aarch64.sve.andqv\0"
  "llvm.aarch64.sve.andv\0"
  "llvm.aarch64.sve.asr\0"
  "llvm.aarch64.sve.asr.u\0"
  "llvm.aarch64.sve.asr.wide\0"
  "llvm.aarch64.sve.asrd\0"
  "llvm.aarch64.sve.bcax\0"
  "llvm.aarch64.sve.bdep.x\0"
  "llvm.aarch64.sve.bext.x\0"
  "llvm.aarch64.sve.bfclamp.single.x2\0"
  "llvm.aarch64.sve.bfclamp.single.x4\0"
  "llvm.aarch64.sve.bfcvt.x2\0"
  "llvm.aarch64.sve.bfcvtn.x2\0"
  "llvm.aarch64.sve.bfdot\0"
  "llvm.aarch64.sve.bfdot.lane.v2\0"
  "llvm.aarch64.sve.bfmlalb\0"
  "llvm.aarch64.sve.bfmlalb.lane.v2\0"
  "llvm.aarch64.sve.bfmlalt\0"
  "llvm.aarch64.sve.bfmlalt.lane.v2\0"
  "llvm.aarch64.sve.bfmlslb\0"
  "llvm.aarch64.sve.bfmlslb.lane\0"
  "llvm.aarch64.sve.bfmlslt\0"
  "llvm.aarch64.sve.bfmlslt.lane\0"
  "llvm.aarch64.sve.bfmmla\0"
  "llvm.aarch64.sve.bgrp.x\0"
  "llvm.aarch64.sve.bic\0"
  "llvm.aarch64.sve.bic.u\0"
  "llvm.aarch64.sve.bic.z\0"
  "llvm.aarch64.sve.brka\0"
  "llvm.aarch64.sve.brka.z\0"
  "llvm.aarch64.sve.brkb\0"
  "llvm.aarch64.sve.brkb.z\0"
  "llvm.aarch64.sve.brkn.z\0"
  "llvm.aarch64.sve.brkpa.z\0"
  "llvm.aarch64.sve.brkpb.z\0"
  "llvm.aarch64.sve.bsl\0"
  "llvm.aarch64.sve.bsl1n\0"
  "llvm.aarch64.sve.bsl2n\0"
  "llvm.aarch64.sve.cadd.x\0"
  "llvm.aarch64.sve.cdot\0"
  "llvm.aarch64.sve.cdot.lane\0"
  "llvm.aarch64.sve.clasta\0"
  "llvm.aarch64.sve.clasta.n\0"
  "llvm.aarch64.sve.clastb\0"
  "llvm.aarch64.sve.clastb.n\0"
  "llvm.aarch64.sve.cls\0"
  "llvm.aarch64.sve.clz\0"
  "llvm.aarch64.sve.cmla.lane.x\0"
  "llvm.aarch64.sve.cmla.x\0"
  "llvm.aarch64.sve.cmpeq\0"
  "llvm.aarch64.sve.cmpeq.wide\0"
  "llvm.aarch64.sve.cmpge\0"
  "llvm.aarch64.sve.cmpge.wide\0"
  "llvm.aarch64.sve.cmpgt\0"
  "llvm.aarch64.sve.cmpgt.wide\0"
  "llvm.aarch64.sve.cmphi\0"
  "llvm.aarch64.sve.cmphi.wide\0"
  "llvm.aarch64.sve.cmphs\0"
  "llvm.aarch64.sve.cmphs.wide\0"
  "llvm.aarch64.sve.cmple.wide\0"
  "llvm.aarch64.sve.cmplo.wide\0"
  "llvm.aarch64.sve.cmpls.wide\0"
  "llvm.aarch64.sve.cmplt.wide\0"
  "llvm.aarch64.sve.cmpne\0"
  "llvm.aarch64.sve.cmpne.wide\0"
  "llvm.aarch64.sve.cnot\0"
  "llvm.aarch64.sve.cnt\0"
  "llvm.aarch64.sve.cntb\0"
  "llvm.aarch64.sve.cntd\0"
  "llvm.aarch64.sve.cnth\0"
  "llvm.aarch64.sve.cntp\0"
  "llvm.aarch64.sve.cntp.c16\0"
  "llvm.aarch64.sve.cntp.c32\0"
  "llvm.aarch64.sve.cntp.c64\0"
  "llvm.aarch64.sve.cntp.c8\0"
  "llvm.aarch64.sve.cntw\0"
  "llvm.aarch64.sve.compact\0"
  "llvm.aarch64.sve.convert.from.svbool\0"
  "llvm.aarch64.sve.convert.to.svbool\0"
  "llvm.aarch64.sve.dup\0"
  "llvm.aarch64.sve.dup.laneq\0"
  "llvm.aarch64.sve.dup.x\0"
  "llvm.aarch64.sve.dupq.lane\0"
  "llvm.aarch64.sve.eor\0"
  "llvm.aarch64.sve.eor.u\0"
  "llvm.aarch64.sve.eor.z\0"
  "llvm.aarch64.sve.eor3\0"
  "llvm.aarch64.sve.eorbt\0"
  "llvm.aarch64.sve.eorqv\0"
  "llvm.aarch64.sve.eortb\0"
  "llvm.aarch64.sve.eorv\0"
  "llvm.aarch64.sve.expand\0"
  "llvm.aarch64.sve.ext\0"
  "llvm.aarch64.sve.extq\0"
  "llvm.aarch64.sve.fabd\0"
  "llvm.aarch64.sve.fabd.u\0"
  "llvm.aarch64.sve.fabs\0"
  "llvm.aarch64.sve.facge\0"
  "llvm.aarch64.sve.facgt\0"
  "llvm.aarch64.sve.fadd\0"
  "llvm.aarch64.sve.fadd.u\0"
  "llvm.aarch64.sve.fadda\0"
  "llvm.aarch64.sve.faddp\0"
  "llvm.aarch64.sve.faddqv\0"
  "llvm.aarch64.sve.faddv\0"
  "llvm.aarch64.sve.famax\0"
  "llvm.aarch64.sve.famax.u\0"
  "llvm.aarch64.sve.famin\0"
  "llvm.aarch64.sve.famin.u\0"
  "llvm.aarch64.sve.fcadd\0"
  "llvm.aarch64.sve.fclamp\0"
  "llvm.aarch64.sve.fclamp.single.x2\0"
  "llvm.aarch64.sve.fclamp.single.x4\0"
  "llvm.aarch64.sve.fcmla\0"
  "llvm.aarch64.sve.fcmla.lane\0"
  "llvm.aarch64.sve.fcmpeq\0"
  "llvm.aarch64.sve.fcmpge\0"
  "llvm.aarch64.sve.fcmpgt\0"
  "llvm.aarch64.sve.fcmpne\0"
  "llvm.aarch64.sve.fcmpuo\0"
  "llvm.aarch64.sve.fcvt\0"
  "llvm.aarch64.sve.fcvt.bf16f32.v2\0"
  "llvm.aarch64.sve.fcvt.f16f32\0"
  "llvm.aarch64.sve.fcvt.f16f64\0"
  "llvm.aarch64.sve.fcvt.f32f16\0"
  "llvm.aarch64.sve.fcvt.f32f64\0"
  "llvm.aarch64.sve.fcvt.f64f16\0"
  "llvm.aarch64.sve.fcvt.f64f32\0"
  "llvm.aarch64.sve.fcvt.widen.x2\0"
  "llvm.aarch64.sve.fcvt.x2\0"
  "llvm.aarch64.sve.fcvtl.widen.x2\0"
  "llvm.aarch64.sve.fcvtlt.f32f16\0"
  "llvm.aarch64.sve.fcvtlt.f64f32\0"
  "llvm.aarch64.sve.fcvtn.x2\0"
  "llvm.aarch64.sve.fcvtnt.bf16f32.v2\0"
  "llvm.aarch64.sve.fcvtnt.f16f32\0"
  "llvm.aarch64.sve.fcvtnt.f32f64\0"
  "llvm.aarch64.sve.fcvtnt.z.bf16f32\0"
  "llvm.aarch64.sve.fcvtnt.z.f16f32\0"
  "llvm.aarch64.sve.fcvtnt.z.f32f64\0"
  "llvm.aarch64.sve.fcvtx.f32f64\0"
  "llvm.aarch64.sve.fcvtxnt.f32f64\0"
  "llvm.aarch64.sve.fcvtxnt.z.f32f64\0"
  "llvm.aarch64.sve.fcvtzs\0"
  "llvm.aarch64.sve.fcvtzs.i32f16\0"
  "llvm.aarch64.sve.fcvtzs.i32f64\0"
  "llvm.aarch64.sve.fcvtzs.i64f16\0"
  "llvm.aarch64.sve.fcvtzs.i64f32\0"
  "llvm.aarch64.sve.fcvtzs.x2\0"
  "llvm.aarch64.sve.fcvtzs.x4\0"
  "llvm.aarch64.sve.fcvtzu\0"
  "llvm.aarch64.sve.fcvtzu.i32f16\0"
  "llvm.aarch64.sve.fcvtzu.i32f64\0"
  "llvm.aarch64.sve.fcvtzu.i64f16\0"
  "llvm.aarch64.sve.fcvtzu.i64f32\0"
  "llvm.aarch64.sve.fcvtzu.x2\0"
  "llvm.aarch64.sve.fcvtzu.x4\0"
  "llvm.aarch64.sve.fdiv\0"
  "llvm.aarch64.sve.fdiv.u\0"
  "llvm.aarch64.sve.fdivr\0"
  "llvm.aarch64.sve.fdot.lane.x2\0"
  "llvm.aarch64.sve.fdot.x2\0"
  "llvm.aarch64.sve.fexpa.x\0"
  "llvm.aarch64.sve.firstp\0"
  "llvm.aarch64.sve.flogb\0"
  "llvm.aarch64.sve.fmad\0"
  "llvm.aarch64.sve.fmax\0"
  "llvm.aarch64.sve.fmax.single.x2\0"
  "llvm.aarch64.sve.fmax.single.x4\0"
  "llvm.aarch64.sve.fmax.u\0"
  "llvm.aarch64.sve.fmax.x2\0"
  "llvm.aarch64.sve.fmax.x4\0"
  "llvm.aarch64.sve.fmaxnm\0"
  "llvm.aarch64.sve.fmaxnm.single.x2\0"
  "llvm.aarch64.sve.fmaxnm.single.x4\0"
  "llvm.aarch64.sve.fmaxnm.u\0"
  "llvm.aarch64.sve.fmaxnm.x2\0"
  "llvm.aarch64.sve.fmaxnm.x4\0"
  "llvm.aarch64.sve.fmaxnmp\0"
  "llvm.aarch64.sve.fmaxnmqv\0"
  "llvm.aarch64.sve.fmaxnmv\0"
  "llvm.aarch64.sve.fmaxp\0"
  "llvm.aarch64.sve.fmaxqv\0"
  "llvm.aarch64.sve.fmaxv\0"
  "llvm.aarch64.sve.fmin\0"
  "llvm.aarch64.sve.fmin.single.x2\0"
  "llvm.aarch64.sve.fmin.single.x4\0"
  "llvm.aarch64.sve.fmin.u\0"
  "llvm.aarch64.sve.fmin.x2\0"
  "llvm.aarch64.sve.fmin.x4\0"
  "llvm.aarch64.sve.fminnm\0"
  "llvm.aarch64.sve.fminnm.single.x2\0"
  "llvm.aarch64.sve.fminnm.single.x4\0"
  "llvm.aarch64.sve.fminnm.u\0"
  "llvm.aarch64.sve.fminnm.x2\0"
  "llvm.aarch64.sve.fminnm.x4\0"
  "llvm.aarch64.sve.fminnmp\0"
  "llvm.aarch64.sve.fminnmqv\0"
  "llvm.aarch64.sve.fminnmv\0"
  "llvm.aarch64.sve.fminp\0"
  "llvm.aarch64.sve.fminqv\0"
  "llvm.aarch64.sve.fminv\0"
  "llvm.aarch64.sve.fmla\0"
  "llvm.aarch64.sve.fmla.lane\0"
  "llvm.aarch64.sve.fmla.u\0"
  "llvm.aarch64.sve.fmlalb\0"
  "llvm.aarch64.sve.fmlalb.lane\0"
  "llvm.aarch64.sve.fmlalt\0"
  "llvm.aarch64.sve.fmlalt.lane\0"
  "llvm.aarch64.sve.fmls\0"
  "llvm.aarch64.sve.fmls.lane\0"
  "llvm.aarch64.sve.fmls.u\0"
  "llvm.aarch64.sve.fmlslb\0"
  "llvm.aarch64.sve.fmlslb.lane\0"
  "llvm.aarch64.sve.fmlslt\0"
  "llvm.aarch64.sve.fmlslt.lane\0"
  "llvm.aarch64.sve.fmmla\0"
  "llvm.aarch64.sve.fmsb\0"
  "llvm.aarch64.sve.fmul\0"
  "llvm.aarch64.sve.fmul.lane\0"
  "llvm.aarch64.sve.fmul.single.x2\0"
  "llvm.aarch64.sve.fmul.single.x4\0"
  "llvm.aarch64.sve.fmul.u\0"
  "llvm.aarch64.sve.fmul.x2\0"
  "llvm.aarch64.sve.fmul.x4\0"
  "llvm.aarch64.sve.fmulx\0"
  "llvm.aarch64.sve.fmulx.u\0"
  "llvm.aarch64.sve.fneg\0"
  "llvm.aarch64.sve.fnmad\0"
  "llvm.aarch64.sve.fnmla\0"
  "llvm.aarch64.sve.fnmla.u\0"
  "llvm.aarch64.sve.fnmls\0"
  "llvm.aarch64.sve.fnmls.u\0"
  "llvm.aarch64.sve.fnmsb\0"
  "llvm.aarch64.sve.fp8.cvt.x2\0"
  "llvm.aarch64.sve.fp8.cvt.x4\0"
  "llvm.aarch64.sve.fp8.cvt1\0"
  "llvm.aarch64.sve.fp8.cvt1.x2\0"
  "llvm.aarch64.sve.fp8.cvt2\0"
  "llvm.aarch64.sve.fp8.cvt2.x2\0"
  "llvm.aarch64.sve.fp8.cvtl1.x2\0"
  "llvm.aarch64.sve.fp8.cvtl2.x2\0"
  "llvm.aarch64.sve.fp8.cvtlt1\0"
  "llvm.aarch64.sve.fp8.cvtlt2\0"
  "llvm.aarch64.sve.fp8.cvtn\0"
  "llvm.aarch64.sve.fp8.cvtn.x4\0"
  "llvm.aarch64.sve.fp8.cvtnb\0"
  "llvm.aarch64.sve.fp8.cvtnt\0"
  "llvm.aarch64.sve.fp8.fdot\0"
  "llvm.aarch64.sve.fp8.fdot.lane\0"
  "llvm.aarch64.sve.fp8.fmlalb\0"
  "llvm.aarch64.sve.fp8.fmlalb.lane\0"
  "llvm.aarch64.sve.fp8.fmlallbb\0"
  "llvm.aarch64.sve.fp8.fmlallbb.lane\0"
  "llvm.aarch64.sve.fp8.fmlallbt\0"
  "llvm.aarch64.sve.fp8.fmlallbt.lane\0"
  "llvm.aarch64.sve.fp8.fmlalltb\0"
  "llvm.aarch64.sve.fp8.fmlalltb.lane\0"
  "llvm.aarch64.sve.fp8.fmlalltt\0"
  "llvm.aarch64.sve.fp8.fmlalltt.lane\0"
  "llvm.aarch64.sve.fp8.fmlalt\0"
  "llvm.aarch64.sve.fp8.fmlalt.lane\0"
  "llvm.aarch64.sve.fp8.fmmla\0"
  "llvm.aarch64.sve.frecpe.x\0"
  "llvm.aarch64.sve.frecps.x\0"
  "llvm.aarch64.sve.frecpx\0"
  "llvm.aarch64.sve.frint32x\0"
  "llvm.aarch64.sve.frint32z\0"
  "llvm.aarch64.sve.frint64x\0"
  "llvm.aarch64.sve.frint64z\0"
  "llvm.aarch64.sve.frinta\0"
  "llvm.aarch64.sve.frinta.x2\0"
  "llvm.aarch64.sve.frinta.x4\0"
  "llvm.aarch64.sve.frinti\0"
  "llvm.aarch64.sve.frintm\0"
  "llvm.aarch64.sve.frintm.x2\0"
  "llvm.aarch64.sve.frintm.x4\0"
  "llvm.aarch64.sve.frintn\0"
  "llvm.aarch64.sve.frintn.x2\0"
  "llvm.aarch64.sve.frintn.x4\0"
  "llvm.aarch64.sve.frintp\0"
  "llvm.aarch64.sve.frintp.x2\0"
  "llvm.aarch64.sve.frintp.x4\0"
  "llvm.aarch64.sve.frintx\0"
  "llvm.aarch64.sve.frintz\0"
  "llvm.aarch64.sve.frsqrte.x\0"
  "llvm.aarch64.sve.frsqrts.x\0"
  "llvm.aarch64.sve.fscale\0"
  "llvm.aarch64.sve.fscale.single.x2\0"
  "llvm.aarch64.sve.fscale.single.x4\0"
  "llvm.aarch64.sve.fscale.x2\0"
  "llvm.aarch64.sve.fscale.x4\0"
  "llvm.aarch64.sve.fsqrt\0"
  "llvm.aarch64.sve.fsub\0"
  "llvm.aarch64.sve.fsub.u\0"
  "llvm.aarch64.sve.fsubr\0"
  "llvm.aarch64.sve.ftmad.x\0"
  "llvm.aarch64.sve.ftsmul.x\0"
  "llvm.aarch64.sve.ftssel.x\0"
  "llvm.aarch64.sve.histcnt\0"
  "llvm.aarch64.sve.histseg\0"
  "llvm.aarch64.sve.index\0"
  "llvm.aarch64.sve.insr\0"
  "llvm.aarch64.sve.lasta\0"
  "llvm.aarch64.sve.lastb\0"
  "llvm.aarch64.sve.lastp\0"
  "llvm.aarch64.sve.ld1\0"
  "llvm.aarch64.sve.ld1.gather\0"
  "llvm.aarch64.sve.ld1.gather.index\0"
  "llvm.aarch64.sve.ld1.gather.scalar.offset\0"
  "llvm.aarch64.sve.ld1.gather.sxtw\0"
  "llvm.aarch64.sve.ld1.gather.sxtw.index\0"
  "llvm.aarch64.sve.ld1.gather.uxtw\0"
  "llvm.aarch64.sve.ld1.gather.uxtw.index\0"
  "llvm.aarch64.sve.ld1.pn.x2\0"
  "llvm.aarch64.sve.ld1.pn.x4\0"
  "llvm.aarch64.sve.ld1q.gather.index\0"
  "llvm.aarch64.sve.ld1q.gather.scalar.offset\0"
  "llvm.aarch64.sve.ld1q.gather.vector.offset\0"
  "llvm.aarch64.sve.ld1ro\0"
  "llvm.aarch64.sve.ld1rq\0"
  "llvm.aarch64.sve.ld1udq\0"
  "llvm.aarch64.sve.ld1uwq\0"
  "llvm.aarch64.sve.ld2.sret\0"
  "llvm.aarch64.sve.ld2q.sret\0"
  "llvm.aarch64.sve.ld3.sret\0"
  "llvm.aarch64.sve.ld3q.sret\0"
  "llvm.aarch64.sve.ld4.sret\0"
  "llvm.aarch64.sve.ld4q.sret\0"
  "llvm.aarch64.sve.ldff1\0"
  "llvm.aarch64.sve.ldff1.gather\0"
  "llvm.aarch64.sve.ldff1.gather.index\0"
  "llvm.aarch64.sve.ldff1.gather.scalar.offset\0"
  "llvm.aarch64.sve.ldff1.gather.sxtw\0"
  "llvm.aarch64.sve.ldff1.gather.sxtw.index\0"
  "llvm.aarch64.sve.ldff1.gather.uxtw\0"
  "llvm.aarch64.sve.ldff1.gather.uxtw.index\0"
  "llvm.aarch64.sve.ldnf1\0"
  "llvm.aarch64.sve.ldnt1\0"
  "llvm.aarch64.sve.ldnt1.gather\0"
  "llvm.aarch64.sve.ldnt1.gather.index\0"
  "llvm.aarch64.sve.ldnt1.gather.scalar.offset\0"
  "llvm.aarch64.sve.ldnt1.gather.uxtw\0"
  "llvm.aarch64.sve.ldnt1.pn.x2\0"
  "llvm.aarch64.sve.ldnt1.pn.x4\0"
  "llvm.aarch64.sve.lsl\0"
  "llvm.aarch64.sve.lsl.u\0"
  "llvm.aarch64.sve.lsl.wide\0"
  "llvm.aarch64.sve.lsr\0"
  "llvm.aarch64.sve.lsr.u\0"
  "llvm.aarch64.sve.lsr.wide\0"
  "llvm.aarch64.sve.luti2.lane\0"
  "llvm.aarch64.sve.luti4.lane\0"
  "llvm.aarch64.sve.luti4.lane.x2\0"
  "llvm.aarch64.sve.mad\0"
  "llvm.aarch64.sve.match\0"
  "llvm.aarch64.sve.mla\0"
  "llvm.aarch64.sve.mla.lane\0"
  "llvm.aarch64.sve.mla.u\0"
  "llvm.aarch64.sve.mls\0"
  "llvm.aarch64.sve.mls.lane\0"
  "llvm.aarch64.sve.mls.u\0"
  "llvm.aarch64.sve.msb\0"
  "llvm.aarch64.sve.mul\0"
  "llvm.aarch64.sve.mul.lane\0"
  "llvm.aarch64.sve.mul.u\0"
  "llvm.aarch64.sve.nand.z\0"
  "llvm.aarch64.sve.nbsl\0"
  "llvm.aarch64.sve.neg\0"
  "llvm.aarch64.sve.nmatch\0"
  "llvm.aarch64.sve.nor.z\0"
  "llvm.aarch64.sve.not\0"
  "llvm.aarch64.sve.orn.z\0"
  "llvm.aarch64.sve.orqv\0"
  "llvm.aarch64.sve.orr\0"
  "llvm.aarch64.sve.orr.u\0"
  "llvm.aarch64.sve.orr.z\0"
  "llvm.aarch64.sve.orv\0"
  "llvm.aarch64.sve.pext\0"
  "llvm.aarch64.sve.pext.x2\0"
  "llvm.aarch64.sve.pfirst\0"
  "llvm.aarch64.sve.pmlal.pair.x2\0"
  "llvm.aarch64.sve.pmov.to.pred.lane\0"
  "llvm.aarch64.sve.pmov.to.pred.lane.zero\0"
  "llvm.aarch64.sve.pmov.to.vector.lane.merging\0"
  "llvm.aarch64.sve.pmov.to.vector.lane.zeroing\0"
  "llvm.aarch64.sve.pmul\0"
  "llvm.aarch64.sve.pmull.pair.x2\0"
  "llvm.aarch64.sve.pmullb.pair\0"
  "llvm.aarch64.sve.pmullt.pair\0"
  "llvm.aarch64.sve.pnext\0"
  "llvm.aarch64.sve.prf\0"
  "llvm.aarch64.sve.prfb.gather.index\0"
  "llvm.aarch64.sve.prfb.gather.scalar.offset\0"
  "llvm.aarch64.sve.prfb.gather.sxtw.index\0"
  "llvm.aarch64.sve.prfb.gather.uxtw.index\0"
  "llvm.aarch64.sve.prfd.gather.index\0"
  "llvm.aarch64.sve.prfd.gather.scalar.offset\0"
  "llvm.aarch64.sve.prfd.gather.sxtw.index\0"
  "llvm.aarch64.sve.prfd.gather.uxtw.index\0"
  "llvm.aarch64.sve.prfh.gather.index\0"
  "llvm.aarch64.sve.prfh.gather.scalar.offset\0"
  "llvm.aarch64.sve.prfh.gather.sxtw.index\0"
  "llvm.aarch64.sve.prfh.gather.uxtw.index\0"
  "llvm.aarch64.sve.prfw.gather.index\0"
  "llvm.aarch64.sve.prfw.gather.scalar.offset\0"
  "llvm.aarch64.sve.prfw.gather.sxtw.index\0"
  "llvm.aarch64.sve.prfw.gather.uxtw.index\0"
  "llvm.aarch64.sve.psel\0"
  "llvm.aarch64.sve.ptest.any\0"
  "llvm.aarch64.sve.ptest.first\0"
  "llvm.aarch64.sve.ptest.last\0"
  "llvm.aarch64.sve.ptrue\0"
  "llvm.aarch64.sve.ptrue.c16\0"
  "llvm.aarch64.sve.ptrue.c32\0"
  "llvm.aarch64.sve.ptrue.c64\0"
  "llvm.aarch64.sve.ptrue.c8\0"
  "llvm.aarch64.sve.punpkhi\0"
  "llvm.aarch64.sve.punpklo\0"
  "llvm.aarch64.sve.raddhnb\0"
  "llvm.aarch64.sve.raddhnt\0"
  "llvm.aarch64.sve.rax1\0"
  "llvm.aarch64.sve.rbit\0"
  "llvm.aarch64.sve.rdffr\0"
  "llvm.aarch64.sve.rdffr.z\0"
  "llvm.aarch64.sve.rev.b16\0"
  "llvm.aarch64.sve.rev.b32\0"
  "llvm.aarch64.sve.rev.b64\0"
  "llvm.aarch64.sve.revb\0"
  "llvm.aarch64.sve.revd\0"
  "llvm.aarch64.sve.revh\0"
  "llvm.aarch64.sve.revw\0"
  "llvm.aarch64.sve.rshrnb\0"
  "llvm.aarch64.sve.rshrnt\0"
  "llvm.aarch64.sve.rsubhnb\0"
  "llvm.aarch64.sve.rsubhnt\0"
  "llvm.aarch64.sve.saba\0"
  "llvm.aarch64.sve.sabalb\0"
  "llvm.aarch64.sve.sabalt\0"
  "llvm.aarch64.sve.sabd\0"
  "llvm.aarch64.sve.sabd.u\0"
  "llvm.aarch64.sve.sabdlb\0"
  "llvm.aarch64.sve.sabdlt\0"
  "llvm.aarch64.sve.sadalp\0"
  "llvm.aarch64.sve.saddlb\0"
  "llvm.aarch64.sve.saddlbt\0"
  "llvm.aarch64.sve.saddlt\0"
  "llvm.aarch64.sve.saddv\0"
  "llvm.aarch64.sve.saddwb\0"
  "llvm.aarch64.sve.saddwt\0"
  "llvm.aarch64.sve.sbclb\0"
  "llvm.aarch64.sve.sbclt\0"
  "llvm.aarch64.sve.sclamp\0"
  "llvm.aarch64.sve.sclamp.single.x2\0"
  "llvm.aarch64.sve.sclamp.single.x4\0"
  "llvm.aarch64.sve.scvtf\0"
  "llvm.aarch64.sve.scvtf.f16i32\0"
  "llvm.aarch64.sve.scvtf.f16i64\0"
  "llvm.aarch64.sve.scvtf.f32i64\0"
  "llvm.aarch64.sve.scvtf.f64i32\0"
  "llvm.aarch64.sve.scvtf.x2\0"
  "llvm.aarch64.sve.scvtf.x4\0"
  "llvm.aarch64.sve.sdiv\0"
  "llvm.aarch64.sve.sdiv.u\0"
  "llvm.aarch64.sve.sdivr\0"
  "llvm.aarch64.sve.sdot\0"
  "llvm.aarch64.sve.sdot.lane\0"
  "llvm.aarch64.sve.sdot.lane.x2\0"
  "llvm.aarch64.sve.sdot.x2\0"
  "llvm.aarch64.sve.sel\0"
  "llvm.aarch64.sve.sel.x2\0"
  "llvm.aarch64.sve.sel.x4\0"
  "llvm.aarch64.sve.setffr\0"
  "llvm.aarch64.sve.shadd\0"
  "llvm.aarch64.sve.shrnb\0"
  "llvm.aarch64.sve.shrnt\0"
  "llvm.aarch64.sve.shsub\0"
  "llvm.aarch64.sve.shsub.u\0"
  "llvm.aarch64.sve.shsubr\0"
  "llvm.aarch64.sve.sli\0"
  "llvm.aarch64.sve.sm4e\0"
  "llvm.aarch64.sve.sm4ekey\0"
  "llvm.aarch64.sve.smax\0"
  "llvm.aarch64.sve.smax.single.x2\0"
  "llvm.aarch64.sve.smax.single.x4\0"
  "llvm.aarch64.sve.smax.u\0"
  "llvm.aarch64.sve.smax.x2\0"
  "llvm.aarch64.sve.smax.x4\0"
  "llvm.aarch64.sve.smaxp\0"
  "llvm.aarch64.sve.smaxqv\0"
  "llvm.aarch64.sve.smaxv\0"
  "llvm.aarch64.sve.smin\0"
  "llvm.aarch64.sve.smin.single.x2\0"
  "llvm.aarch64.sve.smin.single.x4\0"
  "llvm.aarch64.sve.smin.u\0"
  "llvm.aarch64.sve.smin.x2\0"
  "llvm.aarch64.sve.smin.x4\0"
  "llvm.aarch64.sve.sminp\0"
  "llvm.aarch64.sve.sminqv\0"
  "llvm.aarch64.sve.sminv\0"
  "llvm.aarch64.sve.smlalb\0"
  "llvm.aarch64.sve.smlalb.lane\0"
  "llvm.aarch64.sve.smlalt\0"
  "llvm.aarch64.sve.smlalt.lane\0"
  "llvm.aarch64.sve.smlslb\0"
  "llvm.aarch64.sve.smlslb.lane\0"
  "llvm.aarch64.sve.smlslt\0"
  "llvm.aarch64.sve.smlslt.lane\0"
  "llvm.aarch64.sve.smmla\0"
  "llvm.aarch64.sve.smulh\0"
  "llvm.aarch64.sve.smulh.u\0"
  "llvm.aarch64.sve.smullb\0"
  "llvm.aarch64.sve.smullb.lane\0"
  "llvm.aarch64.sve.smullt\0"
  "llvm.aarch64.sve.smullt.lane\0"
  "llvm.aarch64.sve.splice\0"
  "llvm.aarch64.sve.sqabs\0"
  "llvm.aarch64.sve.sqadd\0"
  "llvm.aarch64.sve.sqadd.x\0"
  "llvm.aarch64.sve.sqcadd.x\0"
  "llvm.aarch64.sve.sqcvt.x2\0"
  "llvm.aarch64.sve.sqcvt.x4\0"
  "llvm.aarch64.sve.sqcvtn.x2\0"
  "llvm.aarch64.sve.sqcvtn.x4\0"
  "llvm.aarch64.sve.sqcvtu.x2\0"
  "llvm.aarch64.sve.sqcvtu.x4\0"
  "llvm.aarch64.sve.sqcvtun.x2\0"
  "llvm.aarch64.sve.sqcvtun.x4\0"
  "llvm.aarch64.sve.sqdecb.n32\0"
  "llvm.aarch64.sve.sqdecb.n64\0"
  "llvm.aarch64.sve.sqdecd\0"
  "llvm.aarch64.sve.sqdecd.n32\0"
  "llvm.aarch64.sve.sqdecd.n64\0"
  "llvm.aarch64.sve.sqdech\0"
  "llvm.aarch64.sve.sqdech.n32\0"
  "llvm.aarch64.sve.sqdech.n64\0"
  "llvm.aarch64.sve.sqdecp\0"
  "llvm.aarch64.sve.sqdecp.n32\0"
  "llvm.aarch64.sve.sqdecp.n64\0"
  "llvm.aarch64.sve.sqdecw\0"
  "llvm.aarch64.sve.sqdecw.n32\0"
  "llvm.aarch64.sve.sqdecw.n64\0"
  "llvm.aarch64.sve.sqdmlalb\0"
  "llvm.aarch64.sve.sqdmlalb.lane\0"
  "llvm.aarch64.sve.sqdmlalbt\0"
  "llvm.aarch64.sve.sqdmlalt\0"
  "llvm.aarch64.sve.sqdmlalt.lane\0"
  "llvm.aarch64.sve.sqdmlslb\0"
  "llvm.aarch64.sve.sqdmlslb.lane\0"
  "llvm.aarch64.sve.sqdmlslbt\0"
  "llvm.aarch64.sve.sqdmlslt\0"
  "llvm.aarch64.sve.sqdmlslt.lane\0"
  "llvm.aarch64.sve.sqdmulh\0"
  "llvm.aarch64.sve.sqdmulh.lane\0"
  "llvm.aarch64.sve.sqdmulh.single.vgx2\0"
  "llvm.aarch64.sve.sqdmulh.single.vgx4\0"
  "llvm.aarch64.sve.sqdmulh.vgx2\0"
  "llvm.aarch64.sve.sqdmulh.vgx4\0"
  "llvm.aarch64.sve.sqdmullb\0"
  "llvm.aarch64.sve.sqdmullb.lane\0"
  "llvm.aarch64.sve.sqdmullt\0"
  "llvm.aarch64.sve.sqdmullt.lane\0"
  "llvm.aarch64.sve.sqincb.n32\0"
  "llvm.aarch64.sve.sqincb.n64\0"
  "llvm.aarch64.sve.sqincd\0"
  "llvm.aarch64.sve.sqincd.n32\0"
  "llvm.aarch64.sve.sqincd.n64\0"
  "llvm.aarch64.sve.sqinch\0"
  "llvm.aarch64.sve.sqinch.n32\0"
  "llvm.aarch64.sve.sqinch.n64\0"
  "llvm.aarch64.sve.sqincp\0"
  "llvm.aarch64.sve.sqincp.n32\0"
  "llvm.aarch64.sve.sqincp.n64\0"
  "llvm.aarch64.sve.sqincw\0"
  "llvm.aarch64.sve.sqincw.n32\0"
  "llvm.aarch64.sve.sqincw.n64\0"
  "llvm.aarch64.sve.sqneg\0"
  "llvm.aarch64.sve.sqrdcmlah.lane.x\0"
  "llvm.aarch64.sve.sqrdcmlah.x\0"
  "llvm.aarch64.sve.sqrdmlah\0"
  "llvm.aarch64.sve.sqrdmlah.lane\0"
  "llvm.aarch64.sve.sqrdmlsh\0"
  "llvm.aarch64.sve.sqrdmlsh.lane\0"
  "llvm.aarch64.sve.sqrdmulh\0"
  "llvm.aarch64.sve.sqrdmulh.lane\0"
  "llvm.aarch64.sve.sqrshl\0"
  "llvm.aarch64.sve.sqrshl.u\0"
  "llvm.aarch64.sve.sqrshr.x2\0"
  "llvm.aarch64.sve.sqrshr.x4\0"
  "llvm.aarch64.sve.sqrshrn.x2\0"
  "llvm.aarch64.sve.sqrshrn.x4\0"
  "llvm.aarch64.sve.sqrshrnb\0"
  "llvm.aarch64.sve.sqrshrnt\0"
  "llvm.aarch64.sve.sqrshru.x2\0"
  "llvm.aarch64.sve.sqrshru.x4\0"
  "llvm.aarch64.sve.sqrshrun.x2\0"
  "llvm.aarch64.sve.sqrshrun.x4\0"
  "llvm.aarch64.sve.sqrshrunb\0"
  "llvm.aarch64.sve.sqrshrunt\0"
  "llvm.aarch64.sve.sqshl\0"
  "llvm.aarch64.sve.sqshl.u\0"
  "llvm.aarch64.sve.sqshlu\0"
  "llvm.aarch64.sve.sqshrnb\0"
  "llvm.aarch64.sve.sqshrnt\0"
  "llvm.aarch64.sve.sqshrunb\0"
  "llvm.aarch64.sve.sqshrunt\0"
  "llvm.aarch64.sve.sqsub\0"
  "llvm.aarch64.sve.sqsub.u\0"
  "llvm.aarch64.sve.sqsub.x\0"
  "llvm.aarch64.sve.sqsubr\0"
  "llvm.aarch64.sve.sqxtnb\0"
  "llvm.aarch64.sve.sqxtnt\0"
  "llvm.aarch64.sve.sqxtunb\0"
  "llvm.aarch64.sve.sqxtunt\0"
  "llvm.aarch64.sve.srhadd\0"
  "llvm.aarch64.sve.sri\0"
  "llvm.aarch64.sve.srshl\0"
  "llvm.aarch64.sve.srshl.single.x2\0"
  "llvm.aarch64.sve.srshl.single.x4\0"
  "llvm.aarch64.sve.srshl.u\0"
  "llvm.aarch64.sve.srshl.x2\0"
  "llvm.aarch64.sve.srshl.x4\0"
  "llvm.aarch64.sve.srshr\0"
  "llvm.aarch64.sve.srsra\0"
  "llvm.aarch64.sve.sshllb\0"
  "llvm.aarch64.sve.sshllt\0"
  "llvm.aarch64.sve.ssra\0"
  "llvm.aarch64.sve.ssublb\0"
  "llvm.aarch64.sve.ssublbt\0"
  "llvm.aarch64.sve.ssublt\0"
  "llvm.aarch64.sve.ssubltb\0"
  "llvm.aarch64.sve.ssubwb\0"
  "llvm.aarch64.sve.ssubwt\0"
  "llvm.aarch64.sve.st1\0"
  "llvm.aarch64.sve.st1.pn.x2\0"
  "llvm.aarch64.sve.st1.pn.x4\0"
  "llvm.aarch64.sve.st1.scatter\0"
  "llvm.aarch64.sve.st1.scatter.index\0"
  "llvm.aarch64.sve.st1.scatter.scalar.offset\0"
  "llvm.aarch64.sve.st1.scatter.sxtw\0"
  "llvm.aarch64.sve.st1.scatter.sxtw.index\0"
  "llvm.aarch64.sve.st1.scatter.uxtw\0"
  "llvm.aarch64.sve.st1.scatter.uxtw.index\0"
  "llvm.aarch64.sve.st1dq\0"
  "llvm.aarch64.sve.st1q.scatter.index\0"
  "llvm.aarch64.sve.st1q.scatter.scalar.offset\0"
  "llvm.aarch64.sve.st1q.scatter.vector.offset\0"
  "llvm.aarch64.sve.st1wq\0"
  "llvm.aarch64.sve.st2\0"
  "llvm.aarch64.sve.st2q\0"
  "llvm.aarch64.sve.st3\0"
  "llvm.aarch64.sve.st3q\0"
  "llvm.aarch64.sve.st4\0"
  "llvm.aarch64.sve.st4q\0"
  "llvm.aarch64.sve.stnt1\0"
  "llvm.aarch64.sve.stnt1.pn.x2\0"
  "llvm.aarch64.sve.stnt1.pn.x4\0"
  "llvm.aarch64.sve.stnt1.scatter\0"
  "llvm.aarch64.sve.stnt1.scatter.index\0"
  "llvm.aarch64.sve.stnt1.scatter.scalar.offset\0"
  "llvm.aarch64.sve.stnt1.scatter.uxtw\0"
  "llvm.aarch64.sve.sub\0"
  "llvm.aarch64.sve.sub.u\0"
  "llvm.aarch64.sve.subhnb\0"
  "llvm.aarch64.sve.subhnt\0"
  "llvm.aarch64.sve.subr\0"
  "llvm.aarch64.sve.sudot.lane\0"
  "llvm.aarch64.sve.sunpk.x2\0"
  "llvm.aarch64.sve.sunpk.x4\0"
  "llvm.aarch64.sve.sunpkhi\0"
  "llvm.aarch64.sve.sunpklo\0"
  "llvm.aarch64.sve.suqadd\0"
  "llvm.aarch64.sve.sxtb\0"
  "llvm.aarch64.sve.sxth\0"
  "llvm.aarch64.sve.sxtw\0"
  "llvm.aarch64.sve.tbl\0"
  "llvm.aarch64.sve.tbl2\0"
  "llvm.aarch64.sve.tblq\0"
  "llvm.aarch64.sve.tbx\0"
  "llvm.aarch64.sve.tbxq\0"
  "llvm.aarch64.sve.trn1\0"
  "llvm.aarch64.sve.trn1.b16\0"
  "llvm.aarch64.sve.trn1.b32\0"
  "llvm.aarch64.sve.trn1.b64\0"
  "llvm.aarch64.sve.trn1q\0"
  "llvm.aarch64.sve.trn2\0"
  "llvm.aarch64.sve.trn2.b16\0"
  "llvm.aarch64.sve.trn2.b32\0"
  "llvm.aarch64.sve.trn2.b64\0"
  "llvm.aarch64.sve.trn2q\0"
  "llvm.aarch64.sve.uaba\0"
  "llvm.aarch64.sve.uabalb\0"
  "llvm.aarch64.sve.uabalt\0"
  "llvm.aarch64.sve.uabd\0"
  "llvm.aarch64.sve.uabd.u\0"
  "llvm.aarch64.sve.uabdlb\0"
  "llvm.aarch64.sve.uabdlt\0"
  "llvm.aarch64.sve.uadalp\0"
  "llvm.aarch64.sve.uaddlb\0"
  "llvm.aarch64.sve.uaddlt\0"
  "llvm.aarch64.sve.uaddv\0"
  "llvm.aarch64.sve.uaddwb\0"
  "llvm.aarch64.sve.uaddwt\0"
  "llvm.aarch64.sve.uclamp\0"
  "llvm.aarch64.sve.uclamp.single.x2\0"
  "llvm.aarch64.sve.uclamp.single.x4\0"
  "llvm.aarch64.sve.ucvtf\0"
  "llvm.aarch64.sve.ucvtf.f16i32\0"
  "llvm.aarch64.sve.ucvtf.f16i64\0"
  "llvm.aarch64.sve.ucvtf.f32i64\0"
  "llvm.aarch64.sve.ucvtf.f64i32\0"
  "llvm.aarch64.sve.ucvtf.x2\0"
  "llvm.aarch64.sve.ucvtf.x4\0"
  "llvm.aarch64.sve.udiv\0"
  "llvm.aarch64.sve.udiv.u\0"
  "llvm.aarch64.sve.udivr\0"
  "llvm.aarch64.sve.udot\0"
  "llvm.aarch64.sve.udot.lane\0"
  "llvm.aarch64.sve.udot.lane.x2\0"
  "llvm.aarch64.sve.udot.x2\0"
  "llvm.aarch64.sve.uhadd\0"
  "llvm.aarch64.sve.uhsub\0"
  "llvm.aarch64.sve.uhsub.u\0"
  "llvm.aarch64.sve.uhsubr\0"
  "llvm.aarch64.sve.umax\0"
  "llvm.aarch64.sve.umax.single.x2\0"
  "llvm.aarch64.sve.umax.single.x4\0"
  "llvm.aarch64.sve.umax.u\0"
  "llvm.aarch64.sve.umax.x2\0"
  "llvm.aarch64.sve.umax.x4\0"
  "llvm.aarch64.sve.umaxp\0"
  "llvm.aarch64.sve.umaxqv\0"
  "llvm.aarch64.sve.umaxv\0"
  "llvm.aarch64.sve.umin\0"
  "llvm.aarch64.sve.umin.single.x2\0"
  "llvm.aarch64.sve.umin.single.x4\0"
  "llvm.aarch64.sve.umin.u\0"
  "llvm.aarch64.sve.umin.x2\0"
  "llvm.aarch64.sve.umin.x4\0"
  "llvm.aarch64.sve.uminp\0"
  "llvm.aarch64.sve.uminqv\0"
  "llvm.aarch64.sve.uminv\0"
  "llvm.aarch64.sve.umlalb\0"
  "llvm.aarch64.sve.umlalb.lane\0"
  "llvm.aarch64.sve.umlalt\0"
  "llvm.aarch64.sve.umlalt.lane\0"
  "llvm.aarch64.sve.umlslb\0"
  "llvm.aarch64.sve.umlslb.lane\0"
  "llvm.aarch64.sve.umlslt\0"
  "llvm.aarch64.sve.umlslt.lane\0"
  "llvm.aarch64.sve.ummla\0"
  "llvm.aarch64.sve.umulh\0"
  "llvm.aarch64.sve.umulh.u\0"
  "llvm.aarch64.sve.umullb\0"
  "llvm.aarch64.sve.umullb.lane\0"
  "llvm.aarch64.sve.umullt\0"
  "llvm.aarch64.sve.umullt.lane\0"
  "llvm.aarch64.sve.uqadd\0"
  "llvm.aarch64.sve.uqadd.x\0"
  "llvm.aarch64.sve.uqcvt.x2\0"
  "llvm.aarch64.sve.uqcvt.x4\0"
  "llvm.aarch64.sve.uqcvtn.x2\0"
  "llvm.aarch64.sve.uqcvtn.x4\0"
  "llvm.aarch64.sve.uqdecb.n32\0"
  "llvm.aarch64.sve.uqdecb.n64\0"
  "llvm.aarch64.sve.uqdecd\0"
  "llvm.aarch64.sve.uqdecd.n32\0"
  "llvm.aarch64.sve.uqdecd.n64\0"
  "llvm.aarch64.sve.uqdech\0"
  "llvm.aarch64.sve.uqdech.n32\0"
  "llvm.aarch64.sve.uqdech.n64\0"
  "llvm.aarch64.sve.uqdecp\0"
  "llvm.aarch64.sve.uqdecp.n32\0"
  "llvm.aarch64.sve.uqdecp.n64\0"
  "llvm.aarch64.sve.uqdecw\0"
  "llvm.aarch64.sve.uqdecw.n32\0"
  "llvm.aarch64.sve.uqdecw.n64\0"
  "llvm.aarch64.sve.uqincb.n32\0"
  "llvm.aarch64.sve.uqincb.n64\0"
  "llvm.aarch64.sve.uqincd\0"
  "llvm.aarch64.sve.uqincd.n32\0"
  "llvm.aarch64.sve.uqincd.n64\0"
  "llvm.aarch64.sve.uqinch\0"
  "llvm.aarch64.sve.uqinch.n32\0"
  "llvm.aarch64.sve.uqinch.n64\0"
  "llvm.aarch64.sve.uqincp\0"
  "llvm.aarch64.sve.uqincp.n32\0"
  "llvm.aarch64.sve.uqincp.n64\0"
  "llvm.aarch64.sve.uqincw\0"
  "llvm.aarch64.sve.uqincw.n32\0"
  "llvm.aarch64.sve.uqincw.n64\0"
  "llvm.aarch64.sve.uqrshl\0"
  "llvm.aarch64.sve.uqrshl.u\0"
  "llvm.aarch64.sve.uqrshr.x2\0"
  "llvm.aarch64.sve.uqrshr.x4\0"
  "llvm.aarch64.sve.uqrshrn.x2\0"
  "llvm.aarch64.sve.uqrshrn.x4\0"
  "llvm.aarch64.sve.uqrshrnb\0"
  "llvm.aarch64.sve.uqrshrnt\0"
  "llvm.aarch64.sve.uqshl\0"
  "llvm.aarch64.sve.uqshl.u\0"
  "llvm.aarch64.sve.uqshrnb\0"
  "llvm.aarch64.sve.uqshrnt\0"
  "llvm.aarch64.sve.uqsub\0"
  "llvm.aarch64.sve.uqsub.u\0"
  "llvm.aarch64.sve.uqsub.x\0"
  "llvm.aarch64.sve.uqsubr\0"
  "llvm.aarch64.sve.uqxtnb\0"
  "llvm.aarch64.sve.uqxtnt\0"
  "llvm.aarch64.sve.urecpe\0"
  "llvm.aarch64.sve.urhadd\0"
  "llvm.aarch64.sve.urshl\0"
  "llvm.aarch64.sve.urshl.single.x2\0"
  "llvm.aarch64.sve.urshl.single.x4\0"
  "llvm.aarch64.sve.urshl.u\0"
  "llvm.aarch64.sve.urshl.x2\0"
  "llvm.aarch64.sve.urshl.x4\0"
  "llvm.aarch64.sve.urshr\0"
  "llvm.aarch64.sve.ursqrte\0"
  "llvm.aarch64.sve.ursra\0"
  "llvm.aarch64.sve.usdot\0"
  "llvm.aarch64.sve.usdot.lane\0"
  "llvm.aarch64.sve.ushllb\0"
  "llvm.aarch64.sve.ushllt\0"
  "llvm.aarch64.sve.usmmla\0"
  "llvm.aarch64.sve.usqadd\0"
  "llvm.aarch64.sve.usra\0"
  "llvm.aarch64.sve.usublb\0"
  "llvm.aarch64.sve.usublt\0"
  "llvm.aarch64.sve.usubwb\0"
  "llvm.aarch64.sve.usubwt\0"
  "llvm.aarch64.sve.uunpk.x2\0"
  "llvm.aarch64.sve.uunpk.x4\0"
  "llvm.aarch64.sve.uunpkhi\0"
  "llvm.aarch64.sve.uunpklo\0"
  "llvm.aarch64.sve.uxtb\0"
  "llvm.aarch64.sve.uxth\0"
  "llvm.aarch64.sve.uxtw\0"
  "llvm.aarch64.sve.uzp.x2\0"
  "llvm.aarch64.sve.uzp.x4\0"
  "llvm.aarch64.sve.uzp1\0"
  "llvm.aarch64.sve.uzp1.b16\0"
  "llvm.aarch64.sve.uzp1.b32\0"
  "llvm.aarch64.sve.uzp1.b64\0"
  "llvm.aarch64.sve.uzp1q\0"
  "llvm.aarch64.sve.uzp2\0"
  "llvm.aarch64.sve.uzp2.b16\0"
  "llvm.aarch64.sve.uzp2.b32\0"
  "llvm.aarch64.sve.uzp2.b64\0"
  "llvm.aarch64.sve.uzp2q\0"
  "llvm.aarch64.sve.uzpq.x2\0"
  "llvm.aarch64.sve.uzpq.x4\0"
  "llvm.aarch64.sve.uzpq1\0"
  "llvm.aarch64.sve.uzpq2\0"
  "llvm.aarch64.sve.whilege\0"
  "llvm.aarch64.sve.whilege.c16\0"
  "llvm.aarch64.sve.whilege.c32\0"
  "llvm.aarch64.sve.whilege.c64\0"
  "llvm.aarch64.sve.whilege.c8\0"
  "llvm.aarch64.sve.whilege.x2\0"
  "llvm.aarch64.sve.whilegt\0"
  "llvm.aarch64.sve.whilegt.c16\0"
  "llvm.aarch64.sve.whilegt.c32\0"
  "llvm.aarch64.sve.whilegt.c64\0"
  "llvm.aarch64.sve.whilegt.c8\0"
  "llvm.aarch64.sve.whilegt.x2\0"
  "llvm.aarch64.sve.whilehi\0"
  "llvm.aarch64.sve.whilehi.c16\0"
  "llvm.aarch64.sve.whilehi.c32\0"
  "llvm.aarch64.sve.whilehi.c64\0"
  "llvm.aarch64.sve.whilehi.c8\0"
  "llvm.aarch64.sve.whilehi.x2\0"
  "llvm.aarch64.sve.whilehs\0"
  "llvm.aarch64.sve.whilehs.c16\0"
  "llvm.aarch64.sve.whilehs.c32\0"
  "llvm.aarch64.sve.whilehs.c64\0"
  "llvm.aarch64.sve.whilehs.c8\0"
  "llvm.aarch64.sve.whilehs.x2\0"
  "llvm.aarch64.sve.whilele\0"
  "llvm.aarch64.sve.whilele.c16\0"
  "llvm.aarch64.sve.whilele.c32\0"
  "llvm.aarch64.sve.whilele.c64\0"
  "llvm.aarch64.sve.whilele.c8\0"
  "llvm.aarch64.sve.whilele.x2\0"
  "llvm.aarch64.sve.whilelo\0"
  "llvm.aarch64.sve.whilelo.c16\0"
  "llvm.aarch64.sve.whilelo.c32\0"
  "llvm.aarch64.sve.whilelo.c64\0"
  "llvm.aarch64.sve.whilelo.c8\0"
  "llvm.aarch64.sve.whilelo.x2\0"
  "llvm.aarch64.sve.whilels\0"
  "llvm.aarch64.sve.whilels.c16\0"
  "llvm.aarch64.sve.whilels.c32\0"
  "llvm.aarch64.sve.whilels.c64\0"
  "llvm.aarch64.sve.whilels.c8\0"
  "llvm.aarch64.sve.whilels.x2\0"
  "llvm.aarch64.sve.whilelt\0"
  "llvm.aarch64.sve.whilelt.c16\0"
  "llvm.aarch64.sve.whilelt.c32\0"
  "llvm.aarch64.sve.whilelt.c64\0"
  "llvm.aarch64.sve.whilelt.c8\0"
  "llvm.aarch64.sve.whilelt.x2\0"
  "llvm.aarch64.sve.whilerw.b\0"
  "llvm.aarch64.sve.whilerw.d\0"
  "llvm.aarch64.sve.whilerw.h\0"
  "llvm.aarch64.sve.whilerw.s\0"
  "llvm.aarch64.sve.whilewr.b\0"
  "llvm.aarch64.sve.whilewr.d\0"
  "llvm.aarch64.sve.whilewr.h\0"
  "llvm.aarch64.sve.whilewr.s\0"
  "llvm.aarch64.sve.wrffr\0"
  "llvm.aarch64.sve.xar\0"
  "llvm.aarch64.sve.zip.x2\0"
  "llvm.aarch64.sve.zip.x4\0"
  "llvm.aarch64.sve.zip1\0"
  "llvm.aarch64.sve.zip1.b16\0"
  "llvm.aarch64.sve.zip1.b32\0"
  "llvm.aarch64.sve.zip1.b64\0"
  "llvm.aarch64.sve.zip1q\0"
  "llvm.aarch64.sve.zip2\0"
  "llvm.aarch64.sve.zip2.b16\0"
  "llvm.aarch64.sve.zip2.b32\0"
  "llvm.aarch64.sve.zip2.b64\0"
  "llvm.aarch64.sve.zip2q\0"
  "llvm.aarch64.sve.zipq.x2\0"
  "llvm.aarch64.sve.zipq.x4\0"
  "llvm.aarch64.sve.zipq1\0"
  "llvm.aarch64.sve.zipq2\0"
  "llvm.aarch64.tagp\0"
  "llvm.aarch64.udiv\0"
  "llvm.amdgcn.add.max.i32\0"
  "llvm.amdgcn.add.max.u32\0"
  "llvm.amdgcn.add.min.i32\0"
  "llvm.amdgcn.add.min.u32\0"
  "llvm.amdgcn.addrspacecast.nonnull\0"
  "llvm.amdgcn.alignbyte\0"
  "llvm.amdgcn.ashr.pk.i8.i32\0"
  "llvm.amdgcn.ashr.pk.u8.i32\0"
  "llvm.amdgcn.ballot\0"
  "llvm.amdgcn.bitop3\0"
  "llvm.amdgcn.buffer.wbinvl1\0"
  "llvm.amdgcn.buffer.wbinvl1.sc\0"
  "llvm.amdgcn.buffer.wbinvl1.vol\0"
  "llvm.amdgcn.call.whole.wave\0"
  "llvm.amdgcn.class\0"
  "llvm.amdgcn.cluster.id.x\0"
  "llvm.amdgcn.cluster.id.y\0"
  "llvm.amdgcn.cluster.id.z\0"
  "llvm.amdgcn.cluster.load.async.to.lds.b128\0"
  "llvm.amdgcn.cluster.load.async.to.lds.b32\0"
  "llvm.amdgcn.cluster.load.async.to.lds.b64\0"
  "llvm.amdgcn.cluster.load.async.to.lds.b8\0"
  "llvm.amdgcn.cluster.load.b128\0"
  "llvm.amdgcn.cluster.load.b32\0"
  "llvm.amdgcn.cluster.load.b64\0"
  "llvm.amdgcn.cluster.workgroup.flat.id\0"
  "llvm.amdgcn.cluster.workgroup.id.x\0"
  "llvm.amdgcn.cluster.workgroup.id.y\0"
  "llvm.amdgcn.cluster.workgroup.id.z\0"
  "llvm.amdgcn.cluster.workgroup.max.flat.id\0"
  "llvm.amdgcn.cluster.workgroup.max.id.x\0"
  "llvm.amdgcn.cluster.workgroup.max.id.y\0"
  "llvm.amdgcn.cluster.workgroup.max.id.z\0"
  "llvm.amdgcn.cooperative.atomic.load.16x8B\0"
  "llvm.amdgcn.cooperative.atomic.load.32x4B\0"
  "llvm.amdgcn.cooperative.atomic.load.8x16B\0"
  "llvm.amdgcn.cooperative.atomic.store.16x8B\0"
  "llvm.amdgcn.cooperative.atomic.store.32x4B\0"
  "llvm.amdgcn.cooperative.atomic.store.8x16B\0"
  "llvm.amdgcn.cos\0"
  "llvm.amdgcn.cs.chain\0"
  "llvm.amdgcn.cubeid\0"
  "llvm.amdgcn.cubema\0"
  "llvm.amdgcn.cubesc\0"
  "llvm.amdgcn.cubetc\0"
  "llvm.amdgcn.cvt.f16.bf8\0"
  "llvm.amdgcn.cvt.f16.fp8\0"
  "llvm.amdgcn.cvt.f32.bf8\0"
  "llvm.amdgcn.cvt.f32.fp8\0"
  "llvm.amdgcn.cvt.f32.fp8.e5m3\0"
  "llvm.amdgcn.cvt.off.f32.i4\0"
  "llvm.amdgcn.cvt.pk.bf8.f16\0"
  "llvm.amdgcn.cvt.pk.bf8.f32\0"
  "llvm.amdgcn.cvt.pk.f16.bf8\0"
  "llvm.amdgcn.cvt.pk.f16.fp8\0"
  "llvm.amdgcn.cvt.pk.f32.bf8\0"
  "llvm.amdgcn.cvt.pk.f32.fp8\0"
  "llvm.amdgcn.cvt.pk.fp8.f16\0"
  "llvm.amdgcn.cvt.pk.fp8.f32\0"
  "llvm.amdgcn.cvt.pk.fp8.f32.e5m3\0"
  "llvm.amdgcn.cvt.pk.i16\0"
  "llvm.amdgcn.cvt.pk.u16\0"
  "llvm.amdgcn.cvt.pk.u8.f32\0"
  "llvm.amdgcn.cvt.pknorm.i16\0"
  "llvm.amdgcn.cvt.pknorm.u16\0"
  "llvm.amdgcn.cvt.pkrtz\0"
  "llvm.amdgcn.cvt.scale.pk16.bf16.bf6\0"
  "llvm.amdgcn.cvt.scale.pk16.bf16.fp6\0"
  "llvm.amdgcn.cvt.scale.pk16.f16.bf6\0"
  "llvm.amdgcn.cvt.scale.pk16.f16.fp6\0"
  "llvm.amdgcn.cvt.scale.pk16.f32.bf6\0"
  "llvm.amdgcn.cvt.scale.pk16.f32.fp6\0"
  "llvm.amdgcn.cvt.scale.pk8.bf16.bf8\0"
  "llvm.amdgcn.cvt.scale.pk8.bf16.fp4\0"
  "llvm.amdgcn.cvt.scale.pk8.bf16.fp8\0"
  "llvm.amdgcn.cvt.scale.pk8.f16.bf8\0"
  "llvm.amdgcn.cvt.scale.pk8.f16.fp4\0"
  "llvm.amdgcn.cvt.scale.pk8.f16.fp8\0"
  "llvm.amdgcn.cvt.scale.pk8.f32.bf8\0"
  "llvm.amdgcn.cvt.scale.pk8.f32.fp4\0"
  "llvm.amdgcn.cvt.scale.pk8.f32.fp8\0"
  "llvm.amdgcn.cvt.scalef32.2xpk16.bf6.f32\0"
  "llvm.amdgcn.cvt.scalef32.2xpk16.fp6.f32\0"
  "llvm.amdgcn.cvt.scalef32.f16.bf8\0"
  "llvm.amdgcn.cvt.scalef32.f16.fp8\0"
  "llvm.amdgcn.cvt.scalef32.f32.bf8\0"
  "llvm.amdgcn.cvt.scalef32.f32.fp8\0"
  "llvm.amdgcn.cvt.scalef32.pk.bf16.bf8\0"
  "llvm.amdgcn.cvt.scalef32.pk.bf16.fp4\0"
  "llvm.amdgcn.cvt.scalef32.pk.bf16.fp8\0"
  "llvm.amdgcn.cvt.scalef32.pk.bf8.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk.bf8.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk.bf8.f32\0"
  "llvm.amdgcn.cvt.scalef32.pk.f16.bf8\0"
  "llvm.amdgcn.cvt.scalef32.pk.f16.fp4\0"
  "llvm.amdgcn.cvt.scalef32.pk.f16.fp8\0"
  "llvm.amdgcn.cvt.scalef32.pk.f32.bf8\0"
  "llvm.amdgcn.cvt.scalef32.pk.f32.fp4\0"
  "llvm.amdgcn.cvt.scalef32.pk.f32.fp8\0"
  "llvm.amdgcn.cvt.scalef32.pk.fp4.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk.fp4.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk.fp4.f32\0"
  "llvm.amdgcn.cvt.scalef32.pk.fp8.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk.fp8.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk.fp8.f32\0"
  "llvm.amdgcn.cvt.scalef32.pk16.bf6.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk16.bf6.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk16.bf6.f32\0"
  "llvm.amdgcn.cvt.scalef32.pk16.fp6.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk16.fp6.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk16.fp6.f32\0"
  "llvm.amdgcn.cvt.scalef32.pk32.bf16.bf6\0"
  "llvm.amdgcn.cvt.scalef32.pk32.bf16.fp6\0"
  "llvm.amdgcn.cvt.scalef32.pk32.bf6.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk32.bf6.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk32.f16.bf6\0"
  "llvm.amdgcn.cvt.scalef32.pk32.f16.fp6\0"
  "llvm.amdgcn.cvt.scalef32.pk32.f32.bf6\0"
  "llvm.amdgcn.cvt.scalef32.pk32.f32.fp6\0"
  "llvm.amdgcn.cvt.scalef32.pk32.fp6.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk32.fp6.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk8.bf8.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk8.bf8.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk8.bf8.f32\0"
  "llvm.amdgcn.cvt.scalef32.pk8.fp4.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk8.fp4.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk8.fp4.f32\0"
  "llvm.amdgcn.cvt.scalef32.pk8.fp8.bf16\0"
  "llvm.amdgcn.cvt.scalef32.pk8.fp8.f16\0"
  "llvm.amdgcn.cvt.scalef32.pk8.fp8.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.bf8.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.bf8.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.bf8.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.fp8.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.fp8.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.fp8.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk.fp4.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk.fp4.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk.fp4.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.f32\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.bf16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.f16\0"
  "llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.f32\0"
  "llvm.amdgcn.cvt.sr.bf16.f32\0"
  "llvm.amdgcn.cvt.sr.bf8.f16\0"
  "llvm.amdgcn.cvt.sr.bf8.f32\0"
  "llvm.amdgcn.cvt.sr.f16.f32\0"
  "llvm.amdgcn.cvt.sr.fp8.f16\0"
  "llvm.amdgcn.cvt.sr.fp8.f32\0"
  "llvm.amdgcn.cvt.sr.fp8.f32.e5m3\0"
  "llvm.amdgcn.cvt.sr.pk.bf16.f32\0"
  "llvm.amdgcn.cvt.sr.pk.f16.f32\0"
  "llvm.amdgcn.dead\0"
  "llvm.amdgcn.dispatch.id\0"
  "llvm.amdgcn.dispatch.ptr\0"
  "llvm.amdgcn.div.fixup\0"
  "llvm.amdgcn.div.fmas\0"
  "llvm.amdgcn.div.scale\0"
  "llvm.amdgcn.dot4.f32.bf8.bf8\0"
  "llvm.amdgcn.dot4.f32.bf8.fp8\0"
  "llvm.amdgcn.dot4.f32.fp8.bf8\0"
  "llvm.amdgcn.dot4.f32.fp8.fp8\0"
  "llvm.amdgcn.ds.add.gs.reg.rtn\0"
  "llvm.amdgcn.ds.append\0"
  "llvm.amdgcn.ds.atomic.async.barrier.arrive.b64\0"
  "llvm.amdgcn.ds.atomic.barrier.arrive.rtn.b64\0"
  "llvm.amdgcn.ds.bpermute\0"
  "llvm.amdgcn.ds.bpermute.fi.b32\0"
  "llvm.amdgcn.ds.bvh.stack.push4.pop1.rtn\0"
  "llvm.amdgcn.ds.bvh.stack.push8.pop1.rtn\0"
  "llvm.amdgcn.ds.bvh.stack.push8.pop2.rtn\0"
  "llvm.amdgcn.ds.bvh.stack.rtn\0"
  "llvm.amdgcn.ds.consume\0"
  "llvm.amdgcn.ds.gws.barrier\0"
  "llvm.amdgcn.ds.gws.init\0"
  "llvm.amdgcn.ds.gws.sema.br\0"
  "llvm.amdgcn.ds.gws.sema.p\0"
  "llvm.amdgcn.ds.gws.sema.release.all\0"
  "llvm.amdgcn.ds.gws.sema.v\0"
  "llvm.amdgcn.ds.load.tr16.b128\0"
  "llvm.amdgcn.ds.load.tr4.b64\0"
  "llvm.amdgcn.ds.load.tr6.b96\0"
  "llvm.amdgcn.ds.load.tr8.b64\0"
  "llvm.amdgcn.ds.ordered.add\0"
  "llvm.amdgcn.ds.ordered.swap\0"
  "llvm.amdgcn.ds.permute\0"
  "llvm.amdgcn.ds.read.tr16.b64\0"
  "llvm.amdgcn.ds.read.tr4.b64\0"
  "llvm.amdgcn.ds.read.tr6.b96\0"
  "llvm.amdgcn.ds.read.tr8.b64\0"
  "llvm.amdgcn.ds.sub.gs.reg.rtn\0"
  "llvm.amdgcn.ds.swizzle\0"
  "llvm.amdgcn.else\0"
  "llvm.amdgcn.end.cf\0"
  "llvm.amdgcn.endpgm\0"
  "llvm.amdgcn.exp\0"
  "llvm.amdgcn.exp.compr\0"
  "llvm.amdgcn.exp.row\0"
  "llvm.amdgcn.exp2\0"
  "llvm.amdgcn.fcmp\0"
  "llvm.amdgcn.fdiv.fast\0"
  "llvm.amdgcn.fdot2\0"
  "llvm.amdgcn.fdot2.bf16.bf16\0"
  "llvm.amdgcn.fdot2.f16.f16\0"
  "llvm.amdgcn.fdot2.f32.bf16\0"
  "llvm.amdgcn.fdot2c.f32.bf16\0"
  "llvm.amdgcn.flat.atomic.fmax.num\0"
  "llvm.amdgcn.flat.atomic.fmin.num\0"
  "llvm.amdgcn.flat.load.monitor.b128\0"
  "llvm.amdgcn.flat.load.monitor.b32\0"
  "llvm.amdgcn.flat.load.monitor.b64\0"
  "llvm.amdgcn.flat.prefetch\0"
  "llvm.amdgcn.fma.legacy\0"
  "llvm.amdgcn.fmad.ftz\0"
  "llvm.amdgcn.fmed3\0"
  "llvm.amdgcn.fmul.legacy\0"
  "llvm.amdgcn.fract\0"
  "llvm.amdgcn.frexp.exp\0"
  "llvm.amdgcn.frexp.mant\0"
  "llvm.amdgcn.global.atomic.fmax.num\0"
  "llvm.amdgcn.global.atomic.fmin.num\0"
  "llvm.amdgcn.global.atomic.ordered.add.b64\0"
  "llvm.amdgcn.global.load.async.to.lds.b128\0"
  "llvm.amdgcn.global.load.async.to.lds.b32\0"
  "llvm.amdgcn.global.load.async.to.lds.b64\0"
  "llvm.amdgcn.global.load.async.to.lds.b8\0"
  "llvm.amdgcn.global.load.lds\0"
  "llvm.amdgcn.global.load.monitor.b128\0"
  "llvm.amdgcn.global.load.monitor.b32\0"
  "llvm.amdgcn.global.load.monitor.b64\0"
  "llvm.amdgcn.global.load.tr.b128\0"
  "llvm.amdgcn.global.load.tr.b64\0"
  "llvm.amdgcn.global.load.tr4.b64\0"
  "llvm.amdgcn.global.load.tr6.b96\0"
  "llvm.amdgcn.global.prefetch\0"
  "llvm.amdgcn.global.store.async.from.lds.b128\0"
  "llvm.amdgcn.global.store.async.from.lds.b32\0"
  "llvm.amdgcn.global.store.async.from.lds.b64\0"
  "llvm.amdgcn.global.store.async.from.lds.b8\0"
  "llvm.amdgcn.groupstaticsize\0"
  "llvm.amdgcn.icmp\0"
  "llvm.amdgcn.if\0"
  "llvm.amdgcn.if.break\0"
  "llvm.amdgcn.iglp.opt\0"
  "llvm.amdgcn.image.atomic.add.1d\0"
  "llvm.amdgcn.image.atomic.add.1darray\0"
  "llvm.amdgcn.image.atomic.add.2d\0"
  "llvm.amdgcn.image.atomic.add.2darray\0"
  "llvm.amdgcn.image.atomic.add.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.add.2dmsaa\0"
  "llvm.amdgcn.image.atomic.add.3d\0"
  "llvm.amdgcn.image.atomic.add.cube\0"
  "llvm.amdgcn.image.atomic.add.flt.1d\0"
  "llvm.amdgcn.image.atomic.add.flt.1darray\0"
  "llvm.amdgcn.image.atomic.add.flt.2d\0"
  "llvm.amdgcn.image.atomic.add.flt.2darray\0"
  "llvm.amdgcn.image.atomic.add.flt.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.add.flt.2dmsaa\0"
  "llvm.amdgcn.image.atomic.add.flt.3d\0"
  "llvm.amdgcn.image.atomic.add.flt.cube\0"
  "llvm.amdgcn.image.atomic.and.1d\0"
  "llvm.amdgcn.image.atomic.and.1darray\0"
  "llvm.amdgcn.image.atomic.and.2d\0"
  "llvm.amdgcn.image.atomic.and.2darray\0"
  "llvm.amdgcn.image.atomic.and.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.and.2dmsaa\0"
  "llvm.amdgcn.image.atomic.and.3d\0"
  "llvm.amdgcn.image.atomic.and.cube\0"
  "llvm.amdgcn.image.atomic.cmpswap.1d\0"
  "llvm.amdgcn.image.atomic.cmpswap.1darray\0"
  "llvm.amdgcn.image.atomic.cmpswap.2d\0"
  "llvm.amdgcn.image.atomic.cmpswap.2darray\0"
  "llvm.amdgcn.image.atomic.cmpswap.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.cmpswap.2dmsaa\0"
  "llvm.amdgcn.image.atomic.cmpswap.3d\0"
  "llvm.amdgcn.image.atomic.cmpswap.cube\0"
  "llvm.amdgcn.image.atomic.dec.1d\0"
  "llvm.amdgcn.image.atomic.dec.1darray\0"
  "llvm.amdgcn.image.atomic.dec.2d\0"
  "llvm.amdgcn.image.atomic.dec.2darray\0"
  "llvm.amdgcn.image.atomic.dec.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.dec.2dmsaa\0"
  "llvm.amdgcn.image.atomic.dec.3d\0"
  "llvm.amdgcn.image.atomic.dec.cube\0"
  "llvm.amdgcn.image.atomic.fmax.1d\0"
  "llvm.amdgcn.image.atomic.fmax.1darray\0"
  "llvm.amdgcn.image.atomic.fmax.2d\0"
  "llvm.amdgcn.image.atomic.fmax.2darray\0"
  "llvm.amdgcn.image.atomic.fmax.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.fmax.2dmsaa\0"
  "llvm.amdgcn.image.atomic.fmax.3d\0"
  "llvm.amdgcn.image.atomic.fmax.cube\0"
  "llvm.amdgcn.image.atomic.fmin.1d\0"
  "llvm.amdgcn.image.atomic.fmin.1darray\0"
  "llvm.amdgcn.image.atomic.fmin.2d\0"
  "llvm.amdgcn.image.atomic.fmin.2darray\0"
  "llvm.amdgcn.image.atomic.fmin.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.fmin.2dmsaa\0"
  "llvm.amdgcn.image.atomic.fmin.3d\0"
  "llvm.amdgcn.image.atomic.fmin.cube\0"
  "llvm.amdgcn.image.atomic.inc.1d\0"
  "llvm.amdgcn.image.atomic.inc.1darray\0"
  "llvm.amdgcn.image.atomic.inc.2d\0"
  "llvm.amdgcn.image.atomic.inc.2darray\0"
  "llvm.amdgcn.image.atomic.inc.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.inc.2dmsaa\0"
  "llvm.amdgcn.image.atomic.inc.3d\0"
  "llvm.amdgcn.image.atomic.inc.cube\0"
  "llvm.amdgcn.image.atomic.max.flt.1d\0"
  "llvm.amdgcn.image.atomic.max.flt.1darray\0"
  "llvm.amdgcn.image.atomic.max.flt.2d\0"
  "llvm.amdgcn.image.atomic.max.flt.2darray\0"
  "llvm.amdgcn.image.atomic.max.flt.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.max.flt.2dmsaa\0"
  "llvm.amdgcn.image.atomic.max.flt.3d\0"
  "llvm.amdgcn.image.atomic.max.flt.cube\0"
  "llvm.amdgcn.image.atomic.min.flt.1d\0"
  "llvm.amdgcn.image.atomic.min.flt.1darray\0"
  "llvm.amdgcn.image.atomic.min.flt.2d\0"
  "llvm.amdgcn.image.atomic.min.flt.2darray\0"
  "llvm.amdgcn.image.atomic.min.flt.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.min.flt.2dmsaa\0"
  "llvm.amdgcn.image.atomic.min.flt.3d\0"
  "llvm.amdgcn.image.atomic.min.flt.cube\0"
  "llvm.amdgcn.image.atomic.or.1d\0"
  "llvm.amdgcn.image.atomic.or.1darray\0"
  "llvm.amdgcn.image.atomic.or.2d\0"
  "llvm.amdgcn.image.atomic.or.2darray\0"
  "llvm.amdgcn.image.atomic.or.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.or.2dmsaa\0"
  "llvm.amdgcn.image.atomic.or.3d\0"
  "llvm.amdgcn.image.atomic.or.cube\0"
  "llvm.amdgcn.image.atomic.pk.add.bf16.1d\0"
  "llvm.amdgcn.image.atomic.pk.add.bf16.1darray\0"
  "llvm.amdgcn.image.atomic.pk.add.bf16.2d\0"
  "llvm.amdgcn.image.atomic.pk.add.bf16.2darray\0"
  "llvm.amdgcn.image.atomic.pk.add.bf16.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.pk.add.bf16.2dmsaa\0"
  "llvm.amdgcn.image.atomic.pk.add.bf16.3d\0"
  "llvm.amdgcn.image.atomic.pk.add.bf16.cube\0"
  "llvm.amdgcn.image.atomic.pk.add.f16.1d\0"
  "llvm.amdgcn.image.atomic.pk.add.f16.1darray\0"
  "llvm.amdgcn.image.atomic.pk.add.f16.2d\0"
  "llvm.amdgcn.image.atomic.pk.add.f16.2darray\0"
  "llvm.amdgcn.image.atomic.pk.add.f16.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.pk.add.f16.2dmsaa\0"
  "llvm.amdgcn.image.atomic.pk.add.f16.3d\0"
  "llvm.amdgcn.image.atomic.pk.add.f16.cube\0"
  "llvm.amdgcn.image.atomic.smax.1d\0"
  "llvm.amdgcn.image.atomic.smax.1darray\0"
  "llvm.amdgcn.image.atomic.smax.2d\0"
  "llvm.amdgcn.image.atomic.smax.2darray\0"
  "llvm.amdgcn.image.atomic.smax.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.smax.2dmsaa\0"
  "llvm.amdgcn.image.atomic.smax.3d\0"
  "llvm.amdgcn.image.atomic.smax.cube\0"
  "llvm.amdgcn.image.atomic.smin.1d\0"
  "llvm.amdgcn.image.atomic.smin.1darray\0"
  "llvm.amdgcn.image.atomic.smin.2d\0"
  "llvm.amdgcn.image.atomic.smin.2darray\0"
  "llvm.amdgcn.image.atomic.smin.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.smin.2dmsaa\0"
  "llvm.amdgcn.image.atomic.smin.3d\0"
  "llvm.amdgcn.image.atomic.smin.cube\0"
  "llvm.amdgcn.image.atomic.sub.1d\0"
  "llvm.amdgcn.image.atomic.sub.1darray\0"
  "llvm.amdgcn.image.atomic.sub.2d\0"
  "llvm.amdgcn.image.atomic.sub.2darray\0"
  "llvm.amdgcn.image.atomic.sub.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.sub.2dmsaa\0"
  "llvm.amdgcn.image.atomic.sub.3d\0"
  "llvm.amdgcn.image.atomic.sub.cube\0"
  "llvm.amdgcn.image.atomic.swap.1d\0"
  "llvm.amdgcn.image.atomic.swap.1darray\0"
  "llvm.amdgcn.image.atomic.swap.2d\0"
  "llvm.amdgcn.image.atomic.swap.2darray\0"
  "llvm.amdgcn.image.atomic.swap.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.swap.2dmsaa\0"
  "llvm.amdgcn.image.atomic.swap.3d\0"
  "llvm.amdgcn.image.atomic.swap.cube\0"
  "llvm.amdgcn.image.atomic.umax.1d\0"
  "llvm.amdgcn.image.atomic.umax.1darray\0"
  "llvm.amdgcn.image.atomic.umax.2d\0"
  "llvm.amdgcn.image.atomic.umax.2darray\0"
  "llvm.amdgcn.image.atomic.umax.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.umax.2dmsaa\0"
  "llvm.amdgcn.image.atomic.umax.3d\0"
  "llvm.amdgcn.image.atomic.umax.cube\0"
  "llvm.amdgcn.image.atomic.umin.1d\0"
  "llvm.amdgcn.image.atomic.umin.1darray\0"
  "llvm.amdgcn.image.atomic.umin.2d\0"
  "llvm.amdgcn.image.atomic.umin.2darray\0"
  "llvm.amdgcn.image.atomic.umin.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.umin.2dmsaa\0"
  "llvm.amdgcn.image.atomic.umin.3d\0"
  "llvm.amdgcn.image.atomic.umin.cube\0"
  "llvm.amdgcn.image.atomic.xor.1d\0"
  "llvm.amdgcn.image.atomic.xor.1darray\0"
  "llvm.amdgcn.image.atomic.xor.2d\0"
  "llvm.amdgcn.image.atomic.xor.2darray\0"
  "llvm.amdgcn.image.atomic.xor.2darraymsaa\0"
  "llvm.amdgcn.image.atomic.xor.2dmsaa\0"
  "llvm.amdgcn.image.atomic.xor.3d\0"
  "llvm.amdgcn.image.atomic.xor.cube\0"
  "llvm.amdgcn.image.bvh.dual.intersect.ray\0"
  "llvm.amdgcn.image.bvh.intersect.ray\0"
  "llvm.amdgcn.image.bvh8.intersect.ray\0"
  "llvm.amdgcn.image.gather4.2d\0"
  "llvm.amdgcn.image.gather4.2darray\0"
  "llvm.amdgcn.image.gather4.b.2d\0"
  "llvm.amdgcn.image.gather4.b.2darray\0"
  "llvm.amdgcn.image.gather4.b.cl.2d\0"
  "llvm.amdgcn.image.gather4.b.cl.2darray\0"
  "llvm.amdgcn.image.gather4.b.cl.cube\0"
  "llvm.amdgcn.image.gather4.b.cl.o.2d\0"
  "llvm.amdgcn.image.gather4.b.cl.o.2darray\0"
  "llvm.amdgcn.image.gather4.b.cl.o.cube\0"
  "llvm.amdgcn.image.gather4.b.cube\0"
  "llvm.amdgcn.image.gather4.b.o.2d\0"
  "llvm.amdgcn.image.gather4.b.o.2darray\0"
  "llvm.amdgcn.image.gather4.b.o.cube\0"
  "llvm.amdgcn.image.gather4.c.2d\0"
  "llvm.amdgcn.image.gather4.c.2darray\0"
  "llvm.amdgcn.image.gather4.c.b.2d\0"
  "llvm.amdgcn.image.gather4.c.b.2darray\0"
  "llvm.amdgcn.image.gather4.c.b.cl.2d\0"
  "llvm.amdgcn.image.gather4.c.b.cl.2darray\0"
  "llvm.amdgcn.image.gather4.c.b.cl.cube\0"
  "llvm.amdgcn.image.gather4.c.b.cl.o.2d\0"
  "llvm.amdgcn.image.gather4.c.b.cl.o.2darray\0"
  "llvm.amdgcn.image.gather4.c.b.cl.o.cube\0"
  "llvm.amdgcn.image.gather4.c.b.cube\0"
  "llvm.amdgcn.image.gather4.c.b.o.2d\0"
  "llvm.amdgcn.image.gather4.c.b.o.2darray\0"
  "llvm.amdgcn.image.gather4.c.b.o.cube\0"
  "llvm.amdgcn.image.gather4.c.cl.2d\0"
  "llvm.amdgcn.image.gather4.c.cl.2darray\0"
  "llvm.amdgcn.image.gather4.c.cl.cube\0"
  "llvm.amdgcn.image.gather4.c.cl.o.2d\0"
  "llvm.amdgcn.image.gather4.c.cl.o.2darray\0"
  "llvm.amdgcn.image.gather4.c.cl.o.cube\0"
  "llvm.amdgcn.image.gather4.c.cube\0"
  "llvm.amdgcn.image.gather4.c.l.2d\0"
  "llvm.amdgcn.image.gather4.c.l.2darray\0"
  "llvm.amdgcn.image.gather4.c.l.cube\0"
  "llvm.amdgcn.image.gather4.c.l.o.2d\0"
  "llvm.amdgcn.image.gather4.c.l.o.2darray\0"
  "llvm.amdgcn.image.gather4.c.l.o.cube\0"
  "llvm.amdgcn.image.gather4.c.lz.2d\0"
  "llvm.amdgcn.image.gather4.c.lz.2darray\0"
  "llvm.amdgcn.image.gather4.c.lz.cube\0"
  "llvm.amdgcn.image.gather4.c.lz.o.2d\0"
  "llvm.amdgcn.image.gather4.c.lz.o.2darray\0"
  "llvm.amdgcn.image.gather4.c.lz.o.cube\0"
  "llvm.amdgcn.image.gather4.c.o.2d\0"
  "llvm.amdgcn.image.gather4.c.o.2darray\0"
  "llvm.amdgcn.image.gather4.c.o.cube\0"
  "llvm.amdgcn.image.gather4.cl.2d\0"
  "llvm.amdgcn.image.gather4.cl.2darray\0"
  "llvm.amdgcn.image.gather4.cl.cube\0"
  "llvm.amdgcn.image.gather4.cl.o.2d\0"
  "llvm.amdgcn.image.gather4.cl.o.2darray\0"
  "llvm.amdgcn.image.gather4.cl.o.cube\0"
  "llvm.amdgcn.image.gather4.cube\0"
  "llvm.amdgcn.image.gather4.l.2d\0"
  "llvm.amdgcn.image.gather4.l.2darray\0"
  "llvm.amdgcn.image.gather4.l.cube\0"
  "llvm.amdgcn.image.gather4.l.o.2d\0"
  "llvm.amdgcn.image.gather4.l.o.2darray\0"
  "llvm.amdgcn.image.gather4.l.o.cube\0"
  "llvm.amdgcn.image.gather4.lz.2d\0"
  "llvm.amdgcn.image.gather4.lz.2darray\0"
  "llvm.amdgcn.image.gather4.lz.cube\0"
  "llvm.amdgcn.image.gather4.lz.o.2d\0"
  "llvm.amdgcn.image.gather4.lz.o.2darray\0"
  "llvm.amdgcn.image.gather4.lz.o.cube\0"
  "llvm.amdgcn.image.gather4.o.2d\0"
  "llvm.amdgcn.image.gather4.o.2darray\0"
  "llvm.amdgcn.image.gather4.o.cube\0"
  "llvm.amdgcn.image.getlod.1d\0"
  "llvm.amdgcn.image.getlod.1darray\0"
  "llvm.amdgcn.image.getlod.2d\0"
  "llvm.amdgcn.image.getlod.2darray\0"
  "llvm.amdgcn.image.getlod.3d\0"
  "llvm.amdgcn.image.getlod.cube\0"
  "llvm.amdgcn.image.getresinfo.1d\0"
  "llvm.amdgcn.image.getresinfo.1darray\0"
  "llvm.amdgcn.image.getresinfo.2d\0"
  "llvm.amdgcn.image.getresinfo.2darray\0"
  "llvm.amdgcn.image.getresinfo.2darraymsaa\0"
  "llvm.amdgcn.image.getresinfo.2dmsaa\0"
  "llvm.amdgcn.image.getresinfo.3d\0"
  "llvm.amdgcn.image.getresinfo.cube\0"
  "llvm.amdgcn.image.load.1d\0"
  "llvm.amdgcn.image.load.1darray\0"
  "llvm.amdgcn.image.load.2d\0"
  "llvm.amdgcn.image.load.2darray\0"
  "llvm.amdgcn.image.load.2darraymsaa\0"
  "llvm.amdgcn.image.load.2dmsaa\0"
  "llvm.amdgcn.image.load.3d\0"
  "llvm.amdgcn.image.load.cube\0"
  "llvm.amdgcn.image.load.mip.1d\0"
  "llvm.amdgcn.image.load.mip.1darray\0"
  "llvm.amdgcn.image.load.mip.2d\0"
  "llvm.amdgcn.image.load.mip.2darray\0"
  "llvm.amdgcn.image.load.mip.3d\0"
  "llvm.amdgcn.image.load.mip.cube\0"
  "llvm.amdgcn.image.msaa.load.2darraymsaa\0"
  "llvm.amdgcn.image.msaa.load.2dmsaa\0"
  "llvm.amdgcn.image.msaa.load.x.2darraymsaa\0"
  "llvm.amdgcn.image.msaa.load.x.2dmsaa\0"
  "llvm.amdgcn.image.sample.1d\0"
  "llvm.amdgcn.image.sample.1d.nortn\0"
  "llvm.amdgcn.image.sample.1darray\0"
  "llvm.amdgcn.image.sample.1darray.nortn\0"
  "llvm.amdgcn.image.sample.2d\0"
  "llvm.amdgcn.image.sample.2d.nortn\0"
  "llvm.amdgcn.image.sample.2darray\0"
  "llvm.amdgcn.image.sample.2darray.nortn\0"
  "llvm.amdgcn.image.sample.3d\0"
  "llvm.amdgcn.image.sample.3d.nortn\0"
  "llvm.amdgcn.image.sample.b.1d\0"
  "llvm.amdgcn.image.sample.b.1d.nortn\0"
  "llvm.amdgcn.image.sample.b.1darray\0"
  "llvm.amdgcn.image.sample.b.1darray.nortn\0"
  "llvm.amdgcn.image.sample.b.2d\0"
  "llvm.amdgcn.image.sample.b.2d.nortn\0"
  "llvm.amdgcn.image.sample.b.2darray\0"
  "llvm.amdgcn.image.sample.b.2darray.nortn\0"
  "llvm.amdgcn.image.sample.b.3d\0"
  "llvm.amdgcn.image.sample.b.3d.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.1d\0"
  "llvm.amdgcn.image.sample.b.cl.1d.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.1darray\0"
  "llvm.amdgcn.image.sample.b.cl.1darray.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.2d\0"
  "llvm.amdgcn.image.sample.b.cl.2d.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.2darray\0"
  "llvm.amdgcn.image.sample.b.cl.2darray.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.3d\0"
  "llvm.amdgcn.image.sample.b.cl.3d.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.cube\0"
  "llvm.amdgcn.image.sample.b.cl.cube.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.o.1d\0"
  "llvm.amdgcn.image.sample.b.cl.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.o.1darray\0"
  "llvm.amdgcn.image.sample.b.cl.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.o.2d\0"
  "llvm.amdgcn.image.sample.b.cl.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.o.2darray\0"
  "llvm.amdgcn.image.sample.b.cl.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.o.3d\0"
  "llvm.amdgcn.image.sample.b.cl.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.b.cl.o.cube\0"
  "llvm.amdgcn.image.sample.b.cl.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.b.cube\0"
  "llvm.amdgcn.image.sample.b.cube.nortn\0"
  "llvm.amdgcn.image.sample.b.o.1d\0"
  "llvm.amdgcn.image.sample.b.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.b.o.1darray\0"
  "llvm.amdgcn.image.sample.b.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.b.o.2d\0"
  "llvm.amdgcn.image.sample.b.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.b.o.2darray\0"
  "llvm.amdgcn.image.sample.b.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.b.o.3d\0"
  "llvm.amdgcn.image.sample.b.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.b.o.cube\0"
  "llvm.amdgcn.image.sample.b.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.1d\0"
  "llvm.amdgcn.image.sample.c.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.1darray\0"
  "llvm.amdgcn.image.sample.c.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.2d\0"
  "llvm.amdgcn.image.sample.c.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.2darray\0"
  "llvm.amdgcn.image.sample.c.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.3d\0"
  "llvm.amdgcn.image.sample.c.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.1d\0"
  "llvm.amdgcn.image.sample.c.b.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.1darray\0"
  "llvm.amdgcn.image.sample.c.b.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.b.2d\0"
  "llvm.amdgcn.image.sample.c.b.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.2darray\0"
  "llvm.amdgcn.image.sample.c.b.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.b.3d\0"
  "llvm.amdgcn.image.sample.c.b.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.1d\0"
  "llvm.amdgcn.image.sample.c.b.cl.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.1darray\0"
  "llvm.amdgcn.image.sample.c.b.cl.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.2d\0"
  "llvm.amdgcn.image.sample.c.b.cl.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.2darray\0"
  "llvm.amdgcn.image.sample.c.b.cl.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.3d\0"
  "llvm.amdgcn.image.sample.c.b.cl.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.cube\0"
  "llvm.amdgcn.image.sample.c.b.cl.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.1d\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.1darray\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.2d\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.2darray\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.3d\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.cube\0"
  "llvm.amdgcn.image.sample.c.b.cl.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.b.cube\0"
  "llvm.amdgcn.image.sample.c.b.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.b.o.1d\0"
  "llvm.amdgcn.image.sample.c.b.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.o.1darray\0"
  "llvm.amdgcn.image.sample.c.b.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.b.o.2d\0"
  "llvm.amdgcn.image.sample.c.b.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.o.2darray\0"
  "llvm.amdgcn.image.sample.c.b.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.b.o.3d\0"
  "llvm.amdgcn.image.sample.c.b.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.b.o.cube\0"
  "llvm.amdgcn.image.sample.c.b.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.1d\0"
  "llvm.amdgcn.image.sample.c.cd.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.1darray\0"
  "llvm.amdgcn.image.sample.c.cd.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.2d\0"
  "llvm.amdgcn.image.sample.c.cd.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.2darray\0"
  "llvm.amdgcn.image.sample.c.cd.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.3d\0"
  "llvm.amdgcn.image.sample.c.cd.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.1d\0"
  "llvm.amdgcn.image.sample.c.cd.cl.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.1darray\0"
  "llvm.amdgcn.image.sample.c.cd.cl.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.2d\0"
  "llvm.amdgcn.image.sample.c.cd.cl.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.2darray\0"
  "llvm.amdgcn.image.sample.c.cd.cl.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.3d\0"
  "llvm.amdgcn.image.sample.c.cd.cl.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.cube\0"
  "llvm.amdgcn.image.sample.c.cd.cl.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.1d\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.1darray\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.2d\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.2darray\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.3d\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.cube\0"
  "llvm.amdgcn.image.sample.c.cd.cl.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.cube\0"
  "llvm.amdgcn.image.sample.c.cd.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.o.1d\0"
  "llvm.amdgcn.image.sample.c.cd.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.o.1darray\0"
  "llvm.amdgcn.image.sample.c.cd.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.o.2d\0"
  "llvm.amdgcn.image.sample.c.cd.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.o.2darray\0"
  "llvm.amdgcn.image.sample.c.cd.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.o.3d\0"
  "llvm.amdgcn.image.sample.c.cd.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.cd.o.cube\0"
  "llvm.amdgcn.image.sample.c.cd.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.1d\0"
  "llvm.amdgcn.image.sample.c.cl.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.1darray\0"
  "llvm.amdgcn.image.sample.c.cl.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.2d\0"
  "llvm.amdgcn.image.sample.c.cl.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.2darray\0"
  "llvm.amdgcn.image.sample.c.cl.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.3d\0"
  "llvm.amdgcn.image.sample.c.cl.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.cube\0"
  "llvm.amdgcn.image.sample.c.cl.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.o.1d\0"
  "llvm.amdgcn.image.sample.c.cl.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.o.1darray\0"
  "llvm.amdgcn.image.sample.c.cl.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.o.2d\0"
  "llvm.amdgcn.image.sample.c.cl.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.o.2darray\0"
  "llvm.amdgcn.image.sample.c.cl.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.o.3d\0"
  "llvm.amdgcn.image.sample.c.cl.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.cl.o.cube\0"
  "llvm.amdgcn.image.sample.c.cl.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.cube\0"
  "llvm.amdgcn.image.sample.c.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.d.1d\0"
  "llvm.amdgcn.image.sample.c.d.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.1darray\0"
  "llvm.amdgcn.image.sample.c.d.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.d.2d\0"
  "llvm.amdgcn.image.sample.c.d.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.2darray\0"
  "llvm.amdgcn.image.sample.c.d.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.d.3d\0"
  "llvm.amdgcn.image.sample.c.d.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.1d\0"
  "llvm.amdgcn.image.sample.c.d.cl.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.1darray\0"
  "llvm.amdgcn.image.sample.c.d.cl.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.2d\0"
  "llvm.amdgcn.image.sample.c.d.cl.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.2darray\0"
  "llvm.amdgcn.image.sample.c.d.cl.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.3d\0"
  "llvm.amdgcn.image.sample.c.d.cl.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.cube\0"
  "llvm.amdgcn.image.sample.c.d.cl.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.1d\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.1darray\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.2d\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.2darray\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.3d\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.cube\0"
  "llvm.amdgcn.image.sample.c.d.cl.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.d.cube\0"
  "llvm.amdgcn.image.sample.c.d.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.d.o.1d\0"
  "llvm.amdgcn.image.sample.c.d.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.o.1darray\0"
  "llvm.amdgcn.image.sample.c.d.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.d.o.2d\0"
  "llvm.amdgcn.image.sample.c.d.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.o.2darray\0"
  "llvm.amdgcn.image.sample.c.d.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.d.o.3d\0"
  "llvm.amdgcn.image.sample.c.d.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.d.o.cube\0"
  "llvm.amdgcn.image.sample.c.d.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.l.1d\0"
  "llvm.amdgcn.image.sample.c.l.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.l.1darray\0"
  "llvm.amdgcn.image.sample.c.l.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.l.2d\0"
  "llvm.amdgcn.image.sample.c.l.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.l.2darray\0"
  "llvm.amdgcn.image.sample.c.l.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.l.3d\0"
  "llvm.amdgcn.image.sample.c.l.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.l.cube\0"
  "llvm.amdgcn.image.sample.c.l.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.l.o.1d\0"
  "llvm.amdgcn.image.sample.c.l.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.l.o.1darray\0"
  "llvm.amdgcn.image.sample.c.l.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.l.o.2d\0"
  "llvm.amdgcn.image.sample.c.l.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.l.o.2darray\0"
  "llvm.amdgcn.image.sample.c.l.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.l.o.3d\0"
  "llvm.amdgcn.image.sample.c.l.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.l.o.cube\0"
  "llvm.amdgcn.image.sample.c.l.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.1d\0"
  "llvm.amdgcn.image.sample.c.lz.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.1darray\0"
  "llvm.amdgcn.image.sample.c.lz.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.2d\0"
  "llvm.amdgcn.image.sample.c.lz.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.2darray\0"
  "llvm.amdgcn.image.sample.c.lz.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.3d\0"
  "llvm.amdgcn.image.sample.c.lz.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.cube\0"
  "llvm.amdgcn.image.sample.c.lz.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.o.1d\0"
  "llvm.amdgcn.image.sample.c.lz.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.o.1darray\0"
  "llvm.amdgcn.image.sample.c.lz.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.o.2d\0"
  "llvm.amdgcn.image.sample.c.lz.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.o.2darray\0"
  "llvm.amdgcn.image.sample.c.lz.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.o.3d\0"
  "llvm.amdgcn.image.sample.c.lz.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.lz.o.cube\0"
  "llvm.amdgcn.image.sample.c.lz.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.c.o.1d\0"
  "llvm.amdgcn.image.sample.c.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.c.o.1darray\0"
  "llvm.amdgcn.image.sample.c.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.c.o.2d\0"
  "llvm.amdgcn.image.sample.c.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.c.o.2darray\0"
  "llvm.amdgcn.image.sample.c.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.c.o.3d\0"
  "llvm.amdgcn.image.sample.c.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.c.o.cube\0"
  "llvm.amdgcn.image.sample.c.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.cd.1d\0"
  "llvm.amdgcn.image.sample.cd.1d.nortn\0"
  "llvm.amdgcn.image.sample.cd.1darray\0"
  "llvm.amdgcn.image.sample.cd.1darray.nortn\0"
  "llvm.amdgcn.image.sample.cd.2d\0"
  "llvm.amdgcn.image.sample.cd.2d.nortn\0"
  "llvm.amdgcn.image.sample.cd.2darray\0"
  "llvm.amdgcn.image.sample.cd.2darray.nortn\0"
  "llvm.amdgcn.image.sample.cd.3d\0"
  "llvm.amdgcn.image.sample.cd.3d.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.1d\0"
  "llvm.amdgcn.image.sample.cd.cl.1d.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.1darray\0"
  "llvm.amdgcn.image.sample.cd.cl.1darray.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.2d\0"
  "llvm.amdgcn.image.sample.cd.cl.2d.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.2darray\0"
  "llvm.amdgcn.image.sample.cd.cl.2darray.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.3d\0"
  "llvm.amdgcn.image.sample.cd.cl.3d.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.cube\0"
  "llvm.amdgcn.image.sample.cd.cl.cube.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.o.1d\0"
  "llvm.amdgcn.image.sample.cd.cl.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.o.1darray\0"
  "llvm.amdgcn.image.sample.cd.cl.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.o.2d\0"
  "llvm.amdgcn.image.sample.cd.cl.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.o.2darray\0"
  "llvm.amdgcn.image.sample.cd.cl.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.o.3d\0"
  "llvm.amdgcn.image.sample.cd.cl.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.cd.cl.o.cube\0"
  "llvm.amdgcn.image.sample.cd.cl.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.cd.cube\0"
  "llvm.amdgcn.image.sample.cd.cube.nortn\0"
  "llvm.amdgcn.image.sample.cd.o.1d\0"
  "llvm.amdgcn.image.sample.cd.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.cd.o.1darray\0"
  "llvm.amdgcn.image.sample.cd.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.cd.o.2d\0"
  "llvm.amdgcn.image.sample.cd.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.cd.o.2darray\0"
  "llvm.amdgcn.image.sample.cd.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.cd.o.3d\0"
  "llvm.amdgcn.image.sample.cd.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.cd.o.cube\0"
  "llvm.amdgcn.image.sample.cd.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.cl.1d\0"
  "llvm.amdgcn.image.sample.cl.1d.nortn\0"
  "llvm.amdgcn.image.sample.cl.1darray\0"
  "llvm.amdgcn.image.sample.cl.1darray.nortn\0"
  "llvm.amdgcn.image.sample.cl.2d\0"
  "llvm.amdgcn.image.sample.cl.2d.nortn\0"
  "llvm.amdgcn.image.sample.cl.2darray\0"
  "llvm.amdgcn.image.sample.cl.2darray.nortn\0"
  "llvm.amdgcn.image.sample.cl.3d\0"
  "llvm.amdgcn.image.sample.cl.3d.nortn\0"
  "llvm.amdgcn.image.sample.cl.cube\0"
  "llvm.amdgcn.image.sample.cl.cube.nortn\0"
  "llvm.amdgcn.image.sample.cl.o.1d\0"
  "llvm.amdgcn.image.sample.cl.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.cl.o.1darray\0"
  "llvm.amdgcn.image.sample.cl.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.cl.o.2d\0"
  "llvm.amdgcn.image.sample.cl.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.cl.o.2darray\0"
  "llvm.amdgcn.image.sample.cl.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.cl.o.3d\0"
  "llvm.amdgcn.image.sample.cl.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.cl.o.cube\0"
  "llvm.amdgcn.image.sample.cl.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.cube\0"
  "llvm.amdgcn.image.sample.cube.nortn\0"
  "llvm.amdgcn.image.sample.d.1d\0"
  "llvm.amdgcn.image.sample.d.1d.nortn\0"
  "llvm.amdgcn.image.sample.d.1darray\0"
  "llvm.amdgcn.image.sample.d.1darray.nortn\0"
  "llvm.amdgcn.image.sample.d.2d\0"
  "llvm.amdgcn.image.sample.d.2d.nortn\0"
  "llvm.amdgcn.image.sample.d.2darray\0"
  "llvm.amdgcn.image.sample.d.2darray.nortn\0"
  "llvm.amdgcn.image.sample.d.3d\0"
  "llvm.amdgcn.image.sample.d.3d.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.1d\0"
  "llvm.amdgcn.image.sample.d.cl.1d.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.1darray\0"
  "llvm.amdgcn.image.sample.d.cl.1darray.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.2d\0"
  "llvm.amdgcn.image.sample.d.cl.2d.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.2darray\0"
  "llvm.amdgcn.image.sample.d.cl.2darray.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.3d\0"
  "llvm.amdgcn.image.sample.d.cl.3d.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.cube\0"
  "llvm.amdgcn.image.sample.d.cl.cube.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.o.1d\0"
  "llvm.amdgcn.image.sample.d.cl.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.o.1darray\0"
  "llvm.amdgcn.image.sample.d.cl.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.o.2d\0"
  "llvm.amdgcn.image.sample.d.cl.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.o.2darray\0"
  "llvm.amdgcn.image.sample.d.cl.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.o.3d\0"
  "llvm.amdgcn.image.sample.d.cl.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.d.cl.o.cube\0"
  "llvm.amdgcn.image.sample.d.cl.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.d.cube\0"
  "llvm.amdgcn.image.sample.d.cube.nortn\0"
  "llvm.amdgcn.image.sample.d.o.1d\0"
  "llvm.amdgcn.image.sample.d.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.d.o.1darray\0"
  "llvm.amdgcn.image.sample.d.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.d.o.2d\0"
  "llvm.amdgcn.image.sample.d.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.d.o.2darray\0"
  "llvm.amdgcn.image.sample.d.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.d.o.3d\0"
  "llvm.amdgcn.image.sample.d.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.d.o.cube\0"
  "llvm.amdgcn.image.sample.d.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.l.1d\0"
  "llvm.amdgcn.image.sample.l.1d.nortn\0"
  "llvm.amdgcn.image.sample.l.1darray\0"
  "llvm.amdgcn.image.sample.l.1darray.nortn\0"
  "llvm.amdgcn.image.sample.l.2d\0"
  "llvm.amdgcn.image.sample.l.2d.nortn\0"
  "llvm.amdgcn.image.sample.l.2darray\0"
  "llvm.amdgcn.image.sample.l.2darray.nortn\0"
  "llvm.amdgcn.image.sample.l.3d\0"
  "llvm.amdgcn.image.sample.l.3d.nortn\0"
  "llvm.amdgcn.image.sample.l.cube\0"
  "llvm.amdgcn.image.sample.l.cube.nortn\0"
  "llvm.amdgcn.image.sample.l.o.1d\0"
  "llvm.amdgcn.image.sample.l.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.l.o.1darray\0"
  "llvm.amdgcn.image.sample.l.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.l.o.2d\0"
  "llvm.amdgcn.image.sample.l.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.l.o.2darray\0"
  "llvm.amdgcn.image.sample.l.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.l.o.3d\0"
  "llvm.amdgcn.image.sample.l.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.l.o.cube\0"
  "llvm.amdgcn.image.sample.l.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.lz.1d\0"
  "llvm.amdgcn.image.sample.lz.1d.nortn\0"
  "llvm.amdgcn.image.sample.lz.1darray\0"
  "llvm.amdgcn.image.sample.lz.1darray.nortn\0"
  "llvm.amdgcn.image.sample.lz.2d\0"
  "llvm.amdgcn.image.sample.lz.2d.nortn\0"
  "llvm.amdgcn.image.sample.lz.2darray\0"
  "llvm.amdgcn.image.sample.lz.2darray.nortn\0"
  "llvm.amdgcn.image.sample.lz.3d\0"
  "llvm.amdgcn.image.sample.lz.3d.nortn\0"
  "llvm.amdgcn.image.sample.lz.cube\0"
  "llvm.amdgcn.image.sample.lz.cube.nortn\0"
  "llvm.amdgcn.image.sample.lz.o.1d\0"
  "llvm.amdgcn.image.sample.lz.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.lz.o.1darray\0"
  "llvm.amdgcn.image.sample.lz.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.lz.o.2d\0"
  "llvm.amdgcn.image.sample.lz.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.lz.o.2darray\0"
  "llvm.amdgcn.image.sample.lz.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.lz.o.3d\0"
  "llvm.amdgcn.image.sample.lz.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.lz.o.cube\0"
  "llvm.amdgcn.image.sample.lz.o.cube.nortn\0"
  "llvm.amdgcn.image.sample.o.1d\0"
  "llvm.amdgcn.image.sample.o.1d.nortn\0"
  "llvm.amdgcn.image.sample.o.1darray\0"
  "llvm.amdgcn.image.sample.o.1darray.nortn\0"
  "llvm.amdgcn.image.sample.o.2d\0"
  "llvm.amdgcn.image.sample.o.2d.nortn\0"
  "llvm.amdgcn.image.sample.o.2darray\0"
  "llvm.amdgcn.image.sample.o.2darray.nortn\0"
  "llvm.amdgcn.image.sample.o.3d\0"
  "llvm.amdgcn.image.sample.o.3d.nortn\0"
  "llvm.amdgcn.image.sample.o.cube\0"
  "llvm.amdgcn.image.sample.o.cube.nortn\0"
  "llvm.amdgcn.image.store.1d\0"
  "llvm.amdgcn.image.store.1darray\0"
  "llvm.amdgcn.image.store.2d\0"
  "llvm.amdgcn.image.store.2darray\0"
  "llvm.amdgcn.image.store.2darraymsaa\0"
  "llvm.amdgcn.image.store.2dmsaa\0"
  "llvm.amdgcn.image.store.3d\0"
  "llvm.amdgcn.image.store.cube\0"
  "llvm.amdgcn.image.store.mip.1d\0"
  "llvm.amdgcn.image.store.mip.1darray\0"
  "llvm.amdgcn.image.store.mip.2d\0"
  "llvm.amdgcn.image.store.mip.2darray\0"
  "llvm.amdgcn.image.store.mip.3d\0"
  "llvm.amdgcn.image.store.mip.cube\0"
  "llvm.amdgcn.implicit.buffer.ptr\0"
  "llvm.amdgcn.implicitarg.ptr\0"
  "llvm.amdgcn.init.exec\0"
  "llvm.amdgcn.init.exec.from.input\0"
  "llvm.amdgcn.init.whole.wave\0"
  "llvm.amdgcn.interp.inreg.p10\0"
  "llvm.amdgcn.interp.inreg.p10.f16\0"
  "llvm.amdgcn.interp.inreg.p2\0"
  "llvm.amdgcn.interp.inreg.p2.f16\0"
  "llvm.amdgcn.interp.mov\0"
  "llvm.amdgcn.interp.p1\0"
  "llvm.amdgcn.interp.p1.f16\0"
  "llvm.amdgcn.interp.p10.rtz.f16\0"
  "llvm.amdgcn.interp.p2\0"
  "llvm.amdgcn.interp.p2.f16\0"
  "llvm.amdgcn.interp.p2.rtz.f16\0"
  "llvm.amdgcn.inverse.ballot\0"
  "llvm.amdgcn.is.private\0"
  "llvm.amdgcn.is.shared\0"
  "llvm.amdgcn.kernarg.segment.ptr\0"
  "llvm.amdgcn.kill\0"
  "llvm.amdgcn.lds.direct.load\0"
  "llvm.amdgcn.lds.kernel.id\0"
  "llvm.amdgcn.lds.param.load\0"
  "llvm.amdgcn.lerp\0"
  "llvm.amdgcn.live.mask\0"
  "llvm.amdgcn.load.to.lds\0"
  "llvm.amdgcn.log\0"
  "llvm.amdgcn.log.clamp\0"
  "llvm.amdgcn.loop\0"
  "llvm.amdgcn.make.buffer.rsrc\0"
  "llvm.amdgcn.mbcnt.hi\0"
  "llvm.amdgcn.mbcnt.lo\0"
  "llvm.amdgcn.mfma.f32.16x16x16bf16.1k\0"
  "llvm.amdgcn.mfma.f32.16x16x16f16\0"
  "llvm.amdgcn.mfma.f32.16x16x1f32\0"
  "llvm.amdgcn.mfma.f32.16x16x2bf16\0"
  "llvm.amdgcn.mfma.f32.16x16x32.bf16\0"
  "llvm.amdgcn.mfma.f32.16x16x32.bf8.bf8\0"
  "llvm.amdgcn.mfma.f32.16x16x32.bf8.fp8\0"
  "llvm.amdgcn.mfma.f32.16x16x32.f16\0"
  "llvm.amdgcn.mfma.f32.16x16x32.fp8.bf8\0"
  "llvm.amdgcn.mfma.f32.16x16x32.fp8.fp8\0"
  "llvm.amdgcn.mfma.f32.16x16x4bf16.1k\0"
  "llvm.amdgcn.mfma.f32.16x16x4f16\0"
  "llvm.amdgcn.mfma.f32.16x16x4f32\0"
  "llvm.amdgcn.mfma.f32.16x16x8.xf32\0"
  "llvm.amdgcn.mfma.f32.16x16x8bf16\0"
  "llvm.amdgcn.mfma.f32.32x32x16.bf16\0"
  "llvm.amdgcn.mfma.f32.32x32x16.bf8.bf8\0"
  "llvm.amdgcn.mfma.f32.32x32x16.bf8.fp8\0"
  "llvm.amdgcn.mfma.f32.32x32x16.f16\0"
  "llvm.amdgcn.mfma.f32.32x32x16.fp8.bf8\0"
  "llvm.amdgcn.mfma.f32.32x32x16.fp8.fp8\0"
  "llvm.amdgcn.mfma.f32.32x32x1f32\0"
  "llvm.amdgcn.mfma.f32.32x32x2bf16\0"
  "llvm.amdgcn.mfma.f32.32x32x2f32\0"
  "llvm.amdgcn.mfma.f32.32x32x4.xf32\0"
  "llvm.amdgcn.mfma.f32.32x32x4bf16\0"
  "llvm.amdgcn.mfma.f32.32x32x4bf16.1k\0"
  "llvm.amdgcn.mfma.f32.32x32x4f16\0"
  "llvm.amdgcn.mfma.f32.32x32x8bf16.1k\0"
  "llvm.amdgcn.mfma.f32.32x32x8f16\0"
  "llvm.amdgcn.mfma.f32.4x4x1f32\0"
  "llvm.amdgcn.mfma.f32.4x4x2bf16\0"
  "llvm.amdgcn.mfma.f32.4x4x4bf16.1k\0"
  "llvm.amdgcn.mfma.f32.4x4x4f16\0"
  "llvm.amdgcn.mfma.f64.16x16x4f64\0"
  "llvm.amdgcn.mfma.f64.4x4x4f64\0"
  "llvm.amdgcn.mfma.i32.16x16x16i8\0"
  "llvm.amdgcn.mfma.i32.16x16x32.i8\0"
  "llvm.amdgcn.mfma.i32.16x16x4i8\0"
  "llvm.amdgcn.mfma.i32.16x16x64.i8\0"
  "llvm.amdgcn.mfma.i32.32x32x16.i8\0"
  "llvm.amdgcn.mfma.i32.32x32x32.i8\0"
  "llvm.amdgcn.mfma.i32.32x32x4i8\0"
  "llvm.amdgcn.mfma.i32.32x32x8i8\0"
  "llvm.amdgcn.mfma.i32.4x4x4i8\0"
  "llvm.amdgcn.mfma.scale.f32.16x16x128.f8f6f4\0"
  "llvm.amdgcn.mfma.scale.f32.32x32x64.f8f6f4\0"
  "llvm.amdgcn.mov.dpp\0"
  "llvm.amdgcn.mov.dpp8\0"
  "llvm.amdgcn.mqsad.pk.u16.u8\0"
  "llvm.amdgcn.mqsad.u32.u8\0"
  "llvm.amdgcn.msad.u8\0"
  "llvm.amdgcn.mul.i24\0"
  "llvm.amdgcn.mul.u24\0"
  "llvm.amdgcn.mulhi.i24\0"
  "llvm.amdgcn.mulhi.u24\0"
  "llvm.amdgcn.perm\0"
  "llvm.amdgcn.perm.pk16.b4.u4\0"
  "llvm.amdgcn.perm.pk16.b6.u4\0"
  "llvm.amdgcn.perm.pk16.b8.u4\0"
  "llvm.amdgcn.permlane.bcast\0"
  "llvm.amdgcn.permlane.down\0"
  "llvm.amdgcn.permlane.idx.gen\0"
  "llvm.amdgcn.permlane.up\0"
  "llvm.amdgcn.permlane.xor\0"
  "llvm.amdgcn.permlane16\0"
  "llvm.amdgcn.permlane16.swap\0"
  "llvm.amdgcn.permlane16.var\0"
  "llvm.amdgcn.permlane32.swap\0"
  "llvm.amdgcn.permlane64\0"
  "llvm.amdgcn.permlanex16\0"
  "llvm.amdgcn.permlanex16.var\0"
  "llvm.amdgcn.pk.add.max.i16\0"
  "llvm.amdgcn.pk.add.max.u16\0"
  "llvm.amdgcn.pk.add.min.i16\0"
  "llvm.amdgcn.pk.add.min.u16\0"
  "llvm.amdgcn.pops.exiting.wave.id\0"
  "llvm.amdgcn.prng.b32\0"
  "llvm.amdgcn.ps.live\0"
  "llvm.amdgcn.qsad.pk.u16.u8\0"
  "llvm.amdgcn.queue.ptr\0"
  "llvm.amdgcn.raw.atomic.buffer.load\0"
  "llvm.amdgcn.raw.buffer.atomic.add\0"
  "llvm.amdgcn.raw.buffer.atomic.and\0"
  "llvm.amdgcn.raw.buffer.atomic.cmpswap\0"
  "llvm.amdgcn.raw.buffer.atomic.cond.sub.u32\0"
  "llvm.amdgcn.raw.buffer.atomic.dec\0"
  "llvm.amdgcn.raw.buffer.atomic.fadd\0"
  "llvm.amdgcn.raw.buffer.atomic.fmax\0"
  "llvm.amdgcn.raw.buffer.atomic.fmin\0"
  "llvm.amdgcn.raw.buffer.atomic.inc\0"
  "llvm.amdgcn.raw.buffer.atomic.or\0"
  "llvm.amdgcn.raw.buffer.atomic.smax\0"
  "llvm.amdgcn.raw.buffer.atomic.smin\0"
  "llvm.amdgcn.raw.buffer.atomic.sub\0"
  "llvm.amdgcn.raw.buffer.atomic.sub.clamp.u32\0"
  "llvm.amdgcn.raw.buffer.atomic.swap\0"
  "llvm.amdgcn.raw.buffer.atomic.umax\0"
  "llvm.amdgcn.raw.buffer.atomic.umin\0"
  "llvm.amdgcn.raw.buffer.atomic.xor\0"
  "llvm.amdgcn.raw.buffer.load\0"
  "llvm.amdgcn.raw.buffer.load.format\0"
  "llvm.amdgcn.raw.buffer.load.lds\0"
  "llvm.amdgcn.raw.buffer.store\0"
  "llvm.amdgcn.raw.buffer.store.format\0"
  "llvm.amdgcn.raw.ptr.atomic.buffer.load\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.add\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.and\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.cmpswap\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.cond.sub.u32\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.dec\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.fadd\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.fmax\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.fmin\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.inc\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.or\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.smax\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.smin\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.sub\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.sub.clamp.u32\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.swap\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.umax\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.umin\0"
  "llvm.amdgcn.raw.ptr.buffer.atomic.xor\0"
  "llvm.amdgcn.raw.ptr.buffer.load\0"
  "llvm.amdgcn.raw.ptr.buffer.load.format\0"
  "llvm.amdgcn.raw.ptr.buffer.load.lds\0"
  "llvm.amdgcn.raw.ptr.buffer.store\0"
  "llvm.amdgcn.raw.ptr.buffer.store.format\0"
  "llvm.amdgcn.raw.ptr.tbuffer.load\0"
  "llvm.amdgcn.raw.ptr.tbuffer.store\0"
  "llvm.amdgcn.raw.tbuffer.load\0"
  "llvm.amdgcn.raw.tbuffer.store\0"
  "llvm.amdgcn.rcp\0"
  "llvm.amdgcn.rcp.legacy\0"
  "llvm.amdgcn.readfirstlane\0"
  "llvm.amdgcn.readlane\0"
  "llvm.amdgcn.reloc.constant\0"
  "llvm.amdgcn.rsq\0"
  "llvm.amdgcn.rsq.clamp\0"
  "llvm.amdgcn.rsq.legacy\0"
  "llvm.amdgcn.s.barrier\0"
  "llvm.amdgcn.s.barrier.init\0"
  "llvm.amdgcn.s.barrier.join\0"
  "llvm.amdgcn.s.barrier.leave\0"
  "llvm.amdgcn.s.barrier.signal\0"
  "llvm.amdgcn.s.barrier.signal.isfirst\0"
  "llvm.amdgcn.s.barrier.signal.var\0"
  "llvm.amdgcn.s.barrier.wait\0"
  "llvm.amdgcn.s.bitreplicate\0"
  "llvm.amdgcn.s.buffer.load\0"
  "llvm.amdgcn.s.buffer.prefetch.data\0"
  "llvm.amdgcn.s.cluster.barrier\0"
  "llvm.amdgcn.s.dcache.inv\0"
  "llvm.amdgcn.s.dcache.inv.vol\0"
  "llvm.amdgcn.s.dcache.wb\0"
  "llvm.amdgcn.s.dcache.wb.vol\0"
  "llvm.amdgcn.s.decperflevel\0"
  "llvm.amdgcn.s.get.barrier.state\0"
  "llvm.amdgcn.s.get.named.barrier.state\0"
  "llvm.amdgcn.s.get.waveid.in.workgroup\0"
  "llvm.amdgcn.s.getpc\0"
  "llvm.amdgcn.s.getreg\0"
  "llvm.amdgcn.s.incperflevel\0"
  "llvm.amdgcn.s.memrealtime\0"
  "llvm.amdgcn.s.memtime\0"
  "llvm.amdgcn.s.monitor.sleep\0"
  "llvm.amdgcn.s.nop\0"
  "llvm.amdgcn.s.prefetch.data\0"
  "llvm.amdgcn.s.quadmask\0"
  "llvm.amdgcn.s.sendmsg\0"
  "llvm.amdgcn.s.sendmsg.rtn\0"
  "llvm.amdgcn.s.sendmsghalt\0"
  "llvm.amdgcn.s.sethalt\0"
  "llvm.amdgcn.s.setprio\0"
  "llvm.amdgcn.s.setprio.inc.wg\0"
  "llvm.amdgcn.s.setreg\0"
  "llvm.amdgcn.s.sleep\0"
  "llvm.amdgcn.s.sleep.var\0"
  "llvm.amdgcn.s.ttracedata\0"
  "llvm.amdgcn.s.ttracedata.imm\0"
  "llvm.amdgcn.s.wait.asynccnt\0"
  "llvm.amdgcn.s.wait.bvhcnt\0"
  "llvm.amdgcn.s.wait.dscnt\0"
  "llvm.amdgcn.s.wait.event.export.ready\0"
  "llvm.amdgcn.s.wait.expcnt\0"
  "llvm.amdgcn.s.wait.kmcnt\0"
  "llvm.amdgcn.s.wait.loadcnt\0"
  "llvm.amdgcn.s.wait.samplecnt\0"
  "llvm.amdgcn.s.wait.storecnt\0"
  "llvm.amdgcn.s.wait.tensorcnt\0"
  "llvm.amdgcn.s.waitcnt\0"
  "llvm.amdgcn.s.wakeup.barrier\0"
  "llvm.amdgcn.s.wqm\0"
  "llvm.amdgcn.sad.hi.u8\0"
  "llvm.amdgcn.sad.u16\0"
  "llvm.amdgcn.sad.u8\0"
  "llvm.amdgcn.sat.pk4.i4.i8\0"
  "llvm.amdgcn.sat.pk4.u4.u8\0"
  "llvm.amdgcn.sbfe\0"
  "llvm.amdgcn.sched.barrier\0"
  "llvm.amdgcn.sched.group.barrier\0"
  "llvm.amdgcn.sdot2\0"
  "llvm.amdgcn.sdot4\0"
  "llvm.amdgcn.sdot8\0"
  "llvm.amdgcn.set.inactive\0"
  "llvm.amdgcn.set.inactive.chain.arg\0"
  "llvm.amdgcn.sffbh\0"
  "llvm.amdgcn.sin\0"
  "llvm.amdgcn.smfmac.f32.16x16x128.bf8.bf8\0"
  "llvm.amdgcn.smfmac.f32.16x16x128.bf8.fp8\0"
  "llvm.amdgcn.smfmac.f32.16x16x128.fp8.bf8\0"
  "llvm.amdgcn.smfmac.f32.16x16x128.fp8.fp8\0"
  "llvm.amdgcn.smfmac.f32.16x16x32.bf16\0"
  "llvm.amdgcn.smfmac.f32.16x16x32.f16\0"
  "llvm.amdgcn.smfmac.f32.16x16x64.bf16\0"
  "llvm.amdgcn.smfmac.f32.16x16x64.bf8.bf8\0"
  "llvm.amdgcn.smfmac.f32.16x16x64.bf8.fp8\0"
  "llvm.amdgcn.smfmac.f32.16x16x64.f16\0"
  "llvm.amdgcn.smfmac.f32.16x16x64.fp8.bf8\0"
  "llvm.amdgcn.smfmac.f32.16x16x64.fp8.fp8\0"
  "llvm.amdgcn.smfmac.f32.32x32x16.bf16\0"
  "llvm.amdgcn.smfmac.f32.32x32x16.f16\0"
  "llvm.amdgcn.smfmac.f32.32x32x32.bf16\0"
  "llvm.amdgcn.smfmac.f32.32x32x32.bf8.bf8\0"
  "llvm.amdgcn.smfmac.f32.32x32x32.bf8.fp8\0"
  "llvm.amdgcn.smfmac.f32.32x32x32.f16\0"
  "llvm.amdgcn.smfmac.f32.32x32x32.fp8.bf8\0"
  "llvm.amdgcn.smfmac.f32.32x32x32.fp8.fp8\0"
  "llvm.amdgcn.smfmac.f32.32x32x64.bf8.bf8\0"
  "llvm.amdgcn.smfmac.f32.32x32x64.bf8.fp8\0"
  "llvm.amdgcn.smfmac.f32.32x32x64.fp8.bf8\0"
  "llvm.amdgcn.smfmac.f32.32x32x64.fp8.fp8\0"
  "llvm.amdgcn.smfmac.i32.16x16x128.i8\0"
  "llvm.amdgcn.smfmac.i32.16x16x64.i8\0"
  "llvm.amdgcn.smfmac.i32.32x32x32.i8\0"
  "llvm.amdgcn.smfmac.i32.32x32x64.i8\0"
  "llvm.amdgcn.softwqm\0"
  "llvm.amdgcn.sqrt\0"
  "llvm.amdgcn.strict.wqm\0"
  "llvm.amdgcn.strict.wwm\0"
  "llvm.amdgcn.struct.atomic.buffer.load\0"
  "llvm.amdgcn.struct.buffer.atomic.add\0"
  "llvm.amdgcn.struct.buffer.atomic.and\0"
  "llvm.amdgcn.struct.buffer.atomic.cmpswap\0"
  "llvm.amdgcn.struct.buffer.atomic.cond.sub.u32\0"
  "llvm.amdgcn.struct.buffer.atomic.dec\0"
  "llvm.amdgcn.struct.buffer.atomic.fadd\0"
  "llvm.amdgcn.struct.buffer.atomic.fmax\0"
  "llvm.amdgcn.struct.buffer.atomic.fmin\0"
  "llvm.amdgcn.struct.buffer.atomic.inc\0"
  "llvm.amdgcn.struct.buffer.atomic.or\0"
  "llvm.amdgcn.struct.buffer.atomic.smax\0"
  "llvm.amdgcn.struct.buffer.atomic.smin\0"
  "llvm.amdgcn.struct.buffer.atomic.sub\0"
  "llvm.amdgcn.struct.buffer.atomic.sub.clamp.u32\0"
  "llvm.amdgcn.struct.buffer.atomic.swap\0"
  "llvm.amdgcn.struct.buffer.atomic.umax\0"
  "llvm.amdgcn.struct.buffer.atomic.umin\0"
  "llvm.amdgcn.struct.buffer.atomic.xor\0"
  "llvm.amdgcn.struct.buffer.load\0"
  "llvm.amdgcn.struct.buffer.load.format\0"
  "llvm.amdgcn.struct.buffer.load.lds\0"
  "llvm.amdgcn.struct.buffer.store\0"
  "llvm.amdgcn.struct.buffer.store.format\0"
  "llvm.amdgcn.struct.ptr.atomic.buffer.load\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.add\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.and\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.cmpswap\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.cond.sub.u32\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.dec\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.fadd\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.fmax\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.fmin\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.inc\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.or\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.smax\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.smin\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.sub\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.sub.clamp.u32\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.swap\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.umax\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.umin\0"
  "llvm.amdgcn.struct.ptr.buffer.atomic.xor\0"
  "llvm.amdgcn.struct.ptr.buffer.load\0"
  "llvm.amdgcn.struct.ptr.buffer.load.format\0"
  "llvm.amdgcn.struct.ptr.buffer.load.lds\0"
  "llvm.amdgcn.struct.ptr.buffer.store\0"
  "llvm.amdgcn.struct.ptr.buffer.store.format\0"
  "llvm.amdgcn.struct.ptr.tbuffer.load\0"
  "llvm.amdgcn.struct.ptr.tbuffer.store\0"
  "llvm.amdgcn.struct.tbuffer.load\0"
  "llvm.amdgcn.struct.tbuffer.store\0"
  "llvm.amdgcn.sudot4\0"
  "llvm.amdgcn.sudot8\0"
  "llvm.amdgcn.swmmac.bf16.16x16x32.bf16\0"
  "llvm.amdgcn.swmmac.bf16.16x16x64.bf16\0"
  "llvm.amdgcn.swmmac.bf16f32.16x16x64.bf16\0"
  "llvm.amdgcn.swmmac.f16.16x16x128.bf8.bf8\0"
  "llvm.amdgcn.swmmac.f16.16x16x128.bf8.fp8\0"
  "llvm.amdgcn.swmmac.f16.16x16x128.fp8.bf8\0"
  "llvm.amdgcn.swmmac.f16.16x16x128.fp8.fp8\0"
  "llvm.amdgcn.swmmac.f16.16x16x32.f16\0"
  "llvm.amdgcn.swmmac.f16.16x16x64.f16\0"
  "llvm.amdgcn.swmmac.f32.16x16x128.bf8.bf8\0"
  "llvm.amdgcn.swmmac.f32.16x16x128.bf8.fp8\0"
  "llvm.amdgcn.swmmac.f32.16x16x128.fp8.bf8\0"
  "llvm.amdgcn.swmmac.f32.16x16x128.fp8.fp8\0"
  "llvm.amdgcn.swmmac.f32.16x16x32.bf16\0"
  "llvm.amdgcn.swmmac.f32.16x16x32.bf8.bf8\0"
  "llvm.amdgcn.swmmac.f32.16x16x32.bf8.fp8\0"
  "llvm.amdgcn.swmmac.f32.16x16x32.f16\0"
  "llvm.amdgcn.swmmac.f32.16x16x32.fp8.bf8\0"
  "llvm.amdgcn.swmmac.f32.16x16x32.fp8.fp8\0"
  "llvm.amdgcn.swmmac.f32.16x16x64.bf16\0"
  "llvm.amdgcn.swmmac.f32.16x16x64.f16\0"
  "llvm.amdgcn.swmmac.i32.16x16x128.iu8\0"
  "llvm.amdgcn.swmmac.i32.16x16x32.iu4\0"
  "llvm.amdgcn.swmmac.i32.16x16x32.iu8\0"
  "llvm.amdgcn.swmmac.i32.16x16x64.iu4\0"
  "llvm.amdgcn.tanh\0"
  "llvm.amdgcn.tensor.load.to.lds\0"
  "llvm.amdgcn.tensor.load.to.lds.d2\0"
  "llvm.amdgcn.tensor.store.from.lds\0"
  "llvm.amdgcn.tensor.store.from.lds.d2\0"
  "llvm.amdgcn.trig.preop\0"
  "llvm.amdgcn.ubfe\0"
  "llvm.amdgcn.udot2\0"
  "llvm.amdgcn.udot4\0"
  "llvm.amdgcn.udot8\0"
  "llvm.amdgcn.unreachable\0"
  "llvm.amdgcn.update.dpp\0"
  "llvm.amdgcn.wave.barrier\0"
  "llvm.amdgcn.wave.id\0"
  "llvm.amdgcn.wave.reduce.add\0"
  "llvm.amdgcn.wave.reduce.and\0"
  "llvm.amdgcn.wave.reduce.fadd\0"
  "llvm.amdgcn.wave.reduce.fmax\0"
  "llvm.amdgcn.wave.reduce.fmin\0"
  "llvm.amdgcn.wave.reduce.fsub\0"
  "llvm.amdgcn.wave.reduce.max\0"
  "llvm.amdgcn.wave.reduce.min\0"
  "llvm.amdgcn.wave.reduce.or\0"
  "llvm.amdgcn.wave.reduce.sub\0"
  "llvm.amdgcn.wave.reduce.umax\0"
  "llvm.amdgcn.wave.reduce.umin\0"
  "llvm.amdgcn.wave.reduce.xor\0"
  "llvm.amdgcn.wave.shuffle\0"
  "llvm.amdgcn.wavefrontsize\0"
  "llvm.amdgcn.wmma.bf16.16x16x16.bf16\0"
  "llvm.amdgcn.wmma.bf16.16x16x16.bf16.tied\0"
  "llvm.amdgcn.wmma.bf16.16x16x32.bf16\0"
  "llvm.amdgcn.wmma.bf16f32.16x16x32.bf16\0"
  "llvm.amdgcn.wmma.f16.16x16x128.bf8.bf8\0"
  "llvm.amdgcn.wmma.f16.16x16x128.bf8.fp8\0"
  "llvm.amdgcn.wmma.f16.16x16x128.fp8.bf8\0"
  "llvm.amdgcn.wmma.f16.16x16x128.fp8.fp8\0"
  "llvm.amdgcn.wmma.f16.16x16x16.f16\0"
  "llvm.amdgcn.wmma.f16.16x16x16.f16.tied\0"
  "llvm.amdgcn.wmma.f16.16x16x32.f16\0"
  "llvm.amdgcn.wmma.f16.16x16x64.bf8.bf8\0"
  "llvm.amdgcn.wmma.f16.16x16x64.bf8.fp8\0"
  "llvm.amdgcn.wmma.f16.16x16x64.fp8.bf8\0"
  "llvm.amdgcn.wmma.f16.16x16x64.fp8.fp8\0"
  "llvm.amdgcn.wmma.f32.16x16x128.bf8.bf8\0"
  "llvm.amdgcn.wmma.f32.16x16x128.bf8.fp8\0"
  "llvm.amdgcn.wmma.f32.16x16x128.f8f6f4\0"
  "llvm.amdgcn.wmma.f32.16x16x128.fp8.bf8\0"
  "llvm.amdgcn.wmma.f32.16x16x128.fp8.fp8\0"
  "llvm.amdgcn.wmma.f32.16x16x16.bf16\0"
  "llvm.amdgcn.wmma.f32.16x16x16.bf8.bf8\0"
  "llvm.amdgcn.wmma.f32.16x16x16.bf8.fp8\0"
  "llvm.amdgcn.wmma.f32.16x16x16.f16\0"
  "llvm.amdgcn.wmma.f32.16x16x16.fp8.bf8\0"
  "llvm.amdgcn.wmma.f32.16x16x16.fp8.fp8\0"
  "llvm.amdgcn.wmma.f32.16x16x32.bf16\0"
  "llvm.amdgcn.wmma.f32.16x16x32.f16\0"
  "llvm.amdgcn.wmma.f32.16x16x4.f32\0"
  "llvm.amdgcn.wmma.f32.16x16x64.bf8.bf8\0"
  "llvm.amdgcn.wmma.f32.16x16x64.bf8.fp8\0"
  "llvm.amdgcn.wmma.f32.16x16x64.fp8.bf8\0"
  "llvm.amdgcn.wmma.f32.16x16x64.fp8.fp8\0"
  "llvm.amdgcn.wmma.f32.32x16x128.f4\0"
  "llvm.amdgcn.wmma.i32.16x16x16.iu4\0"
  "llvm.amdgcn.wmma.i32.16x16x16.iu8\0"
  "llvm.amdgcn.wmma.i32.16x16x32.iu4\0"
  "llvm.amdgcn.wmma.i32.16x16x64.iu8\0"
  "llvm.amdgcn.wmma.scale.f32.16x16x128.f8f6f4\0"
  "llvm.amdgcn.wmma.scale.f32.32x16x128.f4\0"
  "llvm.amdgcn.wmma.scale16.f32.16x16x128.f8f6f4\0"
  "llvm.amdgcn.wmma.scale16.f32.32x16x128.f4\0"
  "llvm.amdgcn.workgroup.id.x\0"
  "llvm.amdgcn.workgroup.id.y\0"
  "llvm.amdgcn.workgroup.id.z\0"
  "llvm.amdgcn.workitem.id.x\0"
  "llvm.amdgcn.workitem.id.y\0"
  "llvm.amdgcn.workitem.id.z\0"
  "llvm.amdgcn.wqm\0"
  "llvm.amdgcn.wqm.demote\0"
  "llvm.amdgcn.wqm.vote\0"
  "llvm.amdgcn.writelane\0"
  "llvm.amdgcn.wwm\0"
  "llvm.arm.cde.cx1\0"
  "llvm.arm.cde.cx1a\0"
  "llvm.arm.cde.cx1d\0"
  "llvm.arm.cde.cx1da\0"
  "llvm.arm.cde.cx2\0"
  "llvm.arm.cde.cx2a\0"
  "llvm.arm.cde.cx2d\0"
  "llvm.arm.cde.cx2da\0"
  "llvm.arm.cde.cx3\0"
  "llvm.arm.cde.cx3a\0"
  "llvm.arm.cde.cx3d\0"
  "llvm.arm.cde.cx3da\0"
  "llvm.arm.cde.vcx1\0"
  "llvm.arm.cde.vcx1a\0"
  "llvm.arm.cde.vcx1q\0"
  "llvm.arm.cde.vcx1q.predicated\0"
  "llvm.arm.cde.vcx1qa\0"
  "llvm.arm.cde.vcx1qa.predicated\0"
  "llvm.arm.cde.vcx2\0"
  "llvm.arm.cde.vcx2a\0"
  "llvm.arm.cde.vcx2q\0"
  "llvm.arm.cde.vcx2q.predicated\0"
  "llvm.arm.cde.vcx2qa\0"
  "llvm.arm.cde.vcx2qa.predicated\0"
  "llvm.arm.cde.vcx3\0"
  "llvm.arm.cde.vcx3a\0"
  "llvm.arm.cde.vcx3q\0"
  "llvm.arm.cde.vcx3q.predicated\0"
  "llvm.arm.cde.vcx3qa\0"
  "llvm.arm.cde.vcx3qa.predicated\0"
  "llvm.arm.cdp\0"
  "llvm.arm.cdp2\0"
  "llvm.arm.clrex\0"
  "llvm.arm.cls\0"
  "llvm.arm.cls64\0"
  "llvm.arm.cmse.tt\0"
  "llvm.arm.cmse.tta\0"
  "llvm.arm.cmse.ttat\0"
  "llvm.arm.cmse.ttt\0"
  "llvm.arm.crc32b\0"
  "llvm.arm.crc32cb\0"
  "llvm.arm.crc32ch\0"
  "llvm.arm.crc32cw\0"
  "llvm.arm.crc32h\0"
  "llvm.arm.crc32w\0"
  "llvm.arm.dbg\0"
  "llvm.arm.dmb\0"
  "llvm.arm.dsb\0"
  "llvm.arm.get.fpscr\0"
  "llvm.arm.gnu.eabi.mcount\0"
  "llvm.arm.hint\0"
  "llvm.arm.isb\0"
  "llvm.arm.ldaex\0"
  "llvm.arm.ldaexd\0"
  "llvm.arm.ldc\0"
  "llvm.arm.ldc2\0"
  "llvm.arm.ldc2l\0"
  "llvm.arm.ldcl\0"
  "llvm.arm.ldrex\0"
  "llvm.arm.ldrexd\0"
  "llvm.arm.mcr\0"
  "llvm.arm.mcr2\0"
  "llvm.arm.mcrr\0"
  "llvm.arm.mcrr2\0"
  "llvm.arm.mrc\0"
  "llvm.arm.mrc2\0"
  "llvm.arm.mrrc\0"
  "llvm.arm.mrrc2\0"
  "llvm.arm.mve.abd.predicated\0"
  "llvm.arm.mve.abs.predicated\0"
  "llvm.arm.mve.add.predicated\0"
  "llvm.arm.mve.addlv\0"
  "llvm.arm.mve.addlv.predicated\0"
  "llvm.arm.mve.addv\0"
  "llvm.arm.mve.addv.predicated\0"
  "llvm.arm.mve.and.predicated\0"
  "llvm.arm.mve.asrl\0"
  "llvm.arm.mve.bic.predicated\0"
  "llvm.arm.mve.cls.predicated\0"
  "llvm.arm.mve.clz.predicated\0"
  "llvm.arm.mve.cmp.eq\0"
  "llvm.arm.mve.cmp.ge\0"
  "llvm.arm.mve.cmp.gt\0"
  "llvm.arm.mve.cmp.le\0"
  "llvm.arm.mve.cmp.lt\0"
  "llvm.arm.mve.cmp.ne\0"
  "llvm.arm.mve.eor.predicated\0"
  "llvm.arm.mve.fma\0"
  "llvm.arm.mve.fma.predicated\0"
  "llvm.arm.mve.hadd.predicated\0"
  "llvm.arm.mve.hsub.predicated\0"
  "llvm.arm.mve.lsll\0"
  "llvm.arm.mve.max.predicated\0"
  "llvm.arm.mve.maxav\0"
  "llvm.arm.mve.maxav.predicated\0"
  "llvm.arm.mve.maxnmav\0"
  "llvm.arm.mve.maxnmav.predicated\0"
  "llvm.arm.mve.maxnmv\0"
  "llvm.arm.mve.maxnmv.predicated\0"
  "llvm.arm.mve.maxv\0"
  "llvm.arm.mve.maxv.predicated\0"
  "llvm.arm.mve.min.predicated\0"
  "llvm.arm.mve.minav\0"
  "llvm.arm.mve.minav.predicated\0"
  "llvm.arm.mve.minnmav\0"
  "llvm.arm.mve.minnmav.predicated\0"
  "llvm.arm.mve.minnmv\0"
  "llvm.arm.mve.minnmv.predicated\0"
  "llvm.arm.mve.minv\0"
  "llvm.arm.mve.minv.predicated\0"
  "llvm.arm.mve.mul.predicated\0"
  "llvm.arm.mve.mulh.predicated\0"
  "llvm.arm.mve.mull.int.predicated\0"
  "llvm.arm.mve.mull.poly.predicated\0"
  "llvm.arm.mve.mvn.predicated\0"
  "llvm.arm.mve.neg.predicated\0"
  "llvm.arm.mve.orn.predicated\0"
  "llvm.arm.mve.orr.predicated\0"
  "llvm.arm.mve.pred.i2v\0"
  "llvm.arm.mve.pred.v2i\0"
  "llvm.arm.mve.qabs.predicated\0"
  "llvm.arm.mve.qadd.predicated\0"
  "llvm.arm.mve.qdmulh.predicated\0"
  "llvm.arm.mve.qneg.predicated\0"
  "llvm.arm.mve.qrdmulh.predicated\0"
  "llvm.arm.mve.qsub.predicated\0"
  "llvm.arm.mve.rhadd.predicated\0"
  "llvm.arm.mve.rmulh.predicated\0"
  "llvm.arm.mve.shl.imm.predicated\0"
  "llvm.arm.mve.shr.imm.predicated\0"
  "llvm.arm.mve.sqrshr\0"
  "llvm.arm.mve.sqrshrl\0"
  "llvm.arm.mve.sqshl\0"
  "llvm.arm.mve.sqshll\0"
  "llvm.arm.mve.srshr\0"
  "llvm.arm.mve.srshrl\0"
  "llvm.arm.mve.sub.predicated\0"
  "llvm.arm.mve.uqrshl\0"
  "llvm.arm.mve.uqrshll\0"
  "llvm.arm.mve.uqshl\0"
  "llvm.arm.mve.uqshll\0"
  "llvm.arm.mve.urshr\0"
  "llvm.arm.mve.urshrl\0"
  "llvm.arm.mve.vabav\0"
  "llvm.arm.mve.vabav.predicated\0"
  "llvm.arm.mve.vabd\0"
  "llvm.arm.mve.vadc\0"
  "llvm.arm.mve.vadc.predicated\0"
  "llvm.arm.mve.vadd\0"
  "llvm.arm.mve.vbrsr\0"
  "llvm.arm.mve.vbrsr.predicated\0"
  "llvm.arm.mve.vcaddq\0"
  "llvm.arm.mve.vcaddq.predicated\0"
  "llvm.arm.mve.vcls\0"
  "llvm.arm.mve.vcmlaq\0"
  "llvm.arm.mve.vcmlaq.predicated\0"
  "llvm.arm.mve.vcmulq\0"
  "llvm.arm.mve.vcmulq.predicated\0"
  "llvm.arm.mve.vctp16\0"
  "llvm.arm.mve.vctp32\0"
  "llvm.arm.mve.vctp64\0"
  "llvm.arm.mve.vctp8\0"
  "llvm.arm.mve.vcvt.fix\0"
  "llvm.arm.mve.vcvt.fix.predicated\0"
  "llvm.arm.mve.vcvt.fp.int\0"
  "llvm.arm.mve.vcvt.fp.int.predicated\0"
  "llvm.arm.mve.vcvt.int.fp\0"
  "llvm.arm.mve.vcvt.narrow\0"
  "llvm.arm.mve.vcvt.narrow.predicated\0"
  "llvm.arm.mve.vcvt.widen\0"
  "llvm.arm.mve.vcvt.widen.predicated\0"
  "llvm.arm.mve.vcvta\0"
  "llvm.arm.mve.vcvta.predicated\0"
  "llvm.arm.mve.vcvtm\0"
  "llvm.arm.mve.vcvtm.predicated\0"
  "llvm.arm.mve.vcvtn\0"
  "llvm.arm.mve.vcvtn.predicated\0"
  "llvm.arm.mve.vcvtp\0"
  "llvm.arm.mve.vcvtp.predicated\0"
  "llvm.arm.mve.vddup\0"
  "llvm.arm.mve.vddup.predicated\0"
  "llvm.arm.mve.vdwdup\0"
  "llvm.arm.mve.vdwdup.predicated\0"
  "llvm.arm.mve.vhadd\0"
  "llvm.arm.mve.vhsub\0"
  "llvm.arm.mve.vidup\0"
  "llvm.arm.mve.vidup.predicated\0"
  "llvm.arm.mve.viwdup\0"
  "llvm.arm.mve.viwdup.predicated\0"
  "llvm.arm.mve.vld2q\0"
  "llvm.arm.mve.vld4q\0"
  "llvm.arm.mve.vldr.gather.base\0"
  "llvm.arm.mve.vldr.gather.base.predicated\0"
  "llvm.arm.mve.vldr.gather.base.wb\0"
  "llvm.arm.mve.vldr.gather.base.wb.predicated\0"
  "llvm.arm.mve.vldr.gather.offset\0"
  "llvm.arm.mve.vldr.gather.offset.predicated\0"
  "llvm.arm.mve.vmaxa.predicated\0"
  "llvm.arm.mve.vmaxnm\0"
  "llvm.arm.mve.vmaxnma.predicated\0"
  "llvm.arm.mve.vmina.predicated\0"
  "llvm.arm.mve.vminnm\0"
  "llvm.arm.mve.vminnma.predicated\0"
  "llvm.arm.mve.vmla.n.predicated\0"
  "llvm.arm.mve.vmlas.n.predicated\0"
  "llvm.arm.mve.vmldava\0"
  "llvm.arm.mve.vmldava.predicated\0"
  "llvm.arm.mve.vmlldava\0"
  "llvm.arm.mve.vmlldava.predicated\0"
  "llvm.arm.mve.vmovl.predicated\0"
  "llvm.arm.mve.vmovn.predicated\0"
  "llvm.arm.mve.vmul\0"
  "llvm.arm.mve.vmulh\0"
  "llvm.arm.mve.vmull\0"
  "llvm.arm.mve.vmull.poly\0"
  "llvm.arm.mve.vqdmlad\0"
  "llvm.arm.mve.vqdmlad.predicated\0"
  "llvm.arm.mve.vqdmlah\0"
  "llvm.arm.mve.vqdmlah.predicated\0"
  "llvm.arm.mve.vqdmlash\0"
  "llvm.arm.mve.vqdmlash.predicated\0"
  "llvm.arm.mve.vqdmulh\0"
  "llvm.arm.mve.vqdmull\0"
  "llvm.arm.mve.vqdmull.predicated\0"
  "llvm.arm.mve.vqmovn\0"
  "llvm.arm.mve.vqmovn.predicated\0"
  "llvm.arm.mve.vqrdmlah\0"
  "llvm.arm.mve.vqrdmlah.predicated\0"
  "llvm.arm.mve.vqrdmlash\0"
  "llvm.arm.mve.vqrdmlash.predicated\0"
  "llvm.arm.mve.vqrdmulh\0"
  "llvm.arm.mve.vqshl.imm\0"
  "llvm.arm.mve.vqshl.imm.predicated\0"
  "llvm.arm.mve.vqshlu.imm\0"
  "llvm.arm.mve.vqshlu.imm.predicated\0"
  "llvm.arm.mve.vreinterpretq\0"
  "llvm.arm.mve.vrev.predicated\0"
  "llvm.arm.mve.vrhadd\0"
  "llvm.arm.mve.vrinta\0"
  "llvm.arm.mve.vrinta.predicated\0"
  "llvm.arm.mve.vrintm\0"
  "llvm.arm.mve.vrintm.predicated\0"
  "llvm.arm.mve.vrintn\0"
  "llvm.arm.mve.vrintn.predicated\0"
  "llvm.arm.mve.vrintp\0"
  "llvm.arm.mve.vrintp.predicated\0"
  "llvm.arm.mve.vrintx\0"
  "llvm.arm.mve.vrintx.predicated\0"
  "llvm.arm.mve.vrintz\0"
  "llvm.arm.mve.vrintz.predicated\0"
  "llvm.arm.mve.vrmlldavha\0"
  "llvm.arm.mve.vrmlldavha.predicated\0"
  "llvm.arm.mve.vrmulh\0"
  "llvm.arm.mve.vrshr.imm\0"
  "llvm.arm.mve.vrshr.imm.predicated\0"
  "llvm.arm.mve.vsbc\0"
  "llvm.arm.mve.vsbc.predicated\0"
  "llvm.arm.mve.vshl.scalar\0"
  "llvm.arm.mve.vshl.scalar.predicated\0"
  "llvm.arm.mve.vshl.vector\0"
  "llvm.arm.mve.vshl.vector.predicated\0"
  "llvm.arm.mve.vshlc\0"
  "llvm.arm.mve.vshlc.predicated\0"
  "llvm.arm.mve.vshll.imm\0"
  "llvm.arm.mve.vshll.imm.predicated\0"
  "llvm.arm.mve.vshrn\0"
  "llvm.arm.mve.vshrn.predicated\0"
  "llvm.arm.mve.vsli\0"
  "llvm.arm.mve.vsli.predicated\0"
  "llvm.arm.mve.vsri\0"
  "llvm.arm.mve.vsri.predicated\0"
  "llvm.arm.mve.vst2q\0"
  "llvm.arm.mve.vst4q\0"
  "llvm.arm.mve.vstr.scatter.base\0"
  "llvm.arm.mve.vstr.scatter.base.predicated\0"
  "llvm.arm.mve.vstr.scatter.base.wb\0"
  "llvm.arm.mve.vstr.scatter.base.wb.predicated\0"
  "llvm.arm.mve.vstr.scatter.offset\0"
  "llvm.arm.mve.vstr.scatter.offset.predicated\0"
  "llvm.arm.mve.vsub\0"
  "llvm.arm.neon.aesd\0"
  "llvm.arm.neon.aese\0"
  "llvm.arm.neon.aesimc\0"
  "llvm.arm.neon.aesmc\0"
  "llvm.arm.neon.bfdot\0"
  "llvm.arm.neon.bfmlalb\0"
  "llvm.arm.neon.bfmlalt\0"
  "llvm.arm.neon.bfmmla\0"
  "llvm.arm.neon.sdot\0"
  "llvm.arm.neon.sha1c\0"
  "llvm.arm.neon.sha1h\0"
  "llvm.arm.neon.sha1m\0"
  "llvm.arm.neon.sha1p\0"
  "llvm.arm.neon.sha1su0\0"
  "llvm.arm.neon.sha1su1\0"
  "llvm.arm.neon.sha256h\0"
  "llvm.arm.neon.sha256h2\0"
  "llvm.arm.neon.sha256su0\0"
  "llvm.arm.neon.sha256su1\0"
  "llvm.arm.neon.smmla\0"
  "llvm.arm.neon.udot\0"
  "llvm.arm.neon.ummla\0"
  "llvm.arm.neon.usdot\0"
  "llvm.arm.neon.usmmla\0"
  "llvm.arm.neon.vabds\0"
  "llvm.arm.neon.vabdu\0"
  "llvm.arm.neon.vabs\0"
  "llvm.arm.neon.vacge\0"
  "llvm.arm.neon.vacgt\0"
  "llvm.arm.neon.vbsl\0"
  "llvm.arm.neon.vcadd.rot270\0"
  "llvm.arm.neon.vcadd.rot90\0"
  "llvm.arm.neon.vcls\0"
  "llvm.arm.neon.vcvtas\0"
  "llvm.arm.neon.vcvtau\0"
  "llvm.arm.neon.vcvtbfp2bf\0"
  "llvm.arm.neon.vcvtfp2bf\0"
  "llvm.arm.neon.vcvtfp2fxs\0"
  "llvm.arm.neon.vcvtfp2fxu\0"
  "llvm.arm.neon.vcvtfp2hf\0"
  "llvm.arm.neon.vcvtfxs2fp\0"
  "llvm.arm.neon.vcvtfxu2fp\0"
  "llvm.arm.neon.vcvthf2fp\0"
  "llvm.arm.neon.vcvtms\0"
  "llvm.arm.neon.vcvtmu\0"
  "llvm.arm.neon.vcvtns\0"
  "llvm.arm.neon.vcvtnu\0"
  "llvm.arm.neon.vcvtps\0"
  "llvm.arm.neon.vcvtpu\0"
  "llvm.arm.neon.vhadds\0"
  "llvm.arm.neon.vhaddu\0"
  "llvm.arm.neon.vhsubs\0"
  "llvm.arm.neon.vhsubu\0"
  "llvm.arm.neon.vld1\0"
  "llvm.arm.neon.vld1x2\0"
  "llvm.arm.neon.vld1x3\0"
  "llvm.arm.neon.vld1x4\0"
  "llvm.arm.neon.vld2\0"
  "llvm.arm.neon.vld2dup\0"
  "llvm.arm.neon.vld2lane\0"
  "llvm.arm.neon.vld3\0"
  "llvm.arm.neon.vld3dup\0"
  "llvm.arm.neon.vld3lane\0"
  "llvm.arm.neon.vld4\0"
  "llvm.arm.neon.vld4dup\0"
  "llvm.arm.neon.vld4lane\0"
  "llvm.arm.neon.vmaxnm\0"
  "llvm.arm.neon.vmaxs\0"
  "llvm.arm.neon.vmaxu\0"
  "llvm.arm.neon.vminnm\0"
  "llvm.arm.neon.vmins\0"
  "llvm.arm.neon.vminu\0"
  "llvm.arm.neon.vmullp\0"
  "llvm.arm.neon.vmulls\0"
  "llvm.arm.neon.vmullu\0"
  "llvm.arm.neon.vmulp\0"
  "llvm.arm.neon.vpadals\0"
  "llvm.arm.neon.vpadalu\0"
  "llvm.arm.neon.vpadd\0"
  "llvm.arm.neon.vpaddls\0"
  "llvm.arm.neon.vpaddlu\0"
  "llvm.arm.neon.vpmaxs\0"
  "llvm.arm.neon.vpmaxu\0"
  "llvm.arm.neon.vpmins\0"
  "llvm.arm.neon.vpminu\0"
  "llvm.arm.neon.vqabs\0"
  "llvm.arm.neon.vqdmulh\0"
  "llvm.arm.neon.vqdmull\0"
  "llvm.arm.neon.vqmovns\0"
  "llvm.arm.neon.vqmovnsu\0"
  "llvm.arm.neon.vqmovnu\0"
  "llvm.arm.neon.vqneg\0"
  "llvm.arm.neon.vqrdmlah\0"
  "llvm.arm.neon.vqrdmlsh\0"
  "llvm.arm.neon.vqrdmulh\0"
  "llvm.arm.neon.vqrshiftns\0"
  "llvm.arm.neon.vqrshiftnsu\0"
  "llvm.arm.neon.vqrshiftnu\0"
  "llvm.arm.neon.vqrshifts\0"
  "llvm.arm.neon.vqrshiftu\0"
  "llvm.arm.neon.vqshiftns\0"
  "llvm.arm.neon.vqshiftnsu\0"
  "llvm.arm.neon.vqshiftnu\0"
  "llvm.arm.neon.vqshifts\0"
  "llvm.arm.neon.vqshiftsu\0"
  "llvm.arm.neon.vqshiftu\0"
  "llvm.arm.neon.vraddhn\0"
  "llvm.arm.neon.vrecpe\0"
  "llvm.arm.neon.vrecps\0"
  "llvm.arm.neon.vrhadds\0"
  "llvm.arm.neon.vrhaddu\0"
  "llvm.arm.neon.vrshiftn\0"
  "llvm.arm.neon.vrshifts\0"
  "llvm.arm.neon.vrshiftu\0"
  "llvm.arm.neon.vrsqrte\0"
  "llvm.arm.neon.vrsqrts\0"
  "llvm.arm.neon.vrsubhn\0"
  "llvm.arm.neon.vshiftins\0"
  "llvm.arm.neon.vshifts\0"
  "llvm.arm.neon.vshiftu\0"
  "llvm.arm.neon.vst1\0"
  "llvm.arm.neon.vst1x2\0"
  "llvm.arm.neon.vst1x3\0"
  "llvm.arm.neon.vst1x4\0"
  "llvm.arm.neon.vst2\0"
  "llvm.arm.neon.vst2lane\0"
  "llvm.arm.neon.vst3\0"
  "llvm.arm.neon.vst3lane\0"
  "llvm.arm.neon.vst4\0"
  "llvm.arm.neon.vst4lane\0"
  "llvm.arm.neon.vtbl1\0"
  "llvm.arm.neon.vtbl2\0"
  "llvm.arm.neon.vtbl3\0"
  "llvm.arm.neon.vtbl4\0"
  "llvm.arm.neon.vtbx1\0"
  "llvm.arm.neon.vtbx2\0"
  "llvm.arm.neon.vtbx3\0"
  "llvm.arm.neon.vtbx4\0"
  "llvm.arm.qadd\0"
  "llvm.arm.qadd16\0"
  "llvm.arm.qadd8\0"
  "llvm.arm.qasx\0"
  "llvm.arm.qsax\0"
  "llvm.arm.qsub\0"
  "llvm.arm.qsub16\0"
  "llvm.arm.qsub8\0"
  "llvm.arm.sadd16\0"
  "llvm.arm.sadd8\0"
  "llvm.arm.sasx\0"
  "llvm.arm.sel\0"
  "llvm.arm.set.fpscr\0"
  "llvm.arm.shadd16\0"
  "llvm.arm.shadd8\0"
  "llvm.arm.shasx\0"
  "llvm.arm.shsax\0"
  "llvm.arm.shsub16\0"
  "llvm.arm.shsub8\0"
  "llvm.arm.smlabb\0"
  "llvm.arm.smlabt\0"
  "llvm.arm.smlad\0"
  "llvm.arm.smladx\0"
  "llvm.arm.smlald\0"
  "llvm.arm.smlaldx\0"
  "llvm.arm.smlatb\0"
  "llvm.arm.smlatt\0"
  "llvm.arm.smlawb\0"
  "llvm.arm.smlawt\0"
  "llvm.arm.smlsd\0"
  "llvm.arm.smlsdx\0"
  "llvm.arm.smlsld\0"
  "llvm.arm.smlsldx\0"
  "llvm.arm.smuad\0"
  "llvm.arm.smuadx\0"
  "llvm.arm.smulbb\0"
  "llvm.arm.smulbt\0"
  "llvm.arm.smultb\0"
  "llvm.arm.smultt\0"
  "llvm.arm.smulwb\0"
  "llvm.arm.smulwt\0"
  "llvm.arm.smusd\0"
  "llvm.arm.smusdx\0"
  "llvm.arm.space\0"
  "llvm.arm.ssat\0"
  "llvm.arm.ssat16\0"
  "llvm.arm.ssax\0"
  "llvm.arm.ssub16\0"
  "llvm.arm.ssub8\0"
  "llvm.arm.stc\0"
  "llvm.arm.stc2\0"
  "llvm.arm.stc2l\0"
  "llvm.arm.stcl\0"
  "llvm.arm.stlex\0"
  "llvm.arm.stlexd\0"
  "llvm.arm.strex\0"
  "llvm.arm.strexd\0"
  "llvm.arm.sxtab16\0"
  "llvm.arm.sxtb16\0"
  "llvm.arm.uadd16\0"
  "llvm.arm.uadd8\0"
  "llvm.arm.uasx\0"
  "llvm.arm.uhadd16\0"
  "llvm.arm.uhadd8\0"
  "llvm.arm.uhasx\0"
  "llvm.arm.uhsax\0"
  "llvm.arm.uhsub16\0"
  "llvm.arm.uhsub8\0"
  "llvm.arm.undefined\0"
  "llvm.arm.uqadd16\0"
  "llvm.arm.uqadd8\0"
  "llvm.arm.uqasx\0"
  "llvm.arm.uqsax\0"
  "llvm.arm.uqsub16\0"
  "llvm.arm.uqsub8\0"
  "llvm.arm.usad8\0"
  "llvm.arm.usada8\0"
  "llvm.arm.usat\0"
  "llvm.arm.usat16\0"
  "llvm.arm.usax\0"
  "llvm.arm.usub16\0"
  "llvm.arm.usub8\0"
  "llvm.arm.uxtab16\0"
  "llvm.arm.uxtb16\0"
  "llvm.arm.vcvtr\0"
  "llvm.arm.vcvtru\0"
  "llvm.bpf.btf.type.id\0"
  "llvm.bpf.compare\0"
  "llvm.bpf.getelementptr.and.load\0"
  "llvm.bpf.getelementptr.and.store\0"
  "llvm.bpf.load.byte\0"
  "llvm.bpf.load.half\0"
  "llvm.bpf.load.word\0"
  "llvm.bpf.passthrough\0"
  "llvm.bpf.preserve.enum.value\0"
  "llvm.bpf.preserve.field.info\0"
  "llvm.bpf.preserve.type.info\0"
  "llvm.bpf.pseudo\0"
  "llvm.dx.all\0"
  "llvm.dx.any\0"
  "llvm.dx.asdouble\0"
  "llvm.dx.cross\0"
  "llvm.dx.ddx.coarse\0"
  "llvm.dx.ddx.fine\0"
  "llvm.dx.ddy.coarse\0"
  "llvm.dx.ddy.fine\0"
  "llvm.dx.degrees\0"
  "llvm.dx.discard\0"
  "llvm.dx.dot2\0"
  "llvm.dx.dot2add\0"
  "llvm.dx.dot3\0"
  "llvm.dx.dot4\0"
  "llvm.dx.dot4add.i8packed\0"
  "llvm.dx.dot4add.u8packed\0"
  "llvm.dx.fdot\0"
  "llvm.dx.firstbitlow\0"
  "llvm.dx.firstbitshigh\0"
  "llvm.dx.firstbituhigh\0"
  "llvm.dx.flattened.thread.id.in.group\0"
  "llvm.dx.frac\0"
  "llvm.dx.group.id\0"
  "llvm.dx.group.memory.barrier.with.group.sync\0"
  "llvm.dx.imad\0"
  "llvm.dx.isinf\0"
  "llvm.dx.isnan\0"
  "llvm.dx.legacyf16tof32\0"
  "llvm.dx.lerp\0"
  "llvm.dx.load.input\0"
  "llvm.dx.nclamp\0"
  "llvm.dx.normalize\0"
  "llvm.dx.radians\0"
  "llvm.dx.resource.casthandle\0"
  "llvm.dx.resource.getdimensions.x\0"
  "llvm.dx.resource.getpointer\0"
  "llvm.dx.resource.handlefrombinding\0"
  "llvm.dx.resource.handlefromimplicitbinding\0"
  "llvm.dx.resource.load.cbufferrow.2\0"
  "llvm.dx.resource.load.cbufferrow.4\0"
  "llvm.dx.resource.load.cbufferrow.8\0"
  "llvm.dx.resource.load.rawbuffer\0"
  "llvm.dx.resource.load.typedbuffer\0"
  "llvm.dx.resource.nonuniformindex\0"
  "llvm.dx.resource.store.rawbuffer\0"
  "llvm.dx.resource.store.typedbuffer\0"
  "llvm.dx.resource.updatecounter\0"
  "llvm.dx.rsqrt\0"
  "llvm.dx.saturate\0"
  "llvm.dx.sclamp\0"
  "llvm.dx.sdot\0"
  "llvm.dx.sign\0"
  "llvm.dx.splitdouble\0"
  "llvm.dx.step\0"
  "llvm.dx.store.output\0"
  "llvm.dx.thread.id\0"
  "llvm.dx.thread.id.in.group\0"
  "llvm.dx.uclamp\0"
  "llvm.dx.udot\0"
  "llvm.dx.umad\0"
  "llvm.dx.wave.active.countbits\0"
  "llvm.dx.wave.all\0"
  "llvm.dx.wave.any\0"
  "llvm.dx.wave.ballot\0"
  "llvm.dx.wave.get.lane.count\0"
  "llvm.dx.wave.getlaneindex\0"
  "llvm.dx.wave.is.first.lane\0"
  "llvm.dx.wave.readlane\0"
  "llvm.dx.wave.reduce.max\0"
  "llvm.dx.wave.reduce.min\0"
  "llvm.dx.wave.reduce.sum\0"
  "llvm.dx.wave.reduce.umax\0"
  "llvm.dx.wave.reduce.umin\0"
  "llvm.dx.wave.reduce.usum\0"
  "llvm.hexagon.A2.abs\0"
  "llvm.hexagon.A2.absp\0"
  "llvm.hexagon.A2.abssat\0"
  "llvm.hexagon.A2.add\0"
  "llvm.hexagon.A2.addh.h16.hh\0"
  "llvm.hexagon.A2.addh.h16.hl\0"
  "llvm.hexagon.A2.addh.h16.lh\0"
  "llvm.hexagon.A2.addh.h16.ll\0"
  "llvm.hexagon.A2.addh.h16.sat.hh\0"
  "llvm.hexagon.A2.addh.h16.sat.hl\0"
  "llvm.hexagon.A2.addh.h16.sat.lh\0"
  "llvm.hexagon.A2.addh.h16.sat.ll\0"
  "llvm.hexagon.A2.addh.l16.hl\0"
  "llvm.hexagon.A2.addh.l16.ll\0"
  "llvm.hexagon.A2.addh.l16.sat.hl\0"
  "llvm.hexagon.A2.addh.l16.sat.ll\0"
  "llvm.hexagon.A2.addi\0"
  "llvm.hexagon.A2.addp\0"
  "llvm.hexagon.A2.addpsat\0"
  "llvm.hexagon.A2.addsat\0"
  "llvm.hexagon.A2.addsp\0"
  "llvm.hexagon.A2.and\0"
  "llvm.hexagon.A2.andir\0"
  "llvm.hexagon.A2.andp\0"
  "llvm.hexagon.A2.aslh\0"
  "llvm.hexagon.A2.asrh\0"
  "llvm.hexagon.A2.combine.hh\0"
  "llvm.hexagon.A2.combine.hl\0"
  "llvm.hexagon.A2.combine.lh\0"
  "llvm.hexagon.A2.combine.ll\0"
  "llvm.hexagon.A2.combineii\0"
  "llvm.hexagon.A2.combinew\0"
  "llvm.hexagon.A2.max\0"
  "llvm.hexagon.A2.maxp\0"
  "llvm.hexagon.A2.maxu\0"
  "llvm.hexagon.A2.maxup\0"
  "llvm.hexagon.A2.min\0"
  "llvm.hexagon.A2.minp\0"
  "llvm.hexagon.A2.minu\0"
  "llvm.hexagon.A2.minup\0"
  "llvm.hexagon.A2.neg\0"
  "llvm.hexagon.A2.negp\0"
  "llvm.hexagon.A2.negsat\0"
  "llvm.hexagon.A2.not\0"
  "llvm.hexagon.A2.notp\0"
  "llvm.hexagon.A2.or\0"
  "llvm.hexagon.A2.orir\0"
  "llvm.hexagon.A2.orp\0"
  "llvm.hexagon.A2.roundsat\0"
  "llvm.hexagon.A2.sat\0"
  "llvm.hexagon.A2.satb\0"
  "llvm.hexagon.A2.sath\0"
  "llvm.hexagon.A2.satub\0"
  "llvm.hexagon.A2.satuh\0"
  "llvm.hexagon.A2.sub\0"
  "llvm.hexagon.A2.subh.h16.hh\0"
  "llvm.hexagon.A2.subh.h16.hl\0"
  "llvm.hexagon.A2.subh.h16.lh\0"
  "llvm.hexagon.A2.subh.h16.ll\0"
  "llvm.hexagon.A2.subh.h16.sat.hh\0"
  "llvm.hexagon.A2.subh.h16.sat.hl\0"
  "llvm.hexagon.A2.subh.h16.sat.lh\0"
  "llvm.hexagon.A2.subh.h16.sat.ll\0"
  "llvm.hexagon.A2.subh.l16.hl\0"
  "llvm.hexagon.A2.subh.l16.ll\0"
  "llvm.hexagon.A2.subh.l16.sat.hl\0"
  "llvm.hexagon.A2.subh.l16.sat.ll\0"
  "llvm.hexagon.A2.subp\0"
  "llvm.hexagon.A2.subri\0"
  "llvm.hexagon.A2.subsat\0"
  "llvm.hexagon.A2.svaddh\0"
  "llvm.hexagon.A2.svaddhs\0"
  "llvm.hexagon.A2.svadduhs\0"
  "llvm.hexagon.A2.svavgh\0"
  "llvm.hexagon.A2.svavghs\0"
  "llvm.hexagon.A2.svnavgh\0"
  "llvm.hexagon.A2.svsubh\0"
  "llvm.hexagon.A2.svsubhs\0"
  "llvm.hexagon.A2.svsubuhs\0"
  "llvm.hexagon.A2.swiz\0"
  "llvm.hexagon.A2.sxtb\0"
  "llvm.hexagon.A2.sxth\0"
  "llvm.hexagon.A2.sxtw\0"
  "llvm.hexagon.A2.tfr\0"
  "llvm.hexagon.A2.tfrih\0"
  "llvm.hexagon.A2.tfril\0"
  "llvm.hexagon.A2.tfrp\0"
  "llvm.hexagon.A2.tfrpi\0"
  "llvm.hexagon.A2.tfrsi\0"
  "llvm.hexagon.A2.vabsh\0"
  "llvm.hexagon.A2.vabshsat\0"
  "llvm.hexagon.A2.vabsw\0"
  "llvm.hexagon.A2.vabswsat\0"
  "llvm.hexagon.A2.vaddb.map\0"
  "llvm.hexagon.A2.vaddh\0"
  "llvm.hexagon.A2.vaddhs\0"
  "llvm.hexagon.A2.vaddub\0"
  "llvm.hexagon.A2.vaddubs\0"
  "llvm.hexagon.A2.vadduhs\0"
  "llvm.hexagon.A2.vaddw\0"
  "llvm.hexagon.A2.vaddws\0"
  "llvm.hexagon.A2.vavgh\0"
  "llvm.hexagon.A2.vavghcr\0"
  "llvm.hexagon.A2.vavghr\0"
  "llvm.hexagon.A2.vavgub\0"
  "llvm.hexagon.A2.vavgubr\0"
  "llvm.hexagon.A2.vavguh\0"
  "llvm.hexagon.A2.vavguhr\0"
  "llvm.hexagon.A2.vavguw\0"
  "llvm.hexagon.A2.vavguwr\0"
  "llvm.hexagon.A2.vavgw\0"
  "llvm.hexagon.A2.vavgwcr\0"
  "llvm.hexagon.A2.vavgwr\0"
  "llvm.hexagon.A2.vcmpbeq\0"
  "llvm.hexagon.A2.vcmpbgtu\0"
  "llvm.hexagon.A2.vcmpheq\0"
  "llvm.hexagon.A2.vcmphgt\0"
  "llvm.hexagon.A2.vcmphgtu\0"
  "llvm.hexagon.A2.vcmpweq\0"
  "llvm.hexagon.A2.vcmpwgt\0"
  "llvm.hexagon.A2.vcmpwgtu\0"
  "llvm.hexagon.A2.vconj\0"
  "llvm.hexagon.A2.vmaxb\0"
  "llvm.hexagon.A2.vmaxh\0"
  "llvm.hexagon.A2.vmaxub\0"
  "llvm.hexagon.A2.vmaxuh\0"
  "llvm.hexagon.A2.vmaxuw\0"
  "llvm.hexagon.A2.vmaxw\0"
  "llvm.hexagon.A2.vminb\0"
  "llvm.hexagon.A2.vminh\0"
  "llvm.hexagon.A2.vminub\0"
  "llvm.hexagon.A2.vminuh\0"
  "llvm.hexagon.A2.vminuw\0"
  "llvm.hexagon.A2.vminw\0"
  "llvm.hexagon.A2.vnavgh\0"
  "llvm.hexagon.A2.vnavghcr\0"
  "llvm.hexagon.A2.vnavghr\0"
  "llvm.hexagon.A2.vnavgw\0"
  "llvm.hexagon.A2.vnavgwcr\0"
  "llvm.hexagon.A2.vnavgwr\0"
  "llvm.hexagon.A2.vraddub\0"
  "llvm.hexagon.A2.vraddub.acc\0"
  "llvm.hexagon.A2.vrsadub\0"
  "llvm.hexagon.A2.vrsadub.acc\0"
  "llvm.hexagon.A2.vsubb.map\0"
  "llvm.hexagon.A2.vsubh\0"
  "llvm.hexagon.A2.vsubhs\0"
  "llvm.hexagon.A2.vsubub\0"
  "llvm.hexagon.A2.vsububs\0"
  "llvm.hexagon.A2.vsubuhs\0"
  "llvm.hexagon.A2.vsubw\0"
  "llvm.hexagon.A2.vsubws\0"
  "llvm.hexagon.A2.xor\0"
  "llvm.hexagon.A2.xorp\0"
  "llvm.hexagon.A2.zxtb\0"
  "llvm.hexagon.A2.zxth\0"
  "llvm.hexagon.A4.andn\0"
  "llvm.hexagon.A4.andnp\0"
  "llvm.hexagon.A4.bitsplit\0"
  "llvm.hexagon.A4.bitspliti\0"
  "llvm.hexagon.A4.boundscheck\0"
  "llvm.hexagon.A4.cmpbeq\0"
  "llvm.hexagon.A4.cmpbeqi\0"
  "llvm.hexagon.A4.cmpbgt\0"
  "llvm.hexagon.A4.cmpbgti\0"
  "llvm.hexagon.A4.cmpbgtu\0"
  "llvm.hexagon.A4.cmpbgtui\0"
  "llvm.hexagon.A4.cmpheq\0"
  "llvm.hexagon.A4.cmpheqi\0"
  "llvm.hexagon.A4.cmphgt\0"
  "llvm.hexagon.A4.cmphgti\0"
  "llvm.hexagon.A4.cmphgtu\0"
  "llvm.hexagon.A4.cmphgtui\0"
  "llvm.hexagon.A4.combineir\0"
  "llvm.hexagon.A4.combineri\0"
  "llvm.hexagon.A4.cround.ri\0"
  "llvm.hexagon.A4.cround.rr\0"
  "llvm.hexagon.A4.modwrapu\0"
  "llvm.hexagon.A4.orn\0"
  "llvm.hexagon.A4.ornp\0"
  "llvm.hexagon.A4.rcmpeq\0"
  "llvm.hexagon.A4.rcmpeqi\0"
  "llvm.hexagon.A4.rcmpneq\0"
  "llvm.hexagon.A4.rcmpneqi\0"
  "llvm.hexagon.A4.round.ri\0"
  "llvm.hexagon.A4.round.ri.sat\0"
  "llvm.hexagon.A4.round.rr\0"
  "llvm.hexagon.A4.round.rr.sat\0"
  "llvm.hexagon.A4.tlbmatch\0"
  "llvm.hexagon.A4.vcmpbeq.any\0"
  "llvm.hexagon.A4.vcmpbeqi\0"
  "llvm.hexagon.A4.vcmpbgt\0"
  "llvm.hexagon.A4.vcmpbgti\0"
  "llvm.hexagon.A4.vcmpbgtui\0"
  "llvm.hexagon.A4.vcmpheqi\0"
  "llvm.hexagon.A4.vcmphgti\0"
  "llvm.hexagon.A4.vcmphgtui\0"
  "llvm.hexagon.A4.vcmpweqi\0"
  "llvm.hexagon.A4.vcmpwgti\0"
  "llvm.hexagon.A4.vcmpwgtui\0"
  "llvm.hexagon.A4.vrmaxh\0"
  "llvm.hexagon.A4.vrmaxuh\0"
  "llvm.hexagon.A4.vrmaxuw\0"
  "llvm.hexagon.A4.vrmaxw\0"
  "llvm.hexagon.A4.vrminh\0"
  "llvm.hexagon.A4.vrminuh\0"
  "llvm.hexagon.A4.vrminuw\0"
  "llvm.hexagon.A4.vrminw\0"
  "llvm.hexagon.A5.vaddhubs\0"
  "llvm.hexagon.A6.vcmpbeq.notany\0"
  "llvm.hexagon.A7.clip\0"
  "llvm.hexagon.A7.croundd.ri\0"
  "llvm.hexagon.A7.croundd.rr\0"
  "llvm.hexagon.A7.vclip\0"
  "llvm.hexagon.C2.all8\0"
  "llvm.hexagon.C2.and\0"
  "llvm.hexagon.C2.andn\0"
  "llvm.hexagon.C2.any8\0"
  "llvm.hexagon.C2.bitsclr\0"
  "llvm.hexagon.C2.bitsclri\0"
  "llvm.hexagon.C2.bitsset\0"
  "llvm.hexagon.C2.cmpeq\0"
  "llvm.hexagon.C2.cmpeqi\0"
  "llvm.hexagon.C2.cmpeqp\0"
  "llvm.hexagon.C2.cmpgei\0"
  "llvm.hexagon.C2.cmpgeui\0"
  "llvm.hexagon.C2.cmpgt\0"
  "llvm.hexagon.C2.cmpgti\0"
  "llvm.hexagon.C2.cmpgtp\0"
  "llvm.hexagon.C2.cmpgtu\0"
  "llvm.hexagon.C2.cmpgtui\0"
  "llvm.hexagon.C2.cmpgtup\0"
  "llvm.hexagon.C2.cmplt\0"
  "llvm.hexagon.C2.cmpltu\0"
  "llvm.hexagon.C2.mask\0"
  "llvm.hexagon.C2.mux\0"
  "llvm.hexagon.C2.muxii\0"
  "llvm.hexagon.C2.muxir\0"
  "llvm.hexagon.C2.muxri\0"
  "llvm.hexagon.C2.not\0"
  "llvm.hexagon.C2.or\0"
  "llvm.hexagon.C2.orn\0"
  "llvm.hexagon.C2.pxfer.map\0"
  "llvm.hexagon.C2.tfrpr\0"
  "llvm.hexagon.C2.tfrrp\0"
  "llvm.hexagon.C2.vitpack\0"
  "llvm.hexagon.C2.vmux\0"
  "llvm.hexagon.C2.xor\0"
  "llvm.hexagon.C4.and.and\0"
  "llvm.hexagon.C4.and.andn\0"
  "llvm.hexagon.C4.and.or\0"
  "llvm.hexagon.C4.and.orn\0"
  "llvm.hexagon.C4.cmplte\0"
  "llvm.hexagon.C4.cmpltei\0"
  "llvm.hexagon.C4.cmplteu\0"
  "llvm.hexagon.C4.cmplteui\0"
  "llvm.hexagon.C4.cmpneq\0"
  "llvm.hexagon.C4.cmpneqi\0"
  "llvm.hexagon.C4.fastcorner9\0"
  "llvm.hexagon.C4.fastcorner9.not\0"
  "llvm.hexagon.C4.nbitsclr\0"
  "llvm.hexagon.C4.nbitsclri\0"
  "llvm.hexagon.C4.nbitsset\0"
  "llvm.hexagon.C4.or.and\0"
  "llvm.hexagon.C4.or.andn\0"
  "llvm.hexagon.C4.or.or\0"
  "llvm.hexagon.C4.or.orn\0"
  "llvm.hexagon.F2.conv.d2df\0"
  "llvm.hexagon.F2.conv.d2sf\0"
  "llvm.hexagon.F2.conv.df2d\0"
  "llvm.hexagon.F2.conv.df2d.chop\0"
  "llvm.hexagon.F2.conv.df2sf\0"
  "llvm.hexagon.F2.conv.df2ud\0"
  "llvm.hexagon.F2.conv.df2ud.chop\0"
  "llvm.hexagon.F2.conv.df2uw\0"
  "llvm.hexagon.F2.conv.df2uw.chop\0"
  "llvm.hexagon.F2.conv.df2w\0"
  "llvm.hexagon.F2.conv.df2w.chop\0"
  "llvm.hexagon.F2.conv.sf2d\0"
  "llvm.hexagon.F2.conv.sf2d.chop\0"
  "llvm.hexagon.F2.conv.sf2df\0"
  "llvm.hexagon.F2.conv.sf2ud\0"
  "llvm.hexagon.F2.conv.sf2ud.chop\0"
  "llvm.hexagon.F2.conv.sf2uw\0"
  "llvm.hexagon.F2.conv.sf2uw.chop\0"
  "llvm.hexagon.F2.conv.sf2w\0"
  "llvm.hexagon.F2.conv.sf2w.chop\0"
  "llvm.hexagon.F2.conv.ud2df\0"
  "llvm.hexagon.F2.conv.ud2sf\0"
  "llvm.hexagon.F2.conv.uw2df\0"
  "llvm.hexagon.F2.conv.uw2sf\0"
  "llvm.hexagon.F2.conv.w2df\0"
  "llvm.hexagon.F2.conv.w2sf\0"
  "llvm.hexagon.F2.dfadd\0"
  "llvm.hexagon.F2.dfclass\0"
  "llvm.hexagon.F2.dfcmpeq\0"
  "llvm.hexagon.F2.dfcmpge\0"
  "llvm.hexagon.F2.dfcmpgt\0"
  "llvm.hexagon.F2.dfcmpuo\0"
  "llvm.hexagon.F2.dfimm.n\0"
  "llvm.hexagon.F2.dfimm.p\0"
  "llvm.hexagon.F2.dfmax\0"
  "llvm.hexagon.F2.dfmin\0"
  "llvm.hexagon.F2.dfmpyfix\0"
  "llvm.hexagon.F2.dfmpyhh\0"
  "llvm.hexagon.F2.dfmpylh\0"
  "llvm.hexagon.F2.dfmpyll\0"
  "llvm.hexagon.F2.dfsub\0"
  "llvm.hexagon.F2.sfadd\0"
  "llvm.hexagon.F2.sfclass\0"
  "llvm.hexagon.F2.sfcmpeq\0"
  "llvm.hexagon.F2.sfcmpge\0"
  "llvm.hexagon.F2.sfcmpgt\0"
  "llvm.hexagon.F2.sfcmpuo\0"
  "llvm.hexagon.F2.sffixupd\0"
  "llvm.hexagon.F2.sffixupn\0"
  "llvm.hexagon.F2.sffixupr\0"
  "llvm.hexagon.F2.sffma\0"
  "llvm.hexagon.F2.sffma.lib\0"
  "llvm.hexagon.F2.sffma.sc\0"
  "llvm.hexagon.F2.sffms\0"
  "llvm.hexagon.F2.sffms.lib\0"
  "llvm.hexagon.F2.sfimm.n\0"
  "llvm.hexagon.F2.sfimm.p\0"
  "llvm.hexagon.F2.sfmax\0"
  "llvm.hexagon.F2.sfmin\0"
  "llvm.hexagon.F2.sfmpy\0"
  "llvm.hexagon.F2.sfsub\0"
  "llvm.hexagon.L2.loadrb.pbr\0"
  "llvm.hexagon.L2.loadrb.pci\0"
  "llvm.hexagon.L2.loadrb.pcr\0"
  "llvm.hexagon.L2.loadrd.pbr\0"
  "llvm.hexagon.L2.loadrd.pci\0"
  "llvm.hexagon.L2.loadrd.pcr\0"
  "llvm.hexagon.L2.loadrh.pbr\0"
  "llvm.hexagon.L2.loadrh.pci\0"
  "llvm.hexagon.L2.loadrh.pcr\0"
  "llvm.hexagon.L2.loadri.pbr\0"
  "llvm.hexagon.L2.loadri.pci\0"
  "llvm.hexagon.L2.loadri.pcr\0"
  "llvm.hexagon.L2.loadrub.pbr\0"
  "llvm.hexagon.L2.loadrub.pci\0"
  "llvm.hexagon.L2.loadrub.pcr\0"
  "llvm.hexagon.L2.loadruh.pbr\0"
  "llvm.hexagon.L2.loadruh.pci\0"
  "llvm.hexagon.L2.loadruh.pcr\0"
  "llvm.hexagon.L2.loadw.locked\0"
  "llvm.hexagon.L4.loadd.locked\0"
  "llvm.hexagon.M2.acci\0"
  "llvm.hexagon.M2.accii\0"
  "llvm.hexagon.M2.cmaci.s0\0"
  "llvm.hexagon.M2.cmacr.s0\0"
  "llvm.hexagon.M2.cmacs.s0\0"
  "llvm.hexagon.M2.cmacs.s1\0"
  "llvm.hexagon.M2.cmacsc.s0\0"
  "llvm.hexagon.M2.cmacsc.s1\0"
  "llvm.hexagon.M2.cmpyi.s0\0"
  "llvm.hexagon.M2.cmpyr.s0\0"
  "llvm.hexagon.M2.cmpyrs.s0\0"
  "llvm.hexagon.M2.cmpyrs.s1\0"
  "llvm.hexagon.M2.cmpyrsc.s0\0"
  "llvm.hexagon.M2.cmpyrsc.s1\0"
  "llvm.hexagon.M2.cmpys.s0\0"
  "llvm.hexagon.M2.cmpys.s1\0"
  "llvm.hexagon.M2.cmpysc.s0\0"
  "llvm.hexagon.M2.cmpysc.s1\0"
  "llvm.hexagon.M2.cnacs.s0\0"
  "llvm.hexagon.M2.cnacs.s1\0"
  "llvm.hexagon.M2.cnacsc.s0\0"
  "llvm.hexagon.M2.cnacsc.s1\0"
  "llvm.hexagon.M2.dpmpyss.acc.s0\0"
  "llvm.hexagon.M2.dpmpyss.nac.s0\0"
  "llvm.hexagon.M2.dpmpyss.rnd.s0\0"
  "llvm.hexagon.M2.dpmpyss.s0\0"
  "llvm.hexagon.M2.dpmpyuu.acc.s0\0"
  "llvm.hexagon.M2.dpmpyuu.nac.s0\0"
  "llvm.hexagon.M2.dpmpyuu.s0\0"
  "llvm.hexagon.M2.hmmpyh.rs1\0"
  "llvm.hexagon.M2.hmmpyh.s1\0"
  "llvm.hexagon.M2.hmmpyl.rs1\0"
  "llvm.hexagon.M2.hmmpyl.s1\0"
  "llvm.hexagon.M2.maci\0"
  "llvm.hexagon.M2.macsin\0"
  "llvm.hexagon.M2.macsip\0"
  "llvm.hexagon.M2.mmachs.rs0\0"
  "llvm.hexagon.M2.mmachs.rs1\0"
  "llvm.hexagon.M2.mmachs.s0\0"
  "llvm.hexagon.M2.mmachs.s1\0"
  "llvm.hexagon.M2.mmacls.rs0\0"
  "llvm.hexagon.M2.mmacls.rs1\0"
  "llvm.hexagon.M2.mmacls.s0\0"
  "llvm.hexagon.M2.mmacls.s1\0"
  "llvm.hexagon.M2.mmacuhs.rs0\0"
  "llvm.hexagon.M2.mmacuhs.rs1\0"
  "llvm.hexagon.M2.mmacuhs.s0\0"
  "llvm.hexagon.M2.mmacuhs.s1\0"
  "llvm.hexagon.M2.mmaculs.rs0\0"
  "llvm.hexagon.M2.mmaculs.rs1\0"
  "llvm.hexagon.M2.mmaculs.s0\0"
  "llvm.hexagon.M2.mmaculs.s1\0"
  "llvm.hexagon.M2.mmpyh.rs0\0"
  "llvm.hexagon.M2.mmpyh.rs1\0"
  "llvm.hexagon.M2.mmpyh.s0\0"
  "llvm.hexagon.M2.mmpyh.s1\0"
  "llvm.hexagon.M2.mmpyl.rs0\0"
  "llvm.hexagon.M2.mmpyl.rs1\0"
  "llvm.hexagon.M2.mmpyl.s0\0"
  "llvm.hexagon.M2.mmpyl.s1\0"
  "llvm.hexagon.M2.mmpyuh.rs0\0"
  "llvm.hexagon.M2.mmpyuh.rs1\0"
  "llvm.hexagon.M2.mmpyuh.s0\0"
  "llvm.hexagon.M2.mmpyuh.s1\0"
  "llvm.hexagon.M2.mmpyul.rs0\0"
  "llvm.hexagon.M2.mmpyul.rs1\0"
  "llvm.hexagon.M2.mmpyul.s0\0"
  "llvm.hexagon.M2.mmpyul.s1\0"
  "llvm.hexagon.M2.mnaci\0"
  "llvm.hexagon.M2.mpy.acc.hh.s0\0"
  "llvm.hexagon.M2.mpy.acc.hh.s1\0"
  "llvm.hexagon.M2.mpy.acc.hl.s0\0"
  "llvm.hexagon.M2.mpy.acc.hl.s1\0"
  "llvm.hexagon.M2.mpy.acc.lh.s0\0"
  "llvm.hexagon.M2.mpy.acc.lh.s1\0"
  "llvm.hexagon.M2.mpy.acc.ll.s0\0"
  "llvm.hexagon.M2.mpy.acc.ll.s1\0"
  "llvm.hexagon.M2.mpy.acc.sat.hh.s0\0"
  "llvm.hexagon.M2.mpy.acc.sat.hh.s1\0"
  "llvm.hexagon.M2.mpy.acc.sat.hl.s0\0"
  "llvm.hexagon.M2.mpy.acc.sat.hl.s1\0"
  "llvm.hexagon.M2.mpy.acc.sat.lh.s0\0"
  "llvm.hexagon.M2.mpy.acc.sat.lh.s1\0"
  "llvm.hexagon.M2.mpy.acc.sat.ll.s0\0"
  "llvm.hexagon.M2.mpy.acc.sat.ll.s1\0"
  "llvm.hexagon.M2.mpy.hh.s0\0"
  "llvm.hexagon.M2.mpy.hh.s1\0"
  "llvm.hexagon.M2.mpy.hl.s0\0"
  "llvm.hexagon.M2.mpy.hl.s1\0"
  "llvm.hexagon.M2.mpy.lh.s0\0"
  "llvm.hexagon.M2.mpy.lh.s1\0"
  "llvm.hexagon.M2.mpy.ll.s0\0"
  "llvm.hexagon.M2.mpy.ll.s1\0"
  "llvm.hexagon.M2.mpy.nac.hh.s0\0"
  "llvm.hexagon.M2.mpy.nac.hh.s1\0"
  "llvm.hexagon.M2.mpy.nac.hl.s0\0"
  "llvm.hexagon.M2.mpy.nac.hl.s1\0"
  "llvm.hexagon.M2.mpy.nac.lh.s0\0"
  "llvm.hexagon.M2.mpy.nac.lh.s1\0"
  "llvm.hexagon.M2.mpy.nac.ll.s0\0"
  "llvm.hexagon.M2.mpy.nac.ll.s1\0"
  "llvm.hexagon.M2.mpy.nac.sat.hh.s0\0"
  "llvm.hexagon.M2.mpy.nac.sat.hh.s1\0"
  "llvm.hexagon.M2.mpy.nac.sat.hl.s0\0"
  "llvm.hexagon.M2.mpy.nac.sat.hl.s1\0"
  "llvm.hexagon.M2.mpy.nac.sat.lh.s0\0"
  "llvm.hexagon.M2.mpy.nac.sat.lh.s1\0"
  "llvm.hexagon.M2.mpy.nac.sat.ll.s0\0"
  "llvm.hexagon.M2.mpy.nac.sat.ll.s1\0"
  "llvm.hexagon.M2.mpy.rnd.hh.s0\0"
  "llvm.hexagon.M2.mpy.rnd.hh.s1\0"
  "llvm.hexagon.M2.mpy.rnd.hl.s0\0"
  "llvm.hexagon.M2.mpy.rnd.hl.s1\0"
  "llvm.hexagon.M2.mpy.rnd.lh.s0\0"
  "llvm.hexagon.M2.mpy.rnd.lh.s1\0"
  "llvm.hexagon.M2.mpy.rnd.ll.s0\0"
  "llvm.hexagon.M2.mpy.rnd.ll.s1\0"
  "llvm.hexagon.M2.mpy.sat.hh.s0\0"
  "llvm.hexagon.M2.mpy.sat.hh.s1\0"
  "llvm.hexagon.M2.mpy.sat.hl.s0\0"
  "llvm.hexagon.M2.mpy.sat.hl.s1\0"
  "llvm.hexagon.M2.mpy.sat.lh.s0\0"
  "llvm.hexagon.M2.mpy.sat.lh.s1\0"
  "llvm.hexagon.M2.mpy.sat.ll.s0\0"
  "llvm.hexagon.M2.mpy.sat.ll.s1\0"
  "llvm.hexagon.M2.mpy.sat.rnd.hh.s0\0"
  "llvm.hexagon.M2.mpy.sat.rnd.hh.s1\0"
  "llvm.hexagon.M2.mpy.sat.rnd.hl.s0\0"
  "llvm.hexagon.M2.mpy.sat.rnd.hl.s1\0"
  "llvm.hexagon.M2.mpy.sat.rnd.lh.s0\0"
  "llvm.hexagon.M2.mpy.sat.rnd.lh.s1\0"
  "llvm.hexagon.M2.mpy.sat.rnd.ll.s0\0"
  "llvm.hexagon.M2.mpy.sat.rnd.ll.s1\0"
  "llvm.hexagon.M2.mpy.up\0"
  "llvm.hexagon.M2.mpy.up.s1\0"
  "llvm.hexagon.M2.mpy.up.s1.sat\0"
  "llvm.hexagon.M2.mpyd.acc.hh.s0\0"
  "llvm.hexagon.M2.mpyd.acc.hh.s1\0"
  "llvm.hexagon.M2.mpyd.acc.hl.s0\0"
  "llvm.hexagon.M2.mpyd.acc.hl.s1\0"
  "llvm.hexagon.M2.mpyd.acc.lh.s0\0"
  "llvm.hexagon.M2.mpyd.acc.lh.s1\0"
  "llvm.hexagon.M2.mpyd.acc.ll.s0\0"
  "llvm.hexagon.M2.mpyd.acc.ll.s1\0"
  "llvm.hexagon.M2.mpyd.hh.s0\0"
  "llvm.hexagon.M2.mpyd.hh.s1\0"
  "llvm.hexagon.M2.mpyd.hl.s0\0"
  "llvm.hexagon.M2.mpyd.hl.s1\0"
  "llvm.hexagon.M2.mpyd.lh.s0\0"
  "llvm.hexagon.M2.mpyd.lh.s1\0"
  "llvm.hexagon.M2.mpyd.ll.s0\0"
  "llvm.hexagon.M2.mpyd.ll.s1\0"
  "llvm.hexagon.M2.mpyd.nac.hh.s0\0"
  "llvm.hexagon.M2.mpyd.nac.hh.s1\0"
  "llvm.hexagon.M2.mpyd.nac.hl.s0\0"
  "llvm.hexagon.M2.mpyd.nac.hl.s1\0"
  "llvm.hexagon.M2.mpyd.nac.lh.s0\0"
  "llvm.hexagon.M2.mpyd.nac.lh.s1\0"
  "llvm.hexagon.M2.mpyd.nac.ll.s0\0"
  "llvm.hexagon.M2.mpyd.nac.ll.s1\0"
  "llvm.hexagon.M2.mpyd.rnd.hh.s0\0"
  "llvm.hexagon.M2.mpyd.rnd.hh.s1\0"
  "llvm.hexagon.M2.mpyd.rnd.hl.s0\0"
  "llvm.hexagon.M2.mpyd.rnd.hl.s1\0"
  "llvm.hexagon.M2.mpyd.rnd.lh.s0\0"
  "llvm.hexagon.M2.mpyd.rnd.lh.s1\0"
  "llvm.hexagon.M2.mpyd.rnd.ll.s0\0"
  "llvm.hexagon.M2.mpyd.rnd.ll.s1\0"
  "llvm.hexagon.M2.mpyi\0"
  "llvm.hexagon.M2.mpysmi\0"
  "llvm.hexagon.M2.mpysu.up\0"
  "llvm.hexagon.M2.mpyu.acc.hh.s0\0"
  "llvm.hexagon.M2.mpyu.acc.hh.s1\0"
  "llvm.hexagon.M2.mpyu.acc.hl.s0\0"
  "llvm.hexagon.M2.mpyu.acc.hl.s1\0"
  "llvm.hexagon.M2.mpyu.acc.lh.s0\0"
  "llvm.hexagon.M2.mpyu.acc.lh.s1\0"
  "llvm.hexagon.M2.mpyu.acc.ll.s0\0"
  "llvm.hexagon.M2.mpyu.acc.ll.s1\0"
  "llvm.hexagon.M2.mpyu.hh.s0\0"
  "llvm.hexagon.M2.mpyu.hh.s1\0"
  "llvm.hexagon.M2.mpyu.hl.s0\0"
  "llvm.hexagon.M2.mpyu.hl.s1\0"
  "llvm.hexagon.M2.mpyu.lh.s0\0"
  "llvm.hexagon.M2.mpyu.lh.s1\0"
  "llvm.hexagon.M2.mpyu.ll.s0\0"
  "llvm.hexagon.M2.mpyu.ll.s1\0"
  "llvm.hexagon.M2.mpyu.nac.hh.s0\0"
  "llvm.hexagon.M2.mpyu.nac.hh.s1\0"
  "llvm.hexagon.M2.mpyu.nac.hl.s0\0"
  "llvm.hexagon.M2.mpyu.nac.hl.s1\0"
  "llvm.hexagon.M2.mpyu.nac.lh.s0\0"
  "llvm.hexagon.M2.mpyu.nac.lh.s1\0"
  "llvm.hexagon.M2.mpyu.nac.ll.s0\0"
  "llvm.hexagon.M2.mpyu.nac.ll.s1\0"
  "llvm.hexagon.M2.mpyu.up\0"
  "llvm.hexagon.M2.mpyud.acc.hh.s0\0"
  "llvm.hexagon.M2.mpyud.acc.hh.s1\0"
  "llvm.hexagon.M2.mpyud.acc.hl.s0\0"
  "llvm.hexagon.M2.mpyud.acc.hl.s1\0"
  "llvm.hexagon.M2.mpyud.acc.lh.s0\0"
  "llvm.hexagon.M2.mpyud.acc.lh.s1\0"
  "llvm.hexagon.M2.mpyud.acc.ll.s0\0"
  "llvm.hexagon.M2.mpyud.acc.ll.s1\0"
  "llvm.hexagon.M2.mpyud.hh.s0\0"
  "llvm.hexagon.M2.mpyud.hh.s1\0"
  "llvm.hexagon.M2.mpyud.hl.s0\0"
  "llvm.hexagon.M2.mpyud.hl.s1\0"
  "llvm.hexagon.M2.mpyud.lh.s0\0"
  "llvm.hexagon.M2.mpyud.lh.s1\0"
  "llvm.hexagon.M2.mpyud.ll.s0\0"
  "llvm.hexagon.M2.mpyud.ll.s1\0"
  "llvm.hexagon.M2.mpyud.nac.hh.s0\0"
  "llvm.hexagon.M2.mpyud.nac.hh.s1\0"
  "llvm.hexagon.M2.mpyud.nac.hl.s0\0"
  "llvm.hexagon.M2.mpyud.nac.hl.s1\0"
  "llvm.hexagon.M2.mpyud.nac.lh.s0\0"
  "llvm.hexagon.M2.mpyud.nac.lh.s1\0"
  "llvm.hexagon.M2.mpyud.nac.ll.s0\0"
  "llvm.hexagon.M2.mpyud.nac.ll.s1\0"
  "llvm.hexagon.M2.mpyui\0"
  "llvm.hexagon.M2.nacci\0"
  "llvm.hexagon.M2.naccii\0"
  "llvm.hexagon.M2.subacc\0"
  "llvm.hexagon.M2.vabsdiffh\0"
  "llvm.hexagon.M2.vabsdiffw\0"
  "llvm.hexagon.M2.vcmac.s0.sat.i\0"
  "llvm.hexagon.M2.vcmac.s0.sat.r\0"
  "llvm.hexagon.M2.vcmpy.s0.sat.i\0"
  "llvm.hexagon.M2.vcmpy.s0.sat.r\0"
  "llvm.hexagon.M2.vcmpy.s1.sat.i\0"
  "llvm.hexagon.M2.vcmpy.s1.sat.r\0"
  "llvm.hexagon.M2.vdmacs.s0\0"
  "llvm.hexagon.M2.vdmacs.s1\0"
  "llvm.hexagon.M2.vdmpyrs.s0\0"
  "llvm.hexagon.M2.vdmpyrs.s1\0"
  "llvm.hexagon.M2.vdmpys.s0\0"
  "llvm.hexagon.M2.vdmpys.s1\0"
  "llvm.hexagon.M2.vmac2\0"
  "llvm.hexagon.M2.vmac2es\0"
  "llvm.hexagon.M2.vmac2es.s0\0"
  "llvm.hexagon.M2.vmac2es.s1\0"
  "llvm.hexagon.M2.vmac2s.s0\0"
  "llvm.hexagon.M2.vmac2s.s1\0"
  "llvm.hexagon.M2.vmac2su.s0\0"
  "llvm.hexagon.M2.vmac2su.s1\0"
  "llvm.hexagon.M2.vmpy2es.s0\0"
  "llvm.hexagon.M2.vmpy2es.s1\0"
  "llvm.hexagon.M2.vmpy2s.s0\0"
  "llvm.hexagon.M2.vmpy2s.s0pack\0"
  "llvm.hexagon.M2.vmpy2s.s1\0"
  "llvm.hexagon.M2.vmpy2s.s1pack\0"
  "llvm.hexagon.M2.vmpy2su.s0\0"
  "llvm.hexagon.M2.vmpy2su.s1\0"
  "llvm.hexagon.M2.vraddh\0"
  "llvm.hexagon.M2.vradduh\0"
  "llvm.hexagon.M2.vrcmaci.s0\0"
  "llvm.hexagon.M2.vrcmaci.s0c\0"
  "llvm.hexagon.M2.vrcmacr.s0\0"
  "llvm.hexagon.M2.vrcmacr.s0c\0"
  "llvm.hexagon.M2.vrcmpyi.s0\0"
  "llvm.hexagon.M2.vrcmpyi.s0c\0"
  "llvm.hexagon.M2.vrcmpyr.s0\0"
  "llvm.hexagon.M2.vrcmpyr.s0c\0"
  "llvm.hexagon.M2.vrcmpys.acc.s1\0"
  "llvm.hexagon.M2.vrcmpys.s1\0"
  "llvm.hexagon.M2.vrcmpys.s1rp\0"
  "llvm.hexagon.M2.vrmac.s0\0"
  "llvm.hexagon.M2.vrmpy.s0\0"
  "llvm.hexagon.M2.xor.xacc\0"
  "llvm.hexagon.M4.and.and\0"
  "llvm.hexagon.M4.and.andn\0"
  "llvm.hexagon.M4.and.or\0"
  "llvm.hexagon.M4.and.xor\0"
  "llvm.hexagon.M4.cmpyi.wh\0"
  "llvm.hexagon.M4.cmpyi.whc\0"
  "llvm.hexagon.M4.cmpyr.wh\0"
  "llvm.hexagon.M4.cmpyr.whc\0"
  "llvm.hexagon.M4.mac.up.s1.sat\0"
  "llvm.hexagon.M4.mpyri.addi\0"
  "llvm.hexagon.M4.mpyri.addr\0"
  "llvm.hexagon.M4.mpyri.addr.u2\0"
  "llvm.hexagon.M4.mpyrr.addi\0"
  "llvm.hexagon.M4.mpyrr.addr\0"
  "llvm.hexagon.M4.nac.up.s1.sat\0"
  "llvm.hexagon.M4.or.and\0"
  "llvm.hexagon.M4.or.andn\0"
  "llvm.hexagon.M4.or.or\0"
  "llvm.hexagon.M4.or.xor\0"
  "llvm.hexagon.M4.pmpyw\0"
  "llvm.hexagon.M4.pmpyw.acc\0"
  "llvm.hexagon.M4.vpmpyh\0"
  "llvm.hexagon.M4.vpmpyh.acc\0"
  "llvm.hexagon.M4.vrmpyeh.acc.s0\0"
  "llvm.hexagon.M4.vrmpyeh.acc.s1\0"
  "llvm.hexagon.M4.vrmpyeh.s0\0"
  "llvm.hexagon.M4.vrmpyeh.s1\0"
  "llvm.hexagon.M4.vrmpyoh.acc.s0\0"
  "llvm.hexagon.M4.vrmpyoh.acc.s1\0"
  "llvm.hexagon.M4.vrmpyoh.s0\0"
  "llvm.hexagon.M4.vrmpyoh.s1\0"
  "llvm.hexagon.M4.xor.and\0"
  "llvm.hexagon.M4.xor.andn\0"
  "llvm.hexagon.M4.xor.or\0"
  "llvm.hexagon.M4.xor.xacc\0"
  "llvm.hexagon.M5.vdmacbsu\0"
  "llvm.hexagon.M5.vdmpybsu\0"
  "llvm.hexagon.M5.vmacbsu\0"
  "llvm.hexagon.M5.vmacbuu\0"
  "llvm.hexagon.M5.vmpybsu\0"
  "llvm.hexagon.M5.vmpybuu\0"
  "llvm.hexagon.M5.vrmacbsu\0"
  "llvm.hexagon.M5.vrmacbuu\0"
  "llvm.hexagon.M5.vrmpybsu\0"
  "llvm.hexagon.M5.vrmpybuu\0"
  "llvm.hexagon.M6.vabsdiffb\0"
  "llvm.hexagon.M6.vabsdiffub\0"
  "llvm.hexagon.M7.dcmpyiw\0"
  "llvm.hexagon.M7.dcmpyiw.acc\0"
  "llvm.hexagon.M7.dcmpyiwc\0"
  "llvm.hexagon.M7.dcmpyiwc.acc\0"
  "llvm.hexagon.M7.dcmpyrw\0"
  "llvm.hexagon.M7.dcmpyrw.acc\0"
  "llvm.hexagon.M7.dcmpyrwc\0"
  "llvm.hexagon.M7.dcmpyrwc.acc\0"
  "llvm.hexagon.M7.vdmpy\0"
  "llvm.hexagon.M7.vdmpy.acc\0"
  "llvm.hexagon.M7.wcmpyiw\0"
  "llvm.hexagon.M7.wcmpyiw.rnd\0"
  "llvm.hexagon.M7.wcmpyiwc\0"
  "llvm.hexagon.M7.wcmpyiwc.rnd\0"
  "llvm.hexagon.M7.wcmpyrw\0"
  "llvm.hexagon.M7.wcmpyrw.rnd\0"
  "llvm.hexagon.M7.wcmpyrwc\0"
  "llvm.hexagon.M7.wcmpyrwc.rnd\0"
  "llvm.hexagon.S2.addasl.rrri\0"
  "llvm.hexagon.S2.asl.i.p\0"
  "llvm.hexagon.S2.asl.i.p.acc\0"
  "llvm.hexagon.S2.asl.i.p.and\0"
  "llvm.hexagon.S2.asl.i.p.nac\0"
  "llvm.hexagon.S2.asl.i.p.or\0"
  "llvm.hexagon.S2.asl.i.p.xacc\0"
  "llvm.hexagon.S2.asl.i.r\0"
  "llvm.hexagon.S2.asl.i.r.acc\0"
  "llvm.hexagon.S2.asl.i.r.and\0"
  "llvm.hexagon.S2.asl.i.r.nac\0"
  "llvm.hexagon.S2.asl.i.r.or\0"
  "llvm.hexagon.S2.asl.i.r.sat\0"
  "llvm.hexagon.S2.asl.i.r.xacc\0"
  "llvm.hexagon.S2.asl.i.vh\0"
  "llvm.hexagon.S2.asl.i.vw\0"
  "llvm.hexagon.S2.asl.r.p\0"
  "llvm.hexagon.S2.asl.r.p.acc\0"
  "llvm.hexagon.S2.asl.r.p.and\0"
  "llvm.hexagon.S2.asl.r.p.nac\0"
  "llvm.hexagon.S2.asl.r.p.or\0"
  "llvm.hexagon.S2.asl.r.p.xor\0"
  "llvm.hexagon.S2.asl.r.r\0"
  "llvm.hexagon.S2.asl.r.r.acc\0"
  "llvm.hexagon.S2.asl.r.r.and\0"
  "llvm.hexagon.S2.asl.r.r.nac\0"
  "llvm.hexagon.S2.asl.r.r.or\0"
  "llvm.hexagon.S2.asl.r.r.sat\0"
  "llvm.hexagon.S2.asl.r.vh\0"
  "llvm.hexagon.S2.asl.r.vw\0"
  "llvm.hexagon.S2.asr.i.p\0"
  "llvm.hexagon.S2.asr.i.p.acc\0"
  "llvm.hexagon.S2.asr.i.p.and\0"
  "llvm.hexagon.S2.asr.i.p.nac\0"
  "llvm.hexagon.S2.asr.i.p.or\0"
  "llvm.hexagon.S2.asr.i.p.rnd\0"
  "llvm.hexagon.S2.asr.i.p.rnd.goodsyntax\0"
  "llvm.hexagon.S2.asr.i.r\0"
  "llvm.hexagon.S2.asr.i.r.acc\0"
  "llvm.hexagon.S2.asr.i.r.and\0"
  "llvm.hexagon.S2.asr.i.r.nac\0"
  "llvm.hexagon.S2.asr.i.r.or\0"
  "llvm.hexagon.S2.asr.i.r.rnd\0"
  "llvm.hexagon.S2.asr.i.r.rnd.goodsyntax\0"
  "llvm.hexagon.S2.asr.i.svw.trun\0"
  "llvm.hexagon.S2.asr.i.vh\0"
  "llvm.hexagon.S2.asr.i.vw\0"
  "llvm.hexagon.S2.asr.r.p\0"
  "llvm.hexagon.S2.asr.r.p.acc\0"
  "llvm.hexagon.S2.asr.r.p.and\0"
  "llvm.hexagon.S2.asr.r.p.nac\0"
  "llvm.hexagon.S2.asr.r.p.or\0"
  "llvm.hexagon.S2.asr.r.p.xor\0"
  "llvm.hexagon.S2.asr.r.r\0"
  "llvm.hexagon.S2.asr.r.r.acc\0"
  "llvm.hexagon.S2.asr.r.r.and\0"
  "llvm.hexagon.S2.asr.r.r.nac\0"
  "llvm.hexagon.S2.asr.r.r.or\0"
  "llvm.hexagon.S2.asr.r.r.sat\0"
  "llvm.hexagon.S2.asr.r.svw.trun\0"
  "llvm.hexagon.S2.asr.r.vh\0"
  "llvm.hexagon.S2.asr.r.vw\0"
  "llvm.hexagon.S2.brev\0"
  "llvm.hexagon.S2.brevp\0"
  "llvm.hexagon.S2.cl0\0"
  "llvm.hexagon.S2.cl0p\0"
  "llvm.hexagon.S2.cl1\0"
  "llvm.hexagon.S2.cl1p\0"
  "llvm.hexagon.S2.clb\0"
  "llvm.hexagon.S2.clbnorm\0"
  "llvm.hexagon.S2.clbp\0"
  "llvm.hexagon.S2.clrbit.i\0"
  "llvm.hexagon.S2.clrbit.r\0"
  "llvm.hexagon.S2.ct0\0"
  "llvm.hexagon.S2.ct0p\0"
  "llvm.hexagon.S2.ct1\0"
  "llvm.hexagon.S2.ct1p\0"
  "llvm.hexagon.S2.deinterleave\0"
  "llvm.hexagon.S2.extractu\0"
  "llvm.hexagon.S2.extractu.rp\0"
  "llvm.hexagon.S2.extractup\0"
  "llvm.hexagon.S2.extractup.rp\0"
  "llvm.hexagon.S2.insert\0"
  "llvm.hexagon.S2.insert.rp\0"
  "llvm.hexagon.S2.insertp\0"
  "llvm.hexagon.S2.insertp.rp\0"
  "llvm.hexagon.S2.interleave\0"
  "llvm.hexagon.S2.lfsp\0"
  "llvm.hexagon.S2.lsl.r.p\0"
  "llvm.hexagon.S2.lsl.r.p.acc\0"
  "llvm.hexagon.S2.lsl.r.p.and\0"
  "llvm.hexagon.S2.lsl.r.p.nac\0"
  "llvm.hexagon.S2.lsl.r.p.or\0"
  "llvm.hexagon.S2.lsl.r.p.xor\0"
  "llvm.hexagon.S2.lsl.r.r\0"
  "llvm.hexagon.S2.lsl.r.r.acc\0"
  "llvm.hexagon.S2.lsl.r.r.and\0"
  "llvm.hexagon.S2.lsl.r.r.nac\0"
  "llvm.hexagon.S2.lsl.r.r.or\0"
  "llvm.hexagon.S2.lsl.r.vh\0"
  "llvm.hexagon.S2.lsl.r.vw\0"
  "llvm.hexagon.S2.lsr.i.p\0"
  "llvm.hexagon.S2.lsr.i.p.acc\0"
  "llvm.hexagon.S2.lsr.i.p.and\0"
  "llvm.hexagon.S2.lsr.i.p.nac\0"
  "llvm.hexagon.S2.lsr.i.p.or\0"
  "llvm.hexagon.S2.lsr.i.p.xacc\0"
  "llvm.hexagon.S2.lsr.i.r\0"
  "llvm.hexagon.S2.lsr.i.r.acc\0"
  "llvm.hexagon.S2.lsr.i.r.and\0"
  "llvm.hexagon.S2.lsr.i.r.nac\0"
  "llvm.hexagon.S2.lsr.i.r.or\0"
  "llvm.hexagon.S2.lsr.i.r.xacc\0"
  "llvm.hexagon.S2.lsr.i.vh\0"
  "llvm.hexagon.S2.lsr.i.vw\0"
  "llvm.hexagon.S2.lsr.r.p\0"
  "llvm.hexagon.S2.lsr.r.p.acc\0"
  "llvm.hexagon.S2.lsr.r.p.and\0"
  "llvm.hexagon.S2.lsr.r.p.nac\0"
  "llvm.hexagon.S2.lsr.r.p.or\0"
  "llvm.hexagon.S2.lsr.r.p.xor\0"
  "llvm.hexagon.S2.lsr.r.r\0"
  "llvm.hexagon.S2.lsr.r.r.acc\0"
  "llvm.hexagon.S2.lsr.r.r.and\0"
  "llvm.hexagon.S2.lsr.r.r.nac\0"
  "llvm.hexagon.S2.lsr.r.r.or\0"
  "llvm.hexagon.S2.lsr.r.vh\0"
  "llvm.hexagon.S2.lsr.r.vw\0"
  "llvm.hexagon.S2.mask\0"
  "llvm.hexagon.S2.packhl\0"
  "llvm.hexagon.S2.parityp\0"
  "llvm.hexagon.S2.setbit.i\0"
  "llvm.hexagon.S2.setbit.r\0"
  "llvm.hexagon.S2.shuffeb\0"
  "llvm.hexagon.S2.shuffeh\0"
  "llvm.hexagon.S2.shuffob\0"
  "llvm.hexagon.S2.shuffoh\0"
  "llvm.hexagon.S2.storerb.pbr\0"
  "llvm.hexagon.S2.storerb.pci\0"
  "llvm.hexagon.S2.storerb.pcr\0"
  "llvm.hexagon.S2.storerd.pbr\0"
  "llvm.hexagon.S2.storerd.pci\0"
  "llvm.hexagon.S2.storerd.pcr\0"
  "llvm.hexagon.S2.storerf.pbr\0"
  "llvm.hexagon.S2.storerf.pci\0"
  "llvm.hexagon.S2.storerf.pcr\0"
  "llvm.hexagon.S2.storerh.pbr\0"
  "llvm.hexagon.S2.storerh.pci\0"
  "llvm.hexagon.S2.storerh.pcr\0"
  "llvm.hexagon.S2.storeri.pbr\0"
  "llvm.hexagon.S2.storeri.pci\0"
  "llvm.hexagon.S2.storeri.pcr\0"
  "llvm.hexagon.S2.storew.locked\0"
  "llvm.hexagon.S2.svsathb\0"
  "llvm.hexagon.S2.svsathub\0"
  "llvm.hexagon.S2.tableidxb.goodsyntax\0"
  "llvm.hexagon.S2.tableidxd.goodsyntax\0"
  "llvm.hexagon.S2.tableidxh.goodsyntax\0"
  "llvm.hexagon.S2.tableidxw.goodsyntax\0"
  "llvm.hexagon.S2.togglebit.i\0"
  "llvm.hexagon.S2.togglebit.r\0"
  "llvm.hexagon.S2.tstbit.i\0"
  "llvm.hexagon.S2.tstbit.r\0"
  "llvm.hexagon.S2.valignib\0"
  "llvm.hexagon.S2.valignrb\0"
  "llvm.hexagon.S2.vcnegh\0"
  "llvm.hexagon.S2.vcrotate\0"
  "llvm.hexagon.S2.vrcnegh\0"
  "llvm.hexagon.S2.vrndpackwh\0"
  "llvm.hexagon.S2.vrndpackwhs\0"
  "llvm.hexagon.S2.vsathb\0"
  "llvm.hexagon.S2.vsathb.nopack\0"
  "llvm.hexagon.S2.vsathub\0"
  "llvm.hexagon.S2.vsathub.nopack\0"
  "llvm.hexagon.S2.vsatwh\0"
  "llvm.hexagon.S2.vsatwh.nopack\0"
  "llvm.hexagon.S2.vsatwuh\0"
  "llvm.hexagon.S2.vsatwuh.nopack\0"
  "llvm.hexagon.S2.vsplatrb\0"
  "llvm.hexagon.S2.vsplatrh\0"
  "llvm.hexagon.S2.vspliceib\0"
  "llvm.hexagon.S2.vsplicerb\0"
  "llvm.hexagon.S2.vsxtbh\0"
  "llvm.hexagon.S2.vsxthw\0"
  "llvm.hexagon.S2.vtrunehb\0"
  "llvm.hexagon.S2.vtrunewh\0"
  "llvm.hexagon.S2.vtrunohb\0"
  "llvm.hexagon.S2.vtrunowh\0"
  "llvm.hexagon.S2.vzxtbh\0"
  "llvm.hexagon.S2.vzxthw\0"
  "llvm.hexagon.S4.addaddi\0"
  "llvm.hexagon.S4.addi.asl.ri\0"
  "llvm.hexagon.S4.addi.lsr.ri\0"
  "llvm.hexagon.S4.andi.asl.ri\0"
  "llvm.hexagon.S4.andi.lsr.ri\0"
  "llvm.hexagon.S4.clbaddi\0"
  "llvm.hexagon.S4.clbpaddi\0"
  "llvm.hexagon.S4.clbpnorm\0"
  "llvm.hexagon.S4.extract\0"
  "llvm.hexagon.S4.extract.rp\0"
  "llvm.hexagon.S4.extractp\0"
  "llvm.hexagon.S4.extractp.rp\0"
  "llvm.hexagon.S4.lsli\0"
  "llvm.hexagon.S4.ntstbit.i\0"
  "llvm.hexagon.S4.ntstbit.r\0"
  "llvm.hexagon.S4.or.andi\0"
  "llvm.hexagon.S4.or.andix\0"
  "llvm.hexagon.S4.or.ori\0"
  "llvm.hexagon.S4.ori.asl.ri\0"
  "llvm.hexagon.S4.ori.lsr.ri\0"
  "llvm.hexagon.S4.parity\0"
  "llvm.hexagon.S4.stored.locked\0"
  "llvm.hexagon.S4.subaddi\0"
  "llvm.hexagon.S4.subi.asl.ri\0"
  "llvm.hexagon.S4.subi.lsr.ri\0"
  "llvm.hexagon.S4.vrcrotate\0"
  "llvm.hexagon.S4.vrcrotate.acc\0"
  "llvm.hexagon.S4.vxaddsubh\0"
  "llvm.hexagon.S4.vxaddsubhr\0"
  "llvm.hexagon.S4.vxaddsubw\0"
  "llvm.hexagon.S4.vxsubaddh\0"
  "llvm.hexagon.S4.vxsubaddhr\0"
  "llvm.hexagon.S4.vxsubaddw\0"
  "llvm.hexagon.S5.asrhub.rnd.sat.goodsyntax\0"
  "llvm.hexagon.S5.asrhub.sat\0"
  "llvm.hexagon.S5.popcountp\0"
  "llvm.hexagon.S5.vasrhrnd.goodsyntax\0"
  "llvm.hexagon.S6.rol.i.p\0"
  "llvm.hexagon.S6.rol.i.p.acc\0"
  "llvm.hexagon.S6.rol.i.p.and\0"
  "llvm.hexagon.S6.rol.i.p.nac\0"
  "llvm.hexagon.S6.rol.i.p.or\0"
  "llvm.hexagon.S6.rol.i.p.xacc\0"
  "llvm.hexagon.S6.rol.i.r\0"
  "llvm.hexagon.S6.rol.i.r.acc\0"
  "llvm.hexagon.S6.rol.i.r.and\0"
  "llvm.hexagon.S6.rol.i.r.nac\0"
  "llvm.hexagon.S6.rol.i.r.or\0"
  "llvm.hexagon.S6.rol.i.r.xacc\0"
  "llvm.hexagon.S6.vsplatrbp\0"
  "llvm.hexagon.S6.vtrunehb.ppp\0"
  "llvm.hexagon.S6.vtrunohb.ppp\0"
  "llvm.hexagon.V6.extractw\0"
  "llvm.hexagon.V6.extractw.128B\0"
  "llvm.hexagon.V6.get.qfext\0"
  "llvm.hexagon.V6.get.qfext.128B\0"
  "llvm.hexagon.V6.get.qfext.oracc\0"
  "llvm.hexagon.V6.get.qfext.oracc.128B\0"
  "llvm.hexagon.V6.hi\0"
  "llvm.hexagon.V6.hi.128B\0"
  "llvm.hexagon.V6.lo\0"
  "llvm.hexagon.V6.lo.128B\0"
  "llvm.hexagon.V6.lvsplatb\0"
  "llvm.hexagon.V6.lvsplatb.128B\0"
  "llvm.hexagon.V6.lvsplath\0"
  "llvm.hexagon.V6.lvsplath.128B\0"
  "llvm.hexagon.V6.lvsplatw\0"
  "llvm.hexagon.V6.lvsplatw.128B\0"
  "llvm.hexagon.V6.pred.and\0"
  "llvm.hexagon.V6.pred.and.128B\0"
  "llvm.hexagon.V6.pred.and.n\0"
  "llvm.hexagon.V6.pred.and.n.128B\0"
  "llvm.hexagon.V6.pred.not\0"
  "llvm.hexagon.V6.pred.not.128B\0"
  "llvm.hexagon.V6.pred.or\0"
  "llvm.hexagon.V6.pred.or.128B\0"
  "llvm.hexagon.V6.pred.or.n\0"
  "llvm.hexagon.V6.pred.or.n.128B\0"
  "llvm.hexagon.V6.pred.scalar2\0"
  "llvm.hexagon.V6.pred.scalar2.128B\0"
  "llvm.hexagon.V6.pred.scalar2v2\0"
  "llvm.hexagon.V6.pred.scalar2v2.128B\0"
  "llvm.hexagon.V6.pred.typecast\0"
  "llvm.hexagon.V6.pred.typecast.128B\0"
  "llvm.hexagon.V6.pred.xor\0"
  "llvm.hexagon.V6.pred.xor.128B\0"
  "llvm.hexagon.V6.set.qfext\0"
  "llvm.hexagon.V6.set.qfext.128B\0"
  "llvm.hexagon.V6.shuffeqh\0"
  "llvm.hexagon.V6.shuffeqh.128B\0"
  "llvm.hexagon.V6.shuffeqw\0"
  "llvm.hexagon.V6.shuffeqw.128B\0"
  "llvm.hexagon.V6.v6mpyhubs10\0"
  "llvm.hexagon.V6.v6mpyhubs10.128B\0"
  "llvm.hexagon.V6.v6mpyhubs10.vxx\0"
  "llvm.hexagon.V6.v6mpyhubs10.vxx.128B\0"
  "llvm.hexagon.V6.v6mpyvubs10\0"
  "llvm.hexagon.V6.v6mpyvubs10.128B\0"
  "llvm.hexagon.V6.v6mpyvubs10.vxx\0"
  "llvm.hexagon.V6.v6mpyvubs10.vxx.128B\0"
  "llvm.hexagon.V6.vL32b.npred.ai\0"
  "llvm.hexagon.V6.vL32b.npred.ai.128B\0"
  "llvm.hexagon.V6.vL32b.npred.pi\0"
  "llvm.hexagon.V6.vL32b.npred.pi.128B\0"
  "llvm.hexagon.V6.vL32b.npred.ppu\0"
  "llvm.hexagon.V6.vL32b.npred.ppu.128B\0"
  "llvm.hexagon.V6.vL32b.nt.npred.ai\0"
  "llvm.hexagon.V6.vL32b.nt.npred.ai.128B\0"
  "llvm.hexagon.V6.vL32b.nt.npred.pi\0"
  "llvm.hexagon.V6.vL32b.nt.npred.pi.128B\0"
  "llvm.hexagon.V6.vL32b.nt.npred.ppu\0"
  "llvm.hexagon.V6.vL32b.nt.npred.ppu.128B\0"
  "llvm.hexagon.V6.vL32b.nt.pred.ai\0"
  "llvm.hexagon.V6.vL32b.nt.pred.ai.128B\0"
  "llvm.hexagon.V6.vL32b.nt.pred.pi\0"
  "llvm.hexagon.V6.vL32b.nt.pred.pi.128B\0"
  "llvm.hexagon.V6.vL32b.nt.pred.ppu\0"
  "llvm.hexagon.V6.vL32b.nt.pred.ppu.128B\0"
  "llvm.hexagon.V6.vL32b.pred.ai\0"
  "llvm.hexagon.V6.vL32b.pred.ai.128B\0"
  "llvm.hexagon.V6.vL32b.pred.pi\0"
  "llvm.hexagon.V6.vL32b.pred.pi.128B\0"
  "llvm.hexagon.V6.vL32b.pred.ppu\0"
  "llvm.hexagon.V6.vL32b.pred.ppu.128B\0"
  "llvm.hexagon.V6.vS32Ub.npred.ai\0"
  "llvm.hexagon.V6.vS32Ub.npred.ai.128B\0"
  "llvm.hexagon.V6.vS32Ub.npred.pi\0"
  "llvm.hexagon.V6.vS32Ub.npred.pi.128B\0"
  "llvm.hexagon.V6.vS32Ub.npred.ppu\0"
  "llvm.hexagon.V6.vS32Ub.npred.ppu.128B\0"
  "llvm.hexagon.V6.vS32Ub.pred.ai\0"
  "llvm.hexagon.V6.vS32Ub.pred.ai.128B\0"
  "llvm.hexagon.V6.vS32Ub.pred.pi\0"
  "llvm.hexagon.V6.vS32Ub.pred.pi.128B\0"
  "llvm.hexagon.V6.vS32Ub.pred.ppu\0"
  "llvm.hexagon.V6.vS32Ub.pred.ppu.128B\0"
  "llvm.hexagon.V6.vS32b.npred.ai\0"
  "llvm.hexagon.V6.vS32b.npred.ai.128B\0"
  "llvm.hexagon.V6.vS32b.npred.pi\0"
  "llvm.hexagon.V6.vS32b.npred.pi.128B\0"
  "llvm.hexagon.V6.vS32b.npred.ppu\0"
  "llvm.hexagon.V6.vS32b.npred.ppu.128B\0"
  "llvm.hexagon.V6.vS32b.nqpred.ai\0"
  "llvm.hexagon.V6.vS32b.nqpred.ai.128B\0"
  "llvm.hexagon.V6.vS32b.nt.npred.ai\0"
  "llvm.hexagon.V6.vS32b.nt.npred.ai.128B\0"
  "llvm.hexagon.V6.vS32b.nt.npred.pi\0"
  "llvm.hexagon.V6.vS32b.nt.npred.pi.128B\0"
  "llvm.hexagon.V6.vS32b.nt.npred.ppu\0"
  "llvm.hexagon.V6.vS32b.nt.npred.ppu.128B\0"
  "llvm.hexagon.V6.vS32b.nt.nqpred.ai\0"
  "llvm.hexagon.V6.vS32b.nt.nqpred.ai.128B\0"
  "llvm.hexagon.V6.vS32b.nt.pred.ai\0"
  "llvm.hexagon.V6.vS32b.nt.pred.ai.128B\0"
  "llvm.hexagon.V6.vS32b.nt.pred.pi\0"
  "llvm.hexagon.V6.vS32b.nt.pred.pi.128B\0"
  "llvm.hexagon.V6.vS32b.nt.pred.ppu\0"
  "llvm.hexagon.V6.vS32b.nt.pred.ppu.128B\0"
  "llvm.hexagon.V6.vS32b.nt.qpred.ai\0"
  "llvm.hexagon.V6.vS32b.nt.qpred.ai.128B\0"
  "llvm.hexagon.V6.vS32b.pred.ai\0"
  "llvm.hexagon.V6.vS32b.pred.ai.128B\0"
  "llvm.hexagon.V6.vS32b.pred.pi\0"
  "llvm.hexagon.V6.vS32b.pred.pi.128B\0"
  "llvm.hexagon.V6.vS32b.pred.ppu\0"
  "llvm.hexagon.V6.vS32b.pred.ppu.128B\0"
  "llvm.hexagon.V6.vS32b.qpred.ai\0"
  "llvm.hexagon.V6.vS32b.qpred.ai.128B\0"
  "llvm.hexagon.V6.vabs.f8\0"
  "llvm.hexagon.V6.vabs.f8.128B\0"
  "llvm.hexagon.V6.vabs.hf\0"
  "llvm.hexagon.V6.vabs.hf.128B\0"
  "llvm.hexagon.V6.vabs.qf16.hf\0"
  "llvm.hexagon.V6.vabs.qf16.hf.128B\0"
  "llvm.hexagon.V6.vabs.qf16.qf16\0"
  "llvm.hexagon.V6.vabs.qf16.qf16.128B\0"
  "llvm.hexagon.V6.vabs.qf32.qf32\0"
  "llvm.hexagon.V6.vabs.qf32.qf32.128B\0"
  "llvm.hexagon.V6.vabs.qf32.sf\0"
  "llvm.hexagon.V6.vabs.qf32.sf.128B\0"
  "llvm.hexagon.V6.vabs.sf\0"
  "llvm.hexagon.V6.vabs.sf.128B\0"
  "llvm.hexagon.V6.vabsb\0"
  "llvm.hexagon.V6.vabsb.128B\0"
  "llvm.hexagon.V6.vabsb.sat\0"
  "llvm.hexagon.V6.vabsb.sat.128B\0"
  "llvm.hexagon.V6.vabsdiffh\0"
  "llvm.hexagon.V6.vabsdiffh.128B\0"
  "llvm.hexagon.V6.vabsdiffub\0"
  "llvm.hexagon.V6.vabsdiffub.128B\0"
  "llvm.hexagon.V6.vabsdiffuh\0"
  "llvm.hexagon.V6.vabsdiffuh.128B\0"
  "llvm.hexagon.V6.vabsdiffw\0"
  "llvm.hexagon.V6.vabsdiffw.128B\0"
  "llvm.hexagon.V6.vabsh\0"
  "llvm.hexagon.V6.vabsh.128B\0"
  "llvm.hexagon.V6.vabsh.sat\0"
  "llvm.hexagon.V6.vabsh.sat.128B\0"
  "llvm.hexagon.V6.vabsw\0"
  "llvm.hexagon.V6.vabsw.128B\0"
  "llvm.hexagon.V6.vabsw.sat\0"
  "llvm.hexagon.V6.vabsw.sat.128B\0"
  "llvm.hexagon.V6.vadd.hf\0"
  "llvm.hexagon.V6.vadd.hf.128B\0"
  "llvm.hexagon.V6.vadd.hf.f8\0"
  "llvm.hexagon.V6.vadd.hf.f8.128B\0"
  "llvm.hexagon.V6.vadd.hf.hf\0"
  "llvm.hexagon.V6.vadd.hf.hf.128B\0"
  "llvm.hexagon.V6.vadd.qf16\0"
  "llvm.hexagon.V6.vadd.qf16.128B\0"
  "llvm.hexagon.V6.vadd.qf16.mix\0"
  "llvm.hexagon.V6.vadd.qf16.mix.128B\0"
  "llvm.hexagon.V6.vadd.qf32\0"
  "llvm.hexagon.V6.vadd.qf32.128B\0"
  "llvm.hexagon.V6.vadd.qf32.mix\0"
  "llvm.hexagon.V6.vadd.qf32.mix.128B\0"
  "llvm.hexagon.V6.vadd.sf\0"
  "llvm.hexagon.V6.vadd.sf.128B\0"
  "llvm.hexagon.V6.vadd.sf.bf\0"
  "llvm.hexagon.V6.vadd.sf.bf.128B\0"
  "llvm.hexagon.V6.vadd.sf.hf\0"
  "llvm.hexagon.V6.vadd.sf.hf.128B\0"
  "llvm.hexagon.V6.vadd.sf.sf\0"
  "llvm.hexagon.V6.vadd.sf.sf.128B\0"
  "llvm.hexagon.V6.vaddb\0"
  "llvm.hexagon.V6.vaddb.128B\0"
  "llvm.hexagon.V6.vaddb.dv\0"
  "llvm.hexagon.V6.vaddb.dv.128B\0"
  "llvm.hexagon.V6.vaddbnq\0"
  "llvm.hexagon.V6.vaddbnq.128B\0"
  "llvm.hexagon.V6.vaddbq\0"
  "llvm.hexagon.V6.vaddbq.128B\0"
  "llvm.hexagon.V6.vaddbsat\0"
  "llvm.hexagon.V6.vaddbsat.128B\0"
  "llvm.hexagon.V6.vaddbsat.dv\0"
  "llvm.hexagon.V6.vaddbsat.dv.128B\0"
  "llvm.hexagon.V6.vaddcarry\0"
  "llvm.hexagon.V6.vaddcarry.128B\0"
  "llvm.hexagon.V6.vaddcarryo\0"
  "llvm.hexagon.V6.vaddcarryo.128B\0"
  "llvm.hexagon.V6.vaddcarrysat\0"
  "llvm.hexagon.V6.vaddcarrysat.128B\0"
  "llvm.hexagon.V6.vaddclbh\0"
  "llvm.hexagon.V6.vaddclbh.128B\0"
  "llvm.hexagon.V6.vaddclbw\0"
  "llvm.hexagon.V6.vaddclbw.128B\0"
  "llvm.hexagon.V6.vaddh\0"
  "llvm.hexagon.V6.vaddh.128B\0"
  "llvm.hexagon.V6.vaddh.dv\0"
  "llvm.hexagon.V6.vaddh.dv.128B\0"
  "llvm.hexagon.V6.vaddhnq\0"
  "llvm.hexagon.V6.vaddhnq.128B\0"
  "llvm.hexagon.V6.vaddhq\0"
  "llvm.hexagon.V6.vaddhq.128B\0"
  "llvm.hexagon.V6.vaddhsat\0"
  "llvm.hexagon.V6.vaddhsat.128B\0"
  "llvm.hexagon.V6.vaddhsat.dv\0"
  "llvm.hexagon.V6.vaddhsat.dv.128B\0"
  "llvm.hexagon.V6.vaddhw\0"
  "llvm.hexagon.V6.vaddhw.128B\0"
  "llvm.hexagon.V6.vaddhw.acc\0"
  "llvm.hexagon.V6.vaddhw.acc.128B\0"
  "llvm.hexagon.V6.vaddubh\0"
  "llvm.hexagon.V6.vaddubh.128B\0"
  "llvm.hexagon.V6.vaddubh.acc\0"
  "llvm.hexagon.V6.vaddubh.acc.128B\0"
  "llvm.hexagon.V6.vaddubsat\0"
  "llvm.hexagon.V6.vaddubsat.128B\0"
  "llvm.hexagon.V6.vaddubsat.dv\0"
  "llvm.hexagon.V6.vaddubsat.dv.128B\0"
  "llvm.hexagon.V6.vaddububb.sat\0"
  "llvm.hexagon.V6.vaddububb.sat.128B\0"
  "llvm.hexagon.V6.vadduhsat\0"
  "llvm.hexagon.V6.vadduhsat.128B\0"
  "llvm.hexagon.V6.vadduhsat.dv\0"
  "llvm.hexagon.V6.vadduhsat.dv.128B\0"
  "llvm.hexagon.V6.vadduhw\0"
  "llvm.hexagon.V6.vadduhw.128B\0"
  "llvm.hexagon.V6.vadduhw.acc\0"
  "llvm.hexagon.V6.vadduhw.acc.128B\0"
  "llvm.hexagon.V6.vadduwsat\0"
  "llvm.hexagon.V6.vadduwsat.128B\0"
  "llvm.hexagon.V6.vadduwsat.dv\0"
  "llvm.hexagon.V6.vadduwsat.dv.128B\0"
  "llvm.hexagon.V6.vaddw\0"
  "llvm.hexagon.V6.vaddw.128B\0"
  "llvm.hexagon.V6.vaddw.dv\0"
  "llvm.hexagon.V6.vaddw.dv.128B\0"
  "llvm.hexagon.V6.vaddwnq\0"
  "llvm.hexagon.V6.vaddwnq.128B\0"
  "llvm.hexagon.V6.vaddwq\0"
  "llvm.hexagon.V6.vaddwq.128B\0"
  "llvm.hexagon.V6.vaddwsat\0"
  "llvm.hexagon.V6.vaddwsat.128B\0"
  "llvm.hexagon.V6.vaddwsat.dv\0"
  "llvm.hexagon.V6.vaddwsat.dv.128B\0"
  "llvm.hexagon.V6.valign4\0"
  "llvm.hexagon.V6.valign4.128B\0"
  "llvm.hexagon.V6.valignb\0"
  "llvm.hexagon.V6.valignb.128B\0"
  "llvm.hexagon.V6.valignbi\0"
  "llvm.hexagon.V6.valignbi.128B\0"
  "llvm.hexagon.V6.vand\0"
  "llvm.hexagon.V6.vand.128B\0"
  "llvm.hexagon.V6.vandnqrt\0"
  "llvm.hexagon.V6.vandnqrt.128B\0"
  "llvm.hexagon.V6.vandnqrt.acc\0"
  "llvm.hexagon.V6.vandnqrt.acc.128B\0"
  "llvm.hexagon.V6.vandqrt\0"
  "llvm.hexagon.V6.vandqrt.128B\0"
  "llvm.hexagon.V6.vandqrt.acc\0"
  "llvm.hexagon.V6.vandqrt.acc.128B\0"
  "llvm.hexagon.V6.vandvnqv\0"
  "llvm.hexagon.V6.vandvnqv.128B\0"
  "llvm.hexagon.V6.vandvqv\0"
  "llvm.hexagon.V6.vandvqv.128B\0"
  "llvm.hexagon.V6.vandvrt\0"
  "llvm.hexagon.V6.vandvrt.128B\0"
  "llvm.hexagon.V6.vandvrt.acc\0"
  "llvm.hexagon.V6.vandvrt.acc.128B\0"
  "llvm.hexagon.V6.vaslh\0"
  "llvm.hexagon.V6.vaslh.128B\0"
  "llvm.hexagon.V6.vaslh.acc\0"
  "llvm.hexagon.V6.vaslh.acc.128B\0"
  "llvm.hexagon.V6.vaslhv\0"
  "llvm.hexagon.V6.vaslhv.128B\0"
  "llvm.hexagon.V6.vaslw\0"
  "llvm.hexagon.V6.vaslw.128B\0"
  "llvm.hexagon.V6.vaslw.acc\0"
  "llvm.hexagon.V6.vaslw.acc.128B\0"
  "llvm.hexagon.V6.vaslwv\0"
  "llvm.hexagon.V6.vaslwv.128B\0"
  "llvm.hexagon.V6.vasr.into\0"
  "llvm.hexagon.V6.vasr.into.128B\0"
  "llvm.hexagon.V6.vasrh\0"
  "llvm.hexagon.V6.vasrh.128B\0"
  "llvm.hexagon.V6.vasrh.acc\0"
  "llvm.hexagon.V6.vasrh.acc.128B\0"
  "llvm.hexagon.V6.vasrhbrndsat\0"
  "llvm.hexagon.V6.vasrhbrndsat.128B\0"
  "llvm.hexagon.V6.vasrhbsat\0"
  "llvm.hexagon.V6.vasrhbsat.128B\0"
  "llvm.hexagon.V6.vasrhubrndsat\0"
  "llvm.hexagon.V6.vasrhubrndsat.128B\0"
  "llvm.hexagon.V6.vasrhubsat\0"
  "llvm.hexagon.V6.vasrhubsat.128B\0"
  "llvm.hexagon.V6.vasrhv\0"
  "llvm.hexagon.V6.vasrhv.128B\0"
  "llvm.hexagon.V6.vasruhubrndsat\0"
  "llvm.hexagon.V6.vasruhubrndsat.128B\0"
  "llvm.hexagon.V6.vasruhubsat\0"
  "llvm.hexagon.V6.vasruhubsat.128B\0"
  "llvm.hexagon.V6.vasruwuhrndsat\0"
  "llvm.hexagon.V6.vasruwuhrndsat.128B\0"
  "llvm.hexagon.V6.vasruwuhsat\0"
  "llvm.hexagon.V6.vasruwuhsat.128B\0"
  "llvm.hexagon.V6.vasrvuhubrndsat\0"
  "llvm.hexagon.V6.vasrvuhubrndsat.128B\0"
  "llvm.hexagon.V6.vasrvuhubsat\0"
  "llvm.hexagon.V6.vasrvuhubsat.128B\0"
  "llvm.hexagon.V6.vasrvwuhrndsat\0"
  "llvm.hexagon.V6.vasrvwuhrndsat.128B\0"
  "llvm.hexagon.V6.vasrvwuhsat\0"
  "llvm.hexagon.V6.vasrvwuhsat.128B\0"
  "llvm.hexagon.V6.vasrw\0"
  "llvm.hexagon.V6.vasrw.128B\0"
  "llvm.hexagon.V6.vasrw.acc\0"
  "llvm.hexagon.V6.vasrw.acc.128B\0"
  "llvm.hexagon.V6.vasrwh\0"
  "llvm.hexagon.V6.vasrwh.128B\0"
  "llvm.hexagon.V6.vasrwhrndsat\0"
  "llvm.hexagon.V6.vasrwhrndsat.128B\0"
  "llvm.hexagon.V6.vasrwhsat\0"
  "llvm.hexagon.V6.vasrwhsat.128B\0"
  "llvm.hexagon.V6.vasrwuhrndsat\0"
  "llvm.hexagon.V6.vasrwuhrndsat.128B\0"
  "llvm.hexagon.V6.vasrwuhsat\0"
  "llvm.hexagon.V6.vasrwuhsat.128B\0"
  "llvm.hexagon.V6.vasrwv\0"
  "llvm.hexagon.V6.vasrwv.128B\0"
  "llvm.hexagon.V6.vassign\0"
  "llvm.hexagon.V6.vassign.128B\0"
  "llvm.hexagon.V6.vassign.fp\0"
  "llvm.hexagon.V6.vassign.fp.128B\0"
  "llvm.hexagon.V6.vassignp\0"
  "llvm.hexagon.V6.vassignp.128B\0"
  "llvm.hexagon.V6.vavgb\0"
  "llvm.hexagon.V6.vavgb.128B\0"
  "llvm.hexagon.V6.vavgbrnd\0"
  "llvm.hexagon.V6.vavgbrnd.128B\0"
  "llvm.hexagon.V6.vavgh\0"
  "llvm.hexagon.V6.vavgh.128B\0"
  "llvm.hexagon.V6.vavghrnd\0"
  "llvm.hexagon.V6.vavghrnd.128B\0"
  "llvm.hexagon.V6.vavgub\0"
  "llvm.hexagon.V6.vavgub.128B\0"
  "llvm.hexagon.V6.vavgubrnd\0"
  "llvm.hexagon.V6.vavgubrnd.128B\0"
  "llvm.hexagon.V6.vavguh\0"
  "llvm.hexagon.V6.vavguh.128B\0"
  "llvm.hexagon.V6.vavguhrnd\0"
  "llvm.hexagon.V6.vavguhrnd.128B\0"
  "llvm.hexagon.V6.vavguw\0"
  "llvm.hexagon.V6.vavguw.128B\0"
  "llvm.hexagon.V6.vavguwrnd\0"
  "llvm.hexagon.V6.vavguwrnd.128B\0"
  "llvm.hexagon.V6.vavgw\0"
  "llvm.hexagon.V6.vavgw.128B\0"
  "llvm.hexagon.V6.vavgwrnd\0"
  "llvm.hexagon.V6.vavgwrnd.128B\0"
  "llvm.hexagon.V6.vcl0h\0"
  "llvm.hexagon.V6.vcl0h.128B\0"
  "llvm.hexagon.V6.vcl0w\0"
  "llvm.hexagon.V6.vcl0w.128B\0"
  "llvm.hexagon.V6.vcombine\0"
  "llvm.hexagon.V6.vcombine.128B\0"
  "llvm.hexagon.V6.vconv.bf.qf32\0"
  "llvm.hexagon.V6.vconv.bf.qf32.128B\0"
  "llvm.hexagon.V6.vconv.f8.qf16\0"
  "llvm.hexagon.V6.vconv.f8.qf16.128B\0"
  "llvm.hexagon.V6.vconv.h.hf\0"
  "llvm.hexagon.V6.vconv.h.hf.128B\0"
  "llvm.hexagon.V6.vconv.h.hf.rnd\0"
  "llvm.hexagon.V6.vconv.h.hf.rnd.128B\0"
  "llvm.hexagon.V6.vconv.hf.h\0"
  "llvm.hexagon.V6.vconv.hf.h.128B\0"
  "llvm.hexagon.V6.vconv.hf.qf16\0"
  "llvm.hexagon.V6.vconv.hf.qf16.128B\0"
  "llvm.hexagon.V6.vconv.hf.qf32\0"
  "llvm.hexagon.V6.vconv.hf.qf32.128B\0"
  "llvm.hexagon.V6.vconv.qf16.f8\0"
  "llvm.hexagon.V6.vconv.qf16.f8.128B\0"
  "llvm.hexagon.V6.vconv.qf16.hf\0"
  "llvm.hexagon.V6.vconv.qf16.hf.128B\0"
  "llvm.hexagon.V6.vconv.qf16.qf16\0"
  "llvm.hexagon.V6.vconv.qf16.qf16.128B\0"
  "llvm.hexagon.V6.vconv.qf32.qf32\0"
  "llvm.hexagon.V6.vconv.qf32.qf32.128B\0"
  "llvm.hexagon.V6.vconv.qf32.sf\0"
  "llvm.hexagon.V6.vconv.qf32.sf.128B\0"
  "llvm.hexagon.V6.vconv.sf.qf32\0"
  "llvm.hexagon.V6.vconv.sf.qf32.128B\0"
  "llvm.hexagon.V6.vconv.sf.w\0"
  "llvm.hexagon.V6.vconv.sf.w.128B\0"
  "llvm.hexagon.V6.vconv.w.sf\0"
  "llvm.hexagon.V6.vconv.w.sf.128B\0"
  "llvm.hexagon.V6.vcvt.b.hf\0"
  "llvm.hexagon.V6.vcvt.b.hf.128B\0"
  "llvm.hexagon.V6.vcvt.bf.sf\0"
  "llvm.hexagon.V6.vcvt.bf.sf.128B\0"
  "llvm.hexagon.V6.vcvt.f8.hf\0"
  "llvm.hexagon.V6.vcvt.f8.hf.128B\0"
  "llvm.hexagon.V6.vcvt.h.hf\0"
  "llvm.hexagon.V6.vcvt.h.hf.128B\0"
  "llvm.hexagon.V6.vcvt.hf.b\0"
  "llvm.hexagon.V6.vcvt.hf.b.128B\0"
  "llvm.hexagon.V6.vcvt.hf.f8\0"
  "llvm.hexagon.V6.vcvt.hf.f8.128B\0"
  "llvm.hexagon.V6.vcvt.hf.h\0"
  "llvm.hexagon.V6.vcvt.hf.h.128B\0"
  "llvm.hexagon.V6.vcvt.hf.sf\0"
  "llvm.hexagon.V6.vcvt.hf.sf.128B\0"
  "llvm.hexagon.V6.vcvt.hf.ub\0"
  "llvm.hexagon.V6.vcvt.hf.ub.128B\0"
  "llvm.hexagon.V6.vcvt.hf.uh\0"
  "llvm.hexagon.V6.vcvt.hf.uh.128B\0"
  "llvm.hexagon.V6.vcvt.sf.hf\0"
  "llvm.hexagon.V6.vcvt.sf.hf.128B\0"
  "llvm.hexagon.V6.vcvt.ub.hf\0"
  "llvm.hexagon.V6.vcvt.ub.hf.128B\0"
  "llvm.hexagon.V6.vcvt.uh.hf\0"
  "llvm.hexagon.V6.vcvt.uh.hf.128B\0"
  "llvm.hexagon.V6.vcvt2.b.hf\0"
  "llvm.hexagon.V6.vcvt2.b.hf.128B\0"
  "llvm.hexagon.V6.vcvt2.hf.b\0"
  "llvm.hexagon.V6.vcvt2.hf.b.128B\0"
  "llvm.hexagon.V6.vcvt2.hf.ub\0"
  "llvm.hexagon.V6.vcvt2.hf.ub.128B\0"
  "llvm.hexagon.V6.vcvt2.ub.hf\0"
  "llvm.hexagon.V6.vcvt2.ub.hf.128B\0"
  "llvm.hexagon.V6.vd0\0"
  "llvm.hexagon.V6.vd0.128B\0"
  "llvm.hexagon.V6.vdd0\0"
  "llvm.hexagon.V6.vdd0.128B\0"
  "llvm.hexagon.V6.vdealb\0"
  "llvm.hexagon.V6.vdealb.128B\0"
  "llvm.hexagon.V6.vdealb4w\0"
  "llvm.hexagon.V6.vdealb4w.128B\0"
  "llvm.hexagon.V6.vdealh\0"
  "llvm.hexagon.V6.vdealh.128B\0"
  "llvm.hexagon.V6.vdealvdd\0"
  "llvm.hexagon.V6.vdealvdd.128B\0"
  "llvm.hexagon.V6.vdelta\0"
  "llvm.hexagon.V6.vdelta.128B\0"
  "llvm.hexagon.V6.vdmpy.sf.hf\0"
  "llvm.hexagon.V6.vdmpy.sf.hf.128B\0"
  "llvm.hexagon.V6.vdmpy.sf.hf.acc\0"
  "llvm.hexagon.V6.vdmpy.sf.hf.acc.128B\0"
  "llvm.hexagon.V6.vdmpybus\0"
  "llvm.hexagon.V6.vdmpybus.128B\0"
  "llvm.hexagon.V6.vdmpybus.acc\0"
  "llvm.hexagon.V6.vdmpybus.acc.128B\0"
  "llvm.hexagon.V6.vdmpybus.dv\0"
  "llvm.hexagon.V6.vdmpybus.dv.128B\0"
  "llvm.hexagon.V6.vdmpybus.dv.acc\0"
  "llvm.hexagon.V6.vdmpybus.dv.acc.128B\0"
  "llvm.hexagon.V6.vdmpyhb\0"
  "llvm.hexagon.V6.vdmpyhb.128B\0"
  "llvm.hexagon.V6.vdmpyhb.acc\0"
  "llvm.hexagon.V6.vdmpyhb.acc.128B\0"
  "llvm.hexagon.V6.vdmpyhb.dv\0"
  "llvm.hexagon.V6.vdmpyhb.dv.128B\0"
  "llvm.hexagon.V6.vdmpyhb.dv.acc\0"
  "llvm.hexagon.V6.vdmpyhb.dv.acc.128B\0"
  "llvm.hexagon.V6.vdmpyhisat\0"
  "llvm.hexagon.V6.vdmpyhisat.128B\0"
  "llvm.hexagon.V6.vdmpyhisat.acc\0"
  "llvm.hexagon.V6.vdmpyhisat.acc.128B\0"
  "llvm.hexagon.V6.vdmpyhsat\0"
  "llvm.hexagon.V6.vdmpyhsat.128B\0"
  "llvm.hexagon.V6.vdmpyhsat.acc\0"
  "llvm.hexagon.V6.vdmpyhsat.acc.128B\0"
  "llvm.hexagon.V6.vdmpyhsuisat\0"
  "llvm.hexagon.V6.vdmpyhsuisat.128B\0"
  "llvm.hexagon.V6.vdmpyhsuisat.acc\0"
  "llvm.hexagon.V6.vdmpyhsuisat.acc.128B\0"
  "llvm.hexagon.V6.vdmpyhsusat\0"
  "llvm.hexagon.V6.vdmpyhsusat.128B\0"
  "llvm.hexagon.V6.vdmpyhsusat.acc\0"
  "llvm.hexagon.V6.vdmpyhsusat.acc.128B\0"
  "llvm.hexagon.V6.vdmpyhvsat\0"
  "llvm.hexagon.V6.vdmpyhvsat.128B\0"
  "llvm.hexagon.V6.vdmpyhvsat.acc\0"
  "llvm.hexagon.V6.vdmpyhvsat.acc.128B\0"
  "llvm.hexagon.V6.vdsaduh\0"
  "llvm.hexagon.V6.vdsaduh.128B\0"
  "llvm.hexagon.V6.vdsaduh.acc\0"
  "llvm.hexagon.V6.vdsaduh.acc.128B\0"
  "llvm.hexagon.V6.veqb\0"
  "llvm.hexagon.V6.veqb.128B\0"
  "llvm.hexagon.V6.veqb.and\0"
  "llvm.hexagon.V6.veqb.and.128B\0"
  "llvm.hexagon.V6.veqb.or\0"
  "llvm.hexagon.V6.veqb.or.128B\0"
  "llvm.hexagon.V6.veqb.xor\0"
  "llvm.hexagon.V6.veqb.xor.128B\0"
  "llvm.hexagon.V6.veqh\0"
  "llvm.hexagon.V6.veqh.128B\0"
  "llvm.hexagon.V6.veqh.and\0"
  "llvm.hexagon.V6.veqh.and.128B\0"
  "llvm.hexagon.V6.veqh.or\0"
  "llvm.hexagon.V6.veqh.or.128B\0"
  "llvm.hexagon.V6.veqh.xor\0"
  "llvm.hexagon.V6.veqh.xor.128B\0"
  "llvm.hexagon.V6.veqhf\0"
  "llvm.hexagon.V6.veqhf.128B\0"
  "llvm.hexagon.V6.veqhf.and\0"
  "llvm.hexagon.V6.veqhf.and.128B\0"
  "llvm.hexagon.V6.veqhf.or\0"
  "llvm.hexagon.V6.veqhf.or.128B\0"
  "llvm.hexagon.V6.veqhf.xor\0"
  "llvm.hexagon.V6.veqhf.xor.128B\0"
  "llvm.hexagon.V6.veqsf\0"
  "llvm.hexagon.V6.veqsf.128B\0"
  "llvm.hexagon.V6.veqsf.and\0"
  "llvm.hexagon.V6.veqsf.and.128B\0"
  "llvm.hexagon.V6.veqsf.or\0"
  "llvm.hexagon.V6.veqsf.or.128B\0"
  "llvm.hexagon.V6.veqsf.xor\0"
  "llvm.hexagon.V6.veqsf.xor.128B\0"
  "llvm.hexagon.V6.veqw\0"
  "llvm.hexagon.V6.veqw.128B\0"
  "llvm.hexagon.V6.veqw.and\0"
  "llvm.hexagon.V6.veqw.and.128B\0"
  "llvm.hexagon.V6.veqw.or\0"
  "llvm.hexagon.V6.veqw.or.128B\0"
  "llvm.hexagon.V6.veqw.xor\0"
  "llvm.hexagon.V6.veqw.xor.128B\0"
  "llvm.hexagon.V6.vfmax.f8\0"
  "llvm.hexagon.V6.vfmax.f8.128B\0"
  "llvm.hexagon.V6.vfmax.hf\0"
  "llvm.hexagon.V6.vfmax.hf.128B\0"
  "llvm.hexagon.V6.vfmax.sf\0"
  "llvm.hexagon.V6.vfmax.sf.128B\0"
  "llvm.hexagon.V6.vfmin.f8\0"
  "llvm.hexagon.V6.vfmin.f8.128B\0"
  "llvm.hexagon.V6.vfmin.hf\0"
  "llvm.hexagon.V6.vfmin.hf.128B\0"
  "llvm.hexagon.V6.vfmin.sf\0"
  "llvm.hexagon.V6.vfmin.sf.128B\0"
  "llvm.hexagon.V6.vfneg.f8\0"
  "llvm.hexagon.V6.vfneg.f8.128B\0"
  "llvm.hexagon.V6.vfneg.hf\0"
  "llvm.hexagon.V6.vfneg.hf.128B\0"
  "llvm.hexagon.V6.vfneg.sf\0"
  "llvm.hexagon.V6.vfneg.sf.128B\0"
  "llvm.hexagon.V6.vgather.vscattermh\0"
  "llvm.hexagon.V6.vgather.vscattermh.128B\0"
  "llvm.hexagon.V6.vgathermh\0"
  "llvm.hexagon.V6.vgathermh.128B\0"
  "llvm.hexagon.V6.vgathermhq\0"
  "llvm.hexagon.V6.vgathermhq.128B\0"
  "llvm.hexagon.V6.vgathermhw\0"
  "llvm.hexagon.V6.vgathermhw.128B\0"
  "llvm.hexagon.V6.vgathermhwq\0"
  "llvm.hexagon.V6.vgathermhwq.128B\0"
  "llvm.hexagon.V6.vgathermw\0"
  "llvm.hexagon.V6.vgathermw.128B\0"
  "llvm.hexagon.V6.vgathermwq\0"
  "llvm.hexagon.V6.vgathermwq.128B\0"
  "llvm.hexagon.V6.vgtb\0"
  "llvm.hexagon.V6.vgtb.128B\0"
  "llvm.hexagon.V6.vgtb.and\0"
  "llvm.hexagon.V6.vgtb.and.128B\0"
  "llvm.hexagon.V6.vgtb.or\0"
  "llvm.hexagon.V6.vgtb.or.128B\0"
  "llvm.hexagon.V6.vgtb.xor\0"
  "llvm.hexagon.V6.vgtb.xor.128B\0"
  "llvm.hexagon.V6.vgtbf\0"
  "llvm.hexagon.V6.vgtbf.128B\0"
  "llvm.hexagon.V6.vgtbf.and\0"
  "llvm.hexagon.V6.vgtbf.and.128B\0"
  "llvm.hexagon.V6.vgtbf.or\0"
  "llvm.hexagon.V6.vgtbf.or.128B\0"
  "llvm.hexagon.V6.vgtbf.xor\0"
  "llvm.hexagon.V6.vgtbf.xor.128B\0"
  "llvm.hexagon.V6.vgth\0"
  "llvm.hexagon.V6.vgth.128B\0"
  "llvm.hexagon.V6.vgth.and\0"
  "llvm.hexagon.V6.vgth.and.128B\0"
  "llvm.hexagon.V6.vgth.or\0"
  "llvm.hexagon.V6.vgth.or.128B\0"
  "llvm.hexagon.V6.vgth.xor\0"
  "llvm.hexagon.V6.vgth.xor.128B\0"
  "llvm.hexagon.V6.vgthf\0"
  "llvm.hexagon.V6.vgthf.128B\0"
  "llvm.hexagon.V6.vgthf.and\0"
  "llvm.hexagon.V6.vgthf.and.128B\0"
  "llvm.hexagon.V6.vgthf.or\0"
  "llvm.hexagon.V6.vgthf.or.128B\0"
  "llvm.hexagon.V6.vgthf.xor\0"
  "llvm.hexagon.V6.vgthf.xor.128B\0"
  "llvm.hexagon.V6.vgtsf\0"
  "llvm.hexagon.V6.vgtsf.128B\0"
  "llvm.hexagon.V6.vgtsf.and\0"
  "llvm.hexagon.V6.vgtsf.and.128B\0"
  "llvm.hexagon.V6.vgtsf.or\0"
  "llvm.hexagon.V6.vgtsf.or.128B\0"
  "llvm.hexagon.V6.vgtsf.xor\0"
  "llvm.hexagon.V6.vgtsf.xor.128B\0"
  "llvm.hexagon.V6.vgtub\0"
  "llvm.hexagon.V6.vgtub.128B\0"
  "llvm.hexagon.V6.vgtub.and\0"
  "llvm.hexagon.V6.vgtub.and.128B\0"
  "llvm.hexagon.V6.vgtub.or\0"
  "llvm.hexagon.V6.vgtub.or.128B\0"
  "llvm.hexagon.V6.vgtub.xor\0"
  "llvm.hexagon.V6.vgtub.xor.128B\0"
  "llvm.hexagon.V6.vgtuh\0"
  "llvm.hexagon.V6.vgtuh.128B\0"
  "llvm.hexagon.V6.vgtuh.and\0"
  "llvm.hexagon.V6.vgtuh.and.128B\0"
  "llvm.hexagon.V6.vgtuh.or\0"
  "llvm.hexagon.V6.vgtuh.or.128B\0"
  "llvm.hexagon.V6.vgtuh.xor\0"
  "llvm.hexagon.V6.vgtuh.xor.128B\0"
  "llvm.hexagon.V6.vgtuw\0"
  "llvm.hexagon.V6.vgtuw.128B\0"
  "llvm.hexagon.V6.vgtuw.and\0"
  "llvm.hexagon.V6.vgtuw.and.128B\0"
  "llvm.hexagon.V6.vgtuw.or\0"
  "llvm.hexagon.V6.vgtuw.or.128B\0"
  "llvm.hexagon.V6.vgtuw.xor\0"
  "llvm.hexagon.V6.vgtuw.xor.128B\0"
  "llvm.hexagon.V6.vgtw\0"
  "llvm.hexagon.V6.vgtw.128B\0"
  "llvm.hexagon.V6.vgtw.and\0"
  "llvm.hexagon.V6.vgtw.and.128B\0"
  "llvm.hexagon.V6.vgtw.or\0"
  "llvm.hexagon.V6.vgtw.or.128B\0"
  "llvm.hexagon.V6.vgtw.xor\0"
  "llvm.hexagon.V6.vgtw.xor.128B\0"
  "llvm.hexagon.V6.vilog2.hf\0"
  "llvm.hexagon.V6.vilog2.hf.128B\0"
  "llvm.hexagon.V6.vilog2.qf16\0"
  "llvm.hexagon.V6.vilog2.qf16.128B\0"
  "llvm.hexagon.V6.vilog2.qf32\0"
  "llvm.hexagon.V6.vilog2.qf32.128B\0"
  "llvm.hexagon.V6.vilog2.sf\0"
  "llvm.hexagon.V6.vilog2.sf.128B\0"
  "llvm.hexagon.V6.vinsertwr\0"
  "llvm.hexagon.V6.vinsertwr.128B\0"
  "llvm.hexagon.V6.vlalignb\0"
  "llvm.hexagon.V6.vlalignb.128B\0"
  "llvm.hexagon.V6.vlalignbi\0"
  "llvm.hexagon.V6.vlalignbi.128B\0"
  "llvm.hexagon.V6.vlsrb\0"
  "llvm.hexagon.V6.vlsrb.128B\0"
  "llvm.hexagon.V6.vlsrh\0"
  "llvm.hexagon.V6.vlsrh.128B\0"
  "llvm.hexagon.V6.vlsrhv\0"
  "llvm.hexagon.V6.vlsrhv.128B\0"
  "llvm.hexagon.V6.vlsrw\0"
  "llvm.hexagon.V6.vlsrw.128B\0"
  "llvm.hexagon.V6.vlsrwv\0"
  "llvm.hexagon.V6.vlsrwv.128B\0"
  "llvm.hexagon.V6.vlut4\0"
  "llvm.hexagon.V6.vlut4.128B\0"
  "llvm.hexagon.V6.vlutvvb\0"
  "llvm.hexagon.V6.vlutvvb.128B\0"
  "llvm.hexagon.V6.vlutvvb.nm\0"
  "llvm.hexagon.V6.vlutvvb.nm.128B\0"
  "llvm.hexagon.V6.vlutvvb.oracc\0"
  "llvm.hexagon.V6.vlutvvb.oracc.128B\0"
  "llvm.hexagon.V6.vlutvvb.oracci\0"
  "llvm.hexagon.V6.vlutvvb.oracci.128B\0"
  "llvm.hexagon.V6.vlutvvbi\0"
  "llvm.hexagon.V6.vlutvvbi.128B\0"
  "llvm.hexagon.V6.vlutvwh\0"
  "llvm.hexagon.V6.vlutvwh.128B\0"
  "llvm.hexagon.V6.vlutvwh.nm\0"
  "llvm.hexagon.V6.vlutvwh.nm.128B\0"
  "llvm.hexagon.V6.vlutvwh.oracc\0"
  "llvm.hexagon.V6.vlutvwh.oracc.128B\0"
  "llvm.hexagon.V6.vlutvwh.oracci\0"
  "llvm.hexagon.V6.vlutvwh.oracci.128B\0"
  "llvm.hexagon.V6.vlutvwhi\0"
  "llvm.hexagon.V6.vlutvwhi.128B\0"
  "llvm.hexagon.V6.vmaskedstorenq\0"
  "llvm.hexagon.V6.vmaskedstorenq.128B\0"
  "llvm.hexagon.V6.vmaskedstorentnq\0"
  "llvm.hexagon.V6.vmaskedstorentnq.128B\0"
  "llvm.hexagon.V6.vmaskedstorentq\0"
  "llvm.hexagon.V6.vmaskedstorentq.128B\0"
  "llvm.hexagon.V6.vmaskedstoreq\0"
  "llvm.hexagon.V6.vmaskedstoreq.128B\0"
  "llvm.hexagon.V6.vmax.bf\0"
  "llvm.hexagon.V6.vmax.bf.128B\0"
  "llvm.hexagon.V6.vmax.hf\0"
  "llvm.hexagon.V6.vmax.hf.128B\0"
  "llvm.hexagon.V6.vmax.sf\0"
  "llvm.hexagon.V6.vmax.sf.128B\0"
  "llvm.hexagon.V6.vmaxb\0"
  "llvm.hexagon.V6.vmaxb.128B\0"
  "llvm.hexagon.V6.vmaxh\0"
  "llvm.hexagon.V6.vmaxh.128B\0"
  "llvm.hexagon.V6.vmaxub\0"
  "llvm.hexagon.V6.vmaxub.128B\0"
  "llvm.hexagon.V6.vmaxuh\0"
  "llvm.hexagon.V6.vmaxuh.128B\0"
  "llvm.hexagon.V6.vmaxw\0"
  "llvm.hexagon.V6.vmaxw.128B\0"
  "llvm.hexagon.V6.vmerge.qf\0"
  "llvm.hexagon.V6.vmerge.qf.128B\0"
  "llvm.hexagon.V6.vmin.bf\0"
  "llvm.hexagon.V6.vmin.bf.128B\0"
  "llvm.hexagon.V6.vmin.hf\0"
  "llvm.hexagon.V6.vmin.hf.128B\0"
  "llvm.hexagon.V6.vmin.sf\0"
  "llvm.hexagon.V6.vmin.sf.128B\0"
  "llvm.hexagon.V6.vminb\0"
  "llvm.hexagon.V6.vminb.128B\0"
  "llvm.hexagon.V6.vminh\0"
  "llvm.hexagon.V6.vminh.128B\0"
  "llvm.hexagon.V6.vminub\0"
  "llvm.hexagon.V6.vminub.128B\0"
  "llvm.hexagon.V6.vminuh\0"
  "llvm.hexagon.V6.vminuh.128B\0"
  "llvm.hexagon.V6.vminw\0"
  "llvm.hexagon.V6.vminw.128B\0"
  "llvm.hexagon.V6.vmpabus\0"
  "llvm.hexagon.V6.vmpabus.128B\0"
  "llvm.hexagon.V6.vmpabus.acc\0"
  "llvm.hexagon.V6.vmpabus.acc.128B\0"
  "llvm.hexagon.V6.vmpabusv\0"
  "llvm.hexagon.V6.vmpabusv.128B\0"
  "llvm.hexagon.V6.vmpabuu\0"
  "llvm.hexagon.V6.vmpabuu.128B\0"
  "llvm.hexagon.V6.vmpabuu.acc\0"
  "llvm.hexagon.V6.vmpabuu.acc.128B\0"
  "llvm.hexagon.V6.vmpabuuv\0"
  "llvm.hexagon.V6.vmpabuuv.128B\0"
  "llvm.hexagon.V6.vmpahb\0"
  "llvm.hexagon.V6.vmpahb.128B\0"
  "llvm.hexagon.V6.vmpahb.acc\0"
  "llvm.hexagon.V6.vmpahb.acc.128B\0"
  "llvm.hexagon.V6.vmpahhsat\0"
  "llvm.hexagon.V6.vmpahhsat.128B\0"
  "llvm.hexagon.V6.vmpauhb\0"
  "llvm.hexagon.V6.vmpauhb.128B\0"
  "llvm.hexagon.V6.vmpauhb.acc\0"
  "llvm.hexagon.V6.vmpauhb.acc.128B\0"
  "llvm.hexagon.V6.vmpauhuhsat\0"
  "llvm.hexagon.V6.vmpauhuhsat.128B\0"
  "llvm.hexagon.V6.vmpsuhuhsat\0"
  "llvm.hexagon.V6.vmpsuhuhsat.128B\0"
  "llvm.hexagon.V6.vmpy.hf.f8\0"
  "llvm.hexagon.V6.vmpy.hf.f8.128B\0"
  "llvm.hexagon.V6.vmpy.hf.f8.acc\0"
  "llvm.hexagon.V6.vmpy.hf.f8.acc.128B\0"
  "llvm.hexagon.V6.vmpy.hf.hf\0"
  "llvm.hexagon.V6.vmpy.hf.hf.128B\0"
  "llvm.hexagon.V6.vmpy.hf.hf.acc\0"
  "llvm.hexagon.V6.vmpy.hf.hf.acc.128B\0"
  "llvm.hexagon.V6.vmpy.qf16\0"
  "llvm.hexagon.V6.vmpy.qf16.128B\0"
  "llvm.hexagon.V6.vmpy.qf16.hf\0"
  "llvm.hexagon.V6.vmpy.qf16.hf.128B\0"
  "llvm.hexagon.V6.vmpy.qf16.mix.hf\0"
  "llvm.hexagon.V6.vmpy.qf16.mix.hf.128B\0"
  "llvm.hexagon.V6.vmpy.qf32\0"
  "llvm.hexagon.V6.vmpy.qf32.128B\0"
  "llvm.hexagon.V6.vmpy.qf32.hf\0"
  "llvm.hexagon.V6.vmpy.qf32.hf.128B\0"
  "llvm.hexagon.V6.vmpy.qf32.mix.hf\0"
  "llvm.hexagon.V6.vmpy.qf32.mix.hf.128B\0"
  "llvm.hexagon.V6.vmpy.qf32.qf16\0"
  "llvm.hexagon.V6.vmpy.qf32.qf16.128B\0"
  "llvm.hexagon.V6.vmpy.qf32.sf\0"
  "llvm.hexagon.V6.vmpy.qf32.sf.128B\0"
  "llvm.hexagon.V6.vmpy.rt.hf\0"
  "llvm.hexagon.V6.vmpy.rt.hf.128B\0"
  "llvm.hexagon.V6.vmpy.rt.qf16\0"
  "llvm.hexagon.V6.vmpy.rt.qf16.128B\0"
  "llvm.hexagon.V6.vmpy.rt.sf\0"
  "llvm.hexagon.V6.vmpy.rt.sf.128B\0"
  "llvm.hexagon.V6.vmpy.sf.bf\0"
  "llvm.hexagon.V6.vmpy.sf.bf.128B\0"
  "llvm.hexagon.V6.vmpy.sf.bf.acc\0"
  "llvm.hexagon.V6.vmpy.sf.bf.acc.128B\0"
  "llvm.hexagon.V6.vmpy.sf.hf\0"
  "llvm.hexagon.V6.vmpy.sf.hf.128B\0"
  "llvm.hexagon.V6.vmpy.sf.hf.acc\0"
  "llvm.hexagon.V6.vmpy.sf.hf.acc.128B\0"
  "llvm.hexagon.V6.vmpy.sf.sf\0"
  "llvm.hexagon.V6.vmpy.sf.sf.128B\0"
  "llvm.hexagon.V6.vmpybus\0"
  "llvm.hexagon.V6.vmpybus.128B\0"
  "llvm.hexagon.V6.vmpybus.acc\0"
  "llvm.hexagon.V6.vmpybus.acc.128B\0"
  "llvm.hexagon.V6.vmpybusv\0"
  "llvm.hexagon.V6.vmpybusv.128B\0"
  "llvm.hexagon.V6.vmpybusv.acc\0"
  "llvm.hexagon.V6.vmpybusv.acc.128B\0"
  "llvm.hexagon.V6.vmpybv\0"
  "llvm.hexagon.V6.vmpybv.128B\0"
  "llvm.hexagon.V6.vmpybv.acc\0"
  "llvm.hexagon.V6.vmpybv.acc.128B\0"
  "llvm.hexagon.V6.vmpyewuh\0"
  "llvm.hexagon.V6.vmpyewuh.128B\0"
  "llvm.hexagon.V6.vmpyewuh.64\0"
  "llvm.hexagon.V6.vmpyewuh.64.128B\0"
  "llvm.hexagon.V6.vmpyh\0"
  "llvm.hexagon.V6.vmpyh.128B\0"
  "llvm.hexagon.V6.vmpyh.acc\0"
  "llvm.hexagon.V6.vmpyh.acc.128B\0"
  "llvm.hexagon.V6.vmpyhsat.acc\0"
  "llvm.hexagon.V6.vmpyhsat.acc.128B\0"
  "llvm.hexagon.V6.vmpyhsrs\0"
  "llvm.hexagon.V6.vmpyhsrs.128B\0"
  "llvm.hexagon.V6.vmpyhss\0"
  "llvm.hexagon.V6.vmpyhss.128B\0"
  "llvm.hexagon.V6.vmpyhus\0"
  "llvm.hexagon.V6.vmpyhus.128B\0"
  "llvm.hexagon.V6.vmpyhus.acc\0"
  "llvm.hexagon.V6.vmpyhus.acc.128B\0"
  "llvm.hexagon.V6.vmpyhv\0"
  "llvm.hexagon.V6.vmpyhv.128B\0"
  "llvm.hexagon.V6.vmpyhv.acc\0"
  "llvm.hexagon.V6.vmpyhv.acc.128B\0"
  "llvm.hexagon.V6.vmpyhvsrs\0"
  "llvm.hexagon.V6.vmpyhvsrs.128B\0"
  "llvm.hexagon.V6.vmpyieoh\0"
  "llvm.hexagon.V6.vmpyieoh.128B\0"
  "llvm.hexagon.V6.vmpyiewh.acc\0"
  "llvm.hexagon.V6.vmpyiewh.acc.128B\0"
  "llvm.hexagon.V6.vmpyiewuh\0"
  "llvm.hexagon.V6.vmpyiewuh.128B\0"
  "llvm.hexagon.V6.vmpyiewuh.acc\0"
  "llvm.hexagon.V6.vmpyiewuh.acc.128B\0"
  "llvm.hexagon.V6.vmpyih\0"
  "llvm.hexagon.V6.vmpyih.128B\0"
  "llvm.hexagon.V6.vmpyih.acc\0"
  "llvm.hexagon.V6.vmpyih.acc.128B\0"
  "llvm.hexagon.V6.vmpyihb\0"
  "llvm.hexagon.V6.vmpyihb.128B\0"
  "llvm.hexagon.V6.vmpyihb.acc\0"
  "llvm.hexagon.V6.vmpyihb.acc.128B\0"
  "llvm.hexagon.V6.vmpyiowh\0"
  "llvm.hexagon.V6.vmpyiowh.128B\0"
  "llvm.hexagon.V6.vmpyiwb\0"
  "llvm.hexagon.V6.vmpyiwb.128B\0"
  "llvm.hexagon.V6.vmpyiwb.acc\0"
  "llvm.hexagon.V6.vmpyiwb.acc.128B\0"
  "llvm.hexagon.V6.vmpyiwh\0"
  "llvm.hexagon.V6.vmpyiwh.128B\0"
  "llvm.hexagon.V6.vmpyiwh.acc\0"
  "llvm.hexagon.V6.vmpyiwh.acc.128B\0"
  "llvm.hexagon.V6.vmpyiwub\0"
  "llvm.hexagon.V6.vmpyiwub.128B\0"
  "llvm.hexagon.V6.vmpyiwub.acc\0"
  "llvm.hexagon.V6.vmpyiwub.acc.128B\0"
  "llvm.hexagon.V6.vmpyowh\0"
  "llvm.hexagon.V6.vmpyowh.128B\0"
  "llvm.hexagon.V6.vmpyowh.64.acc\0"
  "llvm.hexagon.V6.vmpyowh.64.acc.128B\0"
  "llvm.hexagon.V6.vmpyowh.rnd\0"
  "llvm.hexagon.V6.vmpyowh.rnd.128B\0"
  "llvm.hexagon.V6.vmpyowh.rnd.sacc\0"
  "llvm.hexagon.V6.vmpyowh.rnd.sacc.128B\0"
  "llvm.hexagon.V6.vmpyowh.sacc\0"
  "llvm.hexagon.V6.vmpyowh.sacc.128B\0"
  "llvm.hexagon.V6.vmpyss.parts\0"
  "llvm.hexagon.V6.vmpyss.parts.128B\0"
  "llvm.hexagon.V6.vmpyub\0"
  "llvm.hexagon.V6.vmpyub.128B\0"
  "llvm.hexagon.V6.vmpyub.acc\0"
  "llvm.hexagon.V6.vmpyub.acc.128B\0"
  "llvm.hexagon.V6.vmpyubv\0"
  "llvm.hexagon.V6.vmpyubv.128B\0"
  "llvm.hexagon.V6.vmpyubv.acc\0"
  "llvm.hexagon.V6.vmpyubv.acc.128B\0"
  "llvm.hexagon.V6.vmpyuh\0"
  "llvm.hexagon.V6.vmpyuh.128B\0"
  "llvm.hexagon.V6.vmpyuh.acc\0"
  "llvm.hexagon.V6.vmpyuh.acc.128B\0"
  "llvm.hexagon.V6.vmpyuhe\0"
  "llvm.hexagon.V6.vmpyuhe.128B\0"
  "llvm.hexagon.V6.vmpyuhe.acc\0"
  "llvm.hexagon.V6.vmpyuhe.acc.128B\0"
  "llvm.hexagon.V6.vmpyuhv\0"
  "llvm.hexagon.V6.vmpyuhv.128B\0"
  "llvm.hexagon.V6.vmpyuhv.acc\0"
  "llvm.hexagon.V6.vmpyuhv.acc.128B\0"
  "llvm.hexagon.V6.vmpyuhvs\0"
  "llvm.hexagon.V6.vmpyuhvs.128B\0"
  "llvm.hexagon.V6.vmpyus.parts\0"
  "llvm.hexagon.V6.vmpyus.parts.128B\0"
  "llvm.hexagon.V6.vmpyuu.parts\0"
  "llvm.hexagon.V6.vmpyuu.parts.128B\0"
  "llvm.hexagon.V6.vmux\0"
  "llvm.hexagon.V6.vmux.128B\0"
  "llvm.hexagon.V6.vnavgb\0"
  "llvm.hexagon.V6.vnavgb.128B\0"
  "llvm.hexagon.V6.vnavgh\0"
  "llvm.hexagon.V6.vnavgh.128B\0"
  "llvm.hexagon.V6.vnavgub\0"
  "llvm.hexagon.V6.vnavgub.128B\0"
  "llvm.hexagon.V6.vnavgw\0"
  "llvm.hexagon.V6.vnavgw.128B\0"
  "llvm.hexagon.V6.vneg.qf16.hf\0"
  "llvm.hexagon.V6.vneg.qf16.hf.128B\0"
  "llvm.hexagon.V6.vneg.qf16.qf16\0"
  "llvm.hexagon.V6.vneg.qf16.qf16.128B\0"
  "llvm.hexagon.V6.vneg.qf32.qf32\0"
  "llvm.hexagon.V6.vneg.qf32.qf32.128B\0"
  "llvm.hexagon.V6.vneg.qf32.sf\0"
  "llvm.hexagon.V6.vneg.qf32.sf.128B\0"
  "llvm.hexagon.V6.vnormamth\0"
  "llvm.hexagon.V6.vnormamth.128B\0"
  "llvm.hexagon.V6.vnormamtw\0"
  "llvm.hexagon.V6.vnormamtw.128B\0"
  "llvm.hexagon.V6.vnot\0"
  "llvm.hexagon.V6.vnot.128B\0"
  "llvm.hexagon.V6.vor\0"
  "llvm.hexagon.V6.vor.128B\0"
  "llvm.hexagon.V6.vpackeb\0"
  "llvm.hexagon.V6.vpackeb.128B\0"
  "llvm.hexagon.V6.vpackeh\0"
  "llvm.hexagon.V6.vpackeh.128B\0"
  "llvm.hexagon.V6.vpackhb.sat\0"
  "llvm.hexagon.V6.vpackhb.sat.128B\0"
  "llvm.hexagon.V6.vpackhub.sat\0"
  "llvm.hexagon.V6.vpackhub.sat.128B\0"
  "llvm.hexagon.V6.vpackob\0"
  "llvm.hexagon.V6.vpackob.128B\0"
  "llvm.hexagon.V6.vpackoh\0"
  "llvm.hexagon.V6.vpackoh.128B\0"
  "llvm.hexagon.V6.vpackwh.sat\0"
  "llvm.hexagon.V6.vpackwh.sat.128B\0"
  "llvm.hexagon.V6.vpackwuh.sat\0"
  "llvm.hexagon.V6.vpackwuh.sat.128B\0"
  "llvm.hexagon.V6.vpopcounth\0"
  "llvm.hexagon.V6.vpopcounth.128B\0"
  "llvm.hexagon.V6.vprefixqb\0"
  "llvm.hexagon.V6.vprefixqb.128B\0"
  "llvm.hexagon.V6.vprefixqh\0"
  "llvm.hexagon.V6.vprefixqh.128B\0"
  "llvm.hexagon.V6.vprefixqw\0"
  "llvm.hexagon.V6.vprefixqw.128B\0"
  "llvm.hexagon.V6.vrdelta\0"
  "llvm.hexagon.V6.vrdelta.128B\0"
  "llvm.hexagon.V6.vrmpybub.rtt\0"
  "llvm.hexagon.V6.vrmpybub.rtt.128B\0"
  "llvm.hexagon.V6.vrmpybub.rtt.acc\0"
  "llvm.hexagon.V6.vrmpybub.rtt.acc.128B\0"
  "llvm.hexagon.V6.vrmpybus\0"
  "llvm.hexagon.V6.vrmpybus.128B\0"
  "llvm.hexagon.V6.vrmpybus.acc\0"
  "llvm.hexagon.V6.vrmpybus.acc.128B\0"
  "llvm.hexagon.V6.vrmpybusi\0"
  "llvm.hexagon.V6.vrmpybusi.128B\0"
  "llvm.hexagon.V6.vrmpybusi.acc\0"
  "llvm.hexagon.V6.vrmpybusi.acc.128B\0"
  "llvm.hexagon.V6.vrmpybusv\0"
  "llvm.hexagon.V6.vrmpybusv.128B\0"
  "llvm.hexagon.V6.vrmpybusv.acc\0"
  "llvm.hexagon.V6.vrmpybusv.acc.128B\0"
  "llvm.hexagon.V6.vrmpybv\0"
  "llvm.hexagon.V6.vrmpybv.128B\0"
  "llvm.hexagon.V6.vrmpybv.acc\0"
  "llvm.hexagon.V6.vrmpybv.acc.128B\0"
  "llvm.hexagon.V6.vrmpyub\0"
  "llvm.hexagon.V6.vrmpyub.128B\0"
  "llvm.hexagon.V6.vrmpyub.acc\0"
  "llvm.hexagon.V6.vrmpyub.acc.128B\0"
  "llvm.hexagon.V6.vrmpyub.rtt\0"
  "llvm.hexagon.V6.vrmpyub.rtt.128B\0"
  "llvm.hexagon.V6.vrmpyub.rtt.acc\0"
  "llvm.hexagon.V6.vrmpyub.rtt.acc.128B\0"
  "llvm.hexagon.V6.vrmpyubi\0"
  "llvm.hexagon.V6.vrmpyubi.128B\0"
  "llvm.hexagon.V6.vrmpyubi.acc\0"
  "llvm.hexagon.V6.vrmpyubi.acc.128B\0"
  "llvm.hexagon.V6.vrmpyubv\0"
  "llvm.hexagon.V6.vrmpyubv.128B\0"
  "llvm.hexagon.V6.vrmpyubv.acc\0"
  "llvm.hexagon.V6.vrmpyubv.acc.128B\0"
  "llvm.hexagon.V6.vror\0"
  "llvm.hexagon.V6.vror.128B\0"
  "llvm.hexagon.V6.vrotr\0"
  "llvm.hexagon.V6.vrotr.128B\0"
  "llvm.hexagon.V6.vroundhb\0"
  "llvm.hexagon.V6.vroundhb.128B\0"
  "llvm.hexagon.V6.vroundhub\0"
  "llvm.hexagon.V6.vroundhub.128B\0"
  "llvm.hexagon.V6.vrounduhub\0"
  "llvm.hexagon.V6.vrounduhub.128B\0"
  "llvm.hexagon.V6.vrounduwuh\0"
  "llvm.hexagon.V6.vrounduwuh.128B\0"
  "llvm.hexagon.V6.vroundwh\0"
  "llvm.hexagon.V6.vroundwh.128B\0"
  "llvm.hexagon.V6.vroundwuh\0"
  "llvm.hexagon.V6.vroundwuh.128B\0"
  "llvm.hexagon.V6.vrsadubi\0"
  "llvm.hexagon.V6.vrsadubi.128B\0"
  "llvm.hexagon.V6.vrsadubi.acc\0"
  "llvm.hexagon.V6.vrsadubi.acc.128B\0"
  "llvm.hexagon.V6.vsatdw\0"
  "llvm.hexagon.V6.vsatdw.128B\0"
  "llvm.hexagon.V6.vsathub\0"
  "llvm.hexagon.V6.vsathub.128B\0"
  "llvm.hexagon.V6.vsatuwuh\0"
  "llvm.hexagon.V6.vsatuwuh.128B\0"
  "llvm.hexagon.V6.vsatwh\0"
  "llvm.hexagon.V6.vsatwh.128B\0"
  "llvm.hexagon.V6.vsb\0"
  "llvm.hexagon.V6.vsb.128B\0"
  "llvm.hexagon.V6.vscattermh\0"
  "llvm.hexagon.V6.vscattermh.128B\0"
  "llvm.hexagon.V6.vscattermh.add\0"
  "llvm.hexagon.V6.vscattermh.add.128B\0"
  "llvm.hexagon.V6.vscattermhq\0"
  "llvm.hexagon.V6.vscattermhq.128B\0"
  "llvm.hexagon.V6.vscattermhw\0"
  "llvm.hexagon.V6.vscattermhw.128B\0"
  "llvm.hexagon.V6.vscattermhw.add\0"
  "llvm.hexagon.V6.vscattermhw.add.128B\0"
  "llvm.hexagon.V6.vscattermhwq\0"
  "llvm.hexagon.V6.vscattermhwq.128B\0"
  "llvm.hexagon.V6.vscattermw\0"
  "llvm.hexagon.V6.vscattermw.128B\0"
  "llvm.hexagon.V6.vscattermw.add\0"
  "llvm.hexagon.V6.vscattermw.add.128B\0"
  "llvm.hexagon.V6.vscattermwq\0"
  "llvm.hexagon.V6.vscattermwq.128B\0"
  "llvm.hexagon.V6.vsh\0"
  "llvm.hexagon.V6.vsh.128B\0"
  "llvm.hexagon.V6.vshufeh\0"
  "llvm.hexagon.V6.vshufeh.128B\0"
  "llvm.hexagon.V6.vshuffb\0"
  "llvm.hexagon.V6.vshuffb.128B\0"
  "llvm.hexagon.V6.vshuffeb\0"
  "llvm.hexagon.V6.vshuffeb.128B\0"
  "llvm.hexagon.V6.vshuffh\0"
  "llvm.hexagon.V6.vshuffh.128B\0"
  "llvm.hexagon.V6.vshuffob\0"
  "llvm.hexagon.V6.vshuffob.128B\0"
  "llvm.hexagon.V6.vshuffvdd\0"
  "llvm.hexagon.V6.vshuffvdd.128B\0"
  "llvm.hexagon.V6.vshufoeb\0"
  "llvm.hexagon.V6.vshufoeb.128B\0"
  "llvm.hexagon.V6.vshufoeh\0"
  "llvm.hexagon.V6.vshufoeh.128B\0"
  "llvm.hexagon.V6.vshufoh\0"
  "llvm.hexagon.V6.vshufoh.128B\0"
  "llvm.hexagon.V6.vsub.hf\0"
  "llvm.hexagon.V6.vsub.hf.128B\0"
  "llvm.hexagon.V6.vsub.hf.f8\0"
  "llvm.hexagon.V6.vsub.hf.f8.128B\0"
  "llvm.hexagon.V6.vsub.hf.hf\0"
  "llvm.hexagon.V6.vsub.hf.hf.128B\0"
  "llvm.hexagon.V6.vsub.hf.mix\0"
  "llvm.hexagon.V6.vsub.hf.mix.128B\0"
  "llvm.hexagon.V6.vsub.qf16\0"
  "llvm.hexagon.V6.vsub.qf16.128B\0"
  "llvm.hexagon.V6.vsub.qf16.mix\0"
  "llvm.hexagon.V6.vsub.qf16.mix.128B\0"
  "llvm.hexagon.V6.vsub.qf32\0"
  "llvm.hexagon.V6.vsub.qf32.128B\0"
  "llvm.hexagon.V6.vsub.qf32.mix\0"
  "llvm.hexagon.V6.vsub.qf32.mix.128B\0"
  "llvm.hexagon.V6.vsub.sf\0"
  "llvm.hexagon.V6.vsub.sf.128B\0"
  "llvm.hexagon.V6.vsub.sf.bf\0"
  "llvm.hexagon.V6.vsub.sf.bf.128B\0"
  "llvm.hexagon.V6.vsub.sf.hf\0"
  "llvm.hexagon.V6.vsub.sf.hf.128B\0"
  "llvm.hexagon.V6.vsub.sf.mix\0"
  "llvm.hexagon.V6.vsub.sf.mix.128B\0"
  "llvm.hexagon.V6.vsub.sf.sf\0"
  "llvm.hexagon.V6.vsub.sf.sf.128B\0"
  "llvm.hexagon.V6.vsubb\0"
  "llvm.hexagon.V6.vsubb.128B\0"
  "llvm.hexagon.V6.vsubb.dv\0"
  "llvm.hexagon.V6.vsubb.dv.128B\0"
  "llvm.hexagon.V6.vsubbnq\0"
  "llvm.hexagon.V6.vsubbnq.128B\0"
  "llvm.hexagon.V6.vsubbq\0"
  "llvm.hexagon.V6.vsubbq.128B\0"
  "llvm.hexagon.V6.vsubbsat\0"
  "llvm.hexagon.V6.vsubbsat.128B\0"
  "llvm.hexagon.V6.vsubbsat.dv\0"
  "llvm.hexagon.V6.vsubbsat.dv.128B\0"
  "llvm.hexagon.V6.vsubcarry\0"
  "llvm.hexagon.V6.vsubcarry.128B\0"
  "llvm.hexagon.V6.vsubcarryo\0"
  "llvm.hexagon.V6.vsubcarryo.128B\0"
  "llvm.hexagon.V6.vsubh\0"
  "llvm.hexagon.V6.vsubh.128B\0"
  "llvm.hexagon.V6.vsubh.dv\0"
  "llvm.hexagon.V6.vsubh.dv.128B\0"
  "llvm.hexagon.V6.vsubhnq\0"
  "llvm.hexagon.V6.vsubhnq.128B\0"
  "llvm.hexagon.V6.vsubhq\0"
  "llvm.hexagon.V6.vsubhq.128B\0"
  "llvm.hexagon.V6.vsubhsat\0"
  "llvm.hexagon.V6.vsubhsat.128B\0"
  "llvm.hexagon.V6.vsubhsat.dv\0"
  "llvm.hexagon.V6.vsubhsat.dv.128B\0"
  "llvm.hexagon.V6.vsubhw\0"
  "llvm.hexagon.V6.vsubhw.128B\0"
  "llvm.hexagon.V6.vsububh\0"
  "llvm.hexagon.V6.vsububh.128B\0"
  "llvm.hexagon.V6.vsububsat\0"
  "llvm.hexagon.V6.vsububsat.128B\0"
  "llvm.hexagon.V6.vsububsat.dv\0"
  "llvm.hexagon.V6.vsububsat.dv.128B\0"
  "llvm.hexagon.V6.vsubububb.sat\0"
  "llvm.hexagon.V6.vsubububb.sat.128B\0"
  "llvm.hexagon.V6.vsubuhsat\0"
  "llvm.hexagon.V6.vsubuhsat.128B\0"
  "llvm.hexagon.V6.vsubuhsat.dv\0"
  "llvm.hexagon.V6.vsubuhsat.dv.128B\0"
  "llvm.hexagon.V6.vsubuhw\0"
  "llvm.hexagon.V6.vsubuhw.128B\0"
  "llvm.hexagon.V6.vsubuwsat\0"
  "llvm.hexagon.V6.vsubuwsat.128B\0"
  "llvm.hexagon.V6.vsubuwsat.dv\0"
  "llvm.hexagon.V6.vsubuwsat.dv.128B\0"
  "llvm.hexagon.V6.vsubw\0"
  "llvm.hexagon.V6.vsubw.128B\0"
  "llvm.hexagon.V6.vsubw.dv\0"
  "llvm.hexagon.V6.vsubw.dv.128B\0"
  "llvm.hexagon.V6.vsubwnq\0"
  "llvm.hexagon.V6.vsubwnq.128B\0"
  "llvm.hexagon.V6.vsubwq\0"
  "llvm.hexagon.V6.vsubwq.128B\0"
  "llvm.hexagon.V6.vsubwsat\0"
  "llvm.hexagon.V6.vsubwsat.128B\0"
  "llvm.hexagon.V6.vsubwsat.dv\0"
  "llvm.hexagon.V6.vsubwsat.dv.128B\0"
  "llvm.hexagon.V6.vswap\0"
  "llvm.hexagon.V6.vswap.128B\0"
  "llvm.hexagon.V6.vtmpyb\0"
  "llvm.hexagon.V6.vtmpyb.128B\0"
  "llvm.hexagon.V6.vtmpyb.acc\0"
  "llvm.hexagon.V6.vtmpyb.acc.128B\0"
  "llvm.hexagon.V6.vtmpybus\0"
  "llvm.hexagon.V6.vtmpybus.128B\0"
  "llvm.hexagon.V6.vtmpybus.acc\0"
  "llvm.hexagon.V6.vtmpybus.acc.128B\0"
  "llvm.hexagon.V6.vtmpyhb\0"
  "llvm.hexagon.V6.vtmpyhb.128B\0"
  "llvm.hexagon.V6.vtmpyhb.acc\0"
  "llvm.hexagon.V6.vtmpyhb.acc.128B\0"
  "llvm.hexagon.V6.vunpackb\0"
  "llvm.hexagon.V6.vunpackb.128B\0"
  "llvm.hexagon.V6.vunpackh\0"
  "llvm.hexagon.V6.vunpackh.128B\0"
  "llvm.hexagon.V6.vunpackob\0"
  "llvm.hexagon.V6.vunpackob.128B\0"
  "llvm.hexagon.V6.vunpackoh\0"
  "llvm.hexagon.V6.vunpackoh.128B\0"
  "llvm.hexagon.V6.vunpackub\0"
  "llvm.hexagon.V6.vunpackub.128B\0"
  "llvm.hexagon.V6.vunpackuh\0"
  "llvm.hexagon.V6.vunpackuh.128B\0"
  "llvm.hexagon.V6.vxor\0"
  "llvm.hexagon.V6.vxor.128B\0"
  "llvm.hexagon.V6.vzb\0"
  "llvm.hexagon.V6.vzb.128B\0"
  "llvm.hexagon.V6.vzh\0"
  "llvm.hexagon.V6.vzh.128B\0"
  "llvm.hexagon.Y2.dccleana\0"
  "llvm.hexagon.Y2.dccleaninva\0"
  "llvm.hexagon.Y2.dcfetch\0"
  "llvm.hexagon.Y2.dcinva\0"
  "llvm.hexagon.Y2.dczeroa\0"
  "llvm.hexagon.Y4.l2fetch\0"
  "llvm.hexagon.Y5.l2fetch\0"
  "llvm.hexagon.Y6.dmlink\0"
  "llvm.hexagon.Y6.dmpause\0"
  "llvm.hexagon.Y6.dmpoll\0"
  "llvm.hexagon.Y6.dmresume\0"
  "llvm.hexagon.Y6.dmstart\0"
  "llvm.hexagon.Y6.dmwait\0"
  "llvm.hexagon.circ.ldb\0"
  "llvm.hexagon.circ.ldd\0"
  "llvm.hexagon.circ.ldh\0"
  "llvm.hexagon.circ.ldub\0"
  "llvm.hexagon.circ.lduh\0"
  "llvm.hexagon.circ.ldw\0"
  "llvm.hexagon.circ.stb\0"
  "llvm.hexagon.circ.std\0"
  "llvm.hexagon.circ.sth\0"
  "llvm.hexagon.circ.sthhi\0"
  "llvm.hexagon.circ.stw\0"
  "llvm.hexagon.instrprof.custom\0"
  "llvm.hexagon.prefetch\0"
  "llvm.hexagon.vmemcpy\0"
  "llvm.hexagon.vmemset\0"
  "llvm.loongarch.asrtgt.d\0"
  "llvm.loongarch.asrtle.d\0"
  "llvm.loongarch.break\0"
  "llvm.loongarch.cacop.d\0"
  "llvm.loongarch.cacop.w\0"
  "llvm.loongarch.cpucfg\0"
  "llvm.loongarch.crc.w.b.w\0"
  "llvm.loongarch.crc.w.d.w\0"
  "llvm.loongarch.crc.w.h.w\0"
  "llvm.loongarch.crc.w.w.w\0"
  "llvm.loongarch.crcc.w.b.w\0"
  "llvm.loongarch.crcc.w.d.w\0"
  "llvm.loongarch.crcc.w.h.w\0"
  "llvm.loongarch.crcc.w.w.w\0"
  "llvm.loongarch.csrrd.d\0"
  "llvm.loongarch.csrrd.w\0"
  "llvm.loongarch.csrwr.d\0"
  "llvm.loongarch.csrwr.w\0"
  "llvm.loongarch.csrxchg.d\0"
  "llvm.loongarch.csrxchg.w\0"
  "llvm.loongarch.dbar\0"
  "llvm.loongarch.frecipe.d\0"
  "llvm.loongarch.frecipe.s\0"
  "llvm.loongarch.frsqrte.d\0"
  "llvm.loongarch.frsqrte.s\0"
  "llvm.loongarch.ibar\0"
  "llvm.loongarch.iocsrrd.b\0"
  "llvm.loongarch.iocsrrd.d\0"
  "llvm.loongarch.iocsrrd.h\0"
  "llvm.loongarch.iocsrrd.w\0"
  "llvm.loongarch.iocsrwr.b\0"
  "llvm.loongarch.iocsrwr.d\0"
  "llvm.loongarch.iocsrwr.h\0"
  "llvm.loongarch.iocsrwr.w\0"
  "llvm.loongarch.lasx.cast.128\0"
  "llvm.loongarch.lasx.cast.128.d\0"
  "llvm.loongarch.lasx.cast.128.s\0"
  "llvm.loongarch.lasx.concat.128\0"
  "llvm.loongarch.lasx.concat.128.d\0"
  "llvm.loongarch.lasx.concat.128.s\0"
  "llvm.loongarch.lasx.extract.128.hi\0"
  "llvm.loongarch.lasx.extract.128.hi.d\0"
  "llvm.loongarch.lasx.extract.128.hi.s\0"
  "llvm.loongarch.lasx.extract.128.lo\0"
  "llvm.loongarch.lasx.extract.128.lo.d\0"
  "llvm.loongarch.lasx.extract.128.lo.s\0"
  "llvm.loongarch.lasx.insert.128.hi\0"
  "llvm.loongarch.lasx.insert.128.hi.d\0"
  "llvm.loongarch.lasx.insert.128.hi.s\0"
  "llvm.loongarch.lasx.insert.128.lo\0"
  "llvm.loongarch.lasx.insert.128.lo.d\0"
  "llvm.loongarch.lasx.insert.128.lo.s\0"
  "llvm.loongarch.lasx.vext2xv.d.b\0"
  "llvm.loongarch.lasx.vext2xv.d.h\0"
  "llvm.loongarch.lasx.vext2xv.d.w\0"
  "llvm.loongarch.lasx.vext2xv.du.bu\0"
  "llvm.loongarch.lasx.vext2xv.du.hu\0"
  "llvm.loongarch.lasx.vext2xv.du.wu\0"
  "llvm.loongarch.lasx.vext2xv.h.b\0"
  "llvm.loongarch.lasx.vext2xv.hu.bu\0"
  "llvm.loongarch.lasx.vext2xv.w.b\0"
  "llvm.loongarch.lasx.vext2xv.w.h\0"
  "llvm.loongarch.lasx.vext2xv.wu.bu\0"
  "llvm.loongarch.lasx.vext2xv.wu.hu\0"
  "llvm.loongarch.lasx.xbnz.b\0"
  "llvm.loongarch.lasx.xbnz.d\0"
  "llvm.loongarch.lasx.xbnz.h\0"
  "llvm.loongarch.lasx.xbnz.v\0"
  "llvm.loongarch.lasx.xbnz.w\0"
  "llvm.loongarch.lasx.xbz.b\0"
  "llvm.loongarch.lasx.xbz.d\0"
  "llvm.loongarch.lasx.xbz.h\0"
  "llvm.loongarch.lasx.xbz.v\0"
  "llvm.loongarch.lasx.xbz.w\0"
  "llvm.loongarch.lasx.xvabsd.b\0"
  "llvm.loongarch.lasx.xvabsd.bu\0"
  "llvm.loongarch.lasx.xvabsd.d\0"
  "llvm.loongarch.lasx.xvabsd.du\0"
  "llvm.loongarch.lasx.xvabsd.h\0"
  "llvm.loongarch.lasx.xvabsd.hu\0"
  "llvm.loongarch.lasx.xvabsd.w\0"
  "llvm.loongarch.lasx.xvabsd.wu\0"
  "llvm.loongarch.lasx.xvadd.b\0"
  "llvm.loongarch.lasx.xvadd.d\0"
  "llvm.loongarch.lasx.xvadd.h\0"
  "llvm.loongarch.lasx.xvadd.q\0"
  "llvm.loongarch.lasx.xvadd.w\0"
  "llvm.loongarch.lasx.xvadda.b\0"
  "llvm.loongarch.lasx.xvadda.d\0"
  "llvm.loongarch.lasx.xvadda.h\0"
  "llvm.loongarch.lasx.xvadda.w\0"
  "llvm.loongarch.lasx.xvaddi.bu\0"
  "llvm.loongarch.lasx.xvaddi.du\0"
  "llvm.loongarch.lasx.xvaddi.hu\0"
  "llvm.loongarch.lasx.xvaddi.wu\0"
  "llvm.loongarch.lasx.xvaddwev.d.w\0"
  "llvm.loongarch.lasx.xvaddwev.d.wu\0"
  "llvm.loongarch.lasx.xvaddwev.d.wu.w\0"
  "llvm.loongarch.lasx.xvaddwev.h.b\0"
  "llvm.loongarch.lasx.xvaddwev.h.bu\0"
  "llvm.loongarch.lasx.xvaddwev.h.bu.b\0"
  "llvm.loongarch.lasx.xvaddwev.q.d\0"
  "llvm.loongarch.lasx.xvaddwev.q.du\0"
  "llvm.loongarch.lasx.xvaddwev.q.du.d\0"
  "llvm.loongarch.lasx.xvaddwev.w.h\0"
  "llvm.loongarch.lasx.xvaddwev.w.hu\0"
  "llvm.loongarch.lasx.xvaddwev.w.hu.h\0"
  "llvm.loongarch.lasx.xvaddwod.d.w\0"
  "llvm.loongarch.lasx.xvaddwod.d.wu\0"
  "llvm.loongarch.lasx.xvaddwod.d.wu.w\0"
  "llvm.loongarch.lasx.xvaddwod.h.b\0"
  "llvm.loongarch.lasx.xvaddwod.h.bu\0"
  "llvm.loongarch.lasx.xvaddwod.h.bu.b\0"
  "llvm.loongarch.lasx.xvaddwod.q.d\0"
  "llvm.loongarch.lasx.xvaddwod.q.du\0"
  "llvm.loongarch.lasx.xvaddwod.q.du.d\0"
  "llvm.loongarch.lasx.xvaddwod.w.h\0"
  "llvm.loongarch.lasx.xvaddwod.w.hu\0"
  "llvm.loongarch.lasx.xvaddwod.w.hu.h\0"
  "llvm.loongarch.lasx.xvand.v\0"
  "llvm.loongarch.lasx.xvandi.b\0"
  "llvm.loongarch.lasx.xvandn.v\0"
  "llvm.loongarch.lasx.xvavg.b\0"
  "llvm.loongarch.lasx.xvavg.bu\0"
  "llvm.loongarch.lasx.xvavg.d\0"
  "llvm.loongarch.lasx.xvavg.du\0"
  "llvm.loongarch.lasx.xvavg.h\0"
  "llvm.loongarch.lasx.xvavg.hu\0"
  "llvm.loongarch.lasx.xvavg.w\0"
  "llvm.loongarch.lasx.xvavg.wu\0"
  "llvm.loongarch.lasx.xvavgr.b\0"
  "llvm.loongarch.lasx.xvavgr.bu\0"
  "llvm.loongarch.lasx.xvavgr.d\0"
  "llvm.loongarch.lasx.xvavgr.du\0"
  "llvm.loongarch.lasx.xvavgr.h\0"
  "llvm.loongarch.lasx.xvavgr.hu\0"
  "llvm.loongarch.lasx.xvavgr.w\0"
  "llvm.loongarch.lasx.xvavgr.wu\0"
  "llvm.loongarch.lasx.xvbitclr.b\0"
  "llvm.loongarch.lasx.xvbitclr.d\0"
  "llvm.loongarch.lasx.xvbitclr.h\0"
  "llvm.loongarch.lasx.xvbitclr.w\0"
  "llvm.loongarch.lasx.xvbitclri.b\0"
  "llvm.loongarch.lasx.xvbitclri.d\0"
  "llvm.loongarch.lasx.xvbitclri.h\0"
  "llvm.loongarch.lasx.xvbitclri.w\0"
  "llvm.loongarch.lasx.xvbitrev.b\0"
  "llvm.loongarch.lasx.xvbitrev.d\0"
  "llvm.loongarch.lasx.xvbitrev.h\0"
  "llvm.loongarch.lasx.xvbitrev.w\0"
  "llvm.loongarch.lasx.xvbitrevi.b\0"
  "llvm.loongarch.lasx.xvbitrevi.d\0"
  "llvm.loongarch.lasx.xvbitrevi.h\0"
  "llvm.loongarch.lasx.xvbitrevi.w\0"
  "llvm.loongarch.lasx.xvbitsel.v\0"
  "llvm.loongarch.lasx.xvbitseli.b\0"
  "llvm.loongarch.lasx.xvbitset.b\0"
  "llvm.loongarch.lasx.xvbitset.d\0"
  "llvm.loongarch.lasx.xvbitset.h\0"
  "llvm.loongarch.lasx.xvbitset.w\0"
  "llvm.loongarch.lasx.xvbitseti.b\0"
  "llvm.loongarch.lasx.xvbitseti.d\0"
  "llvm.loongarch.lasx.xvbitseti.h\0"
  "llvm.loongarch.lasx.xvbitseti.w\0"
  "llvm.loongarch.lasx.xvbsll.v\0"
  "llvm.loongarch.lasx.xvbsrl.v\0"
  "llvm.loongarch.lasx.xvclo.b\0"
  "llvm.loongarch.lasx.xvclo.d\0"
  "llvm.loongarch.lasx.xvclo.h\0"
  "llvm.loongarch.lasx.xvclo.w\0"
  "llvm.loongarch.lasx.xvclz.b\0"
  "llvm.loongarch.lasx.xvclz.d\0"
  "llvm.loongarch.lasx.xvclz.h\0"
  "llvm.loongarch.lasx.xvclz.w\0"
  "llvm.loongarch.lasx.xvdiv.b\0"
  "llvm.loongarch.lasx.xvdiv.bu\0"
  "llvm.loongarch.lasx.xvdiv.d\0"
  "llvm.loongarch.lasx.xvdiv.du\0"
  "llvm.loongarch.lasx.xvdiv.h\0"
  "llvm.loongarch.lasx.xvdiv.hu\0"
  "llvm.loongarch.lasx.xvdiv.w\0"
  "llvm.loongarch.lasx.xvdiv.wu\0"
  "llvm.loongarch.lasx.xvexth.d.w\0"
  "llvm.loongarch.lasx.xvexth.du.wu\0"
  "llvm.loongarch.lasx.xvexth.h.b\0"
  "llvm.loongarch.lasx.xvexth.hu.bu\0"
  "llvm.loongarch.lasx.xvexth.q.d\0"
  "llvm.loongarch.lasx.xvexth.qu.du\0"
  "llvm.loongarch.lasx.xvexth.w.h\0"
  "llvm.loongarch.lasx.xvexth.wu.hu\0"
  "llvm.loongarch.lasx.xvextl.q.d\0"
  "llvm.loongarch.lasx.xvextl.qu.du\0"
  "llvm.loongarch.lasx.xvextrins.b\0"
  "llvm.loongarch.lasx.xvextrins.d\0"
  "llvm.loongarch.lasx.xvextrins.h\0"
  "llvm.loongarch.lasx.xvextrins.w\0"
  "llvm.loongarch.lasx.xvfadd.d\0"
  "llvm.loongarch.lasx.xvfadd.s\0"
  "llvm.loongarch.lasx.xvfclass.d\0"
  "llvm.loongarch.lasx.xvfclass.s\0"
  "llvm.loongarch.lasx.xvfcmp.caf.d\0"
  "llvm.loongarch.lasx.xvfcmp.caf.s\0"
  "llvm.loongarch.lasx.xvfcmp.ceq.d\0"
  "llvm.loongarch.lasx.xvfcmp.ceq.s\0"
  "llvm.loongarch.lasx.xvfcmp.cle.d\0"
  "llvm.loongarch.lasx.xvfcmp.cle.s\0"
  "llvm.loongarch.lasx.xvfcmp.clt.d\0"
  "llvm.loongarch.lasx.xvfcmp.clt.s\0"
  "llvm.loongarch.lasx.xvfcmp.cne.d\0"
  "llvm.loongarch.lasx.xvfcmp.cne.s\0"
  "llvm.loongarch.lasx.xvfcmp.cor.d\0"
  "llvm.loongarch.lasx.xvfcmp.cor.s\0"
  "llvm.loongarch.lasx.xvfcmp.cueq.d\0"
  "llvm.loongarch.lasx.xvfcmp.cueq.s\0"
  "llvm.loongarch.lasx.xvfcmp.cule.d\0"
  "llvm.loongarch.lasx.xvfcmp.cule.s\0"
  "llvm.loongarch.lasx.xvfcmp.cult.d\0"
  "llvm.loongarch.lasx.xvfcmp.cult.s\0"
  "llvm.loongarch.lasx.xvfcmp.cun.d\0"
  "llvm.loongarch.lasx.xvfcmp.cun.s\0"
  "llvm.loongarch.lasx.xvfcmp.cune.d\0"
  "llvm.loongarch.lasx.xvfcmp.cune.s\0"
  "llvm.loongarch.lasx.xvfcmp.saf.d\0"
  "llvm.loongarch.lasx.xvfcmp.saf.s\0"
  "llvm.loongarch.lasx.xvfcmp.seq.d\0"
  "llvm.loongarch.lasx.xvfcmp.seq.s\0"
  "llvm.loongarch.lasx.xvfcmp.sle.d\0"
  "llvm.loongarch.lasx.xvfcmp.sle.s\0"
  "llvm.loongarch.lasx.xvfcmp.slt.d\0"
  "llvm.loongarch.lasx.xvfcmp.slt.s\0"
  "llvm.loongarch.lasx.xvfcmp.sne.d\0"
  "llvm.loongarch.lasx.xvfcmp.sne.s\0"
  "llvm.loongarch.lasx.xvfcmp.sor.d\0"
  "llvm.loongarch.lasx.xvfcmp.sor.s\0"
  "llvm.loongarch.lasx.xvfcmp.sueq.d\0"
  "llvm.loongarch.lasx.xvfcmp.sueq.s\0"
  "llvm.loongarch.lasx.xvfcmp.sule.d\0"
  "llvm.loongarch.lasx.xvfcmp.sule.s\0"
  "llvm.loongarch.lasx.xvfcmp.sult.d\0"
  "llvm.loongarch.lasx.xvfcmp.sult.s\0"
  "llvm.loongarch.lasx.xvfcmp.sun.d\0"
  "llvm.loongarch.lasx.xvfcmp.sun.s\0"
  "llvm.loongarch.lasx.xvfcmp.sune.d\0"
  "llvm.loongarch.lasx.xvfcmp.sune.s\0"
  "llvm.loongarch.lasx.xvfcvt.h.s\0"
  "llvm.loongarch.lasx.xvfcvt.s.d\0"
  "llvm.loongarch.lasx.xvfcvth.d.s\0"
  "llvm.loongarch.lasx.xvfcvth.s.h\0"
  "llvm.loongarch.lasx.xvfcvtl.d.s\0"
  "llvm.loongarch.lasx.xvfcvtl.s.h\0"
  "llvm.loongarch.lasx.xvfdiv.d\0"
  "llvm.loongarch.lasx.xvfdiv.s\0"
  "llvm.loongarch.lasx.xvffint.d.l\0"
  "llvm.loongarch.lasx.xvffint.d.lu\0"
  "llvm.loongarch.lasx.xvffint.s.l\0"
  "llvm.loongarch.lasx.xvffint.s.w\0"
  "llvm.loongarch.lasx.xvffint.s.wu\0"
  "llvm.loongarch.lasx.xvffinth.d.w\0"
  "llvm.loongarch.lasx.xvffintl.d.w\0"
  "llvm.loongarch.lasx.xvflogb.d\0"
  "llvm.loongarch.lasx.xvflogb.s\0"
  "llvm.loongarch.lasx.xvfmadd.d\0"
  "llvm.loongarch.lasx.xvfmadd.s\0"
  "llvm.loongarch.lasx.xvfmax.d\0"
  "llvm.loongarch.lasx.xvfmax.s\0"
  "llvm.loongarch.lasx.xvfmaxa.d\0"
  "llvm.loongarch.lasx.xvfmaxa.s\0"
  "llvm.loongarch.lasx.xvfmin.d\0"
  "llvm.loongarch.lasx.xvfmin.s\0"
  "llvm.loongarch.lasx.xvfmina.d\0"
  "llvm.loongarch.lasx.xvfmina.s\0"
  "llvm.loongarch.lasx.xvfmsub.d\0"
  "llvm.loongarch.lasx.xvfmsub.s\0"
  "llvm.loongarch.lasx.xvfmul.d\0"
  "llvm.loongarch.lasx.xvfmul.s\0"
  "llvm.loongarch.lasx.xvfnmadd.d\0"
  "llvm.loongarch.lasx.xvfnmadd.s\0"
  "llvm.loongarch.lasx.xvfnmsub.d\0"
  "llvm.loongarch.lasx.xvfnmsub.s\0"
  "llvm.loongarch.lasx.xvfrecip.d\0"
  "llvm.loongarch.lasx.xvfrecip.s\0"
  "llvm.loongarch.lasx.xvfrecipe.d\0"
  "llvm.loongarch.lasx.xvfrecipe.s\0"
  "llvm.loongarch.lasx.xvfrint.d\0"
  "llvm.loongarch.lasx.xvfrint.s\0"
  "llvm.loongarch.lasx.xvfrintrm.d\0"
  "llvm.loongarch.lasx.xvfrintrm.s\0"
  "llvm.loongarch.lasx.xvfrintrne.d\0"
  "llvm.loongarch.lasx.xvfrintrne.s\0"
  "llvm.loongarch.lasx.xvfrintrp.d\0"
  "llvm.loongarch.lasx.xvfrintrp.s\0"
  "llvm.loongarch.lasx.xvfrintrz.d\0"
  "llvm.loongarch.lasx.xvfrintrz.s\0"
  "llvm.loongarch.lasx.xvfrsqrt.d\0"
  "llvm.loongarch.lasx.xvfrsqrt.s\0"
  "llvm.loongarch.lasx.xvfrsqrte.d\0"
  "llvm.loongarch.lasx.xvfrsqrte.s\0"
  "llvm.loongarch.lasx.xvfrstp.b\0"
  "llvm.loongarch.lasx.xvfrstp.h\0"
  "llvm.loongarch.lasx.xvfrstpi.b\0"
  "llvm.loongarch.lasx.xvfrstpi.h\0"
  "llvm.loongarch.lasx.xvfsqrt.d\0"
  "llvm.loongarch.lasx.xvfsqrt.s\0"
  "llvm.loongarch.lasx.xvfsub.d\0"
  "llvm.loongarch.lasx.xvfsub.s\0"
  "llvm.loongarch.lasx.xvftint.l.d\0"
  "llvm.loongarch.lasx.xvftint.lu.d\0"
  "llvm.loongarch.lasx.xvftint.w.d\0"
  "llvm.loongarch.lasx.xvftint.w.s\0"
  "llvm.loongarch.lasx.xvftint.wu.s\0"
  "llvm.loongarch.lasx.xvftinth.l.s\0"
  "llvm.loongarch.lasx.xvftintl.l.s\0"
  "llvm.loongarch.lasx.xvftintrm.l.d\0"
  "llvm.loongarch.lasx.xvftintrm.w.d\0"
  "llvm.loongarch.lasx.xvftintrm.w.s\0"
  "llvm.loongarch.lasx.xvftintrmh.l.s\0"
  "llvm.loongarch.lasx.xvftintrml.l.s\0"
  "llvm.loongarch.lasx.xvftintrne.l.d\0"
  "llvm.loongarch.lasx.xvftintrne.w.d\0"
  "llvm.loongarch.lasx.xvftintrne.w.s\0"
  "llvm.loongarch.lasx.xvftintrneh.l.s\0"
  "llvm.loongarch.lasx.xvftintrnel.l.s\0"
  "llvm.loongarch.lasx.xvftintrp.l.d\0"
  "llvm.loongarch.lasx.xvftintrp.w.d\0"
  "llvm.loongarch.lasx.xvftintrp.w.s\0"
  "llvm.loongarch.lasx.xvftintrph.l.s\0"
  "llvm.loongarch.lasx.xvftintrpl.l.s\0"
  "llvm.loongarch.lasx.xvftintrz.l.d\0"
  "llvm.loongarch.lasx.xvftintrz.lu.d\0"
  "llvm.loongarch.lasx.xvftintrz.w.d\0"
  "llvm.loongarch.lasx.xvftintrz.w.s\0"
  "llvm.loongarch.lasx.xvftintrz.wu.s\0"
  "llvm.loongarch.lasx.xvftintrzh.l.s\0"
  "llvm.loongarch.lasx.xvftintrzl.l.s\0"
  "llvm.loongarch.lasx.xvhaddw.d.w\0"
  "llvm.loongarch.lasx.xvhaddw.du.wu\0"
  "llvm.loongarch.lasx.xvhaddw.h.b\0"
  "llvm.loongarch.lasx.xvhaddw.hu.bu\0"
  "llvm.loongarch.lasx.xvhaddw.q.d\0"
  "llvm.loongarch.lasx.xvhaddw.qu.du\0"
  "llvm.loongarch.lasx.xvhaddw.w.h\0"
  "llvm.loongarch.lasx.xvhaddw.wu.hu\0"
  "llvm.loongarch.lasx.xvhsubw.d.w\0"
  "llvm.loongarch.lasx.xvhsubw.du.wu\0"
  "llvm.loongarch.lasx.xvhsubw.h.b\0"
  "llvm.loongarch.lasx.xvhsubw.hu.bu\0"
  "llvm.loongarch.lasx.xvhsubw.q.d\0"
  "llvm.loongarch.lasx.xvhsubw.qu.du\0"
  "llvm.loongarch.lasx.xvhsubw.w.h\0"
  "llvm.loongarch.lasx.xvhsubw.wu.hu\0"
  "llvm.loongarch.lasx.xvilvh.b\0"
  "llvm.loongarch.lasx.xvilvh.d\0"
  "llvm.loongarch.lasx.xvilvh.h\0"
  "llvm.loongarch.lasx.xvilvh.w\0"
  "llvm.loongarch.lasx.xvilvl.b\0"
  "llvm.loongarch.lasx.xvilvl.d\0"
  "llvm.loongarch.lasx.xvilvl.h\0"
  "llvm.loongarch.lasx.xvilvl.w\0"
  "llvm.loongarch.lasx.xvinsgr2vr.d\0"
  "llvm.loongarch.lasx.xvinsgr2vr.w\0"
  "llvm.loongarch.lasx.xvinsve0.d\0"
  "llvm.loongarch.lasx.xvinsve0.w\0"
  "llvm.loongarch.lasx.xvld\0"
  "llvm.loongarch.lasx.xvldi\0"
  "llvm.loongarch.lasx.xvldrepl.b\0"
  "llvm.loongarch.lasx.xvldrepl.d\0"
  "llvm.loongarch.lasx.xvldrepl.h\0"
  "llvm.loongarch.lasx.xvldrepl.w\0"
  "llvm.loongarch.lasx.xvldx\0"
  "llvm.loongarch.lasx.xvmadd.b\0"
  "llvm.loongarch.lasx.xvmadd.d\0"
  "llvm.loongarch.lasx.xvmadd.h\0"
  "llvm.loongarch.lasx.xvmadd.w\0"
  "llvm.loongarch.lasx.xvmaddwev.d.w\0"
  "llvm.loongarch.lasx.xvmaddwev.d.wu\0"
  "llvm.loongarch.lasx.xvmaddwev.d.wu.w\0"
  "llvm.loongarch.lasx.xvmaddwev.h.b\0"
  "llvm.loongarch.lasx.xvmaddwev.h.bu\0"
  "llvm.loongarch.lasx.xvmaddwev.h.bu.b\0"
  "llvm.loongarch.lasx.xvmaddwev.q.d\0"
  "llvm.loongarch.lasx.xvmaddwev.q.du\0"
  "llvm.loongarch.lasx.xvmaddwev.q.du.d\0"
  "llvm.loongarch.lasx.xvmaddwev.w.h\0"
  "llvm.loongarch.lasx.xvmaddwev.w.hu\0"
  "llvm.loongarch.lasx.xvmaddwev.w.hu.h\0"
  "llvm.loongarch.lasx.xvmaddwod.d.w\0"
  "llvm.loongarch.lasx.xvmaddwod.d.wu\0"
  "llvm.loongarch.lasx.xvmaddwod.d.wu.w\0"
  "llvm.loongarch.lasx.xvmaddwod.h.b\0"
  "llvm.loongarch.lasx.xvmaddwod.h.bu\0"
  "llvm.loongarch.lasx.xvmaddwod.h.bu.b\0"
  "llvm.loongarch.lasx.xvmaddwod.q.d\0"
  "llvm.loongarch.lasx.xvmaddwod.q.du\0"
  "llvm.loongarch.lasx.xvmaddwod.q.du.d\0"
  "llvm.loongarch.lasx.xvmaddwod.w.h\0"
  "llvm.loongarch.lasx.xvmaddwod.w.hu\0"
  "llvm.loongarch.lasx.xvmaddwod.w.hu.h\0"
  "llvm.loongarch.lasx.xvmax.b\0"
  "llvm.loongarch.lasx.xvmax.bu\0"
  "llvm.loongarch.lasx.xvmax.d\0"
  "llvm.loongarch.lasx.xvmax.du\0"
  "llvm.loongarch.lasx.xvmax.h\0"
  "llvm.loongarch.lasx.xvmax.hu\0"
  "llvm.loongarch.lasx.xvmax.w\0"
  "llvm.loongarch.lasx.xvmax.wu\0"
  "llvm.loongarch.lasx.xvmaxi.b\0"
  "llvm.loongarch.lasx.xvmaxi.bu\0"
  "llvm.loongarch.lasx.xvmaxi.d\0"
  "llvm.loongarch.lasx.xvmaxi.du\0"
  "llvm.loongarch.lasx.xvmaxi.h\0"
  "llvm.loongarch.lasx.xvmaxi.hu\0"
  "llvm.loongarch.lasx.xvmaxi.w\0"
  "llvm.loongarch.lasx.xvmaxi.wu\0"
  "llvm.loongarch.lasx.xvmin.b\0"
  "llvm.loongarch.lasx.xvmin.bu\0"
  "llvm.loongarch.lasx.xvmin.d\0"
  "llvm.loongarch.lasx.xvmin.du\0"
  "llvm.loongarch.lasx.xvmin.h\0"
  "llvm.loongarch.lasx.xvmin.hu\0"
  "llvm.loongarch.lasx.xvmin.w\0"
  "llvm.loongarch.lasx.xvmin.wu\0"
  "llvm.loongarch.lasx.xvmini.b\0"
  "llvm.loongarch.lasx.xvmini.bu\0"
  "llvm.loongarch.lasx.xvmini.d\0"
  "llvm.loongarch.lasx.xvmini.du\0"
  "llvm.loongarch.lasx.xvmini.h\0"
  "llvm.loongarch.lasx.xvmini.hu\0"
  "llvm.loongarch.lasx.xvmini.w\0"
  "llvm.loongarch.lasx.xvmini.wu\0"
  "llvm.loongarch.lasx.xvmod.b\0"
  "llvm.loongarch.lasx.xvmod.bu\0"
  "llvm.loongarch.lasx.xvmod.d\0"
  "llvm.loongarch.lasx.xvmod.du\0"
  "llvm.loongarch.lasx.xvmod.h\0"
  "llvm.loongarch.lasx.xvmod.hu\0"
  "llvm.loongarch.lasx.xvmod.w\0"
  "llvm.loongarch.lasx.xvmod.wu\0"
  "llvm.loongarch.lasx.xvmskgez.b\0"
  "llvm.loongarch.lasx.xvmskltz.b\0"
  "llvm.loongarch.lasx.xvmskltz.d\0"
  "llvm.loongarch.lasx.xvmskltz.h\0"
  "llvm.loongarch.lasx.xvmskltz.w\0"
  "llvm.loongarch.lasx.xvmsknz.b\0"
  "llvm.loongarch.lasx.xvmsub.b\0"
  "llvm.loongarch.lasx.xvmsub.d\0"
  "llvm.loongarch.lasx.xvmsub.h\0"
  "llvm.loongarch.lasx.xvmsub.w\0"
  "llvm.loongarch.lasx.xvmuh.b\0"
  "llvm.loongarch.lasx.xvmuh.bu\0"
  "llvm.loongarch.lasx.xvmuh.d\0"
  "llvm.loongarch.lasx.xvmuh.du\0"
  "llvm.loongarch.lasx.xvmuh.h\0"
  "llvm.loongarch.lasx.xvmuh.hu\0"
  "llvm.loongarch.lasx.xvmuh.w\0"
  "llvm.loongarch.lasx.xvmuh.wu\0"
  "llvm.loongarch.lasx.xvmul.b\0"
  "llvm.loongarch.lasx.xvmul.d\0"
  "llvm.loongarch.lasx.xvmul.h\0"
  "llvm.loongarch.lasx.xvmul.w\0"
  "llvm.loongarch.lasx.xvmulwev.d.w\0"
  "llvm.loongarch.lasx.xvmulwev.d.wu\0"
  "llvm.loongarch.lasx.xvmulwev.d.wu.w\0"
  "llvm.loongarch.lasx.xvmulwev.h.b\0"
  "llvm.loongarch.lasx.xvmulwev.h.bu\0"
  "llvm.loongarch.lasx.xvmulwev.h.bu.b\0"
  "llvm.loongarch.lasx.xvmulwev.q.d\0"
  "llvm.loongarch.lasx.xvmulwev.q.du\0"
  "llvm.loongarch.lasx.xvmulwev.q.du.d\0"
  "llvm.loongarch.lasx.xvmulwev.w.h\0"
  "llvm.loongarch.lasx.xvmulwev.w.hu\0"
  "llvm.loongarch.lasx.xvmulwev.w.hu.h\0"
  "llvm.loongarch.lasx.xvmulwod.d.w\0"
  "llvm.loongarch.lasx.xvmulwod.d.wu\0"
  "llvm.loongarch.lasx.xvmulwod.d.wu.w\0"
  "llvm.loongarch.lasx.xvmulwod.h.b\0"
  "llvm.loongarch.lasx.xvmulwod.h.bu\0"
  "llvm.loongarch.lasx.xvmulwod.h.bu.b\0"
  "llvm.loongarch.lasx.xvmulwod.q.d\0"
  "llvm.loongarch.lasx.xvmulwod.q.du\0"
  "llvm.loongarch.lasx.xvmulwod.q.du.d\0"
  "llvm.loongarch.lasx.xvmulwod.w.h\0"
  "llvm.loongarch.lasx.xvmulwod.w.hu\0"
  "llvm.loongarch.lasx.xvmulwod.w.hu.h\0"
  "llvm.loongarch.lasx.xvneg.b\0"
  "llvm.loongarch.lasx.xvneg.d\0"
  "llvm.loongarch.lasx.xvneg.h\0"
  "llvm.loongarch.lasx.xvneg.w\0"
  "llvm.loongarch.lasx.xvnor.v\0"
  "llvm.loongarch.lasx.xvnori.b\0"
  "llvm.loongarch.lasx.xvor.v\0"
  "llvm.loongarch.lasx.xvori.b\0"
  "llvm.loongarch.lasx.xvorn.v\0"
  "llvm.loongarch.lasx.xvpackev.b\0"
  "llvm.loongarch.lasx.xvpackev.d\0"
  "llvm.loongarch.lasx.xvpackev.h\0"
  "llvm.loongarch.lasx.xvpackev.w\0"
  "llvm.loongarch.lasx.xvpackod.b\0"
  "llvm.loongarch.lasx.xvpackod.d\0"
  "llvm.loongarch.lasx.xvpackod.h\0"
  "llvm.loongarch.lasx.xvpackod.w\0"
  "llvm.loongarch.lasx.xvpcnt.b\0"
  "llvm.loongarch.lasx.xvpcnt.d\0"
  "llvm.loongarch.lasx.xvpcnt.h\0"
  "llvm.loongarch.lasx.xvpcnt.w\0"
  "llvm.loongarch.lasx.xvperm.w\0"
  "llvm.loongarch.lasx.xvpermi.d\0"
  "llvm.loongarch.lasx.xvpermi.q\0"
  "llvm.loongarch.lasx.xvpermi.w\0"
  "llvm.loongarch.lasx.xvpickev.b\0"
  "llvm.loongarch.lasx.xvpickev.d\0"
  "llvm.loongarch.lasx.xvpickev.h\0"
  "llvm.loongarch.lasx.xvpickev.w\0"
  "llvm.loongarch.lasx.xvpickod.b\0"
  "llvm.loongarch.lasx.xvpickod.d\0"
  "llvm.loongarch.lasx.xvpickod.h\0"
  "llvm.loongarch.lasx.xvpickod.w\0"
  "llvm.loongarch.lasx.xvpickve.d\0"
  "llvm.loongarch.lasx.xvpickve.d.f\0"
  "llvm.loongarch.lasx.xvpickve.w\0"
  "llvm.loongarch.lasx.xvpickve.w.f\0"
  "llvm.loongarch.lasx.xvpickve2gr.d\0"
  "llvm.loongarch.lasx.xvpickve2gr.du\0"
  "llvm.loongarch.lasx.xvpickve2gr.w\0"
  "llvm.loongarch.lasx.xvpickve2gr.wu\0"
  "llvm.loongarch.lasx.xvrepl128vei.b\0"
  "llvm.loongarch.lasx.xvrepl128vei.d\0"
  "llvm.loongarch.lasx.xvrepl128vei.h\0"
  "llvm.loongarch.lasx.xvrepl128vei.w\0"
  "llvm.loongarch.lasx.xvreplgr2vr.b\0"
  "llvm.loongarch.lasx.xvreplgr2vr.d\0"
  "llvm.loongarch.lasx.xvreplgr2vr.h\0"
  "llvm.loongarch.lasx.xvreplgr2vr.w\0"
  "llvm.loongarch.lasx.xvrepli.b\0"
  "llvm.loongarch.lasx.xvrepli.d\0"
  "llvm.loongarch.lasx.xvrepli.h\0"
  "llvm.loongarch.lasx.xvrepli.w\0"
  "llvm.loongarch.lasx.xvreplve.b\0"
  "llvm.loongarch.lasx.xvreplve.d\0"
  "llvm.loongarch.lasx.xvreplve.h\0"
  "llvm.loongarch.lasx.xvreplve.w\0"
  "llvm.loongarch.lasx.xvreplve0.b\0"
  "llvm.loongarch.lasx.xvreplve0.d\0"
  "llvm.loongarch.lasx.xvreplve0.h\0"
  "llvm.loongarch.lasx.xvreplve0.q\0"
  "llvm.loongarch.lasx.xvreplve0.w\0"
  "llvm.loongarch.lasx.xvrotr.b\0"
  "llvm.loongarch.lasx.xvrotr.d\0"
  "llvm.loongarch.lasx.xvrotr.h\0"
  "llvm.loongarch.lasx.xvrotr.w\0"
  "llvm.loongarch.lasx.xvrotri.b\0"
  "llvm.loongarch.lasx.xvrotri.d\0"
  "llvm.loongarch.lasx.xvrotri.h\0"
  "llvm.loongarch.lasx.xvrotri.w\0"
  "llvm.loongarch.lasx.xvsadd.b\0"
  "llvm.loongarch.lasx.xvsadd.bu\0"
  "llvm.loongarch.lasx.xvsadd.d\0"
  "llvm.loongarch.lasx.xvsadd.du\0"
  "llvm.loongarch.lasx.xvsadd.h\0"
  "llvm.loongarch.lasx.xvsadd.hu\0"
  "llvm.loongarch.lasx.xvsadd.w\0"
  "llvm.loongarch.lasx.xvsadd.wu\0"
  "llvm.loongarch.lasx.xvsat.b\0"
  "llvm.loongarch.lasx.xvsat.bu\0"
  "llvm.loongarch.lasx.xvsat.d\0"
  "llvm.loongarch.lasx.xvsat.du\0"
  "llvm.loongarch.lasx.xvsat.h\0"
  "llvm.loongarch.lasx.xvsat.hu\0"
  "llvm.loongarch.lasx.xvsat.w\0"
  "llvm.loongarch.lasx.xvsat.wu\0"
  "llvm.loongarch.lasx.xvseq.b\0"
  "llvm.loongarch.lasx.xvseq.d\0"
  "llvm.loongarch.lasx.xvseq.h\0"
  "llvm.loongarch.lasx.xvseq.w\0"
  "llvm.loongarch.lasx.xvseqi.b\0"
  "llvm.loongarch.lasx.xvseqi.d\0"
  "llvm.loongarch.lasx.xvseqi.h\0"
  "llvm.loongarch.lasx.xvseqi.w\0"
  "llvm.loongarch.lasx.xvshuf.b\0"
  "llvm.loongarch.lasx.xvshuf.d\0"
  "llvm.loongarch.lasx.xvshuf.h\0"
  "llvm.loongarch.lasx.xvshuf.w\0"
  "llvm.loongarch.lasx.xvshuf4i.b\0"
  "llvm.loongarch.lasx.xvshuf4i.d\0"
  "llvm.loongarch.lasx.xvshuf4i.h\0"
  "llvm.loongarch.lasx.xvshuf4i.w\0"
  "llvm.loongarch.lasx.xvsigncov.b\0"
  "llvm.loongarch.lasx.xvsigncov.d\0"
  "llvm.loongarch.lasx.xvsigncov.h\0"
  "llvm.loongarch.lasx.xvsigncov.w\0"
  "llvm.loongarch.lasx.xvsle.b\0"
  "llvm.loongarch.lasx.xvsle.bu\0"
  "llvm.loongarch.lasx.xvsle.d\0"
  "llvm.loongarch.lasx.xvsle.du\0"
  "llvm.loongarch.lasx.xvsle.h\0"
  "llvm.loongarch.lasx.xvsle.hu\0"
  "llvm.loongarch.lasx.xvsle.w\0"
  "llvm.loongarch.lasx.xvsle.wu\0"
  "llvm.loongarch.lasx.xvslei.b\0"
  "llvm.loongarch.lasx.xvslei.bu\0"
  "llvm.loongarch.lasx.xvslei.d\0"
  "llvm.loongarch.lasx.xvslei.du\0"
  "llvm.loongarch.lasx.xvslei.h\0"
  "llvm.loongarch.lasx.xvslei.hu\0"
  "llvm.loongarch.lasx.xvslei.w\0"
  "llvm.loongarch.lasx.xvslei.wu\0"
  "llvm.loongarch.lasx.xvsll.b\0"
  "llvm.loongarch.lasx.xvsll.d\0"
  "llvm.loongarch.lasx.xvsll.h\0"
  "llvm.loongarch.lasx.xvsll.w\0"
  "llvm.loongarch.lasx.xvslli.b\0"
  "llvm.loongarch.lasx.xvslli.d\0"
  "llvm.loongarch.lasx.xvslli.h\0"
  "llvm.loongarch.lasx.xvslli.w\0"
  "llvm.loongarch.lasx.xvsllwil.d.w\0"
  "llvm.loongarch.lasx.xvsllwil.du.wu\0"
  "llvm.loongarch.lasx.xvsllwil.h.b\0"
  "llvm.loongarch.lasx.xvsllwil.hu.bu\0"
  "llvm.loongarch.lasx.xvsllwil.w.h\0"
  "llvm.loongarch.lasx.xvsllwil.wu.hu\0"
  "llvm.loongarch.lasx.xvslt.b\0"
  "llvm.loongarch.lasx.xvslt.bu\0"
  "llvm.loongarch.lasx.xvslt.d\0"
  "llvm.loongarch.lasx.xvslt.du\0"
  "llvm.loongarch.lasx.xvslt.h\0"
  "llvm.loongarch.lasx.xvslt.hu\0"
  "llvm.loongarch.lasx.xvslt.w\0"
  "llvm.loongarch.lasx.xvslt.wu\0"
  "llvm.loongarch.lasx.xvslti.b\0"
  "llvm.loongarch.lasx.xvslti.bu\0"
  "llvm.loongarch.lasx.xvslti.d\0"
  "llvm.loongarch.lasx.xvslti.du\0"
  "llvm.loongarch.lasx.xvslti.h\0"
  "llvm.loongarch.lasx.xvslti.hu\0"
  "llvm.loongarch.lasx.xvslti.w\0"
  "llvm.loongarch.lasx.xvslti.wu\0"
  "llvm.loongarch.lasx.xvsra.b\0"
  "llvm.loongarch.lasx.xvsra.d\0"
  "llvm.loongarch.lasx.xvsra.h\0"
  "llvm.loongarch.lasx.xvsra.w\0"
  "llvm.loongarch.lasx.xvsrai.b\0"
  "llvm.loongarch.lasx.xvsrai.d\0"
  "llvm.loongarch.lasx.xvsrai.h\0"
  "llvm.loongarch.lasx.xvsrai.w\0"
  "llvm.loongarch.lasx.xvsran.b.h\0"
  "llvm.loongarch.lasx.xvsran.h.w\0"
  "llvm.loongarch.lasx.xvsran.w.d\0"
  "llvm.loongarch.lasx.xvsrani.b.h\0"
  "llvm.loongarch.lasx.xvsrani.d.q\0"
  "llvm.loongarch.lasx.xvsrani.h.w\0"
  "llvm.loongarch.lasx.xvsrani.w.d\0"
  "llvm.loongarch.lasx.xvsrar.b\0"
  "llvm.loongarch.lasx.xvsrar.d\0"
  "llvm.loongarch.lasx.xvsrar.h\0"
  "llvm.loongarch.lasx.xvsrar.w\0"
  "llvm.loongarch.lasx.xvsrari.b\0"
  "llvm.loongarch.lasx.xvsrari.d\0"
  "llvm.loongarch.lasx.xvsrari.h\0"
  "llvm.loongarch.lasx.xvsrari.w\0"
  "llvm.loongarch.lasx.xvsrarn.b.h\0"
  "llvm.loongarch.lasx.xvsrarn.h.w\0"
  "llvm.loongarch.lasx.xvsrarn.w.d\0"
  "llvm.loongarch.lasx.xvsrarni.b.h\0"
  "llvm.loongarch.lasx.xvsrarni.d.q\0"
  "llvm.loongarch.lasx.xvsrarni.h.w\0"
  "llvm.loongarch.lasx.xvsrarni.w.d\0"
  "llvm.loongarch.lasx.xvsrl.b\0"
  "llvm.loongarch.lasx.xvsrl.d\0"
  "llvm.loongarch.lasx.xvsrl.h\0"
  "llvm.loongarch.lasx.xvsrl.w\0"
  "llvm.loongarch.lasx.xvsrli.b\0"
  "llvm.loongarch.lasx.xvsrli.d\0"
  "llvm.loongarch.lasx.xvsrli.h\0"
  "llvm.loongarch.lasx.xvsrli.w\0"
  "llvm.loongarch.lasx.xvsrln.b.h\0"
  "llvm.loongarch.lasx.xvsrln.h.w\0"
  "llvm.loongarch.lasx.xvsrln.w.d\0"
  "llvm.loongarch.lasx.xvsrlni.b.h\0"
  "llvm.loongarch.lasx.xvsrlni.d.q\0"
  "llvm.loongarch.lasx.xvsrlni.h.w\0"
  "llvm.loongarch.lasx.xvsrlni.w.d\0"
  "llvm.loongarch.lasx.xvsrlr.b\0"
  "llvm.loongarch.lasx.xvsrlr.d\0"
  "llvm.loongarch.lasx.xvsrlr.h\0"
  "llvm.loongarch.lasx.xvsrlr.w\0"
  "llvm.loongarch.lasx.xvsrlri.b\0"
  "llvm.loongarch.lasx.xvsrlri.d\0"
  "llvm.loongarch.lasx.xvsrlri.h\0"
  "llvm.loongarch.lasx.xvsrlri.w\0"
  "llvm.loongarch.lasx.xvsrlrn.b.h\0"
  "llvm.loongarch.lasx.xvsrlrn.h.w\0"
  "llvm.loongarch.lasx.xvsrlrn.w.d\0"
  "llvm.loongarch.lasx.xvsrlrni.b.h\0"
  "llvm.loongarch.lasx.xvsrlrni.d.q\0"
  "llvm.loongarch.lasx.xvsrlrni.h.w\0"
  "llvm.loongarch.lasx.xvsrlrni.w.d\0"
  "llvm.loongarch.lasx.xvssran.b.h\0"
  "llvm.loongarch.lasx.xvssran.bu.h\0"
  "llvm.loongarch.lasx.xvssran.h.w\0"
  "llvm.loongarch.lasx.xvssran.hu.w\0"
  "llvm.loongarch.lasx.xvssran.w.d\0"
  "llvm.loongarch.lasx.xvssran.wu.d\0"
  "llvm.loongarch.lasx.xvssrani.b.h\0"
  "llvm.loongarch.lasx.xvssrani.bu.h\0"
  "llvm.loongarch.lasx.xvssrani.d.q\0"
  "llvm.loongarch.lasx.xvssrani.du.q\0"
  "llvm.loongarch.lasx.xvssrani.h.w\0"
  "llvm.loongarch.lasx.xvssrani.hu.w\0"
  "llvm.loongarch.lasx.xvssrani.w.d\0"
  "llvm.loongarch.lasx.xvssrani.wu.d\0"
  "llvm.loongarch.lasx.xvssrarn.b.h\0"
  "llvm.loongarch.lasx.xvssrarn.bu.h\0"
  "llvm.loongarch.lasx.xvssrarn.h.w\0"
  "llvm.loongarch.lasx.xvssrarn.hu.w\0"
  "llvm.loongarch.lasx.xvssrarn.w.d\0"
  "llvm.loongarch.lasx.xvssrarn.wu.d\0"
  "llvm.loongarch.lasx.xvssrarni.b.h\0"
  "llvm.loongarch.lasx.xvssrarni.bu.h\0"
  "llvm.loongarch.lasx.xvssrarni.d.q\0"
  "llvm.loongarch.lasx.xvssrarni.du.q\0"
  "llvm.loongarch.lasx.xvssrarni.h.w\0"
  "llvm.loongarch.lasx.xvssrarni.hu.w\0"
  "llvm.loongarch.lasx.xvssrarni.w.d\0"
  "llvm.loongarch.lasx.xvssrarni.wu.d\0"
  "llvm.loongarch.lasx.xvssrln.b.h\0"
  "llvm.loongarch.lasx.xvssrln.bu.h\0"
  "llvm.loongarch.lasx.xvssrln.h.w\0"
  "llvm.loongarch.lasx.xvssrln.hu.w\0"
  "llvm.loongarch.lasx.xvssrln.w.d\0"
  "llvm.loongarch.lasx.xvssrln.wu.d\0"
  "llvm.loongarch.lasx.xvssrlni.b.h\0"
  "llvm.loongarch.lasx.xvssrlni.bu.h\0"
  "llvm.loongarch.lasx.xvssrlni.d.q\0"
  "llvm.loongarch.lasx.xvssrlni.du.q\0"
  "llvm.loongarch.lasx.xvssrlni.h.w\0"
  "llvm.loongarch.lasx.xvssrlni.hu.w\0"
  "llvm.loongarch.lasx.xvssrlni.w.d\0"
  "llvm.loongarch.lasx.xvssrlni.wu.d\0"
  "llvm.loongarch.lasx.xvssrlrn.b.h\0"
  "llvm.loongarch.lasx.xvssrlrn.bu.h\0"
  "llvm.loongarch.lasx.xvssrlrn.h.w\0"
  "llvm.loongarch.lasx.xvssrlrn.hu.w\0"
  "llvm.loongarch.lasx.xvssrlrn.w.d\0"
  "llvm.loongarch.lasx.xvssrlrn.wu.d\0"
  "llvm.loongarch.lasx.xvssrlrni.b.h\0"
  "llvm.loongarch.lasx.xvssrlrni.bu.h\0"
  "llvm.loongarch.lasx.xvssrlrni.d.q\0"
  "llvm.loongarch.lasx.xvssrlrni.du.q\0"
  "llvm.loongarch.lasx.xvssrlrni.h.w\0"
  "llvm.loongarch.lasx.xvssrlrni.hu.w\0"
  "llvm.loongarch.lasx.xvssrlrni.w.d\0"
  "llvm.loongarch.lasx.xvssrlrni.wu.d\0"
  "llvm.loongarch.lasx.xvssub.b\0"
  "llvm.loongarch.lasx.xvssub.bu\0"
  "llvm.loongarch.lasx.xvssub.d\0"
  "llvm.loongarch.lasx.xvssub.du\0"
  "llvm.loongarch.lasx.xvssub.h\0"
  "llvm.loongarch.lasx.xvssub.hu\0"
  "llvm.loongarch.lasx.xvssub.w\0"
  "llvm.loongarch.lasx.xvssub.wu\0"
  "llvm.loongarch.lasx.xvst\0"
  "llvm.loongarch.lasx.xvstelm.b\0"
  "llvm.loongarch.lasx.xvstelm.d\0"
  "llvm.loongarch.lasx.xvstelm.h\0"
  "llvm.loongarch.lasx.xvstelm.w\0"
  "llvm.loongarch.lasx.xvstx\0"
  "llvm.loongarch.lasx.xvsub.b\0"
  "llvm.loongarch.lasx.xvsub.d\0"
  "llvm.loongarch.lasx.xvsub.h\0"
  "llvm.loongarch.lasx.xvsub.q\0"
  "llvm.loongarch.lasx.xvsub.w\0"
  "llvm.loongarch.lasx.xvsubi.bu\0"
  "llvm.loongarch.lasx.xvsubi.du\0"
  "llvm.loongarch.lasx.xvsubi.hu\0"
  "llvm.loongarch.lasx.xvsubi.wu\0"
  "llvm.loongarch.lasx.xvsubwev.d.w\0"
  "llvm.loongarch.lasx.xvsubwev.d.wu\0"
  "llvm.loongarch.lasx.xvsubwev.h.b\0"
  "llvm.loongarch.lasx.xvsubwev.h.bu\0"
  "llvm.loongarch.lasx.xvsubwev.q.d\0"
  "llvm.loongarch.lasx.xvsubwev.q.du\0"
  "llvm.loongarch.lasx.xvsubwev.w.h\0"
  "llvm.loongarch.lasx.xvsubwev.w.hu\0"
  "llvm.loongarch.lasx.xvsubwod.d.w\0"
  "llvm.loongarch.lasx.xvsubwod.d.wu\0"
  "llvm.loongarch.lasx.xvsubwod.h.b\0"
  "llvm.loongarch.lasx.xvsubwod.h.bu\0"
  "llvm.loongarch.lasx.xvsubwod.q.d\0"
  "llvm.loongarch.lasx.xvsubwod.q.du\0"
  "llvm.loongarch.lasx.xvsubwod.w.h\0"
  "llvm.loongarch.lasx.xvsubwod.w.hu\0"
  "llvm.loongarch.lasx.xvxor.v\0"
  "llvm.loongarch.lasx.xvxori.b\0"
  "llvm.loongarch.lddir.d\0"
  "llvm.loongarch.ldpte.d\0"
  "llvm.loongarch.lsx.bnz.b\0"
  "llvm.loongarch.lsx.bnz.d\0"
  "llvm.loongarch.lsx.bnz.h\0"
  "llvm.loongarch.lsx.bnz.v\0"
  "llvm.loongarch.lsx.bnz.w\0"
  "llvm.loongarch.lsx.bz.b\0"
  "llvm.loongarch.lsx.bz.d\0"
  "llvm.loongarch.lsx.bz.h\0"
  "llvm.loongarch.lsx.bz.v\0"
  "llvm.loongarch.lsx.bz.w\0"
  "llvm.loongarch.lsx.vabsd.b\0"
  "llvm.loongarch.lsx.vabsd.bu\0"
  "llvm.loongarch.lsx.vabsd.d\0"
  "llvm.loongarch.lsx.vabsd.du\0"
  "llvm.loongarch.lsx.vabsd.h\0"
  "llvm.loongarch.lsx.vabsd.hu\0"
  "llvm.loongarch.lsx.vabsd.w\0"
  "llvm.loongarch.lsx.vabsd.wu\0"
  "llvm.loongarch.lsx.vadd.b\0"
  "llvm.loongarch.lsx.vadd.d\0"
  "llvm.loongarch.lsx.vadd.h\0"
  "llvm.loongarch.lsx.vadd.q\0"
  "llvm.loongarch.lsx.vadd.w\0"
  "llvm.loongarch.lsx.vadda.b\0"
  "llvm.loongarch.lsx.vadda.d\0"
  "llvm.loongarch.lsx.vadda.h\0"
  "llvm.loongarch.lsx.vadda.w\0"
  "llvm.loongarch.lsx.vaddi.bu\0"
  "llvm.loongarch.lsx.vaddi.du\0"
  "llvm.loongarch.lsx.vaddi.hu\0"
  "llvm.loongarch.lsx.vaddi.wu\0"
  "llvm.loongarch.lsx.vaddwev.d.w\0"
  "llvm.loongarch.lsx.vaddwev.d.wu\0"
  "llvm.loongarch.lsx.vaddwev.d.wu.w\0"
  "llvm.loongarch.lsx.vaddwev.h.b\0"
  "llvm.loongarch.lsx.vaddwev.h.bu\0"
  "llvm.loongarch.lsx.vaddwev.h.bu.b\0"
  "llvm.loongarch.lsx.vaddwev.q.d\0"
  "llvm.loongarch.lsx.vaddwev.q.du\0"
  "llvm.loongarch.lsx.vaddwev.q.du.d\0"
  "llvm.loongarch.lsx.vaddwev.w.h\0"
  "llvm.loongarch.lsx.vaddwev.w.hu\0"
  "llvm.loongarch.lsx.vaddwev.w.hu.h\0"
  "llvm.loongarch.lsx.vaddwod.d.w\0"
  "llvm.loongarch.lsx.vaddwod.d.wu\0"
  "llvm.loongarch.lsx.vaddwod.d.wu.w\0"
  "llvm.loongarch.lsx.vaddwod.h.b\0"
  "llvm.loongarch.lsx.vaddwod.h.bu\0"
  "llvm.loongarch.lsx.vaddwod.h.bu.b\0"
  "llvm.loongarch.lsx.vaddwod.q.d\0"
  "llvm.loongarch.lsx.vaddwod.q.du\0"
  "llvm.loongarch.lsx.vaddwod.q.du.d\0"
  "llvm.loongarch.lsx.vaddwod.w.h\0"
  "llvm.loongarch.lsx.vaddwod.w.hu\0"
  "llvm.loongarch.lsx.vaddwod.w.hu.h\0"
  "llvm.loongarch.lsx.vand.v\0"
  "llvm.loongarch.lsx.vandi.b\0"
  "llvm.loongarch.lsx.vandn.v\0"
  "llvm.loongarch.lsx.vavg.b\0"
  "llvm.loongarch.lsx.vavg.bu\0"
  "llvm.loongarch.lsx.vavg.d\0"
  "llvm.loongarch.lsx.vavg.du\0"
  "llvm.loongarch.lsx.vavg.h\0"
  "llvm.loongarch.lsx.vavg.hu\0"
  "llvm.loongarch.lsx.vavg.w\0"
  "llvm.loongarch.lsx.vavg.wu\0"
  "llvm.loongarch.lsx.vavgr.b\0"
  "llvm.loongarch.lsx.vavgr.bu\0"
  "llvm.loongarch.lsx.vavgr.d\0"
  "llvm.loongarch.lsx.vavgr.du\0"
  "llvm.loongarch.lsx.vavgr.h\0"
  "llvm.loongarch.lsx.vavgr.hu\0"
  "llvm.loongarch.lsx.vavgr.w\0"
  "llvm.loongarch.lsx.vavgr.wu\0"
  "llvm.loongarch.lsx.vbitclr.b\0"
  "llvm.loongarch.lsx.vbitclr.d\0"
  "llvm.loongarch.lsx.vbitclr.h\0"
  "llvm.loongarch.lsx.vbitclr.w\0"
  "llvm.loongarch.lsx.vbitclri.b\0"
  "llvm.loongarch.lsx.vbitclri.d\0"
  "llvm.loongarch.lsx.vbitclri.h\0"
  "llvm.loongarch.lsx.vbitclri.w\0"
  "llvm.loongarch.lsx.vbitrev.b\0"
  "llvm.loongarch.lsx.vbitrev.d\0"
  "llvm.loongarch.lsx.vbitrev.h\0"
  "llvm.loongarch.lsx.vbitrev.w\0"
  "llvm.loongarch.lsx.vbitrevi.b\0"
  "llvm.loongarch.lsx.vbitrevi.d\0"
  "llvm.loongarch.lsx.vbitrevi.h\0"
  "llvm.loongarch.lsx.vbitrevi.w\0"
  "llvm.loongarch.lsx.vbitsel.v\0"
  "llvm.loongarch.lsx.vbitseli.b\0"
  "llvm.loongarch.lsx.vbitset.b\0"
  "llvm.loongarch.lsx.vbitset.d\0"
  "llvm.loongarch.lsx.vbitset.h\0"
  "llvm.loongarch.lsx.vbitset.w\0"
  "llvm.loongarch.lsx.vbitseti.b\0"
  "llvm.loongarch.lsx.vbitseti.d\0"
  "llvm.loongarch.lsx.vbitseti.h\0"
  "llvm.loongarch.lsx.vbitseti.w\0"
  "llvm.loongarch.lsx.vbsll.v\0"
  "llvm.loongarch.lsx.vbsrl.v\0"
  "llvm.loongarch.lsx.vclo.b\0"
  "llvm.loongarch.lsx.vclo.d\0"
  "llvm.loongarch.lsx.vclo.h\0"
  "llvm.loongarch.lsx.vclo.w\0"
  "llvm.loongarch.lsx.vclz.b\0"
  "llvm.loongarch.lsx.vclz.d\0"
  "llvm.loongarch.lsx.vclz.h\0"
  "llvm.loongarch.lsx.vclz.w\0"
  "llvm.loongarch.lsx.vdiv.b\0"
  "llvm.loongarch.lsx.vdiv.bu\0"
  "llvm.loongarch.lsx.vdiv.d\0"
  "llvm.loongarch.lsx.vdiv.du\0"
  "llvm.loongarch.lsx.vdiv.h\0"
  "llvm.loongarch.lsx.vdiv.hu\0"
  "llvm.loongarch.lsx.vdiv.w\0"
  "llvm.loongarch.lsx.vdiv.wu\0"
  "llvm.loongarch.lsx.vexth.d.w\0"
  "llvm.loongarch.lsx.vexth.du.wu\0"
  "llvm.loongarch.lsx.vexth.h.b\0"
  "llvm.loongarch.lsx.vexth.hu.bu\0"
  "llvm.loongarch.lsx.vexth.q.d\0"
  "llvm.loongarch.lsx.vexth.qu.du\0"
  "llvm.loongarch.lsx.vexth.w.h\0"
  "llvm.loongarch.lsx.vexth.wu.hu\0"
  "llvm.loongarch.lsx.vextl.q.d\0"
  "llvm.loongarch.lsx.vextl.qu.du\0"
  "llvm.loongarch.lsx.vextrins.b\0"
  "llvm.loongarch.lsx.vextrins.d\0"
  "llvm.loongarch.lsx.vextrins.h\0"
  "llvm.loongarch.lsx.vextrins.w\0"
  "llvm.loongarch.lsx.vfadd.d\0"
  "llvm.loongarch.lsx.vfadd.s\0"
  "llvm.loongarch.lsx.vfclass.d\0"
  "llvm.loongarch.lsx.vfclass.s\0"
  "llvm.loongarch.lsx.vfcmp.caf.d\0"
  "llvm.loongarch.lsx.vfcmp.caf.s\0"
  "llvm.loongarch.lsx.vfcmp.ceq.d\0"
  "llvm.loongarch.lsx.vfcmp.ceq.s\0"
  "llvm.loongarch.lsx.vfcmp.cle.d\0"
  "llvm.loongarch.lsx.vfcmp.cle.s\0"
  "llvm.loongarch.lsx.vfcmp.clt.d\0"
  "llvm.loongarch.lsx.vfcmp.clt.s\0"
  "llvm.loongarch.lsx.vfcmp.cne.d\0"
  "llvm.loongarch.lsx.vfcmp.cne.s\0"
  "llvm.loongarch.lsx.vfcmp.cor.d\0"
  "llvm.loongarch.lsx.vfcmp.cor.s\0"
  "llvm.loongarch.lsx.vfcmp.cueq.d\0"
  "llvm.loongarch.lsx.vfcmp.cueq.s\0"
  "llvm.loongarch.lsx.vfcmp.cule.d\0"
  "llvm.loongarch.lsx.vfcmp.cule.s\0"
  "llvm.loongarch.lsx.vfcmp.cult.d\0"
  "llvm.loongarch.lsx.vfcmp.cult.s\0"
  "llvm.loongarch.lsx.vfcmp.cun.d\0"
  "llvm.loongarch.lsx.vfcmp.cun.s\0"
  "llvm.loongarch.lsx.vfcmp.cune.d\0"
  "llvm.loongarch.lsx.vfcmp.cune.s\0"
  "llvm.loongarch.lsx.vfcmp.saf.d\0"
  "llvm.loongarch.lsx.vfcmp.saf.s\0"
  "llvm.loongarch.lsx.vfcmp.seq.d\0"
  "llvm.loongarch.lsx.vfcmp.seq.s\0"
  "llvm.loongarch.lsx.vfcmp.sle.d\0"
  "llvm.loongarch.lsx.vfcmp.sle.s\0"
  "llvm.loongarch.lsx.vfcmp.slt.d\0"
  "llvm.loongarch.lsx.vfcmp.slt.s\0"
  "llvm.loongarch.lsx.vfcmp.sne.d\0"
  "llvm.loongarch.lsx.vfcmp.sne.s\0"
  "llvm.loongarch.lsx.vfcmp.sor.d\0"
  "llvm.loongarch.lsx.vfcmp.sor.s\0"
  "llvm.loongarch.lsx.vfcmp.sueq.d\0"
  "llvm.loongarch.lsx.vfcmp.sueq.s\0"
  "llvm.loongarch.lsx.vfcmp.sule.d\0"
  "llvm.loongarch.lsx.vfcmp.sule.s\0"
  "llvm.loongarch.lsx.vfcmp.sult.d\0"
  "llvm.loongarch.lsx.vfcmp.sult.s\0"
  "llvm.loongarch.lsx.vfcmp.sun.d\0"
  "llvm.loongarch.lsx.vfcmp.sun.s\0"
  "llvm.loongarch.lsx.vfcmp.sune.d\0"
  "llvm.loongarch.lsx.vfcmp.sune.s\0"
  "llvm.loongarch.lsx.vfcvt.h.s\0"
  "llvm.loongarch.lsx.vfcvt.s.d\0"
  "llvm.loongarch.lsx.vfcvth.d.s\0"
  "llvm.loongarch.lsx.vfcvth.s.h\0"
  "llvm.loongarch.lsx.vfcvtl.d.s\0"
  "llvm.loongarch.lsx.vfcvtl.s.h\0"
  "llvm.loongarch.lsx.vfdiv.d\0"
  "llvm.loongarch.lsx.vfdiv.s\0"
  "llvm.loongarch.lsx.vffint.d.l\0"
  "llvm.loongarch.lsx.vffint.d.lu\0"
  "llvm.loongarch.lsx.vffint.s.l\0"
  "llvm.loongarch.lsx.vffint.s.w\0"
  "llvm.loongarch.lsx.vffint.s.wu\0"
  "llvm.loongarch.lsx.vffinth.d.w\0"
  "llvm.loongarch.lsx.vffintl.d.w\0"
  "llvm.loongarch.lsx.vflogb.d\0"
  "llvm.loongarch.lsx.vflogb.s\0"
  "llvm.loongarch.lsx.vfmadd.d\0"
  "llvm.loongarch.lsx.vfmadd.s\0"
  "llvm.loongarch.lsx.vfmax.d\0"
  "llvm.loongarch.lsx.vfmax.s\0"
  "llvm.loongarch.lsx.vfmaxa.d\0"
  "llvm.loongarch.lsx.vfmaxa.s\0"
  "llvm.loongarch.lsx.vfmin.d\0"
  "llvm.loongarch.lsx.vfmin.s\0"
  "llvm.loongarch.lsx.vfmina.d\0"
  "llvm.loongarch.lsx.vfmina.s\0"
  "llvm.loongarch.lsx.vfmsub.d\0"
  "llvm.loongarch.lsx.vfmsub.s\0"
  "llvm.loongarch.lsx.vfmul.d\0"
  "llvm.loongarch.lsx.vfmul.s\0"
  "llvm.loongarch.lsx.vfnmadd.d\0"
  "llvm.loongarch.lsx.vfnmadd.s\0"
  "llvm.loongarch.lsx.vfnmsub.d\0"
  "llvm.loongarch.lsx.vfnmsub.s\0"
  "llvm.loongarch.lsx.vfrecip.d\0"
  "llvm.loongarch.lsx.vfrecip.s\0"
  "llvm.loongarch.lsx.vfrecipe.d\0"
  "llvm.loongarch.lsx.vfrecipe.s\0"
  "llvm.loongarch.lsx.vfrint.d\0"
  "llvm.loongarch.lsx.vfrint.s\0"
  "llvm.loongarch.lsx.vfrintrm.d\0"
  "llvm.loongarch.lsx.vfrintrm.s\0"
  "llvm.loongarch.lsx.vfrintrne.d\0"
  "llvm.loongarch.lsx.vfrintrne.s\0"
  "llvm.loongarch.lsx.vfrintrp.d\0"
  "llvm.loongarch.lsx.vfrintrp.s\0"
  "llvm.loongarch.lsx.vfrintrz.d\0"
  "llvm.loongarch.lsx.vfrintrz.s\0"
  "llvm.loongarch.lsx.vfrsqrt.d\0"
  "llvm.loongarch.lsx.vfrsqrt.s\0"
  "llvm.loongarch.lsx.vfrsqrte.d\0"
  "llvm.loongarch.lsx.vfrsqrte.s\0"
  "llvm.loongarch.lsx.vfrstp.b\0"
  "llvm.loongarch.lsx.vfrstp.h\0"
  "llvm.loongarch.lsx.vfrstpi.b\0"
  "llvm.loongarch.lsx.vfrstpi.h\0"
  "llvm.loongarch.lsx.vfsqrt.d\0"
  "llvm.loongarch.lsx.vfsqrt.s\0"
  "llvm.loongarch.lsx.vfsub.d\0"
  "llvm.loongarch.lsx.vfsub.s\0"
  "llvm.loongarch.lsx.vftint.l.d\0"
  "llvm.loongarch.lsx.vftint.lu.d\0"
  "llvm.loongarch.lsx.vftint.w.d\0"
  "llvm.loongarch.lsx.vftint.w.s\0"
  "llvm.loongarch.lsx.vftint.wu.s\0"
  "llvm.loongarch.lsx.vftinth.l.s\0"
  "llvm.loongarch.lsx.vftintl.l.s\0"
  "llvm.loongarch.lsx.vftintrm.l.d\0"
  "llvm.loongarch.lsx.vftintrm.w.d\0"
  "llvm.loongarch.lsx.vftintrm.w.s\0"
  "llvm.loongarch.lsx.vftintrmh.l.s\0"
  "llvm.loongarch.lsx.vftintrml.l.s\0"
  "llvm.loongarch.lsx.vftintrne.l.d\0"
  "llvm.loongarch.lsx.vftintrne.w.d\0"
  "llvm.loongarch.lsx.vftintrne.w.s\0"
  "llvm.loongarch.lsx.vftintrneh.l.s\0"
  "llvm.loongarch.lsx.vftintrnel.l.s\0"
  "llvm.loongarch.lsx.vftintrp.l.d\0"
  "llvm.loongarch.lsx.vftintrp.w.d\0"
  "llvm.loongarch.lsx.vftintrp.w.s\0"
  "llvm.loongarch.lsx.vftintrph.l.s\0"
  "llvm.loongarch.lsx.vftintrpl.l.s\0"
  "llvm.loongarch.lsx.vftintrz.l.d\0"
  "llvm.loongarch.lsx.vftintrz.lu.d\0"
  "llvm.loongarch.lsx.vftintrz.w.d\0"
  "llvm.loongarch.lsx.vftintrz.w.s\0"
  "llvm.loongarch.lsx.vftintrz.wu.s\0"
  "llvm.loongarch.lsx.vftintrzh.l.s\0"
  "llvm.loongarch.lsx.vftintrzl.l.s\0"
  "llvm.loongarch.lsx.vhaddw.d.w\0"
  "llvm.loongarch.lsx.vhaddw.du.wu\0"
  "llvm.loongarch.lsx.vhaddw.h.b\0"
  "llvm.loongarch.lsx.vhaddw.hu.bu\0"
  "llvm.loongarch.lsx.vhaddw.q.d\0"
  "llvm.loongarch.lsx.vhaddw.qu.du\0"
  "llvm.loongarch.lsx.vhaddw.w.h\0"
  "llvm.loongarch.lsx.vhaddw.wu.hu\0"
  "llvm.loongarch.lsx.vhsubw.d.w\0"
  "llvm.loongarch.lsx.vhsubw.du.wu\0"
  "llvm.loongarch.lsx.vhsubw.h.b\0"
  "llvm.loongarch.lsx.vhsubw.hu.bu\0"
  "llvm.loongarch.lsx.vhsubw.q.d\0"
  "llvm.loongarch.lsx.vhsubw.qu.du\0"
  "llvm.loongarch.lsx.vhsubw.w.h\0"
  "llvm.loongarch.lsx.vhsubw.wu.hu\0"
  "llvm.loongarch.lsx.vilvh.b\0"
  "llvm.loongarch.lsx.vilvh.d\0"
  "llvm.loongarch.lsx.vilvh.h\0"
  "llvm.loongarch.lsx.vilvh.w\0"
  "llvm.loongarch.lsx.vilvl.b\0"
  "llvm.loongarch.lsx.vilvl.d\0"
  "llvm.loongarch.lsx.vilvl.h\0"
  "llvm.loongarch.lsx.vilvl.w\0"
  "llvm.loongarch.lsx.vinsgr2vr.b\0"
  "llvm.loongarch.lsx.vinsgr2vr.d\0"
  "llvm.loongarch.lsx.vinsgr2vr.h\0"
  "llvm.loongarch.lsx.vinsgr2vr.w\0"
  "llvm.loongarch.lsx.vld\0"
  "llvm.loongarch.lsx.vldi\0"
  "llvm.loongarch.lsx.vldrepl.b\0"
  "llvm.loongarch.lsx.vldrepl.d\0"
  "llvm.loongarch.lsx.vldrepl.h\0"
  "llvm.loongarch.lsx.vldrepl.w\0"
  "llvm.loongarch.lsx.vldx\0"
  "llvm.loongarch.lsx.vmadd.b\0"
  "llvm.loongarch.lsx.vmadd.d\0"
  "llvm.loongarch.lsx.vmadd.h\0"
  "llvm.loongarch.lsx.vmadd.w\0"
  "llvm.loongarch.lsx.vmaddwev.d.w\0"
  "llvm.loongarch.lsx.vmaddwev.d.wu\0"
  "llvm.loongarch.lsx.vmaddwev.d.wu.w\0"
  "llvm.loongarch.lsx.vmaddwev.h.b\0"
  "llvm.loongarch.lsx.vmaddwev.h.bu\0"
  "llvm.loongarch.lsx.vmaddwev.h.bu.b\0"
  "llvm.loongarch.lsx.vmaddwev.q.d\0"
  "llvm.loongarch.lsx.vmaddwev.q.du\0"
  "llvm.loongarch.lsx.vmaddwev.q.du.d\0"
  "llvm.loongarch.lsx.vmaddwev.w.h\0"
  "llvm.loongarch.lsx.vmaddwev.w.hu\0"
  "llvm.loongarch.lsx.vmaddwev.w.hu.h\0"
  "llvm.loongarch.lsx.vmaddwod.d.w\0"
  "llvm.loongarch.lsx.vmaddwod.d.wu\0"
  "llvm.loongarch.lsx.vmaddwod.d.wu.w\0"
  "llvm.loongarch.lsx.vmaddwod.h.b\0"
  "llvm.loongarch.lsx.vmaddwod.h.bu\0"
  "llvm.loongarch.lsx.vmaddwod.h.bu.b\0"
  "llvm.loongarch.lsx.vmaddwod.q.d\0"
  "llvm.loongarch.lsx.vmaddwod.q.du\0"
  "llvm.loongarch.lsx.vmaddwod.q.du.d\0"
  "llvm.loongarch.lsx.vmaddwod.w.h\0"
  "llvm.loongarch.lsx.vmaddwod.w.hu\0"
  "llvm.loongarch.lsx.vmaddwod.w.hu.h\0"
  "llvm.loongarch.lsx.vmax.b\0"
  "llvm.loongarch.lsx.vmax.bu\0"
  "llvm.loongarch.lsx.vmax.d\0"
  "llvm.loongarch.lsx.vmax.du\0"
  "llvm.loongarch.lsx.vmax.h\0"
  "llvm.loongarch.lsx.vmax.hu\0"
  "llvm.loongarch.lsx.vmax.w\0"
  "llvm.loongarch.lsx.vmax.wu\0"
  "llvm.loongarch.lsx.vmaxi.b\0"
  "llvm.loongarch.lsx.vmaxi.bu\0"
  "llvm.loongarch.lsx.vmaxi.d\0"
  "llvm.loongarch.lsx.vmaxi.du\0"
  "llvm.loongarch.lsx.vmaxi.h\0"
  "llvm.loongarch.lsx.vmaxi.hu\0"
  "llvm.loongarch.lsx.vmaxi.w\0"
  "llvm.loongarch.lsx.vmaxi.wu\0"
  "llvm.loongarch.lsx.vmin.b\0"
  "llvm.loongarch.lsx.vmin.bu\0"
  "llvm.loongarch.lsx.vmin.d\0"
  "llvm.loongarch.lsx.vmin.du\0"
  "llvm.loongarch.lsx.vmin.h\0"
  "llvm.loongarch.lsx.vmin.hu\0"
  "llvm.loongarch.lsx.vmin.w\0"
  "llvm.loongarch.lsx.vmin.wu\0"
  "llvm.loongarch.lsx.vmini.b\0"
  "llvm.loongarch.lsx.vmini.bu\0"
  "llvm.loongarch.lsx.vmini.d\0"
  "llvm.loongarch.lsx.vmini.du\0"
  "llvm.loongarch.lsx.vmini.h\0"
  "llvm.loongarch.lsx.vmini.hu\0"
  "llvm.loongarch.lsx.vmini.w\0"
  "llvm.loongarch.lsx.vmini.wu\0"
  "llvm.loongarch.lsx.vmod.b\0"
  "llvm.loongarch.lsx.vmod.bu\0"
  "llvm.loongarch.lsx.vmod.d\0"
  "llvm.loongarch.lsx.vmod.du\0"
  "llvm.loongarch.lsx.vmod.h\0"
  "llvm.loongarch.lsx.vmod.hu\0"
  "llvm.loongarch.lsx.vmod.w\0"
  "llvm.loongarch.lsx.vmod.wu\0"
  "llvm.loongarch.lsx.vmskgez.b\0"
  "llvm.loongarch.lsx.vmskltz.b\0"
  "llvm.loongarch.lsx.vmskltz.d\0"
  "llvm.loongarch.lsx.vmskltz.h\0"
  "llvm.loongarch.lsx.vmskltz.w\0"
  "llvm.loongarch.lsx.vmsknz.b\0"
  "llvm.loongarch.lsx.vmsub.b\0"
  "llvm.loongarch.lsx.vmsub.d\0"
  "llvm.loongarch.lsx.vmsub.h\0"
  "llvm.loongarch.lsx.vmsub.w\0"
  "llvm.loongarch.lsx.vmuh.b\0"
  "llvm.loongarch.lsx.vmuh.bu\0"
  "llvm.loongarch.lsx.vmuh.d\0"
  "llvm.loongarch.lsx.vmuh.du\0"
  "llvm.loongarch.lsx.vmuh.h\0"
  "llvm.loongarch.lsx.vmuh.hu\0"
  "llvm.loongarch.lsx.vmuh.w\0"
  "llvm.loongarch.lsx.vmuh.wu\0"
  "llvm.loongarch.lsx.vmul.b\0"
  "llvm.loongarch.lsx.vmul.d\0"
  "llvm.loongarch.lsx.vmul.h\0"
  "llvm.loongarch.lsx.vmul.w\0"
  "llvm.loongarch.lsx.vmulwev.d.w\0"
  "llvm.loongarch.lsx.vmulwev.d.wu\0"
  "llvm.loongarch.lsx.vmulwev.d.wu.w\0"
  "llvm.loongarch.lsx.vmulwev.h.b\0"
  "llvm.loongarch.lsx.vmulwev.h.bu\0"
  "llvm.loongarch.lsx.vmulwev.h.bu.b\0"
  "llvm.loongarch.lsx.vmulwev.q.d\0"
  "llvm.loongarch.lsx.vmulwev.q.du\0"
  "llvm.loongarch.lsx.vmulwev.q.du.d\0"
  "llvm.loongarch.lsx.vmulwev.w.h\0"
  "llvm.loongarch.lsx.vmulwev.w.hu\0"
  "llvm.loongarch.lsx.vmulwev.w.hu.h\0"
  "llvm.loongarch.lsx.vmulwod.d.w\0"
  "llvm.loongarch.lsx.vmulwod.d.wu\0"
  "llvm.loongarch.lsx.vmulwod.d.wu.w\0"
  "llvm.loongarch.lsx.vmulwod.h.b\0"
  "llvm.loongarch.lsx.vmulwod.h.bu\0"
  "llvm.loongarch.lsx.vmulwod.h.bu.b\0"
  "llvm.loongarch.lsx.vmulwod.q.d\0"
  "llvm.loongarch.lsx.vmulwod.q.du\0"
  "llvm.loongarch.lsx.vmulwod.q.du.d\0"
  "llvm.loongarch.lsx.vmulwod.w.h\0"
  "llvm.loongarch.lsx.vmulwod.w.hu\0"
  "llvm.loongarch.lsx.vmulwod.w.hu.h\0"
  "llvm.loongarch.lsx.vneg.b\0"
  "llvm.loongarch.lsx.vneg.d\0"
  "llvm.loongarch.lsx.vneg.h\0"
  "llvm.loongarch.lsx.vneg.w\0"
  "llvm.loongarch.lsx.vnor.v\0"
  "llvm.loongarch.lsx.vnori.b\0"
  "llvm.loongarch.lsx.vor.v\0"
  "llvm.loongarch.lsx.vori.b\0"
  "llvm.loongarch.lsx.vorn.v\0"
  "llvm.loongarch.lsx.vpackev.b\0"
  "llvm.loongarch.lsx.vpackev.d\0"
  "llvm.loongarch.lsx.vpackev.h\0"
  "llvm.loongarch.lsx.vpackev.w\0"
  "llvm.loongarch.lsx.vpackod.b\0"
  "llvm.loongarch.lsx.vpackod.d\0"
  "llvm.loongarch.lsx.vpackod.h\0"
  "llvm.loongarch.lsx.vpackod.w\0"
  "llvm.loongarch.lsx.vpcnt.b\0"
  "llvm.loongarch.lsx.vpcnt.d\0"
  "llvm.loongarch.lsx.vpcnt.h\0"
  "llvm.loongarch.lsx.vpcnt.w\0"
  "llvm.loongarch.lsx.vpermi.w\0"
  "llvm.loongarch.lsx.vpickev.b\0"
  "llvm.loongarch.lsx.vpickev.d\0"
  "llvm.loongarch.lsx.vpickev.h\0"
  "llvm.loongarch.lsx.vpickev.w\0"
  "llvm.loongarch.lsx.vpickod.b\0"
  "llvm.loongarch.lsx.vpickod.d\0"
  "llvm.loongarch.lsx.vpickod.h\0"
  "llvm.loongarch.lsx.vpickod.w\0"
  "llvm.loongarch.lsx.vpickve2gr.b\0"
  "llvm.loongarch.lsx.vpickve2gr.bu\0"
  "llvm.loongarch.lsx.vpickve2gr.d\0"
  "llvm.loongarch.lsx.vpickve2gr.du\0"
  "llvm.loongarch.lsx.vpickve2gr.h\0"
  "llvm.loongarch.lsx.vpickve2gr.hu\0"
  "llvm.loongarch.lsx.vpickve2gr.w\0"
  "llvm.loongarch.lsx.vpickve2gr.wu\0"
  "llvm.loongarch.lsx.vreplgr2vr.b\0"
  "llvm.loongarch.lsx.vreplgr2vr.d\0"
  "llvm.loongarch.lsx.vreplgr2vr.h\0"
  "llvm.loongarch.lsx.vreplgr2vr.w\0"
  "llvm.loongarch.lsx.vrepli.b\0"
  "llvm.loongarch.lsx.vrepli.d\0"
  "llvm.loongarch.lsx.vrepli.h\0"
  "llvm.loongarch.lsx.vrepli.w\0"
  "llvm.loongarch.lsx.vreplve.b\0"
  "llvm.loongarch.lsx.vreplve.d\0"
  "llvm.loongarch.lsx.vreplve.h\0"
  "llvm.loongarch.lsx.vreplve.w\0"
  "llvm.loongarch.lsx.vreplvei.b\0"
  "llvm.loongarch.lsx.vreplvei.d\0"
  "llvm.loongarch.lsx.vreplvei.h\0"
  "llvm.loongarch.lsx.vreplvei.w\0"
  "llvm.loongarch.lsx.vrotr.b\0"
  "llvm.loongarch.lsx.vrotr.d\0"
  "llvm.loongarch.lsx.vrotr.h\0"
  "llvm.loongarch.lsx.vrotr.w\0"
  "llvm.loongarch.lsx.vrotri.b\0"
  "llvm.loongarch.lsx.vrotri.d\0"
  "llvm.loongarch.lsx.vrotri.h\0"
  "llvm.loongarch.lsx.vrotri.w\0"
  "llvm.loongarch.lsx.vsadd.b\0"
  "llvm.loongarch.lsx.vsadd.bu\0"
  "llvm.loongarch.lsx.vsadd.d\0"
  "llvm.loongarch.lsx.vsadd.du\0"
  "llvm.loongarch.lsx.vsadd.h\0"
  "llvm.loongarch.lsx.vsadd.hu\0"
  "llvm.loongarch.lsx.vsadd.w\0"
  "llvm.loongarch.lsx.vsadd.wu\0"
  "llvm.loongarch.lsx.vsat.b\0"
  "llvm.loongarch.lsx.vsat.bu\0"
  "llvm.loongarch.lsx.vsat.d\0"
  "llvm.loongarch.lsx.vsat.du\0"
  "llvm.loongarch.lsx.vsat.h\0"
  "llvm.loongarch.lsx.vsat.hu\0"
  "llvm.loongarch.lsx.vsat.w\0"
  "llvm.loongarch.lsx.vsat.wu\0"
  "llvm.loongarch.lsx.vseq.b\0"
  "llvm.loongarch.lsx.vseq.d\0"
  "llvm.loongarch.lsx.vseq.h\0"
  "llvm.loongarch.lsx.vseq.w\0"
  "llvm.loongarch.lsx.vseqi.b\0"
  "llvm.loongarch.lsx.vseqi.d\0"
  "llvm.loongarch.lsx.vseqi.h\0"
  "llvm.loongarch.lsx.vseqi.w\0"
  "llvm.loongarch.lsx.vshuf.b\0"
  "llvm.loongarch.lsx.vshuf.d\0"
  "llvm.loongarch.lsx.vshuf.h\0"
  "llvm.loongarch.lsx.vshuf.w\0"
  "llvm.loongarch.lsx.vshuf4i.b\0"
  "llvm.loongarch.lsx.vshuf4i.d\0"
  "llvm.loongarch.lsx.vshuf4i.h\0"
  "llvm.loongarch.lsx.vshuf4i.w\0"
  "llvm.loongarch.lsx.vsigncov.b\0"
  "llvm.loongarch.lsx.vsigncov.d\0"
  "llvm.loongarch.lsx.vsigncov.h\0"
  "llvm.loongarch.lsx.vsigncov.w\0"
  "llvm.loongarch.lsx.vsle.b\0"
  "llvm.loongarch.lsx.vsle.bu\0"
  "llvm.loongarch.lsx.vsle.d\0"
  "llvm.loongarch.lsx.vsle.du\0"
  "llvm.loongarch.lsx.vsle.h\0"
  "llvm.loongarch.lsx.vsle.hu\0"
  "llvm.loongarch.lsx.vsle.w\0"
  "llvm.loongarch.lsx.vsle.wu\0"
  "llvm.loongarch.lsx.vslei.b\0"
  "llvm.loongarch.lsx.vslei.bu\0"
  "llvm.loongarch.lsx.vslei.d\0"
  "llvm.loongarch.lsx.vslei.du\0"
  "llvm.loongarch.lsx.vslei.h\0"
  "llvm.loongarch.lsx.vslei.hu\0"
  "llvm.loongarch.lsx.vslei.w\0"
  "llvm.loongarch.lsx.vslei.wu\0"
  "llvm.loongarch.lsx.vsll.b\0"
  "llvm.loongarch.lsx.vsll.d\0"
  "llvm.loongarch.lsx.vsll.h\0"
  "llvm.loongarch.lsx.vsll.w\0"
  "llvm.loongarch.lsx.vslli.b\0"
  "llvm.loongarch.lsx.vslli.d\0"
  "llvm.loongarch.lsx.vslli.h\0"
  "llvm.loongarch.lsx.vslli.w\0"
  "llvm.loongarch.lsx.vsllwil.d.w\0"
  "llvm.loongarch.lsx.vsllwil.du.wu\0"
  "llvm.loongarch.lsx.vsllwil.h.b\0"
  "llvm.loongarch.lsx.vsllwil.hu.bu\0"
  "llvm.loongarch.lsx.vsllwil.w.h\0"
  "llvm.loongarch.lsx.vsllwil.wu.hu\0"
  "llvm.loongarch.lsx.vslt.b\0"
  "llvm.loongarch.lsx.vslt.bu\0"
  "llvm.loongarch.lsx.vslt.d\0"
  "llvm.loongarch.lsx.vslt.du\0"
  "llvm.loongarch.lsx.vslt.h\0"
  "llvm.loongarch.lsx.vslt.hu\0"
  "llvm.loongarch.lsx.vslt.w\0"
  "llvm.loongarch.lsx.vslt.wu\0"
  "llvm.loongarch.lsx.vslti.b\0"
  "llvm.loongarch.lsx.vslti.bu\0"
  "llvm.loongarch.lsx.vslti.d\0"
  "llvm.loongarch.lsx.vslti.du\0"
  "llvm.loongarch.lsx.vslti.h\0"
  "llvm.loongarch.lsx.vslti.hu\0"
  "llvm.loongarch.lsx.vslti.w\0"
  "llvm.loongarch.lsx.vslti.wu\0"
  "llvm.loongarch.lsx.vsra.b\0"
  "llvm.loongarch.lsx.vsra.d\0"
  "llvm.loongarch.lsx.vsra.h\0"
  "llvm.loongarch.lsx.vsra.w\0"
  "llvm.loongarch.lsx.vsrai.b\0"
  "llvm.loongarch.lsx.vsrai.d\0"
  "llvm.loongarch.lsx.vsrai.h\0"
  "llvm.loongarch.lsx.vsrai.w\0"
  "llvm.loongarch.lsx.vsran.b.h\0"
  "llvm.loongarch.lsx.vsran.h.w\0"
  "llvm.loongarch.lsx.vsran.w.d\0"
  "llvm.loongarch.lsx.vsrani.b.h\0"
  "llvm.loongarch.lsx.vsrani.d.q\0"
  "llvm.loongarch.lsx.vsrani.h.w\0"
  "llvm.loongarch.lsx.vsrani.w.d\0"
  "llvm.loongarch.lsx.vsrar.b\0"
  "llvm.loongarch.lsx.vsrar.d\0"
  "llvm.loongarch.lsx.vsrar.h\0"
  "llvm.loongarch.lsx.vsrar.w\0"
  "llvm.loongarch.lsx.vsrari.b\0"
  "llvm.loongarch.lsx.vsrari.d\0"
  "llvm.loongarch.lsx.vsrari.h\0"
  "llvm.loongarch.lsx.vsrari.w\0"
  "llvm.loongarch.lsx.vsrarn.b.h\0"
  "llvm.loongarch.lsx.vsrarn.h.w\0"
  "llvm.loongarch.lsx.vsrarn.w.d\0"
  "llvm.loongarch.lsx.vsrarni.b.h\0"
  "llvm.loongarch.lsx.vsrarni.d.q\0"
  "llvm.loongarch.lsx.vsrarni.h.w\0"
  "llvm.loongarch.lsx.vsrarni.w.d\0"
  "llvm.loongarch.lsx.vsrl.b\0"
  "llvm.loongarch.lsx.vsrl.d\0"
  "llvm.loongarch.lsx.vsrl.h\0"
  "llvm.loongarch.lsx.vsrl.w\0"
  "llvm.loongarch.lsx.vsrli.b\0"
  "llvm.loongarch.lsx.vsrli.d\0"
  "llvm.loongarch.lsx.vsrli.h\0"
  "llvm.loongarch.lsx.vsrli.w\0"
  "llvm.loongarch.lsx.vsrln.b.h\0"
  "llvm.loongarch.lsx.vsrln.h.w\0"
  "llvm.loongarch.lsx.vsrln.w.d\0"
  "llvm.loongarch.lsx.vsrlni.b.h\0"
  "llvm.loongarch.lsx.vsrlni.d.q\0"
  "llvm.loongarch.lsx.vsrlni.h.w\0"
  "llvm.loongarch.lsx.vsrlni.w.d\0"
  "llvm.loongarch.lsx.vsrlr.b\0"
  "llvm.loongarch.lsx.vsrlr.d\0"
  "llvm.loongarch.lsx.vsrlr.h\0"
  "llvm.loongarch.lsx.vsrlr.w\0"
  "llvm.loongarch.lsx.vsrlri.b\0"
  "llvm.loongarch.lsx.vsrlri.d\0"
  "llvm.loongarch.lsx.vsrlri.h\0"
  "llvm.loongarch.lsx.vsrlri.w\0"
  "llvm.loongarch.lsx.vsrlrn.b.h\0"
  "llvm.loongarch.lsx.vsrlrn.h.w\0"
  "llvm.loongarch.lsx.vsrlrn.w.d\0"
  "llvm.loongarch.lsx.vsrlrni.b.h\0"
  "llvm.loongarch.lsx.vsrlrni.d.q\0"
  "llvm.loongarch.lsx.vsrlrni.h.w\0"
  "llvm.loongarch.lsx.vsrlrni.w.d\0"
  "llvm.loongarch.lsx.vssran.b.h\0"
  "llvm.loongarch.lsx.vssran.bu.h\0"
  "llvm.loongarch.lsx.vssran.h.w\0"
  "llvm.loongarch.lsx.vssran.hu.w\0"
  "llvm.loongarch.lsx.vssran.w.d\0"
  "llvm.loongarch.lsx.vssran.wu.d\0"
  "llvm.loongarch.lsx.vssrani.b.h\0"
  "llvm.loongarch.lsx.vssrani.bu.h\0"
  "llvm.loongarch.lsx.vssrani.d.q\0"
  "llvm.loongarch.lsx.vssrani.du.q\0"
  "llvm.loongarch.lsx.vssrani.h.w\0"
  "llvm.loongarch.lsx.vssrani.hu.w\0"
  "llvm.loongarch.lsx.vssrani.w.d\0"
  "llvm.loongarch.lsx.vssrani.wu.d\0"
  "llvm.loongarch.lsx.vssrarn.b.h\0"
  "llvm.loongarch.lsx.vssrarn.bu.h\0"
  "llvm.loongarch.lsx.vssrarn.h.w\0"
  "llvm.loongarch.lsx.vssrarn.hu.w\0"
  "llvm.loongarch.lsx.vssrarn.w.d\0"
  "llvm.loongarch.lsx.vssrarn.wu.d\0"
  "llvm.loongarch.lsx.vssrarni.b.h\0"
  "llvm.loongarch.lsx.vssrarni.bu.h\0"
  "llvm.loongarch.lsx.vssrarni.d.q\0"
  "llvm.loongarch.lsx.vssrarni.du.q\0"
  "llvm.loongarch.lsx.vssrarni.h.w\0"
  "llvm.loongarch.lsx.vssrarni.hu.w\0"
  "llvm.loongarch.lsx.vssrarni.w.d\0"
  "llvm.loongarch.lsx.vssrarni.wu.d\0"
  "llvm.loongarch.lsx.vssrln.b.h\0"
  "llvm.loongarch.lsx.vssrln.bu.h\0"
  "llvm.loongarch.lsx.vssrln.h.w\0"
  "llvm.loongarch.lsx.vssrln.hu.w\0"
  "llvm.loongarch.lsx.vssrln.w.d\0"
  "llvm.loongarch.lsx.vssrln.wu.d\0"
  "llvm.loongarch.lsx.vssrlni.b.h\0"
  "llvm.loongarch.lsx.vssrlni.bu.h\0"
  "llvm.loongarch.lsx.vssrlni.d.q\0"
  "llvm.loongarch.lsx.vssrlni.du.q\0"
  "llvm.loongarch.lsx.vssrlni.h.w\0"
  "llvm.loongarch.lsx.vssrlni.hu.w\0"
  "llvm.loongarch.lsx.vssrlni.w.d\0"
  "llvm.loongarch.lsx.vssrlni.wu.d\0"
  "llvm.loongarch.lsx.vssrlrn.b.h\0"
  "llvm.loongarch.lsx.vssrlrn.bu.h\0"
  "llvm.loongarch.lsx.vssrlrn.h.w\0"
  "llvm.loongarch.lsx.vssrlrn.hu.w\0"
  "llvm.loongarch.lsx.vssrlrn.w.d\0"
  "llvm.loongarch.lsx.vssrlrn.wu.d\0"
  "llvm.loongarch.lsx.vssrlrni.b.h\0"
  "llvm.loongarch.lsx.vssrlrni.bu.h\0"
  "llvm.loongarch.lsx.vssrlrni.d.q\0"
  "llvm.loongarch.lsx.vssrlrni.du.q\0"
  "llvm.loongarch.lsx.vssrlrni.h.w\0"
  "llvm.loongarch.lsx.vssrlrni.hu.w\0"
  "llvm.loongarch.lsx.vssrlrni.w.d\0"
  "llvm.loongarch.lsx.vssrlrni.wu.d\0"
  "llvm.loongarch.lsx.vssub.b\0"
  "llvm.loongarch.lsx.vssub.bu\0"
  "llvm.loongarch.lsx.vssub.d\0"
  "llvm.loongarch.lsx.vssub.du\0"
  "llvm.loongarch.lsx.vssub.h\0"
  "llvm.loongarch.lsx.vssub.hu\0"
  "llvm.loongarch.lsx.vssub.w\0"
  "llvm.loongarch.lsx.vssub.wu\0"
  "llvm.loongarch.lsx.vst\0"
  "llvm.loongarch.lsx.vstelm.b\0"
  "llvm.loongarch.lsx.vstelm.d\0"
  "llvm.loongarch.lsx.vstelm.h\0"
  "llvm.loongarch.lsx.vstelm.w\0"
  "llvm.loongarch.lsx.vstx\0"
  "llvm.loongarch.lsx.vsub.b\0"
  "llvm.loongarch.lsx.vsub.d\0"
  "llvm.loongarch.lsx.vsub.h\0"
  "llvm.loongarch.lsx.vsub.q\0"
  "llvm.loongarch.lsx.vsub.w\0"
  "llvm.loongarch.lsx.vsubi.bu\0"
  "llvm.loongarch.lsx.vsubi.du\0"
  "llvm.loongarch.lsx.vsubi.hu\0"
  "llvm.loongarch.lsx.vsubi.wu\0"
  "llvm.loongarch.lsx.vsubwev.d.w\0"
  "llvm.loongarch.lsx.vsubwev.d.wu\0"
  "llvm.loongarch.lsx.vsubwev.h.b\0"
  "llvm.loongarch.lsx.vsubwev.h.bu\0"
  "llvm.loongarch.lsx.vsubwev.q.d\0"
  "llvm.loongarch.lsx.vsubwev.q.du\0"
  "llvm.loongarch.lsx.vsubwev.w.h\0"
  "llvm.loongarch.lsx.vsubwev.w.hu\0"
  "llvm.loongarch.lsx.vsubwod.d.w\0"
  "llvm.loongarch.lsx.vsubwod.d.wu\0"
  "llvm.loongarch.lsx.vsubwod.h.b\0"
  "llvm.loongarch.lsx.vsubwod.h.bu\0"
  "llvm.loongarch.lsx.vsubwod.q.d\0"
  "llvm.loongarch.lsx.vsubwod.q.du\0"
  "llvm.loongarch.lsx.vsubwod.w.h\0"
  "llvm.loongarch.lsx.vsubwod.w.hu\0"
  "llvm.loongarch.lsx.vxor.v\0"
  "llvm.loongarch.lsx.vxori.b\0"
  "llvm.loongarch.masked.atomicrmw.add.i32\0"
  "llvm.loongarch.masked.atomicrmw.add.i64\0"
  "llvm.loongarch.masked.atomicrmw.max.i32\0"
  "llvm.loongarch.masked.atomicrmw.max.i64\0"
  "llvm.loongarch.masked.atomicrmw.min.i32\0"
  "llvm.loongarch.masked.atomicrmw.min.i64\0"
  "llvm.loongarch.masked.atomicrmw.nand.i32\0"
  "llvm.loongarch.masked.atomicrmw.nand.i64\0"
  "llvm.loongarch.masked.atomicrmw.sub.i32\0"
  "llvm.loongarch.masked.atomicrmw.sub.i64\0"
  "llvm.loongarch.masked.atomicrmw.umax.i32\0"
  "llvm.loongarch.masked.atomicrmw.umax.i64\0"
  "llvm.loongarch.masked.atomicrmw.umin.i32\0"
  "llvm.loongarch.masked.atomicrmw.umin.i64\0"
  "llvm.loongarch.masked.atomicrmw.xchg.i32\0"
  "llvm.loongarch.masked.atomicrmw.xchg.i64\0"
  "llvm.loongarch.masked.cmpxchg.i32\0"
  "llvm.loongarch.masked.cmpxchg.i64\0"
  "llvm.loongarch.movfcsr2gr\0"
  "llvm.loongarch.movgr2fcsr\0"
  "llvm.loongarch.syscall\0"
  "llvm.mips.absq.s.ph\0"
  "llvm.mips.absq.s.qb\0"
  "llvm.mips.absq.s.w\0"
  "llvm.mips.add.a.b\0"
  "llvm.mips.add.a.d\0"
  "llvm.mips.add.a.h\0"
  "llvm.mips.add.a.w\0"
  "llvm.mips.addq.ph\0"
  "llvm.mips.addq.s.ph\0"
  "llvm.mips.addq.s.w\0"
  "llvm.mips.addqh.ph\0"
  "llvm.mips.addqh.r.ph\0"
  "llvm.mips.addqh.r.w\0"
  "llvm.mips.addqh.w\0"
  "llvm.mips.adds.a.b\0"
  "llvm.mips.adds.a.d\0"
  "llvm.mips.adds.a.h\0"
  "llvm.mips.adds.a.w\0"
  "llvm.mips.adds.s.b\0"
  "llvm.mips.adds.s.d\0"
  "llvm.mips.adds.s.h\0"
  "llvm.mips.adds.s.w\0"
  "llvm.mips.adds.u.b\0"
  "llvm.mips.adds.u.d\0"
  "llvm.mips.adds.u.h\0"
  "llvm.mips.adds.u.w\0"
  "llvm.mips.addsc\0"
  "llvm.mips.addu.ph\0"
  "llvm.mips.addu.qb\0"
  "llvm.mips.addu.s.ph\0"
  "llvm.mips.addu.s.qb\0"
  "llvm.mips.adduh.qb\0"
  "llvm.mips.adduh.r.qb\0"
  "llvm.mips.addv.b\0"
  "llvm.mips.addv.d\0"
  "llvm.mips.addv.h\0"
  "llvm.mips.addv.w\0"
  "llvm.mips.addvi.b\0"
  "llvm.mips.addvi.d\0"
  "llvm.mips.addvi.h\0"
  "llvm.mips.addvi.w\0"
  "llvm.mips.addwc\0"
  "llvm.mips.and.v\0"
  "llvm.mips.andi.b\0"
  "llvm.mips.append\0"
  "llvm.mips.asub.s.b\0"
  "llvm.mips.asub.s.d\0"
  "llvm.mips.asub.s.h\0"
  "llvm.mips.asub.s.w\0"
  "llvm.mips.asub.u.b\0"
  "llvm.mips.asub.u.d\0"
  "llvm.mips.asub.u.h\0"
  "llvm.mips.asub.u.w\0"
  "llvm.mips.ave.s.b\0"
  "llvm.mips.ave.s.d\0"
  "llvm.mips.ave.s.h\0"
  "llvm.mips.ave.s.w\0"
  "llvm.mips.ave.u.b\0"
  "llvm.mips.ave.u.d\0"
  "llvm.mips.ave.u.h\0"
  "llvm.mips.ave.u.w\0"
  "llvm.mips.aver.s.b\0"
  "llvm.mips.aver.s.d\0"
  "llvm.mips.aver.s.h\0"
  "llvm.mips.aver.s.w\0"
  "llvm.mips.aver.u.b\0"
  "llvm.mips.aver.u.d\0"
  "llvm.mips.aver.u.h\0"
  "llvm.mips.aver.u.w\0"
  "llvm.mips.balign\0"
  "llvm.mips.bclr.b\0"
  "llvm.mips.bclr.d\0"
  "llvm.mips.bclr.h\0"
  "llvm.mips.bclr.w\0"
  "llvm.mips.bclri.b\0"
  "llvm.mips.bclri.d\0"
  "llvm.mips.bclri.h\0"
  "llvm.mips.bclri.w\0"
  "llvm.mips.binsl.b\0"
  "llvm.mips.binsl.d\0"
  "llvm.mips.binsl.h\0"
  "llvm.mips.binsl.w\0"
  "llvm.mips.binsli.b\0"
  "llvm.mips.binsli.d\0"
  "llvm.mips.binsli.h\0"
  "llvm.mips.binsli.w\0"
  "llvm.mips.binsr.b\0"
  "llvm.mips.binsr.d\0"
  "llvm.mips.binsr.h\0"
  "llvm.mips.binsr.w\0"
  "llvm.mips.binsri.b\0"
  "llvm.mips.binsri.d\0"
  "llvm.mips.binsri.h\0"
  "llvm.mips.binsri.w\0"
  "llvm.mips.bitrev\0"
  "llvm.mips.bmnz.v\0"
  "llvm.mips.bmnzi.b\0"
  "llvm.mips.bmz.v\0"
  "llvm.mips.bmzi.b\0"
  "llvm.mips.bneg.b\0"
  "llvm.mips.bneg.d\0"
  "llvm.mips.bneg.h\0"
  "llvm.mips.bneg.w\0"
  "llvm.mips.bnegi.b\0"
  "llvm.mips.bnegi.d\0"
  "llvm.mips.bnegi.h\0"
  "llvm.mips.bnegi.w\0"
  "llvm.mips.bnz.b\0"
  "llvm.mips.bnz.d\0"
  "llvm.mips.bnz.h\0"
  "llvm.mips.bnz.v\0"
  "llvm.mips.bnz.w\0"
  "llvm.mips.bposge32\0"
  "llvm.mips.bsel.v\0"
  "llvm.mips.bseli.b\0"
  "llvm.mips.bset.b\0"
  "llvm.mips.bset.d\0"
  "llvm.mips.bset.h\0"
  "llvm.mips.bset.w\0"
  "llvm.mips.bseti.b\0"
  "llvm.mips.bseti.d\0"
  "llvm.mips.bseti.h\0"
  "llvm.mips.bseti.w\0"
  "llvm.mips.bz.b\0"
  "llvm.mips.bz.d\0"
  "llvm.mips.bz.h\0"
  "llvm.mips.bz.v\0"
  "llvm.mips.bz.w\0"
  "llvm.mips.ceq.b\0"
  "llvm.mips.ceq.d\0"
  "llvm.mips.ceq.h\0"
  "llvm.mips.ceq.w\0"
  "llvm.mips.ceqi.b\0"
  "llvm.mips.ceqi.d\0"
  "llvm.mips.ceqi.h\0"
  "llvm.mips.ceqi.w\0"
  "llvm.mips.cfcmsa\0"
  "llvm.mips.cle.s.b\0"
  "llvm.mips.cle.s.d\0"
  "llvm.mips.cle.s.h\0"
  "llvm.mips.cle.s.w\0"
  "llvm.mips.cle.u.b\0"
  "llvm.mips.cle.u.d\0"
  "llvm.mips.cle.u.h\0"
  "llvm.mips.cle.u.w\0"
  "llvm.mips.clei.s.b\0"
  "llvm.mips.clei.s.d\0"
  "llvm.mips.clei.s.h\0"
  "llvm.mips.clei.s.w\0"
  "llvm.mips.clei.u.b\0"
  "llvm.mips.clei.u.d\0"
  "llvm.mips.clei.u.h\0"
  "llvm.mips.clei.u.w\0"
  "llvm.mips.clt.s.b\0"
  "llvm.mips.clt.s.d\0"
  "llvm.mips.clt.s.h\0"
  "llvm.mips.clt.s.w\0"
  "llvm.mips.clt.u.b\0"
  "llvm.mips.clt.u.d\0"
  "llvm.mips.clt.u.h\0"
  "llvm.mips.clt.u.w\0"
  "llvm.mips.clti.s.b\0"
  "llvm.mips.clti.s.d\0"
  "llvm.mips.clti.s.h\0"
  "llvm.mips.clti.s.w\0"
  "llvm.mips.clti.u.b\0"
  "llvm.mips.clti.u.d\0"
  "llvm.mips.clti.u.h\0"
  "llvm.mips.clti.u.w\0"
  "llvm.mips.cmp.eq.ph\0"
  "llvm.mips.cmp.le.ph\0"
  "llvm.mips.cmp.lt.ph\0"
  "llvm.mips.cmpgdu.eq.qb\0"
  "llvm.mips.cmpgdu.le.qb\0"
  "llvm.mips.cmpgdu.lt.qb\0"
  "llvm.mips.cmpgu.eq.qb\0"
  "llvm.mips.cmpgu.le.qb\0"
  "llvm.mips.cmpgu.lt.qb\0"
  "llvm.mips.cmpu.eq.qb\0"
  "llvm.mips.cmpu.le.qb\0"
  "llvm.mips.cmpu.lt.qb\0"
  "llvm.mips.copy.s.b\0"
  "llvm.mips.copy.s.d\0"
  "llvm.mips.copy.s.h\0"
  "llvm.mips.copy.s.w\0"
  "llvm.mips.copy.u.b\0"
  "llvm.mips.copy.u.d\0"
  "llvm.mips.copy.u.h\0"
  "llvm.mips.copy.u.w\0"
  "llvm.mips.ctcmsa\0"
  "llvm.mips.div.s.b\0"
  "llvm.mips.div.s.d\0"
  "llvm.mips.div.s.h\0"
  "llvm.mips.div.s.w\0"
  "llvm.mips.div.u.b\0"
  "llvm.mips.div.u.d\0"
  "llvm.mips.div.u.h\0"
  "llvm.mips.div.u.w\0"
  "llvm.mips.dlsa\0"
  "llvm.mips.dotp.s.d\0"
  "llvm.mips.dotp.s.h\0"
  "llvm.mips.dotp.s.w\0"
  "llvm.mips.dotp.u.d\0"
  "llvm.mips.dotp.u.h\0"
  "llvm.mips.dotp.u.w\0"
  "llvm.mips.dpa.w.ph\0"
  "llvm.mips.dpadd.s.d\0"
  "llvm.mips.dpadd.s.h\0"
  "llvm.mips.dpadd.s.w\0"
  "llvm.mips.dpadd.u.d\0"
  "llvm.mips.dpadd.u.h\0"
  "llvm.mips.dpadd.u.w\0"
  "llvm.mips.dpaq.s.w.ph\0"
  "llvm.mips.dpaq.sa.l.w\0"
  "llvm.mips.dpaqx.s.w.ph\0"
  "llvm.mips.dpaqx.sa.w.ph\0"
  "llvm.mips.dpau.h.qbl\0"
  "llvm.mips.dpau.h.qbr\0"
  "llvm.mips.dpax.w.ph\0"
  "llvm.mips.dps.w.ph\0"
  "llvm.mips.dpsq.s.w.ph\0"
  "llvm.mips.dpsq.sa.l.w\0"
  "llvm.mips.dpsqx.s.w.ph\0"
  "llvm.mips.dpsqx.sa.w.ph\0"
  "llvm.mips.dpsu.h.qbl\0"
  "llvm.mips.dpsu.h.qbr\0"
  "llvm.mips.dpsub.s.d\0"
  "llvm.mips.dpsub.s.h\0"
  "llvm.mips.dpsub.s.w\0"
  "llvm.mips.dpsub.u.d\0"
  "llvm.mips.dpsub.u.h\0"
  "llvm.mips.dpsub.u.w\0"
  "llvm.mips.dpsx.w.ph\0"
  "llvm.mips.extp\0"
  "llvm.mips.extpdp\0"
  "llvm.mips.extr.r.w\0"
  "llvm.mips.extr.rs.w\0"
  "llvm.mips.extr.s.h\0"
  "llvm.mips.extr.w\0"
  "llvm.mips.fadd.d\0"
  "llvm.mips.fadd.w\0"
  "llvm.mips.fcaf.d\0"
  "llvm.mips.fcaf.w\0"
  "llvm.mips.fceq.d\0"
  "llvm.mips.fceq.w\0"
  "llvm.mips.fclass.d\0"
  "llvm.mips.fclass.w\0"
  "llvm.mips.fcle.d\0"
  "llvm.mips.fcle.w\0"
  "llvm.mips.fclt.d\0"
  "llvm.mips.fclt.w\0"
  "llvm.mips.fcne.d\0"
  "llvm.mips.fcne.w\0"
  "llvm.mips.fcor.d\0"
  "llvm.mips.fcor.w\0"
  "llvm.mips.fcueq.d\0"
  "llvm.mips.fcueq.w\0"
  "llvm.mips.fcule.d\0"
  "llvm.mips.fcule.w\0"
  "llvm.mips.fcult.d\0"
  "llvm.mips.fcult.w\0"
  "llvm.mips.fcun.d\0"
  "llvm.mips.fcun.w\0"
  "llvm.mips.fcune.d\0"
  "llvm.mips.fcune.w\0"
  "llvm.mips.fdiv.d\0"
  "llvm.mips.fdiv.w\0"
  "llvm.mips.fexdo.h\0"
  "llvm.mips.fexdo.w\0"
  "llvm.mips.fexp2.d\0"
  "llvm.mips.fexp2.w\0"
  "llvm.mips.fexupl.d\0"
  "llvm.mips.fexupl.w\0"
  "llvm.mips.fexupr.d\0"
  "llvm.mips.fexupr.w\0"
  "llvm.mips.ffint.s.d\0"
  "llvm.mips.ffint.s.w\0"
  "llvm.mips.ffint.u.d\0"
  "llvm.mips.ffint.u.w\0"
  "llvm.mips.ffql.d\0"
  "llvm.mips.ffql.w\0"
  "llvm.mips.ffqr.d\0"
  "llvm.mips.ffqr.w\0"
  "llvm.mips.fill.b\0"
  "llvm.mips.fill.d\0"
  "llvm.mips.fill.h\0"
  "llvm.mips.fill.w\0"
  "llvm.mips.flog2.d\0"
  "llvm.mips.flog2.w\0"
  "llvm.mips.fmadd.d\0"
  "llvm.mips.fmadd.w\0"
  "llvm.mips.fmax.a.d\0"
  "llvm.mips.fmax.a.w\0"
  "llvm.mips.fmax.d\0"
  "llvm.mips.fmax.w\0"
  "llvm.mips.fmin.a.d\0"
  "llvm.mips.fmin.a.w\0"
  "llvm.mips.fmin.d\0"
  "llvm.mips.fmin.w\0"
  "llvm.mips.fmsub.d\0"
  "llvm.mips.fmsub.w\0"
  "llvm.mips.fmul.d\0"
  "llvm.mips.fmul.w\0"
  "llvm.mips.frcp.d\0"
  "llvm.mips.frcp.w\0"
  "llvm.mips.frint.d\0"
  "llvm.mips.frint.w\0"
  "llvm.mips.frsqrt.d\0"
  "llvm.mips.frsqrt.w\0"
  "llvm.mips.fsaf.d\0"
  "llvm.mips.fsaf.w\0"
  "llvm.mips.fseq.d\0"
  "llvm.mips.fseq.w\0"
  "llvm.mips.fsle.d\0"
  "llvm.mips.fsle.w\0"
  "llvm.mips.fslt.d\0"
  "llvm.mips.fslt.w\0"
  "llvm.mips.fsne.d\0"
  "llvm.mips.fsne.w\0"
  "llvm.mips.fsor.d\0"
  "llvm.mips.fsor.w\0"
  "llvm.mips.fsqrt.d\0"
  "llvm.mips.fsqrt.w\0"
  "llvm.mips.fsub.d\0"
  "llvm.mips.fsub.w\0"
  "llvm.mips.fsueq.d\0"
  "llvm.mips.fsueq.w\0"
  "llvm.mips.fsule.d\0"
  "llvm.mips.fsule.w\0"
  "llvm.mips.fsult.d\0"
  "llvm.mips.fsult.w\0"
  "llvm.mips.fsun.d\0"
  "llvm.mips.fsun.w\0"
  "llvm.mips.fsune.d\0"
  "llvm.mips.fsune.w\0"
  "llvm.mips.ftint.s.d\0"
  "llvm.mips.ftint.s.w\0"
  "llvm.mips.ftint.u.d\0"
  "llvm.mips.ftint.u.w\0"
  "llvm.mips.ftq.h\0"
  "llvm.mips.ftq.w\0"
  "llvm.mips.ftrunc.s.d\0"
  "llvm.mips.ftrunc.s.w\0"
  "llvm.mips.ftrunc.u.d\0"
  "llvm.mips.ftrunc.u.w\0"
  "llvm.mips.hadd.s.d\0"
  "llvm.mips.hadd.s.h\0"
  "llvm.mips.hadd.s.w\0"
  "llvm.mips.hadd.u.d\0"
  "llvm.mips.hadd.u.h\0"
  "llvm.mips.hadd.u.w\0"
  "llvm.mips.hsub.s.d\0"
  "llvm.mips.hsub.s.h\0"
  "llvm.mips.hsub.s.w\0"
  "llvm.mips.hsub.u.d\0"
  "llvm.mips.hsub.u.h\0"
  "llvm.mips.hsub.u.w\0"
  "llvm.mips.ilvev.b\0"
  "llvm.mips.ilvev.d\0"
  "llvm.mips.ilvev.h\0"
  "llvm.mips.ilvev.w\0"
  "llvm.mips.ilvl.b\0"
  "llvm.mips.ilvl.d\0"
  "llvm.mips.ilvl.h\0"
  "llvm.mips.ilvl.w\0"
  "llvm.mips.ilvod.b\0"
  "llvm.mips.ilvod.d\0"
  "llvm.mips.ilvod.h\0"
  "llvm.mips.ilvod.w\0"
  "llvm.mips.ilvr.b\0"
  "llvm.mips.ilvr.d\0"
  "llvm.mips.ilvr.h\0"
  "llvm.mips.ilvr.w\0"
  "llvm.mips.insert.b\0"
  "llvm.mips.insert.d\0"
  "llvm.mips.insert.h\0"
  "llvm.mips.insert.w\0"
  "llvm.mips.insv\0"
  "llvm.mips.insve.b\0"
  "llvm.mips.insve.d\0"
  "llvm.mips.insve.h\0"
  "llvm.mips.insve.w\0"
  "llvm.mips.lbux\0"
  "llvm.mips.ld.b\0"
  "llvm.mips.ld.d\0"
  "llvm.mips.ld.h\0"
  "llvm.mips.ld.w\0"
  "llvm.mips.ldi.b\0"
  "llvm.mips.ldi.d\0"
  "llvm.mips.ldi.h\0"
  "llvm.mips.ldi.w\0"
  "llvm.mips.ldr.d\0"
  "llvm.mips.ldr.w\0"
  "llvm.mips.lhx\0"
  "llvm.mips.lsa\0"
  "llvm.mips.lwx\0"
  "llvm.mips.madd\0"
  "llvm.mips.madd.q.h\0"
  "llvm.mips.madd.q.w\0"
  "llvm.mips.maddr.q.h\0"
  "llvm.mips.maddr.q.w\0"
  "llvm.mips.maddu\0"
  "llvm.mips.maddv.b\0"
  "llvm.mips.maddv.d\0"
  "llvm.mips.maddv.h\0"
  "llvm.mips.maddv.w\0"
  "llvm.mips.maq.s.w.phl\0"
  "llvm.mips.maq.s.w.phr\0"
  "llvm.mips.maq.sa.w.phl\0"
  "llvm.mips.maq.sa.w.phr\0"
  "llvm.mips.max.a.b\0"
  "llvm.mips.max.a.d\0"
  "llvm.mips.max.a.h\0"
  "llvm.mips.max.a.w\0"
  "llvm.mips.max.s.b\0"
  "llvm.mips.max.s.d\0"
  "llvm.mips.max.s.h\0"
  "llvm.mips.max.s.w\0"
  "llvm.mips.max.u.b\0"
  "llvm.mips.max.u.d\0"
  "llvm.mips.max.u.h\0"
  "llvm.mips.max.u.w\0"
  "llvm.mips.maxi.s.b\0"
  "llvm.mips.maxi.s.d\0"
  "llvm.mips.maxi.s.h\0"
  "llvm.mips.maxi.s.w\0"
  "llvm.mips.maxi.u.b\0"
  "llvm.mips.maxi.u.d\0"
  "llvm.mips.maxi.u.h\0"
  "llvm.mips.maxi.u.w\0"
  "llvm.mips.min.a.b\0"
  "llvm.mips.min.a.d\0"
  "llvm.mips.min.a.h\0"
  "llvm.mips.min.a.w\0"
  "llvm.mips.min.s.b\0"
  "llvm.mips.min.s.d\0"
  "llvm.mips.min.s.h\0"
  "llvm.mips.min.s.w\0"
  "llvm.mips.min.u.b\0"
  "llvm.mips.min.u.d\0"
  "llvm.mips.min.u.h\0"
  "llvm.mips.min.u.w\0"
  "llvm.mips.mini.s.b\0"
  "llvm.mips.mini.s.d\0"
  "llvm.mips.mini.s.h\0"
  "llvm.mips.mini.s.w\0"
  "llvm.mips.mini.u.b\0"
  "llvm.mips.mini.u.d\0"
  "llvm.mips.mini.u.h\0"
  "llvm.mips.mini.u.w\0"
  "llvm.mips.mod.s.b\0"
  "llvm.mips.mod.s.d\0"
  "llvm.mips.mod.s.h\0"
  "llvm.mips.mod.s.w\0"
  "llvm.mips.mod.u.b\0"
  "llvm.mips.mod.u.d\0"
  "llvm.mips.mod.u.h\0"
  "llvm.mips.mod.u.w\0"
  "llvm.mips.modsub\0"
  "llvm.mips.move.v\0"
  "llvm.mips.msub\0"
  "llvm.mips.msub.q.h\0"
  "llvm.mips.msub.q.w\0"
  "llvm.mips.msubr.q.h\0"
  "llvm.mips.msubr.q.w\0"
  "llvm.mips.msubu\0"
  "llvm.mips.msubv.b\0"
  "llvm.mips.msubv.d\0"
  "llvm.mips.msubv.h\0"
  "llvm.mips.msubv.w\0"
  "llvm.mips.mthlip\0"
  "llvm.mips.mul.ph\0"
  "llvm.mips.mul.q.h\0"
  "llvm.mips.mul.q.w\0"
  "llvm.mips.mul.s.ph\0"
  "llvm.mips.muleq.s.w.phl\0"
  "llvm.mips.muleq.s.w.phr\0"
  "llvm.mips.muleu.s.ph.qbl\0"
  "llvm.mips.muleu.s.ph.qbr\0"
  "llvm.mips.mulq.rs.ph\0"
  "llvm.mips.mulq.rs.w\0"
  "llvm.mips.mulq.s.ph\0"
  "llvm.mips.mulq.s.w\0"
  "llvm.mips.mulr.q.h\0"
  "llvm.mips.mulr.q.w\0"
  "llvm.mips.mulsa.w.ph\0"
  "llvm.mips.mulsaq.s.w.ph\0"
  "llvm.mips.mult\0"
  "llvm.mips.multu\0"
  "llvm.mips.mulv.b\0"
  "llvm.mips.mulv.d\0"
  "llvm.mips.mulv.h\0"
  "llvm.mips.mulv.w\0"
  "llvm.mips.nloc.b\0"
  "llvm.mips.nloc.d\0"
  "llvm.mips.nloc.h\0"
  "llvm.mips.nloc.w\0"
  "llvm.mips.nlzc.b\0"
  "llvm.mips.nlzc.d\0"
  "llvm.mips.nlzc.h\0"
  "llvm.mips.nlzc.w\0"
  "llvm.mips.nor.v\0"
  "llvm.mips.nori.b\0"
  "llvm.mips.or.v\0"
  "llvm.mips.ori.b\0"
  "llvm.mips.packrl.ph\0"
  "llvm.mips.pckev.b\0"
  "llvm.mips.pckev.d\0"
  "llvm.mips.pckev.h\0"
  "llvm.mips.pckev.w\0"
  "llvm.mips.pckod.b\0"
  "llvm.mips.pckod.d\0"
  "llvm.mips.pckod.h\0"
  "llvm.mips.pckod.w\0"
  "llvm.mips.pcnt.b\0"
  "llvm.mips.pcnt.d\0"
  "llvm.mips.pcnt.h\0"
  "llvm.mips.pcnt.w\0"
  "llvm.mips.pick.ph\0"
  "llvm.mips.pick.qb\0"
  "llvm.mips.preceq.w.phl\0"
  "llvm.mips.preceq.w.phr\0"
  "llvm.mips.precequ.ph.qbl\0"
  "llvm.mips.precequ.ph.qbla\0"
  "llvm.mips.precequ.ph.qbr\0"
  "llvm.mips.precequ.ph.qbra\0"
  "llvm.mips.preceu.ph.qbl\0"
  "llvm.mips.preceu.ph.qbla\0"
  "llvm.mips.preceu.ph.qbr\0"
  "llvm.mips.preceu.ph.qbra\0"
  "llvm.mips.precr.qb.ph\0"
  "llvm.mips.precr.sra.ph.w\0"
  "llvm.mips.precr.sra.r.ph.w\0"
  "llvm.mips.precrq.ph.w\0"
  "llvm.mips.precrq.qb.ph\0"
  "llvm.mips.precrq.rs.ph.w\0"
  "llvm.mips.precrqu.s.qb.ph\0"
  "llvm.mips.prepend\0"
  "llvm.mips.raddu.w.qb\0"
  "llvm.mips.rddsp\0"
  "llvm.mips.repl.ph\0"
  "llvm.mips.repl.qb\0"
  "llvm.mips.sat.s.b\0"
  "llvm.mips.sat.s.d\0"
  "llvm.mips.sat.s.h\0"
  "llvm.mips.sat.s.w\0"
  "llvm.mips.sat.u.b\0"
  "llvm.mips.sat.u.d\0"
  "llvm.mips.sat.u.h\0"
  "llvm.mips.sat.u.w\0"
  "llvm.mips.shf.b\0"
  "llvm.mips.shf.h\0"
  "llvm.mips.shf.w\0"
  "llvm.mips.shilo\0"
  "llvm.mips.shll.ph\0"
  "llvm.mips.shll.qb\0"
  "llvm.mips.shll.s.ph\0"
  "llvm.mips.shll.s.w\0"
  "llvm.mips.shra.ph\0"
  "llvm.mips.shra.qb\0"
  "llvm.mips.shra.r.ph\0"
  "llvm.mips.shra.r.qb\0"
  "llvm.mips.shra.r.w\0"
  "llvm.mips.shrl.ph\0"
  "llvm.mips.shrl.qb\0"
  "llvm.mips.sld.b\0"
  "llvm.mips.sld.d\0"
  "llvm.mips.sld.h\0"
  "llvm.mips.sld.w\0"
  "llvm.mips.sldi.b\0"
  "llvm.mips.sldi.d\0"
  "llvm.mips.sldi.h\0"
  "llvm.mips.sldi.w\0"
  "llvm.mips.sll.b\0"
  "llvm.mips.sll.d\0"
  "llvm.mips.sll.h\0"
  "llvm.mips.sll.w\0"
  "llvm.mips.slli.b\0"
  "llvm.mips.slli.d\0"
  "llvm.mips.slli.h\0"
  "llvm.mips.slli.w\0"
  "llvm.mips.splat.b\0"
  "llvm.mips.splat.d\0"
  "llvm.mips.splat.h\0"
  "llvm.mips.splat.w\0"
  "llvm.mips.splati.b\0"
  "llvm.mips.splati.d\0"
  "llvm.mips.splati.h\0"
  "llvm.mips.splati.w\0"
  "llvm.mips.sra.b\0"
  "llvm.mips.sra.d\0"
  "llvm.mips.sra.h\0"
  "llvm.mips.sra.w\0"
  "llvm.mips.srai.b\0"
  "llvm.mips.srai.d\0"
  "llvm.mips.srai.h\0"
  "llvm.mips.srai.w\0"
  "llvm.mips.srar.b\0"
  "llvm.mips.srar.d\0"
  "llvm.mips.srar.h\0"
  "llvm.mips.srar.w\0"
  "llvm.mips.srari.b\0"
  "llvm.mips.srari.d\0"
  "llvm.mips.srari.h\0"
  "llvm.mips.srari.w\0"
  "llvm.mips.srl.b\0"
  "llvm.mips.srl.d\0"
  "llvm.mips.srl.h\0"
  "llvm.mips.srl.w\0"
  "llvm.mips.srli.b\0"
  "llvm.mips.srli.d\0"
  "llvm.mips.srli.h\0"
  "llvm.mips.srli.w\0"
  "llvm.mips.srlr.b\0"
  "llvm.mips.srlr.d\0"
  "llvm.mips.srlr.h\0"
  "llvm.mips.srlr.w\0"
  "llvm.mips.srlri.b\0"
  "llvm.mips.srlri.d\0"
  "llvm.mips.srlri.h\0"
  "llvm.mips.srlri.w\0"
  "llvm.mips.st.b\0"
  "llvm.mips.st.d\0"
  "llvm.mips.st.h\0"
  "llvm.mips.st.w\0"
  "llvm.mips.str.d\0"
  "llvm.mips.str.w\0"
  "llvm.mips.subq.ph\0"
  "llvm.mips.subq.s.ph\0"
  "llvm.mips.subq.s.w\0"
  "llvm.mips.subqh.ph\0"
  "llvm.mips.subqh.r.ph\0"
  "llvm.mips.subqh.r.w\0"
  "llvm.mips.subqh.w\0"
  "llvm.mips.subs.s.b\0"
  "llvm.mips.subs.s.d\0"
  "llvm.mips.subs.s.h\0"
  "llvm.mips.subs.s.w\0"
  "llvm.mips.subs.u.b\0"
  "llvm.mips.subs.u.d\0"
  "llvm.mips.subs.u.h\0"
  "llvm.mips.subs.u.w\0"
  "llvm.mips.subsus.u.b\0"
  "llvm.mips.subsus.u.d\0"
  "llvm.mips.subsus.u.h\0"
  "llvm.mips.subsus.u.w\0"
  "llvm.mips.subsuu.s.b\0"
  "llvm.mips.subsuu.s.d\0"
  "llvm.mips.subsuu.s.h\0"
  "llvm.mips.subsuu.s.w\0"
  "llvm.mips.subu.ph\0"
  "llvm.mips.subu.qb\0"
  "llvm.mips.subu.s.ph\0"
  "llvm.mips.subu.s.qb\0"
  "llvm.mips.subuh.qb\0"
  "llvm.mips.subuh.r.qb\0"
  "llvm.mips.subv.b\0"
  "llvm.mips.subv.d\0"
  "llvm.mips.subv.h\0"
  "llvm.mips.subv.w\0"
  "llvm.mips.subvi.b\0"
  "llvm.mips.subvi.d\0"
  "llvm.mips.subvi.h\0"
  "llvm.mips.subvi.w\0"
  "llvm.mips.vshf.b\0"
  "llvm.mips.vshf.d\0"
  "llvm.mips.vshf.h\0"
  "llvm.mips.vshf.w\0"
  "llvm.mips.wrdsp\0"
  "llvm.mips.xor.v\0"
  "llvm.mips.xori.b\0"
  "llvm.nvvm.activemask\0"
  "llvm.nvvm.add.rm.d\0"
  "llvm.nvvm.add.rm.f\0"
  "llvm.nvvm.add.rm.ftz.f\0"
  "llvm.nvvm.add.rm.ftz.sat.f\0"
  "llvm.nvvm.add.rm.sat.f\0"
  "llvm.nvvm.add.rn.d\0"
  "llvm.nvvm.add.rn.f\0"
  "llvm.nvvm.add.rn.ftz.f\0"
  "llvm.nvvm.add.rn.ftz.sat.f\0"
  "llvm.nvvm.add.rn.sat.f\0"
  "llvm.nvvm.add.rp.d\0"
  "llvm.nvvm.add.rp.f\0"
  "llvm.nvvm.add.rp.ftz.f\0"
  "llvm.nvvm.add.rp.ftz.sat.f\0"
  "llvm.nvvm.add.rp.sat.f\0"
  "llvm.nvvm.add.rz.d\0"
  "llvm.nvvm.add.rz.f\0"
  "llvm.nvvm.add.rz.ftz.f\0"
  "llvm.nvvm.add.rz.ftz.sat.f\0"
  "llvm.nvvm.add.rz.sat.f\0"
  "llvm.nvvm.applypriority.L2.evict.normal\0"
  "llvm.nvvm.applypriority.global.L2.evict.normal\0"
  "llvm.nvvm.atomic.add.gen.f.cta\0"
  "llvm.nvvm.atomic.add.gen.f.sys\0"
  "llvm.nvvm.atomic.add.gen.i.cta\0"
  "llvm.nvvm.atomic.add.gen.i.sys\0"
  "llvm.nvvm.atomic.and.gen.i.cta\0"
  "llvm.nvvm.atomic.and.gen.i.sys\0"
  "llvm.nvvm.atomic.cas.gen.i.cta\0"
  "llvm.nvvm.atomic.cas.gen.i.sys\0"
  "llvm.nvvm.atomic.dec.gen.i.cta\0"
  "llvm.nvvm.atomic.dec.gen.i.sys\0"
  "llvm.nvvm.atomic.exch.gen.i.cta\0"
  "llvm.nvvm.atomic.exch.gen.i.sys\0"
  "llvm.nvvm.atomic.inc.gen.i.cta\0"
  "llvm.nvvm.atomic.inc.gen.i.sys\0"
  "llvm.nvvm.atomic.max.gen.i.cta\0"
  "llvm.nvvm.atomic.max.gen.i.sys\0"
  "llvm.nvvm.atomic.min.gen.i.cta\0"
  "llvm.nvvm.atomic.min.gen.i.sys\0"
  "llvm.nvvm.atomic.or.gen.i.cta\0"
  "llvm.nvvm.atomic.or.gen.i.sys\0"
  "llvm.nvvm.atomic.xor.gen.i.cta\0"
  "llvm.nvvm.atomic.xor.gen.i.sys\0"
  "llvm.nvvm.bar.warp.sync\0"
  "llvm.nvvm.barrier.cluster.arrive\0"
  "llvm.nvvm.barrier.cluster.arrive.aligned\0"
  "llvm.nvvm.barrier.cluster.arrive.relaxed\0"
  "llvm.nvvm.barrier.cluster.arrive.relaxed.aligned\0"
  "llvm.nvvm.barrier.cluster.wait\0"
  "llvm.nvvm.barrier.cluster.wait.aligned\0"
  "llvm.nvvm.barrier.cta.arrive.aligned.count\0"
  "llvm.nvvm.barrier.cta.arrive.count\0"
  "llvm.nvvm.barrier.cta.red.and.aligned.all\0"
  "llvm.nvvm.barrier.cta.red.and.aligned.count\0"
  "llvm.nvvm.barrier.cta.red.and.all\0"
  "llvm.nvvm.barrier.cta.red.and.count\0"
  "llvm.nvvm.barrier.cta.red.or.aligned.all\0"
  "llvm.nvvm.barrier.cta.red.or.aligned.count\0"
  "llvm.nvvm.barrier.cta.red.or.all\0"
  "llvm.nvvm.barrier.cta.red.or.count\0"
  "llvm.nvvm.barrier.cta.red.popc.aligned.all\0"
  "llvm.nvvm.barrier.cta.red.popc.aligned.count\0"
  "llvm.nvvm.barrier.cta.red.popc.all\0"
  "llvm.nvvm.barrier.cta.red.popc.count\0"
  "llvm.nvvm.barrier.cta.sync.aligned.all\0"
  "llvm.nvvm.barrier.cta.sync.aligned.count\0"
  "llvm.nvvm.barrier.cta.sync.all\0"
  "llvm.nvvm.barrier.cta.sync.count\0"
  "llvm.nvvm.bf16x2.to.ue8m0x2.rp\0"
  "llvm.nvvm.bf16x2.to.ue8m0x2.rp.satfinite\0"
  "llvm.nvvm.bf16x2.to.ue8m0x2.rz\0"
  "llvm.nvvm.bf16x2.to.ue8m0x2.rz.satfinite\0"
  "llvm.nvvm.bf2h.rn\0"
  "llvm.nvvm.bf2h.rn.ftz\0"
  "llvm.nvvm.bmsk.clamp\0"
  "llvm.nvvm.bmsk.wrap\0"
  "llvm.nvvm.ceil.d\0"
  "llvm.nvvm.ceil.f\0"
  "llvm.nvvm.ceil.ftz.f\0"
  "llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.x\0"
  "llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.y\0"
  "llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.z\0"
  "llvm.nvvm.clusterlaunchcontrol.query_cancel.is_canceled\0"
  "llvm.nvvm.clusterlaunchcontrol.try_cancel.async.multicast.shared\0"
  "llvm.nvvm.clusterlaunchcontrol.try_cancel.async.shared\0"
  "llvm.nvvm.compiler.error\0"
  "llvm.nvvm.compiler.warn\0"
  "llvm.nvvm.cos.approx.f\0"
  "llvm.nvvm.cos.approx.ftz.f\0"
  "llvm.nvvm.cp.async.bulk.commit.group\0"
  "llvm.nvvm.cp.async.bulk.global.to.shared.cluster\0"
  "llvm.nvvm.cp.async.bulk.global.to.shared.cta\0"
  "llvm.nvvm.cp.async.bulk.prefetch.L2\0"
  "llvm.nvvm.cp.async.bulk.shared.cta.to.cluster\0"
  "llvm.nvvm.cp.async.bulk.shared.cta.to.global\0"
  "llvm.nvvm.cp.async.bulk.shared.cta.to.global.bytemask\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.gather4.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.g2s.tile.gather4.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.gather4.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.tile.1d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.tile.2d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.tile.3d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.tile.4d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.tile.5d\0"
  "llvm.nvvm.cp.async.bulk.tensor.s2g.tile.scatter4.2d\0"
  "llvm.nvvm.cp.async.bulk.wait.group\0"
  "llvm.nvvm.cp.async.bulk.wait.group.read\0"
  "llvm.nvvm.cp.async.ca.shared.global.16\0"
  "llvm.nvvm.cp.async.ca.shared.global.16.s\0"
  "llvm.nvvm.cp.async.ca.shared.global.4\0"
  "llvm.nvvm.cp.async.ca.shared.global.4.s\0"
  "llvm.nvvm.cp.async.ca.shared.global.8\0"
  "llvm.nvvm.cp.async.ca.shared.global.8.s\0"
  "llvm.nvvm.cp.async.cg.shared.global.16\0"
  "llvm.nvvm.cp.async.cg.shared.global.16.s\0"
  "llvm.nvvm.cp.async.commit.group\0"
  "llvm.nvvm.cp.async.mbarrier.arrive\0"
  "llvm.nvvm.cp.async.mbarrier.arrive.noinc\0"
  "llvm.nvvm.cp.async.mbarrier.arrive.noinc.shared\0"
  "llvm.nvvm.cp.async.mbarrier.arrive.shared\0"
  "llvm.nvvm.cp.async.wait.all\0"
  "llvm.nvvm.cp.async.wait.group\0"
  "llvm.nvvm.d2f.rm\0"
  "llvm.nvvm.d2f.rm.ftz\0"
  "llvm.nvvm.d2f.rn\0"
  "llvm.nvvm.d2f.rn.ftz\0"
  "llvm.nvvm.d2f.rp\0"
  "llvm.nvvm.d2f.rp.ftz\0"
  "llvm.nvvm.d2f.rz\0"
  "llvm.nvvm.d2f.rz.ftz\0"
  "llvm.nvvm.d2i.hi\0"
  "llvm.nvvm.d2i.lo\0"
  "llvm.nvvm.d2i.rm\0"
  "llvm.nvvm.d2i.rn\0"
  "llvm.nvvm.d2i.rp\0"
  "llvm.nvvm.d2i.rz\0"
  "llvm.nvvm.d2ll.rm\0"
  "llvm.nvvm.d2ll.rn\0"
  "llvm.nvvm.d2ll.rp\0"
  "llvm.nvvm.d2ll.rz\0"
  "llvm.nvvm.d2ui.rm\0"
  "llvm.nvvm.d2ui.rn\0"
  "llvm.nvvm.d2ui.rp\0"
  "llvm.nvvm.d2ui.rz\0"
  "llvm.nvvm.d2ull.rm\0"
  "llvm.nvvm.d2ull.rn\0"
  "llvm.nvvm.d2ull.rp\0"
  "llvm.nvvm.d2ull.rz\0"
  "llvm.nvvm.discard.L2\0"
  "llvm.nvvm.discard.global.L2\0"
  "llvm.nvvm.div.approx.f\0"
  "llvm.nvvm.div.approx.ftz.f\0"
  "llvm.nvvm.div.full\0"
  "llvm.nvvm.div.full.ftz\0"
  "llvm.nvvm.div.rm.d\0"
  "llvm.nvvm.div.rm.f\0"
  "llvm.nvvm.div.rm.ftz.f\0"
  "llvm.nvvm.div.rn.d\0"
  "llvm.nvvm.div.rn.f\0"
  "llvm.nvvm.div.rn.ftz.f\0"
  "llvm.nvvm.div.rp.d\0"
  "llvm.nvvm.div.rp.f\0"
  "llvm.nvvm.div.rp.ftz.f\0"
  "llvm.nvvm.div.rz.d\0"
  "llvm.nvvm.div.rz.f\0"
  "llvm.nvvm.div.rz.ftz.f\0"
  "llvm.nvvm.e2m1x2.to.f16x2.rn\0"
  "llvm.nvvm.e2m1x2.to.f16x2.rn.relu\0"
  "llvm.nvvm.e2m3x2.to.f16x2.rn\0"
  "llvm.nvvm.e2m3x2.to.f16x2.rn.relu\0"
  "llvm.nvvm.e3m2x2.to.f16x2.rn\0"
  "llvm.nvvm.e3m2x2.to.f16x2.rn.relu\0"
  "llvm.nvvm.e4m3x2.to.f16x2.rn\0"
  "llvm.nvvm.e4m3x2.to.f16x2.rn.relu\0"
  "llvm.nvvm.e5m2x2.to.f16x2.rn\0"
  "llvm.nvvm.e5m2x2.to.f16x2.rn.relu\0"
  "llvm.nvvm.elect.sync\0"
  "llvm.nvvm.ex2.approx\0"
  "llvm.nvvm.ex2.approx.ftz\0"
  "llvm.nvvm.exit\0"
  "llvm.nvvm.f16x2.to.e4m3x2.rn\0"
  "llvm.nvvm.f16x2.to.e4m3x2.rn.relu\0"
  "llvm.nvvm.f16x2.to.e5m2x2.rn\0"
  "llvm.nvvm.f16x2.to.e5m2x2.rn.relu\0"
  "llvm.nvvm.f2bf16.rn\0"
  "llvm.nvvm.f2bf16.rn.relu\0"
  "llvm.nvvm.f2bf16.rn.relu.satfinite\0"
  "llvm.nvvm.f2bf16.rn.satfinite\0"
  "llvm.nvvm.f2bf16.rz\0"
  "llvm.nvvm.f2bf16.rz.relu\0"
  "llvm.nvvm.f2bf16.rz.relu.satfinite\0"
  "llvm.nvvm.f2bf16.rz.satfinite\0"
  "llvm.nvvm.f2f16.rn\0"
  "llvm.nvvm.f2f16.rn.relu\0"
  "llvm.nvvm.f2f16.rn.relu.satfinite\0"
  "llvm.nvvm.f2f16.rn.satfinite\0"
  "llvm.nvvm.f2f16.rz\0"
  "llvm.nvvm.f2f16.rz.relu\0"
  "llvm.nvvm.f2f16.rz.relu.satfinite\0"
  "llvm.nvvm.f2f16.rz.satfinite\0"
  "llvm.nvvm.f2h.rn\0"
  "llvm.nvvm.f2h.rn.ftz\0"
  "llvm.nvvm.f2i.rm\0"
  "llvm.nvvm.f2i.rm.ftz\0"
  "llvm.nvvm.f2i.rn\0"
  "llvm.nvvm.f2i.rn.ftz\0"
  "llvm.nvvm.f2i.rp\0"
  "llvm.nvvm.f2i.rp.ftz\0"
  "llvm.nvvm.f2i.rz\0"
  "llvm.nvvm.f2i.rz.ftz\0"
  "llvm.nvvm.f2ll.rm\0"
  "llvm.nvvm.f2ll.rm.ftz\0"
  "llvm.nvvm.f2ll.rn\0"
  "llvm.nvvm.f2ll.rn.ftz\0"
  "llvm.nvvm.f2ll.rp\0"
  "llvm.nvvm.f2ll.rp.ftz\0"
  "llvm.nvvm.f2ll.rz\0"
  "llvm.nvvm.f2ll.rz.ftz\0"
  "llvm.nvvm.f2tf32.rn\0"
  "llvm.nvvm.f2tf32.rn.relu\0"
  "llvm.nvvm.f2tf32.rn.relu.satfinite\0"
  "llvm.nvvm.f2tf32.rn.satfinite\0"
  "llvm.nvvm.f2tf32.rna\0"
  "llvm.nvvm.f2tf32.rna.satfinite\0"
  "llvm.nvvm.f2tf32.rz\0"
  "llvm.nvvm.f2tf32.rz.relu\0"
  "llvm.nvvm.f2tf32.rz.relu.satfinite\0"
  "llvm.nvvm.f2tf32.rz.satfinite\0"
  "llvm.nvvm.f2ui.rm\0"
  "llvm.nvvm.f2ui.rm.ftz\0"
  "llvm.nvvm.f2ui.rn\0"
  "llvm.nvvm.f2ui.rn.ftz\0"
  "llvm.nvvm.f2ui.rp\0"
  "llvm.nvvm.f2ui.rp.ftz\0"
  "llvm.nvvm.f2ui.rz\0"
  "llvm.nvvm.f2ui.rz.ftz\0"
  "llvm.nvvm.f2ull.rm\0"
  "llvm.nvvm.f2ull.rm.ftz\0"
  "llvm.nvvm.f2ull.rn\0"
  "llvm.nvvm.f2ull.rn.ftz\0"
  "llvm.nvvm.f2ull.rp\0"
  "llvm.nvvm.f2ull.rp.ftz\0"
  "llvm.nvvm.f2ull.rz\0"
  "llvm.nvvm.f2ull.rz.ftz\0"
  "llvm.nvvm.f32x4.to.e2m1x4.rs.relu.satfinite\0"
  "llvm.nvvm.f32x4.to.e2m1x4.rs.satfinite\0"
  "llvm.nvvm.f32x4.to.e2m3x4.rs.relu.satfinite\0"
  "llvm.nvvm.f32x4.to.e2m3x4.rs.satfinite\0"
  "llvm.nvvm.f32x4.to.e3m2x4.rs.relu.satfinite\0"
  "llvm.nvvm.f32x4.to.e3m2x4.rs.satfinite\0"
  "llvm.nvvm.f32x4.to.e4m3x4.rs.relu.satfinite\0"
  "llvm.nvvm.f32x4.to.e4m3x4.rs.satfinite\0"
  "llvm.nvvm.f32x4.to.e5m2x4.rs.relu.satfinite\0"
  "llvm.nvvm.f32x4.to.e5m2x4.rs.satfinite\0"
  "llvm.nvvm.fabs\0"
  "llvm.nvvm.fabs.ftz\0"
  "llvm.nvvm.fence.acquire.sync_restrict.space.cluster.scope.cluster\0"
  "llvm.nvvm.fence.mbarrier_init.release.cluster\0"
  "llvm.nvvm.fence.proxy.alias\0"
  "llvm.nvvm.fence.proxy.async\0"
  "llvm.nvvm.fence.proxy.async.global\0"
  "llvm.nvvm.fence.proxy.async.shared_cluster\0"
  "llvm.nvvm.fence.proxy.async.shared_cta\0"
  "llvm.nvvm.fence.proxy.async_generic.acquire.sync_restrict.space.cluster.scope.cluster\0"
  "llvm.nvvm.fence.proxy.async_generic.release.sync_restrict.space.cta.scope.cluster\0"
  "llvm.nvvm.fence.proxy.tensormap_generic.acquire.cluster\0"
  "llvm.nvvm.fence.proxy.tensormap_generic.acquire.cta\0"
  "llvm.nvvm.fence.proxy.tensormap_generic.acquire.gpu\0"
  "llvm.nvvm.fence.proxy.tensormap_generic.acquire.sys\0"
  "llvm.nvvm.fence.proxy.tensormap_generic.release.cluster\0"
  "llvm.nvvm.fence.proxy.tensormap_generic.release.cta\0"
  "llvm.nvvm.fence.proxy.tensormap_generic.release.gpu\0"
  "llvm.nvvm.fence.proxy.tensormap_generic.release.sys\0"
  "llvm.nvvm.fence.release.sync_restrict.space.cta.scope.cluster\0"
  "llvm.nvvm.fence.sc.cluster\0"
  "llvm.nvvm.ff.to.e2m1x2.rn.relu.satfinite\0"
  "llvm.nvvm.ff.to.e2m1x2.rn.satfinite\0"
  "llvm.nvvm.ff.to.e2m3x2.rn.relu.satfinite\0"
  "llvm.nvvm.ff.to.e2m3x2.rn.satfinite\0"
  "llvm.nvvm.ff.to.e3m2x2.rn.relu.satfinite\0"
  "llvm.nvvm.ff.to.e3m2x2.rn.satfinite\0"
  "llvm.nvvm.ff.to.e4m3x2.rn\0"
  "llvm.nvvm.ff.to.e4m3x2.rn.relu\0"
  "llvm.nvvm.ff.to.e5m2x2.rn\0"
  "llvm.nvvm.ff.to.e5m2x2.rn.relu\0"
  "llvm.nvvm.ff.to.ue8m0x2.rp\0"
  "llvm.nvvm.ff.to.ue8m0x2.rp.satfinite\0"
  "llvm.nvvm.ff.to.ue8m0x2.rz\0"
  "llvm.nvvm.ff.to.ue8m0x2.rz.satfinite\0"
  "llvm.nvvm.ff2bf16x2.rn\0"
  "llvm.nvvm.ff2bf16x2.rn.relu\0"
  "llvm.nvvm.ff2bf16x2.rn.relu.satfinite\0"
  "llvm.nvvm.ff2bf16x2.rn.satfinite\0"
  "llvm.nvvm.ff2bf16x2.rs\0"
  "llvm.nvvm.ff2bf16x2.rs.relu\0"
  "llvm.nvvm.ff2bf16x2.rs.relu.satfinite\0"
  "llvm.nvvm.ff2bf16x2.rs.satfinite\0"
  "llvm.nvvm.ff2bf16x2.rz\0"
  "llvm.nvvm.ff2bf16x2.rz.relu\0"
  "llvm.nvvm.ff2bf16x2.rz.relu.satfinite\0"
  "llvm.nvvm.ff2bf16x2.rz.satfinite\0"
  "llvm.nvvm.ff2f16x2.rn\0"
  "llvm.nvvm.ff2f16x2.rn.relu\0"
  "llvm.nvvm.ff2f16x2.rn.relu.satfinite\0"
  "llvm.nvvm.ff2f16x2.rn.satfinite\0"
  "llvm.nvvm.ff2f16x2.rs\0"
  "llvm.nvvm.ff2f16x2.rs.relu\0"
  "llvm.nvvm.ff2f16x2.rs.relu.satfinite\0"
  "llvm.nvvm.ff2f16x2.rs.satfinite\0"
  "llvm.nvvm.ff2f16x2.rz\0"
  "llvm.nvvm.ff2f16x2.rz.relu\0"
  "llvm.nvvm.ff2f16x2.rz.relu.satfinite\0"
  "llvm.nvvm.ff2f16x2.rz.satfinite\0"
  "llvm.nvvm.flo.s\0"
  "llvm.nvvm.flo.u\0"
  "llvm.nvvm.floor.d\0"
  "llvm.nvvm.floor.f\0"
  "llvm.nvvm.floor.ftz.f\0"
  "llvm.nvvm.fma.rm.d\0"
  "llvm.nvvm.fma.rm.f\0"
  "llvm.nvvm.fma.rm.ftz.f\0"
  "llvm.nvvm.fma.rm.ftz.sat.f\0"
  "llvm.nvvm.fma.rm.sat.f\0"
  "llvm.nvvm.fma.rn.bf16\0"
  "llvm.nvvm.fma.rn.bf16x2\0"
  "llvm.nvvm.fma.rn.d\0"
  "llvm.nvvm.fma.rn.f\0"
  "llvm.nvvm.fma.rn.f16\0"
  "llvm.nvvm.fma.rn.f16x2\0"
  "llvm.nvvm.fma.rn.ftz.bf16\0"
  "llvm.nvvm.fma.rn.ftz.bf16x2\0"
  "llvm.nvvm.fma.rn.ftz.f\0"
  "llvm.nvvm.fma.rn.ftz.f16\0"
  "llvm.nvvm.fma.rn.ftz.f16x2\0"
  "llvm.nvvm.fma.rn.ftz.relu.bf16\0"
  "llvm.nvvm.fma.rn.ftz.relu.bf16x2\0"
  "llvm.nvvm.fma.rn.ftz.relu.f16\0"
  "llvm.nvvm.fma.rn.ftz.relu.f16x2\0"
  "llvm.nvvm.fma.rn.ftz.sat.bf16\0"
  "llvm.nvvm.fma.rn.ftz.sat.bf16x2\0"
  "llvm.nvvm.fma.rn.ftz.sat.f\0"
  "llvm.nvvm.fma.rn.ftz.sat.f16\0"
  "llvm.nvvm.fma.rn.ftz.sat.f16x2\0"
  "llvm.nvvm.fma.rn.relu.bf16\0"
  "llvm.nvvm.fma.rn.relu.bf16x2\0"
  "llvm.nvvm.fma.rn.relu.f16\0"
  "llvm.nvvm.fma.rn.relu.f16x2\0"
  "llvm.nvvm.fma.rn.sat.bf16\0"
  "llvm.nvvm.fma.rn.sat.bf16x2\0"
  "llvm.nvvm.fma.rn.sat.f\0"
  "llvm.nvvm.fma.rn.sat.f16\0"
  "llvm.nvvm.fma.rn.sat.f16x2\0"
  "llvm.nvvm.fma.rp.d\0"
  "llvm.nvvm.fma.rp.f\0"
  "llvm.nvvm.fma.rp.ftz.f\0"
  "llvm.nvvm.fma.rp.ftz.sat.f\0"
  "llvm.nvvm.fma.rp.sat.f\0"
  "llvm.nvvm.fma.rz.d\0"
  "llvm.nvvm.fma.rz.f\0"
  "llvm.nvvm.fma.rz.ftz.f\0"
  "llvm.nvvm.fma.rz.ftz.sat.f\0"
  "llvm.nvvm.fma.rz.sat.f\0"
  "llvm.nvvm.fmax.bf16\0"
  "llvm.nvvm.fmax.bf16x2\0"
  "llvm.nvvm.fmax.d\0"
  "llvm.nvvm.fmax.f\0"
  "llvm.nvvm.fmax.f16\0"
  "llvm.nvvm.fmax.f16x2\0"
  "llvm.nvvm.fmax.ftz.bf16\0"
  "llvm.nvvm.fmax.ftz.bf16x2\0"
  "llvm.nvvm.fmax.ftz.f\0"
  "llvm.nvvm.fmax.ftz.f16\0"
  "llvm.nvvm.fmax.ftz.f16x2\0"
  "llvm.nvvm.fmax.ftz.nan.bf16\0"
  "llvm.nvvm.fmax.ftz.nan.bf16x2\0"
  "llvm.nvvm.fmax.ftz.nan.f\0"
  "llvm.nvvm.fmax.ftz.nan.f16\0"
  "llvm.nvvm.fmax.ftz.nan.f16x2\0"
  "llvm.nvvm.fmax.ftz.nan.xorsign.abs.bf16\0"
  "llvm.nvvm.fmax.ftz.nan.xorsign.abs.bf16x2\0"
  "llvm.nvvm.fmax.ftz.nan.xorsign.abs.f\0"
  "llvm.nvvm.fmax.ftz.nan.xorsign.abs.f16\0"
  "llvm.nvvm.fmax.ftz.nan.xorsign.abs.f16x2\0"
  "llvm.nvvm.fmax.ftz.xorsign.abs.bf16\0"
  "llvm.nvvm.fmax.ftz.xorsign.abs.bf16x2\0"
  "llvm.nvvm.fmax.ftz.xorsign.abs.f\0"
  "llvm.nvvm.fmax.ftz.xorsign.abs.f16\0"
  "llvm.nvvm.fmax.ftz.xorsign.abs.f16x2\0"
  "llvm.nvvm.fmax.nan.bf16\0"
  "llvm.nvvm.fmax.nan.bf16x2\0"
  "llvm.nvvm.fmax.nan.f\0"
  "llvm.nvvm.fmax.nan.f16\0"
  "llvm.nvvm.fmax.nan.f16x2\0"
  "llvm.nvvm.fmax.nan.xorsign.abs.bf16\0"
  "llvm.nvvm.fmax.nan.xorsign.abs.bf16x2\0"
  "llvm.nvvm.fmax.nan.xorsign.abs.f\0"
  "llvm.nvvm.fmax.nan.xorsign.abs.f16\0"
  "llvm.nvvm.fmax.nan.xorsign.abs.f16x2\0"
  "llvm.nvvm.fmax.xorsign.abs.bf16\0"
  "llvm.nvvm.fmax.xorsign.abs.bf16x2\0"
  "llvm.nvvm.fmax.xorsign.abs.f\0"
  "llvm.nvvm.fmax.xorsign.abs.f16\0"
  "llvm.nvvm.fmax.xorsign.abs.f16x2\0"
  "llvm.nvvm.fmin.bf16\0"
  "llvm.nvvm.fmin.bf16x2\0"
  "llvm.nvvm.fmin.d\0"
  "llvm.nvvm.fmin.f\0"
  "llvm.nvvm.fmin.f16\0"
  "llvm.nvvm.fmin.f16x2\0"
  "llvm.nvvm.fmin.ftz.bf16\0"
  "llvm.nvvm.fmin.ftz.bf16x2\0"
  "llvm.nvvm.fmin.ftz.f\0"
  "llvm.nvvm.fmin.ftz.f16\0"
  "llvm.nvvm.fmin.ftz.f16x2\0"
  "llvm.nvvm.fmin.ftz.nan.bf16\0"
  "llvm.nvvm.fmin.ftz.nan.bf16x2\0"
  "llvm.nvvm.fmin.ftz.nan.f\0"
  "llvm.nvvm.fmin.ftz.nan.f16\0"
  "llvm.nvvm.fmin.ftz.nan.f16x2\0"
  "llvm.nvvm.fmin.ftz.nan.xorsign.abs.bf16\0"
  "llvm.nvvm.fmin.ftz.nan.xorsign.abs.bf16x2\0"
  "llvm.nvvm.fmin.ftz.nan.xorsign.abs.f\0"
  "llvm.nvvm.fmin.ftz.nan.xorsign.abs.f16\0"
  "llvm.nvvm.fmin.ftz.nan.xorsign.abs.f16x2\0"
  "llvm.nvvm.fmin.ftz.xorsign.abs.bf16\0"
  "llvm.nvvm.fmin.ftz.xorsign.abs.bf16x2\0"
  "llvm.nvvm.fmin.ftz.xorsign.abs.f\0"
  "llvm.nvvm.fmin.ftz.xorsign.abs.f16\0"
  "llvm.nvvm.fmin.ftz.xorsign.abs.f16x2\0"
  "llvm.nvvm.fmin.nan.bf16\0"
  "llvm.nvvm.fmin.nan.bf16x2\0"
  "llvm.nvvm.fmin.nan.f\0"
  "llvm.nvvm.fmin.nan.f16\0"
  "llvm.nvvm.fmin.nan.f16x2\0"
  "llvm.nvvm.fmin.nan.xorsign.abs.bf16\0"
  "llvm.nvvm.fmin.nan.xorsign.abs.bf16x2\0"
  "llvm.nvvm.fmin.nan.xorsign.abs.f\0"
  "llvm.nvvm.fmin.nan.xorsign.abs.f16\0"
  "llvm.nvvm.fmin.nan.xorsign.abs.f16x2\0"
  "llvm.nvvm.fmin.xorsign.abs.bf16\0"
  "llvm.nvvm.fmin.xorsign.abs.bf16x2\0"
  "llvm.nvvm.fmin.xorsign.abs.f\0"
  "llvm.nvvm.fmin.xorsign.abs.f16\0"
  "llvm.nvvm.fmin.xorsign.abs.f16x2\0"
  "llvm.nvvm.fns\0"
  "llvm.nvvm.fshl.clamp\0"
  "llvm.nvvm.fshr.clamp\0"
  "llvm.nvvm.getctarank\0"
  "llvm.nvvm.getctarank.shared.cluster\0"
  "llvm.nvvm.griddepcontrol.launch.dependents\0"
  "llvm.nvvm.griddepcontrol.wait\0"
  "llvm.nvvm.i2d.rm\0"
  "llvm.nvvm.i2d.rn\0"
  "llvm.nvvm.i2d.rp\0"
  "llvm.nvvm.i2d.rz\0"
  "llvm.nvvm.i2f.rm\0"
  "llvm.nvvm.i2f.rn\0"
  "llvm.nvvm.i2f.rp\0"
  "llvm.nvvm.i2f.rz\0"
  "llvm.nvvm.idp2a.s.s\0"
  "llvm.nvvm.idp2a.s.u\0"
  "llvm.nvvm.idp2a.u.s\0"
  "llvm.nvvm.idp2a.u.u\0"
  "llvm.nvvm.idp4a.s.s\0"
  "llvm.nvvm.idp4a.s.u\0"
  "llvm.nvvm.idp4a.u.s\0"
  "llvm.nvvm.idp4a.u.u\0"
  "llvm.nvvm.internal.addrspace.wrap\0"
  "llvm.nvvm.is_explicit_cluster\0"
  "llvm.nvvm.isspacep.const\0"
  "llvm.nvvm.isspacep.global\0"
  "llvm.nvvm.isspacep.local\0"
  "llvm.nvvm.isspacep.shared\0"
  "llvm.nvvm.isspacep.shared.cluster\0"
  "llvm.nvvm.istypep.sampler\0"
  "llvm.nvvm.istypep.surface\0"
  "llvm.nvvm.istypep.texture\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8x16.b4x16_p64\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8x16.b6x16_p32\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8x16.b4x16_p64\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8x16.b6x16_p32\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n16.x1.b8x16.b4x16_p64\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n16.x1.b8x16.b6x16_p32\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n16.x2.b8x16.b4x16_p64\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n16.x2.b8x16.b6x16_p32\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n16.x4.b8x16.b4x16_p64\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n16.x4.b8x16.b6x16_p32\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n8.x1.b16\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n8.x1.trans.b16\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n8.x2.b16\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n8.x2.trans.b16\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n8.x4.b16\0"
  "llvm.nvvm.ldmatrix.sync.aligned.m8n8.x4.trans.b16\0"
  "llvm.nvvm.ldu.global.f\0"
  "llvm.nvvm.ldu.global.i\0"
  "llvm.nvvm.ldu.global.p\0"
  "llvm.nvvm.lg2.approx.d\0"
  "llvm.nvvm.lg2.approx.f\0"
  "llvm.nvvm.lg2.approx.ftz.f\0"
  "llvm.nvvm.ll2d.rm\0"
  "llvm.nvvm.ll2d.rn\0"
  "llvm.nvvm.ll2d.rp\0"
  "llvm.nvvm.ll2d.rz\0"
  "llvm.nvvm.ll2f.rm\0"
  "llvm.nvvm.ll2f.rn\0"
  "llvm.nvvm.ll2f.rp\0"
  "llvm.nvvm.ll2f.rz\0"
  "llvm.nvvm.lohi.i2d\0"
  "llvm.nvvm.mapa\0"
  "llvm.nvvm.mapa.shared.cluster\0"
  "llvm.nvvm.match.all.sync.i32p\0"
  "llvm.nvvm.match.all.sync.i64p\0"
  "llvm.nvvm.match.any.sync.i32\0"
  "llvm.nvvm.match.any.sync.i64\0"
  "llvm.nvvm.mbarrier.arrive\0"
  "llvm.nvvm.mbarrier.arrive.drop\0"
  "llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.drop.noComplete\0"
  "llvm.nvvm.mbarrier.arrive.drop.noComplete.shared\0"
  "llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.drop.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.drop.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.drop.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.drop.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.drop.shared\0"
  "llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.expect.tx.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.expect.tx.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.expect.tx.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.expect.tx.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.noComplete\0"
  "llvm.nvvm.mbarrier.arrive.noComplete.shared\0"
  "llvm.nvvm.mbarrier.arrive.relaxed.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.relaxed.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.arrive.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.arrive.shared\0"
  "llvm.nvvm.mbarrier.complete.tx.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.complete.tx.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.complete.tx.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.complete.tx.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.expect.tx.scope.cluster.space.cluster\0"
  "llvm.nvvm.mbarrier.expect.tx.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.expect.tx.scope.cta.space.cluster\0"
  "llvm.nvvm.mbarrier.expect.tx.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.init\0"
  "llvm.nvvm.mbarrier.init.shared\0"
  "llvm.nvvm.mbarrier.inval\0"
  "llvm.nvvm.mbarrier.inval.shared\0"
  "llvm.nvvm.mbarrier.pending.count\0"
  "llvm.nvvm.mbarrier.test.wait\0"
  "llvm.nvvm.mbarrier.test.wait.parity.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.test.wait.parity.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.test.wait.parity.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.test.wait.parity.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.test.wait.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.test.wait.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.test.wait.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.test.wait.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.test.wait.shared\0"
  "llvm.nvvm.mbarrier.try.wait.parity.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.parity.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.parity.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.parity.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.parity.tl.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.parity.tl.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.parity.tl.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.parity.tl.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.tl.relaxed.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.tl.relaxed.scope.cta.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.tl.scope.cluster.space.cta\0"
  "llvm.nvvm.mbarrier.try.wait.tl.scope.cta.space.cta\0"
  "llvm.nvvm.membar.cta\0"
  "llvm.nvvm.membar.gl\0"
  "llvm.nvvm.membar.sys\0"
  "llvm.nvvm.mma.and.popc.m16n8k128.row.col.b1\0"
  "llvm.nvvm.mma.and.popc.m16n8k256.row.col.b1\0"
  "llvm.nvvm.mma.and.popc.m8n8k128.row.col.b1\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4nvf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4nvf4.scale.4x.f32.e2m1.e2m1.f32.ue4m3\0"
  "llvm.nvvm.mma.m16n8k16.row.col.bf16\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f16.e4m3.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f16.e4m3.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f16.e5m2.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f16.e5m2.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f16.f16\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f32.e4m3.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f32.e4m3.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f32.e5m2.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f32.e5m2.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f32.f32\0"
  "llvm.nvvm.mma.m16n8k16.row.col.f64\0"
  "llvm.nvvm.mma.m16n8k16.row.col.s8\0"
  "llvm.nvvm.mma.m16n8k16.row.col.s8.u8\0"
  "llvm.nvvm.mma.m16n8k16.row.col.satfinite.s8\0"
  "llvm.nvvm.mma.m16n8k16.row.col.satfinite.s8.u8\0"
  "llvm.nvvm.mma.m16n8k16.row.col.satfinite.u8\0"
  "llvm.nvvm.mma.m16n8k16.row.col.satfinite.u8.s8\0"
  "llvm.nvvm.mma.m16n8k16.row.col.u8\0"
  "llvm.nvvm.mma.m16n8k16.row.col.u8.s8\0"
  "llvm.nvvm.mma.m16n8k32.row.col.f16.e4m3.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.f16.e4m3.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.f16.e5m2.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.f16.e5m2.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.f32.e4m3.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.f32.e4m3.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.f32.e5m2.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.f32.e5m2.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e2m1.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e2m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e3m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e2m1.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e2m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e3m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e2m1.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e2m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e3m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e2m1.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e2m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e3m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e2m1.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e2m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e3m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e4m3.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e5m2.f16\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e2m1.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e2m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e3m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e2m1.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e2m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e3m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e2m1.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e2m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e3m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e2m1.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e2m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e3m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e2m1.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e2m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e3m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e4m3.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e5m2.f32\0"
  "llvm.nvvm.mma.m16n8k32.row.col.s4\0"
  "llvm.nvvm.mma.m16n8k32.row.col.s4.u4\0"
  "llvm.nvvm.mma.m16n8k32.row.col.s8\0"
  "llvm.nvvm.mma.m16n8k32.row.col.s8.u8\0"
  "llvm.nvvm.mma.m16n8k32.row.col.satfinite.s4\0"
  "llvm.nvvm.mma.m16n8k32.row.col.satfinite.s4.u4\0"
  "llvm.nvvm.mma.m16n8k32.row.col.satfinite.s8\0"
  "llvm.nvvm.mma.m16n8k32.row.col.satfinite.s8.u8\0"
  "llvm.nvvm.mma.m16n8k32.row.col.satfinite.u4\0"
  "llvm.nvvm.mma.m16n8k32.row.col.satfinite.u4.s4\0"
  "llvm.nvvm.mma.m16n8k32.row.col.satfinite.u8\0"
  "llvm.nvvm.mma.m16n8k32.row.col.satfinite.u8.s8\0"
  "llvm.nvvm.mma.m16n8k32.row.col.u4\0"
  "llvm.nvvm.mma.m16n8k32.row.col.u4.s4\0"
  "llvm.nvvm.mma.m16n8k32.row.col.u8\0"
  "llvm.nvvm.mma.m16n8k32.row.col.u8.s8\0"
  "llvm.nvvm.mma.m16n8k4.row.col.f64\0"
  "llvm.nvvm.mma.m16n8k4.row.col.tf32\0"
  "llvm.nvvm.mma.m16n8k64.row.col.s4\0"
  "llvm.nvvm.mma.m16n8k64.row.col.s4.u4\0"
  "llvm.nvvm.mma.m16n8k64.row.col.satfinite.s4\0"
  "llvm.nvvm.mma.m16n8k64.row.col.satfinite.s4.u4\0"
  "llvm.nvvm.mma.m16n8k64.row.col.satfinite.u4\0"
  "llvm.nvvm.mma.m16n8k64.row.col.satfinite.u4.s4\0"
  "llvm.nvvm.mma.m16n8k64.row.col.u4\0"
  "llvm.nvvm.mma.m16n8k64.row.col.u4.s4\0"
  "llvm.nvvm.mma.m16n8k8.row.col.bf16\0"
  "llvm.nvvm.mma.m16n8k8.row.col.f16.f16\0"
  "llvm.nvvm.mma.m16n8k8.row.col.f32.f32\0"
  "llvm.nvvm.mma.m16n8k8.row.col.f64\0"
  "llvm.nvvm.mma.m16n8k8.row.col.tf32\0"
  "llvm.nvvm.mma.m8n8k16.row.col.s8\0"
  "llvm.nvvm.mma.m8n8k16.row.col.s8.u8\0"
  "llvm.nvvm.mma.m8n8k16.row.col.satfinite.s8\0"
  "llvm.nvvm.mma.m8n8k16.row.col.satfinite.s8.u8\0"
  "llvm.nvvm.mma.m8n8k16.row.col.satfinite.u8\0"
  "llvm.nvvm.mma.m8n8k16.row.col.satfinite.u8.s8\0"
  "llvm.nvvm.mma.m8n8k16.row.col.u8\0"
  "llvm.nvvm.mma.m8n8k16.row.col.u8.s8\0"
  "llvm.nvvm.mma.m8n8k32.row.col.s4\0"
  "llvm.nvvm.mma.m8n8k32.row.col.s4.u4\0"
  "llvm.nvvm.mma.m8n8k32.row.col.satfinite.s4\0"
  "llvm.nvvm.mma.m8n8k32.row.col.satfinite.s4.u4\0"
  "llvm.nvvm.mma.m8n8k32.row.col.satfinite.u4\0"
  "llvm.nvvm.mma.m8n8k32.row.col.satfinite.u4.s4\0"
  "llvm.nvvm.mma.m8n8k32.row.col.u4\0"
  "llvm.nvvm.mma.m8n8k32.row.col.u4.s4\0"
  "llvm.nvvm.mma.m8n8k4.col.col.f16.f16\0"
  "llvm.nvvm.mma.m8n8k4.col.col.f32.f16\0"
  "llvm.nvvm.mma.m8n8k4.col.col.f32.f32\0"
  "llvm.nvvm.mma.m8n8k4.col.row.f16.f16\0"
  "llvm.nvvm.mma.m8n8k4.col.row.f32.f16\0"
  "llvm.nvvm.mma.m8n8k4.col.row.f32.f32\0"
  "llvm.nvvm.mma.m8n8k4.row.col.f16.f16\0"
  "llvm.nvvm.mma.m8n8k4.row.col.f32.f16\0"
  "llvm.nvvm.mma.m8n8k4.row.col.f32.f32\0"
  "llvm.nvvm.mma.m8n8k4.row.col.f64\0"
  "llvm.nvvm.mma.m8n8k4.row.row.f16.f16\0"
  "llvm.nvvm.mma.m8n8k4.row.row.f32.f16\0"
  "llvm.nvvm.mma.m8n8k4.row.row.f32.f32\0"
  "llvm.nvvm.mma.sp.m16n8k128.row.col.s4\0"
  "llvm.nvvm.mma.sp.m16n8k128.row.col.s4.u4\0"
  "llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.s4\0"
  "llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.s4.u4\0"
  "llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.u4\0"
  "llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.u4.s4\0"
  "llvm.nvvm.mma.sp.m16n8k128.row.col.u4\0"
  "llvm.nvvm.mma.sp.m16n8k128.row.col.u4.s4\0"
  "llvm.nvvm.mma.sp.m16n8k16.row.col.bf16\0"
  "llvm.nvvm.mma.sp.m16n8k16.row.col.f16.f16\0"
  "llvm.nvvm.mma.sp.m16n8k16.row.col.f32.f32\0"
  "llvm.nvvm.mma.sp.m16n8k16.row.col.tf32\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.bf16\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.f16.f16\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.f32.f32\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.s8\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.s8.u8\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.s8\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.s8.u8\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.u8\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.u8.s8\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.u8\0"
  "llvm.nvvm.mma.sp.m16n8k32.row.col.u8.s8\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e4m3.e4m3.f32\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e4m3.e5m2.f32\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e5m2.e4m3.f32\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e5m2.e5m2.f32\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.s4\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.s4.u4\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.s8\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.s8.u8\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s4\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s4.u4\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s8\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s8.u8\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u4\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u4.s4\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u8\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u8.s8\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.u4\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.u4.s4\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.u8\0"
  "llvm.nvvm.mma.sp.m16n8k64.row.col.u8.s8\0"
  "llvm.nvvm.mma.sp.m16n8k8.row.col.tf32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4nvf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4nvf4.scale.4x.f32.e2m1.e2m1.f32.ue4m3\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m1.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e3m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e4m3.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e5m2.f32.ue8m0\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.s4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.s4.u4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.s4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.s4.u4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.u4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.u4.s4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.u4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.u4.s4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.bf16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.f16.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.f32.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.tf32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.bf16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.f16.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.f32.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.s8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.s8.u8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.s8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.s8.u8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.u8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.u8.s8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.u8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.u8.s8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e4m3.e4m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e4m3.e5m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e5m2.e4m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e5m2.e5m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e2m1.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e2m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e3m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e4m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e5m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e2m1.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e2m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e3m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e4m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e5m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e2m1.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e2m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e3m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e4m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e5m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e2m1.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e2m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e3m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e4m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e5m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e2m1.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e2m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e3m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e4m3.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e5m2.f16\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e2m1.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e2m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e3m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e4m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e5m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e2m1.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e2m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e3m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e4m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e5m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e2m1.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e2m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e3m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e4m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e5m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e2m1.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e2m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e3m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e4m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e5m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e2m1.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e2m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e3m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e4m3.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e5m2.f32\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s4.u4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s8.u8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s4.u4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s8.u8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u4.s4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u8.s8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u4.s4\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u8.s8\0"
  "llvm.nvvm.mma.sp.ordered.metadata.m16n8k8.row.col.tf32\0"
  "llvm.nvvm.mma.xor.popc.m16n8k128.row.col.b1\0"
  "llvm.nvvm.mma.xor.popc.m16n8k256.row.col.b1\0"
  "llvm.nvvm.mma.xor.popc.m8n8k128.row.col.b1\0"
  "llvm.nvvm.move.double\0"
  "llvm.nvvm.move.float\0"
  "llvm.nvvm.move.i16\0"
  "llvm.nvvm.move.i32\0"
  "llvm.nvvm.move.i64\0"
  "llvm.nvvm.move.ptr\0"
  "llvm.nvvm.mul.rm.d\0"
  "llvm.nvvm.mul.rm.f\0"
  "llvm.nvvm.mul.rm.ftz.f\0"
  "llvm.nvvm.mul.rn.d\0"
  "llvm.nvvm.mul.rn.f\0"
  "llvm.nvvm.mul.rn.ftz.f\0"
  "llvm.nvvm.mul.rp.d\0"
  "llvm.nvvm.mul.rp.f\0"
  "llvm.nvvm.mul.rp.ftz.f\0"
  "llvm.nvvm.mul.rz.d\0"
  "llvm.nvvm.mul.rz.f\0"
  "llvm.nvvm.mul.rz.ftz.f\0"
  "llvm.nvvm.mul24.i\0"
  "llvm.nvvm.mul24.ui\0"
  "llvm.nvvm.mulhi.i\0"
  "llvm.nvvm.mulhi.ll\0"
  "llvm.nvvm.mulhi.s\0"
  "llvm.nvvm.mulhi.ui\0"
  "llvm.nvvm.mulhi.ull\0"
  "llvm.nvvm.mulhi.us\0"
  "llvm.nvvm.nanosleep\0"
  "llvm.nvvm.neg.bf16\0"
  "llvm.nvvm.neg.bf16x2\0"
  "llvm.nvvm.pm.event.mask\0"
  "llvm.nvvm.prefetch.L1\0"
  "llvm.nvvm.prefetch.L2\0"
  "llvm.nvvm.prefetch.global.L1\0"
  "llvm.nvvm.prefetch.global.L2\0"
  "llvm.nvvm.prefetch.global.L2.evict.last\0"
  "llvm.nvvm.prefetch.global.L2.evict.normal\0"
  "llvm.nvvm.prefetch.local.L1\0"
  "llvm.nvvm.prefetch.local.L2\0"
  "llvm.nvvm.prefetch.tensormap\0"
  "llvm.nvvm.prefetchu.L1\0"
  "llvm.nvvm.prmt\0"
  "llvm.nvvm.prmt.b4e\0"
  "llvm.nvvm.prmt.ecl\0"
  "llvm.nvvm.prmt.ecr\0"
  "llvm.nvvm.prmt.f4e\0"
  "llvm.nvvm.prmt.rc16\0"
  "llvm.nvvm.prmt.rc8\0"
  "llvm.nvvm.rcp.approx.ftz.d\0"
  "llvm.nvvm.rcp.approx.ftz.f\0"
  "llvm.nvvm.rcp.rm.d\0"
  "llvm.nvvm.rcp.rm.f\0"
  "llvm.nvvm.rcp.rm.ftz.f\0"
  "llvm.nvvm.rcp.rn.d\0"
  "llvm.nvvm.rcp.rn.f\0"
  "llvm.nvvm.rcp.rn.ftz.f\0"
  "llvm.nvvm.rcp.rp.d\0"
  "llvm.nvvm.rcp.rp.f\0"
  "llvm.nvvm.rcp.rp.ftz.f\0"
  "llvm.nvvm.rcp.rz.d\0"
  "llvm.nvvm.rcp.rz.f\0"
  "llvm.nvvm.rcp.rz.ftz.f\0"
  "llvm.nvvm.read.ptx.sreg.aggr_smem_size\0"
  "llvm.nvvm.read.ptx.sreg.clock\0"
  "llvm.nvvm.read.ptx.sreg.clock64\0"
  "llvm.nvvm.read.ptx.sreg.cluster.ctaid.w\0"
  "llvm.nvvm.read.ptx.sreg.cluster.ctaid.x\0"
  "llvm.nvvm.read.ptx.sreg.cluster.ctaid.y\0"
  "llvm.nvvm.read.ptx.sreg.cluster.ctaid.z\0"
  "llvm.nvvm.read.ptx.sreg.cluster.ctarank\0"
  "llvm.nvvm.read.ptx.sreg.cluster.nctaid.w\0"
  "llvm.nvvm.read.ptx.sreg.cluster.nctaid.x\0"
  "llvm.nvvm.read.ptx.sreg.cluster.nctaid.y\0"
  "llvm.nvvm.read.ptx.sreg.cluster.nctaid.z\0"
  "llvm.nvvm.read.ptx.sreg.cluster.nctarank\0"
  "llvm.nvvm.read.ptx.sreg.clusterid.w\0"
  "llvm.nvvm.read.ptx.sreg.clusterid.x\0"
  "llvm.nvvm.read.ptx.sreg.clusterid.y\0"
  "llvm.nvvm.read.ptx.sreg.clusterid.z\0"
  "llvm.nvvm.read.ptx.sreg.ctaid.w\0"
  "llvm.nvvm.read.ptx.sreg.ctaid.x\0"
  "llvm.nvvm.read.ptx.sreg.ctaid.y\0"
  "llvm.nvvm.read.ptx.sreg.ctaid.z\0"
  "llvm.nvvm.read.ptx.sreg.dynamic_smem_size\0"
  "llvm.nvvm.read.ptx.sreg.envreg0\0"
  "llvm.nvvm.read.ptx.sreg.envreg1\0"
  "llvm.nvvm.read.ptx.sreg.envreg10\0"
  "llvm.nvvm.read.ptx.sreg.envreg11\0"
  "llvm.nvvm.read.ptx.sreg.envreg12\0"
  "llvm.nvvm.read.ptx.sreg.envreg13\0"
  "llvm.nvvm.read.ptx.sreg.envreg14\0"
  "llvm.nvvm.read.ptx.sreg.envreg15\0"
  "llvm.nvvm.read.ptx.sreg.envreg16\0"
  "llvm.nvvm.read.ptx.sreg.envreg17\0"
  "llvm.nvvm.read.ptx.sreg.envreg18\0"
  "llvm.nvvm.read.ptx.sreg.envreg19\0"
  "llvm.nvvm.read.ptx.sreg.envreg2\0"
  "llvm.nvvm.read.ptx.sreg.envreg20\0"
  "llvm.nvvm.read.ptx.sreg.envreg21\0"
  "llvm.nvvm.read.ptx.sreg.envreg22\0"
  "llvm.nvvm.read.ptx.sreg.envreg23\0"
  "llvm.nvvm.read.ptx.sreg.envreg24\0"
  "llvm.nvvm.read.ptx.sreg.envreg25\0"
  "llvm.nvvm.read.ptx.sreg.envreg26\0"
  "llvm.nvvm.read.ptx.sreg.envreg27\0"
  "llvm.nvvm.read.ptx.sreg.envreg28\0"
  "llvm.nvvm.read.ptx.sreg.envreg29\0"
  "llvm.nvvm.read.ptx.sreg.envreg3\0"
  "llvm.nvvm.read.ptx.sreg.envreg30\0"
  "llvm.nvvm.read.ptx.sreg.envreg31\0"
  "llvm.nvvm.read.ptx.sreg.envreg4\0"
  "llvm.nvvm.read.ptx.sreg.envreg5\0"
  "llvm.nvvm.read.ptx.sreg.envreg6\0"
  "llvm.nvvm.read.ptx.sreg.envreg7\0"
  "llvm.nvvm.read.ptx.sreg.envreg8\0"
  "llvm.nvvm.read.ptx.sreg.envreg9\0"
  "llvm.nvvm.read.ptx.sreg.globaltimer\0"
  "llvm.nvvm.read.ptx.sreg.globaltimer.lo\0"
  "llvm.nvvm.read.ptx.sreg.gridid\0"
  "llvm.nvvm.read.ptx.sreg.laneid\0"
  "llvm.nvvm.read.ptx.sreg.lanemask.eq\0"
  "llvm.nvvm.read.ptx.sreg.lanemask.ge\0"
  "llvm.nvvm.read.ptx.sreg.lanemask.gt\0"
  "llvm.nvvm.read.ptx.sreg.lanemask.le\0"
  "llvm.nvvm.read.ptx.sreg.lanemask.lt\0"
  "llvm.nvvm.read.ptx.sreg.nclusterid.w\0"
  "llvm.nvvm.read.ptx.sreg.nclusterid.x\0"
  "llvm.nvvm.read.ptx.sreg.nclusterid.y\0"
  "llvm.nvvm.read.ptx.sreg.nclusterid.z\0"
  "llvm.nvvm.read.ptx.sreg.nctaid.w\0"
  "llvm.nvvm.read.ptx.sreg.nctaid.x\0"
  "llvm.nvvm.read.ptx.sreg.nctaid.y\0"
  "llvm.nvvm.read.ptx.sreg.nctaid.z\0"
  "llvm.nvvm.read.ptx.sreg.nsmid\0"
  "llvm.nvvm.read.ptx.sreg.ntid.w\0"
  "llvm.nvvm.read.ptx.sreg.ntid.x\0"
  "llvm.nvvm.read.ptx.sreg.ntid.y\0"
  "llvm.nvvm.read.ptx.sreg.ntid.z\0"
  "llvm.nvvm.read.ptx.sreg.nwarpid\0"
  "llvm.nvvm.read.ptx.sreg.pm0\0"
  "llvm.nvvm.read.ptx.sreg.pm1\0"
  "llvm.nvvm.read.ptx.sreg.pm2\0"
  "llvm.nvvm.read.ptx.sreg.pm3\0"
  "llvm.nvvm.read.ptx.sreg.smid\0"
  "llvm.nvvm.read.ptx.sreg.tid.w\0"
  "llvm.nvvm.read.ptx.sreg.tid.x\0"
  "llvm.nvvm.read.ptx.sreg.tid.y\0"
  "llvm.nvvm.read.ptx.sreg.tid.z\0"
  "llvm.nvvm.read.ptx.sreg.total_smem_size\0"
  "llvm.nvvm.read.ptx.sreg.warpid\0"
  "llvm.nvvm.read.ptx.sreg.warpsize\0"
  "llvm.nvvm.redux.sync.add\0"
  "llvm.nvvm.redux.sync.and\0"
  "llvm.nvvm.redux.sync.fmax\0"
  "llvm.nvvm.redux.sync.fmax.NaN\0"
  "llvm.nvvm.redux.sync.fmax.abs\0"
  "llvm.nvvm.redux.sync.fmax.abs.NaN\0"
  "llvm.nvvm.redux.sync.fmin\0"
  "llvm.nvvm.redux.sync.fmin.NaN\0"
  "llvm.nvvm.redux.sync.fmin.abs\0"
  "llvm.nvvm.redux.sync.fmin.abs.NaN\0"
  "llvm.nvvm.redux.sync.max\0"
  "llvm.nvvm.redux.sync.min\0"
  "llvm.nvvm.redux.sync.or\0"
  "llvm.nvvm.redux.sync.umax\0"
  "llvm.nvvm.redux.sync.umin\0"
  "llvm.nvvm.redux.sync.xor\0"
  "llvm.nvvm.reflect\0"
  "llvm.nvvm.round.d\0"
  "llvm.nvvm.round.f\0"
  "llvm.nvvm.round.ftz.f\0"
  "llvm.nvvm.rsqrt.approx.d\0"
  "llvm.nvvm.rsqrt.approx.f\0"
  "llvm.nvvm.rsqrt.approx.ftz.d\0"
  "llvm.nvvm.rsqrt.approx.ftz.f\0"
  "llvm.nvvm.sad.i\0"
  "llvm.nvvm.sad.ll\0"
  "llvm.nvvm.sad.s\0"
  "llvm.nvvm.sad.ui\0"
  "llvm.nvvm.sad.ull\0"
  "llvm.nvvm.sad.us\0"
  "llvm.nvvm.saturate.d\0"
  "llvm.nvvm.saturate.f\0"
  "llvm.nvvm.saturate.ftz.f\0"
  "llvm.nvvm.setmaxnreg.dec.sync.aligned.u32\0"
  "llvm.nvvm.setmaxnreg.inc.sync.aligned.u32\0"
  "llvm.nvvm.sext.clamp\0"
  "llvm.nvvm.sext.wrap\0"
  "llvm.nvvm.shfl.bfly.f32\0"
  "llvm.nvvm.shfl.bfly.f32p\0"
  "llvm.nvvm.shfl.bfly.i32\0"
  "llvm.nvvm.shfl.bfly.i32p\0"
  "llvm.nvvm.shfl.down.f32\0"
  "llvm.nvvm.shfl.down.f32p\0"
  "llvm.nvvm.shfl.down.i32\0"
  "llvm.nvvm.shfl.down.i32p\0"
  "llvm.nvvm.shfl.idx.f32\0"
  "llvm.nvvm.shfl.idx.f32p\0"
  "llvm.nvvm.shfl.idx.i32\0"
  "llvm.nvvm.shfl.idx.i32p\0"
  "llvm.nvvm.shfl.sync.bfly.f32\0"
  "llvm.nvvm.shfl.sync.bfly.f32p\0"
  "llvm.nvvm.shfl.sync.bfly.i32\0"
  "llvm.nvvm.shfl.sync.bfly.i32p\0"
  "llvm.nvvm.shfl.sync.down.f32\0"
  "llvm.nvvm.shfl.sync.down.f32p\0"
  "llvm.nvvm.shfl.sync.down.i32\0"
  "llvm.nvvm.shfl.sync.down.i32p\0"
  "llvm.nvvm.shfl.sync.idx.f32\0"
  "llvm.nvvm.shfl.sync.idx.f32p\0"
  "llvm.nvvm.shfl.sync.idx.i32\0"
  "llvm.nvvm.shfl.sync.idx.i32p\0"
  "llvm.nvvm.shfl.sync.up.f32\0"
  "llvm.nvvm.shfl.sync.up.f32p\0"
  "llvm.nvvm.shfl.sync.up.i32\0"
  "llvm.nvvm.shfl.sync.up.i32p\0"
  "llvm.nvvm.shfl.up.f32\0"
  "llvm.nvvm.shfl.up.f32p\0"
  "llvm.nvvm.shfl.up.i32\0"
  "llvm.nvvm.shfl.up.i32p\0"
  "llvm.nvvm.sin.approx.f\0"
  "llvm.nvvm.sin.approx.ftz.f\0"
  "llvm.nvvm.sqrt.approx.f\0"
  "llvm.nvvm.sqrt.approx.ftz.f\0"
  "llvm.nvvm.sqrt.f\0"
  "llvm.nvvm.sqrt.rm.d\0"
  "llvm.nvvm.sqrt.rm.f\0"
  "llvm.nvvm.sqrt.rm.ftz.f\0"
  "llvm.nvvm.sqrt.rn.d\0"
  "llvm.nvvm.sqrt.rn.f\0"
  "llvm.nvvm.sqrt.rn.ftz.f\0"
  "llvm.nvvm.sqrt.rp.d\0"
  "llvm.nvvm.sqrt.rp.f\0"
  "llvm.nvvm.sqrt.rp.ftz.f\0"
  "llvm.nvvm.sqrt.rz.d\0"
  "llvm.nvvm.sqrt.rz.f\0"
  "llvm.nvvm.sqrt.rz.ftz.f\0"
  "llvm.nvvm.st.bulk\0"
  "llvm.nvvm.st.bulk.shared.cta\0"
  "llvm.nvvm.stmatrix.sync.aligned.m16n8.x1.trans.b8\0"
  "llvm.nvvm.stmatrix.sync.aligned.m16n8.x2.trans.b8\0"
  "llvm.nvvm.stmatrix.sync.aligned.m16n8.x4.trans.b8\0"
  "llvm.nvvm.stmatrix.sync.aligned.m8n8.x1.b16\0"
  "llvm.nvvm.stmatrix.sync.aligned.m8n8.x1.trans.b16\0"
  "llvm.nvvm.stmatrix.sync.aligned.m8n8.x2.b16\0"
  "llvm.nvvm.stmatrix.sync.aligned.m8n8.x2.trans.b16\0"
  "llvm.nvvm.stmatrix.sync.aligned.m8n8.x4.b16\0"
  "llvm.nvvm.stmatrix.sync.aligned.m8n8.x4.trans.b16\0"
  "llvm.nvvm.suld.1d.array.i16.clamp\0"
  "llvm.nvvm.suld.1d.array.i16.trap\0"
  "llvm.nvvm.suld.1d.array.i16.zero\0"
  "llvm.nvvm.suld.1d.array.i32.clamp\0"
  "llvm.nvvm.suld.1d.array.i32.trap\0"
  "llvm.nvvm.suld.1d.array.i32.zero\0"
  "llvm.nvvm.suld.1d.array.i64.clamp\0"
  "llvm.nvvm.suld.1d.array.i64.trap\0"
  "llvm.nvvm.suld.1d.array.i64.zero\0"
  "llvm.nvvm.suld.1d.array.i8.clamp\0"
  "llvm.nvvm.suld.1d.array.i8.trap\0"
  "llvm.nvvm.suld.1d.array.i8.zero\0"
  "llvm.nvvm.suld.1d.array.v2i16.clamp\0"
  "llvm.nvvm.suld.1d.array.v2i16.trap\0"
  "llvm.nvvm.suld.1d.array.v2i16.zero\0"
  "llvm.nvvm.suld.1d.array.v2i32.clamp\0"
  "llvm.nvvm.suld.1d.array.v2i32.trap\0"
  "llvm.nvvm.suld.1d.array.v2i32.zero\0"
  "llvm.nvvm.suld.1d.array.v2i64.clamp\0"
  "llvm.nvvm.suld.1d.array.v2i64.trap\0"
  "llvm.nvvm.suld.1d.array.v2i64.zero\0"
  "llvm.nvvm.suld.1d.array.v2i8.clamp\0"
  "llvm.nvvm.suld.1d.array.v2i8.trap\0"
  "llvm.nvvm.suld.1d.array.v2i8.zero\0"
  "llvm.nvvm.suld.1d.array.v4i16.clamp\0"
  "llvm.nvvm.suld.1d.array.v4i16.trap\0"
  "llvm.nvvm.suld.1d.array.v4i16.zero\0"
  "llvm.nvvm.suld.1d.array.v4i32.clamp\0"
  "llvm.nvvm.suld.1d.array.v4i32.trap\0"
  "llvm.nvvm.suld.1d.array.v4i32.zero\0"
  "llvm.nvvm.suld.1d.array.v4i8.clamp\0"
  "llvm.nvvm.suld.1d.array.v4i8.trap\0"
  "llvm.nvvm.suld.1d.array.v4i8.zero\0"
  "llvm.nvvm.suld.1d.i16.clamp\0"
  "llvm.nvvm.suld.1d.i16.trap\0"
  "llvm.nvvm.suld.1d.i16.zero\0"
  "llvm.nvvm.suld.1d.i32.clamp\0"
  "llvm.nvvm.suld.1d.i32.trap\0"
  "llvm.nvvm.suld.1d.i32.zero\0"
  "llvm.nvvm.suld.1d.i64.clamp\0"
  "llvm.nvvm.suld.1d.i64.trap\0"
  "llvm.nvvm.suld.1d.i64.zero\0"
  "llvm.nvvm.suld.1d.i8.clamp\0"
  "llvm.nvvm.suld.1d.i8.trap\0"
  "llvm.nvvm.suld.1d.i8.zero\0"
  "llvm.nvvm.suld.1d.v2i16.clamp\0"
  "llvm.nvvm.suld.1d.v2i16.trap\0"
  "llvm.nvvm.suld.1d.v2i16.zero\0"
  "llvm.nvvm.suld.1d.v2i32.clamp\0"
  "llvm.nvvm.suld.1d.v2i32.trap\0"
  "llvm.nvvm.suld.1d.v2i32.zero\0"
  "llvm.nvvm.suld.1d.v2i64.clamp\0"
  "llvm.nvvm.suld.1d.v2i64.trap\0"
  "llvm.nvvm.suld.1d.v2i64.zero\0"
  "llvm.nvvm.suld.1d.v2i8.clamp\0"
  "llvm.nvvm.suld.1d.v2i8.trap\0"
  "llvm.nvvm.suld.1d.v2i8.zero\0"
  "llvm.nvvm.suld.1d.v4i16.clamp\0"
  "llvm.nvvm.suld.1d.v4i16.trap\0"
  "llvm.nvvm.suld.1d.v4i16.zero\0"
  "llvm.nvvm.suld.1d.v4i32.clamp\0"
  "llvm.nvvm.suld.1d.v4i32.trap\0"
  "llvm.nvvm.suld.1d.v4i32.zero\0"
  "llvm.nvvm.suld.1d.v4i8.clamp\0"
  "llvm.nvvm.suld.1d.v4i8.trap\0"
  "llvm.nvvm.suld.1d.v4i8.zero\0"
  "llvm.nvvm.suld.2d.array.i16.clamp\0"
  "llvm.nvvm.suld.2d.array.i16.trap\0"
  "llvm.nvvm.suld.2d.array.i16.zero\0"
  "llvm.nvvm.suld.2d.array.i32.clamp\0"
  "llvm.nvvm.suld.2d.array.i32.trap\0"
  "llvm.nvvm.suld.2d.array.i32.zero\0"
  "llvm.nvvm.suld.2d.array.i64.clamp\0"
  "llvm.nvvm.suld.2d.array.i64.trap\0"
  "llvm.nvvm.suld.2d.array.i64.zero\0"
  "llvm.nvvm.suld.2d.array.i8.clamp\0"
  "llvm.nvvm.suld.2d.array.i8.trap\0"
  "llvm.nvvm.suld.2d.array.i8.zero\0"
  "llvm.nvvm.suld.2d.array.v2i16.clamp\0"
  "llvm.nvvm.suld.2d.array.v2i16.trap\0"
  "llvm.nvvm.suld.2d.array.v2i16.zero\0"
  "llvm.nvvm.suld.2d.array.v2i32.clamp\0"
  "llvm.nvvm.suld.2d.array.v2i32.trap\0"
  "llvm.nvvm.suld.2d.array.v2i32.zero\0"
  "llvm.nvvm.suld.2d.array.v2i64.clamp\0"
  "llvm.nvvm.suld.2d.array.v2i64.trap\0"
  "llvm.nvvm.suld.2d.array.v2i64.zero\0"
  "llvm.nvvm.suld.2d.array.v2i8.clamp\0"
  "llvm.nvvm.suld.2d.array.v2i8.trap\0"
  "llvm.nvvm.suld.2d.array.v2i8.zero\0"
  "llvm.nvvm.suld.2d.array.v4i16.clamp\0"
  "llvm.nvvm.suld.2d.array.v4i16.trap\0"
  "llvm.nvvm.suld.2d.array.v4i16.zero\0"
  "llvm.nvvm.suld.2d.array.v4i32.clamp\0"
  "llvm.nvvm.suld.2d.array.v4i32.trap\0"
  "llvm.nvvm.suld.2d.array.v4i32.zero\0"
  "llvm.nvvm.suld.2d.array.v4i8.clamp\0"
  "llvm.nvvm.suld.2d.array.v4i8.trap\0"
  "llvm.nvvm.suld.2d.array.v4i8.zero\0"
  "llvm.nvvm.suld.2d.i16.clamp\0"
  "llvm.nvvm.suld.2d.i16.trap\0"
  "llvm.nvvm.suld.2d.i16.zero\0"
  "llvm.nvvm.suld.2d.i32.clamp\0"
  "llvm.nvvm.suld.2d.i32.trap\0"
  "llvm.nvvm.suld.2d.i32.zero\0"
  "llvm.nvvm.suld.2d.i64.clamp\0"
  "llvm.nvvm.suld.2d.i64.trap\0"
  "llvm.nvvm.suld.2d.i64.zero\0"
  "llvm.nvvm.suld.2d.i8.clamp\0"
  "llvm.nvvm.suld.2d.i8.trap\0"
  "llvm.nvvm.suld.2d.i8.zero\0"
  "llvm.nvvm.suld.2d.v2i16.clamp\0"
  "llvm.nvvm.suld.2d.v2i16.trap\0"
  "llvm.nvvm.suld.2d.v2i16.zero\0"
  "llvm.nvvm.suld.2d.v2i32.clamp\0"
  "llvm.nvvm.suld.2d.v2i32.trap\0"
  "llvm.nvvm.suld.2d.v2i32.zero\0"
  "llvm.nvvm.suld.2d.v2i64.clamp\0"
  "llvm.nvvm.suld.2d.v2i64.trap\0"
  "llvm.nvvm.suld.2d.v2i64.zero\0"
  "llvm.nvvm.suld.2d.v2i8.clamp\0"
  "llvm.nvvm.suld.2d.v2i8.trap\0"
  "llvm.nvvm.suld.2d.v2i8.zero\0"
  "llvm.nvvm.suld.2d.v4i16.clamp\0"
  "llvm.nvvm.suld.2d.v4i16.trap\0"
  "llvm.nvvm.suld.2d.v4i16.zero\0"
  "llvm.nvvm.suld.2d.v4i32.clamp\0"
  "llvm.nvvm.suld.2d.v4i32.trap\0"
  "llvm.nvvm.suld.2d.v4i32.zero\0"
  "llvm.nvvm.suld.2d.v4i8.clamp\0"
  "llvm.nvvm.suld.2d.v4i8.trap\0"
  "llvm.nvvm.suld.2d.v4i8.zero\0"
  "llvm.nvvm.suld.3d.i16.clamp\0"
  "llvm.nvvm.suld.3d.i16.trap\0"
  "llvm.nvvm.suld.3d.i16.zero\0"
  "llvm.nvvm.suld.3d.i32.clamp\0"
  "llvm.nvvm.suld.3d.i32.trap\0"
  "llvm.nvvm.suld.3d.i32.zero\0"
  "llvm.nvvm.suld.3d.i64.clamp\0"
  "llvm.nvvm.suld.3d.i64.trap\0"
  "llvm.nvvm.suld.3d.i64.zero\0"
  "llvm.nvvm.suld.3d.i8.clamp\0"
  "llvm.nvvm.suld.3d.i8.trap\0"
  "llvm.nvvm.suld.3d.i8.zero\0"
  "llvm.nvvm.suld.3d.v2i16.clamp\0"
  "llvm.nvvm.suld.3d.v2i16.trap\0"
  "llvm.nvvm.suld.3d.v2i16.zero\0"
  "llvm.nvvm.suld.3d.v2i32.clamp\0"
  "llvm.nvvm.suld.3d.v2i32.trap\0"
  "llvm.nvvm.suld.3d.v2i32.zero\0"
  "llvm.nvvm.suld.3d.v2i64.clamp\0"
  "llvm.nvvm.suld.3d.v2i64.trap\0"
  "llvm.nvvm.suld.3d.v2i64.zero\0"
  "llvm.nvvm.suld.3d.v2i8.clamp\0"
  "llvm.nvvm.suld.3d.v2i8.trap\0"
  "llvm.nvvm.suld.3d.v2i8.zero\0"
  "llvm.nvvm.suld.3d.v4i16.clamp\0"
  "llvm.nvvm.suld.3d.v4i16.trap\0"
  "llvm.nvvm.suld.3d.v4i16.zero\0"
  "llvm.nvvm.suld.3d.v4i32.clamp\0"
  "llvm.nvvm.suld.3d.v4i32.trap\0"
  "llvm.nvvm.suld.3d.v4i32.zero\0"
  "llvm.nvvm.suld.3d.v4i8.clamp\0"
  "llvm.nvvm.suld.3d.v4i8.trap\0"
  "llvm.nvvm.suld.3d.v4i8.zero\0"
  "llvm.nvvm.suq.array.size\0"
  "llvm.nvvm.suq.channel.data.type\0"
  "llvm.nvvm.suq.channel.order\0"
  "llvm.nvvm.suq.depth\0"
  "llvm.nvvm.suq.height\0"
  "llvm.nvvm.suq.width\0"
  "llvm.nvvm.sust.b.1d.array.i16.clamp\0"
  "llvm.nvvm.sust.b.1d.array.i16.trap\0"
  "llvm.nvvm.sust.b.1d.array.i16.zero\0"
  "llvm.nvvm.sust.b.1d.array.i32.clamp\0"
  "llvm.nvvm.sust.b.1d.array.i32.trap\0"
  "llvm.nvvm.sust.b.1d.array.i32.zero\0"
  "llvm.nvvm.sust.b.1d.array.i64.clamp\0"
  "llvm.nvvm.sust.b.1d.array.i64.trap\0"
  "llvm.nvvm.sust.b.1d.array.i64.zero\0"
  "llvm.nvvm.sust.b.1d.array.i8.clamp\0"
  "llvm.nvvm.sust.b.1d.array.i8.trap\0"
  "llvm.nvvm.sust.b.1d.array.i8.zero\0"
  "llvm.nvvm.sust.b.1d.array.v2i16.clamp\0"
  "llvm.nvvm.sust.b.1d.array.v2i16.trap\0"
  "llvm.nvvm.sust.b.1d.array.v2i16.zero\0"
  "llvm.nvvm.sust.b.1d.array.v2i32.clamp\0"
  "llvm.nvvm.sust.b.1d.array.v2i32.trap\0"
  "llvm.nvvm.sust.b.1d.array.v2i32.zero\0"
  "llvm.nvvm.sust.b.1d.array.v2i64.clamp\0"
  "llvm.nvvm.sust.b.1d.array.v2i64.trap\0"
  "llvm.nvvm.sust.b.1d.array.v2i64.zero\0"
  "llvm.nvvm.sust.b.1d.array.v2i8.clamp\0"
  "llvm.nvvm.sust.b.1d.array.v2i8.trap\0"
  "llvm.nvvm.sust.b.1d.array.v2i8.zero\0"
  "llvm.nvvm.sust.b.1d.array.v4i16.clamp\0"
  "llvm.nvvm.sust.b.1d.array.v4i16.trap\0"
  "llvm.nvvm.sust.b.1d.array.v4i16.zero\0"
  "llvm.nvvm.sust.b.1d.array.v4i32.clamp\0"
  "llvm.nvvm.sust.b.1d.array.v4i32.trap\0"
  "llvm.nvvm.sust.b.1d.array.v4i32.zero\0"
  "llvm.nvvm.sust.b.1d.array.v4i8.clamp\0"
  "llvm.nvvm.sust.b.1d.array.v4i8.trap\0"
  "llvm.nvvm.sust.b.1d.array.v4i8.zero\0"
  "llvm.nvvm.sust.b.1d.i16.clamp\0"
  "llvm.nvvm.sust.b.1d.i16.trap\0"
  "llvm.nvvm.sust.b.1d.i16.zero\0"
  "llvm.nvvm.sust.b.1d.i32.clamp\0"
  "llvm.nvvm.sust.b.1d.i32.trap\0"
  "llvm.nvvm.sust.b.1d.i32.zero\0"
  "llvm.nvvm.sust.b.1d.i64.clamp\0"
  "llvm.nvvm.sust.b.1d.i64.trap\0"
  "llvm.nvvm.sust.b.1d.i64.zero\0"
  "llvm.nvvm.sust.b.1d.i8.clamp\0"
  "llvm.nvvm.sust.b.1d.i8.trap\0"
  "llvm.nvvm.sust.b.1d.i8.zero\0"
  "llvm.nvvm.sust.b.1d.v2i16.clamp\0"
  "llvm.nvvm.sust.b.1d.v2i16.trap\0"
  "llvm.nvvm.sust.b.1d.v2i16.zero\0"
  "llvm.nvvm.sust.b.1d.v2i32.clamp\0"
  "llvm.nvvm.sust.b.1d.v2i32.trap\0"
  "llvm.nvvm.sust.b.1d.v2i32.zero\0"
  "llvm.nvvm.sust.b.1d.v2i64.clamp\0"
  "llvm.nvvm.sust.b.1d.v2i64.trap\0"
  "llvm.nvvm.sust.b.1d.v2i64.zero\0"
  "llvm.nvvm.sust.b.1d.v2i8.clamp\0"
  "llvm.nvvm.sust.b.1d.v2i8.trap\0"
  "llvm.nvvm.sust.b.1d.v2i8.zero\0"
  "llvm.nvvm.sust.b.1d.v4i16.clamp\0"
  "llvm.nvvm.sust.b.1d.v4i16.trap\0"
  "llvm.nvvm.sust.b.1d.v4i16.zero\0"
  "llvm.nvvm.sust.b.1d.v4i32.clamp\0"
  "llvm.nvvm.sust.b.1d.v4i32.trap\0"
  "llvm.nvvm.sust.b.1d.v4i32.zero\0"
  "llvm.nvvm.sust.b.1d.v4i8.clamp\0"
  "llvm.nvvm.sust.b.1d.v4i8.trap\0"
  "llvm.nvvm.sust.b.1d.v4i8.zero\0"
  "llvm.nvvm.sust.b.2d.array.i16.clamp\0"
  "llvm.nvvm.sust.b.2d.array.i16.trap\0"
  "llvm.nvvm.sust.b.2d.array.i16.zero\0"
  "llvm.nvvm.sust.b.2d.array.i32.clamp\0"
  "llvm.nvvm.sust.b.2d.array.i32.trap\0"
  "llvm.nvvm.sust.b.2d.array.i32.zero\0"
  "llvm.nvvm.sust.b.2d.array.i64.clamp\0"
  "llvm.nvvm.sust.b.2d.array.i64.trap\0"
  "llvm.nvvm.sust.b.2d.array.i64.zero\0"
  "llvm.nvvm.sust.b.2d.array.i8.clamp\0"
  "llvm.nvvm.sust.b.2d.array.i8.trap\0"
  "llvm.nvvm.sust.b.2d.array.i8.zero\0"
  "llvm.nvvm.sust.b.2d.array.v2i16.clamp\0"
  "llvm.nvvm.sust.b.2d.array.v2i16.trap\0"
  "llvm.nvvm.sust.b.2d.array.v2i16.zero\0"
  "llvm.nvvm.sust.b.2d.array.v2i32.clamp\0"
  "llvm.nvvm.sust.b.2d.array.v2i32.trap\0"
  "llvm.nvvm.sust.b.2d.array.v2i32.zero\0"
  "llvm.nvvm.sust.b.2d.array.v2i64.clamp\0"
  "llvm.nvvm.sust.b.2d.array.v2i64.trap\0"
  "llvm.nvvm.sust.b.2d.array.v2i64.zero\0"
  "llvm.nvvm.sust.b.2d.array.v2i8.clamp\0"
  "llvm.nvvm.sust.b.2d.array.v2i8.trap\0"
  "llvm.nvvm.sust.b.2d.array.v2i8.zero\0"
  "llvm.nvvm.sust.b.2d.array.v4i16.clamp\0"
  "llvm.nvvm.sust.b.2d.array.v4i16.trap\0"
  "llvm.nvvm.sust.b.2d.array.v4i16.zero\0"
  "llvm.nvvm.sust.b.2d.array.v4i32.clamp\0"
  "llvm.nvvm.sust.b.2d.array.v4i32.trap\0"
  "llvm.nvvm.sust.b.2d.array.v4i32.zero\0"
  "llvm.nvvm.sust.b.2d.array.v4i8.clamp\0"
  "llvm.nvvm.sust.b.2d.array.v4i8.trap\0"
  "llvm.nvvm.sust.b.2d.array.v4i8.zero\0"
  "llvm.nvvm.sust.b.2d.i16.clamp\0"
  "llvm.nvvm.sust.b.2d.i16.trap\0"
  "llvm.nvvm.sust.b.2d.i16.zero\0"
  "llvm.nvvm.sust.b.2d.i32.clamp\0"
  "llvm.nvvm.sust.b.2d.i32.trap\0"
  "llvm.nvvm.sust.b.2d.i32.zero\0"
  "llvm.nvvm.sust.b.2d.i64.clamp\0"
  "llvm.nvvm.sust.b.2d.i64.trap\0"
  "llvm.nvvm.sust.b.2d.i64.zero\0"
  "llvm.nvvm.sust.b.2d.i8.clamp\0"
  "llvm.nvvm.sust.b.2d.i8.trap\0"
  "llvm.nvvm.sust.b.2d.i8.zero\0"
  "llvm.nvvm.sust.b.2d.v2i16.clamp\0"
  "llvm.nvvm.sust.b.2d.v2i16.trap\0"
  "llvm.nvvm.sust.b.2d.v2i16.zero\0"
  "llvm.nvvm.sust.b.2d.v2i32.clamp\0"
  "llvm.nvvm.sust.b.2d.v2i32.trap\0"
  "llvm.nvvm.sust.b.2d.v2i32.zero\0"
  "llvm.nvvm.sust.b.2d.v2i64.clamp\0"
  "llvm.nvvm.sust.b.2d.v2i64.trap\0"
  "llvm.nvvm.sust.b.2d.v2i64.zero\0"
  "llvm.nvvm.sust.b.2d.v2i8.clamp\0"
  "llvm.nvvm.sust.b.2d.v2i8.trap\0"
  "llvm.nvvm.sust.b.2d.v2i8.zero\0"
  "llvm.nvvm.sust.b.2d.v4i16.clamp\0"
  "llvm.nvvm.sust.b.2d.v4i16.trap\0"
  "llvm.nvvm.sust.b.2d.v4i16.zero\0"
  "llvm.nvvm.sust.b.2d.v4i32.clamp\0"
  "llvm.nvvm.sust.b.2d.v4i32.trap\0"
  "llvm.nvvm.sust.b.2d.v4i32.zero\0"
  "llvm.nvvm.sust.b.2d.v4i8.clamp\0"
  "llvm.nvvm.sust.b.2d.v4i8.trap\0"
  "llvm.nvvm.sust.b.2d.v4i8.zero\0"
  "llvm.nvvm.sust.b.3d.i16.clamp\0"
  "llvm.nvvm.sust.b.3d.i16.trap\0"
  "llvm.nvvm.sust.b.3d.i16.zero\0"
  "llvm.nvvm.sust.b.3d.i32.clamp\0"
  "llvm.nvvm.sust.b.3d.i32.trap\0"
  "llvm.nvvm.sust.b.3d.i32.zero\0"
  "llvm.nvvm.sust.b.3d.i64.clamp\0"
  "llvm.nvvm.sust.b.3d.i64.trap\0"
  "llvm.nvvm.sust.b.3d.i64.zero\0"
  "llvm.nvvm.sust.b.3d.i8.clamp\0"
  "llvm.nvvm.sust.b.3d.i8.trap\0"
  "llvm.nvvm.sust.b.3d.i8.zero\0"
  "llvm.nvvm.sust.b.3d.v2i16.clamp\0"
  "llvm.nvvm.sust.b.3d.v2i16.trap\0"
  "llvm.nvvm.sust.b.3d.v2i16.zero\0"
  "llvm.nvvm.sust.b.3d.v2i32.clamp\0"
  "llvm.nvvm.sust.b.3d.v2i32.trap\0"
  "llvm.nvvm.sust.b.3d.v2i32.zero\0"
  "llvm.nvvm.sust.b.3d.v2i64.clamp\0"
  "llvm.nvvm.sust.b.3d.v2i64.trap\0"
  "llvm.nvvm.sust.b.3d.v2i64.zero\0"
  "llvm.nvvm.sust.b.3d.v2i8.clamp\0"
  "llvm.nvvm.sust.b.3d.v2i8.trap\0"
  "llvm.nvvm.sust.b.3d.v2i8.zero\0"
  "llvm.nvvm.sust.b.3d.v4i16.clamp\0"
  "llvm.nvvm.sust.b.3d.v4i16.trap\0"
  "llvm.nvvm.sust.b.3d.v4i16.zero\0"
  "llvm.nvvm.sust.b.3d.v4i32.clamp\0"
  "llvm.nvvm.sust.b.3d.v4i32.trap\0"
  "llvm.nvvm.sust.b.3d.v4i32.zero\0"
  "llvm.nvvm.sust.b.3d.v4i8.clamp\0"
  "llvm.nvvm.sust.b.3d.v4i8.trap\0"
  "llvm.nvvm.sust.b.3d.v4i8.zero\0"
  "llvm.nvvm.sust.p.1d.array.i16.trap\0"
  "llvm.nvvm.sust.p.1d.array.i32.trap\0"
  "llvm.nvvm.sust.p.1d.array.i8.trap\0"
  "llvm.nvvm.sust.p.1d.array.v2i16.trap\0"
  "llvm.nvvm.sust.p.1d.array.v2i32.trap\0"
  "llvm.nvvm.sust.p.1d.array.v2i8.trap\0"
  "llvm.nvvm.sust.p.1d.array.v4i16.trap\0"
  "llvm.nvvm.sust.p.1d.array.v4i32.trap\0"
  "llvm.nvvm.sust.p.1d.array.v4i8.trap\0"
  "llvm.nvvm.sust.p.1d.i16.trap\0"
  "llvm.nvvm.sust.p.1d.i32.trap\0"
  "llvm.nvvm.sust.p.1d.i8.trap\0"
  "llvm.nvvm.sust.p.1d.v2i16.trap\0"
  "llvm.nvvm.sust.p.1d.v2i32.trap\0"
  "llvm.nvvm.sust.p.1d.v2i8.trap\0"
  "llvm.nvvm.sust.p.1d.v4i16.trap\0"
  "llvm.nvvm.sust.p.1d.v4i32.trap\0"
  "llvm.nvvm.sust.p.1d.v4i8.trap\0"
  "llvm.nvvm.sust.p.2d.array.i16.trap\0"
  "llvm.nvvm.sust.p.2d.array.i32.trap\0"
  "llvm.nvvm.sust.p.2d.array.i8.trap\0"
  "llvm.nvvm.sust.p.2d.array.v2i16.trap\0"
  "llvm.nvvm.sust.p.2d.array.v2i32.trap\0"
  "llvm.nvvm.sust.p.2d.array.v2i8.trap\0"
  "llvm.nvvm.sust.p.2d.array.v4i16.trap\0"
  "llvm.nvvm.sust.p.2d.array.v4i32.trap\0"
  "llvm.nvvm.sust.p.2d.array.v4i8.trap\0"
  "llvm.nvvm.sust.p.2d.i16.trap\0"
  "llvm.nvvm.sust.p.2d.i32.trap\0"
  "llvm.nvvm.sust.p.2d.i8.trap\0"
  "llvm.nvvm.sust.p.2d.v2i16.trap\0"
  "llvm.nvvm.sust.p.2d.v2i32.trap\0"
  "llvm.nvvm.sust.p.2d.v2i8.trap\0"
  "llvm.nvvm.sust.p.2d.v4i16.trap\0"
  "llvm.nvvm.sust.p.2d.v4i32.trap\0"
  "llvm.nvvm.sust.p.2d.v4i8.trap\0"
  "llvm.nvvm.sust.p.3d.i16.trap\0"
  "llvm.nvvm.sust.p.3d.i32.trap\0"
  "llvm.nvvm.sust.p.3d.i8.trap\0"
  "llvm.nvvm.sust.p.3d.v2i16.trap\0"
  "llvm.nvvm.sust.p.3d.v2i32.trap\0"
  "llvm.nvvm.sust.p.3d.v2i8.trap\0"
  "llvm.nvvm.sust.p.3d.v4i16.trap\0"
  "llvm.nvvm.sust.p.3d.v4i32.trap\0"
  "llvm.nvvm.sust.p.3d.v4i8.trap\0"
  "llvm.nvvm.tcgen05.alloc.cg1\0"
  "llvm.nvvm.tcgen05.alloc.cg2\0"
  "llvm.nvvm.tcgen05.alloc.shared.cg1\0"
  "llvm.nvvm.tcgen05.alloc.shared.cg2\0"
  "llvm.nvvm.tcgen05.commit.cg1\0"
  "llvm.nvvm.tcgen05.commit.cg2\0"
  "llvm.nvvm.tcgen05.commit.mc.cg1\0"
  "llvm.nvvm.tcgen05.commit.mc.cg2\0"
  "llvm.nvvm.tcgen05.commit.mc.shared.cg1\0"
  "llvm.nvvm.tcgen05.commit.mc.shared.cg2\0"
  "llvm.nvvm.tcgen05.commit.shared.cg1\0"
  "llvm.nvvm.tcgen05.commit.shared.cg2\0"
  "llvm.nvvm.tcgen05.cp.128x128b.b4x16_p64.cg1\0"
  "llvm.nvvm.tcgen05.cp.128x128b.b4x16_p64.cg2\0"
  "llvm.nvvm.tcgen05.cp.128x128b.b6x16_p32.cg1\0"
  "llvm.nvvm.tcgen05.cp.128x128b.b6x16_p32.cg2\0"
  "llvm.nvvm.tcgen05.cp.128x128b.cg1\0"
  "llvm.nvvm.tcgen05.cp.128x128b.cg2\0"
  "llvm.nvvm.tcgen05.cp.128x256b.b4x16_p64.cg1\0"
  "llvm.nvvm.tcgen05.cp.128x256b.b4x16_p64.cg2\0"
  "llvm.nvvm.tcgen05.cp.128x256b.b6x16_p32.cg1\0"
  "llvm.nvvm.tcgen05.cp.128x256b.b6x16_p32.cg2\0"
  "llvm.nvvm.tcgen05.cp.128x256b.cg1\0"
  "llvm.nvvm.tcgen05.cp.128x256b.cg2\0"
  "llvm.nvvm.tcgen05.cp.32x128b_warpx4.b4x16_p64.cg1\0"
  "llvm.nvvm.tcgen05.cp.32x128b_warpx4.b4x16_p64.cg2\0"
  "llvm.nvvm.tcgen05.cp.32x128b_warpx4.b6x16_p32.cg1\0"
  "llvm.nvvm.tcgen05.cp.32x128b_warpx4.b6x16_p32.cg2\0"
  "llvm.nvvm.tcgen05.cp.32x128b_warpx4.cg1\0"
  "llvm.nvvm.tcgen05.cp.32x128b_warpx4.cg2\0"
  "llvm.nvvm.tcgen05.cp.4x256b.b4x16_p64.cg1\0"
  "llvm.nvvm.tcgen05.cp.4x256b.b4x16_p64.cg2\0"
  "llvm.nvvm.tcgen05.cp.4x256b.b6x16_p32.cg1\0"
  "llvm.nvvm.tcgen05.cp.4x256b.b6x16_p32.cg2\0"
  "llvm.nvvm.tcgen05.cp.4x256b.cg1\0"
  "llvm.nvvm.tcgen05.cp.4x256b.cg2\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b4x16_p64.cg1\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b4x16_p64.cg2\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b6x16_p32.cg1\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b6x16_p32.cg2\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.cg1\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.cg2\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b4x16_p64.cg1\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b4x16_p64.cg2\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b6x16_p32.cg1\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b6x16_p32.cg2\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.cg1\0"
  "llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.cg2\0"
  "llvm.nvvm.tcgen05.dealloc.cg1\0"
  "llvm.nvvm.tcgen05.dealloc.cg2\0"
  "llvm.nvvm.tcgen05.fence.after.thread.sync\0"
  "llvm.nvvm.tcgen05.fence.before.thread.sync\0"
  "llvm.nvvm.tcgen05.ld.16x128b.x1\0"
  "llvm.nvvm.tcgen05.ld.16x128b.x16\0"
  "llvm.nvvm.tcgen05.ld.16x128b.x2\0"
  "llvm.nvvm.tcgen05.ld.16x128b.x32\0"
  "llvm.nvvm.tcgen05.ld.16x128b.x4\0"
  "llvm.nvvm.tcgen05.ld.16x128b.x64\0"
  "llvm.nvvm.tcgen05.ld.16x128b.x8\0"
  "llvm.nvvm.tcgen05.ld.16x256b.x1\0"
  "llvm.nvvm.tcgen05.ld.16x256b.x16\0"
  "llvm.nvvm.tcgen05.ld.16x256b.x2\0"
  "llvm.nvvm.tcgen05.ld.16x256b.x32\0"
  "llvm.nvvm.tcgen05.ld.16x256b.x4\0"
  "llvm.nvvm.tcgen05.ld.16x256b.x8\0"
  "llvm.nvvm.tcgen05.ld.16x32bx2.x1\0"
  "llvm.nvvm.tcgen05.ld.16x32bx2.x128\0"
  "llvm.nvvm.tcgen05.ld.16x32bx2.x16\0"
  "llvm.nvvm.tcgen05.ld.16x32bx2.x2\0"
  "llvm.nvvm.tcgen05.ld.16x32bx2.x32\0"
  "llvm.nvvm.tcgen05.ld.16x32bx2.x4\0"
  "llvm.nvvm.tcgen05.ld.16x32bx2.x64\0"
  "llvm.nvvm.tcgen05.ld.16x32bx2.x8\0"
  "llvm.nvvm.tcgen05.ld.16x64b.x1\0"
  "llvm.nvvm.tcgen05.ld.16x64b.x128\0"
  "llvm.nvvm.tcgen05.ld.16x64b.x16\0"
  "llvm.nvvm.tcgen05.ld.16x64b.x2\0"
  "llvm.nvvm.tcgen05.ld.16x64b.x32\0"
  "llvm.nvvm.tcgen05.ld.16x64b.x4\0"
  "llvm.nvvm.tcgen05.ld.16x64b.x64\0"
  "llvm.nvvm.tcgen05.ld.16x64b.x8\0"
  "llvm.nvvm.tcgen05.ld.32x32b.x1\0"
  "llvm.nvvm.tcgen05.ld.32x32b.x128\0"
  "llvm.nvvm.tcgen05.ld.32x32b.x16\0"
  "llvm.nvvm.tcgen05.ld.32x32b.x2\0"
  "llvm.nvvm.tcgen05.ld.32x32b.x32\0"
  "llvm.nvvm.tcgen05.ld.32x32b.x4\0"
  "llvm.nvvm.tcgen05.ld.32x32b.x64\0"
  "llvm.nvvm.tcgen05.ld.32x32b.x8\0"
  "llvm.nvvm.tcgen05.mma.shared\0"
  "llvm.nvvm.tcgen05.mma.shared.disable_output_lane.cg1\0"
  "llvm.nvvm.tcgen05.mma.shared.disable_output_lane.cg2\0"
  "llvm.nvvm.tcgen05.mma.shared.mxf4.block_scale\0"
  "llvm.nvvm.tcgen05.mma.shared.mxf4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.shared.mxf4nvf4.block_scale.block16\0"
  "llvm.nvvm.tcgen05.mma.shared.mxf4nvf4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.shared.mxf8f6f4.block_scale\0"
  "llvm.nvvm.tcgen05.mma.shared.mxf8f6f4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.shared.scale_d\0"
  "llvm.nvvm.tcgen05.mma.shared.scale_d.disable_output_lane.cg1\0"
  "llvm.nvvm.tcgen05.mma.shared.scale_d.disable_output_lane.cg2\0"
  "llvm.nvvm.tcgen05.mma.sp.shared\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.disable_output_lane.cg1\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.disable_output_lane.cg2\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.mxf4.block_scale\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.mxf4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.mxf4nvf4.block_scale.block16\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.mxf4nvf4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.mxf8f6f4.block_scale\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.mxf8f6f4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.scale_d\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.scale_d.disable_output_lane.cg1\0"
  "llvm.nvvm.tcgen05.mma.sp.shared.scale_d.disable_output_lane.cg2\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.ashift\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg1\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg1.ashift\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg2\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg2.ashift\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.mxf4.block_scale\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.mxf4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.mxf4nvf4.block_scale.block16\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.mxf4nvf4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.mxf8f6f4.block_scale\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.mxf8f6f4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.scale_d\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.ashift\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg1\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg1.ashift\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg2\0"
  "llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg2.ashift\0"
  "llvm.nvvm.tcgen05.mma.tensor\0"
  "llvm.nvvm.tcgen05.mma.tensor.ashift\0"
  "llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg1\0"
  "llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg1.ashift\0"
  "llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg2\0"
  "llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg2.ashift\0"
  "llvm.nvvm.tcgen05.mma.tensor.mxf4.block_scale\0"
  "llvm.nvvm.tcgen05.mma.tensor.mxf4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.tensor.mxf4nvf4.block_scale.block16\0"
  "llvm.nvvm.tcgen05.mma.tensor.mxf4nvf4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.tensor.mxf8f6f4.block_scale\0"
  "llvm.nvvm.tcgen05.mma.tensor.mxf8f6f4.block_scale.block32\0"
  "llvm.nvvm.tcgen05.mma.tensor.scale_d\0"
  "llvm.nvvm.tcgen05.mma.tensor.scale_d.ashift\0"
  "llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg1\0"
  "llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg1.ashift\0"
  "llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg2\0"
  "llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg2.ashift\0"
  "llvm.nvvm.tcgen05.mma.ws.shared\0"
  "llvm.nvvm.tcgen05.mma.ws.shared.zero_col_mask\0"
  "llvm.nvvm.tcgen05.mma.ws.sp.shared\0"
  "llvm.nvvm.tcgen05.mma.ws.sp.shared.zero_col_mask\0"
  "llvm.nvvm.tcgen05.mma.ws.sp.tensor\0"
  "llvm.nvvm.tcgen05.mma.ws.sp.tensor.zero_col_mask\0"
  "llvm.nvvm.tcgen05.mma.ws.tensor\0"
  "llvm.nvvm.tcgen05.mma.ws.tensor.zero_col_mask\0"
  "llvm.nvvm.tcgen05.relinq.alloc.permit.cg1\0"
  "llvm.nvvm.tcgen05.relinq.alloc.permit.cg2\0"
  "llvm.nvvm.tcgen05.shift.down.cg1\0"
  "llvm.nvvm.tcgen05.shift.down.cg2\0"
  "llvm.nvvm.tcgen05.st.16x128b.x1\0"
  "llvm.nvvm.tcgen05.st.16x128b.x16\0"
  "llvm.nvvm.tcgen05.st.16x128b.x2\0"
  "llvm.nvvm.tcgen05.st.16x128b.x32\0"
  "llvm.nvvm.tcgen05.st.16x128b.x4\0"
  "llvm.nvvm.tcgen05.st.16x128b.x64\0"
  "llvm.nvvm.tcgen05.st.16x128b.x8\0"
  "llvm.nvvm.tcgen05.st.16x256b.x1\0"
  "llvm.nvvm.tcgen05.st.16x256b.x16\0"
  "llvm.nvvm.tcgen05.st.16x256b.x2\0"
  "llvm.nvvm.tcgen05.st.16x256b.x32\0"
  "llvm.nvvm.tcgen05.st.16x256b.x4\0"
  "llvm.nvvm.tcgen05.st.16x256b.x8\0"
  "llvm.nvvm.tcgen05.st.16x32bx2.x1\0"
  "llvm.nvvm.tcgen05.st.16x32bx2.x128\0"
  "llvm.nvvm.tcgen05.st.16x32bx2.x16\0"
  "llvm.nvvm.tcgen05.st.16x32bx2.x2\0"
  "llvm.nvvm.tcgen05.st.16x32bx2.x32\0"
  "llvm.nvvm.tcgen05.st.16x32bx2.x4\0"
  "llvm.nvvm.tcgen05.st.16x32bx2.x64\0"
  "llvm.nvvm.tcgen05.st.16x32bx2.x8\0"
  "llvm.nvvm.tcgen05.st.16x64b.x1\0"
  "llvm.nvvm.tcgen05.st.16x64b.x128\0"
  "llvm.nvvm.tcgen05.st.16x64b.x16\0"
  "llvm.nvvm.tcgen05.st.16x64b.x2\0"
  "llvm.nvvm.tcgen05.st.16x64b.x32\0"
  "llvm.nvvm.tcgen05.st.16x64b.x4\0"
  "llvm.nvvm.tcgen05.st.16x64b.x64\0"
  "llvm.nvvm.tcgen05.st.16x64b.x8\0"
  "llvm.nvvm.tcgen05.st.32x32b.x1\0"
  "llvm.nvvm.tcgen05.st.32x32b.x128\0"
  "llvm.nvvm.tcgen05.st.32x32b.x16\0"
  "llvm.nvvm.tcgen05.st.32x32b.x2\0"
  "llvm.nvvm.tcgen05.st.32x32b.x32\0"
  "llvm.nvvm.tcgen05.st.32x32b.x4\0"
  "llvm.nvvm.tcgen05.st.32x32b.x64\0"
  "llvm.nvvm.tcgen05.st.32x32b.x8\0"
  "llvm.nvvm.tcgen05.wait.ld\0"
  "llvm.nvvm.tcgen05.wait.st\0"
  "llvm.nvvm.tensormap.replace.box.dim\0"
  "llvm.nvvm.tensormap.replace.element.stride\0"
  "llvm.nvvm.tensormap.replace.elemtype\0"
  "llvm.nvvm.tensormap.replace.fill.mode\0"
  "llvm.nvvm.tensormap.replace.global.address\0"
  "llvm.nvvm.tensormap.replace.global.dim\0"
  "llvm.nvvm.tensormap.replace.global.stride\0"
  "llvm.nvvm.tensormap.replace.interleave.layout\0"
  "llvm.nvvm.tensormap.replace.rank\0"
  "llvm.nvvm.tensormap.replace.swizzle.atomicity\0"
  "llvm.nvvm.tensormap.replace.swizzle.mode\0"
  "llvm.nvvm.tex.1d.array.grad.v4f32.f32\0"
  "llvm.nvvm.tex.1d.array.grad.v4s32.f32\0"
  "llvm.nvvm.tex.1d.array.grad.v4u32.f32\0"
  "llvm.nvvm.tex.1d.array.level.v4f32.f32\0"
  "llvm.nvvm.tex.1d.array.level.v4s32.f32\0"
  "llvm.nvvm.tex.1d.array.level.v4u32.f32\0"
  "llvm.nvvm.tex.1d.array.v4f32.f32\0"
  "llvm.nvvm.tex.1d.array.v4f32.s32\0"
  "llvm.nvvm.tex.1d.array.v4s32.f32\0"
  "llvm.nvvm.tex.1d.array.v4s32.s32\0"
  "llvm.nvvm.tex.1d.array.v4u32.f32\0"
  "llvm.nvvm.tex.1d.array.v4u32.s32\0"
  "llvm.nvvm.tex.1d.grad.v4f32.f32\0"
  "llvm.nvvm.tex.1d.grad.v4s32.f32\0"
  "llvm.nvvm.tex.1d.grad.v4u32.f32\0"
  "llvm.nvvm.tex.1d.level.v4f32.f32\0"
  "llvm.nvvm.tex.1d.level.v4s32.f32\0"
  "llvm.nvvm.tex.1d.level.v4u32.f32\0"
  "llvm.nvvm.tex.1d.v4f32.f32\0"
  "llvm.nvvm.tex.1d.v4f32.s32\0"
  "llvm.nvvm.tex.1d.v4s32.f32\0"
  "llvm.nvvm.tex.1d.v4s32.s32\0"
  "llvm.nvvm.tex.1d.v4u32.f32\0"
  "llvm.nvvm.tex.1d.v4u32.s32\0"
  "llvm.nvvm.tex.2d.array.grad.v4f32.f32\0"
  "llvm.nvvm.tex.2d.array.grad.v4s32.f32\0"
  "llvm.nvvm.tex.2d.array.grad.v4u32.f32\0"
  "llvm.nvvm.tex.2d.array.level.v4f32.f32\0"
  "llvm.nvvm.tex.2d.array.level.v4s32.f32\0"
  "llvm.nvvm.tex.2d.array.level.v4u32.f32\0"
  "llvm.nvvm.tex.2d.array.v4f32.f32\0"
  "llvm.nvvm.tex.2d.array.v4f32.s32\0"
  "llvm.nvvm.tex.2d.array.v4s32.f32\0"
  "llvm.nvvm.tex.2d.array.v4s32.s32\0"
  "llvm.nvvm.tex.2d.array.v4u32.f32\0"
  "llvm.nvvm.tex.2d.array.v4u32.s32\0"
  "llvm.nvvm.tex.2d.grad.v4f32.f32\0"
  "llvm.nvvm.tex.2d.grad.v4s32.f32\0"
  "llvm.nvvm.tex.2d.grad.v4u32.f32\0"
  "llvm.nvvm.tex.2d.level.v4f32.f32\0"
  "llvm.nvvm.tex.2d.level.v4s32.f32\0"
  "llvm.nvvm.tex.2d.level.v4u32.f32\0"
  "llvm.nvvm.tex.2d.v4f32.f32\0"
  "llvm.nvvm.tex.2d.v4f32.s32\0"
  "llvm.nvvm.tex.2d.v4s32.f32\0"
  "llvm.nvvm.tex.2d.v4s32.s32\0"
  "llvm.nvvm.tex.2d.v4u32.f32\0"
  "llvm.nvvm.tex.2d.v4u32.s32\0"
  "llvm.nvvm.tex.3d.grad.v4f32.f32\0"
  "llvm.nvvm.tex.3d.grad.v4s32.f32\0"
  "llvm.nvvm.tex.3d.grad.v4u32.f32\0"
  "llvm.nvvm.tex.3d.level.v4f32.f32\0"
  "llvm.nvvm.tex.3d.level.v4s32.f32\0"
  "llvm.nvvm.tex.3d.level.v4u32.f32\0"
  "llvm.nvvm.tex.3d.v4f32.f32\0"
  "llvm.nvvm.tex.3d.v4f32.s32\0"
  "llvm.nvvm.tex.3d.v4s32.f32\0"
  "llvm.nvvm.tex.3d.v4s32.s32\0"
  "llvm.nvvm.tex.3d.v4u32.f32\0"
  "llvm.nvvm.tex.3d.v4u32.s32\0"
  "llvm.nvvm.tex.cube.array.level.v4f32.f32\0"
  "llvm.nvvm.tex.cube.array.level.v4s32.f32\0"
  "llvm.nvvm.tex.cube.array.level.v4u32.f32\0"
  "llvm.nvvm.tex.cube.array.v4f32.f32\0"
  "llvm.nvvm.tex.cube.array.v4s32.f32\0"
  "llvm.nvvm.tex.cube.array.v4u32.f32\0"
  "llvm.nvvm.tex.cube.level.v4f32.f32\0"
  "llvm.nvvm.tex.cube.level.v4s32.f32\0"
  "llvm.nvvm.tex.cube.level.v4u32.f32\0"
  "llvm.nvvm.tex.cube.v4f32.f32\0"
  "llvm.nvvm.tex.cube.v4s32.f32\0"
  "llvm.nvvm.tex.cube.v4u32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.grad.v4f32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.grad.v4s32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.grad.v4u32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.level.v4f32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.level.v4s32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.level.v4u32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.v4f32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.v4f32.s32\0"
  "llvm.nvvm.tex.unified.1d.array.v4s32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.v4s32.s32\0"
  "llvm.nvvm.tex.unified.1d.array.v4u32.f32\0"
  "llvm.nvvm.tex.unified.1d.array.v4u32.s32\0"
  "llvm.nvvm.tex.unified.1d.grad.v4f32.f32\0"
  "llvm.nvvm.tex.unified.1d.grad.v4s32.f32\0"
  "llvm.nvvm.tex.unified.1d.grad.v4u32.f32\0"
  "llvm.nvvm.tex.unified.1d.level.v4f32.f32\0"
  "llvm.nvvm.tex.unified.1d.level.v4s32.f32\0"
  "llvm.nvvm.tex.unified.1d.level.v4u32.f32\0"
  "llvm.nvvm.tex.unified.1d.v4f32.f32\0"
  "llvm.nvvm.tex.unified.1d.v4f32.s32\0"
  "llvm.nvvm.tex.unified.1d.v4s32.f32\0"
  "llvm.nvvm.tex.unified.1d.v4s32.s32\0"
  "llvm.nvvm.tex.unified.1d.v4u32.f32\0"
  "llvm.nvvm.tex.unified.1d.v4u32.s32\0"
  "llvm.nvvm.tex.unified.2d.array.grad.v4f32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.grad.v4s32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.grad.v4u32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.level.v4f32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.level.v4s32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.level.v4u32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.v4f32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.v4f32.s32\0"
  "llvm.nvvm.tex.unified.2d.array.v4s32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.v4s32.s32\0"
  "llvm.nvvm.tex.unified.2d.array.v4u32.f32\0"
  "llvm.nvvm.tex.unified.2d.array.v4u32.s32\0"
  "llvm.nvvm.tex.unified.2d.grad.v4f32.f32\0"
  "llvm.nvvm.tex.unified.2d.grad.v4s32.f32\0"
  "llvm.nvvm.tex.unified.2d.grad.v4u32.f32\0"
  "llvm.nvvm.tex.unified.2d.level.v4f32.f32\0"
  "llvm.nvvm.tex.unified.2d.level.v4s32.f32\0"
  "llvm.nvvm.tex.unified.2d.level.v4u32.f32\0"
  "llvm.nvvm.tex.unified.2d.v4f32.f32\0"
  "llvm.nvvm.tex.unified.2d.v4f32.s32\0"
  "llvm.nvvm.tex.unified.2d.v4s32.f32\0"
  "llvm.nvvm.tex.unified.2d.v4s32.s32\0"
  "llvm.nvvm.tex.unified.2d.v4u32.f32\0"
  "llvm.nvvm.tex.unified.2d.v4u32.s32\0"
  "llvm.nvvm.tex.unified.3d.grad.v4f32.f32\0"
  "llvm.nvvm.tex.unified.3d.grad.v4s32.f32\0"
  "llvm.nvvm.tex.unified.3d.grad.v4u32.f32\0"
  "llvm.nvvm.tex.unified.3d.level.v4f32.f32\0"
  "llvm.nvvm.tex.unified.3d.level.v4s32.f32\0"
  "llvm.nvvm.tex.unified.3d.level.v4u32.f32\0"
  "llvm.nvvm.tex.unified.3d.v4f32.f32\0"
  "llvm.nvvm.tex.unified.3d.v4f32.s32\0"
  "llvm.nvvm.tex.unified.3d.v4s32.f32\0"
  "llvm.nvvm.tex.unified.3d.v4s32.s32\0"
  "llvm.nvvm.tex.unified.3d.v4u32.f32\0"
  "llvm.nvvm.tex.unified.3d.v4u32.s32\0"
  "llvm.nvvm.tex.unified.cube.array.grad.v4f32.f32\0"
  "llvm.nvvm.tex.unified.cube.array.grad.v4s32.f32\0"
  "llvm.nvvm.tex.unified.cube.array.grad.v4u32.f32\0"
  "llvm.nvvm.tex.unified.cube.array.level.v4f32.f32\0"
  "llvm.nvvm.tex.unified.cube.array.level.v4s32.f32\0"
  "llvm.nvvm.tex.unified.cube.array.level.v4u32.f32\0"
  "llvm.nvvm.tex.unified.cube.array.v4f32.f32\0"
  "llvm.nvvm.tex.unified.cube.array.v4s32.f32\0"
  "llvm.nvvm.tex.unified.cube.array.v4u32.f32\0"
  "llvm.nvvm.tex.unified.cube.grad.v4f32.f32\0"
  "llvm.nvvm.tex.unified.cube.grad.v4s32.f32\0"
  "llvm.nvvm.tex.unified.cube.grad.v4u32.f32\0"
  "llvm.nvvm.tex.unified.cube.level.v4f32.f32\0"
  "llvm.nvvm.tex.unified.cube.level.v4s32.f32\0"
  "llvm.nvvm.tex.unified.cube.level.v4u32.f32\0"
  "llvm.nvvm.tex.unified.cube.v4f32.f32\0"
  "llvm.nvvm.tex.unified.cube.v4s32.f32\0"
  "llvm.nvvm.tex.unified.cube.v4u32.f32\0"
  "llvm.nvvm.texsurf.handle\0"
  "llvm.nvvm.texsurf.handle.internal\0"
  "llvm.nvvm.tld4.a.2d.v4f32.f32\0"
  "llvm.nvvm.tld4.a.2d.v4s32.f32\0"
  "llvm.nvvm.tld4.a.2d.v4u32.f32\0"
  "llvm.nvvm.tld4.b.2d.v4f32.f32\0"
  "llvm.nvvm.tld4.b.2d.v4s32.f32\0"
  "llvm.nvvm.tld4.b.2d.v4u32.f32\0"
  "llvm.nvvm.tld4.g.2d.v4f32.f32\0"
  "llvm.nvvm.tld4.g.2d.v4s32.f32\0"
  "llvm.nvvm.tld4.g.2d.v4u32.f32\0"
  "llvm.nvvm.tld4.r.2d.v4f32.f32\0"
  "llvm.nvvm.tld4.r.2d.v4s32.f32\0"
  "llvm.nvvm.tld4.r.2d.v4u32.f32\0"
  "llvm.nvvm.tld4.unified.a.2d.v4f32.f32\0"
  "llvm.nvvm.tld4.unified.a.2d.v4s32.f32\0"
  "llvm.nvvm.tld4.unified.a.2d.v4u32.f32\0"
  "llvm.nvvm.tld4.unified.b.2d.v4f32.f32\0"
  "llvm.nvvm.tld4.unified.b.2d.v4s32.f32\0"
  "llvm.nvvm.tld4.unified.b.2d.v4u32.f32\0"
  "llvm.nvvm.tld4.unified.g.2d.v4f32.f32\0"
  "llvm.nvvm.tld4.unified.g.2d.v4s32.f32\0"
  "llvm.nvvm.tld4.unified.g.2d.v4u32.f32\0"
  "llvm.nvvm.tld4.unified.r.2d.v4f32.f32\0"
  "llvm.nvvm.tld4.unified.r.2d.v4s32.f32\0"
  "llvm.nvvm.tld4.unified.r.2d.v4u32.f32\0"
  "llvm.nvvm.trunc.d\0"
  "llvm.nvvm.trunc.f\0"
  "llvm.nvvm.trunc.ftz.f\0"
  "llvm.nvvm.txq.array.size\0"
  "llvm.nvvm.txq.channel.data.type\0"
  "llvm.nvvm.txq.channel.order\0"
  "llvm.nvvm.txq.depth\0"
  "llvm.nvvm.txq.height\0"
  "llvm.nvvm.txq.num.mipmap.levels\0"
  "llvm.nvvm.txq.num.samples\0"
  "llvm.nvvm.txq.width\0"
  "llvm.nvvm.ue8m0x2.to.bf16x2\0"
  "llvm.nvvm.ui2d.rm\0"
  "llvm.nvvm.ui2d.rn\0"
  "llvm.nvvm.ui2d.rp\0"
  "llvm.nvvm.ui2d.rz\0"
  "llvm.nvvm.ui2f.rm\0"
  "llvm.nvvm.ui2f.rn\0"
  "llvm.nvvm.ui2f.rp\0"
  "llvm.nvvm.ui2f.rz\0"
  "llvm.nvvm.ull2d.rm\0"
  "llvm.nvvm.ull2d.rn\0"
  "llvm.nvvm.ull2d.rp\0"
  "llvm.nvvm.ull2d.rz\0"
  "llvm.nvvm.ull2f.rm\0"
  "llvm.nvvm.ull2f.rn\0"
  "llvm.nvvm.ull2f.rp\0"
  "llvm.nvvm.ull2f.rz\0"
  "llvm.nvvm.vote.all\0"
  "llvm.nvvm.vote.all.sync\0"
  "llvm.nvvm.vote.any\0"
  "llvm.nvvm.vote.any.sync\0"
  "llvm.nvvm.vote.ballot\0"
  "llvm.nvvm.vote.ballot.sync\0"
  "llvm.nvvm.vote.uni\0"
  "llvm.nvvm.vote.uni.sync\0"
  "llvm.nvvm.wgmma.commit_group.sync.aligned\0"
  "llvm.nvvm.wgmma.fence.sync.aligned\0"
  "llvm.nvvm.wgmma.wait_group.sync.aligned\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.col.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.col.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.col.s8\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.col.stride.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.col.stride.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.col.stride.s8\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.col.stride.u8\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.col.u8\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.row.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.row.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.row.s8\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.row.stride.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.row.stride.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.row.stride.s8\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.row.stride.u8\0"
  "llvm.nvvm.wmma.m16n16k16.load.a.row.u8\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.col.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.col.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.col.s8\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.col.stride.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.col.stride.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.col.stride.s8\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.col.stride.u8\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.col.u8\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.row.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.row.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.row.s8\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.row.stride.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.row.stride.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.row.stride.s8\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.row.stride.u8\0"
  "llvm.nvvm.wmma.m16n16k16.load.b.row.u8\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.col.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.col.f32\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.col.s32\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.col.stride.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.col.stride.f32\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.col.stride.s32\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.row.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.row.f32\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.row.s32\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.row.stride.f16\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.row.stride.f32\0"
  "llvm.nvvm.wmma.m16n16k16.load.c.row.stride.s32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.s8\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.s8.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.u8\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.col.u8.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.s8\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.s8.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.u8\0"
  "llvm.nvvm.wmma.m16n16k16.mma.col.row.u8.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.s8\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.s8.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.u8\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.col.u8.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.bf16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f16\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f32\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.s8\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.s8.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.u8\0"
  "llvm.nvvm.wmma.m16n16k16.mma.row.row.u8.satfinite\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.col.f16\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.col.f32\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.col.s32\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.col.stride.f16\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.col.stride.f32\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.col.stride.s32\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.row.f16\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.row.f32\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.row.s32\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.row.stride.f16\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.row.stride.f32\0"
  "llvm.nvvm.wmma.m16n16k16.store.d.row.stride.s32\0"
  "llvm.nvvm.wmma.m16n16k8.load.a.col.stride.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.load.a.col.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.load.a.row.stride.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.load.a.row.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.load.b.col.stride.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.load.b.col.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.load.b.row.stride.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.load.b.row.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.load.c.col.f32\0"
  "llvm.nvvm.wmma.m16n16k8.load.c.col.stride.f32\0"
  "llvm.nvvm.wmma.m16n16k8.load.c.row.f32\0"
  "llvm.nvvm.wmma.m16n16k8.load.c.row.stride.f32\0"
  "llvm.nvvm.wmma.m16n16k8.mma.col.col.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.mma.col.row.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.mma.row.col.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.mma.row.row.tf32\0"
  "llvm.nvvm.wmma.m16n16k8.store.d.col.f32\0"
  "llvm.nvvm.wmma.m16n16k8.store.d.col.stride.f32\0"
  "llvm.nvvm.wmma.m16n16k8.store.d.row.f32\0"
  "llvm.nvvm.wmma.m16n16k8.store.d.row.stride.f32\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.col.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.col.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.col.s8\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.col.stride.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.col.stride.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.col.stride.s8\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.col.stride.u8\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.col.u8\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.row.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.row.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.row.s8\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.row.stride.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.row.stride.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.row.stride.s8\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.row.stride.u8\0"
  "llvm.nvvm.wmma.m32n8k16.load.a.row.u8\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.col.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.col.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.col.s8\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.col.stride.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.col.stride.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.col.stride.s8\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.col.stride.u8\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.col.u8\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.row.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.row.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.row.s8\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.row.stride.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.row.stride.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.row.stride.s8\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.row.stride.u8\0"
  "llvm.nvvm.wmma.m32n8k16.load.b.row.u8\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.col.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.col.f32\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.col.s32\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.col.stride.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.col.stride.f32\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.col.stride.s32\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.row.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.row.f32\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.row.s32\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.row.stride.f16\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.row.stride.f32\0"
  "llvm.nvvm.wmma.m32n8k16.load.c.row.stride.s32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.s8\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.s8.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.u8\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.col.u8.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.s8\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.s8.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.u8\0"
  "llvm.nvvm.wmma.m32n8k16.mma.col.row.u8.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.s8\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.s8.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.u8\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.col.u8.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.bf16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f16\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f32\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.s8\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.s8.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.u8\0"
  "llvm.nvvm.wmma.m32n8k16.mma.row.row.u8.satfinite\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.col.f16\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.col.f32\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.col.s32\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.col.stride.f16\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.col.stride.f32\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.col.stride.s32\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.row.f16\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.row.f32\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.row.s32\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.row.stride.f16\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.row.stride.f32\0"
  "llvm.nvvm.wmma.m32n8k16.store.d.row.stride.s32\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.col.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.col.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.col.s8\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.col.stride.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.col.stride.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.col.stride.s8\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.col.stride.u8\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.col.u8\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.row.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.row.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.row.s8\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.row.stride.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.row.stride.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.row.stride.s8\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.row.stride.u8\0"
  "llvm.nvvm.wmma.m8n32k16.load.a.row.u8\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.col.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.col.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.col.s8\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.col.stride.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.col.stride.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.col.stride.s8\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.col.stride.u8\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.col.u8\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.row.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.row.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.row.s8\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.row.stride.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.row.stride.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.row.stride.s8\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.row.stride.u8\0"
  "llvm.nvvm.wmma.m8n32k16.load.b.row.u8\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.col.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.col.f32\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.col.s32\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.col.stride.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.col.stride.f32\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.col.stride.s32\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.row.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.row.f32\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.row.s32\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.row.stride.f16\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.row.stride.f32\0"
  "llvm.nvvm.wmma.m8n32k16.load.c.row.stride.s32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.s8\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.s8.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.u8\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.col.u8.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.s8\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.s8.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.u8\0"
  "llvm.nvvm.wmma.m8n32k16.mma.col.row.u8.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.s8\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.s8.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.u8\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.col.u8.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.bf16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f16.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f32.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f16\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f16.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f32\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f32.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.s8\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.s8.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.u8\0"
  "llvm.nvvm.wmma.m8n32k16.mma.row.row.u8.satfinite\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.col.f16\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.col.f32\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.col.s32\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.col.stride.f16\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.col.stride.f32\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.col.stride.s32\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.row.f16\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.row.f32\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.row.s32\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.row.stride.f16\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.row.stride.f32\0"
  "llvm.nvvm.wmma.m8n32k16.store.d.row.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k128.load.a.row.b1\0"
  "llvm.nvvm.wmma.m8n8k128.load.a.row.stride.b1\0"
  "llvm.nvvm.wmma.m8n8k128.load.b.col.b1\0"
  "llvm.nvvm.wmma.m8n8k128.load.b.col.stride.b1\0"
  "llvm.nvvm.wmma.m8n8k128.load.c.col.s32\0"
  "llvm.nvvm.wmma.m8n8k128.load.c.col.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k128.load.c.row.s32\0"
  "llvm.nvvm.wmma.m8n8k128.load.c.row.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k128.mma.and.popc.row.col.b1\0"
  "llvm.nvvm.wmma.m8n8k128.mma.xor.popc.row.col.b1\0"
  "llvm.nvvm.wmma.m8n8k128.store.d.col.s32\0"
  "llvm.nvvm.wmma.m8n8k128.store.d.col.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k128.store.d.row.s32\0"
  "llvm.nvvm.wmma.m8n8k128.store.d.row.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k32.load.a.row.s4\0"
  "llvm.nvvm.wmma.m8n8k32.load.a.row.stride.s4\0"
  "llvm.nvvm.wmma.m8n8k32.load.a.row.stride.u4\0"
  "llvm.nvvm.wmma.m8n8k32.load.a.row.u4\0"
  "llvm.nvvm.wmma.m8n8k32.load.b.col.s4\0"
  "llvm.nvvm.wmma.m8n8k32.load.b.col.stride.s4\0"
  "llvm.nvvm.wmma.m8n8k32.load.b.col.stride.u4\0"
  "llvm.nvvm.wmma.m8n8k32.load.b.col.u4\0"
  "llvm.nvvm.wmma.m8n8k32.load.c.col.s32\0"
  "llvm.nvvm.wmma.m8n8k32.load.c.col.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k32.load.c.row.s32\0"
  "llvm.nvvm.wmma.m8n8k32.load.c.row.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k32.mma.row.col.s4\0"
  "llvm.nvvm.wmma.m8n8k32.mma.row.col.s4.satfinite\0"
  "llvm.nvvm.wmma.m8n8k32.mma.row.col.u4\0"
  "llvm.nvvm.wmma.m8n8k32.mma.row.col.u4.satfinite\0"
  "llvm.nvvm.wmma.m8n8k32.store.d.col.s32\0"
  "llvm.nvvm.wmma.m8n8k32.store.d.col.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k32.store.d.row.s32\0"
  "llvm.nvvm.wmma.m8n8k32.store.d.row.stride.s32\0"
  "llvm.nvvm.wmma.m8n8k4.load.a.col.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.a.col.stride.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.a.row.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.a.row.stride.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.b.col.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.b.col.stride.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.b.row.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.b.row.stride.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.c.col.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.c.col.stride.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.c.row.f64\0"
  "llvm.nvvm.wmma.m8n8k4.load.c.row.stride.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.col.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.col.rm.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.col.rn.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.col.rp.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.col.rz.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.row.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.row.rm.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.row.rn.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.row.rp.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.col.row.rz.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.col.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.col.rm.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.col.rn.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.col.rp.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.col.rz.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.row.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.row.rm.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.row.rn.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.row.rp.f64\0"
  "llvm.nvvm.wmma.m8n8k4.mma.row.row.rz.f64\0"
  "llvm.nvvm.wmma.m8n8k4.store.d.col.f64\0"
  "llvm.nvvm.wmma.m8n8k4.store.d.col.stride.f64\0"
  "llvm.nvvm.wmma.m8n8k4.store.d.row.f64\0"
  "llvm.nvvm.wmma.m8n8k4.store.d.row.stride.f64\0"
  "llvm.nvvm.zext.clamp\0"
  "llvm.nvvm.zext.wrap\0"
  "llvm.ppc.addex\0"
  "llvm.ppc.addf128.round.to.odd\0"
  "llvm.ppc.addg6s\0"
  "llvm.ppc.addg6sd\0"
  "llvm.ppc.altivec.crypto.vcipher\0"
  "llvm.ppc.altivec.crypto.vcipherlast\0"
  "llvm.ppc.altivec.crypto.vncipher\0"
  "llvm.ppc.altivec.crypto.vncipherlast\0"
  "llvm.ppc.altivec.crypto.vpermxor\0"
  "llvm.ppc.altivec.crypto.vpermxor.be\0"
  "llvm.ppc.altivec.crypto.vpmsumb\0"
  "llvm.ppc.altivec.crypto.vpmsumd\0"
  "llvm.ppc.altivec.crypto.vpmsumh\0"
  "llvm.ppc.altivec.crypto.vpmsumw\0"
  "llvm.ppc.altivec.crypto.vsbox\0"
  "llvm.ppc.altivec.crypto.vshasigmad\0"
  "llvm.ppc.altivec.crypto.vshasigmaw\0"
  "llvm.ppc.altivec.dss\0"
  "llvm.ppc.altivec.dssall\0"
  "llvm.ppc.altivec.dst\0"
  "llvm.ppc.altivec.dstst\0"
  "llvm.ppc.altivec.dststt\0"
  "llvm.ppc.altivec.dstt\0"
  "llvm.ppc.altivec.lvebx\0"
  "llvm.ppc.altivec.lvehx\0"
  "llvm.ppc.altivec.lvewx\0"
  "llvm.ppc.altivec.lvsl\0"
  "llvm.ppc.altivec.lvsr\0"
  "llvm.ppc.altivec.lvx\0"
  "llvm.ppc.altivec.lvxl\0"
  "llvm.ppc.altivec.mfvscr\0"
  "llvm.ppc.altivec.mtvscr\0"
  "llvm.ppc.altivec.mtvsrbm\0"
  "llvm.ppc.altivec.mtvsrdm\0"
  "llvm.ppc.altivec.mtvsrhm\0"
  "llvm.ppc.altivec.mtvsrqm\0"
  "llvm.ppc.altivec.mtvsrwm\0"
  "llvm.ppc.altivec.stvebx\0"
  "llvm.ppc.altivec.stvehx\0"
  "llvm.ppc.altivec.stvewx\0"
  "llvm.ppc.altivec.stvx\0"
  "llvm.ppc.altivec.stvxl\0"
  "llvm.ppc.altivec.vabsdub\0"
  "llvm.ppc.altivec.vabsduh\0"
  "llvm.ppc.altivec.vabsduw\0"
  "llvm.ppc.altivec.vaddcuq\0"
  "llvm.ppc.altivec.vaddcuw\0"
  "llvm.ppc.altivec.vaddecuq\0"
  "llvm.ppc.altivec.vaddeuqm\0"
  "llvm.ppc.altivec.vaddsbs\0"
  "llvm.ppc.altivec.vaddshs\0"
  "llvm.ppc.altivec.vaddsws\0"
  "llvm.ppc.altivec.vaddubs\0"
  "llvm.ppc.altivec.vadduhs\0"
  "llvm.ppc.altivec.vadduws\0"
  "llvm.ppc.altivec.vavgsb\0"
  "llvm.ppc.altivec.vavgsh\0"
  "llvm.ppc.altivec.vavgsw\0"
  "llvm.ppc.altivec.vavgub\0"
  "llvm.ppc.altivec.vavguh\0"
  "llvm.ppc.altivec.vavguw\0"
  "llvm.ppc.altivec.vbpermd\0"
  "llvm.ppc.altivec.vbpermq\0"
  "llvm.ppc.altivec.vcfsx\0"
  "llvm.ppc.altivec.vcfuged\0"
  "llvm.ppc.altivec.vcfux\0"
  "llvm.ppc.altivec.vclrlb\0"
  "llvm.ppc.altivec.vclrrb\0"
  "llvm.ppc.altivec.vclzdm\0"
  "llvm.ppc.altivec.vclzlsbb\0"
  "llvm.ppc.altivec.vcmpbfp\0"
  "llvm.ppc.altivec.vcmpbfp.p\0"
  "llvm.ppc.altivec.vcmpeqfp\0"
  "llvm.ppc.altivec.vcmpeqfp.p\0"
  "llvm.ppc.altivec.vcmpequb\0"
  "llvm.ppc.altivec.vcmpequb.p\0"
  "llvm.ppc.altivec.vcmpequd\0"
  "llvm.ppc.altivec.vcmpequd.p\0"
  "llvm.ppc.altivec.vcmpequh\0"
  "llvm.ppc.altivec.vcmpequh.p\0"
  "llvm.ppc.altivec.vcmpequq\0"
  "llvm.ppc.altivec.vcmpequq.p\0"
  "llvm.ppc.altivec.vcmpequw\0"
  "llvm.ppc.altivec.vcmpequw.p\0"
  "llvm.ppc.altivec.vcmpgefp\0"
  "llvm.ppc.altivec.vcmpgefp.p\0"
  "llvm.ppc.altivec.vcmpgtfp\0"
  "llvm.ppc.altivec.vcmpgtfp.p\0"
  "llvm.ppc.altivec.vcmpgtsb\0"
  "llvm.ppc.altivec.vcmpgtsb.p\0"
  "llvm.ppc.altivec.vcmpgtsd\0"
  "llvm.ppc.altivec.vcmpgtsd.p\0"
  "llvm.ppc.altivec.vcmpgtsh\0"
  "llvm.ppc.altivec.vcmpgtsh.p\0"
  "llvm.ppc.altivec.vcmpgtsq\0"
  "llvm.ppc.altivec.vcmpgtsq.p\0"
  "llvm.ppc.altivec.vcmpgtsw\0"
  "llvm.ppc.altivec.vcmpgtsw.p\0"
  "llvm.ppc.altivec.vcmpgtub\0"
  "llvm.ppc.altivec.vcmpgtub.p\0"
  "llvm.ppc.altivec.vcmpgtud\0"
  "llvm.ppc.altivec.vcmpgtud.p\0"
  "llvm.ppc.altivec.vcmpgtuh\0"
  "llvm.ppc.altivec.vcmpgtuh.p\0"
  "llvm.ppc.altivec.vcmpgtuq\0"
  "llvm.ppc.altivec.vcmpgtuq.p\0"
  "llvm.ppc.altivec.vcmpgtuw\0"
  "llvm.ppc.altivec.vcmpgtuw.p\0"
  "llvm.ppc.altivec.vcmpneb\0"
  "llvm.ppc.altivec.vcmpneb.p\0"
  "llvm.ppc.altivec.vcmpneh\0"
  "llvm.ppc.altivec.vcmpneh.p\0"
  "llvm.ppc.altivec.vcmpnew\0"
  "llvm.ppc.altivec.vcmpnew.p\0"
  "llvm.ppc.altivec.vcmpnezb\0"
  "llvm.ppc.altivec.vcmpnezb.p\0"
  "llvm.ppc.altivec.vcmpnezh\0"
  "llvm.ppc.altivec.vcmpnezh.p\0"
  "llvm.ppc.altivec.vcmpnezw\0"
  "llvm.ppc.altivec.vcmpnezw.p\0"
  "llvm.ppc.altivec.vcntmbb\0"
  "llvm.ppc.altivec.vcntmbd\0"
  "llvm.ppc.altivec.vcntmbh\0"
  "llvm.ppc.altivec.vcntmbw\0"
  "llvm.ppc.altivec.vctsxs\0"
  "llvm.ppc.altivec.vctuxs\0"
  "llvm.ppc.altivec.vctzdm\0"
  "llvm.ppc.altivec.vctzlsbb\0"
  "llvm.ppc.altivec.vdivesd\0"
  "llvm.ppc.altivec.vdivesq\0"
  "llvm.ppc.altivec.vdivesw\0"
  "llvm.ppc.altivec.vdiveud\0"
  "llvm.ppc.altivec.vdiveuq\0"
  "llvm.ppc.altivec.vdiveuw\0"
  "llvm.ppc.altivec.vexpandbm\0"
  "llvm.ppc.altivec.vexpanddm\0"
  "llvm.ppc.altivec.vexpandhm\0"
  "llvm.ppc.altivec.vexpandqm\0"
  "llvm.ppc.altivec.vexpandwm\0"
  "llvm.ppc.altivec.vexptefp\0"
  "llvm.ppc.altivec.vextddvlx\0"
  "llvm.ppc.altivec.vextddvrx\0"
  "llvm.ppc.altivec.vextdubvlx\0"
  "llvm.ppc.altivec.vextdubvrx\0"
  "llvm.ppc.altivec.vextduhvlx\0"
  "llvm.ppc.altivec.vextduhvrx\0"
  "llvm.ppc.altivec.vextduwvlx\0"
  "llvm.ppc.altivec.vextduwvrx\0"
  "llvm.ppc.altivec.vextractbm\0"
  "llvm.ppc.altivec.vextractdm\0"
  "llvm.ppc.altivec.vextracthm\0"
  "llvm.ppc.altivec.vextractqm\0"
  "llvm.ppc.altivec.vextractwm\0"
  "llvm.ppc.altivec.vextsb2d\0"
  "llvm.ppc.altivec.vextsb2w\0"
  "llvm.ppc.altivec.vextsd2q\0"
  "llvm.ppc.altivec.vextsh2d\0"
  "llvm.ppc.altivec.vextsh2w\0"
  "llvm.ppc.altivec.vextsw2d\0"
  "llvm.ppc.altivec.vgbbd\0"
  "llvm.ppc.altivec.vgnb\0"
  "llvm.ppc.altivec.vinsblx\0"
  "llvm.ppc.altivec.vinsbrx\0"
  "llvm.ppc.altivec.vinsbvlx\0"
  "llvm.ppc.altivec.vinsbvrx\0"
  "llvm.ppc.altivec.vinsd\0"
  "llvm.ppc.altivec.vinsdlx\0"
  "llvm.ppc.altivec.vinsdrx\0"
  "llvm.ppc.altivec.vinshlx\0"
  "llvm.ppc.altivec.vinshrx\0"
  "llvm.ppc.altivec.vinshvlx\0"
  "llvm.ppc.altivec.vinshvrx\0"
  "llvm.ppc.altivec.vinsw\0"
  "llvm.ppc.altivec.vinswlx\0"
  "llvm.ppc.altivec.vinswrx\0"
  "llvm.ppc.altivec.vinswvlx\0"
  "llvm.ppc.altivec.vinswvrx\0"
  "llvm.ppc.altivec.vlogefp\0"
  "llvm.ppc.altivec.vmaddfp\0"
  "llvm.ppc.altivec.vmaxfp\0"
  "llvm.ppc.altivec.vmaxsb\0"
  "llvm.ppc.altivec.vmaxsd\0"
  "llvm.ppc.altivec.vmaxsh\0"
  "llvm.ppc.altivec.vmaxsw\0"
  "llvm.ppc.altivec.vmaxub\0"
  "llvm.ppc.altivec.vmaxud\0"
  "llvm.ppc.altivec.vmaxuh\0"
  "llvm.ppc.altivec.vmaxuw\0"
  "llvm.ppc.altivec.vmhaddshs\0"
  "llvm.ppc.altivec.vmhraddshs\0"
  "llvm.ppc.altivec.vminfp\0"
  "llvm.ppc.altivec.vminsb\0"
  "llvm.ppc.altivec.vminsd\0"
  "llvm.ppc.altivec.vminsh\0"
  "llvm.ppc.altivec.vminsw\0"
  "llvm.ppc.altivec.vminub\0"
  "llvm.ppc.altivec.vminud\0"
  "llvm.ppc.altivec.vminuh\0"
  "llvm.ppc.altivec.vminuw\0"
  "llvm.ppc.altivec.vmladduhm\0"
  "llvm.ppc.altivec.vmsumcud\0"
  "llvm.ppc.altivec.vmsummbm\0"
  "llvm.ppc.altivec.vmsumshm\0"
  "llvm.ppc.altivec.vmsumshs\0"
  "llvm.ppc.altivec.vmsumubm\0"
  "llvm.ppc.altivec.vmsumudm\0"
  "llvm.ppc.altivec.vmsumuhm\0"
  "llvm.ppc.altivec.vmsumuhs\0"
  "llvm.ppc.altivec.vmulesb\0"
  "llvm.ppc.altivec.vmulesd\0"
  "llvm.ppc.altivec.vmulesh\0"
  "llvm.ppc.altivec.vmulesw\0"
  "llvm.ppc.altivec.vmuleub\0"
  "llvm.ppc.altivec.vmuleud\0"
  "llvm.ppc.altivec.vmuleuh\0"
  "llvm.ppc.altivec.vmuleuw\0"
  "llvm.ppc.altivec.vmulhsd\0"
  "llvm.ppc.altivec.vmulhsw\0"
  "llvm.ppc.altivec.vmulhud\0"
  "llvm.ppc.altivec.vmulhuw\0"
  "llvm.ppc.altivec.vmulosb\0"
  "llvm.ppc.altivec.vmulosd\0"
  "llvm.ppc.altivec.vmulosh\0"
  "llvm.ppc.altivec.vmulosw\0"
  "llvm.ppc.altivec.vmuloub\0"
  "llvm.ppc.altivec.vmuloud\0"
  "llvm.ppc.altivec.vmulouh\0"
  "llvm.ppc.altivec.vmulouw\0"
  "llvm.ppc.altivec.vnmsubfp\0"
  "llvm.ppc.altivec.vpdepd\0"
  "llvm.ppc.altivec.vperm\0"
  "llvm.ppc.altivec.vpextd\0"
  "llvm.ppc.altivec.vpkpx\0"
  "llvm.ppc.altivec.vpksdss\0"
  "llvm.ppc.altivec.vpksdus\0"
  "llvm.ppc.altivec.vpkshss\0"
  "llvm.ppc.altivec.vpkshus\0"
  "llvm.ppc.altivec.vpkswss\0"
  "llvm.ppc.altivec.vpkswus\0"
  "llvm.ppc.altivec.vpkudus\0"
  "llvm.ppc.altivec.vpkuhus\0"
  "llvm.ppc.altivec.vpkuwus\0"
  "llvm.ppc.altivec.vprtybd\0"
  "llvm.ppc.altivec.vprtybq\0"
  "llvm.ppc.altivec.vprtybw\0"
  "llvm.ppc.altivec.vrefp\0"
  "llvm.ppc.altivec.vrfim\0"
  "llvm.ppc.altivec.vrfin\0"
  "llvm.ppc.altivec.vrfip\0"
  "llvm.ppc.altivec.vrfiz\0"
  "llvm.ppc.altivec.vrlb\0"
  "llvm.ppc.altivec.vrld\0"
  "llvm.ppc.altivec.vrldmi\0"
  "llvm.ppc.altivec.vrldnm\0"
  "llvm.ppc.altivec.vrlh\0"
  "llvm.ppc.altivec.vrlqmi\0"
  "llvm.ppc.altivec.vrlqnm\0"
  "llvm.ppc.altivec.vrlw\0"
  "llvm.ppc.altivec.vrlwmi\0"
  "llvm.ppc.altivec.vrlwnm\0"
  "llvm.ppc.altivec.vrsqrtefp\0"
  "llvm.ppc.altivec.vsel\0"
  "llvm.ppc.altivec.vsl\0"
  "llvm.ppc.altivec.vslb\0"
  "llvm.ppc.altivec.vsldbi\0"
  "llvm.ppc.altivec.vslh\0"
  "llvm.ppc.altivec.vslo\0"
  "llvm.ppc.altivec.vslv\0"
  "llvm.ppc.altivec.vslw\0"
  "llvm.ppc.altivec.vsr\0"
  "llvm.ppc.altivec.vsrab\0"
  "llvm.ppc.altivec.vsrah\0"
  "llvm.ppc.altivec.vsraw\0"
  "llvm.ppc.altivec.vsrb\0"
  "llvm.ppc.altivec.vsrdbi\0"
  "llvm.ppc.altivec.vsrh\0"
  "llvm.ppc.altivec.vsro\0"
  "llvm.ppc.altivec.vsrv\0"
  "llvm.ppc.altivec.vsrw\0"
  "llvm.ppc.altivec.vstribl\0"
  "llvm.ppc.altivec.vstribl.p\0"
  "llvm.ppc.altivec.vstribr\0"
  "llvm.ppc.altivec.vstribr.p\0"
  "llvm.ppc.altivec.vstrihl\0"
  "llvm.ppc.altivec.vstrihl.p\0"
  "llvm.ppc.altivec.vstrihr\0"
  "llvm.ppc.altivec.vstrihr.p\0"
  "llvm.ppc.altivec.vsubcuq\0"
  "llvm.ppc.altivec.vsubcuw\0"
  "llvm.ppc.altivec.vsubecuq\0"
  "llvm.ppc.altivec.vsubeuqm\0"
  "llvm.ppc.altivec.vsubsbs\0"
  "llvm.ppc.altivec.vsubshs\0"
  "llvm.ppc.altivec.vsubsws\0"
  "llvm.ppc.altivec.vsububs\0"
  "llvm.ppc.altivec.vsubuhs\0"
  "llvm.ppc.altivec.vsubuws\0"
  "llvm.ppc.altivec.vsum2sws\0"
  "llvm.ppc.altivec.vsum4sbs\0"
  "llvm.ppc.altivec.vsum4shs\0"
  "llvm.ppc.altivec.vsum4ubs\0"
  "llvm.ppc.altivec.vsumsws\0"
  "llvm.ppc.altivec.vupkhpx\0"
  "llvm.ppc.altivec.vupkhsb\0"
  "llvm.ppc.altivec.vupkhsh\0"
  "llvm.ppc.altivec.vupkhsw\0"
  "llvm.ppc.altivec.vupklpx\0"
  "llvm.ppc.altivec.vupklsb\0"
  "llvm.ppc.altivec.vupklsh\0"
  "llvm.ppc.altivec.vupklsw\0"
  "llvm.ppc.amo.ldat\0"
  "llvm.ppc.amo.lwat\0"
  "llvm.ppc.atomic.load.i128\0"
  "llvm.ppc.atomic.store.i128\0"
  "llvm.ppc.atomicrmw.add.i128\0"
  "llvm.ppc.atomicrmw.and.i128\0"
  "llvm.ppc.atomicrmw.nand.i128\0"
  "llvm.ppc.atomicrmw.or.i128\0"
  "llvm.ppc.atomicrmw.sub.i128\0"
  "llvm.ppc.atomicrmw.xchg.i128\0"
  "llvm.ppc.atomicrmw.xor.i128\0"
  "llvm.ppc.bcdadd\0"
  "llvm.ppc.bcdadd.p\0"
  "llvm.ppc.bcdcopysign\0"
  "llvm.ppc.bcdsetsign\0"
  "llvm.ppc.bcdsub\0"
  "llvm.ppc.bcdsub.p\0"
  "llvm.ppc.bpermd\0"
  "llvm.ppc.cbcdtd\0"
  "llvm.ppc.cbcdtdd\0"
  "llvm.ppc.cdtbcd\0"
  "llvm.ppc.cdtbcdd\0"
  "llvm.ppc.cfence\0"
  "llvm.ppc.cfuged\0"
  "llvm.ppc.cmpb\0"
  "llvm.ppc.cmpeqb\0"
  "llvm.ppc.cmprb\0"
  "llvm.ppc.cmpxchg.i128\0"
  "llvm.ppc.cntlzdm\0"
  "llvm.ppc.cnttzdm\0"
  "llvm.ppc.compare.exp.eq\0"
  "llvm.ppc.compare.exp.gt\0"
  "llvm.ppc.compare.exp.lt\0"
  "llvm.ppc.compare.exp.uo\0"
  "llvm.ppc.convert.f128.to.ppcf128\0"
  "llvm.ppc.convert.ppcf128.to.f128\0"
  "llvm.ppc.darn\0"
  "llvm.ppc.darn32\0"
  "llvm.ppc.darnraw\0"
  "llvm.ppc.dcba\0"
  "llvm.ppc.dcbf\0"
  "llvm.ppc.dcbfl\0"
  "llvm.ppc.dcbflp\0"
  "llvm.ppc.dcbfps\0"
  "llvm.ppc.dcbi\0"
  "llvm.ppc.dcbst\0"
  "llvm.ppc.dcbstps\0"
  "llvm.ppc.dcbt\0"
  "llvm.ppc.dcbt.with.hint\0"
  "llvm.ppc.dcbtst\0"
  "llvm.ppc.dcbtst.with.hint\0"
  "llvm.ppc.dcbtstt\0"
  "llvm.ppc.dcbtt\0"
  "llvm.ppc.dcbz\0"
  "llvm.ppc.dcbzl\0"
  "llvm.ppc.divde\0"
  "llvm.ppc.divdeu\0"
  "llvm.ppc.divf128.round.to.odd\0"
  "llvm.ppc.divwe\0"
  "llvm.ppc.divweu\0"
  "llvm.ppc.eieio\0"
  "llvm.ppc.extract.exp\0"
  "llvm.ppc.extract.sig\0"
  "llvm.ppc.fcfid\0"
  "llvm.ppc.fcfud\0"
  "llvm.ppc.fctid\0"
  "llvm.ppc.fctidz\0"
  "llvm.ppc.fctiw\0"
  "llvm.ppc.fctiwz\0"
  "llvm.ppc.fctudz\0"
  "llvm.ppc.fctuwz\0"
  "llvm.ppc.fence\0"
  "llvm.ppc.fixed.addr.ld\0"
  "llvm.ppc.fmaf128.round.to.odd\0"
  "llvm.ppc.fmsub\0"
  "llvm.ppc.fmsubs\0"
  "llvm.ppc.fnabs\0"
  "llvm.ppc.fnabss\0"
  "llvm.ppc.fnmadd\0"
  "llvm.ppc.fnmadds\0"
  "llvm.ppc.fnmsub\0"
  "llvm.ppc.fre\0"
  "llvm.ppc.fres\0"
  "llvm.ppc.frsqrte\0"
  "llvm.ppc.frsqrtes\0"
  "llvm.ppc.fsel\0"
  "llvm.ppc.fsels\0"
  "llvm.ppc.get.texasr\0"
  "llvm.ppc.get.texasru\0"
  "llvm.ppc.get.tfhar\0"
  "llvm.ppc.get.tfiar\0"
  "llvm.ppc.icbt\0"
  "llvm.ppc.insert.exp\0"
  "llvm.ppc.iospace.eieio\0"
  "llvm.ppc.iospace.lwsync\0"
  "llvm.ppc.iospace.sync\0"
  "llvm.ppc.isync\0"
  "llvm.ppc.lbarx\0"
  "llvm.ppc.ldarx\0"
  "llvm.ppc.lharx\0"
  "llvm.ppc.load2r\0"
  "llvm.ppc.load4r\0"
  "llvm.ppc.load8r\0"
  "llvm.ppc.lwarx\0"
  "llvm.ppc.lwsync\0"
  "llvm.ppc.maddhd\0"
  "llvm.ppc.maddhdu\0"
  "llvm.ppc.maddld\0"
  "llvm.ppc.maxfe\0"
  "llvm.ppc.maxfl\0"
  "llvm.ppc.maxfs\0"
  "llvm.ppc.mffsl\0"
  "llvm.ppc.mfmsr\0"
  "llvm.ppc.mfspr\0"
  "llvm.ppc.mftbu\0"
  "llvm.ppc.minfe\0"
  "llvm.ppc.minfl\0"
  "llvm.ppc.minfs\0"
  "llvm.ppc.mma.assemble.acc\0"
  "llvm.ppc.mma.build.dmr\0"
  "llvm.ppc.mma.disassemble.acc\0"
  "llvm.ppc.mma.disassemble.dmr\0"
  "llvm.ppc.mma.dmmr\0"
  "llvm.ppc.mma.dmsetdmrz\0"
  "llvm.ppc.mma.dmsha2hash\0"
  "llvm.ppc.mma.dmsha3hash\0"
  "llvm.ppc.mma.dmxor\0"
  "llvm.ppc.mma.dmxvbf16gerx2\0"
  "llvm.ppc.mma.dmxvbf16gerx2nn\0"
  "llvm.ppc.mma.dmxvbf16gerx2np\0"
  "llvm.ppc.mma.dmxvbf16gerx2pn\0"
  "llvm.ppc.mma.dmxvbf16gerx2pp\0"
  "llvm.ppc.mma.dmxvf16gerx2\0"
  "llvm.ppc.mma.dmxvf16gerx2nn\0"
  "llvm.ppc.mma.dmxvf16gerx2np\0"
  "llvm.ppc.mma.dmxvf16gerx2pn\0"
  "llvm.ppc.mma.dmxvf16gerx2pp\0"
  "llvm.ppc.mma.dmxvi8gerx4\0"
  "llvm.ppc.mma.dmxvi8gerx4pp\0"
  "llvm.ppc.mma.dmxvi8gerx4spp\0"
  "llvm.ppc.mma.dmxxextfdmr256\0"
  "llvm.ppc.mma.dmxxextfdmr512\0"
  "llvm.ppc.mma.dmxxinstdmr256\0"
  "llvm.ppc.mma.dmxxinstdmr512\0"
  "llvm.ppc.mma.dmxxshapad\0"
  "llvm.ppc.mma.pmdmxvbf16gerx2\0"
  "llvm.ppc.mma.pmdmxvbf16gerx2nn\0"
  "llvm.ppc.mma.pmdmxvbf16gerx2np\0"
  "llvm.ppc.mma.pmdmxvbf16gerx2pn\0"
  "llvm.ppc.mma.pmdmxvbf16gerx2pp\0"
  "llvm.ppc.mma.pmdmxvf16gerx2\0"
  "llvm.ppc.mma.pmdmxvf16gerx2nn\0"
  "llvm.ppc.mma.pmdmxvf16gerx2np\0"
  "llvm.ppc.mma.pmdmxvf16gerx2pn\0"
  "llvm.ppc.mma.pmdmxvf16gerx2pp\0"
  "llvm.ppc.mma.pmdmxvi8gerx4\0"
  "llvm.ppc.mma.pmdmxvi8gerx4pp\0"
  "llvm.ppc.mma.pmdmxvi8gerx4spp\0"
  "llvm.ppc.mma.pmxvbf16ger2\0"
  "llvm.ppc.mma.pmxvbf16ger2nn\0"
  "llvm.ppc.mma.pmxvbf16ger2np\0"
  "llvm.ppc.mma.pmxvbf16ger2pn\0"
  "llvm.ppc.mma.pmxvbf16ger2pp\0"
  "llvm.ppc.mma.pmxvf16ger2\0"
  "llvm.ppc.mma.pmxvf16ger2nn\0"
  "llvm.ppc.mma.pmxvf16ger2np\0"
  "llvm.ppc.mma.pmxvf16ger2pn\0"
  "llvm.ppc.mma.pmxvf16ger2pp\0"
  "llvm.ppc.mma.pmxvf32ger\0"
  "llvm.ppc.mma.pmxvf32gernn\0"
  "llvm.ppc.mma.pmxvf32gernp\0"
  "llvm.ppc.mma.pmxvf32gerpn\0"
  "llvm.ppc.mma.pmxvf32gerpp\0"
  "llvm.ppc.mma.pmxvf64ger\0"
  "llvm.ppc.mma.pmxvf64gernn\0"
  "llvm.ppc.mma.pmxvf64gernp\0"
  "llvm.ppc.mma.pmxvf64gerpn\0"
  "llvm.ppc.mma.pmxvf64gerpp\0"
  "llvm.ppc.mma.pmxvi16ger2\0"
  "llvm.ppc.mma.pmxvi16ger2pp\0"
  "llvm.ppc.mma.pmxvi16ger2s\0"
  "llvm.ppc.mma.pmxvi16ger2spp\0"
  "llvm.ppc.mma.pmxvi4ger8\0"
  "llvm.ppc.mma.pmxvi4ger8pp\0"
  "llvm.ppc.mma.pmxvi8ger4\0"
  "llvm.ppc.mma.pmxvi8ger4pp\0"
  "llvm.ppc.mma.pmxvi8ger4spp\0"
  "llvm.ppc.mma.xvbf16ger2\0"
  "llvm.ppc.mma.xvbf16ger2nn\0"
  "llvm.ppc.mma.xvbf16ger2np\0"
  "llvm.ppc.mma.xvbf16ger2pn\0"
  "llvm.ppc.mma.xvbf16ger2pp\0"
  "llvm.ppc.mma.xvf16ger2\0"
  "llvm.ppc.mma.xvf16ger2nn\0"
  "llvm.ppc.mma.xvf16ger2np\0"
  "llvm.ppc.mma.xvf16ger2pn\0"
  "llvm.ppc.mma.xvf16ger2pp\0"
  "llvm.ppc.mma.xvf32ger\0"
  "llvm.ppc.mma.xvf32gernn\0"
  "llvm.ppc.mma.xvf32gernp\0"
  "llvm.ppc.mma.xvf32gerpn\0"
  "llvm.ppc.mma.xvf32gerpp\0"
  "llvm.ppc.mma.xvf64ger\0"
  "llvm.ppc.mma.xvf64gernn\0"
  "llvm.ppc.mma.xvf64gernp\0"
  "llvm.ppc.mma.xvf64gerpn\0"
  "llvm.ppc.mma.xvf64gerpp\0"
  "llvm.ppc.mma.xvi16ger2\0"
  "llvm.ppc.mma.xvi16ger2pp\0"
  "llvm.ppc.mma.xvi16ger2s\0"
  "llvm.ppc.mma.xvi16ger2spp\0"
  "llvm.ppc.mma.xvi4ger8\0"
  "llvm.ppc.mma.xvi4ger8pp\0"
  "llvm.ppc.mma.xvi8ger4\0"
  "llvm.ppc.mma.xvi8ger4pp\0"
  "llvm.ppc.mma.xvi8ger4spp\0"
  "llvm.ppc.mma.xxmfacc\0"
  "llvm.ppc.mma.xxmtacc\0"
  "llvm.ppc.mma.xxsetaccz\0"
  "llvm.ppc.mtfsb0\0"
  "llvm.ppc.mtfsb1\0"
  "llvm.ppc.mtfsf\0"
  "llvm.ppc.mtfsfi\0"
  "llvm.ppc.mtmsr\0"
  "llvm.ppc.mtspr\0"
  "llvm.ppc.mulf128.round.to.odd\0"
  "llvm.ppc.mulhd\0"
  "llvm.ppc.mulhdu\0"
  "llvm.ppc.mulhw\0"
  "llvm.ppc.mulhwu\0"
  "llvm.ppc.national2packed\0"
  "llvm.ppc.pack.longdouble\0"
  "llvm.ppc.packed2national\0"
  "llvm.ppc.packed2zoned\0"
  "llvm.ppc.pdepd\0"
  "llvm.ppc.pextd\0"
  "llvm.ppc.popcntb\0"
  "llvm.ppc.readflm\0"
  "llvm.ppc.rldimi\0"
  "llvm.ppc.rlwimi\0"
  "llvm.ppc.rlwnm\0"
  "llvm.ppc.scalar.extract.expq\0"
  "llvm.ppc.scalar.insert.exp.qp\0"
  "llvm.ppc.set.texasr\0"
  "llvm.ppc.set.texasru\0"
  "llvm.ppc.set.tfhar\0"
  "llvm.ppc.set.tfiar\0"
  "llvm.ppc.setb\0"
  "llvm.ppc.setflm\0"
  "llvm.ppc.setrnd\0"
  "llvm.ppc.sqrtf128.round.to.odd\0"
  "llvm.ppc.stbcx\0"
  "llvm.ppc.stdcx\0"
  "llvm.ppc.stfiw\0"
  "llvm.ppc.sthcx\0"
  "llvm.ppc.store2r\0"
  "llvm.ppc.store4r\0"
  "llvm.ppc.store8r\0"
  "llvm.ppc.stwcx\0"
  "llvm.ppc.subf128.round.to.odd\0"
  "llvm.ppc.sync\0"
  "llvm.ppc.tabort\0"
  "llvm.ppc.tabortdc\0"
  "llvm.ppc.tabortdci\0"
  "llvm.ppc.tabortwc\0"
  "llvm.ppc.tabortwci\0"
  "llvm.ppc.tbegin\0"
  "llvm.ppc.tcheck\0"
  "llvm.ppc.tdw\0"
  "llvm.ppc.tend\0"
  "llvm.ppc.tendall\0"
  "llvm.ppc.test.data.class\0"
  "llvm.ppc.trap\0"
  "llvm.ppc.trapd\0"
  "llvm.ppc.trechkpt\0"
  "llvm.ppc.treclaim\0"
  "llvm.ppc.tresume\0"
  "llvm.ppc.truncf128.round.to.odd\0"
  "llvm.ppc.tsr\0"
  "llvm.ppc.tsuspend\0"
  "llvm.ppc.ttest\0"
  "llvm.ppc.tw\0"
  "llvm.ppc.unpack.longdouble\0"
  "llvm.ppc.vsx.assemble.pair\0"
  "llvm.ppc.vsx.disassemble.pair\0"
  "llvm.ppc.vsx.lxvd2x\0"
  "llvm.ppc.vsx.lxvd2x.be\0"
  "llvm.ppc.vsx.lxvl\0"
  "llvm.ppc.vsx.lxvll\0"
  "llvm.ppc.vsx.lxvp\0"
  "llvm.ppc.vsx.lxvprl\0"
  "llvm.ppc.vsx.lxvprll\0"
  "llvm.ppc.vsx.lxvrl\0"
  "llvm.ppc.vsx.lxvrll\0"
  "llvm.ppc.vsx.lxvw4x\0"
  "llvm.ppc.vsx.lxvw4x.be\0"
  "llvm.ppc.vsx.stxvd2x\0"
  "llvm.ppc.vsx.stxvd2x.be\0"
  "llvm.ppc.vsx.stxvl\0"
  "llvm.ppc.vsx.stxvll\0"
  "llvm.ppc.vsx.stxvp\0"
  "llvm.ppc.vsx.stxvprl\0"
  "llvm.ppc.vsx.stxvprll\0"
  "llvm.ppc.vsx.stxvrl\0"
  "llvm.ppc.vsx.stxvrll\0"
  "llvm.ppc.vsx.stxvw4x\0"
  "llvm.ppc.vsx.stxvw4x.be\0"
  "llvm.ppc.vsx.xsmaxdp\0"
  "llvm.ppc.vsx.xsmindp\0"
  "llvm.ppc.vsx.xvcmpeqdp\0"
  "llvm.ppc.vsx.xvcmpeqdp.p\0"
  "llvm.ppc.vsx.xvcmpeqsp\0"
  "llvm.ppc.vsx.xvcmpeqsp.p\0"
  "llvm.ppc.vsx.xvcmpgedp\0"
  "llvm.ppc.vsx.xvcmpgedp.p\0"
  "llvm.ppc.vsx.xvcmpgesp\0"
  "llvm.ppc.vsx.xvcmpgesp.p\0"
  "llvm.ppc.vsx.xvcmpgtdp\0"
  "llvm.ppc.vsx.xvcmpgtdp.p\0"
  "llvm.ppc.vsx.xvcmpgtsp\0"
  "llvm.ppc.vsx.xvcmpgtsp.p\0"
  "llvm.ppc.vsx.xvcvbf16spn\0"
  "llvm.ppc.vsx.xvcvdpsp\0"
  "llvm.ppc.vsx.xvcvdpsxws\0"
  "llvm.ppc.vsx.xvcvdpuxws\0"
  "llvm.ppc.vsx.xvcvhpsp\0"
  "llvm.ppc.vsx.xvcvspbf16\0"
  "llvm.ppc.vsx.xvcvspdp\0"
  "llvm.ppc.vsx.xvcvsphp\0"
  "llvm.ppc.vsx.xvcvspsxds\0"
  "llvm.ppc.vsx.xvcvspuxds\0"
  "llvm.ppc.vsx.xvcvsxdsp\0"
  "llvm.ppc.vsx.xvcvsxwdp\0"
  "llvm.ppc.vsx.xvcvuxdsp\0"
  "llvm.ppc.vsx.xvcvuxwdp\0"
  "llvm.ppc.vsx.xvdivdp\0"
  "llvm.ppc.vsx.xvdivsp\0"
  "llvm.ppc.vsx.xviexpdp\0"
  "llvm.ppc.vsx.xviexpsp\0"
  "llvm.ppc.vsx.xvmaxdp\0"
  "llvm.ppc.vsx.xvmaxsp\0"
  "llvm.ppc.vsx.xvmindp\0"
  "llvm.ppc.vsx.xvminsp\0"
  "llvm.ppc.vsx.xvrdpip\0"
  "llvm.ppc.vsx.xvredp\0"
  "llvm.ppc.vsx.xvresp\0"
  "llvm.ppc.vsx.xvrlw\0"
  "llvm.ppc.vsx.xvrspip\0"
  "llvm.ppc.vsx.xvrsqrtedp\0"
  "llvm.ppc.vsx.xvrsqrtesp\0"
  "llvm.ppc.vsx.xvtdivdp\0"
  "llvm.ppc.vsx.xvtdivsp\0"
  "llvm.ppc.vsx.xvtlsbb\0"
  "llvm.ppc.vsx.xvtsqrtdp\0"
  "llvm.ppc.vsx.xvtsqrtsp\0"
  "llvm.ppc.vsx.xvtstdcdp\0"
  "llvm.ppc.vsx.xvtstdcsp\0"
  "llvm.ppc.vsx.xvxexpdp\0"
  "llvm.ppc.vsx.xvxexpsp\0"
  "llvm.ppc.vsx.xvxsigdp\0"
  "llvm.ppc.vsx.xvxsigsp\0"
  "llvm.ppc.vsx.xxblendvb\0"
  "llvm.ppc.vsx.xxblendvd\0"
  "llvm.ppc.vsx.xxblendvh\0"
  "llvm.ppc.vsx.xxblendvw\0"
  "llvm.ppc.vsx.xxeval\0"
  "llvm.ppc.vsx.xxextractuw\0"
  "llvm.ppc.vsx.xxgenpcvbm\0"
  "llvm.ppc.vsx.xxgenpcvdm\0"
  "llvm.ppc.vsx.xxgenpcvhm\0"
  "llvm.ppc.vsx.xxgenpcvwm\0"
  "llvm.ppc.vsx.xxinsertw\0"
  "llvm.ppc.vsx.xxleqv\0"
  "llvm.ppc.vsx.xxpermx\0"
  "llvm.ppc.zoned2packed\0"
  "llvm.r600.cube\0"
  "llvm.r600.ddx\0"
  "llvm.r600.ddy\0"
  "llvm.r600.dot4\0"
  "llvm.r600.group.barrier\0"
  "llvm.r600.implicitarg.ptr\0"
  "llvm.r600.kill\0"
  "llvm.r600.rat.store.typed\0"
  "llvm.r600.read.global.size.x\0"
  "llvm.r600.read.global.size.y\0"
  "llvm.r600.read.global.size.z\0"
  "llvm.r600.read.local.size.x\0"
  "llvm.r600.read.local.size.y\0"
  "llvm.r600.read.local.size.z\0"
  "llvm.r600.read.ngroups.x\0"
  "llvm.r600.read.ngroups.y\0"
  "llvm.r600.read.ngroups.z\0"
  "llvm.r600.read.tgid.x\0"
  "llvm.r600.read.tgid.y\0"
  "llvm.r600.read.tgid.z\0"
  "llvm.r600.read.tidig.x\0"
  "llvm.r600.read.tidig.y\0"
  "llvm.r600.read.tidig.z\0"
  "llvm.r600.recipsqrt.clamped\0"
  "llvm.r600.recipsqrt.ieee\0"
  "llvm.r600.store.stream.output\0"
  "llvm.r600.store.swizzle\0"
  "llvm.r600.tex\0"
  "llvm.r600.texc\0"
  "llvm.r600.txb\0"
  "llvm.r600.txbc\0"
  "llvm.r600.txf\0"
  "llvm.r600.txl\0"
  "llvm.r600.txlc\0"
  "llvm.r600.txq\0"
  "llvm.riscv.aes32dsi\0"
  "llvm.riscv.aes32dsmi\0"
  "llvm.riscv.aes32esi\0"
  "llvm.riscv.aes32esmi\0"
  "llvm.riscv.aes64ds\0"
  "llvm.riscv.aes64dsm\0"
  "llvm.riscv.aes64es\0"
  "llvm.riscv.aes64esm\0"
  "llvm.riscv.aes64im\0"
  "llvm.riscv.aes64ks1i\0"
  "llvm.riscv.aes64ks2\0"
  "llvm.riscv.brev8\0"
  "llvm.riscv.clmul\0"
  "llvm.riscv.clmulh\0"
  "llvm.riscv.clmulr\0"
  "llvm.riscv.cv.alu.addN\0"
  "llvm.riscv.cv.alu.addRN\0"
  "llvm.riscv.cv.alu.adduN\0"
  "llvm.riscv.cv.alu.adduRN\0"
  "llvm.riscv.cv.alu.clip\0"
  "llvm.riscv.cv.alu.clipu\0"
  "llvm.riscv.cv.alu.subN\0"
  "llvm.riscv.cv.alu.subRN\0"
  "llvm.riscv.cv.alu.subuN\0"
  "llvm.riscv.cv.alu.subuRN\0"
  "llvm.riscv.cv.bitmanip.bclr\0"
  "llvm.riscv.cv.bitmanip.bitrev\0"
  "llvm.riscv.cv.bitmanip.bset\0"
  "llvm.riscv.cv.bitmanip.clb\0"
  "llvm.riscv.cv.bitmanip.extract\0"
  "llvm.riscv.cv.bitmanip.extractu\0"
  "llvm.riscv.cv.bitmanip.insert\0"
  "llvm.riscv.cv.elw.elw\0"
  "llvm.riscv.cv.mac.mac\0"
  "llvm.riscv.cv.mac.machhsN\0"
  "llvm.riscv.cv.mac.machhsRN\0"
  "llvm.riscv.cv.mac.machhuN\0"
  "llvm.riscv.cv.mac.machhuRN\0"
  "llvm.riscv.cv.mac.macsN\0"
  "llvm.riscv.cv.mac.macsRN\0"
  "llvm.riscv.cv.mac.macuN\0"
  "llvm.riscv.cv.mac.macuRN\0"
  "llvm.riscv.cv.mac.msu\0"
  "llvm.riscv.cv.mac.mulhhsN\0"
  "llvm.riscv.cv.mac.mulhhsRN\0"
  "llvm.riscv.cv.mac.mulhhuN\0"
  "llvm.riscv.cv.mac.mulhhuRN\0"
  "llvm.riscv.cv.mac.mulsN\0"
  "llvm.riscv.cv.mac.mulsRN\0"
  "llvm.riscv.cv.mac.muluN\0"
  "llvm.riscv.cv.mac.muluRN\0"
  "llvm.riscv.masked.atomicrmw.add\0"
  "llvm.riscv.masked.atomicrmw.max\0"
  "llvm.riscv.masked.atomicrmw.min\0"
  "llvm.riscv.masked.atomicrmw.nand\0"
  "llvm.riscv.masked.atomicrmw.sub\0"
  "llvm.riscv.masked.atomicrmw.umax\0"
  "llvm.riscv.masked.atomicrmw.umin\0"
  "llvm.riscv.masked.atomicrmw.xchg\0"
  "llvm.riscv.masked.cmpxchg\0"
  "llvm.riscv.mips.ehb\0"
  "llvm.riscv.mips.ihb\0"
  "llvm.riscv.mips.pause\0"
  "llvm.riscv.mopr\0"
  "llvm.riscv.moprr\0"
  "llvm.riscv.nds.ffb\0"
  "llvm.riscv.nds.ffmism\0"
  "llvm.riscv.nds.ffzmism\0"
  "llvm.riscv.nds.flmism\0"
  "llvm.riscv.nds.vd4dots\0"
  "llvm.riscv.nds.vd4dots.mask\0"
  "llvm.riscv.nds.vd4dotsu\0"
  "llvm.riscv.nds.vd4dotsu.mask\0"
  "llvm.riscv.nds.vd4dotu\0"
  "llvm.riscv.nds.vd4dotu.mask\0"
  "llvm.riscv.nds.vfncvt.bf16.s\0"
  "llvm.riscv.nds.vfpmadb\0"
  "llvm.riscv.nds.vfpmadb.mask\0"
  "llvm.riscv.nds.vfpmadt\0"
  "llvm.riscv.nds.vfpmadt.mask\0"
  "llvm.riscv.nds.vfwcvt.s.bf16\0"
  "llvm.riscv.nds.vln\0"
  "llvm.riscv.nds.vln.mask\0"
  "llvm.riscv.nds.vlnu\0"
  "llvm.riscv.nds.vlnu.mask\0"
  "llvm.riscv.orc.b\0"
  "llvm.riscv.pause\0"
  "llvm.riscv.seg2.load.mask\0"
  "llvm.riscv.seg2.store.mask\0"
  "llvm.riscv.seg3.load.mask\0"
  "llvm.riscv.seg3.store.mask\0"
  "llvm.riscv.seg4.load.mask\0"
  "llvm.riscv.seg4.store.mask\0"
  "llvm.riscv.seg5.load.mask\0"
  "llvm.riscv.seg5.store.mask\0"
  "llvm.riscv.seg6.load.mask\0"
  "llvm.riscv.seg6.store.mask\0"
  "llvm.riscv.seg7.load.mask\0"
  "llvm.riscv.seg7.store.mask\0"
  "llvm.riscv.seg8.load.mask\0"
  "llvm.riscv.seg8.store.mask\0"
  "llvm.riscv.sf.mm.e4m3.e4m3\0"
  "llvm.riscv.sf.mm.e4m3.e5m2\0"
  "llvm.riscv.sf.mm.e5m2.e4m3\0"
  "llvm.riscv.sf.mm.e5m2.e5m2\0"
  "llvm.riscv.sf.mm.f.f\0"
  "llvm.riscv.sf.mm.s.s\0"
  "llvm.riscv.sf.mm.s.u\0"
  "llvm.riscv.sf.mm.u.s\0"
  "llvm.riscv.sf.mm.u.u\0"
  "llvm.riscv.sf.vc.fv.se\0"
  "llvm.riscv.sf.vc.fvv.se\0"
  "llvm.riscv.sf.vc.fvw.se\0"
  "llvm.riscv.sf.vc.i.se\0"
  "llvm.riscv.sf.vc.iv.se\0"
  "llvm.riscv.sf.vc.ivv.se\0"
  "llvm.riscv.sf.vc.ivw.se\0"
  "llvm.riscv.sf.vc.v.fv\0"
  "llvm.riscv.sf.vc.v.fv.se\0"
  "llvm.riscv.sf.vc.v.fvv\0"
  "llvm.riscv.sf.vc.v.fvv.se\0"
  "llvm.riscv.sf.vc.v.fvw\0"
  "llvm.riscv.sf.vc.v.fvw.se\0"
  "llvm.riscv.sf.vc.v.i\0"
  "llvm.riscv.sf.vc.v.i.se\0"
  "llvm.riscv.sf.vc.v.iv\0"
  "llvm.riscv.sf.vc.v.iv.se\0"
  "llvm.riscv.sf.vc.v.ivv\0"
  "llvm.riscv.sf.vc.v.ivv.se\0"
  "llvm.riscv.sf.vc.v.ivw\0"
  "llvm.riscv.sf.vc.v.ivw.se\0"
  "llvm.riscv.sf.vc.v.vv\0"
  "llvm.riscv.sf.vc.v.vv.se\0"
  "llvm.riscv.sf.vc.v.vvv\0"
  "llvm.riscv.sf.vc.v.vvv.se\0"
  "llvm.riscv.sf.vc.v.vvw\0"
  "llvm.riscv.sf.vc.v.vvw.se\0"
  "llvm.riscv.sf.vc.v.x\0"
  "llvm.riscv.sf.vc.v.x.se\0"
  "llvm.riscv.sf.vc.v.xv\0"
  "llvm.riscv.sf.vc.v.xv.se\0"
  "llvm.riscv.sf.vc.v.xvv\0"
  "llvm.riscv.sf.vc.v.xvv.se\0"
  "llvm.riscv.sf.vc.v.xvw\0"
  "llvm.riscv.sf.vc.v.xvw.se\0"
  "llvm.riscv.sf.vc.vv.se\0"
  "llvm.riscv.sf.vc.vvv.se\0"
  "llvm.riscv.sf.vc.vvw.se\0"
  "llvm.riscv.sf.vc.x.se\0"
  "llvm.riscv.sf.vc.xv.se\0"
  "llvm.riscv.sf.vc.xvv.se\0"
  "llvm.riscv.sf.vc.xvw.se\0"
  "llvm.riscv.sf.vfexp\0"
  "llvm.riscv.sf.vfexp.mask\0"
  "llvm.riscv.sf.vfexpa\0"
  "llvm.riscv.sf.vfexpa.mask\0"
  "llvm.riscv.sf.vfnrclip.x.f.qf\0"
  "llvm.riscv.sf.vfnrclip.x.f.qf.mask\0"
  "llvm.riscv.sf.vfnrclip.xu.f.qf\0"
  "llvm.riscv.sf.vfnrclip.xu.f.qf.mask\0"
  "llvm.riscv.sf.vfwmacc.4x4x4\0"
  "llvm.riscv.sf.vlte16\0"
  "llvm.riscv.sf.vlte32\0"
  "llvm.riscv.sf.vlte64\0"
  "llvm.riscv.sf.vlte8\0"
  "llvm.riscv.sf.vqmacc.2x8x2\0"
  "llvm.riscv.sf.vqmacc.4x8x4\0"
  "llvm.riscv.sf.vqmaccsu.2x8x2\0"
  "llvm.riscv.sf.vqmaccsu.4x8x4\0"
  "llvm.riscv.sf.vqmaccu.2x8x2\0"
  "llvm.riscv.sf.vqmaccu.4x8x4\0"
  "llvm.riscv.sf.vqmaccus.2x8x2\0"
  "llvm.riscv.sf.vqmaccus.4x8x4\0"
  "llvm.riscv.sf.vsettk\0"
  "llvm.riscv.sf.vsettm\0"
  "llvm.riscv.sf.vsettnt\0"
  "llvm.riscv.sf.vste16\0"
  "llvm.riscv.sf.vste32\0"
  "llvm.riscv.sf.vste64\0"
  "llvm.riscv.sf.vste8\0"
  "llvm.riscv.sf.vtdiscard\0"
  "llvm.riscv.sf.vtmv.t.v\0"
  "llvm.riscv.sf.vtmv.v.t\0"
  "llvm.riscv.sf.vtzero.t\0"
  "llvm.riscv.sha256sig0\0"
  "llvm.riscv.sha256sig1\0"
  "llvm.riscv.sha256sum0\0"
  "llvm.riscv.sha256sum1\0"
  "llvm.riscv.sha512sig0\0"
  "llvm.riscv.sha512sig0h\0"
  "llvm.riscv.sha512sig0l\0"
  "llvm.riscv.sha512sig1\0"
  "llvm.riscv.sha512sig1h\0"
  "llvm.riscv.sha512sig1l\0"
  "llvm.riscv.sha512sum0\0"
  "llvm.riscv.sha512sum0r\0"
  "llvm.riscv.sha512sum1\0"
  "llvm.riscv.sha512sum1r\0"
  "llvm.riscv.sm3p0\0"
  "llvm.riscv.sm3p1\0"
  "llvm.riscv.sm4ed\0"
  "llvm.riscv.sm4ks\0"
  "llvm.riscv.sseg2.load.mask\0"
  "llvm.riscv.sseg2.store.mask\0"
  "llvm.riscv.sseg3.load.mask\0"
  "llvm.riscv.sseg3.store.mask\0"
  "llvm.riscv.sseg4.load.mask\0"
  "llvm.riscv.sseg4.store.mask\0"
  "llvm.riscv.sseg5.load.mask\0"
  "llvm.riscv.sseg5.store.mask\0"
  "llvm.riscv.sseg6.load.mask\0"
  "llvm.riscv.sseg6.store.mask\0"
  "llvm.riscv.sseg7.load.mask\0"
  "llvm.riscv.sseg7.store.mask\0"
  "llvm.riscv.sseg8.load.mask\0"
  "llvm.riscv.sseg8.store.mask\0"
  "llvm.riscv.th.vmaqa\0"
  "llvm.riscv.th.vmaqa.mask\0"
  "llvm.riscv.th.vmaqasu\0"
  "llvm.riscv.th.vmaqasu.mask\0"
  "llvm.riscv.th.vmaqau\0"
  "llvm.riscv.th.vmaqau.mask\0"
  "llvm.riscv.th.vmaqaus\0"
  "llvm.riscv.th.vmaqaus.mask\0"
  "llvm.riscv.tuple.extract\0"
  "llvm.riscv.tuple.insert\0"
  "llvm.riscv.unzip\0"
  "llvm.riscv.vaadd\0"
  "llvm.riscv.vaadd.mask\0"
  "llvm.riscv.vaaddu\0"
  "llvm.riscv.vaaddu.mask\0"
  "llvm.riscv.vadc\0"
  "llvm.riscv.vadd\0"
  "llvm.riscv.vadd.mask\0"
  "llvm.riscv.vaesdf.vs\0"
  "llvm.riscv.vaesdf.vv\0"
  "llvm.riscv.vaesdm.vs\0"
  "llvm.riscv.vaesdm.vv\0"
  "llvm.riscv.vaesef.vs\0"
  "llvm.riscv.vaesef.vv\0"
  "llvm.riscv.vaesem.vs\0"
  "llvm.riscv.vaesem.vv\0"
  "llvm.riscv.vaeskf1\0"
  "llvm.riscv.vaeskf2\0"
  "llvm.riscv.vaesz.vs\0"
  "llvm.riscv.vand\0"
  "llvm.riscv.vand.mask\0"
  "llvm.riscv.vandn\0"
  "llvm.riscv.vandn.mask\0"
  "llvm.riscv.vasub\0"
  "llvm.riscv.vasub.mask\0"
  "llvm.riscv.vasubu\0"
  "llvm.riscv.vasubu.mask\0"
  "llvm.riscv.vbrev\0"
  "llvm.riscv.vbrev.mask\0"
  "llvm.riscv.vbrev8\0"
  "llvm.riscv.vbrev8.mask\0"
  "llvm.riscv.vclmul\0"
  "llvm.riscv.vclmul.mask\0"
  "llvm.riscv.vclmulh\0"
  "llvm.riscv.vclmulh.mask\0"
  "llvm.riscv.vclz\0"
  "llvm.riscv.vclz.mask\0"
  "llvm.riscv.vcompress\0"
  "llvm.riscv.vcpop\0"
  "llvm.riscv.vcpop.mask\0"
  "llvm.riscv.vcpopv\0"
  "llvm.riscv.vcpopv.mask\0"
  "llvm.riscv.vctz\0"
  "llvm.riscv.vctz.mask\0"
  "llvm.riscv.vdiv\0"
  "llvm.riscv.vdiv.mask\0"
  "llvm.riscv.vdivu\0"
  "llvm.riscv.vdivu.mask\0"
  "llvm.riscv.vfadd\0"
  "llvm.riscv.vfadd.mask\0"
  "llvm.riscv.vfclass\0"
  "llvm.riscv.vfclass.mask\0"
  "llvm.riscv.vfcvt.f.x.v\0"
  "llvm.riscv.vfcvt.f.x.v.mask\0"
  "llvm.riscv.vfcvt.f.xu.v\0"
  "llvm.riscv.vfcvt.f.xu.v.mask\0"
  "llvm.riscv.vfcvt.rtz.x.f.v\0"
  "llvm.riscv.vfcvt.rtz.x.f.v.mask\0"
  "llvm.riscv.vfcvt.rtz.xu.f.v\0"
  "llvm.riscv.vfcvt.rtz.xu.f.v.mask\0"
  "llvm.riscv.vfcvt.x.f.v\0"
  "llvm.riscv.vfcvt.x.f.v.mask\0"
  "llvm.riscv.vfcvt.xu.f.v\0"
  "llvm.riscv.vfcvt.xu.f.v.mask\0"
  "llvm.riscv.vfdiv\0"
  "llvm.riscv.vfdiv.mask\0"
  "llvm.riscv.vfirst\0"
  "llvm.riscv.vfirst.mask\0"
  "llvm.riscv.vfmacc\0"
  "llvm.riscv.vfmacc.mask\0"
  "llvm.riscv.vfmadd\0"
  "llvm.riscv.vfmadd.mask\0"
  "llvm.riscv.vfmax\0"
  "llvm.riscv.vfmax.mask\0"
  "llvm.riscv.vfmerge\0"
  "llvm.riscv.vfmin\0"
  "llvm.riscv.vfmin.mask\0"
  "llvm.riscv.vfmsac\0"
  "llvm.riscv.vfmsac.mask\0"
  "llvm.riscv.vfmsub\0"
  "llvm.riscv.vfmsub.mask\0"
  "llvm.riscv.vfmul\0"
  "llvm.riscv.vfmul.mask\0"
  "llvm.riscv.vfmv.f.s\0"
  "llvm.riscv.vfmv.s.f\0"
  "llvm.riscv.vfmv.v.f\0"
  "llvm.riscv.vfncvt.f.f.q\0"
  "llvm.riscv.vfncvt.f.f.q.alt\0"
  "llvm.riscv.vfncvt.f.f.q.alt.mask\0"
  "llvm.riscv.vfncvt.f.f.q.mask\0"
  "llvm.riscv.vfncvt.f.f.w\0"
  "llvm.riscv.vfncvt.f.f.w.alt\0"
  "llvm.riscv.vfncvt.f.f.w.alt.mask\0"
  "llvm.riscv.vfncvt.f.f.w.mask\0"
  "llvm.riscv.vfncvt.f.x.w\0"
  "llvm.riscv.vfncvt.f.x.w.mask\0"
  "llvm.riscv.vfncvt.f.xu.w\0"
  "llvm.riscv.vfncvt.f.xu.w.mask\0"
  "llvm.riscv.vfncvt.rod.f.f.w\0"
  "llvm.riscv.vfncvt.rod.f.f.w.mask\0"
  "llvm.riscv.vfncvt.rtz.x.f.w\0"
  "llvm.riscv.vfncvt.rtz.x.f.w.mask\0"
  "llvm.riscv.vfncvt.rtz.xu.f.w\0"
  "llvm.riscv.vfncvt.rtz.xu.f.w.mask\0"
  "llvm.riscv.vfncvt.sat.f.f.q\0"
  "llvm.riscv.vfncvt.sat.f.f.q.alt\0"
  "llvm.riscv.vfncvt.sat.f.f.q.alt.mask\0"
  "llvm.riscv.vfncvt.sat.f.f.q.mask\0"
  "llvm.riscv.vfncvt.sat.f.f.w\0"
  "llvm.riscv.vfncvt.sat.f.f.w.alt\0"
  "llvm.riscv.vfncvt.sat.f.f.w.alt.mask\0"
  "llvm.riscv.vfncvt.sat.f.f.w.mask\0"
  "llvm.riscv.vfncvt.x.f.w\0"
  "llvm.riscv.vfncvt.x.f.w.mask\0"
  "llvm.riscv.vfncvt.xu.f.w\0"
  "llvm.riscv.vfncvt.xu.f.w.mask\0"
  "llvm.riscv.vfncvtbf16.f.f.w\0"
  "llvm.riscv.vfncvtbf16.f.f.w.mask\0"
  "llvm.riscv.vfnmacc\0"
  "llvm.riscv.vfnmacc.mask\0"
  "llvm.riscv.vfnmadd\0"
  "llvm.riscv.vfnmadd.mask\0"
  "llvm.riscv.vfnmsac\0"
  "llvm.riscv.vfnmsac.mask\0"
  "llvm.riscv.vfnmsub\0"
  "llvm.riscv.vfnmsub.mask\0"
  "llvm.riscv.vfrdiv\0"
  "llvm.riscv.vfrdiv.mask\0"
  "llvm.riscv.vfrec7\0"
  "llvm.riscv.vfrec7.mask\0"
  "llvm.riscv.vfredmax\0"
  "llvm.riscv.vfredmax.mask\0"
  "llvm.riscv.vfredmin\0"
  "llvm.riscv.vfredmin.mask\0"
  "llvm.riscv.vfredosum\0"
  "llvm.riscv.vfredosum.mask\0"
  "llvm.riscv.vfredusum\0"
  "llvm.riscv.vfredusum.mask\0"
  "llvm.riscv.vfrsqrt7\0"
  "llvm.riscv.vfrsqrt7.mask\0"
  "llvm.riscv.vfrsub\0"
  "llvm.riscv.vfrsub.mask\0"
  "llvm.riscv.vfsgnj\0"
  "llvm.riscv.vfsgnj.mask\0"
  "llvm.riscv.vfsgnjn\0"
  "llvm.riscv.vfsgnjn.mask\0"
  "llvm.riscv.vfsgnjx\0"
  "llvm.riscv.vfsgnjx.mask\0"
  "llvm.riscv.vfslide1down\0"
  "llvm.riscv.vfslide1down.mask\0"
  "llvm.riscv.vfslide1up\0"
  "llvm.riscv.vfslide1up.mask\0"
  "llvm.riscv.vfsqrt\0"
  "llvm.riscv.vfsqrt.mask\0"
  "llvm.riscv.vfsub\0"
  "llvm.riscv.vfsub.mask\0"
  "llvm.riscv.vfwadd\0"
  "llvm.riscv.vfwadd.mask\0"
  "llvm.riscv.vfwadd.w\0"
  "llvm.riscv.vfwadd.w.mask\0"
  "llvm.riscv.vfwcvt.f.f.v\0"
  "llvm.riscv.vfwcvt.f.f.v.alt\0"
  "llvm.riscv.vfwcvt.f.f.v.alt.mask\0"
  "llvm.riscv.vfwcvt.f.f.v.mask\0"
  "llvm.riscv.vfwcvt.f.x.v\0"
  "llvm.riscv.vfwcvt.f.x.v.mask\0"
  "llvm.riscv.vfwcvt.f.xu.v\0"
  "llvm.riscv.vfwcvt.f.xu.v.mask\0"
  "llvm.riscv.vfwcvt.rtz.x.f.v\0"
  "llvm.riscv.vfwcvt.rtz.x.f.v.mask\0"
  "llvm.riscv.vfwcvt.rtz.xu.f.v\0"
  "llvm.riscv.vfwcvt.rtz.xu.f.v.mask\0"
  "llvm.riscv.vfwcvt.x.f.v\0"
  "llvm.riscv.vfwcvt.x.f.v.mask\0"
  "llvm.riscv.vfwcvt.xu.f.v\0"
  "llvm.riscv.vfwcvt.xu.f.v.mask\0"
  "llvm.riscv.vfwcvtbf16.f.f.v\0"
  "llvm.riscv.vfwcvtbf16.f.f.v.mask\0"
  "llvm.riscv.vfwmacc\0"
  "llvm.riscv.vfwmacc.mask\0"
  "llvm.riscv.vfwmaccbf16\0"
  "llvm.riscv.vfwmaccbf16.mask\0"
  "llvm.riscv.vfwmsac\0"
  "llvm.riscv.vfwmsac.mask\0"
  "llvm.riscv.vfwmul\0"
  "llvm.riscv.vfwmul.mask\0"
  "llvm.riscv.vfwnmacc\0"
  "llvm.riscv.vfwnmacc.mask\0"
  "llvm.riscv.vfwnmsac\0"
  "llvm.riscv.vfwnmsac.mask\0"
  "llvm.riscv.vfwredosum\0"
  "llvm.riscv.vfwredosum.mask\0"
  "llvm.riscv.vfwredusum\0"
  "llvm.riscv.vfwredusum.mask\0"
  "llvm.riscv.vfwsub\0"
  "llvm.riscv.vfwsub.mask\0"
  "llvm.riscv.vfwsub.w\0"
  "llvm.riscv.vfwsub.w.mask\0"
  "llvm.riscv.vghsh\0"
  "llvm.riscv.vgmul.vv\0"
  "llvm.riscv.vid\0"
  "llvm.riscv.vid.mask\0"
  "llvm.riscv.viota\0"
  "llvm.riscv.viota.mask\0"
  "llvm.riscv.vle\0"
  "llvm.riscv.vle.mask\0"
  "llvm.riscv.vleff\0"
  "llvm.riscv.vleff.mask\0"
  "llvm.riscv.vlm\0"
  "llvm.riscv.vloxei\0"
  "llvm.riscv.vloxei.mask\0"
  "llvm.riscv.vloxseg2\0"
  "llvm.riscv.vloxseg2.mask\0"
  "llvm.riscv.vloxseg3\0"
  "llvm.riscv.vloxseg3.mask\0"
  "llvm.riscv.vloxseg4\0"
  "llvm.riscv.vloxseg4.mask\0"
  "llvm.riscv.vloxseg5\0"
  "llvm.riscv.vloxseg5.mask\0"
  "llvm.riscv.vloxseg6\0"
  "llvm.riscv.vloxseg6.mask\0"
  "llvm.riscv.vloxseg7\0"
  "llvm.riscv.vloxseg7.mask\0"
  "llvm.riscv.vloxseg8\0"
  "llvm.riscv.vloxseg8.mask\0"
  "llvm.riscv.vlse\0"
  "llvm.riscv.vlse.mask\0"
  "llvm.riscv.vlseg2\0"
  "llvm.riscv.vlseg2.mask\0"
  "llvm.riscv.vlseg2ff\0"
  "llvm.riscv.vlseg2ff.mask\0"
  "llvm.riscv.vlseg3\0"
  "llvm.riscv.vlseg3.mask\0"
  "llvm.riscv.vlseg3ff\0"
  "llvm.riscv.vlseg3ff.mask\0"
  "llvm.riscv.vlseg4\0"
  "llvm.riscv.vlseg4.mask\0"
  "llvm.riscv.vlseg4ff\0"
  "llvm.riscv.vlseg4ff.mask\0"
  "llvm.riscv.vlseg5\0"
  "llvm.riscv.vlseg5.mask\0"
  "llvm.riscv.vlseg5ff\0"
  "llvm.riscv.vlseg5ff.mask\0"
  "llvm.riscv.vlseg6\0"
  "llvm.riscv.vlseg6.mask\0"
  "llvm.riscv.vlseg6ff\0"
  "llvm.riscv.vlseg6ff.mask\0"
  "llvm.riscv.vlseg7\0"
  "llvm.riscv.vlseg7.mask\0"
  "llvm.riscv.vlseg7ff\0"
  "llvm.riscv.vlseg7ff.mask\0"
  "llvm.riscv.vlseg8\0"
  "llvm.riscv.vlseg8.mask\0"
  "llvm.riscv.vlseg8ff\0"
  "llvm.riscv.vlseg8ff.mask\0"
  "llvm.riscv.vlsseg2\0"
  "llvm.riscv.vlsseg2.mask\0"
  "llvm.riscv.vlsseg3\0"
  "llvm.riscv.vlsseg3.mask\0"
  "llvm.riscv.vlsseg4\0"
  "llvm.riscv.vlsseg4.mask\0"
  "llvm.riscv.vlsseg5\0"
  "llvm.riscv.vlsseg5.mask\0"
  "llvm.riscv.vlsseg6\0"
  "llvm.riscv.vlsseg6.mask\0"
  "llvm.riscv.vlsseg7\0"
  "llvm.riscv.vlsseg7.mask\0"
  "llvm.riscv.vlsseg8\0"
  "llvm.riscv.vlsseg8.mask\0"
  "llvm.riscv.vluxei\0"
  "llvm.riscv.vluxei.mask\0"
  "llvm.riscv.vluxseg2\0"
  "llvm.riscv.vluxseg2.mask\0"
  "llvm.riscv.vluxseg3\0"
  "llvm.riscv.vluxseg3.mask\0"
  "llvm.riscv.vluxseg4\0"
  "llvm.riscv.vluxseg4.mask\0"
  "llvm.riscv.vluxseg5\0"
  "llvm.riscv.vluxseg5.mask\0"
  "llvm.riscv.vluxseg6\0"
  "llvm.riscv.vluxseg6.mask\0"
  "llvm.riscv.vluxseg7\0"
  "llvm.riscv.vluxseg7.mask\0"
  "llvm.riscv.vluxseg8\0"
  "llvm.riscv.vluxseg8.mask\0"
  "llvm.riscv.vmacc\0"
  "llvm.riscv.vmacc.mask\0"
  "llvm.riscv.vmadc\0"
  "llvm.riscv.vmadc.carry.in\0"
  "llvm.riscv.vmadd\0"
  "llvm.riscv.vmadd.mask\0"
  "llvm.riscv.vmand\0"
  "llvm.riscv.vmandn\0"
  "llvm.riscv.vmax\0"
  "llvm.riscv.vmax.mask\0"
  "llvm.riscv.vmaxu\0"
  "llvm.riscv.vmaxu.mask\0"
  "llvm.riscv.vmclr\0"
  "llvm.riscv.vmerge\0"
  "llvm.riscv.vmfeq\0"
  "llvm.riscv.vmfeq.mask\0"
  "llvm.riscv.vmfge\0"
  "llvm.riscv.vmfge.mask\0"
  "llvm.riscv.vmfgt\0"
  "llvm.riscv.vmfgt.mask\0"
  "llvm.riscv.vmfle\0"
  "llvm.riscv.vmfle.mask\0"
  "llvm.riscv.vmflt\0"
  "llvm.riscv.vmflt.mask\0"
  "llvm.riscv.vmfne\0"
  "llvm.riscv.vmfne.mask\0"
  "llvm.riscv.vmin\0"
  "llvm.riscv.vmin.mask\0"
  "llvm.riscv.vminu\0"
  "llvm.riscv.vminu.mask\0"
  "llvm.riscv.vmnand\0"
  "llvm.riscv.vmnor\0"
  "llvm.riscv.vmor\0"
  "llvm.riscv.vmorn\0"
  "llvm.riscv.vmsbc\0"
  "llvm.riscv.vmsbc.borrow.in\0"
  "llvm.riscv.vmsbf\0"
  "llvm.riscv.vmsbf.mask\0"
  "llvm.riscv.vmseq\0"
  "llvm.riscv.vmseq.mask\0"
  "llvm.riscv.vmset\0"
  "llvm.riscv.vmsge\0"
  "llvm.riscv.vmsge.mask\0"
  "llvm.riscv.vmsgeu\0"
  "llvm.riscv.vmsgeu.mask\0"
  "llvm.riscv.vmsgt\0"
  "llvm.riscv.vmsgt.mask\0"
  "llvm.riscv.vmsgtu\0"
  "llvm.riscv.vmsgtu.mask\0"
  "llvm.riscv.vmsif\0"
  "llvm.riscv.vmsif.mask\0"
  "llvm.riscv.vmsle\0"
  "llvm.riscv.vmsle.mask\0"
  "llvm.riscv.vmsleu\0"
  "llvm.riscv.vmsleu.mask\0"
  "llvm.riscv.vmslt\0"
  "llvm.riscv.vmslt.mask\0"
  "llvm.riscv.vmsltu\0"
  "llvm.riscv.vmsltu.mask\0"
  "llvm.riscv.vmsne\0"
  "llvm.riscv.vmsne.mask\0"
  "llvm.riscv.vmsof\0"
  "llvm.riscv.vmsof.mask\0"
  "llvm.riscv.vmul\0"
  "llvm.riscv.vmul.mask\0"
  "llvm.riscv.vmulh\0"
  "llvm.riscv.vmulh.mask\0"
  "llvm.riscv.vmulhsu\0"
  "llvm.riscv.vmulhsu.mask\0"
  "llvm.riscv.vmulhu\0"
  "llvm.riscv.vmulhu.mask\0"
  "llvm.riscv.vmv.s.x\0"
  "llvm.riscv.vmv.v.v\0"
  "llvm.riscv.vmv.v.x\0"
  "llvm.riscv.vmv.x.s\0"
  "llvm.riscv.vmxnor\0"
  "llvm.riscv.vmxor\0"
  "llvm.riscv.vnclip\0"
  "llvm.riscv.vnclip.mask\0"
  "llvm.riscv.vnclipu\0"
  "llvm.riscv.vnclipu.mask\0"
  "llvm.riscv.vnmsac\0"
  "llvm.riscv.vnmsac.mask\0"
  "llvm.riscv.vnmsub\0"
  "llvm.riscv.vnmsub.mask\0"
  "llvm.riscv.vnsra\0"
  "llvm.riscv.vnsra.mask\0"
  "llvm.riscv.vnsrl\0"
  "llvm.riscv.vnsrl.mask\0"
  "llvm.riscv.vor\0"
  "llvm.riscv.vor.mask\0"
  "llvm.riscv.vqdot\0"
  "llvm.riscv.vqdot.mask\0"
  "llvm.riscv.vqdotsu\0"
  "llvm.riscv.vqdotsu.mask\0"
  "llvm.riscv.vqdotu\0"
  "llvm.riscv.vqdotu.mask\0"
  "llvm.riscv.vqdotus\0"
  "llvm.riscv.vqdotus.mask\0"
  "llvm.riscv.vredand\0"
  "llvm.riscv.vredand.mask\0"
  "llvm.riscv.vredmax\0"
  "llvm.riscv.vredmax.mask\0"
  "llvm.riscv.vredmaxu\0"
  "llvm.riscv.vredmaxu.mask\0"
  "llvm.riscv.vredmin\0"
  "llvm.riscv.vredmin.mask\0"
  "llvm.riscv.vredminu\0"
  "llvm.riscv.vredminu.mask\0"
  "llvm.riscv.vredor\0"
  "llvm.riscv.vredor.mask\0"
  "llvm.riscv.vredsum\0"
  "llvm.riscv.vredsum.mask\0"
  "llvm.riscv.vredxor\0"
  "llvm.riscv.vredxor.mask\0"
  "llvm.riscv.vrem\0"
  "llvm.riscv.vrem.mask\0"
  "llvm.riscv.vremu\0"
  "llvm.riscv.vremu.mask\0"
  "llvm.riscv.vrev8\0"
  "llvm.riscv.vrev8.mask\0"
  "llvm.riscv.vrgather.vv\0"
  "llvm.riscv.vrgather.vv.mask\0"
  "llvm.riscv.vrgather.vx\0"
  "llvm.riscv.vrgather.vx.mask\0"
  "llvm.riscv.vrgatherei16.vv\0"
  "llvm.riscv.vrgatherei16.vv.mask\0"
  "llvm.riscv.vrol\0"
  "llvm.riscv.vrol.mask\0"
  "llvm.riscv.vror\0"
  "llvm.riscv.vror.mask\0"
  "llvm.riscv.vrsub\0"
  "llvm.riscv.vrsub.mask\0"
  "llvm.riscv.vsadd\0"
  "llvm.riscv.vsadd.mask\0"
  "llvm.riscv.vsaddu\0"
  "llvm.riscv.vsaddu.mask\0"
  "llvm.riscv.vsbc\0"
  "llvm.riscv.vse\0"
  "llvm.riscv.vse.mask\0"
  "llvm.riscv.vsetvli\0"
  "llvm.riscv.vsetvlimax\0"
  "llvm.riscv.vsext\0"
  "llvm.riscv.vsext.mask\0"
  "llvm.riscv.vsha2ch\0"
  "llvm.riscv.vsha2cl\0"
  "llvm.riscv.vsha2ms\0"
  "llvm.riscv.vslide1down\0"
  "llvm.riscv.vslide1down.mask\0"
  "llvm.riscv.vslide1up\0"
  "llvm.riscv.vslide1up.mask\0"
  "llvm.riscv.vslidedown\0"
  "llvm.riscv.vslidedown.mask\0"
  "llvm.riscv.vslideup\0"
  "llvm.riscv.vslideup.mask\0"
  "llvm.riscv.vsll\0"
  "llvm.riscv.vsll.mask\0"
  "llvm.riscv.vsm\0"
  "llvm.riscv.vsm3c\0"
  "llvm.riscv.vsm3me\0"
  "llvm.riscv.vsm4k\0"
  "llvm.riscv.vsm4r.vs\0"
  "llvm.riscv.vsm4r.vv\0"
  "llvm.riscv.vsmul\0"
  "llvm.riscv.vsmul.mask\0"
  "llvm.riscv.vsoxei\0"
  "llvm.riscv.vsoxei.mask\0"
  "llvm.riscv.vsoxseg2\0"
  "llvm.riscv.vsoxseg2.mask\0"
  "llvm.riscv.vsoxseg3\0"
  "llvm.riscv.vsoxseg3.mask\0"
  "llvm.riscv.vsoxseg4\0"
  "llvm.riscv.vsoxseg4.mask\0"
  "llvm.riscv.vsoxseg5\0"
  "llvm.riscv.vsoxseg5.mask\0"
  "llvm.riscv.vsoxseg6\0"
  "llvm.riscv.vsoxseg6.mask\0"
  "llvm.riscv.vsoxseg7\0"
  "llvm.riscv.vsoxseg7.mask\0"
  "llvm.riscv.vsoxseg8\0"
  "llvm.riscv.vsoxseg8.mask\0"
  "llvm.riscv.vsra\0"
  "llvm.riscv.vsra.mask\0"
  "llvm.riscv.vsrl\0"
  "llvm.riscv.vsrl.mask\0"
  "llvm.riscv.vsse\0"
  "llvm.riscv.vsse.mask\0"
  "llvm.riscv.vsseg2\0"
  "llvm.riscv.vsseg2.mask\0"
  "llvm.riscv.vsseg3\0"
  "llvm.riscv.vsseg3.mask\0"
  "llvm.riscv.vsseg4\0"
  "llvm.riscv.vsseg4.mask\0"
  "llvm.riscv.vsseg5\0"
  "llvm.riscv.vsseg5.mask\0"
  "llvm.riscv.vsseg6\0"
  "llvm.riscv.vsseg6.mask\0"
  "llvm.riscv.vsseg7\0"
  "llvm.riscv.vsseg7.mask\0"
  "llvm.riscv.vsseg8\0"
  "llvm.riscv.vsseg8.mask\0"
  "llvm.riscv.vssra\0"
  "llvm.riscv.vssra.mask\0"
  "llvm.riscv.vssrl\0"
  "llvm.riscv.vssrl.mask\0"
  "llvm.riscv.vssseg2\0"
  "llvm.riscv.vssseg2.mask\0"
  "llvm.riscv.vssseg3\0"
  "llvm.riscv.vssseg3.mask\0"
  "llvm.riscv.vssseg4\0"
  "llvm.riscv.vssseg4.mask\0"
  "llvm.riscv.vssseg5\0"
  "llvm.riscv.vssseg5.mask\0"
  "llvm.riscv.vssseg6\0"
  "llvm.riscv.vssseg6.mask\0"
  "llvm.riscv.vssseg7\0"
  "llvm.riscv.vssseg7.mask\0"
  "llvm.riscv.vssseg8\0"
  "llvm.riscv.vssseg8.mask\0"
  "llvm.riscv.vssub\0"
  "llvm.riscv.vssub.mask\0"
  "llvm.riscv.vssubu\0"
  "llvm.riscv.vssubu.mask\0"
  "llvm.riscv.vsub\0"
  "llvm.riscv.vsub.mask\0"
  "llvm.riscv.vsuxei\0"
  "llvm.riscv.vsuxei.mask\0"
  "llvm.riscv.vsuxseg2\0"
  "llvm.riscv.vsuxseg2.mask\0"
  "llvm.riscv.vsuxseg3\0"
  "llvm.riscv.vsuxseg3.mask\0"
  "llvm.riscv.vsuxseg4\0"
  "llvm.riscv.vsuxseg4.mask\0"
  "llvm.riscv.vsuxseg5\0"
  "llvm.riscv.vsuxseg5.mask\0"
  "llvm.riscv.vsuxseg6\0"
  "llvm.riscv.vsuxseg6.mask\0"
  "llvm.riscv.vsuxseg7\0"
  "llvm.riscv.vsuxseg7.mask\0"
  "llvm.riscv.vsuxseg8\0"
  "llvm.riscv.vsuxseg8.mask\0"
  "llvm.riscv.vwadd\0"
  "llvm.riscv.vwadd.mask\0"
  "llvm.riscv.vwadd.w\0"
  "llvm.riscv.vwadd.w.mask\0"
  "llvm.riscv.vwaddu\0"
  "llvm.riscv.vwaddu.mask\0"
  "llvm.riscv.vwaddu.w\0"
  "llvm.riscv.vwaddu.w.mask\0"
  "llvm.riscv.vwmacc\0"
  "llvm.riscv.vwmacc.mask\0"
  "llvm.riscv.vwmaccsu\0"
  "llvm.riscv.vwmaccsu.mask\0"
  "llvm.riscv.vwmaccu\0"
  "llvm.riscv.vwmaccu.mask\0"
  "llvm.riscv.vwmaccus\0"
  "llvm.riscv.vwmaccus.mask\0"
  "llvm.riscv.vwmul\0"
  "llvm.riscv.vwmul.mask\0"
  "llvm.riscv.vwmulsu\0"
  "llvm.riscv.vwmulsu.mask\0"
  "llvm.riscv.vwmulu\0"
  "llvm.riscv.vwmulu.mask\0"
  "llvm.riscv.vwredsum\0"
  "llvm.riscv.vwredsum.mask\0"
  "llvm.riscv.vwredsumu\0"
  "llvm.riscv.vwredsumu.mask\0"
  "llvm.riscv.vwsll\0"
  "llvm.riscv.vwsll.mask\0"
  "llvm.riscv.vwsub\0"
  "llvm.riscv.vwsub.mask\0"
  "llvm.riscv.vwsub.w\0"
  "llvm.riscv.vwsub.w.mask\0"
  "llvm.riscv.vwsubu\0"
  "llvm.riscv.vwsubu.mask\0"
  "llvm.riscv.vwsubu.w\0"
  "llvm.riscv.vwsubu.w.mask\0"
  "llvm.riscv.vxor\0"
  "llvm.riscv.vxor.mask\0"
  "llvm.riscv.vzext\0"
  "llvm.riscv.vzext.mask\0"
  "llvm.riscv.xperm4\0"
  "llvm.riscv.xperm8\0"
  "llvm.riscv.zip\0"
  "llvm.s390.bdepg\0"
  "llvm.s390.bextg\0"
  "llvm.s390.efpc\0"
  "llvm.s390.etnd\0"
  "llvm.s390.lcbb\0"
  "llvm.s390.ntstg\0"
  "llvm.s390.ppa.txassist\0"
  "llvm.s390.sfpc\0"
  "llvm.s390.tabort\0"
  "llvm.s390.tbegin\0"
  "llvm.s390.tbegin.nofloat\0"
  "llvm.s390.tbeginc\0"
  "llvm.s390.tdc\0"
  "llvm.s390.tend\0"
  "llvm.s390.vaccb\0"
  "llvm.s390.vacccq\0"
  "llvm.s390.vaccf\0"
  "llvm.s390.vaccg\0"
  "llvm.s390.vacch\0"
  "llvm.s390.vaccq\0"
  "llvm.s390.vacq\0"
  "llvm.s390.vaq\0"
  "llvm.s390.vavgb\0"
  "llvm.s390.vavgf\0"
  "llvm.s390.vavgg\0"
  "llvm.s390.vavgh\0"
  "llvm.s390.vavglb\0"
  "llvm.s390.vavglf\0"
  "llvm.s390.vavglg\0"
  "llvm.s390.vavglh\0"
  "llvm.s390.vavglq\0"
  "llvm.s390.vavgq\0"
  "llvm.s390.vbperm\0"
  "llvm.s390.vceqbs\0"
  "llvm.s390.vceqfs\0"
  "llvm.s390.vceqgs\0"
  "llvm.s390.vceqhs\0"
  "llvm.s390.vceqqs\0"
  "llvm.s390.vcfn\0"
  "llvm.s390.vchbs\0"
  "llvm.s390.vchfs\0"
  "llvm.s390.vchgs\0"
  "llvm.s390.vchhs\0"
  "llvm.s390.vchlbs\0"
  "llvm.s390.vchlfs\0"
  "llvm.s390.vchlgs\0"
  "llvm.s390.vchlhs\0"
  "llvm.s390.vchlqs\0"
  "llvm.s390.vchqs\0"
  "llvm.s390.vcksm\0"
  "llvm.s390.vclfnhs\0"
  "llvm.s390.vclfnls\0"
  "llvm.s390.vcnf\0"
  "llvm.s390.vcrnfs\0"
  "llvm.s390.verimb\0"
  "llvm.s390.verimf\0"
  "llvm.s390.verimg\0"
  "llvm.s390.verimh\0"
  "llvm.s390.veval\0"
  "llvm.s390.vfaeb\0"
  "llvm.s390.vfaebs\0"
  "llvm.s390.vfaef\0"
  "llvm.s390.vfaefs\0"
  "llvm.s390.vfaeh\0"
  "llvm.s390.vfaehs\0"
  "llvm.s390.vfaezb\0"
  "llvm.s390.vfaezbs\0"
  "llvm.s390.vfaezf\0"
  "llvm.s390.vfaezfs\0"
  "llvm.s390.vfaezh\0"
  "llvm.s390.vfaezhs\0"
  "llvm.s390.vfcedbs\0"
  "llvm.s390.vfcesbs\0"
  "llvm.s390.vfchdbs\0"
  "llvm.s390.vfchedbs\0"
  "llvm.s390.vfchesbs\0"
  "llvm.s390.vfchsbs\0"
  "llvm.s390.vfeeb\0"
  "llvm.s390.vfeebs\0"
  "llvm.s390.vfeef\0"
  "llvm.s390.vfeefs\0"
  "llvm.s390.vfeeh\0"
  "llvm.s390.vfeehs\0"
  "llvm.s390.vfeezb\0"
  "llvm.s390.vfeezbs\0"
  "llvm.s390.vfeezf\0"
  "llvm.s390.vfeezfs\0"
  "llvm.s390.vfeezh\0"
  "llvm.s390.vfeezhs\0"
  "llvm.s390.vfeneb\0"
  "llvm.s390.vfenebs\0"
  "llvm.s390.vfenef\0"
  "llvm.s390.vfenefs\0"
  "llvm.s390.vfeneh\0"
  "llvm.s390.vfenehs\0"
  "llvm.s390.vfenezb\0"
  "llvm.s390.vfenezbs\0"
  "llvm.s390.vfenezf\0"
  "llvm.s390.vfenezfs\0"
  "llvm.s390.vfenezh\0"
  "llvm.s390.vfenezhs\0"
  "llvm.s390.vfidb\0"
  "llvm.s390.vfisb\0"
  "llvm.s390.vfmaxdb\0"
  "llvm.s390.vfmaxsb\0"
  "llvm.s390.vfmindb\0"
  "llvm.s390.vfminsb\0"
  "llvm.s390.vftcidb\0"
  "llvm.s390.vftcisb\0"
  "llvm.s390.vgemb\0"
  "llvm.s390.vgemf\0"
  "llvm.s390.vgemg\0"
  "llvm.s390.vgemh\0"
  "llvm.s390.vgemq\0"
  "llvm.s390.vgfmab\0"
  "llvm.s390.vgfmaf\0"
  "llvm.s390.vgfmag\0"
  "llvm.s390.vgfmah\0"
  "llvm.s390.vgfmb\0"
  "llvm.s390.vgfmf\0"
  "llvm.s390.vgfmg\0"
  "llvm.s390.vgfmh\0"
  "llvm.s390.vistrb\0"
  "llvm.s390.vistrbs\0"
  "llvm.s390.vistrf\0"
  "llvm.s390.vistrfs\0"
  "llvm.s390.vistrh\0"
  "llvm.s390.vistrhs\0"
  "llvm.s390.vlbb\0"
  "llvm.s390.vll\0"
  "llvm.s390.vlrl\0"
  "llvm.s390.vmaeb\0"
  "llvm.s390.vmaef\0"
  "llvm.s390.vmaeg\0"
  "llvm.s390.vmaeh\0"
  "llvm.s390.vmahb\0"
  "llvm.s390.vmahf\0"
  "llvm.s390.vmahg\0"
  "llvm.s390.vmahh\0"
  "llvm.s390.vmahq\0"
  "llvm.s390.vmaleb\0"
  "llvm.s390.vmalef\0"
  "llvm.s390.vmaleg\0"
  "llvm.s390.vmaleh\0"
  "llvm.s390.vmalhb\0"
  "llvm.s390.vmalhf\0"
  "llvm.s390.vmalhg\0"
  "llvm.s390.vmalhh\0"
  "llvm.s390.vmalhq\0"
  "llvm.s390.vmalob\0"
  "llvm.s390.vmalof\0"
  "llvm.s390.vmalog\0"
  "llvm.s390.vmaloh\0"
  "llvm.s390.vmaob\0"
  "llvm.s390.vmaof\0"
  "llvm.s390.vmaog\0"
  "llvm.s390.vmaoh\0"
  "llvm.s390.vmeb\0"
  "llvm.s390.vmef\0"
  "llvm.s390.vmeg\0"
  "llvm.s390.vmeh\0"
  "llvm.s390.vmhb\0"
  "llvm.s390.vmhf\0"
  "llvm.s390.vmhg\0"
  "llvm.s390.vmhh\0"
  "llvm.s390.vmhq\0"
  "llvm.s390.vmleb\0"
  "llvm.s390.vmlef\0"
  "llvm.s390.vmleg\0"
  "llvm.s390.vmleh\0"
  "llvm.s390.vmlhb\0"
  "llvm.s390.vmlhf\0"
  "llvm.s390.vmlhg\0"
  "llvm.s390.vmlhh\0"
  "llvm.s390.vmlhq\0"
  "llvm.s390.vmlob\0"
  "llvm.s390.vmlof\0"
  "llvm.s390.vmlog\0"
  "llvm.s390.vmloh\0"
  "llvm.s390.vmob\0"
  "llvm.s390.vmof\0"
  "llvm.s390.vmog\0"
  "llvm.s390.vmoh\0"
  "llvm.s390.vmslg\0"
  "llvm.s390.vpdi\0"
  "llvm.s390.vperm\0"
  "llvm.s390.vpklsf\0"
  "llvm.s390.vpklsfs\0"
  "llvm.s390.vpklsg\0"
  "llvm.s390.vpklsgs\0"
  "llvm.s390.vpklsh\0"
  "llvm.s390.vpklshs\0"
  "llvm.s390.vpksf\0"
  "llvm.s390.vpksfs\0"
  "llvm.s390.vpksg\0"
  "llvm.s390.vpksgs\0"
  "llvm.s390.vpksh\0"
  "llvm.s390.vpkshs\0"
  "llvm.s390.vsbcbiq\0"
  "llvm.s390.vsbiq\0"
  "llvm.s390.vscbib\0"
  "llvm.s390.vscbif\0"
  "llvm.s390.vscbig\0"
  "llvm.s390.vscbih\0"
  "llvm.s390.vscbiq\0"
  "llvm.s390.vsl\0"
  "llvm.s390.vslb\0"
  "llvm.s390.vsld\0"
  "llvm.s390.vsldb\0"
  "llvm.s390.vsq\0"
  "llvm.s390.vsra\0"
  "llvm.s390.vsrab\0"
  "llvm.s390.vsrd\0"
  "llvm.s390.vsrl\0"
  "llvm.s390.vsrlb\0"
  "llvm.s390.vstl\0"
  "llvm.s390.vstrcb\0"
  "llvm.s390.vstrcbs\0"
  "llvm.s390.vstrcf\0"
  "llvm.s390.vstrcfs\0"
  "llvm.s390.vstrch\0"
  "llvm.s390.vstrchs\0"
  "llvm.s390.vstrczb\0"
  "llvm.s390.vstrczbs\0"
  "llvm.s390.vstrczf\0"
  "llvm.s390.vstrczfs\0"
  "llvm.s390.vstrczh\0"
  "llvm.s390.vstrczhs\0"
  "llvm.s390.vstrl\0"
  "llvm.s390.vstrsb\0"
  "llvm.s390.vstrsf\0"
  "llvm.s390.vstrsh\0"
  "llvm.s390.vstrszb\0"
  "llvm.s390.vstrszf\0"
  "llvm.s390.vstrszh\0"
  "llvm.s390.vsumb\0"
  "llvm.s390.vsumgf\0"
  "llvm.s390.vsumgh\0"
  "llvm.s390.vsumh\0"
  "llvm.s390.vsumqf\0"
  "llvm.s390.vsumqg\0"
  "llvm.s390.vtm\0"
  "llvm.s390.vuphb\0"
  "llvm.s390.vuphf\0"
  "llvm.s390.vuphg\0"
  "llvm.s390.vuphh\0"
  "llvm.s390.vuplb\0"
  "llvm.s390.vuplf\0"
  "llvm.s390.vuplg\0"
  "llvm.s390.vuplhb\0"
  "llvm.s390.vuplhf\0"
  "llvm.s390.vuplhg\0"
  "llvm.s390.vuplhh\0"
  "llvm.s390.vuplhw\0"
  "llvm.s390.vupllb\0"
  "llvm.s390.vupllf\0"
  "llvm.s390.vupllg\0"
  "llvm.s390.vupllh\0"
  "llvm.spv.all\0"
  "llvm.spv.alloca\0"
  "llvm.spv.alloca.array\0"
  "llvm.spv.any\0"
  "llvm.spv.assign.aliasing.decoration\0"
  "llvm.spv.assign.decoration\0"
  "llvm.spv.assign.fpmaxerror.decoration\0"
  "llvm.spv.assign.name\0"
  "llvm.spv.assign.ptr.type\0"
  "llvm.spv.assign.type\0"
  "llvm.spv.assume\0"
  "llvm.spv.bitcast\0"
  "llvm.spv.cmpxchg\0"
  "llvm.spv.const.composite\0"
  "llvm.spv.cross\0"
  "llvm.spv.ddx\0"
  "llvm.spv.ddx.coarse\0"
  "llvm.spv.ddx.fine\0"
  "llvm.spv.ddy\0"
  "llvm.spv.ddy.coarse\0"
  "llvm.spv.ddy.fine\0"
  "llvm.spv.degrees\0"
  "llvm.spv.discard\0"
  "llvm.spv.distance\0"
  "llvm.spv.dot4add.i8packed\0"
  "llvm.spv.dot4add.u8packed\0"
  "llvm.spv.expect\0"
  "llvm.spv.extractelt\0"
  "llvm.spv.extractv\0"
  "llvm.spv.faceforward\0"
  "llvm.spv.fdot\0"
  "llvm.spv.firstbitlow\0"
  "llvm.spv.firstbitshigh\0"
  "llvm.spv.firstbituhigh\0"
  "llvm.spv.flattened.thread.id.in.group\0"
  "llvm.spv.frac\0"
  "llvm.spv.fwidth\0"
  "llvm.spv.generic.cast.to.ptr.explicit\0"
  "llvm.spv.gep\0"
  "llvm.spv.global.offset\0"
  "llvm.spv.global.size\0"
  "llvm.spv.group.id\0"
  "llvm.spv.group.memory.barrier.with.group.sync\0"
  "llvm.spv.init.global\0"
  "llvm.spv.inline.asm\0"
  "llvm.spv.insertelt\0"
  "llvm.spv.insertv\0"
  "llvm.spv.isinf\0"
  "llvm.spv.isnan\0"
  "llvm.spv.length\0"
  "llvm.spv.lerp\0"
  "llvm.spv.lifetime.end\0"
  "llvm.spv.lifetime.start\0"
  "llvm.spv.load\0"
  "llvm.spv.loop.merge\0"
  "llvm.spv.nclamp\0"
  "llvm.spv.normalize\0"
  "llvm.spv.num.subgroups\0"
  "llvm.spv.num.workgroups\0"
  "llvm.spv.ptrcast\0"
  "llvm.spv.pushconstant.getpointer\0"
  "llvm.spv.radians\0"
  "llvm.spv.reflect\0"
  "llvm.spv.refract\0"
  "llvm.spv.resource.counterhandlefrombinding\0"
  "llvm.spv.resource.counterhandlefromimplicitbinding\0"
  "llvm.spv.resource.getdimensions.x\0"
  "llvm.spv.resource.getpointer\0"
  "llvm.spv.resource.handlefrombinding\0"
  "llvm.spv.resource.handlefromimplicitbinding\0"
  "llvm.spv.resource.load.typedbuffer\0"
  "llvm.spv.resource.nonuniformindex\0"
  "llvm.spv.resource.store.typedbuffer\0"
  "llvm.spv.resource.updatecounter\0"
  "llvm.spv.rsqrt\0"
  "llvm.spv.saturate\0"
  "llvm.spv.sclamp\0"
  "llvm.spv.sdot\0"
  "llvm.spv.selection.merge\0"
  "llvm.spv.sign\0"
  "llvm.spv.smoothstep\0"
  "llvm.spv.step\0"
  "llvm.spv.store\0"
  "llvm.spv.subgroup.id\0"
  "llvm.spv.subgroup.local.invocation.id\0"
  "llvm.spv.subgroup.max.size\0"
  "llvm.spv.subgroup.size\0"
  "llvm.spv.switch\0"
  "llvm.spv.thread.id\0"
  "llvm.spv.thread.id.in.group\0"
  "llvm.spv.track.constant\0"
  "llvm.spv.uclamp\0"
  "llvm.spv.udot\0"
  "llvm.spv.undef\0"
  "llvm.spv.unpackhalf2x16\0"
  "llvm.spv.unreachable\0"
  "llvm.spv.unref.global\0"
  "llvm.spv.value.md\0"
  "llvm.spv.wave.active.countbits\0"
  "llvm.spv.wave.all\0"
  "llvm.spv.wave.any\0"
  "llvm.spv.wave.ballot\0"
  "llvm.spv.wave.get.lane.count\0"
  "llvm.spv.wave.is.first.lane\0"
  "llvm.spv.wave.readlane\0"
  "llvm.spv.wave.reduce.max\0"
  "llvm.spv.wave.reduce.min\0"
  "llvm.spv.wave.reduce.sum\0"
  "llvm.spv.wave.reduce.umax\0"
  "llvm.spv.wave.reduce.umin\0"
  "llvm.spv.workgroup.size\0"
  "llvm.ve.vl.andm.MMM\0"
  "llvm.ve.vl.andm.mmm\0"
  "llvm.ve.vl.eqvm.MMM\0"
  "llvm.ve.vl.eqvm.mmm\0"
  "llvm.ve.vl.extract.vm512l\0"
  "llvm.ve.vl.extract.vm512u\0"
  "llvm.ve.vl.fencec.s\0"
  "llvm.ve.vl.fencei\0"
  "llvm.ve.vl.fencem.s\0"
  "llvm.ve.vl.fidcr.sss\0"
  "llvm.ve.vl.insert.vm512l\0"
  "llvm.ve.vl.insert.vm512u\0"
  "llvm.ve.vl.lcr.sss\0"
  "llvm.ve.vl.lsv.vvss\0"
  "llvm.ve.vl.lvm.MMss\0"
  "llvm.ve.vl.lvm.mmss\0"
  "llvm.ve.vl.lvsd.svs\0"
  "llvm.ve.vl.lvsl.svs\0"
  "llvm.ve.vl.lvss.svs\0"
  "llvm.ve.vl.lzvm.sml\0"
  "llvm.ve.vl.negm.MM\0"
  "llvm.ve.vl.negm.mm\0"
  "llvm.ve.vl.nndm.MMM\0"
  "llvm.ve.vl.nndm.mmm\0"
  "llvm.ve.vl.orm.MMM\0"
  "llvm.ve.vl.orm.mmm\0"
  "llvm.ve.vl.pack.f32a\0"
  "llvm.ve.vl.pack.f32p\0"
  "llvm.ve.vl.pcvm.sml\0"
  "llvm.ve.vl.pfchv.ssl\0"
  "llvm.ve.vl.pfchvnc.ssl\0"
  "llvm.ve.vl.pvadds.vsvMvl\0"
  "llvm.ve.vl.pvadds.vsvl\0"
  "llvm.ve.vl.pvadds.vsvvl\0"
  "llvm.ve.vl.pvadds.vvvMvl\0"
  "llvm.ve.vl.pvadds.vvvl\0"
  "llvm.ve.vl.pvadds.vvvvl\0"
  "llvm.ve.vl.pvaddu.vsvMvl\0"
  "llvm.ve.vl.pvaddu.vsvl\0"
  "llvm.ve.vl.pvaddu.vsvvl\0"
  "llvm.ve.vl.pvaddu.vvvMvl\0"
  "llvm.ve.vl.pvaddu.vvvl\0"
  "llvm.ve.vl.pvaddu.vvvvl\0"
  "llvm.ve.vl.pvand.vsvMvl\0"
  "llvm.ve.vl.pvand.vsvl\0"
  "llvm.ve.vl.pvand.vsvvl\0"
  "llvm.ve.vl.pvand.vvvMvl\0"
  "llvm.ve.vl.pvand.vvvl\0"
  "llvm.ve.vl.pvand.vvvvl\0"
  "llvm.ve.vl.pvbrd.vsMvl\0"
  "llvm.ve.vl.pvbrd.vsl\0"
  "llvm.ve.vl.pvbrd.vsvl\0"
  "llvm.ve.vl.pvbrv.vvMvl\0"
  "llvm.ve.vl.pvbrv.vvl\0"
  "llvm.ve.vl.pvbrv.vvvl\0"
  "llvm.ve.vl.pvbrvlo.vvl\0"
  "llvm.ve.vl.pvbrvlo.vvmvl\0"
  "llvm.ve.vl.pvbrvlo.vvvl\0"
  "llvm.ve.vl.pvbrvup.vvl\0"
  "llvm.ve.vl.pvbrvup.vvmvl\0"
  "llvm.ve.vl.pvbrvup.vvvl\0"
  "llvm.ve.vl.pvcmps.vsvMvl\0"
  "llvm.ve.vl.pvcmps.vsvl\0"
  "llvm.ve.vl.pvcmps.vsvvl\0"
  "llvm.ve.vl.pvcmps.vvvMvl\0"
  "llvm.ve.vl.pvcmps.vvvl\0"
  "llvm.ve.vl.pvcmps.vvvvl\0"
  "llvm.ve.vl.pvcmpu.vsvMvl\0"
  "llvm.ve.vl.pvcmpu.vsvl\0"
  "llvm.ve.vl.pvcmpu.vsvvl\0"
  "llvm.ve.vl.pvcmpu.vvvMvl\0"
  "llvm.ve.vl.pvcmpu.vvvl\0"
  "llvm.ve.vl.pvcmpu.vvvvl\0"
  "llvm.ve.vl.pvcvtsw.vvl\0"
  "llvm.ve.vl.pvcvtsw.vvvl\0"
  "llvm.ve.vl.pvcvtws.vvMvl\0"
  "llvm.ve.vl.pvcvtws.vvl\0"
  "llvm.ve.vl.pvcvtws.vvvl\0"
  "llvm.ve.vl.pvcvtwsrz.vvMvl\0"
  "llvm.ve.vl.pvcvtwsrz.vvl\0"
  "llvm.ve.vl.pvcvtwsrz.vvvl\0"
  "llvm.ve.vl.pveqv.vsvMvl\0"
  "llvm.ve.vl.pveqv.vsvl\0"
  "llvm.ve.vl.pveqv.vsvvl\0"
  "llvm.ve.vl.pveqv.vvvMvl\0"
  "llvm.ve.vl.pveqv.vvvl\0"
  "llvm.ve.vl.pveqv.vvvvl\0"
  "llvm.ve.vl.pvfadd.vsvMvl\0"
  "llvm.ve.vl.pvfadd.vsvl\0"
  "llvm.ve.vl.pvfadd.vsvvl\0"
  "llvm.ve.vl.pvfadd.vvvMvl\0"
  "llvm.ve.vl.pvfadd.vvvl\0"
  "llvm.ve.vl.pvfadd.vvvvl\0"
  "llvm.ve.vl.pvfcmp.vsvMvl\0"
  "llvm.ve.vl.pvfcmp.vsvl\0"
  "llvm.ve.vl.pvfcmp.vsvvl\0"
  "llvm.ve.vl.pvfcmp.vvvMvl\0"
  "llvm.ve.vl.pvfcmp.vvvl\0"
  "llvm.ve.vl.pvfcmp.vvvvl\0"
  "llvm.ve.vl.pvfmad.vsvvMvl\0"
  "llvm.ve.vl.pvfmad.vsvvl\0"
  "llvm.ve.vl.pvfmad.vsvvvl\0"
  "llvm.ve.vl.pvfmad.vvsvMvl\0"
  "llvm.ve.vl.pvfmad.vvsvl\0"
  "llvm.ve.vl.pvfmad.vvsvvl\0"
  "llvm.ve.vl.pvfmad.vvvvMvl\0"
  "llvm.ve.vl.pvfmad.vvvvl\0"
  "llvm.ve.vl.pvfmad.vvvvvl\0"
  "llvm.ve.vl.pvfmax.vsvMvl\0"
  "llvm.ve.vl.pvfmax.vsvl\0"
  "llvm.ve.vl.pvfmax.vsvvl\0"
  "llvm.ve.vl.pvfmax.vvvMvl\0"
  "llvm.ve.vl.pvfmax.vvvl\0"
  "llvm.ve.vl.pvfmax.vvvvl\0"
  "llvm.ve.vl.pvfmin.vsvMvl\0"
  "llvm.ve.vl.pvfmin.vsvl\0"
  "llvm.ve.vl.pvfmin.vsvvl\0"
  "llvm.ve.vl.pvfmin.vvvMvl\0"
  "llvm.ve.vl.pvfmin.vvvl\0"
  "llvm.ve.vl.pvfmin.vvvvl\0"
  "llvm.ve.vl.pvfmkaf.Ml\0"
  "llvm.ve.vl.pvfmkat.Ml\0"
  "llvm.ve.vl.pvfmkseq.MvMl\0"
  "llvm.ve.vl.pvfmkseq.Mvl\0"
  "llvm.ve.vl.pvfmkseqnan.MvMl\0"
  "llvm.ve.vl.pvfmkseqnan.Mvl\0"
  "llvm.ve.vl.pvfmksge.MvMl\0"
  "llvm.ve.vl.pvfmksge.Mvl\0"
  "llvm.ve.vl.pvfmksgenan.MvMl\0"
  "llvm.ve.vl.pvfmksgenan.Mvl\0"
  "llvm.ve.vl.pvfmksgt.MvMl\0"
  "llvm.ve.vl.pvfmksgt.Mvl\0"
  "llvm.ve.vl.pvfmksgtnan.MvMl\0"
  "llvm.ve.vl.pvfmksgtnan.Mvl\0"
  "llvm.ve.vl.pvfmksle.MvMl\0"
  "llvm.ve.vl.pvfmksle.Mvl\0"
  "llvm.ve.vl.pvfmkslenan.MvMl\0"
  "llvm.ve.vl.pvfmkslenan.Mvl\0"
  "llvm.ve.vl.pvfmksloeq.mvl\0"
  "llvm.ve.vl.pvfmksloeq.mvml\0"
  "llvm.ve.vl.pvfmksloeqnan.mvl\0"
  "llvm.ve.vl.pvfmksloeqnan.mvml\0"
  "llvm.ve.vl.pvfmksloge.mvl\0"
  "llvm.ve.vl.pvfmksloge.mvml\0"
  "llvm.ve.vl.pvfmkslogenan.mvl\0"
  "llvm.ve.vl.pvfmkslogenan.mvml\0"
  "llvm.ve.vl.pvfmkslogt.mvl\0"
  "llvm.ve.vl.pvfmkslogt.mvml\0"
  "llvm.ve.vl.pvfmkslogtnan.mvl\0"
  "llvm.ve.vl.pvfmkslogtnan.mvml\0"
  "llvm.ve.vl.pvfmkslole.mvl\0"
  "llvm.ve.vl.pvfmkslole.mvml\0"
  "llvm.ve.vl.pvfmkslolenan.mvl\0"
  "llvm.ve.vl.pvfmkslolenan.mvml\0"
  "llvm.ve.vl.pvfmkslolt.mvl\0"
  "llvm.ve.vl.pvfmkslolt.mvml\0"
  "llvm.ve.vl.pvfmksloltnan.mvl\0"
  "llvm.ve.vl.pvfmksloltnan.mvml\0"
  "llvm.ve.vl.pvfmkslonan.mvl\0"
  "llvm.ve.vl.pvfmkslonan.mvml\0"
  "llvm.ve.vl.pvfmkslone.mvl\0"
  "llvm.ve.vl.pvfmkslone.mvml\0"
  "llvm.ve.vl.pvfmkslonenan.mvl\0"
  "llvm.ve.vl.pvfmkslonenan.mvml\0"
  "llvm.ve.vl.pvfmkslonum.mvl\0"
  "llvm.ve.vl.pvfmkslonum.mvml\0"
  "llvm.ve.vl.pvfmkslt.MvMl\0"
  "llvm.ve.vl.pvfmkslt.Mvl\0"
  "llvm.ve.vl.pvfmksltnan.MvMl\0"
  "llvm.ve.vl.pvfmksltnan.Mvl\0"
  "llvm.ve.vl.pvfmksnan.MvMl\0"
  "llvm.ve.vl.pvfmksnan.Mvl\0"
  "llvm.ve.vl.pvfmksne.MvMl\0"
  "llvm.ve.vl.pvfmksne.Mvl\0"
  "llvm.ve.vl.pvfmksnenan.MvMl\0"
  "llvm.ve.vl.pvfmksnenan.Mvl\0"
  "llvm.ve.vl.pvfmksnum.MvMl\0"
  "llvm.ve.vl.pvfmksnum.Mvl\0"
  "llvm.ve.vl.pvfmksupeq.mvl\0"
  "llvm.ve.vl.pvfmksupeq.mvml\0"
  "llvm.ve.vl.pvfmksupeqnan.mvl\0"
  "llvm.ve.vl.pvfmksupeqnan.mvml\0"
  "llvm.ve.vl.pvfmksupge.mvl\0"
  "llvm.ve.vl.pvfmksupge.mvml\0"
  "llvm.ve.vl.pvfmksupgenan.mvl\0"
  "llvm.ve.vl.pvfmksupgenan.mvml\0"
  "llvm.ve.vl.pvfmksupgt.mvl\0"
  "llvm.ve.vl.pvfmksupgt.mvml\0"
  "llvm.ve.vl.pvfmksupgtnan.mvl\0"
  "llvm.ve.vl.pvfmksupgtnan.mvml\0"
  "llvm.ve.vl.pvfmksuple.mvl\0"
  "llvm.ve.vl.pvfmksuple.mvml\0"
  "llvm.ve.vl.pvfmksuplenan.mvl\0"
  "llvm.ve.vl.pvfmksuplenan.mvml\0"
  "llvm.ve.vl.pvfmksuplt.mvl\0"
  "llvm.ve.vl.pvfmksuplt.mvml\0"
  "llvm.ve.vl.pvfmksupltnan.mvl\0"
  "llvm.ve.vl.pvfmksupltnan.mvml\0"
  "llvm.ve.vl.pvfmksupnan.mvl\0"
  "llvm.ve.vl.pvfmksupnan.mvml\0"
  "llvm.ve.vl.pvfmksupne.mvl\0"
  "llvm.ve.vl.pvfmksupne.mvml\0"
  "llvm.ve.vl.pvfmksupnenan.mvl\0"
  "llvm.ve.vl.pvfmksupnenan.mvml\0"
  "llvm.ve.vl.pvfmksupnum.mvl\0"
  "llvm.ve.vl.pvfmksupnum.mvml\0"
  "llvm.ve.vl.pvfmkweq.MvMl\0"
  "llvm.ve.vl.pvfmkweq.Mvl\0"
  "llvm.ve.vl.pvfmkweqnan.MvMl\0"
  "llvm.ve.vl.pvfmkweqnan.Mvl\0"
  "llvm.ve.vl.pvfmkwge.MvMl\0"
  "llvm.ve.vl.pvfmkwge.Mvl\0"
  "llvm.ve.vl.pvfmkwgenan.MvMl\0"
  "llvm.ve.vl.pvfmkwgenan.Mvl\0"
  "llvm.ve.vl.pvfmkwgt.MvMl\0"
  "llvm.ve.vl.pvfmkwgt.Mvl\0"
  "llvm.ve.vl.pvfmkwgtnan.MvMl\0"
  "llvm.ve.vl.pvfmkwgtnan.Mvl\0"
  "llvm.ve.vl.pvfmkwle.MvMl\0"
  "llvm.ve.vl.pvfmkwle.Mvl\0"
  "llvm.ve.vl.pvfmkwlenan.MvMl\0"
  "llvm.ve.vl.pvfmkwlenan.Mvl\0"
  "llvm.ve.vl.pvfmkwloeq.mvl\0"
  "llvm.ve.vl.pvfmkwloeq.mvml\0"
  "llvm.ve.vl.pvfmkwloeqnan.mvl\0"
  "llvm.ve.vl.pvfmkwloeqnan.mvml\0"
  "llvm.ve.vl.pvfmkwloge.mvl\0"
  "llvm.ve.vl.pvfmkwloge.mvml\0"
  "llvm.ve.vl.pvfmkwlogenan.mvl\0"
  "llvm.ve.vl.pvfmkwlogenan.mvml\0"
  "llvm.ve.vl.pvfmkwlogt.mvl\0"
  "llvm.ve.vl.pvfmkwlogt.mvml\0"
  "llvm.ve.vl.pvfmkwlogtnan.mvl\0"
  "llvm.ve.vl.pvfmkwlogtnan.mvml\0"
  "llvm.ve.vl.pvfmkwlole.mvl\0"
  "llvm.ve.vl.pvfmkwlole.mvml\0"
  "llvm.ve.vl.pvfmkwlolenan.mvl\0"
  "llvm.ve.vl.pvfmkwlolenan.mvml\0"
  "llvm.ve.vl.pvfmkwlolt.mvl\0"
  "llvm.ve.vl.pvfmkwlolt.mvml\0"
  "llvm.ve.vl.pvfmkwloltnan.mvl\0"
  "llvm.ve.vl.pvfmkwloltnan.mvml\0"
  "llvm.ve.vl.pvfmkwlonan.mvl\0"
  "llvm.ve.vl.pvfmkwlonan.mvml\0"
  "llvm.ve.vl.pvfmkwlone.mvl\0"
  "llvm.ve.vl.pvfmkwlone.mvml\0"
  "llvm.ve.vl.pvfmkwlonenan.mvl\0"
  "llvm.ve.vl.pvfmkwlonenan.mvml\0"
  "llvm.ve.vl.pvfmkwlonum.mvl\0"
  "llvm.ve.vl.pvfmkwlonum.mvml\0"
  "llvm.ve.vl.pvfmkwlt.MvMl\0"
  "llvm.ve.vl.pvfmkwlt.Mvl\0"
  "llvm.ve.vl.pvfmkwltnan.MvMl\0"
  "llvm.ve.vl.pvfmkwltnan.Mvl\0"
  "llvm.ve.vl.pvfmkwnan.MvMl\0"
  "llvm.ve.vl.pvfmkwnan.Mvl\0"
  "llvm.ve.vl.pvfmkwne.MvMl\0"
  "llvm.ve.vl.pvfmkwne.Mvl\0"
  "llvm.ve.vl.pvfmkwnenan.MvMl\0"
  "llvm.ve.vl.pvfmkwnenan.Mvl\0"
  "llvm.ve.vl.pvfmkwnum.MvMl\0"
  "llvm.ve.vl.pvfmkwnum.Mvl\0"
  "llvm.ve.vl.pvfmkwupeq.mvl\0"
  "llvm.ve.vl.pvfmkwupeq.mvml\0"
  "llvm.ve.vl.pvfmkwupeqnan.mvl\0"
  "llvm.ve.vl.pvfmkwupeqnan.mvml\0"
  "llvm.ve.vl.pvfmkwupge.mvl\0"
  "llvm.ve.vl.pvfmkwupge.mvml\0"
  "llvm.ve.vl.pvfmkwupgenan.mvl\0"
  "llvm.ve.vl.pvfmkwupgenan.mvml\0"
  "llvm.ve.vl.pvfmkwupgt.mvl\0"
  "llvm.ve.vl.pvfmkwupgt.mvml\0"
  "llvm.ve.vl.pvfmkwupgtnan.mvl\0"
  "llvm.ve.vl.pvfmkwupgtnan.mvml\0"
  "llvm.ve.vl.pvfmkwuple.mvl\0"
  "llvm.ve.vl.pvfmkwuple.mvml\0"
  "llvm.ve.vl.pvfmkwuplenan.mvl\0"
  "llvm.ve.vl.pvfmkwuplenan.mvml\0"
  "llvm.ve.vl.pvfmkwuplt.mvl\0"
  "llvm.ve.vl.pvfmkwuplt.mvml\0"
  "llvm.ve.vl.pvfmkwupltnan.mvl\0"
  "llvm.ve.vl.pvfmkwupltnan.mvml\0"
  "llvm.ve.vl.pvfmkwupnan.mvl\0"
  "llvm.ve.vl.pvfmkwupnan.mvml\0"
  "llvm.ve.vl.pvfmkwupne.mvl\0"
  "llvm.ve.vl.pvfmkwupne.mvml\0"
  "llvm.ve.vl.pvfmkwupnenan.mvl\0"
  "llvm.ve.vl.pvfmkwupnenan.mvml\0"
  "llvm.ve.vl.pvfmkwupnum.mvl\0"
  "llvm.ve.vl.pvfmkwupnum.mvml\0"
  "llvm.ve.vl.pvfmsb.vsvvMvl\0"
  "llvm.ve.vl.pvfmsb.vsvvl\0"
  "llvm.ve.vl.pvfmsb.vsvvvl\0"
  "llvm.ve.vl.pvfmsb.vvsvMvl\0"
  "llvm.ve.vl.pvfmsb.vvsvl\0"
  "llvm.ve.vl.pvfmsb.vvsvvl\0"
  "llvm.ve.vl.pvfmsb.vvvvMvl\0"
  "llvm.ve.vl.pvfmsb.vvvvl\0"
  "llvm.ve.vl.pvfmsb.vvvvvl\0"
  "llvm.ve.vl.pvfmul.vsvMvl\0"
  "llvm.ve.vl.pvfmul.vsvl\0"
  "llvm.ve.vl.pvfmul.vsvvl\0"
  "llvm.ve.vl.pvfmul.vvvMvl\0"
  "llvm.ve.vl.pvfmul.vvvl\0"
  "llvm.ve.vl.pvfmul.vvvvl\0"
  "llvm.ve.vl.pvfnmad.vsvvMvl\0"
  "llvm.ve.vl.pvfnmad.vsvvl\0"
  "llvm.ve.vl.pvfnmad.vsvvvl\0"
  "llvm.ve.vl.pvfnmad.vvsvMvl\0"
  "llvm.ve.vl.pvfnmad.vvsvl\0"
  "llvm.ve.vl.pvfnmad.vvsvvl\0"
  "llvm.ve.vl.pvfnmad.vvvvMvl\0"
  "llvm.ve.vl.pvfnmad.vvvvl\0"
  "llvm.ve.vl.pvfnmad.vvvvvl\0"
  "llvm.ve.vl.pvfnmsb.vsvvMvl\0"
  "llvm.ve.vl.pvfnmsb.vsvvl\0"
  "llvm.ve.vl.pvfnmsb.vsvvvl\0"
  "llvm.ve.vl.pvfnmsb.vvsvMvl\0"
  "llvm.ve.vl.pvfnmsb.vvsvl\0"
  "llvm.ve.vl.pvfnmsb.vvsvvl\0"
  "llvm.ve.vl.pvfnmsb.vvvvMvl\0"
  "llvm.ve.vl.pvfnmsb.vvvvl\0"
  "llvm.ve.vl.pvfnmsb.vvvvvl\0"
  "llvm.ve.vl.pvfsub.vsvMvl\0"
  "llvm.ve.vl.pvfsub.vsvl\0"
  "llvm.ve.vl.pvfsub.vsvvl\0"
  "llvm.ve.vl.pvfsub.vvvMvl\0"
  "llvm.ve.vl.pvfsub.vvvl\0"
  "llvm.ve.vl.pvfsub.vvvvl\0"
  "llvm.ve.vl.pvldz.vvMvl\0"
  "llvm.ve.vl.pvldz.vvl\0"
  "llvm.ve.vl.pvldz.vvvl\0"
  "llvm.ve.vl.pvldzlo.vvl\0"
  "llvm.ve.vl.pvldzlo.vvmvl\0"
  "llvm.ve.vl.pvldzlo.vvvl\0"
  "llvm.ve.vl.pvldzup.vvl\0"
  "llvm.ve.vl.pvldzup.vvmvl\0"
  "llvm.ve.vl.pvldzup.vvvl\0"
  "llvm.ve.vl.pvmaxs.vsvMvl\0"
  "llvm.ve.vl.pvmaxs.vsvl\0"
  "llvm.ve.vl.pvmaxs.vsvvl\0"
  "llvm.ve.vl.pvmaxs.vvvMvl\0"
  "llvm.ve.vl.pvmaxs.vvvl\0"
  "llvm.ve.vl.pvmaxs.vvvvl\0"
  "llvm.ve.vl.pvmins.vsvMvl\0"
  "llvm.ve.vl.pvmins.vsvl\0"
  "llvm.ve.vl.pvmins.vsvvl\0"
  "llvm.ve.vl.pvmins.vvvMvl\0"
  "llvm.ve.vl.pvmins.vvvl\0"
  "llvm.ve.vl.pvmins.vvvvl\0"
  "llvm.ve.vl.pvor.vsvMvl\0"
  "llvm.ve.vl.pvor.vsvl\0"
  "llvm.ve.vl.pvor.vsvvl\0"
  "llvm.ve.vl.pvor.vvvMvl\0"
  "llvm.ve.vl.pvor.vvvl\0"
  "llvm.ve.vl.pvor.vvvvl\0"
  "llvm.ve.vl.pvpcnt.vvMvl\0"
  "llvm.ve.vl.pvpcnt.vvl\0"
  "llvm.ve.vl.pvpcnt.vvvl\0"
  "llvm.ve.vl.pvpcntlo.vvl\0"
  "llvm.ve.vl.pvpcntlo.vvmvl\0"
  "llvm.ve.vl.pvpcntlo.vvvl\0"
  "llvm.ve.vl.pvpcntup.vvl\0"
  "llvm.ve.vl.pvpcntup.vvmvl\0"
  "llvm.ve.vl.pvpcntup.vvvl\0"
  "llvm.ve.vl.pvrcp.vvl\0"
  "llvm.ve.vl.pvrcp.vvvl\0"
  "llvm.ve.vl.pvrsqrt.vvl\0"
  "llvm.ve.vl.pvrsqrt.vvvl\0"
  "llvm.ve.vl.pvrsqrtnex.vvl\0"
  "llvm.ve.vl.pvrsqrtnex.vvvl\0"
  "llvm.ve.vl.pvseq.vl\0"
  "llvm.ve.vl.pvseq.vvl\0"
  "llvm.ve.vl.pvseqlo.vl\0"
  "llvm.ve.vl.pvseqlo.vvl\0"
  "llvm.ve.vl.pvsequp.vl\0"
  "llvm.ve.vl.pvsequp.vvl\0"
  "llvm.ve.vl.pvsla.vvsMvl\0"
  "llvm.ve.vl.pvsla.vvsl\0"
  "llvm.ve.vl.pvsla.vvsvl\0"
  "llvm.ve.vl.pvsla.vvvMvl\0"
  "llvm.ve.vl.pvsla.vvvl\0"
  "llvm.ve.vl.pvsla.vvvvl\0"
  "llvm.ve.vl.pvsll.vvsMvl\0"
  "llvm.ve.vl.pvsll.vvsl\0"
  "llvm.ve.vl.pvsll.vvsvl\0"
  "llvm.ve.vl.pvsll.vvvMvl\0"
  "llvm.ve.vl.pvsll.vvvl\0"
  "llvm.ve.vl.pvsll.vvvvl\0"
  "llvm.ve.vl.pvsra.vvsMvl\0"
  "llvm.ve.vl.pvsra.vvsl\0"
  "llvm.ve.vl.pvsra.vvsvl\0"
  "llvm.ve.vl.pvsra.vvvMvl\0"
  "llvm.ve.vl.pvsra.vvvl\0"
  "llvm.ve.vl.pvsra.vvvvl\0"
  "llvm.ve.vl.pvsrl.vvsMvl\0"
  "llvm.ve.vl.pvsrl.vvsl\0"
  "llvm.ve.vl.pvsrl.vvsvl\0"
  "llvm.ve.vl.pvsrl.vvvMvl\0"
  "llvm.ve.vl.pvsrl.vvvl\0"
  "llvm.ve.vl.pvsrl.vvvvl\0"
  "llvm.ve.vl.pvsubs.vsvMvl\0"
  "llvm.ve.vl.pvsubs.vsvl\0"
  "llvm.ve.vl.pvsubs.vsvvl\0"
  "llvm.ve.vl.pvsubs.vvvMvl\0"
  "llvm.ve.vl.pvsubs.vvvl\0"
  "llvm.ve.vl.pvsubs.vvvvl\0"
  "llvm.ve.vl.pvsubu.vsvMvl\0"
  "llvm.ve.vl.pvsubu.vsvl\0"
  "llvm.ve.vl.pvsubu.vsvvl\0"
  "llvm.ve.vl.pvsubu.vvvMvl\0"
  "llvm.ve.vl.pvsubu.vvvl\0"
  "llvm.ve.vl.pvsubu.vvvvl\0"
  "llvm.ve.vl.pvxor.vsvMvl\0"
  "llvm.ve.vl.pvxor.vsvl\0"
  "llvm.ve.vl.pvxor.vsvvl\0"
  "llvm.ve.vl.pvxor.vvvMvl\0"
  "llvm.ve.vl.pvxor.vvvl\0"
  "llvm.ve.vl.pvxor.vvvvl\0"
  "llvm.ve.vl.scr.sss\0"
  "llvm.ve.vl.svm.sMs\0"
  "llvm.ve.vl.svm.sms\0"
  "llvm.ve.vl.svob\0"
  "llvm.ve.vl.tovm.sml\0"
  "llvm.ve.vl.tscr.ssss\0"
  "llvm.ve.vl.vaddsl.vsvl\0"
  "llvm.ve.vl.vaddsl.vsvmvl\0"
  "llvm.ve.vl.vaddsl.vsvvl\0"
  "llvm.ve.vl.vaddsl.vvvl\0"
  "llvm.ve.vl.vaddsl.vvvmvl\0"
  "llvm.ve.vl.vaddsl.vvvvl\0"
  "llvm.ve.vl.vaddswsx.vsvl\0"
  "llvm.ve.vl.vaddswsx.vsvmvl\0"
  "llvm.ve.vl.vaddswsx.vsvvl\0"
  "llvm.ve.vl.vaddswsx.vvvl\0"
  "llvm.ve.vl.vaddswsx.vvvmvl\0"
  "llvm.ve.vl.vaddswsx.vvvvl\0"
  "llvm.ve.vl.vaddswzx.vsvl\0"
  "llvm.ve.vl.vaddswzx.vsvmvl\0"
  "llvm.ve.vl.vaddswzx.vsvvl\0"
  "llvm.ve.vl.vaddswzx.vvvl\0"
  "llvm.ve.vl.vaddswzx.vvvmvl\0"
  "llvm.ve.vl.vaddswzx.vvvvl\0"
  "llvm.ve.vl.vaddul.vsvl\0"
  "llvm.ve.vl.vaddul.vsvmvl\0"
  "llvm.ve.vl.vaddul.vsvvl\0"
  "llvm.ve.vl.vaddul.vvvl\0"
  "llvm.ve.vl.vaddul.vvvmvl\0"
  "llvm.ve.vl.vaddul.vvvvl\0"
  "llvm.ve.vl.vadduw.vsvl\0"
  "llvm.ve.vl.vadduw.vsvmvl\0"
  "llvm.ve.vl.vadduw.vsvvl\0"
  "llvm.ve.vl.vadduw.vvvl\0"
  "llvm.ve.vl.vadduw.vvvmvl\0"
  "llvm.ve.vl.vadduw.vvvvl\0"
  "llvm.ve.vl.vand.vsvl\0"
  "llvm.ve.vl.vand.vsvmvl\0"
  "llvm.ve.vl.vand.vsvvl\0"
  "llvm.ve.vl.vand.vvvl\0"
  "llvm.ve.vl.vand.vvvmvl\0"
  "llvm.ve.vl.vand.vvvvl\0"
  "llvm.ve.vl.vbrdd.vsl\0"
  "llvm.ve.vl.vbrdd.vsmvl\0"
  "llvm.ve.vl.vbrdd.vsvl\0"
  "llvm.ve.vl.vbrdl.vsl\0"
  "llvm.ve.vl.vbrdl.vsmvl\0"
  "llvm.ve.vl.vbrdl.vsvl\0"
  "llvm.ve.vl.vbrds.vsl\0"
  "llvm.ve.vl.vbrds.vsmvl\0"
  "llvm.ve.vl.vbrds.vsvl\0"
  "llvm.ve.vl.vbrdw.vsl\0"
  "llvm.ve.vl.vbrdw.vsmvl\0"
  "llvm.ve.vl.vbrdw.vsvl\0"
  "llvm.ve.vl.vbrv.vvl\0"
  "llvm.ve.vl.vbrv.vvmvl\0"
  "llvm.ve.vl.vbrv.vvvl\0"
  "llvm.ve.vl.vcmpsl.vsvl\0"
  "llvm.ve.vl.vcmpsl.vsvmvl\0"
  "llvm.ve.vl.vcmpsl.vsvvl\0"
  "llvm.ve.vl.vcmpsl.vvvl\0"
  "llvm.ve.vl.vcmpsl.vvvmvl\0"
  "llvm.ve.vl.vcmpsl.vvvvl\0"
  "llvm.ve.vl.vcmpswsx.vsvl\0"
  "llvm.ve.vl.vcmpswsx.vsvmvl\0"
  "llvm.ve.vl.vcmpswsx.vsvvl\0"
  "llvm.ve.vl.vcmpswsx.vvvl\0"
  "llvm.ve.vl.vcmpswsx.vvvmvl\0"
  "llvm.ve.vl.vcmpswsx.vvvvl\0"
  "llvm.ve.vl.vcmpswzx.vsvl\0"
  "llvm.ve.vl.vcmpswzx.vsvmvl\0"
  "llvm.ve.vl.vcmpswzx.vsvvl\0"
  "llvm.ve.vl.vcmpswzx.vvvl\0"
  "llvm.ve.vl.vcmpswzx.vvvmvl\0"
  "llvm.ve.vl.vcmpswzx.vvvvl\0"
  "llvm.ve.vl.vcmpul.vsvl\0"
  "llvm.ve.vl.vcmpul.vsvmvl\0"
  "llvm.ve.vl.vcmpul.vsvvl\0"
  "llvm.ve.vl.vcmpul.vvvl\0"
  "llvm.ve.vl.vcmpul.vvvmvl\0"
  "llvm.ve.vl.vcmpul.vvvvl\0"
  "llvm.ve.vl.vcmpuw.vsvl\0"
  "llvm.ve.vl.vcmpuw.vsvmvl\0"
  "llvm.ve.vl.vcmpuw.vsvvl\0"
  "llvm.ve.vl.vcmpuw.vvvl\0"
  "llvm.ve.vl.vcmpuw.vvvmvl\0"
  "llvm.ve.vl.vcmpuw.vvvvl\0"
  "llvm.ve.vl.vcp.vvmvl\0"
  "llvm.ve.vl.vcvtdl.vvl\0"
  "llvm.ve.vl.vcvtdl.vvvl\0"
  "llvm.ve.vl.vcvtds.vvl\0"
  "llvm.ve.vl.vcvtds.vvvl\0"
  "llvm.ve.vl.vcvtdw.vvl\0"
  "llvm.ve.vl.vcvtdw.vvvl\0"
  "llvm.ve.vl.vcvtld.vvl\0"
  "llvm.ve.vl.vcvtld.vvmvl\0"
  "llvm.ve.vl.vcvtld.vvvl\0"
  "llvm.ve.vl.vcvtldrz.vvl\0"
  "llvm.ve.vl.vcvtldrz.vvmvl\0"
  "llvm.ve.vl.vcvtldrz.vvvl\0"
  "llvm.ve.vl.vcvtsd.vvl\0"
  "llvm.ve.vl.vcvtsd.vvvl\0"
  "llvm.ve.vl.vcvtsw.vvl\0"
  "llvm.ve.vl.vcvtsw.vvvl\0"
  "llvm.ve.vl.vcvtwdsx.vvl\0"
  "llvm.ve.vl.vcvtwdsx.vvmvl\0"
  "llvm.ve.vl.vcvtwdsx.vvvl\0"
  "llvm.ve.vl.vcvtwdsxrz.vvl\0"
  "llvm.ve.vl.vcvtwdsxrz.vvmvl\0"
  "llvm.ve.vl.vcvtwdsxrz.vvvl\0"
  "llvm.ve.vl.vcvtwdzx.vvl\0"
  "llvm.ve.vl.vcvtwdzx.vvmvl\0"
  "llvm.ve.vl.vcvtwdzx.vvvl\0"
  "llvm.ve.vl.vcvtwdzxrz.vvl\0"
  "llvm.ve.vl.vcvtwdzxrz.vvmvl\0"
  "llvm.ve.vl.vcvtwdzxrz.vvvl\0"
  "llvm.ve.vl.vcvtwssx.vvl\0"
  "llvm.ve.vl.vcvtwssx.vvmvl\0"
  "llvm.ve.vl.vcvtwssx.vvvl\0"
  "llvm.ve.vl.vcvtwssxrz.vvl\0"
  "llvm.ve.vl.vcvtwssxrz.vvmvl\0"
  "llvm.ve.vl.vcvtwssxrz.vvvl\0"
  "llvm.ve.vl.vcvtwszx.vvl\0"
  "llvm.ve.vl.vcvtwszx.vvmvl\0"
  "llvm.ve.vl.vcvtwszx.vvvl\0"
  "llvm.ve.vl.vcvtwszxrz.vvl\0"
  "llvm.ve.vl.vcvtwszxrz.vvmvl\0"
  "llvm.ve.vl.vcvtwszxrz.vvvl\0"
  "llvm.ve.vl.vdivsl.vsvl\0"
  "llvm.ve.vl.vdivsl.vsvmvl\0"
  "llvm.ve.vl.vdivsl.vsvvl\0"
  "llvm.ve.vl.vdivsl.vvsl\0"
  "llvm.ve.vl.vdivsl.vvsmvl\0"
  "llvm.ve.vl.vdivsl.vvsvl\0"
  "llvm.ve.vl.vdivsl.vvvl\0"
  "llvm.ve.vl.vdivsl.vvvmvl\0"
  "llvm.ve.vl.vdivsl.vvvvl\0"
  "llvm.ve.vl.vdivswsx.vsvl\0"
  "llvm.ve.vl.vdivswsx.vsvmvl\0"
  "llvm.ve.vl.vdivswsx.vsvvl\0"
  "llvm.ve.vl.vdivswsx.vvsl\0"
  "llvm.ve.vl.vdivswsx.vvsmvl\0"
  "llvm.ve.vl.vdivswsx.vvsvl\0"
  "llvm.ve.vl.vdivswsx.vvvl\0"
  "llvm.ve.vl.vdivswsx.vvvmvl\0"
  "llvm.ve.vl.vdivswsx.vvvvl\0"
  "llvm.ve.vl.vdivswzx.vsvl\0"
  "llvm.ve.vl.vdivswzx.vsvmvl\0"
  "llvm.ve.vl.vdivswzx.vsvvl\0"
  "llvm.ve.vl.vdivswzx.vvsl\0"
  "llvm.ve.vl.vdivswzx.vvsmvl\0"
  "llvm.ve.vl.vdivswzx.vvsvl\0"
  "llvm.ve.vl.vdivswzx.vvvl\0"
  "llvm.ve.vl.vdivswzx.vvvmvl\0"
  "llvm.ve.vl.vdivswzx.vvvvl\0"
  "llvm.ve.vl.vdivul.vsvl\0"
  "llvm.ve.vl.vdivul.vsvmvl\0"
  "llvm.ve.vl.vdivul.vsvvl\0"
  "llvm.ve.vl.vdivul.vvsl\0"
  "llvm.ve.vl.vdivul.vvsmvl\0"
  "llvm.ve.vl.vdivul.vvsvl\0"
  "llvm.ve.vl.vdivul.vvvl\0"
  "llvm.ve.vl.vdivul.vvvmvl\0"
  "llvm.ve.vl.vdivul.vvvvl\0"
  "llvm.ve.vl.vdivuw.vsvl\0"
  "llvm.ve.vl.vdivuw.vsvmvl\0"
  "llvm.ve.vl.vdivuw.vsvvl\0"
  "llvm.ve.vl.vdivuw.vvsl\0"
  "llvm.ve.vl.vdivuw.vvsmvl\0"
  "llvm.ve.vl.vdivuw.vvsvl\0"
  "llvm.ve.vl.vdivuw.vvvl\0"
  "llvm.ve.vl.vdivuw.vvvmvl\0"
  "llvm.ve.vl.vdivuw.vvvvl\0"
  "llvm.ve.vl.veqv.vsvl\0"
  "llvm.ve.vl.veqv.vsvmvl\0"
  "llvm.ve.vl.veqv.vsvvl\0"
  "llvm.ve.vl.veqv.vvvl\0"
  "llvm.ve.vl.veqv.vvvmvl\0"
  "llvm.ve.vl.veqv.vvvvl\0"
  "llvm.ve.vl.vex.vvmvl\0"
  "llvm.ve.vl.vfaddd.vsvl\0"
  "llvm.ve.vl.vfaddd.vsvmvl\0"
  "llvm.ve.vl.vfaddd.vsvvl\0"
  "llvm.ve.vl.vfaddd.vvvl\0"
  "llvm.ve.vl.vfaddd.vvvmvl\0"
  "llvm.ve.vl.vfaddd.vvvvl\0"
  "llvm.ve.vl.vfadds.vsvl\0"
  "llvm.ve.vl.vfadds.vsvmvl\0"
  "llvm.ve.vl.vfadds.vsvvl\0"
  "llvm.ve.vl.vfadds.vvvl\0"
  "llvm.ve.vl.vfadds.vvvmvl\0"
  "llvm.ve.vl.vfadds.vvvvl\0"
  "llvm.ve.vl.vfcmpd.vsvl\0"
  "llvm.ve.vl.vfcmpd.vsvmvl\0"
  "llvm.ve.vl.vfcmpd.vsvvl\0"
  "llvm.ve.vl.vfcmpd.vvvl\0"
  "llvm.ve.vl.vfcmpd.vvvmvl\0"
  "llvm.ve.vl.vfcmpd.vvvvl\0"
  "llvm.ve.vl.vfcmps.vsvl\0"
  "llvm.ve.vl.vfcmps.vsvmvl\0"
  "llvm.ve.vl.vfcmps.vsvvl\0"
  "llvm.ve.vl.vfcmps.vvvl\0"
  "llvm.ve.vl.vfcmps.vvvmvl\0"
  "llvm.ve.vl.vfcmps.vvvvl\0"
  "llvm.ve.vl.vfdivd.vsvl\0"
  "llvm.ve.vl.vfdivd.vsvmvl\0"
  "llvm.ve.vl.vfdivd.vsvvl\0"
  "llvm.ve.vl.vfdivd.vvvl\0"
  "llvm.ve.vl.vfdivd.vvvmvl\0"
  "llvm.ve.vl.vfdivd.vvvvl\0"
  "llvm.ve.vl.vfdivs.vsvl\0"
  "llvm.ve.vl.vfdivs.vsvmvl\0"
  "llvm.ve.vl.vfdivs.vsvvl\0"
  "llvm.ve.vl.vfdivs.vvvl\0"
  "llvm.ve.vl.vfdivs.vvvmvl\0"
  "llvm.ve.vl.vfdivs.vvvvl\0"
  "llvm.ve.vl.vfmadd.vsvvl\0"
  "llvm.ve.vl.vfmadd.vsvvmvl\0"
  "llvm.ve.vl.vfmadd.vsvvvl\0"
  "llvm.ve.vl.vfmadd.vvsvl\0"
  "llvm.ve.vl.vfmadd.vvsvmvl\0"
  "llvm.ve.vl.vfmadd.vvsvvl\0"
  "llvm.ve.vl.vfmadd.vvvvl\0"
  "llvm.ve.vl.vfmadd.vvvvmvl\0"
  "llvm.ve.vl.vfmadd.vvvvvl\0"
  "llvm.ve.vl.vfmads.vsvvl\0"
  "llvm.ve.vl.vfmads.vsvvmvl\0"
  "llvm.ve.vl.vfmads.vsvvvl\0"
  "llvm.ve.vl.vfmads.vvsvl\0"
  "llvm.ve.vl.vfmads.vvsvmvl\0"
  "llvm.ve.vl.vfmads.vvsvvl\0"
  "llvm.ve.vl.vfmads.vvvvl\0"
  "llvm.ve.vl.vfmads.vvvvmvl\0"
  "llvm.ve.vl.vfmads.vvvvvl\0"
  "llvm.ve.vl.vfmaxd.vsvl\0"
  "llvm.ve.vl.vfmaxd.vsvmvl\0"
  "llvm.ve.vl.vfmaxd.vsvvl\0"
  "llvm.ve.vl.vfmaxd.vvvl\0"
  "llvm.ve.vl.vfmaxd.vvvmvl\0"
  "llvm.ve.vl.vfmaxd.vvvvl\0"
  "llvm.ve.vl.vfmaxs.vsvl\0"
  "llvm.ve.vl.vfmaxs.vsvmvl\0"
  "llvm.ve.vl.vfmaxs.vsvvl\0"
  "llvm.ve.vl.vfmaxs.vvvl\0"
  "llvm.ve.vl.vfmaxs.vvvmvl\0"
  "llvm.ve.vl.vfmaxs.vvvvl\0"
  "llvm.ve.vl.vfmind.vsvl\0"
  "llvm.ve.vl.vfmind.vsvmvl\0"
  "llvm.ve.vl.vfmind.vsvvl\0"
  "llvm.ve.vl.vfmind.vvvl\0"
  "llvm.ve.vl.vfmind.vvvmvl\0"
  "llvm.ve.vl.vfmind.vvvvl\0"
  "llvm.ve.vl.vfmins.vsvl\0"
  "llvm.ve.vl.vfmins.vsvmvl\0"
  "llvm.ve.vl.vfmins.vsvvl\0"
  "llvm.ve.vl.vfmins.vvvl\0"
  "llvm.ve.vl.vfmins.vvvmvl\0"
  "llvm.ve.vl.vfmins.vvvvl\0"
  "llvm.ve.vl.vfmkdeq.mvl\0"
  "llvm.ve.vl.vfmkdeq.mvml\0"
  "llvm.ve.vl.vfmkdeqnan.mvl\0"
  "llvm.ve.vl.vfmkdeqnan.mvml\0"
  "llvm.ve.vl.vfmkdge.mvl\0"
  "llvm.ve.vl.vfmkdge.mvml\0"
  "llvm.ve.vl.vfmkdgenan.mvl\0"
  "llvm.ve.vl.vfmkdgenan.mvml\0"
  "llvm.ve.vl.vfmkdgt.mvl\0"
  "llvm.ve.vl.vfmkdgt.mvml\0"
  "llvm.ve.vl.vfmkdgtnan.mvl\0"
  "llvm.ve.vl.vfmkdgtnan.mvml\0"
  "llvm.ve.vl.vfmkdle.mvl\0"
  "llvm.ve.vl.vfmkdle.mvml\0"
  "llvm.ve.vl.vfmkdlenan.mvl\0"
  "llvm.ve.vl.vfmkdlenan.mvml\0"
  "llvm.ve.vl.vfmkdlt.mvl\0"
  "llvm.ve.vl.vfmkdlt.mvml\0"
  "llvm.ve.vl.vfmkdltnan.mvl\0"
  "llvm.ve.vl.vfmkdltnan.mvml\0"
  "llvm.ve.vl.vfmkdnan.mvl\0"
  "llvm.ve.vl.vfmkdnan.mvml\0"
  "llvm.ve.vl.vfmkdne.mvl\0"
  "llvm.ve.vl.vfmkdne.mvml\0"
  "llvm.ve.vl.vfmkdnenan.mvl\0"
  "llvm.ve.vl.vfmkdnenan.mvml\0"
  "llvm.ve.vl.vfmkdnum.mvl\0"
  "llvm.ve.vl.vfmkdnum.mvml\0"
  "llvm.ve.vl.vfmklaf.ml\0"
  "llvm.ve.vl.vfmklat.ml\0"
  "llvm.ve.vl.vfmkleq.mvl\0"
  "llvm.ve.vl.vfmkleq.mvml\0"
  "llvm.ve.vl.vfmkleqnan.mvl\0"
  "llvm.ve.vl.vfmkleqnan.mvml\0"
  "llvm.ve.vl.vfmklge.mvl\0"
  "llvm.ve.vl.vfmklge.mvml\0"
  "llvm.ve.vl.vfmklgenan.mvl\0"
  "llvm.ve.vl.vfmklgenan.mvml\0"
  "llvm.ve.vl.vfmklgt.mvl\0"
  "llvm.ve.vl.vfmklgt.mvml\0"
  "llvm.ve.vl.vfmklgtnan.mvl\0"
  "llvm.ve.vl.vfmklgtnan.mvml\0"
  "llvm.ve.vl.vfmklle.mvl\0"
  "llvm.ve.vl.vfmklle.mvml\0"
  "llvm.ve.vl.vfmkllenan.mvl\0"
  "llvm.ve.vl.vfmkllenan.mvml\0"
  "llvm.ve.vl.vfmkllt.mvl\0"
  "llvm.ve.vl.vfmkllt.mvml\0"
  "llvm.ve.vl.vfmklltnan.mvl\0"
  "llvm.ve.vl.vfmklltnan.mvml\0"
  "llvm.ve.vl.vfmklnan.mvl\0"
  "llvm.ve.vl.vfmklnan.mvml\0"
  "llvm.ve.vl.vfmklne.mvl\0"
  "llvm.ve.vl.vfmklne.mvml\0"
  "llvm.ve.vl.vfmklnenan.mvl\0"
  "llvm.ve.vl.vfmklnenan.mvml\0"
  "llvm.ve.vl.vfmklnum.mvl\0"
  "llvm.ve.vl.vfmklnum.mvml\0"
  "llvm.ve.vl.vfmkseq.mvl\0"
  "llvm.ve.vl.vfmkseq.mvml\0"
  "llvm.ve.vl.vfmkseqnan.mvl\0"
  "llvm.ve.vl.vfmkseqnan.mvml\0"
  "llvm.ve.vl.vfmksge.mvl\0"
  "llvm.ve.vl.vfmksge.mvml\0"
  "llvm.ve.vl.vfmksgenan.mvl\0"
  "llvm.ve.vl.vfmksgenan.mvml\0"
  "llvm.ve.vl.vfmksgt.mvl\0"
  "llvm.ve.vl.vfmksgt.mvml\0"
  "llvm.ve.vl.vfmksgtnan.mvl\0"
  "llvm.ve.vl.vfmksgtnan.mvml\0"
  "llvm.ve.vl.vfmksle.mvl\0"
  "llvm.ve.vl.vfmksle.mvml\0"
  "llvm.ve.vl.vfmkslenan.mvl\0"
  "llvm.ve.vl.vfmkslenan.mvml\0"
  "llvm.ve.vl.vfmkslt.mvl\0"
  "llvm.ve.vl.vfmkslt.mvml\0"
  "llvm.ve.vl.vfmksltnan.mvl\0"
  "llvm.ve.vl.vfmksltnan.mvml\0"
  "llvm.ve.vl.vfmksnan.mvl\0"
  "llvm.ve.vl.vfmksnan.mvml\0"
  "llvm.ve.vl.vfmksne.mvl\0"
  "llvm.ve.vl.vfmksne.mvml\0"
  "llvm.ve.vl.vfmksnenan.mvl\0"
  "llvm.ve.vl.vfmksnenan.mvml\0"
  "llvm.ve.vl.vfmksnum.mvl\0"
  "llvm.ve.vl.vfmksnum.mvml\0"
  "llvm.ve.vl.vfmkweq.mvl\0"
  "llvm.ve.vl.vfmkweq.mvml\0"
  "llvm.ve.vl.vfmkweqnan.mvl\0"
  "llvm.ve.vl.vfmkweqnan.mvml\0"
  "llvm.ve.vl.vfmkwge.mvl\0"
  "llvm.ve.vl.vfmkwge.mvml\0"
  "llvm.ve.vl.vfmkwgenan.mvl\0"
  "llvm.ve.vl.vfmkwgenan.mvml\0"
  "llvm.ve.vl.vfmkwgt.mvl\0"
  "llvm.ve.vl.vfmkwgt.mvml\0"
  "llvm.ve.vl.vfmkwgtnan.mvl\0"
  "llvm.ve.vl.vfmkwgtnan.mvml\0"
  "llvm.ve.vl.vfmkwle.mvl\0"
  "llvm.ve.vl.vfmkwle.mvml\0"
  "llvm.ve.vl.vfmkwlenan.mvl\0"
  "llvm.ve.vl.vfmkwlenan.mvml\0"
  "llvm.ve.vl.vfmkwlt.mvl\0"
  "llvm.ve.vl.vfmkwlt.mvml\0"
  "llvm.ve.vl.vfmkwltnan.mvl\0"
  "llvm.ve.vl.vfmkwltnan.mvml\0"
  "llvm.ve.vl.vfmkwnan.mvl\0"
  "llvm.ve.vl.vfmkwnan.mvml\0"
  "llvm.ve.vl.vfmkwne.mvl\0"
  "llvm.ve.vl.vfmkwne.mvml\0"
  "llvm.ve.vl.vfmkwnenan.mvl\0"
  "llvm.ve.vl.vfmkwnenan.mvml\0"
  "llvm.ve.vl.vfmkwnum.mvl\0"
  "llvm.ve.vl.vfmkwnum.mvml\0"
  "llvm.ve.vl.vfmsbd.vsvvl\0"
  "llvm.ve.vl.vfmsbd.vsvvmvl\0"
  "llvm.ve.vl.vfmsbd.vsvvvl\0"
  "llvm.ve.vl.vfmsbd.vvsvl\0"
  "llvm.ve.vl.vfmsbd.vvsvmvl\0"
  "llvm.ve.vl.vfmsbd.vvsvvl\0"
  "llvm.ve.vl.vfmsbd.vvvvl\0"
  "llvm.ve.vl.vfmsbd.vvvvmvl\0"
  "llvm.ve.vl.vfmsbd.vvvvvl\0"
  "llvm.ve.vl.vfmsbs.vsvvl\0"
  "llvm.ve.vl.vfmsbs.vsvvmvl\0"
  "llvm.ve.vl.vfmsbs.vsvvvl\0"
  "llvm.ve.vl.vfmsbs.vvsvl\0"
  "llvm.ve.vl.vfmsbs.vvsvmvl\0"
  "llvm.ve.vl.vfmsbs.vvsvvl\0"
  "llvm.ve.vl.vfmsbs.vvvvl\0"
  "llvm.ve.vl.vfmsbs.vvvvmvl\0"
  "llvm.ve.vl.vfmsbs.vvvvvl\0"
  "llvm.ve.vl.vfmuld.vsvl\0"
  "llvm.ve.vl.vfmuld.vsvmvl\0"
  "llvm.ve.vl.vfmuld.vsvvl\0"
  "llvm.ve.vl.vfmuld.vvvl\0"
  "llvm.ve.vl.vfmuld.vvvmvl\0"
  "llvm.ve.vl.vfmuld.vvvvl\0"
  "llvm.ve.vl.vfmuls.vsvl\0"
  "llvm.ve.vl.vfmuls.vsvmvl\0"
  "llvm.ve.vl.vfmuls.vsvvl\0"
  "llvm.ve.vl.vfmuls.vvvl\0"
  "llvm.ve.vl.vfmuls.vvvmvl\0"
  "llvm.ve.vl.vfmuls.vvvvl\0"
  "llvm.ve.vl.vfnmadd.vsvvl\0"
  "llvm.ve.vl.vfnmadd.vsvvmvl\0"
  "llvm.ve.vl.vfnmadd.vsvvvl\0"
  "llvm.ve.vl.vfnmadd.vvsvl\0"
  "llvm.ve.vl.vfnmadd.vvsvmvl\0"
  "llvm.ve.vl.vfnmadd.vvsvvl\0"
  "llvm.ve.vl.vfnmadd.vvvvl\0"
  "llvm.ve.vl.vfnmadd.vvvvmvl\0"
  "llvm.ve.vl.vfnmadd.vvvvvl\0"
  "llvm.ve.vl.vfnmads.vsvvl\0"
  "llvm.ve.vl.vfnmads.vsvvmvl\0"
  "llvm.ve.vl.vfnmads.vsvvvl\0"
  "llvm.ve.vl.vfnmads.vvsvl\0"
  "llvm.ve.vl.vfnmads.vvsvmvl\0"
  "llvm.ve.vl.vfnmads.vvsvvl\0"
  "llvm.ve.vl.vfnmads.vvvvl\0"
  "llvm.ve.vl.vfnmads.vvvvmvl\0"
  "llvm.ve.vl.vfnmads.vvvvvl\0"
  "llvm.ve.vl.vfnmsbd.vsvvl\0"
  "llvm.ve.vl.vfnmsbd.vsvvmvl\0"
  "llvm.ve.vl.vfnmsbd.vsvvvl\0"
  "llvm.ve.vl.vfnmsbd.vvsvl\0"
  "llvm.ve.vl.vfnmsbd.vvsvmvl\0"
  "llvm.ve.vl.vfnmsbd.vvsvvl\0"
  "llvm.ve.vl.vfnmsbd.vvvvl\0"
  "llvm.ve.vl.vfnmsbd.vvvvmvl\0"
  "llvm.ve.vl.vfnmsbd.vvvvvl\0"
  "llvm.ve.vl.vfnmsbs.vsvvl\0"
  "llvm.ve.vl.vfnmsbs.vsvvmvl\0"
  "llvm.ve.vl.vfnmsbs.vsvvvl\0"
  "llvm.ve.vl.vfnmsbs.vvsvl\0"
  "llvm.ve.vl.vfnmsbs.vvsvmvl\0"
  "llvm.ve.vl.vfnmsbs.vvsvvl\0"
  "llvm.ve.vl.vfnmsbs.vvvvl\0"
  "llvm.ve.vl.vfnmsbs.vvvvmvl\0"
  "llvm.ve.vl.vfnmsbs.vvvvvl\0"
  "llvm.ve.vl.vfrmaxdfst.vvl\0"
  "llvm.ve.vl.vfrmaxdfst.vvvl\0"
  "llvm.ve.vl.vfrmaxdlst.vvl\0"
  "llvm.ve.vl.vfrmaxdlst.vvvl\0"
  "llvm.ve.vl.vfrmaxsfst.vvl\0"
  "llvm.ve.vl.vfrmaxsfst.vvvl\0"
  "llvm.ve.vl.vfrmaxslst.vvl\0"
  "llvm.ve.vl.vfrmaxslst.vvvl\0"
  "llvm.ve.vl.vfrmindfst.vvl\0"
  "llvm.ve.vl.vfrmindfst.vvvl\0"
  "llvm.ve.vl.vfrmindlst.vvl\0"
  "llvm.ve.vl.vfrmindlst.vvvl\0"
  "llvm.ve.vl.vfrminsfst.vvl\0"
  "llvm.ve.vl.vfrminsfst.vvvl\0"
  "llvm.ve.vl.vfrminslst.vvl\0"
  "llvm.ve.vl.vfrminslst.vvvl\0"
  "llvm.ve.vl.vfsqrtd.vvl\0"
  "llvm.ve.vl.vfsqrtd.vvvl\0"
  "llvm.ve.vl.vfsqrts.vvl\0"
  "llvm.ve.vl.vfsqrts.vvvl\0"
  "llvm.ve.vl.vfsubd.vsvl\0"
  "llvm.ve.vl.vfsubd.vsvmvl\0"
  "llvm.ve.vl.vfsubd.vsvvl\0"
  "llvm.ve.vl.vfsubd.vvvl\0"
  "llvm.ve.vl.vfsubd.vvvmvl\0"
  "llvm.ve.vl.vfsubd.vvvvl\0"
  "llvm.ve.vl.vfsubs.vsvl\0"
  "llvm.ve.vl.vfsubs.vsvmvl\0"
  "llvm.ve.vl.vfsubs.vsvvl\0"
  "llvm.ve.vl.vfsubs.vvvl\0"
  "llvm.ve.vl.vfsubs.vvvmvl\0"
  "llvm.ve.vl.vfsubs.vvvvl\0"
  "llvm.ve.vl.vfsumd.vvl\0"
  "llvm.ve.vl.vfsumd.vvml\0"
  "llvm.ve.vl.vfsums.vvl\0"
  "llvm.ve.vl.vfsums.vvml\0"
  "llvm.ve.vl.vgt.vvssl\0"
  "llvm.ve.vl.vgt.vvssml\0"
  "llvm.ve.vl.vgt.vvssmvl\0"
  "llvm.ve.vl.vgt.vvssvl\0"
  "llvm.ve.vl.vgtlsx.vvssl\0"
  "llvm.ve.vl.vgtlsx.vvssml\0"
  "llvm.ve.vl.vgtlsx.vvssmvl\0"
  "llvm.ve.vl.vgtlsx.vvssvl\0"
  "llvm.ve.vl.vgtlsxnc.vvssl\0"
  "llvm.ve.vl.vgtlsxnc.vvssml\0"
  "llvm.ve.vl.vgtlsxnc.vvssmvl\0"
  "llvm.ve.vl.vgtlsxnc.vvssvl\0"
  "llvm.ve.vl.vgtlzx.vvssl\0"
  "llvm.ve.vl.vgtlzx.vvssml\0"
  "llvm.ve.vl.vgtlzx.vvssmvl\0"
  "llvm.ve.vl.vgtlzx.vvssvl\0"
  "llvm.ve.vl.vgtlzxnc.vvssl\0"
  "llvm.ve.vl.vgtlzxnc.vvssml\0"
  "llvm.ve.vl.vgtlzxnc.vvssmvl\0"
  "llvm.ve.vl.vgtlzxnc.vvssvl\0"
  "llvm.ve.vl.vgtnc.vvssl\0"
  "llvm.ve.vl.vgtnc.vvssml\0"
  "llvm.ve.vl.vgtnc.vvssmvl\0"
  "llvm.ve.vl.vgtnc.vvssvl\0"
  "llvm.ve.vl.vgtu.vvssl\0"
  "llvm.ve.vl.vgtu.vvssml\0"
  "llvm.ve.vl.vgtu.vvssmvl\0"
  "llvm.ve.vl.vgtu.vvssvl\0"
  "llvm.ve.vl.vgtunc.vvssl\0"
  "llvm.ve.vl.vgtunc.vvssml\0"
  "llvm.ve.vl.vgtunc.vvssmvl\0"
  "llvm.ve.vl.vgtunc.vvssvl\0"
  "llvm.ve.vl.vld.vssl\0"
  "llvm.ve.vl.vld.vssvl\0"
  "llvm.ve.vl.vld2d.vssl\0"
  "llvm.ve.vl.vld2d.vssvl\0"
  "llvm.ve.vl.vld2dnc.vssl\0"
  "llvm.ve.vl.vld2dnc.vssvl\0"
  "llvm.ve.vl.vldl2dsx.vssl\0"
  "llvm.ve.vl.vldl2dsx.vssvl\0"
  "llvm.ve.vl.vldl2dsxnc.vssl\0"
  "llvm.ve.vl.vldl2dsxnc.vssvl\0"
  "llvm.ve.vl.vldl2dzx.vssl\0"
  "llvm.ve.vl.vldl2dzx.vssvl\0"
  "llvm.ve.vl.vldl2dzxnc.vssl\0"
  "llvm.ve.vl.vldl2dzxnc.vssvl\0"
  "llvm.ve.vl.vldlsx.vssl\0"
  "llvm.ve.vl.vldlsx.vssvl\0"
  "llvm.ve.vl.vldlsxnc.vssl\0"
  "llvm.ve.vl.vldlsxnc.vssvl\0"
  "llvm.ve.vl.vldlzx.vssl\0"
  "llvm.ve.vl.vldlzx.vssvl\0"
  "llvm.ve.vl.vldlzxnc.vssl\0"
  "llvm.ve.vl.vldlzxnc.vssvl\0"
  "llvm.ve.vl.vldnc.vssl\0"
  "llvm.ve.vl.vldnc.vssvl\0"
  "llvm.ve.vl.vldu.vssl\0"
  "llvm.ve.vl.vldu.vssvl\0"
  "llvm.ve.vl.vldu2d.vssl\0"
  "llvm.ve.vl.vldu2d.vssvl\0"
  "llvm.ve.vl.vldu2dnc.vssl\0"
  "llvm.ve.vl.vldu2dnc.vssvl\0"
  "llvm.ve.vl.vldunc.vssl\0"
  "llvm.ve.vl.vldunc.vssvl\0"
  "llvm.ve.vl.vldz.vvl\0"
  "llvm.ve.vl.vldz.vvmvl\0"
  "llvm.ve.vl.vldz.vvvl\0"
  "llvm.ve.vl.vmaxsl.vsvl\0"
  "llvm.ve.vl.vmaxsl.vsvmvl\0"
  "llvm.ve.vl.vmaxsl.vsvvl\0"
  "llvm.ve.vl.vmaxsl.vvvl\0"
  "llvm.ve.vl.vmaxsl.vvvmvl\0"
  "llvm.ve.vl.vmaxsl.vvvvl\0"
  "llvm.ve.vl.vmaxswsx.vsvl\0"
  "llvm.ve.vl.vmaxswsx.vsvmvl\0"
  "llvm.ve.vl.vmaxswsx.vsvvl\0"
  "llvm.ve.vl.vmaxswsx.vvvl\0"
  "llvm.ve.vl.vmaxswsx.vvvmvl\0"
  "llvm.ve.vl.vmaxswsx.vvvvl\0"
  "llvm.ve.vl.vmaxswzx.vsvl\0"
  "llvm.ve.vl.vmaxswzx.vsvmvl\0"
  "llvm.ve.vl.vmaxswzx.vsvvl\0"
  "llvm.ve.vl.vmaxswzx.vvvl\0"
  "llvm.ve.vl.vmaxswzx.vvvmvl\0"
  "llvm.ve.vl.vmaxswzx.vvvvl\0"
  "llvm.ve.vl.vminsl.vsvl\0"
  "llvm.ve.vl.vminsl.vsvmvl\0"
  "llvm.ve.vl.vminsl.vsvvl\0"
  "llvm.ve.vl.vminsl.vvvl\0"
  "llvm.ve.vl.vminsl.vvvmvl\0"
  "llvm.ve.vl.vminsl.vvvvl\0"
  "llvm.ve.vl.vminswsx.vsvl\0"
  "llvm.ve.vl.vminswsx.vsvmvl\0"
  "llvm.ve.vl.vminswsx.vsvvl\0"
  "llvm.ve.vl.vminswsx.vvvl\0"
  "llvm.ve.vl.vminswsx.vvvmvl\0"
  "llvm.ve.vl.vminswsx.vvvvl\0"
  "llvm.ve.vl.vminswzx.vsvl\0"
  "llvm.ve.vl.vminswzx.vsvmvl\0"
  "llvm.ve.vl.vminswzx.vsvvl\0"
  "llvm.ve.vl.vminswzx.vvvl\0"
  "llvm.ve.vl.vminswzx.vvvmvl\0"
  "llvm.ve.vl.vminswzx.vvvvl\0"
  "llvm.ve.vl.vmrg.vsvml\0"
  "llvm.ve.vl.vmrg.vsvmvl\0"
  "llvm.ve.vl.vmrg.vvvml\0"
  "llvm.ve.vl.vmrg.vvvmvl\0"
  "llvm.ve.vl.vmrgw.vsvMl\0"
  "llvm.ve.vl.vmrgw.vsvMvl\0"
  "llvm.ve.vl.vmrgw.vvvMl\0"
  "llvm.ve.vl.vmrgw.vvvMvl\0"
  "llvm.ve.vl.vmulsl.vsvl\0"
  "llvm.ve.vl.vmulsl.vsvmvl\0"
  "llvm.ve.vl.vmulsl.vsvvl\0"
  "llvm.ve.vl.vmulsl.vvvl\0"
  "llvm.ve.vl.vmulsl.vvvmvl\0"
  "llvm.ve.vl.vmulsl.vvvvl\0"
  "llvm.ve.vl.vmulslw.vsvl\0"
  "llvm.ve.vl.vmulslw.vsvvl\0"
  "llvm.ve.vl.vmulslw.vvvl\0"
  "llvm.ve.vl.vmulslw.vvvvl\0"
  "llvm.ve.vl.vmulswsx.vsvl\0"
  "llvm.ve.vl.vmulswsx.vsvmvl\0"
  "llvm.ve.vl.vmulswsx.vsvvl\0"
  "llvm.ve.vl.vmulswsx.vvvl\0"
  "llvm.ve.vl.vmulswsx.vvvmvl\0"
  "llvm.ve.vl.vmulswsx.vvvvl\0"
  "llvm.ve.vl.vmulswzx.vsvl\0"
  "llvm.ve.vl.vmulswzx.vsvmvl\0"
  "llvm.ve.vl.vmulswzx.vsvvl\0"
  "llvm.ve.vl.vmulswzx.vvvl\0"
  "llvm.ve.vl.vmulswzx.vvvmvl\0"
  "llvm.ve.vl.vmulswzx.vvvvl\0"
  "llvm.ve.vl.vmulul.vsvl\0"
  "llvm.ve.vl.vmulul.vsvmvl\0"
  "llvm.ve.vl.vmulul.vsvvl\0"
  "llvm.ve.vl.vmulul.vvvl\0"
  "llvm.ve.vl.vmulul.vvvmvl\0"
  "llvm.ve.vl.vmulul.vvvvl\0"
  "llvm.ve.vl.vmuluw.vsvl\0"
  "llvm.ve.vl.vmuluw.vsvmvl\0"
  "llvm.ve.vl.vmuluw.vsvvl\0"
  "llvm.ve.vl.vmuluw.vvvl\0"
  "llvm.ve.vl.vmuluw.vvvmvl\0"
  "llvm.ve.vl.vmuluw.vvvvl\0"
  "llvm.ve.vl.vmv.vsvl\0"
  "llvm.ve.vl.vmv.vsvmvl\0"
  "llvm.ve.vl.vmv.vsvvl\0"
  "llvm.ve.vl.vor.vsvl\0"
  "llvm.ve.vl.vor.vsvmvl\0"
  "llvm.ve.vl.vor.vsvvl\0"
  "llvm.ve.vl.vor.vvvl\0"
  "llvm.ve.vl.vor.vvvmvl\0"
  "llvm.ve.vl.vor.vvvvl\0"
  "llvm.ve.vl.vpcnt.vvl\0"
  "llvm.ve.vl.vpcnt.vvmvl\0"
  "llvm.ve.vl.vpcnt.vvvl\0"
  "llvm.ve.vl.vrand.vvl\0"
  "llvm.ve.vl.vrand.vvml\0"
  "llvm.ve.vl.vrcpd.vvl\0"
  "llvm.ve.vl.vrcpd.vvvl\0"
  "llvm.ve.vl.vrcps.vvl\0"
  "llvm.ve.vl.vrcps.vvvl\0"
  "llvm.ve.vl.vrmaxslfst.vvl\0"
  "llvm.ve.vl.vrmaxslfst.vvvl\0"
  "llvm.ve.vl.vrmaxsllst.vvl\0"
  "llvm.ve.vl.vrmaxsllst.vvvl\0"
  "llvm.ve.vl.vrmaxswfstsx.vvl\0"
  "llvm.ve.vl.vrmaxswfstsx.vvvl\0"
  "llvm.ve.vl.vrmaxswfstzx.vvl\0"
  "llvm.ve.vl.vrmaxswfstzx.vvvl\0"
  "llvm.ve.vl.vrmaxswlstsx.vvl\0"
  "llvm.ve.vl.vrmaxswlstsx.vvvl\0"
  "llvm.ve.vl.vrmaxswlstzx.vvl\0"
  "llvm.ve.vl.vrmaxswlstzx.vvvl\0"
  "llvm.ve.vl.vrminslfst.vvl\0"
  "llvm.ve.vl.vrminslfst.vvvl\0"
  "llvm.ve.vl.vrminsllst.vvl\0"
  "llvm.ve.vl.vrminsllst.vvvl\0"
  "llvm.ve.vl.vrminswfstsx.vvl\0"
  "llvm.ve.vl.vrminswfstsx.vvvl\0"
  "llvm.ve.vl.vrminswfstzx.vvl\0"
  "llvm.ve.vl.vrminswfstzx.vvvl\0"
  "llvm.ve.vl.vrminswlstsx.vvl\0"
  "llvm.ve.vl.vrminswlstsx.vvvl\0"
  "llvm.ve.vl.vrminswlstzx.vvl\0"
  "llvm.ve.vl.vrminswlstzx.vvvl\0"
  "llvm.ve.vl.vror.vvl\0"
  "llvm.ve.vl.vror.vvml\0"
  "llvm.ve.vl.vrsqrtd.vvl\0"
  "llvm.ve.vl.vrsqrtd.vvvl\0"
  "llvm.ve.vl.vrsqrtdnex.vvl\0"
  "llvm.ve.vl.vrsqrtdnex.vvvl\0"
  "llvm.ve.vl.vrsqrts.vvl\0"
  "llvm.ve.vl.vrsqrts.vvvl\0"
  "llvm.ve.vl.vrsqrtsnex.vvl\0"
  "llvm.ve.vl.vrsqrtsnex.vvvl\0"
  "llvm.ve.vl.vrxor.vvl\0"
  "llvm.ve.vl.vrxor.vvml\0"
  "llvm.ve.vl.vsc.vvssl\0"
  "llvm.ve.vl.vsc.vvssml\0"
  "llvm.ve.vl.vscl.vvssl\0"
  "llvm.ve.vl.vscl.vvssml\0"
  "llvm.ve.vl.vsclnc.vvssl\0"
  "llvm.ve.vl.vsclnc.vvssml\0"
  "llvm.ve.vl.vsclncot.vvssl\0"
  "llvm.ve.vl.vsclncot.vvssml\0"
  "llvm.ve.vl.vsclot.vvssl\0"
  "llvm.ve.vl.vsclot.vvssml\0"
  "llvm.ve.vl.vscnc.vvssl\0"
  "llvm.ve.vl.vscnc.vvssml\0"
  "llvm.ve.vl.vscncot.vvssl\0"
  "llvm.ve.vl.vscncot.vvssml\0"
  "llvm.ve.vl.vscot.vvssl\0"
  "llvm.ve.vl.vscot.vvssml\0"
  "llvm.ve.vl.vscu.vvssl\0"
  "llvm.ve.vl.vscu.vvssml\0"
  "llvm.ve.vl.vscunc.vvssl\0"
  "llvm.ve.vl.vscunc.vvssml\0"
  "llvm.ve.vl.vscuncot.vvssl\0"
  "llvm.ve.vl.vscuncot.vvssml\0"
  "llvm.ve.vl.vscuot.vvssl\0"
  "llvm.ve.vl.vscuot.vvssml\0"
  "llvm.ve.vl.vseq.vl\0"
  "llvm.ve.vl.vseq.vvl\0"
  "llvm.ve.vl.vsfa.vvssl\0"
  "llvm.ve.vl.vsfa.vvssmvl\0"
  "llvm.ve.vl.vsfa.vvssvl\0"
  "llvm.ve.vl.vshf.vvvsl\0"
  "llvm.ve.vl.vshf.vvvsvl\0"
  "llvm.ve.vl.vslal.vvsl\0"
  "llvm.ve.vl.vslal.vvsmvl\0"
  "llvm.ve.vl.vslal.vvsvl\0"
  "llvm.ve.vl.vslal.vvvl\0"
  "llvm.ve.vl.vslal.vvvmvl\0"
  "llvm.ve.vl.vslal.vvvvl\0"
  "llvm.ve.vl.vslawsx.vvsl\0"
  "llvm.ve.vl.vslawsx.vvsmvl\0"
  "llvm.ve.vl.vslawsx.vvsvl\0"
  "llvm.ve.vl.vslawsx.vvvl\0"
  "llvm.ve.vl.vslawsx.vvvmvl\0"
  "llvm.ve.vl.vslawsx.vvvvl\0"
  "llvm.ve.vl.vslawzx.vvsl\0"
  "llvm.ve.vl.vslawzx.vvsmvl\0"
  "llvm.ve.vl.vslawzx.vvsvl\0"
  "llvm.ve.vl.vslawzx.vvvl\0"
  "llvm.ve.vl.vslawzx.vvvmvl\0"
  "llvm.ve.vl.vslawzx.vvvvl\0"
  "llvm.ve.vl.vsll.vvsl\0"
  "llvm.ve.vl.vsll.vvsmvl\0"
  "llvm.ve.vl.vsll.vvsvl\0"
  "llvm.ve.vl.vsll.vvvl\0"
  "llvm.ve.vl.vsll.vvvmvl\0"
  "llvm.ve.vl.vsll.vvvvl\0"
  "llvm.ve.vl.vsral.vvsl\0"
  "llvm.ve.vl.vsral.vvsmvl\0"
  "llvm.ve.vl.vsral.vvsvl\0"
  "llvm.ve.vl.vsral.vvvl\0"
  "llvm.ve.vl.vsral.vvvmvl\0"
  "llvm.ve.vl.vsral.vvvvl\0"
  "llvm.ve.vl.vsrawsx.vvsl\0"
  "llvm.ve.vl.vsrawsx.vvsmvl\0"
  "llvm.ve.vl.vsrawsx.vvsvl\0"
  "llvm.ve.vl.vsrawsx.vvvl\0"
  "llvm.ve.vl.vsrawsx.vvvmvl\0"
  "llvm.ve.vl.vsrawsx.vvvvl\0"
  "llvm.ve.vl.vsrawzx.vvsl\0"
  "llvm.ve.vl.vsrawzx.vvsmvl\0"
  "llvm.ve.vl.vsrawzx.vvsvl\0"
  "llvm.ve.vl.vsrawzx.vvvl\0"
  "llvm.ve.vl.vsrawzx.vvvmvl\0"
  "llvm.ve.vl.vsrawzx.vvvvl\0"
  "llvm.ve.vl.vsrl.vvsl\0"
  "llvm.ve.vl.vsrl.vvsmvl\0"
  "llvm.ve.vl.vsrl.vvsvl\0"
  "llvm.ve.vl.vsrl.vvvl\0"
  "llvm.ve.vl.vsrl.vvvmvl\0"
  "llvm.ve.vl.vsrl.vvvvl\0"
  "llvm.ve.vl.vst.vssl\0"
  "llvm.ve.vl.vst.vssml\0"
  "llvm.ve.vl.vst2d.vssl\0"
  "llvm.ve.vl.vst2d.vssml\0"
  "llvm.ve.vl.vst2dnc.vssl\0"
  "llvm.ve.vl.vst2dnc.vssml\0"
  "llvm.ve.vl.vst2dncot.vssl\0"
  "llvm.ve.vl.vst2dncot.vssml\0"
  "llvm.ve.vl.vst2dot.vssl\0"
  "llvm.ve.vl.vst2dot.vssml\0"
  "llvm.ve.vl.vstl.vssl\0"
  "llvm.ve.vl.vstl.vssml\0"
  "llvm.ve.vl.vstl2d.vssl\0"
  "llvm.ve.vl.vstl2d.vssml\0"
  "llvm.ve.vl.vstl2dnc.vssl\0"
  "llvm.ve.vl.vstl2dnc.vssml\0"
  "llvm.ve.vl.vstl2dncot.vssl\0"
  "llvm.ve.vl.vstl2dncot.vssml\0"
  "llvm.ve.vl.vstl2dot.vssl\0"
  "llvm.ve.vl.vstl2dot.vssml\0"
  "llvm.ve.vl.vstlnc.vssl\0"
  "llvm.ve.vl.vstlnc.vssml\0"
  "llvm.ve.vl.vstlncot.vssl\0"
  "llvm.ve.vl.vstlncot.vssml\0"
  "llvm.ve.vl.vstlot.vssl\0"
  "llvm.ve.vl.vstlot.vssml\0"
  "llvm.ve.vl.vstnc.vssl\0"
  "llvm.ve.vl.vstnc.vssml\0"
  "llvm.ve.vl.vstncot.vssl\0"
  "llvm.ve.vl.vstncot.vssml\0"
  "llvm.ve.vl.vstot.vssl\0"
  "llvm.ve.vl.vstot.vssml\0"
  "llvm.ve.vl.vstu.vssl\0"
  "llvm.ve.vl.vstu.vssml\0"
  "llvm.ve.vl.vstu2d.vssl\0"
  "llvm.ve.vl.vstu2d.vssml\0"
  "llvm.ve.vl.vstu2dnc.vssl\0"
  "llvm.ve.vl.vstu2dnc.vssml\0"
  "llvm.ve.vl.vstu2dncot.vssl\0"
  "llvm.ve.vl.vstu2dncot.vssml\0"
  "llvm.ve.vl.vstu2dot.vssl\0"
  "llvm.ve.vl.vstu2dot.vssml\0"
  "llvm.ve.vl.vstunc.vssl\0"
  "llvm.ve.vl.vstunc.vssml\0"
  "llvm.ve.vl.vstuncot.vssl\0"
  "llvm.ve.vl.vstuncot.vssml\0"
  "llvm.ve.vl.vstuot.vssl\0"
  "llvm.ve.vl.vstuot.vssml\0"
  "llvm.ve.vl.vsubsl.vsvl\0"
  "llvm.ve.vl.vsubsl.vsvmvl\0"
  "llvm.ve.vl.vsubsl.vsvvl\0"
  "llvm.ve.vl.vsubsl.vvvl\0"
  "llvm.ve.vl.vsubsl.vvvmvl\0"
  "llvm.ve.vl.vsubsl.vvvvl\0"
  "llvm.ve.vl.vsubswsx.vsvl\0"
  "llvm.ve.vl.vsubswsx.vsvmvl\0"
  "llvm.ve.vl.vsubswsx.vsvvl\0"
  "llvm.ve.vl.vsubswsx.vvvl\0"
  "llvm.ve.vl.vsubswsx.vvvmvl\0"
  "llvm.ve.vl.vsubswsx.vvvvl\0"
  "llvm.ve.vl.vsubswzx.vsvl\0"
  "llvm.ve.vl.vsubswzx.vsvmvl\0"
  "llvm.ve.vl.vsubswzx.vsvvl\0"
  "llvm.ve.vl.vsubswzx.vvvl\0"
  "llvm.ve.vl.vsubswzx.vvvmvl\0"
  "llvm.ve.vl.vsubswzx.vvvvl\0"
  "llvm.ve.vl.vsubul.vsvl\0"
  "llvm.ve.vl.vsubul.vsvmvl\0"
  "llvm.ve.vl.vsubul.vsvvl\0"
  "llvm.ve.vl.vsubul.vvvl\0"
  "llvm.ve.vl.vsubul.vvvmvl\0"
  "llvm.ve.vl.vsubul.vvvvl\0"
  "llvm.ve.vl.vsubuw.vsvl\0"
  "llvm.ve.vl.vsubuw.vsvmvl\0"
  "llvm.ve.vl.vsubuw.vsvvl\0"
  "llvm.ve.vl.vsubuw.vvvl\0"
  "llvm.ve.vl.vsubuw.vvvmvl\0"
  "llvm.ve.vl.vsubuw.vvvvl\0"
  "llvm.ve.vl.vsuml.vvl\0"
  "llvm.ve.vl.vsuml.vvml\0"
  "llvm.ve.vl.vsumwsx.vvl\0"
  "llvm.ve.vl.vsumwsx.vvml\0"
  "llvm.ve.vl.vsumwzx.vvl\0"
  "llvm.ve.vl.vsumwzx.vvml\0"
  "llvm.ve.vl.vxor.vsvl\0"
  "llvm.ve.vl.vxor.vsvmvl\0"
  "llvm.ve.vl.vxor.vsvvl\0"
  "llvm.ve.vl.vxor.vvvl\0"
  "llvm.ve.vl.vxor.vvvmvl\0"
  "llvm.ve.vl.vxor.vvvvl\0"
  "llvm.ve.vl.xorm.MMM\0"
  "llvm.ve.vl.xorm.mmm\0"
  "llvm.wasm.alltrue\0"
  "llvm.wasm.anytrue\0"
  "llvm.wasm.avgr.unsigned\0"
  "llvm.wasm.bitmask\0"
  "llvm.wasm.bitselect\0"
  "llvm.wasm.catch\0"
  "llvm.wasm.dot\0"
  "llvm.wasm.extadd.pairwise.signed\0"
  "llvm.wasm.extadd.pairwise.unsigned\0"
  "llvm.wasm.extract.lane.f16x8\0"
  "llvm.wasm.get.ehselector\0"
  "llvm.wasm.get.exception\0"
  "llvm.wasm.landingpad.index\0"
  "llvm.wasm.loadf16.f32\0"
  "llvm.wasm.lsda\0"
  "llvm.wasm.memory.atomic.notify\0"
  "llvm.wasm.memory.atomic.wait32\0"
  "llvm.wasm.memory.atomic.wait64\0"
  "llvm.wasm.memory.grow\0"
  "llvm.wasm.memory.size\0"
  "llvm.wasm.narrow.signed\0"
  "llvm.wasm.narrow.unsigned\0"
  "llvm.wasm.pmax\0"
  "llvm.wasm.pmin\0"
  "llvm.wasm.q15mulr.sat.signed\0"
  "llvm.wasm.ref.is_null.exn\0"
  "llvm.wasm.ref.is_null.extern\0"
  "llvm.wasm.ref.is_null.func\0"
  "llvm.wasm.ref.null.exn\0"
  "llvm.wasm.ref.null.extern\0"
  "llvm.wasm.ref.null.func\0"
  "llvm.wasm.ref.test.func\0"
  "llvm.wasm.relaxed.dot.bf16x8.add.f32\0"
  "llvm.wasm.relaxed.dot.i8x16.i7x16.add.signed\0"
  "llvm.wasm.relaxed.dot.i8x16.i7x16.signed\0"
  "llvm.wasm.relaxed.laneselect\0"
  "llvm.wasm.relaxed.madd\0"
  "llvm.wasm.relaxed.max\0"
  "llvm.wasm.relaxed.min\0"
  "llvm.wasm.relaxed.nmadd\0"
  "llvm.wasm.relaxed.q15mulr.signed\0"
  "llvm.wasm.relaxed.swizzle\0"
  "llvm.wasm.relaxed.trunc.signed\0"
  "llvm.wasm.relaxed.trunc.signed.zero\0"
  "llvm.wasm.relaxed.trunc.unsigned\0"
  "llvm.wasm.relaxed.trunc.unsigned.zero\0"
  "llvm.wasm.replace.lane.f16x8\0"
  "llvm.wasm.rethrow\0"
  "llvm.wasm.shuffle\0"
  "llvm.wasm.splat.f16x8\0"
  "llvm.wasm.storef16.f32\0"
  "llvm.wasm.swizzle\0"
  "llvm.wasm.table.copy\0"
  "llvm.wasm.table.fill.exnref\0"
  "llvm.wasm.table.fill.externref\0"
  "llvm.wasm.table.fill.funcref\0"
  "llvm.wasm.table.get.exnref\0"
  "llvm.wasm.table.get.externref\0"
  "llvm.wasm.table.get.funcref\0"
  "llvm.wasm.table.grow.exnref\0"
  "llvm.wasm.table.grow.externref\0"
  "llvm.wasm.table.grow.funcref\0"
  "llvm.wasm.table.set.exnref\0"
  "llvm.wasm.table.set.externref\0"
  "llvm.wasm.table.set.funcref\0"
  "llvm.wasm.table.size\0"
  "llvm.wasm.throw\0"
  "llvm.wasm.tls.align\0"
  "llvm.wasm.tls.base\0"
  "llvm.wasm.tls.size\0"
  "llvm.wasm.trunc.saturate.signed\0"
  "llvm.wasm.trunc.saturate.unsigned\0"
  "llvm.wasm.trunc.signed\0"
  "llvm.wasm.trunc.unsigned\0"
  "llvm.x86.aadd32\0"
  "llvm.x86.aadd64\0"
  "llvm.x86.aand32\0"
  "llvm.x86.aand64\0"
  "llvm.x86.addcarry.32\0"
  "llvm.x86.addcarry.64\0"
  "llvm.x86.aesdec128kl\0"
  "llvm.x86.aesdec256kl\0"
  "llvm.x86.aesdecwide128kl\0"
  "llvm.x86.aesdecwide256kl\0"
  "llvm.x86.aesenc128kl\0"
  "llvm.x86.aesenc256kl\0"
  "llvm.x86.aesencwide128kl\0"
  "llvm.x86.aesencwide256kl\0"
  "llvm.x86.aesni.aesdec\0"
  "llvm.x86.aesni.aesdec.256\0"
  "llvm.x86.aesni.aesdec.512\0"
  "llvm.x86.aesni.aesdeclast\0"
  "llvm.x86.aesni.aesdeclast.256\0"
  "llvm.x86.aesni.aesdeclast.512\0"
  "llvm.x86.aesni.aesenc\0"
  "llvm.x86.aesni.aesenc.256\0"
  "llvm.x86.aesni.aesenc.512\0"
  "llvm.x86.aesni.aesenclast\0"
  "llvm.x86.aesni.aesenclast.256\0"
  "llvm.x86.aesni.aesenclast.512\0"
  "llvm.x86.aesni.aesimc\0"
  "llvm.x86.aesni.aeskeygenassist\0"
  "llvm.x86.aor32\0"
  "llvm.x86.aor64\0"
  "llvm.x86.atomic.add.cc\0"
  "llvm.x86.atomic.and.cc\0"
  "llvm.x86.atomic.btc\0"
  "llvm.x86.atomic.btc.rm\0"
  "llvm.x86.atomic.btr\0"
  "llvm.x86.atomic.btr.rm\0"
  "llvm.x86.atomic.bts\0"
  "llvm.x86.atomic.bts.rm\0"
  "llvm.x86.atomic.or.cc\0"
  "llvm.x86.atomic.sub.cc\0"
  "llvm.x86.atomic.xor.cc\0"
  "llvm.x86.avx.addsub.pd.256\0"
  "llvm.x86.avx.addsub.ps.256\0"
  "llvm.x86.avx.blendv.pd.256\0"
  "llvm.x86.avx.blendv.ps.256\0"
  "llvm.x86.avx.cmp.pd.256\0"
  "llvm.x86.avx.cmp.ps.256\0"
  "llvm.x86.avx.cvt.pd2.ps.256\0"
  "llvm.x86.avx.cvt.pd2dq.256\0"
  "llvm.x86.avx.cvt.ps2dq.256\0"
  "llvm.x86.avx.cvtt.pd2dq.256\0"
  "llvm.x86.avx.cvtt.ps2dq.256\0"
  "llvm.x86.avx.dp.ps.256\0"
  "llvm.x86.avx.hadd.pd.256\0"
  "llvm.x86.avx.hadd.ps.256\0"
  "llvm.x86.avx.hsub.pd.256\0"
  "llvm.x86.avx.hsub.ps.256\0"
  "llvm.x86.avx.ldu.dq.256\0"
  "llvm.x86.avx.maskload.pd\0"
  "llvm.x86.avx.maskload.pd.256\0"
  "llvm.x86.avx.maskload.ps\0"
  "llvm.x86.avx.maskload.ps.256\0"
  "llvm.x86.avx.maskstore.pd\0"
  "llvm.x86.avx.maskstore.pd.256\0"
  "llvm.x86.avx.maskstore.ps\0"
  "llvm.x86.avx.maskstore.ps.256\0"
  "llvm.x86.avx.max.pd.256\0"
  "llvm.x86.avx.max.ps.256\0"
  "llvm.x86.avx.min.pd.256\0"
  "llvm.x86.avx.min.ps.256\0"
  "llvm.x86.avx.movmsk.pd.256\0"
  "llvm.x86.avx.movmsk.ps.256\0"
  "llvm.x86.avx.ptestc.256\0"
  "llvm.x86.avx.ptestnzc.256\0"
  "llvm.x86.avx.ptestz.256\0"
  "llvm.x86.avx.rcp.ps.256\0"
  "llvm.x86.avx.round.pd.256\0"
  "llvm.x86.avx.round.ps.256\0"
  "llvm.x86.avx.rsqrt.ps.256\0"
  "llvm.x86.avx.vpermilvar.pd\0"
  "llvm.x86.avx.vpermilvar.pd.256\0"
  "llvm.x86.avx.vpermilvar.ps\0"
  "llvm.x86.avx.vpermilvar.ps.256\0"
  "llvm.x86.avx.vtestc.pd\0"
  "llvm.x86.avx.vtestc.pd.256\0"
  "llvm.x86.avx.vtestc.ps\0"
  "llvm.x86.avx.vtestc.ps.256\0"
  "llvm.x86.avx.vtestnzc.pd\0"
  "llvm.x86.avx.vtestnzc.pd.256\0"
  "llvm.x86.avx.vtestnzc.ps\0"
  "llvm.x86.avx.vtestnzc.ps.256\0"
  "llvm.x86.avx.vtestz.pd\0"
  "llvm.x86.avx.vtestz.pd.256\0"
  "llvm.x86.avx.vtestz.ps\0"
  "llvm.x86.avx.vtestz.ps.256\0"
  "llvm.x86.avx.vzeroall\0"
  "llvm.x86.avx.vzeroupper\0"
  "llvm.x86.avx10.fpclass.bf16.128\0"
  "llvm.x86.avx10.fpclass.bf16.256\0"
  "llvm.x86.avx10.fpclass.bf16.512\0"
  "llvm.x86.avx10.mask.getexp.bf16.128\0"
  "llvm.x86.avx10.mask.getexp.bf16.256\0"
  "llvm.x86.avx10.mask.getexp.bf16.512\0"
  "llvm.x86.avx10.mask.getmant.bf16.128\0"
  "llvm.x86.avx10.mask.getmant.bf16.256\0"
  "llvm.x86.avx10.mask.getmant.bf16.512\0"
  "llvm.x86.avx10.mask.rcp.bf16.128\0"
  "llvm.x86.avx10.mask.rcp.bf16.256\0"
  "llvm.x86.avx10.mask.rcp.bf16.512\0"
  "llvm.x86.avx10.mask.reduce.bf16.128\0"
  "llvm.x86.avx10.mask.reduce.bf16.256\0"
  "llvm.x86.avx10.mask.reduce.bf16.512\0"
  "llvm.x86.avx10.mask.rndscale.bf16.128\0"
  "llvm.x86.avx10.mask.rndscale.bf16.256\0"
  "llvm.x86.avx10.mask.rndscale.bf16.512\0"
  "llvm.x86.avx10.mask.rsqrt.bf16.128\0"
  "llvm.x86.avx10.mask.rsqrt.bf16.256\0"
  "llvm.x86.avx10.mask.rsqrt.bf16.512\0"
  "llvm.x86.avx10.mask.scalef.bf16.128\0"
  "llvm.x86.avx10.mask.scalef.bf16.256\0"
  "llvm.x86.avx10.mask.scalef.bf16.512\0"
  "llvm.x86.avx10.mask.vcvt2ps2phx.128\0"
  "llvm.x86.avx10.mask.vcvt2ps2phx.256\0"
  "llvm.x86.avx10.mask.vcvt2ps2phx.512\0"
  "llvm.x86.avx10.mask.vcvtbiasph2bf8128\0"
  "llvm.x86.avx10.mask.vcvtbiasph2bf8256\0"
  "llvm.x86.avx10.mask.vcvtbiasph2bf8512\0"
  "llvm.x86.avx10.mask.vcvtbiasph2bf8s128\0"
  "llvm.x86.avx10.mask.vcvtbiasph2bf8s256\0"
  "llvm.x86.avx10.mask.vcvtbiasph2bf8s512\0"
  "llvm.x86.avx10.mask.vcvtbiasph2hf8128\0"
  "llvm.x86.avx10.mask.vcvtbiasph2hf8256\0"
  "llvm.x86.avx10.mask.vcvtbiasph2hf8512\0"
  "llvm.x86.avx10.mask.vcvtbiasph2hf8s128\0"
  "llvm.x86.avx10.mask.vcvtbiasph2hf8s256\0"
  "llvm.x86.avx10.mask.vcvtbiasph2hf8s512\0"
  "llvm.x86.avx10.mask.vcvthf82ph128\0"
  "llvm.x86.avx10.mask.vcvthf82ph256\0"
  "llvm.x86.avx10.mask.vcvthf82ph512\0"
  "llvm.x86.avx10.mask.vcvtph2bf8128\0"
  "llvm.x86.avx10.mask.vcvtph2bf8256\0"
  "llvm.x86.avx10.mask.vcvtph2bf8512\0"
  "llvm.x86.avx10.mask.vcvtph2bf8s128\0"
  "llvm.x86.avx10.mask.vcvtph2bf8s256\0"
  "llvm.x86.avx10.mask.vcvtph2bf8s512\0"
  "llvm.x86.avx10.mask.vcvtph2hf8128\0"
  "llvm.x86.avx10.mask.vcvtph2hf8256\0"
  "llvm.x86.avx10.mask.vcvtph2hf8512\0"
  "llvm.x86.avx10.mask.vcvtph2hf8s128\0"
  "llvm.x86.avx10.mask.vcvtph2hf8s256\0"
  "llvm.x86.avx10.mask.vcvtph2hf8s512\0"
  "llvm.x86.avx10.mask.vcvtph2ibs128\0"
  "llvm.x86.avx10.mask.vcvtph2ibs256\0"
  "llvm.x86.avx10.mask.vcvtph2ibs512\0"
  "llvm.x86.avx10.mask.vcvtph2iubs128\0"
  "llvm.x86.avx10.mask.vcvtph2iubs256\0"
  "llvm.x86.avx10.mask.vcvtph2iubs512\0"
  "llvm.x86.avx10.mask.vcvtps2ibs128\0"
  "llvm.x86.avx10.mask.vcvtps2ibs256\0"
  "llvm.x86.avx10.mask.vcvtps2ibs512\0"
  "llvm.x86.avx10.mask.vcvtps2iubs128\0"
  "llvm.x86.avx10.mask.vcvtps2iubs256\0"
  "llvm.x86.avx10.mask.vcvtps2iubs512\0"
  "llvm.x86.avx10.mask.vcvttpd2dqs.128\0"
  "llvm.x86.avx10.mask.vcvttpd2dqs.256\0"
  "llvm.x86.avx10.mask.vcvttpd2dqs.round.512\0"
  "llvm.x86.avx10.mask.vcvttpd2qqs.128\0"
  "llvm.x86.avx10.mask.vcvttpd2qqs.256\0"
  "llvm.x86.avx10.mask.vcvttpd2qqs.round.512\0"
  "llvm.x86.avx10.mask.vcvttpd2udqs.128\0"
  "llvm.x86.avx10.mask.vcvttpd2udqs.256\0"
  "llvm.x86.avx10.mask.vcvttpd2udqs.round.512\0"
  "llvm.x86.avx10.mask.vcvttpd2uqqs.128\0"
  "llvm.x86.avx10.mask.vcvttpd2uqqs.256\0"
  "llvm.x86.avx10.mask.vcvttpd2uqqs.round.512\0"
  "llvm.x86.avx10.mask.vcvttph2ibs128\0"
  "llvm.x86.avx10.mask.vcvttph2ibs256\0"
  "llvm.x86.avx10.mask.vcvttph2ibs512\0"
  "llvm.x86.avx10.mask.vcvttph2iubs128\0"
  "llvm.x86.avx10.mask.vcvttph2iubs256\0"
  "llvm.x86.avx10.mask.vcvttph2iubs512\0"
  "llvm.x86.avx10.mask.vcvttps2dqs.128\0"
  "llvm.x86.avx10.mask.vcvttps2dqs.256\0"
  "llvm.x86.avx10.mask.vcvttps2dqs.round.512\0"
  "llvm.x86.avx10.mask.vcvttps2ibs128\0"
  "llvm.x86.avx10.mask.vcvttps2ibs256\0"
  "llvm.x86.avx10.mask.vcvttps2ibs512\0"
  "llvm.x86.avx10.mask.vcvttps2iubs128\0"
  "llvm.x86.avx10.mask.vcvttps2iubs256\0"
  "llvm.x86.avx10.mask.vcvttps2iubs512\0"
  "llvm.x86.avx10.mask.vcvttps2qqs.128\0"
  "llvm.x86.avx10.mask.vcvttps2qqs.256\0"
  "llvm.x86.avx10.mask.vcvttps2qqs.round.512\0"
  "llvm.x86.avx10.mask.vcvttps2udqs.128\0"
  "llvm.x86.avx10.mask.vcvttps2udqs.256\0"
  "llvm.x86.avx10.mask.vcvttps2udqs.round.512\0"
  "llvm.x86.avx10.mask.vcvttps2uqqs.128\0"
  "llvm.x86.avx10.mask.vcvttps2uqqs.256\0"
  "llvm.x86.avx10.mask.vcvttps2uqqs.round.512\0"
  "llvm.x86.avx10.mask.vminmaxpd.round\0"
  "llvm.x86.avx10.mask.vminmaxpd128\0"
  "llvm.x86.avx10.mask.vminmaxpd256\0"
  "llvm.x86.avx10.mask.vminmaxph.round\0"
  "llvm.x86.avx10.mask.vminmaxph128\0"
  "llvm.x86.avx10.mask.vminmaxph256\0"
  "llvm.x86.avx10.mask.vminmaxps.round\0"
  "llvm.x86.avx10.mask.vminmaxps128\0"
  "llvm.x86.avx10.mask.vminmaxps256\0"
  "llvm.x86.avx10.mask.vminmaxsd.round\0"
  "llvm.x86.avx10.mask.vminmaxsh.round\0"
  "llvm.x86.avx10.mask.vminmaxss.round\0"
  "llvm.x86.avx10.vaddbf16128\0"
  "llvm.x86.avx10.vaddbf16256\0"
  "llvm.x86.avx10.vaddbf16512\0"
  "llvm.x86.avx10.vaddpd256\0"
  "llvm.x86.avx10.vaddph256\0"
  "llvm.x86.avx10.vaddps256\0"
  "llvm.x86.avx10.vcomisbf16eq\0"
  "llvm.x86.avx10.vcomisbf16ge\0"
  "llvm.x86.avx10.vcomisbf16gt\0"
  "llvm.x86.avx10.vcomisbf16le\0"
  "llvm.x86.avx10.vcomisbf16lt\0"
  "llvm.x86.avx10.vcomisbf16neq\0"
  "llvm.x86.avx10.vcvt2ph2bf8128\0"
  "llvm.x86.avx10.vcvt2ph2bf8256\0"
  "llvm.x86.avx10.vcvt2ph2bf8512\0"
  "llvm.x86.avx10.vcvt2ph2bf8s128\0"
  "llvm.x86.avx10.vcvt2ph2bf8s256\0"
  "llvm.x86.avx10.vcvt2ph2bf8s512\0"
  "llvm.x86.avx10.vcvt2ph2hf8128\0"
  "llvm.x86.avx10.vcvt2ph2hf8256\0"
  "llvm.x86.avx10.vcvt2ph2hf8512\0"
  "llvm.x86.avx10.vcvt2ph2hf8s128\0"
  "llvm.x86.avx10.vcvt2ph2hf8s256\0"
  "llvm.x86.avx10.vcvt2ph2hf8s512\0"
  "llvm.x86.avx10.vcvtbf162ibs128\0"
  "llvm.x86.avx10.vcvtbf162ibs256\0"
  "llvm.x86.avx10.vcvtbf162ibs512\0"
  "llvm.x86.avx10.vcvtbf162iubs128\0"
  "llvm.x86.avx10.vcvtbf162iubs256\0"
  "llvm.x86.avx10.vcvtbf162iubs512\0"
  "llvm.x86.avx10.vcvttbf162ibs128\0"
  "llvm.x86.avx10.vcvttbf162ibs256\0"
  "llvm.x86.avx10.vcvttbf162ibs512\0"
  "llvm.x86.avx10.vcvttbf162iubs128\0"
  "llvm.x86.avx10.vcvttbf162iubs256\0"
  "llvm.x86.avx10.vcvttbf162iubs512\0"
  "llvm.x86.avx10.vcvttsd2sis\0"
  "llvm.x86.avx10.vcvttsd2sis64\0"
  "llvm.x86.avx10.vcvttsd2usis\0"
  "llvm.x86.avx10.vcvttsd2usis64\0"
  "llvm.x86.avx10.vcvttss2sis\0"
  "llvm.x86.avx10.vcvttss2sis64\0"
  "llvm.x86.avx10.vcvttss2usis\0"
  "llvm.x86.avx10.vcvttss2usis64\0"
  "llvm.x86.avx10.vdivbf16128\0"
  "llvm.x86.avx10.vdivbf16256\0"
  "llvm.x86.avx10.vdivbf16512\0"
  "llvm.x86.avx10.vdpphps.128\0"
  "llvm.x86.avx10.vdpphps.256\0"
  "llvm.x86.avx10.vdpphps.512\0"
  "llvm.x86.avx10.vfmadd132bf16128\0"
  "llvm.x86.avx10.vfmadd132bf16256\0"
  "llvm.x86.avx10.vfmadd132bf16512\0"
  "llvm.x86.avx10.vfmadd213bf16128\0"
  "llvm.x86.avx10.vfmadd213bf16256\0"
  "llvm.x86.avx10.vfmadd231bf16128\0"
  "llvm.x86.avx10.vfmadd231bf16256\0"
  "llvm.x86.avx10.vfmadd231bf16512\0"
  "llvm.x86.avx10.vfmsub132bf16128\0"
  "llvm.x86.avx10.vfmsub132bf16256\0"
  "llvm.x86.avx10.vfmsub132bf16512\0"
  "llvm.x86.avx10.vfmsub213bf16128\0"
  "llvm.x86.avx10.vfmsub213bf16256\0"
  "llvm.x86.avx10.vfmsub213bf16512\0"
  "llvm.x86.avx10.vfmsub231bf16128\0"
  "llvm.x86.avx10.vfmsub231bf16256\0"
  "llvm.x86.avx10.vfmsub231bf16512\0"
  "llvm.x86.avx10.vfnmadd132bf16128\0"
  "llvm.x86.avx10.vfnmadd132bf16256\0"
  "llvm.x86.avx10.vfnmadd132bf16512\0"
  "llvm.x86.avx10.vfnmadd213bf16128\0"
  "llvm.x86.avx10.vfnmadd213bf16256\0"
  "llvm.x86.avx10.vfnmadd213bf16512\0"
  "llvm.x86.avx10.vfnmadd231bf16128\0"
  "llvm.x86.avx10.vfnmadd231bf16256\0"
  "llvm.x86.avx10.vfnmadd231bf16512\0"
  "llvm.x86.avx10.vfnmsub132bf16128\0"
  "llvm.x86.avx10.vfnmsub132bf16256\0"
  "llvm.x86.avx10.vfnmsub132bf16512\0"
  "llvm.x86.avx10.vfnmsub213bf16128\0"
  "llvm.x86.avx10.vfnmsub213bf16256\0"
  "llvm.x86.avx10.vfnmsub213bf16512\0"
  "llvm.x86.avx10.vfnmsub231bf16128\0"
  "llvm.x86.avx10.vfnmsub231bf16256\0"
  "llvm.x86.avx10.vfnmsub231bf16512\0"
  "llvm.x86.avx10.vmaxbf16128\0"
  "llvm.x86.avx10.vmaxbf16256\0"
  "llvm.x86.avx10.vmaxbf16512\0"
  "llvm.x86.avx10.vminbf16128\0"
  "llvm.x86.avx10.vminbf16256\0"
  "llvm.x86.avx10.vminbf16512\0"
  "llvm.x86.avx10.vminmaxbf16128\0"
  "llvm.x86.avx10.vminmaxbf16256\0"
  "llvm.x86.avx10.vminmaxbf16512\0"
  "llvm.x86.avx10.vminmaxpd128\0"
  "llvm.x86.avx10.vminmaxpd256\0"
  "llvm.x86.avx10.vminmaxph128\0"
  "llvm.x86.avx10.vminmaxph256\0"
  "llvm.x86.avx10.vminmaxps128\0"
  "llvm.x86.avx10.vminmaxps256\0"
  "llvm.x86.avx10.vmovrsb128\0"
  "llvm.x86.avx10.vmovrsb256\0"
  "llvm.x86.avx10.vmovrsb512\0"
  "llvm.x86.avx10.vmovrsd128\0"
  "llvm.x86.avx10.vmovrsd256\0"
  "llvm.x86.avx10.vmovrsd512\0"
  "llvm.x86.avx10.vmovrsq128\0"
  "llvm.x86.avx10.vmovrsq256\0"
  "llvm.x86.avx10.vmovrsq512\0"
  "llvm.x86.avx10.vmovrsw128\0"
  "llvm.x86.avx10.vmovrsw256\0"
  "llvm.x86.avx10.vmovrsw512\0"
  "llvm.x86.avx10.vmpsadbw.512\0"
  "llvm.x86.avx10.vmulbf16128\0"
  "llvm.x86.avx10.vmulbf16256\0"
  "llvm.x86.avx10.vmulbf16512\0"
  "llvm.x86.avx10.vpdpbssd.512\0"
  "llvm.x86.avx10.vpdpbssds.512\0"
  "llvm.x86.avx10.vpdpbsud.512\0"
  "llvm.x86.avx10.vpdpbsuds.512\0"
  "llvm.x86.avx10.vpdpbuud.512\0"
  "llvm.x86.avx10.vpdpbuuds.512\0"
  "llvm.x86.avx10.vpdpwsud.512\0"
  "llvm.x86.avx10.vpdpwsuds.512\0"
  "llvm.x86.avx10.vpdpwusd.512\0"
  "llvm.x86.avx10.vpdpwusds.512\0"
  "llvm.x86.avx10.vpdpwuud.512\0"
  "llvm.x86.avx10.vpdpwuuds.512\0"
  "llvm.x86.avx10.vsubbf16128\0"
  "llvm.x86.avx10.vsubbf16256\0"
  "llvm.x86.avx10.vsubbf16512\0"
  "llvm.x86.avx2.gather.d.d\0"
  "llvm.x86.avx2.gather.d.d.256\0"
  "llvm.x86.avx2.gather.d.pd\0"
  "llvm.x86.avx2.gather.d.pd.256\0"
  "llvm.x86.avx2.gather.d.ps\0"
  "llvm.x86.avx2.gather.d.ps.256\0"
  "llvm.x86.avx2.gather.d.q\0"
  "llvm.x86.avx2.gather.d.q.256\0"
  "llvm.x86.avx2.gather.q.d\0"
  "llvm.x86.avx2.gather.q.d.256\0"
  "llvm.x86.avx2.gather.q.pd\0"
  "llvm.x86.avx2.gather.q.pd.256\0"
  "llvm.x86.avx2.gather.q.ps\0"
  "llvm.x86.avx2.gather.q.ps.256\0"
  "llvm.x86.avx2.gather.q.q\0"
  "llvm.x86.avx2.gather.q.q.256\0"
  "llvm.x86.avx2.maskload.d\0"
  "llvm.x86.avx2.maskload.d.256\0"
  "llvm.x86.avx2.maskload.q\0"
  "llvm.x86.avx2.maskload.q.256\0"
  "llvm.x86.avx2.maskstore.d\0"
  "llvm.x86.avx2.maskstore.d.256\0"
  "llvm.x86.avx2.maskstore.q\0"
  "llvm.x86.avx2.maskstore.q.256\0"
  "llvm.x86.avx2.mpsadbw\0"
  "llvm.x86.avx2.packssdw\0"
  "llvm.x86.avx2.packsswb\0"
  "llvm.x86.avx2.packusdw\0"
  "llvm.x86.avx2.packuswb\0"
  "llvm.x86.avx2.pavg.b\0"
  "llvm.x86.avx2.pavg.w\0"
  "llvm.x86.avx2.pblendvb\0"
  "llvm.x86.avx2.permd\0"
  "llvm.x86.avx2.permps\0"
  "llvm.x86.avx2.phadd.d\0"
  "llvm.x86.avx2.phadd.sw\0"
  "llvm.x86.avx2.phadd.w\0"
  "llvm.x86.avx2.phsub.d\0"
  "llvm.x86.avx2.phsub.sw\0"
  "llvm.x86.avx2.phsub.w\0"
  "llvm.x86.avx2.pmadd.ub.sw\0"
  "llvm.x86.avx2.pmadd.wd\0"
  "llvm.x86.avx2.pmovmskb\0"
  "llvm.x86.avx2.pmul.hr.sw\0"
  "llvm.x86.avx2.pmulh.w\0"
  "llvm.x86.avx2.pmulhu.w\0"
  "llvm.x86.avx2.psad.bw\0"
  "llvm.x86.avx2.pshuf.b\0"
  "llvm.x86.avx2.psign.b\0"
  "llvm.x86.avx2.psign.d\0"
  "llvm.x86.avx2.psign.w\0"
  "llvm.x86.avx2.psll.d\0"
  "llvm.x86.avx2.psll.q\0"
  "llvm.x86.avx2.psll.w\0"
  "llvm.x86.avx2.pslli.d\0"
  "llvm.x86.avx2.pslli.q\0"
  "llvm.x86.avx2.pslli.w\0"
  "llvm.x86.avx2.psllv.d\0"
  "llvm.x86.avx2.psllv.d.256\0"
  "llvm.x86.avx2.psllv.q\0"
  "llvm.x86.avx2.psllv.q.256\0"
  "llvm.x86.avx2.psra.d\0"
  "llvm.x86.avx2.psra.w\0"
  "llvm.x86.avx2.psrai.d\0"
  "llvm.x86.avx2.psrai.w\0"
  "llvm.x86.avx2.psrav.d\0"
  "llvm.x86.avx2.psrav.d.256\0"
  "llvm.x86.avx2.psrl.d\0"
  "llvm.x86.avx2.psrl.q\0"
  "llvm.x86.avx2.psrl.w\0"
  "llvm.x86.avx2.psrli.d\0"
  "llvm.x86.avx2.psrli.q\0"
  "llvm.x86.avx2.psrli.w\0"
  "llvm.x86.avx2.psrlv.d\0"
  "llvm.x86.avx2.psrlv.d.256\0"
  "llvm.x86.avx2.psrlv.q\0"
  "llvm.x86.avx2.psrlv.q.256\0"
  "llvm.x86.avx2.vpdpbssd.128\0"
  "llvm.x86.avx2.vpdpbssd.256\0"
  "llvm.x86.avx2.vpdpbssds.128\0"
  "llvm.x86.avx2.vpdpbssds.256\0"
  "llvm.x86.avx2.vpdpbsud.128\0"
  "llvm.x86.avx2.vpdpbsud.256\0"
  "llvm.x86.avx2.vpdpbsuds.128\0"
  "llvm.x86.avx2.vpdpbsuds.256\0"
  "llvm.x86.avx2.vpdpbuud.128\0"
  "llvm.x86.avx2.vpdpbuud.256\0"
  "llvm.x86.avx2.vpdpbuuds.128\0"
  "llvm.x86.avx2.vpdpbuuds.256\0"
  "llvm.x86.avx2.vpdpwsud.128\0"
  "llvm.x86.avx2.vpdpwsud.256\0"
  "llvm.x86.avx2.vpdpwsuds.128\0"
  "llvm.x86.avx2.vpdpwsuds.256\0"
  "llvm.x86.avx2.vpdpwusd.128\0"
  "llvm.x86.avx2.vpdpwusd.256\0"
  "llvm.x86.avx2.vpdpwusds.128\0"
  "llvm.x86.avx2.vpdpwusds.256\0"
  "llvm.x86.avx2.vpdpwuud.128\0"
  "llvm.x86.avx2.vpdpwuud.256\0"
  "llvm.x86.avx2.vpdpwuuds.128\0"
  "llvm.x86.avx2.vpdpwuuds.256\0"
  "llvm.x86.avx512.add.pd.512\0"
  "llvm.x86.avx512.add.ps.512\0"
  "llvm.x86.avx512.broadcastmb.128\0"
  "llvm.x86.avx512.broadcastmb.256\0"
  "llvm.x86.avx512.broadcastmb.512\0"
  "llvm.x86.avx512.broadcastmw.128\0"
  "llvm.x86.avx512.broadcastmw.256\0"
  "llvm.x86.avx512.broadcastmw.512\0"
  "llvm.x86.avx512.conflict.d.128\0"
  "llvm.x86.avx512.conflict.d.256\0"
  "llvm.x86.avx512.conflict.d.512\0"
  "llvm.x86.avx512.conflict.q.128\0"
  "llvm.x86.avx512.conflict.q.256\0"
  "llvm.x86.avx512.conflict.q.512\0"
  "llvm.x86.avx512.cvtsi2sd64\0"
  "llvm.x86.avx512.cvtsi2ss32\0"
  "llvm.x86.avx512.cvtsi2ss64\0"
  "llvm.x86.avx512.cvttsd2si\0"
  "llvm.x86.avx512.cvttsd2si64\0"
  "llvm.x86.avx512.cvttsd2usi\0"
  "llvm.x86.avx512.cvttsd2usi64\0"
  "llvm.x86.avx512.cvttss2si\0"
  "llvm.x86.avx512.cvttss2si64\0"
  "llvm.x86.avx512.cvttss2usi\0"
  "llvm.x86.avx512.cvttss2usi64\0"
  "llvm.x86.avx512.cvtusi2ss\0"
  "llvm.x86.avx512.cvtusi642sd\0"
  "llvm.x86.avx512.cvtusi642ss\0"
  "llvm.x86.avx512.dbpsadbw.128\0"
  "llvm.x86.avx512.dbpsadbw.256\0"
  "llvm.x86.avx512.dbpsadbw.512\0"
  "llvm.x86.avx512.div.pd.512\0"
  "llvm.x86.avx512.div.ps.512\0"
  "llvm.x86.avx512.fpclass.pd.128\0"
  "llvm.x86.avx512.fpclass.pd.256\0"
  "llvm.x86.avx512.fpclass.pd.512\0"
  "llvm.x86.avx512.fpclass.ps.128\0"
  "llvm.x86.avx512.fpclass.ps.256\0"
  "llvm.x86.avx512.fpclass.ps.512\0"
  "llvm.x86.avx512.gather.dpd.512\0"
  "llvm.x86.avx512.gather.dpi.512\0"
  "llvm.x86.avx512.gather.dpq.512\0"
  "llvm.x86.avx512.gather.dps.512\0"
  "llvm.x86.avx512.gather.qpd.512\0"
  "llvm.x86.avx512.gather.qpi.512\0"
  "llvm.x86.avx512.gather.qpq.512\0"
  "llvm.x86.avx512.gather.qps.512\0"
  "llvm.x86.avx512.gather3div2.df\0"
  "llvm.x86.avx512.gather3div2.di\0"
  "llvm.x86.avx512.gather3div4.df\0"
  "llvm.x86.avx512.gather3div4.di\0"
  "llvm.x86.avx512.gather3div4.sf\0"
  "llvm.x86.avx512.gather3div4.si\0"
  "llvm.x86.avx512.gather3div8.sf\0"
  "llvm.x86.avx512.gather3div8.si\0"
  "llvm.x86.avx512.gather3siv2.df\0"
  "llvm.x86.avx512.gather3siv2.di\0"
  "llvm.x86.avx512.gather3siv4.df\0"
  "llvm.x86.avx512.gather3siv4.di\0"
  "llvm.x86.avx512.gather3siv4.sf\0"
  "llvm.x86.avx512.gather3siv4.si\0"
  "llvm.x86.avx512.gather3siv8.sf\0"
  "llvm.x86.avx512.gather3siv8.si\0"
  "llvm.x86.avx512.kadd.b\0"
  "llvm.x86.avx512.kadd.d\0"
  "llvm.x86.avx512.kadd.q\0"
  "llvm.x86.avx512.kadd.w\0"
  "llvm.x86.avx512.ktestc.b\0"
  "llvm.x86.avx512.ktestc.d\0"
  "llvm.x86.avx512.ktestc.q\0"
  "llvm.x86.avx512.ktestc.w\0"
  "llvm.x86.avx512.ktestz.b\0"
  "llvm.x86.avx512.ktestz.d\0"
  "llvm.x86.avx512.ktestz.q\0"
  "llvm.x86.avx512.ktestz.w\0"
  "llvm.x86.avx512.mask.add.sd.round\0"
  "llvm.x86.avx512.mask.add.ss.round\0"
  "llvm.x86.avx512.mask.cmp.pd.128\0"
  "llvm.x86.avx512.mask.cmp.pd.256\0"
  "llvm.x86.avx512.mask.cmp.pd.512\0"
  "llvm.x86.avx512.mask.cmp.ps.128\0"
  "llvm.x86.avx512.mask.cmp.ps.256\0"
  "llvm.x86.avx512.mask.cmp.ps.512\0"
  "llvm.x86.avx512.mask.cmp.sd\0"
  "llvm.x86.avx512.mask.cmp.ss\0"
  "llvm.x86.avx512.mask.compress\0"
  "llvm.x86.avx512.mask.cvtpd2dq.128\0"
  "llvm.x86.avx512.mask.cvtpd2dq.512\0"
  "llvm.x86.avx512.mask.cvtpd2ps\0"
  "llvm.x86.avx512.mask.cvtpd2ps.512\0"
  "llvm.x86.avx512.mask.cvtpd2qq.128\0"
  "llvm.x86.avx512.mask.cvtpd2qq.256\0"
  "llvm.x86.avx512.mask.cvtpd2qq.512\0"
  "llvm.x86.avx512.mask.cvtpd2udq.128\0"
  "llvm.x86.avx512.mask.cvtpd2udq.256\0"
  "llvm.x86.avx512.mask.cvtpd2udq.512\0"
  "llvm.x86.avx512.mask.cvtpd2uqq.128\0"
  "llvm.x86.avx512.mask.cvtpd2uqq.256\0"
  "llvm.x86.avx512.mask.cvtpd2uqq.512\0"
  "llvm.x86.avx512.mask.cvtps2dq.128\0"
  "llvm.x86.avx512.mask.cvtps2dq.256\0"
  "llvm.x86.avx512.mask.cvtps2dq.512\0"
  "llvm.x86.avx512.mask.cvtps2pd.512\0"
  "llvm.x86.avx512.mask.cvtps2qq.128\0"
  "llvm.x86.avx512.mask.cvtps2qq.256\0"
  "llvm.x86.avx512.mask.cvtps2qq.512\0"
  "llvm.x86.avx512.mask.cvtps2udq.128\0"
  "llvm.x86.avx512.mask.cvtps2udq.256\0"
  "llvm.x86.avx512.mask.cvtps2udq.512\0"
  "llvm.x86.avx512.mask.cvtps2uqq.128\0"
  "llvm.x86.avx512.mask.cvtps2uqq.256\0"
  "llvm.x86.avx512.mask.cvtps2uqq.512\0"
  "llvm.x86.avx512.mask.cvtqq2ps.128\0"
  "llvm.x86.avx512.mask.cvtsd2ss.round\0"
  "llvm.x86.avx512.mask.cvtss2sd.round\0"
  "llvm.x86.avx512.mask.cvttpd2dq.128\0"
  "llvm.x86.avx512.mask.cvttpd2dq.512\0"
  "llvm.x86.avx512.mask.cvttpd2qq.128\0"
  "llvm.x86.avx512.mask.cvttpd2qq.256\0"
  "llvm.x86.avx512.mask.cvttpd2qq.512\0"
  "llvm.x86.avx512.mask.cvttpd2udq.128\0"
  "llvm.x86.avx512.mask.cvttpd2udq.256\0"
  "llvm.x86.avx512.mask.cvttpd2udq.512\0"
  "llvm.x86.avx512.mask.cvttpd2uqq.128\0"
  "llvm.x86.avx512.mask.cvttpd2uqq.256\0"
  "llvm.x86.avx512.mask.cvttpd2uqq.512\0"
  "llvm.x86.avx512.mask.cvttps2dq.512\0"
  "llvm.x86.avx512.mask.cvttps2qq.128\0"
  "llvm.x86.avx512.mask.cvttps2qq.256\0"
  "llvm.x86.avx512.mask.cvttps2qq.512\0"
  "llvm.x86.avx512.mask.cvttps2udq.128\0"
  "llvm.x86.avx512.mask.cvttps2udq.256\0"
  "llvm.x86.avx512.mask.cvttps2udq.512\0"
  "llvm.x86.avx512.mask.cvttps2uqq.128\0"
  "llvm.x86.avx512.mask.cvttps2uqq.256\0"
  "llvm.x86.avx512.mask.cvttps2uqq.512\0"
  "llvm.x86.avx512.mask.cvtuqq2ps.128\0"
  "llvm.x86.avx512.mask.div.sd.round\0"
  "llvm.x86.avx512.mask.div.ss.round\0"
  "llvm.x86.avx512.mask.expand\0"
  "llvm.x86.avx512.mask.fixupimm.pd.128\0"
  "llvm.x86.avx512.mask.fixupimm.pd.256\0"
  "llvm.x86.avx512.mask.fixupimm.pd.512\0"
  "llvm.x86.avx512.mask.fixupimm.ps.128\0"
  "llvm.x86.avx512.mask.fixupimm.ps.256\0"
  "llvm.x86.avx512.mask.fixupimm.ps.512\0"
  "llvm.x86.avx512.mask.fixupimm.sd\0"
  "llvm.x86.avx512.mask.fixupimm.ss\0"
  "llvm.x86.avx512.mask.fpclass.sd\0"
  "llvm.x86.avx512.mask.fpclass.ss\0"
  "llvm.x86.avx512.mask.gather.dpd.512\0"
  "llvm.x86.avx512.mask.gather.dpi.512\0"
  "llvm.x86.avx512.mask.gather.dpq.512\0"
  "llvm.x86.avx512.mask.gather.dps.512\0"
  "llvm.x86.avx512.mask.gather.qpd.512\0"
  "llvm.x86.avx512.mask.gather.qpi.512\0"
  "llvm.x86.avx512.mask.gather.qpq.512\0"
  "llvm.x86.avx512.mask.gather.qps.512\0"
  "llvm.x86.avx512.mask.gather3div2.df\0"
  "llvm.x86.avx512.mask.gather3div2.di\0"
  "llvm.x86.avx512.mask.gather3div4.df\0"
  "llvm.x86.avx512.mask.gather3div4.di\0"
  "llvm.x86.avx512.mask.gather3div4.sf\0"
  "llvm.x86.avx512.mask.gather3div4.si\0"
  "llvm.x86.avx512.mask.gather3div8.sf\0"
  "llvm.x86.avx512.mask.gather3div8.si\0"
  "llvm.x86.avx512.mask.gather3siv2.df\0"
  "llvm.x86.avx512.mask.gather3siv2.di\0"
  "llvm.x86.avx512.mask.gather3siv4.df\0"
  "llvm.x86.avx512.mask.gather3siv4.di\0"
  "llvm.x86.avx512.mask.gather3siv4.sf\0"
  "llvm.x86.avx512.mask.gather3siv4.si\0"
  "llvm.x86.avx512.mask.gather3siv8.sf\0"
  "llvm.x86.avx512.mask.gather3siv8.si\0"
  "llvm.x86.avx512.mask.getexp.pd.128\0"
  "llvm.x86.avx512.mask.getexp.pd.256\0"
  "llvm.x86.avx512.mask.getexp.pd.512\0"
  "llvm.x86.avx512.mask.getexp.ps.128\0"
  "llvm.x86.avx512.mask.getexp.ps.256\0"
  "llvm.x86.avx512.mask.getexp.ps.512\0"
  "llvm.x86.avx512.mask.getexp.sd\0"
  "llvm.x86.avx512.mask.getexp.ss\0"
  "llvm.x86.avx512.mask.getmant.pd.128\0"
  "llvm.x86.avx512.mask.getmant.pd.256\0"
  "llvm.x86.avx512.mask.getmant.pd.512\0"
  "llvm.x86.avx512.mask.getmant.ps.128\0"
  "llvm.x86.avx512.mask.getmant.ps.256\0"
  "llvm.x86.avx512.mask.getmant.ps.512\0"
  "llvm.x86.avx512.mask.getmant.sd\0"
  "llvm.x86.avx512.mask.getmant.ss\0"
  "llvm.x86.avx512.mask.max.sd.round\0"
  "llvm.x86.avx512.mask.max.ss.round\0"
  "llvm.x86.avx512.mask.min.sd.round\0"
  "llvm.x86.avx512.mask.min.ss.round\0"
  "llvm.x86.avx512.mask.mul.sd.round\0"
  "llvm.x86.avx512.mask.mul.ss.round\0"
  "llvm.x86.avx512.mask.pmov.db.128\0"
  "llvm.x86.avx512.mask.pmov.db.256\0"
  "llvm.x86.avx512.mask.pmov.db.512\0"
  "llvm.x86.avx512.mask.pmov.db.mem.128\0"
  "llvm.x86.avx512.mask.pmov.db.mem.256\0"
  "llvm.x86.avx512.mask.pmov.db.mem.512\0"
  "llvm.x86.avx512.mask.pmov.dw.128\0"
  "llvm.x86.avx512.mask.pmov.dw.256\0"
  "llvm.x86.avx512.mask.pmov.dw.512\0"
  "llvm.x86.avx512.mask.pmov.dw.mem.128\0"
  "llvm.x86.avx512.mask.pmov.dw.mem.256\0"
  "llvm.x86.avx512.mask.pmov.dw.mem.512\0"
  "llvm.x86.avx512.mask.pmov.qb.128\0"
  "llvm.x86.avx512.mask.pmov.qb.256\0"
  "llvm.x86.avx512.mask.pmov.qb.512\0"
  "llvm.x86.avx512.mask.pmov.qb.mem.128\0"
  "llvm.x86.avx512.mask.pmov.qb.mem.256\0"
  "llvm.x86.avx512.mask.pmov.qb.mem.512\0"
  "llvm.x86.avx512.mask.pmov.qd.128\0"
  "llvm.x86.avx512.mask.pmov.qd.mem.128\0"
  "llvm.x86.avx512.mask.pmov.qd.mem.256\0"
  "llvm.x86.avx512.mask.pmov.qd.mem.512\0"
  "llvm.x86.avx512.mask.pmov.qw.128\0"
  "llvm.x86.avx512.mask.pmov.qw.256\0"
  "llvm.x86.avx512.mask.pmov.qw.512\0"
  "llvm.x86.avx512.mask.pmov.qw.mem.128\0"
  "llvm.x86.avx512.mask.pmov.qw.mem.256\0"
  "llvm.x86.avx512.mask.pmov.qw.mem.512\0"
  "llvm.x86.avx512.mask.pmov.wb.128\0"
  "llvm.x86.avx512.mask.pmov.wb.mem.128\0"
  "llvm.x86.avx512.mask.pmov.wb.mem.256\0"
  "llvm.x86.avx512.mask.pmov.wb.mem.512\0"
  "llvm.x86.avx512.mask.pmovs.db.128\0"
  "llvm.x86.avx512.mask.pmovs.db.256\0"
  "llvm.x86.avx512.mask.pmovs.db.512\0"
  "llvm.x86.avx512.mask.pmovs.db.mem.128\0"
  "llvm.x86.avx512.mask.pmovs.db.mem.256\0"
  "llvm.x86.avx512.mask.pmovs.db.mem.512\0"
  "llvm.x86.avx512.mask.pmovs.dw.128\0"
  "llvm.x86.avx512.mask.pmovs.dw.256\0"
  "llvm.x86.avx512.mask.pmovs.dw.512\0"
  "llvm.x86.avx512.mask.pmovs.dw.mem.128\0"
  "llvm.x86.avx512.mask.pmovs.dw.mem.256\0"
  "llvm.x86.avx512.mask.pmovs.dw.mem.512\0"
  "llvm.x86.avx512.mask.pmovs.qb.128\0"
  "llvm.x86.avx512.mask.pmovs.qb.256\0"
  "llvm.x86.avx512.mask.pmovs.qb.512\0"
  "llvm.x86.avx512.mask.pmovs.qb.mem.128\0"
  "llvm.x86.avx512.mask.pmovs.qb.mem.256\0"
  "llvm.x86.avx512.mask.pmovs.qb.mem.512\0"
  "llvm.x86.avx512.mask.pmovs.qd.128\0"
  "llvm.x86.avx512.mask.pmovs.qd.256\0"
  "llvm.x86.avx512.mask.pmovs.qd.512\0"
  "llvm.x86.avx512.mask.pmovs.qd.mem.128\0"
  "llvm.x86.avx512.mask.pmovs.qd.mem.256\0"
  "llvm.x86.avx512.mask.pmovs.qd.mem.512\0"
  "llvm.x86.avx512.mask.pmovs.qw.128\0"
  "llvm.x86.avx512.mask.pmovs.qw.256\0"
  "llvm.x86.avx512.mask.pmovs.qw.512\0"
  "llvm.x86.avx512.mask.pmovs.qw.mem.128\0"
  "llvm.x86.avx512.mask.pmovs.qw.mem.256\0"
  "llvm.x86.avx512.mask.pmovs.qw.mem.512\0"
  "llvm.x86.avx512.mask.pmovs.wb.128\0"
  "llvm.x86.avx512.mask.pmovs.wb.256\0"
  "llvm.x86.avx512.mask.pmovs.wb.512\0"
  "llvm.x86.avx512.mask.pmovs.wb.mem.128\0"
  "llvm.x86.avx512.mask.pmovs.wb.mem.256\0"
  "llvm.x86.avx512.mask.pmovs.wb.mem.512\0"
  "llvm.x86.avx512.mask.pmovus.db.128\0"
  "llvm.x86.avx512.mask.pmovus.db.256\0"
  "llvm.x86.avx512.mask.pmovus.db.512\0"
  "llvm.x86.avx512.mask.pmovus.db.mem.128\0"
  "llvm.x86.avx512.mask.pmovus.db.mem.256\0"
  "llvm.x86.avx512.mask.pmovus.db.mem.512\0"
  "llvm.x86.avx512.mask.pmovus.dw.128\0"
  "llvm.x86.avx512.mask.pmovus.dw.256\0"
  "llvm.x86.avx512.mask.pmovus.dw.512\0"
  "llvm.x86.avx512.mask.pmovus.dw.mem.128\0"
  "llvm.x86.avx512.mask.pmovus.dw.mem.256\0"
  "llvm.x86.avx512.mask.pmovus.dw.mem.512\0"
  "llvm.x86.avx512.mask.pmovus.qb.128\0"
  "llvm.x86.avx512.mask.pmovus.qb.256\0"
  "llvm.x86.avx512.mask.pmovus.qb.512\0"
  "llvm.x86.avx512.mask.pmovus.qb.mem.128\0"
  "llvm.x86.avx512.mask.pmovus.qb.mem.256\0"
  "llvm.x86.avx512.mask.pmovus.qb.mem.512\0"
  "llvm.x86.avx512.mask.pmovus.qd.128\0"
  "llvm.x86.avx512.mask.pmovus.qd.256\0"
  "llvm.x86.avx512.mask.pmovus.qd.512\0"
  "llvm.x86.avx512.mask.pmovus.qd.mem.128\0"
  "llvm.x86.avx512.mask.pmovus.qd.mem.256\0"
  "llvm.x86.avx512.mask.pmovus.qd.mem.512\0"
  "llvm.x86.avx512.mask.pmovus.qw.128\0"
  "llvm.x86.avx512.mask.pmovus.qw.256\0"
  "llvm.x86.avx512.mask.pmovus.qw.512\0"
  "llvm.x86.avx512.mask.pmovus.qw.mem.128\0"
  "llvm.x86.avx512.mask.pmovus.qw.mem.256\0"
  "llvm.x86.avx512.mask.pmovus.qw.mem.512\0"
  "llvm.x86.avx512.mask.pmovus.wb.128\0"
  "llvm.x86.avx512.mask.pmovus.wb.256\0"
  "llvm.x86.avx512.mask.pmovus.wb.512\0"
  "llvm.x86.avx512.mask.pmovus.wb.mem.128\0"
  "llvm.x86.avx512.mask.pmovus.wb.mem.256\0"
  "llvm.x86.avx512.mask.pmovus.wb.mem.512\0"
  "llvm.x86.avx512.mask.range.pd.128\0"
  "llvm.x86.avx512.mask.range.pd.256\0"
  "llvm.x86.avx512.mask.range.pd.512\0"
  "llvm.x86.avx512.mask.range.ps.128\0"
  "llvm.x86.avx512.mask.range.ps.256\0"
  "llvm.x86.avx512.mask.range.ps.512\0"
  "llvm.x86.avx512.mask.range.sd\0"
  "llvm.x86.avx512.mask.range.ss\0"
  "llvm.x86.avx512.mask.reduce.pd.128\0"
  "llvm.x86.avx512.mask.reduce.pd.256\0"
  "llvm.x86.avx512.mask.reduce.pd.512\0"
  "llvm.x86.avx512.mask.reduce.ps.128\0"
  "llvm.x86.avx512.mask.reduce.ps.256\0"
  "llvm.x86.avx512.mask.reduce.ps.512\0"
  "llvm.x86.avx512.mask.reduce.sd\0"
  "llvm.x86.avx512.mask.reduce.ss\0"
  "llvm.x86.avx512.mask.rndscale.pd.128\0"
  "llvm.x86.avx512.mask.rndscale.pd.256\0"
  "llvm.x86.avx512.mask.rndscale.pd.512\0"
  "llvm.x86.avx512.mask.rndscale.ps.128\0"
  "llvm.x86.avx512.mask.rndscale.ps.256\0"
  "llvm.x86.avx512.mask.rndscale.ps.512\0"
  "llvm.x86.avx512.mask.rndscale.sd\0"
  "llvm.x86.avx512.mask.rndscale.ss\0"
  "llvm.x86.avx512.mask.scalef.pd.128\0"
  "llvm.x86.avx512.mask.scalef.pd.256\0"
  "llvm.x86.avx512.mask.scalef.pd.512\0"
  "llvm.x86.avx512.mask.scalef.ps.128\0"
  "llvm.x86.avx512.mask.scalef.ps.256\0"
  "llvm.x86.avx512.mask.scalef.ps.512\0"
  "llvm.x86.avx512.mask.scalef.sd\0"
  "llvm.x86.avx512.mask.scalef.ss\0"
  "llvm.x86.avx512.mask.scatter.dpd.512\0"
  "llvm.x86.avx512.mask.scatter.dpi.512\0"
  "llvm.x86.avx512.mask.scatter.dpq.512\0"
  "llvm.x86.avx512.mask.scatter.dps.512\0"
  "llvm.x86.avx512.mask.scatter.qpd.512\0"
  "llvm.x86.avx512.mask.scatter.qpi.512\0"
  "llvm.x86.avx512.mask.scatter.qpq.512\0"
  "llvm.x86.avx512.mask.scatter.qps.512\0"
  "llvm.x86.avx512.mask.scatterdiv2.df\0"
  "llvm.x86.avx512.mask.scatterdiv2.di\0"
  "llvm.x86.avx512.mask.scatterdiv4.df\0"
  "llvm.x86.avx512.mask.scatterdiv4.di\0"
  "llvm.x86.avx512.mask.scatterdiv4.sf\0"
  "llvm.x86.avx512.mask.scatterdiv4.si\0"
  "llvm.x86.avx512.mask.scatterdiv8.sf\0"
  "llvm.x86.avx512.mask.scatterdiv8.si\0"
  "llvm.x86.avx512.mask.scattersiv2.df\0"
  "llvm.x86.avx512.mask.scattersiv2.di\0"
  "llvm.x86.avx512.mask.scattersiv4.df\0"
  "llvm.x86.avx512.mask.scattersiv4.di\0"
  "llvm.x86.avx512.mask.scattersiv4.sf\0"
  "llvm.x86.avx512.mask.scattersiv4.si\0"
  "llvm.x86.avx512.mask.scattersiv8.sf\0"
  "llvm.x86.avx512.mask.scattersiv8.si\0"
  "llvm.x86.avx512.mask.sqrt.sd\0"
  "llvm.x86.avx512.mask.sqrt.ss\0"
  "llvm.x86.avx512.mask.sub.sd.round\0"
  "llvm.x86.avx512.mask.sub.ss.round\0"
  "llvm.x86.avx512.mask.vcvtph2ps.512\0"
  "llvm.x86.avx512.mask.vcvtps2ph.128\0"
  "llvm.x86.avx512.mask.vcvtps2ph.256\0"
  "llvm.x86.avx512.mask.vcvtps2ph.512\0"
  "llvm.x86.avx512.maskz.fixupimm.pd.128\0"
  "llvm.x86.avx512.maskz.fixupimm.pd.256\0"
  "llvm.x86.avx512.maskz.fixupimm.pd.512\0"
  "llvm.x86.avx512.maskz.fixupimm.ps.128\0"
  "llvm.x86.avx512.maskz.fixupimm.ps.256\0"
  "llvm.x86.avx512.maskz.fixupimm.ps.512\0"
  "llvm.x86.avx512.maskz.fixupimm.sd\0"
  "llvm.x86.avx512.maskz.fixupimm.ss\0"
  "llvm.x86.avx512.max.pd.512\0"
  "llvm.x86.avx512.max.ps.512\0"
  "llvm.x86.avx512.min.pd.512\0"
  "llvm.x86.avx512.min.ps.512\0"
  "llvm.x86.avx512.mul.pd.512\0"
  "llvm.x86.avx512.mul.ps.512\0"
  "llvm.x86.avx512.packssdw.512\0"
  "llvm.x86.avx512.packsswb.512\0"
  "llvm.x86.avx512.packusdw.512\0"
  "llvm.x86.avx512.packuswb.512\0"
  "llvm.x86.avx512.pavg.b.512\0"
  "llvm.x86.avx512.pavg.w.512\0"
  "llvm.x86.avx512.permvar.df.256\0"
  "llvm.x86.avx512.permvar.df.512\0"
  "llvm.x86.avx512.permvar.di.256\0"
  "llvm.x86.avx512.permvar.di.512\0"
  "llvm.x86.avx512.permvar.hi.128\0"
  "llvm.x86.avx512.permvar.hi.256\0"
  "llvm.x86.avx512.permvar.hi.512\0"
  "llvm.x86.avx512.permvar.qi.128\0"
  "llvm.x86.avx512.permvar.qi.256\0"
  "llvm.x86.avx512.permvar.qi.512\0"
  "llvm.x86.avx512.permvar.sf.512\0"
  "llvm.x86.avx512.permvar.si.512\0"
  "llvm.x86.avx512.pmaddubs.w.512\0"
  "llvm.x86.avx512.pmaddw.d.512\0"
  "llvm.x86.avx512.pmul.hr.sw.512\0"
  "llvm.x86.avx512.pmulh.w.512\0"
  "llvm.x86.avx512.pmulhu.w.512\0"
  "llvm.x86.avx512.pmultishift.qb.128\0"
  "llvm.x86.avx512.pmultishift.qb.256\0"
  "llvm.x86.avx512.pmultishift.qb.512\0"
  "llvm.x86.avx512.psad.bw.512\0"
  "llvm.x86.avx512.pshuf.b.512\0"
  "llvm.x86.avx512.psll.d.512\0"
  "llvm.x86.avx512.psll.q.512\0"
  "llvm.x86.avx512.psll.w.512\0"
  "llvm.x86.avx512.pslli.d.512\0"
  "llvm.x86.avx512.pslli.q.512\0"
  "llvm.x86.avx512.pslli.w.512\0"
  "llvm.x86.avx512.psllv.d.512\0"
  "llvm.x86.avx512.psllv.q.512\0"
  "llvm.x86.avx512.psllv.w.128\0"
  "llvm.x86.avx512.psllv.w.256\0"
  "llvm.x86.avx512.psllv.w.512\0"
  "llvm.x86.avx512.psra.d.512\0"
  "llvm.x86.avx512.psra.q.128\0"
  "llvm.x86.avx512.psra.q.256\0"
  "llvm.x86.avx512.psra.q.512\0"
  "llvm.x86.avx512.psra.w.512\0"
  "llvm.x86.avx512.psrai.d.512\0"
  "llvm.x86.avx512.psrai.q.128\0"
  "llvm.x86.avx512.psrai.q.256\0"
  "llvm.x86.avx512.psrai.q.512\0"
  "llvm.x86.avx512.psrai.w.512\0"
  "llvm.x86.avx512.psrav.d.512\0"
  "llvm.x86.avx512.psrav.q.128\0"
  "llvm.x86.avx512.psrav.q.256\0"
  "llvm.x86.avx512.psrav.q.512\0"
  "llvm.x86.avx512.psrav.w.128\0"
  "llvm.x86.avx512.psrav.w.256\0"
  "llvm.x86.avx512.psrav.w.512\0"
  "llvm.x86.avx512.psrl.d.512\0"
  "llvm.x86.avx512.psrl.q.512\0"
  "llvm.x86.avx512.psrl.w.512\0"
  "llvm.x86.avx512.psrli.d.512\0"
  "llvm.x86.avx512.psrli.q.512\0"
  "llvm.x86.avx512.psrli.w.512\0"
  "llvm.x86.avx512.psrlv.d.512\0"
  "llvm.x86.avx512.psrlv.q.512\0"
  "llvm.x86.avx512.psrlv.w.128\0"
  "llvm.x86.avx512.psrlv.w.256\0"
  "llvm.x86.avx512.psrlv.w.512\0"
  "llvm.x86.avx512.pternlog.d.128\0"
  "llvm.x86.avx512.pternlog.d.256\0"
  "llvm.x86.avx512.pternlog.d.512\0"
  "llvm.x86.avx512.pternlog.q.128\0"
  "llvm.x86.avx512.pternlog.q.256\0"
  "llvm.x86.avx512.pternlog.q.512\0"
  "llvm.x86.avx512.rcp14.pd.128\0"
  "llvm.x86.avx512.rcp14.pd.256\0"
  "llvm.x86.avx512.rcp14.pd.512\0"
  "llvm.x86.avx512.rcp14.ps.128\0"
  "llvm.x86.avx512.rcp14.ps.256\0"
  "llvm.x86.avx512.rcp14.ps.512\0"
  "llvm.x86.avx512.rcp14.sd\0"
  "llvm.x86.avx512.rcp14.ss\0"
  "llvm.x86.avx512.rsqrt14.pd.128\0"
  "llvm.x86.avx512.rsqrt14.pd.256\0"
  "llvm.x86.avx512.rsqrt14.pd.512\0"
  "llvm.x86.avx512.rsqrt14.ps.128\0"
  "llvm.x86.avx512.rsqrt14.ps.256\0"
  "llvm.x86.avx512.rsqrt14.ps.512\0"
  "llvm.x86.avx512.rsqrt14.sd\0"
  "llvm.x86.avx512.rsqrt14.ss\0"
  "llvm.x86.avx512.scatter.dpd.512\0"
  "llvm.x86.avx512.scatter.dpi.512\0"
  "llvm.x86.avx512.scatter.dpq.512\0"
  "llvm.x86.avx512.scatter.dps.512\0"
  "llvm.x86.avx512.scatter.qpd.512\0"
  "llvm.x86.avx512.scatter.qpi.512\0"
  "llvm.x86.avx512.scatter.qpq.512\0"
  "llvm.x86.avx512.scatter.qps.512\0"
  "llvm.x86.avx512.scatterdiv2.df\0"
  "llvm.x86.avx512.scatterdiv2.di\0"
  "llvm.x86.avx512.scatterdiv4.df\0"
  "llvm.x86.avx512.scatterdiv4.di\0"
  "llvm.x86.avx512.scatterdiv4.sf\0"
  "llvm.x86.avx512.scatterdiv4.si\0"
  "llvm.x86.avx512.scatterdiv8.sf\0"
  "llvm.x86.avx512.scatterdiv8.si\0"
  "llvm.x86.avx512.scattersiv2.df\0"
  "llvm.x86.avx512.scattersiv2.di\0"
  "llvm.x86.avx512.scattersiv4.df\0"
  "llvm.x86.avx512.scattersiv4.di\0"
  "llvm.x86.avx512.scattersiv4.sf\0"
  "llvm.x86.avx512.scattersiv4.si\0"
  "llvm.x86.avx512.scattersiv8.sf\0"
  "llvm.x86.avx512.scattersiv8.si\0"
  "llvm.x86.avx512.sitofp.round\0"
  "llvm.x86.avx512.sqrt.pd.512\0"
  "llvm.x86.avx512.sqrt.ps.512\0"
  "llvm.x86.avx512.sub.pd.512\0"
  "llvm.x86.avx512.sub.ps.512\0"
  "llvm.x86.avx512.uitofp.round\0"
  "llvm.x86.avx512.vcomi.sd\0"
  "llvm.x86.avx512.vcomi.ss\0"
  "llvm.x86.avx512.vcvtsd2si32\0"
  "llvm.x86.avx512.vcvtsd2si64\0"
  "llvm.x86.avx512.vcvtsd2usi32\0"
  "llvm.x86.avx512.vcvtsd2usi64\0"
  "llvm.x86.avx512.vcvtss2si32\0"
  "llvm.x86.avx512.vcvtss2si64\0"
  "llvm.x86.avx512.vcvtss2usi32\0"
  "llvm.x86.avx512.vcvtss2usi64\0"
  "llvm.x86.avx512.vfmadd.f32\0"
  "llvm.x86.avx512.vfmadd.f64\0"
  "llvm.x86.avx512.vfmadd.pd.512\0"
  "llvm.x86.avx512.vfmadd.ps.512\0"
  "llvm.x86.avx512.vfmaddsub.pd.512\0"
  "llvm.x86.avx512.vfmaddsub.ps.512\0"
  "llvm.x86.avx512.vp2intersect.d.128\0"
  "llvm.x86.avx512.vp2intersect.d.256\0"
  "llvm.x86.avx512.vp2intersect.d.512\0"
  "llvm.x86.avx512.vp2intersect.q.128\0"
  "llvm.x86.avx512.vp2intersect.q.256\0"
  "llvm.x86.avx512.vp2intersect.q.512\0"
  "llvm.x86.avx512.vpdpbusd.128\0"
  "llvm.x86.avx512.vpdpbusd.256\0"
  "llvm.x86.avx512.vpdpbusd.512\0"
  "llvm.x86.avx512.vpdpbusds.128\0"
  "llvm.x86.avx512.vpdpbusds.256\0"
  "llvm.x86.avx512.vpdpbusds.512\0"
  "llvm.x86.avx512.vpdpwssd.128\0"
  "llvm.x86.avx512.vpdpwssd.256\0"
  "llvm.x86.avx512.vpdpwssd.512\0"
  "llvm.x86.avx512.vpdpwssds.128\0"
  "llvm.x86.avx512.vpdpwssds.256\0"
  "llvm.x86.avx512.vpdpwssds.512\0"
  "llvm.x86.avx512.vpermi2var.d.128\0"
  "llvm.x86.avx512.vpermi2var.d.256\0"
  "llvm.x86.avx512.vpermi2var.d.512\0"
  "llvm.x86.avx512.vpermi2var.hi.128\0"
  "llvm.x86.avx512.vpermi2var.hi.256\0"
  "llvm.x86.avx512.vpermi2var.hi.512\0"
  "llvm.x86.avx512.vpermi2var.pd.128\0"
  "llvm.x86.avx512.vpermi2var.pd.256\0"
  "llvm.x86.avx512.vpermi2var.pd.512\0"
  "llvm.x86.avx512.vpermi2var.ps.128\0"
  "llvm.x86.avx512.vpermi2var.ps.256\0"
  "llvm.x86.avx512.vpermi2var.ps.512\0"
  "llvm.x86.avx512.vpermi2var.q.128\0"
  "llvm.x86.avx512.vpermi2var.q.256\0"
  "llvm.x86.avx512.vpermi2var.q.512\0"
  "llvm.x86.avx512.vpermi2var.qi.128\0"
  "llvm.x86.avx512.vpermi2var.qi.256\0"
  "llvm.x86.avx512.vpermi2var.qi.512\0"
  "llvm.x86.avx512.vpermilvar.pd.512\0"
  "llvm.x86.avx512.vpermilvar.ps.512\0"
  "llvm.x86.avx512.vpmadd52h.uq.128\0"
  "llvm.x86.avx512.vpmadd52h.uq.256\0"
  "llvm.x86.avx512.vpmadd52h.uq.512\0"
  "llvm.x86.avx512.vpmadd52l.uq.128\0"
  "llvm.x86.avx512.vpmadd52l.uq.256\0"
  "llvm.x86.avx512.vpmadd52l.uq.512\0"
  "llvm.x86.avx512.vpshufbitqmb.128\0"
  "llvm.x86.avx512.vpshufbitqmb.256\0"
  "llvm.x86.avx512.vpshufbitqmb.512\0"
  "llvm.x86.avx512bf16.cvtne2ps2bf16.128\0"
  "llvm.x86.avx512bf16.cvtne2ps2bf16.256\0"
  "llvm.x86.avx512bf16.cvtne2ps2bf16.512\0"
  "llvm.x86.avx512bf16.cvtneps2bf16.256\0"
  "llvm.x86.avx512bf16.cvtneps2bf16.512\0"
  "llvm.x86.avx512bf16.dpbf16ps.128\0"
  "llvm.x86.avx512bf16.dpbf16ps.256\0"
  "llvm.x86.avx512bf16.dpbf16ps.512\0"
  "llvm.x86.avx512bf16.mask.cvtneps2bf16.128\0"
  "llvm.x86.avx512fp16.add.ph.512\0"
  "llvm.x86.avx512fp16.div.ph.512\0"
  "llvm.x86.avx512fp16.fpclass.ph.128\0"
  "llvm.x86.avx512fp16.fpclass.ph.256\0"
  "llvm.x86.avx512fp16.fpclass.ph.512\0"
  "llvm.x86.avx512fp16.mask.add.sh.round\0"
  "llvm.x86.avx512fp16.mask.cmp.ph.128\0"
  "llvm.x86.avx512fp16.mask.cmp.ph.256\0"
  "llvm.x86.avx512fp16.mask.cmp.ph.512\0"
  "llvm.x86.avx512fp16.mask.cmp.sh\0"
  "llvm.x86.avx512fp16.mask.div.sh.round\0"
  "llvm.x86.avx512fp16.mask.fpclass.sh\0"
  "llvm.x86.avx512fp16.mask.getexp.ph.128\0"
  "llvm.x86.avx512fp16.mask.getexp.ph.256\0"
  "llvm.x86.avx512fp16.mask.getexp.ph.512\0"
  "llvm.x86.avx512fp16.mask.getexp.sh\0"
  "llvm.x86.avx512fp16.mask.getmant.ph.128\0"
  "llvm.x86.avx512fp16.mask.getmant.ph.256\0"
  "llvm.x86.avx512fp16.mask.getmant.ph.512\0"
  "llvm.x86.avx512fp16.mask.getmant.sh\0"
  "llvm.x86.avx512fp16.mask.max.sh.round\0"
  "llvm.x86.avx512fp16.mask.min.sh.round\0"
  "llvm.x86.avx512fp16.mask.mul.sh.round\0"
  "llvm.x86.avx512fp16.mask.rcp.ph.128\0"
  "llvm.x86.avx512fp16.mask.rcp.ph.256\0"
  "llvm.x86.avx512fp16.mask.rcp.ph.512\0"
  "llvm.x86.avx512fp16.mask.rcp.sh\0"
  "llvm.x86.avx512fp16.mask.reduce.ph.128\0"
  "llvm.x86.avx512fp16.mask.reduce.ph.256\0"
  "llvm.x86.avx512fp16.mask.reduce.ph.512\0"
  "llvm.x86.avx512fp16.mask.reduce.sh\0"
  "llvm.x86.avx512fp16.mask.rndscale.ph.128\0"
  "llvm.x86.avx512fp16.mask.rndscale.ph.256\0"
  "llvm.x86.avx512fp16.mask.rndscale.ph.512\0"
  "llvm.x86.avx512fp16.mask.rndscale.sh\0"
  "llvm.x86.avx512fp16.mask.rsqrt.ph.128\0"
  "llvm.x86.avx512fp16.mask.rsqrt.ph.256\0"
  "llvm.x86.avx512fp16.mask.rsqrt.ph.512\0"
  "llvm.x86.avx512fp16.mask.rsqrt.sh\0"
  "llvm.x86.avx512fp16.mask.scalef.ph.128\0"
  "llvm.x86.avx512fp16.mask.scalef.ph.256\0"
  "llvm.x86.avx512fp16.mask.scalef.ph.512\0"
  "llvm.x86.avx512fp16.mask.scalef.sh\0"
  "llvm.x86.avx512fp16.mask.sqrt.sh\0"
  "llvm.x86.avx512fp16.mask.sub.sh.round\0"
  "llvm.x86.avx512fp16.mask.vcvtdq2ph.128\0"
  "llvm.x86.avx512fp16.mask.vcvtpd2ph.128\0"
  "llvm.x86.avx512fp16.mask.vcvtpd2ph.256\0"
  "llvm.x86.avx512fp16.mask.vcvtpd2ph.512\0"
  "llvm.x86.avx512fp16.mask.vcvtph2dq.128\0"
  "llvm.x86.avx512fp16.mask.vcvtph2dq.256\0"
  "llvm.x86.avx512fp16.mask.vcvtph2dq.512\0"
  "llvm.x86.avx512fp16.mask.vcvtph2pd.128\0"
  "llvm.x86.avx512fp16.mask.vcvtph2pd.256\0"
  "llvm.x86.avx512fp16.mask.vcvtph2pd.512\0"
  "llvm.x86.avx512fp16.mask.vcvtph2psx.128\0"
  "llvm.x86.avx512fp16.mask.vcvtph2psx.256\0"
  "llvm.x86.avx512fp16.mask.vcvtph2psx.512\0"
  "llvm.x86.avx512fp16.mask.vcvtph2qq.128\0"
  "llvm.x86.avx512fp16.mask.vcvtph2qq.256\0"
  "llvm.x86.avx512fp16.mask.vcvtph2qq.512\0"
  "llvm.x86.avx512fp16.mask.vcvtph2udq.128\0"
  "llvm.x86.avx512fp16.mask.vcvtph2udq.256\0"
  "llvm.x86.avx512fp16.mask.vcvtph2udq.512\0"
  "llvm.x86.avx512fp16.mask.vcvtph2uqq.128\0"
  "llvm.x86.avx512fp16.mask.vcvtph2uqq.256\0"
  "llvm.x86.avx512fp16.mask.vcvtph2uqq.512\0"
  "llvm.x86.avx512fp16.mask.vcvtph2uw.128\0"
  "llvm.x86.avx512fp16.mask.vcvtph2uw.256\0"
  "llvm.x86.avx512fp16.mask.vcvtph2uw.512\0"
  "llvm.x86.avx512fp16.mask.vcvtph2w.128\0"
  "llvm.x86.avx512fp16.mask.vcvtph2w.256\0"
  "llvm.x86.avx512fp16.mask.vcvtph2w.512\0"
  "llvm.x86.avx512fp16.mask.vcvtps2phx.128\0"
  "llvm.x86.avx512fp16.mask.vcvtps2phx.256\0"
  "llvm.x86.avx512fp16.mask.vcvtps2phx.512\0"
  "llvm.x86.avx512fp16.mask.vcvtqq2ph.128\0"
  "llvm.x86.avx512fp16.mask.vcvtqq2ph.256\0"
  "llvm.x86.avx512fp16.mask.vcvtsd2sh.round\0"
  "llvm.x86.avx512fp16.mask.vcvtsh2sd.round\0"
  "llvm.x86.avx512fp16.mask.vcvtsh2ss.round\0"
  "llvm.x86.avx512fp16.mask.vcvtss2sh.round\0"
  "llvm.x86.avx512fp16.mask.vcvttph2dq.128\0"
  "llvm.x86.avx512fp16.mask.vcvttph2dq.256\0"
  "llvm.x86.avx512fp16.mask.vcvttph2dq.512\0"
  "llvm.x86.avx512fp16.mask.vcvttph2qq.128\0"
  "llvm.x86.avx512fp16.mask.vcvttph2qq.256\0"
  "llvm.x86.avx512fp16.mask.vcvttph2qq.512\0"
  "llvm.x86.avx512fp16.mask.vcvttph2udq.128\0"
  "llvm.x86.avx512fp16.mask.vcvttph2udq.256\0"
  "llvm.x86.avx512fp16.mask.vcvttph2udq.512\0"
  "llvm.x86.avx512fp16.mask.vcvttph2uqq.128\0"
  "llvm.x86.avx512fp16.mask.vcvttph2uqq.256\0"
  "llvm.x86.avx512fp16.mask.vcvttph2uqq.512\0"
  "llvm.x86.avx512fp16.mask.vcvttph2uw.128\0"
  "llvm.x86.avx512fp16.mask.vcvttph2uw.256\0"
  "llvm.x86.avx512fp16.mask.vcvttph2uw.512\0"
  "llvm.x86.avx512fp16.mask.vcvttph2w.128\0"
  "llvm.x86.avx512fp16.mask.vcvttph2w.256\0"
  "llvm.x86.avx512fp16.mask.vcvttph2w.512\0"
  "llvm.x86.avx512fp16.mask.vcvtudq2ph.128\0"
  "llvm.x86.avx512fp16.mask.vcvtuqq2ph.128\0"
  "llvm.x86.avx512fp16.mask.vcvtuqq2ph.256\0"
  "llvm.x86.avx512fp16.mask.vfcmadd.cph.128\0"
  "llvm.x86.avx512fp16.mask.vfcmadd.cph.256\0"
  "llvm.x86.avx512fp16.mask.vfcmadd.cph.512\0"
  "llvm.x86.avx512fp16.mask.vfcmadd.csh\0"
  "llvm.x86.avx512fp16.mask.vfcmul.cph.128\0"
  "llvm.x86.avx512fp16.mask.vfcmul.cph.256\0"
  "llvm.x86.avx512fp16.mask.vfcmul.cph.512\0"
  "llvm.x86.avx512fp16.mask.vfcmul.csh\0"
  "llvm.x86.avx512fp16.mask.vfmadd.cph.128\0"
  "llvm.x86.avx512fp16.mask.vfmadd.cph.256\0"
  "llvm.x86.avx512fp16.mask.vfmadd.cph.512\0"
  "llvm.x86.avx512fp16.mask.vfmadd.csh\0"
  "llvm.x86.avx512fp16.mask.vfmul.cph.128\0"
  "llvm.x86.avx512fp16.mask.vfmul.cph.256\0"
  "llvm.x86.avx512fp16.mask.vfmul.cph.512\0"
  "llvm.x86.avx512fp16.mask.vfmul.csh\0"
  "llvm.x86.avx512fp16.maskz.vfcmadd.cph.128\0"
  "llvm.x86.avx512fp16.maskz.vfcmadd.cph.256\0"
  "llvm.x86.avx512fp16.maskz.vfcmadd.cph.512\0"
  "llvm.x86.avx512fp16.maskz.vfcmadd.csh\0"
  "llvm.x86.avx512fp16.maskz.vfmadd.cph.128\0"
  "llvm.x86.avx512fp16.maskz.vfmadd.cph.256\0"
  "llvm.x86.avx512fp16.maskz.vfmadd.cph.512\0"
  "llvm.x86.avx512fp16.maskz.vfmadd.csh\0"
  "llvm.x86.avx512fp16.max.ph.128\0"
  "llvm.x86.avx512fp16.max.ph.256\0"
  "llvm.x86.avx512fp16.max.ph.512\0"
  "llvm.x86.avx512fp16.min.ph.128\0"
  "llvm.x86.avx512fp16.min.ph.256\0"
  "llvm.x86.avx512fp16.min.ph.512\0"
  "llvm.x86.avx512fp16.mul.ph.512\0"
  "llvm.x86.avx512fp16.sqrt.ph.512\0"
  "llvm.x86.avx512fp16.sub.ph.512\0"
  "llvm.x86.avx512fp16.vcomi.sh\0"
  "llvm.x86.avx512fp16.vcvtsh2si32\0"
  "llvm.x86.avx512fp16.vcvtsh2si64\0"
  "llvm.x86.avx512fp16.vcvtsh2usi32\0"
  "llvm.x86.avx512fp16.vcvtsh2usi64\0"
  "llvm.x86.avx512fp16.vcvtsi2sh\0"
  "llvm.x86.avx512fp16.vcvtsi642sh\0"
  "llvm.x86.avx512fp16.vcvttsh2si32\0"
  "llvm.x86.avx512fp16.vcvttsh2si64\0"
  "llvm.x86.avx512fp16.vcvttsh2usi32\0"
  "llvm.x86.avx512fp16.vcvttsh2usi64\0"
  "llvm.x86.avx512fp16.vcvtusi2sh\0"
  "llvm.x86.avx512fp16.vcvtusi642sh\0"
  "llvm.x86.avx512fp16.vfmadd.f16\0"
  "llvm.x86.avx512fp16.vfmadd.ph.512\0"
  "llvm.x86.avx512fp16.vfmaddsub.ph.128\0"
  "llvm.x86.avx512fp16.vfmaddsub.ph.256\0"
  "llvm.x86.avx512fp16.vfmaddsub.ph.512\0"
  "llvm.x86.axor32\0"
  "llvm.x86.axor64\0"
  "llvm.x86.bmi.bextr.32\0"
  "llvm.x86.bmi.bextr.64\0"
  "llvm.x86.bmi.bzhi.32\0"
  "llvm.x86.bmi.bzhi.64\0"
  "llvm.x86.bmi.pdep.32\0"
  "llvm.x86.bmi.pdep.64\0"
  "llvm.x86.bmi.pext.32\0"
  "llvm.x86.bmi.pext.64\0"
  "llvm.x86.cast.tile.to.vector\0"
  "llvm.x86.cast.vector.to.tile\0"
  "llvm.x86.cldemote\0"
  "llvm.x86.clflushopt\0"
  "llvm.x86.clrssbsy\0"
  "llvm.x86.clui\0"
  "llvm.x86.clwb\0"
  "llvm.x86.clzero\0"
  "llvm.x86.cmpccxadd32\0"
  "llvm.x86.cmpccxadd64\0"
  "llvm.x86.directstore32\0"
  "llvm.x86.directstore64\0"
  "llvm.x86.encodekey128\0"
  "llvm.x86.encodekey256\0"
  "llvm.x86.enqcmd\0"
  "llvm.x86.enqcmds\0"
  "llvm.x86.flags.read.u32\0"
  "llvm.x86.flags.read.u64\0"
  "llvm.x86.flags.write.u32\0"
  "llvm.x86.flags.write.u64\0"
  "llvm.x86.fma.vfmaddsub.pd\0"
  "llvm.x86.fma.vfmaddsub.pd.256\0"
  "llvm.x86.fma.vfmaddsub.ps\0"
  "llvm.x86.fma.vfmaddsub.ps.256\0"
  "llvm.x86.fxrstor\0"
  "llvm.x86.fxrstor64\0"
  "llvm.x86.fxsave\0"
  "llvm.x86.fxsave64\0"
  "llvm.x86.incsspd\0"
  "llvm.x86.incsspq\0"
  "llvm.x86.int\0"
  "llvm.x86.invpcid\0"
  "llvm.x86.ldtilecfg\0"
  "llvm.x86.ldtilecfg.internal\0"
  "llvm.x86.llwpcb\0"
  "llvm.x86.loadiwkey\0"
  "llvm.x86.lwpins32\0"
  "llvm.x86.lwpins64\0"
  "llvm.x86.lwpval32\0"
  "llvm.x86.lwpval64\0"
  "llvm.x86.mmx.emms\0"
  "llvm.x86.mmx.maskmovq\0"
  "llvm.x86.mmx.movnt.dq\0"
  "llvm.x86.mmx.packssdw\0"
  "llvm.x86.mmx.packsswb\0"
  "llvm.x86.mmx.packuswb\0"
  "llvm.x86.mmx.padd.b\0"
  "llvm.x86.mmx.padd.d\0"
  "llvm.x86.mmx.padd.q\0"
  "llvm.x86.mmx.padd.w\0"
  "llvm.x86.mmx.padds.b\0"
  "llvm.x86.mmx.padds.w\0"
  "llvm.x86.mmx.paddus.b\0"
  "llvm.x86.mmx.paddus.w\0"
  "llvm.x86.mmx.palignr.b\0"
  "llvm.x86.mmx.pand\0"
  "llvm.x86.mmx.pandn\0"
  "llvm.x86.mmx.pavg.b\0"
  "llvm.x86.mmx.pavg.w\0"
  "llvm.x86.mmx.pcmpeq.b\0"
  "llvm.x86.mmx.pcmpeq.d\0"
  "llvm.x86.mmx.pcmpeq.w\0"
  "llvm.x86.mmx.pcmpgt.b\0"
  "llvm.x86.mmx.pcmpgt.d\0"
  "llvm.x86.mmx.pcmpgt.w\0"
  "llvm.x86.mmx.pextr.w\0"
  "llvm.x86.mmx.pinsr.w\0"
  "llvm.x86.mmx.pmadd.wd\0"
  "llvm.x86.mmx.pmaxs.w\0"
  "llvm.x86.mmx.pmaxu.b\0"
  "llvm.x86.mmx.pmins.w\0"
  "llvm.x86.mmx.pminu.b\0"
  "llvm.x86.mmx.pmovmskb\0"
  "llvm.x86.mmx.pmulh.w\0"
  "llvm.x86.mmx.pmulhu.w\0"
  "llvm.x86.mmx.pmull.w\0"
  "llvm.x86.mmx.pmulu.dq\0"
  "llvm.x86.mmx.por\0"
  "llvm.x86.mmx.psad.bw\0"
  "llvm.x86.mmx.psll.d\0"
  "llvm.x86.mmx.psll.q\0"
  "llvm.x86.mmx.psll.w\0"
  "llvm.x86.mmx.pslli.d\0"
  "llvm.x86.mmx.pslli.q\0"
  "llvm.x86.mmx.pslli.w\0"
  "llvm.x86.mmx.psra.d\0"
  "llvm.x86.mmx.psra.w\0"
  "llvm.x86.mmx.psrai.d\0"
  "llvm.x86.mmx.psrai.w\0"
  "llvm.x86.mmx.psrl.d\0"
  "llvm.x86.mmx.psrl.q\0"
  "llvm.x86.mmx.psrl.w\0"
  "llvm.x86.mmx.psrli.d\0"
  "llvm.x86.mmx.psrli.q\0"
  "llvm.x86.mmx.psrli.w\0"
  "llvm.x86.mmx.psub.b\0"
  "llvm.x86.mmx.psub.d\0"
  "llvm.x86.mmx.psub.q\0"
  "llvm.x86.mmx.psub.w\0"
  "llvm.x86.mmx.psubs.b\0"
  "llvm.x86.mmx.psubs.w\0"
  "llvm.x86.mmx.psubus.b\0"
  "llvm.x86.mmx.psubus.w\0"
  "llvm.x86.mmx.punpckhbw\0"
  "llvm.x86.mmx.punpckhdq\0"
  "llvm.x86.mmx.punpckhwd\0"
  "llvm.x86.mmx.punpcklbw\0"
  "llvm.x86.mmx.punpckldq\0"
  "llvm.x86.mmx.punpcklwd\0"
  "llvm.x86.mmx.pxor\0"
  "llvm.x86.monitorx\0"
  "llvm.x86.movdir64b\0"
  "llvm.x86.movrsdi\0"
  "llvm.x86.movrshi\0"
  "llvm.x86.movrsqi\0"
  "llvm.x86.movrssi\0"
  "llvm.x86.mwaitx\0"
  "llvm.x86.pclmulqdq\0"
  "llvm.x86.pclmulqdq.256\0"
  "llvm.x86.pclmulqdq.512\0"
  "llvm.x86.prefetchrs\0"
  "llvm.x86.ptwrite32\0"
  "llvm.x86.ptwrite64\0"
  "llvm.x86.rdfsbase.32\0"
  "llvm.x86.rdfsbase.64\0"
  "llvm.x86.rdgsbase.32\0"
  "llvm.x86.rdgsbase.64\0"
  "llvm.x86.rdpid\0"
  "llvm.x86.rdpkru\0"
  "llvm.x86.rdpmc\0"
  "llvm.x86.rdpru\0"
  "llvm.x86.rdrand.16\0"
  "llvm.x86.rdrand.32\0"
  "llvm.x86.rdrand.64\0"
  "llvm.x86.rdseed.16\0"
  "llvm.x86.rdseed.32\0"
  "llvm.x86.rdseed.64\0"
  "llvm.x86.rdsspd\0"
  "llvm.x86.rdsspq\0"
  "llvm.x86.rdtsc\0"
  "llvm.x86.rdtscp\0"
  "llvm.x86.rstorssp\0"
  "llvm.x86.saveprevssp\0"
  "llvm.x86.seh.ehguard\0"
  "llvm.x86.seh.ehregnode\0"
  "llvm.x86.seh.lsda\0"
  "llvm.x86.senduipi\0"
  "llvm.x86.serialize\0"
  "llvm.x86.setssbsy\0"
  "llvm.x86.sha1msg1\0"
  "llvm.x86.sha1msg2\0"
  "llvm.x86.sha1nexte\0"
  "llvm.x86.sha1rnds4\0"
  "llvm.x86.sha256msg1\0"
  "llvm.x86.sha256msg2\0"
  "llvm.x86.sha256rnds2\0"
  "llvm.x86.slwpcb\0"
  "llvm.x86.sse.cmp.ps\0"
  "llvm.x86.sse.cmp.ss\0"
  "llvm.x86.sse.comieq.ss\0"
  "llvm.x86.sse.comige.ss\0"
  "llvm.x86.sse.comigt.ss\0"
  "llvm.x86.sse.comile.ss\0"
  "llvm.x86.sse.comilt.ss\0"
  "llvm.x86.sse.comineq.ss\0"
  "llvm.x86.sse.cvtpd2pi\0"
  "llvm.x86.sse.cvtpi2pd\0"
  "llvm.x86.sse.cvtpi2ps\0"
  "llvm.x86.sse.cvtps2pi\0"
  "llvm.x86.sse.cvtss2si\0"
  "llvm.x86.sse.cvtss2si64\0"
  "llvm.x86.sse.cvttpd2pi\0"
  "llvm.x86.sse.cvttps2pi\0"
  "llvm.x86.sse.cvttss2si\0"
  "llvm.x86.sse.cvttss2si64\0"
  "llvm.x86.sse.ldmxcsr\0"
  "llvm.x86.sse.max.ps\0"
  "llvm.x86.sse.max.ss\0"
  "llvm.x86.sse.min.ps\0"
  "llvm.x86.sse.min.ss\0"
  "llvm.x86.sse.movmsk.ps\0"
  "llvm.x86.sse.pshuf.w\0"
  "llvm.x86.sse.rcp.ps\0"
  "llvm.x86.sse.rcp.ss\0"
  "llvm.x86.sse.rsqrt.ps\0"
  "llvm.x86.sse.rsqrt.ss\0"
  "llvm.x86.sse.sfence\0"
  "llvm.x86.sse.stmxcsr\0"
  "llvm.x86.sse.ucomieq.ss\0"
  "llvm.x86.sse.ucomige.ss\0"
  "llvm.x86.sse.ucomigt.ss\0"
  "llvm.x86.sse.ucomile.ss\0"
  "llvm.x86.sse.ucomilt.ss\0"
  "llvm.x86.sse.ucomineq.ss\0"
  "llvm.x86.sse2.clflush\0"
  "llvm.x86.sse2.cmp.pd\0"
  "llvm.x86.sse2.cmp.sd\0"
  "llvm.x86.sse2.comieq.sd\0"
  "llvm.x86.sse2.comige.sd\0"
  "llvm.x86.sse2.comigt.sd\0"
  "llvm.x86.sse2.comile.sd\0"
  "llvm.x86.sse2.comilt.sd\0"
  "llvm.x86.sse2.comineq.sd\0"
  "llvm.x86.sse2.cvtpd2dq\0"
  "llvm.x86.sse2.cvtpd2ps\0"
  "llvm.x86.sse2.cvtps2dq\0"
  "llvm.x86.sse2.cvtsd2si\0"
  "llvm.x86.sse2.cvtsd2si64\0"
  "llvm.x86.sse2.cvtsd2ss\0"
  "llvm.x86.sse2.cvttpd2dq\0"
  "llvm.x86.sse2.cvttps2dq\0"
  "llvm.x86.sse2.cvttsd2si\0"
  "llvm.x86.sse2.cvttsd2si64\0"
  "llvm.x86.sse2.lfence\0"
  "llvm.x86.sse2.maskmov.dqu\0"
  "llvm.x86.sse2.max.pd\0"
  "llvm.x86.sse2.max.sd\0"
  "llvm.x86.sse2.mfence\0"
  "llvm.x86.sse2.min.pd\0"
  "llvm.x86.sse2.min.sd\0"
  "llvm.x86.sse2.movmsk.pd\0"
  "llvm.x86.sse2.packssdw.128\0"
  "llvm.x86.sse2.packsswb.128\0"
  "llvm.x86.sse2.packuswb.128\0"
  "llvm.x86.sse2.pause\0"
  "llvm.x86.sse2.pavg.b\0"
  "llvm.x86.sse2.pavg.w\0"
  "llvm.x86.sse2.pmadd.wd\0"
  "llvm.x86.sse2.pmovmskb.128\0"
  "llvm.x86.sse2.pmulh.w\0"
  "llvm.x86.sse2.pmulhu.w\0"
  "llvm.x86.sse2.psad.bw\0"
  "llvm.x86.sse2.psll.d\0"
  "llvm.x86.sse2.psll.q\0"
  "llvm.x86.sse2.psll.w\0"
  "llvm.x86.sse2.pslli.d\0"
  "llvm.x86.sse2.pslli.q\0"
  "llvm.x86.sse2.pslli.w\0"
  "llvm.x86.sse2.psra.d\0"
  "llvm.x86.sse2.psra.w\0"
  "llvm.x86.sse2.psrai.d\0"
  "llvm.x86.sse2.psrai.w\0"
  "llvm.x86.sse2.psrl.d\0"
  "llvm.x86.sse2.psrl.q\0"
  "llvm.x86.sse2.psrl.w\0"
  "llvm.x86.sse2.psrli.d\0"
  "llvm.x86.sse2.psrli.q\0"
  "llvm.x86.sse2.psrli.w\0"
  "llvm.x86.sse2.ucomieq.sd\0"
  "llvm.x86.sse2.ucomige.sd\0"
  "llvm.x86.sse2.ucomigt.sd\0"
  "llvm.x86.sse2.ucomile.sd\0"
  "llvm.x86.sse2.ucomilt.sd\0"
  "llvm.x86.sse2.ucomineq.sd\0"
  "llvm.x86.sse3.addsub.pd\0"
  "llvm.x86.sse3.addsub.ps\0"
  "llvm.x86.sse3.hadd.pd\0"
  "llvm.x86.sse3.hadd.ps\0"
  "llvm.x86.sse3.hsub.pd\0"
  "llvm.x86.sse3.hsub.ps\0"
  "llvm.x86.sse3.ldu.dq\0"
  "llvm.x86.sse3.monitor\0"
  "llvm.x86.sse3.mwait\0"
  "llvm.x86.sse41.blendvpd\0"
  "llvm.x86.sse41.blendvps\0"
  "llvm.x86.sse41.dppd\0"
  "llvm.x86.sse41.dpps\0"
  "llvm.x86.sse41.insertps\0"
  "llvm.x86.sse41.mpsadbw\0"
  "llvm.x86.sse41.packusdw\0"
  "llvm.x86.sse41.pblendvb\0"
  "llvm.x86.sse41.phminposuw\0"
  "llvm.x86.sse41.ptestc\0"
  "llvm.x86.sse41.ptestnzc\0"
  "llvm.x86.sse41.ptestz\0"
  "llvm.x86.sse41.round.pd\0"
  "llvm.x86.sse41.round.ps\0"
  "llvm.x86.sse41.round.sd\0"
  "llvm.x86.sse41.round.ss\0"
  "llvm.x86.sse42.crc32.32.16\0"
  "llvm.x86.sse42.crc32.32.32\0"
  "llvm.x86.sse42.crc32.32.8\0"
  "llvm.x86.sse42.crc32.64.64\0"
  "llvm.x86.sse42.pcmpestri128\0"
  "llvm.x86.sse42.pcmpestria128\0"
  "llvm.x86.sse42.pcmpestric128\0"
  "llvm.x86.sse42.pcmpestrio128\0"
  "llvm.x86.sse42.pcmpestris128\0"
  "llvm.x86.sse42.pcmpestriz128\0"
  "llvm.x86.sse42.pcmpestrm128\0"
  "llvm.x86.sse42.pcmpistri128\0"
  "llvm.x86.sse42.pcmpistria128\0"
  "llvm.x86.sse42.pcmpistric128\0"
  "llvm.x86.sse42.pcmpistrio128\0"
  "llvm.x86.sse42.pcmpistris128\0"
  "llvm.x86.sse42.pcmpistriz128\0"
  "llvm.x86.sse42.pcmpistrm128\0"
  "llvm.x86.sse4a.extrq\0"
  "llvm.x86.sse4a.extrqi\0"
  "llvm.x86.sse4a.insertq\0"
  "llvm.x86.sse4a.insertqi\0"
  "llvm.x86.ssse3.pabs.b\0"
  "llvm.x86.ssse3.pabs.d\0"
  "llvm.x86.ssse3.pabs.w\0"
  "llvm.x86.ssse3.phadd.d\0"
  "llvm.x86.ssse3.phadd.d.128\0"
  "llvm.x86.ssse3.phadd.sw\0"
  "llvm.x86.ssse3.phadd.sw.128\0"
  "llvm.x86.ssse3.phadd.w\0"
  "llvm.x86.ssse3.phadd.w.128\0"
  "llvm.x86.ssse3.phsub.d\0"
  "llvm.x86.ssse3.phsub.d.128\0"
  "llvm.x86.ssse3.phsub.sw\0"
  "llvm.x86.ssse3.phsub.sw.128\0"
  "llvm.x86.ssse3.phsub.w\0"
  "llvm.x86.ssse3.phsub.w.128\0"
  "llvm.x86.ssse3.pmadd.ub.sw\0"
  "llvm.x86.ssse3.pmadd.ub.sw.128\0"
  "llvm.x86.ssse3.pmul.hr.sw\0"
  "llvm.x86.ssse3.pmul.hr.sw.128\0"
  "llvm.x86.ssse3.pshuf.b\0"
  "llvm.x86.ssse3.pshuf.b.128\0"
  "llvm.x86.ssse3.psign.b\0"
  "llvm.x86.ssse3.psign.b.128\0"
  "llvm.x86.ssse3.psign.d\0"
  "llvm.x86.ssse3.psign.d.128\0"
  "llvm.x86.ssse3.psign.w\0"
  "llvm.x86.ssse3.psign.w.128\0"
  "llvm.x86.sttilecfg\0"
  "llvm.x86.stui\0"
  "llvm.x86.subborrow.32\0"
  "llvm.x86.subborrow.64\0"
  "llvm.x86.tbm.bextri.u32\0"
  "llvm.x86.tbm.bextri.u64\0"
  "llvm.x86.tcmmimfp16ps\0"
  "llvm.x86.tcmmimfp16ps.internal\0"
  "llvm.x86.tcmmrlfp16ps\0"
  "llvm.x86.tcmmrlfp16ps.internal\0"
  "llvm.x86.tcvtrowd2ps\0"
  "llvm.x86.tcvtrowd2ps.internal\0"
  "llvm.x86.tcvtrowd2psi\0"
  "llvm.x86.tcvtrowps2bf16h\0"
  "llvm.x86.tcvtrowps2bf16h.internal\0"
  "llvm.x86.tcvtrowps2bf16hi\0"
  "llvm.x86.tcvtrowps2bf16l\0"
  "llvm.x86.tcvtrowps2bf16l.internal\0"
  "llvm.x86.tcvtrowps2bf16li\0"
  "llvm.x86.tcvtrowps2phh\0"
  "llvm.x86.tcvtrowps2phh.internal\0"
  "llvm.x86.tcvtrowps2phhi\0"
  "llvm.x86.tcvtrowps2phl\0"
  "llvm.x86.tcvtrowps2phl.internal\0"
  "llvm.x86.tcvtrowps2phli\0"
  "llvm.x86.tdpbf16ps\0"
  "llvm.x86.tdpbf16ps.internal\0"
  "llvm.x86.tdpbf8ps\0"
  "llvm.x86.tdpbf8ps.internal\0"
  "llvm.x86.tdpbhf8ps\0"
  "llvm.x86.tdpbhf8ps.internal\0"
  "llvm.x86.tdpbssd\0"
  "llvm.x86.tdpbssd.internal\0"
  "llvm.x86.tdpbsud\0"
  "llvm.x86.tdpbsud.internal\0"
  "llvm.x86.tdpbusd\0"
  "llvm.x86.tdpbusd.internal\0"
  "llvm.x86.tdpbuud\0"
  "llvm.x86.tdpbuud.internal\0"
  "llvm.x86.tdpfp16ps\0"
  "llvm.x86.tdpfp16ps.internal\0"
  "llvm.x86.tdphbf8ps\0"
  "llvm.x86.tdphbf8ps.internal\0"
  "llvm.x86.tdphf8ps\0"
  "llvm.x86.tdphf8ps.internal\0"
  "llvm.x86.testui\0"
  "llvm.x86.tileloadd64\0"
  "llvm.x86.tileloadd64.internal\0"
  "llvm.x86.tileloaddrs64\0"
  "llvm.x86.tileloaddrs64.internal\0"
  "llvm.x86.tileloaddrst164\0"
  "llvm.x86.tileloaddrst164.internal\0"
  "llvm.x86.tileloaddt164\0"
  "llvm.x86.tileloaddt164.internal\0"
  "llvm.x86.tilemovrow\0"
  "llvm.x86.tilemovrow.internal\0"
  "llvm.x86.tilemovrowi\0"
  "llvm.x86.tilerelease\0"
  "llvm.x86.tilestored64\0"
  "llvm.x86.tilestored64.internal\0"
  "llvm.x86.tilezero\0"
  "llvm.x86.tilezero.internal\0"
  "llvm.x86.tmmultf32ps\0"
  "llvm.x86.tmmultf32ps.internal\0"
  "llvm.x86.tpause\0"
  "llvm.x86.umonitor\0"
  "llvm.x86.umwait\0"
  "llvm.x86.urdmsr\0"
  "llvm.x86.uwrmsr\0"
  "llvm.x86.vbcstnebf162ps128\0"
  "llvm.x86.vbcstnebf162ps256\0"
  "llvm.x86.vbcstnesh2ps128\0"
  "llvm.x86.vbcstnesh2ps256\0"
  "llvm.x86.vcvtneebf162ps128\0"
  "llvm.x86.vcvtneebf162ps256\0"
  "llvm.x86.vcvtneeph2ps128\0"
  "llvm.x86.vcvtneeph2ps256\0"
  "llvm.x86.vcvtneobf162ps128\0"
  "llvm.x86.vcvtneobf162ps256\0"
  "llvm.x86.vcvtneoph2ps128\0"
  "llvm.x86.vcvtneoph2ps256\0"
  "llvm.x86.vcvtneps2bf16128\0"
  "llvm.x86.vcvtneps2bf16256\0"
  "llvm.x86.vcvtps2ph.128\0"
  "llvm.x86.vcvtps2ph.256\0"
  "llvm.x86.vgf2p8affineinvqb.128\0"
  "llvm.x86.vgf2p8affineinvqb.256\0"
  "llvm.x86.vgf2p8affineinvqb.512\0"
  "llvm.x86.vgf2p8affineqb.128\0"
  "llvm.x86.vgf2p8affineqb.256\0"
  "llvm.x86.vgf2p8affineqb.512\0"
  "llvm.x86.vgf2p8mulb.128\0"
  "llvm.x86.vgf2p8mulb.256\0"
  "llvm.x86.vgf2p8mulb.512\0"
  "llvm.x86.vsha512msg1\0"
  "llvm.x86.vsha512msg2\0"
  "llvm.x86.vsha512rnds2\0"
  "llvm.x86.vsm3msg1\0"
  "llvm.x86.vsm3msg2\0"
  "llvm.x86.vsm3rnds2\0"
  "llvm.x86.vsm4key4128\0"
  "llvm.x86.vsm4key4256\0"
  "llvm.x86.vsm4key4512\0"
  "llvm.x86.vsm4rnds4128\0"
  "llvm.x86.vsm4rnds4256\0"
  "llvm.x86.vsm4rnds4512\0"
  "llvm.x86.wbinvd\0"
  "llvm.x86.wbnoinvd\0"
  "llvm.x86.wrfsbase.32\0"
  "llvm.x86.wrfsbase.64\0"
  "llvm.x86.wrgsbase.32\0"
  "llvm.x86.wrgsbase.64\0"
  "llvm.x86.wrpkru\0"
  "llvm.x86.wrssd\0"
  "llvm.x86.wrssq\0"
  "llvm.x86.wrussd\0"
  "llvm.x86.wrussq\0"
  "llvm.x86.xabort\0"
  "llvm.x86.xbegin\0"
  "llvm.x86.xend\0"
  "llvm.x86.xgetbv\0"
  "llvm.x86.xop.vfrcz.pd\0"
  "llvm.x86.xop.vfrcz.pd.256\0"
  "llvm.x86.xop.vfrcz.ps\0"
  "llvm.x86.xop.vfrcz.ps.256\0"
  "llvm.x86.xop.vfrcz.sd\0"
  "llvm.x86.xop.vfrcz.ss\0"
  "llvm.x86.xop.vpermil2pd\0"
  "llvm.x86.xop.vpermil2pd.256\0"
  "llvm.x86.xop.vpermil2ps\0"
  "llvm.x86.xop.vpermil2ps.256\0"
  "llvm.x86.xop.vphaddbd\0"
  "llvm.x86.xop.vphaddbq\0"
  "llvm.x86.xop.vphaddbw\0"
  "llvm.x86.xop.vphadddq\0"
  "llvm.x86.xop.vphaddubd\0"
  "llvm.x86.xop.vphaddubq\0"
  "llvm.x86.xop.vphaddubw\0"
  "llvm.x86.xop.vphaddudq\0"
  "llvm.x86.xop.vphadduwd\0"
  "llvm.x86.xop.vphadduwq\0"
  "llvm.x86.xop.vphaddwd\0"
  "llvm.x86.xop.vphaddwq\0"
  "llvm.x86.xop.vphsubbw\0"
  "llvm.x86.xop.vphsubdq\0"
  "llvm.x86.xop.vphsubwd\0"
  "llvm.x86.xop.vpmacsdd\0"
  "llvm.x86.xop.vpmacsdqh\0"
  "llvm.x86.xop.vpmacsdql\0"
  "llvm.x86.xop.vpmacssdd\0"
  "llvm.x86.xop.vpmacssdqh\0"
  "llvm.x86.xop.vpmacssdql\0"
  "llvm.x86.xop.vpmacsswd\0"
  "llvm.x86.xop.vpmacssww\0"
  "llvm.x86.xop.vpmacswd\0"
  "llvm.x86.xop.vpmacsww\0"
  "llvm.x86.xop.vpmadcsswd\0"
  "llvm.x86.xop.vpmadcswd\0"
  "llvm.x86.xop.vpperm\0"
  "llvm.x86.xop.vpshab\0"
  "llvm.x86.xop.vpshad\0"
  "llvm.x86.xop.vpshaq\0"
  "llvm.x86.xop.vpshaw\0"
  "llvm.x86.xop.vpshlb\0"
  "llvm.x86.xop.vpshld\0"
  "llvm.x86.xop.vpshlq\0"
  "llvm.x86.xop.vpshlw\0"
  "llvm.x86.xresldtrk\0"
  "llvm.x86.xrstor\0"
  "llvm.x86.xrstor64\0"
  "llvm.x86.xrstors\0"
  "llvm.x86.xrstors64\0"
  "llvm.x86.xsave\0"
  "llvm.x86.xsave64\0"
  "llvm.x86.xsavec\0"
  "llvm.x86.xsavec64\0"
  "llvm.x86.xsaveopt\0"
  "llvm.x86.xsaveopt64\0"
  "llvm.x86.xsaves\0"
  "llvm.x86.xsaves64\0"
  "llvm.x86.xsetbv\0"
  "llvm.x86.xsusldtrk\0"
  "llvm.x86.xtest\0"
  "llvm.xcore.bitrev\0"
  "llvm.xcore.checkevent\0"
  "llvm.xcore.chkct\0"
  "llvm.xcore.clre\0"
  "llvm.xcore.clrpt\0"
  "llvm.xcore.clrsr\0"
  "llvm.xcore.crc32\0"
  "llvm.xcore.crc8\0"
  "llvm.xcore.edu\0"
  "llvm.xcore.eeu\0"
  "llvm.xcore.endin\0"
  "llvm.xcore.freer\0"
  "llvm.xcore.geted\0"
  "llvm.xcore.getet\0"
  "llvm.xcore.getid\0"
  "llvm.xcore.getps\0"
  "llvm.xcore.getr\0"
  "llvm.xcore.getst\0"
  "llvm.xcore.getts\0"
  "llvm.xcore.in\0"
  "llvm.xcore.inct\0"
  "llvm.xcore.initcp\0"
  "llvm.xcore.initdp\0"
  "llvm.xcore.initlr\0"
  "llvm.xcore.initpc\0"
  "llvm.xcore.initsp\0"
  "llvm.xcore.inshr\0"
  "llvm.xcore.int\0"
  "llvm.xcore.mjoin\0"
  "llvm.xcore.msync\0"
  "llvm.xcore.out\0"
  "llvm.xcore.outct\0"
  "llvm.xcore.outshr\0"
  "llvm.xcore.outt\0"
  "llvm.xcore.peek\0"
  "llvm.xcore.setc\0"
  "llvm.xcore.setclk\0"
  "llvm.xcore.setd\0"
  "llvm.xcore.setev\0"
  "llvm.xcore.setps\0"
  "llvm.xcore.setpsc\0"
  "llvm.xcore.setpt\0"
  "llvm.xcore.setrdy\0"
  "llvm.xcore.setsr\0"
  "llvm.xcore.settw\0"
  "llvm.xcore.setv\0"
  "llvm.xcore.sext\0"
  "llvm.xcore.ssync\0"
  "llvm.xcore.syncr\0"
  "llvm.xcore.testct\0"
  "llvm.xcore.testwct\0"
  "llvm.xcore.waitevent\0"
  "llvm.xcore.zext\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
IntrinsicNameTable = IntrinsicNameTableStorage;

static constexpr unsigned IntrinsicNameOffsetTable[] = {
  1, // not_intrinsic
  15, // llvm.abs
  24, // llvm.acos
  34, // llvm.addressofreturnaddress
  62, // llvm.adjust.trampoline
  85, // llvm.alloc.token.id
  105, // llvm.allow.runtime.check
  130, // llvm.allow.sanitize.address
  158, // llvm.allow.sanitize.hwaddress
  188, // llvm.allow.sanitize.memory
  215, // llvm.allow.sanitize.thread
  242, // llvm.allow.ubsan.check
  265, // llvm.annotation
  281, // llvm.arithmetic.fence
  303, // llvm.asan.check.memaccess
  329, // llvm.asin
  339, // llvm.assume
  351, // llvm.atan
  361, // llvm.atan2
  372, // llvm.bitreverse
  388, // llvm.bswap
  399, // llvm.call.preallocated.arg
  426, // llvm.call.preallocated.setup
  455, // llvm.call.preallocated.teardown
  487, // llvm.callbr.landingpad
  510, // llvm.canonicalize
  528, // llvm.ceil
  538, // llvm.clear_cache
  555, // llvm.clmul
  566, // llvm.codeview.annotation
  591, // llvm.convert.from.fp16
  614, // llvm.convert.to.fp16
  635, // llvm.copysign
  649, // llvm.coro.align
  665, // llvm.coro.alloc
  681, // llvm.coro.alloca.alloc
  704, // llvm.coro.alloca.free
  726, // llvm.coro.alloca.get
  747, // llvm.coro.async.context.alloc
  777, // llvm.coro.async.context.dealloc
  809, // llvm.coro.async.resume
  832, // llvm.coro.async.size.replace
  861, // llvm.coro.await.suspend.bool
  890, // llvm.coro.await.suspend.handle
  921, // llvm.coro.await.suspend.void
  950, // llvm.coro.begin
  966, // llvm.coro.begin.custom.abi
  993, // llvm.coro.destroy
  1011, // llvm.coro.done
  1026, // llvm.coro.end
  1040, // llvm.coro.end.async
  1060, // llvm.coro.end.results
  1082, // llvm.coro.frame
  1098, // llvm.coro.free
  1113, // llvm.coro.id
  1126, // llvm.coro.id.async
  1145, // llvm.coro.id.retcon
  1165, // llvm.coro.id.retcon.once
  1190, // llvm.coro.is_in_ramp
  1211, // llvm.coro.noop
  1226, // llvm.coro.prepare.async
  1250, // llvm.coro.prepare.retcon
  1275, // llvm.coro.promise
  1293, // llvm.coro.resume
  1310, // llvm.coro.save
  1325, // llvm.coro.size
  1340, // llvm.coro.subfn.addr
  1361, // llvm.coro.suspend
  1379, // llvm.coro.suspend.async
  1403, // llvm.coro.suspend.retcon
  1428, // llvm.cos
  1437, // llvm.cosh
  1447, // llvm.ctlz
  1457, // llvm.ctpop
  1468, // llvm.cttz
  1478, // llvm.dbg.assign
  1494, // llvm.dbg.declare
  1511, // llvm.dbg.label
  1526, // llvm.dbg.value
  1541, // llvm.debugtrap
  1556, // llvm.donothing
  1571, // llvm.eh.dwarf.cfa
  1589, // llvm.eh.exceptioncode
  1611, // llvm.eh.exceptionpointer
  1636, // llvm.eh.recoverfp
  1654, // llvm.eh.return.i32
  1673, // llvm.eh.return.i64
  1692, // llvm.eh.sjlj.callsite
  1714, // llvm.eh.sjlj.functioncontext
  1743, // llvm.eh.sjlj.longjmp
  1764, // llvm.eh.sjlj.lsda
  1782, // llvm.eh.sjlj.setjmp
  1802, // llvm.eh.sjlj.setup.dispatch
  1830, // llvm.eh.typeid.for
  1849, // llvm.eh.unwind.init
  1869, // llvm.exp
  1878, // llvm.exp10
  1889, // llvm.exp2
  1899, // llvm.expect
  1911, // llvm.expect.with.probability
  1940, // llvm.experimental.constrained.acos
  1975, // llvm.experimental.constrained.asin
  2010, // llvm.experimental.constrained.atan
  2045, // llvm.experimental.constrained.atan2
  2081, // llvm.experimental.constrained.ceil
  2116, // llvm.experimental.constrained.cos
  2150, // llvm.experimental.constrained.cosh
  2185, // llvm.experimental.constrained.exp
  2219, // llvm.experimental.constrained.exp2
  2254, // llvm.experimental.constrained.fadd
  2289, // llvm.experimental.constrained.fcmp
  2324, // llvm.experimental.constrained.fcmps
  2360, // llvm.experimental.constrained.fdiv
  2395, // llvm.experimental.constrained.floor
  2431, // llvm.experimental.constrained.fma
  2465, // llvm.experimental.constrained.fmul
  2500, // llvm.experimental.constrained.fmuladd
  2538, // llvm.experimental.constrained.fpext
  2574, // llvm.experimental.constrained.fptosi
  2611, // llvm.experimental.constrained.fptoui
  2648, // llvm.experimental.constrained.fptrunc
  2686, // llvm.experimental.constrained.frem
  2721, // llvm.experimental.constrained.fsub
  2756, // llvm.experimental.constrained.ldexp
  2792, // llvm.experimental.constrained.llrint
  2829, // llvm.experimental.constrained.llround
  2867, // llvm.experimental.constrained.log
  2901, // llvm.experimental.constrained.log10
  2937, // llvm.experimental.constrained.log2
  2972, // llvm.experimental.constrained.lrint
  3008, // llvm.experimental.constrained.lround
  3045, // llvm.experimental.constrained.maximum
  3083, // llvm.experimental.constrained.maxnum
  3120, // llvm.experimental.constrained.minimum
  3158, // llvm.experimental.constrained.minnum
  3195, // llvm.experimental.constrained.nearbyint
  3235, // llvm.experimental.constrained.pow
  3269, // llvm.experimental.constrained.powi
  3304, // llvm.experimental.constrained.rint
  3339, // llvm.experimental.constrained.round
  3375, // llvm.experimental.constrained.roundeven
  3415, // llvm.experimental.constrained.sin
  3449, // llvm.experimental.constrained.sinh
  3484, // llvm.experimental.constrained.sitofp
  3521, // llvm.experimental.constrained.sqrt
  3556, // llvm.experimental.constrained.tan
  3590, // llvm.experimental.constrained.tanh
  3625, // llvm.experimental.constrained.trunc
  3661, // llvm.experimental.constrained.uitofp
  3698, // llvm.experimental.convergence.anchor
  3735, // llvm.experimental.convergence.entry
  3771, // llvm.experimental.convergence.loop
  3806, // llvm.experimental.cttz.elts
  3834, // llvm.experimental.deoptimize
  3863, // llvm.experimental.gc.get.pointer.base
  3901, // llvm.experimental.gc.get.pointer.offset
  3941, // llvm.experimental.gc.relocate
  3971, // llvm.experimental.gc.result
  3999, // llvm.experimental.gc.statepoint
  4031, // llvm.experimental.get.vector.length
  4067, // llvm.experimental.guard
  4091, // llvm.experimental.memset.pattern
  4124, // llvm.experimental.noalias.scope.decl
  4161, // llvm.experimental.patchpoint
  4190, // llvm.experimental.patchpoint.void
  4224, // llvm.experimental.stackmap
  4251, // llvm.experimental.vector.compress
  4285, // llvm.experimental.vector.extract.last.active
  4330, // llvm.experimental.vector.histogram.add
  4369, // llvm.experimental.vector.histogram.uadd.sat
  4413, // llvm.experimental.vector.histogram.umax
  4453, // llvm.experimental.vector.histogram.umin
  4493, // llvm.experimental.vector.match
  4524, // llvm.experimental.vp.reverse
  4553, // llvm.experimental.vp.splice
  4581, // llvm.experimental.vp.strided.load
  4615, // llvm.experimental.vp.strided.store
  4650, // llvm.experimental.widenable.condition
  4688, // llvm.fabs
  4698, // llvm.fake.use
  4712, // llvm.floor
  4723, // llvm.fma
  4732, // llvm.fmuladd
  4745, // llvm.fptosi.sat
  4761, // llvm.fptoui.sat
  4777, // llvm.fptrunc.round
  4796, // llvm.frameaddress
  4814, // llvm.frexp
  4825, // llvm.fshl
  4835, // llvm.fshr
  4845, // llvm.gcread
  4857, // llvm.gcroot
  4869, // llvm.gcwrite
  4882, // llvm.get.active.lane.mask
  4908, // llvm.get.dynamic.area.offset
  4937, // llvm.get.fpenv
  4952, // llvm.get.fpmode
  4968, // llvm.get.rounding
  4986, // llvm.hwasan.check.memaccess
  5014, // llvm.hwasan.check.memaccess.fixedshadow
  5054, // llvm.hwasan.check.memaccess.shortgranules
  5096, // llvm.hwasan.check.memaccess.shortgranules.fixedshadow
  5150, // llvm.icall.branch.funnel
  5175, // llvm.init.trampoline
  5196, // llvm.instrprof.callsite
  5220, // llvm.instrprof.cover
  5241, // llvm.instrprof.increment
  5266, // llvm.instrprof.increment.step
  5296, // llvm.instrprof.mcdc.parameters
  5327, // llvm.instrprof.mcdc.tvbitmap.update
  5363, // llvm.instrprof.timestamp
  5388, // llvm.instrprof.value.profile
  5417, // llvm.invariant.end
  5436, // llvm.invariant.start
  5457, // llvm.is.constant
  5474, // llvm.is.fpclass
  5490, // llvm.launder.invariant.group
  5519, // llvm.ldexp
  5530, // llvm.lifetime.end
  5548, // llvm.lifetime.start
  5568, // llvm.llrint
  5580, // llvm.llround
  5593, // llvm.load.relative
  5612, // llvm.localaddress
  5630, // llvm.localescape
  5647, // llvm.localrecover
  5665, // llvm.log
  5674, // llvm.log10
  5685, // llvm.log2
  5695, // llvm.loop.decrement
  5715, // llvm.loop.decrement.reg
  5739, // llvm.loop.dependence.raw.mask
  5769, // llvm.loop.dependence.war.mask
  5799, // llvm.lrint
  5810, // llvm.lround
  5822, // llvm.masked.compressstore
  5848, // llvm.masked.expandload
  5871, // llvm.masked.gather
  5890, // llvm.masked.load
  5907, // llvm.masked.scatter
  5927, // llvm.masked.store
  5945, // llvm.matrix.column.major.load
  5975, // llvm.matrix.column.major.store
  6006, // llvm.matrix.multiply
  6027, // llvm.matrix.transpose
  6049, // llvm.maximum
  6062, // llvm.maximumnum
  6078, // llvm.maxnum
  6090, // llvm.memcpy
  6102, // llvm.memcpy.element.unordered.atomic
  6139, // llvm.memcpy.inline
  6158, // llvm.memmove
  6171, // llvm.memmove.element.unordered.atomic
  6209, // llvm.memset
  6221, // llvm.memset.element.unordered.atomic
  6258, // llvm.memset.inline
  6277, // llvm.minimum
  6290, // llvm.minimumnum
  6306, // llvm.minnum
  6318, // llvm.modf
  6328, // llvm.nearbyint
  6343, // llvm.objc.arc.annotation.bottomup.bbend
  6383, // llvm.objc.arc.annotation.bottomup.bbstart
  6425, // llvm.objc.arc.annotation.topdown.bbend
  6464, // llvm.objc.arc.annotation.topdown.bbstart
  6505, // llvm.objc.autorelease
  6527, // llvm.objc.autoreleasePoolPop
  6556, // llvm.objc.autoreleasePoolPush
  6586, // llvm.objc.autoreleaseReturnValue
  6619, // llvm.objc.claimAutoreleasedReturnValue
  6658, // llvm.objc.clang.arc.noop.use
  6687, // llvm.objc.clang.arc.use
  6711, // llvm.objc.copyWeak
  6730, // llvm.objc.destroyWeak
  6752, // llvm.objc.initWeak
  6771, // llvm.objc.loadWeak
  6790, // llvm.objc.loadWeakRetained
  6817, // llvm.objc.moveWeak
  6836, // llvm.objc.release
  6854, // llvm.objc.retain
  6871, // llvm.objc.retain.autorelease
  6900, // llvm.objc.retainAutorelease
  6928, // llvm.objc.retainAutoreleaseReturnValue
  6967, // llvm.objc.retainAutoreleasedReturnValue
  7007, // llvm.objc.retainBlock
  7029, // llvm.objc.retainedObject
  7054, // llvm.objc.storeStrong
  7076, // llvm.objc.storeWeak
  7096, // llvm.objc.sync.enter
  7117, // llvm.objc.sync.exit
  7137, // llvm.objc.unretainedObject
  7164, // llvm.objc.unretainedPointer
  7192, // llvm.objc.unsafeClaimAutoreleasedReturnValue
  7237, // llvm.objectsize
  7253, // llvm.pcmarker
  7267, // llvm.pow
  7276, // llvm.powi
  7286, // llvm.prefetch
  7300, // llvm.preserve.array.access.index
  7333, // llvm.preserve.static.offset
  7361, // llvm.preserve.struct.access.index
  7395, // llvm.preserve.union.access.index
  7428, // llvm.protected.field.ptr
  7453, // llvm.pseudoprobe
  7470, // llvm.ptr.annotation
  7490, // llvm.ptrauth.auth
  7508, // llvm.ptrauth.blend
  7527, // llvm.ptrauth.resign
  7547, // llvm.ptrauth.sign
  7565, // llvm.ptrauth.sign.generic
  7591, // llvm.ptrauth.strip
  7610, // llvm.ptrmask
  7623, // llvm.public.type.test
  7645, // llvm.read_register
  7664, // llvm.read_volatile_register
  7692, // llvm.readcyclecounter
  7714, // llvm.readsteadycounter
  7737, // llvm.reloc.none
  7753, // llvm.reset.fpenv
  7770, // llvm.reset.fpmode
  7788, // llvm.returnaddress
  7807, // llvm.rint
  7817, // llvm.round
  7828, // llvm.roundeven
  7843, // llvm.sadd.sat
  7857, // llvm.sadd.with.overflow
  7881, // llvm.scmp
  7891, // llvm.sdiv.fix
  7905, // llvm.sdiv.fix.sat
  7923, // llvm.seh.scope.begin
  7944, // llvm.seh.scope.end
  7963, // llvm.seh.try.begin
  7982, // llvm.seh.try.end
  7999, // llvm.set.fpenv
  8014, // llvm.set.fpmode
  8030, // llvm.set.loop.iterations
  8055, // llvm.set.rounding
  8073, // llvm.sideeffect
  8089, // llvm.sin
  8098, // llvm.sincos
  8110, // llvm.sincospi
  8124, // llvm.sinh
  8134, // llvm.smax
  8144, // llvm.smin
  8154, // llvm.smul.fix
  8168, // llvm.smul.fix.sat
  8186, // llvm.smul.with.overflow
  8210, // llvm.sponentry
  8225, // llvm.sqrt
  8235, // llvm.ssa.copy
  8249, // llvm.sshl.sat
  8263, // llvm.ssub.sat
  8277, // llvm.ssub.with.overflow
  8301, // llvm.stackaddress
  8319, // llvm.stackguard
  8335, // llvm.stackprotector
  8355, // llvm.stackrestore
  8373, // llvm.stacksave
  8388, // llvm.start.loop.iterations
  8415, // llvm.stepvector
  8431, // llvm.strip.invariant.group
  8458, // llvm.swift.async.context.addr
  8488, // llvm.tan
  8497, // llvm.tanh
  8507, // llvm.test.set.loop.iterations
  8537, // llvm.test.start.loop.iterations
  8569, // llvm.thread.pointer
  8589, // llvm.threadlocal.address
  8614, // llvm.trap
  8624, // llvm.trunc
  8635, // llvm.type.checked.load
  8658, // llvm.type.checked.load.relative
  8690, // llvm.type.test
  8705, // llvm.uadd.sat
  8719, // llvm.uadd.with.overflow
  8743, // llvm.ubsantrap
  8758, // llvm.ucmp
  8768, // llvm.udiv.fix
  8782, // llvm.udiv.fix.sat
  8800, // llvm.umax
  8810, // llvm.umin
  8820, // llvm.umul.fix
  8834, // llvm.umul.fix.sat
  8852, // llvm.umul.with.overflow
  8876, // llvm.ushl.sat
  8890, // llvm.usub.sat
  8904, // llvm.usub.with.overflow
  8928, // llvm.va_copy
  8941, // llvm.va_end
  8953, // llvm.va_start
  8967, // llvm.var.annotation
  8987, // llvm.vector.deinterleave2
  9013, // llvm.vector.deinterleave3
  9039, // llvm.vector.deinterleave4
  9065, // llvm.vector.deinterleave5
  9091, // llvm.vector.deinterleave6
  9117, // llvm.vector.deinterleave7
  9143, // llvm.vector.deinterleave8
  9169, // llvm.vector.extract
  9189, // llvm.vector.insert
  9208, // llvm.vector.interleave2
  9232, // llvm.vector.interleave3
  9256, // llvm.vector.interleave4
  9280, // llvm.vector.interleave5
  9304, // llvm.vector.interleave6
  9328, // llvm.vector.interleave7
  9352, // llvm.vector.interleave8
  9376, // llvm.vector.partial.reduce.add
  9407, // llvm.vector.partial.reduce.fadd
  9439, // llvm.vector.reduce.add
  9462, // llvm.vector.reduce.and
  9485, // llvm.vector.reduce.fadd
  9509, // llvm.vector.reduce.fmax
  9533, // llvm.vector.reduce.fmaximum
  9561, // llvm.vector.reduce.fmin
  9585, // llvm.vector.reduce.fminimum
  9613, // llvm.vector.reduce.fmul
  9637, // llvm.vector.reduce.mul
  9660, // llvm.vector.reduce.or
  9682, // llvm.vector.reduce.smax
  9706, // llvm.vector.reduce.smin
  9730, // llvm.vector.reduce.umax
  9754, // llvm.vector.reduce.umin
  9778, // llvm.vector.reduce.xor
  9801, // llvm.vector.reverse
  9821, // llvm.vector.splice.left
  9845, // llvm.vector.splice.right
  9870, // llvm.vp.abs
  9882, // llvm.vp.add
  9894, // llvm.vp.and
  9906, // llvm.vp.ashr
  9919, // llvm.vp.bitreverse
  9938, // llvm.vp.bswap
  9952, // llvm.vp.ceil
  9965, // llvm.vp.copysign
  9982, // llvm.vp.ctlz
  9995, // llvm.vp.ctpop
  10009, // llvm.vp.cttz
  10022, // llvm.vp.cttz.elts
  10040, // llvm.vp.fabs
  10053, // llvm.vp.fadd
  10066, // llvm.vp.fcmp
  10079, // llvm.vp.fdiv
  10092, // llvm.vp.floor
  10106, // llvm.vp.fma
  10118, // llvm.vp.fmul
  10131, // llvm.vp.fmuladd
  10147, // llvm.vp.fneg
  10160, // llvm.vp.fpext
  10174, // llvm.vp.fptosi
  10189, // llvm.vp.fptoui
  10204, // llvm.vp.fptrunc
  10220, // llvm.vp.frem
  10233, // llvm.vp.fshl
  10246, // llvm.vp.fshr
  10259, // llvm.vp.fsub
  10272, // llvm.vp.gather
  10287, // llvm.vp.icmp
  10300, // llvm.vp.inttoptr
  10317, // llvm.vp.is.fpclass
  10336, // llvm.vp.llrint
  10351, // llvm.vp.load
  10364, // llvm.vp.load.ff
  10380, // llvm.vp.lrint
  10394, // llvm.vp.lshr
  10407, // llvm.vp.maximum
  10423, // llvm.vp.maxnum
  10438, // llvm.vp.merge
  10452, // llvm.vp.minimum
  10468, // llvm.vp.minnum
  10483, // llvm.vp.mul
  10495, // llvm.vp.nearbyint
  10513, // llvm.vp.or
  10524, // llvm.vp.ptrtoint
  10541, // llvm.vp.reduce.add
  10560, // llvm.vp.reduce.and
  10579, // llvm.vp.reduce.fadd
  10599, // llvm.vp.reduce.fmax
  10619, // llvm.vp.reduce.fmaximum
  10643, // llvm.vp.reduce.fmin
  10663, // llvm.vp.reduce.fminimum
  10687, // llvm.vp.reduce.fmul
  10707, // llvm.vp.reduce.mul
  10726, // llvm.vp.reduce.or
  10744, // llvm.vp.reduce.smax
  10764, // llvm.vp.reduce.smin
  10784, // llvm.vp.reduce.umax
  10804, // llvm.vp.reduce.umin
  10824, // llvm.vp.reduce.xor
  10843, // llvm.vp.rint
  10856, // llvm.vp.round
  10870, // llvm.vp.roundeven
  10888, // llvm.vp.roundtozero
  10908, // llvm.vp.sadd.sat
  10925, // llvm.vp.scatter
  10941, // llvm.vp.sdiv
  10954, // llvm.vp.select
  10969, // llvm.vp.sext
  10982, // llvm.vp.shl
  10994, // llvm.vp.sitofp
  11009, // llvm.vp.smax
  11022, // llvm.vp.smin
  11035, // llvm.vp.sqrt
  11048, // llvm.vp.srem
  11061, // llvm.vp.ssub.sat
  11078, // llvm.vp.store
  11092, // llvm.vp.sub
  11104, // llvm.vp.trunc
  11118, // llvm.vp.uadd.sat
  11135, // llvm.vp.udiv
  11148, // llvm.vp.uitofp
  11163, // llvm.vp.umax
  11176, // llvm.vp.umin
  11189, // llvm.vp.urem
  11202, // llvm.vp.usub.sat
  11219, // llvm.vp.xor
  11231, // llvm.vp.zext
  11244, // llvm.vscale
  11256, // llvm.write_register
  11276, // llvm.xray.customevent
  11298, // llvm.xray.typedevent
  11319, // llvm.aarch64.addg
  11337, // llvm.aarch64.break
  11356, // llvm.aarch64.chkfeat
  11377, // llvm.aarch64.clrex
  11396, // llvm.aarch64.cls
  11413, // llvm.aarch64.cls64
  11432, // llvm.aarch64.crc32b
  11452, // llvm.aarch64.crc32cb
  11473, // llvm.aarch64.crc32ch
  11494, // llvm.aarch64.crc32cw
  11515, // llvm.aarch64.crc32cx
  11536, // llvm.aarch64.crc32h
  11556, // llvm.aarch64.crc32w
  11576, // llvm.aarch64.crc32x
  11596, // llvm.aarch64.crypto.aesd
  11621, // llvm.aarch64.crypto.aese
  11646, // llvm.aarch64.crypto.aesimc
  11673, // llvm.aarch64.crypto.aesmc
  11699, // llvm.aarch64.crypto.bcaxs
  11725, // llvm.aarch64.crypto.bcaxu
  11751, // llvm.aarch64.crypto.eor3s
  11777, // llvm.aarch64.crypto.eor3u
  11803, // llvm.aarch64.crypto.rax1
  11828, // llvm.aarch64.crypto.sha1c
  11854, // llvm.aarch64.crypto.sha1h
  11880, // llvm.aarch64.crypto.sha1m
  11906, // llvm.aarch64.crypto.sha1p
  11932, // llvm.aarch64.crypto.sha1su0
  11960, // llvm.aarch64.crypto.sha1su1
  11988, // llvm.aarch64.crypto.sha256h
  12016, // llvm.aarch64.crypto.sha256h2
  12045, // llvm.aarch64.crypto.sha256su0
  12075, // llvm.aarch64.crypto.sha256su1
  12105, // llvm.aarch64.crypto.sha512h
  12133, // llvm.aarch64.crypto.sha512h2
  12162, // llvm.aarch64.crypto.sha512su0
  12192, // llvm.aarch64.crypto.sha512su1
  12222, // llvm.aarch64.crypto.sm3partw1
  12252, // llvm.aarch64.crypto.sm3partw2
  12282, // llvm.aarch64.crypto.sm3ss1
  12309, // llvm.aarch64.crypto.sm3tt1a
  12337, // llvm.aarch64.crypto.sm3tt1b
  12365, // llvm.aarch64.crypto.sm3tt2a
  12393, // llvm.aarch64.crypto.sm3tt2b
  12421, // llvm.aarch64.crypto.sm4e
  12446, // llvm.aarch64.crypto.sm4ekey
  12474, // llvm.aarch64.crypto.xar
  12498, // llvm.aarch64.dmb
  12515, // llvm.aarch64.dsb
  12532, // llvm.aarch64.fjcvtzs
  12553, // llvm.aarch64.frint32x
  12575, // llvm.aarch64.frint32z
  12597, // llvm.aarch64.frint64x
  12619, // llvm.aarch64.frint64z
  12641, // llvm.aarch64.gcspopm
  12662, // llvm.aarch64.gcsss
  12681, // llvm.aarch64.get.fpcr
  12703, // llvm.aarch64.get.fpsr
  12725, // llvm.aarch64.gmi
  12742, // llvm.aarch64.hint
  12760, // llvm.aarch64.hlt
  12777, // llvm.aarch64.irg
  12794, // llvm.aarch64.irg.sp
  12814, // llvm.aarch64.isb
  12831, // llvm.aarch64.ld64b
  12850, // llvm.aarch64.ldaxp
  12869, // llvm.aarch64.ldaxr
  12888, // llvm.aarch64.ldg
  12905, // llvm.aarch64.ldxp
  12923, // llvm.aarch64.ldxr
  12941, // llvm.aarch64.mops.memset.tag
  12970, // llvm.aarch64.neon.abs
  12992, // llvm.aarch64.neon.addhn
  13016, // llvm.aarch64.neon.addp
  13039, // llvm.aarch64.neon.bfdot
  13063, // llvm.aarch64.neon.bfmlalb
  13089, // llvm.aarch64.neon.bfmlalt
  13115, // llvm.aarch64.neon.bfmmla
  13140, // llvm.aarch64.neon.cls
  13162, // llvm.aarch64.neon.fabd
  13185, // llvm.aarch64.neon.facge
  13209, // llvm.aarch64.neon.facgt
  13233, // llvm.aarch64.neon.faddp
  13257, // llvm.aarch64.neon.faddv
  13281, // llvm.aarch64.neon.famax
  13305, // llvm.aarch64.neon.famin
  13329, // llvm.aarch64.neon.fcvtas
  13354, // llvm.aarch64.neon.fcvtau
  13379, // llvm.aarch64.neon.fcvtms
  13404, // llvm.aarch64.neon.fcvtmu
  13429, // llvm.aarch64.neon.fcvtns
  13454, // llvm.aarch64.neon.fcvtnu
  13479, // llvm.aarch64.neon.fcvtps
  13504, // llvm.aarch64.neon.fcvtpu
  13529, // llvm.aarch64.neon.fcvtxn
  13554, // llvm.aarch64.neon.fcvtzs
  13579, // llvm.aarch64.neon.fcvtzu
  13604, // llvm.aarch64.neon.fmax
  13627, // llvm.aarch64.neon.fmaxnm
  13652, // llvm.aarch64.neon.fmaxnmp
  13678, // llvm.aarch64.neon.fmaxnmv
  13704, // llvm.aarch64.neon.fmaxp
  13728, // llvm.aarch64.neon.fmaxv
  13752, // llvm.aarch64.neon.fmin
  13775, // llvm.aarch64.neon.fminnm
  13800, // llvm.aarch64.neon.fminnmp
  13826, // llvm.aarch64.neon.fminnmv
  13852, // llvm.aarch64.neon.fminp
  13876, // llvm.aarch64.neon.fminv
  13900, // llvm.aarch64.neon.fmlal
  13924, // llvm.aarch64.neon.fmlal2
  13949, // llvm.aarch64.neon.fmlsl
  13973, // llvm.aarch64.neon.fmlsl2
  13998, // llvm.aarch64.neon.fmmla
  14022, // llvm.aarch64.neon.fmulx
  14046, // llvm.aarch64.neon.fp8.cvtl1
  14074, // llvm.aarch64.neon.fp8.cvtl2
  14102, // llvm.aarch64.neon.fp8.fcvtn
  14130, // llvm.aarch64.neon.fp8.fcvtn2
  14159, // llvm.aarch64.neon.fp8.fdot2
  14187, // llvm.aarch64.neon.fp8.fdot2.lane
  14220, // llvm.aarch64.neon.fp8.fdot4
  14248, // llvm.aarch64.neon.fp8.fdot4.lane
  14281, // llvm.aarch64.neon.fp8.fmlalb
  14310, // llvm.aarch64.neon.fp8.fmlalb.lane
  14344, // llvm.aarch64.neon.fp8.fmlallbb
  14375, // llvm.aarch64.neon.fp8.fmlallbb.lane
  14411, // llvm.aarch64.neon.fp8.fmlallbt
  14442, // llvm.aarch64.neon.fp8.fmlallbt.lane
  14478, // llvm.aarch64.neon.fp8.fmlalltb
  14509, // llvm.aarch64.neon.fp8.fmlalltb.lane
  14545, // llvm.aarch64.neon.fp8.fmlalltt
  14576, // llvm.aarch64.neon.fp8.fmlalltt.lane
  14612, // llvm.aarch64.neon.fp8.fmlalt
  14641, // llvm.aarch64.neon.fp8.fmlalt.lane
  14675, // llvm.aarch64.neon.fp8.fscale
  14704, // llvm.aarch64.neon.frecpe
  14729, // llvm.aarch64.neon.frecps
  14754, // llvm.aarch64.neon.frecpx
  14779, // llvm.aarch64.neon.frint32x
  14806, // llvm.aarch64.neon.frint32z
  14833, // llvm.aarch64.neon.frint64x
  14860, // llvm.aarch64.neon.frint64z
  14887, // llvm.aarch64.neon.frsqrte
  14913, // llvm.aarch64.neon.frsqrts
  14939, // llvm.aarch64.neon.ld1x2
  14963, // llvm.aarch64.neon.ld1x3
  14987, // llvm.aarch64.neon.ld1x4
  15011, // llvm.aarch64.neon.ld2
  15033, // llvm.aarch64.neon.ld2lane
  15059, // llvm.aarch64.neon.ld2r
  15082, // llvm.aarch64.neon.ld3
  15104, // llvm.aarch64.neon.ld3lane
  15130, // llvm.aarch64.neon.ld3r
  15153, // llvm.aarch64.neon.ld4
  15175, // llvm.aarch64.neon.ld4lane
  15201, // llvm.aarch64.neon.ld4r
  15224, // llvm.aarch64.neon.pmul
  15247, // llvm.aarch64.neon.pmull
  15271, // llvm.aarch64.neon.pmull64
  15297, // llvm.aarch64.neon.raddhn
  15322, // llvm.aarch64.neon.rshrn
  15346, // llvm.aarch64.neon.rsubhn
  15371, // llvm.aarch64.neon.sabd
  15394, // llvm.aarch64.neon.saddlp
  15419, // llvm.aarch64.neon.saddlv
  15444, // llvm.aarch64.neon.saddv
  15468, // llvm.aarch64.neon.scalar.sqxtn
  15499, // llvm.aarch64.neon.scalar.sqxtun
  15531, // llvm.aarch64.neon.scalar.uqxtn
  15562, // llvm.aarch64.neon.sdot
  15585, // llvm.aarch64.neon.shadd
  15609, // llvm.aarch64.neon.shll
  15632, // llvm.aarch64.neon.shsub
  15656, // llvm.aarch64.neon.smax
  15679, // llvm.aarch64.neon.smaxp
  15703, // llvm.aarch64.neon.smaxv
  15727, // llvm.aarch64.neon.smin
  15750, // llvm.aarch64.neon.sminp
  15774, // llvm.aarch64.neon.sminv
  15798, // llvm.aarch64.neon.smmla
  15822, // llvm.aarch64.neon.smull
  15846, // llvm.aarch64.neon.sqabs
  15870, // llvm.aarch64.neon.sqadd
  15894, // llvm.aarch64.neon.sqdmulh
  15920, // llvm.aarch64.neon.sqdmulh.lane
  15951, // llvm.aarch64.neon.sqdmulh.laneq
  15983, // llvm.aarch64.neon.sqdmull
  16009, // llvm.aarch64.neon.sqdmulls.scalar
  16043, // llvm.aarch64.neon.sqneg
  16067, // llvm.aarch64.neon.sqrdmlah
  16094, // llvm.aarch64.neon.sqrdmlsh
  16121, // llvm.aarch64.neon.sqrdmulh
  16148, // llvm.aarch64.neon.sqrdmulh.lane
  16180, // llvm.aarch64.neon.sqrdmulh.laneq
  16213, // llvm.aarch64.neon.sqrshl
  16238, // llvm.aarch64.neon.sqrshrn
  16264, // llvm.aarch64.neon.sqrshrun
  16291, // llvm.aarch64.neon.sqshl
  16315, // llvm.aarch64.neon.sqshlu
  16340, // llvm.aarch64.neon.sqshrn
  16365, // llvm.aarch64.neon.sqshrun
  16391, // llvm.aarch64.neon.sqsub
  16415, // llvm.aarch64.neon.sqxtn
  16439, // llvm.aarch64.neon.sqxtun
  16464, // llvm.aarch64.neon.srhadd
  16489, // llvm.aarch64.neon.srshl
  16513, // llvm.aarch64.neon.sshl
  16536, // llvm.aarch64.neon.sshll
  16560, // llvm.aarch64.neon.st1x2
  16584, // llvm.aarch64.neon.st1x3
  16608, // llvm.aarch64.neon.st1x4
  16632, // llvm.aarch64.neon.st2
  16654, // llvm.aarch64.neon.st2lane
  16680, // llvm.aarch64.neon.st3
  16702, // llvm.aarch64.neon.st3lane
  16728, // llvm.aarch64.neon.st4
  16750, // llvm.aarch64.neon.st4lane
  16776, // llvm.aarch64.neon.subhn
  16800, // llvm.aarch64.neon.suqadd
  16825, // llvm.aarch64.neon.tbl1
  16848, // llvm.aarch64.neon.tbl2
  16871, // llvm.aarch64.neon.tbl3
  16894, // llvm.aarch64.neon.tbl4
  16917, // llvm.aarch64.neon.tbx1
  16940, // llvm.aarch64.neon.tbx2
  16963, // llvm.aarch64.neon.tbx3
  16986, // llvm.aarch64.neon.tbx4
  17009, // llvm.aarch64.neon.uabd
  17032, // llvm.aarch64.neon.uaddlp
  17057, // llvm.aarch64.neon.uaddlv
  17082, // llvm.aarch64.neon.uaddv
  17106, // llvm.aarch64.neon.udot
  17129, // llvm.aarch64.neon.uhadd
  17153, // llvm.aarch64.neon.uhsub
  17177, // llvm.aarch64.neon.umax
  17200, // llvm.aarch64.neon.umaxp
  17224, // llvm.aarch64.neon.umaxv
  17248, // llvm.aarch64.neon.umin
  17271, // llvm.aarch64.neon.uminp
  17295, // llvm.aarch64.neon.uminv
  17319, // llvm.aarch64.neon.ummla
  17343, // llvm.aarch64.neon.umull
  17367, // llvm.aarch64.neon.uqadd
  17391, // llvm.aarch64.neon.uqrshl
  17416, // llvm.aarch64.neon.uqrshrn
  17442, // llvm.aarch64.neon.uqshl
  17466, // llvm.aarch64.neon.uqshrn
  17491, // llvm.aarch64.neon.uqsub
  17515, // llvm.aarch64.neon.uqxtn
  17539, // llvm.aarch64.neon.urecpe
  17564, // llvm.aarch64.neon.urhadd
  17589, // llvm.aarch64.neon.urshl
  17613, // llvm.aarch64.neon.ursqrte
  17639, // llvm.aarch64.neon.usdot
  17663, // llvm.aarch64.neon.ushl
  17686, // llvm.aarch64.neon.ushll
  17710, // llvm.aarch64.neon.usmmla
  17735, // llvm.aarch64.neon.usqadd
  17760, // llvm.aarch64.neon.vcadd.rot270
  17791, // llvm.aarch64.neon.vcadd.rot90
  17821, // llvm.aarch64.neon.vcmla.rot0
  17850, // llvm.aarch64.neon.vcmla.rot180
  17881, // llvm.aarch64.neon.vcmla.rot270
  17912, // llvm.aarch64.neon.vcmla.rot90
  17942, // llvm.aarch64.neon.vcopy.lane
  17971, // llvm.aarch64.neon.vcvtfp2fxs
  18000, // llvm.aarch64.neon.vcvtfp2fxu
  18029, // llvm.aarch64.neon.vcvtfp2hf
  18057, // llvm.aarch64.neon.vcvtfxs2fp
  18086, // llvm.aarch64.neon.vcvtfxu2fp
  18115, // llvm.aarch64.neon.vcvthf2fp
  18143, // llvm.aarch64.neon.vluti2.lane
  18173, // llvm.aarch64.neon.vluti2.laneq
  18204, // llvm.aarch64.neon.vluti4q.lane
  18235, // llvm.aarch64.neon.vluti4q.lane.x2
  18269, // llvm.aarch64.neon.vluti4q.laneq
  18301, // llvm.aarch64.neon.vluti4q.laneq.x2
  18336, // llvm.aarch64.neon.vsli
  18359, // llvm.aarch64.neon.vsri
  18382, // llvm.aarch64.prefetch
  18404, // llvm.aarch64.range.prefetch
  18432, // llvm.aarch64.rndr
  18450, // llvm.aarch64.rndrrs
  18470, // llvm.aarch64.sdiv
  18488, // llvm.aarch64.set.fpcr
  18510, // llvm.aarch64.set.fpmr
  18532, // llvm.aarch64.set.fpsr
  18554, // llvm.aarch64.settag
  18574, // llvm.aarch64.settag.zero
  18599, // llvm.aarch64.sisd.fabd
  18622, // llvm.aarch64.sisd.fcvtxn
  18647, // llvm.aarch64.sme.add.write.single.za.vg1x2
  18690, // llvm.aarch64.sme.add.write.single.za.vg1x4
  18733, // llvm.aarch64.sme.add.write.za.vg1x2
  18769, // llvm.aarch64.sme.add.write.za.vg1x4
  18805, // llvm.aarch64.sme.add.za16.vg1x2
  18837, // llvm.aarch64.sme.add.za16.vg1x4
  18869, // llvm.aarch64.sme.add.za32.vg1x2
  18901, // llvm.aarch64.sme.add.za32.vg1x4
  18933, // llvm.aarch64.sme.add.za64.vg1x2
  18965, // llvm.aarch64.sme.add.za64.vg1x4
  18997, // llvm.aarch64.sme.addha
  19020, // llvm.aarch64.sme.addva
  19043, // llvm.aarch64.sme.bmopa.za32
  19071, // llvm.aarch64.sme.bmops.za32
  19099, // llvm.aarch64.sme.cntsd
  19122, // llvm.aarch64.sme.famax.x2
  19148, // llvm.aarch64.sme.famax.x4
  19174, // llvm.aarch64.sme.famin.x2
  19200, // llvm.aarch64.sme.famin.x4
  19226, // llvm.aarch64.sme.fdot.lane.za32.vg1x2
  19264, // llvm.aarch64.sme.fdot.lane.za32.vg1x4
  19302, // llvm.aarch64.sme.fdot.single.za32.vg1x2
  19342, // llvm.aarch64.sme.fdot.single.za32.vg1x4
  19382, // llvm.aarch64.sme.fdot.za32.vg1x2
  19415, // llvm.aarch64.sme.fdot.za32.vg1x4
  19448, // llvm.aarch64.sme.fmla.lane.vg1x2
  19481, // llvm.aarch64.sme.fmla.lane.vg1x4
  19514, // llvm.aarch64.sme.fmla.single.vg1x2
  19549, // llvm.aarch64.sme.fmla.single.vg1x4
  19584, // llvm.aarch64.sme.fmla.vg1x2
  19612, // llvm.aarch64.sme.fmla.vg1x4
  19640, // llvm.aarch64.sme.fmlal.lane.vg2x1
  19674, // llvm.aarch64.sme.fmlal.lane.vg2x2
  19708, // llvm.aarch64.sme.fmlal.lane.vg2x4
  19742, // llvm.aarch64.sme.fmlal.single.vg2x1
  19778, // llvm.aarch64.sme.fmlal.single.vg2x2
  19814, // llvm.aarch64.sme.fmlal.single.vg2x4
  19850, // llvm.aarch64.sme.fmlal.vg2x2
  19879, // llvm.aarch64.sme.fmlal.vg2x4
  19908, // llvm.aarch64.sme.fmls.lane.vg1x2
  19941, // llvm.aarch64.sme.fmls.lane.vg1x4
  19974, // llvm.aarch64.sme.fmls.single.vg1x2
  20009, // llvm.aarch64.sme.fmls.single.vg1x4
  20044, // llvm.aarch64.sme.fmls.vg1x2
  20072, // llvm.aarch64.sme.fmls.vg1x4
  20100, // llvm.aarch64.sme.fmlsl.lane.vg2x1
  20134, // llvm.aarch64.sme.fmlsl.lane.vg2x2
  20168, // llvm.aarch64.sme.fmlsl.lane.vg2x4
  20202, // llvm.aarch64.sme.fmlsl.single.vg2x1
  20238, // llvm.aarch64.sme.fmlsl.single.vg2x2
  20274, // llvm.aarch64.sme.fmlsl.single.vg2x4
  20310, // llvm.aarch64.sme.fmlsl.vg2x2
  20339, // llvm.aarch64.sme.fmlsl.vg2x4
  20368, // llvm.aarch64.sme.fp8.fdot.lane.za16.vg1x2
  20410, // llvm.aarch64.sme.fp8.fdot.lane.za16.vg1x4
  20452, // llvm.aarch64.sme.fp8.fdot.lane.za32.vg1x2
  20494, // llvm.aarch64.sme.fp8.fdot.lane.za32.vg1x4
  20536, // llvm.aarch64.sme.fp8.fdot.multi.za16.vg1x2
  20579, // llvm.aarch64.sme.fp8.fdot.multi.za16.vg1x4
  20622, // llvm.aarch64.sme.fp8.fdot.multi.za32.vg1x2
  20665, // llvm.aarch64.sme.fp8.fdot.multi.za32.vg1x4
  20708, // llvm.aarch64.sme.fp8.fdot.single.za16.vg1x2
  20752, // llvm.aarch64.sme.fp8.fdot.single.za16.vg1x4
  20796, // llvm.aarch64.sme.fp8.fdot.single.za32.vg1x2
  20840, // llvm.aarch64.sme.fp8.fdot.single.za32.vg1x4
  20884, // llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x1
  20927, // llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x2
  20970, // llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x4
  21013, // llvm.aarch64.sme.fp8.fmlal.multi.za16.vg2x2
  21057, // llvm.aarch64.sme.fp8.fmlal.multi.za16.vg2x4
  21101, // llvm.aarch64.sme.fp8.fmlal.single.za16.vg2x2
  21146, // llvm.aarch64.sme.fp8.fmlal.single.za16.vg2x4
  21191, // llvm.aarch64.sme.fp8.fmlal.za16.vg2x1
  21229, // llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x1
  21273, // llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x2
  21317, // llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x4
  21361, // llvm.aarch64.sme.fp8.fmlall.multi.za32.vg4x2
  21406, // llvm.aarch64.sme.fp8.fmlall.multi.za32.vg4x4
  21451, // llvm.aarch64.sme.fp8.fmlall.single.za32.vg4x2
  21497, // llvm.aarch64.sme.fp8.fmlall.single.za32.vg4x4
  21543, // llvm.aarch64.sme.fp8.fmlall.za32.vg4x1
  21582, // llvm.aarch64.sme.fp8.fmop4a.za16.1x1
  21619, // llvm.aarch64.sme.fp8.fmop4a.za16.1x2
  21656, // llvm.aarch64.sme.fp8.fmop4a.za16.2x1
  21693, // llvm.aarch64.sme.fp8.fmop4a.za16.2x2
  21730, // llvm.aarch64.sme.fp8.fmop4a.za32.1x1
  21767, // llvm.aarch64.sme.fp8.fmop4a.za32.1x2
  21804, // llvm.aarch64.sme.fp8.fmop4a.za32.2x1
  21841, // llvm.aarch64.sme.fp8.fmop4a.za32.2x2
  21878, // llvm.aarch64.sme.fp8.fmopa.za16
  21910, // llvm.aarch64.sme.fp8.fmopa.za32
  21942, // llvm.aarch64.sme.fp8.fvdot.lane.za16.vg1x2
  21985, // llvm.aarch64.sme.fp8.fvdotb.lane.za32.vg1x4
  22029, // llvm.aarch64.sme.fp8.fvdott.lane.za32.vg1x4
  22073, // llvm.aarch64.sme.fp8.scale.single.x2
  22110, // llvm.aarch64.sme.fp8.scale.single.x4
  22147, // llvm.aarch64.sme.fp8.scale.x2
  22177, // llvm.aarch64.sme.fp8.scale.x4
  22207, // llvm.aarch64.sme.ftmopa.za16
  22236, // llvm.aarch64.sme.ftmopa.za32
  22265, // llvm.aarch64.sme.fvdot.lane.za32.vg1x2
  22304, // llvm.aarch64.sme.get.tpidr2
  22332, // llvm.aarch64.sme.in.streaming.mode
  22367, // llvm.aarch64.sme.ld1b.horiz
  22395, // llvm.aarch64.sme.ld1b.vert
  22422, // llvm.aarch64.sme.ld1d.horiz
  22450, // llvm.aarch64.sme.ld1d.vert
  22477, // llvm.aarch64.sme.ld1h.horiz
  22505, // llvm.aarch64.sme.ld1h.vert
  22532, // llvm.aarch64.sme.ld1q.horiz
  22560, // llvm.aarch64.sme.ld1q.vert
  22587, // llvm.aarch64.sme.ld1w.horiz
  22615, // llvm.aarch64.sme.ld1w.vert
  22642, // llvm.aarch64.sme.ldr
  22663, // llvm.aarch64.sme.ldr.zt
  22687, // llvm.aarch64.sme.luti2.lane.zt
  22718, // llvm.aarch64.sme.luti2.lane.zt.x2
  22752, // llvm.aarch64.sme.luti2.lane.zt.x4
  22786, // llvm.aarch64.sme.luti4.lane.zt
  22817, // llvm.aarch64.sme.luti4.lane.zt.x2
  22851, // llvm.aarch64.sme.luti4.lane.zt.x4
  22885, // llvm.aarch64.sme.luti4.zt.x4
  22914, // llvm.aarch64.sme.mop4a.1x1
  22941, // llvm.aarch64.sme.mop4a.1x2
  22968, // llvm.aarch64.sme.mop4a.2x1
  22995, // llvm.aarch64.sme.mop4a.2x2
  23022, // llvm.aarch64.sme.mop4a.wide.1x1
  23054, // llvm.aarch64.sme.mop4a.wide.1x2
  23086, // llvm.aarch64.sme.mop4a.wide.2x1
  23118, // llvm.aarch64.sme.mop4a.wide.2x2
  23150, // llvm.aarch64.sme.mop4s.1x1
  23177, // llvm.aarch64.sme.mop4s.1x2
  23204, // llvm.aarch64.sme.mop4s.2x1
  23231, // llvm.aarch64.sme.mop4s.2x2
  23258, // llvm.aarch64.sme.mop4s.wide.1x1
  23290, // llvm.aarch64.sme.mop4s.wide.1x2
  23322, // llvm.aarch64.sme.mop4s.wide.2x1
  23354, // llvm.aarch64.sme.mop4s.wide.2x2
  23386, // llvm.aarch64.sme.mopa
  23408, // llvm.aarch64.sme.mopa.nonwide
  23438, // llvm.aarch64.sme.mopa.wide
  23465, // llvm.aarch64.sme.mops
  23487, // llvm.aarch64.sme.mops.nonwide
  23517, // llvm.aarch64.sme.mops.wide
  23544, // llvm.aarch64.sme.read.hor.vg2
  23574, // llvm.aarch64.sme.read.hor.vg4
  23604, // llvm.aarch64.sme.read.horiz
  23632, // llvm.aarch64.sme.read.ver.vg2
  23662, // llvm.aarch64.sme.read.ver.vg4
  23692, // llvm.aarch64.sme.read.vert
  23719, // llvm.aarch64.sme.read.vg1x2
  23747, // llvm.aarch64.sme.read.vg1x4
  23775, // llvm.aarch64.sme.readq.horiz
  23804, // llvm.aarch64.sme.readq.vert
  23832, // llvm.aarch64.sme.readz.horiz
  23861, // llvm.aarch64.sme.readz.horiz.x2
  23893, // llvm.aarch64.sme.readz.horiz.x4
  23925, // llvm.aarch64.sme.readz.q.horiz
  23956, // llvm.aarch64.sme.readz.q.vert
  23986, // llvm.aarch64.sme.readz.vert
  24014, // llvm.aarch64.sme.readz.vert.x2
  24045, // llvm.aarch64.sme.readz.vert.x4
  24076, // llvm.aarch64.sme.readz.x2
  24102, // llvm.aarch64.sme.readz.x4
  24128, // llvm.aarch64.sme.sdot.lane.za32.vg1x2
  24166, // llvm.aarch64.sme.sdot.lane.za32.vg1x4
  24204, // llvm.aarch64.sme.sdot.lane.za64.vg1x2
  24242, // llvm.aarch64.sme.sdot.lane.za64.vg1x4
  24280, // llvm.aarch64.sme.sdot.single.za32.vg1x2
  24320, // llvm.aarch64.sme.sdot.single.za32.vg1x4
  24360, // llvm.aarch64.sme.sdot.single.za64.vg1x2
  24400, // llvm.aarch64.sme.sdot.single.za64.vg1x4
  24440, // llvm.aarch64.sme.sdot.za32.vg1x2
  24473, // llvm.aarch64.sme.sdot.za32.vg1x4
  24506, // llvm.aarch64.sme.sdot.za64.vg1x2
  24539, // llvm.aarch64.sme.sdot.za64.vg1x4
  24572, // llvm.aarch64.sme.set.tpidr2
  24600, // llvm.aarch64.sme.smla.za32.lane.vg4x1
  24638, // llvm.aarch64.sme.smla.za32.lane.vg4x2
  24676, // llvm.aarch64.sme.smla.za32.lane.vg4x4
  24714, // llvm.aarch64.sme.smla.za32.single.vg4x1
  24754, // llvm.aarch64.sme.smla.za32.single.vg4x2
  24794, // llvm.aarch64.sme.smla.za32.single.vg4x4
  24834, // llvm.aarch64.sme.smla.za32.vg4x2
  24867, // llvm.aarch64.sme.smla.za32.vg4x4
  24900, // llvm.aarch64.sme.smla.za64.lane.vg4x1
  24938, // llvm.aarch64.sme.smla.za64.lane.vg4x2
  24976, // llvm.aarch64.sme.smla.za64.lane.vg4x4
  25014, // llvm.aarch64.sme.smla.za64.single.vg4x1
  25054, // llvm.aarch64.sme.smla.za64.single.vg4x2
  25094, // llvm.aarch64.sme.smla.za64.single.vg4x4
  25134, // llvm.aarch64.sme.smla.za64.vg4x2
  25167, // llvm.aarch64.sme.smla.za64.vg4x4
  25200, // llvm.aarch64.sme.smlal.lane.vg2x1
  25234, // llvm.aarch64.sme.smlal.lane.vg2x2
  25268, // llvm.aarch64.sme.smlal.lane.vg2x4
  25302, // llvm.aarch64.sme.smlal.single.vg2x1
  25338, // llvm.aarch64.sme.smlal.single.vg2x2
  25374, // llvm.aarch64.sme.smlal.single.vg2x4
  25410, // llvm.aarch64.sme.smlal.vg2x2
  25439, // llvm.aarch64.sme.smlal.vg2x4
  25468, // llvm.aarch64.sme.smls.za32.lane.vg4x1
  25506, // llvm.aarch64.sme.smls.za32.lane.vg4x2
  25544, // llvm.aarch64.sme.smls.za32.lane.vg4x4
  25582, // llvm.aarch64.sme.smls.za32.single.vg4x1
  25622, // llvm.aarch64.sme.smls.za32.single.vg4x2
  25662, // llvm.aarch64.sme.smls.za32.single.vg4x4
  25702, // llvm.aarch64.sme.smls.za32.vg4x2
  25735, // llvm.aarch64.sme.smls.za32.vg4x4
  25768, // llvm.aarch64.sme.smls.za64.lane.vg4x1
  25806, // llvm.aarch64.sme.smls.za64.lane.vg4x2
  25844, // llvm.aarch64.sme.smls.za64.lane.vg4x4
  25882, // llvm.aarch64.sme.smls.za64.single.vg4x1
  25922, // llvm.aarch64.sme.smls.za64.single.vg4x2
  25962, // llvm.aarch64.sme.smls.za64.single.vg4x4
  26002, // llvm.aarch64.sme.smls.za64.vg4x2
  26035, // llvm.aarch64.sme.smls.za64.vg4x4
  26068, // llvm.aarch64.sme.smlsl.lane.vg2x1
  26102, // llvm.aarch64.sme.smlsl.lane.vg2x2
  26136, // llvm.aarch64.sme.smlsl.lane.vg2x4
  26170, // llvm.aarch64.sme.smlsl.single.vg2x1
  26206, // llvm.aarch64.sme.smlsl.single.vg2x2
  26242, // llvm.aarch64.sme.smlsl.single.vg2x4
  26278, // llvm.aarch64.sme.smlsl.vg2x2
  26307, // llvm.aarch64.sme.smlsl.vg2x4
  26336, // llvm.aarch64.sme.smop4a.wide.1x1
  26369, // llvm.aarch64.sme.smop4a.wide.1x2
  26402, // llvm.aarch64.sme.smop4a.wide.2x1
  26435, // llvm.aarch64.sme.smop4a.wide.2x2
  26468, // llvm.aarch64.sme.smop4a.za64.wide.1x1
  26506, // llvm.aarch64.sme.smop4a.za64.wide.1x2
  26544, // llvm.aarch64.sme.smop4a.za64.wide.2x1
  26582, // llvm.aarch64.sme.smop4a.za64.wide.2x2
  26620, // llvm.aarch64.sme.smop4s.wide.1x1
  26653, // llvm.aarch64.sme.smop4s.wide.1x2
  26686, // llvm.aarch64.sme.smop4s.wide.2x1
  26719, // llvm.aarch64.sme.smop4s.wide.2x2
  26752, // llvm.aarch64.sme.smop4s.za64.wide.1x1
  26790, // llvm.aarch64.sme.smop4s.za64.wide.1x2
  26828, // llvm.aarch64.sme.smop4s.za64.wide.2x1
  26866, // llvm.aarch64.sme.smop4s.za64.wide.2x2
  26904, // llvm.aarch64.sme.smopa.wide
  26932, // llvm.aarch64.sme.smopa.za32
  26960, // llvm.aarch64.sme.smops.wide
  26988, // llvm.aarch64.sme.smops.za32
  27016, // llvm.aarch64.sme.st1b.horiz
  27044, // llvm.aarch64.sme.st1b.vert
  27071, // llvm.aarch64.sme.st1d.horiz
  27099, // llvm.aarch64.sme.st1d.vert
  27126, // llvm.aarch64.sme.st1h.horiz
  27154, // llvm.aarch64.sme.st1h.vert
  27181, // llvm.aarch64.sme.st1q.horiz
  27209, // llvm.aarch64.sme.st1q.vert
  27236, // llvm.aarch64.sme.st1w.horiz
  27264, // llvm.aarch64.sme.st1w.vert
  27291, // llvm.aarch64.sme.stmopa.za32
  27320, // llvm.aarch64.sme.str
  27341, // llvm.aarch64.sme.str.zt
  27365, // llvm.aarch64.sme.sub.write.single.za.vg1x2
  27408, // llvm.aarch64.sme.sub.write.single.za.vg1x4
  27451, // llvm.aarch64.sme.sub.write.za.vg1x2
  27487, // llvm.aarch64.sme.sub.write.za.vg1x4
  27523, // llvm.aarch64.sme.sub.za16.vg1x2
  27555, // llvm.aarch64.sme.sub.za16.vg1x4
  27587, // llvm.aarch64.sme.sub.za32.vg1x2
  27619, // llvm.aarch64.sme.sub.za32.vg1x4
  27651, // llvm.aarch64.sme.sub.za64.vg1x2
  27683, // llvm.aarch64.sme.sub.za64.vg1x4
  27715, // llvm.aarch64.sme.sudot.lane.za32.vg1x2
  27754, // llvm.aarch64.sme.sudot.lane.za32.vg1x4
  27793, // llvm.aarch64.sme.sudot.single.za32.vg1x2
  27834, // llvm.aarch64.sme.sudot.single.za32.vg1x4
  27875, // llvm.aarch64.sme.sumla.za32.lane.vg4x1
  27914, // llvm.aarch64.sme.sumla.za32.lane.vg4x2
  27953, // llvm.aarch64.sme.sumla.za32.lane.vg4x4
  27992, // llvm.aarch64.sme.sumla.za32.single.vg4x2
  28033, // llvm.aarch64.sme.sumla.za32.single.vg4x4
  28074, // llvm.aarch64.sme.sumop4a.wide.1x1
  28108, // llvm.aarch64.sme.sumop4a.wide.1x2
  28142, // llvm.aarch64.sme.sumop4a.wide.2x1
  28176, // llvm.aarch64.sme.sumop4a.wide.2x2
  28210, // llvm.aarch64.sme.sumop4a.za64.wide.1x1
  28249, // llvm.aarch64.sme.sumop4a.za64.wide.1x2
  28288, // llvm.aarch64.sme.sumop4a.za64.wide.2x1
  28327, // llvm.aarch64.sme.sumop4a.za64.wide.2x2
  28366, // llvm.aarch64.sme.sumop4s.wide.1x1
  28400, // llvm.aarch64.sme.sumop4s.wide.1x2
  28434, // llvm.aarch64.sme.sumop4s.wide.2x1
  28468, // llvm.aarch64.sme.sumop4s.wide.2x2
  28502, // llvm.aarch64.sme.sumop4s.za64.wide.1x1
  28541, // llvm.aarch64.sme.sumop4s.za64.wide.1x2
  28580, // llvm.aarch64.sme.sumop4s.za64.wide.2x1
  28619, // llvm.aarch64.sme.sumop4s.za64.wide.2x2
  28658, // llvm.aarch64.sme.sumopa.wide
  28687, // llvm.aarch64.sme.sumops.wide
  28716, // llvm.aarch64.sme.sutmopa.za32
  28746, // llvm.aarch64.sme.suvdot.lane.za32.vg1x4
  28786, // llvm.aarch64.sme.svdot.lane.za32.vg1x2
  28825, // llvm.aarch64.sme.svdot.lane.za32.vg1x4
  28864, // llvm.aarch64.sme.svdot.lane.za64.vg1x4
  28903, // llvm.aarch64.sme.udot.lane.za32.vg1x2
  28941, // llvm.aarch64.sme.udot.lane.za32.vg1x4
  28979, // llvm.aarch64.sme.udot.lane.za64.vg1x2
  29017, // llvm.aarch64.sme.udot.lane.za64.vg1x4
  29055, // llvm.aarch64.sme.udot.single.za32.vg1x2
  29095, // llvm.aarch64.sme.udot.single.za32.vg1x4
  29135, // llvm.aarch64.sme.udot.single.za64.vg1x2
  29175, // llvm.aarch64.sme.udot.single.za64.vg1x4
  29215, // llvm.aarch64.sme.udot.za32.vg1x2
  29248, // llvm.aarch64.sme.udot.za32.vg1x4
  29281, // llvm.aarch64.sme.udot.za64.vg1x2
  29314, // llvm.aarch64.sme.udot.za64.vg1x4
  29347, // llvm.aarch64.sme.umla.za32.lane.vg4x1
  29385, // llvm.aarch64.sme.umla.za32.lane.vg4x2
  29423, // llvm.aarch64.sme.umla.za32.lane.vg4x4
  29461, // llvm.aarch64.sme.umla.za32.single.vg4x1
  29501, // llvm.aarch64.sme.umla.za32.single.vg4x2
  29541, // llvm.aarch64.sme.umla.za32.single.vg4x4
  29581, // llvm.aarch64.sme.umla.za32.vg4x2
  29614, // llvm.aarch64.sme.umla.za32.vg4x4
  29647, // llvm.aarch64.sme.umla.za64.lane.vg4x1
  29685, // llvm.aarch64.sme.umla.za64.lane.vg4x2
  29723, // llvm.aarch64.sme.umla.za64.lane.vg4x4
  29761, // llvm.aarch64.sme.umla.za64.single.vg4x1
  29801, // llvm.aarch64.sme.umla.za64.single.vg4x2
  29841, // llvm.aarch64.sme.umla.za64.single.vg4x4
  29881, // llvm.aarch64.sme.umla.za64.vg4x2
  29914, // llvm.aarch64.sme.umla.za64.vg4x4
  29947, // llvm.aarch64.sme.umlal.lane.vg2x1
  29981, // llvm.aarch64.sme.umlal.lane.vg2x2
  30015, // llvm.aarch64.sme.umlal.lane.vg2x4
  30049, // llvm.aarch64.sme.umlal.single.vg2x1
  30085, // llvm.aarch64.sme.umlal.single.vg2x2
  30121, // llvm.aarch64.sme.umlal.single.vg2x4
  30157, // llvm.aarch64.sme.umlal.vg2x2
  30186, // llvm.aarch64.sme.umlal.vg2x4
  30215, // llvm.aarch64.sme.umls.za32.lane.vg4x1
  30253, // llvm.aarch64.sme.umls.za32.lane.vg4x2
  30291, // llvm.aarch64.sme.umls.za32.lane.vg4x4
  30329, // llvm.aarch64.sme.umls.za32.single.vg4x1
  30369, // llvm.aarch64.sme.umls.za32.single.vg4x2
  30409, // llvm.aarch64.sme.umls.za32.single.vg4x4
  30449, // llvm.aarch64.sme.umls.za32.vg4x2
  30482, // llvm.aarch64.sme.umls.za32.vg4x4
  30515, // llvm.aarch64.sme.umls.za64.lane.vg4x1
  30553, // llvm.aarch64.sme.umls.za64.lane.vg4x2
  30591, // llvm.aarch64.sme.umls.za64.lane.vg4x4
  30629, // llvm.aarch64.sme.umls.za64.single.vg4x1
  30669, // llvm.aarch64.sme.umls.za64.single.vg4x2
  30709, // llvm.aarch64.sme.umls.za64.single.vg4x4
  30749, // llvm.aarch64.sme.umls.za64.vg4x2
  30782, // llvm.aarch64.sme.umls.za64.vg4x4
  30815, // llvm.aarch64.sme.umlsl.lane.vg2x1
  30849, // llvm.aarch64.sme.umlsl.lane.vg2x2
  30883, // llvm.aarch64.sme.umlsl.lane.vg2x4
  30917, // llvm.aarch64.sme.umlsl.single.vg2x1
  30953, // llvm.aarch64.sme.umlsl.single.vg2x2
  30989, // llvm.aarch64.sme.umlsl.single.vg2x4
  31025, // llvm.aarch64.sme.umlsl.vg2x2
  31054, // llvm.aarch64.sme.umlsl.vg2x4
  31083, // llvm.aarch64.sme.umop4a.wide.1x1
  31116, // llvm.aarch64.sme.umop4a.wide.1x2
  31149, // llvm.aarch64.sme.umop4a.wide.2x1
  31182, // llvm.aarch64.sme.umop4a.wide.2x2
  31215, // llvm.aarch64.sme.umop4a.za64.wide.1x1
  31253, // llvm.aarch64.sme.umop4a.za64.wide.1x2
  31291, // llvm.aarch64.sme.umop4a.za64.wide.2x1
  31329, // llvm.aarch64.sme.umop4a.za64.wide.2x2
  31367, // llvm.aarch64.sme.umop4s.wide.1x1
  31400, // llvm.aarch64.sme.umop4s.wide.1x2
  31433, // llvm.aarch64.sme.umop4s.wide.2x1
  31466, // llvm.aarch64.sme.umop4s.wide.2x2
  31499, // llvm.aarch64.sme.umop4s.za64.wide.1x1
  31537, // llvm.aarch64.sme.umop4s.za64.wide.1x2
  31575, // llvm.aarch64.sme.umop4s.za64.wide.2x1
  31613, // llvm.aarch64.sme.umop4s.za64.wide.2x2
  31651, // llvm.aarch64.sme.umopa.wide
  31679, // llvm.aarch64.sme.umopa.za32
  31707, // llvm.aarch64.sme.umops.wide
  31735, // llvm.aarch64.sme.umops.za32
  31763, // llvm.aarch64.sme.usdot.lane.za32.vg1x2
  31802, // llvm.aarch64.sme.usdot.lane.za32.vg1x4
  31841, // llvm.aarch64.sme.usdot.single.za32.vg1x2
  31882, // llvm.aarch64.sme.usdot.single.za32.vg1x4
  31923, // llvm.aarch64.sme.usdot.za32.vg1x2
  31957, // llvm.aarch64.sme.usdot.za32.vg1x4
  31991, // llvm.aarch64.sme.usmla.za32.lane.vg4x1
  32030, // llvm.aarch64.sme.usmla.za32.lane.vg4x2
  32069, // llvm.aarch64.sme.usmla.za32.lane.vg4x4
  32108, // llvm.aarch64.sme.usmla.za32.single.vg4x1
  32149, // llvm.aarch64.sme.usmla.za32.single.vg4x2
  32190, // llvm.aarch64.sme.usmla.za32.single.vg4x4
  32231, // llvm.aarch64.sme.usmla.za32.vg4x2
  32265, // llvm.aarch64.sme.usmla.za32.vg4x4
  32299, // llvm.aarch64.sme.usmop4a.wide.1x1
  32333, // llvm.aarch64.sme.usmop4a.wide.1x2
  32367, // llvm.aarch64.sme.usmop4a.wide.2x1
  32401, // llvm.aarch64.sme.usmop4a.wide.2x2
  32435, // llvm.aarch64.sme.usmop4a.za64.wide.1x1
  32474, // llvm.aarch64.sme.usmop4a.za64.wide.1x2
  32513, // llvm.aarch64.sme.usmop4a.za64.wide.2x1
  32552, // llvm.aarch64.sme.usmop4a.za64.wide.2x2
  32591, // llvm.aarch64.sme.usmop4s.wide.1x1
  32625, // llvm.aarch64.sme.usmop4s.wide.1x2
  32659, // llvm.aarch64.sme.usmop4s.wide.2x1
  32693, // llvm.aarch64.sme.usmop4s.wide.2x2
  32727, // llvm.aarch64.sme.usmop4s.za64.wide.1x1
  32766, // llvm.aarch64.sme.usmop4s.za64.wide.1x2
  32805, // llvm.aarch64.sme.usmop4s.za64.wide.2x1
  32844, // llvm.aarch64.sme.usmop4s.za64.wide.2x2
  32883, // llvm.aarch64.sme.usmopa.wide
  32912, // llvm.aarch64.sme.usmops.wide
  32941, // llvm.aarch64.sme.ustmopa.za32
  32971, // llvm.aarch64.sme.usvdot.lane.za32.vg1x4
  33011, // llvm.aarch64.sme.utmopa.za32
  33040, // llvm.aarch64.sme.uvdot.lane.za32.vg1x2
  33079, // llvm.aarch64.sme.uvdot.lane.za32.vg1x4
  33118, // llvm.aarch64.sme.uvdot.lane.za64.vg1x4
  33157, // llvm.aarch64.sme.write.hor.vg2
  33188, // llvm.aarch64.sme.write.hor.vg4
  33219, // llvm.aarch64.sme.write.horiz
  33248, // llvm.aarch64.sme.write.lane.zt
  33279, // llvm.aarch64.sme.write.ver.vg2
  33310, // llvm.aarch64.sme.write.ver.vg4
  33341, // llvm.aarch64.sme.write.vert
  33369, // llvm.aarch64.sme.write.vg1x2
  33398, // llvm.aarch64.sme.write.vg1x4
  33427, // llvm.aarch64.sme.write.zt
  33453, // llvm.aarch64.sme.writeq.horiz
  33483, // llvm.aarch64.sme.writeq.vert
  33512, // llvm.aarch64.sme.za.disable
  33540, // llvm.aarch64.sme.za.enable
  33567, // llvm.aarch64.sme.zero
  33589, // llvm.aarch64.sme.zero.za64.vg1x2
  33622, // llvm.aarch64.sme.zero.za64.vg1x4
  33655, // llvm.aarch64.sme.zero.za64.vg2x1
  33688, // llvm.aarch64.sme.zero.za64.vg2x2
  33721, // llvm.aarch64.sme.zero.za64.vg2x4
  33754, // llvm.aarch64.sme.zero.za64.vg4x1
  33787, // llvm.aarch64.sme.zero.za64.vg4x2
  33820, // llvm.aarch64.sme.zero.za64.vg4x4
  33853, // llvm.aarch64.sme.zero.zt
  33878, // llvm.aarch64.space
  33897, // llvm.aarch64.st64b
  33916, // llvm.aarch64.st64bv
  33936, // llvm.aarch64.st64bv0
  33957, // llvm.aarch64.stg
  33974, // llvm.aarch64.stgp
  33992, // llvm.aarch64.stlxp
  34011, // llvm.aarch64.stlxr
  34030, // llvm.aarch64.stxp
  34048, // llvm.aarch64.stxr
  34066, // llvm.aarch64.subp
  34084, // llvm.aarch64.sve.abs
  34105, // llvm.aarch64.sve.adclb
  34128, // llvm.aarch64.sve.adclt
  34151, // llvm.aarch64.sve.add
  34172, // llvm.aarch64.sve.add.single.x2
  34203, // llvm.aarch64.sve.add.single.x4
  34234, // llvm.aarch64.sve.add.u
  34257, // llvm.aarch64.sve.addhnb
  34281, // llvm.aarch64.sve.addhnt
  34305, // llvm.aarch64.sve.addp
  34327, // llvm.aarch64.sve.addqv
  34350, // llvm.aarch64.sve.adrb
  34372, // llvm.aarch64.sve.adrd
  34394, // llvm.aarch64.sve.adrh
  34416, // llvm.aarch64.sve.adrw
  34438, // llvm.aarch64.sve.aesd
  34460, // llvm.aarch64.sve.aesd.lane.x2
  34490, // llvm.aarch64.sve.aesd.lane.x4
  34520, // llvm.aarch64.sve.aesdimc.lane.x2
  34553, // llvm.aarch64.sve.aesdimc.lane.x4
  34586, // llvm.aarch64.sve.aese
  34608, // llvm.aarch64.sve.aese.lane.x2
  34638, // llvm.aarch64.sve.aese.lane.x4
  34668, // llvm.aarch64.sve.aesemc.lane.x2
  34700, // llvm.aarch64.sve.aesemc.lane.x4
  34732, // llvm.aarch64.sve.aesimc
  34756, // llvm.aarch64.sve.aesmc
  34779, // llvm.aarch64.sve.and
  34800, // llvm.aarch64.sve.and.u
  34823, // llvm.aarch64.sve.and.z
  34846, // llvm.aarch64.sve.andqv
  34869, // llvm.aarch64.sve.andv
  34891, // llvm.aarch64.sve.asr
  34912, // llvm.aarch64.sve.asr.u
  34935, // llvm.aarch64.sve.asr.wide
  34961, // llvm.aarch64.sve.asrd
  34983, // llvm.aarch64.sve.bcax
  35005, // llvm.aarch64.sve.bdep.x
  35029, // llvm.aarch64.sve.bext.x
  35053, // llvm.aarch64.sve.bfclamp.single.x2
  35088, // llvm.aarch64.sve.bfclamp.single.x4
  35123, // llvm.aarch64.sve.bfcvt.x2
  35149, // llvm.aarch64.sve.bfcvtn.x2
  35176, // llvm.aarch64.sve.bfdot
  35199, // llvm.aarch64.sve.bfdot.lane.v2
  35230, // llvm.aarch64.sve.bfmlalb
  35255, // llvm.aarch64.sve.bfmlalb.lane.v2
  35288, // llvm.aarch64.sve.bfmlalt
  35313, // llvm.aarch64.sve.bfmlalt.lane.v2
  35346, // llvm.aarch64.sve.bfmlslb
  35371, // llvm.aarch64.sve.bfmlslb.lane
  35401, // llvm.aarch64.sve.bfmlslt
  35426, // llvm.aarch64.sve.bfmlslt.lane
  35456, // llvm.aarch64.sve.bfmmla
  35480, // llvm.aarch64.sve.bgrp.x
  35504, // llvm.aarch64.sve.bic
  35525, // llvm.aarch64.sve.bic.u
  35548, // llvm.aarch64.sve.bic.z
  35571, // llvm.aarch64.sve.brka
  35593, // llvm.aarch64.sve.brka.z
  35617, // llvm.aarch64.sve.brkb
  35639, // llvm.aarch64.sve.brkb.z
  35663, // llvm.aarch64.sve.brkn.z
  35687, // llvm.aarch64.sve.brkpa.z
  35712, // llvm.aarch64.sve.brkpb.z
  35737, // llvm.aarch64.sve.bsl
  35758, // llvm.aarch64.sve.bsl1n
  35781, // llvm.aarch64.sve.bsl2n
  35804, // llvm.aarch64.sve.cadd.x
  35828, // llvm.aarch64.sve.cdot
  35850, // llvm.aarch64.sve.cdot.lane
  35877, // llvm.aarch64.sve.clasta
  35901, // llvm.aarch64.sve.clasta.n
  35927, // llvm.aarch64.sve.clastb
  35951, // llvm.aarch64.sve.clastb.n
  35977, // llvm.aarch64.sve.cls
  35998, // llvm.aarch64.sve.clz
  36019, // llvm.aarch64.sve.cmla.lane.x
  36048, // llvm.aarch64.sve.cmla.x
  36072, // llvm.aarch64.sve.cmpeq
  36095, // llvm.aarch64.sve.cmpeq.wide
  36123, // llvm.aarch64.sve.cmpge
  36146, // llvm.aarch64.sve.cmpge.wide
  36174, // llvm.aarch64.sve.cmpgt
  36197, // llvm.aarch64.sve.cmpgt.wide
  36225, // llvm.aarch64.sve.cmphi
  36248, // llvm.aarch64.sve.cmphi.wide
  36276, // llvm.aarch64.sve.cmphs
  36299, // llvm.aarch64.sve.cmphs.wide
  36327, // llvm.aarch64.sve.cmple.wide
  36355, // llvm.aarch64.sve.cmplo.wide
  36383, // llvm.aarch64.sve.cmpls.wide
  36411, // llvm.aarch64.sve.cmplt.wide
  36439, // llvm.aarch64.sve.cmpne
  36462, // llvm.aarch64.sve.cmpne.wide
  36490, // llvm.aarch64.sve.cnot
  36512, // llvm.aarch64.sve.cnt
  36533, // llvm.aarch64.sve.cntb
  36555, // llvm.aarch64.sve.cntd
  36577, // llvm.aarch64.sve.cnth
  36599, // llvm.aarch64.sve.cntp
  36621, // llvm.aarch64.sve.cntp.c16
  36647, // llvm.aarch64.sve.cntp.c32
  36673, // llvm.aarch64.sve.cntp.c64
  36699, // llvm.aarch64.sve.cntp.c8
  36724, // llvm.aarch64.sve.cntw
  36746, // llvm.aarch64.sve.compact
  36771, // llvm.aarch64.sve.convert.from.svbool
  36808, // llvm.aarch64.sve.convert.to.svbool
  36843, // llvm.aarch64.sve.dup
  36864, // llvm.aarch64.sve.dup.laneq
  36891, // llvm.aarch64.sve.dup.x
  36914, // llvm.aarch64.sve.dupq.lane
  36941, // llvm.aarch64.sve.eor
  36962, // llvm.aarch64.sve.eor.u
  36985, // llvm.aarch64.sve.eor.z
  37008, // llvm.aarch64.sve.eor3
  37030, // llvm.aarch64.sve.eorbt
  37053, // llvm.aarch64.sve.eorqv
  37076, // llvm.aarch64.sve.eortb
  37099, // llvm.aarch64.sve.eorv
  37121, // llvm.aarch64.sve.expand
  37145, // llvm.aarch64.sve.ext
  37166, // llvm.aarch64.sve.extq
  37188, // llvm.aarch64.sve.fabd
  37210, // llvm.aarch64.sve.fabd.u
  37234, // llvm.aarch64.sve.fabs
  37256, // llvm.aarch64.sve.facge
  37279, // llvm.aarch64.sve.facgt
  37302, // llvm.aarch64.sve.fadd
  37324, // llvm.aarch64.sve.fadd.u
  37348, // llvm.aarch64.sve.fadda
  37371, // llvm.aarch64.sve.faddp
  37394, // llvm.aarch64.sve.faddqv
  37418, // llvm.aarch64.sve.faddv
  37441, // llvm.aarch64.sve.famax
  37464, // llvm.aarch64.sve.famax.u
  37489, // llvm.aarch64.sve.famin
  37512, // llvm.aarch64.sve.famin.u
  37537, // llvm.aarch64.sve.fcadd
  37560, // llvm.aarch64.sve.fclamp
  37584, // llvm.aarch64.sve.fclamp.single.x2
  37618, // llvm.aarch64.sve.fclamp.single.x4
  37652, // llvm.aarch64.sve.fcmla
  37675, // llvm.aarch64.sve.fcmla.lane
  37703, // llvm.aarch64.sve.fcmpeq
  37727, // llvm.aarch64.sve.fcmpge
  37751, // llvm.aarch64.sve.fcmpgt
  37775, // llvm.aarch64.sve.fcmpne
  37799, // llvm.aarch64.sve.fcmpuo
  37823, // llvm.aarch64.sve.fcvt
  37845, // llvm.aarch64.sve.fcvt.bf16f32.v2
  37878, // llvm.aarch64.sve.fcvt.f16f32
  37907, // llvm.aarch64.sve.fcvt.f16f64
  37936, // llvm.aarch64.sve.fcvt.f32f16
  37965, // llvm.aarch64.sve.fcvt.f32f64
  37994, // llvm.aarch64.sve.fcvt.f64f16
  38023, // llvm.aarch64.sve.fcvt.f64f32
  38052, // llvm.aarch64.sve.fcvt.widen.x2
  38083, // llvm.aarch64.sve.fcvt.x2
  38108, // llvm.aarch64.sve.fcvtl.widen.x2
  38140, // llvm.aarch64.sve.fcvtlt.f32f16
  38171, // llvm.aarch64.sve.fcvtlt.f64f32
  38202, // llvm.aarch64.sve.fcvtn.x2
  38228, // llvm.aarch64.sve.fcvtnt.bf16f32.v2
  38263, // llvm.aarch64.sve.fcvtnt.f16f32
  38294, // llvm.aarch64.sve.fcvtnt.f32f64
  38325, // llvm.aarch64.sve.fcvtnt.z.bf16f32
  38359, // llvm.aarch64.sve.fcvtnt.z.f16f32
  38392, // llvm.aarch64.sve.fcvtnt.z.f32f64
  38425, // llvm.aarch64.sve.fcvtx.f32f64
  38455, // llvm.aarch64.sve.fcvtxnt.f32f64
  38487, // llvm.aarch64.sve.fcvtxnt.z.f32f64
  38521, // llvm.aarch64.sve.fcvtzs
  38545, // llvm.aarch64.sve.fcvtzs.i32f16
  38576, // llvm.aarch64.sve.fcvtzs.i32f64
  38607, // llvm.aarch64.sve.fcvtzs.i64f16
  38638, // llvm.aarch64.sve.fcvtzs.i64f32
  38669, // llvm.aarch64.sve.fcvtzs.x2
  38696, // llvm.aarch64.sve.fcvtzs.x4
  38723, // llvm.aarch64.sve.fcvtzu
  38747, // llvm.aarch64.sve.fcvtzu.i32f16
  38778, // llvm.aarch64.sve.fcvtzu.i32f64
  38809, // llvm.aarch64.sve.fcvtzu.i64f16
  38840, // llvm.aarch64.sve.fcvtzu.i64f32
  38871, // llvm.aarch64.sve.fcvtzu.x2
  38898, // llvm.aarch64.sve.fcvtzu.x4
  38925, // llvm.aarch64.sve.fdiv
  38947, // llvm.aarch64.sve.fdiv.u
  38971, // llvm.aarch64.sve.fdivr
  38994, // llvm.aarch64.sve.fdot.lane.x2
  39024, // llvm.aarch64.sve.fdot.x2
  39049, // llvm.aarch64.sve.fexpa.x
  39074, // llvm.aarch64.sve.firstp
  39098, // llvm.aarch64.sve.flogb
  39121, // llvm.aarch64.sve.fmad
  39143, // llvm.aarch64.sve.fmax
  39165, // llvm.aarch64.sve.fmax.single.x2
  39197, // llvm.aarch64.sve.fmax.single.x4
  39229, // llvm.aarch64.sve.fmax.u
  39253, // llvm.aarch64.sve.fmax.x2
  39278, // llvm.aarch64.sve.fmax.x4
  39303, // llvm.aarch64.sve.fmaxnm
  39327, // llvm.aarch64.sve.fmaxnm.single.x2
  39361, // llvm.aarch64.sve.fmaxnm.single.x4
  39395, // llvm.aarch64.sve.fmaxnm.u
  39421, // llvm.aarch64.sve.fmaxnm.x2
  39448, // llvm.aarch64.sve.fmaxnm.x4
  39475, // llvm.aarch64.sve.fmaxnmp
  39500, // llvm.aarch64.sve.fmaxnmqv
  39526, // llvm.aarch64.sve.fmaxnmv
  39551, // llvm.aarch64.sve.fmaxp
  39574, // llvm.aarch64.sve.fmaxqv
  39598, // llvm.aarch64.sve.fmaxv
  39621, // llvm.aarch64.sve.fmin
  39643, // llvm.aarch64.sve.fmin.single.x2
  39675, // llvm.aarch64.sve.fmin.single.x4
  39707, // llvm.aarch64.sve.fmin.u
  39731, // llvm.aarch64.sve.fmin.x2
  39756, // llvm.aarch64.sve.fmin.x4
  39781, // llvm.aarch64.sve.fminnm
  39805, // llvm.aarch64.sve.fminnm.single.x2
  39839, // llvm.aarch64.sve.fminnm.single.x4
  39873, // llvm.aarch64.sve.fminnm.u
  39899, // llvm.aarch64.sve.fminnm.x2
  39926, // llvm.aarch64.sve.fminnm.x4
  39953, // llvm.aarch64.sve.fminnmp
  39978, // llvm.aarch64.sve.fminnmqv
  40004, // llvm.aarch64.sve.fminnmv
  40029, // llvm.aarch64.sve.fminp
  40052, // llvm.aarch64.sve.fminqv
  40076, // llvm.aarch64.sve.fminv
  40099, // llvm.aarch64.sve.fmla
  40121, // llvm.aarch64.sve.fmla.lane
  40148, // llvm.aarch64.sve.fmla.u
  40172, // llvm.aarch64.sve.fmlalb
  40196, // llvm.aarch64.sve.fmlalb.lane
  40225, // llvm.aarch64.sve.fmlalt
  40249, // llvm.aarch64.sve.fmlalt.lane
  40278, // llvm.aarch64.sve.fmls
  40300, // llvm.aarch64.sve.fmls.lane
  40327, // llvm.aarch64.sve.fmls.u
  40351, // llvm.aarch64.sve.fmlslb
  40375, // llvm.aarch64.sve.fmlslb.lane
  40404, // llvm.aarch64.sve.fmlslt
  40428, // llvm.aarch64.sve.fmlslt.lane
  40457, // llvm.aarch64.sve.fmmla
  40480, // llvm.aarch64.sve.fmsb
  40502, // llvm.aarch64.sve.fmul
  40524, // llvm.aarch64.sve.fmul.lane
  40551, // llvm.aarch64.sve.fmul.single.x2
  40583, // llvm.aarch64.sve.fmul.single.x4
  40615, // llvm.aarch64.sve.fmul.u
  40639, // llvm.aarch64.sve.fmul.x2
  40664, // llvm.aarch64.sve.fmul.x4
  40689, // llvm.aarch64.sve.fmulx
  40712, // llvm.aarch64.sve.fmulx.u
  40737, // llvm.aarch64.sve.fneg
  40759, // llvm.aarch64.sve.fnmad
  40782, // llvm.aarch64.sve.fnmla
  40805, // llvm.aarch64.sve.fnmla.u
  40830, // llvm.aarch64.sve.fnmls
  40853, // llvm.aarch64.sve.fnmls.u
  40878, // llvm.aarch64.sve.fnmsb
  40901, // llvm.aarch64.sve.fp8.cvt.x2
  40929, // llvm.aarch64.sve.fp8.cvt.x4
  40957, // llvm.aarch64.sve.fp8.cvt1
  40983, // llvm.aarch64.sve.fp8.cvt1.x2
  41012, // llvm.aarch64.sve.fp8.cvt2
  41038, // llvm.aarch64.sve.fp8.cvt2.x2
  41067, // llvm.aarch64.sve.fp8.cvtl1.x2
  41097, // llvm.aarch64.sve.fp8.cvtl2.x2
  41127, // llvm.aarch64.sve.fp8.cvtlt1
  41155, // llvm.aarch64.sve.fp8.cvtlt2
  41183, // llvm.aarch64.sve.fp8.cvtn
  41209, // llvm.aarch64.sve.fp8.cvtn.x4
  41238, // llvm.aarch64.sve.fp8.cvtnb
  41265, // llvm.aarch64.sve.fp8.cvtnt
  41292, // llvm.aarch64.sve.fp8.fdot
  41318, // llvm.aarch64.sve.fp8.fdot.lane
  41349, // llvm.aarch64.sve.fp8.fmlalb
  41377, // llvm.aarch64.sve.fp8.fmlalb.lane
  41410, // llvm.aarch64.sve.fp8.fmlallbb
  41440, // llvm.aarch64.sve.fp8.fmlallbb.lane
  41475, // llvm.aarch64.sve.fp8.fmlallbt
  41505, // llvm.aarch64.sve.fp8.fmlallbt.lane
  41540, // llvm.aarch64.sve.fp8.fmlalltb
  41570, // llvm.aarch64.sve.fp8.fmlalltb.lane
  41605, // llvm.aarch64.sve.fp8.fmlalltt
  41635, // llvm.aarch64.sve.fp8.fmlalltt.lane
  41670, // llvm.aarch64.sve.fp8.fmlalt
  41698, // llvm.aarch64.sve.fp8.fmlalt.lane
  41731, // llvm.aarch64.sve.fp8.fmmla
  41758, // llvm.aarch64.sve.frecpe.x
  41784, // llvm.aarch64.sve.frecps.x
  41810, // llvm.aarch64.sve.frecpx
  41834, // llvm.aarch64.sve.frint32x
  41860, // llvm.aarch64.sve.frint32z
  41886, // llvm.aarch64.sve.frint64x
  41912, // llvm.aarch64.sve.frint64z
  41938, // llvm.aarch64.sve.frinta
  41962, // llvm.aarch64.sve.frinta.x2
  41989, // llvm.aarch64.sve.frinta.x4
  42016, // llvm.aarch64.sve.frinti
  42040, // llvm.aarch64.sve.frintm
  42064, // llvm.aarch64.sve.frintm.x2
  42091, // llvm.aarch64.sve.frintm.x4
  42118, // llvm.aarch64.sve.frintn
  42142, // llvm.aarch64.sve.frintn.x2
  42169, // llvm.aarch64.sve.frintn.x4
  42196, // llvm.aarch64.sve.frintp
  42220, // llvm.aarch64.sve.frintp.x2
  42247, // llvm.aarch64.sve.frintp.x4
  42274, // llvm.aarch64.sve.frintx
  42298, // llvm.aarch64.sve.frintz
  42322, // llvm.aarch64.sve.frsqrte.x
  42349, // llvm.aarch64.sve.frsqrts.x
  42376, // llvm.aarch64.sve.fscale
  42400, // llvm.aarch64.sve.fscale.single.x2
  42434, // llvm.aarch64.sve.fscale.single.x4
  42468, // llvm.aarch64.sve.fscale.x2
  42495, // llvm.aarch64.sve.fscale.x4
  42522, // llvm.aarch64.sve.fsqrt
  42545, // llvm.aarch64.sve.fsub
  42567, // llvm.aarch64.sve.fsub.u
  42591, // llvm.aarch64.sve.fsubr
  42614, // llvm.aarch64.sve.ftmad.x
  42639, // llvm.aarch64.sve.ftsmul.x
  42665, // llvm.aarch64.sve.ftssel.x
  42691, // llvm.aarch64.sve.histcnt
  42716, // llvm.aarch64.sve.histseg
  42741, // llvm.aarch64.sve.index
  42764, // llvm.aarch64.sve.insr
  42786, // llvm.aarch64.sve.lasta
  42809, // llvm.aarch64.sve.lastb
  42832, // llvm.aarch64.sve.lastp
  42855, // llvm.aarch64.sve.ld1
  42876, // llvm.aarch64.sve.ld1.gather
  42904, // llvm.aarch64.sve.ld1.gather.index
  42938, // llvm.aarch64.sve.ld1.gather.scalar.offset
  42980, // llvm.aarch64.sve.ld1.gather.sxtw
  43013, // llvm.aarch64.sve.ld1.gather.sxtw.index
  43052, // llvm.aarch64.sve.ld1.gather.uxtw
  43085, // llvm.aarch64.sve.ld1.gather.uxtw.index
  43124, // llvm.aarch64.sve.ld1.pn.x2
  43151, // llvm.aarch64.sve.ld1.pn.x4
  43178, // llvm.aarch64.sve.ld1q.gather.index
  43213, // llvm.aarch64.sve.ld1q.gather.scalar.offset
  43256, // llvm.aarch64.sve.ld1q.gather.vector.offset
  43299, // llvm.aarch64.sve.ld1ro
  43322, // llvm.aarch64.sve.ld1rq
  43345, // llvm.aarch64.sve.ld1udq
  43369, // llvm.aarch64.sve.ld1uwq
  43393, // llvm.aarch64.sve.ld2.sret
  43419, // llvm.aarch64.sve.ld2q.sret
  43446, // llvm.aarch64.sve.ld3.sret
  43472, // llvm.aarch64.sve.ld3q.sret
  43499, // llvm.aarch64.sve.ld4.sret
  43525, // llvm.aarch64.sve.ld4q.sret
  43552, // llvm.aarch64.sve.ldff1
  43575, // llvm.aarch64.sve.ldff1.gather
  43605, // llvm.aarch64.sve.ldff1.gather.index
  43641, // llvm.aarch64.sve.ldff1.gather.scalar.offset
  43685, // llvm.aarch64.sve.ldff1.gather.sxtw
  43720, // llvm.aarch64.sve.ldff1.gather.sxtw.index
  43761, // llvm.aarch64.sve.ldff1.gather.uxtw
  43796, // llvm.aarch64.sve.ldff1.gather.uxtw.index
  43837, // llvm.aarch64.sve.ldnf1
  43860, // llvm.aarch64.sve.ldnt1
  43883, // llvm.aarch64.sve.ldnt1.gather
  43913, // llvm.aarch64.sve.ldnt1.gather.index
  43949, // llvm.aarch64.sve.ldnt1.gather.scalar.offset
  43993, // llvm.aarch64.sve.ldnt1.gather.uxtw
  44028, // llvm.aarch64.sve.ldnt1.pn.x2
  44057, // llvm.aarch64.sve.ldnt1.pn.x4
  44086, // llvm.aarch64.sve.lsl
  44107, // llvm.aarch64.sve.lsl.u
  44130, // llvm.aarch64.sve.lsl.wide
  44156, // llvm.aarch64.sve.lsr
  44177, // llvm.aarch64.sve.lsr.u
  44200, // llvm.aarch64.sve.lsr.wide
  44226, // llvm.aarch64.sve.luti2.lane
  44254, // llvm.aarch64.sve.luti4.lane
  44282, // llvm.aarch64.sve.luti4.lane.x2
  44313, // llvm.aarch64.sve.mad
  44334, // llvm.aarch64.sve.match
  44357, // llvm.aarch64.sve.mla
  44378, // llvm.aarch64.sve.mla.lane
  44404, // llvm.aarch64.sve.mla.u
  44427, // llvm.aarch64.sve.mls
  44448, // llvm.aarch64.sve.mls.lane
  44474, // llvm.aarch64.sve.mls.u
  44497, // llvm.aarch64.sve.msb
  44518, // llvm.aarch64.sve.mul
  44539, // llvm.aarch64.sve.mul.lane
  44565, // llvm.aarch64.sve.mul.u
  44588, // llvm.aarch64.sve.nand.z
  44612, // llvm.aarch64.sve.nbsl
  44634, // llvm.aarch64.sve.neg
  44655, // llvm.aarch64.sve.nmatch
  44679, // llvm.aarch64.sve.nor.z
  44702, // llvm.aarch64.sve.not
  44723, // llvm.aarch64.sve.orn.z
  44746, // llvm.aarch64.sve.orqv
  44768, // llvm.aarch64.sve.orr
  44789, // llvm.aarch64.sve.orr.u
  44812, // llvm.aarch64.sve.orr.z
  44835, // llvm.aarch64.sve.orv
  44856, // llvm.aarch64.sve.pext
  44878, // llvm.aarch64.sve.pext.x2
  44903, // llvm.aarch64.sve.pfirst
  44927, // llvm.aarch64.sve.pmlal.pair.x2
  44958, // llvm.aarch64.sve.pmov.to.pred.lane
  44993, // llvm.aarch64.sve.pmov.to.pred.lane.zero
  45033, // llvm.aarch64.sve.pmov.to.vector.lane.merging
  45078, // llvm.aarch64.sve.pmov.to.vector.lane.zeroing
  45123, // llvm.aarch64.sve.pmul
  45145, // llvm.aarch64.sve.pmull.pair.x2
  45176, // llvm.aarch64.sve.pmullb.pair
  45205, // llvm.aarch64.sve.pmullt.pair
  45234, // llvm.aarch64.sve.pnext
  45257, // llvm.aarch64.sve.prf
  45278, // llvm.aarch64.sve.prfb.gather.index
  45313, // llvm.aarch64.sve.prfb.gather.scalar.offset
  45356, // llvm.aarch64.sve.prfb.gather.sxtw.index
  45396, // llvm.aarch64.sve.prfb.gather.uxtw.index
  45436, // llvm.aarch64.sve.prfd.gather.index
  45471, // llvm.aarch64.sve.prfd.gather.scalar.offset
  45514, // llvm.aarch64.sve.prfd.gather.sxtw.index
  45554, // llvm.aarch64.sve.prfd.gather.uxtw.index
  45594, // llvm.aarch64.sve.prfh.gather.index
  45629, // llvm.aarch64.sve.prfh.gather.scalar.offset
  45672, // llvm.aarch64.sve.prfh.gather.sxtw.index
  45712, // llvm.aarch64.sve.prfh.gather.uxtw.index
  45752, // llvm.aarch64.sve.prfw.gather.index
  45787, // llvm.aarch64.sve.prfw.gather.scalar.offset
  45830, // llvm.aarch64.sve.prfw.gather.sxtw.index
  45870, // llvm.aarch64.sve.prfw.gather.uxtw.index
  45910, // llvm.aarch64.sve.psel
  45932, // llvm.aarch64.sve.ptest.any
  45959, // llvm.aarch64.sve.ptest.first
  45988, // llvm.aarch64.sve.ptest.last
  46016, // llvm.aarch64.sve.ptrue
  46039, // llvm.aarch64.sve.ptrue.c16
  46066, // llvm.aarch64.sve.ptrue.c32
  46093, // llvm.aarch64.sve.ptrue.c64
  46120, // llvm.aarch64.sve.ptrue.c8
  46146, // llvm.aarch64.sve.punpkhi
  46171, // llvm.aarch64.sve.punpklo
  46196, // llvm.aarch64.sve.raddhnb
  46221, // llvm.aarch64.sve.raddhnt
  46246, // llvm.aarch64.sve.rax1
  46268, // llvm.aarch64.sve.rbit
  46290, // llvm.aarch64.sve.rdffr
  46313, // llvm.aarch64.sve.rdffr.z
  46338, // llvm.aarch64.sve.rev.b16
  46363, // llvm.aarch64.sve.rev.b32
  46388, // llvm.aarch64.sve.rev.b64
  46413, // llvm.aarch64.sve.revb
  46435, // llvm.aarch64.sve.revd
  46457, // llvm.aarch64.sve.revh
  46479, // llvm.aarch64.sve.revw
  46501, // llvm.aarch64.sve.rshrnb
  46525, // llvm.aarch64.sve.rshrnt
  46549, // llvm.aarch64.sve.rsubhnb
  46574, // llvm.aarch64.sve.rsubhnt
  46599, // llvm.aarch64.sve.saba
  46621, // llvm.aarch64.sve.sabalb
  46645, // llvm.aarch64.sve.sabalt
  46669, // llvm.aarch64.sve.sabd
  46691, // llvm.aarch64.sve.sabd.u
  46715, // llvm.aarch64.sve.sabdlb
  46739, // llvm.aarch64.sve.sabdlt
  46763, // llvm.aarch64.sve.sadalp
  46787, // llvm.aarch64.sve.saddlb
  46811, // llvm.aarch64.sve.saddlbt
  46836, // llvm.aarch64.sve.saddlt
  46860, // llvm.aarch64.sve.saddv
  46883, // llvm.aarch64.sve.saddwb
  46907, // llvm.aarch64.sve.saddwt
  46931, // llvm.aarch64.sve.sbclb
  46954, // llvm.aarch64.sve.sbclt
  46977, // llvm.aarch64.sve.sclamp
  47001, // llvm.aarch64.sve.sclamp.single.x2
  47035, // llvm.aarch64.sve.sclamp.single.x4
  47069, // llvm.aarch64.sve.scvtf
  47092, // llvm.aarch64.sve.scvtf.f16i32
  47122, // llvm.aarch64.sve.scvtf.f16i64
  47152, // llvm.aarch64.sve.scvtf.f32i64
  47182, // llvm.aarch64.sve.scvtf.f64i32
  47212, // llvm.aarch64.sve.scvtf.x2
  47238, // llvm.aarch64.sve.scvtf.x4
  47264, // llvm.aarch64.sve.sdiv
  47286, // llvm.aarch64.sve.sdiv.u
  47310, // llvm.aarch64.sve.sdivr
  47333, // llvm.aarch64.sve.sdot
  47355, // llvm.aarch64.sve.sdot.lane
  47382, // llvm.aarch64.sve.sdot.lane.x2
  47412, // llvm.aarch64.sve.sdot.x2
  47437, // llvm.aarch64.sve.sel
  47458, // llvm.aarch64.sve.sel.x2
  47482, // llvm.aarch64.sve.sel.x4
  47506, // llvm.aarch64.sve.setffr
  47530, // llvm.aarch64.sve.shadd
  47553, // llvm.aarch64.sve.shrnb
  47576, // llvm.aarch64.sve.shrnt
  47599, // llvm.aarch64.sve.shsub
  47622, // llvm.aarch64.sve.shsub.u
  47647, // llvm.aarch64.sve.shsubr
  47671, // llvm.aarch64.sve.sli
  47692, // llvm.aarch64.sve.sm4e
  47714, // llvm.aarch64.sve.sm4ekey
  47739, // llvm.aarch64.sve.smax
  47761, // llvm.aarch64.sve.smax.single.x2
  47793, // llvm.aarch64.sve.smax.single.x4
  47825, // llvm.aarch64.sve.smax.u
  47849, // llvm.aarch64.sve.smax.x2
  47874, // llvm.aarch64.sve.smax.x4
  47899, // llvm.aarch64.sve.smaxp
  47922, // llvm.aarch64.sve.smaxqv
  47946, // llvm.aarch64.sve.smaxv
  47969, // llvm.aarch64.sve.smin
  47991, // llvm.aarch64.sve.smin.single.x2
  48023, // llvm.aarch64.sve.smin.single.x4
  48055, // llvm.aarch64.sve.smin.u
  48079, // llvm.aarch64.sve.smin.x2
  48104, // llvm.aarch64.sve.smin.x4
  48129, // llvm.aarch64.sve.sminp
  48152, // llvm.aarch64.sve.sminqv
  48176, // llvm.aarch64.sve.sminv
  48199, // llvm.aarch64.sve.smlalb
  48223, // llvm.aarch64.sve.smlalb.lane
  48252, // llvm.aarch64.sve.smlalt
  48276, // llvm.aarch64.sve.smlalt.lane
  48305, // llvm.aarch64.sve.smlslb
  48329, // llvm.aarch64.sve.smlslb.lane
  48358, // llvm.aarch64.sve.smlslt
  48382, // llvm.aarch64.sve.smlslt.lane
  48411, // llvm.aarch64.sve.smmla
  48434, // llvm.aarch64.sve.smulh
  48457, // llvm.aarch64.sve.smulh.u
  48482, // llvm.aarch64.sve.smullb
  48506, // llvm.aarch64.sve.smullb.lane
  48535, // llvm.aarch64.sve.smullt
  48559, // llvm.aarch64.sve.smullt.lane
  48588, // llvm.aarch64.sve.splice
  48612, // llvm.aarch64.sve.sqabs
  48635, // llvm.aarch64.sve.sqadd
  48658, // llvm.aarch64.sve.sqadd.x
  48683, // llvm.aarch64.sve.sqcadd.x
  48709, // llvm.aarch64.sve.sqcvt.x2
  48735, // llvm.aarch64.sve.sqcvt.x4
  48761, // llvm.aarch64.sve.sqcvtn.x2
  48788, // llvm.aarch64.sve.sqcvtn.x4
  48815, // llvm.aarch64.sve.sqcvtu.x2
  48842, // llvm.aarch64.sve.sqcvtu.x4
  48869, // llvm.aarch64.sve.sqcvtun.x2
  48897, // llvm.aarch64.sve.sqcvtun.x4
  48925, // llvm.aarch64.sve.sqdecb.n32
  48953, // llvm.aarch64.sve.sqdecb.n64
  48981, // llvm.aarch64.sve.sqdecd
  49005, // llvm.aarch64.sve.sqdecd.n32
  49033, // llvm.aarch64.sve.sqdecd.n64
  49061, // llvm.aarch64.sve.sqdech
  49085, // llvm.aarch64.sve.sqdech.n32
  49113, // llvm.aarch64.sve.sqdech.n64
  49141, // llvm.aarch64.sve.sqdecp
  49165, // llvm.aarch64.sve.sqdecp.n32
  49193, // llvm.aarch64.sve.sqdecp.n64
  49221, // llvm.aarch64.sve.sqdecw
  49245, // llvm.aarch64.sve.sqdecw.n32
  49273, // llvm.aarch64.sve.sqdecw.n64
  49301, // llvm.aarch64.sve.sqdmlalb
  49327, // llvm.aarch64.sve.sqdmlalb.lane
  49358, // llvm.aarch64.sve.sqdmlalbt
  49385, // llvm.aarch64.sve.sqdmlalt
  49411, // llvm.aarch64.sve.sqdmlalt.lane
  49442, // llvm.aarch64.sve.sqdmlslb
  49468, // llvm.aarch64.sve.sqdmlslb.lane
  49499, // llvm.aarch64.sve.sqdmlslbt
  49526, // llvm.aarch64.sve.sqdmlslt
  49552, // llvm.aarch64.sve.sqdmlslt.lane
  49583, // llvm.aarch64.sve.sqdmulh
  49608, // llvm.aarch64.sve.sqdmulh.lane
  49638, // llvm.aarch64.sve.sqdmulh.single.vgx2
  49675, // llvm.aarch64.sve.sqdmulh.single.vgx4
  49712, // llvm.aarch64.sve.sqdmulh.vgx2
  49742, // llvm.aarch64.sve.sqdmulh.vgx4
  49772, // llvm.aarch64.sve.sqdmullb
  49798, // llvm.aarch64.sve.sqdmullb.lane
  49829, // llvm.aarch64.sve.sqdmullt
  49855, // llvm.aarch64.sve.sqdmullt.lane
  49886, // llvm.aarch64.sve.sqincb.n32
  49914, // llvm.aarch64.sve.sqincb.n64
  49942, // llvm.aarch64.sve.sqincd
  49966, // llvm.aarch64.sve.sqincd.n32
  49994, // llvm.aarch64.sve.sqincd.n64
  50022, // llvm.aarch64.sve.sqinch
  50046, // llvm.aarch64.sve.sqinch.n32
  50074, // llvm.aarch64.sve.sqinch.n64
  50102, // llvm.aarch64.sve.sqincp
  50126, // llvm.aarch64.sve.sqincp.n32
  50154, // llvm.aarch64.sve.sqincp.n64
  50182, // llvm.aarch64.sve.sqincw
  50206, // llvm.aarch64.sve.sqincw.n32
  50234, // llvm.aarch64.sve.sqincw.n64
  50262, // llvm.aarch64.sve.sqneg
  50285, // llvm.aarch64.sve.sqrdcmlah.lane.x
  50319, // llvm.aarch64.sve.sqrdcmlah.x
  50348, // llvm.aarch64.sve.sqrdmlah
  50374, // llvm.aarch64.sve.sqrdmlah.lane
  50405, // llvm.aarch64.sve.sqrdmlsh
  50431, // llvm.aarch64.sve.sqrdmlsh.lane
  50462, // llvm.aarch64.sve.sqrdmulh
  50488, // llvm.aarch64.sve.sqrdmulh.lane
  50519, // llvm.aarch64.sve.sqrshl
  50543, // llvm.aarch64.sve.sqrshl.u
  50569, // llvm.aarch64.sve.sqrshr.x2
  50596, // llvm.aarch64.sve.sqrshr.x4
  50623, // llvm.aarch64.sve.sqrshrn.x2
  50651, // llvm.aarch64.sve.sqrshrn.x4
  50679, // llvm.aarch64.sve.sqrshrnb
  50705, // llvm.aarch64.sve.sqrshrnt
  50731, // llvm.aarch64.sve.sqrshru.x2
  50759, // llvm.aarch64.sve.sqrshru.x4
  50787, // llvm.aarch64.sve.sqrshrun.x2
  50816, // llvm.aarch64.sve.sqrshrun.x4
  50845, // llvm.aarch64.sve.sqrshrunb
  50872, // llvm.aarch64.sve.sqrshrunt
  50899, // llvm.aarch64.sve.sqshl
  50922, // llvm.aarch64.sve.sqshl.u
  50947, // llvm.aarch64.sve.sqshlu
  50971, // llvm.aarch64.sve.sqshrnb
  50996, // llvm.aarch64.sve.sqshrnt
  51021, // llvm.aarch64.sve.sqshrunb
  51047, // llvm.aarch64.sve.sqshrunt
  51073, // llvm.aarch64.sve.sqsub
  51096, // llvm.aarch64.sve.sqsub.u
  51121, // llvm.aarch64.sve.sqsub.x
  51146, // llvm.aarch64.sve.sqsubr
  51170, // llvm.aarch64.sve.sqxtnb
  51194, // llvm.aarch64.sve.sqxtnt
  51218, // llvm.aarch64.sve.sqxtunb
  51243, // llvm.aarch64.sve.sqxtunt
  51268, // llvm.aarch64.sve.srhadd
  51292, // llvm.aarch64.sve.sri
  51313, // llvm.aarch64.sve.srshl
  51336, // llvm.aarch64.sve.srshl.single.x2
  51369, // llvm.aarch64.sve.srshl.single.x4
  51402, // llvm.aarch64.sve.srshl.u
  51427, // llvm.aarch64.sve.srshl.x2
  51453, // llvm.aarch64.sve.srshl.x4
  51479, // llvm.aarch64.sve.srshr
  51502, // llvm.aarch64.sve.srsra
  51525, // llvm.aarch64.sve.sshllb
  51549, // llvm.aarch64.sve.sshllt
  51573, // llvm.aarch64.sve.ssra
  51595, // llvm.aarch64.sve.ssublb
  51619, // llvm.aarch64.sve.ssublbt
  51644, // llvm.aarch64.sve.ssublt
  51668, // llvm.aarch64.sve.ssubltb
  51693, // llvm.aarch64.sve.ssubwb
  51717, // llvm.aarch64.sve.ssubwt
  51741, // llvm.aarch64.sve.st1
  51762, // llvm.aarch64.sve.st1.pn.x2
  51789, // llvm.aarch64.sve.st1.pn.x4
  51816, // llvm.aarch64.sve.st1.scatter
  51845, // llvm.aarch64.sve.st1.scatter.index
  51880, // llvm.aarch64.sve.st1.scatter.scalar.offset
  51923, // llvm.aarch64.sve.st1.scatter.sxtw
  51957, // llvm.aarch64.sve.st1.scatter.sxtw.index
  51997, // llvm.aarch64.sve.st1.scatter.uxtw
  52031, // llvm.aarch64.sve.st1.scatter.uxtw.index
  52071, // llvm.aarch64.sve.st1dq
  52094, // llvm.aarch64.sve.st1q.scatter.index
  52130, // llvm.aarch64.sve.st1q.scatter.scalar.offset
  52174, // llvm.aarch64.sve.st1q.scatter.vector.offset
  52218, // llvm.aarch64.sve.st1wq
  52241, // llvm.aarch64.sve.st2
  52262, // llvm.aarch64.sve.st2q
  52284, // llvm.aarch64.sve.st3
  52305, // llvm.aarch64.sve.st3q
  52327, // llvm.aarch64.sve.st4
  52348, // llvm.aarch64.sve.st4q
  52370, // llvm.aarch64.sve.stnt1
  52393, // llvm.aarch64.sve.stnt1.pn.x2
  52422, // llvm.aarch64.sve.stnt1.pn.x4
  52451, // llvm.aarch64.sve.stnt1.scatter
  52482, // llvm.aarch64.sve.stnt1.scatter.index
  52519, // llvm.aarch64.sve.stnt1.scatter.scalar.offset
  52564, // llvm.aarch64.sve.stnt1.scatter.uxtw
  52600, // llvm.aarch64.sve.sub
  52621, // llvm.aarch64.sve.sub.u
  52644, // llvm.aarch64.sve.subhnb
  52668, // llvm.aarch64.sve.subhnt
  52692, // llvm.aarch64.sve.subr
  52714, // llvm.aarch64.sve.sudot.lane
  52742, // llvm.aarch64.sve.sunpk.x2
  52768, // llvm.aarch64.sve.sunpk.x4
  52794, // llvm.aarch64.sve.sunpkhi
  52819, // llvm.aarch64.sve.sunpklo
  52844, // llvm.aarch64.sve.suqadd
  52868, // llvm.aarch64.sve.sxtb
  52890, // llvm.aarch64.sve.sxth
  52912, // llvm.aarch64.sve.sxtw
  52934, // llvm.aarch64.sve.tbl
  52955, // llvm.aarch64.sve.tbl2
  52977, // llvm.aarch64.sve.tblq
  52999, // llvm.aarch64.sve.tbx
  53020, // llvm.aarch64.sve.tbxq
  53042, // llvm.aarch64.sve.trn1
  53064, // llvm.aarch64.sve.trn1.b16
  53090, // llvm.aarch64.sve.trn1.b32
  53116, // llvm.aarch64.sve.trn1.b64
  53142, // llvm.aarch64.sve.trn1q
  53165, // llvm.aarch64.sve.trn2
  53187, // llvm.aarch64.sve.trn2.b16
  53213, // llvm.aarch64.sve.trn2.b32
  53239, // llvm.aarch64.sve.trn2.b64
  53265, // llvm.aarch64.sve.trn2q
  53288, // llvm.aarch64.sve.uaba
  53310, // llvm.aarch64.sve.uabalb
  53334, // llvm.aarch64.sve.uabalt
  53358, // llvm.aarch64.sve.uabd
  53380, // llvm.aarch64.sve.uabd.u
  53404, // llvm.aarch64.sve.uabdlb
  53428, // llvm.aarch64.sve.uabdlt
  53452, // llvm.aarch64.sve.uadalp
  53476, // llvm.aarch64.sve.uaddlb
  53500, // llvm.aarch64.sve.uaddlt
  53524, // llvm.aarch64.sve.uaddv
  53547, // llvm.aarch64.sve.uaddwb
  53571, // llvm.aarch64.sve.uaddwt
  53595, // llvm.aarch64.sve.uclamp
  53619, // llvm.aarch64.sve.uclamp.single.x2
  53653, // llvm.aarch64.sve.uclamp.single.x4
  53687, // llvm.aarch64.sve.ucvtf
  53710, // llvm.aarch64.sve.ucvtf.f16i32
  53740, // llvm.aarch64.sve.ucvtf.f16i64
  53770, // llvm.aarch64.sve.ucvtf.f32i64
  53800, // llvm.aarch64.sve.ucvtf.f64i32
  53830, // llvm.aarch64.sve.ucvtf.x2
  53856, // llvm.aarch64.sve.ucvtf.x4
  53882, // llvm.aarch64.sve.udiv
  53904, // llvm.aarch64.sve.udiv.u
  53928, // llvm.aarch64.sve.udivr
  53951, // llvm.aarch64.sve.udot
  53973, // llvm.aarch64.sve.udot.lane
  54000, // llvm.aarch64.sve.udot.lane.x2
  54030, // llvm.aarch64.sve.udot.x2
  54055, // llvm.aarch64.sve.uhadd
  54078, // llvm.aarch64.sve.uhsub
  54101, // llvm.aarch64.sve.uhsub.u
  54126, // llvm.aarch64.sve.uhsubr
  54150, // llvm.aarch64.sve.umax
  54172, // llvm.aarch64.sve.umax.single.x2
  54204, // llvm.aarch64.sve.umax.single.x4
  54236, // llvm.aarch64.sve.umax.u
  54260, // llvm.aarch64.sve.umax.x2
  54285, // llvm.aarch64.sve.umax.x4
  54310, // llvm.aarch64.sve.umaxp
  54333, // llvm.aarch64.sve.umaxqv
  54357, // llvm.aarch64.sve.umaxv
  54380, // llvm.aarch64.sve.umin
  54402, // llvm.aarch64.sve.umin.single.x2
  54434, // llvm.aarch64.sve.umin.single.x4
  54466, // llvm.aarch64.sve.umin.u
  54490, // llvm.aarch64.sve.umin.x2
  54515, // llvm.aarch64.sve.umin.x4
  54540, // llvm.aarch64.sve.uminp
  54563, // llvm.aarch64.sve.uminqv
  54587, // llvm.aarch64.sve.uminv
  54610, // llvm.aarch64.sve.umlalb
  54634, // llvm.aarch64.sve.umlalb.lane
  54663, // llvm.aarch64.sve.umlalt
  54687, // llvm.aarch64.sve.umlalt.lane
  54716, // llvm.aarch64.sve.umlslb
  54740, // llvm.aarch64.sve.umlslb.lane
  54769, // llvm.aarch64.sve.umlslt
  54793, // llvm.aarch64.sve.umlslt.lane
  54822, // llvm.aarch64.sve.ummla
  54845, // llvm.aarch64.sve.umulh
  54868, // llvm.aarch64.sve.umulh.u
  54893, // llvm.aarch64.sve.umullb
  54917, // llvm.aarch64.sve.umullb.lane
  54946, // llvm.aarch64.sve.umullt
  54970, // llvm.aarch64.sve.umullt.lane
  54999, // llvm.aarch64.sve.uqadd
  55022, // llvm.aarch64.sve.uqadd.x
  55047, // llvm.aarch64.sve.uqcvt.x2
  55073, // llvm.aarch64.sve.uqcvt.x4
  55099, // llvm.aarch64.sve.uqcvtn.x2
  55126, // llvm.aarch64.sve.uqcvtn.x4
  55153, // llvm.aarch64.sve.uqdecb.n32
  55181, // llvm.aarch64.sve.uqdecb.n64
  55209, // llvm.aarch64.sve.uqdecd
  55233, // llvm.aarch64.sve.uqdecd.n32
  55261, // llvm.aarch64.sve.uqdecd.n64
  55289, // llvm.aarch64.sve.uqdech
  55313, // llvm.aarch64.sve.uqdech.n32
  55341, // llvm.aarch64.sve.uqdech.n64
  55369, // llvm.aarch64.sve.uqdecp
  55393, // llvm.aarch64.sve.uqdecp.n32
  55421, // llvm.aarch64.sve.uqdecp.n64
  55449, // llvm.aarch64.sve.uqdecw
  55473, // llvm.aarch64.sve.uqdecw.n32
  55501, // llvm.aarch64.sve.uqdecw.n64
  55529, // llvm.aarch64.sve.uqincb.n32
  55557, // llvm.aarch64.sve.uqincb.n64
  55585, // llvm.aarch64.sve.uqincd
  55609, // llvm.aarch64.sve.uqincd.n32
  55637, // llvm.aarch64.sve.uqincd.n64
  55665, // llvm.aarch64.sve.uqinch
  55689, // llvm.aarch64.sve.uqinch.n32
  55717, // llvm.aarch64.sve.uqinch.n64
  55745, // llvm.aarch64.sve.uqincp
  55769, // llvm.aarch64.sve.uqincp.n32
  55797, // llvm.aarch64.sve.uqincp.n64
  55825, // llvm.aarch64.sve.uqincw
  55849, // llvm.aarch64.sve.uqincw.n32
  55877, // llvm.aarch64.sve.uqincw.n64
  55905, // llvm.aarch64.sve.uqrshl
  55929, // llvm.aarch64.sve.uqrshl.u
  55955, // llvm.aarch64.sve.uqrshr.x2
  55982, // llvm.aarch64.sve.uqrshr.x4
  56009, // llvm.aarch64.sve.uqrshrn.x2
  56037, // llvm.aarch64.sve.uqrshrn.x4
  56065, // llvm.aarch64.sve.uqrshrnb
  56091, // llvm.aarch64.sve.uqrshrnt
  56117, // llvm.aarch64.sve.uqshl
  56140, // llvm.aarch64.sve.uqshl.u
  56165, // llvm.aarch64.sve.uqshrnb
  56190, // llvm.aarch64.sve.uqshrnt
  56215, // llvm.aarch64.sve.uqsub
  56238, // llvm.aarch64.sve.uqsub.u
  56263, // llvm.aarch64.sve.uqsub.x
  56288, // llvm.aarch64.sve.uqsubr
  56312, // llvm.aarch64.sve.uqxtnb
  56336, // llvm.aarch64.sve.uqxtnt
  56360, // llvm.aarch64.sve.urecpe
  56384, // llvm.aarch64.sve.urhadd
  56408, // llvm.aarch64.sve.urshl
  56431, // llvm.aarch64.sve.urshl.single.x2
  56464, // llvm.aarch64.sve.urshl.single.x4
  56497, // llvm.aarch64.sve.urshl.u
  56522, // llvm.aarch64.sve.urshl.x2
  56548, // llvm.aarch64.sve.urshl.x4
  56574, // llvm.aarch64.sve.urshr
  56597, // llvm.aarch64.sve.ursqrte
  56622, // llvm.aarch64.sve.ursra
  56645, // llvm.aarch64.sve.usdot
  56668, // llvm.aarch64.sve.usdot.lane
  56696, // llvm.aarch64.sve.ushllb
  56720, // llvm.aarch64.sve.ushllt
  56744, // llvm.aarch64.sve.usmmla
  56768, // llvm.aarch64.sve.usqadd
  56792, // llvm.aarch64.sve.usra
  56814, // llvm.aarch64.sve.usublb
  56838, // llvm.aarch64.sve.usublt
  56862, // llvm.aarch64.sve.usubwb
  56886, // llvm.aarch64.sve.usubwt
  56910, // llvm.aarch64.sve.uunpk.x2
  56936, // llvm.aarch64.sve.uunpk.x4
  56962, // llvm.aarch64.sve.uunpkhi
  56987, // llvm.aarch64.sve.uunpklo
  57012, // llvm.aarch64.sve.uxtb
  57034, // llvm.aarch64.sve.uxth
  57056, // llvm.aarch64.sve.uxtw
  57078, // llvm.aarch64.sve.uzp.x2
  57102, // llvm.aarch64.sve.uzp.x4
  57126, // llvm.aarch64.sve.uzp1
  57148, // llvm.aarch64.sve.uzp1.b16
  57174, // llvm.aarch64.sve.uzp1.b32
  57200, // llvm.aarch64.sve.uzp1.b64
  57226, // llvm.aarch64.sve.uzp1q
  57249, // llvm.aarch64.sve.uzp2
  57271, // llvm.aarch64.sve.uzp2.b16
  57297, // llvm.aarch64.sve.uzp2.b32
  57323, // llvm.aarch64.sve.uzp2.b64
  57349, // llvm.aarch64.sve.uzp2q
  57372, // llvm.aarch64.sve.uzpq.x2
  57397, // llvm.aarch64.sve.uzpq.x4
  57422, // llvm.aarch64.sve.uzpq1
  57445, // llvm.aarch64.sve.uzpq2
  57468, // llvm.aarch64.sve.whilege
  57493, // llvm.aarch64.sve.whilege.c16
  57522, // llvm.aarch64.sve.whilege.c32
  57551, // llvm.aarch64.sve.whilege.c64
  57580, // llvm.aarch64.sve.whilege.c8
  57608, // llvm.aarch64.sve.whilege.x2
  57636, // llvm.aarch64.sve.whilegt
  57661, // llvm.aarch64.sve.whilegt.c16
  57690, // llvm.aarch64.sve.whilegt.c32
  57719, // llvm.aarch64.sve.whilegt.c64
  57748, // llvm.aarch64.sve.whilegt.c8
  57776, // llvm.aarch64.sve.whilegt.x2
  57804, // llvm.aarch64.sve.whilehi
  57829, // llvm.aarch64.sve.whilehi.c16
  57858, // llvm.aarch64.sve.whilehi.c32
  57887, // llvm.aarch64.sve.whilehi.c64
  57916, // llvm.aarch64.sve.whilehi.c8
  57944, // llvm.aarch64.sve.whilehi.x2
  57972, // llvm.aarch64.sve.whilehs
  57997, // llvm.aarch64.sve.whilehs.c16
  58026, // llvm.aarch64.sve.whilehs.c32
  58055, // llvm.aarch64.sve.whilehs.c64
  58084, // llvm.aarch64.sve.whilehs.c8
  58112, // llvm.aarch64.sve.whilehs.x2
  58140, // llvm.aarch64.sve.whilele
  58165, // llvm.aarch64.sve.whilele.c16
  58194, // llvm.aarch64.sve.whilele.c32
  58223, // llvm.aarch64.sve.whilele.c64
  58252, // llvm.aarch64.sve.whilele.c8
  58280, // llvm.aarch64.sve.whilele.x2
  58308, // llvm.aarch64.sve.whilelo
  58333, // llvm.aarch64.sve.whilelo.c16
  58362, // llvm.aarch64.sve.whilelo.c32
  58391, // llvm.aarch64.sve.whilelo.c64
  58420, // llvm.aarch64.sve.whilelo.c8
  58448, // llvm.aarch64.sve.whilelo.x2
  58476, // llvm.aarch64.sve.whilels
  58501, // llvm.aarch64.sve.whilels.c16
  58530, // llvm.aarch64.sve.whilels.c32
  58559, // llvm.aarch64.sve.whilels.c64
  58588, // llvm.aarch64.sve.whilels.c8
  58616, // llvm.aarch64.sve.whilels.x2
  58644, // llvm.aarch64.sve.whilelt
  58669, // llvm.aarch64.sve.whilelt.c16
  58698, // llvm.aarch64.sve.whilelt.c32
  58727, // llvm.aarch64.sve.whilelt.c64
  58756, // llvm.aarch64.sve.whilelt.c8
  58784, // llvm.aarch64.sve.whilelt.x2
  58812, // llvm.aarch64.sve.whilerw.b
  58839, // llvm.aarch64.sve.whilerw.d
  58866, // llvm.aarch64.sve.whilerw.h
  58893, // llvm.aarch64.sve.whilerw.s
  58920, // llvm.aarch64.sve.whilewr.b
  58947, // llvm.aarch64.sve.whilewr.d
  58974, // llvm.aarch64.sve.whilewr.h
  59001, // llvm.aarch64.sve.whilewr.s
  59028, // llvm.aarch64.sve.wrffr
  59051, // llvm.aarch64.sve.xar
  59072, // llvm.aarch64.sve.zip.x2
  59096, // llvm.aarch64.sve.zip.x4
  59120, // llvm.aarch64.sve.zip1
  59142, // llvm.aarch64.sve.zip1.b16
  59168, // llvm.aarch64.sve.zip1.b32
  59194, // llvm.aarch64.sve.zip1.b64
  59220, // llvm.aarch64.sve.zip1q
  59243, // llvm.aarch64.sve.zip2
  59265, // llvm.aarch64.sve.zip2.b16
  59291, // llvm.aarch64.sve.zip2.b32
  59317, // llvm.aarch64.sve.zip2.b64
  59343, // llvm.aarch64.sve.zip2q
  59366, // llvm.aarch64.sve.zipq.x2
  59391, // llvm.aarch64.sve.zipq.x4
  59416, // llvm.aarch64.sve.zipq1
  59439, // llvm.aarch64.sve.zipq2
  59462, // llvm.aarch64.tagp
  59480, // llvm.aarch64.udiv
  59498, // llvm.amdgcn.add.max.i32
  59522, // llvm.amdgcn.add.max.u32
  59546, // llvm.amdgcn.add.min.i32
  59570, // llvm.amdgcn.add.min.u32
  59594, // llvm.amdgcn.addrspacecast.nonnull
  59628, // llvm.amdgcn.alignbyte
  59650, // llvm.amdgcn.ashr.pk.i8.i32
  59677, // llvm.amdgcn.ashr.pk.u8.i32
  59704, // llvm.amdgcn.ballot
  59723, // llvm.amdgcn.bitop3
  59742, // llvm.amdgcn.buffer.wbinvl1
  59769, // llvm.amdgcn.buffer.wbinvl1.sc
  59799, // llvm.amdgcn.buffer.wbinvl1.vol
  59830, // llvm.amdgcn.call.whole.wave
  59858, // llvm.amdgcn.class
  59876, // llvm.amdgcn.cluster.id.x
  59901, // llvm.amdgcn.cluster.id.y
  59926, // llvm.amdgcn.cluster.id.z
  59951, // llvm.amdgcn.cluster.load.async.to.lds.b128
  59994, // llvm.amdgcn.cluster.load.async.to.lds.b32
  60036, // llvm.amdgcn.cluster.load.async.to.lds.b64
  60078, // llvm.amdgcn.cluster.load.async.to.lds.b8
  60119, // llvm.amdgcn.cluster.load.b128
  60149, // llvm.amdgcn.cluster.load.b32
  60178, // llvm.amdgcn.cluster.load.b64
  60207, // llvm.amdgcn.cluster.workgroup.flat.id
  60245, // llvm.amdgcn.cluster.workgroup.id.x
  60280, // llvm.amdgcn.cluster.workgroup.id.y
  60315, // llvm.amdgcn.cluster.workgroup.id.z
  60350, // llvm.amdgcn.cluster.workgroup.max.flat.id
  60392, // llvm.amdgcn.cluster.workgroup.max.id.x
  60431, // llvm.amdgcn.cluster.workgroup.max.id.y
  60470, // llvm.amdgcn.cluster.workgroup.max.id.z
  60509, // llvm.amdgcn.cooperative.atomic.load.16x8B
  60551, // llvm.amdgcn.cooperative.atomic.load.32x4B
  60593, // llvm.amdgcn.cooperative.atomic.load.8x16B
  60635, // llvm.amdgcn.cooperative.atomic.store.16x8B
  60678, // llvm.amdgcn.cooperative.atomic.store.32x4B
  60721, // llvm.amdgcn.cooperative.atomic.store.8x16B
  60764, // llvm.amdgcn.cos
  60780, // llvm.amdgcn.cs.chain
  60801, // llvm.amdgcn.cubeid
  60820, // llvm.amdgcn.cubema
  60839, // llvm.amdgcn.cubesc
  60858, // llvm.amdgcn.cubetc
  60877, // llvm.amdgcn.cvt.f16.bf8
  60901, // llvm.amdgcn.cvt.f16.fp8
  60925, // llvm.amdgcn.cvt.f32.bf8
  60949, // llvm.amdgcn.cvt.f32.fp8
  60973, // llvm.amdgcn.cvt.f32.fp8.e5m3
  61002, // llvm.amdgcn.cvt.off.f32.i4
  61029, // llvm.amdgcn.cvt.pk.bf8.f16
  61056, // llvm.amdgcn.cvt.pk.bf8.f32
  61083, // llvm.amdgcn.cvt.pk.f16.bf8
  61110, // llvm.amdgcn.cvt.pk.f16.fp8
  61137, // llvm.amdgcn.cvt.pk.f32.bf8
  61164, // llvm.amdgcn.cvt.pk.f32.fp8
  61191, // llvm.amdgcn.cvt.pk.fp8.f16
  61218, // llvm.amdgcn.cvt.pk.fp8.f32
  61245, // llvm.amdgcn.cvt.pk.fp8.f32.e5m3
  61277, // llvm.amdgcn.cvt.pk.i16
  61300, // llvm.amdgcn.cvt.pk.u16
  61323, // llvm.amdgcn.cvt.pk.u8.f32
  61349, // llvm.amdgcn.cvt.pknorm.i16
  61376, // llvm.amdgcn.cvt.pknorm.u16
  61403, // llvm.amdgcn.cvt.pkrtz
  61425, // llvm.amdgcn.cvt.scale.pk16.bf16.bf6
  61461, // llvm.amdgcn.cvt.scale.pk16.bf16.fp6
  61497, // llvm.amdgcn.cvt.scale.pk16.f16.bf6
  61532, // llvm.amdgcn.cvt.scale.pk16.f16.fp6
  61567, // llvm.amdgcn.cvt.scale.pk16.f32.bf6
  61602, // llvm.amdgcn.cvt.scale.pk16.f32.fp6
  61637, // llvm.amdgcn.cvt.scale.pk8.bf16.bf8
  61672, // llvm.amdgcn.cvt.scale.pk8.bf16.fp4
  61707, // llvm.amdgcn.cvt.scale.pk8.bf16.fp8
  61742, // llvm.amdgcn.cvt.scale.pk8.f16.bf8
  61776, // llvm.amdgcn.cvt.scale.pk8.f16.fp4
  61810, // llvm.amdgcn.cvt.scale.pk8.f16.fp8
  61844, // llvm.amdgcn.cvt.scale.pk8.f32.bf8
  61878, // llvm.amdgcn.cvt.scale.pk8.f32.fp4
  61912, // llvm.amdgcn.cvt.scale.pk8.f32.fp8
  61946, // llvm.amdgcn.cvt.scalef32.2xpk16.bf6.f32
  61986, // llvm.amdgcn.cvt.scalef32.2xpk16.fp6.f32
  62026, // llvm.amdgcn.cvt.scalef32.f16.bf8
  62059, // llvm.amdgcn.cvt.scalef32.f16.fp8
  62092, // llvm.amdgcn.cvt.scalef32.f32.bf8
  62125, // llvm.amdgcn.cvt.scalef32.f32.fp8
  62158, // llvm.amdgcn.cvt.scalef32.pk.bf16.bf8
  62195, // llvm.amdgcn.cvt.scalef32.pk.bf16.fp4
  62232, // llvm.amdgcn.cvt.scalef32.pk.bf16.fp8
  62269, // llvm.amdgcn.cvt.scalef32.pk.bf8.bf16
  62306, // llvm.amdgcn.cvt.scalef32.pk.bf8.f16
  62342, // llvm.amdgcn.cvt.scalef32.pk.bf8.f32
  62378, // llvm.amdgcn.cvt.scalef32.pk.f16.bf8
  62414, // llvm.amdgcn.cvt.scalef32.pk.f16.fp4
  62450, // llvm.amdgcn.cvt.scalef32.pk.f16.fp8
  62486, // llvm.amdgcn.cvt.scalef32.pk.f32.bf8
  62522, // llvm.amdgcn.cvt.scalef32.pk.f32.fp4
  62558, // llvm.amdgcn.cvt.scalef32.pk.f32.fp8
  62594, // llvm.amdgcn.cvt.scalef32.pk.fp4.bf16
  62631, // llvm.amdgcn.cvt.scalef32.pk.fp4.f16
  62667, // llvm.amdgcn.cvt.scalef32.pk.fp4.f32
  62703, // llvm.amdgcn.cvt.scalef32.pk.fp8.bf16
  62740, // llvm.amdgcn.cvt.scalef32.pk.fp8.f16
  62776, // llvm.amdgcn.cvt.scalef32.pk.fp8.f32
  62812, // llvm.amdgcn.cvt.scalef32.pk16.bf6.bf16
  62851, // llvm.amdgcn.cvt.scalef32.pk16.bf6.f16
  62889, // llvm.amdgcn.cvt.scalef32.pk16.bf6.f32
  62927, // llvm.amdgcn.cvt.scalef32.pk16.fp6.bf16
  62966, // llvm.amdgcn.cvt.scalef32.pk16.fp6.f16
  63004, // llvm.amdgcn.cvt.scalef32.pk16.fp6.f32
  63042, // llvm.amdgcn.cvt.scalef32.pk32.bf16.bf6
  63081, // llvm.amdgcn.cvt.scalef32.pk32.bf16.fp6
  63120, // llvm.amdgcn.cvt.scalef32.pk32.bf6.bf16
  63159, // llvm.amdgcn.cvt.scalef32.pk32.bf6.f16
  63197, // llvm.amdgcn.cvt.scalef32.pk32.f16.bf6
  63235, // llvm.amdgcn.cvt.scalef32.pk32.f16.fp6
  63273, // llvm.amdgcn.cvt.scalef32.pk32.f32.bf6
  63311, // llvm.amdgcn.cvt.scalef32.pk32.f32.fp6
  63349, // llvm.amdgcn.cvt.scalef32.pk32.fp6.bf16
  63388, // llvm.amdgcn.cvt.scalef32.pk32.fp6.f16
  63426, // llvm.amdgcn.cvt.scalef32.pk8.bf8.bf16
  63464, // llvm.amdgcn.cvt.scalef32.pk8.bf8.f16
  63501, // llvm.amdgcn.cvt.scalef32.pk8.bf8.f32
  63538, // llvm.amdgcn.cvt.scalef32.pk8.fp4.bf16
  63576, // llvm.amdgcn.cvt.scalef32.pk8.fp4.f16
  63613, // llvm.amdgcn.cvt.scalef32.pk8.fp4.f32
  63650, // llvm.amdgcn.cvt.scalef32.pk8.fp8.bf16
  63688, // llvm.amdgcn.cvt.scalef32.pk8.fp8.f16
  63725, // llvm.amdgcn.cvt.scalef32.pk8.fp8.f32
  63762, // llvm.amdgcn.cvt.scalef32.sr.bf8.bf16
  63799, // llvm.amdgcn.cvt.scalef32.sr.bf8.f16
  63835, // llvm.amdgcn.cvt.scalef32.sr.bf8.f32
  63871, // llvm.amdgcn.cvt.scalef32.sr.fp8.bf16
  63908, // llvm.amdgcn.cvt.scalef32.sr.fp8.f16
  63944, // llvm.amdgcn.cvt.scalef32.sr.fp8.f32
  63980, // llvm.amdgcn.cvt.scalef32.sr.pk.fp4.bf16
  64020, // llvm.amdgcn.cvt.scalef32.sr.pk.fp4.f16
  64059, // llvm.amdgcn.cvt.scalef32.sr.pk.fp4.f32
  64098, // llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.bf16
  64140, // llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.f16
  64181, // llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.f32
  64222, // llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.bf16
  64264, // llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.f16
  64305, // llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.f32
  64346, // llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.bf16
  64388, // llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.f16
  64429, // llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.f32
  64470, // llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.bf16
  64512, // llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.f16
  64553, // llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.f32
  64594, // llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.bf16
  64635, // llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.f16
  64675, // llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.f32
  64715, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.bf16
  64756, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.f16
  64796, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.f32
  64836, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.bf16
  64877, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.f16
  64917, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.f32
  64957, // llvm.amdgcn.cvt.sr.bf16.f32
  64985, // llvm.amdgcn.cvt.sr.bf8.f16
  65012, // llvm.amdgcn.cvt.sr.bf8.f32
  65039, // llvm.amdgcn.cvt.sr.f16.f32
  65066, // llvm.amdgcn.cvt.sr.fp8.f16
  65093, // llvm.amdgcn.cvt.sr.fp8.f32
  65120, // llvm.amdgcn.cvt.sr.fp8.f32.e5m3
  65152, // llvm.amdgcn.cvt.sr.pk.bf16.f32
  65183, // llvm.amdgcn.cvt.sr.pk.f16.f32
  65213, // llvm.amdgcn.dead
  65230, // llvm.amdgcn.dispatch.id
  65254, // llvm.amdgcn.dispatch.ptr
  65279, // llvm.amdgcn.div.fixup
  65301, // llvm.amdgcn.div.fmas
  65322, // llvm.amdgcn.div.scale
  65344, // llvm.amdgcn.dot4.f32.bf8.bf8
  65373, // llvm.amdgcn.dot4.f32.bf8.fp8
  65402, // llvm.amdgcn.dot4.f32.fp8.bf8
  65431, // llvm.amdgcn.dot4.f32.fp8.fp8
  65460, // llvm.amdgcn.ds.add.gs.reg.rtn
  65490, // llvm.amdgcn.ds.append
  65512, // llvm.amdgcn.ds.atomic.async.barrier.arrive.b64
  65559, // llvm.amdgcn.ds.atomic.barrier.arrive.rtn.b64
  65604, // llvm.amdgcn.ds.bpermute
  65628, // llvm.amdgcn.ds.bpermute.fi.b32
  65659, // llvm.amdgcn.ds.bvh.stack.push4.pop1.rtn
  65699, // llvm.amdgcn.ds.bvh.stack.push8.pop1.rtn
  65739, // llvm.amdgcn.ds.bvh.stack.push8.pop2.rtn
  65779, // llvm.amdgcn.ds.bvh.stack.rtn
  65808, // llvm.amdgcn.ds.consume
  65831, // llvm.amdgcn.ds.gws.barrier
  65858, // llvm.amdgcn.ds.gws.init
  65882, // llvm.amdgcn.ds.gws.sema.br
  65909, // llvm.amdgcn.ds.gws.sema.p
  65935, // llvm.amdgcn.ds.gws.sema.release.all
  65971, // llvm.amdgcn.ds.gws.sema.v
  65997, // llvm.amdgcn.ds.load.tr16.b128
  66027, // llvm.amdgcn.ds.load.tr4.b64
  66055, // llvm.amdgcn.ds.load.tr6.b96
  66083, // llvm.amdgcn.ds.load.tr8.b64
  66111, // llvm.amdgcn.ds.ordered.add
  66138, // llvm.amdgcn.ds.ordered.swap
  66166, // llvm.amdgcn.ds.permute
  66189, // llvm.amdgcn.ds.read.tr16.b64
  66218, // llvm.amdgcn.ds.read.tr4.b64
  66246, // llvm.amdgcn.ds.read.tr6.b96
  66274, // llvm.amdgcn.ds.read.tr8.b64
  66302, // llvm.amdgcn.ds.sub.gs.reg.rtn
  66332, // llvm.amdgcn.ds.swizzle
  66355, // llvm.amdgcn.else
  66372, // llvm.amdgcn.end.cf
  66391, // llvm.amdgcn.endpgm
  66410, // llvm.amdgcn.exp
  66426, // llvm.amdgcn.exp.compr
  66448, // llvm.amdgcn.exp.row
  66468, // llvm.amdgcn.exp2
  66485, // llvm.amdgcn.fcmp
  66502, // llvm.amdgcn.fdiv.fast
  66524, // llvm.amdgcn.fdot2
  66542, // llvm.amdgcn.fdot2.bf16.bf16
  66570, // llvm.amdgcn.fdot2.f16.f16
  66596, // llvm.amdgcn.fdot2.f32.bf16
  66623, // llvm.amdgcn.fdot2c.f32.bf16
  66651, // llvm.amdgcn.flat.atomic.fmax.num
  66684, // llvm.amdgcn.flat.atomic.fmin.num
  66717, // llvm.amdgcn.flat.load.monitor.b128
  66752, // llvm.amdgcn.flat.load.monitor.b32
  66786, // llvm.amdgcn.flat.load.monitor.b64
  66820, // llvm.amdgcn.flat.prefetch
  66846, // llvm.amdgcn.fma.legacy
  66869, // llvm.amdgcn.fmad.ftz
  66890, // llvm.amdgcn.fmed3
  66908, // llvm.amdgcn.fmul.legacy
  66932, // llvm.amdgcn.fract
  66950, // llvm.amdgcn.frexp.exp
  66972, // llvm.amdgcn.frexp.mant
  66995, // llvm.amdgcn.global.atomic.fmax.num
  67030, // llvm.amdgcn.global.atomic.fmin.num
  67065, // llvm.amdgcn.global.atomic.ordered.add.b64
  67107, // llvm.amdgcn.global.load.async.to.lds.b128
  67149, // llvm.amdgcn.global.load.async.to.lds.b32
  67190, // llvm.amdgcn.global.load.async.to.lds.b64
  67231, // llvm.amdgcn.global.load.async.to.lds.b8
  67271, // llvm.amdgcn.global.load.lds
  67299, // llvm.amdgcn.global.load.monitor.b128
  67336, // llvm.amdgcn.global.load.monitor.b32
  67372, // llvm.amdgcn.global.load.monitor.b64
  67408, // llvm.amdgcn.global.load.tr.b128
  67440, // llvm.amdgcn.global.load.tr.b64
  67471, // llvm.amdgcn.global.load.tr4.b64
  67503, // llvm.amdgcn.global.load.tr6.b96
  67535, // llvm.amdgcn.global.prefetch
  67563, // llvm.amdgcn.global.store.async.from.lds.b128
  67608, // llvm.amdgcn.global.store.async.from.lds.b32
  67652, // llvm.amdgcn.global.store.async.from.lds.b64
  67696, // llvm.amdgcn.global.store.async.from.lds.b8
  67739, // llvm.amdgcn.groupstaticsize
  67767, // llvm.amdgcn.icmp
  67784, // llvm.amdgcn.if
  67799, // llvm.amdgcn.if.break
  67820, // llvm.amdgcn.iglp.opt
  67841, // llvm.amdgcn.image.atomic.add.1d
  67873, // llvm.amdgcn.image.atomic.add.1darray
  67910, // llvm.amdgcn.image.atomic.add.2d
  67942, // llvm.amdgcn.image.atomic.add.2darray
  67979, // llvm.amdgcn.image.atomic.add.2darraymsaa
  68020, // llvm.amdgcn.image.atomic.add.2dmsaa
  68056, // llvm.amdgcn.image.atomic.add.3d
  68088, // llvm.amdgcn.image.atomic.add.cube
  68122, // llvm.amdgcn.image.atomic.add.flt.1d
  68158, // llvm.amdgcn.image.atomic.add.flt.1darray
  68199, // llvm.amdgcn.image.atomic.add.flt.2d
  68235, // llvm.amdgcn.image.atomic.add.flt.2darray
  68276, // llvm.amdgcn.image.atomic.add.flt.2darraymsaa
  68321, // llvm.amdgcn.image.atomic.add.flt.2dmsaa
  68361, // llvm.amdgcn.image.atomic.add.flt.3d
  68397, // llvm.amdgcn.image.atomic.add.flt.cube
  68435, // llvm.amdgcn.image.atomic.and.1d
  68467, // llvm.amdgcn.image.atomic.and.1darray
  68504, // llvm.amdgcn.image.atomic.and.2d
  68536, // llvm.amdgcn.image.atomic.and.2darray
  68573, // llvm.amdgcn.image.atomic.and.2darraymsaa
  68614, // llvm.amdgcn.image.atomic.and.2dmsaa
  68650, // llvm.amdgcn.image.atomic.and.3d
  68682, // llvm.amdgcn.image.atomic.and.cube
  68716, // llvm.amdgcn.image.atomic.cmpswap.1d
  68752, // llvm.amdgcn.image.atomic.cmpswap.1darray
  68793, // llvm.amdgcn.image.atomic.cmpswap.2d
  68829, // llvm.amdgcn.image.atomic.cmpswap.2darray
  68870, // llvm.amdgcn.image.atomic.cmpswap.2darraymsaa
  68915, // llvm.amdgcn.image.atomic.cmpswap.2dmsaa
  68955, // llvm.amdgcn.image.atomic.cmpswap.3d
  68991, // llvm.amdgcn.image.atomic.cmpswap.cube
  69029, // llvm.amdgcn.image.atomic.dec.1d
  69061, // llvm.amdgcn.image.atomic.dec.1darray
  69098, // llvm.amdgcn.image.atomic.dec.2d
  69130, // llvm.amdgcn.image.atomic.dec.2darray
  69167, // llvm.amdgcn.image.atomic.dec.2darraymsaa
  69208, // llvm.amdgcn.image.atomic.dec.2dmsaa
  69244, // llvm.amdgcn.image.atomic.dec.3d
  69276, // llvm.amdgcn.image.atomic.dec.cube
  69310, // llvm.amdgcn.image.atomic.fmax.1d
  69343, // llvm.amdgcn.image.atomic.fmax.1darray
  69381, // llvm.amdgcn.image.atomic.fmax.2d
  69414, // llvm.amdgcn.image.atomic.fmax.2darray
  69452, // llvm.amdgcn.image.atomic.fmax.2darraymsaa
  69494, // llvm.amdgcn.image.atomic.fmax.2dmsaa
  69531, // llvm.amdgcn.image.atomic.fmax.3d
  69564, // llvm.amdgcn.image.atomic.fmax.cube
  69599, // llvm.amdgcn.image.atomic.fmin.1d
  69632, // llvm.amdgcn.image.atomic.fmin.1darray
  69670, // llvm.amdgcn.image.atomic.fmin.2d
  69703, // llvm.amdgcn.image.atomic.fmin.2darray
  69741, // llvm.amdgcn.image.atomic.fmin.2darraymsaa
  69783, // llvm.amdgcn.image.atomic.fmin.2dmsaa
  69820, // llvm.amdgcn.image.atomic.fmin.3d
  69853, // llvm.amdgcn.image.atomic.fmin.cube
  69888, // llvm.amdgcn.image.atomic.inc.1d
  69920, // llvm.amdgcn.image.atomic.inc.1darray
  69957, // llvm.amdgcn.image.atomic.inc.2d
  69989, // llvm.amdgcn.image.atomic.inc.2darray
  70026, // llvm.amdgcn.image.atomic.inc.2darraymsaa
  70067, // llvm.amdgcn.image.atomic.inc.2dmsaa
  70103, // llvm.amdgcn.image.atomic.inc.3d
  70135, // llvm.amdgcn.image.atomic.inc.cube
  70169, // llvm.amdgcn.image.atomic.max.flt.1d
  70205, // llvm.amdgcn.image.atomic.max.flt.1darray
  70246, // llvm.amdgcn.image.atomic.max.flt.2d
  70282, // llvm.amdgcn.image.atomic.max.flt.2darray
  70323, // llvm.amdgcn.image.atomic.max.flt.2darraymsaa
  70368, // llvm.amdgcn.image.atomic.max.flt.2dmsaa
  70408, // llvm.amdgcn.image.atomic.max.flt.3d
  70444, // llvm.amdgcn.image.atomic.max.flt.cube
  70482, // llvm.amdgcn.image.atomic.min.flt.1d
  70518, // llvm.amdgcn.image.atomic.min.flt.1darray
  70559, // llvm.amdgcn.image.atomic.min.flt.2d
  70595, // llvm.amdgcn.image.atomic.min.flt.2darray
  70636, // llvm.amdgcn.image.atomic.min.flt.2darraymsaa
  70681, // llvm.amdgcn.image.atomic.min.flt.2dmsaa
  70721, // llvm.amdgcn.image.atomic.min.flt.3d
  70757, // llvm.amdgcn.image.atomic.min.flt.cube
  70795, // llvm.amdgcn.image.atomic.or.1d
  70826, // llvm.amdgcn.image.atomic.or.1darray
  70862, // llvm.amdgcn.image.atomic.or.2d
  70893, // llvm.amdgcn.image.atomic.or.2darray
  70929, // llvm.amdgcn.image.atomic.or.2darraymsaa
  70969, // llvm.amdgcn.image.atomic.or.2dmsaa
  71004, // llvm.amdgcn.image.atomic.or.3d
  71035, // llvm.amdgcn.image.atomic.or.cube
  71068, // llvm.amdgcn.image.atomic.pk.add.bf16.1d
  71108, // llvm.amdgcn.image.atomic.pk.add.bf16.1darray
  71153, // llvm.amdgcn.image.atomic.pk.add.bf16.2d
  71193, // llvm.amdgcn.image.atomic.pk.add.bf16.2darray
  71238, // llvm.amdgcn.image.atomic.pk.add.bf16.2darraymsaa
  71287, // llvm.amdgcn.image.atomic.pk.add.bf16.2dmsaa
  71331, // llvm.amdgcn.image.atomic.pk.add.bf16.3d
  71371, // llvm.amdgcn.image.atomic.pk.add.bf16.cube
  71413, // llvm.amdgcn.image.atomic.pk.add.f16.1d
  71452, // llvm.amdgcn.image.atomic.pk.add.f16.1darray
  71496, // llvm.amdgcn.image.atomic.pk.add.f16.2d
  71535, // llvm.amdgcn.image.atomic.pk.add.f16.2darray
  71579, // llvm.amdgcn.image.atomic.pk.add.f16.2darraymsaa
  71627, // llvm.amdgcn.image.atomic.pk.add.f16.2dmsaa
  71670, // llvm.amdgcn.image.atomic.pk.add.f16.3d
  71709, // llvm.amdgcn.image.atomic.pk.add.f16.cube
  71750, // llvm.amdgcn.image.atomic.smax.1d
  71783, // llvm.amdgcn.image.atomic.smax.1darray
  71821, // llvm.amdgcn.image.atomic.smax.2d
  71854, // llvm.amdgcn.image.atomic.smax.2darray
  71892, // llvm.amdgcn.image.atomic.smax.2darraymsaa
  71934, // llvm.amdgcn.image.atomic.smax.2dmsaa
  71971, // llvm.amdgcn.image.atomic.smax.3d
  72004, // llvm.amdgcn.image.atomic.smax.cube
  72039, // llvm.amdgcn.image.atomic.smin.1d
  72072, // llvm.amdgcn.image.atomic.smin.1darray
  72110, // llvm.amdgcn.image.atomic.smin.2d
  72143, // llvm.amdgcn.image.atomic.smin.2darray
  72181, // llvm.amdgcn.image.atomic.smin.2darraymsaa
  72223, // llvm.amdgcn.image.atomic.smin.2dmsaa
  72260, // llvm.amdgcn.image.atomic.smin.3d
  72293, // llvm.amdgcn.image.atomic.smin.cube
  72328, // llvm.amdgcn.image.atomic.sub.1d
  72360, // llvm.amdgcn.image.atomic.sub.1darray
  72397, // llvm.amdgcn.image.atomic.sub.2d
  72429, // llvm.amdgcn.image.atomic.sub.2darray
  72466, // llvm.amdgcn.image.atomic.sub.2darraymsaa
  72507, // llvm.amdgcn.image.atomic.sub.2dmsaa
  72543, // llvm.amdgcn.image.atomic.sub.3d
  72575, // llvm.amdgcn.image.atomic.sub.cube
  72609, // llvm.amdgcn.image.atomic.swap.1d
  72642, // llvm.amdgcn.image.atomic.swap.1darray
  72680, // llvm.amdgcn.image.atomic.swap.2d
  72713, // llvm.amdgcn.image.atomic.swap.2darray
  72751, // llvm.amdgcn.image.atomic.swap.2darraymsaa
  72793, // llvm.amdgcn.image.atomic.swap.2dmsaa
  72830, // llvm.amdgcn.image.atomic.swap.3d
  72863, // llvm.amdgcn.image.atomic.swap.cube
  72898, // llvm.amdgcn.image.atomic.umax.1d
  72931, // llvm.amdgcn.image.atomic.umax.1darray
  72969, // llvm.amdgcn.image.atomic.umax.2d
  73002, // llvm.amdgcn.image.atomic.umax.2darray
  73040, // llvm.amdgcn.image.atomic.umax.2darraymsaa
  73082, // llvm.amdgcn.image.atomic.umax.2dmsaa
  73119, // llvm.amdgcn.image.atomic.umax.3d
  73152, // llvm.amdgcn.image.atomic.umax.cube
  73187, // llvm.amdgcn.image.atomic.umin.1d
  73220, // llvm.amdgcn.image.atomic.umin.1darray
  73258, // llvm.amdgcn.image.atomic.umin.2d
  73291, // llvm.amdgcn.image.atomic.umin.2darray
  73329, // llvm.amdgcn.image.atomic.umin.2darraymsaa
  73371, // llvm.amdgcn.image.atomic.umin.2dmsaa
  73408, // llvm.amdgcn.image.atomic.umin.3d
  73441, // llvm.amdgcn.image.atomic.umin.cube
  73476, // llvm.amdgcn.image.atomic.xor.1d
  73508, // llvm.amdgcn.image.atomic.xor.1darray
  73545, // llvm.amdgcn.image.atomic.xor.2d
  73577, // llvm.amdgcn.image.atomic.xor.2darray
  73614, // llvm.amdgcn.image.atomic.xor.2darraymsaa
  73655, // llvm.amdgcn.image.atomic.xor.2dmsaa
  73691, // llvm.amdgcn.image.atomic.xor.3d
  73723, // llvm.amdgcn.image.atomic.xor.cube
  73757, // llvm.amdgcn.image.bvh.dual.intersect.ray
  73798, // llvm.amdgcn.image.bvh.intersect.ray
  73834, // llvm.amdgcn.image.bvh8.intersect.ray
  73871, // llvm.amdgcn.image.gather4.2d
  73900, // llvm.amdgcn.image.gather4.2darray
  73934, // llvm.amdgcn.image.gather4.b.2d
  73965, // llvm.amdgcn.image.gather4.b.2darray
  74001, // llvm.amdgcn.image.gather4.b.cl.2d
  74035, // llvm.amdgcn.image.gather4.b.cl.2darray
  74074, // llvm.amdgcn.image.gather4.b.cl.cube
  74110, // llvm.amdgcn.image.gather4.b.cl.o.2d
  74146, // llvm.amdgcn.image.gather4.b.cl.o.2darray
  74187, // llvm.amdgcn.image.gather4.b.cl.o.cube
  74225, // llvm.amdgcn.image.gather4.b.cube
  74258, // llvm.amdgcn.image.gather4.b.o.2d
  74291, // llvm.amdgcn.image.gather4.b.o.2darray
  74329, // llvm.amdgcn.image.gather4.b.o.cube
  74364, // llvm.amdgcn.image.gather4.c.2d
  74395, // llvm.amdgcn.image.gather4.c.2darray
  74431, // llvm.amdgcn.image.gather4.c.b.2d
  74464, // llvm.amdgcn.image.gather4.c.b.2darray
  74502, // llvm.amdgcn.image.gather4.c.b.cl.2d
  74538, // llvm.amdgcn.image.gather4.c.b.cl.2darray
  74579, // llvm.amdgcn.image.gather4.c.b.cl.cube
  74617, // llvm.amdgcn.image.gather4.c.b.cl.o.2d
  74655, // llvm.amdgcn.image.gather4.c.b.cl.o.2darray
  74698, // llvm.amdgcn.image.gather4.c.b.cl.o.cube
  74738, // llvm.amdgcn.image.gather4.c.b.cube
  74773, // llvm.amdgcn.image.gather4.c.b.o.2d
  74808, // llvm.amdgcn.image.gather4.c.b.o.2darray
  74848, // llvm.amdgcn.image.gather4.c.b.o.cube
  74885, // llvm.amdgcn.image.gather4.c.cl.2d
  74919, // llvm.amdgcn.image.gather4.c.cl.2darray
  74958, // llvm.amdgcn.image.gather4.c.cl.cube
  74994, // llvm.amdgcn.image.gather4.c.cl.o.2d
  75030, // llvm.amdgcn.image.gather4.c.cl.o.2darray
  75071, // llvm.amdgcn.image.gather4.c.cl.o.cube
  75109, // llvm.amdgcn.image.gather4.c.cube
  75142, // llvm.amdgcn.image.gather4.c.l.2d
  75175, // llvm.amdgcn.image.gather4.c.l.2darray
  75213, // llvm.amdgcn.image.gather4.c.l.cube
  75248, // llvm.amdgcn.image.gather4.c.l.o.2d
  75283, // llvm.amdgcn.image.gather4.c.l.o.2darray
  75323, // llvm.amdgcn.image.gather4.c.l.o.cube
  75360, // llvm.amdgcn.image.gather4.c.lz.2d
  75394, // llvm.amdgcn.image.gather4.c.lz.2darray
  75433, // llvm.amdgcn.image.gather4.c.lz.cube
  75469, // llvm.amdgcn.image.gather4.c.lz.o.2d
  75505, // llvm.amdgcn.image.gather4.c.lz.o.2darray
  75546, // llvm.amdgcn.image.gather4.c.lz.o.cube
  75584, // llvm.amdgcn.image.gather4.c.o.2d
  75617, // llvm.amdgcn.image.gather4.c.o.2darray
  75655, // llvm.amdgcn.image.gather4.c.o.cube
  75690, // llvm.amdgcn.image.gather4.cl.2d
  75722, // llvm.amdgcn.image.gather4.cl.2darray
  75759, // llvm.amdgcn.image.gather4.cl.cube
  75793, // llvm.amdgcn.image.gather4.cl.o.2d
  75827, // llvm.amdgcn.image.gather4.cl.o.2darray
  75866, // llvm.amdgcn.image.gather4.cl.o.cube
  75902, // llvm.amdgcn.image.gather4.cube
  75933, // llvm.amdgcn.image.gather4.l.2d
  75964, // llvm.amdgcn.image.gather4.l.2darray
  76000, // llvm.amdgcn.image.gather4.l.cube
  76033, // llvm.amdgcn.image.gather4.l.o.2d
  76066, // llvm.amdgcn.image.gather4.l.o.2darray
  76104, // llvm.amdgcn.image.gather4.l.o.cube
  76139, // llvm.amdgcn.image.gather4.lz.2d
  76171, // llvm.amdgcn.image.gather4.lz.2darray
  76208, // llvm.amdgcn.image.gather4.lz.cube
  76242, // llvm.amdgcn.image.gather4.lz.o.2d
  76276, // llvm.amdgcn.image.gather4.lz.o.2darray
  76315, // llvm.amdgcn.image.gather4.lz.o.cube
  76351, // llvm.amdgcn.image.gather4.o.2d
  76382, // llvm.amdgcn.image.gather4.o.2darray
  76418, // llvm.amdgcn.image.gather4.o.cube
  76451, // llvm.amdgcn.image.getlod.1d
  76479, // llvm.amdgcn.image.getlod.1darray
  76512, // llvm.amdgcn.image.getlod.2d
  76540, // llvm.amdgcn.image.getlod.2darray
  76573, // llvm.amdgcn.image.getlod.3d
  76601, // llvm.amdgcn.image.getlod.cube
  76631, // llvm.amdgcn.image.getresinfo.1d
  76663, // llvm.amdgcn.image.getresinfo.1darray
  76700, // llvm.amdgcn.image.getresinfo.2d
  76732, // llvm.amdgcn.image.getresinfo.2darray
  76769, // llvm.amdgcn.image.getresinfo.2darraymsaa
  76810, // llvm.amdgcn.image.getresinfo.2dmsaa
  76846, // llvm.amdgcn.image.getresinfo.3d
  76878, // llvm.amdgcn.image.getresinfo.cube
  76912, // llvm.amdgcn.image.load.1d
  76938, // llvm.amdgcn.image.load.1darray
  76969, // llvm.amdgcn.image.load.2d
  76995, // llvm.amdgcn.image.load.2darray
  77026, // llvm.amdgcn.image.load.2darraymsaa
  77061, // llvm.amdgcn.image.load.2dmsaa
  77091, // llvm.amdgcn.image.load.3d
  77117, // llvm.amdgcn.image.load.cube
  77145, // llvm.amdgcn.image.load.mip.1d
  77175, // llvm.amdgcn.image.load.mip.1darray
  77210, // llvm.amdgcn.image.load.mip.2d
  77240, // llvm.amdgcn.image.load.mip.2darray
  77275, // llvm.amdgcn.image.load.mip.3d
  77305, // llvm.amdgcn.image.load.mip.cube
  77337, // llvm.amdgcn.image.msaa.load.2darraymsaa
  77377, // llvm.amdgcn.image.msaa.load.2dmsaa
  77412, // llvm.amdgcn.image.msaa.load.x.2darraymsaa
  77454, // llvm.amdgcn.image.msaa.load.x.2dmsaa
  77491, // llvm.amdgcn.image.sample.1d
  77519, // llvm.amdgcn.image.sample.1d.nortn
  77553, // llvm.amdgcn.image.sample.1darray
  77586, // llvm.amdgcn.image.sample.1darray.nortn
  77625, // llvm.amdgcn.image.sample.2d
  77653, // llvm.amdgcn.image.sample.2d.nortn
  77687, // llvm.amdgcn.image.sample.2darray
  77720, // llvm.amdgcn.image.sample.2darray.nortn
  77759, // llvm.amdgcn.image.sample.3d
  77787, // llvm.amdgcn.image.sample.3d.nortn
  77821, // llvm.amdgcn.image.sample.b.1d
  77851, // llvm.amdgcn.image.sample.b.1d.nortn
  77887, // llvm.amdgcn.image.sample.b.1darray
  77922, // llvm.amdgcn.image.sample.b.1darray.nortn
  77963, // llvm.amdgcn.image.sample.b.2d
  77993, // llvm.amdgcn.image.sample.b.2d.nortn
  78029, // llvm.amdgcn.image.sample.b.2darray
  78064, // llvm.amdgcn.image.sample.b.2darray.nortn
  78105, // llvm.amdgcn.image.sample.b.3d
  78135, // llvm.amdgcn.image.sample.b.3d.nortn
  78171, // llvm.amdgcn.image.sample.b.cl.1d
  78204, // llvm.amdgcn.image.sample.b.cl.1d.nortn
  78243, // llvm.amdgcn.image.sample.b.cl.1darray
  78281, // llvm.amdgcn.image.sample.b.cl.1darray.nortn
  78325, // llvm.amdgcn.image.sample.b.cl.2d
  78358, // llvm.amdgcn.image.sample.b.cl.2d.nortn
  78397, // llvm.amdgcn.image.sample.b.cl.2darray
  78435, // llvm.amdgcn.image.sample.b.cl.2darray.nortn
  78479, // llvm.amdgcn.image.sample.b.cl.3d
  78512, // llvm.amdgcn.image.sample.b.cl.3d.nortn
  78551, // llvm.amdgcn.image.sample.b.cl.cube
  78586, // llvm.amdgcn.image.sample.b.cl.cube.nortn
  78627, // llvm.amdgcn.image.sample.b.cl.o.1d
  78662, // llvm.amdgcn.image.sample.b.cl.o.1d.nortn
  78703, // llvm.amdgcn.image.sample.b.cl.o.1darray
  78743, // llvm.amdgcn.image.sample.b.cl.o.1darray.nortn
  78789, // llvm.amdgcn.image.sample.b.cl.o.2d
  78824, // llvm.amdgcn.image.sample.b.cl.o.2d.nortn
  78865, // llvm.amdgcn.image.sample.b.cl.o.2darray
  78905, // llvm.amdgcn.image.sample.b.cl.o.2darray.nortn
  78951, // llvm.amdgcn.image.sample.b.cl.o.3d
  78986, // llvm.amdgcn.image.sample.b.cl.o.3d.nortn
  79027, // llvm.amdgcn.image.sample.b.cl.o.cube
  79064, // llvm.amdgcn.image.sample.b.cl.o.cube.nortn
  79107, // llvm.amdgcn.image.sample.b.cube
  79139, // llvm.amdgcn.image.sample.b.cube.nortn
  79177, // llvm.amdgcn.image.sample.b.o.1d
  79209, // llvm.amdgcn.image.sample.b.o.1d.nortn
  79247, // llvm.amdgcn.image.sample.b.o.1darray
  79284, // llvm.amdgcn.image.sample.b.o.1darray.nortn
  79327, // llvm.amdgcn.image.sample.b.o.2d
  79359, // llvm.amdgcn.image.sample.b.o.2d.nortn
  79397, // llvm.amdgcn.image.sample.b.o.2darray
  79434, // llvm.amdgcn.image.sample.b.o.2darray.nortn
  79477, // llvm.amdgcn.image.sample.b.o.3d
  79509, // llvm.amdgcn.image.sample.b.o.3d.nortn
  79547, // llvm.amdgcn.image.sample.b.o.cube
  79581, // llvm.amdgcn.image.sample.b.o.cube.nortn
  79621, // llvm.amdgcn.image.sample.c.1d
  79651, // llvm.amdgcn.image.sample.c.1d.nortn
  79687, // llvm.amdgcn.image.sample.c.1darray
  79722, // llvm.amdgcn.image.sample.c.1darray.nortn
  79763, // llvm.amdgcn.image.sample.c.2d
  79793, // llvm.amdgcn.image.sample.c.2d.nortn
  79829, // llvm.amdgcn.image.sample.c.2darray
  79864, // llvm.amdgcn.image.sample.c.2darray.nortn
  79905, // llvm.amdgcn.image.sample.c.3d
  79935, // llvm.amdgcn.image.sample.c.3d.nortn
  79971, // llvm.amdgcn.image.sample.c.b.1d
  80003, // llvm.amdgcn.image.sample.c.b.1d.nortn
  80041, // llvm.amdgcn.image.sample.c.b.1darray
  80078, // llvm.amdgcn.image.sample.c.b.1darray.nortn
  80121, // llvm.amdgcn.image.sample.c.b.2d
  80153, // llvm.amdgcn.image.sample.c.b.2d.nortn
  80191, // llvm.amdgcn.image.sample.c.b.2darray
  80228, // llvm.amdgcn.image.sample.c.b.2darray.nortn
  80271, // llvm.amdgcn.image.sample.c.b.3d
  80303, // llvm.amdgcn.image.sample.c.b.3d.nortn
  80341, // llvm.amdgcn.image.sample.c.b.cl.1d
  80376, // llvm.amdgcn.image.sample.c.b.cl.1d.nortn
  80417, // llvm.amdgcn.image.sample.c.b.cl.1darray
  80457, // llvm.amdgcn.image.sample.c.b.cl.1darray.nortn
  80503, // llvm.amdgcn.image.sample.c.b.cl.2d
  80538, // llvm.amdgcn.image.sample.c.b.cl.2d.nortn
  80579, // llvm.amdgcn.image.sample.c.b.cl.2darray
  80619, // llvm.amdgcn.image.sample.c.b.cl.2darray.nortn
  80665, // llvm.amdgcn.image.sample.c.b.cl.3d
  80700, // llvm.amdgcn.image.sample.c.b.cl.3d.nortn
  80741, // llvm.amdgcn.image.sample.c.b.cl.cube
  80778, // llvm.amdgcn.image.sample.c.b.cl.cube.nortn
  80821, // llvm.amdgcn.image.sample.c.b.cl.o.1d
  80858, // llvm.amdgcn.image.sample.c.b.cl.o.1d.nortn
  80901, // llvm.amdgcn.image.sample.c.b.cl.o.1darray
  80943, // llvm.amdgcn.image.sample.c.b.cl.o.1darray.nortn
  80991, // llvm.amdgcn.image.sample.c.b.cl.o.2d
  81028, // llvm.amdgcn.image.sample.c.b.cl.o.2d.nortn
  81071, // llvm.amdgcn.image.sample.c.b.cl.o.2darray
  81113, // llvm.amdgcn.image.sample.c.b.cl.o.2darray.nortn
  81161, // llvm.amdgcn.image.sample.c.b.cl.o.3d
  81198, // llvm.amdgcn.image.sample.c.b.cl.o.3d.nortn
  81241, // llvm.amdgcn.image.sample.c.b.cl.o.cube
  81280, // llvm.amdgcn.image.sample.c.b.cl.o.cube.nortn
  81325, // llvm.amdgcn.image.sample.c.b.cube
  81359, // llvm.amdgcn.image.sample.c.b.cube.nortn
  81399, // llvm.amdgcn.image.sample.c.b.o.1d
  81433, // llvm.amdgcn.image.sample.c.b.o.1d.nortn
  81473, // llvm.amdgcn.image.sample.c.b.o.1darray
  81512, // llvm.amdgcn.image.sample.c.b.o.1darray.nortn
  81557, // llvm.amdgcn.image.sample.c.b.o.2d
  81591, // llvm.amdgcn.image.sample.c.b.o.2d.nortn
  81631, // llvm.amdgcn.image.sample.c.b.o.2darray
  81670, // llvm.amdgcn.image.sample.c.b.o.2darray.nortn
  81715, // llvm.amdgcn.image.sample.c.b.o.3d
  81749, // llvm.amdgcn.image.sample.c.b.o.3d.nortn
  81789, // llvm.amdgcn.image.sample.c.b.o.cube
  81825, // llvm.amdgcn.image.sample.c.b.o.cube.nortn
  81867, // llvm.amdgcn.image.sample.c.cd.1d
  81900, // llvm.amdgcn.image.sample.c.cd.1d.nortn
  81939, // llvm.amdgcn.image.sample.c.cd.1darray
  81977, // llvm.amdgcn.image.sample.c.cd.1darray.nortn
  82021, // llvm.amdgcn.image.sample.c.cd.2d
  82054, // llvm.amdgcn.image.sample.c.cd.2d.nortn
  82093, // llvm.amdgcn.image.sample.c.cd.2darray
  82131, // llvm.amdgcn.image.sample.c.cd.2darray.nortn
  82175, // llvm.amdgcn.image.sample.c.cd.3d
  82208, // llvm.amdgcn.image.sample.c.cd.3d.nortn
  82247, // llvm.amdgcn.image.sample.c.cd.cl.1d
  82283, // llvm.amdgcn.image.sample.c.cd.cl.1d.nortn
  82325, // llvm.amdgcn.image.sample.c.cd.cl.1darray
  82366, // llvm.amdgcn.image.sample.c.cd.cl.1darray.nortn
  82413, // llvm.amdgcn.image.sample.c.cd.cl.2d
  82449, // llvm.amdgcn.image.sample.c.cd.cl.2d.nortn
  82491, // llvm.amdgcn.image.sample.c.cd.cl.2darray
  82532, // llvm.amdgcn.image.sample.c.cd.cl.2darray.nortn
  82579, // llvm.amdgcn.image.sample.c.cd.cl.3d
  82615, // llvm.amdgcn.image.sample.c.cd.cl.3d.nortn
  82657, // llvm.amdgcn.image.sample.c.cd.cl.cube
  82695, // llvm.amdgcn.image.sample.c.cd.cl.cube.nortn
  82739, // llvm.amdgcn.image.sample.c.cd.cl.o.1d
  82777, // llvm.amdgcn.image.sample.c.cd.cl.o.1d.nortn
  82821, // llvm.amdgcn.image.sample.c.cd.cl.o.1darray
  82864, // llvm.amdgcn.image.sample.c.cd.cl.o.1darray.nortn
  82913, // llvm.amdgcn.image.sample.c.cd.cl.o.2d
  82951, // llvm.amdgcn.image.sample.c.cd.cl.o.2d.nortn
  82995, // llvm.amdgcn.image.sample.c.cd.cl.o.2darray
  83038, // llvm.amdgcn.image.sample.c.cd.cl.o.2darray.nortn
  83087, // llvm.amdgcn.image.sample.c.cd.cl.o.3d
  83125, // llvm.amdgcn.image.sample.c.cd.cl.o.3d.nortn
  83169, // llvm.amdgcn.image.sample.c.cd.cl.o.cube
  83209, // llvm.amdgcn.image.sample.c.cd.cl.o.cube.nortn
  83255, // llvm.amdgcn.image.sample.c.cd.cube
  83290, // llvm.amdgcn.image.sample.c.cd.cube.nortn
  83331, // llvm.amdgcn.image.sample.c.cd.o.1d
  83366, // llvm.amdgcn.image.sample.c.cd.o.1d.nortn
  83407, // llvm.amdgcn.image.sample.c.cd.o.1darray
  83447, // llvm.amdgcn.image.sample.c.cd.o.1darray.nortn
  83493, // llvm.amdgcn.image.sample.c.cd.o.2d
  83528, // llvm.amdgcn.image.sample.c.cd.o.2d.nortn
  83569, // llvm.amdgcn.image.sample.c.cd.o.2darray
  83609, // llvm.amdgcn.image.sample.c.cd.o.2darray.nortn
  83655, // llvm.amdgcn.image.sample.c.cd.o.3d
  83690, // llvm.amdgcn.image.sample.c.cd.o.3d.nortn
  83731, // llvm.amdgcn.image.sample.c.cd.o.cube
  83768, // llvm.amdgcn.image.sample.c.cd.o.cube.nortn
  83811, // llvm.amdgcn.image.sample.c.cl.1d
  83844, // llvm.amdgcn.image.sample.c.cl.1d.nortn
  83883, // llvm.amdgcn.image.sample.c.cl.1darray
  83921, // llvm.amdgcn.image.sample.c.cl.1darray.nortn
  83965, // llvm.amdgcn.image.sample.c.cl.2d
  83998, // llvm.amdgcn.image.sample.c.cl.2d.nortn
  84037, // llvm.amdgcn.image.sample.c.cl.2darray
  84075, // llvm.amdgcn.image.sample.c.cl.2darray.nortn
  84119, // llvm.amdgcn.image.sample.c.cl.3d
  84152, // llvm.amdgcn.image.sample.c.cl.3d.nortn
  84191, // llvm.amdgcn.image.sample.c.cl.cube
  84226, // llvm.amdgcn.image.sample.c.cl.cube.nortn
  84267, // llvm.amdgcn.image.sample.c.cl.o.1d
  84302, // llvm.amdgcn.image.sample.c.cl.o.1d.nortn
  84343, // llvm.amdgcn.image.sample.c.cl.o.1darray
  84383, // llvm.amdgcn.image.sample.c.cl.o.1darray.nortn
  84429, // llvm.amdgcn.image.sample.c.cl.o.2d
  84464, // llvm.amdgcn.image.sample.c.cl.o.2d.nortn
  84505, // llvm.amdgcn.image.sample.c.cl.o.2darray
  84545, // llvm.amdgcn.image.sample.c.cl.o.2darray.nortn
  84591, // llvm.amdgcn.image.sample.c.cl.o.3d
  84626, // llvm.amdgcn.image.sample.c.cl.o.3d.nortn
  84667, // llvm.amdgcn.image.sample.c.cl.o.cube
  84704, // llvm.amdgcn.image.sample.c.cl.o.cube.nortn
  84747, // llvm.amdgcn.image.sample.c.cube
  84779, // llvm.amdgcn.image.sample.c.cube.nortn
  84817, // llvm.amdgcn.image.sample.c.d.1d
  84849, // llvm.amdgcn.image.sample.c.d.1d.nortn
  84887, // llvm.amdgcn.image.sample.c.d.1darray
  84924, // llvm.amdgcn.image.sample.c.d.1darray.nortn
  84967, // llvm.amdgcn.image.sample.c.d.2d
  84999, // llvm.amdgcn.image.sample.c.d.2d.nortn
  85037, // llvm.amdgcn.image.sample.c.d.2darray
  85074, // llvm.amdgcn.image.sample.c.d.2darray.nortn
  85117, // llvm.amdgcn.image.sample.c.d.3d
  85149, // llvm.amdgcn.image.sample.c.d.3d.nortn
  85187, // llvm.amdgcn.image.sample.c.d.cl.1d
  85222, // llvm.amdgcn.image.sample.c.d.cl.1d.nortn
  85263, // llvm.amdgcn.image.sample.c.d.cl.1darray
  85303, // llvm.amdgcn.image.sample.c.d.cl.1darray.nortn
  85349, // llvm.amdgcn.image.sample.c.d.cl.2d
  85384, // llvm.amdgcn.image.sample.c.d.cl.2d.nortn
  85425, // llvm.amdgcn.image.sample.c.d.cl.2darray
  85465, // llvm.amdgcn.image.sample.c.d.cl.2darray.nortn
  85511, // llvm.amdgcn.image.sample.c.d.cl.3d
  85546, // llvm.amdgcn.image.sample.c.d.cl.3d.nortn
  85587, // llvm.amdgcn.image.sample.c.d.cl.cube
  85624, // llvm.amdgcn.image.sample.c.d.cl.cube.nortn
  85667, // llvm.amdgcn.image.sample.c.d.cl.o.1d
  85704, // llvm.amdgcn.image.sample.c.d.cl.o.1d.nortn
  85747, // llvm.amdgcn.image.sample.c.d.cl.o.1darray
  85789, // llvm.amdgcn.image.sample.c.d.cl.o.1darray.nortn
  85837, // llvm.amdgcn.image.sample.c.d.cl.o.2d
  85874, // llvm.amdgcn.image.sample.c.d.cl.o.2d.nortn
  85917, // llvm.amdgcn.image.sample.c.d.cl.o.2darray
  85959, // llvm.amdgcn.image.sample.c.d.cl.o.2darray.nortn
  86007, // llvm.amdgcn.image.sample.c.d.cl.o.3d
  86044, // llvm.amdgcn.image.sample.c.d.cl.o.3d.nortn
  86087, // llvm.amdgcn.image.sample.c.d.cl.o.cube
  86126, // llvm.amdgcn.image.sample.c.d.cl.o.cube.nortn
  86171, // llvm.amdgcn.image.sample.c.d.cube
  86205, // llvm.amdgcn.image.sample.c.d.cube.nortn
  86245, // llvm.amdgcn.image.sample.c.d.o.1d
  86279, // llvm.amdgcn.image.sample.c.d.o.1d.nortn
  86319, // llvm.amdgcn.image.sample.c.d.o.1darray
  86358, // llvm.amdgcn.image.sample.c.d.o.1darray.nortn
  86403, // llvm.amdgcn.image.sample.c.d.o.2d
  86437, // llvm.amdgcn.image.sample.c.d.o.2d.nortn
  86477, // llvm.amdgcn.image.sample.c.d.o.2darray
  86516, // llvm.amdgcn.image.sample.c.d.o.2darray.nortn
  86561, // llvm.amdgcn.image.sample.c.d.o.3d
  86595, // llvm.amdgcn.image.sample.c.d.o.3d.nortn
  86635, // llvm.amdgcn.image.sample.c.d.o.cube
  86671, // llvm.amdgcn.image.sample.c.d.o.cube.nortn
  86713, // llvm.amdgcn.image.sample.c.l.1d
  86745, // llvm.amdgcn.image.sample.c.l.1d.nortn
  86783, // llvm.amdgcn.image.sample.c.l.1darray
  86820, // llvm.amdgcn.image.sample.c.l.1darray.nortn
  86863, // llvm.amdgcn.image.sample.c.l.2d
  86895, // llvm.amdgcn.image.sample.c.l.2d.nortn
  86933, // llvm.amdgcn.image.sample.c.l.2darray
  86970, // llvm.amdgcn.image.sample.c.l.2darray.nortn
  87013, // llvm.amdgcn.image.sample.c.l.3d
  87045, // llvm.amdgcn.image.sample.c.l.3d.nortn
  87083, // llvm.amdgcn.image.sample.c.l.cube
  87117, // llvm.amdgcn.image.sample.c.l.cube.nortn
  87157, // llvm.amdgcn.image.sample.c.l.o.1d
  87191, // llvm.amdgcn.image.sample.c.l.o.1d.nortn
  87231, // llvm.amdgcn.image.sample.c.l.o.1darray
  87270, // llvm.amdgcn.image.sample.c.l.o.1darray.nortn
  87315, // llvm.amdgcn.image.sample.c.l.o.2d
  87349, // llvm.amdgcn.image.sample.c.l.o.2d.nortn
  87389, // llvm.amdgcn.image.sample.c.l.o.2darray
  87428, // llvm.amdgcn.image.sample.c.l.o.2darray.nortn
  87473, // llvm.amdgcn.image.sample.c.l.o.3d
  87507, // llvm.amdgcn.image.sample.c.l.o.3d.nortn
  87547, // llvm.amdgcn.image.sample.c.l.o.cube
  87583, // llvm.amdgcn.image.sample.c.l.o.cube.nortn
  87625, // llvm.amdgcn.image.sample.c.lz.1d
  87658, // llvm.amdgcn.image.sample.c.lz.1d.nortn
  87697, // llvm.amdgcn.image.sample.c.lz.1darray
  87735, // llvm.amdgcn.image.sample.c.lz.1darray.nortn
  87779, // llvm.amdgcn.image.sample.c.lz.2d
  87812, // llvm.amdgcn.image.sample.c.lz.2d.nortn
  87851, // llvm.amdgcn.image.sample.c.lz.2darray
  87889, // llvm.amdgcn.image.sample.c.lz.2darray.nortn
  87933, // llvm.amdgcn.image.sample.c.lz.3d
  87966, // llvm.amdgcn.image.sample.c.lz.3d.nortn
  88005, // llvm.amdgcn.image.sample.c.lz.cube
  88040, // llvm.amdgcn.image.sample.c.lz.cube.nortn
  88081, // llvm.amdgcn.image.sample.c.lz.o.1d
  88116, // llvm.amdgcn.image.sample.c.lz.o.1d.nortn
  88157, // llvm.amdgcn.image.sample.c.lz.o.1darray
  88197, // llvm.amdgcn.image.sample.c.lz.o.1darray.nortn
  88243, // llvm.amdgcn.image.sample.c.lz.o.2d
  88278, // llvm.amdgcn.image.sample.c.lz.o.2d.nortn
  88319, // llvm.amdgcn.image.sample.c.lz.o.2darray
  88359, // llvm.amdgcn.image.sample.c.lz.o.2darray.nortn
  88405, // llvm.amdgcn.image.sample.c.lz.o.3d
  88440, // llvm.amdgcn.image.sample.c.lz.o.3d.nortn
  88481, // llvm.amdgcn.image.sample.c.lz.o.cube
  88518, // llvm.amdgcn.image.sample.c.lz.o.cube.nortn
  88561, // llvm.amdgcn.image.sample.c.o.1d
  88593, // llvm.amdgcn.image.sample.c.o.1d.nortn
  88631, // llvm.amdgcn.image.sample.c.o.1darray
  88668, // llvm.amdgcn.image.sample.c.o.1darray.nortn
  88711, // llvm.amdgcn.image.sample.c.o.2d
  88743, // llvm.amdgcn.image.sample.c.o.2d.nortn
  88781, // llvm.amdgcn.image.sample.c.o.2darray
  88818, // llvm.amdgcn.image.sample.c.o.2darray.nortn
  88861, // llvm.amdgcn.image.sample.c.o.3d
  88893, // llvm.amdgcn.image.sample.c.o.3d.nortn
  88931, // llvm.amdgcn.image.sample.c.o.cube
  88965, // llvm.amdgcn.image.sample.c.o.cube.nortn
  89005, // llvm.amdgcn.image.sample.cd.1d
  89036, // llvm.amdgcn.image.sample.cd.1d.nortn
  89073, // llvm.amdgcn.image.sample.cd.1darray
  89109, // llvm.amdgcn.image.sample.cd.1darray.nortn
  89151, // llvm.amdgcn.image.sample.cd.2d
  89182, // llvm.amdgcn.image.sample.cd.2d.nortn
  89219, // llvm.amdgcn.image.sample.cd.2darray
  89255, // llvm.amdgcn.image.sample.cd.2darray.nortn
  89297, // llvm.amdgcn.image.sample.cd.3d
  89328, // llvm.amdgcn.image.sample.cd.3d.nortn
  89365, // llvm.amdgcn.image.sample.cd.cl.1d
  89399, // llvm.amdgcn.image.sample.cd.cl.1d.nortn
  89439, // llvm.amdgcn.image.sample.cd.cl.1darray
  89478, // llvm.amdgcn.image.sample.cd.cl.1darray.nortn
  89523, // llvm.amdgcn.image.sample.cd.cl.2d
  89557, // llvm.amdgcn.image.sample.cd.cl.2d.nortn
  89597, // llvm.amdgcn.image.sample.cd.cl.2darray
  89636, // llvm.amdgcn.image.sample.cd.cl.2darray.nortn
  89681, // llvm.amdgcn.image.sample.cd.cl.3d
  89715, // llvm.amdgcn.image.sample.cd.cl.3d.nortn
  89755, // llvm.amdgcn.image.sample.cd.cl.cube
  89791, // llvm.amdgcn.image.sample.cd.cl.cube.nortn
  89833, // llvm.amdgcn.image.sample.cd.cl.o.1d
  89869, // llvm.amdgcn.image.sample.cd.cl.o.1d.nortn
  89911, // llvm.amdgcn.image.sample.cd.cl.o.1darray
  89952, // llvm.amdgcn.image.sample.cd.cl.o.1darray.nortn
  89999, // llvm.amdgcn.image.sample.cd.cl.o.2d
  90035, // llvm.amdgcn.image.sample.cd.cl.o.2d.nortn
  90077, // llvm.amdgcn.image.sample.cd.cl.o.2darray
  90118, // llvm.amdgcn.image.sample.cd.cl.o.2darray.nortn
  90165, // llvm.amdgcn.image.sample.cd.cl.o.3d
  90201, // llvm.amdgcn.image.sample.cd.cl.o.3d.nortn
  90243, // llvm.amdgcn.image.sample.cd.cl.o.cube
  90281, // llvm.amdgcn.image.sample.cd.cl.o.cube.nortn
  90325, // llvm.amdgcn.image.sample.cd.cube
  90358, // llvm.amdgcn.image.sample.cd.cube.nortn
  90397, // llvm.amdgcn.image.sample.cd.o.1d
  90430, // llvm.amdgcn.image.sample.cd.o.1d.nortn
  90469, // llvm.amdgcn.image.sample.cd.o.1darray
  90507, // llvm.amdgcn.image.sample.cd.o.1darray.nortn
  90551, // llvm.amdgcn.image.sample.cd.o.2d
  90584, // llvm.amdgcn.image.sample.cd.o.2d.nortn
  90623, // llvm.amdgcn.image.sample.cd.o.2darray
  90661, // llvm.amdgcn.image.sample.cd.o.2darray.nortn
  90705, // llvm.amdgcn.image.sample.cd.o.3d
  90738, // llvm.amdgcn.image.sample.cd.o.3d.nortn
  90777, // llvm.amdgcn.image.sample.cd.o.cube
  90812, // llvm.amdgcn.image.sample.cd.o.cube.nortn
  90853, // llvm.amdgcn.image.sample.cl.1d
  90884, // llvm.amdgcn.image.sample.cl.1d.nortn
  90921, // llvm.amdgcn.image.sample.cl.1darray
  90957, // llvm.amdgcn.image.sample.cl.1darray.nortn
  90999, // llvm.amdgcn.image.sample.cl.2d
  91030, // llvm.amdgcn.image.sample.cl.2d.nortn
  91067, // llvm.amdgcn.image.sample.cl.2darray
  91103, // llvm.amdgcn.image.sample.cl.2darray.nortn
  91145, // llvm.amdgcn.image.sample.cl.3d
  91176, // llvm.amdgcn.image.sample.cl.3d.nortn
  91213, // llvm.amdgcn.image.sample.cl.cube
  91246, // llvm.amdgcn.image.sample.cl.cube.nortn
  91285, // llvm.amdgcn.image.sample.cl.o.1d
  91318, // llvm.amdgcn.image.sample.cl.o.1d.nortn
  91357, // llvm.amdgcn.image.sample.cl.o.1darray
  91395, // llvm.amdgcn.image.sample.cl.o.1darray.nortn
  91439, // llvm.amdgcn.image.sample.cl.o.2d
  91472, // llvm.amdgcn.image.sample.cl.o.2d.nortn
  91511, // llvm.amdgcn.image.sample.cl.o.2darray
  91549, // llvm.amdgcn.image.sample.cl.o.2darray.nortn
  91593, // llvm.amdgcn.image.sample.cl.o.3d
  91626, // llvm.amdgcn.image.sample.cl.o.3d.nortn
  91665, // llvm.amdgcn.image.sample.cl.o.cube
  91700, // llvm.amdgcn.image.sample.cl.o.cube.nortn
  91741, // llvm.amdgcn.image.sample.cube
  91771, // llvm.amdgcn.image.sample.cube.nortn
  91807, // llvm.amdgcn.image.sample.d.1d
  91837, // llvm.amdgcn.image.sample.d.1d.nortn
  91873, // llvm.amdgcn.image.sample.d.1darray
  91908, // llvm.amdgcn.image.sample.d.1darray.nortn
  91949, // llvm.amdgcn.image.sample.d.2d
  91979, // llvm.amdgcn.image.sample.d.2d.nortn
  92015, // llvm.amdgcn.image.sample.d.2darray
  92050, // llvm.amdgcn.image.sample.d.2darray.nortn
  92091, // llvm.amdgcn.image.sample.d.3d
  92121, // llvm.amdgcn.image.sample.d.3d.nortn
  92157, // llvm.amdgcn.image.sample.d.cl.1d
  92190, // llvm.amdgcn.image.sample.d.cl.1d.nortn
  92229, // llvm.amdgcn.image.sample.d.cl.1darray
  92267, // llvm.amdgcn.image.sample.d.cl.1darray.nortn
  92311, // llvm.amdgcn.image.sample.d.cl.2d
  92344, // llvm.amdgcn.image.sample.d.cl.2d.nortn
  92383, // llvm.amdgcn.image.sample.d.cl.2darray
  92421, // llvm.amdgcn.image.sample.d.cl.2darray.nortn
  92465, // llvm.amdgcn.image.sample.d.cl.3d
  92498, // llvm.amdgcn.image.sample.d.cl.3d.nortn
  92537, // llvm.amdgcn.image.sample.d.cl.cube
  92572, // llvm.amdgcn.image.sample.d.cl.cube.nortn
  92613, // llvm.amdgcn.image.sample.d.cl.o.1d
  92648, // llvm.amdgcn.image.sample.d.cl.o.1d.nortn
  92689, // llvm.amdgcn.image.sample.d.cl.o.1darray
  92729, // llvm.amdgcn.image.sample.d.cl.o.1darray.nortn
  92775, // llvm.amdgcn.image.sample.d.cl.o.2d
  92810, // llvm.amdgcn.image.sample.d.cl.o.2d.nortn
  92851, // llvm.amdgcn.image.sample.d.cl.o.2darray
  92891, // llvm.amdgcn.image.sample.d.cl.o.2darray.nortn
  92937, // llvm.amdgcn.image.sample.d.cl.o.3d
  92972, // llvm.amdgcn.image.sample.d.cl.o.3d.nortn
  93013, // llvm.amdgcn.image.sample.d.cl.o.cube
  93050, // llvm.amdgcn.image.sample.d.cl.o.cube.nortn
  93093, // llvm.amdgcn.image.sample.d.cube
  93125, // llvm.amdgcn.image.sample.d.cube.nortn
  93163, // llvm.amdgcn.image.sample.d.o.1d
  93195, // llvm.amdgcn.image.sample.d.o.1d.nortn
  93233, // llvm.amdgcn.image.sample.d.o.1darray
  93270, // llvm.amdgcn.image.sample.d.o.1darray.nortn
  93313, // llvm.amdgcn.image.sample.d.o.2d
  93345, // llvm.amdgcn.image.sample.d.o.2d.nortn
  93383, // llvm.amdgcn.image.sample.d.o.2darray
  93420, // llvm.amdgcn.image.sample.d.o.2darray.nortn
  93463, // llvm.amdgcn.image.sample.d.o.3d
  93495, // llvm.amdgcn.image.sample.d.o.3d.nortn
  93533, // llvm.amdgcn.image.sample.d.o.cube
  93567, // llvm.amdgcn.image.sample.d.o.cube.nortn
  93607, // llvm.amdgcn.image.sample.l.1d
  93637, // llvm.amdgcn.image.sample.l.1d.nortn
  93673, // llvm.amdgcn.image.sample.l.1darray
  93708, // llvm.amdgcn.image.sample.l.1darray.nortn
  93749, // llvm.amdgcn.image.sample.l.2d
  93779, // llvm.amdgcn.image.sample.l.2d.nortn
  93815, // llvm.amdgcn.image.sample.l.2darray
  93850, // llvm.amdgcn.image.sample.l.2darray.nortn
  93891, // llvm.amdgcn.image.sample.l.3d
  93921, // llvm.amdgcn.image.sample.l.3d.nortn
  93957, // llvm.amdgcn.image.sample.l.cube
  93989, // llvm.amdgcn.image.sample.l.cube.nortn
  94027, // llvm.amdgcn.image.sample.l.o.1d
  94059, // llvm.amdgcn.image.sample.l.o.1d.nortn
  94097, // llvm.amdgcn.image.sample.l.o.1darray
  94134, // llvm.amdgcn.image.sample.l.o.1darray.nortn
  94177, // llvm.amdgcn.image.sample.l.o.2d
  94209, // llvm.amdgcn.image.sample.l.o.2d.nortn
  94247, // llvm.amdgcn.image.sample.l.o.2darray
  94284, // llvm.amdgcn.image.sample.l.o.2darray.nortn
  94327, // llvm.amdgcn.image.sample.l.o.3d
  94359, // llvm.amdgcn.image.sample.l.o.3d.nortn
  94397, // llvm.amdgcn.image.sample.l.o.cube
  94431, // llvm.amdgcn.image.sample.l.o.cube.nortn
  94471, // llvm.amdgcn.image.sample.lz.1d
  94502, // llvm.amdgcn.image.sample.lz.1d.nortn
  94539, // llvm.amdgcn.image.sample.lz.1darray
  94575, // llvm.amdgcn.image.sample.lz.1darray.nortn
  94617, // llvm.amdgcn.image.sample.lz.2d
  94648, // llvm.amdgcn.image.sample.lz.2d.nortn
  94685, // llvm.amdgcn.image.sample.lz.2darray
  94721, // llvm.amdgcn.image.sample.lz.2darray.nortn
  94763, // llvm.amdgcn.image.sample.lz.3d
  94794, // llvm.amdgcn.image.sample.lz.3d.nortn
  94831, // llvm.amdgcn.image.sample.lz.cube
  94864, // llvm.amdgcn.image.sample.lz.cube.nortn
  94903, // llvm.amdgcn.image.sample.lz.o.1d
  94936, // llvm.amdgcn.image.sample.lz.o.1d.nortn
  94975, // llvm.amdgcn.image.sample.lz.o.1darray
  95013, // llvm.amdgcn.image.sample.lz.o.1darray.nortn
  95057, // llvm.amdgcn.image.sample.lz.o.2d
  95090, // llvm.amdgcn.image.sample.lz.o.2d.nortn
  95129, // llvm.amdgcn.image.sample.lz.o.2darray
  95167, // llvm.amdgcn.image.sample.lz.o.2darray.nortn
  95211, // llvm.amdgcn.image.sample.lz.o.3d
  95244, // llvm.amdgcn.image.sample.lz.o.3d.nortn
  95283, // llvm.amdgcn.image.sample.lz.o.cube
  95318, // llvm.amdgcn.image.sample.lz.o.cube.nortn
  95359, // llvm.amdgcn.image.sample.o.1d
  95389, // llvm.amdgcn.image.sample.o.1d.nortn
  95425, // llvm.amdgcn.image.sample.o.1darray
  95460, // llvm.amdgcn.image.sample.o.1darray.nortn
  95501, // llvm.amdgcn.image.sample.o.2d
  95531, // llvm.amdgcn.image.sample.o.2d.nortn
  95567, // llvm.amdgcn.image.sample.o.2darray
  95602, // llvm.amdgcn.image.sample.o.2darray.nortn
  95643, // llvm.amdgcn.image.sample.o.3d
  95673, // llvm.amdgcn.image.sample.o.3d.nortn
  95709, // llvm.amdgcn.image.sample.o.cube
  95741, // llvm.amdgcn.image.sample.o.cube.nortn
  95779, // llvm.amdgcn.image.store.1d
  95806, // llvm.amdgcn.image.store.1darray
  95838, // llvm.amdgcn.image.store.2d
  95865, // llvm.amdgcn.image.store.2darray
  95897, // llvm.amdgcn.image.store.2darraymsaa
  95933, // llvm.amdgcn.image.store.2dmsaa
  95964, // llvm.amdgcn.image.store.3d
  95991, // llvm.amdgcn.image.store.cube
  96020, // llvm.amdgcn.image.store.mip.1d
  96051, // llvm.amdgcn.image.store.mip.1darray
  96087, // llvm.amdgcn.image.store.mip.2d
  96118, // llvm.amdgcn.image.store.mip.2darray
  96154, // llvm.amdgcn.image.store.mip.3d
  96185, // llvm.amdgcn.image.store.mip.cube
  96218, // llvm.amdgcn.implicit.buffer.ptr
  96250, // llvm.amdgcn.implicitarg.ptr
  96278, // llvm.amdgcn.init.exec
  96300, // llvm.amdgcn.init.exec.from.input
  96333, // llvm.amdgcn.init.whole.wave
  96361, // llvm.amdgcn.interp.inreg.p10
  96390, // llvm.amdgcn.interp.inreg.p10.f16
  96423, // llvm.amdgcn.interp.inreg.p2
  96451, // llvm.amdgcn.interp.inreg.p2.f16
  96483, // llvm.amdgcn.interp.mov
  96506, // llvm.amdgcn.interp.p1
  96528, // llvm.amdgcn.interp.p1.f16
  96554, // llvm.amdgcn.interp.p10.rtz.f16
  96585, // llvm.amdgcn.interp.p2
  96607, // llvm.amdgcn.interp.p2.f16
  96633, // llvm.amdgcn.interp.p2.rtz.f16
  96663, // llvm.amdgcn.inverse.ballot
  96690, // llvm.amdgcn.is.private
  96713, // llvm.amdgcn.is.shared
  96735, // llvm.amdgcn.kernarg.segment.ptr
  96767, // llvm.amdgcn.kill
  96784, // llvm.amdgcn.lds.direct.load
  96812, // llvm.amdgcn.lds.kernel.id
  96838, // llvm.amdgcn.lds.param.load
  96865, // llvm.amdgcn.lerp
  96882, // llvm.amdgcn.live.mask
  96904, // llvm.amdgcn.load.to.lds
  96928, // llvm.amdgcn.log
  96944, // llvm.amdgcn.log.clamp
  96966, // llvm.amdgcn.loop
  96983, // llvm.amdgcn.make.buffer.rsrc
  97012, // llvm.amdgcn.mbcnt.hi
  97033, // llvm.amdgcn.mbcnt.lo
  97054, // llvm.amdgcn.mfma.f32.16x16x16bf16.1k
  97091, // llvm.amdgcn.mfma.f32.16x16x16f16
  97124, // llvm.amdgcn.mfma.f32.16x16x1f32
  97156, // llvm.amdgcn.mfma.f32.16x16x2bf16
  97189, // llvm.amdgcn.mfma.f32.16x16x32.bf16
  97224, // llvm.amdgcn.mfma.f32.16x16x32.bf8.bf8
  97262, // llvm.amdgcn.mfma.f32.16x16x32.bf8.fp8
  97300, // llvm.amdgcn.mfma.f32.16x16x32.f16
  97334, // llvm.amdgcn.mfma.f32.16x16x32.fp8.bf8
  97372, // llvm.amdgcn.mfma.f32.16x16x32.fp8.fp8
  97410, // llvm.amdgcn.mfma.f32.16x16x4bf16.1k
  97446, // llvm.amdgcn.mfma.f32.16x16x4f16
  97478, // llvm.amdgcn.mfma.f32.16x16x4f32
  97510, // llvm.amdgcn.mfma.f32.16x16x8.xf32
  97544, // llvm.amdgcn.mfma.f32.16x16x8bf16
  97577, // llvm.amdgcn.mfma.f32.32x32x16.bf16
  97612, // llvm.amdgcn.mfma.f32.32x32x16.bf8.bf8
  97650, // llvm.amdgcn.mfma.f32.32x32x16.bf8.fp8
  97688, // llvm.amdgcn.mfma.f32.32x32x16.f16
  97722, // llvm.amdgcn.mfma.f32.32x32x16.fp8.bf8
  97760, // llvm.amdgcn.mfma.f32.32x32x16.fp8.fp8
  97798, // llvm.amdgcn.mfma.f32.32x32x1f32
  97830, // llvm.amdgcn.mfma.f32.32x32x2bf16
  97863, // llvm.amdgcn.mfma.f32.32x32x2f32
  97895, // llvm.amdgcn.mfma.f32.32x32x4.xf32
  97929, // llvm.amdgcn.mfma.f32.32x32x4bf16
  97962, // llvm.amdgcn.mfma.f32.32x32x4bf16.1k
  97998, // llvm.amdgcn.mfma.f32.32x32x4f16
  98030, // llvm.amdgcn.mfma.f32.32x32x8bf16.1k
  98066, // llvm.amdgcn.mfma.f32.32x32x8f16
  98098, // llvm.amdgcn.mfma.f32.4x4x1f32
  98128, // llvm.amdgcn.mfma.f32.4x4x2bf16
  98159, // llvm.amdgcn.mfma.f32.4x4x4bf16.1k
  98193, // llvm.amdgcn.mfma.f32.4x4x4f16
  98223, // llvm.amdgcn.mfma.f64.16x16x4f64
  98255, // llvm.amdgcn.mfma.f64.4x4x4f64
  98285, // llvm.amdgcn.mfma.i32.16x16x16i8
  98317, // llvm.amdgcn.mfma.i32.16x16x32.i8
  98350, // llvm.amdgcn.mfma.i32.16x16x4i8
  98381, // llvm.amdgcn.mfma.i32.16x16x64.i8
  98414, // llvm.amdgcn.mfma.i32.32x32x16.i8
  98447, // llvm.amdgcn.mfma.i32.32x32x32.i8
  98480, // llvm.amdgcn.mfma.i32.32x32x4i8
  98511, // llvm.amdgcn.mfma.i32.32x32x8i8
  98542, // llvm.amdgcn.mfma.i32.4x4x4i8
  98571, // llvm.amdgcn.mfma.scale.f32.16x16x128.f8f6f4
  98615, // llvm.amdgcn.mfma.scale.f32.32x32x64.f8f6f4
  98658, // llvm.amdgcn.mov.dpp
  98678, // llvm.amdgcn.mov.dpp8
  98699, // llvm.amdgcn.mqsad.pk.u16.u8
  98727, // llvm.amdgcn.mqsad.u32.u8
  98752, // llvm.amdgcn.msad.u8
  98772, // llvm.amdgcn.mul.i24
  98792, // llvm.amdgcn.mul.u24
  98812, // llvm.amdgcn.mulhi.i24
  98834, // llvm.amdgcn.mulhi.u24
  98856, // llvm.amdgcn.perm
  98873, // llvm.amdgcn.perm.pk16.b4.u4
  98901, // llvm.amdgcn.perm.pk16.b6.u4
  98929, // llvm.amdgcn.perm.pk16.b8.u4
  98957, // llvm.amdgcn.permlane.bcast
  98984, // llvm.amdgcn.permlane.down
  99010, // llvm.amdgcn.permlane.idx.gen
  99039, // llvm.amdgcn.permlane.up
  99063, // llvm.amdgcn.permlane.xor
  99088, // llvm.amdgcn.permlane16
  99111, // llvm.amdgcn.permlane16.swap
  99139, // llvm.amdgcn.permlane16.var
  99166, // llvm.amdgcn.permlane32.swap
  99194, // llvm.amdgcn.permlane64
  99217, // llvm.amdgcn.permlanex16
  99241, // llvm.amdgcn.permlanex16.var
  99269, // llvm.amdgcn.pk.add.max.i16
  99296, // llvm.amdgcn.pk.add.max.u16
  99323, // llvm.amdgcn.pk.add.min.i16
  99350, // llvm.amdgcn.pk.add.min.u16
  99377, // llvm.amdgcn.pops.exiting.wave.id
  99410, // llvm.amdgcn.prng.b32
  99431, // llvm.amdgcn.ps.live
  99451, // llvm.amdgcn.qsad.pk.u16.u8
  99478, // llvm.amdgcn.queue.ptr
  99500, // llvm.amdgcn.raw.atomic.buffer.load
  99535, // llvm.amdgcn.raw.buffer.atomic.add
  99569, // llvm.amdgcn.raw.buffer.atomic.and
  99603, // llvm.amdgcn.raw.buffer.atomic.cmpswap
  99641, // llvm.amdgcn.raw.buffer.atomic.cond.sub.u32
  99684, // llvm.amdgcn.raw.buffer.atomic.dec
  99718, // llvm.amdgcn.raw.buffer.atomic.fadd
  99753, // llvm.amdgcn.raw.buffer.atomic.fmax
  99788, // llvm.amdgcn.raw.buffer.atomic.fmin
  99823, // llvm.amdgcn.raw.buffer.atomic.inc
  99857, // llvm.amdgcn.raw.buffer.atomic.or
  99890, // llvm.amdgcn.raw.buffer.atomic.smax
  99925, // llvm.amdgcn.raw.buffer.atomic.smin
  99960, // llvm.amdgcn.raw.buffer.atomic.sub
  99994, // llvm.amdgcn.raw.buffer.atomic.sub.clamp.u32
  100038, // llvm.amdgcn.raw.buffer.atomic.swap
  100073, // llvm.amdgcn.raw.buffer.atomic.umax
  100108, // llvm.amdgcn.raw.buffer.atomic.umin
  100143, // llvm.amdgcn.raw.buffer.atomic.xor
  100177, // llvm.amdgcn.raw.buffer.load
  100205, // llvm.amdgcn.raw.buffer.load.format
  100240, // llvm.amdgcn.raw.buffer.load.lds
  100272, // llvm.amdgcn.raw.buffer.store
  100301, // llvm.amdgcn.raw.buffer.store.format
  100337, // llvm.amdgcn.raw.ptr.atomic.buffer.load
  100376, // llvm.amdgcn.raw.ptr.buffer.atomic.add
  100414, // llvm.amdgcn.raw.ptr.buffer.atomic.and
  100452, // llvm.amdgcn.raw.ptr.buffer.atomic.cmpswap
  100494, // llvm.amdgcn.raw.ptr.buffer.atomic.cond.sub.u32
  100541, // llvm.amdgcn.raw.ptr.buffer.atomic.dec
  100579, // llvm.amdgcn.raw.ptr.buffer.atomic.fadd
  100618, // llvm.amdgcn.raw.ptr.buffer.atomic.fmax
  100657, // llvm.amdgcn.raw.ptr.buffer.atomic.fmin
  100696, // llvm.amdgcn.raw.ptr.buffer.atomic.inc
  100734, // llvm.amdgcn.raw.ptr.buffer.atomic.or
  100771, // llvm.amdgcn.raw.ptr.buffer.atomic.smax
  100810, // llvm.amdgcn.raw.ptr.buffer.atomic.smin
  100849, // llvm.amdgcn.raw.ptr.buffer.atomic.sub
  100887, // llvm.amdgcn.raw.ptr.buffer.atomic.sub.clamp.u32
  100935, // llvm.amdgcn.raw.ptr.buffer.atomic.swap
  100974, // llvm.amdgcn.raw.ptr.buffer.atomic.umax
  101013, // llvm.amdgcn.raw.ptr.buffer.atomic.umin
  101052, // llvm.amdgcn.raw.ptr.buffer.atomic.xor
  101090, // llvm.amdgcn.raw.ptr.buffer.load
  101122, // llvm.amdgcn.raw.ptr.buffer.load.format
  101161, // llvm.amdgcn.raw.ptr.buffer.load.lds
  101197, // llvm.amdgcn.raw.ptr.buffer.store
  101230, // llvm.amdgcn.raw.ptr.buffer.store.format
  101270, // llvm.amdgcn.raw.ptr.tbuffer.load
  101303, // llvm.amdgcn.raw.ptr.tbuffer.store
  101337, // llvm.amdgcn.raw.tbuffer.load
  101366, // llvm.amdgcn.raw.tbuffer.store
  101396, // llvm.amdgcn.rcp
  101412, // llvm.amdgcn.rcp.legacy
  101435, // llvm.amdgcn.readfirstlane
  101461, // llvm.amdgcn.readlane
  101482, // llvm.amdgcn.reloc.constant
  101509, // llvm.amdgcn.rsq
  101525, // llvm.amdgcn.rsq.clamp
  101547, // llvm.amdgcn.rsq.legacy
  101570, // llvm.amdgcn.s.barrier
  101592, // llvm.amdgcn.s.barrier.init
  101619, // llvm.amdgcn.s.barrier.join
  101646, // llvm.amdgcn.s.barrier.leave
  101674, // llvm.amdgcn.s.barrier.signal
  101703, // llvm.amdgcn.s.barrier.signal.isfirst
  101740, // llvm.amdgcn.s.barrier.signal.var
  101773, // llvm.amdgcn.s.barrier.wait
  101800, // llvm.amdgcn.s.bitreplicate
  101827, // llvm.amdgcn.s.buffer.load
  101853, // llvm.amdgcn.s.buffer.prefetch.data
  101888, // llvm.amdgcn.s.cluster.barrier
  101918, // llvm.amdgcn.s.dcache.inv
  101943, // llvm.amdgcn.s.dcache.inv.vol
  101972, // llvm.amdgcn.s.dcache.wb
  101996, // llvm.amdgcn.s.dcache.wb.vol
  102024, // llvm.amdgcn.s.decperflevel
  102051, // llvm.amdgcn.s.get.barrier.state
  102083, // llvm.amdgcn.s.get.named.barrier.state
  102121, // llvm.amdgcn.s.get.waveid.in.workgroup
  102159, // llvm.amdgcn.s.getpc
  102179, // llvm.amdgcn.s.getreg
  102200, // llvm.amdgcn.s.incperflevel
  102227, // llvm.amdgcn.s.memrealtime
  102253, // llvm.amdgcn.s.memtime
  102275, // llvm.amdgcn.s.monitor.sleep
  102303, // llvm.amdgcn.s.nop
  102321, // llvm.amdgcn.s.prefetch.data
  102349, // llvm.amdgcn.s.quadmask
  102372, // llvm.amdgcn.s.sendmsg
  102394, // llvm.amdgcn.s.sendmsg.rtn
  102420, // llvm.amdgcn.s.sendmsghalt
  102446, // llvm.amdgcn.s.sethalt
  102468, // llvm.amdgcn.s.setprio
  102490, // llvm.amdgcn.s.setprio.inc.wg
  102519, // llvm.amdgcn.s.setreg
  102540, // llvm.amdgcn.s.sleep
  102560, // llvm.amdgcn.s.sleep.var
  102584, // llvm.amdgcn.s.ttracedata
  102609, // llvm.amdgcn.s.ttracedata.imm
  102638, // llvm.amdgcn.s.wait.asynccnt
  102666, // llvm.amdgcn.s.wait.bvhcnt
  102692, // llvm.amdgcn.s.wait.dscnt
  102717, // llvm.amdgcn.s.wait.event.export.ready
  102755, // llvm.amdgcn.s.wait.expcnt
  102781, // llvm.amdgcn.s.wait.kmcnt
  102806, // llvm.amdgcn.s.wait.loadcnt
  102833, // llvm.amdgcn.s.wait.samplecnt
  102862, // llvm.amdgcn.s.wait.storecnt
  102890, // llvm.amdgcn.s.wait.tensorcnt
  102919, // llvm.amdgcn.s.waitcnt
  102941, // llvm.amdgcn.s.wakeup.barrier
  102970, // llvm.amdgcn.s.wqm
  102988, // llvm.amdgcn.sad.hi.u8
  103010, // llvm.amdgcn.sad.u16
  103030, // llvm.amdgcn.sad.u8
  103049, // llvm.amdgcn.sat.pk4.i4.i8
  103075, // llvm.amdgcn.sat.pk4.u4.u8
  103101, // llvm.amdgcn.sbfe
  103118, // llvm.amdgcn.sched.barrier
  103144, // llvm.amdgcn.sched.group.barrier
  103176, // llvm.amdgcn.sdot2
  103194, // llvm.amdgcn.sdot4
  103212, // llvm.amdgcn.sdot8
  103230, // llvm.amdgcn.set.inactive
  103255, // llvm.amdgcn.set.inactive.chain.arg
  103290, // llvm.amdgcn.sffbh
  103308, // llvm.amdgcn.sin
  103324, // llvm.amdgcn.smfmac.f32.16x16x128.bf8.bf8
  103365, // llvm.amdgcn.smfmac.f32.16x16x128.bf8.fp8
  103406, // llvm.amdgcn.smfmac.f32.16x16x128.fp8.bf8
  103447, // llvm.amdgcn.smfmac.f32.16x16x128.fp8.fp8
  103488, // llvm.amdgcn.smfmac.f32.16x16x32.bf16
  103525, // llvm.amdgcn.smfmac.f32.16x16x32.f16
  103561, // llvm.amdgcn.smfmac.f32.16x16x64.bf16
  103598, // llvm.amdgcn.smfmac.f32.16x16x64.bf8.bf8
  103638, // llvm.amdgcn.smfmac.f32.16x16x64.bf8.fp8
  103678, // llvm.amdgcn.smfmac.f32.16x16x64.f16
  103714, // llvm.amdgcn.smfmac.f32.16x16x64.fp8.bf8
  103754, // llvm.amdgcn.smfmac.f32.16x16x64.fp8.fp8
  103794, // llvm.amdgcn.smfmac.f32.32x32x16.bf16
  103831, // llvm.amdgcn.smfmac.f32.32x32x16.f16
  103867, // llvm.amdgcn.smfmac.f32.32x32x32.bf16
  103904, // llvm.amdgcn.smfmac.f32.32x32x32.bf8.bf8
  103944, // llvm.amdgcn.smfmac.f32.32x32x32.bf8.fp8
  103984, // llvm.amdgcn.smfmac.f32.32x32x32.f16
  104020, // llvm.amdgcn.smfmac.f32.32x32x32.fp8.bf8
  104060, // llvm.amdgcn.smfmac.f32.32x32x32.fp8.fp8
  104100, // llvm.amdgcn.smfmac.f32.32x32x64.bf8.bf8
  104140, // llvm.amdgcn.smfmac.f32.32x32x64.bf8.fp8
  104180, // llvm.amdgcn.smfmac.f32.32x32x64.fp8.bf8
  104220, // llvm.amdgcn.smfmac.f32.32x32x64.fp8.fp8
  104260, // llvm.amdgcn.smfmac.i32.16x16x128.i8
  104296, // llvm.amdgcn.smfmac.i32.16x16x64.i8
  104331, // llvm.amdgcn.smfmac.i32.32x32x32.i8
  104366, // llvm.amdgcn.smfmac.i32.32x32x64.i8
  104401, // llvm.amdgcn.softwqm
  104421, // llvm.amdgcn.sqrt
  104438, // llvm.amdgcn.strict.wqm
  104461, // llvm.amdgcn.strict.wwm
  104484, // llvm.amdgcn.struct.atomic.buffer.load
  104522, // llvm.amdgcn.struct.buffer.atomic.add
  104559, // llvm.amdgcn.struct.buffer.atomic.and
  104596, // llvm.amdgcn.struct.buffer.atomic.cmpswap
  104637, // llvm.amdgcn.struct.buffer.atomic.cond.sub.u32
  104683, // llvm.amdgcn.struct.buffer.atomic.dec
  104720, // llvm.amdgcn.struct.buffer.atomic.fadd
  104758, // llvm.amdgcn.struct.buffer.atomic.fmax
  104796, // llvm.amdgcn.struct.buffer.atomic.fmin
  104834, // llvm.amdgcn.struct.buffer.atomic.inc
  104871, // llvm.amdgcn.struct.buffer.atomic.or
  104907, // llvm.amdgcn.struct.buffer.atomic.smax
  104945, // llvm.amdgcn.struct.buffer.atomic.smin
  104983, // llvm.amdgcn.struct.buffer.atomic.sub
  105020, // llvm.amdgcn.struct.buffer.atomic.sub.clamp.u32
  105067, // llvm.amdgcn.struct.buffer.atomic.swap
  105105, // llvm.amdgcn.struct.buffer.atomic.umax
  105143, // llvm.amdgcn.struct.buffer.atomic.umin
  105181, // llvm.amdgcn.struct.buffer.atomic.xor
  105218, // llvm.amdgcn.struct.buffer.load
  105249, // llvm.amdgcn.struct.buffer.load.format
  105287, // llvm.amdgcn.struct.buffer.load.lds
  105322, // llvm.amdgcn.struct.buffer.store
  105354, // llvm.amdgcn.struct.buffer.store.format
  105393, // llvm.amdgcn.struct.ptr.atomic.buffer.load
  105435, // llvm.amdgcn.struct.ptr.buffer.atomic.add
  105476, // llvm.amdgcn.struct.ptr.buffer.atomic.and
  105517, // llvm.amdgcn.struct.ptr.buffer.atomic.cmpswap
  105562, // llvm.amdgcn.struct.ptr.buffer.atomic.cond.sub.u32
  105612, // llvm.amdgcn.struct.ptr.buffer.atomic.dec
  105653, // llvm.amdgcn.struct.ptr.buffer.atomic.fadd
  105695, // llvm.amdgcn.struct.ptr.buffer.atomic.fmax
  105737, // llvm.amdgcn.struct.ptr.buffer.atomic.fmin
  105779, // llvm.amdgcn.struct.ptr.buffer.atomic.inc
  105820, // llvm.amdgcn.struct.ptr.buffer.atomic.or
  105860, // llvm.amdgcn.struct.ptr.buffer.atomic.smax
  105902, // llvm.amdgcn.struct.ptr.buffer.atomic.smin
  105944, // llvm.amdgcn.struct.ptr.buffer.atomic.sub
  105985, // llvm.amdgcn.struct.ptr.buffer.atomic.sub.clamp.u32
  106036, // llvm.amdgcn.struct.ptr.buffer.atomic.swap
  106078, // llvm.amdgcn.struct.ptr.buffer.atomic.umax
  106120, // llvm.amdgcn.struct.ptr.buffer.atomic.umin
  106162, // llvm.amdgcn.struct.ptr.buffer.atomic.xor
  106203, // llvm.amdgcn.struct.ptr.buffer.load
  106238, // llvm.amdgcn.struct.ptr.buffer.load.format
  106280, // llvm.amdgcn.struct.ptr.buffer.load.lds
  106319, // llvm.amdgcn.struct.ptr.buffer.store
  106355, // llvm.amdgcn.struct.ptr.buffer.store.format
  106398, // llvm.amdgcn.struct.ptr.tbuffer.load
  106434, // llvm.amdgcn.struct.ptr.tbuffer.store
  106471, // llvm.amdgcn.struct.tbuffer.load
  106503, // llvm.amdgcn.struct.tbuffer.store
  106536, // llvm.amdgcn.sudot4
  106555, // llvm.amdgcn.sudot8
  106574, // llvm.amdgcn.swmmac.bf16.16x16x32.bf16
  106612, // llvm.amdgcn.swmmac.bf16.16x16x64.bf16
  106650, // llvm.amdgcn.swmmac.bf16f32.16x16x64.bf16
  106691, // llvm.amdgcn.swmmac.f16.16x16x128.bf8.bf8
  106732, // llvm.amdgcn.swmmac.f16.16x16x128.bf8.fp8
  106773, // llvm.amdgcn.swmmac.f16.16x16x128.fp8.bf8
  106814, // llvm.amdgcn.swmmac.f16.16x16x128.fp8.fp8
  106855, // llvm.amdgcn.swmmac.f16.16x16x32.f16
  106891, // llvm.amdgcn.swmmac.f16.16x16x64.f16
  106927, // llvm.amdgcn.swmmac.f32.16x16x128.bf8.bf8
  106968, // llvm.amdgcn.swmmac.f32.16x16x128.bf8.fp8
  107009, // llvm.amdgcn.swmmac.f32.16x16x128.fp8.bf8
  107050, // llvm.amdgcn.swmmac.f32.16x16x128.fp8.fp8
  107091, // llvm.amdgcn.swmmac.f32.16x16x32.bf16
  107128, // llvm.amdgcn.swmmac.f32.16x16x32.bf8.bf8
  107168, // llvm.amdgcn.swmmac.f32.16x16x32.bf8.fp8
  107208, // llvm.amdgcn.swmmac.f32.16x16x32.f16
  107244, // llvm.amdgcn.swmmac.f32.16x16x32.fp8.bf8
  107284, // llvm.amdgcn.swmmac.f32.16x16x32.fp8.fp8
  107324, // llvm.amdgcn.swmmac.f32.16x16x64.bf16
  107361, // llvm.amdgcn.swmmac.f32.16x16x64.f16
  107397, // llvm.amdgcn.swmmac.i32.16x16x128.iu8
  107434, // llvm.amdgcn.swmmac.i32.16x16x32.iu4
  107470, // llvm.amdgcn.swmmac.i32.16x16x32.iu8
  107506, // llvm.amdgcn.swmmac.i32.16x16x64.iu4
  107542, // llvm.amdgcn.tanh
  107559, // llvm.amdgcn.tensor.load.to.lds
  107590, // llvm.amdgcn.tensor.load.to.lds.d2
  107624, // llvm.amdgcn.tensor.store.from.lds
  107658, // llvm.amdgcn.tensor.store.from.lds.d2
  107695, // llvm.amdgcn.trig.preop
  107718, // llvm.amdgcn.ubfe
  107735, // llvm.amdgcn.udot2
  107753, // llvm.amdgcn.udot4
  107771, // llvm.amdgcn.udot8
  107789, // llvm.amdgcn.unreachable
  107813, // llvm.amdgcn.update.dpp
  107836, // llvm.amdgcn.wave.barrier
  107861, // llvm.amdgcn.wave.id
  107881, // llvm.amdgcn.wave.reduce.add
  107909, // llvm.amdgcn.wave.reduce.and
  107937, // llvm.amdgcn.wave.reduce.fadd
  107966, // llvm.amdgcn.wave.reduce.fmax
  107995, // llvm.amdgcn.wave.reduce.fmin
  108024, // llvm.amdgcn.wave.reduce.fsub
  108053, // llvm.amdgcn.wave.reduce.max
  108081, // llvm.amdgcn.wave.reduce.min
  108109, // llvm.amdgcn.wave.reduce.or
  108136, // llvm.amdgcn.wave.reduce.sub
  108164, // llvm.amdgcn.wave.reduce.umax
  108193, // llvm.amdgcn.wave.reduce.umin
  108222, // llvm.amdgcn.wave.reduce.xor
  108250, // llvm.amdgcn.wave.shuffle
  108275, // llvm.amdgcn.wavefrontsize
  108301, // llvm.amdgcn.wmma.bf16.16x16x16.bf16
  108337, // llvm.amdgcn.wmma.bf16.16x16x16.bf16.tied
  108378, // llvm.amdgcn.wmma.bf16.16x16x32.bf16
  108414, // llvm.amdgcn.wmma.bf16f32.16x16x32.bf16
  108453, // llvm.amdgcn.wmma.f16.16x16x128.bf8.bf8
  108492, // llvm.amdgcn.wmma.f16.16x16x128.bf8.fp8
  108531, // llvm.amdgcn.wmma.f16.16x16x128.fp8.bf8
  108570, // llvm.amdgcn.wmma.f16.16x16x128.fp8.fp8
  108609, // llvm.amdgcn.wmma.f16.16x16x16.f16
  108643, // llvm.amdgcn.wmma.f16.16x16x16.f16.tied
  108682, // llvm.amdgcn.wmma.f16.16x16x32.f16
  108716, // llvm.amdgcn.wmma.f16.16x16x64.bf8.bf8
  108754, // llvm.amdgcn.wmma.f16.16x16x64.bf8.fp8
  108792, // llvm.amdgcn.wmma.f16.16x16x64.fp8.bf8
  108830, // llvm.amdgcn.wmma.f16.16x16x64.fp8.fp8
  108868, // llvm.amdgcn.wmma.f32.16x16x128.bf8.bf8
  108907, // llvm.amdgcn.wmma.f32.16x16x128.bf8.fp8
  108946, // llvm.amdgcn.wmma.f32.16x16x128.f8f6f4
  108984, // llvm.amdgcn.wmma.f32.16x16x128.fp8.bf8
  109023, // llvm.amdgcn.wmma.f32.16x16x128.fp8.fp8
  109062, // llvm.amdgcn.wmma.f32.16x16x16.bf16
  109097, // llvm.amdgcn.wmma.f32.16x16x16.bf8.bf8
  109135, // llvm.amdgcn.wmma.f32.16x16x16.bf8.fp8
  109173, // llvm.amdgcn.wmma.f32.16x16x16.f16
  109207, // llvm.amdgcn.wmma.f32.16x16x16.fp8.bf8
  109245, // llvm.amdgcn.wmma.f32.16x16x16.fp8.fp8
  109283, // llvm.amdgcn.wmma.f32.16x16x32.bf16
  109318, // llvm.amdgcn.wmma.f32.16x16x32.f16
  109352, // llvm.amdgcn.wmma.f32.16x16x4.f32
  109385, // llvm.amdgcn.wmma.f32.16x16x64.bf8.bf8
  109423, // llvm.amdgcn.wmma.f32.16x16x64.bf8.fp8
  109461, // llvm.amdgcn.wmma.f32.16x16x64.fp8.bf8
  109499, // llvm.amdgcn.wmma.f32.16x16x64.fp8.fp8
  109537, // llvm.amdgcn.wmma.f32.32x16x128.f4
  109571, // llvm.amdgcn.wmma.i32.16x16x16.iu4
  109605, // llvm.amdgcn.wmma.i32.16x16x16.iu8
  109639, // llvm.amdgcn.wmma.i32.16x16x32.iu4
  109673, // llvm.amdgcn.wmma.i32.16x16x64.iu8
  109707, // llvm.amdgcn.wmma.scale.f32.16x16x128.f8f6f4
  109751, // llvm.amdgcn.wmma.scale.f32.32x16x128.f4
  109791, // llvm.amdgcn.wmma.scale16.f32.16x16x128.f8f6f4
  109837, // llvm.amdgcn.wmma.scale16.f32.32x16x128.f4
  109879, // llvm.amdgcn.workgroup.id.x
  109906, // llvm.amdgcn.workgroup.id.y
  109933, // llvm.amdgcn.workgroup.id.z
  109960, // llvm.amdgcn.workitem.id.x
  109986, // llvm.amdgcn.workitem.id.y
  110012, // llvm.amdgcn.workitem.id.z
  110038, // llvm.amdgcn.wqm
  110054, // llvm.amdgcn.wqm.demote
  110077, // llvm.amdgcn.wqm.vote
  110098, // llvm.amdgcn.writelane
  110120, // llvm.amdgcn.wwm
  110136, // llvm.arm.cde.cx1
  110153, // llvm.arm.cde.cx1a
  110171, // llvm.arm.cde.cx1d
  110189, // llvm.arm.cde.cx1da
  110208, // llvm.arm.cde.cx2
  110225, // llvm.arm.cde.cx2a
  110243, // llvm.arm.cde.cx2d
  110261, // llvm.arm.cde.cx2da
  110280, // llvm.arm.cde.cx3
  110297, // llvm.arm.cde.cx3a
  110315, // llvm.arm.cde.cx3d
  110333, // llvm.arm.cde.cx3da
  110352, // llvm.arm.cde.vcx1
  110370, // llvm.arm.cde.vcx1a
  110389, // llvm.arm.cde.vcx1q
  110408, // llvm.arm.cde.vcx1q.predicated
  110438, // llvm.arm.cde.vcx1qa
  110458, // llvm.arm.cde.vcx1qa.predicated
  110489, // llvm.arm.cde.vcx2
  110507, // llvm.arm.cde.vcx2a
  110526, // llvm.arm.cde.vcx2q
  110545, // llvm.arm.cde.vcx2q.predicated
  110575, // llvm.arm.cde.vcx2qa
  110595, // llvm.arm.cde.vcx2qa.predicated
  110626, // llvm.arm.cde.vcx3
  110644, // llvm.arm.cde.vcx3a
  110663, // llvm.arm.cde.vcx3q
  110682, // llvm.arm.cde.vcx3q.predicated
  110712, // llvm.arm.cde.vcx3qa
  110732, // llvm.arm.cde.vcx3qa.predicated
  110763, // llvm.arm.cdp
  110776, // llvm.arm.cdp2
  110790, // llvm.arm.clrex
  110805, // llvm.arm.cls
  110818, // llvm.arm.cls64
  110833, // llvm.arm.cmse.tt
  110850, // llvm.arm.cmse.tta
  110868, // llvm.arm.cmse.ttat
  110887, // llvm.arm.cmse.ttt
  110905, // llvm.arm.crc32b
  110921, // llvm.arm.crc32cb
  110938, // llvm.arm.crc32ch
  110955, // llvm.arm.crc32cw
  110972, // llvm.arm.crc32h
  110988, // llvm.arm.crc32w
  111004, // llvm.arm.dbg
  111017, // llvm.arm.dmb
  111030, // llvm.arm.dsb
  111043, // llvm.arm.get.fpscr
  111062, // llvm.arm.gnu.eabi.mcount
  111087, // llvm.arm.hint
  111101, // llvm.arm.isb
  111114, // llvm.arm.ldaex
  111129, // llvm.arm.ldaexd
  111145, // llvm.arm.ldc
  111158, // llvm.arm.ldc2
  111172, // llvm.arm.ldc2l
  111187, // llvm.arm.ldcl
  111201, // llvm.arm.ldrex
  111216, // llvm.arm.ldrexd
  111232, // llvm.arm.mcr
  111245, // llvm.arm.mcr2
  111259, // llvm.arm.mcrr
  111273, // llvm.arm.mcrr2
  111288, // llvm.arm.mrc
  111301, // llvm.arm.mrc2
  111315, // llvm.arm.mrrc
  111329, // llvm.arm.mrrc2
  111344, // llvm.arm.mve.abd.predicated
  111372, // llvm.arm.mve.abs.predicated
  111400, // llvm.arm.mve.add.predicated
  111428, // llvm.arm.mve.addlv
  111447, // llvm.arm.mve.addlv.predicated
  111477, // llvm.arm.mve.addv
  111495, // llvm.arm.mve.addv.predicated
  111524, // llvm.arm.mve.and.predicated
  111552, // llvm.arm.mve.asrl
  111570, // llvm.arm.mve.bic.predicated
  111598, // llvm.arm.mve.cls.predicated
  111626, // llvm.arm.mve.clz.predicated
  111654, // llvm.arm.mve.cmp.eq
  111674, // llvm.arm.mve.cmp.ge
  111694, // llvm.arm.mve.cmp.gt
  111714, // llvm.arm.mve.cmp.le
  111734, // llvm.arm.mve.cmp.lt
  111754, // llvm.arm.mve.cmp.ne
  111774, // llvm.arm.mve.eor.predicated
  111802, // llvm.arm.mve.fma
  111819, // llvm.arm.mve.fma.predicated
  111847, // llvm.arm.mve.hadd.predicated
  111876, // llvm.arm.mve.hsub.predicated
  111905, // llvm.arm.mve.lsll
  111923, // llvm.arm.mve.max.predicated
  111951, // llvm.arm.mve.maxav
  111970, // llvm.arm.mve.maxav.predicated
  112000, // llvm.arm.mve.maxnmav
  112021, // llvm.arm.mve.maxnmav.predicated
  112053, // llvm.arm.mve.maxnmv
  112073, // llvm.arm.mve.maxnmv.predicated
  112104, // llvm.arm.mve.maxv
  112122, // llvm.arm.mve.maxv.predicated
  112151, // llvm.arm.mve.min.predicated
  112179, // llvm.arm.mve.minav
  112198, // llvm.arm.mve.minav.predicated
  112228, // llvm.arm.mve.minnmav
  112249, // llvm.arm.mve.minnmav.predicated
  112281, // llvm.arm.mve.minnmv
  112301, // llvm.arm.mve.minnmv.predicated
  112332, // llvm.arm.mve.minv
  112350, // llvm.arm.mve.minv.predicated
  112379, // llvm.arm.mve.mul.predicated
  112407, // llvm.arm.mve.mulh.predicated
  112436, // llvm.arm.mve.mull.int.predicated
  112469, // llvm.arm.mve.mull.poly.predicated
  112503, // llvm.arm.mve.mvn.predicated
  112531, // llvm.arm.mve.neg.predicated
  112559, // llvm.arm.mve.orn.predicated
  112587, // llvm.arm.mve.orr.predicated
  112615, // llvm.arm.mve.pred.i2v
  112637, // llvm.arm.mve.pred.v2i
  112659, // llvm.arm.mve.qabs.predicated
  112688, // llvm.arm.mve.qadd.predicated
  112717, // llvm.arm.mve.qdmulh.predicated
  112748, // llvm.arm.mve.qneg.predicated
  112777, // llvm.arm.mve.qrdmulh.predicated
  112809, // llvm.arm.mve.qsub.predicated
  112838, // llvm.arm.mve.rhadd.predicated
  112868, // llvm.arm.mve.rmulh.predicated
  112898, // llvm.arm.mve.shl.imm.predicated
  112930, // llvm.arm.mve.shr.imm.predicated
  112962, // llvm.arm.mve.sqrshr
  112982, // llvm.arm.mve.sqrshrl
  113003, // llvm.arm.mve.sqshl
  113022, // llvm.arm.mve.sqshll
  113042, // llvm.arm.mve.srshr
  113061, // llvm.arm.mve.srshrl
  113081, // llvm.arm.mve.sub.predicated
  113109, // llvm.arm.mve.uqrshl
  113129, // llvm.arm.mve.uqrshll
  113150, // llvm.arm.mve.uqshl
  113169, // llvm.arm.mve.uqshll
  113189, // llvm.arm.mve.urshr
  113208, // llvm.arm.mve.urshrl
  113228, // llvm.arm.mve.vabav
  113247, // llvm.arm.mve.vabav.predicated
  113277, // llvm.arm.mve.vabd
  113295, // llvm.arm.mve.vadc
  113313, // llvm.arm.mve.vadc.predicated
  113342, // llvm.arm.mve.vadd
  113360, // llvm.arm.mve.vbrsr
  113379, // llvm.arm.mve.vbrsr.predicated
  113409, // llvm.arm.mve.vcaddq
  113429, // llvm.arm.mve.vcaddq.predicated
  113460, // llvm.arm.mve.vcls
  113478, // llvm.arm.mve.vcmlaq
  113498, // llvm.arm.mve.vcmlaq.predicated
  113529, // llvm.arm.mve.vcmulq
  113549, // llvm.arm.mve.vcmulq.predicated
  113580, // llvm.arm.mve.vctp16
  113600, // llvm.arm.mve.vctp32
  113620, // llvm.arm.mve.vctp64
  113640, // llvm.arm.mve.vctp8
  113659, // llvm.arm.mve.vcvt.fix
  113681, // llvm.arm.mve.vcvt.fix.predicated
  113714, // llvm.arm.mve.vcvt.fp.int
  113739, // llvm.arm.mve.vcvt.fp.int.predicated
  113775, // llvm.arm.mve.vcvt.int.fp
  113800, // llvm.arm.mve.vcvt.narrow
  113825, // llvm.arm.mve.vcvt.narrow.predicated
  113861, // llvm.arm.mve.vcvt.widen
  113885, // llvm.arm.mve.vcvt.widen.predicated
  113920, // llvm.arm.mve.vcvta
  113939, // llvm.arm.mve.vcvta.predicated
  113969, // llvm.arm.mve.vcvtm
  113988, // llvm.arm.mve.vcvtm.predicated
  114018, // llvm.arm.mve.vcvtn
  114037, // llvm.arm.mve.vcvtn.predicated
  114067, // llvm.arm.mve.vcvtp
  114086, // llvm.arm.mve.vcvtp.predicated
  114116, // llvm.arm.mve.vddup
  114135, // llvm.arm.mve.vddup.predicated
  114165, // llvm.arm.mve.vdwdup
  114185, // llvm.arm.mve.vdwdup.predicated
  114216, // llvm.arm.mve.vhadd
  114235, // llvm.arm.mve.vhsub
  114254, // llvm.arm.mve.vidup
  114273, // llvm.arm.mve.vidup.predicated
  114303, // llvm.arm.mve.viwdup
  114323, // llvm.arm.mve.viwdup.predicated
  114354, // llvm.arm.mve.vld2q
  114373, // llvm.arm.mve.vld4q
  114392, // llvm.arm.mve.vldr.gather.base
  114422, // llvm.arm.mve.vldr.gather.base.predicated
  114463, // llvm.arm.mve.vldr.gather.base.wb
  114496, // llvm.arm.mve.vldr.gather.base.wb.predicated
  114540, // llvm.arm.mve.vldr.gather.offset
  114572, // llvm.arm.mve.vldr.gather.offset.predicated
  114615, // llvm.arm.mve.vmaxa.predicated
  114645, // llvm.arm.mve.vmaxnm
  114665, // llvm.arm.mve.vmaxnma.predicated
  114697, // llvm.arm.mve.vmina.predicated
  114727, // llvm.arm.mve.vminnm
  114747, // llvm.arm.mve.vminnma.predicated
  114779, // llvm.arm.mve.vmla.n.predicated
  114810, // llvm.arm.mve.vmlas.n.predicated
  114842, // llvm.arm.mve.vmldava
  114863, // llvm.arm.mve.vmldava.predicated
  114895, // llvm.arm.mve.vmlldava
  114917, // llvm.arm.mve.vmlldava.predicated
  114950, // llvm.arm.mve.vmovl.predicated
  114980, // llvm.arm.mve.vmovn.predicated
  115010, // llvm.arm.mve.vmul
  115028, // llvm.arm.mve.vmulh
  115047, // llvm.arm.mve.vmull
  115066, // llvm.arm.mve.vmull.poly
  115090, // llvm.arm.mve.vqdmlad
  115111, // llvm.arm.mve.vqdmlad.predicated
  115143, // llvm.arm.mve.vqdmlah
  115164, // llvm.arm.mve.vqdmlah.predicated
  115196, // llvm.arm.mve.vqdmlash
  115218, // llvm.arm.mve.vqdmlash.predicated
  115251, // llvm.arm.mve.vqdmulh
  115272, // llvm.arm.mve.vqdmull
  115293, // llvm.arm.mve.vqdmull.predicated
  115325, // llvm.arm.mve.vqmovn
  115345, // llvm.arm.mve.vqmovn.predicated
  115376, // llvm.arm.mve.vqrdmlah
  115398, // llvm.arm.mve.vqrdmlah.predicated
  115431, // llvm.arm.mve.vqrdmlash
  115454, // llvm.arm.mve.vqrdmlash.predicated
  115488, // llvm.arm.mve.vqrdmulh
  115510, // llvm.arm.mve.vqshl.imm
  115533, // llvm.arm.mve.vqshl.imm.predicated
  115567, // llvm.arm.mve.vqshlu.imm
  115591, // llvm.arm.mve.vqshlu.imm.predicated
  115626, // llvm.arm.mve.vreinterpretq
  115653, // llvm.arm.mve.vrev.predicated
  115682, // llvm.arm.mve.vrhadd
  115702, // llvm.arm.mve.vrinta
  115722, // llvm.arm.mve.vrinta.predicated
  115753, // llvm.arm.mve.vrintm
  115773, // llvm.arm.mve.vrintm.predicated
  115804, // llvm.arm.mve.vrintn
  115824, // llvm.arm.mve.vrintn.predicated
  115855, // llvm.arm.mve.vrintp
  115875, // llvm.arm.mve.vrintp.predicated
  115906, // llvm.arm.mve.vrintx
  115926, // llvm.arm.mve.vrintx.predicated
  115957, // llvm.arm.mve.vrintz
  115977, // llvm.arm.mve.vrintz.predicated
  116008, // llvm.arm.mve.vrmlldavha
  116032, // llvm.arm.mve.vrmlldavha.predicated
  116067, // llvm.arm.mve.vrmulh
  116087, // llvm.arm.mve.vrshr.imm
  116110, // llvm.arm.mve.vrshr.imm.predicated
  116144, // llvm.arm.mve.vsbc
  116162, // llvm.arm.mve.vsbc.predicated
  116191, // llvm.arm.mve.vshl.scalar
  116216, // llvm.arm.mve.vshl.scalar.predicated
  116252, // llvm.arm.mve.vshl.vector
  116277, // llvm.arm.mve.vshl.vector.predicated
  116313, // llvm.arm.mve.vshlc
  116332, // llvm.arm.mve.vshlc.predicated
  116362, // llvm.arm.mve.vshll.imm
  116385, // llvm.arm.mve.vshll.imm.predicated
  116419, // llvm.arm.mve.vshrn
  116438, // llvm.arm.mve.vshrn.predicated
  116468, // llvm.arm.mve.vsli
  116486, // llvm.arm.mve.vsli.predicated
  116515, // llvm.arm.mve.vsri
  116533, // llvm.arm.mve.vsri.predicated
  116562, // llvm.arm.mve.vst2q
  116581, // llvm.arm.mve.vst4q
  116600, // llvm.arm.mve.vstr.scatter.base
  116631, // llvm.arm.mve.vstr.scatter.base.predicated
  116673, // llvm.arm.mve.vstr.scatter.base.wb
  116707, // llvm.arm.mve.vstr.scatter.base.wb.predicated
  116752, // llvm.arm.mve.vstr.scatter.offset
  116785, // llvm.arm.mve.vstr.scatter.offset.predicated
  116829, // llvm.arm.mve.vsub
  116847, // llvm.arm.neon.aesd
  116866, // llvm.arm.neon.aese
  116885, // llvm.arm.neon.aesimc
  116906, // llvm.arm.neon.aesmc
  116926, // llvm.arm.neon.bfdot
  116946, // llvm.arm.neon.bfmlalb
  116968, // llvm.arm.neon.bfmlalt
  116990, // llvm.arm.neon.bfmmla
  117011, // llvm.arm.neon.sdot
  117030, // llvm.arm.neon.sha1c
  117050, // llvm.arm.neon.sha1h
  117070, // llvm.arm.neon.sha1m
  117090, // llvm.arm.neon.sha1p
  117110, // llvm.arm.neon.sha1su0
  117132, // llvm.arm.neon.sha1su1
  117154, // llvm.arm.neon.sha256h
  117176, // llvm.arm.neon.sha256h2
  117199, // llvm.arm.neon.sha256su0
  117223, // llvm.arm.neon.sha256su1
  117247, // llvm.arm.neon.smmla
  117267, // llvm.arm.neon.udot
  117286, // llvm.arm.neon.ummla
  117306, // llvm.arm.neon.usdot
  117326, // llvm.arm.neon.usmmla
  117347, // llvm.arm.neon.vabds
  117367, // llvm.arm.neon.vabdu
  117387, // llvm.arm.neon.vabs
  117406, // llvm.arm.neon.vacge
  117426, // llvm.arm.neon.vacgt
  117446, // llvm.arm.neon.vbsl
  117465, // llvm.arm.neon.vcadd.rot270
  117492, // llvm.arm.neon.vcadd.rot90
  117518, // llvm.arm.neon.vcls
  117537, // llvm.arm.neon.vcvtas
  117558, // llvm.arm.neon.vcvtau
  117579, // llvm.arm.neon.vcvtbfp2bf
  117604, // llvm.arm.neon.vcvtfp2bf
  117628, // llvm.arm.neon.vcvtfp2fxs
  117653, // llvm.arm.neon.vcvtfp2fxu
  117678, // llvm.arm.neon.vcvtfp2hf
  117702, // llvm.arm.neon.vcvtfxs2fp
  117727, // llvm.arm.neon.vcvtfxu2fp
  117752, // llvm.arm.neon.vcvthf2fp
  117776, // llvm.arm.neon.vcvtms
  117797, // llvm.arm.neon.vcvtmu
  117818, // llvm.arm.neon.vcvtns
  117839, // llvm.arm.neon.vcvtnu
  117860, // llvm.arm.neon.vcvtps
  117881, // llvm.arm.neon.vcvtpu
  117902, // llvm.arm.neon.vhadds
  117923, // llvm.arm.neon.vhaddu
  117944, // llvm.arm.neon.vhsubs
  117965, // llvm.arm.neon.vhsubu
  117986, // llvm.arm.neon.vld1
  118005, // llvm.arm.neon.vld1x2
  118026, // llvm.arm.neon.vld1x3
  118047, // llvm.arm.neon.vld1x4
  118068, // llvm.arm.neon.vld2
  118087, // llvm.arm.neon.vld2dup
  118109, // llvm.arm.neon.vld2lane
  118132, // llvm.arm.neon.vld3
  118151, // llvm.arm.neon.vld3dup
  118173, // llvm.arm.neon.vld3lane
  118196, // llvm.arm.neon.vld4
  118215, // llvm.arm.neon.vld4dup
  118237, // llvm.arm.neon.vld4lane
  118260, // llvm.arm.neon.vmaxnm
  118281, // llvm.arm.neon.vmaxs
  118301, // llvm.arm.neon.vmaxu
  118321, // llvm.arm.neon.vminnm
  118342, // llvm.arm.neon.vmins
  118362, // llvm.arm.neon.vminu
  118382, // llvm.arm.neon.vmullp
  118403, // llvm.arm.neon.vmulls
  118424, // llvm.arm.neon.vmullu
  118445, // llvm.arm.neon.vmulp
  118465, // llvm.arm.neon.vpadals
  118487, // llvm.arm.neon.vpadalu
  118509, // llvm.arm.neon.vpadd
  118529, // llvm.arm.neon.vpaddls
  118551, // llvm.arm.neon.vpaddlu
  118573, // llvm.arm.neon.vpmaxs
  118594, // llvm.arm.neon.vpmaxu
  118615, // llvm.arm.neon.vpmins
  118636, // llvm.arm.neon.vpminu
  118657, // llvm.arm.neon.vqabs
  118677, // llvm.arm.neon.vqdmulh
  118699, // llvm.arm.neon.vqdmull
  118721, // llvm.arm.neon.vqmovns
  118743, // llvm.arm.neon.vqmovnsu
  118766, // llvm.arm.neon.vqmovnu
  118788, // llvm.arm.neon.vqneg
  118808, // llvm.arm.neon.vqrdmlah
  118831, // llvm.arm.neon.vqrdmlsh
  118854, // llvm.arm.neon.vqrdmulh
  118877, // llvm.arm.neon.vqrshiftns
  118902, // llvm.arm.neon.vqrshiftnsu
  118928, // llvm.arm.neon.vqrshiftnu
  118953, // llvm.arm.neon.vqrshifts
  118977, // llvm.arm.neon.vqrshiftu
  119001, // llvm.arm.neon.vqshiftns
  119025, // llvm.arm.neon.vqshiftnsu
  119050, // llvm.arm.neon.vqshiftnu
  119074, // llvm.arm.neon.vqshifts
  119097, // llvm.arm.neon.vqshiftsu
  119121, // llvm.arm.neon.vqshiftu
  119144, // llvm.arm.neon.vraddhn
  119166, // llvm.arm.neon.vrecpe
  119187, // llvm.arm.neon.vrecps
  119208, // llvm.arm.neon.vrhadds
  119230, // llvm.arm.neon.vrhaddu
  119252, // llvm.arm.neon.vrshiftn
  119275, // llvm.arm.neon.vrshifts
  119298, // llvm.arm.neon.vrshiftu
  119321, // llvm.arm.neon.vrsqrte
  119343, // llvm.arm.neon.vrsqrts
  119365, // llvm.arm.neon.vrsubhn
  119387, // llvm.arm.neon.vshiftins
  119411, // llvm.arm.neon.vshifts
  119433, // llvm.arm.neon.vshiftu
  119455, // llvm.arm.neon.vst1
  119474, // llvm.arm.neon.vst1x2
  119495, // llvm.arm.neon.vst1x3
  119516, // llvm.arm.neon.vst1x4
  119537, // llvm.arm.neon.vst2
  119556, // llvm.arm.neon.vst2lane
  119579, // llvm.arm.neon.vst3
  119598, // llvm.arm.neon.vst3lane
  119621, // llvm.arm.neon.vst4
  119640, // llvm.arm.neon.vst4lane
  119663, // llvm.arm.neon.vtbl1
  119683, // llvm.arm.neon.vtbl2
  119703, // llvm.arm.neon.vtbl3
  119723, // llvm.arm.neon.vtbl4
  119743, // llvm.arm.neon.vtbx1
  119763, // llvm.arm.neon.vtbx2
  119783, // llvm.arm.neon.vtbx3
  119803, // llvm.arm.neon.vtbx4
  119823, // llvm.arm.qadd
  119837, // llvm.arm.qadd16
  119853, // llvm.arm.qadd8
  119868, // llvm.arm.qasx
  119882, // llvm.arm.qsax
  119896, // llvm.arm.qsub
  119910, // llvm.arm.qsub16
  119926, // llvm.arm.qsub8
  119941, // llvm.arm.sadd16
  119957, // llvm.arm.sadd8
  119972, // llvm.arm.sasx
  119986, // llvm.arm.sel
  119999, // llvm.arm.set.fpscr
  120018, // llvm.arm.shadd16
  120035, // llvm.arm.shadd8
  120051, // llvm.arm.shasx
  120066, // llvm.arm.shsax
  120081, // llvm.arm.shsub16
  120098, // llvm.arm.shsub8
  120114, // llvm.arm.smlabb
  120130, // llvm.arm.smlabt
  120146, // llvm.arm.smlad
  120161, // llvm.arm.smladx
  120177, // llvm.arm.smlald
  120193, // llvm.arm.smlaldx
  120210, // llvm.arm.smlatb
  120226, // llvm.arm.smlatt
  120242, // llvm.arm.smlawb
  120258, // llvm.arm.smlawt
  120274, // llvm.arm.smlsd
  120289, // llvm.arm.smlsdx
  120305, // llvm.arm.smlsld
  120321, // llvm.arm.smlsldx
  120338, // llvm.arm.smuad
  120353, // llvm.arm.smuadx
  120369, // llvm.arm.smulbb
  120385, // llvm.arm.smulbt
  120401, // llvm.arm.smultb
  120417, // llvm.arm.smultt
  120433, // llvm.arm.smulwb
  120449, // llvm.arm.smulwt
  120465, // llvm.arm.smusd
  120480, // llvm.arm.smusdx
  120496, // llvm.arm.space
  120511, // llvm.arm.ssat
  120525, // llvm.arm.ssat16
  120541, // llvm.arm.ssax
  120555, // llvm.arm.ssub16
  120571, // llvm.arm.ssub8
  120586, // llvm.arm.stc
  120599, // llvm.arm.stc2
  120613, // llvm.arm.stc2l
  120628, // llvm.arm.stcl
  120642, // llvm.arm.stlex
  120657, // llvm.arm.stlexd
  120673, // llvm.arm.strex
  120688, // llvm.arm.strexd
  120704, // llvm.arm.sxtab16
  120721, // llvm.arm.sxtb16
  120737, // llvm.arm.uadd16
  120753, // llvm.arm.uadd8
  120768, // llvm.arm.uasx
  120782, // llvm.arm.uhadd16
  120799, // llvm.arm.uhadd8
  120815, // llvm.arm.uhasx
  120830, // llvm.arm.uhsax
  120845, // llvm.arm.uhsub16
  120862, // llvm.arm.uhsub8
  120878, // llvm.arm.undefined
  120897, // llvm.arm.uqadd16
  120914, // llvm.arm.uqadd8
  120930, // llvm.arm.uqasx
  120945, // llvm.arm.uqsax
  120960, // llvm.arm.uqsub16
  120977, // llvm.arm.uqsub8
  120993, // llvm.arm.usad8
  121008, // llvm.arm.usada8
  121024, // llvm.arm.usat
  121038, // llvm.arm.usat16
  121054, // llvm.arm.usax
  121068, // llvm.arm.usub16
  121084, // llvm.arm.usub8
  121099, // llvm.arm.uxtab16
  121116, // llvm.arm.uxtb16
  121132, // llvm.arm.vcvtr
  121147, // llvm.arm.vcvtru
  121163, // llvm.bpf.btf.type.id
  121184, // llvm.bpf.compare
  121201, // llvm.bpf.getelementptr.and.load
  121233, // llvm.bpf.getelementptr.and.store
  121266, // llvm.bpf.load.byte
  121285, // llvm.bpf.load.half
  121304, // llvm.bpf.load.word
  121323, // llvm.bpf.passthrough
  121344, // llvm.bpf.preserve.enum.value
  121373, // llvm.bpf.preserve.field.info
  121402, // llvm.bpf.preserve.type.info
  121430, // llvm.bpf.pseudo
  121446, // llvm.dx.all
  121458, // llvm.dx.any
  121470, // llvm.dx.asdouble
  121487, // llvm.dx.cross
  121501, // llvm.dx.ddx.coarse
  121520, // llvm.dx.ddx.fine
  121537, // llvm.dx.ddy.coarse
  121556, // llvm.dx.ddy.fine
  121573, // llvm.dx.degrees
  121589, // llvm.dx.discard
  121605, // llvm.dx.dot2
  121618, // llvm.dx.dot2add
  121634, // llvm.dx.dot3
  121647, // llvm.dx.dot4
  121660, // llvm.dx.dot4add.i8packed
  121685, // llvm.dx.dot4add.u8packed
  121710, // llvm.dx.fdot
  121723, // llvm.dx.firstbitlow
  121743, // llvm.dx.firstbitshigh
  121765, // llvm.dx.firstbituhigh
  121787, // llvm.dx.flattened.thread.id.in.group
  121824, // llvm.dx.frac
  121837, // llvm.dx.group.id
  121854, // llvm.dx.group.memory.barrier.with.group.sync
  121899, // llvm.dx.imad
  121912, // llvm.dx.isinf
  121926, // llvm.dx.isnan
  121940, // llvm.dx.legacyf16tof32
  121963, // llvm.dx.lerp
  121976, // llvm.dx.load.input
  121995, // llvm.dx.nclamp
  122010, // llvm.dx.normalize
  122028, // llvm.dx.radians
  122044, // llvm.dx.resource.casthandle
  122072, // llvm.dx.resource.getdimensions.x
  122105, // llvm.dx.resource.getpointer
  122133, // llvm.dx.resource.handlefrombinding
  122168, // llvm.dx.resource.handlefromimplicitbinding
  122211, // llvm.dx.resource.load.cbufferrow.2
  122246, // llvm.dx.resource.load.cbufferrow.4
  122281, // llvm.dx.resource.load.cbufferrow.8
  122316, // llvm.dx.resource.load.rawbuffer
  122348, // llvm.dx.resource.load.typedbuffer
  122382, // llvm.dx.resource.nonuniformindex
  122415, // llvm.dx.resource.store.rawbuffer
  122448, // llvm.dx.resource.store.typedbuffer
  122483, // llvm.dx.resource.updatecounter
  122514, // llvm.dx.rsqrt
  122528, // llvm.dx.saturate
  122545, // llvm.dx.sclamp
  122560, // llvm.dx.sdot
  122573, // llvm.dx.sign
  122586, // llvm.dx.splitdouble
  122606, // llvm.dx.step
  122619, // llvm.dx.store.output
  122640, // llvm.dx.thread.id
  122658, // llvm.dx.thread.id.in.group
  122685, // llvm.dx.uclamp
  122700, // llvm.dx.udot
  122713, // llvm.dx.umad
  122726, // llvm.dx.wave.active.countbits
  122756, // llvm.dx.wave.all
  122773, // llvm.dx.wave.any
  122790, // llvm.dx.wave.ballot
  122810, // llvm.dx.wave.get.lane.count
  122838, // llvm.dx.wave.getlaneindex
  122864, // llvm.dx.wave.is.first.lane
  122891, // llvm.dx.wave.readlane
  122913, // llvm.dx.wave.reduce.max
  122937, // llvm.dx.wave.reduce.min
  122961, // llvm.dx.wave.reduce.sum
  122985, // llvm.dx.wave.reduce.umax
  123010, // llvm.dx.wave.reduce.umin
  123035, // llvm.dx.wave.reduce.usum
  123060, // llvm.hexagon.A2.abs
  123080, // llvm.hexagon.A2.absp
  123101, // llvm.hexagon.A2.abssat
  123124, // llvm.hexagon.A2.add
  123144, // llvm.hexagon.A2.addh.h16.hh
  123172, // llvm.hexagon.A2.addh.h16.hl
  123200, // llvm.hexagon.A2.addh.h16.lh
  123228, // llvm.hexagon.A2.addh.h16.ll
  123256, // llvm.hexagon.A2.addh.h16.sat.hh
  123288, // llvm.hexagon.A2.addh.h16.sat.hl
  123320, // llvm.hexagon.A2.addh.h16.sat.lh
  123352, // llvm.hexagon.A2.addh.h16.sat.ll
  123384, // llvm.hexagon.A2.addh.l16.hl
  123412, // llvm.hexagon.A2.addh.l16.ll
  123440, // llvm.hexagon.A2.addh.l16.sat.hl
  123472, // llvm.hexagon.A2.addh.l16.sat.ll
  123504, // llvm.hexagon.A2.addi
  123525, // llvm.hexagon.A2.addp
  123546, // llvm.hexagon.A2.addpsat
  123570, // llvm.hexagon.A2.addsat
  123593, // llvm.hexagon.A2.addsp
  123615, // llvm.hexagon.A2.and
  123635, // llvm.hexagon.A2.andir
  123657, // llvm.hexagon.A2.andp
  123678, // llvm.hexagon.A2.aslh
  123699, // llvm.hexagon.A2.asrh
  123720, // llvm.hexagon.A2.combine.hh
  123747, // llvm.hexagon.A2.combine.hl
  123774, // llvm.hexagon.A2.combine.lh
  123801, // llvm.hexagon.A2.combine.ll
  123828, // llvm.hexagon.A2.combineii
  123854, // llvm.hexagon.A2.combinew
  123879, // llvm.hexagon.A2.max
  123899, // llvm.hexagon.A2.maxp
  123920, // llvm.hexagon.A2.maxu
  123941, // llvm.hexagon.A2.maxup
  123963, // llvm.hexagon.A2.min
  123983, // llvm.hexagon.A2.minp
  124004, // llvm.hexagon.A2.minu
  124025, // llvm.hexagon.A2.minup
  124047, // llvm.hexagon.A2.neg
  124067, // llvm.hexagon.A2.negp
  124088, // llvm.hexagon.A2.negsat
  124111, // llvm.hexagon.A2.not
  124131, // llvm.hexagon.A2.notp
  124152, // llvm.hexagon.A2.or
  124171, // llvm.hexagon.A2.orir
  124192, // llvm.hexagon.A2.orp
  124212, // llvm.hexagon.A2.roundsat
  124237, // llvm.hexagon.A2.sat
  124257, // llvm.hexagon.A2.satb
  124278, // llvm.hexagon.A2.sath
  124299, // llvm.hexagon.A2.satub
  124321, // llvm.hexagon.A2.satuh
  124343, // llvm.hexagon.A2.sub
  124363, // llvm.hexagon.A2.subh.h16.hh
  124391, // llvm.hexagon.A2.subh.h16.hl
  124419, // llvm.hexagon.A2.subh.h16.lh
  124447, // llvm.hexagon.A2.subh.h16.ll
  124475, // llvm.hexagon.A2.subh.h16.sat.hh
  124507, // llvm.hexagon.A2.subh.h16.sat.hl
  124539, // llvm.hexagon.A2.subh.h16.sat.lh
  124571, // llvm.hexagon.A2.subh.h16.sat.ll
  124603, // llvm.hexagon.A2.subh.l16.hl
  124631, // llvm.hexagon.A2.subh.l16.ll
  124659, // llvm.hexagon.A2.subh.l16.sat.hl
  124691, // llvm.hexagon.A2.subh.l16.sat.ll
  124723, // llvm.hexagon.A2.subp
  124744, // llvm.hexagon.A2.subri
  124766, // llvm.hexagon.A2.subsat
  124789, // llvm.hexagon.A2.svaddh
  124812, // llvm.hexagon.A2.svaddhs
  124836, // llvm.hexagon.A2.svadduhs
  124861, // llvm.hexagon.A2.svavgh
  124884, // llvm.hexagon.A2.svavghs
  124908, // llvm.hexagon.A2.svnavgh
  124932, // llvm.hexagon.A2.svsubh
  124955, // llvm.hexagon.A2.svsubhs
  124979, // llvm.hexagon.A2.svsubuhs
  125004, // llvm.hexagon.A2.swiz
  125025, // llvm.hexagon.A2.sxtb
  125046, // llvm.hexagon.A2.sxth
  125067, // llvm.hexagon.A2.sxtw
  125088, // llvm.hexagon.A2.tfr
  125108, // llvm.hexagon.A2.tfrih
  125130, // llvm.hexagon.A2.tfril
  125152, // llvm.hexagon.A2.tfrp
  125173, // llvm.hexagon.A2.tfrpi
  125195, // llvm.hexagon.A2.tfrsi
  125217, // llvm.hexagon.A2.vabsh
  125239, // llvm.hexagon.A2.vabshsat
  125264, // llvm.hexagon.A2.vabsw
  125286, // llvm.hexagon.A2.vabswsat
  125311, // llvm.hexagon.A2.vaddb.map
  125337, // llvm.hexagon.A2.vaddh
  125359, // llvm.hexagon.A2.vaddhs
  125382, // llvm.hexagon.A2.vaddub
  125405, // llvm.hexagon.A2.vaddubs
  125429, // llvm.hexagon.A2.vadduhs
  125453, // llvm.hexagon.A2.vaddw
  125475, // llvm.hexagon.A2.vaddws
  125498, // llvm.hexagon.A2.vavgh
  125520, // llvm.hexagon.A2.vavghcr
  125544, // llvm.hexagon.A2.vavghr
  125567, // llvm.hexagon.A2.vavgub
  125590, // llvm.hexagon.A2.vavgubr
  125614, // llvm.hexagon.A2.vavguh
  125637, // llvm.hexagon.A2.vavguhr
  125661, // llvm.hexagon.A2.vavguw
  125684, // llvm.hexagon.A2.vavguwr
  125708, // llvm.hexagon.A2.vavgw
  125730, // llvm.hexagon.A2.vavgwcr
  125754, // llvm.hexagon.A2.vavgwr
  125777, // llvm.hexagon.A2.vcmpbeq
  125801, // llvm.hexagon.A2.vcmpbgtu
  125826, // llvm.hexagon.A2.vcmpheq
  125850, // llvm.hexagon.A2.vcmphgt
  125874, // llvm.hexagon.A2.vcmphgtu
  125899, // llvm.hexagon.A2.vcmpweq
  125923, // llvm.hexagon.A2.vcmpwgt
  125947, // llvm.hexagon.A2.vcmpwgtu
  125972, // llvm.hexagon.A2.vconj
  125994, // llvm.hexagon.A2.vmaxb
  126016, // llvm.hexagon.A2.vmaxh
  126038, // llvm.hexagon.A2.vmaxub
  126061, // llvm.hexagon.A2.vmaxuh
  126084, // llvm.hexagon.A2.vmaxuw
  126107, // llvm.hexagon.A2.vmaxw
  126129, // llvm.hexagon.A2.vminb
  126151, // llvm.hexagon.A2.vminh
  126173, // llvm.hexagon.A2.vminub
  126196, // llvm.hexagon.A2.vminuh
  126219, // llvm.hexagon.A2.vminuw
  126242, // llvm.hexagon.A2.vminw
  126264, // llvm.hexagon.A2.vnavgh
  126287, // llvm.hexagon.A2.vnavghcr
  126312, // llvm.hexagon.A2.vnavghr
  126336, // llvm.hexagon.A2.vnavgw
  126359, // llvm.hexagon.A2.vnavgwcr
  126384, // llvm.hexagon.A2.vnavgwr
  126408, // llvm.hexagon.A2.vraddub
  126432, // llvm.hexagon.A2.vraddub.acc
  126460, // llvm.hexagon.A2.vrsadub
  126484, // llvm.hexagon.A2.vrsadub.acc
  126512, // llvm.hexagon.A2.vsubb.map
  126538, // llvm.hexagon.A2.vsubh
  126560, // llvm.hexagon.A2.vsubhs
  126583, // llvm.hexagon.A2.vsubub
  126606, // llvm.hexagon.A2.vsububs
  126630, // llvm.hexagon.A2.vsubuhs
  126654, // llvm.hexagon.A2.vsubw
  126676, // llvm.hexagon.A2.vsubws
  126699, // llvm.hexagon.A2.xor
  126719, // llvm.hexagon.A2.xorp
  126740, // llvm.hexagon.A2.zxtb
  126761, // llvm.hexagon.A2.zxth
  126782, // llvm.hexagon.A4.andn
  126803, // llvm.hexagon.A4.andnp
  126825, // llvm.hexagon.A4.bitsplit
  126850, // llvm.hexagon.A4.bitspliti
  126876, // llvm.hexagon.A4.boundscheck
  126904, // llvm.hexagon.A4.cmpbeq
  126927, // llvm.hexagon.A4.cmpbeqi
  126951, // llvm.hexagon.A4.cmpbgt
  126974, // llvm.hexagon.A4.cmpbgti
  126998, // llvm.hexagon.A4.cmpbgtu
  127022, // llvm.hexagon.A4.cmpbgtui
  127047, // llvm.hexagon.A4.cmpheq
  127070, // llvm.hexagon.A4.cmpheqi
  127094, // llvm.hexagon.A4.cmphgt
  127117, // llvm.hexagon.A4.cmphgti
  127141, // llvm.hexagon.A4.cmphgtu
  127165, // llvm.hexagon.A4.cmphgtui
  127190, // llvm.hexagon.A4.combineir
  127216, // llvm.hexagon.A4.combineri
  127242, // llvm.hexagon.A4.cround.ri
  127268, // llvm.hexagon.A4.cround.rr
  127294, // llvm.hexagon.A4.modwrapu
  127319, // llvm.hexagon.A4.orn
  127339, // llvm.hexagon.A4.ornp
  127360, // llvm.hexagon.A4.rcmpeq
  127383, // llvm.hexagon.A4.rcmpeqi
  127407, // llvm.hexagon.A4.rcmpneq
  127431, // llvm.hexagon.A4.rcmpneqi
  127456, // llvm.hexagon.A4.round.ri
  127481, // llvm.hexagon.A4.round.ri.sat
  127510, // llvm.hexagon.A4.round.rr
  127535, // llvm.hexagon.A4.round.rr.sat
  127564, // llvm.hexagon.A4.tlbmatch
  127589, // llvm.hexagon.A4.vcmpbeq.any
  127617, // llvm.hexagon.A4.vcmpbeqi
  127642, // llvm.hexagon.A4.vcmpbgt
  127666, // llvm.hexagon.A4.vcmpbgti
  127691, // llvm.hexagon.A4.vcmpbgtui
  127717, // llvm.hexagon.A4.vcmpheqi
  127742, // llvm.hexagon.A4.vcmphgti
  127767, // llvm.hexagon.A4.vcmphgtui
  127793, // llvm.hexagon.A4.vcmpweqi
  127818, // llvm.hexagon.A4.vcmpwgti
  127843, // llvm.hexagon.A4.vcmpwgtui
  127869, // llvm.hexagon.A4.vrmaxh
  127892, // llvm.hexagon.A4.vrmaxuh
  127916, // llvm.hexagon.A4.vrmaxuw
  127940, // llvm.hexagon.A4.vrmaxw
  127963, // llvm.hexagon.A4.vrminh
  127986, // llvm.hexagon.A4.vrminuh
  128010, // llvm.hexagon.A4.vrminuw
  128034, // llvm.hexagon.A4.vrminw
  128057, // llvm.hexagon.A5.vaddhubs
  128082, // llvm.hexagon.A6.vcmpbeq.notany
  128113, // llvm.hexagon.A7.clip
  128134, // llvm.hexagon.A7.croundd.ri
  128161, // llvm.hexagon.A7.croundd.rr
  128188, // llvm.hexagon.A7.vclip
  128210, // llvm.hexagon.C2.all8
  128231, // llvm.hexagon.C2.and
  128251, // llvm.hexagon.C2.andn
  128272, // llvm.hexagon.C2.any8
  128293, // llvm.hexagon.C2.bitsclr
  128317, // llvm.hexagon.C2.bitsclri
  128342, // llvm.hexagon.C2.bitsset
  128366, // llvm.hexagon.C2.cmpeq
  128388, // llvm.hexagon.C2.cmpeqi
  128411, // llvm.hexagon.C2.cmpeqp
  128434, // llvm.hexagon.C2.cmpgei
  128457, // llvm.hexagon.C2.cmpgeui
  128481, // llvm.hexagon.C2.cmpgt
  128503, // llvm.hexagon.C2.cmpgti
  128526, // llvm.hexagon.C2.cmpgtp
  128549, // llvm.hexagon.C2.cmpgtu
  128572, // llvm.hexagon.C2.cmpgtui
  128596, // llvm.hexagon.C2.cmpgtup
  128620, // llvm.hexagon.C2.cmplt
  128642, // llvm.hexagon.C2.cmpltu
  128665, // llvm.hexagon.C2.mask
  128686, // llvm.hexagon.C2.mux
  128706, // llvm.hexagon.C2.muxii
  128728, // llvm.hexagon.C2.muxir
  128750, // llvm.hexagon.C2.muxri
  128772, // llvm.hexagon.C2.not
  128792, // llvm.hexagon.C2.or
  128811, // llvm.hexagon.C2.orn
  128831, // llvm.hexagon.C2.pxfer.map
  128857, // llvm.hexagon.C2.tfrpr
  128879, // llvm.hexagon.C2.tfrrp
  128901, // llvm.hexagon.C2.vitpack
  128925, // llvm.hexagon.C2.vmux
  128946, // llvm.hexagon.C2.xor
  128966, // llvm.hexagon.C4.and.and
  128990, // llvm.hexagon.C4.and.andn
  129015, // llvm.hexagon.C4.and.or
  129038, // llvm.hexagon.C4.and.orn
  129062, // llvm.hexagon.C4.cmplte
  129085, // llvm.hexagon.C4.cmpltei
  129109, // llvm.hexagon.C4.cmplteu
  129133, // llvm.hexagon.C4.cmplteui
  129158, // llvm.hexagon.C4.cmpneq
  129181, // llvm.hexagon.C4.cmpneqi
  129205, // llvm.hexagon.C4.fastcorner9
  129233, // llvm.hexagon.C4.fastcorner9.not
  129265, // llvm.hexagon.C4.nbitsclr
  129290, // llvm.hexagon.C4.nbitsclri
  129316, // llvm.hexagon.C4.nbitsset
  129341, // llvm.hexagon.C4.or.and
  129364, // llvm.hexagon.C4.or.andn
  129388, // llvm.hexagon.C4.or.or
  129410, // llvm.hexagon.C4.or.orn
  129433, // llvm.hexagon.F2.conv.d2df
  129459, // llvm.hexagon.F2.conv.d2sf
  129485, // llvm.hexagon.F2.conv.df2d
  129511, // llvm.hexagon.F2.conv.df2d.chop
  129542, // llvm.hexagon.F2.conv.df2sf
  129569, // llvm.hexagon.F2.conv.df2ud
  129596, // llvm.hexagon.F2.conv.df2ud.chop
  129628, // llvm.hexagon.F2.conv.df2uw
  129655, // llvm.hexagon.F2.conv.df2uw.chop
  129687, // llvm.hexagon.F2.conv.df2w
  129713, // llvm.hexagon.F2.conv.df2w.chop
  129744, // llvm.hexagon.F2.conv.sf2d
  129770, // llvm.hexagon.F2.conv.sf2d.chop
  129801, // llvm.hexagon.F2.conv.sf2df
  129828, // llvm.hexagon.F2.conv.sf2ud
  129855, // llvm.hexagon.F2.conv.sf2ud.chop
  129887, // llvm.hexagon.F2.conv.sf2uw
  129914, // llvm.hexagon.F2.conv.sf2uw.chop
  129946, // llvm.hexagon.F2.conv.sf2w
  129972, // llvm.hexagon.F2.conv.sf2w.chop
  130003, // llvm.hexagon.F2.conv.ud2df
  130030, // llvm.hexagon.F2.conv.ud2sf
  130057, // llvm.hexagon.F2.conv.uw2df
  130084, // llvm.hexagon.F2.conv.uw2sf
  130111, // llvm.hexagon.F2.conv.w2df
  130137, // llvm.hexagon.F2.conv.w2sf
  130163, // llvm.hexagon.F2.dfadd
  130185, // llvm.hexagon.F2.dfclass
  130209, // llvm.hexagon.F2.dfcmpeq
  130233, // llvm.hexagon.F2.dfcmpge
  130257, // llvm.hexagon.F2.dfcmpgt
  130281, // llvm.hexagon.F2.dfcmpuo
  130305, // llvm.hexagon.F2.dfimm.n
  130329, // llvm.hexagon.F2.dfimm.p
  130353, // llvm.hexagon.F2.dfmax
  130375, // llvm.hexagon.F2.dfmin
  130397, // llvm.hexagon.F2.dfmpyfix
  130422, // llvm.hexagon.F2.dfmpyhh
  130446, // llvm.hexagon.F2.dfmpylh
  130470, // llvm.hexagon.F2.dfmpyll
  130494, // llvm.hexagon.F2.dfsub
  130516, // llvm.hexagon.F2.sfadd
  130538, // llvm.hexagon.F2.sfclass
  130562, // llvm.hexagon.F2.sfcmpeq
  130586, // llvm.hexagon.F2.sfcmpge
  130610, // llvm.hexagon.F2.sfcmpgt
  130634, // llvm.hexagon.F2.sfcmpuo
  130658, // llvm.hexagon.F2.sffixupd
  130683, // llvm.hexagon.F2.sffixupn
  130708, // llvm.hexagon.F2.sffixupr
  130733, // llvm.hexagon.F2.sffma
  130755, // llvm.hexagon.F2.sffma.lib
  130781, // llvm.hexagon.F2.sffma.sc
  130806, // llvm.hexagon.F2.sffms
  130828, // llvm.hexagon.F2.sffms.lib
  130854, // llvm.hexagon.F2.sfimm.n
  130878, // llvm.hexagon.F2.sfimm.p
  130902, // llvm.hexagon.F2.sfmax
  130924, // llvm.hexagon.F2.sfmin
  130946, // llvm.hexagon.F2.sfmpy
  130968, // llvm.hexagon.F2.sfsub
  130990, // llvm.hexagon.L2.loadrb.pbr
  131017, // llvm.hexagon.L2.loadrb.pci
  131044, // llvm.hexagon.L2.loadrb.pcr
  131071, // llvm.hexagon.L2.loadrd.pbr
  131098, // llvm.hexagon.L2.loadrd.pci
  131125, // llvm.hexagon.L2.loadrd.pcr
  131152, // llvm.hexagon.L2.loadrh.pbr
  131179, // llvm.hexagon.L2.loadrh.pci
  131206, // llvm.hexagon.L2.loadrh.pcr
  131233, // llvm.hexagon.L2.loadri.pbr
  131260, // llvm.hexagon.L2.loadri.pci
  131287, // llvm.hexagon.L2.loadri.pcr
  131314, // llvm.hexagon.L2.loadrub.pbr
  131342, // llvm.hexagon.L2.loadrub.pci
  131370, // llvm.hexagon.L2.loadrub.pcr
  131398, // llvm.hexagon.L2.loadruh.pbr
  131426, // llvm.hexagon.L2.loadruh.pci
  131454, // llvm.hexagon.L2.loadruh.pcr
  131482, // llvm.hexagon.L2.loadw.locked
  131511, // llvm.hexagon.L4.loadd.locked
  131540, // llvm.hexagon.M2.acci
  131561, // llvm.hexagon.M2.accii
  131583, // llvm.hexagon.M2.cmaci.s0
  131608, // llvm.hexagon.M2.cmacr.s0
  131633, // llvm.hexagon.M2.cmacs.s0
  131658, // llvm.hexagon.M2.cmacs.s1
  131683, // llvm.hexagon.M2.cmacsc.s0
  131709, // llvm.hexagon.M2.cmacsc.s1
  131735, // llvm.hexagon.M2.cmpyi.s0
  131760, // llvm.hexagon.M2.cmpyr.s0
  131785, // llvm.hexagon.M2.cmpyrs.s0
  131811, // llvm.hexagon.M2.cmpyrs.s1
  131837, // llvm.hexagon.M2.cmpyrsc.s0
  131864, // llvm.hexagon.M2.cmpyrsc.s1
  131891, // llvm.hexagon.M2.cmpys.s0
  131916, // llvm.hexagon.M2.cmpys.s1
  131941, // llvm.hexagon.M2.cmpysc.s0
  131967, // llvm.hexagon.M2.cmpysc.s1
  131993, // llvm.hexagon.M2.cnacs.s0
  132018, // llvm.hexagon.M2.cnacs.s1
  132043, // llvm.hexagon.M2.cnacsc.s0
  132069, // llvm.hexagon.M2.cnacsc.s1
  132095, // llvm.hexagon.M2.dpmpyss.acc.s0
  132126, // llvm.hexagon.M2.dpmpyss.nac.s0
  132157, // llvm.hexagon.M2.dpmpyss.rnd.s0
  132188, // llvm.hexagon.M2.dpmpyss.s0
  132215, // llvm.hexagon.M2.dpmpyuu.acc.s0
  132246, // llvm.hexagon.M2.dpmpyuu.nac.s0
  132277, // llvm.hexagon.M2.dpmpyuu.s0
  132304, // llvm.hexagon.M2.hmmpyh.rs1
  132331, // llvm.hexagon.M2.hmmpyh.s1
  132357, // llvm.hexagon.M2.hmmpyl.rs1
  132384, // llvm.hexagon.M2.hmmpyl.s1
  132410, // llvm.hexagon.M2.maci
  132431, // llvm.hexagon.M2.macsin
  132454, // llvm.hexagon.M2.macsip
  132477, // llvm.hexagon.M2.mmachs.rs0
  132504, // llvm.hexagon.M2.mmachs.rs1
  132531, // llvm.hexagon.M2.mmachs.s0
  132557, // llvm.hexagon.M2.mmachs.s1
  132583, // llvm.hexagon.M2.mmacls.rs0
  132610, // llvm.hexagon.M2.mmacls.rs1
  132637, // llvm.hexagon.M2.mmacls.s0
  132663, // llvm.hexagon.M2.mmacls.s1
  132689, // llvm.hexagon.M2.mmacuhs.rs0
  132717, // llvm.hexagon.M2.mmacuhs.rs1
  132745, // llvm.hexagon.M2.mmacuhs.s0
  132772, // llvm.hexagon.M2.mmacuhs.s1
  132799, // llvm.hexagon.M2.mmaculs.rs0
  132827, // llvm.hexagon.M2.mmaculs.rs1
  132855, // llvm.hexagon.M2.mmaculs.s0
  132882, // llvm.hexagon.M2.mmaculs.s1
  132909, // llvm.hexagon.M2.mmpyh.rs0
  132935, // llvm.hexagon.M2.mmpyh.rs1
  132961, // llvm.hexagon.M2.mmpyh.s0
  132986, // llvm.hexagon.M2.mmpyh.s1
  133011, // llvm.hexagon.M2.mmpyl.rs0
  133037, // llvm.hexagon.M2.mmpyl.rs1
  133063, // llvm.hexagon.M2.mmpyl.s0
  133088, // llvm.hexagon.M2.mmpyl.s1
  133113, // llvm.hexagon.M2.mmpyuh.rs0
  133140, // llvm.hexagon.M2.mmpyuh.rs1
  133167, // llvm.hexagon.M2.mmpyuh.s0
  133193, // llvm.hexagon.M2.mmpyuh.s1
  133219, // llvm.hexagon.M2.mmpyul.rs0
  133246, // llvm.hexagon.M2.mmpyul.rs1
  133273, // llvm.hexagon.M2.mmpyul.s0
  133299, // llvm.hexagon.M2.mmpyul.s1
  133325, // llvm.hexagon.M2.mnaci
  133347, // llvm.hexagon.M2.mpy.acc.hh.s0
  133377, // llvm.hexagon.M2.mpy.acc.hh.s1
  133407, // llvm.hexagon.M2.mpy.acc.hl.s0
  133437, // llvm.hexagon.M2.mpy.acc.hl.s1
  133467, // llvm.hexagon.M2.mpy.acc.lh.s0
  133497, // llvm.hexagon.M2.mpy.acc.lh.s1
  133527, // llvm.hexagon.M2.mpy.acc.ll.s0
  133557, // llvm.hexagon.M2.mpy.acc.ll.s1
  133587, // llvm.hexagon.M2.mpy.acc.sat.hh.s0
  133621, // llvm.hexagon.M2.mpy.acc.sat.hh.s1
  133655, // llvm.hexagon.M2.mpy.acc.sat.hl.s0
  133689, // llvm.hexagon.M2.mpy.acc.sat.hl.s1
  133723, // llvm.hexagon.M2.mpy.acc.sat.lh.s0
  133757, // llvm.hexagon.M2.mpy.acc.sat.lh.s1
  133791, // llvm.hexagon.M2.mpy.acc.sat.ll.s0
  133825, // llvm.hexagon.M2.mpy.acc.sat.ll.s1
  133859, // llvm.hexagon.M2.mpy.hh.s0
  133885, // llvm.hexagon.M2.mpy.hh.s1
  133911, // llvm.hexagon.M2.mpy.hl.s0
  133937, // llvm.hexagon.M2.mpy.hl.s1
  133963, // llvm.hexagon.M2.mpy.lh.s0
  133989, // llvm.hexagon.M2.mpy.lh.s1
  134015, // llvm.hexagon.M2.mpy.ll.s0
  134041, // llvm.hexagon.M2.mpy.ll.s1
  134067, // llvm.hexagon.M2.mpy.nac.hh.s0
  134097, // llvm.hexagon.M2.mpy.nac.hh.s1
  134127, // llvm.hexagon.M2.mpy.nac.hl.s0
  134157, // llvm.hexagon.M2.mpy.nac.hl.s1
  134187, // llvm.hexagon.M2.mpy.nac.lh.s0
  134217, // llvm.hexagon.M2.mpy.nac.lh.s1
  134247, // llvm.hexagon.M2.mpy.nac.ll.s0
  134277, // llvm.hexagon.M2.mpy.nac.ll.s1
  134307, // llvm.hexagon.M2.mpy.nac.sat.hh.s0
  134341, // llvm.hexagon.M2.mpy.nac.sat.hh.s1
  134375, // llvm.hexagon.M2.mpy.nac.sat.hl.s0
  134409, // llvm.hexagon.M2.mpy.nac.sat.hl.s1
  134443, // llvm.hexagon.M2.mpy.nac.sat.lh.s0
  134477, // llvm.hexagon.M2.mpy.nac.sat.lh.s1
  134511, // llvm.hexagon.M2.mpy.nac.sat.ll.s0
  134545, // llvm.hexagon.M2.mpy.nac.sat.ll.s1
  134579, // llvm.hexagon.M2.mpy.rnd.hh.s0
  134609, // llvm.hexagon.M2.mpy.rnd.hh.s1
  134639, // llvm.hexagon.M2.mpy.rnd.hl.s0
  134669, // llvm.hexagon.M2.mpy.rnd.hl.s1
  134699, // llvm.hexagon.M2.mpy.rnd.lh.s0
  134729, // llvm.hexagon.M2.mpy.rnd.lh.s1
  134759, // llvm.hexagon.M2.mpy.rnd.ll.s0
  134789, // llvm.hexagon.M2.mpy.rnd.ll.s1
  134819, // llvm.hexagon.M2.mpy.sat.hh.s0
  134849, // llvm.hexagon.M2.mpy.sat.hh.s1
  134879, // llvm.hexagon.M2.mpy.sat.hl.s0
  134909, // llvm.hexagon.M2.mpy.sat.hl.s1
  134939, // llvm.hexagon.M2.mpy.sat.lh.s0
  134969, // llvm.hexagon.M2.mpy.sat.lh.s1
  134999, // llvm.hexagon.M2.mpy.sat.ll.s0
  135029, // llvm.hexagon.M2.mpy.sat.ll.s1
  135059, // llvm.hexagon.M2.mpy.sat.rnd.hh.s0
  135093, // llvm.hexagon.M2.mpy.sat.rnd.hh.s1
  135127, // llvm.hexagon.M2.mpy.sat.rnd.hl.s0
  135161, // llvm.hexagon.M2.mpy.sat.rnd.hl.s1
  135195, // llvm.hexagon.M2.mpy.sat.rnd.lh.s0
  135229, // llvm.hexagon.M2.mpy.sat.rnd.lh.s1
  135263, // llvm.hexagon.M2.mpy.sat.rnd.ll.s0
  135297, // llvm.hexagon.M2.mpy.sat.rnd.ll.s1
  135331, // llvm.hexagon.M2.mpy.up
  135354, // llvm.hexagon.M2.mpy.up.s1
  135380, // llvm.hexagon.M2.mpy.up.s1.sat
  135410, // llvm.hexagon.M2.mpyd.acc.hh.s0
  135441, // llvm.hexagon.M2.mpyd.acc.hh.s1
  135472, // llvm.hexagon.M2.mpyd.acc.hl.s0
  135503, // llvm.hexagon.M2.mpyd.acc.hl.s1
  135534, // llvm.hexagon.M2.mpyd.acc.lh.s0
  135565, // llvm.hexagon.M2.mpyd.acc.lh.s1
  135596, // llvm.hexagon.M2.mpyd.acc.ll.s0
  135627, // llvm.hexagon.M2.mpyd.acc.ll.s1
  135658, // llvm.hexagon.M2.mpyd.hh.s0
  135685, // llvm.hexagon.M2.mpyd.hh.s1
  135712, // llvm.hexagon.M2.mpyd.hl.s0
  135739, // llvm.hexagon.M2.mpyd.hl.s1
  135766, // llvm.hexagon.M2.mpyd.lh.s0
  135793, // llvm.hexagon.M2.mpyd.lh.s1
  135820, // llvm.hexagon.M2.mpyd.ll.s0
  135847, // llvm.hexagon.M2.mpyd.ll.s1
  135874, // llvm.hexagon.M2.mpyd.nac.hh.s0
  135905, // llvm.hexagon.M2.mpyd.nac.hh.s1
  135936, // llvm.hexagon.M2.mpyd.nac.hl.s0
  135967, // llvm.hexagon.M2.mpyd.nac.hl.s1
  135998, // llvm.hexagon.M2.mpyd.nac.lh.s0
  136029, // llvm.hexagon.M2.mpyd.nac.lh.s1
  136060, // llvm.hexagon.M2.mpyd.nac.ll.s0
  136091, // llvm.hexagon.M2.mpyd.nac.ll.s1
  136122, // llvm.hexagon.M2.mpyd.rnd.hh.s0
  136153, // llvm.hexagon.M2.mpyd.rnd.hh.s1
  136184, // llvm.hexagon.M2.mpyd.rnd.hl.s0
  136215, // llvm.hexagon.M2.mpyd.rnd.hl.s1
  136246, // llvm.hexagon.M2.mpyd.rnd.lh.s0
  136277, // llvm.hexagon.M2.mpyd.rnd.lh.s1
  136308, // llvm.hexagon.M2.mpyd.rnd.ll.s0
  136339, // llvm.hexagon.M2.mpyd.rnd.ll.s1
  136370, // llvm.hexagon.M2.mpyi
  136391, // llvm.hexagon.M2.mpysmi
  136414, // llvm.hexagon.M2.mpysu.up
  136439, // llvm.hexagon.M2.mpyu.acc.hh.s0
  136470, // llvm.hexagon.M2.mpyu.acc.hh.s1
  136501, // llvm.hexagon.M2.mpyu.acc.hl.s0
  136532, // llvm.hexagon.M2.mpyu.acc.hl.s1
  136563, // llvm.hexagon.M2.mpyu.acc.lh.s0
  136594, // llvm.hexagon.M2.mpyu.acc.lh.s1
  136625, // llvm.hexagon.M2.mpyu.acc.ll.s0
  136656, // llvm.hexagon.M2.mpyu.acc.ll.s1
  136687, // llvm.hexagon.M2.mpyu.hh.s0
  136714, // llvm.hexagon.M2.mpyu.hh.s1
  136741, // llvm.hexagon.M2.mpyu.hl.s0
  136768, // llvm.hexagon.M2.mpyu.hl.s1
  136795, // llvm.hexagon.M2.mpyu.lh.s0
  136822, // llvm.hexagon.M2.mpyu.lh.s1
  136849, // llvm.hexagon.M2.mpyu.ll.s0
  136876, // llvm.hexagon.M2.mpyu.ll.s1
  136903, // llvm.hexagon.M2.mpyu.nac.hh.s0
  136934, // llvm.hexagon.M2.mpyu.nac.hh.s1
  136965, // llvm.hexagon.M2.mpyu.nac.hl.s0
  136996, // llvm.hexagon.M2.mpyu.nac.hl.s1
  137027, // llvm.hexagon.M2.mpyu.nac.lh.s0
  137058, // llvm.hexagon.M2.mpyu.nac.lh.s1
  137089, // llvm.hexagon.M2.mpyu.nac.ll.s0
  137120, // llvm.hexagon.M2.mpyu.nac.ll.s1
  137151, // llvm.hexagon.M2.mpyu.up
  137175, // llvm.hexagon.M2.mpyud.acc.hh.s0
  137207, // llvm.hexagon.M2.mpyud.acc.hh.s1
  137239, // llvm.hexagon.M2.mpyud.acc.hl.s0
  137271, // llvm.hexagon.M2.mpyud.acc.hl.s1
  137303, // llvm.hexagon.M2.mpyud.acc.lh.s0
  137335, // llvm.hexagon.M2.mpyud.acc.lh.s1
  137367, // llvm.hexagon.M2.mpyud.acc.ll.s0
  137399, // llvm.hexagon.M2.mpyud.acc.ll.s1
  137431, // llvm.hexagon.M2.mpyud.hh.s0
  137459, // llvm.hexagon.M2.mpyud.hh.s1
  137487, // llvm.hexagon.M2.mpyud.hl.s0
  137515, // llvm.hexagon.M2.mpyud.hl.s1
  137543, // llvm.hexagon.M2.mpyud.lh.s0
  137571, // llvm.hexagon.M2.mpyud.lh.s1
  137599, // llvm.hexagon.M2.mpyud.ll.s0
  137627, // llvm.hexagon.M2.mpyud.ll.s1
  137655, // llvm.hexagon.M2.mpyud.nac.hh.s0
  137687, // llvm.hexagon.M2.mpyud.nac.hh.s1
  137719, // llvm.hexagon.M2.mpyud.nac.hl.s0
  137751, // llvm.hexagon.M2.mpyud.nac.hl.s1
  137783, // llvm.hexagon.M2.mpyud.nac.lh.s0
  137815, // llvm.hexagon.M2.mpyud.nac.lh.s1
  137847, // llvm.hexagon.M2.mpyud.nac.ll.s0
  137879, // llvm.hexagon.M2.mpyud.nac.ll.s1
  137911, // llvm.hexagon.M2.mpyui
  137933, // llvm.hexagon.M2.nacci
  137955, // llvm.hexagon.M2.naccii
  137978, // llvm.hexagon.M2.subacc
  138001, // llvm.hexagon.M2.vabsdiffh
  138027, // llvm.hexagon.M2.vabsdiffw
  138053, // llvm.hexagon.M2.vcmac.s0.sat.i
  138084, // llvm.hexagon.M2.vcmac.s0.sat.r
  138115, // llvm.hexagon.M2.vcmpy.s0.sat.i
  138146, // llvm.hexagon.M2.vcmpy.s0.sat.r
  138177, // llvm.hexagon.M2.vcmpy.s1.sat.i
  138208, // llvm.hexagon.M2.vcmpy.s1.sat.r
  138239, // llvm.hexagon.M2.vdmacs.s0
  138265, // llvm.hexagon.M2.vdmacs.s1
  138291, // llvm.hexagon.M2.vdmpyrs.s0
  138318, // llvm.hexagon.M2.vdmpyrs.s1
  138345, // llvm.hexagon.M2.vdmpys.s0
  138371, // llvm.hexagon.M2.vdmpys.s1
  138397, // llvm.hexagon.M2.vmac2
  138419, // llvm.hexagon.M2.vmac2es
  138443, // llvm.hexagon.M2.vmac2es.s0
  138470, // llvm.hexagon.M2.vmac2es.s1
  138497, // llvm.hexagon.M2.vmac2s.s0
  138523, // llvm.hexagon.M2.vmac2s.s1
  138549, // llvm.hexagon.M2.vmac2su.s0
  138576, // llvm.hexagon.M2.vmac2su.s1
  138603, // llvm.hexagon.M2.vmpy2es.s0
  138630, // llvm.hexagon.M2.vmpy2es.s1
  138657, // llvm.hexagon.M2.vmpy2s.s0
  138683, // llvm.hexagon.M2.vmpy2s.s0pack
  138713, // llvm.hexagon.M2.vmpy2s.s1
  138739, // llvm.hexagon.M2.vmpy2s.s1pack
  138769, // llvm.hexagon.M2.vmpy2su.s0
  138796, // llvm.hexagon.M2.vmpy2su.s1
  138823, // llvm.hexagon.M2.vraddh
  138846, // llvm.hexagon.M2.vradduh
  138870, // llvm.hexagon.M2.vrcmaci.s0
  138897, // llvm.hexagon.M2.vrcmaci.s0c
  138925, // llvm.hexagon.M2.vrcmacr.s0
  138952, // llvm.hexagon.M2.vrcmacr.s0c
  138980, // llvm.hexagon.M2.vrcmpyi.s0
  139007, // llvm.hexagon.M2.vrcmpyi.s0c
  139035, // llvm.hexagon.M2.vrcmpyr.s0
  139062, // llvm.hexagon.M2.vrcmpyr.s0c
  139090, // llvm.hexagon.M2.vrcmpys.acc.s1
  139121, // llvm.hexagon.M2.vrcmpys.s1
  139148, // llvm.hexagon.M2.vrcmpys.s1rp
  139177, // llvm.hexagon.M2.vrmac.s0
  139202, // llvm.hexagon.M2.vrmpy.s0
  139227, // llvm.hexagon.M2.xor.xacc
  139252, // llvm.hexagon.M4.and.and
  139276, // llvm.hexagon.M4.and.andn
  139301, // llvm.hexagon.M4.and.or
  139324, // llvm.hexagon.M4.and.xor
  139348, // llvm.hexagon.M4.cmpyi.wh
  139373, // llvm.hexagon.M4.cmpyi.whc
  139399, // llvm.hexagon.M4.cmpyr.wh
  139424, // llvm.hexagon.M4.cmpyr.whc
  139450, // llvm.hexagon.M4.mac.up.s1.sat
  139480, // llvm.hexagon.M4.mpyri.addi
  139507, // llvm.hexagon.M4.mpyri.addr
  139534, // llvm.hexagon.M4.mpyri.addr.u2
  139564, // llvm.hexagon.M4.mpyrr.addi
  139591, // llvm.hexagon.M4.mpyrr.addr
  139618, // llvm.hexagon.M4.nac.up.s1.sat
  139648, // llvm.hexagon.M4.or.and
  139671, // llvm.hexagon.M4.or.andn
  139695, // llvm.hexagon.M4.or.or
  139717, // llvm.hexagon.M4.or.xor
  139740, // llvm.hexagon.M4.pmpyw
  139762, // llvm.hexagon.M4.pmpyw.acc
  139788, // llvm.hexagon.M4.vpmpyh
  139811, // llvm.hexagon.M4.vpmpyh.acc
  139838, // llvm.hexagon.M4.vrmpyeh.acc.s0
  139869, // llvm.hexagon.M4.vrmpyeh.acc.s1
  139900, // llvm.hexagon.M4.vrmpyeh.s0
  139927, // llvm.hexagon.M4.vrmpyeh.s1
  139954, // llvm.hexagon.M4.vrmpyoh.acc.s0
  139985, // llvm.hexagon.M4.vrmpyoh.acc.s1
  140016, // llvm.hexagon.M4.vrmpyoh.s0
  140043, // llvm.hexagon.M4.vrmpyoh.s1
  140070, // llvm.hexagon.M4.xor.and
  140094, // llvm.hexagon.M4.xor.andn
  140119, // llvm.hexagon.M4.xor.or
  140142, // llvm.hexagon.M4.xor.xacc
  140167, // llvm.hexagon.M5.vdmacbsu
  140192, // llvm.hexagon.M5.vdmpybsu
  140217, // llvm.hexagon.M5.vmacbsu
  140241, // llvm.hexagon.M5.vmacbuu
  140265, // llvm.hexagon.M5.vmpybsu
  140289, // llvm.hexagon.M5.vmpybuu
  140313, // llvm.hexagon.M5.vrmacbsu
  140338, // llvm.hexagon.M5.vrmacbuu
  140363, // llvm.hexagon.M5.vrmpybsu
  140388, // llvm.hexagon.M5.vrmpybuu
  140413, // llvm.hexagon.M6.vabsdiffb
  140439, // llvm.hexagon.M6.vabsdiffub
  140466, // llvm.hexagon.M7.dcmpyiw
  140490, // llvm.hexagon.M7.dcmpyiw.acc
  140518, // llvm.hexagon.M7.dcmpyiwc
  140543, // llvm.hexagon.M7.dcmpyiwc.acc
  140572, // llvm.hexagon.M7.dcmpyrw
  140596, // llvm.hexagon.M7.dcmpyrw.acc
  140624, // llvm.hexagon.M7.dcmpyrwc
  140649, // llvm.hexagon.M7.dcmpyrwc.acc
  140678, // llvm.hexagon.M7.vdmpy
  140700, // llvm.hexagon.M7.vdmpy.acc
  140726, // llvm.hexagon.M7.wcmpyiw
  140750, // llvm.hexagon.M7.wcmpyiw.rnd
  140778, // llvm.hexagon.M7.wcmpyiwc
  140803, // llvm.hexagon.M7.wcmpyiwc.rnd
  140832, // llvm.hexagon.M7.wcmpyrw
  140856, // llvm.hexagon.M7.wcmpyrw.rnd
  140884, // llvm.hexagon.M7.wcmpyrwc
  140909, // llvm.hexagon.M7.wcmpyrwc.rnd
  140938, // llvm.hexagon.S2.addasl.rrri
  140966, // llvm.hexagon.S2.asl.i.p
  140990, // llvm.hexagon.S2.asl.i.p.acc
  141018, // llvm.hexagon.S2.asl.i.p.and
  141046, // llvm.hexagon.S2.asl.i.p.nac
  141074, // llvm.hexagon.S2.asl.i.p.or
  141101, // llvm.hexagon.S2.asl.i.p.xacc
  141130, // llvm.hexagon.S2.asl.i.r
  141154, // llvm.hexagon.S2.asl.i.r.acc
  141182, // llvm.hexagon.S2.asl.i.r.and
  141210, // llvm.hexagon.S2.asl.i.r.nac
  141238, // llvm.hexagon.S2.asl.i.r.or
  141265, // llvm.hexagon.S2.asl.i.r.sat
  141293, // llvm.hexagon.S2.asl.i.r.xacc
  141322, // llvm.hexagon.S2.asl.i.vh
  141347, // llvm.hexagon.S2.asl.i.vw
  141372, // llvm.hexagon.S2.asl.r.p
  141396, // llvm.hexagon.S2.asl.r.p.acc
  141424, // llvm.hexagon.S2.asl.r.p.and
  141452, // llvm.hexagon.S2.asl.r.p.nac
  141480, // llvm.hexagon.S2.asl.r.p.or
  141507, // llvm.hexagon.S2.asl.r.p.xor
  141535, // llvm.hexagon.S2.asl.r.r
  141559, // llvm.hexagon.S2.asl.r.r.acc
  141587, // llvm.hexagon.S2.asl.r.r.and
  141615, // llvm.hexagon.S2.asl.r.r.nac
  141643, // llvm.hexagon.S2.asl.r.r.or
  141670, // llvm.hexagon.S2.asl.r.r.sat
  141698, // llvm.hexagon.S2.asl.r.vh
  141723, // llvm.hexagon.S2.asl.r.vw
  141748, // llvm.hexagon.S2.asr.i.p
  141772, // llvm.hexagon.S2.asr.i.p.acc
  141800, // llvm.hexagon.S2.asr.i.p.and
  141828, // llvm.hexagon.S2.asr.i.p.nac
  141856, // llvm.hexagon.S2.asr.i.p.or
  141883, // llvm.hexagon.S2.asr.i.p.rnd
  141911, // llvm.hexagon.S2.asr.i.p.rnd.goodsyntax
  141950, // llvm.hexagon.S2.asr.i.r
  141974, // llvm.hexagon.S2.asr.i.r.acc
  142002, // llvm.hexagon.S2.asr.i.r.and
  142030, // llvm.hexagon.S2.asr.i.r.nac
  142058, // llvm.hexagon.S2.asr.i.r.or
  142085, // llvm.hexagon.S2.asr.i.r.rnd
  142113, // llvm.hexagon.S2.asr.i.r.rnd.goodsyntax
  142152, // llvm.hexagon.S2.asr.i.svw.trun
  142183, // llvm.hexagon.S2.asr.i.vh
  142208, // llvm.hexagon.S2.asr.i.vw
  142233, // llvm.hexagon.S2.asr.r.p
  142257, // llvm.hexagon.S2.asr.r.p.acc
  142285, // llvm.hexagon.S2.asr.r.p.and
  142313, // llvm.hexagon.S2.asr.r.p.nac
  142341, // llvm.hexagon.S2.asr.r.p.or
  142368, // llvm.hexagon.S2.asr.r.p.xor
  142396, // llvm.hexagon.S2.asr.r.r
  142420, // llvm.hexagon.S2.asr.r.r.acc
  142448, // llvm.hexagon.S2.asr.r.r.and
  142476, // llvm.hexagon.S2.asr.r.r.nac
  142504, // llvm.hexagon.S2.asr.r.r.or
  142531, // llvm.hexagon.S2.asr.r.r.sat
  142559, // llvm.hexagon.S2.asr.r.svw.trun
  142590, // llvm.hexagon.S2.asr.r.vh
  142615, // llvm.hexagon.S2.asr.r.vw
  142640, // llvm.hexagon.S2.brev
  142661, // llvm.hexagon.S2.brevp
  142683, // llvm.hexagon.S2.cl0
  142703, // llvm.hexagon.S2.cl0p
  142724, // llvm.hexagon.S2.cl1
  142744, // llvm.hexagon.S2.cl1p
  142765, // llvm.hexagon.S2.clb
  142785, // llvm.hexagon.S2.clbnorm
  142809, // llvm.hexagon.S2.clbp
  142830, // llvm.hexagon.S2.clrbit.i
  142855, // llvm.hexagon.S2.clrbit.r
  142880, // llvm.hexagon.S2.ct0
  142900, // llvm.hexagon.S2.ct0p
  142921, // llvm.hexagon.S2.ct1
  142941, // llvm.hexagon.S2.ct1p
  142962, // llvm.hexagon.S2.deinterleave
  142991, // llvm.hexagon.S2.extractu
  143016, // llvm.hexagon.S2.extractu.rp
  143044, // llvm.hexagon.S2.extractup
  143070, // llvm.hexagon.S2.extractup.rp
  143099, // llvm.hexagon.S2.insert
  143122, // llvm.hexagon.S2.insert.rp
  143148, // llvm.hexagon.S2.insertp
  143172, // llvm.hexagon.S2.insertp.rp
  143199, // llvm.hexagon.S2.interleave
  143226, // llvm.hexagon.S2.lfsp
  143247, // llvm.hexagon.S2.lsl.r.p
  143271, // llvm.hexagon.S2.lsl.r.p.acc
  143299, // llvm.hexagon.S2.lsl.r.p.and
  143327, // llvm.hexagon.S2.lsl.r.p.nac
  143355, // llvm.hexagon.S2.lsl.r.p.or
  143382, // llvm.hexagon.S2.lsl.r.p.xor
  143410, // llvm.hexagon.S2.lsl.r.r
  143434, // llvm.hexagon.S2.lsl.r.r.acc
  143462, // llvm.hexagon.S2.lsl.r.r.and
  143490, // llvm.hexagon.S2.lsl.r.r.nac
  143518, // llvm.hexagon.S2.lsl.r.r.or
  143545, // llvm.hexagon.S2.lsl.r.vh
  143570, // llvm.hexagon.S2.lsl.r.vw
  143595, // llvm.hexagon.S2.lsr.i.p
  143619, // llvm.hexagon.S2.lsr.i.p.acc
  143647, // llvm.hexagon.S2.lsr.i.p.and
  143675, // llvm.hexagon.S2.lsr.i.p.nac
  143703, // llvm.hexagon.S2.lsr.i.p.or
  143730, // llvm.hexagon.S2.lsr.i.p.xacc
  143759, // llvm.hexagon.S2.lsr.i.r
  143783, // llvm.hexagon.S2.lsr.i.r.acc
  143811, // llvm.hexagon.S2.lsr.i.r.and
  143839, // llvm.hexagon.S2.lsr.i.r.nac
  143867, // llvm.hexagon.S2.lsr.i.r.or
  143894, // llvm.hexagon.S2.lsr.i.r.xacc
  143923, // llvm.hexagon.S2.lsr.i.vh
  143948, // llvm.hexagon.S2.lsr.i.vw
  143973, // llvm.hexagon.S2.lsr.r.p
  143997, // llvm.hexagon.S2.lsr.r.p.acc
  144025, // llvm.hexagon.S2.lsr.r.p.and
  144053, // llvm.hexagon.S2.lsr.r.p.nac
  144081, // llvm.hexagon.S2.lsr.r.p.or
  144108, // llvm.hexagon.S2.lsr.r.p.xor
  144136, // llvm.hexagon.S2.lsr.r.r
  144160, // llvm.hexagon.S2.lsr.r.r.acc
  144188, // llvm.hexagon.S2.lsr.r.r.and
  144216, // llvm.hexagon.S2.lsr.r.r.nac
  144244, // llvm.hexagon.S2.lsr.r.r.or
  144271, // llvm.hexagon.S2.lsr.r.vh
  144296, // llvm.hexagon.S2.lsr.r.vw
  144321, // llvm.hexagon.S2.mask
  144342, // llvm.hexagon.S2.packhl
  144365, // llvm.hexagon.S2.parityp
  144389, // llvm.hexagon.S2.setbit.i
  144414, // llvm.hexagon.S2.setbit.r
  144439, // llvm.hexagon.S2.shuffeb
  144463, // llvm.hexagon.S2.shuffeh
  144487, // llvm.hexagon.S2.shuffob
  144511, // llvm.hexagon.S2.shuffoh
  144535, // llvm.hexagon.S2.storerb.pbr
  144563, // llvm.hexagon.S2.storerb.pci
  144591, // llvm.hexagon.S2.storerb.pcr
  144619, // llvm.hexagon.S2.storerd.pbr
  144647, // llvm.hexagon.S2.storerd.pci
  144675, // llvm.hexagon.S2.storerd.pcr
  144703, // llvm.hexagon.S2.storerf.pbr
  144731, // llvm.hexagon.S2.storerf.pci
  144759, // llvm.hexagon.S2.storerf.pcr
  144787, // llvm.hexagon.S2.storerh.pbr
  144815, // llvm.hexagon.S2.storerh.pci
  144843, // llvm.hexagon.S2.storerh.pcr
  144871, // llvm.hexagon.S2.storeri.pbr
  144899, // llvm.hexagon.S2.storeri.pci
  144927, // llvm.hexagon.S2.storeri.pcr
  144955, // llvm.hexagon.S2.storew.locked
  144985, // llvm.hexagon.S2.svsathb
  145009, // llvm.hexagon.S2.svsathub
  145034, // llvm.hexagon.S2.tableidxb.goodsyntax
  145071, // llvm.hexagon.S2.tableidxd.goodsyntax
  145108, // llvm.hexagon.S2.tableidxh.goodsyntax
  145145, // llvm.hexagon.S2.tableidxw.goodsyntax
  145182, // llvm.hexagon.S2.togglebit.i
  145210, // llvm.hexagon.S2.togglebit.r
  145238, // llvm.hexagon.S2.tstbit.i
  145263, // llvm.hexagon.S2.tstbit.r
  145288, // llvm.hexagon.S2.valignib
  145313, // llvm.hexagon.S2.valignrb
  145338, // llvm.hexagon.S2.vcnegh
  145361, // llvm.hexagon.S2.vcrotate
  145386, // llvm.hexagon.S2.vrcnegh
  145410, // llvm.hexagon.S2.vrndpackwh
  145437, // llvm.hexagon.S2.vrndpackwhs
  145465, // llvm.hexagon.S2.vsathb
  145488, // llvm.hexagon.S2.vsathb.nopack
  145518, // llvm.hexagon.S2.vsathub
  145542, // llvm.hexagon.S2.vsathub.nopack
  145573, // llvm.hexagon.S2.vsatwh
  145596, // llvm.hexagon.S2.vsatwh.nopack
  145626, // llvm.hexagon.S2.vsatwuh
  145650, // llvm.hexagon.S2.vsatwuh.nopack
  145681, // llvm.hexagon.S2.vsplatrb
  145706, // llvm.hexagon.S2.vsplatrh
  145731, // llvm.hexagon.S2.vspliceib
  145757, // llvm.hexagon.S2.vsplicerb
  145783, // llvm.hexagon.S2.vsxtbh
  145806, // llvm.hexagon.S2.vsxthw
  145829, // llvm.hexagon.S2.vtrunehb
  145854, // llvm.hexagon.S2.vtrunewh
  145879, // llvm.hexagon.S2.vtrunohb
  145904, // llvm.hexagon.S2.vtrunowh
  145929, // llvm.hexagon.S2.vzxtbh
  145952, // llvm.hexagon.S2.vzxthw
  145975, // llvm.hexagon.S4.addaddi
  145999, // llvm.hexagon.S4.addi.asl.ri
  146027, // llvm.hexagon.S4.addi.lsr.ri
  146055, // llvm.hexagon.S4.andi.asl.ri
  146083, // llvm.hexagon.S4.andi.lsr.ri
  146111, // llvm.hexagon.S4.clbaddi
  146135, // llvm.hexagon.S4.clbpaddi
  146160, // llvm.hexagon.S4.clbpnorm
  146185, // llvm.hexagon.S4.extract
  146209, // llvm.hexagon.S4.extract.rp
  146236, // llvm.hexagon.S4.extractp
  146261, // llvm.hexagon.S4.extractp.rp
  146289, // llvm.hexagon.S4.lsli
  146310, // llvm.hexagon.S4.ntstbit.i
  146336, // llvm.hexagon.S4.ntstbit.r
  146362, // llvm.hexagon.S4.or.andi
  146386, // llvm.hexagon.S4.or.andix
  146411, // llvm.hexagon.S4.or.ori
  146434, // llvm.hexagon.S4.ori.asl.ri
  146461, // llvm.hexagon.S4.ori.lsr.ri
  146488, // llvm.hexagon.S4.parity
  146511, // llvm.hexagon.S4.stored.locked
  146541, // llvm.hexagon.S4.subaddi
  146565, // llvm.hexagon.S4.subi.asl.ri
  146593, // llvm.hexagon.S4.subi.lsr.ri
  146621, // llvm.hexagon.S4.vrcrotate
  146647, // llvm.hexagon.S4.vrcrotate.acc
  146677, // llvm.hexagon.S4.vxaddsubh
  146703, // llvm.hexagon.S4.vxaddsubhr
  146730, // llvm.hexagon.S4.vxaddsubw
  146756, // llvm.hexagon.S4.vxsubaddh
  146782, // llvm.hexagon.S4.vxsubaddhr
  146809, // llvm.hexagon.S4.vxsubaddw
  146835, // llvm.hexagon.S5.asrhub.rnd.sat.goodsyntax
  146877, // llvm.hexagon.S5.asrhub.sat
  146904, // llvm.hexagon.S5.popcountp
  146930, // llvm.hexagon.S5.vasrhrnd.goodsyntax
  146966, // llvm.hexagon.S6.rol.i.p
  146990, // llvm.hexagon.S6.rol.i.p.acc
  147018, // llvm.hexagon.S6.rol.i.p.and
  147046, // llvm.hexagon.S6.rol.i.p.nac
  147074, // llvm.hexagon.S6.rol.i.p.or
  147101, // llvm.hexagon.S6.rol.i.p.xacc
  147130, // llvm.hexagon.S6.rol.i.r
  147154, // llvm.hexagon.S6.rol.i.r.acc
  147182, // llvm.hexagon.S6.rol.i.r.and
  147210, // llvm.hexagon.S6.rol.i.r.nac
  147238, // llvm.hexagon.S6.rol.i.r.or
  147265, // llvm.hexagon.S6.rol.i.r.xacc
  147294, // llvm.hexagon.S6.vsplatrbp
  147320, // llvm.hexagon.S6.vtrunehb.ppp
  147349, // llvm.hexagon.S6.vtrunohb.ppp
  147378, // llvm.hexagon.V6.extractw
  147403, // llvm.hexagon.V6.extractw.128B
  147433, // llvm.hexagon.V6.get.qfext
  147459, // llvm.hexagon.V6.get.qfext.128B
  147490, // llvm.hexagon.V6.get.qfext.oracc
  147522, // llvm.hexagon.V6.get.qfext.oracc.128B
  147559, // llvm.hexagon.V6.hi
  147578, // llvm.hexagon.V6.hi.128B
  147602, // llvm.hexagon.V6.lo
  147621, // llvm.hexagon.V6.lo.128B
  147645, // llvm.hexagon.V6.lvsplatb
  147670, // llvm.hexagon.V6.lvsplatb.128B
  147700, // llvm.hexagon.V6.lvsplath
  147725, // llvm.hexagon.V6.lvsplath.128B
  147755, // llvm.hexagon.V6.lvsplatw
  147780, // llvm.hexagon.V6.lvsplatw.128B
  147810, // llvm.hexagon.V6.pred.and
  147835, // llvm.hexagon.V6.pred.and.128B
  147865, // llvm.hexagon.V6.pred.and.n
  147892, // llvm.hexagon.V6.pred.and.n.128B
  147924, // llvm.hexagon.V6.pred.not
  147949, // llvm.hexagon.V6.pred.not.128B
  147979, // llvm.hexagon.V6.pred.or
  148003, // llvm.hexagon.V6.pred.or.128B
  148032, // llvm.hexagon.V6.pred.or.n
  148058, // llvm.hexagon.V6.pred.or.n.128B
  148089, // llvm.hexagon.V6.pred.scalar2
  148118, // llvm.hexagon.V6.pred.scalar2.128B
  148152, // llvm.hexagon.V6.pred.scalar2v2
  148183, // llvm.hexagon.V6.pred.scalar2v2.128B
  148219, // llvm.hexagon.V6.pred.typecast
  148249, // llvm.hexagon.V6.pred.typecast.128B
  148284, // llvm.hexagon.V6.pred.xor
  148309, // llvm.hexagon.V6.pred.xor.128B
  148339, // llvm.hexagon.V6.set.qfext
  148365, // llvm.hexagon.V6.set.qfext.128B
  148396, // llvm.hexagon.V6.shuffeqh
  148421, // llvm.hexagon.V6.shuffeqh.128B
  148451, // llvm.hexagon.V6.shuffeqw
  148476, // llvm.hexagon.V6.shuffeqw.128B
  148506, // llvm.hexagon.V6.v6mpyhubs10
  148534, // llvm.hexagon.V6.v6mpyhubs10.128B
  148567, // llvm.hexagon.V6.v6mpyhubs10.vxx
  148599, // llvm.hexagon.V6.v6mpyhubs10.vxx.128B
  148636, // llvm.hexagon.V6.v6mpyvubs10
  148664, // llvm.hexagon.V6.v6mpyvubs10.128B
  148697, // llvm.hexagon.V6.v6mpyvubs10.vxx
  148729, // llvm.hexagon.V6.v6mpyvubs10.vxx.128B
  148766, // llvm.hexagon.V6.vL32b.npred.ai
  148797, // llvm.hexagon.V6.vL32b.npred.ai.128B
  148833, // llvm.hexagon.V6.vL32b.npred.pi
  148864, // llvm.hexagon.V6.vL32b.npred.pi.128B
  148900, // llvm.hexagon.V6.vL32b.npred.ppu
  148932, // llvm.hexagon.V6.vL32b.npred.ppu.128B
  148969, // llvm.hexagon.V6.vL32b.nt.npred.ai
  149003, // llvm.hexagon.V6.vL32b.nt.npred.ai.128B
  149042, // llvm.hexagon.V6.vL32b.nt.npred.pi
  149076, // llvm.hexagon.V6.vL32b.nt.npred.pi.128B
  149115, // llvm.hexagon.V6.vL32b.nt.npred.ppu
  149150, // llvm.hexagon.V6.vL32b.nt.npred.ppu.128B
  149190, // llvm.hexagon.V6.vL32b.nt.pred.ai
  149223, // llvm.hexagon.V6.vL32b.nt.pred.ai.128B
  149261, // llvm.hexagon.V6.vL32b.nt.pred.pi
  149294, // llvm.hexagon.V6.vL32b.nt.pred.pi.128B
  149332, // llvm.hexagon.V6.vL32b.nt.pred.ppu
  149366, // llvm.hexagon.V6.vL32b.nt.pred.ppu.128B
  149405, // llvm.hexagon.V6.vL32b.pred.ai
  149435, // llvm.hexagon.V6.vL32b.pred.ai.128B
  149470, // llvm.hexagon.V6.vL32b.pred.pi
  149500, // llvm.hexagon.V6.vL32b.pred.pi.128B
  149535, // llvm.hexagon.V6.vL32b.pred.ppu
  149566, // llvm.hexagon.V6.vL32b.pred.ppu.128B
  149602, // llvm.hexagon.V6.vS32Ub.npred.ai
  149634, // llvm.hexagon.V6.vS32Ub.npred.ai.128B
  149671, // llvm.hexagon.V6.vS32Ub.npred.pi
  149703, // llvm.hexagon.V6.vS32Ub.npred.pi.128B
  149740, // llvm.hexagon.V6.vS32Ub.npred.ppu
  149773, // llvm.hexagon.V6.vS32Ub.npred.ppu.128B
  149811, // llvm.hexagon.V6.vS32Ub.pred.ai
  149842, // llvm.hexagon.V6.vS32Ub.pred.ai.128B
  149878, // llvm.hexagon.V6.vS32Ub.pred.pi
  149909, // llvm.hexagon.V6.vS32Ub.pred.pi.128B
  149945, // llvm.hexagon.V6.vS32Ub.pred.ppu
  149977, // llvm.hexagon.V6.vS32Ub.pred.ppu.128B
  150014, // llvm.hexagon.V6.vS32b.npred.ai
  150045, // llvm.hexagon.V6.vS32b.npred.ai.128B
  150081, // llvm.hexagon.V6.vS32b.npred.pi
  150112, // llvm.hexagon.V6.vS32b.npred.pi.128B
  150148, // llvm.hexagon.V6.vS32b.npred.ppu
  150180, // llvm.hexagon.V6.vS32b.npred.ppu.128B
  150217, // llvm.hexagon.V6.vS32b.nqpred.ai
  150249, // llvm.hexagon.V6.vS32b.nqpred.ai.128B
  150286, // llvm.hexagon.V6.vS32b.nt.npred.ai
  150320, // llvm.hexagon.V6.vS32b.nt.npred.ai.128B
  150359, // llvm.hexagon.V6.vS32b.nt.npred.pi
  150393, // llvm.hexagon.V6.vS32b.nt.npred.pi.128B
  150432, // llvm.hexagon.V6.vS32b.nt.npred.ppu
  150467, // llvm.hexagon.V6.vS32b.nt.npred.ppu.128B
  150507, // llvm.hexagon.V6.vS32b.nt.nqpred.ai
  150542, // llvm.hexagon.V6.vS32b.nt.nqpred.ai.128B
  150582, // llvm.hexagon.V6.vS32b.nt.pred.ai
  150615, // llvm.hexagon.V6.vS32b.nt.pred.ai.128B
  150653, // llvm.hexagon.V6.vS32b.nt.pred.pi
  150686, // llvm.hexagon.V6.vS32b.nt.pred.pi.128B
  150724, // llvm.hexagon.V6.vS32b.nt.pred.ppu
  150758, // llvm.hexagon.V6.vS32b.nt.pred.ppu.128B
  150797, // llvm.hexagon.V6.vS32b.nt.qpred.ai
  150831, // llvm.hexagon.V6.vS32b.nt.qpred.ai.128B
  150870, // llvm.hexagon.V6.vS32b.pred.ai
  150900, // llvm.hexagon.V6.vS32b.pred.ai.128B
  150935, // llvm.hexagon.V6.vS32b.pred.pi
  150965, // llvm.hexagon.V6.vS32b.pred.pi.128B
  151000, // llvm.hexagon.V6.vS32b.pred.ppu
  151031, // llvm.hexagon.V6.vS32b.pred.ppu.128B
  151067, // llvm.hexagon.V6.vS32b.qpred.ai
  151098, // llvm.hexagon.V6.vS32b.qpred.ai.128B
  151134, // llvm.hexagon.V6.vabs.f8
  151158, // llvm.hexagon.V6.vabs.f8.128B
  151187, // llvm.hexagon.V6.vabs.hf
  151211, // llvm.hexagon.V6.vabs.hf.128B
  151240, // llvm.hexagon.V6.vabs.qf16.hf
  151269, // llvm.hexagon.V6.vabs.qf16.hf.128B
  151303, // llvm.hexagon.V6.vabs.qf16.qf16
  151334, // llvm.hexagon.V6.vabs.qf16.qf16.128B
  151370, // llvm.hexagon.V6.vabs.qf32.qf32
  151401, // llvm.hexagon.V6.vabs.qf32.qf32.128B
  151437, // llvm.hexagon.V6.vabs.qf32.sf
  151466, // llvm.hexagon.V6.vabs.qf32.sf.128B
  151500, // llvm.hexagon.V6.vabs.sf
  151524, // llvm.hexagon.V6.vabs.sf.128B
  151553, // llvm.hexagon.V6.vabsb
  151575, // llvm.hexagon.V6.vabsb.128B
  151602, // llvm.hexagon.V6.vabsb.sat
  151628, // llvm.hexagon.V6.vabsb.sat.128B
  151659, // llvm.hexagon.V6.vabsdiffh
  151685, // llvm.hexagon.V6.vabsdiffh.128B
  151716, // llvm.hexagon.V6.vabsdiffub
  151743, // llvm.hexagon.V6.vabsdiffub.128B
  151775, // llvm.hexagon.V6.vabsdiffuh
  151802, // llvm.hexagon.V6.vabsdiffuh.128B
  151834, // llvm.hexagon.V6.vabsdiffw
  151860, // llvm.hexagon.V6.vabsdiffw.128B
  151891, // llvm.hexagon.V6.vabsh
  151913, // llvm.hexagon.V6.vabsh.128B
  151940, // llvm.hexagon.V6.vabsh.sat
  151966, // llvm.hexagon.V6.vabsh.sat.128B
  151997, // llvm.hexagon.V6.vabsw
  152019, // llvm.hexagon.V6.vabsw.128B
  152046, // llvm.hexagon.V6.vabsw.sat
  152072, // llvm.hexagon.V6.vabsw.sat.128B
  152103, // llvm.hexagon.V6.vadd.hf
  152127, // llvm.hexagon.V6.vadd.hf.128B
  152156, // llvm.hexagon.V6.vadd.hf.f8
  152183, // llvm.hexagon.V6.vadd.hf.f8.128B
  152215, // llvm.hexagon.V6.vadd.hf.hf
  152242, // llvm.hexagon.V6.vadd.hf.hf.128B
  152274, // llvm.hexagon.V6.vadd.qf16
  152300, // llvm.hexagon.V6.vadd.qf16.128B
  152331, // llvm.hexagon.V6.vadd.qf16.mix
  152361, // llvm.hexagon.V6.vadd.qf16.mix.128B
  152396, // llvm.hexagon.V6.vadd.qf32
  152422, // llvm.hexagon.V6.vadd.qf32.128B
  152453, // llvm.hexagon.V6.vadd.qf32.mix
  152483, // llvm.hexagon.V6.vadd.qf32.mix.128B
  152518, // llvm.hexagon.V6.vadd.sf
  152542, // llvm.hexagon.V6.vadd.sf.128B
  152571, // llvm.hexagon.V6.vadd.sf.bf
  152598, // llvm.hexagon.V6.vadd.sf.bf.128B
  152630, // llvm.hexagon.V6.vadd.sf.hf
  152657, // llvm.hexagon.V6.vadd.sf.hf.128B
  152689, // llvm.hexagon.V6.vadd.sf.sf
  152716, // llvm.hexagon.V6.vadd.sf.sf.128B
  152748, // llvm.hexagon.V6.vaddb
  152770, // llvm.hexagon.V6.vaddb.128B
  152797, // llvm.hexagon.V6.vaddb.dv
  152822, // llvm.hexagon.V6.vaddb.dv.128B
  152852, // llvm.hexagon.V6.vaddbnq
  152876, // llvm.hexagon.V6.vaddbnq.128B
  152905, // llvm.hexagon.V6.vaddbq
  152928, // llvm.hexagon.V6.vaddbq.128B
  152956, // llvm.hexagon.V6.vaddbsat
  152981, // llvm.hexagon.V6.vaddbsat.128B
  153011, // llvm.hexagon.V6.vaddbsat.dv
  153039, // llvm.hexagon.V6.vaddbsat.dv.128B
  153072, // llvm.hexagon.V6.vaddcarry
  153098, // llvm.hexagon.V6.vaddcarry.128B
  153129, // llvm.hexagon.V6.vaddcarryo
  153156, // llvm.hexagon.V6.vaddcarryo.128B
  153188, // llvm.hexagon.V6.vaddcarrysat
  153217, // llvm.hexagon.V6.vaddcarrysat.128B
  153251, // llvm.hexagon.V6.vaddclbh
  153276, // llvm.hexagon.V6.vaddclbh.128B
  153306, // llvm.hexagon.V6.vaddclbw
  153331, // llvm.hexagon.V6.vaddclbw.128B
  153361, // llvm.hexagon.V6.vaddh
  153383, // llvm.hexagon.V6.vaddh.128B
  153410, // llvm.hexagon.V6.vaddh.dv
  153435, // llvm.hexagon.V6.vaddh.dv.128B
  153465, // llvm.hexagon.V6.vaddhnq
  153489, // llvm.hexagon.V6.vaddhnq.128B
  153518, // llvm.hexagon.V6.vaddhq
  153541, // llvm.hexagon.V6.vaddhq.128B
  153569, // llvm.hexagon.V6.vaddhsat
  153594, // llvm.hexagon.V6.vaddhsat.128B
  153624, // llvm.hexagon.V6.vaddhsat.dv
  153652, // llvm.hexagon.V6.vaddhsat.dv.128B
  153685, // llvm.hexagon.V6.vaddhw
  153708, // llvm.hexagon.V6.vaddhw.128B
  153736, // llvm.hexagon.V6.vaddhw.acc
  153763, // llvm.hexagon.V6.vaddhw.acc.128B
  153795, // llvm.hexagon.V6.vaddubh
  153819, // llvm.hexagon.V6.vaddubh.128B
  153848, // llvm.hexagon.V6.vaddubh.acc
  153876, // llvm.hexagon.V6.vaddubh.acc.128B
  153909, // llvm.hexagon.V6.vaddubsat
  153935, // llvm.hexagon.V6.vaddubsat.128B
  153966, // llvm.hexagon.V6.vaddubsat.dv
  153995, // llvm.hexagon.V6.vaddubsat.dv.128B
  154029, // llvm.hexagon.V6.vaddububb.sat
  154059, // llvm.hexagon.V6.vaddububb.sat.128B
  154094, // llvm.hexagon.V6.vadduhsat
  154120, // llvm.hexagon.V6.vadduhsat.128B
  154151, // llvm.hexagon.V6.vadduhsat.dv
  154180, // llvm.hexagon.V6.vadduhsat.dv.128B
  154214, // llvm.hexagon.V6.vadduhw
  154238, // llvm.hexagon.V6.vadduhw.128B
  154267, // llvm.hexagon.V6.vadduhw.acc
  154295, // llvm.hexagon.V6.vadduhw.acc.128B
  154328, // llvm.hexagon.V6.vadduwsat
  154354, // llvm.hexagon.V6.vadduwsat.128B
  154385, // llvm.hexagon.V6.vadduwsat.dv
  154414, // llvm.hexagon.V6.vadduwsat.dv.128B
  154448, // llvm.hexagon.V6.vaddw
  154470, // llvm.hexagon.V6.vaddw.128B
  154497, // llvm.hexagon.V6.vaddw.dv
  154522, // llvm.hexagon.V6.vaddw.dv.128B
  154552, // llvm.hexagon.V6.vaddwnq
  154576, // llvm.hexagon.V6.vaddwnq.128B
  154605, // llvm.hexagon.V6.vaddwq
  154628, // llvm.hexagon.V6.vaddwq.128B
  154656, // llvm.hexagon.V6.vaddwsat
  154681, // llvm.hexagon.V6.vaddwsat.128B
  154711, // llvm.hexagon.V6.vaddwsat.dv
  154739, // llvm.hexagon.V6.vaddwsat.dv.128B
  154772, // llvm.hexagon.V6.valign4
  154796, // llvm.hexagon.V6.valign4.128B
  154825, // llvm.hexagon.V6.valignb
  154849, // llvm.hexagon.V6.valignb.128B
  154878, // llvm.hexagon.V6.valignbi
  154903, // llvm.hexagon.V6.valignbi.128B
  154933, // llvm.hexagon.V6.vand
  154954, // llvm.hexagon.V6.vand.128B
  154980, // llvm.hexagon.V6.vandnqrt
  155005, // llvm.hexagon.V6.vandnqrt.128B
  155035, // llvm.hexagon.V6.vandnqrt.acc
  155064, // llvm.hexagon.V6.vandnqrt.acc.128B
  155098, // llvm.hexagon.V6.vandqrt
  155122, // llvm.hexagon.V6.vandqrt.128B
  155151, // llvm.hexagon.V6.vandqrt.acc
  155179, // llvm.hexagon.V6.vandqrt.acc.128B
  155212, // llvm.hexagon.V6.vandvnqv
  155237, // llvm.hexagon.V6.vandvnqv.128B
  155267, // llvm.hexagon.V6.vandvqv
  155291, // llvm.hexagon.V6.vandvqv.128B
  155320, // llvm.hexagon.V6.vandvrt
  155344, // llvm.hexagon.V6.vandvrt.128B
  155373, // llvm.hexagon.V6.vandvrt.acc
  155401, // llvm.hexagon.V6.vandvrt.acc.128B
  155434, // llvm.hexagon.V6.vaslh
  155456, // llvm.hexagon.V6.vaslh.128B
  155483, // llvm.hexagon.V6.vaslh.acc
  155509, // llvm.hexagon.V6.vaslh.acc.128B
  155540, // llvm.hexagon.V6.vaslhv
  155563, // llvm.hexagon.V6.vaslhv.128B
  155591, // llvm.hexagon.V6.vaslw
  155613, // llvm.hexagon.V6.vaslw.128B
  155640, // llvm.hexagon.V6.vaslw.acc
  155666, // llvm.hexagon.V6.vaslw.acc.128B
  155697, // llvm.hexagon.V6.vaslwv
  155720, // llvm.hexagon.V6.vaslwv.128B
  155748, // llvm.hexagon.V6.vasr.into
  155774, // llvm.hexagon.V6.vasr.into.128B
  155805, // llvm.hexagon.V6.vasrh
  155827, // llvm.hexagon.V6.vasrh.128B
  155854, // llvm.hexagon.V6.vasrh.acc
  155880, // llvm.hexagon.V6.vasrh.acc.128B
  155911, // llvm.hexagon.V6.vasrhbrndsat
  155940, // llvm.hexagon.V6.vasrhbrndsat.128B
  155974, // llvm.hexagon.V6.vasrhbsat
  156000, // llvm.hexagon.V6.vasrhbsat.128B
  156031, // llvm.hexagon.V6.vasrhubrndsat
  156061, // llvm.hexagon.V6.vasrhubrndsat.128B
  156096, // llvm.hexagon.V6.vasrhubsat
  156123, // llvm.hexagon.V6.vasrhubsat.128B
  156155, // llvm.hexagon.V6.vasrhv
  156178, // llvm.hexagon.V6.vasrhv.128B
  156206, // llvm.hexagon.V6.vasruhubrndsat
  156237, // llvm.hexagon.V6.vasruhubrndsat.128B
  156273, // llvm.hexagon.V6.vasruhubsat
  156301, // llvm.hexagon.V6.vasruhubsat.128B
  156334, // llvm.hexagon.V6.vasruwuhrndsat
  156365, // llvm.hexagon.V6.vasruwuhrndsat.128B
  156401, // llvm.hexagon.V6.vasruwuhsat
  156429, // llvm.hexagon.V6.vasruwuhsat.128B
  156462, // llvm.hexagon.V6.vasrvuhubrndsat
  156494, // llvm.hexagon.V6.vasrvuhubrndsat.128B
  156531, // llvm.hexagon.V6.vasrvuhubsat
  156560, // llvm.hexagon.V6.vasrvuhubsat.128B
  156594, // llvm.hexagon.V6.vasrvwuhrndsat
  156625, // llvm.hexagon.V6.vasrvwuhrndsat.128B
  156661, // llvm.hexagon.V6.vasrvwuhsat
  156689, // llvm.hexagon.V6.vasrvwuhsat.128B
  156722, // llvm.hexagon.V6.vasrw
  156744, // llvm.hexagon.V6.vasrw.128B
  156771, // llvm.hexagon.V6.vasrw.acc
  156797, // llvm.hexagon.V6.vasrw.acc.128B
  156828, // llvm.hexagon.V6.vasrwh
  156851, // llvm.hexagon.V6.vasrwh.128B
  156879, // llvm.hexagon.V6.vasrwhrndsat
  156908, // llvm.hexagon.V6.vasrwhrndsat.128B
  156942, // llvm.hexagon.V6.vasrwhsat
  156968, // llvm.hexagon.V6.vasrwhsat.128B
  156999, // llvm.hexagon.V6.vasrwuhrndsat
  157029, // llvm.hexagon.V6.vasrwuhrndsat.128B
  157064, // llvm.hexagon.V6.vasrwuhsat
  157091, // llvm.hexagon.V6.vasrwuhsat.128B
  157123, // llvm.hexagon.V6.vasrwv
  157146, // llvm.hexagon.V6.vasrwv.128B
  157174, // llvm.hexagon.V6.vassign
  157198, // llvm.hexagon.V6.vassign.128B
  157227, // llvm.hexagon.V6.vassign.fp
  157254, // llvm.hexagon.V6.vassign.fp.128B
  157286, // llvm.hexagon.V6.vassignp
  157311, // llvm.hexagon.V6.vassignp.128B
  157341, // llvm.hexagon.V6.vavgb
  157363, // llvm.hexagon.V6.vavgb.128B
  157390, // llvm.hexagon.V6.vavgbrnd
  157415, // llvm.hexagon.V6.vavgbrnd.128B
  157445, // llvm.hexagon.V6.vavgh
  157467, // llvm.hexagon.V6.vavgh.128B
  157494, // llvm.hexagon.V6.vavghrnd
  157519, // llvm.hexagon.V6.vavghrnd.128B
  157549, // llvm.hexagon.V6.vavgub
  157572, // llvm.hexagon.V6.vavgub.128B
  157600, // llvm.hexagon.V6.vavgubrnd
  157626, // llvm.hexagon.V6.vavgubrnd.128B
  157657, // llvm.hexagon.V6.vavguh
  157680, // llvm.hexagon.V6.vavguh.128B
  157708, // llvm.hexagon.V6.vavguhrnd
  157734, // llvm.hexagon.V6.vavguhrnd.128B
  157765, // llvm.hexagon.V6.vavguw
  157788, // llvm.hexagon.V6.vavguw.128B
  157816, // llvm.hexagon.V6.vavguwrnd
  157842, // llvm.hexagon.V6.vavguwrnd.128B
  157873, // llvm.hexagon.V6.vavgw
  157895, // llvm.hexagon.V6.vavgw.128B
  157922, // llvm.hexagon.V6.vavgwrnd
  157947, // llvm.hexagon.V6.vavgwrnd.128B
  157977, // llvm.hexagon.V6.vcl0h
  157999, // llvm.hexagon.V6.vcl0h.128B
  158026, // llvm.hexagon.V6.vcl0w
  158048, // llvm.hexagon.V6.vcl0w.128B
  158075, // llvm.hexagon.V6.vcombine
  158100, // llvm.hexagon.V6.vcombine.128B
  158130, // llvm.hexagon.V6.vconv.bf.qf32
  158160, // llvm.hexagon.V6.vconv.bf.qf32.128B
  158195, // llvm.hexagon.V6.vconv.f8.qf16
  158225, // llvm.hexagon.V6.vconv.f8.qf16.128B
  158260, // llvm.hexagon.V6.vconv.h.hf
  158287, // llvm.hexagon.V6.vconv.h.hf.128B
  158319, // llvm.hexagon.V6.vconv.h.hf.rnd
  158350, // llvm.hexagon.V6.vconv.h.hf.rnd.128B
  158386, // llvm.hexagon.V6.vconv.hf.h
  158413, // llvm.hexagon.V6.vconv.hf.h.128B
  158445, // llvm.hexagon.V6.vconv.hf.qf16
  158475, // llvm.hexagon.V6.vconv.hf.qf16.128B
  158510, // llvm.hexagon.V6.vconv.hf.qf32
  158540, // llvm.hexagon.V6.vconv.hf.qf32.128B
  158575, // llvm.hexagon.V6.vconv.qf16.f8
  158605, // llvm.hexagon.V6.vconv.qf16.f8.128B
  158640, // llvm.hexagon.V6.vconv.qf16.hf
  158670, // llvm.hexagon.V6.vconv.qf16.hf.128B
  158705, // llvm.hexagon.V6.vconv.qf16.qf16
  158737, // llvm.hexagon.V6.vconv.qf16.qf16.128B
  158774, // llvm.hexagon.V6.vconv.qf32.qf32
  158806, // llvm.hexagon.V6.vconv.qf32.qf32.128B
  158843, // llvm.hexagon.V6.vconv.qf32.sf
  158873, // llvm.hexagon.V6.vconv.qf32.sf.128B
  158908, // llvm.hexagon.V6.vconv.sf.qf32
  158938, // llvm.hexagon.V6.vconv.sf.qf32.128B
  158973, // llvm.hexagon.V6.vconv.sf.w
  159000, // llvm.hexagon.V6.vconv.sf.w.128B
  159032, // llvm.hexagon.V6.vconv.w.sf
  159059, // llvm.hexagon.V6.vconv.w.sf.128B
  159091, // llvm.hexagon.V6.vcvt.b.hf
  159117, // llvm.hexagon.V6.vcvt.b.hf.128B
  159148, // llvm.hexagon.V6.vcvt.bf.sf
  159175, // llvm.hexagon.V6.vcvt.bf.sf.128B
  159207, // llvm.hexagon.V6.vcvt.f8.hf
  159234, // llvm.hexagon.V6.vcvt.f8.hf.128B
  159266, // llvm.hexagon.V6.vcvt.h.hf
  159292, // llvm.hexagon.V6.vcvt.h.hf.128B
  159323, // llvm.hexagon.V6.vcvt.hf.b
  159349, // llvm.hexagon.V6.vcvt.hf.b.128B
  159380, // llvm.hexagon.V6.vcvt.hf.f8
  159407, // llvm.hexagon.V6.vcvt.hf.f8.128B
  159439, // llvm.hexagon.V6.vcvt.hf.h
  159465, // llvm.hexagon.V6.vcvt.hf.h.128B
  159496, // llvm.hexagon.V6.vcvt.hf.sf
  159523, // llvm.hexagon.V6.vcvt.hf.sf.128B
  159555, // llvm.hexagon.V6.vcvt.hf.ub
  159582, // llvm.hexagon.V6.vcvt.hf.ub.128B
  159614, // llvm.hexagon.V6.vcvt.hf.uh
  159641, // llvm.hexagon.V6.vcvt.hf.uh.128B
  159673, // llvm.hexagon.V6.vcvt.sf.hf
  159700, // llvm.hexagon.V6.vcvt.sf.hf.128B
  159732, // llvm.hexagon.V6.vcvt.ub.hf
  159759, // llvm.hexagon.V6.vcvt.ub.hf.128B
  159791, // llvm.hexagon.V6.vcvt.uh.hf
  159818, // llvm.hexagon.V6.vcvt.uh.hf.128B
  159850, // llvm.hexagon.V6.vcvt2.b.hf
  159877, // llvm.hexagon.V6.vcvt2.b.hf.128B
  159909, // llvm.hexagon.V6.vcvt2.hf.b
  159936, // llvm.hexagon.V6.vcvt2.hf.b.128B
  159968, // llvm.hexagon.V6.vcvt2.hf.ub
  159996, // llvm.hexagon.V6.vcvt2.hf.ub.128B
  160029, // llvm.hexagon.V6.vcvt2.ub.hf
  160057, // llvm.hexagon.V6.vcvt2.ub.hf.128B
  160090, // llvm.hexagon.V6.vd0
  160110, // llvm.hexagon.V6.vd0.128B
  160135, // llvm.hexagon.V6.vdd0
  160156, // llvm.hexagon.V6.vdd0.128B
  160182, // llvm.hexagon.V6.vdealb
  160205, // llvm.hexagon.V6.vdealb.128B
  160233, // llvm.hexagon.V6.vdealb4w
  160258, // llvm.hexagon.V6.vdealb4w.128B
  160288, // llvm.hexagon.V6.vdealh
  160311, // llvm.hexagon.V6.vdealh.128B
  160339, // llvm.hexagon.V6.vdealvdd
  160364, // llvm.hexagon.V6.vdealvdd.128B
  160394, // llvm.hexagon.V6.vdelta
  160417, // llvm.hexagon.V6.vdelta.128B
  160445, // llvm.hexagon.V6.vdmpy.sf.hf
  160473, // llvm.hexagon.V6.vdmpy.sf.hf.128B
  160506, // llvm.hexagon.V6.vdmpy.sf.hf.acc
  160538, // llvm.hexagon.V6.vdmpy.sf.hf.acc.128B
  160575, // llvm.hexagon.V6.vdmpybus
  160600, // llvm.hexagon.V6.vdmpybus.128B
  160630, // llvm.hexagon.V6.vdmpybus.acc
  160659, // llvm.hexagon.V6.vdmpybus.acc.128B
  160693, // llvm.hexagon.V6.vdmpybus.dv
  160721, // llvm.hexagon.V6.vdmpybus.dv.128B
  160754, // llvm.hexagon.V6.vdmpybus.dv.acc
  160786, // llvm.hexagon.V6.vdmpybus.dv.acc.128B
  160823, // llvm.hexagon.V6.vdmpyhb
  160847, // llvm.hexagon.V6.vdmpyhb.128B
  160876, // llvm.hexagon.V6.vdmpyhb.acc
  160904, // llvm.hexagon.V6.vdmpyhb.acc.128B
  160937, // llvm.hexagon.V6.vdmpyhb.dv
  160964, // llvm.hexagon.V6.vdmpyhb.dv.128B
  160996, // llvm.hexagon.V6.vdmpyhb.dv.acc
  161027, // llvm.hexagon.V6.vdmpyhb.dv.acc.128B
  161063, // llvm.hexagon.V6.vdmpyhisat
  161090, // llvm.hexagon.V6.vdmpyhisat.128B
  161122, // llvm.hexagon.V6.vdmpyhisat.acc
  161153, // llvm.hexagon.V6.vdmpyhisat.acc.128B
  161189, // llvm.hexagon.V6.vdmpyhsat
  161215, // llvm.hexagon.V6.vdmpyhsat.128B
  161246, // llvm.hexagon.V6.vdmpyhsat.acc
  161276, // llvm.hexagon.V6.vdmpyhsat.acc.128B
  161311, // llvm.hexagon.V6.vdmpyhsuisat
  161340, // llvm.hexagon.V6.vdmpyhsuisat.128B
  161374, // llvm.hexagon.V6.vdmpyhsuisat.acc
  161407, // llvm.hexagon.V6.vdmpyhsuisat.acc.128B
  161445, // llvm.hexagon.V6.vdmpyhsusat
  161473, // llvm.hexagon.V6.vdmpyhsusat.128B
  161506, // llvm.hexagon.V6.vdmpyhsusat.acc
  161538, // llvm.hexagon.V6.vdmpyhsusat.acc.128B
  161575, // llvm.hexagon.V6.vdmpyhvsat
  161602, // llvm.hexagon.V6.vdmpyhvsat.128B
  161634, // llvm.hexagon.V6.vdmpyhvsat.acc
  161665, // llvm.hexagon.V6.vdmpyhvsat.acc.128B
  161701, // llvm.hexagon.V6.vdsaduh
  161725, // llvm.hexagon.V6.vdsaduh.128B
  161754, // llvm.hexagon.V6.vdsaduh.acc
  161782, // llvm.hexagon.V6.vdsaduh.acc.128B
  161815, // llvm.hexagon.V6.veqb
  161836, // llvm.hexagon.V6.veqb.128B
  161862, // llvm.hexagon.V6.veqb.and
  161887, // llvm.hexagon.V6.veqb.and.128B
  161917, // llvm.hexagon.V6.veqb.or
  161941, // llvm.hexagon.V6.veqb.or.128B
  161970, // llvm.hexagon.V6.veqb.xor
  161995, // llvm.hexagon.V6.veqb.xor.128B
  162025, // llvm.hexagon.V6.veqh
  162046, // llvm.hexagon.V6.veqh.128B
  162072, // llvm.hexagon.V6.veqh.and
  162097, // llvm.hexagon.V6.veqh.and.128B
  162127, // llvm.hexagon.V6.veqh.or
  162151, // llvm.hexagon.V6.veqh.or.128B
  162180, // llvm.hexagon.V6.veqh.xor
  162205, // llvm.hexagon.V6.veqh.xor.128B
  162235, // llvm.hexagon.V6.veqhf
  162257, // llvm.hexagon.V6.veqhf.128B
  162284, // llvm.hexagon.V6.veqhf.and
  162310, // llvm.hexagon.V6.veqhf.and.128B
  162341, // llvm.hexagon.V6.veqhf.or
  162366, // llvm.hexagon.V6.veqhf.or.128B
  162396, // llvm.hexagon.V6.veqhf.xor
  162422, // llvm.hexagon.V6.veqhf.xor.128B
  162453, // llvm.hexagon.V6.veqsf
  162475, // llvm.hexagon.V6.veqsf.128B
  162502, // llvm.hexagon.V6.veqsf.and
  162528, // llvm.hexagon.V6.veqsf.and.128B
  162559, // llvm.hexagon.V6.veqsf.or
  162584, // llvm.hexagon.V6.veqsf.or.128B
  162614, // llvm.hexagon.V6.veqsf.xor
  162640, // llvm.hexagon.V6.veqsf.xor.128B
  162671, // llvm.hexagon.V6.veqw
  162692, // llvm.hexagon.V6.veqw.128B
  162718, // llvm.hexagon.V6.veqw.and
  162743, // llvm.hexagon.V6.veqw.and.128B
  162773, // llvm.hexagon.V6.veqw.or
  162797, // llvm.hexagon.V6.veqw.or.128B
  162826, // llvm.hexagon.V6.veqw.xor
  162851, // llvm.hexagon.V6.veqw.xor.128B
  162881, // llvm.hexagon.V6.vfmax.f8
  162906, // llvm.hexagon.V6.vfmax.f8.128B
  162936, // llvm.hexagon.V6.vfmax.hf
  162961, // llvm.hexagon.V6.vfmax.hf.128B
  162991, // llvm.hexagon.V6.vfmax.sf
  163016, // llvm.hexagon.V6.vfmax.sf.128B
  163046, // llvm.hexagon.V6.vfmin.f8
  163071, // llvm.hexagon.V6.vfmin.f8.128B
  163101, // llvm.hexagon.V6.vfmin.hf
  163126, // llvm.hexagon.V6.vfmin.hf.128B
  163156, // llvm.hexagon.V6.vfmin.sf
  163181, // llvm.hexagon.V6.vfmin.sf.128B
  163211, // llvm.hexagon.V6.vfneg.f8
  163236, // llvm.hexagon.V6.vfneg.f8.128B
  163266, // llvm.hexagon.V6.vfneg.hf
  163291, // llvm.hexagon.V6.vfneg.hf.128B
  163321, // llvm.hexagon.V6.vfneg.sf
  163346, // llvm.hexagon.V6.vfneg.sf.128B
  163376, // llvm.hexagon.V6.vgather.vscattermh
  163411, // llvm.hexagon.V6.vgather.vscattermh.128B
  163451, // llvm.hexagon.V6.vgathermh
  163477, // llvm.hexagon.V6.vgathermh.128B
  163508, // llvm.hexagon.V6.vgathermhq
  163535, // llvm.hexagon.V6.vgathermhq.128B
  163567, // llvm.hexagon.V6.vgathermhw
  163594, // llvm.hexagon.V6.vgathermhw.128B
  163626, // llvm.hexagon.V6.vgathermhwq
  163654, // llvm.hexagon.V6.vgathermhwq.128B
  163687, // llvm.hexagon.V6.vgathermw
  163713, // llvm.hexagon.V6.vgathermw.128B
  163744, // llvm.hexagon.V6.vgathermwq
  163771, // llvm.hexagon.V6.vgathermwq.128B
  163803, // llvm.hexagon.V6.vgtb
  163824, // llvm.hexagon.V6.vgtb.128B
  163850, // llvm.hexagon.V6.vgtb.and
  163875, // llvm.hexagon.V6.vgtb.and.128B
  163905, // llvm.hexagon.V6.vgtb.or
  163929, // llvm.hexagon.V6.vgtb.or.128B
  163958, // llvm.hexagon.V6.vgtb.xor
  163983, // llvm.hexagon.V6.vgtb.xor.128B
  164013, // llvm.hexagon.V6.vgtbf
  164035, // llvm.hexagon.V6.vgtbf.128B
  164062, // llvm.hexagon.V6.vgtbf.and
  164088, // llvm.hexagon.V6.vgtbf.and.128B
  164119, // llvm.hexagon.V6.vgtbf.or
  164144, // llvm.hexagon.V6.vgtbf.or.128B
  164174, // llvm.hexagon.V6.vgtbf.xor
  164200, // llvm.hexagon.V6.vgtbf.xor.128B
  164231, // llvm.hexagon.V6.vgth
  164252, // llvm.hexagon.V6.vgth.128B
  164278, // llvm.hexagon.V6.vgth.and
  164303, // llvm.hexagon.V6.vgth.and.128B
  164333, // llvm.hexagon.V6.vgth.or
  164357, // llvm.hexagon.V6.vgth.or.128B
  164386, // llvm.hexagon.V6.vgth.xor
  164411, // llvm.hexagon.V6.vgth.xor.128B
  164441, // llvm.hexagon.V6.vgthf
  164463, // llvm.hexagon.V6.vgthf.128B
  164490, // llvm.hexagon.V6.vgthf.and
  164516, // llvm.hexagon.V6.vgthf.and.128B
  164547, // llvm.hexagon.V6.vgthf.or
  164572, // llvm.hexagon.V6.vgthf.or.128B
  164602, // llvm.hexagon.V6.vgthf.xor
  164628, // llvm.hexagon.V6.vgthf.xor.128B
  164659, // llvm.hexagon.V6.vgtsf
  164681, // llvm.hexagon.V6.vgtsf.128B
  164708, // llvm.hexagon.V6.vgtsf.and
  164734, // llvm.hexagon.V6.vgtsf.and.128B
  164765, // llvm.hexagon.V6.vgtsf.or
  164790, // llvm.hexagon.V6.vgtsf.or.128B
  164820, // llvm.hexagon.V6.vgtsf.xor
  164846, // llvm.hexagon.V6.vgtsf.xor.128B
  164877, // llvm.hexagon.V6.vgtub
  164899, // llvm.hexagon.V6.vgtub.128B
  164926, // llvm.hexagon.V6.vgtub.and
  164952, // llvm.hexagon.V6.vgtub.and.128B
  164983, // llvm.hexagon.V6.vgtub.or
  165008, // llvm.hexagon.V6.vgtub.or.128B
  165038, // llvm.hexagon.V6.vgtub.xor
  165064, // llvm.hexagon.V6.vgtub.xor.128B
  165095, // llvm.hexagon.V6.vgtuh
  165117, // llvm.hexagon.V6.vgtuh.128B
  165144, // llvm.hexagon.V6.vgtuh.and
  165170, // llvm.hexagon.V6.vgtuh.and.128B
  165201, // llvm.hexagon.V6.vgtuh.or
  165226, // llvm.hexagon.V6.vgtuh.or.128B
  165256, // llvm.hexagon.V6.vgtuh.xor
  165282, // llvm.hexagon.V6.vgtuh.xor.128B
  165313, // llvm.hexagon.V6.vgtuw
  165335, // llvm.hexagon.V6.vgtuw.128B
  165362, // llvm.hexagon.V6.vgtuw.and
  165388, // llvm.hexagon.V6.vgtuw.and.128B
  165419, // llvm.hexagon.V6.vgtuw.or
  165444, // llvm.hexagon.V6.vgtuw.or.128B
  165474, // llvm.hexagon.V6.vgtuw.xor
  165500, // llvm.hexagon.V6.vgtuw.xor.128B
  165531, // llvm.hexagon.V6.vgtw
  165552, // llvm.hexagon.V6.vgtw.128B
  165578, // llvm.hexagon.V6.vgtw.and
  165603, // llvm.hexagon.V6.vgtw.and.128B
  165633, // llvm.hexagon.V6.vgtw.or
  165657, // llvm.hexagon.V6.vgtw.or.128B
  165686, // llvm.hexagon.V6.vgtw.xor
  165711, // llvm.hexagon.V6.vgtw.xor.128B
  165741, // llvm.hexagon.V6.vilog2.hf
  165767, // llvm.hexagon.V6.vilog2.hf.128B
  165798, // llvm.hexagon.V6.vilog2.qf16
  165826, // llvm.hexagon.V6.vilog2.qf16.128B
  165859, // llvm.hexagon.V6.vilog2.qf32
  165887, // llvm.hexagon.V6.vilog2.qf32.128B
  165920, // llvm.hexagon.V6.vilog2.sf
  165946, // llvm.hexagon.V6.vilog2.sf.128B
  165977, // llvm.hexagon.V6.vinsertwr
  166003, // llvm.hexagon.V6.vinsertwr.128B
  166034, // llvm.hexagon.V6.vlalignb
  166059, // llvm.hexagon.V6.vlalignb.128B
  166089, // llvm.hexagon.V6.vlalignbi
  166115, // llvm.hexagon.V6.vlalignbi.128B
  166146, // llvm.hexagon.V6.vlsrb
  166168, // llvm.hexagon.V6.vlsrb.128B
  166195, // llvm.hexagon.V6.vlsrh
  166217, // llvm.hexagon.V6.vlsrh.128B
  166244, // llvm.hexagon.V6.vlsrhv
  166267, // llvm.hexagon.V6.vlsrhv.128B
  166295, // llvm.hexagon.V6.vlsrw
  166317, // llvm.hexagon.V6.vlsrw.128B
  166344, // llvm.hexagon.V6.vlsrwv
  166367, // llvm.hexagon.V6.vlsrwv.128B
  166395, // llvm.hexagon.V6.vlut4
  166417, // llvm.hexagon.V6.vlut4.128B
  166444, // llvm.hexagon.V6.vlutvvb
  166468, // llvm.hexagon.V6.vlutvvb.128B
  166497, // llvm.hexagon.V6.vlutvvb.nm
  166524, // llvm.hexagon.V6.vlutvvb.nm.128B
  166556, // llvm.hexagon.V6.vlutvvb.oracc
  166586, // llvm.hexagon.V6.vlutvvb.oracc.128B
  166621, // llvm.hexagon.V6.vlutvvb.oracci
  166652, // llvm.hexagon.V6.vlutvvb.oracci.128B
  166688, // llvm.hexagon.V6.vlutvvbi
  166713, // llvm.hexagon.V6.vlutvvbi.128B
  166743, // llvm.hexagon.V6.vlutvwh
  166767, // llvm.hexagon.V6.vlutvwh.128B
  166796, // llvm.hexagon.V6.vlutvwh.nm
  166823, // llvm.hexagon.V6.vlutvwh.nm.128B
  166855, // llvm.hexagon.V6.vlutvwh.oracc
  166885, // llvm.hexagon.V6.vlutvwh.oracc.128B
  166920, // llvm.hexagon.V6.vlutvwh.oracci
  166951, // llvm.hexagon.V6.vlutvwh.oracci.128B
  166987, // llvm.hexagon.V6.vlutvwhi
  167012, // llvm.hexagon.V6.vlutvwhi.128B
  167042, // llvm.hexagon.V6.vmaskedstorenq
  167073, // llvm.hexagon.V6.vmaskedstorenq.128B
  167109, // llvm.hexagon.V6.vmaskedstorentnq
  167142, // llvm.hexagon.V6.vmaskedstorentnq.128B
  167180, // llvm.hexagon.V6.vmaskedstorentq
  167212, // llvm.hexagon.V6.vmaskedstorentq.128B
  167249, // llvm.hexagon.V6.vmaskedstoreq
  167279, // llvm.hexagon.V6.vmaskedstoreq.128B
  167314, // llvm.hexagon.V6.vmax.bf
  167338, // llvm.hexagon.V6.vmax.bf.128B
  167367, // llvm.hexagon.V6.vmax.hf
  167391, // llvm.hexagon.V6.vmax.hf.128B
  167420, // llvm.hexagon.V6.vmax.sf
  167444, // llvm.hexagon.V6.vmax.sf.128B
  167473, // llvm.hexagon.V6.vmaxb
  167495, // llvm.hexagon.V6.vmaxb.128B
  167522, // llvm.hexagon.V6.vmaxh
  167544, // llvm.hexagon.V6.vmaxh.128B
  167571, // llvm.hexagon.V6.vmaxub
  167594, // llvm.hexagon.V6.vmaxub.128B
  167622, // llvm.hexagon.V6.vmaxuh
  167645, // llvm.hexagon.V6.vmaxuh.128B
  167673, // llvm.hexagon.V6.vmaxw
  167695, // llvm.hexagon.V6.vmaxw.128B
  167722, // llvm.hexagon.V6.vmerge.qf
  167748, // llvm.hexagon.V6.vmerge.qf.128B
  167779, // llvm.hexagon.V6.vmin.bf
  167803, // llvm.hexagon.V6.vmin.bf.128B
  167832, // llvm.hexagon.V6.vmin.hf
  167856, // llvm.hexagon.V6.vmin.hf.128B
  167885, // llvm.hexagon.V6.vmin.sf
  167909, // llvm.hexagon.V6.vmin.sf.128B
  167938, // llvm.hexagon.V6.vminb
  167960, // llvm.hexagon.V6.vminb.128B
  167987, // llvm.hexagon.V6.vminh
  168009, // llvm.hexagon.V6.vminh.128B
  168036, // llvm.hexagon.V6.vminub
  168059, // llvm.hexagon.V6.vminub.128B
  168087, // llvm.hexagon.V6.vminuh
  168110, // llvm.hexagon.V6.vminuh.128B
  168138, // llvm.hexagon.V6.vminw
  168160, // llvm.hexagon.V6.vminw.128B
  168187, // llvm.hexagon.V6.vmpabus
  168211, // llvm.hexagon.V6.vmpabus.128B
  168240, // llvm.hexagon.V6.vmpabus.acc
  168268, // llvm.hexagon.V6.vmpabus.acc.128B
  168301, // llvm.hexagon.V6.vmpabusv
  168326, // llvm.hexagon.V6.vmpabusv.128B
  168356, // llvm.hexagon.V6.vmpabuu
  168380, // llvm.hexagon.V6.vmpabuu.128B
  168409, // llvm.hexagon.V6.vmpabuu.acc
  168437, // llvm.hexagon.V6.vmpabuu.acc.128B
  168470, // llvm.hexagon.V6.vmpabuuv
  168495, // llvm.hexagon.V6.vmpabuuv.128B
  168525, // llvm.hexagon.V6.vmpahb
  168548, // llvm.hexagon.V6.vmpahb.128B
  168576, // llvm.hexagon.V6.vmpahb.acc
  168603, // llvm.hexagon.V6.vmpahb.acc.128B
  168635, // llvm.hexagon.V6.vmpahhsat
  168661, // llvm.hexagon.V6.vmpahhsat.128B
  168692, // llvm.hexagon.V6.vmpauhb
  168716, // llvm.hexagon.V6.vmpauhb.128B
  168745, // llvm.hexagon.V6.vmpauhb.acc
  168773, // llvm.hexagon.V6.vmpauhb.acc.128B
  168806, // llvm.hexagon.V6.vmpauhuhsat
  168834, // llvm.hexagon.V6.vmpauhuhsat.128B
  168867, // llvm.hexagon.V6.vmpsuhuhsat
  168895, // llvm.hexagon.V6.vmpsuhuhsat.128B
  168928, // llvm.hexagon.V6.vmpy.hf.f8
  168955, // llvm.hexagon.V6.vmpy.hf.f8.128B
  168987, // llvm.hexagon.V6.vmpy.hf.f8.acc
  169018, // llvm.hexagon.V6.vmpy.hf.f8.acc.128B
  169054, // llvm.hexagon.V6.vmpy.hf.hf
  169081, // llvm.hexagon.V6.vmpy.hf.hf.128B
  169113, // llvm.hexagon.V6.vmpy.hf.hf.acc
  169144, // llvm.hexagon.V6.vmpy.hf.hf.acc.128B
  169180, // llvm.hexagon.V6.vmpy.qf16
  169206, // llvm.hexagon.V6.vmpy.qf16.128B
  169237, // llvm.hexagon.V6.vmpy.qf16.hf
  169266, // llvm.hexagon.V6.vmpy.qf16.hf.128B
  169300, // llvm.hexagon.V6.vmpy.qf16.mix.hf
  169333, // llvm.hexagon.V6.vmpy.qf16.mix.hf.128B
  169371, // llvm.hexagon.V6.vmpy.qf32
  169397, // llvm.hexagon.V6.vmpy.qf32.128B
  169428, // llvm.hexagon.V6.vmpy.qf32.hf
  169457, // llvm.hexagon.V6.vmpy.qf32.hf.128B
  169491, // llvm.hexagon.V6.vmpy.qf32.mix.hf
  169524, // llvm.hexagon.V6.vmpy.qf32.mix.hf.128B
  169562, // llvm.hexagon.V6.vmpy.qf32.qf16
  169593, // llvm.hexagon.V6.vmpy.qf32.qf16.128B
  169629, // llvm.hexagon.V6.vmpy.qf32.sf
  169658, // llvm.hexagon.V6.vmpy.qf32.sf.128B
  169692, // llvm.hexagon.V6.vmpy.rt.hf
  169719, // llvm.hexagon.V6.vmpy.rt.hf.128B
  169751, // llvm.hexagon.V6.vmpy.rt.qf16
  169780, // llvm.hexagon.V6.vmpy.rt.qf16.128B
  169814, // llvm.hexagon.V6.vmpy.rt.sf
  169841, // llvm.hexagon.V6.vmpy.rt.sf.128B
  169873, // llvm.hexagon.V6.vmpy.sf.bf
  169900, // llvm.hexagon.V6.vmpy.sf.bf.128B
  169932, // llvm.hexagon.V6.vmpy.sf.bf.acc
  169963, // llvm.hexagon.V6.vmpy.sf.bf.acc.128B
  169999, // llvm.hexagon.V6.vmpy.sf.hf
  170026, // llvm.hexagon.V6.vmpy.sf.hf.128B
  170058, // llvm.hexagon.V6.vmpy.sf.hf.acc
  170089, // llvm.hexagon.V6.vmpy.sf.hf.acc.128B
  170125, // llvm.hexagon.V6.vmpy.sf.sf
  170152, // llvm.hexagon.V6.vmpy.sf.sf.128B
  170184, // llvm.hexagon.V6.vmpybus
  170208, // llvm.hexagon.V6.vmpybus.128B
  170237, // llvm.hexagon.V6.vmpybus.acc
  170265, // llvm.hexagon.V6.vmpybus.acc.128B
  170298, // llvm.hexagon.V6.vmpybusv
  170323, // llvm.hexagon.V6.vmpybusv.128B
  170353, // llvm.hexagon.V6.vmpybusv.acc
  170382, // llvm.hexagon.V6.vmpybusv.acc.128B
  170416, // llvm.hexagon.V6.vmpybv
  170439, // llvm.hexagon.V6.vmpybv.128B
  170467, // llvm.hexagon.V6.vmpybv.acc
  170494, // llvm.hexagon.V6.vmpybv.acc.128B
  170526, // llvm.hexagon.V6.vmpyewuh
  170551, // llvm.hexagon.V6.vmpyewuh.128B
  170581, // llvm.hexagon.V6.vmpyewuh.64
  170609, // llvm.hexagon.V6.vmpyewuh.64.128B
  170642, // llvm.hexagon.V6.vmpyh
  170664, // llvm.hexagon.V6.vmpyh.128B
  170691, // llvm.hexagon.V6.vmpyh.acc
  170717, // llvm.hexagon.V6.vmpyh.acc.128B
  170748, // llvm.hexagon.V6.vmpyhsat.acc
  170777, // llvm.hexagon.V6.vmpyhsat.acc.128B
  170811, // llvm.hexagon.V6.vmpyhsrs
  170836, // llvm.hexagon.V6.vmpyhsrs.128B
  170866, // llvm.hexagon.V6.vmpyhss
  170890, // llvm.hexagon.V6.vmpyhss.128B
  170919, // llvm.hexagon.V6.vmpyhus
  170943, // llvm.hexagon.V6.vmpyhus.128B
  170972, // llvm.hexagon.V6.vmpyhus.acc
  171000, // llvm.hexagon.V6.vmpyhus.acc.128B
  171033, // llvm.hexagon.V6.vmpyhv
  171056, // llvm.hexagon.V6.vmpyhv.128B
  171084, // llvm.hexagon.V6.vmpyhv.acc
  171111, // llvm.hexagon.V6.vmpyhv.acc.128B
  171143, // llvm.hexagon.V6.vmpyhvsrs
  171169, // llvm.hexagon.V6.vmpyhvsrs.128B
  171200, // llvm.hexagon.V6.vmpyieoh
  171225, // llvm.hexagon.V6.vmpyieoh.128B
  171255, // llvm.hexagon.V6.vmpyiewh.acc
  171284, // llvm.hexagon.V6.vmpyiewh.acc.128B
  171318, // llvm.hexagon.V6.vmpyiewuh
  171344, // llvm.hexagon.V6.vmpyiewuh.128B
  171375, // llvm.hexagon.V6.vmpyiewuh.acc
  171405, // llvm.hexagon.V6.vmpyiewuh.acc.128B
  171440, // llvm.hexagon.V6.vmpyih
  171463, // llvm.hexagon.V6.vmpyih.128B
  171491, // llvm.hexagon.V6.vmpyih.acc
  171518, // llvm.hexagon.V6.vmpyih.acc.128B
  171550, // llvm.hexagon.V6.vmpyihb
  171574, // llvm.hexagon.V6.vmpyihb.128B
  171603, // llvm.hexagon.V6.vmpyihb.acc
  171631, // llvm.hexagon.V6.vmpyihb.acc.128B
  171664, // llvm.hexagon.V6.vmpyiowh
  171689, // llvm.hexagon.V6.vmpyiowh.128B
  171719, // llvm.hexagon.V6.vmpyiwb
  171743, // llvm.hexagon.V6.vmpyiwb.128B
  171772, // llvm.hexagon.V6.vmpyiwb.acc
  171800, // llvm.hexagon.V6.vmpyiwb.acc.128B
  171833, // llvm.hexagon.V6.vmpyiwh
  171857, // llvm.hexagon.V6.vmpyiwh.128B
  171886, // llvm.hexagon.V6.vmpyiwh.acc
  171914, // llvm.hexagon.V6.vmpyiwh.acc.128B
  171947, // llvm.hexagon.V6.vmpyiwub
  171972, // llvm.hexagon.V6.vmpyiwub.128B
  172002, // llvm.hexagon.V6.vmpyiwub.acc
  172031, // llvm.hexagon.V6.vmpyiwub.acc.128B
  172065, // llvm.hexagon.V6.vmpyowh
  172089, // llvm.hexagon.V6.vmpyowh.128B
  172118, // llvm.hexagon.V6.vmpyowh.64.acc
  172149, // llvm.hexagon.V6.vmpyowh.64.acc.128B
  172185, // llvm.hexagon.V6.vmpyowh.rnd
  172213, // llvm.hexagon.V6.vmpyowh.rnd.128B
  172246, // llvm.hexagon.V6.vmpyowh.rnd.sacc
  172279, // llvm.hexagon.V6.vmpyowh.rnd.sacc.128B
  172317, // llvm.hexagon.V6.vmpyowh.sacc
  172346, // llvm.hexagon.V6.vmpyowh.sacc.128B
  172380, // llvm.hexagon.V6.vmpyss.parts
  172409, // llvm.hexagon.V6.vmpyss.parts.128B
  172443, // llvm.hexagon.V6.vmpyub
  172466, // llvm.hexagon.V6.vmpyub.128B
  172494, // llvm.hexagon.V6.vmpyub.acc
  172521, // llvm.hexagon.V6.vmpyub.acc.128B
  172553, // llvm.hexagon.V6.vmpyubv
  172577, // llvm.hexagon.V6.vmpyubv.128B
  172606, // llvm.hexagon.V6.vmpyubv.acc
  172634, // llvm.hexagon.V6.vmpyubv.acc.128B
  172667, // llvm.hexagon.V6.vmpyuh
  172690, // llvm.hexagon.V6.vmpyuh.128B
  172718, // llvm.hexagon.V6.vmpyuh.acc
  172745, // llvm.hexagon.V6.vmpyuh.acc.128B
  172777, // llvm.hexagon.V6.vmpyuhe
  172801, // llvm.hexagon.V6.vmpyuhe.128B
  172830, // llvm.hexagon.V6.vmpyuhe.acc
  172858, // llvm.hexagon.V6.vmpyuhe.acc.128B
  172891, // llvm.hexagon.V6.vmpyuhv
  172915, // llvm.hexagon.V6.vmpyuhv.128B
  172944, // llvm.hexagon.V6.vmpyuhv.acc
  172972, // llvm.hexagon.V6.vmpyuhv.acc.128B
  173005, // llvm.hexagon.V6.vmpyuhvs
  173030, // llvm.hexagon.V6.vmpyuhvs.128B
  173060, // llvm.hexagon.V6.vmpyus.parts
  173089, // llvm.hexagon.V6.vmpyus.parts.128B
  173123, // llvm.hexagon.V6.vmpyuu.parts
  173152, // llvm.hexagon.V6.vmpyuu.parts.128B
  173186, // llvm.hexagon.V6.vmux
  173207, // llvm.hexagon.V6.vmux.128B
  173233, // llvm.hexagon.V6.vnavgb
  173256, // llvm.hexagon.V6.vnavgb.128B
  173284, // llvm.hexagon.V6.vnavgh
  173307, // llvm.hexagon.V6.vnavgh.128B
  173335, // llvm.hexagon.V6.vnavgub
  173359, // llvm.hexagon.V6.vnavgub.128B
  173388, // llvm.hexagon.V6.vnavgw
  173411, // llvm.hexagon.V6.vnavgw.128B
  173439, // llvm.hexagon.V6.vneg.qf16.hf
  173468, // llvm.hexagon.V6.vneg.qf16.hf.128B
  173502, // llvm.hexagon.V6.vneg.qf16.qf16
  173533, // llvm.hexagon.V6.vneg.qf16.qf16.128B
  173569, // llvm.hexagon.V6.vneg.qf32.qf32
  173600, // llvm.hexagon.V6.vneg.qf32.qf32.128B
  173636, // llvm.hexagon.V6.vneg.qf32.sf
  173665, // llvm.hexagon.V6.vneg.qf32.sf.128B
  173699, // llvm.hexagon.V6.vnormamth
  173725, // llvm.hexagon.V6.vnormamth.128B
  173756, // llvm.hexagon.V6.vnormamtw
  173782, // llvm.hexagon.V6.vnormamtw.128B
  173813, // llvm.hexagon.V6.vnot
  173834, // llvm.hexagon.V6.vnot.128B
  173860, // llvm.hexagon.V6.vor
  173880, // llvm.hexagon.V6.vor.128B
  173905, // llvm.hexagon.V6.vpackeb
  173929, // llvm.hexagon.V6.vpackeb.128B
  173958, // llvm.hexagon.V6.vpackeh
  173982, // llvm.hexagon.V6.vpackeh.128B
  174011, // llvm.hexagon.V6.vpackhb.sat
  174039, // llvm.hexagon.V6.vpackhb.sat.128B
  174072, // llvm.hexagon.V6.vpackhub.sat
  174101, // llvm.hexagon.V6.vpackhub.sat.128B
  174135, // llvm.hexagon.V6.vpackob
  174159, // llvm.hexagon.V6.vpackob.128B
  174188, // llvm.hexagon.V6.vpackoh
  174212, // llvm.hexagon.V6.vpackoh.128B
  174241, // llvm.hexagon.V6.vpackwh.sat
  174269, // llvm.hexagon.V6.vpackwh.sat.128B
  174302, // llvm.hexagon.V6.vpackwuh.sat
  174331, // llvm.hexagon.V6.vpackwuh.sat.128B
  174365, // llvm.hexagon.V6.vpopcounth
  174392, // llvm.hexagon.V6.vpopcounth.128B
  174424, // llvm.hexagon.V6.vprefixqb
  174450, // llvm.hexagon.V6.vprefixqb.128B
  174481, // llvm.hexagon.V6.vprefixqh
  174507, // llvm.hexagon.V6.vprefixqh.128B
  174538, // llvm.hexagon.V6.vprefixqw
  174564, // llvm.hexagon.V6.vprefixqw.128B
  174595, // llvm.hexagon.V6.vrdelta
  174619, // llvm.hexagon.V6.vrdelta.128B
  174648, // llvm.hexagon.V6.vrmpybub.rtt
  174677, // llvm.hexagon.V6.vrmpybub.rtt.128B
  174711, // llvm.hexagon.V6.vrmpybub.rtt.acc
  174744, // llvm.hexagon.V6.vrmpybub.rtt.acc.128B
  174782, // llvm.hexagon.V6.vrmpybus
  174807, // llvm.hexagon.V6.vrmpybus.128B
  174837, // llvm.hexagon.V6.vrmpybus.acc
  174866, // llvm.hexagon.V6.vrmpybus.acc.128B
  174900, // llvm.hexagon.V6.vrmpybusi
  174926, // llvm.hexagon.V6.vrmpybusi.128B
  174957, // llvm.hexagon.V6.vrmpybusi.acc
  174987, // llvm.hexagon.V6.vrmpybusi.acc.128B
  175022, // llvm.hexagon.V6.vrmpybusv
  175048, // llvm.hexagon.V6.vrmpybusv.128B
  175079, // llvm.hexagon.V6.vrmpybusv.acc
  175109, // llvm.hexagon.V6.vrmpybusv.acc.128B
  175144, // llvm.hexagon.V6.vrmpybv
  175168, // llvm.hexagon.V6.vrmpybv.128B
  175197, // llvm.hexagon.V6.vrmpybv.acc
  175225, // llvm.hexagon.V6.vrmpybv.acc.128B
  175258, // llvm.hexagon.V6.vrmpyub
  175282, // llvm.hexagon.V6.vrmpyub.128B
  175311, // llvm.hexagon.V6.vrmpyub.acc
  175339, // llvm.hexagon.V6.vrmpyub.acc.128B
  175372, // llvm.hexagon.V6.vrmpyub.rtt
  175400, // llvm.hexagon.V6.vrmpyub.rtt.128B
  175433, // llvm.hexagon.V6.vrmpyub.rtt.acc
  175465, // llvm.hexagon.V6.vrmpyub.rtt.acc.128B
  175502, // llvm.hexagon.V6.vrmpyubi
  175527, // llvm.hexagon.V6.vrmpyubi.128B
  175557, // llvm.hexagon.V6.vrmpyubi.acc
  175586, // llvm.hexagon.V6.vrmpyubi.acc.128B
  175620, // llvm.hexagon.V6.vrmpyubv
  175645, // llvm.hexagon.V6.vrmpyubv.128B
  175675, // llvm.hexagon.V6.vrmpyubv.acc
  175704, // llvm.hexagon.V6.vrmpyubv.acc.128B
  175738, // llvm.hexagon.V6.vror
  175759, // llvm.hexagon.V6.vror.128B
  175785, // llvm.hexagon.V6.vrotr
  175807, // llvm.hexagon.V6.vrotr.128B
  175834, // llvm.hexagon.V6.vroundhb
  175859, // llvm.hexagon.V6.vroundhb.128B
  175889, // llvm.hexagon.V6.vroundhub
  175915, // llvm.hexagon.V6.vroundhub.128B
  175946, // llvm.hexagon.V6.vrounduhub
  175973, // llvm.hexagon.V6.vrounduhub.128B
  176005, // llvm.hexagon.V6.vrounduwuh
  176032, // llvm.hexagon.V6.vrounduwuh.128B
  176064, // llvm.hexagon.V6.vroundwh
  176089, // llvm.hexagon.V6.vroundwh.128B
  176119, // llvm.hexagon.V6.vroundwuh
  176145, // llvm.hexagon.V6.vroundwuh.128B
  176176, // llvm.hexagon.V6.vrsadubi
  176201, // llvm.hexagon.V6.vrsadubi.128B
  176231, // llvm.hexagon.V6.vrsadubi.acc
  176260, // llvm.hexagon.V6.vrsadubi.acc.128B
  176294, // llvm.hexagon.V6.vsatdw
  176317, // llvm.hexagon.V6.vsatdw.128B
  176345, // llvm.hexagon.V6.vsathub
  176369, // llvm.hexagon.V6.vsathub.128B
  176398, // llvm.hexagon.V6.vsatuwuh
  176423, // llvm.hexagon.V6.vsatuwuh.128B
  176453, // llvm.hexagon.V6.vsatwh
  176476, // llvm.hexagon.V6.vsatwh.128B
  176504, // llvm.hexagon.V6.vsb
  176524, // llvm.hexagon.V6.vsb.128B
  176549, // llvm.hexagon.V6.vscattermh
  176576, // llvm.hexagon.V6.vscattermh.128B
  176608, // llvm.hexagon.V6.vscattermh.add
  176639, // llvm.hexagon.V6.vscattermh.add.128B
  176675, // llvm.hexagon.V6.vscattermhq
  176703, // llvm.hexagon.V6.vscattermhq.128B
  176736, // llvm.hexagon.V6.vscattermhw
  176764, // llvm.hexagon.V6.vscattermhw.128B
  176797, // llvm.hexagon.V6.vscattermhw.add
  176829, // llvm.hexagon.V6.vscattermhw.add.128B
  176866, // llvm.hexagon.V6.vscattermhwq
  176895, // llvm.hexagon.V6.vscattermhwq.128B
  176929, // llvm.hexagon.V6.vscattermw
  176956, // llvm.hexagon.V6.vscattermw.128B
  176988, // llvm.hexagon.V6.vscattermw.add
  177019, // llvm.hexagon.V6.vscattermw.add.128B
  177055, // llvm.hexagon.V6.vscattermwq
  177083, // llvm.hexagon.V6.vscattermwq.128B
  177116, // llvm.hexagon.V6.vsh
  177136, // llvm.hexagon.V6.vsh.128B
  177161, // llvm.hexagon.V6.vshufeh
  177185, // llvm.hexagon.V6.vshufeh.128B
  177214, // llvm.hexagon.V6.vshuffb
  177238, // llvm.hexagon.V6.vshuffb.128B
  177267, // llvm.hexagon.V6.vshuffeb
  177292, // llvm.hexagon.V6.vshuffeb.128B
  177322, // llvm.hexagon.V6.vshuffh
  177346, // llvm.hexagon.V6.vshuffh.128B
  177375, // llvm.hexagon.V6.vshuffob
  177400, // llvm.hexagon.V6.vshuffob.128B
  177430, // llvm.hexagon.V6.vshuffvdd
  177456, // llvm.hexagon.V6.vshuffvdd.128B
  177487, // llvm.hexagon.V6.vshufoeb
  177512, // llvm.hexagon.V6.vshufoeb.128B
  177542, // llvm.hexagon.V6.vshufoeh
  177567, // llvm.hexagon.V6.vshufoeh.128B
  177597, // llvm.hexagon.V6.vshufoh
  177621, // llvm.hexagon.V6.vshufoh.128B
  177650, // llvm.hexagon.V6.vsub.hf
  177674, // llvm.hexagon.V6.vsub.hf.128B
  177703, // llvm.hexagon.V6.vsub.hf.f8
  177730, // llvm.hexagon.V6.vsub.hf.f8.128B
  177762, // llvm.hexagon.V6.vsub.hf.hf
  177789, // llvm.hexagon.V6.vsub.hf.hf.128B
  177821, // llvm.hexagon.V6.vsub.hf.mix
  177849, // llvm.hexagon.V6.vsub.hf.mix.128B
  177882, // llvm.hexagon.V6.vsub.qf16
  177908, // llvm.hexagon.V6.vsub.qf16.128B
  177939, // llvm.hexagon.V6.vsub.qf16.mix
  177969, // llvm.hexagon.V6.vsub.qf16.mix.128B
  178004, // llvm.hexagon.V6.vsub.qf32
  178030, // llvm.hexagon.V6.vsub.qf32.128B
  178061, // llvm.hexagon.V6.vsub.qf32.mix
  178091, // llvm.hexagon.V6.vsub.qf32.mix.128B
  178126, // llvm.hexagon.V6.vsub.sf
  178150, // llvm.hexagon.V6.vsub.sf.128B
  178179, // llvm.hexagon.V6.vsub.sf.bf
  178206, // llvm.hexagon.V6.vsub.sf.bf.128B
  178238, // llvm.hexagon.V6.vsub.sf.hf
  178265, // llvm.hexagon.V6.vsub.sf.hf.128B
  178297, // llvm.hexagon.V6.vsub.sf.mix
  178325, // llvm.hexagon.V6.vsub.sf.mix.128B
  178358, // llvm.hexagon.V6.vsub.sf.sf
  178385, // llvm.hexagon.V6.vsub.sf.sf.128B
  178417, // llvm.hexagon.V6.vsubb
  178439, // llvm.hexagon.V6.vsubb.128B
  178466, // llvm.hexagon.V6.vsubb.dv
  178491, // llvm.hexagon.V6.vsubb.dv.128B
  178521, // llvm.hexagon.V6.vsubbnq
  178545, // llvm.hexagon.V6.vsubbnq.128B
  178574, // llvm.hexagon.V6.vsubbq
  178597, // llvm.hexagon.V6.vsubbq.128B
  178625, // llvm.hexagon.V6.vsubbsat
  178650, // llvm.hexagon.V6.vsubbsat.128B
  178680, // llvm.hexagon.V6.vsubbsat.dv
  178708, // llvm.hexagon.V6.vsubbsat.dv.128B
  178741, // llvm.hexagon.V6.vsubcarry
  178767, // llvm.hexagon.V6.vsubcarry.128B
  178798, // llvm.hexagon.V6.vsubcarryo
  178825, // llvm.hexagon.V6.vsubcarryo.128B
  178857, // llvm.hexagon.V6.vsubh
  178879, // llvm.hexagon.V6.vsubh.128B
  178906, // llvm.hexagon.V6.vsubh.dv
  178931, // llvm.hexagon.V6.vsubh.dv.128B
  178961, // llvm.hexagon.V6.vsubhnq
  178985, // llvm.hexagon.V6.vsubhnq.128B
  179014, // llvm.hexagon.V6.vsubhq
  179037, // llvm.hexagon.V6.vsubhq.128B
  179065, // llvm.hexagon.V6.vsubhsat
  179090, // llvm.hexagon.V6.vsubhsat.128B
  179120, // llvm.hexagon.V6.vsubhsat.dv
  179148, // llvm.hexagon.V6.vsubhsat.dv.128B
  179181, // llvm.hexagon.V6.vsubhw
  179204, // llvm.hexagon.V6.vsubhw.128B
  179232, // llvm.hexagon.V6.vsububh
  179256, // llvm.hexagon.V6.vsububh.128B
  179285, // llvm.hexagon.V6.vsububsat
  179311, // llvm.hexagon.V6.vsububsat.128B
  179342, // llvm.hexagon.V6.vsububsat.dv
  179371, // llvm.hexagon.V6.vsububsat.dv.128B
  179405, // llvm.hexagon.V6.vsubububb.sat
  179435, // llvm.hexagon.V6.vsubububb.sat.128B
  179470, // llvm.hexagon.V6.vsubuhsat
  179496, // llvm.hexagon.V6.vsubuhsat.128B
  179527, // llvm.hexagon.V6.vsubuhsat.dv
  179556, // llvm.hexagon.V6.vsubuhsat.dv.128B
  179590, // llvm.hexagon.V6.vsubuhw
  179614, // llvm.hexagon.V6.vsubuhw.128B
  179643, // llvm.hexagon.V6.vsubuwsat
  179669, // llvm.hexagon.V6.vsubuwsat.128B
  179700, // llvm.hexagon.V6.vsubuwsat.dv
  179729, // llvm.hexagon.V6.vsubuwsat.dv.128B
  179763, // llvm.hexagon.V6.vsubw
  179785, // llvm.hexagon.V6.vsubw.128B
  179812, // llvm.hexagon.V6.vsubw.dv
  179837, // llvm.hexagon.V6.vsubw.dv.128B
  179867, // llvm.hexagon.V6.vsubwnq
  179891, // llvm.hexagon.V6.vsubwnq.128B
  179920, // llvm.hexagon.V6.vsubwq
  179943, // llvm.hexagon.V6.vsubwq.128B
  179971, // llvm.hexagon.V6.vsubwsat
  179996, // llvm.hexagon.V6.vsubwsat.128B
  180026, // llvm.hexagon.V6.vsubwsat.dv
  180054, // llvm.hexagon.V6.vsubwsat.dv.128B
  180087, // llvm.hexagon.V6.vswap
  180109, // llvm.hexagon.V6.vswap.128B
  180136, // llvm.hexagon.V6.vtmpyb
  180159, // llvm.hexagon.V6.vtmpyb.128B
  180187, // llvm.hexagon.V6.vtmpyb.acc
  180214, // llvm.hexagon.V6.vtmpyb.acc.128B
  180246, // llvm.hexagon.V6.vtmpybus
  180271, // llvm.hexagon.V6.vtmpybus.128B
  180301, // llvm.hexagon.V6.vtmpybus.acc
  180330, // llvm.hexagon.V6.vtmpybus.acc.128B
  180364, // llvm.hexagon.V6.vtmpyhb
  180388, // llvm.hexagon.V6.vtmpyhb.128B
  180417, // llvm.hexagon.V6.vtmpyhb.acc
  180445, // llvm.hexagon.V6.vtmpyhb.acc.128B
  180478, // llvm.hexagon.V6.vunpackb
  180503, // llvm.hexagon.V6.vunpackb.128B
  180533, // llvm.hexagon.V6.vunpackh
  180558, // llvm.hexagon.V6.vunpackh.128B
  180588, // llvm.hexagon.V6.vunpackob
  180614, // llvm.hexagon.V6.vunpackob.128B
  180645, // llvm.hexagon.V6.vunpackoh
  180671, // llvm.hexagon.V6.vunpackoh.128B
  180702, // llvm.hexagon.V6.vunpackub
  180728, // llvm.hexagon.V6.vunpackub.128B
  180759, // llvm.hexagon.V6.vunpackuh
  180785, // llvm.hexagon.V6.vunpackuh.128B
  180816, // llvm.hexagon.V6.vxor
  180837, // llvm.hexagon.V6.vxor.128B
  180863, // llvm.hexagon.V6.vzb
  180883, // llvm.hexagon.V6.vzb.128B
  180908, // llvm.hexagon.V6.vzh
  180928, // llvm.hexagon.V6.vzh.128B
  180953, // llvm.hexagon.Y2.dccleana
  180978, // llvm.hexagon.Y2.dccleaninva
  181006, // llvm.hexagon.Y2.dcfetch
  181030, // llvm.hexagon.Y2.dcinva
  181053, // llvm.hexagon.Y2.dczeroa
  181077, // llvm.hexagon.Y4.l2fetch
  181101, // llvm.hexagon.Y5.l2fetch
  181125, // llvm.hexagon.Y6.dmlink
  181148, // llvm.hexagon.Y6.dmpause
  181172, // llvm.hexagon.Y6.dmpoll
  181195, // llvm.hexagon.Y6.dmresume
  181220, // llvm.hexagon.Y6.dmstart
  181244, // llvm.hexagon.Y6.dmwait
  181267, // llvm.hexagon.circ.ldb
  181289, // llvm.hexagon.circ.ldd
  181311, // llvm.hexagon.circ.ldh
  181333, // llvm.hexagon.circ.ldub
  181356, // llvm.hexagon.circ.lduh
  181379, // llvm.hexagon.circ.ldw
  181401, // llvm.hexagon.circ.stb
  181423, // llvm.hexagon.circ.std
  181445, // llvm.hexagon.circ.sth
  181467, // llvm.hexagon.circ.sthhi
  181491, // llvm.hexagon.circ.stw
  181513, // llvm.hexagon.instrprof.custom
  181543, // llvm.hexagon.prefetch
  181565, // llvm.hexagon.vmemcpy
  181586, // llvm.hexagon.vmemset
  181607, // llvm.loongarch.asrtgt.d
  181631, // llvm.loongarch.asrtle.d
  181655, // llvm.loongarch.break
  181676, // llvm.loongarch.cacop.d
  181699, // llvm.loongarch.cacop.w
  181722, // llvm.loongarch.cpucfg
  181744, // llvm.loongarch.crc.w.b.w
  181769, // llvm.loongarch.crc.w.d.w
  181794, // llvm.loongarch.crc.w.h.w
  181819, // llvm.loongarch.crc.w.w.w
  181844, // llvm.loongarch.crcc.w.b.w
  181870, // llvm.loongarch.crcc.w.d.w
  181896, // llvm.loongarch.crcc.w.h.w
  181922, // llvm.loongarch.crcc.w.w.w
  181948, // llvm.loongarch.csrrd.d
  181971, // llvm.loongarch.csrrd.w
  181994, // llvm.loongarch.csrwr.d
  182017, // llvm.loongarch.csrwr.w
  182040, // llvm.loongarch.csrxchg.d
  182065, // llvm.loongarch.csrxchg.w
  182090, // llvm.loongarch.dbar
  182110, // llvm.loongarch.frecipe.d
  182135, // llvm.loongarch.frecipe.s
  182160, // llvm.loongarch.frsqrte.d
  182185, // llvm.loongarch.frsqrte.s
  182210, // llvm.loongarch.ibar
  182230, // llvm.loongarch.iocsrrd.b
  182255, // llvm.loongarch.iocsrrd.d
  182280, // llvm.loongarch.iocsrrd.h
  182305, // llvm.loongarch.iocsrrd.w
  182330, // llvm.loongarch.iocsrwr.b
  182355, // llvm.loongarch.iocsrwr.d
  182380, // llvm.loongarch.iocsrwr.h
  182405, // llvm.loongarch.iocsrwr.w
  182430, // llvm.loongarch.lasx.cast.128
  182459, // llvm.loongarch.lasx.cast.128.d
  182490, // llvm.loongarch.lasx.cast.128.s
  182521, // llvm.loongarch.lasx.concat.128
  182552, // llvm.loongarch.lasx.concat.128.d
  182585, // llvm.loongarch.lasx.concat.128.s
  182618, // llvm.loongarch.lasx.extract.128.hi
  182653, // llvm.loongarch.lasx.extract.128.hi.d
  182690, // llvm.loongarch.lasx.extract.128.hi.s
  182727, // llvm.loongarch.lasx.extract.128.lo
  182762, // llvm.loongarch.lasx.extract.128.lo.d
  182799, // llvm.loongarch.lasx.extract.128.lo.s
  182836, // llvm.loongarch.lasx.insert.128.hi
  182870, // llvm.loongarch.lasx.insert.128.hi.d
  182906, // llvm.loongarch.lasx.insert.128.hi.s
  182942, // llvm.loongarch.lasx.insert.128.lo
  182976, // llvm.loongarch.lasx.insert.128.lo.d
  183012, // llvm.loongarch.lasx.insert.128.lo.s
  183048, // llvm.loongarch.lasx.vext2xv.d.b
  183080, // llvm.loongarch.lasx.vext2xv.d.h
  183112, // llvm.loongarch.lasx.vext2xv.d.w
  183144, // llvm.loongarch.lasx.vext2xv.du.bu
  183178, // llvm.loongarch.lasx.vext2xv.du.hu
  183212, // llvm.loongarch.lasx.vext2xv.du.wu
  183246, // llvm.loongarch.lasx.vext2xv.h.b
  183278, // llvm.loongarch.lasx.vext2xv.hu.bu
  183312, // llvm.loongarch.lasx.vext2xv.w.b
  183344, // llvm.loongarch.lasx.vext2xv.w.h
  183376, // llvm.loongarch.lasx.vext2xv.wu.bu
  183410, // llvm.loongarch.lasx.vext2xv.wu.hu
  183444, // llvm.loongarch.lasx.xbnz.b
  183471, // llvm.loongarch.lasx.xbnz.d
  183498, // llvm.loongarch.lasx.xbnz.h
  183525, // llvm.loongarch.lasx.xbnz.v
  183552, // llvm.loongarch.lasx.xbnz.w
  183579, // llvm.loongarch.lasx.xbz.b
  183605, // llvm.loongarch.lasx.xbz.d
  183631, // llvm.loongarch.lasx.xbz.h
  183657, // llvm.loongarch.lasx.xbz.v
  183683, // llvm.loongarch.lasx.xbz.w
  183709, // llvm.loongarch.lasx.xvabsd.b
  183738, // llvm.loongarch.lasx.xvabsd.bu
  183768, // llvm.loongarch.lasx.xvabsd.d
  183797, // llvm.loongarch.lasx.xvabsd.du
  183827, // llvm.loongarch.lasx.xvabsd.h
  183856, // llvm.loongarch.lasx.xvabsd.hu
  183886, // llvm.loongarch.lasx.xvabsd.w
  183915, // llvm.loongarch.lasx.xvabsd.wu
  183945, // llvm.loongarch.lasx.xvadd.b
  183973, // llvm.loongarch.lasx.xvadd.d
  184001, // llvm.loongarch.lasx.xvadd.h
  184029, // llvm.loongarch.lasx.xvadd.q
  184057, // llvm.loongarch.lasx.xvadd.w
  184085, // llvm.loongarch.lasx.xvadda.b
  184114, // llvm.loongarch.lasx.xvadda.d
  184143, // llvm.loongarch.lasx.xvadda.h
  184172, // llvm.loongarch.lasx.xvadda.w
  184201, // llvm.loongarch.lasx.xvaddi.bu
  184231, // llvm.loongarch.lasx.xvaddi.du
  184261, // llvm.loongarch.lasx.xvaddi.hu
  184291, // llvm.loongarch.lasx.xvaddi.wu
  184321, // llvm.loongarch.lasx.xvaddwev.d.w
  184354, // llvm.loongarch.lasx.xvaddwev.d.wu
  184388, // llvm.loongarch.lasx.xvaddwev.d.wu.w
  184424, // llvm.loongarch.lasx.xvaddwev.h.b
  184457, // llvm.loongarch.lasx.xvaddwev.h.bu
  184491, // llvm.loongarch.lasx.xvaddwev.h.bu.b
  184527, // llvm.loongarch.lasx.xvaddwev.q.d
  184560, // llvm.loongarch.lasx.xvaddwev.q.du
  184594, // llvm.loongarch.lasx.xvaddwev.q.du.d
  184630, // llvm.loongarch.lasx.xvaddwev.w.h
  184663, // llvm.loongarch.lasx.xvaddwev.w.hu
  184697, // llvm.loongarch.lasx.xvaddwev.w.hu.h
  184733, // llvm.loongarch.lasx.xvaddwod.d.w
  184766, // llvm.loongarch.lasx.xvaddwod.d.wu
  184800, // llvm.loongarch.lasx.xvaddwod.d.wu.w
  184836, // llvm.loongarch.lasx.xvaddwod.h.b
  184869, // llvm.loongarch.lasx.xvaddwod.h.bu
  184903, // llvm.loongarch.lasx.xvaddwod.h.bu.b
  184939, // llvm.loongarch.lasx.xvaddwod.q.d
  184972, // llvm.loongarch.lasx.xvaddwod.q.du
  185006, // llvm.loongarch.lasx.xvaddwod.q.du.d
  185042, // llvm.loongarch.lasx.xvaddwod.w.h
  185075, // llvm.loongarch.lasx.xvaddwod.w.hu
  185109, // llvm.loongarch.lasx.xvaddwod.w.hu.h
  185145, // llvm.loongarch.lasx.xvand.v
  185173, // llvm.loongarch.lasx.xvandi.b
  185202, // llvm.loongarch.lasx.xvandn.v
  185231, // llvm.loongarch.lasx.xvavg.b
  185259, // llvm.loongarch.lasx.xvavg.bu
  185288, // llvm.loongarch.lasx.xvavg.d
  185316, // llvm.loongarch.lasx.xvavg.du
  185345, // llvm.loongarch.lasx.xvavg.h
  185373, // llvm.loongarch.lasx.xvavg.hu
  185402, // llvm.loongarch.lasx.xvavg.w
  185430, // llvm.loongarch.lasx.xvavg.wu
  185459, // llvm.loongarch.lasx.xvavgr.b
  185488, // llvm.loongarch.lasx.xvavgr.bu
  185518, // llvm.loongarch.lasx.xvavgr.d
  185547, // llvm.loongarch.lasx.xvavgr.du
  185577, // llvm.loongarch.lasx.xvavgr.h
  185606, // llvm.loongarch.lasx.xvavgr.hu
  185636, // llvm.loongarch.lasx.xvavgr.w
  185665, // llvm.loongarch.lasx.xvavgr.wu
  185695, // llvm.loongarch.lasx.xvbitclr.b
  185726, // llvm.loongarch.lasx.xvbitclr.d
  185757, // llvm.loongarch.lasx.xvbitclr.h
  185788, // llvm.loongarch.lasx.xvbitclr.w
  185819, // llvm.loongarch.lasx.xvbitclri.b
  185851, // llvm.loongarch.lasx.xvbitclri.d
  185883, // llvm.loongarch.lasx.xvbitclri.h
  185915, // llvm.loongarch.lasx.xvbitclri.w
  185947, // llvm.loongarch.lasx.xvbitrev.b
  185978, // llvm.loongarch.lasx.xvbitrev.d
  186009, // llvm.loongarch.lasx.xvbitrev.h
  186040, // llvm.loongarch.lasx.xvbitrev.w
  186071, // llvm.loongarch.lasx.xvbitrevi.b
  186103, // llvm.loongarch.lasx.xvbitrevi.d
  186135, // llvm.loongarch.lasx.xvbitrevi.h
  186167, // llvm.loongarch.lasx.xvbitrevi.w
  186199, // llvm.loongarch.lasx.xvbitsel.v
  186230, // llvm.loongarch.lasx.xvbitseli.b
  186262, // llvm.loongarch.lasx.xvbitset.b
  186293, // llvm.loongarch.lasx.xvbitset.d
  186324, // llvm.loongarch.lasx.xvbitset.h
  186355, // llvm.loongarch.lasx.xvbitset.w
  186386, // llvm.loongarch.lasx.xvbitseti.b
  186418, // llvm.loongarch.lasx.xvbitseti.d
  186450, // llvm.loongarch.lasx.xvbitseti.h
  186482, // llvm.loongarch.lasx.xvbitseti.w
  186514, // llvm.loongarch.lasx.xvbsll.v
  186543, // llvm.loongarch.lasx.xvbsrl.v
  186572, // llvm.loongarch.lasx.xvclo.b
  186600, // llvm.loongarch.lasx.xvclo.d
  186628, // llvm.loongarch.lasx.xvclo.h
  186656, // llvm.loongarch.lasx.xvclo.w
  186684, // llvm.loongarch.lasx.xvclz.b
  186712, // llvm.loongarch.lasx.xvclz.d
  186740, // llvm.loongarch.lasx.xvclz.h
  186768, // llvm.loongarch.lasx.xvclz.w
  186796, // llvm.loongarch.lasx.xvdiv.b
  186824, // llvm.loongarch.lasx.xvdiv.bu
  186853, // llvm.loongarch.lasx.xvdiv.d
  186881, // llvm.loongarch.lasx.xvdiv.du
  186910, // llvm.loongarch.lasx.xvdiv.h
  186938, // llvm.loongarch.lasx.xvdiv.hu
  186967, // llvm.loongarch.lasx.xvdiv.w
  186995, // llvm.loongarch.lasx.xvdiv.wu
  187024, // llvm.loongarch.lasx.xvexth.d.w
  187055, // llvm.loongarch.lasx.xvexth.du.wu
  187088, // llvm.loongarch.lasx.xvexth.h.b
  187119, // llvm.loongarch.lasx.xvexth.hu.bu
  187152, // llvm.loongarch.lasx.xvexth.q.d
  187183, // llvm.loongarch.lasx.xvexth.qu.du
  187216, // llvm.loongarch.lasx.xvexth.w.h
  187247, // llvm.loongarch.lasx.xvexth.wu.hu
  187280, // llvm.loongarch.lasx.xvextl.q.d
  187311, // llvm.loongarch.lasx.xvextl.qu.du
  187344, // llvm.loongarch.lasx.xvextrins.b
  187376, // llvm.loongarch.lasx.xvextrins.d
  187408, // llvm.loongarch.lasx.xvextrins.h
  187440, // llvm.loongarch.lasx.xvextrins.w
  187472, // llvm.loongarch.lasx.xvfadd.d
  187501, // llvm.loongarch.lasx.xvfadd.s
  187530, // llvm.loongarch.lasx.xvfclass.d
  187561, // llvm.loongarch.lasx.xvfclass.s
  187592, // llvm.loongarch.lasx.xvfcmp.caf.d
  187625, // llvm.loongarch.lasx.xvfcmp.caf.s
  187658, // llvm.loongarch.lasx.xvfcmp.ceq.d
  187691, // llvm.loongarch.lasx.xvfcmp.ceq.s
  187724, // llvm.loongarch.lasx.xvfcmp.cle.d
  187757, // llvm.loongarch.lasx.xvfcmp.cle.s
  187790, // llvm.loongarch.lasx.xvfcmp.clt.d
  187823, // llvm.loongarch.lasx.xvfcmp.clt.s
  187856, // llvm.loongarch.lasx.xvfcmp.cne.d
  187889, // llvm.loongarch.lasx.xvfcmp.cne.s
  187922, // llvm.loongarch.lasx.xvfcmp.cor.d
  187955, // llvm.loongarch.lasx.xvfcmp.cor.s
  187988, // llvm.loongarch.lasx.xvfcmp.cueq.d
  188022, // llvm.loongarch.lasx.xvfcmp.cueq.s
  188056, // llvm.loongarch.lasx.xvfcmp.cule.d
  188090, // llvm.loongarch.lasx.xvfcmp.cule.s
  188124, // llvm.loongarch.lasx.xvfcmp.cult.d
  188158, // llvm.loongarch.lasx.xvfcmp.cult.s
  188192, // llvm.loongarch.lasx.xvfcmp.cun.d
  188225, // llvm.loongarch.lasx.xvfcmp.cun.s
  188258, // llvm.loongarch.lasx.xvfcmp.cune.d
  188292, // llvm.loongarch.lasx.xvfcmp.cune.s
  188326, // llvm.loongarch.lasx.xvfcmp.saf.d
  188359, // llvm.loongarch.lasx.xvfcmp.saf.s
  188392, // llvm.loongarch.lasx.xvfcmp.seq.d
  188425, // llvm.loongarch.lasx.xvfcmp.seq.s
  188458, // llvm.loongarch.lasx.xvfcmp.sle.d
  188491, // llvm.loongarch.lasx.xvfcmp.sle.s
  188524, // llvm.loongarch.lasx.xvfcmp.slt.d
  188557, // llvm.loongarch.lasx.xvfcmp.slt.s
  188590, // llvm.loongarch.lasx.xvfcmp.sne.d
  188623, // llvm.loongarch.lasx.xvfcmp.sne.s
  188656, // llvm.loongarch.lasx.xvfcmp.sor.d
  188689, // llvm.loongarch.lasx.xvfcmp.sor.s
  188722, // llvm.loongarch.lasx.xvfcmp.sueq.d
  188756, // llvm.loongarch.lasx.xvfcmp.sueq.s
  188790, // llvm.loongarch.lasx.xvfcmp.sule.d
  188824, // llvm.loongarch.lasx.xvfcmp.sule.s
  188858, // llvm.loongarch.lasx.xvfcmp.sult.d
  188892, // llvm.loongarch.lasx.xvfcmp.sult.s
  188926, // llvm.loongarch.lasx.xvfcmp.sun.d
  188959, // llvm.loongarch.lasx.xvfcmp.sun.s
  188992, // llvm.loongarch.lasx.xvfcmp.sune.d
  189026, // llvm.loongarch.lasx.xvfcmp.sune.s
  189060, // llvm.loongarch.lasx.xvfcvt.h.s
  189091, // llvm.loongarch.lasx.xvfcvt.s.d
  189122, // llvm.loongarch.lasx.xvfcvth.d.s
  189154, // llvm.loongarch.lasx.xvfcvth.s.h
  189186, // llvm.loongarch.lasx.xvfcvtl.d.s
  189218, // llvm.loongarch.lasx.xvfcvtl.s.h
  189250, // llvm.loongarch.lasx.xvfdiv.d
  189279, // llvm.loongarch.lasx.xvfdiv.s
  189308, // llvm.loongarch.lasx.xvffint.d.l
  189340, // llvm.loongarch.lasx.xvffint.d.lu
  189373, // llvm.loongarch.lasx.xvffint.s.l
  189405, // llvm.loongarch.lasx.xvffint.s.w
  189437, // llvm.loongarch.lasx.xvffint.s.wu
  189470, // llvm.loongarch.lasx.xvffinth.d.w
  189503, // llvm.loongarch.lasx.xvffintl.d.w
  189536, // llvm.loongarch.lasx.xvflogb.d
  189566, // llvm.loongarch.lasx.xvflogb.s
  189596, // llvm.loongarch.lasx.xvfmadd.d
  189626, // llvm.loongarch.lasx.xvfmadd.s
  189656, // llvm.loongarch.lasx.xvfmax.d
  189685, // llvm.loongarch.lasx.xvfmax.s
  189714, // llvm.loongarch.lasx.xvfmaxa.d
  189744, // llvm.loongarch.lasx.xvfmaxa.s
  189774, // llvm.loongarch.lasx.xvfmin.d
  189803, // llvm.loongarch.lasx.xvfmin.s
  189832, // llvm.loongarch.lasx.xvfmina.d
  189862, // llvm.loongarch.lasx.xvfmina.s
  189892, // llvm.loongarch.lasx.xvfmsub.d
  189922, // llvm.loongarch.lasx.xvfmsub.s
  189952, // llvm.loongarch.lasx.xvfmul.d
  189981, // llvm.loongarch.lasx.xvfmul.s
  190010, // llvm.loongarch.lasx.xvfnmadd.d
  190041, // llvm.loongarch.lasx.xvfnmadd.s
  190072, // llvm.loongarch.lasx.xvfnmsub.d
  190103, // llvm.loongarch.lasx.xvfnmsub.s
  190134, // llvm.loongarch.lasx.xvfrecip.d
  190165, // llvm.loongarch.lasx.xvfrecip.s
  190196, // llvm.loongarch.lasx.xvfrecipe.d
  190228, // llvm.loongarch.lasx.xvfrecipe.s
  190260, // llvm.loongarch.lasx.xvfrint.d
  190290, // llvm.loongarch.lasx.xvfrint.s
  190320, // llvm.loongarch.lasx.xvfrintrm.d
  190352, // llvm.loongarch.lasx.xvfrintrm.s
  190384, // llvm.loongarch.lasx.xvfrintrne.d
  190417, // llvm.loongarch.lasx.xvfrintrne.s
  190450, // llvm.loongarch.lasx.xvfrintrp.d
  190482, // llvm.loongarch.lasx.xvfrintrp.s
  190514, // llvm.loongarch.lasx.xvfrintrz.d
  190546, // llvm.loongarch.lasx.xvfrintrz.s
  190578, // llvm.loongarch.lasx.xvfrsqrt.d
  190609, // llvm.loongarch.lasx.xvfrsqrt.s
  190640, // llvm.loongarch.lasx.xvfrsqrte.d
  190672, // llvm.loongarch.lasx.xvfrsqrte.s
  190704, // llvm.loongarch.lasx.xvfrstp.b
  190734, // llvm.loongarch.lasx.xvfrstp.h
  190764, // llvm.loongarch.lasx.xvfrstpi.b
  190795, // llvm.loongarch.lasx.xvfrstpi.h
  190826, // llvm.loongarch.lasx.xvfsqrt.d
  190856, // llvm.loongarch.lasx.xvfsqrt.s
  190886, // llvm.loongarch.lasx.xvfsub.d
  190915, // llvm.loongarch.lasx.xvfsub.s
  190944, // llvm.loongarch.lasx.xvftint.l.d
  190976, // llvm.loongarch.lasx.xvftint.lu.d
  191009, // llvm.loongarch.lasx.xvftint.w.d
  191041, // llvm.loongarch.lasx.xvftint.w.s
  191073, // llvm.loongarch.lasx.xvftint.wu.s
  191106, // llvm.loongarch.lasx.xvftinth.l.s
  191139, // llvm.loongarch.lasx.xvftintl.l.s
  191172, // llvm.loongarch.lasx.xvftintrm.l.d
  191206, // llvm.loongarch.lasx.xvftintrm.w.d
  191240, // llvm.loongarch.lasx.xvftintrm.w.s
  191274, // llvm.loongarch.lasx.xvftintrmh.l.s
  191309, // llvm.loongarch.lasx.xvftintrml.l.s
  191344, // llvm.loongarch.lasx.xvftintrne.l.d
  191379, // llvm.loongarch.lasx.xvftintrne.w.d
  191414, // llvm.loongarch.lasx.xvftintrne.w.s
  191449, // llvm.loongarch.lasx.xvftintrneh.l.s
  191485, // llvm.loongarch.lasx.xvftintrnel.l.s
  191521, // llvm.loongarch.lasx.xvftintrp.l.d
  191555, // llvm.loongarch.lasx.xvftintrp.w.d
  191589, // llvm.loongarch.lasx.xvftintrp.w.s
  191623, // llvm.loongarch.lasx.xvftintrph.l.s
  191658, // llvm.loongarch.lasx.xvftintrpl.l.s
  191693, // llvm.loongarch.lasx.xvftintrz.l.d
  191727, // llvm.loongarch.lasx.xvftintrz.lu.d
  191762, // llvm.loongarch.lasx.xvftintrz.w.d
  191796, // llvm.loongarch.lasx.xvftintrz.w.s
  191830, // llvm.loongarch.lasx.xvftintrz.wu.s
  191865, // llvm.loongarch.lasx.xvftintrzh.l.s
  191900, // llvm.loongarch.lasx.xvftintrzl.l.s
  191935, // llvm.loongarch.lasx.xvhaddw.d.w
  191967, // llvm.loongarch.lasx.xvhaddw.du.wu
  192001, // llvm.loongarch.lasx.xvhaddw.h.b
  192033, // llvm.loongarch.lasx.xvhaddw.hu.bu
  192067, // llvm.loongarch.lasx.xvhaddw.q.d
  192099, // llvm.loongarch.lasx.xvhaddw.qu.du
  192133, // llvm.loongarch.lasx.xvhaddw.w.h
  192165, // llvm.loongarch.lasx.xvhaddw.wu.hu
  192199, // llvm.loongarch.lasx.xvhsubw.d.w
  192231, // llvm.loongarch.lasx.xvhsubw.du.wu
  192265, // llvm.loongarch.lasx.xvhsubw.h.b
  192297, // llvm.loongarch.lasx.xvhsubw.hu.bu
  192331, // llvm.loongarch.lasx.xvhsubw.q.d
  192363, // llvm.loongarch.lasx.xvhsubw.qu.du
  192397, // llvm.loongarch.lasx.xvhsubw.w.h
  192429, // llvm.loongarch.lasx.xvhsubw.wu.hu
  192463, // llvm.loongarch.lasx.xvilvh.b
  192492, // llvm.loongarch.lasx.xvilvh.d
  192521, // llvm.loongarch.lasx.xvilvh.h
  192550, // llvm.loongarch.lasx.xvilvh.w
  192579, // llvm.loongarch.lasx.xvilvl.b
  192608, // llvm.loongarch.lasx.xvilvl.d
  192637, // llvm.loongarch.lasx.xvilvl.h
  192666, // llvm.loongarch.lasx.xvilvl.w
  192695, // llvm.loongarch.lasx.xvinsgr2vr.d
  192728, // llvm.loongarch.lasx.xvinsgr2vr.w
  192761, // llvm.loongarch.lasx.xvinsve0.d
  192792, // llvm.loongarch.lasx.xvinsve0.w
  192823, // llvm.loongarch.lasx.xvld
  192848, // llvm.loongarch.lasx.xvldi
  192874, // llvm.loongarch.lasx.xvldrepl.b
  192905, // llvm.loongarch.lasx.xvldrepl.d
  192936, // llvm.loongarch.lasx.xvldrepl.h
  192967, // llvm.loongarch.lasx.xvldrepl.w
  192998, // llvm.loongarch.lasx.xvldx
  193024, // llvm.loongarch.lasx.xvmadd.b
  193053, // llvm.loongarch.lasx.xvmadd.d
  193082, // llvm.loongarch.lasx.xvmadd.h
  193111, // llvm.loongarch.lasx.xvmadd.w
  193140, // llvm.loongarch.lasx.xvmaddwev.d.w
  193174, // llvm.loongarch.lasx.xvmaddwev.d.wu
  193209, // llvm.loongarch.lasx.xvmaddwev.d.wu.w
  193246, // llvm.loongarch.lasx.xvmaddwev.h.b
  193280, // llvm.loongarch.lasx.xvmaddwev.h.bu
  193315, // llvm.loongarch.lasx.xvmaddwev.h.bu.b
  193352, // llvm.loongarch.lasx.xvmaddwev.q.d
  193386, // llvm.loongarch.lasx.xvmaddwev.q.du
  193421, // llvm.loongarch.lasx.xvmaddwev.q.du.d
  193458, // llvm.loongarch.lasx.xvmaddwev.w.h
  193492, // llvm.loongarch.lasx.xvmaddwev.w.hu
  193527, // llvm.loongarch.lasx.xvmaddwev.w.hu.h
  193564, // llvm.loongarch.lasx.xvmaddwod.d.w
  193598, // llvm.loongarch.lasx.xvmaddwod.d.wu
  193633, // llvm.loongarch.lasx.xvmaddwod.d.wu.w
  193670, // llvm.loongarch.lasx.xvmaddwod.h.b
  193704, // llvm.loongarch.lasx.xvmaddwod.h.bu
  193739, // llvm.loongarch.lasx.xvmaddwod.h.bu.b
  193776, // llvm.loongarch.lasx.xvmaddwod.q.d
  193810, // llvm.loongarch.lasx.xvmaddwod.q.du
  193845, // llvm.loongarch.lasx.xvmaddwod.q.du.d
  193882, // llvm.loongarch.lasx.xvmaddwod.w.h
  193916, // llvm.loongarch.lasx.xvmaddwod.w.hu
  193951, // llvm.loongarch.lasx.xvmaddwod.w.hu.h
  193988, // llvm.loongarch.lasx.xvmax.b
  194016, // llvm.loongarch.lasx.xvmax.bu
  194045, // llvm.loongarch.lasx.xvmax.d
  194073, // llvm.loongarch.lasx.xvmax.du
  194102, // llvm.loongarch.lasx.xvmax.h
  194130, // llvm.loongarch.lasx.xvmax.hu
  194159, // llvm.loongarch.lasx.xvmax.w
  194187, // llvm.loongarch.lasx.xvmax.wu
  194216, // llvm.loongarch.lasx.xvmaxi.b
  194245, // llvm.loongarch.lasx.xvmaxi.bu
  194275, // llvm.loongarch.lasx.xvmaxi.d
  194304, // llvm.loongarch.lasx.xvmaxi.du
  194334, // llvm.loongarch.lasx.xvmaxi.h
  194363, // llvm.loongarch.lasx.xvmaxi.hu
  194393, // llvm.loongarch.lasx.xvmaxi.w
  194422, // llvm.loongarch.lasx.xvmaxi.wu
  194452, // llvm.loongarch.lasx.xvmin.b
  194480, // llvm.loongarch.lasx.xvmin.bu
  194509, // llvm.loongarch.lasx.xvmin.d
  194537, // llvm.loongarch.lasx.xvmin.du
  194566, // llvm.loongarch.lasx.xvmin.h
  194594, // llvm.loongarch.lasx.xvmin.hu
  194623, // llvm.loongarch.lasx.xvmin.w
  194651, // llvm.loongarch.lasx.xvmin.wu
  194680, // llvm.loongarch.lasx.xvmini.b
  194709, // llvm.loongarch.lasx.xvmini.bu
  194739, // llvm.loongarch.lasx.xvmini.d
  194768, // llvm.loongarch.lasx.xvmini.du
  194798, // llvm.loongarch.lasx.xvmini.h
  194827, // llvm.loongarch.lasx.xvmini.hu
  194857, // llvm.loongarch.lasx.xvmini.w
  194886, // llvm.loongarch.lasx.xvmini.wu
  194916, // llvm.loongarch.lasx.xvmod.b
  194944, // llvm.loongarch.lasx.xvmod.bu
  194973, // llvm.loongarch.lasx.xvmod.d
  195001, // llvm.loongarch.lasx.xvmod.du
  195030, // llvm.loongarch.lasx.xvmod.h
  195058, // llvm.loongarch.lasx.xvmod.hu
  195087, // llvm.loongarch.lasx.xvmod.w
  195115, // llvm.loongarch.lasx.xvmod.wu
  195144, // llvm.loongarch.lasx.xvmskgez.b
  195175, // llvm.loongarch.lasx.xvmskltz.b
  195206, // llvm.loongarch.lasx.xvmskltz.d
  195237, // llvm.loongarch.lasx.xvmskltz.h
  195268, // llvm.loongarch.lasx.xvmskltz.w
  195299, // llvm.loongarch.lasx.xvmsknz.b
  195329, // llvm.loongarch.lasx.xvmsub.b
  195358, // llvm.loongarch.lasx.xvmsub.d
  195387, // llvm.loongarch.lasx.xvmsub.h
  195416, // llvm.loongarch.lasx.xvmsub.w
  195445, // llvm.loongarch.lasx.xvmuh.b
  195473, // llvm.loongarch.lasx.xvmuh.bu
  195502, // llvm.loongarch.lasx.xvmuh.d
  195530, // llvm.loongarch.lasx.xvmuh.du
  195559, // llvm.loongarch.lasx.xvmuh.h
  195587, // llvm.loongarch.lasx.xvmuh.hu
  195616, // llvm.loongarch.lasx.xvmuh.w
  195644, // llvm.loongarch.lasx.xvmuh.wu
  195673, // llvm.loongarch.lasx.xvmul.b
  195701, // llvm.loongarch.lasx.xvmul.d
  195729, // llvm.loongarch.lasx.xvmul.h
  195757, // llvm.loongarch.lasx.xvmul.w
  195785, // llvm.loongarch.lasx.xvmulwev.d.w
  195818, // llvm.loongarch.lasx.xvmulwev.d.wu
  195852, // llvm.loongarch.lasx.xvmulwev.d.wu.w
  195888, // llvm.loongarch.lasx.xvmulwev.h.b
  195921, // llvm.loongarch.lasx.xvmulwev.h.bu
  195955, // llvm.loongarch.lasx.xvmulwev.h.bu.b
  195991, // llvm.loongarch.lasx.xvmulwev.q.d
  196024, // llvm.loongarch.lasx.xvmulwev.q.du
  196058, // llvm.loongarch.lasx.xvmulwev.q.du.d
  196094, // llvm.loongarch.lasx.xvmulwev.w.h
  196127, // llvm.loongarch.lasx.xvmulwev.w.hu
  196161, // llvm.loongarch.lasx.xvmulwev.w.hu.h
  196197, // llvm.loongarch.lasx.xvmulwod.d.w
  196230, // llvm.loongarch.lasx.xvmulwod.d.wu
  196264, // llvm.loongarch.lasx.xvmulwod.d.wu.w
  196300, // llvm.loongarch.lasx.xvmulwod.h.b
  196333, // llvm.loongarch.lasx.xvmulwod.h.bu
  196367, // llvm.loongarch.lasx.xvmulwod.h.bu.b
  196403, // llvm.loongarch.lasx.xvmulwod.q.d
  196436, // llvm.loongarch.lasx.xvmulwod.q.du
  196470, // llvm.loongarch.lasx.xvmulwod.q.du.d
  196506, // llvm.loongarch.lasx.xvmulwod.w.h
  196539, // llvm.loongarch.lasx.xvmulwod.w.hu
  196573, // llvm.loongarch.lasx.xvmulwod.w.hu.h
  196609, // llvm.loongarch.lasx.xvneg.b
  196637, // llvm.loongarch.lasx.xvneg.d
  196665, // llvm.loongarch.lasx.xvneg.h
  196693, // llvm.loongarch.lasx.xvneg.w
  196721, // llvm.loongarch.lasx.xvnor.v
  196749, // llvm.loongarch.lasx.xvnori.b
  196778, // llvm.loongarch.lasx.xvor.v
  196805, // llvm.loongarch.lasx.xvori.b
  196833, // llvm.loongarch.lasx.xvorn.v
  196861, // llvm.loongarch.lasx.xvpackev.b
  196892, // llvm.loongarch.lasx.xvpackev.d
  196923, // llvm.loongarch.lasx.xvpackev.h
  196954, // llvm.loongarch.lasx.xvpackev.w
  196985, // llvm.loongarch.lasx.xvpackod.b
  197016, // llvm.loongarch.lasx.xvpackod.d
  197047, // llvm.loongarch.lasx.xvpackod.h
  197078, // llvm.loongarch.lasx.xvpackod.w
  197109, // llvm.loongarch.lasx.xvpcnt.b
  197138, // llvm.loongarch.lasx.xvpcnt.d
  197167, // llvm.loongarch.lasx.xvpcnt.h
  197196, // llvm.loongarch.lasx.xvpcnt.w
  197225, // llvm.loongarch.lasx.xvperm.w
  197254, // llvm.loongarch.lasx.xvpermi.d
  197284, // llvm.loongarch.lasx.xvpermi.q
  197314, // llvm.loongarch.lasx.xvpermi.w
  197344, // llvm.loongarch.lasx.xvpickev.b
  197375, // llvm.loongarch.lasx.xvpickev.d
  197406, // llvm.loongarch.lasx.xvpickev.h
  197437, // llvm.loongarch.lasx.xvpickev.w
  197468, // llvm.loongarch.lasx.xvpickod.b
  197499, // llvm.loongarch.lasx.xvpickod.d
  197530, // llvm.loongarch.lasx.xvpickod.h
  197561, // llvm.loongarch.lasx.xvpickod.w
  197592, // llvm.loongarch.lasx.xvpickve.d
  197623, // llvm.loongarch.lasx.xvpickve.d.f
  197656, // llvm.loongarch.lasx.xvpickve.w
  197687, // llvm.loongarch.lasx.xvpickve.w.f
  197720, // llvm.loongarch.lasx.xvpickve2gr.d
  197754, // llvm.loongarch.lasx.xvpickve2gr.du
  197789, // llvm.loongarch.lasx.xvpickve2gr.w
  197823, // llvm.loongarch.lasx.xvpickve2gr.wu
  197858, // llvm.loongarch.lasx.xvrepl128vei.b
  197893, // llvm.loongarch.lasx.xvrepl128vei.d
  197928, // llvm.loongarch.lasx.xvrepl128vei.h
  197963, // llvm.loongarch.lasx.xvrepl128vei.w
  197998, // llvm.loongarch.lasx.xvreplgr2vr.b
  198032, // llvm.loongarch.lasx.xvreplgr2vr.d
  198066, // llvm.loongarch.lasx.xvreplgr2vr.h
  198100, // llvm.loongarch.lasx.xvreplgr2vr.w
  198134, // llvm.loongarch.lasx.xvrepli.b
  198164, // llvm.loongarch.lasx.xvrepli.d
  198194, // llvm.loongarch.lasx.xvrepli.h
  198224, // llvm.loongarch.lasx.xvrepli.w
  198254, // llvm.loongarch.lasx.xvreplve.b
  198285, // llvm.loongarch.lasx.xvreplve.d
  198316, // llvm.loongarch.lasx.xvreplve.h
  198347, // llvm.loongarch.lasx.xvreplve.w
  198378, // llvm.loongarch.lasx.xvreplve0.b
  198410, // llvm.loongarch.lasx.xvreplve0.d
  198442, // llvm.loongarch.lasx.xvreplve0.h
  198474, // llvm.loongarch.lasx.xvreplve0.q
  198506, // llvm.loongarch.lasx.xvreplve0.w
  198538, // llvm.loongarch.lasx.xvrotr.b
  198567, // llvm.loongarch.lasx.xvrotr.d
  198596, // llvm.loongarch.lasx.xvrotr.h
  198625, // llvm.loongarch.lasx.xvrotr.w
  198654, // llvm.loongarch.lasx.xvrotri.b
  198684, // llvm.loongarch.lasx.xvrotri.d
  198714, // llvm.loongarch.lasx.xvrotri.h
  198744, // llvm.loongarch.lasx.xvrotri.w
  198774, // llvm.loongarch.lasx.xvsadd.b
  198803, // llvm.loongarch.lasx.xvsadd.bu
  198833, // llvm.loongarch.lasx.xvsadd.d
  198862, // llvm.loongarch.lasx.xvsadd.du
  198892, // llvm.loongarch.lasx.xvsadd.h
  198921, // llvm.loongarch.lasx.xvsadd.hu
  198951, // llvm.loongarch.lasx.xvsadd.w
  198980, // llvm.loongarch.lasx.xvsadd.wu
  199010, // llvm.loongarch.lasx.xvsat.b
  199038, // llvm.loongarch.lasx.xvsat.bu
  199067, // llvm.loongarch.lasx.xvsat.d
  199095, // llvm.loongarch.lasx.xvsat.du
  199124, // llvm.loongarch.lasx.xvsat.h
  199152, // llvm.loongarch.lasx.xvsat.hu
  199181, // llvm.loongarch.lasx.xvsat.w
  199209, // llvm.loongarch.lasx.xvsat.wu
  199238, // llvm.loongarch.lasx.xvseq.b
  199266, // llvm.loongarch.lasx.xvseq.d
  199294, // llvm.loongarch.lasx.xvseq.h
  199322, // llvm.loongarch.lasx.xvseq.w
  199350, // llvm.loongarch.lasx.xvseqi.b
  199379, // llvm.loongarch.lasx.xvseqi.d
  199408, // llvm.loongarch.lasx.xvseqi.h
  199437, // llvm.loongarch.lasx.xvseqi.w
  199466, // llvm.loongarch.lasx.xvshuf.b
  199495, // llvm.loongarch.lasx.xvshuf.d
  199524, // llvm.loongarch.lasx.xvshuf.h
  199553, // llvm.loongarch.lasx.xvshuf.w
  199582, // llvm.loongarch.lasx.xvshuf4i.b
  199613, // llvm.loongarch.lasx.xvshuf4i.d
  199644, // llvm.loongarch.lasx.xvshuf4i.h
  199675, // llvm.loongarch.lasx.xvshuf4i.w
  199706, // llvm.loongarch.lasx.xvsigncov.b
  199738, // llvm.loongarch.lasx.xvsigncov.d
  199770, // llvm.loongarch.lasx.xvsigncov.h
  199802, // llvm.loongarch.lasx.xvsigncov.w
  199834, // llvm.loongarch.lasx.xvsle.b
  199862, // llvm.loongarch.lasx.xvsle.bu
  199891, // llvm.loongarch.lasx.xvsle.d
  199919, // llvm.loongarch.lasx.xvsle.du
  199948, // llvm.loongarch.lasx.xvsle.h
  199976, // llvm.loongarch.lasx.xvsle.hu
  200005, // llvm.loongarch.lasx.xvsle.w
  200033, // llvm.loongarch.lasx.xvsle.wu
  200062, // llvm.loongarch.lasx.xvslei.b
  200091, // llvm.loongarch.lasx.xvslei.bu
  200121, // llvm.loongarch.lasx.xvslei.d
  200150, // llvm.loongarch.lasx.xvslei.du
  200180, // llvm.loongarch.lasx.xvslei.h
  200209, // llvm.loongarch.lasx.xvslei.hu
  200239, // llvm.loongarch.lasx.xvslei.w
  200268, // llvm.loongarch.lasx.xvslei.wu
  200298, // llvm.loongarch.lasx.xvsll.b
  200326, // llvm.loongarch.lasx.xvsll.d
  200354, // llvm.loongarch.lasx.xvsll.h
  200382, // llvm.loongarch.lasx.xvsll.w
  200410, // llvm.loongarch.lasx.xvslli.b
  200439, // llvm.loongarch.lasx.xvslli.d
  200468, // llvm.loongarch.lasx.xvslli.h
  200497, // llvm.loongarch.lasx.xvslli.w
  200526, // llvm.loongarch.lasx.xvsllwil.d.w
  200559, // llvm.loongarch.lasx.xvsllwil.du.wu
  200594, // llvm.loongarch.lasx.xvsllwil.h.b
  200627, // llvm.loongarch.lasx.xvsllwil.hu.bu
  200662, // llvm.loongarch.lasx.xvsllwil.w.h
  200695, // llvm.loongarch.lasx.xvsllwil.wu.hu
  200730, // llvm.loongarch.lasx.xvslt.b
  200758, // llvm.loongarch.lasx.xvslt.bu
  200787, // llvm.loongarch.lasx.xvslt.d
  200815, // llvm.loongarch.lasx.xvslt.du
  200844, // llvm.loongarch.lasx.xvslt.h
  200872, // llvm.loongarch.lasx.xvslt.hu
  200901, // llvm.loongarch.lasx.xvslt.w
  200929, // llvm.loongarch.lasx.xvslt.wu
  200958, // llvm.loongarch.lasx.xvslti.b
  200987, // llvm.loongarch.lasx.xvslti.bu
  201017, // llvm.loongarch.lasx.xvslti.d
  201046, // llvm.loongarch.lasx.xvslti.du
  201076, // llvm.loongarch.lasx.xvslti.h
  201105, // llvm.loongarch.lasx.xvslti.hu
  201135, // llvm.loongarch.lasx.xvslti.w
  201164, // llvm.loongarch.lasx.xvslti.wu
  201194, // llvm.loongarch.lasx.xvsra.b
  201222, // llvm.loongarch.lasx.xvsra.d
  201250, // llvm.loongarch.lasx.xvsra.h
  201278, // llvm.loongarch.lasx.xvsra.w
  201306, // llvm.loongarch.lasx.xvsrai.b
  201335, // llvm.loongarch.lasx.xvsrai.d
  201364, // llvm.loongarch.lasx.xvsrai.h
  201393, // llvm.loongarch.lasx.xvsrai.w
  201422, // llvm.loongarch.lasx.xvsran.b.h
  201453, // llvm.loongarch.lasx.xvsran.h.w
  201484, // llvm.loongarch.lasx.xvsran.w.d
  201515, // llvm.loongarch.lasx.xvsrani.b.h
  201547, // llvm.loongarch.lasx.xvsrani.d.q
  201579, // llvm.loongarch.lasx.xvsrani.h.w
  201611, // llvm.loongarch.lasx.xvsrani.w.d
  201643, // llvm.loongarch.lasx.xvsrar.b
  201672, // llvm.loongarch.lasx.xvsrar.d
  201701, // llvm.loongarch.lasx.xvsrar.h
  201730, // llvm.loongarch.lasx.xvsrar.w
  201759, // llvm.loongarch.lasx.xvsrari.b
  201789, // llvm.loongarch.lasx.xvsrari.d
  201819, // llvm.loongarch.lasx.xvsrari.h
  201849, // llvm.loongarch.lasx.xvsrari.w
  201879, // llvm.loongarch.lasx.xvsrarn.b.h
  201911, // llvm.loongarch.lasx.xvsrarn.h.w
  201943, // llvm.loongarch.lasx.xvsrarn.w.d
  201975, // llvm.loongarch.lasx.xvsrarni.b.h
  202008, // llvm.loongarch.lasx.xvsrarni.d.q
  202041, // llvm.loongarch.lasx.xvsrarni.h.w
  202074, // llvm.loongarch.lasx.xvsrarni.w.d
  202107, // llvm.loongarch.lasx.xvsrl.b
  202135, // llvm.loongarch.lasx.xvsrl.d
  202163, // llvm.loongarch.lasx.xvsrl.h
  202191, // llvm.loongarch.lasx.xvsrl.w
  202219, // llvm.loongarch.lasx.xvsrli.b
  202248, // llvm.loongarch.lasx.xvsrli.d
  202277, // llvm.loongarch.lasx.xvsrli.h
  202306, // llvm.loongarch.lasx.xvsrli.w
  202335, // llvm.loongarch.lasx.xvsrln.b.h
  202366, // llvm.loongarch.lasx.xvsrln.h.w
  202397, // llvm.loongarch.lasx.xvsrln.w.d
  202428, // llvm.loongarch.lasx.xvsrlni.b.h
  202460, // llvm.loongarch.lasx.xvsrlni.d.q
  202492, // llvm.loongarch.lasx.xvsrlni.h.w
  202524, // llvm.loongarch.lasx.xvsrlni.w.d
  202556, // llvm.loongarch.lasx.xvsrlr.b
  202585, // llvm.loongarch.lasx.xvsrlr.d
  202614, // llvm.loongarch.lasx.xvsrlr.h
  202643, // llvm.loongarch.lasx.xvsrlr.w
  202672, // llvm.loongarch.lasx.xvsrlri.b
  202702, // llvm.loongarch.lasx.xvsrlri.d
  202732, // llvm.loongarch.lasx.xvsrlri.h
  202762, // llvm.loongarch.lasx.xvsrlri.w
  202792, // llvm.loongarch.lasx.xvsrlrn.b.h
  202824, // llvm.loongarch.lasx.xvsrlrn.h.w
  202856, // llvm.loongarch.lasx.xvsrlrn.w.d
  202888, // llvm.loongarch.lasx.xvsrlrni.b.h
  202921, // llvm.loongarch.lasx.xvsrlrni.d.q
  202954, // llvm.loongarch.lasx.xvsrlrni.h.w
  202987, // llvm.loongarch.lasx.xvsrlrni.w.d
  203020, // llvm.loongarch.lasx.xvssran.b.h
  203052, // llvm.loongarch.lasx.xvssran.bu.h
  203085, // llvm.loongarch.lasx.xvssran.h.w
  203117, // llvm.loongarch.lasx.xvssran.hu.w
  203150, // llvm.loongarch.lasx.xvssran.w.d
  203182, // llvm.loongarch.lasx.xvssran.wu.d
  203215, // llvm.loongarch.lasx.xvssrani.b.h
  203248, // llvm.loongarch.lasx.xvssrani.bu.h
  203282, // llvm.loongarch.lasx.xvssrani.d.q
  203315, // llvm.loongarch.lasx.xvssrani.du.q
  203349, // llvm.loongarch.lasx.xvssrani.h.w
  203382, // llvm.loongarch.lasx.xvssrani.hu.w
  203416, // llvm.loongarch.lasx.xvssrani.w.d
  203449, // llvm.loongarch.lasx.xvssrani.wu.d
  203483, // llvm.loongarch.lasx.xvssrarn.b.h
  203516, // llvm.loongarch.lasx.xvssrarn.bu.h
  203550, // llvm.loongarch.lasx.xvssrarn.h.w
  203583, // llvm.loongarch.lasx.xvssrarn.hu.w
  203617, // llvm.loongarch.lasx.xvssrarn.w.d
  203650, // llvm.loongarch.lasx.xvssrarn.wu.d
  203684, // llvm.loongarch.lasx.xvssrarni.b.h
  203718, // llvm.loongarch.lasx.xvssrarni.bu.h
  203753, // llvm.loongarch.lasx.xvssrarni.d.q
  203787, // llvm.loongarch.lasx.xvssrarni.du.q
  203822, // llvm.loongarch.lasx.xvssrarni.h.w
  203856, // llvm.loongarch.lasx.xvssrarni.hu.w
  203891, // llvm.loongarch.lasx.xvssrarni.w.d
  203925, // llvm.loongarch.lasx.xvssrarni.wu.d
  203960, // llvm.loongarch.lasx.xvssrln.b.h
  203992, // llvm.loongarch.lasx.xvssrln.bu.h
  204025, // llvm.loongarch.lasx.xvssrln.h.w
  204057, // llvm.loongarch.lasx.xvssrln.hu.w
  204090, // llvm.loongarch.lasx.xvssrln.w.d
  204122, // llvm.loongarch.lasx.xvssrln.wu.d
  204155, // llvm.loongarch.lasx.xvssrlni.b.h
  204188, // llvm.loongarch.lasx.xvssrlni.bu.h
  204222, // llvm.loongarch.lasx.xvssrlni.d.q
  204255, // llvm.loongarch.lasx.xvssrlni.du.q
  204289, // llvm.loongarch.lasx.xvssrlni.h.w
  204322, // llvm.loongarch.lasx.xvssrlni.hu.w
  204356, // llvm.loongarch.lasx.xvssrlni.w.d
  204389, // llvm.loongarch.lasx.xvssrlni.wu.d
  204423, // llvm.loongarch.lasx.xvssrlrn.b.h
  204456, // llvm.loongarch.lasx.xvssrlrn.bu.h
  204490, // llvm.loongarch.lasx.xvssrlrn.h.w
  204523, // llvm.loongarch.lasx.xvssrlrn.hu.w
  204557, // llvm.loongarch.lasx.xvssrlrn.w.d
  204590, // llvm.loongarch.lasx.xvssrlrn.wu.d
  204624, // llvm.loongarch.lasx.xvssrlrni.b.h
  204658, // llvm.loongarch.lasx.xvssrlrni.bu.h
  204693, // llvm.loongarch.lasx.xvssrlrni.d.q
  204727, // llvm.loongarch.lasx.xvssrlrni.du.q
  204762, // llvm.loongarch.lasx.xvssrlrni.h.w
  204796, // llvm.loongarch.lasx.xvssrlrni.hu.w
  204831, // llvm.loongarch.lasx.xvssrlrni.w.d
  204865, // llvm.loongarch.lasx.xvssrlrni.wu.d
  204900, // llvm.loongarch.lasx.xvssub.b
  204929, // llvm.loongarch.lasx.xvssub.bu
  204959, // llvm.loongarch.lasx.xvssub.d
  204988, // llvm.loongarch.lasx.xvssub.du
  205018, // llvm.loongarch.lasx.xvssub.h
  205047, // llvm.loongarch.lasx.xvssub.hu
  205077, // llvm.loongarch.lasx.xvssub.w
  205106, // llvm.loongarch.lasx.xvssub.wu
  205136, // llvm.loongarch.lasx.xvst
  205161, // llvm.loongarch.lasx.xvstelm.b
  205191, // llvm.loongarch.lasx.xvstelm.d
  205221, // llvm.loongarch.lasx.xvstelm.h
  205251, // llvm.loongarch.lasx.xvstelm.w
  205281, // llvm.loongarch.lasx.xvstx
  205307, // llvm.loongarch.lasx.xvsub.b
  205335, // llvm.loongarch.lasx.xvsub.d
  205363, // llvm.loongarch.lasx.xvsub.h
  205391, // llvm.loongarch.lasx.xvsub.q
  205419, // llvm.loongarch.lasx.xvsub.w
  205447, // llvm.loongarch.lasx.xvsubi.bu
  205477, // llvm.loongarch.lasx.xvsubi.du
  205507, // llvm.loongarch.lasx.xvsubi.hu
  205537, // llvm.loongarch.lasx.xvsubi.wu
  205567, // llvm.loongarch.lasx.xvsubwev.d.w
  205600, // llvm.loongarch.lasx.xvsubwev.d.wu
  205634, // llvm.loongarch.lasx.xvsubwev.h.b
  205667, // llvm.loongarch.lasx.xvsubwev.h.bu
  205701, // llvm.loongarch.lasx.xvsubwev.q.d
  205734, // llvm.loongarch.lasx.xvsubwev.q.du
  205768, // llvm.loongarch.lasx.xvsubwev.w.h
  205801, // llvm.loongarch.lasx.xvsubwev.w.hu
  205835, // llvm.loongarch.lasx.xvsubwod.d.w
  205868, // llvm.loongarch.lasx.xvsubwod.d.wu
  205902, // llvm.loongarch.lasx.xvsubwod.h.b
  205935, // llvm.loongarch.lasx.xvsubwod.h.bu
  205969, // llvm.loongarch.lasx.xvsubwod.q.d
  206002, // llvm.loongarch.lasx.xvsubwod.q.du
  206036, // llvm.loongarch.lasx.xvsubwod.w.h
  206069, // llvm.loongarch.lasx.xvsubwod.w.hu
  206103, // llvm.loongarch.lasx.xvxor.v
  206131, // llvm.loongarch.lasx.xvxori.b
  206160, // llvm.loongarch.lddir.d
  206183, // llvm.loongarch.ldpte.d
  206206, // llvm.loongarch.lsx.bnz.b
  206231, // llvm.loongarch.lsx.bnz.d
  206256, // llvm.loongarch.lsx.bnz.h
  206281, // llvm.loongarch.lsx.bnz.v
  206306, // llvm.loongarch.lsx.bnz.w
  206331, // llvm.loongarch.lsx.bz.b
  206355, // llvm.loongarch.lsx.bz.d
  206379, // llvm.loongarch.lsx.bz.h
  206403, // llvm.loongarch.lsx.bz.v
  206427, // llvm.loongarch.lsx.bz.w
  206451, // llvm.loongarch.lsx.vabsd.b
  206478, // llvm.loongarch.lsx.vabsd.bu
  206506, // llvm.loongarch.lsx.vabsd.d
  206533, // llvm.loongarch.lsx.vabsd.du
  206561, // llvm.loongarch.lsx.vabsd.h
  206588, // llvm.loongarch.lsx.vabsd.hu
  206616, // llvm.loongarch.lsx.vabsd.w
  206643, // llvm.loongarch.lsx.vabsd.wu
  206671, // llvm.loongarch.lsx.vadd.b
  206697, // llvm.loongarch.lsx.vadd.d
  206723, // llvm.loongarch.lsx.vadd.h
  206749, // llvm.loongarch.lsx.vadd.q
  206775, // llvm.loongarch.lsx.vadd.w
  206801, // llvm.loongarch.lsx.vadda.b
  206828, // llvm.loongarch.lsx.vadda.d
  206855, // llvm.loongarch.lsx.vadda.h
  206882, // llvm.loongarch.lsx.vadda.w
  206909, // llvm.loongarch.lsx.vaddi.bu
  206937, // llvm.loongarch.lsx.vaddi.du
  206965, // llvm.loongarch.lsx.vaddi.hu
  206993, // llvm.loongarch.lsx.vaddi.wu
  207021, // llvm.loongarch.lsx.vaddwev.d.w
  207052, // llvm.loongarch.lsx.vaddwev.d.wu
  207084, // llvm.loongarch.lsx.vaddwev.d.wu.w
  207118, // llvm.loongarch.lsx.vaddwev.h.b
  207149, // llvm.loongarch.lsx.vaddwev.h.bu
  207181, // llvm.loongarch.lsx.vaddwev.h.bu.b
  207215, // llvm.loongarch.lsx.vaddwev.q.d
  207246, // llvm.loongarch.lsx.vaddwev.q.du
  207278, // llvm.loongarch.lsx.vaddwev.q.du.d
  207312, // llvm.loongarch.lsx.vaddwev.w.h
  207343, // llvm.loongarch.lsx.vaddwev.w.hu
  207375, // llvm.loongarch.lsx.vaddwev.w.hu.h
  207409, // llvm.loongarch.lsx.vaddwod.d.w
  207440, // llvm.loongarch.lsx.vaddwod.d.wu
  207472, // llvm.loongarch.lsx.vaddwod.d.wu.w
  207506, // llvm.loongarch.lsx.vaddwod.h.b
  207537, // llvm.loongarch.lsx.vaddwod.h.bu
  207569, // llvm.loongarch.lsx.vaddwod.h.bu.b
  207603, // llvm.loongarch.lsx.vaddwod.q.d
  207634, // llvm.loongarch.lsx.vaddwod.q.du
  207666, // llvm.loongarch.lsx.vaddwod.q.du.d
  207700, // llvm.loongarch.lsx.vaddwod.w.h
  207731, // llvm.loongarch.lsx.vaddwod.w.hu
  207763, // llvm.loongarch.lsx.vaddwod.w.hu.h
  207797, // llvm.loongarch.lsx.vand.v
  207823, // llvm.loongarch.lsx.vandi.b
  207850, // llvm.loongarch.lsx.vandn.v
  207877, // llvm.loongarch.lsx.vavg.b
  207903, // llvm.loongarch.lsx.vavg.bu
  207930, // llvm.loongarch.lsx.vavg.d
  207956, // llvm.loongarch.lsx.vavg.du
  207983, // llvm.loongarch.lsx.vavg.h
  208009, // llvm.loongarch.lsx.vavg.hu
  208036, // llvm.loongarch.lsx.vavg.w
  208062, // llvm.loongarch.lsx.vavg.wu
  208089, // llvm.loongarch.lsx.vavgr.b
  208116, // llvm.loongarch.lsx.vavgr.bu
  208144, // llvm.loongarch.lsx.vavgr.d
  208171, // llvm.loongarch.lsx.vavgr.du
  208199, // llvm.loongarch.lsx.vavgr.h
  208226, // llvm.loongarch.lsx.vavgr.hu
  208254, // llvm.loongarch.lsx.vavgr.w
  208281, // llvm.loongarch.lsx.vavgr.wu
  208309, // llvm.loongarch.lsx.vbitclr.b
  208338, // llvm.loongarch.lsx.vbitclr.d
  208367, // llvm.loongarch.lsx.vbitclr.h
  208396, // llvm.loongarch.lsx.vbitclr.w
  208425, // llvm.loongarch.lsx.vbitclri.b
  208455, // llvm.loongarch.lsx.vbitclri.d
  208485, // llvm.loongarch.lsx.vbitclri.h
  208515, // llvm.loongarch.lsx.vbitclri.w
  208545, // llvm.loongarch.lsx.vbitrev.b
  208574, // llvm.loongarch.lsx.vbitrev.d
  208603, // llvm.loongarch.lsx.vbitrev.h
  208632, // llvm.loongarch.lsx.vbitrev.w
  208661, // llvm.loongarch.lsx.vbitrevi.b
  208691, // llvm.loongarch.lsx.vbitrevi.d
  208721, // llvm.loongarch.lsx.vbitrevi.h
  208751, // llvm.loongarch.lsx.vbitrevi.w
  208781, // llvm.loongarch.lsx.vbitsel.v
  208810, // llvm.loongarch.lsx.vbitseli.b
  208840, // llvm.loongarch.lsx.vbitset.b
  208869, // llvm.loongarch.lsx.vbitset.d
  208898, // llvm.loongarch.lsx.vbitset.h
  208927, // llvm.loongarch.lsx.vbitset.w
  208956, // llvm.loongarch.lsx.vbitseti.b
  208986, // llvm.loongarch.lsx.vbitseti.d
  209016, // llvm.loongarch.lsx.vbitseti.h
  209046, // llvm.loongarch.lsx.vbitseti.w
  209076, // llvm.loongarch.lsx.vbsll.v
  209103, // llvm.loongarch.lsx.vbsrl.v
  209130, // llvm.loongarch.lsx.vclo.b
  209156, // llvm.loongarch.lsx.vclo.d
  209182, // llvm.loongarch.lsx.vclo.h
  209208, // llvm.loongarch.lsx.vclo.w
  209234, // llvm.loongarch.lsx.vclz.b
  209260, // llvm.loongarch.lsx.vclz.d
  209286, // llvm.loongarch.lsx.vclz.h
  209312, // llvm.loongarch.lsx.vclz.w
  209338, // llvm.loongarch.lsx.vdiv.b
  209364, // llvm.loongarch.lsx.vdiv.bu
  209391, // llvm.loongarch.lsx.vdiv.d
  209417, // llvm.loongarch.lsx.vdiv.du
  209444, // llvm.loongarch.lsx.vdiv.h
  209470, // llvm.loongarch.lsx.vdiv.hu
  209497, // llvm.loongarch.lsx.vdiv.w
  209523, // llvm.loongarch.lsx.vdiv.wu
  209550, // llvm.loongarch.lsx.vexth.d.w
  209579, // llvm.loongarch.lsx.vexth.du.wu
  209610, // llvm.loongarch.lsx.vexth.h.b
  209639, // llvm.loongarch.lsx.vexth.hu.bu
  209670, // llvm.loongarch.lsx.vexth.q.d
  209699, // llvm.loongarch.lsx.vexth.qu.du
  209730, // llvm.loongarch.lsx.vexth.w.h
  209759, // llvm.loongarch.lsx.vexth.wu.hu
  209790, // llvm.loongarch.lsx.vextl.q.d
  209819, // llvm.loongarch.lsx.vextl.qu.du
  209850, // llvm.loongarch.lsx.vextrins.b
  209880, // llvm.loongarch.lsx.vextrins.d
  209910, // llvm.loongarch.lsx.vextrins.h
  209940, // llvm.loongarch.lsx.vextrins.w
  209970, // llvm.loongarch.lsx.vfadd.d
  209997, // llvm.loongarch.lsx.vfadd.s
  210024, // llvm.loongarch.lsx.vfclass.d
  210053, // llvm.loongarch.lsx.vfclass.s
  210082, // llvm.loongarch.lsx.vfcmp.caf.d
  210113, // llvm.loongarch.lsx.vfcmp.caf.s
  210144, // llvm.loongarch.lsx.vfcmp.ceq.d
  210175, // llvm.loongarch.lsx.vfcmp.ceq.s
  210206, // llvm.loongarch.lsx.vfcmp.cle.d
  210237, // llvm.loongarch.lsx.vfcmp.cle.s
  210268, // llvm.loongarch.lsx.vfcmp.clt.d
  210299, // llvm.loongarch.lsx.vfcmp.clt.s
  210330, // llvm.loongarch.lsx.vfcmp.cne.d
  210361, // llvm.loongarch.lsx.vfcmp.cne.s
  210392, // llvm.loongarch.lsx.vfcmp.cor.d
  210423, // llvm.loongarch.lsx.vfcmp.cor.s
  210454, // llvm.loongarch.lsx.vfcmp.cueq.d
  210486, // llvm.loongarch.lsx.vfcmp.cueq.s
  210518, // llvm.loongarch.lsx.vfcmp.cule.d
  210550, // llvm.loongarch.lsx.vfcmp.cule.s
  210582, // llvm.loongarch.lsx.vfcmp.cult.d
  210614, // llvm.loongarch.lsx.vfcmp.cult.s
  210646, // llvm.loongarch.lsx.vfcmp.cun.d
  210677, // llvm.loongarch.lsx.vfcmp.cun.s
  210708, // llvm.loongarch.lsx.vfcmp.cune.d
  210740, // llvm.loongarch.lsx.vfcmp.cune.s
  210772, // llvm.loongarch.lsx.vfcmp.saf.d
  210803, // llvm.loongarch.lsx.vfcmp.saf.s
  210834, // llvm.loongarch.lsx.vfcmp.seq.d
  210865, // llvm.loongarch.lsx.vfcmp.seq.s
  210896, // llvm.loongarch.lsx.vfcmp.sle.d
  210927, // llvm.loongarch.lsx.vfcmp.sle.s
  210958, // llvm.loongarch.lsx.vfcmp.slt.d
  210989, // llvm.loongarch.lsx.vfcmp.slt.s
  211020, // llvm.loongarch.lsx.vfcmp.sne.d
  211051, // llvm.loongarch.lsx.vfcmp.sne.s
  211082, // llvm.loongarch.lsx.vfcmp.sor.d
  211113, // llvm.loongarch.lsx.vfcmp.sor.s
  211144, // llvm.loongarch.lsx.vfcmp.sueq.d
  211176, // llvm.loongarch.lsx.vfcmp.sueq.s
  211208, // llvm.loongarch.lsx.vfcmp.sule.d
  211240, // llvm.loongarch.lsx.vfcmp.sule.s
  211272, // llvm.loongarch.lsx.vfcmp.sult.d
  211304, // llvm.loongarch.lsx.vfcmp.sult.s
  211336, // llvm.loongarch.lsx.vfcmp.sun.d
  211367, // llvm.loongarch.lsx.vfcmp.sun.s
  211398, // llvm.loongarch.lsx.vfcmp.sune.d
  211430, // llvm.loongarch.lsx.vfcmp.sune.s
  211462, // llvm.loongarch.lsx.vfcvt.h.s
  211491, // llvm.loongarch.lsx.vfcvt.s.d
  211520, // llvm.loongarch.lsx.vfcvth.d.s
  211550, // llvm.loongarch.lsx.vfcvth.s.h
  211580, // llvm.loongarch.lsx.vfcvtl.d.s
  211610, // llvm.loongarch.lsx.vfcvtl.s.h
  211640, // llvm.loongarch.lsx.vfdiv.d
  211667, // llvm.loongarch.lsx.vfdiv.s
  211694, // llvm.loongarch.lsx.vffint.d.l
  211724, // llvm.loongarch.lsx.vffint.d.lu
  211755, // llvm.loongarch.lsx.vffint.s.l
  211785, // llvm.loongarch.lsx.vffint.s.w
  211815, // llvm.loongarch.lsx.vffint.s.wu
  211846, // llvm.loongarch.lsx.vffinth.d.w
  211877, // llvm.loongarch.lsx.vffintl.d.w
  211908, // llvm.loongarch.lsx.vflogb.d
  211936, // llvm.loongarch.lsx.vflogb.s
  211964, // llvm.loongarch.lsx.vfmadd.d
  211992, // llvm.loongarch.lsx.vfmadd.s
  212020, // llvm.loongarch.lsx.vfmax.d
  212047, // llvm.loongarch.lsx.vfmax.s
  212074, // llvm.loongarch.lsx.vfmaxa.d
  212102, // llvm.loongarch.lsx.vfmaxa.s
  212130, // llvm.loongarch.lsx.vfmin.d
  212157, // llvm.loongarch.lsx.vfmin.s
  212184, // llvm.loongarch.lsx.vfmina.d
  212212, // llvm.loongarch.lsx.vfmina.s
  212240, // llvm.loongarch.lsx.vfmsub.d
  212268, // llvm.loongarch.lsx.vfmsub.s
  212296, // llvm.loongarch.lsx.vfmul.d
  212323, // llvm.loongarch.lsx.vfmul.s
  212350, // llvm.loongarch.lsx.vfnmadd.d
  212379, // llvm.loongarch.lsx.vfnmadd.s
  212408, // llvm.loongarch.lsx.vfnmsub.d
  212437, // llvm.loongarch.lsx.vfnmsub.s
  212466, // llvm.loongarch.lsx.vfrecip.d
  212495, // llvm.loongarch.lsx.vfrecip.s
  212524, // llvm.loongarch.lsx.vfrecipe.d
  212554, // llvm.loongarch.lsx.vfrecipe.s
  212584, // llvm.loongarch.lsx.vfrint.d
  212612, // llvm.loongarch.lsx.vfrint.s
  212640, // llvm.loongarch.lsx.vfrintrm.d
  212670, // llvm.loongarch.lsx.vfrintrm.s
  212700, // llvm.loongarch.lsx.vfrintrne.d
  212731, // llvm.loongarch.lsx.vfrintrne.s
  212762, // llvm.loongarch.lsx.vfrintrp.d
  212792, // llvm.loongarch.lsx.vfrintrp.s
  212822, // llvm.loongarch.lsx.vfrintrz.d
  212852, // llvm.loongarch.lsx.vfrintrz.s
  212882, // llvm.loongarch.lsx.vfrsqrt.d
  212911, // llvm.loongarch.lsx.vfrsqrt.s
  212940, // llvm.loongarch.lsx.vfrsqrte.d
  212970, // llvm.loongarch.lsx.vfrsqrte.s
  213000, // llvm.loongarch.lsx.vfrstp.b
  213028, // llvm.loongarch.lsx.vfrstp.h
  213056, // llvm.loongarch.lsx.vfrstpi.b
  213085, // llvm.loongarch.lsx.vfrstpi.h
  213114, // llvm.loongarch.lsx.vfsqrt.d
  213142, // llvm.loongarch.lsx.vfsqrt.s
  213170, // llvm.loongarch.lsx.vfsub.d
  213197, // llvm.loongarch.lsx.vfsub.s
  213224, // llvm.loongarch.lsx.vftint.l.d
  213254, // llvm.loongarch.lsx.vftint.lu.d
  213285, // llvm.loongarch.lsx.vftint.w.d
  213315, // llvm.loongarch.lsx.vftint.w.s
  213345, // llvm.loongarch.lsx.vftint.wu.s
  213376, // llvm.loongarch.lsx.vftinth.l.s
  213407, // llvm.loongarch.lsx.vftintl.l.s
  213438, // llvm.loongarch.lsx.vftintrm.l.d
  213470, // llvm.loongarch.lsx.vftintrm.w.d
  213502, // llvm.loongarch.lsx.vftintrm.w.s
  213534, // llvm.loongarch.lsx.vftintrmh.l.s
  213567, // llvm.loongarch.lsx.vftintrml.l.s
  213600, // llvm.loongarch.lsx.vftintrne.l.d
  213633, // llvm.loongarch.lsx.vftintrne.w.d
  213666, // llvm.loongarch.lsx.vftintrne.w.s
  213699, // llvm.loongarch.lsx.vftintrneh.l.s
  213733, // llvm.loongarch.lsx.vftintrnel.l.s
  213767, // llvm.loongarch.lsx.vftintrp.l.d
  213799, // llvm.loongarch.lsx.vftintrp.w.d
  213831, // llvm.loongarch.lsx.vftintrp.w.s
  213863, // llvm.loongarch.lsx.vftintrph.l.s
  213896, // llvm.loongarch.lsx.vftintrpl.l.s
  213929, // llvm.loongarch.lsx.vftintrz.l.d
  213961, // llvm.loongarch.lsx.vftintrz.lu.d
  213994, // llvm.loongarch.lsx.vftintrz.w.d
  214026, // llvm.loongarch.lsx.vftintrz.w.s
  214058, // llvm.loongarch.lsx.vftintrz.wu.s
  214091, // llvm.loongarch.lsx.vftintrzh.l.s
  214124, // llvm.loongarch.lsx.vftintrzl.l.s
  214157, // llvm.loongarch.lsx.vhaddw.d.w
  214187, // llvm.loongarch.lsx.vhaddw.du.wu
  214219, // llvm.loongarch.lsx.vhaddw.h.b
  214249, // llvm.loongarch.lsx.vhaddw.hu.bu
  214281, // llvm.loongarch.lsx.vhaddw.q.d
  214311, // llvm.loongarch.lsx.vhaddw.qu.du
  214343, // llvm.loongarch.lsx.vhaddw.w.h
  214373, // llvm.loongarch.lsx.vhaddw.wu.hu
  214405, // llvm.loongarch.lsx.vhsubw.d.w
  214435, // llvm.loongarch.lsx.vhsubw.du.wu
  214467, // llvm.loongarch.lsx.vhsubw.h.b
  214497, // llvm.loongarch.lsx.vhsubw.hu.bu
  214529, // llvm.loongarch.lsx.vhsubw.q.d
  214559, // llvm.loongarch.lsx.vhsubw.qu.du
  214591, // llvm.loongarch.lsx.vhsubw.w.h
  214621, // llvm.loongarch.lsx.vhsubw.wu.hu
  214653, // llvm.loongarch.lsx.vilvh.b
  214680, // llvm.loongarch.lsx.vilvh.d
  214707, // llvm.loongarch.lsx.vilvh.h
  214734, // llvm.loongarch.lsx.vilvh.w
  214761, // llvm.loongarch.lsx.vilvl.b
  214788, // llvm.loongarch.lsx.vilvl.d
  214815, // llvm.loongarch.lsx.vilvl.h
  214842, // llvm.loongarch.lsx.vilvl.w
  214869, // llvm.loongarch.lsx.vinsgr2vr.b
  214900, // llvm.loongarch.lsx.vinsgr2vr.d
  214931, // llvm.loongarch.lsx.vinsgr2vr.h
  214962, // llvm.loongarch.lsx.vinsgr2vr.w
  214993, // llvm.loongarch.lsx.vld
  215016, // llvm.loongarch.lsx.vldi
  215040, // llvm.loongarch.lsx.vldrepl.b
  215069, // llvm.loongarch.lsx.vldrepl.d
  215098, // llvm.loongarch.lsx.vldrepl.h
  215127, // llvm.loongarch.lsx.vldrepl.w
  215156, // llvm.loongarch.lsx.vldx
  215180, // llvm.loongarch.lsx.vmadd.b
  215207, // llvm.loongarch.lsx.vmadd.d
  215234, // llvm.loongarch.lsx.vmadd.h
  215261, // llvm.loongarch.lsx.vmadd.w
  215288, // llvm.loongarch.lsx.vmaddwev.d.w
  215320, // llvm.loongarch.lsx.vmaddwev.d.wu
  215353, // llvm.loongarch.lsx.vmaddwev.d.wu.w
  215388, // llvm.loongarch.lsx.vmaddwev.h.b
  215420, // llvm.loongarch.lsx.vmaddwev.h.bu
  215453, // llvm.loongarch.lsx.vmaddwev.h.bu.b
  215488, // llvm.loongarch.lsx.vmaddwev.q.d
  215520, // llvm.loongarch.lsx.vmaddwev.q.du
  215553, // llvm.loongarch.lsx.vmaddwev.q.du.d
  215588, // llvm.loongarch.lsx.vmaddwev.w.h
  215620, // llvm.loongarch.lsx.vmaddwev.w.hu
  215653, // llvm.loongarch.lsx.vmaddwev.w.hu.h
  215688, // llvm.loongarch.lsx.vmaddwod.d.w
  215720, // llvm.loongarch.lsx.vmaddwod.d.wu
  215753, // llvm.loongarch.lsx.vmaddwod.d.wu.w
  215788, // llvm.loongarch.lsx.vmaddwod.h.b
  215820, // llvm.loongarch.lsx.vmaddwod.h.bu
  215853, // llvm.loongarch.lsx.vmaddwod.h.bu.b
  215888, // llvm.loongarch.lsx.vmaddwod.q.d
  215920, // llvm.loongarch.lsx.vmaddwod.q.du
  215953, // llvm.loongarch.lsx.vmaddwod.q.du.d
  215988, // llvm.loongarch.lsx.vmaddwod.w.h
  216020, // llvm.loongarch.lsx.vmaddwod.w.hu
  216053, // llvm.loongarch.lsx.vmaddwod.w.hu.h
  216088, // llvm.loongarch.lsx.vmax.b
  216114, // llvm.loongarch.lsx.vmax.bu
  216141, // llvm.loongarch.lsx.vmax.d
  216167, // llvm.loongarch.lsx.vmax.du
  216194, // llvm.loongarch.lsx.vmax.h
  216220, // llvm.loongarch.lsx.vmax.hu
  216247, // llvm.loongarch.lsx.vmax.w
  216273, // llvm.loongarch.lsx.vmax.wu
  216300, // llvm.loongarch.lsx.vmaxi.b
  216327, // llvm.loongarch.lsx.vmaxi.bu
  216355, // llvm.loongarch.lsx.vmaxi.d
  216382, // llvm.loongarch.lsx.vmaxi.du
  216410, // llvm.loongarch.lsx.vmaxi.h
  216437, // llvm.loongarch.lsx.vmaxi.hu
  216465, // llvm.loongarch.lsx.vmaxi.w
  216492, // llvm.loongarch.lsx.vmaxi.wu
  216520, // llvm.loongarch.lsx.vmin.b
  216546, // llvm.loongarch.lsx.vmin.bu
  216573, // llvm.loongarch.lsx.vmin.d
  216599, // llvm.loongarch.lsx.vmin.du
  216626, // llvm.loongarch.lsx.vmin.h
  216652, // llvm.loongarch.lsx.vmin.hu
  216679, // llvm.loongarch.lsx.vmin.w
  216705, // llvm.loongarch.lsx.vmin.wu
  216732, // llvm.loongarch.lsx.vmini.b
  216759, // llvm.loongarch.lsx.vmini.bu
  216787, // llvm.loongarch.lsx.vmini.d
  216814, // llvm.loongarch.lsx.vmini.du
  216842, // llvm.loongarch.lsx.vmini.h
  216869, // llvm.loongarch.lsx.vmini.hu
  216897, // llvm.loongarch.lsx.vmini.w
  216924, // llvm.loongarch.lsx.vmini.wu
  216952, // llvm.loongarch.lsx.vmod.b
  216978, // llvm.loongarch.lsx.vmod.bu
  217005, // llvm.loongarch.lsx.vmod.d
  217031, // llvm.loongarch.lsx.vmod.du
  217058, // llvm.loongarch.lsx.vmod.h
  217084, // llvm.loongarch.lsx.vmod.hu
  217111, // llvm.loongarch.lsx.vmod.w
  217137, // llvm.loongarch.lsx.vmod.wu
  217164, // llvm.loongarch.lsx.vmskgez.b
  217193, // llvm.loongarch.lsx.vmskltz.b
  217222, // llvm.loongarch.lsx.vmskltz.d
  217251, // llvm.loongarch.lsx.vmskltz.h
  217280, // llvm.loongarch.lsx.vmskltz.w
  217309, // llvm.loongarch.lsx.vmsknz.b
  217337, // llvm.loongarch.lsx.vmsub.b
  217364, // llvm.loongarch.lsx.vmsub.d
  217391, // llvm.loongarch.lsx.vmsub.h
  217418, // llvm.loongarch.lsx.vmsub.w
  217445, // llvm.loongarch.lsx.vmuh.b
  217471, // llvm.loongarch.lsx.vmuh.bu
  217498, // llvm.loongarch.lsx.vmuh.d
  217524, // llvm.loongarch.lsx.vmuh.du
  217551, // llvm.loongarch.lsx.vmuh.h
  217577, // llvm.loongarch.lsx.vmuh.hu
  217604, // llvm.loongarch.lsx.vmuh.w
  217630, // llvm.loongarch.lsx.vmuh.wu
  217657, // llvm.loongarch.lsx.vmul.b
  217683, // llvm.loongarch.lsx.vmul.d
  217709, // llvm.loongarch.lsx.vmul.h
  217735, // llvm.loongarch.lsx.vmul.w
  217761, // llvm.loongarch.lsx.vmulwev.d.w
  217792, // llvm.loongarch.lsx.vmulwev.d.wu
  217824, // llvm.loongarch.lsx.vmulwev.d.wu.w
  217858, // llvm.loongarch.lsx.vmulwev.h.b
  217889, // llvm.loongarch.lsx.vmulwev.h.bu
  217921, // llvm.loongarch.lsx.vmulwev.h.bu.b
  217955, // llvm.loongarch.lsx.vmulwev.q.d
  217986, // llvm.loongarch.lsx.vmulwev.q.du
  218018, // llvm.loongarch.lsx.vmulwev.q.du.d
  218052, // llvm.loongarch.lsx.vmulwev.w.h
  218083, // llvm.loongarch.lsx.vmulwev.w.hu
  218115, // llvm.loongarch.lsx.vmulwev.w.hu.h
  218149, // llvm.loongarch.lsx.vmulwod.d.w
  218180, // llvm.loongarch.lsx.vmulwod.d.wu
  218212, // llvm.loongarch.lsx.vmulwod.d.wu.w
  218246, // llvm.loongarch.lsx.vmulwod.h.b
  218277, // llvm.loongarch.lsx.vmulwod.h.bu
  218309, // llvm.loongarch.lsx.vmulwod.h.bu.b
  218343, // llvm.loongarch.lsx.vmulwod.q.d
  218374, // llvm.loongarch.lsx.vmulwod.q.du
  218406, // llvm.loongarch.lsx.vmulwod.q.du.d
  218440, // llvm.loongarch.lsx.vmulwod.w.h
  218471, // llvm.loongarch.lsx.vmulwod.w.hu
  218503, // llvm.loongarch.lsx.vmulwod.w.hu.h
  218537, // llvm.loongarch.lsx.vneg.b
  218563, // llvm.loongarch.lsx.vneg.d
  218589, // llvm.loongarch.lsx.vneg.h
  218615, // llvm.loongarch.lsx.vneg.w
  218641, // llvm.loongarch.lsx.vnor.v
  218667, // llvm.loongarch.lsx.vnori.b
  218694, // llvm.loongarch.lsx.vor.v
  218719, // llvm.loongarch.lsx.vori.b
  218745, // llvm.loongarch.lsx.vorn.v
  218771, // llvm.loongarch.lsx.vpackev.b
  218800, // llvm.loongarch.lsx.vpackev.d
  218829, // llvm.loongarch.lsx.vpackev.h
  218858, // llvm.loongarch.lsx.vpackev.w
  218887, // llvm.loongarch.lsx.vpackod.b
  218916, // llvm.loongarch.lsx.vpackod.d
  218945, // llvm.loongarch.lsx.vpackod.h
  218974, // llvm.loongarch.lsx.vpackod.w
  219003, // llvm.loongarch.lsx.vpcnt.b
  219030, // llvm.loongarch.lsx.vpcnt.d
  219057, // llvm.loongarch.lsx.vpcnt.h
  219084, // llvm.loongarch.lsx.vpcnt.w
  219111, // llvm.loongarch.lsx.vpermi.w
  219139, // llvm.loongarch.lsx.vpickev.b
  219168, // llvm.loongarch.lsx.vpickev.d
  219197, // llvm.loongarch.lsx.vpickev.h
  219226, // llvm.loongarch.lsx.vpickev.w
  219255, // llvm.loongarch.lsx.vpickod.b
  219284, // llvm.loongarch.lsx.vpickod.d
  219313, // llvm.loongarch.lsx.vpickod.h
  219342, // llvm.loongarch.lsx.vpickod.w
  219371, // llvm.loongarch.lsx.vpickve2gr.b
  219403, // llvm.loongarch.lsx.vpickve2gr.bu
  219436, // llvm.loongarch.lsx.vpickve2gr.d
  219468, // llvm.loongarch.lsx.vpickve2gr.du
  219501, // llvm.loongarch.lsx.vpickve2gr.h
  219533, // llvm.loongarch.lsx.vpickve2gr.hu
  219566, // llvm.loongarch.lsx.vpickve2gr.w
  219598, // llvm.loongarch.lsx.vpickve2gr.wu
  219631, // llvm.loongarch.lsx.vreplgr2vr.b
  219663, // llvm.loongarch.lsx.vreplgr2vr.d
  219695, // llvm.loongarch.lsx.vreplgr2vr.h
  219727, // llvm.loongarch.lsx.vreplgr2vr.w
  219759, // llvm.loongarch.lsx.vrepli.b
  219787, // llvm.loongarch.lsx.vrepli.d
  219815, // llvm.loongarch.lsx.vrepli.h
  219843, // llvm.loongarch.lsx.vrepli.w
  219871, // llvm.loongarch.lsx.vreplve.b
  219900, // llvm.loongarch.lsx.vreplve.d
  219929, // llvm.loongarch.lsx.vreplve.h
  219958, // llvm.loongarch.lsx.vreplve.w
  219987, // llvm.loongarch.lsx.vreplvei.b
  220017, // llvm.loongarch.lsx.vreplvei.d
  220047, // llvm.loongarch.lsx.vreplvei.h
  220077, // llvm.loongarch.lsx.vreplvei.w
  220107, // llvm.loongarch.lsx.vrotr.b
  220134, // llvm.loongarch.lsx.vrotr.d
  220161, // llvm.loongarch.lsx.vrotr.h
  220188, // llvm.loongarch.lsx.vrotr.w
  220215, // llvm.loongarch.lsx.vrotri.b
  220243, // llvm.loongarch.lsx.vrotri.d
  220271, // llvm.loongarch.lsx.vrotri.h
  220299, // llvm.loongarch.lsx.vrotri.w
  220327, // llvm.loongarch.lsx.vsadd.b
  220354, // llvm.loongarch.lsx.vsadd.bu
  220382, // llvm.loongarch.lsx.vsadd.d
  220409, // llvm.loongarch.lsx.vsadd.du
  220437, // llvm.loongarch.lsx.vsadd.h
  220464, // llvm.loongarch.lsx.vsadd.hu
  220492, // llvm.loongarch.lsx.vsadd.w
  220519, // llvm.loongarch.lsx.vsadd.wu
  220547, // llvm.loongarch.lsx.vsat.b
  220573, // llvm.loongarch.lsx.vsat.bu
  220600, // llvm.loongarch.lsx.vsat.d
  220626, // llvm.loongarch.lsx.vsat.du
  220653, // llvm.loongarch.lsx.vsat.h
  220679, // llvm.loongarch.lsx.vsat.hu
  220706, // llvm.loongarch.lsx.vsat.w
  220732, // llvm.loongarch.lsx.vsat.wu
  220759, // llvm.loongarch.lsx.vseq.b
  220785, // llvm.loongarch.lsx.vseq.d
  220811, // llvm.loongarch.lsx.vseq.h
  220837, // llvm.loongarch.lsx.vseq.w
  220863, // llvm.loongarch.lsx.vseqi.b
  220890, // llvm.loongarch.lsx.vseqi.d
  220917, // llvm.loongarch.lsx.vseqi.h
  220944, // llvm.loongarch.lsx.vseqi.w
  220971, // llvm.loongarch.lsx.vshuf.b
  220998, // llvm.loongarch.lsx.vshuf.d
  221025, // llvm.loongarch.lsx.vshuf.h
  221052, // llvm.loongarch.lsx.vshuf.w
  221079, // llvm.loongarch.lsx.vshuf4i.b
  221108, // llvm.loongarch.lsx.vshuf4i.d
  221137, // llvm.loongarch.lsx.vshuf4i.h
  221166, // llvm.loongarch.lsx.vshuf4i.w
  221195, // llvm.loongarch.lsx.vsigncov.b
  221225, // llvm.loongarch.lsx.vsigncov.d
  221255, // llvm.loongarch.lsx.vsigncov.h
  221285, // llvm.loongarch.lsx.vsigncov.w
  221315, // llvm.loongarch.lsx.vsle.b
  221341, // llvm.loongarch.lsx.vsle.bu
  221368, // llvm.loongarch.lsx.vsle.d
  221394, // llvm.loongarch.lsx.vsle.du
  221421, // llvm.loongarch.lsx.vsle.h
  221447, // llvm.loongarch.lsx.vsle.hu
  221474, // llvm.loongarch.lsx.vsle.w
  221500, // llvm.loongarch.lsx.vsle.wu
  221527, // llvm.loongarch.lsx.vslei.b
  221554, // llvm.loongarch.lsx.vslei.bu
  221582, // llvm.loongarch.lsx.vslei.d
  221609, // llvm.loongarch.lsx.vslei.du
  221637, // llvm.loongarch.lsx.vslei.h
  221664, // llvm.loongarch.lsx.vslei.hu
  221692, // llvm.loongarch.lsx.vslei.w
  221719, // llvm.loongarch.lsx.vslei.wu
  221747, // llvm.loongarch.lsx.vsll.b
  221773, // llvm.loongarch.lsx.vsll.d
  221799, // llvm.loongarch.lsx.vsll.h
  221825, // llvm.loongarch.lsx.vsll.w
  221851, // llvm.loongarch.lsx.vslli.b
  221878, // llvm.loongarch.lsx.vslli.d
  221905, // llvm.loongarch.lsx.vslli.h
  221932, // llvm.loongarch.lsx.vslli.w
  221959, // llvm.loongarch.lsx.vsllwil.d.w
  221990, // llvm.loongarch.lsx.vsllwil.du.wu
  222023, // llvm.loongarch.lsx.vsllwil.h.b
  222054, // llvm.loongarch.lsx.vsllwil.hu.bu
  222087, // llvm.loongarch.lsx.vsllwil.w.h
  222118, // llvm.loongarch.lsx.vsllwil.wu.hu
  222151, // llvm.loongarch.lsx.vslt.b
  222177, // llvm.loongarch.lsx.vslt.bu
  222204, // llvm.loongarch.lsx.vslt.d
  222230, // llvm.loongarch.lsx.vslt.du
  222257, // llvm.loongarch.lsx.vslt.h
  222283, // llvm.loongarch.lsx.vslt.hu
  222310, // llvm.loongarch.lsx.vslt.w
  222336, // llvm.loongarch.lsx.vslt.wu
  222363, // llvm.loongarch.lsx.vslti.b
  222390, // llvm.loongarch.lsx.vslti.bu
  222418, // llvm.loongarch.lsx.vslti.d
  222445, // llvm.loongarch.lsx.vslti.du
  222473, // llvm.loongarch.lsx.vslti.h
  222500, // llvm.loongarch.lsx.vslti.hu
  222528, // llvm.loongarch.lsx.vslti.w
  222555, // llvm.loongarch.lsx.vslti.wu
  222583, // llvm.loongarch.lsx.vsra.b
  222609, // llvm.loongarch.lsx.vsra.d
  222635, // llvm.loongarch.lsx.vsra.h
  222661, // llvm.loongarch.lsx.vsra.w
  222687, // llvm.loongarch.lsx.vsrai.b
  222714, // llvm.loongarch.lsx.vsrai.d
  222741, // llvm.loongarch.lsx.vsrai.h
  222768, // llvm.loongarch.lsx.vsrai.w
  222795, // llvm.loongarch.lsx.vsran.b.h
  222824, // llvm.loongarch.lsx.vsran.h.w
  222853, // llvm.loongarch.lsx.vsran.w.d
  222882, // llvm.loongarch.lsx.vsrani.b.h
  222912, // llvm.loongarch.lsx.vsrani.d.q
  222942, // llvm.loongarch.lsx.vsrani.h.w
  222972, // llvm.loongarch.lsx.vsrani.w.d
  223002, // llvm.loongarch.lsx.vsrar.b
  223029, // llvm.loongarch.lsx.vsrar.d
  223056, // llvm.loongarch.lsx.vsrar.h
  223083, // llvm.loongarch.lsx.vsrar.w
  223110, // llvm.loongarch.lsx.vsrari.b
  223138, // llvm.loongarch.lsx.vsrari.d
  223166, // llvm.loongarch.lsx.vsrari.h
  223194, // llvm.loongarch.lsx.vsrari.w
  223222, // llvm.loongarch.lsx.vsrarn.b.h
  223252, // llvm.loongarch.lsx.vsrarn.h.w
  223282, // llvm.loongarch.lsx.vsrarn.w.d
  223312, // llvm.loongarch.lsx.vsrarni.b.h
  223343, // llvm.loongarch.lsx.vsrarni.d.q
  223374, // llvm.loongarch.lsx.vsrarni.h.w
  223405, // llvm.loongarch.lsx.vsrarni.w.d
  223436, // llvm.loongarch.lsx.vsrl.b
  223462, // llvm.loongarch.lsx.vsrl.d
  223488, // llvm.loongarch.lsx.vsrl.h
  223514, // llvm.loongarch.lsx.vsrl.w
  223540, // llvm.loongarch.lsx.vsrli.b
  223567, // llvm.loongarch.lsx.vsrli.d
  223594, // llvm.loongarch.lsx.vsrli.h
  223621, // llvm.loongarch.lsx.vsrli.w
  223648, // llvm.loongarch.lsx.vsrln.b.h
  223677, // llvm.loongarch.lsx.vsrln.h.w
  223706, // llvm.loongarch.lsx.vsrln.w.d
  223735, // llvm.loongarch.lsx.vsrlni.b.h
  223765, // llvm.loongarch.lsx.vsrlni.d.q
  223795, // llvm.loongarch.lsx.vsrlni.h.w
  223825, // llvm.loongarch.lsx.vsrlni.w.d
  223855, // llvm.loongarch.lsx.vsrlr.b
  223882, // llvm.loongarch.lsx.vsrlr.d
  223909, // llvm.loongarch.lsx.vsrlr.h
  223936, // llvm.loongarch.lsx.vsrlr.w
  223963, // llvm.loongarch.lsx.vsrlri.b
  223991, // llvm.loongarch.lsx.vsrlri.d
  224019, // llvm.loongarch.lsx.vsrlri.h
  224047, // llvm.loongarch.lsx.vsrlri.w
  224075, // llvm.loongarch.lsx.vsrlrn.b.h
  224105, // llvm.loongarch.lsx.vsrlrn.h.w
  224135, // llvm.loongarch.lsx.vsrlrn.w.d
  224165, // llvm.loongarch.lsx.vsrlrni.b.h
  224196, // llvm.loongarch.lsx.vsrlrni.d.q
  224227, // llvm.loongarch.lsx.vsrlrni.h.w
  224258, // llvm.loongarch.lsx.vsrlrni.w.d
  224289, // llvm.loongarch.lsx.vssran.b.h
  224319, // llvm.loongarch.lsx.vssran.bu.h
  224350, // llvm.loongarch.lsx.vssran.h.w
  224380, // llvm.loongarch.lsx.vssran.hu.w
  224411, // llvm.loongarch.lsx.vssran.w.d
  224441, // llvm.loongarch.lsx.vssran.wu.d
  224472, // llvm.loongarch.lsx.vssrani.b.h
  224503, // llvm.loongarch.lsx.vssrani.bu.h
  224535, // llvm.loongarch.lsx.vssrani.d.q
  224566, // llvm.loongarch.lsx.vssrani.du.q
  224598, // llvm.loongarch.lsx.vssrani.h.w
  224629, // llvm.loongarch.lsx.vssrani.hu.w
  224661, // llvm.loongarch.lsx.vssrani.w.d
  224692, // llvm.loongarch.lsx.vssrani.wu.d
  224724, // llvm.loongarch.lsx.vssrarn.b.h
  224755, // llvm.loongarch.lsx.vssrarn.bu.h
  224787, // llvm.loongarch.lsx.vssrarn.h.w
  224818, // llvm.loongarch.lsx.vssrarn.hu.w
  224850, // llvm.loongarch.lsx.vssrarn.w.d
  224881, // llvm.loongarch.lsx.vssrarn.wu.d
  224913, // llvm.loongarch.lsx.vssrarni.b.h
  224945, // llvm.loongarch.lsx.vssrarni.bu.h
  224978, // llvm.loongarch.lsx.vssrarni.d.q
  225010, // llvm.loongarch.lsx.vssrarni.du.q
  225043, // llvm.loongarch.lsx.vssrarni.h.w
  225075, // llvm.loongarch.lsx.vssrarni.hu.w
  225108, // llvm.loongarch.lsx.vssrarni.w.d
  225140, // llvm.loongarch.lsx.vssrarni.wu.d
  225173, // llvm.loongarch.lsx.vssrln.b.h
  225203, // llvm.loongarch.lsx.vssrln.bu.h
  225234, // llvm.loongarch.lsx.vssrln.h.w
  225264, // llvm.loongarch.lsx.vssrln.hu.w
  225295, // llvm.loongarch.lsx.vssrln.w.d
  225325, // llvm.loongarch.lsx.vssrln.wu.d
  225356, // llvm.loongarch.lsx.vssrlni.b.h
  225387, // llvm.loongarch.lsx.vssrlni.bu.h
  225419, // llvm.loongarch.lsx.vssrlni.d.q
  225450, // llvm.loongarch.lsx.vssrlni.du.q
  225482, // llvm.loongarch.lsx.vssrlni.h.w
  225513, // llvm.loongarch.lsx.vssrlni.hu.w
  225545, // llvm.loongarch.lsx.vssrlni.w.d
  225576, // llvm.loongarch.lsx.vssrlni.wu.d
  225608, // llvm.loongarch.lsx.vssrlrn.b.h
  225639, // llvm.loongarch.lsx.vssrlrn.bu.h
  225671, // llvm.loongarch.lsx.vssrlrn.h.w
  225702, // llvm.loongarch.lsx.vssrlrn.hu.w
  225734, // llvm.loongarch.lsx.vssrlrn.w.d
  225765, // llvm.loongarch.lsx.vssrlrn.wu.d
  225797, // llvm.loongarch.lsx.vssrlrni.b.h
  225829, // llvm.loongarch.lsx.vssrlrni.bu.h
  225862, // llvm.loongarch.lsx.vssrlrni.d.q
  225894, // llvm.loongarch.lsx.vssrlrni.du.q
  225927, // llvm.loongarch.lsx.vssrlrni.h.w
  225959, // llvm.loongarch.lsx.vssrlrni.hu.w
  225992, // llvm.loongarch.lsx.vssrlrni.w.d
  226024, // llvm.loongarch.lsx.vssrlrni.wu.d
  226057, // llvm.loongarch.lsx.vssub.b
  226084, // llvm.loongarch.lsx.vssub.bu
  226112, // llvm.loongarch.lsx.vssub.d
  226139, // llvm.loongarch.lsx.vssub.du
  226167, // llvm.loongarch.lsx.vssub.h
  226194, // llvm.loongarch.lsx.vssub.hu
  226222, // llvm.loongarch.lsx.vssub.w
  226249, // llvm.loongarch.lsx.vssub.wu
  226277, // llvm.loongarch.lsx.vst
  226300, // llvm.loongarch.lsx.vstelm.b
  226328, // llvm.loongarch.lsx.vstelm.d
  226356, // llvm.loongarch.lsx.vstelm.h
  226384, // llvm.loongarch.lsx.vstelm.w
  226412, // llvm.loongarch.lsx.vstx
  226436, // llvm.loongarch.lsx.vsub.b
  226462, // llvm.loongarch.lsx.vsub.d
  226488, // llvm.loongarch.lsx.vsub.h
  226514, // llvm.loongarch.lsx.vsub.q
  226540, // llvm.loongarch.lsx.vsub.w
  226566, // llvm.loongarch.lsx.vsubi.bu
  226594, // llvm.loongarch.lsx.vsubi.du
  226622, // llvm.loongarch.lsx.vsubi.hu
  226650, // llvm.loongarch.lsx.vsubi.wu
  226678, // llvm.loongarch.lsx.vsubwev.d.w
  226709, // llvm.loongarch.lsx.vsubwev.d.wu
  226741, // llvm.loongarch.lsx.vsubwev.h.b
  226772, // llvm.loongarch.lsx.vsubwev.h.bu
  226804, // llvm.loongarch.lsx.vsubwev.q.d
  226835, // llvm.loongarch.lsx.vsubwev.q.du
  226867, // llvm.loongarch.lsx.vsubwev.w.h
  226898, // llvm.loongarch.lsx.vsubwev.w.hu
  226930, // llvm.loongarch.lsx.vsubwod.d.w
  226961, // llvm.loongarch.lsx.vsubwod.d.wu
  226993, // llvm.loongarch.lsx.vsubwod.h.b
  227024, // llvm.loongarch.lsx.vsubwod.h.bu
  227056, // llvm.loongarch.lsx.vsubwod.q.d
  227087, // llvm.loongarch.lsx.vsubwod.q.du
  227119, // llvm.loongarch.lsx.vsubwod.w.h
  227150, // llvm.loongarch.lsx.vsubwod.w.hu
  227182, // llvm.loongarch.lsx.vxor.v
  227208, // llvm.loongarch.lsx.vxori.b
  227235, // llvm.loongarch.masked.atomicrmw.add.i32
  227275, // llvm.loongarch.masked.atomicrmw.add.i64
  227315, // llvm.loongarch.masked.atomicrmw.max.i32
  227355, // llvm.loongarch.masked.atomicrmw.max.i64
  227395, // llvm.loongarch.masked.atomicrmw.min.i32
  227435, // llvm.loongarch.masked.atomicrmw.min.i64
  227475, // llvm.loongarch.masked.atomicrmw.nand.i32
  227516, // llvm.loongarch.masked.atomicrmw.nand.i64
  227557, // llvm.loongarch.masked.atomicrmw.sub.i32
  227597, // llvm.loongarch.masked.atomicrmw.sub.i64
  227637, // llvm.loongarch.masked.atomicrmw.umax.i32
  227678, // llvm.loongarch.masked.atomicrmw.umax.i64
  227719, // llvm.loongarch.masked.atomicrmw.umin.i32
  227760, // llvm.loongarch.masked.atomicrmw.umin.i64
  227801, // llvm.loongarch.masked.atomicrmw.xchg.i32
  227842, // llvm.loongarch.masked.atomicrmw.xchg.i64
  227883, // llvm.loongarch.masked.cmpxchg.i32
  227917, // llvm.loongarch.masked.cmpxchg.i64
  227951, // llvm.loongarch.movfcsr2gr
  227977, // llvm.loongarch.movgr2fcsr
  228003, // llvm.loongarch.syscall
  228026, // llvm.mips.absq.s.ph
  228046, // llvm.mips.absq.s.qb
  228066, // llvm.mips.absq.s.w
  228085, // llvm.mips.add.a.b
  228103, // llvm.mips.add.a.d
  228121, // llvm.mips.add.a.h
  228139, // llvm.mips.add.a.w
  228157, // llvm.mips.addq.ph
  228175, // llvm.mips.addq.s.ph
  228195, // llvm.mips.addq.s.w
  228214, // llvm.mips.addqh.ph
  228233, // llvm.mips.addqh.r.ph
  228254, // llvm.mips.addqh.r.w
  228274, // llvm.mips.addqh.w
  228292, // llvm.mips.adds.a.b
  228311, // llvm.mips.adds.a.d
  228330, // llvm.mips.adds.a.h
  228349, // llvm.mips.adds.a.w
  228368, // llvm.mips.adds.s.b
  228387, // llvm.mips.adds.s.d
  228406, // llvm.mips.adds.s.h
  228425, // llvm.mips.adds.s.w
  228444, // llvm.mips.adds.u.b
  228463, // llvm.mips.adds.u.d
  228482, // llvm.mips.adds.u.h
  228501, // llvm.mips.adds.u.w
  228520, // llvm.mips.addsc
  228536, // llvm.mips.addu.ph
  228554, // llvm.mips.addu.qb
  228572, // llvm.mips.addu.s.ph
  228592, // llvm.mips.addu.s.qb
  228612, // llvm.mips.adduh.qb
  228631, // llvm.mips.adduh.r.qb
  228652, // llvm.mips.addv.b
  228669, // llvm.mips.addv.d
  228686, // llvm.mips.addv.h
  228703, // llvm.mips.addv.w
  228720, // llvm.mips.addvi.b
  228738, // llvm.mips.addvi.d
  228756, // llvm.mips.addvi.h
  228774, // llvm.mips.addvi.w
  228792, // llvm.mips.addwc
  228808, // llvm.mips.and.v
  228824, // llvm.mips.andi.b
  228841, // llvm.mips.append
  228858, // llvm.mips.asub.s.b
  228877, // llvm.mips.asub.s.d
  228896, // llvm.mips.asub.s.h
  228915, // llvm.mips.asub.s.w
  228934, // llvm.mips.asub.u.b
  228953, // llvm.mips.asub.u.d
  228972, // llvm.mips.asub.u.h
  228991, // llvm.mips.asub.u.w
  229010, // llvm.mips.ave.s.b
  229028, // llvm.mips.ave.s.d
  229046, // llvm.mips.ave.s.h
  229064, // llvm.mips.ave.s.w
  229082, // llvm.mips.ave.u.b
  229100, // llvm.mips.ave.u.d
  229118, // llvm.mips.ave.u.h
  229136, // llvm.mips.ave.u.w
  229154, // llvm.mips.aver.s.b
  229173, // llvm.mips.aver.s.d
  229192, // llvm.mips.aver.s.h
  229211, // llvm.mips.aver.s.w
  229230, // llvm.mips.aver.u.b
  229249, // llvm.mips.aver.u.d
  229268, // llvm.mips.aver.u.h
  229287, // llvm.mips.aver.u.w
  229306, // llvm.mips.balign
  229323, // llvm.mips.bclr.b
  229340, // llvm.mips.bclr.d
  229357, // llvm.mips.bclr.h
  229374, // llvm.mips.bclr.w
  229391, // llvm.mips.bclri.b
  229409, // llvm.mips.bclri.d
  229427, // llvm.mips.bclri.h
  229445, // llvm.mips.bclri.w
  229463, // llvm.mips.binsl.b
  229481, // llvm.mips.binsl.d
  229499, // llvm.mips.binsl.h
  229517, // llvm.mips.binsl.w
  229535, // llvm.mips.binsli.b
  229554, // llvm.mips.binsli.d
  229573, // llvm.mips.binsli.h
  229592, // llvm.mips.binsli.w
  229611, // llvm.mips.binsr.b
  229629, // llvm.mips.binsr.d
  229647, // llvm.mips.binsr.h
  229665, // llvm.mips.binsr.w
  229683, // llvm.mips.binsri.b
  229702, // llvm.mips.binsri.d
  229721, // llvm.mips.binsri.h
  229740, // llvm.mips.binsri.w
  229759, // llvm.mips.bitrev
  229776, // llvm.mips.bmnz.v
  229793, // llvm.mips.bmnzi.b
  229811, // llvm.mips.bmz.v
  229827, // llvm.mips.bmzi.b
  229844, // llvm.mips.bneg.b
  229861, // llvm.mips.bneg.d
  229878, // llvm.mips.bneg.h
  229895, // llvm.mips.bneg.w
  229912, // llvm.mips.bnegi.b
  229930, // llvm.mips.bnegi.d
  229948, // llvm.mips.bnegi.h
  229966, // llvm.mips.bnegi.w
  229984, // llvm.mips.bnz.b
  230000, // llvm.mips.bnz.d
  230016, // llvm.mips.bnz.h
  230032, // llvm.mips.bnz.v
  230048, // llvm.mips.bnz.w
  230064, // llvm.mips.bposge32
  230083, // llvm.mips.bsel.v
  230100, // llvm.mips.bseli.b
  230118, // llvm.mips.bset.b
  230135, // llvm.mips.bset.d
  230152, // llvm.mips.bset.h
  230169, // llvm.mips.bset.w
  230186, // llvm.mips.bseti.b
  230204, // llvm.mips.bseti.d
  230222, // llvm.mips.bseti.h
  230240, // llvm.mips.bseti.w
  230258, // llvm.mips.bz.b
  230273, // llvm.mips.bz.d
  230288, // llvm.mips.bz.h
  230303, // llvm.mips.bz.v
  230318, // llvm.mips.bz.w
  230333, // llvm.mips.ceq.b
  230349, // llvm.mips.ceq.d
  230365, // llvm.mips.ceq.h
  230381, // llvm.mips.ceq.w
  230397, // llvm.mips.ceqi.b
  230414, // llvm.mips.ceqi.d
  230431, // llvm.mips.ceqi.h
  230448, // llvm.mips.ceqi.w
  230465, // llvm.mips.cfcmsa
  230482, // llvm.mips.cle.s.b
  230500, // llvm.mips.cle.s.d
  230518, // llvm.mips.cle.s.h
  230536, // llvm.mips.cle.s.w
  230554, // llvm.mips.cle.u.b
  230572, // llvm.mips.cle.u.d
  230590, // llvm.mips.cle.u.h
  230608, // llvm.mips.cle.u.w
  230626, // llvm.mips.clei.s.b
  230645, // llvm.mips.clei.s.d
  230664, // llvm.mips.clei.s.h
  230683, // llvm.mips.clei.s.w
  230702, // llvm.mips.clei.u.b
  230721, // llvm.mips.clei.u.d
  230740, // llvm.mips.clei.u.h
  230759, // llvm.mips.clei.u.w
  230778, // llvm.mips.clt.s.b
  230796, // llvm.mips.clt.s.d
  230814, // llvm.mips.clt.s.h
  230832, // llvm.mips.clt.s.w
  230850, // llvm.mips.clt.u.b
  230868, // llvm.mips.clt.u.d
  230886, // llvm.mips.clt.u.h
  230904, // llvm.mips.clt.u.w
  230922, // llvm.mips.clti.s.b
  230941, // llvm.mips.clti.s.d
  230960, // llvm.mips.clti.s.h
  230979, // llvm.mips.clti.s.w
  230998, // llvm.mips.clti.u.b
  231017, // llvm.mips.clti.u.d
  231036, // llvm.mips.clti.u.h
  231055, // llvm.mips.clti.u.w
  231074, // llvm.mips.cmp.eq.ph
  231094, // llvm.mips.cmp.le.ph
  231114, // llvm.mips.cmp.lt.ph
  231134, // llvm.mips.cmpgdu.eq.qb
  231157, // llvm.mips.cmpgdu.le.qb
  231180, // llvm.mips.cmpgdu.lt.qb
  231203, // llvm.mips.cmpgu.eq.qb
  231225, // llvm.mips.cmpgu.le.qb
  231247, // llvm.mips.cmpgu.lt.qb
  231269, // llvm.mips.cmpu.eq.qb
  231290, // llvm.mips.cmpu.le.qb
  231311, // llvm.mips.cmpu.lt.qb
  231332, // llvm.mips.copy.s.b
  231351, // llvm.mips.copy.s.d
  231370, // llvm.mips.copy.s.h
  231389, // llvm.mips.copy.s.w
  231408, // llvm.mips.copy.u.b
  231427, // llvm.mips.copy.u.d
  231446, // llvm.mips.copy.u.h
  231465, // llvm.mips.copy.u.w
  231484, // llvm.mips.ctcmsa
  231501, // llvm.mips.div.s.b
  231519, // llvm.mips.div.s.d
  231537, // llvm.mips.div.s.h
  231555, // llvm.mips.div.s.w
  231573, // llvm.mips.div.u.b
  231591, // llvm.mips.div.u.d
  231609, // llvm.mips.div.u.h
  231627, // llvm.mips.div.u.w
  231645, // llvm.mips.dlsa
  231660, // llvm.mips.dotp.s.d
  231679, // llvm.mips.dotp.s.h
  231698, // llvm.mips.dotp.s.w
  231717, // llvm.mips.dotp.u.d
  231736, // llvm.mips.dotp.u.h
  231755, // llvm.mips.dotp.u.w
  231774, // llvm.mips.dpa.w.ph
  231793, // llvm.mips.dpadd.s.d
  231813, // llvm.mips.dpadd.s.h
  231833, // llvm.mips.dpadd.s.w
  231853, // llvm.mips.dpadd.u.d
  231873, // llvm.mips.dpadd.u.h
  231893, // llvm.mips.dpadd.u.w
  231913, // llvm.mips.dpaq.s.w.ph
  231935, // llvm.mips.dpaq.sa.l.w
  231957, // llvm.mips.dpaqx.s.w.ph
  231980, // llvm.mips.dpaqx.sa.w.ph
  232004, // llvm.mips.dpau.h.qbl
  232025, // llvm.mips.dpau.h.qbr
  232046, // llvm.mips.dpax.w.ph
  232066, // llvm.mips.dps.w.ph
  232085, // llvm.mips.dpsq.s.w.ph
  232107, // llvm.mips.dpsq.sa.l.w
  232129, // llvm.mips.dpsqx.s.w.ph
  232152, // llvm.mips.dpsqx.sa.w.ph
  232176, // llvm.mips.dpsu.h.qbl
  232197, // llvm.mips.dpsu.h.qbr
  232218, // llvm.mips.dpsub.s.d
  232238, // llvm.mips.dpsub.s.h
  232258, // llvm.mips.dpsub.s.w
  232278, // llvm.mips.dpsub.u.d
  232298, // llvm.mips.dpsub.u.h
  232318, // llvm.mips.dpsub.u.w
  232338, // llvm.mips.dpsx.w.ph
  232358, // llvm.mips.extp
  232373, // llvm.mips.extpdp
  232390, // llvm.mips.extr.r.w
  232409, // llvm.mips.extr.rs.w
  232429, // llvm.mips.extr.s.h
  232448, // llvm.mips.extr.w
  232465, // llvm.mips.fadd.d
  232482, // llvm.mips.fadd.w
  232499, // llvm.mips.fcaf.d
  232516, // llvm.mips.fcaf.w
  232533, // llvm.mips.fceq.d
  232550, // llvm.mips.fceq.w
  232567, // llvm.mips.fclass.d
  232586, // llvm.mips.fclass.w
  232605, // llvm.mips.fcle.d
  232622, // llvm.mips.fcle.w
  232639, // llvm.mips.fclt.d
  232656, // llvm.mips.fclt.w
  232673, // llvm.mips.fcne.d
  232690, // llvm.mips.fcne.w
  232707, // llvm.mips.fcor.d
  232724, // llvm.mips.fcor.w
  232741, // llvm.mips.fcueq.d
  232759, // llvm.mips.fcueq.w
  232777, // llvm.mips.fcule.d
  232795, // llvm.mips.fcule.w
  232813, // llvm.mips.fcult.d
  232831, // llvm.mips.fcult.w
  232849, // llvm.mips.fcun.d
  232866, // llvm.mips.fcun.w
  232883, // llvm.mips.fcune.d
  232901, // llvm.mips.fcune.w
  232919, // llvm.mips.fdiv.d
  232936, // llvm.mips.fdiv.w
  232953, // llvm.mips.fexdo.h
  232971, // llvm.mips.fexdo.w
  232989, // llvm.mips.fexp2.d
  233007, // llvm.mips.fexp2.w
  233025, // llvm.mips.fexupl.d
  233044, // llvm.mips.fexupl.w
  233063, // llvm.mips.fexupr.d
  233082, // llvm.mips.fexupr.w
  233101, // llvm.mips.ffint.s.d
  233121, // llvm.mips.ffint.s.w
  233141, // llvm.mips.ffint.u.d
  233161, // llvm.mips.ffint.u.w
  233181, // llvm.mips.ffql.d
  233198, // llvm.mips.ffql.w
  233215, // llvm.mips.ffqr.d
  233232, // llvm.mips.ffqr.w
  233249, // llvm.mips.fill.b
  233266, // llvm.mips.fill.d
  233283, // llvm.mips.fill.h
  233300, // llvm.mips.fill.w
  233317, // llvm.mips.flog2.d
  233335, // llvm.mips.flog2.w
  233353, // llvm.mips.fmadd.d
  233371, // llvm.mips.fmadd.w
  233389, // llvm.mips.fmax.a.d
  233408, // llvm.mips.fmax.a.w
  233427, // llvm.mips.fmax.d
  233444, // llvm.mips.fmax.w
  233461, // llvm.mips.fmin.a.d
  233480, // llvm.mips.fmin.a.w
  233499, // llvm.mips.fmin.d
  233516, // llvm.mips.fmin.w
  233533, // llvm.mips.fmsub.d
  233551, // llvm.mips.fmsub.w
  233569, // llvm.mips.fmul.d
  233586, // llvm.mips.fmul.w
  233603, // llvm.mips.frcp.d
  233620, // llvm.mips.frcp.w
  233637, // llvm.mips.frint.d
  233655, // llvm.mips.frint.w
  233673, // llvm.mips.frsqrt.d
  233692, // llvm.mips.frsqrt.w
  233711, // llvm.mips.fsaf.d
  233728, // llvm.mips.fsaf.w
  233745, // llvm.mips.fseq.d
  233762, // llvm.mips.fseq.w
  233779, // llvm.mips.fsle.d
  233796, // llvm.mips.fsle.w
  233813, // llvm.mips.fslt.d
  233830, // llvm.mips.fslt.w
  233847, // llvm.mips.fsne.d
  233864, // llvm.mips.fsne.w
  233881, // llvm.mips.fsor.d
  233898, // llvm.mips.fsor.w
  233915, // llvm.mips.fsqrt.d
  233933, // llvm.mips.fsqrt.w
  233951, // llvm.mips.fsub.d
  233968, // llvm.mips.fsub.w
  233985, // llvm.mips.fsueq.d
  234003, // llvm.mips.fsueq.w
  234021, // llvm.mips.fsule.d
  234039, // llvm.mips.fsule.w
  234057, // llvm.mips.fsult.d
  234075, // llvm.mips.fsult.w
  234093, // llvm.mips.fsun.d
  234110, // llvm.mips.fsun.w
  234127, // llvm.mips.fsune.d
  234145, // llvm.mips.fsune.w
  234163, // llvm.mips.ftint.s.d
  234183, // llvm.mips.ftint.s.w
  234203, // llvm.mips.ftint.u.d
  234223, // llvm.mips.ftint.u.w
  234243, // llvm.mips.ftq.h
  234259, // llvm.mips.ftq.w
  234275, // llvm.mips.ftrunc.s.d
  234296, // llvm.mips.ftrunc.s.w
  234317, // llvm.mips.ftrunc.u.d
  234338, // llvm.mips.ftrunc.u.w
  234359, // llvm.mips.hadd.s.d
  234378, // llvm.mips.hadd.s.h
  234397, // llvm.mips.hadd.s.w
  234416, // llvm.mips.hadd.u.d
  234435, // llvm.mips.hadd.u.h
  234454, // llvm.mips.hadd.u.w
  234473, // llvm.mips.hsub.s.d
  234492, // llvm.mips.hsub.s.h
  234511, // llvm.mips.hsub.s.w
  234530, // llvm.mips.hsub.u.d
  234549, // llvm.mips.hsub.u.h
  234568, // llvm.mips.hsub.u.w
  234587, // llvm.mips.ilvev.b
  234605, // llvm.mips.ilvev.d
  234623, // llvm.mips.ilvev.h
  234641, // llvm.mips.ilvev.w
  234659, // llvm.mips.ilvl.b
  234676, // llvm.mips.ilvl.d
  234693, // llvm.mips.ilvl.h
  234710, // llvm.mips.ilvl.w
  234727, // llvm.mips.ilvod.b
  234745, // llvm.mips.ilvod.d
  234763, // llvm.mips.ilvod.h
  234781, // llvm.mips.ilvod.w
  234799, // llvm.mips.ilvr.b
  234816, // llvm.mips.ilvr.d
  234833, // llvm.mips.ilvr.h
  234850, // llvm.mips.ilvr.w
  234867, // llvm.mips.insert.b
  234886, // llvm.mips.insert.d
  234905, // llvm.mips.insert.h
  234924, // llvm.mips.insert.w
  234943, // llvm.mips.insv
  234958, // llvm.mips.insve.b
  234976, // llvm.mips.insve.d
  234994, // llvm.mips.insve.h
  235012, // llvm.mips.insve.w
  235030, // llvm.mips.lbux
  235045, // llvm.mips.ld.b
  235060, // llvm.mips.ld.d
  235075, // llvm.mips.ld.h
  235090, // llvm.mips.ld.w
  235105, // llvm.mips.ldi.b
  235121, // llvm.mips.ldi.d
  235137, // llvm.mips.ldi.h
  235153, // llvm.mips.ldi.w
  235169, // llvm.mips.ldr.d
  235185, // llvm.mips.ldr.w
  235201, // llvm.mips.lhx
  235215, // llvm.mips.lsa
  235229, // llvm.mips.lwx
  235243, // llvm.mips.madd
  235258, // llvm.mips.madd.q.h
  235277, // llvm.mips.madd.q.w
  235296, // llvm.mips.maddr.q.h
  235316, // llvm.mips.maddr.q.w
  235336, // llvm.mips.maddu
  235352, // llvm.mips.maddv.b
  235370, // llvm.mips.maddv.d
  235388, // llvm.mips.maddv.h
  235406, // llvm.mips.maddv.w
  235424, // llvm.mips.maq.s.w.phl
  235446, // llvm.mips.maq.s.w.phr
  235468, // llvm.mips.maq.sa.w.phl
  235491, // llvm.mips.maq.sa.w.phr
  235514, // llvm.mips.max.a.b
  235532, // llvm.mips.max.a.d
  235550, // llvm.mips.max.a.h
  235568, // llvm.mips.max.a.w
  235586, // llvm.mips.max.s.b
  235604, // llvm.mips.max.s.d
  235622, // llvm.mips.max.s.h
  235640, // llvm.mips.max.s.w
  235658, // llvm.mips.max.u.b
  235676, // llvm.mips.max.u.d
  235694, // llvm.mips.max.u.h
  235712, // llvm.mips.max.u.w
  235730, // llvm.mips.maxi.s.b
  235749, // llvm.mips.maxi.s.d
  235768, // llvm.mips.maxi.s.h
  235787, // llvm.mips.maxi.s.w
  235806, // llvm.mips.maxi.u.b
  235825, // llvm.mips.maxi.u.d
  235844, // llvm.mips.maxi.u.h
  235863, // llvm.mips.maxi.u.w
  235882, // llvm.mips.min.a.b
  235900, // llvm.mips.min.a.d
  235918, // llvm.mips.min.a.h
  235936, // llvm.mips.min.a.w
  235954, // llvm.mips.min.s.b
  235972, // llvm.mips.min.s.d
  235990, // llvm.mips.min.s.h
  236008, // llvm.mips.min.s.w
  236026, // llvm.mips.min.u.b
  236044, // llvm.mips.min.u.d
  236062, // llvm.mips.min.u.h
  236080, // llvm.mips.min.u.w
  236098, // llvm.mips.mini.s.b
  236117, // llvm.mips.mini.s.d
  236136, // llvm.mips.mini.s.h
  236155, // llvm.mips.mini.s.w
  236174, // llvm.mips.mini.u.b
  236193, // llvm.mips.mini.u.d
  236212, // llvm.mips.mini.u.h
  236231, // llvm.mips.mini.u.w
  236250, // llvm.mips.mod.s.b
  236268, // llvm.mips.mod.s.d
  236286, // llvm.mips.mod.s.h
  236304, // llvm.mips.mod.s.w
  236322, // llvm.mips.mod.u.b
  236340, // llvm.mips.mod.u.d
  236358, // llvm.mips.mod.u.h
  236376, // llvm.mips.mod.u.w
  236394, // llvm.mips.modsub
  236411, // llvm.mips.move.v
  236428, // llvm.mips.msub
  236443, // llvm.mips.msub.q.h
  236462, // llvm.mips.msub.q.w
  236481, // llvm.mips.msubr.q.h
  236501, // llvm.mips.msubr.q.w
  236521, // llvm.mips.msubu
  236537, // llvm.mips.msubv.b
  236555, // llvm.mips.msubv.d
  236573, // llvm.mips.msubv.h
  236591, // llvm.mips.msubv.w
  236609, // llvm.mips.mthlip
  236626, // llvm.mips.mul.ph
  236643, // llvm.mips.mul.q.h
  236661, // llvm.mips.mul.q.w
  236679, // llvm.mips.mul.s.ph
  236698, // llvm.mips.muleq.s.w.phl
  236722, // llvm.mips.muleq.s.w.phr
  236746, // llvm.mips.muleu.s.ph.qbl
  236771, // llvm.mips.muleu.s.ph.qbr
  236796, // llvm.mips.mulq.rs.ph
  236817, // llvm.mips.mulq.rs.w
  236837, // llvm.mips.mulq.s.ph
  236857, // llvm.mips.mulq.s.w
  236876, // llvm.mips.mulr.q.h
  236895, // llvm.mips.mulr.q.w
  236914, // llvm.mips.mulsa.w.ph
  236935, // llvm.mips.mulsaq.s.w.ph
  236959, // llvm.mips.mult
  236974, // llvm.mips.multu
  236990, // llvm.mips.mulv.b
  237007, // llvm.mips.mulv.d
  237024, // llvm.mips.mulv.h
  237041, // llvm.mips.mulv.w
  237058, // llvm.mips.nloc.b
  237075, // llvm.mips.nloc.d
  237092, // llvm.mips.nloc.h
  237109, // llvm.mips.nloc.w
  237126, // llvm.mips.nlzc.b
  237143, // llvm.mips.nlzc.d
  237160, // llvm.mips.nlzc.h
  237177, // llvm.mips.nlzc.w
  237194, // llvm.mips.nor.v
  237210, // llvm.mips.nori.b
  237227, // llvm.mips.or.v
  237242, // llvm.mips.ori.b
  237258, // llvm.mips.packrl.ph
  237278, // llvm.mips.pckev.b
  237296, // llvm.mips.pckev.d
  237314, // llvm.mips.pckev.h
  237332, // llvm.mips.pckev.w
  237350, // llvm.mips.pckod.b
  237368, // llvm.mips.pckod.d
  237386, // llvm.mips.pckod.h
  237404, // llvm.mips.pckod.w
  237422, // llvm.mips.pcnt.b
  237439, // llvm.mips.pcnt.d
  237456, // llvm.mips.pcnt.h
  237473, // llvm.mips.pcnt.w
  237490, // llvm.mips.pick.ph
  237508, // llvm.mips.pick.qb
  237526, // llvm.mips.preceq.w.phl
  237549, // llvm.mips.preceq.w.phr
  237572, // llvm.mips.precequ.ph.qbl
  237597, // llvm.mips.precequ.ph.qbla
  237623, // llvm.mips.precequ.ph.qbr
  237648, // llvm.mips.precequ.ph.qbra
  237674, // llvm.mips.preceu.ph.qbl
  237698, // llvm.mips.preceu.ph.qbla
  237723, // llvm.mips.preceu.ph.qbr
  237747, // llvm.mips.preceu.ph.qbra
  237772, // llvm.mips.precr.qb.ph
  237794, // llvm.mips.precr.sra.ph.w
  237819, // llvm.mips.precr.sra.r.ph.w
  237846, // llvm.mips.precrq.ph.w
  237868, // llvm.mips.precrq.qb.ph
  237891, // llvm.mips.precrq.rs.ph.w
  237916, // llvm.mips.precrqu.s.qb.ph
  237942, // llvm.mips.prepend
  237960, // llvm.mips.raddu.w.qb
  237981, // llvm.mips.rddsp
  237997, // llvm.mips.repl.ph
  238015, // llvm.mips.repl.qb
  238033, // llvm.mips.sat.s.b
  238051, // llvm.mips.sat.s.d
  238069, // llvm.mips.sat.s.h
  238087, // llvm.mips.sat.s.w
  238105, // llvm.mips.sat.u.b
  238123, // llvm.mips.sat.u.d
  238141, // llvm.mips.sat.u.h
  238159, // llvm.mips.sat.u.w
  238177, // llvm.mips.shf.b
  238193, // llvm.mips.shf.h
  238209, // llvm.mips.shf.w
  238225, // llvm.mips.shilo
  238241, // llvm.mips.shll.ph
  238259, // llvm.mips.shll.qb
  238277, // llvm.mips.shll.s.ph
  238297, // llvm.mips.shll.s.w
  238316, // llvm.mips.shra.ph
  238334, // llvm.mips.shra.qb
  238352, // llvm.mips.shra.r.ph
  238372, // llvm.mips.shra.r.qb
  238392, // llvm.mips.shra.r.w
  238411, // llvm.mips.shrl.ph
  238429, // llvm.mips.shrl.qb
  238447, // llvm.mips.sld.b
  238463, // llvm.mips.sld.d
  238479, // llvm.mips.sld.h
  238495, // llvm.mips.sld.w
  238511, // llvm.mips.sldi.b
  238528, // llvm.mips.sldi.d
  238545, // llvm.mips.sldi.h
  238562, // llvm.mips.sldi.w
  238579, // llvm.mips.sll.b
  238595, // llvm.mips.sll.d
  238611, // llvm.mips.sll.h
  238627, // llvm.mips.sll.w
  238643, // llvm.mips.slli.b
  238660, // llvm.mips.slli.d
  238677, // llvm.mips.slli.h
  238694, // llvm.mips.slli.w
  238711, // llvm.mips.splat.b
  238729, // llvm.mips.splat.d
  238747, // llvm.mips.splat.h
  238765, // llvm.mips.splat.w
  238783, // llvm.mips.splati.b
  238802, // llvm.mips.splati.d
  238821, // llvm.mips.splati.h
  238840, // llvm.mips.splati.w
  238859, // llvm.mips.sra.b
  238875, // llvm.mips.sra.d
  238891, // llvm.mips.sra.h
  238907, // llvm.mips.sra.w
  238923, // llvm.mips.srai.b
  238940, // llvm.mips.srai.d
  238957, // llvm.mips.srai.h
  238974, // llvm.mips.srai.w
  238991, // llvm.mips.srar.b
  239008, // llvm.mips.srar.d
  239025, // llvm.mips.srar.h
  239042, // llvm.mips.srar.w
  239059, // llvm.mips.srari.b
  239077, // llvm.mips.srari.d
  239095, // llvm.mips.srari.h
  239113, // llvm.mips.srari.w
  239131, // llvm.mips.srl.b
  239147, // llvm.mips.srl.d
  239163, // llvm.mips.srl.h
  239179, // llvm.mips.srl.w
  239195, // llvm.mips.srli.b
  239212, // llvm.mips.srli.d
  239229, // llvm.mips.srli.h
  239246, // llvm.mips.srli.w
  239263, // llvm.mips.srlr.b
  239280, // llvm.mips.srlr.d
  239297, // llvm.mips.srlr.h
  239314, // llvm.mips.srlr.w
  239331, // llvm.mips.srlri.b
  239349, // llvm.mips.srlri.d
  239367, // llvm.mips.srlri.h
  239385, // llvm.mips.srlri.w
  239403, // llvm.mips.st.b
  239418, // llvm.mips.st.d
  239433, // llvm.mips.st.h
  239448, // llvm.mips.st.w
  239463, // llvm.mips.str.d
  239479, // llvm.mips.str.w
  239495, // llvm.mips.subq.ph
  239513, // llvm.mips.subq.s.ph
  239533, // llvm.mips.subq.s.w
  239552, // llvm.mips.subqh.ph
  239571, // llvm.mips.subqh.r.ph
  239592, // llvm.mips.subqh.r.w
  239612, // llvm.mips.subqh.w
  239630, // llvm.mips.subs.s.b
  239649, // llvm.mips.subs.s.d
  239668, // llvm.mips.subs.s.h
  239687, // llvm.mips.subs.s.w
  239706, // llvm.mips.subs.u.b
  239725, // llvm.mips.subs.u.d
  239744, // llvm.mips.subs.u.h
  239763, // llvm.mips.subs.u.w
  239782, // llvm.mips.subsus.u.b
  239803, // llvm.mips.subsus.u.d
  239824, // llvm.mips.subsus.u.h
  239845, // llvm.mips.subsus.u.w
  239866, // llvm.mips.subsuu.s.b
  239887, // llvm.mips.subsuu.s.d
  239908, // llvm.mips.subsuu.s.h
  239929, // llvm.mips.subsuu.s.w
  239950, // llvm.mips.subu.ph
  239968, // llvm.mips.subu.qb
  239986, // llvm.mips.subu.s.ph
  240006, // llvm.mips.subu.s.qb
  240026, // llvm.mips.subuh.qb
  240045, // llvm.mips.subuh.r.qb
  240066, // llvm.mips.subv.b
  240083, // llvm.mips.subv.d
  240100, // llvm.mips.subv.h
  240117, // llvm.mips.subv.w
  240134, // llvm.mips.subvi.b
  240152, // llvm.mips.subvi.d
  240170, // llvm.mips.subvi.h
  240188, // llvm.mips.subvi.w
  240206, // llvm.mips.vshf.b
  240223, // llvm.mips.vshf.d
  240240, // llvm.mips.vshf.h
  240257, // llvm.mips.vshf.w
  240274, // llvm.mips.wrdsp
  240290, // llvm.mips.xor.v
  240306, // llvm.mips.xori.b
  240323, // llvm.nvvm.activemask
  240344, // llvm.nvvm.add.rm.d
  240363, // llvm.nvvm.add.rm.f
  240382, // llvm.nvvm.add.rm.ftz.f
  240405, // llvm.nvvm.add.rm.ftz.sat.f
  240432, // llvm.nvvm.add.rm.sat.f
  240455, // llvm.nvvm.add.rn.d
  240474, // llvm.nvvm.add.rn.f
  240493, // llvm.nvvm.add.rn.ftz.f
  240516, // llvm.nvvm.add.rn.ftz.sat.f
  240543, // llvm.nvvm.add.rn.sat.f
  240566, // llvm.nvvm.add.rp.d
  240585, // llvm.nvvm.add.rp.f
  240604, // llvm.nvvm.add.rp.ftz.f
  240627, // llvm.nvvm.add.rp.ftz.sat.f
  240654, // llvm.nvvm.add.rp.sat.f
  240677, // llvm.nvvm.add.rz.d
  240696, // llvm.nvvm.add.rz.f
  240715, // llvm.nvvm.add.rz.ftz.f
  240738, // llvm.nvvm.add.rz.ftz.sat.f
  240765, // llvm.nvvm.add.rz.sat.f
  240788, // llvm.nvvm.applypriority.L2.evict.normal
  240828, // llvm.nvvm.applypriority.global.L2.evict.normal
  240875, // llvm.nvvm.atomic.add.gen.f.cta
  240906, // llvm.nvvm.atomic.add.gen.f.sys
  240937, // llvm.nvvm.atomic.add.gen.i.cta
  240968, // llvm.nvvm.atomic.add.gen.i.sys
  240999, // llvm.nvvm.atomic.and.gen.i.cta
  241030, // llvm.nvvm.atomic.and.gen.i.sys
  241061, // llvm.nvvm.atomic.cas.gen.i.cta
  241092, // llvm.nvvm.atomic.cas.gen.i.sys
  241123, // llvm.nvvm.atomic.dec.gen.i.cta
  241154, // llvm.nvvm.atomic.dec.gen.i.sys
  241185, // llvm.nvvm.atomic.exch.gen.i.cta
  241217, // llvm.nvvm.atomic.exch.gen.i.sys
  241249, // llvm.nvvm.atomic.inc.gen.i.cta
  241280, // llvm.nvvm.atomic.inc.gen.i.sys
  241311, // llvm.nvvm.atomic.max.gen.i.cta
  241342, // llvm.nvvm.atomic.max.gen.i.sys
  241373, // llvm.nvvm.atomic.min.gen.i.cta
  241404, // llvm.nvvm.atomic.min.gen.i.sys
  241435, // llvm.nvvm.atomic.or.gen.i.cta
  241465, // llvm.nvvm.atomic.or.gen.i.sys
  241495, // llvm.nvvm.atomic.xor.gen.i.cta
  241526, // llvm.nvvm.atomic.xor.gen.i.sys
  241557, // llvm.nvvm.bar.warp.sync
  241581, // llvm.nvvm.barrier.cluster.arrive
  241614, // llvm.nvvm.barrier.cluster.arrive.aligned
  241655, // llvm.nvvm.barrier.cluster.arrive.relaxed
  241696, // llvm.nvvm.barrier.cluster.arrive.relaxed.aligned
  241745, // llvm.nvvm.barrier.cluster.wait
  241776, // llvm.nvvm.barrier.cluster.wait.aligned
  241815, // llvm.nvvm.barrier.cta.arrive.aligned.count
  241858, // llvm.nvvm.barrier.cta.arrive.count
  241893, // llvm.nvvm.barrier.cta.red.and.aligned.all
  241935, // llvm.nvvm.barrier.cta.red.and.aligned.count
  241979, // llvm.nvvm.barrier.cta.red.and.all
  242013, // llvm.nvvm.barrier.cta.red.and.count
  242049, // llvm.nvvm.barrier.cta.red.or.aligned.all
  242090, // llvm.nvvm.barrier.cta.red.or.aligned.count
  242133, // llvm.nvvm.barrier.cta.red.or.all
  242166, // llvm.nvvm.barrier.cta.red.or.count
  242201, // llvm.nvvm.barrier.cta.red.popc.aligned.all
  242244, // llvm.nvvm.barrier.cta.red.popc.aligned.count
  242289, // llvm.nvvm.barrier.cta.red.popc.all
  242324, // llvm.nvvm.barrier.cta.red.popc.count
  242361, // llvm.nvvm.barrier.cta.sync.aligned.all
  242400, // llvm.nvvm.barrier.cta.sync.aligned.count
  242441, // llvm.nvvm.barrier.cta.sync.all
  242472, // llvm.nvvm.barrier.cta.sync.count
  242505, // llvm.nvvm.bf16x2.to.ue8m0x2.rp
  242536, // llvm.nvvm.bf16x2.to.ue8m0x2.rp.satfinite
  242577, // llvm.nvvm.bf16x2.to.ue8m0x2.rz
  242608, // llvm.nvvm.bf16x2.to.ue8m0x2.rz.satfinite
  242649, // llvm.nvvm.bf2h.rn
  242667, // llvm.nvvm.bf2h.rn.ftz
  242689, // llvm.nvvm.bmsk.clamp
  242710, // llvm.nvvm.bmsk.wrap
  242730, // llvm.nvvm.ceil.d
  242747, // llvm.nvvm.ceil.f
  242764, // llvm.nvvm.ceil.ftz.f
  242785, // llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.x
  242847, // llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.y
  242909, // llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.z
  242971, // llvm.nvvm.clusterlaunchcontrol.query_cancel.is_canceled
  243027, // llvm.nvvm.clusterlaunchcontrol.try_cancel.async.multicast.shared
  243092, // llvm.nvvm.clusterlaunchcontrol.try_cancel.async.shared
  243147, // llvm.nvvm.compiler.error
  243172, // llvm.nvvm.compiler.warn
  243196, // llvm.nvvm.cos.approx.f
  243219, // llvm.nvvm.cos.approx.ftz.f
  243246, // llvm.nvvm.cp.async.bulk.commit.group
  243283, // llvm.nvvm.cp.async.bulk.global.to.shared.cluster
  243332, // llvm.nvvm.cp.async.bulk.global.to.shared.cta
  243377, // llvm.nvvm.cp.async.bulk.prefetch.L2
  243413, // llvm.nvvm.cp.async.bulk.shared.cta.to.cluster
  243459, // llvm.nvvm.cp.async.bulk.shared.cta.to.global
  243504, // llvm.nvvm.cp.async.bulk.shared.cta.to.global.bytemask
  243558, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.3d
  243607, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.4d
  243656, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.5d
  243705, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.3d
  243760, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.4d
  243815, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.5d
  243870, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.3d
  243921, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.4d
  243972, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.5d
  244023, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.1d
  244070, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.2d
  244117, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.3d
  244164, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.4d
  244211, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.5d
  244258, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.gather4.2d
  244313, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.3d
  244358, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.4d
  244403, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.5d
  244448, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.3d
  244499, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.4d
  244550, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.5d
  244601, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.3d
  244648, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.4d
  244695, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.5d
  244742, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.1d
  244785, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.2d
  244828, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.3d
  244871, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.4d
  244914, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.5d
  244957, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.gather4.2d
  245008, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.3d
  245058, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.4d
  245108, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.5d
  245158, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.3d
  245214, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.4d
  245270, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.5d
  245326, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.3d
  245378, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.4d
  245430, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.5d
  245482, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.1d
  245530, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.2d
  245578, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.3d
  245626, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.4d
  245674, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.5d
  245722, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.gather4.2d
  245778, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.3d
  245830, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.4d
  245882, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.5d
  245934, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.1d
  245984, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.2d
  246034, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.3d
  246084, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.4d
  246134, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.5d
  246184, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.3d
  246236, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.4d
  246288, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.5d
  246340, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.1d
  246390, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.2d
  246440, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.3d
  246490, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.4d
  246540, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.5d
  246590, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.3d
  246642, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.4d
  246694, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.5d
  246746, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.1d
  246796, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.2d
  246846, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.3d
  246896, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.4d
  246946, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.5d
  246996, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.3d
  247048, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.4d
  247100, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.5d
  247152, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.1d
  247202, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.2d
  247252, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.3d
  247302, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.4d
  247352, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.5d
  247402, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.3d
  247454, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.4d
  247506, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.5d
  247558, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.1d
  247608, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.2d
  247658, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.3d
  247708, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.4d
  247758, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.5d
  247808, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.3d
  247860, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.4d
  247912, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.5d
  247964, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.1d
  248014, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.2d
  248064, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.3d
  248114, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.4d
  248164, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.5d
  248214, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.3d
  248265, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.4d
  248316, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.5d
  248367, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.1d
  248416, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.2d
  248465, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.3d
  248514, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.4d
  248563, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.5d
  248612, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.3d
  248664, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.4d
  248716, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.5d
  248768, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.1d
  248818, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.2d
  248868, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.3d
  248918, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.4d
  248968, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.5d
  249018, // llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.3d
  249063, // llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.4d
  249108, // llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.5d
  249153, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.1d
  249196, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.2d
  249239, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.3d
  249282, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.4d
  249325, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.5d
  249368, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.scatter4.2d
  249420, // llvm.nvvm.cp.async.bulk.wait.group
  249455, // llvm.nvvm.cp.async.bulk.wait.group.read
  249495, // llvm.nvvm.cp.async.ca.shared.global.16
  249534, // llvm.nvvm.cp.async.ca.shared.global.16.s
  249575, // llvm.nvvm.cp.async.ca.shared.global.4
  249613, // llvm.nvvm.cp.async.ca.shared.global.4.s
  249653, // llvm.nvvm.cp.async.ca.shared.global.8
  249691, // llvm.nvvm.cp.async.ca.shared.global.8.s
  249731, // llvm.nvvm.cp.async.cg.shared.global.16
  249770, // llvm.nvvm.cp.async.cg.shared.global.16.s
  249811, // llvm.nvvm.cp.async.commit.group
  249843, // llvm.nvvm.cp.async.mbarrier.arrive
  249878, // llvm.nvvm.cp.async.mbarrier.arrive.noinc
  249919, // llvm.nvvm.cp.async.mbarrier.arrive.noinc.shared
  249967, // llvm.nvvm.cp.async.mbarrier.arrive.shared
  250009, // llvm.nvvm.cp.async.wait.all
  250037, // llvm.nvvm.cp.async.wait.group
  250067, // llvm.nvvm.d2f.rm
  250084, // llvm.nvvm.d2f.rm.ftz
  250105, // llvm.nvvm.d2f.rn
  250122, // llvm.nvvm.d2f.rn.ftz
  250143, // llvm.nvvm.d2f.rp
  250160, // llvm.nvvm.d2f.rp.ftz
  250181, // llvm.nvvm.d2f.rz
  250198, // llvm.nvvm.d2f.rz.ftz
  250219, // llvm.nvvm.d2i.hi
  250236, // llvm.nvvm.d2i.lo
  250253, // llvm.nvvm.d2i.rm
  250270, // llvm.nvvm.d2i.rn
  250287, // llvm.nvvm.d2i.rp
  250304, // llvm.nvvm.d2i.rz
  250321, // llvm.nvvm.d2ll.rm
  250339, // llvm.nvvm.d2ll.rn
  250357, // llvm.nvvm.d2ll.rp
  250375, // llvm.nvvm.d2ll.rz
  250393, // llvm.nvvm.d2ui.rm
  250411, // llvm.nvvm.d2ui.rn
  250429, // llvm.nvvm.d2ui.rp
  250447, // llvm.nvvm.d2ui.rz
  250465, // llvm.nvvm.d2ull.rm
  250484, // llvm.nvvm.d2ull.rn
  250503, // llvm.nvvm.d2ull.rp
  250522, // llvm.nvvm.d2ull.rz
  250541, // llvm.nvvm.discard.L2
  250562, // llvm.nvvm.discard.global.L2
  250590, // llvm.nvvm.div.approx.f
  250613, // llvm.nvvm.div.approx.ftz.f
  250640, // llvm.nvvm.div.full
  250659, // llvm.nvvm.div.full.ftz
  250682, // llvm.nvvm.div.rm.d
  250701, // llvm.nvvm.div.rm.f
  250720, // llvm.nvvm.div.rm.ftz.f
  250743, // llvm.nvvm.div.rn.d
  250762, // llvm.nvvm.div.rn.f
  250781, // llvm.nvvm.div.rn.ftz.f
  250804, // llvm.nvvm.div.rp.d
  250823, // llvm.nvvm.div.rp.f
  250842, // llvm.nvvm.div.rp.ftz.f
  250865, // llvm.nvvm.div.rz.d
  250884, // llvm.nvvm.div.rz.f
  250903, // llvm.nvvm.div.rz.ftz.f
  250926, // llvm.nvvm.e2m1x2.to.f16x2.rn
  250955, // llvm.nvvm.e2m1x2.to.f16x2.rn.relu
  250989, // llvm.nvvm.e2m3x2.to.f16x2.rn
  251018, // llvm.nvvm.e2m3x2.to.f16x2.rn.relu
  251052, // llvm.nvvm.e3m2x2.to.f16x2.rn
  251081, // llvm.nvvm.e3m2x2.to.f16x2.rn.relu
  251115, // llvm.nvvm.e4m3x2.to.f16x2.rn
  251144, // llvm.nvvm.e4m3x2.to.f16x2.rn.relu
  251178, // llvm.nvvm.e5m2x2.to.f16x2.rn
  251207, // llvm.nvvm.e5m2x2.to.f16x2.rn.relu
  251241, // llvm.nvvm.elect.sync
  251262, // llvm.nvvm.ex2.approx
  251283, // llvm.nvvm.ex2.approx.ftz
  251308, // llvm.nvvm.exit
  251323, // llvm.nvvm.f16x2.to.e4m3x2.rn
  251352, // llvm.nvvm.f16x2.to.e4m3x2.rn.relu
  251386, // llvm.nvvm.f16x2.to.e5m2x2.rn
  251415, // llvm.nvvm.f16x2.to.e5m2x2.rn.relu
  251449, // llvm.nvvm.f2bf16.rn
  251469, // llvm.nvvm.f2bf16.rn.relu
  251494, // llvm.nvvm.f2bf16.rn.relu.satfinite
  251529, // llvm.nvvm.f2bf16.rn.satfinite
  251559, // llvm.nvvm.f2bf16.rz
  251579, // llvm.nvvm.f2bf16.rz.relu
  251604, // llvm.nvvm.f2bf16.rz.relu.satfinite
  251639, // llvm.nvvm.f2bf16.rz.satfinite
  251669, // llvm.nvvm.f2f16.rn
  251688, // llvm.nvvm.f2f16.rn.relu
  251712, // llvm.nvvm.f2f16.rn.relu.satfinite
  251746, // llvm.nvvm.f2f16.rn.satfinite
  251775, // llvm.nvvm.f2f16.rz
  251794, // llvm.nvvm.f2f16.rz.relu
  251818, // llvm.nvvm.f2f16.rz.relu.satfinite
  251852, // llvm.nvvm.f2f16.rz.satfinite
  251881, // llvm.nvvm.f2h.rn
  251898, // llvm.nvvm.f2h.rn.ftz
  251919, // llvm.nvvm.f2i.rm
  251936, // llvm.nvvm.f2i.rm.ftz
  251957, // llvm.nvvm.f2i.rn
  251974, // llvm.nvvm.f2i.rn.ftz
  251995, // llvm.nvvm.f2i.rp
  252012, // llvm.nvvm.f2i.rp.ftz
  252033, // llvm.nvvm.f2i.rz
  252050, // llvm.nvvm.f2i.rz.ftz
  252071, // llvm.nvvm.f2ll.rm
  252089, // llvm.nvvm.f2ll.rm.ftz
  252111, // llvm.nvvm.f2ll.rn
  252129, // llvm.nvvm.f2ll.rn.ftz
  252151, // llvm.nvvm.f2ll.rp
  252169, // llvm.nvvm.f2ll.rp.ftz
  252191, // llvm.nvvm.f2ll.rz
  252209, // llvm.nvvm.f2ll.rz.ftz
  252231, // llvm.nvvm.f2tf32.rn
  252251, // llvm.nvvm.f2tf32.rn.relu
  252276, // llvm.nvvm.f2tf32.rn.relu.satfinite
  252311, // llvm.nvvm.f2tf32.rn.satfinite
  252341, // llvm.nvvm.f2tf32.rna
  252362, // llvm.nvvm.f2tf32.rna.satfinite
  252393, // llvm.nvvm.f2tf32.rz
  252413, // llvm.nvvm.f2tf32.rz.relu
  252438, // llvm.nvvm.f2tf32.rz.relu.satfinite
  252473, // llvm.nvvm.f2tf32.rz.satfinite
  252503, // llvm.nvvm.f2ui.rm
  252521, // llvm.nvvm.f2ui.rm.ftz
  252543, // llvm.nvvm.f2ui.rn
  252561, // llvm.nvvm.f2ui.rn.ftz
  252583, // llvm.nvvm.f2ui.rp
  252601, // llvm.nvvm.f2ui.rp.ftz
  252623, // llvm.nvvm.f2ui.rz
  252641, // llvm.nvvm.f2ui.rz.ftz
  252663, // llvm.nvvm.f2ull.rm
  252682, // llvm.nvvm.f2ull.rm.ftz
  252705, // llvm.nvvm.f2ull.rn
  252724, // llvm.nvvm.f2ull.rn.ftz
  252747, // llvm.nvvm.f2ull.rp
  252766, // llvm.nvvm.f2ull.rp.ftz
  252789, // llvm.nvvm.f2ull.rz
  252808, // llvm.nvvm.f2ull.rz.ftz
  252831, // llvm.nvvm.f32x4.to.e2m1x4.rs.relu.satfinite
  252875, // llvm.nvvm.f32x4.to.e2m1x4.rs.satfinite
  252914, // llvm.nvvm.f32x4.to.e2m3x4.rs.relu.satfinite
  252958, // llvm.nvvm.f32x4.to.e2m3x4.rs.satfinite
  252997, // llvm.nvvm.f32x4.to.e3m2x4.rs.relu.satfinite
  253041, // llvm.nvvm.f32x4.to.e3m2x4.rs.satfinite
  253080, // llvm.nvvm.f32x4.to.e4m3x4.rs.relu.satfinite
  253124, // llvm.nvvm.f32x4.to.e4m3x4.rs.satfinite
  253163, // llvm.nvvm.f32x4.to.e5m2x4.rs.relu.satfinite
  253207, // llvm.nvvm.f32x4.to.e5m2x4.rs.satfinite
  253246, // llvm.nvvm.fabs
  253261, // llvm.nvvm.fabs.ftz
  253280, // llvm.nvvm.fence.acquire.sync_restrict.space.cluster.scope.cluster
  253346, // llvm.nvvm.fence.mbarrier_init.release.cluster
  253392, // llvm.nvvm.fence.proxy.alias
  253420, // llvm.nvvm.fence.proxy.async
  253448, // llvm.nvvm.fence.proxy.async.global
  253483, // llvm.nvvm.fence.proxy.async.shared_cluster
  253526, // llvm.nvvm.fence.proxy.async.shared_cta
  253565, // llvm.nvvm.fence.proxy.async_generic.acquire.sync_restrict.space.cluster.scope.cluster
  253651, // llvm.nvvm.fence.proxy.async_generic.release.sync_restrict.space.cta.scope.cluster
  253733, // llvm.nvvm.fence.proxy.tensormap_generic.acquire.cluster
  253789, // llvm.nvvm.fence.proxy.tensormap_generic.acquire.cta
  253841, // llvm.nvvm.fence.proxy.tensormap_generic.acquire.gpu
  253893, // llvm.nvvm.fence.proxy.tensormap_generic.acquire.sys
  253945, // llvm.nvvm.fence.proxy.tensormap_generic.release.cluster
  254001, // llvm.nvvm.fence.proxy.tensormap_generic.release.cta
  254053, // llvm.nvvm.fence.proxy.tensormap_generic.release.gpu
  254105, // llvm.nvvm.fence.proxy.tensormap_generic.release.sys
  254157, // llvm.nvvm.fence.release.sync_restrict.space.cta.scope.cluster
  254219, // llvm.nvvm.fence.sc.cluster
  254246, // llvm.nvvm.ff.to.e2m1x2.rn.relu.satfinite
  254287, // llvm.nvvm.ff.to.e2m1x2.rn.satfinite
  254323, // llvm.nvvm.ff.to.e2m3x2.rn.relu.satfinite
  254364, // llvm.nvvm.ff.to.e2m3x2.rn.satfinite
  254400, // llvm.nvvm.ff.to.e3m2x2.rn.relu.satfinite
  254441, // llvm.nvvm.ff.to.e3m2x2.rn.satfinite
  254477, // llvm.nvvm.ff.to.e4m3x2.rn
  254503, // llvm.nvvm.ff.to.e4m3x2.rn.relu
  254534, // llvm.nvvm.ff.to.e5m2x2.rn
  254560, // llvm.nvvm.ff.to.e5m2x2.rn.relu
  254591, // llvm.nvvm.ff.to.ue8m0x2.rp
  254618, // llvm.nvvm.ff.to.ue8m0x2.rp.satfinite
  254655, // llvm.nvvm.ff.to.ue8m0x2.rz
  254682, // llvm.nvvm.ff.to.ue8m0x2.rz.satfinite
  254719, // llvm.nvvm.ff2bf16x2.rn
  254742, // llvm.nvvm.ff2bf16x2.rn.relu
  254770, // llvm.nvvm.ff2bf16x2.rn.relu.satfinite
  254808, // llvm.nvvm.ff2bf16x2.rn.satfinite
  254841, // llvm.nvvm.ff2bf16x2.rs
  254864, // llvm.nvvm.ff2bf16x2.rs.relu
  254892, // llvm.nvvm.ff2bf16x2.rs.relu.satfinite
  254930, // llvm.nvvm.ff2bf16x2.rs.satfinite
  254963, // llvm.nvvm.ff2bf16x2.rz
  254986, // llvm.nvvm.ff2bf16x2.rz.relu
  255014, // llvm.nvvm.ff2bf16x2.rz.relu.satfinite
  255052, // llvm.nvvm.ff2bf16x2.rz.satfinite
  255085, // llvm.nvvm.ff2f16x2.rn
  255107, // llvm.nvvm.ff2f16x2.rn.relu
  255134, // llvm.nvvm.ff2f16x2.rn.relu.satfinite
  255171, // llvm.nvvm.ff2f16x2.rn.satfinite
  255203, // llvm.nvvm.ff2f16x2.rs
  255225, // llvm.nvvm.ff2f16x2.rs.relu
  255252, // llvm.nvvm.ff2f16x2.rs.relu.satfinite
  255289, // llvm.nvvm.ff2f16x2.rs.satfinite
  255321, // llvm.nvvm.ff2f16x2.rz
  255343, // llvm.nvvm.ff2f16x2.rz.relu
  255370, // llvm.nvvm.ff2f16x2.rz.relu.satfinite
  255407, // llvm.nvvm.ff2f16x2.rz.satfinite
  255439, // llvm.nvvm.flo.s
  255455, // llvm.nvvm.flo.u
  255471, // llvm.nvvm.floor.d
  255489, // llvm.nvvm.floor.f
  255507, // llvm.nvvm.floor.ftz.f
  255529, // llvm.nvvm.fma.rm.d
  255548, // llvm.nvvm.fma.rm.f
  255567, // llvm.nvvm.fma.rm.ftz.f
  255590, // llvm.nvvm.fma.rm.ftz.sat.f
  255617, // llvm.nvvm.fma.rm.sat.f
  255640, // llvm.nvvm.fma.rn.bf16
  255662, // llvm.nvvm.fma.rn.bf16x2
  255686, // llvm.nvvm.fma.rn.d
  255705, // llvm.nvvm.fma.rn.f
  255724, // llvm.nvvm.fma.rn.f16
  255745, // llvm.nvvm.fma.rn.f16x2
  255768, // llvm.nvvm.fma.rn.ftz.bf16
  255794, // llvm.nvvm.fma.rn.ftz.bf16x2
  255822, // llvm.nvvm.fma.rn.ftz.f
  255845, // llvm.nvvm.fma.rn.ftz.f16
  255870, // llvm.nvvm.fma.rn.ftz.f16x2
  255897, // llvm.nvvm.fma.rn.ftz.relu.bf16
  255928, // llvm.nvvm.fma.rn.ftz.relu.bf16x2
  255961, // llvm.nvvm.fma.rn.ftz.relu.f16
  255991, // llvm.nvvm.fma.rn.ftz.relu.f16x2
  256023, // llvm.nvvm.fma.rn.ftz.sat.bf16
  256053, // llvm.nvvm.fma.rn.ftz.sat.bf16x2
  256085, // llvm.nvvm.fma.rn.ftz.sat.f
  256112, // llvm.nvvm.fma.rn.ftz.sat.f16
  256141, // llvm.nvvm.fma.rn.ftz.sat.f16x2
  256172, // llvm.nvvm.fma.rn.relu.bf16
  256199, // llvm.nvvm.fma.rn.relu.bf16x2
  256228, // llvm.nvvm.fma.rn.relu.f16
  256254, // llvm.nvvm.fma.rn.relu.f16x2
  256282, // llvm.nvvm.fma.rn.sat.bf16
  256308, // llvm.nvvm.fma.rn.sat.bf16x2
  256336, // llvm.nvvm.fma.rn.sat.f
  256359, // llvm.nvvm.fma.rn.sat.f16
  256384, // llvm.nvvm.fma.rn.sat.f16x2
  256411, // llvm.nvvm.fma.rp.d
  256430, // llvm.nvvm.fma.rp.f
  256449, // llvm.nvvm.fma.rp.ftz.f
  256472, // llvm.nvvm.fma.rp.ftz.sat.f
  256499, // llvm.nvvm.fma.rp.sat.f
  256522, // llvm.nvvm.fma.rz.d
  256541, // llvm.nvvm.fma.rz.f
  256560, // llvm.nvvm.fma.rz.ftz.f
  256583, // llvm.nvvm.fma.rz.ftz.sat.f
  256610, // llvm.nvvm.fma.rz.sat.f
  256633, // llvm.nvvm.fmax.bf16
  256653, // llvm.nvvm.fmax.bf16x2
  256675, // llvm.nvvm.fmax.d
  256692, // llvm.nvvm.fmax.f
  256709, // llvm.nvvm.fmax.f16
  256728, // llvm.nvvm.fmax.f16x2
  256749, // llvm.nvvm.fmax.ftz.bf16
  256773, // llvm.nvvm.fmax.ftz.bf16x2
  256799, // llvm.nvvm.fmax.ftz.f
  256820, // llvm.nvvm.fmax.ftz.f16
  256843, // llvm.nvvm.fmax.ftz.f16x2
  256868, // llvm.nvvm.fmax.ftz.nan.bf16
  256896, // llvm.nvvm.fmax.ftz.nan.bf16x2
  256926, // llvm.nvvm.fmax.ftz.nan.f
  256951, // llvm.nvvm.fmax.ftz.nan.f16
  256978, // llvm.nvvm.fmax.ftz.nan.f16x2
  257007, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.bf16
  257047, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.bf16x2
  257089, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.f
  257126, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.f16
  257165, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.f16x2
  257206, // llvm.nvvm.fmax.ftz.xorsign.abs.bf16
  257242, // llvm.nvvm.fmax.ftz.xorsign.abs.bf16x2
  257280, // llvm.nvvm.fmax.ftz.xorsign.abs.f
  257313, // llvm.nvvm.fmax.ftz.xorsign.abs.f16
  257348, // llvm.nvvm.fmax.ftz.xorsign.abs.f16x2
  257385, // llvm.nvvm.fmax.nan.bf16
  257409, // llvm.nvvm.fmax.nan.bf16x2
  257435, // llvm.nvvm.fmax.nan.f
  257456, // llvm.nvvm.fmax.nan.f16
  257479, // llvm.nvvm.fmax.nan.f16x2
  257504, // llvm.nvvm.fmax.nan.xorsign.abs.bf16
  257540, // llvm.nvvm.fmax.nan.xorsign.abs.bf16x2
  257578, // llvm.nvvm.fmax.nan.xorsign.abs.f
  257611, // llvm.nvvm.fmax.nan.xorsign.abs.f16
  257646, // llvm.nvvm.fmax.nan.xorsign.abs.f16x2
  257683, // llvm.nvvm.fmax.xorsign.abs.bf16
  257715, // llvm.nvvm.fmax.xorsign.abs.bf16x2
  257749, // llvm.nvvm.fmax.xorsign.abs.f
  257778, // llvm.nvvm.fmax.xorsign.abs.f16
  257809, // llvm.nvvm.fmax.xorsign.abs.f16x2
  257842, // llvm.nvvm.fmin.bf16
  257862, // llvm.nvvm.fmin.bf16x2
  257884, // llvm.nvvm.fmin.d
  257901, // llvm.nvvm.fmin.f
  257918, // llvm.nvvm.fmin.f16
  257937, // llvm.nvvm.fmin.f16x2
  257958, // llvm.nvvm.fmin.ftz.bf16
  257982, // llvm.nvvm.fmin.ftz.bf16x2
  258008, // llvm.nvvm.fmin.ftz.f
  258029, // llvm.nvvm.fmin.ftz.f16
  258052, // llvm.nvvm.fmin.ftz.f16x2
  258077, // llvm.nvvm.fmin.ftz.nan.bf16
  258105, // llvm.nvvm.fmin.ftz.nan.bf16x2
  258135, // llvm.nvvm.fmin.ftz.nan.f
  258160, // llvm.nvvm.fmin.ftz.nan.f16
  258187, // llvm.nvvm.fmin.ftz.nan.f16x2
  258216, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.bf16
  258256, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.bf16x2
  258298, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.f
  258335, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.f16
  258374, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.f16x2
  258415, // llvm.nvvm.fmin.ftz.xorsign.abs.bf16
  258451, // llvm.nvvm.fmin.ftz.xorsign.abs.bf16x2
  258489, // llvm.nvvm.fmin.ftz.xorsign.abs.f
  258522, // llvm.nvvm.fmin.ftz.xorsign.abs.f16
  258557, // llvm.nvvm.fmin.ftz.xorsign.abs.f16x2
  258594, // llvm.nvvm.fmin.nan.bf16
  258618, // llvm.nvvm.fmin.nan.bf16x2
  258644, // llvm.nvvm.fmin.nan.f
  258665, // llvm.nvvm.fmin.nan.f16
  258688, // llvm.nvvm.fmin.nan.f16x2
  258713, // llvm.nvvm.fmin.nan.xorsign.abs.bf16
  258749, // llvm.nvvm.fmin.nan.xorsign.abs.bf16x2
  258787, // llvm.nvvm.fmin.nan.xorsign.abs.f
  258820, // llvm.nvvm.fmin.nan.xorsign.abs.f16
  258855, // llvm.nvvm.fmin.nan.xorsign.abs.f16x2
  258892, // llvm.nvvm.fmin.xorsign.abs.bf16
  258924, // llvm.nvvm.fmin.xorsign.abs.bf16x2
  258958, // llvm.nvvm.fmin.xorsign.abs.f
  258987, // llvm.nvvm.fmin.xorsign.abs.f16
  259018, // llvm.nvvm.fmin.xorsign.abs.f16x2
  259051, // llvm.nvvm.fns
  259065, // llvm.nvvm.fshl.clamp
  259086, // llvm.nvvm.fshr.clamp
  259107, // llvm.nvvm.getctarank
  259128, // llvm.nvvm.getctarank.shared.cluster
  259164, // llvm.nvvm.griddepcontrol.launch.dependents
  259207, // llvm.nvvm.griddepcontrol.wait
  259237, // llvm.nvvm.i2d.rm
  259254, // llvm.nvvm.i2d.rn
  259271, // llvm.nvvm.i2d.rp
  259288, // llvm.nvvm.i2d.rz
  259305, // llvm.nvvm.i2f.rm
  259322, // llvm.nvvm.i2f.rn
  259339, // llvm.nvvm.i2f.rp
  259356, // llvm.nvvm.i2f.rz
  259373, // llvm.nvvm.idp2a.s.s
  259393, // llvm.nvvm.idp2a.s.u
  259413, // llvm.nvvm.idp2a.u.s
  259433, // llvm.nvvm.idp2a.u.u
  259453, // llvm.nvvm.idp4a.s.s
  259473, // llvm.nvvm.idp4a.s.u
  259493, // llvm.nvvm.idp4a.u.s
  259513, // llvm.nvvm.idp4a.u.u
  259533, // llvm.nvvm.internal.addrspace.wrap
  259567, // llvm.nvvm.is_explicit_cluster
  259597, // llvm.nvvm.isspacep.const
  259622, // llvm.nvvm.isspacep.global
  259648, // llvm.nvvm.isspacep.local
  259673, // llvm.nvvm.isspacep.shared
  259699, // llvm.nvvm.isspacep.shared.cluster
  259733, // llvm.nvvm.istypep.sampler
  259759, // llvm.nvvm.istypep.surface
  259785, // llvm.nvvm.istypep.texture
  259811, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8
  259862, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8x16.b4x16_p64
  259926, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8x16.b6x16_p32
  259990, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8
  260041, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8x16.b4x16_p64
  260105, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8x16.b6x16_p32
  260169, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x1.b8x16.b4x16_p64
  260226, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x1.b8x16.b6x16_p32
  260283, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x2.b8x16.b4x16_p64
  260340, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x2.b8x16.b6x16_p32
  260397, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x4.b8x16.b4x16_p64
  260454, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x4.b8x16.b6x16_p32
  260511, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x1.b16
  260555, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x1.trans.b16
  260605, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x2.b16
  260649, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x2.trans.b16
  260699, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x4.b16
  260743, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x4.trans.b16
  260793, // llvm.nvvm.ldu.global.f
  260816, // llvm.nvvm.ldu.global.i
  260839, // llvm.nvvm.ldu.global.p
  260862, // llvm.nvvm.lg2.approx.d
  260885, // llvm.nvvm.lg2.approx.f
  260908, // llvm.nvvm.lg2.approx.ftz.f
  260935, // llvm.nvvm.ll2d.rm
  260953, // llvm.nvvm.ll2d.rn
  260971, // llvm.nvvm.ll2d.rp
  260989, // llvm.nvvm.ll2d.rz
  261007, // llvm.nvvm.ll2f.rm
  261025, // llvm.nvvm.ll2f.rn
  261043, // llvm.nvvm.ll2f.rp
  261061, // llvm.nvvm.ll2f.rz
  261079, // llvm.nvvm.lohi.i2d
  261098, // llvm.nvvm.mapa
  261113, // llvm.nvvm.mapa.shared.cluster
  261143, // llvm.nvvm.match.all.sync.i32p
  261173, // llvm.nvvm.match.all.sync.i64p
  261203, // llvm.nvvm.match.any.sync.i32
  261232, // llvm.nvvm.match.any.sync.i64
  261261, // llvm.nvvm.mbarrier.arrive
  261287, // llvm.nvvm.mbarrier.arrive.drop
  261318, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cluster.space.cluster
  261395, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cluster.space.cta
  261468, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cta.space.cluster
  261541, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cta.space.cta
  261610, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cluster.space.cluster
  261679, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cluster.space.cta
  261744, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cta.space.cluster
  261809, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cta.space.cta
  261870, // llvm.nvvm.mbarrier.arrive.drop.noComplete
  261912, // llvm.nvvm.mbarrier.arrive.drop.noComplete.shared
  261961, // llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cluster.space.cluster
  262028, // llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cluster.space.cta
  262091, // llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cta.space.cluster
  262154, // llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cta.space.cta
  262213, // llvm.nvvm.mbarrier.arrive.drop.scope.cluster.space.cluster
  262272, // llvm.nvvm.mbarrier.arrive.drop.scope.cluster.space.cta
  262327, // llvm.nvvm.mbarrier.arrive.drop.scope.cta.space.cluster
  262382, // llvm.nvvm.mbarrier.arrive.drop.scope.cta.space.cta
  262433, // llvm.nvvm.mbarrier.arrive.drop.shared
  262471, // llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cluster.space.cluster
  262543, // llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cluster.space.cta
  262611, // llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cta.space.cluster
  262679, // llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cta.space.cta
  262743, // llvm.nvvm.mbarrier.arrive.expect.tx.scope.cluster.space.cluster
  262807, // llvm.nvvm.mbarrier.arrive.expect.tx.scope.cluster.space.cta
  262867, // llvm.nvvm.mbarrier.arrive.expect.tx.scope.cta.space.cluster
  262927, // llvm.nvvm.mbarrier.arrive.expect.tx.scope.cta.space.cta
  262983, // llvm.nvvm.mbarrier.arrive.noComplete
  263020, // llvm.nvvm.mbarrier.arrive.noComplete.shared
  263064, // llvm.nvvm.mbarrier.arrive.relaxed.scope.cluster.space.cluster
  263126, // llvm.nvvm.mbarrier.arrive.relaxed.scope.cluster.space.cta
  263184, // llvm.nvvm.mbarrier.arrive.relaxed.scope.cta.space.cluster
  263242, // llvm.nvvm.mbarrier.arrive.relaxed.scope.cta.space.cta
  263296, // llvm.nvvm.mbarrier.arrive.scope.cluster.space.cluster
  263350, // llvm.nvvm.mbarrier.arrive.scope.cluster.space.cta
  263400, // llvm.nvvm.mbarrier.arrive.scope.cta.space.cluster
  263450, // llvm.nvvm.mbarrier.arrive.scope.cta.space.cta
  263496, // llvm.nvvm.mbarrier.arrive.shared
  263529, // llvm.nvvm.mbarrier.complete.tx.scope.cluster.space.cluster
  263588, // llvm.nvvm.mbarrier.complete.tx.scope.cluster.space.cta
  263643, // llvm.nvvm.mbarrier.complete.tx.scope.cta.space.cluster
  263698, // llvm.nvvm.mbarrier.complete.tx.scope.cta.space.cta
  263749, // llvm.nvvm.mbarrier.expect.tx.scope.cluster.space.cluster
  263806, // llvm.nvvm.mbarrier.expect.tx.scope.cluster.space.cta
  263859, // llvm.nvvm.mbarrier.expect.tx.scope.cta.space.cluster
  263912, // llvm.nvvm.mbarrier.expect.tx.scope.cta.space.cta
  263961, // llvm.nvvm.mbarrier.init
  263985, // llvm.nvvm.mbarrier.init.shared
  264016, // llvm.nvvm.mbarrier.inval
  264041, // llvm.nvvm.mbarrier.inval.shared
  264073, // llvm.nvvm.mbarrier.pending.count
  264106, // llvm.nvvm.mbarrier.test.wait
  264135, // llvm.nvvm.mbarrier.test.wait.parity.relaxed.scope.cluster.space.cta
  264203, // llvm.nvvm.mbarrier.test.wait.parity.relaxed.scope.cta.space.cta
  264267, // llvm.nvvm.mbarrier.test.wait.parity.scope.cluster.space.cta
  264327, // llvm.nvvm.mbarrier.test.wait.parity.scope.cta.space.cta
  264383, // llvm.nvvm.mbarrier.test.wait.relaxed.scope.cluster.space.cta
  264444, // llvm.nvvm.mbarrier.test.wait.relaxed.scope.cta.space.cta
  264501, // llvm.nvvm.mbarrier.test.wait.scope.cluster.space.cta
  264554, // llvm.nvvm.mbarrier.test.wait.scope.cta.space.cta
  264603, // llvm.nvvm.mbarrier.test.wait.shared
  264639, // llvm.nvvm.mbarrier.try.wait.parity.relaxed.scope.cluster.space.cta
  264706, // llvm.nvvm.mbarrier.try.wait.parity.relaxed.scope.cta.space.cta
  264769, // llvm.nvvm.mbarrier.try.wait.parity.scope.cluster.space.cta
  264828, // llvm.nvvm.mbarrier.try.wait.parity.scope.cta.space.cta
  264883, // llvm.nvvm.mbarrier.try.wait.parity.tl.relaxed.scope.cluster.space.cta
  264953, // llvm.nvvm.mbarrier.try.wait.parity.tl.relaxed.scope.cta.space.cta
  265019, // llvm.nvvm.mbarrier.try.wait.parity.tl.scope.cluster.space.cta
  265081, // llvm.nvvm.mbarrier.try.wait.parity.tl.scope.cta.space.cta
  265139, // llvm.nvvm.mbarrier.try.wait.relaxed.scope.cluster.space.cta
  265199, // llvm.nvvm.mbarrier.try.wait.relaxed.scope.cta.space.cta
  265255, // llvm.nvvm.mbarrier.try.wait.scope.cluster.space.cta
  265307, // llvm.nvvm.mbarrier.try.wait.scope.cta.space.cta
  265355, // llvm.nvvm.mbarrier.try.wait.tl.relaxed.scope.cluster.space.cta
  265418, // llvm.nvvm.mbarrier.try.wait.tl.relaxed.scope.cta.space.cta
  265477, // llvm.nvvm.mbarrier.try.wait.tl.scope.cluster.space.cta
  265532, // llvm.nvvm.mbarrier.try.wait.tl.scope.cta.space.cta
  265583, // llvm.nvvm.membar.cta
  265604, // llvm.nvvm.membar.gl
  265624, // llvm.nvvm.membar.sys
  265645, // llvm.nvvm.mma.and.popc.m16n8k128.row.col.b1
  265689, // llvm.nvvm.mma.and.popc.m16n8k256.row.col.b1
  265733, // llvm.nvvm.mma.and.popc.m8n8k128.row.col.b1
  265776, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e2m1.f32.ue8m0
  265852, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e2m3.f32.ue8m0
  265928, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e3m2.f32.ue8m0
  266004, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e4m3.f32.ue8m0
  266080, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e5m2.f32.ue8m0
  266156, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e2m1.f32.ue8m0
  266232, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e2m3.f32.ue8m0
  266308, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e3m2.f32.ue8m0
  266384, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e4m3.f32.ue8m0
  266460, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e5m2.f32.ue8m0
  266536, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e2m1.f32.ue8m0
  266612, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e2m3.f32.ue8m0
  266688, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e3m2.f32.ue8m0
  266764, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e4m3.f32.ue8m0
  266840, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e5m2.f32.ue8m0
  266916, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e2m1.f32.ue8m0
  266992, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e2m3.f32.ue8m0
  267068, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e3m2.f32.ue8m0
  267144, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e4m3.f32.ue8m0
  267220, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e5m2.f32.ue8m0
  267296, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e2m1.f32.ue8m0
  267372, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e2m3.f32.ue8m0
  267448, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e3m2.f32.ue8m0
  267524, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e4m3.f32.ue8m0
  267600, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e5m2.f32.ue8m0
  267676, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m1.f32.ue8m0
  267761, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m3.f32.ue8m0
  267846, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e3m2.f32.ue8m0
  267931, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e4m3.f32.ue8m0
  268016, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e5m2.f32.ue8m0
  268101, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m1.f32.ue8m0
  268186, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m3.f32.ue8m0
  268271, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e3m2.f32.ue8m0
  268356, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e4m3.f32.ue8m0
  268441, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e5m2.f32.ue8m0
  268526, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m1.f32.ue8m0
  268611, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m3.f32.ue8m0
  268696, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e3m2.f32.ue8m0
  268781, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e4m3.f32.ue8m0
  268866, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e5m2.f32.ue8m0
  268951, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m1.f32.ue8m0
  269036, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m3.f32.ue8m0
  269121, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e3m2.f32.ue8m0
  269206, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e4m3.f32.ue8m0
  269291, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e5m2.f32.ue8m0
  269376, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m1.f32.ue8m0
  269461, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m3.f32.ue8m0
  269546, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e3m2.f32.ue8m0
  269631, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e4m3.f32.ue8m0
  269716, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e5m2.f32.ue8m0
  269801, // llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4.f32.e2m1.e2m1.f32.ue8m0
  269873, // llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0
  269954, // llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4nvf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0
  270039, // llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4nvf4.scale.4x.f32.e2m1.e2m1.f32.ue4m3
  270124, // llvm.nvvm.mma.m16n8k16.row.col.bf16
  270160, // llvm.nvvm.mma.m16n8k16.row.col.f16.e4m3.e4m3.f16
  270209, // llvm.nvvm.mma.m16n8k16.row.col.f16.e4m3.e5m2.f16
  270258, // llvm.nvvm.mma.m16n8k16.row.col.f16.e5m2.e4m3.f16
  270307, // llvm.nvvm.mma.m16n8k16.row.col.f16.e5m2.e5m2.f16
  270356, // llvm.nvvm.mma.m16n8k16.row.col.f16.f16
  270395, // llvm.nvvm.mma.m16n8k16.row.col.f32.e4m3.e4m3.f32
  270444, // llvm.nvvm.mma.m16n8k16.row.col.f32.e4m3.e5m2.f32
  270493, // llvm.nvvm.mma.m16n8k16.row.col.f32.e5m2.e4m3.f32
  270542, // llvm.nvvm.mma.m16n8k16.row.col.f32.e5m2.e5m2.f32
  270591, // llvm.nvvm.mma.m16n8k16.row.col.f32.f32
  270630, // llvm.nvvm.mma.m16n8k16.row.col.f64
  270665, // llvm.nvvm.mma.m16n8k16.row.col.s8
  270699, // llvm.nvvm.mma.m16n8k16.row.col.s8.u8
  270736, // llvm.nvvm.mma.m16n8k16.row.col.satfinite.s8
  270780, // llvm.nvvm.mma.m16n8k16.row.col.satfinite.s8.u8
  270827, // llvm.nvvm.mma.m16n8k16.row.col.satfinite.u8
  270871, // llvm.nvvm.mma.m16n8k16.row.col.satfinite.u8.s8
  270918, // llvm.nvvm.mma.m16n8k16.row.col.u8
  270952, // llvm.nvvm.mma.m16n8k16.row.col.u8.s8
  270989, // llvm.nvvm.mma.m16n8k32.row.col.f16.e4m3.e4m3.f16
  271038, // llvm.nvvm.mma.m16n8k32.row.col.f16.e4m3.e5m2.f16
  271087, // llvm.nvvm.mma.m16n8k32.row.col.f16.e5m2.e4m3.f16
  271136, // llvm.nvvm.mma.m16n8k32.row.col.f16.e5m2.e5m2.f16
  271185, // llvm.nvvm.mma.m16n8k32.row.col.f32.e4m3.e4m3.f32
  271234, // llvm.nvvm.mma.m16n8k32.row.col.f32.e4m3.e5m2.f32
  271283, // llvm.nvvm.mma.m16n8k32.row.col.f32.e5m2.e4m3.f32
  271332, // llvm.nvvm.mma.m16n8k32.row.col.f32.e5m2.e5m2.f32
  271381, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e2m1.f16
  271442, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e2m3.f16
  271503, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e3m2.f16
  271564, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e4m3.f16
  271625, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e5m2.f16
  271686, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e2m1.f16
  271747, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e2m3.f16
  271808, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e3m2.f16
  271869, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e4m3.f16
  271930, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e5m2.f16
  271991, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e2m1.f16
  272052, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e2m3.f16
  272113, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e3m2.f16
  272174, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e4m3.f16
  272235, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e5m2.f16
  272296, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e2m1.f16
  272357, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e2m3.f16
  272418, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e3m2.f16
  272479, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e4m3.f16
  272540, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e5m2.f16
  272601, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e2m1.f16
  272662, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e2m3.f16
  272723, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e3m2.f16
  272784, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e4m3.f16
  272845, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e5m2.f16
  272906, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e2m1.f32
  272967, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e2m3.f32
  273028, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e3m2.f32
  273089, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e4m3.f32
  273150, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e5m2.f32
  273211, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e2m1.f32
  273272, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e2m3.f32
  273333, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e3m2.f32
  273394, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e4m3.f32
  273455, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e5m2.f32
  273516, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e2m1.f32
  273577, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e2m3.f32
  273638, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e3m2.f32
  273699, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e4m3.f32
  273760, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e5m2.f32
  273821, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e2m1.f32
  273882, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e2m3.f32
  273943, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e3m2.f32
  274004, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e4m3.f32
  274065, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e5m2.f32
  274126, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e2m1.f32
  274187, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e2m3.f32
  274248, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e3m2.f32
  274309, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e4m3.f32
  274370, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e5m2.f32
  274431, // llvm.nvvm.mma.m16n8k32.row.col.s4
  274465, // llvm.nvvm.mma.m16n8k32.row.col.s4.u4
  274502, // llvm.nvvm.mma.m16n8k32.row.col.s8
  274536, // llvm.nvvm.mma.m16n8k32.row.col.s8.u8
  274573, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.s4
  274617, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.s4.u4
  274664, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.s8
  274708, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.s8.u8
  274755, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.u4
  274799, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.u4.s4
  274846, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.u8
  274890, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.u8.s8
  274937, // llvm.nvvm.mma.m16n8k32.row.col.u4
  274971, // llvm.nvvm.mma.m16n8k32.row.col.u4.s4
  275008, // llvm.nvvm.mma.m16n8k32.row.col.u8
  275042, // llvm.nvvm.mma.m16n8k32.row.col.u8.s8
  275079, // llvm.nvvm.mma.m16n8k4.row.col.f64
  275113, // llvm.nvvm.mma.m16n8k4.row.col.tf32
  275148, // llvm.nvvm.mma.m16n8k64.row.col.s4
  275182, // llvm.nvvm.mma.m16n8k64.row.col.s4.u4
  275219, // llvm.nvvm.mma.m16n8k64.row.col.satfinite.s4
  275263, // llvm.nvvm.mma.m16n8k64.row.col.satfinite.s4.u4
  275310, // llvm.nvvm.mma.m16n8k64.row.col.satfinite.u4
  275354, // llvm.nvvm.mma.m16n8k64.row.col.satfinite.u4.s4
  275401, // llvm.nvvm.mma.m16n8k64.row.col.u4
  275435, // llvm.nvvm.mma.m16n8k64.row.col.u4.s4
  275472, // llvm.nvvm.mma.m16n8k8.row.col.bf16
  275507, // llvm.nvvm.mma.m16n8k8.row.col.f16.f16
  275545, // llvm.nvvm.mma.m16n8k8.row.col.f32.f32
  275583, // llvm.nvvm.mma.m16n8k8.row.col.f64
  275617, // llvm.nvvm.mma.m16n8k8.row.col.tf32
  275652, // llvm.nvvm.mma.m8n8k16.row.col.s8
  275685, // llvm.nvvm.mma.m8n8k16.row.col.s8.u8
  275721, // llvm.nvvm.mma.m8n8k16.row.col.satfinite.s8
  275764, // llvm.nvvm.mma.m8n8k16.row.col.satfinite.s8.u8
  275810, // llvm.nvvm.mma.m8n8k16.row.col.satfinite.u8
  275853, // llvm.nvvm.mma.m8n8k16.row.col.satfinite.u8.s8
  275899, // llvm.nvvm.mma.m8n8k16.row.col.u8
  275932, // llvm.nvvm.mma.m8n8k16.row.col.u8.s8
  275968, // llvm.nvvm.mma.m8n8k32.row.col.s4
  276001, // llvm.nvvm.mma.m8n8k32.row.col.s4.u4
  276037, // llvm.nvvm.mma.m8n8k32.row.col.satfinite.s4
  276080, // llvm.nvvm.mma.m8n8k32.row.col.satfinite.s4.u4
  276126, // llvm.nvvm.mma.m8n8k32.row.col.satfinite.u4
  276169, // llvm.nvvm.mma.m8n8k32.row.col.satfinite.u4.s4
  276215, // llvm.nvvm.mma.m8n8k32.row.col.u4
  276248, // llvm.nvvm.mma.m8n8k32.row.col.u4.s4
  276284, // llvm.nvvm.mma.m8n8k4.col.col.f16.f16
  276321, // llvm.nvvm.mma.m8n8k4.col.col.f32.f16
  276358, // llvm.nvvm.mma.m8n8k4.col.col.f32.f32
  276395, // llvm.nvvm.mma.m8n8k4.col.row.f16.f16
  276432, // llvm.nvvm.mma.m8n8k4.col.row.f32.f16
  276469, // llvm.nvvm.mma.m8n8k4.col.row.f32.f32
  276506, // llvm.nvvm.mma.m8n8k4.row.col.f16.f16
  276543, // llvm.nvvm.mma.m8n8k4.row.col.f32.f16
  276580, // llvm.nvvm.mma.m8n8k4.row.col.f32.f32
  276617, // llvm.nvvm.mma.m8n8k4.row.col.f64
  276650, // llvm.nvvm.mma.m8n8k4.row.row.f16.f16
  276687, // llvm.nvvm.mma.m8n8k4.row.row.f32.f16
  276724, // llvm.nvvm.mma.m8n8k4.row.row.f32.f32
  276761, // llvm.nvvm.mma.sp.m16n8k128.row.col.s4
  276799, // llvm.nvvm.mma.sp.m16n8k128.row.col.s4.u4
  276840, // llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.s4
  276888, // llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.s4.u4
  276939, // llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.u4
  276987, // llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.u4.s4
  277038, // llvm.nvvm.mma.sp.m16n8k128.row.col.u4
  277076, // llvm.nvvm.mma.sp.m16n8k128.row.col.u4.s4
  277117, // llvm.nvvm.mma.sp.m16n8k16.row.col.bf16
  277156, // llvm.nvvm.mma.sp.m16n8k16.row.col.f16.f16
  277198, // llvm.nvvm.mma.sp.m16n8k16.row.col.f32.f32
  277240, // llvm.nvvm.mma.sp.m16n8k16.row.col.tf32
  277279, // llvm.nvvm.mma.sp.m16n8k32.row.col.bf16
  277318, // llvm.nvvm.mma.sp.m16n8k32.row.col.f16.f16
  277360, // llvm.nvvm.mma.sp.m16n8k32.row.col.f32.f32
  277402, // llvm.nvvm.mma.sp.m16n8k32.row.col.s8
  277439, // llvm.nvvm.mma.sp.m16n8k32.row.col.s8.u8
  277479, // llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.s8
  277526, // llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.s8.u8
  277576, // llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.u8
  277623, // llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.u8.s8
  277673, // llvm.nvvm.mma.sp.m16n8k32.row.col.u8
  277710, // llvm.nvvm.mma.sp.m16n8k32.row.col.u8.s8
  277750, // llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e4m3.e4m3.f32
  277802, // llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e4m3.e5m2.f32
  277854, // llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e5m2.e4m3.f32
  277906, // llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e5m2.e5m2.f32
  277958, // llvm.nvvm.mma.sp.m16n8k64.row.col.s4
  277995, // llvm.nvvm.mma.sp.m16n8k64.row.col.s4.u4
  278035, // llvm.nvvm.mma.sp.m16n8k64.row.col.s8
  278072, // llvm.nvvm.mma.sp.m16n8k64.row.col.s8.u8
  278112, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s4
  278159, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s4.u4
  278209, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s8
  278256, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s8.u8
  278306, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u4
  278353, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u4.s4
  278403, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u8
  278450, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u8.s8
  278500, // llvm.nvvm.mma.sp.m16n8k64.row.col.u4
  278537, // llvm.nvvm.mma.sp.m16n8k64.row.col.u4.s4
  278577, // llvm.nvvm.mma.sp.m16n8k64.row.col.u8
  278614, // llvm.nvvm.mma.sp.m16n8k64.row.col.u8.s8
  278654, // llvm.nvvm.mma.sp.m16n8k8.row.col.tf32
  278692, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4.f32.e2m1.e2m1.f32.ue8m0
  278785, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0
  278887, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4nvf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0
  278993, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4nvf4.scale.4x.f32.e2m1.e2m1.f32.ue4m3
  279099, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e2m1.f32.ue8m0
  279195, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e2m3.f32.ue8m0
  279291, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e3m2.f32.ue8m0
  279387, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e4m3.f32.ue8m0
  279483, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e5m2.f32.ue8m0
  279579, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e2m1.f32.ue8m0
  279675, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e2m3.f32.ue8m0
  279771, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e3m2.f32.ue8m0
  279867, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e4m3.f32.ue8m0
  279963, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e5m2.f32.ue8m0
  280059, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e2m1.f32.ue8m0
  280155, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e2m3.f32.ue8m0
  280251, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e3m2.f32.ue8m0
  280347, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e4m3.f32.ue8m0
  280443, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e5m2.f32.ue8m0
  280539, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e2m1.f32.ue8m0
  280635, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e2m3.f32.ue8m0
  280731, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e3m2.f32.ue8m0
  280827, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e4m3.f32.ue8m0
  280923, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e5m2.f32.ue8m0
  281019, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e2m1.f32.ue8m0
  281115, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e2m3.f32.ue8m0
  281211, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e3m2.f32.ue8m0
  281307, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e4m3.f32.ue8m0
  281403, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e5m2.f32.ue8m0
  281499, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m1.f32.ue8m0
  281604, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m3.f32.ue8m0
  281709, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e3m2.f32.ue8m0
  281814, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e4m3.f32.ue8m0
  281919, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e5m2.f32.ue8m0
  282024, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m1.f32.ue8m0
  282129, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m3.f32.ue8m0
  282234, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e3m2.f32.ue8m0
  282339, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e4m3.f32.ue8m0
  282444, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e5m2.f32.ue8m0
  282549, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m1.f32.ue8m0
  282654, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m3.f32.ue8m0
  282759, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e3m2.f32.ue8m0
  282864, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e4m3.f32.ue8m0
  282969, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e5m2.f32.ue8m0
  283074, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m1.f32.ue8m0
  283179, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m3.f32.ue8m0
  283284, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e3m2.f32.ue8m0
  283389, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e4m3.f32.ue8m0
  283494, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e5m2.f32.ue8m0
  283599, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m1.f32.ue8m0
  283704, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m3.f32.ue8m0
  283809, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e3m2.f32.ue8m0
  283914, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e4m3.f32.ue8m0
  284019, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e5m2.f32.ue8m0
  284124, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.s4
  284179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.s4.u4
  284237, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.s4
  284302, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.s4.u4
  284370, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.u4
  284435, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.u4.s4
  284503, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.u4
  284558, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.u4.s4
  284616, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.bf16
  284672, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.f16.f16
  284731, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.f32.f32
  284790, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.tf32
  284846, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.bf16
  284902, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.f16.f16
  284961, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.f32.f32
  285020, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.s8
  285074, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.s8.u8
  285131, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.s8
  285195, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.s8.u8
  285262, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.u8
  285326, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.u8.s8
  285393, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.u8
  285447, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.u8.s8
  285504, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e4m3.e4m3.f32
  285573, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e4m3.e5m2.f32
  285642, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e5m2.e4m3.f32
  285711, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e5m2.e5m2.f32
  285780, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e2m1.f16
  285861, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e2m3.f16
  285942, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e3m2.f16
  286023, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e4m3.f16
  286104, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e5m2.f16
  286185, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e2m1.f16
  286266, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e2m3.f16
  286347, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e3m2.f16
  286428, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e4m3.f16
  286509, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e5m2.f16
  286590, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e2m1.f16
  286671, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e2m3.f16
  286752, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e3m2.f16
  286833, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e4m3.f16
  286914, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e5m2.f16
  286995, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e2m1.f16
  287076, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e2m3.f16
  287157, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e3m2.f16
  287238, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e4m3.f16
  287319, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e5m2.f16
  287400, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e2m1.f16
  287481, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e2m3.f16
  287562, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e3m2.f16
  287643, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e4m3.f16
  287724, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e5m2.f16
  287805, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e2m1.f32
  287886, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e2m3.f32
  287967, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e3m2.f32
  288048, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e4m3.f32
  288129, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e5m2.f32
  288210, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e2m1.f32
  288291, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e2m3.f32
  288372, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e3m2.f32
  288453, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e4m3.f32
  288534, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e5m2.f32
  288615, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e2m1.f32
  288696, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e2m3.f32
  288777, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e3m2.f32
  288858, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e4m3.f32
  288939, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e5m2.f32
  289020, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e2m1.f32
  289101, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e2m3.f32
  289182, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e3m2.f32
  289263, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e4m3.f32
  289344, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e5m2.f32
  289425, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e2m1.f32
  289506, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e2m3.f32
  289587, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e3m2.f32
  289668, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e4m3.f32
  289749, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e5m2.f32
  289830, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s4
  289884, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s4.u4
  289941, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s8
  289995, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s8.u8
  290052, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s4
  290116, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s4.u4
  290183, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s8
  290247, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s8.u8
  290314, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u4
  290378, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u4.s4
  290445, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u8
  290509, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u8.s8
  290576, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u4
  290630, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u4.s4
  290687, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u8
  290741, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u8.s8
  290798, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k8.row.col.tf32
  290853, // llvm.nvvm.mma.xor.popc.m16n8k128.row.col.b1
  290897, // llvm.nvvm.mma.xor.popc.m16n8k256.row.col.b1
  290941, // llvm.nvvm.mma.xor.popc.m8n8k128.row.col.b1
  290984, // llvm.nvvm.move.double
  291006, // llvm.nvvm.move.float
  291027, // llvm.nvvm.move.i16
  291046, // llvm.nvvm.move.i32
  291065, // llvm.nvvm.move.i64
  291084, // llvm.nvvm.move.ptr
  291103, // llvm.nvvm.mul.rm.d
  291122, // llvm.nvvm.mul.rm.f
  291141, // llvm.nvvm.mul.rm.ftz.f
  291164, // llvm.nvvm.mul.rn.d
  291183, // llvm.nvvm.mul.rn.f
  291202, // llvm.nvvm.mul.rn.ftz.f
  291225, // llvm.nvvm.mul.rp.d
  291244, // llvm.nvvm.mul.rp.f
  291263, // llvm.nvvm.mul.rp.ftz.f
  291286, // llvm.nvvm.mul.rz.d
  291305, // llvm.nvvm.mul.rz.f
  291324, // llvm.nvvm.mul.rz.ftz.f
  291347, // llvm.nvvm.mul24.i
  291365, // llvm.nvvm.mul24.ui
  291384, // llvm.nvvm.mulhi.i
  291402, // llvm.nvvm.mulhi.ll
  291421, // llvm.nvvm.mulhi.s
  291439, // llvm.nvvm.mulhi.ui
  291458, // llvm.nvvm.mulhi.ull
  291478, // llvm.nvvm.mulhi.us
  291497, // llvm.nvvm.nanosleep
  291517, // llvm.nvvm.neg.bf16
  291536, // llvm.nvvm.neg.bf16x2
  291557, // llvm.nvvm.pm.event.mask
  291581, // llvm.nvvm.prefetch.L1
  291603, // llvm.nvvm.prefetch.L2
  291625, // llvm.nvvm.prefetch.global.L1
  291654, // llvm.nvvm.prefetch.global.L2
  291683, // llvm.nvvm.prefetch.global.L2.evict.last
  291723, // llvm.nvvm.prefetch.global.L2.evict.normal
  291765, // llvm.nvvm.prefetch.local.L1
  291793, // llvm.nvvm.prefetch.local.L2
  291821, // llvm.nvvm.prefetch.tensormap
  291850, // llvm.nvvm.prefetchu.L1
  291873, // llvm.nvvm.prmt
  291888, // llvm.nvvm.prmt.b4e
  291907, // llvm.nvvm.prmt.ecl
  291926, // llvm.nvvm.prmt.ecr
  291945, // llvm.nvvm.prmt.f4e
  291964, // llvm.nvvm.prmt.rc16
  291984, // llvm.nvvm.prmt.rc8
  292003, // llvm.nvvm.rcp.approx.ftz.d
  292030, // llvm.nvvm.rcp.approx.ftz.f
  292057, // llvm.nvvm.rcp.rm.d
  292076, // llvm.nvvm.rcp.rm.f
  292095, // llvm.nvvm.rcp.rm.ftz.f
  292118, // llvm.nvvm.rcp.rn.d
  292137, // llvm.nvvm.rcp.rn.f
  292156, // llvm.nvvm.rcp.rn.ftz.f
  292179, // llvm.nvvm.rcp.rp.d
  292198, // llvm.nvvm.rcp.rp.f
  292217, // llvm.nvvm.rcp.rp.ftz.f
  292240, // llvm.nvvm.rcp.rz.d
  292259, // llvm.nvvm.rcp.rz.f
  292278, // llvm.nvvm.rcp.rz.ftz.f
  292301, // llvm.nvvm.read.ptx.sreg.aggr_smem_size
  292340, // llvm.nvvm.read.ptx.sreg.clock
  292370, // llvm.nvvm.read.ptx.sreg.clock64
  292402, // llvm.nvvm.read.ptx.sreg.cluster.ctaid.w
  292442, // llvm.nvvm.read.ptx.sreg.cluster.ctaid.x
  292482, // llvm.nvvm.read.ptx.sreg.cluster.ctaid.y
  292522, // llvm.nvvm.read.ptx.sreg.cluster.ctaid.z
  292562, // llvm.nvvm.read.ptx.sreg.cluster.ctarank
  292602, // llvm.nvvm.read.ptx.sreg.cluster.nctaid.w
  292643, // llvm.nvvm.read.ptx.sreg.cluster.nctaid.x
  292684, // llvm.nvvm.read.ptx.sreg.cluster.nctaid.y
  292725, // llvm.nvvm.read.ptx.sreg.cluster.nctaid.z
  292766, // llvm.nvvm.read.ptx.sreg.cluster.nctarank
  292807, // llvm.nvvm.read.ptx.sreg.clusterid.w
  292843, // llvm.nvvm.read.ptx.sreg.clusterid.x
  292879, // llvm.nvvm.read.ptx.sreg.clusterid.y
  292915, // llvm.nvvm.read.ptx.sreg.clusterid.z
  292951, // llvm.nvvm.read.ptx.sreg.ctaid.w
  292983, // llvm.nvvm.read.ptx.sreg.ctaid.x
  293015, // llvm.nvvm.read.ptx.sreg.ctaid.y
  293047, // llvm.nvvm.read.ptx.sreg.ctaid.z
  293079, // llvm.nvvm.read.ptx.sreg.dynamic_smem_size
  293121, // llvm.nvvm.read.ptx.sreg.envreg0
  293153, // llvm.nvvm.read.ptx.sreg.envreg1
  293185, // llvm.nvvm.read.ptx.sreg.envreg10
  293218, // llvm.nvvm.read.ptx.sreg.envreg11
  293251, // llvm.nvvm.read.ptx.sreg.envreg12
  293284, // llvm.nvvm.read.ptx.sreg.envreg13
  293317, // llvm.nvvm.read.ptx.sreg.envreg14
  293350, // llvm.nvvm.read.ptx.sreg.envreg15
  293383, // llvm.nvvm.read.ptx.sreg.envreg16
  293416, // llvm.nvvm.read.ptx.sreg.envreg17
  293449, // llvm.nvvm.read.ptx.sreg.envreg18
  293482, // llvm.nvvm.read.ptx.sreg.envreg19
  293515, // llvm.nvvm.read.ptx.sreg.envreg2
  293547, // llvm.nvvm.read.ptx.sreg.envreg20
  293580, // llvm.nvvm.read.ptx.sreg.envreg21
  293613, // llvm.nvvm.read.ptx.sreg.envreg22
  293646, // llvm.nvvm.read.ptx.sreg.envreg23
  293679, // llvm.nvvm.read.ptx.sreg.envreg24
  293712, // llvm.nvvm.read.ptx.sreg.envreg25
  293745, // llvm.nvvm.read.ptx.sreg.envreg26
  293778, // llvm.nvvm.read.ptx.sreg.envreg27
  293811, // llvm.nvvm.read.ptx.sreg.envreg28
  293844, // llvm.nvvm.read.ptx.sreg.envreg29
  293877, // llvm.nvvm.read.ptx.sreg.envreg3
  293909, // llvm.nvvm.read.ptx.sreg.envreg30
  293942, // llvm.nvvm.read.ptx.sreg.envreg31
  293975, // llvm.nvvm.read.ptx.sreg.envreg4
  294007, // llvm.nvvm.read.ptx.sreg.envreg5
  294039, // llvm.nvvm.read.ptx.sreg.envreg6
  294071, // llvm.nvvm.read.ptx.sreg.envreg7
  294103, // llvm.nvvm.read.ptx.sreg.envreg8
  294135, // llvm.nvvm.read.ptx.sreg.envreg9
  294167, // llvm.nvvm.read.ptx.sreg.globaltimer
  294203, // llvm.nvvm.read.ptx.sreg.globaltimer.lo
  294242, // llvm.nvvm.read.ptx.sreg.gridid
  294273, // llvm.nvvm.read.ptx.sreg.laneid
  294304, // llvm.nvvm.read.ptx.sreg.lanemask.eq
  294340, // llvm.nvvm.read.ptx.sreg.lanemask.ge
  294376, // llvm.nvvm.read.ptx.sreg.lanemask.gt
  294412, // llvm.nvvm.read.ptx.sreg.lanemask.le
  294448, // llvm.nvvm.read.ptx.sreg.lanemask.lt
  294484, // llvm.nvvm.read.ptx.sreg.nclusterid.w
  294521, // llvm.nvvm.read.ptx.sreg.nclusterid.x
  294558, // llvm.nvvm.read.ptx.sreg.nclusterid.y
  294595, // llvm.nvvm.read.ptx.sreg.nclusterid.z
  294632, // llvm.nvvm.read.ptx.sreg.nctaid.w
  294665, // llvm.nvvm.read.ptx.sreg.nctaid.x
  294698, // llvm.nvvm.read.ptx.sreg.nctaid.y
  294731, // llvm.nvvm.read.ptx.sreg.nctaid.z
  294764, // llvm.nvvm.read.ptx.sreg.nsmid
  294794, // llvm.nvvm.read.ptx.sreg.ntid.w
  294825, // llvm.nvvm.read.ptx.sreg.ntid.x
  294856, // llvm.nvvm.read.ptx.sreg.ntid.y
  294887, // llvm.nvvm.read.ptx.sreg.ntid.z
  294918, // llvm.nvvm.read.ptx.sreg.nwarpid
  294950, // llvm.nvvm.read.ptx.sreg.pm0
  294978, // llvm.nvvm.read.ptx.sreg.pm1
  295006, // llvm.nvvm.read.ptx.sreg.pm2
  295034, // llvm.nvvm.read.ptx.sreg.pm3
  295062, // llvm.nvvm.read.ptx.sreg.smid
  295091, // llvm.nvvm.read.ptx.sreg.tid.w
  295121, // llvm.nvvm.read.ptx.sreg.tid.x
  295151, // llvm.nvvm.read.ptx.sreg.tid.y
  295181, // llvm.nvvm.read.ptx.sreg.tid.z
  295211, // llvm.nvvm.read.ptx.sreg.total_smem_size
  295251, // llvm.nvvm.read.ptx.sreg.warpid
  295282, // llvm.nvvm.read.ptx.sreg.warpsize
  295315, // llvm.nvvm.redux.sync.add
  295340, // llvm.nvvm.redux.sync.and
  295365, // llvm.nvvm.redux.sync.fmax
  295391, // llvm.nvvm.redux.sync.fmax.NaN
  295421, // llvm.nvvm.redux.sync.fmax.abs
  295451, // llvm.nvvm.redux.sync.fmax.abs.NaN
  295485, // llvm.nvvm.redux.sync.fmin
  295511, // llvm.nvvm.redux.sync.fmin.NaN
  295541, // llvm.nvvm.redux.sync.fmin.abs
  295571, // llvm.nvvm.redux.sync.fmin.abs.NaN
  295605, // llvm.nvvm.redux.sync.max
  295630, // llvm.nvvm.redux.sync.min
  295655, // llvm.nvvm.redux.sync.or
  295679, // llvm.nvvm.redux.sync.umax
  295705, // llvm.nvvm.redux.sync.umin
  295731, // llvm.nvvm.redux.sync.xor
  295756, // llvm.nvvm.reflect
  295774, // llvm.nvvm.round.d
  295792, // llvm.nvvm.round.f
  295810, // llvm.nvvm.round.ftz.f
  295832, // llvm.nvvm.rsqrt.approx.d
  295857, // llvm.nvvm.rsqrt.approx.f
  295882, // llvm.nvvm.rsqrt.approx.ftz.d
  295911, // llvm.nvvm.rsqrt.approx.ftz.f
  295940, // llvm.nvvm.sad.i
  295956, // llvm.nvvm.sad.ll
  295973, // llvm.nvvm.sad.s
  295989, // llvm.nvvm.sad.ui
  296006, // llvm.nvvm.sad.ull
  296024, // llvm.nvvm.sad.us
  296041, // llvm.nvvm.saturate.d
  296062, // llvm.nvvm.saturate.f
  296083, // llvm.nvvm.saturate.ftz.f
  296108, // llvm.nvvm.setmaxnreg.dec.sync.aligned.u32
  296150, // llvm.nvvm.setmaxnreg.inc.sync.aligned.u32
  296192, // llvm.nvvm.sext.clamp
  296213, // llvm.nvvm.sext.wrap
  296233, // llvm.nvvm.shfl.bfly.f32
  296257, // llvm.nvvm.shfl.bfly.f32p
  296282, // llvm.nvvm.shfl.bfly.i32
  296306, // llvm.nvvm.shfl.bfly.i32p
  296331, // llvm.nvvm.shfl.down.f32
  296355, // llvm.nvvm.shfl.down.f32p
  296380, // llvm.nvvm.shfl.down.i32
  296404, // llvm.nvvm.shfl.down.i32p
  296429, // llvm.nvvm.shfl.idx.f32
  296452, // llvm.nvvm.shfl.idx.f32p
  296476, // llvm.nvvm.shfl.idx.i32
  296499, // llvm.nvvm.shfl.idx.i32p
  296523, // llvm.nvvm.shfl.sync.bfly.f32
  296552, // llvm.nvvm.shfl.sync.bfly.f32p
  296582, // llvm.nvvm.shfl.sync.bfly.i32
  296611, // llvm.nvvm.shfl.sync.bfly.i32p
  296641, // llvm.nvvm.shfl.sync.down.f32
  296670, // llvm.nvvm.shfl.sync.down.f32p
  296700, // llvm.nvvm.shfl.sync.down.i32
  296729, // llvm.nvvm.shfl.sync.down.i32p
  296759, // llvm.nvvm.shfl.sync.idx.f32
  296787, // llvm.nvvm.shfl.sync.idx.f32p
  296816, // llvm.nvvm.shfl.sync.idx.i32
  296844, // llvm.nvvm.shfl.sync.idx.i32p
  296873, // llvm.nvvm.shfl.sync.up.f32
  296900, // llvm.nvvm.shfl.sync.up.f32p
  296928, // llvm.nvvm.shfl.sync.up.i32
  296955, // llvm.nvvm.shfl.sync.up.i32p
  296983, // llvm.nvvm.shfl.up.f32
  297005, // llvm.nvvm.shfl.up.f32p
  297028, // llvm.nvvm.shfl.up.i32
  297050, // llvm.nvvm.shfl.up.i32p
  297073, // llvm.nvvm.sin.approx.f
  297096, // llvm.nvvm.sin.approx.ftz.f
  297123, // llvm.nvvm.sqrt.approx.f
  297147, // llvm.nvvm.sqrt.approx.ftz.f
  297175, // llvm.nvvm.sqrt.f
  297192, // llvm.nvvm.sqrt.rm.d
  297212, // llvm.nvvm.sqrt.rm.f
  297232, // llvm.nvvm.sqrt.rm.ftz.f
  297256, // llvm.nvvm.sqrt.rn.d
  297276, // llvm.nvvm.sqrt.rn.f
  297296, // llvm.nvvm.sqrt.rn.ftz.f
  297320, // llvm.nvvm.sqrt.rp.d
  297340, // llvm.nvvm.sqrt.rp.f
  297360, // llvm.nvvm.sqrt.rp.ftz.f
  297384, // llvm.nvvm.sqrt.rz.d
  297404, // llvm.nvvm.sqrt.rz.f
  297424, // llvm.nvvm.sqrt.rz.ftz.f
  297448, // llvm.nvvm.st.bulk
  297466, // llvm.nvvm.st.bulk.shared.cta
  297495, // llvm.nvvm.stmatrix.sync.aligned.m16n8.x1.trans.b8
  297545, // llvm.nvvm.stmatrix.sync.aligned.m16n8.x2.trans.b8
  297595, // llvm.nvvm.stmatrix.sync.aligned.m16n8.x4.trans.b8
  297645, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x1.b16
  297689, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x1.trans.b16
  297739, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x2.b16
  297783, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x2.trans.b16
  297833, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x4.b16
  297877, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x4.trans.b16
  297927, // llvm.nvvm.suld.1d.array.i16.clamp
  297961, // llvm.nvvm.suld.1d.array.i16.trap
  297994, // llvm.nvvm.suld.1d.array.i16.zero
  298027, // llvm.nvvm.suld.1d.array.i32.clamp
  298061, // llvm.nvvm.suld.1d.array.i32.trap
  298094, // llvm.nvvm.suld.1d.array.i32.zero
  298127, // llvm.nvvm.suld.1d.array.i64.clamp
  298161, // llvm.nvvm.suld.1d.array.i64.trap
  298194, // llvm.nvvm.suld.1d.array.i64.zero
  298227, // llvm.nvvm.suld.1d.array.i8.clamp
  298260, // llvm.nvvm.suld.1d.array.i8.trap
  298292, // llvm.nvvm.suld.1d.array.i8.zero
  298324, // llvm.nvvm.suld.1d.array.v2i16.clamp
  298360, // llvm.nvvm.suld.1d.array.v2i16.trap
  298395, // llvm.nvvm.suld.1d.array.v2i16.zero
  298430, // llvm.nvvm.suld.1d.array.v2i32.clamp
  298466, // llvm.nvvm.suld.1d.array.v2i32.trap
  298501, // llvm.nvvm.suld.1d.array.v2i32.zero
  298536, // llvm.nvvm.suld.1d.array.v2i64.clamp
  298572, // llvm.nvvm.suld.1d.array.v2i64.trap
  298607, // llvm.nvvm.suld.1d.array.v2i64.zero
  298642, // llvm.nvvm.suld.1d.array.v2i8.clamp
  298677, // llvm.nvvm.suld.1d.array.v2i8.trap
  298711, // llvm.nvvm.suld.1d.array.v2i8.zero
  298745, // llvm.nvvm.suld.1d.array.v4i16.clamp
  298781, // llvm.nvvm.suld.1d.array.v4i16.trap
  298816, // llvm.nvvm.suld.1d.array.v4i16.zero
  298851, // llvm.nvvm.suld.1d.array.v4i32.clamp
  298887, // llvm.nvvm.suld.1d.array.v4i32.trap
  298922, // llvm.nvvm.suld.1d.array.v4i32.zero
  298957, // llvm.nvvm.suld.1d.array.v4i8.clamp
  298992, // llvm.nvvm.suld.1d.array.v4i8.trap
  299026, // llvm.nvvm.suld.1d.array.v4i8.zero
  299060, // llvm.nvvm.suld.1d.i16.clamp
  299088, // llvm.nvvm.suld.1d.i16.trap
  299115, // llvm.nvvm.suld.1d.i16.zero
  299142, // llvm.nvvm.suld.1d.i32.clamp
  299170, // llvm.nvvm.suld.1d.i32.trap
  299197, // llvm.nvvm.suld.1d.i32.zero
  299224, // llvm.nvvm.suld.1d.i64.clamp
  299252, // llvm.nvvm.suld.1d.i64.trap
  299279, // llvm.nvvm.suld.1d.i64.zero
  299306, // llvm.nvvm.suld.1d.i8.clamp
  299333, // llvm.nvvm.suld.1d.i8.trap
  299359, // llvm.nvvm.suld.1d.i8.zero
  299385, // llvm.nvvm.suld.1d.v2i16.clamp
  299415, // llvm.nvvm.suld.1d.v2i16.trap
  299444, // llvm.nvvm.suld.1d.v2i16.zero
  299473, // llvm.nvvm.suld.1d.v2i32.clamp
  299503, // llvm.nvvm.suld.1d.v2i32.trap
  299532, // llvm.nvvm.suld.1d.v2i32.zero
  299561, // llvm.nvvm.suld.1d.v2i64.clamp
  299591, // llvm.nvvm.suld.1d.v2i64.trap
  299620, // llvm.nvvm.suld.1d.v2i64.zero
  299649, // llvm.nvvm.suld.1d.v2i8.clamp
  299678, // llvm.nvvm.suld.1d.v2i8.trap
  299706, // llvm.nvvm.suld.1d.v2i8.zero
  299734, // llvm.nvvm.suld.1d.v4i16.clamp
  299764, // llvm.nvvm.suld.1d.v4i16.trap
  299793, // llvm.nvvm.suld.1d.v4i16.zero
  299822, // llvm.nvvm.suld.1d.v4i32.clamp
  299852, // llvm.nvvm.suld.1d.v4i32.trap
  299881, // llvm.nvvm.suld.1d.v4i32.zero
  299910, // llvm.nvvm.suld.1d.v4i8.clamp
  299939, // llvm.nvvm.suld.1d.v4i8.trap
  299967, // llvm.nvvm.suld.1d.v4i8.zero
  299995, // llvm.nvvm.suld.2d.array.i16.clamp
  300029, // llvm.nvvm.suld.2d.array.i16.trap
  300062, // llvm.nvvm.suld.2d.array.i16.zero
  300095, // llvm.nvvm.suld.2d.array.i32.clamp
  300129, // llvm.nvvm.suld.2d.array.i32.trap
  300162, // llvm.nvvm.suld.2d.array.i32.zero
  300195, // llvm.nvvm.suld.2d.array.i64.clamp
  300229, // llvm.nvvm.suld.2d.array.i64.trap
  300262, // llvm.nvvm.suld.2d.array.i64.zero
  300295, // llvm.nvvm.suld.2d.array.i8.clamp
  300328, // llvm.nvvm.suld.2d.array.i8.trap
  300360, // llvm.nvvm.suld.2d.array.i8.zero
  300392, // llvm.nvvm.suld.2d.array.v2i16.clamp
  300428, // llvm.nvvm.suld.2d.array.v2i16.trap
  300463, // llvm.nvvm.suld.2d.array.v2i16.zero
  300498, // llvm.nvvm.suld.2d.array.v2i32.clamp
  300534, // llvm.nvvm.suld.2d.array.v2i32.trap
  300569, // llvm.nvvm.suld.2d.array.v2i32.zero
  300604, // llvm.nvvm.suld.2d.array.v2i64.clamp
  300640, // llvm.nvvm.suld.2d.array.v2i64.trap
  300675, // llvm.nvvm.suld.2d.array.v2i64.zero
  300710, // llvm.nvvm.suld.2d.array.v2i8.clamp
  300745, // llvm.nvvm.suld.2d.array.v2i8.trap
  300779, // llvm.nvvm.suld.2d.array.v2i8.zero
  300813, // llvm.nvvm.suld.2d.array.v4i16.clamp
  300849, // llvm.nvvm.suld.2d.array.v4i16.trap
  300884, // llvm.nvvm.suld.2d.array.v4i16.zero
  300919, // llvm.nvvm.suld.2d.array.v4i32.clamp
  300955, // llvm.nvvm.suld.2d.array.v4i32.trap
  300990, // llvm.nvvm.suld.2d.array.v4i32.zero
  301025, // llvm.nvvm.suld.2d.array.v4i8.clamp
  301060, // llvm.nvvm.suld.2d.array.v4i8.trap
  301094, // llvm.nvvm.suld.2d.array.v4i8.zero
  301128, // llvm.nvvm.suld.2d.i16.clamp
  301156, // llvm.nvvm.suld.2d.i16.trap
  301183, // llvm.nvvm.suld.2d.i16.zero
  301210, // llvm.nvvm.suld.2d.i32.clamp
  301238, // llvm.nvvm.suld.2d.i32.trap
  301265, // llvm.nvvm.suld.2d.i32.zero
  301292, // llvm.nvvm.suld.2d.i64.clamp
  301320, // llvm.nvvm.suld.2d.i64.trap
  301347, // llvm.nvvm.suld.2d.i64.zero
  301374, // llvm.nvvm.suld.2d.i8.clamp
  301401, // llvm.nvvm.suld.2d.i8.trap
  301427, // llvm.nvvm.suld.2d.i8.zero
  301453, // llvm.nvvm.suld.2d.v2i16.clamp
  301483, // llvm.nvvm.suld.2d.v2i16.trap
  301512, // llvm.nvvm.suld.2d.v2i16.zero
  301541, // llvm.nvvm.suld.2d.v2i32.clamp
  301571, // llvm.nvvm.suld.2d.v2i32.trap
  301600, // llvm.nvvm.suld.2d.v2i32.zero
  301629, // llvm.nvvm.suld.2d.v2i64.clamp
  301659, // llvm.nvvm.suld.2d.v2i64.trap
  301688, // llvm.nvvm.suld.2d.v2i64.zero
  301717, // llvm.nvvm.suld.2d.v2i8.clamp
  301746, // llvm.nvvm.suld.2d.v2i8.trap
  301774, // llvm.nvvm.suld.2d.v2i8.zero
  301802, // llvm.nvvm.suld.2d.v4i16.clamp
  301832, // llvm.nvvm.suld.2d.v4i16.trap
  301861, // llvm.nvvm.suld.2d.v4i16.zero
  301890, // llvm.nvvm.suld.2d.v4i32.clamp
  301920, // llvm.nvvm.suld.2d.v4i32.trap
  301949, // llvm.nvvm.suld.2d.v4i32.zero
  301978, // llvm.nvvm.suld.2d.v4i8.clamp
  302007, // llvm.nvvm.suld.2d.v4i8.trap
  302035, // llvm.nvvm.suld.2d.v4i8.zero
  302063, // llvm.nvvm.suld.3d.i16.clamp
  302091, // llvm.nvvm.suld.3d.i16.trap
  302118, // llvm.nvvm.suld.3d.i16.zero
  302145, // llvm.nvvm.suld.3d.i32.clamp
  302173, // llvm.nvvm.suld.3d.i32.trap
  302200, // llvm.nvvm.suld.3d.i32.zero
  302227, // llvm.nvvm.suld.3d.i64.clamp
  302255, // llvm.nvvm.suld.3d.i64.trap
  302282, // llvm.nvvm.suld.3d.i64.zero
  302309, // llvm.nvvm.suld.3d.i8.clamp
  302336, // llvm.nvvm.suld.3d.i8.trap
  302362, // llvm.nvvm.suld.3d.i8.zero
  302388, // llvm.nvvm.suld.3d.v2i16.clamp
  302418, // llvm.nvvm.suld.3d.v2i16.trap
  302447, // llvm.nvvm.suld.3d.v2i16.zero
  302476, // llvm.nvvm.suld.3d.v2i32.clamp
  302506, // llvm.nvvm.suld.3d.v2i32.trap
  302535, // llvm.nvvm.suld.3d.v2i32.zero
  302564, // llvm.nvvm.suld.3d.v2i64.clamp
  302594, // llvm.nvvm.suld.3d.v2i64.trap
  302623, // llvm.nvvm.suld.3d.v2i64.zero
  302652, // llvm.nvvm.suld.3d.v2i8.clamp
  302681, // llvm.nvvm.suld.3d.v2i8.trap
  302709, // llvm.nvvm.suld.3d.v2i8.zero
  302737, // llvm.nvvm.suld.3d.v4i16.clamp
  302767, // llvm.nvvm.suld.3d.v4i16.trap
  302796, // llvm.nvvm.suld.3d.v4i16.zero
  302825, // llvm.nvvm.suld.3d.v4i32.clamp
  302855, // llvm.nvvm.suld.3d.v4i32.trap
  302884, // llvm.nvvm.suld.3d.v4i32.zero
  302913, // llvm.nvvm.suld.3d.v4i8.clamp
  302942, // llvm.nvvm.suld.3d.v4i8.trap
  302970, // llvm.nvvm.suld.3d.v4i8.zero
  302998, // llvm.nvvm.suq.array.size
  303023, // llvm.nvvm.suq.channel.data.type
  303055, // llvm.nvvm.suq.channel.order
  303083, // llvm.nvvm.suq.depth
  303103, // llvm.nvvm.suq.height
  303124, // llvm.nvvm.suq.width
  303144, // llvm.nvvm.sust.b.1d.array.i16.clamp
  303180, // llvm.nvvm.sust.b.1d.array.i16.trap
  303215, // llvm.nvvm.sust.b.1d.array.i16.zero
  303250, // llvm.nvvm.sust.b.1d.array.i32.clamp
  303286, // llvm.nvvm.sust.b.1d.array.i32.trap
  303321, // llvm.nvvm.sust.b.1d.array.i32.zero
  303356, // llvm.nvvm.sust.b.1d.array.i64.clamp
  303392, // llvm.nvvm.sust.b.1d.array.i64.trap
  303427, // llvm.nvvm.sust.b.1d.array.i64.zero
  303462, // llvm.nvvm.sust.b.1d.array.i8.clamp
  303497, // llvm.nvvm.sust.b.1d.array.i8.trap
  303531, // llvm.nvvm.sust.b.1d.array.i8.zero
  303565, // llvm.nvvm.sust.b.1d.array.v2i16.clamp
  303603, // llvm.nvvm.sust.b.1d.array.v2i16.trap
  303640, // llvm.nvvm.sust.b.1d.array.v2i16.zero
  303677, // llvm.nvvm.sust.b.1d.array.v2i32.clamp
  303715, // llvm.nvvm.sust.b.1d.array.v2i32.trap
  303752, // llvm.nvvm.sust.b.1d.array.v2i32.zero
  303789, // llvm.nvvm.sust.b.1d.array.v2i64.clamp
  303827, // llvm.nvvm.sust.b.1d.array.v2i64.trap
  303864, // llvm.nvvm.sust.b.1d.array.v2i64.zero
  303901, // llvm.nvvm.sust.b.1d.array.v2i8.clamp
  303938, // llvm.nvvm.sust.b.1d.array.v2i8.trap
  303974, // llvm.nvvm.sust.b.1d.array.v2i8.zero
  304010, // llvm.nvvm.sust.b.1d.array.v4i16.clamp
  304048, // llvm.nvvm.sust.b.1d.array.v4i16.trap
  304085, // llvm.nvvm.sust.b.1d.array.v4i16.zero
  304122, // llvm.nvvm.sust.b.1d.array.v4i32.clamp
  304160, // llvm.nvvm.sust.b.1d.array.v4i32.trap
  304197, // llvm.nvvm.sust.b.1d.array.v4i32.zero
  304234, // llvm.nvvm.sust.b.1d.array.v4i8.clamp
  304271, // llvm.nvvm.sust.b.1d.array.v4i8.trap
  304307, // llvm.nvvm.sust.b.1d.array.v4i8.zero
  304343, // llvm.nvvm.sust.b.1d.i16.clamp
  304373, // llvm.nvvm.sust.b.1d.i16.trap
  304402, // llvm.nvvm.sust.b.1d.i16.zero
  304431, // llvm.nvvm.sust.b.1d.i32.clamp
  304461, // llvm.nvvm.sust.b.1d.i32.trap
  304490, // llvm.nvvm.sust.b.1d.i32.zero
  304519, // llvm.nvvm.sust.b.1d.i64.clamp
  304549, // llvm.nvvm.sust.b.1d.i64.trap
  304578, // llvm.nvvm.sust.b.1d.i64.zero
  304607, // llvm.nvvm.sust.b.1d.i8.clamp
  304636, // llvm.nvvm.sust.b.1d.i8.trap
  304664, // llvm.nvvm.sust.b.1d.i8.zero
  304692, // llvm.nvvm.sust.b.1d.v2i16.clamp
  304724, // llvm.nvvm.sust.b.1d.v2i16.trap
  304755, // llvm.nvvm.sust.b.1d.v2i16.zero
  304786, // llvm.nvvm.sust.b.1d.v2i32.clamp
  304818, // llvm.nvvm.sust.b.1d.v2i32.trap
  304849, // llvm.nvvm.sust.b.1d.v2i32.zero
  304880, // llvm.nvvm.sust.b.1d.v2i64.clamp
  304912, // llvm.nvvm.sust.b.1d.v2i64.trap
  304943, // llvm.nvvm.sust.b.1d.v2i64.zero
  304974, // llvm.nvvm.sust.b.1d.v2i8.clamp
  305005, // llvm.nvvm.sust.b.1d.v2i8.trap
  305035, // llvm.nvvm.sust.b.1d.v2i8.zero
  305065, // llvm.nvvm.sust.b.1d.v4i16.clamp
  305097, // llvm.nvvm.sust.b.1d.v4i16.trap
  305128, // llvm.nvvm.sust.b.1d.v4i16.zero
  305159, // llvm.nvvm.sust.b.1d.v4i32.clamp
  305191, // llvm.nvvm.sust.b.1d.v4i32.trap
  305222, // llvm.nvvm.sust.b.1d.v4i32.zero
  305253, // llvm.nvvm.sust.b.1d.v4i8.clamp
  305284, // llvm.nvvm.sust.b.1d.v4i8.trap
  305314, // llvm.nvvm.sust.b.1d.v4i8.zero
  305344, // llvm.nvvm.sust.b.2d.array.i16.clamp
  305380, // llvm.nvvm.sust.b.2d.array.i16.trap
  305415, // llvm.nvvm.sust.b.2d.array.i16.zero
  305450, // llvm.nvvm.sust.b.2d.array.i32.clamp
  305486, // llvm.nvvm.sust.b.2d.array.i32.trap
  305521, // llvm.nvvm.sust.b.2d.array.i32.zero
  305556, // llvm.nvvm.sust.b.2d.array.i64.clamp
  305592, // llvm.nvvm.sust.b.2d.array.i64.trap
  305627, // llvm.nvvm.sust.b.2d.array.i64.zero
  305662, // llvm.nvvm.sust.b.2d.array.i8.clamp
  305697, // llvm.nvvm.sust.b.2d.array.i8.trap
  305731, // llvm.nvvm.sust.b.2d.array.i8.zero
  305765, // llvm.nvvm.sust.b.2d.array.v2i16.clamp
  305803, // llvm.nvvm.sust.b.2d.array.v2i16.trap
  305840, // llvm.nvvm.sust.b.2d.array.v2i16.zero
  305877, // llvm.nvvm.sust.b.2d.array.v2i32.clamp
  305915, // llvm.nvvm.sust.b.2d.array.v2i32.trap
  305952, // llvm.nvvm.sust.b.2d.array.v2i32.zero
  305989, // llvm.nvvm.sust.b.2d.array.v2i64.clamp
  306027, // llvm.nvvm.sust.b.2d.array.v2i64.trap
  306064, // llvm.nvvm.sust.b.2d.array.v2i64.zero
  306101, // llvm.nvvm.sust.b.2d.array.v2i8.clamp
  306138, // llvm.nvvm.sust.b.2d.array.v2i8.trap
  306174, // llvm.nvvm.sust.b.2d.array.v2i8.zero
  306210, // llvm.nvvm.sust.b.2d.array.v4i16.clamp
  306248, // llvm.nvvm.sust.b.2d.array.v4i16.trap
  306285, // llvm.nvvm.sust.b.2d.array.v4i16.zero
  306322, // llvm.nvvm.sust.b.2d.array.v4i32.clamp
  306360, // llvm.nvvm.sust.b.2d.array.v4i32.trap
  306397, // llvm.nvvm.sust.b.2d.array.v4i32.zero
  306434, // llvm.nvvm.sust.b.2d.array.v4i8.clamp
  306471, // llvm.nvvm.sust.b.2d.array.v4i8.trap
  306507, // llvm.nvvm.sust.b.2d.array.v4i8.zero
  306543, // llvm.nvvm.sust.b.2d.i16.clamp
  306573, // llvm.nvvm.sust.b.2d.i16.trap
  306602, // llvm.nvvm.sust.b.2d.i16.zero
  306631, // llvm.nvvm.sust.b.2d.i32.clamp
  306661, // llvm.nvvm.sust.b.2d.i32.trap
  306690, // llvm.nvvm.sust.b.2d.i32.zero
  306719, // llvm.nvvm.sust.b.2d.i64.clamp
  306749, // llvm.nvvm.sust.b.2d.i64.trap
  306778, // llvm.nvvm.sust.b.2d.i64.zero
  306807, // llvm.nvvm.sust.b.2d.i8.clamp
  306836, // llvm.nvvm.sust.b.2d.i8.trap
  306864, // llvm.nvvm.sust.b.2d.i8.zero
  306892, // llvm.nvvm.sust.b.2d.v2i16.clamp
  306924, // llvm.nvvm.sust.b.2d.v2i16.trap
  306955, // llvm.nvvm.sust.b.2d.v2i16.zero
  306986, // llvm.nvvm.sust.b.2d.v2i32.clamp
  307018, // llvm.nvvm.sust.b.2d.v2i32.trap
  307049, // llvm.nvvm.sust.b.2d.v2i32.zero
  307080, // llvm.nvvm.sust.b.2d.v2i64.clamp
  307112, // llvm.nvvm.sust.b.2d.v2i64.trap
  307143, // llvm.nvvm.sust.b.2d.v2i64.zero
  307174, // llvm.nvvm.sust.b.2d.v2i8.clamp
  307205, // llvm.nvvm.sust.b.2d.v2i8.trap
  307235, // llvm.nvvm.sust.b.2d.v2i8.zero
  307265, // llvm.nvvm.sust.b.2d.v4i16.clamp
  307297, // llvm.nvvm.sust.b.2d.v4i16.trap
  307328, // llvm.nvvm.sust.b.2d.v4i16.zero
  307359, // llvm.nvvm.sust.b.2d.v4i32.clamp
  307391, // llvm.nvvm.sust.b.2d.v4i32.trap
  307422, // llvm.nvvm.sust.b.2d.v4i32.zero
  307453, // llvm.nvvm.sust.b.2d.v4i8.clamp
  307484, // llvm.nvvm.sust.b.2d.v4i8.trap
  307514, // llvm.nvvm.sust.b.2d.v4i8.zero
  307544, // llvm.nvvm.sust.b.3d.i16.clamp
  307574, // llvm.nvvm.sust.b.3d.i16.trap
  307603, // llvm.nvvm.sust.b.3d.i16.zero
  307632, // llvm.nvvm.sust.b.3d.i32.clamp
  307662, // llvm.nvvm.sust.b.3d.i32.trap
  307691, // llvm.nvvm.sust.b.3d.i32.zero
  307720, // llvm.nvvm.sust.b.3d.i64.clamp
  307750, // llvm.nvvm.sust.b.3d.i64.trap
  307779, // llvm.nvvm.sust.b.3d.i64.zero
  307808, // llvm.nvvm.sust.b.3d.i8.clamp
  307837, // llvm.nvvm.sust.b.3d.i8.trap
  307865, // llvm.nvvm.sust.b.3d.i8.zero
  307893, // llvm.nvvm.sust.b.3d.v2i16.clamp
  307925, // llvm.nvvm.sust.b.3d.v2i16.trap
  307956, // llvm.nvvm.sust.b.3d.v2i16.zero
  307987, // llvm.nvvm.sust.b.3d.v2i32.clamp
  308019, // llvm.nvvm.sust.b.3d.v2i32.trap
  308050, // llvm.nvvm.sust.b.3d.v2i32.zero
  308081, // llvm.nvvm.sust.b.3d.v2i64.clamp
  308113, // llvm.nvvm.sust.b.3d.v2i64.trap
  308144, // llvm.nvvm.sust.b.3d.v2i64.zero
  308175, // llvm.nvvm.sust.b.3d.v2i8.clamp
  308206, // llvm.nvvm.sust.b.3d.v2i8.trap
  308236, // llvm.nvvm.sust.b.3d.v2i8.zero
  308266, // llvm.nvvm.sust.b.3d.v4i16.clamp
  308298, // llvm.nvvm.sust.b.3d.v4i16.trap
  308329, // llvm.nvvm.sust.b.3d.v4i16.zero
  308360, // llvm.nvvm.sust.b.3d.v4i32.clamp
  308392, // llvm.nvvm.sust.b.3d.v4i32.trap
  308423, // llvm.nvvm.sust.b.3d.v4i32.zero
  308454, // llvm.nvvm.sust.b.3d.v4i8.clamp
  308485, // llvm.nvvm.sust.b.3d.v4i8.trap
  308515, // llvm.nvvm.sust.b.3d.v4i8.zero
  308545, // llvm.nvvm.sust.p.1d.array.i16.trap
  308580, // llvm.nvvm.sust.p.1d.array.i32.trap
  308615, // llvm.nvvm.sust.p.1d.array.i8.trap
  308649, // llvm.nvvm.sust.p.1d.array.v2i16.trap
  308686, // llvm.nvvm.sust.p.1d.array.v2i32.trap
  308723, // llvm.nvvm.sust.p.1d.array.v2i8.trap
  308759, // llvm.nvvm.sust.p.1d.array.v4i16.trap
  308796, // llvm.nvvm.sust.p.1d.array.v4i32.trap
  308833, // llvm.nvvm.sust.p.1d.array.v4i8.trap
  308869, // llvm.nvvm.sust.p.1d.i16.trap
  308898, // llvm.nvvm.sust.p.1d.i32.trap
  308927, // llvm.nvvm.sust.p.1d.i8.trap
  308955, // llvm.nvvm.sust.p.1d.v2i16.trap
  308986, // llvm.nvvm.sust.p.1d.v2i32.trap
  309017, // llvm.nvvm.sust.p.1d.v2i8.trap
  309047, // llvm.nvvm.sust.p.1d.v4i16.trap
  309078, // llvm.nvvm.sust.p.1d.v4i32.trap
  309109, // llvm.nvvm.sust.p.1d.v4i8.trap
  309139, // llvm.nvvm.sust.p.2d.array.i16.trap
  309174, // llvm.nvvm.sust.p.2d.array.i32.trap
  309209, // llvm.nvvm.sust.p.2d.array.i8.trap
  309243, // llvm.nvvm.sust.p.2d.array.v2i16.trap
  309280, // llvm.nvvm.sust.p.2d.array.v2i32.trap
  309317, // llvm.nvvm.sust.p.2d.array.v2i8.trap
  309353, // llvm.nvvm.sust.p.2d.array.v4i16.trap
  309390, // llvm.nvvm.sust.p.2d.array.v4i32.trap
  309427, // llvm.nvvm.sust.p.2d.array.v4i8.trap
  309463, // llvm.nvvm.sust.p.2d.i16.trap
  309492, // llvm.nvvm.sust.p.2d.i32.trap
  309521, // llvm.nvvm.sust.p.2d.i8.trap
  309549, // llvm.nvvm.sust.p.2d.v2i16.trap
  309580, // llvm.nvvm.sust.p.2d.v2i32.trap
  309611, // llvm.nvvm.sust.p.2d.v2i8.trap
  309641, // llvm.nvvm.sust.p.2d.v4i16.trap
  309672, // llvm.nvvm.sust.p.2d.v4i32.trap
  309703, // llvm.nvvm.sust.p.2d.v4i8.trap
  309733, // llvm.nvvm.sust.p.3d.i16.trap
  309762, // llvm.nvvm.sust.p.3d.i32.trap
  309791, // llvm.nvvm.sust.p.3d.i8.trap
  309819, // llvm.nvvm.sust.p.3d.v2i16.trap
  309850, // llvm.nvvm.sust.p.3d.v2i32.trap
  309881, // llvm.nvvm.sust.p.3d.v2i8.trap
  309911, // llvm.nvvm.sust.p.3d.v4i16.trap
  309942, // llvm.nvvm.sust.p.3d.v4i32.trap
  309973, // llvm.nvvm.sust.p.3d.v4i8.trap
  310003, // llvm.nvvm.tcgen05.alloc.cg1
  310031, // llvm.nvvm.tcgen05.alloc.cg2
  310059, // llvm.nvvm.tcgen05.alloc.shared.cg1
  310094, // llvm.nvvm.tcgen05.alloc.shared.cg2
  310129, // llvm.nvvm.tcgen05.commit.cg1
  310158, // llvm.nvvm.tcgen05.commit.cg2
  310187, // llvm.nvvm.tcgen05.commit.mc.cg1
  310219, // llvm.nvvm.tcgen05.commit.mc.cg2
  310251, // llvm.nvvm.tcgen05.commit.mc.shared.cg1
  310290, // llvm.nvvm.tcgen05.commit.mc.shared.cg2
  310329, // llvm.nvvm.tcgen05.commit.shared.cg1
  310365, // llvm.nvvm.tcgen05.commit.shared.cg2
  310401, // llvm.nvvm.tcgen05.cp.128x128b.b4x16_p64.cg1
  310445, // llvm.nvvm.tcgen05.cp.128x128b.b4x16_p64.cg2
  310489, // llvm.nvvm.tcgen05.cp.128x128b.b6x16_p32.cg1
  310533, // llvm.nvvm.tcgen05.cp.128x128b.b6x16_p32.cg2
  310577, // llvm.nvvm.tcgen05.cp.128x128b.cg1
  310611, // llvm.nvvm.tcgen05.cp.128x128b.cg2
  310645, // llvm.nvvm.tcgen05.cp.128x256b.b4x16_p64.cg1
  310689, // llvm.nvvm.tcgen05.cp.128x256b.b4x16_p64.cg2
  310733, // llvm.nvvm.tcgen05.cp.128x256b.b6x16_p32.cg1
  310777, // llvm.nvvm.tcgen05.cp.128x256b.b6x16_p32.cg2
  310821, // llvm.nvvm.tcgen05.cp.128x256b.cg1
  310855, // llvm.nvvm.tcgen05.cp.128x256b.cg2
  310889, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.b4x16_p64.cg1
  310939, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.b4x16_p64.cg2
  310989, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.b6x16_p32.cg1
  311039, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.b6x16_p32.cg2
  311089, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.cg1
  311129, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.cg2
  311169, // llvm.nvvm.tcgen05.cp.4x256b.b4x16_p64.cg1
  311211, // llvm.nvvm.tcgen05.cp.4x256b.b4x16_p64.cg2
  311253, // llvm.nvvm.tcgen05.cp.4x256b.b6x16_p32.cg1
  311295, // llvm.nvvm.tcgen05.cp.4x256b.b6x16_p32.cg2
  311337, // llvm.nvvm.tcgen05.cp.4x256b.cg1
  311369, // llvm.nvvm.tcgen05.cp.4x256b.cg2
  311401, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b4x16_p64.cg1
  311457, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b4x16_p64.cg2
  311513, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b6x16_p32.cg1
  311569, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b6x16_p32.cg2
  311625, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.cg1
  311671, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.cg2
  311717, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b4x16_p64.cg1
  311773, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b4x16_p64.cg2
  311829, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b6x16_p32.cg1
  311885, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b6x16_p32.cg2
  311941, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.cg1
  311987, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.cg2
  312033, // llvm.nvvm.tcgen05.dealloc.cg1
  312063, // llvm.nvvm.tcgen05.dealloc.cg2
  312093, // llvm.nvvm.tcgen05.fence.after.thread.sync
  312135, // llvm.nvvm.tcgen05.fence.before.thread.sync
  312178, // llvm.nvvm.tcgen05.ld.16x128b.x1
  312210, // llvm.nvvm.tcgen05.ld.16x128b.x16
  312243, // llvm.nvvm.tcgen05.ld.16x128b.x2
  312275, // llvm.nvvm.tcgen05.ld.16x128b.x32
  312308, // llvm.nvvm.tcgen05.ld.16x128b.x4
  312340, // llvm.nvvm.tcgen05.ld.16x128b.x64
  312373, // llvm.nvvm.tcgen05.ld.16x128b.x8
  312405, // llvm.nvvm.tcgen05.ld.16x256b.x1
  312437, // llvm.nvvm.tcgen05.ld.16x256b.x16
  312470, // llvm.nvvm.tcgen05.ld.16x256b.x2
  312502, // llvm.nvvm.tcgen05.ld.16x256b.x32
  312535, // llvm.nvvm.tcgen05.ld.16x256b.x4
  312567, // llvm.nvvm.tcgen05.ld.16x256b.x8
  312599, // llvm.nvvm.tcgen05.ld.16x32bx2.x1
  312632, // llvm.nvvm.tcgen05.ld.16x32bx2.x128
  312667, // llvm.nvvm.tcgen05.ld.16x32bx2.x16
  312701, // llvm.nvvm.tcgen05.ld.16x32bx2.x2
  312734, // llvm.nvvm.tcgen05.ld.16x32bx2.x32
  312768, // llvm.nvvm.tcgen05.ld.16x32bx2.x4
  312801, // llvm.nvvm.tcgen05.ld.16x32bx2.x64
  312835, // llvm.nvvm.tcgen05.ld.16x32bx2.x8
  312868, // llvm.nvvm.tcgen05.ld.16x64b.x1
  312899, // llvm.nvvm.tcgen05.ld.16x64b.x128
  312932, // llvm.nvvm.tcgen05.ld.16x64b.x16
  312964, // llvm.nvvm.tcgen05.ld.16x64b.x2
  312995, // llvm.nvvm.tcgen05.ld.16x64b.x32
  313027, // llvm.nvvm.tcgen05.ld.16x64b.x4
  313058, // llvm.nvvm.tcgen05.ld.16x64b.x64
  313090, // llvm.nvvm.tcgen05.ld.16x64b.x8
  313121, // llvm.nvvm.tcgen05.ld.32x32b.x1
  313152, // llvm.nvvm.tcgen05.ld.32x32b.x128
  313185, // llvm.nvvm.tcgen05.ld.32x32b.x16
  313217, // llvm.nvvm.tcgen05.ld.32x32b.x2
  313248, // llvm.nvvm.tcgen05.ld.32x32b.x32
  313280, // llvm.nvvm.tcgen05.ld.32x32b.x4
  313311, // llvm.nvvm.tcgen05.ld.32x32b.x64
  313343, // llvm.nvvm.tcgen05.ld.32x32b.x8
  313374, // llvm.nvvm.tcgen05.mma.shared
  313403, // llvm.nvvm.tcgen05.mma.shared.disable_output_lane.cg1
  313456, // llvm.nvvm.tcgen05.mma.shared.disable_output_lane.cg2
  313509, // llvm.nvvm.tcgen05.mma.shared.mxf4.block_scale
  313555, // llvm.nvvm.tcgen05.mma.shared.mxf4.block_scale.block32
  313609, // llvm.nvvm.tcgen05.mma.shared.mxf4nvf4.block_scale.block16
  313667, // llvm.nvvm.tcgen05.mma.shared.mxf4nvf4.block_scale.block32
  313725, // llvm.nvvm.tcgen05.mma.shared.mxf8f6f4.block_scale
  313775, // llvm.nvvm.tcgen05.mma.shared.mxf8f6f4.block_scale.block32
  313833, // llvm.nvvm.tcgen05.mma.shared.scale_d
  313870, // llvm.nvvm.tcgen05.mma.shared.scale_d.disable_output_lane.cg1
  313931, // llvm.nvvm.tcgen05.mma.shared.scale_d.disable_output_lane.cg2
  313992, // llvm.nvvm.tcgen05.mma.sp.shared
  314024, // llvm.nvvm.tcgen05.mma.sp.shared.disable_output_lane.cg1
  314080, // llvm.nvvm.tcgen05.mma.sp.shared.disable_output_lane.cg2
  314136, // llvm.nvvm.tcgen05.mma.sp.shared.mxf4.block_scale
  314185, // llvm.nvvm.tcgen05.mma.sp.shared.mxf4.block_scale.block32
  314242, // llvm.nvvm.tcgen05.mma.sp.shared.mxf4nvf4.block_scale.block16
  314303, // llvm.nvvm.tcgen05.mma.sp.shared.mxf4nvf4.block_scale.block32
  314364, // llvm.nvvm.tcgen05.mma.sp.shared.mxf8f6f4.block_scale
  314417, // llvm.nvvm.tcgen05.mma.sp.shared.mxf8f6f4.block_scale.block32
  314478, // llvm.nvvm.tcgen05.mma.sp.shared.scale_d
  314518, // llvm.nvvm.tcgen05.mma.sp.shared.scale_d.disable_output_lane.cg1
  314582, // llvm.nvvm.tcgen05.mma.sp.shared.scale_d.disable_output_lane.cg2
  314646, // llvm.nvvm.tcgen05.mma.sp.tensor
  314678, // llvm.nvvm.tcgen05.mma.sp.tensor.ashift
  314717, // llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg1
  314773, // llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg1.ashift
  314836, // llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg2
  314892, // llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg2.ashift
  314955, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf4.block_scale
  315004, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf4.block_scale.block32
  315061, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf4nvf4.block_scale.block16
  315122, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf4nvf4.block_scale.block32
  315183, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf8f6f4.block_scale
  315236, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf8f6f4.block_scale.block32
  315297, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d
  315337, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.ashift
  315384, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg1
  315448, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg1.ashift
  315519, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg2
  315583, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg2.ashift
  315654, // llvm.nvvm.tcgen05.mma.tensor
  315683, // llvm.nvvm.tcgen05.mma.tensor.ashift
  315719, // llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg1
  315772, // llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg1.ashift
  315832, // llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg2
  315885, // llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg2.ashift
  315945, // llvm.nvvm.tcgen05.mma.tensor.mxf4.block_scale
  315991, // llvm.nvvm.tcgen05.mma.tensor.mxf4.block_scale.block32
  316045, // llvm.nvvm.tcgen05.mma.tensor.mxf4nvf4.block_scale.block16
  316103, // llvm.nvvm.tcgen05.mma.tensor.mxf4nvf4.block_scale.block32
  316161, // llvm.nvvm.tcgen05.mma.tensor.mxf8f6f4.block_scale
  316211, // llvm.nvvm.tcgen05.mma.tensor.mxf8f6f4.block_scale.block32
  316269, // llvm.nvvm.tcgen05.mma.tensor.scale_d
  316306, // llvm.nvvm.tcgen05.mma.tensor.scale_d.ashift
  316350, // llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg1
  316411, // llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg1.ashift
  316479, // llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg2
  316540, // llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg2.ashift
  316608, // llvm.nvvm.tcgen05.mma.ws.shared
  316640, // llvm.nvvm.tcgen05.mma.ws.shared.zero_col_mask
  316686, // llvm.nvvm.tcgen05.mma.ws.sp.shared
  316721, // llvm.nvvm.tcgen05.mma.ws.sp.shared.zero_col_mask
  316770, // llvm.nvvm.tcgen05.mma.ws.sp.tensor
  316805, // llvm.nvvm.tcgen05.mma.ws.sp.tensor.zero_col_mask
  316854, // llvm.nvvm.tcgen05.mma.ws.tensor
  316886, // llvm.nvvm.tcgen05.mma.ws.tensor.zero_col_mask
  316932, // llvm.nvvm.tcgen05.relinq.alloc.permit.cg1
  316974, // llvm.nvvm.tcgen05.relinq.alloc.permit.cg2
  317016, // llvm.nvvm.tcgen05.shift.down.cg1
  317049, // llvm.nvvm.tcgen05.shift.down.cg2
  317082, // llvm.nvvm.tcgen05.st.16x128b.x1
  317114, // llvm.nvvm.tcgen05.st.16x128b.x16
  317147, // llvm.nvvm.tcgen05.st.16x128b.x2
  317179, // llvm.nvvm.tcgen05.st.16x128b.x32
  317212, // llvm.nvvm.tcgen05.st.16x128b.x4
  317244, // llvm.nvvm.tcgen05.st.16x128b.x64
  317277, // llvm.nvvm.tcgen05.st.16x128b.x8
  317309, // llvm.nvvm.tcgen05.st.16x256b.x1
  317341, // llvm.nvvm.tcgen05.st.16x256b.x16
  317374, // llvm.nvvm.tcgen05.st.16x256b.x2
  317406, // llvm.nvvm.tcgen05.st.16x256b.x32
  317439, // llvm.nvvm.tcgen05.st.16x256b.x4
  317471, // llvm.nvvm.tcgen05.st.16x256b.x8
  317503, // llvm.nvvm.tcgen05.st.16x32bx2.x1
  317536, // llvm.nvvm.tcgen05.st.16x32bx2.x128
  317571, // llvm.nvvm.tcgen05.st.16x32bx2.x16
  317605, // llvm.nvvm.tcgen05.st.16x32bx2.x2
  317638, // llvm.nvvm.tcgen05.st.16x32bx2.x32
  317672, // llvm.nvvm.tcgen05.st.16x32bx2.x4
  317705, // llvm.nvvm.tcgen05.st.16x32bx2.x64
  317739, // llvm.nvvm.tcgen05.st.16x32bx2.x8
  317772, // llvm.nvvm.tcgen05.st.16x64b.x1
  317803, // llvm.nvvm.tcgen05.st.16x64b.x128
  317836, // llvm.nvvm.tcgen05.st.16x64b.x16
  317868, // llvm.nvvm.tcgen05.st.16x64b.x2
  317899, // llvm.nvvm.tcgen05.st.16x64b.x32
  317931, // llvm.nvvm.tcgen05.st.16x64b.x4
  317962, // llvm.nvvm.tcgen05.st.16x64b.x64
  317994, // llvm.nvvm.tcgen05.st.16x64b.x8
  318025, // llvm.nvvm.tcgen05.st.32x32b.x1
  318056, // llvm.nvvm.tcgen05.st.32x32b.x128
  318089, // llvm.nvvm.tcgen05.st.32x32b.x16
  318121, // llvm.nvvm.tcgen05.st.32x32b.x2
  318152, // llvm.nvvm.tcgen05.st.32x32b.x32
  318184, // llvm.nvvm.tcgen05.st.32x32b.x4
  318215, // llvm.nvvm.tcgen05.st.32x32b.x64
  318247, // llvm.nvvm.tcgen05.st.32x32b.x8
  318278, // llvm.nvvm.tcgen05.wait.ld
  318304, // llvm.nvvm.tcgen05.wait.st
  318330, // llvm.nvvm.tensormap.replace.box.dim
  318366, // llvm.nvvm.tensormap.replace.element.stride
  318409, // llvm.nvvm.tensormap.replace.elemtype
  318446, // llvm.nvvm.tensormap.replace.fill.mode
  318484, // llvm.nvvm.tensormap.replace.global.address
  318527, // llvm.nvvm.tensormap.replace.global.dim
  318566, // llvm.nvvm.tensormap.replace.global.stride
  318608, // llvm.nvvm.tensormap.replace.interleave.layout
  318654, // llvm.nvvm.tensormap.replace.rank
  318687, // llvm.nvvm.tensormap.replace.swizzle.atomicity
  318733, // llvm.nvvm.tensormap.replace.swizzle.mode
  318774, // llvm.nvvm.tex.1d.array.grad.v4f32.f32
  318812, // llvm.nvvm.tex.1d.array.grad.v4s32.f32
  318850, // llvm.nvvm.tex.1d.array.grad.v4u32.f32
  318888, // llvm.nvvm.tex.1d.array.level.v4f32.f32
  318927, // llvm.nvvm.tex.1d.array.level.v4s32.f32
  318966, // llvm.nvvm.tex.1d.array.level.v4u32.f32
  319005, // llvm.nvvm.tex.1d.array.v4f32.f32
  319038, // llvm.nvvm.tex.1d.array.v4f32.s32
  319071, // llvm.nvvm.tex.1d.array.v4s32.f32
  319104, // llvm.nvvm.tex.1d.array.v4s32.s32
  319137, // llvm.nvvm.tex.1d.array.v4u32.f32
  319170, // llvm.nvvm.tex.1d.array.v4u32.s32
  319203, // llvm.nvvm.tex.1d.grad.v4f32.f32
  319235, // llvm.nvvm.tex.1d.grad.v4s32.f32
  319267, // llvm.nvvm.tex.1d.grad.v4u32.f32
  319299, // llvm.nvvm.tex.1d.level.v4f32.f32
  319332, // llvm.nvvm.tex.1d.level.v4s32.f32
  319365, // llvm.nvvm.tex.1d.level.v4u32.f32
  319398, // llvm.nvvm.tex.1d.v4f32.f32
  319425, // llvm.nvvm.tex.1d.v4f32.s32
  319452, // llvm.nvvm.tex.1d.v4s32.f32
  319479, // llvm.nvvm.tex.1d.v4s32.s32
  319506, // llvm.nvvm.tex.1d.v4u32.f32
  319533, // llvm.nvvm.tex.1d.v4u32.s32
  319560, // llvm.nvvm.tex.2d.array.grad.v4f32.f32
  319598, // llvm.nvvm.tex.2d.array.grad.v4s32.f32
  319636, // llvm.nvvm.tex.2d.array.grad.v4u32.f32
  319674, // llvm.nvvm.tex.2d.array.level.v4f32.f32
  319713, // llvm.nvvm.tex.2d.array.level.v4s32.f32
  319752, // llvm.nvvm.tex.2d.array.level.v4u32.f32
  319791, // llvm.nvvm.tex.2d.array.v4f32.f32
  319824, // llvm.nvvm.tex.2d.array.v4f32.s32
  319857, // llvm.nvvm.tex.2d.array.v4s32.f32
  319890, // llvm.nvvm.tex.2d.array.v4s32.s32
  319923, // llvm.nvvm.tex.2d.array.v4u32.f32
  319956, // llvm.nvvm.tex.2d.array.v4u32.s32
  319989, // llvm.nvvm.tex.2d.grad.v4f32.f32
  320021, // llvm.nvvm.tex.2d.grad.v4s32.f32
  320053, // llvm.nvvm.tex.2d.grad.v4u32.f32
  320085, // llvm.nvvm.tex.2d.level.v4f32.f32
  320118, // llvm.nvvm.tex.2d.level.v4s32.f32
  320151, // llvm.nvvm.tex.2d.level.v4u32.f32
  320184, // llvm.nvvm.tex.2d.v4f32.f32
  320211, // llvm.nvvm.tex.2d.v4f32.s32
  320238, // llvm.nvvm.tex.2d.v4s32.f32
  320265, // llvm.nvvm.tex.2d.v4s32.s32
  320292, // llvm.nvvm.tex.2d.v4u32.f32
  320319, // llvm.nvvm.tex.2d.v4u32.s32
  320346, // llvm.nvvm.tex.3d.grad.v4f32.f32
  320378, // llvm.nvvm.tex.3d.grad.v4s32.f32
  320410, // llvm.nvvm.tex.3d.grad.v4u32.f32
  320442, // llvm.nvvm.tex.3d.level.v4f32.f32
  320475, // llvm.nvvm.tex.3d.level.v4s32.f32
  320508, // llvm.nvvm.tex.3d.level.v4u32.f32
  320541, // llvm.nvvm.tex.3d.v4f32.f32
  320568, // llvm.nvvm.tex.3d.v4f32.s32
  320595, // llvm.nvvm.tex.3d.v4s32.f32
  320622, // llvm.nvvm.tex.3d.v4s32.s32
  320649, // llvm.nvvm.tex.3d.v4u32.f32
  320676, // llvm.nvvm.tex.3d.v4u32.s32
  320703, // llvm.nvvm.tex.cube.array.level.v4f32.f32
  320744, // llvm.nvvm.tex.cube.array.level.v4s32.f32
  320785, // llvm.nvvm.tex.cube.array.level.v4u32.f32
  320826, // llvm.nvvm.tex.cube.array.v4f32.f32
  320861, // llvm.nvvm.tex.cube.array.v4s32.f32
  320896, // llvm.nvvm.tex.cube.array.v4u32.f32
  320931, // llvm.nvvm.tex.cube.level.v4f32.f32
  320966, // llvm.nvvm.tex.cube.level.v4s32.f32
  321001, // llvm.nvvm.tex.cube.level.v4u32.f32
  321036, // llvm.nvvm.tex.cube.v4f32.f32
  321065, // llvm.nvvm.tex.cube.v4s32.f32
  321094, // llvm.nvvm.tex.cube.v4u32.f32
  321123, // llvm.nvvm.tex.unified.1d.array.grad.v4f32.f32
  321169, // llvm.nvvm.tex.unified.1d.array.grad.v4s32.f32
  321215, // llvm.nvvm.tex.unified.1d.array.grad.v4u32.f32
  321261, // llvm.nvvm.tex.unified.1d.array.level.v4f32.f32
  321308, // llvm.nvvm.tex.unified.1d.array.level.v4s32.f32
  321355, // llvm.nvvm.tex.unified.1d.array.level.v4u32.f32
  321402, // llvm.nvvm.tex.unified.1d.array.v4f32.f32
  321443, // llvm.nvvm.tex.unified.1d.array.v4f32.s32
  321484, // llvm.nvvm.tex.unified.1d.array.v4s32.f32
  321525, // llvm.nvvm.tex.unified.1d.array.v4s32.s32
  321566, // llvm.nvvm.tex.unified.1d.array.v4u32.f32
  321607, // llvm.nvvm.tex.unified.1d.array.v4u32.s32
  321648, // llvm.nvvm.tex.unified.1d.grad.v4f32.f32
  321688, // llvm.nvvm.tex.unified.1d.grad.v4s32.f32
  321728, // llvm.nvvm.tex.unified.1d.grad.v4u32.f32
  321768, // llvm.nvvm.tex.unified.1d.level.v4f32.f32
  321809, // llvm.nvvm.tex.unified.1d.level.v4s32.f32
  321850, // llvm.nvvm.tex.unified.1d.level.v4u32.f32
  321891, // llvm.nvvm.tex.unified.1d.v4f32.f32
  321926, // llvm.nvvm.tex.unified.1d.v4f32.s32
  321961, // llvm.nvvm.tex.unified.1d.v4s32.f32
  321996, // llvm.nvvm.tex.unified.1d.v4s32.s32
  322031, // llvm.nvvm.tex.unified.1d.v4u32.f32
  322066, // llvm.nvvm.tex.unified.1d.v4u32.s32
  322101, // llvm.nvvm.tex.unified.2d.array.grad.v4f32.f32
  322147, // llvm.nvvm.tex.unified.2d.array.grad.v4s32.f32
  322193, // llvm.nvvm.tex.unified.2d.array.grad.v4u32.f32
  322239, // llvm.nvvm.tex.unified.2d.array.level.v4f32.f32
  322286, // llvm.nvvm.tex.unified.2d.array.level.v4s32.f32
  322333, // llvm.nvvm.tex.unified.2d.array.level.v4u32.f32
  322380, // llvm.nvvm.tex.unified.2d.array.v4f32.f32
  322421, // llvm.nvvm.tex.unified.2d.array.v4f32.s32
  322462, // llvm.nvvm.tex.unified.2d.array.v4s32.f32
  322503, // llvm.nvvm.tex.unified.2d.array.v4s32.s32
  322544, // llvm.nvvm.tex.unified.2d.array.v4u32.f32
  322585, // llvm.nvvm.tex.unified.2d.array.v4u32.s32
  322626, // llvm.nvvm.tex.unified.2d.grad.v4f32.f32
  322666, // llvm.nvvm.tex.unified.2d.grad.v4s32.f32
  322706, // llvm.nvvm.tex.unified.2d.grad.v4u32.f32
  322746, // llvm.nvvm.tex.unified.2d.level.v4f32.f32
  322787, // llvm.nvvm.tex.unified.2d.level.v4s32.f32
  322828, // llvm.nvvm.tex.unified.2d.level.v4u32.f32
  322869, // llvm.nvvm.tex.unified.2d.v4f32.f32
  322904, // llvm.nvvm.tex.unified.2d.v4f32.s32
  322939, // llvm.nvvm.tex.unified.2d.v4s32.f32
  322974, // llvm.nvvm.tex.unified.2d.v4s32.s32
  323009, // llvm.nvvm.tex.unified.2d.v4u32.f32
  323044, // llvm.nvvm.tex.unified.2d.v4u32.s32
  323079, // llvm.nvvm.tex.unified.3d.grad.v4f32.f32
  323119, // llvm.nvvm.tex.unified.3d.grad.v4s32.f32
  323159, // llvm.nvvm.tex.unified.3d.grad.v4u32.f32
  323199, // llvm.nvvm.tex.unified.3d.level.v4f32.f32
  323240, // llvm.nvvm.tex.unified.3d.level.v4s32.f32
  323281, // llvm.nvvm.tex.unified.3d.level.v4u32.f32
  323322, // llvm.nvvm.tex.unified.3d.v4f32.f32
  323357, // llvm.nvvm.tex.unified.3d.v4f32.s32
  323392, // llvm.nvvm.tex.unified.3d.v4s32.f32
  323427, // llvm.nvvm.tex.unified.3d.v4s32.s32
  323462, // llvm.nvvm.tex.unified.3d.v4u32.f32
  323497, // llvm.nvvm.tex.unified.3d.v4u32.s32
  323532, // llvm.nvvm.tex.unified.cube.array.grad.v4f32.f32
  323580, // llvm.nvvm.tex.unified.cube.array.grad.v4s32.f32
  323628, // llvm.nvvm.tex.unified.cube.array.grad.v4u32.f32
  323676, // llvm.nvvm.tex.unified.cube.array.level.v4f32.f32
  323725, // llvm.nvvm.tex.unified.cube.array.level.v4s32.f32
  323774, // llvm.nvvm.tex.unified.cube.array.level.v4u32.f32
  323823, // llvm.nvvm.tex.unified.cube.array.v4f32.f32
  323866, // llvm.nvvm.tex.unified.cube.array.v4s32.f32
  323909, // llvm.nvvm.tex.unified.cube.array.v4u32.f32
  323952, // llvm.nvvm.tex.unified.cube.grad.v4f32.f32
  323994, // llvm.nvvm.tex.unified.cube.grad.v4s32.f32
  324036, // llvm.nvvm.tex.unified.cube.grad.v4u32.f32
  324078, // llvm.nvvm.tex.unified.cube.level.v4f32.f32
  324121, // llvm.nvvm.tex.unified.cube.level.v4s32.f32
  324164, // llvm.nvvm.tex.unified.cube.level.v4u32.f32
  324207, // llvm.nvvm.tex.unified.cube.v4f32.f32
  324244, // llvm.nvvm.tex.unified.cube.v4s32.f32
  324281, // llvm.nvvm.tex.unified.cube.v4u32.f32
  324318, // llvm.nvvm.texsurf.handle
  324343, // llvm.nvvm.texsurf.handle.internal
  324377, // llvm.nvvm.tld4.a.2d.v4f32.f32
  324407, // llvm.nvvm.tld4.a.2d.v4s32.f32
  324437, // llvm.nvvm.tld4.a.2d.v4u32.f32
  324467, // llvm.nvvm.tld4.b.2d.v4f32.f32
  324497, // llvm.nvvm.tld4.b.2d.v4s32.f32
  324527, // llvm.nvvm.tld4.b.2d.v4u32.f32
  324557, // llvm.nvvm.tld4.g.2d.v4f32.f32
  324587, // llvm.nvvm.tld4.g.2d.v4s32.f32
  324617, // llvm.nvvm.tld4.g.2d.v4u32.f32
  324647, // llvm.nvvm.tld4.r.2d.v4f32.f32
  324677, // llvm.nvvm.tld4.r.2d.v4s32.f32
  324707, // llvm.nvvm.tld4.r.2d.v4u32.f32
  324737, // llvm.nvvm.tld4.unified.a.2d.v4f32.f32
  324775, // llvm.nvvm.tld4.unified.a.2d.v4s32.f32
  324813, // llvm.nvvm.tld4.unified.a.2d.v4u32.f32
  324851, // llvm.nvvm.tld4.unified.b.2d.v4f32.f32
  324889, // llvm.nvvm.tld4.unified.b.2d.v4s32.f32
  324927, // llvm.nvvm.tld4.unified.b.2d.v4u32.f32
  324965, // llvm.nvvm.tld4.unified.g.2d.v4f32.f32
  325003, // llvm.nvvm.tld4.unified.g.2d.v4s32.f32
  325041, // llvm.nvvm.tld4.unified.g.2d.v4u32.f32
  325079, // llvm.nvvm.tld4.unified.r.2d.v4f32.f32
  325117, // llvm.nvvm.tld4.unified.r.2d.v4s32.f32
  325155, // llvm.nvvm.tld4.unified.r.2d.v4u32.f32
  325193, // llvm.nvvm.trunc.d
  325211, // llvm.nvvm.trunc.f
  325229, // llvm.nvvm.trunc.ftz.f
  325251, // llvm.nvvm.txq.array.size
  325276, // llvm.nvvm.txq.channel.data.type
  325308, // llvm.nvvm.txq.channel.order
  325336, // llvm.nvvm.txq.depth
  325356, // llvm.nvvm.txq.height
  325377, // llvm.nvvm.txq.num.mipmap.levels
  325409, // llvm.nvvm.txq.num.samples
  325435, // llvm.nvvm.txq.width
  325455, // llvm.nvvm.ue8m0x2.to.bf16x2
  325483, // llvm.nvvm.ui2d.rm
  325501, // llvm.nvvm.ui2d.rn
  325519, // llvm.nvvm.ui2d.rp
  325537, // llvm.nvvm.ui2d.rz
  325555, // llvm.nvvm.ui2f.rm
  325573, // llvm.nvvm.ui2f.rn
  325591, // llvm.nvvm.ui2f.rp
  325609, // llvm.nvvm.ui2f.rz
  325627, // llvm.nvvm.ull2d.rm
  325646, // llvm.nvvm.ull2d.rn
  325665, // llvm.nvvm.ull2d.rp
  325684, // llvm.nvvm.ull2d.rz
  325703, // llvm.nvvm.ull2f.rm
  325722, // llvm.nvvm.ull2f.rn
  325741, // llvm.nvvm.ull2f.rp
  325760, // llvm.nvvm.ull2f.rz
  325779, // llvm.nvvm.vote.all
  325798, // llvm.nvvm.vote.all.sync
  325822, // llvm.nvvm.vote.any
  325841, // llvm.nvvm.vote.any.sync
  325865, // llvm.nvvm.vote.ballot
  325887, // llvm.nvvm.vote.ballot.sync
  325914, // llvm.nvvm.vote.uni
  325933, // llvm.nvvm.vote.uni.sync
  325957, // llvm.nvvm.wgmma.commit_group.sync.aligned
  325999, // llvm.nvvm.wgmma.fence.sync.aligned
  326034, // llvm.nvvm.wgmma.wait_group.sync.aligned
  326074, // llvm.nvvm.wmma.m16n16k16.load.a.col.bf16
  326115, // llvm.nvvm.wmma.m16n16k16.load.a.col.f16
  326155, // llvm.nvvm.wmma.m16n16k16.load.a.col.s8
  326194, // llvm.nvvm.wmma.m16n16k16.load.a.col.stride.bf16
  326242, // llvm.nvvm.wmma.m16n16k16.load.a.col.stride.f16
  326289, // llvm.nvvm.wmma.m16n16k16.load.a.col.stride.s8
  326335, // llvm.nvvm.wmma.m16n16k16.load.a.col.stride.u8
  326381, // llvm.nvvm.wmma.m16n16k16.load.a.col.u8
  326420, // llvm.nvvm.wmma.m16n16k16.load.a.row.bf16
  326461, // llvm.nvvm.wmma.m16n16k16.load.a.row.f16
  326501, // llvm.nvvm.wmma.m16n16k16.load.a.row.s8
  326540, // llvm.nvvm.wmma.m16n16k16.load.a.row.stride.bf16
  326588, // llvm.nvvm.wmma.m16n16k16.load.a.row.stride.f16
  326635, // llvm.nvvm.wmma.m16n16k16.load.a.row.stride.s8
  326681, // llvm.nvvm.wmma.m16n16k16.load.a.row.stride.u8
  326727, // llvm.nvvm.wmma.m16n16k16.load.a.row.u8
  326766, // llvm.nvvm.wmma.m16n16k16.load.b.col.bf16
  326807, // llvm.nvvm.wmma.m16n16k16.load.b.col.f16
  326847, // llvm.nvvm.wmma.m16n16k16.load.b.col.s8
  326886, // llvm.nvvm.wmma.m16n16k16.load.b.col.stride.bf16
  326934, // llvm.nvvm.wmma.m16n16k16.load.b.col.stride.f16
  326981, // llvm.nvvm.wmma.m16n16k16.load.b.col.stride.s8
  327027, // llvm.nvvm.wmma.m16n16k16.load.b.col.stride.u8
  327073, // llvm.nvvm.wmma.m16n16k16.load.b.col.u8
  327112, // llvm.nvvm.wmma.m16n16k16.load.b.row.bf16
  327153, // llvm.nvvm.wmma.m16n16k16.load.b.row.f16
  327193, // llvm.nvvm.wmma.m16n16k16.load.b.row.s8
  327232, // llvm.nvvm.wmma.m16n16k16.load.b.row.stride.bf16
  327280, // llvm.nvvm.wmma.m16n16k16.load.b.row.stride.f16
  327327, // llvm.nvvm.wmma.m16n16k16.load.b.row.stride.s8
  327373, // llvm.nvvm.wmma.m16n16k16.load.b.row.stride.u8
  327419, // llvm.nvvm.wmma.m16n16k16.load.b.row.u8
  327458, // llvm.nvvm.wmma.m16n16k16.load.c.col.f16
  327498, // llvm.nvvm.wmma.m16n16k16.load.c.col.f32
  327538, // llvm.nvvm.wmma.m16n16k16.load.c.col.s32
  327578, // llvm.nvvm.wmma.m16n16k16.load.c.col.stride.f16
  327625, // llvm.nvvm.wmma.m16n16k16.load.c.col.stride.f32
  327672, // llvm.nvvm.wmma.m16n16k16.load.c.col.stride.s32
  327719, // llvm.nvvm.wmma.m16n16k16.load.c.row.f16
  327759, // llvm.nvvm.wmma.m16n16k16.load.c.row.f32
  327799, // llvm.nvvm.wmma.m16n16k16.load.c.row.s32
  327839, // llvm.nvvm.wmma.m16n16k16.load.c.row.stride.f16
  327886, // llvm.nvvm.wmma.m16n16k16.load.c.row.stride.f32
  327933, // llvm.nvvm.wmma.m16n16k16.load.c.row.stride.s32
  327980, // llvm.nvvm.wmma.m16n16k16.mma.col.col.bf16
  328022, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f16
  328067, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f16.satfinite
  328122, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f32
  328167, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f32.satfinite
  328222, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f16
  328267, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f16.satfinite
  328322, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f32
  328367, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f32.satfinite
  328422, // llvm.nvvm.wmma.m16n16k16.mma.col.col.s8
  328462, // llvm.nvvm.wmma.m16n16k16.mma.col.col.s8.satfinite
  328512, // llvm.nvvm.wmma.m16n16k16.mma.col.col.u8
  328552, // llvm.nvvm.wmma.m16n16k16.mma.col.col.u8.satfinite
  328602, // llvm.nvvm.wmma.m16n16k16.mma.col.row.bf16
  328644, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f16
  328689, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f16.satfinite
  328744, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f32
  328789, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f32.satfinite
  328844, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f16
  328889, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f16.satfinite
  328944, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f32
  328989, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f32.satfinite
  329044, // llvm.nvvm.wmma.m16n16k16.mma.col.row.s8
  329084, // llvm.nvvm.wmma.m16n16k16.mma.col.row.s8.satfinite
  329134, // llvm.nvvm.wmma.m16n16k16.mma.col.row.u8
  329174, // llvm.nvvm.wmma.m16n16k16.mma.col.row.u8.satfinite
  329224, // llvm.nvvm.wmma.m16n16k16.mma.row.col.bf16
  329266, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f16
  329311, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f16.satfinite
  329366, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f32
  329411, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f32.satfinite
  329466, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f16
  329511, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f16.satfinite
  329566, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f32
  329611, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f32.satfinite
  329666, // llvm.nvvm.wmma.m16n16k16.mma.row.col.s8
  329706, // llvm.nvvm.wmma.m16n16k16.mma.row.col.s8.satfinite
  329756, // llvm.nvvm.wmma.m16n16k16.mma.row.col.u8
  329796, // llvm.nvvm.wmma.m16n16k16.mma.row.col.u8.satfinite
  329846, // llvm.nvvm.wmma.m16n16k16.mma.row.row.bf16
  329888, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f16
  329933, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f16.satfinite
  329988, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f32
  330033, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f32.satfinite
  330088, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f16
  330133, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f16.satfinite
  330188, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f32
  330233, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f32.satfinite
  330288, // llvm.nvvm.wmma.m16n16k16.mma.row.row.s8
  330328, // llvm.nvvm.wmma.m16n16k16.mma.row.row.s8.satfinite
  330378, // llvm.nvvm.wmma.m16n16k16.mma.row.row.u8
  330418, // llvm.nvvm.wmma.m16n16k16.mma.row.row.u8.satfinite
  330468, // llvm.nvvm.wmma.m16n16k16.store.d.col.f16
  330509, // llvm.nvvm.wmma.m16n16k16.store.d.col.f32
  330550, // llvm.nvvm.wmma.m16n16k16.store.d.col.s32
  330591, // llvm.nvvm.wmma.m16n16k16.store.d.col.stride.f16
  330639, // llvm.nvvm.wmma.m16n16k16.store.d.col.stride.f32
  330687, // llvm.nvvm.wmma.m16n16k16.store.d.col.stride.s32
  330735, // llvm.nvvm.wmma.m16n16k16.store.d.row.f16
  330776, // llvm.nvvm.wmma.m16n16k16.store.d.row.f32
  330817, // llvm.nvvm.wmma.m16n16k16.store.d.row.s32
  330858, // llvm.nvvm.wmma.m16n16k16.store.d.row.stride.f16
  330906, // llvm.nvvm.wmma.m16n16k16.store.d.row.stride.f32
  330954, // llvm.nvvm.wmma.m16n16k16.store.d.row.stride.s32
  331002, // llvm.nvvm.wmma.m16n16k8.load.a.col.stride.tf32
  331049, // llvm.nvvm.wmma.m16n16k8.load.a.col.tf32
  331089, // llvm.nvvm.wmma.m16n16k8.load.a.row.stride.tf32
  331136, // llvm.nvvm.wmma.m16n16k8.load.a.row.tf32
  331176, // llvm.nvvm.wmma.m16n16k8.load.b.col.stride.tf32
  331223, // llvm.nvvm.wmma.m16n16k8.load.b.col.tf32
  331263, // llvm.nvvm.wmma.m16n16k8.load.b.row.stride.tf32
  331310, // llvm.nvvm.wmma.m16n16k8.load.b.row.tf32
  331350, // llvm.nvvm.wmma.m16n16k8.load.c.col.f32
  331389, // llvm.nvvm.wmma.m16n16k8.load.c.col.stride.f32
  331435, // llvm.nvvm.wmma.m16n16k8.load.c.row.f32
  331474, // llvm.nvvm.wmma.m16n16k8.load.c.row.stride.f32
  331520, // llvm.nvvm.wmma.m16n16k8.mma.col.col.tf32
  331561, // llvm.nvvm.wmma.m16n16k8.mma.col.row.tf32
  331602, // llvm.nvvm.wmma.m16n16k8.mma.row.col.tf32
  331643, // llvm.nvvm.wmma.m16n16k8.mma.row.row.tf32
  331684, // llvm.nvvm.wmma.m16n16k8.store.d.col.f32
  331724, // llvm.nvvm.wmma.m16n16k8.store.d.col.stride.f32
  331771, // llvm.nvvm.wmma.m16n16k8.store.d.row.f32
  331811, // llvm.nvvm.wmma.m16n16k8.store.d.row.stride.f32
  331858, // llvm.nvvm.wmma.m32n8k16.load.a.col.bf16
  331898, // llvm.nvvm.wmma.m32n8k16.load.a.col.f16
  331937, // llvm.nvvm.wmma.m32n8k16.load.a.col.s8
  331975, // llvm.nvvm.wmma.m32n8k16.load.a.col.stride.bf16
  332022, // llvm.nvvm.wmma.m32n8k16.load.a.col.stride.f16
  332068, // llvm.nvvm.wmma.m32n8k16.load.a.col.stride.s8
  332113, // llvm.nvvm.wmma.m32n8k16.load.a.col.stride.u8
  332158, // llvm.nvvm.wmma.m32n8k16.load.a.col.u8
  332196, // llvm.nvvm.wmma.m32n8k16.load.a.row.bf16
  332236, // llvm.nvvm.wmma.m32n8k16.load.a.row.f16
  332275, // llvm.nvvm.wmma.m32n8k16.load.a.row.s8
  332313, // llvm.nvvm.wmma.m32n8k16.load.a.row.stride.bf16
  332360, // llvm.nvvm.wmma.m32n8k16.load.a.row.stride.f16
  332406, // llvm.nvvm.wmma.m32n8k16.load.a.row.stride.s8
  332451, // llvm.nvvm.wmma.m32n8k16.load.a.row.stride.u8
  332496, // llvm.nvvm.wmma.m32n8k16.load.a.row.u8
  332534, // llvm.nvvm.wmma.m32n8k16.load.b.col.bf16
  332574, // llvm.nvvm.wmma.m32n8k16.load.b.col.f16
  332613, // llvm.nvvm.wmma.m32n8k16.load.b.col.s8
  332651, // llvm.nvvm.wmma.m32n8k16.load.b.col.stride.bf16
  332698, // llvm.nvvm.wmma.m32n8k16.load.b.col.stride.f16
  332744, // llvm.nvvm.wmma.m32n8k16.load.b.col.stride.s8
  332789, // llvm.nvvm.wmma.m32n8k16.load.b.col.stride.u8
  332834, // llvm.nvvm.wmma.m32n8k16.load.b.col.u8
  332872, // llvm.nvvm.wmma.m32n8k16.load.b.row.bf16
  332912, // llvm.nvvm.wmma.m32n8k16.load.b.row.f16
  332951, // llvm.nvvm.wmma.m32n8k16.load.b.row.s8
  332989, // llvm.nvvm.wmma.m32n8k16.load.b.row.stride.bf16
  333036, // llvm.nvvm.wmma.m32n8k16.load.b.row.stride.f16
  333082, // llvm.nvvm.wmma.m32n8k16.load.b.row.stride.s8
  333127, // llvm.nvvm.wmma.m32n8k16.load.b.row.stride.u8
  333172, // llvm.nvvm.wmma.m32n8k16.load.b.row.u8
  333210, // llvm.nvvm.wmma.m32n8k16.load.c.col.f16
  333249, // llvm.nvvm.wmma.m32n8k16.load.c.col.f32
  333288, // llvm.nvvm.wmma.m32n8k16.load.c.col.s32
  333327, // llvm.nvvm.wmma.m32n8k16.load.c.col.stride.f16
  333373, // llvm.nvvm.wmma.m32n8k16.load.c.col.stride.f32
  333419, // llvm.nvvm.wmma.m32n8k16.load.c.col.stride.s32
  333465, // llvm.nvvm.wmma.m32n8k16.load.c.row.f16
  333504, // llvm.nvvm.wmma.m32n8k16.load.c.row.f32
  333543, // llvm.nvvm.wmma.m32n8k16.load.c.row.s32
  333582, // llvm.nvvm.wmma.m32n8k16.load.c.row.stride.f16
  333628, // llvm.nvvm.wmma.m32n8k16.load.c.row.stride.f32
  333674, // llvm.nvvm.wmma.m32n8k16.load.c.row.stride.s32
  333720, // llvm.nvvm.wmma.m32n8k16.mma.col.col.bf16
  333761, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f16
  333805, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f16.satfinite
  333859, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f32
  333903, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f32.satfinite
  333957, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f16
  334001, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f16.satfinite
  334055, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f32
  334099, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f32.satfinite
  334153, // llvm.nvvm.wmma.m32n8k16.mma.col.col.s8
  334192, // llvm.nvvm.wmma.m32n8k16.mma.col.col.s8.satfinite
  334241, // llvm.nvvm.wmma.m32n8k16.mma.col.col.u8
  334280, // llvm.nvvm.wmma.m32n8k16.mma.col.col.u8.satfinite
  334329, // llvm.nvvm.wmma.m32n8k16.mma.col.row.bf16
  334370, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f16
  334414, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f16.satfinite
  334468, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f32
  334512, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f32.satfinite
  334566, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f16
  334610, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f16.satfinite
  334664, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f32
  334708, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f32.satfinite
  334762, // llvm.nvvm.wmma.m32n8k16.mma.col.row.s8
  334801, // llvm.nvvm.wmma.m32n8k16.mma.col.row.s8.satfinite
  334850, // llvm.nvvm.wmma.m32n8k16.mma.col.row.u8
  334889, // llvm.nvvm.wmma.m32n8k16.mma.col.row.u8.satfinite
  334938, // llvm.nvvm.wmma.m32n8k16.mma.row.col.bf16
  334979, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f16
  335023, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f16.satfinite
  335077, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f32
  335121, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f32.satfinite
  335175, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f16
  335219, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f16.satfinite
  335273, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f32
  335317, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f32.satfinite
  335371, // llvm.nvvm.wmma.m32n8k16.mma.row.col.s8
  335410, // llvm.nvvm.wmma.m32n8k16.mma.row.col.s8.satfinite
  335459, // llvm.nvvm.wmma.m32n8k16.mma.row.col.u8
  335498, // llvm.nvvm.wmma.m32n8k16.mma.row.col.u8.satfinite
  335547, // llvm.nvvm.wmma.m32n8k16.mma.row.row.bf16
  335588, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f16
  335632, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f16.satfinite
  335686, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f32
  335730, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f32.satfinite
  335784, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f16
  335828, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f16.satfinite
  335882, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f32
  335926, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f32.satfinite
  335980, // llvm.nvvm.wmma.m32n8k16.mma.row.row.s8
  336019, // llvm.nvvm.wmma.m32n8k16.mma.row.row.s8.satfinite
  336068, // llvm.nvvm.wmma.m32n8k16.mma.row.row.u8
  336107, // llvm.nvvm.wmma.m32n8k16.mma.row.row.u8.satfinite
  336156, // llvm.nvvm.wmma.m32n8k16.store.d.col.f16
  336196, // llvm.nvvm.wmma.m32n8k16.store.d.col.f32
  336236, // llvm.nvvm.wmma.m32n8k16.store.d.col.s32
  336276, // llvm.nvvm.wmma.m32n8k16.store.d.col.stride.f16
  336323, // llvm.nvvm.wmma.m32n8k16.store.d.col.stride.f32
  336370, // llvm.nvvm.wmma.m32n8k16.store.d.col.stride.s32
  336417, // llvm.nvvm.wmma.m32n8k16.store.d.row.f16
  336457, // llvm.nvvm.wmma.m32n8k16.store.d.row.f32
  336497, // llvm.nvvm.wmma.m32n8k16.store.d.row.s32
  336537, // llvm.nvvm.wmma.m32n8k16.store.d.row.stride.f16
  336584, // llvm.nvvm.wmma.m32n8k16.store.d.row.stride.f32
  336631, // llvm.nvvm.wmma.m32n8k16.store.d.row.stride.s32
  336678, // llvm.nvvm.wmma.m8n32k16.load.a.col.bf16
  336718, // llvm.nvvm.wmma.m8n32k16.load.a.col.f16
  336757, // llvm.nvvm.wmma.m8n32k16.load.a.col.s8
  336795, // llvm.nvvm.wmma.m8n32k16.load.a.col.stride.bf16
  336842, // llvm.nvvm.wmma.m8n32k16.load.a.col.stride.f16
  336888, // llvm.nvvm.wmma.m8n32k16.load.a.col.stride.s8
  336933, // llvm.nvvm.wmma.m8n32k16.load.a.col.stride.u8
  336978, // llvm.nvvm.wmma.m8n32k16.load.a.col.u8
  337016, // llvm.nvvm.wmma.m8n32k16.load.a.row.bf16
  337056, // llvm.nvvm.wmma.m8n32k16.load.a.row.f16
  337095, // llvm.nvvm.wmma.m8n32k16.load.a.row.s8
  337133, // llvm.nvvm.wmma.m8n32k16.load.a.row.stride.bf16
  337180, // llvm.nvvm.wmma.m8n32k16.load.a.row.stride.f16
  337226, // llvm.nvvm.wmma.m8n32k16.load.a.row.stride.s8
  337271, // llvm.nvvm.wmma.m8n32k16.load.a.row.stride.u8
  337316, // llvm.nvvm.wmma.m8n32k16.load.a.row.u8
  337354, // llvm.nvvm.wmma.m8n32k16.load.b.col.bf16
  337394, // llvm.nvvm.wmma.m8n32k16.load.b.col.f16
  337433, // llvm.nvvm.wmma.m8n32k16.load.b.col.s8
  337471, // llvm.nvvm.wmma.m8n32k16.load.b.col.stride.bf16
  337518, // llvm.nvvm.wmma.m8n32k16.load.b.col.stride.f16
  337564, // llvm.nvvm.wmma.m8n32k16.load.b.col.stride.s8
  337609, // llvm.nvvm.wmma.m8n32k16.load.b.col.stride.u8
  337654, // llvm.nvvm.wmma.m8n32k16.load.b.col.u8
  337692, // llvm.nvvm.wmma.m8n32k16.load.b.row.bf16
  337732, // llvm.nvvm.wmma.m8n32k16.load.b.row.f16
  337771, // llvm.nvvm.wmma.m8n32k16.load.b.row.s8
  337809, // llvm.nvvm.wmma.m8n32k16.load.b.row.stride.bf16
  337856, // llvm.nvvm.wmma.m8n32k16.load.b.row.stride.f16
  337902, // llvm.nvvm.wmma.m8n32k16.load.b.row.stride.s8
  337947, // llvm.nvvm.wmma.m8n32k16.load.b.row.stride.u8
  337992, // llvm.nvvm.wmma.m8n32k16.load.b.row.u8
  338030, // llvm.nvvm.wmma.m8n32k16.load.c.col.f16
  338069, // llvm.nvvm.wmma.m8n32k16.load.c.col.f32
  338108, // llvm.nvvm.wmma.m8n32k16.load.c.col.s32
  338147, // llvm.nvvm.wmma.m8n32k16.load.c.col.stride.f16
  338193, // llvm.nvvm.wmma.m8n32k16.load.c.col.stride.f32
  338239, // llvm.nvvm.wmma.m8n32k16.load.c.col.stride.s32
  338285, // llvm.nvvm.wmma.m8n32k16.load.c.row.f16
  338324, // llvm.nvvm.wmma.m8n32k16.load.c.row.f32
  338363, // llvm.nvvm.wmma.m8n32k16.load.c.row.s32
  338402, // llvm.nvvm.wmma.m8n32k16.load.c.row.stride.f16
  338448, // llvm.nvvm.wmma.m8n32k16.load.c.row.stride.f32
  338494, // llvm.nvvm.wmma.m8n32k16.load.c.row.stride.s32
  338540, // llvm.nvvm.wmma.m8n32k16.mma.col.col.bf16
  338581, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f16
  338625, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f16.satfinite
  338679, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f32
  338723, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f32.satfinite
  338777, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f16
  338821, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f16.satfinite
  338875, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f32
  338919, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f32.satfinite
  338973, // llvm.nvvm.wmma.m8n32k16.mma.col.col.s8
  339012, // llvm.nvvm.wmma.m8n32k16.mma.col.col.s8.satfinite
  339061, // llvm.nvvm.wmma.m8n32k16.mma.col.col.u8
  339100, // llvm.nvvm.wmma.m8n32k16.mma.col.col.u8.satfinite
  339149, // llvm.nvvm.wmma.m8n32k16.mma.col.row.bf16
  339190, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f16
  339234, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f16.satfinite
  339288, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f32
  339332, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f32.satfinite
  339386, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f16
  339430, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f16.satfinite
  339484, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f32
  339528, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f32.satfinite
  339582, // llvm.nvvm.wmma.m8n32k16.mma.col.row.s8
  339621, // llvm.nvvm.wmma.m8n32k16.mma.col.row.s8.satfinite
  339670, // llvm.nvvm.wmma.m8n32k16.mma.col.row.u8
  339709, // llvm.nvvm.wmma.m8n32k16.mma.col.row.u8.satfinite
  339758, // llvm.nvvm.wmma.m8n32k16.mma.row.col.bf16
  339799, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f16
  339843, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f16.satfinite
  339897, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f32
  339941, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f32.satfinite
  339995, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f16
  340039, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f16.satfinite
  340093, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f32
  340137, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f32.satfinite
  340191, // llvm.nvvm.wmma.m8n32k16.mma.row.col.s8
  340230, // llvm.nvvm.wmma.m8n32k16.mma.row.col.s8.satfinite
  340279, // llvm.nvvm.wmma.m8n32k16.mma.row.col.u8
  340318, // llvm.nvvm.wmma.m8n32k16.mma.row.col.u8.satfinite
  340367, // llvm.nvvm.wmma.m8n32k16.mma.row.row.bf16
  340408, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f16
  340452, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f16.satfinite
  340506, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f32
  340550, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f32.satfinite
  340604, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f16
  340648, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f16.satfinite
  340702, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f32
  340746, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f32.satfinite
  340800, // llvm.nvvm.wmma.m8n32k16.mma.row.row.s8
  340839, // llvm.nvvm.wmma.m8n32k16.mma.row.row.s8.satfinite
  340888, // llvm.nvvm.wmma.m8n32k16.mma.row.row.u8
  340927, // llvm.nvvm.wmma.m8n32k16.mma.row.row.u8.satfinite
  340976, // llvm.nvvm.wmma.m8n32k16.store.d.col.f16
  341016, // llvm.nvvm.wmma.m8n32k16.store.d.col.f32
  341056, // llvm.nvvm.wmma.m8n32k16.store.d.col.s32
  341096, // llvm.nvvm.wmma.m8n32k16.store.d.col.stride.f16
  341143, // llvm.nvvm.wmma.m8n32k16.store.d.col.stride.f32
  341190, // llvm.nvvm.wmma.m8n32k16.store.d.col.stride.s32
  341237, // llvm.nvvm.wmma.m8n32k16.store.d.row.f16
  341277, // llvm.nvvm.wmma.m8n32k16.store.d.row.f32
  341317, // llvm.nvvm.wmma.m8n32k16.store.d.row.s32
  341357, // llvm.nvvm.wmma.m8n32k16.store.d.row.stride.f16
  341404, // llvm.nvvm.wmma.m8n32k16.store.d.row.stride.f32
  341451, // llvm.nvvm.wmma.m8n32k16.store.d.row.stride.s32
  341498, // llvm.nvvm.wmma.m8n8k128.load.a.row.b1
  341536, // llvm.nvvm.wmma.m8n8k128.load.a.row.stride.b1
  341581, // llvm.nvvm.wmma.m8n8k128.load.b.col.b1
  341619, // llvm.nvvm.wmma.m8n8k128.load.b.col.stride.b1
  341664, // llvm.nvvm.wmma.m8n8k128.load.c.col.s32
  341703, // llvm.nvvm.wmma.m8n8k128.load.c.col.stride.s32
  341749, // llvm.nvvm.wmma.m8n8k128.load.c.row.s32
  341788, // llvm.nvvm.wmma.m8n8k128.load.c.row.stride.s32
  341834, // llvm.nvvm.wmma.m8n8k128.mma.and.popc.row.col.b1
  341882, // llvm.nvvm.wmma.m8n8k128.mma.xor.popc.row.col.b1
  341930, // llvm.nvvm.wmma.m8n8k128.store.d.col.s32
  341970, // llvm.nvvm.wmma.m8n8k128.store.d.col.stride.s32
  342017, // llvm.nvvm.wmma.m8n8k128.store.d.row.s32
  342057, // llvm.nvvm.wmma.m8n8k128.store.d.row.stride.s32
  342104, // llvm.nvvm.wmma.m8n8k32.load.a.row.s4
  342141, // llvm.nvvm.wmma.m8n8k32.load.a.row.stride.s4
  342185, // llvm.nvvm.wmma.m8n8k32.load.a.row.stride.u4
  342229, // llvm.nvvm.wmma.m8n8k32.load.a.row.u4
  342266, // llvm.nvvm.wmma.m8n8k32.load.b.col.s4
  342303, // llvm.nvvm.wmma.m8n8k32.load.b.col.stride.s4
  342347, // llvm.nvvm.wmma.m8n8k32.load.b.col.stride.u4
  342391, // llvm.nvvm.wmma.m8n8k32.load.b.col.u4
  342428, // llvm.nvvm.wmma.m8n8k32.load.c.col.s32
  342466, // llvm.nvvm.wmma.m8n8k32.load.c.col.stride.s32
  342511, // llvm.nvvm.wmma.m8n8k32.load.c.row.s32
  342549, // llvm.nvvm.wmma.m8n8k32.load.c.row.stride.s32
  342594, // llvm.nvvm.wmma.m8n8k32.mma.row.col.s4
  342632, // llvm.nvvm.wmma.m8n8k32.mma.row.col.s4.satfinite
  342680, // llvm.nvvm.wmma.m8n8k32.mma.row.col.u4
  342718, // llvm.nvvm.wmma.m8n8k32.mma.row.col.u4.satfinite
  342766, // llvm.nvvm.wmma.m8n8k32.store.d.col.s32
  342805, // llvm.nvvm.wmma.m8n8k32.store.d.col.stride.s32
  342851, // llvm.nvvm.wmma.m8n8k32.store.d.row.s32
  342890, // llvm.nvvm.wmma.m8n8k32.store.d.row.stride.s32
  342936, // llvm.nvvm.wmma.m8n8k4.load.a.col.f64
  342973, // llvm.nvvm.wmma.m8n8k4.load.a.col.stride.f64
  343017, // llvm.nvvm.wmma.m8n8k4.load.a.row.f64
  343054, // llvm.nvvm.wmma.m8n8k4.load.a.row.stride.f64
  343098, // llvm.nvvm.wmma.m8n8k4.load.b.col.f64
  343135, // llvm.nvvm.wmma.m8n8k4.load.b.col.stride.f64
  343179, // llvm.nvvm.wmma.m8n8k4.load.b.row.f64
  343216, // llvm.nvvm.wmma.m8n8k4.load.b.row.stride.f64
  343260, // llvm.nvvm.wmma.m8n8k4.load.c.col.f64
  343297, // llvm.nvvm.wmma.m8n8k4.load.c.col.stride.f64
  343341, // llvm.nvvm.wmma.m8n8k4.load.c.row.f64
  343378, // llvm.nvvm.wmma.m8n8k4.load.c.row.stride.f64
  343422, // llvm.nvvm.wmma.m8n8k4.mma.col.col.f64
  343460, // llvm.nvvm.wmma.m8n8k4.mma.col.col.rm.f64
  343501, // llvm.nvvm.wmma.m8n8k4.mma.col.col.rn.f64
  343542, // llvm.nvvm.wmma.m8n8k4.mma.col.col.rp.f64
  343583, // llvm.nvvm.wmma.m8n8k4.mma.col.col.rz.f64
  343624, // llvm.nvvm.wmma.m8n8k4.mma.col.row.f64
  343662, // llvm.nvvm.wmma.m8n8k4.mma.col.row.rm.f64
  343703, // llvm.nvvm.wmma.m8n8k4.mma.col.row.rn.f64
  343744, // llvm.nvvm.wmma.m8n8k4.mma.col.row.rp.f64
  343785, // llvm.nvvm.wmma.m8n8k4.mma.col.row.rz.f64
  343826, // llvm.nvvm.wmma.m8n8k4.mma.row.col.f64
  343864, // llvm.nvvm.wmma.m8n8k4.mma.row.col.rm.f64
  343905, // llvm.nvvm.wmma.m8n8k4.mma.row.col.rn.f64
  343946, // llvm.nvvm.wmma.m8n8k4.mma.row.col.rp.f64
  343987, // llvm.nvvm.wmma.m8n8k4.mma.row.col.rz.f64
  344028, // llvm.nvvm.wmma.m8n8k4.mma.row.row.f64
  344066, // llvm.nvvm.wmma.m8n8k4.mma.row.row.rm.f64
  344107, // llvm.nvvm.wmma.m8n8k4.mma.row.row.rn.f64
  344148, // llvm.nvvm.wmma.m8n8k4.mma.row.row.rp.f64
  344189, // llvm.nvvm.wmma.m8n8k4.mma.row.row.rz.f64
  344230, // llvm.nvvm.wmma.m8n8k4.store.d.col.f64
  344268, // llvm.nvvm.wmma.m8n8k4.store.d.col.stride.f64
  344313, // llvm.nvvm.wmma.m8n8k4.store.d.row.f64
  344351, // llvm.nvvm.wmma.m8n8k4.store.d.row.stride.f64
  344396, // llvm.nvvm.zext.clamp
  344417, // llvm.nvvm.zext.wrap
  344437, // llvm.ppc.addex
  344452, // llvm.ppc.addf128.round.to.odd
  344482, // llvm.ppc.addg6s
  344498, // llvm.ppc.addg6sd
  344515, // llvm.ppc.altivec.crypto.vcipher
  344547, // llvm.ppc.altivec.crypto.vcipherlast
  344583, // llvm.ppc.altivec.crypto.vncipher
  344616, // llvm.ppc.altivec.crypto.vncipherlast
  344653, // llvm.ppc.altivec.crypto.vpermxor
  344686, // llvm.ppc.altivec.crypto.vpermxor.be
  344722, // llvm.ppc.altivec.crypto.vpmsumb
  344754, // llvm.ppc.altivec.crypto.vpmsumd
  344786, // llvm.ppc.altivec.crypto.vpmsumh
  344818, // llvm.ppc.altivec.crypto.vpmsumw
  344850, // llvm.ppc.altivec.crypto.vsbox
  344880, // llvm.ppc.altivec.crypto.vshasigmad
  344915, // llvm.ppc.altivec.crypto.vshasigmaw
  344950, // llvm.ppc.altivec.dss
  344971, // llvm.ppc.altivec.dssall
  344995, // llvm.ppc.altivec.dst
  345016, // llvm.ppc.altivec.dstst
  345039, // llvm.ppc.altivec.dststt
  345063, // llvm.ppc.altivec.dstt
  345085, // llvm.ppc.altivec.lvebx
  345108, // llvm.ppc.altivec.lvehx
  345131, // llvm.ppc.altivec.lvewx
  345154, // llvm.ppc.altivec.lvsl
  345176, // llvm.ppc.altivec.lvsr
  345198, // llvm.ppc.altivec.lvx
  345219, // llvm.ppc.altivec.lvxl
  345241, // llvm.ppc.altivec.mfvscr
  345265, // llvm.ppc.altivec.mtvscr
  345289, // llvm.ppc.altivec.mtvsrbm
  345314, // llvm.ppc.altivec.mtvsrdm
  345339, // llvm.ppc.altivec.mtvsrhm
  345364, // llvm.ppc.altivec.mtvsrqm
  345389, // llvm.ppc.altivec.mtvsrwm
  345414, // llvm.ppc.altivec.stvebx
  345438, // llvm.ppc.altivec.stvehx
  345462, // llvm.ppc.altivec.stvewx
  345486, // llvm.ppc.altivec.stvx
  345508, // llvm.ppc.altivec.stvxl
  345531, // llvm.ppc.altivec.vabsdub
  345556, // llvm.ppc.altivec.vabsduh
  345581, // llvm.ppc.altivec.vabsduw
  345606, // llvm.ppc.altivec.vaddcuq
  345631, // llvm.ppc.altivec.vaddcuw
  345656, // llvm.ppc.altivec.vaddecuq
  345682, // llvm.ppc.altivec.vaddeuqm
  345708, // llvm.ppc.altivec.vaddsbs
  345733, // llvm.ppc.altivec.vaddshs
  345758, // llvm.ppc.altivec.vaddsws
  345783, // llvm.ppc.altivec.vaddubs
  345808, // llvm.ppc.altivec.vadduhs
  345833, // llvm.ppc.altivec.vadduws
  345858, // llvm.ppc.altivec.vavgsb
  345882, // llvm.ppc.altivec.vavgsh
  345906, // llvm.ppc.altivec.vavgsw
  345930, // llvm.ppc.altivec.vavgub
  345954, // llvm.ppc.altivec.vavguh
  345978, // llvm.ppc.altivec.vavguw
  346002, // llvm.ppc.altivec.vbpermd
  346027, // llvm.ppc.altivec.vbpermq
  346052, // llvm.ppc.altivec.vcfsx
  346075, // llvm.ppc.altivec.vcfuged
  346100, // llvm.ppc.altivec.vcfux
  346123, // llvm.ppc.altivec.vclrlb
  346147, // llvm.ppc.altivec.vclrrb
  346171, // llvm.ppc.altivec.vclzdm
  346195, // llvm.ppc.altivec.vclzlsbb
  346221, // llvm.ppc.altivec.vcmpbfp
  346246, // llvm.ppc.altivec.vcmpbfp.p
  346273, // llvm.ppc.altivec.vcmpeqfp
  346299, // llvm.ppc.altivec.vcmpeqfp.p
  346327, // llvm.ppc.altivec.vcmpequb
  346353, // llvm.ppc.altivec.vcmpequb.p
  346381, // llvm.ppc.altivec.vcmpequd
  346407, // llvm.ppc.altivec.vcmpequd.p
  346435, // llvm.ppc.altivec.vcmpequh
  346461, // llvm.ppc.altivec.vcmpequh.p
  346489, // llvm.ppc.altivec.vcmpequq
  346515, // llvm.ppc.altivec.vcmpequq.p
  346543, // llvm.ppc.altivec.vcmpequw
  346569, // llvm.ppc.altivec.vcmpequw.p
  346597, // llvm.ppc.altivec.vcmpgefp
  346623, // llvm.ppc.altivec.vcmpgefp.p
  346651, // llvm.ppc.altivec.vcmpgtfp
  346677, // llvm.ppc.altivec.vcmpgtfp.p
  346705, // llvm.ppc.altivec.vcmpgtsb
  346731, // llvm.ppc.altivec.vcmpgtsb.p
  346759, // llvm.ppc.altivec.vcmpgtsd
  346785, // llvm.ppc.altivec.vcmpgtsd.p
  346813, // llvm.ppc.altivec.vcmpgtsh
  346839, // llvm.ppc.altivec.vcmpgtsh.p
  346867, // llvm.ppc.altivec.vcmpgtsq
  346893, // llvm.ppc.altivec.vcmpgtsq.p
  346921, // llvm.ppc.altivec.vcmpgtsw
  346947, // llvm.ppc.altivec.vcmpgtsw.p
  346975, // llvm.ppc.altivec.vcmpgtub
  347001, // llvm.ppc.altivec.vcmpgtub.p
  347029, // llvm.ppc.altivec.vcmpgtud
  347055, // llvm.ppc.altivec.vcmpgtud.p
  347083, // llvm.ppc.altivec.vcmpgtuh
  347109, // llvm.ppc.altivec.vcmpgtuh.p
  347137, // llvm.ppc.altivec.vcmpgtuq
  347163, // llvm.ppc.altivec.vcmpgtuq.p
  347191, // llvm.ppc.altivec.vcmpgtuw
  347217, // llvm.ppc.altivec.vcmpgtuw.p
  347245, // llvm.ppc.altivec.vcmpneb
  347270, // llvm.ppc.altivec.vcmpneb.p
  347297, // llvm.ppc.altivec.vcmpneh
  347322, // llvm.ppc.altivec.vcmpneh.p
  347349, // llvm.ppc.altivec.vcmpnew
  347374, // llvm.ppc.altivec.vcmpnew.p
  347401, // llvm.ppc.altivec.vcmpnezb
  347427, // llvm.ppc.altivec.vcmpnezb.p
  347455, // llvm.ppc.altivec.vcmpnezh
  347481, // llvm.ppc.altivec.vcmpnezh.p
  347509, // llvm.ppc.altivec.vcmpnezw
  347535, // llvm.ppc.altivec.vcmpnezw.p
  347563, // llvm.ppc.altivec.vcntmbb
  347588, // llvm.ppc.altivec.vcntmbd
  347613, // llvm.ppc.altivec.vcntmbh
  347638, // llvm.ppc.altivec.vcntmbw
  347663, // llvm.ppc.altivec.vctsxs
  347687, // llvm.ppc.altivec.vctuxs
  347711, // llvm.ppc.altivec.vctzdm
  347735, // llvm.ppc.altivec.vctzlsbb
  347761, // llvm.ppc.altivec.vdivesd
  347786, // llvm.ppc.altivec.vdivesq
  347811, // llvm.ppc.altivec.vdivesw
  347836, // llvm.ppc.altivec.vdiveud
  347861, // llvm.ppc.altivec.vdiveuq
  347886, // llvm.ppc.altivec.vdiveuw
  347911, // llvm.ppc.altivec.vexpandbm
  347938, // llvm.ppc.altivec.vexpanddm
  347965, // llvm.ppc.altivec.vexpandhm
  347992, // llvm.ppc.altivec.vexpandqm
  348019, // llvm.ppc.altivec.vexpandwm
  348046, // llvm.ppc.altivec.vexptefp
  348072, // llvm.ppc.altivec.vextddvlx
  348099, // llvm.ppc.altivec.vextddvrx
  348126, // llvm.ppc.altivec.vextdubvlx
  348154, // llvm.ppc.altivec.vextdubvrx
  348182, // llvm.ppc.altivec.vextduhvlx
  348210, // llvm.ppc.altivec.vextduhvrx
  348238, // llvm.ppc.altivec.vextduwvlx
  348266, // llvm.ppc.altivec.vextduwvrx
  348294, // llvm.ppc.altivec.vextractbm
  348322, // llvm.ppc.altivec.vextractdm
  348350, // llvm.ppc.altivec.vextracthm
  348378, // llvm.ppc.altivec.vextractqm
  348406, // llvm.ppc.altivec.vextractwm
  348434, // llvm.ppc.altivec.vextsb2d
  348460, // llvm.ppc.altivec.vextsb2w
  348486, // llvm.ppc.altivec.vextsd2q
  348512, // llvm.ppc.altivec.vextsh2d
  348538, // llvm.ppc.altivec.vextsh2w
  348564, // llvm.ppc.altivec.vextsw2d
  348590, // llvm.ppc.altivec.vgbbd
  348613, // llvm.ppc.altivec.vgnb
  348635, // llvm.ppc.altivec.vinsblx
  348660, // llvm.ppc.altivec.vinsbrx
  348685, // llvm.ppc.altivec.vinsbvlx
  348711, // llvm.ppc.altivec.vinsbvrx
  348737, // llvm.ppc.altivec.vinsd
  348760, // llvm.ppc.altivec.vinsdlx
  348785, // llvm.ppc.altivec.vinsdrx
  348810, // llvm.ppc.altivec.vinshlx
  348835, // llvm.ppc.altivec.vinshrx
  348860, // llvm.ppc.altivec.vinshvlx
  348886, // llvm.ppc.altivec.vinshvrx
  348912, // llvm.ppc.altivec.vinsw
  348935, // llvm.ppc.altivec.vinswlx
  348960, // llvm.ppc.altivec.vinswrx
  348985, // llvm.ppc.altivec.vinswvlx
  349011, // llvm.ppc.altivec.vinswvrx
  349037, // llvm.ppc.altivec.vlogefp
  349062, // llvm.ppc.altivec.vmaddfp
  349087, // llvm.ppc.altivec.vmaxfp
  349111, // llvm.ppc.altivec.vmaxsb
  349135, // llvm.ppc.altivec.vmaxsd
  349159, // llvm.ppc.altivec.vmaxsh
  349183, // llvm.ppc.altivec.vmaxsw
  349207, // llvm.ppc.altivec.vmaxub
  349231, // llvm.ppc.altivec.vmaxud
  349255, // llvm.ppc.altivec.vmaxuh
  349279, // llvm.ppc.altivec.vmaxuw
  349303, // llvm.ppc.altivec.vmhaddshs
  349330, // llvm.ppc.altivec.vmhraddshs
  349358, // llvm.ppc.altivec.vminfp
  349382, // llvm.ppc.altivec.vminsb
  349406, // llvm.ppc.altivec.vminsd
  349430, // llvm.ppc.altivec.vminsh
  349454, // llvm.ppc.altivec.vminsw
  349478, // llvm.ppc.altivec.vminub
  349502, // llvm.ppc.altivec.vminud
  349526, // llvm.ppc.altivec.vminuh
  349550, // llvm.ppc.altivec.vminuw
  349574, // llvm.ppc.altivec.vmladduhm
  349601, // llvm.ppc.altivec.vmsumcud
  349627, // llvm.ppc.altivec.vmsummbm
  349653, // llvm.ppc.altivec.vmsumshm
  349679, // llvm.ppc.altivec.vmsumshs
  349705, // llvm.ppc.altivec.vmsumubm
  349731, // llvm.ppc.altivec.vmsumudm
  349757, // llvm.ppc.altivec.vmsumuhm
  349783, // llvm.ppc.altivec.vmsumuhs
  349809, // llvm.ppc.altivec.vmulesb
  349834, // llvm.ppc.altivec.vmulesd
  349859, // llvm.ppc.altivec.vmulesh
  349884, // llvm.ppc.altivec.vmulesw
  349909, // llvm.ppc.altivec.vmuleub
  349934, // llvm.ppc.altivec.vmuleud
  349959, // llvm.ppc.altivec.vmuleuh
  349984, // llvm.ppc.altivec.vmuleuw
  350009, // llvm.ppc.altivec.vmulhsd
  350034, // llvm.ppc.altivec.vmulhsw
  350059, // llvm.ppc.altivec.vmulhud
  350084, // llvm.ppc.altivec.vmulhuw
  350109, // llvm.ppc.altivec.vmulosb
  350134, // llvm.ppc.altivec.vmulosd
  350159, // llvm.ppc.altivec.vmulosh
  350184, // llvm.ppc.altivec.vmulosw
  350209, // llvm.ppc.altivec.vmuloub
  350234, // llvm.ppc.altivec.vmuloud
  350259, // llvm.ppc.altivec.vmulouh
  350284, // llvm.ppc.altivec.vmulouw
  350309, // llvm.ppc.altivec.vnmsubfp
  350335, // llvm.ppc.altivec.vpdepd
  350359, // llvm.ppc.altivec.vperm
  350382, // llvm.ppc.altivec.vpextd
  350406, // llvm.ppc.altivec.vpkpx
  350429, // llvm.ppc.altivec.vpksdss
  350454, // llvm.ppc.altivec.vpksdus
  350479, // llvm.ppc.altivec.vpkshss
  350504, // llvm.ppc.altivec.vpkshus
  350529, // llvm.ppc.altivec.vpkswss
  350554, // llvm.ppc.altivec.vpkswus
  350579, // llvm.ppc.altivec.vpkudus
  350604, // llvm.ppc.altivec.vpkuhus
  350629, // llvm.ppc.altivec.vpkuwus
  350654, // llvm.ppc.altivec.vprtybd
  350679, // llvm.ppc.altivec.vprtybq
  350704, // llvm.ppc.altivec.vprtybw
  350729, // llvm.ppc.altivec.vrefp
  350752, // llvm.ppc.altivec.vrfim
  350775, // llvm.ppc.altivec.vrfin
  350798, // llvm.ppc.altivec.vrfip
  350821, // llvm.ppc.altivec.vrfiz
  350844, // llvm.ppc.altivec.vrlb
  350866, // llvm.ppc.altivec.vrld
  350888, // llvm.ppc.altivec.vrldmi
  350912, // llvm.ppc.altivec.vrldnm
  350936, // llvm.ppc.altivec.vrlh
  350958, // llvm.ppc.altivec.vrlqmi
  350982, // llvm.ppc.altivec.vrlqnm
  351006, // llvm.ppc.altivec.vrlw
  351028, // llvm.ppc.altivec.vrlwmi
  351052, // llvm.ppc.altivec.vrlwnm
  351076, // llvm.ppc.altivec.vrsqrtefp
  351103, // llvm.ppc.altivec.vsel
  351125, // llvm.ppc.altivec.vsl
  351146, // llvm.ppc.altivec.vslb
  351168, // llvm.ppc.altivec.vsldbi
  351192, // llvm.ppc.altivec.vslh
  351214, // llvm.ppc.altivec.vslo
  351236, // llvm.ppc.altivec.vslv
  351258, // llvm.ppc.altivec.vslw
  351280, // llvm.ppc.altivec.vsr
  351301, // llvm.ppc.altivec.vsrab
  351324, // llvm.ppc.altivec.vsrah
  351347, // llvm.ppc.altivec.vsraw
  351370, // llvm.ppc.altivec.vsrb
  351392, // llvm.ppc.altivec.vsrdbi
  351416, // llvm.ppc.altivec.vsrh
  351438, // llvm.ppc.altivec.vsro
  351460, // llvm.ppc.altivec.vsrv
  351482, // llvm.ppc.altivec.vsrw
  351504, // llvm.ppc.altivec.vstribl
  351529, // llvm.ppc.altivec.vstribl.p
  351556, // llvm.ppc.altivec.vstribr
  351581, // llvm.ppc.altivec.vstribr.p
  351608, // llvm.ppc.altivec.vstrihl
  351633, // llvm.ppc.altivec.vstrihl.p
  351660, // llvm.ppc.altivec.vstrihr
  351685, // llvm.ppc.altivec.vstrihr.p
  351712, // llvm.ppc.altivec.vsubcuq
  351737, // llvm.ppc.altivec.vsubcuw
  351762, // llvm.ppc.altivec.vsubecuq
  351788, // llvm.ppc.altivec.vsubeuqm
  351814, // llvm.ppc.altivec.vsubsbs
  351839, // llvm.ppc.altivec.vsubshs
  351864, // llvm.ppc.altivec.vsubsws
  351889, // llvm.ppc.altivec.vsububs
  351914, // llvm.ppc.altivec.vsubuhs
  351939, // llvm.ppc.altivec.vsubuws
  351964, // llvm.ppc.altivec.vsum2sws
  351990, // llvm.ppc.altivec.vsum4sbs
  352016, // llvm.ppc.altivec.vsum4shs
  352042, // llvm.ppc.altivec.vsum4ubs
  352068, // llvm.ppc.altivec.vsumsws
  352093, // llvm.ppc.altivec.vupkhpx
  352118, // llvm.ppc.altivec.vupkhsb
  352143, // llvm.ppc.altivec.vupkhsh
  352168, // llvm.ppc.altivec.vupkhsw
  352193, // llvm.ppc.altivec.vupklpx
  352218, // llvm.ppc.altivec.vupklsb
  352243, // llvm.ppc.altivec.vupklsh
  352268, // llvm.ppc.altivec.vupklsw
  352293, // llvm.ppc.amo.ldat
  352311, // llvm.ppc.amo.lwat
  352329, // llvm.ppc.atomic.load.i128
  352355, // llvm.ppc.atomic.store.i128
  352382, // llvm.ppc.atomicrmw.add.i128
  352410, // llvm.ppc.atomicrmw.and.i128
  352438, // llvm.ppc.atomicrmw.nand.i128
  352467, // llvm.ppc.atomicrmw.or.i128
  352494, // llvm.ppc.atomicrmw.sub.i128
  352522, // llvm.ppc.atomicrmw.xchg.i128
  352551, // llvm.ppc.atomicrmw.xor.i128
  352579, // llvm.ppc.bcdadd
  352595, // llvm.ppc.bcdadd.p
  352613, // llvm.ppc.bcdcopysign
  352634, // llvm.ppc.bcdsetsign
  352654, // llvm.ppc.bcdsub
  352670, // llvm.ppc.bcdsub.p
  352688, // llvm.ppc.bpermd
  352704, // llvm.ppc.cbcdtd
  352720, // llvm.ppc.cbcdtdd
  352737, // llvm.ppc.cdtbcd
  352753, // llvm.ppc.cdtbcdd
  352770, // llvm.ppc.cfence
  352786, // llvm.ppc.cfuged
  352802, // llvm.ppc.cmpb
  352816, // llvm.ppc.cmpeqb
  352832, // llvm.ppc.cmprb
  352847, // llvm.ppc.cmpxchg.i128
  352869, // llvm.ppc.cntlzdm
  352886, // llvm.ppc.cnttzdm
  352903, // llvm.ppc.compare.exp.eq
  352927, // llvm.ppc.compare.exp.gt
  352951, // llvm.ppc.compare.exp.lt
  352975, // llvm.ppc.compare.exp.uo
  352999, // llvm.ppc.convert.f128.to.ppcf128
  353032, // llvm.ppc.convert.ppcf128.to.f128
  353065, // llvm.ppc.darn
  353079, // llvm.ppc.darn32
  353095, // llvm.ppc.darnraw
  353112, // llvm.ppc.dcba
  353126, // llvm.ppc.dcbf
  353140, // llvm.ppc.dcbfl
  353155, // llvm.ppc.dcbflp
  353171, // llvm.ppc.dcbfps
  353187, // llvm.ppc.dcbi
  353201, // llvm.ppc.dcbst
  353216, // llvm.ppc.dcbstps
  353233, // llvm.ppc.dcbt
  353247, // llvm.ppc.dcbt.with.hint
  353271, // llvm.ppc.dcbtst
  353287, // llvm.ppc.dcbtst.with.hint
  353313, // llvm.ppc.dcbtstt
  353330, // llvm.ppc.dcbtt
  353345, // llvm.ppc.dcbz
  353359, // llvm.ppc.dcbzl
  353374, // llvm.ppc.divde
  353389, // llvm.ppc.divdeu
  353405, // llvm.ppc.divf128.round.to.odd
  353435, // llvm.ppc.divwe
  353450, // llvm.ppc.divweu
  353466, // llvm.ppc.eieio
  353481, // llvm.ppc.extract.exp
  353502, // llvm.ppc.extract.sig
  353523, // llvm.ppc.fcfid
  353538, // llvm.ppc.fcfud
  353553, // llvm.ppc.fctid
  353568, // llvm.ppc.fctidz
  353584, // llvm.ppc.fctiw
  353599, // llvm.ppc.fctiwz
  353615, // llvm.ppc.fctudz
  353631, // llvm.ppc.fctuwz
  353647, // llvm.ppc.fence
  353662, // llvm.ppc.fixed.addr.ld
  353685, // llvm.ppc.fmaf128.round.to.odd
  353715, // llvm.ppc.fmsub
  353730, // llvm.ppc.fmsubs
  353746, // llvm.ppc.fnabs
  353761, // llvm.ppc.fnabss
  353777, // llvm.ppc.fnmadd
  353793, // llvm.ppc.fnmadds
  353810, // llvm.ppc.fnmsub
  353826, // llvm.ppc.fre
  353839, // llvm.ppc.fres
  353853, // llvm.ppc.frsqrte
  353870, // llvm.ppc.frsqrtes
  353888, // llvm.ppc.fsel
  353902, // llvm.ppc.fsels
  353917, // llvm.ppc.get.texasr
  353937, // llvm.ppc.get.texasru
  353958, // llvm.ppc.get.tfhar
  353977, // llvm.ppc.get.tfiar
  353996, // llvm.ppc.icbt
  354010, // llvm.ppc.insert.exp
  354030, // llvm.ppc.iospace.eieio
  354053, // llvm.ppc.iospace.lwsync
  354077, // llvm.ppc.iospace.sync
  354099, // llvm.ppc.isync
  354114, // llvm.ppc.lbarx
  354129, // llvm.ppc.ldarx
  354144, // llvm.ppc.lharx
  354159, // llvm.ppc.load2r
  354175, // llvm.ppc.load4r
  354191, // llvm.ppc.load8r
  354207, // llvm.ppc.lwarx
  354222, // llvm.ppc.lwsync
  354238, // llvm.ppc.maddhd
  354254, // llvm.ppc.maddhdu
  354271, // llvm.ppc.maddld
  354287, // llvm.ppc.maxfe
  354302, // llvm.ppc.maxfl
  354317, // llvm.ppc.maxfs
  354332, // llvm.ppc.mffsl
  354347, // llvm.ppc.mfmsr
  354362, // llvm.ppc.mfspr
  354377, // llvm.ppc.mftbu
  354392, // llvm.ppc.minfe
  354407, // llvm.ppc.minfl
  354422, // llvm.ppc.minfs
  354437, // llvm.ppc.mma.assemble.acc
  354463, // llvm.ppc.mma.build.dmr
  354486, // llvm.ppc.mma.disassemble.acc
  354515, // llvm.ppc.mma.disassemble.dmr
  354544, // llvm.ppc.mma.dmmr
  354562, // llvm.ppc.mma.dmsetdmrz
  354585, // llvm.ppc.mma.dmsha2hash
  354609, // llvm.ppc.mma.dmsha3hash
  354633, // llvm.ppc.mma.dmxor
  354652, // llvm.ppc.mma.dmxvbf16gerx2
  354679, // llvm.ppc.mma.dmxvbf16gerx2nn
  354708, // llvm.ppc.mma.dmxvbf16gerx2np
  354737, // llvm.ppc.mma.dmxvbf16gerx2pn
  354766, // llvm.ppc.mma.dmxvbf16gerx2pp
  354795, // llvm.ppc.mma.dmxvf16gerx2
  354821, // llvm.ppc.mma.dmxvf16gerx2nn
  354849, // llvm.ppc.mma.dmxvf16gerx2np
  354877, // llvm.ppc.mma.dmxvf16gerx2pn
  354905, // llvm.ppc.mma.dmxvf16gerx2pp
  354933, // llvm.ppc.mma.dmxvi8gerx4
  354958, // llvm.ppc.mma.dmxvi8gerx4pp
  354985, // llvm.ppc.mma.dmxvi8gerx4spp
  355013, // llvm.ppc.mma.dmxxextfdmr256
  355041, // llvm.ppc.mma.dmxxextfdmr512
  355069, // llvm.ppc.mma.dmxxinstdmr256
  355097, // llvm.ppc.mma.dmxxinstdmr512
  355125, // llvm.ppc.mma.dmxxshapad
  355149, // llvm.ppc.mma.pmdmxvbf16gerx2
  355178, // llvm.ppc.mma.pmdmxvbf16gerx2nn
  355209, // llvm.ppc.mma.pmdmxvbf16gerx2np
  355240, // llvm.ppc.mma.pmdmxvbf16gerx2pn
  355271, // llvm.ppc.mma.pmdmxvbf16gerx2pp
  355302, // llvm.ppc.mma.pmdmxvf16gerx2
  355330, // llvm.ppc.mma.pmdmxvf16gerx2nn
  355360, // llvm.ppc.mma.pmdmxvf16gerx2np
  355390, // llvm.ppc.mma.pmdmxvf16gerx2pn
  355420, // llvm.ppc.mma.pmdmxvf16gerx2pp
  355450, // llvm.ppc.mma.pmdmxvi8gerx4
  355477, // llvm.ppc.mma.pmdmxvi8gerx4pp
  355506, // llvm.ppc.mma.pmdmxvi8gerx4spp
  355536, // llvm.ppc.mma.pmxvbf16ger2
  355562, // llvm.ppc.mma.pmxvbf16ger2nn
  355590, // llvm.ppc.mma.pmxvbf16ger2np
  355618, // llvm.ppc.mma.pmxvbf16ger2pn
  355646, // llvm.ppc.mma.pmxvbf16ger2pp
  355674, // llvm.ppc.mma.pmxvf16ger2
  355699, // llvm.ppc.mma.pmxvf16ger2nn
  355726, // llvm.ppc.mma.pmxvf16ger2np
  355753, // llvm.ppc.mma.pmxvf16ger2pn
  355780, // llvm.ppc.mma.pmxvf16ger2pp
  355807, // llvm.ppc.mma.pmxvf32ger
  355831, // llvm.ppc.mma.pmxvf32gernn
  355857, // llvm.ppc.mma.pmxvf32gernp
  355883, // llvm.ppc.mma.pmxvf32gerpn
  355909, // llvm.ppc.mma.pmxvf32gerpp
  355935, // llvm.ppc.mma.pmxvf64ger
  355959, // llvm.ppc.mma.pmxvf64gernn
  355985, // llvm.ppc.mma.pmxvf64gernp
  356011, // llvm.ppc.mma.pmxvf64gerpn
  356037, // llvm.ppc.mma.pmxvf64gerpp
  356063, // llvm.ppc.mma.pmxvi16ger2
  356088, // llvm.ppc.mma.pmxvi16ger2pp
  356115, // llvm.ppc.mma.pmxvi16ger2s
  356141, // llvm.ppc.mma.pmxvi16ger2spp
  356169, // llvm.ppc.mma.pmxvi4ger8
  356193, // llvm.ppc.mma.pmxvi4ger8pp
  356219, // llvm.ppc.mma.pmxvi8ger4
  356243, // llvm.ppc.mma.pmxvi8ger4pp
  356269, // llvm.ppc.mma.pmxvi8ger4spp
  356296, // llvm.ppc.mma.xvbf16ger2
  356320, // llvm.ppc.mma.xvbf16ger2nn
  356346, // llvm.ppc.mma.xvbf16ger2np
  356372, // llvm.ppc.mma.xvbf16ger2pn
  356398, // llvm.ppc.mma.xvbf16ger2pp
  356424, // llvm.ppc.mma.xvf16ger2
  356447, // llvm.ppc.mma.xvf16ger2nn
  356472, // llvm.ppc.mma.xvf16ger2np
  356497, // llvm.ppc.mma.xvf16ger2pn
  356522, // llvm.ppc.mma.xvf16ger2pp
  356547, // llvm.ppc.mma.xvf32ger
  356569, // llvm.ppc.mma.xvf32gernn
  356593, // llvm.ppc.mma.xvf32gernp
  356617, // llvm.ppc.mma.xvf32gerpn
  356641, // llvm.ppc.mma.xvf32gerpp
  356665, // llvm.ppc.mma.xvf64ger
  356687, // llvm.ppc.mma.xvf64gernn
  356711, // llvm.ppc.mma.xvf64gernp
  356735, // llvm.ppc.mma.xvf64gerpn
  356759, // llvm.ppc.mma.xvf64gerpp
  356783, // llvm.ppc.mma.xvi16ger2
  356806, // llvm.ppc.mma.xvi16ger2pp
  356831, // llvm.ppc.mma.xvi16ger2s
  356855, // llvm.ppc.mma.xvi16ger2spp
  356881, // llvm.ppc.mma.xvi4ger8
  356903, // llvm.ppc.mma.xvi4ger8pp
  356927, // llvm.ppc.mma.xvi8ger4
  356949, // llvm.ppc.mma.xvi8ger4pp
  356973, // llvm.ppc.mma.xvi8ger4spp
  356998, // llvm.ppc.mma.xxmfacc
  357019, // llvm.ppc.mma.xxmtacc
  357040, // llvm.ppc.mma.xxsetaccz
  357063, // llvm.ppc.mtfsb0
  357079, // llvm.ppc.mtfsb1
  357095, // llvm.ppc.mtfsf
  357110, // llvm.ppc.mtfsfi
  357126, // llvm.ppc.mtmsr
  357141, // llvm.ppc.mtspr
  357156, // llvm.ppc.mulf128.round.to.odd
  357186, // llvm.ppc.mulhd
  357201, // llvm.ppc.mulhdu
  357217, // llvm.ppc.mulhw
  357232, // llvm.ppc.mulhwu
  357248, // llvm.ppc.national2packed
  357273, // llvm.ppc.pack.longdouble
  357298, // llvm.ppc.packed2national
  357323, // llvm.ppc.packed2zoned
  357345, // llvm.ppc.pdepd
  357360, // llvm.ppc.pextd
  357375, // llvm.ppc.popcntb
  357392, // llvm.ppc.readflm
  357409, // llvm.ppc.rldimi
  357425, // llvm.ppc.rlwimi
  357441, // llvm.ppc.rlwnm
  357456, // llvm.ppc.scalar.extract.expq
  357485, // llvm.ppc.scalar.insert.exp.qp
  357515, // llvm.ppc.set.texasr
  357535, // llvm.ppc.set.texasru
  357556, // llvm.ppc.set.tfhar
  357575, // llvm.ppc.set.tfiar
  357594, // llvm.ppc.setb
  357608, // llvm.ppc.setflm
  357624, // llvm.ppc.setrnd
  357640, // llvm.ppc.sqrtf128.round.to.odd
  357671, // llvm.ppc.stbcx
  357686, // llvm.ppc.stdcx
  357701, // llvm.ppc.stfiw
  357716, // llvm.ppc.sthcx
  357731, // llvm.ppc.store2r
  357748, // llvm.ppc.store4r
  357765, // llvm.ppc.store8r
  357782, // llvm.ppc.stwcx
  357797, // llvm.ppc.subf128.round.to.odd
  357827, // llvm.ppc.sync
  357841, // llvm.ppc.tabort
  357857, // llvm.ppc.tabortdc
  357875, // llvm.ppc.tabortdci
  357894, // llvm.ppc.tabortwc
  357912, // llvm.ppc.tabortwci
  357931, // llvm.ppc.tbegin
  357947, // llvm.ppc.tcheck
  357963, // llvm.ppc.tdw
  357976, // llvm.ppc.tend
  357990, // llvm.ppc.tendall
  358007, // llvm.ppc.test.data.class
  358032, // llvm.ppc.trap
  358046, // llvm.ppc.trapd
  358061, // llvm.ppc.trechkpt
  358079, // llvm.ppc.treclaim
  358097, // llvm.ppc.tresume
  358114, // llvm.ppc.truncf128.round.to.odd
  358146, // llvm.ppc.tsr
  358159, // llvm.ppc.tsuspend
  358177, // llvm.ppc.ttest
  358192, // llvm.ppc.tw
  358204, // llvm.ppc.unpack.longdouble
  358231, // llvm.ppc.vsx.assemble.pair
  358258, // llvm.ppc.vsx.disassemble.pair
  358288, // llvm.ppc.vsx.lxvd2x
  358308, // llvm.ppc.vsx.lxvd2x.be
  358331, // llvm.ppc.vsx.lxvl
  358349, // llvm.ppc.vsx.lxvll
  358368, // llvm.ppc.vsx.lxvp
  358386, // llvm.ppc.vsx.lxvprl
  358406, // llvm.ppc.vsx.lxvprll
  358427, // llvm.ppc.vsx.lxvrl
  358446, // llvm.ppc.vsx.lxvrll
  358466, // llvm.ppc.vsx.lxvw4x
  358486, // llvm.ppc.vsx.lxvw4x.be
  358509, // llvm.ppc.vsx.stxvd2x
  358530, // llvm.ppc.vsx.stxvd2x.be
  358554, // llvm.ppc.vsx.stxvl
  358573, // llvm.ppc.vsx.stxvll
  358593, // llvm.ppc.vsx.stxvp
  358612, // llvm.ppc.vsx.stxvprl
  358633, // llvm.ppc.vsx.stxvprll
  358655, // llvm.ppc.vsx.stxvrl
  358675, // llvm.ppc.vsx.stxvrll
  358696, // llvm.ppc.vsx.stxvw4x
  358717, // llvm.ppc.vsx.stxvw4x.be
  358741, // llvm.ppc.vsx.xsmaxdp
  358762, // llvm.ppc.vsx.xsmindp
  358783, // llvm.ppc.vsx.xvcmpeqdp
  358806, // llvm.ppc.vsx.xvcmpeqdp.p
  358831, // llvm.ppc.vsx.xvcmpeqsp
  358854, // llvm.ppc.vsx.xvcmpeqsp.p
  358879, // llvm.ppc.vsx.xvcmpgedp
  358902, // llvm.ppc.vsx.xvcmpgedp.p
  358927, // llvm.ppc.vsx.xvcmpgesp
  358950, // llvm.ppc.vsx.xvcmpgesp.p
  358975, // llvm.ppc.vsx.xvcmpgtdp
  358998, // llvm.ppc.vsx.xvcmpgtdp.p
  359023, // llvm.ppc.vsx.xvcmpgtsp
  359046, // llvm.ppc.vsx.xvcmpgtsp.p
  359071, // llvm.ppc.vsx.xvcvbf16spn
  359096, // llvm.ppc.vsx.xvcvdpsp
  359118, // llvm.ppc.vsx.xvcvdpsxws
  359142, // llvm.ppc.vsx.xvcvdpuxws
  359166, // llvm.ppc.vsx.xvcvhpsp
  359188, // llvm.ppc.vsx.xvcvspbf16
  359212, // llvm.ppc.vsx.xvcvspdp
  359234, // llvm.ppc.vsx.xvcvsphp
  359256, // llvm.ppc.vsx.xvcvspsxds
  359280, // llvm.ppc.vsx.xvcvspuxds
  359304, // llvm.ppc.vsx.xvcvsxdsp
  359327, // llvm.ppc.vsx.xvcvsxwdp
  359350, // llvm.ppc.vsx.xvcvuxdsp
  359373, // llvm.ppc.vsx.xvcvuxwdp
  359396, // llvm.ppc.vsx.xvdivdp
  359417, // llvm.ppc.vsx.xvdivsp
  359438, // llvm.ppc.vsx.xviexpdp
  359460, // llvm.ppc.vsx.xviexpsp
  359482, // llvm.ppc.vsx.xvmaxdp
  359503, // llvm.ppc.vsx.xvmaxsp
  359524, // llvm.ppc.vsx.xvmindp
  359545, // llvm.ppc.vsx.xvminsp
  359566, // llvm.ppc.vsx.xvrdpip
  359587, // llvm.ppc.vsx.xvredp
  359607, // llvm.ppc.vsx.xvresp
  359627, // llvm.ppc.vsx.xvrlw
  359646, // llvm.ppc.vsx.xvrspip
  359667, // llvm.ppc.vsx.xvrsqrtedp
  359691, // llvm.ppc.vsx.xvrsqrtesp
  359715, // llvm.ppc.vsx.xvtdivdp
  359737, // llvm.ppc.vsx.xvtdivsp
  359759, // llvm.ppc.vsx.xvtlsbb
  359780, // llvm.ppc.vsx.xvtsqrtdp
  359803, // llvm.ppc.vsx.xvtsqrtsp
  359826, // llvm.ppc.vsx.xvtstdcdp
  359849, // llvm.ppc.vsx.xvtstdcsp
  359872, // llvm.ppc.vsx.xvxexpdp
  359894, // llvm.ppc.vsx.xvxexpsp
  359916, // llvm.ppc.vsx.xvxsigdp
  359938, // llvm.ppc.vsx.xvxsigsp
  359960, // llvm.ppc.vsx.xxblendvb
  359983, // llvm.ppc.vsx.xxblendvd
  360006, // llvm.ppc.vsx.xxblendvh
  360029, // llvm.ppc.vsx.xxblendvw
  360052, // llvm.ppc.vsx.xxeval
  360072, // llvm.ppc.vsx.xxextractuw
  360097, // llvm.ppc.vsx.xxgenpcvbm
  360121, // llvm.ppc.vsx.xxgenpcvdm
  360145, // llvm.ppc.vsx.xxgenpcvhm
  360169, // llvm.ppc.vsx.xxgenpcvwm
  360193, // llvm.ppc.vsx.xxinsertw
  360216, // llvm.ppc.vsx.xxleqv
  360236, // llvm.ppc.vsx.xxpermx
  360257, // llvm.ppc.zoned2packed
  360279, // llvm.r600.cube
  360294, // llvm.r600.ddx
  360308, // llvm.r600.ddy
  360322, // llvm.r600.dot4
  360337, // llvm.r600.group.barrier
  360361, // llvm.r600.implicitarg.ptr
  360387, // llvm.r600.kill
  360402, // llvm.r600.rat.store.typed
  360428, // llvm.r600.read.global.size.x
  360457, // llvm.r600.read.global.size.y
  360486, // llvm.r600.read.global.size.z
  360515, // llvm.r600.read.local.size.x
  360543, // llvm.r600.read.local.size.y
  360571, // llvm.r600.read.local.size.z
  360599, // llvm.r600.read.ngroups.x
  360624, // llvm.r600.read.ngroups.y
  360649, // llvm.r600.read.ngroups.z
  360674, // llvm.r600.read.tgid.x
  360696, // llvm.r600.read.tgid.y
  360718, // llvm.r600.read.tgid.z
  360740, // llvm.r600.read.tidig.x
  360763, // llvm.r600.read.tidig.y
  360786, // llvm.r600.read.tidig.z
  360809, // llvm.r600.recipsqrt.clamped
  360837, // llvm.r600.recipsqrt.ieee
  360862, // llvm.r600.store.stream.output
  360892, // llvm.r600.store.swizzle
  360916, // llvm.r600.tex
  360930, // llvm.r600.texc
  360945, // llvm.r600.txb
  360959, // llvm.r600.txbc
  360974, // llvm.r600.txf
  360988, // llvm.r600.txl
  361002, // llvm.r600.txlc
  361017, // llvm.r600.txq
  361031, // llvm.riscv.aes32dsi
  361051, // llvm.riscv.aes32dsmi
  361072, // llvm.riscv.aes32esi
  361092, // llvm.riscv.aes32esmi
  361113, // llvm.riscv.aes64ds
  361132, // llvm.riscv.aes64dsm
  361152, // llvm.riscv.aes64es
  361171, // llvm.riscv.aes64esm
  361191, // llvm.riscv.aes64im
  361210, // llvm.riscv.aes64ks1i
  361231, // llvm.riscv.aes64ks2
  361251, // llvm.riscv.brev8
  361268, // llvm.riscv.clmul
  361285, // llvm.riscv.clmulh
  361303, // llvm.riscv.clmulr
  361321, // llvm.riscv.cv.alu.addN
  361344, // llvm.riscv.cv.alu.addRN
  361368, // llvm.riscv.cv.alu.adduN
  361392, // llvm.riscv.cv.alu.adduRN
  361417, // llvm.riscv.cv.alu.clip
  361440, // llvm.riscv.cv.alu.clipu
  361464, // llvm.riscv.cv.alu.subN
  361487, // llvm.riscv.cv.alu.subRN
  361511, // llvm.riscv.cv.alu.subuN
  361535, // llvm.riscv.cv.alu.subuRN
  361560, // llvm.riscv.cv.bitmanip.bclr
  361588, // llvm.riscv.cv.bitmanip.bitrev
  361618, // llvm.riscv.cv.bitmanip.bset
  361646, // llvm.riscv.cv.bitmanip.clb
  361673, // llvm.riscv.cv.bitmanip.extract
  361704, // llvm.riscv.cv.bitmanip.extractu
  361736, // llvm.riscv.cv.bitmanip.insert
  361766, // llvm.riscv.cv.elw.elw
  361788, // llvm.riscv.cv.mac.mac
  361810, // llvm.riscv.cv.mac.machhsN
  361836, // llvm.riscv.cv.mac.machhsRN
  361863, // llvm.riscv.cv.mac.machhuN
  361889, // llvm.riscv.cv.mac.machhuRN
  361916, // llvm.riscv.cv.mac.macsN
  361940, // llvm.riscv.cv.mac.macsRN
  361965, // llvm.riscv.cv.mac.macuN
  361989, // llvm.riscv.cv.mac.macuRN
  362014, // llvm.riscv.cv.mac.msu
  362036, // llvm.riscv.cv.mac.mulhhsN
  362062, // llvm.riscv.cv.mac.mulhhsRN
  362089, // llvm.riscv.cv.mac.mulhhuN
  362115, // llvm.riscv.cv.mac.mulhhuRN
  362142, // llvm.riscv.cv.mac.mulsN
  362166, // llvm.riscv.cv.mac.mulsRN
  362191, // llvm.riscv.cv.mac.muluN
  362215, // llvm.riscv.cv.mac.muluRN
  362240, // llvm.riscv.masked.atomicrmw.add
  362272, // llvm.riscv.masked.atomicrmw.max
  362304, // llvm.riscv.masked.atomicrmw.min
  362336, // llvm.riscv.masked.atomicrmw.nand
  362369, // llvm.riscv.masked.atomicrmw.sub
  362401, // llvm.riscv.masked.atomicrmw.umax
  362434, // llvm.riscv.masked.atomicrmw.umin
  362467, // llvm.riscv.masked.atomicrmw.xchg
  362500, // llvm.riscv.masked.cmpxchg
  362526, // llvm.riscv.mips.ehb
  362546, // llvm.riscv.mips.ihb
  362566, // llvm.riscv.mips.pause
  362588, // llvm.riscv.mopr
  362604, // llvm.riscv.moprr
  362621, // llvm.riscv.nds.ffb
  362640, // llvm.riscv.nds.ffmism
  362662, // llvm.riscv.nds.ffzmism
  362685, // llvm.riscv.nds.flmism
  362707, // llvm.riscv.nds.vd4dots
  362730, // llvm.riscv.nds.vd4dots.mask
  362758, // llvm.riscv.nds.vd4dotsu
  362782, // llvm.riscv.nds.vd4dotsu.mask
  362811, // llvm.riscv.nds.vd4dotu
  362834, // llvm.riscv.nds.vd4dotu.mask
  362862, // llvm.riscv.nds.vfncvt.bf16.s
  362891, // llvm.riscv.nds.vfpmadb
  362914, // llvm.riscv.nds.vfpmadb.mask
  362942, // llvm.riscv.nds.vfpmadt
  362965, // llvm.riscv.nds.vfpmadt.mask
  362993, // llvm.riscv.nds.vfwcvt.s.bf16
  363022, // llvm.riscv.nds.vln
  363041, // llvm.riscv.nds.vln.mask
  363065, // llvm.riscv.nds.vlnu
  363085, // llvm.riscv.nds.vlnu.mask
  363110, // llvm.riscv.orc.b
  363127, // llvm.riscv.pause
  363144, // llvm.riscv.seg2.load.mask
  363170, // llvm.riscv.seg2.store.mask
  363197, // llvm.riscv.seg3.load.mask
  363223, // llvm.riscv.seg3.store.mask
  363250, // llvm.riscv.seg4.load.mask
  363276, // llvm.riscv.seg4.store.mask
  363303, // llvm.riscv.seg5.load.mask
  363329, // llvm.riscv.seg5.store.mask
  363356, // llvm.riscv.seg6.load.mask
  363382, // llvm.riscv.seg6.store.mask
  363409, // llvm.riscv.seg7.load.mask
  363435, // llvm.riscv.seg7.store.mask
  363462, // llvm.riscv.seg8.load.mask
  363488, // llvm.riscv.seg8.store.mask
  363515, // llvm.riscv.sf.mm.e4m3.e4m3
  363542, // llvm.riscv.sf.mm.e4m3.e5m2
  363569, // llvm.riscv.sf.mm.e5m2.e4m3
  363596, // llvm.riscv.sf.mm.e5m2.e5m2
  363623, // llvm.riscv.sf.mm.f.f
  363644, // llvm.riscv.sf.mm.s.s
  363665, // llvm.riscv.sf.mm.s.u
  363686, // llvm.riscv.sf.mm.u.s
  363707, // llvm.riscv.sf.mm.u.u
  363728, // llvm.riscv.sf.vc.fv.se
  363751, // llvm.riscv.sf.vc.fvv.se
  363775, // llvm.riscv.sf.vc.fvw.se
  363799, // llvm.riscv.sf.vc.i.se
  363821, // llvm.riscv.sf.vc.iv.se
  363844, // llvm.riscv.sf.vc.ivv.se
  363868, // llvm.riscv.sf.vc.ivw.se
  363892, // llvm.riscv.sf.vc.v.fv
  363914, // llvm.riscv.sf.vc.v.fv.se
  363939, // llvm.riscv.sf.vc.v.fvv
  363962, // llvm.riscv.sf.vc.v.fvv.se
  363988, // llvm.riscv.sf.vc.v.fvw
  364011, // llvm.riscv.sf.vc.v.fvw.se
  364037, // llvm.riscv.sf.vc.v.i
  364058, // llvm.riscv.sf.vc.v.i.se
  364082, // llvm.riscv.sf.vc.v.iv
  364104, // llvm.riscv.sf.vc.v.iv.se
  364129, // llvm.riscv.sf.vc.v.ivv
  364152, // llvm.riscv.sf.vc.v.ivv.se
  364178, // llvm.riscv.sf.vc.v.ivw
  364201, // llvm.riscv.sf.vc.v.ivw.se
  364227, // llvm.riscv.sf.vc.v.vv
  364249, // llvm.riscv.sf.vc.v.vv.se
  364274, // llvm.riscv.sf.vc.v.vvv
  364297, // llvm.riscv.sf.vc.v.vvv.se
  364323, // llvm.riscv.sf.vc.v.vvw
  364346, // llvm.riscv.sf.vc.v.vvw.se
  364372, // llvm.riscv.sf.vc.v.x
  364393, // llvm.riscv.sf.vc.v.x.se
  364417, // llvm.riscv.sf.vc.v.xv
  364439, // llvm.riscv.sf.vc.v.xv.se
  364464, // llvm.riscv.sf.vc.v.xvv
  364487, // llvm.riscv.sf.vc.v.xvv.se
  364513, // llvm.riscv.sf.vc.v.xvw
  364536, // llvm.riscv.sf.vc.v.xvw.se
  364562, // llvm.riscv.sf.vc.vv.se
  364585, // llvm.riscv.sf.vc.vvv.se
  364609, // llvm.riscv.sf.vc.vvw.se
  364633, // llvm.riscv.sf.vc.x.se
  364655, // llvm.riscv.sf.vc.xv.se
  364678, // llvm.riscv.sf.vc.xvv.se
  364702, // llvm.riscv.sf.vc.xvw.se
  364726, // llvm.riscv.sf.vfexp
  364746, // llvm.riscv.sf.vfexp.mask
  364771, // llvm.riscv.sf.vfexpa
  364792, // llvm.riscv.sf.vfexpa.mask
  364818, // llvm.riscv.sf.vfnrclip.x.f.qf
  364848, // llvm.riscv.sf.vfnrclip.x.f.qf.mask
  364883, // llvm.riscv.sf.vfnrclip.xu.f.qf
  364914, // llvm.riscv.sf.vfnrclip.xu.f.qf.mask
  364950, // llvm.riscv.sf.vfwmacc.4x4x4
  364978, // llvm.riscv.sf.vlte16
  364999, // llvm.riscv.sf.vlte32
  365020, // llvm.riscv.sf.vlte64
  365041, // llvm.riscv.sf.vlte8
  365061, // llvm.riscv.sf.vqmacc.2x8x2
  365088, // llvm.riscv.sf.vqmacc.4x8x4
  365115, // llvm.riscv.sf.vqmaccsu.2x8x2
  365144, // llvm.riscv.sf.vqmaccsu.4x8x4
  365173, // llvm.riscv.sf.vqmaccu.2x8x2
  365201, // llvm.riscv.sf.vqmaccu.4x8x4
  365229, // llvm.riscv.sf.vqmaccus.2x8x2
  365258, // llvm.riscv.sf.vqmaccus.4x8x4
  365287, // llvm.riscv.sf.vsettk
  365308, // llvm.riscv.sf.vsettm
  365329, // llvm.riscv.sf.vsettnt
  365351, // llvm.riscv.sf.vste16
  365372, // llvm.riscv.sf.vste32
  365393, // llvm.riscv.sf.vste64
  365414, // llvm.riscv.sf.vste8
  365434, // llvm.riscv.sf.vtdiscard
  365458, // llvm.riscv.sf.vtmv.t.v
  365481, // llvm.riscv.sf.vtmv.v.t
  365504, // llvm.riscv.sf.vtzero.t
  365527, // llvm.riscv.sha256sig0
  365549, // llvm.riscv.sha256sig1
  365571, // llvm.riscv.sha256sum0
  365593, // llvm.riscv.sha256sum1
  365615, // llvm.riscv.sha512sig0
  365637, // llvm.riscv.sha512sig0h
  365660, // llvm.riscv.sha512sig0l
  365683, // llvm.riscv.sha512sig1
  365705, // llvm.riscv.sha512sig1h
  365728, // llvm.riscv.sha512sig1l
  365751, // llvm.riscv.sha512sum0
  365773, // llvm.riscv.sha512sum0r
  365796, // llvm.riscv.sha512sum1
  365818, // llvm.riscv.sha512sum1r
  365841, // llvm.riscv.sm3p0
  365858, // llvm.riscv.sm3p1
  365875, // llvm.riscv.sm4ed
  365892, // llvm.riscv.sm4ks
  365909, // llvm.riscv.sseg2.load.mask
  365936, // llvm.riscv.sseg2.store.mask
  365964, // llvm.riscv.sseg3.load.mask
  365991, // llvm.riscv.sseg3.store.mask
  366019, // llvm.riscv.sseg4.load.mask
  366046, // llvm.riscv.sseg4.store.mask
  366074, // llvm.riscv.sseg5.load.mask
  366101, // llvm.riscv.sseg5.store.mask
  366129, // llvm.riscv.sseg6.load.mask
  366156, // llvm.riscv.sseg6.store.mask
  366184, // llvm.riscv.sseg7.load.mask
  366211, // llvm.riscv.sseg7.store.mask
  366239, // llvm.riscv.sseg8.load.mask
  366266, // llvm.riscv.sseg8.store.mask
  366294, // llvm.riscv.th.vmaqa
  366314, // llvm.riscv.th.vmaqa.mask
  366339, // llvm.riscv.th.vmaqasu
  366361, // llvm.riscv.th.vmaqasu.mask
  366388, // llvm.riscv.th.vmaqau
  366409, // llvm.riscv.th.vmaqau.mask
  366435, // llvm.riscv.th.vmaqaus
  366457, // llvm.riscv.th.vmaqaus.mask
  366484, // llvm.riscv.tuple.extract
  366509, // llvm.riscv.tuple.insert
  366533, // llvm.riscv.unzip
  366550, // llvm.riscv.vaadd
  366567, // llvm.riscv.vaadd.mask
  366589, // llvm.riscv.vaaddu
  366607, // llvm.riscv.vaaddu.mask
  366630, // llvm.riscv.vadc
  366646, // llvm.riscv.vadd
  366662, // llvm.riscv.vadd.mask
  366683, // llvm.riscv.vaesdf.vs
  366704, // llvm.riscv.vaesdf.vv
  366725, // llvm.riscv.vaesdm.vs
  366746, // llvm.riscv.vaesdm.vv
  366767, // llvm.riscv.vaesef.vs
  366788, // llvm.riscv.vaesef.vv
  366809, // llvm.riscv.vaesem.vs
  366830, // llvm.riscv.vaesem.vv
  366851, // llvm.riscv.vaeskf1
  366870, // llvm.riscv.vaeskf2
  366889, // llvm.riscv.vaesz.vs
  366909, // llvm.riscv.vand
  366925, // llvm.riscv.vand.mask
  366946, // llvm.riscv.vandn
  366963, // llvm.riscv.vandn.mask
  366985, // llvm.riscv.vasub
  367002, // llvm.riscv.vasub.mask
  367024, // llvm.riscv.vasubu
  367042, // llvm.riscv.vasubu.mask
  367065, // llvm.riscv.vbrev
  367082, // llvm.riscv.vbrev.mask
  367104, // llvm.riscv.vbrev8
  367122, // llvm.riscv.vbrev8.mask
  367145, // llvm.riscv.vclmul
  367163, // llvm.riscv.vclmul.mask
  367186, // llvm.riscv.vclmulh
  367205, // llvm.riscv.vclmulh.mask
  367229, // llvm.riscv.vclz
  367245, // llvm.riscv.vclz.mask
  367266, // llvm.riscv.vcompress
  367287, // llvm.riscv.vcpop
  367304, // llvm.riscv.vcpop.mask
  367326, // llvm.riscv.vcpopv
  367344, // llvm.riscv.vcpopv.mask
  367367, // llvm.riscv.vctz
  367383, // llvm.riscv.vctz.mask
  367404, // llvm.riscv.vdiv
  367420, // llvm.riscv.vdiv.mask
  367441, // llvm.riscv.vdivu
  367458, // llvm.riscv.vdivu.mask
  367480, // llvm.riscv.vfadd
  367497, // llvm.riscv.vfadd.mask
  367519, // llvm.riscv.vfclass
  367538, // llvm.riscv.vfclass.mask
  367562, // llvm.riscv.vfcvt.f.x.v
  367585, // llvm.riscv.vfcvt.f.x.v.mask
  367613, // llvm.riscv.vfcvt.f.xu.v
  367637, // llvm.riscv.vfcvt.f.xu.v.mask
  367666, // llvm.riscv.vfcvt.rtz.x.f.v
  367693, // llvm.riscv.vfcvt.rtz.x.f.v.mask
  367725, // llvm.riscv.vfcvt.rtz.xu.f.v
  367753, // llvm.riscv.vfcvt.rtz.xu.f.v.mask
  367786, // llvm.riscv.vfcvt.x.f.v
  367809, // llvm.riscv.vfcvt.x.f.v.mask
  367837, // llvm.riscv.vfcvt.xu.f.v
  367861, // llvm.riscv.vfcvt.xu.f.v.mask
  367890, // llvm.riscv.vfdiv
  367907, // llvm.riscv.vfdiv.mask
  367929, // llvm.riscv.vfirst
  367947, // llvm.riscv.vfirst.mask
  367970, // llvm.riscv.vfmacc
  367988, // llvm.riscv.vfmacc.mask
  368011, // llvm.riscv.vfmadd
  368029, // llvm.riscv.vfmadd.mask
  368052, // llvm.riscv.vfmax
  368069, // llvm.riscv.vfmax.mask
  368091, // llvm.riscv.vfmerge
  368110, // llvm.riscv.vfmin
  368127, // llvm.riscv.vfmin.mask
  368149, // llvm.riscv.vfmsac
  368167, // llvm.riscv.vfmsac.mask
  368190, // llvm.riscv.vfmsub
  368208, // llvm.riscv.vfmsub.mask
  368231, // llvm.riscv.vfmul
  368248, // llvm.riscv.vfmul.mask
  368270, // llvm.riscv.vfmv.f.s
  368290, // llvm.riscv.vfmv.s.f
  368310, // llvm.riscv.vfmv.v.f
  368330, // llvm.riscv.vfncvt.f.f.q
  368354, // llvm.riscv.vfncvt.f.f.q.alt
  368382, // llvm.riscv.vfncvt.f.f.q.alt.mask
  368415, // llvm.riscv.vfncvt.f.f.q.mask
  368444, // llvm.riscv.vfncvt.f.f.w
  368468, // llvm.riscv.vfncvt.f.f.w.alt
  368496, // llvm.riscv.vfncvt.f.f.w.alt.mask
  368529, // llvm.riscv.vfncvt.f.f.w.mask
  368558, // llvm.riscv.vfncvt.f.x.w
  368582, // llvm.riscv.vfncvt.f.x.w.mask
  368611, // llvm.riscv.vfncvt.f.xu.w
  368636, // llvm.riscv.vfncvt.f.xu.w.mask
  368666, // llvm.riscv.vfncvt.rod.f.f.w
  368694, // llvm.riscv.vfncvt.rod.f.f.w.mask
  368727, // llvm.riscv.vfncvt.rtz.x.f.w
  368755, // llvm.riscv.vfncvt.rtz.x.f.w.mask
  368788, // llvm.riscv.vfncvt.rtz.xu.f.w
  368817, // llvm.riscv.vfncvt.rtz.xu.f.w.mask
  368851, // llvm.riscv.vfncvt.sat.f.f.q
  368879, // llvm.riscv.vfncvt.sat.f.f.q.alt
  368911, // llvm.riscv.vfncvt.sat.f.f.q.alt.mask
  368948, // llvm.riscv.vfncvt.sat.f.f.q.mask
  368981, // llvm.riscv.vfncvt.sat.f.f.w
  369009, // llvm.riscv.vfncvt.sat.f.f.w.alt
  369041, // llvm.riscv.vfncvt.sat.f.f.w.alt.mask
  369078, // llvm.riscv.vfncvt.sat.f.f.w.mask
  369111, // llvm.riscv.vfncvt.x.f.w
  369135, // llvm.riscv.vfncvt.x.f.w.mask
  369164, // llvm.riscv.vfncvt.xu.f.w
  369189, // llvm.riscv.vfncvt.xu.f.w.mask
  369219, // llvm.riscv.vfncvtbf16.f.f.w
  369247, // llvm.riscv.vfncvtbf16.f.f.w.mask
  369280, // llvm.riscv.vfnmacc
  369299, // llvm.riscv.vfnmacc.mask
  369323, // llvm.riscv.vfnmadd
  369342, // llvm.riscv.vfnmadd.mask
  369366, // llvm.riscv.vfnmsac
  369385, // llvm.riscv.vfnmsac.mask
  369409, // llvm.riscv.vfnmsub
  369428, // llvm.riscv.vfnmsub.mask
  369452, // llvm.riscv.vfrdiv
  369470, // llvm.riscv.vfrdiv.mask
  369493, // llvm.riscv.vfrec7
  369511, // llvm.riscv.vfrec7.mask
  369534, // llvm.riscv.vfredmax
  369554, // llvm.riscv.vfredmax.mask
  369579, // llvm.riscv.vfredmin
  369599, // llvm.riscv.vfredmin.mask
  369624, // llvm.riscv.vfredosum
  369645, // llvm.riscv.vfredosum.mask
  369671, // llvm.riscv.vfredusum
  369692, // llvm.riscv.vfredusum.mask
  369718, // llvm.riscv.vfrsqrt7
  369738, // llvm.riscv.vfrsqrt7.mask
  369763, // llvm.riscv.vfrsub
  369781, // llvm.riscv.vfrsub.mask
  369804, // llvm.riscv.vfsgnj
  369822, // llvm.riscv.vfsgnj.mask
  369845, // llvm.riscv.vfsgnjn
  369864, // llvm.riscv.vfsgnjn.mask
  369888, // llvm.riscv.vfsgnjx
  369907, // llvm.riscv.vfsgnjx.mask
  369931, // llvm.riscv.vfslide1down
  369955, // llvm.riscv.vfslide1down.mask
  369984, // llvm.riscv.vfslide1up
  370006, // llvm.riscv.vfslide1up.mask
  370033, // llvm.riscv.vfsqrt
  370051, // llvm.riscv.vfsqrt.mask
  370074, // llvm.riscv.vfsub
  370091, // llvm.riscv.vfsub.mask
  370113, // llvm.riscv.vfwadd
  370131, // llvm.riscv.vfwadd.mask
  370154, // llvm.riscv.vfwadd.w
  370174, // llvm.riscv.vfwadd.w.mask
  370199, // llvm.riscv.vfwcvt.f.f.v
  370223, // llvm.riscv.vfwcvt.f.f.v.alt
  370251, // llvm.riscv.vfwcvt.f.f.v.alt.mask
  370284, // llvm.riscv.vfwcvt.f.f.v.mask
  370313, // llvm.riscv.vfwcvt.f.x.v
  370337, // llvm.riscv.vfwcvt.f.x.v.mask
  370366, // llvm.riscv.vfwcvt.f.xu.v
  370391, // llvm.riscv.vfwcvt.f.xu.v.mask
  370421, // llvm.riscv.vfwcvt.rtz.x.f.v
  370449, // llvm.riscv.vfwcvt.rtz.x.f.v.mask
  370482, // llvm.riscv.vfwcvt.rtz.xu.f.v
  370511, // llvm.riscv.vfwcvt.rtz.xu.f.v.mask
  370545, // llvm.riscv.vfwcvt.x.f.v
  370569, // llvm.riscv.vfwcvt.x.f.v.mask
  370598, // llvm.riscv.vfwcvt.xu.f.v
  370623, // llvm.riscv.vfwcvt.xu.f.v.mask
  370653, // llvm.riscv.vfwcvtbf16.f.f.v
  370681, // llvm.riscv.vfwcvtbf16.f.f.v.mask
  370714, // llvm.riscv.vfwmacc
  370733, // llvm.riscv.vfwmacc.mask
  370757, // llvm.riscv.vfwmaccbf16
  370780, // llvm.riscv.vfwmaccbf16.mask
  370808, // llvm.riscv.vfwmsac
  370827, // llvm.riscv.vfwmsac.mask
  370851, // llvm.riscv.vfwmul
  370869, // llvm.riscv.vfwmul.mask
  370892, // llvm.riscv.vfwnmacc
  370912, // llvm.riscv.vfwnmacc.mask
  370937, // llvm.riscv.vfwnmsac
  370957, // llvm.riscv.vfwnmsac.mask
  370982, // llvm.riscv.vfwredosum
  371004, // llvm.riscv.vfwredosum.mask
  371031, // llvm.riscv.vfwredusum
  371053, // llvm.riscv.vfwredusum.mask
  371080, // llvm.riscv.vfwsub
  371098, // llvm.riscv.vfwsub.mask
  371121, // llvm.riscv.vfwsub.w
  371141, // llvm.riscv.vfwsub.w.mask
  371166, // llvm.riscv.vghsh
  371183, // llvm.riscv.vgmul.vv
  371203, // llvm.riscv.vid
  371218, // llvm.riscv.vid.mask
  371238, // llvm.riscv.viota
  371255, // llvm.riscv.viota.mask
  371277, // llvm.riscv.vle
  371292, // llvm.riscv.vle.mask
  371312, // llvm.riscv.vleff
  371329, // llvm.riscv.vleff.mask
  371351, // llvm.riscv.vlm
  371366, // llvm.riscv.vloxei
  371384, // llvm.riscv.vloxei.mask
  371407, // llvm.riscv.vloxseg2
  371427, // llvm.riscv.vloxseg2.mask
  371452, // llvm.riscv.vloxseg3
  371472, // llvm.riscv.vloxseg3.mask
  371497, // llvm.riscv.vloxseg4
  371517, // llvm.riscv.vloxseg4.mask
  371542, // llvm.riscv.vloxseg5
  371562, // llvm.riscv.vloxseg5.mask
  371587, // llvm.riscv.vloxseg6
  371607, // llvm.riscv.vloxseg6.mask
  371632, // llvm.riscv.vloxseg7
  371652, // llvm.riscv.vloxseg7.mask
  371677, // llvm.riscv.vloxseg8
  371697, // llvm.riscv.vloxseg8.mask
  371722, // llvm.riscv.vlse
  371738, // llvm.riscv.vlse.mask
  371759, // llvm.riscv.vlseg2
  371777, // llvm.riscv.vlseg2.mask
  371800, // llvm.riscv.vlseg2ff
  371820, // llvm.riscv.vlseg2ff.mask
  371845, // llvm.riscv.vlseg3
  371863, // llvm.riscv.vlseg3.mask
  371886, // llvm.riscv.vlseg3ff
  371906, // llvm.riscv.vlseg3ff.mask
  371931, // llvm.riscv.vlseg4
  371949, // llvm.riscv.vlseg4.mask
  371972, // llvm.riscv.vlseg4ff
  371992, // llvm.riscv.vlseg4ff.mask
  372017, // llvm.riscv.vlseg5
  372035, // llvm.riscv.vlseg5.mask
  372058, // llvm.riscv.vlseg5ff
  372078, // llvm.riscv.vlseg5ff.mask
  372103, // llvm.riscv.vlseg6
  372121, // llvm.riscv.vlseg6.mask
  372144, // llvm.riscv.vlseg6ff
  372164, // llvm.riscv.vlseg6ff.mask
  372189, // llvm.riscv.vlseg7
  372207, // llvm.riscv.vlseg7.mask
  372230, // llvm.riscv.vlseg7ff
  372250, // llvm.riscv.vlseg7ff.mask
  372275, // llvm.riscv.vlseg8
  372293, // llvm.riscv.vlseg8.mask
  372316, // llvm.riscv.vlseg8ff
  372336, // llvm.riscv.vlseg8ff.mask
  372361, // llvm.riscv.vlsseg2
  372380, // llvm.riscv.vlsseg2.mask
  372404, // llvm.riscv.vlsseg3
  372423, // llvm.riscv.vlsseg3.mask
  372447, // llvm.riscv.vlsseg4
  372466, // llvm.riscv.vlsseg4.mask
  372490, // llvm.riscv.vlsseg5
  372509, // llvm.riscv.vlsseg5.mask
  372533, // llvm.riscv.vlsseg6
  372552, // llvm.riscv.vlsseg6.mask
  372576, // llvm.riscv.vlsseg7
  372595, // llvm.riscv.vlsseg7.mask
  372619, // llvm.riscv.vlsseg8
  372638, // llvm.riscv.vlsseg8.mask
  372662, // llvm.riscv.vluxei
  372680, // llvm.riscv.vluxei.mask
  372703, // llvm.riscv.vluxseg2
  372723, // llvm.riscv.vluxseg2.mask
  372748, // llvm.riscv.vluxseg3
  372768, // llvm.riscv.vluxseg3.mask
  372793, // llvm.riscv.vluxseg4
  372813, // llvm.riscv.vluxseg4.mask
  372838, // llvm.riscv.vluxseg5
  372858, // llvm.riscv.vluxseg5.mask
  372883, // llvm.riscv.vluxseg6
  372903, // llvm.riscv.vluxseg6.mask
  372928, // llvm.riscv.vluxseg7
  372948, // llvm.riscv.vluxseg7.mask
  372973, // llvm.riscv.vluxseg8
  372993, // llvm.riscv.vluxseg8.mask
  373018, // llvm.riscv.vmacc
  373035, // llvm.riscv.vmacc.mask
  373057, // llvm.riscv.vmadc
  373074, // llvm.riscv.vmadc.carry.in
  373100, // llvm.riscv.vmadd
  373117, // llvm.riscv.vmadd.mask
  373139, // llvm.riscv.vmand
  373156, // llvm.riscv.vmandn
  373174, // llvm.riscv.vmax
  373190, // llvm.riscv.vmax.mask
  373211, // llvm.riscv.vmaxu
  373228, // llvm.riscv.vmaxu.mask
  373250, // llvm.riscv.vmclr
  373267, // llvm.riscv.vmerge
  373285, // llvm.riscv.vmfeq
  373302, // llvm.riscv.vmfeq.mask
  373324, // llvm.riscv.vmfge
  373341, // llvm.riscv.vmfge.mask
  373363, // llvm.riscv.vmfgt
  373380, // llvm.riscv.vmfgt.mask
  373402, // llvm.riscv.vmfle
  373419, // llvm.riscv.vmfle.mask
  373441, // llvm.riscv.vmflt
  373458, // llvm.riscv.vmflt.mask
  373480, // llvm.riscv.vmfne
  373497, // llvm.riscv.vmfne.mask
  373519, // llvm.riscv.vmin
  373535, // llvm.riscv.vmin.mask
  373556, // llvm.riscv.vminu
  373573, // llvm.riscv.vminu.mask
  373595, // llvm.riscv.vmnand
  373613, // llvm.riscv.vmnor
  373630, // llvm.riscv.vmor
  373646, // llvm.riscv.vmorn
  373663, // llvm.riscv.vmsbc
  373680, // llvm.riscv.vmsbc.borrow.in
  373707, // llvm.riscv.vmsbf
  373724, // llvm.riscv.vmsbf.mask
  373746, // llvm.riscv.vmseq
  373763, // llvm.riscv.vmseq.mask
  373785, // llvm.riscv.vmset
  373802, // llvm.riscv.vmsge
  373819, // llvm.riscv.vmsge.mask
  373841, // llvm.riscv.vmsgeu
  373859, // llvm.riscv.vmsgeu.mask
  373882, // llvm.riscv.vmsgt
  373899, // llvm.riscv.vmsgt.mask
  373921, // llvm.riscv.vmsgtu
  373939, // llvm.riscv.vmsgtu.mask
  373962, // llvm.riscv.vmsif
  373979, // llvm.riscv.vmsif.mask
  374001, // llvm.riscv.vmsle
  374018, // llvm.riscv.vmsle.mask
  374040, // llvm.riscv.vmsleu
  374058, // llvm.riscv.vmsleu.mask
  374081, // llvm.riscv.vmslt
  374098, // llvm.riscv.vmslt.mask
  374120, // llvm.riscv.vmsltu
  374138, // llvm.riscv.vmsltu.mask
  374161, // llvm.riscv.vmsne
  374178, // llvm.riscv.vmsne.mask
  374200, // llvm.riscv.vmsof
  374217, // llvm.riscv.vmsof.mask
  374239, // llvm.riscv.vmul
  374255, // llvm.riscv.vmul.mask
  374276, // llvm.riscv.vmulh
  374293, // llvm.riscv.vmulh.mask
  374315, // llvm.riscv.vmulhsu
  374334, // llvm.riscv.vmulhsu.mask
  374358, // llvm.riscv.vmulhu
  374376, // llvm.riscv.vmulhu.mask
  374399, // llvm.riscv.vmv.s.x
  374418, // llvm.riscv.vmv.v.v
  374437, // llvm.riscv.vmv.v.x
  374456, // llvm.riscv.vmv.x.s
  374475, // llvm.riscv.vmxnor
  374493, // llvm.riscv.vmxor
  374510, // llvm.riscv.vnclip
  374528, // llvm.riscv.vnclip.mask
  374551, // llvm.riscv.vnclipu
  374570, // llvm.riscv.vnclipu.mask
  374594, // llvm.riscv.vnmsac
  374612, // llvm.riscv.vnmsac.mask
  374635, // llvm.riscv.vnmsub
  374653, // llvm.riscv.vnmsub.mask
  374676, // llvm.riscv.vnsra
  374693, // llvm.riscv.vnsra.mask
  374715, // llvm.riscv.vnsrl
  374732, // llvm.riscv.vnsrl.mask
  374754, // llvm.riscv.vor
  374769, // llvm.riscv.vor.mask
  374789, // llvm.riscv.vqdot
  374806, // llvm.riscv.vqdot.mask
  374828, // llvm.riscv.vqdotsu
  374847, // llvm.riscv.vqdotsu.mask
  374871, // llvm.riscv.vqdotu
  374889, // llvm.riscv.vqdotu.mask
  374912, // llvm.riscv.vqdotus
  374931, // llvm.riscv.vqdotus.mask
  374955, // llvm.riscv.vredand
  374974, // llvm.riscv.vredand.mask
  374998, // llvm.riscv.vredmax
  375017, // llvm.riscv.vredmax.mask
  375041, // llvm.riscv.vredmaxu
  375061, // llvm.riscv.vredmaxu.mask
  375086, // llvm.riscv.vredmin
  375105, // llvm.riscv.vredmin.mask
  375129, // llvm.riscv.vredminu
  375149, // llvm.riscv.vredminu.mask
  375174, // llvm.riscv.vredor
  375192, // llvm.riscv.vredor.mask
  375215, // llvm.riscv.vredsum
  375234, // llvm.riscv.vredsum.mask
  375258, // llvm.riscv.vredxor
  375277, // llvm.riscv.vredxor.mask
  375301, // llvm.riscv.vrem
  375317, // llvm.riscv.vrem.mask
  375338, // llvm.riscv.vremu
  375355, // llvm.riscv.vremu.mask
  375377, // llvm.riscv.vrev8
  375394, // llvm.riscv.vrev8.mask
  375416, // llvm.riscv.vrgather.vv
  375439, // llvm.riscv.vrgather.vv.mask
  375467, // llvm.riscv.vrgather.vx
  375490, // llvm.riscv.vrgather.vx.mask
  375518, // llvm.riscv.vrgatherei16.vv
  375545, // llvm.riscv.vrgatherei16.vv.mask
  375577, // llvm.riscv.vrol
  375593, // llvm.riscv.vrol.mask
  375614, // llvm.riscv.vror
  375630, // llvm.riscv.vror.mask
  375651, // llvm.riscv.vrsub
  375668, // llvm.riscv.vrsub.mask
  375690, // llvm.riscv.vsadd
  375707, // llvm.riscv.vsadd.mask
  375729, // llvm.riscv.vsaddu
  375747, // llvm.riscv.vsaddu.mask
  375770, // llvm.riscv.vsbc
  375786, // llvm.riscv.vse
  375801, // llvm.riscv.vse.mask
  375821, // llvm.riscv.vsetvli
  375840, // llvm.riscv.vsetvlimax
  375862, // llvm.riscv.vsext
  375879, // llvm.riscv.vsext.mask
  375901, // llvm.riscv.vsha2ch
  375920, // llvm.riscv.vsha2cl
  375939, // llvm.riscv.vsha2ms
  375958, // llvm.riscv.vslide1down
  375981, // llvm.riscv.vslide1down.mask
  376009, // llvm.riscv.vslide1up
  376030, // llvm.riscv.vslide1up.mask
  376056, // llvm.riscv.vslidedown
  376078, // llvm.riscv.vslidedown.mask
  376105, // llvm.riscv.vslideup
  376125, // llvm.riscv.vslideup.mask
  376150, // llvm.riscv.vsll
  376166, // llvm.riscv.vsll.mask
  376187, // llvm.riscv.vsm
  376202, // llvm.riscv.vsm3c
  376219, // llvm.riscv.vsm3me
  376237, // llvm.riscv.vsm4k
  376254, // llvm.riscv.vsm4r.vs
  376274, // llvm.riscv.vsm4r.vv
  376294, // llvm.riscv.vsmul
  376311, // llvm.riscv.vsmul.mask
  376333, // llvm.riscv.vsoxei
  376351, // llvm.riscv.vsoxei.mask
  376374, // llvm.riscv.vsoxseg2
  376394, // llvm.riscv.vsoxseg2.mask
  376419, // llvm.riscv.vsoxseg3
  376439, // llvm.riscv.vsoxseg3.mask
  376464, // llvm.riscv.vsoxseg4
  376484, // llvm.riscv.vsoxseg4.mask
  376509, // llvm.riscv.vsoxseg5
  376529, // llvm.riscv.vsoxseg5.mask
  376554, // llvm.riscv.vsoxseg6
  376574, // llvm.riscv.vsoxseg6.mask
  376599, // llvm.riscv.vsoxseg7
  376619, // llvm.riscv.vsoxseg7.mask
  376644, // llvm.riscv.vsoxseg8
  376664, // llvm.riscv.vsoxseg8.mask
  376689, // llvm.riscv.vsra
  376705, // llvm.riscv.vsra.mask
  376726, // llvm.riscv.vsrl
  376742, // llvm.riscv.vsrl.mask
  376763, // llvm.riscv.vsse
  376779, // llvm.riscv.vsse.mask
  376800, // llvm.riscv.vsseg2
  376818, // llvm.riscv.vsseg2.mask
  376841, // llvm.riscv.vsseg3
  376859, // llvm.riscv.vsseg3.mask
  376882, // llvm.riscv.vsseg4
  376900, // llvm.riscv.vsseg4.mask
  376923, // llvm.riscv.vsseg5
  376941, // llvm.riscv.vsseg5.mask
  376964, // llvm.riscv.vsseg6
  376982, // llvm.riscv.vsseg6.mask
  377005, // llvm.riscv.vsseg7
  377023, // llvm.riscv.vsseg7.mask
  377046, // llvm.riscv.vsseg8
  377064, // llvm.riscv.vsseg8.mask
  377087, // llvm.riscv.vssra
  377104, // llvm.riscv.vssra.mask
  377126, // llvm.riscv.vssrl
  377143, // llvm.riscv.vssrl.mask
  377165, // llvm.riscv.vssseg2
  377184, // llvm.riscv.vssseg2.mask
  377208, // llvm.riscv.vssseg3
  377227, // llvm.riscv.vssseg3.mask
  377251, // llvm.riscv.vssseg4
  377270, // llvm.riscv.vssseg4.mask
  377294, // llvm.riscv.vssseg5
  377313, // llvm.riscv.vssseg5.mask
  377337, // llvm.riscv.vssseg6
  377356, // llvm.riscv.vssseg6.mask
  377380, // llvm.riscv.vssseg7
  377399, // llvm.riscv.vssseg7.mask
  377423, // llvm.riscv.vssseg8
  377442, // llvm.riscv.vssseg8.mask
  377466, // llvm.riscv.vssub
  377483, // llvm.riscv.vssub.mask
  377505, // llvm.riscv.vssubu
  377523, // llvm.riscv.vssubu.mask
  377546, // llvm.riscv.vsub
  377562, // llvm.riscv.vsub.mask
  377583, // llvm.riscv.vsuxei
  377601, // llvm.riscv.vsuxei.mask
  377624, // llvm.riscv.vsuxseg2
  377644, // llvm.riscv.vsuxseg2.mask
  377669, // llvm.riscv.vsuxseg3
  377689, // llvm.riscv.vsuxseg3.mask
  377714, // llvm.riscv.vsuxseg4
  377734, // llvm.riscv.vsuxseg4.mask
  377759, // llvm.riscv.vsuxseg5
  377779, // llvm.riscv.vsuxseg5.mask
  377804, // llvm.riscv.vsuxseg6
  377824, // llvm.riscv.vsuxseg6.mask
  377849, // llvm.riscv.vsuxseg7
  377869, // llvm.riscv.vsuxseg7.mask
  377894, // llvm.riscv.vsuxseg8
  377914, // llvm.riscv.vsuxseg8.mask
  377939, // llvm.riscv.vwadd
  377956, // llvm.riscv.vwadd.mask
  377978, // llvm.riscv.vwadd.w
  377997, // llvm.riscv.vwadd.w.mask
  378021, // llvm.riscv.vwaddu
  378039, // llvm.riscv.vwaddu.mask
  378062, // llvm.riscv.vwaddu.w
  378082, // llvm.riscv.vwaddu.w.mask
  378107, // llvm.riscv.vwmacc
  378125, // llvm.riscv.vwmacc.mask
  378148, // llvm.riscv.vwmaccsu
  378168, // llvm.riscv.vwmaccsu.mask
  378193, // llvm.riscv.vwmaccu
  378212, // llvm.riscv.vwmaccu.mask
  378236, // llvm.riscv.vwmaccus
  378256, // llvm.riscv.vwmaccus.mask
  378281, // llvm.riscv.vwmul
  378298, // llvm.riscv.vwmul.mask
  378320, // llvm.riscv.vwmulsu
  378339, // llvm.riscv.vwmulsu.mask
  378363, // llvm.riscv.vwmulu
  378381, // llvm.riscv.vwmulu.mask
  378404, // llvm.riscv.vwredsum
  378424, // llvm.riscv.vwredsum.mask
  378449, // llvm.riscv.vwredsumu
  378470, // llvm.riscv.vwredsumu.mask
  378496, // llvm.riscv.vwsll
  378513, // llvm.riscv.vwsll.mask
  378535, // llvm.riscv.vwsub
  378552, // llvm.riscv.vwsub.mask
  378574, // llvm.riscv.vwsub.w
  378593, // llvm.riscv.vwsub.w.mask
  378617, // llvm.riscv.vwsubu
  378635, // llvm.riscv.vwsubu.mask
  378658, // llvm.riscv.vwsubu.w
  378678, // llvm.riscv.vwsubu.w.mask
  378703, // llvm.riscv.vxor
  378719, // llvm.riscv.vxor.mask
  378740, // llvm.riscv.vzext
  378757, // llvm.riscv.vzext.mask
  378779, // llvm.riscv.xperm4
  378797, // llvm.riscv.xperm8
  378815, // llvm.riscv.zip
  378830, // llvm.s390.bdepg
  378846, // llvm.s390.bextg
  378862, // llvm.s390.efpc
  378877, // llvm.s390.etnd
  378892, // llvm.s390.lcbb
  378907, // llvm.s390.ntstg
  378923, // llvm.s390.ppa.txassist
  378946, // llvm.s390.sfpc
  378961, // llvm.s390.tabort
  378978, // llvm.s390.tbegin
  378995, // llvm.s390.tbegin.nofloat
  379020, // llvm.s390.tbeginc
  379038, // llvm.s390.tdc
  379052, // llvm.s390.tend
  379067, // llvm.s390.vaccb
  379083, // llvm.s390.vacccq
  379100, // llvm.s390.vaccf
  379116, // llvm.s390.vaccg
  379132, // llvm.s390.vacch
  379148, // llvm.s390.vaccq
  379164, // llvm.s390.vacq
  379179, // llvm.s390.vaq
  379193, // llvm.s390.vavgb
  379209, // llvm.s390.vavgf
  379225, // llvm.s390.vavgg
  379241, // llvm.s390.vavgh
  379257, // llvm.s390.vavglb
  379274, // llvm.s390.vavglf
  379291, // llvm.s390.vavglg
  379308, // llvm.s390.vavglh
  379325, // llvm.s390.vavglq
  379342, // llvm.s390.vavgq
  379358, // llvm.s390.vbperm
  379375, // llvm.s390.vceqbs
  379392, // llvm.s390.vceqfs
  379409, // llvm.s390.vceqgs
  379426, // llvm.s390.vceqhs
  379443, // llvm.s390.vceqqs
  379460, // llvm.s390.vcfn
  379475, // llvm.s390.vchbs
  379491, // llvm.s390.vchfs
  379507, // llvm.s390.vchgs
  379523, // llvm.s390.vchhs
  379539, // llvm.s390.vchlbs
  379556, // llvm.s390.vchlfs
  379573, // llvm.s390.vchlgs
  379590, // llvm.s390.vchlhs
  379607, // llvm.s390.vchlqs
  379624, // llvm.s390.vchqs
  379640, // llvm.s390.vcksm
  379656, // llvm.s390.vclfnhs
  379674, // llvm.s390.vclfnls
  379692, // llvm.s390.vcnf
  379707, // llvm.s390.vcrnfs
  379724, // llvm.s390.verimb
  379741, // llvm.s390.verimf
  379758, // llvm.s390.verimg
  379775, // llvm.s390.verimh
  379792, // llvm.s390.veval
  379808, // llvm.s390.vfaeb
  379824, // llvm.s390.vfaebs
  379841, // llvm.s390.vfaef
  379857, // llvm.s390.vfaefs
  379874, // llvm.s390.vfaeh
  379890, // llvm.s390.vfaehs
  379907, // llvm.s390.vfaezb
  379924, // llvm.s390.vfaezbs
  379942, // llvm.s390.vfaezf
  379959, // llvm.s390.vfaezfs
  379977, // llvm.s390.vfaezh
  379994, // llvm.s390.vfaezhs
  380012, // llvm.s390.vfcedbs
  380030, // llvm.s390.vfcesbs
  380048, // llvm.s390.vfchdbs
  380066, // llvm.s390.vfchedbs
  380085, // llvm.s390.vfchesbs
  380104, // llvm.s390.vfchsbs
  380122, // llvm.s390.vfeeb
  380138, // llvm.s390.vfeebs
  380155, // llvm.s390.vfeef
  380171, // llvm.s390.vfeefs
  380188, // llvm.s390.vfeeh
  380204, // llvm.s390.vfeehs
  380221, // llvm.s390.vfeezb
  380238, // llvm.s390.vfeezbs
  380256, // llvm.s390.vfeezf
  380273, // llvm.s390.vfeezfs
  380291, // llvm.s390.vfeezh
  380308, // llvm.s390.vfeezhs
  380326, // llvm.s390.vfeneb
  380343, // llvm.s390.vfenebs
  380361, // llvm.s390.vfenef
  380378, // llvm.s390.vfenefs
  380396, // llvm.s390.vfeneh
  380413, // llvm.s390.vfenehs
  380431, // llvm.s390.vfenezb
  380449, // llvm.s390.vfenezbs
  380468, // llvm.s390.vfenezf
  380486, // llvm.s390.vfenezfs
  380505, // llvm.s390.vfenezh
  380523, // llvm.s390.vfenezhs
  380542, // llvm.s390.vfidb
  380558, // llvm.s390.vfisb
  380574, // llvm.s390.vfmaxdb
  380592, // llvm.s390.vfmaxsb
  380610, // llvm.s390.vfmindb
  380628, // llvm.s390.vfminsb
  380646, // llvm.s390.vftcidb
  380664, // llvm.s390.vftcisb
  380682, // llvm.s390.vgemb
  380698, // llvm.s390.vgemf
  380714, // llvm.s390.vgemg
  380730, // llvm.s390.vgemh
  380746, // llvm.s390.vgemq
  380762, // llvm.s390.vgfmab
  380779, // llvm.s390.vgfmaf
  380796, // llvm.s390.vgfmag
  380813, // llvm.s390.vgfmah
  380830, // llvm.s390.vgfmb
  380846, // llvm.s390.vgfmf
  380862, // llvm.s390.vgfmg
  380878, // llvm.s390.vgfmh
  380894, // llvm.s390.vistrb
  380911, // llvm.s390.vistrbs
  380929, // llvm.s390.vistrf
  380946, // llvm.s390.vistrfs
  380964, // llvm.s390.vistrh
  380981, // llvm.s390.vistrhs
  380999, // llvm.s390.vlbb
  381014, // llvm.s390.vll
  381028, // llvm.s390.vlrl
  381043, // llvm.s390.vmaeb
  381059, // llvm.s390.vmaef
  381075, // llvm.s390.vmaeg
  381091, // llvm.s390.vmaeh
  381107, // llvm.s390.vmahb
  381123, // llvm.s390.vmahf
  381139, // llvm.s390.vmahg
  381155, // llvm.s390.vmahh
  381171, // llvm.s390.vmahq
  381187, // llvm.s390.vmaleb
  381204, // llvm.s390.vmalef
  381221, // llvm.s390.vmaleg
  381238, // llvm.s390.vmaleh
  381255, // llvm.s390.vmalhb
  381272, // llvm.s390.vmalhf
  381289, // llvm.s390.vmalhg
  381306, // llvm.s390.vmalhh
  381323, // llvm.s390.vmalhq
  381340, // llvm.s390.vmalob
  381357, // llvm.s390.vmalof
  381374, // llvm.s390.vmalog
  381391, // llvm.s390.vmaloh
  381408, // llvm.s390.vmaob
  381424, // llvm.s390.vmaof
  381440, // llvm.s390.vmaog
  381456, // llvm.s390.vmaoh
  381472, // llvm.s390.vmeb
  381487, // llvm.s390.vmef
  381502, // llvm.s390.vmeg
  381517, // llvm.s390.vmeh
  381532, // llvm.s390.vmhb
  381547, // llvm.s390.vmhf
  381562, // llvm.s390.vmhg
  381577, // llvm.s390.vmhh
  381592, // llvm.s390.vmhq
  381607, // llvm.s390.vmleb
  381623, // llvm.s390.vmlef
  381639, // llvm.s390.vmleg
  381655, // llvm.s390.vmleh
  381671, // llvm.s390.vmlhb
  381687, // llvm.s390.vmlhf
  381703, // llvm.s390.vmlhg
  381719, // llvm.s390.vmlhh
  381735, // llvm.s390.vmlhq
  381751, // llvm.s390.vmlob
  381767, // llvm.s390.vmlof
  381783, // llvm.s390.vmlog
  381799, // llvm.s390.vmloh
  381815, // llvm.s390.vmob
  381830, // llvm.s390.vmof
  381845, // llvm.s390.vmog
  381860, // llvm.s390.vmoh
  381875, // llvm.s390.vmslg
  381891, // llvm.s390.vpdi
  381906, // llvm.s390.vperm
  381922, // llvm.s390.vpklsf
  381939, // llvm.s390.vpklsfs
  381957, // llvm.s390.vpklsg
  381974, // llvm.s390.vpklsgs
  381992, // llvm.s390.vpklsh
  382009, // llvm.s390.vpklshs
  382027, // llvm.s390.vpksf
  382043, // llvm.s390.vpksfs
  382060, // llvm.s390.vpksg
  382076, // llvm.s390.vpksgs
  382093, // llvm.s390.vpksh
  382109, // llvm.s390.vpkshs
  382126, // llvm.s390.vsbcbiq
  382144, // llvm.s390.vsbiq
  382160, // llvm.s390.vscbib
  382177, // llvm.s390.vscbif
  382194, // llvm.s390.vscbig
  382211, // llvm.s390.vscbih
  382228, // llvm.s390.vscbiq
  382245, // llvm.s390.vsl
  382259, // llvm.s390.vslb
  382274, // llvm.s390.vsld
  382289, // llvm.s390.vsldb
  382305, // llvm.s390.vsq
  382319, // llvm.s390.vsra
  382334, // llvm.s390.vsrab
  382350, // llvm.s390.vsrd
  382365, // llvm.s390.vsrl
  382380, // llvm.s390.vsrlb
  382396, // llvm.s390.vstl
  382411, // llvm.s390.vstrcb
  382428, // llvm.s390.vstrcbs
  382446, // llvm.s390.vstrcf
  382463, // llvm.s390.vstrcfs
  382481, // llvm.s390.vstrch
  382498, // llvm.s390.vstrchs
  382516, // llvm.s390.vstrczb
  382534, // llvm.s390.vstrczbs
  382553, // llvm.s390.vstrczf
  382571, // llvm.s390.vstrczfs
  382590, // llvm.s390.vstrczh
  382608, // llvm.s390.vstrczhs
  382627, // llvm.s390.vstrl
  382643, // llvm.s390.vstrsb
  382660, // llvm.s390.vstrsf
  382677, // llvm.s390.vstrsh
  382694, // llvm.s390.vstrszb
  382712, // llvm.s390.vstrszf
  382730, // llvm.s390.vstrszh
  382748, // llvm.s390.vsumb
  382764, // llvm.s390.vsumgf
  382781, // llvm.s390.vsumgh
  382798, // llvm.s390.vsumh
  382814, // llvm.s390.vsumqf
  382831, // llvm.s390.vsumqg
  382848, // llvm.s390.vtm
  382862, // llvm.s390.vuphb
  382878, // llvm.s390.vuphf
  382894, // llvm.s390.vuphg
  382910, // llvm.s390.vuphh
  382926, // llvm.s390.vuplb
  382942, // llvm.s390.vuplf
  382958, // llvm.s390.vuplg
  382974, // llvm.s390.vuplhb
  382991, // llvm.s390.vuplhf
  383008, // llvm.s390.vuplhg
  383025, // llvm.s390.vuplhh
  383042, // llvm.s390.vuplhw
  383059, // llvm.s390.vupllb
  383076, // llvm.s390.vupllf
  383093, // llvm.s390.vupllg
  383110, // llvm.s390.vupllh
  383127, // llvm.spv.all
  383140, // llvm.spv.alloca
  383156, // llvm.spv.alloca.array
  383178, // llvm.spv.any
  383191, // llvm.spv.assign.aliasing.decoration
  383227, // llvm.spv.assign.decoration
  383254, // llvm.spv.assign.fpmaxerror.decoration
  383292, // llvm.spv.assign.name
  383313, // llvm.spv.assign.ptr.type
  383338, // llvm.spv.assign.type
  383359, // llvm.spv.assume
  383375, // llvm.spv.bitcast
  383392, // llvm.spv.cmpxchg
  383409, // llvm.spv.const.composite
  383434, // llvm.spv.cross
  383449, // llvm.spv.ddx
  383462, // llvm.spv.ddx.coarse
  383482, // llvm.spv.ddx.fine
  383500, // llvm.spv.ddy
  383513, // llvm.spv.ddy.coarse
  383533, // llvm.spv.ddy.fine
  383551, // llvm.spv.degrees
  383568, // llvm.spv.discard
  383585, // llvm.spv.distance
  383603, // llvm.spv.dot4add.i8packed
  383629, // llvm.spv.dot4add.u8packed
  383655, // llvm.spv.expect
  383671, // llvm.spv.extractelt
  383691, // llvm.spv.extractv
  383709, // llvm.spv.faceforward
  383730, // llvm.spv.fdot
  383744, // llvm.spv.firstbitlow
  383765, // llvm.spv.firstbitshigh
  383788, // llvm.spv.firstbituhigh
  383811, // llvm.spv.flattened.thread.id.in.group
  383849, // llvm.spv.frac
  383863, // llvm.spv.fwidth
  383879, // llvm.spv.generic.cast.to.ptr.explicit
  383917, // llvm.spv.gep
  383930, // llvm.spv.global.offset
  383953, // llvm.spv.global.size
  383974, // llvm.spv.group.id
  383992, // llvm.spv.group.memory.barrier.with.group.sync
  384038, // llvm.spv.init.global
  384059, // llvm.spv.inline.asm
  384079, // llvm.spv.insertelt
  384098, // llvm.spv.insertv
  384115, // llvm.spv.isinf
  384130, // llvm.spv.isnan
  384145, // llvm.spv.length
  384161, // llvm.spv.lerp
  384175, // llvm.spv.lifetime.end
  384197, // llvm.spv.lifetime.start
  384221, // llvm.spv.load
  384235, // llvm.spv.loop.merge
  384255, // llvm.spv.nclamp
  384271, // llvm.spv.normalize
  384290, // llvm.spv.num.subgroups
  384313, // llvm.spv.num.workgroups
  384337, // llvm.spv.ptrcast
  384354, // llvm.spv.pushconstant.getpointer
  384387, // llvm.spv.radians
  384404, // llvm.spv.reflect
  384421, // llvm.spv.refract
  384438, // llvm.spv.resource.counterhandlefrombinding
  384481, // llvm.spv.resource.counterhandlefromimplicitbinding
  384532, // llvm.spv.resource.getdimensions.x
  384566, // llvm.spv.resource.getpointer
  384595, // llvm.spv.resource.handlefrombinding
  384631, // llvm.spv.resource.handlefromimplicitbinding
  384675, // llvm.spv.resource.load.typedbuffer
  384710, // llvm.spv.resource.nonuniformindex
  384744, // llvm.spv.resource.store.typedbuffer
  384780, // llvm.spv.resource.updatecounter
  384812, // llvm.spv.rsqrt
  384827, // llvm.spv.saturate
  384845, // llvm.spv.sclamp
  384861, // llvm.spv.sdot
  384875, // llvm.spv.selection.merge
  384900, // llvm.spv.sign
  384914, // llvm.spv.smoothstep
  384934, // llvm.spv.step
  384948, // llvm.spv.store
  384963, // llvm.spv.subgroup.id
  384984, // llvm.spv.subgroup.local.invocation.id
  385022, // llvm.spv.subgroup.max.size
  385049, // llvm.spv.subgroup.size
  385072, // llvm.spv.switch
  385088, // llvm.spv.thread.id
  385107, // llvm.spv.thread.id.in.group
  385135, // llvm.spv.track.constant
  385159, // llvm.spv.uclamp
  385175, // llvm.spv.udot
  385189, // llvm.spv.undef
  385204, // llvm.spv.unpackhalf2x16
  385228, // llvm.spv.unreachable
  385249, // llvm.spv.unref.global
  385271, // llvm.spv.value.md
  385289, // llvm.spv.wave.active.countbits
  385320, // llvm.spv.wave.all
  385338, // llvm.spv.wave.any
  385356, // llvm.spv.wave.ballot
  385377, // llvm.spv.wave.get.lane.count
  385406, // llvm.spv.wave.is.first.lane
  385434, // llvm.spv.wave.readlane
  385457, // llvm.spv.wave.reduce.max
  385482, // llvm.spv.wave.reduce.min
  385507, // llvm.spv.wave.reduce.sum
  385532, // llvm.spv.wave.reduce.umax
  385558, // llvm.spv.wave.reduce.umin
  385584, // llvm.spv.workgroup.size
  385608, // llvm.ve.vl.andm.MMM
  385628, // llvm.ve.vl.andm.mmm
  385648, // llvm.ve.vl.eqvm.MMM
  385668, // llvm.ve.vl.eqvm.mmm
  385688, // llvm.ve.vl.extract.vm512l
  385714, // llvm.ve.vl.extract.vm512u
  385740, // llvm.ve.vl.fencec.s
  385760, // llvm.ve.vl.fencei
  385778, // llvm.ve.vl.fencem.s
  385798, // llvm.ve.vl.fidcr.sss
  385819, // llvm.ve.vl.insert.vm512l
  385844, // llvm.ve.vl.insert.vm512u
  385869, // llvm.ve.vl.lcr.sss
  385888, // llvm.ve.vl.lsv.vvss
  385908, // llvm.ve.vl.lvm.MMss
  385928, // llvm.ve.vl.lvm.mmss
  385948, // llvm.ve.vl.lvsd.svs
  385968, // llvm.ve.vl.lvsl.svs
  385988, // llvm.ve.vl.lvss.svs
  386008, // llvm.ve.vl.lzvm.sml
  386028, // llvm.ve.vl.negm.MM
  386047, // llvm.ve.vl.negm.mm
  386066, // llvm.ve.vl.nndm.MMM
  386086, // llvm.ve.vl.nndm.mmm
  386106, // llvm.ve.vl.orm.MMM
  386125, // llvm.ve.vl.orm.mmm
  386144, // llvm.ve.vl.pack.f32a
  386165, // llvm.ve.vl.pack.f32p
  386186, // llvm.ve.vl.pcvm.sml
  386206, // llvm.ve.vl.pfchv.ssl
  386227, // llvm.ve.vl.pfchvnc.ssl
  386250, // llvm.ve.vl.pvadds.vsvMvl
  386275, // llvm.ve.vl.pvadds.vsvl
  386298, // llvm.ve.vl.pvadds.vsvvl
  386322, // llvm.ve.vl.pvadds.vvvMvl
  386347, // llvm.ve.vl.pvadds.vvvl
  386370, // llvm.ve.vl.pvadds.vvvvl
  386394, // llvm.ve.vl.pvaddu.vsvMvl
  386419, // llvm.ve.vl.pvaddu.vsvl
  386442, // llvm.ve.vl.pvaddu.vsvvl
  386466, // llvm.ve.vl.pvaddu.vvvMvl
  386491, // llvm.ve.vl.pvaddu.vvvl
  386514, // llvm.ve.vl.pvaddu.vvvvl
  386538, // llvm.ve.vl.pvand.vsvMvl
  386562, // llvm.ve.vl.pvand.vsvl
  386584, // llvm.ve.vl.pvand.vsvvl
  386607, // llvm.ve.vl.pvand.vvvMvl
  386631, // llvm.ve.vl.pvand.vvvl
  386653, // llvm.ve.vl.pvand.vvvvl
  386676, // llvm.ve.vl.pvbrd.vsMvl
  386699, // llvm.ve.vl.pvbrd.vsl
  386720, // llvm.ve.vl.pvbrd.vsvl
  386742, // llvm.ve.vl.pvbrv.vvMvl
  386765, // llvm.ve.vl.pvbrv.vvl
  386786, // llvm.ve.vl.pvbrv.vvvl
  386808, // llvm.ve.vl.pvbrvlo.vvl
  386831, // llvm.ve.vl.pvbrvlo.vvmvl
  386856, // llvm.ve.vl.pvbrvlo.vvvl
  386880, // llvm.ve.vl.pvbrvup.vvl
  386903, // llvm.ve.vl.pvbrvup.vvmvl
  386928, // llvm.ve.vl.pvbrvup.vvvl
  386952, // llvm.ve.vl.pvcmps.vsvMvl
  386977, // llvm.ve.vl.pvcmps.vsvl
  387000, // llvm.ve.vl.pvcmps.vsvvl
  387024, // llvm.ve.vl.pvcmps.vvvMvl
  387049, // llvm.ve.vl.pvcmps.vvvl
  387072, // llvm.ve.vl.pvcmps.vvvvl
  387096, // llvm.ve.vl.pvcmpu.vsvMvl
  387121, // llvm.ve.vl.pvcmpu.vsvl
  387144, // llvm.ve.vl.pvcmpu.vsvvl
  387168, // llvm.ve.vl.pvcmpu.vvvMvl
  387193, // llvm.ve.vl.pvcmpu.vvvl
  387216, // llvm.ve.vl.pvcmpu.vvvvl
  387240, // llvm.ve.vl.pvcvtsw.vvl
  387263, // llvm.ve.vl.pvcvtsw.vvvl
  387287, // llvm.ve.vl.pvcvtws.vvMvl
  387312, // llvm.ve.vl.pvcvtws.vvl
  387335, // llvm.ve.vl.pvcvtws.vvvl
  387359, // llvm.ve.vl.pvcvtwsrz.vvMvl
  387386, // llvm.ve.vl.pvcvtwsrz.vvl
  387411, // llvm.ve.vl.pvcvtwsrz.vvvl
  387437, // llvm.ve.vl.pveqv.vsvMvl
  387461, // llvm.ve.vl.pveqv.vsvl
  387483, // llvm.ve.vl.pveqv.vsvvl
  387506, // llvm.ve.vl.pveqv.vvvMvl
  387530, // llvm.ve.vl.pveqv.vvvl
  387552, // llvm.ve.vl.pveqv.vvvvl
  387575, // llvm.ve.vl.pvfadd.vsvMvl
  387600, // llvm.ve.vl.pvfadd.vsvl
  387623, // llvm.ve.vl.pvfadd.vsvvl
  387647, // llvm.ve.vl.pvfadd.vvvMvl
  387672, // llvm.ve.vl.pvfadd.vvvl
  387695, // llvm.ve.vl.pvfadd.vvvvl
  387719, // llvm.ve.vl.pvfcmp.vsvMvl
  387744, // llvm.ve.vl.pvfcmp.vsvl
  387767, // llvm.ve.vl.pvfcmp.vsvvl
  387791, // llvm.ve.vl.pvfcmp.vvvMvl
  387816, // llvm.ve.vl.pvfcmp.vvvl
  387839, // llvm.ve.vl.pvfcmp.vvvvl
  387863, // llvm.ve.vl.pvfmad.vsvvMvl
  387889, // llvm.ve.vl.pvfmad.vsvvl
  387913, // llvm.ve.vl.pvfmad.vsvvvl
  387938, // llvm.ve.vl.pvfmad.vvsvMvl
  387964, // llvm.ve.vl.pvfmad.vvsvl
  387988, // llvm.ve.vl.pvfmad.vvsvvl
  388013, // llvm.ve.vl.pvfmad.vvvvMvl
  388039, // llvm.ve.vl.pvfmad.vvvvl
  388063, // llvm.ve.vl.pvfmad.vvvvvl
  388088, // llvm.ve.vl.pvfmax.vsvMvl
  388113, // llvm.ve.vl.pvfmax.vsvl
  388136, // llvm.ve.vl.pvfmax.vsvvl
  388160, // llvm.ve.vl.pvfmax.vvvMvl
  388185, // llvm.ve.vl.pvfmax.vvvl
  388208, // llvm.ve.vl.pvfmax.vvvvl
  388232, // llvm.ve.vl.pvfmin.vsvMvl
  388257, // llvm.ve.vl.pvfmin.vsvl
  388280, // llvm.ve.vl.pvfmin.vsvvl
  388304, // llvm.ve.vl.pvfmin.vvvMvl
  388329, // llvm.ve.vl.pvfmin.vvvl
  388352, // llvm.ve.vl.pvfmin.vvvvl
  388376, // llvm.ve.vl.pvfmkaf.Ml
  388398, // llvm.ve.vl.pvfmkat.Ml
  388420, // llvm.ve.vl.pvfmkseq.MvMl
  388445, // llvm.ve.vl.pvfmkseq.Mvl
  388469, // llvm.ve.vl.pvfmkseqnan.MvMl
  388497, // llvm.ve.vl.pvfmkseqnan.Mvl
  388524, // llvm.ve.vl.pvfmksge.MvMl
  388549, // llvm.ve.vl.pvfmksge.Mvl
  388573, // llvm.ve.vl.pvfmksgenan.MvMl
  388601, // llvm.ve.vl.pvfmksgenan.Mvl
  388628, // llvm.ve.vl.pvfmksgt.MvMl
  388653, // llvm.ve.vl.pvfmksgt.Mvl
  388677, // llvm.ve.vl.pvfmksgtnan.MvMl
  388705, // llvm.ve.vl.pvfmksgtnan.Mvl
  388732, // llvm.ve.vl.pvfmksle.MvMl
  388757, // llvm.ve.vl.pvfmksle.Mvl
  388781, // llvm.ve.vl.pvfmkslenan.MvMl
  388809, // llvm.ve.vl.pvfmkslenan.Mvl
  388836, // llvm.ve.vl.pvfmksloeq.mvl
  388862, // llvm.ve.vl.pvfmksloeq.mvml
  388889, // llvm.ve.vl.pvfmksloeqnan.mvl
  388918, // llvm.ve.vl.pvfmksloeqnan.mvml
  388948, // llvm.ve.vl.pvfmksloge.mvl
  388974, // llvm.ve.vl.pvfmksloge.mvml
  389001, // llvm.ve.vl.pvfmkslogenan.mvl
  389030, // llvm.ve.vl.pvfmkslogenan.mvml
  389060, // llvm.ve.vl.pvfmkslogt.mvl
  389086, // llvm.ve.vl.pvfmkslogt.mvml
  389113, // llvm.ve.vl.pvfmkslogtnan.mvl
  389142, // llvm.ve.vl.pvfmkslogtnan.mvml
  389172, // llvm.ve.vl.pvfmkslole.mvl
  389198, // llvm.ve.vl.pvfmkslole.mvml
  389225, // llvm.ve.vl.pvfmkslolenan.mvl
  389254, // llvm.ve.vl.pvfmkslolenan.mvml
  389284, // llvm.ve.vl.pvfmkslolt.mvl
  389310, // llvm.ve.vl.pvfmkslolt.mvml
  389337, // llvm.ve.vl.pvfmksloltnan.mvl
  389366, // llvm.ve.vl.pvfmksloltnan.mvml
  389396, // llvm.ve.vl.pvfmkslonan.mvl
  389423, // llvm.ve.vl.pvfmkslonan.mvml
  389451, // llvm.ve.vl.pvfmkslone.mvl
  389477, // llvm.ve.vl.pvfmkslone.mvml
  389504, // llvm.ve.vl.pvfmkslonenan.mvl
  389533, // llvm.ve.vl.pvfmkslonenan.mvml
  389563, // llvm.ve.vl.pvfmkslonum.mvl
  389590, // llvm.ve.vl.pvfmkslonum.mvml
  389618, // llvm.ve.vl.pvfmkslt.MvMl
  389643, // llvm.ve.vl.pvfmkslt.Mvl
  389667, // llvm.ve.vl.pvfmksltnan.MvMl
  389695, // llvm.ve.vl.pvfmksltnan.Mvl
  389722, // llvm.ve.vl.pvfmksnan.MvMl
  389748, // llvm.ve.vl.pvfmksnan.Mvl
  389773, // llvm.ve.vl.pvfmksne.MvMl
  389798, // llvm.ve.vl.pvfmksne.Mvl
  389822, // llvm.ve.vl.pvfmksnenan.MvMl
  389850, // llvm.ve.vl.pvfmksnenan.Mvl
  389877, // llvm.ve.vl.pvfmksnum.MvMl
  389903, // llvm.ve.vl.pvfmksnum.Mvl
  389928, // llvm.ve.vl.pvfmksupeq.mvl
  389954, // llvm.ve.vl.pvfmksupeq.mvml
  389981, // llvm.ve.vl.pvfmksupeqnan.mvl
  390010, // llvm.ve.vl.pvfmksupeqnan.mvml
  390040, // llvm.ve.vl.pvfmksupge.mvl
  390066, // llvm.ve.vl.pvfmksupge.mvml
  390093, // llvm.ve.vl.pvfmksupgenan.mvl
  390122, // llvm.ve.vl.pvfmksupgenan.mvml
  390152, // llvm.ve.vl.pvfmksupgt.mvl
  390178, // llvm.ve.vl.pvfmksupgt.mvml
  390205, // llvm.ve.vl.pvfmksupgtnan.mvl
  390234, // llvm.ve.vl.pvfmksupgtnan.mvml
  390264, // llvm.ve.vl.pvfmksuple.mvl
  390290, // llvm.ve.vl.pvfmksuple.mvml
  390317, // llvm.ve.vl.pvfmksuplenan.mvl
  390346, // llvm.ve.vl.pvfmksuplenan.mvml
  390376, // llvm.ve.vl.pvfmksuplt.mvl
  390402, // llvm.ve.vl.pvfmksuplt.mvml
  390429, // llvm.ve.vl.pvfmksupltnan.mvl
  390458, // llvm.ve.vl.pvfmksupltnan.mvml
  390488, // llvm.ve.vl.pvfmksupnan.mvl
  390515, // llvm.ve.vl.pvfmksupnan.mvml
  390543, // llvm.ve.vl.pvfmksupne.mvl
  390569, // llvm.ve.vl.pvfmksupne.mvml
  390596, // llvm.ve.vl.pvfmksupnenan.mvl
  390625, // llvm.ve.vl.pvfmksupnenan.mvml
  390655, // llvm.ve.vl.pvfmksupnum.mvl
  390682, // llvm.ve.vl.pvfmksupnum.mvml
  390710, // llvm.ve.vl.pvfmkweq.MvMl
  390735, // llvm.ve.vl.pvfmkweq.Mvl
  390759, // llvm.ve.vl.pvfmkweqnan.MvMl
  390787, // llvm.ve.vl.pvfmkweqnan.Mvl
  390814, // llvm.ve.vl.pvfmkwge.MvMl
  390839, // llvm.ve.vl.pvfmkwge.Mvl
  390863, // llvm.ve.vl.pvfmkwgenan.MvMl
  390891, // llvm.ve.vl.pvfmkwgenan.Mvl
  390918, // llvm.ve.vl.pvfmkwgt.MvMl
  390943, // llvm.ve.vl.pvfmkwgt.Mvl
  390967, // llvm.ve.vl.pvfmkwgtnan.MvMl
  390995, // llvm.ve.vl.pvfmkwgtnan.Mvl
  391022, // llvm.ve.vl.pvfmkwle.MvMl
  391047, // llvm.ve.vl.pvfmkwle.Mvl
  391071, // llvm.ve.vl.pvfmkwlenan.MvMl
  391099, // llvm.ve.vl.pvfmkwlenan.Mvl
  391126, // llvm.ve.vl.pvfmkwloeq.mvl
  391152, // llvm.ve.vl.pvfmkwloeq.mvml
  391179, // llvm.ve.vl.pvfmkwloeqnan.mvl
  391208, // llvm.ve.vl.pvfmkwloeqnan.mvml
  391238, // llvm.ve.vl.pvfmkwloge.mvl
  391264, // llvm.ve.vl.pvfmkwloge.mvml
  391291, // llvm.ve.vl.pvfmkwlogenan.mvl
  391320, // llvm.ve.vl.pvfmkwlogenan.mvml
  391350, // llvm.ve.vl.pvfmkwlogt.mvl
  391376, // llvm.ve.vl.pvfmkwlogt.mvml
  391403, // llvm.ve.vl.pvfmkwlogtnan.mvl
  391432, // llvm.ve.vl.pvfmkwlogtnan.mvml
  391462, // llvm.ve.vl.pvfmkwlole.mvl
  391488, // llvm.ve.vl.pvfmkwlole.mvml
  391515, // llvm.ve.vl.pvfmkwlolenan.mvl
  391544, // llvm.ve.vl.pvfmkwlolenan.mvml
  391574, // llvm.ve.vl.pvfmkwlolt.mvl
  391600, // llvm.ve.vl.pvfmkwlolt.mvml
  391627, // llvm.ve.vl.pvfmkwloltnan.mvl
  391656, // llvm.ve.vl.pvfmkwloltnan.mvml
  391686, // llvm.ve.vl.pvfmkwlonan.mvl
  391713, // llvm.ve.vl.pvfmkwlonan.mvml
  391741, // llvm.ve.vl.pvfmkwlone.mvl
  391767, // llvm.ve.vl.pvfmkwlone.mvml
  391794, // llvm.ve.vl.pvfmkwlonenan.mvl
  391823, // llvm.ve.vl.pvfmkwlonenan.mvml
  391853, // llvm.ve.vl.pvfmkwlonum.mvl
  391880, // llvm.ve.vl.pvfmkwlonum.mvml
  391908, // llvm.ve.vl.pvfmkwlt.MvMl
  391933, // llvm.ve.vl.pvfmkwlt.Mvl
  391957, // llvm.ve.vl.pvfmkwltnan.MvMl
  391985, // llvm.ve.vl.pvfmkwltnan.Mvl
  392012, // llvm.ve.vl.pvfmkwnan.MvMl
  392038, // llvm.ve.vl.pvfmkwnan.Mvl
  392063, // llvm.ve.vl.pvfmkwne.MvMl
  392088, // llvm.ve.vl.pvfmkwne.Mvl
  392112, // llvm.ve.vl.pvfmkwnenan.MvMl
  392140, // llvm.ve.vl.pvfmkwnenan.Mvl
  392167, // llvm.ve.vl.pvfmkwnum.MvMl
  392193, // llvm.ve.vl.pvfmkwnum.Mvl
  392218, // llvm.ve.vl.pvfmkwupeq.mvl
  392244, // llvm.ve.vl.pvfmkwupeq.mvml
  392271, // llvm.ve.vl.pvfmkwupeqnan.mvl
  392300, // llvm.ve.vl.pvfmkwupeqnan.mvml
  392330, // llvm.ve.vl.pvfmkwupge.mvl
  392356, // llvm.ve.vl.pvfmkwupge.mvml
  392383, // llvm.ve.vl.pvfmkwupgenan.mvl
  392412, // llvm.ve.vl.pvfmkwupgenan.mvml
  392442, // llvm.ve.vl.pvfmkwupgt.mvl
  392468, // llvm.ve.vl.pvfmkwupgt.mvml
  392495, // llvm.ve.vl.pvfmkwupgtnan.mvl
  392524, // llvm.ve.vl.pvfmkwupgtnan.mvml
  392554, // llvm.ve.vl.pvfmkwuple.mvl
  392580, // llvm.ve.vl.pvfmkwuple.mvml
  392607, // llvm.ve.vl.pvfmkwuplenan.mvl
  392636, // llvm.ve.vl.pvfmkwuplenan.mvml
  392666, // llvm.ve.vl.pvfmkwuplt.mvl
  392692, // llvm.ve.vl.pvfmkwuplt.mvml
  392719, // llvm.ve.vl.pvfmkwupltnan.mvl
  392748, // llvm.ve.vl.pvfmkwupltnan.mvml
  392778, // llvm.ve.vl.pvfmkwupnan.mvl
  392805, // llvm.ve.vl.pvfmkwupnan.mvml
  392833, // llvm.ve.vl.pvfmkwupne.mvl
  392859, // llvm.ve.vl.pvfmkwupne.mvml
  392886, // llvm.ve.vl.pvfmkwupnenan.mvl
  392915, // llvm.ve.vl.pvfmkwupnenan.mvml
  392945, // llvm.ve.vl.pvfmkwupnum.mvl
  392972, // llvm.ve.vl.pvfmkwupnum.mvml
  393000, // llvm.ve.vl.pvfmsb.vsvvMvl
  393026, // llvm.ve.vl.pvfmsb.vsvvl
  393050, // llvm.ve.vl.pvfmsb.vsvvvl
  393075, // llvm.ve.vl.pvfmsb.vvsvMvl
  393101, // llvm.ve.vl.pvfmsb.vvsvl
  393125, // llvm.ve.vl.pvfmsb.vvsvvl
  393150, // llvm.ve.vl.pvfmsb.vvvvMvl
  393176, // llvm.ve.vl.pvfmsb.vvvvl
  393200, // llvm.ve.vl.pvfmsb.vvvvvl
  393225, // llvm.ve.vl.pvfmul.vsvMvl
  393250, // llvm.ve.vl.pvfmul.vsvl
  393273, // llvm.ve.vl.pvfmul.vsvvl
  393297, // llvm.ve.vl.pvfmul.vvvMvl
  393322, // llvm.ve.vl.pvfmul.vvvl
  393345, // llvm.ve.vl.pvfmul.vvvvl
  393369, // llvm.ve.vl.pvfnmad.vsvvMvl
  393396, // llvm.ve.vl.pvfnmad.vsvvl
  393421, // llvm.ve.vl.pvfnmad.vsvvvl
  393447, // llvm.ve.vl.pvfnmad.vvsvMvl
  393474, // llvm.ve.vl.pvfnmad.vvsvl
  393499, // llvm.ve.vl.pvfnmad.vvsvvl
  393525, // llvm.ve.vl.pvfnmad.vvvvMvl
  393552, // llvm.ve.vl.pvfnmad.vvvvl
  393577, // llvm.ve.vl.pvfnmad.vvvvvl
  393603, // llvm.ve.vl.pvfnmsb.vsvvMvl
  393630, // llvm.ve.vl.pvfnmsb.vsvvl
  393655, // llvm.ve.vl.pvfnmsb.vsvvvl
  393681, // llvm.ve.vl.pvfnmsb.vvsvMvl
  393708, // llvm.ve.vl.pvfnmsb.vvsvl
  393733, // llvm.ve.vl.pvfnmsb.vvsvvl
  393759, // llvm.ve.vl.pvfnmsb.vvvvMvl
  393786, // llvm.ve.vl.pvfnmsb.vvvvl
  393811, // llvm.ve.vl.pvfnmsb.vvvvvl
  393837, // llvm.ve.vl.pvfsub.vsvMvl
  393862, // llvm.ve.vl.pvfsub.vsvl
  393885, // llvm.ve.vl.pvfsub.vsvvl
  393909, // llvm.ve.vl.pvfsub.vvvMvl
  393934, // llvm.ve.vl.pvfsub.vvvl
  393957, // llvm.ve.vl.pvfsub.vvvvl
  393981, // llvm.ve.vl.pvldz.vvMvl
  394004, // llvm.ve.vl.pvldz.vvl
  394025, // llvm.ve.vl.pvldz.vvvl
  394047, // llvm.ve.vl.pvldzlo.vvl
  394070, // llvm.ve.vl.pvldzlo.vvmvl
  394095, // llvm.ve.vl.pvldzlo.vvvl
  394119, // llvm.ve.vl.pvldzup.vvl
  394142, // llvm.ve.vl.pvldzup.vvmvl
  394167, // llvm.ve.vl.pvldzup.vvvl
  394191, // llvm.ve.vl.pvmaxs.vsvMvl
  394216, // llvm.ve.vl.pvmaxs.vsvl
  394239, // llvm.ve.vl.pvmaxs.vsvvl
  394263, // llvm.ve.vl.pvmaxs.vvvMvl
  394288, // llvm.ve.vl.pvmaxs.vvvl
  394311, // llvm.ve.vl.pvmaxs.vvvvl
  394335, // llvm.ve.vl.pvmins.vsvMvl
  394360, // llvm.ve.vl.pvmins.vsvl
  394383, // llvm.ve.vl.pvmins.vsvvl
  394407, // llvm.ve.vl.pvmins.vvvMvl
  394432, // llvm.ve.vl.pvmins.vvvl
  394455, // llvm.ve.vl.pvmins.vvvvl
  394479, // llvm.ve.vl.pvor.vsvMvl
  394502, // llvm.ve.vl.pvor.vsvl
  394523, // llvm.ve.vl.pvor.vsvvl
  394545, // llvm.ve.vl.pvor.vvvMvl
  394568, // llvm.ve.vl.pvor.vvvl
  394589, // llvm.ve.vl.pvor.vvvvl
  394611, // llvm.ve.vl.pvpcnt.vvMvl
  394635, // llvm.ve.vl.pvpcnt.vvl
  394657, // llvm.ve.vl.pvpcnt.vvvl
  394680, // llvm.ve.vl.pvpcntlo.vvl
  394704, // llvm.ve.vl.pvpcntlo.vvmvl
  394730, // llvm.ve.vl.pvpcntlo.vvvl
  394755, // llvm.ve.vl.pvpcntup.vvl
  394779, // llvm.ve.vl.pvpcntup.vvmvl
  394805, // llvm.ve.vl.pvpcntup.vvvl
  394830, // llvm.ve.vl.pvrcp.vvl
  394851, // llvm.ve.vl.pvrcp.vvvl
  394873, // llvm.ve.vl.pvrsqrt.vvl
  394896, // llvm.ve.vl.pvrsqrt.vvvl
  394920, // llvm.ve.vl.pvrsqrtnex.vvl
  394946, // llvm.ve.vl.pvrsqrtnex.vvvl
  394973, // llvm.ve.vl.pvseq.vl
  394993, // llvm.ve.vl.pvseq.vvl
  395014, // llvm.ve.vl.pvseqlo.vl
  395036, // llvm.ve.vl.pvseqlo.vvl
  395059, // llvm.ve.vl.pvsequp.vl
  395081, // llvm.ve.vl.pvsequp.vvl
  395104, // llvm.ve.vl.pvsla.vvsMvl
  395128, // llvm.ve.vl.pvsla.vvsl
  395150, // llvm.ve.vl.pvsla.vvsvl
  395173, // llvm.ve.vl.pvsla.vvvMvl
  395197, // llvm.ve.vl.pvsla.vvvl
  395219, // llvm.ve.vl.pvsla.vvvvl
  395242, // llvm.ve.vl.pvsll.vvsMvl
  395266, // llvm.ve.vl.pvsll.vvsl
  395288, // llvm.ve.vl.pvsll.vvsvl
  395311, // llvm.ve.vl.pvsll.vvvMvl
  395335, // llvm.ve.vl.pvsll.vvvl
  395357, // llvm.ve.vl.pvsll.vvvvl
  395380, // llvm.ve.vl.pvsra.vvsMvl
  395404, // llvm.ve.vl.pvsra.vvsl
  395426, // llvm.ve.vl.pvsra.vvsvl
  395449, // llvm.ve.vl.pvsra.vvvMvl
  395473, // llvm.ve.vl.pvsra.vvvl
  395495, // llvm.ve.vl.pvsra.vvvvl
  395518, // llvm.ve.vl.pvsrl.vvsMvl
  395542, // llvm.ve.vl.pvsrl.vvsl
  395564, // llvm.ve.vl.pvsrl.vvsvl
  395587, // llvm.ve.vl.pvsrl.vvvMvl
  395611, // llvm.ve.vl.pvsrl.vvvl
  395633, // llvm.ve.vl.pvsrl.vvvvl
  395656, // llvm.ve.vl.pvsubs.vsvMvl
  395681, // llvm.ve.vl.pvsubs.vsvl
  395704, // llvm.ve.vl.pvsubs.vsvvl
  395728, // llvm.ve.vl.pvsubs.vvvMvl
  395753, // llvm.ve.vl.pvsubs.vvvl
  395776, // llvm.ve.vl.pvsubs.vvvvl
  395800, // llvm.ve.vl.pvsubu.vsvMvl
  395825, // llvm.ve.vl.pvsubu.vsvl
  395848, // llvm.ve.vl.pvsubu.vsvvl
  395872, // llvm.ve.vl.pvsubu.vvvMvl
  395897, // llvm.ve.vl.pvsubu.vvvl
  395920, // llvm.ve.vl.pvsubu.vvvvl
  395944, // llvm.ve.vl.pvxor.vsvMvl
  395968, // llvm.ve.vl.pvxor.vsvl
  395990, // llvm.ve.vl.pvxor.vsvvl
  396013, // llvm.ve.vl.pvxor.vvvMvl
  396037, // llvm.ve.vl.pvxor.vvvl
  396059, // llvm.ve.vl.pvxor.vvvvl
  396082, // llvm.ve.vl.scr.sss
  396101, // llvm.ve.vl.svm.sMs
  396120, // llvm.ve.vl.svm.sms
  396139, // llvm.ve.vl.svob
  396155, // llvm.ve.vl.tovm.sml
  396175, // llvm.ve.vl.tscr.ssss
  396196, // llvm.ve.vl.vaddsl.vsvl
  396219, // llvm.ve.vl.vaddsl.vsvmvl
  396244, // llvm.ve.vl.vaddsl.vsvvl
  396268, // llvm.ve.vl.vaddsl.vvvl
  396291, // llvm.ve.vl.vaddsl.vvvmvl
  396316, // llvm.ve.vl.vaddsl.vvvvl
  396340, // llvm.ve.vl.vaddswsx.vsvl
  396365, // llvm.ve.vl.vaddswsx.vsvmvl
  396392, // llvm.ve.vl.vaddswsx.vsvvl
  396418, // llvm.ve.vl.vaddswsx.vvvl
  396443, // llvm.ve.vl.vaddswsx.vvvmvl
  396470, // llvm.ve.vl.vaddswsx.vvvvl
  396496, // llvm.ve.vl.vaddswzx.vsvl
  396521, // llvm.ve.vl.vaddswzx.vsvmvl
  396548, // llvm.ve.vl.vaddswzx.vsvvl
  396574, // llvm.ve.vl.vaddswzx.vvvl
  396599, // llvm.ve.vl.vaddswzx.vvvmvl
  396626, // llvm.ve.vl.vaddswzx.vvvvl
  396652, // llvm.ve.vl.vaddul.vsvl
  396675, // llvm.ve.vl.vaddul.vsvmvl
  396700, // llvm.ve.vl.vaddul.vsvvl
  396724, // llvm.ve.vl.vaddul.vvvl
  396747, // llvm.ve.vl.vaddul.vvvmvl
  396772, // llvm.ve.vl.vaddul.vvvvl
  396796, // llvm.ve.vl.vadduw.vsvl
  396819, // llvm.ve.vl.vadduw.vsvmvl
  396844, // llvm.ve.vl.vadduw.vsvvl
  396868, // llvm.ve.vl.vadduw.vvvl
  396891, // llvm.ve.vl.vadduw.vvvmvl
  396916, // llvm.ve.vl.vadduw.vvvvl
  396940, // llvm.ve.vl.vand.vsvl
  396961, // llvm.ve.vl.vand.vsvmvl
  396984, // llvm.ve.vl.vand.vsvvl
  397006, // llvm.ve.vl.vand.vvvl
  397027, // llvm.ve.vl.vand.vvvmvl
  397050, // llvm.ve.vl.vand.vvvvl
  397072, // llvm.ve.vl.vbrdd.vsl
  397093, // llvm.ve.vl.vbrdd.vsmvl
  397116, // llvm.ve.vl.vbrdd.vsvl
  397138, // llvm.ve.vl.vbrdl.vsl
  397159, // llvm.ve.vl.vbrdl.vsmvl
  397182, // llvm.ve.vl.vbrdl.vsvl
  397204, // llvm.ve.vl.vbrds.vsl
  397225, // llvm.ve.vl.vbrds.vsmvl
  397248, // llvm.ve.vl.vbrds.vsvl
  397270, // llvm.ve.vl.vbrdw.vsl
  397291, // llvm.ve.vl.vbrdw.vsmvl
  397314, // llvm.ve.vl.vbrdw.vsvl
  397336, // llvm.ve.vl.vbrv.vvl
  397356, // llvm.ve.vl.vbrv.vvmvl
  397378, // llvm.ve.vl.vbrv.vvvl
  397399, // llvm.ve.vl.vcmpsl.vsvl
  397422, // llvm.ve.vl.vcmpsl.vsvmvl
  397447, // llvm.ve.vl.vcmpsl.vsvvl
  397471, // llvm.ve.vl.vcmpsl.vvvl
  397494, // llvm.ve.vl.vcmpsl.vvvmvl
  397519, // llvm.ve.vl.vcmpsl.vvvvl
  397543, // llvm.ve.vl.vcmpswsx.vsvl
  397568, // llvm.ve.vl.vcmpswsx.vsvmvl
  397595, // llvm.ve.vl.vcmpswsx.vsvvl
  397621, // llvm.ve.vl.vcmpswsx.vvvl
  397646, // llvm.ve.vl.vcmpswsx.vvvmvl
  397673, // llvm.ve.vl.vcmpswsx.vvvvl
  397699, // llvm.ve.vl.vcmpswzx.vsvl
  397724, // llvm.ve.vl.vcmpswzx.vsvmvl
  397751, // llvm.ve.vl.vcmpswzx.vsvvl
  397777, // llvm.ve.vl.vcmpswzx.vvvl
  397802, // llvm.ve.vl.vcmpswzx.vvvmvl
  397829, // llvm.ve.vl.vcmpswzx.vvvvl
  397855, // llvm.ve.vl.vcmpul.vsvl
  397878, // llvm.ve.vl.vcmpul.vsvmvl
  397903, // llvm.ve.vl.vcmpul.vsvvl
  397927, // llvm.ve.vl.vcmpul.vvvl
  397950, // llvm.ve.vl.vcmpul.vvvmvl
  397975, // llvm.ve.vl.vcmpul.vvvvl
  397999, // llvm.ve.vl.vcmpuw.vsvl
  398022, // llvm.ve.vl.vcmpuw.vsvmvl
  398047, // llvm.ve.vl.vcmpuw.vsvvl
  398071, // llvm.ve.vl.vcmpuw.vvvl
  398094, // llvm.ve.vl.vcmpuw.vvvmvl
  398119, // llvm.ve.vl.vcmpuw.vvvvl
  398143, // llvm.ve.vl.vcp.vvmvl
  398164, // llvm.ve.vl.vcvtdl.vvl
  398186, // llvm.ve.vl.vcvtdl.vvvl
  398209, // llvm.ve.vl.vcvtds.vvl
  398231, // llvm.ve.vl.vcvtds.vvvl
  398254, // llvm.ve.vl.vcvtdw.vvl
  398276, // llvm.ve.vl.vcvtdw.vvvl
  398299, // llvm.ve.vl.vcvtld.vvl
  398321, // llvm.ve.vl.vcvtld.vvmvl
  398345, // llvm.ve.vl.vcvtld.vvvl
  398368, // llvm.ve.vl.vcvtldrz.vvl
  398392, // llvm.ve.vl.vcvtldrz.vvmvl
  398418, // llvm.ve.vl.vcvtldrz.vvvl
  398443, // llvm.ve.vl.vcvtsd.vvl
  398465, // llvm.ve.vl.vcvtsd.vvvl
  398488, // llvm.ve.vl.vcvtsw.vvl
  398510, // llvm.ve.vl.vcvtsw.vvvl
  398533, // llvm.ve.vl.vcvtwdsx.vvl
  398557, // llvm.ve.vl.vcvtwdsx.vvmvl
  398583, // llvm.ve.vl.vcvtwdsx.vvvl
  398608, // llvm.ve.vl.vcvtwdsxrz.vvl
  398634, // llvm.ve.vl.vcvtwdsxrz.vvmvl
  398662, // llvm.ve.vl.vcvtwdsxrz.vvvl
  398689, // llvm.ve.vl.vcvtwdzx.vvl
  398713, // llvm.ve.vl.vcvtwdzx.vvmvl
  398739, // llvm.ve.vl.vcvtwdzx.vvvl
  398764, // llvm.ve.vl.vcvtwdzxrz.vvl
  398790, // llvm.ve.vl.vcvtwdzxrz.vvmvl
  398818, // llvm.ve.vl.vcvtwdzxrz.vvvl
  398845, // llvm.ve.vl.vcvtwssx.vvl
  398869, // llvm.ve.vl.vcvtwssx.vvmvl
  398895, // llvm.ve.vl.vcvtwssx.vvvl
  398920, // llvm.ve.vl.vcvtwssxrz.vvl
  398946, // llvm.ve.vl.vcvtwssxrz.vvmvl
  398974, // llvm.ve.vl.vcvtwssxrz.vvvl
  399001, // llvm.ve.vl.vcvtwszx.vvl
  399025, // llvm.ve.vl.vcvtwszx.vvmvl
  399051, // llvm.ve.vl.vcvtwszx.vvvl
  399076, // llvm.ve.vl.vcvtwszxrz.vvl
  399102, // llvm.ve.vl.vcvtwszxrz.vvmvl
  399130, // llvm.ve.vl.vcvtwszxrz.vvvl
  399157, // llvm.ve.vl.vdivsl.vsvl
  399180, // llvm.ve.vl.vdivsl.vsvmvl
  399205, // llvm.ve.vl.vdivsl.vsvvl
  399229, // llvm.ve.vl.vdivsl.vvsl
  399252, // llvm.ve.vl.vdivsl.vvsmvl
  399277, // llvm.ve.vl.vdivsl.vvsvl
  399301, // llvm.ve.vl.vdivsl.vvvl
  399324, // llvm.ve.vl.vdivsl.vvvmvl
  399349, // llvm.ve.vl.vdivsl.vvvvl
  399373, // llvm.ve.vl.vdivswsx.vsvl
  399398, // llvm.ve.vl.vdivswsx.vsvmvl
  399425, // llvm.ve.vl.vdivswsx.vsvvl
  399451, // llvm.ve.vl.vdivswsx.vvsl
  399476, // llvm.ve.vl.vdivswsx.vvsmvl
  399503, // llvm.ve.vl.vdivswsx.vvsvl
  399529, // llvm.ve.vl.vdivswsx.vvvl
  399554, // llvm.ve.vl.vdivswsx.vvvmvl
  399581, // llvm.ve.vl.vdivswsx.vvvvl
  399607, // llvm.ve.vl.vdivswzx.vsvl
  399632, // llvm.ve.vl.vdivswzx.vsvmvl
  399659, // llvm.ve.vl.vdivswzx.vsvvl
  399685, // llvm.ve.vl.vdivswzx.vvsl
  399710, // llvm.ve.vl.vdivswzx.vvsmvl
  399737, // llvm.ve.vl.vdivswzx.vvsvl
  399763, // llvm.ve.vl.vdivswzx.vvvl
  399788, // llvm.ve.vl.vdivswzx.vvvmvl
  399815, // llvm.ve.vl.vdivswzx.vvvvl
  399841, // llvm.ve.vl.vdivul.vsvl
  399864, // llvm.ve.vl.vdivul.vsvmvl
  399889, // llvm.ve.vl.vdivul.vsvvl
  399913, // llvm.ve.vl.vdivul.vvsl
  399936, // llvm.ve.vl.vdivul.vvsmvl
  399961, // llvm.ve.vl.vdivul.vvsvl
  399985, // llvm.ve.vl.vdivul.vvvl
  400008, // llvm.ve.vl.vdivul.vvvmvl
  400033, // llvm.ve.vl.vdivul.vvvvl
  400057, // llvm.ve.vl.vdivuw.vsvl
  400080, // llvm.ve.vl.vdivuw.vsvmvl
  400105, // llvm.ve.vl.vdivuw.vsvvl
  400129, // llvm.ve.vl.vdivuw.vvsl
  400152, // llvm.ve.vl.vdivuw.vvsmvl
  400177, // llvm.ve.vl.vdivuw.vvsvl
  400201, // llvm.ve.vl.vdivuw.vvvl
  400224, // llvm.ve.vl.vdivuw.vvvmvl
  400249, // llvm.ve.vl.vdivuw.vvvvl
  400273, // llvm.ve.vl.veqv.vsvl
  400294, // llvm.ve.vl.veqv.vsvmvl
  400317, // llvm.ve.vl.veqv.vsvvl
  400339, // llvm.ve.vl.veqv.vvvl
  400360, // llvm.ve.vl.veqv.vvvmvl
  400383, // llvm.ve.vl.veqv.vvvvl
  400405, // llvm.ve.vl.vex.vvmvl
  400426, // llvm.ve.vl.vfaddd.vsvl
  400449, // llvm.ve.vl.vfaddd.vsvmvl
  400474, // llvm.ve.vl.vfaddd.vsvvl
  400498, // llvm.ve.vl.vfaddd.vvvl
  400521, // llvm.ve.vl.vfaddd.vvvmvl
  400546, // llvm.ve.vl.vfaddd.vvvvl
  400570, // llvm.ve.vl.vfadds.vsvl
  400593, // llvm.ve.vl.vfadds.vsvmvl
  400618, // llvm.ve.vl.vfadds.vsvvl
  400642, // llvm.ve.vl.vfadds.vvvl
  400665, // llvm.ve.vl.vfadds.vvvmvl
  400690, // llvm.ve.vl.vfadds.vvvvl
  400714, // llvm.ve.vl.vfcmpd.vsvl
  400737, // llvm.ve.vl.vfcmpd.vsvmvl
  400762, // llvm.ve.vl.vfcmpd.vsvvl
  400786, // llvm.ve.vl.vfcmpd.vvvl
  400809, // llvm.ve.vl.vfcmpd.vvvmvl
  400834, // llvm.ve.vl.vfcmpd.vvvvl
  400858, // llvm.ve.vl.vfcmps.vsvl
  400881, // llvm.ve.vl.vfcmps.vsvmvl
  400906, // llvm.ve.vl.vfcmps.vsvvl
  400930, // llvm.ve.vl.vfcmps.vvvl
  400953, // llvm.ve.vl.vfcmps.vvvmvl
  400978, // llvm.ve.vl.vfcmps.vvvvl
  401002, // llvm.ve.vl.vfdivd.vsvl
  401025, // llvm.ve.vl.vfdivd.vsvmvl
  401050, // llvm.ve.vl.vfdivd.vsvvl
  401074, // llvm.ve.vl.vfdivd.vvvl
  401097, // llvm.ve.vl.vfdivd.vvvmvl
  401122, // llvm.ve.vl.vfdivd.vvvvl
  401146, // llvm.ve.vl.vfdivs.vsvl
  401169, // llvm.ve.vl.vfdivs.vsvmvl
  401194, // llvm.ve.vl.vfdivs.vsvvl
  401218, // llvm.ve.vl.vfdivs.vvvl
  401241, // llvm.ve.vl.vfdivs.vvvmvl
  401266, // llvm.ve.vl.vfdivs.vvvvl
  401290, // llvm.ve.vl.vfmadd.vsvvl
  401314, // llvm.ve.vl.vfmadd.vsvvmvl
  401340, // llvm.ve.vl.vfmadd.vsvvvl
  401365, // llvm.ve.vl.vfmadd.vvsvl
  401389, // llvm.ve.vl.vfmadd.vvsvmvl
  401415, // llvm.ve.vl.vfmadd.vvsvvl
  401440, // llvm.ve.vl.vfmadd.vvvvl
  401464, // llvm.ve.vl.vfmadd.vvvvmvl
  401490, // llvm.ve.vl.vfmadd.vvvvvl
  401515, // llvm.ve.vl.vfmads.vsvvl
  401539, // llvm.ve.vl.vfmads.vsvvmvl
  401565, // llvm.ve.vl.vfmads.vsvvvl
  401590, // llvm.ve.vl.vfmads.vvsvl
  401614, // llvm.ve.vl.vfmads.vvsvmvl
  401640, // llvm.ve.vl.vfmads.vvsvvl
  401665, // llvm.ve.vl.vfmads.vvvvl
  401689, // llvm.ve.vl.vfmads.vvvvmvl
  401715, // llvm.ve.vl.vfmads.vvvvvl
  401740, // llvm.ve.vl.vfmaxd.vsvl
  401763, // llvm.ve.vl.vfmaxd.vsvmvl
  401788, // llvm.ve.vl.vfmaxd.vsvvl
  401812, // llvm.ve.vl.vfmaxd.vvvl
  401835, // llvm.ve.vl.vfmaxd.vvvmvl
  401860, // llvm.ve.vl.vfmaxd.vvvvl
  401884, // llvm.ve.vl.vfmaxs.vsvl
  401907, // llvm.ve.vl.vfmaxs.vsvmvl
  401932, // llvm.ve.vl.vfmaxs.vsvvl
  401956, // llvm.ve.vl.vfmaxs.vvvl
  401979, // llvm.ve.vl.vfmaxs.vvvmvl
  402004, // llvm.ve.vl.vfmaxs.vvvvl
  402028, // llvm.ve.vl.vfmind.vsvl
  402051, // llvm.ve.vl.vfmind.vsvmvl
  402076, // llvm.ve.vl.vfmind.vsvvl
  402100, // llvm.ve.vl.vfmind.vvvl
  402123, // llvm.ve.vl.vfmind.vvvmvl
  402148, // llvm.ve.vl.vfmind.vvvvl
  402172, // llvm.ve.vl.vfmins.vsvl
  402195, // llvm.ve.vl.vfmins.vsvmvl
  402220, // llvm.ve.vl.vfmins.vsvvl
  402244, // llvm.ve.vl.vfmins.vvvl
  402267, // llvm.ve.vl.vfmins.vvvmvl
  402292, // llvm.ve.vl.vfmins.vvvvl
  402316, // llvm.ve.vl.vfmkdeq.mvl
  402339, // llvm.ve.vl.vfmkdeq.mvml
  402363, // llvm.ve.vl.vfmkdeqnan.mvl
  402389, // llvm.ve.vl.vfmkdeqnan.mvml
  402416, // llvm.ve.vl.vfmkdge.mvl
  402439, // llvm.ve.vl.vfmkdge.mvml
  402463, // llvm.ve.vl.vfmkdgenan.mvl
  402489, // llvm.ve.vl.vfmkdgenan.mvml
  402516, // llvm.ve.vl.vfmkdgt.mvl
  402539, // llvm.ve.vl.vfmkdgt.mvml
  402563, // llvm.ve.vl.vfmkdgtnan.mvl
  402589, // llvm.ve.vl.vfmkdgtnan.mvml
  402616, // llvm.ve.vl.vfmkdle.mvl
  402639, // llvm.ve.vl.vfmkdle.mvml
  402663, // llvm.ve.vl.vfmkdlenan.mvl
  402689, // llvm.ve.vl.vfmkdlenan.mvml
  402716, // llvm.ve.vl.vfmkdlt.mvl
  402739, // llvm.ve.vl.vfmkdlt.mvml
  402763, // llvm.ve.vl.vfmkdltnan.mvl
  402789, // llvm.ve.vl.vfmkdltnan.mvml
  402816, // llvm.ve.vl.vfmkdnan.mvl
  402840, // llvm.ve.vl.vfmkdnan.mvml
  402865, // llvm.ve.vl.vfmkdne.mvl
  402888, // llvm.ve.vl.vfmkdne.mvml
  402912, // llvm.ve.vl.vfmkdnenan.mvl
  402938, // llvm.ve.vl.vfmkdnenan.mvml
  402965, // llvm.ve.vl.vfmkdnum.mvl
  402989, // llvm.ve.vl.vfmkdnum.mvml
  403014, // llvm.ve.vl.vfmklaf.ml
  403036, // llvm.ve.vl.vfmklat.ml
  403058, // llvm.ve.vl.vfmkleq.mvl
  403081, // llvm.ve.vl.vfmkleq.mvml
  403105, // llvm.ve.vl.vfmkleqnan.mvl
  403131, // llvm.ve.vl.vfmkleqnan.mvml
  403158, // llvm.ve.vl.vfmklge.mvl
  403181, // llvm.ve.vl.vfmklge.mvml
  403205, // llvm.ve.vl.vfmklgenan.mvl
  403231, // llvm.ve.vl.vfmklgenan.mvml
  403258, // llvm.ve.vl.vfmklgt.mvl
  403281, // llvm.ve.vl.vfmklgt.mvml
  403305, // llvm.ve.vl.vfmklgtnan.mvl
  403331, // llvm.ve.vl.vfmklgtnan.mvml
  403358, // llvm.ve.vl.vfmklle.mvl
  403381, // llvm.ve.vl.vfmklle.mvml
  403405, // llvm.ve.vl.vfmkllenan.mvl
  403431, // llvm.ve.vl.vfmkllenan.mvml
  403458, // llvm.ve.vl.vfmkllt.mvl
  403481, // llvm.ve.vl.vfmkllt.mvml
  403505, // llvm.ve.vl.vfmklltnan.mvl
  403531, // llvm.ve.vl.vfmklltnan.mvml
  403558, // llvm.ve.vl.vfmklnan.mvl
  403582, // llvm.ve.vl.vfmklnan.mvml
  403607, // llvm.ve.vl.vfmklne.mvl
  403630, // llvm.ve.vl.vfmklne.mvml
  403654, // llvm.ve.vl.vfmklnenan.mvl
  403680, // llvm.ve.vl.vfmklnenan.mvml
  403707, // llvm.ve.vl.vfmklnum.mvl
  403731, // llvm.ve.vl.vfmklnum.mvml
  403756, // llvm.ve.vl.vfmkseq.mvl
  403779, // llvm.ve.vl.vfmkseq.mvml
  403803, // llvm.ve.vl.vfmkseqnan.mvl
  403829, // llvm.ve.vl.vfmkseqnan.mvml
  403856, // llvm.ve.vl.vfmksge.mvl
  403879, // llvm.ve.vl.vfmksge.mvml
  403903, // llvm.ve.vl.vfmksgenan.mvl
  403929, // llvm.ve.vl.vfmksgenan.mvml
  403956, // llvm.ve.vl.vfmksgt.mvl
  403979, // llvm.ve.vl.vfmksgt.mvml
  404003, // llvm.ve.vl.vfmksgtnan.mvl
  404029, // llvm.ve.vl.vfmksgtnan.mvml
  404056, // llvm.ve.vl.vfmksle.mvl
  404079, // llvm.ve.vl.vfmksle.mvml
  404103, // llvm.ve.vl.vfmkslenan.mvl
  404129, // llvm.ve.vl.vfmkslenan.mvml
  404156, // llvm.ve.vl.vfmkslt.mvl
  404179, // llvm.ve.vl.vfmkslt.mvml
  404203, // llvm.ve.vl.vfmksltnan.mvl
  404229, // llvm.ve.vl.vfmksltnan.mvml
  404256, // llvm.ve.vl.vfmksnan.mvl
  404280, // llvm.ve.vl.vfmksnan.mvml
  404305, // llvm.ve.vl.vfmksne.mvl
  404328, // llvm.ve.vl.vfmksne.mvml
  404352, // llvm.ve.vl.vfmksnenan.mvl
  404378, // llvm.ve.vl.vfmksnenan.mvml
  404405, // llvm.ve.vl.vfmksnum.mvl
  404429, // llvm.ve.vl.vfmksnum.mvml
  404454, // llvm.ve.vl.vfmkweq.mvl
  404477, // llvm.ve.vl.vfmkweq.mvml
  404501, // llvm.ve.vl.vfmkweqnan.mvl
  404527, // llvm.ve.vl.vfmkweqnan.mvml
  404554, // llvm.ve.vl.vfmkwge.mvl
  404577, // llvm.ve.vl.vfmkwge.mvml
  404601, // llvm.ve.vl.vfmkwgenan.mvl
  404627, // llvm.ve.vl.vfmkwgenan.mvml
  404654, // llvm.ve.vl.vfmkwgt.mvl
  404677, // llvm.ve.vl.vfmkwgt.mvml
  404701, // llvm.ve.vl.vfmkwgtnan.mvl
  404727, // llvm.ve.vl.vfmkwgtnan.mvml
  404754, // llvm.ve.vl.vfmkwle.mvl
  404777, // llvm.ve.vl.vfmkwle.mvml
  404801, // llvm.ve.vl.vfmkwlenan.mvl
  404827, // llvm.ve.vl.vfmkwlenan.mvml
  404854, // llvm.ve.vl.vfmkwlt.mvl
  404877, // llvm.ve.vl.vfmkwlt.mvml
  404901, // llvm.ve.vl.vfmkwltnan.mvl
  404927, // llvm.ve.vl.vfmkwltnan.mvml
  404954, // llvm.ve.vl.vfmkwnan.mvl
  404978, // llvm.ve.vl.vfmkwnan.mvml
  405003, // llvm.ve.vl.vfmkwne.mvl
  405026, // llvm.ve.vl.vfmkwne.mvml
  405050, // llvm.ve.vl.vfmkwnenan.mvl
  405076, // llvm.ve.vl.vfmkwnenan.mvml
  405103, // llvm.ve.vl.vfmkwnum.mvl
  405127, // llvm.ve.vl.vfmkwnum.mvml
  405152, // llvm.ve.vl.vfmsbd.vsvvl
  405176, // llvm.ve.vl.vfmsbd.vsvvmvl
  405202, // llvm.ve.vl.vfmsbd.vsvvvl
  405227, // llvm.ve.vl.vfmsbd.vvsvl
  405251, // llvm.ve.vl.vfmsbd.vvsvmvl
  405277, // llvm.ve.vl.vfmsbd.vvsvvl
  405302, // llvm.ve.vl.vfmsbd.vvvvl
  405326, // llvm.ve.vl.vfmsbd.vvvvmvl
  405352, // llvm.ve.vl.vfmsbd.vvvvvl
  405377, // llvm.ve.vl.vfmsbs.vsvvl
  405401, // llvm.ve.vl.vfmsbs.vsvvmvl
  405427, // llvm.ve.vl.vfmsbs.vsvvvl
  405452, // llvm.ve.vl.vfmsbs.vvsvl
  405476, // llvm.ve.vl.vfmsbs.vvsvmvl
  405502, // llvm.ve.vl.vfmsbs.vvsvvl
  405527, // llvm.ve.vl.vfmsbs.vvvvl
  405551, // llvm.ve.vl.vfmsbs.vvvvmvl
  405577, // llvm.ve.vl.vfmsbs.vvvvvl
  405602, // llvm.ve.vl.vfmuld.vsvl
  405625, // llvm.ve.vl.vfmuld.vsvmvl
  405650, // llvm.ve.vl.vfmuld.vsvvl
  405674, // llvm.ve.vl.vfmuld.vvvl
  405697, // llvm.ve.vl.vfmuld.vvvmvl
  405722, // llvm.ve.vl.vfmuld.vvvvl
  405746, // llvm.ve.vl.vfmuls.vsvl
  405769, // llvm.ve.vl.vfmuls.vsvmvl
  405794, // llvm.ve.vl.vfmuls.vsvvl
  405818, // llvm.ve.vl.vfmuls.vvvl
  405841, // llvm.ve.vl.vfmuls.vvvmvl
  405866, // llvm.ve.vl.vfmuls.vvvvl
  405890, // llvm.ve.vl.vfnmadd.vsvvl
  405915, // llvm.ve.vl.vfnmadd.vsvvmvl
  405942, // llvm.ve.vl.vfnmadd.vsvvvl
  405968, // llvm.ve.vl.vfnmadd.vvsvl
  405993, // llvm.ve.vl.vfnmadd.vvsvmvl
  406020, // llvm.ve.vl.vfnmadd.vvsvvl
  406046, // llvm.ve.vl.vfnmadd.vvvvl
  406071, // llvm.ve.vl.vfnmadd.vvvvmvl
  406098, // llvm.ve.vl.vfnmadd.vvvvvl
  406124, // llvm.ve.vl.vfnmads.vsvvl
  406149, // llvm.ve.vl.vfnmads.vsvvmvl
  406176, // llvm.ve.vl.vfnmads.vsvvvl
  406202, // llvm.ve.vl.vfnmads.vvsvl
  406227, // llvm.ve.vl.vfnmads.vvsvmvl
  406254, // llvm.ve.vl.vfnmads.vvsvvl
  406280, // llvm.ve.vl.vfnmads.vvvvl
  406305, // llvm.ve.vl.vfnmads.vvvvmvl
  406332, // llvm.ve.vl.vfnmads.vvvvvl
  406358, // llvm.ve.vl.vfnmsbd.vsvvl
  406383, // llvm.ve.vl.vfnmsbd.vsvvmvl
  406410, // llvm.ve.vl.vfnmsbd.vsvvvl
  406436, // llvm.ve.vl.vfnmsbd.vvsvl
  406461, // llvm.ve.vl.vfnmsbd.vvsvmvl
  406488, // llvm.ve.vl.vfnmsbd.vvsvvl
  406514, // llvm.ve.vl.vfnmsbd.vvvvl
  406539, // llvm.ve.vl.vfnmsbd.vvvvmvl
  406566, // llvm.ve.vl.vfnmsbd.vvvvvl
  406592, // llvm.ve.vl.vfnmsbs.vsvvl
  406617, // llvm.ve.vl.vfnmsbs.vsvvmvl
  406644, // llvm.ve.vl.vfnmsbs.vsvvvl
  406670, // llvm.ve.vl.vfnmsbs.vvsvl
  406695, // llvm.ve.vl.vfnmsbs.vvsvmvl
  406722, // llvm.ve.vl.vfnmsbs.vvsvvl
  406748, // llvm.ve.vl.vfnmsbs.vvvvl
  406773, // llvm.ve.vl.vfnmsbs.vvvvmvl
  406800, // llvm.ve.vl.vfnmsbs.vvvvvl
  406826, // llvm.ve.vl.vfrmaxdfst.vvl
  406852, // llvm.ve.vl.vfrmaxdfst.vvvl
  406879, // llvm.ve.vl.vfrmaxdlst.vvl
  406905, // llvm.ve.vl.vfrmaxdlst.vvvl
  406932, // llvm.ve.vl.vfrmaxsfst.vvl
  406958, // llvm.ve.vl.vfrmaxsfst.vvvl
  406985, // llvm.ve.vl.vfrmaxslst.vvl
  407011, // llvm.ve.vl.vfrmaxslst.vvvl
  407038, // llvm.ve.vl.vfrmindfst.vvl
  407064, // llvm.ve.vl.vfrmindfst.vvvl
  407091, // llvm.ve.vl.vfrmindlst.vvl
  407117, // llvm.ve.vl.vfrmindlst.vvvl
  407144, // llvm.ve.vl.vfrminsfst.vvl
  407170, // llvm.ve.vl.vfrminsfst.vvvl
  407197, // llvm.ve.vl.vfrminslst.vvl
  407223, // llvm.ve.vl.vfrminslst.vvvl
  407250, // llvm.ve.vl.vfsqrtd.vvl
  407273, // llvm.ve.vl.vfsqrtd.vvvl
  407297, // llvm.ve.vl.vfsqrts.vvl
  407320, // llvm.ve.vl.vfsqrts.vvvl
  407344, // llvm.ve.vl.vfsubd.vsvl
  407367, // llvm.ve.vl.vfsubd.vsvmvl
  407392, // llvm.ve.vl.vfsubd.vsvvl
  407416, // llvm.ve.vl.vfsubd.vvvl
  407439, // llvm.ve.vl.vfsubd.vvvmvl
  407464, // llvm.ve.vl.vfsubd.vvvvl
  407488, // llvm.ve.vl.vfsubs.vsvl
  407511, // llvm.ve.vl.vfsubs.vsvmvl
  407536, // llvm.ve.vl.vfsubs.vsvvl
  407560, // llvm.ve.vl.vfsubs.vvvl
  407583, // llvm.ve.vl.vfsubs.vvvmvl
  407608, // llvm.ve.vl.vfsubs.vvvvl
  407632, // llvm.ve.vl.vfsumd.vvl
  407654, // llvm.ve.vl.vfsumd.vvml
  407677, // llvm.ve.vl.vfsums.vvl
  407699, // llvm.ve.vl.vfsums.vvml
  407722, // llvm.ve.vl.vgt.vvssl
  407743, // llvm.ve.vl.vgt.vvssml
  407765, // llvm.ve.vl.vgt.vvssmvl
  407788, // llvm.ve.vl.vgt.vvssvl
  407810, // llvm.ve.vl.vgtlsx.vvssl
  407834, // llvm.ve.vl.vgtlsx.vvssml
  407859, // llvm.ve.vl.vgtlsx.vvssmvl
  407885, // llvm.ve.vl.vgtlsx.vvssvl
  407910, // llvm.ve.vl.vgtlsxnc.vvssl
  407936, // llvm.ve.vl.vgtlsxnc.vvssml
  407963, // llvm.ve.vl.vgtlsxnc.vvssmvl
  407991, // llvm.ve.vl.vgtlsxnc.vvssvl
  408018, // llvm.ve.vl.vgtlzx.vvssl
  408042, // llvm.ve.vl.vgtlzx.vvssml
  408067, // llvm.ve.vl.vgtlzx.vvssmvl
  408093, // llvm.ve.vl.vgtlzx.vvssvl
  408118, // llvm.ve.vl.vgtlzxnc.vvssl
  408144, // llvm.ve.vl.vgtlzxnc.vvssml
  408171, // llvm.ve.vl.vgtlzxnc.vvssmvl
  408199, // llvm.ve.vl.vgtlzxnc.vvssvl
  408226, // llvm.ve.vl.vgtnc.vvssl
  408249, // llvm.ve.vl.vgtnc.vvssml
  408273, // llvm.ve.vl.vgtnc.vvssmvl
  408298, // llvm.ve.vl.vgtnc.vvssvl
  408322, // llvm.ve.vl.vgtu.vvssl
  408344, // llvm.ve.vl.vgtu.vvssml
  408367, // llvm.ve.vl.vgtu.vvssmvl
  408391, // llvm.ve.vl.vgtu.vvssvl
  408414, // llvm.ve.vl.vgtunc.vvssl
  408438, // llvm.ve.vl.vgtunc.vvssml
  408463, // llvm.ve.vl.vgtunc.vvssmvl
  408489, // llvm.ve.vl.vgtunc.vvssvl
  408514, // llvm.ve.vl.vld.vssl
  408534, // llvm.ve.vl.vld.vssvl
  408555, // llvm.ve.vl.vld2d.vssl
  408577, // llvm.ve.vl.vld2d.vssvl
  408600, // llvm.ve.vl.vld2dnc.vssl
  408624, // llvm.ve.vl.vld2dnc.vssvl
  408649, // llvm.ve.vl.vldl2dsx.vssl
  408674, // llvm.ve.vl.vldl2dsx.vssvl
  408700, // llvm.ve.vl.vldl2dsxnc.vssl
  408727, // llvm.ve.vl.vldl2dsxnc.vssvl
  408755, // llvm.ve.vl.vldl2dzx.vssl
  408780, // llvm.ve.vl.vldl2dzx.vssvl
  408806, // llvm.ve.vl.vldl2dzxnc.vssl
  408833, // llvm.ve.vl.vldl2dzxnc.vssvl
  408861, // llvm.ve.vl.vldlsx.vssl
  408884, // llvm.ve.vl.vldlsx.vssvl
  408908, // llvm.ve.vl.vldlsxnc.vssl
  408933, // llvm.ve.vl.vldlsxnc.vssvl
  408959, // llvm.ve.vl.vldlzx.vssl
  408982, // llvm.ve.vl.vldlzx.vssvl
  409006, // llvm.ve.vl.vldlzxnc.vssl
  409031, // llvm.ve.vl.vldlzxnc.vssvl
  409057, // llvm.ve.vl.vldnc.vssl
  409079, // llvm.ve.vl.vldnc.vssvl
  409102, // llvm.ve.vl.vldu.vssl
  409123, // llvm.ve.vl.vldu.vssvl
  409145, // llvm.ve.vl.vldu2d.vssl
  409168, // llvm.ve.vl.vldu2d.vssvl
  409192, // llvm.ve.vl.vldu2dnc.vssl
  409217, // llvm.ve.vl.vldu2dnc.vssvl
  409243, // llvm.ve.vl.vldunc.vssl
  409266, // llvm.ve.vl.vldunc.vssvl
  409290, // llvm.ve.vl.vldz.vvl
  409310, // llvm.ve.vl.vldz.vvmvl
  409332, // llvm.ve.vl.vldz.vvvl
  409353, // llvm.ve.vl.vmaxsl.vsvl
  409376, // llvm.ve.vl.vmaxsl.vsvmvl
  409401, // llvm.ve.vl.vmaxsl.vsvvl
  409425, // llvm.ve.vl.vmaxsl.vvvl
  409448, // llvm.ve.vl.vmaxsl.vvvmvl
  409473, // llvm.ve.vl.vmaxsl.vvvvl
  409497, // llvm.ve.vl.vmaxswsx.vsvl
  409522, // llvm.ve.vl.vmaxswsx.vsvmvl
  409549, // llvm.ve.vl.vmaxswsx.vsvvl
  409575, // llvm.ve.vl.vmaxswsx.vvvl
  409600, // llvm.ve.vl.vmaxswsx.vvvmvl
  409627, // llvm.ve.vl.vmaxswsx.vvvvl
  409653, // llvm.ve.vl.vmaxswzx.vsvl
  409678, // llvm.ve.vl.vmaxswzx.vsvmvl
  409705, // llvm.ve.vl.vmaxswzx.vsvvl
  409731, // llvm.ve.vl.vmaxswzx.vvvl
  409756, // llvm.ve.vl.vmaxswzx.vvvmvl
  409783, // llvm.ve.vl.vmaxswzx.vvvvl
  409809, // llvm.ve.vl.vminsl.vsvl
  409832, // llvm.ve.vl.vminsl.vsvmvl
  409857, // llvm.ve.vl.vminsl.vsvvl
  409881, // llvm.ve.vl.vminsl.vvvl
  409904, // llvm.ve.vl.vminsl.vvvmvl
  409929, // llvm.ve.vl.vminsl.vvvvl
  409953, // llvm.ve.vl.vminswsx.vsvl
  409978, // llvm.ve.vl.vminswsx.vsvmvl
  410005, // llvm.ve.vl.vminswsx.vsvvl
  410031, // llvm.ve.vl.vminswsx.vvvl
  410056, // llvm.ve.vl.vminswsx.vvvmvl
  410083, // llvm.ve.vl.vminswsx.vvvvl
  410109, // llvm.ve.vl.vminswzx.vsvl
  410134, // llvm.ve.vl.vminswzx.vsvmvl
  410161, // llvm.ve.vl.vminswzx.vsvvl
  410187, // llvm.ve.vl.vminswzx.vvvl
  410212, // llvm.ve.vl.vminswzx.vvvmvl
  410239, // llvm.ve.vl.vminswzx.vvvvl
  410265, // llvm.ve.vl.vmrg.vsvml
  410287, // llvm.ve.vl.vmrg.vsvmvl
  410310, // llvm.ve.vl.vmrg.vvvml
  410332, // llvm.ve.vl.vmrg.vvvmvl
  410355, // llvm.ve.vl.vmrgw.vsvMl
  410378, // llvm.ve.vl.vmrgw.vsvMvl
  410402, // llvm.ve.vl.vmrgw.vvvMl
  410425, // llvm.ve.vl.vmrgw.vvvMvl
  410449, // llvm.ve.vl.vmulsl.vsvl
  410472, // llvm.ve.vl.vmulsl.vsvmvl
  410497, // llvm.ve.vl.vmulsl.vsvvl
  410521, // llvm.ve.vl.vmulsl.vvvl
  410544, // llvm.ve.vl.vmulsl.vvvmvl
  410569, // llvm.ve.vl.vmulsl.vvvvl
  410593, // llvm.ve.vl.vmulslw.vsvl
  410617, // llvm.ve.vl.vmulslw.vsvvl
  410642, // llvm.ve.vl.vmulslw.vvvl
  410666, // llvm.ve.vl.vmulslw.vvvvl
  410691, // llvm.ve.vl.vmulswsx.vsvl
  410716, // llvm.ve.vl.vmulswsx.vsvmvl
  410743, // llvm.ve.vl.vmulswsx.vsvvl
  410769, // llvm.ve.vl.vmulswsx.vvvl
  410794, // llvm.ve.vl.vmulswsx.vvvmvl
  410821, // llvm.ve.vl.vmulswsx.vvvvl
  410847, // llvm.ve.vl.vmulswzx.vsvl
  410872, // llvm.ve.vl.vmulswzx.vsvmvl
  410899, // llvm.ve.vl.vmulswzx.vsvvl
  410925, // llvm.ve.vl.vmulswzx.vvvl
  410950, // llvm.ve.vl.vmulswzx.vvvmvl
  410977, // llvm.ve.vl.vmulswzx.vvvvl
  411003, // llvm.ve.vl.vmulul.vsvl
  411026, // llvm.ve.vl.vmulul.vsvmvl
  411051, // llvm.ve.vl.vmulul.vsvvl
  411075, // llvm.ve.vl.vmulul.vvvl
  411098, // llvm.ve.vl.vmulul.vvvmvl
  411123, // llvm.ve.vl.vmulul.vvvvl
  411147, // llvm.ve.vl.vmuluw.vsvl
  411170, // llvm.ve.vl.vmuluw.vsvmvl
  411195, // llvm.ve.vl.vmuluw.vsvvl
  411219, // llvm.ve.vl.vmuluw.vvvl
  411242, // llvm.ve.vl.vmuluw.vvvmvl
  411267, // llvm.ve.vl.vmuluw.vvvvl
  411291, // llvm.ve.vl.vmv.vsvl
  411311, // llvm.ve.vl.vmv.vsvmvl
  411333, // llvm.ve.vl.vmv.vsvvl
  411354, // llvm.ve.vl.vor.vsvl
  411374, // llvm.ve.vl.vor.vsvmvl
  411396, // llvm.ve.vl.vor.vsvvl
  411417, // llvm.ve.vl.vor.vvvl
  411437, // llvm.ve.vl.vor.vvvmvl
  411459, // llvm.ve.vl.vor.vvvvl
  411480, // llvm.ve.vl.vpcnt.vvl
  411501, // llvm.ve.vl.vpcnt.vvmvl
  411524, // llvm.ve.vl.vpcnt.vvvl
  411546, // llvm.ve.vl.vrand.vvl
  411567, // llvm.ve.vl.vrand.vvml
  411589, // llvm.ve.vl.vrcpd.vvl
  411610, // llvm.ve.vl.vrcpd.vvvl
  411632, // llvm.ve.vl.vrcps.vvl
  411653, // llvm.ve.vl.vrcps.vvvl
  411675, // llvm.ve.vl.vrmaxslfst.vvl
  411701, // llvm.ve.vl.vrmaxslfst.vvvl
  411728, // llvm.ve.vl.vrmaxsllst.vvl
  411754, // llvm.ve.vl.vrmaxsllst.vvvl
  411781, // llvm.ve.vl.vrmaxswfstsx.vvl
  411809, // llvm.ve.vl.vrmaxswfstsx.vvvl
  411838, // llvm.ve.vl.vrmaxswfstzx.vvl
  411866, // llvm.ve.vl.vrmaxswfstzx.vvvl
  411895, // llvm.ve.vl.vrmaxswlstsx.vvl
  411923, // llvm.ve.vl.vrmaxswlstsx.vvvl
  411952, // llvm.ve.vl.vrmaxswlstzx.vvl
  411980, // llvm.ve.vl.vrmaxswlstzx.vvvl
  412009, // llvm.ve.vl.vrminslfst.vvl
  412035, // llvm.ve.vl.vrminslfst.vvvl
  412062, // llvm.ve.vl.vrminsllst.vvl
  412088, // llvm.ve.vl.vrminsllst.vvvl
  412115, // llvm.ve.vl.vrminswfstsx.vvl
  412143, // llvm.ve.vl.vrminswfstsx.vvvl
  412172, // llvm.ve.vl.vrminswfstzx.vvl
  412200, // llvm.ve.vl.vrminswfstzx.vvvl
  412229, // llvm.ve.vl.vrminswlstsx.vvl
  412257, // llvm.ve.vl.vrminswlstsx.vvvl
  412286, // llvm.ve.vl.vrminswlstzx.vvl
  412314, // llvm.ve.vl.vrminswlstzx.vvvl
  412343, // llvm.ve.vl.vror.vvl
  412363, // llvm.ve.vl.vror.vvml
  412384, // llvm.ve.vl.vrsqrtd.vvl
  412407, // llvm.ve.vl.vrsqrtd.vvvl
  412431, // llvm.ve.vl.vrsqrtdnex.vvl
  412457, // llvm.ve.vl.vrsqrtdnex.vvvl
  412484, // llvm.ve.vl.vrsqrts.vvl
  412507, // llvm.ve.vl.vrsqrts.vvvl
  412531, // llvm.ve.vl.vrsqrtsnex.vvl
  412557, // llvm.ve.vl.vrsqrtsnex.vvvl
  412584, // llvm.ve.vl.vrxor.vvl
  412605, // llvm.ve.vl.vrxor.vvml
  412627, // llvm.ve.vl.vsc.vvssl
  412648, // llvm.ve.vl.vsc.vvssml
  412670, // llvm.ve.vl.vscl.vvssl
  412692, // llvm.ve.vl.vscl.vvssml
  412715, // llvm.ve.vl.vsclnc.vvssl
  412739, // llvm.ve.vl.vsclnc.vvssml
  412764, // llvm.ve.vl.vsclncot.vvssl
  412790, // llvm.ve.vl.vsclncot.vvssml
  412817, // llvm.ve.vl.vsclot.vvssl
  412841, // llvm.ve.vl.vsclot.vvssml
  412866, // llvm.ve.vl.vscnc.vvssl
  412889, // llvm.ve.vl.vscnc.vvssml
  412913, // llvm.ve.vl.vscncot.vvssl
  412938, // llvm.ve.vl.vscncot.vvssml
  412964, // llvm.ve.vl.vscot.vvssl
  412987, // llvm.ve.vl.vscot.vvssml
  413011, // llvm.ve.vl.vscu.vvssl
  413033, // llvm.ve.vl.vscu.vvssml
  413056, // llvm.ve.vl.vscunc.vvssl
  413080, // llvm.ve.vl.vscunc.vvssml
  413105, // llvm.ve.vl.vscuncot.vvssl
  413131, // llvm.ve.vl.vscuncot.vvssml
  413158, // llvm.ve.vl.vscuot.vvssl
  413182, // llvm.ve.vl.vscuot.vvssml
  413207, // llvm.ve.vl.vseq.vl
  413226, // llvm.ve.vl.vseq.vvl
  413246, // llvm.ve.vl.vsfa.vvssl
  413268, // llvm.ve.vl.vsfa.vvssmvl
  413292, // llvm.ve.vl.vsfa.vvssvl
  413315, // llvm.ve.vl.vshf.vvvsl
  413337, // llvm.ve.vl.vshf.vvvsvl
  413360, // llvm.ve.vl.vslal.vvsl
  413382, // llvm.ve.vl.vslal.vvsmvl
  413406, // llvm.ve.vl.vslal.vvsvl
  413429, // llvm.ve.vl.vslal.vvvl
  413451, // llvm.ve.vl.vslal.vvvmvl
  413475, // llvm.ve.vl.vslal.vvvvl
  413498, // llvm.ve.vl.vslawsx.vvsl
  413522, // llvm.ve.vl.vslawsx.vvsmvl
  413548, // llvm.ve.vl.vslawsx.vvsvl
  413573, // llvm.ve.vl.vslawsx.vvvl
  413597, // llvm.ve.vl.vslawsx.vvvmvl
  413623, // llvm.ve.vl.vslawsx.vvvvl
  413648, // llvm.ve.vl.vslawzx.vvsl
  413672, // llvm.ve.vl.vslawzx.vvsmvl
  413698, // llvm.ve.vl.vslawzx.vvsvl
  413723, // llvm.ve.vl.vslawzx.vvvl
  413747, // llvm.ve.vl.vslawzx.vvvmvl
  413773, // llvm.ve.vl.vslawzx.vvvvl
  413798, // llvm.ve.vl.vsll.vvsl
  413819, // llvm.ve.vl.vsll.vvsmvl
  413842, // llvm.ve.vl.vsll.vvsvl
  413864, // llvm.ve.vl.vsll.vvvl
  413885, // llvm.ve.vl.vsll.vvvmvl
  413908, // llvm.ve.vl.vsll.vvvvl
  413930, // llvm.ve.vl.vsral.vvsl
  413952, // llvm.ve.vl.vsral.vvsmvl
  413976, // llvm.ve.vl.vsral.vvsvl
  413999, // llvm.ve.vl.vsral.vvvl
  414021, // llvm.ve.vl.vsral.vvvmvl
  414045, // llvm.ve.vl.vsral.vvvvl
  414068, // llvm.ve.vl.vsrawsx.vvsl
  414092, // llvm.ve.vl.vsrawsx.vvsmvl
  414118, // llvm.ve.vl.vsrawsx.vvsvl
  414143, // llvm.ve.vl.vsrawsx.vvvl
  414167, // llvm.ve.vl.vsrawsx.vvvmvl
  414193, // llvm.ve.vl.vsrawsx.vvvvl
  414218, // llvm.ve.vl.vsrawzx.vvsl
  414242, // llvm.ve.vl.vsrawzx.vvsmvl
  414268, // llvm.ve.vl.vsrawzx.vvsvl
  414293, // llvm.ve.vl.vsrawzx.vvvl
  414317, // llvm.ve.vl.vsrawzx.vvvmvl
  414343, // llvm.ve.vl.vsrawzx.vvvvl
  414368, // llvm.ve.vl.vsrl.vvsl
  414389, // llvm.ve.vl.vsrl.vvsmvl
  414412, // llvm.ve.vl.vsrl.vvsvl
  414434, // llvm.ve.vl.vsrl.vvvl
  414455, // llvm.ve.vl.vsrl.vvvmvl
  414478, // llvm.ve.vl.vsrl.vvvvl
  414500, // llvm.ve.vl.vst.vssl
  414520, // llvm.ve.vl.vst.vssml
  414541, // llvm.ve.vl.vst2d.vssl
  414563, // llvm.ve.vl.vst2d.vssml
  414586, // llvm.ve.vl.vst2dnc.vssl
  414610, // llvm.ve.vl.vst2dnc.vssml
  414635, // llvm.ve.vl.vst2dncot.vssl
  414661, // llvm.ve.vl.vst2dncot.vssml
  414688, // llvm.ve.vl.vst2dot.vssl
  414712, // llvm.ve.vl.vst2dot.vssml
  414737, // llvm.ve.vl.vstl.vssl
  414758, // llvm.ve.vl.vstl.vssml
  414780, // llvm.ve.vl.vstl2d.vssl
  414803, // llvm.ve.vl.vstl2d.vssml
  414827, // llvm.ve.vl.vstl2dnc.vssl
  414852, // llvm.ve.vl.vstl2dnc.vssml
  414878, // llvm.ve.vl.vstl2dncot.vssl
  414905, // llvm.ve.vl.vstl2dncot.vssml
  414933, // llvm.ve.vl.vstl2dot.vssl
  414958, // llvm.ve.vl.vstl2dot.vssml
  414984, // llvm.ve.vl.vstlnc.vssl
  415007, // llvm.ve.vl.vstlnc.vssml
  415031, // llvm.ve.vl.vstlncot.vssl
  415056, // llvm.ve.vl.vstlncot.vssml
  415082, // llvm.ve.vl.vstlot.vssl
  415105, // llvm.ve.vl.vstlot.vssml
  415129, // llvm.ve.vl.vstnc.vssl
  415151, // llvm.ve.vl.vstnc.vssml
  415174, // llvm.ve.vl.vstncot.vssl
  415198, // llvm.ve.vl.vstncot.vssml
  415223, // llvm.ve.vl.vstot.vssl
  415245, // llvm.ve.vl.vstot.vssml
  415268, // llvm.ve.vl.vstu.vssl
  415289, // llvm.ve.vl.vstu.vssml
  415311, // llvm.ve.vl.vstu2d.vssl
  415334, // llvm.ve.vl.vstu2d.vssml
  415358, // llvm.ve.vl.vstu2dnc.vssl
  415383, // llvm.ve.vl.vstu2dnc.vssml
  415409, // llvm.ve.vl.vstu2dncot.vssl
  415436, // llvm.ve.vl.vstu2dncot.vssml
  415464, // llvm.ve.vl.vstu2dot.vssl
  415489, // llvm.ve.vl.vstu2dot.vssml
  415515, // llvm.ve.vl.vstunc.vssl
  415538, // llvm.ve.vl.vstunc.vssml
  415562, // llvm.ve.vl.vstuncot.vssl
  415587, // llvm.ve.vl.vstuncot.vssml
  415613, // llvm.ve.vl.vstuot.vssl
  415636, // llvm.ve.vl.vstuot.vssml
  415660, // llvm.ve.vl.vsubsl.vsvl
  415683, // llvm.ve.vl.vsubsl.vsvmvl
  415708, // llvm.ve.vl.vsubsl.vsvvl
  415732, // llvm.ve.vl.vsubsl.vvvl
  415755, // llvm.ve.vl.vsubsl.vvvmvl
  415780, // llvm.ve.vl.vsubsl.vvvvl
  415804, // llvm.ve.vl.vsubswsx.vsvl
  415829, // llvm.ve.vl.vsubswsx.vsvmvl
  415856, // llvm.ve.vl.vsubswsx.vsvvl
  415882, // llvm.ve.vl.vsubswsx.vvvl
  415907, // llvm.ve.vl.vsubswsx.vvvmvl
  415934, // llvm.ve.vl.vsubswsx.vvvvl
  415960, // llvm.ve.vl.vsubswzx.vsvl
  415985, // llvm.ve.vl.vsubswzx.vsvmvl
  416012, // llvm.ve.vl.vsubswzx.vsvvl
  416038, // llvm.ve.vl.vsubswzx.vvvl
  416063, // llvm.ve.vl.vsubswzx.vvvmvl
  416090, // llvm.ve.vl.vsubswzx.vvvvl
  416116, // llvm.ve.vl.vsubul.vsvl
  416139, // llvm.ve.vl.vsubul.vsvmvl
  416164, // llvm.ve.vl.vsubul.vsvvl
  416188, // llvm.ve.vl.vsubul.vvvl
  416211, // llvm.ve.vl.vsubul.vvvmvl
  416236, // llvm.ve.vl.vsubul.vvvvl
  416260, // llvm.ve.vl.vsubuw.vsvl
  416283, // llvm.ve.vl.vsubuw.vsvmvl
  416308, // llvm.ve.vl.vsubuw.vsvvl
  416332, // llvm.ve.vl.vsubuw.vvvl
  416355, // llvm.ve.vl.vsubuw.vvvmvl
  416380, // llvm.ve.vl.vsubuw.vvvvl
  416404, // llvm.ve.vl.vsuml.vvl
  416425, // llvm.ve.vl.vsuml.vvml
  416447, // llvm.ve.vl.vsumwsx.vvl
  416470, // llvm.ve.vl.vsumwsx.vvml
  416494, // llvm.ve.vl.vsumwzx.vvl
  416517, // llvm.ve.vl.vsumwzx.vvml
  416541, // llvm.ve.vl.vxor.vsvl
  416562, // llvm.ve.vl.vxor.vsvmvl
  416585, // llvm.ve.vl.vxor.vsvvl
  416607, // llvm.ve.vl.vxor.vvvl
  416628, // llvm.ve.vl.vxor.vvvmvl
  416651, // llvm.ve.vl.vxor.vvvvl
  416673, // llvm.ve.vl.xorm.MMM
  416693, // llvm.ve.vl.xorm.mmm
  416713, // llvm.wasm.alltrue
  416731, // llvm.wasm.anytrue
  416749, // llvm.wasm.avgr.unsigned
  416773, // llvm.wasm.bitmask
  416791, // llvm.wasm.bitselect
  416811, // llvm.wasm.catch
  416827, // llvm.wasm.dot
  416841, // llvm.wasm.extadd.pairwise.signed
  416874, // llvm.wasm.extadd.pairwise.unsigned
  416909, // llvm.wasm.extract.lane.f16x8
  416938, // llvm.wasm.get.ehselector
  416963, // llvm.wasm.get.exception
  416987, // llvm.wasm.landingpad.index
  417014, // llvm.wasm.loadf16.f32
  417036, // llvm.wasm.lsda
  417051, // llvm.wasm.memory.atomic.notify
  417082, // llvm.wasm.memory.atomic.wait32
  417113, // llvm.wasm.memory.atomic.wait64
  417144, // llvm.wasm.memory.grow
  417166, // llvm.wasm.memory.size
  417188, // llvm.wasm.narrow.signed
  417212, // llvm.wasm.narrow.unsigned
  417238, // llvm.wasm.pmax
  417253, // llvm.wasm.pmin
  417268, // llvm.wasm.q15mulr.sat.signed
  417297, // llvm.wasm.ref.is_null.exn
  417323, // llvm.wasm.ref.is_null.extern
  417352, // llvm.wasm.ref.is_null.func
  417379, // llvm.wasm.ref.null.exn
  417402, // llvm.wasm.ref.null.extern
  417428, // llvm.wasm.ref.null.func
  417452, // llvm.wasm.ref.test.func
  417476, // llvm.wasm.relaxed.dot.bf16x8.add.f32
  417513, // llvm.wasm.relaxed.dot.i8x16.i7x16.add.signed
  417558, // llvm.wasm.relaxed.dot.i8x16.i7x16.signed
  417599, // llvm.wasm.relaxed.laneselect
  417628, // llvm.wasm.relaxed.madd
  417651, // llvm.wasm.relaxed.max
  417673, // llvm.wasm.relaxed.min
  417695, // llvm.wasm.relaxed.nmadd
  417719, // llvm.wasm.relaxed.q15mulr.signed
  417752, // llvm.wasm.relaxed.swizzle
  417778, // llvm.wasm.relaxed.trunc.signed
  417809, // llvm.wasm.relaxed.trunc.signed.zero
  417845, // llvm.wasm.relaxed.trunc.unsigned
  417878, // llvm.wasm.relaxed.trunc.unsigned.zero
  417916, // llvm.wasm.replace.lane.f16x8
  417945, // llvm.wasm.rethrow
  417963, // llvm.wasm.shuffle
  417981, // llvm.wasm.splat.f16x8
  418003, // llvm.wasm.storef16.f32
  418026, // llvm.wasm.swizzle
  418044, // llvm.wasm.table.copy
  418065, // llvm.wasm.table.fill.exnref
  418093, // llvm.wasm.table.fill.externref
  418124, // llvm.wasm.table.fill.funcref
  418153, // llvm.wasm.table.get.exnref
  418180, // llvm.wasm.table.get.externref
  418210, // llvm.wasm.table.get.funcref
  418238, // llvm.wasm.table.grow.exnref
  418266, // llvm.wasm.table.grow.externref
  418297, // llvm.wasm.table.grow.funcref
  418326, // llvm.wasm.table.set.exnref
  418353, // llvm.wasm.table.set.externref
  418383, // llvm.wasm.table.set.funcref
  418411, // llvm.wasm.table.size
  418432, // llvm.wasm.throw
  418448, // llvm.wasm.tls.align
  418468, // llvm.wasm.tls.base
  418487, // llvm.wasm.tls.size
  418506, // llvm.wasm.trunc.saturate.signed
  418538, // llvm.wasm.trunc.saturate.unsigned
  418572, // llvm.wasm.trunc.signed
  418595, // llvm.wasm.trunc.unsigned
  418620, // llvm.x86.aadd32
  418636, // llvm.x86.aadd64
  418652, // llvm.x86.aand32
  418668, // llvm.x86.aand64
  418684, // llvm.x86.addcarry.32
  418705, // llvm.x86.addcarry.64
  418726, // llvm.x86.aesdec128kl
  418747, // llvm.x86.aesdec256kl
  418768, // llvm.x86.aesdecwide128kl
  418793, // llvm.x86.aesdecwide256kl
  418818, // llvm.x86.aesenc128kl
  418839, // llvm.x86.aesenc256kl
  418860, // llvm.x86.aesencwide128kl
  418885, // llvm.x86.aesencwide256kl
  418910, // llvm.x86.aesni.aesdec
  418932, // llvm.x86.aesni.aesdec.256
  418958, // llvm.x86.aesni.aesdec.512
  418984, // llvm.x86.aesni.aesdeclast
  419010, // llvm.x86.aesni.aesdeclast.256
  419040, // llvm.x86.aesni.aesdeclast.512
  419070, // llvm.x86.aesni.aesenc
  419092, // llvm.x86.aesni.aesenc.256
  419118, // llvm.x86.aesni.aesenc.512
  419144, // llvm.x86.aesni.aesenclast
  419170, // llvm.x86.aesni.aesenclast.256
  419200, // llvm.x86.aesni.aesenclast.512
  419230, // llvm.x86.aesni.aesimc
  419252, // llvm.x86.aesni.aeskeygenassist
  419283, // llvm.x86.aor32
  419298, // llvm.x86.aor64
  419313, // llvm.x86.atomic.add.cc
  419336, // llvm.x86.atomic.and.cc
  419359, // llvm.x86.atomic.btc
  419379, // llvm.x86.atomic.btc.rm
  419402, // llvm.x86.atomic.btr
  419422, // llvm.x86.atomic.btr.rm
  419445, // llvm.x86.atomic.bts
  419465, // llvm.x86.atomic.bts.rm
  419488, // llvm.x86.atomic.or.cc
  419510, // llvm.x86.atomic.sub.cc
  419533, // llvm.x86.atomic.xor.cc
  419556, // llvm.x86.avx.addsub.pd.256
  419583, // llvm.x86.avx.addsub.ps.256
  419610, // llvm.x86.avx.blendv.pd.256
  419637, // llvm.x86.avx.blendv.ps.256
  419664, // llvm.x86.avx.cmp.pd.256
  419688, // llvm.x86.avx.cmp.ps.256
  419712, // llvm.x86.avx.cvt.pd2.ps.256
  419740, // llvm.x86.avx.cvt.pd2dq.256
  419767, // llvm.x86.avx.cvt.ps2dq.256
  419794, // llvm.x86.avx.cvtt.pd2dq.256
  419822, // llvm.x86.avx.cvtt.ps2dq.256
  419850, // llvm.x86.avx.dp.ps.256
  419873, // llvm.x86.avx.hadd.pd.256
  419898, // llvm.x86.avx.hadd.ps.256
  419923, // llvm.x86.avx.hsub.pd.256
  419948, // llvm.x86.avx.hsub.ps.256
  419973, // llvm.x86.avx.ldu.dq.256
  419997, // llvm.x86.avx.maskload.pd
  420022, // llvm.x86.avx.maskload.pd.256
  420051, // llvm.x86.avx.maskload.ps
  420076, // llvm.x86.avx.maskload.ps.256
  420105, // llvm.x86.avx.maskstore.pd
  420131, // llvm.x86.avx.maskstore.pd.256
  420161, // llvm.x86.avx.maskstore.ps
  420187, // llvm.x86.avx.maskstore.ps.256
  420217, // llvm.x86.avx.max.pd.256
  420241, // llvm.x86.avx.max.ps.256
  420265, // llvm.x86.avx.min.pd.256
  420289, // llvm.x86.avx.min.ps.256
  420313, // llvm.x86.avx.movmsk.pd.256
  420340, // llvm.x86.avx.movmsk.ps.256
  420367, // llvm.x86.avx.ptestc.256
  420391, // llvm.x86.avx.ptestnzc.256
  420417, // llvm.x86.avx.ptestz.256
  420441, // llvm.x86.avx.rcp.ps.256
  420465, // llvm.x86.avx.round.pd.256
  420491, // llvm.x86.avx.round.ps.256
  420517, // llvm.x86.avx.rsqrt.ps.256
  420543, // llvm.x86.avx.vpermilvar.pd
  420570, // llvm.x86.avx.vpermilvar.pd.256
  420601, // llvm.x86.avx.vpermilvar.ps
  420628, // llvm.x86.avx.vpermilvar.ps.256
  420659, // llvm.x86.avx.vtestc.pd
  420682, // llvm.x86.avx.vtestc.pd.256
  420709, // llvm.x86.avx.vtestc.ps
  420732, // llvm.x86.avx.vtestc.ps.256
  420759, // llvm.x86.avx.vtestnzc.pd
  420784, // llvm.x86.avx.vtestnzc.pd.256
  420813, // llvm.x86.avx.vtestnzc.ps
  420838, // llvm.x86.avx.vtestnzc.ps.256
  420867, // llvm.x86.avx.vtestz.pd
  420890, // llvm.x86.avx.vtestz.pd.256
  420917, // llvm.x86.avx.vtestz.ps
  420940, // llvm.x86.avx.vtestz.ps.256
  420967, // llvm.x86.avx.vzeroall
  420989, // llvm.x86.avx.vzeroupper
  421013, // llvm.x86.avx10.fpclass.bf16.128
  421045, // llvm.x86.avx10.fpclass.bf16.256
  421077, // llvm.x86.avx10.fpclass.bf16.512
  421109, // llvm.x86.avx10.mask.getexp.bf16.128
  421145, // llvm.x86.avx10.mask.getexp.bf16.256
  421181, // llvm.x86.avx10.mask.getexp.bf16.512
  421217, // llvm.x86.avx10.mask.getmant.bf16.128
  421254, // llvm.x86.avx10.mask.getmant.bf16.256
  421291, // llvm.x86.avx10.mask.getmant.bf16.512
  421328, // llvm.x86.avx10.mask.rcp.bf16.128
  421361, // llvm.x86.avx10.mask.rcp.bf16.256
  421394, // llvm.x86.avx10.mask.rcp.bf16.512
  421427, // llvm.x86.avx10.mask.reduce.bf16.128
  421463, // llvm.x86.avx10.mask.reduce.bf16.256
  421499, // llvm.x86.avx10.mask.reduce.bf16.512
  421535, // llvm.x86.avx10.mask.rndscale.bf16.128
  421573, // llvm.x86.avx10.mask.rndscale.bf16.256
  421611, // llvm.x86.avx10.mask.rndscale.bf16.512
  421649, // llvm.x86.avx10.mask.rsqrt.bf16.128
  421684, // llvm.x86.avx10.mask.rsqrt.bf16.256
  421719, // llvm.x86.avx10.mask.rsqrt.bf16.512
  421754, // llvm.x86.avx10.mask.scalef.bf16.128
  421790, // llvm.x86.avx10.mask.scalef.bf16.256
  421826, // llvm.x86.avx10.mask.scalef.bf16.512
  421862, // llvm.x86.avx10.mask.vcvt2ps2phx.128
  421898, // llvm.x86.avx10.mask.vcvt2ps2phx.256
  421934, // llvm.x86.avx10.mask.vcvt2ps2phx.512
  421970, // llvm.x86.avx10.mask.vcvtbiasph2bf8128
  422008, // llvm.x86.avx10.mask.vcvtbiasph2bf8256
  422046, // llvm.x86.avx10.mask.vcvtbiasph2bf8512
  422084, // llvm.x86.avx10.mask.vcvtbiasph2bf8s128
  422123, // llvm.x86.avx10.mask.vcvtbiasph2bf8s256
  422162, // llvm.x86.avx10.mask.vcvtbiasph2bf8s512
  422201, // llvm.x86.avx10.mask.vcvtbiasph2hf8128
  422239, // llvm.x86.avx10.mask.vcvtbiasph2hf8256
  422277, // llvm.x86.avx10.mask.vcvtbiasph2hf8512
  422315, // llvm.x86.avx10.mask.vcvtbiasph2hf8s128
  422354, // llvm.x86.avx10.mask.vcvtbiasph2hf8s256
  422393, // llvm.x86.avx10.mask.vcvtbiasph2hf8s512
  422432, // llvm.x86.avx10.mask.vcvthf82ph128
  422466, // llvm.x86.avx10.mask.vcvthf82ph256
  422500, // llvm.x86.avx10.mask.vcvthf82ph512
  422534, // llvm.x86.avx10.mask.vcvtph2bf8128
  422568, // llvm.x86.avx10.mask.vcvtph2bf8256
  422602, // llvm.x86.avx10.mask.vcvtph2bf8512
  422636, // llvm.x86.avx10.mask.vcvtph2bf8s128
  422671, // llvm.x86.avx10.mask.vcvtph2bf8s256
  422706, // llvm.x86.avx10.mask.vcvtph2bf8s512
  422741, // llvm.x86.avx10.mask.vcvtph2hf8128
  422775, // llvm.x86.avx10.mask.vcvtph2hf8256
  422809, // llvm.x86.avx10.mask.vcvtph2hf8512
  422843, // llvm.x86.avx10.mask.vcvtph2hf8s128
  422878, // llvm.x86.avx10.mask.vcvtph2hf8s256
  422913, // llvm.x86.avx10.mask.vcvtph2hf8s512
  422948, // llvm.x86.avx10.mask.vcvtph2ibs128
  422982, // llvm.x86.avx10.mask.vcvtph2ibs256
  423016, // llvm.x86.avx10.mask.vcvtph2ibs512
  423050, // llvm.x86.avx10.mask.vcvtph2iubs128
  423085, // llvm.x86.avx10.mask.vcvtph2iubs256
  423120, // llvm.x86.avx10.mask.vcvtph2iubs512
  423155, // llvm.x86.avx10.mask.vcvtps2ibs128
  423189, // llvm.x86.avx10.mask.vcvtps2ibs256
  423223, // llvm.x86.avx10.mask.vcvtps2ibs512
  423257, // llvm.x86.avx10.mask.vcvtps2iubs128
  423292, // llvm.x86.avx10.mask.vcvtps2iubs256
  423327, // llvm.x86.avx10.mask.vcvtps2iubs512
  423362, // llvm.x86.avx10.mask.vcvttpd2dqs.128
  423398, // llvm.x86.avx10.mask.vcvttpd2dqs.256
  423434, // llvm.x86.avx10.mask.vcvttpd2dqs.round.512
  423476, // llvm.x86.avx10.mask.vcvttpd2qqs.128
  423512, // llvm.x86.avx10.mask.vcvttpd2qqs.256
  423548, // llvm.x86.avx10.mask.vcvttpd2qqs.round.512
  423590, // llvm.x86.avx10.mask.vcvttpd2udqs.128
  423627, // llvm.x86.avx10.mask.vcvttpd2udqs.256
  423664, // llvm.x86.avx10.mask.vcvttpd2udqs.round.512
  423707, // llvm.x86.avx10.mask.vcvttpd2uqqs.128
  423744, // llvm.x86.avx10.mask.vcvttpd2uqqs.256
  423781, // llvm.x86.avx10.mask.vcvttpd2uqqs.round.512
  423824, // llvm.x86.avx10.mask.vcvttph2ibs128
  423859, // llvm.x86.avx10.mask.vcvttph2ibs256
  423894, // llvm.x86.avx10.mask.vcvttph2ibs512
  423929, // llvm.x86.avx10.mask.vcvttph2iubs128
  423965, // llvm.x86.avx10.mask.vcvttph2iubs256
  424001, // llvm.x86.avx10.mask.vcvttph2iubs512
  424037, // llvm.x86.avx10.mask.vcvttps2dqs.128
  424073, // llvm.x86.avx10.mask.vcvttps2dqs.256
  424109, // llvm.x86.avx10.mask.vcvttps2dqs.round.512
  424151, // llvm.x86.avx10.mask.vcvttps2ibs128
  424186, // llvm.x86.avx10.mask.vcvttps2ibs256
  424221, // llvm.x86.avx10.mask.vcvttps2ibs512
  424256, // llvm.x86.avx10.mask.vcvttps2iubs128
  424292, // llvm.x86.avx10.mask.vcvttps2iubs256
  424328, // llvm.x86.avx10.mask.vcvttps2iubs512
  424364, // llvm.x86.avx10.mask.vcvttps2qqs.128
  424400, // llvm.x86.avx10.mask.vcvttps2qqs.256
  424436, // llvm.x86.avx10.mask.vcvttps2qqs.round.512
  424478, // llvm.x86.avx10.mask.vcvttps2udqs.128
  424515, // llvm.x86.avx10.mask.vcvttps2udqs.256
  424552, // llvm.x86.avx10.mask.vcvttps2udqs.round.512
  424595, // llvm.x86.avx10.mask.vcvttps2uqqs.128
  424632, // llvm.x86.avx10.mask.vcvttps2uqqs.256
  424669, // llvm.x86.avx10.mask.vcvttps2uqqs.round.512
  424712, // llvm.x86.avx10.mask.vminmaxpd.round
  424748, // llvm.x86.avx10.mask.vminmaxpd128
  424781, // llvm.x86.avx10.mask.vminmaxpd256
  424814, // llvm.x86.avx10.mask.vminmaxph.round
  424850, // llvm.x86.avx10.mask.vminmaxph128
  424883, // llvm.x86.avx10.mask.vminmaxph256
  424916, // llvm.x86.avx10.mask.vminmaxps.round
  424952, // llvm.x86.avx10.mask.vminmaxps128
  424985, // llvm.x86.avx10.mask.vminmaxps256
  425018, // llvm.x86.avx10.mask.vminmaxsd.round
  425054, // llvm.x86.avx10.mask.vminmaxsh.round
  425090, // llvm.x86.avx10.mask.vminmaxss.round
  425126, // llvm.x86.avx10.vaddbf16128
  425153, // llvm.x86.avx10.vaddbf16256
  425180, // llvm.x86.avx10.vaddbf16512
  425207, // llvm.x86.avx10.vaddpd256
  425232, // llvm.x86.avx10.vaddph256
  425257, // llvm.x86.avx10.vaddps256
  425282, // llvm.x86.avx10.vcomisbf16eq
  425310, // llvm.x86.avx10.vcomisbf16ge
  425338, // llvm.x86.avx10.vcomisbf16gt
  425366, // llvm.x86.avx10.vcomisbf16le
  425394, // llvm.x86.avx10.vcomisbf16lt
  425422, // llvm.x86.avx10.vcomisbf16neq
  425451, // llvm.x86.avx10.vcvt2ph2bf8128
  425481, // llvm.x86.avx10.vcvt2ph2bf8256
  425511, // llvm.x86.avx10.vcvt2ph2bf8512
  425541, // llvm.x86.avx10.vcvt2ph2bf8s128
  425572, // llvm.x86.avx10.vcvt2ph2bf8s256
  425603, // llvm.x86.avx10.vcvt2ph2bf8s512
  425634, // llvm.x86.avx10.vcvt2ph2hf8128
  425664, // llvm.x86.avx10.vcvt2ph2hf8256
  425694, // llvm.x86.avx10.vcvt2ph2hf8512
  425724, // llvm.x86.avx10.vcvt2ph2hf8s128
  425755, // llvm.x86.avx10.vcvt2ph2hf8s256
  425786, // llvm.x86.avx10.vcvt2ph2hf8s512
  425817, // llvm.x86.avx10.vcvtbf162ibs128
  425848, // llvm.x86.avx10.vcvtbf162ibs256
  425879, // llvm.x86.avx10.vcvtbf162ibs512
  425910, // llvm.x86.avx10.vcvtbf162iubs128
  425942, // llvm.x86.avx10.vcvtbf162iubs256
  425974, // llvm.x86.avx10.vcvtbf162iubs512
  426006, // llvm.x86.avx10.vcvttbf162ibs128
  426038, // llvm.x86.avx10.vcvttbf162ibs256
  426070, // llvm.x86.avx10.vcvttbf162ibs512
  426102, // llvm.x86.avx10.vcvttbf162iubs128
  426135, // llvm.x86.avx10.vcvttbf162iubs256
  426168, // llvm.x86.avx10.vcvttbf162iubs512
  426201, // llvm.x86.avx10.vcvttsd2sis
  426228, // llvm.x86.avx10.vcvttsd2sis64
  426257, // llvm.x86.avx10.vcvttsd2usis
  426285, // llvm.x86.avx10.vcvttsd2usis64
  426315, // llvm.x86.avx10.vcvttss2sis
  426342, // llvm.x86.avx10.vcvttss2sis64
  426371, // llvm.x86.avx10.vcvttss2usis
  426399, // llvm.x86.avx10.vcvttss2usis64
  426429, // llvm.x86.avx10.vdivbf16128
  426456, // llvm.x86.avx10.vdivbf16256
  426483, // llvm.x86.avx10.vdivbf16512
  426510, // llvm.x86.avx10.vdpphps.128
  426537, // llvm.x86.avx10.vdpphps.256
  426564, // llvm.x86.avx10.vdpphps.512
  426591, // llvm.x86.avx10.vfmadd132bf16128
  426623, // llvm.x86.avx10.vfmadd132bf16256
  426655, // llvm.x86.avx10.vfmadd132bf16512
  426687, // llvm.x86.avx10.vfmadd213bf16128
  426719, // llvm.x86.avx10.vfmadd213bf16256
  426751, // llvm.x86.avx10.vfmadd231bf16128
  426783, // llvm.x86.avx10.vfmadd231bf16256
  426815, // llvm.x86.avx10.vfmadd231bf16512
  426847, // llvm.x86.avx10.vfmsub132bf16128
  426879, // llvm.x86.avx10.vfmsub132bf16256
  426911, // llvm.x86.avx10.vfmsub132bf16512
  426943, // llvm.x86.avx10.vfmsub213bf16128
  426975, // llvm.x86.avx10.vfmsub213bf16256
  427007, // llvm.x86.avx10.vfmsub213bf16512
  427039, // llvm.x86.avx10.vfmsub231bf16128
  427071, // llvm.x86.avx10.vfmsub231bf16256
  427103, // llvm.x86.avx10.vfmsub231bf16512
  427135, // llvm.x86.avx10.vfnmadd132bf16128
  427168, // llvm.x86.avx10.vfnmadd132bf16256
  427201, // llvm.x86.avx10.vfnmadd132bf16512
  427234, // llvm.x86.avx10.vfnmadd213bf16128
  427267, // llvm.x86.avx10.vfnmadd213bf16256
  427300, // llvm.x86.avx10.vfnmadd213bf16512
  427333, // llvm.x86.avx10.vfnmadd231bf16128
  427366, // llvm.x86.avx10.vfnmadd231bf16256
  427399, // llvm.x86.avx10.vfnmadd231bf16512
  427432, // llvm.x86.avx10.vfnmsub132bf16128
  427465, // llvm.x86.avx10.vfnmsub132bf16256
  427498, // llvm.x86.avx10.vfnmsub132bf16512
  427531, // llvm.x86.avx10.vfnmsub213bf16128
  427564, // llvm.x86.avx10.vfnmsub213bf16256
  427597, // llvm.x86.avx10.vfnmsub213bf16512
  427630, // llvm.x86.avx10.vfnmsub231bf16128
  427663, // llvm.x86.avx10.vfnmsub231bf16256
  427696, // llvm.x86.avx10.vfnmsub231bf16512
  427729, // llvm.x86.avx10.vmaxbf16128
  427756, // llvm.x86.avx10.vmaxbf16256
  427783, // llvm.x86.avx10.vmaxbf16512
  427810, // llvm.x86.avx10.vminbf16128
  427837, // llvm.x86.avx10.vminbf16256
  427864, // llvm.x86.avx10.vminbf16512
  427891, // llvm.x86.avx10.vminmaxbf16128
  427921, // llvm.x86.avx10.vminmaxbf16256
  427951, // llvm.x86.avx10.vminmaxbf16512
  427981, // llvm.x86.avx10.vminmaxpd128
  428009, // llvm.x86.avx10.vminmaxpd256
  428037, // llvm.x86.avx10.vminmaxph128
  428065, // llvm.x86.avx10.vminmaxph256
  428093, // llvm.x86.avx10.vminmaxps128
  428121, // llvm.x86.avx10.vminmaxps256
  428149, // llvm.x86.avx10.vmovrsb128
  428175, // llvm.x86.avx10.vmovrsb256
  428201, // llvm.x86.avx10.vmovrsb512
  428227, // llvm.x86.avx10.vmovrsd128
  428253, // llvm.x86.avx10.vmovrsd256
  428279, // llvm.x86.avx10.vmovrsd512
  428305, // llvm.x86.avx10.vmovrsq128
  428331, // llvm.x86.avx10.vmovrsq256
  428357, // llvm.x86.avx10.vmovrsq512
  428383, // llvm.x86.avx10.vmovrsw128
  428409, // llvm.x86.avx10.vmovrsw256
  428435, // llvm.x86.avx10.vmovrsw512
  428461, // llvm.x86.avx10.vmpsadbw.512
  428489, // llvm.x86.avx10.vmulbf16128
  428516, // llvm.x86.avx10.vmulbf16256
  428543, // llvm.x86.avx10.vmulbf16512
  428570, // llvm.x86.avx10.vpdpbssd.512
  428598, // llvm.x86.avx10.vpdpbssds.512
  428627, // llvm.x86.avx10.vpdpbsud.512
  428655, // llvm.x86.avx10.vpdpbsuds.512
  428684, // llvm.x86.avx10.vpdpbuud.512
  428712, // llvm.x86.avx10.vpdpbuuds.512
  428741, // llvm.x86.avx10.vpdpwsud.512
  428769, // llvm.x86.avx10.vpdpwsuds.512
  428798, // llvm.x86.avx10.vpdpwusd.512
  428826, // llvm.x86.avx10.vpdpwusds.512
  428855, // llvm.x86.avx10.vpdpwuud.512
  428883, // llvm.x86.avx10.vpdpwuuds.512
  428912, // llvm.x86.avx10.vsubbf16128
  428939, // llvm.x86.avx10.vsubbf16256
  428966, // llvm.x86.avx10.vsubbf16512
  428993, // llvm.x86.avx2.gather.d.d
  429018, // llvm.x86.avx2.gather.d.d.256
  429047, // llvm.x86.avx2.gather.d.pd
  429073, // llvm.x86.avx2.gather.d.pd.256
  429103, // llvm.x86.avx2.gather.d.ps
  429129, // llvm.x86.avx2.gather.d.ps.256
  429159, // llvm.x86.avx2.gather.d.q
  429184, // llvm.x86.avx2.gather.d.q.256
  429213, // llvm.x86.avx2.gather.q.d
  429238, // llvm.x86.avx2.gather.q.d.256
  429267, // llvm.x86.avx2.gather.q.pd
  429293, // llvm.x86.avx2.gather.q.pd.256
  429323, // llvm.x86.avx2.gather.q.ps
  429349, // llvm.x86.avx2.gather.q.ps.256
  429379, // llvm.x86.avx2.gather.q.q
  429404, // llvm.x86.avx2.gather.q.q.256
  429433, // llvm.x86.avx2.maskload.d
  429458, // llvm.x86.avx2.maskload.d.256
  429487, // llvm.x86.avx2.maskload.q
  429512, // llvm.x86.avx2.maskload.q.256
  429541, // llvm.x86.avx2.maskstore.d
  429567, // llvm.x86.avx2.maskstore.d.256
  429597, // llvm.x86.avx2.maskstore.q
  429623, // llvm.x86.avx2.maskstore.q.256
  429653, // llvm.x86.avx2.mpsadbw
  429675, // llvm.x86.avx2.packssdw
  429698, // llvm.x86.avx2.packsswb
  429721, // llvm.x86.avx2.packusdw
  429744, // llvm.x86.avx2.packuswb
  429767, // llvm.x86.avx2.pavg.b
  429788, // llvm.x86.avx2.pavg.w
  429809, // llvm.x86.avx2.pblendvb
  429832, // llvm.x86.avx2.permd
  429852, // llvm.x86.avx2.permps
  429873, // llvm.x86.avx2.phadd.d
  429895, // llvm.x86.avx2.phadd.sw
  429918, // llvm.x86.avx2.phadd.w
  429940, // llvm.x86.avx2.phsub.d
  429962, // llvm.x86.avx2.phsub.sw
  429985, // llvm.x86.avx2.phsub.w
  430007, // llvm.x86.avx2.pmadd.ub.sw
  430033, // llvm.x86.avx2.pmadd.wd
  430056, // llvm.x86.avx2.pmovmskb
  430079, // llvm.x86.avx2.pmul.hr.sw
  430104, // llvm.x86.avx2.pmulh.w
  430126, // llvm.x86.avx2.pmulhu.w
  430149, // llvm.x86.avx2.psad.bw
  430171, // llvm.x86.avx2.pshuf.b
  430193, // llvm.x86.avx2.psign.b
  430215, // llvm.x86.avx2.psign.d
  430237, // llvm.x86.avx2.psign.w
  430259, // llvm.x86.avx2.psll.d
  430280, // llvm.x86.avx2.psll.q
  430301, // llvm.x86.avx2.psll.w
  430322, // llvm.x86.avx2.pslli.d
  430344, // llvm.x86.avx2.pslli.q
  430366, // llvm.x86.avx2.pslli.w
  430388, // llvm.x86.avx2.psllv.d
  430410, // llvm.x86.avx2.psllv.d.256
  430436, // llvm.x86.avx2.psllv.q
  430458, // llvm.x86.avx2.psllv.q.256
  430484, // llvm.x86.avx2.psra.d
  430505, // llvm.x86.avx2.psra.w
  430526, // llvm.x86.avx2.psrai.d
  430548, // llvm.x86.avx2.psrai.w
  430570, // llvm.x86.avx2.psrav.d
  430592, // llvm.x86.avx2.psrav.d.256
  430618, // llvm.x86.avx2.psrl.d
  430639, // llvm.x86.avx2.psrl.q
  430660, // llvm.x86.avx2.psrl.w
  430681, // llvm.x86.avx2.psrli.d
  430703, // llvm.x86.avx2.psrli.q
  430725, // llvm.x86.avx2.psrli.w
  430747, // llvm.x86.avx2.psrlv.d
  430769, // llvm.x86.avx2.psrlv.d.256
  430795, // llvm.x86.avx2.psrlv.q
  430817, // llvm.x86.avx2.psrlv.q.256
  430843, // llvm.x86.avx2.vpdpbssd.128
  430870, // llvm.x86.avx2.vpdpbssd.256
  430897, // llvm.x86.avx2.vpdpbssds.128
  430925, // llvm.x86.avx2.vpdpbssds.256
  430953, // llvm.x86.avx2.vpdpbsud.128
  430980, // llvm.x86.avx2.vpdpbsud.256
  431007, // llvm.x86.avx2.vpdpbsuds.128
  431035, // llvm.x86.avx2.vpdpbsuds.256
  431063, // llvm.x86.avx2.vpdpbuud.128
  431090, // llvm.x86.avx2.vpdpbuud.256
  431117, // llvm.x86.avx2.vpdpbuuds.128
  431145, // llvm.x86.avx2.vpdpbuuds.256
  431173, // llvm.x86.avx2.vpdpwsud.128
  431200, // llvm.x86.avx2.vpdpwsud.256
  431227, // llvm.x86.avx2.vpdpwsuds.128
  431255, // llvm.x86.avx2.vpdpwsuds.256
  431283, // llvm.x86.avx2.vpdpwusd.128
  431310, // llvm.x86.avx2.vpdpwusd.256
  431337, // llvm.x86.avx2.vpdpwusds.128
  431365, // llvm.x86.avx2.vpdpwusds.256
  431393, // llvm.x86.avx2.vpdpwuud.128
  431420, // llvm.x86.avx2.vpdpwuud.256
  431447, // llvm.x86.avx2.vpdpwuuds.128
  431475, // llvm.x86.avx2.vpdpwuuds.256
  431503, // llvm.x86.avx512.add.pd.512
  431530, // llvm.x86.avx512.add.ps.512
  431557, // llvm.x86.avx512.broadcastmb.128
  431589, // llvm.x86.avx512.broadcastmb.256
  431621, // llvm.x86.avx512.broadcastmb.512
  431653, // llvm.x86.avx512.broadcastmw.128
  431685, // llvm.x86.avx512.broadcastmw.256
  431717, // llvm.x86.avx512.broadcastmw.512
  431749, // llvm.x86.avx512.conflict.d.128
  431780, // llvm.x86.avx512.conflict.d.256
  431811, // llvm.x86.avx512.conflict.d.512
  431842, // llvm.x86.avx512.conflict.q.128
  431873, // llvm.x86.avx512.conflict.q.256
  431904, // llvm.x86.avx512.conflict.q.512
  431935, // llvm.x86.avx512.cvtsi2sd64
  431962, // llvm.x86.avx512.cvtsi2ss32
  431989, // llvm.x86.avx512.cvtsi2ss64
  432016, // llvm.x86.avx512.cvttsd2si
  432042, // llvm.x86.avx512.cvttsd2si64
  432070, // llvm.x86.avx512.cvttsd2usi
  432097, // llvm.x86.avx512.cvttsd2usi64
  432126, // llvm.x86.avx512.cvttss2si
  432152, // llvm.x86.avx512.cvttss2si64
  432180, // llvm.x86.avx512.cvttss2usi
  432207, // llvm.x86.avx512.cvttss2usi64
  432236, // llvm.x86.avx512.cvtusi2ss
  432262, // llvm.x86.avx512.cvtusi642sd
  432290, // llvm.x86.avx512.cvtusi642ss
  432318, // llvm.x86.avx512.dbpsadbw.128
  432347, // llvm.x86.avx512.dbpsadbw.256
  432376, // llvm.x86.avx512.dbpsadbw.512
  432405, // llvm.x86.avx512.div.pd.512
  432432, // llvm.x86.avx512.div.ps.512
  432459, // llvm.x86.avx512.fpclass.pd.128
  432490, // llvm.x86.avx512.fpclass.pd.256
  432521, // llvm.x86.avx512.fpclass.pd.512
  432552, // llvm.x86.avx512.fpclass.ps.128
  432583, // llvm.x86.avx512.fpclass.ps.256
  432614, // llvm.x86.avx512.fpclass.ps.512
  432645, // llvm.x86.avx512.gather.dpd.512
  432676, // llvm.x86.avx512.gather.dpi.512
  432707, // llvm.x86.avx512.gather.dpq.512
  432738, // llvm.x86.avx512.gather.dps.512
  432769, // llvm.x86.avx512.gather.qpd.512
  432800, // llvm.x86.avx512.gather.qpi.512
  432831, // llvm.x86.avx512.gather.qpq.512
  432862, // llvm.x86.avx512.gather.qps.512
  432893, // llvm.x86.avx512.gather3div2.df
  432924, // llvm.x86.avx512.gather3div2.di
  432955, // llvm.x86.avx512.gather3div4.df
  432986, // llvm.x86.avx512.gather3div4.di
  433017, // llvm.x86.avx512.gather3div4.sf
  433048, // llvm.x86.avx512.gather3div4.si
  433079, // llvm.x86.avx512.gather3div8.sf
  433110, // llvm.x86.avx512.gather3div8.si
  433141, // llvm.x86.avx512.gather3siv2.df
  433172, // llvm.x86.avx512.gather3siv2.di
  433203, // llvm.x86.avx512.gather3siv4.df
  433234, // llvm.x86.avx512.gather3siv4.di
  433265, // llvm.x86.avx512.gather3siv4.sf
  433296, // llvm.x86.avx512.gather3siv4.si
  433327, // llvm.x86.avx512.gather3siv8.sf
  433358, // llvm.x86.avx512.gather3siv8.si
  433389, // llvm.x86.avx512.kadd.b
  433412, // llvm.x86.avx512.kadd.d
  433435, // llvm.x86.avx512.kadd.q
  433458, // llvm.x86.avx512.kadd.w
  433481, // llvm.x86.avx512.ktestc.b
  433506, // llvm.x86.avx512.ktestc.d
  433531, // llvm.x86.avx512.ktestc.q
  433556, // llvm.x86.avx512.ktestc.w
  433581, // llvm.x86.avx512.ktestz.b
  433606, // llvm.x86.avx512.ktestz.d
  433631, // llvm.x86.avx512.ktestz.q
  433656, // llvm.x86.avx512.ktestz.w
  433681, // llvm.x86.avx512.mask.add.sd.round
  433715, // llvm.x86.avx512.mask.add.ss.round
  433749, // llvm.x86.avx512.mask.cmp.pd.128
  433781, // llvm.x86.avx512.mask.cmp.pd.256
  433813, // llvm.x86.avx512.mask.cmp.pd.512
  433845, // llvm.x86.avx512.mask.cmp.ps.128
  433877, // llvm.x86.avx512.mask.cmp.ps.256
  433909, // llvm.x86.avx512.mask.cmp.ps.512
  433941, // llvm.x86.avx512.mask.cmp.sd
  433969, // llvm.x86.avx512.mask.cmp.ss
  433997, // llvm.x86.avx512.mask.compress
  434027, // llvm.x86.avx512.mask.cvtpd2dq.128
  434061, // llvm.x86.avx512.mask.cvtpd2dq.512
  434095, // llvm.x86.avx512.mask.cvtpd2ps
  434125, // llvm.x86.avx512.mask.cvtpd2ps.512
  434159, // llvm.x86.avx512.mask.cvtpd2qq.128
  434193, // llvm.x86.avx512.mask.cvtpd2qq.256
  434227, // llvm.x86.avx512.mask.cvtpd2qq.512
  434261, // llvm.x86.avx512.mask.cvtpd2udq.128
  434296, // llvm.x86.avx512.mask.cvtpd2udq.256
  434331, // llvm.x86.avx512.mask.cvtpd2udq.512
  434366, // llvm.x86.avx512.mask.cvtpd2uqq.128
  434401, // llvm.x86.avx512.mask.cvtpd2uqq.256
  434436, // llvm.x86.avx512.mask.cvtpd2uqq.512
  434471, // llvm.x86.avx512.mask.cvtps2dq.128
  434505, // llvm.x86.avx512.mask.cvtps2dq.256
  434539, // llvm.x86.avx512.mask.cvtps2dq.512
  434573, // llvm.x86.avx512.mask.cvtps2pd.512
  434607, // llvm.x86.avx512.mask.cvtps2qq.128
  434641, // llvm.x86.avx512.mask.cvtps2qq.256
  434675, // llvm.x86.avx512.mask.cvtps2qq.512
  434709, // llvm.x86.avx512.mask.cvtps2udq.128
  434744, // llvm.x86.avx512.mask.cvtps2udq.256
  434779, // llvm.x86.avx512.mask.cvtps2udq.512
  434814, // llvm.x86.avx512.mask.cvtps2uqq.128
  434849, // llvm.x86.avx512.mask.cvtps2uqq.256
  434884, // llvm.x86.avx512.mask.cvtps2uqq.512
  434919, // llvm.x86.avx512.mask.cvtqq2ps.128
  434953, // llvm.x86.avx512.mask.cvtsd2ss.round
  434989, // llvm.x86.avx512.mask.cvtss2sd.round
  435025, // llvm.x86.avx512.mask.cvttpd2dq.128
  435060, // llvm.x86.avx512.mask.cvttpd2dq.512
  435095, // llvm.x86.avx512.mask.cvttpd2qq.128
  435130, // llvm.x86.avx512.mask.cvttpd2qq.256
  435165, // llvm.x86.avx512.mask.cvttpd2qq.512
  435200, // llvm.x86.avx512.mask.cvttpd2udq.128
  435236, // llvm.x86.avx512.mask.cvttpd2udq.256
  435272, // llvm.x86.avx512.mask.cvttpd2udq.512
  435308, // llvm.x86.avx512.mask.cvttpd2uqq.128
  435344, // llvm.x86.avx512.mask.cvttpd2uqq.256
  435380, // llvm.x86.avx512.mask.cvttpd2uqq.512
  435416, // llvm.x86.avx512.mask.cvttps2dq.512
  435451, // llvm.x86.avx512.mask.cvttps2qq.128
  435486, // llvm.x86.avx512.mask.cvttps2qq.256
  435521, // llvm.x86.avx512.mask.cvttps2qq.512
  435556, // llvm.x86.avx512.mask.cvttps2udq.128
  435592, // llvm.x86.avx512.mask.cvttps2udq.256
  435628, // llvm.x86.avx512.mask.cvttps2udq.512
  435664, // llvm.x86.avx512.mask.cvttps2uqq.128
  435700, // llvm.x86.avx512.mask.cvttps2uqq.256
  435736, // llvm.x86.avx512.mask.cvttps2uqq.512
  435772, // llvm.x86.avx512.mask.cvtuqq2ps.128
  435807, // llvm.x86.avx512.mask.div.sd.round
  435841, // llvm.x86.avx512.mask.div.ss.round
  435875, // llvm.x86.avx512.mask.expand
  435903, // llvm.x86.avx512.mask.fixupimm.pd.128
  435940, // llvm.x86.avx512.mask.fixupimm.pd.256
  435977, // llvm.x86.avx512.mask.fixupimm.pd.512
  436014, // llvm.x86.avx512.mask.fixupimm.ps.128
  436051, // llvm.x86.avx512.mask.fixupimm.ps.256
  436088, // llvm.x86.avx512.mask.fixupimm.ps.512
  436125, // llvm.x86.avx512.mask.fixupimm.sd
  436158, // llvm.x86.avx512.mask.fixupimm.ss
  436191, // llvm.x86.avx512.mask.fpclass.sd
  436223, // llvm.x86.avx512.mask.fpclass.ss
  436255, // llvm.x86.avx512.mask.gather.dpd.512
  436291, // llvm.x86.avx512.mask.gather.dpi.512
  436327, // llvm.x86.avx512.mask.gather.dpq.512
  436363, // llvm.x86.avx512.mask.gather.dps.512
  436399, // llvm.x86.avx512.mask.gather.qpd.512
  436435, // llvm.x86.avx512.mask.gather.qpi.512
  436471, // llvm.x86.avx512.mask.gather.qpq.512
  436507, // llvm.x86.avx512.mask.gather.qps.512
  436543, // llvm.x86.avx512.mask.gather3div2.df
  436579, // llvm.x86.avx512.mask.gather3div2.di
  436615, // llvm.x86.avx512.mask.gather3div4.df
  436651, // llvm.x86.avx512.mask.gather3div4.di
  436687, // llvm.x86.avx512.mask.gather3div4.sf
  436723, // llvm.x86.avx512.mask.gather3div4.si
  436759, // llvm.x86.avx512.mask.gather3div8.sf
  436795, // llvm.x86.avx512.mask.gather3div8.si
  436831, // llvm.x86.avx512.mask.gather3siv2.df
  436867, // llvm.x86.avx512.mask.gather3siv2.di
  436903, // llvm.x86.avx512.mask.gather3siv4.df
  436939, // llvm.x86.avx512.mask.gather3siv4.di
  436975, // llvm.x86.avx512.mask.gather3siv4.sf
  437011, // llvm.x86.avx512.mask.gather3siv4.si
  437047, // llvm.x86.avx512.mask.gather3siv8.sf
  437083, // llvm.x86.avx512.mask.gather3siv8.si
  437119, // llvm.x86.avx512.mask.getexp.pd.128
  437154, // llvm.x86.avx512.mask.getexp.pd.256
  437189, // llvm.x86.avx512.mask.getexp.pd.512
  437224, // llvm.x86.avx512.mask.getexp.ps.128
  437259, // llvm.x86.avx512.mask.getexp.ps.256
  437294, // llvm.x86.avx512.mask.getexp.ps.512
  437329, // llvm.x86.avx512.mask.getexp.sd
  437360, // llvm.x86.avx512.mask.getexp.ss
  437391, // llvm.x86.avx512.mask.getmant.pd.128
  437427, // llvm.x86.avx512.mask.getmant.pd.256
  437463, // llvm.x86.avx512.mask.getmant.pd.512
  437499, // llvm.x86.avx512.mask.getmant.ps.128
  437535, // llvm.x86.avx512.mask.getmant.ps.256
  437571, // llvm.x86.avx512.mask.getmant.ps.512
  437607, // llvm.x86.avx512.mask.getmant.sd
  437639, // llvm.x86.avx512.mask.getmant.ss
  437671, // llvm.x86.avx512.mask.max.sd.round
  437705, // llvm.x86.avx512.mask.max.ss.round
  437739, // llvm.x86.avx512.mask.min.sd.round
  437773, // llvm.x86.avx512.mask.min.ss.round
  437807, // llvm.x86.avx512.mask.mul.sd.round
  437841, // llvm.x86.avx512.mask.mul.ss.round
  437875, // llvm.x86.avx512.mask.pmov.db.128
  437908, // llvm.x86.avx512.mask.pmov.db.256
  437941, // llvm.x86.avx512.mask.pmov.db.512
  437974, // llvm.x86.avx512.mask.pmov.db.mem.128
  438011, // llvm.x86.avx512.mask.pmov.db.mem.256
  438048, // llvm.x86.avx512.mask.pmov.db.mem.512
  438085, // llvm.x86.avx512.mask.pmov.dw.128
  438118, // llvm.x86.avx512.mask.pmov.dw.256
  438151, // llvm.x86.avx512.mask.pmov.dw.512
  438184, // llvm.x86.avx512.mask.pmov.dw.mem.128
  438221, // llvm.x86.avx512.mask.pmov.dw.mem.256
  438258, // llvm.x86.avx512.mask.pmov.dw.mem.512
  438295, // llvm.x86.avx512.mask.pmov.qb.128
  438328, // llvm.x86.avx512.mask.pmov.qb.256
  438361, // llvm.x86.avx512.mask.pmov.qb.512
  438394, // llvm.x86.avx512.mask.pmov.qb.mem.128
  438431, // llvm.x86.avx512.mask.pmov.qb.mem.256
  438468, // llvm.x86.avx512.mask.pmov.qb.mem.512
  438505, // llvm.x86.avx512.mask.pmov.qd.128
  438538, // llvm.x86.avx512.mask.pmov.qd.mem.128
  438575, // llvm.x86.avx512.mask.pmov.qd.mem.256
  438612, // llvm.x86.avx512.mask.pmov.qd.mem.512
  438649, // llvm.x86.avx512.mask.pmov.qw.128
  438682, // llvm.x86.avx512.mask.pmov.qw.256
  438715, // llvm.x86.avx512.mask.pmov.qw.512
  438748, // llvm.x86.avx512.mask.pmov.qw.mem.128
  438785, // llvm.x86.avx512.mask.pmov.qw.mem.256
  438822, // llvm.x86.avx512.mask.pmov.qw.mem.512
  438859, // llvm.x86.avx512.mask.pmov.wb.128
  438892, // llvm.x86.avx512.mask.pmov.wb.mem.128
  438929, // llvm.x86.avx512.mask.pmov.wb.mem.256
  438966, // llvm.x86.avx512.mask.pmov.wb.mem.512
  439003, // llvm.x86.avx512.mask.pmovs.db.128
  439037, // llvm.x86.avx512.mask.pmovs.db.256
  439071, // llvm.x86.avx512.mask.pmovs.db.512
  439105, // llvm.x86.avx512.mask.pmovs.db.mem.128
  439143, // llvm.x86.avx512.mask.pmovs.db.mem.256
  439181, // llvm.x86.avx512.mask.pmovs.db.mem.512
  439219, // llvm.x86.avx512.mask.pmovs.dw.128
  439253, // llvm.x86.avx512.mask.pmovs.dw.256
  439287, // llvm.x86.avx512.mask.pmovs.dw.512
  439321, // llvm.x86.avx512.mask.pmovs.dw.mem.128
  439359, // llvm.x86.avx512.mask.pmovs.dw.mem.256
  439397, // llvm.x86.avx512.mask.pmovs.dw.mem.512
  439435, // llvm.x86.avx512.mask.pmovs.qb.128
  439469, // llvm.x86.avx512.mask.pmovs.qb.256
  439503, // llvm.x86.avx512.mask.pmovs.qb.512
  439537, // llvm.x86.avx512.mask.pmovs.qb.mem.128
  439575, // llvm.x86.avx512.mask.pmovs.qb.mem.256
  439613, // llvm.x86.avx512.mask.pmovs.qb.mem.512
  439651, // llvm.x86.avx512.mask.pmovs.qd.128
  439685, // llvm.x86.avx512.mask.pmovs.qd.256
  439719, // llvm.x86.avx512.mask.pmovs.qd.512
  439753, // llvm.x86.avx512.mask.pmovs.qd.mem.128
  439791, // llvm.x86.avx512.mask.pmovs.qd.mem.256
  439829, // llvm.x86.avx512.mask.pmovs.qd.mem.512
  439867, // llvm.x86.avx512.mask.pmovs.qw.128
  439901, // llvm.x86.avx512.mask.pmovs.qw.256
  439935, // llvm.x86.avx512.mask.pmovs.qw.512
  439969, // llvm.x86.avx512.mask.pmovs.qw.mem.128
  440007, // llvm.x86.avx512.mask.pmovs.qw.mem.256
  440045, // llvm.x86.avx512.mask.pmovs.qw.mem.512
  440083, // llvm.x86.avx512.mask.pmovs.wb.128
  440117, // llvm.x86.avx512.mask.pmovs.wb.256
  440151, // llvm.x86.avx512.mask.pmovs.wb.512
  440185, // llvm.x86.avx512.mask.pmovs.wb.mem.128
  440223, // llvm.x86.avx512.mask.pmovs.wb.mem.256
  440261, // llvm.x86.avx512.mask.pmovs.wb.mem.512
  440299, // llvm.x86.avx512.mask.pmovus.db.128
  440334, // llvm.x86.avx512.mask.pmovus.db.256
  440369, // llvm.x86.avx512.mask.pmovus.db.512
  440404, // llvm.x86.avx512.mask.pmovus.db.mem.128
  440443, // llvm.x86.avx512.mask.pmovus.db.mem.256
  440482, // llvm.x86.avx512.mask.pmovus.db.mem.512
  440521, // llvm.x86.avx512.mask.pmovus.dw.128
  440556, // llvm.x86.avx512.mask.pmovus.dw.256
  440591, // llvm.x86.avx512.mask.pmovus.dw.512
  440626, // llvm.x86.avx512.mask.pmovus.dw.mem.128
  440665, // llvm.x86.avx512.mask.pmovus.dw.mem.256
  440704, // llvm.x86.avx512.mask.pmovus.dw.mem.512
  440743, // llvm.x86.avx512.mask.pmovus.qb.128
  440778, // llvm.x86.avx512.mask.pmovus.qb.256
  440813, // llvm.x86.avx512.mask.pmovus.qb.512
  440848, // llvm.x86.avx512.mask.pmovus.qb.mem.128
  440887, // llvm.x86.avx512.mask.pmovus.qb.mem.256
  440926, // llvm.x86.avx512.mask.pmovus.qb.mem.512
  440965, // llvm.x86.avx512.mask.pmovus.qd.128
  441000, // llvm.x86.avx512.mask.pmovus.qd.256
  441035, // llvm.x86.avx512.mask.pmovus.qd.512
  441070, // llvm.x86.avx512.mask.pmovus.qd.mem.128
  441109, // llvm.x86.avx512.mask.pmovus.qd.mem.256
  441148, // llvm.x86.avx512.mask.pmovus.qd.mem.512
  441187, // llvm.x86.avx512.mask.pmovus.qw.128
  441222, // llvm.x86.avx512.mask.pmovus.qw.256
  441257, // llvm.x86.avx512.mask.pmovus.qw.512
  441292, // llvm.x86.avx512.mask.pmovus.qw.mem.128
  441331, // llvm.x86.avx512.mask.pmovus.qw.mem.256
  441370, // llvm.x86.avx512.mask.pmovus.qw.mem.512
  441409, // llvm.x86.avx512.mask.pmovus.wb.128
  441444, // llvm.x86.avx512.mask.pmovus.wb.256
  441479, // llvm.x86.avx512.mask.pmovus.wb.512
  441514, // llvm.x86.avx512.mask.pmovus.wb.mem.128
  441553, // llvm.x86.avx512.mask.pmovus.wb.mem.256
  441592, // llvm.x86.avx512.mask.pmovus.wb.mem.512
  441631, // llvm.x86.avx512.mask.range.pd.128
  441665, // llvm.x86.avx512.mask.range.pd.256
  441699, // llvm.x86.avx512.mask.range.pd.512
  441733, // llvm.x86.avx512.mask.range.ps.128
  441767, // llvm.x86.avx512.mask.range.ps.256
  441801, // llvm.x86.avx512.mask.range.ps.512
  441835, // llvm.x86.avx512.mask.range.sd
  441865, // llvm.x86.avx512.mask.range.ss
  441895, // llvm.x86.avx512.mask.reduce.pd.128
  441930, // llvm.x86.avx512.mask.reduce.pd.256
  441965, // llvm.x86.avx512.mask.reduce.pd.512
  442000, // llvm.x86.avx512.mask.reduce.ps.128
  442035, // llvm.x86.avx512.mask.reduce.ps.256
  442070, // llvm.x86.avx512.mask.reduce.ps.512
  442105, // llvm.x86.avx512.mask.reduce.sd
  442136, // llvm.x86.avx512.mask.reduce.ss
  442167, // llvm.x86.avx512.mask.rndscale.pd.128
  442204, // llvm.x86.avx512.mask.rndscale.pd.256
  442241, // llvm.x86.avx512.mask.rndscale.pd.512
  442278, // llvm.x86.avx512.mask.rndscale.ps.128
  442315, // llvm.x86.avx512.mask.rndscale.ps.256
  442352, // llvm.x86.avx512.mask.rndscale.ps.512
  442389, // llvm.x86.avx512.mask.rndscale.sd
  442422, // llvm.x86.avx512.mask.rndscale.ss
  442455, // llvm.x86.avx512.mask.scalef.pd.128
  442490, // llvm.x86.avx512.mask.scalef.pd.256
  442525, // llvm.x86.avx512.mask.scalef.pd.512
  442560, // llvm.x86.avx512.mask.scalef.ps.128
  442595, // llvm.x86.avx512.mask.scalef.ps.256
  442630, // llvm.x86.avx512.mask.scalef.ps.512
  442665, // llvm.x86.avx512.mask.scalef.sd
  442696, // llvm.x86.avx512.mask.scalef.ss
  442727, // llvm.x86.avx512.mask.scatter.dpd.512
  442764, // llvm.x86.avx512.mask.scatter.dpi.512
  442801, // llvm.x86.avx512.mask.scatter.dpq.512
  442838, // llvm.x86.avx512.mask.scatter.dps.512
  442875, // llvm.x86.avx512.mask.scatter.qpd.512
  442912, // llvm.x86.avx512.mask.scatter.qpi.512
  442949, // llvm.x86.avx512.mask.scatter.qpq.512
  442986, // llvm.x86.avx512.mask.scatter.qps.512
  443023, // llvm.x86.avx512.mask.scatterdiv2.df
  443059, // llvm.x86.avx512.mask.scatterdiv2.di
  443095, // llvm.x86.avx512.mask.scatterdiv4.df
  443131, // llvm.x86.avx512.mask.scatterdiv4.di
  443167, // llvm.x86.avx512.mask.scatterdiv4.sf
  443203, // llvm.x86.avx512.mask.scatterdiv4.si
  443239, // llvm.x86.avx512.mask.scatterdiv8.sf
  443275, // llvm.x86.avx512.mask.scatterdiv8.si
  443311, // llvm.x86.avx512.mask.scattersiv2.df
  443347, // llvm.x86.avx512.mask.scattersiv2.di
  443383, // llvm.x86.avx512.mask.scattersiv4.df
  443419, // llvm.x86.avx512.mask.scattersiv4.di
  443455, // llvm.x86.avx512.mask.scattersiv4.sf
  443491, // llvm.x86.avx512.mask.scattersiv4.si
  443527, // llvm.x86.avx512.mask.scattersiv8.sf
  443563, // llvm.x86.avx512.mask.scattersiv8.si
  443599, // llvm.x86.avx512.mask.sqrt.sd
  443628, // llvm.x86.avx512.mask.sqrt.ss
  443657, // llvm.x86.avx512.mask.sub.sd.round
  443691, // llvm.x86.avx512.mask.sub.ss.round
  443725, // llvm.x86.avx512.mask.vcvtph2ps.512
  443760, // llvm.x86.avx512.mask.vcvtps2ph.128
  443795, // llvm.x86.avx512.mask.vcvtps2ph.256
  443830, // llvm.x86.avx512.mask.vcvtps2ph.512
  443865, // llvm.x86.avx512.maskz.fixupimm.pd.128
  443903, // llvm.x86.avx512.maskz.fixupimm.pd.256
  443941, // llvm.x86.avx512.maskz.fixupimm.pd.512
  443979, // llvm.x86.avx512.maskz.fixupimm.ps.128
  444017, // llvm.x86.avx512.maskz.fixupimm.ps.256
  444055, // llvm.x86.avx512.maskz.fixupimm.ps.512
  444093, // llvm.x86.avx512.maskz.fixupimm.sd
  444127, // llvm.x86.avx512.maskz.fixupimm.ss
  444161, // llvm.x86.avx512.max.pd.512
  444188, // llvm.x86.avx512.max.ps.512
  444215, // llvm.x86.avx512.min.pd.512
  444242, // llvm.x86.avx512.min.ps.512
  444269, // llvm.x86.avx512.mul.pd.512
  444296, // llvm.x86.avx512.mul.ps.512
  444323, // llvm.x86.avx512.packssdw.512
  444352, // llvm.x86.avx512.packsswb.512
  444381, // llvm.x86.avx512.packusdw.512
  444410, // llvm.x86.avx512.packuswb.512
  444439, // llvm.x86.avx512.pavg.b.512
  444466, // llvm.x86.avx512.pavg.w.512
  444493, // llvm.x86.avx512.permvar.df.256
  444524, // llvm.x86.avx512.permvar.df.512
  444555, // llvm.x86.avx512.permvar.di.256
  444586, // llvm.x86.avx512.permvar.di.512
  444617, // llvm.x86.avx512.permvar.hi.128
  444648, // llvm.x86.avx512.permvar.hi.256
  444679, // llvm.x86.avx512.permvar.hi.512
  444710, // llvm.x86.avx512.permvar.qi.128
  444741, // llvm.x86.avx512.permvar.qi.256
  444772, // llvm.x86.avx512.permvar.qi.512
  444803, // llvm.x86.avx512.permvar.sf.512
  444834, // llvm.x86.avx512.permvar.si.512
  444865, // llvm.x86.avx512.pmaddubs.w.512
  444896, // llvm.x86.avx512.pmaddw.d.512
  444925, // llvm.x86.avx512.pmul.hr.sw.512
  444956, // llvm.x86.avx512.pmulh.w.512
  444984, // llvm.x86.avx512.pmulhu.w.512
  445013, // llvm.x86.avx512.pmultishift.qb.128
  445048, // llvm.x86.avx512.pmultishift.qb.256
  445083, // llvm.x86.avx512.pmultishift.qb.512
  445118, // llvm.x86.avx512.psad.bw.512
  445146, // llvm.x86.avx512.pshuf.b.512
  445174, // llvm.x86.avx512.psll.d.512
  445201, // llvm.x86.avx512.psll.q.512
  445228, // llvm.x86.avx512.psll.w.512
  445255, // llvm.x86.avx512.pslli.d.512
  445283, // llvm.x86.avx512.pslli.q.512
  445311, // llvm.x86.avx512.pslli.w.512
  445339, // llvm.x86.avx512.psllv.d.512
  445367, // llvm.x86.avx512.psllv.q.512
  445395, // llvm.x86.avx512.psllv.w.128
  445423, // llvm.x86.avx512.psllv.w.256
  445451, // llvm.x86.avx512.psllv.w.512
  445479, // llvm.x86.avx512.psra.d.512
  445506, // llvm.x86.avx512.psra.q.128
  445533, // llvm.x86.avx512.psra.q.256
  445560, // llvm.x86.avx512.psra.q.512
  445587, // llvm.x86.avx512.psra.w.512
  445614, // llvm.x86.avx512.psrai.d.512
  445642, // llvm.x86.avx512.psrai.q.128
  445670, // llvm.x86.avx512.psrai.q.256
  445698, // llvm.x86.avx512.psrai.q.512
  445726, // llvm.x86.avx512.psrai.w.512
  445754, // llvm.x86.avx512.psrav.d.512
  445782, // llvm.x86.avx512.psrav.q.128
  445810, // llvm.x86.avx512.psrav.q.256
  445838, // llvm.x86.avx512.psrav.q.512
  445866, // llvm.x86.avx512.psrav.w.128
  445894, // llvm.x86.avx512.psrav.w.256
  445922, // llvm.x86.avx512.psrav.w.512
  445950, // llvm.x86.avx512.psrl.d.512
  445977, // llvm.x86.avx512.psrl.q.512
  446004, // llvm.x86.avx512.psrl.w.512
  446031, // llvm.x86.avx512.psrli.d.512
  446059, // llvm.x86.avx512.psrli.q.512
  446087, // llvm.x86.avx512.psrli.w.512
  446115, // llvm.x86.avx512.psrlv.d.512
  446143, // llvm.x86.avx512.psrlv.q.512
  446171, // llvm.x86.avx512.psrlv.w.128
  446199, // llvm.x86.avx512.psrlv.w.256
  446227, // llvm.x86.avx512.psrlv.w.512
  446255, // llvm.x86.avx512.pternlog.d.128
  446286, // llvm.x86.avx512.pternlog.d.256
  446317, // llvm.x86.avx512.pternlog.d.512
  446348, // llvm.x86.avx512.pternlog.q.128
  446379, // llvm.x86.avx512.pternlog.q.256
  446410, // llvm.x86.avx512.pternlog.q.512
  446441, // llvm.x86.avx512.rcp14.pd.128
  446470, // llvm.x86.avx512.rcp14.pd.256
  446499, // llvm.x86.avx512.rcp14.pd.512
  446528, // llvm.x86.avx512.rcp14.ps.128
  446557, // llvm.x86.avx512.rcp14.ps.256
  446586, // llvm.x86.avx512.rcp14.ps.512
  446615, // llvm.x86.avx512.rcp14.sd
  446640, // llvm.x86.avx512.rcp14.ss
  446665, // llvm.x86.avx512.rsqrt14.pd.128
  446696, // llvm.x86.avx512.rsqrt14.pd.256
  446727, // llvm.x86.avx512.rsqrt14.pd.512
  446758, // llvm.x86.avx512.rsqrt14.ps.128
  446789, // llvm.x86.avx512.rsqrt14.ps.256
  446820, // llvm.x86.avx512.rsqrt14.ps.512
  446851, // llvm.x86.avx512.rsqrt14.sd
  446878, // llvm.x86.avx512.rsqrt14.ss
  446905, // llvm.x86.avx512.scatter.dpd.512
  446937, // llvm.x86.avx512.scatter.dpi.512
  446969, // llvm.x86.avx512.scatter.dpq.512
  447001, // llvm.x86.avx512.scatter.dps.512
  447033, // llvm.x86.avx512.scatter.qpd.512
  447065, // llvm.x86.avx512.scatter.qpi.512
  447097, // llvm.x86.avx512.scatter.qpq.512
  447129, // llvm.x86.avx512.scatter.qps.512
  447161, // llvm.x86.avx512.scatterdiv2.df
  447192, // llvm.x86.avx512.scatterdiv2.di
  447223, // llvm.x86.avx512.scatterdiv4.df
  447254, // llvm.x86.avx512.scatterdiv4.di
  447285, // llvm.x86.avx512.scatterdiv4.sf
  447316, // llvm.x86.avx512.scatterdiv4.si
  447347, // llvm.x86.avx512.scatterdiv8.sf
  447378, // llvm.x86.avx512.scatterdiv8.si
  447409, // llvm.x86.avx512.scattersiv2.df
  447440, // llvm.x86.avx512.scattersiv2.di
  447471, // llvm.x86.avx512.scattersiv4.df
  447502, // llvm.x86.avx512.scattersiv4.di
  447533, // llvm.x86.avx512.scattersiv4.sf
  447564, // llvm.x86.avx512.scattersiv4.si
  447595, // llvm.x86.avx512.scattersiv8.sf
  447626, // llvm.x86.avx512.scattersiv8.si
  447657, // llvm.x86.avx512.sitofp.round
  447686, // llvm.x86.avx512.sqrt.pd.512
  447714, // llvm.x86.avx512.sqrt.ps.512
  447742, // llvm.x86.avx512.sub.pd.512
  447769, // llvm.x86.avx512.sub.ps.512
  447796, // llvm.x86.avx512.uitofp.round
  447825, // llvm.x86.avx512.vcomi.sd
  447850, // llvm.x86.avx512.vcomi.ss
  447875, // llvm.x86.avx512.vcvtsd2si32
  447903, // llvm.x86.avx512.vcvtsd2si64
  447931, // llvm.x86.avx512.vcvtsd2usi32
  447960, // llvm.x86.avx512.vcvtsd2usi64
  447989, // llvm.x86.avx512.vcvtss2si32
  448017, // llvm.x86.avx512.vcvtss2si64
  448045, // llvm.x86.avx512.vcvtss2usi32
  448074, // llvm.x86.avx512.vcvtss2usi64
  448103, // llvm.x86.avx512.vfmadd.f32
  448130, // llvm.x86.avx512.vfmadd.f64
  448157, // llvm.x86.avx512.vfmadd.pd.512
  448187, // llvm.x86.avx512.vfmadd.ps.512
  448217, // llvm.x86.avx512.vfmaddsub.pd.512
  448250, // llvm.x86.avx512.vfmaddsub.ps.512
  448283, // llvm.x86.avx512.vp2intersect.d.128
  448318, // llvm.x86.avx512.vp2intersect.d.256
  448353, // llvm.x86.avx512.vp2intersect.d.512
  448388, // llvm.x86.avx512.vp2intersect.q.128
  448423, // llvm.x86.avx512.vp2intersect.q.256
  448458, // llvm.x86.avx512.vp2intersect.q.512
  448493, // llvm.x86.avx512.vpdpbusd.128
  448522, // llvm.x86.avx512.vpdpbusd.256
  448551, // llvm.x86.avx512.vpdpbusd.512
  448580, // llvm.x86.avx512.vpdpbusds.128
  448610, // llvm.x86.avx512.vpdpbusds.256
  448640, // llvm.x86.avx512.vpdpbusds.512
  448670, // llvm.x86.avx512.vpdpwssd.128
  448699, // llvm.x86.avx512.vpdpwssd.256
  448728, // llvm.x86.avx512.vpdpwssd.512
  448757, // llvm.x86.avx512.vpdpwssds.128
  448787, // llvm.x86.avx512.vpdpwssds.256
  448817, // llvm.x86.avx512.vpdpwssds.512
  448847, // llvm.x86.avx512.vpermi2var.d.128
  448880, // llvm.x86.avx512.vpermi2var.d.256
  448913, // llvm.x86.avx512.vpermi2var.d.512
  448946, // llvm.x86.avx512.vpermi2var.hi.128
  448980, // llvm.x86.avx512.vpermi2var.hi.256
  449014, // llvm.x86.avx512.vpermi2var.hi.512
  449048, // llvm.x86.avx512.vpermi2var.pd.128
  449082, // llvm.x86.avx512.vpermi2var.pd.256
  449116, // llvm.x86.avx512.vpermi2var.pd.512
  449150, // llvm.x86.avx512.vpermi2var.ps.128
  449184, // llvm.x86.avx512.vpermi2var.ps.256
  449218, // llvm.x86.avx512.vpermi2var.ps.512
  449252, // llvm.x86.avx512.vpermi2var.q.128
  449285, // llvm.x86.avx512.vpermi2var.q.256
  449318, // llvm.x86.avx512.vpermi2var.q.512
  449351, // llvm.x86.avx512.vpermi2var.qi.128
  449385, // llvm.x86.avx512.vpermi2var.qi.256
  449419, // llvm.x86.avx512.vpermi2var.qi.512
  449453, // llvm.x86.avx512.vpermilvar.pd.512
  449487, // llvm.x86.avx512.vpermilvar.ps.512
  449521, // llvm.x86.avx512.vpmadd52h.uq.128
  449554, // llvm.x86.avx512.vpmadd52h.uq.256
  449587, // llvm.x86.avx512.vpmadd52h.uq.512
  449620, // llvm.x86.avx512.vpmadd52l.uq.128
  449653, // llvm.x86.avx512.vpmadd52l.uq.256
  449686, // llvm.x86.avx512.vpmadd52l.uq.512
  449719, // llvm.x86.avx512.vpshufbitqmb.128
  449752, // llvm.x86.avx512.vpshufbitqmb.256
  449785, // llvm.x86.avx512.vpshufbitqmb.512
  449818, // llvm.x86.avx512bf16.cvtne2ps2bf16.128
  449856, // llvm.x86.avx512bf16.cvtne2ps2bf16.256
  449894, // llvm.x86.avx512bf16.cvtne2ps2bf16.512
  449932, // llvm.x86.avx512bf16.cvtneps2bf16.256
  449969, // llvm.x86.avx512bf16.cvtneps2bf16.512
  450006, // llvm.x86.avx512bf16.dpbf16ps.128
  450039, // llvm.x86.avx512bf16.dpbf16ps.256
  450072, // llvm.x86.avx512bf16.dpbf16ps.512
  450105, // llvm.x86.avx512bf16.mask.cvtneps2bf16.128
  450147, // llvm.x86.avx512fp16.add.ph.512
  450178, // llvm.x86.avx512fp16.div.ph.512
  450209, // llvm.x86.avx512fp16.fpclass.ph.128
  450244, // llvm.x86.avx512fp16.fpclass.ph.256
  450279, // llvm.x86.avx512fp16.fpclass.ph.512
  450314, // llvm.x86.avx512fp16.mask.add.sh.round
  450352, // llvm.x86.avx512fp16.mask.cmp.ph.128
  450388, // llvm.x86.avx512fp16.mask.cmp.ph.256
  450424, // llvm.x86.avx512fp16.mask.cmp.ph.512
  450460, // llvm.x86.avx512fp16.mask.cmp.sh
  450492, // llvm.x86.avx512fp16.mask.div.sh.round
  450530, // llvm.x86.avx512fp16.mask.fpclass.sh
  450566, // llvm.x86.avx512fp16.mask.getexp.ph.128
  450605, // llvm.x86.avx512fp16.mask.getexp.ph.256
  450644, // llvm.x86.avx512fp16.mask.getexp.ph.512
  450683, // llvm.x86.avx512fp16.mask.getexp.sh
  450718, // llvm.x86.avx512fp16.mask.getmant.ph.128
  450758, // llvm.x86.avx512fp16.mask.getmant.ph.256
  450798, // llvm.x86.avx512fp16.mask.getmant.ph.512
  450838, // llvm.x86.avx512fp16.mask.getmant.sh
  450874, // llvm.x86.avx512fp16.mask.max.sh.round
  450912, // llvm.x86.avx512fp16.mask.min.sh.round
  450950, // llvm.x86.avx512fp16.mask.mul.sh.round
  450988, // llvm.x86.avx512fp16.mask.rcp.ph.128
  451024, // llvm.x86.avx512fp16.mask.rcp.ph.256
  451060, // llvm.x86.avx512fp16.mask.rcp.ph.512
  451096, // llvm.x86.avx512fp16.mask.rcp.sh
  451128, // llvm.x86.avx512fp16.mask.reduce.ph.128
  451167, // llvm.x86.avx512fp16.mask.reduce.ph.256
  451206, // llvm.x86.avx512fp16.mask.reduce.ph.512
  451245, // llvm.x86.avx512fp16.mask.reduce.sh
  451280, // llvm.x86.avx512fp16.mask.rndscale.ph.128
  451321, // llvm.x86.avx512fp16.mask.rndscale.ph.256
  451362, // llvm.x86.avx512fp16.mask.rndscale.ph.512
  451403, // llvm.x86.avx512fp16.mask.rndscale.sh
  451440, // llvm.x86.avx512fp16.mask.rsqrt.ph.128
  451478, // llvm.x86.avx512fp16.mask.rsqrt.ph.256
  451516, // llvm.x86.avx512fp16.mask.rsqrt.ph.512
  451554, // llvm.x86.avx512fp16.mask.rsqrt.sh
  451588, // llvm.x86.avx512fp16.mask.scalef.ph.128
  451627, // llvm.x86.avx512fp16.mask.scalef.ph.256
  451666, // llvm.x86.avx512fp16.mask.scalef.ph.512
  451705, // llvm.x86.avx512fp16.mask.scalef.sh
  451740, // llvm.x86.avx512fp16.mask.sqrt.sh
  451773, // llvm.x86.avx512fp16.mask.sub.sh.round
  451811, // llvm.x86.avx512fp16.mask.vcvtdq2ph.128
  451850, // llvm.x86.avx512fp16.mask.vcvtpd2ph.128
  451889, // llvm.x86.avx512fp16.mask.vcvtpd2ph.256
  451928, // llvm.x86.avx512fp16.mask.vcvtpd2ph.512
  451967, // llvm.x86.avx512fp16.mask.vcvtph2dq.128
  452006, // llvm.x86.avx512fp16.mask.vcvtph2dq.256
  452045, // llvm.x86.avx512fp16.mask.vcvtph2dq.512
  452084, // llvm.x86.avx512fp16.mask.vcvtph2pd.128
  452123, // llvm.x86.avx512fp16.mask.vcvtph2pd.256
  452162, // llvm.x86.avx512fp16.mask.vcvtph2pd.512
  452201, // llvm.x86.avx512fp16.mask.vcvtph2psx.128
  452241, // llvm.x86.avx512fp16.mask.vcvtph2psx.256
  452281, // llvm.x86.avx512fp16.mask.vcvtph2psx.512
  452321, // llvm.x86.avx512fp16.mask.vcvtph2qq.128
  452360, // llvm.x86.avx512fp16.mask.vcvtph2qq.256
  452399, // llvm.x86.avx512fp16.mask.vcvtph2qq.512
  452438, // llvm.x86.avx512fp16.mask.vcvtph2udq.128
  452478, // llvm.x86.avx512fp16.mask.vcvtph2udq.256
  452518, // llvm.x86.avx512fp16.mask.vcvtph2udq.512
  452558, // llvm.x86.avx512fp16.mask.vcvtph2uqq.128
  452598, // llvm.x86.avx512fp16.mask.vcvtph2uqq.256
  452638, // llvm.x86.avx512fp16.mask.vcvtph2uqq.512
  452678, // llvm.x86.avx512fp16.mask.vcvtph2uw.128
  452717, // llvm.x86.avx512fp16.mask.vcvtph2uw.256
  452756, // llvm.x86.avx512fp16.mask.vcvtph2uw.512
  452795, // llvm.x86.avx512fp16.mask.vcvtph2w.128
  452833, // llvm.x86.avx512fp16.mask.vcvtph2w.256
  452871, // llvm.x86.avx512fp16.mask.vcvtph2w.512
  452909, // llvm.x86.avx512fp16.mask.vcvtps2phx.128
  452949, // llvm.x86.avx512fp16.mask.vcvtps2phx.256
  452989, // llvm.x86.avx512fp16.mask.vcvtps2phx.512
  453029, // llvm.x86.avx512fp16.mask.vcvtqq2ph.128
  453068, // llvm.x86.avx512fp16.mask.vcvtqq2ph.256
  453107, // llvm.x86.avx512fp16.mask.vcvtsd2sh.round
  453148, // llvm.x86.avx512fp16.mask.vcvtsh2sd.round
  453189, // llvm.x86.avx512fp16.mask.vcvtsh2ss.round
  453230, // llvm.x86.avx512fp16.mask.vcvtss2sh.round
  453271, // llvm.x86.avx512fp16.mask.vcvttph2dq.128
  453311, // llvm.x86.avx512fp16.mask.vcvttph2dq.256
  453351, // llvm.x86.avx512fp16.mask.vcvttph2dq.512
  453391, // llvm.x86.avx512fp16.mask.vcvttph2qq.128
  453431, // llvm.x86.avx512fp16.mask.vcvttph2qq.256
  453471, // llvm.x86.avx512fp16.mask.vcvttph2qq.512
  453511, // llvm.x86.avx512fp16.mask.vcvttph2udq.128
  453552, // llvm.x86.avx512fp16.mask.vcvttph2udq.256
  453593, // llvm.x86.avx512fp16.mask.vcvttph2udq.512
  453634, // llvm.x86.avx512fp16.mask.vcvttph2uqq.128
  453675, // llvm.x86.avx512fp16.mask.vcvttph2uqq.256
  453716, // llvm.x86.avx512fp16.mask.vcvttph2uqq.512
  453757, // llvm.x86.avx512fp16.mask.vcvttph2uw.128
  453797, // llvm.x86.avx512fp16.mask.vcvttph2uw.256
  453837, // llvm.x86.avx512fp16.mask.vcvttph2uw.512
  453877, // llvm.x86.avx512fp16.mask.vcvttph2w.128
  453916, // llvm.x86.avx512fp16.mask.vcvttph2w.256
  453955, // llvm.x86.avx512fp16.mask.vcvttph2w.512
  453994, // llvm.x86.avx512fp16.mask.vcvtudq2ph.128
  454034, // llvm.x86.avx512fp16.mask.vcvtuqq2ph.128
  454074, // llvm.x86.avx512fp16.mask.vcvtuqq2ph.256
  454114, // llvm.x86.avx512fp16.mask.vfcmadd.cph.128
  454155, // llvm.x86.avx512fp16.mask.vfcmadd.cph.256
  454196, // llvm.x86.avx512fp16.mask.vfcmadd.cph.512
  454237, // llvm.x86.avx512fp16.mask.vfcmadd.csh
  454274, // llvm.x86.avx512fp16.mask.vfcmul.cph.128
  454314, // llvm.x86.avx512fp16.mask.vfcmul.cph.256
  454354, // llvm.x86.avx512fp16.mask.vfcmul.cph.512
  454394, // llvm.x86.avx512fp16.mask.vfcmul.csh
  454430, // llvm.x86.avx512fp16.mask.vfmadd.cph.128
  454470, // llvm.x86.avx512fp16.mask.vfmadd.cph.256
  454510, // llvm.x86.avx512fp16.mask.vfmadd.cph.512
  454550, // llvm.x86.avx512fp16.mask.vfmadd.csh
  454586, // llvm.x86.avx512fp16.mask.vfmul.cph.128
  454625, // llvm.x86.avx512fp16.mask.vfmul.cph.256
  454664, // llvm.x86.avx512fp16.mask.vfmul.cph.512
  454703, // llvm.x86.avx512fp16.mask.vfmul.csh
  454738, // llvm.x86.avx512fp16.maskz.vfcmadd.cph.128
  454780, // llvm.x86.avx512fp16.maskz.vfcmadd.cph.256
  454822, // llvm.x86.avx512fp16.maskz.vfcmadd.cph.512
  454864, // llvm.x86.avx512fp16.maskz.vfcmadd.csh
  454902, // llvm.x86.avx512fp16.maskz.vfmadd.cph.128
  454943, // llvm.x86.avx512fp16.maskz.vfmadd.cph.256
  454984, // llvm.x86.avx512fp16.maskz.vfmadd.cph.512
  455025, // llvm.x86.avx512fp16.maskz.vfmadd.csh
  455062, // llvm.x86.avx512fp16.max.ph.128
  455093, // llvm.x86.avx512fp16.max.ph.256
  455124, // llvm.x86.avx512fp16.max.ph.512
  455155, // llvm.x86.avx512fp16.min.ph.128
  455186, // llvm.x86.avx512fp16.min.ph.256
  455217, // llvm.x86.avx512fp16.min.ph.512
  455248, // llvm.x86.avx512fp16.mul.ph.512
  455279, // llvm.x86.avx512fp16.sqrt.ph.512
  455311, // llvm.x86.avx512fp16.sub.ph.512
  455342, // llvm.x86.avx512fp16.vcomi.sh
  455371, // llvm.x86.avx512fp16.vcvtsh2si32
  455403, // llvm.x86.avx512fp16.vcvtsh2si64
  455435, // llvm.x86.avx512fp16.vcvtsh2usi32
  455468, // llvm.x86.avx512fp16.vcvtsh2usi64
  455501, // llvm.x86.avx512fp16.vcvtsi2sh
  455531, // llvm.x86.avx512fp16.vcvtsi642sh
  455563, // llvm.x86.avx512fp16.vcvttsh2si32
  455596, // llvm.x86.avx512fp16.vcvttsh2si64
  455629, // llvm.x86.avx512fp16.vcvttsh2usi32
  455663, // llvm.x86.avx512fp16.vcvttsh2usi64
  455697, // llvm.x86.avx512fp16.vcvtusi2sh
  455728, // llvm.x86.avx512fp16.vcvtusi642sh
  455761, // llvm.x86.avx512fp16.vfmadd.f16
  455792, // llvm.x86.avx512fp16.vfmadd.ph.512
  455826, // llvm.x86.avx512fp16.vfmaddsub.ph.128
  455863, // llvm.x86.avx512fp16.vfmaddsub.ph.256
  455900, // llvm.x86.avx512fp16.vfmaddsub.ph.512
  455937, // llvm.x86.axor32
  455953, // llvm.x86.axor64
  455969, // llvm.x86.bmi.bextr.32
  455991, // llvm.x86.bmi.bextr.64
  456013, // llvm.x86.bmi.bzhi.32
  456034, // llvm.x86.bmi.bzhi.64
  456055, // llvm.x86.bmi.pdep.32
  456076, // llvm.x86.bmi.pdep.64
  456097, // llvm.x86.bmi.pext.32
  456118, // llvm.x86.bmi.pext.64
  456139, // llvm.x86.cast.tile.to.vector
  456168, // llvm.x86.cast.vector.to.tile
  456197, // llvm.x86.cldemote
  456215, // llvm.x86.clflushopt
  456235, // llvm.x86.clrssbsy
  456253, // llvm.x86.clui
  456267, // llvm.x86.clwb
  456281, // llvm.x86.clzero
  456297, // llvm.x86.cmpccxadd32
  456318, // llvm.x86.cmpccxadd64
  456339, // llvm.x86.directstore32
  456362, // llvm.x86.directstore64
  456385, // llvm.x86.encodekey128
  456407, // llvm.x86.encodekey256
  456429, // llvm.x86.enqcmd
  456445, // llvm.x86.enqcmds
  456462, // llvm.x86.flags.read.u32
  456486, // llvm.x86.flags.read.u64
  456510, // llvm.x86.flags.write.u32
  456535, // llvm.x86.flags.write.u64
  456560, // llvm.x86.fma.vfmaddsub.pd
  456586, // llvm.x86.fma.vfmaddsub.pd.256
  456616, // llvm.x86.fma.vfmaddsub.ps
  456642, // llvm.x86.fma.vfmaddsub.ps.256
  456672, // llvm.x86.fxrstor
  456689, // llvm.x86.fxrstor64
  456708, // llvm.x86.fxsave
  456724, // llvm.x86.fxsave64
  456742, // llvm.x86.incsspd
  456759, // llvm.x86.incsspq
  456776, // llvm.x86.int
  456789, // llvm.x86.invpcid
  456806, // llvm.x86.ldtilecfg
  456825, // llvm.x86.ldtilecfg.internal
  456853, // llvm.x86.llwpcb
  456869, // llvm.x86.loadiwkey
  456888, // llvm.x86.lwpins32
  456906, // llvm.x86.lwpins64
  456924, // llvm.x86.lwpval32
  456942, // llvm.x86.lwpval64
  456960, // llvm.x86.mmx.emms
  456978, // llvm.x86.mmx.maskmovq
  457000, // llvm.x86.mmx.movnt.dq
  457022, // llvm.x86.mmx.packssdw
  457044, // llvm.x86.mmx.packsswb
  457066, // llvm.x86.mmx.packuswb
  457088, // llvm.x86.mmx.padd.b
  457108, // llvm.x86.mmx.padd.d
  457128, // llvm.x86.mmx.padd.q
  457148, // llvm.x86.mmx.padd.w
  457168, // llvm.x86.mmx.padds.b
  457189, // llvm.x86.mmx.padds.w
  457210, // llvm.x86.mmx.paddus.b
  457232, // llvm.x86.mmx.paddus.w
  457254, // llvm.x86.mmx.palignr.b
  457277, // llvm.x86.mmx.pand
  457295, // llvm.x86.mmx.pandn
  457314, // llvm.x86.mmx.pavg.b
  457334, // llvm.x86.mmx.pavg.w
  457354, // llvm.x86.mmx.pcmpeq.b
  457376, // llvm.x86.mmx.pcmpeq.d
  457398, // llvm.x86.mmx.pcmpeq.w
  457420, // llvm.x86.mmx.pcmpgt.b
  457442, // llvm.x86.mmx.pcmpgt.d
  457464, // llvm.x86.mmx.pcmpgt.w
  457486, // llvm.x86.mmx.pextr.w
  457507, // llvm.x86.mmx.pinsr.w
  457528, // llvm.x86.mmx.pmadd.wd
  457550, // llvm.x86.mmx.pmaxs.w
  457571, // llvm.x86.mmx.pmaxu.b
  457592, // llvm.x86.mmx.pmins.w
  457613, // llvm.x86.mmx.pminu.b
  457634, // llvm.x86.mmx.pmovmskb
  457656, // llvm.x86.mmx.pmulh.w
  457677, // llvm.x86.mmx.pmulhu.w
  457699, // llvm.x86.mmx.pmull.w
  457720, // llvm.x86.mmx.pmulu.dq
  457742, // llvm.x86.mmx.por
  457759, // llvm.x86.mmx.psad.bw
  457780, // llvm.x86.mmx.psll.d
  457800, // llvm.x86.mmx.psll.q
  457820, // llvm.x86.mmx.psll.w
  457840, // llvm.x86.mmx.pslli.d
  457861, // llvm.x86.mmx.pslli.q
  457882, // llvm.x86.mmx.pslli.w
  457903, // llvm.x86.mmx.psra.d
  457923, // llvm.x86.mmx.psra.w
  457943, // llvm.x86.mmx.psrai.d
  457964, // llvm.x86.mmx.psrai.w
  457985, // llvm.x86.mmx.psrl.d
  458005, // llvm.x86.mmx.psrl.q
  458025, // llvm.x86.mmx.psrl.w
  458045, // llvm.x86.mmx.psrli.d
  458066, // llvm.x86.mmx.psrli.q
  458087, // llvm.x86.mmx.psrli.w
  458108, // llvm.x86.mmx.psub.b
  458128, // llvm.x86.mmx.psub.d
  458148, // llvm.x86.mmx.psub.q
  458168, // llvm.x86.mmx.psub.w
  458188, // llvm.x86.mmx.psubs.b
  458209, // llvm.x86.mmx.psubs.w
  458230, // llvm.x86.mmx.psubus.b
  458252, // llvm.x86.mmx.psubus.w
  458274, // llvm.x86.mmx.punpckhbw
  458297, // llvm.x86.mmx.punpckhdq
  458320, // llvm.x86.mmx.punpckhwd
  458343, // llvm.x86.mmx.punpcklbw
  458366, // llvm.x86.mmx.punpckldq
  458389, // llvm.x86.mmx.punpcklwd
  458412, // llvm.x86.mmx.pxor
  458430, // llvm.x86.monitorx
  458448, // llvm.x86.movdir64b
  458467, // llvm.x86.movrsdi
  458484, // llvm.x86.movrshi
  458501, // llvm.x86.movrsqi
  458518, // llvm.x86.movrssi
  458535, // llvm.x86.mwaitx
  458551, // llvm.x86.pclmulqdq
  458570, // llvm.x86.pclmulqdq.256
  458593, // llvm.x86.pclmulqdq.512
  458616, // llvm.x86.prefetchrs
  458636, // llvm.x86.ptwrite32
  458655, // llvm.x86.ptwrite64
  458674, // llvm.x86.rdfsbase.32
  458695, // llvm.x86.rdfsbase.64
  458716, // llvm.x86.rdgsbase.32
  458737, // llvm.x86.rdgsbase.64
  458758, // llvm.x86.rdpid
  458773, // llvm.x86.rdpkru
  458789, // llvm.x86.rdpmc
  458804, // llvm.x86.rdpru
  458819, // llvm.x86.rdrand.16
  458838, // llvm.x86.rdrand.32
  458857, // llvm.x86.rdrand.64
  458876, // llvm.x86.rdseed.16
  458895, // llvm.x86.rdseed.32
  458914, // llvm.x86.rdseed.64
  458933, // llvm.x86.rdsspd
  458949, // llvm.x86.rdsspq
  458965, // llvm.x86.rdtsc
  458980, // llvm.x86.rdtscp
  458996, // llvm.x86.rstorssp
  459014, // llvm.x86.saveprevssp
  459035, // llvm.x86.seh.ehguard
  459056, // llvm.x86.seh.ehregnode
  459079, // llvm.x86.seh.lsda
  459097, // llvm.x86.senduipi
  459115, // llvm.x86.serialize
  459134, // llvm.x86.setssbsy
  459152, // llvm.x86.sha1msg1
  459170, // llvm.x86.sha1msg2
  459188, // llvm.x86.sha1nexte
  459207, // llvm.x86.sha1rnds4
  459226, // llvm.x86.sha256msg1
  459246, // llvm.x86.sha256msg2
  459266, // llvm.x86.sha256rnds2
  459287, // llvm.x86.slwpcb
  459303, // llvm.x86.sse.cmp.ps
  459323, // llvm.x86.sse.cmp.ss
  459343, // llvm.x86.sse.comieq.ss
  459366, // llvm.x86.sse.comige.ss
  459389, // llvm.x86.sse.comigt.ss
  459412, // llvm.x86.sse.comile.ss
  459435, // llvm.x86.sse.comilt.ss
  459458, // llvm.x86.sse.comineq.ss
  459482, // llvm.x86.sse.cvtpd2pi
  459504, // llvm.x86.sse.cvtpi2pd
  459526, // llvm.x86.sse.cvtpi2ps
  459548, // llvm.x86.sse.cvtps2pi
  459570, // llvm.x86.sse.cvtss2si
  459592, // llvm.x86.sse.cvtss2si64
  459616, // llvm.x86.sse.cvttpd2pi
  459639, // llvm.x86.sse.cvttps2pi
  459662, // llvm.x86.sse.cvttss2si
  459685, // llvm.x86.sse.cvttss2si64
  459710, // llvm.x86.sse.ldmxcsr
  459731, // llvm.x86.sse.max.ps
  459751, // llvm.x86.sse.max.ss
  459771, // llvm.x86.sse.min.ps
  459791, // llvm.x86.sse.min.ss
  459811, // llvm.x86.sse.movmsk.ps
  459834, // llvm.x86.sse.pshuf.w
  459855, // llvm.x86.sse.rcp.ps
  459875, // llvm.x86.sse.rcp.ss
  459895, // llvm.x86.sse.rsqrt.ps
  459917, // llvm.x86.sse.rsqrt.ss
  459939, // llvm.x86.sse.sfence
  459959, // llvm.x86.sse.stmxcsr
  459980, // llvm.x86.sse.ucomieq.ss
  460004, // llvm.x86.sse.ucomige.ss
  460028, // llvm.x86.sse.ucomigt.ss
  460052, // llvm.x86.sse.ucomile.ss
  460076, // llvm.x86.sse.ucomilt.ss
  460100, // llvm.x86.sse.ucomineq.ss
  460125, // llvm.x86.sse2.clflush
  460147, // llvm.x86.sse2.cmp.pd
  460168, // llvm.x86.sse2.cmp.sd
  460189, // llvm.x86.sse2.comieq.sd
  460213, // llvm.x86.sse2.comige.sd
  460237, // llvm.x86.sse2.comigt.sd
  460261, // llvm.x86.sse2.comile.sd
  460285, // llvm.x86.sse2.comilt.sd
  460309, // llvm.x86.sse2.comineq.sd
  460334, // llvm.x86.sse2.cvtpd2dq
  460357, // llvm.x86.sse2.cvtpd2ps
  460380, // llvm.x86.sse2.cvtps2dq
  460403, // llvm.x86.sse2.cvtsd2si
  460426, // llvm.x86.sse2.cvtsd2si64
  460451, // llvm.x86.sse2.cvtsd2ss
  460474, // llvm.x86.sse2.cvttpd2dq
  460498, // llvm.x86.sse2.cvttps2dq
  460522, // llvm.x86.sse2.cvttsd2si
  460546, // llvm.x86.sse2.cvttsd2si64
  460572, // llvm.x86.sse2.lfence
  460593, // llvm.x86.sse2.maskmov.dqu
  460619, // llvm.x86.sse2.max.pd
  460640, // llvm.x86.sse2.max.sd
  460661, // llvm.x86.sse2.mfence
  460682, // llvm.x86.sse2.min.pd
  460703, // llvm.x86.sse2.min.sd
  460724, // llvm.x86.sse2.movmsk.pd
  460748, // llvm.x86.sse2.packssdw.128
  460775, // llvm.x86.sse2.packsswb.128
  460802, // llvm.x86.sse2.packuswb.128
  460829, // llvm.x86.sse2.pause
  460849, // llvm.x86.sse2.pavg.b
  460870, // llvm.x86.sse2.pavg.w
  460891, // llvm.x86.sse2.pmadd.wd
  460914, // llvm.x86.sse2.pmovmskb.128
  460941, // llvm.x86.sse2.pmulh.w
  460963, // llvm.x86.sse2.pmulhu.w
  460986, // llvm.x86.sse2.psad.bw
  461008, // llvm.x86.sse2.psll.d
  461029, // llvm.x86.sse2.psll.q
  461050, // llvm.x86.sse2.psll.w
  461071, // llvm.x86.sse2.pslli.d
  461093, // llvm.x86.sse2.pslli.q
  461115, // llvm.x86.sse2.pslli.w
  461137, // llvm.x86.sse2.psra.d
  461158, // llvm.x86.sse2.psra.w
  461179, // llvm.x86.sse2.psrai.d
  461201, // llvm.x86.sse2.psrai.w
  461223, // llvm.x86.sse2.psrl.d
  461244, // llvm.x86.sse2.psrl.q
  461265, // llvm.x86.sse2.psrl.w
  461286, // llvm.x86.sse2.psrli.d
  461308, // llvm.x86.sse2.psrli.q
  461330, // llvm.x86.sse2.psrli.w
  461352, // llvm.x86.sse2.ucomieq.sd
  461377, // llvm.x86.sse2.ucomige.sd
  461402, // llvm.x86.sse2.ucomigt.sd
  461427, // llvm.x86.sse2.ucomile.sd
  461452, // llvm.x86.sse2.ucomilt.sd
  461477, // llvm.x86.sse2.ucomineq.sd
  461503, // llvm.x86.sse3.addsub.pd
  461527, // llvm.x86.sse3.addsub.ps
  461551, // llvm.x86.sse3.hadd.pd
  461573, // llvm.x86.sse3.hadd.ps
  461595, // llvm.x86.sse3.hsub.pd
  461617, // llvm.x86.sse3.hsub.ps
  461639, // llvm.x86.sse3.ldu.dq
  461660, // llvm.x86.sse3.monitor
  461682, // llvm.x86.sse3.mwait
  461702, // llvm.x86.sse41.blendvpd
  461726, // llvm.x86.sse41.blendvps
  461750, // llvm.x86.sse41.dppd
  461770, // llvm.x86.sse41.dpps
  461790, // llvm.x86.sse41.insertps
  461814, // llvm.x86.sse41.mpsadbw
  461837, // llvm.x86.sse41.packusdw
  461861, // llvm.x86.sse41.pblendvb
  461885, // llvm.x86.sse41.phminposuw
  461911, // llvm.x86.sse41.ptestc
  461933, // llvm.x86.sse41.ptestnzc
  461957, // llvm.x86.sse41.ptestz
  461979, // llvm.x86.sse41.round.pd
  462003, // llvm.x86.sse41.round.ps
  462027, // llvm.x86.sse41.round.sd
  462051, // llvm.x86.sse41.round.ss
  462075, // llvm.x86.sse42.crc32.32.16
  462102, // llvm.x86.sse42.crc32.32.32
  462129, // llvm.x86.sse42.crc32.32.8
  462155, // llvm.x86.sse42.crc32.64.64
  462182, // llvm.x86.sse42.pcmpestri128
  462210, // llvm.x86.sse42.pcmpestria128
  462239, // llvm.x86.sse42.pcmpestric128
  462268, // llvm.x86.sse42.pcmpestrio128
  462297, // llvm.x86.sse42.pcmpestris128
  462326, // llvm.x86.sse42.pcmpestriz128
  462355, // llvm.x86.sse42.pcmpestrm128
  462383, // llvm.x86.sse42.pcmpistri128
  462411, // llvm.x86.sse42.pcmpistria128
  462440, // llvm.x86.sse42.pcmpistric128
  462469, // llvm.x86.sse42.pcmpistrio128
  462498, // llvm.x86.sse42.pcmpistris128
  462527, // llvm.x86.sse42.pcmpistriz128
  462556, // llvm.x86.sse42.pcmpistrm128
  462584, // llvm.x86.sse4a.extrq
  462605, // llvm.x86.sse4a.extrqi
  462627, // llvm.x86.sse4a.insertq
  462650, // llvm.x86.sse4a.insertqi
  462674, // llvm.x86.ssse3.pabs.b
  462696, // llvm.x86.ssse3.pabs.d
  462718, // llvm.x86.ssse3.pabs.w
  462740, // llvm.x86.ssse3.phadd.d
  462763, // llvm.x86.ssse3.phadd.d.128
  462790, // llvm.x86.ssse3.phadd.sw
  462814, // llvm.x86.ssse3.phadd.sw.128
  462842, // llvm.x86.ssse3.phadd.w
  462865, // llvm.x86.ssse3.phadd.w.128
  462892, // llvm.x86.ssse3.phsub.d
  462915, // llvm.x86.ssse3.phsub.d.128
  462942, // llvm.x86.ssse3.phsub.sw
  462966, // llvm.x86.ssse3.phsub.sw.128
  462994, // llvm.x86.ssse3.phsub.w
  463017, // llvm.x86.ssse3.phsub.w.128
  463044, // llvm.x86.ssse3.pmadd.ub.sw
  463071, // llvm.x86.ssse3.pmadd.ub.sw.128
  463102, // llvm.x86.ssse3.pmul.hr.sw
  463128, // llvm.x86.ssse3.pmul.hr.sw.128
  463158, // llvm.x86.ssse3.pshuf.b
  463181, // llvm.x86.ssse3.pshuf.b.128
  463208, // llvm.x86.ssse3.psign.b
  463231, // llvm.x86.ssse3.psign.b.128
  463258, // llvm.x86.ssse3.psign.d
  463281, // llvm.x86.ssse3.psign.d.128
  463308, // llvm.x86.ssse3.psign.w
  463331, // llvm.x86.ssse3.psign.w.128
  463358, // llvm.x86.sttilecfg
  463377, // llvm.x86.stui
  463391, // llvm.x86.subborrow.32
  463413, // llvm.x86.subborrow.64
  463435, // llvm.x86.tbm.bextri.u32
  463459, // llvm.x86.tbm.bextri.u64
  463483, // llvm.x86.tcmmimfp16ps
  463505, // llvm.x86.tcmmimfp16ps.internal
  463536, // llvm.x86.tcmmrlfp16ps
  463558, // llvm.x86.tcmmrlfp16ps.internal
  463589, // llvm.x86.tcvtrowd2ps
  463610, // llvm.x86.tcvtrowd2ps.internal
  463640, // llvm.x86.tcvtrowd2psi
  463662, // llvm.x86.tcvtrowps2bf16h
  463687, // llvm.x86.tcvtrowps2bf16h.internal
  463721, // llvm.x86.tcvtrowps2bf16hi
  463747, // llvm.x86.tcvtrowps2bf16l
  463772, // llvm.x86.tcvtrowps2bf16l.internal
  463806, // llvm.x86.tcvtrowps2bf16li
  463832, // llvm.x86.tcvtrowps2phh
  463855, // llvm.x86.tcvtrowps2phh.internal
  463887, // llvm.x86.tcvtrowps2phhi
  463911, // llvm.x86.tcvtrowps2phl
  463934, // llvm.x86.tcvtrowps2phl.internal
  463966, // llvm.x86.tcvtrowps2phli
  463990, // llvm.x86.tdpbf16ps
  464009, // llvm.x86.tdpbf16ps.internal
  464037, // llvm.x86.tdpbf8ps
  464055, // llvm.x86.tdpbf8ps.internal
  464082, // llvm.x86.tdpbhf8ps
  464101, // llvm.x86.tdpbhf8ps.internal
  464129, // llvm.x86.tdpbssd
  464146, // llvm.x86.tdpbssd.internal
  464172, // llvm.x86.tdpbsud
  464189, // llvm.x86.tdpbsud.internal
  464215, // llvm.x86.tdpbusd
  464232, // llvm.x86.tdpbusd.internal
  464258, // llvm.x86.tdpbuud
  464275, // llvm.x86.tdpbuud.internal
  464301, // llvm.x86.tdpfp16ps
  464320, // llvm.x86.tdpfp16ps.internal
  464348, // llvm.x86.tdphbf8ps
  464367, // llvm.x86.tdphbf8ps.internal
  464395, // llvm.x86.tdphf8ps
  464413, // llvm.x86.tdphf8ps.internal
  464440, // llvm.x86.testui
  464456, // llvm.x86.tileloadd64
  464477, // llvm.x86.tileloadd64.internal
  464507, // llvm.x86.tileloaddrs64
  464530, // llvm.x86.tileloaddrs64.internal
  464562, // llvm.x86.tileloaddrst164
  464587, // llvm.x86.tileloaddrst164.internal
  464621, // llvm.x86.tileloaddt164
  464644, // llvm.x86.tileloaddt164.internal
  464676, // llvm.x86.tilemovrow
  464696, // llvm.x86.tilemovrow.internal
  464725, // llvm.x86.tilemovrowi
  464746, // llvm.x86.tilerelease
  464767, // llvm.x86.tilestored64
  464789, // llvm.x86.tilestored64.internal
  464820, // llvm.x86.tilezero
  464838, // llvm.x86.tilezero.internal
  464865, // llvm.x86.tmmultf32ps
  464886, // llvm.x86.tmmultf32ps.internal
  464916, // llvm.x86.tpause
  464932, // llvm.x86.umonitor
  464950, // llvm.x86.umwait
  464966, // llvm.x86.urdmsr
  464982, // llvm.x86.uwrmsr
  464998, // llvm.x86.vbcstnebf162ps128
  465025, // llvm.x86.vbcstnebf162ps256
  465052, // llvm.x86.vbcstnesh2ps128
  465077, // llvm.x86.vbcstnesh2ps256
  465102, // llvm.x86.vcvtneebf162ps128
  465129, // llvm.x86.vcvtneebf162ps256
  465156, // llvm.x86.vcvtneeph2ps128
  465181, // llvm.x86.vcvtneeph2ps256
  465206, // llvm.x86.vcvtneobf162ps128
  465233, // llvm.x86.vcvtneobf162ps256
  465260, // llvm.x86.vcvtneoph2ps128
  465285, // llvm.x86.vcvtneoph2ps256
  465310, // llvm.x86.vcvtneps2bf16128
  465336, // llvm.x86.vcvtneps2bf16256
  465362, // llvm.x86.vcvtps2ph.128
  465385, // llvm.x86.vcvtps2ph.256
  465408, // llvm.x86.vgf2p8affineinvqb.128
  465439, // llvm.x86.vgf2p8affineinvqb.256
  465470, // llvm.x86.vgf2p8affineinvqb.512
  465501, // llvm.x86.vgf2p8affineqb.128
  465529, // llvm.x86.vgf2p8affineqb.256
  465557, // llvm.x86.vgf2p8affineqb.512
  465585, // llvm.x86.vgf2p8mulb.128
  465609, // llvm.x86.vgf2p8mulb.256
  465633, // llvm.x86.vgf2p8mulb.512
  465657, // llvm.x86.vsha512msg1
  465678, // llvm.x86.vsha512msg2
  465699, // llvm.x86.vsha512rnds2
  465721, // llvm.x86.vsm3msg1
  465739, // llvm.x86.vsm3msg2
  465757, // llvm.x86.vsm3rnds2
  465776, // llvm.x86.vsm4key4128
  465797, // llvm.x86.vsm4key4256
  465818, // llvm.x86.vsm4key4512
  465839, // llvm.x86.vsm4rnds4128
  465861, // llvm.x86.vsm4rnds4256
  465883, // llvm.x86.vsm4rnds4512
  465905, // llvm.x86.wbinvd
  465921, // llvm.x86.wbnoinvd
  465939, // llvm.x86.wrfsbase.32
  465960, // llvm.x86.wrfsbase.64
  465981, // llvm.x86.wrgsbase.32
  466002, // llvm.x86.wrgsbase.64
  466023, // llvm.x86.wrpkru
  466039, // llvm.x86.wrssd
  466054, // llvm.x86.wrssq
  466069, // llvm.x86.wrussd
  466085, // llvm.x86.wrussq
  466101, // llvm.x86.xabort
  466117, // llvm.x86.xbegin
  466133, // llvm.x86.xend
  466147, // llvm.x86.xgetbv
  466163, // llvm.x86.xop.vfrcz.pd
  466185, // llvm.x86.xop.vfrcz.pd.256
  466211, // llvm.x86.xop.vfrcz.ps
  466233, // llvm.x86.xop.vfrcz.ps.256
  466259, // llvm.x86.xop.vfrcz.sd
  466281, // llvm.x86.xop.vfrcz.ss
  466303, // llvm.x86.xop.vpermil2pd
  466327, // llvm.x86.xop.vpermil2pd.256
  466355, // llvm.x86.xop.vpermil2ps
  466379, // llvm.x86.xop.vpermil2ps.256
  466407, // llvm.x86.xop.vphaddbd
  466429, // llvm.x86.xop.vphaddbq
  466451, // llvm.x86.xop.vphaddbw
  466473, // llvm.x86.xop.vphadddq
  466495, // llvm.x86.xop.vphaddubd
  466518, // llvm.x86.xop.vphaddubq
  466541, // llvm.x86.xop.vphaddubw
  466564, // llvm.x86.xop.vphaddudq
  466587, // llvm.x86.xop.vphadduwd
  466610, // llvm.x86.xop.vphadduwq
  466633, // llvm.x86.xop.vphaddwd
  466655, // llvm.x86.xop.vphaddwq
  466677, // llvm.x86.xop.vphsubbw
  466699, // llvm.x86.xop.vphsubdq
  466721, // llvm.x86.xop.vphsubwd
  466743, // llvm.x86.xop.vpmacsdd
  466765, // llvm.x86.xop.vpmacsdqh
  466788, // llvm.x86.xop.vpmacsdql
  466811, // llvm.x86.xop.vpmacssdd
  466834, // llvm.x86.xop.vpmacssdqh
  466858, // llvm.x86.xop.vpmacssdql
  466882, // llvm.x86.xop.vpmacsswd
  466905, // llvm.x86.xop.vpmacssww
  466928, // llvm.x86.xop.vpmacswd
  466950, // llvm.x86.xop.vpmacsww
  466972, // llvm.x86.xop.vpmadcsswd
  466996, // llvm.x86.xop.vpmadcswd
  467019, // llvm.x86.xop.vpperm
  467039, // llvm.x86.xop.vpshab
  467059, // llvm.x86.xop.vpshad
  467079, // llvm.x86.xop.vpshaq
  467099, // llvm.x86.xop.vpshaw
  467119, // llvm.x86.xop.vpshlb
  467139, // llvm.x86.xop.vpshld
  467159, // llvm.x86.xop.vpshlq
  467179, // llvm.x86.xop.vpshlw
  467199, // llvm.x86.xresldtrk
  467218, // llvm.x86.xrstor
  467234, // llvm.x86.xrstor64
  467252, // llvm.x86.xrstors
  467269, // llvm.x86.xrstors64
  467288, // llvm.x86.xsave
  467303, // llvm.x86.xsave64
  467320, // llvm.x86.xsavec
  467336, // llvm.x86.xsavec64
  467354, // llvm.x86.xsaveopt
  467372, // llvm.x86.xsaveopt64
  467392, // llvm.x86.xsaves
  467408, // llvm.x86.xsaves64
  467426, // llvm.x86.xsetbv
  467442, // llvm.x86.xsusldtrk
  467461, // llvm.x86.xtest
  467476, // llvm.xcore.bitrev
  467494, // llvm.xcore.checkevent
  467516, // llvm.xcore.chkct
  467533, // llvm.xcore.clre
  467549, // llvm.xcore.clrpt
  467566, // llvm.xcore.clrsr
  467583, // llvm.xcore.crc32
  467600, // llvm.xcore.crc8
  467616, // llvm.xcore.edu
  467631, // llvm.xcore.eeu
  467646, // llvm.xcore.endin
  467663, // llvm.xcore.freer
  467680, // llvm.xcore.geted
  467697, // llvm.xcore.getet
  467714, // llvm.xcore.getid
  467731, // llvm.xcore.getps
  467748, // llvm.xcore.getr
  467764, // llvm.xcore.getst
  467781, // llvm.xcore.getts
  467798, // llvm.xcore.in
  467812, // llvm.xcore.inct
  467828, // llvm.xcore.initcp
  467846, // llvm.xcore.initdp
  467864, // llvm.xcore.initlr
  467882, // llvm.xcore.initpc
  467900, // llvm.xcore.initsp
  467918, // llvm.xcore.inshr
  467935, // llvm.xcore.int
  467950, // llvm.xcore.mjoin
  467967, // llvm.xcore.msync
  467984, // llvm.xcore.out
  467999, // llvm.xcore.outct
  468016, // llvm.xcore.outshr
  468034, // llvm.xcore.outt
  468050, // llvm.xcore.peek
  468066, // llvm.xcore.setc
  468082, // llvm.xcore.setclk
  468100, // llvm.xcore.setd
  468116, // llvm.xcore.setev
  468133, // llvm.xcore.setps
  468150, // llvm.xcore.setpsc
  468168, // llvm.xcore.setpt
  468185, // llvm.xcore.setrdy
  468203, // llvm.xcore.setsr
  468220, // llvm.xcore.settw
  468237, // llvm.xcore.setv
  468253, // llvm.xcore.sext
  468269, // llvm.xcore.ssync
  468286, // llvm.xcore.syncr
  468303, // llvm.xcore.testct
  468321, // llvm.xcore.testwct
  468340, // llvm.xcore.waitevent
  468361, // llvm.xcore.zext

}; // IntrinsicNameOffsetTable

#endif

// Intrinsic ID to overload bitset.
#ifdef GET_INTRINSIC_OVERLOAD_TABLE
static constexpr uint8_t OTable[] = {
  0
   | (1<<1) | (1<<2) | (1<<3) | (1<<5),
  0 | (1<<4) | (1<<5) | (1<<7),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<3),
  0,
  0,
  0,
  0 | (1<<1) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2),
  0 | (1<<3),
  0 | (1<<5) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<1) | (1<<3) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<2) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0,
  0 | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0,
  0,
  0,
  0 | (1<<5) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<7),
  0 | (1<<1) | (1<<2),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<1) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0,
  0,
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0,
  0,
  0,
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0,
  0 | (1<<3) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<5),
  0 | (1<<3) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1),
  0,
  0,
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1),
  0,
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1),
  0 | (1<<4) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2),
  0,
  0 | (1<<6),
  0 | (1<<0) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0),
  0 | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2),
  0,
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<6),
  0 | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<5),
  0 | (1<<7),
  0 | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<6),
  0 | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<2) | (1<<5),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<4) | (1<<7),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<4) | (1<<5),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<5),
  0 | (1<<0) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<3) | (1<<6),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<4) | (1<<5),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<3) | (1<<4),
  0 | (1<<1) | (1<<2) | (1<<7),
  0 | (1<<0) | (1<<5) | (1<<6),
  0 | (1<<3) | (1<<4),
  0 | (1<<1) | (1<<2) | (1<<7),
  0 | (1<<0) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<6) | (1<<7),
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<6),
  0 | (1<<2) | (1<<3) | (1<<7),
  0 | (1<<0),
  0 | (1<<0) | (1<<1) | (1<<2),
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<3) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<5) | (1<<6),
  0 | (1<<7),
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0,
  0,
  0 | (1<<0) | (1<<5),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0,
  0,
  0,
  0,
  0,
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<5) | (1<<6),
  0,
  0 | (1<<2) | (1<<6) | (1<<7),
  0,
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<4),
  0 | (1<<7),
  0,
  0,
  0 | (1<<1) | (1<<2) | (1<<4),
  0,
  0,
  0 | (1<<2),
  0 | (1<<0) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1),
  0,
  0,
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<6) | (1<<7),
  0 | (1<<4) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<6),
  0 | (1<<1) | (1<<2),
  0 | (1<<7),
  0 | (1<<0) | (1<<2) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<2) | (1<<3) | (1<<4) | (1<<6),
  0 | (1<<0),
  0,
  0 | (1<<7),
  0 | (1<<5),
  0 | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0 | (1<<3),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<7),
  0 | (1<<3),
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<4) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<2) | (1<<4),
  0,
  0,
  0,
  0 | (1<<1) | (1<<2) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<2) | (1<<4) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<1) | (1<<3) | (1<<4) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<4) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0 | (1<<0) | (1<<1) | (1<<2),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<4) | (1<<5),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<2) | (1<<3),
  0,
  0,
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0,
  0,
  0,
  0,
  0,
  0 | (1<<1) | (1<<2),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<2) | (1<<3),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<7),
  0 | (1<<0),
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<2) | (1<<3),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<6) | (1<<7),
  0,
  0,
  0 | (1<<4),
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<2),
  0,
  0,
  0,
  0 | (1<<3),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<2) | (1<<3),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2),
  0,
  0,
  0,
  0,
  0,
  0 | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4),
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0),
  0,
  0 | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<7),
  0 | (1<<1),
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<1),
  0,
  0,
  0,
  0 | (1<<2),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<1),
  0 | (1<<5),
  0,
  0,
  0,
  0,
  0 | (1<<0),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<7),
  0 | (1<<0),
  0,
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1),
  0,
  0,
  0,
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2),
  0,
  0 | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0,
  0 | (1<<2),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<7),
  0 | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<7),
  0 | (1<<0) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<6),
  0 | (1<<0),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<5) | (1<<6),
  0,
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0,
  0 | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5),
  0,
  0,
  0,
  0 | (1<<1) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0,
  0,
  0,
  0 | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<0),
  0,
  0,
  0,
  0,
  0,
  0 | (1<<6),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<6),
  0 | (1<<3),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<0) | (1<<1),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<1) | (1<<3) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<7),
  0 | (1<<0) | (1<<1) | (1<<3) | (1<<4) | (1<<7),
  0 | (1<<0) | (1<<1)
};

return (OTable[id/8] & (1 << (id%8))) != 0;
#endif // GET_INTRINSIC_OVERLOAD_TABLE

// Global intrinsic function declaration type table.
#ifdef GET_INTRINSIC_GENERATOR_GLOBAL
static constexpr uint16_t IIT_Table[] = {
  (1U<<15) | 1022, 0x7f2f, 0x4f, 0xee, (1U<<15) | 20220, (1U<<15) | 20221, 0x1, 
  0x1, 0x1, 0x1, 0x21, (1U<<15) | 14212, 0x7f2f, 0x4e0, 0x7f2f, 
  0x10, 0x7f2f, (1U<<15) | 17288, 0x7f1f, 0x7f1f, (1U<<15) | 14310, (1U<<15) | 14307, (1U<<15) | 20193, 
  0x7f0f, 0x7f2f, 0x7f2f, 0xee0, (1U<<15) | 17279, (1U<<15) | 20217, 0x32f, 0x2f3, 
  (1U<<15) | 17288, 0x1f, (1U<<15) | 20198, (1U<<15) | 4272, (1U<<15) | 20193, (1U<<15) | 20205, 0xeee, 0xe0, 
  0xe, 0xee0, (1U<<15) | 19379, (1U<<15) | 19374, (1U<<15) | 19374, (1U<<15) | 19403, (1U<<15) | 14083, 0xe0, 
  0xe1, (1U<<15) | 20196, (1U<<15) | 21285, (1U<<15) | 21365, 0xe, (1U<<15) | 19403, (1U<<15) | 19384, (1U<<15) | 19261, 
  (1U<<15) | 19390, (1U<<15) | 19390, 0x1, 0xe, 0xee, 0xee, 0x14ee, 0xe0, 
  (1U<<15) | 19404, 0x1f, 0x2ee, (1U<<15) | 1331, (1U<<15) | 21358, (1U<<15) | 21260, 0x7f2f, 0x7f2f, 
  (1U<<15) | 1022, 0x7f1f, (1U<<15) | 1022, (1U<<15) | 20490, (1U<<15) | 20485, (1U<<15) | 20217, (1U<<15) | 20485, 0x0, 
  0x0, 0x4e, (1U<<15) | 20202, (1U<<15) | 20201, 0xeee, 0xe40, 0xe50, 0x40, 
  0xe0, 0xe0, 0xe, 0xe4, 0x0, 0x4f4, 0x0, 0x7f2f, 
  0x7f2f, 0x7f2f, (1U<<15) | 17279, (1U<<15) | 18201, (1U<<15) | 20428, (1U<<15) | 20428, (1U<<15) | 20428, (1U<<15) | 20435, 
  (1U<<15) | 20275, (1U<<15) | 20428, (1U<<15) | 20428, (1U<<15) | 20428, (1U<<15) | 20428, (1U<<15) | 20435, (1U<<15) | 20425, (1U<<15) | 20425, 
  (1U<<15) | 20435, (1U<<15) | 20275, (1U<<15) | 20444, (1U<<15) | 20435, (1U<<15) | 20444, (1U<<15) | 20301, (1U<<15) | 20295, (1U<<15) | 20295, 
  (1U<<15) | 20478, (1U<<15) | 20435, (1U<<15) | 20435, (1U<<15) | 20462, (1U<<15) | 20471, (1U<<15) | 20295, (1U<<15) | 20428, (1U<<15) | 20428, 
  (1U<<15) | 20428, (1U<<15) | 20471, (1U<<15) | 20295, (1U<<15) | 20281, (1U<<15) | 20281, (1U<<15) | 20281, (1U<<15) | 20281, (1U<<15) | 20428, 
  (1U<<15) | 20435, (1U<<15) | 20417, (1U<<15) | 20428, (1U<<15) | 20275, (1U<<15) | 20275, (1U<<15) | 20428, (1U<<15) | 20428, (1U<<15) | 20455, 
  (1U<<15) | 20428, (1U<<15) | 20428, (1U<<15) | 20428, (1U<<15) | 20275, (1U<<15) | 20455, (1U<<15) | 20194, (1U<<15) | 20194, (1U<<15) | 20194, 
  (1U<<15) | 1198, (1U<<15) | 21260, (1U<<15) | 18980, 0x4f5, (1U<<15) | 11731, (1U<<15) | 20192, (1U<<15) | 21295, (1U<<15) | 359, 
  (1U<<15) | 21270, (1U<<15) | 1313, (1U<<15) | 20217, (1U<<15) | 21309, (1U<<15) | 21310, (1U<<15) | 21304, (1U<<15) | 17003, (1U<<15) | 17918, 
  (1U<<15) | 322, (1U<<15) | 322, (1U<<15) | 322, (1U<<15) | 322, (1U<<15) | 331, (1U<<15) | 3836, (1U<<15) | 5612, (1U<<15) | 3901, 
  (1U<<15) | 3900, 0x1, 0x7f2f, (1U<<15) | 21261, 0x7f2f, (1U<<15) | 17344, (1U<<15) | 17344, (1U<<15) | 18648, 
  (1U<<15) | 18648, (1U<<15) | 20301, 0x44f, (1U<<15) | 17713, (1U<<15) | 17333, (1U<<15) | 17333, 0xeee, 0xee0, 
  (1U<<15) | 19374, (1U<<15) | 19559, 0x1f, 0x1f, 0x1f, 0x4, 0x4ee0, 0x54e0, 
  0x4ee0, 0x54e0, (1U<<15) | 21261, (1U<<15) | 19374, (1U<<15) | 19267, (1U<<15) | 10354, (1U<<15) | 10354, (1U<<15) | 14603, 
  0x45e0, (1U<<15) | 19292, (1U<<15) | 10354, (1U<<15) | 10330, (1U<<15) | 14117, 0x4f5e, 0xf1, (1U<<15) | 5309, 
  0x7f4f, (1U<<15) | 18583, 0x4f0, 0x4f0, (1U<<15) | 18648, (1U<<15) | 18648, 0x1fee, 0xe, 
  (1U<<15) | 21261, 0x4eee, 0x7f2f, 0x7f2f, 0x7f2f, 0x1f1, (1U<<15) | 17279, (1U<<15) | 15300, 
  (1U<<15) | 15300, (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 351, (1U<<15) | 17023, (1U<<15) | 16984, (1U<<15) | 17013, (1U<<15) | 288, 
  (1U<<15) | 342, (1U<<15) | 5643, (1U<<15) | 5642, (1U<<15) | 10217, (1U<<15) | 10675, (1U<<15) | 17288, (1U<<15) | 17288, (1U<<15) | 17288, 
  (1U<<15) | 1322, (1U<<15) | 14293, (1U<<15) | 1322, (1U<<15) | 1322, (1U<<15) | 14293, (1U<<15) | 1108, (1U<<15) | 13249, (1U<<15) | 1108, 
  (1U<<15) | 17288, (1U<<15) | 17288, (1U<<15) | 17288, (1U<<15) | 2813, 0x7f2f, 0xee0, 0xee0, 0xee0, 
  0xee0, 0xee, 0xe0, 0xe, 0xee, 0xee, (1U<<15) | 21261, (1U<<15) | 21261, 
  0xee0, 0xe0, 0xeee, 0xee, 0xee, 0xee0, 0xe0, 0xee, 
  0xee, 0xee, 0xee, 0xee, 0xee, 0xee, 0xee0, 0xeee, 
  0xe4, 0xe4, 0xee, 0xee, 0xee, (1U<<15) | 35, 0x40, (1U<<15) | 17288, 
  (1U<<15) | 18583, (1U<<15) | 9220, (1U<<15) | 11080, 0xee, (1U<<15) | 11080, (1U<<15) | 13804, (1U<<15) | 917, (1U<<15) | 14629, 
  (1U<<15) | 19452, 0x5455, 0x555, (1U<<15) | 14622, 0x5455, 0x555, 0x455, (1U<<15) | 18576, 
  (1U<<15) | 20307, (1U<<15) | 20220, (1U<<15) | 20220, 0x5, 0x5, (1U<<15) | 20217, 0x0, 0x0, 
  0x4e, 0x7f2f, 0x7f2f, 0x7f2f, (1U<<15) | 17279, (1U<<15) | 17257, (1U<<15) | 19552, (1U<<15) | 12503, 
  (1U<<15) | 12503, 0x0, 0x0, 0x0, 0x0, 0x1f0, 0x1f0, 0x1f0, 
  0x40, 0x0, 0x7f2f, (1U<<15) | 2813, (1U<<15) | 2813, 0x7f2f, (1U<<15) | 17279, (1U<<15) | 17279, 
  (1U<<15) | 12503, (1U<<15) | 12503, (1U<<15) | 17257, 0x4f, 0x7f2f, 0x7f0f, (1U<<15) | 17279, (1U<<15) | 17279, 
  (1U<<15) | 17257, 0x4f, 0xe, 0xee0, 0x4f0, 0x4f, 0x7f1f, 0x3f, 
  0x7f4f, 0xe, 0x7f2f, 0x7f2f, 0x1f1, (1U<<15) | 16976, 0x4f, 0x7f4f, 
  0x0, 0x7f2f, (1U<<15) | 20267, (1U<<15) | 20267, (1U<<15) | 20307, (1U<<15) | 17279, (1U<<15) | 17257, 0x20, 
  (1U<<15) | 19552, (1U<<15) | 12503, (1U<<15) | 12503, (1U<<15) | 17279, (1U<<15) | 17279, (1U<<15) | 12503, (1U<<15) | 12503, (1U<<15) | 17257, 
  (1U<<15) | 17279, (1U<<15) | 17279, (1U<<15) | 17257, (1U<<15) | 17244, 0x4f0, 0x4f0, (1U<<15) | 19441, (1U<<15) | 3383, 
  (1U<<15) | 3394, (1U<<15) | 3408, (1U<<15) | 3425, (1U<<15) | 3445, (1U<<15) | 3468, (1U<<15) | 3531, (1U<<15) | 15249, (1U<<15) | 15255, 
  (1U<<15) | 1453, (1U<<15) | 2890, (1U<<15) | 3613, (1U<<15) | 14524, (1U<<15) | 15318, (1U<<15) | 15714, (1U<<15) | 18164, (1U<<15) | 18714, 
  (1U<<15) | 18653, (1U<<15) | 3512, (1U<<15) | 3512, (1U<<15) | 3517, (1U<<15) | 3512, (1U<<15) | 3512, (1U<<15) | 3512, (1U<<15) | 3512, 
  (1U<<15) | 3517, (1U<<15) | 3512, (1U<<15) | 3512, (1U<<15) | 3512, (1U<<15) | 3512, (1U<<15) | 3512, (1U<<15) | 3512, (1U<<15) | 3512, 
  0x7f3f, (1U<<15) | 12513, (1U<<15) | 12513, (1U<<15) | 3804, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3836, 
  (1U<<15) | 3836, (1U<<15) | 3836, (1U<<15) | 3845, (1U<<15) | 3804, (1U<<15) | 3836, (1U<<15) | 3804, (1U<<15) | 4212, (1U<<15) | 3836, 
  (1U<<15) | 3845, (1U<<15) | 3912, (1U<<15) | 3845, (1U<<15) | 3836, (1U<<15) | 3856, (1U<<15) | 3845, (1U<<15) | 3856, (1U<<15) | 3836, 
  (1U<<15) | 3869, (1U<<15) | 3869, (1U<<15) | 3869, (1U<<15) | 3869, (1U<<15) | 3845, (1U<<15) | 3856, (1U<<15) | 3856, (1U<<15) | 3845, 
  (1U<<15) | 3794, (1U<<15) | 3912, (1U<<15) | 3869, (1U<<15) | 3825, (1U<<15) | 3869, (1U<<15) | 3879, (1U<<15) | 3888, (1U<<15) | 3869, 
  (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 12492, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3836, 
  (1U<<15) | 3845, (1U<<15) | 3869, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, 
  (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, (1U<<15) | 3814, 
  (1U<<15) | 3814, (1U<<15) | 3836, (1U<<15) | 3836, (1U<<15) | 3836, (1U<<15) | 3836, (1U<<15) | 3845, (1U<<15) | 3793, (1U<<15) | 3845, 
  (1U<<15) | 12492, (1U<<15) | 3869, (1U<<15) | 3845, (1U<<15) | 3869, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3836, (1U<<15) | 3845, 
  (1U<<15) | 3845, (1U<<15) | 3878, (1U<<15) | 3845, (1U<<15) | 3869, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3869, (1U<<15) | 3845, 
  (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3845, (1U<<15) | 3869, 0x1f, (1U<<15) | 1273, 0x5e0, 
  0x5e50, 0x5ee, 0x40, 0x55, 0x0, 0x44, 0x54, 0x444, 
  0x444, 0x444, 0x444, 0x544, 0x444, 0x444, 0x544, (1U<<15) | 2476, 
  (1U<<15) | 2476, 0x2c2c, 0x2c2c, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 14964, 
  (1U<<15) | 13311, 0x44, (1U<<15) | 13311, (1U<<15) | 13311, (1U<<15) | 13403, (1U<<15) | 13405, (1U<<15) | 13403, (1U<<15) | 13403, 
  (1U<<15) | 13405, (1U<<15) | 13403, (1U<<15) | 14962, (1U<<15) | 14962, (1U<<15) | 14964, (1U<<15) | 14962, (1U<<15) | 13403, (1U<<15) | 13403, 
  (1U<<15) | 13403, (1U<<15) | 14655, (1U<<15) | 14655, (1U<<15) | 14655, (1U<<15) | 14655, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 14809, 
  0x40, 0x40, 0x84, 0x7f2f, 0x7f2f, 0x7f2f, 0x7f2f, 0x55, 
  0xee, 0x5, 0x5, 0x5e5, 0x40, 0x40, 0x5ee, 0x5e, 
  0x40, (1U<<15) | 19348, (1U<<15) | 19337, 0x4f5, 0xeee, (1U<<15) | 19337, 0x4f5, 0x52ee, 
  0x7f0f, (1U<<15) | 17896, (1U<<15) | 17297, (1U<<15) | 19601, (1U<<15) | 21788, (1U<<15) | 21788, (1U<<15) | 21788, 0x7f3f, 
  (1U<<15) | 17297, (1U<<15) | 19577, (1U<<15) | 19577, (1U<<15) | 17297, (1U<<15) | 18702, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 18648, 
  (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 18709, 
  (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 17288, (1U<<15) | 17288, (1U<<15) | 17297, (1U<<15) | 18702, (1U<<15) | 17297, (1U<<15) | 18702, 
  (1U<<15) | 17288, (1U<<15) | 17288, (1U<<15) | 17297, (1U<<15) | 18702, (1U<<15) | 17297, (1U<<15) | 18702, (1U<<15) | 19601, (1U<<15) | 19601, 
  (1U<<15) | 19601, (1U<<15) | 19601, (1U<<15) | 19601, (1U<<15) | 17288, (1U<<15) | 18709, (1U<<15) | 18709, (1U<<15) | 19584, (1U<<15) | 19601, 
  (1U<<15) | 19601, (1U<<15) | 5089, (1U<<15) | 19601, (1U<<15) | 5089, (1U<<15) | 2556, (1U<<15) | 5040, (1U<<15) | 2556, (1U<<15) | 5040, 
  (1U<<15) | 2556, (1U<<15) | 5040, (1U<<15) | 2556, (1U<<15) | 5040, (1U<<15) | 2556, (1U<<15) | 5040, (1U<<15) | 2556, (1U<<15) | 5040, 
  (1U<<15) | 18054, 0x7f2f, (1U<<15) | 17288, 0x7f2f, 0x7f2f, 0x7f2f, 0x7f2f, 0x7f2f, 
  0x7f2f, (1U<<15) | 17288, (1U<<15) | 18946, (1U<<15) | 18955, (1U<<15) | 18966, (1U<<15) | 18946, (1U<<15) | 18985, (1U<<15) | 18946, 
  (1U<<15) | 18955, (1U<<15) | 18999, (1U<<15) | 18955, (1U<<15) | 18966, (1U<<15) | 19017, (1U<<15) | 18966, (1U<<15) | 17297, (1U<<15) | 17908, 
  0x552c, (1U<<15) | 17896, (1U<<15) | 12613, (1U<<15) | 17896, (1U<<15) | 17297, (1U<<15) | 18709, (1U<<15) | 18697, (1U<<15) | 18697, 
  (1U<<15) | 18485, (1U<<15) | 18485, (1U<<15) | 18485, (1U<<15) | 19601, (1U<<15) | 17297, (1U<<15) | 17903, (1U<<15) | 17297, (1U<<15) | 17297, 
  (1U<<15) | 17297, (1U<<15) | 18697, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 18697, (1U<<15) | 19601, (1U<<15) | 17908, 0x7f1f, 
  (1U<<15) | 17279, (1U<<15) | 17279, (1U<<15) | 13263, (1U<<15) | 13263, (1U<<15) | 17908, 0x445, 0x7f1f, (1U<<15) | 17333, 
  (1U<<15) | 17333, (1U<<15) | 17279, (1U<<15) | 13263, (1U<<15) | 13263, (1U<<15) | 17279, (1U<<15) | 12613, (1U<<15) | 12613, (1U<<15) | 17279, 
  (1U<<15) | 17279, (1U<<15) | 12613, (1U<<15) | 12613, (1U<<15) | 17279, (1U<<15) | 17886, (1U<<15) | 17886, (1U<<15) | 17297, (1U<<15) | 17279, 
  (1U<<15) | 17279, (1U<<15) | 12619, (1U<<15) | 19073, (1U<<15) | 19081, (1U<<15) | 19102, (1U<<15) | 19073, (1U<<15) | 19039, (1U<<15) | 19081, 
  (1U<<15) | 19048, (1U<<15) | 19102, (1U<<15) | 19059, (1U<<15) | 17896, (1U<<15) | 17279, (1U<<15) | 17112, (1U<<15) | 17092, (1U<<15) | 17068, 
  (1U<<15) | 17040, (1U<<15) | 17119, (1U<<15) | 17101, (1U<<15) | 17079, (1U<<15) | 17053, (1U<<15) | 17297, (1U<<15) | 18709, (1U<<15) | 18697, 
  (1U<<15) | 18697, (1U<<15) | 19601, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 18697, (1U<<15) | 17297, 
  (1U<<15) | 17297, (1U<<15) | 18697, (1U<<15) | 19601, (1U<<15) | 17908, (1U<<15) | 17279, (1U<<15) | 17279, (1U<<15) | 12613, (1U<<15) | 17279, 
  (1U<<15) | 12613, (1U<<15) | 17279, (1U<<15) | 17886, 0x7f3f, (1U<<15) | 17297, (1U<<15) | 17279, 0x7f3f, (1U<<15) | 19601, 
  (1U<<15) | 17279, (1U<<15) | 12619, (1U<<15) | 19601, (1U<<15) | 17279, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17355, (1U<<15) | 17355, 
  (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 14836, (1U<<15) | 13474, (1U<<15) | 13474, 0x7a3a, (1U<<15) | 13257, (1U<<15) | 13257, 
  0x3a7a, (1U<<15) | 4982, (1U<<15) | 5081, (1U<<15) | 4964, (1U<<15) | 4972, (1U<<15) | 5063, (1U<<15) | 5071, (1U<<15) | 12513, 
  (1U<<15) | 12513, (1U<<15) | 9213, (1U<<15) | 14610, (1U<<15) | 556, (1U<<15) | 556, (1U<<15) | 17279, 0x50, 0x50, 
  0x50, 0x5e0, 0x5e0, (1U<<15) | 17288, 0x87, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, 
  (1U<<15) | 17514, (1U<<15) | 17164, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17364, (1U<<15) | 3338, 
  (1U<<15) | 3338, (1U<<15) | 3494, (1U<<15) | 3494, 0x5, (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 17461, (1U<<15) | 17577, 
  (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17164, 
  (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17295, (1U<<15) | 17433, 
  (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, 
  (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 5168, (1U<<15) | 5149, (1U<<15) | 5168, (1U<<15) | 5149, (1U<<15) | 2685, (1U<<15) | 2640, 
  (1U<<15) | 2685, (1U<<15) | 2640, (1U<<15) | 2700, (1U<<15) | 2667, (1U<<15) | 2700, (1U<<15) | 2667, (1U<<15) | 5181, (1U<<15) | 5168, 
  (1U<<15) | 5149, (1U<<15) | 2685, (1U<<15) | 2640, (1U<<15) | 2700, (1U<<15) | 2667, (1U<<15) | 2712, (1U<<15) | 5181, (1U<<15) | 5168, 
  (1U<<15) | 5149, (1U<<15) | 2685, (1U<<15) | 2640, (1U<<15) | 2700, (1U<<15) | 2667, (1U<<15) | 2712, (1U<<15) | 17164, (1U<<15) | 17295, 
  (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 2625, (1U<<15) | 2625, 
  (1U<<15) | 5168, (1U<<15) | 5168, (1U<<15) | 5168, (1U<<15) | 18070, (1U<<15) | 18083, (1U<<15) | 18104, (1U<<15) | 18119, (1U<<15) | 5248, 
  (1U<<15) | 5248, (1U<<15) | 12511, 0x5, 0x1, (1U<<15) | 11155, (1U<<15) | 11155, (1U<<15) | 11131, (1U<<15) | 11131, 
  (1U<<15) | 11147, (1U<<15) | 11147, (1U<<15) | 11163, (1U<<15) | 11163, (1U<<15) | 11139, (1U<<15) | 11139, 0x4e40, 0xe40, 
  (1U<<15) | 5203, (1U<<15) | 5211, (1U<<15) | 5223, (1U<<15) | 5203, (1U<<15) | 5211, (1U<<15) | 5223, (1U<<15) | 2721, (1U<<15) | 17164, 
  (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, 
  (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 3494, 
  (1U<<15) | 3494, (1U<<15) | 3494, (1U<<15) | 3494, (1U<<15) | 3494, (1U<<15) | 3494, (1U<<15) | 10673, (1U<<15) | 10692, (1U<<15) | 5625, 
  (1U<<15) | 10673, (1U<<15) | 10692, (1U<<15) | 5625, (1U<<15) | 12461, (1U<<15) | 12564, (1U<<15) | 5625, (1U<<15) | 5625, 0x443f, 
  (1U<<15) | 10673, (1U<<15) | 10692, 0x443f, 0x443f, 0x443f, (1U<<15) | 10673, (1U<<15) | 10692, (1U<<15) | 12461, 
  (1U<<15) | 12564, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17295, 
  (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 17364, (1U<<15) | 17514, 0x50, (1U<<15) | 12469, (1U<<15) | 12511, 
  (1U<<15) | 12599, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, 
  (1U<<15) | 12599, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, 
  (1U<<15) | 12599, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, 
  (1U<<15) | 12599, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, 
  (1U<<15) | 12599, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, 
  (1U<<15) | 12599, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 17164, (1U<<15) | 17295, 
  (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, 
  (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 3494, (1U<<15) | 3494, 
  (1U<<15) | 3494, (1U<<15) | 3494, (1U<<15) | 11155, (1U<<15) | 11155, (1U<<15) | 11131, (1U<<15) | 11131, (1U<<15) | 11147, (1U<<15) | 11147, 
  (1U<<15) | 11163, (1U<<15) | 11163, (1U<<15) | 11139, (1U<<15) | 11139, (1U<<15) | 5248, 0x4e40, 0xe40, (1U<<15) | 17295, 
  (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 17164, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17364, (1U<<15) | 17164, 
  (1U<<15) | 17364, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, 
  (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, 
  (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 3494, (1U<<15) | 3494, (1U<<15) | 5248, (1U<<15) | 12599, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 12599, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17295, 
  (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, (1U<<15) | 12511, (1U<<15) | 12599, 
  (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, 
  (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, 
  (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 3494, (1U<<15) | 3494, (1U<<15) | 3494, 
  (1U<<15) | 3494, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 12469, 
  (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17433, (1U<<15) | 17364, (1U<<15) | 17514, (1U<<15) | 17164, 
  (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, 
  (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 17164, (1U<<15) | 17295, (1U<<15) | 17295, (1U<<15) | 17364, (1U<<15) | 3494, 
  (1U<<15) | 3494, (1U<<15) | 5248, (1U<<15) | 12599, (1U<<15) | 5248, (1U<<15) | 12511, (1U<<15) | 12599, (1U<<15) | 12599, (1U<<15) | 17171, 
  (1U<<15) | 17375, (1U<<15) | 3349, (1U<<15) | 5537, (1U<<15) | 17171, (1U<<15) | 17375, (1U<<15) | 3349, (1U<<15) | 17164, (1U<<15) | 17364, 
  0x3f40, (1U<<15) | 3349, (1U<<15) | 3349, 0x0, 0x0, 0x40, 0x40, 0x40, 
  0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x545, 
  (1U<<15) | 14761, (1U<<15) | 14772, (1U<<15) | 14772, 0xee0, 0x55e0, (1U<<15) | 19343, 0x4f54, (1U<<15) | 19343, 
  0x4f54, 0xee5, (1U<<15) | 17003, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, 
  (1U<<15) | 17269, (1U<<15) | 17201, (1U<<15) | 17199, (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, 
  (1U<<15) | 17297, (1U<<15) | 2657, (1U<<15) | 5099, (1U<<15) | 5118, (1U<<15) | 5099, (1U<<15) | 5118, (1U<<15) | 2657, (1U<<15) | 5099, 
  (1U<<15) | 5118, (1U<<15) | 5099, (1U<<15) | 5118, (1U<<15) | 2633, (1U<<15) | 2633, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, 
  (1U<<15) | 18689, (1U<<15) | 3365, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 15120, (1U<<15) | 12446, (1U<<15) | 17355, (1U<<15) | 17297, 
  (1U<<15) | 17297, (1U<<15) | 17461, (1U<<15) | 17554, (1U<<15) | 16889, (1U<<15) | 16889, (1U<<15) | 21769, (1U<<15) | 14365, (1U<<15) | 21769, 
  (1U<<15) | 14365, (1U<<15) | 21769, (1U<<15) | 14365, (1U<<15) | 21769, (1U<<15) | 14365, (1U<<15) | 21769, (1U<<15) | 14365, (1U<<15) | 21769, 
  (1U<<15) | 17297, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17003, (1U<<15) | 16995, (1U<<15) | 17003, (1U<<15) | 16995, 
  (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 12513, (1U<<15) | 12654, 
  (1U<<15) | 10762, (1U<<15) | 17269, (1U<<15) | 3507, (1U<<15) | 17269, (1U<<15) | 3507, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 10694, 
  (1U<<15) | 12566, (1U<<15) | 17142, (1U<<15) | 15072, (1U<<15) | 17142, (1U<<15) | 15072, (1U<<15) | 17142, (1U<<15) | 15072, (1U<<15) | 17142, 
  (1U<<15) | 15072, (1U<<15) | 17142, (1U<<15) | 15072, (1U<<15) | 15072, (1U<<15) | 15072, (1U<<15) | 15072, (1U<<15) | 15072, (1U<<15) | 17142, 
  (1U<<15) | 15072, (1U<<15) | 17003, (1U<<15) | 3373, 0x45, 0x45, 0x45, (1U<<15) | 17193, (1U<<15) | 14489, 
  (1U<<15) | 14489, (1U<<15) | 14489, (1U<<15) | 14489, 0x45, (1U<<15) | 16995, (1U<<15) | 1227, (1U<<15) | 0, (1U<<15) | 17928, 
  (1U<<15) | 12463, (1U<<15) | 17956, (1U<<15) | 14830, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17355, (1U<<15) | 17355, 
  (1U<<15) | 18689, (1U<<15) | 17355, (1U<<15) | 3365, (1U<<15) | 16995, (1U<<15) | 12513, (1U<<15) | 12513, (1U<<15) | 17269, (1U<<15) | 17269, 
  (1U<<15) | 17003, (1U<<15) | 17142, (1U<<15) | 17142, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 3507, (1U<<15) | 17269, (1U<<15) | 18689, 
  (1U<<15) | 3365, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 12492, (1U<<15) | 17355, (1U<<15) | 17461, 
  (1U<<15) | 17554, (1U<<15) | 12541, (1U<<15) | 10694, (1U<<15) | 17142, (1U<<15) | 17142, (1U<<15) | 17142, (1U<<15) | 17142, (1U<<15) | 17142, 
  (1U<<15) | 18669, (1U<<15) | 16860, (1U<<15) | 16847, (1U<<15) | 18388, (1U<<15) | 15656, (1U<<15) | 18401, (1U<<15) | 15630, (1U<<15) | 16834, 
  (1U<<15) | 17990, (1U<<15) | 17201, (1U<<15) | 17990, (1U<<15) | 15656, (1U<<15) | 16834, (1U<<15) | 17201, (1U<<15) | 16860, (1U<<15) | 16847, 
  (1U<<15) | 18401, (1U<<15) | 16860, (1U<<15) | 16847, (1U<<15) | 18401, (1U<<15) | 18401, (1U<<15) | 18401, (1U<<15) | 18401, (1U<<15) | 18679, 
  (1U<<15) | 15643, (1U<<15) | 18375, (1U<<15) | 15617, (1U<<15) | 16821, (1U<<15) | 19599, (1U<<15) | 19702, (1U<<15) | 18679, (1U<<15) | 15643, 
  (1U<<15) | 18375, (1U<<15) | 15617, (1U<<15) | 16821, (1U<<15) | 19599, (1U<<15) | 19702, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, 
  (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 18039, (1U<<15) | 17193, (1U<<15) | 3373, (1U<<15) | 17321, (1U<<15) | 17269, (1U<<15) | 17420, 
  (1U<<15) | 17533, (1U<<15) | 17269, (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17269, 
  (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 3365, (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 3365, 
  (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17269, (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 17269, (1U<<15) | 17420, 
  (1U<<15) | 17533, (1U<<15) | 17269, (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 3365, (1U<<15) | 17269, 
  (1U<<15) | 18689, (1U<<15) | 3365, (1U<<15) | 17321, (1U<<15) | 12566, (1U<<15) | 17321, (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18006, 
  (1U<<15) | 12644, (1U<<15) | 17321, (1U<<15) | 12566, (1U<<15) | 17321, (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 12644, 
  (1U<<15) | 19601, (1U<<15) | 17321, (1U<<15) | 17269, (1U<<15) | 12513, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17269, (1U<<15) | 17461, 
  (1U<<15) | 17577, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17003, (1U<<15) | 17321, (1U<<15) | 17321, (1U<<15) | 17321, (1U<<15) | 17321, 
  (1U<<15) | 17321, (1U<<15) | 17321, (1U<<15) | 17156, (1U<<15) | 16873, (1U<<15) | 2750, (1U<<15) | 2756, (1U<<15) | 2750, (1U<<15) | 2756, 
  (1U<<15) | 2756, (1U<<15) | 2756, (1U<<15) | 2750, (1U<<15) | 2750, (1U<<15) | 17156, (1U<<15) | 16873, (1U<<15) | 17156, (1U<<15) | 17153, 
  (1U<<15) | 2739, (1U<<15) | 5191, (1U<<15) | 2739, (1U<<15) | 5191, (1U<<15) | 2739, (1U<<15) | 5191, (1U<<15) | 2739, (1U<<15) | 5191, 
  (1U<<15) | 2739, (1U<<15) | 5191, (1U<<15) | 2739, (1U<<15) | 5191, (1U<<15) | 2739, (1U<<15) | 5191, (1U<<15) | 2739, 0x7f3f, 
  (1U<<15) | 17297, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17353, 
  (1U<<15) | 17495, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17353, (1U<<15) | 17495, (1U<<15) | 17003, (1U<<15) | 17353, (1U<<15) | 17495, 
  (1U<<15) | 17003, (1U<<15) | 17353, (1U<<15) | 17495, (1U<<15) | 17003, (1U<<15) | 17003, 0x7f3f, (1U<<15) | 17297, (1U<<15) | 18044, 
  (1U<<15) | 18070, (1U<<15) | 18083, (1U<<15) | 18104, (1U<<15) | 18119, (1U<<15) | 17003, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, 
  (1U<<15) | 12513, (1U<<15) | 18054, (1U<<15) | 18054, (1U<<15) | 17269, (1U<<15) | 17297, (1U<<15) | 17968, (1U<<15) | 17961, (1U<<15) | 3365, 
  (1U<<15) | 3365, (1U<<15) | 17193, (1U<<15) | 19128, (1U<<15) | 14573, (1U<<15) | 14573, (1U<<15) | 15230, (1U<<15) | 5597, (1U<<15) | 5597, 
  (1U<<15) | 5597, (1U<<15) | 5597, (1U<<15) | 19407, (1U<<15) | 19428, (1U<<15) | 15132, (1U<<15) | 15240, (1U<<15) | 15132, (1U<<15) | 19128, 
  (1U<<15) | 19128, (1U<<15) | 19201, (1U<<15) | 19201, (1U<<15) | 19135, (1U<<15) | 19135, (1U<<15) | 19158, (1U<<15) | 19158, (1U<<15) | 19185, 
  (1U<<15) | 19185, (1U<<15) | 19128, (1U<<15) | 14573, (1U<<15) | 14573, (1U<<15) | 15230, (1U<<15) | 5597, (1U<<15) | 5597, (1U<<15) | 5597, 
  (1U<<15) | 5597, (1U<<15) | 19128, (1U<<15) | 19128, (1U<<15) | 14573, (1U<<15) | 14573, (1U<<15) | 15230, (1U<<15) | 5597, (1U<<15) | 19407, 
  (1U<<15) | 19428, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 15120, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 15120, (1U<<15) | 5239, 
  (1U<<15) | 5239, (1U<<15) | 5250, (1U<<15) | 17321, (1U<<15) | 17142, (1U<<15) | 17321, (1U<<15) | 12566, (1U<<15) | 17321, (1U<<15) | 17321, 
  (1U<<15) | 12566, (1U<<15) | 17321, (1U<<15) | 17321, (1U<<15) | 17269, (1U<<15) | 12513, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17355, 
  (1U<<15) | 17003, (1U<<15) | 17142, (1U<<15) | 17269, (1U<<15) | 17003, (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 17269, (1U<<15) | 17269, 
  (1U<<15) | 17269, (1U<<15) | 3365, (1U<<15) | 14484, (1U<<15) | 14493, (1U<<15) | 16995, (1U<<15) | 15099, (1U<<15) | 5520, (1U<<15) | 3343, 
  (1U<<15) | 3836, (1U<<15) | 298, (1U<<15) | 17297, (1U<<15) | 15084, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 16995, (1U<<15) | 14044, 
  (1U<<15) | 5557, (1U<<15) | 11776, (1U<<15) | 5557, (1U<<15) | 5557, (1U<<15) | 5557, (1U<<15) | 11776, (1U<<15) | 5557, (1U<<15) | 5557, 
  (1U<<15) | 5557, (1U<<15) | 11776, (1U<<15) | 5557, (1U<<15) | 5557, (1U<<15) | 5557, (1U<<15) | 11776, (1U<<15) | 5557, (1U<<15) | 5557, 
  (1U<<15) | 5527, (1U<<15) | 17147, (1U<<15) | 17147, (1U<<15) | 17147, 0x43f, (1U<<15) | 21911, (1U<<15) | 21911, (1U<<15) | 21911, 
  (1U<<15) | 21911, (1U<<15) | 3388, (1U<<15) | 3388, (1U<<15) | 17201, (1U<<15) | 17199, (1U<<15) | 15089, (1U<<15) | 17003, (1U<<15) | 1229, 
  (1U<<15) | 1236, (1U<<15) | 1236, (1U<<15) | 1236, (1U<<15) | 1236, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17003, 
  (1U<<15) | 5551, (1U<<15) | 5549, (1U<<15) | 17201, (1U<<15) | 17199, (1U<<15) | 17355, (1U<<15) | 18006, (1U<<15) | 18006, (1U<<15) | 17269, 
  (1U<<15) | 17269, (1U<<15) | 17999, (1U<<15) | 17999, (1U<<15) | 17980, (1U<<15) | 17999, (1U<<15) | 17999, (1U<<15) | 17999, (1U<<15) | 3358, 
  (1U<<15) | 17992, (1U<<15) | 17992, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 17461, (1U<<15) | 17554, (1U<<15) | 18669, 
  (1U<<15) | 13507, (1U<<15) | 15046, (1U<<15) | 15059, (1U<<15) | 13494, (1U<<15) | 19599, (1U<<15) | 19702, (1U<<15) | 17269, (1U<<15) | 17269, 
  (1U<<15) | 17269, (1U<<15) | 18030, (1U<<15) | 12654, (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 17269, (1U<<15) | 17677, (1U<<15) | 17604, 
  0x0, (1U<<15) | 17269, (1U<<15) | 5551, (1U<<15) | 5549, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 12513, 
  (1U<<15) | 13520, (1U<<15) | 13520, (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17269, (1U<<15) | 17461, (1U<<15) | 17577, 
  (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 3365, (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17269, (1U<<15) | 17461, 
  (1U<<15) | 17577, (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 3365, (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 12644, 
  (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18030, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17999, 
  (1U<<15) | 12636, (1U<<15) | 17999, (1U<<15) | 12636, (1U<<15) | 17269, (1U<<15) | 17003, (1U<<15) | 17269, (1U<<15) | 17297, (1U<<15) | 12513, 
  (1U<<15) | 17201, (1U<<15) | 17387, (1U<<15) | 17201, (1U<<15) | 17387, (1U<<15) | 17201, (1U<<15) | 17387, (1U<<15) | 17201, (1U<<15) | 17387, 
  0x4444, 0x4455, (1U<<15) | 10675, 0x4444, 0x4455, (1U<<15) | 10675, 0x4444, 0x4455, 
  (1U<<15) | 304, 0x3f44, 0x3f55, (1U<<15) | 10675, 0x4444, 0x4455, (1U<<15) | 18006, (1U<<15) | 12644, 
  (1U<<15) | 18006, (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 18006, (1U<<15) | 12644, 
  (1U<<15) | 17297, (1U<<15) | 12513, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 17999, (1U<<15) | 12636, 
  (1U<<15) | 17999, (1U<<15) | 12636, 0x4444, 0x4455, (1U<<15) | 10675, 0x4444, 0x4455, (1U<<15) | 10675, 
  0x4444, 0x4455, (1U<<15) | 304, 0x3f44, 0x3f55, (1U<<15) | 10675, 0x4444, 0x4455, 
  (1U<<15) | 17003, (1U<<15) | 10694, (1U<<15) | 12566, (1U<<15) | 17355, (1U<<15) | 12566, (1U<<15) | 17355, (1U<<15) | 12566, (1U<<15) | 17297, 
  (1U<<15) | 12513, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 12477, (1U<<15) | 12576, (1U<<15) | 12477, (1U<<15) | 12576, (1U<<15) | 5551, 
  (1U<<15) | 5549, (1U<<15) | 12477, (1U<<15) | 12576, (1U<<15) | 12477, (1U<<15) | 12576, (1U<<15) | 5551, (1U<<15) | 5549, (1U<<15) | 17269, 
  (1U<<15) | 17269, (1U<<15) | 12446, (1U<<15) | 5551, (1U<<15) | 5549, (1U<<15) | 5551, (1U<<15) | 5549, (1U<<15) | 17269, (1U<<15) | 17269, 
  (1U<<15) | 17297, (1U<<15) | 17269, (1U<<15) | 3526, (1U<<15) | 3524, (1U<<15) | 3526, (1U<<15) | 3524, (1U<<15) | 17269, (1U<<15) | 12513, 
  (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17269, (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 12446, (1U<<15) | 12513, 
  (1U<<15) | 12630, (1U<<15) | 12630, (1U<<15) | 12513, (1U<<15) | 17999, (1U<<15) | 17999, (1U<<15) | 17999, (1U<<15) | 17999, (1U<<15) | 17992, 
  (1U<<15) | 17992, (1U<<15) | 19127, (1U<<15) | 19408, (1U<<15) | 19416, (1U<<15) | 14572, (1U<<15) | 14572, (1U<<15) | 15229, (1U<<15) | 5596, 
  (1U<<15) | 5596, (1U<<15) | 5596, (1U<<15) | 5596, (1U<<15) | 19200, (1U<<15) | 15131, (1U<<15) | 15239, (1U<<15) | 15131, (1U<<15) | 19200, 
  (1U<<15) | 19136, (1U<<15) | 19136, (1U<<15) | 19146, (1U<<15) | 19146, (1U<<15) | 19171, (1U<<15) | 19171, (1U<<15) | 19127, (1U<<15) | 19408, 
  (1U<<15) | 19416, (1U<<15) | 14572, (1U<<15) | 14572, (1U<<15) | 15229, (1U<<15) | 5596, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17201, 
  (1U<<15) | 17199, (1U<<15) | 17269, (1U<<15) | 12654, (1U<<15) | 17990, (1U<<15) | 18015, (1U<<15) | 17975, (1U<<15) | 17975, (1U<<15) | 17269, 
  (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 18054, (1U<<15) | 18061, (1U<<15) | 18054, (1U<<15) | 18061, (1U<<15) | 18061, 
  (1U<<15) | 17297, (1U<<15) | 1233, (1U<<15) | 1233, (1U<<15) | 1233, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 1233, (1U<<15) | 1233, 
  (1U<<15) | 1233, (1U<<15) | 17297, (1U<<15) | 17355, (1U<<15) | 18006, (1U<<15) | 18006, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17999, 
  (1U<<15) | 17999, (1U<<15) | 17980, (1U<<15) | 17999, (1U<<15) | 17999, (1U<<15) | 3358, (1U<<15) | 17992, (1U<<15) | 17992, (1U<<15) | 17355, 
  (1U<<15) | 17461, (1U<<15) | 17554, (1U<<15) | 18669, (1U<<15) | 13507, (1U<<15) | 15046, (1U<<15) | 15059, (1U<<15) | 13494, (1U<<15) | 19599, 
  (1U<<15) | 19702, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 18030, (1U<<15) | 12654, (1U<<15) | 12644, (1U<<15) | 18006, 
  (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17269, 
  (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 3365, (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, 
  (1U<<15) | 17269, (1U<<15) | 17461, (1U<<15) | 17577, (1U<<15) | 17269, (1U<<15) | 18689, (1U<<15) | 3365, (1U<<15) | 18006, (1U<<15) | 12644, 
  (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18006, (1U<<15) | 12644, (1U<<15) | 18030, (1U<<15) | 17269, 
  (1U<<15) | 17269, (1U<<15) | 17999, (1U<<15) | 12636, (1U<<15) | 17999, (1U<<15) | 12636, (1U<<15) | 17269, (1U<<15) | 17297, (1U<<15) | 17201, 
  (1U<<15) | 17387, (1U<<15) | 17201, (1U<<15) | 17387, 0x4444, 0x4455, (1U<<15) | 10675, 0x4444, 0x4455, 
  (1U<<15) | 10675, 0x4444, 0x4455, (1U<<15) | 304, 0x3f44, 0x3f55, (1U<<15) | 10675, 0x4444, 
  0x4455, 0x4444, 0x4455, (1U<<15) | 10675, 0x4444, 0x4455, (1U<<15) | 10675, 0x4444, 
  0x4455, (1U<<15) | 304, 0x3f44, 0x3f55, (1U<<15) | 10675, 0x4444, 0x4455, (1U<<15) | 17269, 
  (1U<<15) | 17269, (1U<<15) | 12477, (1U<<15) | 12576, (1U<<15) | 12477, (1U<<15) | 12576, (1U<<15) | 5551, (1U<<15) | 5549, (1U<<15) | 17269, 
  (1U<<15) | 17269, (1U<<15) | 5551, (1U<<15) | 5549, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17297, (1U<<15) | 17269, (1U<<15) | 3526, 
  (1U<<15) | 3524, (1U<<15) | 17003, (1U<<15) | 17269, (1U<<15) | 17269, (1U<<15) | 17420, (1U<<15) | 17533, (1U<<15) | 17269, (1U<<15) | 17461, 
  (1U<<15) | 17577, (1U<<15) | 12446, (1U<<15) | 17003, (1U<<15) | 12513, (1U<<15) | 18030, (1U<<15) | 12654, (1U<<15) | 12630, (1U<<15) | 12630, 
  (1U<<15) | 18030, (1U<<15) | 17269, (1U<<15) | 12513, (1U<<15) | 17999, (1U<<15) | 17999, (1U<<15) | 17992, (1U<<15) | 17992, (1U<<15) | 17990, 
  (1U<<15) | 18015, (1U<<15) | 17975, (1U<<15) | 17975, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17003, (1U<<15) | 17353, (1U<<15) | 17495, 
  (1U<<15) | 17297, (1U<<15) | 1233, (1U<<15) | 1233, (1U<<15) | 1233, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 1233, (1U<<15) | 1233, 
  (1U<<15) | 1233, (1U<<15) | 17297, (1U<<15) | 17353, (1U<<15) | 17495, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 19559, (1U<<15) | 11863, 
  (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 14800, (1U<<15) | 19559, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, 
  (1U<<15) | 11863, (1U<<15) | 14800, (1U<<15) | 19559, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 14800, 
  (1U<<15) | 19559, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 14800, (1U<<15) | 19559, (1U<<15) | 11863, 
  (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 14800, (1U<<15) | 19559, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, 
  (1U<<15) | 11863, (1U<<15) | 14800, (1U<<15) | 19559, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 14800, 
  (1U<<15) | 19559, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 11863, (1U<<15) | 14800, (1U<<15) | 19610, (1U<<15) | 19610, 
  (1U<<15) | 19610, (1U<<15) | 19610, (1U<<15) | 19610, (1U<<15) | 19610, (1U<<15) | 19610, (1U<<15) | 19610, (1U<<15) | 1228, (1U<<15) | 12513, 
  (1U<<15) | 17353, (1U<<15) | 17495, (1U<<15) | 17297, (1U<<15) | 1233, (1U<<15) | 1233, (1U<<15) | 1233, (1U<<15) | 17297, (1U<<15) | 17297, 
  (1U<<15) | 1233, (1U<<15) | 1233, (1U<<15) | 1233, (1U<<15) | 17297, (1U<<15) | 17353, (1U<<15) | 17495, (1U<<15) | 17297, (1U<<15) | 17297, 
  (1U<<15) | 15293, (1U<<15) | 17279, (1U<<15) | 381, (1U<<15) | 381, (1U<<15) | 381, (1U<<15) | 381, (1U<<15) | 18980, 0x4444, 
  0x4443, 0x4443, 0x11f, (1U<<15) | 12554, 0x0, 0x0, 0x0, (1U<<15) | 21348, 
  0x42f1, 0x4, 0x4, 0x4, (1U<<15) | 8525, (1U<<15) | 8525, (1U<<15) | 8525, (1U<<15) | 8525, 
  (1U<<15) | 5908, (1U<<15) | 5908, (1U<<15) | 5908, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, (1U<<15) | 20253, (1U<<15) | 20254, (1U<<15) | 20260, (1U<<15) | 20237, (1U<<15) | 20230, 
  (1U<<15) | 20245, 0x7f2f, (1U<<15) | 21317, 0x7777, 0x7777, 0x7777, 0x7777, 0x446, 
  0x446, 0x447, 0x447, 0x447, 0x47, 0x693, (1U<<15) | 437, 0x369, 
  0x369, 0x1479, 0x1479, 0x693, (1U<<15) | 437, (1U<<15) | 437, 0x4439, 0x4439, 
  0x4474, 0x7739, 0x7739, 0x7769, (1U<<15) | 10257, (1U<<15) | 10257, (1U<<15) | 10243, (1U<<15) | 10243, 
  (1U<<15) | 10250, (1U<<15) | 10250, (1U<<15) | 9911, (1U<<15) | 10237, (1U<<15) | 9911, (1U<<15) | 9897, (1U<<15) | 9384, (1U<<15) | 9897, 
  (1U<<15) | 9904, (1U<<15) | 9601, (1U<<15) | 9904, (1U<<15) | 16718, (1U<<15) | 16718, (1U<<15) | 420, (1U<<15) | 420, 0x4747, 
  0x4747, (1U<<15) | 978, (1U<<15) | 12189, (1U<<15) | 978, (1U<<15) | 1081, (1U<<15) | 984, (1U<<15) | 1001, (1U<<15) | 966, 
  (1U<<15) | 12159, (1U<<15) | 966, (1U<<15) | 972, (1U<<15) | 12174, (1U<<15) | 972, (1U<<15) | 12664, (1U<<15) | 12195, (1U<<15) | 12208, 
  (1U<<15) | 1081, (1U<<15) | 984, (1U<<15) | 1001, (1U<<15) | 18152, (1U<<15) | 15912, (1U<<15) | 16712, (1U<<15) | 18152, (1U<<15) | 15912, 
  (1U<<15) | 16712, (1U<<15) | 15862, (1U<<15) | 15862, (1U<<15) | 18158, (1U<<15) | 15918, (1U<<15) | 15850, (1U<<15) | 15850, (1U<<15) | 15856, 
  (1U<<15) | 15856, (1U<<15) | 18158, (1U<<15) | 15918, (1U<<15) | 18146, (1U<<15) | 15906, (1U<<15) | 16706, (1U<<15) | 18147, 0x76b4, 
  0x77b4, (1U<<15) | 18146, (1U<<15) | 15906, (1U<<15) | 16706, (1U<<15) | 12180, (1U<<15) | 12150, (1U<<15) | 12165, (1U<<15) | 12180, 
  (1U<<15) | 12150, (1U<<15) | 12165, (1U<<15) | 12187, (1U<<15) | 12157, (1U<<15) | 12172, (1U<<15) | 15836, (1U<<15) | 15794, (1U<<15) | 15815, 
  (1U<<15) | 15836, (1U<<15) | 15794, (1U<<15) | 15815, (1U<<15) | 15843, (1U<<15) | 15801, (1U<<15) | 15822, (1U<<15) | 15843, (1U<<15) | 15801, 
  (1U<<15) | 15822, (1U<<15) | 15829, (1U<<15) | 15780, (1U<<15) | 15808, (1U<<15) | 15830, (1U<<15) | 15781, (1U<<15) | 15809, (1U<<15) | 15829, 
  (1U<<15) | 15780, (1U<<15) | 15808, (1U<<15) | 443, (1U<<15) | 9378, (1U<<15) | 9417, (1U<<15) | 429, (1U<<15) | 9378, (1U<<15) | 9417, 
  (1U<<15) | 9417, (1U<<15) | 12228, (1U<<15) | 12202, 0xf, 0x5, (1U<<15) | 14329, (1U<<15) | 17344, (1U<<15) | 1039, 
  (1U<<15) | 1028, 0x7447, 0x7447, 0x7447, 0x7447, 0x441f, 0x14f4, (1U<<15) | 3572, 
  (1U<<15) | 14567, 0x444, 0x444, (1U<<15) | 10811, (1U<<15) | 10944, (1U<<15) | 10954, (1U<<15) | 10811, 0x14f4, 
  0x440, 0x440, 0x440, 0x40, 0x40, 0x40, (1U<<15) | 3571, (1U<<15) | 3571, 
  (1U<<15) | 3571, (1U<<15) | 3571, (1U<<15) | 59, (1U<<15) | 59, 0x444, (1U<<15) | 3571, (1U<<15) | 3571, (1U<<15) | 3571, 
  (1U<<15) | 3571, 0x441f, 0x444, (1U<<15) | 18532, 0x1f0, 0x0, (1U<<15) | 215, (1U<<15) | 179, 
  (1U<<15) | 3945, 0x7f2f, (1U<<15) | 14185, 0x777, (1U<<15) | 993, (1U<<15) | 21852, (1U<<15) | 15346, (1U<<15) | 1090, 
  (1U<<15) | 1090, (1U<<15) | 20208, (1U<<15) | 20208, 0x4e0f, 0x4e0f, 0x4e0f, 0x4e0, 0x7777, 
  (1U<<15) | 17344, (1U<<15) | 17344, 0x777, 0x7f2f, (1U<<15) | 18648, 0x7f2f, (1U<<15) | 20208, (1U<<15) | 20208, 
  (1U<<15) | 14547, (1U<<15) | 8429, (1U<<15) | 8429, (1U<<15) | 8429, (1U<<15) | 8429, (1U<<15) | 8525, (1U<<15) | 4285, (1U<<15) | 4285, 
  (1U<<15) | 4285, (1U<<15) | 1335, (1U<<15) | 1335, (1U<<15) | 1335, (1U<<15) | 1335, (1U<<15) | 4286, (1U<<15) | 8429, (1U<<15) | 8429, 
  (1U<<15) | 8429, (1U<<15) | 8429, 0x4, (1U<<15) | 14177, (1U<<15) | 229, (1U<<15) | 16978, 0x40, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11429, 
  (1U<<15) | 11504, (1U<<15) | 11504, (1U<<15) | 11591, (1U<<15) | 11690, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11440, 
  (1U<<15) | 11517, (1U<<15) | 11517, (1U<<15) | 11606, (1U<<15) | 11707, (1U<<15) | 11606, (1U<<15) | 11606, (1U<<15) | 11606, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11429, 
  (1U<<15) | 11504, (1U<<15) | 11504, (1U<<15) | 11591, (1U<<15) | 11690, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11429, 
  (1U<<15) | 11504, (1U<<15) | 11504, (1U<<15) | 11591, (1U<<15) | 11690, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11429, 
  (1U<<15) | 11504, (1U<<15) | 11504, (1U<<15) | 11591, (1U<<15) | 11690, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11429, 
  (1U<<15) | 11504, (1U<<15) | 11504, (1U<<15) | 11591, (1U<<15) | 11690, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11429, 
  (1U<<15) | 11504, (1U<<15) | 11504, (1U<<15) | 11591, (1U<<15) | 11690, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11429, 
  (1U<<15) | 11504, (1U<<15) | 11504, (1U<<15) | 11591, (1U<<15) | 11690, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11591, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11407, 
  (1U<<15) | 11478, (1U<<15) | 11478, (1U<<15) | 11561, (1U<<15) | 11656, (1U<<15) | 11561, (1U<<15) | 11561, (1U<<15) | 11561, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11418, 
  (1U<<15) | 11491, (1U<<15) | 11491, (1U<<15) | 11576, (1U<<15) | 11673, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 11576, (1U<<15) | 13345, 
  (1U<<15) | 13480, (1U<<15) | 13326, (1U<<15) | 6132, (1U<<15) | 6436, (1U<<15) | 7416, (1U<<15) | 7668, (1U<<15) | 7668, (1U<<15) | 8064, 
  (1U<<15) | 8064, (1U<<15) | 7687, (1U<<15) | 8085, (1U<<15) | 8085, (1U<<15) | 7668, (1U<<15) | 7433, (1U<<15) | 7687, (1U<<15) | 7687, 
  (1U<<15) | 6198, (1U<<15) | 6510, (1U<<15) | 7379, (1U<<15) | 7627, (1U<<15) | 7627, (1U<<15) | 8019, (1U<<15) | 8019, (1U<<15) | 7647, 
  (1U<<15) | 8041, (1U<<15) | 8041, (1U<<15) | 7627, (1U<<15) | 7397, (1U<<15) | 7647, (1U<<15) | 7647, (1U<<15) | 6510, (1U<<15) | 6970, 
  (1U<<15) | 6970, (1U<<15) | 6528, (1U<<15) | 6990, (1U<<15) | 6990, (1U<<15) | 6510, (1U<<15) | 6510, (1U<<15) | 6970, (1U<<15) | 6970, 
  (1U<<15) | 6528, (1U<<15) | 6990, (1U<<15) | 6990, (1U<<15) | 6198, (1U<<15) | 6510, (1U<<15) | 6510, (1U<<15) | 6214, (1U<<15) | 6528, 
  (1U<<15) | 6528, (1U<<15) | 6214, (1U<<15) | 6528, (1U<<15) | 6528, (1U<<15) | 6436, (1U<<15) | 6888, (1U<<15) | 6888, (1U<<15) | 6453, 
  (1U<<15) | 6907, (1U<<15) | 6907, (1U<<15) | 6436, (1U<<15) | 6436, (1U<<15) | 6888, (1U<<15) | 6888, (1U<<15) | 6453, (1U<<15) | 6907, 
  (1U<<15) | 6907, (1U<<15) | 6132, (1U<<15) | 6436, (1U<<15) | 6436, (1U<<15) | 6147, (1U<<15) | 6453, (1U<<15) | 6453, (1U<<15) | 6147, 
  (1U<<15) | 6453, (1U<<15) | 6453, (1U<<15) | 5944, (1U<<15) | 6132, (1U<<15) | 6132, (1U<<15) | 6436, (1U<<15) | 6436, (1U<<15) | 6436, 
  (1U<<15) | 11397, (1U<<15) | 11397, (1U<<15) | 11397, (1U<<15) | 11397, (1U<<15) | 11397, (1U<<15) | 11397, (1U<<15) | 11397, (1U<<15) | 11397, 
  (1U<<15) | 11386, (1U<<15) | 11453, (1U<<15) | 11453, (1U<<15) | 11532, (1U<<15) | 11623, (1U<<15) | 11532, (1U<<15) | 11532, (1U<<15) | 11532, 
  (1U<<15) | 11453, (1U<<15) | 11532, (1U<<15) | 11532, (1U<<15) | 11623, (1U<<15) | 11623, (1U<<15) | 11623, (1U<<15) | 11623, (1U<<15) | 11532, 
  (1U<<15) | 11623, (1U<<15) | 11532, (1U<<15) | 5944, (1U<<15) | 5652, (1U<<15) | 6132, (1U<<15) | 5704, (1U<<15) | 6132, (1U<<15) | 5704, 
  (1U<<15) | 6436, (1U<<15) | 5764, (1U<<15) | 6436, (1U<<15) | 5764, (1U<<15) | 7276, (1U<<15) | 5915, (1U<<15) | 7416, (1U<<15) | 6099, 
  (1U<<15) | 7416, (1U<<15) | 6099, (1U<<15) | 7668, (1U<<15) | 6399, (1U<<15) | 7668, (1U<<15) | 6399, (1U<<15) | 7416, (1U<<15) | 6099, 
  (1U<<15) | 7668, (1U<<15) | 6399, (1U<<15) | 7668, (1U<<15) | 6399, (1U<<15) | 8064, (1U<<15) | 6847, (1U<<15) | 8064, (1U<<15) | 6847, 
  (1U<<15) | 8064, (1U<<15) | 6847, (1U<<15) | 7433, (1U<<15) | 6115, (1U<<15) | 7687, (1U<<15) | 6417, (1U<<15) | 7687, (1U<<15) | 6417, 
  (1U<<15) | 8085, (1U<<15) | 6867, (1U<<15) | 8085, (1U<<15) | 6867, (1U<<15) | 8085, (1U<<15) | 6867, (1U<<15) | 7668, (1U<<15) | 6399, 
  (1U<<15) | 7291, (1U<<15) | 5929, (1U<<15) | 7433, (1U<<15) | 6115, (1U<<15) | 7433, (1U<<15) | 6115, (1U<<15) | 7687, (1U<<15) | 6417, 
  (1U<<15) | 7687, (1U<<15) | 6417, (1U<<15) | 7687, (1U<<15) | 6417, (1U<<15) | 6002, (1U<<15) | 5677, (1U<<15) | 6198, (1U<<15) | 5733, 
  (1U<<15) | 6198, (1U<<15) | 5733, (1U<<15) | 6510, (1U<<15) | 5797, (1U<<15) | 6510, (1U<<15) | 5797, (1U<<15) | 7243, (1U<<15) | 5971, 
  (1U<<15) | 7379, (1U<<15) | 6163, (1U<<15) | 7379, (1U<<15) | 6163, (1U<<15) | 7627, (1U<<15) | 6471, (1U<<15) | 7627, (1U<<15) | 6471, 
  (1U<<15) | 7379, (1U<<15) | 6163, (1U<<15) | 7627, (1U<<15) | 6471, (1U<<15) | 7627, (1U<<15) | 6471, (1U<<15) | 8019, (1U<<15) | 6927, 
  (1U<<15) | 8019, (1U<<15) | 6927, (1U<<15) | 8019, (1U<<15) | 6927, (1U<<15) | 7397, (1U<<15) | 6180, (1U<<15) | 7647, (1U<<15) | 6490, 
  (1U<<15) | 7647, (1U<<15) | 6490, (1U<<15) | 8041, (1U<<15) | 6948, (1U<<15) | 8041, (1U<<15) | 6948, (1U<<15) | 8041, (1U<<15) | 6948, 
  (1U<<15) | 7627, (1U<<15) | 6471, (1U<<15) | 7259, (1U<<15) | 5986, (1U<<15) | 7397, (1U<<15) | 6180, (1U<<15) | 7397, (1U<<15) | 6180, 
  (1U<<15) | 7647, (1U<<15) | 6490, (1U<<15) | 7647, (1U<<15) | 6490, (1U<<15) | 7647, (1U<<15) | 6490, (1U<<15) | 7342, (1U<<15) | 6064, 
  (1U<<15) | 7490, (1U<<15) | 6268, (1U<<15) | 7578, (1U<<15) | 6352, (1U<<15) | 7846, (1U<<15) | 6680, (1U<<15) | 7958, (1U<<15) | 6788, 
  (1U<<15) | 7490, (1U<<15) | 6268, (1U<<15) | 7750, (1U<<15) | 6588, (1U<<15) | 7846, (1U<<15) | 6680, (1U<<15) | 8162, (1U<<15) | 7064, 
  (1U<<15) | 8282, (1U<<15) | 7180, (1U<<15) | 8162, (1U<<15) | 7064, (1U<<15) | 7510, (1U<<15) | 6287, (1U<<15) | 7772, (1U<<15) | 6609, 
  (1U<<15) | 7872, (1U<<15) | 6705, (1U<<15) | 8190, (1U<<15) | 7091, (1U<<15) | 8314, (1U<<15) | 7211, (1U<<15) | 8190, (1U<<15) | 7091, 
  (1U<<15) | 7846, (1U<<15) | 6680, (1U<<15) | 7360, (1U<<15) | 6081, (1U<<15) | 7510, (1U<<15) | 6287, (1U<<15) | 7602, (1U<<15) | 6375, 
  (1U<<15) | 7872, (1U<<15) | 6705, (1U<<15) | 7988, (1U<<15) | 6817, (1U<<15) | 7872, (1U<<15) | 6705, (1U<<15) | 6198, (1U<<15) | 5733, 
  (1U<<15) | 6510, (1U<<15) | 5797, (1U<<15) | 6510, (1U<<15) | 5797, (1U<<15) | 6970, (1U<<15) | 5869, (1U<<15) | 6970, (1U<<15) | 5869, 
  (1U<<15) | 6970, (1U<<15) | 5869, (1U<<15) | 6214, (1U<<15) | 5748, (1U<<15) | 6528, (1U<<15) | 5814, (1U<<15) | 6528, (1U<<15) | 5814, 
  (1U<<15) | 6990, (1U<<15) | 5888, (1U<<15) | 6990, (1U<<15) | 5888, (1U<<15) | 6990, (1U<<15) | 5888, (1U<<15) | 6510, (1U<<15) | 5797, 
  (1U<<15) | 7342, (1U<<15) | 6064, (1U<<15) | 7490, (1U<<15) | 6268, (1U<<15) | 7578, (1U<<15) | 6352, (1U<<15) | 7846, (1U<<15) | 6680, 
  (1U<<15) | 7958, (1U<<15) | 6788, (1U<<15) | 7490, (1U<<15) | 6268, (1U<<15) | 7750, (1U<<15) | 6588, (1U<<15) | 7846, (1U<<15) | 6680, 
  (1U<<15) | 8162, (1U<<15) | 7064, (1U<<15) | 8282, (1U<<15) | 7180, (1U<<15) | 8162, (1U<<15) | 7064, (1U<<15) | 7510, (1U<<15) | 6287, 
  (1U<<15) | 7772, (1U<<15) | 6609, (1U<<15) | 7872, (1U<<15) | 6705, (1U<<15) | 8190, (1U<<15) | 7091, (1U<<15) | 8314, (1U<<15) | 7211, 
  (1U<<15) | 8190, (1U<<15) | 7091, (1U<<15) | 7846, (1U<<15) | 6680, (1U<<15) | 7360, (1U<<15) | 6081, (1U<<15) | 7510, (1U<<15) | 6287, 
  (1U<<15) | 7602, (1U<<15) | 6375, (1U<<15) | 7872, (1U<<15) | 6705, (1U<<15) | 7988, (1U<<15) | 6817, (1U<<15) | 7872, (1U<<15) | 6705, 
  (1U<<15) | 6198, (1U<<15) | 5733, (1U<<15) | 6510, (1U<<15) | 5797, (1U<<15) | 6510, (1U<<15) | 5797, (1U<<15) | 6970, (1U<<15) | 5869, 
  (1U<<15) | 6970, (1U<<15) | 5869, (1U<<15) | 6970, (1U<<15) | 5869, (1U<<15) | 6214, (1U<<15) | 5748, (1U<<15) | 6528, (1U<<15) | 5814, 
  (1U<<15) | 6528, (1U<<15) | 5814, (1U<<15) | 6990, (1U<<15) | 5888, (1U<<15) | 6990, (1U<<15) | 5888, (1U<<15) | 6990, (1U<<15) | 5888, 
  (1U<<15) | 6002, (1U<<15) | 5677, (1U<<15) | 6198, (1U<<15) | 5733, (1U<<15) | 6198, (1U<<15) | 5733, (1U<<15) | 6510, (1U<<15) | 5797, 
  (1U<<15) | 6510, (1U<<15) | 5797, (1U<<15) | 6510, (1U<<15) | 5797, (1U<<15) | 6016, (1U<<15) | 5690, (1U<<15) | 6214, (1U<<15) | 5748, 
  (1U<<15) | 6214, (1U<<15) | 5748, (1U<<15) | 6528, (1U<<15) | 5814, (1U<<15) | 6528, (1U<<15) | 5814, (1U<<15) | 6528, (1U<<15) | 5814, 
  (1U<<15) | 6016, (1U<<15) | 5690, (1U<<15) | 6214, (1U<<15) | 5748, (1U<<15) | 6214, (1U<<15) | 5748, (1U<<15) | 6528, (1U<<15) | 5814, 
  (1U<<15) | 6528, (1U<<15) | 5814, (1U<<15) | 6528, (1U<<15) | 5814, (1U<<15) | 7307, (1U<<15) | 6031, (1U<<15) | 7451, (1U<<15) | 6231, 
  (1U<<15) | 7531, (1U<<15) | 6307, (1U<<15) | 7795, (1U<<15) | 6631, (1U<<15) | 7899, (1U<<15) | 6731, (1U<<15) | 7451, (1U<<15) | 6231, 
  (1U<<15) | 7707, (1U<<15) | 6547, (1U<<15) | 7795, (1U<<15) | 6631, (1U<<15) | 8107, (1U<<15) | 7011, (1U<<15) | 8219, (1U<<15) | 7119, 
  (1U<<15) | 8107, (1U<<15) | 7011, (1U<<15) | 7470, (1U<<15) | 6249, (1U<<15) | 7728, (1U<<15) | 6567, (1U<<15) | 7820, (1U<<15) | 6655, 
  (1U<<15) | 8134, (1U<<15) | 7037, (1U<<15) | 8250, (1U<<15) | 7149, (1U<<15) | 8134, (1U<<15) | 7037, (1U<<15) | 7795, (1U<<15) | 6631, 
  (1U<<15) | 7324, (1U<<15) | 6047, (1U<<15) | 7470, (1U<<15) | 6249, (1U<<15) | 7554, (1U<<15) | 6329, (1U<<15) | 7820, (1U<<15) | 6655, 
  (1U<<15) | 7928, (1U<<15) | 6759, (1U<<15) | 7820, (1U<<15) | 6655, (1U<<15) | 6132, (1U<<15) | 5704, (1U<<15) | 6436, (1U<<15) | 5764, 
  (1U<<15) | 6436, (1U<<15) | 5764, (1U<<15) | 6888, (1U<<15) | 5832, (1U<<15) | 6888, (1U<<15) | 5832, (1U<<15) | 6888, (1U<<15) | 5832, 
  (1U<<15) | 6147, (1U<<15) | 5718, (1U<<15) | 6453, (1U<<15) | 5780, (1U<<15) | 6453, (1U<<15) | 5780, (1U<<15) | 6907, (1U<<15) | 5850, 
  (1U<<15) | 6907, (1U<<15) | 5850, (1U<<15) | 6907, (1U<<15) | 5850, (1U<<15) | 6436, (1U<<15) | 5764, (1U<<15) | 7307, (1U<<15) | 6031, 
  (1U<<15) | 7451, (1U<<15) | 6231, (1U<<15) | 7531, (1U<<15) | 6307, (1U<<15) | 7795, (1U<<15) | 6631, (1U<<15) | 7899, (1U<<15) | 6731, 
  (1U<<15) | 7451, (1U<<15) | 6231, (1U<<15) | 7707, (1U<<15) | 6547, (1U<<15) | 7795, (1U<<15) | 6631, (1U<<15) | 8107, (1U<<15) | 7011, 
  (1U<<15) | 8219, (1U<<15) | 7119, (1U<<15) | 8107, (1U<<15) | 7011, (1U<<15) | 7470, (1U<<15) | 6249, (1U<<15) | 7728, (1U<<15) | 6567, 
  (1U<<15) | 7820, (1U<<15) | 6655, (1U<<15) | 8134, (1U<<15) | 7037, (1U<<15) | 8250, (1U<<15) | 7149, (1U<<15) | 8134, (1U<<15) | 7037, 
  (1U<<15) | 7795, (1U<<15) | 6631, (1U<<15) | 7324, (1U<<15) | 6047, (1U<<15) | 7470, (1U<<15) | 6249, (1U<<15) | 7554, (1U<<15) | 6329, 
  (1U<<15) | 7820, (1U<<15) | 6655, (1U<<15) | 7928, (1U<<15) | 6759, (1U<<15) | 7820, (1U<<15) | 6655, (1U<<15) | 6132, (1U<<15) | 5704, 
  (1U<<15) | 6436, (1U<<15) | 5764, (1U<<15) | 6436, (1U<<15) | 5764, (1U<<15) | 6888, (1U<<15) | 5832, (1U<<15) | 6888, (1U<<15) | 5832, 
  (1U<<15) | 6888, (1U<<15) | 5832, (1U<<15) | 6147, (1U<<15) | 5718, (1U<<15) | 6453, (1U<<15) | 5780, (1U<<15) | 6453, (1U<<15) | 5780, 
  (1U<<15) | 6907, (1U<<15) | 5850, (1U<<15) | 6907, (1U<<15) | 5850, (1U<<15) | 6907, (1U<<15) | 5850, (1U<<15) | 5944, (1U<<15) | 5652, 
  (1U<<15) | 6132, (1U<<15) | 5704, (1U<<15) | 6132, (1U<<15) | 5704, (1U<<15) | 6436, (1U<<15) | 5764, (1U<<15) | 6436, (1U<<15) | 5764, 
  (1U<<15) | 6436, (1U<<15) | 5764, (1U<<15) | 5957, (1U<<15) | 5664, (1U<<15) | 6147, (1U<<15) | 5718, (1U<<15) | 6147, (1U<<15) | 5718, 
  (1U<<15) | 6453, (1U<<15) | 5780, (1U<<15) | 6453, (1U<<15) | 5780, (1U<<15) | 6453, (1U<<15) | 5780, (1U<<15) | 5957, (1U<<15) | 5664, 
  (1U<<15) | 6147, (1U<<15) | 5718, (1U<<15) | 6147, (1U<<15) | 5718, (1U<<15) | 6453, (1U<<15) | 5780, (1U<<15) | 6453, (1U<<15) | 5780, 
  (1U<<15) | 6453, (1U<<15) | 5780, (1U<<15) | 11396, (1U<<15) | 11465, (1U<<15) | 11465, (1U<<15) | 11546, (1U<<15) | 11639, (1U<<15) | 11546, 
  (1U<<15) | 11546, (1U<<15) | 11546, (1U<<15) | 11465, (1U<<15) | 11546, (1U<<15) | 11546, (1U<<15) | 11639, (1U<<15) | 11639, (1U<<15) | 11639, 
  (1U<<15) | 14329, (1U<<15) | 14329, 0x50, 0x440, 0x1, 0x7777, (1U<<15) | 1016, 0x7777, 
  (1U<<15) | 1010, (1U<<15) | 8990, (1U<<15) | 9424, (1U<<15) | 3938, (1U<<15) | 1016, (1U<<15) | 9423, (1U<<15) | 3937, (1U<<15) | 1010, 
  0x1f1, 0xe1, 0xe1, (1U<<15) | 14329, 0x10, 0x40f, 0x4, 0x4447, 
  0x4444, 0x1, (1U<<15) | 8534, 0x7f2f, 0x7f2f, 0x1f1, (1U<<15) | 11768, 0x444, 
  0x444, (1U<<15) | 9449, (1U<<15) | 9507, (1U<<15) | 9725, (1U<<15) | 9607, (1U<<15) | 9577, (1U<<15) | 9497, (1U<<15) | 9497, 
  (1U<<15) | 9531, (1U<<15) | 9497, (1U<<15) | 9497, (1U<<15) | 9619, (1U<<15) | 9677, (1U<<15) | 9555, (1U<<15) | 9565, (1U<<15) | 9437, 
  (1U<<15) | 9747, (1U<<15) | 9667, (1U<<15) | 9667, (1U<<15) | 9701, (1U<<15) | 9667, (1U<<15) | 9667, (1U<<15) | 9807, (1U<<15) | 9771, 
  (1U<<15) | 9725, (1U<<15) | 9735, (1U<<15) | 9607, (1U<<15) | 9783, (1U<<15) | 9795, (1U<<15) | 9619, (1U<<15) | 9677, (1U<<15) | 9555, 
  (1U<<15) | 9437, (1U<<15) | 9449, (1U<<15) | 9507, (1U<<15) | 9837, (1U<<15) | 9829, (1U<<15) | 9059, (1U<<15) | 9105, (1U<<15) | 9147, 
  (1U<<15) | 9081, (1U<<15) | 9193, (1U<<15) | 9169, (1U<<15) | 9203, (1U<<15) | 9147, (1U<<15) | 9059, (1U<<15) | 8846, (1U<<15) | 8861, 
  (1U<<15) | 387, (1U<<15) | 12440, 0x5455, (1U<<15) | 13319, 0x4444, 0x441f, 0x441f, 0x444, 
  0x444, 0x4444, (1U<<15) | 13228, (1U<<15) | 13235, (1U<<15) | 13242, 0x4444, 0x4444, 0x444, 
  0x4444, 0x4444, (1U<<15) | 117, (1U<<15) | 70, (1U<<15) | 72, (1U<<15) | 70, 0x7f0f, (1U<<15) | 117, 
  (1U<<15) | 72, (1U<<15) | 278, (1U<<15) | 278, (1U<<15) | 278, (1U<<15) | 278, 0x4, 0x44, 0x1, 
  0x5455, (1U<<15) | 14329, (1U<<15) | 9042, (1U<<15) | 9115, (1U<<15) | 9115, (1U<<15) | 9135, (1U<<15) | 9115, (1U<<15) | 9115, 
  (1U<<15) | 9125, (1U<<15) | 9125, (1U<<15) | 9125, (1U<<15) | 9115, (1U<<15) | 9115, (1U<<15) | 9115, (1U<<15) | 9115, (1U<<15) | 9115, 
  (1U<<15) | 9115, (1U<<15) | 9115, (1U<<15) | 9115, (1U<<15) | 9115, (1U<<15) | 9115, (1U<<15) | 9042, (1U<<15) | 9042, (1U<<15) | 8576, 
  (1U<<15) | 9041, (1U<<15) | 9050, (1U<<15) | 9848, (1U<<15) | 9865, (1U<<15) | 9865, (1U<<15) | 9885, (1U<<15) | 9865, (1U<<15) | 9865, 
  (1U<<15) | 9875, (1U<<15) | 9875, (1U<<15) | 9875, (1U<<15) | 9865, (1U<<15) | 9865, (1U<<15) | 9865, (1U<<15) | 9865, (1U<<15) | 9865, 
  (1U<<15) | 9865, (1U<<15) | 9865, (1U<<15) | 9865, (1U<<15) | 9865, (1U<<15) | 9865, (1U<<15) | 9848, (1U<<15) | 9857, (1U<<15) | 8587, 
  (1U<<15) | 9847, (1U<<15) | 9856, (1U<<15) | 8997, (1U<<15) | 8996, (1U<<15) | 8920, (1U<<15) | 8919, 0x7f2f, 0x77, 
  0x7f0f, (1U<<15) | 12440, (1U<<15) | 20227, 0x7f2f, 0x7f2f, 0x77, 0x0, (1U<<15) | 5566, 
  (1U<<15) | 3572, 0x30, 0x40, 0x41, (1U<<15) | 5566, 0x30, 0x45, (1U<<15) | 9918, 
  (1U<<15) | 10798, 0x0, 0x0, 0x0, 0x0, 0x0, 0x40, 0x44, 
  (1U<<15) | 3582, 0x4, 0x5, 0x44, 0x40, 0x5, 0x5, 0x30, 
  0x30, 0x44f0, (1U<<15) | 18535, 0x440, 0x41f, 0x440, 0x40, 0x30, 
  0x30, 0x440, 0x40, 0x40, 0x40, 0x30, 0x30, 0x30, 
  0x30, 0x0, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30, 
  0x40, (1U<<15) | 3572, (1U<<15) | 18535, 0x4444, 0x4444, 0x4444, 0x43, 0x43, 
  (1U<<15) | 10666, 0x40, 0x4440, (1U<<15) | 365, (1U<<15) | 381, (1U<<15) | 381, (1U<<15) | 17250, (1U<<15) | 17279, 
  0x7f1f, 0x7f2f, (1U<<15) | 9485, (1U<<15) | 9485, (1U<<15) | 9485, (1U<<15) | 9485, (1U<<15) | 9461, (1U<<15) | 9519, 
  (1U<<15) | 9589, (1U<<15) | 9473, (1U<<15) | 9473, (1U<<15) | 9543, (1U<<15) | 9473, (1U<<15) | 9473, (1U<<15) | 9631, (1U<<15) | 9689, 
  (1U<<15) | 9759, (1U<<15) | 9643, (1U<<15) | 9643, (1U<<15) | 9713, (1U<<15) | 9643, (1U<<15) | 9643, (1U<<15) | 9655, (1U<<15) | 9655, 
  (1U<<15) | 9655, (1U<<15) | 9655, (1U<<15) | 9093, (1U<<15) | 9069, (1U<<15) | 9157, (1U<<15) | 9181, 0x7f0f, 0x7f2f, 
  0x7f0f, 0x7f0f, (1U<<15) | 8920, (1U<<15) | 8929, (1U<<15) | 8929, (1U<<15) | 8951, (1U<<15) | 8929, (1U<<15) | 8929, 
  (1U<<15) | 8940, (1U<<15) | 8940, (1U<<15) | 8940, (1U<<15) | 8929, (1U<<15) | 8929, (1U<<15) | 8929, (1U<<15) | 8929, (1U<<15) | 8929, 
  (1U<<15) | 8929, (1U<<15) | 8929, (1U<<15) | 8929, (1U<<15) | 8929, (1U<<15) | 8929, (1U<<15) | 8920, (1U<<15) | 8920, (1U<<15) | 8607, 
  (1U<<15) | 8919, (1U<<15) | 8919, (1U<<15) | 8997, (1U<<15) | 9006, (1U<<15) | 9006, (1U<<15) | 9028, (1U<<15) | 9006, (1U<<15) | 9006, 
  (1U<<15) | 9017, (1U<<15) | 9017, (1U<<15) | 9017, (1U<<15) | 9006, (1U<<15) | 9006, (1U<<15) | 9006, (1U<<15) | 9006, (1U<<15) | 9006, 
  (1U<<15) | 9006, (1U<<15) | 9006, (1U<<15) | 9006, (1U<<15) | 9006, (1U<<15) | 9006, (1U<<15) | 8997, (1U<<15) | 8997, (1U<<15) | 8619, 
  (1U<<15) | 8996, (1U<<15) | 8996, (1U<<15) | 8909, (1U<<15) | 8908, (1U<<15) | 8877, (1U<<15) | 8876, (1U<<15) | 373, (1U<<15) | 373, 
  (1U<<15) | 21162, (1U<<15) | 263, (1U<<15) | 263, (1U<<15) | 250, (1U<<15) | 250, (1U<<15) | 250, (1U<<15) | 250, (1U<<15) | 21184, 
  (1U<<15) | 263, (1U<<15) | 250, (1U<<15) | 250, (1U<<15) | 250, (1U<<15) | 250, (1U<<15) | 21173, (1U<<15) | 21173, (1U<<15) | 21173, 
  (1U<<15) | 21184, (1U<<15) | 21173, (1U<<15) | 21173, (1U<<15) | 263, (1U<<15) | 263, (1U<<15) | 43, (1U<<15) | 1350, (1U<<15) | 1350, 
  (1U<<15) | 1350, 0x7f2f, (1U<<15) | 10881, (1U<<15) | 10964, (1U<<15) | 10881, (1U<<15) | 10964, (1U<<15) | 12455, (1U<<15) | 10666, 
  (1U<<15) | 365, (1U<<15) | 381, (1U<<15) | 381, 0x0, (1U<<15) | 396, 0x0, 0x4, (1U<<15) | 12440, 
  (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, 
  (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, (1U<<15) | 12440, 0x4, (1U<<15) | 1061, (1U<<15) | 1061, 
  (1U<<15) | 189, (1U<<15) | 236, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 1071, (1U<<15) | 1071, 
  (1U<<15) | 189, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 17208, 
  (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 17796, (1U<<15) | 17796, (1U<<15) | 17796, (1U<<15) | 17805, (1U<<15) | 17796, (1U<<15) | 17796, 
  (1U<<15) | 189, (1U<<15) | 189, (1U<<15) | 189, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 203, (1U<<15) | 17220, 
  (1U<<15) | 1049, (1U<<15) | 1049, (1U<<15) | 1049, (1U<<15) | 21, (1U<<15) | 79, (1U<<15) | 99, (1U<<15) | 141, (1U<<15) | 161, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x7f0f, 0x10, 
  0x11, (1U<<15) | 17236, 0x7f0f, 0x444, 0x4444, (1U<<15) | 8543, (1U<<15) | 8598, 0x4444, 
  (1U<<15) | 8570, (1U<<15) | 8568, (1U<<15) | 8631, (1U<<15) | 8570, (1U<<15) | 8600, (1U<<15) | 8598, (1U<<15) | 8641, 0x442f, 
  (1U<<15) | 12485, 0x442c, (1U<<15) | 18844, (1U<<15) | 5056, (1U<<15) | 18844, (1U<<15) | 12485, (1U<<15) | 12521, (1U<<15) | 5056, 
  (1U<<15) | 18741, (1U<<15) | 5023, (1U<<15) | 18741, (1U<<15) | 12521, (1U<<15) | 12588, (1U<<15) | 5023, (1U<<15) | 18728, (1U<<15) | 5002, 
  (1U<<15) | 18728, (1U<<15) | 8599, (1U<<15) | 8599, 0x0, 0x44, 0x54, 0xe4, 0xe4, 
  0xe4, 0xe4, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x40, 0x40, 0x40, 0x4, 0x0, 0x40, 0x40, 0x4f4, 
  (1U<<15) | 19240, (1U<<15) | 19241, (1U<<15) | 19241, (1U<<15) | 19241, (1U<<15) | 19241, 0x4f4, (1U<<15) | 19240, (1U<<15) | 8599, 
  (1U<<15) | 8599, (1U<<15) | 8569, (1U<<15) | 8569, (1U<<15) | 8600, (1U<<15) | 8600, (1U<<15) | 8568, (1U<<15) | 8568, (1U<<15) | 17743, 
  (1U<<15) | 17755, (1U<<15) | 17764, 0x43f5, (1U<<15) | 18767, 0x43f4, (1U<<15) | 18760, (1U<<15) | 17764, (1U<<15) | 8568, 
  (1U<<15) | 17764, (1U<<15) | 17755, (1U<<15) | 17755, (1U<<15) | 19584, (1U<<15) | 19584, (1U<<15) | 19584, (1U<<15) | 19584, (1U<<15) | 19584, 
  (1U<<15) | 19584, (1U<<15) | 17764, (1U<<15) | 17355, (1U<<15) | 18910, (1U<<15) | 17743, (1U<<15) | 17743, (1U<<15) | 8568, (1U<<15) | 17743, 
  0x3f44, (1U<<15) | 18707, (1U<<15) | 18878, (1U<<15) | 20398, (1U<<15) | 18878, (1U<<15) | 20398, (1U<<15) | 5543, (1U<<15) | 18759, 
  (1U<<15) | 17743, 0x3f44, (1U<<15) | 18707, (1U<<15) | 18878, (1U<<15) | 20398, (1U<<15) | 18878, (1U<<15) | 20398, (1U<<15) | 5543, 
  (1U<<15) | 18759, (1U<<15) | 17764, (1U<<15) | 17743, (1U<<15) | 17851, (1U<<15) | 17874, (1U<<15) | 17755, (1U<<15) | 17755, (1U<<15) | 17764, 
  (1U<<15) | 17764, 0x43f, 0x3f4, (1U<<15) | 17755, (1U<<15) | 17743, (1U<<15) | 17764, (1U<<15) | 17755, (1U<<15) | 17764, 
  (1U<<15) | 17743, (1U<<15) | 17743, (1U<<15) | 17743, (1U<<15) | 17733, (1U<<15) | 17722, 0x444, (1U<<15) | 8598, 0x444, 
  (1U<<15) | 8568, 0x444, (1U<<15) | 8568, (1U<<15) | 17764, 0x444, (1U<<15) | 8598, 0x444, (1U<<15) | 8568, 
  0x444, (1U<<15) | 8568, (1U<<15) | 17185, (1U<<15) | 18891, (1U<<15) | 12513, (1U<<15) | 12530, (1U<<15) | 18863, (1U<<15) | 17297, 
  (1U<<15) | 12463, (1U<<15) | 18853, (1U<<15) | 17312, (1U<<15) | 18933, 0x7f3f, (1U<<15) | 17398, (1U<<15) | 18921, (1U<<15) | 17304, 
  (1U<<15) | 18921, 0x41b, 0x41a, 0x419, 0x41c, (1U<<15) | 13598, (1U<<15) | 20356, (1U<<15) | 13592, 
  (1U<<15) | 17864, (1U<<15) | 13592, (1U<<15) | 12323, (1U<<15) | 1126, (1U<<15) | 12101, (1U<<15) | 1116, (1U<<15) | 18753, (1U<<15) | 20407, 
  (1U<<15) | 18753, (1U<<15) | 20407, (1U<<15) | 18753, (1U<<15) | 20407, (1U<<15) | 18753, (1U<<15) | 20407, (1U<<15) | 8517, (1U<<15) | 18824, 
  (1U<<15) | 8559, (1U<<15) | 18785, (1U<<15) | 12513, (1U<<15) | 12513, (1U<<15) | 8517, (1U<<15) | 18824, (1U<<15) | 8559, (1U<<15) | 18785, 
  (1U<<15) | 19072, (1U<<15) | 19114, (1U<<15) | 13592, (1U<<15) | 20338, (1U<<15) | 14193, (1U<<15) | 20367, (1U<<15) | 10227, (1U<<15) | 21379, 
  (1U<<15) | 18901, (1U<<15) | 17297, (1U<<15) | 18901, (1U<<15) | 18901, (1U<<15) | 17297, (1U<<15) | 18901, (1U<<15) | 18853, (1U<<15) | 18853, 
  (1U<<15) | 17183, (1U<<15) | 18889, (1U<<15) | 17179, (1U<<15) | 18885, (1U<<15) | 17840, (1U<<15) | 20346, (1U<<15) | 17297, (1U<<15) | 12513, 
  (1U<<15) | 11323, (1U<<15) | 14195, (1U<<15) | 9817, (1U<<15) | 18798, (1U<<15) | 12513, (1U<<15) | 18853, (1U<<15) | 12513, (1U<<15) | 18853, 
  (1U<<15) | 17297, (1U<<15) | 14195, (1U<<15) | 17874, (1U<<15) | 10171, (1U<<15) | 20326, (1U<<15) | 12513, (1U<<15) | 18853, (1U<<15) | 12513, 
  (1U<<15) | 18853, (1U<<15) | 17297, (1U<<15) | 10675, (1U<<15) | 17722, (1U<<15) | 12463, (1U<<15) | 17733, (1U<<15) | 18709, (1U<<15) | 17733, 
  (1U<<15) | 12513, 0x7f3f, (1U<<15) | 17755, 0x7f3f, (1U<<15) | 17755, 0x7f3f, (1U<<15) | 17755, 0x7f3f, 
  (1U<<15) | 17755, 0x7f3f, (1U<<15) | 17755, 0x7f3f, (1U<<15) | 17755, (1U<<15) | 17179, (1U<<15) | 18885, (1U<<15) | 12513, 
  (1U<<15) | 10675, (1U<<15) | 17722, (1U<<15) | 12530, (1U<<15) | 18863, (1U<<15) | 8987, (1U<<15) | 18774, (1U<<15) | 10171, (1U<<15) | 17826, 
  (1U<<15) | 10682, (1U<<15) | 18812, (1U<<15) | 10163, (1U<<15) | 17814, (1U<<15) | 8887, (1U<<15) | 20311, (1U<<15) | 12513, (1U<<15) | 18853, 
  (1U<<15) | 12513, (1U<<15) | 18853, (1U<<15) | 14203, (1U<<15) | 14233, (1U<<15) | 18752, (1U<<15) | 20379, (1U<<15) | 18836, (1U<<15) | 20388, 
  (1U<<15) | 11737, (1U<<15) | 21391, (1U<<15) | 17297, (1U<<15) | 2476, (1U<<15) | 2476, 0x2c2c, 0x2c2c, (1U<<15) | 19601, 
  (1U<<15) | 21788, (1U<<15) | 21788, (1U<<15) | 21788, (1U<<15) | 19601, (1U<<15) | 13311, 0x44, (1U<<15) | 13311, (1U<<15) | 13311, 
  (1U<<15) | 13403, (1U<<15) | 13405, (1U<<15) | 13403, (1U<<15) | 13403, (1U<<15) | 13405, (1U<<15) | 13403, (1U<<15) | 19601, (1U<<15) | 19601, 
  (1U<<15) | 19601, (1U<<15) | 19601, (1U<<15) | 19601, (1U<<15) | 17297, (1U<<15) | 17297, 0x7f3f, (1U<<15) | 19584, (1U<<15) | 19584, 
  (1U<<15) | 17355, (1U<<15) | 17297, (1U<<15) | 17297, 0x7f3f, (1U<<15) | 18709, (1U<<15) | 18709, (1U<<15) | 18149, 0x7a3f, 
  (1U<<15) | 13474, (1U<<15) | 13474, 0x7a3a, (1U<<15) | 13257, (1U<<15) | 13257, 0x3a7a, (1U<<15) | 18709, (1U<<15) | 18709, 
  (1U<<15) | 18709, (1U<<15) | 18709, (1U<<15) | 18709, (1U<<15) | 18709, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, 
  (1U<<15) | 13798, (1U<<15) | 19072, (1U<<15) | 19091, (1U<<15) | 19114, (1U<<15) | 13810, (1U<<15) | 13810, (1U<<15) | 10747, (1U<<15) | 13820, 
  (1U<<15) | 13820, (1U<<15) | 10728, (1U<<15) | 13832, (1U<<15) | 13832, (1U<<15) | 10705, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, 
  (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17908, (1U<<15) | 17908, (1U<<15) | 17908, (1U<<15) | 17297, (1U<<15) | 18894, 
  (1U<<15) | 18894, (1U<<15) | 17297, (1U<<15) | 18709, (1U<<15) | 18709, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, 
  0x7f3f, (1U<<15) | 17297, (1U<<15) | 17908, (1U<<15) | 17891, (1U<<15) | 17891, (1U<<15) | 17891, 0x7f3f, (1U<<15) | 17355, 
  (1U<<15) | 17355, (1U<<15) | 17297, (1U<<15) | 17896, (1U<<15) | 17896, (1U<<15) | 17896, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17896, 
  (1U<<15) | 17896, (1U<<15) | 17896, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17896, 0x7f3f, (1U<<15) | 17297, 
  (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17896, (1U<<15) | 17297, (1U<<15) | 17297, 0x7f3f, (1U<<15) | 17297, (1U<<15) | 17896, 
  (1U<<15) | 17355, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 13605, (1U<<15) | 19591, (1U<<15) | 19680, (1U<<15) | 19690, (1U<<15) | 14203, 
  (1U<<15) | 11332, (1U<<15) | 14222, (1U<<15) | 11342, (1U<<15) | 14233, (1U<<15) | 11354, (1U<<15) | 2424, (1U<<15) | 2422, (1U<<15) | 2420, 
  (1U<<15) | 2418, (1U<<15) | 2422, (1U<<15) | 2420, (1U<<15) | 2418, (1U<<15) | 2416, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x40, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x5445, 0x5445, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x5445, 0x5445, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, (1U<<15) | 19241, (1U<<15) | 19241, 
  (1U<<15) | 19241, (1U<<15) | 19241, 0x4f44, (1U<<15) | 19249, 0x4f44, (1U<<15) | 19249, 0x444, 0x44, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x40, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x4444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x44, 0x2f7, 0x2f7, 0x545, (1U<<15) | 18540, (1U<<15) | 21275, (1U<<15) | 21274, 0x5e5, 
  0x5e5, 0x5e5, (1U<<15) | 18491, 0x5e45, 0x54f4, 0x544, 0x555, 0xf1, 
  0xf1, (1U<<15) | 17032, (1U<<15) | 17288, 0x7f2f, 0x7f2f, 0x7f2f, 0x7f2f, 0x2f7f, 
  0x10, (1U<<15) | 17342, (1U<<15) | 15339, (1U<<15) | 17446, (1U<<15) | 17476, 0x4444, 0x4444, (1U<<15) | 17947, 
  (1U<<15) | 1256, (1U<<15) | 1256, (1U<<15) | 1256, 0x4, 0x7f2f, 0x44, 0x0, (1U<<15) | 17333, 
  (1U<<15) | 2807, (1U<<15) | 2807, (1U<<15) | 1262, (1U<<15) | 17286, (1U<<15) | 4437, (1U<<15) | 17344, 0x2f7f, 0x7f2f, 
  (1U<<15) | 18485, 0xf4, (1U<<15) | 12877, (1U<<15) | 19246, (1U<<15) | 19246, (1U<<15) | 14279, (1U<<15) | 14345, (1U<<15) | 14502, 
  (1U<<15) | 10788, (1U<<15) | 12862, 0x44, (1U<<15) | 18497, (1U<<15) | 18490, 0x20f4, 0x7f2f, 0x7f2f, 
  (1U<<15) | 17333, (1U<<15) | 17938, (1U<<15) | 6, (1U<<15) | 18191, (1U<<15) | 17128, (1U<<15) | 12, 0x44, 0x44, 
  (1U<<15) | 17333, (1U<<15) | 17938, (1U<<15) | 17333, 0x14, 0x11, 0x11, 0x14a, 0x4, 
  0x4, 0x1, (1U<<15) | 12440, 0x7f0f, 0x7f0f, 0x7f0f, 0x7f1f, 0x7f1f, 
  0x7f1f, 0x44, 0x55, 0x44, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x555, 0x555, 0x444, 0x545, 0x444, 0x444, 
  0x555, 0x44, 0x44, 0x444, 0x444, 0x444, 0x444, 0x445, 
  0x445, 0x444, 0x555, 0x444, 0x555, 0x444, 0x555, 0x444, 
  0x555, 0x44, 0x55, 0x44, 0x44, 0x55, 0x444, 0x444, 
  0x555, 0x54, 0x54, 0x44, 0x44, 0x44, 0x44, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x555, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x44, 0x44, 0x44, 0x45, 0x44, 0x444, 0x444, 0x55, 
  0x45, 0x44, 0x55, 0x55, 0x55, 0x55, 0x555, 0x555, 
  0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 
  0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 
  0x555, 0x555, 0x554, 0x554, 0x554, 0x554, 0x554, 0x554, 
  0x554, 0x554, 0x55, 0x555, 0x555, 0x555, 0x555, 0x555, 
  0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 
  0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x5555, 0x555, 
  0x5555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 
  0x555, 0x444, 0x555, 0x44, 0x44, 0x444, 0x555, 0x445, 
  0x445, 0x544, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x445, 0x445, 
  0x444, 0x444, 0x444, 0x444, 0x555, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x454, 0x554, 0x454, 
  0x554, 0x454, 0x454, 0x454, 0x454, 0x454, 0x454, 0x454, 
  0x454, 0x4555, 0x4555, 0x4555, 0x4555, 0x4555, 0x4555, 0x4555, 
  0x4555, 0x554, 0x554, 0x444, 0x455, 0x455, 0x455, 0x44, 
  0x444, 0x444, 0x44, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x554, 0x444, 0x444, 0x444, 0x444, 0x554, 0x444, 0x444, 
  0x554, 0x444, 0x444, 0x45, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x44, 0x444, 0x444, 0x44, 0x44, 0x44, 0x444, 0x5545, 
  0x444, 0x4444, 0x4444, 0x4444, 0x4444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x58, 0x57, 0x85, 0x85, 
  0x87, 0x85, 0x85, 0x84, 0x84, 0x84, 0x84, 0x75, 
  0x75, 0x78, 0x75, 0x75, 0x74, 0x74, 0x74, 0x74, 
  0x58, 0x57, 0x48, 0x47, 0x48, 0x47, 0x888, 0x484, 
  0x884, 0x884, 0x884, 0x884, 0x48, 0x48, 0x888, 0x888, 
  0x888, (1U<<15) | 18219, (1U<<15) | 18219, 0x888, 0x888, 0x777, 0x474, 0x774, 
  0x774, 0x774, 0x774, 0x777, 0x777, 0x77, 0x7777, 0x7777, 
  (1U<<15) | 12222, 0x7777, 0x7777, 0x47, 0x47, 0x777, 0x777, 0x777, 
  0x777, (1U<<15) | 14069, (1U<<15) | 19274, (1U<<15) | 19303, (1U<<15) | 14076, (1U<<15) | 19283, (1U<<15) | 19311, (1U<<15) | 14069, 
  (1U<<15) | 19274, (1U<<15) | 19303, (1U<<15) | 14069, (1U<<15) | 19274, (1U<<15) | 19303, (1U<<15) | 14069, (1U<<15) | 19274, (1U<<15) | 19303, 
  (1U<<15) | 14069, (1U<<15) | 19274, (1U<<15) | 19303, 0xe4, 0xe5, 0x4444, 0x4444, 0x4455, 
  0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 0x445, 0x445, 0x444, 
  0x444, 0x444, 0x444, 0x445, 0x445, 0x445, 0x445, 0x4455, 
  0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 0x444, 0x445, 0x4455, 
  0x4455, 0x445, 0x444, 0x444, 0x444, 0x444, 0x4444, 0x4444, 
  0x4444, 0x5555, 0x5555, 0x5555, 0x5555, 0x5555, 0x5555, 0x5555, 
  0x5555, 0x5555, 0x5555, 0x5555, 0x5555, 0x5555, 0x5555, 0x5555, 
  0x5555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 
  0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 0x555, 
  0x555, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x4455, 0x4455, 0x4455, 
  0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 0x445, 0x445, 0x445, 
  0x445, 0x445, 0x445, 0x445, 0x445, 0x4455, 0x4455, 0x4455, 
  0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 0x445, 0x445, 0x445, 
  0x445, 0x445, 0x445, 0x445, 0x445, 0x444, 0x444, 0x444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 0x444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x444, 0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 
  0x4455, 0x445, 0x445, 0x445, 0x445, 0x445, 0x445, 0x445, 
  0x445, 0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 0x4455, 
  0x4455, 0x444, 0x4444, 0x4444, 0x4444, 0x555, 0x555, 0x5555, 
  0x5555, 0x555, 0x555, 0x555, 0x555, 0x5555, 0x5555, 0x554, 
  0x554, 0x555, 0x555, 0x4455, 0x5555, 0x5555, 0x5555, 0x4455, 
  0x4455, 0x4455, 0x4455, 0x555, 0x555, 0x445, 0x444, 0x445, 
  0x444, 0x445, 0x445, 0x554, 0x554, 0x5555, 0x5555, 0x5555, 
  0x5555, 0x555, 0x555, 0x555, 0x555, 0x4555, 0x455, 0x454, 
  0x5555, 0x555, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x454, 
  0x454, 0x454, 0x454, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x445, 0x4455, 
  0x445, 0x4455, 0x5555, 0x5555, 0x555, 0x555, 0x5555, 0x5555, 
  0x555, 0x555, 0x4444, 0x4444, 0x4444, 0x5555, 0x5555, 0x555, 
  0x4455, 0x4455, 0x445, 0x445, 0x5555, 0x5555, 0x555, 0x555, 
  0x555, 0x555, 0x555, 0x5555, 0x555, 0x5555, 0x555, 0x5555, 
  0x555, 0x5555, 0x555, 0x5555, 0x554, 0x554, 0x554, 0x554, 
  0x554, 0x554, 0x554, 0x554, 0x4444, 0x455, 0x4555, 0x4555, 
  0x4555, 0x4555, 0x4555, 0x444, 0x4444, 0x4444, 0x4444, 0x4444, 
  0x444, 0x4444, 0x455, 0x455, 0x455, 0x4555, 0x4555, 0x4555, 
  0x4555, 0x4555, 0x444, 0x4444, 0x4444, 0x4444, 0x4444, 0x444, 
  0x455, 0x455, 0x455, 0x4555, 0x4555, 0x4555, 0x4555, 0x455, 
  0x455, 0x444, 0x4444, 0x4444, 0x4444, 0x4444, 0x444, 0x444, 
  0x454, 0x455, 0x455, 0x455, 0x4555, 0x4555, 0x4555, 0x4555, 
  0x4555, 0x444, 0x4444, 0x4444, 0x4444, 0x4444, 0x444, 0x454, 
  0x455, 0x455, 0x44, 0x55, 0x44, 0x54, 0x44, 0x54, 
  0x44, 0x44, 0x54, 0x444, 0x444, 0x44, 0x54, 0x44, 
  0x54, 0x55, 0x4444, 0x544, 0x4455, 0x555, (1U<<15) | 8570, 0x5444, 
  (1U<<15) | 10313, 0x5555, 0x55, 0x555, 0x455, 0x4555, 0x4555, 0x4555, 
  0x4555, 0x4555, 0x444, 0x4444, 0x4444, 0x4444, 0x4444, 0x455, 
  0x455, 0x455, 0x4555, 0x4555, 0x4555, 0x4555, 0x4555, 0x444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x455, 0x455, 0x455, 
  0x4555, 0x4555, 0x4555, 0x4555, 0x4555, 0x444, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x455, 0x455, 0x444, 0x445, 0x554, 0x444, 
  0x444, 0x555, 0x555, 0x555, 0x555, 0x44ee, (1U<<15) | 19254, (1U<<15) | 19277, 
  0x45ee, (1U<<15) | 19324, (1U<<15) | 19331, 0x44ee, (1U<<15) | 19254, (1U<<15) | 19277, 0x44ee, (1U<<15) | 19254, 
  (1U<<15) | 19277, 0x44ee, (1U<<15) | 19254, (1U<<15) | 19277, 0x4e4, 0x44, 0x44, (1U<<15) | 8570, 
  (1U<<15) | 8570, (1U<<15) | 8570, (1U<<15) | 8570, 0x444, 0x444, 0x444, 0x444, 0x4555, 
  0x4555, 0x455, 0x455, 0x4555, 0x54, 0x54, 0x54, 0x55, 
  0x54, 0x55, 0x54, 0x55, 0x54, 0x55, 0x44, 0x45, 
  0x4555, 0x4555, 0x45, 0x45, 0x54, 0x555, 0x54, 0x555, 
  0x45, 0x45, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x444, 
  0x454, 0x54, 0x4444, 0x544, 0x4455, 0x555, 0x444, 0x444, 
  0x444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x444, 0x5e4, 
  0x4444, 0x4444, 0x4444, 0x4455, (1U<<15) | 10313, 0x555, 0x555, 0x555, 
  0x555, 0x555, 0x555, 0x454, 0x454, 0x54, 0x455, 0x455, 
  0x4555, 0x4555, 0x4555, 0x4555, 0x4555, 0x444, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x4444, 0x45, 0x555, 0x555, 0x44c4, 0x44d4, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, 0x4d4c, (1U<<15) | 14267, 0x4d4c, (1U<<15) | 14267, 
  0x44c, 0x44d, 0x44c, 0x44d, 0x44c, 0x44d, (1U<<15) | 1278, (1U<<15) | 1401, 
  (1U<<15) | 1278, (1U<<15) | 1401, (1U<<15) | 1280, (1U<<15) | 1403, (1U<<15) | 1278, (1U<<15) | 1401, (1U<<15) | 1278, (1U<<15) | 1401, 
  (1U<<15) | 4281, (1U<<15) | 4362, (1U<<15) | 4281, (1U<<15) | 4362, (1U<<15) | 18709, (1U<<15) | 18709, (1U<<15) | 1278, (1U<<15) | 1401, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 1278, (1U<<15) | 1401, (1U<<15) | 1278, (1U<<15) | 1401, (1U<<15) | 11105, (1U<<15) | 11378, 
  (1U<<15) | 11103, (1U<<15) | 11376, (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 11103, (1U<<15) | 11376, (1U<<15) | 13996, (1U<<15) | 14002, 
  (1U<<15) | 14008, (1U<<15) | 14017, (1U<<15) | 14008, (1U<<15) | 14017, (1U<<15) | 13996, (1U<<15) | 14002, (1U<<15) | 14008, (1U<<15) | 14017, 
  (1U<<15) | 14008, (1U<<15) | 14017, (1U<<15) | 13996, (1U<<15) | 14002, (1U<<15) | 14008, (1U<<15) | 14017, (1U<<15) | 14008, (1U<<15) | 14017, 
  (1U<<15) | 13996, (1U<<15) | 14002, (1U<<15) | 14008, (1U<<15) | 14017, (1U<<15) | 14008, (1U<<15) | 14017, (1U<<15) | 13758, (1U<<15) | 13943, 
  (1U<<15) | 13765, (1U<<15) | 13950, (1U<<15) | 13765, (1U<<15) | 13950, (1U<<15) | 13758, (1U<<15) | 13943, (1U<<15) | 13765, (1U<<15) | 13950, 
  (1U<<15) | 13765, (1U<<15) | 13950, (1U<<15) | 13758, (1U<<15) | 13943, (1U<<15) | 13765, (1U<<15) | 13950, (1U<<15) | 13765, (1U<<15) | 13950, 
  (1U<<15) | 13779, (1U<<15) | 13989, (1U<<15) | 13758, (1U<<15) | 13943, (1U<<15) | 13765, (1U<<15) | 13950, (1U<<15) | 13765, (1U<<15) | 13950, 
  (1U<<15) | 13779, (1U<<15) | 13989, (1U<<15) | 13758, (1U<<15) | 13943, (1U<<15) | 13765, (1U<<15) | 13950, (1U<<15) | 13765, (1U<<15) | 13950, 
  (1U<<15) | 13779, (1U<<15) | 13989, (1U<<15) | 13758, (1U<<15) | 13943, (1U<<15) | 13765, (1U<<15) | 13950, (1U<<15) | 13765, (1U<<15) | 13950, 
  (1U<<15) | 13779, (1U<<15) | 13989, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 13665, (1U<<15) | 13889, 
  (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 1291, (1U<<15) | 1408, 
  (1U<<15) | 13663, (1U<<15) | 13887, (1U<<15) | 1304, (1U<<15) | 1421, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13665, (1U<<15) | 13889, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, 
  (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 4279, (1U<<15) | 4360, (1U<<15) | 4277, (1U<<15) | 4358, (1U<<15) | 4279, (1U<<15) | 4360, 
  (1U<<15) | 4277, (1U<<15) | 4358, (1U<<15) | 13620, (1U<<15) | 13846, (1U<<15) | 13620, (1U<<15) | 13846, (1U<<15) | 11027, (1U<<15) | 11089, 
  (1U<<15) | 11025, (1U<<15) | 11087, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13717, (1U<<15) | 13934, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 13744, (1U<<15) | 13975, 
  (1U<<15) | 13744, (1U<<15) | 13975, (1U<<15) | 13744, (1U<<15) | 13975, (1U<<15) | 13744, (1U<<15) | 13975, (1U<<15) | 11037, (1U<<15) | 11107, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4c, 0x4d4d, 
  0x4c4c, 0x4d4d, 0x4d4d, (1U<<15) | 14274, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  (1U<<15) | 13719, (1U<<15) | 13936, 0x4d4c, (1U<<15) | 14267, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4d4c, (1U<<15) | 14267, 
  0x4c4d, (1U<<15) | 13960, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4c, 0x4d4d, 
  0x4c4d, (1U<<15) | 13960, 0x4c4d, (1U<<15) | 13960, 0x4c4c, 0x4d4d, (1U<<15) | 13710, (1U<<15) | 13927, 
  0x4c4d, (1U<<15) | 13960, 0x4c4c, 0x4d4d, 0x4c4d, (1U<<15) | 13960, (1U<<15) | 13710, (1U<<15) | 13927, 
  0x4c4c, 0x4d4d, (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4d, (1U<<15) | 13960, 0x4c4d, (1U<<15) | 13960, 
  (1U<<15) | 13710, (1U<<15) | 13927, 0x4c, 0x4d, 0x4d, (1U<<15) | 14257, 0x4c4c, 0x4d4d, 
  (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4c, 0x4d4d, (1U<<15) | 11064, (1U<<15) | 11115, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11107, (1U<<15) | 11380, (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11107, (1U<<15) | 11380, (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 11097, (1U<<15) | 11370, (1U<<15) | 11095, (1U<<15) | 11368, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11097, (1U<<15) | 11370, (1U<<15) | 11095, (1U<<15) | 11368, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13708, (1U<<15) | 13925, 
  (1U<<15) | 11107, (1U<<15) | 11380, (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, (1U<<15) | 3564, (1U<<15) | 13871, 
  (1U<<15) | 13636, (1U<<15) | 13871, (1U<<15) | 13627, (1U<<15) | 13862, (1U<<15) | 13871, (1U<<15) | 14260, (1U<<15) | 13853, (1U<<15) | 14251, 
  (1U<<15) | 13636, (1U<<15) | 13871, (1U<<15) | 13627, (1U<<15) | 13862, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13656, (1U<<15) | 13880, (1U<<15) | 13654, (1U<<15) | 13878, 
  (1U<<15) | 13654, (1U<<15) | 13878, (1U<<15) | 13654, (1U<<15) | 13878, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 14667, (1U<<15) | 14683, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11052, (1U<<15) | 11103, (1U<<15) | 11052, (1U<<15) | 11103, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11064, (1U<<15) | 11115, (1U<<15) | 11064, (1U<<15) | 11115, (1U<<15) | 11062, (1U<<15) | 11113, (1U<<15) | 11062, (1U<<15) | 11113, 
  (1U<<15) | 11064, (1U<<15) | 11115, (1U<<15) | 13779, (1U<<15) | 13989, (1U<<15) | 13779, (1U<<15) | 13989, (1U<<15) | 13779, (1U<<15) | 13989, 
  (1U<<15) | 13779, (1U<<15) | 13989, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 11107, (1U<<15) | 11380, (1U<<15) | 11105, (1U<<15) | 11378, 
  (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 11107, (1U<<15) | 11380, (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 11107, (1U<<15) | 11380, (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 14665, (1U<<15) | 14681, (1U<<15) | 11107, (1U<<15) | 11380, 
  (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 14665, (1U<<15) | 14681, (1U<<15) | 14665, (1U<<15) | 14681, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 11037, (1U<<15) | 11107, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, 
  (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 11074, (1U<<15) | 11125, 
  (1U<<15) | 11072, (1U<<15) | 11123, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 11074, (1U<<15) | 11125, 
  (1U<<15) | 11072, (1U<<15) | 11123, (1U<<15) | 11072, (1U<<15) | 11123, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11037, (1U<<15) | 11107, 
  (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 11037, (1U<<15) | 11107, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13708, (1U<<15) | 13925, 
  (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 17353, (1U<<15) | 17353, (1U<<15) | 11074, (1U<<15) | 11125, (1U<<15) | 11072, (1U<<15) | 11123, 
  (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, (1U<<15) | 11074, (1U<<15) | 11125, (1U<<15) | 11072, (1U<<15) | 11123, 
  (1U<<15) | 11037, (1U<<15) | 11107, (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13717, (1U<<15) | 13934, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 17353, (1U<<15) | 17353, (1U<<15) | 17353, (1U<<15) | 17353, (1U<<15) | 13665, (1U<<15) | 13889, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 
  0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, 0x4c4c, 0x4d4d, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  0x4c4c, 0x4d4d, (1U<<15) | 1299, (1U<<15) | 1416, (1U<<15) | 1299, (1U<<15) | 1416, (1U<<15) | 1299, (1U<<15) | 1416, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 14675, (1U<<15) | 14691, (1U<<15) | 14673, (1U<<15) | 14689, (1U<<15) | 11037, (1U<<15) | 11107, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 10183, (1U<<15) | 10210, (1U<<15) | 10181, (1U<<15) | 10208, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 11037, (1U<<15) | 11107, 
  (1U<<15) | 11054, (1U<<15) | 11105, (1U<<15) | 14675, (1U<<15) | 14691, (1U<<15) | 14673, (1U<<15) | 14689, (1U<<15) | 10183, (1U<<15) | 10210, 
  (1U<<15) | 10181, (1U<<15) | 10208, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13708, (1U<<15) | 13925, (1U<<15) | 11037, (1U<<15) | 11107, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 10183, (1U<<15) | 10210, 
  (1U<<15) | 10181, (1U<<15) | 10208, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4d, (1U<<15) | 13960, (1U<<15) | 13690, (1U<<15) | 13907, (1U<<15) | 13690, (1U<<15) | 13907, 
  (1U<<15) | 13698, (1U<<15) | 13915, (1U<<15) | 13726, (1U<<15) | 13957, (1U<<15) | 13726, (1U<<15) | 13957, (1U<<15) | 13734, (1U<<15) | 13965, 
  (1U<<15) | 13690, (1U<<15) | 13907, (1U<<15) | 13690, (1U<<15) | 13907, (1U<<15) | 13698, (1U<<15) | 13915, 0x4c4d, (1U<<15) | 13960, 
  (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4c, 0x4d4d, (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4c, 0x4d4d, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 11064, (1U<<15) | 11115, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 1291, (1U<<15) | 1408, (1U<<15) | 13663, (1U<<15) | 13887, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13719, (1U<<15) | 13936, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 13719, (1U<<15) | 13936, 
  (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13665, (1U<<15) | 13889, (1U<<15) | 13710, (1U<<15) | 13927, (1U<<15) | 13927, (1U<<15) | 14272, 
  (1U<<15) | 13674, (1U<<15) | 13898, (1U<<15) | 11107, (1U<<15) | 11380, (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 11107, (1U<<15) | 11380, 
  (1U<<15) | 11105, (1U<<15) | 11378, (1U<<15) | 11107, (1U<<15) | 11380, (1U<<15) | 11105, (1U<<15) | 11378, 0x4c4d, (1U<<15) | 13960, 
  0x4c4d, (1U<<15) | 13960, (1U<<15) | 13751, (1U<<15) | 13982, (1U<<15) | 13751, (1U<<15) | 13982, 0x4c4d, (1U<<15) | 13960, 
  0x4c4d, (1U<<15) | 13960, (1U<<15) | 13710, (1U<<15) | 13927, 0x4c4d, (1U<<15) | 13960, 0x4c4d, (1U<<15) | 13960, 
  0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0x4e0, 0x5e0, 0xee0, 
  0x4, 0x4, 0xe0, 0xe0, 0x4, (1U<<15) | 11227, (1U<<15) | 11227, (1U<<15) | 11227, 
  (1U<<15) | 11227, (1U<<15) | 11227, (1U<<15) | 11227, (1U<<15) | 10196, (1U<<15) | 10360, (1U<<15) | 10196, (1U<<15) | 10196, (1U<<15) | 10196, 
  0x4e0, 0xe0, 0x4ee0, 0x44e0, 0x550, 0x550, 0x40, 0x5550, 
  0x4440, 0x44, 0x444, 0x454, 0x444, 0x444, 0x444, 0x454, 
  0x444, 0x444, 0x45, 0x44, 0x455, 0x444, 0x4555, 0x4444, 
  0x40, 0x88, 0x77, 0x88, 0x77, 0x40, 0x44, 0x45, 
  0x44, 0x44, 0x440, 0x450, 0x440, 0x440, 0x595a, (1U<<15) | 18366, 
  0x7a7b, (1U<<15) | 14971, (1U<<15) | 18357, (1U<<15) | 16796, 0x5a59, (1U<<15) | 18440, 0x7b7a, 0x5a59, 
  (1U<<15) | 18440, 0x7b7a, (1U<<15) | 15008, (1U<<15) | 18364, (1U<<15) | 16810, (1U<<15) | 15008, (1U<<15) | 18364, (1U<<15) | 16810, 
  0x2d5a, 0x3c5a, 0x4b5a, 0x2d5a, 0x3c5a, 0x4b5a, 0x2d3c, 0x2d3c, 
  0x2d4b, 0x3c4b, 0x2d4b, 0x3c4b, 0x2d4, 0x5a4, 0x3c4, 0x2d4, 
  0x4b4, 0x2d4, 0x5a4, 0x3c4, 0x2d4, 0x4b4, (1U<<15) | 2775, (1U<<15) | 2775, 
  (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 15162, 
  (1U<<15) | 3297, (1U<<15) | 15162, (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 5263, 
  (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 2784, (1U<<15) | 2784, 
  (1U<<15) | 2784, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 13559, 
  (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 2784, (1U<<15) | 2784, (1U<<15) | 2784, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 15162, 
  (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 2775, (1U<<15) | 5263, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 2775, 
  (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 2775, 
  (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 15162, 
  (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 2775, (1U<<15) | 15162, 
  (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 2773, (1U<<15) | 5261, 
  (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, 
  (1U<<15) | 5263, (1U<<15) | 5263, 0x2d2d, 0x5a5a, 0x3c3c, 0x4b4b, 0x2d2d, 0x5a5a, 
  0x3c3c, 0x4b4b, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, 
  (1U<<15) | 13550, (1U<<15) | 13550, 0x4b5a, 0x4b5a, 0x2d3c, 0x2d3c, 0x5a5a, 0x5a5a, 
  0x3c4b, 0x3c4b, 0x5a5a, 0x5a5a, (1U<<15) | 5261, (1U<<15) | 12012, (1U<<15) | 5494, (1U<<15) | 10992, 
  (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18423, 0x7b4b, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, 
  (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, 
  (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, 
  (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, 
  (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, 
  (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, (1U<<15) | 18452, (1U<<15) | 16927, 
  (1U<<15) | 16920, (1U<<15) | 18459, 0x7b8a, 0x3c7b, 0x7b8a, 0x3c7b, (1U<<15) | 18468, (1U<<15) | 16936, 
  0x5a8a, 0x5a8a, (1U<<15) | 15169, 0x4b7b, 0x4b7b, 0x4b8a, 0x4b8a, (1U<<15) | 18447, 
  0x7b7b, (1U<<15) | 18466, (1U<<15) | 16934, (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18468, 
  (1U<<15) | 16936, (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18466, (1U<<15) | 16934, (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18466, 
  (1U<<15) | 16934, (1U<<15) | 18466, (1U<<15) | 16934, (1U<<15) | 18447, 0x7b7b, (1U<<15) | 18447, 0x7b7b, (1U<<15) | 18447, 
  0x7b7b, (1U<<15) | 18447, 0x7b7b, (1U<<15) | 18447, 0x7b7b, (1U<<15) | 18447, 0x7b7b, (1U<<15) | 18447, 
  0x7b7b, (1U<<15) | 18447, 0x7b7b, (1U<<15) | 18447, 0x7b7b, (1U<<15) | 2773, (1U<<15) | 3295, (1U<<15) | 5261, 
  (1U<<15) | 5494, (1U<<15) | 18447, 0x7b7b, (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18423, (1U<<15) | 18423, (1U<<15) | 18445, 
  0x7b4b, 0x7b4b, 0x7b5a, 0x7b5a, (1U<<15) | 18423, (1U<<15) | 18445, 0x7b4b, 0x7b5a, 
  0x7b5a, (1U<<15) | 18423, (1U<<15) | 18445, 0x7b4b, 0x7b5a, 0x7b5a, (1U<<15) | 18423, (1U<<15) | 18445, 
  0x7b4b, 0x7b5a, 0x7b5a, (1U<<15) | 18423, (1U<<15) | 18423, (1U<<15) | 18445, 0x7b4b, 0x7b4b, 
  0x7b5a, 0x7b5a, (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 2784, (1U<<15) | 2784, (1U<<15) | 15162, (1U<<15) | 15162, 
  (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 2784, (1U<<15) | 2784, (1U<<15) | 15162, (1U<<15) | 15162, 
  (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 15162, 
  (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 11850, (1U<<15) | 10069, (1U<<15) | 12012, (1U<<15) | 10992, 0x4e2d, 0x45a, 
  0x4e2d, 0x4e5a, 0x4e3c, 0x4e4b, 0x5e2d, (1U<<15) | 2773, (1U<<15) | 15160, (1U<<15) | 3295, 
  (1U<<15) | 13548, (1U<<15) | 13557, (1U<<15) | 13557, (1U<<15) | 13557, (1U<<15) | 2782, (1U<<15) | 2782, (1U<<15) | 2782, (1U<<15) | 15160, 
  (1U<<15) | 15160, (1U<<15) | 15160, (1U<<15) | 3304, (1U<<15) | 3304, (1U<<15) | 3304, (1U<<15) | 13557, (1U<<15) | 13557, (1U<<15) | 13557, 
  (1U<<15) | 2782, (1U<<15) | 2782, (1U<<15) | 2782, (1U<<15) | 15160, (1U<<15) | 15160, (1U<<15) | 15160, (1U<<15) | 3304, (1U<<15) | 3304, 
  (1U<<15) | 3304, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, 
  (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 5496, (1U<<15) | 10975, 
  (1U<<15) | 10975, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, 
  (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 5496, (1U<<15) | 10975, 
  (1U<<15) | 10975, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, 
  (1U<<15) | 13550, 0x2d2d, 0x2d2d, 0x5a5a, 0x3c3c, 0x4b4b, 0x2d2d, (1U<<15) | 2773, 
  (1U<<15) | 15160, (1U<<15) | 3295, (1U<<15) | 13548, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, 
  (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 13559, 
  (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 2784, (1U<<15) | 2784, (1U<<15) | 2784, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 15162, 
  (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 2784, (1U<<15) | 2784, 
  (1U<<15) | 2784, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 3306, 0x2d2d, 
  0x5a5a, 0x3c3c, 0x4b4b, (1U<<15) | 2775, (1U<<15) | 5263, (1U<<15) | 2775, (1U<<15) | 5263, (1U<<15) | 2775, 
  (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, 
  0x2d2d, 0x5a5a, 0x3c3c, 0x4b4b, (1U<<15) | 13550, (1U<<15) | 11995, (1U<<15) | 5261, (1U<<15) | 10992, 
  (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, 
  (1U<<15) | 11995, (1U<<15) | 12796, (1U<<15) | 10975, (1U<<15) | 12399, 0x45a5, 0x45a5, 0x44b4, 0x44b4, 
  (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, 0x42d, 0x55a, 0x43c, 0x44b, 
  0x42d, 0x45a, 0x43c, 0x44b, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, 
  0x2d2d, 0x5a5a, 0x3c3c, 0x2d2d, 0x4b4b, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, 
  (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 15162, 
  (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 5263, (1U<<15) | 11995, 
  (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 10975, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, 
  (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 2773, (1U<<15) | 15160, (1U<<15) | 3295, 
  (1U<<15) | 13548, (1U<<15) | 5263, (1U<<15) | 12012, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, 
  (1U<<15) | 13550, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, 
  (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 5496, (1U<<15) | 10975, 
  (1U<<15) | 10975, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, 
  (1U<<15) | 10975, (1U<<15) | 11000, (1U<<15) | 11000, (1U<<15) | 5277, (1U<<15) | 5277, (1U<<15) | 5502, (1U<<15) | 5502, (1U<<15) | 2775, 
  (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 13550, (1U<<15) | 5263, 
  (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 10975, (1U<<15) | 2775, 
  (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 3288, 
  (1U<<15) | 13541, (1U<<15) | 15153, (1U<<15) | 5261, (1U<<15) | 12012, (1U<<15) | 5494, (1U<<15) | 10992, (1U<<15) | 2775, (1U<<15) | 15162, 
  (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 3288, (1U<<15) | 13541, 
  (1U<<15) | 15153, (1U<<15) | 5261, (1U<<15) | 12012, (1U<<15) | 5494, (1U<<15) | 10992, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, 
  (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 3288, (1U<<15) | 13541, (1U<<15) | 15153, 
  (1U<<15) | 5261, (1U<<15) | 12012, (1U<<15) | 5494, (1U<<15) | 10992, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 13550, 
  (1U<<15) | 5263, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 3288, (1U<<15) | 13541, (1U<<15) | 15153, (1U<<15) | 5261, 
  (1U<<15) | 12012, (1U<<15) | 5494, (1U<<15) | 10992, (1U<<15) | 3288, (1U<<15) | 3288, (1U<<15) | 13541, (1U<<15) | 13541, (1U<<15) | 15153, 
  (1U<<15) | 15153, (1U<<15) | 5261, (1U<<15) | 5261, (1U<<15) | 12012, (1U<<15) | 12012, (1U<<15) | 5494, (1U<<15) | 5494, (1U<<15) | 10992, 
  (1U<<15) | 10992, (1U<<15) | 3288, (1U<<15) | 3288, (1U<<15) | 13541, (1U<<15) | 13541, (1U<<15) | 15153, (1U<<15) | 15153, (1U<<15) | 5261, 
  (1U<<15) | 5261, (1U<<15) | 12012, (1U<<15) | 12012, (1U<<15) | 5494, (1U<<15) | 5494, (1U<<15) | 10992, (1U<<15) | 10992, (1U<<15) | 3288, 
  (1U<<15) | 3288, (1U<<15) | 13541, (1U<<15) | 13541, (1U<<15) | 15153, (1U<<15) | 15153, (1U<<15) | 5261, (1U<<15) | 5261, (1U<<15) | 12012, 
  (1U<<15) | 12012, (1U<<15) | 5494, (1U<<15) | 5494, (1U<<15) | 10992, (1U<<15) | 10992, (1U<<15) | 3288, (1U<<15) | 3288, (1U<<15) | 13541, 
  (1U<<15) | 13541, (1U<<15) | 15153, (1U<<15) | 15153, (1U<<15) | 5261, (1U<<15) | 5261, (1U<<15) | 12012, (1U<<15) | 12012, (1U<<15) | 5494, 
  (1U<<15) | 5494, (1U<<15) | 10992, (1U<<15) | 10992, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 15162, (1U<<15) | 3297, 
  (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 13550, (1U<<15) | 14032, (1U<<15) | 11178, (1U<<15) | 11220, (1U<<15) | 11192, (1U<<15) | 11206, 
  (1U<<15) | 15275, (1U<<15) | 2775, (1U<<15) | 15162, (1U<<15) | 3297, (1U<<15) | 15162, (1U<<15) | 13550, (1U<<15) | 5263, (1U<<15) | 11995, 
  (1U<<15) | 5496, (1U<<15) | 10975, (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 2784, (1U<<15) | 2784, (1U<<15) | 15162, (1U<<15) | 15162, 
  (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 13559, (1U<<15) | 13559, (1U<<15) | 2784, (1U<<15) | 2784, (1U<<15) | 15162, (1U<<15) | 15162, 
  (1U<<15) | 3306, (1U<<15) | 3306, (1U<<15) | 2775, (1U<<15) | 5263, 0x555, 0x550, 0x2c4, 0x594, 
  0x3b4, 0x2c4, 0x4a4, 0x2c4, 0x594, 0x3b4, 0x2c4, 0x4a4, 
  (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 14964, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 13414, 
  (1U<<15) | 2516, (1U<<15) | 2516, (1U<<15) | 2516, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3244, (1U<<15) | 3244, 
  (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 2516, (1U<<15) | 2516, (1U<<15) | 14964, 
  (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3244, (1U<<15) | 3244, (1U<<15) | 3244, (1U<<15) | 2476, (1U<<15) | 4994, (1U<<15) | 2476, 
  (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 2474, (1U<<15) | 4992, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, 
  (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 4994, (1U<<15) | 4994, 0x2c2c, 0x5959, 0x3b3b, 0x4a4a, 
  0x2c2c, 0x5959, 0x3b3b, 0x4a4a, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, 
  (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, 0x4a59, 0x4a59, 0x2c3b, 0x2c3b, 
  0x5959, 0x5959, 0x3b4a, 0x3b4a, 0x5959, 0x5959, (1U<<15) | 4992, (1U<<15) | 11954, 
  (1U<<15) | 5442, (1U<<15) | 10863, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18291, 0x7a4a, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 16756, (1U<<15) | 18341, 0x7a89, 0x3b7a, 0x7a89, 0x3b7a, 
  (1U<<15) | 18350, (1U<<15) | 16789, 0x5989, 0x5989, (1U<<15) | 14978, 0x4a7a, 0x4a7a, 0x4a89, 
  0x4a89, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18348, (1U<<15) | 16787, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18350, 
  (1U<<15) | 16789, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18348, (1U<<15) | 16787, (1U<<15) | 18350, 
  (1U<<15) | 16789, (1U<<15) | 18348, (1U<<15) | 16787, (1U<<15) | 18348, (1U<<15) | 16787, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18312, 
  0x7a7a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18312, 
  0x7a7a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 2474, 
  (1U<<15) | 3206, (1U<<15) | 4992, (1U<<15) | 5442, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18291, 
  (1U<<15) | 18291, (1U<<15) | 18327, 0x7a4a, 0x7a4a, 0x7a59, 0x7a59, (1U<<15) | 18291, (1U<<15) | 18327, 
  0x7a4a, 0x7a59, 0x7a59, (1U<<15) | 18291, (1U<<15) | 18327, 0x7a4a, 0x7a59, 0x7a59, 
  (1U<<15) | 18291, (1U<<15) | 18327, 0x7a4a, 0x7a59, 0x7a59, (1U<<15) | 18291, (1U<<15) | 18291, (1U<<15) | 18327, 
  0x7a4a, 0x7a4a, 0x7a59, 0x7a59, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 2516, 
  (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3244, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 2516, 
  (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3244, (1U<<15) | 3244, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 8406, (1U<<15) | 11843, (1U<<15) | 8413, (1U<<15) | 9950, 
  0x4e2c, 0x459, 0x4e2c, 0x4e59, 0x4e3b, 0x4e4a, 0x5e2c, (1U<<15) | 2474, 
  (1U<<15) | 14962, (1U<<15) | 3206, (1U<<15) | 13403, (1U<<15) | 13412, (1U<<15) | 13412, (1U<<15) | 13412, (1U<<15) | 2514, (1U<<15) | 2514, 
  (1U<<15) | 2514, (1U<<15) | 14962, (1U<<15) | 14962, (1U<<15) | 14962, (1U<<15) | 3242, (1U<<15) | 3242, (1U<<15) | 3242, (1U<<15) | 13412, 
  (1U<<15) | 13412, (1U<<15) | 13412, (1U<<15) | 2514, (1U<<15) | 2514, (1U<<15) | 2514, (1U<<15) | 14962, (1U<<15) | 14962, (1U<<15) | 14962, 
  (1U<<15) | 3242, (1U<<15) | 3242, (1U<<15) | 3242, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 10822, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 10822, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, 0x2c2c, 0x2c2c, 0x5959, 0x3b3b, 0x4a4a, 
  0x2c2c, (1U<<15) | 2474, (1U<<15) | 14962, (1U<<15) | 3206, (1U<<15) | 13403, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 2516, (1U<<15) | 2516, (1U<<15) | 14964, 
  (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3244, (1U<<15) | 3244, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 13414, 
  (1U<<15) | 2516, (1U<<15) | 2516, (1U<<15) | 2516, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3244, (1U<<15) | 3244, 
  (1U<<15) | 3244, 0x2c2c, 0x5959, 0x3b3b, 0x4a4a, (1U<<15) | 2476, (1U<<15) | 4994, (1U<<15) | 2476, 
  (1U<<15) | 4994, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, 
  (1U<<15) | 3208, (1U<<15) | 13405, 0x2c2c, 0x5959, 0x3b3b, 0x4a4a, (1U<<15) | 10863, (1U<<15) | 2476, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 0x42c4, 
  0x42c4, 0x4595, 0x4595, 0x43b4, 0x43b4, 0x44a4, 0x44a4, 0x42c, 
  0x559, 0x43b, 0x44a, 0x42c, 0x459, 0x43b, 0x44a, (1U<<15) | 4994, 
  (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 2476, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 2476, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 4994, 
  (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 10822, (1U<<15) | 2476, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 2474, 
  (1U<<15) | 14962, (1U<<15) | 3206, (1U<<15) | 13403, (1U<<15) | 4994, (1U<<15) | 11954, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 2476, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 10822, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, 
  (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 10904, (1U<<15) | 10904, (1U<<15) | 5050, (1U<<15) | 5050, (1U<<15) | 5482, 
  (1U<<15) | 5482, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 10822, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 10822, (1U<<15) | 3199, (1U<<15) | 13376, (1U<<15) | 14927, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 10863, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 3199, (1U<<15) | 13376, (1U<<15) | 14927, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 10863, (1U<<15) | 2476, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 3199, 
  (1U<<15) | 13376, (1U<<15) | 14927, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 10863, (1U<<15) | 2476, (1U<<15) | 14964, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 3199, (1U<<15) | 13376, 
  (1U<<15) | 14927, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 10863, (1U<<15) | 3199, (1U<<15) | 3199, (1U<<15) | 13376, 
  (1U<<15) | 13376, (1U<<15) | 14927, (1U<<15) | 14927, (1U<<15) | 4992, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 11954, (1U<<15) | 5442, 
  (1U<<15) | 5442, (1U<<15) | 10863, (1U<<15) | 10863, (1U<<15) | 3199, (1U<<15) | 3199, (1U<<15) | 13376, (1U<<15) | 13376, (1U<<15) | 14927, 
  (1U<<15) | 14927, (1U<<15) | 4992, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 5442, (1U<<15) | 10863, 
  (1U<<15) | 10863, (1U<<15) | 3199, (1U<<15) | 3199, (1U<<15) | 13376, (1U<<15) | 13376, (1U<<15) | 14927, (1U<<15) | 14927, (1U<<15) | 4992, 
  (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 5442, (1U<<15) | 10863, (1U<<15) | 10863, (1U<<15) | 3199, 
  (1U<<15) | 3199, (1U<<15) | 13376, (1U<<15) | 13376, (1U<<15) | 14927, (1U<<15) | 14927, (1U<<15) | 4992, (1U<<15) | 4992, (1U<<15) | 11954, 
  (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 5442, (1U<<15) | 10863, (1U<<15) | 10863, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 14964, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 14026, (1U<<15) | 11171, (1U<<15) | 11213, 
  (1U<<15) | 11185, (1U<<15) | 11199, (1U<<15) | 15269, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 14964, (1U<<15) | 13405, 
  (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 2516, 
  (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3244, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 2516, 
  (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 3244, (1U<<15) | 3244, (1U<<15) | 2476, (1U<<15) | 4994, (1U<<15) | 9227, (1U<<15) | 14746, 
  (1U<<15) | 8972, (1U<<15) | 14753, (1U<<15) | 8972, (1U<<15) | 14753, (1U<<15) | 9227, (1U<<15) | 14746, (1U<<15) | 9227, (1U<<15) | 14746, 
  (1U<<15) | 9227, (1U<<15) | 14746, (1U<<15) | 9227, (1U<<15) | 14746, (1U<<15) | 9227, (1U<<15) | 14746, (1U<<15) | 8972, (1U<<15) | 14753, 
  0x44, 0x440, 0x40, 0x3939, 0x2a2a, 0x44, (1U<<15) | 2476, (1U<<15) | 14964, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 3170, (1U<<15) | 3170, 0x444, (1U<<15) | 3170, (1U<<15) | 3170, 0x444, 
  0x444, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 0x444, (1U<<15) | 3170, (1U<<15) | 2396, 
  (1U<<15) | 3170, (1U<<15) | 2396, (1U<<15) | 2396, (1U<<15) | 2396, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 0x444, (1U<<15) | 2476, (1U<<15) | 4994, 0x4444, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  0x4444, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 10822, (1U<<15) | 2474, (1U<<15) | 14962, (1U<<15) | 3206, (1U<<15) | 13403, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 5442, 
  (1U<<15) | 10863, (1U<<15) | 2474, (1U<<15) | 14962, (1U<<15) | 3206, (1U<<15) | 13403, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 5442, 
  (1U<<15) | 10863, 0x44, (1U<<15) | 2474, (1U<<15) | 4992, (1U<<15) | 2474, (1U<<15) | 4992, (1U<<15) | 2476, (1U<<15) | 14964, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 0x2c4, 0x594, 
  0x3b4, 0x2c4, 0x4a4, 0x4, (1U<<15) | 2474, (1U<<15) | 4992, (1U<<15) | 2476, (1U<<15) | 14964, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 0x2c4, 0x594, 
  0x3b4, 0x2c4, 0x4a4, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, 
  (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 0x44, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 3157, (1U<<15) | 3157, (1U<<15) | 3157, (1U<<15) | 2403, 
  (1U<<15) | 2403, (1U<<15) | 2403, (1U<<15) | 2403, (1U<<15) | 2403, (1U<<15) | 2403, (1U<<15) | 2390, (1U<<15) | 2390, (1U<<15) | 2390, 
  0x42c4, 0x4595, 0x43b4, 0x44a4, 0x42c4, 0x4595, 0x43b4, 0x44a4, 
  0x440, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, 0x4555, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 3244, 
  (1U<<15) | 3183, (1U<<15) | 13412, (1U<<15) | 2514, (1U<<15) | 3242, (1U<<15) | 13412, (1U<<15) | 2514, (1U<<15) | 3242, (1U<<15) | 3183, 
  0x4455, (1U<<15) | 3183, (1U<<15) | 3183, (1U<<15) | 2409, (1U<<15) | 2409, (1U<<15) | 3183, (1U<<15) | 3183, (1U<<15) | 3183, 
  0x4455, (1U<<15) | 3183, (1U<<15) | 3183, (1U<<15) | 2409, (1U<<15) | 2409, (1U<<15) | 13412, (1U<<15) | 2514, (1U<<15) | 3242, 
  (1U<<15) | 13412, (1U<<15) | 2514, (1U<<15) | 3242, (1U<<15) | 3183, 0x454, 0x454, 0x454, 0x454, 
  0x454, 0x454, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18291, 0x7a4a, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 16780, (1U<<15) | 18341, 
  (1U<<15) | 15022, (1U<<15) | 13455, 0x7a89, 0x6b7a, 0x7a89, 0x6b7a, 0x5989, 0x4a7a, 
  0x5989, 0x4a7a, 0x4a89, 0x3b7a, 0x4a89, 0x3b7a, 0x42c, 0x559, 
  0x43b, 0x44a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18348, (1U<<15) | 16787, (1U<<15) | 18350, (1U<<15) | 16789, 
  (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18348, (1U<<15) | 16787, 
  (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18312, 0x7a7a, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18350, (1U<<15) | 16789, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18334, (1U<<15) | 16773, 
  (1U<<15) | 18334, (1U<<15) | 16773, (1U<<15) | 18291, 0x7a4a, (1U<<15) | 18291, 0x7a4a, (1U<<15) | 16756, (1U<<15) | 18327, 
  (1U<<15) | 18291, 0x7a4a, (1U<<15) | 18291, 0x7a4a, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 3244, (1U<<15) | 13414, 
  (1U<<15) | 2516, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 3244, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 8406, (1U<<15) | 14641, (1U<<15) | 8413, (1U<<15) | 9950, 0x444, (1U<<15) | 2585, (1U<<15) | 14856, (1U<<15) | 3280, 
  (1U<<15) | 13311, 0x4e4, 0x4e2c, 0x4e59, 0x4e3b, 0x4e4a, 0x42c, 0x459, 
  0x43b, 0x44a, 0x4e59, 0x4e4a, 0x4e4, 0x4444, 0x4e4, 0x4455, 
  (1U<<15) | 3206, (1U<<15) | 13403, (1U<<15) | 3206, (1U<<15) | 13403, 0x4455, (1U<<15) | 2474, (1U<<15) | 14962, (1U<<15) | 3206, 
  (1U<<15) | 13403, (1U<<15) | 3183, (1U<<15) | 3183, (1U<<15) | 3183, (1U<<15) | 3183, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 10822, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 10822, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 0x444, 0x2c2c, 0x4455, 
  (1U<<15) | 3206, (1U<<15) | 13403, (1U<<15) | 3206, (1U<<15) | 13403, 0x4455, (1U<<15) | 2474, (1U<<15) | 14962, (1U<<15) | 3206, 
  (1U<<15) | 13403, 0x455, (1U<<15) | 3170, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 3170, (1U<<15) | 3177, (1U<<15) | 3177, 
  (1U<<15) | 3150, (1U<<15) | 3150, (1U<<15) | 3170, 0x444, (1U<<15) | 3170, 0x444, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 3183, (1U<<15) | 3183, 0x445, 0x445, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 
  0x2c2c, 0x5959, 0x3b3b, 0x4a4a, 0x2c2c, 0x5959, 0x3b3b, 0x4a4a, 
  (1U<<15) | 2476, (1U<<15) | 4994, (1U<<15) | 2476, (1U<<15) | 4994, (1U<<15) | 3170, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, 0x2c2c, 0x5959, 0x3b3b, 
  0x4a4a, (1U<<15) | 3170, (1U<<15) | 2396, 0x394, 0x394, 0x2a39, 0x2a39, 0x2a39, 
  0x2a39, 0x2a39, 0x2a39, 0x2a39, 0x2a39, (1U<<15) | 3163, (1U<<15) | 8511, (1U<<15) | 8511, 
  0x4439, (1U<<15) | 3163, 0x4439, (1U<<15) | 3163, 0x4444, 0x2a4, 0x44, 0x439, 
  0x42a, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 10822, (1U<<15) | 4994, (1U<<15) | 5444, (1U<<15) | 10822, 0x455, (1U<<15) | 5434, (1U<<15) | 4958, (1U<<15) | 5434, 
  0x444, (1U<<15) | 5434, (1U<<15) | 4958, (1U<<15) | 5434, (1U<<15) | 4958, 0x444, (1U<<15) | 5434, (1U<<15) | 4958, 
  (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 10863, (1U<<15) | 4992, (1U<<15) | 11954, (1U<<15) | 5442, (1U<<15) | 10863, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, 
  (1U<<15) | 14026, (1U<<15) | 14063, (1U<<15) | 14038, (1U<<15) | 14050, (1U<<15) | 14063, (1U<<15) | 14050, (1U<<15) | 3170, (1U<<15) | 3170, 
  0x444, (1U<<15) | 3170, (1U<<15) | 3170, 0x444, 0x444, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 3170, (1U<<15) | 2396, (1U<<15) | 3170, 
  (1U<<15) | 2396, (1U<<15) | 2396, (1U<<15) | 2396, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 4994, 
  (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 2474, (1U<<15) | 14962, (1U<<15) | 3206, (1U<<15) | 13403, 0x440, 
  (1U<<15) | 2476, (1U<<15) | 4994, 0x4, 0x888, 0x777, 0x777, 0x777, 0x777, 
  0x888, 0x777, 0x777, 0x777, 0x777, 0x888, 0x777, 0x777, 
  0x777, 0x777, 0x888, 0x777, 0x777, 0x777, 0x777, 0x5e0, 
  (1U<<15) | 14542, (1U<<15) | 17782, (1U<<15) | 17782, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17704, 
  (1U<<15) | 17704, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, 
  (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, (1U<<15) | 17775, 0x40, 
  0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x440, 0x440, 
  0x141, 0x1441, 0x141, 0x1441, 0x141, 0x1441, 0x141, 0x1441, 
  0x144, 0x1444, 0x144, 0x1444, 0x40, 0x440, 0x40, 0x440, 
  (1U<<15) | 21751, (1U<<15) | 21751, (1U<<15) | 21751, (1U<<15) | 21751, (1U<<15) | 21748, (1U<<15) | 21748, 0x444, 0x444, 
  0x88, 0x77, 0x77, (1U<<15) | 21406, (1U<<15) | 21406, (1U<<15) | 21406, (1U<<15) | 21403, (1U<<15) | 3576, 
  (1U<<15) | 3576, 0x4f0, 0x4f0, 0x77, 0x77, 0x0, (1U<<15) | 128, (1U<<15) | 698, 
  (1U<<15) | 691, (1U<<15) | 5576, (1U<<15) | 709, (1U<<15) | 2902, (1U<<15) | 678, (1U<<15) | 630, (1U<<15) | 574, (1U<<15) | 655, 
  (1U<<15) | 630, (1U<<15) | 603, (1U<<15) | 655, (1U<<15) | 630, (1U<<15) | 603, (1U<<15) | 858, (1U<<15) | 833, (1U<<15) | 805, 
  (1U<<15) | 774, (1U<<15) | 740, (1U<<15) | 740, (1U<<15) | 3702, (1U<<15) | 3667, (1U<<15) | 3628, (1U<<15) | 3685, (1U<<15) | 3667, 
  (1U<<15) | 3648, (1U<<15) | 3685, (1U<<15) | 3667, (1U<<15) | 3648, (1U<<15) | 3780, (1U<<15) | 3766, (1U<<15) | 3751, (1U<<15) | 3735, 
  (1U<<15) | 3718, (1U<<15) | 3718, (1U<<15) | 669, (1U<<15) | 619, (1U<<15) | 561, (1U<<15) | 645, (1U<<15) | 619, (1U<<15) | 591, 
  (1U<<15) | 645, (1U<<15) | 619, (1U<<15) | 591, (1U<<15) | 844, (1U<<15) | 817, (1U<<15) | 787, (1U<<15) | 754, (1U<<15) | 718, 
  (1U<<15) | 718, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 795, (1U<<15) | 763, (1U<<15) | 728, (1U<<15) | 850, (1U<<15) | 824, (1U<<15) | 795, (1U<<15) | 763, 
  (1U<<15) | 728, (1U<<15) | 728, 0x40, 0x40, (1U<<15) | 1340, (1U<<15) | 4291, (1U<<15) | 1340, (1U<<15) | 4291, 
  (1U<<15) | 1340, (1U<<15) | 4291, (1U<<15) | 1340, (1U<<15) | 4291, 0x0, 0xe0, 0xe0, (1U<<15) | 3572, 
  (1U<<15) | 3572, 0x0, 0x40, 0x87, 0x87, 0x87, 0x87, 0x87, 
  0x87, 0x87, 0x87, 0x84, 0x84, 0x84, 0x84, 0x84, 
  0x84, 0x85, 0x85, 0x85, 0x85, 0x84, 0x84, 0x84, 
  0x84, 0x85, 0x85, 0x85, 0x85, 0x5e0, (1U<<15) | 14542, 0x777, 
  0x777, 0x777, 0x777, 0x888, 0x777, 0x777, 0x888, 0x777, 
  0x777, 0x888, 0x777, 0x777, 0x888, 0x777, 0x777, 0x369, 
  0x369, 0x369, 0x369, 0x369, 0x369, 0x369, 0x369, 0x369, 
  0x369, (1U<<15) | 3925, 0x7f2f, 0x7f2f, 0x0, 0x693, 0x693, 0x693, 
  0x693, (1U<<15) | 18149, (1U<<15) | 18149, (1U<<15) | 18149, (1U<<15) | 18149, (1U<<15) | 18149, (1U<<15) | 18149, (1U<<15) | 18149, 
  (1U<<15) | 18149, 0x76, 0x76, 0x76, 0x76, 0x76, 0x76, 0x76, 
  0x76, 0x73, 0x73, 0x74, 0x74, 0x74, 0x74, 0x74, 
  0x74, 0x74, 0x74, 0x75, 0x75, 0x75, 0x75, 0x75, 
  0x75, 0x75, 0x75, 0x74, 0x74, 0x74, 0x74, 0x74, 
  0x74, 0x74, 0x74, 0x74, 0x74, 0x74, 0x74, 0x74, 
  0x74, 0x74, 0x74, 0x74, 0x74, 0x75, 0x75, 0x75, 
  0x75, 0x75, 0x75, 0x75, 0x75, 0x47a3, 0x47a3, (1U<<15) | 12245, 
  (1U<<15) | 12245, (1U<<15) | 12245, (1U<<15) | 12245, (1U<<15) | 12245, (1U<<15) | 12245, (1U<<15) | 12245, (1U<<15) | 12245, 0x7f2f, 
  0x7f2f, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 
  0x0, 0x0, 0x4e0, 0x4e0, 0x4e0, 0x4e0, 0x0, 0x0, 
  0x0, 0x0, 0x0, 0x0, 0x773, 0x773, 0x773, 0x773, 
  0x773, 0x773, 0x773, 0x773, 0x773, 0x773, 0x773, 0x773, 
  0x773, 0x773, (1U<<15) | 16726, (1U<<15) | 16726, (1U<<15) | 16726, (1U<<15) | 16726, (1U<<15) | 12228, (1U<<15) | 12228, 
  (1U<<15) | 12228, (1U<<15) | 12228, (1U<<15) | 16726, (1U<<15) | 16726, (1U<<15) | 16726, (1U<<15) | 16726, 0x7769, 0x7769, 
  0x7769, 0x7769, (1U<<15) | 12202, (1U<<15) | 12202, (1U<<15) | 12202, (1U<<15) | 12202, 0x7769, 0x7769, 
  0x7769, 0x7769, 0x11f4, 0x11f4, 0x88, 0x77, 0x77, (1U<<15) | 18219, 
  0x7777, 0x7777, 0x7777, 0x7777, (1U<<15) | 21859, (1U<<15) | 21755, (1U<<15) | 18219, 0x7777, 
  0x6666, (1U<<15) | 15387, (1U<<15) | 21859, (1U<<15) | 21755, 0x7777, 0x6666, (1U<<15) | 15387, (1U<<15) | 21859, 
  (1U<<15) | 21755, 0x6666, (1U<<15) | 15387, (1U<<15) | 21859, (1U<<15) | 21755, 0x7777, 0x6666, (1U<<15) | 15387, 
  (1U<<15) | 21859, (1U<<15) | 21755, 0x6666, (1U<<15) | 15387, (1U<<15) | 21859, (1U<<15) | 21755, 0x7777, 0x6666, 
  (1U<<15) | 15387, (1U<<15) | 18219, 0x7777, 0x7777, 0x7777, 0x7777, (1U<<15) | 18219, 0x7777, 
  0x7777, 0x7777, 0x7777, (1U<<15) | 21860, (1U<<15) | 21757, 0x888, 0x777, 0x666, 
  (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 
  0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, 
  (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 
  0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, 
  (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x888, 0x777, 
  0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, 
  (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, 
  (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 
  0x777, 0x666, (1U<<15) | 15389, (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, 
  (1U<<15) | 21860, (1U<<15) | 21757, 0x777, 0x666, (1U<<15) | 15389, 0x4444, (1U<<15) | 17333, (1U<<15) | 17333, 
  0xe4, (1U<<15) | 3582, 0x0, 0x0, 0x48, 0x48, 0x48, 0x48, 
  0x47, 0x47, 0x47, 0x47, (1U<<15) | 3931, (1U<<15) | 3931, (1U<<15) | 3931, (1U<<15) | 3931, 
  0x4444, 0x4444, 0x4444, 0x4444, (1U<<15) | 18980, 0x1, 0xe1, 0xe1, 
  0xe1, 0xe1, 0xe1, 0x51, 0x51, 0x51, (1U<<15) | 14088, (1U<<15) | 14088, 
  (1U<<15) | 14088, (1U<<15) | 14095, (1U<<15) | 14095, (1U<<15) | 14095, 0x4f4, 0x4f4, (1U<<15) | 14088, (1U<<15) | 14088, 
  (1U<<15) | 14095, (1U<<15) | 14095, 0x4f4, 0x4f4, (1U<<15) | 14088, (1U<<15) | 14088, (1U<<15) | 14095, (1U<<15) | 14095, 
  (1U<<15) | 13792, (1U<<15) | 13786, (1U<<15) | 13804, 0x88, 0x77, 0x77, 0x58, 0x58, 
  0x58, 0x58, 0x57, 0x57, 0x57, 0x57, 0x448, 0x4ee, 
  (1U<<15) | 5590, (1U<<15) | 5635, (1U<<15) | 14583, 0x444, 0x544, 0xe5, 0xe5, (1U<<15) | 12625, 
  (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, 0x4e5, 
  (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, 
  (1U<<15) | 5585, (1U<<15) | 3586, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, 
  (1U<<15) | 12625, (1U<<15) | 5585, 0x4e5, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, 
  (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 12625, (1U<<15) | 5585, (1U<<15) | 3586, (1U<<15) | 12625, (1U<<15) | 5566, (1U<<15) | 12625, 
  (1U<<15) | 5566, (1U<<15) | 12625, (1U<<15) | 5566, (1U<<15) | 12625, (1U<<15) | 5566, 0x4e0, (1U<<15) | 5566, 0xe0, 
  (1U<<15) | 3572, 0x54, 0x5e1, (1U<<15) | 5571, (1U<<15) | 5571, (1U<<15) | 5571, (1U<<15) | 5571, (1U<<15) | 14562, 
  (1U<<15) | 14562, (1U<<15) | 14562, (1U<<15) | 14562, (1U<<15) | 14562, (1U<<15) | 5571, (1U<<15) | 5571, (1U<<15) | 5571, (1U<<15) | 5571, 
  (1U<<15) | 8431, (1U<<15) | 8431, (1U<<15) | 8431, (1U<<15) | 8431, (1U<<15) | 14562, (1U<<15) | 14562, (1U<<15) | 14562, (1U<<15) | 14562, 
  (1U<<15) | 11785, (1U<<15) | 11785, (1U<<15) | 11785, (1U<<15) | 11785, 0x0, 0x0, 0x0, (1U<<15) | 8652, 
  (1U<<15) | 8666, (1U<<15) | 8598, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, 
  (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, 
  (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, 
  (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, 
  (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, 
  (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, 
  (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, (1U<<15) | 3000, 
  (1U<<15) | 16108, (1U<<15) | 15370, (1U<<15) | 15370, (1U<<15) | 15370, (1U<<15) | 15370, (1U<<15) | 15413, (1U<<15) | 16125, (1U<<15) | 16125, 
  (1U<<15) | 16125, (1U<<15) | 16125, (1U<<15) | 16243, (1U<<15) | 18255, (1U<<15) | 8652, (1U<<15) | 8652, (1U<<15) | 8652, (1U<<15) | 8652, 
  (1U<<15) | 8652, (1U<<15) | 8652, (1U<<15) | 8652, (1U<<15) | 8652, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, 
  (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, 
  (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, 
  (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, 
  (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 15353, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, 
  (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, 
  (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, 
  (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 16108, (1U<<15) | 8652, (1U<<15) | 8652, 
  (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8652, (1U<<15) | 8652, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8652, (1U<<15) | 8652, 
  (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8652, (1U<<15) | 8652, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 18224, (1U<<15) | 16125, 
  (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, 
  (1U<<15) | 16125, (1U<<15) | 15396, (1U<<15) | 16266, (1U<<15) | 18238, (1U<<15) | 16108, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, 
  (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, 
  (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 15436, (1U<<15) | 15565, (1U<<15) | 16573, 
  (1U<<15) | 15436, (1U<<15) | 15565, (1U<<15) | 16573, (1U<<15) | 15436, (1U<<15) | 15565, (1U<<15) | 16573, (1U<<15) | 18215, (1U<<15) | 15436, 
  (1U<<15) | 15565, (1U<<15) | 16573, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8706, 
  (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 10585, (1U<<15) | 10387, (1U<<15) | 10631, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10408, 
  (1U<<15) | 10602, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, 
  (1U<<15) | 8666, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8706, 
  (1U<<15) | 8706, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8706, 
  (1U<<15) | 8706, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 10585, (1U<<15) | 2973, (1U<<15) | 2973, 
  (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, 
  (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, 
  (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, 
  (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, 
  (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, 
  (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, 
  (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 2973, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8706, 
  (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 10585, (1U<<15) | 10387, (1U<<15) | 10631, (1U<<15) | 10564, 
  (1U<<15) | 10564, (1U<<15) | 10408, (1U<<15) | 10602, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, 
  (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10366, 
  (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, 
  (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, 
  (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, (1U<<15) | 10366, 
  (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, 
  (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, 
  (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, (1U<<15) | 10564, 
  (1U<<15) | 10564, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8706, 
  (1U<<15) | 8706, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8706, (1U<<15) | 8706, (1U<<15) | 8666, (1U<<15) | 8666, (1U<<15) | 8706, 
  (1U<<15) | 8706, (1U<<15) | 10585, (1U<<15) | 8652, (1U<<15) | 8666, (1U<<15) | 8598, 0x88, 0x77, 0x33, 
  0x44, 0x55, (1U<<15) | 18980, 0x888, 0x777, 0x777, 0x888, 0x777, 
  0x777, 0x888, 0x777, 0x777, 0x888, 0x777, 0x777, 0x444, 
  0x444, 0x444, 0x555, 0x333, 0x444, 0x555, 0x333, 0x40, 
  (1U<<15) | 21856, (1U<<15) | 21759, 0x30, 0xe0, 0xe0, (1U<<15) | 1336, (1U<<15) | 1336, (1U<<15) | 1336, 
  (1U<<15) | 1336, (1U<<15) | 15306, (1U<<15) | 15306, 0x4f0, 0xe0, 0x4444, 0x4444, 0x444, 
  0x444, 0x4444, 0x444, 0x444, 0x88, 0x77, 0x88, 0x77, 
  0x77, 0x88, 0x77, 0x77, 0x88, 0x77, 0x77, 0x88, 
  0x77, 0x77, 0x4, 0x4, 0x5, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x5, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x444, 0x444, 0x477, 0x477, 0x477, 
  0x477, 0x477, 0x477, 0x477, 0x477, 0x444, 0x444, 0x444, 
  0x444, 0x444, 0x444, 0xe4, 0x88, 0x77, 0x77, 0x88, 
  0x77, 0x88, 0x77, 0x4444, 0x5555, 0x3333, 0x4444, 0x5555, 
  0x3333, 0x88, 0x77, 0x77, 0x40, 0x40, 0x444, 0x444, 
  0x4477, (1U<<15) | 10541, 0x4444, (1U<<15) | 8437, 0x4477, (1U<<15) | 10541, 0x4444, (1U<<15) | 8437, 
  0x4477, (1U<<15) | 10541, 0x4444, (1U<<15) | 8437, (1U<<15) | 10558, (1U<<15) | 10549, (1U<<15) | 8570, (1U<<15) | 8550, 
  (1U<<15) | 10558, (1U<<15) | 10549, (1U<<15) | 8570, (1U<<15) | 8550, (1U<<15) | 10558, (1U<<15) | 10549, (1U<<15) | 8570, (1U<<15) | 8550, 
  (1U<<15) | 10558, (1U<<15) | 10549, (1U<<15) | 8570, (1U<<15) | 8550, 0x4477, (1U<<15) | 10541, 0x4444, (1U<<15) | 8437, 
  0x77, 0x77, 0x77, 0x77, 0x77, 0x88, 0x77, 0x77, 
  0x88, 0x77, 0x77, 0x88, 0x77, 0x77, 0x88, 0x77, 
  0x77, 0x55e0, (1U<<15) | 14719, 0x44f0, (1U<<15) | 10202, (1U<<15) | 8964, 0x44f0, 0x44f0, 
  (1U<<15) | 10202, (1U<<15) | 10202, (1U<<15) | 8964, (1U<<15) | 8964, 0x4453, 0x4453, 0x4453, 0x4454, 
  0x4454, 0x4454, 0x4455, 0x4455, 0x4455, 0x4453, 0x4453, 0x4453, 
  (1U<<15) | 10264, (1U<<15) | 10264, (1U<<15) | 10264, (1U<<15) | 10282, (1U<<15) | 10282, (1U<<15) | 10282, (1U<<15) | 10311, (1U<<15) | 10311, 
  (1U<<15) | 10311, (1U<<15) | 10264, (1U<<15) | 10264, (1U<<15) | 10264, (1U<<15) | 10272, (1U<<15) | 10272, (1U<<15) | 10272, (1U<<15) | 10290, 
  (1U<<15) | 10290, (1U<<15) | 10290, (1U<<15) | 10272, (1U<<15) | 10272, (1U<<15) | 10272, 0x453, 0x453, 0x453, 
  0x454, 0x454, 0x454, 0x455, 0x455, 0x455, 0x453, 0x453, 
  0x453, (1U<<15) | 11752, (1U<<15) | 11752, (1U<<15) | 11752, (1U<<15) | 11791, (1U<<15) | 11791, (1U<<15) | 11791, (1U<<15) | 11817, 
  (1U<<15) | 11817, (1U<<15) | 11817, (1U<<15) | 11752, (1U<<15) | 11752, (1U<<15) | 11752, (1U<<15) | 11759, (1U<<15) | 11759, (1U<<15) | 11759, 
  (1U<<15) | 11798, (1U<<15) | 11798, (1U<<15) | 11798, (1U<<15) | 11759, (1U<<15) | 11759, (1U<<15) | 11759, (1U<<15) | 9268, (1U<<15) | 9268, 
  (1U<<15) | 9268, (1U<<15) | 9288, (1U<<15) | 9288, (1U<<15) | 9288, (1U<<15) | 9311, (1U<<15) | 9311, (1U<<15) | 9311, (1U<<15) | 9268, 
  (1U<<15) | 9268, (1U<<15) | 9268, (1U<<15) | 9265, (1U<<15) | 9265, (1U<<15) | 9265, (1U<<15) | 9285, (1U<<15) | 9285, (1U<<15) | 9285, 
  (1U<<15) | 9317, (1U<<15) | 9317, (1U<<15) | 9317, (1U<<15) | 9265, (1U<<15) | 9265, (1U<<15) | 9265, (1U<<15) | 9274, (1U<<15) | 9274, 
  (1U<<15) | 9274, (1U<<15) | 9294, (1U<<15) | 9294, (1U<<15) | 9294, (1U<<15) | 9274, (1U<<15) | 9274, (1U<<15) | 9274, 0x4453, 
  0x4453, 0x4453, 0x4454, 0x4454, 0x4454, 0x4455, 0x4455, 0x4455, 
  0x4453, 0x4453, 0x4453, (1U<<15) | 10264, (1U<<15) | 10264, (1U<<15) | 10264, (1U<<15) | 10282, (1U<<15) | 10282, 
  (1U<<15) | 10282, (1U<<15) | 10311, (1U<<15) | 10311, (1U<<15) | 10311, (1U<<15) | 10264, (1U<<15) | 10264, (1U<<15) | 10264, (1U<<15) | 10272, 
  (1U<<15) | 10272, (1U<<15) | 10272, (1U<<15) | 10290, (1U<<15) | 10290, (1U<<15) | 10290, (1U<<15) | 10272, (1U<<15) | 10272, (1U<<15) | 10272, 
  (1U<<15) | 9268, (1U<<15) | 9268, (1U<<15) | 9268, (1U<<15) | 9288, (1U<<15) | 9288, (1U<<15) | 9288, (1U<<15) | 9311, (1U<<15) | 9311, 
  (1U<<15) | 9311, (1U<<15) | 9268, (1U<<15) | 9268, (1U<<15) | 9268, (1U<<15) | 9265, (1U<<15) | 9265, (1U<<15) | 9265, (1U<<15) | 9285, 
  (1U<<15) | 9285, (1U<<15) | 9285, (1U<<15) | 9317, (1U<<15) | 9317, (1U<<15) | 9317, (1U<<15) | 9265, (1U<<15) | 9265, (1U<<15) | 9265, 
  (1U<<15) | 9274, (1U<<15) | 9274, (1U<<15) | 9274, (1U<<15) | 9294, (1U<<15) | 9294, (1U<<15) | 9294, (1U<<15) | 9274, (1U<<15) | 9274, 
  (1U<<15) | 9274, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, (1U<<15) | 3091, 
  (1U<<15) | 3091, (1U<<15) | 3091, (1U<<15) | 9234, (1U<<15) | 9234, (1U<<15) | 9234, (1U<<15) | 14597, (1U<<15) | 14597, (1U<<15) | 14597, 
  (1U<<15) | 3091, (1U<<15) | 3091, (1U<<15) | 3091, (1U<<15) | 3031, (1U<<15) | 3031, (1U<<15) | 3031, (1U<<15) | 8980, (1U<<15) | 8980, 
  (1U<<15) | 8980, (1U<<15) | 14733, (1U<<15) | 14733, (1U<<15) | 14733, (1U<<15) | 3031, (1U<<15) | 3031, (1U<<15) | 3031, (1U<<15) | 2956, 
  (1U<<15) | 2956, (1U<<15) | 2956, (1U<<15) | 8837, (1U<<15) | 8837, (1U<<15) | 8837, (1U<<15) | 2956, (1U<<15) | 2956, (1U<<15) | 2956, 
  0x3450, 0x3450, 0x3450, 0x4450, 0x4450, 0x4450, 0x5450, 0x5450, 
  0x5450, 0x3450, 0x3450, 0x3450, (1U<<15) | 3038, (1U<<15) | 3038, (1U<<15) | 3038, (1U<<15) | 9234, 
  (1U<<15) | 9234, (1U<<15) | 9234, (1U<<15) | 14740, (1U<<15) | 14740, (1U<<15) | 14740, (1U<<15) | 3038, (1U<<15) | 3038, (1U<<15) | 3038, 
  (1U<<15) | 2965, (1U<<15) | 2965, (1U<<15) | 2965, (1U<<15) | 8900, (1U<<15) | 8900, (1U<<15) | 8900, (1U<<15) | 2965, (1U<<15) | 2965, 
  (1U<<15) | 2965, (1U<<15) | 3084, (1U<<15) | 3084, (1U<<15) | 3084, (1U<<15) | 8980, (1U<<15) | 8980, (1U<<15) | 8980, (1U<<15) | 14590, 
  (1U<<15) | 14590, (1U<<15) | 14590, (1U<<15) | 3084, (1U<<15) | 3084, (1U<<15) | 3084, (1U<<15) | 3023, (1U<<15) | 3023, (1U<<15) | 3023, 
  (1U<<15) | 8900, (1U<<15) | 8900, (1U<<15) | 8900, (1U<<15) | 14725, (1U<<15) | 14725, (1U<<15) | 14725, (1U<<15) | 3023, (1U<<15) | 3023, 
  (1U<<15) | 3023, (1U<<15) | 2946, (1U<<15) | 2946, (1U<<15) | 2946, (1U<<15) | 8827, (1U<<15) | 8827, (1U<<15) | 8827, (1U<<15) | 2946, 
  (1U<<15) | 2946, (1U<<15) | 2946, (1U<<15) | 3091, (1U<<15) | 3091, (1U<<15) | 3091, (1U<<15) | 9234, (1U<<15) | 9234, (1U<<15) | 9234, 
  (1U<<15) | 14597, (1U<<15) | 14597, (1U<<15) | 14597, (1U<<15) | 3091, (1U<<15) | 3091, (1U<<15) | 3091, (1U<<15) | 3031, (1U<<15) | 3031, 
  (1U<<15) | 3031, (1U<<15) | 8980, (1U<<15) | 8980, (1U<<15) | 8980, (1U<<15) | 14733, (1U<<15) | 14733, (1U<<15) | 14733, (1U<<15) | 3031, 
  (1U<<15) | 3031, (1U<<15) | 3031, (1U<<15) | 2956, (1U<<15) | 2956, (1U<<15) | 2956, (1U<<15) | 8837, (1U<<15) | 8837, (1U<<15) | 8837, 
  (1U<<15) | 2956, (1U<<15) | 2956, (1U<<15) | 2956, (1U<<15) | 3084, (1U<<15) | 3084, (1U<<15) | 3084, (1U<<15) | 8980, (1U<<15) | 8980, 
  (1U<<15) | 8980, (1U<<15) | 14590, (1U<<15) | 14590, (1U<<15) | 14590, (1U<<15) | 3084, (1U<<15) | 3084, (1U<<15) | 3084, (1U<<15) | 3023, 
  (1U<<15) | 3023, (1U<<15) | 3023, (1U<<15) | 8900, (1U<<15) | 8900, (1U<<15) | 8900, (1U<<15) | 14725, (1U<<15) | 14725, (1U<<15) | 14725, 
  (1U<<15) | 3023, (1U<<15) | 3023, (1U<<15) | 3023, (1U<<15) | 2946, (1U<<15) | 2946, (1U<<15) | 2946, (1U<<15) | 8827, (1U<<15) | 8827, 
  (1U<<15) | 8827, (1U<<15) | 2946, (1U<<15) | 2946, (1U<<15) | 2946, (1U<<15) | 3091, (1U<<15) | 9234, (1U<<15) | 3091, (1U<<15) | 3031, 
  (1U<<15) | 8980, (1U<<15) | 3031, (1U<<15) | 2956, (1U<<15) | 8837, (1U<<15) | 2956, 0x3450, 0x4450, 0x3450, 
  (1U<<15) | 3038, (1U<<15) | 9234, (1U<<15) | 3038, (1U<<15) | 2965, (1U<<15) | 8900, (1U<<15) | 2965, (1U<<15) | 3084, (1U<<15) | 8980, 
  (1U<<15) | 3084, (1U<<15) | 3023, (1U<<15) | 8900, (1U<<15) | 3023, (1U<<15) | 2946, (1U<<15) | 8827, (1U<<15) | 2946, (1U<<15) | 3091, 
  (1U<<15) | 9234, (1U<<15) | 3091, (1U<<15) | 3031, (1U<<15) | 8980, (1U<<15) | 3031, (1U<<15) | 2956, (1U<<15) | 8837, (1U<<15) | 2956, 
  (1U<<15) | 3084, (1U<<15) | 8980, (1U<<15) | 3084, (1U<<15) | 3023, (1U<<15) | 8900, (1U<<15) | 3023, (1U<<15) | 2946, (1U<<15) | 8827, 
  (1U<<15) | 2946, 0x4e0, 0x4e0, (1U<<15) | 5566, (1U<<15) | 5566, 0xe0, 0xe0, 0x3e0, 
  0x3e0, (1U<<15) | 3075, (1U<<15) | 3075, (1U<<15) | 3572, (1U<<15) | 3572, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, 
  (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, 
  (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, 
  (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, 
  (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, (1U<<15) | 14825, 
  (1U<<15) | 14825, (1U<<15) | 12145, (1U<<15) | 12145, 0x0, 0x0, (1U<<15) | 924, (1U<<15) | 948, (1U<<15) | 930, 
  (1U<<15) | 954, (1U<<15) | 936, (1U<<15) | 960, (1U<<15) | 942, (1U<<15) | 930, (1U<<15) | 954, (1U<<15) | 936, (1U<<15) | 960, 
  (1U<<15) | 942, (1U<<15) | 948, (1U<<15) | 869, (1U<<15) | 910, (1U<<15) | 889, (1U<<15) | 868, (1U<<15) | 896, (1U<<15) | 875, 
  (1U<<15) | 903, (1U<<15) | 882, (1U<<15) | 925, (1U<<15) | 960, (1U<<15) | 942, (1U<<15) | 924, (1U<<15) | 948, (1U<<15) | 930, 
  (1U<<15) | 954, (1U<<15) | 936, (1U<<15) | 925, (1U<<15) | 960, (1U<<15) | 942, (1U<<15) | 924, (1U<<15) | 948, (1U<<15) | 930, 
  (1U<<15) | 954, (1U<<15) | 936, (1U<<15) | 8445, (1U<<15) | 9925, (1U<<15) | 10044, (1U<<15) | 10479, (1U<<15) | 10479, (1U<<15) | 10479, 
  (1U<<15) | 10479, (1U<<15) | 10479, (1U<<15) | 10479, (1U<<15) | 9240, (1U<<15) | 9957, (1U<<15) | 10076, (1U<<15) | 9390, (1U<<15) | 10015, 
  (1U<<15) | 10134, (1U<<15) | 10508, (1U<<15) | 10508, (1U<<15) | 10508, (1U<<15) | 10508, (1U<<15) | 10508, (1U<<15) | 10508, (1U<<15) | 9338, 
  (1U<<15) | 9984, (1U<<15) | 10103, (1U<<15) | 9403, (1U<<15) | 9403, (1U<<15) | 10029, (1U<<15) | 10029, (1U<<15) | 10148, (1U<<15) | 10148, 
  (1U<<15) | 10524, (1U<<15) | 10524, (1U<<15) | 10524, (1U<<15) | 10524, (1U<<15) | 10524, (1U<<15) | 10524, (1U<<15) | 9352, (1U<<15) | 9352, 
  (1U<<15) | 9999, (1U<<15) | 9999, (1U<<15) | 10118, (1U<<15) | 10118, (1U<<15) | 8456, (1U<<15) | 8456, (1U<<15) | 9937, (1U<<15) | 9937, 
  (1U<<15) | 10056, (1U<<15) | 10056, (1U<<15) | 10493, (1U<<15) | 10493, (1U<<15) | 10493, (1U<<15) | 10493, (1U<<15) | 10493, (1U<<15) | 10493, 
  (1U<<15) | 9252, (1U<<15) | 9252, (1U<<15) | 9970, (1U<<15) | 9970, (1U<<15) | 10089, (1U<<15) | 10089, (1U<<15) | 8445, (1U<<15) | 9240, 
  (1U<<15) | 9390, (1U<<15) | 9338, (1U<<15) | 9403, (1U<<15) | 9352, (1U<<15) | 8456, (1U<<15) | 9252, 0x0, 0x0, 
  (1U<<15) | 15710, (1U<<15) | 15710, (1U<<15) | 459, (1U<<15) | 519, (1U<<15) | 474, (1U<<15) | 534, (1U<<15) | 489, (1U<<15) | 549, 
  (1U<<15) | 504, (1U<<15) | 474, (1U<<15) | 534, (1U<<15) | 489, (1U<<15) | 549, (1U<<15) | 504, (1U<<15) | 519, (1U<<15) | 407, 
  (1U<<15) | 541, (1U<<15) | 496, (1U<<15) | 451, (1U<<15) | 511, (1U<<15) | 466, (1U<<15) | 526, (1U<<15) | 481, (1U<<15) | 414, 
  (1U<<15) | 549, (1U<<15) | 504, (1U<<15) | 459, (1U<<15) | 519, (1U<<15) | 474, (1U<<15) | 534, (1U<<15) | 489, (1U<<15) | 414, 
  (1U<<15) | 549, (1U<<15) | 504, (1U<<15) | 459, (1U<<15) | 519, (1U<<15) | 474, (1U<<15) | 534, (1U<<15) | 489, 0x0, 
  0x0, (1U<<15) | 10202, (1U<<15) | 10202, 0x44f0, 0x44f0, 0x54f0, (1U<<15) | 10202, (1U<<15) | 14616, 
  0x44f0, 0x44f0, 0x44f0, 0x44f0, (1U<<15) | 16037, (1U<<15) | 16024, (1U<<15) | 16024, (1U<<15) | 15947, 
  (1U<<15) | 15935, (1U<<15) | 15935, (1U<<15) | 15759, (1U<<15) | 10319, (1U<<15) | 15748, (1U<<15) | 10300, (1U<<15) | 15748, (1U<<15) | 10300, 
  (1U<<15) | 16085, (1U<<15) | 16073, (1U<<15) | 16073, (1U<<15) | 15991, (1U<<15) | 15980, (1U<<15) | 15980, (1U<<15) | 15887, (1U<<15) | 11824, 
  (1U<<15) | 15877, (1U<<15) | 11807, (1U<<15) | 15877, (1U<<15) | 11807, (1U<<15) | 16314, (1U<<15) | 16298, (1U<<15) | 16298, (1U<<15) | 16037, 
  (1U<<15) | 16024, (1U<<15) | 16024, (1U<<15) | 15947, (1U<<15) | 9326, (1U<<15) | 15935, (1U<<15) | 9305, (1U<<15) | 15935, (1U<<15) | 9305, 
  (1U<<15) | 16374, (1U<<15) | 16359, (1U<<15) | 16359, (1U<<15) | 16085, (1U<<15) | 16073, (1U<<15) | 16073, (1U<<15) | 15991, (1U<<15) | 10319, 
  (1U<<15) | 15980, (1U<<15) | 10300, (1U<<15) | 15980, (1U<<15) | 10300, (1U<<15) | 16671, (1U<<15) | 16653, (1U<<15) | 16653, (1U<<15) | 16218, 
  (1U<<15) | 16205, (1U<<15) | 16205, (1U<<15) | 16085, (1U<<15) | 9326, (1U<<15) | 16073, (1U<<15) | 9305, (1U<<15) | 16073, (1U<<15) | 9305, 
  (1U<<15) | 16166, (1U<<15) | 16152, (1U<<15) | 16152, (1U<<15) | 16037, (1U<<15) | 16024, (1U<<15) | 16024, (1U<<15) | 16218, (1U<<15) | 16205, 
  (1U<<15) | 16205, (1U<<15) | 16085, (1U<<15) | 16073, (1U<<15) | 16073, (1U<<15) | 16050, (1U<<15) | 16012, (1U<<15) | 16012, (1U<<15) | 15959, 
  (1U<<15) | 15924, (1U<<15) | 15924, (1U<<15) | 15770, (1U<<15) | 10337, (1U<<15) | 15738, (1U<<15) | 10290, (1U<<15) | 15738, (1U<<15) | 10290, 
  (1U<<15) | 16097, (1U<<15) | 16062, (1U<<15) | 16062, (1U<<15) | 16002, (1U<<15) | 15970, (1U<<15) | 15970, (1U<<15) | 15897, (1U<<15) | 11875, 
  (1U<<15) | 15868, (1U<<15) | 11798, (1U<<15) | 15868, (1U<<15) | 11798, (1U<<15) | 16330, (1U<<15) | 16283, (1U<<15) | 16283, (1U<<15) | 16050, 
  (1U<<15) | 16012, (1U<<15) | 16012, (1U<<15) | 15959, (1U<<15) | 9367, (1U<<15) | 15924, (1U<<15) | 9294, (1U<<15) | 15924, (1U<<15) | 9294, 
  (1U<<15) | 16389, (1U<<15) | 16345, (1U<<15) | 16345, (1U<<15) | 16097, (1U<<15) | 16062, (1U<<15) | 16062, (1U<<15) | 16002, (1U<<15) | 10337, 
  (1U<<15) | 15970, (1U<<15) | 10290, (1U<<15) | 15970, (1U<<15) | 10290, (1U<<15) | 16689, (1U<<15) | 16636, (1U<<15) | 16636, (1U<<15) | 16231, 
  (1U<<15) | 16193, (1U<<15) | 16193, (1U<<15) | 16097, (1U<<15) | 9367, (1U<<15) | 16062, (1U<<15) | 9294, (1U<<15) | 16062, (1U<<15) | 9294, 
  (1U<<15) | 16618, (1U<<15) | 16600, (1U<<15) | 16600, (1U<<15) | 16180, (1U<<15) | 16139, (1U<<15) | 16139, (1U<<15) | 16050, (1U<<15) | 16012, 
  (1U<<15) | 16012, (1U<<15) | 16689, (1U<<15) | 16636, (1U<<15) | 16636, (1U<<15) | 16231, (1U<<15) | 16193, (1U<<15) | 16193, (1U<<15) | 16097, 
  (1U<<15) | 16062, (1U<<15) | 16062, (1U<<15) | 14246, 0x4f5, (1U<<15) | 15991, (1U<<15) | 15980, (1U<<15) | 15980, (1U<<15) | 15991, 
  (1U<<15) | 15980, (1U<<15) | 15980, (1U<<15) | 15991, (1U<<15) | 15980, (1U<<15) | 15980, (1U<<15) | 15991, (1U<<15) | 15980, (1U<<15) | 15980, 
  (1U<<15) | 16002, (1U<<15) | 15970, (1U<<15) | 15970, (1U<<15) | 16002, (1U<<15) | 15970, (1U<<15) | 15970, (1U<<15) | 16002, (1U<<15) | 15970, 
  (1U<<15) | 15970, (1U<<15) | 16002, (1U<<15) | 15970, (1U<<15) | 15970, 0x88, 0x77, 0x77, 0x54, 
  0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, (1U<<15) | 3590, 
  0x48, 0x48, 0x48, 0x48, 0x47, 0x47, 0x47, 0x47, 
  0x58, 0x58, 0x58, 0x58, 0x57, 0x57, 0x57, 0x57, 
  0x11, 0x141, 0x11, 0x141, 0x14, 0x144, 0x11, 0x141, 
  0x0, 0x0, 0x50, (1U<<15) | 14095, (1U<<15) | 14136, (1U<<15) | 14088, (1U<<15) | 11241, (1U<<15) | 11279, 
  (1U<<15) | 11233, (1U<<15) | 11233, (1U<<15) | 14088, (1U<<15) | 14095, (1U<<15) | 14136, (1U<<15) | 14088, (1U<<15) | 11241, (1U<<15) | 11279, 
  (1U<<15) | 11233, (1U<<15) | 11233, (1U<<15) | 14088, (1U<<15) | 14095, (1U<<15) | 14136, (1U<<15) | 14088, (1U<<15) | 11241, (1U<<15) | 11279, 
  (1U<<15) | 11233, (1U<<15) | 11233, (1U<<15) | 14088, (1U<<15) | 14095, (1U<<15) | 14136, (1U<<15) | 14088, (1U<<15) | 11241, (1U<<15) | 11279, 
  (1U<<15) | 11233, (1U<<15) | 11233, (1U<<15) | 14088, (1U<<15) | 14123, (1U<<15) | 14157, (1U<<15) | 14104, (1U<<15) | 11265, (1U<<15) | 11301, 
  (1U<<15) | 11251, (1U<<15) | 14123, (1U<<15) | 14157, (1U<<15) | 14104, (1U<<15) | 11265, (1U<<15) | 11301, (1U<<15) | 11251, (1U<<15) | 16432, 
  (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, 
  (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 16432, (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, 
  (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 8751, 
  (1U<<15) | 8751, (1U<<15) | 16432, (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, 
  (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 16432, (1U<<15) | 15463, 
  (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8751, 
  (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 8751, (1U<<15) | 15384, (1U<<15) | 16459, (1U<<15) | 8815, (1U<<15) | 12074, (1U<<15) | 12215, 
  (1U<<15) | 8788, (1U<<15) | 15384, (1U<<15) | 16459, (1U<<15) | 8815, (1U<<15) | 12074, (1U<<15) | 12215, (1U<<15) | 8788, (1U<<15) | 11241, 
  (1U<<15) | 14095, (1U<<15) | 11241, (1U<<15) | 14095, (1U<<15) | 11241, (1U<<15) | 14095, (1U<<15) | 11241, (1U<<15) | 14095, (1U<<15) | 14157, 
  (1U<<15) | 11301, (1U<<15) | 14157, (1U<<15) | 11301, (1U<<15) | 16432, (1U<<15) | 16432, (1U<<15) | 16432, (1U<<15) | 16432, (1U<<15) | 16459, 
  (1U<<15) | 12215, (1U<<15) | 16459, (1U<<15) | 12215, (1U<<15) | 14104, (1U<<15) | 14136, (1U<<15) | 14095, (1U<<15) | 11251, (1U<<15) | 11279, 
  (1U<<15) | 11241, (1U<<15) | 11241, (1U<<15) | 14095, (1U<<15) | 14104, (1U<<15) | 14136, (1U<<15) | 14095, (1U<<15) | 11251, (1U<<15) | 11279, 
  (1U<<15) | 11241, (1U<<15) | 11241, (1U<<15) | 14095, (1U<<15) | 14088, (1U<<15) | 14136, 0x4f4, (1U<<15) | 11233, (1U<<15) | 11279, 
  0x44f4, 0x44f4, 0x4f4, (1U<<15) | 14088, (1U<<15) | 14136, 0x4f4, (1U<<15) | 11233, (1U<<15) | 11279, 
  0x44f4, 0x44f4, 0x4f4, (1U<<15) | 14123, (1U<<15) | 14157, (1U<<15) | 14104, (1U<<15) | 11265, (1U<<15) | 11301, 
  (1U<<15) | 11251, (1U<<15) | 14123, (1U<<15) | 14157, (1U<<15) | 14104, (1U<<15) | 11265, (1U<<15) | 11301, (1U<<15) | 11251, (1U<<15) | 16403, 
  (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, 
  (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 16403, (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, 
  (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, 
  (1U<<15) | 8727, (1U<<15) | 16403, (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, 
  (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 16403, (1U<<15) | 15463, 
  (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8727, 
  (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 15384, (1U<<15) | 16459, (1U<<15) | 8815, (1U<<15) | 12074, (1U<<15) | 12215, 
  (1U<<15) | 8788, (1U<<15) | 15384, (1U<<15) | 16459, (1U<<15) | 8815, (1U<<15) | 12074, (1U<<15) | 12215, (1U<<15) | 8788, (1U<<15) | 14088, 
  (1U<<15) | 14136, 0x4f4, (1U<<15) | 11233, (1U<<15) | 11279, 0x44f4, 0x44f4, 0x4f4, (1U<<15) | 14088, 
  (1U<<15) | 14136, 0x4f4, (1U<<15) | 11233, (1U<<15) | 11279, 0x44f4, 0x44f4, 0x4f4, (1U<<15) | 14104, 
  (1U<<15) | 14136, (1U<<15) | 14095, (1U<<15) | 11251, (1U<<15) | 11279, (1U<<15) | 11241, (1U<<15) | 11241, (1U<<15) | 14095, (1U<<15) | 14104, 
  (1U<<15) | 14136, (1U<<15) | 14095, (1U<<15) | 11251, (1U<<15) | 11279, (1U<<15) | 11241, (1U<<15) | 11241, (1U<<15) | 14095, (1U<<15) | 14123, 
  (1U<<15) | 14157, (1U<<15) | 14104, (1U<<15) | 11265, (1U<<15) | 11301, (1U<<15) | 11251, (1U<<15) | 14123, (1U<<15) | 14157, (1U<<15) | 14104, 
  (1U<<15) | 11265, (1U<<15) | 11301, (1U<<15) | 11251, (1U<<15) | 16403, (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, 
  (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, 
  (1U<<15) | 16403, (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, 
  (1U<<15) | 16522, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 16403, (1U<<15) | 15463, (1U<<15) | 15463, 
  (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8727, (1U<<15) | 8727, 
  (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 16403, (1U<<15) | 15463, (1U<<15) | 15463, (1U<<15) | 16471, (1U<<15) | 16471, (1U<<15) | 15514, 
  (1U<<15) | 15514, (1U<<15) | 16522, (1U<<15) | 16522, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 8727, (1U<<15) | 15384, 
  (1U<<15) | 16459, (1U<<15) | 8815, (1U<<15) | 12074, (1U<<15) | 12215, (1U<<15) | 8788, (1U<<15) | 15384, (1U<<15) | 16459, (1U<<15) | 8815, 
  (1U<<15) | 12074, (1U<<15) | 12215, (1U<<15) | 8788, 0x4f4, 0x44f4, 0x4f4, 0x44f4, (1U<<15) | 14088, 
  (1U<<15) | 11233, (1U<<15) | 14088, (1U<<15) | 11233, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 10202, (1U<<15) | 9220, (1U<<15) | 10202, 
  (1U<<15) | 9220, 0x4f4, 0x44f4, 0x44f4, 0x4f4, 0x4f4, 0x44f4, 0x44f4, 
  0x4f4, (1U<<15) | 14088, (1U<<15) | 11233, (1U<<15) | 14088, (1U<<15) | 11233, (1U<<15) | 8598, (1U<<15) | 8598, (1U<<15) | 8598, 
  (1U<<15) | 8598, (1U<<15) | 10202, (1U<<15) | 9220, (1U<<15) | 10202, (1U<<15) | 9220, 0x4f8, 0x44f8, 0x4f8, 
  0x44f8, 0x4f8, 0x44f8, 0x4f8, 0x44f8, (1U<<15) | 14170, (1U<<15) | 11315, (1U<<15) | 14170, 
  (1U<<15) | 11315, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, 
  (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, 
  (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18215, (1U<<15) | 18209, (1U<<15) | 12671, (1U<<15) | 18209, 
  (1U<<15) | 12671, 0x444, 0x444, 0x4555, (1U<<15) | 21710, 0x444, 0x555, (1U<<15) | 14964, 
  (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 14964, (1U<<15) | 2474, (1U<<15) | 2474, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 13405, 0x5959, (1U<<15) | 10347, (1U<<15) | 9950, 0x40, 0x0, 0x44e0, 0x44e0, 
  0x44e0, 0x44e0, 0xe2c, 0xe3b, 0xe4a, 0xe2c, 0xe2c, 0xe4a, 
  0xe4a, 0x3b, 0x4a0, 0x52c, 0x559, 0x53b, (1U<<15) | 15310, 0x54a, 
  (1U<<15) | 19213, (1U<<15) | 19229, (1U<<15) | 19298, (1U<<15) | 19298, (1U<<15) | 19298, (1U<<15) | 2476, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 21438, (1U<<15) | 13405, (1U<<15) | 21436, (1U<<15) | 21436, (1U<<15) | 2476, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 2614, (1U<<15) | 2549, (1U<<15) | 10938, (1U<<15) | 14964, (1U<<15) | 10938, (1U<<15) | 4994, (1U<<15) | 4994, (1U<<15) | 14964, 
  0x2c4, (1U<<15) | 16773, (1U<<15) | 16766, (1U<<15) | 16773, (1U<<15) | 16766, (1U<<15) | 2476, (1U<<15) | 2533, (1U<<15) | 14964, 
  (1U<<15) | 14887, (1U<<15) | 3208, (1U<<15) | 3235, (1U<<15) | 21438, (1U<<15) | 21429, (1U<<15) | 13405, (1U<<15) | 13396, (1U<<15) | 16773, 
  (1U<<15) | 16766, (1U<<15) | 16773, (1U<<15) | 16766, (1U<<15) | 2476, (1U<<15) | 2533, (1U<<15) | 14964, (1U<<15) | 14887, (1U<<15) | 3208, 
  (1U<<15) | 3235, (1U<<15) | 21438, (1U<<15) | 21429, (1U<<15) | 13405, (1U<<15) | 13396, (1U<<15) | 2476, (1U<<15) | 2533, (1U<<15) | 14964, 
  (1U<<15) | 14887, (1U<<15) | 3208, (1U<<15) | 3235, (1U<<15) | 21438, (1U<<15) | 21429, (1U<<15) | 13405, (1U<<15) | 13396, (1U<<15) | 2476, 
  (1U<<15) | 2533, (1U<<15) | 3208, (1U<<15) | 3235, (1U<<15) | 13405, (1U<<15) | 13396, (1U<<15) | 2476, (1U<<15) | 2533, (1U<<15) | 3208, 
  (1U<<15) | 3235, (1U<<15) | 13405, (1U<<15) | 13396, 0x42c5, 0x4595, 0x43b5, 0x44a5, (1U<<15) | 12270, 
  (1U<<15) | 12270, (1U<<15) | 14964, 0x2c4, (1U<<15) | 14964, (1U<<15) | 21438, (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 21438, 
  (1U<<15) | 13405, 0x2c2c, 0x5959, 0x3b3b, (1U<<15) | 21431, 0x4a4a, 0x7a7a, (1U<<15) | 11954, 
  (1U<<15) | 11954, (1U<<15) | 5032, (1U<<15) | 5032, (1U<<15) | 5474, (1U<<15) | 5474, (1U<<15) | 10892, (1U<<15) | 10892, 0x2c4, 
  0x594, 0x3b4, (1U<<15) | 21416, 0x4a4, 0x2c59, 0x2c4a, (1U<<15) | 15041, 0x3b59, 
  0x3b4a, 0x4a59, 0x2c2c, (1U<<15) | 14340, (1U<<15) | 8406, (1U<<15) | 8406, (1U<<15) | 2585, (1U<<15) | 2585, 
  (1U<<15) | 11843, (1U<<15) | 14793, (1U<<15) | 14793, (1U<<15) | 8413, (1U<<15) | 8413, (1U<<15) | 3280, (1U<<15) | 3280, (1U<<15) | 9950, 
  (1U<<15) | 9950, (1U<<15) | 9950, (1U<<15) | 13311, (1U<<15) | 13311, 0x7a7a, (1U<<15) | 16787, (1U<<15) | 16789, (1U<<15) | 2476, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 3206, 
  (1U<<15) | 3206, (1U<<15) | 16789, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 14964, 
  (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 3206, (1U<<15) | 21420, (1U<<15) | 13271, (1U<<15) | 13287, (1U<<15) | 13287, (1U<<15) | 13271, 
  (1U<<15) | 21420, (1U<<15) | 13287, (1U<<15) | 13287, (1U<<15) | 2516, (1U<<15) | 14999, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 2516, 
  (1U<<15) | 14999, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 14964, (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 13405, (1U<<15) | 2516, 
  (1U<<15) | 14999, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 2516, (1U<<15) | 14999, (1U<<15) | 3244, (1U<<15) | 13414, (1U<<15) | 16787, 
  (1U<<15) | 14964, (1U<<15) | 2605, (1U<<15) | 14964, (1U<<15) | 13376, (1U<<15) | 14927, (1U<<15) | 14927, (1U<<15) | 3199, (1U<<15) | 3199, 
  (1U<<15) | 13376, (1U<<15) | 13376, (1U<<15) | 14927, (1U<<15) | 3199, (1U<<15) | 13376, 0x5959, (1U<<15) | 21431, 0x4a4a, 
  0x7a7a, 0x7a7a, 0x7a7a, 0x7a7a, 0x7a7a, (1U<<15) | 2476, (1U<<15) | 14964, (1U<<15) | 14962, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 21436, (1U<<15) | 21438, (1U<<15) | 13405, (1U<<15) | 13403, (1U<<15) | 13405, 0x7a7a, 
  (1U<<15) | 13403, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 4992, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 13405, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 4992, (1U<<15) | 3208, (1U<<15) | 13405, 
  (1U<<15) | 2476, (1U<<15) | 13405, 0x2c2c, 0x2c44, 0x2c2c, 0x2c44, 0x3b3b, 0x3b44, 
  0x3b3b, 0x3b44, (1U<<15) | 21438, (1U<<15) | 13405, (1U<<15) | 21436, (1U<<15) | 21436, (1U<<15) | 2476, (1U<<15) | 3208, 
  (1U<<15) | 13405, (1U<<15) | 2476, (1U<<15) | 3208, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 13280, (1U<<15) | 13296, (1U<<15) | 13280, 
  (1U<<15) | 13405, 0x3b4a, 0x2c3b, 0x3b4a, 0x4a59, 0x3b4a, 0x2c3b, 0x3b4a, 
  0x4a59, 0x45e5, 0x44e4, (1U<<15) | 19337, (1U<<15) | 19338, (1U<<15) | 14817, (1U<<15) | 14817, (1U<<15) | 14817, 
  (1U<<15) | 14817, (1U<<15) | 14817, (1U<<15) | 14817, (1U<<15) | 14817, (1U<<15) | 4992, (1U<<15) | 2533, (1U<<15) | 2476, (1U<<15) | 4994, 
  (1U<<15) | 4992, (1U<<15) | 2533, 0x555, 0x44, 0x55, 0x44, 0x55, 0xf0, 
  0x555, (1U<<15) | 20155, 0x555, 0x4444, (1U<<15) | 14783, 0x555, 0x555, 0x884, 
  0x884, 0x884, 0x884, (1U<<15) | 21714, (1U<<15) | 21896, 0x5, 0x4, 0x5, 
  0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 
  0xe0, 0x4e0, 0xe0, 0x4e0, 0xe0, 0xe0, 0xe0, 0xe0, 
  0x555, 0x555, (1U<<15) | 21710, 0x444, 0x444, 0x0, 0x84, 0x85, 
  0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 
  0x0, 0x44, (1U<<15) | 21709, (1U<<15) | 18219, 0x7777, 0x88, 0x77, (1U<<15) | 18219, 
  0x7777, (1U<<15) | 17344, 0x88, 0x77, 0x88, 0x77, (1U<<15) | 18219, 0x7777, 
  0x5, 0x5, 0x5, 0x5, 0xe0, 0x588, 0x0, 0x0, 
  0x0, 0x0, 0xe4, 0xe5, 0xe4, 0xe4, 0xe4, 0xe5, 
  0xe4, 0x0, 0x5555, 0x5555, 0x5555, (1U<<15) | 21373, (1U<<15) | 21335, (1U<<15) | 21329, 
  0x8, 0x4, 0x41f, 0x4, (1U<<15) | 21373, (1U<<15) | 21335, (1U<<15) | 21329, (1U<<15) | 2472, 
  (1U<<15) | 2483, (1U<<15) | 1376, (1U<<15) | 1396, (1U<<15) | 1391, (1U<<15) | 1393, (1U<<15) | 4340, (1U<<15) | 4431, (1U<<15) | 1389, 
  (1U<<15) | 2442, (1U<<15) | 2440, (1U<<15) | 2440, (1U<<15) | 2440, (1U<<15) | 2440, (1U<<15) | 2442, (1U<<15) | 2440, (1U<<15) | 2440, 
  (1U<<15) | 2440, (1U<<15) | 2440, (1U<<15) | 2442, (1U<<15) | 2440, (1U<<15) | 2440, (1U<<15) | 4352, (1U<<15) | 4348, (1U<<15) | 4366, 
  (1U<<15) | 4374, (1U<<15) | 8477, (1U<<15) | 8489, (1U<<15) | 8487, (1U<<15) | 8487, (1U<<15) | 8487, (1U<<15) | 8487, (1U<<15) | 8489, 
  (1U<<15) | 8487, (1U<<15) | 8487, (1U<<15) | 8487, (1U<<15) | 8487, (1U<<15) | 8489, (1U<<15) | 8487, (1U<<15) | 8487, (1U<<15) | 8501, 
  (1U<<15) | 8499, (1U<<15) | 8499, (1U<<15) | 8499, (1U<<15) | 8499, (1U<<15) | 8501, (1U<<15) | 8499, (1U<<15) | 8499, (1U<<15) | 8499, 
  (1U<<15) | 8499, (1U<<15) | 8404, (1U<<15) | 8402, (1U<<15) | 8402, (1U<<15) | 8402, (1U<<15) | 8402, (1U<<15) | 8393, (1U<<15) | 8391, 
  (1U<<15) | 8391, (1U<<15) | 8391, (1U<<15) | 8391, (1U<<15) | 8501, (1U<<15) | 8499, (1U<<15) | 8501, (1U<<15) | 8499, (1U<<15) | 8501, 
  (1U<<15) | 8499, (1U<<15) | 8501, (1U<<15) | 8499, (1U<<15) | 8499, (1U<<15) | 2458, (1U<<15) | 2456, (1U<<15) | 2456, (1U<<15) | 2456, 
  (1U<<15) | 2456, (1U<<15) | 2458, (1U<<15) | 2456, (1U<<15) | 2456, (1U<<15) | 2456, (1U<<15) | 2456, (1U<<15) | 2458, (1U<<15) | 2456, 
  (1U<<15) | 2456, (1U<<15) | 2456, (1U<<15) | 2456, (1U<<15) | 2433, (1U<<15) | 2431, (1U<<15) | 2431, (1U<<15) | 2431, (1U<<15) | 2431, 
  (1U<<15) | 2458, (1U<<15) | 2456, (1U<<15) | 2458, (1U<<15) | 2456, (1U<<15) | 2458, (1U<<15) | 2456, (1U<<15) | 2458, (1U<<15) | 2456, 
  (1U<<15) | 2456, (1U<<15) | 1366, (1U<<15) | 1366, (1U<<15) | 1368, 0x40, 0x40, 0x840, 0x440, 
  0x40, 0x1f40, (1U<<15) | 21710, 0x555, 0x555, 0x444, 0x444, (1U<<15) | 4994, 
  (1U<<15) | 18278, 0x2c2c, (1U<<15) | 4994, 0x555, 0x555, (1U<<15) | 18535, 0x8, (1U<<15) | 14635, 
  (1U<<15) | 8570, 0x4444, (1U<<15) | 21680, (1U<<15) | 15314, 0x50, 0x50, 0x50, 0x50, 
  0x555, 0x88, 0x48, (1U<<15) | 21711, 0x4e4, 0x5e4, 0x8e0, 0x4e4, 
  0xe40, 0xe40, 0xe50, 0x4e4, (1U<<15) | 21710, 0x0, 0x44, 0x4444, 
  0x4444, 0x4444, 0x4444, 0x44, 0x4, 0x4550, 0x44, 0x4, 
  0x42f4, 0x40, 0x50, 0x4, 0x44, 0x4, (1U<<15) | 21683, 0x44, 
  0x4, 0x5, 0x4440, (1U<<15) | 14454, (1U<<15) | 2465, (1U<<15) | 1444, 0xe89, 0xe89, 
  0x5e4a, 0x5e4a, (1U<<15) | 19209, (1U<<15) | 15264, (1U<<15) | 15264, 0x5e4a, 0x5e4a, 0xe4a, 
  0xe4a, (1U<<15) | 19369, (1U<<15) | 19369, (1U<<15) | 15287, (1U<<15) | 15287, (1U<<15) | 19208, (1U<<15) | 15263, (1U<<15) | 15263, 
  (1U<<15) | 15287, (1U<<15) | 15287, (1U<<15) | 19298, (1U<<15) | 19298, 0x888, 0x888, (1U<<15) | 18334, (1U<<15) | 18310, 
  (1U<<15) | 16773, (1U<<15) | 16766, (1U<<15) | 18334, (1U<<15) | 18310, (1U<<15) | 16773, (1U<<15) | 16766, (1U<<15) | 18334, (1U<<15) | 18310, 
  (1U<<15) | 16773, (1U<<15) | 16766, 0x2c2c, (1U<<15) | 18305, (1U<<15) | 18282, (1U<<15) | 18282, 0x3b7a, 0x2c2c, 
  0x7a89, 0x7a7a, 0x7a59, 0x7a59, 0x597a, 0x4a89, 0x597a, 0x4a89, 
  (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 14985, (1U<<15) | 13421, (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18350, (1U<<15) | 16789, 
  (1U<<15) | 18312, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 13405, 0x7a7a, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 18311, 
  (1U<<15) | 16767, 0x42c4, 0x894, 0x7a4, (1U<<15) | 12727, (1U<<15) | 12270, (1U<<15) | 18291, 0x7a4a, 
  (1U<<15) | 18291, 0x7a4a, (1U<<15) | 2474, (1U<<15) | 14962, (1U<<15) | 3206, (1U<<15) | 13403, (1U<<15) | 11962, (1U<<15) | 11938, 
  (1U<<15) | 4994, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 10822, (1U<<15) | 11926, (1U<<15) | 13405, (1U<<15) | 4990, (1U<<15) | 4994, 
  0x7a7a, (1U<<15) | 8801, (1U<<15) | 8801, (1U<<15) | 16790, 0x0, (1U<<15) | 17915, 0x70, (1U<<15) | 10821, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 
  0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x4, 0x7f2f, 
  0x7f2f, (1U<<15) | 9430, (1U<<15) | 10652, (1U<<15) | 8801, (1U<<15) | 8801, (1U<<15) | 8801, (1U<<15) | 8801, (1U<<15) | 8774, 
  (1U<<15) | 8801, (1U<<15) | 8801, (1U<<15) | 8774, 0x4444, 0x4444, 0x4444, 0x4444, 0x555, 
  0x555, 0x555, 0x555, 0x55, 0x455, 0x555, 0x7f1f, (1U<<15) | 17279, 
  (1U<<15) | 17279, (1U<<15) | 17279, 0x4444, 0x4444, 0x4444, 0x4444, 0x444, 0x444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x444, 0x4444, 0x444, 0x44, 
  0x444, 0x444, 0x4444, 0xe4, 0x4444, (1U<<15) | 8570, (1U<<15) | 8570, (1U<<15) | 8570, 
  (1U<<15) | 8570, (1U<<15) | 8570, (1U<<15) | 8570, (1U<<15) | 8570, (1U<<15) | 8570, 0x4444, 0x4444, 0x4444, 
  0x4444, 0x4444, 0x4444, 0x4444, 0x4444, 0x4444, (1U<<15) | 17693, (1U<<15) | 17632, 
  (1U<<15) | 17632, (1U<<15) | 17693, (1U<<15) | 17693, (1U<<15) | 17693, (1U<<15) | 17693, (1U<<15) | 17693, (1U<<15) | 17632, 0x0, 
  0x0, 0x0, (1U<<15) | 17279, (1U<<15) | 17333, (1U<<15) | 17279, (1U<<15) | 17279, (1U<<15) | 17279, (1U<<15) | 17279, 
  (1U<<15) | 21563, (1U<<15) | 21489, (1U<<15) | 21563, (1U<<15) | 21489, (1U<<15) | 21563, (1U<<15) | 21489, (1U<<15) | 20640, (1U<<15) | 20627, 
  (1U<<15) | 20740, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 20162, (1U<<15) | 20179, (1U<<15) | 20557, (1U<<15) | 20179, (1U<<15) | 20557, 
  0x7f1f, 0x0, (1U<<15) | 19826, (1U<<15) | 19827, (1U<<15) | 19855, (1U<<15) | 19840, (1U<<15) | 19888, (1U<<15) | 19871, 
  (1U<<15) | 19925, (1U<<15) | 19906, (1U<<15) | 19966, (1U<<15) | 19945, (1U<<15) | 20011, (1U<<15) | 19988, (1U<<15) | 20060, (1U<<15) | 20035, 
  (1U<<15) | 17645, (1U<<15) | 17645, (1U<<15) | 17645, (1U<<15) | 17645, (1U<<15) | 17645, (1U<<15) | 17661, (1U<<15) | 17661, (1U<<15) | 17661, 
  (1U<<15) | 17661, (1U<<15) | 21204, (1U<<15) | 21697, (1U<<15) | 21697, (1U<<15) | 20100, (1U<<15) | 21204, (1U<<15) | 21697, (1U<<15) | 21697, 
  (1U<<15) | 21686, (1U<<15) | 21686, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21227, (1U<<15) | 21227, 
  (1U<<15) | 21686, (1U<<15) | 21686, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21686, (1U<<15) | 21686, 
  (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21227, (1U<<15) | 21227, (1U<<15) | 21686, (1U<<15) | 21686, 
  (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21864, (1U<<15) | 21204, (1U<<15) | 21697, (1U<<15) | 21697, (1U<<15) | 20100, 
  (1U<<15) | 21204, (1U<<15) | 21697, (1U<<15) | 21697, (1U<<15) | 18599, (1U<<15) | 19522, (1U<<15) | 18599, (1U<<15) | 19522, (1U<<15) | 20693, 
  (1U<<15) | 20774, (1U<<15) | 20693, (1U<<15) | 20774, (1U<<15) | 21576, (1U<<15) | 17789, (1U<<15) | 17789, (1U<<15) | 17789, (1U<<15) | 17789, 
  (1U<<15) | 21576, (1U<<15) | 21576, (1U<<15) | 21576, (1U<<15) | 21576, (1U<<15) | 21576, (1U<<15) | 21576, (1U<<15) | 21576, (1U<<15) | 21576, 
  (1U<<15) | 17333, (1U<<15) | 17333, (1U<<15) | 17333, (1U<<15) | 17789, (1U<<15) | 17789, (1U<<15) | 17789, (1U<<15) | 17789, 0x0, 
  (1U<<15) | 18568, (1U<<15) | 19559, (1U<<15) | 17408, 0x44, 0x44, 0x44, 0x44, 0x55, 
  0x444, 0x444, 0x55, 0x444, 0x444, 0x55, 0x444, 0x55, 
  0x444, 0x44, 0x44, 0x4444, 0x4444, (1U<<15) | 20863, (1U<<15) | 20864, (1U<<15) | 20896, 
  (1U<<15) | 20879, (1U<<15) | 20933, (1U<<15) | 20914, (1U<<15) | 20974, (1U<<15) | 20953, (1U<<15) | 21019, (1U<<15) | 20996, (1U<<15) | 21068, 
  (1U<<15) | 21043, (1U<<15) | 21121, (1U<<15) | 21094, (1U<<15) | 21563, (1U<<15) | 21521, (1U<<15) | 21563, (1U<<15) | 21521, (1U<<15) | 21563, 
  (1U<<15) | 21521, (1U<<15) | 21563, (1U<<15) | 21521, (1U<<15) | 12871, (1U<<15) | 13612, 0x7f1f, (1U<<15) | 20627, (1U<<15) | 20740, 
  (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 19801, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20640, (1U<<15) | 19566, (1U<<15) | 20640, 
  (1U<<15) | 19566, (1U<<15) | 20640, (1U<<15) | 19566, (1U<<15) | 20640, (1U<<15) | 19566, (1U<<15) | 20144, (1U<<15) | 20627, (1U<<15) | 20640, 
  (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 20627, (1U<<15) | 20740, 
  (1U<<15) | 18599, (1U<<15) | 19522, (1U<<15) | 18599, (1U<<15) | 19522, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, 
  (1U<<15) | 18599, (1U<<15) | 19522, (1U<<15) | 18520, (1U<<15) | 18569, (1U<<15) | 18590, (1U<<15) | 18599, (1U<<15) | 19522, (1U<<15) | 18599, 
  (1U<<15) | 19522, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 18559, 
  (1U<<15) | 19496, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20162, (1U<<15) | 20543, (1U<<15) | 20162, 
  (1U<<15) | 20543, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 18569, 
  (1U<<15) | 18590, (1U<<15) | 20792, (1U<<15) | 20722, (1U<<15) | 20792, (1U<<15) | 20722, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 19801, 
  (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20792, (1U<<15) | 20722, (1U<<15) | 20792, (1U<<15) | 20722, (1U<<15) | 20627, (1U<<15) | 20740, 
  (1U<<15) | 2822, (1U<<15) | 18628, (1U<<15) | 18628, (1U<<15) | 20640, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20758, (1U<<15) | 20640, 
  (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20758, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20162, 
  (1U<<15) | 20543, (1U<<15) | 20162, (1U<<15) | 20543, (1U<<15) | 20162, (1U<<15) | 20543, (1U<<15) | 20640, (1U<<15) | 20640, (1U<<15) | 20758, 
  (1U<<15) | 20758, (1U<<15) | 20640, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20758, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20640, 
  (1U<<15) | 20758, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20792, (1U<<15) | 20722, (1U<<15) | 20792, (1U<<15) | 20722, (1U<<15) | 20792, 
  (1U<<15) | 20722, (1U<<15) | 20792, (1U<<15) | 20722, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 19566, (1U<<15) | 19651, (1U<<15) | 20116, 
  (1U<<15) | 20086, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 20614, (1U<<15) | 20571, (1U<<15) | 20614, (1U<<15) | 20571, (1U<<15) | 18599, 
  (1U<<15) | 19522, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, 
  (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 19566, (1U<<15) | 19651, (1U<<15) | 20627, 
  (1U<<15) | 20740, (1U<<15) | 21537, (1U<<15) | 21614, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 20162, (1U<<15) | 20162, (1U<<15) | 20543, 
  (1U<<15) | 20543, (1U<<15) | 20162, (1U<<15) | 20543, (1U<<15) | 20162, (1U<<15) | 20543, (1U<<15) | 20162, (1U<<15) | 20543, (1U<<15) | 20162, 
  (1U<<15) | 20543, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20640, (1U<<15) | 20758, (1U<<15) | 20162, (1U<<15) | 20543, (1U<<15) | 21650, 
  (1U<<15) | 21632, (1U<<15) | 21650, (1U<<15) | 21632, (1U<<15) | 21650, (1U<<15) | 21632, (1U<<15) | 21537, (1U<<15) | 21614, (1U<<15) | 21650, 
  (1U<<15) | 21632, (1U<<15) | 21650, (1U<<15) | 21632, (1U<<15) | 20614, (1U<<15) | 20571, (1U<<15) | 20614, (1U<<15) | 20571, (1U<<15) | 21537, 
  (1U<<15) | 21614, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 20627, (1U<<15) | 19566, (1U<<15) | 18592, (1U<<15) | 19510, (1U<<15) | 18510, 
  (1U<<15) | 19464, (1U<<15) | 20179, (1U<<15) | 20557, (1U<<15) | 19755, (1U<<15) | 19633, (1U<<15) | 20172, (1U<<15) | 21248, (1U<<15) | 21505, 
  (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, 
  (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 20682, (1U<<15) | 20706, 
  (1U<<15) | 20671, (1U<<15) | 21665, (1U<<15) | 19740, (1U<<15) | 19721, (1U<<15) | 20671, (1U<<15) | 21665, (1U<<15) | 19740, (1U<<15) | 19721, 
  (1U<<15) | 20671, (1U<<15) | 21665, (1U<<15) | 19740, (1U<<15) | 19721, (1U<<15) | 20671, (1U<<15) | 21665, (1U<<15) | 19740, (1U<<15) | 19721, 
  (1U<<15) | 20671, (1U<<15) | 21665, (1U<<15) | 19740, (1U<<15) | 19721, (1U<<15) | 20671, (1U<<15) | 21665, (1U<<15) | 19740, (1U<<15) | 19721, 
  (1U<<15) | 20671, (1U<<15) | 21665, (1U<<15) | 19740, (1U<<15) | 19721, (1U<<15) | 20819, (1U<<15) | 20832, (1U<<15) | 20819, (1U<<15) | 20832, 
  (1U<<15) | 20819, (1U<<15) | 20832, (1U<<15) | 20819, (1U<<15) | 20832, (1U<<15) | 20819, (1U<<15) | 20832, (1U<<15) | 20819, (1U<<15) | 20832, 
  (1U<<15) | 20819, (1U<<15) | 20832, (1U<<15) | 21248, (1U<<15) | 21505, (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, 
  (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 21601, (1U<<15) | 21731, 
  (1U<<15) | 21601, (1U<<15) | 21731, (1U<<15) | 20601, (1U<<15) | 20511, (1U<<15) | 20134, (1U<<15) | 19788, (1U<<15) | 20601, (1U<<15) | 20511, 
  (1U<<15) | 18599, (1U<<15) | 18599, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 18563, (1U<<15) | 19801, 
  (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, 
  (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, 
  (1U<<15) | 18599, (1U<<15) | 18599, (1U<<15) | 18599, (1U<<15) | 18599, (1U<<15) | 20134, (1U<<15) | 19788, (1U<<15) | 18592, (1U<<15) | 18608, 
  (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 18563, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, 
  (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 18592, (1U<<15) | 18608, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, 
  (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 20134, (1U<<15) | 19785, (1U<<15) | 18592, 
  (1U<<15) | 18608, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, 
  (1U<<15) | 20527, (1U<<15) | 18619, (1U<<15) | 18599, (1U<<15) | 18619, (1U<<15) | 1268, (1U<<15) | 18599, (1U<<15) | 18599, (1U<<15) | 21537, 
  (1U<<15) | 21614, (1U<<15) | 21537, (1U<<15) | 21614, (1U<<15) | 20601, (1U<<15) | 20511, (1U<<15) | 20601, (1U<<15) | 20511, (1U<<15) | 21216, 
  (1U<<15) | 21457, (1U<<15) | 21216, (1U<<15) | 21457, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 21537, (1U<<15) | 21457, (1U<<15) | 21537, 
  (1U<<15) | 21457, (1U<<15) | 21537, (1U<<15) | 21457, (1U<<15) | 21550, (1U<<15) | 21473, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 20116, 
  (1U<<15) | 20086, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 20116, 
  (1U<<15) | 20086, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, 
  (1U<<15) | 20527, (1U<<15) | 18599, (1U<<15) | 19522, (1U<<15) | 18637, (1U<<15) | 19536, (1U<<15) | 19566, (1U<<15) | 19617, (1U<<15) | 18547, 
  (1U<<15) | 19479, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, 
  (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 19801, (1U<<15) | 20171, (1U<<15) | 19815, (1U<<15) | 17333, (1U<<15) | 17279, 
  (1U<<15) | 20162, (1U<<15) | 20587, (1U<<15) | 20627, (1U<<15) | 20627, (1U<<15) | 20627, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, 
  (1U<<15) | 20527, (1U<<15) | 19667, (1U<<15) | 19617, (1U<<15) | 19667, (1U<<15) | 19617, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20171, 
  (1U<<15) | 20627, (1U<<15) | 20144, (1U<<15) | 20144, (1U<<15) | 20640, (1U<<15) | 19566, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 21238, 
  (1U<<15) | 21149, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, 
  (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 20144, 
  (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20661, (1U<<15) | 20498, (1U<<15) | 20651, (1U<<15) | 21589, (1U<<15) | 20651, 
  (1U<<15) | 21589, (1U<<15) | 20651, (1U<<15) | 21589, (1U<<15) | 20651, (1U<<15) | 21589, (1U<<15) | 20651, (1U<<15) | 21589, (1U<<15) | 20651, 
  (1U<<15) | 21589, (1U<<15) | 20651, (1U<<15) | 21589, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 20627, (1U<<15) | 20740, (1U<<15) | 20807, 
  (1U<<15) | 20849, (1U<<15) | 20807, (1U<<15) | 20849, (1U<<15) | 20807, (1U<<15) | 20849, (1U<<15) | 20807, (1U<<15) | 20849, (1U<<15) | 20807, 
  (1U<<15) | 20849, (1U<<15) | 20807, (1U<<15) | 20849, (1U<<15) | 20807, (1U<<15) | 20849, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, 
  (1U<<15) | 20527, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 21238, (1U<<15) | 21149, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, 
  (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21589, 
  (1U<<15) | 21717, (1U<<15) | 21589, (1U<<15) | 21717, (1U<<15) | 21216, (1U<<15) | 21457, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 21216, 
  (1U<<15) | 21457, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 21563, (1U<<15) | 21489, (1U<<15) | 21563, (1U<<15) | 21489, (1U<<15) | 21563, 
  (1U<<15) | 21489, (1U<<15) | 21563, (1U<<15) | 21489, (1U<<15) | 21216, (1U<<15) | 21457, (1U<<15) | 21216, (1U<<15) | 21457, (1U<<15) | 21216, 
  (1U<<15) | 21457, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 20116, (1U<<15) | 20086, (1U<<15) | 21216, (1U<<15) | 21457, (1U<<15) | 21216, 
  (1U<<15) | 21457, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 21216, (1U<<15) | 21457, (1U<<15) | 20144, (1U<<15) | 20527, (1U<<15) | 20144, 
  (1U<<15) | 20527, (1U<<15) | 20162, (1U<<15) | 20587, (1U<<15) | 17279, (1U<<15) | 17279, 0x7f1f, 0x555, 0x555, 
  0x4, 0x4, 0x4e4, 0xe50, 0x40, 0x40, 0x50, 0x4e4, 
  0x4e4, 0x4e0, 0x52f4, 0x4, (1U<<15) | 2476, (1U<<15) | 21452, (1U<<15) | 13405, (1U<<15) | 14964, 
  (1U<<15) | 3208, (1U<<15) | 21453, (1U<<15) | 21452, (1U<<15) | 21453, (1U<<15) | 2476, (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 3208, 
  (1U<<15) | 2476, (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 21453, (1U<<15) | 21453, (1U<<15) | 2549, (1U<<15) | 2523, 
  (1U<<15) | 13393, (1U<<15) | 14894, (1U<<15) | 3225, (1U<<15) | 21445, (1U<<15) | 5444, (1U<<15) | 2523, (1U<<15) | 13393, (1U<<15) | 14894, 
  (1U<<15) | 3225, (1U<<15) | 2523, (1U<<15) | 13393, (1U<<15) | 14894, (1U<<15) | 3225, (1U<<15) | 21445, (1U<<15) | 21445, (1U<<15) | 13405, 
  (1U<<15) | 5488, (1U<<15) | 5488, (1U<<15) | 5444, (1U<<15) | 12331, (1U<<15) | 4990, (1U<<15) | 10871, (1U<<15) | 11962, (1U<<15) | 5440, 
  (1U<<15) | 4990, (1U<<15) | 4992, (1U<<15) | 5021, (1U<<15) | 10863, (1U<<15) | 10847, (1U<<15) | 5442, (1U<<15) | 5463, (1U<<15) | 4992, 
  (1U<<15) | 5021, (1U<<15) | 10863, (1U<<15) | 10847, (1U<<15) | 5442, (1U<<15) | 5463, (1U<<15) | 18317, (1U<<15) | 16763, (1U<<15) | 18317, 
  (1U<<15) | 18317, (1U<<15) | 16763, (1U<<15) | 16763, (1U<<15) | 2476, (1U<<15) | 2523, (1U<<15) | 13405, (1U<<15) | 13393, (1U<<15) | 3208, 
  (1U<<15) | 3225, (1U<<15) | 2476, (1U<<15) | 2523, (1U<<15) | 13405, (1U<<15) | 13393, (1U<<15) | 3208, (1U<<15) | 3225, (1U<<15) | 2476, 
  (1U<<15) | 2523, (1U<<15) | 13405, (1U<<15) | 13393, (1U<<15) | 3208, (1U<<15) | 3225, (1U<<15) | 2476, (1U<<15) | 2523, (1U<<15) | 13405, 
  (1U<<15) | 13393, (1U<<15) | 3208, (1U<<15) | 3225, (1U<<15) | 10774, (1U<<15) | 10659, (1U<<15) | 12742, (1U<<15) | 12339, (1U<<15) | 12742, 
  (1U<<15) | 12339, (1U<<15) | 12695, (1U<<15) | 12257, 0x3b2c, 0x2c4a, 0x2c59, 0x2c3b, (1U<<15) | 2621, 
  (1U<<15) | 3190, (1U<<15) | 14864, (1U<<15) | 21409, (1U<<15) | 13287, (1U<<15) | 2516, (1U<<15) | 13414, (1U<<15) | 15000, (1U<<15) | 3244, 
  0x2c2c, (1U<<15) | 2577, 0x4a4a, (1U<<15) | 13303, 0x3b3b, (1U<<15) | 3272, 0x4e2c, (1U<<15) | 19235, 
  (1U<<15) | 19235, (1U<<15) | 3190, (1U<<15) | 14864, (1U<<15) | 21409, (1U<<15) | 13287, (1U<<15) | 2474, (1U<<15) | 13403, (1U<<15) | 14962, 
  (1U<<15) | 3206, (1U<<15) | 21452, (1U<<15) | 3190, (1U<<15) | 14864, (1U<<15) | 21409, (1U<<15) | 13287, (1U<<15) | 2474, (1U<<15) | 13403, 
  (1U<<15) | 14962, (1U<<15) | 3206, (1U<<15) | 21452, (1U<<15) | 3190, (1U<<15) | 14864, (1U<<15) | 21409, (1U<<15) | 13287, (1U<<15) | 3190, 
  (1U<<15) | 14864, (1U<<15) | 21409, (1U<<15) | 13287, (1U<<15) | 2516, (1U<<15) | 13414, (1U<<15) | 15000, (1U<<15) | 3244, (1U<<15) | 2476, 
  (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 21453, (1U<<15) | 2516, (1U<<15) | 13414, (1U<<15) | 15000, (1U<<15) | 3244, 
  (1U<<15) | 2476, (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 21453, (1U<<15) | 2516, (1U<<15) | 13414, (1U<<15) | 15000, 
  (1U<<15) | 3244, (1U<<15) | 2516, (1U<<15) | 13414, (1U<<15) | 15000, (1U<<15) | 3244, (1U<<15) | 14332, (1U<<15) | 11954, (1U<<15) | 2474, 
  (1U<<15) | 13376, (1U<<15) | 13383, (1U<<15) | 14927, (1U<<15) | 14884, (1U<<15) | 3199, (1U<<15) | 3215, (1U<<15) | 13376, (1U<<15) | 13383, 
  (1U<<15) | 14927, (1U<<15) | 14884, (1U<<15) | 3199, (1U<<15) | 3215, (1U<<15) | 21452, (1U<<15) | 21452, (1U<<15) | 2476, (1U<<15) | 13405, 
  (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 21453, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 4992, (1U<<15) | 4992, (1U<<15) | 21453, 
  (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 4992, (1U<<15) | 2476, (1U<<15) | 2476, (1U<<15) | 19234, (1U<<15) | 4990, (1U<<15) | 5000, 
  (1U<<15) | 10871, (1U<<15) | 10858, (1U<<15) | 5440, (1U<<15) | 5450, (1U<<15) | 4990, (1U<<15) | 5000, (1U<<15) | 10871, (1U<<15) | 10858, 
  (1U<<15) | 5440, (1U<<15) | 5450, (1U<<15) | 19234, (1U<<15) | 2502, (1U<<15) | 2593, (1U<<15) | 2565, (1U<<15) | 2502, (1U<<15) | 2593, 
  (1U<<15) | 2565, (1U<<15) | 2542, (1U<<15) | 13414, (1U<<15) | 3251, (1U<<15) | 3244, (1U<<15) | 13435, (1U<<15) | 15000, (1U<<15) | 2527, 
  0x2c3b, 0x4a59, (1U<<15) | 15042, 0x3b4a, 0x2c3b, 0x4a59, (1U<<15) | 15042, 0x2c3b, 
  0x4a59, (1U<<15) | 15042, 0x3b4a, 0x3b4a, 0x2c3b, 0x4a59, (1U<<15) | 15042, 0x3b4a, 
  0xf1, 0x20f, (1U<<15) | 2384, 0xf1, (1U<<15) | 20224, (1U<<15) | 20215, (1U<<15) | 20215, (1U<<15) | 20215, 
  (1U<<15) | 14314, (1U<<15) | 20215, 0x10, (1U<<15) | 18485, (1U<<15) | 21265, (1U<<15) | 21260, (1U<<15) | 17288, 0x7f2f, 
  0x7f2f, 0x7f2f, 0x7f2f, 0x7f2f, 0x7f2f, 0x2f7f, 0x0, (1U<<15) | 17135, 
  0x4444, 0x4444, (1U<<15) | 17279, (1U<<15) | 20127, (1U<<15) | 21290, (1U<<15) | 17286, (1U<<15) | 17947, (1U<<15) | 1256, 
  (1U<<15) | 1256, (1U<<15) | 1256, 0x4, 0x2f7f, 0x7f2f, (1U<<15) | 14327, (1U<<15) | 21341, 0x41f, 
  0x41f, 0x41f, 0x0, (1U<<15) | 18484, (1U<<15) | 21368, (1U<<15) | 21195, (1U<<15) | 21264, (1U<<15) | 2807, 
  (1U<<15) | 2807, (1U<<15) | 2822, (1U<<15) | 17286, 0x4f50, 0x4f50, (1U<<15) | 1584, (1U<<15) | 21261, (1U<<15) | 17344, 
  0x2f7f, 0x4, 0x41f, (1U<<15) | 14320, (1U<<15) | 18505, 0x2f7f, (1U<<15) | 17128, (1U<<15) | 18660, 
  (1U<<15) | 10781, (1U<<15) | 10781, 0xf4, (1U<<15) | 12877, (1U<<15) | 19246, (1U<<15) | 19246, (1U<<15) | 12856, 0x44, 
  (1U<<15) | 18721, 0x20f4, 0x2f7f, 0x7f2f, (1U<<15) | 17333, (1U<<15) | 17938, 0x40f0, (1U<<15) | 6, 
  (1U<<15) | 17286, (1U<<15) | 2815, (1U<<15) | 1662, 0x4, 0x4, 0x4, 0x4, (1U<<15) | 21259, 
  0x41f, 0x41f, (1U<<15) | 20289, (1U<<15) | 17333, (1U<<15) | 17938, 0x4, 0x42f, 0x0, 
  0xf0, (1U<<15) | 20217, 0x14, 0x11, 0x11, 0x14a, 0x4, 0x1, 
  (1U<<15) | 12440, 0x7f0f, 0x7f0f, 0x7f0f, 0x7f0f, 0x7f0f, 0x41f, (1U<<15) | 1364, 
  (1U<<15) | 1437, (1U<<15) | 1364, (1U<<15) | 1437, (1U<<15) | 1371, (1U<<15) | 1371, 0x40, 0x0, 0x40, 
  0x455, (1U<<15) | 1430, (1U<<15) | 1430, 0x555, (1U<<15) | 14648, (1U<<15) | 14697, (1U<<15) | 14704, (1U<<15) | 13128, 
  (1U<<15) | 13099, (1U<<15) | 13119, (1U<<15) | 4390, (1U<<15) | 1366, (1U<<15) | 1439, (1U<<15) | 1364, (1U<<15) | 1437, (1U<<15) | 1364, 
  (1U<<15) | 1437, 0xe5, 0xee5, (1U<<15) | 4390, 0x4e50, 0x4e50, (1U<<15) | 12907, (1U<<15) | 13108, 
  (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, 
  (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, 
  (1U<<15) | 12885, (1U<<15) | 11902, (1U<<15) | 13108, (1U<<15) | 12921, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, 
  (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, 
  (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, 
  (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 12921, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 12921, (1U<<15) | 13136, (1U<<15) | 13178, 
  (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, 
  (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, 
  (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12918, (1U<<15) | 13144, (1U<<15) | 13175, (1U<<15) | 12905, (1U<<15) | 13106, (1U<<15) | 13142, 
  (1U<<15) | 12931, (1U<<15) | 13210, (1U<<15) | 13208, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, 
  (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 4317, 
  (1U<<15) | 4317, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, 
  (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, 
  (1U<<15) | 12888, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, 
  (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, 
  (1U<<15) | 12888, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, 
  (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, 
  (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, 
  (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 4313, (1U<<15) | 12888, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, 
  (1U<<15) | 4395, (1U<<15) | 12918, (1U<<15) | 13144, (1U<<15) | 13175, (1U<<15) | 12905, (1U<<15) | 13106, (1U<<15) | 13142, (1U<<15) | 12931, 
  (1U<<15) | 13210, (1U<<15) | 13208, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, 
  (1U<<15) | 12918, (1U<<15) | 13144, (1U<<15) | 13175, (1U<<15) | 12905, (1U<<15) | 13106, (1U<<15) | 13142, (1U<<15) | 12931, (1U<<15) | 13210, 
  (1U<<15) | 13208, (1U<<15) | 12918, (1U<<15) | 13144, (1U<<15) | 13175, (1U<<15) | 12905, (1U<<15) | 13106, (1U<<15) | 13142, (1U<<15) | 12931, 
  (1U<<15) | 13210, (1U<<15) | 13208, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, 
  (1U<<15) | 12921, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, 
  (1U<<15) | 13178, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, 
  (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, 
  (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12921, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, 
  (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, 
  (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 12890, (1U<<15) | 13136, (1U<<15) | 12890, (1U<<15) | 13136, (1U<<15) | 12890, (1U<<15) | 13136, 
  (1U<<15) | 12883, (1U<<15) | 11900, (1U<<15) | 13106, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12883, (1U<<15) | 11900, 
  (1U<<15) | 13106, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12883, (1U<<15) | 11900, (1U<<15) | 13106, (1U<<15) | 12933, 
  (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12883, (1U<<15) | 11900, (1U<<15) | 13106, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, 
  (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, 
  (1U<<15) | 13144, (1U<<15) | 12933, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 12907, (1U<<15) | 13108, (1U<<15) | 13144, (1U<<15) | 12933, 
  (1U<<15) | 13178, (1U<<15) | 13210, 0x5550, (1U<<15) | 14552, (1U<<15) | 14557, 0x0, (1U<<15) | 4390, 0x5555, 
  (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, 
  (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, 
  (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, 
  (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13108, (1U<<15) | 13008, 
  (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 12684, (1U<<15) | 12988, (1U<<15) | 13126, (1U<<15) | 11902, 
  (1U<<15) | 12970, (1U<<15) | 13108, (1U<<15) | 12234, (1U<<15) | 12979, (1U<<15) | 13117, (1U<<15) | 10806, (1U<<15) | 12947, (1U<<15) | 13087, 
  (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, 
  (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, 
  (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, 
  (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, 
  (1U<<15) | 13210, (1U<<15) | 13048, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, 
  (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, 
  (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, 
  (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, 
  (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13136, 
  (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 11900, (1U<<15) | 12968, (1U<<15) | 13106, 
  (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 10804, (1U<<15) | 12945, 
  (1U<<15) | 13085, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 10804, 
  (1U<<15) | 12945, (1U<<15) | 13085, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, 
  (1U<<15) | 11900, (1U<<15) | 12968, (1U<<15) | 13106, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, 
  (1U<<15) | 13133, (1U<<15) | 10804, (1U<<15) | 12945, (1U<<15) | 13085, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13108, 
  (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13048, (1U<<15) | 13126, (1U<<15) | 13034, 
  (1U<<15) | 13166, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13117, (1U<<15) | 13021, (1U<<15) | 13155, (1U<<15) | 13178, 
  (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13126, (1U<<15) | 13034, (1U<<15) | 13166, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, 
  (1U<<15) | 13117, (1U<<15) | 13021, (1U<<15) | 13155, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13126, (1U<<15) | 13034, 
  (1U<<15) | 13166, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13117, (1U<<15) | 13021, (1U<<15) | 13155, (1U<<15) | 13178, 
  (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13166, (1U<<15) | 13058, (1U<<15) | 13197, (1U<<15) | 13124, (1U<<15) | 13032, (1U<<15) | 13164, 
  (1U<<15) | 13210, (1U<<15) | 13071, (1U<<15) | 13208, (1U<<15) | 13155, (1U<<15) | 13045, (1U<<15) | 13186, (1U<<15) | 13115, (1U<<15) | 13019, 
  (1U<<15) | 13153, (1U<<15) | 13210, (1U<<15) | 13071, (1U<<15) | 13208, (1U<<15) | 13126, (1U<<15) | 13034, (1U<<15) | 13166, (1U<<15) | 13178, 
  (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13117, (1U<<15) | 13021, (1U<<15) | 13155, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, 
  (1U<<15) | 13126, (1U<<15) | 13034, (1U<<15) | 13166, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13117, (1U<<15) | 13021, 
  (1U<<15) | 13155, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 4370, (1U<<15) | 4370, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, 
  (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 12950, (1U<<15) | 4395, (1U<<15) | 13166, (1U<<15) | 13058, 
  (1U<<15) | 13197, (1U<<15) | 13124, (1U<<15) | 13032, (1U<<15) | 13164, (1U<<15) | 13210, (1U<<15) | 13071, (1U<<15) | 13208, (1U<<15) | 13155, 
  (1U<<15) | 13045, (1U<<15) | 13186, (1U<<15) | 13115, (1U<<15) | 13019, (1U<<15) | 13153, (1U<<15) | 13210, (1U<<15) | 13071, (1U<<15) | 13208, 
  (1U<<15) | 13126, (1U<<15) | 13034, (1U<<15) | 13166, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13117, (1U<<15) | 13021, 
  (1U<<15) | 13155, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13166, (1U<<15) | 13058, (1U<<15) | 13197, (1U<<15) | 13124, 
  (1U<<15) | 13032, (1U<<15) | 13164, (1U<<15) | 13210, (1U<<15) | 13071, (1U<<15) | 13208, (1U<<15) | 13155, (1U<<15) | 13045, (1U<<15) | 13186, 
  (1U<<15) | 13115, (1U<<15) | 13019, (1U<<15) | 13153, (1U<<15) | 13210, (1U<<15) | 13071, (1U<<15) | 13208, (1U<<15) | 13166, (1U<<15) | 13058, 
  (1U<<15) | 13197, (1U<<15) | 13124, (1U<<15) | 13032, (1U<<15) | 13164, (1U<<15) | 13210, (1U<<15) | 13071, (1U<<15) | 13208, (1U<<15) | 13155, 
  (1U<<15) | 13045, (1U<<15) | 13186, (1U<<15) | 13115, (1U<<15) | 13019, (1U<<15) | 13153, (1U<<15) | 13210, (1U<<15) | 13071, (1U<<15) | 13208, 
  (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, 
  (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, 
  (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13126, (1U<<15) | 13034, (1U<<15) | 13166, (1U<<15) | 13178, 
  (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13117, (1U<<15) | 13021, (1U<<15) | 13155, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, 
  (1U<<15) | 13136, (1U<<15) | 4414, (1U<<15) | 13136, (1U<<15) | 4414, (1U<<15) | 11835, (1U<<15) | 4385, (1U<<15) | 12956, (1U<<15) | 13094, 
  (1U<<15) | 11835, (1U<<15) | 4385, (1U<<15) | 12956, (1U<<15) | 13094, (1U<<15) | 11835, (1U<<15) | 4385, (1U<<15) | 12956, (1U<<15) | 13094, 
  (1U<<15) | 11835, (1U<<15) | 4385, (1U<<15) | 12956, (1U<<15) | 13094, (1U<<15) | 11835, (1U<<15) | 4385, (1U<<15) | 12956, (1U<<15) | 13094, 
  (1U<<15) | 11835, (1U<<15) | 4385, (1U<<15) | 12956, (1U<<15) | 13094, (1U<<15) | 11835, (1U<<15) | 4385, (1U<<15) | 12956, (1U<<15) | 13094, 
  (1U<<15) | 11835, (1U<<15) | 4385, (1U<<15) | 12956, (1U<<15) | 13094, (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, 
  (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, 
  (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, 
  (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, 
  (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 14057, (1U<<15) | 13220, (1U<<15) | 13136, (1U<<15) | 13048, (1U<<15) | 13178, (1U<<15) | 13108, 
  (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, 
  (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, 
  (1U<<15) | 13210, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, 
  (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, 
  (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 4403, (1U<<15) | 13008, (1U<<15) | 4412, (1U<<15) | 13073, (1U<<15) | 4321, 
  (1U<<15) | 12894, (1U<<15) | 4330, (1U<<15) | 12933, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, 
  (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, 
  (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, 
  (1U<<15) | 13210, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, 
  (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, 
  (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13136, (1U<<15) | 13048, 
  (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 4414, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, 
  (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, 
  (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, 
  (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, 
  (1U<<15) | 4414, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 13178, (1U<<15) | 13136, 
  (1U<<15) | 13178, (1U<<15) | 13136, (1U<<15) | 4414, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 11834, 
  (1U<<15) | 4384, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 11834, 
  (1U<<15) | 4384, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 11834, 
  (1U<<15) | 4384, (1U<<15) | 11834, (1U<<15) | 4384, (1U<<15) | 12890, (1U<<15) | 13136, (1U<<15) | 11835, (1U<<15) | 12956, (1U<<15) | 13094, 
  (1U<<15) | 11898, (1U<<15) | 13104, (1U<<15) | 11900, (1U<<15) | 12968, (1U<<15) | 13106, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, 
  (1U<<15) | 10804, (1U<<15) | 12945, (1U<<15) | 13085, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 10804, (1U<<15) | 12945, 
  (1U<<15) | 13085, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 11900, (1U<<15) | 12968, (1U<<15) | 13106, (1U<<15) | 13178, 
  (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 11900, (1U<<15) | 12968, (1U<<15) | 13106, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, 
  (1U<<15) | 10804, (1U<<15) | 12945, (1U<<15) | 13085, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 10804, (1U<<15) | 12945, 
  (1U<<15) | 13085, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 11900, (1U<<15) | 12968, (1U<<15) | 13106, (1U<<15) | 13178, 
  (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, 
  (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, 
  (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, 
  (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, 
  (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, 
  (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 14056, (1U<<15) | 4422, 
  (1U<<15) | 14056, (1U<<15) | 4422, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, 
  (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, 
  (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13108, (1U<<15) | 13008, (1U<<15) | 13144, (1U<<15) | 13178, 
  (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 13087, (1U<<15) | 12997, (1U<<15) | 13133, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, 
  (1U<<15) | 13136, (1U<<15) | 4414, (1U<<15) | 13136, (1U<<15) | 4414, (1U<<15) | 13136, (1U<<15) | 4414, (1U<<15) | 13108, (1U<<15) | 13008, 
  (1U<<15) | 13144, (1U<<15) | 13178, (1U<<15) | 13073, (1U<<15) | 13210, (1U<<15) | 1364, (1U<<15) | 1437, 0x3f4, 0x3f4, 
  (1U<<15) | 17297, 0x3f4, (1U<<15) | 17355, 0x4e, (1U<<15) | 3244, (1U<<15) | 17975, (1U<<15) | 17975, 0x46b7, 
  (1U<<15) | 20202, (1U<<15) | 20205, (1U<<15) | 14306, 0xe7, 0xe, 0x4e4, 0x54e4, 0x55e4, 
  (1U<<15) | 17230, 0x41f, (1U<<15) | 19584, (1U<<15) | 19584, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 3208, 0x4, 
  (1U<<15) | 21902, (1U<<15) | 21908, 0x0, (1U<<15) | 21903, (1U<<15) | 21909, (1U<<15) | 21354, (1U<<15) | 16731, (1U<<15) | 13271, 
  (1U<<15) | 2516, (1U<<15) | 17355, (1U<<15) | 17355, (1U<<15) | 17297, (1U<<15) | 17297, (1U<<15) | 17355, (1U<<15) | 3208, (1U<<15) | 2476, 
  0x7a4a, (1U<<15) | 18282, 0x7a4a, (1U<<15) | 18282, (1U<<15) | 15787, 0x0, (1U<<15) | 8683, 0x76b, 
  0xe70, (1U<<15) | 2476, (1U<<15) | 8468, (1U<<15) | 5909, (1U<<15) | 14464, (1U<<15) | 14477, (1U<<15) | 4287, (1U<<15) | 4303, 
  (1U<<15) | 4308, (1U<<15) | 4298, (1U<<15) | 14458, (1U<<15) | 14471, (1U<<15) | 4286, (1U<<15) | 21899, (1U<<15) | 21905, (1U<<15) | 1346, 
  0xe40, 0x1f, 0xe, 0x1f, (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 18648, (1U<<15) | 18648, 
  0x4e0, 0x5e0, 0x4e0, 0x5e0, (1U<<15) | 8347, (1U<<15) | 14711, (1U<<15) | 19360, (1U<<15) | 19360, 
  (1U<<15) | 14934, (1U<<15) | 14934, (1U<<15) | 19360, (1U<<15) | 19360, (1U<<15) | 14934, (1U<<15) | 14934, (1U<<15) | 14964, (1U<<15) | 15162, 
  (1U<<15) | 15215, (1U<<15) | 14964, (1U<<15) | 15162, (1U<<15) | 15215, (1U<<15) | 14964, (1U<<15) | 15162, (1U<<15) | 15215, (1U<<15) | 14964, 
  (1U<<15) | 15162, (1U<<15) | 15215, 0x5959, (1U<<15) | 1920, 0x4e0, 0x5e0, (1U<<15) | 4266, (1U<<15) | 4266, 
  0x2e1f, 0x1fe2, 0x2e1f, 0x1fe2, 0x2e1f, 0x1fe2, (1U<<15) | 4266, (1U<<15) | 4266, 
  (1U<<15) | 4266, (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18466, (1U<<15) | 16934, (1U<<15) | 2368, (1U<<15) | 2266, (1U<<15) | 18435, 
  (1U<<15) | 18414, 0x7b4b, (1U<<15) | 18414, 0x7b4b, (1U<<15) | 2266, (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18468, 
  (1U<<15) | 16936, 0xe2d, (1U<<15) | 15035, (1U<<15) | 15196, (1U<<15) | 13468, (1U<<15) | 13586, (1U<<15) | 18296, (1U<<15) | 18428, 
  (1U<<15) | 16749, (1U<<15) | 16913, (1U<<15) | 18468, (1U<<15) | 16936, (1U<<15) | 18468, (1U<<15) | 16936, 0x8a4, 0x7b4, 
  (1U<<15) | 15154, (1U<<15) | 15154, (1U<<15) | 15154, 0x7b7b, (1U<<15) | 12796, (1U<<15) | 12399, 0x7b7b, (1U<<15) | 15022, 
  (1U<<15) | 15183, (1U<<15) | 13455, (1U<<15) | 13573, (1U<<15) | 18311, (1U<<15) | 18446, (1U<<15) | 16767, (1U<<15) | 16928, (1U<<15) | 18311, 
  (1U<<15) | 18446, (1U<<15) | 16767, (1U<<15) | 16928, (1U<<15) | 18311, (1U<<15) | 18446, (1U<<15) | 16767, (1U<<15) | 16928, 0x0, 
  0x0, (1U<<15) | 14359, (1U<<15) | 14387, (1U<<15) | 14401, (1U<<15) | 2882, (1U<<15) | 3605, (1U<<15) | 14418, (1U<<15) | 2871, 
  (1U<<15) | 3594, (1U<<15) | 14407, (1U<<15) | 2882, (1U<<15) | 3605, (1U<<15) | 14418, (1U<<15) | 2871, (1U<<15) | 3594, (1U<<15) | 14407, 
  (1U<<15) | 2871, (1U<<15) | 3594, (1U<<15) | 14407, (1U<<15) | 2882, (1U<<15) | 3605, (1U<<15) | 14418, (1U<<15) | 2880, (1U<<15) | 3603, 
  (1U<<15) | 14416, (1U<<15) | 2122, (1U<<15) | 3132, (1U<<15) | 10468, (1U<<15) | 1542, (1U<<15) | 2936, (1U<<15) | 5299, (1U<<15) | 1542, 
  (1U<<15) | 2936, (1U<<15) | 5299, (1U<<15) | 1542, (1U<<15) | 2936, (1U<<15) | 5299, (1U<<15) | 1542, (1U<<15) | 2936, (1U<<15) | 5299, 
  (1U<<15) | 2061, (1U<<15) | 3103, (1U<<15) | 12127, (1U<<15) | 1544, (1U<<15) | 2928, (1U<<15) | 5291, (1U<<15) | 1544, (1U<<15) | 2928, 
  (1U<<15) | 5291, (1U<<15) | 1544, (1U<<15) | 2928, (1U<<15) | 5291, (1U<<15) | 1544, (1U<<15) | 2928, (1U<<15) | 5291, (1U<<15) | 1648, 
  (1U<<15) | 3061, (1U<<15) | 8420, (1U<<15) | 1648, (1U<<15) | 3061, (1U<<15) | 8420, (1U<<15) | 1816, (1U<<15) | 1883, (1U<<15) | 5355, 
  (1U<<15) | 1816, (1U<<15) | 1883, (1U<<15) | 5355, (1U<<15) | 1834, (1U<<15) | 1842, (1U<<15) | 4568, (1U<<15) | 1953, (1U<<15) | 2023, 
  (1U<<15) | 4715, (1U<<15) | 1834, (1U<<15) | 1842, (1U<<15) | 4568, (1U<<15) | 1953, (1U<<15) | 2023, (1U<<15) | 4715, (1U<<15) | 1648, 
  (1U<<15) | 3061, (1U<<15) | 8420, (1U<<15) | 1648, (1U<<15) | 3061, (1U<<15) | 8420, (1U<<15) | 1816, (1U<<15) | 1883, (1U<<15) | 5355, 
  (1U<<15) | 1816, (1U<<15) | 1883, (1U<<15) | 5355, (1U<<15) | 1816, (1U<<15) | 1883, (1U<<15) | 5355, (1U<<15) | 1945, (1U<<15) | 2015, 
  (1U<<15) | 4706, (1U<<15) | 1816, (1U<<15) | 1883, (1U<<15) | 5355, (1U<<15) | 1945, (1U<<15) | 2015, (1U<<15) | 4706, (1U<<15) | 4917, 
  (1U<<15) | 2274, (1U<<15) | 2325, (1U<<15) | 10445, (1U<<15) | 2069, (1U<<15) | 3111, (1U<<15) | 5402, (1U<<15) | 2156, (1U<<15) | 2234, 
  (1U<<15) | 4872, (1U<<15) | 4764, (1U<<15) | 4818, (1U<<15) | 21799, (1U<<15) | 21822, (1U<<15) | 21845, (1U<<15) | 12794, (1U<<15) | 12113, 
  (1U<<15) | 12397, (1U<<15) | 21782, (1U<<15) | 21782, (1U<<15) | 21782, (1U<<15) | 21782, (1U<<15) | 21782, (1U<<15) | 21782, (1U<<15) | 15592, 
  (1U<<15) | 15669, (1U<<15) | 15694, (1U<<15) | 15592, (1U<<15) | 15669, (1U<<15) | 15694, (1U<<15) | 15592, (1U<<15) | 15669, (1U<<15) | 15694, 
  (1U<<15) | 15592, (1U<<15) | 15669, (1U<<15) | 15694, (1U<<15) | 21764, (1U<<15) | 21806, (1U<<15) | 21829, (1U<<15) | 21764, (1U<<15) | 21806, 
  (1U<<15) | 21829, (1U<<15) | 21764, (1U<<15) | 21806, (1U<<15) | 21829, (1U<<15) | 21764, (1U<<15) | 21806, (1U<<15) | 21829, 0x4894, 
  0x4895, 0x4894, 0x4895, 0x47a4, 0x47a5, 0x47a4, 0x47a5, (1U<<15) | 21799, 
  (1U<<15) | 21822, (1U<<15) | 21845, (1U<<15) | 15608, (1U<<15) | 15685, (1U<<15) | 15701, (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, 
  (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, 
  (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, (1U<<15) | 21797, (1U<<15) | 21820, 
  (1U<<15) | 21843, (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, (1U<<15) | 21797, 
  (1U<<15) | 21820, (1U<<15) | 21843, (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, (1U<<15) | 21797, (1U<<15) | 21820, (1U<<15) | 21843, 
  (1U<<15) | 21799, (1U<<15) | 21822, (1U<<15) | 21845, (1U<<15) | 21799, (1U<<15) | 21822, (1U<<15) | 21845, (1U<<15) | 14379, (1U<<15) | 14393, 
  (1U<<15) | 14418, (1U<<15) | 12742, (1U<<15) | 12794, (1U<<15) | 12093, (1U<<15) | 12113, (1U<<15) | 12339, (1U<<15) | 12397, 0xe2c, 
  0xe2d, (1U<<15) | 19225, 0xe4a, 0xe4b, 0xe4c, 0xe59, 0xe5a, 0xe5b, 
  0xe3b, 0xe3c, 0xe3d, (1U<<15) | 1576, (1U<<15) | 21799, (1U<<15) | 21822, (1U<<15) | 21845, (1U<<15) | 2850, 
  (1U<<15) | 2850, (1U<<15) | 2850, (1U<<15) | 2850, (1U<<15) | 2850, (1U<<15) | 2850, (1U<<15) | 3329, (1U<<15) | 3329, (1U<<15) | 3329, 
  (1U<<15) | 3329, (1U<<15) | 3329, (1U<<15) | 3329, (1U<<15) | 21799, (1U<<15) | 21822, (1U<<15) | 21845, (1U<<15) | 1759, (1U<<15) | 1856, 
  (1U<<15) | 2285, (1U<<15) | 2336, (1U<<15) | 2167, (1U<<15) | 2245, (1U<<15) | 1907, (1U<<15) | 1977, (1U<<15) | 1778, (1U<<15) | 1797, 
  (1U<<15) | 2296, (1U<<15) | 2347, (1U<<15) | 2186, (1U<<15) | 2197, (1U<<15) | 1926, (1U<<15) | 1996, (1U<<15) | 13462, (1U<<15) | 13580, 
  (1U<<15) | 15029, (1U<<15) | 15190, (1U<<15) | 13428, (1U<<15) | 13566, (1U<<15) | 14992, (1U<<15) | 15176, (1U<<15) | 1560, (1U<<15) | 13541, 
  (1U<<15) | 3288, (1U<<15) | 13541, (1U<<15) | 3288, (1U<<15) | 2775, (1U<<15) | 3297, (1U<<15) | 2773, (1U<<15) | 13550, (1U<<15) | 13573, 
  (1U<<15) | 13550, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 13550, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 2784, (1U<<15) | 3306, 
  0x2d4, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 3297, (1U<<15) | 2800, (1U<<15) | 2775, (1U<<15) | 2775, (1U<<15) | 13550, 
  (1U<<15) | 3297, (1U<<15) | 13441, (1U<<15) | 15008, (1U<<15) | 3258, (1U<<15) | 10975, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 13405, 
  (1U<<15) | 13550, (1U<<15) | 14964, (1U<<15) | 15162, (1U<<15) | 13441, (1U<<15) | 3258, (1U<<15) | 10975, (1U<<15) | 5496, (1U<<15) | 13405, 
  (1U<<15) | 13550, (1U<<15) | 13441, (1U<<15) | 15008, (1U<<15) | 3258, (1U<<15) | 10975, (1U<<15) | 11995, (1U<<15) | 5496, (1U<<15) | 13405, 
  (1U<<15) | 13550, (1U<<15) | 14964, (1U<<15) | 15162, (1U<<15) | 2540, (1U<<15) | 2791, (1U<<15) | 2540, (1U<<15) | 2791, (1U<<15) | 2540, 
  (1U<<15) | 2791, (1U<<15) | 2540, (1U<<15) | 2791, (1U<<15) | 2540, (1U<<15) | 2791, (1U<<15) | 2540, (1U<<15) | 2791, (1U<<15) | 3242, 
  (1U<<15) | 3304, (1U<<15) | 3242, (1U<<15) | 3304, (1U<<15) | 3242, (1U<<15) | 3304, (1U<<15) | 3242, (1U<<15) | 3304, (1U<<15) | 3242, 
  (1U<<15) | 3304, (1U<<15) | 3242, (1U<<15) | 3304, (1U<<15) | 12848, (1U<<15) | 12432, 0x259, 0x25a, 0x25b, 
  0x34a, 0x34b, 0x34c, 0x4a4a, 0x4b4b, 0x4c4c, 0x5959, 0x5a5a, 
  0x5b5b, (1U<<15) | 11891, (1U<<15) | 10659, (1U<<15) | 11884, 0x4894, 0x4895, 0x4894, 0x4895, 
  0x47a4, 0x47a5, 0x47a4, 0x47a5, (1U<<15) | 10659, (1U<<15) | 11891, (1U<<15) | 11884, (1U<<15) | 5013, 
  (1U<<15) | 5269, (1U<<15) | 5316, (1U<<15) | 12848, (1U<<15) | 12432, (1U<<15) | 12689, (1U<<15) | 12750, (1U<<15) | 12802, (1U<<15) | 12239, 
  (1U<<15) | 12347, (1U<<15) | 12405, (1U<<15) | 4607, (1U<<15) | 5364, (1U<<15) | 4587, (1U<<15) | 5374, (1U<<15) | 4754, (1U<<15) | 4724, 
  (1U<<15) | 4734, (1U<<15) | 4744, (1U<<15) | 4647, (1U<<15) | 4627, (1U<<15) | 4687, (1U<<15) | 4667, (1U<<15) | 4637, (1U<<15) | 4617, 
  (1U<<15) | 4677, (1U<<15) | 4657, (1U<<15) | 4548, (1U<<15) | 4518, (1U<<15) | 4558, (1U<<15) | 4528, (1U<<15) | 4538, (1U<<15) | 4508, 
  (1U<<15) | 4597, (1U<<15) | 4577, (1U<<15) | 1165, (1U<<15) | 1243, (1U<<15) | 1278, (1U<<15) | 1204, (1U<<15) | 1172, (1U<<15) | 1250, 
  (1U<<15) | 1285, (1U<<15) | 1211, (1U<<15) | 1172, (1U<<15) | 1250, (1U<<15) | 1285, (1U<<15) | 1211, (1U<<15) | 4906, (1U<<15) | 4841, 
  (1U<<15) | 1098, (1U<<15) | 1146, (1U<<15) | 4113, (1U<<15) | 1136, (1U<<15) | 1188, (1U<<15) | 4222, (1U<<15) | 4499, (1U<<15) | 4490, 
  (1U<<15) | 312, (1U<<15) | 1834, (1U<<15) | 4568, (1U<<15) | 2226, (1U<<15) | 4863, (1U<<15) | 1953, (1U<<15) | 2023, (1U<<15) | 4715, 
  (1U<<15) | 1834, (1U<<15) | 1842, (1U<<15) | 4568, (1U<<15) | 1953, (1U<<15) | 2023, (1U<<15) | 4715, (1U<<15) | 1816, (1U<<15) | 1883, 
  (1U<<15) | 5355, (1U<<15) | 4938, (1U<<15) | 1945, (1U<<15) | 2015, (1U<<15) | 4706, (1U<<15) | 1816, (1U<<15) | 1883, (1U<<15) | 5355, 
  (1U<<15) | 1945, (1U<<15) | 2015, (1U<<15) | 4706, (1U<<15) | 2178, (1U<<15) | 4852, (1U<<15) | 4895, (1U<<15) | 1834, (1U<<15) | 4568, 
  (1U<<15) | 1953, (1U<<15) | 2023, (1U<<15) | 4715, (1U<<15) | 1834, (1U<<15) | 1842, (1U<<15) | 4568, (1U<<15) | 1953, (1U<<15) | 2023, 
  (1U<<15) | 4715, (1U<<15) | 5355, (1U<<15) | 1945, (1U<<15) | 2015, (1U<<15) | 4706, (1U<<15) | 1816, (1U<<15) | 1883, (1U<<15) | 5355, 
  (1U<<15) | 1945, (1U<<15) | 2015, (1U<<15) | 4706, (1U<<15) | 2178, (1U<<15) | 4906, (1U<<15) | 4841, (1U<<15) | 312, (1U<<15) | 1711, 
  (1U<<15) | 1722, (1U<<15) | 4469, (1U<<15) | 1689, (1U<<15) | 1700, (1U<<15) | 5334, (1U<<15) | 4457, (1U<<15) | 4445, (1U<<15) | 1745, 
  (1U<<15) | 1739, (1U<<15) | 4157, (1U<<15) | 4233, (1U<<15) | 4135, (1U<<15) | 4244, (1U<<15) | 4201, (1U<<15) | 4168, (1U<<15) | 4179, 
  (1U<<15) | 4190, (1U<<15) | 4014, (1U<<15) | 3992, (1U<<15) | 4102, (1U<<15) | 4080, (1U<<15) | 4003, (1U<<15) | 3981, (1U<<15) | 4091, 
  (1U<<15) | 4069, (1U<<15) | 3970, (1U<<15) | 3959, (1U<<15) | 4058, (1U<<15) | 4036, (1U<<15) | 4047, (1U<<15) | 4025, (1U<<15) | 4146, 
  (1U<<15) | 4124, (1U<<15) | 2317, (1U<<15) | 2368, (1U<<15) | 4949, (1U<<15) | 2218, (1U<<15) | 2266, (1U<<15) | 5425, (1U<<15) | 4906, 
  (1U<<15) | 4841, (1U<<15) | 2276, (1U<<15) | 2327, (1U<<15) | 4919, (1U<<15) | 2158, (1U<<15) | 2236, (1U<<15) | 5404, (1U<<15) | 4872, 
  (1U<<15) | 4818, (1U<<15) | 4906, (1U<<15) | 4841, (1U<<15) | 4906, (1U<<15) | 4841, (1U<<15) | 4906, (1U<<15) | 4841, (1U<<15) | 1502, 
  (1U<<15) | 1510, (1U<<15) | 2920, (1U<<15) | 1850, (1U<<15) | 1901, (1U<<15) | 3097, (1U<<15) | 1608, (1U<<15) | 1616, (1U<<15) | 3053, 
  (1U<<15) | 1850, (1U<<15) | 1901, (1U<<15) | 3097, (1U<<15) | 1518, (1U<<15) | 1526, (1U<<15) | 1534, (1U<<15) | 1971, (1U<<15) | 2041, 
  (1U<<15) | 2055, (1U<<15) | 1770, (1U<<15) | 1971, (1U<<15) | 2041, (1U<<15) | 2055, (1U<<15) | 1624, (1U<<15) | 1632, (1U<<15) | 1640, 
  (1U<<15) | 1971, (1U<<15) | 2041, (1U<<15) | 2055, (1U<<15) | 1494, (1U<<15) | 1656, (1U<<15) | 3069, (1U<<15) | 5514, (1U<<15) | 1502, 
  (1U<<15) | 1510, (1U<<15) | 2920, (1U<<15) | 1850, (1U<<15) | 1901, (1U<<15) | 3097, (1U<<15) | 1608, (1U<<15) | 1616, (1U<<15) | 3053, 
  (1U<<15) | 1850, (1U<<15) | 1901, (1U<<15) | 3097, (1U<<15) | 1518, (1U<<15) | 1526, (1U<<15) | 1534, (1U<<15) | 1971, (1U<<15) | 2041, 
  (1U<<15) | 2055, (1U<<15) | 1770, (1U<<15) | 1789, (1U<<15) | 1867, (1U<<15) | 1971, (1U<<15) | 2041, (1U<<15) | 2055, (1U<<15) | 1624, 
  (1U<<15) | 1632, (1U<<15) | 1640, (1U<<15) | 1971, (1U<<15) | 2041, (1U<<15) | 2055, (1U<<15) | 1494, (1U<<15) | 2912, (1U<<15) | 5283, 
  (1U<<15) | 1656, (1U<<15) | 3069, (1U<<15) | 5514, (1U<<15) | 1502, (1U<<15) | 1510, (1U<<15) | 2920, (1U<<15) | 1850, (1U<<15) | 1901, 
  (1U<<15) | 3097, (1U<<15) | 1608, (1U<<15) | 1616, (1U<<15) | 3053, (1U<<15) | 1850, (1U<<15) | 1901, (1U<<15) | 3097, (1U<<15) | 1518, 
  (1U<<15) | 1526, (1U<<15) | 1534, (1U<<15) | 1971, (1U<<15) | 2041, (1U<<15) | 2055, (1U<<15) | 1770, (1U<<15) | 1789, (1U<<15) | 1867, 
  (1U<<15) | 1971, (1U<<15) | 2041, (1U<<15) | 2055, (1U<<15) | 1624, (1U<<15) | 1632, (1U<<15) | 1640, (1U<<15) | 1971, (1U<<15) | 2041, 
  (1U<<15) | 2055, (1U<<15) | 1494, (1U<<15) | 2912, (1U<<15) | 5283, (1U<<15) | 1656, (1U<<15) | 3069, (1U<<15) | 5514, (1U<<15) | 2274, 
  (1U<<15) | 2325, (1U<<15) | 4917, (1U<<15) | 2156, (1U<<15) | 2234, (1U<<15) | 5402, (1U<<15) | 8379, (1U<<15) | 8367, (1U<<15) | 2276, 
  (1U<<15) | 2327, (1U<<15) | 4919, (1U<<15) | 2158, (1U<<15) | 2236, (1U<<15) | 5404, (1U<<15) | 8379, (1U<<15) | 8367, (1U<<15) | 2276, 
  (1U<<15) | 2327, (1U<<15) | 4919, (1U<<15) | 2158, (1U<<15) | 2236, (1U<<15) | 5404, (1U<<15) | 8379, (1U<<15) | 8367, (1U<<15) | 2315, 
  (1U<<15) | 2366, (1U<<15) | 4947, (1U<<15) | 2216, (1U<<15) | 2264, (1U<<15) | 5423, (1U<<15) | 4906, (1U<<15) | 4841, (1U<<15) | 12808, 
  (1U<<15) | 11033, (1U<<15) | 12020, (1U<<15) | 12411, (1U<<15) | 12827, (1U<<15) | 11006, (1U<<15) | 12039, (1U<<15) | 12378, (1U<<15) | 12723, 
  (1U<<15) | 11934, (1U<<15) | 12775, (1U<<15) | 11991, (1U<<15) | 12285, (1U<<15) | 10900, (1U<<15) | 12304, (1U<<15) | 10919, (1U<<15) | 12704, 
  (1U<<15) | 11907, (1U<<15) | 12756, (1U<<15) | 11972, (1U<<15) | 12266, (1U<<15) | 10828, (1U<<15) | 12359, (1U<<15) | 10971, (1U<<15) | 4906, 
  (1U<<15) | 4841, (1U<<15) | 4906, (1U<<15) | 4841, (1U<<15) | 5393, (1U<<15) | 1590, (1U<<15) | 1599, (1U<<15) | 3044, (1U<<15) | 1711, 
  (1U<<15) | 1722, (1U<<15) | 4469, (1U<<15) | 1689, (1U<<15) | 1700, (1U<<15) | 5334, (1U<<15) | 4457, (1U<<15) | 4445, (1U<<15) | 12848, 
  (1U<<15) | 12432, (1U<<15) | 12848, (1U<<15) | 12432, (1U<<15) | 12848, (1U<<15) | 12432, (1U<<15) | 13683, (1U<<15) | 3313, (1U<<15) | 13683, 
  (1U<<15) | 3313, (1U<<15) | 2836, (1U<<15) | 3322, (1U<<15) | 15183, (1U<<15) | 15222, (1U<<15) | 15162, (1U<<15) | 15215, (1U<<15) | 3208, 
  (1U<<15) | 3297, (1U<<15) | 3322, (1U<<15) | 2476, (1U<<15) | 2775, (1U<<15) | 2836, (1U<<15) | 13772, (1U<<15) | 13710, (1U<<15) | 2843, 
  (1U<<15) | 3331, (1U<<15) | 3322, (1U<<15) | 3322, (1U<<15) | 3322, (1U<<15) | 2476, (1U<<15) | 2775, (1U<<15) | 2836, (1U<<15) | 2859, 
  (1U<<15) | 2836, (1U<<15) | 13448, (1U<<15) | 15015, (1U<<15) | 3265, (1U<<15) | 11037, (1U<<15) | 12043, (1U<<15) | 5508, (1U<<15) | 13710, 
  (1U<<15) | 15215, (1U<<15) | 3208, (1U<<15) | 3297, (1U<<15) | 3322, (1U<<15) | 13448, (1U<<15) | 14964, (1U<<15) | 15008, (1U<<15) | 15015, 
  (1U<<15) | 3265, (1U<<15) | 11037, (1U<<15) | 11938, (1U<<15) | 11995, (1U<<15) | 12043, (1U<<15) | 5508, (1U<<15) | 13710, (1U<<15) | 14964, 
  (1U<<15) | 15162, (1U<<15) | 15215, (1U<<15) | 3208, (1U<<15) | 3297, (1U<<15) | 3322, (1U<<15) | 13448, (1U<<15) | 15015, (1U<<15) | 3265, 
  (1U<<15) | 11037, (1U<<15) | 12043, (1U<<15) | 5508, (1U<<15) | 13710, (1U<<15) | 15215, (1U<<15) | 3208, (1U<<15) | 3297, (1U<<15) | 3322, 
  (1U<<15) | 10871, (1U<<15) | 10990, (1U<<15) | 11052, (1U<<15) | 11962, (1U<<15) | 12010, (1U<<15) | 12058, (1U<<15) | 2317, (1U<<15) | 2368, 
  (1U<<15) | 2376, (1U<<15) | 2218, (1U<<15) | 2266, (1U<<15) | 3142, (1U<<15) | 2315, (1U<<15) | 2216, (1U<<15) | 2317, (1U<<15) | 2368, 
  (1U<<15) | 2376, (1U<<15) | 2218, (1U<<15) | 2266, (1U<<15) | 3142, (1U<<15) | 2315, (1U<<15) | 2216, (1U<<15) | 12818, (1U<<15) | 11043, 
  (1U<<15) | 12030, (1U<<15) | 12421, (1U<<15) | 12837, (1U<<15) | 11016, (1U<<15) | 12049, (1U<<15) | 12388, (1U<<15) | 12733, (1U<<15) | 11944, 
  (1U<<15) | 12785, (1U<<15) | 12001, (1U<<15) | 12295, (1U<<15) | 10910, (1U<<15) | 12314, (1U<<15) | 10929, (1U<<15) | 12714, (1U<<15) | 11917, 
  (1U<<15) | 12766, (1U<<15) | 11982, (1U<<15) | 12276, (1U<<15) | 10838, (1U<<15) | 12369, (1U<<15) | 10981, (1U<<15) | 13257, (1U<<15) | 12850, 
  (1U<<15) | 12434, (1U<<15) | 12848, (1U<<15) | 12432, (1U<<15) | 13257, (1U<<15) | 10773, (1U<<15) | 10658, 0x4894, 0x4895, 
  0x4894, 0x4895, 0x47a4, 0x47a5, 0x47a4, 0x47a5, (1U<<15) | 12222, (1U<<15) | 12678, 
  (1U<<15) | 12846, (1U<<15) | 12430, (1U<<15) | 12846, (1U<<15) | 12430, (1U<<15) | 13365, (1U<<15) | 13530, (1U<<15) | 13643, (1U<<15) | 14873, 
  (1U<<15) | 15142, (1U<<15) | 15202, (1U<<15) | 2540, (1U<<15) | 2791, (1U<<15) | 2850, (1U<<15) | 2540, (1U<<15) | 2791, (1U<<15) | 2850, 
  (1U<<15) | 3242, (1U<<15) | 3304, (1U<<15) | 3329, (1U<<15) | 3242, (1U<<15) | 3304, (1U<<15) | 3329, (1U<<15) | 13403, (1U<<15) | 13548, 
  (1U<<15) | 13708, (1U<<15) | 3206, (1U<<15) | 3295, (1U<<15) | 3320, (1U<<15) | 18287, (1U<<15) | 18419, (1U<<15) | 18475, (1U<<15) | 16740, 
  (1U<<15) | 16904, (1U<<15) | 16955, (1U<<15) | 14962, (1U<<15) | 15160, (1U<<15) | 15213, (1U<<15) | 2474, (1U<<15) | 2773, (1U<<15) | 2834, 
  (1U<<15) | 15222, (1U<<15) | 13772, (1U<<15) | 14962, (1U<<15) | 15160, (1U<<15) | 15213, (1U<<15) | 14962, (1U<<15) | 15160, (1U<<15) | 15213, 
  (1U<<15) | 2449, (1U<<15) | 2766, (1U<<15) | 2827, (1U<<15) | 16803, (1U<<15) | 16943, (1U<<15) | 16964, (1U<<15) | 16950, (1U<<15) | 16971, 
  (1U<<15) | 21788, (1U<<15) | 21811, (1U<<15) | 21834, (1U<<15) | 1156, (1U<<15) | 12137, (1U<<15) | 12137, (1U<<15) | 12087, (1U<<15) | 12107, 
  (1U<<15) | 12121, (1U<<15) | 4776, (1U<<15) | 1178, (1U<<15) | 1217, (1U<<15) | 4255, (1U<<15) | 4481, (1U<<15) | 4776, (1U<<15) | 1733, 
  (1U<<15) | 2106, (1U<<15) | 3124, (1U<<15) | 10459, (1U<<15) | 4776, (1U<<15) | 2071, (1U<<15) | 3113, (1U<<15) | 10447, (1U<<15) | 4764, 
  (1U<<15) | 4776, (1U<<15) | 4776, (1U<<15) | 4776, (1U<<15) | 2106, (1U<<15) | 3124, (1U<<15) | 12137, (1U<<15) | 2104, (1U<<15) | 2071, 
  (1U<<15) | 3113, (1U<<15) | 10447, (1U<<15) | 8355, (1U<<15) | 2071, (1U<<15) | 3113, (1U<<15) | 10447, (1U<<15) | 8355, (1U<<15) | 2106, 
  (1U<<15) | 3124, (1U<<15) | 12137, (1U<<15) | 2104, (1U<<15) | 2104, (1U<<15) | 3122, (1U<<15) | 10457, (1U<<15) | 4776, (1U<<15) | 4776, 
  (1U<<15) | 4776, (1U<<15) | 2080, (1U<<15) | 2140, (1U<<15) | 2148, (1U<<15) | 4809, (1U<<15) | 1808, (1U<<15) | 1875, (1U<<15) | 5346, 
  (1U<<15) | 2307, (1U<<15) | 2358, (1U<<15) | 4929, (1U<<15) | 2208, (1U<<15) | 2256, (1U<<15) | 5414, (1U<<15) | 1937, (1U<<15) | 2007, 
  (1U<<15) | 4697, (1U<<15) | 1808, (1U<<15) | 1875, (1U<<15) | 5346, (1U<<15) | 1937, (1U<<15) | 2007, (1U<<15) | 4697, (1U<<15) | 1648, 
  (1U<<15) | 3061, (1U<<15) | 8420, (1U<<15) | 1648, (1U<<15) | 3061, (1U<<15) | 8420, (1U<<15) | 2114, (1U<<15) | 2132, (1U<<15) | 5384, 
  (1U<<15) | 2088, (1U<<15) | 2096, (1U<<15) | 4798, (1U<<15) | 4884, (1U<<15) | 4830, (1U<<15) | 4787, (1U<<15) | 1808, (1U<<15) | 1875, 
  (1U<<15) | 5346, (1U<<15) | 1937, (1U<<15) | 2007, (1U<<15) | 4697, (1U<<15) | 1808, (1U<<15) | 1875, (1U<<15) | 5346, (1U<<15) | 1937, 
  (1U<<15) | 2007, (1U<<15) | 4697, (1U<<15) | 1648, (1U<<15) | 3061, (1U<<15) | 8420, (1U<<15) | 1648, (1U<<15) | 3061, (1U<<15) | 8420, 
  (1U<<15) | 2080, (1U<<15) | 2088, (1U<<15) | 2096, (1U<<15) | 2216, (1U<<15) | 2264, (1U<<15) | 5423, (1U<<15) | 4841, (1U<<15) | 2216, 
  (1U<<15) | 2264, (1U<<15) | 5423, (1U<<15) | 4841, (1U<<15) | 2216, (1U<<15) | 2264, (1U<<15) | 5423, (1U<<15) | 4841, (1U<<15) | 2216, 
  (1U<<15) | 2264, (1U<<15) | 5423, (1U<<15) | 4841, (1U<<15) | 2216, (1U<<15) | 2264, (1U<<15) | 5423, (1U<<15) | 4841, (1U<<15) | 2216, 
  (1U<<15) | 2264, (1U<<15) | 5423, (1U<<15) | 4841, (1U<<15) | 15601, (1U<<15) | 15678, (1U<<15) | 12137, (1U<<15) | 15601, (1U<<15) | 15678, 
  (1U<<15) | 12137, (1U<<15) | 12137, (1U<<15) | 12139, (1U<<15) | 12137, (1U<<15) | 10437, 0x46b4, 0x46b5, 0x46b4, 
  0x46b5, (1U<<15) | 10438, (1U<<15) | 11868, 0x46b4, 0x46b5, 0x46b4, 0x46b5, (1U<<15) | 10438, 
  (1U<<15) | 11868, (1U<<15) | 12068, (1U<<15) | 12135, (1U<<15) | 15599, (1U<<15) | 15676, (1U<<15) | 12135, 0x4e0, 0x5e0, 
  0x444, 0x555, 0x444, 0x555, 0x444, 0x555, 0x444, 0x555, 
  (1U<<15) | 21877, (1U<<15) | 3560, 0xe0, 0xe0, 0xe0, 0x0, 0xe0, 0xe0, 
  (1U<<15) | 10190, (1U<<15) | 11857, 0x4e0, 0x5e0, (1U<<15) | 14845, (1U<<15) | 14904, 0xee2, 0xee2, 
  0x4, 0x5, 0x40, 0x50, (1U<<15) | 18348, (1U<<15) | 18466, (1U<<15) | 16787, (1U<<15) | 16934, 
  0xe0, 0xe0, 0xe0, 0xe0, 0x40, 0x50, 0x20, 0xe40, 
  0xe0, 0xe0, 0xe0, (1U<<15) | 11953, 0x4442, 0x4452, 0x4440, 0x4450, 
  0x0, (1U<<15) | 19398, (1U<<15) | 19781, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, 
  (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 2866, (1U<<15) | 20188, 
  (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, 
  (1U<<15) | 20188, (1U<<15) | 14289, (1U<<15) | 11726, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, 
  (1U<<15) | 19768, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, 
  (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 14302, (1U<<15) | 14302, (1U<<15) | 14302, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 14302, 
  (1U<<15) | 14302, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 14302, (1U<<15) | 14302, (1U<<15) | 14302, (1U<<15) | 20188, 
  (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, 
  (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, (1U<<15) | 20188, 0x44e0, 0xee0, 
  0xe5, 0xe3, 0xe2, 0xe4, 0x4440, (1U<<15) | 1918, (1U<<15) | 1988, (1U<<15) | 2047, 
  0xe0, 0x40, 0x50, 0x4, 0x5, 0x4, 0x5, 0x4, 
  0x4, 0x45, 0x45, (1U<<15) | 5329, (1U<<15) | 5607, (1U<<15) | 11747, (1U<<15) | 5329, (1U<<15) | 5607, 
  (1U<<15) | 11747, 0x44, 0x55, 0x5, (1U<<15) | 11747, 0xe0, 0x0, 0xe0, 
  0xe0, 0xee, 0x50, 0x0, 0x0, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 13405, 
  (1U<<15) | 1751, (1U<<15) | 13405, (1U<<15) | 13405, (1U<<15) | 13403, 0xe, (1U<<15) | 2218, (1U<<15) | 2218, (1U<<15) | 16767, 
  (1U<<15) | 16767, (1U<<15) | 16767, (1U<<15) | 16767, (1U<<15) | 16767, (1U<<15) | 16767, (1U<<15) | 18371, (1U<<15) | 19777, (1U<<15) | 19771, 
  (1U<<15) | 16817, 0x7a4, 0x7a5, (1U<<15) | 18371, (1U<<15) | 16817, 0x7a4, 0x7a5, 0xe0, 
  (1U<<15) | 16789, (1U<<15) | 16789, (1U<<15) | 16789, (1U<<15) | 16789, 0x7a4, (1U<<15) | 2867, 0x7a7a, 0x7a7a, 
  0x7a7a, 0x7a7a, 0x0, 0xe0, (1U<<15) | 16767, (1U<<15) | 16767, (1U<<15) | 16767, (1U<<15) | 16767, 
  (1U<<15) | 16767, (1U<<15) | 16767, 0xe0, (1U<<15) | 2317, (1U<<15) | 2317, (1U<<15) | 18311, (1U<<15) | 18311, (1U<<15) | 18311, 
  (1U<<15) | 18311, (1U<<15) | 18311, (1U<<15) | 18311, (1U<<15) | 18282, (1U<<15) | 18305, 0x7a4a, 0x894, 0x895, 
  (1U<<15) | 18303, (1U<<15) | 18282, 0x7a4a, 0x894, 0x895, 0x0, (1U<<15) | 19218, (1U<<15) | 18350, 
  (1U<<15) | 18350, 0x0, (1U<<15) | 18350, (1U<<15) | 18350, 0x894, (1U<<15) | 13376, (1U<<15) | 3199, (1U<<15) | 3199, 
  0x0, (1U<<15) | 2476, (1U<<15) | 3208, (1U<<15) | 3244, 0x2c4, (1U<<15) | 3208, (1U<<15) | 3208, (1U<<15) | 2549, 
  (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 10822, (1U<<15) | 11938, (1U<<15) | 5444, (1U<<15) | 13405, (1U<<15) | 3208, 
  (1U<<15) | 10822, (1U<<15) | 5444, (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 10822, (1U<<15) | 11938, (1U<<15) | 5444, 
  (1U<<15) | 18311, (1U<<15) | 18311, (1U<<15) | 18311, (1U<<15) | 18311, (1U<<15) | 18311, (1U<<15) | 18311, (1U<<15) | 18350, (1U<<15) | 16789, 
  (1U<<15) | 18350, (1U<<15) | 16789, (1U<<15) | 18350, (1U<<15) | 16789, 0xe2c, 0x44e0, 0x440, (1U<<15) | 18348, 
  (1U<<15) | 16787, (1U<<15) | 2317, (1U<<15) | 2218, (1U<<15) | 2218, (1U<<15) | 1479, (1U<<15) | 13376, (1U<<15) | 2474, 0x3b3b, 
  (1U<<15) | 14888, (1U<<15) | 14888, (1U<<15) | 14888, (1U<<15) | 12744, (1U<<15) | 12333, (1U<<15) | 12742, (1U<<15) | 12339, 0x344, 
  0x444, 0x244, 0x555, (1U<<15) | 1680, (1U<<15) | 1680, (1U<<15) | 1680, (1U<<15) | 1680, (1U<<15) | 1680, 
  (1U<<15) | 1680, (1U<<15) | 1670, (1U<<15) | 1487, (1U<<15) | 1487, (1U<<15) | 1487, (1U<<15) | 1487, (1U<<15) | 1487, (1U<<15) | 1487, 
  (1U<<15) | 1471, (1U<<15) | 2614, (1U<<15) | 1464, (1U<<15) | 14964, (1U<<15) | 1462, (1U<<15) | 20189, (1U<<15) | 20189, (1U<<15) | 20189, 
  (1U<<15) | 20188, (1U<<15) | 13405, (1U<<15) | 20188, (1U<<15) | 3208, (1U<<15) | 20188, (1U<<15) | 3208, (1U<<15) | 20188, (1U<<15) | 13405, 
  (1U<<15) | 20188, (1U<<15) | 3208, (1U<<15) | 20188, (1U<<15) | 3208, (1U<<15) | 20188, (1U<<15) | 2516, (1U<<15) | 20188, (1U<<15) | 3208, 
  (1U<<15) | 20188, (1U<<15) | 2476, (1U<<15) | 20188, (1U<<15) | 2476, (1U<<15) | 20188, (1U<<15) | 13405, (1U<<15) | 20188, (1U<<15) | 3208, 
  0xe0, 0x0, (1U<<15) | 8347, (1U<<15) | 14711, 0x444, 0x555, 0x2220, (1U<<15) | 21888, 
  0x2220, (1U<<15) | 21888, 0x427c, (1U<<15) | 14440, 0x427c, (1U<<15) | 5324, (1U<<15) | 14447, (1U<<15) | 5324, 
  (1U<<15) | 5324, (1U<<15) | 14447, (1U<<15) | 5324, 0x426d, (1U<<15) | 14433, 0x426d, 0x426d, (1U<<15) | 14433, 
  0x426d, 0x2220, (1U<<15) | 21888, 0x2220, (1U<<15) | 21888, 0x2220, (1U<<15) | 21888, 0x2220, 
  (1U<<15) | 21888, 0x2220, (1U<<15) | 21888, 0x2220, (1U<<15) | 21888, 0x2220, (1U<<15) | 21888, 0x2220, 
  (1U<<15) | 21888, 0x2220, (1U<<15) | 21888, 0x2220, (1U<<15) | 21888, 0x2, 0x5e20, (1U<<15) | 15281, 
  0x5e20, (1U<<15) | 15281, 0x5e20, (1U<<15) | 15281, 0x5e20, (1U<<15) | 15281, 0x424c, (1U<<15) | 14426, 
  0x424c, 0x0, 0x5e20, (1U<<15) | 21881, 0x20, (1U<<15) | 3080, 0x2220, (1U<<15) | 21888, 
  0x4442, 0xe0, 0x4442, 0x55, 0x550, 0xe7a, 0xe7b, 0xe7a, 
  0xe7b, 0xe7a, 0xe7b, 0xe7a, 0xe7b, 0xe7a, 0xe7b, 0xe7a, 
  0xe7b, (1U<<15) | 16899, (1U<<15) | 16950, (1U<<15) | 12251, (1U<<15) | 12353, (1U<<15) | 1471, (1U<<15) | 1552, (1U<<15) | 1568, 
  (1U<<15) | 1471, (1U<<15) | 1552, (1U<<15) | 1568, (1U<<15) | 2476, (1U<<15) | 2775, (1U<<15) | 2836, (1U<<15) | 15008, (1U<<15) | 15162, 
  (1U<<15) | 15006, (1U<<15) | 13403, (1U<<15) | 13403, (1U<<15) | 10871, (1U<<15) | 13405, (1U<<15) | 13550, (1U<<15) | 13710, (1U<<15) | 13405, 
  (1U<<15) | 13550, (1U<<15) | 13710, 0x0, 0x0, 0x40, 0x50, 0x40, 0x50, 
  0x40, 0xe40, 0xe50, 0xe40, 0xe50, 0x20, 0x4, 0x0, 
  0x45, (1U<<15) | 18312, (1U<<15) | 18447, 0x7a7a, 0x7b7b, (1U<<15) | 18312, 0x7a7a, (1U<<15) | 1961, 
  (1U<<15) | 2031, (1U<<15) | 1824, (1U<<15) | 1891, 0x2c4a, 0x2c59, 0x2c3b, 0x4a59, 0x2c4a, 
  0x2c59, 0x2c3b, 0x4a59, 0x3b4a, 0x3b59, 0x3b4a, 0x3b59, 0x2c3b, 
  0x4a59, 0x3b4a, (1U<<15) | 13403, (1U<<15) | 14864, (1U<<15) | 14864, (1U<<15) | 13403, (1U<<15) | 14864, (1U<<15) | 14864, 
  (1U<<15) | 13287, (1U<<15) | 3206, (1U<<15) | 13287, (1U<<15) | 3206, (1U<<15) | 13287, (1U<<15) | 13287, (1U<<15) | 2474, (1U<<15) | 2476, 
  (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 3208, (1U<<15) | 2476, (1U<<15) | 13405, (1U<<15) | 14964, (1U<<15) | 3208, 0x0, 
  0x44e0, 0x44e0, 0x44e0, 0x44e0, 0x44e0, 0x44e0, 0x44e0, 0x44e0, 
  0x44e0, 0x44e0, 0x44e0, 0x44e0, 0x4440, 0x0, 0x4, 0x44, 
  0xee, 0x44f0, 0x0, 0x4f0, 0x40, 0x4444, (1U<<15) | 8568, 0x4f0, 
  0x4f0, 0x4f4, 0x4f0, 0x4, 0x4, 0x4, 0x44, 0x44f, 
  (1U<<15) | 18980, 0x4f4, 0x4f4, 0x4f4, (1U<<15) | 19319, (1U<<15) | 19319, (1U<<15) | 19319, (1U<<15) | 19319, 
  (1U<<15) | 19319, 0x44f4, 0x4f4, 0x4f0, 0x4f0, 0x44f0, 0x44f0, 0x44f4, 
  0x44f0, 0x4f4, 0x44f0, (1U<<15) | 18979, 0x44f0, (1U<<15) | 19319, 0x440, 0x44f0, 
  0x44f0, (1U<<15) | 18979, 0x40, 0x44f0, (1U<<15) | 19319, 0x444, 0x0, 0x4f0, 
  0x4f4, 0x4f4, 0xe, 0x444, 0
};

static constexpr unsigned char IIT_LongEncodingTable[] = {
  /* 0 */ 35, 12, 1, 15, 0, 0,
  /* 6 */ 28, 0, 4, 15, 0, 0,
  /* 12 */ 0, 4, 4, 4, 2, 4, 15, 0, 0,
  /* 21 */ 15, 1, 1, 15, 9, 1, 15, 15, 15, 7, 1, 1, 1, 0,
  /* 35 */ 15, 1, 15, 12, 1, 1, 1, 0,
  /* 43 */ 15, 1, 1, 15, 9, 1, 15, 17, 15, 7, 15, 25, 1, 1, 1, 0,
  /* 59 */ 4, 24, 2, 4, 4, 4, 1, 4, 1, 1, 0,
  /* 70 */ 21, 0, 4, 4, 4, 4, 1, 1, 0,
  /* 79 */ 15, 2, 4, 15, 9, 4, 15, 17, 3, 15, 7, 4, 4, 4, 4, 4, 4, 1, 1, 0,
  /* 99 */ 15, 2, 15, 9, 15, 17, 3, 15, 7, 4, 4, 4, 4, 4, 4, 1, 1, 0,
  /* 117 */ 15, 0, 15, 7, 15, 7, 4, 4, 1, 1, 0,
  /* 128 */ 0, 24, 7, 24, 3, 24, 1, 4, 3, 5, 1, 1, 0,
  /* 141 */ 15, 2, 4, 15, 9, 4, 15, 17, 3, 15, 7, 4, 4, 5, 4, 4, 5, 1, 1, 0,
  /* 161 */ 15, 2, 15, 9, 15, 17, 3, 15, 7, 4, 4, 5, 4, 4, 5, 1, 1, 0,
  /* 179 */ 0, 4, 4, 15, 3, 15, 7, 1, 1, 0,
  /* 189 */ 15, 2, 1, 15, 10, 1, 15, 15, 3, 15, 7, 1, 1, 0,
  /* 203 */ 15, 2, 15, 9, 15, 15, 3, 15, 7, 1, 1, 0,
  /* 215 */ 0, 4, 4, 15, 0, 15, 7, 15, 7, 15, 7, 1, 1, 0,
  /* 229 */ 21, 0, 1, 15, 1, 1, 0,
  /* 236 */ 15, 2, 1, 15, 10, 1, 15, 15, 3, 15, 18, 1, 1, 0,
  /* 250 */ 15, 2, 15, 9, 15, 17, 15, 7, 15, 25, 1, 1, 0,
  /* 263 */ 15, 2, 1, 15, 10, 1, 15, 18, 15, 7, 15, 25, 1, 1, 0,
  /* 278 */ 9, 3, 9, 3, 9, 3, 9, 3, 1, 0,
  /* 288 */ 0, 15, 3, 29, 1, 0, 28, 3, 1, 0,
  /* 298 */ 15, 3, 28, 3, 1, 0,
  /* 304 */ 15, 3, 15, 7, 28, 3, 1, 0,
  /* 312 */ 15, 3, 15, 7, 15, 7, 28, 3, 1, 0,
  /* 322 */ 0, 15, 3, 15, 9, 28, 3, 1, 0,
  /* 331 */ 28, 3, 1, 15, 3, 15, 11, 28, 3, 1, 0,
  /* 342 */ 0, 15, 3, 15, 12, 28, 3, 1, 0,
  /* 351 */ 0, 15, 3, 14, 28, 3, 1, 0,
  /* 359 */ 4, 15, 1, 4, 1, 0,
  /* 365 */ 4, 9, 3, 9, 3, 4, 1, 0,
  /* 373 */ 4, 1, 4, 1, 4, 4, 1, 0,
  /* 381 */ 4, 4, 4, 4, 1, 0,
  /* 387 */ 15, 1, 15, 7, 4, 4, 4, 1, 0,
  /* 396 */ 15, 0, 15, 7, 15, 7, 4, 4, 4, 1, 0,
  /* 407 */ 0, 24, 6, 5, 4, 1, 0,
  /* 414 */ 0, 24, 6, 4, 1, 0,
  /* 420 */ 9, 6, 9, 6, 4, 7, 4, 1, 0,
  /* 429 */ 9, 6, 9, 6, 7, 4, 1, 0,
  /* 437 */ 4, 7, 7, 4, 1, 0,
  /* 443 */ 9, 40, 9, 40, 7, 4, 1, 0,
  /* 451 */ 0, 24, 6, 5, 9, 4, 1, 0,
  /* 459 */ 0, 24, 6, 9, 4, 1, 0,
  /* 466 */ 0, 24, 6, 5, 10, 4, 1, 0,
  /* 474 */ 0, 24, 6, 10, 4, 1, 0,
  /* 481 */ 0, 24, 6, 5, 11, 4, 1, 0,
  /* 489 */ 0, 24, 6, 11, 4, 1, 0,
  /* 496 */ 0, 24, 6, 5, 12, 4, 1, 0,
  /* 504 */ 0, 24, 6, 12, 4, 1, 0,
  /* 511 */ 0, 24, 6, 5, 13, 4, 1, 0,
  /* 519 */ 0, 24, 6, 13, 4, 1, 0,
  /* 526 */ 0, 24, 6, 5, 16, 4, 1, 0,
  /* 534 */ 0, 24, 6, 16, 4, 1, 0,
  /* 541 */ 0, 24, 6, 5, 39, 4, 1, 0,
  /* 549 */ 0, 24, 6, 39, 4, 1, 0,
  /* 556 */ 21, 0, 5, 1, 0,
  /* 561 */ 0, 14, 4, 4, 4, 4, 4, 3, 3, 3, 5, 1, 0,
  /* 574 */ 0, 24, 3, 24, 3, 14, 4, 4, 4, 4, 4, 3, 3, 3, 5, 1, 0,
  /* 591 */ 0, 14, 4, 4, 4, 4, 4, 3, 3, 5, 1, 0,
  /* 603 */ 0, 24, 3, 24, 3, 14, 4, 4, 4, 4, 4, 3, 3, 5, 1, 0,
  /* 619 */ 0, 14, 4, 4, 4, 4, 3, 3, 5, 1, 0,
  /* 630 */ 0, 24, 3, 24, 3, 14, 4, 4, 4, 4, 3, 3, 5, 1, 0,
  /* 645 */ 0, 14, 4, 4, 4, 3, 3, 5, 1, 0,
  /* 655 */ 0, 24, 3, 24, 3, 14, 4, 4, 4, 3, 3, 5, 1, 0,
  /* 669 */ 0, 14, 4, 4, 4, 3, 5, 1, 0,
  /* 678 */ 0, 24, 3, 24, 3, 14, 4, 4, 4, 3, 5, 1, 0,
  /* 691 */ 0, 24, 1, 4, 5, 1, 0,
  /* 698 */ 0, 24, 3, 24, 3, 24, 1, 4, 5, 1, 0,
  /* 709 */ 0, 24, 1, 24, 3, 4, 5, 1, 0,
  /* 718 */ 0, 14, 4, 4, 4, 4, 4, 5, 1, 0,
  /* 728 */ 0, 24, 3, 14, 4, 4, 4, 4, 4, 5, 1, 0,
  /* 740 */ 0, 24, 3, 24, 3, 14, 4, 4, 4, 4, 4, 5, 1, 0,
  /* 754 */ 0, 14, 4, 4, 4, 4, 5, 1, 0,
  /* 763 */ 0, 24, 3, 14, 4, 4, 4, 4, 5, 1, 0,
  /* 774 */ 0, 24, 3, 24, 3, 14, 4, 4, 4, 4, 5, 1, 0,
  /* 787 */ 0, 14, 4, 4, 4, 5, 1, 0,
  /* 795 */ 0, 24, 3, 14, 4, 4, 4, 5, 1, 0,
  /* 805 */ 0, 24, 3, 24, 3, 14, 4, 4, 4, 5, 1, 0,
  /* 817 */ 0, 14, 4, 4, 5, 1, 0,
  /* 824 */ 0, 24, 3, 14, 4, 4, 5, 1, 0,
  /* 833 */ 0, 24, 3, 24, 3, 14, 4, 4, 5, 1, 0,
  /* 844 */ 0, 14, 4, 5, 1, 0,
  /* 850 */ 0, 24, 3, 14, 4, 5, 1, 0,
  /* 858 */ 0, 24, 3, 24, 3, 14, 4, 5, 1, 0,
  /* 868 */ 9, 4, 24, 6, 5, 1, 0,
  /* 875 */ 10, 4, 24, 6, 5, 1, 0,
  /* 882 */ 11, 4, 24, 6, 5, 1, 0,
  /* 889 */ 12, 4, 24, 6, 5, 1, 0,
  /* 896 */ 13, 4, 24, 6, 5, 1, 0,
  /* 903 */ 16, 4, 24, 6, 5, 1, 0,
  /* 910 */ 39, 4, 24, 6, 5, 1, 0,
  /* 917 */ 15, 4, 15, 7, 5, 1, 0,
  /* 924 */ 9, 4, 24, 6, 1, 0,
  /* 930 */ 10, 4, 24, 6, 1, 0,
  /* 936 */ 11, 4, 24, 6, 1, 0,
  /* 942 */ 12, 4, 24, 6, 1, 0,
  /* 948 */ 13, 4, 24, 6, 1, 0,
  /* 954 */ 16, 4, 24, 6, 1, 0,
  /* 960 */ 39, 4, 24, 6, 1, 0,
  /* 966 */ 9, 6, 4, 7, 1, 0,
  /* 972 */ 9, 7, 4, 7, 1, 0,
  /* 978 */ 9, 40, 4, 7, 1, 0,
  /* 984 */ 9, 3, 9, 3, 9, 6, 7, 1, 0,
  /* 993 */ 7, 9, 6, 9, 6, 7, 1, 0,
  /* 1001 */ 9, 3, 9, 3, 7, 7, 7, 1, 0,
  /* 1010 */ 6, 7, 7, 7, 1, 0,
  /* 1016 */ 7, 7, 7, 7, 1, 0,
  /* 1022 */ 15, 1, 15, 7, 1, 0,
  /* 1028 */ 21, 0, 15, 2, 1, 15, 7, 15, 7, 1, 0,
  /* 1039 */ 15, 2, 15, 7, 15, 7, 15, 7, 1, 0,
  /* 1049 */ 15, 1, 1, 15, 9, 1, 15, 15, 15, 7, 1, 0,
  /* 1061 */ 15, 1, 15, 9, 15, 15, 15, 7, 1, 0,
  /* 1071 */ 15, 2, 15, 10, 15, 15, 15, 7, 1, 0,
  /* 1081 */ 9, 3, 9, 3, 9, 40, 7, 1, 0,
  /* 1090 */ 7, 9, 40, 9, 40, 7, 1, 0,
  /* 1098 */ 9, 1, 9, 8, 9, 8, 4, 9, 1, 0,
  /* 1108 */ 0, 15, 4, 2, 15, 9, 1, 0,
  /* 1116 */ 10, 7, 10, 7, 11, 6, 4, 10, 1, 0,
  /* 1126 */ 11, 6, 11, 6, 10, 7, 4, 10, 1, 0,
  /* 1136 */ 10, 1, 10, 7, 10, 7, 4, 10, 1, 0,
  /* 1146 */ 10, 1, 10, 8, 10, 8, 4, 10, 1, 0,
  /* 1156 */ 11, 40, 10, 7, 11, 40, 10, 1, 0,
  /* 1165 */ 11, 1, 11, 1, 11, 1, 0,
  /* 1172 */ 4, 11, 1, 11, 1, 0,
  /* 1178 */ 11, 1, 11, 6, 11, 6, 4, 11, 1, 0,
  /* 1188 */ 11, 1, 11, 7, 11, 7, 4, 11, 1, 0,
  /* 1198 */ 15, 1, 15, 11, 1, 0,
  /* 1204 */ 12, 1, 12, 1, 12, 1, 0,
  /* 1211 */ 4, 12, 1, 12, 1, 0,
  /* 1217 */ 12, 1, 12, 6, 12, 6, 4, 12, 1, 0,
  /* 1227 */ 15, 0, 35, 12, 1, 0,
  /* 1233 */ 35, 12, 1, 35, 12, 1, 35, 12, 1, 0,
  /* 1243 */ 13, 1, 13, 1, 13, 1, 0,
  /* 1250 */ 4, 13, 1, 13, 1, 0,
  /* 1256 */ 28, 1, 4, 15, 1, 0,
  /* 1262 */ 28, 1, 7, 15, 1, 0,
  /* 1268 */ 34, 7, 15, 1, 0,
  /* 1273 */ 0, 19, 15, 1, 0,
  /* 1278 */ 16, 1, 16, 1, 16, 1, 0,
  /* 1285 */ 4, 16, 1, 16, 1, 0,
  /* 1291 */ 21, 0, 12, 4, 16, 1, 12, 4, 12, 4, 16, 1, 0,
  /* 1304 */ 12, 4, 12, 4, 12, 4, 16, 1, 0,
  /* 1313 */ 0, 15, 4, 15, 8, 15, 17, 1, 0,
  /* 1322 */ 0, 15, 4, 15, 12, 15, 17, 1, 0,
  /* 1331 */ 2, 18, 1, 0,
  /* 1335 */ 15, 0, 24, 1, 0,
  /* 1340 */ 0, 24, 3, 24, 1, 0,
  /* 1346 */ 4, 24, 1, 0,
  /* 1350 */ 15, 1, 1, 15, 9, 1, 15, 17, 15, 7, 15, 25, 1, 0,
  /* 1364 */ 31, 1, 31, 1, 31, 1, 0,
  /* 1371 */ 41, 1, 31, 1, 0,
  /* 1376 */ 21, 2, 12, 2, 12, 2, 12, 2, 12, 2, 31, 1, 0,
  /* 1389 */ 32, 1, 32, 1, 32, 1, 0,
  /* 1396 */ 0, 14, 32, 1, 0,
  /* 1401 */ 39, 1, 39, 1, 39, 1, 0,
  /* 1408 */ 21, 0, 13, 4, 39, 1, 13, 4, 13, 4, 39, 1, 0,
  /* 1421 */ 13, 4, 13, 4, 13, 4, 39, 1, 0,
  /* 1430 */ 31, 1, 31, 1, 41, 1, 0,
  /* 1437 */ 41, 1, 41, 1, 41, 1, 0,
  /* 1444 */ 21, 0, 12, 2, 12, 2, 41, 1, 0,
  /* 1453 */ 15, 3, 27, 0, 2, 27, 0, 2, 0,
  /* 1462 */ 9, 5, 9, 5, 9, 5, 2, 2, 0,
  /* 1471 */ 12, 2, 12, 2, 12, 2, 2, 0,
  /* 1479 */ 11, 3, 12, 2, 12, 2, 2, 0,
  /* 1487 */ 4, 12, 2, 12, 2, 2, 0,
  /* 1494 */ 12, 2, 11, 3, 12, 2, 2, 0,
  /* 1502 */ 12, 2, 10, 4, 12, 2, 2, 0,
  /* 1510 */ 12, 2, 11, 4, 12, 2, 2, 0,
  /* 1518 */ 12, 2, 9, 5, 12, 2, 2, 0,
  /* 1526 */ 12, 2, 10, 5, 12, 2, 2, 0,
  /* 1534 */ 12, 2, 11, 5, 12, 2, 2, 0,
  /* 1542 */ 12, 2, 12, 2, 11, 6, 12, 2, 2, 0,
  /* 1552 */ 13, 2, 13, 2, 13, 2, 2, 0,
  /* 1560 */ 12, 3, 13, 2, 13, 2, 2, 0,
  /* 1568 */ 16, 2, 16, 2, 16, 2, 2, 0,
  /* 1576 */ 13, 3, 16, 2, 16, 2, 2, 0,
  /* 1584 */ 4, 15, 4, 3, 2, 0,
  /* 1590 */ 11, 3, 10, 7, 4, 11, 3, 2, 0,
  /* 1599 */ 11, 3, 11, 7, 4, 11, 3, 2, 0,
  /* 1608 */ 11, 3, 10, 4, 11, 3, 2, 0,
  /* 1616 */ 11, 3, 11, 4, 11, 3, 2, 0,
  /* 1624 */ 11, 3, 9, 5, 11, 3, 2, 0,
  /* 1632 */ 11, 3, 10, 5, 11, 3, 2, 0,
  /* 1640 */ 11, 3, 11, 5, 11, 3, 2, 0,
  /* 1648 */ 11, 3, 11, 6, 11, 3, 2, 0,
  /* 1656 */ 0, 14, 11, 3, 2, 0,
  /* 1662 */ 0, 15, 0, 15, 12, 3, 2, 0,
  /* 1670 */ 12, 2, 12, 2, 4, 12, 2, 4, 2, 0,
  /* 1680 */ 4, 12, 2, 4, 12, 2, 4, 2, 0,
  /* 1689 */ 10, 7, 10, 7, 10, 7, 10, 4, 4, 2, 0,
  /* 1700 */ 11, 7, 11, 7, 11, 7, 11, 4, 4, 2, 0,
  /* 1711 */ 9, 8, 9, 8, 9, 8, 9, 5, 4, 2, 0,
  /* 1722 */ 10, 8, 10, 8, 10, 8, 10, 5, 4, 2, 0,
  /* 1733 */ 2, 11, 6, 4, 2, 0,
  /* 1739 */ 2, 10, 7, 4, 2, 0,
  /* 1745 */ 2, 9, 8, 4, 2, 0,
  /* 1751 */ 10, 4, 10, 4, 10, 4, 2, 0,
  /* 1759 */ 10, 4, 10, 4, 14, 10, 4, 10, 4, 2, 0,
  /* 1770 */ 10, 4, 9, 5, 10, 4, 2, 0,
  /* 1778 */ 10, 4, 10, 4, 14, 9, 5, 10, 4, 2, 0,
  /* 1789 */ 10, 4, 10, 5, 10, 4, 2, 0,
  /* 1797 */ 10, 4, 10, 4, 14, 10, 5, 10, 4, 2, 0,
  /* 1808 */ 10, 4, 11, 6, 10, 4, 2, 0,
  /* 1816 */ 10, 4, 10, 7, 10, 4, 2, 0,
  /* 1824 */ 10, 7, 10, 7, 10, 7, 10, 4, 2, 0,
  /* 1834 */ 10, 4, 9, 8, 10, 4, 2, 0,
  /* 1842 */ 10, 4, 10, 8, 10, 4, 2, 0,
  /* 1850 */ 0, 14, 10, 4, 2, 0,
  /* 1856 */ 11, 4, 11, 4, 14, 11, 4, 11, 4, 2, 0,
  /* 1867 */ 11, 4, 11, 5, 11, 4, 2, 0,
  /* 1875 */ 11, 4, 11, 6, 11, 4, 2, 0,
  /* 1883 */ 11, 4, 11, 7, 11, 4, 2, 0,
  /* 1891 */ 11, 7, 11, 7, 11, 7, 11, 4, 2, 0,
  /* 1901 */ 0, 14, 11, 4, 2, 0,
  /* 1907 */ 9, 5, 9, 5, 14, 10, 4, 9, 5, 2, 0,
  /* 1918 */ 9, 5, 9, 5, 9, 5, 2, 0,
  /* 1926 */ 9, 5, 9, 5, 14, 9, 5, 9, 5, 2, 0,
  /* 1937 */ 9, 5, 11, 6, 9, 5, 2, 0,
  /* 1945 */ 9, 5, 10, 7, 9, 5, 2, 0,
  /* 1953 */ 9, 5, 9, 8, 9, 5, 2, 0,
  /* 1961 */ 9, 8, 9, 8, 9, 8, 9, 5, 2, 0,
  /* 1971 */ 0, 14, 9, 5, 2, 0,
  /* 1977 */ 10, 5, 10, 5, 14, 10, 4, 10, 5, 2, 0,
  /* 1988 */ 10, 5, 10, 5, 10, 5, 2, 0,
  /* 1996 */ 10, 5, 10, 5, 14, 10, 5, 10, 5, 2, 0,
  /* 2007 */ 10, 5, 11, 6, 10, 5, 2, 0,
  /* 2015 */ 10, 5, 10, 7, 10, 5, 2, 0,
  /* 2023 */ 10, 5, 10, 8, 10, 5, 2, 0,
  /* 2031 */ 10, 8, 10, 8, 10, 8, 10, 5, 2, 0,
  /* 2041 */ 0, 14, 10, 5, 2, 0,
  /* 2047 */ 11, 5, 11, 5, 11, 5, 2, 0,
  /* 2055 */ 0, 14, 11, 5, 2, 0,
  /* 2061 */ 11, 6, 12, 2, 11, 6, 2, 0,
  /* 2069 */ 11, 6, 11, 6, 11, 6, 4, 11, 6, 2, 0,
  /* 2080 */ 11, 6, 10, 4, 11, 6, 2, 0,
  /* 2088 */ 11, 6, 9, 5, 11, 6, 2, 0,
  /* 2096 */ 11, 6, 10, 5, 11, 6, 2, 0,
  /* 2104 */ 11, 6, 11, 6, 11, 6, 11, 6, 2, 0,
  /* 2114 */ 11, 6, 10, 7, 11, 6, 2, 0,
  /* 2122 */ 11, 6, 10, 7, 10, 7, 11, 6, 2, 0,
  /* 2132 */ 11, 6, 11, 7, 11, 6, 2, 0,
  /* 2140 */ 11, 6, 9, 8, 11, 6, 2, 0,
  /* 2148 */ 11, 6, 10, 8, 11, 6, 2, 0,
  /* 2156 */ 10, 7, 10, 7, 10, 7, 4, 10, 7, 2, 0,
  /* 2167 */ 10, 7, 10, 7, 14, 10, 4, 10, 7, 2, 0,
  /* 2178 */ 10, 7, 9, 5, 10, 7, 2, 0,
  /* 2186 */ 10, 7, 10, 7, 14, 9, 5, 10, 7, 2, 0,
  /* 2197 */ 10, 7, 10, 7, 14, 10, 5, 10, 7, 2, 0,
  /* 2208 */ 10, 7, 11, 6, 10, 7, 2, 0,
  /* 2216 */ 10, 7, 10, 7, 10, 7, 10, 7, 2, 0,
  /* 2226 */ 10, 7, 9, 8, 10, 7, 2, 0,
  /* 2234 */ 11, 7, 11, 7, 11, 7, 4, 11, 7, 2, 0,
  /* 2245 */ 11, 7, 11, 7, 14, 11, 4, 11, 7, 2, 0,
  /* 2256 */ 11, 7, 11, 6, 11, 7, 2, 0,
  /* 2264 */ 11, 7, 11, 7, 11, 7, 11, 7, 2, 0,
  /* 2274 */ 9, 8, 9, 8, 9, 8, 4, 9, 8, 2, 0,
  /* 2285 */ 9, 8, 9, 8, 14, 10, 4, 9, 8, 2, 0,
  /* 2296 */ 9, 8, 9, 8, 14, 9, 5, 9, 8, 2, 0,
  /* 2307 */ 9, 8, 11, 6, 9, 8, 2, 0,
  /* 2315 */ 9, 8, 9, 8, 9, 8, 9, 8, 2, 0,
  /* 2325 */ 10, 8, 10, 8, 10, 8, 4, 10, 8, 2, 0,
  /* 2336 */ 10, 8, 10, 8, 14, 10, 4, 10, 8, 2, 0,
  /* 2347 */ 10, 8, 10, 8, 14, 10, 5, 10, 8, 2, 0,
  /* 2358 */ 10, 8, 11, 6, 10, 8, 2, 0,
  /* 2366 */ 10, 8, 10, 8, 10, 8, 10, 8, 2, 0,
  /* 2376 */ 11, 8, 11, 8, 11, 8, 2, 0,
  /* 2384 */ 15, 0, 15, 9, 2, 0,
  /* 2390 */ 0, 10, 2, 10, 2, 0,
  /* 2396 */ 10, 2, 10, 2, 10, 2, 0,
  /* 2403 */ 4, 10, 2, 10, 2, 0,
  /* 2409 */ 5, 5, 10, 2, 10, 2, 0,
  /* 2416 */ 11, 2, 11, 2, 11, 2, 11, 2, 11, 2, 11, 2, 11, 2, 0,
  /* 2431 */ 31, 1, 31, 1, 41, 1, 12, 2, 0,
  /* 2440 */ 32, 1, 32, 1, 41, 1, 12, 2, 0,
  /* 2449 */ 12, 1, 12, 2, 12, 2, 0,
  /* 2456 */ 31, 1, 31, 1, 12, 2, 12, 2, 0,
  /* 2465 */ 41, 1, 12, 2, 12, 2, 0,
  /* 2472 */ 31, 1, 12, 2, 12, 2, 12, 2, 12, 2, 0,
  /* 2483 */ 32, 1, 12, 2, 12, 2, 12, 2, 12, 2, 12, 2, 12, 2, 12, 2, 12, 2, 0,
  /* 2502 */ 21, 0, 12, 2, 4, 12, 2, 12, 2, 12, 2, 0,
  /* 2514 */ 11, 3, 11, 3, 12, 2, 12, 2, 0,
  /* 2523 */ 21, 0, 12, 2, 4, 12, 2, 12, 2, 0,
  /* 2533 */ 4, 4, 12, 2, 12, 2, 0,
  /* 2540 */ 10, 4, 10, 4, 12, 2, 12, 2, 0,
  /* 2549 */ 9, 5, 12, 2, 12, 2, 0,
  /* 2556 */ 15, 3, 15, 7, 12, 2, 12, 2, 0,
  /* 2565 */ 21, 0, 12, 2, 4, 11, 3, 11, 3, 12, 2, 0,
  /* 2577 */ 21, 0, 12, 2, 4, 12, 2, 0,
  /* 2585 */ 12, 2, 12, 2, 4, 12, 2, 0,
  /* 2593 */ 21, 0, 12, 2, 4, 10, 4, 10, 4, 12, 2, 0,
  /* 2605 */ 10, 4, 10, 4, 10, 4, 12, 2, 0,
  /* 2614 */ 9, 5, 9, 5, 12, 2, 0,
  /* 2621 */ 30, 12, 2, 0,
  /* 2625 */ 0, 4, 35, 12, 1, 35, 12, 1, 35, 12, 2, 35, 12, 2, 0,
  /* 2640 */ 0, 4, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 0,
  /* 2667 */ 0, 4, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 0,
  /* 2685 */ 0, 4, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 0,
  /* 2700 */ 0, 4, 35, 12, 2, 35, 12, 2, 35, 12, 2, 0,
  /* 2712 */ 0, 4, 35, 12, 2, 35, 12, 2, 0,
  /* 2721 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 4, 35, 12, 2, 35, 12, 2, 0,
  /* 2739 */ 15, 3, 15, 7, 35, 12, 2, 35, 12, 2, 0,
  /* 2750 */ 15, 3, 35, 12, 2, 0,
  /* 2756 */ 21, 0, 15, 3, 15, 7, 35, 12, 2, 0,
  /* 2766 */ 13, 1, 13, 2, 13, 2, 0,
  /* 2773 */ 13, 2, 13, 2, 13, 2, 13, 2, 0,
  /* 2782 */ 12, 3, 12, 3, 13, 2, 13, 2, 0,
  /* 2791 */ 11, 4, 11, 4, 13, 2, 13, 2, 0,
  /* 2800 */ 10, 5, 13, 2, 13, 2, 0,
  /* 2807 */ 28, 2, 1, 15, 2, 0,
  /* 2813 */ 21, 0, 15, 7, 15, 7, 15, 2, 0,
  /* 2822 */ 34, 7, 15, 2, 0,
  /* 2827 */ 16, 1, 16, 2, 16, 2, 0,
  /* 2834 */ 16, 2, 16, 2, 16, 2, 16, 2, 0,
  /* 2843 */ 13, 3, 16, 2, 16, 2, 0,
  /* 2850 */ 12, 4, 12, 4, 16, 2, 16, 2, 0,
  /* 2859 */ 11, 5, 16, 2, 16, 2, 0,
  /* 2866 */ 17, 17, 17, 2, 0,
  /* 2871 */ 11, 40, 11, 40, 4, 11, 40, 2, 0,
  /* 2880 */ 11, 40, 11, 40, 11, 40, 11, 40, 2, 0,
  /* 2890 */ 15, 3, 27, 0, 3, 27, 0, 3, 27, 0, 3, 0,
  /* 2902 */ 0, 24, 1, 24, 3, 4, 5, 1, 3, 0,
  /* 2912 */ 12, 2, 12, 3, 12, 2, 3, 0,
  /* 2920 */ 12, 2, 12, 4, 12, 2, 3, 0,
  /* 2928 */ 12, 2, 12, 6, 12, 2, 3, 0,
  /* 2936 */ 12, 2, 13, 2, 12, 6, 12, 2, 3, 0,
  /* 2946 */ 0, 5, 4, 4, 4, 3, 3, 3, 3, 0,
  /* 2956 */ 0, 5, 4, 4, 3, 3, 3, 3, 0,
  /* 2965 */ 0, 5, 4, 3, 3, 3, 3, 0,
  /* 2973 */ 21, 2, 7, 7, 7, 7, 4, 4, 4, 4, 4, 4, 4, 4, 7, 7, 7, 7, 4, 4, 4, 3, 3, 4, 3, 3, 0,
  /* 3000 */ 21, 2, 7, 7, 7, 7, 4, 4, 4, 4, 4, 4, 7, 7, 7, 7, 4, 3, 3, 4, 3, 3, 0,
  /* 3023 */ 0, 5, 4, 4, 4, 3, 3, 0,
  /* 3031 */ 0, 5, 4, 4, 3, 3, 0,
  /* 3038 */ 0, 5, 4, 3, 3, 0,
  /* 3044 */ 12, 3, 12, 7, 4, 12, 3, 3, 0,
  /* 3053 */ 12, 3, 12, 4, 12, 3, 3, 0,
  /* 3061 */ 12, 3, 12, 6, 12, 3, 3, 0,
  /* 3069 */ 0, 14, 12, 3, 3, 0,
  /* 3075 */ 0, 24, 3, 3, 0,
  /* 3080 */ 42, 3, 3, 0,
  /* 3084 */ 0, 5, 4, 4, 4, 3, 0,
  /* 3091 */ 0, 5, 4, 4, 3, 0,
  /* 3097 */ 0, 14, 12, 4, 3, 0,
  /* 3103 */ 12, 6, 12, 2, 12, 6, 3, 0,
  /* 3111 */ 12, 6, 12, 6, 12, 6, 4, 12, 6, 3, 0,
  /* 3122 */ 12, 6, 12, 6, 12, 6, 12, 6, 3, 0,
  /* 3132 */ 12, 6, 11, 7, 11, 7, 12, 6, 3, 0,
  /* 3142 */ 12, 7, 12, 7, 12, 7, 3, 0,
  /* 3150 */ 9, 3, 10, 2, 9, 3, 0,
  /* 3157 */ 0, 9, 3, 9, 3, 0,
  /* 3163 */ 10, 2, 9, 3, 9, 3, 0,
  /* 3170 */ 9, 3, 9, 3, 9, 3, 0,
  /* 3177 */ 4, 9, 3, 9, 3, 0,
  /* 3183 */ 5, 5, 9, 3, 9, 3, 0,
  /* 3190 */ 11, 3, 12, 2, 12, 2, 11, 3, 0,
  /* 3199 */ 12, 2, 11, 3, 11, 3, 0,
  /* 3206 */ 11, 3, 11, 3, 11, 3, 11, 3, 0,
  /* 3215 */ 21, 0, 12, 2, 4, 11, 3, 11, 3, 0,
  /* 3225 */ 21, 0, 11, 3, 4, 11, 3, 11, 3, 0,
  /* 3235 */ 4, 4, 11, 3, 11, 3, 0,
  /* 3242 */ 10, 4, 10, 4, 11, 3, 11, 3, 0,
  /* 3251 */ 9, 5, 11, 3, 11, 3, 0,
  /* 3258 */ 12, 3, 12, 3, 11, 3, 0,
  /* 3265 */ 13, 3, 13, 3, 11, 3, 0,
  /* 3272 */ 21, 0, 11, 3, 4, 11, 3, 0,
  /* 3280 */ 11, 3, 11, 3, 4, 11, 3, 0,
  /* 3288 */ 13, 2, 12, 3, 12, 3, 0,
  /* 3295 */ 12, 3, 12, 3, 12, 3, 12, 3, 0,
  /* 3304 */ 11, 4, 11, 4, 12, 3, 12, 3, 0,
  /* 3313 */ 16, 2, 13, 3, 13, 3, 0,
  /* 3320 */ 13, 3, 13, 3, 13, 3, 13, 3, 0,
  /* 3329 */ 12, 4, 12, 4, 13, 3, 13, 3, 0,
  /* 3338 */ 0, 4, 28, 3, 1, 28, 3, 1, 15, 3, 0,
  /* 3349 */ 0, 4, 4, 28, 3, 1, 15, 3, 0,
  /* 3358 */ 5, 28, 3, 1, 15, 3, 0,
  /* 3365 */ 34, 7, 28, 3, 1, 15, 3, 0,
  /* 3373 */ 38, 7, 38, 7, 28, 3, 1, 15, 3, 0,
  /* 3383 */ 21, 0, 27, 0, 2, 27, 0, 2, 15, 3, 0,
  /* 3394 */ 21, 1, 27, 0, 3, 27, 0, 3, 27, 0, 3, 15, 3, 0,
  /* 3408 */ 21, 2, 27, 0, 4, 27, 0, 4, 27, 0, 4, 27, 0, 4, 15, 3, 0,
  /* 3425 */ 21, 3, 27, 0, 5, 27, 0, 5, 27, 0, 5, 27, 0, 5, 27, 0, 5, 15, 3, 0,
  /* 3445 */ 21, 4, 27, 0, 6, 27, 0, 6, 27, 0, 6, 27, 0, 6, 27, 0, 6, 27, 0, 6, 15, 3, 0,
  /* 3468 */ 21, 5, 27, 0, 7, 27, 0, 7, 27, 0, 7, 27, 0, 7, 27, 0, 7, 27, 0, 7, 27, 0, 7, 15, 3, 0,
  /* 3494 */ 0, 4, 28, 3, 1, 28, 3, 1, 15, 7, 15, 3, 0,
  /* 3507 */ 34, 7, 28, 3, 1, 34, 7, 15, 3, 0,
  /* 3517 */ 34, 7, 34, 7, 15, 3, 0,
  /* 3524 */ 36, 7, 36, 7, 15, 3, 0,
  /* 3531 */ 21, 6, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 15, 3, 0,
  /* 3560 */ 42, 15, 3, 0,
  /* 3564 */ 0, 14, 4, 4, 16, 3, 0,
  /* 3571 */ 15, 0, 24, 3, 0,
  /* 3576 */ 0, 24, 3, 24, 3, 0,
  /* 3582 */ 4, 24, 3, 0,
  /* 3586 */ 5, 24, 3, 0,
  /* 3590 */ 9, 40, 3, 0,
  /* 3594 */ 12, 40, 12, 40, 4, 12, 40, 3, 0,
  /* 3603 */ 12, 40, 12, 40, 12, 40, 12, 40, 3, 0,
  /* 3613 */ 15, 3, 27, 0, 4, 27, 0, 4, 27, 0, 4, 27, 0, 4, 0,
  /* 3628 */ 0, 24, 7, 24, 3, 14, 4, 4, 4, 4, 4, 3, 3, 3, 3, 5, 1, 1, 4, 0,
  /* 3648 */ 0, 24, 7, 24, 3, 14, 4, 4, 4, 4, 4, 3, 3, 3, 5, 1, 1, 4, 0,
  /* 3667 */ 0, 24, 7, 24, 3, 14, 4, 4, 4, 4, 3, 3, 3, 5, 1, 1, 4, 0,
  /* 3685 */ 0, 24, 7, 24, 3, 14, 4, 4, 4, 3, 3, 3, 5, 1, 1, 4, 0,
  /* 3702 */ 0, 24, 7, 24, 3, 14, 4, 4, 4, 3, 3, 5, 1, 1, 4, 0,
  /* 3718 */ 0, 24, 7, 24, 3, 14, 4, 4, 4, 4, 4, 3, 5, 1, 1, 4, 0,
  /* 3735 */ 0, 24, 7, 24, 3, 14, 4, 4, 4, 4, 3, 5, 1, 1, 4, 0,
  /* 3751 */ 0, 24, 7, 24, 3, 14, 4, 4, 4, 3, 5, 1, 1, 4, 0,
  /* 3766 */ 0, 24, 7, 24, 3, 14, 4, 4, 3, 5, 1, 1, 4, 0,
  /* 3780 */ 0, 24, 7, 24, 3, 14, 4, 3, 5, 1, 1, 4, 0,
  /* 3793 */ 0, 15, 3, 29, 1, 0, 28, 3, 1, 4, 0,
  /* 3804 */ 15, 3, 15, 7, 1, 28, 3, 1, 4, 0,
  /* 3814 */ 34, 7, 34, 7, 15, 3, 28, 3, 1, 4, 0,
  /* 3825 */ 28, 3, 1, 15, 3, 4, 28, 3, 1, 4, 0,
  /* 3836 */ 15, 3, 15, 7, 28, 3, 1, 4, 0,
  /* 3845 */ 15, 3, 15, 7, 15, 7, 28, 3, 1, 4, 0,
  /* 3856 */ 15, 3, 15, 7, 15, 7, 15, 7, 28, 3, 1, 4, 0,
  /* 3869 */ 15, 3, 15, 11, 28, 3, 1, 4, 0,
  /* 3878 */ 0, 15, 3, 15, 12, 28, 3, 1, 4, 0,
  /* 3888 */ 21, 0, 15, 3, 4, 15, 12, 28, 3, 1, 4, 0,
  /* 3900 */ 0, 15, 3, 15, 12, 15, 17, 28, 3, 1, 4, 0,
  /* 3912 */ 28, 3, 1, 15, 3, 15, 7, 19, 28, 3, 1, 4, 0,
  /* 3925 */ 21, 0, 4, 1, 4, 0,
  /* 3931 */ 4, 4, 4, 1, 4, 0,
  /* 3937 */ 6, 7, 7, 4, 4, 1, 4, 0,
  /* 3945 */ 0, 4, 4, 15, 0, 15, 7, 15, 7, 15, 7, 1, 4, 0,
  /* 3959 */ 9, 5, 9, 5, 14, 10, 4, 9, 1, 4, 0,
  /* 3970 */ 9, 8, 9, 8, 14, 10, 4, 9, 1, 4, 0,
  /* 3981 */ 10, 4, 10, 4, 14, 9, 5, 9, 1, 4, 0,
  /* 3992 */ 9, 5, 9, 5, 14, 9, 5, 9, 1, 4, 0,
  /* 4003 */ 10, 7, 10, 7, 14, 9, 5, 9, 1, 4, 0,
  /* 4014 */ 9, 8, 9, 8, 14, 9, 5, 9, 1, 4, 0,
  /* 4025 */ 10, 4, 10, 4, 14, 10, 4, 10, 1, 4, 0,
  /* 4036 */ 10, 5, 10, 5, 14, 10, 4, 10, 1, 4, 0,
  /* 4047 */ 10, 7, 10, 7, 14, 10, 4, 10, 1, 4, 0,
  /* 4058 */ 10, 8, 10, 8, 14, 10, 4, 10, 1, 4, 0,
  /* 4069 */ 10, 4, 10, 4, 14, 10, 5, 10, 1, 4, 0,
  /* 4080 */ 10, 5, 10, 5, 14, 10, 5, 10, 1, 4, 0,
  /* 4091 */ 10, 7, 10, 7, 14, 10, 5, 10, 1, 4, 0,
  /* 4102 */ 10, 8, 10, 8, 14, 10, 5, 10, 1, 4, 0,
  /* 4113 */ 11, 1, 11, 8, 11, 8, 4, 11, 1, 4, 0,
  /* 4124 */ 11, 4, 11, 4, 14, 11, 4, 11, 1, 4, 0,
  /* 4135 */ 11, 5, 11, 5, 14, 11, 4, 11, 1, 4, 0,
  /* 4146 */ 11, 7, 11, 7, 14, 11, 4, 11, 1, 4, 0,
  /* 4157 */ 11, 8, 11, 8, 14, 11, 4, 11, 1, 4, 0,
  /* 4168 */ 11, 4, 11, 4, 14, 11, 5, 11, 1, 4, 0,
  /* 4179 */ 11, 5, 11, 5, 14, 11, 5, 11, 1, 4, 0,
  /* 4190 */ 11, 7, 11, 7, 14, 11, 5, 11, 1, 4, 0,
  /* 4201 */ 11, 8, 11, 8, 14, 11, 5, 11, 1, 4, 0,
  /* 4212 */ 15, 1, 15, 11, 1, 28, 11, 1, 4, 0,
  /* 4222 */ 12, 1, 12, 7, 12, 7, 4, 12, 1, 4, 0,
  /* 4233 */ 12, 4, 12, 4, 14, 12, 4, 12, 1, 4, 0,
  /* 4244 */ 12, 7, 12, 7, 14, 12, 4, 12, 1, 4, 0,
  /* 4255 */ 13, 1, 13, 6, 13, 6, 4, 13, 1, 4, 0,
  /* 4266 */ 2, 14, 15, 1, 4, 0,
  /* 4272 */ 18, 15, 1, 4, 0,
  /* 4277 */ 12, 4, 12, 4, 16, 1, 4, 0,
  /* 4285 */ 15, 0, 24, 1, 4, 0,
  /* 4291 */ 0, 24, 3, 24, 1, 4, 0,
  /* 4298 */ 4, 24, 1, 4, 0,
  /* 4303 */ 45, 24, 1, 4, 0,
  /* 4308 */ 46, 24, 1, 4, 0,
  /* 4313 */ 31, 1, 41, 8, 31, 1, 4, 0,
  /* 4321 */ 41, 8, 4, 41, 8, 31, 1, 4, 0,
  /* 4330 */ 41, 8, 41, 8, 41, 8, 31, 1, 4, 0,
  /* 4340 */ 32, 1, 32, 1, 32, 1, 4, 0,
  /* 4348 */ 21, 0, 41, 1, 41, 1, 32, 1, 4, 0,
  /* 4358 */ 13, 4, 13, 4, 39, 1, 4, 0,
  /* 4366 */ 32, 1, 32, 1, 41, 1, 4, 0,
  /* 4374 */ 32, 1, 32, 1, 41, 1, 41, 1, 4, 0,
  /* 4384 */ 0, 41, 8, 41, 8, 5, 5, 41, 1, 4, 0,
  /* 4395 */ 41, 1, 41, 8, 41, 1, 4, 0,
  /* 4403 */ 41, 8, 5, 41, 8, 41, 1, 4, 0,
  /* 4412 */ 41, 8, 41, 8, 41, 8, 41, 1, 4, 0,
  /* 4422 */ 0, 41, 8, 5, 14, 41, 1, 4, 0,
  /* 4431 */ 52, 1, 52, 1, 4, 0,
  /* 4437 */ 15, 0, 4, 4, 4, 2, 4, 0,
  /* 4445 */ 10, 7, 10, 7, 10, 7, 10, 4, 4, 2, 4, 0,
  /* 4457 */ 9, 8, 9, 8, 9, 8, 9, 5, 4, 2, 4, 0,
  /* 4469 */ 11, 8, 11, 8, 11, 8, 11, 5, 4, 2, 4, 0,
  /* 4481 */ 2, 11, 6, 11, 6, 4, 2, 4, 0,
  /* 4490 */ 2, 10, 7, 10, 7, 4, 2, 4, 0,
  /* 4499 */ 2, 9, 8, 9, 8, 4, 2, 4, 0,
  /* 4508 */ 10, 4, 10, 4, 14, 10, 4, 2, 4, 0,
  /* 4518 */ 9, 5, 9, 5, 14, 10, 4, 2, 4, 0,
  /* 4528 */ 10, 5, 10, 5, 14, 10, 4, 2, 4, 0,
  /* 4538 */ 10, 7, 10, 7, 14, 10, 4, 2, 4, 0,
  /* 4548 */ 9, 8, 9, 8, 14, 10, 4, 2, 4, 0,
  /* 4558 */ 10, 8, 10, 8, 14, 10, 4, 2, 4, 0,
  /* 4568 */ 11, 4, 11, 8, 11, 4, 2, 4, 0,
  /* 4577 */ 11, 4, 11, 4, 14, 11, 4, 2, 4, 0,
  /* 4587 */ 11, 5, 11, 5, 14, 11, 4, 2, 4, 0,
  /* 4597 */ 11, 7, 11, 7, 14, 11, 4, 2, 4, 0,
  /* 4607 */ 11, 8, 11, 8, 14, 11, 4, 2, 4, 0,
  /* 4617 */ 10, 4, 10, 4, 14, 9, 5, 2, 4, 0,
  /* 4627 */ 9, 5, 9, 5, 14, 9, 5, 2, 4, 0,
  /* 4637 */ 10, 7, 10, 7, 14, 9, 5, 2, 4, 0,
  /* 4647 */ 9, 8, 9, 8, 14, 9, 5, 2, 4, 0,
  /* 4657 */ 10, 4, 10, 4, 14, 10, 5, 2, 4, 0,
  /* 4667 */ 10, 5, 10, 5, 14, 10, 5, 2, 4, 0,
  /* 4677 */ 10, 7, 10, 7, 14, 10, 5, 2, 4, 0,
  /* 4687 */ 10, 8, 10, 8, 14, 10, 5, 2, 4, 0,
  /* 4697 */ 11, 5, 11, 6, 11, 5, 2, 4, 0,
  /* 4706 */ 11, 5, 11, 7, 11, 5, 2, 4, 0,
  /* 4715 */ 11, 5, 11, 8, 11, 5, 2, 4, 0,
  /* 4724 */ 11, 4, 11, 4, 14, 11, 5, 2, 4, 0,
  /* 4734 */ 11, 5, 11, 5, 14, 11, 5, 2, 4, 0,
  /* 4744 */ 11, 7, 11, 7, 14, 11, 5, 2, 4, 0,
  /* 4754 */ 11, 8, 11, 8, 14, 11, 5, 2, 4, 0,
  /* 4764 */ 11, 6, 11, 6, 11, 6, 4, 11, 6, 2, 4, 0,
  /* 4776 */ 11, 6, 11, 6, 11, 6, 11, 6, 2, 4, 0,
  /* 4787 */ 11, 6, 11, 6, 10, 7, 11, 6, 2, 4, 0,
  /* 4798 */ 11, 6, 11, 6, 9, 8, 11, 6, 2, 4, 0,
  /* 4809 */ 11, 6, 11, 8, 11, 6, 2, 4, 0,
  /* 4818 */ 10, 7, 10, 7, 10, 7, 4, 10, 7, 2, 4, 0,
  /* 4830 */ 10, 7, 10, 7, 11, 6, 10, 7, 2, 4, 0,
  /* 4841 */ 10, 7, 10, 7, 10, 7, 10, 7, 2, 4, 0,
  /* 4852 */ 10, 7, 10, 7, 9, 8, 10, 7, 2, 4, 0,
  /* 4863 */ 11, 7, 11, 8, 11, 7, 2, 4, 0,
  /* 4872 */ 9, 8, 9, 8, 9, 8, 4, 9, 8, 2, 4, 0,
  /* 4884 */ 9, 8, 9, 8, 11, 6, 9, 8, 2, 4, 0,
  /* 4895 */ 9, 8, 9, 8, 10, 7, 9, 8, 2, 4, 0,
  /* 4906 */ 9, 8, 9, 8, 9, 8, 9, 8, 2, 4, 0,
  /* 4917 */ 11, 8, 11, 8, 11, 8, 4, 11, 8, 2, 4, 0,
  /* 4929 */ 11, 8, 11, 6, 11, 8, 2, 4, 0,
  /* 4938 */ 11, 8, 11, 7, 11, 8, 2, 4, 0,
  /* 4947 */ 11, 8, 11, 8, 11, 8, 11, 8, 2, 4, 0,
  /* 4958 */ 10, 2, 10, 2, 4, 0,
  /* 4964 */ 15, 3, 15, 7, 11, 2, 4, 0,
  /* 4972 */ 15, 3, 15, 7, 15, 7, 11, 2, 4, 0,
  /* 4982 */ 15, 3, 15, 11, 11, 2, 4, 0,
  /* 4990 */ 12, 2, 12, 2, 12, 2, 12, 2, 4, 0,
  /* 5000 */ 21, 0, 12, 2, 4, 12, 2, 12, 2, 12, 2, 4, 0,
  /* 5013 */ 11, 3, 12, 2, 12, 2, 4, 0,
  /* 5021 */ 21, 0, 12, 2, 4, 12, 2, 12, 2, 4, 0,
  /* 5032 */ 9, 5, 12, 2, 12, 2, 4, 0,
  /* 5040 */ 15, 3, 15, 7, 12, 2, 12, 2, 4, 0,
  /* 5050 */ 11, 3, 12, 2, 4, 0,
  /* 5056 */ 12, 2, 4, 12, 2, 4, 0,
  /* 5063 */ 15, 3, 15, 7, 12, 2, 4, 0,
  /* 5071 */ 15, 3, 15, 7, 15, 7, 12, 2, 4, 0,
  /* 5081 */ 15, 3, 15, 11, 12, 2, 4, 0,
  /* 5089 */ 15, 3, 15, 7, 15, 11, 12, 2, 4, 0,
  /* 5099 */ 21, 0, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 4, 0,
  /* 5118 */ 21, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 4, 0,
  /* 5149 */ 0, 4, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 35, 12, 2, 4, 0,
  /* 5168 */ 0, 4, 35, 12, 2, 35, 12, 2, 35, 12, 2, 4, 0,
  /* 5181 */ 0, 4, 35, 12, 2, 35, 12, 2, 4, 0,
  /* 5191 */ 15, 3, 15, 7, 35, 12, 2, 35, 12, 2, 4, 0,
  /* 5203 */ 15, 3, 4, 35, 12, 2, 4, 0,
  /* 5211 */ 21, 0, 15, 3, 15, 7, 4, 35, 12, 2, 4, 0,
  /* 5223 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 4, 35, 12, 2, 4, 0,
  /* 5239 */ 15, 3, 15, 7, 35, 12, 2, 4, 0,
  /* 5248 */ 0, 4, 15, 3, 15, 7, 15, 7, 35, 12, 2, 4, 0,
  /* 5261 */ 13, 2, 13, 2, 13, 2, 4, 0,
  /* 5269 */ 12, 3, 13, 2, 13, 2, 4, 0,
  /* 5277 */ 12, 3, 13, 2, 4, 0,
  /* 5283 */ 13, 2, 13, 3, 13, 2, 4, 0,
  /* 5291 */ 13, 2, 13, 6, 13, 2, 4, 0,
  /* 5299 */ 13, 2, 16, 2, 13, 6, 13, 2, 4, 0,
  /* 5309 */ 28, 2, 1, 15, 2, 4, 0,
  /* 5316 */ 13, 3, 16, 2, 16, 2, 4, 0,
  /* 5324 */ 13, 40, 2, 4, 0,
  /* 5329 */ 21, 0, 3, 4, 0,
  /* 5334 */ 12, 7, 12, 7, 12, 7, 12, 4, 4, 3, 4, 0,
  /* 5346 */ 12, 4, 12, 6, 12, 4, 3, 4, 0,
  /* 5355 */ 12, 4, 12, 7, 12, 4, 3, 4, 0,
  /* 5364 */ 12, 4, 12, 4, 14, 12, 4, 3, 4, 0,
  /* 5374 */ 12, 7, 12, 7, 14, 12, 4, 3, 4, 0,
  /* 5384 */ 12, 6, 12, 7, 12, 6, 3, 4, 0,
  /* 5393 */ 12, 7, 12, 3, 12, 7, 3, 4, 0,
  /* 5402 */ 12, 7, 12, 7, 12, 7, 4, 12, 7, 3, 4, 0,
  /* 5414 */ 12, 7, 12, 6, 12, 7, 3, 4, 0,
  /* 5423 */ 12, 7, 12, 7, 12, 7, 12, 7, 3, 4, 0,
  /* 5434 */ 9, 3, 9, 3, 4, 0,
  /* 5440 */ 11, 3, 11, 3, 11, 3, 11, 3, 4, 0,
  /* 5450 */ 21, 0, 11, 3, 4, 11, 3, 11, 3, 11, 3, 4, 0,
  /* 5463 */ 21, 0, 11, 3, 4, 11, 3, 11, 3, 4, 0,
  /* 5474 */ 9, 5, 11, 3, 11, 3, 4, 0,
  /* 5482 */ 10, 4, 11, 3, 4, 0,
  /* 5488 */ 10, 7, 11, 3, 4, 0,
  /* 5494 */ 12, 3, 12, 3, 12, 3, 4, 0,
  /* 5502 */ 11, 4, 12, 3, 4, 0,
  /* 5508 */ 13, 3, 13, 3, 4, 0,
  /* 5514 */ 0, 14, 13, 3, 4, 0,
  /* 5520 */ 28, 3, 1, 15, 3, 4, 0,
  /* 5527 */ 35, 12, 1, 35, 12, 1, 15, 3, 4, 0,
  /* 5537 */ 0, 4, 15, 3, 4, 0,
  /* 5543 */ 4, 4, 15, 3, 4, 0,
  /* 5549 */ 36, 7, 36, 7, 15, 3, 4, 0,
  /* 5557 */ 0, 28, 3, 1, 14, 15, 3, 4, 0,
  /* 5566 */ 0, 24, 3, 4, 0,
  /* 5571 */ 1, 24, 3, 4, 0,
  /* 5576 */ 0, 24, 7, 24, 3, 24, 3, 4, 0,
  /* 5585 */ 5, 24, 3, 4, 0,
  /* 5590 */ 24, 7, 24, 3, 4, 0,
  /* 5596 */ 0, 15, 3, 28, 3, 1, 14, 28, 3, 4, 0,
  /* 5607 */ 21, 0, 4, 4, 0,
  /* 5612 */ 15, 3, 15, 7, 15, 7, 4, 28, 3, 1, 4, 4, 0,
  /* 5625 */ 15, 3, 15, 7, 28, 3, 1, 4, 4, 0,
  /* 5635 */ 21, 0, 4, 1, 4, 4, 0,
  /* 5642 */ 0, 15, 3, 14, 15, 9, 1, 4, 4, 0,
  /* 5652 */ 0, 4, 15, 2, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5664 */ 0, 4, 4, 15, 2, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5677 */ 0, 4, 7, 15, 2, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5690 */ 0, 4, 4, 7, 15, 2, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5704 */ 0, 4, 15, 2, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5718 */ 0, 4, 4, 15, 2, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5733 */ 0, 4, 7, 15, 2, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5748 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5764 */ 0, 4, 15, 2, 15, 7, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5780 */ 0, 4, 4, 15, 2, 15, 7, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5797 */ 0, 4, 7, 15, 2, 15, 7, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5814 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5832 */ 0, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5850 */ 0, 4, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5869 */ 0, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5888 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 8, 15, 16, 1, 4, 4, 0,
  /* 5908 */ 15, 0, 24, 1, 4, 4, 0,
  /* 5915 */ 0, 4, 15, 2, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 5929 */ 0, 4, 4, 15, 2, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 5944 */ 15, 0, 4, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 5957 */ 15, 0, 4, 4, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 5971 */ 0, 4, 15, 2, 7, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 5986 */ 0, 4, 4, 15, 2, 7, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6002 */ 15, 0, 4, 7, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6016 */ 15, 0, 4, 4, 7, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6031 */ 0, 4, 15, 2, 15, 7, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6047 */ 0, 4, 4, 15, 2, 15, 7, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6064 */ 0, 4, 7, 15, 2, 15, 7, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6081 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 10, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6099 */ 0, 4, 15, 2, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6115 */ 0, 4, 4, 15, 2, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6132 */ 15, 0, 4, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6147 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6163 */ 0, 4, 15, 2, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6180 */ 0, 4, 4, 15, 2, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6198 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6214 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6231 */ 0, 4, 15, 2, 15, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6249 */ 0, 4, 4, 15, 2, 15, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6268 */ 0, 4, 7, 15, 2, 15, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6287 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6307 */ 0, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6329 */ 0, 4, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6352 */ 0, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6375 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6399 */ 0, 4, 15, 2, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6417 */ 0, 4, 4, 15, 2, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6436 */ 15, 0, 4, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6453 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6471 */ 0, 4, 15, 2, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6490 */ 0, 4, 4, 15, 2, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6510 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6528 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6547 */ 0, 4, 15, 2, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6567 */ 0, 4, 4, 15, 2, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6588 */ 0, 4, 7, 15, 2, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6609 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6631 */ 0, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6655 */ 0, 4, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6680 */ 0, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6705 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6731 */ 0, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6759 */ 0, 4, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6788 */ 0, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6817 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6847 */ 0, 4, 15, 2, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6867 */ 0, 4, 4, 15, 2, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6888 */ 15, 0, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6907 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6927 */ 0, 4, 15, 2, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6948 */ 0, 4, 4, 15, 2, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6970 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 6990 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7011 */ 0, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7037 */ 0, 4, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7064 */ 0, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7091 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7119 */ 0, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7149 */ 0, 4, 4, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7180 */ 0, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7211 */ 0, 4, 4, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 16, 15, 24, 1, 4, 4, 0,
  /* 7243 */ 15, 0, 4, 15, 10, 7, 15, 18, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7259 */ 15, 0, 4, 4, 15, 10, 7, 15, 18, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7276 */ 15, 0, 4, 15, 10, 15, 18, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7291 */ 15, 0, 4, 4, 15, 10, 15, 18, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7307 */ 15, 0, 4, 15, 10, 15, 15, 15, 18, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7324 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 18, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7342 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 18, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7360 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 18, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7379 */ 15, 0, 4, 15, 10, 7, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7397 */ 15, 0, 4, 4, 15, 10, 7, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7416 */ 15, 0, 4, 15, 10, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7433 */ 15, 0, 4, 4, 15, 10, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7451 */ 15, 0, 4, 15, 10, 15, 15, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7470 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7490 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7510 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7531 */ 15, 0, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7554 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7578 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7602 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7627 */ 15, 0, 4, 15, 10, 7, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7647 */ 15, 0, 4, 4, 15, 10, 7, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7668 */ 15, 0, 4, 15, 10, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7687 */ 15, 0, 4, 4, 15, 10, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7707 */ 15, 0, 4, 15, 10, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7728 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7750 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7772 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7795 */ 15, 0, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7820 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7846 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7872 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7899 */ 15, 0, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7928 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7958 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 7988 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8019 */ 15, 0, 4, 15, 10, 7, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8041 */ 15, 0, 4, 4, 15, 10, 7, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8064 */ 15, 0, 4, 15, 10, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8085 */ 15, 0, 4, 4, 15, 10, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8107 */ 15, 0, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8134 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8162 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8190 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8219 */ 15, 0, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8250 */ 15, 0, 4, 4, 15, 10, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8282 */ 15, 0, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8314 */ 15, 0, 4, 4, 7, 15, 10, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 18, 15, 23, 15, 23, 15, 23, 15, 24, 15, 32, 1, 4, 4, 0,
  /* 8347 */ 21, 0, 2, 4, 2, 4, 4, 0,
  /* 8355 */ 11, 6, 11, 6, 11, 6, 11, 6, 2, 4, 4, 0,
  /* 8367 */ 10, 7, 10, 7, 10, 7, 10, 7, 2, 4, 4, 0,
  /* 8379 */ 9, 8, 9, 8, 9, 8, 9, 8, 2, 4, 4, 0,
  /* 8391 */ 31, 1, 31, 1, 41, 1, 12, 2, 4, 4, 0,
  /* 8402 */ 31, 1, 31, 1, 12, 2, 12, 2, 4, 4, 0,
  /* 8413 */ 11, 3, 11, 3, 4, 4, 0,
  /* 8420 */ 13, 3, 13, 6, 13, 3, 4, 4, 0,
  /* 8429 */ 0, 24, 1, 24, 3, 4, 4, 0,
  /* 8437 */ 21, 0, 4, 1, 4, 4, 4, 0,
  /* 8445 */ 0, 24, 6, 5, 5, 4, 1, 4, 4, 4, 0,
  /* 8456 */ 0, 24, 6, 24, 6, 5, 4, 1, 4, 4, 4, 0,
  /* 8468 */ 0, 24, 1, 24, 1, 4, 4, 4, 0,
  /* 8477 */ 32, 1, 32, 1, 12, 2, 4, 4, 4, 0,
  /* 8487 */ 32, 1, 32, 1, 41, 1, 12, 2, 4, 4, 4, 0,
  /* 8499 */ 31, 1, 31, 1, 12, 2, 12, 2, 4, 4, 4, 0,
  /* 8511 */ 9, 3, 4, 4, 4, 0,
  /* 8517 */ 21, 0, 15, 3, 4, 4, 4, 0,
  /* 8525 */ 0, 24, 1, 24, 3, 4, 4, 4, 0,
  /* 8534 */ 0, 15, 4, 24, 3, 4, 4, 4, 0,
  /* 8543 */ 21, 0, 4, 4, 4, 4, 0,
  /* 8550 */ 21, 0, 4, 1, 4, 4, 4, 4, 0,
  /* 8559 */ 21, 0, 15, 3, 4, 4, 4, 4, 0,
  /* 8568 */ 21, 0, 4, 4, 4, 4, 4, 0,
  /* 8576 */ 0, 10, 4, 24, 3, 4, 4, 4, 4, 4, 0,
  /* 8587 */ 0, 24, 8, 24, 3, 4, 4, 4, 4, 4, 0,
  /* 8598 */ 21, 0, 4, 4, 4, 4, 4, 4, 0,
  /* 8607 */ 0, 10, 4, 24, 3, 4, 4, 4, 4, 4, 4, 0,
  /* 8619 */ 0, 24, 8, 24, 3, 4, 4, 4, 4, 4, 4, 0,
  /* 8631 */ 21, 0, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8641 */ 21, 0, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8652 */ 21, 2, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8666 */ 21, 2, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8683 */ 12, 2, 12, 2, 12, 2, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8706 */ 21, 2, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8727 */ 21, 6, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8751 */ 21, 6, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8774 */ 10, 4, 10, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8788 */ 0, 15, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8801 */ 10, 7, 10, 7, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8815 */ 0, 15, 4, 4, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8827 */ 0, 5, 4, 4, 4, 4, 4, 4, 4, 0,
  /* 8837 */ 0, 5, 4, 4, 4, 4, 4, 4, 0,
  /* 8846 */ 10, 7, 15, 3, 15, 11, 10, 7, 4, 4, 4, 4, 4, 4, 0,
  /* 8861 */ 12, 7, 15, 3, 15, 11, 12, 7, 4, 4, 4, 4, 4, 4, 0,
  /* 8876 */ 0, 15, 0, 10, 4, 4, 4, 4, 4, 4, 0,
  /* 8887 */ 15, 3, 15, 7, 15, 11, 4, 4, 4, 4, 4, 4, 0,
  /* 8900 */ 0, 5, 4, 4, 4, 4, 4, 0,
  /* 8908 */ 0, 15, 0, 24, 8, 4, 4, 4, 4, 4, 0,
  /* 8919 */ 0, 15, 0, 10, 4, 4, 4, 4, 4, 0,
  /* 8929 */ 15, 0, 15, 7, 10, 4, 4, 4, 4, 4, 0,
  /* 8940 */ 15, 2, 15, 7, 10, 4, 4, 4, 4, 4, 0,
  /* 8951 */ 15, 1, 15, 7, 15, 7, 10, 4, 4, 4, 4, 4, 0,
  /* 8964 */ 0, 15, 4, 4, 4, 4, 4, 0,
  /* 8972 */ 4, 15, 4, 4, 4, 4, 4, 0,
  /* 8980 */ 0, 5, 4, 4, 4, 4, 0,
  /* 8987 */ 15, 3, 15, 7, 4, 4, 4, 4, 0,
  /* 8996 */ 0, 15, 0, 24, 8, 4, 4, 4, 4, 0,
  /* 9006 */ 15, 0, 15, 7, 24, 8, 4, 4, 4, 4, 0,
  /* 9017 */ 15, 2, 15, 7, 24, 8, 4, 4, 4, 4, 0,
  /* 9028 */ 15, 1, 15, 7, 15, 7, 24, 8, 4, 4, 4, 4, 0,
  /* 9041 */ 0, 15, 0, 10, 4, 4, 4, 4, 0,
  /* 9050 */ 0, 15, 2, 10, 4, 4, 4, 4, 0,
  /* 9059 */ 10, 4, 4, 4, 10, 4, 4, 4, 4, 0,
  /* 9069 */ 10, 4, 9, 4, 10, 4, 10, 4, 4, 4, 4, 0,
  /* 9081 */ 10, 4, 10, 4, 10, 4, 10, 4, 4, 4, 4, 0,
  /* 9093 */ 10, 4, 10, 4, 11, 4, 10, 4, 4, 4, 4, 0,
  /* 9105 */ 10, 4, 5, 5, 10, 4, 4, 4, 4, 0,
  /* 9115 */ 15, 0, 15, 7, 10, 4, 4, 4, 4, 0,
  /* 9125 */ 15, 2, 15, 7, 10, 4, 4, 4, 4, 0,
  /* 9135 */ 15, 1, 15, 7, 15, 7, 10, 4, 4, 4, 4, 0,
  /* 9147 */ 12, 4, 4, 4, 12, 4, 4, 4, 4, 0,
  /* 9157 */ 12, 4, 9, 4, 10, 4, 12, 4, 4, 4, 4, 0,
  /* 9169 */ 12, 4, 10, 4, 10, 4, 12, 4, 4, 4, 4, 0,
  /* 9181 */ 12, 4, 10, 4, 11, 4, 12, 4, 4, 4, 4, 0,
  /* 9193 */ 12, 4, 5, 5, 12, 4, 4, 4, 4, 0,
  /* 9203 */ 13, 4, 4, 4, 13, 4, 4, 4, 4, 0,
  /* 9213 */ 0, 14, 4, 4, 4, 4, 0,
  /* 9220 */ 0, 15, 4, 4, 4, 4, 0,
  /* 9227 */ 4, 15, 4, 4, 4, 4, 0,
  /* 9234 */ 0, 5, 4, 4, 4, 0,
  /* 9240 */ 0, 24, 6, 5, 5, 4, 1, 5, 4, 4, 4, 0,
  /* 9252 */ 0, 24, 6, 24, 6, 5, 4, 1, 5, 4, 4, 4, 0,
  /* 9265 */ 21, 0, 3, 3, 5, 4, 4, 4, 0,
  /* 9274 */ 21, 2, 3, 3, 3, 3, 5, 4, 4, 4, 0,
  /* 9285 */ 21, 0, 4, 4, 5, 4, 4, 4, 0,
  /* 9294 */ 21, 2, 4, 4, 4, 4, 5, 4, 4, 4, 0,
  /* 9305 */ 21, 2, 4, 4, 4, 4, 5, 5, 4, 4, 4, 0,
  /* 9317 */ 21, 0, 5, 5, 5, 4, 4, 4, 0,
  /* 9326 */ 21, 2, 7, 7, 7, 7, 5, 5, 4, 4, 4, 0,
  /* 9338 */ 0, 24, 6, 5, 5, 4, 1, 24, 6, 5, 4, 4, 4, 0,
  /* 9352 */ 0, 24, 6, 24, 6, 5, 4, 1, 24, 6, 5, 4, 4, 4, 0,
  /* 9367 */ 21, 2, 7, 7, 7, 7, 5, 4, 4, 4, 0,
  /* 9378 */ 4, 6, 4, 4, 4, 0,
  /* 9384 */ 11, 6, 4, 4, 4, 0,
  /* 9390 */ 0, 24, 6, 5, 5, 4, 1, 24, 6, 4, 4, 4, 0,
  /* 9403 */ 0, 24, 6, 24, 6, 5, 4, 1, 24, 6, 4, 4, 4, 0,
  /* 9417 */ 4, 7, 4, 4, 4, 0,
  /* 9423 */ 7, 7, 7, 4, 4, 4, 0,
  /* 9430 */ 0, 10, 7, 4, 4, 4, 0,
  /* 9437 */ 10, 7, 9, 3, 9, 3, 10, 7, 4, 4, 4, 0,
  /* 9449 */ 10, 7, 10, 3, 10, 3, 10, 7, 4, 4, 4, 0,
  /* 9461 */ 10, 7, 10, 3, 11, 3, 10, 7, 4, 4, 4, 0,
  /* 9473 */ 10, 7, 9, 4, 10, 4, 10, 7, 4, 4, 4, 0,
  /* 9485 */ 10, 7, 10, 4, 11, 4, 10, 7, 4, 4, 4, 0,
  /* 9497 */ 10, 7, 5, 5, 10, 7, 4, 4, 4, 0,
  /* 9507 */ 10, 7, 10, 6, 10, 6, 10, 7, 4, 4, 4, 0,
  /* 9519 */ 10, 7, 10, 6, 11, 6, 10, 7, 4, 4, 4, 0,
  /* 9531 */ 10, 7, 11, 6, 11, 6, 10, 7, 4, 4, 4, 0,
  /* 9543 */ 10, 7, 11, 6, 12, 6, 10, 7, 4, 4, 4, 0,
  /* 9555 */ 10, 7, 7, 7, 10, 7, 4, 4, 4, 0,
  /* 9565 */ 10, 7, 9, 7, 9, 7, 10, 7, 4, 4, 4, 0,
  /* 9577 */ 10, 7, 11, 40, 11, 40, 10, 7, 4, 4, 4, 0,
  /* 9589 */ 10, 7, 11, 40, 12, 40, 10, 7, 4, 4, 4, 0,
  /* 9601 */ 11, 7, 4, 4, 4, 0,
  /* 9607 */ 12, 7, 9, 3, 9, 3, 12, 7, 4, 4, 4, 0,
  /* 9619 */ 12, 7, 10, 3, 10, 3, 12, 7, 4, 4, 4, 0,
  /* 9631 */ 12, 7, 10, 3, 11, 3, 12, 7, 4, 4, 4, 0,
  /* 9643 */ 12, 7, 9, 4, 10, 4, 12, 7, 4, 4, 4, 0,
  /* 9655 */ 12, 7, 10, 4, 11, 4, 12, 7, 4, 4, 4, 0,
  /* 9667 */ 12, 7, 5, 5, 12, 7, 4, 4, 4, 0,
  /* 9677 */ 12, 7, 10, 6, 10, 6, 12, 7, 4, 4, 4, 0,
  /* 9689 */ 12, 7, 10, 6, 11, 6, 12, 7, 4, 4, 4, 0,
  /* 9701 */ 12, 7, 11, 6, 11, 6, 12, 7, 4, 4, 4, 0,
  /* 9713 */ 12, 7, 11, 6, 12, 6, 12, 7, 4, 4, 4, 0,
  /* 9725 */ 12, 7, 7, 7, 12, 7, 4, 4, 4, 0,
  /* 9735 */ 12, 7, 9, 7, 9, 7, 12, 7, 4, 4, 4, 0,
  /* 9747 */ 12, 7, 11, 40, 11, 40, 12, 7, 4, 4, 4, 0,
  /* 9759 */ 12, 7, 11, 40, 12, 40, 12, 7, 4, 4, 4, 0,
  /* 9771 */ 13, 7, 9, 3, 9, 3, 13, 7, 4, 4, 4, 0,
  /* 9783 */ 13, 7, 10, 3, 10, 3, 13, 7, 4, 4, 4, 0,
  /* 9795 */ 13, 7, 10, 6, 10, 6, 13, 7, 4, 4, 4, 0,
  /* 9807 */ 13, 7, 7, 7, 13, 7, 4, 4, 4, 0,
  /* 9817 */ 15, 3, 15, 7, 15, 7, 15, 7, 4, 4, 4, 0,
  /* 9829 */ 8, 8, 8, 8, 4, 4, 4, 0,
  /* 9837 */ 10, 8, 8, 8, 10, 8, 4, 4, 4, 0,
  /* 9847 */ 0, 15, 0, 24, 8, 4, 4, 4, 0,
  /* 9856 */ 0, 15, 2, 24, 8, 4, 4, 4, 0,
  /* 9865 */ 15, 0, 15, 7, 24, 8, 4, 4, 4, 0,
  /* 9875 */ 15, 2, 15, 7, 24, 8, 4, 4, 4, 0,
  /* 9885 */ 15, 1, 15, 7, 15, 7, 24, 8, 4, 4, 4, 0,
  /* 9897 */ 11, 6, 9, 4, 4, 4, 0,
  /* 9904 */ 11, 7, 9, 4, 4, 4, 0,
  /* 9911 */ 11, 40, 9, 4, 4, 4, 0,
  /* 9918 */ 15, 0, 10, 4, 4, 4, 0,
  /* 9925 */ 0, 24, 6, 5, 5, 4, 1, 10, 4, 4, 4, 0,
  /* 9937 */ 0, 24, 6, 24, 6, 5, 4, 1, 10, 4, 4, 4, 0,
  /* 9950 */ 10, 4, 10, 4, 4, 4, 0,
  /* 9957 */ 0, 24, 6, 5, 5, 4, 1, 5, 10, 4, 4, 4, 0,
  /* 9970 */ 0, 24, 6, 24, 6, 5, 4, 1, 5, 10, 4, 4, 4, 0,
  /* 9984 */ 0, 24, 6, 5, 5, 4, 1, 24, 6, 5, 10, 4, 4, 4, 0,
  /* 9999 */ 0, 24, 6, 24, 6, 5, 4, 1, 24, 6, 5, 10, 4, 4, 4, 0,
  /* 10015 */ 0, 24, 6, 5, 5, 4, 1, 24, 6, 10, 4, 4, 4, 0,
  /* 10029 */ 0, 24, 6, 24, 6, 5, 4, 1, 24, 6, 10, 4, 4, 4, 0,
  /* 10044 */ 0, 24, 6, 5, 5, 4, 1, 11, 4, 4, 4, 0,
  /* 10056 */ 0, 24, 6, 24, 6, 5, 4, 1, 11, 4, 4, 4, 0,
  /* 10069 */ 11, 4, 11, 4, 4, 4, 0,
  /* 10076 */ 0, 24, 6, 5, 5, 4, 1, 5, 11, 4, 4, 4, 0,
  /* 10089 */ 0, 24, 6, 24, 6, 5, 4, 1, 5, 11, 4, 4, 4, 0,
  /* 10103 */ 0, 24, 6, 5, 5, 4, 1, 24, 6, 5, 11, 4, 4, 4, 0,
  /* 10118 */ 0, 24, 6, 24, 6, 5, 4, 1, 24, 6, 5, 11, 4, 4, 4, 0,
  /* 10134 */ 0, 24, 6, 5, 5, 4, 1, 24, 6, 11, 4, 4, 4, 0,
  /* 10148 */ 0, 24, 6, 24, 6, 5, 4, 1, 24, 6, 11, 4, 4, 4, 0,
  /* 10163 */ 15, 3, 15, 11, 4, 4, 4, 0,
  /* 10171 */ 15, 3, 15, 7, 15, 11, 4, 4, 4, 0,
  /* 10181 */ 13, 4, 13, 4, 13, 4, 4, 4, 0,
  /* 10190 */ 4, 14, 4, 4, 4, 0,
  /* 10196 */ 14, 14, 4, 4, 4, 0,
  /* 10202 */ 0, 15, 4, 4, 4, 0,
  /* 10208 */ 16, 4, 16, 4, 16, 4, 4, 4, 0,
  /* 10217 */ 15, 3, 15, 11, 15, 19, 4, 4, 4, 0,
  /* 10227 */ 15, 3, 15, 12, 15, 19, 4, 4, 4, 0,
  /* 10237 */ 11, 40, 4, 4, 4, 0,
  /* 10243 */ 12, 6, 44, 4, 4, 4, 0,
  /* 10250 */ 12, 7, 44, 4, 4, 4, 0,
  /* 10257 */ 12, 40, 44, 4, 4, 4, 0,
  /* 10264 */ 21, 0, 3, 3, 5, 4, 4, 0,
  /* 10272 */ 21, 2, 3, 3, 3, 3, 5, 4, 4, 0,
  /* 10282 */ 21, 0, 4, 4, 5, 4, 4, 0,
  /* 10290 */ 21, 2, 4, 4, 4, 4, 5, 4, 4, 0,
  /* 10300 */ 21, 2, 4, 4, 4, 4, 5, 5, 4, 4, 0,
  /* 10311 */ 21, 0, 5, 5, 5, 4, 4, 0,
  /* 10319 */ 21, 2, 7, 7, 7, 7, 5, 5, 4, 4, 0,
  /* 10330 */ 0, 14, 5, 5, 4, 4, 0,
  /* 10337 */ 21, 2, 7, 7, 7, 7, 5, 4, 4, 0,
  /* 10347 */ 9, 5, 9, 5, 4, 4, 0,
  /* 10354 */ 0, 14, 5, 4, 4, 0,
  /* 10360 */ 14, 14, 5, 4, 4, 0,
  /* 10366 */ 21, 0, 9, 6, 9, 6, 4, 4, 4, 4, 4, 4, 4, 4, 9, 6, 9, 6, 4, 4, 0,
  /* 10387 */ 21, 0, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 4, 4, 0,
  /* 10408 */ 21, 0, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 4, 4, 0,
  /* 10437 */ 4, 11, 6, 11, 6, 4, 4, 0,
  /* 10445 */ 13, 6, 13, 6, 13, 6, 4, 13, 6, 4, 4, 0,
  /* 10457 */ 13, 6, 13, 6, 13, 6, 13, 6, 4, 4, 0,
  /* 10468 */ 13, 6, 12, 7, 12, 7, 13, 6, 4, 4, 0,
  /* 10479 */ 0, 24, 6, 5, 5, 4, 1, 24, 6, 24, 6, 4, 4, 0,
  /* 10493 */ 0, 24, 6, 24, 6, 5, 4, 1, 24, 6, 24, 6, 4, 4, 0,
  /* 10508 */ 0, 24, 6, 5, 5, 4, 1, 24, 6, 24, 6, 24, 6, 4, 4, 0,
  /* 10524 */ 0, 24, 6, 24, 6, 5, 4, 1, 24, 6, 24, 6, 24, 6, 4, 4, 0,
  /* 10541 */ 21, 0, 7, 1, 7, 4, 4, 0,
  /* 10549 */ 21, 0, 7, 1, 4, 7, 4, 4, 0,
  /* 10558 */ 7, 4, 7, 4, 4, 0,
  /* 10564 */ 21, 2, 7, 7, 7, 7, 4, 4, 4, 4, 4, 4, 4, 4, 7, 7, 7, 7, 4, 4, 0,
  /* 10585 */ 21, 2, 7, 7, 7, 7, 4, 4, 4, 4, 7, 7, 7, 7, 4, 4, 0,
  /* 10602 */ 21, 2, 7, 7, 7, 7, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 7, 7, 7, 7, 4, 4, 0,
  /* 10631 */ 21, 2, 7, 7, 7, 7, 9, 6, 9, 6, 9, 6, 9, 6, 7, 7, 7, 7, 4, 4, 0,
  /* 10652 */ 0, 10, 7, 4, 4, 0,
  /* 10658 */ 4, 10, 7, 10, 7, 4, 4, 0,
  /* 10666 */ 15, 1, 15, 7, 4, 4, 0,
  /* 10673 */ 21, 0, 15, 3, 15, 7, 4, 4, 0,
  /* 10682 */ 21, 0, 4, 15, 3, 15, 7, 4, 4, 0,
  /* 10692 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 4, 4, 0,
  /* 10705 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 12, 15, 7, 15, 7, 15, 7, 15, 7, 4, 4, 0,
  /* 10728 */ 21, 1, 15, 3, 15, 7, 15, 7, 15, 12, 15, 7, 15, 7, 15, 7, 4, 4, 0,
  /* 10747 */ 21, 0, 15, 3, 15, 7, 15, 12, 15, 7, 15, 7, 4, 4, 0,
  /* 10762 */ 15, 3, 15, 7, 37, 7, 37, 7, 4, 4, 0,
  /* 10773 */ 4, 9, 8, 9, 8, 4, 4, 0,
  /* 10781 */ 15, 0, 15, 8, 4, 4, 0,
  /* 10788 */ 21, 0, 15, 0, 1, 15, 8, 4, 4, 0,
  /* 10798 */ 0, 24, 8, 4, 4, 0,
  /* 10804 */ 41, 8, 41, 8, 4, 4, 0,
  /* 10811 */ 21, 0, 4, 4, 4, 4, 10, 4, 4, 0,
  /* 10821 */ 0, 10, 4, 10, 4, 4, 0,
  /* 10828 */ 0, 14, 10, 1, 10, 4, 10, 4, 4, 0,
  /* 10838 */ 0, 14, 2, 10, 4, 10, 4, 4, 0,
  /* 10847 */ 21, 0, 10, 4, 4, 10, 4, 10, 4, 4, 0,
  /* 10858 */ 21, 0, 10, 4, 4, 10, 4, 10, 4, 10, 4, 4, 0,
  /* 10871 */ 10, 4, 10, 4, 10, 4, 10, 4, 4, 0,
  /* 10881 */ 0, 10, 4, 11, 4, 10, 4, 10, 4, 4, 0,
  /* 10892 */ 9, 5, 10, 4, 10, 4, 4, 0,
  /* 10900 */ 0, 14, 9, 1, 9, 5, 10, 4, 4, 0,
  /* 10910 */ 0, 14, 2, 9, 5, 10, 4, 4, 0,
  /* 10919 */ 0, 14, 10, 1, 10, 5, 10, 4, 4, 0,
  /* 10929 */ 0, 14, 2, 10, 5, 10, 4, 4, 0,
  /* 10938 */ 10, 7, 10, 4, 4, 0,
  /* 10944 */ 21, 0, 4, 4, 4, 4, 11, 4, 4, 0,
  /* 10954 */ 21, 0, 5, 4, 4, 4, 11, 4, 4, 0,
  /* 10964 */ 0, 10, 4, 11, 4, 4, 0,
  /* 10971 */ 0, 14, 11, 1, 11, 4, 11, 4, 4, 0,
  /* 10981 */ 0, 14, 2, 11, 4, 11, 4, 4, 0,
  /* 10990 */ 11, 4, 11, 4, 11, 4, 11, 4, 4, 0,
  /* 11000 */ 10, 5, 11, 4, 4, 0,
  /* 11006 */ 0, 14, 11, 1, 11, 5, 11, 4, 4, 0,
  /* 11016 */ 0, 14, 2, 11, 5, 11, 4, 4, 0,
  /* 11025 */ 16, 1, 16, 1, 12, 4, 4, 0,
  /* 11033 */ 0, 14, 12, 1, 12, 4, 12, 4, 4, 0,
  /* 11043 */ 0, 14, 3, 12, 4, 12, 4, 4, 0,
  /* 11052 */ 12, 4, 12, 4, 12, 4, 12, 4, 4, 0,
  /* 11062 */ 13, 4, 13, 4, 12, 4, 12, 4, 4, 0,
  /* 11072 */ 13, 4, 13, 4, 12, 4, 4, 0,
  /* 11080 */ 15, 4, 15, 12, 4, 4, 0,
  /* 11087 */ 39, 1, 39, 1, 13, 4, 4, 0,
  /* 11095 */ 12, 4, 12, 4, 13, 4, 4, 0,
  /* 11103 */ 13, 4, 13, 4, 13, 4, 13, 4, 4, 0,
  /* 11113 */ 16, 4, 16, 4, 13, 4, 13, 4, 4, 0,
  /* 11123 */ 16, 4, 16, 4, 13, 4, 4, 0,
  /* 11131 */ 0, 35, 9, 1, 14, 4, 4, 0,
  /* 11139 */ 0, 35, 10, 1, 14, 4, 4, 0,
  /* 11147 */ 0, 35, 11, 1, 14, 4, 4, 0,
  /* 11155 */ 0, 35, 12, 1, 14, 4, 4, 0,
  /* 11163 */ 0, 35, 25, 1, 14, 4, 4, 0,
  /* 11171 */ 0, 12, 2, 14, 4, 4, 0,
  /* 11178 */ 0, 13, 2, 14, 4, 4, 0,
  /* 11185 */ 0, 11, 3, 14, 4, 4, 0,
  /* 11192 */ 0, 12, 3, 14, 4, 4, 0,
  /* 11199 */ 0, 10, 4, 14, 4, 4, 0,
  /* 11206 */ 0, 11, 4, 14, 4, 4, 0,
  /* 11213 */ 0, 9, 5, 14, 4, 4, 0,
  /* 11220 */ 0, 10, 5, 14, 4, 4, 0,
  /* 11227 */ 14, 14, 14, 4, 4, 0,
  /* 11233 */ 21, 0, 4, 4, 15, 4, 4, 0,
  /* 11241 */ 21, 2, 4, 4, 4, 4, 15, 4, 4, 0,
  /* 11251 */ 21, 6, 4, 4, 4, 4, 4, 4, 4, 4, 15, 4, 4, 0,
  /* 11265 */ 21, 2, 9, 6, 9, 6, 9, 6, 9, 6, 15, 4, 4, 0,
  /* 11279 */ 21, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 15, 4, 4, 0,
  /* 11301 */ 21, 6, 7, 7, 7, 7, 7, 7, 7, 7, 15, 4, 4, 0,
  /* 11315 */ 21, 0, 8, 8, 15, 4, 4, 0,
  /* 11323 */ 15, 3, 15, 11, 15, 15, 4, 4, 0,
  /* 11332 */ 0, 15, 4, 15, 11, 15, 15, 4, 4, 0,
  /* 11342 */ 0, 15, 4, 15, 11, 15, 15, 15, 15, 4, 4, 0,
  /* 11354 */ 0, 15, 4, 15, 11, 15, 15, 15, 15, 15, 15, 4, 4, 0,
  /* 11368 */ 13, 4, 13, 4, 16, 4, 4, 0,
  /* 11376 */ 16, 4, 16, 4, 16, 4, 16, 4, 4, 0,
  /* 11386 */ 15, 0, 4, 15, 9, 15, 16, 4, 4, 0,
  /* 11396 */ 0, 15, 2, 4, 15, 9, 15, 16, 4, 4, 0,
  /* 11407 */ 15, 0, 15, 7, 15, 9, 15, 16, 4, 4, 0,
  /* 11418 */ 15, 1, 15, 7, 15, 9, 15, 16, 4, 4, 0,
  /* 11429 */ 15, 2, 15, 7, 15, 9, 15, 16, 4, 4, 0,
  /* 11440 */ 15, 1, 15, 7, 15, 7, 15, 9, 15, 16, 4, 4, 0,
  /* 11453 */ 15, 0, 4, 15, 9, 15, 15, 15, 16, 4, 4, 0,
  /* 11465 */ 0, 15, 2, 4, 15, 9, 15, 15, 15, 16, 4, 4, 0,
  /* 11478 */ 15, 0, 15, 7, 15, 9, 15, 15, 15, 16, 4, 4, 0,
  /* 11491 */ 15, 1, 15, 7, 15, 9, 15, 15, 15, 16, 4, 4, 0,
  /* 11504 */ 15, 2, 15, 7, 15, 9, 15, 15, 15, 16, 4, 4, 0,
  /* 11517 */ 15, 1, 15, 7, 15, 7, 15, 9, 15, 15, 15, 16, 4, 4, 0,
  /* 11532 */ 15, 0, 4, 15, 9, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11546 */ 0, 15, 2, 4, 15, 9, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11561 */ 15, 0, 15, 7, 15, 9, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11576 */ 15, 1, 15, 7, 15, 9, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11591 */ 15, 2, 15, 7, 15, 9, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11606 */ 15, 1, 15, 7, 15, 7, 15, 9, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11623 */ 15, 0, 4, 15, 9, 15, 15, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11639 */ 0, 15, 2, 4, 15, 9, 15, 15, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11656 */ 15, 0, 15, 7, 15, 9, 15, 15, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11673 */ 15, 1, 15, 7, 15, 9, 15, 15, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11690 */ 15, 2, 15, 7, 15, 9, 15, 15, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11707 */ 15, 1, 15, 7, 15, 7, 15, 9, 15, 15, 15, 15, 15, 15, 15, 16, 4, 4, 0,
  /* 11726 */ 17, 17, 4, 4, 0,
  /* 11731 */ 15, 0, 18, 4, 4, 0,
  /* 11737 */ 0, 15, 4, 15, 11, 15, 19, 4, 4, 0,
  /* 11747 */ 21, 0, 5, 4, 0,
  /* 11752 */ 21, 0, 3, 3, 5, 4, 0,
  /* 11759 */ 21, 2, 3, 3, 3, 3, 5, 4, 0,
  /* 11768 */ 15, 4, 15, 12, 3, 5, 4, 0,
  /* 11776 */ 0, 28, 3, 1, 15, 3, 5, 4, 0,
  /* 11785 */ 1, 24, 3, 5, 4, 0,
  /* 11791 */ 21, 0, 4, 4, 5, 4, 0,
  /* 11798 */ 21, 2, 4, 4, 4, 4, 5, 4, 0,
  /* 11807 */ 21, 2, 4, 4, 4, 4, 5, 5, 4, 0,
  /* 11817 */ 21, 0, 5, 5, 5, 4, 0,
  /* 11824 */ 21, 2, 7, 7, 7, 7, 5, 5, 4, 0,
  /* 11834 */ 0, 41, 8, 41, 8, 5, 5, 4, 0,
  /* 11843 */ 9, 5, 9, 5, 5, 4, 0,
  /* 11850 */ 10, 5, 10, 5, 5, 4, 0,
  /* 11857 */ 5, 14, 5, 5, 4, 0,
  /* 11863 */ 49, 5, 5, 4, 0,
  /* 11868 */ 11, 6, 11, 6, 5, 4, 0,
  /* 11875 */ 21, 2, 7, 7, 7, 7, 5, 4, 0,
  /* 11884 */ 10, 7, 10, 7, 5, 4, 0,
  /* 11891 */ 9, 8, 9, 8, 5, 4, 0,
  /* 11898 */ 41, 8, 41, 8, 41, 8, 5, 4, 0,
  /* 11907 */ 0, 14, 9, 1, 10, 4, 9, 5, 4, 0,
  /* 11917 */ 0, 14, 2, 10, 4, 9, 5, 4, 0,
  /* 11926 */ 10, 4, 10, 4, 9, 5, 4, 0,
  /* 11934 */ 0, 14, 9, 1, 9, 5, 9, 5, 4, 0,
  /* 11944 */ 0, 14, 2, 9, 5, 9, 5, 4, 0,
  /* 11953 */ 0, 9, 5, 9, 5, 9, 5, 4, 0,
  /* 11962 */ 9, 5, 9, 5, 9, 5, 9, 5, 4, 0,
  /* 11972 */ 0, 14, 10, 1, 10, 4, 10, 5, 4, 0,
  /* 11982 */ 0, 14, 2, 10, 4, 10, 5, 4, 0,
  /* 11991 */ 0, 14, 10, 1, 10, 5, 10, 5, 4, 0,
  /* 12001 */ 0, 14, 2, 10, 5, 10, 5, 4, 0,
  /* 12010 */ 10, 5, 10, 5, 10, 5, 10, 5, 4, 0,
  /* 12020 */ 0, 14, 11, 1, 11, 4, 11, 5, 4, 0,
  /* 12030 */ 0, 14, 2, 11, 4, 11, 5, 4, 0,
  /* 12039 */ 0, 14, 11, 1, 11, 5, 11, 5, 4, 0,
  /* 12049 */ 0, 14, 2, 11, 5, 11, 5, 4, 0,
  /* 12058 */ 11, 5, 11, 5, 11, 5, 11, 5, 4, 0,
  /* 12068 */ 6, 6, 6, 6, 4, 0,
  /* 12074 */ 0, 15, 4, 9, 6, 9, 6, 9, 6, 9, 6, 4, 0,
  /* 12087 */ 11, 1, 11, 6, 4, 0,
  /* 12093 */ 11, 6, 11, 6, 11, 6, 4, 0,
  /* 12101 */ 10, 7, 11, 6, 4, 0,
  /* 12107 */ 12, 1, 12, 6, 4, 0,
  /* 12113 */ 12, 6, 12, 6, 12, 6, 4, 0,
  /* 12121 */ 13, 1, 13, 6, 4, 0,
  /* 12127 */ 13, 6, 13, 2, 13, 6, 4, 0,
  /* 12135 */ 13, 6, 13, 6, 13, 6, 13, 6, 4, 0,
  /* 12145 */ 0, 24, 6, 4, 0,
  /* 12150 */ 4, 4, 6, 4, 7, 4, 0,
  /* 12157 */ 4, 4, 9, 6, 4, 7, 4, 0,
  /* 12165 */ 4, 4, 7, 4, 7, 4, 0,
  /* 12172 */ 4, 4, 9, 7, 4, 7, 4, 0,
  /* 12180 */ 4, 4, 40, 4, 7, 4, 0,
  /* 12187 */ 4, 4, 9, 40, 4, 7, 4, 0,
  /* 12195 */ 4, 4, 9, 6, 7, 4, 0,
  /* 12202 */ 9, 6, 7, 7, 4, 0,
  /* 12208 */ 4, 4, 7, 7, 7, 4, 0,
  /* 12215 */ 0, 15, 4, 7, 7, 7, 7, 7, 7, 7, 7, 4, 0,
  /* 12228 */ 9, 40, 7, 7, 4, 0,
  /* 12234 */ 41, 8, 7, 4, 0,
  /* 12239 */ 10, 1, 10, 7, 4, 0,
  /* 12245 */ 10, 2, 10, 7, 4, 0,
  /* 12251 */ 11, 3, 10, 7, 4, 0,
  /* 12257 */ 21, 0, 10, 4, 4, 10, 7, 4, 0,
  /* 12266 */ 0, 14, 10, 1, 10, 4, 10, 7, 4, 0,
  /* 12276 */ 0, 14, 2, 10, 4, 10, 7, 4, 0,
  /* 12285 */ 0, 14, 9, 1, 9, 5, 10, 7, 4, 0,
  /* 12295 */ 0, 14, 2, 9, 5, 10, 7, 4, 0,
  /* 12304 */ 0, 14, 10, 1, 10, 5, 10, 7, 4, 0,
  /* 12314 */ 0, 14, 2, 10, 5, 10, 7, 4, 0,
  /* 12323 */ 11, 6, 11, 6, 10, 7, 4, 0,
  /* 12331 */ 11, 3, 10, 7, 10, 7, 4, 0,
  /* 12339 */ 10, 7, 10, 7, 10, 7, 4, 0,
  /* 12347 */ 11, 1, 11, 7, 4, 0,
  /* 12353 */ 11, 3, 11, 7, 4, 0,
  /* 12359 */ 0, 14, 11, 1, 11, 4, 11, 7, 4, 0,
  /* 12369 */ 0, 14, 2, 11, 4, 11, 7, 4, 0,
  /* 12378 */ 0, 14, 11, 1, 11, 5, 11, 7, 4, 0,
  /* 12388 */ 0, 14, 2, 11, 5, 11, 7, 4, 0,
  /* 12397 */ 11, 7, 11, 7, 11, 7, 4, 0,
  /* 12405 */ 12, 1, 12, 7, 4, 0,
  /* 12411 */ 0, 14, 12, 1, 12, 4, 12, 7, 4, 0,
  /* 12421 */ 0, 14, 3, 12, 4, 12, 7, 4, 0,
  /* 12430 */ 12, 7, 12, 7, 12, 7, 12, 7, 4, 0,
  /* 12440 */ 15, 0, 15, 7, 4, 0,
  /* 12446 */ 15, 3, 28, 3, 1, 15, 7, 4, 0,
  /* 12455 */ 15, 2, 15, 7, 4, 0,
  /* 12461 */ 21, 0, 15, 3, 15, 7, 4, 0,
  /* 12469 */ 0, 4, 15, 3, 15, 7, 4, 0,
  /* 12477 */ 36, 7, 15, 3, 15, 7, 4, 0,
  /* 12485 */ 15, 2, 4, 15, 7, 4, 0,
  /* 12492 */ 15, 3, 28, 3, 1, 15, 7, 15, 7, 4, 0,
  /* 12503 */ 15, 1, 15, 7, 15, 7, 4, 0,
  /* 12511 */ 0, 4, 15, 3, 15, 7, 15, 7, 4, 0,
  /* 12521 */ 15, 2, 4, 15, 7, 15, 7, 4, 0,
  /* 12530 */ 21, 0, 15, 3, 4, 15, 7, 15, 7, 4, 0,
  /* 12541 */ 15, 3, 28, 3, 1, 15, 7, 15, 7, 15, 7, 4, 0,
  /* 12554 */ 15, 1, 15, 7, 15, 7, 15, 7, 4, 0,
  /* 12564 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 4, 0,
  /* 12576 */ 37, 7, 15, 3, 15, 7, 15, 7, 15, 7, 4, 0,
  /* 12588 */ 15, 2, 4, 15, 7, 15, 7, 15, 7, 4, 0,
  /* 12599 */ 0, 4, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 4, 0,
  /* 12613 */ 15, 1, 22, 7, 4, 0,
  /* 12619 */ 15, 3, 23, 7, 4, 0,
  /* 12625 */ 0, 24, 7, 4, 0,
  /* 12630 */ 15, 3, 36, 7, 4, 0,
  /* 12636 */ 15, 3, 36, 7, 36, 7, 4, 0,
  /* 12644 */ 15, 3, 15, 7, 36, 7, 36, 7, 4, 0,
  /* 12654 */ 15, 3, 15, 7, 37, 7, 37, 7, 4, 0,
  /* 12664 */ 4, 4, 9, 40, 7, 4, 0,
  /* 12671 */ 0, 15, 4, 8, 8, 4, 0,
  /* 12678 */ 8, 8, 8, 8, 4, 0,
  /* 12684 */ 41, 8, 8, 4, 0,
  /* 12689 */ 9, 1, 9, 8, 4, 0,
  /* 12695 */ 21, 0, 9, 5, 4, 9, 8, 4, 0,
  /* 12704 */ 0, 14, 9, 1, 10, 4, 9, 8, 4, 0,
  /* 12714 */ 0, 14, 2, 10, 4, 9, 8, 4, 0,
  /* 12723 */ 0, 14, 9, 1, 9, 5, 9, 8, 4, 0,
  /* 12733 */ 0, 14, 2, 9, 5, 9, 8, 4, 0,
  /* 12742 */ 9, 8, 9, 8, 9, 8, 4, 0,
  /* 12750 */ 10, 1, 10, 8, 4, 0,
  /* 12756 */ 0, 14, 10, 1, 10, 4, 10, 8, 4, 0,
  /* 12766 */ 0, 14, 2, 10, 4, 10, 8, 4, 0,
  /* 12775 */ 0, 14, 10, 1, 10, 5, 10, 8, 4, 0,
  /* 12785 */ 0, 14, 2, 10, 5, 10, 8, 4, 0,
  /* 12794 */ 10, 8, 10, 8, 10, 8, 4, 0,
  /* 12802 */ 11, 1, 11, 8, 4, 0,
  /* 12808 */ 0, 14, 11, 1, 11, 4, 11, 8, 4, 0,
  /* 12818 */ 0, 14, 2, 11, 4, 11, 8, 4, 0,
  /* 12827 */ 0, 14, 11, 1, 11, 5, 11, 8, 4, 0,
  /* 12837 */ 0, 14, 2, 11, 5, 11, 8, 4, 0,
  /* 12846 */ 11, 8, 11, 8, 11, 8, 11, 8, 4, 0,
  /* 12856 */ 15, 0, 15, 8, 4, 0,
  /* 12862 */ 21, 0, 15, 0, 1, 15, 8, 4, 0,
  /* 12871 */ 15, 3, 15, 8, 4, 0,
  /* 12877 */ 15, 4, 15, 8, 4, 0,
  /* 12883 */ 41, 8, 41, 8, 5, 31, 1, 41, 8, 4, 0,
  /* 12894 */ 41, 8, 4, 41, 8, 31, 1, 41, 8, 4, 0,
  /* 12905 */ 41, 8, 41, 8, 5, 41, 8, 31, 1, 41, 8, 4, 0,
  /* 12918 */ 41, 8, 5, 41, 8, 41, 8, 31, 1, 41, 8, 4, 0,
  /* 12931 */ 41, 8, 41, 8, 41, 8, 41, 8, 31, 1, 41, 8, 4, 0,
  /* 12945 */ 41, 8, 41, 8, 4, 41, 1, 41, 8, 4, 0,
  /* 12956 */ 41, 8, 41, 8, 5, 5, 41, 1, 41, 8, 4, 0,
  /* 12968 */ 41, 8, 41, 8, 5, 41, 1, 41, 8, 4, 0,
  /* 12979 */ 41, 8, 7, 41, 1, 41, 8, 4, 0,
  /* 12988 */ 41, 8, 8, 41, 1, 41, 8, 4, 0,
  /* 12997 */ 41, 8, 4, 41, 8, 41, 1, 41, 8, 4, 0,
  /* 13008 */ 41, 8, 5, 41, 8, 41, 1, 41, 8, 4, 0,
  /* 13019 */ 41, 8, 41, 8, 7, 41, 8, 41, 1, 41, 8, 4, 0,
  /* 13032 */ 41, 8, 41, 8, 8, 41, 8, 41, 1, 41, 8, 4, 0,
  /* 13045 */ 41, 8, 7, 41, 8, 41, 8, 41, 1, 41, 8, 4, 0,
  /* 13058 */ 41, 8, 8, 41, 8, 41, 8, 41, 1, 41, 8, 4, 0,
  /* 13071 */ 41, 8, 41, 8, 41, 8, 41, 8, 41, 1, 41, 8, 4, 0,
  /* 13085 */ 41, 8, 41, 8, 4, 41, 8, 4, 0,
  /* 13094 */ 41, 8, 41, 8, 5, 5, 41, 8, 4, 0,
  /* 13104 */ 41, 8, 41, 8, 41, 8, 5, 41, 8, 4, 0,
  /* 13115 */ 41, 8, 41, 8, 7, 41, 8, 4, 0,
  /* 13124 */ 41, 8, 41, 8, 8, 41, 8, 4, 0,
  /* 13133 */ 41, 8, 4, 41, 8, 41, 8, 4, 0,
  /* 13142 */ 41, 8, 41, 8, 5, 41, 8, 41, 8, 4, 0,
  /* 13153 */ 41, 8, 41, 8, 7, 41, 8, 41, 8, 4, 0,
  /* 13164 */ 41, 8, 41, 8, 8, 41, 8, 41, 8, 4, 0,
  /* 13175 */ 41, 8, 5, 41, 8, 41, 8, 41, 8, 4, 0,
  /* 13186 */ 41, 8, 7, 41, 8, 41, 8, 41, 8, 4, 0,
  /* 13197 */ 41, 8, 8, 41, 8, 41, 8, 41, 8, 4, 0,
  /* 13208 */ 41, 8, 41, 8, 41, 8, 41, 8, 41, 8, 4, 0,
  /* 13220 */ 41, 8, 5, 14, 41, 8, 4, 0,
  /* 13228 */ 9, 4, 4, 4, 9, 4, 0,
  /* 13235 */ 44, 4, 4, 5, 9, 4, 0,
  /* 13242 */ 10, 4, 5, 5, 9, 4, 0,
  /* 13249 */ 0, 15, 4, 2, 15, 9, 4, 0,
  /* 13257 */ 15, 2, 15, 9, 4, 0,
  /* 13263 */ 15, 1, 15, 7, 15, 9, 4, 0,
  /* 13271 */ 10, 4, 12, 2, 12, 2, 10, 4, 0,
  /* 13280 */ 10, 4, 12, 2, 10, 4, 0,
  /* 13287 */ 10, 4, 11, 3, 11, 3, 10, 4, 0,
  /* 13296 */ 10, 4, 11, 3, 10, 4, 0,
  /* 13303 */ 21, 0, 10, 4, 4, 10, 4, 0,
  /* 13311 */ 10, 4, 10, 4, 4, 10, 4, 0,
  /* 13319 */ 10, 4, 5, 4, 10, 4, 0,
  /* 13326 */ 21, 1, 51, 4, 44, 7, 44, 7, 5, 7, 2, 44, 7, 44, 7, 4, 10, 4, 0,
  /* 13345 */ 21, 1, 51, 4, 44, 7, 44, 7, 5, 7, 2, 44, 7, 44, 7, 9, 4, 10, 4, 0,
  /* 13365 */ 21, 0, 10, 1, 10, 1, 10, 4, 10, 4, 0,
  /* 13376 */ 11, 3, 10, 4, 10, 4, 0,
  /* 13383 */ 21, 0, 11, 3, 4, 10, 4, 10, 4, 0,
  /* 13393 */ 21, 0, 10, 4, 4, 10, 4, 10, 4, 0,
  /* 13403 */ 10, 4, 10, 4, 10, 4, 10, 4, 0,
  /* 13412 */ 9, 5, 9, 5, 10, 4, 10, 4, 0,
  /* 13421 */ 10, 7, 10, 4, 10, 4, 0,
  /* 13428 */ 0, 14, 10, 4, 10, 4, 0,
  /* 13435 */ 30, 10, 4, 10, 4, 0,
  /* 13441 */ 11, 4, 11, 4, 10, 4, 0,
  /* 13448 */ 12, 4, 12, 4, 10, 4, 0,
  /* 13455 */ 10, 7, 10, 7, 10, 4, 0,
  /* 13462 */ 10, 4, 14, 10, 4, 0,
  /* 13468 */ 10, 7, 14, 10, 4, 0,
  /* 13474 */ 15, 1, 15, 10, 4, 0,
  /* 13480 */ 10, 4, 15, 1, 7, 44, 7, 15, 11, 15, 15, 10, 4, 0,
  /* 13494 */ 35, 9, 8, 35, 9, 8, 35, 9, 1, 35, 10, 4, 0,
  /* 13507 */ 35, 11, 6, 35, 11, 6, 35, 10, 1, 35, 10, 4, 0,
  /* 13520 */ 35, 10, 4, 35, 10, 4, 35, 10, 4, 0,
  /* 13530 */ 21, 0, 11, 1, 11, 1, 11, 4, 11, 4, 0,
  /* 13541 */ 12, 3, 11, 4, 11, 4, 0,
  /* 13548 */ 11, 4, 11, 4, 11, 4, 11, 4, 0,
  /* 13557 */ 10, 5, 10, 5, 11, 4, 11, 4, 0,
  /* 13566 */ 0, 14, 11, 4, 11, 4, 0,
  /* 13573 */ 11, 7, 11, 7, 11, 4, 0,
  /* 13580 */ 11, 4, 14, 11, 4, 0,
  /* 13586 */ 11, 7, 14, 11, 4, 0,
  /* 13592 */ 15, 3, 15, 11, 4, 0,
  /* 13598 */ 15, 3, 4, 15, 11, 4, 0,
  /* 13605 */ 0, 15, 4, 15, 11, 4, 0,
  /* 13612 */ 15, 0, 15, 7, 15, 11, 4, 0,
  /* 13620 */ 12, 4, 16, 1, 12, 4, 0,
  /* 13627 */ 0, 14, 16, 1, 4, 4, 12, 4, 0,
  /* 13636 */ 0, 14, 4, 4, 12, 4, 0,
  /* 13643 */ 21, 0, 12, 1, 12, 1, 12, 4, 12, 4, 0,
  /* 13654 */ 16, 1, 16, 1, 12, 4, 12, 4, 0,
  /* 13663 */ 21, 0, 12, 4, 16, 1, 12, 4, 12, 4, 0,
  /* 13674 */ 13, 4, 16, 1, 12, 4, 12, 4, 0,
  /* 13683 */ 13, 3, 12, 4, 12, 4, 0,
  /* 13690 */ 0, 4, 4, 12, 4, 12, 4, 0,
  /* 13698 */ 0, 16, 1, 4, 4, 12, 4, 12, 4, 0,
  /* 13708 */ 12, 4, 12, 4, 12, 4, 12, 4, 0,
  /* 13717 */ 13, 4, 13, 4, 12, 4, 12, 4, 0,
  /* 13726 */ 0, 4, 4, 13, 4, 12, 4, 0,
  /* 13734 */ 0, 16, 1, 4, 4, 13, 4, 12, 4, 0,
  /* 13744 */ 12, 4, 13, 4, 12, 4, 0,
  /* 13751 */ 13, 4, 13, 4, 12, 4, 0,
  /* 13758 */ 0, 1, 14, 4, 12, 4, 0,
  /* 13765 */ 14, 1, 14, 4, 12, 4, 0,
  /* 13772 */ 12, 7, 12, 7, 12, 4, 0,
  /* 13779 */ 0, 16, 1, 14, 12, 4, 0,
  /* 13786 */ 15, 1, 15, 12, 4, 0,
  /* 13792 */ 15, 2, 15, 12, 4, 0,
  /* 13798 */ 15, 3, 15, 12, 4, 0,
  /* 13804 */ 15, 4, 15, 12, 4, 0,
  /* 13810 */ 21, 0, 15, 3, 15, 7, 15, 12, 4, 0,
  /* 13820 */ 21, 1, 15, 3, 15, 7, 15, 7, 15, 12, 4, 0,
  /* 13832 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 12, 4, 0,
  /* 13846 */ 13, 4, 39, 1, 13, 4, 0,
  /* 13853 */ 0, 14, 16, 1, 4, 4, 13, 4, 0,
  /* 13862 */ 0, 14, 39, 1, 4, 4, 13, 4, 0,
  /* 13871 */ 0, 14, 4, 4, 13, 4, 0,
  /* 13878 */ 39, 1, 39, 1, 13, 4, 13, 4, 0,
  /* 13887 */ 21, 0, 13, 4, 39, 1, 13, 4, 13, 4, 0,
  /* 13898 */ 16, 4, 39, 1, 13, 4, 13, 4, 0,
  /* 13907 */ 0, 4, 4, 13, 4, 13, 4, 0,
  /* 13915 */ 0, 39, 1, 4, 4, 13, 4, 13, 4, 0,
  /* 13925 */ 13, 4, 13, 4, 13, 4, 13, 4, 0,
  /* 13934 */ 16, 4, 16, 4, 13, 4, 13, 4, 0,
  /* 13943 */ 0, 1, 14, 4, 13, 4, 0,
  /* 13950 */ 14, 1, 14, 4, 13, 4, 0,
  /* 13957 */ 0, 4, 4, 16, 4, 13, 4, 0,
  /* 13965 */ 0, 39, 1, 4, 4, 16, 4, 13, 4, 0,
  /* 13975 */ 13, 4, 16, 4, 13, 4, 0,
  /* 13982 */ 16, 4, 16, 4, 13, 4, 0,
  /* 13989 */ 0, 39, 1, 14, 13, 4, 0,
  /* 13996 */ 12, 4, 1, 14, 4, 0,
  /* 14002 */ 13, 4, 1, 14, 4, 0,
  /* 14008 */ 21, 0, 12, 4, 14, 1, 14, 4, 0,
  /* 14017 */ 21, 0, 13, 4, 14, 1, 14, 4, 0,
  /* 14026 */ 0, 12, 2, 14, 4, 0,
  /* 14032 */ 0, 13, 2, 14, 4, 0,
  /* 14038 */ 0, 11, 3, 14, 4, 0,
  /* 14044 */ 0, 15, 3, 14, 4, 0,
  /* 14050 */ 0, 10, 4, 14, 4, 0,
  /* 14056 */ 0, 41, 8, 5, 14, 4, 0,
  /* 14063 */ 0, 9, 5, 14, 4, 0,
  /* 14069 */ 21, 0, 4, 14, 14, 4, 0,
  /* 14076 */ 21, 0, 5, 14, 14, 4, 0,
  /* 14083 */ 14, 18, 14, 4, 0,
  /* 14088 */ 21, 0, 4, 4, 15, 4, 0,
  /* 14095 */ 21, 2, 4, 4, 4, 4, 15, 4, 0,
  /* 14104 */ 21, 6, 4, 4, 4, 4, 4, 4, 4, 4, 15, 4, 0,
  /* 14117 */ 0, 14, 5, 15, 4, 0,
  /* 14123 */ 21, 2, 9, 6, 9, 6, 9, 6, 9, 6, 15, 4, 0,
  /* 14136 */ 21, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 15, 4, 0,
  /* 14157 */ 21, 6, 7, 7, 7, 7, 7, 7, 7, 7, 15, 4, 0,
  /* 14170 */ 21, 0, 8, 8, 15, 4, 0,
  /* 14177 */ 15, 1, 15, 9, 15, 15, 4, 0,
  /* 14185 */ 15, 1, 15, 10, 15, 15, 4, 0,
  /* 14193 */ 21, 0, 15, 3, 15, 11, 15, 15, 4, 0,
  /* 14203 */ 0, 15, 4, 15, 11, 15, 15, 4, 0,
  /* 14212 */ 15, 1, 15, 7, 15, 12, 15, 15, 4, 0,
  /* 14222 */ 0, 15, 4, 15, 11, 15, 15, 15, 15, 4, 0,
  /* 14233 */ 0, 15, 4, 15, 11, 15, 15, 15, 15, 15, 15, 4, 0,
  /* 14246 */ 5, 19, 15, 4, 0,
  /* 14251 */ 0, 14, 39, 1, 4, 4, 16, 4, 0,
  /* 14260 */ 0, 14, 4, 4, 16, 4, 0,
  /* 14267 */ 13, 4, 16, 4, 0,
  /* 14272 */ 16, 4, 16, 4, 16, 4, 0,
  /* 14279 */ 21, 0, 15, 0, 15, 8, 15, 16, 4, 0,
  /* 14289 */ 4, 17, 4, 0,
  /* 14293 */ 0, 15, 4, 15, 12, 15, 17, 4, 0,
  /* 14302 */ 17, 17, 4, 0,
  /* 14306 */ 0, 18, 4, 0,
  /* 14310 */ 14, 18, 4, 0,
  /* 14314 */ 0, 15, 0, 19, 4, 0,
  /* 14320 */ 15, 0, 15, 8, 19, 4, 0,
  /* 14327 */ 15, 4, 24, 4, 0,
  /* 14332 */ 30, 9, 5, 9, 5, 30, 4, 0,
  /* 14340 */ 5, 25, 30, 4, 0,
  /* 14345 */ 21, 2, 15, 0, 15, 8, 15, 16, 15, 24, 15, 32, 4, 0,
  /* 14359 */ 11, 1, 11, 40, 4, 0,
  /* 14365 */ 35, 10, 7, 35, 10, 7, 35, 11, 40, 35, 11, 40, 4, 0,
  /* 14379 */ 11, 40, 11, 40, 11, 40, 4, 0,
  /* 14387 */ 12, 1, 12, 40, 4, 0,
  /* 14393 */ 12, 40, 12, 40, 12, 40, 4, 0,
  /* 14401 */ 13, 1, 13, 40, 4, 0,
  /* 14407 */ 13, 40, 13, 40, 4, 13, 40, 4, 0,
  /* 14416 */ 13, 40, 13, 40, 13, 40, 13, 40, 4, 0,
  /* 14426 */ 12, 4, 3, 3, 42, 4, 0,
  /* 14433 */ 13, 6, 3, 3, 42, 4, 0,
  /* 14440 */ 12, 7, 3, 3, 42, 4, 0,
  /* 14447 */ 13, 40, 3, 3, 42, 4, 0,
  /* 14454 */ 8, 43, 4, 0,
  /* 14458 */ 4, 24, 1, 45, 4, 0,
  /* 14464 */ 0, 24, 1, 4, 45, 4, 0,
  /* 14471 */ 4, 24, 1, 46, 4, 0,
  /* 14477 */ 0, 24, 1, 4, 46, 4, 0,
  /* 14484 */ 15, 3, 49, 4, 0,
  /* 14489 */ 5, 49, 4, 0,
  /* 14493 */ 21, 0, 15, 3, 15, 7, 49, 4, 0,
  /* 14502 */ 21, 6, 15, 0, 15, 8, 15, 16, 15, 24, 15, 32, 15, 40, 15, 48, 15, 56, 15, 64, 4, 0,
  /* 14524 */ 15, 3, 27, 0, 5, 27, 0, 5, 27, 0, 5, 27, 0, 5, 27, 0, 5, 0,
  /* 14542 */ 0, 24, 1, 5, 0,
  /* 14547 */ 5, 24, 1, 5, 0,
  /* 14552 */ 5, 31, 1, 5, 0,
  /* 14557 */ 5, 41, 1, 5, 0,
  /* 14562 */ 1, 24, 3, 5, 0,
  /* 14567 */ 5, 24, 3, 5, 0,
  /* 14572 */ 0, 15, 3, 28, 3, 1, 14, 28, 3, 5, 0,
  /* 14583 */ 21, 0, 4, 1, 4, 5, 0,
  /* 14590 */ 0, 5, 4, 4, 4, 5, 0,
  /* 14597 */ 0, 5, 4, 4, 5, 0,
  /* 14603 */ 0, 14, 5, 4, 4, 5, 0,
  /* 14610 */ 0, 14, 4, 4, 5, 0,
  /* 14616 */ 0, 15, 4, 4, 5, 0,
  /* 14622 */ 5, 5, 4, 5, 4, 5, 0,
  /* 14629 */ 0, 5, 5, 4, 5, 0,
  /* 14635 */ 5, 5, 5, 4, 5, 0,
  /* 14641 */ 9, 5, 9, 5, 4, 5, 0,
  /* 14648 */ 41, 8, 41, 8, 4, 5, 0,
  /* 14655 */ 10, 4, 10, 4, 10, 4, 10, 4, 5, 0,
  /* 14665 */ 12, 4, 12, 4, 12, 4, 5, 0,
  /* 14673 */ 13, 4, 13, 4, 12, 4, 5, 0,
  /* 14681 */ 13, 4, 13, 4, 13, 4, 5, 0,
  /* 14689 */ 16, 4, 16, 4, 13, 4, 5, 0,
  /* 14697 */ 31, 1, 31, 1, 5, 5, 0,
  /* 14704 */ 41, 1, 41, 1, 5, 5, 0,
  /* 14711 */ 21, 0, 2, 5, 2, 5, 5, 0,
  /* 14719 */ 0, 24, 3, 5, 5, 0,
  /* 14725 */ 0, 5, 4, 4, 4, 5, 5, 0,
  /* 14733 */ 0, 5, 4, 4, 5, 5, 0,
  /* 14740 */ 0, 5, 4, 5, 5, 0,
  /* 14746 */ 5, 15, 4, 5, 5, 5, 0,
  /* 14753 */ 5, 15, 4, 5, 5, 5, 5, 0,
  /* 14761 */ 0, 14, 5, 5, 5, 5, 5, 5, 5, 5, 0,
  /* 14772 */ 5, 14, 5, 5, 5, 5, 5, 5, 5, 5, 0,
  /* 14783 */ 21, 0, 5, 5, 14, 5, 5, 5, 5, 0,
  /* 14793 */ 9, 5, 9, 5, 5, 5, 0,
  /* 14800 */ 21, 0, 15, 3, 15, 7, 5, 5, 0,
  /* 14809 */ 9, 5, 9, 5, 9, 5, 5, 0,
  /* 14817 */ 21, 0, 5, 5, 14, 5, 5, 0,
  /* 14825 */ 0, 24, 6, 5, 0,
  /* 14830 */ 15, 3, 15, 7, 5, 0,
  /* 14836 */ 15, 3, 15, 11, 5, 15, 7, 5, 0,
  /* 14845 */ 21, 5, 4, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 4, 9, 5, 0,
  /* 14864 */ 9, 5, 10, 4, 10, 4, 9, 5, 0,
  /* 14873 */ 21, 0, 9, 1, 9, 1, 9, 5, 9, 5, 0,
  /* 14884 */ 21, 0, 10, 4, 4, 9, 5, 9, 5, 0,
  /* 14894 */ 21, 0, 9, 5, 4, 9, 5, 9, 5, 0,
  /* 14904 */ 21, 6, 4, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 4, 9, 5, 9, 5, 0,
  /* 14927 */ 10, 4, 9, 5, 9, 5, 0,
  /* 14934 */ 21, 7, 2, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 14, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 9, 5, 0,
  /* 14971 */ 10, 5, 9, 5, 9, 5, 0,
  /* 14978 */ 10, 7, 9, 5, 9, 5, 0,
  /* 14985 */ 9, 8, 9, 5, 9, 5, 0,
  /* 14992 */ 0, 14, 9, 5, 9, 5, 0,
  /* 14999 */ 25, 30, 9, 5, 9, 5, 0,
  /* 15006 */ 10, 5, 10, 5, 10, 5, 9, 5, 0,
  /* 15015 */ 11, 5, 11, 5, 9, 5, 0,
  /* 15022 */ 9, 8, 9, 8, 9, 5, 0,
  /* 15029 */ 9, 5, 14, 9, 5, 0,
  /* 15035 */ 9, 8, 14, 9, 5, 0,
  /* 15041 */ 25, 30, 9, 5, 0,
  /* 15046 */ 35, 11, 6, 35, 11, 6, 35, 9, 1, 35, 9, 5, 0,
  /* 15059 */ 35, 10, 7, 35, 10, 7, 35, 9, 1, 35, 9, 5, 0,
  /* 15072 */ 28, 3, 1, 28, 3, 1, 15, 3, 35, 9, 5, 0,
  /* 15084 */ 21, 0, 35, 9, 5, 35, 9, 5, 35, 9, 5, 35, 9, 5, 0,
  /* 15099 */ 21, 0, 35, 9, 5, 35, 9, 5, 35, 9, 5, 35, 9, 5, 35, 9, 5, 35, 9, 5, 0,
  /* 15120 */ 15, 3, 28, 3, 1, 15, 7, 35, 9, 5, 0,
  /* 15131 */ 0, 15, 3, 35, 25, 1, 14, 35, 9, 5, 0,
  /* 15142 */ 21, 0, 10, 1, 10, 1, 10, 5, 10, 5, 0,
  /* 15153 */ 11, 4, 10, 5, 10, 5, 0,
  /* 15160 */ 10, 5, 10, 5, 10, 5, 10, 5, 0,
  /* 15169 */ 11, 7, 10, 5, 10, 5, 0,
  /* 15176 */ 0, 14, 10, 5, 10, 5, 0,
  /* 15183 */ 10, 8, 10, 8, 10, 5, 0,
  /* 15190 */ 10, 5, 14, 10, 5, 0,
  /* 15196 */ 10, 8, 14, 10, 5, 0,
  /* 15202 */ 21, 0, 11, 1, 11, 1, 11, 5, 11, 5, 0,
  /* 15213 */ 11, 5, 11, 5, 11, 5, 11, 5, 0,
  /* 15222 */ 11, 8, 11, 8, 11, 5, 0,
  /* 15229 */ 0, 15, 3, 28, 3, 1, 15, 11, 5, 0,
  /* 15239 */ 0, 15, 3, 35, 25, 1, 15, 11, 5, 0,
  /* 15249 */ 15, 3, 15, 11, 5, 0,
  /* 15255 */ 15, 3, 15, 7, 15, 11, 5, 0,
  /* 15263 */ 0, 41, 1, 14, 5, 0,
  /* 15269 */ 0, 12, 2, 14, 5, 0,
  /* 15275 */ 0, 13, 2, 14, 5, 0,
  /* 15281 */ 42, 3, 3, 14, 5, 0,
  /* 15287 */ 0, 10, 4, 14, 5, 0,
  /* 15293 */ 15, 4, 15, 7, 14, 5, 0,
  /* 15300 */ 15, 3, 14, 14, 5, 0,
  /* 15306 */ 0, 24, 5, 0,
  /* 15310 */ 25, 30, 5, 0,
  /* 15314 */ 33, 33, 5, 0,
  /* 15318 */ 15, 3, 27, 0, 6, 27, 0, 6, 27, 0, 6, 27, 0, 6, 27, 0, 6, 27, 0, 6, 0,
  /* 15339 */ 7, 7, 6, 6, 6, 6, 0,
  /* 15346 */ 6, 9, 6, 9, 6, 6, 0,
  /* 15353 */ 21, 0, 9, 6, 9, 6, 4, 4, 4, 4, 4, 4, 9, 6, 9, 6, 0,
  /* 15370 */ 21, 0, 9, 6, 9, 6, 4, 4, 4, 9, 6, 9, 6, 0,
  /* 15384 */ 0, 15, 4, 9, 6, 9, 6, 9, 6, 9, 6, 0,
  /* 15396 */ 21, 0, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 0,
  /* 15413 */ 21, 0, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 0,
  /* 15436 */ 21, 2, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 0,
  /* 15463 */ 21, 2, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 0,
  /* 15514 */ 21, 6, 7, 7, 7, 7, 7, 7, 7, 7, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 0,
  /* 15565 */ 21, 6, 7, 7, 7, 7, 7, 7, 7, 7, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 0,
  /* 15592 */ 12, 2, 11, 6, 11, 6, 0,
  /* 15599 */ 11, 6, 11, 6, 11, 6, 11, 6, 0,
  /* 15608 */ 10, 7, 10, 7, 11, 6, 11, 6, 0,
  /* 15617 */ 35, 9, 5, 35, 9, 5, 35, 9, 1, 35, 11, 6, 0,
  /* 15630 */ 35, 9, 8, 35, 9, 8, 35, 9, 1, 35, 11, 6, 0,
  /* 15643 */ 35, 10, 4, 35, 10, 4, 35, 10, 1, 35, 11, 6, 0,
  /* 15656 */ 35, 10, 7, 35, 10, 7, 35, 10, 1, 35, 11, 6, 0,
  /* 15669 */ 13, 2, 12, 6, 12, 6, 0,
  /* 15676 */ 12, 6, 12, 6, 12, 6, 12, 6, 0,
  /* 15685 */ 11, 7, 11, 7, 12, 6, 12, 6, 0,
  /* 15694 */ 16, 2, 13, 6, 13, 6, 0,
  /* 15701 */ 12, 7, 12, 7, 13, 6, 13, 6, 0,
  /* 15710 */ 0, 24, 6, 0,
  /* 15714 */ 15, 3, 27, 0, 7, 27, 0, 7, 27, 0, 7, 27, 0, 7, 27, 0, 7, 27, 0, 7, 27, 0, 7, 0,
  /* 15738 */ 21, 2, 4, 4, 4, 4, 5, 4, 7, 0,
  /* 15748 */ 21, 2, 4, 4, 4, 4, 5, 5, 4, 7, 0,
  /* 15759 */ 21, 2, 7, 7, 7, 7, 5, 5, 4, 7, 0,
  /* 15770 */ 21, 2, 7, 7, 7, 7, 5, 4, 7, 0,
  /* 15780 */ 9, 4, 11, 6, 4, 7, 0,
  /* 15787 */ 11, 6, 11, 6, 4, 7, 0,
  /* 15794 */ 44, 4, 12, 6, 4, 7, 0,
  /* 15801 */ 50, 4, 13, 6, 4, 7, 0,
  /* 15808 */ 9, 4, 11, 7, 4, 7, 0,
  /* 15815 */ 44, 4, 12, 7, 4, 7, 0,
  /* 15822 */ 50, 4, 13, 7, 4, 7, 0,
  /* 15829 */ 9, 4, 11, 40, 4, 7, 0,
  /* 15836 */ 44, 4, 12, 40, 4, 7, 0,
  /* 15843 */ 50, 4, 13, 40, 4, 7, 0,
  /* 15850 */ 13, 6, 50, 4, 7, 0,
  /* 15856 */ 13, 7, 50, 4, 7, 0,
  /* 15862 */ 13, 40, 50, 4, 7, 0,
  /* 15868 */ 21, 2, 4, 4, 4, 4, 5, 7, 0,
  /* 15877 */ 21, 2, 4, 4, 4, 4, 5, 5, 7, 0,
  /* 15887 */ 21, 2, 7, 7, 7, 7, 5, 5, 7, 0,
  /* 15897 */ 21, 2, 7, 7, 7, 7, 5, 7, 0,
  /* 15906 */ 9, 4, 11, 6, 7, 0,
  /* 15912 */ 44, 4, 12, 6, 7, 0,
  /* 15918 */ 50, 4, 13, 6, 7, 0,
  /* 15924 */ 21, 2, 4, 4, 4, 4, 5, 4, 7, 7, 0,
  /* 15935 */ 21, 2, 4, 4, 4, 4, 5, 5, 4, 7, 7, 0,
  /* 15947 */ 21, 2, 7, 7, 7, 7, 5, 5, 4, 7, 7, 0,
  /* 15959 */ 21, 2, 7, 7, 7, 7, 5, 4, 7, 7, 0,
  /* 15970 */ 21, 2, 4, 4, 4, 4, 5, 7, 7, 0,
  /* 15980 */ 21, 2, 4, 4, 4, 4, 5, 5, 7, 7, 0,
  /* 15991 */ 21, 2, 7, 7, 7, 7, 5, 5, 7, 7, 0,
  /* 16002 */ 21, 2, 7, 7, 7, 7, 5, 7, 7, 0,
  /* 16012 */ 21, 2, 4, 4, 4, 4, 5, 4, 7, 7, 7, 0,
  /* 16024 */ 21, 2, 4, 4, 4, 4, 5, 5, 4, 7, 7, 7, 0,
  /* 16037 */ 21, 2, 7, 7, 7, 7, 5, 5, 4, 7, 7, 7, 0,
  /* 16050 */ 21, 2, 7, 7, 7, 7, 5, 4, 7, 7, 7, 0,
  /* 16062 */ 21, 2, 4, 4, 4, 4, 5, 7, 7, 7, 0,
  /* 16073 */ 21, 2, 4, 4, 4, 4, 5, 5, 7, 7, 7, 0,
  /* 16085 */ 21, 2, 7, 7, 7, 7, 5, 5, 7, 7, 7, 0,
  /* 16097 */ 21, 2, 7, 7, 7, 7, 5, 7, 7, 7, 0,
  /* 16108 */ 21, 2, 7, 7, 7, 7, 4, 4, 4, 4, 4, 4, 7, 7, 7, 7, 0,
  /* 16125 */ 21, 2, 7, 7, 7, 7, 4, 4, 4, 7, 7, 7, 7, 0,
  /* 16139 */ 21, 2, 4, 4, 4, 4, 5, 4, 7, 7, 7, 7, 0,
  /* 16152 */ 21, 2, 4, 4, 4, 4, 5, 5, 4, 7, 7, 7, 7, 0,
  /* 16166 */ 21, 2, 7, 7, 7, 7, 5, 5, 4, 7, 7, 7, 7, 0,
  /* 16180 */ 21, 2, 7, 7, 7, 7, 5, 4, 7, 7, 7, 7, 0,
  /* 16193 */ 21, 2, 4, 4, 4, 4, 5, 7, 7, 7, 7, 0,
  /* 16205 */ 21, 2, 4, 4, 4, 4, 5, 5, 7, 7, 7, 7, 0,
  /* 16218 */ 21, 2, 7, 7, 7, 7, 5, 5, 7, 7, 7, 7, 0,
  /* 16231 */ 21, 2, 7, 7, 7, 7, 5, 7, 7, 7, 7, 0,
  /* 16243 */ 21, 2, 7, 7, 7, 7, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 7, 7, 7, 7, 0,
  /* 16266 */ 21, 2, 7, 7, 7, 7, 9, 6, 9, 6, 9, 6, 7, 7, 7, 7, 0,
  /* 16283 */ 21, 2, 4, 4, 4, 4, 5, 4, 7, 7, 7, 7, 7, 7, 0,
  /* 16298 */ 21, 2, 4, 4, 4, 4, 5, 5, 4, 7, 7, 7, 7, 7, 7, 0,
  /* 16314 */ 21, 2, 7, 7, 7, 7, 5, 5, 4, 7, 7, 7, 7, 7, 7, 0,
  /* 16330 */ 21, 2, 7, 7, 7, 7, 5, 4, 7, 7, 7, 7, 7, 7, 0,
  /* 16345 */ 21, 2, 4, 4, 4, 4, 5, 7, 7, 7, 7, 7, 7, 0,
  /* 16359 */ 21, 2, 4, 4, 4, 4, 5, 5, 7, 7, 7, 7, 7, 7, 0,
  /* 16374 */ 21, 2, 7, 7, 7, 7, 5, 5, 7, 7, 7, 7, 7, 7, 0,
  /* 16389 */ 21, 2, 7, 7, 7, 7, 5, 7, 7, 7, 7, 7, 7, 0,
  /* 16403 */ 21, 6, 7, 7, 7, 7, 7, 7, 7, 7, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16432 */ 21, 6, 7, 7, 7, 7, 7, 7, 7, 7, 4, 4, 4, 4, 4, 4, 4, 4, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16459 */ 0, 15, 4, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16471 */ 21, 2, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16522 */ 21, 6, 7, 7, 7, 7, 7, 7, 7, 7, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 9, 6, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16573 */ 21, 6, 7, 7, 7, 7, 7, 7, 7, 7, 9, 6, 9, 6, 9, 6, 9, 6, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16600 */ 21, 2, 4, 4, 4, 4, 5, 4, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16618 */ 21, 2, 7, 7, 7, 7, 5, 4, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16636 */ 21, 2, 4, 4, 4, 4, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16653 */ 21, 2, 4, 4, 4, 4, 5, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16671 */ 21, 2, 7, 7, 7, 7, 5, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16689 */ 21, 2, 7, 7, 7, 7, 5, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0,
  /* 16706 */ 9, 4, 11, 7, 7, 0,
  /* 16712 */ 44, 4, 12, 7, 7, 0,
  /* 16718 */ 50, 4, 12, 7, 12, 7, 7, 0,
  /* 16726 */ 9, 40, 7, 7, 0,
  /* 16731 */ 10, 7, 11, 3, 11, 3, 10, 7, 0,
  /* 16740 */ 10, 7, 10, 7, 10, 4, 10, 7, 0,
  /* 16749 */ 0, 14, 10, 4, 10, 7, 0,
  /* 16756 */ 11, 3, 10, 7, 10, 7, 0,
  /* 16763 */ 21, 0, 10, 4, 4, 10, 7, 10, 7, 0,
  /* 16773 */ 10, 4, 10, 7, 10, 7, 0,
  /* 16780 */ 11, 6, 10, 7, 10, 7, 0,
  /* 16787 */ 10, 7, 10, 7, 10, 7, 10, 7, 0,
  /* 16796 */ 11, 7, 10, 7, 10, 7, 0,
  /* 16803 */ 11, 40, 10, 7, 10, 7, 0,
  /* 16810 */ 11, 7, 11, 7, 10, 7, 0,
  /* 16817 */ 17, 10, 7, 0,
  /* 16821 */ 35, 9, 5, 35, 9, 5, 35, 9, 1, 35, 10, 7, 0,
  /* 16834 */ 35, 9, 8, 35, 9, 8, 35, 9, 1, 35, 10, 7, 0,
  /* 16847 */ 35, 11, 6, 35, 11, 6, 35, 10, 1, 35, 10, 7, 0,
  /* 16860 */ 35, 11, 40, 35, 11, 40, 35, 10, 1, 35, 10, 7, 0,
  /* 16873 */ 35, 12, 2, 35, 10, 7, 35, 10, 7, 35, 10, 7, 35, 10, 7, 0,
  /* 16889 */ 35, 11, 40, 35, 10, 7, 35, 10, 7, 0,
  /* 16899 */ 11, 40, 10, 7, 0,
  /* 16904 */ 11, 7, 11, 7, 11, 4, 11, 7, 0,
  /* 16913 */ 0, 14, 11, 4, 11, 7, 0,
  /* 16920 */ 12, 3, 11, 7, 11, 7, 0,
  /* 16927 */ 11, 4, 11, 7, 11, 7, 0,
  /* 16934 */ 11, 7, 11, 7, 11, 7, 11, 7, 0,
  /* 16943 */ 12, 40, 11, 7, 11, 7, 0,
  /* 16950 */ 11, 40, 11, 7, 0,
  /* 16955 */ 12, 7, 12, 7, 12, 4, 12, 7, 0,
  /* 16964 */ 13, 40, 12, 7, 12, 7, 0,
  /* 16971 */ 12, 40, 12, 7, 0,
  /* 16976 */ 21, 0, 15, 1, 1, 15, 7, 0,
  /* 16984 */ 15, 3, 29, 1, 0, 28, 3, 1, 15, 7, 0,
  /* 16995 */ 15, 3, 28, 3, 1, 15, 7, 0,
  /* 17003 */ 15, 3, 15, 7, 28, 3, 1, 15, 7, 0,
  /* 17013 */ 15, 3, 15, 12, 28, 3, 1, 15, 7, 0,
  /* 17023 */ 15, 3, 14, 28, 3, 1, 15, 7, 0,
  /* 17032 */ 28, 1, 8, 15, 1, 15, 7, 0,
  /* 17040 */ 15, 3, 12, 2, 12, 2, 12, 2, 12, 2, 15, 7, 0,
  /* 17053 */ 15, 3, 15, 7, 12, 2, 12, 2, 12, 2, 12, 2, 15, 7, 0,
  /* 17068 */ 15, 3, 12, 2, 12, 2, 12, 2, 15, 7, 0,
  /* 17079 */ 15, 3, 15, 7, 12, 2, 12, 2, 12, 2, 15, 7, 0,
  /* 17092 */ 15, 3, 12, 2, 12, 2, 15, 7, 0,
  /* 17101 */ 15, 3, 15, 7, 12, 2, 12, 2, 15, 7, 0,
  /* 17112 */ 15, 3, 12, 2, 15, 7, 0,
  /* 17119 */ 15, 3, 15, 7, 12, 2, 15, 7, 0,
  /* 17128 */ 15, 7, 15, 2, 15, 7, 0,
  /* 17135 */ 34, 7, 15, 2, 15, 7, 0,
  /* 17142 */ 28, 3, 1, 28, 3, 1, 15, 3, 15, 7, 0,
  /* 17153 */ 35, 12, 2, 35, 12, 2, 15, 3, 15, 7, 0,
  /* 17164 */ 0, 4, 15, 3, 15, 7, 0,
  /* 17171 */ 0, 4, 4, 15, 3, 15, 7, 0,
  /* 17179 */ 21, 0, 4, 4, 4, 4, 4, 4, 4, 15, 3, 15, 7, 0,
  /* 17193 */ 5, 15, 3, 15, 7, 0,
  /* 17199 */ 36, 7, 36, 7, 15, 3, 15, 7, 0,
  /* 17208 */ 15, 2, 4, 15, 9, 4, 15, 17, 3, 15, 7, 0,
  /* 17220 */ 15, 2, 15, 9, 15, 17, 3, 15, 7, 0,
  /* 17230 */ 15, 1, 4, 15, 7, 0,
  /* 17236 */ 15, 0, 15, 7, 4, 15, 7, 0,
  /* 17244 */ 0, 15, 4, 15, 7, 0,
  /* 17250 */ 15, 0, 15, 7, 15, 7, 0,
  /* 17257 */ 21, 0, 15, 1, 28, 1, 1, 15, 7, 15, 7, 0,
  /* 17269 */ 15, 3, 28, 3, 1, 15, 7, 15, 7, 0,
  /* 17279 */ 15, 1, 15, 7, 15, 7, 0,
  /* 17286 */ 15, 7, 15, 2, 15, 7, 15, 7, 0,
  /* 17295 */ 0, 4, 15, 3, 15, 7, 15, 7, 0,
  /* 17304 */ 15, 3, 4, 15, 7, 15, 7, 0,
  /* 17312 */ 15, 3, 4, 4, 15, 7, 15, 7, 0,
  /* 17321 */ 15, 3, 28, 3, 1, 15, 7, 15, 7, 15, 7, 0,
  /* 17333 */ 15, 1, 15, 7, 15, 7, 15, 7, 0,
  /* 17342 */ 15, 7, 15, 2, 15, 7, 15, 7, 15, 7, 0,
  /* 17353 */ 21, 0, 15, 3, 15, 7, 15, 7, 15, 7, 0,
  /* 17364 */ 0, 4, 15, 3, 15, 7, 15, 7, 15, 7, 0,
  /* 17375 */ 0, 4, 4, 15, 3, 15, 7, 15, 7, 15, 7, 0,
  /* 17387 */ 37, 7, 15, 3, 15, 7, 15, 7, 15, 7, 0,
  /* 17398 */ 15, 3, 4, 15, 7, 15, 7, 15, 7, 0,
  /* 17408 */ 0, 15, 1, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17420 */ 21, 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17433 */ 0, 4, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17446 */ 15, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17461 */ 21, 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17476 */ 15, 7, 15, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17495 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17514 */ 0, 4, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17533 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17554 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17577 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17604 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 49, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17632 */ 15, 1, 15, 12, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17645 */ 0, 15, 1, 15, 11, 15, 15, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17661 */ 0, 15, 1, 15, 11, 15, 19, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17677 */ 21, 0, 15, 3, 15, 7, 49, 15, 7, 15, 7, 15, 7, 15, 7, 0,
  /* 17693 */ 15, 1, 15, 12, 15, 7, 15, 7, 15, 7, 0,
  /* 17704 */ 15, 1, 15, 12, 15, 7, 15, 7, 0,
  /* 17713 */ 21, 0, 15, 2, 15, 9, 15, 7, 0,
  /* 17722 */ 15, 3, 15, 7, 4, 4, 15, 11, 15, 7, 0,
  /* 17733 */ 15, 3, 15, 7, 4, 15, 11, 15, 7, 0,
  /* 17743 */ 15, 3, 15, 7, 15, 7, 4, 15, 11, 15, 7, 0,
  /* 17755 */ 15, 3, 15, 7, 15, 11, 15, 7, 0,
  /* 17764 */ 15, 3, 15, 7, 15, 7, 15, 11, 15, 7, 0,
  /* 17775 */ 15, 1, 15, 12, 15, 7, 0,
  /* 17782 */ 15, 2, 15, 12, 15, 7, 0,
  /* 17789 */ 0, 15, 1, 14, 15, 7, 0,
  /* 17796 */ 15, 2, 15, 9, 15, 15, 15, 7, 0,
  /* 17805 */ 15, 2, 15, 10, 15, 15, 15, 7, 0,
  /* 17814 */ 15, 3, 15, 11, 4, 4, 4, 15, 19, 15, 7, 0,
  /* 17826 */ 15, 3, 15, 7, 15, 11, 4, 4, 4, 15, 19, 15, 7, 0,
  /* 17840 */ 15, 3, 15, 11, 4, 4, 15, 19, 15, 7, 0,
  /* 17851 */ 15, 3, 15, 11, 15, 15, 4, 4, 15, 19, 15, 7, 0,
  /* 17864 */ 15, 3, 15, 11, 4, 15, 19, 15, 7, 0,
  /* 17874 */ 15, 3, 15, 11, 15, 15, 4, 15, 19, 15, 7, 0,
  /* 17886 */ 15, 1, 22, 7, 0,
  /* 17891 */ 15, 3, 22, 7, 0,
  /* 17896 */ 15, 3, 22, 7, 22, 7, 0,
  /* 17903 */ 15, 3, 23, 7, 0,
  /* 17908 */ 15, 3, 23, 7, 23, 7, 0,
  /* 17915 */ 24, 7, 0,
  /* 17918 */ 34, 7, 15, 3, 28, 3, 1, 34, 7, 0,
  /* 17928 */ 15, 3, 15, 7, 28, 3, 1, 34, 7, 0,
  /* 17938 */ 34, 7, 15, 1, 28, 1, 34, 7, 0,
  /* 17947 */ 34, 7, 15, 2, 28, 2, 34, 7, 0,
  /* 17956 */ 15, 3, 34, 7, 0,
  /* 17961 */ 15, 3, 15, 7, 34, 7, 0,
  /* 17968 */ 15, 3, 34, 7, 34, 7, 0,
  /* 17975 */ 15, 3, 36, 7, 0,
  /* 17980 */ 15, 3, 28, 3, 1, 15, 7, 36, 7, 0,
  /* 17990 */ 21, 0, 15, 3, 15, 7, 36, 7, 0,
  /* 17999 */ 15, 3, 36, 7, 36, 7, 0,
  /* 18006 */ 15, 3, 15, 7, 36, 7, 36, 7, 0,
  /* 18015 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 36, 7, 36, 7, 0,
  /* 18030 */ 15, 3, 15, 7, 37, 7, 37, 7, 0,
  /* 18039 */ 15, 3, 38, 7, 0,
  /* 18044 */ 15, 3, 28, 3, 1, 15, 7, 38, 7, 0,
  /* 18054 */ 15, 3, 15, 7, 38, 7, 0,
  /* 18061 */ 15, 3, 15, 7, 15, 7, 38, 7, 0,
  /* 18070 */ 21, 0, 15, 3, 15, 7, 15, 7, 15, 7, 38, 7, 0,
  /* 18083 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 38, 7, 0,
  /* 18104 */ 21, 0, 15, 3, 15, 7, 15, 7, 15, 7, 38, 7, 38, 7, 0,
  /* 18119 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 38, 7, 38, 7, 38, 7, 38, 7, 0,
  /* 18146 */ 9, 4, 11, 40, 7, 0,
  /* 18152 */ 44, 4, 12, 40, 7, 0,
  /* 18158 */ 50, 4, 13, 40, 7, 0,
  /* 18164 */ 15, 3, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 27, 0, 8, 0,
  /* 18191 */ 21, 0, 15, 1, 15, 7, 28, 1, 8, 0,
  /* 18201 */ 15, 1, 15, 7, 15, 7, 8, 0,
  /* 18209 */ 0, 15, 4, 8, 8, 0,
  /* 18215 */ 21, 0, 8, 8, 8, 8, 8, 8, 0,
  /* 18224 */ 21, 2, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0,
  /* 18238 */ 21, 2, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0,
  /* 18255 */ 21, 2, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0,
  /* 18278 */ 43, 8, 8, 0,
  /* 18282 */ 10, 4, 9, 8, 0,
  /* 18287 */ 9, 8, 9, 8, 9, 5, 9, 8, 0,
  /* 18296 */ 0, 14, 9, 5, 9, 8, 0,
  /* 18303 */ 10, 7, 10, 7, 9, 8, 0,
  /* 18310 */ 4, 4, 9, 8, 9, 8, 0,
  /* 18317 */ 21, 0, 9, 5, 4, 9, 8, 9, 8, 0,
  /* 18327 */ 10, 4, 9, 8, 9, 8, 0,
  /* 18334 */ 9, 5, 9, 8, 9, 8, 0,
  /* 18341 */ 10, 7, 9, 8, 9, 8, 0,
  /* 18348 */ 9, 8, 9, 8, 9, 8, 9, 8, 0,
  /* 18357 */ 10, 8, 9, 8, 9, 8, 0,
  /* 18364 */ 10, 8, 10, 8, 9, 8, 0,
  /* 18371 */ 17, 9, 8, 0,
  /* 18375 */ 35, 10, 4, 35, 10, 4, 35, 9, 1, 35, 9, 8, 0,
  /* 18388 */ 35, 11, 6, 35, 11, 6, 35, 9, 1, 35, 9, 8, 0,
  /* 18401 */ 35, 10, 7, 35, 10, 7, 35, 9, 1, 35, 9, 8, 0,
  /* 18414 */ 10, 4, 10, 8, 0,
  /* 18419 */ 10, 8, 10, 8, 10, 5, 10, 8, 0,
  /* 18428 */ 0, 14, 10, 5, 10, 8, 0,
  /* 18435 */ 10, 7, 10, 8, 0,
  /* 18440 */ 9, 8, 10, 8, 0,
  /* 18445 */ 11, 4, 10, 8, 10, 8, 0,
  /* 18452 */ 10, 5, 10, 8, 10, 8, 0,
  /* 18459 */ 11, 7, 10, 8, 10, 8, 0,
  /* 18466 */ 10, 8, 10, 8, 10, 8, 10, 8, 0,
  /* 18475 */ 11, 8, 11, 8, 11, 5, 11, 8, 0,
  /* 18484 */ 0, 15, 0, 15, 8, 0,
  /* 18490 */ 0, 15, 0, 4, 15, 8, 0,
  /* 18497 */ 0, 15, 0, 4, 4, 15, 8, 0,
  /* 18505 */ 15, 4, 15, 8, 0,
  /* 18510 */ 15, 3, 15, 7, 28, 3, 1, 15, 9, 0,
  /* 18520 */ 15, 3, 15, 7, 15, 7, 28, 3, 1, 15, 9, 0,
  /* 18532 */ 21, 0, 1, 15, 1, 15, 9, 0,
  /* 18540 */ 1, 4, 15, 1, 15, 9, 0,
  /* 18547 */ 15, 3, 15, 7, 15, 7, 28, 3, 3, 15, 9, 0,
  /* 18559 */ 38, 7, 38, 7, 15, 3, 15, 9, 0,
  /* 18568 */ 0, 15, 15, 15, 3, 15, 9, 0,
  /* 18576 */ 15, 0, 15, 7, 15, 9, 0,
  /* 18583 */ 15, 2, 15, 7, 15, 9, 0,
  /* 18590 */ 15, 15, 15, 3, 15, 7, 15, 9, 0,
  /* 18599 */ 15, 3, 15, 7, 15, 7, 15, 9, 0,
  /* 18608 */ 15, 1, 15, 7, 15, 7, 15, 7, 15, 9, 0,
  /* 18619 */ 15, 1, 15, 7, 34, 7, 15, 9, 0,
  /* 18628 */ 15, 2, 15, 7, 34, 7, 15, 9, 0,
  /* 18637 */ 15, 3, 15, 7, 15, 7, 38, 7, 15, 9, 0,
  /* 18648 */ 15, 1, 15, 10, 0,
  /* 18653 */ 15, 7, 15, 2, 15, 10, 0,
  /* 18660 */ 15, 7, 15, 2, 15, 7, 15, 10, 0,
  /* 18669 */ 15, 3, 15, 7, 28, 3, 1, 15, 11, 0,
  /* 18679 */ 15, 3, 38, 7, 28, 3, 1, 15, 11, 0,
  /* 18689 */ 15, 3, 28, 11, 1, 15, 11, 0,
  /* 18697 */ 15, 1, 15, 11, 0,
  /* 18702 */ 15, 2, 15, 11, 0,
  /* 18707 */ 4, 4, 15, 3, 15, 11, 0,
  /* 18714 */ 15, 7, 15, 3, 15, 11, 0,
  /* 18721 */ 0, 15, 0, 4, 15, 11, 0,
  /* 18728 */ 15, 3, 4, 15, 7, 12, 2, 12, 2, 4, 15, 11, 0,
  /* 18741 */ 15, 3, 4, 15, 7, 12, 2, 4, 15, 11, 0,
  /* 18752 */ 0, 15, 3, 4, 15, 11, 0,
  /* 18759 */ 4, 4, 15, 3, 4, 15, 11, 0,
  /* 18767 */ 5, 15, 3, 4, 15, 11, 0,
  /* 18774 */ 15, 3, 15, 7, 4, 4, 4, 4, 15, 11, 0,
  /* 18785 */ 21, 0, 15, 3, 4, 15, 7, 4, 4, 4, 15, 11, 0,
  /* 18798 */ 15, 3, 15, 7, 15, 7, 15, 7, 4, 4, 4, 15, 11, 0,
  /* 18812 */ 21, 0, 4, 15, 3, 15, 7, 4, 4, 15, 11, 0,
  /* 18824 */ 21, 0, 15, 3, 4, 15, 7, 4, 4, 15, 11, 0,
  /* 18836 */ 15, 3, 15, 7, 4, 15, 11, 0,
  /* 18844 */ 15, 3, 4, 15, 7, 4, 15, 11, 0,
  /* 18853 */ 15, 3, 15, 7, 15, 7, 4, 15, 11, 0,
  /* 18863 */ 21, 0, 15, 3, 4, 15, 7, 15, 7, 15, 7, 4, 15, 11, 0,
  /* 18878 */ 15, 2, 15, 7, 15, 11, 0,
  /* 18885 */ 21, 0, 4, 4, 4, 4, 4, 4, 4, 15, 3, 15, 7, 15, 11, 0,
  /* 18901 */ 15, 3, 15, 7, 15, 7, 15, 11, 0,
  /* 18910 */ 15, 3, 15, 7, 15, 7, 15, 7, 15, 11, 0,
  /* 18921 */ 15, 3, 4, 15, 7, 15, 7, 15, 7, 15, 11, 0,
  /* 18933 */ 15, 3, 4, 4, 15, 7, 15, 7, 15, 7, 15, 11, 0,
  /* 18946 */ 21, 0, 15, 7, 15, 3, 15, 12, 0,
  /* 18955 */ 21, 1, 15, 7, 15, 7, 15, 3, 15, 12, 0,
  /* 18966 */ 21, 2, 15, 7, 15, 7, 15, 7, 15, 3, 15, 12, 0,
  /* 18979 */ 0, 15, 4, 15, 12, 0,
  /* 18985 */ 21, 0, 15, 7, 15, 7, 15, 7, 15, 3, 5, 15, 12, 0,
  /* 18999 */ 21, 1, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 3, 5, 15, 12, 0,
  /* 19017 */ 21, 2, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 3, 5, 15, 12, 0,
  /* 19039 */ 0, 15, 3, 15, 7, 5, 15, 12, 0,
  /* 19048 */ 0, 15, 3, 15, 7, 15, 7, 5, 15, 12, 0,
  /* 19059 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 5, 15, 12, 0,
  /* 19072 */ 21, 0, 15, 3, 15, 7, 15, 12, 0,
  /* 19081 */ 0, 15, 3, 15, 7, 15, 7, 15, 12, 0,
  /* 19091 */ 21, 1, 15, 3, 15, 7, 15, 7, 15, 12, 0,
  /* 19102 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 12, 0,
  /* 19114 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 12, 0,
  /* 19127 */ 0, 15, 3, 28, 3, 1, 14, 0,
  /* 19135 */ 21, 0, 15, 3, 15, 7, 28, 3, 1, 14, 0,
  /* 19146 */ 0, 15, 3, 15, 7, 15, 7, 28, 3, 1, 14, 0,
  /* 19158 */ 21, 1, 15, 3, 15, 7, 15, 7, 28, 3, 1, 14, 0,
  /* 19171 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 28, 3, 1, 14, 0,
  /* 19185 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 28, 3, 1, 14, 0,
  /* 19200 */ 0, 15, 3, 35, 25, 1, 14, 0,
  /* 19208 */ 0, 41, 1, 14, 0,
  /* 19213 */ 0, 12, 2, 14, 0,
  /* 19218 */ 0, 12, 2, 12, 2, 14, 0,
  /* 19225 */ 16, 2, 14, 0,
  /* 19229 */ 0, 11, 3, 14, 0,
  /* 19234 */ 0, 12, 2, 4, 14, 0,
  /* 19240 */ 21, 0, 4, 4, 14, 0,
  /* 19246 */ 15, 0, 4, 4, 4, 4, 14, 0,
  /* 19254 */ 14, 14, 4, 4, 4, 14, 0,
  /* 19261 */ 18, 4, 4, 4, 14, 0,
  /* 19267 */ 0, 14, 5, 4, 4, 14, 0,
  /* 19274 */ 21, 0, 4, 14, 14, 4, 4, 14, 0,
  /* 19283 */ 21, 0, 5, 14, 14, 4, 4, 14, 0,
  /* 19292 */ 0, 14, 5, 4, 14, 0,
  /* 19298 */ 0, 10, 4, 14, 0,
  /* 19303 */ 21, 0, 4, 14, 14, 4, 14, 0,
  /* 19311 */ 21, 0, 5, 14, 14, 4, 14, 0,
  /* 19319 */ 0, 15, 4, 14, 0,
  /* 19324 */ 14, 14, 4, 4, 5, 14, 0,
  /* 19331 */ 14, 14, 4, 5, 14, 0,
  /* 19337 */ 21, 0, 5, 5, 14, 0,
  /* 19343 */ 4, 5, 5, 14, 0,
  /* 19348 */ 21, 6, 5, 5, 5, 5, 5, 5, 5, 5, 14, 0,
  /* 19360 */ 21, 0, 2, 9, 5, 9, 5, 14, 0,
  /* 19369 */ 0, 9, 8, 14, 0,
  /* 19374 */ 0, 14, 14, 14, 0,
  /* 19379 */ 1, 14, 14, 14, 0,
  /* 19384 */ 18, 4, 14, 14, 14, 0,
  /* 19390 */ 18, 4, 4, 14, 14, 14, 14, 0,
  /* 19398 */ 0, 17, 17, 14, 0,
  /* 19403 */ 14, 18, 14, 0,
  /* 19407 */ 21, 0, 15, 3, 15, 7, 49, 14, 0,
  /* 19416 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 49, 14, 0,
  /* 19428 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 49, 14, 0,
  /* 19441 */ 0, 15, 4, 15, 12, 15, 15, 4, 15, 15, 0,
  /* 19452 */ 15, 4, 15, 7, 15, 12, 15, 15, 4, 15, 15, 0,
  /* 19464 */ 15, 3, 15, 7, 28, 3, 1, 28, 3, 1, 15, 9, 15, 15, 0,
  /* 19479 */ 15, 3, 15, 7, 15, 7, 28, 3, 3, 28, 3, 1, 15, 9, 15, 15, 0,
  /* 19496 */ 38, 7, 38, 7, 15, 3, 28, 3, 1, 15, 9, 15, 15, 0,
  /* 19510 */ 15, 3, 15, 7, 28, 3, 1, 15, 9, 15, 15, 0,
  /* 19522 */ 15, 3, 15, 7, 15, 7, 28, 3, 1, 15, 9, 15, 15, 0,
  /* 19536 */ 15, 3, 15, 7, 15, 7, 38, 7, 28, 3, 1, 15, 9, 15, 15, 0,
  /* 19552 */ 15, 1, 15, 9, 15, 15, 0,
  /* 19559 */ 15, 3, 15, 9, 15, 15, 0,
  /* 19566 */ 15, 3, 15, 7, 15, 7, 15, 9, 15, 15, 0,
  /* 19577 */ 15, 1, 15, 10, 15, 15, 0,
  /* 19584 */ 15, 3, 15, 11, 15, 15, 0,
  /* 19591 */ 0, 15, 4, 15, 11, 15, 15, 0,
  /* 19599 */ 21, 0, 15, 3, 15, 7, 15, 11, 15, 15, 0,
  /* 19610 */ 15, 3, 15, 12, 15, 15, 0,
  /* 19617 */ 15, 3, 15, 7, 15, 7, 15, 9, 28, 3, 1, 15, 15, 15, 15, 0,
  /* 19633 */ 21, 0, 15, 3, 15, 9, 15, 7, 15, 20, 28, 3, 1, 15, 15, 15, 15, 0,
  /* 19651 */ 15, 3, 15, 7, 15, 7, 28, 3, 1, 15, 9, 15, 15, 15, 15, 0,
  /* 19667 */ 15, 3, 15, 7, 15, 7, 15, 9, 15, 15, 15, 15, 0,
  /* 19680 */ 0, 15, 4, 15, 11, 15, 15, 15, 15, 0,
  /* 19690 */ 0, 15, 4, 15, 11, 15, 15, 15, 15, 15, 15, 0,
  /* 19702 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 11, 15, 15, 15, 15, 15, 15, 0,
  /* 19721 */ 21, 0, 15, 0, 15, 9, 15, 7, 15, 20, 15, 27, 15, 15, 15, 15, 15, 15, 0,
  /* 19740 */ 21, 0, 15, 0, 15, 9, 15, 7, 15, 20, 15, 15, 15, 15, 0,
  /* 19755 */ 21, 0, 15, 3, 15, 9, 15, 7, 15, 20, 15, 15, 0,
  /* 19768 */ 4, 17, 0,
  /* 19771 */ 10, 7, 10, 7, 17, 0,
  /* 19777 */ 9, 8, 17, 0,
  /* 19781 */ 0, 14, 17, 0,
  /* 19785 */ 28, 3, 1, 28, 3, 1, 15, 3, 15, 8, 28, 3, 1, 15, 17, 0,
  /* 19801 */ 15, 3, 15, 7, 15, 7, 15, 8, 28, 3, 1, 15, 17, 0,
  /* 19815 */ 0, 15, 3, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19826 */ 21, 0, 15, 3, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19840 */ 0, 15, 3, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19855 */ 21, 1, 15, 3, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19871 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19888 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19906 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19925 */ 21, 3, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19945 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19966 */ 21, 4, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 19988 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 20011 */ 21, 5, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 20035 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 20060 */ 21, 6, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 28, 3, 1, 15, 17, 0,
  /* 20086 */ 15, 3, 15, 7, 15, 11, 15, 7, 28, 11, 1, 15, 17, 0,
  /* 20100 */ 0, 15, 1, 15, 7, 15, 7, 15, 8, 15, 7, 15, 7, 15, 17, 0,
  /* 20116 */ 15, 3, 15, 7, 15, 11, 15, 7, 15, 17, 0,
  /* 20127 */ 15, 0, 15, 8, 15, 17, 0,
  /* 20134 */ 28, 3, 1, 15, 3, 15, 8, 15, 17, 0,
  /* 20144 */ 15, 3, 15, 7, 15, 7, 15, 8, 15, 17, 0,
  /* 20155 */ 15, 1, 15, 9, 15, 17, 0,
  /* 20162 */ 15, 3, 15, 7, 15, 11, 15, 17, 0,
  /* 20171 */ 0, 15, 3, 15, 12, 15, 17, 0,
  /* 20179 */ 15, 3, 15, 7, 15, 12, 15, 17, 0,
  /* 20188 */ 17, 17, 17, 0,
  /* 20192 */ 15, 0, 18, 0,
  /* 20196 */ 0, 14, 1, 18, 0,
  /* 20201 */ 15, 4, 18, 0,
  /* 20205 */ 14, 18, 0,
  /* 20208 */ 15, 2, 15, 12, 15, 18, 0,
  /* 20215 */ 0, 15, 0, 19, 0,
  /* 20220 */ 15, 1, 19, 0,
  /* 20224 */ 0, 15, 0, 4, 19, 0,
  /* 20230 */ 0, 15, 4, 4, 4, 19, 0,
  /* 20237 */ 0, 15, 4, 9, 4, 4, 19, 0,
  /* 20245 */ 0, 15, 4, 10, 4, 4, 19, 0,
  /* 20253 */ 9, 4, 15, 4, 4, 19, 0,
  /* 20260 */ 10, 4, 15, 4, 4, 19, 0,
  /* 20267 */ 21, 0, 14, 1, 14, 4, 19, 0,
  /* 20275 */ 15, 2, 15, 7, 19, 0,
  /* 20281 */ 15, 2, 15, 7, 15, 7, 19, 0,
  /* 20289 */ 15, 0, 15, 8, 19, 0,
  /* 20295 */ 15, 1, 15, 10, 19, 0,
  /* 20301 */ 15, 2, 15, 10, 19, 0,
  /* 20307 */ 1, 14, 19, 0,
  /* 20311 */ 15, 3, 15, 7, 15, 11, 4, 4, 4, 4, 4, 4, 15, 19, 0,
  /* 20326 */ 15, 3, 15, 7, 15, 11, 4, 4, 4, 15, 19, 0,
  /* 20338 */ 15, 3, 15, 11, 4, 15, 19, 0,
  /* 20346 */ 15, 3, 15, 7, 15, 11, 4, 15, 19, 0,
  /* 20356 */ 15, 3, 4, 15, 7, 15, 11, 4, 15, 19, 0,
  /* 20367 */ 21, 0, 15, 3, 15, 11, 15, 15, 4, 15, 19, 0,
  /* 20379 */ 0, 15, 3, 4, 15, 11, 15, 19, 0,
  /* 20388 */ 15, 3, 15, 7, 4, 15, 11, 15, 19, 0,
  /* 20398 */ 15, 2, 15, 7, 15, 11, 15, 19, 0,
  /* 20407 */ 15, 3, 4, 15, 7, 15, 11, 15, 19, 0,
  /* 20417 */ 15, 2, 15, 7, 4, 19, 19, 0,
  /* 20425 */ 28, 2, 1, 15, 2, 15, 7, 19, 19, 0,
  /* 20435 */ 15, 2, 15, 7, 15, 7, 19, 19, 0,
  /* 20444 */ 15, 2, 15, 7, 15, 7, 15, 7, 19, 19, 0,
  /* 20455 */ 15, 2, 15, 9, 19, 19, 0,
  /* 20462 */ 15, 2, 15, 7, 15, 9, 19, 19, 0,
  /* 20471 */ 15, 1, 15, 10, 19, 19, 0,
  /* 20478 */ 15, 2, 15, 10, 19, 19, 0,
  /* 20485 */ 0, 19, 19, 19, 0,
  /* 20490 */ 0, 19, 19, 19, 19, 19, 19, 0,
  /* 20498 */ 0, 15, 3, 15, 12, 15, 17, 28, 3, 1, 15, 23, 0,
  /* 20511 */ 15, 3, 15, 7, 15, 8, 15, 7, 28, 3, 1, 15, 17, 15, 23, 0,
  /* 20527 */ 15, 3, 15, 7, 15, 7, 15, 8, 28, 3, 1, 15, 17, 15, 23, 0,
  /* 20543 */ 15, 3, 15, 7, 15, 11, 28, 3, 1, 15, 17, 15, 23, 0,
  /* 20557 */ 15, 3, 15, 7, 15, 12, 28, 3, 1, 15, 17, 15, 23, 0,
  /* 20571 */ 15, 3, 15, 7, 15, 11, 15, 7, 28, 11, 1, 15, 17, 15, 23, 0,
  /* 20587 */ 15, 3, 15, 7, 15, 11, 28, 11, 1, 15, 17, 15, 23, 0,
  /* 20601 */ 15, 3, 15, 7, 15, 8, 15, 7, 15, 17, 15, 23, 0,
  /* 20614 */ 15, 3, 15, 7, 15, 11, 15, 7, 15, 17, 15, 23, 0,
  /* 20627 */ 15, 3, 15, 7, 15, 7, 15, 8, 15, 17, 15, 23, 0,
  /* 20640 */ 15, 3, 15, 7, 15, 11, 15, 17, 15, 23, 0,
  /* 20651 */ 0, 15, 0, 15, 12, 15, 17, 15, 23, 0,
  /* 20661 */ 0, 15, 3, 15, 12, 15, 17, 15, 23, 0,
  /* 20671 */ 15, 0, 15, 7, 15, 12, 15, 17, 15, 23, 0,
  /* 20682 */ 15, 3, 15, 7, 15, 12, 15, 17, 15, 23, 0,
  /* 20693 */ 15, 3, 15, 7, 15, 10, 34, 15, 15, 17, 15, 23, 0,
  /* 20706 */ 15, 3, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 23, 15, 23, 0,
  /* 20722 */ 15, 3, 15, 7, 15, 8, 15, 7, 28, 3, 1, 15, 17, 15, 23, 15, 23, 0,
  /* 20740 */ 15, 3, 15, 7, 15, 7, 15, 8, 28, 3, 1, 15, 17, 15, 23, 15, 23, 0,
  /* 20758 */ 15, 3, 15, 7, 15, 11, 28, 3, 1, 15, 17, 15, 23, 15, 23, 0,
  /* 20774 */ 15, 3, 15, 7, 15, 10, 34, 15, 28, 3, 1, 15, 17, 15, 23, 15, 23, 0,
  /* 20792 */ 15, 3, 15, 7, 15, 8, 15, 7, 15, 17, 15, 23, 15, 23, 0,
  /* 20807 */ 0, 15, 0, 15, 12, 15, 17, 15, 23, 15, 23, 0,
  /* 20819 */ 15, 0, 15, 7, 15, 12, 15, 17, 15, 23, 15, 23, 0,
  /* 20832 */ 15, 0, 15, 7, 15, 12, 15, 17, 15, 27, 15, 23, 15, 23, 15, 23, 0,
  /* 20849 */ 0, 15, 0, 15, 12, 15, 17, 15, 27, 15, 23, 15, 23, 0,
  /* 20863 */ 21, 0, 15, 3, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 20879 */ 0, 15, 3, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 20896 */ 21, 1, 15, 3, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 20914 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 20933 */ 21, 2, 15, 3, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 20953 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 20974 */ 21, 3, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 20996 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 21019 */ 21, 4, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 21043 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 21068 */ 21, 5, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 21094 */ 0, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 21121 */ 21, 6, 15, 3, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 7, 15, 12, 15, 17, 28, 3, 1, 15, 25, 0,
  /* 21149 */ 0, 15, 3, 15, 12, 15, 19, 28, 3, 1, 15, 25, 0,
  /* 21162 */ 15, 1, 15, 9, 15, 17, 15, 7, 15, 25, 0,
  /* 21173 */ 15, 2, 15, 9, 15, 17, 15, 7, 15, 25, 0,
  /* 21184 */ 15, 2, 15, 10, 15, 18, 15, 7, 15, 25, 0,
  /* 21195 */ 15, 0, 15, 8, 15, 16, 15, 25, 0,
  /* 21204 */ 0, 15, 1, 15, 7, 15, 11, 15, 16, 15, 25, 0,
  /* 21216 */ 15, 3, 15, 7, 15, 11, 15, 16, 15, 25, 0,
  /* 21227 */ 15, 3, 15, 9, 15, 15, 15, 16, 15, 25, 0,
  /* 21238 */ 0, 15, 3, 15, 12, 15, 17, 15, 25, 0,
  /* 21248 */ 15, 3, 15, 7, 15, 12, 15, 19, 15, 25, 0,
  /* 21259 */ 0, 15, 0, 26, 0,
  /* 21264 */ 4, 4, 15, 0, 26, 0,
  /* 21270 */ 0, 1, 26, 0,
  /* 21274 */ 0, 15, 0, 14, 1, 2, 2, 2, 1, 26, 0,
  /* 21285 */ 0, 14, 1, 26, 0,
  /* 21290 */ 15, 0, 4, 26, 0,
  /* 21295 */ 18, 5, 4, 15, 4, 4, 4, 26, 0,
  /* 21304 */ 0, 5, 4, 26, 0,
  /* 21309 */ 15, 0, 5, 4, 14, 4, 26, 0,
  /* 21317 */ 0, 15, 4, 15, 9, 15, 16, 15, 24, 4, 26, 0,
  /* 21329 */ 7, 7, 7, 7, 26, 0,
  /* 21335 */ 8, 8, 8, 8, 26, 0,
  /* 21341 */ 15, 4, 1, 15, 8, 26, 0,
  /* 21348 */ 15, 0, 15, 12, 26, 0,
  /* 21354 */ 4, 14, 26, 0,
  /* 21358 */ 15, 0, 4, 14, 14, 26, 0,
  /* 21365 */ 18, 26, 0,
  /* 21368 */ 0, 19, 19, 26, 0,
  /* 21373 */ 43, 43, 43, 43, 26, 0,
  /* 21379 */ 15, 3, 15, 12, 15, 19, 4, 4, 4, 15, 27, 0,
  /* 21391 */ 0, 15, 4, 15, 11, 15, 19, 4, 4, 15, 27, 0,
  /* 21403 */ 1, 30, 0,
  /* 21406 */ 4, 30, 0,
  /* 21409 */ 30, 9, 5, 9, 5, 30, 0,
  /* 21416 */ 4, 25, 30, 0,
  /* 21420 */ 25, 30, 9, 5, 9, 5, 25, 30, 0,
  /* 21429 */ 4, 4, 25, 30, 25, 30, 0,
  /* 21436 */ 25, 30, 25, 30, 25, 30, 25, 30, 0,
  /* 21445 */ 21, 0, 30, 4, 30, 30, 0,
  /* 21452 */ 30, 30, 30, 30, 0,
  /* 21457 */ 15, 3, 15, 7, 15, 11, 15, 16, 28, 3, 1, 15, 25, 15, 31, 0,
  /* 21473 */ 15, 3, 15, 7, 15, 11, 15, 17, 28, 3, 1, 15, 25, 15, 31, 0,
  /* 21489 */ 15, 3, 15, 7, 15, 8, 15, 19, 28, 3, 1, 15, 25, 15, 31, 0,
  /* 21505 */ 15, 3, 15, 7, 15, 12, 15, 19, 28, 3, 1, 15, 25, 15, 31, 0,
  /* 21521 */ 15, 3, 15, 7, 15, 8, 15, 19, 28, 19, 1, 15, 25, 15, 31, 0,
  /* 21537 */ 15, 3, 15, 7, 15, 11, 15, 16, 15, 25, 15, 31, 0,
  /* 21550 */ 15, 3, 15, 7, 15, 11, 15, 17, 15, 25, 15, 31, 0,
  /* 21563 */ 15, 3, 15, 7, 15, 8, 15, 19, 15, 25, 15, 31, 0,
  /* 21576 */ 15, 3, 15, 7, 15, 11, 15, 19, 15, 25, 15, 31, 0,
  /* 21589 */ 0, 15, 0, 15, 12, 15, 19, 15, 25, 15, 31, 0,
  /* 21601 */ 15, 0, 15, 7, 15, 12, 15, 19, 15, 25, 15, 31, 0,
  /* 21614 */ 15, 3, 15, 7, 15, 11, 15, 16, 28, 3, 1, 15, 25, 15, 31, 15, 31, 0,
  /* 21632 */ 15, 3, 15, 7, 15, 8, 15, 19, 28, 3, 1, 15, 25, 15, 31, 15, 31, 0,
  /* 21650 */ 15, 3, 15, 7, 15, 8, 15, 19, 15, 25, 15, 31, 15, 31, 0,
  /* 21665 */ 15, 0, 15, 7, 15, 12, 15, 19, 15, 25, 15, 31, 15, 31, 0,
  /* 21680 */ 5, 33, 0,
  /* 21683 */ 8, 33, 0,
  /* 21686 */ 15, 3, 15, 9, 15, 19, 15, 24, 15, 33, 0,
  /* 21697 */ 0, 15, 1, 15, 11, 15, 19, 15, 24, 15, 33, 0,
  /* 21709 */ 33, 33, 33, 33, 0,
  /* 21714 */ 43, 33, 0,
  /* 21717 */ 0, 15, 0, 15, 12, 15, 19, 15, 27, 15, 33, 15, 39, 0,
  /* 21731 */ 15, 0, 15, 7, 15, 12, 15, 19, 15, 27, 15, 33, 15, 39, 15, 39, 0,
  /* 21748 */ 3, 40, 0,
  /* 21751 */ 3, 9, 40, 0,
  /* 21755 */ 9, 40, 9, 40, 9, 40, 9, 40, 0,
  /* 21764 */ 11, 3, 11, 40, 0,
  /* 21769 */ 35, 10, 7, 35, 10, 7, 35, 11, 40, 35, 11, 40, 0,
  /* 21782 */ 4, 11, 40, 11, 40, 0,
  /* 21788 */ 10, 7, 10, 7, 11, 40, 11, 40, 0,
  /* 21797 */ 11, 40, 11, 40, 11, 40, 11, 40, 0,
  /* 21806 */ 12, 3, 12, 40, 0,
  /* 21811 */ 11, 7, 11, 7, 12, 40, 12, 40, 0,
  /* 21820 */ 12, 40, 12, 40, 12, 40, 12, 40, 0,
  /* 21829 */ 13, 3, 13, 40, 0,
  /* 21834 */ 12, 7, 12, 7, 13, 40, 13, 40, 0,
  /* 21843 */ 13, 40, 13, 40, 13, 40, 13, 40, 0,
  /* 21852 */ 40, 9, 40, 9, 40, 40, 0,
  /* 21859 */ 40, 40, 40, 40, 0,
  /* 21864 */ 15, 3, 15, 9, 15, 19, 15, 27, 15, 32, 15, 41, 0,
  /* 21877 */ 15, 3, 42, 0,
  /* 21881 */ 0, 3, 3, 14, 5, 42, 0,
  /* 21888 */ 42, 3, 3, 3, 42, 42, 42, 0,
  /* 21896 */ 33, 43, 0,
  /* 21899 */ 0, 24, 1, 4, 45, 0,
  /* 21905 */ 0, 24, 1, 4, 46, 0,
  /* 21911 */ 49, 0,
  255
};
#endif

// Add parameter attributes that are not common to all intrinsics.
#ifdef GET_INTRINSIC_ATTRIBUTES
static AttributeSet getIntrinsicArgAttributeSet(LLVMContext &C, unsigned ID,
                                                Type *ArgType) {
  unsigned BitWidth = ArgType->getScalarSizeInBits();
  switch (ID) {
  default: llvm_unreachable("Invalid attribute set number");
  case 0:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
    });
  case 1:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
    });
  case 2:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::WriteOnly),
    });
  case 3:
    return AttributeSet::get(C, {
      Attribute::getWithCaptureInfo(C, CaptureInfo::none()),
      Attribute::get(C, Attribute::ReadOnly),
    });
  case 4:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ReadNone),
    });
  case 5:
    return AttributeSet::get(C, {
      Attribute::getWithCaptureInfo(C, CaptureInfo::none()),
    });
  case 6:
    return AttributeSet::get(C, {
      Attribute::getWithCaptureInfo(C, CaptureInfo::none()),
      Attribute::get(C, Attribute::ReadNone),
    });
  case 7:
    return AttributeSet::get(C, {
      Attribute::getWithCaptureInfo(C, CaptureInfo::none()),
      Attribute::get(C, Attribute::WriteOnly),
    });
  case 8:
    return AttributeSet::get(C, {
      Attribute::getWithCaptureInfo(C, CaptureInfo::none()),
      Attribute::get(C, Attribute::NoAlias),
      Attribute::get(C, Attribute::WriteOnly),
    });
  case 9:
    return AttributeSet::get(C, {
      Attribute::getWithCaptureInfo(C, CaptureInfo::none()),
      Attribute::get(C, Attribute::NoAlias),
      Attribute::get(C, Attribute::ReadOnly),
    });
  case 10:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::Returned),
    });
  case 11:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, -1, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 2, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 12:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NonNull),
    });
  case 13:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ReadOnly),
    });
  case 14:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 16, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 15:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 4, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 16:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::NonNull),
      Attribute::get(C, Attribute::Alignment, 4),
    });
  case 17:
    return AttributeSet::get(C, {
      Attribute::getWithCaptureInfo(C, CaptureInfo::none()),
      Attribute::get(C, Attribute::Alignment, 4),
    });
  case 18:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Alignment, 4),
    });
  case 19:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 32, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 65, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 20:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 1024, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 21:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 3, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 22:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoAlias),
      Attribute::get(C, Attribute::WriteOnly),
    });
  case 23:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoAlias),
      Attribute::get(C, Attribute::ReadOnly),
    });
  case 24:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 128, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 129, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 25:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 1, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 26:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 2, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 27:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 1, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 28:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 2147483647, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 29:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 65535, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 30:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 1, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 2147483648, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 31:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 1, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 65536, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 32:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 32, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 33:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 1, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 1025, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 34:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 1, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 65, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 35:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 64, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 36:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUndef),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 32, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 33, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 37:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 24, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 257, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 38:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 1, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 3, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  case 39:
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::ImmArg),
      Attribute::get(C, Attribute::Range, ConstantRange(APInt(BitWidth, 0, /*isSigned=*/true, /*implicitTrunc=*/true), APInt(BitWidth, 5, /*isSigned=*/true, /*implicitTrunc=*/true))),
    });
  }
} // getIntrinsicArgAttributeSet

static AttributeSet getIntrinsicFnAttributeSet(LLVMContext &C, unsigned ID) {
  switch (ID) {
    default: llvm_unreachable("Invalid attribute set number");
  case 0: // llvm.abs
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Speculatable),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 1: // llvm.addressofreturnaddress
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 2: // llvm.adjust.trampoline
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Ref, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1)),
    });
  case 3: // llvm.allow.runtime.check
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 4: // llvm.arithmetic.fence
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Speculatable),
      Attribute::get(C, Attribute::NoCreateUndefOrPoison),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 5: // llvm.asan.check.memaccess
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
    });
  case 6: // llvm.assume
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: Mod, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(8)),
    });
  case 7: // llvm.call.preallocated.arg
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
    });
  case 8: // llvm.callbr.landingpad
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoMerge),
    });
  case 9: // llvm.codeview.annotation
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::NoDuplicate),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 10: // llvm.coro.align
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 11: // llvm.coro.done
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 12: // llvm.coro.free
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: Ref, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1)),
    });
  case 13: // llvm.eh.sjlj.longjmp
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoReturn),
    });
  case 14: // llvm.experimental.constrained.acos
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::StrictFP),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 15: // llvm.experimental.convergence.anchor
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 16: // llvm.experimental.memset.pattern
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Mod, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2)),
    });
  case 17: // llvm.experimental.stackmap
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
    });
  case 18: // llvm.experimental.vector.histogram.add
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 19: // llvm.experimental.vp.strided.store
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Mod, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2)),
    });
  case 20: // llvm.experimental.widenable.condition
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Speculatable),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 21: // llvm.fake.use
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 22: // llvm.loop.decrement
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::NoDuplicate),
    });
  case 23: // llvm.masked.expandload
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Ref, InaccessibleMem: Ref, ErrnoMem: Ref, Other: Ref, TargetMem0: Ref, TargetMem1: Ref
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1365)),
    });
  case 24: // llvm.masked.scatter
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 25: // llvm.memcpy
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 26: // llvm.memcpy.element.unordered.atomic
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 27: // llvm.memset.element.unordered.atomic
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Mod, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2)),
    });
  case 28: // llvm.prefetch
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: ModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(15)),
    });
  case 29: // llvm.read_volatile_register
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
    });
  case 30: // llvm.reloc.none
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
    });
  case 31: // llvm.seh.try.begin
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 32: // llvm.trap
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoReturn),
      Attribute::get(C, Attribute::Cold),
      // ArgMem: NoModRef, InaccessibleMem: Mod, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(8)),
    });
  case 33: // llvm.write_register
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
    });
  case 34: // llvm.xray.customevent
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 35: // llvm.aarch64.break
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoReturn),
      Attribute::get(C, Attribute::Cold),
    });
  case 36: // llvm.aarch64.dmb
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
    });
  case 37: // llvm.aarch64.neon.fp8.cvtl1
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: Ref, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(4)),
    });
  case 38: // llvm.aarch64.prefetch
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: ModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(15)),
    });
  case 39: // llvm.amdgcn.ballot
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      Attribute::get(C, Attribute::NoCreateUndefOrPoison),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 40: // llvm.amdgcn.call.whole.wave
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::Convergent),
    });
  case 41: // llvm.amdgcn.cluster.load.async.to.lds.b128
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: ModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(15)),
    });
  case 42: // llvm.amdgcn.cluster.load.b128
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: Ref, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1)),
    });
  case 43: // llvm.amdgcn.cluster.workgroup.flat.id
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::Speculatable),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 44: // llvm.amdgcn.cooperative.atomic.load.16x8B
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: Ref, InaccessibleMem: Ref, ErrnoMem: Ref, Other: Ref, TargetMem0: Ref, TargetMem1: Ref
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1365)),
    });
  case 45: // llvm.amdgcn.cooperative.atomic.store.16x8B
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 46: // llvm.amdgcn.cs.chain
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoReturn),
      Attribute::get(C, Attribute::Convergent),
    });
  case 47: // llvm.amdgcn.cvt.scale.pk16.bf16.bf6
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      Attribute::get(C, Attribute::NoCreateUndefOrPoison),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 48: // llvm.amdgcn.ds.add.gs.reg.rtn
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
    });
  case 49: // llvm.amdgcn.ds.append
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 50: // llvm.amdgcn.ds.atomic.async.barrier.arrive.b64
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: ModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(15)),
    });
  case 51: // llvm.amdgcn.ds.atomic.barrier.arrive.rtn.b64
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 52: // llvm.amdgcn.ds.bpermute
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 53: // llvm.amdgcn.ds.bvh.stack.push4.pop1.rtn
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
    });
  case 54: // llvm.amdgcn.ds.gws.barrier
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 55: // llvm.amdgcn.ds.gws.init
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: Mod, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(8)),
    });
  case 56: // llvm.amdgcn.endpgm
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoReturn),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::Cold),
      Attribute::get(C, Attribute::Convergent),
    });
  case 57: // llvm.amdgcn.if.break
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 58: // llvm.amdgcn.iglp.opt
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
    });
  case 59: // llvm.amdgcn.image.bvh.dual.intersect.ray
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Ref, InaccessibleMem: Ref, ErrnoMem: Ref, Other: Ref, TargetMem0: Ref, TargetMem1: Ref
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1365)),
    });
  case 60: // llvm.amdgcn.image.sample.1d
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: Ref, InaccessibleMem: Ref, ErrnoMem: Ref, Other: Ref, TargetMem0: Ref, TargetMem1: Ref
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1365)),
    });
  case 61: // llvm.amdgcn.image.sample.1d.nortn
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
    });
  case 62: // llvm.amdgcn.init.whole.wave
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::Convergent),
    });
  case 63: // llvm.amdgcn.kill
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
    });
  case 64: // llvm.amdgcn.lds.direct.load
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Speculatable),
      // ArgMem: Ref, InaccessibleMem: Ref, ErrnoMem: Ref, Other: Ref, TargetMem0: Ref, TargetMem1: Ref
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1365)),
    });
  case 65: // llvm.amdgcn.mbcnt.hi
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::NoCreateUndefOrPoison),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 66: // llvm.amdgcn.perm
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Speculatable),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 67: // llvm.amdgcn.s.sendmsg
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::WillReturn),
    });
  case 68: // llvm.amdgcn.s.sendmsghalt
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
    });
  case 69: // llvm.amdgcn.s.sleep.var
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
    });
  case 70: // llvm.amdgcn.s.wait.asynccnt
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 71: // llvm.amdgcn.strict.wqm
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      Attribute::get(C, Attribute::Speculatable),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 72: // llvm.amdgcn.swmmac.bf16.16x16x32.bf16
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      Attribute::get(C, Attribute::NoCreateUndefOrPoison),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 73: // llvm.amdgcn.tensor.load.to.lds
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: ModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(15)),
    });
  case 74: // llvm.amdgcn.unreachable
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::Convergent),
    });
  case 75: // llvm.amdgcn.wqm.demote
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      // ArgMem: NoModRef, InaccessibleMem: Mod, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(8)),
    });
  case 76: // llvm.dx.flattened.thread.id.in.group
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 77: // llvm.hexagon.F2.dfadd
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 78: // llvm.hexagon.Y6.dmlink
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 79: // llvm.loongarch.lasx.xvst
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: Mod, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2)),
    });
  case 80: // llvm.mips.insv
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: Ref, InaccessibleMem: Ref, ErrnoMem: Ref, Other: Ref, TargetMem0: Ref, TargetMem1: Ref
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1365)),
    });
  case 81: // llvm.nvvm.activemask
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 82: // llvm.nvvm.atomic.add.gen.f.cta
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 83: // llvm.nvvm.bar.warp.sync
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::Convergent),
    });
  case 84: // llvm.nvvm.cp.async.bulk.global.to.shared.cluster
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 85: // llvm.nvvm.elect.sync
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 86: // llvm.nvvm.exit
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoReturn),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 87: // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: Ref, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1)),
    });
  case 88: // llvm.nvvm.ldu.global.f
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Ref, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1)),
    });
  case 89: // llvm.nvvm.match.all.sync.i32p
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 90: // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cluster.space.cluster
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 91: // llvm.nvvm.mbarrier.inval
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: Mod, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2)),
    });
  case 92: // llvm.nvvm.mbarrier.pending.count
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 93: // llvm.nvvm.mma.and.popc.m16n8k128.row.col.b1
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 94: // llvm.nvvm.nanosleep
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
    });
  case 95: // llvm.nvvm.suld.1d.array.i16.trap
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      // ArgMem: Ref, InaccessibleMem: Ref, ErrnoMem: Ref, Other: Ref, TargetMem0: Ref, TargetMem1: Ref
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1365)),
    });
  case 96: // llvm.nvvm.sust.b.1d.array.i16.clamp
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 97: // llvm.nvvm.sust.b.1d.array.i16.trap
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoFree),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 98: // llvm.nvvm.tcgen05.alloc.cg1
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: ModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(15)),
    });
  case 99: // llvm.nvvm.tcgen05.dealloc.cg1
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 100: // llvm.nvvm.tcgen05.relinq.alloc.permit.cg1
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::Convergent),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 101: // llvm.nvvm.wmma.m16n16k16.store.d.col.f16
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      // ArgMem: Mod, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2)),
    });
  case 102: // llvm.ppc.darn
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::NoMerge),
    });
  case 103: // llvm.ppc.stbcx
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoDuplicate),
      // ArgMem: Mod, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2)),
    });
  case 104: // llvm.r600.group.barrier
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Convergent),
    });
  case 105: // llvm.riscv.cv.elw.elw
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: Ref, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(1)),
    });
  case 106: // llvm.riscv.cv.mac.mac
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      Attribute::get(C, Attribute::Speculatable),
      // ArgMem: NoModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(0)),
    });
  case 107: // llvm.riscv.sf.vste16
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoCallback),
      Attribute::get(C, Attribute::NoSync),
      Attribute::get(C, Attribute::NoFree),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 108: // llvm.s390.tabort
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoReturn),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 109: // llvm.s390.tbegin
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::NoDuplicate),
      // ArgMem: Mod, InaccessibleMem: Mod, ErrnoMem: Mod, Other: Mod, TargetMem0: Mod, TargetMem1: Mod
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2730)),
    });
  case 110: // llvm.spv.lifetime.end
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      Attribute::get(C, Attribute::WillReturn),
      // ArgMem: ModRef, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(3)),
    });
  case 111: // llvm.ve.vl.pfchv.ssl
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: ModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(15)),
    });
  case 112: // llvm.wasm.memory.atomic.notify
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  case 113: // llvm.wasm.memory.atomic.wait32
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: ModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(15)),
    });
  case 114: // llvm.wasm.rethrow
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoReturn),
    });
  case 115: // llvm.x86.sse.stmxcsr
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: Mod, InaccessibleMem: NoModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(2)),
    });
  case 116: // llvm.x86.urdmsr
    return AttributeSet::get(C, {
      Attribute::get(C, Attribute::NoUnwind),
      // ArgMem: NoModRef, InaccessibleMem: ModRef, ErrnoMem: NoModRef, Other: NoModRef, TargetMem0: NoModRef, TargetMem1: NoModRef
      Attribute::getWithMemoryEffects(C, MemoryEffects::createFromIntValue(12)),
    });
  }
} // getIntrinsicFnAttributeSet
static constexpr uint16_t IntrinsicsToAttributesMap[] = {
    0 << 8 | 0, // llvm.abs
    0 << 8 | 1, // llvm.acos
    1 << 8 | 1, // llvm.addressofreturnaddress
    2 << 8 | 1, // llvm.adjust.trampoline
    1 << 8 | 2, // llvm.alloc.token.id
    3 << 8 | 2, // llvm.allow.runtime.check
    3 << 8 | 2, // llvm.allow.sanitize.address
    3 << 8 | 2, // llvm.allow.sanitize.hwaddress
    3 << 8 | 2, // llvm.allow.sanitize.memory
    3 << 8 | 2, // llvm.allow.sanitize.thread
    3 << 8 | 3, // llvm.allow.ubsan.check
    3 << 8 | 1, // llvm.annotation
    4 << 8 | 1, // llvm.arithmetic.fence
    5 << 8 | 0, // llvm.asan.check.memaccess
    0 << 8 | 1, // llvm.asin
    6 << 8 | 4, // llvm.assume
    0 << 8 | 1, // llvm.atan
    0 << 8 | 1, // llvm.atan2
    4 << 8 | 1, // llvm.bitreverse
    4 << 8 | 1, // llvm.bswap
    7 << 8 | 0, // llvm.call.preallocated.arg
    7 << 8 | 5, // llvm.call.preallocated.setup
    7 << 8 | 1, // llvm.call.preallocated.teardown
    8 << 8 | 1, // llvm.callbr.landingpad
    4 << 8 | 1, // llvm.canonicalize
    4 << 8 | 1, // llvm.ceil
    5 << 8 | 1, // llvm.clear_cache
    4 << 8 | 1, // llvm.clmul
    9 << 8 | 1, // llvm.codeview.annotation
    1 << 8 | 1, // llvm.convert.from.fp16
    1 << 8 | 1, // llvm.convert.to.fp16
    4 << 8 | 1, // llvm.copysign
    10 << 8 | 1, // llvm.coro.align
    5 << 8 | 1, // llvm.coro.alloc
    5 << 8 | 1, // llvm.coro.alloca.alloc
    5 << 8 | 1, // llvm.coro.alloca.free
    5 << 8 | 1, // llvm.coro.alloca.get
    5 << 8 | 1, // llvm.coro.async.context.alloc
    5 << 8 | 1, // llvm.coro.async.context.dealloc
    8 << 8 | 1, // llvm.coro.async.resume
    5 << 8 | 1, // llvm.coro.async.size.replace
    127 << 8 | 1, // llvm.coro.await.suspend.bool
    127 << 8 | 1, // llvm.coro.await.suspend.handle
    127 << 8 | 1, // llvm.coro.await.suspend.void
    5 << 8 | 6, // llvm.coro.begin
    5 << 8 | 6, // llvm.coro.begin.custom.abi
    127 << 8 | 1, // llvm.coro.destroy
    11 << 8 | 7, // llvm.coro.done
    5 << 8 | 1, // llvm.coro.end
    5 << 8 | 1, // llvm.coro.end.async
    5 << 8 | 1, // llvm.coro.end.results
    10 << 8 | 1, // llvm.coro.frame
    12 << 8 | 8, // llvm.coro.free
    2 << 8 | 9, // llvm.coro.id
    5 << 8 | 1, // llvm.coro.id.async
    5 << 8 | 1, // llvm.coro.id.retcon
    5 << 8 | 1, // llvm.coro.id.retcon.once
    10 << 8 | 1, // llvm.coro.is_in_ramp
    10 << 8 | 1, // llvm.coro.noop
    10 << 8 | 1, // llvm.coro.prepare.async
    10 << 8 | 1, // llvm.coro.prepare.retcon
    10 << 8 | 10, // llvm.coro.promise
    127 << 8 | 1, // llvm.coro.resume
    8 << 8 | 1, // llvm.coro.save
    10 << 8 | 1, // llvm.coro.size
    2 << 8 | 7, // llvm.coro.subfn.addr
    5 << 8 | 1, // llvm.coro.suspend
    8 << 8 | 1, // llvm.coro.suspend.async
    5 << 8 | 1, // llvm.coro.suspend.retcon
    4 << 8 | 1, // llvm.cos
    0 << 8 | 1, // llvm.cosh
    0 << 8 | 0, // llvm.ctlz
    4 << 8 | 1, // llvm.ctpop
    0 << 8 | 0, // llvm.cttz
    0 << 8 | 1, // llvm.dbg.assign
    0 << 8 | 1, // llvm.dbg.declare
    0 << 8 | 1, // llvm.dbg.label
    0 << 8 | 1, // llvm.dbg.value
    5 << 8 | 1, // llvm.debugtrap
    1 << 8 | 1, // llvm.donothing
    5 << 8 | 1, // llvm.eh.dwarf.cfa
    10 << 8 | 1, // llvm.eh.exceptioncode
    10 << 8 | 1, // llvm.eh.exceptionpointer
    1 << 8 | 1, // llvm.eh.recoverfp
    5 << 8 | 1, // llvm.eh.return.i32
    5 << 8 | 1, // llvm.eh.return.i64
    10 << 8 | 5, // llvm.eh.sjlj.callsite
    5 << 8 | 1, // llvm.eh.sjlj.functioncontext
    13 << 8 | 1, // llvm.eh.sjlj.longjmp
    10 << 8 | 1, // llvm.eh.sjlj.lsda
    5 << 8 | 1, // llvm.eh.sjlj.setjmp
    5 << 8 | 1, // llvm.eh.sjlj.setup.dispatch
    10 << 8 | 1, // llvm.eh.typeid.for
    5 << 8 | 1, // llvm.eh.unwind.init
    4 << 8 | 1, // llvm.exp
    4 << 8 | 1, // llvm.exp10
    4 << 8 | 1, // llvm.exp2
    1 << 8 | 1, // llvm.expect
    1 << 8 | 11, // llvm.expect.with.probability
    14 << 8 | 1, // llvm.experimental.constrained.acos
    14 << 8 | 1, // llvm.experimental.constrained.asin
    14 << 8 | 1, // llvm.experimental.constrained.atan
    14 << 8 | 1, // llvm.experimental.constrained.atan2
    14 << 8 | 1, // llvm.experimental.constrained.ceil
    14 << 8 | 1, // llvm.experimental.constrained.cos
    14 << 8 | 1, // llvm.experimental.constrained.cosh
    14 << 8 | 1, // llvm.experimental.constrained.exp
    14 << 8 | 1, // llvm.experimental.constrained.exp2
    14 << 8 | 1, // llvm.experimental.constrained.fadd
    14 << 8 | 1, // llvm.experimental.constrained.fcmp
    14 << 8 | 1, // llvm.experimental.constrained.fcmps
    14 << 8 | 1, // llvm.experimental.constrained.fdiv
    14 << 8 | 1, // llvm.experimental.constrained.floor
    14 << 8 | 1, // llvm.experimental.constrained.fma
    14 << 8 | 1, // llvm.experimental.constrained.fmul
    14 << 8 | 1, // llvm.experimental.constrained.fmuladd
    14 << 8 | 1, // llvm.experimental.constrained.fpext
    14 << 8 | 1, // llvm.experimental.constrained.fptosi
    14 << 8 | 1, // llvm.experimental.constrained.fptoui
    14 << 8 | 1, // llvm.experimental.constrained.fptrunc
    14 << 8 | 1, // llvm.experimental.constrained.frem
    14 << 8 | 1, // llvm.experimental.constrained.fsub
    14 << 8 | 1, // llvm.experimental.constrained.ldexp
    14 << 8 | 1, // llvm.experimental.constrained.llrint
    14 << 8 | 1, // llvm.experimental.constrained.llround
    14 << 8 | 1, // llvm.experimental.constrained.log
    14 << 8 | 1, // llvm.experimental.constrained.log10
    14 << 8 | 1, // llvm.experimental.constrained.log2
    14 << 8 | 1, // llvm.experimental.constrained.lrint
    14 << 8 | 1, // llvm.experimental.constrained.lround
    14 << 8 | 1, // llvm.experimental.constrained.maximum
    14 << 8 | 1, // llvm.experimental.constrained.maxnum
    14 << 8 | 1, // llvm.experimental.constrained.minimum
    14 << 8 | 1, // llvm.experimental.constrained.minnum
    14 << 8 | 1, // llvm.experimental.constrained.nearbyint
    14 << 8 | 1, // llvm.experimental.constrained.pow
    14 << 8 | 1, // llvm.experimental.constrained.powi
    14 << 8 | 1, // llvm.experimental.constrained.rint
    14 << 8 | 1, // llvm.experimental.constrained.round
    14 << 8 | 1, // llvm.experimental.constrained.roundeven
    14 << 8 | 1, // llvm.experimental.constrained.sin
    14 << 8 | 1, // llvm.experimental.constrained.sinh
    14 << 8 | 1, // llvm.experimental.constrained.sitofp
    14 << 8 | 1, // llvm.experimental.constrained.sqrt
    14 << 8 | 1, // llvm.experimental.constrained.tan
    14 << 8 | 1, // llvm.experimental.constrained.tanh
    14 << 8 | 1, // llvm.experimental.constrained.trunc
    14 << 8 | 1, // llvm.experimental.constrained.uitofp
    15 << 8 | 1, // llvm.experimental.convergence.anchor
    15 << 8 | 1, // llvm.experimental.convergence.entry
    15 << 8 | 1, // llvm.experimental.convergence.loop
    0 << 8 | 0, // llvm.experimental.cttz.elts
    127 << 8 | 1, // llvm.experimental.deoptimize
    1 << 8 | 12, // llvm.experimental.gc.get.pointer.base
    1 << 8 | 12, // llvm.experimental.gc.get.pointer.offset
    1 << 8 | 13, // llvm.experimental.gc.relocate
    1 << 8 | 1, // llvm.experimental.gc.result
    127 << 8 | 14, // llvm.experimental.gc.statepoint
    0 << 8 | 13, // llvm.experimental.get.vector.length
    127 << 8 | 1, // llvm.experimental.guard
    16 << 8 | 15, // llvm.experimental.memset.pattern
    3 << 8 | 1, // llvm.experimental.noalias.scope.decl
    127 << 8 | 16, // llvm.experimental.patchpoint
    127 << 8 | 16, // llvm.experimental.patchpoint.void
    17 << 8 | 17, // llvm.experimental.stackmap
    1 << 8 | 1, // llvm.experimental.vector.compress
    0 << 8 | 1, // llvm.experimental.vector.extract.last.active
    18 << 8 | 1, // llvm.experimental.vector.histogram.add
    18 << 8 | 1, // llvm.experimental.vector.histogram.uadd.sat
    18 << 8 | 1, // llvm.experimental.vector.histogram.umax
    18 << 8 | 1, // llvm.experimental.vector.histogram.umin
    0 << 8 | 1, // llvm.experimental.vector.match
    0 << 8 | 1, // llvm.experimental.vp.reverse
    0 << 8 | 11, // llvm.experimental.vp.splice
    2 << 8 | 10, // llvm.experimental.vp.strided.load
    19 << 8 | 18, // llvm.experimental.vp.strided.store
    20 << 8 | 2, // llvm.experimental.widenable.condition
    4 << 8 | 1, // llvm.fabs
    21 << 8 | 1, // llvm.fake.use
    4 << 8 | 1, // llvm.floor
    4 << 8 | 1, // llvm.fma
    4 << 8 | 1, // llvm.fmuladd
    4 << 8 | 1, // llvm.fptosi.sat
    4 << 8 | 1, // llvm.fptoui.sat
    4 << 8 | 1, // llvm.fptrunc.round
    1 << 8 | 5, // llvm.frameaddress
    4 << 8 | 1, // llvm.frexp
    4 << 8 | 1, // llvm.fshl
    4 << 8 | 1, // llvm.fshr
    12 << 8 | 1, // llvm.gcread
    5 << 8 | 1, // llvm.gcroot
    11 << 8 | 19, // llvm.gcwrite
    0 << 8 | 1, // llvm.get.active.lane.mask
    7 << 8 | 1, // llvm.get.dynamic.area.offset
    3 << 8 | 1, // llvm.get.fpenv
    3 << 8 | 1, // llvm.get.fpmode
    3 << 8 | 1, // llvm.get.rounding
    5 << 8 | 11, // llvm.hwasan.check.memaccess
    5 << 8 | 13, // llvm.hwasan.check.memaccess.fixedshadow
    5 << 8 | 11, // llvm.hwasan.check.memaccess.shortgranules
    5 << 8 | 13, // llvm.hwasan.check.memaccess.shortgranules.fixedshadow
    7 << 8 | 1, // llvm.icall.branch.funnel
    18 << 8 | 20, // llvm.init.trampoline
    5 << 8 | 1, // llvm.instrprof.callsite
    5 << 8 | 1, // llvm.instrprof.cover
    5 << 8 | 1, // llvm.instrprof.increment
    5 << 8 | 1, // llvm.instrprof.increment.step
    5 << 8 | 1, // llvm.instrprof.mcdc.parameters
    5 << 8 | 1, // llvm.instrprof.mcdc.tvbitmap.update
    5 << 8 | 1, // llvm.instrprof.timestamp
    5 << 8 | 1, // llvm.instrprof.value.profile
    18 << 8 | 21, // llvm.invariant.end
    18 << 8 | 22, // llvm.invariant.start
    15 << 8 | 1, // llvm.is.constant
    4 << 8 | 0, // llvm.is.fpclass
    20 << 8 | 1, // llvm.launder.invariant.group
    4 << 8 | 1, // llvm.ldexp
    18 << 8 | 10, // llvm.lifetime.end
    18 << 8 | 10, // llvm.lifetime.start
    4 << 8 | 1, // llvm.llrint
    4 << 8 | 1, // llvm.llround
    2 << 8 | 1, // llvm.load.relative
    1 << 8 | 1, // llvm.localaddress
    7 << 8 | 1, // llvm.localescape
    1 << 8 | 11, // llvm.localrecover
    4 << 8 | 1, // llvm.log
    4 << 8 | 1, // llvm.log10
    4 << 8 | 1, // llvm.log2
    22 << 8 | 1, // llvm.loop.decrement
    22 << 8 | 1, // llvm.loop.decrement.reg
    1 << 8 | 11, // llvm.loop.dependence.raw.mask
    1 << 8 | 11, // llvm.loop.dependence.war.mask
    4 << 8 | 1, // llvm.lrint
    4 << 8 | 1, // llvm.lround
    19 << 8 | 18, // llvm.masked.compressstore
    23 << 8 | 10, // llvm.masked.expandload
    23 << 8 | 1, // llvm.masked.gather
    2 << 8 | 10, // llvm.masked.load
    24 << 8 | 1, // llvm.masked.scatter
    19 << 8 | 18, // llvm.masked.store
    2 << 8 | 23, // llvm.matrix.column.major.load
    19 << 8 | 24, // llvm.matrix.column.major.store
    0 << 8 | 25, // llvm.matrix.multiply
    0 << 8 | 13, // llvm.matrix.transpose
    4 << 8 | 1, // llvm.maximum
    4 << 8 | 1, // llvm.maximumnum
    4 << 8 | 1, // llvm.maxnum
    25 << 8 | 26, // llvm.memcpy
    26 << 8 | 27, // llvm.memcpy.element.unordered.atomic
    25 << 8 | 26, // llvm.memcpy.inline
    25 << 8 | 27, // llvm.memmove
    26 << 8 | 27, // llvm.memmove.element.unordered.atomic
    16 << 8 | 15, // llvm.memset
    27 << 8 | 15, // llvm.memset.element.unordered.atomic
    16 << 8 | 15, // llvm.memset.inline
    4 << 8 | 1, // llvm.minimum
    4 << 8 | 1, // llvm.minimumnum
    4 << 8 | 1, // llvm.minnum
    0 << 8 | 1, // llvm.modf
    4 << 8 | 1, // llvm.nearbyint
    5 << 8 | 1, // llvm.objc.arc.annotation.bottomup.bbend
    5 << 8 | 1, // llvm.objc.arc.annotation.bottomup.bbstart
    5 << 8 | 1, // llvm.objc.arc.annotation.topdown.bbend
    5 << 8 | 1, // llvm.objc.arc.annotation.topdown.bbstart
    5 << 8 | 28, // llvm.objc.autorelease
    5 << 8 | 1, // llvm.objc.autoreleasePoolPop
    5 << 8 | 1, // llvm.objc.autoreleasePoolPush
    5 << 8 | 28, // llvm.objc.autoreleaseReturnValue
    5 << 8 | 1, // llvm.objc.claimAutoreleasedReturnValue
    3 << 8 | 1, // llvm.objc.clang.arc.noop.use
    5 << 8 | 1, // llvm.objc.clang.arc.use
    5 << 8 | 1, // llvm.objc.copyWeak
    5 << 8 | 1, // llvm.objc.destroyWeak
    5 << 8 | 1, // llvm.objc.initWeak
    5 << 8 | 1, // llvm.objc.loadWeak
    5 << 8 | 1, // llvm.objc.loadWeakRetained
    5 << 8 | 1, // llvm.objc.moveWeak
    5 << 8 | 1, // llvm.objc.release
    5 << 8 | 28, // llvm.objc.retain
    5 << 8 | 28, // llvm.objc.retain.autorelease
    5 << 8 | 28, // llvm.objc.retainAutorelease
    5 << 8 | 28, // llvm.objc.retainAutoreleaseReturnValue
    5 << 8 | 1, // llvm.objc.retainAutoreleasedReturnValue
    5 << 8 | 1, // llvm.objc.retainBlock
    5 << 8 | 1, // llvm.objc.retainedObject
    5 << 8 | 1, // llvm.objc.storeStrong
    5 << 8 | 1, // llvm.objc.storeWeak
    5 << 8 | 1, // llvm.objc.sync.enter
    5 << 8 | 1, // llvm.objc.sync.exit
    5 << 8 | 1, // llvm.objc.unretainedObject
    5 << 8 | 1, // llvm.objc.unretainedPointer
    5 << 8 | 1, // llvm.objc.unsafeClaimAutoreleasedReturnValue
    0 << 8 | 29, // llvm.objectsize
    7 << 8 | 1, // llvm.pcmarker
    4 << 8 | 1, // llvm.pow
    4 << 8 | 1, // llvm.powi
    28 << 8 | 30, // llvm.prefetch
    1 << 8 | 13, // llvm.preserve.array.access.index
    0 << 8 | 31, // llvm.preserve.static.offset
    1 << 8 | 13, // llvm.preserve.struct.access.index
    1 << 8 | 0, // llvm.preserve.union.access.index
    1 << 8 | 11, // llvm.protected.field.ptr
    3 << 8 | 1, // llvm.pseudoprobe
    3 << 8 | 1, // llvm.ptr.annotation
    10 << 8 | 0, // llvm.ptrauth.auth
    1 << 8 | 1, // llvm.ptrauth.blend
    10 << 8 | 32, // llvm.ptrauth.resign
    1 << 8 | 0, // llvm.ptrauth.sign
    1 << 8 | 1, // llvm.ptrauth.sign.generic
    1 << 8 | 0, // llvm.ptrauth.strip
    4 << 8 | 1, // llvm.ptrmask
    0 << 8 | 1, // llvm.public.type.test
    23 << 8 | 1, // llvm.read_register
    29 << 8 | 1, // llvm.read_volatile_register
    7 << 8 | 1, // llvm.readcyclecounter
    7 << 8 | 1, // llvm.readsteadycounter
    30 << 8 | 1, // llvm.reloc.none
    3 << 8 | 1, // llvm.reset.fpenv
    3 << 8 | 1, // llvm.reset.fpmode
    1 << 8 | 5, // llvm.returnaddress
    4 << 8 | 1, // llvm.rint
    4 << 8 | 1, // llvm.round
    4 << 8 | 1, // llvm.roundeven
    4 << 8 | 1, // llvm.sadd.sat
    4 << 8 | 1, // llvm.sadd.with.overflow
    4 << 8 | 33, // llvm.scmp
    1 << 8 | 11, // llvm.sdiv.fix
    1 << 8 | 11, // llvm.sdiv.fix.sat
    10 << 8 | 1, // llvm.seh.scope.begin
    10 << 8 | 1, // llvm.seh.scope.end
    31 << 8 | 1, // llvm.seh.try.begin
    31 << 8 | 1, // llvm.seh.try.end
    3 << 8 | 1, // llvm.set.fpenv
    3 << 8 | 1, // llvm.set.fpmode
    22 << 8 | 1, // llvm.set.loop.iterations
    3 << 8 | 1, // llvm.set.rounding
    3 << 8 | 1, // llvm.sideeffect
    4 << 8 | 1, // llvm.sin
    0 << 8 | 1, // llvm.sincos
    0 << 8 | 1, // llvm.sincospi
    0 << 8 | 1, // llvm.sinh
    4 << 8 | 1, // llvm.smax
    4 << 8 | 1, // llvm.smin
    0 << 8 | 11, // llvm.smul.fix
    0 << 8 | 11, // llvm.smul.fix.sat
    4 << 8 | 1, // llvm.smul.with.overflow
    1 << 8 | 1, // llvm.sponentry
    4 << 8 | 1, // llvm.sqrt
    1 << 8 | 28, // llvm.ssa.copy
    0 << 8 | 1, // llvm.sshl.sat
    4 << 8 | 1, // llvm.ssub.sat
    4 << 8 | 1, // llvm.ssub.with.overflow
    7 << 8 | 1, // llvm.stackaddress
    7 << 8 | 1, // llvm.stackguard
    7 << 8 | 1, // llvm.stackprotector
    7 << 8 | 1, // llvm.stackrestore
    7 << 8 | 1, // llvm.stacksave
    22 << 8 | 1, // llvm.start.loop.iterations
    1 << 8 | 1, // llvm.stepvector
    0 << 8 | 1, // llvm.strip.invariant.group
    5 << 8 | 1, // llvm.swift.async.context.addr
    0 << 8 | 1, // llvm.tan
    0 << 8 | 1, // llvm.tanh
    22 << 8 | 1, // llvm.test.set.loop.iterations
    22 << 8 | 1, // llvm.test.start.loop.iterations
    1 << 8 | 1, // llvm.thread.pointer
    0 << 8 | 34, // llvm.threadlocal.address
    32 << 8 | 1, // llvm.trap
    4 << 8 | 1, // llvm.trunc
    1 << 8 | 1, // llvm.type.checked.load
    1 << 8 | 1, // llvm.type.checked.load.relative
    0 << 8 | 1, // llvm.type.test
    4 << 8 | 1, // llvm.uadd.sat
    4 << 8 | 1, // llvm.uadd.with.overflow
    32 << 8 | 5, // llvm.ubsantrap
    4 << 8 | 33, // llvm.ucmp
    1 << 8 | 11, // llvm.udiv.fix
    1 << 8 | 11, // llvm.udiv.fix.sat
    4 << 8 | 1, // llvm.umax
    4 << 8 | 1, // llvm.umin
    0 << 8 | 11, // llvm.umul.fix
    0 << 8 | 11, // llvm.umul.fix.sat
    4 << 8 | 1, // llvm.umul.with.overflow
    0 << 8 | 1, // llvm.ushl.sat
    4 << 8 | 1, // llvm.usub.sat
    4 << 8 | 1, // llvm.usub.with.overflow
    7 << 8 | 1, // llvm.va_copy
    7 << 8 | 1, // llvm.va_end
    7 << 8 | 1, // llvm.va_start
    3 << 8 | 1, // llvm.var.annotation
    0 << 8 | 1, // llvm.vector.deinterleave2
    0 << 8 | 1, // llvm.vector.deinterleave3
    0 << 8 | 1, // llvm.vector.deinterleave4
    0 << 8 | 1, // llvm.vector.deinterleave5
    0 << 8 | 1, // llvm.vector.deinterleave6
    0 << 8 | 1, // llvm.vector.deinterleave7
    0 << 8 | 1, // llvm.vector.deinterleave8
    0 << 8 | 0, // llvm.vector.extract
    0 << 8 | 11, // llvm.vector.insert
    0 << 8 | 1, // llvm.vector.interleave2
    0 << 8 | 1, // llvm.vector.interleave3
    0 << 8 | 1, // llvm.vector.interleave4
    0 << 8 | 1, // llvm.vector.interleave5
    0 << 8 | 1, // llvm.vector.interleave6
    0 << 8 | 1, // llvm.vector.interleave7
    0 << 8 | 1, // llvm.vector.interleave8
    0 << 8 | 1, // llvm.vector.partial.reduce.add
    1 << 8 | 1, // llvm.vector.partial.reduce.fadd
    0 << 8 | 1, // llvm.vector.reduce.add
    0 << 8 | 1, // llvm.vector.reduce.and
    0 << 8 | 1, // llvm.vector.reduce.fadd
    0 << 8 | 1, // llvm.vector.reduce.fmax
    0 << 8 | 1, // llvm.vector.reduce.fmaximum
    0 << 8 | 1, // llvm.vector.reduce.fmin
    0 << 8 | 1, // llvm.vector.reduce.fminimum
    0 << 8 | 1, // llvm.vector.reduce.fmul
    0 << 8 | 1, // llvm.vector.reduce.mul
    0 << 8 | 1, // llvm.vector.reduce.or
    0 << 8 | 1, // llvm.vector.reduce.smax
    0 << 8 | 1, // llvm.vector.reduce.smin
    0 << 8 | 1, // llvm.vector.reduce.umax
    0 << 8 | 1, // llvm.vector.reduce.umin
    0 << 8 | 1, // llvm.vector.reduce.xor
    0 << 8 | 1, // llvm.vector.reverse
    0 << 8 | 11, // llvm.vector.splice.left
    0 << 8 | 11, // llvm.vector.splice.right
    0 << 8 | 1, // llvm.vp.abs
    0 << 8 | 1, // llvm.vp.add
    0 << 8 | 1, // llvm.vp.and
    0 << 8 | 1, // llvm.vp.ashr
    0 << 8 | 1, // llvm.vp.bitreverse
    0 << 8 | 1, // llvm.vp.bswap
    0 << 8 | 1, // llvm.vp.ceil
    0 << 8 | 1, // llvm.vp.copysign
    0 << 8 | 0, // llvm.vp.ctlz
    0 << 8 | 1, // llvm.vp.ctpop
    0 << 8 | 0, // llvm.vp.cttz
    0 << 8 | 0, // llvm.vp.cttz.elts
    0 << 8 | 1, // llvm.vp.fabs
    0 << 8 | 1, // llvm.vp.fadd
    0 << 8 | 1, // llvm.vp.fcmp
    0 << 8 | 1, // llvm.vp.fdiv
    0 << 8 | 1, // llvm.vp.floor
    0 << 8 | 1, // llvm.vp.fma
    0 << 8 | 1, // llvm.vp.fmul
    0 << 8 | 1, // llvm.vp.fmuladd
    0 << 8 | 1, // llvm.vp.fneg
    0 << 8 | 1, // llvm.vp.fpext
    0 << 8 | 1, // llvm.vp.fptosi
    0 << 8 | 1, // llvm.vp.fptoui
    0 << 8 | 1, // llvm.vp.fptrunc
    0 << 8 | 1, // llvm.vp.frem
    0 << 8 | 1, // llvm.vp.fshl
    0 << 8 | 1, // llvm.vp.fshr
    0 << 8 | 1, // llvm.vp.fsub
    23 << 8 | 1, // llvm.vp.gather
    0 << 8 | 1, // llvm.vp.icmp
    0 << 8 | 1, // llvm.vp.inttoptr
    0 << 8 | 0, // llvm.vp.is.fpclass
    0 << 8 | 1, // llvm.vp.llrint
    2 << 8 | 10, // llvm.vp.load
    2 << 8 | 10, // llvm.vp.load.ff
    0 << 8 | 1, // llvm.vp.lrint
    0 << 8 | 1, // llvm.vp.lshr
    0 << 8 | 1, // llvm.vp.maximum
    0 << 8 | 1, // llvm.vp.maxnum
    0 << 8 | 1, // llvm.vp.merge
    0 << 8 | 1, // llvm.vp.minimum
    0 << 8 | 1, // llvm.vp.minnum
    0 << 8 | 1, // llvm.vp.mul
    0 << 8 | 1, // llvm.vp.nearbyint
    0 << 8 | 1, // llvm.vp.or
    0 << 8 | 1, // llvm.vp.ptrtoint
    0 << 8 | 1, // llvm.vp.reduce.add
    0 << 8 | 1, // llvm.vp.reduce.and
    0 << 8 | 1, // llvm.vp.reduce.fadd
    0 << 8 | 1, // llvm.vp.reduce.fmax
    0 << 8 | 1, // llvm.vp.reduce.fmaximum
    0 << 8 | 1, // llvm.vp.reduce.fmin
    0 << 8 | 1, // llvm.vp.reduce.fminimum
    0 << 8 | 1, // llvm.vp.reduce.fmul
    0 << 8 | 1, // llvm.vp.reduce.mul
    0 << 8 | 1, // llvm.vp.reduce.or
    0 << 8 | 1, // llvm.vp.reduce.smax
    0 << 8 | 1, // llvm.vp.reduce.smin
    0 << 8 | 1, // llvm.vp.reduce.umax
    0 << 8 | 1, // llvm.vp.reduce.umin
    0 << 8 | 1, // llvm.vp.reduce.xor
    0 << 8 | 1, // llvm.vp.rint
    0 << 8 | 1, // llvm.vp.round
    0 << 8 | 1, // llvm.vp.roundeven
    0 << 8 | 1, // llvm.vp.roundtozero
    0 << 8 | 1, // llvm.vp.sadd.sat
    7 << 8 | 1, // llvm.vp.scatter
    1 << 8 | 1, // llvm.vp.sdiv
    0 << 8 | 1, // llvm.vp.select
    0 << 8 | 1, // llvm.vp.sext
    0 << 8 | 1, // llvm.vp.shl
    0 << 8 | 1, // llvm.vp.sitofp
    0 << 8 | 1, // llvm.vp.smax
    0 << 8 | 1, // llvm.vp.smin
    0 << 8 | 1, // llvm.vp.sqrt
    1 << 8 | 1, // llvm.vp.srem
    0 << 8 | 1, // llvm.vp.ssub.sat
    19 << 8 | 18, // llvm.vp.store
    0 << 8 | 1, // llvm.vp.sub
    0 << 8 | 1, // llvm.vp.trunc
    0 << 8 | 1, // llvm.vp.uadd.sat
    1 << 8 | 1, // llvm.vp.udiv
    0 << 8 | 1, // llvm.vp.uitofp
    0 << 8 | 1, // llvm.vp.umax
    0 << 8 | 1, // llvm.vp.umin
    1 << 8 | 1, // llvm.vp.urem
    0 << 8 | 1, // llvm.vp.usub.sat
    0 << 8 | 1, // llvm.vp.xor
    0 << 8 | 1, // llvm.vp.zext
    0 << 8 | 1, // llvm.vscale
    33 << 8 | 1, // llvm.write_register
    34 << 8 | 7, // llvm.xray.customevent
    34 << 8 | 8, // llvm.xray.typedevent
    1 << 8 | 1, // llvm.aarch64.addg
    35 << 8 | 5, // llvm.aarch64.break
    1 << 8 | 1, // llvm.aarch64.chkfeat
    5 << 8 | 1, // llvm.aarch64.clrex
    1 << 8 | 1, // llvm.aarch64.cls
    1 << 8 | 1, // llvm.aarch64.cls64
    1 << 8 | 1, // llvm.aarch64.crc32b
    1 << 8 | 1, // llvm.aarch64.crc32cb
    1 << 8 | 1, // llvm.aarch64.crc32ch
    1 << 8 | 1, // llvm.aarch64.crc32cw
    1 << 8 | 1, // llvm.aarch64.crc32cx
    1 << 8 | 1, // llvm.aarch64.crc32h
    1 << 8 | 1, // llvm.aarch64.crc32w
    1 << 8 | 1, // llvm.aarch64.crc32x
    1 << 8 | 1, // llvm.aarch64.crypto.aesd
    1 << 8 | 1, // llvm.aarch64.crypto.aese
    1 << 8 | 1, // llvm.aarch64.crypto.aesimc
    1 << 8 | 1, // llvm.aarch64.crypto.aesmc
    1 << 8 | 1, // llvm.aarch64.crypto.bcaxs
    1 << 8 | 1, // llvm.aarch64.crypto.bcaxu
    1 << 8 | 1, // llvm.aarch64.crypto.eor3s
    1 << 8 | 1, // llvm.aarch64.crypto.eor3u
    1 << 8 | 1, // llvm.aarch64.crypto.rax1
    1 << 8 | 1, // llvm.aarch64.crypto.sha1c
    1 << 8 | 1, // llvm.aarch64.crypto.sha1h
    1 << 8 | 1, // llvm.aarch64.crypto.sha1m
    1 << 8 | 1, // llvm.aarch64.crypto.sha1p
    1 << 8 | 1, // llvm.aarch64.crypto.sha1su0
    1 << 8 | 1, // llvm.aarch64.crypto.sha1su1
    1 << 8 | 1, // llvm.aarch64.crypto.sha256h
    1 << 8 | 1, // llvm.aarch64.crypto.sha256h2
    1 << 8 | 1, // llvm.aarch64.crypto.sha256su0
    1 << 8 | 1, // llvm.aarch64.crypto.sha256su1
    1 << 8 | 1, // llvm.aarch64.crypto.sha512h
    1 << 8 | 1, // llvm.aarch64.crypto.sha512h2
    1 << 8 | 1, // llvm.aarch64.crypto.sha512su0
    1 << 8 | 1, // llvm.aarch64.crypto.sha512su1
    10 << 8 | 1, // llvm.aarch64.crypto.sm3partw1
    10 << 8 | 1, // llvm.aarch64.crypto.sm3partw2
    10 << 8 | 1, // llvm.aarch64.crypto.sm3ss1
    10 << 8 | 35, // llvm.aarch64.crypto.sm3tt1a
    10 << 8 | 35, // llvm.aarch64.crypto.sm3tt1b
    10 << 8 | 35, // llvm.aarch64.crypto.sm3tt2a
    10 << 8 | 35, // llvm.aarch64.crypto.sm3tt2b
    10 << 8 | 1, // llvm.aarch64.crypto.sm4e
    10 << 8 | 1, // llvm.aarch64.crypto.sm4ekey
    1 << 8 | 11, // llvm.aarch64.crypto.xar
    36 << 8 | 1, // llvm.aarch64.dmb
    36 << 8 | 1, // llvm.aarch64.dsb
    1 << 8 | 1, // llvm.aarch64.fjcvtzs
    1 << 8 | 1, // llvm.aarch64.frint32x
    1 << 8 | 1, // llvm.aarch64.frint32z
    1 << 8 | 1, // llvm.aarch64.frint64x
    1 << 8 | 1, // llvm.aarch64.frint64z
    7 << 8 | 1, // llvm.aarch64.gcspopm
    7 << 8 | 1, // llvm.aarch64.gcsss
    30 << 8 | 1, // llvm.aarch64.get.fpcr
    30 << 8 | 1, // llvm.aarch64.get.fpsr
    1 << 8 | 1, // llvm.aarch64.gmi
    7 << 8 | 1, // llvm.aarch64.hint
    35 << 8 | 5, // llvm.aarch64.hlt
    30 << 8 | 1, // llvm.aarch64.irg
    30 << 8 | 1, // llvm.aarch64.irg.sp
    36 << 8 | 1, // llvm.aarch64.isb
    5 << 8 | 1, // llvm.aarch64.ld64b
    36 << 8 | 1, // llvm.aarch64.ldaxp
    36 << 8 | 1, // llvm.aarch64.ldaxr
    23 << 8 | 1, // llvm.aarch64.ldg
    36 << 8 | 1, // llvm.aarch64.ldxp
    36 << 8 | 1, // llvm.aarch64.ldxr
    19 << 8 | 36, // llvm.aarch64.mops.memset.tag
    1 << 8 | 1, // llvm.aarch64.neon.abs
    1 << 8 | 1, // llvm.aarch64.neon.addhn
    1 << 8 | 1, // llvm.aarch64.neon.addp
    1 << 8 | 1, // llvm.aarch64.neon.bfdot
    1 << 8 | 1, // llvm.aarch64.neon.bfmlalb
    1 << 8 | 1, // llvm.aarch64.neon.bfmlalt
    1 << 8 | 1, // llvm.aarch64.neon.bfmmla
    1 << 8 | 1, // llvm.aarch64.neon.cls
    1 << 8 | 1, // llvm.aarch64.neon.fabd
    1 << 8 | 1, // llvm.aarch64.neon.facge
    1 << 8 | 1, // llvm.aarch64.neon.facgt
    1 << 8 | 1, // llvm.aarch64.neon.faddp
    1 << 8 | 1, // llvm.aarch64.neon.faddv
    1 << 8 | 1, // llvm.aarch64.neon.famax
    1 << 8 | 1, // llvm.aarch64.neon.famin
    1 << 8 | 1, // llvm.aarch64.neon.fcvtas
    1 << 8 | 1, // llvm.aarch64.neon.fcvtau
    1 << 8 | 1, // llvm.aarch64.neon.fcvtms
    1 << 8 | 1, // llvm.aarch64.neon.fcvtmu
    1 << 8 | 1, // llvm.aarch64.neon.fcvtns
    1 << 8 | 1, // llvm.aarch64.neon.fcvtnu
    1 << 8 | 1, // llvm.aarch64.neon.fcvtps
    1 << 8 | 1, // llvm.aarch64.neon.fcvtpu
    1 << 8 | 1, // llvm.aarch64.neon.fcvtxn
    1 << 8 | 1, // llvm.aarch64.neon.fcvtzs
    1 << 8 | 1, // llvm.aarch64.neon.fcvtzu
    1 << 8 | 1, // llvm.aarch64.neon.fmax
    1 << 8 | 1, // llvm.aarch64.neon.fmaxnm
    1 << 8 | 1, // llvm.aarch64.neon.fmaxnmp
    1 << 8 | 1, // llvm.aarch64.neon.fmaxnmv
    1 << 8 | 1, // llvm.aarch64.neon.fmaxp
    1 << 8 | 1, // llvm.aarch64.neon.fmaxv
    1 << 8 | 1, // llvm.aarch64.neon.fmin
    1 << 8 | 1, // llvm.aarch64.neon.fminnm
    1 << 8 | 1, // llvm.aarch64.neon.fminnmp
    1 << 8 | 1, // llvm.aarch64.neon.fminnmv
    1 << 8 | 1, // llvm.aarch64.neon.fminp
    1 << 8 | 1, // llvm.aarch64.neon.fminv
    1 << 8 | 1, // llvm.aarch64.neon.fmlal
    1 << 8 | 1, // llvm.aarch64.neon.fmlal2
    1 << 8 | 1, // llvm.aarch64.neon.fmlsl
    1 << 8 | 1, // llvm.aarch64.neon.fmlsl2
    1 << 8 | 1, // llvm.aarch64.neon.fmmla
    1 << 8 | 1, // llvm.aarch64.neon.fmulx
    37 << 8 | 1, // llvm.aarch64.neon.fp8.cvtl1
    37 << 8 | 1, // llvm.aarch64.neon.fp8.cvtl2
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fcvtn
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fcvtn2
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fdot2
    37 << 8 | 35, // llvm.aarch64.neon.fp8.fdot2.lane
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fdot4
    37 << 8 | 35, // llvm.aarch64.neon.fp8.fdot4.lane
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fmlalb
    37 << 8 | 35, // llvm.aarch64.neon.fp8.fmlalb.lane
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fmlallbb
    37 << 8 | 35, // llvm.aarch64.neon.fp8.fmlallbb.lane
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fmlallbt
    37 << 8 | 35, // llvm.aarch64.neon.fp8.fmlallbt.lane
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fmlalltb
    37 << 8 | 35, // llvm.aarch64.neon.fp8.fmlalltb.lane
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fmlalltt
    37 << 8 | 35, // llvm.aarch64.neon.fp8.fmlalltt.lane
    37 << 8 | 1, // llvm.aarch64.neon.fp8.fmlalt
    37 << 8 | 35, // llvm.aarch64.neon.fp8.fmlalt.lane
    1 << 8 | 1, // llvm.aarch64.neon.fp8.fscale
    1 << 8 | 1, // llvm.aarch64.neon.frecpe
    1 << 8 | 1, // llvm.aarch64.neon.frecps
    1 << 8 | 1, // llvm.aarch64.neon.frecpx
    1 << 8 | 1, // llvm.aarch64.neon.frint32x
    1 << 8 | 1, // llvm.aarch64.neon.frint32z
    1 << 8 | 1, // llvm.aarch64.neon.frint64x
    1 << 8 | 1, // llvm.aarch64.neon.frint64z
    1 << 8 | 1, // llvm.aarch64.neon.frsqrte
    1 << 8 | 1, // llvm.aarch64.neon.frsqrts
    2 << 8 | 1, // llvm.aarch64.neon.ld1x2
    2 << 8 | 1, // llvm.aarch64.neon.ld1x3
    2 << 8 | 1, // llvm.aarch64.neon.ld1x4
    2 << 8 | 1, // llvm.aarch64.neon.ld2
    2 << 8 | 1, // llvm.aarch64.neon.ld2lane
    2 << 8 | 1, // llvm.aarch64.neon.ld2r
    2 << 8 | 1, // llvm.aarch64.neon.ld3
    2 << 8 | 1, // llvm.aarch64.neon.ld3lane
    2 << 8 | 1, // llvm.aarch64.neon.ld3r
    2 << 8 | 1, // llvm.aarch64.neon.ld4
    2 << 8 | 1, // llvm.aarch64.neon.ld4lane
    2 << 8 | 1, // llvm.aarch64.neon.ld4r
    1 << 8 | 1, // llvm.aarch64.neon.pmul
    1 << 8 | 1, // llvm.aarch64.neon.pmull
    1 << 8 | 1, // llvm.aarch64.neon.pmull64
    1 << 8 | 1, // llvm.aarch64.neon.raddhn
    1 << 8 | 0, // llvm.aarch64.neon.rshrn
    1 << 8 | 1, // llvm.aarch64.neon.rsubhn
    1 << 8 | 1, // llvm.aarch64.neon.sabd
    1 << 8 | 1, // llvm.aarch64.neon.saddlp
    1 << 8 | 1, // llvm.aarch64.neon.saddlv
    1 << 8 | 1, // llvm.aarch64.neon.saddv
    1 << 8 | 1, // llvm.aarch64.neon.scalar.sqxtn
    1 << 8 | 1, // llvm.aarch64.neon.scalar.sqxtun
    1 << 8 | 1, // llvm.aarch64.neon.scalar.uqxtn
    1 << 8 | 1, // llvm.aarch64.neon.sdot
    1 << 8 | 1, // llvm.aarch64.neon.shadd
    1 << 8 | 1, // llvm.aarch64.neon.shll
    1 << 8 | 1, // llvm.aarch64.neon.shsub
    1 << 8 | 1, // llvm.aarch64.neon.smax
    1 << 8 | 1, // llvm.aarch64.neon.smaxp
    1 << 8 | 1, // llvm.aarch64.neon.smaxv
    1 << 8 | 1, // llvm.aarch64.neon.smin
    1 << 8 | 1, // llvm.aarch64.neon.sminp
    1 << 8 | 1, // llvm.aarch64.neon.sminv
    1 << 8 | 1, // llvm.aarch64.neon.smmla
    1 << 8 | 1, // llvm.aarch64.neon.smull
    1 << 8 | 1, // llvm.aarch64.neon.sqabs
    1 << 8 | 1, // llvm.aarch64.neon.sqadd
    1 << 8 | 1, // llvm.aarch64.neon.sqdmulh
    1 << 8 | 1, // llvm.aarch64.neon.sqdmulh.lane
    1 << 8 | 1, // llvm.aarch64.neon.sqdmulh.laneq
    1 << 8 | 1, // llvm.aarch64.neon.sqdmull
    1 << 8 | 1, // llvm.aarch64.neon.sqdmulls.scalar
    1 << 8 | 1, // llvm.aarch64.neon.sqneg
    1 << 8 | 1, // llvm.aarch64.neon.sqrdmlah
    1 << 8 | 1, // llvm.aarch64.neon.sqrdmlsh
    1 << 8 | 1, // llvm.aarch64.neon.sqrdmulh
    1 << 8 | 1, // llvm.aarch64.neon.sqrdmulh.lane
    1 << 8 | 1, // llvm.aarch64.neon.sqrdmulh.laneq
    1 << 8 | 1, // llvm.aarch64.neon.sqrshl
    1 << 8 | 0, // llvm.aarch64.neon.sqrshrn
    1 << 8 | 0, // llvm.aarch64.neon.sqrshrun
    1 << 8 | 1, // llvm.aarch64.neon.sqshl
    1 << 8 | 1, // llvm.aarch64.neon.sqshlu
    1 << 8 | 0, // llvm.aarch64.neon.sqshrn
    1 << 8 | 0, // llvm.aarch64.neon.sqshrun
    1 << 8 | 1, // llvm.aarch64.neon.sqsub
    1 << 8 | 1, // llvm.aarch64.neon.sqxtn
    1 << 8 | 1, // llvm.aarch64.neon.sqxtun
    1 << 8 | 1, // llvm.aarch64.neon.srhadd
    1 << 8 | 1, // llvm.aarch64.neon.srshl
    1 << 8 | 1, // llvm.aarch64.neon.sshl
    1 << 8 | 1, // llvm.aarch64.neon.sshll
    19 << 8 | 37, // llvm.aarch64.neon.st1x2
    19 << 8 | 38, // llvm.aarch64.neon.st1x3
    19 << 8 | 39, // llvm.aarch64.neon.st1x4
    19 << 8 | 37, // llvm.aarch64.neon.st2
    19 << 8 | 38, // llvm.aarch64.neon.st2lane
    19 << 8 | 38, // llvm.aarch64.neon.st3
    19 << 8 | 39, // llvm.aarch64.neon.st3lane
    19 << 8 | 39, // llvm.aarch64.neon.st4
    19 << 8 | 40, // llvm.aarch64.neon.st4lane
    1 << 8 | 1, // llvm.aarch64.neon.subhn
    1 << 8 | 1, // llvm.aarch64.neon.suqadd
    1 << 8 | 1, // llvm.aarch64.neon.tbl1
    1 << 8 | 1, // llvm.aarch64.neon.tbl2
    1 << 8 | 1, // llvm.aarch64.neon.tbl3
    1 << 8 | 1, // llvm.aarch64.neon.tbl4
    1 << 8 | 1, // llvm.aarch64.neon.tbx1
    1 << 8 | 1, // llvm.aarch64.neon.tbx2
    1 << 8 | 1, // llvm.aarch64.neon.tbx3
    1 << 8 | 1, // llvm.aarch64.neon.tbx4
    1 << 8 | 1, // llvm.aarch64.neon.uabd
    1 << 8 | 1, // llvm.aarch64.neon.uaddlp
    1 << 8 | 1, // llvm.aarch64.neon.uaddlv
    1 << 8 | 1, // llvm.aarch64.neon.uaddv
    1 << 8 | 1, // llvm.aarch64.neon.udot
    1 << 8 | 1, // llvm.aarch64.neon.uhadd
    1 << 8 | 1, // llvm.aarch64.neon.uhsub
    1 << 8 | 1, // llvm.aarch64.neon.umax
    1 << 8 | 1, // llvm.aarch64.neon.umaxp
    1 << 8 | 1, // llvm.aarch64.neon.umaxv
    1 << 8 | 1, // llvm.aarch64.neon.umin
    1 << 8 | 1, // llvm.aarch64.neon.uminp
    1 << 8 | 1, // llvm.aarch64.neon.uminv
    1 << 8 | 1, // llvm.aarch64.neon.ummla
    1 << 8 | 1, // llvm.aarch64.neon.umull
    1 << 8 | 1, // llvm.aarch64.neon.uqadd
    1 << 8 | 1, // llvm.aarch64.neon.uqrshl
    1 << 8 | 0, // llvm.aarch64.neon.uqrshrn
    1 << 8 | 1, // llvm.aarch64.neon.uqshl
    1 << 8 | 0, // llvm.aarch64.neon.uqshrn
    1 << 8 | 1, // llvm.aarch64.neon.uqsub
    1 << 8 | 1, // llvm.aarch64.neon.uqxtn
    1 << 8 | 1, // llvm.aarch64.neon.urecpe
    1 << 8 | 1, // llvm.aarch64.neon.urhadd
    1 << 8 | 1, // llvm.aarch64.neon.urshl
    1 << 8 | 1, // llvm.aarch64.neon.ursqrte
    1 << 8 | 1, // llvm.aarch64.neon.usdot
    1 << 8 | 1, // llvm.aarch64.neon.ushl
    1 << 8 | 1, // llvm.aarch64.neon.ushll
    1 << 8 | 1, // llvm.aarch64.neon.usmmla
    1 << 8 | 1, // llvm.aarch64.neon.usqadd
    1 << 8 | 1, // llvm.aarch64.neon.vcadd.rot270
    1 << 8 | 1, // llvm.aarch64.neon.vcadd.rot90
    1 << 8 | 1, // llvm.aarch64.neon.vcmla.rot0
    1 << 8 | 1, // llvm.aarch64.neon.vcmla.rot180
    1 << 8 | 1, // llvm.aarch64.neon.vcmla.rot270
    1 << 8 | 1, // llvm.aarch64.neon.vcmla.rot90
    1 << 8 | 1, // llvm.aarch64.neon.vcopy.lane
    1 << 8 | 0, // llvm.aarch64.neon.vcvtfp2fxs
    1 << 8 | 0, // llvm.aarch64.neon.vcvtfp2fxu
    1 << 8 | 1, // llvm.aarch64.neon.vcvtfp2hf
    1 << 8 | 0, // llvm.aarch64.neon.vcvtfxs2fp
    1 << 8 | 0, // llvm.aarch64.neon.vcvtfxu2fp
    1 << 8 | 1, // llvm.aarch64.neon.vcvthf2fp
    1 << 8 | 11, // llvm.aarch64.neon.vluti2.lane
    1 << 8 | 11, // llvm.aarch64.neon.vluti2.laneq
    1 << 8 | 11, // llvm.aarch64.neon.vluti4q.lane
    1 << 8 | 35, // llvm.aarch64.neon.vluti4q.lane.x2
    1 << 8 | 11, // llvm.aarch64.neon.vluti4q.laneq
    1 << 8 | 35, // llvm.aarch64.neon.vluti4q.laneq.x2
    1 << 8 | 11, // llvm.aarch64.neon.vsli
    1 << 8 | 11, // llvm.aarch64.neon.vsri
    38 << 8 | 41, // llvm.aarch64.prefetch
    38 << 8 | 42, // llvm.aarch64.range.prefetch
    30 << 8 | 1, // llvm.aarch64.rndr
    30 << 8 | 1, // llvm.aarch64.rndrrs
    1 << 8 | 1, // llvm.aarch64.sdiv
    30 << 8 | 1, // llvm.aarch64.set.fpcr
    6 << 8 | 1, // llvm.aarch64.set.fpmr
    30 << 8 | 1, // llvm.aarch64.set.fpsr
    19 << 8 | 36, // llvm.aarch64.settag
    19 << 8 | 36, // llvm.aarch64.settag.zero
    1 << 8 | 1, // llvm.aarch64.sisd.fabd
    1 << 8 | 1, // llvm.aarch64.sisd.fcvtxn
    6 << 8 | 1, // llvm.aarch64.sme.add.write.single.za.vg1x2
    6 << 8 | 1, // llvm.aarch64.sme.add.write.single.za.vg1x4
    6 << 8 | 1, // llvm.aarch64.sme.add.write.za.vg1x2
    6 << 8 | 1, // llvm.aarch64.sme.add.write.za.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.add.za16.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.add.za16.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.add.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.add.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.add.za64.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.add.za64.vg1x4
    3 << 8 | 5, // llvm.aarch64.sme.addha
    3 << 8 | 5, // llvm.aarch64.sme.addva
    3 << 8 | 5, // llvm.aarch64.sme.bmopa.za32
    3 << 8 | 5, // llvm.aarch64.sme.bmops.za32
    1 << 8 | 1, // llvm.aarch64.sme.cntsd
    1 << 8 | 1, // llvm.aarch64.sme.famax.x2
    1 << 8 | 1, // llvm.aarch64.sme.famax.x4
    1 << 8 | 1, // llvm.aarch64.sme.famin.x2
    1 << 8 | 1, // llvm.aarch64.sme.famin.x4
    3 << 8 | 43, // llvm.aarch64.sme.fdot.lane.za32.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.fdot.lane.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fdot.single.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fdot.single.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fdot.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fdot.za32.vg1x4
    3 << 8 | 43, // llvm.aarch64.sme.fmla.lane.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.fmla.lane.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fmla.single.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fmla.single.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fmla.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fmla.vg1x4
    3 << 8 | 35, // llvm.aarch64.sme.fmlal.lane.vg2x1
    3 << 8 | 43, // llvm.aarch64.sme.fmlal.lane.vg2x2
    3 << 8 | 44, // llvm.aarch64.sme.fmlal.lane.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.fmlal.single.vg2x1
    3 << 8 | 1, // llvm.aarch64.sme.fmlal.single.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.fmlal.single.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.fmlal.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.fmlal.vg2x4
    3 << 8 | 43, // llvm.aarch64.sme.fmls.lane.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.fmls.lane.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fmls.single.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fmls.single.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fmls.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fmls.vg1x4
    3 << 8 | 35, // llvm.aarch64.sme.fmlsl.lane.vg2x1
    3 << 8 | 43, // llvm.aarch64.sme.fmlsl.lane.vg2x2
    3 << 8 | 44, // llvm.aarch64.sme.fmlsl.lane.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.fmlsl.single.vg2x1
    3 << 8 | 1, // llvm.aarch64.sme.fmlsl.single.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.fmlsl.single.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.fmlsl.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.fmlsl.vg2x4
    3 << 8 | 43, // llvm.aarch64.sme.fp8.fdot.lane.za16.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.fp8.fdot.lane.za16.vg1x4
    3 << 8 | 43, // llvm.aarch64.sme.fp8.fdot.lane.za32.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.fp8.fdot.lane.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fdot.multi.za16.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fdot.multi.za16.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fdot.multi.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fdot.multi.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fdot.single.za16.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fdot.single.za16.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fdot.single.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fdot.single.za32.vg1x4
    3 << 8 | 35, // llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x1
    3 << 8 | 43, // llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x2
    3 << 8 | 44, // llvm.aarch64.sme.fp8.fmlal.lane.za16.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlal.multi.za16.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlal.multi.za16.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlal.single.za16.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlal.single.za16.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlal.za16.vg2x1
    3 << 8 | 35, // llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.fp8.fmlall.lane.za32.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlall.multi.za32.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlall.multi.za32.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlall.single.za32.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlall.single.za32.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.fp8.fmlall.za32.vg4x1
    21 << 8 | 5, // llvm.aarch64.sme.fp8.fmop4a.za16.1x1
    21 << 8 | 5, // llvm.aarch64.sme.fp8.fmop4a.za16.1x2
    21 << 8 | 5, // llvm.aarch64.sme.fp8.fmop4a.za16.2x1
    21 << 8 | 5, // llvm.aarch64.sme.fp8.fmop4a.za16.2x2
    21 << 8 | 5, // llvm.aarch64.sme.fp8.fmop4a.za32.1x1
    21 << 8 | 5, // llvm.aarch64.sme.fp8.fmop4a.za32.1x2
    21 << 8 | 5, // llvm.aarch64.sme.fp8.fmop4a.za32.2x1
    21 << 8 | 5, // llvm.aarch64.sme.fp8.fmop4a.za32.2x2
    3 << 8 | 5, // llvm.aarch64.sme.fp8.fmopa.za16
    3 << 8 | 5, // llvm.aarch64.sme.fp8.fmopa.za32
    3 << 8 | 43, // llvm.aarch64.sme.fp8.fvdot.lane.za16.vg1x2
    3 << 8 | 43, // llvm.aarch64.sme.fp8.fvdotb.lane.za32.vg1x4
    3 << 8 | 43, // llvm.aarch64.sme.fp8.fvdott.lane.za32.vg1x4
    1 << 8 | 1, // llvm.aarch64.sme.fp8.scale.single.x2
    1 << 8 | 1, // llvm.aarch64.sme.fp8.scale.single.x4
    1 << 8 | 1, // llvm.aarch64.sme.fp8.scale.x2
    1 << 8 | 1, // llvm.aarch64.sme.fp8.scale.x4
    3 << 8 | 45, // llvm.aarch64.sme.ftmopa.za16
    3 << 8 | 45, // llvm.aarch64.sme.ftmopa.za32
    3 << 8 | 43, // llvm.aarch64.sme.fvdot.lane.za32.vg1x2
    30 << 8 | 1, // llvm.aarch64.sme.get.tpidr2
    1 << 8 | 1, // llvm.aarch64.sme.in.streaming.mode
    28 << 8 | 11, // llvm.aarch64.sme.ld1b.horiz
    28 << 8 | 11, // llvm.aarch64.sme.ld1b.vert
    28 << 8 | 11, // llvm.aarch64.sme.ld1d.horiz
    28 << 8 | 11, // llvm.aarch64.sme.ld1d.vert
    28 << 8 | 11, // llvm.aarch64.sme.ld1h.horiz
    28 << 8 | 11, // llvm.aarch64.sme.ld1h.vert
    28 << 8 | 11, // llvm.aarch64.sme.ld1q.horiz
    28 << 8 | 11, // llvm.aarch64.sme.ld1q.vert
    28 << 8 | 11, // llvm.aarch64.sme.ld1w.horiz
    28 << 8 | 11, // llvm.aarch64.sme.ld1w.vert
    28 << 8 | 1, // llvm.aarch64.sme.ldr
    28 << 8 | 1, // llvm.aarch64.sme.ldr.zt
    37 << 8 | 46, // llvm.aarch64.sme.luti2.lane.zt
    37 << 8 | 46, // llvm.aarch64.sme.luti2.lane.zt.x2
    37 << 8 | 46, // llvm.aarch64.sme.luti2.lane.zt.x4
    37 << 8 | 46, // llvm.aarch64.sme.luti4.lane.zt
    37 << 8 | 46, // llvm.aarch64.sme.luti4.lane.zt.x2
    37 << 8 | 46, // llvm.aarch64.sme.luti4.lane.zt.x4
    37 << 8 | 5, // llvm.aarch64.sme.luti4.zt.x4
    21 << 8 | 5, // llvm.aarch64.sme.mop4a.1x1
    21 << 8 | 5, // llvm.aarch64.sme.mop4a.1x2
    21 << 8 | 5, // llvm.aarch64.sme.mop4a.2x1
    21 << 8 | 5, // llvm.aarch64.sme.mop4a.2x2
    21 << 8 | 5, // llvm.aarch64.sme.mop4a.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.mop4a.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.mop4a.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.mop4a.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.mop4s.1x1
    21 << 8 | 5, // llvm.aarch64.sme.mop4s.1x2
    21 << 8 | 5, // llvm.aarch64.sme.mop4s.2x1
    21 << 8 | 5, // llvm.aarch64.sme.mop4s.2x2
    21 << 8 | 5, // llvm.aarch64.sme.mop4s.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.mop4s.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.mop4s.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.mop4s.wide.2x2
    3 << 8 | 5, // llvm.aarch64.sme.mopa
    3 << 8 | 5, // llvm.aarch64.sme.mopa.nonwide
    3 << 8 | 5, // llvm.aarch64.sme.mopa.wide
    3 << 8 | 5, // llvm.aarch64.sme.mops
    3 << 8 | 5, // llvm.aarch64.sme.mops.nonwide
    3 << 8 | 5, // llvm.aarch64.sme.mops.wide
    37 << 8 | 1, // llvm.aarch64.sme.read.hor.vg2
    37 << 8 | 1, // llvm.aarch64.sme.read.hor.vg4
    37 << 8 | 11, // llvm.aarch64.sme.read.horiz
    37 << 8 | 1, // llvm.aarch64.sme.read.ver.vg2
    37 << 8 | 1, // llvm.aarch64.sme.read.ver.vg4
    37 << 8 | 11, // llvm.aarch64.sme.read.vert
    37 << 8 | 1, // llvm.aarch64.sme.read.vg1x2
    37 << 8 | 1, // llvm.aarch64.sme.read.vg1x4
    37 << 8 | 11, // llvm.aarch64.sme.readq.horiz
    37 << 8 | 11, // llvm.aarch64.sme.readq.vert
    3 << 8 | 5, // llvm.aarch64.sme.readz.horiz
    3 << 8 | 5, // llvm.aarch64.sme.readz.horiz.x2
    3 << 8 | 5, // llvm.aarch64.sme.readz.horiz.x4
    3 << 8 | 5, // llvm.aarch64.sme.readz.q.horiz
    3 << 8 | 5, // llvm.aarch64.sme.readz.q.vert
    3 << 8 | 5, // llvm.aarch64.sme.readz.vert
    3 << 8 | 5, // llvm.aarch64.sme.readz.vert.x2
    3 << 8 | 5, // llvm.aarch64.sme.readz.vert.x4
    3 << 8 | 1, // llvm.aarch64.sme.readz.x2
    3 << 8 | 1, // llvm.aarch64.sme.readz.x4
    3 << 8 | 43, // llvm.aarch64.sme.sdot.lane.za32.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.sdot.lane.za32.vg1x4
    3 << 8 | 43, // llvm.aarch64.sme.sdot.lane.za64.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.sdot.lane.za64.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.sdot.single.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.sdot.single.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.sdot.single.za64.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.sdot.single.za64.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.sdot.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.sdot.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.sdot.za64.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.sdot.za64.vg1x4
    30 << 8 | 1, // llvm.aarch64.sme.set.tpidr2
    3 << 8 | 35, // llvm.aarch64.sme.smla.za32.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.smla.za32.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.smla.za32.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.smla.za32.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.smla.za32.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.smla.za32.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.smla.za32.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.smla.za32.vg4x4
    3 << 8 | 35, // llvm.aarch64.sme.smla.za64.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.smla.za64.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.smla.za64.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.smla.za64.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.smla.za64.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.smla.za64.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.smla.za64.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.smla.za64.vg4x4
    3 << 8 | 35, // llvm.aarch64.sme.smlal.lane.vg2x1
    3 << 8 | 43, // llvm.aarch64.sme.smlal.lane.vg2x2
    3 << 8 | 44, // llvm.aarch64.sme.smlal.lane.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.smlal.single.vg2x1
    3 << 8 | 1, // llvm.aarch64.sme.smlal.single.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.smlal.single.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.smlal.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.smlal.vg2x4
    3 << 8 | 35, // llvm.aarch64.sme.smls.za32.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.smls.za32.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.smls.za32.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.smls.za32.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.smls.za32.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.smls.za32.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.smls.za32.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.smls.za32.vg4x4
    3 << 8 | 35, // llvm.aarch64.sme.smls.za64.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.smls.za64.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.smls.za64.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.smls.za64.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.smls.za64.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.smls.za64.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.smls.za64.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.smls.za64.vg4x4
    3 << 8 | 35, // llvm.aarch64.sme.smlsl.lane.vg2x1
    3 << 8 | 43, // llvm.aarch64.sme.smlsl.lane.vg2x2
    3 << 8 | 44, // llvm.aarch64.sme.smlsl.lane.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.smlsl.single.vg2x1
    3 << 8 | 1, // llvm.aarch64.sme.smlsl.single.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.smlsl.single.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.smlsl.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.smlsl.vg2x4
    21 << 8 | 5, // llvm.aarch64.sme.smop4a.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.smop4a.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.smop4a.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.smop4a.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.smop4a.za64.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.smop4a.za64.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.smop4a.za64.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.smop4a.za64.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.smop4s.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.smop4s.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.smop4s.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.smop4s.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.smop4s.za64.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.smop4s.za64.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.smop4s.za64.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.smop4s.za64.wide.2x2
    3 << 8 | 5, // llvm.aarch64.sme.smopa.wide
    3 << 8 | 5, // llvm.aarch64.sme.smopa.za32
    3 << 8 | 5, // llvm.aarch64.sme.smops.wide
    3 << 8 | 5, // llvm.aarch64.sme.smops.za32
    28 << 8 | 11, // llvm.aarch64.sme.st1b.horiz
    28 << 8 | 11, // llvm.aarch64.sme.st1b.vert
    28 << 8 | 11, // llvm.aarch64.sme.st1d.horiz
    28 << 8 | 11, // llvm.aarch64.sme.st1d.vert
    28 << 8 | 11, // llvm.aarch64.sme.st1h.horiz
    28 << 8 | 11, // llvm.aarch64.sme.st1h.vert
    28 << 8 | 11, // llvm.aarch64.sme.st1q.horiz
    28 << 8 | 11, // llvm.aarch64.sme.st1q.vert
    28 << 8 | 11, // llvm.aarch64.sme.st1w.horiz
    28 << 8 | 11, // llvm.aarch64.sme.st1w.vert
    3 << 8 | 45, // llvm.aarch64.sme.stmopa.za32
    28 << 8 | 1, // llvm.aarch64.sme.str
    28 << 8 | 1, // llvm.aarch64.sme.str.zt
    6 << 8 | 1, // llvm.aarch64.sme.sub.write.single.za.vg1x2
    6 << 8 | 1, // llvm.aarch64.sme.sub.write.single.za.vg1x4
    6 << 8 | 1, // llvm.aarch64.sme.sub.write.za.vg1x2
    6 << 8 | 1, // llvm.aarch64.sme.sub.write.za.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.sub.za16.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.sub.za16.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.sub.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.sub.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.sub.za64.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.sub.za64.vg1x4
    3 << 8 | 43, // llvm.aarch64.sme.sudot.lane.za32.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.sudot.lane.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.sudot.single.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.sudot.single.za32.vg1x4
    3 << 8 | 35, // llvm.aarch64.sme.sumla.za32.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.sumla.za32.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.sumla.za32.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.sumla.za32.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.sumla.za32.single.vg4x4
    21 << 8 | 5, // llvm.aarch64.sme.sumop4a.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.sumop4a.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.sumop4a.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.sumop4a.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.sumop4a.za64.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.sumop4a.za64.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.sumop4a.za64.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.sumop4a.za64.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.sumop4s.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.sumop4s.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.sumop4s.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.sumop4s.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.sumop4s.za64.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.sumop4s.za64.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.sumop4s.za64.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.sumop4s.za64.wide.2x2
    3 << 8 | 5, // llvm.aarch64.sme.sumopa.wide
    3 << 8 | 5, // llvm.aarch64.sme.sumops.wide
    3 << 8 | 45, // llvm.aarch64.sme.sutmopa.za32
    3 << 8 | 44, // llvm.aarch64.sme.suvdot.lane.za32.vg1x4
    3 << 8 | 43, // llvm.aarch64.sme.svdot.lane.za32.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.svdot.lane.za32.vg1x4
    3 << 8 | 44, // llvm.aarch64.sme.svdot.lane.za64.vg1x4
    3 << 8 | 43, // llvm.aarch64.sme.udot.lane.za32.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.udot.lane.za32.vg1x4
    3 << 8 | 43, // llvm.aarch64.sme.udot.lane.za64.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.udot.lane.za64.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.udot.single.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.udot.single.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.udot.single.za64.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.udot.single.za64.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.udot.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.udot.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.udot.za64.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.udot.za64.vg1x4
    3 << 8 | 35, // llvm.aarch64.sme.umla.za32.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.umla.za32.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.umla.za32.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.umla.za32.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.umla.za32.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.umla.za32.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.umla.za32.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.umla.za32.vg4x4
    3 << 8 | 35, // llvm.aarch64.sme.umla.za64.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.umla.za64.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.umla.za64.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.umla.za64.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.umla.za64.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.umla.za64.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.umla.za64.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.umla.za64.vg4x4
    3 << 8 | 35, // llvm.aarch64.sme.umlal.lane.vg2x1
    3 << 8 | 43, // llvm.aarch64.sme.umlal.lane.vg2x2
    3 << 8 | 44, // llvm.aarch64.sme.umlal.lane.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.umlal.single.vg2x1
    3 << 8 | 1, // llvm.aarch64.sme.umlal.single.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.umlal.single.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.umlal.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.umlal.vg2x4
    3 << 8 | 35, // llvm.aarch64.sme.umls.za32.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.umls.za32.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.umls.za32.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.umls.za32.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.umls.za32.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.umls.za32.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.umls.za32.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.umls.za32.vg4x4
    3 << 8 | 35, // llvm.aarch64.sme.umls.za64.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.umls.za64.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.umls.za64.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.umls.za64.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.umls.za64.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.umls.za64.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.umls.za64.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.umls.za64.vg4x4
    3 << 8 | 35, // llvm.aarch64.sme.umlsl.lane.vg2x1
    3 << 8 | 43, // llvm.aarch64.sme.umlsl.lane.vg2x2
    3 << 8 | 44, // llvm.aarch64.sme.umlsl.lane.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.umlsl.single.vg2x1
    3 << 8 | 1, // llvm.aarch64.sme.umlsl.single.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.umlsl.single.vg2x4
    3 << 8 | 1, // llvm.aarch64.sme.umlsl.vg2x2
    3 << 8 | 1, // llvm.aarch64.sme.umlsl.vg2x4
    21 << 8 | 5, // llvm.aarch64.sme.umop4a.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.umop4a.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.umop4a.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.umop4a.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.umop4a.za64.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.umop4a.za64.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.umop4a.za64.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.umop4a.za64.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.umop4s.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.umop4s.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.umop4s.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.umop4s.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.umop4s.za64.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.umop4s.za64.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.umop4s.za64.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.umop4s.za64.wide.2x2
    3 << 8 | 5, // llvm.aarch64.sme.umopa.wide
    3 << 8 | 5, // llvm.aarch64.sme.umopa.za32
    3 << 8 | 5, // llvm.aarch64.sme.umops.wide
    3 << 8 | 5, // llvm.aarch64.sme.umops.za32
    3 << 8 | 43, // llvm.aarch64.sme.usdot.lane.za32.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.usdot.lane.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.usdot.single.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.usdot.single.za32.vg1x4
    3 << 8 | 1, // llvm.aarch64.sme.usdot.za32.vg1x2
    3 << 8 | 1, // llvm.aarch64.sme.usdot.za32.vg1x4
    3 << 8 | 35, // llvm.aarch64.sme.usmla.za32.lane.vg4x1
    3 << 8 | 43, // llvm.aarch64.sme.usmla.za32.lane.vg4x2
    3 << 8 | 44, // llvm.aarch64.sme.usmla.za32.lane.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.usmla.za32.single.vg4x1
    3 << 8 | 1, // llvm.aarch64.sme.usmla.za32.single.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.usmla.za32.single.vg4x4
    3 << 8 | 1, // llvm.aarch64.sme.usmla.za32.vg4x2
    3 << 8 | 1, // llvm.aarch64.sme.usmla.za32.vg4x4
    21 << 8 | 5, // llvm.aarch64.sme.usmop4a.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.usmop4a.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.usmop4a.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.usmop4a.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.usmop4a.za64.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.usmop4a.za64.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.usmop4a.za64.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.usmop4a.za64.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.usmop4s.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.usmop4s.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.usmop4s.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.usmop4s.wide.2x2
    21 << 8 | 5, // llvm.aarch64.sme.usmop4s.za64.wide.1x1
    21 << 8 | 5, // llvm.aarch64.sme.usmop4s.za64.wide.1x2
    21 << 8 | 5, // llvm.aarch64.sme.usmop4s.za64.wide.2x1
    21 << 8 | 5, // llvm.aarch64.sme.usmop4s.za64.wide.2x2
    3 << 8 | 5, // llvm.aarch64.sme.usmopa.wide
    3 << 8 | 5, // llvm.aarch64.sme.usmops.wide
    3 << 8 | 45, // llvm.aarch64.sme.ustmopa.za32
    3 << 8 | 44, // llvm.aarch64.sme.usvdot.lane.za32.vg1x4
    3 << 8 | 45, // llvm.aarch64.sme.utmopa.za32
    3 << 8 | 43, // llvm.aarch64.sme.uvdot.lane.za32.vg1x2
    3 << 8 | 44, // llvm.aarch64.sme.uvdot.lane.za32.vg1x4
    3 << 8 | 44, // llvm.aarch64.sme.uvdot.lane.za64.vg1x4
    6 << 8 | 5, // llvm.aarch64.sme.write.hor.vg2
    6 << 8 | 5, // llvm.aarch64.sme.write.hor.vg4
    3 << 8 | 5, // llvm.aarch64.sme.write.horiz
    3 << 8 | 46, // llvm.aarch64.sme.write.lane.zt
    6 << 8 | 5, // llvm.aarch64.sme.write.ver.vg2
    6 << 8 | 5, // llvm.aarch64.sme.write.ver.vg4
    3 << 8 | 5, // llvm.aarch64.sme.write.vert
    6 << 8 | 1, // llvm.aarch64.sme.write.vg1x2
    6 << 8 | 1, // llvm.aarch64.sme.write.vg1x4
    6 << 8 | 5, // llvm.aarch64.sme.write.zt
    3 << 8 | 5, // llvm.aarch64.sme.writeq.horiz
    3 << 8 | 5, // llvm.aarch64.sme.writeq.vert
    6 << 8 | 1, // llvm.aarch64.sme.za.disable
    6 << 8 | 1, // llvm.aarch64.sme.za.enable
    6 << 8 | 5, // llvm.aarch64.sme.zero
    6 << 8 | 1, // llvm.aarch64.sme.zero.za64.vg1x2
    6 << 8 | 1, // llvm.aarch64.sme.zero.za64.vg1x4
    6 << 8 | 1, // llvm.aarch64.sme.zero.za64.vg2x1
    6 << 8 | 1, // llvm.aarch64.sme.zero.za64.vg2x2
    6 << 8 | 1, // llvm.aarch64.sme.zero.za64.vg2x4
    6 << 8 | 1, // llvm.aarch64.sme.zero.za64.vg4x1
    6 << 8 | 1, // llvm.aarch64.sme.zero.za64.vg4x2
    6 << 8 | 1, // llvm.aarch64.sme.zero.za64.vg4x4
    6 << 8 | 5, // llvm.aarch64.sme.zero.zt
    7 << 8 | 1, // llvm.aarch64.space
    5 << 8 | 1, // llvm.aarch64.st64b
    5 << 8 | 1, // llvm.aarch64.st64bv
    5 << 8 | 1, // llvm.aarch64.st64bv0
    24 << 8 | 1, // llvm.aarch64.stg
    19 << 8 | 36, // llvm.aarch64.stgp
    36 << 8 | 1, // llvm.aarch64.stlxp
    36 << 8 | 1, // llvm.aarch64.stlxr
    36 << 8 | 1, // llvm.aarch64.stxp
    36 << 8 | 1, // llvm.aarch64.stxr
    1 << 8 | 1, // llvm.aarch64.subp
    0 << 8 | 1, // llvm.aarch64.sve.abs
    0 << 8 | 1, // llvm.aarch64.sve.adclb
    0 << 8 | 1, // llvm.aarch64.sve.adclt
    0 << 8 | 1, // llvm.aarch64.sve.add
    1 << 8 | 1, // llvm.aarch64.sve.add.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.add.single.x4
    0 << 8 | 1, // llvm.aarch64.sve.add.u
    0 << 8 | 1, // llvm.aarch64.sve.addhnb
    0 << 8 | 1, // llvm.aarch64.sve.addhnt
    0 << 8 | 1, // llvm.aarch64.sve.addp
    0 << 8 | 1, // llvm.aarch64.sve.addqv
    0 << 8 | 1, // llvm.aarch64.sve.adrb
    0 << 8 | 1, // llvm.aarch64.sve.adrd
    0 << 8 | 1, // llvm.aarch64.sve.adrh
    0 << 8 | 1, // llvm.aarch64.sve.adrw
    0 << 8 | 1, // llvm.aarch64.sve.aesd
    1 << 8 | 35, // llvm.aarch64.sve.aesd.lane.x2
    1 << 8 | 47, // llvm.aarch64.sve.aesd.lane.x4
    1 << 8 | 35, // llvm.aarch64.sve.aesdimc.lane.x2
    1 << 8 | 47, // llvm.aarch64.sve.aesdimc.lane.x4
    0 << 8 | 1, // llvm.aarch64.sve.aese
    1 << 8 | 35, // llvm.aarch64.sve.aese.lane.x2
    1 << 8 | 47, // llvm.aarch64.sve.aese.lane.x4
    1 << 8 | 35, // llvm.aarch64.sve.aesemc.lane.x2
    1 << 8 | 47, // llvm.aarch64.sve.aesemc.lane.x4
    0 << 8 | 1, // llvm.aarch64.sve.aesimc
    0 << 8 | 1, // llvm.aarch64.sve.aesmc
    0 << 8 | 1, // llvm.aarch64.sve.and
    0 << 8 | 1, // llvm.aarch64.sve.and.u
    0 << 8 | 1, // llvm.aarch64.sve.and.z
    0 << 8 | 1, // llvm.aarch64.sve.andqv
    0 << 8 | 1, // llvm.aarch64.sve.andv
    0 << 8 | 1, // llvm.aarch64.sve.asr
    0 << 8 | 1, // llvm.aarch64.sve.asr.u
    0 << 8 | 1, // llvm.aarch64.sve.asr.wide
    0 << 8 | 11, // llvm.aarch64.sve.asrd
    0 << 8 | 1, // llvm.aarch64.sve.bcax
    0 << 8 | 1, // llvm.aarch64.sve.bdep.x
    0 << 8 | 1, // llvm.aarch64.sve.bext.x
    1 << 8 | 1, // llvm.aarch64.sve.bfclamp.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.bfclamp.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.bfcvt.x2
    1 << 8 | 1, // llvm.aarch64.sve.bfcvtn.x2
    1 << 8 | 1, // llvm.aarch64.sve.bfdot
    1 << 8 | 35, // llvm.aarch64.sve.bfdot.lane.v2
    1 << 8 | 1, // llvm.aarch64.sve.bfmlalb
    1 << 8 | 35, // llvm.aarch64.sve.bfmlalb.lane.v2
    1 << 8 | 1, // llvm.aarch64.sve.bfmlalt
    1 << 8 | 35, // llvm.aarch64.sve.bfmlalt.lane.v2
    1 << 8 | 1, // llvm.aarch64.sve.bfmlslb
    1 << 8 | 35, // llvm.aarch64.sve.bfmlslb.lane
    1 << 8 | 1, // llvm.aarch64.sve.bfmlslt
    1 << 8 | 35, // llvm.aarch64.sve.bfmlslt.lane
    1 << 8 | 1, // llvm.aarch64.sve.bfmmla
    0 << 8 | 1, // llvm.aarch64.sve.bgrp.x
    0 << 8 | 1, // llvm.aarch64.sve.bic
    0 << 8 | 1, // llvm.aarch64.sve.bic.u
    0 << 8 | 1, // llvm.aarch64.sve.bic.z
    0 << 8 | 1, // llvm.aarch64.sve.brka
    0 << 8 | 1, // llvm.aarch64.sve.brka.z
    0 << 8 | 1, // llvm.aarch64.sve.brkb
    0 << 8 | 1, // llvm.aarch64.sve.brkb.z
    0 << 8 | 1, // llvm.aarch64.sve.brkn.z
    0 << 8 | 1, // llvm.aarch64.sve.brkpa.z
    0 << 8 | 1, // llvm.aarch64.sve.brkpb.z
    0 << 8 | 1, // llvm.aarch64.sve.bsl
    0 << 8 | 1, // llvm.aarch64.sve.bsl1n
    0 << 8 | 1, // llvm.aarch64.sve.bsl2n
    0 << 8 | 11, // llvm.aarch64.sve.cadd.x
    0 << 8 | 35, // llvm.aarch64.sve.cdot
    0 << 8 | 48, // llvm.aarch64.sve.cdot.lane
    0 << 8 | 1, // llvm.aarch64.sve.clasta
    0 << 8 | 1, // llvm.aarch64.sve.clasta.n
    0 << 8 | 1, // llvm.aarch64.sve.clastb
    0 << 8 | 1, // llvm.aarch64.sve.clastb.n
    0 << 8 | 1, // llvm.aarch64.sve.cls
    0 << 8 | 1, // llvm.aarch64.sve.clz
    0 << 8 | 48, // llvm.aarch64.sve.cmla.lane.x
    0 << 8 | 35, // llvm.aarch64.sve.cmla.x
    0 << 8 | 1, // llvm.aarch64.sve.cmpeq
    0 << 8 | 1, // llvm.aarch64.sve.cmpeq.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmpge
    0 << 8 | 1, // llvm.aarch64.sve.cmpge.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmpgt
    0 << 8 | 1, // llvm.aarch64.sve.cmpgt.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmphi
    0 << 8 | 1, // llvm.aarch64.sve.cmphi.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmphs
    0 << 8 | 1, // llvm.aarch64.sve.cmphs.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmple.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmplo.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmpls.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmplt.wide
    0 << 8 | 1, // llvm.aarch64.sve.cmpne
    0 << 8 | 1, // llvm.aarch64.sve.cmpne.wide
    0 << 8 | 1, // llvm.aarch64.sve.cnot
    0 << 8 | 1, // llvm.aarch64.sve.cnt
    0 << 8 | 5, // llvm.aarch64.sve.cntb
    0 << 8 | 5, // llvm.aarch64.sve.cntd
    0 << 8 | 5, // llvm.aarch64.sve.cnth
    0 << 8 | 1, // llvm.aarch64.sve.cntp
    1 << 8 | 0, // llvm.aarch64.sve.cntp.c16
    1 << 8 | 0, // llvm.aarch64.sve.cntp.c32
    1 << 8 | 0, // llvm.aarch64.sve.cntp.c64
    1 << 8 | 0, // llvm.aarch64.sve.cntp.c8
    0 << 8 | 5, // llvm.aarch64.sve.cntw
    0 << 8 | 1, // llvm.aarch64.sve.compact
    0 << 8 | 1, // llvm.aarch64.sve.convert.from.svbool
    0 << 8 | 1, // llvm.aarch64.sve.convert.to.svbool
    0 << 8 | 1, // llvm.aarch64.sve.dup
    0 << 8 | 0, // llvm.aarch64.sve.dup.laneq
    0 << 8 | 1, // llvm.aarch64.sve.dup.x
    0 << 8 | 1, // llvm.aarch64.sve.dupq.lane
    0 << 8 | 1, // llvm.aarch64.sve.eor
    0 << 8 | 1, // llvm.aarch64.sve.eor.u
    0 << 8 | 1, // llvm.aarch64.sve.eor.z
    0 << 8 | 1, // llvm.aarch64.sve.eor3
    0 << 8 | 1, // llvm.aarch64.sve.eorbt
    0 << 8 | 1, // llvm.aarch64.sve.eorqv
    0 << 8 | 1, // llvm.aarch64.sve.eortb
    0 << 8 | 1, // llvm.aarch64.sve.eorv
    0 << 8 | 1, // llvm.aarch64.sve.expand
    0 << 8 | 11, // llvm.aarch64.sve.ext
    0 << 8 | 11, // llvm.aarch64.sve.extq
    1 << 8 | 1, // llvm.aarch64.sve.fabd
    1 << 8 | 1, // llvm.aarch64.sve.fabd.u
    1 << 8 | 1, // llvm.aarch64.sve.fabs
    1 << 8 | 1, // llvm.aarch64.sve.facge
    1 << 8 | 1, // llvm.aarch64.sve.facgt
    1 << 8 | 1, // llvm.aarch64.sve.fadd
    1 << 8 | 1, // llvm.aarch64.sve.fadd.u
    1 << 8 | 1, // llvm.aarch64.sve.fadda
    1 << 8 | 1, // llvm.aarch64.sve.faddp
    1 << 8 | 1, // llvm.aarch64.sve.faddqv
    1 << 8 | 1, // llvm.aarch64.sve.faddv
    1 << 8 | 1, // llvm.aarch64.sve.famax
    1 << 8 | 1, // llvm.aarch64.sve.famax.u
    1 << 8 | 1, // llvm.aarch64.sve.famin
    1 << 8 | 1, // llvm.aarch64.sve.famin.u
    1 << 8 | 35, // llvm.aarch64.sve.fcadd
    1 << 8 | 1, // llvm.aarch64.sve.fclamp
    1 << 8 | 1, // llvm.aarch64.sve.fclamp.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.fclamp.single.x4
    1 << 8 | 43, // llvm.aarch64.sve.fcmla
    1 << 8 | 48, // llvm.aarch64.sve.fcmla.lane
    1 << 8 | 1, // llvm.aarch64.sve.fcmpeq
    1 << 8 | 1, // llvm.aarch64.sve.fcmpge
    1 << 8 | 1, // llvm.aarch64.sve.fcmpgt
    1 << 8 | 1, // llvm.aarch64.sve.fcmpne
    1 << 8 | 1, // llvm.aarch64.sve.fcmpuo
    1 << 8 | 1, // llvm.aarch64.sve.fcvt
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.bf16f32.v2
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.f16f32
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.f16f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.f32f16
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.f32f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.f64f16
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.f64f32
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.widen.x2
    1 << 8 | 1, // llvm.aarch64.sve.fcvt.x2
    1 << 8 | 1, // llvm.aarch64.sve.fcvtl.widen.x2
    1 << 8 | 1, // llvm.aarch64.sve.fcvtlt.f32f16
    1 << 8 | 1, // llvm.aarch64.sve.fcvtlt.f64f32
    1 << 8 | 1, // llvm.aarch64.sve.fcvtn.x2
    1 << 8 | 1, // llvm.aarch64.sve.fcvtnt.bf16f32.v2
    1 << 8 | 1, // llvm.aarch64.sve.fcvtnt.f16f32
    1 << 8 | 1, // llvm.aarch64.sve.fcvtnt.f32f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvtnt.z.bf16f32
    1 << 8 | 1, // llvm.aarch64.sve.fcvtnt.z.f16f32
    1 << 8 | 1, // llvm.aarch64.sve.fcvtnt.z.f32f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvtx.f32f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvtxnt.f32f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvtxnt.z.f32f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzs
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzs.i32f16
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzs.i32f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzs.i64f16
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzs.i64f32
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzs.x2
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzs.x4
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzu
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzu.i32f16
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzu.i32f64
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzu.i64f16
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzu.i64f32
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzu.x2
    1 << 8 | 1, // llvm.aarch64.sve.fcvtzu.x4
    1 << 8 | 1, // llvm.aarch64.sve.fdiv
    1 << 8 | 1, // llvm.aarch64.sve.fdiv.u
    1 << 8 | 1, // llvm.aarch64.sve.fdivr
    1 << 8 | 35, // llvm.aarch64.sve.fdot.lane.x2
    1 << 8 | 1, // llvm.aarch64.sve.fdot.x2
    1 << 8 | 1, // llvm.aarch64.sve.fexpa.x
    0 << 8 | 1, // llvm.aarch64.sve.firstp
    1 << 8 | 1, // llvm.aarch64.sve.flogb
    1 << 8 | 1, // llvm.aarch64.sve.fmad
    1 << 8 | 1, // llvm.aarch64.sve.fmax
    1 << 8 | 1, // llvm.aarch64.sve.fmax.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.fmax.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.fmax.u
    1 << 8 | 1, // llvm.aarch64.sve.fmax.x2
    1 << 8 | 1, // llvm.aarch64.sve.fmax.x4
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnm
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnm.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnm.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnm.u
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnm.x2
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnm.x4
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnmp
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnmqv
    1 << 8 | 1, // llvm.aarch64.sve.fmaxnmv
    1 << 8 | 1, // llvm.aarch64.sve.fmaxp
    1 << 8 | 1, // llvm.aarch64.sve.fmaxqv
    1 << 8 | 1, // llvm.aarch64.sve.fmaxv
    1 << 8 | 1, // llvm.aarch64.sve.fmin
    1 << 8 | 1, // llvm.aarch64.sve.fmin.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.fmin.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.fmin.u
    1 << 8 | 1, // llvm.aarch64.sve.fmin.x2
    1 << 8 | 1, // llvm.aarch64.sve.fmin.x4
    1 << 8 | 1, // llvm.aarch64.sve.fminnm
    1 << 8 | 1, // llvm.aarch64.sve.fminnm.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.fminnm.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.fminnm.u
    1 << 8 | 1, // llvm.aarch64.sve.fminnm.x2
    1 << 8 | 1, // llvm.aarch64.sve.fminnm.x4
    1 << 8 | 1, // llvm.aarch64.sve.fminnmp
    1 << 8 | 1, // llvm.aarch64.sve.fminnmqv
    1 << 8 | 1, // llvm.aarch64.sve.fminnmv
    1 << 8 | 1, // llvm.aarch64.sve.fminp
    1 << 8 | 1, // llvm.aarch64.sve.fminqv
    1 << 8 | 1, // llvm.aarch64.sve.fminv
    1 << 8 | 1, // llvm.aarch64.sve.fmla
    1 << 8 | 35, // llvm.aarch64.sve.fmla.lane
    1 << 8 | 1, // llvm.aarch64.sve.fmla.u
    1 << 8 | 1, // llvm.aarch64.sve.fmlalb
    1 << 8 | 35, // llvm.aarch64.sve.fmlalb.lane
    1 << 8 | 1, // llvm.aarch64.sve.fmlalt
    1 << 8 | 35, // llvm.aarch64.sve.fmlalt.lane
    1 << 8 | 1, // llvm.aarch64.sve.fmls
    1 << 8 | 35, // llvm.aarch64.sve.fmls.lane
    1 << 8 | 1, // llvm.aarch64.sve.fmls.u
    1 << 8 | 1, // llvm.aarch64.sve.fmlslb
    1 << 8 | 35, // llvm.aarch64.sve.fmlslb.lane
    1 << 8 | 1, // llvm.aarch64.sve.fmlslt
    1 << 8 | 35, // llvm.aarch64.sve.fmlslt.lane
    1 << 8 | 1, // llvm.aarch64.sve.fmmla
    1 << 8 | 1, // llvm.aarch64.sve.fmsb
    1 << 8 | 1, // llvm.aarch64.sve.fmul
    1 << 8 | 11, // llvm.aarch64.sve.fmul.lane
    1 << 8 | 1, // llvm.aarch64.sve.fmul.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.fmul.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.fmul.u
    1 << 8 | 1, // llvm.aarch64.sve.fmul.x2
    1 << 8 | 1, // llvm.aarch64.sve.fmul.x4
    1 << 8 | 1, // llvm.aarch64.sve.fmulx
    1 << 8 | 1, // llvm.aarch64.sve.fmulx.u
    1 << 8 | 1, // llvm.aarch64.sve.fneg
    1 << 8 | 1, // llvm.aarch64.sve.fnmad
    1 << 8 | 1, // llvm.aarch64.sve.fnmla
    1 << 8 | 1, // llvm.aarch64.sve.fnmla.u
    1 << 8 | 1, // llvm.aarch64.sve.fnmls
    1 << 8 | 1, // llvm.aarch64.sve.fnmls.u
    1 << 8 | 1, // llvm.aarch64.sve.fnmsb
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvt.x2
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvt.x4
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvt1
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvt1.x2
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvt2
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvt2.x2
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvtl1.x2
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvtl2.x2
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvtlt1
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvtlt2
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvtn
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvtn.x4
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvtnb
    37 << 8 | 1, // llvm.aarch64.sve.fp8.cvtnt
    37 << 8 | 1, // llvm.aarch64.sve.fp8.fdot
    37 << 8 | 35, // llvm.aarch64.sve.fp8.fdot.lane
    37 << 8 | 1, // llvm.aarch64.sve.fp8.fmlalb
    37 << 8 | 35, // llvm.aarch64.sve.fp8.fmlalb.lane
    37 << 8 | 1, // llvm.aarch64.sve.fp8.fmlallbb
    37 << 8 | 35, // llvm.aarch64.sve.fp8.fmlallbb.lane
    37 << 8 | 1, // llvm.aarch64.sve.fp8.fmlallbt
    37 << 8 | 35, // llvm.aarch64.sve.fp8.fmlallbt.lane
    37 << 8 | 1, // llvm.aarch64.sve.fp8.fmlalltb
    37 << 8 | 35, // llvm.aarch64.sve.fp8.fmlalltb.lane
    37 << 8 | 1, // llvm.aarch64.sve.fp8.fmlalltt
    37 << 8 | 35, // llvm.aarch64.sve.fp8.fmlalltt.lane
    37 << 8 | 1, // llvm.aarch64.sve.fp8.fmlalt
    37 << 8 | 35, // llvm.aarch64.sve.fp8.fmlalt.lane
    37 << 8 | 1, // llvm.aarch64.sve.fp8.fmmla
    1 << 8 | 1, // llvm.aarch64.sve.frecpe.x
    1 << 8 | 1, // llvm.aarch64.sve.frecps.x
    1 << 8 | 1, // llvm.aarch64.sve.frecpx
    1 << 8 | 1, // llvm.aarch64.sve.frint32x
    1 << 8 | 1, // llvm.aarch64.sve.frint32z
    1 << 8 | 1, // llvm.aarch64.sve.frint64x
    1 << 8 | 1, // llvm.aarch64.sve.frint64z
    1 << 8 | 1, // llvm.aarch64.sve.frinta
    1 << 8 | 1, // llvm.aarch64.sve.frinta.x2
    1 << 8 | 1, // llvm.aarch64.sve.frinta.x4
    1 << 8 | 1, // llvm.aarch64.sve.frinti
    1 << 8 | 1, // llvm.aarch64.sve.frintm
    1 << 8 | 1, // llvm.aarch64.sve.frintm.x2
    1 << 8 | 1, // llvm.aarch64.sve.frintm.x4
    1 << 8 | 1, // llvm.aarch64.sve.frintn
    1 << 8 | 1, // llvm.aarch64.sve.frintn.x2
    1 << 8 | 1, // llvm.aarch64.sve.frintn.x4
    1 << 8 | 1, // llvm.aarch64.sve.frintp
    1 << 8 | 1, // llvm.aarch64.sve.frintp.x2
    1 << 8 | 1, // llvm.aarch64.sve.frintp.x4
    1 << 8 | 1, // llvm.aarch64.sve.frintx
    1 << 8 | 1, // llvm.aarch64.sve.frintz
    1 << 8 | 1, // llvm.aarch64.sve.frsqrte.x
    1 << 8 | 1, // llvm.aarch64.sve.frsqrts.x
    1 << 8 | 1, // llvm.aarch64.sve.fscale
    1 << 8 | 1, // llvm.aarch64.sve.fscale.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.fscale.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.fscale.x2
    1 << 8 | 1, // llvm.aarch64.sve.fscale.x4
    1 << 8 | 1, // llvm.aarch64.sve.fsqrt
    1 << 8 | 1, // llvm.aarch64.sve.fsub
    1 << 8 | 1, // llvm.aarch64.sve.fsub.u
    1 << 8 | 1, // llvm.aarch64.sve.fsubr
    1 << 8 | 11, // llvm.aarch64.sve.ftmad.x
    1 << 8 | 1, // llvm.aarch64.sve.ftsmul.x
    1 << 8 | 1, // llvm.aarch64.sve.ftssel.x
    0 << 8 | 1, // llvm.aarch64.sve.histcnt
    0 << 8 | 1, // llvm.aarch64.sve.histseg
    0 << 8 | 1, // llvm.aarch64.sve.index
    0 << 8 | 1, // llvm.aarch64.sve.insr
    0 << 8 | 1, // llvm.aarch64.sve.lasta
    0 << 8 | 1, // llvm.aarch64.sve.lastb
    0 << 8 | 1, // llvm.aarch64.sve.lastp
    2 << 8 | 1, // llvm.aarch64.sve.ld1
    2 << 8 | 1, // llvm.aarch64.sve.ld1.gather
    2 << 8 | 1, // llvm.aarch64.sve.ld1.gather.index
    23 << 8 | 1, // llvm.aarch64.sve.ld1.gather.scalar.offset
    2 << 8 | 1, // llvm.aarch64.sve.ld1.gather.sxtw
    2 << 8 | 1, // llvm.aarch64.sve.ld1.gather.sxtw.index
    2 << 8 | 1, // llvm.aarch64.sve.ld1.gather.uxtw
    2 << 8 | 1, // llvm.aarch64.sve.ld1.gather.uxtw.index
    2 << 8 | 1, // llvm.aarch64.sve.ld1.pn.x2
    2 << 8 | 1, // llvm.aarch64.sve.ld1.pn.x4
    2 << 8 | 1, // llvm.aarch64.sve.ld1q.gather.index
    23 << 8 | 1, // llvm.aarch64.sve.ld1q.gather.scalar.offset
    2 << 8 | 1, // llvm.aarch64.sve.ld1q.gather.vector.offset
    2 << 8 | 1, // llvm.aarch64.sve.ld1ro
    2 << 8 | 1, // llvm.aarch64.sve.ld1rq
    2 << 8 | 1, // llvm.aarch64.sve.ld1udq
    2 << 8 | 1, // llvm.aarch64.sve.ld1uwq
    2 << 8 | 1, // llvm.aarch64.sve.ld2.sret
    2 << 8 | 1, // llvm.aarch64.sve.ld2q.sret
    2 << 8 | 1, // llvm.aarch64.sve.ld3.sret
    2 << 8 | 1, // llvm.aarch64.sve.ld3q.sret
    2 << 8 | 1, // llvm.aarch64.sve.ld4.sret
    2 << 8 | 1, // llvm.aarch64.sve.ld4q.sret
    28 << 8 | 1, // llvm.aarch64.sve.ldff1
    28 << 8 | 1, // llvm.aarch64.sve.ldff1.gather
    28 << 8 | 1, // llvm.aarch64.sve.ldff1.gather.index
    28 << 8 | 1, // llvm.aarch64.sve.ldff1.gather.scalar.offset
    28 << 8 | 1, // llvm.aarch64.sve.ldff1.gather.sxtw
    28 << 8 | 1, // llvm.aarch64.sve.ldff1.gather.sxtw.index
    28 << 8 | 1, // llvm.aarch64.sve.ldff1.gather.uxtw
    28 << 8 | 1, // llvm.aarch64.sve.ldff1.gather.uxtw.index
    28 << 8 | 1, // llvm.aarch64.sve.ldnf1
    2 << 8 | 1, // llvm.aarch64.sve.ldnt1
    2 << 8 | 1, // llvm.aarch64.sve.ldnt1.gather
    2 << 8 | 1, // llvm.aarch64.sve.ldnt1.gather.index
    23 << 8 | 1, // llvm.aarch64.sve.ldnt1.gather.scalar.offset
    2 << 8 | 1, // llvm.aarch64.sve.ldnt1.gather.uxtw
    2 << 8 | 1, // llvm.aarch64.sve.ldnt1.pn.x2
    2 << 8 | 1, // llvm.aarch64.sve.ldnt1.pn.x4
    0 << 8 | 1, // llvm.aarch64.sve.lsl
    0 << 8 | 1, // llvm.aarch64.sve.lsl.u
    0 << 8 | 1, // llvm.aarch64.sve.lsl.wide
    0 << 8 | 1, // llvm.aarch64.sve.lsr
    0 << 8 | 1, // llvm.aarch64.sve.lsr.u
    0 << 8 | 1, // llvm.aarch64.sve.lsr.wide
    0 << 8 | 11, // llvm.aarch64.sve.luti2.lane
    0 << 8 | 11, // llvm.aarch64.sve.luti4.lane
    0 << 8 | 35, // llvm.aarch64.sve.luti4.lane.x2
    0 << 8 | 1, // llvm.aarch64.sve.mad
    0 << 8 | 1, // llvm.aarch64.sve.match
    0 << 8 | 1, // llvm.aarch64.sve.mla
    0 << 8 | 35, // llvm.aarch64.sve.mla.lane
    0 << 8 | 1, // llvm.aarch64.sve.mla.u
    0 << 8 | 1, // llvm.aarch64.sve.mls
    0 << 8 | 35, // llvm.aarch64.sve.mls.lane
    0 << 8 | 1, // llvm.aarch64.sve.mls.u
    0 << 8 | 1, // llvm.aarch64.sve.msb
    0 << 8 | 1, // llvm.aarch64.sve.mul
    0 << 8 | 11, // llvm.aarch64.sve.mul.lane
    0 << 8 | 1, // llvm.aarch64.sve.mul.u
    0 << 8 | 1, // llvm.aarch64.sve.nand.z
    0 << 8 | 1, // llvm.aarch64.sve.nbsl
    0 << 8 | 1, // llvm.aarch64.sve.neg
    0 << 8 | 1, // llvm.aarch64.sve.nmatch
    0 << 8 | 1, // llvm.aarch64.sve.nor.z
    0 << 8 | 1, // llvm.aarch64.sve.not
    0 << 8 | 1, // llvm.aarch64.sve.orn.z
    0 << 8 | 1, // llvm.aarch64.sve.orqv
    0 << 8 | 1, // llvm.aarch64.sve.orr
    0 << 8 | 1, // llvm.aarch64.sve.orr.u
    0 << 8 | 1, // llvm.aarch64.sve.orr.z
    0 << 8 | 1, // llvm.aarch64.sve.orv
    1 << 8 | 0, // llvm.aarch64.sve.pext
    1 << 8 | 0, // llvm.aarch64.sve.pext.x2
    0 << 8 | 1, // llvm.aarch64.sve.pfirst
    1 << 8 | 1, // llvm.aarch64.sve.pmlal.pair.x2
    0 << 8 | 0, // llvm.aarch64.sve.pmov.to.pred.lane
    0 << 8 | 1, // llvm.aarch64.sve.pmov.to.pred.lane.zero
    0 << 8 | 11, // llvm.aarch64.sve.pmov.to.vector.lane.merging
    0 << 8 | 1, // llvm.aarch64.sve.pmov.to.vector.lane.zeroing
    0 << 8 | 1, // llvm.aarch64.sve.pmul
    1 << 8 | 1, // llvm.aarch64.sve.pmull.pair.x2
    0 << 8 | 1, // llvm.aarch64.sve.pmullb.pair
    0 << 8 | 1, // llvm.aarch64.sve.pmullt.pair
    0 << 8 | 1, // llvm.aarch64.sve.pnext
    18 << 8 | 11, // llvm.aarch64.sve.prf
    28 << 8 | 49, // llvm.aarch64.sve.prfb.gather.index
    28 << 8 | 35, // llvm.aarch64.sve.prfb.gather.scalar.offset
    28 << 8 | 49, // llvm.aarch64.sve.prfb.gather.sxtw.index
    28 << 8 | 49, // llvm.aarch64.sve.prfb.gather.uxtw.index
    28 << 8 | 49, // llvm.aarch64.sve.prfd.gather.index
    28 << 8 | 35, // llvm.aarch64.sve.prfd.gather.scalar.offset
    28 << 8 | 49, // llvm.aarch64.sve.prfd.gather.sxtw.index
    28 << 8 | 49, // llvm.aarch64.sve.prfd.gather.uxtw.index
    28 << 8 | 49, // llvm.aarch64.sve.prfh.gather.index
    28 << 8 | 35, // llvm.aarch64.sve.prfh.gather.scalar.offset
    28 << 8 | 49, // llvm.aarch64.sve.prfh.gather.sxtw.index
    28 << 8 | 49, // llvm.aarch64.sve.prfh.gather.uxtw.index
    28 << 8 | 49, // llvm.aarch64.sve.prfw.gather.index
    28 << 8 | 35, // llvm.aarch64.sve.prfw.gather.scalar.offset
    28 << 8 | 49, // llvm.aarch64.sve.prfw.gather.sxtw.index
    28 << 8 | 49, // llvm.aarch64.sve.prfw.gather.uxtw.index
    1 << 8 | 1, // llvm.aarch64.sve.psel
    0 << 8 | 1, // llvm.aarch64.sve.ptest.any
    0 << 8 | 1, // llvm.aarch64.sve.ptest.first
    0 << 8 | 1, // llvm.aarch64.sve.ptest.last
    1 << 8 | 5, // llvm.aarch64.sve.ptrue
    1 << 8 | 1, // llvm.aarch64.sve.ptrue.c16
    1 << 8 | 1, // llvm.aarch64.sve.ptrue.c32
    1 << 8 | 1, // llvm.aarch64.sve.ptrue.c64
    1 << 8 | 1, // llvm.aarch64.sve.ptrue.c8
    0 << 8 | 1, // llvm.aarch64.sve.punpkhi
    0 << 8 | 1, // llvm.aarch64.sve.punpklo
    0 << 8 | 1, // llvm.aarch64.sve.raddhnb
    0 << 8 | 1, // llvm.aarch64.sve.raddhnt
    0 << 8 | 1, // llvm.aarch64.sve.rax1
    0 << 8 | 1, // llvm.aarch64.sve.rbit
    37 << 8 | 1, // llvm.aarch64.sve.rdffr
    37 << 8 | 1, // llvm.aarch64.sve.rdffr.z
    0 << 8 | 1, // llvm.aarch64.sve.rev.b16
    0 << 8 | 1, // llvm.aarch64.sve.rev.b32
    0 << 8 | 1, // llvm.aarch64.sve.rev.b64
    0 << 8 | 1, // llvm.aarch64.sve.revb
    0 << 8 | 1, // llvm.aarch64.sve.revd
    0 << 8 | 1, // llvm.aarch64.sve.revh
    0 << 8 | 1, // llvm.aarch64.sve.revw
    0 << 8 | 0, // llvm.aarch64.sve.rshrnb
    0 << 8 | 11, // llvm.aarch64.sve.rshrnt
    0 << 8 | 1, // llvm.aarch64.sve.rsubhnb
    0 << 8 | 1, // llvm.aarch64.sve.rsubhnt
    0 << 8 | 1, // llvm.aarch64.sve.saba
    0 << 8 | 1, // llvm.aarch64.sve.sabalb
    0 << 8 | 1, // llvm.aarch64.sve.sabalt
    0 << 8 | 1, // llvm.aarch64.sve.sabd
    0 << 8 | 1, // llvm.aarch64.sve.sabd.u
    0 << 8 | 1, // llvm.aarch64.sve.sabdlb
    0 << 8 | 1, // llvm.aarch64.sve.sabdlt
    0 << 8 | 1, // llvm.aarch64.sve.sadalp
    0 << 8 | 1, // llvm.aarch64.sve.saddlb
    0 << 8 | 1, // llvm.aarch64.sve.saddlbt
    0 << 8 | 1, // llvm.aarch64.sve.saddlt
    0 << 8 | 1, // llvm.aarch64.sve.saddv
    0 << 8 | 1, // llvm.aarch64.sve.saddwb
    0 << 8 | 1, // llvm.aarch64.sve.saddwt
    0 << 8 | 1, // llvm.aarch64.sve.sbclb
    0 << 8 | 1, // llvm.aarch64.sve.sbclt
    0 << 8 | 1, // llvm.aarch64.sve.sclamp
    1 << 8 | 1, // llvm.aarch64.sve.sclamp.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.sclamp.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.scvtf
    1 << 8 | 1, // llvm.aarch64.sve.scvtf.f16i32
    1 << 8 | 1, // llvm.aarch64.sve.scvtf.f16i64
    1 << 8 | 1, // llvm.aarch64.sve.scvtf.f32i64
    1 << 8 | 1, // llvm.aarch64.sve.scvtf.f64i32
    1 << 8 | 1, // llvm.aarch64.sve.scvtf.x2
    1 << 8 | 1, // llvm.aarch64.sve.scvtf.x4
    0 << 8 | 1, // llvm.aarch64.sve.sdiv
    0 << 8 | 1, // llvm.aarch64.sve.sdiv.u
    0 << 8 | 1, // llvm.aarch64.sve.sdivr
    0 << 8 | 1, // llvm.aarch64.sve.sdot
    0 << 8 | 35, // llvm.aarch64.sve.sdot.lane
    0 << 8 | 35, // llvm.aarch64.sve.sdot.lane.x2
    0 << 8 | 1, // llvm.aarch64.sve.sdot.x2
    0 << 8 | 1, // llvm.aarch64.sve.sel
    1 << 8 | 1, // llvm.aarch64.sve.sel.x2
    1 << 8 | 1, // llvm.aarch64.sve.sel.x4
    6 << 8 | 1, // llvm.aarch64.sve.setffr
    0 << 8 | 1, // llvm.aarch64.sve.shadd
    0 << 8 | 0, // llvm.aarch64.sve.shrnb
    0 << 8 | 11, // llvm.aarch64.sve.shrnt
    0 << 8 | 1, // llvm.aarch64.sve.shsub
    0 << 8 | 1, // llvm.aarch64.sve.shsub.u
    0 << 8 | 1, // llvm.aarch64.sve.shsubr
    0 << 8 | 11, // llvm.aarch64.sve.sli
    0 << 8 | 1, // llvm.aarch64.sve.sm4e
    0 << 8 | 1, // llvm.aarch64.sve.sm4ekey
    0 << 8 | 1, // llvm.aarch64.sve.smax
    1 << 8 | 1, // llvm.aarch64.sve.smax.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.smax.single.x4
    0 << 8 | 1, // llvm.aarch64.sve.smax.u
    1 << 8 | 1, // llvm.aarch64.sve.smax.x2
    1 << 8 | 1, // llvm.aarch64.sve.smax.x4
    0 << 8 | 1, // llvm.aarch64.sve.smaxp
    0 << 8 | 1, // llvm.aarch64.sve.smaxqv
    0 << 8 | 1, // llvm.aarch64.sve.smaxv
    0 << 8 | 1, // llvm.aarch64.sve.smin
    1 << 8 | 1, // llvm.aarch64.sve.smin.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.smin.single.x4
    0 << 8 | 1, // llvm.aarch64.sve.smin.u
    1 << 8 | 1, // llvm.aarch64.sve.smin.x2
    1 << 8 | 1, // llvm.aarch64.sve.smin.x4
    0 << 8 | 1, // llvm.aarch64.sve.sminp
    0 << 8 | 1, // llvm.aarch64.sve.sminqv
    0 << 8 | 1, // llvm.aarch64.sve.sminv
    0 << 8 | 1, // llvm.aarch64.sve.smlalb
    0 << 8 | 35, // llvm.aarch64.sve.smlalb.lane
    0 << 8 | 1, // llvm.aarch64.sve.smlalt
    0 << 8 | 35, // llvm.aarch64.sve.smlalt.lane
    0 << 8 | 1, // llvm.aarch64.sve.smlslb
    0 << 8 | 35, // llvm.aarch64.sve.smlslb.lane
    0 << 8 | 1, // llvm.aarch64.sve.smlslt
    0 << 8 | 35, // llvm.aarch64.sve.smlslt.lane
    0 << 8 | 1, // llvm.aarch64.sve.smmla
    0 << 8 | 1, // llvm.aarch64.sve.smulh
    0 << 8 | 1, // llvm.aarch64.sve.smulh.u
    0 << 8 | 1, // llvm.aarch64.sve.smullb
    0 << 8 | 11, // llvm.aarch64.sve.smullb.lane
    0 << 8 | 1, // llvm.aarch64.sve.smullt
    0 << 8 | 11, // llvm.aarch64.sve.smullt.lane
    0 << 8 | 1, // llvm.aarch64.sve.splice
    0 << 8 | 1, // llvm.aarch64.sve.sqabs
    0 << 8 | 1, // llvm.aarch64.sve.sqadd
    0 << 8 | 1, // llvm.aarch64.sve.sqadd.x
    0 << 8 | 11, // llvm.aarch64.sve.sqcadd.x
    1 << 8 | 1, // llvm.aarch64.sve.sqcvt.x2
    1 << 8 | 1, // llvm.aarch64.sve.sqcvt.x4
    1 << 8 | 1, // llvm.aarch64.sve.sqcvtn.x2
    1 << 8 | 1, // llvm.aarch64.sve.sqcvtn.x4
    1 << 8 | 1, // llvm.aarch64.sve.sqcvtu.x2
    1 << 8 | 1, // llvm.aarch64.sve.sqcvtu.x4
    1 << 8 | 1, // llvm.aarch64.sve.sqcvtun.x2
    1 << 8 | 1, // llvm.aarch64.sve.sqcvtun.x4
    0 << 8 | 13, // llvm.aarch64.sve.sqdecb.n32
    0 << 8 | 13, // llvm.aarch64.sve.sqdecb.n64
    0 << 8 | 13, // llvm.aarch64.sve.sqdecd
    0 << 8 | 13, // llvm.aarch64.sve.sqdecd.n32
    0 << 8 | 13, // llvm.aarch64.sve.sqdecd.n64
    0 << 8 | 13, // llvm.aarch64.sve.sqdech
    0 << 8 | 13, // llvm.aarch64.sve.sqdech.n32
    0 << 8 | 13, // llvm.aarch64.sve.sqdech.n64
    0 << 8 | 1, // llvm.aarch64.sve.sqdecp
    0 << 8 | 1, // llvm.aarch64.sve.sqdecp.n32
    0 << 8 | 1, // llvm.aarch64.sve.sqdecp.n64
    0 << 8 | 13, // llvm.aarch64.sve.sqdecw
    0 << 8 | 13, // llvm.aarch64.sve.sqdecw.n32
    0 << 8 | 13, // llvm.aarch64.sve.sqdecw.n64
    0 << 8 | 1, // llvm.aarch64.sve.sqdmlalb
    0 << 8 | 35, // llvm.aarch64.sve.sqdmlalb.lane
    0 << 8 | 1, // llvm.aarch64.sve.sqdmlalbt
    0 << 8 | 1, // llvm.aarch64.sve.sqdmlalt
    0 << 8 | 35, // llvm.aarch64.sve.sqdmlalt.lane
    0 << 8 | 1, // llvm.aarch64.sve.sqdmlslb
    0 << 8 | 35, // llvm.aarch64.sve.sqdmlslb.lane
    0 << 8 | 1, // llvm.aarch64.sve.sqdmlslbt
    0 << 8 | 1, // llvm.aarch64.sve.sqdmlslt
    0 << 8 | 35, // llvm.aarch64.sve.sqdmlslt.lane
    0 << 8 | 1, // llvm.aarch64.sve.sqdmulh
    0 << 8 | 11, // llvm.aarch64.sve.sqdmulh.lane
    1 << 8 | 1, // llvm.aarch64.sve.sqdmulh.single.vgx2
    1 << 8 | 1, // llvm.aarch64.sve.sqdmulh.single.vgx4
    1 << 8 | 1, // llvm.aarch64.sve.sqdmulh.vgx2
    1 << 8 | 1, // llvm.aarch64.sve.sqdmulh.vgx4
    0 << 8 | 1, // llvm.aarch64.sve.sqdmullb
    0 << 8 | 11, // llvm.aarch64.sve.sqdmullb.lane
    0 << 8 | 1, // llvm.aarch64.sve.sqdmullt
    0 << 8 | 11, // llvm.aarch64.sve.sqdmullt.lane
    0 << 8 | 13, // llvm.aarch64.sve.sqincb.n32
    0 << 8 | 13, // llvm.aarch64.sve.sqincb.n64
    0 << 8 | 13, // llvm.aarch64.sve.sqincd
    0 << 8 | 13, // llvm.aarch64.sve.sqincd.n32
    0 << 8 | 13, // llvm.aarch64.sve.sqincd.n64
    0 << 8 | 13, // llvm.aarch64.sve.sqinch
    0 << 8 | 13, // llvm.aarch64.sve.sqinch.n32
    0 << 8 | 13, // llvm.aarch64.sve.sqinch.n64
    0 << 8 | 1, // llvm.aarch64.sve.sqincp
    0 << 8 | 1, // llvm.aarch64.sve.sqincp.n32
    0 << 8 | 1, // llvm.aarch64.sve.sqincp.n64
    0 << 8 | 13, // llvm.aarch64.sve.sqincw
    0 << 8 | 13, // llvm.aarch64.sve.sqincw.n32
    0 << 8 | 13, // llvm.aarch64.sve.sqincw.n64
    0 << 8 | 1, // llvm.aarch64.sve.sqneg
    0 << 8 | 48, // llvm.aarch64.sve.sqrdcmlah.lane.x
    0 << 8 | 35, // llvm.aarch64.sve.sqrdcmlah.x
    0 << 8 | 1, // llvm.aarch64.sve.sqrdmlah
    0 << 8 | 35, // llvm.aarch64.sve.sqrdmlah.lane
    0 << 8 | 1, // llvm.aarch64.sve.sqrdmlsh
    0 << 8 | 35, // llvm.aarch64.sve.sqrdmlsh.lane
    0 << 8 | 1, // llvm.aarch64.sve.sqrdmulh
    0 << 8 | 11, // llvm.aarch64.sve.sqrdmulh.lane
    0 << 8 | 1, // llvm.aarch64.sve.sqrshl
    0 << 8 | 1, // llvm.aarch64.sve.sqrshl.u
    1 << 8 | 11, // llvm.aarch64.sve.sqrshr.x2
    1 << 8 | 43, // llvm.aarch64.sve.sqrshr.x4
    1 << 8 | 11, // llvm.aarch64.sve.sqrshrn.x2
    1 << 8 | 43, // llvm.aarch64.sve.sqrshrn.x4
    0 << 8 | 0, // llvm.aarch64.sve.sqrshrnb
    0 << 8 | 11, // llvm.aarch64.sve.sqrshrnt
    1 << 8 | 11, // llvm.aarch64.sve.sqrshru.x2
    1 << 8 | 43, // llvm.aarch64.sve.sqrshru.x4
    1 << 8 | 11, // llvm.aarch64.sve.sqrshrun.x2
    1 << 8 | 43, // llvm.aarch64.sve.sqrshrun.x4
    0 << 8 | 0, // llvm.aarch64.sve.sqrshrunb
    0 << 8 | 11, // llvm.aarch64.sve.sqrshrunt
    0 << 8 | 1, // llvm.aarch64.sve.sqshl
    0 << 8 | 1, // llvm.aarch64.sve.sqshl.u
    0 << 8 | 11, // llvm.aarch64.sve.sqshlu
    0 << 8 | 0, // llvm.aarch64.sve.sqshrnb
    0 << 8 | 11, // llvm.aarch64.sve.sqshrnt
    0 << 8 | 0, // llvm.aarch64.sve.sqshrunb
    0 << 8 | 11, // llvm.aarch64.sve.sqshrunt
    0 << 8 | 1, // llvm.aarch64.sve.sqsub
    0 << 8 | 1, // llvm.aarch64.sve.sqsub.u
    0 << 8 | 1, // llvm.aarch64.sve.sqsub.x
    0 << 8 | 1, // llvm.aarch64.sve.sqsubr
    0 << 8 | 1, // llvm.aarch64.sve.sqxtnb
    0 << 8 | 1, // llvm.aarch64.sve.sqxtnt
    0 << 8 | 1, // llvm.aarch64.sve.sqxtunb
    0 << 8 | 1, // llvm.aarch64.sve.sqxtunt
    0 << 8 | 1, // llvm.aarch64.sve.srhadd
    0 << 8 | 11, // llvm.aarch64.sve.sri
    0 << 8 | 1, // llvm.aarch64.sve.srshl
    1 << 8 | 1, // llvm.aarch64.sve.srshl.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.srshl.single.x4
    0 << 8 | 1, // llvm.aarch64.sve.srshl.u
    1 << 8 | 1, // llvm.aarch64.sve.srshl.x2
    1 << 8 | 1, // llvm.aarch64.sve.srshl.x4
    0 << 8 | 11, // llvm.aarch64.sve.srshr
    0 << 8 | 11, // llvm.aarch64.sve.srsra
    0 << 8 | 0, // llvm.aarch64.sve.sshllb
    0 << 8 | 0, // llvm.aarch64.sve.sshllt
    0 << 8 | 11, // llvm.aarch64.sve.ssra
    0 << 8 | 1, // llvm.aarch64.sve.ssublb
    0 << 8 | 1, // llvm.aarch64.sve.ssublbt
    0 << 8 | 1, // llvm.aarch64.sve.ssublt
    0 << 8 | 1, // llvm.aarch64.sve.ssubltb
    0 << 8 | 1, // llvm.aarch64.sve.ssubwb
    0 << 8 | 1, // llvm.aarch64.sve.ssubwt
    18 << 8 | 37, // llvm.aarch64.sve.st1
    19 << 8 | 1, // llvm.aarch64.sve.st1.pn.x2
    19 << 8 | 1, // llvm.aarch64.sve.st1.pn.x4
    19 << 8 | 1, // llvm.aarch64.sve.st1.scatter
    19 << 8 | 1, // llvm.aarch64.sve.st1.scatter.index
    24 << 8 | 1, // llvm.aarch64.sve.st1.scatter.scalar.offset
    19 << 8 | 1, // llvm.aarch64.sve.st1.scatter.sxtw
    19 << 8 | 1, // llvm.aarch64.sve.st1.scatter.sxtw.index
    19 << 8 | 1, // llvm.aarch64.sve.st1.scatter.uxtw
    19 << 8 | 1, // llvm.aarch64.sve.st1.scatter.uxtw.index
    19 << 8 | 1, // llvm.aarch64.sve.st1dq
    19 << 8 | 1, // llvm.aarch64.sve.st1q.scatter.index
    24 << 8 | 1, // llvm.aarch64.sve.st1q.scatter.scalar.offset
    19 << 8 | 1, // llvm.aarch64.sve.st1q.scatter.vector.offset
    19 << 8 | 1, // llvm.aarch64.sve.st1wq
    18 << 8 | 38, // llvm.aarch64.sve.st2
    18 << 8 | 38, // llvm.aarch64.sve.st2q
    18 << 8 | 39, // llvm.aarch64.sve.st3
    18 << 8 | 39, // llvm.aarch64.sve.st3q
    18 << 8 | 40, // llvm.aarch64.sve.st4
    18 << 8 | 40, // llvm.aarch64.sve.st4q
    18 << 8 | 37, // llvm.aarch64.sve.stnt1
    19 << 8 | 1, // llvm.aarch64.sve.stnt1.pn.x2
    19 << 8 | 1, // llvm.aarch64.sve.stnt1.pn.x4
    19 << 8 | 1, // llvm.aarch64.sve.stnt1.scatter
    19 << 8 | 1, // llvm.aarch64.sve.stnt1.scatter.index
    24 << 8 | 1, // llvm.aarch64.sve.stnt1.scatter.scalar.offset
    19 << 8 | 1, // llvm.aarch64.sve.stnt1.scatter.uxtw
    0 << 8 | 1, // llvm.aarch64.sve.sub
    0 << 8 | 1, // llvm.aarch64.sve.sub.u
    0 << 8 | 1, // llvm.aarch64.sve.subhnb
    0 << 8 | 1, // llvm.aarch64.sve.subhnt
    0 << 8 | 1, // llvm.aarch64.sve.subr
    0 << 8 | 35, // llvm.aarch64.sve.sudot.lane
    1 << 8 | 1, // llvm.aarch64.sve.sunpk.x2
    1 << 8 | 1, // llvm.aarch64.sve.sunpk.x4
    0 << 8 | 1, // llvm.aarch64.sve.sunpkhi
    0 << 8 | 1, // llvm.aarch64.sve.sunpklo
    0 << 8 | 1, // llvm.aarch64.sve.suqadd
    0 << 8 | 1, // llvm.aarch64.sve.sxtb
    0 << 8 | 1, // llvm.aarch64.sve.sxth
    0 << 8 | 1, // llvm.aarch64.sve.sxtw
    0 << 8 | 1, // llvm.aarch64.sve.tbl
    0 << 8 | 1, // llvm.aarch64.sve.tbl2
    0 << 8 | 1, // llvm.aarch64.sve.tblq
    0 << 8 | 1, // llvm.aarch64.sve.tbx
    0 << 8 | 1, // llvm.aarch64.sve.tbxq
    0 << 8 | 1, // llvm.aarch64.sve.trn1
    0 << 8 | 1, // llvm.aarch64.sve.trn1.b16
    0 << 8 | 1, // llvm.aarch64.sve.trn1.b32
    0 << 8 | 1, // llvm.aarch64.sve.trn1.b64
    0 << 8 | 1, // llvm.aarch64.sve.trn1q
    0 << 8 | 1, // llvm.aarch64.sve.trn2
    0 << 8 | 1, // llvm.aarch64.sve.trn2.b16
    0 << 8 | 1, // llvm.aarch64.sve.trn2.b32
    0 << 8 | 1, // llvm.aarch64.sve.trn2.b64
    0 << 8 | 1, // llvm.aarch64.sve.trn2q
    0 << 8 | 1, // llvm.aarch64.sve.uaba
    0 << 8 | 1, // llvm.aarch64.sve.uabalb
    0 << 8 | 1, // llvm.aarch64.sve.uabalt
    0 << 8 | 1, // llvm.aarch64.sve.uabd
    0 << 8 | 1, // llvm.aarch64.sve.uabd.u
    0 << 8 | 1, // llvm.aarch64.sve.uabdlb
    0 << 8 | 1, // llvm.aarch64.sve.uabdlt
    0 << 8 | 1, // llvm.aarch64.sve.uadalp
    0 << 8 | 1, // llvm.aarch64.sve.uaddlb
    0 << 8 | 1, // llvm.aarch64.sve.uaddlt
    0 << 8 | 1, // llvm.aarch64.sve.uaddv
    0 << 8 | 1, // llvm.aarch64.sve.uaddwb
    0 << 8 | 1, // llvm.aarch64.sve.uaddwt
    0 << 8 | 1, // llvm.aarch64.sve.uclamp
    1 << 8 | 1, // llvm.aarch64.sve.uclamp.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.uclamp.single.x4
    1 << 8 | 1, // llvm.aarch64.sve.ucvtf
    1 << 8 | 1, // llvm.aarch64.sve.ucvtf.f16i32
    1 << 8 | 1, // llvm.aarch64.sve.ucvtf.f16i64
    1 << 8 | 1, // llvm.aarch64.sve.ucvtf.f32i64
    1 << 8 | 1, // llvm.aarch64.sve.ucvtf.f64i32
    1 << 8 | 1, // llvm.aarch64.sve.ucvtf.x2
    1 << 8 | 1, // llvm.aarch64.sve.ucvtf.x4
    0 << 8 | 1, // llvm.aarch64.sve.udiv
    0 << 8 | 1, // llvm.aarch64.sve.udiv.u
    0 << 8 | 1, // llvm.aarch64.sve.udivr
    0 << 8 | 1, // llvm.aarch64.sve.udot
    0 << 8 | 35, // llvm.aarch64.sve.udot.lane
    0 << 8 | 35, // llvm.aarch64.sve.udot.lane.x2
    0 << 8 | 1, // llvm.aarch64.sve.udot.x2
    0 << 8 | 1, // llvm.aarch64.sve.uhadd
    0 << 8 | 1, // llvm.aarch64.sve.uhsub
    0 << 8 | 1, // llvm.aarch64.sve.uhsub.u
    0 << 8 | 1, // llvm.aarch64.sve.uhsubr
    0 << 8 | 1, // llvm.aarch64.sve.umax
    1 << 8 | 1, // llvm.aarch64.sve.umax.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.umax.single.x4
    0 << 8 | 1, // llvm.aarch64.sve.umax.u
    1 << 8 | 1, // llvm.aarch64.sve.umax.x2
    1 << 8 | 1, // llvm.aarch64.sve.umax.x4
    0 << 8 | 1, // llvm.aarch64.sve.umaxp
    0 << 8 | 1, // llvm.aarch64.sve.umaxqv
    0 << 8 | 1, // llvm.aarch64.sve.umaxv
    0 << 8 | 1, // llvm.aarch64.sve.umin
    1 << 8 | 1, // llvm.aarch64.sve.umin.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.umin.single.x4
    0 << 8 | 1, // llvm.aarch64.sve.umin.u
    1 << 8 | 1, // llvm.aarch64.sve.umin.x2
    1 << 8 | 1, // llvm.aarch64.sve.umin.x4
    0 << 8 | 1, // llvm.aarch64.sve.uminp
    0 << 8 | 1, // llvm.aarch64.sve.uminqv
    0 << 8 | 1, // llvm.aarch64.sve.uminv
    0 << 8 | 1, // llvm.aarch64.sve.umlalb
    0 << 8 | 35, // llvm.aarch64.sve.umlalb.lane
    0 << 8 | 1, // llvm.aarch64.sve.umlalt
    0 << 8 | 35, // llvm.aarch64.sve.umlalt.lane
    0 << 8 | 1, // llvm.aarch64.sve.umlslb
    0 << 8 | 35, // llvm.aarch64.sve.umlslb.lane
    0 << 8 | 1, // llvm.aarch64.sve.umlslt
    0 << 8 | 35, // llvm.aarch64.sve.umlslt.lane
    0 << 8 | 1, // llvm.aarch64.sve.ummla
    0 << 8 | 1, // llvm.aarch64.sve.umulh
    0 << 8 | 1, // llvm.aarch64.sve.umulh.u
    0 << 8 | 1, // llvm.aarch64.sve.umullb
    0 << 8 | 11, // llvm.aarch64.sve.umullb.lane
    0 << 8 | 1, // llvm.aarch64.sve.umullt
    0 << 8 | 11, // llvm.aarch64.sve.umullt.lane
    0 << 8 | 1, // llvm.aarch64.sve.uqadd
    0 << 8 | 1, // llvm.aarch64.sve.uqadd.x
    1 << 8 | 1, // llvm.aarch64.sve.uqcvt.x2
    1 << 8 | 1, // llvm.aarch64.sve.uqcvt.x4
    1 << 8 | 1, // llvm.aarch64.sve.uqcvtn.x2
    1 << 8 | 1, // llvm.aarch64.sve.uqcvtn.x4
    0 << 8 | 13, // llvm.aarch64.sve.uqdecb.n32
    0 << 8 | 13, // llvm.aarch64.sve.uqdecb.n64
    0 << 8 | 13, // llvm.aarch64.sve.uqdecd
    0 << 8 | 13, // llvm.aarch64.sve.uqdecd.n32
    0 << 8 | 13, // llvm.aarch64.sve.uqdecd.n64
    0 << 8 | 13, // llvm.aarch64.sve.uqdech
    0 << 8 | 13, // llvm.aarch64.sve.uqdech.n32
    0 << 8 | 13, // llvm.aarch64.sve.uqdech.n64
    0 << 8 | 1, // llvm.aarch64.sve.uqdecp
    0 << 8 | 1, // llvm.aarch64.sve.uqdecp.n32
    0 << 8 | 1, // llvm.aarch64.sve.uqdecp.n64
    0 << 8 | 13, // llvm.aarch64.sve.uqdecw
    0 << 8 | 13, // llvm.aarch64.sve.uqdecw.n32
    0 << 8 | 13, // llvm.aarch64.sve.uqdecw.n64
    0 << 8 | 13, // llvm.aarch64.sve.uqincb.n32
    0 << 8 | 13, // llvm.aarch64.sve.uqincb.n64
    0 << 8 | 13, // llvm.aarch64.sve.uqincd
    0 << 8 | 13, // llvm.aarch64.sve.uqincd.n32
    0 << 8 | 13, // llvm.aarch64.sve.uqincd.n64
    0 << 8 | 13, // llvm.aarch64.sve.uqinch
    0 << 8 | 13, // llvm.aarch64.sve.uqinch.n32
    0 << 8 | 13, // llvm.aarch64.sve.uqinch.n64
    0 << 8 | 1, // llvm.aarch64.sve.uqincp
    0 << 8 | 1, // llvm.aarch64.sve.uqincp.n32
    0 << 8 | 1, // llvm.aarch64.sve.uqincp.n64
    0 << 8 | 13, // llvm.aarch64.sve.uqincw
    0 << 8 | 13, // llvm.aarch64.sve.uqincw.n32
    0 << 8 | 13, // llvm.aarch64.sve.uqincw.n64
    0 << 8 | 1, // llvm.aarch64.sve.uqrshl
    0 << 8 | 1, // llvm.aarch64.sve.uqrshl.u
    1 << 8 | 11, // llvm.aarch64.sve.uqrshr.x2
    1 << 8 | 43, // llvm.aarch64.sve.uqrshr.x4
    1 << 8 | 11, // llvm.aarch64.sve.uqrshrn.x2
    1 << 8 | 43, // llvm.aarch64.sve.uqrshrn.x4
    0 << 8 | 0, // llvm.aarch64.sve.uqrshrnb
    0 << 8 | 11, // llvm.aarch64.sve.uqrshrnt
    0 << 8 | 1, // llvm.aarch64.sve.uqshl
    0 << 8 | 1, // llvm.aarch64.sve.uqshl.u
    0 << 8 | 0, // llvm.aarch64.sve.uqshrnb
    0 << 8 | 11, // llvm.aarch64.sve.uqshrnt
    0 << 8 | 1, // llvm.aarch64.sve.uqsub
    0 << 8 | 1, // llvm.aarch64.sve.uqsub.u
    0 << 8 | 1, // llvm.aarch64.sve.uqsub.x
    0 << 8 | 1, // llvm.aarch64.sve.uqsubr
    0 << 8 | 1, // llvm.aarch64.sve.uqxtnb
    0 << 8 | 1, // llvm.aarch64.sve.uqxtnt
    0 << 8 | 1, // llvm.aarch64.sve.urecpe
    0 << 8 | 1, // llvm.aarch64.sve.urhadd
    0 << 8 | 1, // llvm.aarch64.sve.urshl
    1 << 8 | 1, // llvm.aarch64.sve.urshl.single.x2
    1 << 8 | 1, // llvm.aarch64.sve.urshl.single.x4
    0 << 8 | 1, // llvm.aarch64.sve.urshl.u
    1 << 8 | 1, // llvm.aarch64.sve.urshl.x2
    1 << 8 | 1, // llvm.aarch64.sve.urshl.x4
    0 << 8 | 11, // llvm.aarch64.sve.urshr
    0 << 8 | 1, // llvm.aarch64.sve.ursqrte
    0 << 8 | 11, // llvm.aarch64.sve.ursra
    0 << 8 | 1, // llvm.aarch64.sve.usdot
    0 << 8 | 35, // llvm.aarch64.sve.usdot.lane
    0 << 8 | 0, // llvm.aarch64.sve.ushllb
    0 << 8 | 0, // llvm.aarch64.sve.ushllt
    0 << 8 | 1, // llvm.aarch64.sve.usmmla
    0 << 8 | 1, // llvm.aarch64.sve.usqadd
    0 << 8 | 11, // llvm.aarch64.sve.usra
    0 << 8 | 1, // llvm.aarch64.sve.usublb
    0 << 8 | 1, // llvm.aarch64.sve.usublt
    0 << 8 | 1, // llvm.aarch64.sve.usubwb
    0 << 8 | 1, // llvm.aarch64.sve.usubwt
    1 << 8 | 1, // llvm.aarch64.sve.uunpk.x2
    1 << 8 | 1, // llvm.aarch64.sve.uunpk.x4
    0 << 8 | 1, // llvm.aarch64.sve.uunpkhi
    0 << 8 | 1, // llvm.aarch64.sve.uunpklo
    0 << 8 | 1, // llvm.aarch64.sve.uxtb
    0 << 8 | 1, // llvm.aarch64.sve.uxth
    0 << 8 | 1, // llvm.aarch64.sve.uxtw
    1 << 8 | 1, // llvm.aarch64.sve.uzp.x2
    1 << 8 | 1, // llvm.aarch64.sve.uzp.x4
    0 << 8 | 1, // llvm.aarch64.sve.uzp1
    0 << 8 | 1, // llvm.aarch64.sve.uzp1.b16
    0 << 8 | 1, // llvm.aarch64.sve.uzp1.b32
    0 << 8 | 1, // llvm.aarch64.sve.uzp1.b64
    0 << 8 | 1, // llvm.aarch64.sve.uzp1q
    0 << 8 | 1, // llvm.aarch64.sve.uzp2
    0 << 8 | 1, // llvm.aarch64.sve.uzp2.b16
    0 << 8 | 1, // llvm.aarch64.sve.uzp2.b32
    0 << 8 | 1, // llvm.aarch64.sve.uzp2.b64
    0 << 8 | 1, // llvm.aarch64.sve.uzp2q
    1 << 8 | 1, // llvm.aarch64.sve.uzpq.x2
    1 << 8 | 1, // llvm.aarch64.sve.uzpq.x4
    0 << 8 | 1, // llvm.aarch64.sve.uzpq1
    0 << 8 | 1, // llvm.aarch64.sve.uzpq2
    0 << 8 | 1, // llvm.aarch64.sve.whilege
    1 << 8 | 11, // llvm.aarch64.sve.whilege.c16
    1 << 8 | 11, // llvm.aarch64.sve.whilege.c32
    1 << 8 | 11, // llvm.aarch64.sve.whilege.c64
    1 << 8 | 11, // llvm.aarch64.sve.whilege.c8
    1 << 8 | 1, // llvm.aarch64.sve.whilege.x2
    0 << 8 | 1, // llvm.aarch64.sve.whilegt
    1 << 8 | 11, // llvm.aarch64.sve.whilegt.c16
    1 << 8 | 11, // llvm.aarch64.sve.whilegt.c32
    1 << 8 | 11, // llvm.aarch64.sve.whilegt.c64
    1 << 8 | 11, // llvm.aarch64.sve.whilegt.c8
    1 << 8 | 1, // llvm.aarch64.sve.whilegt.x2
    0 << 8 | 1, // llvm.aarch64.sve.whilehi
    1 << 8 | 11, // llvm.aarch64.sve.whilehi.c16
    1 << 8 | 11, // llvm.aarch64.sve.whilehi.c32
    1 << 8 | 11, // llvm.aarch64.sve.whilehi.c64
    1 << 8 | 11, // llvm.aarch64.sve.whilehi.c8
    1 << 8 | 1, // llvm.aarch64.sve.whilehi.x2
    0 << 8 | 1, // llvm.aarch64.sve.whilehs
    1 << 8 | 11, // llvm.aarch64.sve.whilehs.c16
    1 << 8 | 11, // llvm.aarch64.sve.whilehs.c32
    1 << 8 | 11, // llvm.aarch64.sve.whilehs.c64
    1 << 8 | 11, // llvm.aarch64.sve.whilehs.c8
    1 << 8 | 1, // llvm.aarch64.sve.whilehs.x2
    0 << 8 | 1, // llvm.aarch64.sve.whilele
    1 << 8 | 11, // llvm.aarch64.sve.whilele.c16
    1 << 8 | 11, // llvm.aarch64.sve.whilele.c32
    1 << 8 | 11, // llvm.aarch64.sve.whilele.c64
    1 << 8 | 11, // llvm.aarch64.sve.whilele.c8
    1 << 8 | 1, // llvm.aarch64.sve.whilele.x2
    0 << 8 | 1, // llvm.aarch64.sve.whilelo
    1 << 8 | 11, // llvm.aarch64.sve.whilelo.c16
    1 << 8 | 11, // llvm.aarch64.sve.whilelo.c32
    1 << 8 | 11, // llvm.aarch64.sve.whilelo.c64
    1 << 8 | 11, // llvm.aarch64.sve.whilelo.c8
    1 << 8 | 1, // llvm.aarch64.sve.whilelo.x2
    0 << 8 | 1, // llvm.aarch64.sve.whilels
    1 << 8 | 11, // llvm.aarch64.sve.whilels.c16
    1 << 8 | 11, // llvm.aarch64.sve.whilels.c32
    1 << 8 | 11, // llvm.aarch64.sve.whilels.c64
    1 << 8 | 11, // llvm.aarch64.sve.whilels.c8
    1 << 8 | 1, // llvm.aarch64.sve.whilels.x2
    0 << 8 | 1, // llvm.aarch64.sve.whilelt
    1 << 8 | 11, // llvm.aarch64.sve.whilelt.c16
    1 << 8 | 11, // llvm.aarch64.sve.whilelt.c32
    1 << 8 | 11, // llvm.aarch64.sve.whilelt.c64
    1 << 8 | 11, // llvm.aarch64.sve.whilelt.c8
    1 << 8 | 1, // llvm.aarch64.sve.whilelt.x2
    0 << 8 | 1, // llvm.aarch64.sve.whilerw.b
    0 << 8 | 1, // llvm.aarch64.sve.whilerw.d
    0 << 8 | 1, // llvm.aarch64.sve.whilerw.h
    0 << 8 | 1, // llvm.aarch64.sve.whilerw.s
    0 << 8 | 1, // llvm.aarch64.sve.whilewr.b
    0 << 8 | 1, // llvm.aarch64.sve.whilewr.d
    0 << 8 | 1, // llvm.aarch64.sve.whilewr.h
    0 << 8 | 1, // llvm.aarch64.sve.whilewr.s
    6 << 8 | 1, // llvm.aarch64.sve.wrffr
    0 << 8 | 11, // llvm.aarch64.sve.xar
    1 << 8 | 1, // llvm.aarch64.sve.zip.x2
    1 << 8 | 1, // llvm.aarch64.sve.zip.x4
    0 << 8 | 1, // llvm.aarch64.sve.zip1
    0 << 8 | 1, // llvm.aarch64.sve.zip1.b16
    0 << 8 | 1, // llvm.aarch64.sve.zip1.b32
    0 << 8 | 1, // llvm.aarch64.sve.zip1.b64
    0 << 8 | 1, // llvm.aarch64.sve.zip1q
    0 << 8 | 1, // llvm.aarch64.sve.zip2
    0 << 8 | 1, // llvm.aarch64.sve.zip2.b16
    0 << 8 | 1, // llvm.aarch64.sve.zip2.b32
    0 << 8 | 1, // llvm.aarch64.sve.zip2.b64
    0 << 8 | 1, // llvm.aarch64.sve.zip2q
    1 << 8 | 1, // llvm.aarch64.sve.zipq.x2
    1 << 8 | 1, // llvm.aarch64.sve.zipq.x4
    0 << 8 | 1, // llvm.aarch64.sve.zipq1
    0 << 8 | 1, // llvm.aarch64.sve.zipq2
    1 << 8 | 11, // llvm.aarch64.tagp
    1 << 8 | 1, // llvm.aarch64.udiv
    4 << 8 | 35, // llvm.amdgcn.add.max.i32
    4 << 8 | 35, // llvm.amdgcn.add.max.u32
    4 << 8 | 35, // llvm.amdgcn.add.min.i32
    4 << 8 | 35, // llvm.amdgcn.add.min.u32
    0 << 8 | 1, // llvm.amdgcn.addrspacecast.nonnull
    0 << 8 | 1, // llvm.amdgcn.alignbyte
    4 << 8 | 1, // llvm.amdgcn.ashr.pk.i8.i32
    4 << 8 | 1, // llvm.amdgcn.ashr.pk.u8.i32
    39 << 8 | 1, // llvm.amdgcn.ballot
    4 << 8 | 35, // llvm.amdgcn.bitop3
    30 << 8 | 1, // llvm.amdgcn.buffer.wbinvl1
    30 << 8 | 1, // llvm.amdgcn.buffer.wbinvl1.sc
    30 << 8 | 1, // llvm.amdgcn.buffer.wbinvl1.vol
    40 << 8 | 1, // llvm.amdgcn.call.whole.wave
    4 << 8 | 1, // llvm.amdgcn.class
    0 << 8 | 2, // llvm.amdgcn.cluster.id.x
    0 << 8 | 2, // llvm.amdgcn.cluster.id.y
    0 << 8 | 2, // llvm.amdgcn.cluster.id.z
    41 << 8 | 50, // llvm.amdgcn.cluster.load.async.to.lds.b128
    41 << 8 | 50, // llvm.amdgcn.cluster.load.async.to.lds.b32
    41 << 8 | 50, // llvm.amdgcn.cluster.load.async.to.lds.b64
    41 << 8 | 50, // llvm.amdgcn.cluster.load.async.to.lds.b8
    42 << 8 | 51, // llvm.amdgcn.cluster.load.b128
    42 << 8 | 51, // llvm.amdgcn.cluster.load.b32
    42 << 8 | 51, // llvm.amdgcn.cluster.load.b64
    43 << 8 | 1, // llvm.amdgcn.cluster.workgroup.flat.id
    0 << 8 | 2, // llvm.amdgcn.cluster.workgroup.id.x
    0 << 8 | 2, // llvm.amdgcn.cluster.workgroup.id.y
    0 << 8 | 2, // llvm.amdgcn.cluster.workgroup.id.z
    43 << 8 | 1, // llvm.amdgcn.cluster.workgroup.max.flat.id
    0 << 8 | 2, // llvm.amdgcn.cluster.workgroup.max.id.x
    0 << 8 | 2, // llvm.amdgcn.cluster.workgroup.max.id.y
    0 << 8 | 2, // llvm.amdgcn.cluster.workgroup.max.id.z
    44 << 8 | 51, // llvm.amdgcn.cooperative.atomic.load.16x8B
    44 << 8 | 51, // llvm.amdgcn.cooperative.atomic.load.32x4B
    44 << 8 | 51, // llvm.amdgcn.cooperative.atomic.load.8x16B
    45 << 8 | 52, // llvm.amdgcn.cooperative.atomic.store.16x8B
    45 << 8 | 52, // llvm.amdgcn.cooperative.atomic.store.32x4B
    45 << 8 | 52, // llvm.amdgcn.cooperative.atomic.store.8x16B
    4 << 8 | 1, // llvm.amdgcn.cos
    46 << 8 | 43, // llvm.amdgcn.cs.chain
    4 << 8 | 1, // llvm.amdgcn.cubeid
    4 << 8 | 1, // llvm.amdgcn.cubema
    4 << 8 | 1, // llvm.amdgcn.cubesc
    4 << 8 | 1, // llvm.amdgcn.cubetc
    4 << 8 | 0, // llvm.amdgcn.cvt.f16.bf8
    1 << 8 | 0, // llvm.amdgcn.cvt.f16.fp8
    1 << 8 | 0, // llvm.amdgcn.cvt.f32.bf8
    1 << 8 | 0, // llvm.amdgcn.cvt.f32.fp8
    1 << 8 | 0, // llvm.amdgcn.cvt.f32.fp8.e5m3
    4 << 8 | 1, // llvm.amdgcn.cvt.off.f32.i4
    4 << 8 | 1, // llvm.amdgcn.cvt.pk.bf8.f16
    4 << 8 | 35, // llvm.amdgcn.cvt.pk.bf8.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.pk.f16.bf8
    4 << 8 | 1, // llvm.amdgcn.cvt.pk.f16.fp8
    4 << 8 | 0, // llvm.amdgcn.cvt.pk.f32.bf8
    4 << 8 | 0, // llvm.amdgcn.cvt.pk.f32.fp8
    4 << 8 | 1, // llvm.amdgcn.cvt.pk.fp8.f16
    4 << 8 | 35, // llvm.amdgcn.cvt.pk.fp8.f32
    4 << 8 | 35, // llvm.amdgcn.cvt.pk.fp8.f32.e5m3
    4 << 8 | 1, // llvm.amdgcn.cvt.pk.i16
    4 << 8 | 1, // llvm.amdgcn.cvt.pk.u16
    4 << 8 | 1, // llvm.amdgcn.cvt.pk.u8.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.pknorm.i16
    4 << 8 | 1, // llvm.amdgcn.cvt.pknorm.u16
    4 << 8 | 1, // llvm.amdgcn.cvt.pkrtz
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk16.bf16.bf6
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk16.bf16.fp6
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk16.f16.bf6
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk16.f16.fp6
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk16.f32.bf6
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk16.f32.fp6
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.bf16.bf8
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.bf16.fp4
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.bf16.fp8
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.f16.bf8
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.f16.fp4
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.f16.fp8
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.f32.bf8
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.f32.fp4
    47 << 8 | 53, // llvm.amdgcn.cvt.scale.pk8.f32.fp8
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.2xpk16.bf6.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.2xpk16.fp6.f32
    4 << 8 | 54, // llvm.amdgcn.cvt.scalef32.f16.bf8
    4 << 8 | 54, // llvm.amdgcn.cvt.scalef32.f16.fp8
    4 << 8 | 55, // llvm.amdgcn.cvt.scalef32.f32.bf8
    4 << 8 | 55, // llvm.amdgcn.cvt.scalef32.f32.fp8
    4 << 8 | 11, // llvm.amdgcn.cvt.scalef32.pk.bf16.bf8
    4 << 8 | 55, // llvm.amdgcn.cvt.scalef32.pk.bf16.fp4
    4 << 8 | 11, // llvm.amdgcn.cvt.scalef32.pk.bf16.fp8
    4 << 8 | 35, // llvm.amdgcn.cvt.scalef32.pk.bf8.bf16
    4 << 8 | 35, // llvm.amdgcn.cvt.scalef32.pk.bf8.f16
    4 << 8 | 43, // llvm.amdgcn.cvt.scalef32.pk.bf8.f32
    4 << 8 | 11, // llvm.amdgcn.cvt.scalef32.pk.f16.bf8
    4 << 8 | 55, // llvm.amdgcn.cvt.scalef32.pk.f16.fp4
    4 << 8 | 11, // llvm.amdgcn.cvt.scalef32.pk.f16.fp8
    4 << 8 | 11, // llvm.amdgcn.cvt.scalef32.pk.f32.bf8
    4 << 8 | 55, // llvm.amdgcn.cvt.scalef32.pk.f32.fp4
    4 << 8 | 11, // llvm.amdgcn.cvt.scalef32.pk.f32.fp8
    4 << 8 | 56, // llvm.amdgcn.cvt.scalef32.pk.fp4.bf16
    4 << 8 | 56, // llvm.amdgcn.cvt.scalef32.pk.fp4.f16
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.pk.fp4.f32
    4 << 8 | 35, // llvm.amdgcn.cvt.scalef32.pk.fp8.bf16
    4 << 8 | 35, // llvm.amdgcn.cvt.scalef32.pk.fp8.f16
    4 << 8 | 43, // llvm.amdgcn.cvt.scalef32.pk.fp8.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk16.bf6.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk16.bf6.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk16.bf6.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk16.fp6.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk16.fp6.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk16.fp6.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.bf16.bf6
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.bf16.fp6
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.bf6.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.bf6.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.f16.bf6
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.f16.fp6
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.f32.bf6
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.f32.fp6
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.fp6.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk32.fp6.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.bf8.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.bf8.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.bf8.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.fp4.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.fp4.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.fp4.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.fp8.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.fp8.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.pk8.fp8.f32
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.bf8.bf16
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.bf8.f16
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.bf8.f32
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.fp8.bf16
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.fp8.f16
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.fp8.f32
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.pk.fp4.bf16
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.pk.fp4.f16
    4 << 8 | 57, // llvm.amdgcn.cvt.scalef32.sr.pk.fp4.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk16.bf6.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk16.fp6.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk32.bf6.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk32.fp6.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.bf8.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp4.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.bf16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.f16
    4 << 8 | 1, // llvm.amdgcn.cvt.scalef32.sr.pk8.fp8.f32
    4 << 8 | 35, // llvm.amdgcn.cvt.sr.bf16.f32
    4 << 8 | 56, // llvm.amdgcn.cvt.sr.bf8.f16
    4 << 8 | 35, // llvm.amdgcn.cvt.sr.bf8.f32
    4 << 8 | 35, // llvm.amdgcn.cvt.sr.f16.f32
    4 << 8 | 56, // llvm.amdgcn.cvt.sr.fp8.f16
    4 << 8 | 35, // llvm.amdgcn.cvt.sr.fp8.f32
    4 << 8 | 35, // llvm.amdgcn.cvt.sr.fp8.f32.e5m3
    4 << 8 | 1, // llvm.amdgcn.cvt.sr.pk.bf16.f32
    4 << 8 | 1, // llvm.amdgcn.cvt.sr.pk.f16.f32
    1 << 8 | 1, // llvm.amdgcn.dead
    0 << 8 | 2, // llvm.amdgcn.dispatch.id
    0 << 8 | 58, // llvm.amdgcn.dispatch.ptr
    4 << 8 | 1, // llvm.amdgcn.div.fixup
    4 << 8 | 1, // llvm.amdgcn.div.fmas
    4 << 8 | 11, // llvm.amdgcn.div.scale
    4 << 8 | 1, // llvm.amdgcn.dot4.f32.bf8.bf8
    4 << 8 | 1, // llvm.amdgcn.dot4.f32.bf8.fp8
    4 << 8 | 1, // llvm.amdgcn.dot4.f32.fp8.bf8
    4 << 8 | 1, // llvm.amdgcn.dot4.f32.fp8.fp8
    48 << 8 | 0, // llvm.amdgcn.ds.add.gs.reg.rtn
    49 << 8 | 59, // llvm.amdgcn.ds.append
    50 << 8 | 1, // llvm.amdgcn.ds.atomic.async.barrier.arrive.b64
    51 << 8 | 10, // llvm.amdgcn.ds.atomic.barrier.arrive.rtn.b64
    52 << 8 | 1, // llvm.amdgcn.ds.bpermute
    52 << 8 | 1, // llvm.amdgcn.ds.bpermute.fi.b32
    53 << 8 | 35, // llvm.amdgcn.ds.bvh.stack.push4.pop1.rtn
    53 << 8 | 35, // llvm.amdgcn.ds.bvh.stack.push8.pop1.rtn
    53 << 8 | 35, // llvm.amdgcn.ds.bvh.stack.push8.pop2.rtn
    53 << 8 | 35, // llvm.amdgcn.ds.bvh.stack.rtn
    49 << 8 | 59, // llvm.amdgcn.ds.consume
    54 << 8 | 1, // llvm.amdgcn.ds.gws.barrier
    55 << 8 | 1, // llvm.amdgcn.ds.gws.init
    54 << 8 | 1, // llvm.amdgcn.ds.gws.sema.br
    54 << 8 | 1, // llvm.amdgcn.ds.gws.sema.p
    54 << 8 | 1, // llvm.amdgcn.ds.gws.sema.release.all
    54 << 8 | 1, // llvm.amdgcn.ds.gws.sema.v
    42 << 8 | 10, // llvm.amdgcn.ds.load.tr16.b128
    42 << 8 | 10, // llvm.amdgcn.ds.load.tr4.b64
    42 << 8 | 10, // llvm.amdgcn.ds.load.tr6.b96
    42 << 8 | 10, // llvm.amdgcn.ds.load.tr8.b64
    53 << 8 | 60, // llvm.amdgcn.ds.ordered.add
    53 << 8 | 60, // llvm.amdgcn.ds.ordered.swap
    52 << 8 | 1, // llvm.amdgcn.ds.permute
    42 << 8 | 10, // llvm.amdgcn.ds.read.tr16.b64
    42 << 8 | 10, // llvm.amdgcn.ds.read.tr4.b64
    42 << 8 | 10, // llvm.amdgcn.ds.read.tr6.b96
    42 << 8 | 10, // llvm.amdgcn.ds.read.tr8.b64
    48 << 8 | 0, // llvm.amdgcn.ds.sub.gs.reg.rtn
    52 << 8 | 0, // llvm.amdgcn.ds.swizzle
    53 << 8 | 1, // llvm.amdgcn.else
    53 << 8 | 1, // llvm.amdgcn.end.cf
    56 << 8 | 1, // llvm.amdgcn.endpgm
    6 << 8 | 61, // llvm.amdgcn.exp
    6 << 8 | 62, // llvm.amdgcn.exp.compr
    6 << 8 | 63, // llvm.amdgcn.exp.row
    4 << 8 | 1, // llvm.amdgcn.exp2
    52 << 8 | 11, // llvm.amdgcn.fcmp
    4 << 8 | 1, // llvm.amdgcn.fdiv.fast
    4 << 8 | 35, // llvm.amdgcn.fdot2
    4 << 8 | 1, // llvm.amdgcn.fdot2.bf16.bf16
    4 << 8 | 1, // llvm.amdgcn.fdot2.f16.f16
    4 << 8 | 35, // llvm.amdgcn.fdot2.f32.bf16
    4 << 8 | 35, // llvm.amdgcn.fdot2c.f32.bf16
    25 << 8 | 10, // llvm.amdgcn.flat.atomic.fmax.num
    25 << 8 | 10, // llvm.amdgcn.flat.atomic.fmin.num
    42 << 8 | 51, // llvm.amdgcn.flat.load.monitor.b128
    42 << 8 | 51, // llvm.amdgcn.flat.load.monitor.b32
    42 << 8 | 51, // llvm.amdgcn.flat.load.monitor.b64
    41 << 8 | 64, // llvm.amdgcn.flat.prefetch
    4 << 8 | 1, // llvm.amdgcn.fma.legacy
    4 << 8 | 1, // llvm.amdgcn.fmad.ftz
    4 << 8 | 1, // llvm.amdgcn.fmed3
    4 << 8 | 1, // llvm.amdgcn.fmul.legacy
    4 << 8 | 1, // llvm.amdgcn.fract
    4 << 8 | 1, // llvm.amdgcn.frexp.exp
    4 << 8 | 1, // llvm.amdgcn.frexp.mant
    25 << 8 | 10, // llvm.amdgcn.global.atomic.fmax.num
    25 << 8 | 10, // llvm.amdgcn.global.atomic.fmin.num
    25 << 8 | 10, // llvm.amdgcn.global.atomic.ordered.add.b64
    41 << 8 | 50, // llvm.amdgcn.global.load.async.to.lds.b128
    41 << 8 | 50, // llvm.amdgcn.global.load.async.to.lds.b32
    41 << 8 | 50, // llvm.amdgcn.global.load.async.to.lds.b64
    41 << 8 | 50, // llvm.amdgcn.global.load.async.to.lds.b8
    53 << 8 | 65, // llvm.amdgcn.global.load.lds
    42 << 8 | 51, // llvm.amdgcn.global.load.monitor.b128
    42 << 8 | 51, // llvm.amdgcn.global.load.monitor.b32
    42 << 8 | 51, // llvm.amdgcn.global.load.monitor.b64
    42 << 8 | 10, // llvm.amdgcn.global.load.tr.b128
    42 << 8 | 10, // llvm.amdgcn.global.load.tr.b64
    42 << 8 | 10, // llvm.amdgcn.global.load.tr4.b64
    42 << 8 | 10, // llvm.amdgcn.global.load.tr6.b96
    41 << 8 | 64, // llvm.amdgcn.global.prefetch
    41 << 8 | 66, // llvm.amdgcn.global.store.async.from.lds.b128
    41 << 8 | 66, // llvm.amdgcn.global.store.async.from.lds.b32
    41 << 8 | 66, // llvm.amdgcn.global.store.async.from.lds.b64
    41 << 8 | 66, // llvm.amdgcn.global.store.async.from.lds.b8
    0 << 8 | 2, // llvm.amdgcn.groupstaticsize
    52 << 8 | 11, // llvm.amdgcn.icmp
    53 << 8 | 1, // llvm.amdgcn.if
    57 << 8 | 1, // llvm.amdgcn.if.break
    58 << 8 | 5, // llvm.amdgcn.iglp.opt
    53 << 8 | 48, // llvm.amdgcn.image.atomic.add.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.add.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.add.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.add.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.add.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.add.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.add.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.add.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.add.flt.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.add.flt.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.add.flt.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.add.flt.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.add.flt.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.add.flt.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.add.flt.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.add.flt.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.and.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.and.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.and.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.and.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.and.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.and.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.and.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.and.cube
    53 << 8 | 67, // llvm.amdgcn.image.atomic.cmpswap.1d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.cmpswap.1darray
    53 << 8 | 68, // llvm.amdgcn.image.atomic.cmpswap.2d
    53 << 8 | 69, // llvm.amdgcn.image.atomic.cmpswap.2darray
    53 << 8 | 70, // llvm.amdgcn.image.atomic.cmpswap.2darraymsaa
    53 << 8 | 69, // llvm.amdgcn.image.atomic.cmpswap.2dmsaa
    53 << 8 | 69, // llvm.amdgcn.image.atomic.cmpswap.3d
    53 << 8 | 69, // llvm.amdgcn.image.atomic.cmpswap.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.dec.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.dec.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.dec.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.dec.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.dec.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.dec.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.dec.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.dec.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.fmax.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.fmax.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.fmax.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.fmax.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.fmax.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.fmax.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.fmax.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.fmax.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.fmin.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.fmin.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.fmin.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.fmin.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.fmin.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.fmin.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.fmin.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.fmin.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.inc.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.inc.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.inc.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.inc.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.inc.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.inc.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.inc.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.inc.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.max.flt.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.max.flt.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.max.flt.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.max.flt.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.max.flt.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.max.flt.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.max.flt.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.max.flt.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.min.flt.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.min.flt.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.min.flt.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.min.flt.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.min.flt.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.min.flt.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.min.flt.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.min.flt.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.or.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.or.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.or.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.or.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.or.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.or.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.or.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.or.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.pk.add.bf16.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.pk.add.bf16.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.pk.add.bf16.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.pk.add.bf16.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.pk.add.bf16.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.pk.add.bf16.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.pk.add.bf16.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.pk.add.bf16.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.pk.add.f16.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.pk.add.f16.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.pk.add.f16.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.pk.add.f16.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.pk.add.f16.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.pk.add.f16.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.pk.add.f16.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.pk.add.f16.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.smax.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.smax.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.smax.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.smax.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.smax.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.smax.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.smax.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.smax.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.smin.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.smin.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.smin.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.smin.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.smin.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.smin.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.smin.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.smin.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.sub.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.sub.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.sub.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.sub.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.sub.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.sub.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.sub.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.sub.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.swap.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.swap.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.swap.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.swap.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.swap.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.swap.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.swap.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.swap.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.umax.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.umax.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.umax.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.umax.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.umax.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.umax.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.umax.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.umax.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.umin.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.umin.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.umin.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.umin.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.umin.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.umin.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.umin.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.umin.cube
    53 << 8 | 48, // llvm.amdgcn.image.atomic.xor.1d
    53 << 8 | 67, // llvm.amdgcn.image.atomic.xor.1darray
    53 << 8 | 67, // llvm.amdgcn.image.atomic.xor.2d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.xor.2darray
    53 << 8 | 69, // llvm.amdgcn.image.atomic.xor.2darraymsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.xor.2dmsaa
    53 << 8 | 68, // llvm.amdgcn.image.atomic.xor.3d
    53 << 8 | 68, // llvm.amdgcn.image.atomic.xor.cube
    59 << 8 | 1, // llvm.amdgcn.image.bvh.dual.intersect.ray
    23 << 8 | 1, // llvm.amdgcn.image.bvh.intersect.ray
    59 << 8 | 1, // llvm.amdgcn.image.bvh8.intersect.ray
    23 << 8 | 71, // llvm.amdgcn.image.gather4.2d
    23 << 8 | 72, // llvm.amdgcn.image.gather4.2darray
    23 << 8 | 72, // llvm.amdgcn.image.gather4.b.2d
    23 << 8 | 73, // llvm.amdgcn.image.gather4.b.2darray
    23 << 8 | 73, // llvm.amdgcn.image.gather4.b.cl.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.b.cl.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.b.cl.cube
    23 << 8 | 74, // llvm.amdgcn.image.gather4.b.cl.o.2d
    23 << 8 | 75, // llvm.amdgcn.image.gather4.b.cl.o.2darray
    23 << 8 | 75, // llvm.amdgcn.image.gather4.b.cl.o.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.b.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.b.o.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.b.o.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.b.o.cube
    23 << 8 | 72, // llvm.amdgcn.image.gather4.c.2d
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.2darray
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.b.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.b.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.b.cl.2d
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.b.cl.2darray
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.b.cl.cube
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.b.cl.o.2d
    23 << 8 | 76, // llvm.amdgcn.image.gather4.c.b.cl.o.2darray
    23 << 8 | 76, // llvm.amdgcn.image.gather4.c.b.cl.o.cube
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.b.cube
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.b.o.2d
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.b.o.2darray
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.b.o.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.cl.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.cl.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.cl.cube
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.cl.o.2d
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.cl.o.2darray
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.cl.o.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.l.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.l.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.l.cube
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.l.o.2d
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.l.o.2darray
    23 << 8 | 75, // llvm.amdgcn.image.gather4.c.l.o.cube
    23 << 8 | 72, // llvm.amdgcn.image.gather4.c.lz.2d
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.lz.2darray
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.lz.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.lz.o.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.lz.o.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.lz.o.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.c.o.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.o.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.c.o.cube
    23 << 8 | 72, // llvm.amdgcn.image.gather4.cl.2d
    23 << 8 | 73, // llvm.amdgcn.image.gather4.cl.2darray
    23 << 8 | 73, // llvm.amdgcn.image.gather4.cl.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.cl.o.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.cl.o.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.cl.o.cube
    23 << 8 | 72, // llvm.amdgcn.image.gather4.cube
    23 << 8 | 72, // llvm.amdgcn.image.gather4.l.2d
    23 << 8 | 73, // llvm.amdgcn.image.gather4.l.2darray
    23 << 8 | 73, // llvm.amdgcn.image.gather4.l.cube
    23 << 8 | 73, // llvm.amdgcn.image.gather4.l.o.2d
    23 << 8 | 74, // llvm.amdgcn.image.gather4.l.o.2darray
    23 << 8 | 74, // llvm.amdgcn.image.gather4.l.o.cube
    23 << 8 | 71, // llvm.amdgcn.image.gather4.lz.2d
    23 << 8 | 72, // llvm.amdgcn.image.gather4.lz.2darray
    23 << 8 | 72, // llvm.amdgcn.image.gather4.lz.cube
    23 << 8 | 72, // llvm.amdgcn.image.gather4.lz.o.2d
    23 << 8 | 73, // llvm.amdgcn.image.gather4.lz.o.2darray
    23 << 8 | 73, // llvm.amdgcn.image.gather4.lz.o.cube
    23 << 8 | 72, // llvm.amdgcn.image.gather4.o.2d
    23 << 8 | 73, // llvm.amdgcn.image.gather4.o.2darray
    23 << 8 | 73, // llvm.amdgcn.image.gather4.o.cube
    15 << 8 | 77, // llvm.amdgcn.image.getlod.1d
    15 << 8 | 71, // llvm.amdgcn.image.getlod.1darray
    15 << 8 | 71, // llvm.amdgcn.image.getlod.2d
    15 << 8 | 72, // llvm.amdgcn.image.getlod.2darray
    15 << 8 | 72, // llvm.amdgcn.image.getlod.3d
    15 << 8 | 72, // llvm.amdgcn.image.getlod.cube
    1 << 8 | 78, // llvm.amdgcn.image.getresinfo.1d
    1 << 8 | 78, // llvm.amdgcn.image.getresinfo.1darray
    1 << 8 | 78, // llvm.amdgcn.image.getresinfo.2d
    1 << 8 | 78, // llvm.amdgcn.image.getresinfo.2darray
    1 << 8 | 78, // llvm.amdgcn.image.getresinfo.2darraymsaa
    1 << 8 | 78, // llvm.amdgcn.image.getresinfo.2dmsaa
    1 << 8 | 78, // llvm.amdgcn.image.getresinfo.3d
    1 << 8 | 78, // llvm.amdgcn.image.getresinfo.cube
    23 << 8 | 78, // llvm.amdgcn.image.load.1d
    23 << 8 | 79, // llvm.amdgcn.image.load.1darray
    23 << 8 | 79, // llvm.amdgcn.image.load.2d
    23 << 8 | 80, // llvm.amdgcn.image.load.2darray
    23 << 8 | 81, // llvm.amdgcn.image.load.2darraymsaa
    23 << 8 | 80, // llvm.amdgcn.image.load.2dmsaa
    23 << 8 | 80, // llvm.amdgcn.image.load.3d
    23 << 8 | 80, // llvm.amdgcn.image.load.cube
    23 << 8 | 79, // llvm.amdgcn.image.load.mip.1d
    23 << 8 | 80, // llvm.amdgcn.image.load.mip.1darray
    23 << 8 | 80, // llvm.amdgcn.image.load.mip.2d
    23 << 8 | 81, // llvm.amdgcn.image.load.mip.2darray
    23 << 8 | 81, // llvm.amdgcn.image.load.mip.3d
    23 << 8 | 81, // llvm.amdgcn.image.load.mip.cube
    23 << 8 | 81, // llvm.amdgcn.image.msaa.load.2darraymsaa
    23 << 8 | 80, // llvm.amdgcn.image.msaa.load.2dmsaa
    23 << 8 | 81, // llvm.amdgcn.image.msaa.load.x.2darraymsaa
    23 << 8 | 80, // llvm.amdgcn.image.msaa.load.x.2dmsaa
    60 << 8 | 77, // llvm.amdgcn.image.sample.1d
    61 << 8 | 77, // llvm.amdgcn.image.sample.1d.nortn
    60 << 8 | 71, // llvm.amdgcn.image.sample.1darray
    61 << 8 | 71, // llvm.amdgcn.image.sample.1darray.nortn
    60 << 8 | 71, // llvm.amdgcn.image.sample.2d
    61 << 8 | 71, // llvm.amdgcn.image.sample.2d.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.2darray
    61 << 8 | 72, // llvm.amdgcn.image.sample.2darray.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.3d
    61 << 8 | 72, // llvm.amdgcn.image.sample.3d.nortn
    60 << 8 | 71, // llvm.amdgcn.image.sample.b.1d
    61 << 8 | 71, // llvm.amdgcn.image.sample.b.1d.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.b.1darray
    61 << 8 | 72, // llvm.amdgcn.image.sample.b.1darray.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.b.2d
    61 << 8 | 72, // llvm.amdgcn.image.sample.b.2d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.b.2darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.b.2darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.b.3d
    61 << 8 | 73, // llvm.amdgcn.image.sample.b.3d.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.b.cl.1d
    61 << 8 | 72, // llvm.amdgcn.image.sample.b.cl.1d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.b.cl.1darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.b.cl.1darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.b.cl.2d
    61 << 8 | 73, // llvm.amdgcn.image.sample.b.cl.2d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.2darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.2darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.3d
    61 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.3d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.cube
    61 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.cube.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.b.cl.o.1d
    61 << 8 | 73, // llvm.amdgcn.image.sample.b.cl.o.1d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.o.1darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.o.1darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.o.2d
    61 << 8 | 74, // llvm.amdgcn.image.sample.b.cl.o.2d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.b.cl.o.2darray
    61 << 8 | 75, // llvm.amdgcn.image.sample.b.cl.o.2darray.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.b.cl.o.3d
    61 << 8 | 75, // llvm.amdgcn.image.sample.b.cl.o.3d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.b.cl.o.cube
    61 << 8 | 75, // llvm.amdgcn.image.sample.b.cl.o.cube.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.b.cube
    61 << 8 | 73, // llvm.amdgcn.image.sample.b.cube.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.b.o.1d
    61 << 8 | 72, // llvm.amdgcn.image.sample.b.o.1d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.b.o.1darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.b.o.1darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.b.o.2d
    61 << 8 | 73, // llvm.amdgcn.image.sample.b.o.2d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.b.o.2darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.b.o.2darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.b.o.3d
    61 << 8 | 74, // llvm.amdgcn.image.sample.b.o.3d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.b.o.cube
    61 << 8 | 74, // llvm.amdgcn.image.sample.b.o.cube.nortn
    60 << 8 | 71, // llvm.amdgcn.image.sample.c.1d
    61 << 8 | 71, // llvm.amdgcn.image.sample.c.1d.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.c.1darray
    61 << 8 | 72, // llvm.amdgcn.image.sample.c.1darray.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.c.2d
    61 << 8 | 72, // llvm.amdgcn.image.sample.c.2d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.2darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.2darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.3d
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.3d.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.c.b.1d
    61 << 8 | 72, // llvm.amdgcn.image.sample.c.b.1d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.b.1darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.b.1darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.b.2d
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.b.2d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.b.2darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.b.2darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.b.3d
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.b.3d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.b.cl.1d
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.b.cl.1d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.b.cl.1darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.b.cl.1darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.b.cl.2d
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.b.cl.2d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.2darray
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.2darray.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.3d
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.3d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.cube
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.cube.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.b.cl.o.1d
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.b.cl.o.1d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.o.1darray
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.o.1darray.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.o.2d
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.b.cl.o.2d.nortn
    60 << 8 | 76, // llvm.amdgcn.image.sample.c.b.cl.o.2darray
    61 << 8 | 76, // llvm.amdgcn.image.sample.c.b.cl.o.2darray.nortn
    60 << 8 | 76, // llvm.amdgcn.image.sample.c.b.cl.o.3d
    61 << 8 | 76, // llvm.amdgcn.image.sample.c.b.cl.o.3d.nortn
    60 << 8 | 76, // llvm.amdgcn.image.sample.c.b.cl.o.cube
    61 << 8 | 76, // llvm.amdgcn.image.sample.c.b.cl.o.cube.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.b.cube
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.b.cube.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.b.o.1d
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.b.o.1d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.b.o.1darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.b.o.1darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.b.o.2d
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.b.o.2d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.b.o.2darray
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.b.o.2darray.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.b.o.3d
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.b.o.3d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.b.o.cube
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.b.o.cube.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.cd.1d
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.cd.1d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.cd.1darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.cd.1darray.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.c.cd.2d
    7 << 8 | 76, // llvm.amdgcn.image.sample.c.cd.2d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.c.cd.2darray
    7 << 8 | 82, // llvm.amdgcn.image.sample.c.cd.2darray.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.c.cd.3d
    7 << 8 | 83, // llvm.amdgcn.image.sample.c.cd.3d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.cd.cl.1d
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.cd.cl.1d.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.cd.cl.1darray
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.cd.cl.1darray.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.c.cd.cl.2d
    7 << 8 | 82, // llvm.amdgcn.image.sample.c.cd.cl.2d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.cl.2darray
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.cl.2darray.nortn
    23 << 8 | 85, // llvm.amdgcn.image.sample.c.cd.cl.3d
    7 << 8 | 85, // llvm.amdgcn.image.sample.c.cd.cl.3d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.cl.cube
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.cl.cube.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.cd.cl.o.1d
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.cd.cl.o.1d.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.c.cd.cl.o.1darray
    7 << 8 | 76, // llvm.amdgcn.image.sample.c.cd.cl.o.1darray.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.cl.o.2d
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.cl.o.2d.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.c.cd.cl.o.2darray
    7 << 8 | 83, // llvm.amdgcn.image.sample.c.cd.cl.o.2darray.nortn
    23 << 8 | 86, // llvm.amdgcn.image.sample.c.cd.cl.o.3d
    7 << 8 | 86, // llvm.amdgcn.image.sample.c.cd.cl.o.3d.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.c.cd.cl.o.cube
    7 << 8 | 83, // llvm.amdgcn.image.sample.c.cd.cl.o.cube.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.c.cd.cube
    7 << 8 | 82, // llvm.amdgcn.image.sample.c.cd.cube.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.cd.o.1d
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.cd.o.1d.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.cd.o.1darray
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.cd.o.1darray.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.c.cd.o.2d
    7 << 8 | 82, // llvm.amdgcn.image.sample.c.cd.o.2d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.o.2darray
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.o.2darray.nortn
    23 << 8 | 85, // llvm.amdgcn.image.sample.c.cd.o.3d
    7 << 8 | 85, // llvm.amdgcn.image.sample.c.cd.o.3d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.o.cube
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.cd.o.cube.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.c.cl.1d
    61 << 8 | 72, // llvm.amdgcn.image.sample.c.cl.1d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.cl.1darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.cl.1darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.cl.2d
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.cl.2d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.2darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.2darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.3d
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.3d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.cube
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.cube.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.cl.o.1d
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.cl.o.1d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.o.1darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.o.1darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.o.2d
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.cl.o.2d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.cl.o.2darray
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.cl.o.2darray.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.cl.o.3d
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.cl.o.3d.nortn
    60 << 8 | 75, // llvm.amdgcn.image.sample.c.cl.o.cube
    61 << 8 | 75, // llvm.amdgcn.image.sample.c.cl.o.cube.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.cube
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.cube.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.d.1d
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.d.1d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.d.1darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.d.1darray.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.c.d.2d
    7 << 8 | 76, // llvm.amdgcn.image.sample.c.d.2d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.c.d.2darray
    7 << 8 | 82, // llvm.amdgcn.image.sample.c.d.2darray.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.c.d.3d
    7 << 8 | 83, // llvm.amdgcn.image.sample.c.d.3d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.d.cl.1d
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.d.cl.1d.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.d.cl.1darray
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.d.cl.1darray.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.c.d.cl.2d
    7 << 8 | 82, // llvm.amdgcn.image.sample.c.d.cl.2d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.d.cl.2darray
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.d.cl.2darray.nortn
    23 << 8 | 85, // llvm.amdgcn.image.sample.c.d.cl.3d
    7 << 8 | 85, // llvm.amdgcn.image.sample.c.d.cl.3d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.d.cl.cube
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.d.cl.cube.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.d.cl.o.1d
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.d.cl.o.1d.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.c.d.cl.o.1darray
    7 << 8 | 76, // llvm.amdgcn.image.sample.c.d.cl.o.1darray.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.d.cl.o.2d
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.d.cl.o.2d.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.c.d.cl.o.2darray
    7 << 8 | 83, // llvm.amdgcn.image.sample.c.d.cl.o.2darray.nortn
    23 << 8 | 86, // llvm.amdgcn.image.sample.c.d.cl.o.3d
    7 << 8 | 86, // llvm.amdgcn.image.sample.c.d.cl.o.3d.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.c.d.cl.o.cube
    7 << 8 | 83, // llvm.amdgcn.image.sample.c.d.cl.o.cube.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.c.d.cube
    7 << 8 | 82, // llvm.amdgcn.image.sample.c.d.cube.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.d.o.1d
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.d.o.1d.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.d.o.1darray
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.d.o.1darray.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.c.d.o.2d
    7 << 8 | 82, // llvm.amdgcn.image.sample.c.d.o.2d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.d.o.2darray
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.d.o.2darray.nortn
    23 << 8 | 85, // llvm.amdgcn.image.sample.c.d.o.3d
    7 << 8 | 85, // llvm.amdgcn.image.sample.c.d.o.3d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.c.d.o.cube
    7 << 8 | 84, // llvm.amdgcn.image.sample.c.d.o.cube.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.c.l.1d
    7 << 8 | 72, // llvm.amdgcn.image.sample.c.l.1d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.l.1darray
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.l.1darray.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.l.2d
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.l.2d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.l.2darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.l.2darray.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.l.3d
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.l.3d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.l.cube
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.l.cube.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.l.o.1d
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.l.o.1d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.l.o.1darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.l.o.1darray.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.l.o.2d
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.l.o.2d.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.l.o.2darray
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.l.o.2darray.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.l.o.3d
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.l.o.3d.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.c.l.o.cube
    7 << 8 | 75, // llvm.amdgcn.image.sample.c.l.o.cube.nortn
    23 << 8 | 71, // llvm.amdgcn.image.sample.c.lz.1d
    7 << 8 | 71, // llvm.amdgcn.image.sample.c.lz.1d.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.c.lz.1darray
    7 << 8 | 72, // llvm.amdgcn.image.sample.c.lz.1darray.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.c.lz.2d
    7 << 8 | 72, // llvm.amdgcn.image.sample.c.lz.2d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.2darray
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.2darray.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.3d
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.3d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.cube
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.cube.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.c.lz.o.1d
    7 << 8 | 72, // llvm.amdgcn.image.sample.c.lz.o.1d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.o.1darray
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.o.1darray.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.o.2d
    7 << 8 | 73, // llvm.amdgcn.image.sample.c.lz.o.2d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.lz.o.2darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.lz.o.2darray.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.lz.o.3d
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.lz.o.3d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.c.lz.o.cube
    7 << 8 | 74, // llvm.amdgcn.image.sample.c.lz.o.cube.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.c.o.1d
    61 << 8 | 72, // llvm.amdgcn.image.sample.c.o.1d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.o.1darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.o.1darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.c.o.2d
    61 << 8 | 73, // llvm.amdgcn.image.sample.c.o.2d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.o.2darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.o.2darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.o.3d
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.o.3d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.c.o.cube
    61 << 8 | 74, // llvm.amdgcn.image.sample.c.o.cube.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.cd.1d
    7 << 8 | 72, // llvm.amdgcn.image.sample.cd.1d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.cd.1darray
    7 << 8 | 73, // llvm.amdgcn.image.sample.cd.1darray.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.cd.2d
    7 << 8 | 75, // llvm.amdgcn.image.sample.cd.2d.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.cd.2darray
    7 << 8 | 76, // llvm.amdgcn.image.sample.cd.2darray.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.cd.3d
    7 << 8 | 84, // llvm.amdgcn.image.sample.cd.3d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.cd.cl.1d
    7 << 8 | 73, // llvm.amdgcn.image.sample.cd.cl.1d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.cd.cl.1darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.cd.cl.1darray.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.cd.cl.2d
    7 << 8 | 76, // llvm.amdgcn.image.sample.cd.cl.2d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.cd.cl.2darray
    7 << 8 | 82, // llvm.amdgcn.image.sample.cd.cl.2darray.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.cd.cl.3d
    7 << 8 | 83, // llvm.amdgcn.image.sample.cd.cl.3d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.cd.cl.cube
    7 << 8 | 82, // llvm.amdgcn.image.sample.cd.cl.cube.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.cd.cl.o.1d
    7 << 8 | 74, // llvm.amdgcn.image.sample.cd.cl.o.1d.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.cd.cl.o.1darray
    7 << 8 | 75, // llvm.amdgcn.image.sample.cd.cl.o.1darray.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.cd.cl.o.2d
    7 << 8 | 82, // llvm.amdgcn.image.sample.cd.cl.o.2d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.cd.cl.o.2darray
    7 << 8 | 84, // llvm.amdgcn.image.sample.cd.cl.o.2darray.nortn
    23 << 8 | 85, // llvm.amdgcn.image.sample.cd.cl.o.3d
    7 << 8 | 85, // llvm.amdgcn.image.sample.cd.cl.o.3d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.cd.cl.o.cube
    7 << 8 | 84, // llvm.amdgcn.image.sample.cd.cl.o.cube.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.cd.cube
    7 << 8 | 76, // llvm.amdgcn.image.sample.cd.cube.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.cd.o.1d
    7 << 8 | 73, // llvm.amdgcn.image.sample.cd.o.1d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.cd.o.1darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.cd.o.1darray.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.cd.o.2d
    7 << 8 | 76, // llvm.amdgcn.image.sample.cd.o.2d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.cd.o.2darray
    7 << 8 | 82, // llvm.amdgcn.image.sample.cd.o.2darray.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.cd.o.3d
    7 << 8 | 83, // llvm.amdgcn.image.sample.cd.o.3d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.cd.o.cube
    7 << 8 | 82, // llvm.amdgcn.image.sample.cd.o.cube.nortn
    60 << 8 | 71, // llvm.amdgcn.image.sample.cl.1d
    61 << 8 | 71, // llvm.amdgcn.image.sample.cl.1d.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.cl.1darray
    61 << 8 | 72, // llvm.amdgcn.image.sample.cl.1darray.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.cl.2d
    61 << 8 | 72, // llvm.amdgcn.image.sample.cl.2d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.cl.2darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.cl.2darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.cl.3d
    61 << 8 | 73, // llvm.amdgcn.image.sample.cl.3d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.cl.cube
    61 << 8 | 73, // llvm.amdgcn.image.sample.cl.cube.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.cl.o.1d
    61 << 8 | 72, // llvm.amdgcn.image.sample.cl.o.1d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.cl.o.1darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.cl.o.1darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.cl.o.2d
    61 << 8 | 73, // llvm.amdgcn.image.sample.cl.o.2d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.cl.o.2darray
    61 << 8 | 74, // llvm.amdgcn.image.sample.cl.o.2darray.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.cl.o.3d
    61 << 8 | 74, // llvm.amdgcn.image.sample.cl.o.3d.nortn
    60 << 8 | 74, // llvm.amdgcn.image.sample.cl.o.cube
    61 << 8 | 74, // llvm.amdgcn.image.sample.cl.o.cube.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.cube
    61 << 8 | 72, // llvm.amdgcn.image.sample.cube.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.d.1d
    7 << 8 | 72, // llvm.amdgcn.image.sample.d.1d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.d.1darray
    7 << 8 | 73, // llvm.amdgcn.image.sample.d.1darray.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.d.2d
    7 << 8 | 75, // llvm.amdgcn.image.sample.d.2d.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.d.2darray
    7 << 8 | 76, // llvm.amdgcn.image.sample.d.2darray.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.d.3d
    7 << 8 | 84, // llvm.amdgcn.image.sample.d.3d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.d.cl.1d
    7 << 8 | 73, // llvm.amdgcn.image.sample.d.cl.1d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.d.cl.1darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.d.cl.1darray.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.d.cl.2d
    7 << 8 | 76, // llvm.amdgcn.image.sample.d.cl.2d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.d.cl.2darray
    7 << 8 | 82, // llvm.amdgcn.image.sample.d.cl.2darray.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.d.cl.3d
    7 << 8 | 83, // llvm.amdgcn.image.sample.d.cl.3d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.d.cl.cube
    7 << 8 | 82, // llvm.amdgcn.image.sample.d.cl.cube.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.d.cl.o.1d
    7 << 8 | 74, // llvm.amdgcn.image.sample.d.cl.o.1d.nortn
    23 << 8 | 75, // llvm.amdgcn.image.sample.d.cl.o.1darray
    7 << 8 | 75, // llvm.amdgcn.image.sample.d.cl.o.1darray.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.d.cl.o.2d
    7 << 8 | 82, // llvm.amdgcn.image.sample.d.cl.o.2d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.d.cl.o.2darray
    7 << 8 | 84, // llvm.amdgcn.image.sample.d.cl.o.2darray.nortn
    23 << 8 | 85, // llvm.amdgcn.image.sample.d.cl.o.3d
    7 << 8 | 85, // llvm.amdgcn.image.sample.d.cl.o.3d.nortn
    23 << 8 | 84, // llvm.amdgcn.image.sample.d.cl.o.cube
    7 << 8 | 84, // llvm.amdgcn.image.sample.d.cl.o.cube.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.d.cube
    7 << 8 | 76, // llvm.amdgcn.image.sample.d.cube.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.d.o.1d
    7 << 8 | 73, // llvm.amdgcn.image.sample.d.o.1d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.d.o.1darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.d.o.1darray.nortn
    23 << 8 | 76, // llvm.amdgcn.image.sample.d.o.2d
    7 << 8 | 76, // llvm.amdgcn.image.sample.d.o.2d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.d.o.2darray
    7 << 8 | 82, // llvm.amdgcn.image.sample.d.o.2darray.nortn
    23 << 8 | 83, // llvm.amdgcn.image.sample.d.o.3d
    7 << 8 | 83, // llvm.amdgcn.image.sample.d.o.3d.nortn
    23 << 8 | 82, // llvm.amdgcn.image.sample.d.o.cube
    7 << 8 | 82, // llvm.amdgcn.image.sample.d.o.cube.nortn
    23 << 8 | 71, // llvm.amdgcn.image.sample.l.1d
    7 << 8 | 71, // llvm.amdgcn.image.sample.l.1d.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.l.1darray
    7 << 8 | 72, // llvm.amdgcn.image.sample.l.1darray.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.l.2d
    7 << 8 | 72, // llvm.amdgcn.image.sample.l.2d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.l.2darray
    7 << 8 | 73, // llvm.amdgcn.image.sample.l.2darray.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.l.3d
    7 << 8 | 73, // llvm.amdgcn.image.sample.l.3d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.l.cube
    7 << 8 | 73, // llvm.amdgcn.image.sample.l.cube.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.l.o.1d
    7 << 8 | 72, // llvm.amdgcn.image.sample.l.o.1d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.l.o.1darray
    7 << 8 | 73, // llvm.amdgcn.image.sample.l.o.1darray.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.l.o.2d
    7 << 8 | 73, // llvm.amdgcn.image.sample.l.o.2d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.l.o.2darray
    7 << 8 | 74, // llvm.amdgcn.image.sample.l.o.2darray.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.l.o.3d
    7 << 8 | 74, // llvm.amdgcn.image.sample.l.o.3d.nortn
    23 << 8 | 74, // llvm.amdgcn.image.sample.l.o.cube
    7 << 8 | 74, // llvm.amdgcn.image.sample.l.o.cube.nortn
    23 << 8 | 77, // llvm.amdgcn.image.sample.lz.1d
    7 << 8 | 77, // llvm.amdgcn.image.sample.lz.1d.nortn
    23 << 8 | 71, // llvm.amdgcn.image.sample.lz.1darray
    7 << 8 | 71, // llvm.amdgcn.image.sample.lz.1darray.nortn
    23 << 8 | 71, // llvm.amdgcn.image.sample.lz.2d
    7 << 8 | 71, // llvm.amdgcn.image.sample.lz.2d.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.lz.2darray
    7 << 8 | 72, // llvm.amdgcn.image.sample.lz.2darray.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.lz.3d
    7 << 8 | 72, // llvm.amdgcn.image.sample.lz.3d.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.lz.cube
    7 << 8 | 72, // llvm.amdgcn.image.sample.lz.cube.nortn
    23 << 8 | 71, // llvm.amdgcn.image.sample.lz.o.1d
    7 << 8 | 71, // llvm.amdgcn.image.sample.lz.o.1d.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.lz.o.1darray
    7 << 8 | 72, // llvm.amdgcn.image.sample.lz.o.1darray.nortn
    23 << 8 | 72, // llvm.amdgcn.image.sample.lz.o.2d
    7 << 8 | 72, // llvm.amdgcn.image.sample.lz.o.2d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.lz.o.2darray
    7 << 8 | 73, // llvm.amdgcn.image.sample.lz.o.2darray.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.lz.o.3d
    7 << 8 | 73, // llvm.amdgcn.image.sample.lz.o.3d.nortn
    23 << 8 | 73, // llvm.amdgcn.image.sample.lz.o.cube
    7 << 8 | 73, // llvm.amdgcn.image.sample.lz.o.cube.nortn
    60 << 8 | 71, // llvm.amdgcn.image.sample.o.1d
    61 << 8 | 71, // llvm.amdgcn.image.sample.o.1d.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.o.1darray
    61 << 8 | 72, // llvm.amdgcn.image.sample.o.1darray.nortn
    60 << 8 | 72, // llvm.amdgcn.image.sample.o.2d
    61 << 8 | 72, // llvm.amdgcn.image.sample.o.2d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.o.2darray
    61 << 8 | 73, // llvm.amdgcn.image.sample.o.2darray.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.o.3d
    61 << 8 | 73, // llvm.amdgcn.image.sample.o.3d.nortn
    60 << 8 | 73, // llvm.amdgcn.image.sample.o.cube
    61 << 8 | 73, // llvm.amdgcn.image.sample.o.cube.nortn
    24 << 8 | 87, // llvm.amdgcn.image.store.1d
    24 << 8 | 88, // llvm.amdgcn.image.store.1darray
    24 << 8 | 88, // llvm.amdgcn.image.store.2d
    24 << 8 | 89, // llvm.amdgcn.image.store.2darray
    24 << 8 | 90, // llvm.amdgcn.image.store.2darraymsaa
    24 << 8 | 89, // llvm.amdgcn.image.store.2dmsaa
    24 << 8 | 89, // llvm.amdgcn.image.store.3d
    24 << 8 | 89, // llvm.amdgcn.image.store.cube
    24 << 8 | 88, // llvm.amdgcn.image.store.mip.1d
    24 << 8 | 89, // llvm.amdgcn.image.store.mip.1darray
    24 << 8 | 89, // llvm.amdgcn.image.store.mip.2d
    24 << 8 | 90, // llvm.amdgcn.image.store.mip.2darray
    24 << 8 | 90, // llvm.amdgcn.image.store.mip.3d
    24 << 8 | 90, // llvm.amdgcn.image.store.mip.cube
    0 << 8 | 91, // llvm.amdgcn.implicit.buffer.ptr
    0 << 8 | 91, // llvm.amdgcn.implicitarg.ptr
    58 << 8 | 5, // llvm.amdgcn.init.exec
    58 << 8 | 0, // llvm.amdgcn.init.exec.from.input
    62 << 8 | 1, // llvm.amdgcn.init.whole.wave
    0 << 8 | 1, // llvm.amdgcn.interp.inreg.p10
    0 << 8 | 35, // llvm.amdgcn.interp.inreg.p10.f16
    0 << 8 | 1, // llvm.amdgcn.interp.inreg.p2
    0 << 8 | 35, // llvm.amdgcn.interp.inreg.p2.f16
    0 << 8 | 92, // llvm.amdgcn.interp.mov
    0 << 8 | 13, // llvm.amdgcn.interp.p1
    0 << 8 | 29, // llvm.amdgcn.interp.p1.f16
    0 << 8 | 35, // llvm.amdgcn.interp.p10.rtz.f16
    0 << 8 | 93, // llvm.amdgcn.interp.p2
    0 << 8 | 25, // llvm.amdgcn.interp.p2.f16
    0 << 8 | 35, // llvm.amdgcn.interp.p2.rtz.f16
    39 << 8 | 1, // llvm.amdgcn.inverse.ballot
    4 << 8 | 1, // llvm.amdgcn.is.private
    4 << 8 | 1, // llvm.amdgcn.is.shared
    0 << 8 | 91, // llvm.amdgcn.kernarg.segment.ptr
    63 << 8 | 1, // llvm.amdgcn.kill
    64 << 8 | 1, // llvm.amdgcn.lds.direct.load
    0 << 8 | 2, // llvm.amdgcn.lds.kernel.id
    0 << 8 | 17, // llvm.amdgcn.lds.param.load
    4 << 8 | 1, // llvm.amdgcn.lerp
    37 << 8 | 2, // llvm.amdgcn.live.mask
    25 << 8 | 94, // llvm.amdgcn.load.to.lds
    4 << 8 | 1, // llvm.amdgcn.log
    4 << 8 | 1, // llvm.amdgcn.log.clamp
    53 << 8 | 1, // llvm.amdgcn.loop
    4 << 8 | 31, // llvm.amdgcn.make.buffer.rsrc
    65 << 8 | 1, // llvm.amdgcn.mbcnt.hi
    65 << 8 | 1, // llvm.amdgcn.mbcnt.lo
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x16bf16.1k
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x16f16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x1f32
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x2bf16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x32.bf16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x32.bf8.bf8
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x32.bf8.fp8
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x32.f16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x32.fp8.bf8
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x32.fp8.fp8
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x4bf16.1k
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x4f16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x4f32
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x8.xf32
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.16x16x8bf16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x16.bf16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x16.bf8.bf8
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x16.bf8.fp8
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x16.f16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x16.fp8.bf8
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x16.fp8.fp8
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x1f32
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x2bf16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x2f32
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x4.xf32
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x4bf16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x4bf16.1k
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x4f16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x8bf16.1k
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.32x32x8f16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.4x4x1f32
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.4x4x2bf16
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.4x4x4bf16.1k
    47 << 8 | 95, // llvm.amdgcn.mfma.f32.4x4x4f16
    47 << 8 | 95, // llvm.amdgcn.mfma.f64.16x16x4f64
    47 << 8 | 95, // llvm.amdgcn.mfma.f64.4x4x4f64
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.16x16x16i8
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.16x16x32.i8
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.16x16x4i8
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.16x16x64.i8
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.32x32x16.i8
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.32x32x32.i8
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.32x32x4i8
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.32x32x8i8
    47 << 8 | 95, // llvm.amdgcn.mfma.i32.4x4x4i8
    47 << 8 | 96, // llvm.amdgcn.mfma.scale.f32.16x16x128.f8f6f4
    47 << 8 | 96, // llvm.amdgcn.mfma.scale.f32.32x32x64.f8f6f4
    52 << 8 | 97, // llvm.amdgcn.mov.dpp
    52 << 8 | 0, // llvm.amdgcn.mov.dpp8
    4 << 8 | 1, // llvm.amdgcn.mqsad.pk.u16.u8
    4 << 8 | 1, // llvm.amdgcn.mqsad.u32.u8
    4 << 8 | 1, // llvm.amdgcn.msad.u8
    4 << 8 | 1, // llvm.amdgcn.mul.i24
    4 << 8 | 1, // llvm.amdgcn.mul.u24
    4 << 8 | 1, // llvm.amdgcn.mulhi.i24
    4 << 8 | 1, // llvm.amdgcn.mulhi.u24
    66 << 8 | 1, // llvm.amdgcn.perm
    4 << 8 | 1, // llvm.amdgcn.perm.pk16.b4.u4
    4 << 8 | 1, // llvm.amdgcn.perm.pk16.b6.u4
    4 << 8 | 1, // llvm.amdgcn.perm.pk16.b8.u4
    52 << 8 | 1, // llvm.amdgcn.permlane.bcast
    52 << 8 | 1, // llvm.amdgcn.permlane.down
    52 << 8 | 1, // llvm.amdgcn.permlane.idx.gen
    52 << 8 | 1, // llvm.amdgcn.permlane.up
    52 << 8 | 1, // llvm.amdgcn.permlane.xor
    52 << 8 | 67, // llvm.amdgcn.permlane16
    52 << 8 | 93, // llvm.amdgcn.permlane16.swap
    52 << 8 | 48, // llvm.amdgcn.permlane16.var
    52 << 8 | 93, // llvm.amdgcn.permlane32.swap
    52 << 8 | 1, // llvm.amdgcn.permlane64
    52 << 8 | 67, // llvm.amdgcn.permlanex16
    52 << 8 | 48, // llvm.amdgcn.permlanex16.var
    4 << 8 | 35, // llvm.amdgcn.pk.add.max.i16
    4 << 8 | 35, // llvm.amdgcn.pk.add.max.u16
    4 << 8 | 35, // llvm.amdgcn.pk.add.min.i16
    4 << 8 | 35, // llvm.amdgcn.pk.add.min.u16
    30 << 8 | 1, // llvm.amdgcn.pops.exiting.wave.id
    65 << 8 | 1, // llvm.amdgcn.prng.b32
    1 << 8 | 1, // llvm.amdgcn.ps.live
    4 << 8 | 1, // llvm.amdgcn.qsad.pk.u16.u8
    0 << 8 | 58, // llvm.amdgcn.queue.ptr
    53 << 8 | 35, // llvm.amdgcn.raw.atomic.buffer.load
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.add
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.and
    53 << 8 | 47, // llvm.amdgcn.raw.buffer.atomic.cmpswap
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.cond.sub.u32
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.dec
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.fadd
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.fmax
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.fmin
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.inc
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.or
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.smax
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.smin
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.sub
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.sub.clamp.u32
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.swap
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.umax
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.umin
    53 << 8 | 43, // llvm.amdgcn.raw.buffer.atomic.xor
    23 << 8 | 35, // llvm.amdgcn.raw.buffer.load
    23 << 8 | 35, // llvm.amdgcn.raw.buffer.load.format
    53 << 8 | 98, // llvm.amdgcn.raw.buffer.load.lds
    24 << 8 | 43, // llvm.amdgcn.raw.buffer.store
    24 << 8 | 43, // llvm.amdgcn.raw.buffer.store.format
    25 << 8 | 99, // llvm.amdgcn.raw.ptr.atomic.buffer.load
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.add
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.and
    25 << 8 | 101, // llvm.amdgcn.raw.ptr.buffer.atomic.cmpswap
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.cond.sub.u32
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.dec
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.fadd
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.fmax
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.fmin
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.inc
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.or
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.smax
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.smin
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.sub
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.sub.clamp.u32
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.swap
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.umax
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.umin
    25 << 8 | 100, // llvm.amdgcn.raw.ptr.buffer.atomic.xor
    2 << 8 | 102, // llvm.amdgcn.raw.ptr.buffer.load
    2 << 8 | 102, // llvm.amdgcn.raw.ptr.buffer.load.format
    25 << 8 | 103, // llvm.amdgcn.raw.ptr.buffer.load.lds
    19 << 8 | 104, // llvm.amdgcn.raw.ptr.buffer.store
    19 << 8 | 104, // llvm.amdgcn.raw.ptr.buffer.store.format
    2 << 8 | 105, // llvm.amdgcn.raw.ptr.tbuffer.load
    19 << 8 | 106, // llvm.amdgcn.raw.ptr.tbuffer.store
    23 << 8 | 48, // llvm.amdgcn.raw.tbuffer.load
    24 << 8 | 67, // llvm.amdgcn.raw.tbuffer.store
    4 << 8 | 1, // llvm.amdgcn.rcp
    4 << 8 | 1, // llvm.amdgcn.rcp.legacy
    39 << 8 | 1, // llvm.amdgcn.readfirstlane
    52 << 8 | 1, // llvm.amdgcn.readlane
    0 << 8 | 1, // llvm.amdgcn.reloc.constant
    4 << 8 | 1, // llvm.amdgcn.rsq
    4 << 8 | 1, // llvm.amdgcn.rsq.clamp
    4 << 8 | 1, // llvm.amdgcn.rsq.legacy
    58 << 8 | 1, // llvm.amdgcn.s.barrier
    58 << 8 | 1, // llvm.amdgcn.s.barrier.init
    58 << 8 | 1, // llvm.amdgcn.s.barrier.join
    58 << 8 | 5, // llvm.amdgcn.s.barrier.leave
    58 << 8 | 5, // llvm.amdgcn.s.barrier.signal
    58 << 8 | 5, // llvm.amdgcn.s.barrier.signal.isfirst
    58 << 8 | 1, // llvm.amdgcn.s.barrier.signal.var
    58 << 8 | 5, // llvm.amdgcn.s.barrier.wait
    15 << 8 | 1, // llvm.amdgcn.s.bitreplicate
    1 << 8 | 11, // llvm.amdgcn.s.buffer.load
    28 << 8 | 0, // llvm.amdgcn.s.buffer.prefetch.data
    58 << 8 | 1, // llvm.amdgcn.s.cluster.barrier
    30 << 8 | 1, // llvm.amdgcn.s.dcache.inv
    30 << 8 | 1, // llvm.amdgcn.s.dcache.inv.vol
    48 << 8 | 1, // llvm.amdgcn.s.dcache.wb
    48 << 8 | 1, // llvm.amdgcn.s.dcache.wb.vol
    30 << 8 | 5, // llvm.amdgcn.s.decperflevel
    58 << 8 | 1, // llvm.amdgcn.s.get.barrier.state
    58 << 8 | 1, // llvm.amdgcn.s.get.named.barrier.state
    48 << 8 | 2, // llvm.amdgcn.s.get.waveid.in.workgroup
    0 << 8 | 2, // llvm.amdgcn.s.getpc
    30 << 8 | 5, // llvm.amdgcn.s.getreg
    30 << 8 | 5, // llvm.amdgcn.s.incperflevel
    48 << 8 | 1, // llvm.amdgcn.s.memrealtime
    30 << 8 | 1, // llvm.amdgcn.s.memtime
    30 << 8 | 5, // llvm.amdgcn.s.monitor.sleep
    30 << 8 | 5, // llvm.amdgcn.s.nop
    41 << 8 | 10, // llvm.amdgcn.s.prefetch.data
    15 << 8 | 1, // llvm.amdgcn.s.quadmask
    67 << 8 | 5, // llvm.amdgcn.s.sendmsg
    67 << 8 | 5, // llvm.amdgcn.s.sendmsg.rtn
    68 << 8 | 5, // llvm.amdgcn.s.sendmsghalt
    30 << 8 | 5, // llvm.amdgcn.s.sethalt
    30 << 8 | 5, // llvm.amdgcn.s.setprio
    30 << 8 | 5, // llvm.amdgcn.s.setprio.inc.wg
    30 << 8 | 5, // llvm.amdgcn.s.setreg
    30 << 8 | 5, // llvm.amdgcn.s.sleep
    69 << 8 | 1, // llvm.amdgcn.s.sleep.var
    30 << 8 | 1, // llvm.amdgcn.s.ttracedata
    30 << 8 | 5, // llvm.amdgcn.s.ttracedata.imm
    70 << 8 | 5, // llvm.amdgcn.s.wait.asynccnt
    48 << 8 | 5, // llvm.amdgcn.s.wait.bvhcnt
    48 << 8 | 5, // llvm.amdgcn.s.wait.dscnt
    69 << 8 | 1, // llvm.amdgcn.s.wait.event.export.ready
    48 << 8 | 5, // llvm.amdgcn.s.wait.expcnt
    48 << 8 | 5, // llvm.amdgcn.s.wait.kmcnt
    48 << 8 | 5, // llvm.amdgcn.s.wait.loadcnt
    48 << 8 | 5, // llvm.amdgcn.s.wait.samplecnt
    48 << 8 | 5, // llvm.amdgcn.s.wait.storecnt
    70 << 8 | 5, // llvm.amdgcn.s.wait.tensorcnt
    48 << 8 | 5, // llvm.amdgcn.s.waitcnt
    58 << 8 | 1, // llvm.amdgcn.s.wakeup.barrier
    15 << 8 | 1, // llvm.amdgcn.s.wqm
    4 << 8 | 1, // llvm.amdgcn.sad.hi.u8
    4 << 8 | 1, // llvm.amdgcn.sad.u16
    4 << 8 | 1, // llvm.amdgcn.sad.u8
    4 << 8 | 1, // llvm.amdgcn.sat.pk4.i4.i8
    4 << 8 | 1, // llvm.amdgcn.sat.pk4.u4.u8
    4 << 8 | 1, // llvm.amdgcn.sbfe
    58 << 8 | 5, // llvm.amdgcn.sched.barrier
    58 << 8 | 92, // llvm.amdgcn.sched.group.barrier
    4 << 8 | 35, // llvm.amdgcn.sdot2
    4 << 8 | 35, // llvm.amdgcn.sdot4
    4 << 8 | 35, // llvm.amdgcn.sdot8
    52 << 8 | 1, // llvm.amdgcn.set.inactive
    52 << 8 | 1, // llvm.amdgcn.set.inactive.chain.arg
    4 << 8 | 1, // llvm.amdgcn.sffbh
    4 << 8 | 1, // llvm.amdgcn.sin
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x128.bf8.bf8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x128.bf8.fp8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x128.fp8.bf8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x128.fp8.fp8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x32.bf16
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x32.f16
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x64.bf16
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x64.bf8.bf8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x64.bf8.fp8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x64.f16
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x64.fp8.bf8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.16x16x64.fp8.fp8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x16.bf16
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x16.f16
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x32.bf16
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x32.bf8.bf8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x32.bf8.fp8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x32.f16
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x32.fp8.bf8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x32.fp8.fp8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x64.bf8.bf8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x64.bf8.fp8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x64.fp8.bf8
    47 << 8 | 67, // llvm.amdgcn.smfmac.f32.32x32x64.fp8.fp8
    47 << 8 | 67, // llvm.amdgcn.smfmac.i32.16x16x128.i8
    47 << 8 | 67, // llvm.amdgcn.smfmac.i32.16x16x64.i8
    47 << 8 | 67, // llvm.amdgcn.smfmac.i32.32x32x32.i8
    47 << 8 | 67, // llvm.amdgcn.smfmac.i32.32x32x64.i8
    66 << 8 | 1, // llvm.amdgcn.softwqm
    4 << 8 | 1, // llvm.amdgcn.sqrt
    71 << 8 | 1, // llvm.amdgcn.strict.wqm
    71 << 8 | 1, // llvm.amdgcn.strict.wwm
    53 << 8 | 43, // llvm.amdgcn.struct.atomic.buffer.load
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.add
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.and
    53 << 8 | 44, // llvm.amdgcn.struct.buffer.atomic.cmpswap
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.cond.sub.u32
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.dec
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.fadd
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.fmax
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.fmin
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.inc
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.or
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.smax
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.smin
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.sub
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.sub.clamp.u32
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.swap
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.umax
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.umin
    53 << 8 | 47, // llvm.amdgcn.struct.buffer.atomic.xor
    23 << 8 | 43, // llvm.amdgcn.struct.buffer.load
    23 << 8 | 43, // llvm.amdgcn.struct.buffer.load.format
    53 << 8 | 107, // llvm.amdgcn.struct.buffer.load.lds
    24 << 8 | 47, // llvm.amdgcn.struct.buffer.store
    24 << 8 | 47, // llvm.amdgcn.struct.buffer.store.format
    25 << 8 | 108, // llvm.amdgcn.struct.ptr.atomic.buffer.load
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.add
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.and
    25 << 8 | 110, // llvm.amdgcn.struct.ptr.buffer.atomic.cmpswap
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.cond.sub.u32
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.dec
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.fadd
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.fmax
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.fmin
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.inc
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.or
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.smax
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.smin
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.sub
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.sub.clamp.u32
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.swap
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.umax
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.umin
    25 << 8 | 109, // llvm.amdgcn.struct.ptr.buffer.atomic.xor
    2 << 8 | 111, // llvm.amdgcn.struct.ptr.buffer.load
    2 << 8 | 111, // llvm.amdgcn.struct.ptr.buffer.load.format
    25 << 8 | 112, // llvm.amdgcn.struct.ptr.buffer.load.lds
    19 << 8 | 113, // llvm.amdgcn.struct.ptr.buffer.store
    19 << 8 | 113, // llvm.amdgcn.struct.ptr.buffer.store.format
    2 << 8 | 114, // llvm.amdgcn.struct.ptr.tbuffer.load
    19 << 8 | 115, // llvm.amdgcn.struct.ptr.tbuffer.store
    23 << 8 | 67, // llvm.amdgcn.struct.tbuffer.load
    24 << 8 | 68, // llvm.amdgcn.struct.tbuffer.store
    4 << 8 | 116, // llvm.amdgcn.sudot4
    4 << 8 | 116, // llvm.amdgcn.sudot8
    72 << 8 | 1, // llvm.amdgcn.swmmac.bf16.16x16x32.bf16
    72 << 8 | 117, // llvm.amdgcn.swmmac.bf16.16x16x64.bf16
    72 << 8 | 117, // llvm.amdgcn.swmmac.bf16f32.16x16x64.bf16
    72 << 8 | 67, // llvm.amdgcn.swmmac.f16.16x16x128.bf8.bf8
    72 << 8 | 67, // llvm.amdgcn.swmmac.f16.16x16x128.bf8.fp8
    72 << 8 | 67, // llvm.amdgcn.swmmac.f16.16x16x128.fp8.bf8
    72 << 8 | 67, // llvm.amdgcn.swmmac.f16.16x16x128.fp8.fp8
    72 << 8 | 1, // llvm.amdgcn.swmmac.f16.16x16x32.f16
    72 << 8 | 117, // llvm.amdgcn.swmmac.f16.16x16x64.f16
    72 << 8 | 67, // llvm.amdgcn.swmmac.f32.16x16x128.bf8.bf8
    72 << 8 | 67, // llvm.amdgcn.swmmac.f32.16x16x128.bf8.fp8
    72 << 8 | 67, // llvm.amdgcn.swmmac.f32.16x16x128.fp8.bf8
    72 << 8 | 67, // llvm.amdgcn.swmmac.f32.16x16x128.fp8.fp8
    72 << 8 | 1, // llvm.amdgcn.swmmac.f32.16x16x32.bf16
    72 << 8 | 1, // llvm.amdgcn.swmmac.f32.16x16x32.bf8.bf8
    72 << 8 | 1, // llvm.amdgcn.swmmac.f32.16x16x32.bf8.fp8
    72 << 8 | 1, // llvm.amdgcn.swmmac.f32.16x16x32.f16
    72 << 8 | 1, // llvm.amdgcn.swmmac.f32.16x16x32.fp8.bf8
    72 << 8 | 1, // llvm.amdgcn.swmmac.f32.16x16x32.fp8.fp8
    72 << 8 | 117, // llvm.amdgcn.swmmac.f32.16x16x64.bf16
    72 << 8 | 117, // llvm.amdgcn.swmmac.f32.16x16x64.f16
    72 << 8 | 118, // llvm.amdgcn.swmmac.i32.16x16x128.iu8
    72 << 8 | 119, // llvm.amdgcn.swmmac.i32.16x16x32.iu4
    72 << 8 | 119, // llvm.amdgcn.swmmac.i32.16x16x32.iu8
    72 << 8 | 119, // llvm.amdgcn.swmmac.i32.16x16x64.iu4
    4 << 8 | 1, // llvm.amdgcn.tanh
    73 << 8 | 43, // llvm.amdgcn.tensor.load.to.lds
    73 << 8 | 11, // llvm.amdgcn.tensor.load.to.lds.d2
    73 << 8 | 43, // llvm.amdgcn.tensor.store.from.lds
    73 << 8 | 11, // llvm.amdgcn.tensor.store.from.lds.d2
    4 << 8 | 1, // llvm.amdgcn.trig.preop
    4 << 8 | 1, // llvm.amdgcn.ubfe
    4 << 8 | 35, // llvm.amdgcn.udot2
    4 << 8 | 35, // llvm.amdgcn.udot4
    4 << 8 | 35, // llvm.amdgcn.udot8
    74 << 8 | 1, // llvm.amdgcn.unreachable
    52 << 8 | 120, // llvm.amdgcn.update.dpp
    58 << 8 | 1, // llvm.amdgcn.wave.barrier
    0 << 8 | 2, // llvm.amdgcn.wave.id
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.add
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.and
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.fadd
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.fmax
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.fmin
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.fsub
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.max
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.min
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.or
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.sub
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.umax
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.umin
    52 << 8 | 0, // llvm.amdgcn.wave.reduce.xor
    15 << 8 | 1, // llvm.amdgcn.wave.shuffle
    0 << 8 | 121, // llvm.amdgcn.wavefrontsize
    39 << 8 | 35, // llvm.amdgcn.wmma.bf16.16x16x16.bf16
    39 << 8 | 35, // llvm.amdgcn.wmma.bf16.16x16x16.bf16.tied
    52 << 8 | 122, // llvm.amdgcn.wmma.bf16.16x16x32.bf16
    52 << 8 | 122, // llvm.amdgcn.wmma.bf16f32.16x16x32.bf16
    39 << 8 | 123, // llvm.amdgcn.wmma.f16.16x16x128.bf8.bf8
    39 << 8 | 123, // llvm.amdgcn.wmma.f16.16x16x128.bf8.fp8
    39 << 8 | 123, // llvm.amdgcn.wmma.f16.16x16x128.fp8.bf8
    39 << 8 | 123, // llvm.amdgcn.wmma.f16.16x16x128.fp8.fp8
    39 << 8 | 35, // llvm.amdgcn.wmma.f16.16x16x16.f16
    39 << 8 | 35, // llvm.amdgcn.wmma.f16.16x16x16.f16.tied
    52 << 8 | 122, // llvm.amdgcn.wmma.f16.16x16x32.f16
    39 << 8 | 123, // llvm.amdgcn.wmma.f16.16x16x64.bf8.bf8
    39 << 8 | 123, // llvm.amdgcn.wmma.f16.16x16x64.bf8.fp8
    39 << 8 | 123, // llvm.amdgcn.wmma.f16.16x16x64.fp8.bf8
    39 << 8 | 123, // llvm.amdgcn.wmma.f16.16x16x64.fp8.fp8
    39 << 8 | 123, // llvm.amdgcn.wmma.f32.16x16x128.bf8.bf8
    39 << 8 | 123, // llvm.amdgcn.wmma.f32.16x16x128.bf8.fp8
    52 << 8 | 124, // llvm.amdgcn.wmma.f32.16x16x128.f8f6f4
    39 << 8 | 123, // llvm.amdgcn.wmma.f32.16x16x128.fp8.bf8
    39 << 8 | 123, // llvm.amdgcn.wmma.f32.16x16x128.fp8.fp8
    39 << 8 | 1, // llvm.amdgcn.wmma.f32.16x16x16.bf16
    39 << 8 | 1, // llvm.amdgcn.wmma.f32.16x16x16.bf8.bf8
    39 << 8 | 1, // llvm.amdgcn.wmma.f32.16x16x16.bf8.fp8
    39 << 8 | 1, // llvm.amdgcn.wmma.f32.16x16x16.f16
    39 << 8 | 1, // llvm.amdgcn.wmma.f32.16x16x16.fp8.bf8
    39 << 8 | 1, // llvm.amdgcn.wmma.f32.16x16x16.fp8.fp8
    52 << 8 | 122, // llvm.amdgcn.wmma.f32.16x16x32.bf16
    52 << 8 | 122, // llvm.amdgcn.wmma.f32.16x16x32.f16
    52 << 8 | 122, // llvm.amdgcn.wmma.f32.16x16x4.f32
    39 << 8 | 123, // llvm.amdgcn.wmma.f32.16x16x64.bf8.bf8
    39 << 8 | 123, // llvm.amdgcn.wmma.f32.16x16x64.bf8.fp8
    39 << 8 | 123, // llvm.amdgcn.wmma.f32.16x16x64.fp8.bf8
    39 << 8 | 123, // llvm.amdgcn.wmma.f32.16x16x64.fp8.fp8
    39 << 8 | 11, // llvm.amdgcn.wmma.f32.32x16x128.f4
    39 << 8 | 116, // llvm.amdgcn.wmma.i32.16x16x16.iu4
    39 << 8 | 116, // llvm.amdgcn.wmma.i32.16x16x16.iu8
    39 << 8 | 116, // llvm.amdgcn.wmma.i32.16x16x32.iu4
    39 << 8 | 125, // llvm.amdgcn.wmma.i32.16x16x64.iu8
    39 << 8 | 126, // llvm.amdgcn.wmma.scale.f32.16x16x128.f8f6f4
    39 << 8 | 127, // llvm.amdgcn.wmma.scale.f32.32x16x128.f4
    39 << 8 | 126, // llvm.amdgcn.wmma.scale16.f32.16x16x128.f8f6f4
    39 << 8 | 127, // llvm.amdgcn.wmma.scale16.f32.32x16x128.f4
    0 << 8 | 2, // llvm.amdgcn.workgroup.id.x
    0 << 8 | 2, // llvm.amdgcn.workgroup.id.y
    0 << 8 | 2, // llvm.amdgcn.workgroup.id.z
    0 << 8 | 128, // llvm.amdgcn.workitem.id.x
    0 << 8 | 128, // llvm.amdgcn.workitem.id.y
    0 << 8 | 128, // llvm.amdgcn.workitem.id.z
    66 << 8 | 1, // llvm.amdgcn.wqm
    75 << 8 | 1, // llvm.amdgcn.wqm.demote
    52 << 8 | 1, // llvm.amdgcn.wqm.vote
    52 << 8 | 1, // llvm.amdgcn.writelane
    71 << 8 | 1, // llvm.amdgcn.wwm
    1 << 8 | 17, // llvm.arm.cde.cx1
    1 << 8 | 46, // llvm.arm.cde.cx1a
    1 << 8 | 17, // llvm.arm.cde.cx1d
    1 << 8 | 129, // llvm.arm.cde.cx1da
    1 << 8 | 46, // llvm.arm.cde.cx2
    1 << 8 | 129, // llvm.arm.cde.cx2a
    1 << 8 | 46, // llvm.arm.cde.cx2d
    1 << 8 | 130, // llvm.arm.cde.cx2da
    1 << 8 | 129, // llvm.arm.cde.cx3
    1 << 8 | 130, // llvm.arm.cde.cx3a
    1 << 8 | 129, // llvm.arm.cde.cx3d
    1 << 8 | 45, // llvm.arm.cde.cx3da
    1 << 8 | 17, // llvm.arm.cde.vcx1
    1 << 8 | 46, // llvm.arm.cde.vcx1a
    1 << 8 | 17, // llvm.arm.cde.vcx1q
    1 << 8 | 46, // llvm.arm.cde.vcx1q.predicated
    1 << 8 | 46, // llvm.arm.cde.vcx1qa
    1 << 8 | 46, // llvm.arm.cde.vcx1qa.predicated
    1 << 8 | 46, // llvm.arm.cde.vcx2
    1 << 8 | 129, // llvm.arm.cde.vcx2a
    1 << 8 | 46, // llvm.arm.cde.vcx2q
    1 << 8 | 129, // llvm.arm.cde.vcx2q.predicated
    1 << 8 | 129, // llvm.arm.cde.vcx2qa
    1 << 8 | 129, // llvm.arm.cde.vcx2qa.predicated
    1 << 8 | 129, // llvm.arm.cde.vcx3
    1 << 8 | 130, // llvm.arm.cde.vcx3a
    1 << 8 | 129, // llvm.arm.cde.vcx3q
    1 << 8 | 130, // llvm.arm.cde.vcx3q.predicated
    1 << 8 | 130, // llvm.arm.cde.vcx3qa
    1 << 8 | 130, // llvm.arm.cde.vcx3qa.predicated
    5 << 8 | 131, // llvm.arm.cdp
    5 << 8 | 131, // llvm.arm.cdp2
    5 << 8 | 1, // llvm.arm.clrex
    1 << 8 | 1, // llvm.arm.cls
    1 << 8 | 1, // llvm.arm.cls64
    10 << 8 | 1, // llvm.arm.cmse.tt
    10 << 8 | 1, // llvm.arm.cmse.tta
    10 << 8 | 1, // llvm.arm.cmse.ttat
    10 << 8 | 1, // llvm.arm.cmse.ttt
    1 << 8 | 1, // llvm.arm.crc32b
    1 << 8 | 1, // llvm.arm.crc32cb
    1 << 8 | 1, // llvm.arm.crc32ch
    1 << 8 | 1, // llvm.arm.crc32cw
    1 << 8 | 1, // llvm.arm.crc32h
    1 << 8 | 1, // llvm.arm.crc32w
    5 << 8 | 1, // llvm.arm.dbg
    5 << 8 | 1, // llvm.arm.dmb
    5 << 8 | 1, // llvm.arm.dsb
    23 << 8 | 1, // llvm.arm.get.fpscr
    5 << 8 | 1, // llvm.arm.gnu.eabi.mcount
    5 << 8 | 1, // llvm.arm.hint
    5 << 8 | 1, // llvm.arm.isb
    5 << 8 | 1, // llvm.arm.ldaex
    5 << 8 | 1, // llvm.arm.ldaexd
    5 << 8 | 17, // llvm.arm.ldc
    5 << 8 | 17, // llvm.arm.ldc2
    5 << 8 | 17, // llvm.arm.ldc2l
    5 << 8 | 17, // llvm.arm.ldcl
    5 << 8 | 1, // llvm.arm.ldrex
    5 << 8 | 1, // llvm.arm.ldrexd
    5 << 8 | 132, // llvm.arm.mcr
    5 << 8 | 132, // llvm.arm.mcr2
    5 << 8 | 133, // llvm.arm.mcrr
    5 << 8 | 133, // llvm.arm.mcrr2
    5 << 8 | 134, // llvm.arm.mrc
    5 << 8 | 134, // llvm.arm.mrc2
    5 << 8 | 92, // llvm.arm.mrrc
    5 << 8 | 92, // llvm.arm.mrrc2
    1 << 8 | 1, // llvm.arm.mve.abd.predicated
    1 << 8 | 1, // llvm.arm.mve.abs.predicated
    1 << 8 | 1, // llvm.arm.mve.add.predicated
    1 << 8 | 1, // llvm.arm.mve.addlv
    1 << 8 | 1, // llvm.arm.mve.addlv.predicated
    1 << 8 | 1, // llvm.arm.mve.addv
    1 << 8 | 1, // llvm.arm.mve.addv.predicated
    1 << 8 | 1, // llvm.arm.mve.and.predicated
    1 << 8 | 1, // llvm.arm.mve.asrl
    1 << 8 | 1, // llvm.arm.mve.bic.predicated
    1 << 8 | 1, // llvm.arm.mve.cls.predicated
    1 << 8 | 1, // llvm.arm.mve.clz.predicated
    1 << 8 | 1, // llvm.arm.mve.cmp.eq
    1 << 8 | 1, // llvm.arm.mve.cmp.ge
    1 << 8 | 1, // llvm.arm.mve.cmp.gt
    1 << 8 | 1, // llvm.arm.mve.cmp.le
    1 << 8 | 1, // llvm.arm.mve.cmp.lt
    1 << 8 | 1, // llvm.arm.mve.cmp.ne
    1 << 8 | 1, // llvm.arm.mve.eor.predicated
    1 << 8 | 1, // llvm.arm.mve.fma
    1 << 8 | 1, // llvm.arm.mve.fma.predicated
    1 << 8 | 1, // llvm.arm.mve.hadd.predicated
    1 << 8 | 1, // llvm.arm.mve.hsub.predicated
    1 << 8 | 1, // llvm.arm.mve.lsll
    1 << 8 | 1, // llvm.arm.mve.max.predicated
    1 << 8 | 1, // llvm.arm.mve.maxav
    1 << 8 | 1, // llvm.arm.mve.maxav.predicated
    1 << 8 | 1, // llvm.arm.mve.maxnmav
    1 << 8 | 1, // llvm.arm.mve.maxnmav.predicated
    1 << 8 | 1, // llvm.arm.mve.maxnmv
    1 << 8 | 1, // llvm.arm.mve.maxnmv.predicated
    1 << 8 | 1, // llvm.arm.mve.maxv
    1 << 8 | 1, // llvm.arm.mve.maxv.predicated
    1 << 8 | 1, // llvm.arm.mve.min.predicated
    1 << 8 | 1, // llvm.arm.mve.minav
    1 << 8 | 1, // llvm.arm.mve.minav.predicated
    1 << 8 | 1, // llvm.arm.mve.minnmav
    1 << 8 | 1, // llvm.arm.mve.minnmav.predicated
    1 << 8 | 1, // llvm.arm.mve.minnmv
    1 << 8 | 1, // llvm.arm.mve.minnmv.predicated
    1 << 8 | 1, // llvm.arm.mve.minv
    1 << 8 | 1, // llvm.arm.mve.minv.predicated
    1 << 8 | 1, // llvm.arm.mve.mul.predicated
    1 << 8 | 1, // llvm.arm.mve.mulh.predicated
    1 << 8 | 1, // llvm.arm.mve.mull.int.predicated
    1 << 8 | 1, // llvm.arm.mve.mull.poly.predicated
    1 << 8 | 1, // llvm.arm.mve.mvn.predicated
    1 << 8 | 1, // llvm.arm.mve.neg.predicated
    1 << 8 | 1, // llvm.arm.mve.orn.predicated
    1 << 8 | 1, // llvm.arm.mve.orr.predicated
    1 << 8 | 1, // llvm.arm.mve.pred.i2v
    1 << 8 | 1, // llvm.arm.mve.pred.v2i
    1 << 8 | 1, // llvm.arm.mve.qabs.predicated
    1 << 8 | 1, // llvm.arm.mve.qadd.predicated
    1 << 8 | 1, // llvm.arm.mve.qdmulh.predicated
    1 << 8 | 1, // llvm.arm.mve.qneg.predicated
    1 << 8 | 1, // llvm.arm.mve.qrdmulh.predicated
    1 << 8 | 1, // llvm.arm.mve.qsub.predicated
    1 << 8 | 1, // llvm.arm.mve.rhadd.predicated
    1 << 8 | 1, // llvm.arm.mve.rmulh.predicated
    1 << 8 | 1, // llvm.arm.mve.shl.imm.predicated
    1 << 8 | 1, // llvm.arm.mve.shr.imm.predicated
    1 << 8 | 1, // llvm.arm.mve.sqrshr
    1 << 8 | 1, // llvm.arm.mve.sqrshrl
    1 << 8 | 1, // llvm.arm.mve.sqshl
    1 << 8 | 1, // llvm.arm.mve.sqshll
    1 << 8 | 1, // llvm.arm.mve.srshr
    1 << 8 | 1, // llvm.arm.mve.srshrl
    1 << 8 | 1, // llvm.arm.mve.sub.predicated
    1 << 8 | 1, // llvm.arm.mve.uqrshl
    1 << 8 | 1, // llvm.arm.mve.uqrshll
    1 << 8 | 1, // llvm.arm.mve.uqshl
    1 << 8 | 1, // llvm.arm.mve.uqshll
    1 << 8 | 1, // llvm.arm.mve.urshr
    1 << 8 | 1, // llvm.arm.mve.urshrl
    1 << 8 | 1, // llvm.arm.mve.vabav
    1 << 8 | 1, // llvm.arm.mve.vabav.predicated
    1 << 8 | 1, // llvm.arm.mve.vabd
    1 << 8 | 1, // llvm.arm.mve.vadc
    1 << 8 | 1, // llvm.arm.mve.vadc.predicated
    1 << 8 | 1, // llvm.arm.mve.vadd
    1 << 8 | 1, // llvm.arm.mve.vbrsr
    1 << 8 | 1, // llvm.arm.mve.vbrsr.predicated
    1 << 8 | 1, // llvm.arm.mve.vcaddq
    1 << 8 | 1, // llvm.arm.mve.vcaddq.predicated
    1 << 8 | 1, // llvm.arm.mve.vcls
    1 << 8 | 1, // llvm.arm.mve.vcmlaq
    1 << 8 | 1, // llvm.arm.mve.vcmlaq.predicated
    1 << 8 | 1, // llvm.arm.mve.vcmulq
    1 << 8 | 1, // llvm.arm.mve.vcmulq.predicated
    1 << 8 | 1, // llvm.arm.mve.vctp16
    1 << 8 | 1, // llvm.arm.mve.vctp32
    1 << 8 | 1, // llvm.arm.mve.vctp64
    1 << 8 | 1, // llvm.arm.mve.vctp8
    1 << 8 | 1, // llvm.arm.mve.vcvt.fix
    1 << 8 | 1, // llvm.arm.mve.vcvt.fix.predicated
    1 << 8 | 1, // llvm.arm.mve.vcvt.fp.int
    1 << 8 | 1, // llvm.arm.mve.vcvt.fp.int.predicated
    1 << 8 | 1, // llvm.arm.mve.vcvt.int.fp
    1 << 8 | 1, // llvm.arm.mve.vcvt.narrow
    1 << 8 | 1, // llvm.arm.mve.vcvt.narrow.predicated
    1 << 8 | 1, // llvm.arm.mve.vcvt.widen
    1 << 8 | 1, // llvm.arm.mve.vcvt.widen.predicated
    1 << 8 | 1, // llvm.arm.mve.vcvta
    1 << 8 | 1, // llvm.arm.mve.vcvta.predicated
    1 << 8 | 1, // llvm.arm.mve.vcvtm
    1 << 8 | 1, // llvm.arm.mve.vcvtm.predicated
    1 << 8 | 1, // llvm.arm.mve.vcvtn
    1 << 8 | 1, // llvm.arm.mve.vcvtn.predicated
    1 << 8 | 1, // llvm.arm.mve.vcvtp
    1 << 8 | 1, // llvm.arm.mve.vcvtp.predicated
    1 << 8 | 1, // llvm.arm.mve.vddup
    1 << 8 | 1, // llvm.arm.mve.vddup.predicated
    1 << 8 | 1, // llvm.arm.mve.vdwdup
    1 << 8 | 1, // llvm.arm.mve.vdwdup.predicated
    1 << 8 | 1, // llvm.arm.mve.vhadd
    1 << 8 | 1, // llvm.arm.mve.vhsub
    1 << 8 | 1, // llvm.arm.mve.vidup
    1 << 8 | 1, // llvm.arm.mve.vidup.predicated
    1 << 8 | 1, // llvm.arm.mve.viwdup
    1 << 8 | 1, // llvm.arm.mve.viwdup.predicated
    2 << 8 | 1, // llvm.arm.mve.vld2q
    2 << 8 | 1, // llvm.arm.mve.vld4q
    23 << 8 | 1, // llvm.arm.mve.vldr.gather.base
    23 << 8 | 1, // llvm.arm.mve.vldr.gather.base.predicated
    23 << 8 | 1, // llvm.arm.mve.vldr.gather.base.wb
    23 << 8 | 1, // llvm.arm.mve.vldr.gather.base.wb.predicated
    23 << 8 | 1, // llvm.arm.mve.vldr.gather.offset
    23 << 8 | 1, // llvm.arm.mve.vldr.gather.offset.predicated
    1 << 8 | 1, // llvm.arm.mve.vmaxa.predicated
    1 << 8 | 1, // llvm.arm.mve.vmaxnm
    1 << 8 | 1, // llvm.arm.mve.vmaxnma.predicated
    1 << 8 | 1, // llvm.arm.mve.vmina.predicated
    1 << 8 | 1, // llvm.arm.mve.vminnm
    1 << 8 | 1, // llvm.arm.mve.vminnma.predicated
    1 << 8 | 1, // llvm.arm.mve.vmla.n.predicated
    1 << 8 | 1, // llvm.arm.mve.vmlas.n.predicated
    1 << 8 | 1, // llvm.arm.mve.vmldava
    1 << 8 | 1, // llvm.arm.mve.vmldava.predicated
    1 << 8 | 1, // llvm.arm.mve.vmlldava
    1 << 8 | 1, // llvm.arm.mve.vmlldava.predicated
    1 << 8 | 1, // llvm.arm.mve.vmovl.predicated
    1 << 8 | 1, // llvm.arm.mve.vmovn.predicated
    1 << 8 | 1, // llvm.arm.mve.vmul
    1 << 8 | 1, // llvm.arm.mve.vmulh
    1 << 8 | 1, // llvm.arm.mve.vmull
    1 << 8 | 1, // llvm.arm.mve.vmull.poly
    1 << 8 | 1, // llvm.arm.mve.vqdmlad
    1 << 8 | 1, // llvm.arm.mve.vqdmlad.predicated
    1 << 8 | 1, // llvm.arm.mve.vqdmlah
    1 << 8 | 1, // llvm.arm.mve.vqdmlah.predicated
    1 << 8 | 1, // llvm.arm.mve.vqdmlash
    1 << 8 | 1, // llvm.arm.mve.vqdmlash.predicated
    1 << 8 | 1, // llvm.arm.mve.vqdmulh
    1 << 8 | 1, // llvm.arm.mve.vqdmull
    1 << 8 | 1, // llvm.arm.mve.vqdmull.predicated
    1 << 8 | 1, // llvm.arm.mve.vqmovn
    1 << 8 | 1, // llvm.arm.mve.vqmovn.predicated
    1 << 8 | 1, // llvm.arm.mve.vqrdmlah
    1 << 8 | 1, // llvm.arm.mve.vqrdmlah.predicated
    1 << 8 | 1, // llvm.arm.mve.vqrdmlash
    1 << 8 | 1, // llvm.arm.mve.vqrdmlash.predicated
    1 << 8 | 1, // llvm.arm.mve.vqrdmulh
    1 << 8 | 1, // llvm.arm.mve.vqshl.imm
    1 << 8 | 1, // llvm.arm.mve.vqshl.imm.predicated
    1 << 8 | 1, // llvm.arm.mve.vqshlu.imm
    1 << 8 | 1, // llvm.arm.mve.vqshlu.imm.predicated
    1 << 8 | 1, // llvm.arm.mve.vreinterpretq
    1 << 8 | 1, // llvm.arm.mve.vrev.predicated
    1 << 8 | 1, // llvm.arm.mve.vrhadd
    1 << 8 | 1, // llvm.arm.mve.vrinta
    1 << 8 | 1, // llvm.arm.mve.vrinta.predicated
    1 << 8 | 1, // llvm.arm.mve.vrintm
    1 << 8 | 1, // llvm.arm.mve.vrintm.predicated
    1 << 8 | 1, // llvm.arm.mve.vrintn
    1 << 8 | 1, // llvm.arm.mve.vrintn.predicated
    1 << 8 | 1, // llvm.arm.mve.vrintp
    1 << 8 | 1, // llvm.arm.mve.vrintp.predicated
    1 << 8 | 1, // llvm.arm.mve.vrintx
    1 << 8 | 1, // llvm.arm.mve.vrintx.predicated
    1 << 8 | 1, // llvm.arm.mve.vrintz
    1 << 8 | 1, // llvm.arm.mve.vrintz.predicated
    1 << 8 | 1, // llvm.arm.mve.vrmlldavha
    1 << 8 | 1, // llvm.arm.mve.vrmlldavha.predicated
    1 << 8 | 1, // llvm.arm.mve.vrmulh
    1 << 8 | 1, // llvm.arm.mve.vrshr.imm
    1 << 8 | 1, // llvm.arm.mve.vrshr.imm.predicated
    1 << 8 | 1, // llvm.arm.mve.vsbc
    1 << 8 | 1, // llvm.arm.mve.vsbc.predicated
    1 << 8 | 1, // llvm.arm.mve.vshl.scalar
    1 << 8 | 1, // llvm.arm.mve.vshl.scalar.predicated
    1 << 8 | 1, // llvm.arm.mve.vshl.vector
    1 << 8 | 1, // llvm.arm.mve.vshl.vector.predicated
    1 << 8 | 1, // llvm.arm.mve.vshlc
    1 << 8 | 1, // llvm.arm.mve.vshlc.predicated
    1 << 8 | 1, // llvm.arm.mve.vshll.imm
    1 << 8 | 1, // llvm.arm.mve.vshll.imm.predicated
    1 << 8 | 1, // llvm.arm.mve.vshrn
    1 << 8 | 1, // llvm.arm.mve.vshrn.predicated
    1 << 8 | 1, // llvm.arm.mve.vsli
    1 << 8 | 1, // llvm.arm.mve.vsli.predicated
    1 << 8 | 1, // llvm.arm.mve.vsri
    1 << 8 | 1, // llvm.arm.mve.vsri.predicated
    19 << 8 | 1, // llvm.arm.mve.vst2q
    19 << 8 | 1, // llvm.arm.mve.vst4q
    24 << 8 | 1, // llvm.arm.mve.vstr.scatter.base
    24 << 8 | 1, // llvm.arm.mve.vstr.scatter.base.predicated
    24 << 8 | 1, // llvm.arm.mve.vstr.scatter.base.wb
    24 << 8 | 1, // llvm.arm.mve.vstr.scatter.base.wb.predicated
    24 << 8 | 1, // llvm.arm.mve.vstr.scatter.offset
    24 << 8 | 1, // llvm.arm.mve.vstr.scatter.offset.predicated
    1 << 8 | 1, // llvm.arm.mve.vsub
    1 << 8 | 1, // llvm.arm.neon.aesd
    1 << 8 | 1, // llvm.arm.neon.aese
    1 << 8 | 1, // llvm.arm.neon.aesimc
    1 << 8 | 1, // llvm.arm.neon.aesmc
    1 << 8 | 1, // llvm.arm.neon.bfdot
    1 << 8 | 1, // llvm.arm.neon.bfmlalb
    1 << 8 | 1, // llvm.arm.neon.bfmlalt
    1 << 8 | 1, // llvm.arm.neon.bfmmla
    1 << 8 | 1, // llvm.arm.neon.sdot
    1 << 8 | 1, // llvm.arm.neon.sha1c
    1 << 8 | 1, // llvm.arm.neon.sha1h
    1 << 8 | 1, // llvm.arm.neon.sha1m
    1 << 8 | 1, // llvm.arm.neon.sha1p
    1 << 8 | 1, // llvm.arm.neon.sha1su0
    1 << 8 | 1, // llvm.arm.neon.sha1su1
    1 << 8 | 1, // llvm.arm.neon.sha256h
    1 << 8 | 1, // llvm.arm.neon.sha256h2
    1 << 8 | 1, // llvm.arm.neon.sha256su0
    1 << 8 | 1, // llvm.arm.neon.sha256su1
    1 << 8 | 1, // llvm.arm.neon.smmla
    1 << 8 | 1, // llvm.arm.neon.udot
    1 << 8 | 1, // llvm.arm.neon.ummla
    1 << 8 | 1, // llvm.arm.neon.usdot
    1 << 8 | 1, // llvm.arm.neon.usmmla
    1 << 8 | 1, // llvm.arm.neon.vabds
    1 << 8 | 1, // llvm.arm.neon.vabdu
    1 << 8 | 1, // llvm.arm.neon.vabs
    1 << 8 | 1, // llvm.arm.neon.vacge
    1 << 8 | 1, // llvm.arm.neon.vacgt
    1 << 8 | 1, // llvm.arm.neon.vbsl
    1 << 8 | 1, // llvm.arm.neon.vcadd.rot270
    1 << 8 | 1, // llvm.arm.neon.vcadd.rot90
    1 << 8 | 1, // llvm.arm.neon.vcls
    1 << 8 | 1, // llvm.arm.neon.vcvtas
    1 << 8 | 1, // llvm.arm.neon.vcvtau
    1 << 8 | 1, // llvm.arm.neon.vcvtbfp2bf
    1 << 8 | 1, // llvm.arm.neon.vcvtfp2bf
    1 << 8 | 1, // llvm.arm.neon.vcvtfp2fxs
    1 << 8 | 1, // llvm.arm.neon.vcvtfp2fxu
    1 << 8 | 1, // llvm.arm.neon.vcvtfp2hf
    1 << 8 | 1, // llvm.arm.neon.vcvtfxs2fp
    1 << 8 | 1, // llvm.arm.neon.vcvtfxu2fp
    1 << 8 | 1, // llvm.arm.neon.vcvthf2fp
    1 << 8 | 1, // llvm.arm.neon.vcvtms
    1 << 8 | 1, // llvm.arm.neon.vcvtmu
    1 << 8 | 1, // llvm.arm.neon.vcvtns
    1 << 8 | 1, // llvm.arm.neon.vcvtnu
    1 << 8 | 1, // llvm.arm.neon.vcvtps
    1 << 8 | 1, // llvm.arm.neon.vcvtpu
    1 << 8 | 1, // llvm.arm.neon.vhadds
    1 << 8 | 1, // llvm.arm.neon.vhaddu
    1 << 8 | 1, // llvm.arm.neon.vhsubs
    1 << 8 | 1, // llvm.arm.neon.vhsubu
    2 << 8 | 1, // llvm.arm.neon.vld1
    2 << 8 | 1, // llvm.arm.neon.vld1x2
    2 << 8 | 1, // llvm.arm.neon.vld1x3
    2 << 8 | 1, // llvm.arm.neon.vld1x4
    2 << 8 | 1, // llvm.arm.neon.vld2
    2 << 8 | 1, // llvm.arm.neon.vld2dup
    2 << 8 | 1, // llvm.arm.neon.vld2lane
    2 << 8 | 1, // llvm.arm.neon.vld3
    2 << 8 | 1, // llvm.arm.neon.vld3dup
    2 << 8 | 1, // llvm.arm.neon.vld3lane
    2 << 8 | 1, // llvm.arm.neon.vld4
    2 << 8 | 1, // llvm.arm.neon.vld4dup
    2 << 8 | 1, // llvm.arm.neon.vld4lane
    1 << 8 | 1, // llvm.arm.neon.vmaxnm
    1 << 8 | 1, // llvm.arm.neon.vmaxs
    1 << 8 | 1, // llvm.arm.neon.vmaxu
    1 << 8 | 1, // llvm.arm.neon.vminnm
    1 << 8 | 1, // llvm.arm.neon.vmins
    1 << 8 | 1, // llvm.arm.neon.vminu
    1 << 8 | 1, // llvm.arm.neon.vmullp
    1 << 8 | 1, // llvm.arm.neon.vmulls
    1 << 8 | 1, // llvm.arm.neon.vmullu
    1 << 8 | 1, // llvm.arm.neon.vmulp
    1 << 8 | 1, // llvm.arm.neon.vpadals
    1 << 8 | 1, // llvm.arm.neon.vpadalu
    1 << 8 | 1, // llvm.arm.neon.vpadd
    1 << 8 | 1, // llvm.arm.neon.vpaddls
    1 << 8 | 1, // llvm.arm.neon.vpaddlu
    1 << 8 | 1, // llvm.arm.neon.vpmaxs
    1 << 8 | 1, // llvm.arm.neon.vpmaxu
    1 << 8 | 1, // llvm.arm.neon.vpmins
    1 << 8 | 1, // llvm.arm.neon.vpminu
    1 << 8 | 1, // llvm.arm.neon.vqabs
    1 << 8 | 1, // llvm.arm.neon.vqdmulh
    1 << 8 | 1, // llvm.arm.neon.vqdmull
    1 << 8 | 1, // llvm.arm.neon.vqmovns
    1 << 8 | 1, // llvm.arm.neon.vqmovnsu
    1 << 8 | 1, // llvm.arm.neon.vqmovnu
    1 << 8 | 1, // llvm.arm.neon.vqneg
    1 << 8 | 1, // llvm.arm.neon.vqrdmlah
    1 << 8 | 1, // llvm.arm.neon.vqrdmlsh
    1 << 8 | 1, // llvm.arm.neon.vqrdmulh
    1 << 8 | 1, // llvm.arm.neon.vqrshiftns
    1 << 8 | 1, // llvm.arm.neon.vqrshiftnsu
    1 << 8 | 1, // llvm.arm.neon.vqrshiftnu
    1 << 8 | 1, // llvm.arm.neon.vqrshifts
    1 << 8 | 1, // llvm.arm.neon.vqrshiftu
    1 << 8 | 1, // llvm.arm.neon.vqshiftns
    1 << 8 | 1, // llvm.arm.neon.vqshiftnsu
    1 << 8 | 1, // llvm.arm.neon.vqshiftnu
    1 << 8 | 1, // llvm.arm.neon.vqshifts
    1 << 8 | 1, // llvm.arm.neon.vqshiftsu
    1 << 8 | 1, // llvm.arm.neon.vqshiftu
    1 << 8 | 1, // llvm.arm.neon.vraddhn
    1 << 8 | 1, // llvm.arm.neon.vrecpe
    1 << 8 | 1, // llvm.arm.neon.vrecps
    1 << 8 | 1, // llvm.arm.neon.vrhadds
    1 << 8 | 1, // llvm.arm.neon.vrhaddu
    1 << 8 | 1, // llvm.arm.neon.vrshiftn
    1 << 8 | 1, // llvm.arm.neon.vrshifts
    1 << 8 | 1, // llvm.arm.neon.vrshiftu
    1 << 8 | 1, // llvm.arm.neon.vrsqrte
    1 << 8 | 1, // llvm.arm.neon.vrsqrts
    1 << 8 | 1, // llvm.arm.neon.vrsubhn
    1 << 8 | 1, // llvm.arm.neon.vshiftins
    1 << 8 | 1, // llvm.arm.neon.vshifts
    1 << 8 | 1, // llvm.arm.neon.vshiftu
    19 << 8 | 1, // llvm.arm.neon.vst1
    19 << 8 | 10, // llvm.arm.neon.vst1x2
    19 << 8 | 10, // llvm.arm.neon.vst1x3
    19 << 8 | 10, // llvm.arm.neon.vst1x4
    19 << 8 | 1, // llvm.arm.neon.vst2
    19 << 8 | 1, // llvm.arm.neon.vst2lane
    19 << 8 | 1, // llvm.arm.neon.vst3
    19 << 8 | 1, // llvm.arm.neon.vst3lane
    19 << 8 | 1, // llvm.arm.neon.vst4
    19 << 8 | 1, // llvm.arm.neon.vst4lane
    1 << 8 | 1, // llvm.arm.neon.vtbl1
    1 << 8 | 1, // llvm.arm.neon.vtbl2
    1 << 8 | 1, // llvm.arm.neon.vtbl3
    1 << 8 | 1, // llvm.arm.neon.vtbl4
    1 << 8 | 1, // llvm.arm.neon.vtbx1
    1 << 8 | 1, // llvm.arm.neon.vtbx2
    1 << 8 | 1, // llvm.arm.neon.vtbx3
    1 << 8 | 1, // llvm.arm.neon.vtbx4
    1 << 8 | 1, // llvm.arm.qadd
    1 << 8 | 1, // llvm.arm.qadd16
    1 << 8 | 1, // llvm.arm.qadd8
    1 << 8 | 1, // llvm.arm.qasx
    1 << 8 | 1, // llvm.arm.qsax
    1 << 8 | 1, // llvm.arm.qsub
    1 << 8 | 1, // llvm.arm.qsub16
    1 << 8 | 1, // llvm.arm.qsub8
    7 << 8 | 1, // llvm.arm.sadd16
    7 << 8 | 1, // llvm.arm.sadd8
    7 << 8 | 1, // llvm.arm.sasx
    23 << 8 | 1, // llvm.arm.sel
    7 << 8 | 1, // llvm.arm.set.fpscr
    1 << 8 | 1, // llvm.arm.shadd16
    1 << 8 | 1, // llvm.arm.shadd8
    1 << 8 | 1, // llvm.arm.shasx
    1 << 8 | 1, // llvm.arm.shsax
    1 << 8 | 1, // llvm.arm.shsub16
    1 << 8 | 1, // llvm.arm.shsub8
    1 << 8 | 1, // llvm.arm.smlabb
    1 << 8 | 1, // llvm.arm.smlabt
    1 << 8 | 1, // llvm.arm.smlad
    1 << 8 | 1, // llvm.arm.smladx
    1 << 8 | 1, // llvm.arm.smlald
    1 << 8 | 1, // llvm.arm.smlaldx
    1 << 8 | 1, // llvm.arm.smlatb
    1 << 8 | 1, // llvm.arm.smlatt
    1 << 8 | 1, // llvm.arm.smlawb
    1 << 8 | 1, // llvm.arm.smlawt
    1 << 8 | 1, // llvm.arm.smlsd
    1 << 8 | 1, // llvm.arm.smlsdx
    1 << 8 | 1, // llvm.arm.smlsld
    1 << 8 | 1, // llvm.arm.smlsldx
    1 << 8 | 1, // llvm.arm.smuad
    1 << 8 | 1, // llvm.arm.smuadx
    1 << 8 | 1, // llvm.arm.smulbb
    1 << 8 | 1, // llvm.arm.smulbt
    1 << 8 | 1, // llvm.arm.smultb
    1 << 8 | 1, // llvm.arm.smultt
    1 << 8 | 1, // llvm.arm.smulwb
    1 << 8 | 1, // llvm.arm.smulwt
    1 << 8 | 1, // llvm.arm.smusd
    1 << 8 | 1, // llvm.arm.smusdx
    5 << 8 | 5, // llvm.arm.space
    1 << 8 | 1, // llvm.arm.ssat
    1 << 8 | 1, // llvm.arm.ssat16
    7 << 8 | 1, // llvm.arm.ssax
    7 << 8 | 1, // llvm.arm.ssub16
    7 << 8 | 1, // llvm.arm.ssub8
    5 << 8 | 17, // llvm.arm.stc
    5 << 8 | 17, // llvm.arm.stc2
    5 << 8 | 17, // llvm.arm.stc2l
    5 << 8 | 17, // llvm.arm.stcl
    5 << 8 | 1, // llvm.arm.stlex
    5 << 8 | 1, // llvm.arm.stlexd
    5 << 8 | 1, // llvm.arm.strex
    5 << 8 | 1, // llvm.arm.strexd
    1 << 8 | 1, // llvm.arm.sxtab16
    1 << 8 | 1, // llvm.arm.sxtb16
    7 << 8 | 1, // llvm.arm.uadd16
    7 << 8 | 1, // llvm.arm.uadd8
    7 << 8 | 1, // llvm.arm.uasx
    1 << 8 | 1, // llvm.arm.uhadd16
    1 << 8 | 1, // llvm.arm.uhadd8
    1 << 8 | 1, // llvm.arm.uhasx
    1 << 8 | 1, // llvm.arm.uhsax
    1 << 8 | 1, // llvm.arm.uhsub16
    1 << 8 | 1, // llvm.arm.uhsub8
    5 << 8 | 1, // llvm.arm.undefined
    1 << 8 | 1, // llvm.arm.uqadd16
    1 << 8 | 1, // llvm.arm.uqadd8
    1 << 8 | 1, // llvm.arm.uqasx
    1 << 8 | 1, // llvm.arm.uqsax
    1 << 8 | 1, // llvm.arm.uqsub16
    1 << 8 | 1, // llvm.arm.uqsub8
    1 << 8 | 1, // llvm.arm.usad8
    1 << 8 | 1, // llvm.arm.usada8
    1 << 8 | 1, // llvm.arm.usat
    1 << 8 | 1, // llvm.arm.usat16
    7 << 8 | 1, // llvm.arm.usax
    7 << 8 | 1, // llvm.arm.usub16
    7 << 8 | 1, // llvm.arm.usub8
    1 << 8 | 1, // llvm.arm.uxtab16
    1 << 8 | 1, // llvm.arm.uxtb16
    1 << 8 | 1, // llvm.arm.vcvtr
    1 << 8 | 1, // llvm.arm.vcvtru
    10 << 8 | 1, // llvm.bpf.btf.type.id
    10 << 8 | 1, // llvm.bpf.compare
    53 << 8 | 135, // llvm.bpf.getelementptr.and.load
    53 << 8 | 136, // llvm.bpf.getelementptr.and.store
    23 << 8 | 1, // llvm.bpf.load.byte
    23 << 8 | 1, // llvm.bpf.load.half
    23 << 8 | 1, // llvm.bpf.load.word
    10 << 8 | 1, // llvm.bpf.passthrough
    10 << 8 | 1, // llvm.bpf.preserve.enum.value
    10 << 8 | 0, // llvm.bpf.preserve.field.info
    10 << 8 | 1, // llvm.bpf.preserve.type.info
    5 << 8 | 1, // llvm.bpf.pseudo
    1 << 8 | 1, // llvm.dx.all
    1 << 8 | 1, // llvm.dx.any
    1 << 8 | 1, // llvm.dx.asdouble
    1 << 8 | 1, // llvm.dx.cross
    1 << 8 | 1, // llvm.dx.ddx.coarse
    1 << 8 | 1, // llvm.dx.ddx.fine
    1 << 8 | 1, // llvm.dx.ddy.coarse
    1 << 8 | 1, // llvm.dx.ddy.fine
    1 << 8 | 1, // llvm.dx.degrees
    7 << 8 | 1, // llvm.dx.discard
    1 << 8 | 1, // llvm.dx.dot2
    1 << 8 | 1, // llvm.dx.dot2add
    1 << 8 | 1, // llvm.dx.dot3
    1 << 8 | 1, // llvm.dx.dot4
    1 << 8 | 1, // llvm.dx.dot4add.i8packed
    1 << 8 | 1, // llvm.dx.dot4add.u8packed
    1 << 8 | 1, // llvm.dx.fdot
    1 << 8 | 1, // llvm.dx.firstbitlow
    1 << 8 | 1, // llvm.dx.firstbitshigh
    1 << 8 | 1, // llvm.dx.firstbituhigh
    76 << 8 | 1, // llvm.dx.flattened.thread.id.in.group
    1 << 8 | 1, // llvm.dx.frac
    76 << 8 | 1, // llvm.dx.group.id
    61 << 8 | 1, // llvm.dx.group.memory.barrier.with.group.sync
    1 << 8 | 1, // llvm.dx.imad
    1 << 8 | 1, // llvm.dx.isinf
    1 << 8 | 1, // llvm.dx.isnan
    1 << 8 | 1, // llvm.dx.legacyf16tof32
    1 << 8 | 1, // llvm.dx.lerp
    61 << 8 | 1, // llvm.dx.load.input
    1 << 8 | 1, // llvm.dx.nclamp
    1 << 8 | 1, // llvm.dx.normalize
    1 << 8 | 1, // llvm.dx.radians
    5 << 8 | 1, // llvm.dx.resource.casthandle
    23 << 8 | 1, // llvm.dx.resource.getdimensions.x
    1 << 8 | 1, // llvm.dx.resource.getpointer
    1 << 8 | 1, // llvm.dx.resource.handlefrombinding
    1 << 8 | 1, // llvm.dx.resource.handlefromimplicitbinding
    23 << 8 | 1, // llvm.dx.resource.load.cbufferrow.2
    23 << 8 | 1, // llvm.dx.resource.load.cbufferrow.4
    23 << 8 | 1, // llvm.dx.resource.load.cbufferrow.8
    23 << 8 | 1, // llvm.dx.resource.load.rawbuffer
    23 << 8 | 1, // llvm.dx.resource.load.typedbuffer
    1 << 8 | 1, // llvm.dx.resource.nonuniformindex
    24 << 8 | 1, // llvm.dx.resource.store.rawbuffer
    24 << 8 | 1, // llvm.dx.resource.store.typedbuffer
    28 << 8 | 1, // llvm.dx.resource.updatecounter
    1 << 8 | 1, // llvm.dx.rsqrt
    1 << 8 | 1, // llvm.dx.saturate
    1 << 8 | 1, // llvm.dx.sclamp
    1 << 8 | 1, // llvm.dx.sdot
    1 << 8 | 1, // llvm.dx.sign
    1 << 8 | 1, // llvm.dx.splitdouble
    1 << 8 | 1, // llvm.dx.step
    61 << 8 | 1, // llvm.dx.store.output
    76 << 8 | 1, // llvm.dx.thread.id
    76 << 8 | 1, // llvm.dx.thread.id.in.group
    1 << 8 | 1, // llvm.dx.uclamp
    1 << 8 | 1, // llvm.dx.udot
    1 << 8 | 1, // llvm.dx.umad
    15 << 8 | 1, // llvm.dx.wave.active.countbits
    15 << 8 | 1, // llvm.dx.wave.all
    15 << 8 | 1, // llvm.dx.wave.any
    15 << 8 | 1, // llvm.dx.wave.ballot
    61 << 8 | 1, // llvm.dx.wave.get.lane.count
    15 << 8 | 1, // llvm.dx.wave.getlaneindex
    61 << 8 | 1, // llvm.dx.wave.is.first.lane
    15 << 8 | 1, // llvm.dx.wave.readlane
    15 << 8 | 1, // llvm.dx.wave.reduce.max
    15 << 8 | 1, // llvm.dx.wave.reduce.min
    15 << 8 | 1, // llvm.dx.wave.reduce.sum
    15 << 8 | 1, // llvm.dx.wave.reduce.umax
    15 << 8 | 1, // llvm.dx.wave.reduce.umin
    15 << 8 | 1, // llvm.dx.wave.reduce.usum
    1 << 8 | 1, // llvm.hexagon.A2.abs
    1 << 8 | 1, // llvm.hexagon.A2.absp
    1 << 8 | 1, // llvm.hexagon.A2.abssat
    1 << 8 | 1, // llvm.hexagon.A2.add
    1 << 8 | 1, // llvm.hexagon.A2.addh.h16.hh
    1 << 8 | 1, // llvm.hexagon.A2.addh.h16.hl
    1 << 8 | 1, // llvm.hexagon.A2.addh.h16.lh
    1 << 8 | 1, // llvm.hexagon.A2.addh.h16.ll
    1 << 8 | 1, // llvm.hexagon.A2.addh.h16.sat.hh
    1 << 8 | 1, // llvm.hexagon.A2.addh.h16.sat.hl
    1 << 8 | 1, // llvm.hexagon.A2.addh.h16.sat.lh
    1 << 8 | 1, // llvm.hexagon.A2.addh.h16.sat.ll
    1 << 8 | 1, // llvm.hexagon.A2.addh.l16.hl
    1 << 8 | 1, // llvm.hexagon.A2.addh.l16.ll
    1 << 8 | 1, // llvm.hexagon.A2.addh.l16.sat.hl
    1 << 8 | 1, // llvm.hexagon.A2.addh.l16.sat.ll
    1 << 8 | 0, // llvm.hexagon.A2.addi
    1 << 8 | 1, // llvm.hexagon.A2.addp
    1 << 8 | 1, // llvm.hexagon.A2.addpsat
    1 << 8 | 1, // llvm.hexagon.A2.addsat
    1 << 8 | 1, // llvm.hexagon.A2.addsp
    1 << 8 | 1, // llvm.hexagon.A2.and
    1 << 8 | 0, // llvm.hexagon.A2.andir
    1 << 8 | 1, // llvm.hexagon.A2.andp
    1 << 8 | 1, // llvm.hexagon.A2.aslh
    1 << 8 | 1, // llvm.hexagon.A2.asrh
    1 << 8 | 1, // llvm.hexagon.A2.combine.hh
    1 << 8 | 1, // llvm.hexagon.A2.combine.hl
    1 << 8 | 1, // llvm.hexagon.A2.combine.lh
    1 << 8 | 1, // llvm.hexagon.A2.combine.ll
    1 << 8 | 17, // llvm.hexagon.A2.combineii
    1 << 8 | 1, // llvm.hexagon.A2.combinew
    1 << 8 | 1, // llvm.hexagon.A2.max
    1 << 8 | 1, // llvm.hexagon.A2.maxp
    1 << 8 | 1, // llvm.hexagon.A2.maxu
    1 << 8 | 1, // llvm.hexagon.A2.maxup
    1 << 8 | 1, // llvm.hexagon.A2.min
    1 << 8 | 1, // llvm.hexagon.A2.minp
    1 << 8 | 1, // llvm.hexagon.A2.minu
    1 << 8 | 1, // llvm.hexagon.A2.minup
    1 << 8 | 1, // llvm.hexagon.A2.neg
    1 << 8 | 1, // llvm.hexagon.A2.negp
    1 << 8 | 1, // llvm.hexagon.A2.negsat
    1 << 8 | 1, // llvm.hexagon.A2.not
    1 << 8 | 1, // llvm.hexagon.A2.notp
    1 << 8 | 1, // llvm.hexagon.A2.or
    1 << 8 | 0, // llvm.hexagon.A2.orir
    1 << 8 | 1, // llvm.hexagon.A2.orp
    1 << 8 | 1, // llvm.hexagon.A2.roundsat
    1 << 8 | 1, // llvm.hexagon.A2.sat
    1 << 8 | 1, // llvm.hexagon.A2.satb
    1 << 8 | 1, // llvm.hexagon.A2.sath
    1 << 8 | 1, // llvm.hexagon.A2.satub
    1 << 8 | 1, // llvm.hexagon.A2.satuh
    1 << 8 | 1, // llvm.hexagon.A2.sub
    1 << 8 | 1, // llvm.hexagon.A2.subh.h16.hh
    1 << 8 | 1, // llvm.hexagon.A2.subh.h16.hl
    1 << 8 | 1, // llvm.hexagon.A2.subh.h16.lh
    1 << 8 | 1, // llvm.hexagon.A2.subh.h16.ll
    1 << 8 | 1, // llvm.hexagon.A2.subh.h16.sat.hh
    1 << 8 | 1, // llvm.hexagon.A2.subh.h16.sat.hl
    1 << 8 | 1, // llvm.hexagon.A2.subh.h16.sat.lh
    1 << 8 | 1, // llvm.hexagon.A2.subh.h16.sat.ll
    1 << 8 | 1, // llvm.hexagon.A2.subh.l16.hl
    1 << 8 | 1, // llvm.hexagon.A2.subh.l16.ll
    1 << 8 | 1, // llvm.hexagon.A2.subh.l16.sat.hl
    1 << 8 | 1, // llvm.hexagon.A2.subh.l16.sat.ll
    1 << 8 | 1, // llvm.hexagon.A2.subp
    1 << 8 | 5, // llvm.hexagon.A2.subri
    1 << 8 | 1, // llvm.hexagon.A2.subsat
    1 << 8 | 1, // llvm.hexagon.A2.svaddh
    1 << 8 | 1, // llvm.hexagon.A2.svaddhs
    1 << 8 | 1, // llvm.hexagon.A2.svadduhs
    1 << 8 | 1, // llvm.hexagon.A2.svavgh
    1 << 8 | 1, // llvm.hexagon.A2.svavghs
    1 << 8 | 1, // llvm.hexagon.A2.svnavgh
    1 << 8 | 1, // llvm.hexagon.A2.svsubh
    1 << 8 | 1, // llvm.hexagon.A2.svsubhs
    1 << 8 | 1, // llvm.hexagon.A2.svsubuhs
    1 << 8 | 1, // llvm.hexagon.A2.swiz
    1 << 8 | 1, // llvm.hexagon.A2.sxtb
    1 << 8 | 1, // llvm.hexagon.A2.sxth
    1 << 8 | 1, // llvm.hexagon.A2.sxtw
    1 << 8 | 1, // llvm.hexagon.A2.tfr
    1 << 8 | 0, // llvm.hexagon.A2.tfrih
    1 << 8 | 0, // llvm.hexagon.A2.tfril
    1 << 8 | 1, // llvm.hexagon.A2.tfrp
    1 << 8 | 5, // llvm.hexagon.A2.tfrpi
    1 << 8 | 5, // llvm.hexagon.A2.tfrsi
    1 << 8 | 1, // llvm.hexagon.A2.vabsh
    1 << 8 | 1, // llvm.hexagon.A2.vabshsat
    1 << 8 | 1, // llvm.hexagon.A2.vabsw
    1 << 8 | 1, // llvm.hexagon.A2.vabswsat
    1 << 8 | 1, // llvm.hexagon.A2.vaddb.map
    1 << 8 | 1, // llvm.hexagon.A2.vaddh
    1 << 8 | 1, // llvm.hexagon.A2.vaddhs
    1 << 8 | 1, // llvm.hexagon.A2.vaddub
    1 << 8 | 1, // llvm.hexagon.A2.vaddubs
    1 << 8 | 1, // llvm.hexagon.A2.vadduhs
    1 << 8 | 1, // llvm.hexagon.A2.vaddw
    1 << 8 | 1, // llvm.hexagon.A2.vaddws
    1 << 8 | 1, // llvm.hexagon.A2.vavgh
    1 << 8 | 1, // llvm.hexagon.A2.vavghcr
    1 << 8 | 1, // llvm.hexagon.A2.vavghr
    1 << 8 | 1, // llvm.hexagon.A2.vavgub
    1 << 8 | 1, // llvm.hexagon.A2.vavgubr
    1 << 8 | 1, // llvm.hexagon.A2.vavguh
    1 << 8 | 1, // llvm.hexagon.A2.vavguhr
    1 << 8 | 1, // llvm.hexagon.A2.vavguw
    1 << 8 | 1, // llvm.hexagon.A2.vavguwr
    1 << 8 | 1, // llvm.hexagon.A2.vavgw
    1 << 8 | 1, // llvm.hexagon.A2.vavgwcr
    1 << 8 | 1, // llvm.hexagon.A2.vavgwr
    1 << 8 | 1, // llvm.hexagon.A2.vcmpbeq
    1 << 8 | 1, // llvm.hexagon.A2.vcmpbgtu
    1 << 8 | 1, // llvm.hexagon.A2.vcmpheq
    1 << 8 | 1, // llvm.hexagon.A2.vcmphgt
    1 << 8 | 1, // llvm.hexagon.A2.vcmphgtu
    1 << 8 | 1, // llvm.hexagon.A2.vcmpweq
    1 << 8 | 1, // llvm.hexagon.A2.vcmpwgt
    1 << 8 | 1, // llvm.hexagon.A2.vcmpwgtu
    1 << 8 | 1, // llvm.hexagon.A2.vconj
    1 << 8 | 1, // llvm.hexagon.A2.vmaxb
    1 << 8 | 1, // llvm.hexagon.A2.vmaxh
    1 << 8 | 1, // llvm.hexagon.A2.vmaxub
    1 << 8 | 1, // llvm.hexagon.A2.vmaxuh
    1 << 8 | 1, // llvm.hexagon.A2.vmaxuw
    1 << 8 | 1, // llvm.hexagon.A2.vmaxw
    1 << 8 | 1, // llvm.hexagon.A2.vminb
    1 << 8 | 1, // llvm.hexagon.A2.vminh
    1 << 8 | 1, // llvm.hexagon.A2.vminub
    1 << 8 | 1, // llvm.hexagon.A2.vminuh
    1 << 8 | 1, // llvm.hexagon.A2.vminuw
    1 << 8 | 1, // llvm.hexagon.A2.vminw
    1 << 8 | 1, // llvm.hexagon.A2.vnavgh
    1 << 8 | 1, // llvm.hexagon.A2.vnavghcr
    1 << 8 | 1, // llvm.hexagon.A2.vnavghr
    1 << 8 | 1, // llvm.hexagon.A2.vnavgw
    1 << 8 | 1, // llvm.hexagon.A2.vnavgwcr
    1 << 8 | 1, // llvm.hexagon.A2.vnavgwr
    1 << 8 | 1, // llvm.hexagon.A2.vraddub
    1 << 8 | 1, // llvm.hexagon.A2.vraddub.acc
    1 << 8 | 1, // llvm.hexagon.A2.vrsadub
    1 << 8 | 1, // llvm.hexagon.A2.vrsadub.acc
    1 << 8 | 1, // llvm.hexagon.A2.vsubb.map
    1 << 8 | 1, // llvm.hexagon.A2.vsubh
    1 << 8 | 1, // llvm.hexagon.A2.vsubhs
    1 << 8 | 1, // llvm.hexagon.A2.vsubub
    1 << 8 | 1, // llvm.hexagon.A2.vsububs
    1 << 8 | 1, // llvm.hexagon.A2.vsubuhs
    1 << 8 | 1, // llvm.hexagon.A2.vsubw
    1 << 8 | 1, // llvm.hexagon.A2.vsubws
    1 << 8 | 1, // llvm.hexagon.A2.xor
    1 << 8 | 1, // llvm.hexagon.A2.xorp
    1 << 8 | 1, // llvm.hexagon.A2.zxtb
    1 << 8 | 1, // llvm.hexagon.A2.zxth
    1 << 8 | 1, // llvm.hexagon.A4.andn
    1 << 8 | 1, // llvm.hexagon.A4.andnp
    1 << 8 | 1, // llvm.hexagon.A4.bitsplit
    1 << 8 | 0, // llvm.hexagon.A4.bitspliti
    1 << 8 | 1, // llvm.hexagon.A4.boundscheck
    1 << 8 | 1, // llvm.hexagon.A4.cmpbeq
    1 << 8 | 0, // llvm.hexagon.A4.cmpbeqi
    1 << 8 | 1, // llvm.hexagon.A4.cmpbgt
    1 << 8 | 0, // llvm.hexagon.A4.cmpbgti
    1 << 8 | 1, // llvm.hexagon.A4.cmpbgtu
    1 << 8 | 0, // llvm.hexagon.A4.cmpbgtui
    1 << 8 | 1, // llvm.hexagon.A4.cmpheq
    1 << 8 | 0, // llvm.hexagon.A4.cmpheqi
    1 << 8 | 1, // llvm.hexagon.A4.cmphgt
    1 << 8 | 0, // llvm.hexagon.A4.cmphgti
    1 << 8 | 1, // llvm.hexagon.A4.cmphgtu
    1 << 8 | 0, // llvm.hexagon.A4.cmphgtui
    1 << 8 | 5, // llvm.hexagon.A4.combineir
    1 << 8 | 0, // llvm.hexagon.A4.combineri
    1 << 8 | 0, // llvm.hexagon.A4.cround.ri
    1 << 8 | 1, // llvm.hexagon.A4.cround.rr
    1 << 8 | 1, // llvm.hexagon.A4.modwrapu
    1 << 8 | 1, // llvm.hexagon.A4.orn
    1 << 8 | 1, // llvm.hexagon.A4.ornp
    1 << 8 | 1, // llvm.hexagon.A4.rcmpeq
    1 << 8 | 0, // llvm.hexagon.A4.rcmpeqi
    1 << 8 | 1, // llvm.hexagon.A4.rcmpneq
    1 << 8 | 0, // llvm.hexagon.A4.rcmpneqi
    1 << 8 | 0, // llvm.hexagon.A4.round.ri
    1 << 8 | 0, // llvm.hexagon.A4.round.ri.sat
    1 << 8 | 1, // llvm.hexagon.A4.round.rr
    1 << 8 | 1, // llvm.hexagon.A4.round.rr.sat
    1 << 8 | 1, // llvm.hexagon.A4.tlbmatch
    1 << 8 | 1, // llvm.hexagon.A4.vcmpbeq.any
    1 << 8 | 0, // llvm.hexagon.A4.vcmpbeqi
    1 << 8 | 1, // llvm.hexagon.A4.vcmpbgt
    1 << 8 | 0, // llvm.hexagon.A4.vcmpbgti
    1 << 8 | 0, // llvm.hexagon.A4.vcmpbgtui
    1 << 8 | 0, // llvm.hexagon.A4.vcmpheqi
    1 << 8 | 0, // llvm.hexagon.A4.vcmphgti
    1 << 8 | 0, // llvm.hexagon.A4.vcmphgtui
    1 << 8 | 0, // llvm.hexagon.A4.vcmpweqi
    1 << 8 | 0, // llvm.hexagon.A4.vcmpwgti
    1 << 8 | 0, // llvm.hexagon.A4.vcmpwgtui
    1 << 8 | 1, // llvm.hexagon.A4.vrmaxh
    1 << 8 | 1, // llvm.hexagon.A4.vrmaxuh
    1 << 8 | 1, // llvm.hexagon.A4.vrmaxuw
    1 << 8 | 1, // llvm.hexagon.A4.vrmaxw
    1 << 8 | 1, // llvm.hexagon.A4.vrminh
    1 << 8 | 1, // llvm.hexagon.A4.vrminuh
    1 << 8 | 1, // llvm.hexagon.A4.vrminuw
    1 << 8 | 1, // llvm.hexagon.A4.vrminw
    1 << 8 | 1, // llvm.hexagon.A5.vaddhubs
    1 << 8 | 1, // llvm.hexagon.A6.vcmpbeq.notany
    1 << 8 | 0, // llvm.hexagon.A7.clip
    1 << 8 | 0, // llvm.hexagon.A7.croundd.ri
    1 << 8 | 1, // llvm.hexagon.A7.croundd.rr
    1 << 8 | 0, // llvm.hexagon.A7.vclip
    1 << 8 | 1, // llvm.hexagon.C2.all8
    1 << 8 | 1, // llvm.hexagon.C2.and
    1 << 8 | 1, // llvm.hexagon.C2.andn
    1 << 8 | 1, // llvm.hexagon.C2.any8
    1 << 8 | 1, // llvm.hexagon.C2.bitsclr
    1 << 8 | 0, // llvm.hexagon.C2.bitsclri
    1 << 8 | 1, // llvm.hexagon.C2.bitsset
    1 << 8 | 1, // llvm.hexagon.C2.cmpeq
    1 << 8 | 0, // llvm.hexagon.C2.cmpeqi
    1 << 8 | 1, // llvm.hexagon.C2.cmpeqp
    1 << 8 | 0, // llvm.hexagon.C2.cmpgei
    1 << 8 | 0, // llvm.hexagon.C2.cmpgeui
    1 << 8 | 1, // llvm.hexagon.C2.cmpgt
    1 << 8 | 0, // llvm.hexagon.C2.cmpgti
    1 << 8 | 1, // llvm.hexagon.C2.cmpgtp
    1 << 8 | 1, // llvm.hexagon.C2.cmpgtu
    1 << 8 | 0, // llvm.hexagon.C2.cmpgtui
    1 << 8 | 1, // llvm.hexagon.C2.cmpgtup
    1 << 8 | 1, // llvm.hexagon.C2.cmplt
    1 << 8 | 1, // llvm.hexagon.C2.cmpltu
    1 << 8 | 1, // llvm.hexagon.C2.mask
    1 << 8 | 1, // llvm.hexagon.C2.mux
    1 << 8 | 13, // llvm.hexagon.C2.muxii
    1 << 8 | 11, // llvm.hexagon.C2.muxir
    1 << 8 | 0, // llvm.hexagon.C2.muxri
    1 << 8 | 1, // llvm.hexagon.C2.not
    1 << 8 | 1, // llvm.hexagon.C2.or
    1 << 8 | 1, // llvm.hexagon.C2.orn
    1 << 8 | 1, // llvm.hexagon.C2.pxfer.map
    1 << 8 | 1, // llvm.hexagon.C2.tfrpr
    1 << 8 | 1, // llvm.hexagon.C2.tfrrp
    1 << 8 | 1, // llvm.hexagon.C2.vitpack
    1 << 8 | 1, // llvm.hexagon.C2.vmux
    1 << 8 | 1, // llvm.hexagon.C2.xor
    1 << 8 | 1, // llvm.hexagon.C4.and.and
    1 << 8 | 1, // llvm.hexagon.C4.and.andn
    1 << 8 | 1, // llvm.hexagon.C4.and.or
    1 << 8 | 1, // llvm.hexagon.C4.and.orn
    1 << 8 | 1, // llvm.hexagon.C4.cmplte
    1 << 8 | 0, // llvm.hexagon.C4.cmpltei
    1 << 8 | 1, // llvm.hexagon.C4.cmplteu
    1 << 8 | 0, // llvm.hexagon.C4.cmplteui
    1 << 8 | 1, // llvm.hexagon.C4.cmpneq
    1 << 8 | 0, // llvm.hexagon.C4.cmpneqi
    1 << 8 | 1, // llvm.hexagon.C4.fastcorner9
    1 << 8 | 1, // llvm.hexagon.C4.fastcorner9.not
    1 << 8 | 1, // llvm.hexagon.C4.nbitsclr
    1 << 8 | 0, // llvm.hexagon.C4.nbitsclri
    1 << 8 | 1, // llvm.hexagon.C4.nbitsset
    1 << 8 | 1, // llvm.hexagon.C4.or.and
    1 << 8 | 1, // llvm.hexagon.C4.or.andn
    1 << 8 | 1, // llvm.hexagon.C4.or.or
    1 << 8 | 1, // llvm.hexagon.C4.or.orn
    1 << 8 | 1, // llvm.hexagon.F2.conv.d2df
    1 << 8 | 1, // llvm.hexagon.F2.conv.d2sf
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2d
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2d.chop
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2sf
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2ud
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2ud.chop
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2uw
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2uw.chop
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2w
    1 << 8 | 1, // llvm.hexagon.F2.conv.df2w.chop
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2d
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2d.chop
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2df
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2ud
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2ud.chop
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2uw
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2uw.chop
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2w
    1 << 8 | 1, // llvm.hexagon.F2.conv.sf2w.chop
    1 << 8 | 1, // llvm.hexagon.F2.conv.ud2df
    1 << 8 | 1, // llvm.hexagon.F2.conv.ud2sf
    1 << 8 | 1, // llvm.hexagon.F2.conv.uw2df
    1 << 8 | 1, // llvm.hexagon.F2.conv.uw2sf
    1 << 8 | 1, // llvm.hexagon.F2.conv.w2df
    1 << 8 | 1, // llvm.hexagon.F2.conv.w2sf
    77 << 8 | 1, // llvm.hexagon.F2.dfadd
    77 << 8 | 0, // llvm.hexagon.F2.dfclass
    77 << 8 | 1, // llvm.hexagon.F2.dfcmpeq
    77 << 8 | 1, // llvm.hexagon.F2.dfcmpge
    77 << 8 | 1, // llvm.hexagon.F2.dfcmpgt
    77 << 8 | 1, // llvm.hexagon.F2.dfcmpuo
    77 << 8 | 5, // llvm.hexagon.F2.dfimm.n
    77 << 8 | 5, // llvm.hexagon.F2.dfimm.p
    77 << 8 | 1, // llvm.hexagon.F2.dfmax
    77 << 8 | 1, // llvm.hexagon.F2.dfmin
    77 << 8 | 1, // llvm.hexagon.F2.dfmpyfix
    77 << 8 | 1, // llvm.hexagon.F2.dfmpyhh
    77 << 8 | 1, // llvm.hexagon.F2.dfmpylh
    77 << 8 | 1, // llvm.hexagon.F2.dfmpyll
    77 << 8 | 1, // llvm.hexagon.F2.dfsub
    77 << 8 | 1, // llvm.hexagon.F2.sfadd
    77 << 8 | 0, // llvm.hexagon.F2.sfclass
    77 << 8 | 1, // llvm.hexagon.F2.sfcmpeq
    77 << 8 | 1, // llvm.hexagon.F2.sfcmpge
    77 << 8 | 1, // llvm.hexagon.F2.sfcmpgt
    77 << 8 | 1, // llvm.hexagon.F2.sfcmpuo
    77 << 8 | 1, // llvm.hexagon.F2.sffixupd
    77 << 8 | 1, // llvm.hexagon.F2.sffixupn
    77 << 8 | 1, // llvm.hexagon.F2.sffixupr
    77 << 8 | 1, // llvm.hexagon.F2.sffma
    77 << 8 | 1, // llvm.hexagon.F2.sffma.lib
    77 << 8 | 1, // llvm.hexagon.F2.sffma.sc
    77 << 8 | 1, // llvm.hexagon.F2.sffms
    77 << 8 | 1, // llvm.hexagon.F2.sffms.lib
    77 << 8 | 5, // llvm.hexagon.F2.sfimm.n
    77 << 8 | 5, // llvm.hexagon.F2.sfimm.p
    77 << 8 | 1, // llvm.hexagon.F2.sfmax
    77 << 8 | 1, // llvm.hexagon.F2.sfmin
    77 << 8 | 1, // llvm.hexagon.F2.sfmpy
    77 << 8 | 1, // llvm.hexagon.F2.sfsub
    23 << 8 | 1, // llvm.hexagon.L2.loadrb.pbr
    18 << 8 | 38, // llvm.hexagon.L2.loadrb.pci
    18 << 8 | 37, // llvm.hexagon.L2.loadrb.pcr
    23 << 8 | 1, // llvm.hexagon.L2.loadrd.pbr
    18 << 8 | 38, // llvm.hexagon.L2.loadrd.pci
    18 << 8 | 37, // llvm.hexagon.L2.loadrd.pcr
    23 << 8 | 1, // llvm.hexagon.L2.loadrh.pbr
    18 << 8 | 38, // llvm.hexagon.L2.loadrh.pci
    18 << 8 | 37, // llvm.hexagon.L2.loadrh.pcr
    23 << 8 | 1, // llvm.hexagon.L2.loadri.pbr
    18 << 8 | 38, // llvm.hexagon.L2.loadri.pci
    18 << 8 | 37, // llvm.hexagon.L2.loadri.pcr
    23 << 8 | 1, // llvm.hexagon.L2.loadrub.pbr
    18 << 8 | 38, // llvm.hexagon.L2.loadrub.pci
    18 << 8 | 37, // llvm.hexagon.L2.loadrub.pcr
    23 << 8 | 1, // llvm.hexagon.L2.loadruh.pbr
    18 << 8 | 38, // llvm.hexagon.L2.loadruh.pci
    18 << 8 | 37, // llvm.hexagon.L2.loadruh.pcr
    11 << 8 | 10, // llvm.hexagon.L2.loadw.locked
    11 << 8 | 10, // llvm.hexagon.L4.loadd.locked
    1 << 8 | 1, // llvm.hexagon.M2.acci
    1 << 8 | 11, // llvm.hexagon.M2.accii
    1 << 8 | 1, // llvm.hexagon.M2.cmaci.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmacr.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmacs.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmacs.s1
    1 << 8 | 1, // llvm.hexagon.M2.cmacsc.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmacsc.s1
    1 << 8 | 1, // llvm.hexagon.M2.cmpyi.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmpyr.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmpyrs.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmpyrs.s1
    1 << 8 | 1, // llvm.hexagon.M2.cmpyrsc.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmpyrsc.s1
    1 << 8 | 1, // llvm.hexagon.M2.cmpys.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmpys.s1
    1 << 8 | 1, // llvm.hexagon.M2.cmpysc.s0
    1 << 8 | 1, // llvm.hexagon.M2.cmpysc.s1
    1 << 8 | 1, // llvm.hexagon.M2.cnacs.s0
    1 << 8 | 1, // llvm.hexagon.M2.cnacs.s1
    1 << 8 | 1, // llvm.hexagon.M2.cnacsc.s0
    1 << 8 | 1, // llvm.hexagon.M2.cnacsc.s1
    1 << 8 | 1, // llvm.hexagon.M2.dpmpyss.acc.s0
    1 << 8 | 1, // llvm.hexagon.M2.dpmpyss.nac.s0
    1 << 8 | 1, // llvm.hexagon.M2.dpmpyss.rnd.s0
    1 << 8 | 1, // llvm.hexagon.M2.dpmpyss.s0
    1 << 8 | 1, // llvm.hexagon.M2.dpmpyuu.acc.s0
    1 << 8 | 1, // llvm.hexagon.M2.dpmpyuu.nac.s0
    1 << 8 | 1, // llvm.hexagon.M2.dpmpyuu.s0
    1 << 8 | 1, // llvm.hexagon.M2.hmmpyh.rs1
    1 << 8 | 1, // llvm.hexagon.M2.hmmpyh.s1
    1 << 8 | 1, // llvm.hexagon.M2.hmmpyl.rs1
    1 << 8 | 1, // llvm.hexagon.M2.hmmpyl.s1
    1 << 8 | 1, // llvm.hexagon.M2.maci
    1 << 8 | 11, // llvm.hexagon.M2.macsin
    1 << 8 | 11, // llvm.hexagon.M2.macsip
    1 << 8 | 1, // llvm.hexagon.M2.mmachs.rs0
    1 << 8 | 1, // llvm.hexagon.M2.mmachs.rs1
    1 << 8 | 1, // llvm.hexagon.M2.mmachs.s0
    1 << 8 | 1, // llvm.hexagon.M2.mmachs.s1
    1 << 8 | 1, // llvm.hexagon.M2.mmacls.rs0
    1 << 8 | 1, // llvm.hexagon.M2.mmacls.rs1
    1 << 8 | 1, // llvm.hexagon.M2.mmacls.s0
    1 << 8 | 1, // llvm.hexagon.M2.mmacls.s1
    1 << 8 | 1, // llvm.hexagon.M2.mmacuhs.rs0
    1 << 8 | 1, // llvm.hexagon.M2.mmacuhs.rs1
    1 << 8 | 1, // llvm.hexagon.M2.mmacuhs.s0
    1 << 8 | 1, // llvm.hexagon.M2.mmacuhs.s1
    1 << 8 | 1, // llvm.hexagon.M2.mmaculs.rs0
    1 << 8 | 1, // llvm.hexagon.M2.mmaculs.rs1
    1 << 8 | 1, // llvm.hexagon.M2.mmaculs.s0
    1 << 8 | 1, // llvm.hexagon.M2.mmaculs.s1
    1 << 8 | 1, // llvm.hexagon.M2.mmpyh.rs0
    1 << 8 | 1, // llvm.hexagon.M2.mmpyh.rs1
    1 << 8 | 1, // llvm.hexagon.M2.mmpyh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mmpyh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mmpyl.rs0
    1 << 8 | 1, // llvm.hexagon.M2.mmpyl.rs1
    1 << 8 | 1, // llvm.hexagon.M2.mmpyl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mmpyl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mmpyuh.rs0
    1 << 8 | 1, // llvm.hexagon.M2.mmpyuh.rs1
    1 << 8 | 1, // llvm.hexagon.M2.mmpyuh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mmpyuh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mmpyul.rs0
    1 << 8 | 1, // llvm.hexagon.M2.mmpyul.rs1
    1 << 8 | 1, // llvm.hexagon.M2.mmpyul.s0
    1 << 8 | 1, // llvm.hexagon.M2.mmpyul.s1
    1 << 8 | 1, // llvm.hexagon.M2.mnaci
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.sat.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.sat.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.sat.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.sat.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.sat.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.sat.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.sat.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.acc.sat.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.sat.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.sat.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.sat.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.sat.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.sat.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.sat.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.sat.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.nac.sat.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.rnd.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.rnd.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.rnd.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.rnd.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.rnd.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.rnd.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.rnd.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.rnd.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.rnd.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.rnd.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.rnd.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.rnd.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.rnd.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.rnd.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.rnd.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpy.sat.rnd.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.up
    1 << 8 | 1, // llvm.hexagon.M2.mpy.up.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpy.up.s1.sat
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.acc.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.acc.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.acc.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.acc.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.acc.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.acc.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.acc.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.acc.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.nac.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.nac.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.nac.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.nac.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.nac.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.nac.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.nac.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.nac.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.rnd.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.rnd.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.rnd.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.rnd.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.rnd.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.rnd.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.rnd.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyd.rnd.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyi
    1 << 8 | 0, // llvm.hexagon.M2.mpysmi
    1 << 8 | 1, // llvm.hexagon.M2.mpysu.up
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.acc.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.acc.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.acc.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.acc.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.acc.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.acc.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.acc.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.acc.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.nac.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.nac.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.nac.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.nac.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.nac.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.nac.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.nac.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.nac.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyu.up
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.acc.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.acc.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.acc.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.acc.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.acc.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.acc.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.acc.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.acc.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.nac.hh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.nac.hh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.nac.hl.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.nac.hl.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.nac.lh.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.nac.lh.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.nac.ll.s0
    1 << 8 | 1, // llvm.hexagon.M2.mpyud.nac.ll.s1
    1 << 8 | 1, // llvm.hexagon.M2.mpyui
    1 << 8 | 1, // llvm.hexagon.M2.nacci
    1 << 8 | 11, // llvm.hexagon.M2.naccii
    1 << 8 | 1, // llvm.hexagon.M2.subacc
    1 << 8 | 1, // llvm.hexagon.M2.vabsdiffh
    1 << 8 | 1, // llvm.hexagon.M2.vabsdiffw
    1 << 8 | 1, // llvm.hexagon.M2.vcmac.s0.sat.i
    1 << 8 | 1, // llvm.hexagon.M2.vcmac.s0.sat.r
    1 << 8 | 1, // llvm.hexagon.M2.vcmpy.s0.sat.i
    1 << 8 | 1, // llvm.hexagon.M2.vcmpy.s0.sat.r
    1 << 8 | 1, // llvm.hexagon.M2.vcmpy.s1.sat.i
    1 << 8 | 1, // llvm.hexagon.M2.vcmpy.s1.sat.r
    1 << 8 | 1, // llvm.hexagon.M2.vdmacs.s0
    1 << 8 | 1, // llvm.hexagon.M2.vdmacs.s1
    1 << 8 | 1, // llvm.hexagon.M2.vdmpyrs.s0
    1 << 8 | 1, // llvm.hexagon.M2.vdmpyrs.s1
    1 << 8 | 1, // llvm.hexagon.M2.vdmpys.s0
    1 << 8 | 1, // llvm.hexagon.M2.vdmpys.s1
    1 << 8 | 1, // llvm.hexagon.M2.vmac2
    1 << 8 | 1, // llvm.hexagon.M2.vmac2es
    1 << 8 | 1, // llvm.hexagon.M2.vmac2es.s0
    1 << 8 | 1, // llvm.hexagon.M2.vmac2es.s1
    1 << 8 | 1, // llvm.hexagon.M2.vmac2s.s0
    1 << 8 | 1, // llvm.hexagon.M2.vmac2s.s1
    1 << 8 | 1, // llvm.hexagon.M2.vmac2su.s0
    1 << 8 | 1, // llvm.hexagon.M2.vmac2su.s1
    1 << 8 | 1, // llvm.hexagon.M2.vmpy2es.s0
    1 << 8 | 1, // llvm.hexagon.M2.vmpy2es.s1
    1 << 8 | 1, // llvm.hexagon.M2.vmpy2s.s0
    1 << 8 | 1, // llvm.hexagon.M2.vmpy2s.s0pack
    1 << 8 | 1, // llvm.hexagon.M2.vmpy2s.s1
    1 << 8 | 1, // llvm.hexagon.M2.vmpy2s.s1pack
    1 << 8 | 1, // llvm.hexagon.M2.vmpy2su.s0
    1 << 8 | 1, // llvm.hexagon.M2.vmpy2su.s1
    1 << 8 | 1, // llvm.hexagon.M2.vraddh
    1 << 8 | 1, // llvm.hexagon.M2.vradduh
    1 << 8 | 1, // llvm.hexagon.M2.vrcmaci.s0
    1 << 8 | 1, // llvm.hexagon.M2.vrcmaci.s0c
    1 << 8 | 1, // llvm.hexagon.M2.vrcmacr.s0
    1 << 8 | 1, // llvm.hexagon.M2.vrcmacr.s0c
    1 << 8 | 1, // llvm.hexagon.M2.vrcmpyi.s0
    1 << 8 | 1, // llvm.hexagon.M2.vrcmpyi.s0c
    1 << 8 | 1, // llvm.hexagon.M2.vrcmpyr.s0
    1 << 8 | 1, // llvm.hexagon.M2.vrcmpyr.s0c
    1 << 8 | 1, // llvm.hexagon.M2.vrcmpys.acc.s1
    1 << 8 | 1, // llvm.hexagon.M2.vrcmpys.s1
    1 << 8 | 1, // llvm.hexagon.M2.vrcmpys.s1rp
    1 << 8 | 1, // llvm.hexagon.M2.vrmac.s0
    1 << 8 | 1, // llvm.hexagon.M2.vrmpy.s0
    1 << 8 | 1, // llvm.hexagon.M2.xor.xacc
    1 << 8 | 1, // llvm.hexagon.M4.and.and
    1 << 8 | 1, // llvm.hexagon.M4.and.andn
    1 << 8 | 1, // llvm.hexagon.M4.and.or
    1 << 8 | 1, // llvm.hexagon.M4.and.xor
    1 << 8 | 1, // llvm.hexagon.M4.cmpyi.wh
    1 << 8 | 1, // llvm.hexagon.M4.cmpyi.whc
    1 << 8 | 1, // llvm.hexagon.M4.cmpyr.wh
    1 << 8 | 1, // llvm.hexagon.M4.cmpyr.whc
    1 << 8 | 1, // llvm.hexagon.M4.mac.up.s1.sat
    1 << 8 | 46, // llvm.hexagon.M4.mpyri.addi
    1 << 8 | 11, // llvm.hexagon.M4.mpyri.addr
    1 << 8 | 0, // llvm.hexagon.M4.mpyri.addr.u2
    1 << 8 | 5, // llvm.hexagon.M4.mpyrr.addi
    1 << 8 | 1, // llvm.hexagon.M4.mpyrr.addr
    1 << 8 | 1, // llvm.hexagon.M4.nac.up.s1.sat
    1 << 8 | 1, // llvm.hexagon.M4.or.and
    1 << 8 | 1, // llvm.hexagon.M4.or.andn
    1 << 8 | 1, // llvm.hexagon.M4.or.or
    1 << 8 | 1, // llvm.hexagon.M4.or.xor
    1 << 8 | 1, // llvm.hexagon.M4.pmpyw
    1 << 8 | 1, // llvm.hexagon.M4.pmpyw.acc
    1 << 8 | 1, // llvm.hexagon.M4.vpmpyh
    1 << 8 | 1, // llvm.hexagon.M4.vpmpyh.acc
    1 << 8 | 1, // llvm.hexagon.M4.vrmpyeh.acc.s0
    1 << 8 | 1, // llvm.hexagon.M4.vrmpyeh.acc.s1
    1 << 8 | 1, // llvm.hexagon.M4.vrmpyeh.s0
    1 << 8 | 1, // llvm.hexagon.M4.vrmpyeh.s1
    1 << 8 | 1, // llvm.hexagon.M4.vrmpyoh.acc.s0
    1 << 8 | 1, // llvm.hexagon.M4.vrmpyoh.acc.s1
    1 << 8 | 1, // llvm.hexagon.M4.vrmpyoh.s0
    1 << 8 | 1, // llvm.hexagon.M4.vrmpyoh.s1
    1 << 8 | 1, // llvm.hexagon.M4.xor.and
    1 << 8 | 1, // llvm.hexagon.M4.xor.andn
    1 << 8 | 1, // llvm.hexagon.M4.xor.or
    1 << 8 | 1, // llvm.hexagon.M4.xor.xacc
    1 << 8 | 1, // llvm.hexagon.M5.vdmacbsu
    1 << 8 | 1, // llvm.hexagon.M5.vdmpybsu
    1 << 8 | 1, // llvm.hexagon.M5.vmacbsu
    1 << 8 | 1, // llvm.hexagon.M5.vmacbuu
    1 << 8 | 1, // llvm.hexagon.M5.vmpybsu
    1 << 8 | 1, // llvm.hexagon.M5.vmpybuu
    1 << 8 | 1, // llvm.hexagon.M5.vrmacbsu
    1 << 8 | 1, // llvm.hexagon.M5.vrmacbuu
    1 << 8 | 1, // llvm.hexagon.M5.vrmpybsu
    1 << 8 | 1, // llvm.hexagon.M5.vrmpybuu
    1 << 8 | 1, // llvm.hexagon.M6.vabsdiffb
    1 << 8 | 1, // llvm.hexagon.M6.vabsdiffub
    1 << 8 | 1, // llvm.hexagon.M7.dcmpyiw
    1 << 8 | 1, // llvm.hexagon.M7.dcmpyiw.acc
    1 << 8 | 1, // llvm.hexagon.M7.dcmpyiwc
    1 << 8 | 1, // llvm.hexagon.M7.dcmpyiwc.acc
    1 << 8 | 1, // llvm.hexagon.M7.dcmpyrw
    1 << 8 | 1, // llvm.hexagon.M7.dcmpyrw.acc
    1 << 8 | 1, // llvm.hexagon.M7.dcmpyrwc
    1 << 8 | 1, // llvm.hexagon.M7.dcmpyrwc.acc
    1 << 8 | 1, // llvm.hexagon.M7.vdmpy
    1 << 8 | 1, // llvm.hexagon.M7.vdmpy.acc
    1 << 8 | 1, // llvm.hexagon.M7.wcmpyiw
    1 << 8 | 1, // llvm.hexagon.M7.wcmpyiw.rnd
    1 << 8 | 1, // llvm.hexagon.M7.wcmpyiwc
    1 << 8 | 1, // llvm.hexagon.M7.wcmpyiwc.rnd
    1 << 8 | 1, // llvm.hexagon.M7.wcmpyrw
    1 << 8 | 1, // llvm.hexagon.M7.wcmpyrw.rnd
    1 << 8 | 1, // llvm.hexagon.M7.wcmpyrwc
    1 << 8 | 1, // llvm.hexagon.M7.wcmpyrwc.rnd
    1 << 8 | 11, // llvm.hexagon.S2.addasl.rrri
    1 << 8 | 0, // llvm.hexagon.S2.asl.i.p
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.p.acc
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.p.and
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.p.nac
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.p.or
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.p.xacc
    1 << 8 | 0, // llvm.hexagon.S2.asl.i.r
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.r.acc
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.r.and
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.r.nac
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.r.or
    1 << 8 | 0, // llvm.hexagon.S2.asl.i.r.sat
    1 << 8 | 11, // llvm.hexagon.S2.asl.i.r.xacc
    1 << 8 | 0, // llvm.hexagon.S2.asl.i.vh
    1 << 8 | 0, // llvm.hexagon.S2.asl.i.vw
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.p
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.p.acc
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.p.and
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.p.nac
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.p.or
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.p.xor
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.r
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.r.acc
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.r.and
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.r.nac
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.r.or
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.r.sat
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.vh
    1 << 8 | 1, // llvm.hexagon.S2.asl.r.vw
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.p
    1 << 8 | 11, // llvm.hexagon.S2.asr.i.p.acc
    1 << 8 | 11, // llvm.hexagon.S2.asr.i.p.and
    1 << 8 | 11, // llvm.hexagon.S2.asr.i.p.nac
    1 << 8 | 11, // llvm.hexagon.S2.asr.i.p.or
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.p.rnd
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.p.rnd.goodsyntax
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.r
    1 << 8 | 11, // llvm.hexagon.S2.asr.i.r.acc
    1 << 8 | 11, // llvm.hexagon.S2.asr.i.r.and
    1 << 8 | 11, // llvm.hexagon.S2.asr.i.r.nac
    1 << 8 | 11, // llvm.hexagon.S2.asr.i.r.or
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.r.rnd
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.r.rnd.goodsyntax
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.svw.trun
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.vh
    1 << 8 | 0, // llvm.hexagon.S2.asr.i.vw
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.p
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.p.acc
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.p.and
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.p.nac
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.p.or
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.p.xor
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.r
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.r.acc
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.r.and
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.r.nac
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.r.or
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.r.sat
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.svw.trun
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.vh
    1 << 8 | 1, // llvm.hexagon.S2.asr.r.vw
    1 << 8 | 1, // llvm.hexagon.S2.brev
    1 << 8 | 1, // llvm.hexagon.S2.brevp
    1 << 8 | 1, // llvm.hexagon.S2.cl0
    1 << 8 | 1, // llvm.hexagon.S2.cl0p
    1 << 8 | 1, // llvm.hexagon.S2.cl1
    1 << 8 | 1, // llvm.hexagon.S2.cl1p
    1 << 8 | 1, // llvm.hexagon.S2.clb
    1 << 8 | 1, // llvm.hexagon.S2.clbnorm
    1 << 8 | 1, // llvm.hexagon.S2.clbp
    1 << 8 | 0, // llvm.hexagon.S2.clrbit.i
    1 << 8 | 1, // llvm.hexagon.S2.clrbit.r
    1 << 8 | 1, // llvm.hexagon.S2.ct0
    1 << 8 | 1, // llvm.hexagon.S2.ct0p
    1 << 8 | 1, // llvm.hexagon.S2.ct1
    1 << 8 | 1, // llvm.hexagon.S2.ct1p
    1 << 8 | 1, // llvm.hexagon.S2.deinterleave
    1 << 8 | 13, // llvm.hexagon.S2.extractu
    1 << 8 | 1, // llvm.hexagon.S2.extractu.rp
    1 << 8 | 13, // llvm.hexagon.S2.extractup
    1 << 8 | 1, // llvm.hexagon.S2.extractup.rp
    1 << 8 | 93, // llvm.hexagon.S2.insert
    1 << 8 | 1, // llvm.hexagon.S2.insert.rp
    1 << 8 | 93, // llvm.hexagon.S2.insertp
    1 << 8 | 1, // llvm.hexagon.S2.insertp.rp
    1 << 8 | 1, // llvm.hexagon.S2.interleave
    1 << 8 | 1, // llvm.hexagon.S2.lfsp
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.p
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.p.acc
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.p.and
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.p.nac
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.p.or
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.p.xor
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.r
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.r.acc
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.r.and
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.r.nac
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.r.or
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.vh
    1 << 8 | 1, // llvm.hexagon.S2.lsl.r.vw
    1 << 8 | 0, // llvm.hexagon.S2.lsr.i.p
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.p.acc
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.p.and
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.p.nac
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.p.or
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.p.xacc
    1 << 8 | 0, // llvm.hexagon.S2.lsr.i.r
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.r.acc
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.r.and
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.r.nac
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.r.or
    1 << 8 | 11, // llvm.hexagon.S2.lsr.i.r.xacc
    1 << 8 | 0, // llvm.hexagon.S2.lsr.i.vh
    1 << 8 | 0, // llvm.hexagon.S2.lsr.i.vw
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.p
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.p.acc
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.p.and
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.p.nac
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.p.or
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.p.xor
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.r
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.r.acc
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.r.and
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.r.nac
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.r.or
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.vh
    1 << 8 | 1, // llvm.hexagon.S2.lsr.r.vw
    1 << 8 | 17, // llvm.hexagon.S2.mask
    1 << 8 | 1, // llvm.hexagon.S2.packhl
    1 << 8 | 1, // llvm.hexagon.S2.parityp
    1 << 8 | 0, // llvm.hexagon.S2.setbit.i
    1 << 8 | 1, // llvm.hexagon.S2.setbit.r
    1 << 8 | 1, // llvm.hexagon.S2.shuffeb
    1 << 8 | 1, // llvm.hexagon.S2.shuffeh
    1 << 8 | 1, // llvm.hexagon.S2.shuffob
    1 << 8 | 1, // llvm.hexagon.S2.shuffoh
    24 << 8 | 1, // llvm.hexagon.S2.storerb.pbr
    18 << 8 | 39, // llvm.hexagon.S2.storerb.pci
    18 << 8 | 38, // llvm.hexagon.S2.storerb.pcr
    24 << 8 | 1, // llvm.hexagon.S2.storerd.pbr
    18 << 8 | 39, // llvm.hexagon.S2.storerd.pci
    18 << 8 | 38, // llvm.hexagon.S2.storerd.pcr
    24 << 8 | 1, // llvm.hexagon.S2.storerf.pbr
    18 << 8 | 39, // llvm.hexagon.S2.storerf.pci
    18 << 8 | 38, // llvm.hexagon.S2.storerf.pcr
    24 << 8 | 1, // llvm.hexagon.S2.storerh.pbr
    18 << 8 | 39, // llvm.hexagon.S2.storerh.pci
    18 << 8 | 38, // llvm.hexagon.S2.storerh.pcr
    24 << 8 | 1, // llvm.hexagon.S2.storeri.pbr
    18 << 8 | 39, // llvm.hexagon.S2.storeri.pci
    18 << 8 | 38, // llvm.hexagon.S2.storeri.pcr
    11 << 8 | 10, // llvm.hexagon.S2.storew.locked
    1 << 8 | 1, // llvm.hexagon.S2.svsathb
    1 << 8 | 1, // llvm.hexagon.S2.svsathub
    1 << 8 | 93, // llvm.hexagon.S2.tableidxb.goodsyntax
    1 << 8 | 93, // llvm.hexagon.S2.tableidxd.goodsyntax
    1 << 8 | 93, // llvm.hexagon.S2.tableidxh.goodsyntax
    1 << 8 | 93, // llvm.hexagon.S2.tableidxw.goodsyntax
    1 << 8 | 0, // llvm.hexagon.S2.togglebit.i
    1 << 8 | 1, // llvm.hexagon.S2.togglebit.r
    1 << 8 | 0, // llvm.hexagon.S2.tstbit.i
    1 << 8 | 1, // llvm.hexagon.S2.tstbit.r
    1 << 8 | 11, // llvm.hexagon.S2.valignib
    1 << 8 | 1, // llvm.hexagon.S2.valignrb
    1 << 8 | 1, // llvm.hexagon.S2.vcnegh
    1 << 8 | 1, // llvm.hexagon.S2.vcrotate
    1 << 8 | 1, // llvm.hexagon.S2.vrcnegh
    1 << 8 | 1, // llvm.hexagon.S2.vrndpackwh
    1 << 8 | 1, // llvm.hexagon.S2.vrndpackwhs
    1 << 8 | 1, // llvm.hexagon.S2.vsathb
    1 << 8 | 1, // llvm.hexagon.S2.vsathb.nopack
    1 << 8 | 1, // llvm.hexagon.S2.vsathub
    1 << 8 | 1, // llvm.hexagon.S2.vsathub.nopack
    1 << 8 | 1, // llvm.hexagon.S2.vsatwh
    1 << 8 | 1, // llvm.hexagon.S2.vsatwh.nopack
    1 << 8 | 1, // llvm.hexagon.S2.vsatwuh
    1 << 8 | 1, // llvm.hexagon.S2.vsatwuh.nopack
    1 << 8 | 1, // llvm.hexagon.S2.vsplatrb
    1 << 8 | 1, // llvm.hexagon.S2.vsplatrh
    1 << 8 | 11, // llvm.hexagon.S2.vspliceib
    1 << 8 | 1, // llvm.hexagon.S2.vsplicerb
    1 << 8 | 1, // llvm.hexagon.S2.vsxtbh
    1 << 8 | 1, // llvm.hexagon.S2.vsxthw
    1 << 8 | 1, // llvm.hexagon.S2.vtrunehb
    1 << 8 | 1, // llvm.hexagon.S2.vtrunewh
    1 << 8 | 1, // llvm.hexagon.S2.vtrunohb
    1 << 8 | 1, // llvm.hexagon.S2.vtrunowh
    1 << 8 | 1, // llvm.hexagon.S2.vzxtbh
    1 << 8 | 1, // llvm.hexagon.S2.vzxthw
    1 << 8 | 11, // llvm.hexagon.S4.addaddi
    1 << 8 | 46, // llvm.hexagon.S4.addi.asl.ri
    1 << 8 | 46, // llvm.hexagon.S4.addi.lsr.ri
    1 << 8 | 46, // llvm.hexagon.S4.andi.asl.ri
    1 << 8 | 46, // llvm.hexagon.S4.andi.lsr.ri
    1 << 8 | 0, // llvm.hexagon.S4.clbaddi
    1 << 8 | 0, // llvm.hexagon.S4.clbpaddi
    1 << 8 | 1, // llvm.hexagon.S4.clbpnorm
    1 << 8 | 13, // llvm.hexagon.S4.extract
    1 << 8 | 1, // llvm.hexagon.S4.extract.rp
    1 << 8 | 13, // llvm.hexagon.S4.extractp
    1 << 8 | 1, // llvm.hexagon.S4.extractp.rp
    1 << 8 | 5, // llvm.hexagon.S4.lsli
    1 << 8 | 0, // llvm.hexagon.S4.ntstbit.i
    1 << 8 | 1, // llvm.hexagon.S4.ntstbit.r
    1 << 8 | 11, // llvm.hexagon.S4.or.andi
    1 << 8 | 11, // llvm.hexagon.S4.or.andix
    1 << 8 | 11, // llvm.hexagon.S4.or.ori
    1 << 8 | 46, // llvm.hexagon.S4.ori.asl.ri
    1 << 8 | 46, // llvm.hexagon.S4.ori.lsr.ri
    1 << 8 | 1, // llvm.hexagon.S4.parity
    11 << 8 | 10, // llvm.hexagon.S4.stored.locked
    1 << 8 | 0, // llvm.hexagon.S4.subaddi
    1 << 8 | 46, // llvm.hexagon.S4.subi.asl.ri
    1 << 8 | 46, // llvm.hexagon.S4.subi.lsr.ri
    1 << 8 | 11, // llvm.hexagon.S4.vrcrotate
    1 << 8 | 35, // llvm.hexagon.S4.vrcrotate.acc
    1 << 8 | 1, // llvm.hexagon.S4.vxaddsubh
    1 << 8 | 1, // llvm.hexagon.S4.vxaddsubhr
    1 << 8 | 1, // llvm.hexagon.S4.vxaddsubw
    1 << 8 | 1, // llvm.hexagon.S4.vxsubaddh
    1 << 8 | 1, // llvm.hexagon.S4.vxsubaddhr
    1 << 8 | 1, // llvm.hexagon.S4.vxsubaddw
    1 << 8 | 0, // llvm.hexagon.S5.asrhub.rnd.sat.goodsyntax
    1 << 8 | 0, // llvm.hexagon.S5.asrhub.sat
    1 << 8 | 1, // llvm.hexagon.S5.popcountp
    1 << 8 | 0, // llvm.hexagon.S5.vasrhrnd.goodsyntax
    1 << 8 | 0, // llvm.hexagon.S6.rol.i.p
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.p.acc
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.p.and
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.p.nac
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.p.or
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.p.xacc
    1 << 8 | 0, // llvm.hexagon.S6.rol.i.r
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.r.acc
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.r.and
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.r.nac
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.r.or
    1 << 8 | 11, // llvm.hexagon.S6.rol.i.r.xacc
    1 << 8 | 1, // llvm.hexagon.S6.vsplatrbp
    1 << 8 | 1, // llvm.hexagon.S6.vtrunehb.ppp
    1 << 8 | 1, // llvm.hexagon.S6.vtrunohb.ppp
    1 << 8 | 1, // llvm.hexagon.V6.extractw
    1 << 8 | 1, // llvm.hexagon.V6.extractw.128B
    1 << 8 | 1, // llvm.hexagon.V6.get.qfext
    1 << 8 | 1, // llvm.hexagon.V6.get.qfext.128B
    1 << 8 | 1, // llvm.hexagon.V6.get.qfext.oracc
    1 << 8 | 1, // llvm.hexagon.V6.get.qfext.oracc.128B
    1 << 8 | 1, // llvm.hexagon.V6.hi
    1 << 8 | 1, // llvm.hexagon.V6.hi.128B
    1 << 8 | 1, // llvm.hexagon.V6.lo
    1 << 8 | 1, // llvm.hexagon.V6.lo.128B
    1 << 8 | 1, // llvm.hexagon.V6.lvsplatb
    1 << 8 | 1, // llvm.hexagon.V6.lvsplatb.128B
    1 << 8 | 1, // llvm.hexagon.V6.lvsplath
    1 << 8 | 1, // llvm.hexagon.V6.lvsplath.128B
    1 << 8 | 1, // llvm.hexagon.V6.lvsplatw
    1 << 8 | 1, // llvm.hexagon.V6.lvsplatw.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.and
    1 << 8 | 1, // llvm.hexagon.V6.pred.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.and.n
    1 << 8 | 1, // llvm.hexagon.V6.pred.and.n.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.not
    1 << 8 | 1, // llvm.hexagon.V6.pred.not.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.or
    1 << 8 | 1, // llvm.hexagon.V6.pred.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.or.n
    1 << 8 | 1, // llvm.hexagon.V6.pred.or.n.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.scalar2
    1 << 8 | 1, // llvm.hexagon.V6.pred.scalar2.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.scalar2v2
    1 << 8 | 1, // llvm.hexagon.V6.pred.scalar2v2.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.typecast
    1 << 8 | 1, // llvm.hexagon.V6.pred.typecast.128B
    1 << 8 | 1, // llvm.hexagon.V6.pred.xor
    1 << 8 | 1, // llvm.hexagon.V6.pred.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.set.qfext
    1 << 8 | 1, // llvm.hexagon.V6.set.qfext.128B
    1 << 8 | 1, // llvm.hexagon.V6.shuffeqh
    1 << 8 | 1, // llvm.hexagon.V6.shuffeqh.128B
    1 << 8 | 1, // llvm.hexagon.V6.shuffeqw
    1 << 8 | 1, // llvm.hexagon.V6.shuffeqw.128B
    1 << 8 | 11, // llvm.hexagon.V6.v6mpyhubs10
    1 << 8 | 11, // llvm.hexagon.V6.v6mpyhubs10.128B
    1 << 8 | 35, // llvm.hexagon.V6.v6mpyhubs10.vxx
    1 << 8 | 35, // llvm.hexagon.V6.v6mpyhubs10.vxx.128B
    1 << 8 | 11, // llvm.hexagon.V6.v6mpyvubs10
    1 << 8 | 11, // llvm.hexagon.V6.v6mpyvubs10.128B
    1 << 8 | 35, // llvm.hexagon.V6.v6mpyvubs10.vxx
    1 << 8 | 35, // llvm.hexagon.V6.v6mpyvubs10.vxx.128B
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.npred.ai
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.npred.ai.128B
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.npred.pi
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.npred.pi.128B
    2 << 8 | 18, // llvm.hexagon.V6.vL32b.npred.ppu
    2 << 8 | 18, // llvm.hexagon.V6.vL32b.npred.ppu.128B
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.nt.npred.ai
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.nt.npred.ai.128B
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.nt.npred.pi
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.nt.npred.pi.128B
    2 << 8 | 18, // llvm.hexagon.V6.vL32b.nt.npred.ppu
    2 << 8 | 18, // llvm.hexagon.V6.vL32b.nt.npred.ppu.128B
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.nt.pred.ai
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.nt.pred.ai.128B
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.nt.pred.pi
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.nt.pred.pi.128B
    2 << 8 | 18, // llvm.hexagon.V6.vL32b.nt.pred.ppu
    2 << 8 | 18, // llvm.hexagon.V6.vL32b.nt.pred.ppu.128B
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.pred.ai
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.pred.ai.128B
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.pred.pi
    2 << 8 | 137, // llvm.hexagon.V6.vL32b.pred.pi.128B
    2 << 8 | 18, // llvm.hexagon.V6.vL32b.pred.ppu
    2 << 8 | 18, // llvm.hexagon.V6.vL32b.pred.ppu.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32Ub.npred.ai
    19 << 8 | 137, // llvm.hexagon.V6.vS32Ub.npred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32Ub.npred.pi
    19 << 8 | 137, // llvm.hexagon.V6.vS32Ub.npred.pi.128B
    19 << 8 | 18, // llvm.hexagon.V6.vS32Ub.npred.ppu
    19 << 8 | 18, // llvm.hexagon.V6.vS32Ub.npred.ppu.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32Ub.pred.ai
    19 << 8 | 137, // llvm.hexagon.V6.vS32Ub.pred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32Ub.pred.pi
    19 << 8 | 137, // llvm.hexagon.V6.vS32Ub.pred.pi.128B
    19 << 8 | 18, // llvm.hexagon.V6.vS32Ub.pred.ppu
    19 << 8 | 18, // llvm.hexagon.V6.vS32Ub.pred.ppu.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.npred.ai
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.npred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.npred.pi
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.npred.pi.128B
    19 << 8 | 18, // llvm.hexagon.V6.vS32b.npred.ppu
    19 << 8 | 18, // llvm.hexagon.V6.vS32b.npred.ppu.128B
    24 << 8 | 1, // llvm.hexagon.V6.vS32b.nqpred.ai
    24 << 8 | 1, // llvm.hexagon.V6.vS32b.nqpred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.nt.npred.ai
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.nt.npred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.nt.npred.pi
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.nt.npred.pi.128B
    19 << 8 | 18, // llvm.hexagon.V6.vS32b.nt.npred.ppu
    19 << 8 | 18, // llvm.hexagon.V6.vS32b.nt.npred.ppu.128B
    24 << 8 | 1, // llvm.hexagon.V6.vS32b.nt.nqpred.ai
    24 << 8 | 1, // llvm.hexagon.V6.vS32b.nt.nqpred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.nt.pred.ai
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.nt.pred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.nt.pred.pi
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.nt.pred.pi.128B
    19 << 8 | 18, // llvm.hexagon.V6.vS32b.nt.pred.ppu
    19 << 8 | 18, // llvm.hexagon.V6.vS32b.nt.pred.ppu.128B
    24 << 8 | 1, // llvm.hexagon.V6.vS32b.nt.qpred.ai
    24 << 8 | 1, // llvm.hexagon.V6.vS32b.nt.qpred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.pred.ai
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.pred.ai.128B
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.pred.pi
    19 << 8 | 137, // llvm.hexagon.V6.vS32b.pred.pi.128B
    19 << 8 | 18, // llvm.hexagon.V6.vS32b.pred.ppu
    19 << 8 | 18, // llvm.hexagon.V6.vS32b.pred.ppu.128B
    24 << 8 | 1, // llvm.hexagon.V6.vS32b.qpred.ai
    24 << 8 | 1, // llvm.hexagon.V6.vS32b.qpred.ai.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabs.f8
    1 << 8 | 1, // llvm.hexagon.V6.vabs.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabs.hf
    1 << 8 | 1, // llvm.hexagon.V6.vabs.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabs.qf16.hf
    1 << 8 | 1, // llvm.hexagon.V6.vabs.qf16.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabs.qf16.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vabs.qf16.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabs.qf32.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vabs.qf32.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabs.qf32.sf
    1 << 8 | 1, // llvm.hexagon.V6.vabs.qf32.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabs.sf
    1 << 8 | 1, // llvm.hexagon.V6.vabs.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsb
    1 << 8 | 1, // llvm.hexagon.V6.vabsb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsb.sat
    1 << 8 | 1, // llvm.hexagon.V6.vabsb.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsdiffh
    1 << 8 | 1, // llvm.hexagon.V6.vabsdiffh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsdiffub
    1 << 8 | 1, // llvm.hexagon.V6.vabsdiffub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsdiffuh
    1 << 8 | 1, // llvm.hexagon.V6.vabsdiffuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsdiffw
    1 << 8 | 1, // llvm.hexagon.V6.vabsdiffw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsh
    1 << 8 | 1, // llvm.hexagon.V6.vabsh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsh.sat
    1 << 8 | 1, // llvm.hexagon.V6.vabsh.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsw
    1 << 8 | 1, // llvm.hexagon.V6.vabsw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vabsw.sat
    1 << 8 | 1, // llvm.hexagon.V6.vabsw.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.hf
    1 << 8 | 1, // llvm.hexagon.V6.vadd.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.hf.f8
    1 << 8 | 1, // llvm.hexagon.V6.vadd.hf.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.hf.hf
    1 << 8 | 1, // llvm.hexagon.V6.vadd.hf.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vadd.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.qf16.mix
    1 << 8 | 1, // llvm.hexagon.V6.vadd.qf16.mix.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vadd.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.qf32.mix
    1 << 8 | 1, // llvm.hexagon.V6.vadd.qf32.mix.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.sf
    1 << 8 | 1, // llvm.hexagon.V6.vadd.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.sf.bf
    1 << 8 | 1, // llvm.hexagon.V6.vadd.sf.bf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.sf.hf
    1 << 8 | 1, // llvm.hexagon.V6.vadd.sf.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadd.sf.sf
    1 << 8 | 1, // llvm.hexagon.V6.vadd.sf.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddb
    1 << 8 | 1, // llvm.hexagon.V6.vaddb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddb.dv
    1 << 8 | 1, // llvm.hexagon.V6.vaddb.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddbnq
    1 << 8 | 1, // llvm.hexagon.V6.vaddbnq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddbq
    1 << 8 | 1, // llvm.hexagon.V6.vaddbq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddbsat
    1 << 8 | 1, // llvm.hexagon.V6.vaddbsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddbsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vaddbsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddcarry
    1 << 8 | 1, // llvm.hexagon.V6.vaddcarry.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddcarryo
    1 << 8 | 1, // llvm.hexagon.V6.vaddcarryo.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddcarrysat
    1 << 8 | 1, // llvm.hexagon.V6.vaddcarrysat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddclbh
    1 << 8 | 1, // llvm.hexagon.V6.vaddclbh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddclbw
    1 << 8 | 1, // llvm.hexagon.V6.vaddclbw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddh
    1 << 8 | 1, // llvm.hexagon.V6.vaddh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddh.dv
    1 << 8 | 1, // llvm.hexagon.V6.vaddh.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddhnq
    1 << 8 | 1, // llvm.hexagon.V6.vaddhnq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddhq
    1 << 8 | 1, // llvm.hexagon.V6.vaddhq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddhsat
    1 << 8 | 1, // llvm.hexagon.V6.vaddhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddhsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vaddhsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddhw
    1 << 8 | 1, // llvm.hexagon.V6.vaddhw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddhw.acc
    1 << 8 | 1, // llvm.hexagon.V6.vaddhw.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddubh
    1 << 8 | 1, // llvm.hexagon.V6.vaddubh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddubh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vaddubh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddubsat
    1 << 8 | 1, // llvm.hexagon.V6.vaddubsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddubsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vaddubsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddububb.sat
    1 << 8 | 1, // llvm.hexagon.V6.vaddububb.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadduhsat
    1 << 8 | 1, // llvm.hexagon.V6.vadduhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadduhsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vadduhsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadduhw
    1 << 8 | 1, // llvm.hexagon.V6.vadduhw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadduhw.acc
    1 << 8 | 1, // llvm.hexagon.V6.vadduhw.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadduwsat
    1 << 8 | 1, // llvm.hexagon.V6.vadduwsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vadduwsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vadduwsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddw
    1 << 8 | 1, // llvm.hexagon.V6.vaddw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddw.dv
    1 << 8 | 1, // llvm.hexagon.V6.vaddw.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddwnq
    1 << 8 | 1, // llvm.hexagon.V6.vaddwnq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddwq
    1 << 8 | 1, // llvm.hexagon.V6.vaddwq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddwsat
    1 << 8 | 1, // llvm.hexagon.V6.vaddwsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaddwsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vaddwsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.valign4
    1 << 8 | 1, // llvm.hexagon.V6.valign4.128B
    1 << 8 | 1, // llvm.hexagon.V6.valignb
    1 << 8 | 1, // llvm.hexagon.V6.valignb.128B
    1 << 8 | 11, // llvm.hexagon.V6.valignbi
    1 << 8 | 11, // llvm.hexagon.V6.valignbi.128B
    1 << 8 | 1, // llvm.hexagon.V6.vand
    1 << 8 | 1, // llvm.hexagon.V6.vand.128B
    1 << 8 | 1, // llvm.hexagon.V6.vandnqrt
    1 << 8 | 1, // llvm.hexagon.V6.vandnqrt.128B
    1 << 8 | 1, // llvm.hexagon.V6.vandnqrt.acc
    1 << 8 | 1, // llvm.hexagon.V6.vandnqrt.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vandqrt
    1 << 8 | 1, // llvm.hexagon.V6.vandqrt.128B
    1 << 8 | 1, // llvm.hexagon.V6.vandqrt.acc
    1 << 8 | 1, // llvm.hexagon.V6.vandqrt.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vandvnqv
    1 << 8 | 1, // llvm.hexagon.V6.vandvnqv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vandvqv
    1 << 8 | 1, // llvm.hexagon.V6.vandvqv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vandvrt
    1 << 8 | 1, // llvm.hexagon.V6.vandvrt.128B
    1 << 8 | 1, // llvm.hexagon.V6.vandvrt.acc
    1 << 8 | 1, // llvm.hexagon.V6.vandvrt.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaslh
    1 << 8 | 1, // llvm.hexagon.V6.vaslh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaslh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vaslh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaslhv
    1 << 8 | 1, // llvm.hexagon.V6.vaslhv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaslw
    1 << 8 | 1, // llvm.hexagon.V6.vaslw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaslw.acc
    1 << 8 | 1, // llvm.hexagon.V6.vaslw.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vaslwv
    1 << 8 | 1, // llvm.hexagon.V6.vaslwv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasr.into
    1 << 8 | 1, // llvm.hexagon.V6.vasr.into.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrh
    1 << 8 | 1, // llvm.hexagon.V6.vasrh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vasrh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrhbrndsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrhbrndsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrhbsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrhbsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrhubrndsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrhubrndsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrhubsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrhubsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrhv
    1 << 8 | 1, // llvm.hexagon.V6.vasrhv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasruhubrndsat
    1 << 8 | 1, // llvm.hexagon.V6.vasruhubrndsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasruhubsat
    1 << 8 | 1, // llvm.hexagon.V6.vasruhubsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasruwuhrndsat
    1 << 8 | 1, // llvm.hexagon.V6.vasruwuhrndsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasruwuhsat
    1 << 8 | 1, // llvm.hexagon.V6.vasruwuhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrvuhubrndsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrvuhubrndsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrvuhubsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrvuhubsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrvwuhrndsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrvwuhrndsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrvwuhsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrvwuhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrw
    1 << 8 | 1, // llvm.hexagon.V6.vasrw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrw.acc
    1 << 8 | 1, // llvm.hexagon.V6.vasrw.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrwh
    1 << 8 | 1, // llvm.hexagon.V6.vasrwh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrwhrndsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrwhrndsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrwhsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrwhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrwuhrndsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrwuhrndsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrwuhsat
    1 << 8 | 1, // llvm.hexagon.V6.vasrwuhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vasrwv
    1 << 8 | 1, // llvm.hexagon.V6.vasrwv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vassign
    1 << 8 | 1, // llvm.hexagon.V6.vassign.128B
    1 << 8 | 1, // llvm.hexagon.V6.vassign.fp
    1 << 8 | 1, // llvm.hexagon.V6.vassign.fp.128B
    1 << 8 | 1, // llvm.hexagon.V6.vassignp
    1 << 8 | 1, // llvm.hexagon.V6.vassignp.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavgb
    1 << 8 | 1, // llvm.hexagon.V6.vavgb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavgbrnd
    1 << 8 | 1, // llvm.hexagon.V6.vavgbrnd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavgh
    1 << 8 | 1, // llvm.hexagon.V6.vavgh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavghrnd
    1 << 8 | 1, // llvm.hexagon.V6.vavghrnd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavgub
    1 << 8 | 1, // llvm.hexagon.V6.vavgub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavgubrnd
    1 << 8 | 1, // llvm.hexagon.V6.vavgubrnd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavguh
    1 << 8 | 1, // llvm.hexagon.V6.vavguh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavguhrnd
    1 << 8 | 1, // llvm.hexagon.V6.vavguhrnd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavguw
    1 << 8 | 1, // llvm.hexagon.V6.vavguw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavguwrnd
    1 << 8 | 1, // llvm.hexagon.V6.vavguwrnd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavgw
    1 << 8 | 1, // llvm.hexagon.V6.vavgw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vavgwrnd
    1 << 8 | 1, // llvm.hexagon.V6.vavgwrnd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcl0h
    1 << 8 | 1, // llvm.hexagon.V6.vcl0h.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcl0w
    1 << 8 | 1, // llvm.hexagon.V6.vcl0w.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcombine
    1 << 8 | 1, // llvm.hexagon.V6.vcombine.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.bf.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vconv.bf.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.f8.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vconv.f8.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.h.hf
    1 << 8 | 1, // llvm.hexagon.V6.vconv.h.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.h.hf.rnd
    1 << 8 | 1, // llvm.hexagon.V6.vconv.h.hf.rnd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.hf.h
    1 << 8 | 1, // llvm.hexagon.V6.vconv.hf.h.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.hf.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vconv.hf.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.hf.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vconv.hf.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf16.f8
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf16.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf16.hf
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf16.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf16.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf16.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf32.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf32.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf32.sf
    1 << 8 | 1, // llvm.hexagon.V6.vconv.qf32.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.sf.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vconv.sf.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.sf.w
    1 << 8 | 1, // llvm.hexagon.V6.vconv.sf.w.128B
    1 << 8 | 1, // llvm.hexagon.V6.vconv.w.sf
    1 << 8 | 1, // llvm.hexagon.V6.vconv.w.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.b.hf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.b.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.bf.sf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.bf.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.f8.hf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.f8.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.h.hf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.h.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.b
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.b.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.f8
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.h
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.h.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.sf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.ub
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.ub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.uh
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.hf.uh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.sf.hf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.sf.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.ub.hf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.ub.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.uh.hf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt.uh.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt2.b.hf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt2.b.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt2.hf.b
    1 << 8 | 1, // llvm.hexagon.V6.vcvt2.hf.b.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt2.hf.ub
    1 << 8 | 1, // llvm.hexagon.V6.vcvt2.hf.ub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vcvt2.ub.hf
    1 << 8 | 1, // llvm.hexagon.V6.vcvt2.ub.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vd0
    1 << 8 | 1, // llvm.hexagon.V6.vd0.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdd0
    1 << 8 | 1, // llvm.hexagon.V6.vdd0.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdealb
    1 << 8 | 1, // llvm.hexagon.V6.vdealb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdealb4w
    1 << 8 | 1, // llvm.hexagon.V6.vdealb4w.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdealh
    1 << 8 | 1, // llvm.hexagon.V6.vdealh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdealvdd
    1 << 8 | 1, // llvm.hexagon.V6.vdealvdd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdelta
    1 << 8 | 1, // llvm.hexagon.V6.vdelta.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpy.sf.hf
    1 << 8 | 1, // llvm.hexagon.V6.vdmpy.sf.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpy.sf.hf.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpy.sf.hf.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpybus
    1 << 8 | 1, // llvm.hexagon.V6.vdmpybus.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpybus.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpybus.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpybus.dv
    1 << 8 | 1, // llvm.hexagon.V6.vdmpybus.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpybus.dv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpybus.dv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhb
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhb.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhb.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhb.dv
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhb.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhb.dv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhb.dv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhisat
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhisat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhisat.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhisat.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsat
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsat.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsat.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsuisat
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsuisat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsuisat.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsuisat.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsusat
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsusat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsusat.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhsusat.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhvsat
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhvsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhvsat.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdmpyhvsat.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdsaduh
    1 << 8 | 1, // llvm.hexagon.V6.vdsaduh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vdsaduh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vdsaduh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqb
    1 << 8 | 1, // llvm.hexagon.V6.veqb.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqb.and
    1 << 8 | 1, // llvm.hexagon.V6.veqb.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqb.or
    1 << 8 | 1, // llvm.hexagon.V6.veqb.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqb.xor
    1 << 8 | 1, // llvm.hexagon.V6.veqb.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqh
    1 << 8 | 1, // llvm.hexagon.V6.veqh.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqh.and
    1 << 8 | 1, // llvm.hexagon.V6.veqh.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqh.or
    1 << 8 | 1, // llvm.hexagon.V6.veqh.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqh.xor
    1 << 8 | 1, // llvm.hexagon.V6.veqh.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqhf
    1 << 8 | 1, // llvm.hexagon.V6.veqhf.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqhf.and
    1 << 8 | 1, // llvm.hexagon.V6.veqhf.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqhf.or
    1 << 8 | 1, // llvm.hexagon.V6.veqhf.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqhf.xor
    1 << 8 | 1, // llvm.hexagon.V6.veqhf.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqsf
    1 << 8 | 1, // llvm.hexagon.V6.veqsf.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqsf.and
    1 << 8 | 1, // llvm.hexagon.V6.veqsf.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqsf.or
    1 << 8 | 1, // llvm.hexagon.V6.veqsf.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqsf.xor
    1 << 8 | 1, // llvm.hexagon.V6.veqsf.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqw
    1 << 8 | 1, // llvm.hexagon.V6.veqw.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqw.and
    1 << 8 | 1, // llvm.hexagon.V6.veqw.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqw.or
    1 << 8 | 1, // llvm.hexagon.V6.veqw.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.veqw.xor
    1 << 8 | 1, // llvm.hexagon.V6.veqw.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfmax.f8
    1 << 8 | 1, // llvm.hexagon.V6.vfmax.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfmax.hf
    1 << 8 | 1, // llvm.hexagon.V6.vfmax.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfmax.sf
    1 << 8 | 1, // llvm.hexagon.V6.vfmax.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfmin.f8
    1 << 8 | 1, // llvm.hexagon.V6.vfmin.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfmin.hf
    1 << 8 | 1, // llvm.hexagon.V6.vfmin.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfmin.sf
    1 << 8 | 1, // llvm.hexagon.V6.vfmin.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfneg.f8
    1 << 8 | 1, // llvm.hexagon.V6.vfneg.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfneg.hf
    1 << 8 | 1, // llvm.hexagon.V6.vfneg.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vfneg.sf
    1 << 8 | 1, // llvm.hexagon.V6.vfneg.sf.128B
    18 << 8 | 1, // llvm.hexagon.V6.vgather.vscattermh
    18 << 8 | 1, // llvm.hexagon.V6.vgather.vscattermh.128B
    18 << 8 | 1, // llvm.hexagon.V6.vgathermh
    18 << 8 | 1, // llvm.hexagon.V6.vgathermh.128B
    18 << 8 | 1, // llvm.hexagon.V6.vgathermhq
    18 << 8 | 1, // llvm.hexagon.V6.vgathermhq.128B
    18 << 8 | 1, // llvm.hexagon.V6.vgathermhw
    18 << 8 | 1, // llvm.hexagon.V6.vgathermhw.128B
    18 << 8 | 1, // llvm.hexagon.V6.vgathermhwq
    18 << 8 | 1, // llvm.hexagon.V6.vgathermhwq.128B
    18 << 8 | 1, // llvm.hexagon.V6.vgathermw
    18 << 8 | 1, // llvm.hexagon.V6.vgathermw.128B
    18 << 8 | 1, // llvm.hexagon.V6.vgathermwq
    18 << 8 | 1, // llvm.hexagon.V6.vgathermwq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtb
    1 << 8 | 1, // llvm.hexagon.V6.vgtb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtb.and
    1 << 8 | 1, // llvm.hexagon.V6.vgtb.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtb.or
    1 << 8 | 1, // llvm.hexagon.V6.vgtb.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtb.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgtb.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtbf
    1 << 8 | 1, // llvm.hexagon.V6.vgtbf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtbf.and
    1 << 8 | 1, // llvm.hexagon.V6.vgtbf.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtbf.or
    1 << 8 | 1, // llvm.hexagon.V6.vgtbf.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtbf.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgtbf.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgth
    1 << 8 | 1, // llvm.hexagon.V6.vgth.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgth.and
    1 << 8 | 1, // llvm.hexagon.V6.vgth.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgth.or
    1 << 8 | 1, // llvm.hexagon.V6.vgth.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgth.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgth.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgthf
    1 << 8 | 1, // llvm.hexagon.V6.vgthf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgthf.and
    1 << 8 | 1, // llvm.hexagon.V6.vgthf.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgthf.or
    1 << 8 | 1, // llvm.hexagon.V6.vgthf.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgthf.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgthf.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtsf
    1 << 8 | 1, // llvm.hexagon.V6.vgtsf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtsf.and
    1 << 8 | 1, // llvm.hexagon.V6.vgtsf.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtsf.or
    1 << 8 | 1, // llvm.hexagon.V6.vgtsf.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtsf.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgtsf.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtub
    1 << 8 | 1, // llvm.hexagon.V6.vgtub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtub.and
    1 << 8 | 1, // llvm.hexagon.V6.vgtub.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtub.or
    1 << 8 | 1, // llvm.hexagon.V6.vgtub.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtub.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgtub.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtuh
    1 << 8 | 1, // llvm.hexagon.V6.vgtuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtuh.and
    1 << 8 | 1, // llvm.hexagon.V6.vgtuh.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtuh.or
    1 << 8 | 1, // llvm.hexagon.V6.vgtuh.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtuh.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgtuh.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtuw
    1 << 8 | 1, // llvm.hexagon.V6.vgtuw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtuw.and
    1 << 8 | 1, // llvm.hexagon.V6.vgtuw.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtuw.or
    1 << 8 | 1, // llvm.hexagon.V6.vgtuw.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtuw.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgtuw.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtw
    1 << 8 | 1, // llvm.hexagon.V6.vgtw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtw.and
    1 << 8 | 1, // llvm.hexagon.V6.vgtw.and.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtw.or
    1 << 8 | 1, // llvm.hexagon.V6.vgtw.or.128B
    1 << 8 | 1, // llvm.hexagon.V6.vgtw.xor
    1 << 8 | 1, // llvm.hexagon.V6.vgtw.xor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vilog2.hf
    1 << 8 | 1, // llvm.hexagon.V6.vilog2.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vilog2.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vilog2.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vilog2.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vilog2.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vilog2.sf
    1 << 8 | 1, // llvm.hexagon.V6.vilog2.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vinsertwr
    1 << 8 | 1, // llvm.hexagon.V6.vinsertwr.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlalignb
    1 << 8 | 1, // llvm.hexagon.V6.vlalignb.128B
    1 << 8 | 11, // llvm.hexagon.V6.vlalignbi
    1 << 8 | 11, // llvm.hexagon.V6.vlalignbi.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlsrb
    1 << 8 | 1, // llvm.hexagon.V6.vlsrb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlsrh
    1 << 8 | 1, // llvm.hexagon.V6.vlsrh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlsrhv
    1 << 8 | 1, // llvm.hexagon.V6.vlsrhv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlsrw
    1 << 8 | 1, // llvm.hexagon.V6.vlsrw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlsrwv
    1 << 8 | 1, // llvm.hexagon.V6.vlsrwv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlut4
    1 << 8 | 1, // llvm.hexagon.V6.vlut4.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlutvvb
    1 << 8 | 1, // llvm.hexagon.V6.vlutvvb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlutvvb.nm
    1 << 8 | 1, // llvm.hexagon.V6.vlutvvb.nm.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlutvvb.oracc
    1 << 8 | 1, // llvm.hexagon.V6.vlutvvb.oracc.128B
    1 << 8 | 35, // llvm.hexagon.V6.vlutvvb.oracci
    1 << 8 | 35, // llvm.hexagon.V6.vlutvvb.oracci.128B
    1 << 8 | 11, // llvm.hexagon.V6.vlutvvbi
    1 << 8 | 11, // llvm.hexagon.V6.vlutvvbi.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlutvwh
    1 << 8 | 1, // llvm.hexagon.V6.vlutvwh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlutvwh.nm
    1 << 8 | 1, // llvm.hexagon.V6.vlutvwh.nm.128B
    1 << 8 | 1, // llvm.hexagon.V6.vlutvwh.oracc
    1 << 8 | 1, // llvm.hexagon.V6.vlutvwh.oracc.128B
    1 << 8 | 35, // llvm.hexagon.V6.vlutvwh.oracci
    1 << 8 | 35, // llvm.hexagon.V6.vlutvwh.oracci.128B
    1 << 8 | 11, // llvm.hexagon.V6.vlutvwhi
    1 << 8 | 11, // llvm.hexagon.V6.vlutvwhi.128B
    24 << 8 | 1, // llvm.hexagon.V6.vmaskedstorenq
    24 << 8 | 1, // llvm.hexagon.V6.vmaskedstorenq.128B
    24 << 8 | 1, // llvm.hexagon.V6.vmaskedstorentnq
    24 << 8 | 1, // llvm.hexagon.V6.vmaskedstorentnq.128B
    24 << 8 | 1, // llvm.hexagon.V6.vmaskedstorentq
    24 << 8 | 1, // llvm.hexagon.V6.vmaskedstorentq.128B
    24 << 8 | 1, // llvm.hexagon.V6.vmaskedstoreq
    24 << 8 | 1, // llvm.hexagon.V6.vmaskedstoreq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmax.bf
    1 << 8 | 1, // llvm.hexagon.V6.vmax.bf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmax.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmax.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmax.sf
    1 << 8 | 1, // llvm.hexagon.V6.vmax.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmaxb
    1 << 8 | 1, // llvm.hexagon.V6.vmaxb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmaxh
    1 << 8 | 1, // llvm.hexagon.V6.vmaxh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmaxub
    1 << 8 | 1, // llvm.hexagon.V6.vmaxub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmaxuh
    1 << 8 | 1, // llvm.hexagon.V6.vmaxuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmaxw
    1 << 8 | 1, // llvm.hexagon.V6.vmaxw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmerge.qf
    1 << 8 | 1, // llvm.hexagon.V6.vmerge.qf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmin.bf
    1 << 8 | 1, // llvm.hexagon.V6.vmin.bf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmin.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmin.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmin.sf
    1 << 8 | 1, // llvm.hexagon.V6.vmin.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vminb
    1 << 8 | 1, // llvm.hexagon.V6.vminb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vminh
    1 << 8 | 1, // llvm.hexagon.V6.vminh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vminub
    1 << 8 | 1, // llvm.hexagon.V6.vminub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vminuh
    1 << 8 | 1, // llvm.hexagon.V6.vminuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vminw
    1 << 8 | 1, // llvm.hexagon.V6.vminw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpabus
    1 << 8 | 1, // llvm.hexagon.V6.vmpabus.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpabus.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpabus.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpabusv
    1 << 8 | 1, // llvm.hexagon.V6.vmpabusv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpabuu
    1 << 8 | 1, // llvm.hexagon.V6.vmpabuu.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpabuu.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpabuu.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpabuuv
    1 << 8 | 1, // llvm.hexagon.V6.vmpabuuv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpahb
    1 << 8 | 1, // llvm.hexagon.V6.vmpahb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpahb.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpahb.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpahhsat
    1 << 8 | 1, // llvm.hexagon.V6.vmpahhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpauhb
    1 << 8 | 1, // llvm.hexagon.V6.vmpauhb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpauhb.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpauhb.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpauhuhsat
    1 << 8 | 1, // llvm.hexagon.V6.vmpauhuhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpsuhuhsat
    1 << 8 | 1, // llvm.hexagon.V6.vmpsuhuhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.hf.f8
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.hf.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.hf.f8.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.hf.f8.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.hf.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.hf.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.hf.hf.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.hf.hf.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf16.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf16.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf16.mix.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf16.mix.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.mix.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.mix.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.sf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.qf32.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.rt.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.rt.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.rt.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.rt.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.rt.sf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.rt.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.bf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.bf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.bf.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.bf.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.hf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.hf.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.hf.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.sf
    1 << 8 | 1, // llvm.hexagon.V6.vmpy.sf.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpybus
    1 << 8 | 1, // llvm.hexagon.V6.vmpybus.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpybus.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpybus.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpybusv
    1 << 8 | 1, // llvm.hexagon.V6.vmpybusv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpybusv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpybusv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpybv
    1 << 8 | 1, // llvm.hexagon.V6.vmpybv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpybv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpybv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyewuh
    1 << 8 | 1, // llvm.hexagon.V6.vmpyewuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyewuh.64
    1 << 8 | 1, // llvm.hexagon.V6.vmpyewuh.64.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyh
    1 << 8 | 1, // llvm.hexagon.V6.vmpyh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhsat.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhsat.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhsrs
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhsrs.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhss
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhss.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhus
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhus.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhus.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhus.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhv
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhvsrs
    1 << 8 | 1, // llvm.hexagon.V6.vmpyhvsrs.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyieoh
    1 << 8 | 1, // llvm.hexagon.V6.vmpyieoh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiewh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiewh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiewuh
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiewuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiewuh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiewuh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyih
    1 << 8 | 1, // llvm.hexagon.V6.vmpyih.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyih.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyih.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyihb
    1 << 8 | 1, // llvm.hexagon.V6.vmpyihb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyihb.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyihb.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiowh
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiowh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwb
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwb.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwb.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwh
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwub
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwub.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyiwub.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.64.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.64.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.rnd
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.rnd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.rnd.sacc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.rnd.sacc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.sacc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyowh.sacc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyss.parts
    1 << 8 | 1, // llvm.hexagon.V6.vmpyss.parts.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyub
    1 << 8 | 1, // llvm.hexagon.V6.vmpyub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyub.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyub.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyubv
    1 << 8 | 1, // llvm.hexagon.V6.vmpyubv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyubv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyubv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuh
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuh.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuh.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhe
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhe.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhe.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhe.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhv
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhvs
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuhvs.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyus.parts
    1 << 8 | 1, // llvm.hexagon.V6.vmpyus.parts.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuu.parts
    1 << 8 | 1, // llvm.hexagon.V6.vmpyuu.parts.128B
    1 << 8 | 1, // llvm.hexagon.V6.vmux
    1 << 8 | 1, // llvm.hexagon.V6.vmux.128B
    1 << 8 | 1, // llvm.hexagon.V6.vnavgb
    1 << 8 | 1, // llvm.hexagon.V6.vnavgb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vnavgh
    1 << 8 | 1, // llvm.hexagon.V6.vnavgh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vnavgub
    1 << 8 | 1, // llvm.hexagon.V6.vnavgub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vnavgw
    1 << 8 | 1, // llvm.hexagon.V6.vnavgw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vneg.qf16.hf
    1 << 8 | 1, // llvm.hexagon.V6.vneg.qf16.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vneg.qf16.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vneg.qf16.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vneg.qf32.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vneg.qf32.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vneg.qf32.sf
    1 << 8 | 1, // llvm.hexagon.V6.vneg.qf32.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vnormamth
    1 << 8 | 1, // llvm.hexagon.V6.vnormamth.128B
    1 << 8 | 1, // llvm.hexagon.V6.vnormamtw
    1 << 8 | 1, // llvm.hexagon.V6.vnormamtw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vnot
    1 << 8 | 1, // llvm.hexagon.V6.vnot.128B
    1 << 8 | 1, // llvm.hexagon.V6.vor
    1 << 8 | 1, // llvm.hexagon.V6.vor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpackeb
    1 << 8 | 1, // llvm.hexagon.V6.vpackeb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpackeh
    1 << 8 | 1, // llvm.hexagon.V6.vpackeh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpackhb.sat
    1 << 8 | 1, // llvm.hexagon.V6.vpackhb.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpackhub.sat
    1 << 8 | 1, // llvm.hexagon.V6.vpackhub.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpackob
    1 << 8 | 1, // llvm.hexagon.V6.vpackob.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpackoh
    1 << 8 | 1, // llvm.hexagon.V6.vpackoh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpackwh.sat
    1 << 8 | 1, // llvm.hexagon.V6.vpackwh.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpackwuh.sat
    1 << 8 | 1, // llvm.hexagon.V6.vpackwuh.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vpopcounth
    1 << 8 | 1, // llvm.hexagon.V6.vpopcounth.128B
    1 << 8 | 1, // llvm.hexagon.V6.vprefixqb
    1 << 8 | 1, // llvm.hexagon.V6.vprefixqb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vprefixqh
    1 << 8 | 1, // llvm.hexagon.V6.vprefixqh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vprefixqw
    1 << 8 | 1, // llvm.hexagon.V6.vprefixqw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrdelta
    1 << 8 | 1, // llvm.hexagon.V6.vrdelta.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybub.rtt
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybub.rtt.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybub.rtt.acc
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybub.rtt.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybus
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybus.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybus.acc
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybus.acc.128B
    1 << 8 | 11, // llvm.hexagon.V6.vrmpybusi
    1 << 8 | 11, // llvm.hexagon.V6.vrmpybusi.128B
    1 << 8 | 35, // llvm.hexagon.V6.vrmpybusi.acc
    1 << 8 | 35, // llvm.hexagon.V6.vrmpybusi.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybusv
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybusv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybusv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybusv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybv
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vrmpybv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyub
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyub.acc
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyub.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyub.rtt
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyub.rtt.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyub.rtt.acc
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyub.rtt.acc.128B
    1 << 8 | 11, // llvm.hexagon.V6.vrmpyubi
    1 << 8 | 11, // llvm.hexagon.V6.vrmpyubi.128B
    1 << 8 | 35, // llvm.hexagon.V6.vrmpyubi.acc
    1 << 8 | 35, // llvm.hexagon.V6.vrmpyubi.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyubv
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyubv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyubv.acc
    1 << 8 | 1, // llvm.hexagon.V6.vrmpyubv.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vror
    1 << 8 | 1, // llvm.hexagon.V6.vror.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrotr
    1 << 8 | 1, // llvm.hexagon.V6.vrotr.128B
    1 << 8 | 1, // llvm.hexagon.V6.vroundhb
    1 << 8 | 1, // llvm.hexagon.V6.vroundhb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vroundhub
    1 << 8 | 1, // llvm.hexagon.V6.vroundhub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrounduhub
    1 << 8 | 1, // llvm.hexagon.V6.vrounduhub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vrounduwuh
    1 << 8 | 1, // llvm.hexagon.V6.vrounduwuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vroundwh
    1 << 8 | 1, // llvm.hexagon.V6.vroundwh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vroundwuh
    1 << 8 | 1, // llvm.hexagon.V6.vroundwuh.128B
    1 << 8 | 11, // llvm.hexagon.V6.vrsadubi
    1 << 8 | 11, // llvm.hexagon.V6.vrsadubi.128B
    1 << 8 | 35, // llvm.hexagon.V6.vrsadubi.acc
    1 << 8 | 35, // llvm.hexagon.V6.vrsadubi.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsatdw
    1 << 8 | 1, // llvm.hexagon.V6.vsatdw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsathub
    1 << 8 | 1, // llvm.hexagon.V6.vsathub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsatuwuh
    1 << 8 | 1, // llvm.hexagon.V6.vsatuwuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsatwh
    1 << 8 | 1, // llvm.hexagon.V6.vsatwh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsb
    1 << 8 | 1, // llvm.hexagon.V6.vsb.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermh
    24 << 8 | 1, // llvm.hexagon.V6.vscattermh.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermh.add
    24 << 8 | 1, // llvm.hexagon.V6.vscattermh.add.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermhq
    24 << 8 | 1, // llvm.hexagon.V6.vscattermhq.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermhw
    24 << 8 | 1, // llvm.hexagon.V6.vscattermhw.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermhw.add
    24 << 8 | 1, // llvm.hexagon.V6.vscattermhw.add.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermhwq
    24 << 8 | 1, // llvm.hexagon.V6.vscattermhwq.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermw
    24 << 8 | 1, // llvm.hexagon.V6.vscattermw.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermw.add
    24 << 8 | 1, // llvm.hexagon.V6.vscattermw.add.128B
    24 << 8 | 1, // llvm.hexagon.V6.vscattermwq
    24 << 8 | 1, // llvm.hexagon.V6.vscattermwq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsh
    1 << 8 | 1, // llvm.hexagon.V6.vsh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshufeh
    1 << 8 | 1, // llvm.hexagon.V6.vshufeh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshuffb
    1 << 8 | 1, // llvm.hexagon.V6.vshuffb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshuffeb
    1 << 8 | 1, // llvm.hexagon.V6.vshuffeb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshuffh
    1 << 8 | 1, // llvm.hexagon.V6.vshuffh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshuffob
    1 << 8 | 1, // llvm.hexagon.V6.vshuffob.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshuffvdd
    1 << 8 | 1, // llvm.hexagon.V6.vshuffvdd.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshufoeb
    1 << 8 | 1, // llvm.hexagon.V6.vshufoeb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshufoeh
    1 << 8 | 1, // llvm.hexagon.V6.vshufoeh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vshufoh
    1 << 8 | 1, // llvm.hexagon.V6.vshufoh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.hf
    1 << 8 | 1, // llvm.hexagon.V6.vsub.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.hf.f8
    1 << 8 | 1, // llvm.hexagon.V6.vsub.hf.f8.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.hf.hf
    1 << 8 | 1, // llvm.hexagon.V6.vsub.hf.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.hf.mix
    1 << 8 | 1, // llvm.hexagon.V6.vsub.hf.mix.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.qf16
    1 << 8 | 1, // llvm.hexagon.V6.vsub.qf16.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.qf16.mix
    1 << 8 | 1, // llvm.hexagon.V6.vsub.qf16.mix.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.qf32
    1 << 8 | 1, // llvm.hexagon.V6.vsub.qf32.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.qf32.mix
    1 << 8 | 1, // llvm.hexagon.V6.vsub.qf32.mix.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.bf
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.bf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.hf
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.hf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.mix
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.mix.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.sf
    1 << 8 | 1, // llvm.hexagon.V6.vsub.sf.sf.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubb
    1 << 8 | 1, // llvm.hexagon.V6.vsubb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubb.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsubb.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubbnq
    1 << 8 | 1, // llvm.hexagon.V6.vsubbnq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubbq
    1 << 8 | 1, // llvm.hexagon.V6.vsubbq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubbsat
    1 << 8 | 1, // llvm.hexagon.V6.vsubbsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubbsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsubbsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubcarry
    1 << 8 | 1, // llvm.hexagon.V6.vsubcarry.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubcarryo
    1 << 8 | 1, // llvm.hexagon.V6.vsubcarryo.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubh
    1 << 8 | 1, // llvm.hexagon.V6.vsubh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubh.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsubh.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubhnq
    1 << 8 | 1, // llvm.hexagon.V6.vsubhnq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubhq
    1 << 8 | 1, // llvm.hexagon.V6.vsubhq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubhsat
    1 << 8 | 1, // llvm.hexagon.V6.vsubhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubhsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsubhsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubhw
    1 << 8 | 1, // llvm.hexagon.V6.vsubhw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsububh
    1 << 8 | 1, // llvm.hexagon.V6.vsububh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsububsat
    1 << 8 | 1, // llvm.hexagon.V6.vsububsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsububsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsububsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubububb.sat
    1 << 8 | 1, // llvm.hexagon.V6.vsubububb.sat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubuhsat
    1 << 8 | 1, // llvm.hexagon.V6.vsubuhsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubuhsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsubuhsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubuhw
    1 << 8 | 1, // llvm.hexagon.V6.vsubuhw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubuwsat
    1 << 8 | 1, // llvm.hexagon.V6.vsubuwsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubuwsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsubuwsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubw
    1 << 8 | 1, // llvm.hexagon.V6.vsubw.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubw.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsubw.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubwnq
    1 << 8 | 1, // llvm.hexagon.V6.vsubwnq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubwq
    1 << 8 | 1, // llvm.hexagon.V6.vsubwq.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubwsat
    1 << 8 | 1, // llvm.hexagon.V6.vsubwsat.128B
    1 << 8 | 1, // llvm.hexagon.V6.vsubwsat.dv
    1 << 8 | 1, // llvm.hexagon.V6.vsubwsat.dv.128B
    1 << 8 | 1, // llvm.hexagon.V6.vswap
    1 << 8 | 1, // llvm.hexagon.V6.vswap.128B
    1 << 8 | 1, // llvm.hexagon.V6.vtmpyb
    1 << 8 | 1, // llvm.hexagon.V6.vtmpyb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vtmpyb.acc
    1 << 8 | 1, // llvm.hexagon.V6.vtmpyb.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vtmpybus
    1 << 8 | 1, // llvm.hexagon.V6.vtmpybus.128B
    1 << 8 | 1, // llvm.hexagon.V6.vtmpybus.acc
    1 << 8 | 1, // llvm.hexagon.V6.vtmpybus.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vtmpyhb
    1 << 8 | 1, // llvm.hexagon.V6.vtmpyhb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vtmpyhb.acc
    1 << 8 | 1, // llvm.hexagon.V6.vtmpyhb.acc.128B
    1 << 8 | 1, // llvm.hexagon.V6.vunpackb
    1 << 8 | 1, // llvm.hexagon.V6.vunpackb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vunpackh
    1 << 8 | 1, // llvm.hexagon.V6.vunpackh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vunpackob
    1 << 8 | 1, // llvm.hexagon.V6.vunpackob.128B
    1 << 8 | 1, // llvm.hexagon.V6.vunpackoh
    1 << 8 | 1, // llvm.hexagon.V6.vunpackoh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vunpackub
    1 << 8 | 1, // llvm.hexagon.V6.vunpackub.128B
    1 << 8 | 1, // llvm.hexagon.V6.vunpackuh
    1 << 8 | 1, // llvm.hexagon.V6.vunpackuh.128B
    1 << 8 | 1, // llvm.hexagon.V6.vxor
    1 << 8 | 1, // llvm.hexagon.V6.vxor.128B
    1 << 8 | 1, // llvm.hexagon.V6.vzb
    1 << 8 | 1, // llvm.hexagon.V6.vzb.128B
    1 << 8 | 1, // llvm.hexagon.V6.vzh
    1 << 8 | 1, // llvm.hexagon.V6.vzh.128B
    7 << 8 | 1, // llvm.hexagon.Y2.dccleana
    7 << 8 | 1, // llvm.hexagon.Y2.dccleaninva
    7 << 8 | 1, // llvm.hexagon.Y2.dcfetch
    7 << 8 | 1, // llvm.hexagon.Y2.dcinva
    7 << 8 | 1, // llvm.hexagon.Y2.dczeroa
    7 << 8 | 1, // llvm.hexagon.Y4.l2fetch
    7 << 8 | 1, // llvm.hexagon.Y5.l2fetch
    78 << 8 | 1, // llvm.hexagon.Y6.dmlink
    78 << 8 | 1, // llvm.hexagon.Y6.dmpause
    78 << 8 | 1, // llvm.hexagon.Y6.dmpoll
    78 << 8 | 1, // llvm.hexagon.Y6.dmresume
    78 << 8 | 1, // llvm.hexagon.Y6.dmstart
    78 << 8 | 1, // llvm.hexagon.Y6.dmwait
    18 << 8 | 35, // llvm.hexagon.circ.ldb
    18 << 8 | 35, // llvm.hexagon.circ.ldd
    18 << 8 | 35, // llvm.hexagon.circ.ldh
    18 << 8 | 35, // llvm.hexagon.circ.ldub
    18 << 8 | 35, // llvm.hexagon.circ.lduh
    18 << 8 | 35, // llvm.hexagon.circ.ldw
    24 << 8 | 35, // llvm.hexagon.circ.stb
    24 << 8 | 35, // llvm.hexagon.circ.std
    24 << 8 | 35, // llvm.hexagon.circ.sth
    24 << 8 | 35, // llvm.hexagon.circ.sthhi
    24 << 8 | 35, // llvm.hexagon.circ.stw
    3 << 8 | 1, // llvm.hexagon.instrprof.custom
    7 << 8 | 1, // llvm.hexagon.prefetch
    18 << 8 | 138, // llvm.hexagon.vmemcpy
    18 << 8 | 36, // llvm.hexagon.vmemset
    5 << 8 | 1, // llvm.loongarch.asrtgt.d
    5 << 8 | 1, // llvm.loongarch.asrtle.d
    5 << 8 | 5, // llvm.loongarch.break
    5 << 8 | 46, // llvm.loongarch.cacop.d
    5 << 8 | 46, // llvm.loongarch.cacop.w
    5 << 8 | 1, // llvm.loongarch.cpucfg
    5 << 8 | 1, // llvm.loongarch.crc.w.b.w
    5 << 8 | 1, // llvm.loongarch.crc.w.d.w
    5 << 8 | 1, // llvm.loongarch.crc.w.h.w
    5 << 8 | 1, // llvm.loongarch.crc.w.w.w
    5 << 8 | 1, // llvm.loongarch.crcc.w.b.w
    5 << 8 | 1, // llvm.loongarch.crcc.w.d.w
    5 << 8 | 1, // llvm.loongarch.crcc.w.h.w
    5 << 8 | 1, // llvm.loongarch.crcc.w.w.w
    5 << 8 | 5, // llvm.loongarch.csrrd.d
    5 << 8 | 5, // llvm.loongarch.csrrd.w
    5 << 8 | 0, // llvm.loongarch.csrwr.d
    5 << 8 | 0, // llvm.loongarch.csrwr.w
    5 << 8 | 11, // llvm.loongarch.csrxchg.d
    5 << 8 | 11, // llvm.loongarch.csrxchg.w
    5 << 8 | 5, // llvm.loongarch.dbar
    10 << 8 | 1, // llvm.loongarch.frecipe.d
    10 << 8 | 1, // llvm.loongarch.frecipe.s
    10 << 8 | 1, // llvm.loongarch.frsqrte.d
    10 << 8 | 1, // llvm.loongarch.frsqrte.s
    5 << 8 | 5, // llvm.loongarch.ibar
    5 << 8 | 1, // llvm.loongarch.iocsrrd.b
    5 << 8 | 1, // llvm.loongarch.iocsrrd.d
    5 << 8 | 1, // llvm.loongarch.iocsrrd.h
    5 << 8 | 1, // llvm.loongarch.iocsrrd.w
    5 << 8 | 1, // llvm.loongarch.iocsrwr.b
    5 << 8 | 1, // llvm.loongarch.iocsrwr.d
    5 << 8 | 1, // llvm.loongarch.iocsrwr.h
    5 << 8 | 1, // llvm.loongarch.iocsrwr.w
    10 << 8 | 1, // llvm.loongarch.lasx.cast.128
    10 << 8 | 1, // llvm.loongarch.lasx.cast.128.d
    10 << 8 | 1, // llvm.loongarch.lasx.cast.128.s
    10 << 8 | 1, // llvm.loongarch.lasx.concat.128
    10 << 8 | 1, // llvm.loongarch.lasx.concat.128.d
    10 << 8 | 1, // llvm.loongarch.lasx.concat.128.s
    10 << 8 | 1, // llvm.loongarch.lasx.extract.128.hi
    10 << 8 | 1, // llvm.loongarch.lasx.extract.128.hi.d
    10 << 8 | 1, // llvm.loongarch.lasx.extract.128.hi.s
    10 << 8 | 1, // llvm.loongarch.lasx.extract.128.lo
    10 << 8 | 1, // llvm.loongarch.lasx.extract.128.lo.d
    10 << 8 | 1, // llvm.loongarch.lasx.extract.128.lo.s
    10 << 8 | 1, // llvm.loongarch.lasx.insert.128.hi
    10 << 8 | 1, // llvm.loongarch.lasx.insert.128.hi.d
    10 << 8 | 1, // llvm.loongarch.lasx.insert.128.hi.s
    10 << 8 | 1, // llvm.loongarch.lasx.insert.128.lo
    10 << 8 | 1, // llvm.loongarch.lasx.insert.128.lo.d
    10 << 8 | 1, // llvm.loongarch.lasx.insert.128.lo.s
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.d.b
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.d.h
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.du.bu
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.du.hu
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.du.wu
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.hu.bu
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.w.b
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.wu.bu
    10 << 8 | 1, // llvm.loongarch.lasx.vext2xv.wu.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xbnz.b
    10 << 8 | 1, // llvm.loongarch.lasx.xbnz.d
    10 << 8 | 1, // llvm.loongarch.lasx.xbnz.h
    10 << 8 | 1, // llvm.loongarch.lasx.xbnz.v
    10 << 8 | 1, // llvm.loongarch.lasx.xbnz.w
    10 << 8 | 1, // llvm.loongarch.lasx.xbz.b
    10 << 8 | 1, // llvm.loongarch.lasx.xbz.d
    10 << 8 | 1, // llvm.loongarch.lasx.xbz.h
    10 << 8 | 1, // llvm.loongarch.lasx.xbz.v
    10 << 8 | 1, // llvm.loongarch.lasx.xbz.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvabsd.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvabsd.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvabsd.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvabsd.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvabsd.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvabsd.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvabsd.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvabsd.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvadd.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvadd.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvadd.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvadd.q
    10 << 8 | 1, // llvm.loongarch.lasx.xvadd.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvadda.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvadda.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvadda.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvadda.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvaddi.bu
    10 << 8 | 0, // llvm.loongarch.lasx.xvaddi.du
    10 << 8 | 0, // llvm.loongarch.lasx.xvaddi.hu
    10 << 8 | 0, // llvm.loongarch.lasx.xvaddi.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.d.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.h.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.q.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.q.du.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.w.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwev.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.d.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.h.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.q.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.q.du.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.w.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvaddwod.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvand.v
    10 << 8 | 0, // llvm.loongarch.lasx.xvandi.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvandn.v
    10 << 8 | 1, // llvm.loongarch.lasx.xvavg.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvavg.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvavg.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvavg.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvavg.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvavg.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvavg.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvavg.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvavgr.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvavgr.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvavgr.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvavgr.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvavgr.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvavgr.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvavgr.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvavgr.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitclr.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitclr.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitclr.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitclr.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitclri.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitclri.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitclri.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitclri.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitrev.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitrev.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitrev.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitrev.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitrevi.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitrevi.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitrevi.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitrevi.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitsel.v
    10 << 8 | 11, // llvm.loongarch.lasx.xvbitseli.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitset.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitset.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitset.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvbitset.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitseti.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitseti.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitseti.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvbitseti.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvbsll.v
    10 << 8 | 0, // llvm.loongarch.lasx.xvbsrl.v
    10 << 8 | 1, // llvm.loongarch.lasx.xvclo.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvclo.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvclo.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvclo.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvclz.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvclz.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvclz.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvclz.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvdiv.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvdiv.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvdiv.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvdiv.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvdiv.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvdiv.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvdiv.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvdiv.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvexth.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvexth.du.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvexth.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvexth.hu.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvexth.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvexth.qu.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvexth.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvexth.wu.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvextl.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvextl.qu.du
    10 << 8 | 11, // llvm.loongarch.lasx.xvextrins.b
    10 << 8 | 11, // llvm.loongarch.lasx.xvextrins.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvextrins.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvextrins.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvfadd.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfadd.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfclass.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfclass.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.caf.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.caf.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.ceq.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.ceq.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cle.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cle.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.clt.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.clt.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cne.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cne.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cor.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cor.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cueq.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cueq.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cule.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cule.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cult.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cult.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cun.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cun.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cune.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.cune.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.saf.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.saf.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.seq.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.seq.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sle.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sle.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.slt.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.slt.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sne.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sne.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sor.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sor.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sueq.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sueq.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sule.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sule.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sult.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sult.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sun.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sun.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sune.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcmp.sune.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcvt.h.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcvt.s.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcvth.d.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcvth.s.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcvtl.d.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfcvtl.s.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvfdiv.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfdiv.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvffint.d.l
    10 << 8 | 1, // llvm.loongarch.lasx.xvffint.d.lu
    10 << 8 | 1, // llvm.loongarch.lasx.xvffint.s.l
    10 << 8 | 1, // llvm.loongarch.lasx.xvffint.s.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvffint.s.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvffinth.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvffintl.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvflogb.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvflogb.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmadd.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmadd.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmax.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmax.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmaxa.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmaxa.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmin.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmin.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmina.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmina.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmsub.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmsub.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmul.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfmul.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfnmadd.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfnmadd.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfnmsub.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfnmsub.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrecip.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrecip.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrecipe.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrecipe.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrint.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrint.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrintrm.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrintrm.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrintrne.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrintrne.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrintrp.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrintrp.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrintrz.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrintrz.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrsqrt.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrsqrt.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrsqrte.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrsqrte.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrstp.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvfrstp.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvfrstpi.b
    10 << 8 | 11, // llvm.loongarch.lasx.xvfrstpi.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvfsqrt.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfsqrt.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvfsub.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvfsub.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftint.l.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftint.lu.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftint.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftint.w.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftint.wu.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftinth.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintl.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrm.l.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrm.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrm.w.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrmh.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrml.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrne.l.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrne.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrne.w.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrneh.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrnel.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrp.l.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrp.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrp.w.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrph.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrpl.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrz.l.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrz.lu.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrz.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrz.w.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrz.wu.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrzh.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvftintrzl.l.s
    10 << 8 | 1, // llvm.loongarch.lasx.xvhaddw.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvhaddw.du.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvhaddw.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvhaddw.hu.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvhaddw.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvhaddw.qu.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvhaddw.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvhaddw.wu.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvhsubw.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvhsubw.du.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvhsubw.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvhsubw.hu.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvhsubw.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvhsubw.qu.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvhsubw.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvhsubw.wu.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvilvh.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvilvh.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvilvh.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvilvh.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvilvl.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvilvl.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvilvl.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvilvl.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvinsgr2vr.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvinsgr2vr.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvinsve0.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvinsve0.w
    2 << 8 | 0, // llvm.loongarch.lasx.xvld
    10 << 8 | 5, // llvm.loongarch.lasx.xvldi
    2 << 8 | 0, // llvm.loongarch.lasx.xvldrepl.b
    2 << 8 | 0, // llvm.loongarch.lasx.xvldrepl.d
    2 << 8 | 0, // llvm.loongarch.lasx.xvldrepl.h
    2 << 8 | 0, // llvm.loongarch.lasx.xvldrepl.w
    2 << 8 | 1, // llvm.loongarch.lasx.xvldx
    10 << 8 | 1, // llvm.loongarch.lasx.xvmadd.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmadd.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmadd.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmadd.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.d.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.h.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.q.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.q.du.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.w.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwev.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.d.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.h.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.q.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.q.du.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.w.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmaddwod.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmax.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmax.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmax.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmax.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvmax.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmax.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmax.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmax.wu
    10 << 8 | 0, // llvm.loongarch.lasx.xvmaxi.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvmaxi.bu
    10 << 8 | 0, // llvm.loongarch.lasx.xvmaxi.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvmaxi.du
    10 << 8 | 0, // llvm.loongarch.lasx.xvmaxi.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvmaxi.hu
    10 << 8 | 0, // llvm.loongarch.lasx.xvmaxi.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvmaxi.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmin.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmin.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmin.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmin.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvmin.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmin.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmin.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmin.wu
    10 << 8 | 0, // llvm.loongarch.lasx.xvmini.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvmini.bu
    10 << 8 | 0, // llvm.loongarch.lasx.xvmini.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvmini.du
    10 << 8 | 0, // llvm.loongarch.lasx.xvmini.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvmini.hu
    10 << 8 | 0, // llvm.loongarch.lasx.xvmini.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvmini.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmod.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmod.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmod.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmod.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvmod.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmod.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmod.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmod.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmskgez.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmskltz.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmskltz.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmskltz.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmskltz.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmsknz.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmsub.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmsub.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmsub.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmsub.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmuh.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmuh.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmuh.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmuh.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvmuh.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmuh.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmuh.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmuh.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmul.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmul.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmul.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmul.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.d.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.h.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.q.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.q.du.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.w.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwev.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.d.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.h.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.q.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.q.du.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.w.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvmulwod.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvneg.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvneg.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvneg.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvneg.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvnor.v
    10 << 8 | 0, // llvm.loongarch.lasx.xvnori.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvor.v
    10 << 8 | 0, // llvm.loongarch.lasx.xvori.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvorn.v
    10 << 8 | 1, // llvm.loongarch.lasx.xvpackev.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvpackev.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvpackev.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvpackev.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvpackod.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvpackod.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvpackod.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvpackod.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvpcnt.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvpcnt.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvpcnt.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvpcnt.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvperm.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvpermi.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvpermi.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvpermi.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvpickev.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvpickev.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvpickev.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvpickev.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvpickod.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvpickod.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvpickod.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvpickod.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvpickve.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvpickve.d.f
    10 << 8 | 0, // llvm.loongarch.lasx.xvpickve.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvpickve.w.f
    10 << 8 | 0, // llvm.loongarch.lasx.xvpickve2gr.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvpickve2gr.du
    10 << 8 | 0, // llvm.loongarch.lasx.xvpickve2gr.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvpickve2gr.wu
    10 << 8 | 0, // llvm.loongarch.lasx.xvrepl128vei.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvrepl128vei.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvrepl128vei.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvrepl128vei.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplgr2vr.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplgr2vr.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplgr2vr.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplgr2vr.w
    10 << 8 | 5, // llvm.loongarch.lasx.xvrepli.b
    10 << 8 | 5, // llvm.loongarch.lasx.xvrepli.d
    10 << 8 | 5, // llvm.loongarch.lasx.xvrepli.h
    10 << 8 | 5, // llvm.loongarch.lasx.xvrepli.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve0.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve0.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve0.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve0.q
    10 << 8 | 1, // llvm.loongarch.lasx.xvreplve0.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvrotr.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvrotr.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvrotr.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvrotr.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvrotri.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvrotri.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvrotri.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvrotri.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsadd.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsadd.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsadd.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsadd.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvsadd.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsadd.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsadd.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsadd.wu
    10 << 8 | 0, // llvm.loongarch.lasx.xvsat.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvsat.bu
    10 << 8 | 0, // llvm.loongarch.lasx.xvsat.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvsat.du
    10 << 8 | 0, // llvm.loongarch.lasx.xvsat.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvsat.hu
    10 << 8 | 0, // llvm.loongarch.lasx.xvsat.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvsat.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvseq.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvseq.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvseq.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvseq.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvseqi.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvseqi.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvseqi.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvseqi.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvshuf.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvshuf.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvshuf.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvshuf.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvshuf4i.b
    10 << 8 | 11, // llvm.loongarch.lasx.xvshuf4i.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvshuf4i.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvshuf4i.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsigncov.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsigncov.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsigncov.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsigncov.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsle.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsle.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsle.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsle.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvsle.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsle.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsle.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsle.wu
    10 << 8 | 0, // llvm.loongarch.lasx.xvslei.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvslei.bu
    10 << 8 | 0, // llvm.loongarch.lasx.xvslei.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvslei.du
    10 << 8 | 0, // llvm.loongarch.lasx.xvslei.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvslei.hu
    10 << 8 | 0, // llvm.loongarch.lasx.xvslei.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvslei.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsll.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsll.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsll.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsll.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvslli.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvslli.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvslli.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvslli.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvsllwil.d.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvsllwil.du.wu
    10 << 8 | 0, // llvm.loongarch.lasx.xvsllwil.h.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvsllwil.hu.bu
    10 << 8 | 0, // llvm.loongarch.lasx.xvsllwil.w.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvsllwil.wu.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvslt.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvslt.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvslt.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvslt.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvslt.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvslt.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvslt.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvslt.wu
    10 << 8 | 0, // llvm.loongarch.lasx.xvslti.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvslti.bu
    10 << 8 | 0, // llvm.loongarch.lasx.xvslti.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvslti.du
    10 << 8 | 0, // llvm.loongarch.lasx.xvslti.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvslti.hu
    10 << 8 | 0, // llvm.loongarch.lasx.xvslti.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvslti.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsra.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsra.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsra.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsra.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrai.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrai.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrai.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrai.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsran.b.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsran.h.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsran.w.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrani.b.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrani.d.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrani.h.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrani.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrar.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrar.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrar.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrar.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrari.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrari.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrari.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrari.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrarn.b.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrarn.h.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrarn.w.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrarni.b.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrarni.d.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrarni.h.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrarni.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrl.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrl.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrl.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrl.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrli.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrli.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrli.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrli.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrln.b.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrln.h.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrln.w.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrlni.b.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrlni.d.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrlni.h.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrlni.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrlr.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrlr.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrlr.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrlr.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrlri.b
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrlri.d
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrlri.h
    10 << 8 | 0, // llvm.loongarch.lasx.xvsrlri.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrlrn.b.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrlrn.h.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsrlrn.w.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrlrni.b.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrlrni.d.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrlrni.h.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvsrlrni.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssran.b.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssran.bu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssran.h.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssran.hu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssran.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssran.wu.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrani.b.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrani.bu.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrani.d.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrani.du.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrani.h.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrani.hu.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrani.w.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrani.wu.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrarn.b.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrarn.bu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrarn.h.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrarn.hu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrarn.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrarn.wu.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrarni.b.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrarni.bu.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrarni.d.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrarni.du.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrarni.h.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrarni.hu.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrarni.w.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrarni.wu.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrln.b.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrln.bu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrln.h.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrln.hu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrln.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrln.wu.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlni.b.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlni.bu.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlni.d.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlni.du.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlni.h.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlni.hu.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlni.w.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlni.wu.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrlrn.b.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrlrn.bu.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrlrn.h.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrlrn.hu.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrlrn.w.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssrlrn.wu.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlrni.b.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlrni.bu.h
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlrni.d.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlrni.du.q
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlrni.h.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlrni.hu.w
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlrni.w.d
    10 << 8 | 11, // llvm.loongarch.lasx.xvssrlrni.wu.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssub.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvssub.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvssub.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvssub.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvssub.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvssub.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvssub.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvssub.wu
    79 << 8 | 11, // llvm.loongarch.lasx.xvst
    79 << 8 | 93, // llvm.loongarch.lasx.xvstelm.b
    79 << 8 | 93, // llvm.loongarch.lasx.xvstelm.d
    79 << 8 | 93, // llvm.loongarch.lasx.xvstelm.h
    79 << 8 | 93, // llvm.loongarch.lasx.xvstelm.w
    79 << 8 | 1, // llvm.loongarch.lasx.xvstx
    10 << 8 | 1, // llvm.loongarch.lasx.xvsub.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsub.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsub.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsub.q
    10 << 8 | 1, // llvm.loongarch.lasx.xvsub.w
    10 << 8 | 0, // llvm.loongarch.lasx.xvsubi.bu
    10 << 8 | 0, // llvm.loongarch.lasx.xvsubi.du
    10 << 8 | 0, // llvm.loongarch.lasx.xvsubi.hu
    10 << 8 | 0, // llvm.loongarch.lasx.xvsubi.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwev.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwev.d.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwev.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwev.h.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwev.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwev.q.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwev.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwev.w.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwod.d.w
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwod.d.wu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwod.h.b
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwod.h.bu
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwod.q.d
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwod.q.du
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwod.w.h
    10 << 8 | 1, // llvm.loongarch.lasx.xvsubwod.w.hu
    10 << 8 | 1, // llvm.loongarch.lasx.xvxor.v
    10 << 8 | 0, // llvm.loongarch.lasx.xvxori.b
    5 << 8 | 0, // llvm.loongarch.lddir.d
    5 << 8 | 0, // llvm.loongarch.ldpte.d
    10 << 8 | 1, // llvm.loongarch.lsx.bnz.b
    10 << 8 | 1, // llvm.loongarch.lsx.bnz.d
    10 << 8 | 1, // llvm.loongarch.lsx.bnz.h
    10 << 8 | 1, // llvm.loongarch.lsx.bnz.v
    10 << 8 | 1, // llvm.loongarch.lsx.bnz.w
    10 << 8 | 1, // llvm.loongarch.lsx.bz.b
    10 << 8 | 1, // llvm.loongarch.lsx.bz.d
    10 << 8 | 1, // llvm.loongarch.lsx.bz.h
    10 << 8 | 1, // llvm.loongarch.lsx.bz.v
    10 << 8 | 1, // llvm.loongarch.lsx.bz.w
    10 << 8 | 1, // llvm.loongarch.lsx.vabsd.b
    10 << 8 | 1, // llvm.loongarch.lsx.vabsd.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vabsd.d
    10 << 8 | 1, // llvm.loongarch.lsx.vabsd.du
    10 << 8 | 1, // llvm.loongarch.lsx.vabsd.h
    10 << 8 | 1, // llvm.loongarch.lsx.vabsd.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vabsd.w
    10 << 8 | 1, // llvm.loongarch.lsx.vabsd.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vadd.b
    10 << 8 | 1, // llvm.loongarch.lsx.vadd.d
    10 << 8 | 1, // llvm.loongarch.lsx.vadd.h
    10 << 8 | 1, // llvm.loongarch.lsx.vadd.q
    10 << 8 | 1, // llvm.loongarch.lsx.vadd.w
    10 << 8 | 1, // llvm.loongarch.lsx.vadda.b
    10 << 8 | 1, // llvm.loongarch.lsx.vadda.d
    10 << 8 | 1, // llvm.loongarch.lsx.vadda.h
    10 << 8 | 1, // llvm.loongarch.lsx.vadda.w
    10 << 8 | 0, // llvm.loongarch.lsx.vaddi.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vaddi.du
    10 << 8 | 0, // llvm.loongarch.lsx.vaddi.hu
    10 << 8 | 0, // llvm.loongarch.lsx.vaddi.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.d.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.h.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.q.du
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.q.du.d
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.w.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwev.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.d.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.h.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.q.du
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.q.du.d
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.w.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vaddwod.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vand.v
    10 << 8 | 0, // llvm.loongarch.lsx.vandi.b
    10 << 8 | 1, // llvm.loongarch.lsx.vandn.v
    10 << 8 | 1, // llvm.loongarch.lsx.vavg.b
    10 << 8 | 1, // llvm.loongarch.lsx.vavg.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vavg.d
    10 << 8 | 1, // llvm.loongarch.lsx.vavg.du
    10 << 8 | 1, // llvm.loongarch.lsx.vavg.h
    10 << 8 | 1, // llvm.loongarch.lsx.vavg.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vavg.w
    10 << 8 | 1, // llvm.loongarch.lsx.vavg.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vavgr.b
    10 << 8 | 1, // llvm.loongarch.lsx.vavgr.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vavgr.d
    10 << 8 | 1, // llvm.loongarch.lsx.vavgr.du
    10 << 8 | 1, // llvm.loongarch.lsx.vavgr.h
    10 << 8 | 1, // llvm.loongarch.lsx.vavgr.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vavgr.w
    10 << 8 | 1, // llvm.loongarch.lsx.vavgr.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vbitclr.b
    10 << 8 | 1, // llvm.loongarch.lsx.vbitclr.d
    10 << 8 | 1, // llvm.loongarch.lsx.vbitclr.h
    10 << 8 | 1, // llvm.loongarch.lsx.vbitclr.w
    10 << 8 | 0, // llvm.loongarch.lsx.vbitclri.b
    10 << 8 | 0, // llvm.loongarch.lsx.vbitclri.d
    10 << 8 | 0, // llvm.loongarch.lsx.vbitclri.h
    10 << 8 | 0, // llvm.loongarch.lsx.vbitclri.w
    10 << 8 | 1, // llvm.loongarch.lsx.vbitrev.b
    10 << 8 | 1, // llvm.loongarch.lsx.vbitrev.d
    10 << 8 | 1, // llvm.loongarch.lsx.vbitrev.h
    10 << 8 | 1, // llvm.loongarch.lsx.vbitrev.w
    10 << 8 | 0, // llvm.loongarch.lsx.vbitrevi.b
    10 << 8 | 0, // llvm.loongarch.lsx.vbitrevi.d
    10 << 8 | 0, // llvm.loongarch.lsx.vbitrevi.h
    10 << 8 | 0, // llvm.loongarch.lsx.vbitrevi.w
    10 << 8 | 1, // llvm.loongarch.lsx.vbitsel.v
    10 << 8 | 11, // llvm.loongarch.lsx.vbitseli.b
    10 << 8 | 1, // llvm.loongarch.lsx.vbitset.b
    10 << 8 | 1, // llvm.loongarch.lsx.vbitset.d
    10 << 8 | 1, // llvm.loongarch.lsx.vbitset.h
    10 << 8 | 1, // llvm.loongarch.lsx.vbitset.w
    10 << 8 | 0, // llvm.loongarch.lsx.vbitseti.b
    10 << 8 | 0, // llvm.loongarch.lsx.vbitseti.d
    10 << 8 | 0, // llvm.loongarch.lsx.vbitseti.h
    10 << 8 | 0, // llvm.loongarch.lsx.vbitseti.w
    10 << 8 | 0, // llvm.loongarch.lsx.vbsll.v
    10 << 8 | 0, // llvm.loongarch.lsx.vbsrl.v
    10 << 8 | 1, // llvm.loongarch.lsx.vclo.b
    10 << 8 | 1, // llvm.loongarch.lsx.vclo.d
    10 << 8 | 1, // llvm.loongarch.lsx.vclo.h
    10 << 8 | 1, // llvm.loongarch.lsx.vclo.w
    10 << 8 | 1, // llvm.loongarch.lsx.vclz.b
    10 << 8 | 1, // llvm.loongarch.lsx.vclz.d
    10 << 8 | 1, // llvm.loongarch.lsx.vclz.h
    10 << 8 | 1, // llvm.loongarch.lsx.vclz.w
    10 << 8 | 1, // llvm.loongarch.lsx.vdiv.b
    10 << 8 | 1, // llvm.loongarch.lsx.vdiv.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vdiv.d
    10 << 8 | 1, // llvm.loongarch.lsx.vdiv.du
    10 << 8 | 1, // llvm.loongarch.lsx.vdiv.h
    10 << 8 | 1, // llvm.loongarch.lsx.vdiv.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vdiv.w
    10 << 8 | 1, // llvm.loongarch.lsx.vdiv.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vexth.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vexth.du.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vexth.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vexth.hu.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vexth.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vexth.qu.du
    10 << 8 | 1, // llvm.loongarch.lsx.vexth.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vexth.wu.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vextl.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vextl.qu.du
    10 << 8 | 11, // llvm.loongarch.lsx.vextrins.b
    10 << 8 | 11, // llvm.loongarch.lsx.vextrins.d
    10 << 8 | 11, // llvm.loongarch.lsx.vextrins.h
    10 << 8 | 11, // llvm.loongarch.lsx.vextrins.w
    10 << 8 | 1, // llvm.loongarch.lsx.vfadd.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfadd.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfclass.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfclass.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.caf.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.caf.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.ceq.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.ceq.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cle.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cle.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.clt.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.clt.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cne.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cne.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cor.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cor.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cueq.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cueq.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cule.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cule.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cult.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cult.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cun.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cun.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cune.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.cune.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.saf.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.saf.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.seq.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.seq.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sle.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sle.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.slt.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.slt.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sne.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sne.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sor.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sor.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sueq.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sueq.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sule.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sule.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sult.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sult.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sun.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sun.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sune.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcmp.sune.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcvt.h.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcvt.s.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfcvth.d.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcvth.s.h
    10 << 8 | 1, // llvm.loongarch.lsx.vfcvtl.d.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfcvtl.s.h
    10 << 8 | 1, // llvm.loongarch.lsx.vfdiv.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfdiv.s
    10 << 8 | 1, // llvm.loongarch.lsx.vffint.d.l
    10 << 8 | 1, // llvm.loongarch.lsx.vffint.d.lu
    10 << 8 | 1, // llvm.loongarch.lsx.vffint.s.l
    10 << 8 | 1, // llvm.loongarch.lsx.vffint.s.w
    10 << 8 | 1, // llvm.loongarch.lsx.vffint.s.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vffinth.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vffintl.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vflogb.d
    10 << 8 | 1, // llvm.loongarch.lsx.vflogb.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfmadd.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfmadd.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfmax.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfmax.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfmaxa.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfmaxa.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfmin.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfmin.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfmina.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfmina.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfmsub.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfmsub.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfmul.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfmul.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfnmadd.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfnmadd.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfnmsub.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfnmsub.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrecip.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrecip.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrecipe.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrecipe.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrint.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrint.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrintrm.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrintrm.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrintrne.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrintrne.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrintrp.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrintrp.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrintrz.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrintrz.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrsqrt.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrsqrt.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrsqrte.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfrsqrte.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfrstp.b
    10 << 8 | 1, // llvm.loongarch.lsx.vfrstp.h
    10 << 8 | 11, // llvm.loongarch.lsx.vfrstpi.b
    10 << 8 | 11, // llvm.loongarch.lsx.vfrstpi.h
    10 << 8 | 1, // llvm.loongarch.lsx.vfsqrt.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfsqrt.s
    10 << 8 | 1, // llvm.loongarch.lsx.vfsub.d
    10 << 8 | 1, // llvm.loongarch.lsx.vfsub.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftint.l.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftint.lu.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftint.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftint.w.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftint.wu.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftinth.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintl.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrm.l.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrm.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrm.w.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrmh.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrml.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrne.l.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrne.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrne.w.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrneh.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrnel.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrp.l.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrp.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrp.w.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrph.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrpl.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrz.l.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrz.lu.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrz.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrz.w.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrz.wu.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrzh.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vftintrzl.l.s
    10 << 8 | 1, // llvm.loongarch.lsx.vhaddw.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vhaddw.du.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vhaddw.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vhaddw.hu.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vhaddw.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vhaddw.qu.du
    10 << 8 | 1, // llvm.loongarch.lsx.vhaddw.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vhaddw.wu.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vhsubw.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vhsubw.du.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vhsubw.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vhsubw.hu.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vhsubw.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vhsubw.qu.du
    10 << 8 | 1, // llvm.loongarch.lsx.vhsubw.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vhsubw.wu.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vilvh.b
    10 << 8 | 1, // llvm.loongarch.lsx.vilvh.d
    10 << 8 | 1, // llvm.loongarch.lsx.vilvh.h
    10 << 8 | 1, // llvm.loongarch.lsx.vilvh.w
    10 << 8 | 1, // llvm.loongarch.lsx.vilvl.b
    10 << 8 | 1, // llvm.loongarch.lsx.vilvl.d
    10 << 8 | 1, // llvm.loongarch.lsx.vilvl.h
    10 << 8 | 1, // llvm.loongarch.lsx.vilvl.w
    10 << 8 | 11, // llvm.loongarch.lsx.vinsgr2vr.b
    10 << 8 | 11, // llvm.loongarch.lsx.vinsgr2vr.d
    10 << 8 | 11, // llvm.loongarch.lsx.vinsgr2vr.h
    10 << 8 | 11, // llvm.loongarch.lsx.vinsgr2vr.w
    2 << 8 | 0, // llvm.loongarch.lsx.vld
    10 << 8 | 5, // llvm.loongarch.lsx.vldi
    2 << 8 | 0, // llvm.loongarch.lsx.vldrepl.b
    2 << 8 | 0, // llvm.loongarch.lsx.vldrepl.d
    2 << 8 | 0, // llvm.loongarch.lsx.vldrepl.h
    2 << 8 | 0, // llvm.loongarch.lsx.vldrepl.w
    2 << 8 | 1, // llvm.loongarch.lsx.vldx
    10 << 8 | 1, // llvm.loongarch.lsx.vmadd.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmadd.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmadd.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmadd.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.d.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.h.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.q.du
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.q.du.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.w.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwev.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.d.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.h.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.q.du
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.q.du.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.w.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vmaddwod.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmax.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmax.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vmax.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmax.du
    10 << 8 | 1, // llvm.loongarch.lsx.vmax.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmax.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vmax.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmax.wu
    10 << 8 | 0, // llvm.loongarch.lsx.vmaxi.b
    10 << 8 | 0, // llvm.loongarch.lsx.vmaxi.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vmaxi.d
    10 << 8 | 0, // llvm.loongarch.lsx.vmaxi.du
    10 << 8 | 0, // llvm.loongarch.lsx.vmaxi.h
    10 << 8 | 0, // llvm.loongarch.lsx.vmaxi.hu
    10 << 8 | 0, // llvm.loongarch.lsx.vmaxi.w
    10 << 8 | 0, // llvm.loongarch.lsx.vmaxi.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vmin.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmin.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vmin.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmin.du
    10 << 8 | 1, // llvm.loongarch.lsx.vmin.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmin.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vmin.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmin.wu
    10 << 8 | 0, // llvm.loongarch.lsx.vmini.b
    10 << 8 | 0, // llvm.loongarch.lsx.vmini.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vmini.d
    10 << 8 | 0, // llvm.loongarch.lsx.vmini.du
    10 << 8 | 0, // llvm.loongarch.lsx.vmini.h
    10 << 8 | 0, // llvm.loongarch.lsx.vmini.hu
    10 << 8 | 0, // llvm.loongarch.lsx.vmini.w
    10 << 8 | 0, // llvm.loongarch.lsx.vmini.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vmod.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmod.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vmod.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmod.du
    10 << 8 | 1, // llvm.loongarch.lsx.vmod.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmod.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vmod.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmod.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vmskgez.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmskltz.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmskltz.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmskltz.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmskltz.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmsknz.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmsub.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmsub.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmsub.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmsub.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmuh.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmuh.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vmuh.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmuh.du
    10 << 8 | 1, // llvm.loongarch.lsx.vmuh.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmuh.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vmuh.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmuh.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vmul.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmul.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmul.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmul.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.d.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.h.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.q.du
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.q.du.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.w.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwev.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.d.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.d.wu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.h.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.h.bu.b
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.q.du
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.q.du.d
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.w.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vmulwod.w.hu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vneg.b
    10 << 8 | 1, // llvm.loongarch.lsx.vneg.d
    10 << 8 | 1, // llvm.loongarch.lsx.vneg.h
    10 << 8 | 1, // llvm.loongarch.lsx.vneg.w
    10 << 8 | 1, // llvm.loongarch.lsx.vnor.v
    10 << 8 | 0, // llvm.loongarch.lsx.vnori.b
    10 << 8 | 1, // llvm.loongarch.lsx.vor.v
    10 << 8 | 0, // llvm.loongarch.lsx.vori.b
    10 << 8 | 1, // llvm.loongarch.lsx.vorn.v
    10 << 8 | 1, // llvm.loongarch.lsx.vpackev.b
    10 << 8 | 1, // llvm.loongarch.lsx.vpackev.d
    10 << 8 | 1, // llvm.loongarch.lsx.vpackev.h
    10 << 8 | 1, // llvm.loongarch.lsx.vpackev.w
    10 << 8 | 1, // llvm.loongarch.lsx.vpackod.b
    10 << 8 | 1, // llvm.loongarch.lsx.vpackod.d
    10 << 8 | 1, // llvm.loongarch.lsx.vpackod.h
    10 << 8 | 1, // llvm.loongarch.lsx.vpackod.w
    10 << 8 | 1, // llvm.loongarch.lsx.vpcnt.b
    10 << 8 | 1, // llvm.loongarch.lsx.vpcnt.d
    10 << 8 | 1, // llvm.loongarch.lsx.vpcnt.h
    10 << 8 | 1, // llvm.loongarch.lsx.vpcnt.w
    10 << 8 | 11, // llvm.loongarch.lsx.vpermi.w
    10 << 8 | 1, // llvm.loongarch.lsx.vpickev.b
    10 << 8 | 1, // llvm.loongarch.lsx.vpickev.d
    10 << 8 | 1, // llvm.loongarch.lsx.vpickev.h
    10 << 8 | 1, // llvm.loongarch.lsx.vpickev.w
    10 << 8 | 1, // llvm.loongarch.lsx.vpickod.b
    10 << 8 | 1, // llvm.loongarch.lsx.vpickod.d
    10 << 8 | 1, // llvm.loongarch.lsx.vpickod.h
    10 << 8 | 1, // llvm.loongarch.lsx.vpickod.w
    10 << 8 | 0, // llvm.loongarch.lsx.vpickve2gr.b
    10 << 8 | 0, // llvm.loongarch.lsx.vpickve2gr.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vpickve2gr.d
    10 << 8 | 0, // llvm.loongarch.lsx.vpickve2gr.du
    10 << 8 | 0, // llvm.loongarch.lsx.vpickve2gr.h
    10 << 8 | 0, // llvm.loongarch.lsx.vpickve2gr.hu
    10 << 8 | 0, // llvm.loongarch.lsx.vpickve2gr.w
    10 << 8 | 0, // llvm.loongarch.lsx.vpickve2gr.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vreplgr2vr.b
    10 << 8 | 1, // llvm.loongarch.lsx.vreplgr2vr.d
    10 << 8 | 1, // llvm.loongarch.lsx.vreplgr2vr.h
    10 << 8 | 1, // llvm.loongarch.lsx.vreplgr2vr.w
    10 << 8 | 5, // llvm.loongarch.lsx.vrepli.b
    10 << 8 | 5, // llvm.loongarch.lsx.vrepli.d
    10 << 8 | 5, // llvm.loongarch.lsx.vrepli.h
    10 << 8 | 5, // llvm.loongarch.lsx.vrepli.w
    10 << 8 | 1, // llvm.loongarch.lsx.vreplve.b
    10 << 8 | 1, // llvm.loongarch.lsx.vreplve.d
    10 << 8 | 1, // llvm.loongarch.lsx.vreplve.h
    10 << 8 | 1, // llvm.loongarch.lsx.vreplve.w
    10 << 8 | 0, // llvm.loongarch.lsx.vreplvei.b
    10 << 8 | 0, // llvm.loongarch.lsx.vreplvei.d
    10 << 8 | 0, // llvm.loongarch.lsx.vreplvei.h
    10 << 8 | 0, // llvm.loongarch.lsx.vreplvei.w
    10 << 8 | 1, // llvm.loongarch.lsx.vrotr.b
    10 << 8 | 1, // llvm.loongarch.lsx.vrotr.d
    10 << 8 | 1, // llvm.loongarch.lsx.vrotr.h
    10 << 8 | 1, // llvm.loongarch.lsx.vrotr.w
    10 << 8 | 0, // llvm.loongarch.lsx.vrotri.b
    10 << 8 | 0, // llvm.loongarch.lsx.vrotri.d
    10 << 8 | 0, // llvm.loongarch.lsx.vrotri.h
    10 << 8 | 0, // llvm.loongarch.lsx.vrotri.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsadd.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsadd.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vsadd.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsadd.du
    10 << 8 | 1, // llvm.loongarch.lsx.vsadd.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsadd.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vsadd.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsadd.wu
    10 << 8 | 0, // llvm.loongarch.lsx.vsat.b
    10 << 8 | 0, // llvm.loongarch.lsx.vsat.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vsat.d
    10 << 8 | 0, // llvm.loongarch.lsx.vsat.du
    10 << 8 | 0, // llvm.loongarch.lsx.vsat.h
    10 << 8 | 0, // llvm.loongarch.lsx.vsat.hu
    10 << 8 | 0, // llvm.loongarch.lsx.vsat.w
    10 << 8 | 0, // llvm.loongarch.lsx.vsat.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vseq.b
    10 << 8 | 1, // llvm.loongarch.lsx.vseq.d
    10 << 8 | 1, // llvm.loongarch.lsx.vseq.h
    10 << 8 | 1, // llvm.loongarch.lsx.vseq.w
    10 << 8 | 0, // llvm.loongarch.lsx.vseqi.b
    10 << 8 | 0, // llvm.loongarch.lsx.vseqi.d
    10 << 8 | 0, // llvm.loongarch.lsx.vseqi.h
    10 << 8 | 0, // llvm.loongarch.lsx.vseqi.w
    10 << 8 | 1, // llvm.loongarch.lsx.vshuf.b
    10 << 8 | 1, // llvm.loongarch.lsx.vshuf.d
    10 << 8 | 1, // llvm.loongarch.lsx.vshuf.h
    10 << 8 | 1, // llvm.loongarch.lsx.vshuf.w
    10 << 8 | 0, // llvm.loongarch.lsx.vshuf4i.b
    10 << 8 | 11, // llvm.loongarch.lsx.vshuf4i.d
    10 << 8 | 0, // llvm.loongarch.lsx.vshuf4i.h
    10 << 8 | 0, // llvm.loongarch.lsx.vshuf4i.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsigncov.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsigncov.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsigncov.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsigncov.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsle.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsle.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vsle.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsle.du
    10 << 8 | 1, // llvm.loongarch.lsx.vsle.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsle.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vsle.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsle.wu
    10 << 8 | 0, // llvm.loongarch.lsx.vslei.b
    10 << 8 | 0, // llvm.loongarch.lsx.vslei.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vslei.d
    10 << 8 | 0, // llvm.loongarch.lsx.vslei.du
    10 << 8 | 0, // llvm.loongarch.lsx.vslei.h
    10 << 8 | 0, // llvm.loongarch.lsx.vslei.hu
    10 << 8 | 0, // llvm.loongarch.lsx.vslei.w
    10 << 8 | 0, // llvm.loongarch.lsx.vslei.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vsll.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsll.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsll.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsll.w
    10 << 8 | 0, // llvm.loongarch.lsx.vslli.b
    10 << 8 | 0, // llvm.loongarch.lsx.vslli.d
    10 << 8 | 0, // llvm.loongarch.lsx.vslli.h
    10 << 8 | 0, // llvm.loongarch.lsx.vslli.w
    10 << 8 | 0, // llvm.loongarch.lsx.vsllwil.d.w
    10 << 8 | 0, // llvm.loongarch.lsx.vsllwil.du.wu
    10 << 8 | 0, // llvm.loongarch.lsx.vsllwil.h.b
    10 << 8 | 0, // llvm.loongarch.lsx.vsllwil.hu.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vsllwil.w.h
    10 << 8 | 0, // llvm.loongarch.lsx.vsllwil.wu.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vslt.b
    10 << 8 | 1, // llvm.loongarch.lsx.vslt.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vslt.d
    10 << 8 | 1, // llvm.loongarch.lsx.vslt.du
    10 << 8 | 1, // llvm.loongarch.lsx.vslt.h
    10 << 8 | 1, // llvm.loongarch.lsx.vslt.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vslt.w
    10 << 8 | 1, // llvm.loongarch.lsx.vslt.wu
    10 << 8 | 0, // llvm.loongarch.lsx.vslti.b
    10 << 8 | 0, // llvm.loongarch.lsx.vslti.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vslti.d
    10 << 8 | 0, // llvm.loongarch.lsx.vslti.du
    10 << 8 | 0, // llvm.loongarch.lsx.vslti.h
    10 << 8 | 0, // llvm.loongarch.lsx.vslti.hu
    10 << 8 | 0, // llvm.loongarch.lsx.vslti.w
    10 << 8 | 0, // llvm.loongarch.lsx.vslti.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vsra.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsra.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsra.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsra.w
    10 << 8 | 0, // llvm.loongarch.lsx.vsrai.b
    10 << 8 | 0, // llvm.loongarch.lsx.vsrai.d
    10 << 8 | 0, // llvm.loongarch.lsx.vsrai.h
    10 << 8 | 0, // llvm.loongarch.lsx.vsrai.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsran.b.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsran.h.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsran.w.d
    10 << 8 | 11, // llvm.loongarch.lsx.vsrani.b.h
    10 << 8 | 11, // llvm.loongarch.lsx.vsrani.d.q
    10 << 8 | 11, // llvm.loongarch.lsx.vsrani.h.w
    10 << 8 | 11, // llvm.loongarch.lsx.vsrani.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsrar.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsrar.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsrar.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsrar.w
    10 << 8 | 0, // llvm.loongarch.lsx.vsrari.b
    10 << 8 | 0, // llvm.loongarch.lsx.vsrari.d
    10 << 8 | 0, // llvm.loongarch.lsx.vsrari.h
    10 << 8 | 0, // llvm.loongarch.lsx.vsrari.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsrarn.b.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsrarn.h.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsrarn.w.d
    10 << 8 | 11, // llvm.loongarch.lsx.vsrarni.b.h
    10 << 8 | 11, // llvm.loongarch.lsx.vsrarni.d.q
    10 << 8 | 11, // llvm.loongarch.lsx.vsrarni.h.w
    10 << 8 | 11, // llvm.loongarch.lsx.vsrarni.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsrl.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsrl.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsrl.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsrl.w
    10 << 8 | 0, // llvm.loongarch.lsx.vsrli.b
    10 << 8 | 0, // llvm.loongarch.lsx.vsrli.d
    10 << 8 | 0, // llvm.loongarch.lsx.vsrli.h
    10 << 8 | 0, // llvm.loongarch.lsx.vsrli.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsrln.b.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsrln.h.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsrln.w.d
    10 << 8 | 11, // llvm.loongarch.lsx.vsrlni.b.h
    10 << 8 | 11, // llvm.loongarch.lsx.vsrlni.d.q
    10 << 8 | 11, // llvm.loongarch.lsx.vsrlni.h.w
    10 << 8 | 11, // llvm.loongarch.lsx.vsrlni.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsrlr.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsrlr.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsrlr.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsrlr.w
    10 << 8 | 0, // llvm.loongarch.lsx.vsrlri.b
    10 << 8 | 0, // llvm.loongarch.lsx.vsrlri.d
    10 << 8 | 0, // llvm.loongarch.lsx.vsrlri.h
    10 << 8 | 0, // llvm.loongarch.lsx.vsrlri.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsrlrn.b.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsrlrn.h.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsrlrn.w.d
    10 << 8 | 11, // llvm.loongarch.lsx.vsrlrni.b.h
    10 << 8 | 11, // llvm.loongarch.lsx.vsrlrni.d.q
    10 << 8 | 11, // llvm.loongarch.lsx.vsrlrni.h.w
    10 << 8 | 11, // llvm.loongarch.lsx.vsrlrni.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssran.b.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssran.bu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssran.h.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssran.hu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssran.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssran.wu.d
    10 << 8 | 11, // llvm.loongarch.lsx.vssrani.b.h
    10 << 8 | 11, // llvm.loongarch.lsx.vssrani.bu.h
    10 << 8 | 11, // llvm.loongarch.lsx.vssrani.d.q
    10 << 8 | 11, // llvm.loongarch.lsx.vssrani.du.q
    10 << 8 | 11, // llvm.loongarch.lsx.vssrani.h.w
    10 << 8 | 11, // llvm.loongarch.lsx.vssrani.hu.w
    10 << 8 | 11, // llvm.loongarch.lsx.vssrani.w.d
    10 << 8 | 11, // llvm.loongarch.lsx.vssrani.wu.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssrarn.b.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssrarn.bu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssrarn.h.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssrarn.hu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssrarn.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssrarn.wu.d
    10 << 8 | 11, // llvm.loongarch.lsx.vssrarni.b.h
    10 << 8 | 11, // llvm.loongarch.lsx.vssrarni.bu.h
    10 << 8 | 11, // llvm.loongarch.lsx.vssrarni.d.q
    10 << 8 | 11, // llvm.loongarch.lsx.vssrarni.du.q
    10 << 8 | 11, // llvm.loongarch.lsx.vssrarni.h.w
    10 << 8 | 11, // llvm.loongarch.lsx.vssrarni.hu.w
    10 << 8 | 11, // llvm.loongarch.lsx.vssrarni.w.d
    10 << 8 | 11, // llvm.loongarch.lsx.vssrarni.wu.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssrln.b.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssrln.bu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssrln.h.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssrln.hu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssrln.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssrln.wu.d
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlni.b.h
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlni.bu.h
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlni.d.q
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlni.du.q
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlni.h.w
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlni.hu.w
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlni.w.d
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlni.wu.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssrlrn.b.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssrlrn.bu.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssrlrn.h.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssrlrn.hu.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssrlrn.w.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssrlrn.wu.d
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlrni.b.h
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlrni.bu.h
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlrni.d.q
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlrni.du.q
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlrni.h.w
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlrni.hu.w
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlrni.w.d
    10 << 8 | 11, // llvm.loongarch.lsx.vssrlrni.wu.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssub.b
    10 << 8 | 1, // llvm.loongarch.lsx.vssub.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vssub.d
    10 << 8 | 1, // llvm.loongarch.lsx.vssub.du
    10 << 8 | 1, // llvm.loongarch.lsx.vssub.h
    10 << 8 | 1, // llvm.loongarch.lsx.vssub.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vssub.w
    10 << 8 | 1, // llvm.loongarch.lsx.vssub.wu
    79 << 8 | 11, // llvm.loongarch.lsx.vst
    79 << 8 | 93, // llvm.loongarch.lsx.vstelm.b
    79 << 8 | 93, // llvm.loongarch.lsx.vstelm.d
    79 << 8 | 93, // llvm.loongarch.lsx.vstelm.h
    79 << 8 | 93, // llvm.loongarch.lsx.vstelm.w
    79 << 8 | 1, // llvm.loongarch.lsx.vstx
    10 << 8 | 1, // llvm.loongarch.lsx.vsub.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsub.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsub.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsub.q
    10 << 8 | 1, // llvm.loongarch.lsx.vsub.w
    10 << 8 | 0, // llvm.loongarch.lsx.vsubi.bu
    10 << 8 | 0, // llvm.loongarch.lsx.vsubi.du
    10 << 8 | 0, // llvm.loongarch.lsx.vsubi.hu
    10 << 8 | 0, // llvm.loongarch.lsx.vsubi.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwev.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwev.d.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwev.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwev.h.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwev.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwev.q.du
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwev.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwev.w.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwod.d.w
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwod.d.wu
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwod.h.b
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwod.h.bu
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwod.q.d
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwod.q.du
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwod.w.h
    10 << 8 | 1, // llvm.loongarch.lsx.vsubwod.w.hu
    10 << 8 | 1, // llvm.loongarch.lsx.vxor.v
    10 << 8 | 0, // llvm.loongarch.lsx.vxori.b
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.add.i32
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.add.i64
    11 << 8 | 108, // llvm.loongarch.masked.atomicrmw.max.i32
    11 << 8 | 108, // llvm.loongarch.masked.atomicrmw.max.i64
    11 << 8 | 108, // llvm.loongarch.masked.atomicrmw.min.i32
    11 << 8 | 108, // llvm.loongarch.masked.atomicrmw.min.i64
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.nand.i32
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.nand.i64
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.sub.i32
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.sub.i64
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.umax.i32
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.umax.i64
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.umin.i32
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.umin.i64
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.xchg.i32
    11 << 8 | 99, // llvm.loongarch.masked.atomicrmw.xchg.i64
    11 << 8 | 108, // llvm.loongarch.masked.cmpxchg.i32
    11 << 8 | 108, // llvm.loongarch.masked.cmpxchg.i64
    5 << 8 | 5, // llvm.loongarch.movfcsr2gr
    5 << 8 | 5, // llvm.loongarch.movgr2fcsr
    5 << 8 | 5, // llvm.loongarch.syscall
    5 << 8 | 1, // llvm.mips.absq.s.ph
    5 << 8 | 1, // llvm.mips.absq.s.qb
    5 << 8 | 1, // llvm.mips.absq.s.w
    10 << 8 | 1, // llvm.mips.add.a.b
    10 << 8 | 1, // llvm.mips.add.a.d
    10 << 8 | 1, // llvm.mips.add.a.h
    10 << 8 | 1, // llvm.mips.add.a.w
    10 << 8 | 1, // llvm.mips.addq.ph
    10 << 8 | 1, // llvm.mips.addq.s.ph
    5 << 8 | 1, // llvm.mips.addq.s.w
    10 << 8 | 1, // llvm.mips.addqh.ph
    10 << 8 | 1, // llvm.mips.addqh.r.ph
    10 << 8 | 1, // llvm.mips.addqh.r.w
    10 << 8 | 1, // llvm.mips.addqh.w
    10 << 8 | 1, // llvm.mips.adds.a.b
    10 << 8 | 1, // llvm.mips.adds.a.d
    10 << 8 | 1, // llvm.mips.adds.a.h
    10 << 8 | 1, // llvm.mips.adds.a.w
    10 << 8 | 1, // llvm.mips.adds.s.b
    10 << 8 | 1, // llvm.mips.adds.s.d
    10 << 8 | 1, // llvm.mips.adds.s.h
    10 << 8 | 1, // llvm.mips.adds.s.w
    10 << 8 | 1, // llvm.mips.adds.u.b
    10 << 8 | 1, // llvm.mips.adds.u.d
    10 << 8 | 1, // llvm.mips.adds.u.h
    10 << 8 | 1, // llvm.mips.adds.u.w
    5 << 8 | 1, // llvm.mips.addsc
    5 << 8 | 1, // llvm.mips.addu.ph
    10 << 8 | 1, // llvm.mips.addu.qb
    5 << 8 | 1, // llvm.mips.addu.s.ph
    10 << 8 | 1, // llvm.mips.addu.s.qb
    10 << 8 | 1, // llvm.mips.adduh.qb
    10 << 8 | 1, // llvm.mips.adduh.r.qb
    10 << 8 | 1, // llvm.mips.addv.b
    10 << 8 | 1, // llvm.mips.addv.d
    10 << 8 | 1, // llvm.mips.addv.h
    10 << 8 | 1, // llvm.mips.addv.w
    10 << 8 | 0, // llvm.mips.addvi.b
    10 << 8 | 0, // llvm.mips.addvi.d
    10 << 8 | 0, // llvm.mips.addvi.h
    10 << 8 | 0, // llvm.mips.addvi.w
    5 << 8 | 1, // llvm.mips.addwc
    10 << 8 | 1, // llvm.mips.and.v
    10 << 8 | 0, // llvm.mips.andi.b
    10 << 8 | 11, // llvm.mips.append
    10 << 8 | 1, // llvm.mips.asub.s.b
    10 << 8 | 1, // llvm.mips.asub.s.d
    10 << 8 | 1, // llvm.mips.asub.s.h
    10 << 8 | 1, // llvm.mips.asub.s.w
    10 << 8 | 1, // llvm.mips.asub.u.b
    10 << 8 | 1, // llvm.mips.asub.u.d
    10 << 8 | 1, // llvm.mips.asub.u.h
    10 << 8 | 1, // llvm.mips.asub.u.w
    10 << 8 | 1, // llvm.mips.ave.s.b
    10 << 8 | 1, // llvm.mips.ave.s.d
    10 << 8 | 1, // llvm.mips.ave.s.h
    10 << 8 | 1, // llvm.mips.ave.s.w
    10 << 8 | 1, // llvm.mips.ave.u.b
    10 << 8 | 1, // llvm.mips.ave.u.d
    10 << 8 | 1, // llvm.mips.ave.u.h
    10 << 8 | 1, // llvm.mips.ave.u.w
    10 << 8 | 1, // llvm.mips.aver.s.b
    10 << 8 | 1, // llvm.mips.aver.s.d
    10 << 8 | 1, // llvm.mips.aver.s.h
    10 << 8 | 1, // llvm.mips.aver.s.w
    10 << 8 | 1, // llvm.mips.aver.u.b
    10 << 8 | 1, // llvm.mips.aver.u.d
    10 << 8 | 1, // llvm.mips.aver.u.h
    10 << 8 | 1, // llvm.mips.aver.u.w
    10 << 8 | 11, // llvm.mips.balign
    10 << 8 | 1, // llvm.mips.bclr.b
    10 << 8 | 1, // llvm.mips.bclr.d
    10 << 8 | 1, // llvm.mips.bclr.h
    10 << 8 | 1, // llvm.mips.bclr.w
    10 << 8 | 0, // llvm.mips.bclri.b
    10 << 8 | 0, // llvm.mips.bclri.d
    10 << 8 | 0, // llvm.mips.bclri.h
    10 << 8 | 0, // llvm.mips.bclri.w
    10 << 8 | 1, // llvm.mips.binsl.b
    10 << 8 | 1, // llvm.mips.binsl.d
    10 << 8 | 1, // llvm.mips.binsl.h
    10 << 8 | 1, // llvm.mips.binsl.w
    10 << 8 | 11, // llvm.mips.binsli.b
    10 << 8 | 11, // llvm.mips.binsli.d
    10 << 8 | 11, // llvm.mips.binsli.h
    10 << 8 | 11, // llvm.mips.binsli.w
    10 << 8 | 1, // llvm.mips.binsr.b
    10 << 8 | 1, // llvm.mips.binsr.d
    10 << 8 | 1, // llvm.mips.binsr.h
    10 << 8 | 1, // llvm.mips.binsr.w
    10 << 8 | 11, // llvm.mips.binsri.b
    10 << 8 | 11, // llvm.mips.binsri.d
    10 << 8 | 11, // llvm.mips.binsri.h
    10 << 8 | 11, // llvm.mips.binsri.w
    10 << 8 | 1, // llvm.mips.bitrev
    10 << 8 | 1, // llvm.mips.bmnz.v
    10 << 8 | 11, // llvm.mips.bmnzi.b
    10 << 8 | 1, // llvm.mips.bmz.v
    10 << 8 | 11, // llvm.mips.bmzi.b
    10 << 8 | 1, // llvm.mips.bneg.b
    10 << 8 | 1, // llvm.mips.bneg.d
    10 << 8 | 1, // llvm.mips.bneg.h
    10 << 8 | 1, // llvm.mips.bneg.w
    10 << 8 | 0, // llvm.mips.bnegi.b
    10 << 8 | 0, // llvm.mips.bnegi.d
    10 << 8 | 0, // llvm.mips.bnegi.h
    10 << 8 | 0, // llvm.mips.bnegi.w
    10 << 8 | 1, // llvm.mips.bnz.b
    10 << 8 | 1, // llvm.mips.bnz.d
    10 << 8 | 1, // llvm.mips.bnz.h
    10 << 8 | 1, // llvm.mips.bnz.v
    10 << 8 | 1, // llvm.mips.bnz.w
    23 << 8 | 1, // llvm.mips.bposge32
    10 << 8 | 1, // llvm.mips.bsel.v
    10 << 8 | 11, // llvm.mips.bseli.b
    10 << 8 | 1, // llvm.mips.bset.b
    10 << 8 | 1, // llvm.mips.bset.d
    10 << 8 | 1, // llvm.mips.bset.h
    10 << 8 | 1, // llvm.mips.bset.w
    10 << 8 | 0, // llvm.mips.bseti.b
    10 << 8 | 0, // llvm.mips.bseti.d
    10 << 8 | 0, // llvm.mips.bseti.h
    10 << 8 | 0, // llvm.mips.bseti.w
    10 << 8 | 1, // llvm.mips.bz.b
    10 << 8 | 1, // llvm.mips.bz.d
    10 << 8 | 1, // llvm.mips.bz.h
    10 << 8 | 1, // llvm.mips.bz.v
    10 << 8 | 1, // llvm.mips.bz.w
    10 << 8 | 1, // llvm.mips.ceq.b
    10 << 8 | 1, // llvm.mips.ceq.d
    10 << 8 | 1, // llvm.mips.ceq.h
    10 << 8 | 1, // llvm.mips.ceq.w
    10 << 8 | 0, // llvm.mips.ceqi.b
    10 << 8 | 0, // llvm.mips.ceqi.d
    10 << 8 | 0, // llvm.mips.ceqi.h
    10 << 8 | 0, // llvm.mips.ceqi.w
    5 << 8 | 5, // llvm.mips.cfcmsa
    10 << 8 | 1, // llvm.mips.cle.s.b
    10 << 8 | 1, // llvm.mips.cle.s.d
    10 << 8 | 1, // llvm.mips.cle.s.h
    10 << 8 | 1, // llvm.mips.cle.s.w
    10 << 8 | 1, // llvm.mips.cle.u.b
    10 << 8 | 1, // llvm.mips.cle.u.d
    10 << 8 | 1, // llvm.mips.cle.u.h
    10 << 8 | 1, // llvm.mips.cle.u.w
    10 << 8 | 0, // llvm.mips.clei.s.b
    10 << 8 | 0, // llvm.mips.clei.s.d
    10 << 8 | 0, // llvm.mips.clei.s.h
    10 << 8 | 0, // llvm.mips.clei.s.w
    10 << 8 | 0, // llvm.mips.clei.u.b
    10 << 8 | 0, // llvm.mips.clei.u.d
    10 << 8 | 0, // llvm.mips.clei.u.h
    10 << 8 | 0, // llvm.mips.clei.u.w
    10 << 8 | 1, // llvm.mips.clt.s.b
    10 << 8 | 1, // llvm.mips.clt.s.d
    10 << 8 | 1, // llvm.mips.clt.s.h
    10 << 8 | 1, // llvm.mips.clt.s.w
    10 << 8 | 1, // llvm.mips.clt.u.b
    10 << 8 | 1, // llvm.mips.clt.u.d
    10 << 8 | 1, // llvm.mips.clt.u.h
    10 << 8 | 1, // llvm.mips.clt.u.w
    10 << 8 | 0, // llvm.mips.clti.s.b
    10 << 8 | 0, // llvm.mips.clti.s.d
    10 << 8 | 0, // llvm.mips.clti.s.h
    10 << 8 | 0, // llvm.mips.clti.s.w
    10 << 8 | 0, // llvm.mips.clti.u.b
    10 << 8 | 0, // llvm.mips.clti.u.d
    10 << 8 | 0, // llvm.mips.clti.u.h
    10 << 8 | 0, // llvm.mips.clti.u.w
    5 << 8 | 1, // llvm.mips.cmp.eq.ph
    5 << 8 | 1, // llvm.mips.cmp.le.ph
    5 << 8 | 1, // llvm.mips.cmp.lt.ph
    5 << 8 | 1, // llvm.mips.cmpgdu.eq.qb
    5 << 8 | 1, // llvm.mips.cmpgdu.le.qb
    5 << 8 | 1, // llvm.mips.cmpgdu.lt.qb
    5 << 8 | 1, // llvm.mips.cmpgu.eq.qb
    5 << 8 | 1, // llvm.mips.cmpgu.le.qb
    5 << 8 | 1, // llvm.mips.cmpgu.lt.qb
    5 << 8 | 1, // llvm.mips.cmpu.eq.qb
    5 << 8 | 1, // llvm.mips.cmpu.le.qb
    5 << 8 | 1, // llvm.mips.cmpu.lt.qb
    10 << 8 | 1, // llvm.mips.copy.s.b
    10 << 8 | 1, // llvm.mips.copy.s.d
    10 << 8 | 1, // llvm.mips.copy.s.h
    10 << 8 | 1, // llvm.mips.copy.s.w
    10 << 8 | 1, // llvm.mips.copy.u.b
    10 << 8 | 1, // llvm.mips.copy.u.d
    10 << 8 | 1, // llvm.mips.copy.u.h
    10 << 8 | 1, // llvm.mips.copy.u.w
    5 << 8 | 5, // llvm.mips.ctcmsa
    10 << 8 | 1, // llvm.mips.div.s.b
    10 << 8 | 1, // llvm.mips.div.s.d
    10 << 8 | 1, // llvm.mips.div.s.h
    10 << 8 | 1, // llvm.mips.div.s.w
    10 << 8 | 1, // llvm.mips.div.u.b
    10 << 8 | 1, // llvm.mips.div.u.d
    10 << 8 | 1, // llvm.mips.div.u.h
    10 << 8 | 1, // llvm.mips.div.u.w
    10 << 8 | 1, // llvm.mips.dlsa
    10 << 8 | 1, // llvm.mips.dotp.s.d
    10 << 8 | 1, // llvm.mips.dotp.s.h
    10 << 8 | 1, // llvm.mips.dotp.s.w
    10 << 8 | 1, // llvm.mips.dotp.u.d
    10 << 8 | 1, // llvm.mips.dotp.u.h
    10 << 8 | 1, // llvm.mips.dotp.u.w
    10 << 8 | 1, // llvm.mips.dpa.w.ph
    10 << 8 | 1, // llvm.mips.dpadd.s.d
    10 << 8 | 1, // llvm.mips.dpadd.s.h
    10 << 8 | 1, // llvm.mips.dpadd.s.w
    10 << 8 | 1, // llvm.mips.dpadd.u.d
    10 << 8 | 1, // llvm.mips.dpadd.u.h
    10 << 8 | 1, // llvm.mips.dpadd.u.w
    5 << 8 | 1, // llvm.mips.dpaq.s.w.ph
    5 << 8 | 1, // llvm.mips.dpaq.sa.l.w
    5 << 8 | 1, // llvm.mips.dpaqx.s.w.ph
    5 << 8 | 1, // llvm.mips.dpaqx.sa.w.ph
    10 << 8 | 1, // llvm.mips.dpau.h.qbl
    10 << 8 | 1, // llvm.mips.dpau.h.qbr
    10 << 8 | 1, // llvm.mips.dpax.w.ph
    10 << 8 | 1, // llvm.mips.dps.w.ph
    5 << 8 | 1, // llvm.mips.dpsq.s.w.ph
    5 << 8 | 1, // llvm.mips.dpsq.sa.l.w
    5 << 8 | 1, // llvm.mips.dpsqx.s.w.ph
    5 << 8 | 1, // llvm.mips.dpsqx.sa.w.ph
    10 << 8 | 1, // llvm.mips.dpsu.h.qbl
    10 << 8 | 1, // llvm.mips.dpsu.h.qbr
    10 << 8 | 1, // llvm.mips.dpsub.s.d
    10 << 8 | 1, // llvm.mips.dpsub.s.h
    10 << 8 | 1, // llvm.mips.dpsub.s.w
    10 << 8 | 1, // llvm.mips.dpsub.u.d
    10 << 8 | 1, // llvm.mips.dpsub.u.h
    10 << 8 | 1, // llvm.mips.dpsub.u.w
    10 << 8 | 1, // llvm.mips.dpsx.w.ph
    5 << 8 | 1, // llvm.mips.extp
    5 << 8 | 1, // llvm.mips.extpdp
    5 << 8 | 1, // llvm.mips.extr.r.w
    5 << 8 | 1, // llvm.mips.extr.rs.w
    5 << 8 | 1, // llvm.mips.extr.s.h
    5 << 8 | 1, // llvm.mips.extr.w
    10 << 8 | 1, // llvm.mips.fadd.d
    10 << 8 | 1, // llvm.mips.fadd.w
    10 << 8 | 1, // llvm.mips.fcaf.d
    10 << 8 | 1, // llvm.mips.fcaf.w
    10 << 8 | 1, // llvm.mips.fceq.d
    10 << 8 | 1, // llvm.mips.fceq.w
    10 << 8 | 1, // llvm.mips.fclass.d
    10 << 8 | 1, // llvm.mips.fclass.w
    10 << 8 | 1, // llvm.mips.fcle.d
    10 << 8 | 1, // llvm.mips.fcle.w
    10 << 8 | 1, // llvm.mips.fclt.d
    10 << 8 | 1, // llvm.mips.fclt.w
    10 << 8 | 1, // llvm.mips.fcne.d
    10 << 8 | 1, // llvm.mips.fcne.w
    10 << 8 | 1, // llvm.mips.fcor.d
    10 << 8 | 1, // llvm.mips.fcor.w
    10 << 8 | 1, // llvm.mips.fcueq.d
    10 << 8 | 1, // llvm.mips.fcueq.w
    10 << 8 | 1, // llvm.mips.fcule.d
    10 << 8 | 1, // llvm.mips.fcule.w
    10 << 8 | 1, // llvm.mips.fcult.d
    10 << 8 | 1, // llvm.mips.fcult.w
    10 << 8 | 1, // llvm.mips.fcun.d
    10 << 8 | 1, // llvm.mips.fcun.w
    10 << 8 | 1, // llvm.mips.fcune.d
    10 << 8 | 1, // llvm.mips.fcune.w
    10 << 8 | 1, // llvm.mips.fdiv.d
    10 << 8 | 1, // llvm.mips.fdiv.w
    10 << 8 | 1, // llvm.mips.fexdo.h
    10 << 8 | 1, // llvm.mips.fexdo.w
    10 << 8 | 1, // llvm.mips.fexp2.d
    10 << 8 | 1, // llvm.mips.fexp2.w
    10 << 8 | 1, // llvm.mips.fexupl.d
    10 << 8 | 1, // llvm.mips.fexupl.w
    10 << 8 | 1, // llvm.mips.fexupr.d
    10 << 8 | 1, // llvm.mips.fexupr.w
    10 << 8 | 1, // llvm.mips.ffint.s.d
    10 << 8 | 1, // llvm.mips.ffint.s.w
    10 << 8 | 1, // llvm.mips.ffint.u.d
    10 << 8 | 1, // llvm.mips.ffint.u.w
    10 << 8 | 1, // llvm.mips.ffql.d
    10 << 8 | 1, // llvm.mips.ffql.w
    10 << 8 | 1, // llvm.mips.ffqr.d
    10 << 8 | 1, // llvm.mips.ffqr.w
    10 << 8 | 1, // llvm.mips.fill.b
    10 << 8 | 1, // llvm.mips.fill.d
    10 << 8 | 1, // llvm.mips.fill.h
    10 << 8 | 1, // llvm.mips.fill.w
    10 << 8 | 1, // llvm.mips.flog2.d
    10 << 8 | 1, // llvm.mips.flog2.w
    10 << 8 | 1, // llvm.mips.fmadd.d
    10 << 8 | 1, // llvm.mips.fmadd.w
    10 << 8 | 1, // llvm.mips.fmax.a.d
    10 << 8 | 1, // llvm.mips.fmax.a.w
    10 << 8 | 1, // llvm.mips.fmax.d
    10 << 8 | 1, // llvm.mips.fmax.w
    10 << 8 | 1, // llvm.mips.fmin.a.d
    10 << 8 | 1, // llvm.mips.fmin.a.w
    10 << 8 | 1, // llvm.mips.fmin.d
    10 << 8 | 1, // llvm.mips.fmin.w
    10 << 8 | 1, // llvm.mips.fmsub.d
    10 << 8 | 1, // llvm.mips.fmsub.w
    10 << 8 | 1, // llvm.mips.fmul.d
    10 << 8 | 1, // llvm.mips.fmul.w
    10 << 8 | 1, // llvm.mips.frcp.d
    10 << 8 | 1, // llvm.mips.frcp.w
    10 << 8 | 1, // llvm.mips.frint.d
    10 << 8 | 1, // llvm.mips.frint.w
    10 << 8 | 1, // llvm.mips.frsqrt.d
    10 << 8 | 1, // llvm.mips.frsqrt.w
    10 << 8 | 1, // llvm.mips.fsaf.d
    10 << 8 | 1, // llvm.mips.fsaf.w
    10 << 8 | 1, // llvm.mips.fseq.d
    10 << 8 | 1, // llvm.mips.fseq.w
    10 << 8 | 1, // llvm.mips.fsle.d
    10 << 8 | 1, // llvm.mips.fsle.w
    10 << 8 | 1, // llvm.mips.fslt.d
    10 << 8 | 1, // llvm.mips.fslt.w
    10 << 8 | 1, // llvm.mips.fsne.d
    10 << 8 | 1, // llvm.mips.fsne.w
    10 << 8 | 1, // llvm.mips.fsor.d
    10 << 8 | 1, // llvm.mips.fsor.w
    10 << 8 | 1, // llvm.mips.fsqrt.d
    10 << 8 | 1, // llvm.mips.fsqrt.w
    10 << 8 | 1, // llvm.mips.fsub.d
    10 << 8 | 1, // llvm.mips.fsub.w
    10 << 8 | 1, // llvm.mips.fsueq.d
    10 << 8 | 1, // llvm.mips.fsueq.w
    10 << 8 | 1, // llvm.mips.fsule.d
    10 << 8 | 1, // llvm.mips.fsule.w
    10 << 8 | 1, // llvm.mips.fsult.d
    10 << 8 | 1, // llvm.mips.fsult.w
    10 << 8 | 1, // llvm.mips.fsun.d
    10 << 8 | 1, // llvm.mips.fsun.w
    10 << 8 | 1, // llvm.mips.fsune.d
    10 << 8 | 1, // llvm.mips.fsune.w
    10 << 8 | 1, // llvm.mips.ftint.s.d
    10 << 8 | 1, // llvm.mips.ftint.s.w
    10 << 8 | 1, // llvm.mips.ftint.u.d
    10 << 8 | 1, // llvm.mips.ftint.u.w
    10 << 8 | 1, // llvm.mips.ftq.h
    10 << 8 | 1, // llvm.mips.ftq.w
    10 << 8 | 1, // llvm.mips.ftrunc.s.d
    10 << 8 | 1, // llvm.mips.ftrunc.s.w
    10 << 8 | 1, // llvm.mips.ftrunc.u.d
    10 << 8 | 1, // llvm.mips.ftrunc.u.w
    10 << 8 | 1, // llvm.mips.hadd.s.d
    10 << 8 | 1, // llvm.mips.hadd.s.h
    10 << 8 | 1, // llvm.mips.hadd.s.w
    10 << 8 | 1, // llvm.mips.hadd.u.d
    10 << 8 | 1, // llvm.mips.hadd.u.h
    10 << 8 | 1, // llvm.mips.hadd.u.w
    10 << 8 | 1, // llvm.mips.hsub.s.d
    10 << 8 | 1, // llvm.mips.hsub.s.h
    10 << 8 | 1, // llvm.mips.hsub.s.w
    10 << 8 | 1, // llvm.mips.hsub.u.d
    10 << 8 | 1, // llvm.mips.hsub.u.h
    10 << 8 | 1, // llvm.mips.hsub.u.w
    10 << 8 | 1, // llvm.mips.ilvev.b
    10 << 8 | 1, // llvm.mips.ilvev.d
    10 << 8 | 1, // llvm.mips.ilvev.h
    10 << 8 | 1, // llvm.mips.ilvev.w
    10 << 8 | 1, // llvm.mips.ilvl.b
    10 << 8 | 1, // llvm.mips.ilvl.d
    10 << 8 | 1, // llvm.mips.ilvl.h
    10 << 8 | 1, // llvm.mips.ilvl.w
    10 << 8 | 1, // llvm.mips.ilvod.b
    10 << 8 | 1, // llvm.mips.ilvod.d
    10 << 8 | 1, // llvm.mips.ilvod.h
    10 << 8 | 1, // llvm.mips.ilvod.w
    10 << 8 | 1, // llvm.mips.ilvr.b
    10 << 8 | 1, // llvm.mips.ilvr.d
    10 << 8 | 1, // llvm.mips.ilvr.h
    10 << 8 | 1, // llvm.mips.ilvr.w
    10 << 8 | 1, // llvm.mips.insert.b
    10 << 8 | 1, // llvm.mips.insert.d
    10 << 8 | 1, // llvm.mips.insert.h
    10 << 8 | 1, // llvm.mips.insert.w
    80 << 8 | 1, // llvm.mips.insv
    10 << 8 | 0, // llvm.mips.insve.b
    10 << 8 | 0, // llvm.mips.insve.d
    10 << 8 | 0, // llvm.mips.insve.h
    10 << 8 | 0, // llvm.mips.insve.w
    2 << 8 | 1, // llvm.mips.lbux
    2 << 8 | 1, // llvm.mips.ld.b
    2 << 8 | 1, // llvm.mips.ld.d
    2 << 8 | 1, // llvm.mips.ld.h
    2 << 8 | 1, // llvm.mips.ld.w
    10 << 8 | 5, // llvm.mips.ldi.b
    10 << 8 | 5, // llvm.mips.ldi.d
    10 << 8 | 5, // llvm.mips.ldi.h
    10 << 8 | 5, // llvm.mips.ldi.w
    2 << 8 | 1, // llvm.mips.ldr.d
    2 << 8 | 1, // llvm.mips.ldr.w
    2 << 8 | 1, // llvm.mips.lhx
    10 << 8 | 1, // llvm.mips.lsa
    2 << 8 | 1, // llvm.mips.lwx
    10 << 8 | 1, // llvm.mips.madd
    10 << 8 | 1, // llvm.mips.madd.q.h
    10 << 8 | 1, // llvm.mips.madd.q.w
    10 << 8 | 1, // llvm.mips.maddr.q.h
    10 << 8 | 1, // llvm.mips.maddr.q.w
    10 << 8 | 1, // llvm.mips.maddu
    10 << 8 | 1, // llvm.mips.maddv.b
    10 << 8 | 1, // llvm.mips.maddv.d
    10 << 8 | 1, // llvm.mips.maddv.h
    10 << 8 | 1, // llvm.mips.maddv.w
    5 << 8 | 1, // llvm.mips.maq.s.w.phl
    5 << 8 | 1, // llvm.mips.maq.s.w.phr
    5 << 8 | 1, // llvm.mips.maq.sa.w.phl
    5 << 8 | 1, // llvm.mips.maq.sa.w.phr
    10 << 8 | 1, // llvm.mips.max.a.b
    10 << 8 | 1, // llvm.mips.max.a.d
    10 << 8 | 1, // llvm.mips.max.a.h
    10 << 8 | 1, // llvm.mips.max.a.w
    10 << 8 | 1, // llvm.mips.max.s.b
    10 << 8 | 1, // llvm.mips.max.s.d
    10 << 8 | 1, // llvm.mips.max.s.h
    10 << 8 | 1, // llvm.mips.max.s.w
    10 << 8 | 1, // llvm.mips.max.u.b
    10 << 8 | 1, // llvm.mips.max.u.d
    10 << 8 | 1, // llvm.mips.max.u.h
    10 << 8 | 1, // llvm.mips.max.u.w
    10 << 8 | 0, // llvm.mips.maxi.s.b
    10 << 8 | 0, // llvm.mips.maxi.s.d
    10 << 8 | 0, // llvm.mips.maxi.s.h
    10 << 8 | 0, // llvm.mips.maxi.s.w
    10 << 8 | 0, // llvm.mips.maxi.u.b
    10 << 8 | 0, // llvm.mips.maxi.u.d
    10 << 8 | 0, // llvm.mips.maxi.u.h
    10 << 8 | 0, // llvm.mips.maxi.u.w
    10 << 8 | 1, // llvm.mips.min.a.b
    10 << 8 | 1, // llvm.mips.min.a.d
    10 << 8 | 1, // llvm.mips.min.a.h
    10 << 8 | 1, // llvm.mips.min.a.w
    10 << 8 | 1, // llvm.mips.min.s.b
    10 << 8 | 1, // llvm.mips.min.s.d
    10 << 8 | 1, // llvm.mips.min.s.h
    10 << 8 | 1, // llvm.mips.min.s.w
    10 << 8 | 1, // llvm.mips.min.u.b
    10 << 8 | 1, // llvm.mips.min.u.d
    10 << 8 | 1, // llvm.mips.min.u.h
    10 << 8 | 1, // llvm.mips.min.u.w
    10 << 8 | 0, // llvm.mips.mini.s.b
    10 << 8 | 0, // llvm.mips.mini.s.d
    10 << 8 | 0, // llvm.mips.mini.s.h
    10 << 8 | 0, // llvm.mips.mini.s.w
    10 << 8 | 0, // llvm.mips.mini.u.b
    10 << 8 | 0, // llvm.mips.mini.u.d
    10 << 8 | 0, // llvm.mips.mini.u.h
    10 << 8 | 0, // llvm.mips.mini.u.w
    10 << 8 | 1, // llvm.mips.mod.s.b
    10 << 8 | 1, // llvm.mips.mod.s.d
    10 << 8 | 1, // llvm.mips.mod.s.h
    10 << 8 | 1, // llvm.mips.mod.s.w
    10 << 8 | 1, // llvm.mips.mod.u.b
    10 << 8 | 1, // llvm.mips.mod.u.d
    10 << 8 | 1, // llvm.mips.mod.u.h
    10 << 8 | 1, // llvm.mips.mod.u.w
    10 << 8 | 1, // llvm.mips.modsub
    10 << 8 | 1, // llvm.mips.move.v
    10 << 8 | 1, // llvm.mips.msub
    10 << 8 | 1, // llvm.mips.msub.q.h
    10 << 8 | 1, // llvm.mips.msub.q.w
    10 << 8 | 1, // llvm.mips.msubr.q.h
    10 << 8 | 1, // llvm.mips.msubr.q.w
    10 << 8 | 1, // llvm.mips.msubu
    10 << 8 | 1, // llvm.mips.msubv.b
    10 << 8 | 1, // llvm.mips.msubv.d
    10 << 8 | 1, // llvm.mips.msubv.h
    10 << 8 | 1, // llvm.mips.msubv.w
    5 << 8 | 1, // llvm.mips.mthlip
    5 << 8 | 1, // llvm.mips.mul.ph
    10 << 8 | 1, // llvm.mips.mul.q.h
    10 << 8 | 1, // llvm.mips.mul.q.w
    5 << 8 | 1, // llvm.mips.mul.s.ph
    5 << 8 | 1, // llvm.mips.muleq.s.w.phl
    5 << 8 | 1, // llvm.mips.muleq.s.w.phr
    5 << 8 | 1, // llvm.mips.muleu.s.ph.qbl
    5 << 8 | 1, // llvm.mips.muleu.s.ph.qbr
    5 << 8 | 1, // llvm.mips.mulq.rs.ph
    5 << 8 | 1, // llvm.mips.mulq.rs.w
    5 << 8 | 1, // llvm.mips.mulq.s.ph
    5 << 8 | 1, // llvm.mips.mulq.s.w
    10 << 8 | 1, // llvm.mips.mulr.q.h
    10 << 8 | 1, // llvm.mips.mulr.q.w
    10 << 8 | 1, // llvm.mips.mulsa.w.ph
    5 << 8 | 1, // llvm.mips.mulsaq.s.w.ph
    10 << 8 | 1, // llvm.mips.mult
    10 << 8 | 1, // llvm.mips.multu
    10 << 8 | 1, // llvm.mips.mulv.b
    10 << 8 | 1, // llvm.mips.mulv.d
    10 << 8 | 1, // llvm.mips.mulv.h
    10 << 8 | 1, // llvm.mips.mulv.w
    10 << 8 | 1, // llvm.mips.nloc.b
    10 << 8 | 1, // llvm.mips.nloc.d
    10 << 8 | 1, // llvm.mips.nloc.h
    10 << 8 | 1, // llvm.mips.nloc.w
    10 << 8 | 1, // llvm.mips.nlzc.b
    10 << 8 | 1, // llvm.mips.nlzc.d
    10 << 8 | 1, // llvm.mips.nlzc.h
    10 << 8 | 1, // llvm.mips.nlzc.w
    10 << 8 | 1, // llvm.mips.nor.v
    10 << 8 | 0, // llvm.mips.nori.b
    10 << 8 | 1, // llvm.mips.or.v
    10 << 8 | 0, // llvm.mips.ori.b
    10 << 8 | 1, // llvm.mips.packrl.ph
    10 << 8 | 1, // llvm.mips.pckev.b
    10 << 8 | 1, // llvm.mips.pckev.d
    10 << 8 | 1, // llvm.mips.pckev.h
    10 << 8 | 1, // llvm.mips.pckev.w
    10 << 8 | 1, // llvm.mips.pckod.b
    10 << 8 | 1, // llvm.mips.pckod.d
    10 << 8 | 1, // llvm.mips.pckod.h
    10 << 8 | 1, // llvm.mips.pckod.w
    10 << 8 | 1, // llvm.mips.pcnt.b
    10 << 8 | 1, // llvm.mips.pcnt.d
    10 << 8 | 1, // llvm.mips.pcnt.h
    10 << 8 | 1, // llvm.mips.pcnt.w
    23 << 8 | 1, // llvm.mips.pick.ph
    23 << 8 | 1, // llvm.mips.pick.qb
    10 << 8 | 1, // llvm.mips.preceq.w.phl
    10 << 8 | 1, // llvm.mips.preceq.w.phr
    10 << 8 | 1, // llvm.mips.precequ.ph.qbl
    10 << 8 | 1, // llvm.mips.precequ.ph.qbla
    10 << 8 | 1, // llvm.mips.precequ.ph.qbr
    10 << 8 | 1, // llvm.mips.precequ.ph.qbra
    10 << 8 | 1, // llvm.mips.preceu.ph.qbl
    10 << 8 | 1, // llvm.mips.preceu.ph.qbla
    10 << 8 | 1, // llvm.mips.preceu.ph.qbr
    10 << 8 | 1, // llvm.mips.preceu.ph.qbra
    5 << 8 | 1, // llvm.mips.precr.qb.ph
    10 << 8 | 11, // llvm.mips.precr.sra.ph.w
    10 << 8 | 11, // llvm.mips.precr.sra.r.ph.w
    10 << 8 | 1, // llvm.mips.precrq.ph.w
    10 << 8 | 1, // llvm.mips.precrq.qb.ph
    5 << 8 | 1, // llvm.mips.precrq.rs.ph.w
    5 << 8 | 1, // llvm.mips.precrqu.s.qb.ph
    10 << 8 | 11, // llvm.mips.prepend
    10 << 8 | 1, // llvm.mips.raddu.w.qb
    23 << 8 | 5, // llvm.mips.rddsp
    10 << 8 | 1, // llvm.mips.repl.ph
    10 << 8 | 1, // llvm.mips.repl.qb
    10 << 8 | 0, // llvm.mips.sat.s.b
    10 << 8 | 0, // llvm.mips.sat.s.d
    10 << 8 | 0, // llvm.mips.sat.s.h
    10 << 8 | 0, // llvm.mips.sat.s.w
    10 << 8 | 0, // llvm.mips.sat.u.b
    10 << 8 | 0, // llvm.mips.sat.u.d
    10 << 8 | 0, // llvm.mips.sat.u.h
    10 << 8 | 0, // llvm.mips.sat.u.w
    10 << 8 | 0, // llvm.mips.shf.b
    10 << 8 | 0, // llvm.mips.shf.h
    10 << 8 | 0, // llvm.mips.shf.w
    10 << 8 | 1, // llvm.mips.shilo
    5 << 8 | 1, // llvm.mips.shll.ph
    5 << 8 | 1, // llvm.mips.shll.qb
    5 << 8 | 1, // llvm.mips.shll.s.ph
    5 << 8 | 1, // llvm.mips.shll.s.w
    10 << 8 | 1, // llvm.mips.shra.ph
    10 << 8 | 1, // llvm.mips.shra.qb
    10 << 8 | 1, // llvm.mips.shra.r.ph
    10 << 8 | 1, // llvm.mips.shra.r.qb
    10 << 8 | 1, // llvm.mips.shra.r.w
    10 << 8 | 1, // llvm.mips.shrl.ph
    10 << 8 | 1, // llvm.mips.shrl.qb
    10 << 8 | 1, // llvm.mips.sld.b
    10 << 8 | 1, // llvm.mips.sld.d
    10 << 8 | 1, // llvm.mips.sld.h
    10 << 8 | 1, // llvm.mips.sld.w
    10 << 8 | 11, // llvm.mips.sldi.b
    10 << 8 | 11, // llvm.mips.sldi.d
    10 << 8 | 11, // llvm.mips.sldi.h
    10 << 8 | 11, // llvm.mips.sldi.w
    10 << 8 | 1, // llvm.mips.sll.b
    10 << 8 | 1, // llvm.mips.sll.d
    10 << 8 | 1, // llvm.mips.sll.h
    10 << 8 | 1, // llvm.mips.sll.w
    10 << 8 | 0, // llvm.mips.slli.b
    10 << 8 | 0, // llvm.mips.slli.d
    10 << 8 | 0, // llvm.mips.slli.h
    10 << 8 | 0, // llvm.mips.slli.w
    10 << 8 | 1, // llvm.mips.splat.b
    10 << 8 | 1, // llvm.mips.splat.d
    10 << 8 | 1, // llvm.mips.splat.h
    10 << 8 | 1, // llvm.mips.splat.w
    10 << 8 | 0, // llvm.mips.splati.b
    10 << 8 | 0, // llvm.mips.splati.d
    10 << 8 | 0, // llvm.mips.splati.h
    10 << 8 | 0, // llvm.mips.splati.w
    10 << 8 | 1, // llvm.mips.sra.b
    10 << 8 | 1, // llvm.mips.sra.d
    10 << 8 | 1, // llvm.mips.sra.h
    10 << 8 | 1, // llvm.mips.sra.w
    10 << 8 | 0, // llvm.mips.srai.b
    10 << 8 | 0, // llvm.mips.srai.d
    10 << 8 | 0, // llvm.mips.srai.h
    10 << 8 | 0, // llvm.mips.srai.w
    10 << 8 | 1, // llvm.mips.srar.b
    10 << 8 | 1, // llvm.mips.srar.d
    10 << 8 | 1, // llvm.mips.srar.h
    10 << 8 | 1, // llvm.mips.srar.w
    10 << 8 | 0, // llvm.mips.srari.b
    10 << 8 | 0, // llvm.mips.srari.d
    10 << 8 | 0, // llvm.mips.srari.h
    10 << 8 | 0, // llvm.mips.srari.w
    10 << 8 | 1, // llvm.mips.srl.b
    10 << 8 | 1, // llvm.mips.srl.d
    10 << 8 | 1, // llvm.mips.srl.h
    10 << 8 | 1, // llvm.mips.srl.w
    10 << 8 | 0, // llvm.mips.srli.b
    10 << 8 | 0, // llvm.mips.srli.d
    10 << 8 | 0, // llvm.mips.srli.h
    10 << 8 | 0, // llvm.mips.srli.w
    10 << 8 | 1, // llvm.mips.srlr.b
    10 << 8 | 1, // llvm.mips.srlr.d
    10 << 8 | 1, // llvm.mips.srlr.h
    10 << 8 | 1, // llvm.mips.srlr.w
    10 << 8 | 0, // llvm.mips.srlri.b
    10 << 8 | 0, // llvm.mips.srlri.d
    10 << 8 | 0, // llvm.mips.srlri.h
    10 << 8 | 0, // llvm.mips.srlri.w
    11 << 8 | 1, // llvm.mips.st.b
    11 << 8 | 1, // llvm.mips.st.d
    11 << 8 | 1, // llvm.mips.st.h
    11 << 8 | 1, // llvm.mips.st.w
    11 << 8 | 1, // llvm.mips.str.d
    11 << 8 | 1, // llvm.mips.str.w
    10 << 8 | 1, // llvm.mips.subq.ph
    10 << 8 | 1, // llvm.mips.subq.s.ph
    5 << 8 | 1, // llvm.mips.subq.s.w
    10 << 8 | 1, // llvm.mips.subqh.ph
    10 << 8 | 1, // llvm.mips.subqh.r.ph
    10 << 8 | 1, // llvm.mips.subqh.r.w
    10 << 8 | 1, // llvm.mips.subqh.w
    10 << 8 | 1, // llvm.mips.subs.s.b
    10 << 8 | 1, // llvm.mips.subs.s.d
    10 << 8 | 1, // llvm.mips.subs.s.h
    10 << 8 | 1, // llvm.mips.subs.s.w
    10 << 8 | 1, // llvm.mips.subs.u.b
    10 << 8 | 1, // llvm.mips.subs.u.d
    10 << 8 | 1, // llvm.mips.subs.u.h
    10 << 8 | 1, // llvm.mips.subs.u.w
    10 << 8 | 1, // llvm.mips.subsus.u.b
    10 << 8 | 1, // llvm.mips.subsus.u.d
    10 << 8 | 1, // llvm.mips.subsus.u.h
    10 << 8 | 1, // llvm.mips.subsus.u.w
    10 << 8 | 1, // llvm.mips.subsuu.s.b
    10 << 8 | 1, // llvm.mips.subsuu.s.d
    10 << 8 | 1, // llvm.mips.subsuu.s.h
    10 << 8 | 1, // llvm.mips.subsuu.s.w
    5 << 8 | 1, // llvm.mips.subu.ph
    10 << 8 | 1, // llvm.mips.subu.qb
    5 << 8 | 1, // llvm.mips.subu.s.ph
    10 << 8 | 1, // llvm.mips.subu.s.qb
    10 << 8 | 1, // llvm.mips.subuh.qb
    10 << 8 | 1, // llvm.mips.subuh.r.qb
    10 << 8 | 1, // llvm.mips.subv.b
    10 << 8 | 1, // llvm.mips.subv.d
    10 << 8 | 1, // llvm.mips.subv.h
    10 << 8 | 1, // llvm.mips.subv.w
    10 << 8 | 0, // llvm.mips.subvi.b
    10 << 8 | 0, // llvm.mips.subvi.d
    10 << 8 | 0, // llvm.mips.subvi.h
    10 << 8 | 0, // llvm.mips.subvi.w
    10 << 8 | 1, // llvm.mips.vshf.b
    10 << 8 | 1, // llvm.mips.vshf.d
    10 << 8 | 1, // llvm.mips.vshf.h
    10 << 8 | 1, // llvm.mips.vshf.w
    5 << 8 | 0, // llvm.mips.wrdsp
    10 << 8 | 1, // llvm.mips.xor.v
    10 << 8 | 0, // llvm.mips.xori.b
    81 << 8 | 1, // llvm.nvvm.activemask
    0 << 8 | 1, // llvm.nvvm.add.rm.d
    0 << 8 | 1, // llvm.nvvm.add.rm.f
    0 << 8 | 1, // llvm.nvvm.add.rm.ftz.f
    0 << 8 | 1, // llvm.nvvm.add.rm.ftz.sat.f
    0 << 8 | 1, // llvm.nvvm.add.rm.sat.f
    0 << 8 | 1, // llvm.nvvm.add.rn.d
    0 << 8 | 1, // llvm.nvvm.add.rn.f
    0 << 8 | 1, // llvm.nvvm.add.rn.ftz.f
    0 << 8 | 1, // llvm.nvvm.add.rn.ftz.sat.f
    0 << 8 | 1, // llvm.nvvm.add.rn.sat.f
    0 << 8 | 1, // llvm.nvvm.add.rp.d
    0 << 8 | 1, // llvm.nvvm.add.rp.f
    0 << 8 | 1, // llvm.nvvm.add.rp.ftz.f
    0 << 8 | 1, // llvm.nvvm.add.rp.ftz.sat.f
    0 << 8 | 1, // llvm.nvvm.add.rp.sat.f
    0 << 8 | 1, // llvm.nvvm.add.rz.d
    0 << 8 | 1, // llvm.nvvm.add.rz.f
    0 << 8 | 1, // llvm.nvvm.add.rz.ftz.f
    0 << 8 | 1, // llvm.nvvm.add.rz.ftz.sat.f
    0 << 8 | 1, // llvm.nvvm.add.rz.sat.f
    18 << 8 | 51, // llvm.nvvm.applypriority.L2.evict.normal
    18 << 8 | 51, // llvm.nvvm.applypriority.global.L2.evict.normal
    82 << 8 | 10, // llvm.nvvm.atomic.add.gen.f.cta
    82 << 8 | 10, // llvm.nvvm.atomic.add.gen.f.sys
    82 << 8 | 10, // llvm.nvvm.atomic.add.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.add.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.and.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.and.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.cas.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.cas.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.dec.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.dec.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.exch.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.exch.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.inc.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.inc.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.max.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.max.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.min.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.min.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.or.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.or.gen.i.sys
    82 << 8 | 10, // llvm.nvvm.atomic.xor.gen.i.cta
    82 << 8 | 10, // llvm.nvvm.atomic.xor.gen.i.sys
    83 << 8 | 1, // llvm.nvvm.bar.warp.sync
    83 << 8 | 1, // llvm.nvvm.barrier.cluster.arrive
    83 << 8 | 1, // llvm.nvvm.barrier.cluster.arrive.aligned
    83 << 8 | 1, // llvm.nvvm.barrier.cluster.arrive.relaxed
    83 << 8 | 1, // llvm.nvvm.barrier.cluster.arrive.relaxed.aligned
    83 << 8 | 1, // llvm.nvvm.barrier.cluster.wait
    83 << 8 | 1, // llvm.nvvm.barrier.cluster.wait.aligned
    83 << 8 | 1, // llvm.nvvm.barrier.cta.arrive.aligned.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.arrive.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.and.aligned.all
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.and.aligned.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.and.all
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.and.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.or.aligned.all
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.or.aligned.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.or.all
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.or.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.popc.aligned.all
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.popc.aligned.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.popc.all
    83 << 8 | 1, // llvm.nvvm.barrier.cta.red.popc.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.sync.aligned.all
    83 << 8 | 1, // llvm.nvvm.barrier.cta.sync.aligned.count
    83 << 8 | 1, // llvm.nvvm.barrier.cta.sync.all
    83 << 8 | 1, // llvm.nvvm.barrier.cta.sync.count
    0 << 8 | 1, // llvm.nvvm.bf16x2.to.ue8m0x2.rp
    0 << 8 | 1, // llvm.nvvm.bf16x2.to.ue8m0x2.rp.satfinite
    0 << 8 | 1, // llvm.nvvm.bf16x2.to.ue8m0x2.rz
    0 << 8 | 1, // llvm.nvvm.bf16x2.to.ue8m0x2.rz.satfinite
    0 << 8 | 1, // llvm.nvvm.bf2h.rn
    0 << 8 | 1, // llvm.nvvm.bf2h.rn.ftz
    0 << 8 | 1, // llvm.nvvm.bmsk.clamp
    0 << 8 | 1, // llvm.nvvm.bmsk.wrap
    0 << 8 | 1, // llvm.nvvm.ceil.d
    0 << 8 | 1, // llvm.nvvm.ceil.f
    0 << 8 | 1, // llvm.nvvm.ceil.ftz.f
    0 << 8 | 1, // llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.x
    0 << 8 | 1, // llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.y
    0 << 8 | 1, // llvm.nvvm.clusterlaunchcontrol.query_cancel.get_first_ctaid.z
    0 << 8 | 1, // llvm.nvvm.clusterlaunchcontrol.query_cancel.is_canceled
    78 << 8 | 1, // llvm.nvvm.clusterlaunchcontrol.try_cancel.async.multicast.shared
    78 << 8 | 1, // llvm.nvvm.clusterlaunchcontrol.try_cancel.async.shared
    5 << 8 | 1, // llvm.nvvm.compiler.error
    5 << 8 | 1, // llvm.nvvm.compiler.warn
    1 << 8 | 1, // llvm.nvvm.cos.approx.f
    1 << 8 | 1, // llvm.nvvm.cos.approx.ftz.f
    5 << 8 | 1, // llvm.nvvm.cp.async.bulk.commit.group
    84 << 8 | 139, // llvm.nvvm.cp.async.bulk.global.to.shared.cluster
    84 << 8 | 140, // llvm.nvvm.cp.async.bulk.global.to.shared.cta
    84 << 8 | 102, // llvm.nvvm.cp.async.bulk.prefetch.L2
    84 << 8 | 141, // llvm.nvvm.cp.async.bulk.shared.cta.to.cluster
    84 << 8 | 142, // llvm.nvvm.cp.async.bulk.shared.cta.to.global
    84 << 8 | 142, // llvm.nvvm.cp.async.bulk.shared.cta.to.global.bytemask
    61 << 8 | 143, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.3d
    61 << 8 | 144, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.4d
    61 << 8 | 145, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.5d
    61 << 8 | 146, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.3d
    61 << 8 | 144, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.4d
    61 << 8 | 147, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.128.5d
    61 << 8 | 146, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.3d
    61 << 8 | 144, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.4d
    61 << 8 | 147, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.im2col.w.5d
    61 << 8 | 140, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.1d
    61 << 8 | 148, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.2d
    61 << 8 | 149, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.3d
    61 << 8 | 143, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.4d
    61 << 8 | 146, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.5d
    61 << 8 | 146, // llvm.nvvm.cp.async.bulk.tensor.g2s.cta.tile.gather4.2d
    61 << 8 | 150, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.3d
    61 << 8 | 151, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.4d
    61 << 8 | 152, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.5d
    61 << 8 | 153, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.3d
    61 << 8 | 151, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.4d
    61 << 8 | 154, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.128.5d
    61 << 8 | 153, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.3d
    61 << 8 | 151, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.4d
    61 << 8 | 154, // llvm.nvvm.cp.async.bulk.tensor.g2s.im2col.w.5d
    61 << 8 | 155, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.1d
    61 << 8 | 156, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.2d
    61 << 8 | 157, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.3d
    61 << 8 | 150, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.4d
    61 << 8 | 153, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.5d
    61 << 8 | 153, // llvm.nvvm.cp.async.bulk.tensor.g2s.tile.gather4.2d
    61 << 8 | 158, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.3d
    61 << 8 | 159, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.4d
    61 << 8 | 160, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.5d
    61 << 8 | 161, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.3d
    61 << 8 | 159, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.4d
    61 << 8 | 162, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.128.5d
    61 << 8 | 161, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.3d
    61 << 8 | 159, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.4d
    61 << 8 | 162, // llvm.nvvm.cp.async.bulk.tensor.prefetch.im2col.w.5d
    61 << 8 | 163, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.1d
    61 << 8 | 164, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.2d
    61 << 8 | 165, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.3d
    61 << 8 | 158, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.4d
    61 << 8 | 161, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.5d
    61 << 8 | 161, // llvm.nvvm.cp.async.bulk.tensor.prefetch.tile.gather4.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.add.tile.5d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.and.tile.5d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.dec.tile.5d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.inc.tile.5d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.max.tile.5d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.min.tile.5d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.or.tile.5d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.reduce.xor.tile.5d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.s2g.im2col.5d
    61 << 8 | 169, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.1d
    61 << 8 | 170, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.2d
    61 << 8 | 166, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.3d
    61 << 8 | 167, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.4d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.5d
    61 << 8 | 168, // llvm.nvvm.cp.async.bulk.tensor.s2g.tile.scatter4.2d
    5 << 8 | 5, // llvm.nvvm.cp.async.bulk.wait.group
    5 << 8 | 5, // llvm.nvvm.cp.async.bulk.wait.group.read
    82 << 8 | 171, // llvm.nvvm.cp.async.ca.shared.global.16
    82 << 8 | 171, // llvm.nvvm.cp.async.ca.shared.global.16.s
    82 << 8 | 171, // llvm.nvvm.cp.async.ca.shared.global.4
    82 << 8 | 171, // llvm.nvvm.cp.async.ca.shared.global.4.s
    82 << 8 | 171, // llvm.nvvm.cp.async.ca.shared.global.8
    82 << 8 | 171, // llvm.nvvm.cp.async.ca.shared.global.8.s
    82 << 8 | 171, // llvm.nvvm.cp.async.cg.shared.global.16
    82 << 8 | 171, // llvm.nvvm.cp.async.cg.shared.global.16.s
    5 << 8 | 1, // llvm.nvvm.cp.async.commit.group
    83 << 8 | 1, // llvm.nvvm.cp.async.mbarrier.arrive
    83 << 8 | 1, // llvm.nvvm.cp.async.mbarrier.arrive.noinc
    83 << 8 | 1, // llvm.nvvm.cp.async.mbarrier.arrive.noinc.shared
    83 << 8 | 1, // llvm.nvvm.cp.async.mbarrier.arrive.shared
    5 << 8 | 1, // llvm.nvvm.cp.async.wait.all
    5 << 8 | 5, // llvm.nvvm.cp.async.wait.group
    0 << 8 | 1, // llvm.nvvm.d2f.rm
    0 << 8 | 1, // llvm.nvvm.d2f.rm.ftz
    0 << 8 | 1, // llvm.nvvm.d2f.rn
    0 << 8 | 1, // llvm.nvvm.d2f.rn.ftz
    0 << 8 | 1, // llvm.nvvm.d2f.rp
    0 << 8 | 1, // llvm.nvvm.d2f.rp.ftz
    0 << 8 | 1, // llvm.nvvm.d2f.rz
    0 << 8 | 1, // llvm.nvvm.d2f.rz.ftz
    0 << 8 | 1, // llvm.nvvm.d2i.hi
    0 << 8 | 1, // llvm.nvvm.d2i.lo
    0 << 8 | 1, // llvm.nvvm.d2i.rm
    0 << 8 | 1, // llvm.nvvm.d2i.rn
    0 << 8 | 1, // llvm.nvvm.d2i.rp
    0 << 8 | 1, // llvm.nvvm.d2i.rz
    0 << 8 | 1, // llvm.nvvm.d2ll.rm
    0 << 8 | 1, // llvm.nvvm.d2ll.rn
    0 << 8 | 1, // llvm.nvvm.d2ll.rp
    0 << 8 | 1, // llvm.nvvm.d2ll.rz
    0 << 8 | 1, // llvm.nvvm.d2ui.rm
    0 << 8 | 1, // llvm.nvvm.d2ui.rn
    0 << 8 | 1, // llvm.nvvm.d2ui.rp
    0 << 8 | 1, // llvm.nvvm.d2ui.rz
    0 << 8 | 1, // llvm.nvvm.d2ull.rm
    0 << 8 | 1, // llvm.nvvm.d2ull.rn
    0 << 8 | 1, // llvm.nvvm.d2ull.rp
    0 << 8 | 1, // llvm.nvvm.d2ull.rz
    30 << 8 | 64, // llvm.nvvm.discard.L2
    30 << 8 | 64, // llvm.nvvm.discard.global.L2
    1 << 8 | 1, // llvm.nvvm.div.approx.f
    1 << 8 | 1, // llvm.nvvm.div.approx.ftz.f
    1 << 8 | 1, // llvm.nvvm.div.full
    1 << 8 | 1, // llvm.nvvm.div.full.ftz
    1 << 8 | 1, // llvm.nvvm.div.rm.d
    1 << 8 | 1, // llvm.nvvm.div.rm.f
    1 << 8 | 1, // llvm.nvvm.div.rm.ftz.f
    1 << 8 | 1, // llvm.nvvm.div.rn.d
    1 << 8 | 1, // llvm.nvvm.div.rn.f
    1 << 8 | 1, // llvm.nvvm.div.rn.ftz.f
    1 << 8 | 1, // llvm.nvvm.div.rp.d
    1 << 8 | 1, // llvm.nvvm.div.rp.f
    1 << 8 | 1, // llvm.nvvm.div.rp.ftz.f
    1 << 8 | 1, // llvm.nvvm.div.rz.d
    1 << 8 | 1, // llvm.nvvm.div.rz.f
    1 << 8 | 1, // llvm.nvvm.div.rz.ftz.f
    0 << 8 | 1, // llvm.nvvm.e2m1x2.to.f16x2.rn
    0 << 8 | 1, // llvm.nvvm.e2m1x2.to.f16x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.e2m3x2.to.f16x2.rn
    0 << 8 | 1, // llvm.nvvm.e2m3x2.to.f16x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.e3m2x2.to.f16x2.rn
    0 << 8 | 1, // llvm.nvvm.e3m2x2.to.f16x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.e4m3x2.to.f16x2.rn
    0 << 8 | 1, // llvm.nvvm.e4m3x2.to.f16x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.e5m2x2.to.f16x2.rn
    0 << 8 | 1, // llvm.nvvm.e5m2x2.to.f16x2.rn.relu
    85 << 8 | 1, // llvm.nvvm.elect.sync
    1 << 8 | 1, // llvm.nvvm.ex2.approx
    1 << 8 | 1, // llvm.nvvm.ex2.approx.ftz
    86 << 8 | 1, // llvm.nvvm.exit
    0 << 8 | 1, // llvm.nvvm.f16x2.to.e4m3x2.rn
    0 << 8 | 1, // llvm.nvvm.f16x2.to.e4m3x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.f16x2.to.e5m2x2.rn
    0 << 8 | 1, // llvm.nvvm.f16x2.to.e5m2x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.f2bf16.rn
    0 << 8 | 1, // llvm.nvvm.f2bf16.rn.relu
    0 << 8 | 1, // llvm.nvvm.f2bf16.rn.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f2bf16.rn.satfinite
    0 << 8 | 1, // llvm.nvvm.f2bf16.rz
    0 << 8 | 1, // llvm.nvvm.f2bf16.rz.relu
    0 << 8 | 1, // llvm.nvvm.f2bf16.rz.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f2bf16.rz.satfinite
    0 << 8 | 1, // llvm.nvvm.f2f16.rn
    0 << 8 | 1, // llvm.nvvm.f2f16.rn.relu
    0 << 8 | 1, // llvm.nvvm.f2f16.rn.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f2f16.rn.satfinite
    0 << 8 | 1, // llvm.nvvm.f2f16.rz
    0 << 8 | 1, // llvm.nvvm.f2f16.rz.relu
    0 << 8 | 1, // llvm.nvvm.f2f16.rz.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f2f16.rz.satfinite
    0 << 8 | 1, // llvm.nvvm.f2h.rn
    0 << 8 | 1, // llvm.nvvm.f2h.rn.ftz
    0 << 8 | 1, // llvm.nvvm.f2i.rm
    0 << 8 | 1, // llvm.nvvm.f2i.rm.ftz
    0 << 8 | 1, // llvm.nvvm.f2i.rn
    0 << 8 | 1, // llvm.nvvm.f2i.rn.ftz
    0 << 8 | 1, // llvm.nvvm.f2i.rp
    0 << 8 | 1, // llvm.nvvm.f2i.rp.ftz
    0 << 8 | 1, // llvm.nvvm.f2i.rz
    0 << 8 | 1, // llvm.nvvm.f2i.rz.ftz
    0 << 8 | 1, // llvm.nvvm.f2ll.rm
    0 << 8 | 1, // llvm.nvvm.f2ll.rm.ftz
    0 << 8 | 1, // llvm.nvvm.f2ll.rn
    0 << 8 | 1, // llvm.nvvm.f2ll.rn.ftz
    0 << 8 | 1, // llvm.nvvm.f2ll.rp
    0 << 8 | 1, // llvm.nvvm.f2ll.rp.ftz
    0 << 8 | 1, // llvm.nvvm.f2ll.rz
    0 << 8 | 1, // llvm.nvvm.f2ll.rz.ftz
    0 << 8 | 1, // llvm.nvvm.f2tf32.rn
    0 << 8 | 1, // llvm.nvvm.f2tf32.rn.relu
    0 << 8 | 1, // llvm.nvvm.f2tf32.rn.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f2tf32.rn.satfinite
    0 << 8 | 1, // llvm.nvvm.f2tf32.rna
    0 << 8 | 1, // llvm.nvvm.f2tf32.rna.satfinite
    0 << 8 | 1, // llvm.nvvm.f2tf32.rz
    0 << 8 | 1, // llvm.nvvm.f2tf32.rz.relu
    0 << 8 | 1, // llvm.nvvm.f2tf32.rz.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f2tf32.rz.satfinite
    0 << 8 | 1, // llvm.nvvm.f2ui.rm
    0 << 8 | 1, // llvm.nvvm.f2ui.rm.ftz
    0 << 8 | 1, // llvm.nvvm.f2ui.rn
    0 << 8 | 1, // llvm.nvvm.f2ui.rn.ftz
    0 << 8 | 1, // llvm.nvvm.f2ui.rp
    0 << 8 | 1, // llvm.nvvm.f2ui.rp.ftz
    0 << 8 | 1, // llvm.nvvm.f2ui.rz
    0 << 8 | 1, // llvm.nvvm.f2ui.rz.ftz
    0 << 8 | 1, // llvm.nvvm.f2ull.rm
    0 << 8 | 1, // llvm.nvvm.f2ull.rm.ftz
    0 << 8 | 1, // llvm.nvvm.f2ull.rn
    0 << 8 | 1, // llvm.nvvm.f2ull.rn.ftz
    0 << 8 | 1, // llvm.nvvm.f2ull.rp
    0 << 8 | 1, // llvm.nvvm.f2ull.rp.ftz
    0 << 8 | 1, // llvm.nvvm.f2ull.rz
    0 << 8 | 1, // llvm.nvvm.f2ull.rz.ftz
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e2m1x4.rs.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e2m1x4.rs.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e2m3x4.rs.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e2m3x4.rs.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e3m2x4.rs.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e3m2x4.rs.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e4m3x4.rs.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e4m3x4.rs.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e5m2x4.rs.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.f32x4.to.e5m2x4.rs.satfinite
    0 << 8 | 1, // llvm.nvvm.fabs
    0 << 8 | 1, // llvm.nvvm.fabs.ftz
    33 << 8 | 1, // llvm.nvvm.fence.acquire.sync_restrict.space.cluster.scope.cluster
    33 << 8 | 1, // llvm.nvvm.fence.mbarrier_init.release.cluster
    33 << 8 | 1, // llvm.nvvm.fence.proxy.alias
    33 << 8 | 1, // llvm.nvvm.fence.proxy.async
    33 << 8 | 1, // llvm.nvvm.fence.proxy.async.global
    33 << 8 | 1, // llvm.nvvm.fence.proxy.async.shared_cluster
    33 << 8 | 1, // llvm.nvvm.fence.proxy.async.shared_cta
    33 << 8 | 1, // llvm.nvvm.fence.proxy.async_generic.acquire.sync_restrict.space.cluster.scope.cluster
    33 << 8 | 1, // llvm.nvvm.fence.proxy.async_generic.release.sync_restrict.space.cta.scope.cluster
    82 << 8 | 172, // llvm.nvvm.fence.proxy.tensormap_generic.acquire.cluster
    82 << 8 | 172, // llvm.nvvm.fence.proxy.tensormap_generic.acquire.cta
    82 << 8 | 172, // llvm.nvvm.fence.proxy.tensormap_generic.acquire.gpu
    82 << 8 | 172, // llvm.nvvm.fence.proxy.tensormap_generic.acquire.sys
    33 << 8 | 1, // llvm.nvvm.fence.proxy.tensormap_generic.release.cluster
    33 << 8 | 1, // llvm.nvvm.fence.proxy.tensormap_generic.release.cta
    33 << 8 | 1, // llvm.nvvm.fence.proxy.tensormap_generic.release.gpu
    33 << 8 | 1, // llvm.nvvm.fence.proxy.tensormap_generic.release.sys
    33 << 8 | 1, // llvm.nvvm.fence.release.sync_restrict.space.cta.scope.cluster
    33 << 8 | 1, // llvm.nvvm.fence.sc.cluster
    0 << 8 | 1, // llvm.nvvm.ff.to.e2m1x2.rn.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff.to.e2m1x2.rn.satfinite
    0 << 8 | 1, // llvm.nvvm.ff.to.e2m3x2.rn.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff.to.e2m3x2.rn.satfinite
    0 << 8 | 1, // llvm.nvvm.ff.to.e3m2x2.rn.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff.to.e3m2x2.rn.satfinite
    0 << 8 | 1, // llvm.nvvm.ff.to.e4m3x2.rn
    0 << 8 | 1, // llvm.nvvm.ff.to.e4m3x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.ff.to.e5m2x2.rn
    0 << 8 | 1, // llvm.nvvm.ff.to.e5m2x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.ff.to.ue8m0x2.rp
    0 << 8 | 1, // llvm.nvvm.ff.to.ue8m0x2.rp.satfinite
    0 << 8 | 1, // llvm.nvvm.ff.to.ue8m0x2.rz
    0 << 8 | 1, // llvm.nvvm.ff.to.ue8m0x2.rz.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rn
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rn.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rn.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rs
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rs.relu
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rs.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rs.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rz
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rz.relu
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rz.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2bf16x2.rz.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rn
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rn.relu
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rn.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rn.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rs
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rs.relu
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rs.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rs.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rz
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rz.relu
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rz.relu.satfinite
    0 << 8 | 1, // llvm.nvvm.ff2f16x2.rz.satfinite
    0 << 8 | 0, // llvm.nvvm.flo.s
    0 << 8 | 0, // llvm.nvvm.flo.u
    0 << 8 | 1, // llvm.nvvm.floor.d
    0 << 8 | 1, // llvm.nvvm.floor.f
    0 << 8 | 1, // llvm.nvvm.floor.ftz.f
    4 << 8 | 1, // llvm.nvvm.fma.rm.d
    0 << 8 | 1, // llvm.nvvm.fma.rm.f
    0 << 8 | 1, // llvm.nvvm.fma.rm.ftz.f
    0 << 8 | 1, // llvm.nvvm.fma.rm.ftz.sat.f
    0 << 8 | 1, // llvm.nvvm.fma.rm.sat.f
    0 << 8 | 1, // llvm.nvvm.fma.rn.bf16
    0 << 8 | 1, // llvm.nvvm.fma.rn.bf16x2
    4 << 8 | 1, // llvm.nvvm.fma.rn.d
    0 << 8 | 1, // llvm.nvvm.fma.rn.f
    0 << 8 | 1, // llvm.nvvm.fma.rn.f16
    0 << 8 | 1, // llvm.nvvm.fma.rn.f16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.bf16
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.bf16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.f
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.f16
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.f16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.relu.bf16
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.relu.bf16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.relu.f16
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.relu.f16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.sat.bf16
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.sat.bf16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.sat.f
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.sat.f16
    0 << 8 | 1, // llvm.nvvm.fma.rn.ftz.sat.f16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.relu.bf16
    0 << 8 | 1, // llvm.nvvm.fma.rn.relu.bf16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.relu.f16
    0 << 8 | 1, // llvm.nvvm.fma.rn.relu.f16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.sat.bf16
    0 << 8 | 1, // llvm.nvvm.fma.rn.sat.bf16x2
    0 << 8 | 1, // llvm.nvvm.fma.rn.sat.f
    0 << 8 | 1, // llvm.nvvm.fma.rn.sat.f16
    0 << 8 | 1, // llvm.nvvm.fma.rn.sat.f16x2
    4 << 8 | 1, // llvm.nvvm.fma.rp.d
    0 << 8 | 1, // llvm.nvvm.fma.rp.f
    0 << 8 | 1, // llvm.nvvm.fma.rp.ftz.f
    0 << 8 | 1, // llvm.nvvm.fma.rp.ftz.sat.f
    0 << 8 | 1, // llvm.nvvm.fma.rp.sat.f
    4 << 8 | 1, // llvm.nvvm.fma.rz.d
    0 << 8 | 1, // llvm.nvvm.fma.rz.f
    0 << 8 | 1, // llvm.nvvm.fma.rz.ftz.f
    0 << 8 | 1, // llvm.nvvm.fma.rz.ftz.sat.f
    0 << 8 | 1, // llvm.nvvm.fma.rz.sat.f
    0 << 8 | 1, // llvm.nvvm.fmax.bf16
    0 << 8 | 1, // llvm.nvvm.fmax.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmax.d
    0 << 8 | 1, // llvm.nvvm.fmax.f
    0 << 8 | 1, // llvm.nvvm.fmax.f16
    0 << 8 | 1, // llvm.nvvm.fmax.f16x2
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.bf16
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.f
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.f16
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.f16x2
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.bf16
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.f
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.f16
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.f16x2
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.bf16
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.f
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.f16
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.nan.xorsign.abs.f16x2
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.xorsign.abs.bf16
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.xorsign.abs.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.xorsign.abs.f
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.xorsign.abs.f16
    0 << 8 | 1, // llvm.nvvm.fmax.ftz.xorsign.abs.f16x2
    0 << 8 | 1, // llvm.nvvm.fmax.nan.bf16
    0 << 8 | 1, // llvm.nvvm.fmax.nan.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmax.nan.f
    0 << 8 | 1, // llvm.nvvm.fmax.nan.f16
    0 << 8 | 1, // llvm.nvvm.fmax.nan.f16x2
    0 << 8 | 1, // llvm.nvvm.fmax.nan.xorsign.abs.bf16
    0 << 8 | 1, // llvm.nvvm.fmax.nan.xorsign.abs.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmax.nan.xorsign.abs.f
    0 << 8 | 1, // llvm.nvvm.fmax.nan.xorsign.abs.f16
    0 << 8 | 1, // llvm.nvvm.fmax.nan.xorsign.abs.f16x2
    0 << 8 | 1, // llvm.nvvm.fmax.xorsign.abs.bf16
    0 << 8 | 1, // llvm.nvvm.fmax.xorsign.abs.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmax.xorsign.abs.f
    0 << 8 | 1, // llvm.nvvm.fmax.xorsign.abs.f16
    0 << 8 | 1, // llvm.nvvm.fmax.xorsign.abs.f16x2
    0 << 8 | 1, // llvm.nvvm.fmin.bf16
    0 << 8 | 1, // llvm.nvvm.fmin.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmin.d
    0 << 8 | 1, // llvm.nvvm.fmin.f
    0 << 8 | 1, // llvm.nvvm.fmin.f16
    0 << 8 | 1, // llvm.nvvm.fmin.f16x2
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.bf16
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.f
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.f16
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.f16x2
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.bf16
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.f
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.f16
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.f16x2
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.bf16
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.f
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.f16
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.nan.xorsign.abs.f16x2
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.xorsign.abs.bf16
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.xorsign.abs.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.xorsign.abs.f
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.xorsign.abs.f16
    0 << 8 | 1, // llvm.nvvm.fmin.ftz.xorsign.abs.f16x2
    0 << 8 | 1, // llvm.nvvm.fmin.nan.bf16
    0 << 8 | 1, // llvm.nvvm.fmin.nan.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmin.nan.f
    0 << 8 | 1, // llvm.nvvm.fmin.nan.f16
    0 << 8 | 1, // llvm.nvvm.fmin.nan.f16x2
    0 << 8 | 1, // llvm.nvvm.fmin.nan.xorsign.abs.bf16
    0 << 8 | 1, // llvm.nvvm.fmin.nan.xorsign.abs.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmin.nan.xorsign.abs.f
    0 << 8 | 1, // llvm.nvvm.fmin.nan.xorsign.abs.f16
    0 << 8 | 1, // llvm.nvvm.fmin.nan.xorsign.abs.f16x2
    0 << 8 | 1, // llvm.nvvm.fmin.xorsign.abs.bf16
    0 << 8 | 1, // llvm.nvvm.fmin.xorsign.abs.bf16x2
    0 << 8 | 1, // llvm.nvvm.fmin.xorsign.abs.f
    0 << 8 | 1, // llvm.nvvm.fmin.xorsign.abs.f16
    0 << 8 | 1, // llvm.nvvm.fmin.xorsign.abs.f16x2
    0 << 8 | 1, // llvm.nvvm.fns
    0 << 8 | 1, // llvm.nvvm.fshl.clamp
    0 << 8 | 1, // llvm.nvvm.fshr.clamp
    0 << 8 | 10, // llvm.nvvm.getctarank
    0 << 8 | 10, // llvm.nvvm.getctarank.shared.cluster
    29 << 8 | 1, // llvm.nvvm.griddepcontrol.launch.dependents
    29 << 8 | 1, // llvm.nvvm.griddepcontrol.wait
    0 << 8 | 1, // llvm.nvvm.i2d.rm
    0 << 8 | 1, // llvm.nvvm.i2d.rn
    0 << 8 | 1, // llvm.nvvm.i2d.rp
    0 << 8 | 1, // llvm.nvvm.i2d.rz
    0 << 8 | 1, // llvm.nvvm.i2f.rm
    0 << 8 | 1, // llvm.nvvm.i2f.rn
    0 << 8 | 1, // llvm.nvvm.i2f.rp
    0 << 8 | 1, // llvm.nvvm.i2f.rz
    0 << 8 | 11, // llvm.nvvm.idp2a.s.s
    0 << 8 | 11, // llvm.nvvm.idp2a.s.u
    0 << 8 | 11, // llvm.nvvm.idp2a.u.s
    0 << 8 | 11, // llvm.nvvm.idp2a.u.u
    0 << 8 | 1, // llvm.nvvm.idp4a.s.s
    0 << 8 | 1, // llvm.nvvm.idp4a.s.u
    0 << 8 | 1, // llvm.nvvm.idp4a.u.s
    0 << 8 | 1, // llvm.nvvm.idp4a.u.u
    0 << 8 | 173, // llvm.nvvm.internal.addrspace.wrap
    0 << 8 | 2, // llvm.nvvm.is_explicit_cluster
    0 << 8 | 10, // llvm.nvvm.isspacep.const
    0 << 8 | 10, // llvm.nvvm.isspacep.global
    0 << 8 | 10, // llvm.nvvm.isspacep.local
    0 << 8 | 10, // llvm.nvvm.isspacep.shared
    0 << 8 | 10, // llvm.nvvm.isspacep.shared.cluster
    1 << 8 | 1, // llvm.nvvm.istypep.sampler
    1 << 8 | 1, // llvm.nvvm.istypep.surface
    1 << 8 | 1, // llvm.nvvm.istypep.texture
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8x16.b4x16_p64
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x1.trans.b8x16.b6x16_p32
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8x16.b4x16_p64
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m16n16.x2.trans.b8x16.b6x16_p32
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x1.b8x16.b4x16_p64
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x1.b8x16.b6x16_p32
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x2.b8x16.b4x16_p64
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x2.b8x16.b6x16_p32
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x4.b8x16.b4x16_p64
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n16.x4.b8x16.b6x16_p32
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x1.b16
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x1.trans.b16
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x2.b16
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x2.trans.b16
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x4.b16
    87 << 8 | 7, // llvm.nvvm.ldmatrix.sync.aligned.m8n8.x4.trans.b16
    88 << 8 | 10, // llvm.nvvm.ldu.global.f
    88 << 8 | 10, // llvm.nvvm.ldu.global.i
    88 << 8 | 10, // llvm.nvvm.ldu.global.p
    1 << 8 | 1, // llvm.nvvm.lg2.approx.d
    1 << 8 | 1, // llvm.nvvm.lg2.approx.f
    1 << 8 | 1, // llvm.nvvm.lg2.approx.ftz.f
    0 << 8 | 1, // llvm.nvvm.ll2d.rm
    0 << 8 | 1, // llvm.nvvm.ll2d.rn
    0 << 8 | 1, // llvm.nvvm.ll2d.rp
    0 << 8 | 1, // llvm.nvvm.ll2d.rz
    0 << 8 | 1, // llvm.nvvm.ll2f.rm
    0 << 8 | 1, // llvm.nvvm.ll2f.rn
    0 << 8 | 1, // llvm.nvvm.ll2f.rp
    0 << 8 | 1, // llvm.nvvm.ll2f.rz
    0 << 8 | 1, // llvm.nvvm.lohi.i2d
    0 << 8 | 10, // llvm.nvvm.mapa
    0 << 8 | 10, // llvm.nvvm.mapa.shared.cluster
    89 << 8 | 1, // llvm.nvvm.match.all.sync.i32p
    89 << 8 | 1, // llvm.nvvm.match.all.sync.i64p
    89 << 8 | 1, // llvm.nvvm.match.any.sync.i32
    89 << 8 | 1, // llvm.nvvm.match.any.sync.i64
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cluster.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cluster.space.cta
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cta.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.relaxed.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cluster.space.cluster
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cluster.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cta.space.cluster
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.expect.tx.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.noComplete
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.noComplete.shared
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cluster.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cluster.space.cta
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cta.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.relaxed.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.scope.cluster.space.cluster
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.scope.cluster.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.scope.cta.space.cluster
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.drop.shared
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cluster.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cluster.space.cta
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cta.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.expect.tx.relaxed.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.expect.tx.scope.cluster.space.cluster
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.expect.tx.scope.cluster.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.expect.tx.scope.cta.space.cluster
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.expect.tx.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.noComplete
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.noComplete.shared
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.relaxed.scope.cluster.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.relaxed.scope.cluster.space.cta
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.relaxed.scope.cta.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.arrive.relaxed.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.scope.cluster.space.cluster
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.scope.cluster.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.scope.cta.space.cluster
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.arrive.shared
    90 << 8 | 1, // llvm.nvvm.mbarrier.complete.tx.scope.cluster.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.complete.tx.scope.cluster.space.cta
    90 << 8 | 1, // llvm.nvvm.mbarrier.complete.tx.scope.cta.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.complete.tx.scope.cta.space.cta
    90 << 8 | 1, // llvm.nvvm.mbarrier.expect.tx.scope.cluster.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.expect.tx.scope.cluster.space.cta
    90 << 8 | 1, // llvm.nvvm.mbarrier.expect.tx.scope.cta.space.cluster
    90 << 8 | 1, // llvm.nvvm.mbarrier.expect.tx.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.init
    83 << 8 | 1, // llvm.nvvm.mbarrier.init.shared
    91 << 8 | 36, // llvm.nvvm.mbarrier.inval
    91 << 8 | 36, // llvm.nvvm.mbarrier.inval.shared
    92 << 8 | 1, // llvm.nvvm.mbarrier.pending.count
    83 << 8 | 1, // llvm.nvvm.mbarrier.test.wait
    87 << 8 | 10, // llvm.nvvm.mbarrier.test.wait.parity.relaxed.scope.cluster.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.test.wait.parity.relaxed.scope.cta.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.test.wait.parity.scope.cluster.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.test.wait.parity.scope.cta.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.test.wait.relaxed.scope.cluster.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.test.wait.relaxed.scope.cta.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.test.wait.scope.cluster.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.test.wait.scope.cta.space.cta
    83 << 8 | 1, // llvm.nvvm.mbarrier.test.wait.shared
    87 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.parity.relaxed.scope.cluster.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.parity.relaxed.scope.cta.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.parity.scope.cluster.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.parity.scope.cta.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.parity.tl.relaxed.scope.cluster.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.parity.tl.relaxed.scope.cta.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.parity.tl.scope.cluster.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.parity.tl.scope.cta.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.relaxed.scope.cluster.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.relaxed.scope.cta.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.scope.cluster.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.scope.cta.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.tl.relaxed.scope.cluster.space.cta
    87 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.tl.relaxed.scope.cta.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.tl.scope.cluster.space.cta
    83 << 8 | 10, // llvm.nvvm.mbarrier.try.wait.tl.scope.cta.space.cta
    33 << 8 | 1, // llvm.nvvm.membar.cta
    33 << 8 | 1, // llvm.nvvm.membar.gl
    33 << 8 | 1, // llvm.nvvm.membar.sys
    93 << 8 | 1, // llvm.nvvm.mma.and.popc.m16n8k128.row.col.b1
    93 << 8 | 1, // llvm.nvvm.mma.and.popc.m16n8k256.row.col.b1
    93 << 8 | 1, // llvm.nvvm.mma.and.popc.m8n8k128.row.col.b1
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m1.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e2m3.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e3m2.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e4m3.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.f32.e5m2.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m1.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e2m3.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e3m2.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e4m3.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e3m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e4m3.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k32.row.col.mxf8f6f4.scale.1x.f32.e5m2.e5m2.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4nvf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 1, // llvm.nvvm.mma.block.scale.m16n8k64.row.col.mxf4nvf4.scale.4x.f32.e2m1.e2m1.f32.ue4m3
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.bf16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f16.e4m3.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f16.e4m3.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f16.e5m2.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f16.e5m2.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f32.e4m3.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f32.e4m3.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f32.e5m2.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f32.e5m2.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.f64
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.s8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.s8.u8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.satfinite.s8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.satfinite.s8.u8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.satfinite.u8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.satfinite.u8.s8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.u8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k16.row.col.u8.s8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.f16.e4m3.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.f16.e4m3.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.f16.e5m2.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.f16.e5m2.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.f32.e4m3.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.f32.e4m3.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.f32.e5m2.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.f32.e5m2.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e2m1.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e2m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e3m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m1.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e2m1.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e2m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e3m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e2m3.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e2m1.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e2m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e3m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e3m2.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e2m1.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e2m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e3m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e4m3.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e2m1.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e2m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e3m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e4m3.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f16.e5m2.e5m2.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e2m1.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e2m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e3m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m1.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e2m1.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e2m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e3m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e2m3.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e2m1.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e2m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e3m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e3m2.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e2m1.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e2m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e3m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e4m3.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e2m1.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e2m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e3m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e4m3.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.kind.f8f6f4.f32.e5m2.e5m2.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.s4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.s4.u4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.s8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.s8.u8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.s4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.s4.u4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.s8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.s8.u8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.u4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.u4.s4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.u8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.satfinite.u8.s8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.u4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.u4.s4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.u8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k32.row.col.u8.s8
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k4.row.col.f64
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k4.row.col.tf32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k64.row.col.s4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k64.row.col.s4.u4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k64.row.col.satfinite.s4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k64.row.col.satfinite.s4.u4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k64.row.col.satfinite.u4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k64.row.col.satfinite.u4.s4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k64.row.col.u4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k64.row.col.u4.s4
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k8.row.col.bf16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k8.row.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k8.row.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k8.row.col.f64
    93 << 8 | 1, // llvm.nvvm.mma.m16n8k8.row.col.tf32
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k16.row.col.s8
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k16.row.col.s8.u8
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k16.row.col.satfinite.s8
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k16.row.col.satfinite.s8.u8
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k16.row.col.satfinite.u8
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k16.row.col.satfinite.u8.s8
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k16.row.col.u8
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k16.row.col.u8.s8
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k32.row.col.s4
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k32.row.col.s4.u4
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k32.row.col.satfinite.s4
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k32.row.col.satfinite.s4.u4
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k32.row.col.satfinite.u4
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k32.row.col.satfinite.u4.s4
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k32.row.col.u4
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k32.row.col.u4.s4
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.col.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.col.col.f32.f16
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.col.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.col.row.f16.f16
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.col.row.f32.f16
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.col.row.f32.f32
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.row.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.row.col.f32.f16
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.row.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.row.col.f64
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.row.row.f16.f16
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.row.row.f32.f16
    93 << 8 | 1, // llvm.nvvm.mma.m8n8k4.row.row.f32.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k128.row.col.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k128.row.col.s4.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.s4.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k128.row.col.satfinite.u4.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k128.row.col.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k128.row.col.u4.s4
    93 << 8 | 175, // llvm.nvvm.mma.sp.m16n8k16.row.col.bf16
    93 << 8 | 176, // llvm.nvvm.mma.sp.m16n8k16.row.col.f16.f16
    93 << 8 | 175, // llvm.nvvm.mma.sp.m16n8k16.row.col.f32.f32
    93 << 8 | 177, // llvm.nvvm.mma.sp.m16n8k16.row.col.tf32
    93 << 8 | 177, // llvm.nvvm.mma.sp.m16n8k32.row.col.bf16
    93 << 8 | 178, // llvm.nvvm.mma.sp.m16n8k32.row.col.f16.f16
    93 << 8 | 177, // llvm.nvvm.mma.sp.m16n8k32.row.col.f32.f32
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k32.row.col.s8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k32.row.col.s8.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.s8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.s8.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k32.row.col.satfinite.u8.s8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k32.row.col.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k32.row.col.u8.s8
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e4m3.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e4m3.e5m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e5m2.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.f32.e5m2.e5m2.f32
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k64.row.col.s4
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k64.row.col.s4.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.s8
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.s8.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s4
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s4.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s8
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.s8.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u4
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u4.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u8
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.satfinite.u8.s8
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k64.row.col.u4
    93 << 8 | 179, // llvm.nvvm.mma.sp.m16n8k64.row.col.u4.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.u8
    93 << 8 | 174, // llvm.nvvm.mma.sp.m16n8k64.row.col.u8.s8
    93 << 8 | 175, // llvm.nvvm.mma.sp.m16n8k8.row.col.tf32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4nvf4.scale.2x.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k128.row.col.mxf4nvf4.scale.4x.f32.e2m1.e2m1.f32.ue4m3
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m1.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e2m3.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e3m2.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e4m3.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.f32.e5m2.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m1.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e2m3.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e3m2.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e4m3.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m1.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e2m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e3m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e4m3.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.block.scale.m16n8k64.row.col.mxf8f6f4.scale.1x.f32.e5m2.e5m2.f32.ue8m0
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.s4.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.s4.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.satfinite.u4.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k128.row.col.u4.s4
    93 << 8 | 175, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.bf16
    93 << 8 | 176, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.f16.f16
    93 << 8 | 175, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.f32.f32
    93 << 8 | 177, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k16.row.col.tf32
    93 << 8 | 177, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.bf16
    93 << 8 | 178, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.f16.f16
    93 << 8 | 177, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.f32.f32
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.s8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.s8.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.s8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.s8.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.satfinite.u8.s8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k32.row.col.u8.s8
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e4m3.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e4m3.e5m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e5m2.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.f32.e5m2.e5m2.f32
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e2m1.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e2m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e3m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e4m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m1.e5m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e2m1.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e2m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e3m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e4m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e2m3.e5m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e2m1.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e2m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e3m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e4m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e3m2.e5m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e2m1.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e2m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e3m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e4m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e4m3.e5m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e2m1.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e2m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e3m2.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e4m3.f16
    93 << 8 | 180, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f16.e5m2.e5m2.f16
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e2m1.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e2m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e3m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m1.e5m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e2m1.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e2m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e3m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e2m3.e5m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e2m1.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e2m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e3m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e3m2.e5m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e2m1.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e2m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e3m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e4m3.e5m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e2m1.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e2m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e3m2.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e4m3.f32
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.kind.f8f6f4.f32.e5m2.e5m2.f32
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s4
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s4.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s8
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.s8.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s4
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s4.u4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s8
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.s8.u8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u4
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u4.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u8
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.satfinite.u8.s8
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u4
    93 << 8 | 179, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u4.s4
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u8
    93 << 8 | 174, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k64.row.col.u8.s8
    93 << 8 | 175, // llvm.nvvm.mma.sp.ordered.metadata.m16n8k8.row.col.tf32
    93 << 8 | 1, // llvm.nvvm.mma.xor.popc.m16n8k128.row.col.b1
    93 << 8 | 1, // llvm.nvvm.mma.xor.popc.m16n8k256.row.col.b1
    93 << 8 | 1, // llvm.nvvm.mma.xor.popc.m8n8k128.row.col.b1
    1 << 8 | 1, // llvm.nvvm.move.double
    1 << 8 | 1, // llvm.nvvm.move.float
    1 << 8 | 1, // llvm.nvvm.move.i16
    1 << 8 | 1, // llvm.nvvm.move.i32
    1 << 8 | 1, // llvm.nvvm.move.i64
    1 << 8 | 1, // llvm.nvvm.move.ptr
    0 << 8 | 1, // llvm.nvvm.mul.rm.d
    0 << 8 | 1, // llvm.nvvm.mul.rm.f
    0 << 8 | 1, // llvm.nvvm.mul.rm.ftz.f
    0 << 8 | 1, // llvm.nvvm.mul.rn.d
    0 << 8 | 1, // llvm.nvvm.mul.rn.f
    0 << 8 | 1, // llvm.nvvm.mul.rn.ftz.f
    0 << 8 | 1, // llvm.nvvm.mul.rp.d
    0 << 8 | 1, // llvm.nvvm.mul.rp.f
    0 << 8 | 1, // llvm.nvvm.mul.rp.ftz.f
    0 << 8 | 1, // llvm.nvvm.mul.rz.d
    0 << 8 | 1, // llvm.nvvm.mul.rz.f
    0 << 8 | 1, // llvm.nvvm.mul.rz.ftz.f
    0 << 8 | 1, // llvm.nvvm.mul24.i
    0 << 8 | 1, // llvm.nvvm.mul24.ui
    0 << 8 | 1, // llvm.nvvm.mulhi.i
    0 << 8 | 1, // llvm.nvvm.mulhi.ll
    0 << 8 | 1, // llvm.nvvm.mulhi.s
    0 << 8 | 1, // llvm.nvvm.mulhi.ui
    0 << 8 | 1, // llvm.nvvm.mulhi.ull
    0 << 8 | 1, // llvm.nvvm.mulhi.us
    94 << 8 | 1, // llvm.nvvm.nanosleep
    0 << 8 | 1, // llvm.nvvm.neg.bf16
    0 << 8 | 1, // llvm.nvvm.neg.bf16x2
    94 << 8 | 5, // llvm.nvvm.pm.event.mask
    18 << 8 | 7, // llvm.nvvm.prefetch.L1
    18 << 8 | 7, // llvm.nvvm.prefetch.L2
    18 << 8 | 7, // llvm.nvvm.prefetch.global.L1
    18 << 8 | 7, // llvm.nvvm.prefetch.global.L2
    18 << 8 | 7, // llvm.nvvm.prefetch.global.L2.evict.last
    18 << 8 | 7, // llvm.nvvm.prefetch.global.L2.evict.normal
    18 << 8 | 7, // llvm.nvvm.prefetch.local.L1
    18 << 8 | 7, // llvm.nvvm.prefetch.local.L2
    18 << 8 | 7, // llvm.nvvm.prefetch.tensormap
    18 << 8 | 7, // llvm.nvvm.prefetchu.L1
    0 << 8 | 1, // llvm.nvvm.prmt
    0 << 8 | 1, // llvm.nvvm.prmt.b4e
    0 << 8 | 1, // llvm.nvvm.prmt.ecl
    0 << 8 | 1, // llvm.nvvm.prmt.ecr
    0 << 8 | 1, // llvm.nvvm.prmt.f4e
    0 << 8 | 1, // llvm.nvvm.prmt.rc16
    0 << 8 | 1, // llvm.nvvm.prmt.rc8
    1 << 8 | 1, // llvm.nvvm.rcp.approx.ftz.d
    1 << 8 | 1, // llvm.nvvm.rcp.approx.ftz.f
    1 << 8 | 1, // llvm.nvvm.rcp.rm.d
    1 << 8 | 1, // llvm.nvvm.rcp.rm.f
    1 << 8 | 1, // llvm.nvvm.rcp.rm.ftz.f
    1 << 8 | 1, // llvm.nvvm.rcp.rn.d
    1 << 8 | 1, // llvm.nvvm.rcp.rn.f
    1 << 8 | 1, // llvm.nvvm.rcp.rn.ftz.f
    1 << 8 | 1, // llvm.nvvm.rcp.rp.d
    1 << 8 | 1, // llvm.nvvm.rcp.rp.f
    1 << 8 | 1, // llvm.nvvm.rcp.rp.ftz.f
    1 << 8 | 1, // llvm.nvvm.rcp.rz.d
    1 << 8 | 1, // llvm.nvvm.rcp.rz.f
    1 << 8 | 1, // llvm.nvvm.rcp.rz.ftz.f
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.aggr_smem_size
    70 << 8 | 2, // llvm.nvvm.read.ptx.sreg.clock
    70 << 8 | 2, // llvm.nvvm.read.ptx.sreg.clock64
    0 << 8 | 181, // llvm.nvvm.read.ptx.sreg.cluster.ctaid.w
    0 << 8 | 182, // llvm.nvvm.read.ptx.sreg.cluster.ctaid.x
    0 << 8 | 183, // llvm.nvvm.read.ptx.sreg.cluster.ctaid.y
    0 << 8 | 183, // llvm.nvvm.read.ptx.sreg.cluster.ctaid.z
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.cluster.ctarank
    0 << 8 | 181, // llvm.nvvm.read.ptx.sreg.cluster.nctaid.w
    0 << 8 | 184, // llvm.nvvm.read.ptx.sreg.cluster.nctaid.x
    0 << 8 | 185, // llvm.nvvm.read.ptx.sreg.cluster.nctaid.y
    0 << 8 | 185, // llvm.nvvm.read.ptx.sreg.cluster.nctaid.z
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.cluster.nctarank
    0 << 8 | 181, // llvm.nvvm.read.ptx.sreg.clusterid.w
    0 << 8 | 182, // llvm.nvvm.read.ptx.sreg.clusterid.x
    0 << 8 | 183, // llvm.nvvm.read.ptx.sreg.clusterid.y
    0 << 8 | 183, // llvm.nvvm.read.ptx.sreg.clusterid.z
    0 << 8 | 181, // llvm.nvvm.read.ptx.sreg.ctaid.w
    0 << 8 | 182, // llvm.nvvm.read.ptx.sreg.ctaid.x
    0 << 8 | 183, // llvm.nvvm.read.ptx.sreg.ctaid.y
    0 << 8 | 183, // llvm.nvvm.read.ptx.sreg.ctaid.z
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.dynamic_smem_size
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg0
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg1
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg10
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg11
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg12
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg13
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg14
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg15
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg16
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg17
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg18
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg19
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg2
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg20
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg21
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg22
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg23
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg24
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg25
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg26
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg27
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg28
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg29
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg3
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg30
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg31
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg4
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg5
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg6
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg7
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg8
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.envreg9
    70 << 8 | 2, // llvm.nvvm.read.ptx.sreg.globaltimer
    70 << 8 | 2, // llvm.nvvm.read.ptx.sreg.globaltimer.lo
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.gridid
    0 << 8 | 186, // llvm.nvvm.read.ptx.sreg.laneid
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.lanemask.eq
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.lanemask.ge
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.lanemask.gt
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.lanemask.le
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.lanemask.lt
    0 << 8 | 181, // llvm.nvvm.read.ptx.sreg.nclusterid.w
    0 << 8 | 184, // llvm.nvvm.read.ptx.sreg.nclusterid.x
    0 << 8 | 185, // llvm.nvvm.read.ptx.sreg.nclusterid.y
    0 << 8 | 185, // llvm.nvvm.read.ptx.sreg.nclusterid.z
    0 << 8 | 181, // llvm.nvvm.read.ptx.sreg.nctaid.w
    0 << 8 | 184, // llvm.nvvm.read.ptx.sreg.nctaid.x
    0 << 8 | 185, // llvm.nvvm.read.ptx.sreg.nctaid.y
    0 << 8 | 185, // llvm.nvvm.read.ptx.sreg.nctaid.z
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.nsmid
    0 << 8 | 181, // llvm.nvvm.read.ptx.sreg.ntid.w
    0 << 8 | 187, // llvm.nvvm.read.ptx.sreg.ntid.x
    0 << 8 | 187, // llvm.nvvm.read.ptx.sreg.ntid.y
    0 << 8 | 188, // llvm.nvvm.read.ptx.sreg.ntid.z
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.nwarpid
    70 << 8 | 2, // llvm.nvvm.read.ptx.sreg.pm0
    70 << 8 | 2, // llvm.nvvm.read.ptx.sreg.pm1
    70 << 8 | 2, // llvm.nvvm.read.ptx.sreg.pm2
    70 << 8 | 2, // llvm.nvvm.read.ptx.sreg.pm3
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.smid
    0 << 8 | 181, // llvm.nvvm.read.ptx.sreg.tid.w
    0 << 8 | 128, // llvm.nvvm.read.ptx.sreg.tid.x
    0 << 8 | 128, // llvm.nvvm.read.ptx.sreg.tid.y
    0 << 8 | 189, // llvm.nvvm.read.ptx.sreg.tid.z
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.total_smem_size
    0 << 8 | 2, // llvm.nvvm.read.ptx.sreg.warpid
    0 << 8 | 190, // llvm.nvvm.read.ptx.sreg.warpsize
    89 << 8 | 1, // llvm.nvvm.redux.sync.add
    89 << 8 | 1, // llvm.nvvm.redux.sync.and
    89 << 8 | 1, // llvm.nvvm.redux.sync.fmax
    89 << 8 | 1, // llvm.nvvm.redux.sync.fmax.NaN
    89 << 8 | 1, // llvm.nvvm.redux.sync.fmax.abs
    89 << 8 | 1, // llvm.nvvm.redux.sync.fmax.abs.NaN
    89 << 8 | 1, // llvm.nvvm.redux.sync.fmin
    89 << 8 | 1, // llvm.nvvm.redux.sync.fmin.NaN
    89 << 8 | 1, // llvm.nvvm.redux.sync.fmin.abs
    89 << 8 | 1, // llvm.nvvm.redux.sync.fmin.abs.NaN
    89 << 8 | 1, // llvm.nvvm.redux.sync.max
    89 << 8 | 1, // llvm.nvvm.redux.sync.min
    89 << 8 | 1, // llvm.nvvm.redux.sync.or
    89 << 8 | 1, // llvm.nvvm.redux.sync.umax
    89 << 8 | 1, // llvm.nvvm.redux.sync.umin
    89 << 8 | 1, // llvm.nvvm.redux.sync.xor
    0 << 8 | 1, // llvm.nvvm.reflect
    0 << 8 | 1, // llvm.nvvm.round.d
    0 << 8 | 1, // llvm.nvvm.round.f
    0 << 8 | 1, // llvm.nvvm.round.ftz.f
    1 << 8 | 1, // llvm.nvvm.rsqrt.approx.d
    1 << 8 | 1, // llvm.nvvm.rsqrt.approx.f
    1 << 8 | 1, // llvm.nvvm.rsqrt.approx.ftz.d
    1 << 8 | 1, // llvm.nvvm.rsqrt.approx.ftz.f
    0 << 8 | 1, // llvm.nvvm.sad.i
    0 << 8 | 1, // llvm.nvvm.sad.ll
    0 << 8 | 1, // llvm.nvvm.sad.s
    0 << 8 | 1, // llvm.nvvm.sad.ui
    0 << 8 | 1, // llvm.nvvm.sad.ull
    0 << 8 | 1, // llvm.nvvm.sad.us
    0 << 8 | 1, // llvm.nvvm.saturate.d
    0 << 8 | 1, // llvm.nvvm.saturate.f
    0 << 8 | 1, // llvm.nvvm.saturate.ftz.f
    94 << 8 | 191, // llvm.nvvm.setmaxnreg.dec.sync.aligned.u32
    94 << 8 | 191, // llvm.nvvm.setmaxnreg.inc.sync.aligned.u32
    0 << 8 | 1, // llvm.nvvm.sext.clamp
    0 << 8 | 1, // llvm.nvvm.sext.wrap
    89 << 8 | 1, // llvm.nvvm.shfl.bfly.f32
    89 << 8 | 1, // llvm.nvvm.shfl.bfly.f32p
    89 << 8 | 1, // llvm.nvvm.shfl.bfly.i32
    89 << 8 | 1, // llvm.nvvm.shfl.bfly.i32p
    89 << 8 | 1, // llvm.nvvm.shfl.down.f32
    89 << 8 | 1, // llvm.nvvm.shfl.down.f32p
    89 << 8 | 1, // llvm.nvvm.shfl.down.i32
    89 << 8 | 1, // llvm.nvvm.shfl.down.i32p
    89 << 8 | 1, // llvm.nvvm.shfl.idx.f32
    89 << 8 | 1, // llvm.nvvm.shfl.idx.f32p
    89 << 8 | 1, // llvm.nvvm.shfl.idx.i32
    89 << 8 | 1, // llvm.nvvm.shfl.idx.i32p
    89 << 8 | 1, // llvm.nvvm.shfl.sync.bfly.f32
    89 << 8 | 1, // llvm.nvvm.shfl.sync.bfly.f32p
    89 << 8 | 1, // llvm.nvvm.shfl.sync.bfly.i32
    89 << 8 | 1, // llvm.nvvm.shfl.sync.bfly.i32p
    89 << 8 | 1, // llvm.nvvm.shfl.sync.down.f32
    89 << 8 | 1, // llvm.nvvm.shfl.sync.down.f32p
    89 << 8 | 1, // llvm.nvvm.shfl.sync.down.i32
    89 << 8 | 1, // llvm.nvvm.shfl.sync.down.i32p
    89 << 8 | 1, // llvm.nvvm.shfl.sync.idx.f32
    89 << 8 | 1, // llvm.nvvm.shfl.sync.idx.f32p
    89 << 8 | 1, // llvm.nvvm.shfl.sync.idx.i32
    89 << 8 | 1, // llvm.nvvm.shfl.sync.idx.i32p
    89 << 8 | 1, // llvm.nvvm.shfl.sync.up.f32
    89 << 8 | 1, // llvm.nvvm.shfl.sync.up.f32p
    89 << 8 | 1, // llvm.nvvm.shfl.sync.up.i32
    89 << 8 | 1, // llvm.nvvm.shfl.sync.up.i32p
    89 << 8 | 1, // llvm.nvvm.shfl.up.f32
    89 << 8 | 1, // llvm.nvvm.shfl.up.f32p
    89 << 8 | 1, // llvm.nvvm.shfl.up.i32
    89 << 8 | 1, // llvm.nvvm.shfl.up.i32p
    1 << 8 | 1, // llvm.nvvm.sin.approx.f
    1 << 8 | 1, // llvm.nvvm.sin.approx.ftz.f
    1 << 8 | 1, // llvm.nvvm.sqrt.approx.f
    1 << 8 | 1, // llvm.nvvm.sqrt.approx.ftz.f
    1 << 8 | 1, // llvm.nvvm.sqrt.f
    1 << 8 | 1, // llvm.nvvm.sqrt.rm.d
    1 << 8 | 1, // llvm.nvvm.sqrt.rm.f
    1 << 8 | 1, // llvm.nvvm.sqrt.rm.ftz.f
    1 << 8 | 1, // llvm.nvvm.sqrt.rn.d
    1 << 8 | 1, // llvm.nvvm.sqrt.rn.f
    1 << 8 | 1, // llvm.nvvm.sqrt.rn.ftz.f
    1 << 8 | 1, // llvm.nvvm.sqrt.rp.d
    1 << 8 | 1, // llvm.nvvm.sqrt.rp.f
    1 << 8 | 1, // llvm.nvvm.sqrt.rp.ftz.f
    1 << 8 | 1, // llvm.nvvm.sqrt.rz.d
    1 << 8 | 1, // llvm.nvvm.sqrt.rz.f
    1 << 8 | 1, // llvm.nvvm.sqrt.rz.ftz.f
    19 << 8 | 52, // llvm.nvvm.st.bulk
    19 << 8 | 52, // llvm.nvvm.st.bulk.shared.cta
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m16n8.x1.trans.b8
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m16n8.x2.trans.b8
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m16n8.x4.trans.b8
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x1.b16
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x1.trans.b16
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x2.b16
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x2.trans.b16
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x4.b16
    91 << 8 | 36, // llvm.nvvm.stmatrix.sync.aligned.m8n8.x4.trans.b16
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v2i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.array.v4i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.v2i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.v2i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.v2i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.v2i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.v2i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.v2i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.v2i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.v2i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.v2i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.v2i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.v2i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.v2i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.v4i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.v4i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.v4i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.v4i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.v4i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.v4i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.1d.v4i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.1d.v4i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.1d.v4i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v2i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.array.v4i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.v2i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.v2i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.v2i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.v2i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.v2i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.v2i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.v2i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.v2i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.v2i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.v2i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.v2i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.v2i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.v4i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.v4i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.v4i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.v4i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.v4i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.v4i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.2d.v4i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.2d.v4i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.2d.v4i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.v2i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.v2i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.v2i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.v2i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.v2i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.v2i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.v2i64.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.v2i64.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.v2i64.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.v2i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.v2i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.v2i8.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.v4i16.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.v4i16.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.v4i16.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.v4i32.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.v4i32.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.v4i32.zero
    59 << 8 | 1, // llvm.nvvm.suld.3d.v4i8.clamp
    95 << 8 | 1, // llvm.nvvm.suld.3d.v4i8.trap
    59 << 8 | 1, // llvm.nvvm.suld.3d.v4i8.zero
    1 << 8 | 1, // llvm.nvvm.suq.array.size
    1 << 8 | 1, // llvm.nvvm.suq.channel.data.type
    1 << 8 | 1, // llvm.nvvm.suq.channel.order
    1 << 8 | 1, // llvm.nvvm.suq.depth
    1 << 8 | 1, // llvm.nvvm.suq.height
    1 << 8 | 1, // llvm.nvvm.suq.width
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v2i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.array.v4i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v2i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.1d.v4i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v2i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.array.v4i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v2i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.2d.v4i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i64.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i64.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i64.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v2i8.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i16.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i16.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i16.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i32.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i32.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i32.zero
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i8.clamp
    97 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i8.trap
    96 << 8 | 1, // llvm.nvvm.sust.b.3d.v4i8.zero
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.v2i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.v2i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.v2i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.v4i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.v4i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.array.v4i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.v2i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.v2i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.v2i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.v4i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.v4i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.1d.v4i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.v2i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.v2i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.v2i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.v4i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.v4i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.array.v4i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.v2i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.v2i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.v2i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.v4i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.v4i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.2d.v4i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.v2i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.v2i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.v2i8.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.v4i16.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.v4i32.trap
    97 << 8 | 1, // llvm.nvvm.sust.p.3d.v4i8.trap
    98 << 8 | 36, // llvm.nvvm.tcgen05.alloc.cg1
    98 << 8 | 36, // llvm.nvvm.tcgen05.alloc.cg2
    98 << 8 | 36, // llvm.nvvm.tcgen05.alloc.shared.cg1
    98 << 8 | 36, // llvm.nvvm.tcgen05.alloc.shared.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.commit.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.commit.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.commit.mc.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.commit.mc.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.commit.mc.shared.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.commit.mc.shared.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.commit.shared.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.commit.shared.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x128b.b4x16_p64.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x128b.b4x16_p64.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x128b.b6x16_p32.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x128b.b6x16_p32.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x128b.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x128b.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x256b.b4x16_p64.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x256b.b4x16_p64.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x256b.b6x16_p32.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x256b.b6x16_p32.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x256b.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.128x256b.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.b4x16_p64.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.b4x16_p64.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.b6x16_p32.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.b6x16_p32.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.32x128b_warpx4.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.4x256b.b4x16_p64.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.4x256b.b4x16_p64.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.4x256b.b6x16_p32.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.4x256b.b6x16_p32.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.4x256b.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.4x256b.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b4x16_p64.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b4x16_p64.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b6x16_p32.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.b6x16_p32.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_01_23.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b4x16_p64.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b4x16_p64.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b6x16_p32.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.b6x16_p32.cg2
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.cg1
    98 << 8 | 10, // llvm.nvvm.tcgen05.cp.64x128b_warpx2_02_13.cg2
    99 << 8 | 10, // llvm.nvvm.tcgen05.dealloc.cg1
    99 << 8 | 10, // llvm.nvvm.tcgen05.dealloc.cg2
    29 << 8 | 1, // llvm.nvvm.tcgen05.fence.after.thread.sync
    29 << 8 | 1, // llvm.nvvm.tcgen05.fence.before.thread.sync
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x128b.x1
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x128b.x16
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x128b.x2
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x128b.x32
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x128b.x4
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x128b.x64
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x128b.x8
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x256b.x1
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x256b.x16
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x256b.x2
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x256b.x32
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x256b.x4
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x256b.x8
    99 << 8 | 192, // llvm.nvvm.tcgen05.ld.16x32bx2.x1
    99 << 8 | 192, // llvm.nvvm.tcgen05.ld.16x32bx2.x128
    99 << 8 | 192, // llvm.nvvm.tcgen05.ld.16x32bx2.x16
    99 << 8 | 192, // llvm.nvvm.tcgen05.ld.16x32bx2.x2
    99 << 8 | 192, // llvm.nvvm.tcgen05.ld.16x32bx2.x32
    99 << 8 | 192, // llvm.nvvm.tcgen05.ld.16x32bx2.x4
    99 << 8 | 192, // llvm.nvvm.tcgen05.ld.16x32bx2.x64
    99 << 8 | 192, // llvm.nvvm.tcgen05.ld.16x32bx2.x8
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x64b.x1
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x64b.x128
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x64b.x16
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x64b.x2
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x64b.x32
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x64b.x4
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x64b.x64
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.16x64b.x8
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.32x32b.x1
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.32x32b.x128
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.32x32b.x16
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.32x32b.x2
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.32x32b.x32
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.32x32b.x4
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.32x32b.x64
    99 << 8 | 64, // llvm.nvvm.tcgen05.ld.32x32b.x8
    18 << 8 | 193, // llvm.nvvm.tcgen05.mma.shared
    18 << 8 | 194, // llvm.nvvm.tcgen05.mma.shared.disable_output_lane.cg1
    18 << 8 | 194, // llvm.nvvm.tcgen05.mma.shared.disable_output_lane.cg2
    18 << 8 | 195, // llvm.nvvm.tcgen05.mma.shared.mxf4.block_scale
    18 << 8 | 195, // llvm.nvvm.tcgen05.mma.shared.mxf4.block_scale.block32
    18 << 8 | 195, // llvm.nvvm.tcgen05.mma.shared.mxf4nvf4.block_scale.block16
    18 << 8 | 195, // llvm.nvvm.tcgen05.mma.shared.mxf4nvf4.block_scale.block32
    18 << 8 | 195, // llvm.nvvm.tcgen05.mma.shared.mxf8f6f4.block_scale
    18 << 8 | 195, // llvm.nvvm.tcgen05.mma.shared.mxf8f6f4.block_scale.block32
    18 << 8 | 196, // llvm.nvvm.tcgen05.mma.shared.scale_d
    18 << 8 | 197, // llvm.nvvm.tcgen05.mma.shared.scale_d.disable_output_lane.cg1
    18 << 8 | 197, // llvm.nvvm.tcgen05.mma.shared.scale_d.disable_output_lane.cg2
    18 << 8 | 198, // llvm.nvvm.tcgen05.mma.sp.shared
    18 << 8 | 199, // llvm.nvvm.tcgen05.mma.sp.shared.disable_output_lane.cg1
    18 << 8 | 199, // llvm.nvvm.tcgen05.mma.sp.shared.disable_output_lane.cg2
    18 << 8 | 200, // llvm.nvvm.tcgen05.mma.sp.shared.mxf4.block_scale
    18 << 8 | 200, // llvm.nvvm.tcgen05.mma.sp.shared.mxf4.block_scale.block32
    18 << 8 | 200, // llvm.nvvm.tcgen05.mma.sp.shared.mxf4nvf4.block_scale.block16
    18 << 8 | 200, // llvm.nvvm.tcgen05.mma.sp.shared.mxf4nvf4.block_scale.block32
    18 << 8 | 200, // llvm.nvvm.tcgen05.mma.sp.shared.mxf8f6f4.block_scale
    18 << 8 | 200, // llvm.nvvm.tcgen05.mma.sp.shared.mxf8f6f4.block_scale.block32
    18 << 8 | 201, // llvm.nvvm.tcgen05.mma.sp.shared.scale_d
    18 << 8 | 202, // llvm.nvvm.tcgen05.mma.sp.shared.scale_d.disable_output_lane.cg1
    18 << 8 | 202, // llvm.nvvm.tcgen05.mma.sp.shared.scale_d.disable_output_lane.cg2
    18 << 8 | 203, // llvm.nvvm.tcgen05.mma.sp.tensor
    18 << 8 | 204, // llvm.nvvm.tcgen05.mma.sp.tensor.ashift
    18 << 8 | 205, // llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg1
    18 << 8 | 206, // llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg1.ashift
    18 << 8 | 205, // llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg2
    18 << 8 | 206, // llvm.nvvm.tcgen05.mma.sp.tensor.disable_output_lane.cg2.ashift
    18 << 8 | 207, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf4.block_scale
    18 << 8 | 207, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf4.block_scale.block32
    18 << 8 | 207, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf4nvf4.block_scale.block16
    18 << 8 | 207, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf4nvf4.block_scale.block32
    18 << 8 | 207, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf8f6f4.block_scale
    18 << 8 | 207, // llvm.nvvm.tcgen05.mma.sp.tensor.mxf8f6f4.block_scale.block32
    18 << 8 | 208, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d
    18 << 8 | 209, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.ashift
    18 << 8 | 210, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg1
    18 << 8 | 211, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg1.ashift
    18 << 8 | 210, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg2
    18 << 8 | 211, // llvm.nvvm.tcgen05.mma.sp.tensor.scale_d.disable_output_lane.cg2.ashift
    18 << 8 | 212, // llvm.nvvm.tcgen05.mma.tensor
    18 << 8 | 213, // llvm.nvvm.tcgen05.mma.tensor.ashift
    18 << 8 | 214, // llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg1
    18 << 8 | 215, // llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg1.ashift
    18 << 8 | 214, // llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg2
    18 << 8 | 215, // llvm.nvvm.tcgen05.mma.tensor.disable_output_lane.cg2.ashift
    18 << 8 | 216, // llvm.nvvm.tcgen05.mma.tensor.mxf4.block_scale
    18 << 8 | 216, // llvm.nvvm.tcgen05.mma.tensor.mxf4.block_scale.block32
    18 << 8 | 216, // llvm.nvvm.tcgen05.mma.tensor.mxf4nvf4.block_scale.block16
    18 << 8 | 216, // llvm.nvvm.tcgen05.mma.tensor.mxf4nvf4.block_scale.block32
    18 << 8 | 216, // llvm.nvvm.tcgen05.mma.tensor.mxf8f6f4.block_scale
    18 << 8 | 216, // llvm.nvvm.tcgen05.mma.tensor.mxf8f6f4.block_scale.block32
    18 << 8 | 217, // llvm.nvvm.tcgen05.mma.tensor.scale_d
    18 << 8 | 218, // llvm.nvvm.tcgen05.mma.tensor.scale_d.ashift
    18 << 8 | 219, // llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg1
    18 << 8 | 220, // llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg1.ashift
    18 << 8 | 219, // llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg2
    18 << 8 | 220, // llvm.nvvm.tcgen05.mma.tensor.scale_d.disable_output_lane.cg2.ashift
    18 << 8 | 221, // llvm.nvvm.tcgen05.mma.ws.shared
    18 << 8 | 222, // llvm.nvvm.tcgen05.mma.ws.shared.zero_col_mask
    18 << 8 | 222, // llvm.nvvm.tcgen05.mma.ws.sp.shared
    18 << 8 | 223, // llvm.nvvm.tcgen05.mma.ws.sp.shared.zero_col_mask
    18 << 8 | 224, // llvm.nvvm.tcgen05.mma.ws.sp.tensor
    18 << 8 | 225, // llvm.nvvm.tcgen05.mma.ws.sp.tensor.zero_col_mask
    18 << 8 | 226, // llvm.nvvm.tcgen05.mma.ws.tensor
    18 << 8 | 224, // llvm.nvvm.tcgen05.mma.ws.tensor.zero_col_mask
    100 << 8 | 1, // llvm.nvvm.tcgen05.relinq.alloc.permit.cg1
    100 << 8 | 1, // llvm.nvvm.tcgen05.relinq.alloc.permit.cg2
    99 << 8 | 10, // llvm.nvvm.tcgen05.shift.down.cg1
    99 << 8 | 10, // llvm.nvvm.tcgen05.shift.down.cg2
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x128b.x1
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x128b.x16
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x128b.x2
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x128b.x32
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x128b.x4
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x128b.x64
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x128b.x8
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x256b.x1
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x256b.x16
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x256b.x2
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x256b.x32
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x256b.x4
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x256b.x8
    99 << 8 | 228, // llvm.nvvm.tcgen05.st.16x32bx2.x1
    99 << 8 | 228, // llvm.nvvm.tcgen05.st.16x32bx2.x128
    99 << 8 | 228, // llvm.nvvm.tcgen05.st.16x32bx2.x16
    99 << 8 | 228, // llvm.nvvm.tcgen05.st.16x32bx2.x2
    99 << 8 | 228, // llvm.nvvm.tcgen05.st.16x32bx2.x32
    99 << 8 | 228, // llvm.nvvm.tcgen05.st.16x32bx2.x4
    99 << 8 | 228, // llvm.nvvm.tcgen05.st.16x32bx2.x64
    99 << 8 | 228, // llvm.nvvm.tcgen05.st.16x32bx2.x8
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x64b.x1
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x64b.x128
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x64b.x16
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x64b.x2
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x64b.x32
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x64b.x4
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x64b.x64
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.16x64b.x8
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.32x32b.x1
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.32x32b.x128
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.32x32b.x16
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.32x32b.x2
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.32x32b.x32
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.32x32b.x4
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.32x32b.x64
    99 << 8 | 227, // llvm.nvvm.tcgen05.st.32x32b.x8
    100 << 8 | 1, // llvm.nvvm.tcgen05.wait.ld
    100 << 8 | 1, // llvm.nvvm.tcgen05.wait.st
    19 << 8 | 64, // llvm.nvvm.tensormap.replace.box.dim
    19 << 8 | 64, // llvm.nvvm.tensormap.replace.element.stride
    19 << 8 | 229, // llvm.nvvm.tensormap.replace.elemtype
    19 << 8 | 230, // llvm.nvvm.tensormap.replace.fill.mode
    19 << 8 | 10, // llvm.nvvm.tensormap.replace.global.address
    19 << 8 | 64, // llvm.nvvm.tensormap.replace.global.dim
    19 << 8 | 64, // llvm.nvvm.tensormap.replace.global.stride
    19 << 8 | 231, // llvm.nvvm.tensormap.replace.interleave.layout
    19 << 8 | 10, // llvm.nvvm.tensormap.replace.rank
    19 << 8 | 232, // llvm.nvvm.tensormap.replace.swizzle.atomicity
    19 << 8 | 233, // llvm.nvvm.tensormap.replace.swizzle.mode
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.array.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.1d.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.1d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.1d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.1d.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.array.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.2d.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.2d.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.3d.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.3d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.3d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.3d.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.cube.array.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.array.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.array.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.array.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.array.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.array.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.cube.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.array.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.1d.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.array.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.2d.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.v4f32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.v4s32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.3d.v4u32.s32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.array.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.grad.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.grad.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.grad.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.level.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.level.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.level.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tex.unified.cube.v4u32.f32
    0 << 8 | 1, // llvm.nvvm.texsurf.handle
    0 << 8 | 1, // llvm.nvvm.texsurf.handle.internal
    59 << 8 | 1, // llvm.nvvm.tld4.a.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.a.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.a.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.b.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.b.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.b.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.g.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.g.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.g.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.r.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.r.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.r.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.a.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.a.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.a.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.b.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.b.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.b.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.g.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.g.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.g.2d.v4u32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.r.2d.v4f32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.r.2d.v4s32.f32
    59 << 8 | 1, // llvm.nvvm.tld4.unified.r.2d.v4u32.f32
    0 << 8 | 1, // llvm.nvvm.trunc.d
    0 << 8 | 1, // llvm.nvvm.trunc.f
    0 << 8 | 1, // llvm.nvvm.trunc.ftz.f
    1 << 8 | 1, // llvm.nvvm.txq.array.size
    1 << 8 | 1, // llvm.nvvm.txq.channel.data.type
    1 << 8 | 1, // llvm.nvvm.txq.channel.order
    1 << 8 | 1, // llvm.nvvm.txq.depth
    1 << 8 | 1, // llvm.nvvm.txq.height
    1 << 8 | 1, // llvm.nvvm.txq.num.mipmap.levels
    1 << 8 | 1, // llvm.nvvm.txq.num.samples
    1 << 8 | 1, // llvm.nvvm.txq.width
    0 << 8 | 1, // llvm.nvvm.ue8m0x2.to.bf16x2
    0 << 8 | 1, // llvm.nvvm.ui2d.rm
    0 << 8 | 1, // llvm.nvvm.ui2d.rn
    0 << 8 | 1, // llvm.nvvm.ui2d.rp
    0 << 8 | 1, // llvm.nvvm.ui2d.rz
    0 << 8 | 1, // llvm.nvvm.ui2f.rm
    0 << 8 | 1, // llvm.nvvm.ui2f.rn
    0 << 8 | 1, // llvm.nvvm.ui2f.rp
    0 << 8 | 1, // llvm.nvvm.ui2f.rz
    0 << 8 | 1, // llvm.nvvm.ull2d.rm
    0 << 8 | 1, // llvm.nvvm.ull2d.rn
    0 << 8 | 1, // llvm.nvvm.ull2d.rp
    0 << 8 | 1, // llvm.nvvm.ull2d.rz
    0 << 8 | 1, // llvm.nvvm.ull2f.rm
    0 << 8 | 1, // llvm.nvvm.ull2f.rn
    0 << 8 | 1, // llvm.nvvm.ull2f.rp
    0 << 8 | 1, // llvm.nvvm.ull2f.rz
    89 << 8 | 1, // llvm.nvvm.vote.all
    89 << 8 | 1, // llvm.nvvm.vote.all.sync
    89 << 8 | 1, // llvm.nvvm.vote.any
    89 << 8 | 1, // llvm.nvvm.vote.any.sync
    89 << 8 | 1, // llvm.nvvm.vote.ballot
    89 << 8 | 1, // llvm.nvvm.vote.ballot.sync
    89 << 8 | 1, // llvm.nvvm.vote.uni
    89 << 8 | 1, // llvm.nvvm.vote.uni.sync
    40 << 8 | 1, // llvm.nvvm.wgmma.commit_group.sync.aligned
    40 << 8 | 1, // llvm.nvvm.wgmma.fence.sync.aligned
    40 << 8 | 5, // llvm.nvvm.wgmma.wait_group.sync.aligned
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.col.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.col.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.col.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.col.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.col.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.col.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.row.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.row.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.row.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.row.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.row.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.a.row.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.col.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.col.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.col.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.col.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.col.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.col.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.row.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.row.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.row.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.row.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.row.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.b.row.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.col.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.col.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.col.stride.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.col.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.row.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.row.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.row.stride.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k16.load.c.row.stride.s32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.col.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.col.row.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.col.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k16.mma.row.row.u8.satfinite
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.col.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.col.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.col.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.col.stride.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.col.stride.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.col.stride.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.row.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.row.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.row.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.row.stride.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.row.stride.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k16.store.d.row.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.a.col.stride.tf32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.a.col.tf32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.a.row.stride.tf32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.a.row.tf32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.b.col.stride.tf32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.b.col.tf32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.b.row.stride.tf32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.b.row.tf32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.c.col.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.c.col.stride.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.c.row.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m16n16k8.load.c.row.stride.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k8.mma.col.col.tf32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k8.mma.col.row.tf32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k8.mma.row.col.tf32
    93 << 8 | 1, // llvm.nvvm.wmma.m16n16k8.mma.row.row.tf32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k8.store.d.col.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k8.store.d.col.stride.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k8.store.d.row.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m16n16k8.store.d.row.stride.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.col.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.col.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.col.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.col.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.col.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.col.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.row.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.row.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.row.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.row.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.row.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.a.row.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.col.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.col.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.col.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.col.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.col.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.col.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.row.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.row.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.row.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.row.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.row.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.b.row.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.col.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.col.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.col.stride.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.col.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.row.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.row.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.row.stride.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m32n8k16.load.c.row.stride.s32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.col.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.col.row.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.col.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m32n8k16.mma.row.row.u8.satfinite
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.col.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.col.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.col.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.col.stride.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.col.stride.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.col.stride.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.row.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.row.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.row.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.row.stride.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.row.stride.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m32n8k16.store.d.row.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.col.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.col.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.col.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.col.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.col.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.col.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.row.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.row.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.row.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.row.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.row.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.a.row.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.col.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.col.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.col.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.col.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.col.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.col.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.row.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.row.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.row.stride.bf16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.row.stride.s8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.row.stride.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.b.row.u8
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.col.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.col.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.col.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.col.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.col.stride.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.col.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.row.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.row.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.row.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.row.stride.f16
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.row.stride.f32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n32k16.load.c.row.stride.s32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.col.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.col.row.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.col.u8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.bf16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f16.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f16
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f16.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.f32.f32.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.s8
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.s8.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.u8
    93 << 8 | 1, // llvm.nvvm.wmma.m8n32k16.mma.row.row.u8.satfinite
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.col.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.col.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.col.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.col.stride.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.col.stride.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.col.stride.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.row.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.row.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.row.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.row.stride.f16
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.row.stride.f32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n32k16.store.d.row.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k128.load.a.row.b1
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k128.load.a.row.stride.b1
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k128.load.b.col.b1
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k128.load.b.col.stride.b1
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k128.load.c.col.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k128.load.c.col.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k128.load.c.row.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k128.load.c.row.stride.s32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k128.mma.and.popc.row.col.b1
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k128.mma.xor.popc.row.col.b1
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k128.store.d.col.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k128.store.d.col.stride.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k128.store.d.row.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k128.store.d.row.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.a.row.s4
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.a.row.stride.s4
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.a.row.stride.u4
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.a.row.u4
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.b.col.s4
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.b.col.stride.s4
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.b.col.stride.u4
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.b.col.u4
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.c.col.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.c.col.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.c.row.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k32.load.c.row.stride.s32
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k32.mma.row.col.s4
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k32.mma.row.col.s4.satfinite
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k32.mma.row.col.u4
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k32.mma.row.col.u4.satfinite
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k32.store.d.col.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k32.store.d.col.stride.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k32.store.d.row.s32
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k32.store.d.row.stride.s32
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.a.col.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.a.col.stride.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.a.row.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.a.row.stride.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.b.col.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.b.col.stride.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.b.row.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.b.row.stride.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.c.col.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.c.col.stride.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.c.row.f64
    88 << 8 | 7, // llvm.nvvm.wmma.m8n8k4.load.c.row.stride.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.col.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.col.rm.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.col.rn.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.col.rp.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.col.rz.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.row.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.row.rm.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.row.rn.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.row.rp.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.col.row.rz.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.col.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.col.rm.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.col.rn.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.col.rp.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.col.rz.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.row.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.row.rm.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.row.rn.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.row.rp.f64
    93 << 8 | 1, // llvm.nvvm.wmma.m8n8k4.mma.row.row.rz.f64
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k4.store.d.col.f64
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k4.store.d.col.stride.f64
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k4.store.d.row.f64
    101 << 8 | 36, // llvm.nvvm.wmma.m8n8k4.store.d.row.stride.f64
    0 << 8 | 1, // llvm.nvvm.zext.clamp
    0 << 8 | 1, // llvm.nvvm.zext.wrap
    30 << 8 | 11, // llvm.ppc.addex
    1 << 8 | 1, // llvm.ppc.addf128.round.to.odd
    1 << 8 | 1, // llvm.ppc.addg6s
    1 << 8 | 1, // llvm.ppc.addg6sd
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vcipher
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vcipherlast
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vncipher
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vncipherlast
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vpermxor
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vpermxor.be
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vpmsumb
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vpmsumd
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vpmsumh
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vpmsumw
    1 << 8 | 1, // llvm.ppc.altivec.crypto.vsbox
    1 << 8 | 13, // llvm.ppc.altivec.crypto.vshasigmad
    1 << 8 | 13, // llvm.ppc.altivec.crypto.vshasigmaw
    5 << 8 | 1, // llvm.ppc.altivec.dss
    5 << 8 | 1, // llvm.ppc.altivec.dssall
    5 << 8 | 1, // llvm.ppc.altivec.dst
    5 << 8 | 1, // llvm.ppc.altivec.dstst
    5 << 8 | 1, // llvm.ppc.altivec.dststt
    5 << 8 | 1, // llvm.ppc.altivec.dstt
    2 << 8 | 1, // llvm.ppc.altivec.lvebx
    2 << 8 | 1, // llvm.ppc.altivec.lvehx
    2 << 8 | 1, // llvm.ppc.altivec.lvewx
    1 << 8 | 1, // llvm.ppc.altivec.lvsl
    1 << 8 | 1, // llvm.ppc.altivec.lvsr
    2 << 8 | 1, // llvm.ppc.altivec.lvx
    2 << 8 | 1, // llvm.ppc.altivec.lvxl
    29 << 8 | 1, // llvm.ppc.altivec.mfvscr
    29 << 8 | 1, // llvm.ppc.altivec.mtvscr
    1 << 8 | 1, // llvm.ppc.altivec.mtvsrbm
    1 << 8 | 1, // llvm.ppc.altivec.mtvsrdm
    1 << 8 | 1, // llvm.ppc.altivec.mtvsrhm
    1 << 8 | 1, // llvm.ppc.altivec.mtvsrqm
    1 << 8 | 1, // llvm.ppc.altivec.mtvsrwm
    19 << 8 | 1, // llvm.ppc.altivec.stvebx
    19 << 8 | 1, // llvm.ppc.altivec.stvehx
    19 << 8 | 1, // llvm.ppc.altivec.stvewx
    19 << 8 | 1, // llvm.ppc.altivec.stvx
    19 << 8 | 1, // llvm.ppc.altivec.stvxl
    1 << 8 | 1, // llvm.ppc.altivec.vabsdub
    1 << 8 | 1, // llvm.ppc.altivec.vabsduh
    1 << 8 | 1, // llvm.ppc.altivec.vabsduw
    1 << 8 | 1, // llvm.ppc.altivec.vaddcuq
    1 << 8 | 1, // llvm.ppc.altivec.vaddcuw
    1 << 8 | 1, // llvm.ppc.altivec.vaddecuq
    1 << 8 | 1, // llvm.ppc.altivec.vaddeuqm
    1 << 8 | 1, // llvm.ppc.altivec.vaddsbs
    1 << 8 | 1, // llvm.ppc.altivec.vaddshs
    1 << 8 | 1, // llvm.ppc.altivec.vaddsws
    1 << 8 | 1, // llvm.ppc.altivec.vaddubs
    1 << 8 | 1, // llvm.ppc.altivec.vadduhs
    1 << 8 | 1, // llvm.ppc.altivec.vadduws
    1 << 8 | 1, // llvm.ppc.altivec.vavgsb
    1 << 8 | 1, // llvm.ppc.altivec.vavgsh
    1 << 8 | 1, // llvm.ppc.altivec.vavgsw
    1 << 8 | 1, // llvm.ppc.altivec.vavgub
    1 << 8 | 1, // llvm.ppc.altivec.vavguh
    1 << 8 | 1, // llvm.ppc.altivec.vavguw
    1 << 8 | 1, // llvm.ppc.altivec.vbpermd
    1 << 8 | 1, // llvm.ppc.altivec.vbpermq
    1 << 8 | 0, // llvm.ppc.altivec.vcfsx
    1 << 8 | 1, // llvm.ppc.altivec.vcfuged
    1 << 8 | 0, // llvm.ppc.altivec.vcfux
    1 << 8 | 1, // llvm.ppc.altivec.vclrlb
    1 << 8 | 1, // llvm.ppc.altivec.vclrrb
    1 << 8 | 1, // llvm.ppc.altivec.vclzdm
    1 << 8 | 1, // llvm.ppc.altivec.vclzlsbb
    1 << 8 | 1, // llvm.ppc.altivec.vcmpbfp
    1 << 8 | 1, // llvm.ppc.altivec.vcmpbfp.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpeqfp
    1 << 8 | 1, // llvm.ppc.altivec.vcmpeqfp.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequb
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequb.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequd
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequd.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequh
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequh.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequq
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequq.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequw
    1 << 8 | 1, // llvm.ppc.altivec.vcmpequw.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgefp
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgefp.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtfp
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtfp.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsb
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsb.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsd
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsd.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsh
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsh.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsq
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsq.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsw
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtsw.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtub
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtub.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtud
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtud.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtuh
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtuh.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtuq
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtuq.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtuw
    1 << 8 | 1, // llvm.ppc.altivec.vcmpgtuw.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpneb
    1 << 8 | 1, // llvm.ppc.altivec.vcmpneb.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpneh
    1 << 8 | 1, // llvm.ppc.altivec.vcmpneh.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpnew
    1 << 8 | 1, // llvm.ppc.altivec.vcmpnew.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpnezb
    1 << 8 | 1, // llvm.ppc.altivec.vcmpnezb.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpnezh
    1 << 8 | 1, // llvm.ppc.altivec.vcmpnezh.p
    1 << 8 | 1, // llvm.ppc.altivec.vcmpnezw
    1 << 8 | 1, // llvm.ppc.altivec.vcmpnezw.p
    1 << 8 | 0, // llvm.ppc.altivec.vcntmbb
    1 << 8 | 0, // llvm.ppc.altivec.vcntmbd
    1 << 8 | 0, // llvm.ppc.altivec.vcntmbh
    1 << 8 | 0, // llvm.ppc.altivec.vcntmbw
    1 << 8 | 0, // llvm.ppc.altivec.vctsxs
    1 << 8 | 0, // llvm.ppc.altivec.vctuxs
    1 << 8 | 1, // llvm.ppc.altivec.vctzdm
    1 << 8 | 1, // llvm.ppc.altivec.vctzlsbb
    1 << 8 | 1, // llvm.ppc.altivec.vdivesd
    1 << 8 | 1, // llvm.ppc.altivec.vdivesq
    1 << 8 | 1, // llvm.ppc.altivec.vdivesw
    1 << 8 | 1, // llvm.ppc.altivec.vdiveud
    1 << 8 | 1, // llvm.ppc.altivec.vdiveuq
    1 << 8 | 1, // llvm.ppc.altivec.vdiveuw
    1 << 8 | 1, // llvm.ppc.altivec.vexpandbm
    1 << 8 | 1, // llvm.ppc.altivec.vexpanddm
    1 << 8 | 1, // llvm.ppc.altivec.vexpandhm
    1 << 8 | 1, // llvm.ppc.altivec.vexpandqm
    1 << 8 | 1, // llvm.ppc.altivec.vexpandwm
    1 << 8 | 1, // llvm.ppc.altivec.vexptefp
    1 << 8 | 1, // llvm.ppc.altivec.vextddvlx
    1 << 8 | 1, // llvm.ppc.altivec.vextddvrx
    1 << 8 | 1, // llvm.ppc.altivec.vextdubvlx
    1 << 8 | 1, // llvm.ppc.altivec.vextdubvrx
    1 << 8 | 1, // llvm.ppc.altivec.vextduhvlx
    1 << 8 | 1, // llvm.ppc.altivec.vextduhvrx
    1 << 8 | 1, // llvm.ppc.altivec.vextduwvlx
    1 << 8 | 1, // llvm.ppc.altivec.vextduwvrx
    1 << 8 | 1, // llvm.ppc.altivec.vextractbm
    1 << 8 | 1, // llvm.ppc.altivec.vextractdm
    1 << 8 | 1, // llvm.ppc.altivec.vextracthm
    1 << 8 | 1, // llvm.ppc.altivec.vextractqm
    1 << 8 | 1, // llvm.ppc.altivec.vextractwm
    1 << 8 | 1, // llvm.ppc.altivec.vextsb2d
    1 << 8 | 1, // llvm.ppc.altivec.vextsb2w
    1 << 8 | 1, // llvm.ppc.altivec.vextsd2q
    1 << 8 | 1, // llvm.ppc.altivec.vextsh2d
    1 << 8 | 1, // llvm.ppc.altivec.vextsh2w
    1 << 8 | 1, // llvm.ppc.altivec.vextsw2d
    1 << 8 | 1, // llvm.ppc.altivec.vgbbd
    1 << 8 | 0, // llvm.ppc.altivec.vgnb
    1 << 8 | 1, // llvm.ppc.altivec.vinsblx
    1 << 8 | 1, // llvm.ppc.altivec.vinsbrx
    1 << 8 | 1, // llvm.ppc.altivec.vinsbvlx
    1 << 8 | 1, // llvm.ppc.altivec.vinsbvrx
    1 << 8 | 11, // llvm.ppc.altivec.vinsd
    1 << 8 | 1, // llvm.ppc.altivec.vinsdlx
    1 << 8 | 1, // llvm.ppc.altivec.vinsdrx
    1 << 8 | 1, // llvm.ppc.altivec.vinshlx
    1 << 8 | 1, // llvm.ppc.altivec.vinshrx
    1 << 8 | 1, // llvm.ppc.altivec.vinshvlx
    1 << 8 | 1, // llvm.ppc.altivec.vinshvrx
    1 << 8 | 11, // llvm.ppc.altivec.vinsw
    1 << 8 | 1, // llvm.ppc.altivec.vinswlx
    1 << 8 | 1, // llvm.ppc.altivec.vinswrx
    1 << 8 | 1, // llvm.ppc.altivec.vinswvlx
    1 << 8 | 1, // llvm.ppc.altivec.vinswvrx
    1 << 8 | 1, // llvm.ppc.altivec.vlogefp
    1 << 8 | 1, // llvm.ppc.altivec.vmaddfp
    1 << 8 | 1, // llvm.ppc.altivec.vmaxfp
    1 << 8 | 1, // llvm.ppc.altivec.vmaxsb
    1 << 8 | 1, // llvm.ppc.altivec.vmaxsd
    1 << 8 | 1, // llvm.ppc.altivec.vmaxsh
    1 << 8 | 1, // llvm.ppc.altivec.vmaxsw
    1 << 8 | 1, // llvm.ppc.altivec.vmaxub
    1 << 8 | 1, // llvm.ppc.altivec.vmaxud
    1 << 8 | 1, // llvm.ppc.altivec.vmaxuh
    1 << 8 | 1, // llvm.ppc.altivec.vmaxuw
    30 << 8 | 1, // llvm.ppc.altivec.vmhaddshs
    30 << 8 | 1, // llvm.ppc.altivec.vmhraddshs
    1 << 8 | 1, // llvm.ppc.altivec.vminfp
    1 << 8 | 1, // llvm.ppc.altivec.vminsb
    1 << 8 | 1, // llvm.ppc.altivec.vminsd
    1 << 8 | 1, // llvm.ppc.altivec.vminsh
    1 << 8 | 1, // llvm.ppc.altivec.vminsw
    1 << 8 | 1, // llvm.ppc.altivec.vminub
    1 << 8 | 1, // llvm.ppc.altivec.vminud
    1 << 8 | 1, // llvm.ppc.altivec.vminuh
    1 << 8 | 1, // llvm.ppc.altivec.vminuw
    1 << 8 | 1, // llvm.ppc.altivec.vmladduhm
    1 << 8 | 1, // llvm.ppc.altivec.vmsumcud
    1 << 8 | 1, // llvm.ppc.altivec.vmsummbm
    1 << 8 | 1, // llvm.ppc.altivec.vmsumshm
    30 << 8 | 1, // llvm.ppc.altivec.vmsumshs
    1 << 8 | 1, // llvm.ppc.altivec.vmsumubm
    1 << 8 | 1, // llvm.ppc.altivec.vmsumudm
    1 << 8 | 1, // llvm.ppc.altivec.vmsumuhm
    30 << 8 | 1, // llvm.ppc.altivec.vmsumuhs
    1 << 8 | 1, // llvm.ppc.altivec.vmulesb
    1 << 8 | 1, // llvm.ppc.altivec.vmulesd
    1 << 8 | 1, // llvm.ppc.altivec.vmulesh
    1 << 8 | 1, // llvm.ppc.altivec.vmulesw
    1 << 8 | 1, // llvm.ppc.altivec.vmuleub
    1 << 8 | 1, // llvm.ppc.altivec.vmuleud
    1 << 8 | 1, // llvm.ppc.altivec.vmuleuh
    1 << 8 | 1, // llvm.ppc.altivec.vmuleuw
    1 << 8 | 1, // llvm.ppc.altivec.vmulhsd
    1 << 8 | 1, // llvm.ppc.altivec.vmulhsw
    1 << 8 | 1, // llvm.ppc.altivec.vmulhud
    1 << 8 | 1, // llvm.ppc.altivec.vmulhuw
    1 << 8 | 1, // llvm.ppc.altivec.vmulosb
    1 << 8 | 1, // llvm.ppc.altivec.vmulosd
    1 << 8 | 1, // llvm.ppc.altivec.vmulosh
    1 << 8 | 1, // llvm.ppc.altivec.vmulosw
    1 << 8 | 1, // llvm.ppc.altivec.vmuloub
    1 << 8 | 1, // llvm.ppc.altivec.vmuloud
    1 << 8 | 1, // llvm.ppc.altivec.vmulouh
    1 << 8 | 1, // llvm.ppc.altivec.vmulouw
    1 << 8 | 1, // llvm.ppc.altivec.vnmsubfp
    1 << 8 | 1, // llvm.ppc.altivec.vpdepd
    1 << 8 | 1, // llvm.ppc.altivec.vperm
    1 << 8 | 1, // llvm.ppc.altivec.vpextd
    1 << 8 | 1, // llvm.ppc.altivec.vpkpx
    30 << 8 | 1, // llvm.ppc.altivec.vpksdss
    30 << 8 | 1, // llvm.ppc.altivec.vpksdus
    30 << 8 | 1, // llvm.ppc.altivec.vpkshss
    30 << 8 | 1, // llvm.ppc.altivec.vpkshus
    30 << 8 | 1, // llvm.ppc.altivec.vpkswss
    30 << 8 | 1, // llvm.ppc.altivec.vpkswus
    30 << 8 | 1, // llvm.ppc.altivec.vpkudus
    30 << 8 | 1, // llvm.ppc.altivec.vpkuhus
    30 << 8 | 1, // llvm.ppc.altivec.vpkuwus
    1 << 8 | 1, // llvm.ppc.altivec.vprtybd
    1 << 8 | 1, // llvm.ppc.altivec.vprtybq
    1 << 8 | 1, // llvm.ppc.altivec.vprtybw
    1 << 8 | 1, // llvm.ppc.altivec.vrefp
    1 << 8 | 1, // llvm.ppc.altivec.vrfim
    1 << 8 | 1, // llvm.ppc.altivec.vrfin
    1 << 8 | 1, // llvm.ppc.altivec.vrfip
    1 << 8 | 1, // llvm.ppc.altivec.vrfiz
    1 << 8 | 1, // llvm.ppc.altivec.vrlb
    1 << 8 | 1, // llvm.ppc.altivec.vrld
    1 << 8 | 1, // llvm.ppc.altivec.vrldmi
    1 << 8 | 1, // llvm.ppc.altivec.vrldnm
    1 << 8 | 1, // llvm.ppc.altivec.vrlh
    1 << 8 | 1, // llvm.ppc.altivec.vrlqmi
    1 << 8 | 1, // llvm.ppc.altivec.vrlqnm
    1 << 8 | 1, // llvm.ppc.altivec.vrlw
    1 << 8 | 1, // llvm.ppc.altivec.vrlwmi
    1 << 8 | 1, // llvm.ppc.altivec.vrlwnm
    1 << 8 | 1, // llvm.ppc.altivec.vrsqrtefp
    1 << 8 | 1, // llvm.ppc.altivec.vsel
    1 << 8 | 1, // llvm.ppc.altivec.vsl
    1 << 8 | 1, // llvm.ppc.altivec.vslb
    1 << 8 | 11, // llvm.ppc.altivec.vsldbi
    1 << 8 | 1, // llvm.ppc.altivec.vslh
    1 << 8 | 1, // llvm.ppc.altivec.vslo
    1 << 8 | 1, // llvm.ppc.altivec.vslv
    1 << 8 | 1, // llvm.ppc.altivec.vslw
    1 << 8 | 1, // llvm.ppc.altivec.vsr
    1 << 8 | 1, // llvm.ppc.altivec.vsrab
    1 << 8 | 1, // llvm.ppc.altivec.vsrah
    1 << 8 | 1, // llvm.ppc.altivec.vsraw
    1 << 8 | 1, // llvm.ppc.altivec.vsrb
    1 << 8 | 11, // llvm.ppc.altivec.vsrdbi
    1 << 8 | 1, // llvm.ppc.altivec.vsrh
    1 << 8 | 1, // llvm.ppc.altivec.vsro
    1 << 8 | 1, // llvm.ppc.altivec.vsrv
    1 << 8 | 1, // llvm.ppc.altivec.vsrw
    1 << 8 | 1, // llvm.ppc.altivec.vstribl
    1 << 8 | 1, // llvm.ppc.altivec.vstribl.p
    1 << 8 | 1, // llvm.ppc.altivec.vstribr
    1 << 8 | 1, // llvm.ppc.altivec.vstribr.p
    1 << 8 | 1, // llvm.ppc.altivec.vstrihl
    1 << 8 | 1, // llvm.ppc.altivec.vstrihl.p
    1 << 8 | 1, // llvm.ppc.altivec.vstrihr
    1 << 8 | 1, // llvm.ppc.altivec.vstrihr.p
    1 << 8 | 1, // llvm.ppc.altivec.vsubcuq
    1 << 8 | 1, // llvm.ppc.altivec.vsubcuw
    1 << 8 | 1, // llvm.ppc.altivec.vsubecuq
    1 << 8 | 1, // llvm.ppc.altivec.vsubeuqm
    1 << 8 | 1, // llvm.ppc.altivec.vsubsbs
    1 << 8 | 1, // llvm.ppc.altivec.vsubshs
    1 << 8 | 1, // llvm.ppc.altivec.vsubsws
    1 << 8 | 1, // llvm.ppc.altivec.vsububs
    1 << 8 | 1, // llvm.ppc.altivec.vsubuhs
    1 << 8 | 1, // llvm.ppc.altivec.vsubuws
    30 << 8 | 1, // llvm.ppc.altivec.vsum2sws
    30 << 8 | 1, // llvm.ppc.altivec.vsum4sbs
    30 << 8 | 1, // llvm.ppc.altivec.vsum4shs
    30 << 8 | 1, // llvm.ppc.altivec.vsum4ubs
    30 << 8 | 1, // llvm.ppc.altivec.vsumsws
    1 << 8 | 1, // llvm.ppc.altivec.vupkhpx
    1 << 8 | 1, // llvm.ppc.altivec.vupkhsb
    1 << 8 | 1, // llvm.ppc.altivec.vupkhsh
    1 << 8 | 1, // llvm.ppc.altivec.vupkhsw
    1 << 8 | 1, // llvm.ppc.altivec.vupklpx
    1 << 8 | 1, // llvm.ppc.altivec.vupklsb
    1 << 8 | 1, // llvm.ppc.altivec.vupklsh
    1 << 8 | 1, // llvm.ppc.altivec.vupklsw
    18 << 8 | 11, // llvm.ppc.amo.ldat
    18 << 8 | 11, // llvm.ppc.amo.lwat
    12 << 8 | 10, // llvm.ppc.atomic.load.i128
    79 << 8 | 37, // llvm.ppc.atomic.store.i128
    11 << 8 | 10, // llvm.ppc.atomicrmw.add.i128
    11 << 8 | 10, // llvm.ppc.atomicrmw.and.i128
    11 << 8 | 10, // llvm.ppc.atomicrmw.nand.i128
    11 << 8 | 10, // llvm.ppc.atomicrmw.or.i128
    11 << 8 | 10, // llvm.ppc.atomicrmw.sub.i128
    11 << 8 | 10, // llvm.ppc.atomicrmw.xchg.i128
    11 << 8 | 10, // llvm.ppc.atomicrmw.xor.i128
    1 << 8 | 11, // llvm.ppc.bcdadd
    1 << 8 | 5, // llvm.ppc.bcdadd.p
    1 << 8 | 1, // llvm.ppc.bcdcopysign
    1 << 8 | 0, // llvm.ppc.bcdsetsign
    1 << 8 | 11, // llvm.ppc.bcdsub
    1 << 8 | 5, // llvm.ppc.bcdsub.p
    1 << 8 | 1, // llvm.ppc.bpermd
    1 << 8 | 1, // llvm.ppc.cbcdtd
    1 << 8 | 1, // llvm.ppc.cbcdtdd
    1 << 8 | 1, // llvm.ppc.cdtbcd
    1 << 8 | 1, // llvm.ppc.cdtbcdd
    5 << 8 | 1, // llvm.ppc.cfence
    1 << 8 | 1, // llvm.ppc.cfuged
    1 << 8 | 1, // llvm.ppc.cmpb
    1 << 8 | 1, // llvm.ppc.cmpeqb
    1 << 8 | 5, // llvm.ppc.cmprb
    11 << 8 | 10, // llvm.ppc.cmpxchg.i128
    1 << 8 | 1, // llvm.ppc.cntlzdm
    1 << 8 | 1, // llvm.ppc.cnttzdm
    1 << 8 | 1, // llvm.ppc.compare.exp.eq
    1 << 8 | 1, // llvm.ppc.compare.exp.gt
    1 << 8 | 1, // llvm.ppc.compare.exp.lt
    1 << 8 | 1, // llvm.ppc.compare.exp.uo
    1 << 8 | 1, // llvm.ppc.convert.f128.to.ppcf128
    1 << 8 | 1, // llvm.ppc.convert.ppcf128.to.f128
    102 << 8 | 1, // llvm.ppc.darn
    102 << 8 | 1, // llvm.ppc.darn32
    102 << 8 | 1, // llvm.ppc.darnraw
    5 << 8 | 1, // llvm.ppc.dcba
    11 << 8 | 1, // llvm.ppc.dcbf
    11 << 8 | 1, // llvm.ppc.dcbfl
    11 << 8 | 1, // llvm.ppc.dcbflp
    11 << 8 | 1, // llvm.ppc.dcbfps
    5 << 8 | 1, // llvm.ppc.dcbi
    5 << 8 | 1, // llvm.ppc.dcbst
    11 << 8 | 1, // llvm.ppc.dcbstps
    11 << 8 | 10, // llvm.ppc.dcbt
    11 << 8 | 64, // llvm.ppc.dcbt.with.hint
    11 << 8 | 10, // llvm.ppc.dcbtst
    11 << 8 | 64, // llvm.ppc.dcbtst.with.hint
    11 << 8 | 10, // llvm.ppc.dcbtstt
    11 << 8 | 10, // llvm.ppc.dcbtt
    5 << 8 | 1, // llvm.ppc.dcbz
    5 << 8 | 1, // llvm.ppc.dcbzl
    1 << 8 | 1, // llvm.ppc.divde
    1 << 8 | 1, // llvm.ppc.divdeu
    1 << 8 | 1, // llvm.ppc.divf128.round.to.odd
    1 << 8 | 1, // llvm.ppc.divwe
    1 << 8 | 1, // llvm.ppc.divweu
    5 << 8 | 1, // llvm.ppc.eieio
    1 << 8 | 1, // llvm.ppc.extract.exp
    1 << 8 | 1, // llvm.ppc.extract.sig
    1 << 8 | 1, // llvm.ppc.fcfid
    1 << 8 | 1, // llvm.ppc.fcfud
    1 << 8 | 1, // llvm.ppc.fctid
    1 << 8 | 1, // llvm.ppc.fctidz
    1 << 8 | 1, // llvm.ppc.fctiw
    1 << 8 | 1, // llvm.ppc.fctiwz
    1 << 8 | 1, // llvm.ppc.fctudz
    1 << 8 | 1, // llvm.ppc.fctuwz
    102 << 8 | 1, // llvm.ppc.fence
    3 << 8 | 5, // llvm.ppc.fixed.addr.ld
    1 << 8 | 1, // llvm.ppc.fmaf128.round.to.odd
    1 << 8 | 1, // llvm.ppc.fmsub
    1 << 8 | 1, // llvm.ppc.fmsubs
    1 << 8 | 1, // llvm.ppc.fnabs
    1 << 8 | 1, // llvm.ppc.fnabss
    1 << 8 | 1, // llvm.ppc.fnmadd
    1 << 8 | 1, // llvm.ppc.fnmadds
    1 << 8 | 1, // llvm.ppc.fnmsub
    1 << 8 | 1, // llvm.ppc.fre
    1 << 8 | 1, // llvm.ppc.fres
    1 << 8 | 1, // llvm.ppc.frsqrte
    1 << 8 | 1, // llvm.ppc.frsqrtes
    1 << 8 | 1, // llvm.ppc.fsel
    1 << 8 | 1, // llvm.ppc.fsels
    5 << 8 | 1, // llvm.ppc.get.texasr
    5 << 8 | 1, // llvm.ppc.get.texasru
    5 << 8 | 1, // llvm.ppc.get.tfhar
    5 << 8 | 1, // llvm.ppc.get.tfiar
    5 << 8 | 1, // llvm.ppc.icbt
    1 << 8 | 1, // llvm.ppc.insert.exp
    5 << 8 | 1, // llvm.ppc.iospace.eieio
    5 << 8 | 1, // llvm.ppc.iospace.lwsync
    5 << 8 | 1, // llvm.ppc.iospace.sync
    5 << 8 | 1, // llvm.ppc.isync
    12 << 8 | 1, // llvm.ppc.lbarx
    12 << 8 | 1, // llvm.ppc.ldarx
    12 << 8 | 1, // llvm.ppc.lharx
    2 << 8 | 1, // llvm.ppc.load2r
    2 << 8 | 1, // llvm.ppc.load4r
    2 << 8 | 1, // llvm.ppc.load8r
    12 << 8 | 1, // llvm.ppc.lwarx
    5 << 8 | 1, // llvm.ppc.lwsync
    1 << 8 | 1, // llvm.ppc.maddhd
    1 << 8 | 1, // llvm.ppc.maddhdu
    1 << 8 | 1, // llvm.ppc.maddld
    1 << 8 | 1, // llvm.ppc.maxfe
    1 << 8 | 1, // llvm.ppc.maxfl
    1 << 8 | 1, // llvm.ppc.maxfs
    102 << 8 | 1, // llvm.ppc.mffsl
    1 << 8 | 1, // llvm.ppc.mfmsr
    7 << 8 | 5, // llvm.ppc.mfspr
    1 << 8 | 1, // llvm.ppc.mftbu
    1 << 8 | 1, // llvm.ppc.minfe
    1 << 8 | 1, // llvm.ppc.minfl
    1 << 8 | 1, // llvm.ppc.minfs
    1 << 8 | 1, // llvm.ppc.mma.assemble.acc
    1 << 8 | 1, // llvm.ppc.mma.build.dmr
    1 << 8 | 1, // llvm.ppc.mma.disassemble.acc
    19 << 8 | 1, // llvm.ppc.mma.disassemble.dmr
    1 << 8 | 1, // llvm.ppc.mma.dmmr
    1 << 8 | 1, // llvm.ppc.mma.dmsetdmrz
    1 << 8 | 11, // llvm.ppc.mma.dmsha2hash
    1 << 8 | 0, // llvm.ppc.mma.dmsha3hash
    1 << 8 | 1, // llvm.ppc.mma.dmxor
    1 << 8 | 1, // llvm.ppc.mma.dmxvbf16gerx2
    1 << 8 | 1, // llvm.ppc.mma.dmxvbf16gerx2nn
    1 << 8 | 1, // llvm.ppc.mma.dmxvbf16gerx2np
    1 << 8 | 1, // llvm.ppc.mma.dmxvbf16gerx2pn
    1 << 8 | 1, // llvm.ppc.mma.dmxvbf16gerx2pp
    1 << 8 | 1, // llvm.ppc.mma.dmxvf16gerx2
    1 << 8 | 1, // llvm.ppc.mma.dmxvf16gerx2nn
    1 << 8 | 1, // llvm.ppc.mma.dmxvf16gerx2np
    1 << 8 | 1, // llvm.ppc.mma.dmxvf16gerx2pn
    1 << 8 | 1, // llvm.ppc.mma.dmxvf16gerx2pp
    1 << 8 | 1, // llvm.ppc.mma.dmxvi8gerx4
    1 << 8 | 1, // llvm.ppc.mma.dmxvi8gerx4pp
    1 << 8 | 1, // llvm.ppc.mma.dmxvi8gerx4spp
    1 << 8 | 1, // llvm.ppc.mma.dmxxextfdmr256
    1 << 8 | 1, // llvm.ppc.mma.dmxxextfdmr512
    1 << 8 | 1, // llvm.ppc.mma.dmxxinstdmr256
    1 << 8 | 1, // llvm.ppc.mma.dmxxinstdmr512
    1 << 8 | 25, // llvm.ppc.mma.dmxxshapad
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvbf16gerx2
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvbf16gerx2nn
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvbf16gerx2np
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvbf16gerx2pn
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvbf16gerx2pp
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvf16gerx2
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvf16gerx2nn
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvf16gerx2np
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvf16gerx2pn
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvf16gerx2pp
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvi8gerx4
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvi8gerx4pp
    1 << 8 | 1, // llvm.ppc.mma.pmdmxvi8gerx4spp
    1 << 8 | 1, // llvm.ppc.mma.pmxvbf16ger2
    1 << 8 | 1, // llvm.ppc.mma.pmxvbf16ger2nn
    1 << 8 | 1, // llvm.ppc.mma.pmxvbf16ger2np
    1 << 8 | 1, // llvm.ppc.mma.pmxvbf16ger2pn
    1 << 8 | 1, // llvm.ppc.mma.pmxvbf16ger2pp
    1 << 8 | 1, // llvm.ppc.mma.pmxvf16ger2
    1 << 8 | 1, // llvm.ppc.mma.pmxvf16ger2nn
    1 << 8 | 1, // llvm.ppc.mma.pmxvf16ger2np
    1 << 8 | 1, // llvm.ppc.mma.pmxvf16ger2pn
    1 << 8 | 1, // llvm.ppc.mma.pmxvf16ger2pp
    1 << 8 | 1, // llvm.ppc.mma.pmxvf32ger
    1 << 8 | 1, // llvm.ppc.mma.pmxvf32gernn
    1 << 8 | 1, // llvm.ppc.mma.pmxvf32gernp
    1 << 8 | 1, // llvm.ppc.mma.pmxvf32gerpn
    1 << 8 | 1, // llvm.ppc.mma.pmxvf32gerpp
    1 << 8 | 1, // llvm.ppc.mma.pmxvf64ger
    1 << 8 | 1, // llvm.ppc.mma.pmxvf64gernn
    1 << 8 | 1, // llvm.ppc.mma.pmxvf64gernp
    1 << 8 | 1, // llvm.ppc.mma.pmxvf64gerpn
    1 << 8 | 1, // llvm.ppc.mma.pmxvf64gerpp
    1 << 8 | 1, // llvm.ppc.mma.pmxvi16ger2
    1 << 8 | 1, // llvm.ppc.mma.pmxvi16ger2pp
    1 << 8 | 1, // llvm.ppc.mma.pmxvi16ger2s
    1 << 8 | 1, // llvm.ppc.mma.pmxvi16ger2spp
    1 << 8 | 1, // llvm.ppc.mma.pmxvi4ger8
    1 << 8 | 1, // llvm.ppc.mma.pmxvi4ger8pp
    1 << 8 | 1, // llvm.ppc.mma.pmxvi8ger4
    1 << 8 | 1, // llvm.ppc.mma.pmxvi8ger4pp
    1 << 8 | 1, // llvm.ppc.mma.pmxvi8ger4spp
    1 << 8 | 1, // llvm.ppc.mma.xvbf16ger2
    1 << 8 | 1, // llvm.ppc.mma.xvbf16ger2nn
    1 << 8 | 1, // llvm.ppc.mma.xvbf16ger2np
    1 << 8 | 1, // llvm.ppc.mma.xvbf16ger2pn
    1 << 8 | 1, // llvm.ppc.mma.xvbf16ger2pp
    1 << 8 | 1, // llvm.ppc.mma.xvf16ger2
    1 << 8 | 1, // llvm.ppc.mma.xvf16ger2nn
    1 << 8 | 1, // llvm.ppc.mma.xvf16ger2np
    1 << 8 | 1, // llvm.ppc.mma.xvf16ger2pn
    1 << 8 | 1, // llvm.ppc.mma.xvf16ger2pp
    1 << 8 | 1, // llvm.ppc.mma.xvf32ger
    1 << 8 | 1, // llvm.ppc.mma.xvf32gernn
    1 << 8 | 1, // llvm.ppc.mma.xvf32gernp
    1 << 8 | 1, // llvm.ppc.mma.xvf32gerpn
    1 << 8 | 1, // llvm.ppc.mma.xvf32gerpp
    1 << 8 | 1, // llvm.ppc.mma.xvf64ger
    1 << 8 | 1, // llvm.ppc.mma.xvf64gernn
    1 << 8 | 1, // llvm.ppc.mma.xvf64gernp
    1 << 8 | 1, // llvm.ppc.mma.xvf64gerpn
    1 << 8 | 1, // llvm.ppc.mma.xvf64gerpp
    1 << 8 | 1, // llvm.ppc.mma.xvi16ger2
    1 << 8 | 1, // llvm.ppc.mma.xvi16ger2pp
    1 << 8 | 1, // llvm.ppc.mma.xvi16ger2s
    1 << 8 | 1, // llvm.ppc.mma.xvi16ger2spp
    1 << 8 | 1, // llvm.ppc.mma.xvi4ger8
    1 << 8 | 1, // llvm.ppc.mma.xvi4ger8pp
    1 << 8 | 1, // llvm.ppc.mma.xvi8ger4
    1 << 8 | 1, // llvm.ppc.mma.xvi8ger4pp
    1 << 8 | 1, // llvm.ppc.mma.xvi8ger4spp
    1 << 8 | 1, // llvm.ppc.mma.xxmfacc
    1 << 8 | 1, // llvm.ppc.mma.xxmtacc
    1 << 8 | 1, // llvm.ppc.mma.xxsetaccz
    30 << 8 | 5, // llvm.ppc.mtfsb0
    30 << 8 | 5, // llvm.ppc.mtfsb1
    30 << 8 | 5, // llvm.ppc.mtfsf
    30 << 8 | 17, // llvm.ppc.mtfsfi
    5 << 8 | 1, // llvm.ppc.mtmsr
    7 << 8 | 5, // llvm.ppc.mtspr
    1 << 8 | 1, // llvm.ppc.mulf128.round.to.odd
    1 << 8 | 1, // llvm.ppc.mulhd
    1 << 8 | 1, // llvm.ppc.mulhdu
    1 << 8 | 1, // llvm.ppc.mulhw
    1 << 8 | 1, // llvm.ppc.mulhwu
    1 << 8 | 0, // llvm.ppc.national2packed
    1 << 8 | 1, // llvm.ppc.pack.longdouble
    1 << 8 | 1, // llvm.ppc.packed2national
    1 << 8 | 0, // llvm.ppc.packed2zoned
    1 << 8 | 1, // llvm.ppc.pdepd
    1 << 8 | 1, // llvm.ppc.pextd
    1 << 8 | 1, // llvm.ppc.popcntb
    102 << 8 | 1, // llvm.ppc.readflm
    1 << 8 | 93, // llvm.ppc.rldimi
    1 << 8 | 93, // llvm.ppc.rlwimi
    1 << 8 | 11, // llvm.ppc.rlwnm
    1 << 8 | 1, // llvm.ppc.scalar.extract.expq
    1 << 8 | 1, // llvm.ppc.scalar.insert.exp.qp
    5 << 8 | 1, // llvm.ppc.set.texasr
    5 << 8 | 1, // llvm.ppc.set.texasru
    5 << 8 | 1, // llvm.ppc.set.tfhar
    5 << 8 | 1, // llvm.ppc.set.tfiar
    1 << 8 | 1, // llvm.ppc.setb
    30 << 8 | 1, // llvm.ppc.setflm
    30 << 8 | 1, // llvm.ppc.setrnd
    1 << 8 | 1, // llvm.ppc.sqrtf128.round.to.odd
    103 << 8 | 1, // llvm.ppc.stbcx
    103 << 8 | 1, // llvm.ppc.stdcx
    24 << 8 | 1, // llvm.ppc.stfiw
    103 << 8 | 1, // llvm.ppc.sthcx
    34 << 8 | 1, // llvm.ppc.store2r
    34 << 8 | 1, // llvm.ppc.store4r
    34 << 8 | 1, // llvm.ppc.store8r
    79 << 8 | 1, // llvm.ppc.stwcx
    1 << 8 | 1, // llvm.ppc.subf128.round.to.odd
    5 << 8 | 1, // llvm.ppc.sync
    5 << 8 | 1, // llvm.ppc.tabort
    5 << 8 | 1, // llvm.ppc.tabortdc
    5 << 8 | 1, // llvm.ppc.tabortdci
    5 << 8 | 1, // llvm.ppc.tabortwc
    5 << 8 | 1, // llvm.ppc.tabortwci
    5 << 8 | 5, // llvm.ppc.tbegin
    5 << 8 | 1, // llvm.ppc.tcheck
    5 << 8 | 11, // llvm.ppc.tdw
    5 << 8 | 5, // llvm.ppc.tend
    5 << 8 | 1, // llvm.ppc.tendall
    1 << 8 | 0, // llvm.ppc.test.data.class
    5 << 8 | 1, // llvm.ppc.trap
    5 << 8 | 1, // llvm.ppc.trapd
    5 << 8 | 1, // llvm.ppc.trechkpt
    5 << 8 | 1, // llvm.ppc.treclaim
    5 << 8 | 1, // llvm.ppc.tresume
    1 << 8 | 1, // llvm.ppc.truncf128.round.to.odd
    5 << 8 | 1, // llvm.ppc.tsr
    5 << 8 | 1, // llvm.ppc.tsuspend
    5 << 8 | 1, // llvm.ppc.ttest
    5 << 8 | 11, // llvm.ppc.tw
    1 << 8 | 1, // llvm.ppc.unpack.longdouble
    1 << 8 | 1, // llvm.ppc.vsx.assemble.pair
    1 << 8 | 1, // llvm.ppc.vsx.disassemble.pair
    2 << 8 | 1, // llvm.ppc.vsx.lxvd2x
    2 << 8 | 1, // llvm.ppc.vsx.lxvd2x.be
    2 << 8 | 1, // llvm.ppc.vsx.lxvl
    2 << 8 | 1, // llvm.ppc.vsx.lxvll
    2 << 8 | 1, // llvm.ppc.vsx.lxvp
    2 << 8 | 1, // llvm.ppc.vsx.lxvprl
    2 << 8 | 1, // llvm.ppc.vsx.lxvprll
    2 << 8 | 1, // llvm.ppc.vsx.lxvrl
    2 << 8 | 1, // llvm.ppc.vsx.lxvrll
    2 << 8 | 1, // llvm.ppc.vsx.lxvw4x
    2 << 8 | 1, // llvm.ppc.vsx.lxvw4x.be
    79 << 8 | 1, // llvm.ppc.vsx.stxvd2x
    79 << 8 | 1, // llvm.ppc.vsx.stxvd2x.be
    79 << 8 | 1, // llvm.ppc.vsx.stxvl
    79 << 8 | 1, // llvm.ppc.vsx.stxvll
    79 << 8 | 1, // llvm.ppc.vsx.stxvp
    79 << 8 | 1, // llvm.ppc.vsx.stxvprl
    79 << 8 | 1, // llvm.ppc.vsx.stxvprll
    79 << 8 | 1, // llvm.ppc.vsx.stxvrl
    79 << 8 | 1, // llvm.ppc.vsx.stxvrll
    79 << 8 | 1, // llvm.ppc.vsx.stxvw4x
    79 << 8 | 1, // llvm.ppc.vsx.stxvw4x.be
    1 << 8 | 1, // llvm.ppc.vsx.xsmaxdp
    1 << 8 | 1, // llvm.ppc.vsx.xsmindp
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpeqdp
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpeqdp.p
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpeqsp
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpeqsp.p
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpgedp
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpgedp.p
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpgesp
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpgesp.p
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpgtdp
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpgtdp.p
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpgtsp
    1 << 8 | 1, // llvm.ppc.vsx.xvcmpgtsp.p
    1 << 8 | 1, // llvm.ppc.vsx.xvcvbf16spn
    1 << 8 | 1, // llvm.ppc.vsx.xvcvdpsp
    1 << 8 | 1, // llvm.ppc.vsx.xvcvdpsxws
    1 << 8 | 1, // llvm.ppc.vsx.xvcvdpuxws
    1 << 8 | 1, // llvm.ppc.vsx.xvcvhpsp
    1 << 8 | 1, // llvm.ppc.vsx.xvcvspbf16
    1 << 8 | 1, // llvm.ppc.vsx.xvcvspdp
    1 << 8 | 1, // llvm.ppc.vsx.xvcvsphp
    1 << 8 | 1, // llvm.ppc.vsx.xvcvspsxds
    1 << 8 | 1, // llvm.ppc.vsx.xvcvspuxds
    1 << 8 | 1, // llvm.ppc.vsx.xvcvsxdsp
    1 << 8 | 1, // llvm.ppc.vsx.xvcvsxwdp
    1 << 8 | 1, // llvm.ppc.vsx.xvcvuxdsp
    1 << 8 | 1, // llvm.ppc.vsx.xvcvuxwdp
    1 << 8 | 1, // llvm.ppc.vsx.xvdivdp
    1 << 8 | 1, // llvm.ppc.vsx.xvdivsp
    1 << 8 | 1, // llvm.ppc.vsx.xviexpdp
    1 << 8 | 1, // llvm.ppc.vsx.xviexpsp
    1 << 8 | 1, // llvm.ppc.vsx.xvmaxdp
    1 << 8 | 1, // llvm.ppc.vsx.xvmaxsp
    1 << 8 | 1, // llvm.ppc.vsx.xvmindp
    1 << 8 | 1, // llvm.ppc.vsx.xvminsp
    1 << 8 | 1, // llvm.ppc.vsx.xvrdpip
    1 << 8 | 1, // llvm.ppc.vsx.xvredp
    1 << 8 | 1, // llvm.ppc.vsx.xvresp
    1 << 8 | 1, // llvm.ppc.vsx.xvrlw
    1 << 8 | 1, // llvm.ppc.vsx.xvrspip
    1 << 8 | 1, // llvm.ppc.vsx.xvrsqrtedp
    1 << 8 | 1, // llvm.ppc.vsx.xvrsqrtesp
    1 << 8 | 1, // llvm.ppc.vsx.xvtdivdp
    1 << 8 | 1, // llvm.ppc.vsx.xvtdivsp
    1 << 8 | 1, // llvm.ppc.vsx.xvtlsbb
    1 << 8 | 1, // llvm.ppc.vsx.xvtsqrtdp
    1 << 8 | 1, // llvm.ppc.vsx.xvtsqrtsp
    1 << 8 | 0, // llvm.ppc.vsx.xvtstdcdp
    1 << 8 | 0, // llvm.ppc.vsx.xvtstdcsp
    1 << 8 | 1, // llvm.ppc.vsx.xvxexpdp
    1 << 8 | 1, // llvm.ppc.vsx.xvxexpsp
    1 << 8 | 1, // llvm.ppc.vsx.xvxsigdp
    1 << 8 | 1, // llvm.ppc.vsx.xvxsigsp
    1 << 8 | 1, // llvm.ppc.vsx.xxblendvb
    1 << 8 | 1, // llvm.ppc.vsx.xxblendvd
    1 << 8 | 1, // llvm.ppc.vsx.xxblendvh
    1 << 8 | 1, // llvm.ppc.vsx.xxblendvw
    1 << 8 | 35, // llvm.ppc.vsx.xxeval
    1 << 8 | 1, // llvm.ppc.vsx.xxextractuw
    1 << 8 | 1, // llvm.ppc.vsx.xxgenpcvbm
    1 << 8 | 1, // llvm.ppc.vsx.xxgenpcvdm
    1 << 8 | 1, // llvm.ppc.vsx.xxgenpcvhm
    1 << 8 | 1, // llvm.ppc.vsx.xxgenpcvwm
    1 << 8 | 1, // llvm.ppc.vsx.xxinsertw
    1 << 8 | 1, // llvm.ppc.vsx.xxleqv
    1 << 8 | 35, // llvm.ppc.vsx.xxpermx
    1 << 8 | 0, // llvm.ppc.zoned2packed
    0 << 8 | 1, // llvm.r600.cube
    1 << 8 | 1, // llvm.r600.ddx
    1 << 8 | 1, // llvm.r600.ddy
    0 << 8 | 1, // llvm.r600.dot4
    104 << 8 | 1, // llvm.r600.group.barrier
    0 << 8 | 2, // llvm.r600.implicitarg.ptr
    7 << 8 | 1, // llvm.r600.kill
    7 << 8 | 1, // llvm.r600.rat.store.typed
    0 << 8 | 2, // llvm.r600.read.global.size.x
    0 << 8 | 2, // llvm.r600.read.global.size.y
    0 << 8 | 2, // llvm.r600.read.global.size.z
    0 << 8 | 2, // llvm.r600.read.local.size.x
    0 << 8 | 2, // llvm.r600.read.local.size.y
    0 << 8 | 2, // llvm.r600.read.local.size.z
    0 << 8 | 2, // llvm.r600.read.ngroups.x
    0 << 8 | 2, // llvm.r600.read.ngroups.y
    0 << 8 | 2, // llvm.r600.read.ngroups.z
    0 << 8 | 2, // llvm.r600.read.tgid.x
    0 << 8 | 2, // llvm.r600.read.tgid.y
    0 << 8 | 2, // llvm.r600.read.tgid.z
    0 << 8 | 128, // llvm.r600.read.tidig.x
    0 << 8 | 128, // llvm.r600.read.tidig.y
    0 << 8 | 128, // llvm.r600.read.tidig.z
    0 << 8 | 1, // llvm.r600.recipsqrt.clamped
    0 << 8 | 1, // llvm.r600.recipsqrt.ieee
    7 << 8 | 1, // llvm.r600.store.stream.output
    53 << 8 | 1, // llvm.r600.store.swizzle
    1 << 8 | 1, // llvm.r600.tex
    1 << 8 | 1, // llvm.r600.texc
    1 << 8 | 1, // llvm.r600.txb
    1 << 8 | 1, // llvm.r600.txbc
    1 << 8 | 1, // llvm.r600.txf
    1 << 8 | 1, // llvm.r600.txl
    1 << 8 | 1, // llvm.r600.txlc
    1 << 8 | 1, // llvm.r600.txq
    0 << 8 | 11, // llvm.riscv.aes32dsi
    0 << 8 | 11, // llvm.riscv.aes32dsmi
    0 << 8 | 11, // llvm.riscv.aes32esi
    0 << 8 | 11, // llvm.riscv.aes32esmi
    0 << 8 | 1, // llvm.riscv.aes64ds
    0 << 8 | 1, // llvm.riscv.aes64dsm
    0 << 8 | 1, // llvm.riscv.aes64es
    0 << 8 | 1, // llvm.riscv.aes64esm
    0 << 8 | 1, // llvm.riscv.aes64im
    0 << 8 | 0, // llvm.riscv.aes64ks1i
    0 << 8 | 1, // llvm.riscv.aes64ks2
    0 << 8 | 1, // llvm.riscv.brev8
    0 << 8 | 1, // llvm.riscv.clmul
    0 << 8 | 1, // llvm.riscv.clmulh
    0 << 8 | 1, // llvm.riscv.clmulr
    0 << 8 | 1, // llvm.riscv.cv.alu.addN
    0 << 8 | 1, // llvm.riscv.cv.alu.addRN
    0 << 8 | 1, // llvm.riscv.cv.alu.adduN
    0 << 8 | 1, // llvm.riscv.cv.alu.adduRN
    0 << 8 | 1, // llvm.riscv.cv.alu.clip
    0 << 8 | 1, // llvm.riscv.cv.alu.clipu
    0 << 8 | 1, // llvm.riscv.cv.alu.subN
    0 << 8 | 1, // llvm.riscv.cv.alu.subRN
    0 << 8 | 1, // llvm.riscv.cv.alu.subuN
    0 << 8 | 1, // llvm.riscv.cv.alu.subuRN
    0 << 8 | 1, // llvm.riscv.cv.bitmanip.bclr
    0 << 8 | 13, // llvm.riscv.cv.bitmanip.bitrev
    0 << 8 | 1, // llvm.riscv.cv.bitmanip.bset
    0 << 8 | 1, // llvm.riscv.cv.bitmanip.clb
    0 << 8 | 1, // llvm.riscv.cv.bitmanip.extract
    0 << 8 | 1, // llvm.riscv.cv.bitmanip.extractu
    0 << 8 | 1, // llvm.riscv.cv.bitmanip.insert
    105 << 8 | 1, // llvm.riscv.cv.elw.elw
    106 << 8 | 1, // llvm.riscv.cv.mac.mac
    106 << 8 | 35, // llvm.riscv.cv.mac.machhsN
    106 << 8 | 35, // llvm.riscv.cv.mac.machhsRN
    106 << 8 | 35, // llvm.riscv.cv.mac.machhuN
    106 << 8 | 35, // llvm.riscv.cv.mac.machhuRN
    106 << 8 | 35, // llvm.riscv.cv.mac.macsN
    106 << 8 | 35, // llvm.riscv.cv.mac.macsRN
    106 << 8 | 35, // llvm.riscv.cv.mac.macuN
    106 << 8 | 35, // llvm.riscv.cv.mac.macuRN
    106 << 8 | 1, // llvm.riscv.cv.mac.msu
    106 << 8 | 11, // llvm.riscv.cv.mac.mulhhsN
    106 << 8 | 11, // llvm.riscv.cv.mac.mulhhsRN
    106 << 8 | 11, // llvm.riscv.cv.mac.mulhhuN
    106 << 8 | 11, // llvm.riscv.cv.mac.mulhhuRN
    106 << 8 | 11, // llvm.riscv.cv.mac.mulsN
    106 << 8 | 11, // llvm.riscv.cv.mac.mulsRN
    106 << 8 | 11, // llvm.riscv.cv.mac.muluN
    106 << 8 | 11, // llvm.riscv.cv.mac.muluRN
    11 << 8 | 99, // llvm.riscv.masked.atomicrmw.add
    11 << 8 | 108, // llvm.riscv.masked.atomicrmw.max
    11 << 8 | 108, // llvm.riscv.masked.atomicrmw.min
    11 << 8 | 99, // llvm.riscv.masked.atomicrmw.nand
    11 << 8 | 99, // llvm.riscv.masked.atomicrmw.sub
    11 << 8 | 99, // llvm.riscv.masked.atomicrmw.umax
    11 << 8 | 99, // llvm.riscv.masked.atomicrmw.umin
    11 << 8 | 99, // llvm.riscv.masked.atomicrmw.xchg
    11 << 8 | 108, // llvm.riscv.masked.cmpxchg
    29 << 8 | 1, // llvm.riscv.mips.ehb
    29 << 8 | 1, // llvm.riscv.mips.ihb
    29 << 8 | 1, // llvm.riscv.mips.pause
    0 << 8 | 0, // llvm.riscv.mopr
    0 << 8 | 11, // llvm.riscv.moprr
    43 << 8 | 1, // llvm.riscv.nds.ffb
    43 << 8 | 1, // llvm.riscv.nds.ffmism
    43 << 8 | 1, // llvm.riscv.nds.ffzmism
    43 << 8 | 1, // llvm.riscv.nds.flmism
    1 << 8 | 43, // llvm.riscv.nds.vd4dots
    1 << 8 | 47, // llvm.riscv.nds.vd4dots.mask
    1 << 8 | 43, // llvm.riscv.nds.vd4dotsu
    1 << 8 | 47, // llvm.riscv.nds.vd4dotsu.mask
    1 << 8 | 43, // llvm.riscv.nds.vd4dotu
    1 << 8 | 47, // llvm.riscv.nds.vd4dotu.mask
    1 << 8 | 11, // llvm.riscv.nds.vfncvt.bf16.s
    1 << 8 | 35, // llvm.riscv.nds.vfpmadb
    1 << 8 | 234, // llvm.riscv.nds.vfpmadb.mask
    1 << 8 | 35, // llvm.riscv.nds.vfpmadt
    1 << 8 | 234, // llvm.riscv.nds.vfpmadt.mask
    1 << 8 | 1, // llvm.riscv.nds.vfwcvt.s.bf16
    2 << 8 | 18, // llvm.riscv.nds.vln
    2 << 8 | 100, // llvm.riscv.nds.vln.mask
    2 << 8 | 18, // llvm.riscv.nds.vlnu
    2 << 8 | 100, // llvm.riscv.nds.vlnu.mask
    0 << 8 | 1, // llvm.riscv.orc.b
    30 << 8 | 1, // llvm.riscv.pause
    23 << 8 | 10, // llvm.riscv.seg2.load.mask
    24 << 8 | 37, // llvm.riscv.seg2.store.mask
    23 << 8 | 10, // llvm.riscv.seg3.load.mask
    24 << 8 | 38, // llvm.riscv.seg3.store.mask
    23 << 8 | 10, // llvm.riscv.seg4.load.mask
    24 << 8 | 39, // llvm.riscv.seg4.store.mask
    23 << 8 | 10, // llvm.riscv.seg5.load.mask
    24 << 8 | 40, // llvm.riscv.seg5.store.mask
    23 << 8 | 10, // llvm.riscv.seg6.load.mask
    24 << 8 | 235, // llvm.riscv.seg6.store.mask
    23 << 8 | 10, // llvm.riscv.seg7.load.mask
    24 << 8 | 236, // llvm.riscv.seg7.store.mask
    23 << 8 | 10, // llvm.riscv.seg8.load.mask
    24 << 8 | 237, // llvm.riscv.seg8.store.mask
    30 << 8 | 238, // llvm.riscv.sf.mm.e4m3.e4m3
    30 << 8 | 238, // llvm.riscv.sf.mm.e4m3.e5m2
    30 << 8 | 238, // llvm.riscv.sf.mm.e5m2.e4m3
    30 << 8 | 238, // llvm.riscv.sf.mm.e5m2.e5m2
    30 << 8 | 238, // llvm.riscv.sf.mm.f.f
    30 << 8 | 238, // llvm.riscv.sf.mm.s.s
    30 << 8 | 238, // llvm.riscv.sf.mm.s.u
    30 << 8 | 238, // llvm.riscv.sf.mm.u.s
    30 << 8 | 238, // llvm.riscv.sf.mm.u.u
    29 << 8 | 17, // llvm.riscv.sf.vc.fv.se
    29 << 8 | 5, // llvm.riscv.sf.vc.fvv.se
    29 << 8 | 5, // llvm.riscv.sf.vc.fvw.se
    29 << 8 | 131, // llvm.riscv.sf.vc.i.se
    29 << 8 | 16, // llvm.riscv.sf.vc.iv.se
    29 << 8 | 129, // llvm.riscv.sf.vc.ivv.se
    29 << 8 | 129, // llvm.riscv.sf.vc.ivw.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.fv
    29 << 8 | 5, // llvm.riscv.sf.vc.v.fv.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.fvv
    29 << 8 | 5, // llvm.riscv.sf.vc.v.fvv.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.fvw
    29 << 8 | 5, // llvm.riscv.sf.vc.v.fvw.se
    10 << 8 | 92, // llvm.riscv.sf.vc.v.i
    29 << 8 | 92, // llvm.riscv.sf.vc.v.i.se
    10 << 8 | 46, // llvm.riscv.sf.vc.v.iv
    29 << 8 | 46, // llvm.riscv.sf.vc.v.iv.se
    10 << 8 | 129, // llvm.riscv.sf.vc.v.ivv
    29 << 8 | 129, // llvm.riscv.sf.vc.v.ivv.se
    10 << 8 | 129, // llvm.riscv.sf.vc.v.ivw
    29 << 8 | 129, // llvm.riscv.sf.vc.v.ivw.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.vv
    29 << 8 | 5, // llvm.riscv.sf.vc.v.vv.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.vvv
    29 << 8 | 5, // llvm.riscv.sf.vc.v.vvv.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.vvw
    29 << 8 | 5, // llvm.riscv.sf.vc.v.vvw.se
    10 << 8 | 17, // llvm.riscv.sf.vc.v.x
    29 << 8 | 17, // llvm.riscv.sf.vc.v.x.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.xv
    29 << 8 | 5, // llvm.riscv.sf.vc.v.xv.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.xvv
    29 << 8 | 5, // llvm.riscv.sf.vc.v.xvv.se
    10 << 8 | 5, // llvm.riscv.sf.vc.v.xvw
    29 << 8 | 5, // llvm.riscv.sf.vc.v.xvw.se
    29 << 8 | 17, // llvm.riscv.sf.vc.vv.se
    29 << 8 | 5, // llvm.riscv.sf.vc.vvv.se
    29 << 8 | 5, // llvm.riscv.sf.vc.vvw.se
    29 << 8 | 239, // llvm.riscv.sf.vc.x.se
    29 << 8 | 17, // llvm.riscv.sf.vc.xv.se
    29 << 8 | 5, // llvm.riscv.sf.vc.xvv.se
    29 << 8 | 5, // llvm.riscv.sf.vc.xvw.se
    1 << 8 | 1, // llvm.riscv.sf.vfexp
    1 << 8 | 43, // llvm.riscv.sf.vfexp.mask
    1 << 8 | 1, // llvm.riscv.sf.vfexpa
    1 << 8 | 43, // llvm.riscv.sf.vfexpa.mask
    1 << 8 | 35, // llvm.riscv.sf.vfnrclip.x.f.qf
    1 << 8 | 234, // llvm.riscv.sf.vfnrclip.x.f.qf.mask
    1 << 8 | 35, // llvm.riscv.sf.vfnrclip.xu.f.qf
    1 << 8 | 234, // llvm.riscv.sf.vfnrclip.xu.f.qf.mask
    1 << 8 | 43, // llvm.riscv.sf.vfwmacc.4x4x4
    30 << 8 | 18, // llvm.riscv.sf.vlte16
    30 << 8 | 18, // llvm.riscv.sf.vlte32
    30 << 8 | 18, // llvm.riscv.sf.vlte64
    30 << 8 | 18, // llvm.riscv.sf.vlte8
    1 << 8 | 43, // llvm.riscv.sf.vqmacc.2x8x2
    1 << 8 | 43, // llvm.riscv.sf.vqmacc.4x8x4
    1 << 8 | 43, // llvm.riscv.sf.vqmaccsu.2x8x2
    1 << 8 | 43, // llvm.riscv.sf.vqmaccsu.4x8x4
    1 << 8 | 43, // llvm.riscv.sf.vqmaccu.2x8x2
    1 << 8 | 43, // llvm.riscv.sf.vqmaccu.4x8x4
    1 << 8 | 43, // llvm.riscv.sf.vqmaccus.2x8x2
    1 << 8 | 43, // llvm.riscv.sf.vqmaccus.4x8x4
    1 << 8 | 13, // llvm.riscv.sf.vsettk
    1 << 8 | 13, // llvm.riscv.sf.vsettm
    1 << 8 | 13, // llvm.riscv.sf.vsettnt
    107 << 8 | 18, // llvm.riscv.sf.vste16
    107 << 8 | 18, // llvm.riscv.sf.vste32
    107 << 8 | 18, // llvm.riscv.sf.vste64
    107 << 8 | 18, // llvm.riscv.sf.vste8
    30 << 8 | 1, // llvm.riscv.sf.vtdiscard
    30 << 8 | 1, // llvm.riscv.sf.vtmv.t.v
    30 << 8 | 1, // llvm.riscv.sf.vtmv.v.t
    30 << 8 | 78, // llvm.riscv.sf.vtzero.t
    0 << 8 | 1, // llvm.riscv.sha256sig0
    0 << 8 | 1, // llvm.riscv.sha256sig1
    0 << 8 | 1, // llvm.riscv.sha256sum0
    0 << 8 | 1, // llvm.riscv.sha256sum1
    0 << 8 | 1, // llvm.riscv.sha512sig0
    0 << 8 | 1, // llvm.riscv.sha512sig0h
    0 << 8 | 1, // llvm.riscv.sha512sig0l
    0 << 8 | 1, // llvm.riscv.sha512sig1
    0 << 8 | 1, // llvm.riscv.sha512sig1h
    0 << 8 | 1, // llvm.riscv.sha512sig1l
    0 << 8 | 1, // llvm.riscv.sha512sum0
    0 << 8 | 1, // llvm.riscv.sha512sum0r
    0 << 8 | 1, // llvm.riscv.sha512sum1
    0 << 8 | 1, // llvm.riscv.sha512sum1r
    0 << 8 | 1, // llvm.riscv.sm3p0
    0 << 8 | 1, // llvm.riscv.sm3p1
    0 << 8 | 11, // llvm.riscv.sm4ed
    0 << 8 | 11, // llvm.riscv.sm4ks
    23 << 8 | 10, // llvm.riscv.sseg2.load.mask
    24 << 8 | 37, // llvm.riscv.sseg2.store.mask
    23 << 8 | 10, // llvm.riscv.sseg3.load.mask
    24 << 8 | 38, // llvm.riscv.sseg3.store.mask
    23 << 8 | 10, // llvm.riscv.sseg4.load.mask
    24 << 8 | 39, // llvm.riscv.sseg4.store.mask
    23 << 8 | 10, // llvm.riscv.sseg5.load.mask
    24 << 8 | 40, // llvm.riscv.sseg5.store.mask
    23 << 8 | 10, // llvm.riscv.sseg6.load.mask
    24 << 8 | 235, // llvm.riscv.sseg6.store.mask
    23 << 8 | 10, // llvm.riscv.sseg7.load.mask
    24 << 8 | 236, // llvm.riscv.sseg7.store.mask
    23 << 8 | 10, // llvm.riscv.sseg8.load.mask
    24 << 8 | 237, // llvm.riscv.sseg8.store.mask
    1 << 8 | 43, // llvm.riscv.th.vmaqa
    1 << 8 | 47, // llvm.riscv.th.vmaqa.mask
    1 << 8 | 43, // llvm.riscv.th.vmaqasu
    1 << 8 | 47, // llvm.riscv.th.vmaqasu.mask
    1 << 8 | 43, // llvm.riscv.th.vmaqau
    1 << 8 | 47, // llvm.riscv.th.vmaqau.mask
    1 << 8 | 43, // llvm.riscv.th.vmaqaus
    1 << 8 | 47, // llvm.riscv.th.vmaqaus.mask
    0 << 8 | 0, // llvm.riscv.tuple.extract
    0 << 8 | 11, // llvm.riscv.tuple.insert
    0 << 8 | 1, // llvm.riscv.unzip
    1 << 8 | 35, // llvm.riscv.vaadd
    1 << 8 | 234, // llvm.riscv.vaadd.mask
    1 << 8 | 35, // llvm.riscv.vaaddu
    1 << 8 | 234, // llvm.riscv.vaaddu.mask
    1 << 8 | 1, // llvm.riscv.vadc
    1 << 8 | 1, // llvm.riscv.vadd
    1 << 8 | 47, // llvm.riscv.vadd.mask
    1 << 8 | 35, // llvm.riscv.vaesdf.vs
    1 << 8 | 35, // llvm.riscv.vaesdf.vv
    1 << 8 | 35, // llvm.riscv.vaesdm.vs
    1 << 8 | 35, // llvm.riscv.vaesdm.vv
    1 << 8 | 35, // llvm.riscv.vaesef.vs
    1 << 8 | 35, // llvm.riscv.vaesef.vv
    1 << 8 | 35, // llvm.riscv.vaesem.vs
    1 << 8 | 35, // llvm.riscv.vaesem.vv
    1 << 8 | 11, // llvm.riscv.vaeskf1
    1 << 8 | 240, // llvm.riscv.vaeskf2
    1 << 8 | 35, // llvm.riscv.vaesz.vs
    1 << 8 | 1, // llvm.riscv.vand
    1 << 8 | 47, // llvm.riscv.vand.mask
    1 << 8 | 1, // llvm.riscv.vandn
    1 << 8 | 47, // llvm.riscv.vandn.mask
    1 << 8 | 35, // llvm.riscv.vasub
    1 << 8 | 234, // llvm.riscv.vasub.mask
    1 << 8 | 35, // llvm.riscv.vasubu
    1 << 8 | 234, // llvm.riscv.vasubu.mask
    1 << 8 | 1, // llvm.riscv.vbrev
    1 << 8 | 43, // llvm.riscv.vbrev.mask
    1 << 8 | 1, // llvm.riscv.vbrev8
    1 << 8 | 43, // llvm.riscv.vbrev8.mask
    1 << 8 | 1, // llvm.riscv.vclmul
    1 << 8 | 47, // llvm.riscv.vclmul.mask
    1 << 8 | 1, // llvm.riscv.vclmulh
    1 << 8 | 47, // llvm.riscv.vclmulh.mask
    1 << 8 | 1, // llvm.riscv.vclz
    1 << 8 | 43, // llvm.riscv.vclz.mask
    1 << 8 | 1, // llvm.riscv.vcompress
    1 << 8 | 1, // llvm.riscv.vcpop
    1 << 8 | 1, // llvm.riscv.vcpop.mask
    1 << 8 | 1, // llvm.riscv.vcpopv
    1 << 8 | 43, // llvm.riscv.vcpopv.mask
    1 << 8 | 1, // llvm.riscv.vctz
    1 << 8 | 43, // llvm.riscv.vctz.mask
    1 << 8 | 1, // llvm.riscv.vdiv
    1 << 8 | 47, // llvm.riscv.vdiv.mask
    1 << 8 | 1, // llvm.riscv.vdivu
    1 << 8 | 47, // llvm.riscv.vdivu.mask
    1 << 8 | 35, // llvm.riscv.vfadd
    1 << 8 | 234, // llvm.riscv.vfadd.mask
    1 << 8 | 1, // llvm.riscv.vfclass
    1 << 8 | 43, // llvm.riscv.vfclass.mask
    1 << 8 | 11, // llvm.riscv.vfcvt.f.x.v
    1 << 8 | 241, // llvm.riscv.vfcvt.f.x.v.mask
    1 << 8 | 11, // llvm.riscv.vfcvt.f.xu.v
    1 << 8 | 241, // llvm.riscv.vfcvt.f.xu.v.mask
    1 << 8 | 1, // llvm.riscv.vfcvt.rtz.x.f.v
    1 << 8 | 43, // llvm.riscv.vfcvt.rtz.x.f.v.mask
    1 << 8 | 1, // llvm.riscv.vfcvt.rtz.xu.f.v
    1 << 8 | 43, // llvm.riscv.vfcvt.rtz.xu.f.v.mask
    1 << 8 | 11, // llvm.riscv.vfcvt.x.f.v
    1 << 8 | 241, // llvm.riscv.vfcvt.x.f.v.mask
    1 << 8 | 11, // llvm.riscv.vfcvt.xu.f.v
    1 << 8 | 241, // llvm.riscv.vfcvt.xu.f.v.mask
    1 << 8 | 35, // llvm.riscv.vfdiv
    1 << 8 | 234, // llvm.riscv.vfdiv.mask
    1 << 8 | 1, // llvm.riscv.vfirst
    1 << 8 | 1, // llvm.riscv.vfirst.mask
    1 << 8 | 241, // llvm.riscv.vfmacc
    1 << 8 | 234, // llvm.riscv.vfmacc.mask
    1 << 8 | 241, // llvm.riscv.vfmadd
    1 << 8 | 234, // llvm.riscv.vfmadd.mask
    1 << 8 | 1, // llvm.riscv.vfmax
    1 << 8 | 47, // llvm.riscv.vfmax.mask
    1 << 8 | 1, // llvm.riscv.vfmerge
    1 << 8 | 1, // llvm.riscv.vfmin
    1 << 8 | 47, // llvm.riscv.vfmin.mask
    1 << 8 | 241, // llvm.riscv.vfmsac
    1 << 8 | 234, // llvm.riscv.vfmsac.mask
    1 << 8 | 241, // llvm.riscv.vfmsub
    1 << 8 | 234, // llvm.riscv.vfmsub.mask
    1 << 8 | 35, // llvm.riscv.vfmul
    1 << 8 | 234, // llvm.riscv.vfmul.mask
    1 << 8 | 1, // llvm.riscv.vfmv.f.s
    1 << 8 | 1, // llvm.riscv.vfmv.s.f
    1 << 8 | 1, // llvm.riscv.vfmv.v.f
    1 << 8 | 11, // llvm.riscv.vfncvt.f.f.q
    1 << 8 | 11, // llvm.riscv.vfncvt.f.f.q.alt
    1 << 8 | 241, // llvm.riscv.vfncvt.f.f.q.alt.mask
    1 << 8 | 241, // llvm.riscv.vfncvt.f.f.q.mask
    1 << 8 | 11, // llvm.riscv.vfncvt.f.f.w
    1 << 8 | 11, // llvm.riscv.vfncvt.f.f.w.alt
    1 << 8 | 241, // llvm.riscv.vfncvt.f.f.w.alt.mask
    1 << 8 | 241, // llvm.riscv.vfncvt.f.f.w.mask
    1 << 8 | 11, // llvm.riscv.vfncvt.f.x.w
    1 << 8 | 241, // llvm.riscv.vfncvt.f.x.w.mask
    1 << 8 | 11, // llvm.riscv.vfncvt.f.xu.w
    1 << 8 | 241, // llvm.riscv.vfncvt.f.xu.w.mask
    1 << 8 | 1, // llvm.riscv.vfncvt.rod.f.f.w
    1 << 8 | 43, // llvm.riscv.vfncvt.rod.f.f.w.mask
    1 << 8 | 1, // llvm.riscv.vfncvt.rtz.x.f.w
    1 << 8 | 43, // llvm.riscv.vfncvt.rtz.x.f.w.mask
    1 << 8 | 1, // llvm.riscv.vfncvt.rtz.xu.f.w
    1 << 8 | 43, // llvm.riscv.vfncvt.rtz.xu.f.w.mask
    1 << 8 | 11, // llvm.riscv.vfncvt.sat.f.f.q
    1 << 8 | 11, // llvm.riscv.vfncvt.sat.f.f.q.alt
    1 << 8 | 241, // llvm.riscv.vfncvt.sat.f.f.q.alt.mask
    1 << 8 | 241, // llvm.riscv.vfncvt.sat.f.f.q.mask
    1 << 8 | 11, // llvm.riscv.vfncvt.sat.f.f.w
    1 << 8 | 11, // llvm.riscv.vfncvt.sat.f.f.w.alt
    1 << 8 | 241, // llvm.riscv.vfncvt.sat.f.f.w.alt.mask
    1 << 8 | 241, // llvm.riscv.vfncvt.sat.f.f.w.mask
    1 << 8 | 11, // llvm.riscv.vfncvt.x.f.w
    1 << 8 | 241, // llvm.riscv.vfncvt.x.f.w.mask
    1 << 8 | 11, // llvm.riscv.vfncvt.xu.f.w
    1 << 8 | 241, // llvm.riscv.vfncvt.xu.f.w.mask
    1 << 8 | 11, // llvm.riscv.vfncvtbf16.f.f.w
    1 << 8 | 241, // llvm.riscv.vfncvtbf16.f.f.w.mask
    1 << 8 | 241, // llvm.riscv.vfnmacc
    1 << 8 | 234, // llvm.riscv.vfnmacc.mask
    1 << 8 | 241, // llvm.riscv.vfnmadd
    1 << 8 | 234, // llvm.riscv.vfnmadd.mask
    1 << 8 | 241, // llvm.riscv.vfnmsac
    1 << 8 | 234, // llvm.riscv.vfnmsac.mask
    1 << 8 | 241, // llvm.riscv.vfnmsub
    1 << 8 | 234, // llvm.riscv.vfnmsub.mask
    1 << 8 | 35, // llvm.riscv.vfrdiv
    1 << 8 | 234, // llvm.riscv.vfrdiv.mask
    1 << 8 | 11, // llvm.riscv.vfrec7
    1 << 8 | 241, // llvm.riscv.vfrec7.mask
    1 << 8 | 1, // llvm.riscv.vfredmax
    1 << 8 | 1, // llvm.riscv.vfredmax.mask
    1 << 8 | 1, // llvm.riscv.vfredmin
    1 << 8 | 1, // llvm.riscv.vfredmin.mask
    1 << 8 | 35, // llvm.riscv.vfredosum
    1 << 8 | 43, // llvm.riscv.vfredosum.mask
    1 << 8 | 35, // llvm.riscv.vfredusum
    1 << 8 | 43, // llvm.riscv.vfredusum.mask
    1 << 8 | 1, // llvm.riscv.vfrsqrt7
    1 << 8 | 43, // llvm.riscv.vfrsqrt7.mask
    1 << 8 | 35, // llvm.riscv.vfrsub
    1 << 8 | 234, // llvm.riscv.vfrsub.mask
    1 << 8 | 1, // llvm.riscv.vfsgnj
    1 << 8 | 47, // llvm.riscv.vfsgnj.mask
    1 << 8 | 1, // llvm.riscv.vfsgnjn
    1 << 8 | 47, // llvm.riscv.vfsgnjn.mask
    1 << 8 | 1, // llvm.riscv.vfsgnjx
    1 << 8 | 47, // llvm.riscv.vfsgnjx.mask
    1 << 8 | 1, // llvm.riscv.vfslide1down
    1 << 8 | 47, // llvm.riscv.vfslide1down.mask
    1 << 8 | 1, // llvm.riscv.vfslide1up
    1 << 8 | 47, // llvm.riscv.vfslide1up.mask
    1 << 8 | 11, // llvm.riscv.vfsqrt
    1 << 8 | 241, // llvm.riscv.vfsqrt.mask
    1 << 8 | 35, // llvm.riscv.vfsub
    1 << 8 | 234, // llvm.riscv.vfsub.mask
    1 << 8 | 35, // llvm.riscv.vfwadd
    1 << 8 | 234, // llvm.riscv.vfwadd.mask
    1 << 8 | 35, // llvm.riscv.vfwadd.w
    1 << 8 | 234, // llvm.riscv.vfwadd.w.mask
    1 << 8 | 1, // llvm.riscv.vfwcvt.f.f.v
    1 << 8 | 1, // llvm.riscv.vfwcvt.f.f.v.alt
    1 << 8 | 43, // llvm.riscv.vfwcvt.f.f.v.alt.mask
    1 << 8 | 43, // llvm.riscv.vfwcvt.f.f.v.mask
    1 << 8 | 1, // llvm.riscv.vfwcvt.f.x.v
    1 << 8 | 43, // llvm.riscv.vfwcvt.f.x.v.mask
    1 << 8 | 1, // llvm.riscv.vfwcvt.f.xu.v
    1 << 8 | 43, // llvm.riscv.vfwcvt.f.xu.v.mask
    1 << 8 | 1, // llvm.riscv.vfwcvt.rtz.x.f.v
    1 << 8 | 43, // llvm.riscv.vfwcvt.rtz.x.f.v.mask
    1 << 8 | 1, // llvm.riscv.vfwcvt.rtz.xu.f.v
    1 << 8 | 43, // llvm.riscv.vfwcvt.rtz.xu.f.v.mask
    1 << 8 | 11, // llvm.riscv.vfwcvt.x.f.v
    1 << 8 | 241, // llvm.riscv.vfwcvt.x.f.v.mask
    1 << 8 | 11, // llvm.riscv.vfwcvt.xu.f.v
    1 << 8 | 241, // llvm.riscv.vfwcvt.xu.f.v.mask
    1 << 8 | 1, // llvm.riscv.vfwcvtbf16.f.f.v
    1 << 8 | 43, // llvm.riscv.vfwcvtbf16.f.f.v.mask
    1 << 8 | 241, // llvm.riscv.vfwmacc
    1 << 8 | 234, // llvm.riscv.vfwmacc.mask
    1 << 8 | 241, // llvm.riscv.vfwmaccbf16
    1 << 8 | 234, // llvm.riscv.vfwmaccbf16.mask
    1 << 8 | 241, // llvm.riscv.vfwmsac
    1 << 8 | 234, // llvm.riscv.vfwmsac.mask
    1 << 8 | 35, // llvm.riscv.vfwmul
    1 << 8 | 234, // llvm.riscv.vfwmul.mask
    1 << 8 | 241, // llvm.riscv.vfwnmacc
    1 << 8 | 234, // llvm.riscv.vfwnmacc.mask
    1 << 8 | 241, // llvm.riscv.vfwnmsac
    1 << 8 | 234, // llvm.riscv.vfwnmsac.mask
    1 << 8 | 35, // llvm.riscv.vfwredosum
    1 << 8 | 43, // llvm.riscv.vfwredosum.mask
    1 << 8 | 35, // llvm.riscv.vfwredusum
    1 << 8 | 43, // llvm.riscv.vfwredusum.mask
    1 << 8 | 35, // llvm.riscv.vfwsub
    1 << 8 | 234, // llvm.riscv.vfwsub.mask
    1 << 8 | 35, // llvm.riscv.vfwsub.w
    1 << 8 | 234, // llvm.riscv.vfwsub.w.mask
    1 << 8 | 43, // llvm.riscv.vghsh
    1 << 8 | 35, // llvm.riscv.vgmul.vv
    1 << 8 | 1, // llvm.riscv.vid
    1 << 8 | 35, // llvm.riscv.vid.mask
    1 << 8 | 1, // llvm.riscv.viota
    1 << 8 | 43, // llvm.riscv.viota.mask
    2 << 8 | 18, // llvm.riscv.vle
    2 << 8 | 100, // llvm.riscv.vle.mask
    7 << 8 | 18, // llvm.riscv.vleff
    7 << 8 | 100, // llvm.riscv.vleff.mask
    2 << 8 | 10, // llvm.riscv.vlm
    23 << 8 | 18, // llvm.riscv.vloxei
    23 << 8 | 109, // llvm.riscv.vloxei.mask
    23 << 8 | 100, // llvm.riscv.vloxseg2
    23 << 8 | 242, // llvm.riscv.vloxseg2.mask
    23 << 8 | 100, // llvm.riscv.vloxseg3
    23 << 8 | 242, // llvm.riscv.vloxseg3.mask
    23 << 8 | 100, // llvm.riscv.vloxseg4
    23 << 8 | 242, // llvm.riscv.vloxseg4.mask
    23 << 8 | 100, // llvm.riscv.vloxseg5
    23 << 8 | 242, // llvm.riscv.vloxseg5.mask
    23 << 8 | 100, // llvm.riscv.vloxseg6
    23 << 8 | 242, // llvm.riscv.vloxseg6.mask
    23 << 8 | 100, // llvm.riscv.vloxseg7
    23 << 8 | 242, // llvm.riscv.vloxseg7.mask
    23 << 8 | 100, // llvm.riscv.vloxseg8
    23 << 8 | 242, // llvm.riscv.vloxseg8.mask
    23 << 8 | 18, // llvm.riscv.vlse
    23 << 8 | 109, // llvm.riscv.vlse.mask
    2 << 8 | 49, // llvm.riscv.vlseg2
    2 << 8 | 243, // llvm.riscv.vlseg2.mask
    7 << 8 | 49, // llvm.riscv.vlseg2ff
    7 << 8 | 243, // llvm.riscv.vlseg2ff.mask
    2 << 8 | 49, // llvm.riscv.vlseg3
    2 << 8 | 243, // llvm.riscv.vlseg3.mask
    7 << 8 | 49, // llvm.riscv.vlseg3ff
    7 << 8 | 243, // llvm.riscv.vlseg3ff.mask
    2 << 8 | 49, // llvm.riscv.vlseg4
    2 << 8 | 243, // llvm.riscv.vlseg4.mask
    7 << 8 | 49, // llvm.riscv.vlseg4ff
    7 << 8 | 243, // llvm.riscv.vlseg4ff.mask
    2 << 8 | 49, // llvm.riscv.vlseg5
    2 << 8 | 243, // llvm.riscv.vlseg5.mask
    7 << 8 | 49, // llvm.riscv.vlseg5ff
    7 << 8 | 243, // llvm.riscv.vlseg5ff.mask
    2 << 8 | 49, // llvm.riscv.vlseg6
    2 << 8 | 243, // llvm.riscv.vlseg6.mask
    7 << 8 | 49, // llvm.riscv.vlseg6ff
    7 << 8 | 243, // llvm.riscv.vlseg6ff.mask
    2 << 8 | 49, // llvm.riscv.vlseg7
    2 << 8 | 243, // llvm.riscv.vlseg7.mask
    7 << 8 | 49, // llvm.riscv.vlseg7ff
    7 << 8 | 243, // llvm.riscv.vlseg7ff.mask
    2 << 8 | 49, // llvm.riscv.vlseg8
    2 << 8 | 243, // llvm.riscv.vlseg8.mask
    7 << 8 | 49, // llvm.riscv.vlseg8ff
    7 << 8 | 243, // llvm.riscv.vlseg8ff.mask
    23 << 8 | 100, // llvm.riscv.vlsseg2
    23 << 8 | 242, // llvm.riscv.vlsseg2.mask
    23 << 8 | 100, // llvm.riscv.vlsseg3
    23 << 8 | 242, // llvm.riscv.vlsseg3.mask
    23 << 8 | 100, // llvm.riscv.vlsseg4
    23 << 8 | 242, // llvm.riscv.vlsseg4.mask
    23 << 8 | 100, // llvm.riscv.vlsseg5
    23 << 8 | 242, // llvm.riscv.vlsseg5.mask
    23 << 8 | 100, // llvm.riscv.vlsseg6
    23 << 8 | 242, // llvm.riscv.vlsseg6.mask
    23 << 8 | 100, // llvm.riscv.vlsseg7
    23 << 8 | 242, // llvm.riscv.vlsseg7.mask
    23 << 8 | 100, // llvm.riscv.vlsseg8
    23 << 8 | 242, // llvm.riscv.vlsseg8.mask
    23 << 8 | 18, // llvm.riscv.vluxei
    23 << 8 | 109, // llvm.riscv.vluxei.mask
    23 << 8 | 100, // llvm.riscv.vluxseg2
    23 << 8 | 242, // llvm.riscv.vluxseg2.mask
    23 << 8 | 100, // llvm.riscv.vluxseg3
    23 << 8 | 242, // llvm.riscv.vluxseg3.mask
    23 << 8 | 100, // llvm.riscv.vluxseg4
    23 << 8 | 242, // llvm.riscv.vluxseg4.mask
    23 << 8 | 100, // llvm.riscv.vluxseg5
    23 << 8 | 242, // llvm.riscv.vluxseg5.mask
    23 << 8 | 100, // llvm.riscv.vluxseg6
    23 << 8 | 242, // llvm.riscv.vluxseg6.mask
    23 << 8 | 100, // llvm.riscv.vluxseg7
    23 << 8 | 242, // llvm.riscv.vluxseg7.mask
    23 << 8 | 100, // llvm.riscv.vluxseg8
    23 << 8 | 242, // llvm.riscv.vluxseg8.mask
    1 << 8 | 43, // llvm.riscv.vmacc
    1 << 8 | 47, // llvm.riscv.vmacc.mask
    1 << 8 | 1, // llvm.riscv.vmadc
    1 << 8 | 1, // llvm.riscv.vmadc.carry.in
    1 << 8 | 43, // llvm.riscv.vmadd
    1 << 8 | 47, // llvm.riscv.vmadd.mask
    1 << 8 | 1, // llvm.riscv.vmand
    1 << 8 | 1, // llvm.riscv.vmandn
    1 << 8 | 1, // llvm.riscv.vmax
    1 << 8 | 47, // llvm.riscv.vmax.mask
    1 << 8 | 1, // llvm.riscv.vmaxu
    1 << 8 | 47, // llvm.riscv.vmaxu.mask
    1 << 8 | 1, // llvm.riscv.vmclr
    1 << 8 | 1, // llvm.riscv.vmerge
    1 << 8 | 1, // llvm.riscv.vmfeq
    1 << 8 | 1, // llvm.riscv.vmfeq.mask
    1 << 8 | 1, // llvm.riscv.vmfge
    1 << 8 | 1, // llvm.riscv.vmfge.mask
    1 << 8 | 1, // llvm.riscv.vmfgt
    1 << 8 | 1, // llvm.riscv.vmfgt.mask
    1 << 8 | 1, // llvm.riscv.vmfle
    1 << 8 | 1, // llvm.riscv.vmfle.mask
    1 << 8 | 1, // llvm.riscv.vmflt
    1 << 8 | 1, // llvm.riscv.vmflt.mask
    1 << 8 | 1, // llvm.riscv.vmfne
    1 << 8 | 1, // llvm.riscv.vmfne.mask
    1 << 8 | 1, // llvm.riscv.vmin
    1 << 8 | 47, // llvm.riscv.vmin.mask
    1 << 8 | 1, // llvm.riscv.vminu
    1 << 8 | 47, // llvm.riscv.vminu.mask
    1 << 8 | 1, // llvm.riscv.vmnand
    1 << 8 | 1, // llvm.riscv.vmnor
    1 << 8 | 1, // llvm.riscv.vmor
    1 << 8 | 1, // llvm.riscv.vmorn
    1 << 8 | 1, // llvm.riscv.vmsbc
    1 << 8 | 1, // llvm.riscv.vmsbc.borrow.in
    1 << 8 | 1, // llvm.riscv.vmsbf
    1 << 8 | 1, // llvm.riscv.vmsbf.mask
    1 << 8 | 1, // llvm.riscv.vmseq
    1 << 8 | 1, // llvm.riscv.vmseq.mask
    1 << 8 | 1, // llvm.riscv.vmset
    1 << 8 | 1, // llvm.riscv.vmsge
    1 << 8 | 1, // llvm.riscv.vmsge.mask
    1 << 8 | 1, // llvm.riscv.vmsgeu
    1 << 8 | 1, // llvm.riscv.vmsgeu.mask
    1 << 8 | 1, // llvm.riscv.vmsgt
    1 << 8 | 1, // llvm.riscv.vmsgt.mask
    1 << 8 | 1, // llvm.riscv.vmsgtu
    1 << 8 | 1, // llvm.riscv.vmsgtu.mask
    1 << 8 | 1, // llvm.riscv.vmsif
    1 << 8 | 1, // llvm.riscv.vmsif.mask
    1 << 8 | 1, // llvm.riscv.vmsle
    1 << 8 | 1, // llvm.riscv.vmsle.mask
    1 << 8 | 1, // llvm.riscv.vmsleu
    1 << 8 | 1, // llvm.riscv.vmsleu.mask
    1 << 8 | 1, // llvm.riscv.vmslt
    1 << 8 | 1, // llvm.riscv.vmslt.mask
    1 << 8 | 1, // llvm.riscv.vmsltu
    1 << 8 | 1, // llvm.riscv.vmsltu.mask
    1 << 8 | 1, // llvm.riscv.vmsne
    1 << 8 | 1, // llvm.riscv.vmsne.mask
    1 << 8 | 1, // llvm.riscv.vmsof
    1 << 8 | 1, // llvm.riscv.vmsof.mask
    1 << 8 | 1, // llvm.riscv.vmul
    1 << 8 | 47, // llvm.riscv.vmul.mask
    1 << 8 | 1, // llvm.riscv.vmulh
    1 << 8 | 47, // llvm.riscv.vmulh.mask
    1 << 8 | 1, // llvm.riscv.vmulhsu
    1 << 8 | 47, // llvm.riscv.vmulhsu.mask
    1 << 8 | 1, // llvm.riscv.vmulhu
    1 << 8 | 47, // llvm.riscv.vmulhu.mask
    1 << 8 | 1, // llvm.riscv.vmv.s.x
    1 << 8 | 1, // llvm.riscv.vmv.v.v
    1 << 8 | 1, // llvm.riscv.vmv.v.x
    1 << 8 | 1, // llvm.riscv.vmv.x.s
    1 << 8 | 1, // llvm.riscv.vmxnor
    1 << 8 | 1, // llvm.riscv.vmxor
    1 << 8 | 35, // llvm.riscv.vnclip
    1 << 8 | 234, // llvm.riscv.vnclip.mask
    1 << 8 | 35, // llvm.riscv.vnclipu
    1 << 8 | 234, // llvm.riscv.vnclipu.mask
    1 << 8 | 43, // llvm.riscv.vnmsac
    1 << 8 | 47, // llvm.riscv.vnmsac.mask
    1 << 8 | 43, // llvm.riscv.vnmsub
    1 << 8 | 47, // llvm.riscv.vnmsub.mask
    1 << 8 | 1, // llvm.riscv.vnsra
    1 << 8 | 47, // llvm.riscv.vnsra.mask
    1 << 8 | 1, // llvm.riscv.vnsrl
    1 << 8 | 47, // llvm.riscv.vnsrl.mask
    1 << 8 | 1, // llvm.riscv.vor
    1 << 8 | 47, // llvm.riscv.vor.mask
    1 << 8 | 43, // llvm.riscv.vqdot
    1 << 8 | 47, // llvm.riscv.vqdot.mask
    1 << 8 | 43, // llvm.riscv.vqdotsu
    1 << 8 | 47, // llvm.riscv.vqdotsu.mask
    1 << 8 | 43, // llvm.riscv.vqdotu
    1 << 8 | 47, // llvm.riscv.vqdotu.mask
    1 << 8 | 43, // llvm.riscv.vqdotus
    1 << 8 | 47, // llvm.riscv.vqdotus.mask
    1 << 8 | 1, // llvm.riscv.vredand
    1 << 8 | 1, // llvm.riscv.vredand.mask
    1 << 8 | 1, // llvm.riscv.vredmax
    1 << 8 | 1, // llvm.riscv.vredmax.mask
    1 << 8 | 1, // llvm.riscv.vredmaxu
    1 << 8 | 1, // llvm.riscv.vredmaxu.mask
    1 << 8 | 1, // llvm.riscv.vredmin
    1 << 8 | 1, // llvm.riscv.vredmin.mask
    1 << 8 | 1, // llvm.riscv.vredminu
    1 << 8 | 1, // llvm.riscv.vredminu.mask
    1 << 8 | 1, // llvm.riscv.vredor
    1 << 8 | 1, // llvm.riscv.vredor.mask
    1 << 8 | 1, // llvm.riscv.vredsum
    1 << 8 | 1, // llvm.riscv.vredsum.mask
    1 << 8 | 1, // llvm.riscv.vredxor
    1 << 8 | 1, // llvm.riscv.vredxor.mask
    1 << 8 | 1, // llvm.riscv.vrem
    1 << 8 | 47, // llvm.riscv.vrem.mask
    1 << 8 | 1, // llvm.riscv.vremu
    1 << 8 | 47, // llvm.riscv.vremu.mask
    1 << 8 | 1, // llvm.riscv.vrev8
    1 << 8 | 43, // llvm.riscv.vrev8.mask
    1 << 8 | 1, // llvm.riscv.vrgather.vv
    1 << 8 | 47, // llvm.riscv.vrgather.vv.mask
    1 << 8 | 1, // llvm.riscv.vrgather.vx
    1 << 8 | 47, // llvm.riscv.vrgather.vx.mask
    1 << 8 | 1, // llvm.riscv.vrgatherei16.vv
    1 << 8 | 47, // llvm.riscv.vrgatherei16.vv.mask
    1 << 8 | 1, // llvm.riscv.vrol
    1 << 8 | 47, // llvm.riscv.vrol.mask
    1 << 8 | 1, // llvm.riscv.vror
    1 << 8 | 47, // llvm.riscv.vror.mask
    1 << 8 | 1, // llvm.riscv.vrsub
    1 << 8 | 47, // llvm.riscv.vrsub.mask
    1 << 8 | 1, // llvm.riscv.vsadd
    1 << 8 | 47, // llvm.riscv.vsadd.mask
    1 << 8 | 1, // llvm.riscv.vsaddu
    1 << 8 | 47, // llvm.riscv.vsaddu.mask
    1 << 8 | 1, // llvm.riscv.vsbc
    19 << 8 | 18, // llvm.riscv.vse
    19 << 8 | 18, // llvm.riscv.vse.mask
    1 << 8 | 13, // llvm.riscv.vsetvli
    1 << 8 | 17, // llvm.riscv.vsetvlimax
    1 << 8 | 1, // llvm.riscv.vsext
    1 << 8 | 43, // llvm.riscv.vsext.mask
    1 << 8 | 43, // llvm.riscv.vsha2ch
    1 << 8 | 43, // llvm.riscv.vsha2cl
    1 << 8 | 43, // llvm.riscv.vsha2ms
    1 << 8 | 1, // llvm.riscv.vslide1down
    1 << 8 | 47, // llvm.riscv.vslide1down.mask
    1 << 8 | 1, // llvm.riscv.vslide1up
    1 << 8 | 47, // llvm.riscv.vslide1up.mask
    1 << 8 | 43, // llvm.riscv.vslidedown
    1 << 8 | 47, // llvm.riscv.vslidedown.mask
    1 << 8 | 43, // llvm.riscv.vslideup
    1 << 8 | 47, // llvm.riscv.vslideup.mask
    1 << 8 | 1, // llvm.riscv.vsll
    1 << 8 | 47, // llvm.riscv.vsll.mask
    19 << 8 | 18, // llvm.riscv.vsm
    1 << 8 | 240, // llvm.riscv.vsm3c
    1 << 8 | 1, // llvm.riscv.vsm3me
    1 << 8 | 11, // llvm.riscv.vsm4k
    1 << 8 | 35, // llvm.riscv.vsm4r.vs
    1 << 8 | 35, // llvm.riscv.vsm4r.vv
    1 << 8 | 35, // llvm.riscv.vsmul
    1 << 8 | 234, // llvm.riscv.vsmul.mask
    24 << 8 | 18, // llvm.riscv.vsoxei
    24 << 8 | 18, // llvm.riscv.vsoxei.mask
    24 << 8 | 100, // llvm.riscv.vsoxseg2
    24 << 8 | 109, // llvm.riscv.vsoxseg2.mask
    24 << 8 | 100, // llvm.riscv.vsoxseg3
    24 << 8 | 109, // llvm.riscv.vsoxseg3.mask
    24 << 8 | 100, // llvm.riscv.vsoxseg4
    24 << 8 | 109, // llvm.riscv.vsoxseg4.mask
    24 << 8 | 100, // llvm.riscv.vsoxseg5
    24 << 8 | 109, // llvm.riscv.vsoxseg5.mask
    24 << 8 | 100, // llvm.riscv.vsoxseg6
    24 << 8 | 109, // llvm.riscv.vsoxseg6.mask
    24 << 8 | 100, // llvm.riscv.vsoxseg7
    24 << 8 | 109, // llvm.riscv.vsoxseg7.mask
    24 << 8 | 100, // llvm.riscv.vsoxseg8
    24 << 8 | 109, // llvm.riscv.vsoxseg8.mask
    1 << 8 | 1, // llvm.riscv.vsra
    1 << 8 | 47, // llvm.riscv.vsra.mask
    1 << 8 | 1, // llvm.riscv.vsrl
    1 << 8 | 47, // llvm.riscv.vsrl.mask
    24 << 8 | 18, // llvm.riscv.vsse
    24 << 8 | 18, // llvm.riscv.vsse.mask
    19 << 8 | 49, // llvm.riscv.vsseg2
    19 << 8 | 100, // llvm.riscv.vsseg2.mask
    19 << 8 | 49, // llvm.riscv.vsseg3
    19 << 8 | 100, // llvm.riscv.vsseg3.mask
    19 << 8 | 49, // llvm.riscv.vsseg4
    19 << 8 | 100, // llvm.riscv.vsseg4.mask
    19 << 8 | 49, // llvm.riscv.vsseg5
    19 << 8 | 100, // llvm.riscv.vsseg5.mask
    19 << 8 | 49, // llvm.riscv.vsseg6
    19 << 8 | 100, // llvm.riscv.vsseg6.mask
    19 << 8 | 49, // llvm.riscv.vsseg7
    19 << 8 | 100, // llvm.riscv.vsseg7.mask
    19 << 8 | 49, // llvm.riscv.vsseg8
    19 << 8 | 100, // llvm.riscv.vsseg8.mask
    1 << 8 | 35, // llvm.riscv.vssra
    1 << 8 | 234, // llvm.riscv.vssra.mask
    1 << 8 | 35, // llvm.riscv.vssrl
    1 << 8 | 234, // llvm.riscv.vssrl.mask
    24 << 8 | 100, // llvm.riscv.vssseg2
    24 << 8 | 109, // llvm.riscv.vssseg2.mask
    24 << 8 | 100, // llvm.riscv.vssseg3
    24 << 8 | 109, // llvm.riscv.vssseg3.mask
    24 << 8 | 100, // llvm.riscv.vssseg4
    24 << 8 | 109, // llvm.riscv.vssseg4.mask
    24 << 8 | 100, // llvm.riscv.vssseg5
    24 << 8 | 109, // llvm.riscv.vssseg5.mask
    24 << 8 | 100, // llvm.riscv.vssseg6
    24 << 8 | 109, // llvm.riscv.vssseg6.mask
    24 << 8 | 100, // llvm.riscv.vssseg7
    24 << 8 | 109, // llvm.riscv.vssseg7.mask
    24 << 8 | 100, // llvm.riscv.vssseg8
    24 << 8 | 109, // llvm.riscv.vssseg8.mask
    1 << 8 | 1, // llvm.riscv.vssub
    1 << 8 | 47, // llvm.riscv.vssub.mask
    1 << 8 | 1, // llvm.riscv.vssubu
    1 << 8 | 47, // llvm.riscv.vssubu.mask
    1 << 8 | 1, // llvm.riscv.vsub
    1 << 8 | 47, // llvm.riscv.vsub.mask
    24 << 8 | 18, // llvm.riscv.vsuxei
    24 << 8 | 18, // llvm.riscv.vsuxei.mask
    24 << 8 | 100, // llvm.riscv.vsuxseg2
    24 << 8 | 109, // llvm.riscv.vsuxseg2.mask
    24 << 8 | 100, // llvm.riscv.vsuxseg3
    24 << 8 | 109, // llvm.riscv.vsuxseg3.mask
    24 << 8 | 100, // llvm.riscv.vsuxseg4
    24 << 8 | 109, // llvm.riscv.vsuxseg4.mask
    24 << 8 | 100, // llvm.riscv.vsuxseg5
    24 << 8 | 109, // llvm.riscv.vsuxseg5.mask
    24 << 8 | 100, // llvm.riscv.vsuxseg6
    24 << 8 | 109, // llvm.riscv.vsuxseg6.mask
    24 << 8 | 100, // llvm.riscv.vsuxseg7
    24 << 8 | 109, // llvm.riscv.vsuxseg7.mask
    24 << 8 | 100, // llvm.riscv.vsuxseg8
    24 << 8 | 109, // llvm.riscv.vsuxseg8.mask
    1 << 8 | 1, // llvm.riscv.vwadd
    1 << 8 | 47, // llvm.riscv.vwadd.mask
    1 << 8 | 1, // llvm.riscv.vwadd.w
    1 << 8 | 47, // llvm.riscv.vwadd.w.mask
    1 << 8 | 1, // llvm.riscv.vwaddu
    1 << 8 | 47, // llvm.riscv.vwaddu.mask
    1 << 8 | 1, // llvm.riscv.vwaddu.w
    1 << 8 | 47, // llvm.riscv.vwaddu.w.mask
    1 << 8 | 43, // llvm.riscv.vwmacc
    1 << 8 | 47, // llvm.riscv.vwmacc.mask
    1 << 8 | 43, // llvm.riscv.vwmaccsu
    1 << 8 | 47, // llvm.riscv.vwmaccsu.mask
    1 << 8 | 43, // llvm.riscv.vwmaccu
    1 << 8 | 47, // llvm.riscv.vwmaccu.mask
    1 << 8 | 43, // llvm.riscv.vwmaccus
    1 << 8 | 47, // llvm.riscv.vwmaccus.mask
    1 << 8 | 1, // llvm.riscv.vwmul
    1 << 8 | 47, // llvm.riscv.vwmul.mask
    1 << 8 | 1, // llvm.riscv.vwmulsu
    1 << 8 | 47, // llvm.riscv.vwmulsu.mask
    1 << 8 | 1, // llvm.riscv.vwmulu
    1 << 8 | 47, // llvm.riscv.vwmulu.mask
    1 << 8 | 1, // llvm.riscv.vwredsum
    1 << 8 | 1, // llvm.riscv.vwredsum.mask
    1 << 8 | 1, // llvm.riscv.vwredsumu
    1 << 8 | 1, // llvm.riscv.vwredsumu.mask
    1 << 8 | 1, // llvm.riscv.vwsll
    1 << 8 | 47, // llvm.riscv.vwsll.mask
    1 << 8 | 1, // llvm.riscv.vwsub
    1 << 8 | 47, // llvm.riscv.vwsub.mask
    1 << 8 | 1, // llvm.riscv.vwsub.w
    1 << 8 | 47, // llvm.riscv.vwsub.w.mask
    1 << 8 | 1, // llvm.riscv.vwsubu
    1 << 8 | 47, // llvm.riscv.vwsubu.mask
    1 << 8 | 1, // llvm.riscv.vwsubu.w
    1 << 8 | 47, // llvm.riscv.vwsubu.w.mask
    1 << 8 | 1, // llvm.riscv.vxor
    1 << 8 | 47, // llvm.riscv.vxor.mask
    1 << 8 | 1, // llvm.riscv.vzext
    1 << 8 | 43, // llvm.riscv.vzext.mask
    0 << 8 | 1, // llvm.riscv.xperm4
    0 << 8 | 1, // llvm.riscv.xperm8
    0 << 8 | 1, // llvm.riscv.zip
    10 << 8 | 1, // llvm.s390.bdepg
    10 << 8 | 1, // llvm.s390.bextg
    5 << 8 | 1, // llvm.s390.efpc
    10 << 8 | 1, // llvm.s390.etnd
    10 << 8 | 0, // llvm.s390.lcbb
    79 << 8 | 1, // llvm.s390.ntstg
    5 << 8 | 1, // llvm.s390.ppa.txassist
    5 << 8 | 1, // llvm.s390.sfpc
    108 << 8 | 1, // llvm.s390.tabort
    109 << 8 | 1, // llvm.s390.tbegin
    109 << 8 | 1, // llvm.s390.tbegin.nofloat
    109 << 8 | 1, // llvm.s390.tbeginc
    10 << 8 | 1, // llvm.s390.tdc
    5 << 8 | 1, // llvm.s390.tend
    10 << 8 | 1, // llvm.s390.vaccb
    10 << 8 | 1, // llvm.s390.vacccq
    10 << 8 | 1, // llvm.s390.vaccf
    10 << 8 | 1, // llvm.s390.vaccg
    10 << 8 | 1, // llvm.s390.vacch
    10 << 8 | 1, // llvm.s390.vaccq
    10 << 8 | 1, // llvm.s390.vacq
    10 << 8 | 1, // llvm.s390.vaq
    10 << 8 | 1, // llvm.s390.vavgb
    10 << 8 | 1, // llvm.s390.vavgf
    10 << 8 | 1, // llvm.s390.vavgg
    10 << 8 | 1, // llvm.s390.vavgh
    10 << 8 | 1, // llvm.s390.vavglb
    10 << 8 | 1, // llvm.s390.vavglf
    10 << 8 | 1, // llvm.s390.vavglg
    10 << 8 | 1, // llvm.s390.vavglh
    10 << 8 | 1, // llvm.s390.vavglq
    10 << 8 | 1, // llvm.s390.vavgq
    10 << 8 | 1, // llvm.s390.vbperm
    10 << 8 | 1, // llvm.s390.vceqbs
    10 << 8 | 1, // llvm.s390.vceqfs
    10 << 8 | 1, // llvm.s390.vceqgs
    10 << 8 | 1, // llvm.s390.vceqhs
    10 << 8 | 1, // llvm.s390.vceqqs
    10 << 8 | 0, // llvm.s390.vcfn
    10 << 8 | 1, // llvm.s390.vchbs
    10 << 8 | 1, // llvm.s390.vchfs
    10 << 8 | 1, // llvm.s390.vchgs
    10 << 8 | 1, // llvm.s390.vchhs
    10 << 8 | 1, // llvm.s390.vchlbs
    10 << 8 | 1, // llvm.s390.vchlfs
    10 << 8 | 1, // llvm.s390.vchlgs
    10 << 8 | 1, // llvm.s390.vchlhs
    10 << 8 | 1, // llvm.s390.vchlqs
    10 << 8 | 1, // llvm.s390.vchqs
    10 << 8 | 1, // llvm.s390.vcksm
    10 << 8 | 0, // llvm.s390.vclfnhs
    10 << 8 | 0, // llvm.s390.vclfnls
    10 << 8 | 0, // llvm.s390.vcnf
    10 << 8 | 11, // llvm.s390.vcrnfs
    10 << 8 | 35, // llvm.s390.verimb
    10 << 8 | 35, // llvm.s390.verimf
    10 << 8 | 35, // llvm.s390.verimg
    10 << 8 | 35, // llvm.s390.verimh
    10 << 8 | 35, // llvm.s390.veval
    10 << 8 | 11, // llvm.s390.vfaeb
    10 << 8 | 11, // llvm.s390.vfaebs
    10 << 8 | 11, // llvm.s390.vfaef
    10 << 8 | 11, // llvm.s390.vfaefs
    10 << 8 | 11, // llvm.s390.vfaeh
    10 << 8 | 11, // llvm.s390.vfaehs
    10 << 8 | 11, // llvm.s390.vfaezb
    10 << 8 | 11, // llvm.s390.vfaezbs
    10 << 8 | 11, // llvm.s390.vfaezf
    10 << 8 | 11, // llvm.s390.vfaezfs
    10 << 8 | 11, // llvm.s390.vfaezh
    10 << 8 | 11, // llvm.s390.vfaezhs
    10 << 8 | 1, // llvm.s390.vfcedbs
    10 << 8 | 1, // llvm.s390.vfcesbs
    10 << 8 | 1, // llvm.s390.vfchdbs
    10 << 8 | 1, // llvm.s390.vfchedbs
    10 << 8 | 1, // llvm.s390.vfchesbs
    10 << 8 | 1, // llvm.s390.vfchsbs
    10 << 8 | 1, // llvm.s390.vfeeb
    10 << 8 | 1, // llvm.s390.vfeebs
    10 << 8 | 1, // llvm.s390.vfeef
    10 << 8 | 1, // llvm.s390.vfeefs
    10 << 8 | 1, // llvm.s390.vfeeh
    10 << 8 | 1, // llvm.s390.vfeehs
    10 << 8 | 1, // llvm.s390.vfeezb
    10 << 8 | 1, // llvm.s390.vfeezbs
    10 << 8 | 1, // llvm.s390.vfeezf
    10 << 8 | 1, // llvm.s390.vfeezfs
    10 << 8 | 1, // llvm.s390.vfeezh
    10 << 8 | 1, // llvm.s390.vfeezhs
    10 << 8 | 1, // llvm.s390.vfeneb
    10 << 8 | 1, // llvm.s390.vfenebs
    10 << 8 | 1, // llvm.s390.vfenef
    10 << 8 | 1, // llvm.s390.vfenefs
    10 << 8 | 1, // llvm.s390.vfeneh
    10 << 8 | 1, // llvm.s390.vfenehs
    10 << 8 | 1, // llvm.s390.vfenezb
    10 << 8 | 1, // llvm.s390.vfenezbs
    10 << 8 | 1, // llvm.s390.vfenezf
    10 << 8 | 1, // llvm.s390.vfenezfs
    10 << 8 | 1, // llvm.s390.vfenezh
    10 << 8 | 1, // llvm.s390.vfenezhs
    10 << 8 | 13, // llvm.s390.vfidb
    10 << 8 | 13, // llvm.s390.vfisb
    10 << 8 | 11, // llvm.s390.vfmaxdb
    10 << 8 | 11, // llvm.s390.vfmaxsb
    10 << 8 | 11, // llvm.s390.vfmindb
    10 << 8 | 11, // llvm.s390.vfminsb
    10 << 8 | 0, // llvm.s390.vftcidb
    10 << 8 | 0, // llvm.s390.vftcisb
    10 << 8 | 1, // llvm.s390.vgemb
    10 << 8 | 1, // llvm.s390.vgemf
    10 << 8 | 1, // llvm.s390.vgemg
    10 << 8 | 1, // llvm.s390.vgemh
    10 << 8 | 1, // llvm.s390.vgemq
    10 << 8 | 1, // llvm.s390.vgfmab
    10 << 8 | 1, // llvm.s390.vgfmaf
    10 << 8 | 1, // llvm.s390.vgfmag
    10 << 8 | 1, // llvm.s390.vgfmah
    10 << 8 | 1, // llvm.s390.vgfmb
    10 << 8 | 1, // llvm.s390.vgfmf
    10 << 8 | 1, // llvm.s390.vgfmg
    10 << 8 | 1, // llvm.s390.vgfmh
    10 << 8 | 1, // llvm.s390.vistrb
    10 << 8 | 1, // llvm.s390.vistrbs
    10 << 8 | 1, // llvm.s390.vistrf
    10 << 8 | 1, // llvm.s390.vistrfs
    10 << 8 | 1, // llvm.s390.vistrh
    10 << 8 | 1, // llvm.s390.vistrhs
    2 << 8 | 0, // llvm.s390.vlbb
    2 << 8 | 1, // llvm.s390.vll
    2 << 8 | 1, // llvm.s390.vlrl
    10 << 8 | 1, // llvm.s390.vmaeb
    10 << 8 | 1, // llvm.s390.vmaef
    10 << 8 | 1, // llvm.s390.vmaeg
    10 << 8 | 1, // llvm.s390.vmaeh
    10 << 8 | 1, // llvm.s390.vmahb
    10 << 8 | 1, // llvm.s390.vmahf
    10 << 8 | 1, // llvm.s390.vmahg
    10 << 8 | 1, // llvm.s390.vmahh
    10 << 8 | 1, // llvm.s390.vmahq
    10 << 8 | 1, // llvm.s390.vmaleb
    10 << 8 | 1, // llvm.s390.vmalef
    10 << 8 | 1, // llvm.s390.vmaleg
    10 << 8 | 1, // llvm.s390.vmaleh
    10 << 8 | 1, // llvm.s390.vmalhb
    10 << 8 | 1, // llvm.s390.vmalhf
    10 << 8 | 1, // llvm.s390.vmalhg
    10 << 8 | 1, // llvm.s390.vmalhh
    10 << 8 | 1, // llvm.s390.vmalhq
    10 << 8 | 1, // llvm.s390.vmalob
    10 << 8 | 1, // llvm.s390.vmalof
    10 << 8 | 1, // llvm.s390.vmalog
    10 << 8 | 1, // llvm.s390.vmaloh
    10 << 8 | 1, // llvm.s390.vmaob
    10 << 8 | 1, // llvm.s390.vmaof
    10 << 8 | 1, // llvm.s390.vmaog
    10 << 8 | 1, // llvm.s390.vmaoh
    10 << 8 | 1, // llvm.s390.vmeb
    10 << 8 | 1, // llvm.s390.vmef
    10 << 8 | 1, // llvm.s390.vmeg
    10 << 8 | 1, // llvm.s390.vmeh
    10 << 8 | 1, // llvm.s390.vmhb
    10 << 8 | 1, // llvm.s390.vmhf
    10 << 8 | 1, // llvm.s390.vmhg
    10 << 8 | 1, // llvm.s390.vmhh
    10 << 8 | 1, // llvm.s390.vmhq
    10 << 8 | 1, // llvm.s390.vmleb
    10 << 8 | 1, // llvm.s390.vmlef
    10 << 8 | 1, // llvm.s390.vmleg
    10 << 8 | 1, // llvm.s390.vmleh
    10 << 8 | 1, // llvm.s390.vmlhb
    10 << 8 | 1, // llvm.s390.vmlhf
    10 << 8 | 1, // llvm.s390.vmlhg
    10 << 8 | 1, // llvm.s390.vmlhh
    10 << 8 | 1, // llvm.s390.vmlhq
    10 << 8 | 1, // llvm.s390.vmlob
    10 << 8 | 1, // llvm.s390.vmlof
    10 << 8 | 1, // llvm.s390.vmlog
    10 << 8 | 1, // llvm.s390.vmloh
    10 << 8 | 1, // llvm.s390.vmob
    10 << 8 | 1, // llvm.s390.vmof
    10 << 8 | 1, // llvm.s390.vmog
    10 << 8 | 1, // llvm.s390.vmoh
    10 << 8 | 35, // llvm.s390.vmslg
    10 << 8 | 11, // llvm.s390.vpdi
    10 << 8 | 1, // llvm.s390.vperm
    10 << 8 | 1, // llvm.s390.vpklsf
    10 << 8 | 1, // llvm.s390.vpklsfs
    10 << 8 | 1, // llvm.s390.vpklsg
    10 << 8 | 1, // llvm.s390.vpklsgs
    10 << 8 | 1, // llvm.s390.vpklsh
    10 << 8 | 1, // llvm.s390.vpklshs
    10 << 8 | 1, // llvm.s390.vpksf
    10 << 8 | 1, // llvm.s390.vpksfs
    10 << 8 | 1, // llvm.s390.vpksg
    10 << 8 | 1, // llvm.s390.vpksgs
    10 << 8 | 1, // llvm.s390.vpksh
    10 << 8 | 1, // llvm.s390.vpkshs
    10 << 8 | 1, // llvm.s390.vsbcbiq
    10 << 8 | 1, // llvm.s390.vsbiq
    10 << 8 | 1, // llvm.s390.vscbib
    10 << 8 | 1, // llvm.s390.vscbif
    10 << 8 | 1, // llvm.s390.vscbig
    10 << 8 | 1, // llvm.s390.vscbih
    10 << 8 | 1, // llvm.s390.vscbiq
    10 << 8 | 1, // llvm.s390.vsl
    10 << 8 | 1, // llvm.s390.vslb
    10 << 8 | 11, // llvm.s390.vsld
    10 << 8 | 11, // llvm.s390.vsldb
    10 << 8 | 1, // llvm.s390.vsq
    10 << 8 | 1, // llvm.s390.vsra
    10 << 8 | 1, // llvm.s390.vsrab
    10 << 8 | 11, // llvm.s390.vsrd
    10 << 8 | 1, // llvm.s390.vsrl
    10 << 8 | 1, // llvm.s390.vsrlb
    79 << 8 | 1, // llvm.s390.vstl
    10 << 8 | 35, // llvm.s390.vstrcb
    10 << 8 | 35, // llvm.s390.vstrcbs
    10 << 8 | 35, // llvm.s390.vstrcf
    10 << 8 | 35, // llvm.s390.vstrcfs
    10 << 8 | 35, // llvm.s390.vstrch
    10 << 8 | 35, // llvm.s390.vstrchs
    10 << 8 | 35, // llvm.s390.vstrczb
    10 << 8 | 35, // llvm.s390.vstrczbs
    10 << 8 | 35, // llvm.s390.vstrczf
    10 << 8 | 35, // llvm.s390.vstrczfs
    10 << 8 | 35, // llvm.s390.vstrczh
    10 << 8 | 35, // llvm.s390.vstrczhs
    79 << 8 | 1, // llvm.s390.vstrl
    10 << 8 | 1, // llvm.s390.vstrsb
    10 << 8 | 1, // llvm.s390.vstrsf
    10 << 8 | 1, // llvm.s390.vstrsh
    10 << 8 | 1, // llvm.s390.vstrszb
    10 << 8 | 1, // llvm.s390.vstrszf
    10 << 8 | 1, // llvm.s390.vstrszh
    10 << 8 | 1, // llvm.s390.vsumb
    10 << 8 | 1, // llvm.s390.vsumgf
    10 << 8 | 1, // llvm.s390.vsumgh
    10 << 8 | 1, // llvm.s390.vsumh
    10 << 8 | 1, // llvm.s390.vsumqf
    10 << 8 | 1, // llvm.s390.vsumqg
    10 << 8 | 1, // llvm.s390.vtm
    10 << 8 | 1, // llvm.s390.vuphb
    10 << 8 | 1, // llvm.s390.vuphf
    10 << 8 | 1, // llvm.s390.vuphg
    10 << 8 | 1, // llvm.s390.vuphh
    10 << 8 | 1, // llvm.s390.vuplb
    10 << 8 | 1, // llvm.s390.vuplf
    10 << 8 | 1, // llvm.s390.vuplg
    10 << 8 | 1, // llvm.s390.vuplhb
    10 << 8 | 1, // llvm.s390.vuplhf
    10 << 8 | 1, // llvm.s390.vuplhg
    10 << 8 | 1, // llvm.s390.vuplhh
    10 << 8 | 1, // llvm.s390.vuplhw
    10 << 8 | 1, // llvm.s390.vupllb
    10 << 8 | 1, // llvm.s390.vupllf
    10 << 8 | 1, // llvm.s390.vupllg
    10 << 8 | 1, // llvm.s390.vupllh
    1 << 8 | 1, // llvm.spv.all
    5 << 8 | 5, // llvm.spv.alloca
    5 << 8 | 0, // llvm.spv.alloca.array
    1 << 8 | 1, // llvm.spv.any
    5 << 8 | 0, // llvm.spv.assign.aliasing.decoration
    5 << 8 | 1, // llvm.spv.assign.decoration
    5 << 8 | 1, // llvm.spv.assign.fpmaxerror.decoration
    5 << 8 | 1, // llvm.spv.assign.name
    5 << 8 | 11, // llvm.spv.assign.ptr.type
    5 << 8 | 1, // llvm.spv.assign.type
    5 << 8 | 1, // llvm.spv.assume
    5 << 8 | 1, // llvm.spv.bitcast
    5 << 8 | 1, // llvm.spv.cmpxchg
    5 << 8 | 1, // llvm.spv.const.composite
    1 << 8 | 1, // llvm.spv.cross
    1 << 8 | 1, // llvm.spv.ddx
    1 << 8 | 1, // llvm.spv.ddx.coarse
    1 << 8 | 1, // llvm.spv.ddx.fine
    1 << 8 | 1, // llvm.spv.ddy
    1 << 8 | 1, // llvm.spv.ddy.coarse
    1 << 8 | 1, // llvm.spv.ddy.fine
    1 << 8 | 1, // llvm.spv.degrees
    7 << 8 | 1, // llvm.spv.discard
    1 << 8 | 1, // llvm.spv.distance
    1 << 8 | 1, // llvm.spv.dot4add.i8packed
    1 << 8 | 1, // llvm.spv.dot4add.u8packed
    5 << 8 | 1, // llvm.spv.expect
    5 << 8 | 1, // llvm.spv.extractelt
    5 << 8 | 1, // llvm.spv.extractv
    1 << 8 | 1, // llvm.spv.faceforward
    1 << 8 | 1, // llvm.spv.fdot
    1 << 8 | 1, // llvm.spv.firstbitlow
    1 << 8 | 1, // llvm.spv.firstbitshigh
    1 << 8 | 1, // llvm.spv.firstbituhigh
    76 << 8 | 1, // llvm.spv.flattened.thread.id.in.group
    1 << 8 | 1, // llvm.spv.frac
    1 << 8 | 1, // llvm.spv.fwidth
    1 << 8 | 2, // llvm.spv.generic.cast.to.ptr.explicit
    5 << 8 | 5, // llvm.spv.gep
    76 << 8 | 1, // llvm.spv.global.offset
    76 << 8 | 1, // llvm.spv.global.size
    76 << 8 | 1, // llvm.spv.group.id
    61 << 8 | 1, // llvm.spv.group.memory.barrier.with.group.sync
    5 << 8 | 1, // llvm.spv.init.global
    5 << 8 | 1, // llvm.spv.inline.asm
    5 << 8 | 1, // llvm.spv.insertelt
    5 << 8 | 1, // llvm.spv.insertv
    1 << 8 | 1, // llvm.spv.isinf
    1 << 8 | 1, // llvm.spv.isnan
    1 << 8 | 1, // llvm.spv.length
    1 << 8 | 1, // llvm.spv.lerp
    110 << 8 | 22, // llvm.spv.lifetime.end
    110 << 8 | 22, // llvm.spv.lifetime.start
    5 << 8 | 13, // llvm.spv.load
    5 << 8 | 1, // llvm.spv.loop.merge
    1 << 8 | 1, // llvm.spv.nclamp
    1 << 8 | 1, // llvm.spv.normalize
    76 << 8 | 2, // llvm.spv.num.subgroups
    76 << 8 | 1, // llvm.spv.num.workgroups
    5 << 8 | 11, // llvm.spv.ptrcast
    1 << 8 | 1, // llvm.spv.pushconstant.getpointer
    1 << 8 | 1, // llvm.spv.radians
    1 << 8 | 1, // llvm.spv.reflect
    1 << 8 | 1, // llvm.spv.refract
    1 << 8 | 1, // llvm.spv.resource.counterhandlefrombinding
    1 << 8 | 1, // llvm.spv.resource.counterhandlefromimplicitbinding
    23 << 8 | 1, // llvm.spv.resource.getdimensions.x
    1 << 8 | 1, // llvm.spv.resource.getpointer
    1 << 8 | 1, // llvm.spv.resource.handlefrombinding
    1 << 8 | 1, // llvm.spv.resource.handlefromimplicitbinding
    7 << 8 | 1, // llvm.spv.resource.load.typedbuffer
    1 << 8 | 1, // llvm.spv.resource.nonuniformindex
    7 << 8 | 1, // llvm.spv.resource.store.typedbuffer
    28 << 8 | 1, // llvm.spv.resource.updatecounter
    1 << 8 | 1, // llvm.spv.rsqrt
    1 << 8 | 1, // llvm.spv.saturate
    1 << 8 | 1, // llvm.spv.sclamp
    1 << 8 | 1, // llvm.spv.sdot
    5 << 8 | 0, // llvm.spv.selection.merge
    1 << 8 | 1, // llvm.spv.sign
    1 << 8 | 1, // llvm.spv.smoothstep
    1 << 8 | 1, // llvm.spv.step
    5 << 8 | 93, // llvm.spv.store
    76 << 8 | 2, // llvm.spv.subgroup.id
    76 << 8 | 2, // llvm.spv.subgroup.local.invocation.id
    76 << 8 | 2, // llvm.spv.subgroup.max.size
    76 << 8 | 2, // llvm.spv.subgroup.size
    5 << 8 | 1, // llvm.spv.switch
    76 << 8 | 1, // llvm.spv.thread.id
    76 << 8 | 1, // llvm.spv.thread.id.in.group
    5 << 8 | 1, // llvm.spv.track.constant
    1 << 8 | 1, // llvm.spv.uclamp
    1 << 8 | 1, // llvm.spv.udot
    5 << 8 | 1, // llvm.spv.undef
    1 << 8 | 1, // llvm.spv.unpackhalf2x16
    5 << 8 | 1, // llvm.spv.unreachable
    5 << 8 | 1, // llvm.spv.unref.global
    5 << 8 | 1, // llvm.spv.value.md
    15 << 8 | 1, // llvm.spv.wave.active.countbits
    15 << 8 | 1, // llvm.spv.wave.all
    15 << 8 | 1, // llvm.spv.wave.any
    15 << 8 | 1, // llvm.spv.wave.ballot
    61 << 8 | 1, // llvm.spv.wave.get.lane.count
    61 << 8 | 1, // llvm.spv.wave.is.first.lane
    15 << 8 | 1, // llvm.spv.wave.readlane
    15 << 8 | 1, // llvm.spv.wave.reduce.max
    15 << 8 | 1, // llvm.spv.wave.reduce.min
    15 << 8 | 1, // llvm.spv.wave.reduce.sum
    15 << 8 | 1, // llvm.spv.wave.reduce.umax
    15 << 8 | 1, // llvm.spv.wave.reduce.umin
    76 << 8 | 1, // llvm.spv.workgroup.size
    10 << 8 | 1, // llvm.ve.vl.andm.MMM
    10 << 8 | 1, // llvm.ve.vl.andm.mmm
    10 << 8 | 1, // llvm.ve.vl.eqvm.MMM
    10 << 8 | 1, // llvm.ve.vl.eqvm.mmm
    10 << 8 | 1, // llvm.ve.vl.extract.vm512l
    10 << 8 | 1, // llvm.ve.vl.extract.vm512u
    29 << 8 | 1, // llvm.ve.vl.fencec.s
    29 << 8 | 1, // llvm.ve.vl.fencei
    29 << 8 | 1, // llvm.ve.vl.fencem.s
    29 << 8 | 1, // llvm.ve.vl.fidcr.sss
    10 << 8 | 1, // llvm.ve.vl.insert.vm512l
    10 << 8 | 1, // llvm.ve.vl.insert.vm512u
    10 << 8 | 1, // llvm.ve.vl.lcr.sss
    10 << 8 | 1, // llvm.ve.vl.lsv.vvss
    10 << 8 | 1, // llvm.ve.vl.lvm.MMss
    10 << 8 | 1, // llvm.ve.vl.lvm.mmss
    10 << 8 | 1, // llvm.ve.vl.lvsd.svs
    10 << 8 | 1, // llvm.ve.vl.lvsl.svs
    10 << 8 | 1, // llvm.ve.vl.lvss.svs
    10 << 8 | 1, // llvm.ve.vl.lzvm.sml
    10 << 8 | 1, // llvm.ve.vl.negm.MM
    10 << 8 | 1, // llvm.ve.vl.negm.mm
    10 << 8 | 1, // llvm.ve.vl.nndm.MMM
    10 << 8 | 1, // llvm.ve.vl.nndm.mmm
    10 << 8 | 1, // llvm.ve.vl.orm.MMM
    10 << 8 | 1, // llvm.ve.vl.orm.mmm
    23 << 8 | 1, // llvm.ve.vl.pack.f32a
    23 << 8 | 1, // llvm.ve.vl.pack.f32p
    10 << 8 | 1, // llvm.ve.vl.pcvm.sml
    111 << 8 | 1, // llvm.ve.vl.pfchv.ssl
    111 << 8 | 1, // llvm.ve.vl.pfchvnc.ssl
    10 << 8 | 1, // llvm.ve.vl.pvadds.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvadds.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvadds.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvadds.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvadds.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvadds.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvaddu.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvaddu.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvaddu.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvaddu.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvaddu.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvaddu.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvand.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvand.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvand.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvand.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvand.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvand.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvbrd.vsMvl
    10 << 8 | 1, // llvm.ve.vl.pvbrd.vsl
    10 << 8 | 1, // llvm.ve.vl.pvbrd.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvbrv.vvMvl
    10 << 8 | 1, // llvm.ve.vl.pvbrv.vvl
    10 << 8 | 1, // llvm.ve.vl.pvbrv.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvbrvlo.vvl
    10 << 8 | 1, // llvm.ve.vl.pvbrvlo.vvmvl
    10 << 8 | 1, // llvm.ve.vl.pvbrvlo.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvbrvup.vvl
    10 << 8 | 1, // llvm.ve.vl.pvbrvup.vvmvl
    10 << 8 | 1, // llvm.ve.vl.pvbrvup.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvcmps.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvcmps.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvcmps.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvcmps.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvcmps.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvcmps.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvcmpu.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvcmpu.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvcmpu.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvcmpu.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvcmpu.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvcmpu.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvcvtsw.vvl
    10 << 8 | 1, // llvm.ve.vl.pvcvtsw.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvcvtws.vvMvl
    10 << 8 | 1, // llvm.ve.vl.pvcvtws.vvl
    10 << 8 | 1, // llvm.ve.vl.pvcvtws.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvcvtwsrz.vvMvl
    10 << 8 | 1, // llvm.ve.vl.pvcvtwsrz.vvl
    10 << 8 | 1, // llvm.ve.vl.pvcvtwsrz.vvvl
    10 << 8 | 1, // llvm.ve.vl.pveqv.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pveqv.vsvl
    10 << 8 | 1, // llvm.ve.vl.pveqv.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pveqv.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pveqv.vvvl
    10 << 8 | 1, // llvm.ve.vl.pveqv.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfadd.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfadd.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvfadd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfadd.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfadd.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvfadd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfcmp.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfcmp.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvfcmp.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfcmp.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfcmp.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvfcmp.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vsvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vvsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vvsvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vvvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmad.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmax.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmax.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvfmax.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmax.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmax.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmax.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmin.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmin.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvfmin.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmin.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmin.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmin.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkaf.Ml
    10 << 8 | 1, // llvm.ve.vl.pvfmkat.Ml
    10 << 8 | 1, // llvm.ve.vl.pvfmkseq.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkseq.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkseqnan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkseqnan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksge.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksge.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksgenan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksgenan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksgt.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksgt.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksgtnan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksgtnan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksle.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksle.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslenan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslenan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksloeq.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksloeq.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksloeqnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksloeqnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksloge.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksloge.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslogenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslogenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslogt.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslogt.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslogtnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslogtnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslole.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslole.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslolenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslolenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslolt.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslolt.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksloltnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksloltnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslonan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslonan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslone.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslone.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslonenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslonenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslonum.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslonum.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkslt.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkslt.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksltnan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksltnan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksnan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksnan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksne.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksne.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksnenan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksnenan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksnum.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmksnum.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupeq.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupeq.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupeqnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupeqnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupge.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupge.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupgenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupgenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupgt.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupgt.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupgtnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupgtnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksuple.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksuple.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksuplenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksuplenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksuplt.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksuplt.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupltnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupltnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupne.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupne.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupnenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupnenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmksupnum.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmksupnum.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkweq.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkweq.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkweqnan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkweqnan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwge.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwge.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwgenan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwgenan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwgt.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwgt.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwgtnan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwgtnan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwle.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwle.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlenan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlenan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwloeq.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwloeq.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwloeqnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwloeqnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwloge.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwloge.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlogenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlogenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlogt.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlogt.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlogtnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlogtnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlole.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlole.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlolenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlolenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlolt.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlolt.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwloltnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwloltnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlonan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlonan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlone.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlone.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlonenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlonenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlonum.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlonum.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlt.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwlt.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwltnan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwltnan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwnan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwnan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwne.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwne.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwnenan.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwnenan.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwnum.MvMl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwnum.Mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupeq.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupeq.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupeqnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupeqnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupge.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupge.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupgenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupgenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupgt.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupgt.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupgtnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupgtnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwuple.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwuple.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwuplenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwuplenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwuplt.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwuplt.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupltnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupltnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupnan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupnan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupne.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupne.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupnenan.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupnenan.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupnum.mvl
    10 << 8 | 1, // llvm.ve.vl.pvfmkwupnum.mvml
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vsvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vvsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vvsvl
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vvvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmsb.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmul.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmul.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvfmul.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmul.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfmul.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvfmul.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vsvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vvsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vvsvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vvvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmad.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vsvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vvsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vvsvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vvvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfnmsb.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.pvfsub.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfsub.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvfsub.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvfsub.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvfsub.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvfsub.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvldz.vvMvl
    10 << 8 | 1, // llvm.ve.vl.pvldz.vvl
    10 << 8 | 1, // llvm.ve.vl.pvldz.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvldzlo.vvl
    10 << 8 | 1, // llvm.ve.vl.pvldzlo.vvmvl
    10 << 8 | 1, // llvm.ve.vl.pvldzlo.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvldzup.vvl
    10 << 8 | 1, // llvm.ve.vl.pvldzup.vvmvl
    10 << 8 | 1, // llvm.ve.vl.pvldzup.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvmaxs.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvmaxs.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvmaxs.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvmaxs.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvmaxs.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvmaxs.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvmins.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvmins.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvmins.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvmins.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvmins.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvmins.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvor.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvor.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvor.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvor.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvor.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvor.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvpcnt.vvMvl
    10 << 8 | 1, // llvm.ve.vl.pvpcnt.vvl
    10 << 8 | 1, // llvm.ve.vl.pvpcnt.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvpcntlo.vvl
    10 << 8 | 1, // llvm.ve.vl.pvpcntlo.vvmvl
    10 << 8 | 1, // llvm.ve.vl.pvpcntlo.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvpcntup.vvl
    10 << 8 | 1, // llvm.ve.vl.pvpcntup.vvmvl
    10 << 8 | 1, // llvm.ve.vl.pvpcntup.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvrcp.vvl
    10 << 8 | 1, // llvm.ve.vl.pvrcp.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvrsqrt.vvl
    10 << 8 | 1, // llvm.ve.vl.pvrsqrt.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvrsqrtnex.vvl
    10 << 8 | 1, // llvm.ve.vl.pvrsqrtnex.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvseq.vl
    10 << 8 | 1, // llvm.ve.vl.pvseq.vvl
    10 << 8 | 1, // llvm.ve.vl.pvseqlo.vl
    10 << 8 | 1, // llvm.ve.vl.pvseqlo.vvl
    10 << 8 | 1, // llvm.ve.vl.pvsequp.vl
    10 << 8 | 1, // llvm.ve.vl.pvsequp.vvl
    10 << 8 | 1, // llvm.ve.vl.pvsla.vvsMvl
    10 << 8 | 1, // llvm.ve.vl.pvsla.vvsl
    10 << 8 | 1, // llvm.ve.vl.pvsla.vvsvl
    10 << 8 | 1, // llvm.ve.vl.pvsla.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvsla.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvsla.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvsll.vvsMvl
    10 << 8 | 1, // llvm.ve.vl.pvsll.vvsl
    10 << 8 | 1, // llvm.ve.vl.pvsll.vvsvl
    10 << 8 | 1, // llvm.ve.vl.pvsll.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvsll.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvsll.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvsra.vvsMvl
    10 << 8 | 1, // llvm.ve.vl.pvsra.vvsl
    10 << 8 | 1, // llvm.ve.vl.pvsra.vvsvl
    10 << 8 | 1, // llvm.ve.vl.pvsra.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvsra.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvsra.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvsrl.vvsMvl
    10 << 8 | 1, // llvm.ve.vl.pvsrl.vvsl
    10 << 8 | 1, // llvm.ve.vl.pvsrl.vvsvl
    10 << 8 | 1, // llvm.ve.vl.pvsrl.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvsrl.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvsrl.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvsubs.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvsubs.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvsubs.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvsubs.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvsubs.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvsubs.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvsubu.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvsubu.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvsubu.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvsubu.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvsubu.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvsubu.vvvvl
    10 << 8 | 1, // llvm.ve.vl.pvxor.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.pvxor.vsvl
    10 << 8 | 1, // llvm.ve.vl.pvxor.vsvvl
    10 << 8 | 1, // llvm.ve.vl.pvxor.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.pvxor.vvvl
    10 << 8 | 1, // llvm.ve.vl.pvxor.vvvvl
    29 << 8 | 1, // llvm.ve.vl.scr.sss
    10 << 8 | 1, // llvm.ve.vl.svm.sMs
    10 << 8 | 1, // llvm.ve.vl.svm.sms
    29 << 8 | 1, // llvm.ve.vl.svob
    10 << 8 | 1, // llvm.ve.vl.tovm.sml
    29 << 8 | 1, // llvm.ve.vl.tscr.ssss
    10 << 8 | 1, // llvm.ve.vl.vaddsl.vsvl
    10 << 8 | 1, // llvm.ve.vl.vaddsl.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vaddsl.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vaddsl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vaddsl.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vaddsl.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vaddswsx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vaddswsx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vaddswsx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vaddswsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vaddswsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vaddswsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vaddswzx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vaddswzx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vaddswzx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vaddswzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vaddswzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vaddswzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vaddul.vsvl
    10 << 8 | 1, // llvm.ve.vl.vaddul.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vaddul.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vaddul.vvvl
    10 << 8 | 1, // llvm.ve.vl.vaddul.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vaddul.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vadduw.vsvl
    10 << 8 | 1, // llvm.ve.vl.vadduw.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vadduw.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vadduw.vvvl
    10 << 8 | 1, // llvm.ve.vl.vadduw.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vadduw.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vand.vsvl
    10 << 8 | 1, // llvm.ve.vl.vand.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vand.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vand.vvvl
    10 << 8 | 1, // llvm.ve.vl.vand.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vand.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vbrdd.vsl
    10 << 8 | 1, // llvm.ve.vl.vbrdd.vsmvl
    10 << 8 | 1, // llvm.ve.vl.vbrdd.vsvl
    10 << 8 | 1, // llvm.ve.vl.vbrdl.vsl
    10 << 8 | 1, // llvm.ve.vl.vbrdl.vsmvl
    10 << 8 | 1, // llvm.ve.vl.vbrdl.vsvl
    10 << 8 | 1, // llvm.ve.vl.vbrds.vsl
    10 << 8 | 1, // llvm.ve.vl.vbrds.vsmvl
    10 << 8 | 1, // llvm.ve.vl.vbrds.vsvl
    10 << 8 | 1, // llvm.ve.vl.vbrdw.vsl
    10 << 8 | 1, // llvm.ve.vl.vbrdw.vsmvl
    10 << 8 | 1, // llvm.ve.vl.vbrdw.vsvl
    10 << 8 | 1, // llvm.ve.vl.vbrv.vvl
    10 << 8 | 1, // llvm.ve.vl.vbrv.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vbrv.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpsl.vsvl
    10 << 8 | 1, // llvm.ve.vl.vcmpsl.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpsl.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpsl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpsl.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpsl.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswsx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswsx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswsx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswzx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswzx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswzx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpswzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpul.vsvl
    10 << 8 | 1, // llvm.ve.vl.vcmpul.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpul.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpul.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpul.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpul.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpuw.vsvl
    10 << 8 | 1, // llvm.ve.vl.vcmpuw.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpuw.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpuw.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcmpuw.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vcmpuw.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vcp.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtdl.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtdl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtds.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtds.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtdw.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtdw.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtld.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtld.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtld.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtldrz.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtldrz.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtldrz.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtsd.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtsd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtsw.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtsw.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdsx.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdsx.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdsxrz.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdsxrz.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdsxrz.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdzx.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdzx.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdzxrz.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdzxrz.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwdzxrz.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwssx.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwssx.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwssx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwssxrz.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwssxrz.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwssxrz.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwszx.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwszx.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwszx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwszxrz.vvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwszxrz.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vcvtwszxrz.vvvl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vsvl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vvsl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivsl.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vvsl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivswsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vvsl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivswzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vsvl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vvsl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vvvl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivul.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vsvl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vvsl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vvvl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vdivuw.vvvvl
    10 << 8 | 1, // llvm.ve.vl.veqv.vsvl
    10 << 8 | 1, // llvm.ve.vl.veqv.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.veqv.vsvvl
    10 << 8 | 1, // llvm.ve.vl.veqv.vvvl
    10 << 8 | 1, // llvm.ve.vl.veqv.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.veqv.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vex.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vfaddd.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfaddd.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfaddd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfaddd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfaddd.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfaddd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfadds.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfadds.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfadds.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfadds.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfadds.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfadds.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfcmpd.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfcmpd.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfcmpd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfcmpd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfcmpd.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfcmpd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfcmps.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfcmps.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfcmps.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfcmps.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfcmps.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfcmps.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfdivd.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfdivd.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfdivd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfdivd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfdivd.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfdivd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfdivs.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfdivs.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfdivs.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfdivs.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfdivs.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfdivs.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vsvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vvsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vvvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmadd.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vsvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vvsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vvvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmads.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxd.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxd.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxd.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxs.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxs.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxs.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxs.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxs.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmaxs.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmind.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfmind.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmind.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmind.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfmind.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmind.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmins.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfmins.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmins.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmins.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfmins.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmins.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdeq.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdeq.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdeqnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdeqnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdge.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdge.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdgenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdgenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdgt.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdgt.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdgtnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdgtnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdle.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdle.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdlenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdlenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdlt.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdlt.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdltnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdltnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdne.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdne.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdnenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdnenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkdnum.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkdnum.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklaf.ml
    10 << 8 | 1, // llvm.ve.vl.vfmklat.ml
    10 << 8 | 1, // llvm.ve.vl.vfmkleq.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkleq.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkleqnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkleqnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklge.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklge.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklgenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklgenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklgt.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklgt.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklgtnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklgtnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklle.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklle.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkllenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkllenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkllt.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkllt.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklltnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklltnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklne.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklne.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklnenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklnenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmklnum.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmklnum.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkseq.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkseq.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkseqnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkseqnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksge.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksge.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksgenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksgenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksgt.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksgt.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksgtnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksgtnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksle.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksle.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkslenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkslenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkslt.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkslt.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksltnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksltnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksne.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksne.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksnenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksnenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmksnum.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmksnum.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkweq.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkweq.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkweqnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkweqnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwge.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwge.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwgenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwgenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwgt.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwgt.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwgtnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwgtnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwle.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwle.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwlenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwlenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwlt.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwlt.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwltnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwltnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwnan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwnan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwne.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwne.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwnenan.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwnenan.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmkwnum.mvl
    10 << 8 | 1, // llvm.ve.vl.vfmkwnum.mvml
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vsvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vvsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vvvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbd.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vsvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vvsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vvvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmsbs.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmuld.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfmuld.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmuld.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmuld.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfmuld.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmuld.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfmuls.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfmuls.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmuls.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfmuls.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfmuls.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfmuls.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vsvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vvsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vvvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmadd.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vsvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vvsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vvvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmads.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vsvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vvsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vvvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbd.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vsvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vsvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vvsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vvsvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vvvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfnmsbs.vvvvvl
    10 << 8 | 1, // llvm.ve.vl.vfrmaxdfst.vvl
    10 << 8 | 1, // llvm.ve.vl.vfrmaxdfst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfrmaxdlst.vvl
    10 << 8 | 1, // llvm.ve.vl.vfrmaxdlst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfrmaxsfst.vvl
    10 << 8 | 1, // llvm.ve.vl.vfrmaxsfst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfrmaxslst.vvl
    10 << 8 | 1, // llvm.ve.vl.vfrmaxslst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfrmindfst.vvl
    10 << 8 | 1, // llvm.ve.vl.vfrmindfst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfrmindlst.vvl
    10 << 8 | 1, // llvm.ve.vl.vfrmindlst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfrminsfst.vvl
    10 << 8 | 1, // llvm.ve.vl.vfrminsfst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfrminslst.vvl
    10 << 8 | 1, // llvm.ve.vl.vfrminslst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfsqrtd.vvl
    10 << 8 | 1, // llvm.ve.vl.vfsqrtd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfsqrts.vvl
    10 << 8 | 1, // llvm.ve.vl.vfsqrts.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfsubd.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfsubd.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfsubd.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfsubd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfsubd.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfsubd.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfsubs.vsvl
    10 << 8 | 1, // llvm.ve.vl.vfsubs.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vfsubs.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vfsubs.vvvl
    10 << 8 | 1, // llvm.ve.vl.vfsubs.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vfsubs.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vfsumd.vvl
    10 << 8 | 1, // llvm.ve.vl.vfsumd.vvml
    10 << 8 | 1, // llvm.ve.vl.vfsums.vvl
    10 << 8 | 1, // llvm.ve.vl.vfsums.vvml
    23 << 8 | 1, // llvm.ve.vl.vgt.vvssl
    23 << 8 | 1, // llvm.ve.vl.vgt.vvssml
    23 << 8 | 1, // llvm.ve.vl.vgt.vvssmvl
    23 << 8 | 1, // llvm.ve.vl.vgt.vvssvl
    23 << 8 | 1, // llvm.ve.vl.vgtlsx.vvssl
    23 << 8 | 1, // llvm.ve.vl.vgtlsx.vvssml
    23 << 8 | 1, // llvm.ve.vl.vgtlsx.vvssmvl
    23 << 8 | 1, // llvm.ve.vl.vgtlsx.vvssvl
    23 << 8 | 1, // llvm.ve.vl.vgtlsxnc.vvssl
    23 << 8 | 1, // llvm.ve.vl.vgtlsxnc.vvssml
    23 << 8 | 1, // llvm.ve.vl.vgtlsxnc.vvssmvl
    23 << 8 | 1, // llvm.ve.vl.vgtlsxnc.vvssvl
    23 << 8 | 1, // llvm.ve.vl.vgtlzx.vvssl
    23 << 8 | 1, // llvm.ve.vl.vgtlzx.vvssml
    23 << 8 | 1, // llvm.ve.vl.vgtlzx.vvssmvl
    23 << 8 | 1, // llvm.ve.vl.vgtlzx.vvssvl
    23 << 8 | 1, // llvm.ve.vl.vgtlzxnc.vvssl
    23 << 8 | 1, // llvm.ve.vl.vgtlzxnc.vvssml
    23 << 8 | 1, // llvm.ve.vl.vgtlzxnc.vvssmvl
    23 << 8 | 1, // llvm.ve.vl.vgtlzxnc.vvssvl
    23 << 8 | 1, // llvm.ve.vl.vgtnc.vvssl
    23 << 8 | 1, // llvm.ve.vl.vgtnc.vvssml
    23 << 8 | 1, // llvm.ve.vl.vgtnc.vvssmvl
    23 << 8 | 1, // llvm.ve.vl.vgtnc.vvssvl
    23 << 8 | 1, // llvm.ve.vl.vgtu.vvssl
    23 << 8 | 1, // llvm.ve.vl.vgtu.vvssml
    23 << 8 | 1, // llvm.ve.vl.vgtu.vvssmvl
    23 << 8 | 1, // llvm.ve.vl.vgtu.vvssvl
    23 << 8 | 1, // llvm.ve.vl.vgtunc.vvssl
    23 << 8 | 1, // llvm.ve.vl.vgtunc.vvssml
    23 << 8 | 1, // llvm.ve.vl.vgtunc.vvssmvl
    23 << 8 | 1, // llvm.ve.vl.vgtunc.vvssvl
    23 << 8 | 1, // llvm.ve.vl.vld.vssl
    23 << 8 | 1, // llvm.ve.vl.vld.vssvl
    23 << 8 | 1, // llvm.ve.vl.vld2d.vssl
    23 << 8 | 1, // llvm.ve.vl.vld2d.vssvl
    23 << 8 | 1, // llvm.ve.vl.vld2dnc.vssl
    23 << 8 | 1, // llvm.ve.vl.vld2dnc.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldl2dsx.vssl
    23 << 8 | 1, // llvm.ve.vl.vldl2dsx.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldl2dsxnc.vssl
    23 << 8 | 1, // llvm.ve.vl.vldl2dsxnc.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldl2dzx.vssl
    23 << 8 | 1, // llvm.ve.vl.vldl2dzx.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldl2dzxnc.vssl
    23 << 8 | 1, // llvm.ve.vl.vldl2dzxnc.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldlsx.vssl
    23 << 8 | 1, // llvm.ve.vl.vldlsx.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldlsxnc.vssl
    23 << 8 | 1, // llvm.ve.vl.vldlsxnc.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldlzx.vssl
    23 << 8 | 1, // llvm.ve.vl.vldlzx.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldlzxnc.vssl
    23 << 8 | 1, // llvm.ve.vl.vldlzxnc.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldnc.vssl
    23 << 8 | 1, // llvm.ve.vl.vldnc.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldu.vssl
    23 << 8 | 1, // llvm.ve.vl.vldu.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldu2d.vssl
    23 << 8 | 1, // llvm.ve.vl.vldu2d.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldu2dnc.vssl
    23 << 8 | 1, // llvm.ve.vl.vldu2dnc.vssvl
    23 << 8 | 1, // llvm.ve.vl.vldunc.vssl
    23 << 8 | 1, // llvm.ve.vl.vldunc.vssvl
    10 << 8 | 1, // llvm.ve.vl.vldz.vvl
    10 << 8 | 1, // llvm.ve.vl.vldz.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vldz.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxsl.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmaxsl.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmaxsl.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxsl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxsl.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmaxsl.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswsx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswsx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswsx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswzx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswzx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswzx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmaxswzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vminsl.vsvl
    10 << 8 | 1, // llvm.ve.vl.vminsl.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vminsl.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vminsl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vminsl.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vminsl.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vminswsx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vminswsx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vminswsx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vminswsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vminswsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vminswsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vminswzx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vminswzx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vminswzx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vminswzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vminswzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vminswzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmrg.vsvml
    10 << 8 | 1, // llvm.ve.vl.vmrg.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmrg.vvvml
    10 << 8 | 1, // llvm.ve.vl.vmrg.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmrgw.vsvMl
    10 << 8 | 1, // llvm.ve.vl.vmrgw.vsvMvl
    10 << 8 | 1, // llvm.ve.vl.vmrgw.vvvMl
    10 << 8 | 1, // llvm.ve.vl.vmrgw.vvvMvl
    10 << 8 | 1, // llvm.ve.vl.vmulsl.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmulsl.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmulsl.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmulsl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmulsl.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmulsl.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmulslw.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmulslw.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmulslw.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmulslw.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmulswsx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmulswsx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmulswsx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmulswsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmulswsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmulswsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmulswzx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmulswzx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmulswzx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmulswzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmulswzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmulswzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmulul.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmulul.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmulul.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmulul.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmulul.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmulul.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmuluw.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmuluw.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmuluw.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vmuluw.vvvl
    10 << 8 | 1, // llvm.ve.vl.vmuluw.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vmuluw.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vmv.vsvl
    10 << 8 | 1, // llvm.ve.vl.vmv.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vmv.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vor.vsvl
    10 << 8 | 1, // llvm.ve.vl.vor.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vor.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vor.vvvl
    10 << 8 | 1, // llvm.ve.vl.vor.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vor.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vpcnt.vvl
    10 << 8 | 1, // llvm.ve.vl.vpcnt.vvmvl
    10 << 8 | 1, // llvm.ve.vl.vpcnt.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrand.vvl
    10 << 8 | 1, // llvm.ve.vl.vrand.vvml
    10 << 8 | 1, // llvm.ve.vl.vrcpd.vvl
    10 << 8 | 1, // llvm.ve.vl.vrcpd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrcps.vvl
    10 << 8 | 1, // llvm.ve.vl.vrcps.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxslfst.vvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxslfst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxsllst.vvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxsllst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxswfstsx.vvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxswfstsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxswfstzx.vvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxswfstzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxswlstsx.vvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxswlstsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxswlstzx.vvl
    10 << 8 | 1, // llvm.ve.vl.vrmaxswlstzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrminslfst.vvl
    10 << 8 | 1, // llvm.ve.vl.vrminslfst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrminsllst.vvl
    10 << 8 | 1, // llvm.ve.vl.vrminsllst.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrminswfstsx.vvl
    10 << 8 | 1, // llvm.ve.vl.vrminswfstsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrminswfstzx.vvl
    10 << 8 | 1, // llvm.ve.vl.vrminswfstzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrminswlstsx.vvl
    10 << 8 | 1, // llvm.ve.vl.vrminswlstsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrminswlstzx.vvl
    10 << 8 | 1, // llvm.ve.vl.vrminswlstzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vror.vvl
    10 << 8 | 1, // llvm.ve.vl.vror.vvml
    10 << 8 | 1, // llvm.ve.vl.vrsqrtd.vvl
    10 << 8 | 1, // llvm.ve.vl.vrsqrtd.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrsqrtdnex.vvl
    10 << 8 | 1, // llvm.ve.vl.vrsqrtdnex.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrsqrts.vvl
    10 << 8 | 1, // llvm.ve.vl.vrsqrts.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrsqrtsnex.vvl
    10 << 8 | 1, // llvm.ve.vl.vrsqrtsnex.vvvl
    10 << 8 | 1, // llvm.ve.vl.vrxor.vvl
    10 << 8 | 1, // llvm.ve.vl.vrxor.vvml
    34 << 8 | 1, // llvm.ve.vl.vsc.vvssl
    34 << 8 | 1, // llvm.ve.vl.vsc.vvssml
    34 << 8 | 1, // llvm.ve.vl.vscl.vvssl
    34 << 8 | 1, // llvm.ve.vl.vscl.vvssml
    34 << 8 | 1, // llvm.ve.vl.vsclnc.vvssl
    34 << 8 | 1, // llvm.ve.vl.vsclnc.vvssml
    34 << 8 | 1, // llvm.ve.vl.vsclncot.vvssl
    34 << 8 | 1, // llvm.ve.vl.vsclncot.vvssml
    34 << 8 | 1, // llvm.ve.vl.vsclot.vvssl
    34 << 8 | 1, // llvm.ve.vl.vsclot.vvssml
    34 << 8 | 1, // llvm.ve.vl.vscnc.vvssl
    34 << 8 | 1, // llvm.ve.vl.vscnc.vvssml
    34 << 8 | 1, // llvm.ve.vl.vscncot.vvssl
    34 << 8 | 1, // llvm.ve.vl.vscncot.vvssml
    34 << 8 | 1, // llvm.ve.vl.vscot.vvssl
    34 << 8 | 1, // llvm.ve.vl.vscot.vvssml
    34 << 8 | 1, // llvm.ve.vl.vscu.vvssl
    34 << 8 | 1, // llvm.ve.vl.vscu.vvssml
    34 << 8 | 1, // llvm.ve.vl.vscunc.vvssl
    34 << 8 | 1, // llvm.ve.vl.vscunc.vvssml
    34 << 8 | 1, // llvm.ve.vl.vscuncot.vvssl
    34 << 8 | 1, // llvm.ve.vl.vscuncot.vvssml
    34 << 8 | 1, // llvm.ve.vl.vscuot.vvssl
    34 << 8 | 1, // llvm.ve.vl.vscuot.vvssml
    10 << 8 | 1, // llvm.ve.vl.vseq.vl
    10 << 8 | 1, // llvm.ve.vl.vseq.vvl
    10 << 8 | 1, // llvm.ve.vl.vsfa.vvssl
    10 << 8 | 1, // llvm.ve.vl.vsfa.vvssmvl
    10 << 8 | 1, // llvm.ve.vl.vsfa.vvssvl
    10 << 8 | 1, // llvm.ve.vl.vshf.vvvsl
    10 << 8 | 1, // llvm.ve.vl.vshf.vvvsvl
    10 << 8 | 1, // llvm.ve.vl.vslal.vvsl
    10 << 8 | 1, // llvm.ve.vl.vslal.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vslal.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vslal.vvvl
    10 << 8 | 1, // llvm.ve.vl.vslal.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vslal.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vslawsx.vvsl
    10 << 8 | 1, // llvm.ve.vl.vslawsx.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vslawsx.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vslawsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vslawsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vslawsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vslawzx.vvsl
    10 << 8 | 1, // llvm.ve.vl.vslawzx.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vslawzx.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vslawzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vslawzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vslawzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsll.vvsl
    10 << 8 | 1, // llvm.ve.vl.vsll.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vsll.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vsll.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsll.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsll.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsral.vvsl
    10 << 8 | 1, // llvm.ve.vl.vsral.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vsral.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vsral.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsral.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsral.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsrawsx.vvsl
    10 << 8 | 1, // llvm.ve.vl.vsrawsx.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vsrawsx.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vsrawsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsrawsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsrawsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsrawzx.vvsl
    10 << 8 | 1, // llvm.ve.vl.vsrawzx.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vsrawzx.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vsrawzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsrawzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsrawzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsrl.vvsl
    10 << 8 | 1, // llvm.ve.vl.vsrl.vvsmvl
    10 << 8 | 1, // llvm.ve.vl.vsrl.vvsvl
    10 << 8 | 1, // llvm.ve.vl.vsrl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsrl.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsrl.vvvvl
    34 << 8 | 1, // llvm.ve.vl.vst.vssl
    34 << 8 | 1, // llvm.ve.vl.vst.vssml
    34 << 8 | 1, // llvm.ve.vl.vst2d.vssl
    34 << 8 | 1, // llvm.ve.vl.vst2d.vssml
    34 << 8 | 1, // llvm.ve.vl.vst2dnc.vssl
    34 << 8 | 1, // llvm.ve.vl.vst2dnc.vssml
    34 << 8 | 1, // llvm.ve.vl.vst2dncot.vssl
    34 << 8 | 1, // llvm.ve.vl.vst2dncot.vssml
    34 << 8 | 1, // llvm.ve.vl.vst2dot.vssl
    34 << 8 | 1, // llvm.ve.vl.vst2dot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstl.vssl
    34 << 8 | 1, // llvm.ve.vl.vstl.vssml
    34 << 8 | 1, // llvm.ve.vl.vstl2d.vssl
    34 << 8 | 1, // llvm.ve.vl.vstl2d.vssml
    34 << 8 | 1, // llvm.ve.vl.vstl2dnc.vssl
    34 << 8 | 1, // llvm.ve.vl.vstl2dnc.vssml
    34 << 8 | 1, // llvm.ve.vl.vstl2dncot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstl2dncot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstl2dot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstl2dot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstlnc.vssl
    34 << 8 | 1, // llvm.ve.vl.vstlnc.vssml
    34 << 8 | 1, // llvm.ve.vl.vstlncot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstlncot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstlot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstlot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstnc.vssl
    34 << 8 | 1, // llvm.ve.vl.vstnc.vssml
    34 << 8 | 1, // llvm.ve.vl.vstncot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstncot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstu.vssl
    34 << 8 | 1, // llvm.ve.vl.vstu.vssml
    34 << 8 | 1, // llvm.ve.vl.vstu2d.vssl
    34 << 8 | 1, // llvm.ve.vl.vstu2d.vssml
    34 << 8 | 1, // llvm.ve.vl.vstu2dnc.vssl
    34 << 8 | 1, // llvm.ve.vl.vstu2dnc.vssml
    34 << 8 | 1, // llvm.ve.vl.vstu2dncot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstu2dncot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstu2dot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstu2dot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstunc.vssl
    34 << 8 | 1, // llvm.ve.vl.vstunc.vssml
    34 << 8 | 1, // llvm.ve.vl.vstuncot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstuncot.vssml
    34 << 8 | 1, // llvm.ve.vl.vstuot.vssl
    34 << 8 | 1, // llvm.ve.vl.vstuot.vssml
    10 << 8 | 1, // llvm.ve.vl.vsubsl.vsvl
    10 << 8 | 1, // llvm.ve.vl.vsubsl.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubsl.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vsubsl.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsubsl.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubsl.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsubswsx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vsubswsx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubswsx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vsubswsx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsubswsx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubswsx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsubswzx.vsvl
    10 << 8 | 1, // llvm.ve.vl.vsubswzx.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubswzx.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vsubswzx.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsubswzx.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubswzx.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsubul.vsvl
    10 << 8 | 1, // llvm.ve.vl.vsubul.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubul.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vsubul.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsubul.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubul.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsubuw.vsvl
    10 << 8 | 1, // llvm.ve.vl.vsubuw.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubuw.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vsubuw.vvvl
    10 << 8 | 1, // llvm.ve.vl.vsubuw.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vsubuw.vvvvl
    10 << 8 | 1, // llvm.ve.vl.vsuml.vvl
    10 << 8 | 1, // llvm.ve.vl.vsuml.vvml
    10 << 8 | 1, // llvm.ve.vl.vsumwsx.vvl
    10 << 8 | 1, // llvm.ve.vl.vsumwsx.vvml
    10 << 8 | 1, // llvm.ve.vl.vsumwzx.vvl
    10 << 8 | 1, // llvm.ve.vl.vsumwzx.vvml
    10 << 8 | 1, // llvm.ve.vl.vxor.vsvl
    10 << 8 | 1, // llvm.ve.vl.vxor.vsvmvl
    10 << 8 | 1, // llvm.ve.vl.vxor.vsvvl
    10 << 8 | 1, // llvm.ve.vl.vxor.vvvl
    10 << 8 | 1, // llvm.ve.vl.vxor.vvvmvl
    10 << 8 | 1, // llvm.ve.vl.vxor.vvvvl
    10 << 8 | 1, // llvm.ve.vl.xorm.MMM
    10 << 8 | 1, // llvm.ve.vl.xorm.mmm
    0 << 8 | 1, // llvm.wasm.alltrue
    0 << 8 | 1, // llvm.wasm.anytrue
    0 << 8 | 1, // llvm.wasm.avgr.unsigned
    0 << 8 | 1, // llvm.wasm.bitmask
    0 << 8 | 1, // llvm.wasm.bitselect
    30 << 8 | 5, // llvm.wasm.catch
    0 << 8 | 1, // llvm.wasm.dot
    0 << 8 | 1, // llvm.wasm.extadd.pairwise.signed
    0 << 8 | 1, // llvm.wasm.extadd.pairwise.unsigned
    0 << 8 | 1, // llvm.wasm.extract.lane.f16x8
    30 << 8 | 1, // llvm.wasm.get.ehselector
    30 << 8 | 1, // llvm.wasm.get.exception
    1 << 8 | 0, // llvm.wasm.landingpad.index
    2 << 8 | 1, // llvm.wasm.loadf16.f32
    1 << 8 | 1, // llvm.wasm.lsda
    112 << 8 | 10, // llvm.wasm.memory.atomic.notify
    113 << 8 | 7, // llvm.wasm.memory.atomic.wait32
    113 << 8 | 7, // llvm.wasm.memory.atomic.wait64
    7 << 8 | 1, // llvm.wasm.memory.grow
    23 << 8 | 1, // llvm.wasm.memory.size
    0 << 8 | 1, // llvm.wasm.narrow.signed
    0 << 8 | 1, // llvm.wasm.narrow.unsigned
    0 << 8 | 1, // llvm.wasm.pmax
    0 << 8 | 1, // llvm.wasm.pmin
    0 << 8 | 1, // llvm.wasm.q15mulr.sat.signed
    1 << 8 | 1, // llvm.wasm.ref.is_null.exn
    1 << 8 | 1, // llvm.wasm.ref.is_null.extern
    1 << 8 | 1, // llvm.wasm.ref.is_null.func
    1 << 8 | 1, // llvm.wasm.ref.null.exn
    1 << 8 | 1, // llvm.wasm.ref.null.extern
    1 << 8 | 1, // llvm.wasm.ref.null.func
    1 << 8 | 1, // llvm.wasm.ref.test.func
    0 << 8 | 1, // llvm.wasm.relaxed.dot.bf16x8.add.f32
    0 << 8 | 1, // llvm.wasm.relaxed.dot.i8x16.i7x16.add.signed
    0 << 8 | 1, // llvm.wasm.relaxed.dot.i8x16.i7x16.signed
    0 << 8 | 1, // llvm.wasm.relaxed.laneselect
    0 << 8 | 1, // llvm.wasm.relaxed.madd
    0 << 8 | 1, // llvm.wasm.relaxed.max
    0 << 8 | 1, // llvm.wasm.relaxed.min
    0 << 8 | 1, // llvm.wasm.relaxed.nmadd
    0 << 8 | 1, // llvm.wasm.relaxed.q15mulr.signed
    0 << 8 | 1, // llvm.wasm.relaxed.swizzle
    0 << 8 | 1, // llvm.wasm.relaxed.trunc.signed
    0 << 8 | 1, // llvm.wasm.relaxed.trunc.signed.zero
    0 << 8 | 1, // llvm.wasm.relaxed.trunc.unsigned
    0 << 8 | 1, // llvm.wasm.relaxed.trunc.unsigned.zero
    0 << 8 | 1, // llvm.wasm.replace.lane.f16x8
    114 << 8 | 1, // llvm.wasm.rethrow
    0 << 8 | 244, // llvm.wasm.shuffle
    0 << 8 | 1, // llvm.wasm.splat.f16x8
    79 << 8 | 1, // llvm.wasm.storef16.f32
    0 << 8 | 1, // llvm.wasm.swizzle
    7 << 8 | 1, // llvm.wasm.table.copy
    7 << 8 | 1, // llvm.wasm.table.fill.exnref
    7 << 8 | 1, // llvm.wasm.table.fill.externref
    7 << 8 | 1, // llvm.wasm.table.fill.funcref
    23 << 8 | 1, // llvm.wasm.table.get.exnref
    23 << 8 | 1, // llvm.wasm.table.get.externref
    23 << 8 | 1, // llvm.wasm.table.get.funcref
    7 << 8 | 1, // llvm.wasm.table.grow.exnref
    7 << 8 | 1, // llvm.wasm.table.grow.externref
    7 << 8 | 1, // llvm.wasm.table.grow.funcref
    24 << 8 | 1, // llvm.wasm.table.set.exnref
    24 << 8 | 1, // llvm.wasm.table.set.externref
    24 << 8 | 1, // llvm.wasm.table.set.funcref
    23 << 8 | 1, // llvm.wasm.table.size
    114 << 8 | 5, // llvm.wasm.throw
    0 << 8 | 1, // llvm.wasm.tls.align
    23 << 8 | 1, // llvm.wasm.tls.base
    0 << 8 | 1, // llvm.wasm.tls.size
    0 << 8 | 1, // llvm.wasm.trunc.saturate.signed
    0 << 8 | 1, // llvm.wasm.trunc.saturate.unsigned
    10 << 8 | 1, // llvm.wasm.trunc.signed
    10 << 8 | 1, // llvm.wasm.trunc.unsigned
    11 << 8 | 1, // llvm.x86.aadd32
    11 << 8 | 1, // llvm.x86.aadd64
    11 << 8 | 1, // llvm.x86.aand32
    11 << 8 | 1, // llvm.x86.aand64
    1 << 8 | 1, // llvm.x86.addcarry.32
    1 << 8 | 1, // llvm.x86.addcarry.64
    5 << 8 | 1, // llvm.x86.aesdec128kl
    5 << 8 | 1, // llvm.x86.aesdec256kl
    5 << 8 | 1, // llvm.x86.aesdecwide128kl
    5 << 8 | 1, // llvm.x86.aesdecwide256kl
    5 << 8 | 1, // llvm.x86.aesenc128kl
    5 << 8 | 1, // llvm.x86.aesenc256kl
    5 << 8 | 1, // llvm.x86.aesencwide128kl
    5 << 8 | 1, // llvm.x86.aesencwide256kl
    1 << 8 | 1, // llvm.x86.aesni.aesdec
    1 << 8 | 1, // llvm.x86.aesni.aesdec.256
    1 << 8 | 1, // llvm.x86.aesni.aesdec.512
    1 << 8 | 1, // llvm.x86.aesni.aesdeclast
    1 << 8 | 1, // llvm.x86.aesni.aesdeclast.256
    1 << 8 | 1, // llvm.x86.aesni.aesdeclast.512
    1 << 8 | 1, // llvm.x86.aesni.aesenc
    1 << 8 | 1, // llvm.x86.aesni.aesenc.256
    1 << 8 | 1, // llvm.x86.aesni.aesenc.512
    1 << 8 | 1, // llvm.x86.aesni.aesenclast
    1 << 8 | 1, // llvm.x86.aesni.aesenclast.256
    1 << 8 | 1, // llvm.x86.aesni.aesenclast.512
    1 << 8 | 1, // llvm.x86.aesni.aesimc
    1 << 8 | 0, // llvm.x86.aesni.aeskeygenassist
    11 << 8 | 1, // llvm.x86.aor32
    11 << 8 | 1, // llvm.x86.aor64
    5 << 8 | 11, // llvm.x86.atomic.add.cc
    5 << 8 | 11, // llvm.x86.atomic.and.cc
    5 << 8 | 0, // llvm.x86.atomic.btc
    5 << 8 | 1, // llvm.x86.atomic.btc.rm
    5 << 8 | 0, // llvm.x86.atomic.btr
    5 << 8 | 1, // llvm.x86.atomic.btr.rm
    5 << 8 | 0, // llvm.x86.atomic.bts
    5 << 8 | 1, // llvm.x86.atomic.bts.rm
    5 << 8 | 11, // llvm.x86.atomic.or.cc
    5 << 8 | 11, // llvm.x86.atomic.sub.cc
    5 << 8 | 11, // llvm.x86.atomic.xor.cc
    1 << 8 | 1, // llvm.x86.avx.addsub.pd.256
    1 << 8 | 1, // llvm.x86.avx.addsub.ps.256
    1 << 8 | 1, // llvm.x86.avx.blendv.pd.256
    1 << 8 | 1, // llvm.x86.avx.blendv.ps.256
    1 << 8 | 11, // llvm.x86.avx.cmp.pd.256
    1 << 8 | 11, // llvm.x86.avx.cmp.ps.256
    1 << 8 | 1, // llvm.x86.avx.cvt.pd2.ps.256
    1 << 8 | 1, // llvm.x86.avx.cvt.pd2dq.256
    1 << 8 | 1, // llvm.x86.avx.cvt.ps2dq.256
    1 << 8 | 1, // llvm.x86.avx.cvtt.pd2dq.256
    1 << 8 | 1, // llvm.x86.avx.cvtt.ps2dq.256
    1 << 8 | 11, // llvm.x86.avx.dp.ps.256
    1 << 8 | 1, // llvm.x86.avx.hadd.pd.256
    1 << 8 | 1, // llvm.x86.avx.hadd.ps.256
    1 << 8 | 1, // llvm.x86.avx.hsub.pd.256
    1 << 8 | 1, // llvm.x86.avx.hsub.ps.256
    23 << 8 | 1, // llvm.x86.avx.ldu.dq.256
    2 << 8 | 1, // llvm.x86.avx.maskload.pd
    2 << 8 | 1, // llvm.x86.avx.maskload.pd.256
    2 << 8 | 1, // llvm.x86.avx.maskload.ps
    2 << 8 | 1, // llvm.x86.avx.maskload.ps.256
    11 << 8 | 1, // llvm.x86.avx.maskstore.pd
    11 << 8 | 1, // llvm.x86.avx.maskstore.pd.256
    11 << 8 | 1, // llvm.x86.avx.maskstore.ps
    11 << 8 | 1, // llvm.x86.avx.maskstore.ps.256
    1 << 8 | 1, // llvm.x86.avx.max.pd.256
    1 << 8 | 1, // llvm.x86.avx.max.ps.256
    1 << 8 | 1, // llvm.x86.avx.min.pd.256
    1 << 8 | 1, // llvm.x86.avx.min.ps.256
    1 << 8 | 1, // llvm.x86.avx.movmsk.pd.256
    1 << 8 | 1, // llvm.x86.avx.movmsk.ps.256
    1 << 8 | 1, // llvm.x86.avx.ptestc.256
    1 << 8 | 1, // llvm.x86.avx.ptestnzc.256
    1 << 8 | 1, // llvm.x86.avx.ptestz.256
    1 << 8 | 1, // llvm.x86.avx.rcp.ps.256
    1 << 8 | 0, // llvm.x86.avx.round.pd.256
    1 << 8 | 0, // llvm.x86.avx.round.ps.256
    1 << 8 | 1, // llvm.x86.avx.rsqrt.ps.256
    1 << 8 | 1, // llvm.x86.avx.vpermilvar.pd
    1 << 8 | 1, // llvm.x86.avx.vpermilvar.pd.256
    1 << 8 | 1, // llvm.x86.avx.vpermilvar.ps
    1 << 8 | 1, // llvm.x86.avx.vpermilvar.ps.256
    1 << 8 | 1, // llvm.x86.avx.vtestc.pd
    1 << 8 | 1, // llvm.x86.avx.vtestc.pd.256
    1 << 8 | 1, // llvm.x86.avx.vtestc.ps
    1 << 8 | 1, // llvm.x86.avx.vtestc.ps.256
    1 << 8 | 1, // llvm.x86.avx.vtestnzc.pd
    1 << 8 | 1, // llvm.x86.avx.vtestnzc.pd.256
    1 << 8 | 1, // llvm.x86.avx.vtestnzc.ps
    1 << 8 | 1, // llvm.x86.avx.vtestnzc.ps.256
    1 << 8 | 1, // llvm.x86.avx.vtestz.pd
    1 << 8 | 1, // llvm.x86.avx.vtestz.pd.256
    1 << 8 | 1, // llvm.x86.avx.vtestz.ps
    1 << 8 | 1, // llvm.x86.avx.vtestz.ps.256
    29 << 8 | 1, // llvm.x86.avx.vzeroall
    29 << 8 | 1, // llvm.x86.avx.vzeroupper
    1 << 8 | 0, // llvm.x86.avx10.fpclass.bf16.128
    1 << 8 | 0, // llvm.x86.avx10.fpclass.bf16.256
    1 << 8 | 0, // llvm.x86.avx10.fpclass.bf16.512
    1 << 8 | 1, // llvm.x86.avx10.mask.getexp.bf16.128
    1 << 8 | 1, // llvm.x86.avx10.mask.getexp.bf16.256
    1 << 8 | 1, // llvm.x86.avx10.mask.getexp.bf16.512
    1 << 8 | 0, // llvm.x86.avx10.mask.getmant.bf16.128
    1 << 8 | 0, // llvm.x86.avx10.mask.getmant.bf16.256
    1 << 8 | 0, // llvm.x86.avx10.mask.getmant.bf16.512
    1 << 8 | 1, // llvm.x86.avx10.mask.rcp.bf16.128
    1 << 8 | 1, // llvm.x86.avx10.mask.rcp.bf16.256
    1 << 8 | 1, // llvm.x86.avx10.mask.rcp.bf16.512
    1 << 8 | 0, // llvm.x86.avx10.mask.reduce.bf16.128
    1 << 8 | 0, // llvm.x86.avx10.mask.reduce.bf16.256
    1 << 8 | 0, // llvm.x86.avx10.mask.reduce.bf16.512
    1 << 8 | 0, // llvm.x86.avx10.mask.rndscale.bf16.128
    1 << 8 | 0, // llvm.x86.avx10.mask.rndscale.bf16.256
    1 << 8 | 0, // llvm.x86.avx10.mask.rndscale.bf16.512
    1 << 8 | 1, // llvm.x86.avx10.mask.rsqrt.bf16.128
    1 << 8 | 1, // llvm.x86.avx10.mask.rsqrt.bf16.256
    1 << 8 | 1, // llvm.x86.avx10.mask.rsqrt.bf16.512
    1 << 8 | 1, // llvm.x86.avx10.mask.scalef.bf16.128
    1 << 8 | 1, // llvm.x86.avx10.mask.scalef.bf16.256
    1 << 8 | 1, // llvm.x86.avx10.mask.scalef.bf16.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvt2ps2phx.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvt2ps2phx.256
    1 << 8 | 43, // llvm.x86.avx10.mask.vcvt2ps2phx.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2bf8128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2bf8256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2bf8512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2bf8s128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2bf8s256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2bf8s512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2hf8128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2hf8256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2hf8512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2hf8s128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2hf8s256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtbiasph2hf8s512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvthf82ph128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvthf82ph256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvthf82ph512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2bf8128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2bf8256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2bf8512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2bf8s128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2bf8s256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2bf8s512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2hf8128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2hf8256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2hf8512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2hf8s128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2hf8s256
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2hf8s512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2ibs128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2ibs256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvtph2ibs512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2iubs128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtph2iubs256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvtph2iubs512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtps2ibs128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtps2ibs256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvtps2ibs512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtps2iubs128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvtps2iubs256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvtps2iubs512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttpd2dqs.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttpd2dqs.256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttpd2dqs.round.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttpd2qqs.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttpd2qqs.256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttpd2qqs.round.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttpd2udqs.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttpd2udqs.256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttpd2udqs.round.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttpd2uqqs.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttpd2uqqs.256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttpd2uqqs.round.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttph2ibs128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttph2ibs256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttph2ibs512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttph2iubs128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttph2iubs256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttph2iubs512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2dqs.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2dqs.256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttps2dqs.round.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2ibs128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2ibs256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttps2ibs512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2iubs128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2iubs256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttps2iubs512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2qqs.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2qqs.256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttps2qqs.round.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2udqs.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2udqs.256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttps2udqs.round.512
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2uqqs.128
    1 << 8 | 1, // llvm.x86.avx10.mask.vcvttps2uqqs.256
    1 << 8 | 35, // llvm.x86.avx10.mask.vcvttps2uqqs.round.512
    1 << 8 | 245, // llvm.x86.avx10.mask.vminmaxpd.round
    1 << 8 | 11, // llvm.x86.avx10.mask.vminmaxpd128
    1 << 8 | 11, // llvm.x86.avx10.mask.vminmaxpd256
    1 << 8 | 245, // llvm.x86.avx10.mask.vminmaxph.round
    1 << 8 | 11, // llvm.x86.avx10.mask.vminmaxph128
    1 << 8 | 11, // llvm.x86.avx10.mask.vminmaxph256
    1 << 8 | 245, // llvm.x86.avx10.mask.vminmaxps.round
    1 << 8 | 11, // llvm.x86.avx10.mask.vminmaxps128
    1 << 8 | 11, // llvm.x86.avx10.mask.vminmaxps256
    1 << 8 | 245, // llvm.x86.avx10.mask.vminmaxsd.round
    1 << 8 | 245, // llvm.x86.avx10.mask.vminmaxsh.round
    1 << 8 | 245, // llvm.x86.avx10.mask.vminmaxss.round
    1 << 8 | 1, // llvm.x86.avx10.vaddbf16128
    1 << 8 | 1, // llvm.x86.avx10.vaddbf16256
    1 << 8 | 1, // llvm.x86.avx10.vaddbf16512
    1 << 8 | 11, // llvm.x86.avx10.vaddpd256
    1 << 8 | 11, // llvm.x86.avx10.vaddph256
    1 << 8 | 11, // llvm.x86.avx10.vaddps256
    1 << 8 | 1, // llvm.x86.avx10.vcomisbf16eq
    1 << 8 | 1, // llvm.x86.avx10.vcomisbf16ge
    1 << 8 | 1, // llvm.x86.avx10.vcomisbf16gt
    1 << 8 | 1, // llvm.x86.avx10.vcomisbf16le
    1 << 8 | 1, // llvm.x86.avx10.vcomisbf16lt
    1 << 8 | 1, // llvm.x86.avx10.vcomisbf16neq
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2bf8128
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2bf8256
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2bf8512
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2bf8s128
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2bf8s256
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2bf8s512
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2hf8128
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2hf8256
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2hf8512
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2hf8s128
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2hf8s256
    1 << 8 | 1, // llvm.x86.avx10.vcvt2ph2hf8s512
    1 << 8 | 1, // llvm.x86.avx10.vcvtbf162ibs128
    1 << 8 | 1, // llvm.x86.avx10.vcvtbf162ibs256
    1 << 8 | 1, // llvm.x86.avx10.vcvtbf162ibs512
    1 << 8 | 1, // llvm.x86.avx10.vcvtbf162iubs128
    1 << 8 | 1, // llvm.x86.avx10.vcvtbf162iubs256
    1 << 8 | 1, // llvm.x86.avx10.vcvtbf162iubs512
    1 << 8 | 1, // llvm.x86.avx10.vcvttbf162ibs128
    1 << 8 | 1, // llvm.x86.avx10.vcvttbf162ibs256
    1 << 8 | 1, // llvm.x86.avx10.vcvttbf162ibs512
    1 << 8 | 1, // llvm.x86.avx10.vcvttbf162iubs128
    1 << 8 | 1, // llvm.x86.avx10.vcvttbf162iubs256
    1 << 8 | 1, // llvm.x86.avx10.vcvttbf162iubs512
    1 << 8 | 0, // llvm.x86.avx10.vcvttsd2sis
    1 << 8 | 0, // llvm.x86.avx10.vcvttsd2sis64
    1 << 8 | 0, // llvm.x86.avx10.vcvttsd2usis
    1 << 8 | 0, // llvm.x86.avx10.vcvttsd2usis64
    1 << 8 | 0, // llvm.x86.avx10.vcvttss2sis
    1 << 8 | 0, // llvm.x86.avx10.vcvttss2sis64
    1 << 8 | 0, // llvm.x86.avx10.vcvttss2usis
    1 << 8 | 0, // llvm.x86.avx10.vcvttss2usis64
    1 << 8 | 1, // llvm.x86.avx10.vdivbf16128
    1 << 8 | 1, // llvm.x86.avx10.vdivbf16256
    1 << 8 | 1, // llvm.x86.avx10.vdivbf16512
    1 << 8 | 1, // llvm.x86.avx10.vdpphps.128
    1 << 8 | 1, // llvm.x86.avx10.vdpphps.256
    1 << 8 | 1, // llvm.x86.avx10.vdpphps.512
    1 << 8 | 1, // llvm.x86.avx10.vfmadd132bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfmadd132bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfmadd132bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfmadd213bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfmadd213bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfmadd231bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfmadd231bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfmadd231bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfmsub132bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfmsub132bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfmsub132bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfmsub213bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfmsub213bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfmsub213bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfmsub231bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfmsub231bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfmsub231bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd132bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd132bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd132bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd213bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd213bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd213bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd231bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd231bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfnmadd231bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub132bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub132bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub132bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub213bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub213bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub213bf16512
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub231bf16128
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub231bf16256
    1 << 8 | 1, // llvm.x86.avx10.vfnmsub231bf16512
    1 << 8 | 1, // llvm.x86.avx10.vmaxbf16128
    1 << 8 | 1, // llvm.x86.avx10.vmaxbf16256
    1 << 8 | 1, // llvm.x86.avx10.vmaxbf16512
    1 << 8 | 1, // llvm.x86.avx10.vminbf16128
    1 << 8 | 1, // llvm.x86.avx10.vminbf16256
    1 << 8 | 1, // llvm.x86.avx10.vminbf16512
    1 << 8 | 11, // llvm.x86.avx10.vminmaxbf16128
    1 << 8 | 11, // llvm.x86.avx10.vminmaxbf16256
    1 << 8 | 11, // llvm.x86.avx10.vminmaxbf16512
    1 << 8 | 11, // llvm.x86.avx10.vminmaxpd128
    1 << 8 | 11, // llvm.x86.avx10.vminmaxpd256
    1 << 8 | 11, // llvm.x86.avx10.vminmaxph128
    1 << 8 | 11, // llvm.x86.avx10.vminmaxph256
    1 << 8 | 11, // llvm.x86.avx10.vminmaxps128
    1 << 8 | 11, // llvm.x86.avx10.vminmaxps256
    23 << 8 | 1, // llvm.x86.avx10.vmovrsb128
    23 << 8 | 1, // llvm.x86.avx10.vmovrsb256
    23 << 8 | 1, // llvm.x86.avx10.vmovrsb512
    23 << 8 | 1, // llvm.x86.avx10.vmovrsd128
    23 << 8 | 1, // llvm.x86.avx10.vmovrsd256
    23 << 8 | 1, // llvm.x86.avx10.vmovrsd512
    23 << 8 | 1, // llvm.x86.avx10.vmovrsq128
    23 << 8 | 1, // llvm.x86.avx10.vmovrsq256
    23 << 8 | 1, // llvm.x86.avx10.vmovrsq512
    23 << 8 | 1, // llvm.x86.avx10.vmovrsw128
    23 << 8 | 1, // llvm.x86.avx10.vmovrsw256
    23 << 8 | 1, // llvm.x86.avx10.vmovrsw512
    1 << 8 | 11, // llvm.x86.avx10.vmpsadbw.512
    1 << 8 | 1, // llvm.x86.avx10.vmulbf16128
    1 << 8 | 1, // llvm.x86.avx10.vmulbf16256
    1 << 8 | 1, // llvm.x86.avx10.vmulbf16512
    1 << 8 | 1, // llvm.x86.avx10.vpdpbssd.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpbssds.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpbsud.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpbsuds.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpbuud.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpbuuds.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpwsud.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpwsuds.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpwusd.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpwusds.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpwuud.512
    1 << 8 | 1, // llvm.x86.avx10.vpdpwuuds.512
    1 << 8 | 1, // llvm.x86.avx10.vsubbf16128
    1 << 8 | 1, // llvm.x86.avx10.vsubbf16256
    1 << 8 | 1, // llvm.x86.avx10.vsubbf16512
    23 << 8 | 43, // llvm.x86.avx2.gather.d.d
    23 << 8 | 43, // llvm.x86.avx2.gather.d.d.256
    23 << 8 | 43, // llvm.x86.avx2.gather.d.pd
    23 << 8 | 43, // llvm.x86.avx2.gather.d.pd.256
    23 << 8 | 43, // llvm.x86.avx2.gather.d.ps
    23 << 8 | 43, // llvm.x86.avx2.gather.d.ps.256
    23 << 8 | 43, // llvm.x86.avx2.gather.d.q
    23 << 8 | 43, // llvm.x86.avx2.gather.d.q.256
    23 << 8 | 43, // llvm.x86.avx2.gather.q.d
    23 << 8 | 43, // llvm.x86.avx2.gather.q.d.256
    23 << 8 | 43, // llvm.x86.avx2.gather.q.pd
    23 << 8 | 43, // llvm.x86.avx2.gather.q.pd.256
    23 << 8 | 43, // llvm.x86.avx2.gather.q.ps
    23 << 8 | 43, // llvm.x86.avx2.gather.q.ps.256
    23 << 8 | 43, // llvm.x86.avx2.gather.q.q
    23 << 8 | 43, // llvm.x86.avx2.gather.q.q.256
    2 << 8 | 1, // llvm.x86.avx2.maskload.d
    2 << 8 | 1, // llvm.x86.avx2.maskload.d.256
    2 << 8 | 1, // llvm.x86.avx2.maskload.q
    2 << 8 | 1, // llvm.x86.avx2.maskload.q.256
    11 << 8 | 1, // llvm.x86.avx2.maskstore.d
    11 << 8 | 1, // llvm.x86.avx2.maskstore.d.256
    11 << 8 | 1, // llvm.x86.avx2.maskstore.q
    11 << 8 | 1, // llvm.x86.avx2.maskstore.q.256
    1 << 8 | 11, // llvm.x86.avx2.mpsadbw
    1 << 8 | 1, // llvm.x86.avx2.packssdw
    1 << 8 | 1, // llvm.x86.avx2.packsswb
    1 << 8 | 1, // llvm.x86.avx2.packusdw
    1 << 8 | 1, // llvm.x86.avx2.packuswb
    1 << 8 | 1, // llvm.x86.avx2.pavg.b
    1 << 8 | 1, // llvm.x86.avx2.pavg.w
    1 << 8 | 1, // llvm.x86.avx2.pblendvb
    1 << 8 | 1, // llvm.x86.avx2.permd
    1 << 8 | 1, // llvm.x86.avx2.permps
    1 << 8 | 1, // llvm.x86.avx2.phadd.d
    1 << 8 | 1, // llvm.x86.avx2.phadd.sw
    1 << 8 | 1, // llvm.x86.avx2.phadd.w
    1 << 8 | 1, // llvm.x86.avx2.phsub.d
    1 << 8 | 1, // llvm.x86.avx2.phsub.sw
    1 << 8 | 1, // llvm.x86.avx2.phsub.w
    1 << 8 | 1, // llvm.x86.avx2.pmadd.ub.sw
    1 << 8 | 1, // llvm.x86.avx2.pmadd.wd
    1 << 8 | 1, // llvm.x86.avx2.pmovmskb
    1 << 8 | 1, // llvm.x86.avx2.pmul.hr.sw
    1 << 8 | 1, // llvm.x86.avx2.pmulh.w
    1 << 8 | 1, // llvm.x86.avx2.pmulhu.w
    1 << 8 | 1, // llvm.x86.avx2.psad.bw
    1 << 8 | 1, // llvm.x86.avx2.pshuf.b
    1 << 8 | 1, // llvm.x86.avx2.psign.b
    1 << 8 | 1, // llvm.x86.avx2.psign.d
    1 << 8 | 1, // llvm.x86.avx2.psign.w
    1 << 8 | 1, // llvm.x86.avx2.psll.d
    1 << 8 | 1, // llvm.x86.avx2.psll.q
    1 << 8 | 1, // llvm.x86.avx2.psll.w
    1 << 8 | 1, // llvm.x86.avx2.pslli.d
    1 << 8 | 1, // llvm.x86.avx2.pslli.q
    1 << 8 | 1, // llvm.x86.avx2.pslli.w
    1 << 8 | 1, // llvm.x86.avx2.psllv.d
    1 << 8 | 1, // llvm.x86.avx2.psllv.d.256
    1 << 8 | 1, // llvm.x86.avx2.psllv.q
    1 << 8 | 1, // llvm.x86.avx2.psllv.q.256
    1 << 8 | 1, // llvm.x86.avx2.psra.d
    1 << 8 | 1, // llvm.x86.avx2.psra.w
    1 << 8 | 1, // llvm.x86.avx2.psrai.d
    1 << 8 | 1, // llvm.x86.avx2.psrai.w
    1 << 8 | 1, // llvm.x86.avx2.psrav.d
    1 << 8 | 1, // llvm.x86.avx2.psrav.d.256
    1 << 8 | 1, // llvm.x86.avx2.psrl.d
    1 << 8 | 1, // llvm.x86.avx2.psrl.q
    1 << 8 | 1, // llvm.x86.avx2.psrl.w
    1 << 8 | 1, // llvm.x86.avx2.psrli.d
    1 << 8 | 1, // llvm.x86.avx2.psrli.q
    1 << 8 | 1, // llvm.x86.avx2.psrli.w
    1 << 8 | 1, // llvm.x86.avx2.psrlv.d
    1 << 8 | 1, // llvm.x86.avx2.psrlv.d.256
    1 << 8 | 1, // llvm.x86.avx2.psrlv.q
    1 << 8 | 1, // llvm.x86.avx2.psrlv.q.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpbssd.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpbssd.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpbssds.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpbssds.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpbsud.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpbsud.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpbsuds.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpbsuds.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpbuud.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpbuud.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpbuuds.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpbuuds.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpwsud.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpwsud.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpwsuds.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpwsuds.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpwusd.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpwusd.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpwusds.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpwusds.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpwuud.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpwuud.256
    1 << 8 | 1, // llvm.x86.avx2.vpdpwuuds.128
    1 << 8 | 1, // llvm.x86.avx2.vpdpwuuds.256
    1 << 8 | 11, // llvm.x86.avx512.add.pd.512
    1 << 8 | 11, // llvm.x86.avx512.add.ps.512
    1 << 8 | 1, // llvm.x86.avx512.broadcastmb.128
    1 << 8 | 1, // llvm.x86.avx512.broadcastmb.256
    1 << 8 | 1, // llvm.x86.avx512.broadcastmb.512
    1 << 8 | 1, // llvm.x86.avx512.broadcastmw.128
    1 << 8 | 1, // llvm.x86.avx512.broadcastmw.256
    1 << 8 | 1, // llvm.x86.avx512.broadcastmw.512
    1 << 8 | 1, // llvm.x86.avx512.conflict.d.128
    1 << 8 | 1, // llvm.x86.avx512.conflict.d.256
    1 << 8 | 1, // llvm.x86.avx512.conflict.d.512
    1 << 8 | 1, // llvm.x86.avx512.conflict.q.128
    1 << 8 | 1, // llvm.x86.avx512.conflict.q.256
    1 << 8 | 1, // llvm.x86.avx512.conflict.q.512
    1 << 8 | 11, // llvm.x86.avx512.cvtsi2sd64
    1 << 8 | 11, // llvm.x86.avx512.cvtsi2ss32
    1 << 8 | 11, // llvm.x86.avx512.cvtsi2ss64
    1 << 8 | 0, // llvm.x86.avx512.cvttsd2si
    1 << 8 | 0, // llvm.x86.avx512.cvttsd2si64
    1 << 8 | 0, // llvm.x86.avx512.cvttsd2usi
    1 << 8 | 0, // llvm.x86.avx512.cvttsd2usi64
    1 << 8 | 0, // llvm.x86.avx512.cvttss2si
    1 << 8 | 0, // llvm.x86.avx512.cvttss2si64
    1 << 8 | 0, // llvm.x86.avx512.cvttss2usi
    1 << 8 | 0, // llvm.x86.avx512.cvttss2usi64
    1 << 8 | 11, // llvm.x86.avx512.cvtusi2ss
    1 << 8 | 11, // llvm.x86.avx512.cvtusi642sd
    1 << 8 | 11, // llvm.x86.avx512.cvtusi642ss
    1 << 8 | 11, // llvm.x86.avx512.dbpsadbw.128
    1 << 8 | 11, // llvm.x86.avx512.dbpsadbw.256
    1 << 8 | 11, // llvm.x86.avx512.dbpsadbw.512
    1 << 8 | 11, // llvm.x86.avx512.div.pd.512
    1 << 8 | 11, // llvm.x86.avx512.div.ps.512
    1 << 8 | 0, // llvm.x86.avx512.fpclass.pd.128
    1 << 8 | 0, // llvm.x86.avx512.fpclass.pd.256
    1 << 8 | 0, // llvm.x86.avx512.fpclass.pd.512
    1 << 8 | 0, // llvm.x86.avx512.fpclass.ps.128
    1 << 8 | 0, // llvm.x86.avx512.fpclass.ps.256
    1 << 8 | 0, // llvm.x86.avx512.fpclass.ps.512
    23 << 8 | 43, // llvm.x86.avx512.gather.dpd.512
    23 << 8 | 43, // llvm.x86.avx512.gather.dpi.512
    23 << 8 | 43, // llvm.x86.avx512.gather.dpq.512
    23 << 8 | 43, // llvm.x86.avx512.gather.dps.512
    23 << 8 | 43, // llvm.x86.avx512.gather.qpd.512
    23 << 8 | 43, // llvm.x86.avx512.gather.qpi.512
    23 << 8 | 43, // llvm.x86.avx512.gather.qpq.512
    23 << 8 | 43, // llvm.x86.avx512.gather.qps.512
    23 << 8 | 43, // llvm.x86.avx512.gather3div2.df
    23 << 8 | 43, // llvm.x86.avx512.gather3div2.di
    23 << 8 | 43, // llvm.x86.avx512.gather3div4.df
    23 << 8 | 43, // llvm.x86.avx512.gather3div4.di
    23 << 8 | 43, // llvm.x86.avx512.gather3div4.sf
    23 << 8 | 43, // llvm.x86.avx512.gather3div4.si
    23 << 8 | 43, // llvm.x86.avx512.gather3div8.sf
    23 << 8 | 43, // llvm.x86.avx512.gather3div8.si
    23 << 8 | 43, // llvm.x86.avx512.gather3siv2.df
    23 << 8 | 43, // llvm.x86.avx512.gather3siv2.di
    23 << 8 | 43, // llvm.x86.avx512.gather3siv4.df
    23 << 8 | 43, // llvm.x86.avx512.gather3siv4.di
    23 << 8 | 43, // llvm.x86.avx512.gather3siv4.sf
    23 << 8 | 43, // llvm.x86.avx512.gather3siv4.si
    23 << 8 | 43, // llvm.x86.avx512.gather3siv8.sf
    23 << 8 | 43, // llvm.x86.avx512.gather3siv8.si
    1 << 8 | 1, // llvm.x86.avx512.kadd.b
    1 << 8 | 1, // llvm.x86.avx512.kadd.d
    1 << 8 | 1, // llvm.x86.avx512.kadd.q
    1 << 8 | 1, // llvm.x86.avx512.kadd.w
    1 << 8 | 1, // llvm.x86.avx512.ktestc.b
    1 << 8 | 1, // llvm.x86.avx512.ktestc.d
    1 << 8 | 1, // llvm.x86.avx512.ktestc.q
    1 << 8 | 1, // llvm.x86.avx512.ktestc.w
    1 << 8 | 1, // llvm.x86.avx512.ktestz.b
    1 << 8 | 1, // llvm.x86.avx512.ktestz.d
    1 << 8 | 1, // llvm.x86.avx512.ktestz.q
    1 << 8 | 1, // llvm.x86.avx512.ktestz.w
    1 << 8 | 43, // llvm.x86.avx512.mask.add.sd.round
    1 << 8 | 43, // llvm.x86.avx512.mask.add.ss.round
    1 << 8 | 11, // llvm.x86.avx512.mask.cmp.pd.128
    1 << 8 | 11, // llvm.x86.avx512.mask.cmp.pd.256
    1 << 8 | 240, // llvm.x86.avx512.mask.cmp.pd.512
    1 << 8 | 11, // llvm.x86.avx512.mask.cmp.ps.128
    1 << 8 | 11, // llvm.x86.avx512.mask.cmp.ps.256
    1 << 8 | 240, // llvm.x86.avx512.mask.cmp.ps.512
    1 << 8 | 240, // llvm.x86.avx512.mask.cmp.sd
    1 << 8 | 240, // llvm.x86.avx512.mask.cmp.ss
    1 << 8 | 1, // llvm.x86.avx512.mask.compress
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtpd2dq.128
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtpd2dq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtpd2ps
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtpd2ps.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtpd2qq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtpd2qq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtpd2qq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtpd2udq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtpd2udq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtpd2udq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtpd2uqq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtpd2uqq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtpd2uqq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtps2dq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtps2dq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtps2dq.512
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtps2pd.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtps2qq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtps2qq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtps2qq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtps2udq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtps2udq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtps2udq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtps2uqq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtps2uqq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvtps2uqq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtqq2ps.128
    1 << 8 | 43, // llvm.x86.avx512.mask.cvtsd2ss.round
    1 << 8 | 43, // llvm.x86.avx512.mask.cvtss2sd.round
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttpd2dq.128
    1 << 8 | 35, // llvm.x86.avx512.mask.cvttpd2dq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttpd2qq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttpd2qq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvttpd2qq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttpd2udq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttpd2udq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvttpd2udq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttpd2uqq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttpd2uqq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvttpd2uqq.512
    1 << 8 | 35, // llvm.x86.avx512.mask.cvttps2dq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttps2qq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttps2qq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvttps2qq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttps2udq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttps2udq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvttps2udq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttps2uqq.128
    1 << 8 | 1, // llvm.x86.avx512.mask.cvttps2uqq.256
    1 << 8 | 35, // llvm.x86.avx512.mask.cvttps2uqq.512
    1 << 8 | 1, // llvm.x86.avx512.mask.cvtuqq2ps.128
    1 << 8 | 43, // llvm.x86.avx512.mask.div.sd.round
    1 << 8 | 43, // llvm.x86.avx512.mask.div.ss.round
    1 << 8 | 1, // llvm.x86.avx512.mask.expand
    1 << 8 | 35, // llvm.x86.avx512.mask.fixupimm.pd.128
    1 << 8 | 35, // llvm.x86.avx512.mask.fixupimm.pd.256
    1 << 8 | 241, // llvm.x86.avx512.mask.fixupimm.pd.512
    1 << 8 | 35, // llvm.x86.avx512.mask.fixupimm.ps.128
    1 << 8 | 35, // llvm.x86.avx512.mask.fixupimm.ps.256
    1 << 8 | 241, // llvm.x86.avx512.mask.fixupimm.ps.512
    1 << 8 | 241, // llvm.x86.avx512.mask.fixupimm.sd
    1 << 8 | 241, // llvm.x86.avx512.mask.fixupimm.ss
    1 << 8 | 0, // llvm.x86.avx512.mask.fpclass.sd
    1 << 8 | 0, // llvm.x86.avx512.mask.fpclass.ss
    23 << 8 | 43, // llvm.x86.avx512.mask.gather.dpd.512
    23 << 8 | 43, // llvm.x86.avx512.mask.gather.dpi.512
    23 << 8 | 43, // llvm.x86.avx512.mask.gather.dpq.512
    23 << 8 | 43, // llvm.x86.avx512.mask.gather.dps.512
    23 << 8 | 43, // llvm.x86.avx512.mask.gather.qpd.512
    23 << 8 | 43, // llvm.x86.avx512.mask.gather.qpi.512
    23 << 8 | 43, // llvm.x86.avx512.mask.gather.qpq.512
    23 << 8 | 43, // llvm.x86.avx512.mask.gather.qps.512
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3div2.df
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3div2.di
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3div4.df
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3div4.di
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3div4.sf
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3div4.si
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3div8.sf
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3div8.si
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3siv2.df
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3siv2.di
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3siv4.df
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3siv4.di
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3siv4.sf
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3siv4.si
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3siv8.sf
    23 << 8 | 43, // llvm.x86.avx512.mask.gather3siv8.si
    1 << 8 | 1, // llvm.x86.avx512.mask.getexp.pd.128
    1 << 8 | 1, // llvm.x86.avx512.mask.getexp.pd.256
    1 << 8 | 35, // llvm.x86.avx512.mask.getexp.pd.512
    1 << 8 | 1, // llvm.x86.avx512.mask.getexp.ps.128
    1 << 8 | 1, // llvm.x86.avx512.mask.getexp.ps.256
    1 << 8 | 35, // llvm.x86.avx512.mask.getexp.ps.512
    1 << 8 | 43, // llvm.x86.avx512.mask.getexp.sd
    1 << 8 | 43, // llvm.x86.avx512.mask.getexp.ss
    1 << 8 | 0, // llvm.x86.avx512.mask.getmant.pd.128
    1 << 8 | 0, // llvm.x86.avx512.mask.getmant.pd.256
    1 << 8 | 246, // llvm.x86.avx512.mask.getmant.pd.512
    1 << 8 | 0, // llvm.x86.avx512.mask.getmant.ps.128
    1 << 8 | 0, // llvm.x86.avx512.mask.getmant.ps.256
    1 << 8 | 246, // llvm.x86.avx512.mask.getmant.ps.512
    1 << 8 | 245, // llvm.x86.avx512.mask.getmant.sd
    1 << 8 | 245, // llvm.x86.avx512.mask.getmant.ss
    1 << 8 | 43, // llvm.x86.avx512.mask.max.sd.round
    1 << 8 | 43, // llvm.x86.avx512.mask.max.ss.round
    1 << 8 | 43, // llvm.x86.avx512.mask.min.sd.round
    1 << 8 | 43, // llvm.x86.avx512.mask.min.ss.round
    1 << 8 | 43, // llvm.x86.avx512.mask.mul.sd.round
    1 << 8 | 43, // llvm.x86.avx512.mask.mul.ss.round
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.db.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.db.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.db.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.db.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.db.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.db.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.dw.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.dw.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.dw.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.dw.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.dw.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.dw.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.qb.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.qb.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.qb.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qb.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qb.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qb.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.qd.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qd.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qd.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qd.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.qw.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.qw.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.qw.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qw.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qw.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.qw.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmov.wb.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.wb.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.wb.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmov.wb.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.db.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.db.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.db.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.db.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.db.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.db.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.dw.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.dw.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.dw.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.dw.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.dw.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.dw.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qb.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qb.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qb.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qb.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qb.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qb.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qd.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qd.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qd.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qd.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qd.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qd.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qw.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qw.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qw.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qw.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qw.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.qw.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.wb.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.wb.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovs.wb.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.wb.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.wb.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovs.wb.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.db.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.db.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.db.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.db.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.db.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.db.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.dw.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.dw.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.dw.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.dw.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.dw.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.dw.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qb.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qb.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qb.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qb.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qb.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qb.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qd.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qd.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qd.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qd.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qd.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qd.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qw.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qw.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qw.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qw.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qw.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.qw.mem.512
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.wb.128
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.wb.256
    1 << 8 | 1, // llvm.x86.avx512.mask.pmovus.wb.512
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.wb.mem.128
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.wb.mem.256
    18 << 8 | 1, // llvm.x86.avx512.mask.pmovus.wb.mem.512
    1 << 8 | 11, // llvm.x86.avx512.mask.range.pd.128
    1 << 8 | 11, // llvm.x86.avx512.mask.range.pd.256
    1 << 8 | 245, // llvm.x86.avx512.mask.range.pd.512
    1 << 8 | 11, // llvm.x86.avx512.mask.range.ps.128
    1 << 8 | 11, // llvm.x86.avx512.mask.range.ps.256
    1 << 8 | 245, // llvm.x86.avx512.mask.range.ps.512
    1 << 8 | 67, // llvm.x86.avx512.mask.range.sd
    1 << 8 | 67, // llvm.x86.avx512.mask.range.ss
    1 << 8 | 0, // llvm.x86.avx512.mask.reduce.pd.128
    1 << 8 | 0, // llvm.x86.avx512.mask.reduce.pd.256
    1 << 8 | 246, // llvm.x86.avx512.mask.reduce.pd.512
    1 << 8 | 0, // llvm.x86.avx512.mask.reduce.ps.128
    1 << 8 | 0, // llvm.x86.avx512.mask.reduce.ps.256
    1 << 8 | 246, // llvm.x86.avx512.mask.reduce.ps.512
    1 << 8 | 67, // llvm.x86.avx512.mask.reduce.sd
    1 << 8 | 67, // llvm.x86.avx512.mask.reduce.ss
    1 << 8 | 0, // llvm.x86.avx512.mask.rndscale.pd.128
    1 << 8 | 0, // llvm.x86.avx512.mask.rndscale.pd.256
    1 << 8 | 246, // llvm.x86.avx512.mask.rndscale.pd.512
    1 << 8 | 0, // llvm.x86.avx512.mask.rndscale.ps.128
    1 << 8 | 0, // llvm.x86.avx512.mask.rndscale.ps.256
    1 << 8 | 246, // llvm.x86.avx512.mask.rndscale.ps.512
    1 << 8 | 67, // llvm.x86.avx512.mask.rndscale.sd
    1 << 8 | 67, // llvm.x86.avx512.mask.rndscale.ss
    1 << 8 | 1, // llvm.x86.avx512.mask.scalef.pd.128
    1 << 8 | 1, // llvm.x86.avx512.mask.scalef.pd.256
    1 << 8 | 43, // llvm.x86.avx512.mask.scalef.pd.512
    1 << 8 | 1, // llvm.x86.avx512.mask.scalef.ps.128
    1 << 8 | 1, // llvm.x86.avx512.mask.scalef.ps.256
    1 << 8 | 43, // llvm.x86.avx512.mask.scalef.ps.512
    1 << 8 | 43, // llvm.x86.avx512.mask.scalef.sd
    1 << 8 | 43, // llvm.x86.avx512.mask.scalef.ss
    5 << 8 | 43, // llvm.x86.avx512.mask.scatter.dpd.512
    5 << 8 | 43, // llvm.x86.avx512.mask.scatter.dpi.512
    5 << 8 | 43, // llvm.x86.avx512.mask.scatter.dpq.512
    5 << 8 | 43, // llvm.x86.avx512.mask.scatter.dps.512
    5 << 8 | 43, // llvm.x86.avx512.mask.scatter.qpd.512
    5 << 8 | 43, // llvm.x86.avx512.mask.scatter.qpi.512
    5 << 8 | 43, // llvm.x86.avx512.mask.scatter.qpq.512
    5 << 8 | 43, // llvm.x86.avx512.mask.scatter.qps.512
    5 << 8 | 43, // llvm.x86.avx512.mask.scatterdiv2.df
    5 << 8 | 43, // llvm.x86.avx512.mask.scatterdiv2.di
    5 << 8 | 43, // llvm.x86.avx512.mask.scatterdiv4.df
    5 << 8 | 43, // llvm.x86.avx512.mask.scatterdiv4.di
    5 << 8 | 43, // llvm.x86.avx512.mask.scatterdiv4.sf
    5 << 8 | 43, // llvm.x86.avx512.mask.scatterdiv4.si
    5 << 8 | 43, // llvm.x86.avx512.mask.scatterdiv8.sf
    5 << 8 | 43, // llvm.x86.avx512.mask.scatterdiv8.si
    5 << 8 | 43, // llvm.x86.avx512.mask.scattersiv2.df
    5 << 8 | 43, // llvm.x86.avx512.mask.scattersiv2.di
    5 << 8 | 43, // llvm.x86.avx512.mask.scattersiv4.df
    5 << 8 | 43, // llvm.x86.avx512.mask.scattersiv4.di
    5 << 8 | 43, // llvm.x86.avx512.mask.scattersiv4.sf
    5 << 8 | 43, // llvm.x86.avx512.mask.scattersiv4.si
    5 << 8 | 43, // llvm.x86.avx512.mask.scattersiv8.sf
    5 << 8 | 43, // llvm.x86.avx512.mask.scattersiv8.si
    1 << 8 | 43, // llvm.x86.avx512.mask.sqrt.sd
    1 << 8 | 43, // llvm.x86.avx512.mask.sqrt.ss
    1 << 8 | 43, // llvm.x86.avx512.mask.sub.sd.round
    1 << 8 | 43, // llvm.x86.avx512.mask.sub.ss.round
    1 << 8 | 35, // llvm.x86.avx512.mask.vcvtph2ps.512
    1 << 8 | 0, // llvm.x86.avx512.mask.vcvtps2ph.128
    1 << 8 | 0, // llvm.x86.avx512.mask.vcvtps2ph.256
    1 << 8 | 0, // llvm.x86.avx512.mask.vcvtps2ph.512
    1 << 8 | 35, // llvm.x86.avx512.maskz.fixupimm.pd.128
    1 << 8 | 35, // llvm.x86.avx512.maskz.fixupimm.pd.256
    1 << 8 | 241, // llvm.x86.avx512.maskz.fixupimm.pd.512
    1 << 8 | 35, // llvm.x86.avx512.maskz.fixupimm.ps.128
    1 << 8 | 35, // llvm.x86.avx512.maskz.fixupimm.ps.256
    1 << 8 | 241, // llvm.x86.avx512.maskz.fixupimm.ps.512
    1 << 8 | 241, // llvm.x86.avx512.maskz.fixupimm.sd
    1 << 8 | 241, // llvm.x86.avx512.maskz.fixupimm.ss
    1 << 8 | 11, // llvm.x86.avx512.max.pd.512
    1 << 8 | 11, // llvm.x86.avx512.max.ps.512
    1 << 8 | 11, // llvm.x86.avx512.min.pd.512
    1 << 8 | 11, // llvm.x86.avx512.min.ps.512
    1 << 8 | 11, // llvm.x86.avx512.mul.pd.512
    1 << 8 | 11, // llvm.x86.avx512.mul.ps.512
    1 << 8 | 1, // llvm.x86.avx512.packssdw.512
    1 << 8 | 1, // llvm.x86.avx512.packsswb.512
    1 << 8 | 1, // llvm.x86.avx512.packusdw.512
    1 << 8 | 1, // llvm.x86.avx512.packuswb.512
    1 << 8 | 1, // llvm.x86.avx512.pavg.b.512
    1 << 8 | 1, // llvm.x86.avx512.pavg.w.512
    1 << 8 | 1, // llvm.x86.avx512.permvar.df.256
    1 << 8 | 1, // llvm.x86.avx512.permvar.df.512
    1 << 8 | 1, // llvm.x86.avx512.permvar.di.256
    1 << 8 | 1, // llvm.x86.avx512.permvar.di.512
    1 << 8 | 1, // llvm.x86.avx512.permvar.hi.128
    1 << 8 | 1, // llvm.x86.avx512.permvar.hi.256
    1 << 8 | 1, // llvm.x86.avx512.permvar.hi.512
    1 << 8 | 1, // llvm.x86.avx512.permvar.qi.128
    1 << 8 | 1, // llvm.x86.avx512.permvar.qi.256
    1 << 8 | 1, // llvm.x86.avx512.permvar.qi.512
    1 << 8 | 1, // llvm.x86.avx512.permvar.sf.512
    1 << 8 | 1, // llvm.x86.avx512.permvar.si.512
    1 << 8 | 1, // llvm.x86.avx512.pmaddubs.w.512
    1 << 8 | 1, // llvm.x86.avx512.pmaddw.d.512
    1 << 8 | 1, // llvm.x86.avx512.pmul.hr.sw.512
    1 << 8 | 1, // llvm.x86.avx512.pmulh.w.512
    1 << 8 | 1, // llvm.x86.avx512.pmulhu.w.512
    1 << 8 | 1, // llvm.x86.avx512.pmultishift.qb.128
    1 << 8 | 1, // llvm.x86.avx512.pmultishift.qb.256
    1 << 8 | 1, // llvm.x86.avx512.pmultishift.qb.512
    1 << 8 | 1, // llvm.x86.avx512.psad.bw.512
    1 << 8 | 1, // llvm.x86.avx512.pshuf.b.512
    1 << 8 | 1, // llvm.x86.avx512.psll.d.512
    1 << 8 | 1, // llvm.x86.avx512.psll.q.512
    1 << 8 | 1, // llvm.x86.avx512.psll.w.512
    1 << 8 | 1, // llvm.x86.avx512.pslli.d.512
    1 << 8 | 1, // llvm.x86.avx512.pslli.q.512
    1 << 8 | 1, // llvm.x86.avx512.pslli.w.512
    1 << 8 | 1, // llvm.x86.avx512.psllv.d.512
    1 << 8 | 1, // llvm.x86.avx512.psllv.q.512
    1 << 8 | 1, // llvm.x86.avx512.psllv.w.128
    1 << 8 | 1, // llvm.x86.avx512.psllv.w.256
    1 << 8 | 1, // llvm.x86.avx512.psllv.w.512
    1 << 8 | 1, // llvm.x86.avx512.psra.d.512
    1 << 8 | 1, // llvm.x86.avx512.psra.q.128
    1 << 8 | 1, // llvm.x86.avx512.psra.q.256
    1 << 8 | 1, // llvm.x86.avx512.psra.q.512
    1 << 8 | 1, // llvm.x86.avx512.psra.w.512
    1 << 8 | 1, // llvm.x86.avx512.psrai.d.512
    1 << 8 | 1, // llvm.x86.avx512.psrai.q.128
    1 << 8 | 1, // llvm.x86.avx512.psrai.q.256
    1 << 8 | 1, // llvm.x86.avx512.psrai.q.512
    1 << 8 | 1, // llvm.x86.avx512.psrai.w.512
    1 << 8 | 1, // llvm.x86.avx512.psrav.d.512
    1 << 8 | 1, // llvm.x86.avx512.psrav.q.128
    1 << 8 | 1, // llvm.x86.avx512.psrav.q.256
    1 << 8 | 1, // llvm.x86.avx512.psrav.q.512
    1 << 8 | 1, // llvm.x86.avx512.psrav.w.128
    1 << 8 | 1, // llvm.x86.avx512.psrav.w.256
    1 << 8 | 1, // llvm.x86.avx512.psrav.w.512
    1 << 8 | 1, // llvm.x86.avx512.psrl.d.512
    1 << 8 | 1, // llvm.x86.avx512.psrl.q.512
    1 << 8 | 1, // llvm.x86.avx512.psrl.w.512
    1 << 8 | 1, // llvm.x86.avx512.psrli.d.512
    1 << 8 | 1, // llvm.x86.avx512.psrli.q.512
    1 << 8 | 1, // llvm.x86.avx512.psrli.w.512
    1 << 8 | 1, // llvm.x86.avx512.psrlv.d.512
    1 << 8 | 1, // llvm.x86.avx512.psrlv.q.512
    1 << 8 | 1, // llvm.x86.avx512.psrlv.w.128
    1 << 8 | 1, // llvm.x86.avx512.psrlv.w.256
    1 << 8 | 1, // llvm.x86.avx512.psrlv.w.512
    1 << 8 | 35, // llvm.x86.avx512.pternlog.d.128
    1 << 8 | 35, // llvm.x86.avx512.pternlog.d.256
    1 << 8 | 35, // llvm.x86.avx512.pternlog.d.512
    1 << 8 | 35, // llvm.x86.avx512.pternlog.q.128
    1 << 8 | 35, // llvm.x86.avx512.pternlog.q.256
    1 << 8 | 35, // llvm.x86.avx512.pternlog.q.512
    1 << 8 | 1, // llvm.x86.avx512.rcp14.pd.128
    1 << 8 | 1, // llvm.x86.avx512.rcp14.pd.256
    1 << 8 | 1, // llvm.x86.avx512.rcp14.pd.512
    1 << 8 | 1, // llvm.x86.avx512.rcp14.ps.128
    1 << 8 | 1, // llvm.x86.avx512.rcp14.ps.256
    1 << 8 | 1, // llvm.x86.avx512.rcp14.ps.512
    1 << 8 | 1, // llvm.x86.avx512.rcp14.sd
    1 << 8 | 1, // llvm.x86.avx512.rcp14.ss
    1 << 8 | 1, // llvm.x86.avx512.rsqrt14.pd.128
    1 << 8 | 1, // llvm.x86.avx512.rsqrt14.pd.256
    1 << 8 | 1, // llvm.x86.avx512.rsqrt14.pd.512
    1 << 8 | 1, // llvm.x86.avx512.rsqrt14.ps.128
    1 << 8 | 1, // llvm.x86.avx512.rsqrt14.ps.256
    1 << 8 | 1, // llvm.x86.avx512.rsqrt14.ps.512
    1 << 8 | 1, // llvm.x86.avx512.rsqrt14.sd
    1 << 8 | 1, // llvm.x86.avx512.rsqrt14.ss
    5 << 8 | 43, // llvm.x86.avx512.scatter.dpd.512
    5 << 8 | 43, // llvm.x86.avx512.scatter.dpi.512
    5 << 8 | 43, // llvm.x86.avx512.scatter.dpq.512
    5 << 8 | 43, // llvm.x86.avx512.scatter.dps.512
    5 << 8 | 43, // llvm.x86.avx512.scatter.qpd.512
    5 << 8 | 43, // llvm.x86.avx512.scatter.qpi.512
    5 << 8 | 43, // llvm.x86.avx512.scatter.qpq.512
    5 << 8 | 43, // llvm.x86.avx512.scatter.qps.512
    5 << 8 | 43, // llvm.x86.avx512.scatterdiv2.df
    5 << 8 | 43, // llvm.x86.avx512.scatterdiv2.di
    5 << 8 | 43, // llvm.x86.avx512.scatterdiv4.df
    5 << 8 | 43, // llvm.x86.avx512.scatterdiv4.di
    5 << 8 | 43, // llvm.x86.avx512.scatterdiv4.sf
    5 << 8 | 43, // llvm.x86.avx512.scatterdiv4.si
    5 << 8 | 43, // llvm.x86.avx512.scatterdiv8.sf
    5 << 8 | 43, // llvm.x86.avx512.scatterdiv8.si
    5 << 8 | 43, // llvm.x86.avx512.scattersiv2.df
    5 << 8 | 43, // llvm.x86.avx512.scattersiv2.di
    5 << 8 | 43, // llvm.x86.avx512.scattersiv4.df
    5 << 8 | 43, // llvm.x86.avx512.scattersiv4.di
    5 << 8 | 43, // llvm.x86.avx512.scattersiv4.sf
    5 << 8 | 43, // llvm.x86.avx512.scattersiv4.si
    5 << 8 | 43, // llvm.x86.avx512.scattersiv8.sf
    5 << 8 | 43, // llvm.x86.avx512.scattersiv8.si
    1 << 8 | 0, // llvm.x86.avx512.sitofp.round
    1 << 8 | 0, // llvm.x86.avx512.sqrt.pd.512
    1 << 8 | 0, // llvm.x86.avx512.sqrt.ps.512
    1 << 8 | 11, // llvm.x86.avx512.sub.pd.512
    1 << 8 | 11, // llvm.x86.avx512.sub.ps.512
    1 << 8 | 0, // llvm.x86.avx512.uitofp.round
    1 << 8 | 93, // llvm.x86.avx512.vcomi.sd
    1 << 8 | 93, // llvm.x86.avx512.vcomi.ss
    1 << 8 | 0, // llvm.x86.avx512.vcvtsd2si32
    1 << 8 | 0, // llvm.x86.avx512.vcvtsd2si64
    1 << 8 | 0, // llvm.x86.avx512.vcvtsd2usi32
    1 << 8 | 0, // llvm.x86.avx512.vcvtsd2usi64
    1 << 8 | 0, // llvm.x86.avx512.vcvtss2si32
    1 << 8 | 0, // llvm.x86.avx512.vcvtss2si64
    1 << 8 | 0, // llvm.x86.avx512.vcvtss2usi32
    1 << 8 | 0, // llvm.x86.avx512.vcvtss2usi64
    1 << 8 | 35, // llvm.x86.avx512.vfmadd.f32
    1 << 8 | 35, // llvm.x86.avx512.vfmadd.f64
    1 << 8 | 35, // llvm.x86.avx512.vfmadd.pd.512
    1 << 8 | 35, // llvm.x86.avx512.vfmadd.ps.512
    1 << 8 | 35, // llvm.x86.avx512.vfmaddsub.pd.512
    1 << 8 | 35, // llvm.x86.avx512.vfmaddsub.ps.512
    1 << 8 | 1, // llvm.x86.avx512.vp2intersect.d.128
    1 << 8 | 1, // llvm.x86.avx512.vp2intersect.d.256
    1 << 8 | 1, // llvm.x86.avx512.vp2intersect.d.512
    1 << 8 | 1, // llvm.x86.avx512.vp2intersect.q.128
    1 << 8 | 1, // llvm.x86.avx512.vp2intersect.q.256
    1 << 8 | 1, // llvm.x86.avx512.vp2intersect.q.512
    1 << 8 | 1, // llvm.x86.avx512.vpdpbusd.128
    1 << 8 | 1, // llvm.x86.avx512.vpdpbusd.256
    1 << 8 | 1, // llvm.x86.avx512.vpdpbusd.512
    1 << 8 | 1, // llvm.x86.avx512.vpdpbusds.128
    1 << 8 | 1, // llvm.x86.avx512.vpdpbusds.256
    1 << 8 | 1, // llvm.x86.avx512.vpdpbusds.512
    1 << 8 | 1, // llvm.x86.avx512.vpdpwssd.128
    1 << 8 | 1, // llvm.x86.avx512.vpdpwssd.256
    1 << 8 | 1, // llvm.x86.avx512.vpdpwssd.512
    1 << 8 | 1, // llvm.x86.avx512.vpdpwssds.128
    1 << 8 | 1, // llvm.x86.avx512.vpdpwssds.256
    1 << 8 | 1, // llvm.x86.avx512.vpdpwssds.512
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.d.128
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.d.256
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.d.512
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.hi.128
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.hi.256
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.hi.512
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.pd.128
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.pd.256
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.pd.512
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.ps.128
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.ps.256
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.ps.512
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.q.128
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.q.256
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.q.512
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.qi.128
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.qi.256
    1 << 8 | 1, // llvm.x86.avx512.vpermi2var.qi.512
    1 << 8 | 1, // llvm.x86.avx512.vpermilvar.pd.512
    1 << 8 | 1, // llvm.x86.avx512.vpermilvar.ps.512
    1 << 8 | 1, // llvm.x86.avx512.vpmadd52h.uq.128
    1 << 8 | 1, // llvm.x86.avx512.vpmadd52h.uq.256
    1 << 8 | 1, // llvm.x86.avx512.vpmadd52h.uq.512
    1 << 8 | 1, // llvm.x86.avx512.vpmadd52l.uq.128
    1 << 8 | 1, // llvm.x86.avx512.vpmadd52l.uq.256
    1 << 8 | 1, // llvm.x86.avx512.vpmadd52l.uq.512
    1 << 8 | 1, // llvm.x86.avx512.vpshufbitqmb.128
    1 << 8 | 1, // llvm.x86.avx512.vpshufbitqmb.256
    1 << 8 | 1, // llvm.x86.avx512.vpshufbitqmb.512
    1 << 8 | 1, // llvm.x86.avx512bf16.cvtne2ps2bf16.128
    1 << 8 | 1, // llvm.x86.avx512bf16.cvtne2ps2bf16.256
    1 << 8 | 1, // llvm.x86.avx512bf16.cvtne2ps2bf16.512
    1 << 8 | 1, // llvm.x86.avx512bf16.cvtneps2bf16.256
    1 << 8 | 1, // llvm.x86.avx512bf16.cvtneps2bf16.512
    1 << 8 | 1, // llvm.x86.avx512bf16.dpbf16ps.128
    1 << 8 | 1, // llvm.x86.avx512bf16.dpbf16ps.256
    1 << 8 | 1, // llvm.x86.avx512bf16.dpbf16ps.512
    1 << 8 | 1, // llvm.x86.avx512bf16.mask.cvtneps2bf16.128
    1 << 8 | 11, // llvm.x86.avx512fp16.add.ph.512
    1 << 8 | 11, // llvm.x86.avx512fp16.div.ph.512
    1 << 8 | 0, // llvm.x86.avx512fp16.fpclass.ph.128
    1 << 8 | 0, // llvm.x86.avx512fp16.fpclass.ph.256
    1 << 8 | 0, // llvm.x86.avx512fp16.fpclass.ph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.add.sh.round
    1 << 8 | 11, // llvm.x86.avx512fp16.mask.cmp.ph.128
    1 << 8 | 11, // llvm.x86.avx512fp16.mask.cmp.ph.256
    1 << 8 | 240, // llvm.x86.avx512fp16.mask.cmp.ph.512
    1 << 8 | 240, // llvm.x86.avx512fp16.mask.cmp.sh
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.div.sh.round
    1 << 8 | 0, // llvm.x86.avx512fp16.mask.fpclass.sh
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.getexp.ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.getexp.ph.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.getexp.ph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.getexp.sh
    1 << 8 | 0, // llvm.x86.avx512fp16.mask.getmant.ph.128
    1 << 8 | 0, // llvm.x86.avx512fp16.mask.getmant.ph.256
    1 << 8 | 246, // llvm.x86.avx512fp16.mask.getmant.ph.512
    1 << 8 | 245, // llvm.x86.avx512fp16.mask.getmant.sh
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.max.sh.round
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.min.sh.round
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.mul.sh.round
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.rcp.ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.rcp.ph.256
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.rcp.ph.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.rcp.sh
    1 << 8 | 0, // llvm.x86.avx512fp16.mask.reduce.ph.128
    1 << 8 | 0, // llvm.x86.avx512fp16.mask.reduce.ph.256
    1 << 8 | 246, // llvm.x86.avx512fp16.mask.reduce.ph.512
    1 << 8 | 67, // llvm.x86.avx512fp16.mask.reduce.sh
    1 << 8 | 0, // llvm.x86.avx512fp16.mask.rndscale.ph.128
    1 << 8 | 0, // llvm.x86.avx512fp16.mask.rndscale.ph.256
    1 << 8 | 246, // llvm.x86.avx512fp16.mask.rndscale.ph.512
    1 << 8 | 67, // llvm.x86.avx512fp16.mask.rndscale.sh
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.rsqrt.ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.rsqrt.ph.256
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.rsqrt.ph.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.rsqrt.sh
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.scalef.ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.scalef.ph.256
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.scalef.ph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.scalef.sh
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.sqrt.sh
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.sub.sh.round
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtdq2ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtpd2ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtpd2ph.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtpd2ph.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2dq.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2dq.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtph2dq.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2pd.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2pd.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtph2pd.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2psx.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2psx.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtph2psx.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2qq.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2qq.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtph2qq.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2udq.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2udq.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtph2udq.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2uqq.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2uqq.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtph2uqq.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2uw.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2uw.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtph2uw.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2w.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtph2w.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtph2w.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtps2phx.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtps2phx.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvtps2phx.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtqq2ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtqq2ph.256
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vcvtsd2sh.round
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vcvtsh2sd.round
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vcvtsh2ss.round
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vcvtss2sh.round
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2dq.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2dq.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvttph2dq.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2qq.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2qq.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvttph2qq.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2udq.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2udq.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvttph2udq.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2uqq.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2uqq.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvttph2uqq.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2uw.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2uw.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvttph2uw.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2w.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvttph2w.256
    1 << 8 | 35, // llvm.x86.avx512fp16.mask.vcvttph2w.512
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtudq2ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtuqq2ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vcvtuqq2ph.256
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vfcmadd.cph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vfcmadd.cph.256
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vfcmadd.cph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vfcmadd.csh
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vfcmul.cph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vfcmul.cph.256
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vfcmul.cph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vfcmul.csh
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vfmadd.cph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vfmadd.cph.256
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vfmadd.cph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vfmadd.csh
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vfmul.cph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.mask.vfmul.cph.256
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vfmul.cph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.mask.vfmul.csh
    1 << 8 | 1, // llvm.x86.avx512fp16.maskz.vfcmadd.cph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.maskz.vfcmadd.cph.256
    1 << 8 | 43, // llvm.x86.avx512fp16.maskz.vfcmadd.cph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.maskz.vfcmadd.csh
    1 << 8 | 1, // llvm.x86.avx512fp16.maskz.vfmadd.cph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.maskz.vfmadd.cph.256
    1 << 8 | 43, // llvm.x86.avx512fp16.maskz.vfmadd.cph.512
    1 << 8 | 43, // llvm.x86.avx512fp16.maskz.vfmadd.csh
    1 << 8 | 1, // llvm.x86.avx512fp16.max.ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.max.ph.256
    1 << 8 | 11, // llvm.x86.avx512fp16.max.ph.512
    1 << 8 | 1, // llvm.x86.avx512fp16.min.ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.min.ph.256
    1 << 8 | 11, // llvm.x86.avx512fp16.min.ph.512
    1 << 8 | 11, // llvm.x86.avx512fp16.mul.ph.512
    1 << 8 | 0, // llvm.x86.avx512fp16.sqrt.ph.512
    1 << 8 | 11, // llvm.x86.avx512fp16.sub.ph.512
    1 << 8 | 93, // llvm.x86.avx512fp16.vcomi.sh
    1 << 8 | 0, // llvm.x86.avx512fp16.vcvtsh2si32
    1 << 8 | 0, // llvm.x86.avx512fp16.vcvtsh2si64
    1 << 8 | 0, // llvm.x86.avx512fp16.vcvtsh2usi32
    1 << 8 | 0, // llvm.x86.avx512fp16.vcvtsh2usi64
    1 << 8 | 11, // llvm.x86.avx512fp16.vcvtsi2sh
    1 << 8 | 11, // llvm.x86.avx512fp16.vcvtsi642sh
    1 << 8 | 0, // llvm.x86.avx512fp16.vcvttsh2si32
    1 << 8 | 0, // llvm.x86.avx512fp16.vcvttsh2si64
    1 << 8 | 0, // llvm.x86.avx512fp16.vcvttsh2usi32
    1 << 8 | 0, // llvm.x86.avx512fp16.vcvttsh2usi64
    1 << 8 | 11, // llvm.x86.avx512fp16.vcvtusi2sh
    1 << 8 | 11, // llvm.x86.avx512fp16.vcvtusi642sh
    1 << 8 | 35, // llvm.x86.avx512fp16.vfmadd.f16
    1 << 8 | 35, // llvm.x86.avx512fp16.vfmadd.ph.512
    1 << 8 | 1, // llvm.x86.avx512fp16.vfmaddsub.ph.128
    1 << 8 | 1, // llvm.x86.avx512fp16.vfmaddsub.ph.256
    1 << 8 | 35, // llvm.x86.avx512fp16.vfmaddsub.ph.512
    11 << 8 | 1, // llvm.x86.axor32
    11 << 8 | 1, // llvm.x86.axor64
    1 << 8 | 1, // llvm.x86.bmi.bextr.32
    1 << 8 | 1, // llvm.x86.bmi.bextr.64
    1 << 8 | 1, // llvm.x86.bmi.bzhi.32
    1 << 8 | 1, // llvm.x86.bmi.bzhi.64
    1 << 8 | 1, // llvm.x86.bmi.pdep.32
    1 << 8 | 1, // llvm.x86.bmi.pdep.64
    1 << 8 | 1, // llvm.x86.bmi.pext.32
    1 << 8 | 1, // llvm.x86.bmi.pext.64
    1 << 8 | 1, // llvm.x86.cast.tile.to.vector
    1 << 8 | 1, // llvm.x86.cast.vector.to.tile
    5 << 8 | 1, // llvm.x86.cldemote
    5 << 8 | 1, // llvm.x86.clflushopt
    5 << 8 | 1, // llvm.x86.clrssbsy
    5 << 8 | 1, // llvm.x86.clui
    5 << 8 | 1, // llvm.x86.clwb
    5 << 8 | 1, // llvm.x86.clzero
    11 << 8 | 35, // llvm.x86.cmpccxadd32
    11 << 8 | 35, // llvm.x86.cmpccxadd64
    5 << 8 | 1, // llvm.x86.directstore32
    5 << 8 | 1, // llvm.x86.directstore64
    5 << 8 | 1, // llvm.x86.encodekey128
    5 << 8 | 1, // llvm.x86.encodekey256
    5 << 8 | 1, // llvm.x86.enqcmd
    5 << 8 | 1, // llvm.x86.enqcmds
    5 << 8 | 1, // llvm.x86.flags.read.u32
    5 << 8 | 1, // llvm.x86.flags.read.u64
    5 << 8 | 1, // llvm.x86.flags.write.u32
    5 << 8 | 1, // llvm.x86.flags.write.u64
    1 << 8 | 1, // llvm.x86.fma.vfmaddsub.pd
    1 << 8 | 1, // llvm.x86.fma.vfmaddsub.pd.256
    1 << 8 | 1, // llvm.x86.fma.vfmaddsub.ps
    1 << 8 | 1, // llvm.x86.fma.vfmaddsub.ps.256
    5 << 8 | 1, // llvm.x86.fxrstor
    5 << 8 | 1, // llvm.x86.fxrstor64
    5 << 8 | 1, // llvm.x86.fxsave
    5 << 8 | 1, // llvm.x86.fxsave64
    5 << 8 | 1, // llvm.x86.incsspd
    5 << 8 | 1, // llvm.x86.incsspq
    5 << 8 | 5, // llvm.x86.int
    5 << 8 | 1, // llvm.x86.invpcid
    5 << 8 | 1, // llvm.x86.ldtilecfg
    5 << 8 | 1, // llvm.x86.ldtilecfg.internal
    5 << 8 | 1, // llvm.x86.llwpcb
    5 << 8 | 1, // llvm.x86.loadiwkey
    5 << 8 | 11, // llvm.x86.lwpins32
    5 << 8 | 11, // llvm.x86.lwpins64
    5 << 8 | 11, // llvm.x86.lwpval32
    5 << 8 | 11, // llvm.x86.lwpval64
    5 << 8 | 1, // llvm.x86.mmx.emms
    5 << 8 | 1, // llvm.x86.mmx.maskmovq
    5 << 8 | 1, // llvm.x86.mmx.movnt.dq
    1 << 8 | 1, // llvm.x86.mmx.packssdw
    1 << 8 | 1, // llvm.x86.mmx.packsswb
    1 << 8 | 1, // llvm.x86.mmx.packuswb
    1 << 8 | 1, // llvm.x86.mmx.padd.b
    1 << 8 | 1, // llvm.x86.mmx.padd.d
    1 << 8 | 1, // llvm.x86.mmx.padd.q
    1 << 8 | 1, // llvm.x86.mmx.padd.w
    1 << 8 | 1, // llvm.x86.mmx.padds.b
    1 << 8 | 1, // llvm.x86.mmx.padds.w
    1 << 8 | 1, // llvm.x86.mmx.paddus.b
    1 << 8 | 1, // llvm.x86.mmx.paddus.w
    1 << 8 | 11, // llvm.x86.mmx.palignr.b
    1 << 8 | 1, // llvm.x86.mmx.pand
    1 << 8 | 1, // llvm.x86.mmx.pandn
    1 << 8 | 1, // llvm.x86.mmx.pavg.b
    1 << 8 | 1, // llvm.x86.mmx.pavg.w
    1 << 8 | 1, // llvm.x86.mmx.pcmpeq.b
    1 << 8 | 1, // llvm.x86.mmx.pcmpeq.d
    1 << 8 | 1, // llvm.x86.mmx.pcmpeq.w
    1 << 8 | 1, // llvm.x86.mmx.pcmpgt.b
    1 << 8 | 1, // llvm.x86.mmx.pcmpgt.d
    1 << 8 | 1, // llvm.x86.mmx.pcmpgt.w
    1 << 8 | 0, // llvm.x86.mmx.pextr.w
    1 << 8 | 11, // llvm.x86.mmx.pinsr.w
    1 << 8 | 1, // llvm.x86.mmx.pmadd.wd
    1 << 8 | 1, // llvm.x86.mmx.pmaxs.w
    1 << 8 | 1, // llvm.x86.mmx.pmaxu.b
    1 << 8 | 1, // llvm.x86.mmx.pmins.w
    1 << 8 | 1, // llvm.x86.mmx.pminu.b
    1 << 8 | 1, // llvm.x86.mmx.pmovmskb
    1 << 8 | 1, // llvm.x86.mmx.pmulh.w
    1 << 8 | 1, // llvm.x86.mmx.pmulhu.w
    1 << 8 | 1, // llvm.x86.mmx.pmull.w
    1 << 8 | 1, // llvm.x86.mmx.pmulu.dq
    1 << 8 | 1, // llvm.x86.mmx.por
    1 << 8 | 1, // llvm.x86.mmx.psad.bw
    1 << 8 | 1, // llvm.x86.mmx.psll.d
    1 << 8 | 1, // llvm.x86.mmx.psll.q
    1 << 8 | 1, // llvm.x86.mmx.psll.w
    1 << 8 | 1, // llvm.x86.mmx.pslli.d
    1 << 8 | 1, // llvm.x86.mmx.pslli.q
    1 << 8 | 1, // llvm.x86.mmx.pslli.w
    1 << 8 | 1, // llvm.x86.mmx.psra.d
    1 << 8 | 1, // llvm.x86.mmx.psra.w
    1 << 8 | 1, // llvm.x86.mmx.psrai.d
    1 << 8 | 1, // llvm.x86.mmx.psrai.w
    1 << 8 | 1, // llvm.x86.mmx.psrl.d
    1 << 8 | 1, // llvm.x86.mmx.psrl.q
    1 << 8 | 1, // llvm.x86.mmx.psrl.w
    1 << 8 | 1, // llvm.x86.mmx.psrli.d
    1 << 8 | 1, // llvm.x86.mmx.psrli.q
    1 << 8 | 1, // llvm.x86.mmx.psrli.w
    1 << 8 | 1, // llvm.x86.mmx.psub.b
    1 << 8 | 1, // llvm.x86.mmx.psub.d
    1 << 8 | 1, // llvm.x86.mmx.psub.q
    1 << 8 | 1, // llvm.x86.mmx.psub.w
    1 << 8 | 1, // llvm.x86.mmx.psubs.b
    1 << 8 | 1, // llvm.x86.mmx.psubs.w
    1 << 8 | 1, // llvm.x86.mmx.psubus.b
    1 << 8 | 1, // llvm.x86.mmx.psubus.w
    1 << 8 | 1, // llvm.x86.mmx.punpckhbw
    1 << 8 | 1, // llvm.x86.mmx.punpckhdq
    1 << 8 | 1, // llvm.x86.mmx.punpckhwd
    1 << 8 | 1, // llvm.x86.mmx.punpcklbw
    1 << 8 | 1, // llvm.x86.mmx.punpckldq
    1 << 8 | 1, // llvm.x86.mmx.punpcklwd
    1 << 8 | 1, // llvm.x86.mmx.pxor
    5 << 8 | 1, // llvm.x86.monitorx
    5 << 8 | 1, // llvm.x86.movdir64b
    80 << 8 | 1, // llvm.x86.movrsdi
    80 << 8 | 1, // llvm.x86.movrshi
    80 << 8 | 1, // llvm.x86.movrsqi
    80 << 8 | 1, // llvm.x86.movrssi
    5 << 8 | 1, // llvm.x86.mwaitx
    1 << 8 | 11, // llvm.x86.pclmulqdq
    1 << 8 | 11, // llvm.x86.pclmulqdq.256
    1 << 8 | 11, // llvm.x86.pclmulqdq.512
    5 << 8 | 1, // llvm.x86.prefetchrs
    5 << 8 | 1, // llvm.x86.ptwrite32
    5 << 8 | 1, // llvm.x86.ptwrite64
    5 << 8 | 1, // llvm.x86.rdfsbase.32
    5 << 8 | 1, // llvm.x86.rdfsbase.64
    5 << 8 | 1, // llvm.x86.rdgsbase.32
    5 << 8 | 1, // llvm.x86.rdgsbase.64
    5 << 8 | 1, // llvm.x86.rdpid
    5 << 8 | 1, // llvm.x86.rdpkru
    5 << 8 | 1, // llvm.x86.rdpmc
    5 << 8 | 1, // llvm.x86.rdpru
    5 << 8 | 1, // llvm.x86.rdrand.16
    5 << 8 | 1, // llvm.x86.rdrand.32
    5 << 8 | 1, // llvm.x86.rdrand.64
    5 << 8 | 1, // llvm.x86.rdseed.16
    5 << 8 | 1, // llvm.x86.rdseed.32
    5 << 8 | 1, // llvm.x86.rdseed.64
    5 << 8 | 1, // llvm.x86.rdsspd
    5 << 8 | 1, // llvm.x86.rdsspq
    5 << 8 | 1, // llvm.x86.rdtsc
    5 << 8 | 1, // llvm.x86.rdtscp
    5 << 8 | 1, // llvm.x86.rstorssp
    5 << 8 | 1, // llvm.x86.saveprevssp
    5 << 8 | 1, // llvm.x86.seh.ehguard
    5 << 8 | 1, // llvm.x86.seh.ehregnode
    10 << 8 | 1, // llvm.x86.seh.lsda
    5 << 8 | 1, // llvm.x86.senduipi
    5 << 8 | 1, // llvm.x86.serialize
    5 << 8 | 1, // llvm.x86.setssbsy
    1 << 8 | 1, // llvm.x86.sha1msg1
    1 << 8 | 1, // llvm.x86.sha1msg2
    1 << 8 | 1, // llvm.x86.sha1nexte
    1 << 8 | 11, // llvm.x86.sha1rnds4
    1 << 8 | 1, // llvm.x86.sha256msg1
    1 << 8 | 1, // llvm.x86.sha256msg2
    1 << 8 | 1, // llvm.x86.sha256rnds2
    5 << 8 | 1, // llvm.x86.slwpcb
    1 << 8 | 11, // llvm.x86.sse.cmp.ps
    1 << 8 | 11, // llvm.x86.sse.cmp.ss
    1 << 8 | 1, // llvm.x86.sse.comieq.ss
    1 << 8 | 1, // llvm.x86.sse.comige.ss
    1 << 8 | 1, // llvm.x86.sse.comigt.ss
    1 << 8 | 1, // llvm.x86.sse.comile.ss
    1 << 8 | 1, // llvm.x86.sse.comilt.ss
    1 << 8 | 1, // llvm.x86.sse.comineq.ss
    1 << 8 | 1, // llvm.x86.sse.cvtpd2pi
    1 << 8 | 1, // llvm.x86.sse.cvtpi2pd
    1 << 8 | 1, // llvm.x86.sse.cvtpi2ps
    1 << 8 | 1, // llvm.x86.sse.cvtps2pi
    1 << 8 | 1, // llvm.x86.sse.cvtss2si
    1 << 8 | 1, // llvm.x86.sse.cvtss2si64
    1 << 8 | 1, // llvm.x86.sse.cvttpd2pi
    1 << 8 | 1, // llvm.x86.sse.cvttps2pi
    1 << 8 | 1, // llvm.x86.sse.cvttss2si
    1 << 8 | 1, // llvm.x86.sse.cvttss2si64
    29 << 8 | 1, // llvm.x86.sse.ldmxcsr
    1 << 8 | 1, // llvm.x86.sse.max.ps
    1 << 8 | 1, // llvm.x86.sse.max.ss
    1 << 8 | 1, // llvm.x86.sse.min.ps
    1 << 8 | 1, // llvm.x86.sse.min.ss
    1 << 8 | 1, // llvm.x86.sse.movmsk.ps
    1 << 8 | 0, // llvm.x86.sse.pshuf.w
    1 << 8 | 1, // llvm.x86.sse.rcp.ps
    1 << 8 | 1, // llvm.x86.sse.rcp.ss
    1 << 8 | 1, // llvm.x86.sse.rsqrt.ps
    1 << 8 | 1, // llvm.x86.sse.rsqrt.ss
    5 << 8 | 1, // llvm.x86.sse.sfence
    115 << 8 | 1, // llvm.x86.sse.stmxcsr
    1 << 8 | 1, // llvm.x86.sse.ucomieq.ss
    1 << 8 | 1, // llvm.x86.sse.ucomige.ss
    1 << 8 | 1, // llvm.x86.sse.ucomigt.ss
    1 << 8 | 1, // llvm.x86.sse.ucomile.ss
    1 << 8 | 1, // llvm.x86.sse.ucomilt.ss
    1 << 8 | 1, // llvm.x86.sse.ucomineq.ss
    5 << 8 | 1, // llvm.x86.sse2.clflush
    1 << 8 | 11, // llvm.x86.sse2.cmp.pd
    1 << 8 | 11, // llvm.x86.sse2.cmp.sd
    1 << 8 | 1, // llvm.x86.sse2.comieq.sd
    1 << 8 | 1, // llvm.x86.sse2.comige.sd
    1 << 8 | 1, // llvm.x86.sse2.comigt.sd
    1 << 8 | 1, // llvm.x86.sse2.comile.sd
    1 << 8 | 1, // llvm.x86.sse2.comilt.sd
    1 << 8 | 1, // llvm.x86.sse2.comineq.sd
    1 << 8 | 1, // llvm.x86.sse2.cvtpd2dq
    1 << 8 | 1, // llvm.x86.sse2.cvtpd2ps
    1 << 8 | 1, // llvm.x86.sse2.cvtps2dq
    1 << 8 | 1, // llvm.x86.sse2.cvtsd2si
    1 << 8 | 1, // llvm.x86.sse2.cvtsd2si64
    1 << 8 | 1, // llvm.x86.sse2.cvtsd2ss
    1 << 8 | 1, // llvm.x86.sse2.cvttpd2dq
    1 << 8 | 1, // llvm.x86.sse2.cvttps2dq
    1 << 8 | 1, // llvm.x86.sse2.cvttsd2si
    1 << 8 | 1, // llvm.x86.sse2.cvttsd2si64
    5 << 8 | 1, // llvm.x86.sse2.lfence
    5 << 8 | 1, // llvm.x86.sse2.maskmov.dqu
    1 << 8 | 1, // llvm.x86.sse2.max.pd
    1 << 8 | 1, // llvm.x86.sse2.max.sd
    5 << 8 | 1, // llvm.x86.sse2.mfence
    1 << 8 | 1, // llvm.x86.sse2.min.pd
    1 << 8 | 1, // llvm.x86.sse2.min.sd
    1 << 8 | 1, // llvm.x86.sse2.movmsk.pd
    1 << 8 | 1, // llvm.x86.sse2.packssdw.128
    1 << 8 | 1, // llvm.x86.sse2.packsswb.128
    1 << 8 | 1, // llvm.x86.sse2.packuswb.128
    5 << 8 | 1, // llvm.x86.sse2.pause
    1 << 8 | 1, // llvm.x86.sse2.pavg.b
    1 << 8 | 1, // llvm.x86.sse2.pavg.w
    1 << 8 | 1, // llvm.x86.sse2.pmadd.wd
    1 << 8 | 1, // llvm.x86.sse2.pmovmskb.128
    1 << 8 | 1, // llvm.x86.sse2.pmulh.w
    1 << 8 | 1, // llvm.x86.sse2.pmulhu.w
    1 << 8 | 1, // llvm.x86.sse2.psad.bw
    1 << 8 | 1, // llvm.x86.sse2.psll.d
    1 << 8 | 1, // llvm.x86.sse2.psll.q
    1 << 8 | 1, // llvm.x86.sse2.psll.w
    1 << 8 | 1, // llvm.x86.sse2.pslli.d
    1 << 8 | 1, // llvm.x86.sse2.pslli.q
    1 << 8 | 1, // llvm.x86.sse2.pslli.w
    1 << 8 | 1, // llvm.x86.sse2.psra.d
    1 << 8 | 1, // llvm.x86.sse2.psra.w
    1 << 8 | 1, // llvm.x86.sse2.psrai.d
    1 << 8 | 1, // llvm.x86.sse2.psrai.w
    1 << 8 | 1, // llvm.x86.sse2.psrl.d
    1 << 8 | 1, // llvm.x86.sse2.psrl.q
    1 << 8 | 1, // llvm.x86.sse2.psrl.w
    1 << 8 | 1, // llvm.x86.sse2.psrli.d
    1 << 8 | 1, // llvm.x86.sse2.psrli.q
    1 << 8 | 1, // llvm.x86.sse2.psrli.w
    1 << 8 | 1, // llvm.x86.sse2.ucomieq.sd
    1 << 8 | 1, // llvm.x86.sse2.ucomige.sd
    1 << 8 | 1, // llvm.x86.sse2.ucomigt.sd
    1 << 8 | 1, // llvm.x86.sse2.ucomile.sd
    1 << 8 | 1, // llvm.x86.sse2.ucomilt.sd
    1 << 8 | 1, // llvm.x86.sse2.ucomineq.sd
    1 << 8 | 1, // llvm.x86.sse3.addsub.pd
    1 << 8 | 1, // llvm.x86.sse3.addsub.ps
    1 << 8 | 1, // llvm.x86.sse3.hadd.pd
    1 << 8 | 1, // llvm.x86.sse3.hadd.ps
    1 << 8 | 1, // llvm.x86.sse3.hsub.pd
    1 << 8 | 1, // llvm.x86.sse3.hsub.ps
    23 << 8 | 1, // llvm.x86.sse3.ldu.dq
    5 << 8 | 1, // llvm.x86.sse3.monitor
    5 << 8 | 1, // llvm.x86.sse3.mwait
    1 << 8 | 1, // llvm.x86.sse41.blendvpd
    1 << 8 | 1, // llvm.x86.sse41.blendvps
    1 << 8 | 11, // llvm.x86.sse41.dppd
    1 << 8 | 11, // llvm.x86.sse41.dpps
    1 << 8 | 11, // llvm.x86.sse41.insertps
    1 << 8 | 11, // llvm.x86.sse41.mpsadbw
    1 << 8 | 1, // llvm.x86.sse41.packusdw
    1 << 8 | 1, // llvm.x86.sse41.pblendvb
    1 << 8 | 1, // llvm.x86.sse41.phminposuw
    1 << 8 | 1, // llvm.x86.sse41.ptestc
    1 << 8 | 1, // llvm.x86.sse41.ptestnzc
    1 << 8 | 1, // llvm.x86.sse41.ptestz
    1 << 8 | 0, // llvm.x86.sse41.round.pd
    1 << 8 | 0, // llvm.x86.sse41.round.ps
    1 << 8 | 11, // llvm.x86.sse41.round.sd
    1 << 8 | 11, // llvm.x86.sse41.round.ss
    1 << 8 | 1, // llvm.x86.sse42.crc32.32.16
    1 << 8 | 1, // llvm.x86.sse42.crc32.32.32
    1 << 8 | 1, // llvm.x86.sse42.crc32.32.8
    1 << 8 | 1, // llvm.x86.sse42.crc32.64.64
    1 << 8 | 43, // llvm.x86.sse42.pcmpestri128
    1 << 8 | 43, // llvm.x86.sse42.pcmpestria128
    1 << 8 | 43, // llvm.x86.sse42.pcmpestric128
    1 << 8 | 43, // llvm.x86.sse42.pcmpestrio128
    1 << 8 | 43, // llvm.x86.sse42.pcmpestris128
    1 << 8 | 43, // llvm.x86.sse42.pcmpestriz128
    1 << 8 | 43, // llvm.x86.sse42.pcmpestrm128
    1 << 8 | 11, // llvm.x86.sse42.pcmpistri128
    1 << 8 | 11, // llvm.x86.sse42.pcmpistria128
    1 << 8 | 11, // llvm.x86.sse42.pcmpistric128
    1 << 8 | 11, // llvm.x86.sse42.pcmpistrio128
    1 << 8 | 11, // llvm.x86.sse42.pcmpistris128
    1 << 8 | 11, // llvm.x86.sse42.pcmpistriz128
    1 << 8 | 11, // llvm.x86.sse42.pcmpistrm128
    1 << 8 | 1, // llvm.x86.sse4a.extrq
    1 << 8 | 13, // llvm.x86.sse4a.extrqi
    1 << 8 | 1, // llvm.x86.sse4a.insertq
    1 << 8 | 93, // llvm.x86.sse4a.insertqi
    1 << 8 | 1, // llvm.x86.ssse3.pabs.b
    1 << 8 | 1, // llvm.x86.ssse3.pabs.d
    1 << 8 | 1, // llvm.x86.ssse3.pabs.w
    1 << 8 | 1, // llvm.x86.ssse3.phadd.d
    1 << 8 | 1, // llvm.x86.ssse3.phadd.d.128
    1 << 8 | 1, // llvm.x86.ssse3.phadd.sw
    1 << 8 | 1, // llvm.x86.ssse3.phadd.sw.128
    1 << 8 | 1, // llvm.x86.ssse3.phadd.w
    1 << 8 | 1, // llvm.x86.ssse3.phadd.w.128
    1 << 8 | 1, // llvm.x86.ssse3.phsub.d
    1 << 8 | 1, // llvm.x86.ssse3.phsub.d.128
    1 << 8 | 1, // llvm.x86.ssse3.phsub.sw
    1 << 8 | 1, // llvm.x86.ssse3.phsub.sw.128
    1 << 8 | 1, // llvm.x86.ssse3.phsub.w
    1 << 8 | 1, // llvm.x86.ssse3.phsub.w.128
    1 << 8 | 1, // llvm.x86.ssse3.pmadd.ub.sw
    1 << 8 | 1, // llvm.x86.ssse3.pmadd.ub.sw.128
    1 << 8 | 1, // llvm.x86.ssse3.pmul.hr.sw
    1 << 8 | 1, // llvm.x86.ssse3.pmul.hr.sw.128
    1 << 8 | 1, // llvm.x86.ssse3.pshuf.b
    1 << 8 | 1, // llvm.x86.ssse3.pshuf.b.128
    1 << 8 | 1, // llvm.x86.ssse3.psign.b
    1 << 8 | 1, // llvm.x86.ssse3.psign.b.128
    1 << 8 | 1, // llvm.x86.ssse3.psign.d
    1 << 8 | 1, // llvm.x86.ssse3.psign.d.128
    1 << 8 | 1, // llvm.x86.ssse3.psign.w
    1 << 8 | 1, // llvm.x86.ssse3.psign.w.128
    5 << 8 | 1, // llvm.x86.sttilecfg
    5 << 8 | 1, // llvm.x86.stui
    1 << 8 | 1, // llvm.x86.subborrow.32
    1 << 8 | 1, // llvm.x86.subborrow.64
    1 << 8 | 0, // llvm.x86.tbm.bextri.u32
    1 << 8 | 0, // llvm.x86.tbm.bextri.u64
    5 << 8 | 92, // llvm.x86.tcmmimfp16ps
    5 << 8 | 1, // llvm.x86.tcmmimfp16ps.internal
    5 << 8 | 92, // llvm.x86.tcmmrlfp16ps
    5 << 8 | 1, // llvm.x86.tcmmrlfp16ps.internal
    5 << 8 | 5, // llvm.x86.tcvtrowd2ps
    5 << 8 | 1, // llvm.x86.tcvtrowd2ps.internal
    5 << 8 | 17, // llvm.x86.tcvtrowd2psi
    5 << 8 | 5, // llvm.x86.tcvtrowps2bf16h
    5 << 8 | 1, // llvm.x86.tcvtrowps2bf16h.internal
    5 << 8 | 17, // llvm.x86.tcvtrowps2bf16hi
    5 << 8 | 5, // llvm.x86.tcvtrowps2bf16l
    5 << 8 | 1, // llvm.x86.tcvtrowps2bf16l.internal
    5 << 8 | 17, // llvm.x86.tcvtrowps2bf16li
    5 << 8 | 5, // llvm.x86.tcvtrowps2phh
    5 << 8 | 1, // llvm.x86.tcvtrowps2phh.internal
    5 << 8 | 17, // llvm.x86.tcvtrowps2phhi
    5 << 8 | 5, // llvm.x86.tcvtrowps2phl
    5 << 8 | 1, // llvm.x86.tcvtrowps2phl.internal
    5 << 8 | 17, // llvm.x86.tcvtrowps2phli
    5 << 8 | 92, // llvm.x86.tdpbf16ps
    5 << 8 | 1, // llvm.x86.tdpbf16ps.internal
    5 << 8 | 92, // llvm.x86.tdpbf8ps
    5 << 8 | 1, // llvm.x86.tdpbf8ps.internal
    5 << 8 | 92, // llvm.x86.tdpbhf8ps
    5 << 8 | 1, // llvm.x86.tdpbhf8ps.internal
    5 << 8 | 92, // llvm.x86.tdpbssd
    5 << 8 | 1, // llvm.x86.tdpbssd.internal
    5 << 8 | 92, // llvm.x86.tdpbsud
    5 << 8 | 1, // llvm.x86.tdpbsud.internal
    5 << 8 | 92, // llvm.x86.tdpbusd
    5 << 8 | 1, // llvm.x86.tdpbusd.internal
    5 << 8 | 92, // llvm.x86.tdpbuud
    5 << 8 | 1, // llvm.x86.tdpbuud.internal
    5 << 8 | 92, // llvm.x86.tdpfp16ps
    5 << 8 | 1, // llvm.x86.tdpfp16ps.internal
    5 << 8 | 92, // llvm.x86.tdphbf8ps
    5 << 8 | 1, // llvm.x86.tdphbf8ps.internal
    5 << 8 | 92, // llvm.x86.tdphf8ps
    5 << 8 | 1, // llvm.x86.tdphf8ps.internal
    5 << 8 | 1, // llvm.x86.testui
    5 << 8 | 5, // llvm.x86.tileloadd64
    5 << 8 | 1, // llvm.x86.tileloadd64.internal
    5 << 8 | 5, // llvm.x86.tileloaddrs64
    5 << 8 | 1, // llvm.x86.tileloaddrs64.internal
    5 << 8 | 5, // llvm.x86.tileloaddrst164
    5 << 8 | 1, // llvm.x86.tileloaddrst164.internal
    5 << 8 | 5, // llvm.x86.tileloaddt164
    5 << 8 | 1, // llvm.x86.tileloaddt164.internal
    5 << 8 | 5, // llvm.x86.tilemovrow
    5 << 8 | 1, // llvm.x86.tilemovrow.internal
    5 << 8 | 17, // llvm.x86.tilemovrowi
    5 << 8 | 1, // llvm.x86.tilerelease
    5 << 8 | 5, // llvm.x86.tilestored64
    5 << 8 | 1, // llvm.x86.tilestored64.internal
    5 << 8 | 5, // llvm.x86.tilezero
    5 << 8 | 1, // llvm.x86.tilezero.internal
    5 << 8 | 92, // llvm.x86.tmmultf32ps
    5 << 8 | 1, // llvm.x86.tmmultf32ps.internal
    5 << 8 | 1, // llvm.x86.tpause
    5 << 8 | 1, // llvm.x86.umonitor
    5 << 8 | 1, // llvm.x86.umwait
    116 << 8 | 1, // llvm.x86.urdmsr
    116 << 8 | 1, // llvm.x86.uwrmsr
    2 << 8 | 1, // llvm.x86.vbcstnebf162ps128
    2 << 8 | 1, // llvm.x86.vbcstnebf162ps256
    2 << 8 | 1, // llvm.x86.vbcstnesh2ps128
    2 << 8 | 1, // llvm.x86.vbcstnesh2ps256
    2 << 8 | 1, // llvm.x86.vcvtneebf162ps128
    2 << 8 | 1, // llvm.x86.vcvtneebf162ps256
    2 << 8 | 1, // llvm.x86.vcvtneeph2ps128
    2 << 8 | 1, // llvm.x86.vcvtneeph2ps256
    2 << 8 | 1, // llvm.x86.vcvtneobf162ps128
    2 << 8 | 1, // llvm.x86.vcvtneobf162ps256
    2 << 8 | 1, // llvm.x86.vcvtneoph2ps128
    2 << 8 | 1, // llvm.x86.vcvtneoph2ps256
    1 << 8 | 1, // llvm.x86.vcvtneps2bf16128
    1 << 8 | 1, // llvm.x86.vcvtneps2bf16256
    1 << 8 | 0, // llvm.x86.vcvtps2ph.128
    1 << 8 | 0, // llvm.x86.vcvtps2ph.256
    1 << 8 | 11, // llvm.x86.vgf2p8affineinvqb.128
    1 << 8 | 11, // llvm.x86.vgf2p8affineinvqb.256
    1 << 8 | 11, // llvm.x86.vgf2p8affineinvqb.512
    1 << 8 | 11, // llvm.x86.vgf2p8affineqb.128
    1 << 8 | 11, // llvm.x86.vgf2p8affineqb.256
    1 << 8 | 11, // llvm.x86.vgf2p8affineqb.512
    1 << 8 | 1, // llvm.x86.vgf2p8mulb.128
    1 << 8 | 1, // llvm.x86.vgf2p8mulb.256
    1 << 8 | 1, // llvm.x86.vgf2p8mulb.512
    1 << 8 | 1, // llvm.x86.vsha512msg1
    1 << 8 | 1, // llvm.x86.vsha512msg2
    1 << 8 | 1, // llvm.x86.vsha512rnds2
    1 << 8 | 1, // llvm.x86.vsm3msg1
    1 << 8 | 1, // llvm.x86.vsm3msg2
    1 << 8 | 35, // llvm.x86.vsm3rnds2
    1 << 8 | 1, // llvm.x86.vsm4key4128
    1 << 8 | 1, // llvm.x86.vsm4key4256
    1 << 8 | 1, // llvm.x86.vsm4key4512
    1 << 8 | 1, // llvm.x86.vsm4rnds4128
    1 << 8 | 1, // llvm.x86.vsm4rnds4256
    1 << 8 | 1, // llvm.x86.vsm4rnds4512
    5 << 8 | 1, // llvm.x86.wbinvd
    5 << 8 | 1, // llvm.x86.wbnoinvd
    5 << 8 | 1, // llvm.x86.wrfsbase.32
    5 << 8 | 1, // llvm.x86.wrfsbase.64
    5 << 8 | 1, // llvm.x86.wrgsbase.32
    5 << 8 | 1, // llvm.x86.wrgsbase.64
    5 << 8 | 1, // llvm.x86.wrpkru
    5 << 8 | 1, // llvm.x86.wrssd
    5 << 8 | 1, // llvm.x86.wrssq
    5 << 8 | 1, // llvm.x86.wrussd
    5 << 8 | 1, // llvm.x86.wrussq
    5 << 8 | 5, // llvm.x86.xabort
    5 << 8 | 1, // llvm.x86.xbegin
    5 << 8 | 1, // llvm.x86.xend
    5 << 8 | 1, // llvm.x86.xgetbv
    1 << 8 | 1, // llvm.x86.xop.vfrcz.pd
    1 << 8 | 1, // llvm.x86.xop.vfrcz.pd.256
    1 << 8 | 1, // llvm.x86.xop.vfrcz.ps
    1 << 8 | 1, // llvm.x86.xop.vfrcz.ps.256
    1 << 8 | 1, // llvm.x86.xop.vfrcz.sd
    1 << 8 | 1, // llvm.x86.xop.vfrcz.ss
    1 << 8 | 35, // llvm.x86.xop.vpermil2pd
    1 << 8 | 35, // llvm.x86.xop.vpermil2pd.256
    1 << 8 | 35, // llvm.x86.xop.vpermil2ps
    1 << 8 | 35, // llvm.x86.xop.vpermil2ps.256
    1 << 8 | 1, // llvm.x86.xop.vphaddbd
    1 << 8 | 1, // llvm.x86.xop.vphaddbq
    1 << 8 | 1, // llvm.x86.xop.vphaddbw
    1 << 8 | 1, // llvm.x86.xop.vphadddq
    1 << 8 | 1, // llvm.x86.xop.vphaddubd
    1 << 8 | 1, // llvm.x86.xop.vphaddubq
    1 << 8 | 1, // llvm.x86.xop.vphaddubw
    1 << 8 | 1, // llvm.x86.xop.vphaddudq
    1 << 8 | 1, // llvm.x86.xop.vphadduwd
    1 << 8 | 1, // llvm.x86.xop.vphadduwq
    1 << 8 | 1, // llvm.x86.xop.vphaddwd
    1 << 8 | 1, // llvm.x86.xop.vphaddwq
    1 << 8 | 1, // llvm.x86.xop.vphsubbw
    1 << 8 | 1, // llvm.x86.xop.vphsubdq
    1 << 8 | 1, // llvm.x86.xop.vphsubwd
    1 << 8 | 1, // llvm.x86.xop.vpmacsdd
    1 << 8 | 1, // llvm.x86.xop.vpmacsdqh
    1 << 8 | 1, // llvm.x86.xop.vpmacsdql
    1 << 8 | 1, // llvm.x86.xop.vpmacssdd
    1 << 8 | 1, // llvm.x86.xop.vpmacssdqh
    1 << 8 | 1, // llvm.x86.xop.vpmacssdql
    1 << 8 | 1, // llvm.x86.xop.vpmacsswd
    1 << 8 | 1, // llvm.x86.xop.vpmacssww
    1 << 8 | 1, // llvm.x86.xop.vpmacswd
    1 << 8 | 1, // llvm.x86.xop.vpmacsww
    1 << 8 | 1, // llvm.x86.xop.vpmadcsswd
    1 << 8 | 1, // llvm.x86.xop.vpmadcswd
    1 << 8 | 1, // llvm.x86.xop.vpperm
    1 << 8 | 1, // llvm.x86.xop.vpshab
    1 << 8 | 1, // llvm.x86.xop.vpshad
    1 << 8 | 1, // llvm.x86.xop.vpshaq
    1 << 8 | 1, // llvm.x86.xop.vpshaw
    1 << 8 | 1, // llvm.x86.xop.vpshlb
    1 << 8 | 1, // llvm.x86.xop.vpshld
    1 << 8 | 1, // llvm.x86.xop.vpshlq
    1 << 8 | 1, // llvm.x86.xop.vpshlw
    5 << 8 | 1, // llvm.x86.xresldtrk
    5 << 8 | 1, // llvm.x86.xrstor
    5 << 8 | 1, // llvm.x86.xrstor64
    5 << 8 | 1, // llvm.x86.xrstors
    5 << 8 | 1, // llvm.x86.xrstors64
    5 << 8 | 1, // llvm.x86.xsave
    5 << 8 | 1, // llvm.x86.xsave64
    5 << 8 | 1, // llvm.x86.xsavec
    5 << 8 | 1, // llvm.x86.xsavec64
    5 << 8 | 1, // llvm.x86.xsaveopt
    5 << 8 | 1, // llvm.x86.xsaveopt64
    5 << 8 | 1, // llvm.x86.xsaves
    5 << 8 | 1, // llvm.x86.xsaves64
    5 << 8 | 1, // llvm.x86.xsetbv
    5 << 8 | 1, // llvm.x86.xsusldtrk
    5 << 8 | 1, // llvm.x86.xtest
    10 << 8 | 1, // llvm.xcore.bitrev
    5 << 8 | 1, // llvm.xcore.checkevent
    5 << 8 | 10, // llvm.xcore.chkct
    5 << 8 | 1, // llvm.xcore.clre
    5 << 8 | 10, // llvm.xcore.clrpt
    5 << 8 | 1, // llvm.xcore.clrsr
    10 << 8 | 1, // llvm.xcore.crc32
    10 << 8 | 1, // llvm.xcore.crc8
    5 << 8 | 10, // llvm.xcore.edu
    5 << 8 | 10, // llvm.xcore.eeu
    5 << 8 | 10, // llvm.xcore.endin
    5 << 8 | 10, // llvm.xcore.freer
    5 << 8 | 1, // llvm.xcore.geted
    5 << 8 | 1, // llvm.xcore.getet
    10 << 8 | 1, // llvm.xcore.getid
    5 << 8 | 1, // llvm.xcore.getps
    5 << 8 | 1, // llvm.xcore.getr
    5 << 8 | 10, // llvm.xcore.getst
    5 << 8 | 10, // llvm.xcore.getts
    5 << 8 | 10, // llvm.xcore.in
    5 << 8 | 10, // llvm.xcore.inct
    5 << 8 | 10, // llvm.xcore.initcp
    5 << 8 | 10, // llvm.xcore.initdp
    5 << 8 | 10, // llvm.xcore.initlr
    5 << 8 | 10, // llvm.xcore.initpc
    5 << 8 | 10, // llvm.xcore.initsp
    5 << 8 | 10, // llvm.xcore.inshr
    5 << 8 | 10, // llvm.xcore.int
    5 << 8 | 10, // llvm.xcore.mjoin
    5 << 8 | 10, // llvm.xcore.msync
    5 << 8 | 10, // llvm.xcore.out
    5 << 8 | 10, // llvm.xcore.outct
    5 << 8 | 10, // llvm.xcore.outshr
    5 << 8 | 10, // llvm.xcore.outt
    5 << 8 | 10, // llvm.xcore.peek
    5 << 8 | 10, // llvm.xcore.setc
    5 << 8 | 247, // llvm.xcore.setclk
    5 << 8 | 10, // llvm.xcore.setd
    5 << 8 | 10, // llvm.xcore.setev
    5 << 8 | 1, // llvm.xcore.setps
    5 << 8 | 10, // llvm.xcore.setpsc
    5 << 8 | 10, // llvm.xcore.setpt
    5 << 8 | 247, // llvm.xcore.setrdy
    5 << 8 | 1, // llvm.xcore.setsr
    5 << 8 | 10, // llvm.xcore.settw
    5 << 8 | 10, // llvm.xcore.setv
    10 << 8 | 1, // llvm.xcore.sext
    5 << 8 | 1, // llvm.xcore.ssync
    5 << 8 | 10, // llvm.xcore.syncr
    5 << 8 | 10, // llvm.xcore.testct
    5 << 8 | 10, // llvm.xcore.testwct
    80 << 8 | 1, // llvm.xcore.waitevent
    10 << 8 | 1, // llvm.xcore.zext
}; // IntrinsicsToAttributesMap

namespace {
struct ArgNoAttrIDPair {
  uint16_t ArgNo, ArgAttrID;
};
} // namespace

// Number of entries: 663
static constexpr ArgNoAttrIDPair ArgAttrIdTable[] = {
  /* 0 */ {0, 1}, 
  /* 1 */ {0, 11}, 
  /* 2 */ {0, 16}, 
  /* 3 */ {0, 18}, 
  /* 4 */ {0, 19}, 
  /* 5 */ {0, 20}, 
  /* 6 */ {0, 27}, 
  /* 7 */ {0, 28}, 
  /* 8 */ {0, 29}, 
  /* 9 */ {0, 30}, 
  /* 10 */ {0, 31}, 
  /* 11 */ {0, 32}, 
  /* 12 */ {0, 33}, 
  /* 13 */ {0, 34}, 
  /* 14 */ {0, 35}, 
  /* 15 */ {0, 36}, 
  /* 16 */ {0, 1}, {1, 0}, 
  /* 18 */ {0, 1}, {1, 1}, 
  /* 20 */ {1, 3}, 
  /* 21 */ {1, 4}, 
  /* 22 */ {1, 5}, 
  /* 23 */ {1, 6}, 
  /* 24 */ {1, 7}, 
  /* 25 */ {1, 10}, 
  /* 26 */ {0, 12}, {1, 12}, 
  /* 28 */ {1, 37}, 
  /* 29 */ {1, 0}, {2, 0}, 
  /* 31 */ {1, 3}, {2, 0}, 
  /* 33 */ {1, 5}, {2, 0}, 
  /* 35 */ {1, 17}, {2, 0}, 
  /* 37 */ {2, 2}, 
  /* 38 */ {1, 7}, {2, 3}, 
  /* 40 */ {1, 0}, {2, 5}, 
  /* 42 */ {1, 5}, {2, 5}, 
  /* 44 */ {1, 5}, {2, 14}, 
  /* 46 */ {1, 5}, {2, 15}, 
  /* 48 */ {1, 5}, {2, 21}, 
  /* 50 */ {1, 22}, {2, 23}, 
  /* 52 */ {2, 24}, 
  /* 53 */ {1, 5}, {2, 26}, 
  /* 55 */ {1, 5}, {2, 39}, 
  /* 57 */ {1, 0}, {3, 0}, 
  /* 59 */ {1, 5}, {3, 0}, 
  /* 61 */ {1, 7}, {3, 0}, 
  /* 63 */ {1, 0}, {2, 0}, {3, 0}, 
  /* 66 */ {1, 5}, {2, 0}, {3, 0}, 
  /* 69 */ {1, 13}, {2, 0}, {3, 0}, 
  /* 72 */ {2, 5}, {3, 0}, 
  /* 74 */ {2, 4}, {3, 3}, 
  /* 76 */ {1, 7}, {2, 4}, {3, 4}, 
  /* 79 */ {2, 0}, {3, 5}, 
  /* 81 */ {2, 5}, {3, 5}, 
  /* 83 */ {1, 2}, {3, 13}, 
  /* 85 */ {3, 14}, 
  /* 86 */ {3, 15}, 
  /* 87 */ {1, 0}, {4, 0}, 
  /* 89 */ {1, 3}, {4, 0}, 
  /* 91 */ {1, 5}, {4, 0}, 
  /* 93 */ {1, 7}, {4, 0}, 
  /* 95 */ {1, 13}, {4, 0}, 
  /* 97 */ {1, 0}, {2, 0}, {4, 0}, 
  /* 100 */ {1, 5}, {2, 0}, {4, 0}, 
  /* 103 */ {1, 7}, {2, 3}, {4, 0}, 
  /* 106 */ {2, 5}, {4, 0}, 
  /* 108 */ {1, 8}, {2, 9}, {4, 0}, 
  /* 111 */ {1, 3}, {2, 0}, {3, 0}, {4, 0}, 
  /* 115 */ {1, 7}, {2, 3}, {3, 0}, {4, 0}, 
  /* 119 */ {1, 3}, {2, 7}, {3, 0}, {4, 0}, 
  /* 123 */ {4, 5}, 
  /* 124 */ {4, 15}, 
  /* 125 */ {1, 0}, {5, 0}, 
  /* 127 */ {1, 3}, {5, 0}, 
  /* 129 */ {1, 5}, {5, 0}, 
  /* 131 */ {1, 13}, {5, 0}, 
  /* 133 */ {1, 0}, {2, 0}, {5, 0}, 
  /* 136 */ {2, 5}, {5, 0}, 
  /* 138 */ {2, 7}, {5, 0}, 
  /* 140 */ {1, 2}, {2, 13}, {5, 0}, 
  /* 143 */ {1, 13}, {2, 13}, {5, 0}, 
  /* 146 */ {1, 0}, {3, 0}, {5, 0}, 
  /* 149 */ {1, 0}, {4, 0}, {5, 0}, 
  /* 152 */ {1, 3}, {4, 0}, {5, 0}, 
  /* 155 */ {1, 0}, {2, 0}, {4, 0}, {5, 0}, 
  /* 159 */ {1, 5}, {3, 0}, {4, 0}, {5, 0}, 
  /* 163 */ {1, 0}, {2, 0}, {3, 0}, {4, 0}, {5, 0}, 
  /* 168 */ {1, 13}, {2, 0}, {3, 0}, {4, 0}, {5, 0}, 
  /* 173 */ {1, 5}, {2, 5}, {3, 0}, {4, 0}, {5, 0}, 
  /* 178 */ {1, 3}, {2, 7}, {3, 0}, {4, 0}, {5, 0}, 
  /* 183 */ {4, 15}, {5, 0}, 
  /* 185 */ {5, 5}, 
  /* 186 */ {5, 15}, 
  /* 187 */ {1, 0}, {6, 0}, 
  /* 189 */ {1, 13}, {6, 0}, 
  /* 191 */ {2, 5}, {6, 0}, 
  /* 193 */ {2, 7}, {6, 0}, 
  /* 195 */ {1, 13}, {2, 13}, {6, 0}, 
  /* 198 */ {1, 0}, {3, 0}, {6, 0}, 
  /* 201 */ {3, 5}, {6, 0}, 
  /* 203 */ {1, 2}, {3, 13}, {6, 0}, 
  /* 206 */ {4, 0}, {6, 0}, 
  /* 208 */ {1, 0}, {5, 0}, {6, 0}, 
  /* 211 */ {1, 3}, {5, 0}, {6, 0}, 
  /* 214 */ {1, 0}, {2, 0}, {5, 0}, {6, 0}, 
  /* 218 */ {2, 5}, {5, 0}, {6, 0}, 
  /* 221 */ {2, 7}, {5, 0}, {6, 0}, 
  /* 224 */ {1, 0}, {2, 0}, {3, 0}, {5, 0}, {6, 0}, 
  /* 229 */ {1, 0}, {2, 0}, {4, 0}, {5, 0}, {6, 0}, 
  /* 234 */ {2, 7}, {4, 0}, {5, 0}, {6, 0}, 
  /* 238 */ {1, 0}, {2, 0}, {3, 0}, {4, 0}, {5, 0}, {6, 0}, 
  /* 244 */ {1, 5}, {2, 0}, {3, 0}, {4, 0}, {5, 0}, {6, 0}, 
  /* 250 */ {6, 5}, 
  /* 251 */ {1, 0}, {7, 0}, 
  /* 253 */ {1, 13}, {7, 0}, 
  /* 255 */ {1, 0}, {2, 0}, {7, 0}, 
  /* 258 */ {1, 13}, {2, 13}, {7, 0}, 
  /* 261 */ {1, 0}, {3, 0}, {7, 0}, 
  /* 264 */ {3, 5}, {7, 0}, 
  /* 266 */ {1, 2}, {3, 13}, {7, 0}, 
  /* 269 */ {5, 0}, {7, 0}, 
  /* 271 */ {1, 0}, {6, 0}, {7, 0}, 
  /* 274 */ {2, 0}, {6, 0}, {7, 0}, 
  /* 277 */ {2, 5}, {6, 0}, {7, 0}, 
  /* 280 */ {2, 7}, {6, 0}, {7, 0}, 
  /* 283 */ {2, 5}, {3, 0}, {6, 0}, {7, 0}, 
  /* 287 */ {1, 3}, {2, 7}, {3, 0}, {6, 0}, {7, 0}, 
  /* 292 */ {1, 0}, {5, 0}, {6, 0}, {7, 0}, 
  /* 296 */ {2, 5}, {3, 0}, {4, 0}, {5, 0}, {6, 0}, {7, 0}, 
  /* 302 */ {7, 5}, 
  /* 303 */ {1, 13}, {8, 0}, 
  /* 305 */ {1, 13}, {2, 13}, {8, 0}, 
  /* 308 */ {1, 2}, {3, 13}, {8, 0}, 
  /* 311 */ {4, 0}, {5, 0}, {6, 0}, {8, 0}, 
  /* 315 */ {1, 0}, {7, 0}, {8, 0}, 
  /* 318 */ {1, 0}, {2, 0}, {7, 0}, {8, 0}, 
  /* 322 */ {1, 0}, {3, 0}, {7, 0}, {8, 0}, 
  /* 326 */ {2, 5}, {3, 0}, {7, 0}, {8, 0}, 
  /* 330 */ {1, 3}, {2, 7}, {3, 0}, {7, 0}, {8, 0}, 
  /* 335 */ {1, 2}, {3, 13}, {7, 0}, {8, 0}, 
  /* 339 */ {1, 0}, {3, 0}, {5, 0}, {7, 0}, {8, 0}, 
  /* 344 */ {1, 0}, {6, 0}, {7, 0}, {8, 0}, 
  /* 348 */ {1, 0}, {3, 0}, {6, 0}, {7, 0}, {8, 0}, 
  /* 353 */ {1, 5}, {3, 0}, {4, 0}, {5, 0}, {6, 0}, {7, 0}, {8, 0}, 
  /* 360 */ {8, 5}, 
  /* 361 */ {1, 2}, {7, 15}, {8, 15}, 
  /* 364 */ {1, 2}, {2, 13}, {7, 15}, {8, 15}, 
  /* 368 */ {1, 2}, {6, 15}, {7, 15}, {8, 15}, 
  /* 372 */ {1, 2}, {2, 13}, {6, 15}, {7, 15}, {8, 15}, 
  /* 377 */ {1, 2}, {6, 15}, {7, 38}, {8, 15}, 
  /* 381 */ {1, 2}, {2, 13}, {6, 15}, {7, 38}, {8, 15}, 
  /* 386 */ {1, 2}, {2, 13}, {7, 15}, {8, 26}, 
  /* 390 */ {1, 2}, {2, 13}, {6, 15}, {7, 38}, {8, 26}, 
  /* 395 */ {1, 13}, {9, 0}, 
  /* 397 */ {1, 13}, {2, 13}, {9, 0}, 
  /* 400 */ {1, 2}, {3, 13}, {9, 0}, 
  /* 403 */ {2, 0}, {8, 0}, {9, 0}, 
  /* 406 */ {1, 0}, {7, 0}, {8, 0}, {9, 0}, 
  /* 410 */ {1, 0}, {3, 0}, {7, 0}, {8, 0}, {9, 0}, 
  /* 415 */ {9, 5}, 
  /* 416 */ {1, 2}, {8, 15}, {9, 15}, 
  /* 419 */ {1, 2}, {2, 13}, {8, 15}, {9, 15}, 
  /* 423 */ {1, 2}, {7, 15}, {8, 15}, {9, 15}, 
  /* 427 */ {1, 2}, {2, 13}, {7, 15}, {8, 15}, {9, 15}, 
  /* 432 */ {1, 2}, {6, 14}, {8, 26}, {9, 15}, 
  /* 436 */ {1, 2}, {2, 13}, {6, 14}, {8, 26}, {9, 15}, 
  /* 441 */ {1, 2}, {8, 38}, {9, 15}, 
  /* 444 */ {1, 2}, {2, 13}, {8, 38}, {9, 15}, 
  /* 448 */ {1, 2}, {7, 15}, {8, 38}, {9, 15}, 
  /* 452 */ {1, 2}, {2, 13}, {7, 15}, {8, 38}, {9, 15}, 
  /* 457 */ {1, 2}, {6, 14}, {7, 26}, {8, 38}, {9, 15}, 
  /* 462 */ {1, 2}, {2, 13}, {6, 14}, {7, 26}, {8, 38}, {9, 15}, 
  /* 468 */ {1, 2}, {3, 13}, {7, 0}, {8, 0}, {9, 21}, 
  /* 473 */ {1, 2}, {2, 13}, {8, 15}, {9, 26}, 
  /* 477 */ {1, 2}, {2, 13}, {6, 14}, {8, 26}, {9, 26}, 
  /* 482 */ {1, 2}, {2, 13}, {7, 15}, {8, 38}, {9, 26}, 
  /* 487 */ {1, 2}, {2, 13}, {6, 14}, {7, 26}, {8, 38}, {9, 26}, 
  /* 493 */ {1, 13}, {10, 0}, 
  /* 495 */ {1, 2}, {3, 13}, {10, 0}, 
  /* 498 */ {1, 0}, {8, 0}, {9, 0}, {10, 0}, 
  /* 502 */ {1, 2}, {8, 15}, {9, 15}, {10, 15}, 
  /* 506 */ {1, 2}, {2, 13}, {8, 15}, {9, 15}, {10, 15}, 
  /* 511 */ {1, 2}, {7, 14}, {9, 26}, {10, 15}, 
  /* 515 */ {1, 2}, {2, 13}, {7, 14}, {9, 26}, {10, 15}, 
  /* 520 */ {1, 2}, {9, 38}, {10, 15}, 
  /* 523 */ {1, 2}, {2, 13}, {9, 38}, {10, 15}, 
  /* 527 */ {1, 2}, {7, 14}, {8, 26}, {9, 38}, {10, 15}, 
  /* 532 */ {1, 2}, {2, 13}, {7, 14}, {8, 26}, {9, 38}, {10, 15}, 
  /* 538 */ {1, 2}, {3, 13}, {8, 0}, {9, 0}, {10, 21}, 
  /* 543 */ {1, 2}, {2, 13}, {7, 14}, {9, 26}, {10, 26}, 
  /* 548 */ {1, 2}, {2, 13}, {7, 14}, {8, 26}, {9, 38}, {10, 26}, 
  /* 554 */ {1, 13}, {11, 0}, 
  /* 556 */ {1, 2}, {3, 13}, {11, 0}, 
  /* 559 */ {1, 0}, {9, 0}, {10, 0}, {11, 0}, 
  /* 563 */ {1, 2}, {3, 13}, {9, 0}, {10, 0}, {11, 21}, 
  /* 568 */ {1, 2}, {3, 13}, {12, 0}, 
  /* 571 */ {3, 0}, {5, 0}, {6, 0}, {8, 0}, {9, 0}, {11, 0}, {12, 0}, 
  /* 578 */ {1, 0}, {10, 0}, {11, 0}, {12, 0}, 
  /* 582 */ {1, 2}, {3, 13}, {10, 0}, {11, 0}, {12, 21}, 
  /* 587 */ {12, 25}, 
  /* 588 */ {12, 26}, 
  /* 589 */ {1, 2}, {3, 13}, {13, 0}, 
  /* 592 */ {1, 0}, {11, 0}, {12, 0}, {13, 0}, 
  /* 596 */ {1, 2}, {3, 13}, {11, 0}, {12, 0}, {13, 21}, 
  /* 601 */ {1, 0}, {3, 0}, {5, 0}, {7, 0}, {8, 0}, {10, 0}, {11, 0}, {13, 0}, {14, 0}, 
  /* 610 */ {1, 0}, {12, 0}, {13, 0}, {14, 0}, 
  /* 614 */ {1, 2}, {3, 13}, {12, 0}, {13, 0}, {14, 21}, 
  /* 619 */ {14, 25}, 
  /* 620 */ {14, 26}, 
  /* 621 */ {1, 0}, {13, 0}, {14, 0}, {15, 0}, 
  /* 625 */ {1, 2}, {3, 13}, {13, 0}, {14, 0}, {15, 21}, 
  /* 630 */ {1, 0}, {14, 0}, {15, 0}, {16, 0}, 
  /* 634 */ {1, 2}, {3, 13}, {14, 0}, {15, 0}, {16, 21}, 
  /* 639 */ {1, 0}, {15, 0}, {16, 0}, {17, 0}, 
  /* 643 */ {1, 0}, {16, 0}, {17, 0}, {18, 0}, 
  /* 647 */ {3, 0}, {4, 0}, {5, 0}, {6, 0}, {7, 0}, {8, 0}, {9, 0}, {10, 0}, {11, 0}, {12, 0}, {13, 0}, {14, 0}, {15, 0}, {16, 0}, {17, 0}, {18, 0}, 
}; // ArgAttrIdTable

namespace {
struct ArgAttributesInfo {
  uint16_t StartIndex;
  uint16_t NumAttrs;
};
} // namespace

// Number of entries: 248
static constexpr ArgAttributesInfo ArgAttributesInfoTable[] = {
  {30, 1},
  {0, 0},
  {0, 1},
  {16, 2},
  {19, 1},
  {17, 1},
  {37, 1},
  {20, 1},
  {39, 1},
  {74, 2},
  {22, 1},
  {58, 1},
  {23, 1},
  {64, 2},
  {155, 4},
  {93, 2},
  {97, 3},
  {29, 2},
  {41, 1},
  {81, 2},
  {76, 3},
  {79, 2},
  {40, 2},
  {159, 4},
  {234, 4},
  {160, 3},
  {108, 3},
  {103, 3},
  {25, 1},
  {112, 3},
  {111, 4},
  {21, 1},
  {98, 2},
  {1, 1},
  {26, 2},
  {88, 1},
  {24, 1},
  {80, 1},
  {123, 1},
  {185, 1},
  {250, 1},
  {168, 5},
  {69, 3},
  {126, 1},
  {252, 1},
  {187, 2},
  {57, 2},
  {188, 1},
  {150, 2},
  {106, 2},
  {119, 4},
  {31, 2},
  {61, 2},
  {85, 1},
  {183, 2},
  {86, 1},
  {124, 1},
  {186, 1},
  {2, 1},
  {35, 2},
  {353, 7},
  {318, 4},
  {214, 4},
  {255, 3},
  {33, 2},
  {173, 5},
  {115, 4},
  {209, 2},
  {272, 2},
  {316, 2},
  {404, 2},
  {344, 4},
  {406, 4},
  {498, 4},
  {559, 4},
  {578, 4},
  {592, 4},
  {292, 4},
  {149, 3},
  {208, 3},
  {271, 3},
  {315, 3},
  {610, 4},
  {630, 4},
  {621, 4},
  {639, 4},
  {643, 4},
  {215, 3},
  {274, 3},
  {319, 3},
  {403, 3},
  {3, 1},
  {63, 3},
  {113, 2},
  {178, 5},
  {231, 3},
  {311, 4},
  {164, 4},
  {283, 4},
  {91, 2},
  {136, 2},
  {201, 2},
  {89, 2},
  {287, 5},
  {138, 2},
  {152, 3},
  {221, 3},
  {326, 4},
  {129, 2},
  {191, 2},
  {264, 2},
  {127, 2},
  {330, 5},
  {193, 2},
  {211, 3},
  {280, 3},
  {198, 3},
  {322, 4},
  {410, 5},
  {261, 3},
  {240, 4},
  {4, 1},
  {339, 5},
  {226, 3},
  {146, 3},
  {348, 5},
  {601, 9},
  {571, 7},
  {5, 1},
  {87, 2},
  {125, 2},
  {238, 6},
  {229, 5},
  {133, 3},
  {163, 5},
  {244, 6},
  {296, 6},
  {72, 2},
  {38, 2},
  {335, 4},
  {203, 3},
  {83, 2},
  {140, 3},
  {400, 3},
  {556, 3},
  {589, 3},
  {495, 3},
  {568, 3},
  {266, 3},
  {308, 3},
  {582, 5},
  {614, 5},
  {634, 5},
  {596, 5},
  {625, 5},
  {468, 5},
  {538, 5},
  {563, 5},
  {253, 2},
  {395, 2},
  {554, 2},
  {303, 2},
  {493, 2},
  {95, 2},
  {131, 2},
  {189, 2},
  {258, 3},
  {305, 3},
  {397, 3},
  {143, 3},
  {195, 3},
  {50, 2},
  {52, 1},
  {18, 2},
  {619, 1},
  {505, 1},
  {363, 1},
  {620, 1},
  {588, 1},
  {547, 1},
  {587, 1},
  {6, 1},
  {7, 1},
  {8, 1},
  {9, 1},
  {10, 1},
  {11, 1},
  {12, 1},
  {13, 1},
  {14, 1},
  {15, 1},
  {28, 1},
  {66, 3},
  {377, 4},
  {361, 3},
  {441, 3},
  {457, 5},
  {432, 4},
  {448, 4},
  {416, 3},
  {520, 3},
  {527, 5},
  {511, 4},
  {452, 5},
  {482, 5},
  {419, 4},
  {473, 4},
  {523, 4},
  {532, 6},
  {548, 6},
  {515, 5},
  {543, 5},
  {381, 5},
  {390, 5},
  {364, 4},
  {386, 4},
  {444, 4},
  {462, 6},
  {487, 6},
  {436, 5},
  {477, 5},
  {368, 4},
  {423, 4},
  {502, 4},
  {427, 5},
  {506, 5},
  {372, 5},
  {59, 2},
  {100, 3},
  {44, 2},
  {53, 2},
  {48, 2},
  {46, 2},
  {55, 2},
  {269, 2},
  {302, 1},
  {360, 1},
  {415, 1},
  {251, 2},
  {224, 5},
  {147, 2},
  {206, 2},
  {277, 3},
  {218, 3},
  {647, 16},
  {199, 2},
  {134, 2},
  {42, 2},
}; // ArgAttributesInfoTable


template <typename IDTy>
inline std::pair<uint32_t, uint32_t> unpackID(const IDTy PackedID) {
  constexpr uint8_t UniqAttributesBitSize = 8;
  const uint32_t FnAttrID = PackedID >> UniqAttributesBitSize;
  const uint32_t ArgAttrID = PackedID &
    maskTrailingOnes<uint32_t>(UniqAttributesBitSize);
  return {FnAttrID, ArgAttrID};
}

AttributeList Intrinsic::getAttributes(LLVMContext &C, ID id,
                                       FunctionType *FT) {
  if (id == 0)
    return AttributeList();
  auto [FnAttrID, ArgAttrID] = unpackID(IntrinsicsToAttributesMap[id - 1]);
  using PairTy = std::pair<unsigned, AttributeSet>;
  alignas(PairTy) char ASStorage[sizeof(PairTy) * 17];
  PairTy *AS = reinterpret_cast<PairTy *>(ASStorage);

  // Construct an ArrayRef for easier range checking.
  ArrayRef<ArgAttributesInfo> ArgAttributesInfoTableAR(ArgAttributesInfoTable);
  if (ArgAttrID >= ArgAttributesInfoTableAR.size())
    llvm_unreachable("Invalid arguments attribute ID");

  auto [StartIndex, NumAttrs] = ArgAttributesInfoTableAR[ArgAttrID];
  for (unsigned Idx = 0; Idx < NumAttrs; ++Idx) {
    auto [ArgNo, ArgAttrID] = ArgAttrIdTable[StartIndex + Idx];
    AS[Idx] = {ArgNo,
        getIntrinsicArgAttributeSet(C, ArgAttrID, FT->getContainedType(ArgNo))};
  }
  if (FnAttrID != 127) {
    AS[NumAttrs++] = {AttributeList::FunctionIndex,
                      getIntrinsicFnAttributeSet(C, FnAttrID)};
  }
  return AttributeList::get(C, ArrayRef(AS, NumAttrs));
}

AttributeSet Intrinsic::getFnAttributes(LLVMContext &C, ID id) {
  if (id == 0)
    return AttributeSet();
  auto [FnAttrID, _] = unpackID(IntrinsicsToAttributesMap[id - 1]);
  if (FnAttrID == 127)
    return AttributeSet();
  return getIntrinsicFnAttributeSet(C, FnAttrID);
}
#endif // GET_INTRINSIC_ATTRIBUTES

// Intrinsic ID to pretty print bitset.
#ifdef GET_INTRINSIC_PRETTY_PRINT_TABLE
static constexpr uint8_t PPTable[] = {
  0
  ,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<4),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0 | (1<<3) | (1<<4),
  0 | (1<<0) | (1<<2) | (1<<3),
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0
};

return (PPTable[id/8] & (1 << (id%8))) != 0;
#endif // GET_INTRINSIC_PRETTY_PRINT_TABLE


#ifdef GET_INTRINSIC_PRETTY_PRINT_ARGUMENTS
void Intrinsic::printImmArg(ID IID, unsigned ArgIdx, raw_ostream &OS, const Constant *ImmArgVal) {
  using namespace Intrinsic;
  switch (IID) {
  case nvvm_tcgen05_mma_tensor:
    switch (ArgIdx) {
    case 5:
      OS << "kind=";
      nvvm::printTcgen05MMAKind(OS, ImmArgVal);
      return;
    case 6:
      OS << "cta_group=";
      return;
    case 7:
      OS << "collector=";
      nvvm::printTcgen05CollectorUsageOp(OS, ImmArgVal);
      return;
    }
    break;
  case nvvm_tensormap_replace_elemtype:
    switch (ArgIdx) {
    case 1:
      OS << "elemtype=";
      nvvm::printTensormapElemType(OS, ImmArgVal);
      return;
    }
    break;
  case nvvm_tensormap_replace_fill_mode:
    switch (ArgIdx) {
    case 1:
      OS << "fill_mode=";
      nvvm::printTensormapFillMode(OS, ImmArgVal);
      return;
    }
    break;
  case nvvm_tensormap_replace_interleave_layout:
    switch (ArgIdx) {
    case 1:
      OS << "interleave_layout=";
      nvvm::printTensormapInterleaveLayout(OS, ImmArgVal);
      return;
    }
    break;
  case nvvm_tensormap_replace_swizzle_atomicity:
    switch (ArgIdx) {
    case 1:
      OS << "swizzle_atomicity=";
      nvvm::printTensormapSwizzleAtomicity(OS, ImmArgVal);
      return;
    }
    break;
  case nvvm_tensormap_replace_swizzle_mode:
    switch (ArgIdx) {
    case 1:
      OS << "swizzle_mode=";
      nvvm::printTensormapSwizzleMode(OS, ImmArgVal);
      return;
    }
    break;
  default:
    break;
  }
}
#endif // GET_INTRINSIC_PRETTY_PRINT_ARGUMENTS

// Get the LLVM intrinsic that corresponds to a builtin. This is used by the
// C front-end. The builtin name is passed in as BuiltinName, and a target
// prefix (e.g. 'ppc') is passed in as TargetPrefix.
#ifdef GET_LLVM_INTRINSIC_FOR_CLANG_BUILTIN
Intrinsic::ID
Intrinsic::getIntrinsicForClangBuiltin(StringRef TargetPrefix, 
                                      StringRef BuiltinName) {
  using namespace Intrinsic;

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinNamesStorage[] =
  "\0"
  "adjust_trampoline\0"
  "allow_runtime_check\0"
  "debugtrap\0"
  "init_trampoline\0"
  "object_size\0"
  "stack_restore\0"
  "stack_save\0"
  "thread_pointer\0"
  "trap\0"
  "unwind_init\0"
  "arm_chkfeat\0"
  "arm_dmb\0"
  "arm_dsb\0"
  "arm_gcspopm\0"
  "arm_gcsss\0"
  "arm_in_streaming_mode\0"
  "arm_isb\0"
  "arm_prefetch\0"
  "arm_range_prefetch\0"
  "sve_svaesd_u8\0"
  "sve_svaese_u8\0"
  "sve_svaesimc_u8\0"
  "sve_svaesmc_u8\0"
  "sve_svrax1_u64\0"
  "sve_svrdffr\0"
  "sve_svrdffr_z\0"
  "sve_svsetffr\0"
  "sve_svsm4e_u32\0"
  "sve_svsm4ekey_u32\0"
  "sve_svwrffr\0"
  "add_max_i32\0"
  "add_max_u32\0"
  "add_min_i32\0"
  "add_min_u32\0"
  "alignbyte\0"
  "ashr_pk_i8_i32\0"
  "ashr_pk_u8_i32\0"
  "buffer_wbinvl1\0"
  "buffer_wbinvl1_sc\0"
  "buffer_wbinvl1_vol\0"
  "cluster_id_x\0"
  "cluster_id_y\0"
  "cluster_id_z\0"
  "cluster_load_async_to_lds_b128\0"
  "cluster_load_async_to_lds_b32\0"
  "cluster_load_async_to_lds_b64\0"
  "cluster_load_async_to_lds_b8\0"
  "cluster_workgroup_flat_id\0"
  "cluster_workgroup_id_x\0"
  "cluster_workgroup_id_y\0"
  "cluster_workgroup_id_z\0"
  "cluster_workgroup_max_flat_id\0"
  "cluster_workgroup_max_id_x\0"
  "cluster_workgroup_max_id_y\0"
  "cluster_workgroup_max_id_z\0"
  "cubeid\0"
  "cubema\0"
  "cubesc\0"
  "cubetc\0"
  "cvt_f16_bf8\0"
  "cvt_f16_fp8\0"
  "cvt_f32_bf8\0"
  "cvt_f32_fp8\0"
  "cvt_f32_fp8_e5m3\0"
  "cvt_off_f32_i4\0"
  "cvt_pk_bf8_f16\0"
  "cvt_pk_bf8_f32\0"
  "cvt_pk_f16_bf8\0"
  "cvt_pk_f16_fp8\0"
  "cvt_pk_f32_bf8\0"
  "cvt_pk_f32_fp8\0"
  "cvt_pk_fp8_f16\0"
  "cvt_pk_fp8_f32\0"
  "cvt_pk_fp8_f32_e5m3\0"
  "cvt_pk_i16\0"
  "cvt_pk_u16\0"
  "cvt_pk_u8_f32\0"
  "cvt_pknorm_i16\0"
  "cvt_pknorm_u16\0"
  "cvt_pkrtz\0"
  "cvt_scale_pk16_bf16_bf6\0"
  "cvt_scale_pk16_bf16_fp6\0"
  "cvt_scale_pk16_f16_bf6\0"
  "cvt_scale_pk16_f16_fp6\0"
  "cvt_scale_pk16_f32_bf6\0"
  "cvt_scale_pk16_f32_fp6\0"
  "cvt_scale_pk8_bf16_bf8\0"
  "cvt_scale_pk8_bf16_fp4\0"
  "cvt_scale_pk8_bf16_fp8\0"
  "cvt_scale_pk8_f16_bf8\0"
  "cvt_scale_pk8_f16_fp4\0"
  "cvt_scale_pk8_f16_fp8\0"
  "cvt_scale_pk8_f32_bf8\0"
  "cvt_scale_pk8_f32_fp4\0"
  "cvt_scale_pk8_f32_fp8\0"
  "cvt_scalef32_2xpk16_bf6_f32\0"
  "cvt_scalef32_2xpk16_fp6_f32\0"
  "cvt_scalef32_f16_bf8\0"
  "cvt_scalef32_f16_fp8\0"
  "cvt_scalef32_f32_bf8\0"
  "cvt_scalef32_f32_fp8\0"
  "cvt_scalef32_pk16_bf6_bf16\0"
  "cvt_scalef32_pk16_bf6_f16\0"
  "cvt_scalef32_pk16_bf6_f32\0"
  "cvt_scalef32_pk16_fp6_bf16\0"
  "cvt_scalef32_pk16_fp6_f16\0"
  "cvt_scalef32_pk16_fp6_f32\0"
  "cvt_scalef32_pk32_bf16_bf6\0"
  "cvt_scalef32_pk32_bf16_fp6\0"
  "cvt_scalef32_pk32_bf6_bf16\0"
  "cvt_scalef32_pk32_bf6_f16\0"
  "cvt_scalef32_pk32_f16_bf6\0"
  "cvt_scalef32_pk32_f16_fp6\0"
  "cvt_scalef32_pk32_f32_bf6\0"
  "cvt_scalef32_pk32_f32_fp6\0"
  "cvt_scalef32_pk32_fp6_bf16\0"
  "cvt_scalef32_pk32_fp6_f16\0"
  "cvt_scalef32_pk8_bf8_bf16\0"
  "cvt_scalef32_pk8_bf8_f16\0"
  "cvt_scalef32_pk8_bf8_f32\0"
  "cvt_scalef32_pk8_fp4_bf16\0"
  "cvt_scalef32_pk8_fp4_f16\0"
  "cvt_scalef32_pk8_fp4_f32\0"
  "cvt_scalef32_pk8_fp8_bf16\0"
  "cvt_scalef32_pk8_fp8_f16\0"
  "cvt_scalef32_pk8_fp8_f32\0"
  "cvt_scalef32_pk_bf16_bf8\0"
  "cvt_scalef32_pk_bf16_fp4\0"
  "cvt_scalef32_pk_bf16_fp8\0"
  "cvt_scalef32_pk_bf8_bf16\0"
  "cvt_scalef32_pk_bf8_f16\0"
  "cvt_scalef32_pk_bf8_f32\0"
  "cvt_scalef32_pk_f16_bf8\0"
  "cvt_scalef32_pk_f16_fp4\0"
  "cvt_scalef32_pk_f16_fp8\0"
  "cvt_scalef32_pk_f32_bf8\0"
  "cvt_scalef32_pk_f32_fp4\0"
  "cvt_scalef32_pk_f32_fp8\0"
  "cvt_scalef32_pk_fp4_bf16\0"
  "cvt_scalef32_pk_fp4_f16\0"
  "cvt_scalef32_pk_fp4_f32\0"
  "cvt_scalef32_pk_fp8_bf16\0"
  "cvt_scalef32_pk_fp8_f16\0"
  "cvt_scalef32_pk_fp8_f32\0"
  "cvt_scalef32_sr_bf8_bf16\0"
  "cvt_scalef32_sr_bf8_f16\0"
  "cvt_scalef32_sr_bf8_f32\0"
  "cvt_scalef32_sr_fp8_bf16\0"
  "cvt_scalef32_sr_fp8_f16\0"
  "cvt_scalef32_sr_fp8_f32\0"
  "cvt_scalef32_sr_pk16_bf6_bf16\0"
  "cvt_scalef32_sr_pk16_bf6_f16\0"
  "cvt_scalef32_sr_pk16_bf6_f32\0"
  "cvt_scalef32_sr_pk16_fp6_bf16\0"
  "cvt_scalef32_sr_pk16_fp6_f16\0"
  "cvt_scalef32_sr_pk16_fp6_f32\0"
  "cvt_scalef32_sr_pk32_bf6_bf16\0"
  "cvt_scalef32_sr_pk32_bf6_f16\0"
  "cvt_scalef32_sr_pk32_bf6_f32\0"
  "cvt_scalef32_sr_pk32_fp6_bf16\0"
  "cvt_scalef32_sr_pk32_fp6_f16\0"
  "cvt_scalef32_sr_pk32_fp6_f32\0"
  "cvt_scalef32_sr_pk8_bf8_bf16\0"
  "cvt_scalef32_sr_pk8_bf8_f16\0"
  "cvt_scalef32_sr_pk8_bf8_f32\0"
  "cvt_scalef32_sr_pk8_fp4_bf16\0"
  "cvt_scalef32_sr_pk8_fp4_f16\0"
  "cvt_scalef32_sr_pk8_fp4_f32\0"
  "cvt_scalef32_sr_pk8_fp8_bf16\0"
  "cvt_scalef32_sr_pk8_fp8_f16\0"
  "cvt_scalef32_sr_pk8_fp8_f32\0"
  "cvt_scalef32_sr_pk_fp4_bf16\0"
  "cvt_scalef32_sr_pk_fp4_f16\0"
  "cvt_scalef32_sr_pk_fp4_f32\0"
  "cvt_sr_bf16_f32\0"
  "cvt_sr_bf8_f16\0"
  "cvt_sr_bf8_f32\0"
  "cvt_sr_f16_f32\0"
  "cvt_sr_fp8_f16\0"
  "cvt_sr_fp8_f32\0"
  "cvt_sr_fp8_f32_e5m3\0"
  "cvt_sr_pk_bf16_f32\0"
  "cvt_sr_pk_f16_f32\0"
  "dispatch_id\0"
  "dot4_f32_bf8_bf8\0"
  "dot4_f32_bf8_fp8\0"
  "dot4_f32_fp8_bf8\0"
  "dot4_f32_fp8_fp8\0"
  "ds_add_gs_reg_rtn\0"
  "ds_atomic_async_barrier_arrive_b64\0"
  "ds_atomic_barrier_arrive_rtn_b64\0"
  "ds_bpermute\0"
  "ds_bpermute_fi_b32\0"
  "ds_gws_barrier\0"
  "ds_gws_init\0"
  "ds_gws_sema_br\0"
  "ds_gws_sema_p\0"
  "ds_gws_sema_release_all\0"
  "ds_gws_sema_v\0"
  "ds_permute\0"
  "ds_sub_gs_reg_rtn\0"
  "ds_swizzle\0"
  "endpgm\0"
  "fdot2\0"
  "fdot2_bf16_bf16\0"
  "fdot2_f16_f16\0"
  "fdot2_f32_bf16\0"
  "fdot2c_f32_bf16\0"
  "flat_prefetch\0"
  "fmul_legacy\0"
  "global_load_async_to_lds_b128\0"
  "global_load_async_to_lds_b32\0"
  "global_load_async_to_lds_b64\0"
  "global_load_async_to_lds_b8\0"
  "global_load_lds\0"
  "global_prefetch\0"
  "global_store_async_from_lds_b128\0"
  "global_store_async_from_lds_b32\0"
  "global_store_async_from_lds_b64\0"
  "global_store_async_from_lds_b8\0"
  "groupstaticsize\0"
  "iglp_opt\0"
  "implicit_buffer_ptr\0"
  "implicitarg_ptr\0"
  "interp_mov\0"
  "interp_p1\0"
  "interp_p1_f16\0"
  "interp_p2\0"
  "interp_p2_f16\0"
  "is_private\0"
  "is_shared\0"
  "kernarg_segment_ptr\0"
  "lerp\0"
  "mbcnt_hi\0"
  "mbcnt_lo\0"
  "mfma_f32_16x16x16bf16_1k\0"
  "mfma_f32_16x16x16f16\0"
  "mfma_f32_16x16x1f32\0"
  "mfma_f32_16x16x2bf16\0"
  "mfma_f32_16x16x32_bf16\0"
  "mfma_f32_16x16x32_bf8_bf8\0"
  "mfma_f32_16x16x32_bf8_fp8\0"
  "mfma_f32_16x16x32_f16\0"
  "mfma_f32_16x16x32_fp8_bf8\0"
  "mfma_f32_16x16x32_fp8_fp8\0"
  "mfma_f32_16x16x4bf16_1k\0"
  "mfma_f32_16x16x4f16\0"
  "mfma_f32_16x16x4f32\0"
  "mfma_f32_16x16x8_xf32\0"
  "mfma_f32_16x16x8bf16\0"
  "mfma_f32_32x32x16_bf16\0"
  "mfma_f32_32x32x16_bf8_bf8\0"
  "mfma_f32_32x32x16_bf8_fp8\0"
  "mfma_f32_32x32x16_f16\0"
  "mfma_f32_32x32x16_fp8_bf8\0"
  "mfma_f32_32x32x16_fp8_fp8\0"
  "mfma_f32_32x32x1f32\0"
  "mfma_f32_32x32x2bf16\0"
  "mfma_f32_32x32x2f32\0"
  "mfma_f32_32x32x4_xf32\0"
  "mfma_f32_32x32x4bf16\0"
  "mfma_f32_32x32x4bf16_1k\0"
  "mfma_f32_32x32x4f16\0"
  "mfma_f32_32x32x8bf16_1k\0"
  "mfma_f32_32x32x8f16\0"
  "mfma_f32_4x4x1f32\0"
  "mfma_f32_4x4x2bf16\0"
  "mfma_f32_4x4x4bf16_1k\0"
  "mfma_f32_4x4x4f16\0"
  "mfma_f64_16x16x4f64\0"
  "mfma_f64_4x4x4f64\0"
  "mfma_i32_16x16x16i8\0"
  "mfma_i32_16x16x32_i8\0"
  "mfma_i32_16x16x4i8\0"
  "mfma_i32_16x16x64_i8\0"
  "mfma_i32_32x32x16_i8\0"
  "mfma_i32_32x32x32_i8\0"
  "mfma_i32_32x32x4i8\0"
  "mfma_i32_32x32x8i8\0"
  "mfma_i32_4x4x4i8\0"
  "mqsad_pk_u16_u8\0"
  "mqsad_u32_u8\0"
  "msad_u8\0"
  "perm\0"
  "perm_pk16_b4_u4\0"
  "perm_pk16_b6_u4\0"
  "perm_pk16_b8_u4\0"
  "permlane16_var\0"
  "permlane_bcast\0"
  "permlane_down\0"
  "permlane_idx_gen\0"
  "permlane_up\0"
  "permlane_xor\0"
  "permlanex16_var\0"
  "pk_add_max_i16\0"
  "pk_add_max_u16\0"
  "pk_add_min_i16\0"
  "pk_add_min_u16\0"
  "prng_b32\0"
  "qsad_pk_u16_u8\0"
  "queue_ptr\0"
  "raw_ptr_buffer_load_lds\0"
  "rcp_legacy\0"
  "rsq_legacy\0"
  "s_barrier\0"
  "s_barrier_init\0"
  "s_barrier_join\0"
  "s_barrier_leave\0"
  "s_barrier_signal\0"
  "s_barrier_signal_isfirst\0"
  "s_barrier_signal_var\0"
  "s_barrier_wait\0"
  "s_buffer_prefetch_data\0"
  "s_cluster_barrier\0"
  "s_dcache_inv\0"
  "s_dcache_inv_vol\0"
  "s_dcache_wb\0"
  "s_dcache_wb_vol\0"
  "s_decperflevel\0"
  "s_get_barrier_state\0"
  "s_get_named_barrier_state\0"
  "s_get_waveid_in_workgroup\0"
  "s_getpc\0"
  "s_getreg\0"
  "s_incperflevel\0"
  "s_memrealtime\0"
  "s_memtime\0"
  "s_monitor_sleep\0"
  "s_sendmsg\0"
  "s_sendmsghalt\0"
  "s_setprio\0"
  "s_setprio_inc_wg\0"
  "s_setreg\0"
  "s_sleep\0"
  "s_sleep_var\0"
  "s_ttracedata\0"
  "s_ttracedata_imm\0"
  "s_wait_asynccnt\0"
  "s_wait_event_export_ready\0"
  "s_wait_tensorcnt\0"
  "s_waitcnt\0"
  "s_wakeup_barrier\0"
  "sad_hi_u8\0"
  "sad_u16\0"
  "sad_u8\0"
  "sat_pk4_i4_i8\0"
  "sat_pk4_u4_u8\0"
  "sched_barrier\0"
  "sched_group_barrier\0"
  "sdot2\0"
  "sdot4\0"
  "sdot8\0"
  "smfmac_f32_16x16x128_bf8_bf8\0"
  "smfmac_f32_16x16x128_bf8_fp8\0"
  "smfmac_f32_16x16x128_fp8_bf8\0"
  "smfmac_f32_16x16x128_fp8_fp8\0"
  "smfmac_f32_16x16x32_bf16\0"
  "smfmac_f32_16x16x32_f16\0"
  "smfmac_f32_16x16x64_bf16\0"
  "smfmac_f32_16x16x64_bf8_bf8\0"
  "smfmac_f32_16x16x64_bf8_fp8\0"
  "smfmac_f32_16x16x64_f16\0"
  "smfmac_f32_16x16x64_fp8_bf8\0"
  "smfmac_f32_16x16x64_fp8_fp8\0"
  "smfmac_f32_32x32x16_bf16\0"
  "smfmac_f32_32x32x16_f16\0"
  "smfmac_f32_32x32x32_bf16\0"
  "smfmac_f32_32x32x32_bf8_bf8\0"
  "smfmac_f32_32x32x32_bf8_fp8\0"
  "smfmac_f32_32x32x32_f16\0"
  "smfmac_f32_32x32x32_fp8_bf8\0"
  "smfmac_f32_32x32x32_fp8_fp8\0"
  "smfmac_f32_32x32x64_bf8_bf8\0"
  "smfmac_f32_32x32x64_bf8_fp8\0"
  "smfmac_f32_32x32x64_fp8_bf8\0"
  "smfmac_f32_32x32x64_fp8_fp8\0"
  "smfmac_i32_16x16x128_i8\0"
  "smfmac_i32_16x16x64_i8\0"
  "smfmac_i32_32x32x32_i8\0"
  "smfmac_i32_32x32x64_i8\0"
  "struct_ptr_buffer_load_lds\0"
  "sudot4\0"
  "sudot8\0"
  "tensor_load_to_lds\0"
  "tensor_load_to_lds_d2\0"
  "tensor_store_from_lds\0"
  "tensor_store_from_lds_d2\0"
  "udot2\0"
  "udot4\0"
  "udot8\0"
  "wave_barrier\0"
  "wavefrontsize\0"
  "workgroup_id_x\0"
  "workgroup_id_y\0"
  "workgroup_id_z\0"
  "workitem_id_x\0"
  "workitem_id_y\0"
  "workitem_id_z\0"
  "cdp\0"
  "cdp2\0"
  "cmse_TT\0"
  "cmse_TTA\0"
  "cmse_TTAT\0"
  "cmse_TTT\0"
  "dmb\0"
  "dsb\0"
  "get_fpscr\0"
  "isb\0"
  "ldc\0"
  "ldc2\0"
  "ldc2l\0"
  "ldcl\0"
  "mcr\0"
  "mcr2\0"
  "mrc\0"
  "mrc2\0"
  "qadd\0"
  "qadd16\0"
  "qadd8\0"
  "qasx\0"
  "qsax\0"
  "qsub\0"
  "qsub16\0"
  "qsub8\0"
  "sadd16\0"
  "sadd8\0"
  "sasx\0"
  "sel\0"
  "set_fpscr\0"
  "shadd16\0"
  "shadd8\0"
  "shasx\0"
  "shsax\0"
  "shsub16\0"
  "shsub8\0"
  "smlabb\0"
  "smlabt\0"
  "smlad\0"
  "smladx\0"
  "smlald\0"
  "smlaldx\0"
  "smlatb\0"
  "smlatt\0"
  "smlawb\0"
  "smlawt\0"
  "smlsd\0"
  "smlsdx\0"
  "smlsld\0"
  "smlsldx\0"
  "smuad\0"
  "smuadx\0"
  "smulbb\0"
  "smulbt\0"
  "smultb\0"
  "smultt\0"
  "smulwb\0"
  "smulwt\0"
  "smusd\0"
  "smusdx\0"
  "ssat\0"
  "ssat16\0"
  "ssax\0"
  "ssub16\0"
  "ssub8\0"
  "stc\0"
  "stc2\0"
  "stc2l\0"
  "stcl\0"
  "sxtab16\0"
  "sxtb16\0"
  "uadd16\0"
  "uadd8\0"
  "uasx\0"
  "uhadd16\0"
  "uhadd8\0"
  "uhasx\0"
  "uhsax\0"
  "uhsub16\0"
  "uhsub8\0"
  "uqadd16\0"
  "uqadd8\0"
  "uqasx\0"
  "uqsax\0"
  "uqsub16\0"
  "uqsub8\0"
  "usad8\0"
  "usada8\0"
  "usat\0"
  "usat16\0"
  "usax\0"
  "usub16\0"
  "usub8\0"
  "uxtab16\0"
  "uxtb16\0"
  "btf_type_id\0"
  "compare\0"
  "getelementptr_and_load\0"
  "getelementptr_and_store\0"
  "load_byte\0"
  "load_half\0"
  "load_word\0"
  "passthrough\0"
  "preserve_enum_value\0"
  "preserve_field_info\0"
  "preserve_type_info\0"
  "pseudo\0"
  "HEXAGON_A2_abs\0"
  "HEXAGON_A2_absp\0"
  "HEXAGON_A2_abssat\0"
  "HEXAGON_A2_add\0"
  "HEXAGON_A2_addh_h16_hh\0"
  "HEXAGON_A2_addh_h16_hl\0"
  "HEXAGON_A2_addh_h16_lh\0"
  "HEXAGON_A2_addh_h16_ll\0"
  "HEXAGON_A2_addh_h16_sat_hh\0"
  "HEXAGON_A2_addh_h16_sat_hl\0"
  "HEXAGON_A2_addh_h16_sat_lh\0"
  "HEXAGON_A2_addh_h16_sat_ll\0"
  "HEXAGON_A2_addh_l16_hl\0"
  "HEXAGON_A2_addh_l16_ll\0"
  "HEXAGON_A2_addh_l16_sat_hl\0"
  "HEXAGON_A2_addh_l16_sat_ll\0"
  "HEXAGON_A2_addi\0"
  "HEXAGON_A2_addp\0"
  "HEXAGON_A2_addpsat\0"
  "HEXAGON_A2_addsat\0"
  "HEXAGON_A2_addsp\0"
  "HEXAGON_A2_and\0"
  "HEXAGON_A2_andir\0"
  "HEXAGON_A2_andp\0"
  "HEXAGON_A2_aslh\0"
  "HEXAGON_A2_asrh\0"
  "HEXAGON_A2_combine_hh\0"
  "HEXAGON_A2_combine_hl\0"
  "HEXAGON_A2_combine_lh\0"
  "HEXAGON_A2_combine_ll\0"
  "HEXAGON_A2_combineii\0"
  "HEXAGON_A2_combinew\0"
  "HEXAGON_A2_max\0"
  "HEXAGON_A2_maxp\0"
  "HEXAGON_A2_maxu\0"
  "HEXAGON_A2_maxup\0"
  "HEXAGON_A2_min\0"
  "HEXAGON_A2_minp\0"
  "HEXAGON_A2_minu\0"
  "HEXAGON_A2_minup\0"
  "HEXAGON_A2_neg\0"
  "HEXAGON_A2_negp\0"
  "HEXAGON_A2_negsat\0"
  "HEXAGON_A2_not\0"
  "HEXAGON_A2_notp\0"
  "HEXAGON_A2_or\0"
  "HEXAGON_A2_orir\0"
  "HEXAGON_A2_orp\0"
  "HEXAGON_A2_roundsat\0"
  "HEXAGON_A2_sat\0"
  "HEXAGON_A2_satb\0"
  "HEXAGON_A2_sath\0"
  "HEXAGON_A2_satub\0"
  "HEXAGON_A2_satuh\0"
  "HEXAGON_A2_sub\0"
  "HEXAGON_A2_subh_h16_hh\0"
  "HEXAGON_A2_subh_h16_hl\0"
  "HEXAGON_A2_subh_h16_lh\0"
  "HEXAGON_A2_subh_h16_ll\0"
  "HEXAGON_A2_subh_h16_sat_hh\0"
  "HEXAGON_A2_subh_h16_sat_hl\0"
  "HEXAGON_A2_subh_h16_sat_lh\0"
  "HEXAGON_A2_subh_h16_sat_ll\0"
  "HEXAGON_A2_subh_l16_hl\0"
  "HEXAGON_A2_subh_l16_ll\0"
  "HEXAGON_A2_subh_l16_sat_hl\0"
  "HEXAGON_A2_subh_l16_sat_ll\0"
  "HEXAGON_A2_subp\0"
  "HEXAGON_A2_subri\0"
  "HEXAGON_A2_subsat\0"
  "HEXAGON_A2_svaddh\0"
  "HEXAGON_A2_svaddhs\0"
  "HEXAGON_A2_svadduhs\0"
  "HEXAGON_A2_svavgh\0"
  "HEXAGON_A2_svavghs\0"
  "HEXAGON_A2_svnavgh\0"
  "HEXAGON_A2_svsubh\0"
  "HEXAGON_A2_svsubhs\0"
  "HEXAGON_A2_svsubuhs\0"
  "HEXAGON_A2_swiz\0"
  "HEXAGON_A2_sxtb\0"
  "HEXAGON_A2_sxth\0"
  "HEXAGON_A2_sxtw\0"
  "HEXAGON_A2_tfr\0"
  "HEXAGON_A2_tfrih\0"
  "HEXAGON_A2_tfril\0"
  "HEXAGON_A2_tfrp\0"
  "HEXAGON_A2_tfrpi\0"
  "HEXAGON_A2_tfrsi\0"
  "HEXAGON_A2_vabsh\0"
  "HEXAGON_A2_vabshsat\0"
  "HEXAGON_A2_vabsw\0"
  "HEXAGON_A2_vabswsat\0"
  "HEXAGON_A2_vaddb_map\0"
  "HEXAGON_A2_vaddh\0"
  "HEXAGON_A2_vaddhs\0"
  "HEXAGON_A2_vaddub\0"
  "HEXAGON_A2_vaddubs\0"
  "HEXAGON_A2_vadduhs\0"
  "HEXAGON_A2_vaddw\0"
  "HEXAGON_A2_vaddws\0"
  "HEXAGON_A2_vavgh\0"
  "HEXAGON_A2_vavghcr\0"
  "HEXAGON_A2_vavghr\0"
  "HEXAGON_A2_vavgub\0"
  "HEXAGON_A2_vavgubr\0"
  "HEXAGON_A2_vavguh\0"
  "HEXAGON_A2_vavguhr\0"
  "HEXAGON_A2_vavguw\0"
  "HEXAGON_A2_vavguwr\0"
  "HEXAGON_A2_vavgw\0"
  "HEXAGON_A2_vavgwcr\0"
  "HEXAGON_A2_vavgwr\0"
  "HEXAGON_A2_vcmpbeq\0"
  "HEXAGON_A2_vcmpbgtu\0"
  "HEXAGON_A2_vcmpheq\0"
  "HEXAGON_A2_vcmphgt\0"
  "HEXAGON_A2_vcmphgtu\0"
  "HEXAGON_A2_vcmpweq\0"
  "HEXAGON_A2_vcmpwgt\0"
  "HEXAGON_A2_vcmpwgtu\0"
  "HEXAGON_A2_vconj\0"
  "HEXAGON_A2_vmaxb\0"
  "HEXAGON_A2_vmaxh\0"
  "HEXAGON_A2_vmaxub\0"
  "HEXAGON_A2_vmaxuh\0"
  "HEXAGON_A2_vmaxuw\0"
  "HEXAGON_A2_vmaxw\0"
  "HEXAGON_A2_vminb\0"
  "HEXAGON_A2_vminh\0"
  "HEXAGON_A2_vminub\0"
  "HEXAGON_A2_vminuh\0"
  "HEXAGON_A2_vminuw\0"
  "HEXAGON_A2_vminw\0"
  "HEXAGON_A2_vnavgh\0"
  "HEXAGON_A2_vnavghcr\0"
  "HEXAGON_A2_vnavghr\0"
  "HEXAGON_A2_vnavgw\0"
  "HEXAGON_A2_vnavgwcr\0"
  "HEXAGON_A2_vnavgwr\0"
  "HEXAGON_A2_vraddub\0"
  "HEXAGON_A2_vraddub_acc\0"
  "HEXAGON_A2_vrsadub\0"
  "HEXAGON_A2_vrsadub_acc\0"
  "HEXAGON_A2_vsubb_map\0"
  "HEXAGON_A2_vsubh\0"
  "HEXAGON_A2_vsubhs\0"
  "HEXAGON_A2_vsubub\0"
  "HEXAGON_A2_vsububs\0"
  "HEXAGON_A2_vsubuhs\0"
  "HEXAGON_A2_vsubw\0"
  "HEXAGON_A2_vsubws\0"
  "HEXAGON_A2_xor\0"
  "HEXAGON_A2_xorp\0"
  "HEXAGON_A2_zxtb\0"
  "HEXAGON_A2_zxth\0"
  "HEXAGON_A4_andn\0"
  "HEXAGON_A4_andnp\0"
  "HEXAGON_A4_bitsplit\0"
  "HEXAGON_A4_bitspliti\0"
  "HEXAGON_A4_boundscheck\0"
  "HEXAGON_A4_cmpbeq\0"
  "HEXAGON_A4_cmpbeqi\0"
  "HEXAGON_A4_cmpbgt\0"
  "HEXAGON_A4_cmpbgti\0"
  "HEXAGON_A4_cmpbgtu\0"
  "HEXAGON_A4_cmpbgtui\0"
  "HEXAGON_A4_cmpheq\0"
  "HEXAGON_A4_cmpheqi\0"
  "HEXAGON_A4_cmphgt\0"
  "HEXAGON_A4_cmphgti\0"
  "HEXAGON_A4_cmphgtu\0"
  "HEXAGON_A4_cmphgtui\0"
  "HEXAGON_A4_combineir\0"
  "HEXAGON_A4_combineri\0"
  "HEXAGON_A4_cround_ri\0"
  "HEXAGON_A4_cround_rr\0"
  "HEXAGON_A4_modwrapu\0"
  "HEXAGON_A4_orn\0"
  "HEXAGON_A4_ornp\0"
  "HEXAGON_A4_rcmpeq\0"
  "HEXAGON_A4_rcmpeqi\0"
  "HEXAGON_A4_rcmpneq\0"
  "HEXAGON_A4_rcmpneqi\0"
  "HEXAGON_A4_round_ri\0"
  "HEXAGON_A4_round_ri_sat\0"
  "HEXAGON_A4_round_rr\0"
  "HEXAGON_A4_round_rr_sat\0"
  "HEXAGON_A4_tlbmatch\0"
  "HEXAGON_A4_vcmpbeq_any\0"
  "HEXAGON_A4_vcmpbeqi\0"
  "HEXAGON_A4_vcmpbgt\0"
  "HEXAGON_A4_vcmpbgti\0"
  "HEXAGON_A4_vcmpbgtui\0"
  "HEXAGON_A4_vcmpheqi\0"
  "HEXAGON_A4_vcmphgti\0"
  "HEXAGON_A4_vcmphgtui\0"
  "HEXAGON_A4_vcmpweqi\0"
  "HEXAGON_A4_vcmpwgti\0"
  "HEXAGON_A4_vcmpwgtui\0"
  "HEXAGON_A4_vrmaxh\0"
  "HEXAGON_A4_vrmaxuh\0"
  "HEXAGON_A4_vrmaxuw\0"
  "HEXAGON_A4_vrmaxw\0"
  "HEXAGON_A4_vrminh\0"
  "HEXAGON_A4_vrminuh\0"
  "HEXAGON_A4_vrminuw\0"
  "HEXAGON_A4_vrminw\0"
  "HEXAGON_A5_vaddhubs\0"
  "HEXAGON_A6_vcmpbeq_notany\0"
  "HEXAGON_A7_clip\0"
  "HEXAGON_A7_croundd_ri\0"
  "HEXAGON_A7_croundd_rr\0"
  "HEXAGON_A7_vclip\0"
  "HEXAGON_C2_all8\0"
  "HEXAGON_C2_and\0"
  "HEXAGON_C2_andn\0"
  "HEXAGON_C2_any8\0"
  "HEXAGON_C2_bitsclr\0"
  "HEXAGON_C2_bitsclri\0"
  "HEXAGON_C2_bitsset\0"
  "HEXAGON_C2_cmpeq\0"
  "HEXAGON_C2_cmpeqi\0"
  "HEXAGON_C2_cmpeqp\0"
  "HEXAGON_C2_cmpgei\0"
  "HEXAGON_C2_cmpgeui\0"
  "HEXAGON_C2_cmpgt\0"
  "HEXAGON_C2_cmpgti\0"
  "HEXAGON_C2_cmpgtp\0"
  "HEXAGON_C2_cmpgtu\0"
  "HEXAGON_C2_cmpgtui\0"
  "HEXAGON_C2_cmpgtup\0"
  "HEXAGON_C2_cmplt\0"
  "HEXAGON_C2_cmpltu\0"
  "HEXAGON_C2_mask\0"
  "HEXAGON_C2_mux\0"
  "HEXAGON_C2_muxii\0"
  "HEXAGON_C2_muxir\0"
  "HEXAGON_C2_muxri\0"
  "HEXAGON_C2_not\0"
  "HEXAGON_C2_or\0"
  "HEXAGON_C2_orn\0"
  "HEXAGON_C2_pxfer_map\0"
  "HEXAGON_C2_tfrpr\0"
  "HEXAGON_C2_tfrrp\0"
  "HEXAGON_C2_vitpack\0"
  "HEXAGON_C2_vmux\0"
  "HEXAGON_C2_xor\0"
  "HEXAGON_C4_and_and\0"
  "HEXAGON_C4_and_andn\0"
  "HEXAGON_C4_and_or\0"
  "HEXAGON_C4_and_orn\0"
  "HEXAGON_C4_cmplte\0"
  "HEXAGON_C4_cmpltei\0"
  "HEXAGON_C4_cmplteu\0"
  "HEXAGON_C4_cmplteui\0"
  "HEXAGON_C4_cmpneq\0"
  "HEXAGON_C4_cmpneqi\0"
  "HEXAGON_C4_fastcorner9\0"
  "HEXAGON_C4_fastcorner9_not\0"
  "HEXAGON_C4_nbitsclr\0"
  "HEXAGON_C4_nbitsclri\0"
  "HEXAGON_C4_nbitsset\0"
  "HEXAGON_C4_or_and\0"
  "HEXAGON_C4_or_andn\0"
  "HEXAGON_C4_or_or\0"
  "HEXAGON_C4_or_orn\0"
  "HEXAGON_F2_conv_d2df\0"
  "HEXAGON_F2_conv_d2sf\0"
  "HEXAGON_F2_conv_df2d\0"
  "HEXAGON_F2_conv_df2d_chop\0"
  "HEXAGON_F2_conv_df2sf\0"
  "HEXAGON_F2_conv_df2ud\0"
  "HEXAGON_F2_conv_df2ud_chop\0"
  "HEXAGON_F2_conv_df2uw\0"
  "HEXAGON_F2_conv_df2uw_chop\0"
  "HEXAGON_F2_conv_df2w\0"
  "HEXAGON_F2_conv_df2w_chop\0"
  "HEXAGON_F2_conv_sf2d\0"
  "HEXAGON_F2_conv_sf2d_chop\0"
  "HEXAGON_F2_conv_sf2df\0"
  "HEXAGON_F2_conv_sf2ud\0"
  "HEXAGON_F2_conv_sf2ud_chop\0"
  "HEXAGON_F2_conv_sf2uw\0"
  "HEXAGON_F2_conv_sf2uw_chop\0"
  "HEXAGON_F2_conv_sf2w\0"
  "HEXAGON_F2_conv_sf2w_chop\0"
  "HEXAGON_F2_conv_ud2df\0"
  "HEXAGON_F2_conv_ud2sf\0"
  "HEXAGON_F2_conv_uw2df\0"
  "HEXAGON_F2_conv_uw2sf\0"
  "HEXAGON_F2_conv_w2df\0"
  "HEXAGON_F2_conv_w2sf\0"
  "HEXAGON_F2_dfadd\0"
  "HEXAGON_F2_dfclass\0"
  "HEXAGON_F2_dfcmpeq\0"
  "HEXAGON_F2_dfcmpge\0"
  "HEXAGON_F2_dfcmpgt\0"
  "HEXAGON_F2_dfcmpuo\0"
  "HEXAGON_F2_dfimm_n\0"
  "HEXAGON_F2_dfimm_p\0"
  "HEXAGON_F2_dfmax\0"
  "HEXAGON_F2_dfmin\0"
  "HEXAGON_F2_dfmpyfix\0"
  "HEXAGON_F2_dfmpyhh\0"
  "HEXAGON_F2_dfmpylh\0"
  "HEXAGON_F2_dfmpyll\0"
  "HEXAGON_F2_dfsub\0"
  "HEXAGON_F2_sfadd\0"
  "HEXAGON_F2_sfclass\0"
  "HEXAGON_F2_sfcmpeq\0"
  "HEXAGON_F2_sfcmpge\0"
  "HEXAGON_F2_sfcmpgt\0"
  "HEXAGON_F2_sfcmpuo\0"
  "HEXAGON_F2_sffixupd\0"
  "HEXAGON_F2_sffixupn\0"
  "HEXAGON_F2_sffixupr\0"
  "HEXAGON_F2_sffma\0"
  "HEXAGON_F2_sffma_lib\0"
  "HEXAGON_F2_sffma_sc\0"
  "HEXAGON_F2_sffms\0"
  "HEXAGON_F2_sffms_lib\0"
  "HEXAGON_F2_sfimm_n\0"
  "HEXAGON_F2_sfimm_p\0"
  "HEXAGON_F2_sfmax\0"
  "HEXAGON_F2_sfmin\0"
  "HEXAGON_F2_sfmpy\0"
  "HEXAGON_F2_sfsub\0"
  "HEXAGON_L2_loadw_locked\0"
  "HEXAGON_M2_acci\0"
  "HEXAGON_M2_accii\0"
  "HEXAGON_M2_cmaci_s0\0"
  "HEXAGON_M2_cmacr_s0\0"
  "HEXAGON_M2_cmacs_s0\0"
  "HEXAGON_M2_cmacs_s1\0"
  "HEXAGON_M2_cmacsc_s0\0"
  "HEXAGON_M2_cmacsc_s1\0"
  "HEXAGON_M2_cmpyi_s0\0"
  "HEXAGON_M2_cmpyr_s0\0"
  "HEXAGON_M2_cmpyrs_s0\0"
  "HEXAGON_M2_cmpyrs_s1\0"
  "HEXAGON_M2_cmpyrsc_s0\0"
  "HEXAGON_M2_cmpyrsc_s1\0"
  "HEXAGON_M2_cmpys_s0\0"
  "HEXAGON_M2_cmpys_s1\0"
  "HEXAGON_M2_cmpysc_s0\0"
  "HEXAGON_M2_cmpysc_s1\0"
  "HEXAGON_M2_cnacs_s0\0"
  "HEXAGON_M2_cnacs_s1\0"
  "HEXAGON_M2_cnacsc_s0\0"
  "HEXAGON_M2_cnacsc_s1\0"
  "HEXAGON_M2_dpmpyss_acc_s0\0"
  "HEXAGON_M2_dpmpyss_nac_s0\0"
  "HEXAGON_M2_dpmpyss_rnd_s0\0"
  "HEXAGON_M2_dpmpyss_s0\0"
  "HEXAGON_M2_dpmpyuu_acc_s0\0"
  "HEXAGON_M2_dpmpyuu_nac_s0\0"
  "HEXAGON_M2_dpmpyuu_s0\0"
  "HEXAGON_M2_hmmpyh_rs1\0"
  "HEXAGON_M2_hmmpyh_s1\0"
  "HEXAGON_M2_hmmpyl_rs1\0"
  "HEXAGON_M2_hmmpyl_s1\0"
  "HEXAGON_M2_maci\0"
  "HEXAGON_M2_macsin\0"
  "HEXAGON_M2_macsip\0"
  "HEXAGON_M2_mmachs_rs0\0"
  "HEXAGON_M2_mmachs_rs1\0"
  "HEXAGON_M2_mmachs_s0\0"
  "HEXAGON_M2_mmachs_s1\0"
  "HEXAGON_M2_mmacls_rs0\0"
  "HEXAGON_M2_mmacls_rs1\0"
  "HEXAGON_M2_mmacls_s0\0"
  "HEXAGON_M2_mmacls_s1\0"
  "HEXAGON_M2_mmacuhs_rs0\0"
  "HEXAGON_M2_mmacuhs_rs1\0"
  "HEXAGON_M2_mmacuhs_s0\0"
  "HEXAGON_M2_mmacuhs_s1\0"
  "HEXAGON_M2_mmaculs_rs0\0"
  "HEXAGON_M2_mmaculs_rs1\0"
  "HEXAGON_M2_mmaculs_s0\0"
  "HEXAGON_M2_mmaculs_s1\0"
  "HEXAGON_M2_mmpyh_rs0\0"
  "HEXAGON_M2_mmpyh_rs1\0"
  "HEXAGON_M2_mmpyh_s0\0"
  "HEXAGON_M2_mmpyh_s1\0"
  "HEXAGON_M2_mmpyl_rs0\0"
  "HEXAGON_M2_mmpyl_rs1\0"
  "HEXAGON_M2_mmpyl_s0\0"
  "HEXAGON_M2_mmpyl_s1\0"
  "HEXAGON_M2_mmpyuh_rs0\0"
  "HEXAGON_M2_mmpyuh_rs1\0"
  "HEXAGON_M2_mmpyuh_s0\0"
  "HEXAGON_M2_mmpyuh_s1\0"
  "HEXAGON_M2_mmpyul_rs0\0"
  "HEXAGON_M2_mmpyul_rs1\0"
  "HEXAGON_M2_mmpyul_s0\0"
  "HEXAGON_M2_mmpyul_s1\0"
  "HEXAGON_M2_mnaci\0"
  "HEXAGON_M2_mpy_acc_hh_s0\0"
  "HEXAGON_M2_mpy_acc_hh_s1\0"
  "HEXAGON_M2_mpy_acc_hl_s0\0"
  "HEXAGON_M2_mpy_acc_hl_s1\0"
  "HEXAGON_M2_mpy_acc_lh_s0\0"
  "HEXAGON_M2_mpy_acc_lh_s1\0"
  "HEXAGON_M2_mpy_acc_ll_s0\0"
  "HEXAGON_M2_mpy_acc_ll_s1\0"
  "HEXAGON_M2_mpy_acc_sat_hh_s0\0"
  "HEXAGON_M2_mpy_acc_sat_hh_s1\0"
  "HEXAGON_M2_mpy_acc_sat_hl_s0\0"
  "HEXAGON_M2_mpy_acc_sat_hl_s1\0"
  "HEXAGON_M2_mpy_acc_sat_lh_s0\0"
  "HEXAGON_M2_mpy_acc_sat_lh_s1\0"
  "HEXAGON_M2_mpy_acc_sat_ll_s0\0"
  "HEXAGON_M2_mpy_acc_sat_ll_s1\0"
  "HEXAGON_M2_mpy_hh_s0\0"
  "HEXAGON_M2_mpy_hh_s1\0"
  "HEXAGON_M2_mpy_hl_s0\0"
  "HEXAGON_M2_mpy_hl_s1\0"
  "HEXAGON_M2_mpy_lh_s0\0"
  "HEXAGON_M2_mpy_lh_s1\0"
  "HEXAGON_M2_mpy_ll_s0\0"
  "HEXAGON_M2_mpy_ll_s1\0"
  "HEXAGON_M2_mpy_nac_hh_s0\0"
  "HEXAGON_M2_mpy_nac_hh_s1\0"
  "HEXAGON_M2_mpy_nac_hl_s0\0"
  "HEXAGON_M2_mpy_nac_hl_s1\0"
  "HEXAGON_M2_mpy_nac_lh_s0\0"
  "HEXAGON_M2_mpy_nac_lh_s1\0"
  "HEXAGON_M2_mpy_nac_ll_s0\0"
  "HEXAGON_M2_mpy_nac_ll_s1\0"
  "HEXAGON_M2_mpy_nac_sat_hh_s0\0"
  "HEXAGON_M2_mpy_nac_sat_hh_s1\0"
  "HEXAGON_M2_mpy_nac_sat_hl_s0\0"
  "HEXAGON_M2_mpy_nac_sat_hl_s1\0"
  "HEXAGON_M2_mpy_nac_sat_lh_s0\0"
  "HEXAGON_M2_mpy_nac_sat_lh_s1\0"
  "HEXAGON_M2_mpy_nac_sat_ll_s0\0"
  "HEXAGON_M2_mpy_nac_sat_ll_s1\0"
  "HEXAGON_M2_mpy_rnd_hh_s0\0"
  "HEXAGON_M2_mpy_rnd_hh_s1\0"
  "HEXAGON_M2_mpy_rnd_hl_s0\0"
  "HEXAGON_M2_mpy_rnd_hl_s1\0"
  "HEXAGON_M2_mpy_rnd_lh_s0\0"
  "HEXAGON_M2_mpy_rnd_lh_s1\0"
  "HEXAGON_M2_mpy_rnd_ll_s0\0"
  "HEXAGON_M2_mpy_rnd_ll_s1\0"
  "HEXAGON_M2_mpy_sat_hh_s0\0"
  "HEXAGON_M2_mpy_sat_hh_s1\0"
  "HEXAGON_M2_mpy_sat_hl_s0\0"
  "HEXAGON_M2_mpy_sat_hl_s1\0"
  "HEXAGON_M2_mpy_sat_lh_s0\0"
  "HEXAGON_M2_mpy_sat_lh_s1\0"
  "HEXAGON_M2_mpy_sat_ll_s0\0"
  "HEXAGON_M2_mpy_sat_ll_s1\0"
  "HEXAGON_M2_mpy_sat_rnd_hh_s0\0"
  "HEXAGON_M2_mpy_sat_rnd_hh_s1\0"
  "HEXAGON_M2_mpy_sat_rnd_hl_s0\0"
  "HEXAGON_M2_mpy_sat_rnd_hl_s1\0"
  "HEXAGON_M2_mpy_sat_rnd_lh_s0\0"
  "HEXAGON_M2_mpy_sat_rnd_lh_s1\0"
  "HEXAGON_M2_mpy_sat_rnd_ll_s0\0"
  "HEXAGON_M2_mpy_sat_rnd_ll_s1\0"
  "HEXAGON_M2_mpy_up\0"
  "HEXAGON_M2_mpy_up_s1\0"
  "HEXAGON_M2_mpy_up_s1_sat\0"
  "HEXAGON_M2_mpyd_acc_hh_s0\0"
  "HEXAGON_M2_mpyd_acc_hh_s1\0"
  "HEXAGON_M2_mpyd_acc_hl_s0\0"
  "HEXAGON_M2_mpyd_acc_hl_s1\0"
  "HEXAGON_M2_mpyd_acc_lh_s0\0"
  "HEXAGON_M2_mpyd_acc_lh_s1\0"
  "HEXAGON_M2_mpyd_acc_ll_s0\0"
  "HEXAGON_M2_mpyd_acc_ll_s1\0"
  "HEXAGON_M2_mpyd_hh_s0\0"
  "HEXAGON_M2_mpyd_hh_s1\0"
  "HEXAGON_M2_mpyd_hl_s0\0"
  "HEXAGON_M2_mpyd_hl_s1\0"
  "HEXAGON_M2_mpyd_lh_s0\0"
  "HEXAGON_M2_mpyd_lh_s1\0"
  "HEXAGON_M2_mpyd_ll_s0\0"
  "HEXAGON_M2_mpyd_ll_s1\0"
  "HEXAGON_M2_mpyd_nac_hh_s0\0"
  "HEXAGON_M2_mpyd_nac_hh_s1\0"
  "HEXAGON_M2_mpyd_nac_hl_s0\0"
  "HEXAGON_M2_mpyd_nac_hl_s1\0"
  "HEXAGON_M2_mpyd_nac_lh_s0\0"
  "HEXAGON_M2_mpyd_nac_lh_s1\0"
  "HEXAGON_M2_mpyd_nac_ll_s0\0"
  "HEXAGON_M2_mpyd_nac_ll_s1\0"
  "HEXAGON_M2_mpyd_rnd_hh_s0\0"
  "HEXAGON_M2_mpyd_rnd_hh_s1\0"
  "HEXAGON_M2_mpyd_rnd_hl_s0\0"
  "HEXAGON_M2_mpyd_rnd_hl_s1\0"
  "HEXAGON_M2_mpyd_rnd_lh_s0\0"
  "HEXAGON_M2_mpyd_rnd_lh_s1\0"
  "HEXAGON_M2_mpyd_rnd_ll_s0\0"
  "HEXAGON_M2_mpyd_rnd_ll_s1\0"
  "HEXAGON_M2_mpyi\0"
  "HEXAGON_M2_mpysmi\0"
  "HEXAGON_M2_mpysu_up\0"
  "HEXAGON_M2_mpyu_acc_hh_s0\0"
  "HEXAGON_M2_mpyu_acc_hh_s1\0"
  "HEXAGON_M2_mpyu_acc_hl_s0\0"
  "HEXAGON_M2_mpyu_acc_hl_s1\0"
  "HEXAGON_M2_mpyu_acc_lh_s0\0"
  "HEXAGON_M2_mpyu_acc_lh_s1\0"
  "HEXAGON_M2_mpyu_acc_ll_s0\0"
  "HEXAGON_M2_mpyu_acc_ll_s1\0"
  "HEXAGON_M2_mpyu_hh_s0\0"
  "HEXAGON_M2_mpyu_hh_s1\0"
  "HEXAGON_M2_mpyu_hl_s0\0"
  "HEXAGON_M2_mpyu_hl_s1\0"
  "HEXAGON_M2_mpyu_lh_s0\0"
  "HEXAGON_M2_mpyu_lh_s1\0"
  "HEXAGON_M2_mpyu_ll_s0\0"
  "HEXAGON_M2_mpyu_ll_s1\0"
  "HEXAGON_M2_mpyu_nac_hh_s0\0"
  "HEXAGON_M2_mpyu_nac_hh_s1\0"
  "HEXAGON_M2_mpyu_nac_hl_s0\0"
  "HEXAGON_M2_mpyu_nac_hl_s1\0"
  "HEXAGON_M2_mpyu_nac_lh_s0\0"
  "HEXAGON_M2_mpyu_nac_lh_s1\0"
  "HEXAGON_M2_mpyu_nac_ll_s0\0"
  "HEXAGON_M2_mpyu_nac_ll_s1\0"
  "HEXAGON_M2_mpyu_up\0"
  "HEXAGON_M2_mpyud_acc_hh_s0\0"
  "HEXAGON_M2_mpyud_acc_hh_s1\0"
  "HEXAGON_M2_mpyud_acc_hl_s0\0"
  "HEXAGON_M2_mpyud_acc_hl_s1\0"
  "HEXAGON_M2_mpyud_acc_lh_s0\0"
  "HEXAGON_M2_mpyud_acc_lh_s1\0"
  "HEXAGON_M2_mpyud_acc_ll_s0\0"
  "HEXAGON_M2_mpyud_acc_ll_s1\0"
  "HEXAGON_M2_mpyud_hh_s0\0"
  "HEXAGON_M2_mpyud_hh_s1\0"
  "HEXAGON_M2_mpyud_hl_s0\0"
  "HEXAGON_M2_mpyud_hl_s1\0"
  "HEXAGON_M2_mpyud_lh_s0\0"
  "HEXAGON_M2_mpyud_lh_s1\0"
  "HEXAGON_M2_mpyud_ll_s0\0"
  "HEXAGON_M2_mpyud_ll_s1\0"
  "HEXAGON_M2_mpyud_nac_hh_s0\0"
  "HEXAGON_M2_mpyud_nac_hh_s1\0"
  "HEXAGON_M2_mpyud_nac_hl_s0\0"
  "HEXAGON_M2_mpyud_nac_hl_s1\0"
  "HEXAGON_M2_mpyud_nac_lh_s0\0"
  "HEXAGON_M2_mpyud_nac_lh_s1\0"
  "HEXAGON_M2_mpyud_nac_ll_s0\0"
  "HEXAGON_M2_mpyud_nac_ll_s1\0"
  "HEXAGON_M2_mpyui\0"
  "HEXAGON_M2_nacci\0"
  "HEXAGON_M2_naccii\0"
  "HEXAGON_M2_subacc\0"
  "HEXAGON_M2_vabsdiffh\0"
  "HEXAGON_M2_vabsdiffw\0"
  "HEXAGON_M2_vcmac_s0_sat_i\0"
  "HEXAGON_M2_vcmac_s0_sat_r\0"
  "HEXAGON_M2_vcmpy_s0_sat_i\0"
  "HEXAGON_M2_vcmpy_s0_sat_r\0"
  "HEXAGON_M2_vcmpy_s1_sat_i\0"
  "HEXAGON_M2_vcmpy_s1_sat_r\0"
  "HEXAGON_M2_vdmacs_s0\0"
  "HEXAGON_M2_vdmacs_s1\0"
  "HEXAGON_M2_vdmpyrs_s0\0"
  "HEXAGON_M2_vdmpyrs_s1\0"
  "HEXAGON_M2_vdmpys_s0\0"
  "HEXAGON_M2_vdmpys_s1\0"
  "HEXAGON_M2_vmac2\0"
  "HEXAGON_M2_vmac2es\0"
  "HEXAGON_M2_vmac2es_s0\0"
  "HEXAGON_M2_vmac2es_s1\0"
  "HEXAGON_M2_vmac2s_s0\0"
  "HEXAGON_M2_vmac2s_s1\0"
  "HEXAGON_M2_vmac2su_s0\0"
  "HEXAGON_M2_vmac2su_s1\0"
  "HEXAGON_M2_vmpy2es_s0\0"
  "HEXAGON_M2_vmpy2es_s1\0"
  "HEXAGON_M2_vmpy2s_s0\0"
  "HEXAGON_M2_vmpy2s_s0pack\0"
  "HEXAGON_M2_vmpy2s_s1\0"
  "HEXAGON_M2_vmpy2s_s1pack\0"
  "HEXAGON_M2_vmpy2su_s0\0"
  "HEXAGON_M2_vmpy2su_s1\0"
  "HEXAGON_M2_vraddh\0"
  "HEXAGON_M2_vradduh\0"
  "HEXAGON_M2_vrcmaci_s0\0"
  "HEXAGON_M2_vrcmaci_s0c\0"
  "HEXAGON_M2_vrcmacr_s0\0"
  "HEXAGON_M2_vrcmacr_s0c\0"
  "HEXAGON_M2_vrcmpyi_s0\0"
  "HEXAGON_M2_vrcmpyi_s0c\0"
  "HEXAGON_M2_vrcmpyr_s0\0"
  "HEXAGON_M2_vrcmpyr_s0c\0"
  "HEXAGON_M2_vrcmpys_acc_s1\0"
  "HEXAGON_M2_vrcmpys_s1\0"
  "HEXAGON_M2_vrcmpys_s1rp\0"
  "HEXAGON_M2_vrmac_s0\0"
  "HEXAGON_M2_vrmpy_s0\0"
  "HEXAGON_M2_xor_xacc\0"
  "HEXAGON_M4_and_and\0"
  "HEXAGON_M4_and_andn\0"
  "HEXAGON_M4_and_or\0"
  "HEXAGON_M4_and_xor\0"
  "HEXAGON_M4_cmpyi_wh\0"
  "HEXAGON_M4_cmpyi_whc\0"
  "HEXAGON_M4_cmpyr_wh\0"
  "HEXAGON_M4_cmpyr_whc\0"
  "HEXAGON_M4_mac_up_s1_sat\0"
  "HEXAGON_M4_mpyri_addi\0"
  "HEXAGON_M4_mpyri_addr\0"
  "HEXAGON_M4_mpyri_addr_u2\0"
  "HEXAGON_M4_mpyrr_addi\0"
  "HEXAGON_M4_mpyrr_addr\0"
  "HEXAGON_M4_nac_up_s1_sat\0"
  "HEXAGON_M4_or_and\0"
  "HEXAGON_M4_or_andn\0"
  "HEXAGON_M4_or_or\0"
  "HEXAGON_M4_or_xor\0"
  "HEXAGON_M4_pmpyw\0"
  "HEXAGON_M4_pmpyw_acc\0"
  "HEXAGON_M4_vpmpyh\0"
  "HEXAGON_M4_vpmpyh_acc\0"
  "HEXAGON_M4_vrmpyeh_acc_s0\0"
  "HEXAGON_M4_vrmpyeh_acc_s1\0"
  "HEXAGON_M4_vrmpyeh_s0\0"
  "HEXAGON_M4_vrmpyeh_s1\0"
  "HEXAGON_M4_vrmpyoh_acc_s0\0"
  "HEXAGON_M4_vrmpyoh_acc_s1\0"
  "HEXAGON_M4_vrmpyoh_s0\0"
  "HEXAGON_M4_vrmpyoh_s1\0"
  "HEXAGON_M4_xor_and\0"
  "HEXAGON_M4_xor_andn\0"
  "HEXAGON_M4_xor_or\0"
  "HEXAGON_M4_xor_xacc\0"
  "HEXAGON_M5_vdmacbsu\0"
  "HEXAGON_M5_vdmpybsu\0"
  "HEXAGON_M5_vmacbsu\0"
  "HEXAGON_M5_vmacbuu\0"
  "HEXAGON_M5_vmpybsu\0"
  "HEXAGON_M5_vmpybuu\0"
  "HEXAGON_M5_vrmacbsu\0"
  "HEXAGON_M5_vrmacbuu\0"
  "HEXAGON_M5_vrmpybsu\0"
  "HEXAGON_M5_vrmpybuu\0"
  "HEXAGON_M6_vabsdiffb\0"
  "HEXAGON_M6_vabsdiffub\0"
  "HEXAGON_M7_dcmpyiw\0"
  "HEXAGON_M7_dcmpyiw_acc\0"
  "HEXAGON_M7_dcmpyiwc\0"
  "HEXAGON_M7_dcmpyiwc_acc\0"
  "HEXAGON_M7_dcmpyrw\0"
  "HEXAGON_M7_dcmpyrw_acc\0"
  "HEXAGON_M7_dcmpyrwc\0"
  "HEXAGON_M7_dcmpyrwc_acc\0"
  "HEXAGON_M7_vdmpy\0"
  "HEXAGON_M7_vdmpy_acc\0"
  "HEXAGON_M7_wcmpyiw\0"
  "HEXAGON_M7_wcmpyiw_rnd\0"
  "HEXAGON_M7_wcmpyiwc\0"
  "HEXAGON_M7_wcmpyiwc_rnd\0"
  "HEXAGON_M7_wcmpyrw\0"
  "HEXAGON_M7_wcmpyrw_rnd\0"
  "HEXAGON_M7_wcmpyrwc\0"
  "HEXAGON_M7_wcmpyrwc_rnd\0"
  "HEXAGON_S2_addasl_rrri\0"
  "HEXAGON_S2_asl_i_p\0"
  "HEXAGON_S2_asl_i_p_acc\0"
  "HEXAGON_S2_asl_i_p_and\0"
  "HEXAGON_S2_asl_i_p_nac\0"
  "HEXAGON_S2_asl_i_p_or\0"
  "HEXAGON_S2_asl_i_p_xacc\0"
  "HEXAGON_S2_asl_i_r\0"
  "HEXAGON_S2_asl_i_r_acc\0"
  "HEXAGON_S2_asl_i_r_and\0"
  "HEXAGON_S2_asl_i_r_nac\0"
  "HEXAGON_S2_asl_i_r_or\0"
  "HEXAGON_S2_asl_i_r_sat\0"
  "HEXAGON_S2_asl_i_r_xacc\0"
  "HEXAGON_S2_asl_i_vh\0"
  "HEXAGON_S2_asl_i_vw\0"
  "HEXAGON_S2_asl_r_p\0"
  "HEXAGON_S2_asl_r_p_acc\0"
  "HEXAGON_S2_asl_r_p_and\0"
  "HEXAGON_S2_asl_r_p_nac\0"
  "HEXAGON_S2_asl_r_p_or\0"
  "HEXAGON_S2_asl_r_p_xor\0"
  "HEXAGON_S2_asl_r_r\0"
  "HEXAGON_S2_asl_r_r_acc\0"
  "HEXAGON_S2_asl_r_r_and\0"
  "HEXAGON_S2_asl_r_r_nac\0"
  "HEXAGON_S2_asl_r_r_or\0"
  "HEXAGON_S2_asl_r_r_sat\0"
  "HEXAGON_S2_asl_r_vh\0"
  "HEXAGON_S2_asl_r_vw\0"
  "HEXAGON_S2_asr_i_p\0"
  "HEXAGON_S2_asr_i_p_acc\0"
  "HEXAGON_S2_asr_i_p_and\0"
  "HEXAGON_S2_asr_i_p_nac\0"
  "HEXAGON_S2_asr_i_p_or\0"
  "HEXAGON_S2_asr_i_p_rnd\0"
  "HEXAGON_S2_asr_i_p_rnd_goodsyntax\0"
  "HEXAGON_S2_asr_i_r\0"
  "HEXAGON_S2_asr_i_r_acc\0"
  "HEXAGON_S2_asr_i_r_and\0"
  "HEXAGON_S2_asr_i_r_nac\0"
  "HEXAGON_S2_asr_i_r_or\0"
  "HEXAGON_S2_asr_i_r_rnd\0"
  "HEXAGON_S2_asr_i_r_rnd_goodsyntax\0"
  "HEXAGON_S2_asr_i_svw_trun\0"
  "HEXAGON_S2_asr_i_vh\0"
  "HEXAGON_S2_asr_i_vw\0"
  "HEXAGON_S2_asr_r_p\0"
  "HEXAGON_S2_asr_r_p_acc\0"
  "HEXAGON_S2_asr_r_p_and\0"
  "HEXAGON_S2_asr_r_p_nac\0"
  "HEXAGON_S2_asr_r_p_or\0"
  "HEXAGON_S2_asr_r_p_xor\0"
  "HEXAGON_S2_asr_r_r\0"
  "HEXAGON_S2_asr_r_r_acc\0"
  "HEXAGON_S2_asr_r_r_and\0"
  "HEXAGON_S2_asr_r_r_nac\0"
  "HEXAGON_S2_asr_r_r_or\0"
  "HEXAGON_S2_asr_r_r_sat\0"
  "HEXAGON_S2_asr_r_svw_trun\0"
  "HEXAGON_S2_asr_r_vh\0"
  "HEXAGON_S2_asr_r_vw\0"
  "HEXAGON_S2_brev\0"
  "HEXAGON_S2_brevp\0"
  "HEXAGON_S2_cl0\0"
  "HEXAGON_S2_cl0p\0"
  "HEXAGON_S2_cl1\0"
  "HEXAGON_S2_cl1p\0"
  "HEXAGON_S2_clb\0"
  "HEXAGON_S2_clbnorm\0"
  "HEXAGON_S2_clbp\0"
  "HEXAGON_S2_clrbit_i\0"
  "HEXAGON_S2_clrbit_r\0"
  "HEXAGON_S2_ct0\0"
  "HEXAGON_S2_ct0p\0"
  "HEXAGON_S2_ct1\0"
  "HEXAGON_S2_ct1p\0"
  "HEXAGON_S2_deinterleave\0"
  "HEXAGON_S2_extractu\0"
  "HEXAGON_S2_extractu_rp\0"
  "HEXAGON_S2_extractup\0"
  "HEXAGON_S2_extractup_rp\0"
  "HEXAGON_S2_insert\0"
  "HEXAGON_S2_insert_rp\0"
  "HEXAGON_S2_insertp\0"
  "HEXAGON_S2_insertp_rp\0"
  "HEXAGON_S2_interleave\0"
  "HEXAGON_S2_lfsp\0"
  "HEXAGON_S2_lsl_r_p\0"
  "HEXAGON_S2_lsl_r_p_acc\0"
  "HEXAGON_S2_lsl_r_p_and\0"
  "HEXAGON_S2_lsl_r_p_nac\0"
  "HEXAGON_S2_lsl_r_p_or\0"
  "HEXAGON_S2_lsl_r_p_xor\0"
  "HEXAGON_S2_lsl_r_r\0"
  "HEXAGON_S2_lsl_r_r_acc\0"
  "HEXAGON_S2_lsl_r_r_and\0"
  "HEXAGON_S2_lsl_r_r_nac\0"
  "HEXAGON_S2_lsl_r_r_or\0"
  "HEXAGON_S2_lsl_r_vh\0"
  "HEXAGON_S2_lsl_r_vw\0"
  "HEXAGON_S2_lsr_i_p\0"
  "HEXAGON_S2_lsr_i_p_acc\0"
  "HEXAGON_S2_lsr_i_p_and\0"
  "HEXAGON_S2_lsr_i_p_nac\0"
  "HEXAGON_S2_lsr_i_p_or\0"
  "HEXAGON_S2_lsr_i_p_xacc\0"
  "HEXAGON_S2_lsr_i_r\0"
  "HEXAGON_S2_lsr_i_r_acc\0"
  "HEXAGON_S2_lsr_i_r_and\0"
  "HEXAGON_S2_lsr_i_r_nac\0"
  "HEXAGON_S2_lsr_i_r_or\0"
  "HEXAGON_S2_lsr_i_r_xacc\0"
  "HEXAGON_S2_lsr_i_vh\0"
  "HEXAGON_S2_lsr_i_vw\0"
  "HEXAGON_S2_lsr_r_p\0"
  "HEXAGON_S2_lsr_r_p_acc\0"
  "HEXAGON_S2_lsr_r_p_and\0"
  "HEXAGON_S2_lsr_r_p_nac\0"
  "HEXAGON_S2_lsr_r_p_or\0"
  "HEXAGON_S2_lsr_r_p_xor\0"
  "HEXAGON_S2_lsr_r_r\0"
  "HEXAGON_S2_lsr_r_r_acc\0"
  "HEXAGON_S2_lsr_r_r_and\0"
  "HEXAGON_S2_lsr_r_r_nac\0"
  "HEXAGON_S2_lsr_r_r_or\0"
  "HEXAGON_S2_lsr_r_vh\0"
  "HEXAGON_S2_lsr_r_vw\0"
  "HEXAGON_S2_mask\0"
  "HEXAGON_S2_packhl\0"
  "HEXAGON_S2_parityp\0"
  "HEXAGON_S2_setbit_i\0"
  "HEXAGON_S2_setbit_r\0"
  "HEXAGON_S2_shuffeb\0"
  "HEXAGON_S2_shuffeh\0"
  "HEXAGON_S2_shuffob\0"
  "HEXAGON_S2_shuffoh\0"
  "HEXAGON_S2_storew_locked\0"
  "HEXAGON_S2_svsathb\0"
  "HEXAGON_S2_svsathub\0"
  "HEXAGON_S2_tableidxb_goodsyntax\0"
  "HEXAGON_S2_tableidxd_goodsyntax\0"
  "HEXAGON_S2_tableidxh_goodsyntax\0"
  "HEXAGON_S2_tableidxw_goodsyntax\0"
  "HEXAGON_S2_togglebit_i\0"
  "HEXAGON_S2_togglebit_r\0"
  "HEXAGON_S2_tstbit_i\0"
  "HEXAGON_S2_tstbit_r\0"
  "HEXAGON_S2_valignib\0"
  "HEXAGON_S2_valignrb\0"
  "HEXAGON_S2_vcnegh\0"
  "HEXAGON_S2_vcrotate\0"
  "HEXAGON_S2_vrcnegh\0"
  "HEXAGON_S2_vrndpackwh\0"
  "HEXAGON_S2_vrndpackwhs\0"
  "HEXAGON_S2_vsathb\0"
  "HEXAGON_S2_vsathb_nopack\0"
  "HEXAGON_S2_vsathub\0"
  "HEXAGON_S2_vsathub_nopack\0"
  "HEXAGON_S2_vsatwh\0"
  "HEXAGON_S2_vsatwh_nopack\0"
  "HEXAGON_S2_vsatwuh\0"
  "HEXAGON_S2_vsatwuh_nopack\0"
  "HEXAGON_S2_vsplatrb\0"
  "HEXAGON_S2_vsplatrh\0"
  "HEXAGON_S2_vspliceib\0"
  "HEXAGON_S2_vsplicerb\0"
  "HEXAGON_S2_vsxtbh\0"
  "HEXAGON_S2_vsxthw\0"
  "HEXAGON_S2_vtrunehb\0"
  "HEXAGON_S2_vtrunewh\0"
  "HEXAGON_S2_vtrunohb\0"
  "HEXAGON_S2_vtrunowh\0"
  "HEXAGON_S2_vzxtbh\0"
  "HEXAGON_S2_vzxthw\0"
  "HEXAGON_S4_addaddi\0"
  "HEXAGON_S4_addi_asl_ri\0"
  "HEXAGON_S4_addi_lsr_ri\0"
  "HEXAGON_S4_andi_asl_ri\0"
  "HEXAGON_S4_andi_lsr_ri\0"
  "HEXAGON_S4_clbaddi\0"
  "HEXAGON_S4_clbpaddi\0"
  "HEXAGON_S4_clbpnorm\0"
  "HEXAGON_S4_extract\0"
  "HEXAGON_S4_extract_rp\0"
  "HEXAGON_S4_extractp\0"
  "HEXAGON_S4_extractp_rp\0"
  "HEXAGON_S4_lsli\0"
  "HEXAGON_S4_ntstbit_i\0"
  "HEXAGON_S4_ntstbit_r\0"
  "HEXAGON_S4_or_andi\0"
  "HEXAGON_S4_or_andix\0"
  "HEXAGON_S4_or_ori\0"
  "HEXAGON_S4_ori_asl_ri\0"
  "HEXAGON_S4_ori_lsr_ri\0"
  "HEXAGON_S4_parity\0"
  "HEXAGON_S4_stored_locked\0"
  "HEXAGON_S4_subaddi\0"
  "HEXAGON_S4_subi_asl_ri\0"
  "HEXAGON_S4_subi_lsr_ri\0"
  "HEXAGON_S4_vrcrotate\0"
  "HEXAGON_S4_vrcrotate_acc\0"
  "HEXAGON_S4_vxaddsubh\0"
  "HEXAGON_S4_vxaddsubhr\0"
  "HEXAGON_S4_vxaddsubw\0"
  "HEXAGON_S4_vxsubaddh\0"
  "HEXAGON_S4_vxsubaddhr\0"
  "HEXAGON_S4_vxsubaddw\0"
  "HEXAGON_S5_asrhub_rnd_sat_goodsyntax\0"
  "HEXAGON_S5_asrhub_sat\0"
  "HEXAGON_S5_popcountp\0"
  "HEXAGON_S5_vasrhrnd_goodsyntax\0"
  "HEXAGON_S6_rol_i_p\0"
  "HEXAGON_S6_rol_i_p_acc\0"
  "HEXAGON_S6_rol_i_p_and\0"
  "HEXAGON_S6_rol_i_p_nac\0"
  "HEXAGON_S6_rol_i_p_or\0"
  "HEXAGON_S6_rol_i_p_xacc\0"
  "HEXAGON_S6_rol_i_r\0"
  "HEXAGON_S6_rol_i_r_acc\0"
  "HEXAGON_S6_rol_i_r_and\0"
  "HEXAGON_S6_rol_i_r_nac\0"
  "HEXAGON_S6_rol_i_r_or\0"
  "HEXAGON_S6_rol_i_r_xacc\0"
  "HEXAGON_S6_vsplatrbp\0"
  "HEXAGON_S6_vtrunehb_ppp\0"
  "HEXAGON_S6_vtrunohb_ppp\0"
  "HEXAGON_V6_extractw\0"
  "HEXAGON_V6_extractw_128B\0"
  "HEXAGON_V6_get_qfext\0"
  "HEXAGON_V6_get_qfext_128B\0"
  "HEXAGON_V6_get_qfext_oracc\0"
  "HEXAGON_V6_get_qfext_oracc_128B\0"
  "HEXAGON_V6_hi\0"
  "HEXAGON_V6_hi_128B\0"
  "HEXAGON_V6_lo\0"
  "HEXAGON_V6_lo_128B\0"
  "HEXAGON_V6_lvsplatb\0"
  "HEXAGON_V6_lvsplatb_128B\0"
  "HEXAGON_V6_lvsplath\0"
  "HEXAGON_V6_lvsplath_128B\0"
  "HEXAGON_V6_lvsplatw\0"
  "HEXAGON_V6_lvsplatw_128B\0"
  "HEXAGON_V6_pred_and\0"
  "HEXAGON_V6_pred_and_128B\0"
  "HEXAGON_V6_pred_and_n\0"
  "HEXAGON_V6_pred_and_n_128B\0"
  "HEXAGON_V6_pred_not\0"
  "HEXAGON_V6_pred_not_128B\0"
  "HEXAGON_V6_pred_or\0"
  "HEXAGON_V6_pred_or_128B\0"
  "HEXAGON_V6_pred_or_n\0"
  "HEXAGON_V6_pred_or_n_128B\0"
  "HEXAGON_V6_pred_scalar2\0"
  "HEXAGON_V6_pred_scalar2_128B\0"
  "HEXAGON_V6_pred_scalar2v2\0"
  "HEXAGON_V6_pred_scalar2v2_128B\0"
  "HEXAGON_V6_pred_xor\0"
  "HEXAGON_V6_pred_xor_128B\0"
  "HEXAGON_V6_set_qfext\0"
  "HEXAGON_V6_set_qfext_128B\0"
  "HEXAGON_V6_shuffeqh\0"
  "HEXAGON_V6_shuffeqh_128B\0"
  "HEXAGON_V6_shuffeqw\0"
  "HEXAGON_V6_shuffeqw_128B\0"
  "HEXAGON_V6_v6mpyhubs10\0"
  "HEXAGON_V6_v6mpyhubs10_128B\0"
  "HEXAGON_V6_v6mpyhubs10_vxx\0"
  "HEXAGON_V6_v6mpyhubs10_vxx_128B\0"
  "HEXAGON_V6_v6mpyvubs10\0"
  "HEXAGON_V6_v6mpyvubs10_128B\0"
  "HEXAGON_V6_v6mpyvubs10_vxx\0"
  "HEXAGON_V6_v6mpyvubs10_vxx_128B\0"
  "HEXAGON_V6_vS32b_nqpred_ai\0"
  "HEXAGON_V6_vS32b_nqpred_ai_128B\0"
  "HEXAGON_V6_vS32b_nt_nqpred_ai\0"
  "HEXAGON_V6_vS32b_nt_nqpred_ai_128B\0"
  "HEXAGON_V6_vS32b_nt_qpred_ai\0"
  "HEXAGON_V6_vS32b_nt_qpred_ai_128B\0"
  "HEXAGON_V6_vS32b_qpred_ai\0"
  "HEXAGON_V6_vS32b_qpred_ai_128B\0"
  "HEXAGON_V6_vabs_f8\0"
  "HEXAGON_V6_vabs_f8_128B\0"
  "HEXAGON_V6_vabs_hf\0"
  "HEXAGON_V6_vabs_hf_128B\0"
  "HEXAGON_V6_vabs_qf16_hf\0"
  "HEXAGON_V6_vabs_qf16_hf_128B\0"
  "HEXAGON_V6_vabs_qf16_qf16\0"
  "HEXAGON_V6_vabs_qf16_qf16_128B\0"
  "HEXAGON_V6_vabs_qf32_qf32\0"
  "HEXAGON_V6_vabs_qf32_qf32_128B\0"
  "HEXAGON_V6_vabs_qf32_sf\0"
  "HEXAGON_V6_vabs_qf32_sf_128B\0"
  "HEXAGON_V6_vabs_sf\0"
  "HEXAGON_V6_vabs_sf_128B\0"
  "HEXAGON_V6_vabsb\0"
  "HEXAGON_V6_vabsb_128B\0"
  "HEXAGON_V6_vabsb_sat\0"
  "HEXAGON_V6_vabsb_sat_128B\0"
  "HEXAGON_V6_vabsdiffh\0"
  "HEXAGON_V6_vabsdiffh_128B\0"
  "HEXAGON_V6_vabsdiffub\0"
  "HEXAGON_V6_vabsdiffub_128B\0"
  "HEXAGON_V6_vabsdiffuh\0"
  "HEXAGON_V6_vabsdiffuh_128B\0"
  "HEXAGON_V6_vabsdiffw\0"
  "HEXAGON_V6_vabsdiffw_128B\0"
  "HEXAGON_V6_vabsh\0"
  "HEXAGON_V6_vabsh_128B\0"
  "HEXAGON_V6_vabsh_sat\0"
  "HEXAGON_V6_vabsh_sat_128B\0"
  "HEXAGON_V6_vabsw\0"
  "HEXAGON_V6_vabsw_128B\0"
  "HEXAGON_V6_vabsw_sat\0"
  "HEXAGON_V6_vabsw_sat_128B\0"
  "HEXAGON_V6_vadd_hf\0"
  "HEXAGON_V6_vadd_hf_128B\0"
  "HEXAGON_V6_vadd_hf_f8\0"
  "HEXAGON_V6_vadd_hf_f8_128B\0"
  "HEXAGON_V6_vadd_hf_hf\0"
  "HEXAGON_V6_vadd_hf_hf_128B\0"
  "HEXAGON_V6_vadd_qf16\0"
  "HEXAGON_V6_vadd_qf16_128B\0"
  "HEXAGON_V6_vadd_qf16_mix\0"
  "HEXAGON_V6_vadd_qf16_mix_128B\0"
  "HEXAGON_V6_vadd_qf32\0"
  "HEXAGON_V6_vadd_qf32_128B\0"
  "HEXAGON_V6_vadd_qf32_mix\0"
  "HEXAGON_V6_vadd_qf32_mix_128B\0"
  "HEXAGON_V6_vadd_sf\0"
  "HEXAGON_V6_vadd_sf_128B\0"
  "HEXAGON_V6_vadd_sf_bf\0"
  "HEXAGON_V6_vadd_sf_bf_128B\0"
  "HEXAGON_V6_vadd_sf_hf\0"
  "HEXAGON_V6_vadd_sf_hf_128B\0"
  "HEXAGON_V6_vadd_sf_sf\0"
  "HEXAGON_V6_vadd_sf_sf_128B\0"
  "HEXAGON_V6_vaddb\0"
  "HEXAGON_V6_vaddb_128B\0"
  "HEXAGON_V6_vaddb_dv\0"
  "HEXAGON_V6_vaddb_dv_128B\0"
  "HEXAGON_V6_vaddbnq\0"
  "HEXAGON_V6_vaddbnq_128B\0"
  "HEXAGON_V6_vaddbq\0"
  "HEXAGON_V6_vaddbq_128B\0"
  "HEXAGON_V6_vaddbsat\0"
  "HEXAGON_V6_vaddbsat_128B\0"
  "HEXAGON_V6_vaddbsat_dv\0"
  "HEXAGON_V6_vaddbsat_dv_128B\0"
  "HEXAGON_V6_vaddcarrysat\0"
  "HEXAGON_V6_vaddcarrysat_128B\0"
  "HEXAGON_V6_vaddclbh\0"
  "HEXAGON_V6_vaddclbh_128B\0"
  "HEXAGON_V6_vaddclbw\0"
  "HEXAGON_V6_vaddclbw_128B\0"
  "HEXAGON_V6_vaddh\0"
  "HEXAGON_V6_vaddh_128B\0"
  "HEXAGON_V6_vaddh_dv\0"
  "HEXAGON_V6_vaddh_dv_128B\0"
  "HEXAGON_V6_vaddhnq\0"
  "HEXAGON_V6_vaddhnq_128B\0"
  "HEXAGON_V6_vaddhq\0"
  "HEXAGON_V6_vaddhq_128B\0"
  "HEXAGON_V6_vaddhsat\0"
  "HEXAGON_V6_vaddhsat_128B\0"
  "HEXAGON_V6_vaddhsat_dv\0"
  "HEXAGON_V6_vaddhsat_dv_128B\0"
  "HEXAGON_V6_vaddhw\0"
  "HEXAGON_V6_vaddhw_128B\0"
  "HEXAGON_V6_vaddhw_acc\0"
  "HEXAGON_V6_vaddhw_acc_128B\0"
  "HEXAGON_V6_vaddubh\0"
  "HEXAGON_V6_vaddubh_128B\0"
  "HEXAGON_V6_vaddubh_acc\0"
  "HEXAGON_V6_vaddubh_acc_128B\0"
  "HEXAGON_V6_vaddubsat\0"
  "HEXAGON_V6_vaddubsat_128B\0"
  "HEXAGON_V6_vaddubsat_dv\0"
  "HEXAGON_V6_vaddubsat_dv_128B\0"
  "HEXAGON_V6_vaddububb_sat\0"
  "HEXAGON_V6_vaddububb_sat_128B\0"
  "HEXAGON_V6_vadduhsat\0"
  "HEXAGON_V6_vadduhsat_128B\0"
  "HEXAGON_V6_vadduhsat_dv\0"
  "HEXAGON_V6_vadduhsat_dv_128B\0"
  "HEXAGON_V6_vadduhw\0"
  "HEXAGON_V6_vadduhw_128B\0"
  "HEXAGON_V6_vadduhw_acc\0"
  "HEXAGON_V6_vadduhw_acc_128B\0"
  "HEXAGON_V6_vadduwsat\0"
  "HEXAGON_V6_vadduwsat_128B\0"
  "HEXAGON_V6_vadduwsat_dv\0"
  "HEXAGON_V6_vadduwsat_dv_128B\0"
  "HEXAGON_V6_vaddw\0"
  "HEXAGON_V6_vaddw_128B\0"
  "HEXAGON_V6_vaddw_dv\0"
  "HEXAGON_V6_vaddw_dv_128B\0"
  "HEXAGON_V6_vaddwnq\0"
  "HEXAGON_V6_vaddwnq_128B\0"
  "HEXAGON_V6_vaddwq\0"
  "HEXAGON_V6_vaddwq_128B\0"
  "HEXAGON_V6_vaddwsat\0"
  "HEXAGON_V6_vaddwsat_128B\0"
  "HEXAGON_V6_vaddwsat_dv\0"
  "HEXAGON_V6_vaddwsat_dv_128B\0"
  "HEXAGON_V6_valign4\0"
  "HEXAGON_V6_valign4_128B\0"
  "HEXAGON_V6_valignb\0"
  "HEXAGON_V6_valignb_128B\0"
  "HEXAGON_V6_valignbi\0"
  "HEXAGON_V6_valignbi_128B\0"
  "HEXAGON_V6_vand\0"
  "HEXAGON_V6_vand_128B\0"
  "HEXAGON_V6_vandnqrt\0"
  "HEXAGON_V6_vandnqrt_128B\0"
  "HEXAGON_V6_vandnqrt_acc\0"
  "HEXAGON_V6_vandnqrt_acc_128B\0"
  "HEXAGON_V6_vandqrt\0"
  "HEXAGON_V6_vandqrt_128B\0"
  "HEXAGON_V6_vandqrt_acc\0"
  "HEXAGON_V6_vandqrt_acc_128B\0"
  "HEXAGON_V6_vandvnqv\0"
  "HEXAGON_V6_vandvnqv_128B\0"
  "HEXAGON_V6_vandvqv\0"
  "HEXAGON_V6_vandvqv_128B\0"
  "HEXAGON_V6_vandvrt\0"
  "HEXAGON_V6_vandvrt_128B\0"
  "HEXAGON_V6_vandvrt_acc\0"
  "HEXAGON_V6_vandvrt_acc_128B\0"
  "HEXAGON_V6_vaslh\0"
  "HEXAGON_V6_vaslh_128B\0"
  "HEXAGON_V6_vaslh_acc\0"
  "HEXAGON_V6_vaslh_acc_128B\0"
  "HEXAGON_V6_vaslhv\0"
  "HEXAGON_V6_vaslhv_128B\0"
  "HEXAGON_V6_vaslw\0"
  "HEXAGON_V6_vaslw_128B\0"
  "HEXAGON_V6_vaslw_acc\0"
  "HEXAGON_V6_vaslw_acc_128B\0"
  "HEXAGON_V6_vaslwv\0"
  "HEXAGON_V6_vaslwv_128B\0"
  "HEXAGON_V6_vasr_into\0"
  "HEXAGON_V6_vasr_into_128B\0"
  "HEXAGON_V6_vasrh\0"
  "HEXAGON_V6_vasrh_128B\0"
  "HEXAGON_V6_vasrh_acc\0"
  "HEXAGON_V6_vasrh_acc_128B\0"
  "HEXAGON_V6_vasrhbrndsat\0"
  "HEXAGON_V6_vasrhbrndsat_128B\0"
  "HEXAGON_V6_vasrhbsat\0"
  "HEXAGON_V6_vasrhbsat_128B\0"
  "HEXAGON_V6_vasrhubrndsat\0"
  "HEXAGON_V6_vasrhubrndsat_128B\0"
  "HEXAGON_V6_vasrhubsat\0"
  "HEXAGON_V6_vasrhubsat_128B\0"
  "HEXAGON_V6_vasrhv\0"
  "HEXAGON_V6_vasrhv_128B\0"
  "HEXAGON_V6_vasruhubrndsat\0"
  "HEXAGON_V6_vasruhubrndsat_128B\0"
  "HEXAGON_V6_vasruhubsat\0"
  "HEXAGON_V6_vasruhubsat_128B\0"
  "HEXAGON_V6_vasruwuhrndsat\0"
  "HEXAGON_V6_vasruwuhrndsat_128B\0"
  "HEXAGON_V6_vasruwuhsat\0"
  "HEXAGON_V6_vasruwuhsat_128B\0"
  "HEXAGON_V6_vasrvuhubrndsat\0"
  "HEXAGON_V6_vasrvuhubrndsat_128B\0"
  "HEXAGON_V6_vasrvuhubsat\0"
  "HEXAGON_V6_vasrvuhubsat_128B\0"
  "HEXAGON_V6_vasrvwuhrndsat\0"
  "HEXAGON_V6_vasrvwuhrndsat_128B\0"
  "HEXAGON_V6_vasrvwuhsat\0"
  "HEXAGON_V6_vasrvwuhsat_128B\0"
  "HEXAGON_V6_vasrw\0"
  "HEXAGON_V6_vasrw_128B\0"
  "HEXAGON_V6_vasrw_acc\0"
  "HEXAGON_V6_vasrw_acc_128B\0"
  "HEXAGON_V6_vasrwh\0"
  "HEXAGON_V6_vasrwh_128B\0"
  "HEXAGON_V6_vasrwhrndsat\0"
  "HEXAGON_V6_vasrwhrndsat_128B\0"
  "HEXAGON_V6_vasrwhsat\0"
  "HEXAGON_V6_vasrwhsat_128B\0"
  "HEXAGON_V6_vasrwuhrndsat\0"
  "HEXAGON_V6_vasrwuhrndsat_128B\0"
  "HEXAGON_V6_vasrwuhsat\0"
  "HEXAGON_V6_vasrwuhsat_128B\0"
  "HEXAGON_V6_vasrwv\0"
  "HEXAGON_V6_vasrwv_128B\0"
  "HEXAGON_V6_vassign\0"
  "HEXAGON_V6_vassign_128B\0"
  "HEXAGON_V6_vassign_fp\0"
  "HEXAGON_V6_vassign_fp_128B\0"
  "HEXAGON_V6_vassignp\0"
  "HEXAGON_V6_vassignp_128B\0"
  "HEXAGON_V6_vavgb\0"
  "HEXAGON_V6_vavgb_128B\0"
  "HEXAGON_V6_vavgbrnd\0"
  "HEXAGON_V6_vavgbrnd_128B\0"
  "HEXAGON_V6_vavgh\0"
  "HEXAGON_V6_vavgh_128B\0"
  "HEXAGON_V6_vavghrnd\0"
  "HEXAGON_V6_vavghrnd_128B\0"
  "HEXAGON_V6_vavgub\0"
  "HEXAGON_V6_vavgub_128B\0"
  "HEXAGON_V6_vavgubrnd\0"
  "HEXAGON_V6_vavgubrnd_128B\0"
  "HEXAGON_V6_vavguh\0"
  "HEXAGON_V6_vavguh_128B\0"
  "HEXAGON_V6_vavguhrnd\0"
  "HEXAGON_V6_vavguhrnd_128B\0"
  "HEXAGON_V6_vavguw\0"
  "HEXAGON_V6_vavguw_128B\0"
  "HEXAGON_V6_vavguwrnd\0"
  "HEXAGON_V6_vavguwrnd_128B\0"
  "HEXAGON_V6_vavgw\0"
  "HEXAGON_V6_vavgw_128B\0"
  "HEXAGON_V6_vavgwrnd\0"
  "HEXAGON_V6_vavgwrnd_128B\0"
  "HEXAGON_V6_vcl0h\0"
  "HEXAGON_V6_vcl0h_128B\0"
  "HEXAGON_V6_vcl0w\0"
  "HEXAGON_V6_vcl0w_128B\0"
  "HEXAGON_V6_vcombine\0"
  "HEXAGON_V6_vcombine_128B\0"
  "HEXAGON_V6_vconv_bf_qf32\0"
  "HEXAGON_V6_vconv_bf_qf32_128B\0"
  "HEXAGON_V6_vconv_f8_qf16\0"
  "HEXAGON_V6_vconv_f8_qf16_128B\0"
  "HEXAGON_V6_vconv_h_hf\0"
  "HEXAGON_V6_vconv_h_hf_128B\0"
  "HEXAGON_V6_vconv_h_hf_rnd\0"
  "HEXAGON_V6_vconv_h_hf_rnd_128B\0"
  "HEXAGON_V6_vconv_hf_h\0"
  "HEXAGON_V6_vconv_hf_h_128B\0"
  "HEXAGON_V6_vconv_hf_qf16\0"
  "HEXAGON_V6_vconv_hf_qf16_128B\0"
  "HEXAGON_V6_vconv_hf_qf32\0"
  "HEXAGON_V6_vconv_hf_qf32_128B\0"
  "HEXAGON_V6_vconv_qf16_f8\0"
  "HEXAGON_V6_vconv_qf16_f8_128B\0"
  "HEXAGON_V6_vconv_qf16_hf\0"
  "HEXAGON_V6_vconv_qf16_hf_128B\0"
  "HEXAGON_V6_vconv_qf16_qf16\0"
  "HEXAGON_V6_vconv_qf16_qf16_128B\0"
  "HEXAGON_V6_vconv_qf32_qf32\0"
  "HEXAGON_V6_vconv_qf32_qf32_128B\0"
  "HEXAGON_V6_vconv_qf32_sf\0"
  "HEXAGON_V6_vconv_qf32_sf_128B\0"
  "HEXAGON_V6_vconv_sf_qf32\0"
  "HEXAGON_V6_vconv_sf_qf32_128B\0"
  "HEXAGON_V6_vconv_sf_w\0"
  "HEXAGON_V6_vconv_sf_w_128B\0"
  "HEXAGON_V6_vconv_w_sf\0"
  "HEXAGON_V6_vconv_w_sf_128B\0"
  "HEXAGON_V6_vcvt2_b_hf\0"
  "HEXAGON_V6_vcvt2_b_hf_128B\0"
  "HEXAGON_V6_vcvt2_hf_b\0"
  "HEXAGON_V6_vcvt2_hf_b_128B\0"
  "HEXAGON_V6_vcvt2_hf_ub\0"
  "HEXAGON_V6_vcvt2_hf_ub_128B\0"
  "HEXAGON_V6_vcvt2_ub_hf\0"
  "HEXAGON_V6_vcvt2_ub_hf_128B\0"
  "HEXAGON_V6_vcvt_b_hf\0"
  "HEXAGON_V6_vcvt_b_hf_128B\0"
  "HEXAGON_V6_vcvt_bf_sf\0"
  "HEXAGON_V6_vcvt_bf_sf_128B\0"
  "HEXAGON_V6_vcvt_f8_hf\0"
  "HEXAGON_V6_vcvt_f8_hf_128B\0"
  "HEXAGON_V6_vcvt_h_hf\0"
  "HEXAGON_V6_vcvt_h_hf_128B\0"
  "HEXAGON_V6_vcvt_hf_b\0"
  "HEXAGON_V6_vcvt_hf_b_128B\0"
  "HEXAGON_V6_vcvt_hf_f8\0"
  "HEXAGON_V6_vcvt_hf_f8_128B\0"
  "HEXAGON_V6_vcvt_hf_h\0"
  "HEXAGON_V6_vcvt_hf_h_128B\0"
  "HEXAGON_V6_vcvt_hf_sf\0"
  "HEXAGON_V6_vcvt_hf_sf_128B\0"
  "HEXAGON_V6_vcvt_hf_ub\0"
  "HEXAGON_V6_vcvt_hf_ub_128B\0"
  "HEXAGON_V6_vcvt_hf_uh\0"
  "HEXAGON_V6_vcvt_hf_uh_128B\0"
  "HEXAGON_V6_vcvt_sf_hf\0"
  "HEXAGON_V6_vcvt_sf_hf_128B\0"
  "HEXAGON_V6_vcvt_ub_hf\0"
  "HEXAGON_V6_vcvt_ub_hf_128B\0"
  "HEXAGON_V6_vcvt_uh_hf\0"
  "HEXAGON_V6_vcvt_uh_hf_128B\0"
  "HEXAGON_V6_vd0\0"
  "HEXAGON_V6_vd0_128B\0"
  "HEXAGON_V6_vdd0\0"
  "HEXAGON_V6_vdd0_128B\0"
  "HEXAGON_V6_vdealb\0"
  "HEXAGON_V6_vdealb4w\0"
  "HEXAGON_V6_vdealb4w_128B\0"
  "HEXAGON_V6_vdealb_128B\0"
  "HEXAGON_V6_vdealh\0"
  "HEXAGON_V6_vdealh_128B\0"
  "HEXAGON_V6_vdealvdd\0"
  "HEXAGON_V6_vdealvdd_128B\0"
  "HEXAGON_V6_vdelta\0"
  "HEXAGON_V6_vdelta_128B\0"
  "HEXAGON_V6_vdmpy_sf_hf\0"
  "HEXAGON_V6_vdmpy_sf_hf_128B\0"
  "HEXAGON_V6_vdmpy_sf_hf_acc\0"
  "HEXAGON_V6_vdmpy_sf_hf_acc_128B\0"
  "HEXAGON_V6_vdmpybus\0"
  "HEXAGON_V6_vdmpybus_128B\0"
  "HEXAGON_V6_vdmpybus_acc\0"
  "HEXAGON_V6_vdmpybus_acc_128B\0"
  "HEXAGON_V6_vdmpybus_dv\0"
  "HEXAGON_V6_vdmpybus_dv_128B\0"
  "HEXAGON_V6_vdmpybus_dv_acc\0"
  "HEXAGON_V6_vdmpybus_dv_acc_128B\0"
  "HEXAGON_V6_vdmpyhb\0"
  "HEXAGON_V6_vdmpyhb_128B\0"
  "HEXAGON_V6_vdmpyhb_acc\0"
  "HEXAGON_V6_vdmpyhb_acc_128B\0"
  "HEXAGON_V6_vdmpyhb_dv\0"
  "HEXAGON_V6_vdmpyhb_dv_128B\0"
  "HEXAGON_V6_vdmpyhb_dv_acc\0"
  "HEXAGON_V6_vdmpyhb_dv_acc_128B\0"
  "HEXAGON_V6_vdmpyhisat\0"
  "HEXAGON_V6_vdmpyhisat_128B\0"
  "HEXAGON_V6_vdmpyhisat_acc\0"
  "HEXAGON_V6_vdmpyhisat_acc_128B\0"
  "HEXAGON_V6_vdmpyhsat\0"
  "HEXAGON_V6_vdmpyhsat_128B\0"
  "HEXAGON_V6_vdmpyhsat_acc\0"
  "HEXAGON_V6_vdmpyhsat_acc_128B\0"
  "HEXAGON_V6_vdmpyhsuisat\0"
  "HEXAGON_V6_vdmpyhsuisat_128B\0"
  "HEXAGON_V6_vdmpyhsuisat_acc\0"
  "HEXAGON_V6_vdmpyhsuisat_acc_128B\0"
  "HEXAGON_V6_vdmpyhsusat\0"
  "HEXAGON_V6_vdmpyhsusat_128B\0"
  "HEXAGON_V6_vdmpyhsusat_acc\0"
  "HEXAGON_V6_vdmpyhsusat_acc_128B\0"
  "HEXAGON_V6_vdmpyhvsat\0"
  "HEXAGON_V6_vdmpyhvsat_128B\0"
  "HEXAGON_V6_vdmpyhvsat_acc\0"
  "HEXAGON_V6_vdmpyhvsat_acc_128B\0"
  "HEXAGON_V6_vdsaduh\0"
  "HEXAGON_V6_vdsaduh_128B\0"
  "HEXAGON_V6_vdsaduh_acc\0"
  "HEXAGON_V6_vdsaduh_acc_128B\0"
  "HEXAGON_V6_veqb\0"
  "HEXAGON_V6_veqb_128B\0"
  "HEXAGON_V6_veqb_and\0"
  "HEXAGON_V6_veqb_and_128B\0"
  "HEXAGON_V6_veqb_or\0"
  "HEXAGON_V6_veqb_or_128B\0"
  "HEXAGON_V6_veqb_xor\0"
  "HEXAGON_V6_veqb_xor_128B\0"
  "HEXAGON_V6_veqh\0"
  "HEXAGON_V6_veqh_128B\0"
  "HEXAGON_V6_veqh_and\0"
  "HEXAGON_V6_veqh_and_128B\0"
  "HEXAGON_V6_veqh_or\0"
  "HEXAGON_V6_veqh_or_128B\0"
  "HEXAGON_V6_veqh_xor\0"
  "HEXAGON_V6_veqh_xor_128B\0"
  "HEXAGON_V6_veqhf\0"
  "HEXAGON_V6_veqhf_128B\0"
  "HEXAGON_V6_veqhf_and\0"
  "HEXAGON_V6_veqhf_and_128B\0"
  "HEXAGON_V6_veqhf_or\0"
  "HEXAGON_V6_veqhf_or_128B\0"
  "HEXAGON_V6_veqhf_xor\0"
  "HEXAGON_V6_veqhf_xor_128B\0"
  "HEXAGON_V6_veqsf\0"
  "HEXAGON_V6_veqsf_128B\0"
  "HEXAGON_V6_veqsf_and\0"
  "HEXAGON_V6_veqsf_and_128B\0"
  "HEXAGON_V6_veqsf_or\0"
  "HEXAGON_V6_veqsf_or_128B\0"
  "HEXAGON_V6_veqsf_xor\0"
  "HEXAGON_V6_veqsf_xor_128B\0"
  "HEXAGON_V6_veqw\0"
  "HEXAGON_V6_veqw_128B\0"
  "HEXAGON_V6_veqw_and\0"
  "HEXAGON_V6_veqw_and_128B\0"
  "HEXAGON_V6_veqw_or\0"
  "HEXAGON_V6_veqw_or_128B\0"
  "HEXAGON_V6_veqw_xor\0"
  "HEXAGON_V6_veqw_xor_128B\0"
  "HEXAGON_V6_vfmax_f8\0"
  "HEXAGON_V6_vfmax_f8_128B\0"
  "HEXAGON_V6_vfmax_hf\0"
  "HEXAGON_V6_vfmax_hf_128B\0"
  "HEXAGON_V6_vfmax_sf\0"
  "HEXAGON_V6_vfmax_sf_128B\0"
  "HEXAGON_V6_vfmin_f8\0"
  "HEXAGON_V6_vfmin_f8_128B\0"
  "HEXAGON_V6_vfmin_hf\0"
  "HEXAGON_V6_vfmin_hf_128B\0"
  "HEXAGON_V6_vfmin_sf\0"
  "HEXAGON_V6_vfmin_sf_128B\0"
  "HEXAGON_V6_vfneg_f8\0"
  "HEXAGON_V6_vfneg_f8_128B\0"
  "HEXAGON_V6_vfneg_hf\0"
  "HEXAGON_V6_vfneg_hf_128B\0"
  "HEXAGON_V6_vfneg_sf\0"
  "HEXAGON_V6_vfneg_sf_128B\0"
  "HEXAGON_V6_vgather_vscattermh\0"
  "HEXAGON_V6_vgather_vscattermh_128B\0"
  "HEXAGON_V6_vgathermh\0"
  "HEXAGON_V6_vgathermh_128B\0"
  "HEXAGON_V6_vgathermhq\0"
  "HEXAGON_V6_vgathermhq_128B\0"
  "HEXAGON_V6_vgathermhw\0"
  "HEXAGON_V6_vgathermhw_128B\0"
  "HEXAGON_V6_vgathermhwq\0"
  "HEXAGON_V6_vgathermhwq_128B\0"
  "HEXAGON_V6_vgathermw\0"
  "HEXAGON_V6_vgathermw_128B\0"
  "HEXAGON_V6_vgathermwq\0"
  "HEXAGON_V6_vgathermwq_128B\0"
  "HEXAGON_V6_vgtb\0"
  "HEXAGON_V6_vgtb_128B\0"
  "HEXAGON_V6_vgtb_and\0"
  "HEXAGON_V6_vgtb_and_128B\0"
  "HEXAGON_V6_vgtb_or\0"
  "HEXAGON_V6_vgtb_or_128B\0"
  "HEXAGON_V6_vgtb_xor\0"
  "HEXAGON_V6_vgtb_xor_128B\0"
  "HEXAGON_V6_vgtbf\0"
  "HEXAGON_V6_vgtbf_128B\0"
  "HEXAGON_V6_vgtbf_and\0"
  "HEXAGON_V6_vgtbf_and_128B\0"
  "HEXAGON_V6_vgtbf_or\0"
  "HEXAGON_V6_vgtbf_or_128B\0"
  "HEXAGON_V6_vgtbf_xor\0"
  "HEXAGON_V6_vgtbf_xor_128B\0"
  "HEXAGON_V6_vgth\0"
  "HEXAGON_V6_vgth_128B\0"
  "HEXAGON_V6_vgth_and\0"
  "HEXAGON_V6_vgth_and_128B\0"
  "HEXAGON_V6_vgth_or\0"
  "HEXAGON_V6_vgth_or_128B\0"
  "HEXAGON_V6_vgth_xor\0"
  "HEXAGON_V6_vgth_xor_128B\0"
  "HEXAGON_V6_vgthf\0"
  "HEXAGON_V6_vgthf_128B\0"
  "HEXAGON_V6_vgthf_and\0"
  "HEXAGON_V6_vgthf_and_128B\0"
  "HEXAGON_V6_vgthf_or\0"
  "HEXAGON_V6_vgthf_or_128B\0"
  "HEXAGON_V6_vgthf_xor\0"
  "HEXAGON_V6_vgthf_xor_128B\0"
  "HEXAGON_V6_vgtsf\0"
  "HEXAGON_V6_vgtsf_128B\0"
  "HEXAGON_V6_vgtsf_and\0"
  "HEXAGON_V6_vgtsf_and_128B\0"
  "HEXAGON_V6_vgtsf_or\0"
  "HEXAGON_V6_vgtsf_or_128B\0"
  "HEXAGON_V6_vgtsf_xor\0"
  "HEXAGON_V6_vgtsf_xor_128B\0"
  "HEXAGON_V6_vgtub\0"
  "HEXAGON_V6_vgtub_128B\0"
  "HEXAGON_V6_vgtub_and\0"
  "HEXAGON_V6_vgtub_and_128B\0"
  "HEXAGON_V6_vgtub_or\0"
  "HEXAGON_V6_vgtub_or_128B\0"
  "HEXAGON_V6_vgtub_xor\0"
  "HEXAGON_V6_vgtub_xor_128B\0"
  "HEXAGON_V6_vgtuh\0"
  "HEXAGON_V6_vgtuh_128B\0"
  "HEXAGON_V6_vgtuh_and\0"
  "HEXAGON_V6_vgtuh_and_128B\0"
  "HEXAGON_V6_vgtuh_or\0"
  "HEXAGON_V6_vgtuh_or_128B\0"
  "HEXAGON_V6_vgtuh_xor\0"
  "HEXAGON_V6_vgtuh_xor_128B\0"
  "HEXAGON_V6_vgtuw\0"
  "HEXAGON_V6_vgtuw_128B\0"
  "HEXAGON_V6_vgtuw_and\0"
  "HEXAGON_V6_vgtuw_and_128B\0"
  "HEXAGON_V6_vgtuw_or\0"
  "HEXAGON_V6_vgtuw_or_128B\0"
  "HEXAGON_V6_vgtuw_xor\0"
  "HEXAGON_V6_vgtuw_xor_128B\0"
  "HEXAGON_V6_vgtw\0"
  "HEXAGON_V6_vgtw_128B\0"
  "HEXAGON_V6_vgtw_and\0"
  "HEXAGON_V6_vgtw_and_128B\0"
  "HEXAGON_V6_vgtw_or\0"
  "HEXAGON_V6_vgtw_or_128B\0"
  "HEXAGON_V6_vgtw_xor\0"
  "HEXAGON_V6_vgtw_xor_128B\0"
  "HEXAGON_V6_vilog2_hf\0"
  "HEXAGON_V6_vilog2_hf_128B\0"
  "HEXAGON_V6_vilog2_qf16\0"
  "HEXAGON_V6_vilog2_qf16_128B\0"
  "HEXAGON_V6_vilog2_qf32\0"
  "HEXAGON_V6_vilog2_qf32_128B\0"
  "HEXAGON_V6_vilog2_sf\0"
  "HEXAGON_V6_vilog2_sf_128B\0"
  "HEXAGON_V6_vinsertwr\0"
  "HEXAGON_V6_vinsertwr_128B\0"
  "HEXAGON_V6_vlalignb\0"
  "HEXAGON_V6_vlalignb_128B\0"
  "HEXAGON_V6_vlalignbi\0"
  "HEXAGON_V6_vlalignbi_128B\0"
  "HEXAGON_V6_vlsrb\0"
  "HEXAGON_V6_vlsrb_128B\0"
  "HEXAGON_V6_vlsrh\0"
  "HEXAGON_V6_vlsrh_128B\0"
  "HEXAGON_V6_vlsrhv\0"
  "HEXAGON_V6_vlsrhv_128B\0"
  "HEXAGON_V6_vlsrw\0"
  "HEXAGON_V6_vlsrw_128B\0"
  "HEXAGON_V6_vlsrwv\0"
  "HEXAGON_V6_vlsrwv_128B\0"
  "HEXAGON_V6_vlut4\0"
  "HEXAGON_V6_vlut4_128B\0"
  "HEXAGON_V6_vlutvvb\0"
  "HEXAGON_V6_vlutvvb_128B\0"
  "HEXAGON_V6_vlutvvb_nm\0"
  "HEXAGON_V6_vlutvvb_nm_128B\0"
  "HEXAGON_V6_vlutvvb_oracc\0"
  "HEXAGON_V6_vlutvvb_oracc_128B\0"
  "HEXAGON_V6_vlutvvb_oracci\0"
  "HEXAGON_V6_vlutvvb_oracci_128B\0"
  "HEXAGON_V6_vlutvvbi\0"
  "HEXAGON_V6_vlutvvbi_128B\0"
  "HEXAGON_V6_vlutvwh\0"
  "HEXAGON_V6_vlutvwh_128B\0"
  "HEXAGON_V6_vlutvwh_nm\0"
  "HEXAGON_V6_vlutvwh_nm_128B\0"
  "HEXAGON_V6_vlutvwh_oracc\0"
  "HEXAGON_V6_vlutvwh_oracc_128B\0"
  "HEXAGON_V6_vlutvwh_oracci\0"
  "HEXAGON_V6_vlutvwh_oracci_128B\0"
  "HEXAGON_V6_vlutvwhi\0"
  "HEXAGON_V6_vlutvwhi_128B\0"
  "HEXAGON_V6_vmax_bf\0"
  "HEXAGON_V6_vmax_bf_128B\0"
  "HEXAGON_V6_vmax_hf\0"
  "HEXAGON_V6_vmax_hf_128B\0"
  "HEXAGON_V6_vmax_sf\0"
  "HEXAGON_V6_vmax_sf_128B\0"
  "HEXAGON_V6_vmaxb\0"
  "HEXAGON_V6_vmaxb_128B\0"
  "HEXAGON_V6_vmaxh\0"
  "HEXAGON_V6_vmaxh_128B\0"
  "HEXAGON_V6_vmaxub\0"
  "HEXAGON_V6_vmaxub_128B\0"
  "HEXAGON_V6_vmaxuh\0"
  "HEXAGON_V6_vmaxuh_128B\0"
  "HEXAGON_V6_vmaxw\0"
  "HEXAGON_V6_vmaxw_128B\0"
  "HEXAGON_V6_vmerge_qf\0"
  "HEXAGON_V6_vmerge_qf_128B\0"
  "HEXAGON_V6_vmin_bf\0"
  "HEXAGON_V6_vmin_bf_128B\0"
  "HEXAGON_V6_vmin_hf\0"
  "HEXAGON_V6_vmin_hf_128B\0"
  "HEXAGON_V6_vmin_sf\0"
  "HEXAGON_V6_vmin_sf_128B\0"
  "HEXAGON_V6_vminb\0"
  "HEXAGON_V6_vminb_128B\0"
  "HEXAGON_V6_vminh\0"
  "HEXAGON_V6_vminh_128B\0"
  "HEXAGON_V6_vminub\0"
  "HEXAGON_V6_vminub_128B\0"
  "HEXAGON_V6_vminuh\0"
  "HEXAGON_V6_vminuh_128B\0"
  "HEXAGON_V6_vminw\0"
  "HEXAGON_V6_vminw_128B\0"
  "HEXAGON_V6_vmpabus\0"
  "HEXAGON_V6_vmpabus_128B\0"
  "HEXAGON_V6_vmpabus_acc\0"
  "HEXAGON_V6_vmpabus_acc_128B\0"
  "HEXAGON_V6_vmpabusv\0"
  "HEXAGON_V6_vmpabusv_128B\0"
  "HEXAGON_V6_vmpabuu\0"
  "HEXAGON_V6_vmpabuu_128B\0"
  "HEXAGON_V6_vmpabuu_acc\0"
  "HEXAGON_V6_vmpabuu_acc_128B\0"
  "HEXAGON_V6_vmpabuuv\0"
  "HEXAGON_V6_vmpabuuv_128B\0"
  "HEXAGON_V6_vmpahb\0"
  "HEXAGON_V6_vmpahb_128B\0"
  "HEXAGON_V6_vmpahb_acc\0"
  "HEXAGON_V6_vmpahb_acc_128B\0"
  "HEXAGON_V6_vmpahhsat\0"
  "HEXAGON_V6_vmpahhsat_128B\0"
  "HEXAGON_V6_vmpauhb\0"
  "HEXAGON_V6_vmpauhb_128B\0"
  "HEXAGON_V6_vmpauhb_acc\0"
  "HEXAGON_V6_vmpauhb_acc_128B\0"
  "HEXAGON_V6_vmpauhuhsat\0"
  "HEXAGON_V6_vmpauhuhsat_128B\0"
  "HEXAGON_V6_vmpsuhuhsat\0"
  "HEXAGON_V6_vmpsuhuhsat_128B\0"
  "HEXAGON_V6_vmpy_hf_f8\0"
  "HEXAGON_V6_vmpy_hf_f8_128B\0"
  "HEXAGON_V6_vmpy_hf_f8_acc\0"
  "HEXAGON_V6_vmpy_hf_f8_acc_128B\0"
  "HEXAGON_V6_vmpy_hf_hf\0"
  "HEXAGON_V6_vmpy_hf_hf_128B\0"
  "HEXAGON_V6_vmpy_hf_hf_acc\0"
  "HEXAGON_V6_vmpy_hf_hf_acc_128B\0"
  "HEXAGON_V6_vmpy_qf16\0"
  "HEXAGON_V6_vmpy_qf16_128B\0"
  "HEXAGON_V6_vmpy_qf16_hf\0"
  "HEXAGON_V6_vmpy_qf16_hf_128B\0"
  "HEXAGON_V6_vmpy_qf16_mix_hf\0"
  "HEXAGON_V6_vmpy_qf16_mix_hf_128B\0"
  "HEXAGON_V6_vmpy_qf32\0"
  "HEXAGON_V6_vmpy_qf32_128B\0"
  "HEXAGON_V6_vmpy_qf32_hf\0"
  "HEXAGON_V6_vmpy_qf32_hf_128B\0"
  "HEXAGON_V6_vmpy_qf32_mix_hf\0"
  "HEXAGON_V6_vmpy_qf32_mix_hf_128B\0"
  "HEXAGON_V6_vmpy_qf32_qf16\0"
  "HEXAGON_V6_vmpy_qf32_qf16_128B\0"
  "HEXAGON_V6_vmpy_qf32_sf\0"
  "HEXAGON_V6_vmpy_qf32_sf_128B\0"
  "HEXAGON_V6_vmpy_rt_hf\0"
  "HEXAGON_V6_vmpy_rt_hf_128B\0"
  "HEXAGON_V6_vmpy_rt_qf16\0"
  "HEXAGON_V6_vmpy_rt_qf16_128B\0"
  "HEXAGON_V6_vmpy_rt_sf\0"
  "HEXAGON_V6_vmpy_rt_sf_128B\0"
  "HEXAGON_V6_vmpy_sf_bf\0"
  "HEXAGON_V6_vmpy_sf_bf_128B\0"
  "HEXAGON_V6_vmpy_sf_bf_acc\0"
  "HEXAGON_V6_vmpy_sf_bf_acc_128B\0"
  "HEXAGON_V6_vmpy_sf_hf\0"
  "HEXAGON_V6_vmpy_sf_hf_128B\0"
  "HEXAGON_V6_vmpy_sf_hf_acc\0"
  "HEXAGON_V6_vmpy_sf_hf_acc_128B\0"
  "HEXAGON_V6_vmpy_sf_sf\0"
  "HEXAGON_V6_vmpy_sf_sf_128B\0"
  "HEXAGON_V6_vmpybus\0"
  "HEXAGON_V6_vmpybus_128B\0"
  "HEXAGON_V6_vmpybus_acc\0"
  "HEXAGON_V6_vmpybus_acc_128B\0"
  "HEXAGON_V6_vmpybusv\0"
  "HEXAGON_V6_vmpybusv_128B\0"
  "HEXAGON_V6_vmpybusv_acc\0"
  "HEXAGON_V6_vmpybusv_acc_128B\0"
  "HEXAGON_V6_vmpybv\0"
  "HEXAGON_V6_vmpybv_128B\0"
  "HEXAGON_V6_vmpybv_acc\0"
  "HEXAGON_V6_vmpybv_acc_128B\0"
  "HEXAGON_V6_vmpyewuh\0"
  "HEXAGON_V6_vmpyewuh_128B\0"
  "HEXAGON_V6_vmpyewuh_64\0"
  "HEXAGON_V6_vmpyewuh_64_128B\0"
  "HEXAGON_V6_vmpyh\0"
  "HEXAGON_V6_vmpyh_128B\0"
  "HEXAGON_V6_vmpyh_acc\0"
  "HEXAGON_V6_vmpyh_acc_128B\0"
  "HEXAGON_V6_vmpyhsat_acc\0"
  "HEXAGON_V6_vmpyhsat_acc_128B\0"
  "HEXAGON_V6_vmpyhsrs\0"
  "HEXAGON_V6_vmpyhsrs_128B\0"
  "HEXAGON_V6_vmpyhss\0"
  "HEXAGON_V6_vmpyhss_128B\0"
  "HEXAGON_V6_vmpyhus\0"
  "HEXAGON_V6_vmpyhus_128B\0"
  "HEXAGON_V6_vmpyhus_acc\0"
  "HEXAGON_V6_vmpyhus_acc_128B\0"
  "HEXAGON_V6_vmpyhv\0"
  "HEXAGON_V6_vmpyhv_128B\0"
  "HEXAGON_V6_vmpyhv_acc\0"
  "HEXAGON_V6_vmpyhv_acc_128B\0"
  "HEXAGON_V6_vmpyhvsrs\0"
  "HEXAGON_V6_vmpyhvsrs_128B\0"
  "HEXAGON_V6_vmpyieoh\0"
  "HEXAGON_V6_vmpyieoh_128B\0"
  "HEXAGON_V6_vmpyiewh_acc\0"
  "HEXAGON_V6_vmpyiewh_acc_128B\0"
  "HEXAGON_V6_vmpyiewuh\0"
  "HEXAGON_V6_vmpyiewuh_128B\0"
  "HEXAGON_V6_vmpyiewuh_acc\0"
  "HEXAGON_V6_vmpyiewuh_acc_128B\0"
  "HEXAGON_V6_vmpyih\0"
  "HEXAGON_V6_vmpyih_128B\0"
  "HEXAGON_V6_vmpyih_acc\0"
  "HEXAGON_V6_vmpyih_acc_128B\0"
  "HEXAGON_V6_vmpyihb\0"
  "HEXAGON_V6_vmpyihb_128B\0"
  "HEXAGON_V6_vmpyihb_acc\0"
  "HEXAGON_V6_vmpyihb_acc_128B\0"
  "HEXAGON_V6_vmpyiowh\0"
  "HEXAGON_V6_vmpyiowh_128B\0"
  "HEXAGON_V6_vmpyiwb\0"
  "HEXAGON_V6_vmpyiwb_128B\0"
  "HEXAGON_V6_vmpyiwb_acc\0"
  "HEXAGON_V6_vmpyiwb_acc_128B\0"
  "HEXAGON_V6_vmpyiwh\0"
  "HEXAGON_V6_vmpyiwh_128B\0"
  "HEXAGON_V6_vmpyiwh_acc\0"
  "HEXAGON_V6_vmpyiwh_acc_128B\0"
  "HEXAGON_V6_vmpyiwub\0"
  "HEXAGON_V6_vmpyiwub_128B\0"
  "HEXAGON_V6_vmpyiwub_acc\0"
  "HEXAGON_V6_vmpyiwub_acc_128B\0"
  "HEXAGON_V6_vmpyowh\0"
  "HEXAGON_V6_vmpyowh_128B\0"
  "HEXAGON_V6_vmpyowh_64_acc\0"
  "HEXAGON_V6_vmpyowh_64_acc_128B\0"
  "HEXAGON_V6_vmpyowh_rnd\0"
  "HEXAGON_V6_vmpyowh_rnd_128B\0"
  "HEXAGON_V6_vmpyowh_rnd_sacc\0"
  "HEXAGON_V6_vmpyowh_rnd_sacc_128B\0"
  "HEXAGON_V6_vmpyowh_sacc\0"
  "HEXAGON_V6_vmpyowh_sacc_128B\0"
  "HEXAGON_V6_vmpyub\0"
  "HEXAGON_V6_vmpyub_128B\0"
  "HEXAGON_V6_vmpyub_acc\0"
  "HEXAGON_V6_vmpyub_acc_128B\0"
  "HEXAGON_V6_vmpyubv\0"
  "HEXAGON_V6_vmpyubv_128B\0"
  "HEXAGON_V6_vmpyubv_acc\0"
  "HEXAGON_V6_vmpyubv_acc_128B\0"
  "HEXAGON_V6_vmpyuh\0"
  "HEXAGON_V6_vmpyuh_128B\0"
  "HEXAGON_V6_vmpyuh_acc\0"
  "HEXAGON_V6_vmpyuh_acc_128B\0"
  "HEXAGON_V6_vmpyuhe\0"
  "HEXAGON_V6_vmpyuhe_128B\0"
  "HEXAGON_V6_vmpyuhe_acc\0"
  "HEXAGON_V6_vmpyuhe_acc_128B\0"
  "HEXAGON_V6_vmpyuhv\0"
  "HEXAGON_V6_vmpyuhv_128B\0"
  "HEXAGON_V6_vmpyuhv_acc\0"
  "HEXAGON_V6_vmpyuhv_acc_128B\0"
  "HEXAGON_V6_vmpyuhvs\0"
  "HEXAGON_V6_vmpyuhvs_128B\0"
  "HEXAGON_V6_vmux\0"
  "HEXAGON_V6_vmux_128B\0"
  "HEXAGON_V6_vnavgb\0"
  "HEXAGON_V6_vnavgb_128B\0"
  "HEXAGON_V6_vnavgh\0"
  "HEXAGON_V6_vnavgh_128B\0"
  "HEXAGON_V6_vnavgub\0"
  "HEXAGON_V6_vnavgub_128B\0"
  "HEXAGON_V6_vnavgw\0"
  "HEXAGON_V6_vnavgw_128B\0"
  "HEXAGON_V6_vneg_qf16_hf\0"
  "HEXAGON_V6_vneg_qf16_hf_128B\0"
  "HEXAGON_V6_vneg_qf16_qf16\0"
  "HEXAGON_V6_vneg_qf16_qf16_128B\0"
  "HEXAGON_V6_vneg_qf32_qf32\0"
  "HEXAGON_V6_vneg_qf32_qf32_128B\0"
  "HEXAGON_V6_vneg_qf32_sf\0"
  "HEXAGON_V6_vneg_qf32_sf_128B\0"
  "HEXAGON_V6_vnormamth\0"
  "HEXAGON_V6_vnormamth_128B\0"
  "HEXAGON_V6_vnormamtw\0"
  "HEXAGON_V6_vnormamtw_128B\0"
  "HEXAGON_V6_vnot\0"
  "HEXAGON_V6_vnot_128B\0"
  "HEXAGON_V6_vor\0"
  "HEXAGON_V6_vor_128B\0"
  "HEXAGON_V6_vpackeb\0"
  "HEXAGON_V6_vpackeb_128B\0"
  "HEXAGON_V6_vpackeh\0"
  "HEXAGON_V6_vpackeh_128B\0"
  "HEXAGON_V6_vpackhb_sat\0"
  "HEXAGON_V6_vpackhb_sat_128B\0"
  "HEXAGON_V6_vpackhub_sat\0"
  "HEXAGON_V6_vpackhub_sat_128B\0"
  "HEXAGON_V6_vpackob\0"
  "HEXAGON_V6_vpackob_128B\0"
  "HEXAGON_V6_vpackoh\0"
  "HEXAGON_V6_vpackoh_128B\0"
  "HEXAGON_V6_vpackwh_sat\0"
  "HEXAGON_V6_vpackwh_sat_128B\0"
  "HEXAGON_V6_vpackwuh_sat\0"
  "HEXAGON_V6_vpackwuh_sat_128B\0"
  "HEXAGON_V6_vpopcounth\0"
  "HEXAGON_V6_vpopcounth_128B\0"
  "HEXAGON_V6_vprefixqb\0"
  "HEXAGON_V6_vprefixqb_128B\0"
  "HEXAGON_V6_vprefixqh\0"
  "HEXAGON_V6_vprefixqh_128B\0"
  "HEXAGON_V6_vprefixqw\0"
  "HEXAGON_V6_vprefixqw_128B\0"
  "HEXAGON_V6_vrdelta\0"
  "HEXAGON_V6_vrdelta_128B\0"
  "HEXAGON_V6_vrmpybub_rtt\0"
  "HEXAGON_V6_vrmpybub_rtt_128B\0"
  "HEXAGON_V6_vrmpybub_rtt_acc\0"
  "HEXAGON_V6_vrmpybub_rtt_acc_128B\0"
  "HEXAGON_V6_vrmpybus\0"
  "HEXAGON_V6_vrmpybus_128B\0"
  "HEXAGON_V6_vrmpybus_acc\0"
  "HEXAGON_V6_vrmpybus_acc_128B\0"
  "HEXAGON_V6_vrmpybusi\0"
  "HEXAGON_V6_vrmpybusi_128B\0"
  "HEXAGON_V6_vrmpybusi_acc\0"
  "HEXAGON_V6_vrmpybusi_acc_128B\0"
  "HEXAGON_V6_vrmpybusv\0"
  "HEXAGON_V6_vrmpybusv_128B\0"
  "HEXAGON_V6_vrmpybusv_acc\0"
  "HEXAGON_V6_vrmpybusv_acc_128B\0"
  "HEXAGON_V6_vrmpybv\0"
  "HEXAGON_V6_vrmpybv_128B\0"
  "HEXAGON_V6_vrmpybv_acc\0"
  "HEXAGON_V6_vrmpybv_acc_128B\0"
  "HEXAGON_V6_vrmpyub\0"
  "HEXAGON_V6_vrmpyub_128B\0"
  "HEXAGON_V6_vrmpyub_acc\0"
  "HEXAGON_V6_vrmpyub_acc_128B\0"
  "HEXAGON_V6_vrmpyub_rtt\0"
  "HEXAGON_V6_vrmpyub_rtt_128B\0"
  "HEXAGON_V6_vrmpyub_rtt_acc\0"
  "HEXAGON_V6_vrmpyub_rtt_acc_128B\0"
  "HEXAGON_V6_vrmpyubi\0"
  "HEXAGON_V6_vrmpyubi_128B\0"
  "HEXAGON_V6_vrmpyubi_acc\0"
  "HEXAGON_V6_vrmpyubi_acc_128B\0"
  "HEXAGON_V6_vrmpyubv\0"
  "HEXAGON_V6_vrmpyubv_128B\0"
  "HEXAGON_V6_vrmpyubv_acc\0"
  "HEXAGON_V6_vrmpyubv_acc_128B\0"
  "HEXAGON_V6_vror\0"
  "HEXAGON_V6_vror_128B\0"
  "HEXAGON_V6_vrotr\0"
  "HEXAGON_V6_vrotr_128B\0"
  "HEXAGON_V6_vroundhb\0"
  "HEXAGON_V6_vroundhb_128B\0"
  "HEXAGON_V6_vroundhub\0"
  "HEXAGON_V6_vroundhub_128B\0"
  "HEXAGON_V6_vrounduhub\0"
  "HEXAGON_V6_vrounduhub_128B\0"
  "HEXAGON_V6_vrounduwuh\0"
  "HEXAGON_V6_vrounduwuh_128B\0"
  "HEXAGON_V6_vroundwh\0"
  "HEXAGON_V6_vroundwh_128B\0"
  "HEXAGON_V6_vroundwuh\0"
  "HEXAGON_V6_vroundwuh_128B\0"
  "HEXAGON_V6_vrsadubi\0"
  "HEXAGON_V6_vrsadubi_128B\0"
  "HEXAGON_V6_vrsadubi_acc\0"
  "HEXAGON_V6_vrsadubi_acc_128B\0"
  "HEXAGON_V6_vsatdw\0"
  "HEXAGON_V6_vsatdw_128B\0"
  "HEXAGON_V6_vsathub\0"
  "HEXAGON_V6_vsathub_128B\0"
  "HEXAGON_V6_vsatuwuh\0"
  "HEXAGON_V6_vsatuwuh_128B\0"
  "HEXAGON_V6_vsatwh\0"
  "HEXAGON_V6_vsatwh_128B\0"
  "HEXAGON_V6_vsb\0"
  "HEXAGON_V6_vsb_128B\0"
  "HEXAGON_V6_vscattermh\0"
  "HEXAGON_V6_vscattermh_128B\0"
  "HEXAGON_V6_vscattermh_add\0"
  "HEXAGON_V6_vscattermh_add_128B\0"
  "HEXAGON_V6_vscattermhq\0"
  "HEXAGON_V6_vscattermhq_128B\0"
  "HEXAGON_V6_vscattermhw\0"
  "HEXAGON_V6_vscattermhw_128B\0"
  "HEXAGON_V6_vscattermhw_add\0"
  "HEXAGON_V6_vscattermhw_add_128B\0"
  "HEXAGON_V6_vscattermhwq\0"
  "HEXAGON_V6_vscattermhwq_128B\0"
  "HEXAGON_V6_vscattermw\0"
  "HEXAGON_V6_vscattermw_128B\0"
  "HEXAGON_V6_vscattermw_add\0"
  "HEXAGON_V6_vscattermw_add_128B\0"
  "HEXAGON_V6_vscattermwq\0"
  "HEXAGON_V6_vscattermwq_128B\0"
  "HEXAGON_V6_vsh\0"
  "HEXAGON_V6_vsh_128B\0"
  "HEXAGON_V6_vshufeh\0"
  "HEXAGON_V6_vshufeh_128B\0"
  "HEXAGON_V6_vshuffb\0"
  "HEXAGON_V6_vshuffb_128B\0"
  "HEXAGON_V6_vshuffeb\0"
  "HEXAGON_V6_vshuffeb_128B\0"
  "HEXAGON_V6_vshuffh\0"
  "HEXAGON_V6_vshuffh_128B\0"
  "HEXAGON_V6_vshuffob\0"
  "HEXAGON_V6_vshuffob_128B\0"
  "HEXAGON_V6_vshuffvdd\0"
  "HEXAGON_V6_vshuffvdd_128B\0"
  "HEXAGON_V6_vshufoeb\0"
  "HEXAGON_V6_vshufoeb_128B\0"
  "HEXAGON_V6_vshufoeh\0"
  "HEXAGON_V6_vshufoeh_128B\0"
  "HEXAGON_V6_vshufoh\0"
  "HEXAGON_V6_vshufoh_128B\0"
  "HEXAGON_V6_vsub_hf\0"
  "HEXAGON_V6_vsub_hf_128B\0"
  "HEXAGON_V6_vsub_hf_f8\0"
  "HEXAGON_V6_vsub_hf_f8_128B\0"
  "HEXAGON_V6_vsub_hf_hf\0"
  "HEXAGON_V6_vsub_hf_hf_128B\0"
  "HEXAGON_V6_vsub_hf_mix\0"
  "HEXAGON_V6_vsub_hf_mix_128B\0"
  "HEXAGON_V6_vsub_qf16\0"
  "HEXAGON_V6_vsub_qf16_128B\0"
  "HEXAGON_V6_vsub_qf16_mix\0"
  "HEXAGON_V6_vsub_qf16_mix_128B\0"
  "HEXAGON_V6_vsub_qf32\0"
  "HEXAGON_V6_vsub_qf32_128B\0"
  "HEXAGON_V6_vsub_qf32_mix\0"
  "HEXAGON_V6_vsub_qf32_mix_128B\0"
  "HEXAGON_V6_vsub_sf\0"
  "HEXAGON_V6_vsub_sf_128B\0"
  "HEXAGON_V6_vsub_sf_bf\0"
  "HEXAGON_V6_vsub_sf_bf_128B\0"
  "HEXAGON_V6_vsub_sf_hf\0"
  "HEXAGON_V6_vsub_sf_hf_128B\0"
  "HEXAGON_V6_vsub_sf_mix\0"
  "HEXAGON_V6_vsub_sf_mix_128B\0"
  "HEXAGON_V6_vsub_sf_sf\0"
  "HEXAGON_V6_vsub_sf_sf_128B\0"
  "HEXAGON_V6_vsubb\0"
  "HEXAGON_V6_vsubb_128B\0"
  "HEXAGON_V6_vsubb_dv\0"
  "HEXAGON_V6_vsubb_dv_128B\0"
  "HEXAGON_V6_vsubbnq\0"
  "HEXAGON_V6_vsubbnq_128B\0"
  "HEXAGON_V6_vsubbq\0"
  "HEXAGON_V6_vsubbq_128B\0"
  "HEXAGON_V6_vsubbsat\0"
  "HEXAGON_V6_vsubbsat_128B\0"
  "HEXAGON_V6_vsubbsat_dv\0"
  "HEXAGON_V6_vsubbsat_dv_128B\0"
  "HEXAGON_V6_vsubh\0"
  "HEXAGON_V6_vsubh_128B\0"
  "HEXAGON_V6_vsubh_dv\0"
  "HEXAGON_V6_vsubh_dv_128B\0"
  "HEXAGON_V6_vsubhnq\0"
  "HEXAGON_V6_vsubhnq_128B\0"
  "HEXAGON_V6_vsubhq\0"
  "HEXAGON_V6_vsubhq_128B\0"
  "HEXAGON_V6_vsubhsat\0"
  "HEXAGON_V6_vsubhsat_128B\0"
  "HEXAGON_V6_vsubhsat_dv\0"
  "HEXAGON_V6_vsubhsat_dv_128B\0"
  "HEXAGON_V6_vsubhw\0"
  "HEXAGON_V6_vsubhw_128B\0"
  "HEXAGON_V6_vsububh\0"
  "HEXAGON_V6_vsububh_128B\0"
  "HEXAGON_V6_vsububsat\0"
  "HEXAGON_V6_vsububsat_128B\0"
  "HEXAGON_V6_vsububsat_dv\0"
  "HEXAGON_V6_vsububsat_dv_128B\0"
  "HEXAGON_V6_vsubububb_sat\0"
  "HEXAGON_V6_vsubububb_sat_128B\0"
  "HEXAGON_V6_vsubuhsat\0"
  "HEXAGON_V6_vsubuhsat_128B\0"
  "HEXAGON_V6_vsubuhsat_dv\0"
  "HEXAGON_V6_vsubuhsat_dv_128B\0"
  "HEXAGON_V6_vsubuhw\0"
  "HEXAGON_V6_vsubuhw_128B\0"
  "HEXAGON_V6_vsubuwsat\0"
  "HEXAGON_V6_vsubuwsat_128B\0"
  "HEXAGON_V6_vsubuwsat_dv\0"
  "HEXAGON_V6_vsubuwsat_dv_128B\0"
  "HEXAGON_V6_vsubw\0"
  "HEXAGON_V6_vsubw_128B\0"
  "HEXAGON_V6_vsubw_dv\0"
  "HEXAGON_V6_vsubw_dv_128B\0"
  "HEXAGON_V6_vsubwnq\0"
  "HEXAGON_V6_vsubwnq_128B\0"
  "HEXAGON_V6_vsubwq\0"
  "HEXAGON_V6_vsubwq_128B\0"
  "HEXAGON_V6_vsubwsat\0"
  "HEXAGON_V6_vsubwsat_128B\0"
  "HEXAGON_V6_vsubwsat_dv\0"
  "HEXAGON_V6_vsubwsat_dv_128B\0"
  "HEXAGON_V6_vswap\0"
  "HEXAGON_V6_vswap_128B\0"
  "HEXAGON_V6_vtmpyb\0"
  "HEXAGON_V6_vtmpyb_128B\0"
  "HEXAGON_V6_vtmpyb_acc\0"
  "HEXAGON_V6_vtmpyb_acc_128B\0"
  "HEXAGON_V6_vtmpybus\0"
  "HEXAGON_V6_vtmpybus_128B\0"
  "HEXAGON_V6_vtmpybus_acc\0"
  "HEXAGON_V6_vtmpybus_acc_128B\0"
  "HEXAGON_V6_vtmpyhb\0"
  "HEXAGON_V6_vtmpyhb_128B\0"
  "HEXAGON_V6_vtmpyhb_acc\0"
  "HEXAGON_V6_vtmpyhb_acc_128B\0"
  "HEXAGON_V6_vunpackb\0"
  "HEXAGON_V6_vunpackb_128B\0"
  "HEXAGON_V6_vunpackh\0"
  "HEXAGON_V6_vunpackh_128B\0"
  "HEXAGON_V6_vunpackob\0"
  "HEXAGON_V6_vunpackob_128B\0"
  "HEXAGON_V6_vunpackoh\0"
  "HEXAGON_V6_vunpackoh_128B\0"
  "HEXAGON_V6_vunpackub\0"
  "HEXAGON_V6_vunpackub_128B\0"
  "HEXAGON_V6_vunpackuh\0"
  "HEXAGON_V6_vunpackuh_128B\0"
  "HEXAGON_V6_vxor\0"
  "HEXAGON_V6_vxor_128B\0"
  "HEXAGON_V6_vzb\0"
  "HEXAGON_V6_vzb_128B\0"
  "HEXAGON_V6_vzh\0"
  "HEXAGON_V6_vzh_128B\0"
  "HEXAGON_Y2_dccleana\0"
  "HEXAGON_Y2_dccleaninva\0"
  "HEXAGON_Y2_dcfetch\0"
  "HEXAGON_Y2_dcinva\0"
  "HEXAGON_Y2_dczeroa\0"
  "HEXAGON_Y4_l2fetch\0"
  "HEXAGON_Y5_l2fetch\0"
  "HEXAGON_Y6_dmlink\0"
  "HEXAGON_Y6_dmpause\0"
  "HEXAGON_Y6_dmpoll\0"
  "HEXAGON_Y6_dmresume\0"
  "HEXAGON_Y6_dmstart\0"
  "HEXAGON_Y6_dmwait\0"
  "HEXAGON_prefetch\0"
  "_HEXAGON_L4_loadd_locked\0"
  "brev_stb\0"
  "brev_std\0"
  "brev_sth\0"
  "brev_sthhi\0"
  "brev_stw\0"
  "circ_ldb\0"
  "circ_ldd\0"
  "circ_ldh\0"
  "circ_ldub\0"
  "circ_lduh\0"
  "circ_ldw\0"
  "circ_stb\0"
  "circ_std\0"
  "circ_sth\0"
  "circ_sthhi\0"
  "circ_stw\0"
  "hexagon_vmemcpy\0"
  "hexagon_vmemset\0"
  "asx_cast_128\0"
  "asx_cast_128_d\0"
  "asx_cast_128_s\0"
  "asx_concat_128\0"
  "asx_concat_128_d\0"
  "asx_concat_128_s\0"
  "asx_extract_128_hi\0"
  "asx_extract_128_hi_d\0"
  "asx_extract_128_hi_s\0"
  "asx_extract_128_lo\0"
  "asx_extract_128_lo_d\0"
  "asx_extract_128_lo_s\0"
  "asx_insert_128_hi\0"
  "asx_insert_128_hi_d\0"
  "asx_insert_128_hi_s\0"
  "asx_insert_128_lo\0"
  "asx_insert_128_lo_d\0"
  "asx_insert_128_lo_s\0"
  "asx_vext2xv_d_b\0"
  "asx_vext2xv_d_h\0"
  "asx_vext2xv_d_w\0"
  "asx_vext2xv_du_bu\0"
  "asx_vext2xv_du_hu\0"
  "asx_vext2xv_du_wu\0"
  "asx_vext2xv_h_b\0"
  "asx_vext2xv_hu_bu\0"
  "asx_vext2xv_w_b\0"
  "asx_vext2xv_w_h\0"
  "asx_vext2xv_wu_bu\0"
  "asx_vext2xv_wu_hu\0"
  "asx_xbnz_b\0"
  "asx_xbnz_d\0"
  "asx_xbnz_h\0"
  "asx_xbnz_v\0"
  "asx_xbnz_w\0"
  "asx_xbz_b\0"
  "asx_xbz_d\0"
  "asx_xbz_h\0"
  "asx_xbz_v\0"
  "asx_xbz_w\0"
  "asx_xvabsd_b\0"
  "asx_xvabsd_bu\0"
  "asx_xvabsd_d\0"
  "asx_xvabsd_du\0"
  "asx_xvabsd_h\0"
  "asx_xvabsd_hu\0"
  "asx_xvabsd_w\0"
  "asx_xvabsd_wu\0"
  "asx_xvadd_b\0"
  "asx_xvadd_d\0"
  "asx_xvadd_h\0"
  "asx_xvadd_q\0"
  "asx_xvadd_w\0"
  "asx_xvadda_b\0"
  "asx_xvadda_d\0"
  "asx_xvadda_h\0"
  "asx_xvadda_w\0"
  "asx_xvaddi_bu\0"
  "asx_xvaddi_du\0"
  "asx_xvaddi_hu\0"
  "asx_xvaddi_wu\0"
  "asx_xvaddwev_d_w\0"
  "asx_xvaddwev_d_wu\0"
  "asx_xvaddwev_d_wu_w\0"
  "asx_xvaddwev_h_b\0"
  "asx_xvaddwev_h_bu\0"
  "asx_xvaddwev_h_bu_b\0"
  "asx_xvaddwev_q_d\0"
  "asx_xvaddwev_q_du\0"
  "asx_xvaddwev_q_du_d\0"
  "asx_xvaddwev_w_h\0"
  "asx_xvaddwev_w_hu\0"
  "asx_xvaddwev_w_hu_h\0"
  "asx_xvaddwod_d_w\0"
  "asx_xvaddwod_d_wu\0"
  "asx_xvaddwod_d_wu_w\0"
  "asx_xvaddwod_h_b\0"
  "asx_xvaddwod_h_bu\0"
  "asx_xvaddwod_h_bu_b\0"
  "asx_xvaddwod_q_d\0"
  "asx_xvaddwod_q_du\0"
  "asx_xvaddwod_q_du_d\0"
  "asx_xvaddwod_w_h\0"
  "asx_xvaddwod_w_hu\0"
  "asx_xvaddwod_w_hu_h\0"
  "asx_xvand_v\0"
  "asx_xvandi_b\0"
  "asx_xvandn_v\0"
  "asx_xvavg_b\0"
  "asx_xvavg_bu\0"
  "asx_xvavg_d\0"
  "asx_xvavg_du\0"
  "asx_xvavg_h\0"
  "asx_xvavg_hu\0"
  "asx_xvavg_w\0"
  "asx_xvavg_wu\0"
  "asx_xvavgr_b\0"
  "asx_xvavgr_bu\0"
  "asx_xvavgr_d\0"
  "asx_xvavgr_du\0"
  "asx_xvavgr_h\0"
  "asx_xvavgr_hu\0"
  "asx_xvavgr_w\0"
  "asx_xvavgr_wu\0"
  "asx_xvbitclr_b\0"
  "asx_xvbitclr_d\0"
  "asx_xvbitclr_h\0"
  "asx_xvbitclr_w\0"
  "asx_xvbitclri_b\0"
  "asx_xvbitclri_d\0"
  "asx_xvbitclri_h\0"
  "asx_xvbitclri_w\0"
  "asx_xvbitrev_b\0"
  "asx_xvbitrev_d\0"
  "asx_xvbitrev_h\0"
  "asx_xvbitrev_w\0"
  "asx_xvbitrevi_b\0"
  "asx_xvbitrevi_d\0"
  "asx_xvbitrevi_h\0"
  "asx_xvbitrevi_w\0"
  "asx_xvbitsel_v\0"
  "asx_xvbitseli_b\0"
  "asx_xvbitset_b\0"
  "asx_xvbitset_d\0"
  "asx_xvbitset_h\0"
  "asx_xvbitset_w\0"
  "asx_xvbitseti_b\0"
  "asx_xvbitseti_d\0"
  "asx_xvbitseti_h\0"
  "asx_xvbitseti_w\0"
  "asx_xvbsll_v\0"
  "asx_xvbsrl_v\0"
  "asx_xvclo_b\0"
  "asx_xvclo_d\0"
  "asx_xvclo_h\0"
  "asx_xvclo_w\0"
  "asx_xvclz_b\0"
  "asx_xvclz_d\0"
  "asx_xvclz_h\0"
  "asx_xvclz_w\0"
  "asx_xvdiv_b\0"
  "asx_xvdiv_bu\0"
  "asx_xvdiv_d\0"
  "asx_xvdiv_du\0"
  "asx_xvdiv_h\0"
  "asx_xvdiv_hu\0"
  "asx_xvdiv_w\0"
  "asx_xvdiv_wu\0"
  "asx_xvexth_d_w\0"
  "asx_xvexth_du_wu\0"
  "asx_xvexth_h_b\0"
  "asx_xvexth_hu_bu\0"
  "asx_xvexth_q_d\0"
  "asx_xvexth_qu_du\0"
  "asx_xvexth_w_h\0"
  "asx_xvexth_wu_hu\0"
  "asx_xvextl_q_d\0"
  "asx_xvextl_qu_du\0"
  "asx_xvextrins_b\0"
  "asx_xvextrins_d\0"
  "asx_xvextrins_h\0"
  "asx_xvextrins_w\0"
  "asx_xvfadd_d\0"
  "asx_xvfadd_s\0"
  "asx_xvfclass_d\0"
  "asx_xvfclass_s\0"
  "asx_xvfcmp_caf_d\0"
  "asx_xvfcmp_caf_s\0"
  "asx_xvfcmp_ceq_d\0"
  "asx_xvfcmp_ceq_s\0"
  "asx_xvfcmp_cle_d\0"
  "asx_xvfcmp_cle_s\0"
  "asx_xvfcmp_clt_d\0"
  "asx_xvfcmp_clt_s\0"
  "asx_xvfcmp_cne_d\0"
  "asx_xvfcmp_cne_s\0"
  "asx_xvfcmp_cor_d\0"
  "asx_xvfcmp_cor_s\0"
  "asx_xvfcmp_cueq_d\0"
  "asx_xvfcmp_cueq_s\0"
  "asx_xvfcmp_cule_d\0"
  "asx_xvfcmp_cule_s\0"
  "asx_xvfcmp_cult_d\0"
  "asx_xvfcmp_cult_s\0"
  "asx_xvfcmp_cun_d\0"
  "asx_xvfcmp_cun_s\0"
  "asx_xvfcmp_cune_d\0"
  "asx_xvfcmp_cune_s\0"
  "asx_xvfcmp_saf_d\0"
  "asx_xvfcmp_saf_s\0"
  "asx_xvfcmp_seq_d\0"
  "asx_xvfcmp_seq_s\0"
  "asx_xvfcmp_sle_d\0"
  "asx_xvfcmp_sle_s\0"
  "asx_xvfcmp_slt_d\0"
  "asx_xvfcmp_slt_s\0"
  "asx_xvfcmp_sne_d\0"
  "asx_xvfcmp_sne_s\0"
  "asx_xvfcmp_sor_d\0"
  "asx_xvfcmp_sor_s\0"
  "asx_xvfcmp_sueq_d\0"
  "asx_xvfcmp_sueq_s\0"
  "asx_xvfcmp_sule_d\0"
  "asx_xvfcmp_sule_s\0"
  "asx_xvfcmp_sult_d\0"
  "asx_xvfcmp_sult_s\0"
  "asx_xvfcmp_sun_d\0"
  "asx_xvfcmp_sun_s\0"
  "asx_xvfcmp_sune_d\0"
  "asx_xvfcmp_sune_s\0"
  "asx_xvfcvt_h_s\0"
  "asx_xvfcvt_s_d\0"
  "asx_xvfcvth_d_s\0"
  "asx_xvfcvth_s_h\0"
  "asx_xvfcvtl_d_s\0"
  "asx_xvfcvtl_s_h\0"
  "asx_xvfdiv_d\0"
  "asx_xvfdiv_s\0"
  "asx_xvffint_d_l\0"
  "asx_xvffint_d_lu\0"
  "asx_xvffint_s_l\0"
  "asx_xvffint_s_w\0"
  "asx_xvffint_s_wu\0"
  "asx_xvffinth_d_w\0"
  "asx_xvffintl_d_w\0"
  "asx_xvflogb_d\0"
  "asx_xvflogb_s\0"
  "asx_xvfmadd_d\0"
  "asx_xvfmadd_s\0"
  "asx_xvfmax_d\0"
  "asx_xvfmax_s\0"
  "asx_xvfmaxa_d\0"
  "asx_xvfmaxa_s\0"
  "asx_xvfmin_d\0"
  "asx_xvfmin_s\0"
  "asx_xvfmina_d\0"
  "asx_xvfmina_s\0"
  "asx_xvfmsub_d\0"
  "asx_xvfmsub_s\0"
  "asx_xvfmul_d\0"
  "asx_xvfmul_s\0"
  "asx_xvfnmadd_d\0"
  "asx_xvfnmadd_s\0"
  "asx_xvfnmsub_d\0"
  "asx_xvfnmsub_s\0"
  "asx_xvfrecip_d\0"
  "asx_xvfrecip_s\0"
  "asx_xvfrecipe_d\0"
  "asx_xvfrecipe_s\0"
  "asx_xvfrint_d\0"
  "asx_xvfrint_s\0"
  "asx_xvfrintrm_d\0"
  "asx_xvfrintrm_s\0"
  "asx_xvfrintrne_d\0"
  "asx_xvfrintrne_s\0"
  "asx_xvfrintrp_d\0"
  "asx_xvfrintrp_s\0"
  "asx_xvfrintrz_d\0"
  "asx_xvfrintrz_s\0"
  "asx_xvfrsqrt_d\0"
  "asx_xvfrsqrt_s\0"
  "asx_xvfrsqrte_d\0"
  "asx_xvfrsqrte_s\0"
  "asx_xvfrstp_b\0"
  "asx_xvfrstp_h\0"
  "asx_xvfrstpi_b\0"
  "asx_xvfrstpi_h\0"
  "asx_xvfsqrt_d\0"
  "asx_xvfsqrt_s\0"
  "asx_xvfsub_d\0"
  "asx_xvfsub_s\0"
  "asx_xvftint_l_d\0"
  "asx_xvftint_lu_d\0"
  "asx_xvftint_w_d\0"
  "asx_xvftint_w_s\0"
  "asx_xvftint_wu_s\0"
  "asx_xvftinth_l_s\0"
  "asx_xvftintl_l_s\0"
  "asx_xvftintrm_l_d\0"
  "asx_xvftintrm_w_d\0"
  "asx_xvftintrm_w_s\0"
  "asx_xvftintrmh_l_s\0"
  "asx_xvftintrml_l_s\0"
  "asx_xvftintrne_l_d\0"
  "asx_xvftintrne_w_d\0"
  "asx_xvftintrne_w_s\0"
  "asx_xvftintrneh_l_s\0"
  "asx_xvftintrnel_l_s\0"
  "asx_xvftintrp_l_d\0"
  "asx_xvftintrp_w_d\0"
  "asx_xvftintrp_w_s\0"
  "asx_xvftintrph_l_s\0"
  "asx_xvftintrpl_l_s\0"
  "asx_xvftintrz_l_d\0"
  "asx_xvftintrz_lu_d\0"
  "asx_xvftintrz_w_d\0"
  "asx_xvftintrz_w_s\0"
  "asx_xvftintrz_wu_s\0"
  "asx_xvftintrzh_l_s\0"
  "asx_xvftintrzl_l_s\0"
  "asx_xvhaddw_d_w\0"
  "asx_xvhaddw_du_wu\0"
  "asx_xvhaddw_h_b\0"
  "asx_xvhaddw_hu_bu\0"
  "asx_xvhaddw_q_d\0"
  "asx_xvhaddw_qu_du\0"
  "asx_xvhaddw_w_h\0"
  "asx_xvhaddw_wu_hu\0"
  "asx_xvhsubw_d_w\0"
  "asx_xvhsubw_du_wu\0"
  "asx_xvhsubw_h_b\0"
  "asx_xvhsubw_hu_bu\0"
  "asx_xvhsubw_q_d\0"
  "asx_xvhsubw_qu_du\0"
  "asx_xvhsubw_w_h\0"
  "asx_xvhsubw_wu_hu\0"
  "asx_xvilvh_b\0"
  "asx_xvilvh_d\0"
  "asx_xvilvh_h\0"
  "asx_xvilvh_w\0"
  "asx_xvilvl_b\0"
  "asx_xvilvl_d\0"
  "asx_xvilvl_h\0"
  "asx_xvilvl_w\0"
  "asx_xvinsgr2vr_d\0"
  "asx_xvinsgr2vr_w\0"
  "asx_xvinsve0_d\0"
  "asx_xvinsve0_w\0"
  "asx_xvld\0"
  "asx_xvldi\0"
  "asx_xvldrepl_b\0"
  "asx_xvldrepl_d\0"
  "asx_xvldrepl_h\0"
  "asx_xvldrepl_w\0"
  "asx_xvldx\0"
  "asx_xvmadd_b\0"
  "asx_xvmadd_d\0"
  "asx_xvmadd_h\0"
  "asx_xvmadd_w\0"
  "asx_xvmaddwev_d_w\0"
  "asx_xvmaddwev_d_wu\0"
  "asx_xvmaddwev_d_wu_w\0"
  "asx_xvmaddwev_h_b\0"
  "asx_xvmaddwev_h_bu\0"
  "asx_xvmaddwev_h_bu_b\0"
  "asx_xvmaddwev_q_d\0"
  "asx_xvmaddwev_q_du\0"
  "asx_xvmaddwev_q_du_d\0"
  "asx_xvmaddwev_w_h\0"
  "asx_xvmaddwev_w_hu\0"
  "asx_xvmaddwev_w_hu_h\0"
  "asx_xvmaddwod_d_w\0"
  "asx_xvmaddwod_d_wu\0"
  "asx_xvmaddwod_d_wu_w\0"
  "asx_xvmaddwod_h_b\0"
  "asx_xvmaddwod_h_bu\0"
  "asx_xvmaddwod_h_bu_b\0"
  "asx_xvmaddwod_q_d\0"
  "asx_xvmaddwod_q_du\0"
  "asx_xvmaddwod_q_du_d\0"
  "asx_xvmaddwod_w_h\0"
  "asx_xvmaddwod_w_hu\0"
  "asx_xvmaddwod_w_hu_h\0"
  "asx_xvmax_b\0"
  "asx_xvmax_bu\0"
  "asx_xvmax_d\0"
  "asx_xvmax_du\0"
  "asx_xvmax_h\0"
  "asx_xvmax_hu\0"
  "asx_xvmax_w\0"
  "asx_xvmax_wu\0"
  "asx_xvmaxi_b\0"
  "asx_xvmaxi_bu\0"
  "asx_xvmaxi_d\0"
  "asx_xvmaxi_du\0"
  "asx_xvmaxi_h\0"
  "asx_xvmaxi_hu\0"
  "asx_xvmaxi_w\0"
  "asx_xvmaxi_wu\0"
  "asx_xvmin_b\0"
  "asx_xvmin_bu\0"
  "asx_xvmin_d\0"
  "asx_xvmin_du\0"
  "asx_xvmin_h\0"
  "asx_xvmin_hu\0"
  "asx_xvmin_w\0"
  "asx_xvmin_wu\0"
  "asx_xvmini_b\0"
  "asx_xvmini_bu\0"
  "asx_xvmini_d\0"
  "asx_xvmini_du\0"
  "asx_xvmini_h\0"
  "asx_xvmini_hu\0"
  "asx_xvmini_w\0"
  "asx_xvmini_wu\0"
  "asx_xvmod_b\0"
  "asx_xvmod_bu\0"
  "asx_xvmod_d\0"
  "asx_xvmod_du\0"
  "asx_xvmod_h\0"
  "asx_xvmod_hu\0"
  "asx_xvmod_w\0"
  "asx_xvmod_wu\0"
  "asx_xvmskgez_b\0"
  "asx_xvmskltz_b\0"
  "asx_xvmskltz_d\0"
  "asx_xvmskltz_h\0"
  "asx_xvmskltz_w\0"
  "asx_xvmsknz_b\0"
  "asx_xvmsub_b\0"
  "asx_xvmsub_d\0"
  "asx_xvmsub_h\0"
  "asx_xvmsub_w\0"
  "asx_xvmuh_b\0"
  "asx_xvmuh_bu\0"
  "asx_xvmuh_d\0"
  "asx_xvmuh_du\0"
  "asx_xvmuh_h\0"
  "asx_xvmuh_hu\0"
  "asx_xvmuh_w\0"
  "asx_xvmuh_wu\0"
  "asx_xvmul_b\0"
  "asx_xvmul_d\0"
  "asx_xvmul_h\0"
  "asx_xvmul_w\0"
  "asx_xvmulwev_d_w\0"
  "asx_xvmulwev_d_wu\0"
  "asx_xvmulwev_d_wu_w\0"
  "asx_xvmulwev_h_b\0"
  "asx_xvmulwev_h_bu\0"
  "asx_xvmulwev_h_bu_b\0"
  "asx_xvmulwev_q_d\0"
  "asx_xvmulwev_q_du\0"
  "asx_xvmulwev_q_du_d\0"
  "asx_xvmulwev_w_h\0"
  "asx_xvmulwev_w_hu\0"
  "asx_xvmulwev_w_hu_h\0"
  "asx_xvmulwod_d_w\0"
  "asx_xvmulwod_d_wu\0"
  "asx_xvmulwod_d_wu_w\0"
  "asx_xvmulwod_h_b\0"
  "asx_xvmulwod_h_bu\0"
  "asx_xvmulwod_h_bu_b\0"
  "asx_xvmulwod_q_d\0"
  "asx_xvmulwod_q_du\0"
  "asx_xvmulwod_q_du_d\0"
  "asx_xvmulwod_w_h\0"
  "asx_xvmulwod_w_hu\0"
  "asx_xvmulwod_w_hu_h\0"
  "asx_xvneg_b\0"
  "asx_xvneg_d\0"
  "asx_xvneg_h\0"
  "asx_xvneg_w\0"
  "asx_xvnor_v\0"
  "asx_xvnori_b\0"
  "asx_xvor_v\0"
  "asx_xvori_b\0"
  "asx_xvorn_v\0"
  "asx_xvpackev_b\0"
  "asx_xvpackev_d\0"
  "asx_xvpackev_h\0"
  "asx_xvpackev_w\0"
  "asx_xvpackod_b\0"
  "asx_xvpackod_d\0"
  "asx_xvpackod_h\0"
  "asx_xvpackod_w\0"
  "asx_xvpcnt_b\0"
  "asx_xvpcnt_d\0"
  "asx_xvpcnt_h\0"
  "asx_xvpcnt_w\0"
  "asx_xvperm_w\0"
  "asx_xvpermi_d\0"
  "asx_xvpermi_q\0"
  "asx_xvpermi_w\0"
  "asx_xvpickev_b\0"
  "asx_xvpickev_d\0"
  "asx_xvpickev_h\0"
  "asx_xvpickev_w\0"
  "asx_xvpickod_b\0"
  "asx_xvpickod_d\0"
  "asx_xvpickod_h\0"
  "asx_xvpickod_w\0"
  "asx_xvpickve2gr_d\0"
  "asx_xvpickve2gr_du\0"
  "asx_xvpickve2gr_w\0"
  "asx_xvpickve2gr_wu\0"
  "asx_xvpickve_d\0"
  "asx_xvpickve_d_f\0"
  "asx_xvpickve_w\0"
  "asx_xvpickve_w_f\0"
  "asx_xvrepl128vei_b\0"
  "asx_xvrepl128vei_d\0"
  "asx_xvrepl128vei_h\0"
  "asx_xvrepl128vei_w\0"
  "asx_xvreplgr2vr_b\0"
  "asx_xvreplgr2vr_d\0"
  "asx_xvreplgr2vr_h\0"
  "asx_xvreplgr2vr_w\0"
  "asx_xvrepli_b\0"
  "asx_xvrepli_d\0"
  "asx_xvrepli_h\0"
  "asx_xvrepli_w\0"
  "asx_xvreplve0_b\0"
  "asx_xvreplve0_d\0"
  "asx_xvreplve0_h\0"
  "asx_xvreplve0_q\0"
  "asx_xvreplve0_w\0"
  "asx_xvreplve_b\0"
  "asx_xvreplve_d\0"
  "asx_xvreplve_h\0"
  "asx_xvreplve_w\0"
  "asx_xvrotr_b\0"
  "asx_xvrotr_d\0"
  "asx_xvrotr_h\0"
  "asx_xvrotr_w\0"
  "asx_xvrotri_b\0"
  "asx_xvrotri_d\0"
  "asx_xvrotri_h\0"
  "asx_xvrotri_w\0"
  "asx_xvsadd_b\0"
  "asx_xvsadd_bu\0"
  "asx_xvsadd_d\0"
  "asx_xvsadd_du\0"
  "asx_xvsadd_h\0"
  "asx_xvsadd_hu\0"
  "asx_xvsadd_w\0"
  "asx_xvsadd_wu\0"
  "asx_xvsat_b\0"
  "asx_xvsat_bu\0"
  "asx_xvsat_d\0"
  "asx_xvsat_du\0"
  "asx_xvsat_h\0"
  "asx_xvsat_hu\0"
  "asx_xvsat_w\0"
  "asx_xvsat_wu\0"
  "asx_xvseq_b\0"
  "asx_xvseq_d\0"
  "asx_xvseq_h\0"
  "asx_xvseq_w\0"
  "asx_xvseqi_b\0"
  "asx_xvseqi_d\0"
  "asx_xvseqi_h\0"
  "asx_xvseqi_w\0"
  "asx_xvshuf4i_b\0"
  "asx_xvshuf4i_d\0"
  "asx_xvshuf4i_h\0"
  "asx_xvshuf4i_w\0"
  "asx_xvshuf_b\0"
  "asx_xvshuf_d\0"
  "asx_xvshuf_h\0"
  "asx_xvshuf_w\0"
  "asx_xvsigncov_b\0"
  "asx_xvsigncov_d\0"
  "asx_xvsigncov_h\0"
  "asx_xvsigncov_w\0"
  "asx_xvsle_b\0"
  "asx_xvsle_bu\0"
  "asx_xvsle_d\0"
  "asx_xvsle_du\0"
  "asx_xvsle_h\0"
  "asx_xvsle_hu\0"
  "asx_xvsle_w\0"
  "asx_xvsle_wu\0"
  "asx_xvslei_b\0"
  "asx_xvslei_bu\0"
  "asx_xvslei_d\0"
  "asx_xvslei_du\0"
  "asx_xvslei_h\0"
  "asx_xvslei_hu\0"
  "asx_xvslei_w\0"
  "asx_xvslei_wu\0"
  "asx_xvsll_b\0"
  "asx_xvsll_d\0"
  "asx_xvsll_h\0"
  "asx_xvsll_w\0"
  "asx_xvslli_b\0"
  "asx_xvslli_d\0"
  "asx_xvslli_h\0"
  "asx_xvslli_w\0"
  "asx_xvsllwil_d_w\0"
  "asx_xvsllwil_du_wu\0"
  "asx_xvsllwil_h_b\0"
  "asx_xvsllwil_hu_bu\0"
  "asx_xvsllwil_w_h\0"
  "asx_xvsllwil_wu_hu\0"
  "asx_xvslt_b\0"
  "asx_xvslt_bu\0"
  "asx_xvslt_d\0"
  "asx_xvslt_du\0"
  "asx_xvslt_h\0"
  "asx_xvslt_hu\0"
  "asx_xvslt_w\0"
  "asx_xvslt_wu\0"
  "asx_xvslti_b\0"
  "asx_xvslti_bu\0"
  "asx_xvslti_d\0"
  "asx_xvslti_du\0"
  "asx_xvslti_h\0"
  "asx_xvslti_hu\0"
  "asx_xvslti_w\0"
  "asx_xvslti_wu\0"
  "asx_xvsra_b\0"
  "asx_xvsra_d\0"
  "asx_xvsra_h\0"
  "asx_xvsra_w\0"
  "asx_xvsrai_b\0"
  "asx_xvsrai_d\0"
  "asx_xvsrai_h\0"
  "asx_xvsrai_w\0"
  "asx_xvsran_b_h\0"
  "asx_xvsran_h_w\0"
  "asx_xvsran_w_d\0"
  "asx_xvsrani_b_h\0"
  "asx_xvsrani_d_q\0"
  "asx_xvsrani_h_w\0"
  "asx_xvsrani_w_d\0"
  "asx_xvsrar_b\0"
  "asx_xvsrar_d\0"
  "asx_xvsrar_h\0"
  "asx_xvsrar_w\0"
  "asx_xvsrari_b\0"
  "asx_xvsrari_d\0"
  "asx_xvsrari_h\0"
  "asx_xvsrari_w\0"
  "asx_xvsrarn_b_h\0"
  "asx_xvsrarn_h_w\0"
  "asx_xvsrarn_w_d\0"
  "asx_xvsrarni_b_h\0"
  "asx_xvsrarni_d_q\0"
  "asx_xvsrarni_h_w\0"
  "asx_xvsrarni_w_d\0"
  "asx_xvsrl_b\0"
  "asx_xvsrl_d\0"
  "asx_xvsrl_h\0"
  "asx_xvsrl_w\0"
  "asx_xvsrli_b\0"
  "asx_xvsrli_d\0"
  "asx_xvsrli_h\0"
  "asx_xvsrli_w\0"
  "asx_xvsrln_b_h\0"
  "asx_xvsrln_h_w\0"
  "asx_xvsrln_w_d\0"
  "asx_xvsrlni_b_h\0"
  "asx_xvsrlni_d_q\0"
  "asx_xvsrlni_h_w\0"
  "asx_xvsrlni_w_d\0"
  "asx_xvsrlr_b\0"
  "asx_xvsrlr_d\0"
  "asx_xvsrlr_h\0"
  "asx_xvsrlr_w\0"
  "asx_xvsrlri_b\0"
  "asx_xvsrlri_d\0"
  "asx_xvsrlri_h\0"
  "asx_xvsrlri_w\0"
  "asx_xvsrlrn_b_h\0"
  "asx_xvsrlrn_h_w\0"
  "asx_xvsrlrn_w_d\0"
  "asx_xvsrlrni_b_h\0"
  "asx_xvsrlrni_d_q\0"
  "asx_xvsrlrni_h_w\0"
  "asx_xvsrlrni_w_d\0"
  "asx_xvssran_b_h\0"
  "asx_xvssran_bu_h\0"
  "asx_xvssran_h_w\0"
  "asx_xvssran_hu_w\0"
  "asx_xvssran_w_d\0"
  "asx_xvssran_wu_d\0"
  "asx_xvssrani_b_h\0"
  "asx_xvssrani_bu_h\0"
  "asx_xvssrani_d_q\0"
  "asx_xvssrani_du_q\0"
  "asx_xvssrani_h_w\0"
  "asx_xvssrani_hu_w\0"
  "asx_xvssrani_w_d\0"
  "asx_xvssrani_wu_d\0"
  "asx_xvssrarn_b_h\0"
  "asx_xvssrarn_bu_h\0"
  "asx_xvssrarn_h_w\0"
  "asx_xvssrarn_hu_w\0"
  "asx_xvssrarn_w_d\0"
  "asx_xvssrarn_wu_d\0"
  "asx_xvssrarni_b_h\0"
  "asx_xvssrarni_bu_h\0"
  "asx_xvssrarni_d_q\0"
  "asx_xvssrarni_du_q\0"
  "asx_xvssrarni_h_w\0"
  "asx_xvssrarni_hu_w\0"
  "asx_xvssrarni_w_d\0"
  "asx_xvssrarni_wu_d\0"
  "asx_xvssrln_b_h\0"
  "asx_xvssrln_bu_h\0"
  "asx_xvssrln_h_w\0"
  "asx_xvssrln_hu_w\0"
  "asx_xvssrln_w_d\0"
  "asx_xvssrln_wu_d\0"
  "asx_xvssrlni_b_h\0"
  "asx_xvssrlni_bu_h\0"
  "asx_xvssrlni_d_q\0"
  "asx_xvssrlni_du_q\0"
  "asx_xvssrlni_h_w\0"
  "asx_xvssrlni_hu_w\0"
  "asx_xvssrlni_w_d\0"
  "asx_xvssrlni_wu_d\0"
  "asx_xvssrlrn_b_h\0"
  "asx_xvssrlrn_bu_h\0"
  "asx_xvssrlrn_h_w\0"
  "asx_xvssrlrn_hu_w\0"
  "asx_xvssrlrn_w_d\0"
  "asx_xvssrlrn_wu_d\0"
  "asx_xvssrlrni_b_h\0"
  "asx_xvssrlrni_bu_h\0"
  "asx_xvssrlrni_d_q\0"
  "asx_xvssrlrni_du_q\0"
  "asx_xvssrlrni_h_w\0"
  "asx_xvssrlrni_hu_w\0"
  "asx_xvssrlrni_w_d\0"
  "asx_xvssrlrni_wu_d\0"
  "asx_xvssub_b\0"
  "asx_xvssub_bu\0"
  "asx_xvssub_d\0"
  "asx_xvssub_du\0"
  "asx_xvssub_h\0"
  "asx_xvssub_hu\0"
  "asx_xvssub_w\0"
  "asx_xvssub_wu\0"
  "asx_xvst\0"
  "asx_xvstelm_b\0"
  "asx_xvstelm_d\0"
  "asx_xvstelm_h\0"
  "asx_xvstelm_w\0"
  "asx_xvstx\0"
  "asx_xvsub_b\0"
  "asx_xvsub_d\0"
  "asx_xvsub_h\0"
  "asx_xvsub_q\0"
  "asx_xvsub_w\0"
  "asx_xvsubi_bu\0"
  "asx_xvsubi_du\0"
  "asx_xvsubi_hu\0"
  "asx_xvsubi_wu\0"
  "asx_xvsubwev_d_w\0"
  "asx_xvsubwev_d_wu\0"
  "asx_xvsubwev_h_b\0"
  "asx_xvsubwev_h_bu\0"
  "asx_xvsubwev_q_d\0"
  "asx_xvsubwev_q_du\0"
  "asx_xvsubwev_w_h\0"
  "asx_xvsubwev_w_hu\0"
  "asx_xvsubwod_d_w\0"
  "asx_xvsubwod_d_wu\0"
  "asx_xvsubwod_h_b\0"
  "asx_xvsubwod_h_bu\0"
  "asx_xvsubwod_q_d\0"
  "asx_xvsubwod_q_du\0"
  "asx_xvsubwod_w_h\0"
  "asx_xvsubwod_w_hu\0"
  "asx_xvxor_v\0"
  "asx_xvxori_b\0"
  "oongarch_asrtgt_d\0"
  "oongarch_asrtle_d\0"
  "oongarch_break\0"
  "oongarch_cacop_d\0"
  "oongarch_cacop_w\0"
  "oongarch_cpucfg\0"
  "oongarch_crc_w_b_w\0"
  "oongarch_crc_w_d_w\0"
  "oongarch_crc_w_h_w\0"
  "oongarch_crc_w_w_w\0"
  "oongarch_crcc_w_b_w\0"
  "oongarch_crcc_w_d_w\0"
  "oongarch_crcc_w_h_w\0"
  "oongarch_crcc_w_w_w\0"
  "oongarch_csrrd_d\0"
  "oongarch_csrrd_w\0"
  "oongarch_csrwr_d\0"
  "oongarch_csrwr_w\0"
  "oongarch_csrxchg_d\0"
  "oongarch_csrxchg_w\0"
  "oongarch_dbar\0"
  "oongarch_frecipe_d\0"
  "oongarch_frecipe_s\0"
  "oongarch_frsqrte_d\0"
  "oongarch_frsqrte_s\0"
  "oongarch_ibar\0"
  "oongarch_iocsrrd_b\0"
  "oongarch_iocsrrd_d\0"
  "oongarch_iocsrrd_h\0"
  "oongarch_iocsrrd_w\0"
  "oongarch_iocsrwr_b\0"
  "oongarch_iocsrwr_d\0"
  "oongarch_iocsrwr_h\0"
  "oongarch_iocsrwr_w\0"
  "oongarch_lddir_d\0"
  "oongarch_ldpte_d\0"
  "oongarch_movfcsr2gr\0"
  "oongarch_movgr2fcsr\0"
  "oongarch_syscall\0"
  "sx_bnz_b\0"
  "sx_bnz_d\0"
  "sx_bnz_h\0"
  "sx_bnz_v\0"
  "sx_bnz_w\0"
  "sx_bz_b\0"
  "sx_bz_d\0"
  "sx_bz_h\0"
  "sx_bz_v\0"
  "sx_bz_w\0"
  "sx_vabsd_b\0"
  "sx_vabsd_bu\0"
  "sx_vabsd_d\0"
  "sx_vabsd_du\0"
  "sx_vabsd_h\0"
  "sx_vabsd_hu\0"
  "sx_vabsd_w\0"
  "sx_vabsd_wu\0"
  "sx_vadd_b\0"
  "sx_vadd_d\0"
  "sx_vadd_h\0"
  "sx_vadd_q\0"
  "sx_vadd_w\0"
  "sx_vadda_b\0"
  "sx_vadda_d\0"
  "sx_vadda_h\0"
  "sx_vadda_w\0"
  "sx_vaddi_bu\0"
  "sx_vaddi_du\0"
  "sx_vaddi_hu\0"
  "sx_vaddi_wu\0"
  "sx_vaddwev_d_w\0"
  "sx_vaddwev_d_wu\0"
  "sx_vaddwev_d_wu_w\0"
  "sx_vaddwev_h_b\0"
  "sx_vaddwev_h_bu\0"
  "sx_vaddwev_h_bu_b\0"
  "sx_vaddwev_q_d\0"
  "sx_vaddwev_q_du\0"
  "sx_vaddwev_q_du_d\0"
  "sx_vaddwev_w_h\0"
  "sx_vaddwev_w_hu\0"
  "sx_vaddwev_w_hu_h\0"
  "sx_vaddwod_d_w\0"
  "sx_vaddwod_d_wu\0"
  "sx_vaddwod_d_wu_w\0"
  "sx_vaddwod_h_b\0"
  "sx_vaddwod_h_bu\0"
  "sx_vaddwod_h_bu_b\0"
  "sx_vaddwod_q_d\0"
  "sx_vaddwod_q_du\0"
  "sx_vaddwod_q_du_d\0"
  "sx_vaddwod_w_h\0"
  "sx_vaddwod_w_hu\0"
  "sx_vaddwod_w_hu_h\0"
  "sx_vand_v\0"
  "sx_vandi_b\0"
  "sx_vandn_v\0"
  "sx_vavg_b\0"
  "sx_vavg_bu\0"
  "sx_vavg_d\0"
  "sx_vavg_du\0"
  "sx_vavg_h\0"
  "sx_vavg_hu\0"
  "sx_vavg_w\0"
  "sx_vavg_wu\0"
  "sx_vavgr_b\0"
  "sx_vavgr_bu\0"
  "sx_vavgr_d\0"
  "sx_vavgr_du\0"
  "sx_vavgr_h\0"
  "sx_vavgr_hu\0"
  "sx_vavgr_w\0"
  "sx_vavgr_wu\0"
  "sx_vbitclr_b\0"
  "sx_vbitclr_d\0"
  "sx_vbitclr_h\0"
  "sx_vbitclr_w\0"
  "sx_vbitclri_b\0"
  "sx_vbitclri_d\0"
  "sx_vbitclri_h\0"
  "sx_vbitclri_w\0"
  "sx_vbitrev_b\0"
  "sx_vbitrev_d\0"
  "sx_vbitrev_h\0"
  "sx_vbitrev_w\0"
  "sx_vbitrevi_b\0"
  "sx_vbitrevi_d\0"
  "sx_vbitrevi_h\0"
  "sx_vbitrevi_w\0"
  "sx_vbitsel_v\0"
  "sx_vbitseli_b\0"
  "sx_vbitset_b\0"
  "sx_vbitset_d\0"
  "sx_vbitset_h\0"
  "sx_vbitset_w\0"
  "sx_vbitseti_b\0"
  "sx_vbitseti_d\0"
  "sx_vbitseti_h\0"
  "sx_vbitseti_w\0"
  "sx_vbsll_v\0"
  "sx_vbsrl_v\0"
  "sx_vclo_b\0"
  "sx_vclo_d\0"
  "sx_vclo_h\0"
  "sx_vclo_w\0"
  "sx_vclz_b\0"
  "sx_vclz_d\0"
  "sx_vclz_h\0"
  "sx_vclz_w\0"
  "sx_vdiv_b\0"
  "sx_vdiv_bu\0"
  "sx_vdiv_d\0"
  "sx_vdiv_du\0"
  "sx_vdiv_h\0"
  "sx_vdiv_hu\0"
  "sx_vdiv_w\0"
  "sx_vdiv_wu\0"
  "sx_vexth_d_w\0"
  "sx_vexth_du_wu\0"
  "sx_vexth_h_b\0"
  "sx_vexth_hu_bu\0"
  "sx_vexth_q_d\0"
  "sx_vexth_qu_du\0"
  "sx_vexth_w_h\0"
  "sx_vexth_wu_hu\0"
  "sx_vextl_q_d\0"
  "sx_vextl_qu_du\0"
  "sx_vextrins_b\0"
  "sx_vextrins_d\0"
  "sx_vextrins_h\0"
  "sx_vextrins_w\0"
  "sx_vfadd_d\0"
  "sx_vfadd_s\0"
  "sx_vfclass_d\0"
  "sx_vfclass_s\0"
  "sx_vfcmp_caf_d\0"
  "sx_vfcmp_caf_s\0"
  "sx_vfcmp_ceq_d\0"
  "sx_vfcmp_ceq_s\0"
  "sx_vfcmp_cle_d\0"
  "sx_vfcmp_cle_s\0"
  "sx_vfcmp_clt_d\0"
  "sx_vfcmp_clt_s\0"
  "sx_vfcmp_cne_d\0"
  "sx_vfcmp_cne_s\0"
  "sx_vfcmp_cor_d\0"
  "sx_vfcmp_cor_s\0"
  "sx_vfcmp_cueq_d\0"
  "sx_vfcmp_cueq_s\0"
  "sx_vfcmp_cule_d\0"
  "sx_vfcmp_cule_s\0"
  "sx_vfcmp_cult_d\0"
  "sx_vfcmp_cult_s\0"
  "sx_vfcmp_cun_d\0"
  "sx_vfcmp_cun_s\0"
  "sx_vfcmp_cune_d\0"
  "sx_vfcmp_cune_s\0"
  "sx_vfcmp_saf_d\0"
  "sx_vfcmp_saf_s\0"
  "sx_vfcmp_seq_d\0"
  "sx_vfcmp_seq_s\0"
  "sx_vfcmp_sle_d\0"
  "sx_vfcmp_sle_s\0"
  "sx_vfcmp_slt_d\0"
  "sx_vfcmp_slt_s\0"
  "sx_vfcmp_sne_d\0"
  "sx_vfcmp_sne_s\0"
  "sx_vfcmp_sor_d\0"
  "sx_vfcmp_sor_s\0"
  "sx_vfcmp_sueq_d\0"
  "sx_vfcmp_sueq_s\0"
  "sx_vfcmp_sule_d\0"
  "sx_vfcmp_sule_s\0"
  "sx_vfcmp_sult_d\0"
  "sx_vfcmp_sult_s\0"
  "sx_vfcmp_sun_d\0"
  "sx_vfcmp_sun_s\0"
  "sx_vfcmp_sune_d\0"
  "sx_vfcmp_sune_s\0"
  "sx_vfcvt_h_s\0"
  "sx_vfcvt_s_d\0"
  "sx_vfcvth_d_s\0"
  "sx_vfcvth_s_h\0"
  "sx_vfcvtl_d_s\0"
  "sx_vfcvtl_s_h\0"
  "sx_vfdiv_d\0"
  "sx_vfdiv_s\0"
  "sx_vffint_d_l\0"
  "sx_vffint_d_lu\0"
  "sx_vffint_s_l\0"
  "sx_vffint_s_w\0"
  "sx_vffint_s_wu\0"
  "sx_vffinth_d_w\0"
  "sx_vffintl_d_w\0"
  "sx_vflogb_d\0"
  "sx_vflogb_s\0"
  "sx_vfmadd_d\0"
  "sx_vfmadd_s\0"
  "sx_vfmax_d\0"
  "sx_vfmax_s\0"
  "sx_vfmaxa_d\0"
  "sx_vfmaxa_s\0"
  "sx_vfmin_d\0"
  "sx_vfmin_s\0"
  "sx_vfmina_d\0"
  "sx_vfmina_s\0"
  "sx_vfmsub_d\0"
  "sx_vfmsub_s\0"
  "sx_vfmul_d\0"
  "sx_vfmul_s\0"
  "sx_vfnmadd_d\0"
  "sx_vfnmadd_s\0"
  "sx_vfnmsub_d\0"
  "sx_vfnmsub_s\0"
  "sx_vfrecip_d\0"
  "sx_vfrecip_s\0"
  "sx_vfrecipe_d\0"
  "sx_vfrecipe_s\0"
  "sx_vfrint_d\0"
  "sx_vfrint_s\0"
  "sx_vfrintrm_d\0"
  "sx_vfrintrm_s\0"
  "sx_vfrintrne_d\0"
  "sx_vfrintrne_s\0"
  "sx_vfrintrp_d\0"
  "sx_vfrintrp_s\0"
  "sx_vfrintrz_d\0"
  "sx_vfrintrz_s\0"
  "sx_vfrsqrt_d\0"
  "sx_vfrsqrt_s\0"
  "sx_vfrsqrte_d\0"
  "sx_vfrsqrte_s\0"
  "sx_vfrstp_b\0"
  "sx_vfrstp_h\0"
  "sx_vfrstpi_b\0"
  "sx_vfrstpi_h\0"
  "sx_vfsqrt_d\0"
  "sx_vfsqrt_s\0"
  "sx_vfsub_d\0"
  "sx_vfsub_s\0"
  "sx_vftint_l_d\0"
  "sx_vftint_lu_d\0"
  "sx_vftint_w_d\0"
  "sx_vftint_w_s\0"
  "sx_vftint_wu_s\0"
  "sx_vftinth_l_s\0"
  "sx_vftintl_l_s\0"
  "sx_vftintrm_l_d\0"
  "sx_vftintrm_w_d\0"
  "sx_vftintrm_w_s\0"
  "sx_vftintrmh_l_s\0"
  "sx_vftintrml_l_s\0"
  "sx_vftintrne_l_d\0"
  "sx_vftintrne_w_d\0"
  "sx_vftintrne_w_s\0"
  "sx_vftintrneh_l_s\0"
  "sx_vftintrnel_l_s\0"
  "sx_vftintrp_l_d\0"
  "sx_vftintrp_w_d\0"
  "sx_vftintrp_w_s\0"
  "sx_vftintrph_l_s\0"
  "sx_vftintrpl_l_s\0"
  "sx_vftintrz_l_d\0"
  "sx_vftintrz_lu_d\0"
  "sx_vftintrz_w_d\0"
  "sx_vftintrz_w_s\0"
  "sx_vftintrz_wu_s\0"
  "sx_vftintrzh_l_s\0"
  "sx_vftintrzl_l_s\0"
  "sx_vhaddw_d_w\0"
  "sx_vhaddw_du_wu\0"
  "sx_vhaddw_h_b\0"
  "sx_vhaddw_hu_bu\0"
  "sx_vhaddw_q_d\0"
  "sx_vhaddw_qu_du\0"
  "sx_vhaddw_w_h\0"
  "sx_vhaddw_wu_hu\0"
  "sx_vhsubw_d_w\0"
  "sx_vhsubw_du_wu\0"
  "sx_vhsubw_h_b\0"
  "sx_vhsubw_hu_bu\0"
  "sx_vhsubw_q_d\0"
  "sx_vhsubw_qu_du\0"
  "sx_vhsubw_w_h\0"
  "sx_vhsubw_wu_hu\0"
  "sx_vilvh_b\0"
  "sx_vilvh_d\0"
  "sx_vilvh_h\0"
  "sx_vilvh_w\0"
  "sx_vilvl_b\0"
  "sx_vilvl_d\0"
  "sx_vilvl_h\0"
  "sx_vilvl_w\0"
  "sx_vinsgr2vr_b\0"
  "sx_vinsgr2vr_d\0"
  "sx_vinsgr2vr_h\0"
  "sx_vinsgr2vr_w\0"
  "sx_vld\0"
  "sx_vldi\0"
  "sx_vldrepl_b\0"
  "sx_vldrepl_d\0"
  "sx_vldrepl_h\0"
  "sx_vldrepl_w\0"
  "sx_vldx\0"
  "sx_vmadd_b\0"
  "sx_vmadd_d\0"
  "sx_vmadd_h\0"
  "sx_vmadd_w\0"
  "sx_vmaddwev_d_w\0"
  "sx_vmaddwev_d_wu\0"
  "sx_vmaddwev_d_wu_w\0"
  "sx_vmaddwev_h_b\0"
  "sx_vmaddwev_h_bu\0"
  "sx_vmaddwev_h_bu_b\0"
  "sx_vmaddwev_q_d\0"
  "sx_vmaddwev_q_du\0"
  "sx_vmaddwev_q_du_d\0"
  "sx_vmaddwev_w_h\0"
  "sx_vmaddwev_w_hu\0"
  "sx_vmaddwev_w_hu_h\0"
  "sx_vmaddwod_d_w\0"
  "sx_vmaddwod_d_wu\0"
  "sx_vmaddwod_d_wu_w\0"
  "sx_vmaddwod_h_b\0"
  "sx_vmaddwod_h_bu\0"
  "sx_vmaddwod_h_bu_b\0"
  "sx_vmaddwod_q_d\0"
  "sx_vmaddwod_q_du\0"
  "sx_vmaddwod_q_du_d\0"
  "sx_vmaddwod_w_h\0"
  "sx_vmaddwod_w_hu\0"
  "sx_vmaddwod_w_hu_h\0"
  "sx_vmax_b\0"
  "sx_vmax_bu\0"
  "sx_vmax_d\0"
  "sx_vmax_du\0"
  "sx_vmax_h\0"
  "sx_vmax_hu\0"
  "sx_vmax_w\0"
  "sx_vmax_wu\0"
  "sx_vmaxi_b\0"
  "sx_vmaxi_bu\0"
  "sx_vmaxi_d\0"
  "sx_vmaxi_du\0"
  "sx_vmaxi_h\0"
  "sx_vmaxi_hu\0"
  "sx_vmaxi_w\0"
  "sx_vmaxi_wu\0"
  "sx_vmin_b\0"
  "sx_vmin_bu\0"
  "sx_vmin_d\0"
  "sx_vmin_du\0"
  "sx_vmin_h\0"
  "sx_vmin_hu\0"
  "sx_vmin_w\0"
  "sx_vmin_wu\0"
  "sx_vmini_b\0"
  "sx_vmini_bu\0"
  "sx_vmini_d\0"
  "sx_vmini_du\0"
  "sx_vmini_h\0"
  "sx_vmini_hu\0"
  "sx_vmini_w\0"
  "sx_vmini_wu\0"
  "sx_vmod_b\0"
  "sx_vmod_bu\0"
  "sx_vmod_d\0"
  "sx_vmod_du\0"
  "sx_vmod_h\0"
  "sx_vmod_hu\0"
  "sx_vmod_w\0"
  "sx_vmod_wu\0"
  "sx_vmskgez_b\0"
  "sx_vmskltz_b\0"
  "sx_vmskltz_d\0"
  "sx_vmskltz_h\0"
  "sx_vmskltz_w\0"
  "sx_vmsknz_b\0"
  "sx_vmsub_b\0"
  "sx_vmsub_d\0"
  "sx_vmsub_h\0"
  "sx_vmsub_w\0"
  "sx_vmuh_b\0"
  "sx_vmuh_bu\0"
  "sx_vmuh_d\0"
  "sx_vmuh_du\0"
  "sx_vmuh_h\0"
  "sx_vmuh_hu\0"
  "sx_vmuh_w\0"
  "sx_vmuh_wu\0"
  "sx_vmul_b\0"
  "sx_vmul_d\0"
  "sx_vmul_h\0"
  "sx_vmul_w\0"
  "sx_vmulwev_d_w\0"
  "sx_vmulwev_d_wu\0"
  "sx_vmulwev_d_wu_w\0"
  "sx_vmulwev_h_b\0"
  "sx_vmulwev_h_bu\0"
  "sx_vmulwev_h_bu_b\0"
  "sx_vmulwev_q_d\0"
  "sx_vmulwev_q_du\0"
  "sx_vmulwev_q_du_d\0"
  "sx_vmulwev_w_h\0"
  "sx_vmulwev_w_hu\0"
  "sx_vmulwev_w_hu_h\0"
  "sx_vmulwod_d_w\0"
  "sx_vmulwod_d_wu\0"
  "sx_vmulwod_d_wu_w\0"
  "sx_vmulwod_h_b\0"
  "sx_vmulwod_h_bu\0"
  "sx_vmulwod_h_bu_b\0"
  "sx_vmulwod_q_d\0"
  "sx_vmulwod_q_du\0"
  "sx_vmulwod_q_du_d\0"
  "sx_vmulwod_w_h\0"
  "sx_vmulwod_w_hu\0"
  "sx_vmulwod_w_hu_h\0"
  "sx_vneg_b\0"
  "sx_vneg_d\0"
  "sx_vneg_h\0"
  "sx_vneg_w\0"
  "sx_vnor_v\0"
  "sx_vnori_b\0"
  "sx_vor_v\0"
  "sx_vori_b\0"
  "sx_vorn_v\0"
  "sx_vpackev_b\0"
  "sx_vpackev_d\0"
  "sx_vpackev_h\0"
  "sx_vpackev_w\0"
  "sx_vpackod_b\0"
  "sx_vpackod_d\0"
  "sx_vpackod_h\0"
  "sx_vpackod_w\0"
  "sx_vpcnt_b\0"
  "sx_vpcnt_d\0"
  "sx_vpcnt_h\0"
  "sx_vpcnt_w\0"
  "sx_vpermi_w\0"
  "sx_vpickev_b\0"
  "sx_vpickev_d\0"
  "sx_vpickev_h\0"
  "sx_vpickev_w\0"
  "sx_vpickod_b\0"
  "sx_vpickod_d\0"
  "sx_vpickod_h\0"
  "sx_vpickod_w\0"
  "sx_vpickve2gr_b\0"
  "sx_vpickve2gr_bu\0"
  "sx_vpickve2gr_d\0"
  "sx_vpickve2gr_du\0"
  "sx_vpickve2gr_h\0"
  "sx_vpickve2gr_hu\0"
  "sx_vpickve2gr_w\0"
  "sx_vpickve2gr_wu\0"
  "sx_vreplgr2vr_b\0"
  "sx_vreplgr2vr_d\0"
  "sx_vreplgr2vr_h\0"
  "sx_vreplgr2vr_w\0"
  "sx_vrepli_b\0"
  "sx_vrepli_d\0"
  "sx_vrepli_h\0"
  "sx_vrepli_w\0"
  "sx_vreplve_b\0"
  "sx_vreplve_d\0"
  "sx_vreplve_h\0"
  "sx_vreplve_w\0"
  "sx_vreplvei_b\0"
  "sx_vreplvei_d\0"
  "sx_vreplvei_h\0"
  "sx_vreplvei_w\0"
  "sx_vrotr_b\0"
  "sx_vrotr_d\0"
  "sx_vrotr_h\0"
  "sx_vrotr_w\0"
  "sx_vrotri_b\0"
  "sx_vrotri_d\0"
  "sx_vrotri_h\0"
  "sx_vrotri_w\0"
  "sx_vsadd_b\0"
  "sx_vsadd_bu\0"
  "sx_vsadd_d\0"
  "sx_vsadd_du\0"
  "sx_vsadd_h\0"
  "sx_vsadd_hu\0"
  "sx_vsadd_w\0"
  "sx_vsadd_wu\0"
  "sx_vsat_b\0"
  "sx_vsat_bu\0"
  "sx_vsat_d\0"
  "sx_vsat_du\0"
  "sx_vsat_h\0"
  "sx_vsat_hu\0"
  "sx_vsat_w\0"
  "sx_vsat_wu\0"
  "sx_vseq_b\0"
  "sx_vseq_d\0"
  "sx_vseq_h\0"
  "sx_vseq_w\0"
  "sx_vseqi_b\0"
  "sx_vseqi_d\0"
  "sx_vseqi_h\0"
  "sx_vseqi_w\0"
  "sx_vshuf4i_b\0"
  "sx_vshuf4i_d\0"
  "sx_vshuf4i_h\0"
  "sx_vshuf4i_w\0"
  "sx_vshuf_b\0"
  "sx_vshuf_d\0"
  "sx_vshuf_h\0"
  "sx_vshuf_w\0"
  "sx_vsigncov_b\0"
  "sx_vsigncov_d\0"
  "sx_vsigncov_h\0"
  "sx_vsigncov_w\0"
  "sx_vsle_b\0"
  "sx_vsle_bu\0"
  "sx_vsle_d\0"
  "sx_vsle_du\0"
  "sx_vsle_h\0"
  "sx_vsle_hu\0"
  "sx_vsle_w\0"
  "sx_vsle_wu\0"
  "sx_vslei_b\0"
  "sx_vslei_bu\0"
  "sx_vslei_d\0"
  "sx_vslei_du\0"
  "sx_vslei_h\0"
  "sx_vslei_hu\0"
  "sx_vslei_w\0"
  "sx_vslei_wu\0"
  "sx_vsll_b\0"
  "sx_vsll_d\0"
  "sx_vsll_h\0"
  "sx_vsll_w\0"
  "sx_vslli_b\0"
  "sx_vslli_d\0"
  "sx_vslli_h\0"
  "sx_vslli_w\0"
  "sx_vsllwil_d_w\0"
  "sx_vsllwil_du_wu\0"
  "sx_vsllwil_h_b\0"
  "sx_vsllwil_hu_bu\0"
  "sx_vsllwil_w_h\0"
  "sx_vsllwil_wu_hu\0"
  "sx_vslt_b\0"
  "sx_vslt_bu\0"
  "sx_vslt_d\0"
  "sx_vslt_du\0"
  "sx_vslt_h\0"
  "sx_vslt_hu\0"
  "sx_vslt_w\0"
  "sx_vslt_wu\0"
  "sx_vslti_b\0"
  "sx_vslti_bu\0"
  "sx_vslti_d\0"
  "sx_vslti_du\0"
  "sx_vslti_h\0"
  "sx_vslti_hu\0"
  "sx_vslti_w\0"
  "sx_vslti_wu\0"
  "sx_vsra_b\0"
  "sx_vsra_d\0"
  "sx_vsra_h\0"
  "sx_vsra_w\0"
  "sx_vsrai_b\0"
  "sx_vsrai_d\0"
  "sx_vsrai_h\0"
  "sx_vsrai_w\0"
  "sx_vsran_b_h\0"
  "sx_vsran_h_w\0"
  "sx_vsran_w_d\0"
  "sx_vsrani_b_h\0"
  "sx_vsrani_d_q\0"
  "sx_vsrani_h_w\0"
  "sx_vsrani_w_d\0"
  "sx_vsrar_b\0"
  "sx_vsrar_d\0"
  "sx_vsrar_h\0"
  "sx_vsrar_w\0"
  "sx_vsrari_b\0"
  "sx_vsrari_d\0"
  "sx_vsrari_h\0"
  "sx_vsrari_w\0"
  "sx_vsrarn_b_h\0"
  "sx_vsrarn_h_w\0"
  "sx_vsrarn_w_d\0"
  "sx_vsrarni_b_h\0"
  "sx_vsrarni_d_q\0"
  "sx_vsrarni_h_w\0"
  "sx_vsrarni_w_d\0"
  "sx_vsrl_b\0"
  "sx_vsrl_d\0"
  "sx_vsrl_h\0"
  "sx_vsrl_w\0"
  "sx_vsrli_b\0"
  "sx_vsrli_d\0"
  "sx_vsrli_h\0"
  "sx_vsrli_w\0"
  "sx_vsrln_b_h\0"
  "sx_vsrln_h_w\0"
  "sx_vsrln_w_d\0"
  "sx_vsrlni_b_h\0"
  "sx_vsrlni_d_q\0"
  "sx_vsrlni_h_w\0"
  "sx_vsrlni_w_d\0"
  "sx_vsrlr_b\0"
  "sx_vsrlr_d\0"
  "sx_vsrlr_h\0"
  "sx_vsrlr_w\0"
  "sx_vsrlri_b\0"
  "sx_vsrlri_d\0"
  "sx_vsrlri_h\0"
  "sx_vsrlri_w\0"
  "sx_vsrlrn_b_h\0"
  "sx_vsrlrn_h_w\0"
  "sx_vsrlrn_w_d\0"
  "sx_vsrlrni_b_h\0"
  "sx_vsrlrni_d_q\0"
  "sx_vsrlrni_h_w\0"
  "sx_vsrlrni_w_d\0"
  "sx_vssran_b_h\0"
  "sx_vssran_bu_h\0"
  "sx_vssran_h_w\0"
  "sx_vssran_hu_w\0"
  "sx_vssran_w_d\0"
  "sx_vssran_wu_d\0"
  "sx_vssrani_b_h\0"
  "sx_vssrani_bu_h\0"
  "sx_vssrani_d_q\0"
  "sx_vssrani_du_q\0"
  "sx_vssrani_h_w\0"
  "sx_vssrani_hu_w\0"
  "sx_vssrani_w_d\0"
  "sx_vssrani_wu_d\0"
  "sx_vssrarn_b_h\0"
  "sx_vssrarn_bu_h\0"
  "sx_vssrarn_h_w\0"
  "sx_vssrarn_hu_w\0"
  "sx_vssrarn_w_d\0"
  "sx_vssrarn_wu_d\0"
  "sx_vssrarni_b_h\0"
  "sx_vssrarni_bu_h\0"
  "sx_vssrarni_d_q\0"
  "sx_vssrarni_du_q\0"
  "sx_vssrarni_h_w\0"
  "sx_vssrarni_hu_w\0"
  "sx_vssrarni_w_d\0"
  "sx_vssrarni_wu_d\0"
  "sx_vssrln_b_h\0"
  "sx_vssrln_bu_h\0"
  "sx_vssrln_h_w\0"
  "sx_vssrln_hu_w\0"
  "sx_vssrln_w_d\0"
  "sx_vssrln_wu_d\0"
  "sx_vssrlni_b_h\0"
  "sx_vssrlni_bu_h\0"
  "sx_vssrlni_d_q\0"
  "sx_vssrlni_du_q\0"
  "sx_vssrlni_h_w\0"
  "sx_vssrlni_hu_w\0"
  "sx_vssrlni_w_d\0"
  "sx_vssrlni_wu_d\0"
  "sx_vssrlrn_b_h\0"
  "sx_vssrlrn_bu_h\0"
  "sx_vssrlrn_h_w\0"
  "sx_vssrlrn_hu_w\0"
  "sx_vssrlrn_w_d\0"
  "sx_vssrlrn_wu_d\0"
  "sx_vssrlrni_b_h\0"
  "sx_vssrlrni_bu_h\0"
  "sx_vssrlrni_d_q\0"
  "sx_vssrlrni_du_q\0"
  "sx_vssrlrni_h_w\0"
  "sx_vssrlrni_hu_w\0"
  "sx_vssrlrni_w_d\0"
  "sx_vssrlrni_wu_d\0"
  "sx_vssub_b\0"
  "sx_vssub_bu\0"
  "sx_vssub_d\0"
  "sx_vssub_du\0"
  "sx_vssub_h\0"
  "sx_vssub_hu\0"
  "sx_vssub_w\0"
  "sx_vssub_wu\0"
  "sx_vst\0"
  "sx_vstelm_b\0"
  "sx_vstelm_d\0"
  "sx_vstelm_h\0"
  "sx_vstelm_w\0"
  "sx_vstx\0"
  "sx_vsub_b\0"
  "sx_vsub_d\0"
  "sx_vsub_h\0"
  "sx_vsub_q\0"
  "sx_vsub_w\0"
  "sx_vsubi_bu\0"
  "sx_vsubi_du\0"
  "sx_vsubi_hu\0"
  "sx_vsubi_wu\0"
  "sx_vsubwev_d_w\0"
  "sx_vsubwev_d_wu\0"
  "sx_vsubwev_h_b\0"
  "sx_vsubwev_h_bu\0"
  "sx_vsubwev_q_d\0"
  "sx_vsubwev_q_du\0"
  "sx_vsubwev_w_h\0"
  "sx_vsubwev_w_hu\0"
  "sx_vsubwod_d_w\0"
  "sx_vsubwod_d_wu\0"
  "sx_vsubwod_h_b\0"
  "sx_vsubwod_h_bu\0"
  "sx_vsubwod_q_d\0"
  "sx_vsubwod_q_du\0"
  "sx_vsubwod_w_h\0"
  "sx_vsubwod_w_hu\0"
  "sx_vxor_v\0"
  "sx_vxori_b\0"
  "ips_absq_s_ph\0"
  "ips_absq_s_qb\0"
  "ips_absq_s_w\0"
  "ips_addq_ph\0"
  "ips_addq_s_ph\0"
  "ips_addq_s_w\0"
  "ips_addqh_ph\0"
  "ips_addqh_r_ph\0"
  "ips_addqh_r_w\0"
  "ips_addqh_w\0"
  "ips_addsc\0"
  "ips_addu_ph\0"
  "ips_addu_qb\0"
  "ips_addu_s_ph\0"
  "ips_addu_s_qb\0"
  "ips_adduh_qb\0"
  "ips_adduh_r_qb\0"
  "ips_addwc\0"
  "ips_append\0"
  "ips_balign\0"
  "ips_bitrev\0"
  "ips_bposge32\0"
  "ips_cmp_eq_ph\0"
  "ips_cmp_le_ph\0"
  "ips_cmp_lt_ph\0"
  "ips_cmpgdu_eq_qb\0"
  "ips_cmpgdu_le_qb\0"
  "ips_cmpgdu_lt_qb\0"
  "ips_cmpgu_eq_qb\0"
  "ips_cmpgu_le_qb\0"
  "ips_cmpgu_lt_qb\0"
  "ips_cmpu_eq_qb\0"
  "ips_cmpu_le_qb\0"
  "ips_cmpu_lt_qb\0"
  "ips_dlsa\0"
  "ips_dpa_w_ph\0"
  "ips_dpaq_s_w_ph\0"
  "ips_dpaq_sa_l_w\0"
  "ips_dpaqx_s_w_ph\0"
  "ips_dpaqx_sa_w_ph\0"
  "ips_dpau_h_qbl\0"
  "ips_dpau_h_qbr\0"
  "ips_dpax_w_ph\0"
  "ips_dps_w_ph\0"
  "ips_dpsq_s_w_ph\0"
  "ips_dpsq_sa_l_w\0"
  "ips_dpsqx_s_w_ph\0"
  "ips_dpsqx_sa_w_ph\0"
  "ips_dpsu_h_qbl\0"
  "ips_dpsu_h_qbr\0"
  "ips_dpsx_w_ph\0"
  "ips_extp\0"
  "ips_extpdp\0"
  "ips_extr_r_w\0"
  "ips_extr_rs_w\0"
  "ips_extr_s_h\0"
  "ips_extr_w\0"
  "ips_insv\0"
  "ips_lbux\0"
  "ips_lhx\0"
  "ips_lsa\0"
  "ips_lwx\0"
  "ips_madd\0"
  "ips_maddu\0"
  "ips_maq_s_w_phl\0"
  "ips_maq_s_w_phr\0"
  "ips_maq_sa_w_phl\0"
  "ips_maq_sa_w_phr\0"
  "ips_modsub\0"
  "ips_msub\0"
  "ips_msubu\0"
  "ips_mthlip\0"
  "ips_mul_ph\0"
  "ips_mul_s_ph\0"
  "ips_muleq_s_w_phl\0"
  "ips_muleq_s_w_phr\0"
  "ips_muleu_s_ph_qbl\0"
  "ips_muleu_s_ph_qbr\0"
  "ips_mulq_rs_ph\0"
  "ips_mulq_rs_w\0"
  "ips_mulq_s_ph\0"
  "ips_mulq_s_w\0"
  "ips_mulsa_w_ph\0"
  "ips_mulsaq_s_w_ph\0"
  "ips_mult\0"
  "ips_multu\0"
  "ips_packrl_ph\0"
  "ips_pick_ph\0"
  "ips_pick_qb\0"
  "ips_preceq_w_phl\0"
  "ips_preceq_w_phr\0"
  "ips_precequ_ph_qbl\0"
  "ips_precequ_ph_qbla\0"
  "ips_precequ_ph_qbr\0"
  "ips_precequ_ph_qbra\0"
  "ips_preceu_ph_qbl\0"
  "ips_preceu_ph_qbla\0"
  "ips_preceu_ph_qbr\0"
  "ips_preceu_ph_qbra\0"
  "ips_precr_qb_ph\0"
  "ips_precr_sra_ph_w\0"
  "ips_precr_sra_r_ph_w\0"
  "ips_precrq_ph_w\0"
  "ips_precrq_qb_ph\0"
  "ips_precrq_rs_ph_w\0"
  "ips_precrqu_s_qb_ph\0"
  "ips_prepend\0"
  "ips_raddu_w_qb\0"
  "ips_rddsp\0"
  "ips_repl_ph\0"
  "ips_repl_qb\0"
  "ips_shilo\0"
  "ips_shll_ph\0"
  "ips_shll_qb\0"
  "ips_shll_s_ph\0"
  "ips_shll_s_w\0"
  "ips_shra_ph\0"
  "ips_shra_qb\0"
  "ips_shra_r_ph\0"
  "ips_shra_r_qb\0"
  "ips_shra_r_w\0"
  "ips_shrl_ph\0"
  "ips_shrl_qb\0"
  "ips_subq_ph\0"
  "ips_subq_s_ph\0"
  "ips_subq_s_w\0"
  "ips_subqh_ph\0"
  "ips_subqh_r_ph\0"
  "ips_subqh_r_w\0"
  "ips_subqh_w\0"
  "ips_subu_ph\0"
  "ips_subu_qb\0"
  "ips_subu_s_ph\0"
  "ips_subu_s_qb\0"
  "ips_subuh_qb\0"
  "ips_subuh_r_qb\0"
  "ips_wrdsp\0"
  "sa_add_a_b\0"
  "sa_add_a_d\0"
  "sa_add_a_h\0"
  "sa_add_a_w\0"
  "sa_adds_a_b\0"
  "sa_adds_a_d\0"
  "sa_adds_a_h\0"
  "sa_adds_a_w\0"
  "sa_adds_s_b\0"
  "sa_adds_s_d\0"
  "sa_adds_s_h\0"
  "sa_adds_s_w\0"
  "sa_adds_u_b\0"
  "sa_adds_u_d\0"
  "sa_adds_u_h\0"
  "sa_adds_u_w\0"
  "sa_addv_b\0"
  "sa_addv_d\0"
  "sa_addv_h\0"
  "sa_addv_w\0"
  "sa_addvi_b\0"
  "sa_addvi_d\0"
  "sa_addvi_h\0"
  "sa_addvi_w\0"
  "sa_and_v\0"
  "sa_andi_b\0"
  "sa_asub_s_b\0"
  "sa_asub_s_d\0"
  "sa_asub_s_h\0"
  "sa_asub_s_w\0"
  "sa_asub_u_b\0"
  "sa_asub_u_d\0"
  "sa_asub_u_h\0"
  "sa_asub_u_w\0"
  "sa_ave_s_b\0"
  "sa_ave_s_d\0"
  "sa_ave_s_h\0"
  "sa_ave_s_w\0"
  "sa_ave_u_b\0"
  "sa_ave_u_d\0"
  "sa_ave_u_h\0"
  "sa_ave_u_w\0"
  "sa_aver_s_b\0"
  "sa_aver_s_d\0"
  "sa_aver_s_h\0"
  "sa_aver_s_w\0"
  "sa_aver_u_b\0"
  "sa_aver_u_d\0"
  "sa_aver_u_h\0"
  "sa_aver_u_w\0"
  "sa_bclr_b\0"
  "sa_bclr_d\0"
  "sa_bclr_h\0"
  "sa_bclr_w\0"
  "sa_bclri_b\0"
  "sa_bclri_d\0"
  "sa_bclri_h\0"
  "sa_bclri_w\0"
  "sa_binsl_b\0"
  "sa_binsl_d\0"
  "sa_binsl_h\0"
  "sa_binsl_w\0"
  "sa_binsli_b\0"
  "sa_binsli_d\0"
  "sa_binsli_h\0"
  "sa_binsli_w\0"
  "sa_binsr_b\0"
  "sa_binsr_d\0"
  "sa_binsr_h\0"
  "sa_binsr_w\0"
  "sa_binsri_b\0"
  "sa_binsri_d\0"
  "sa_binsri_h\0"
  "sa_binsri_w\0"
  "sa_bmnz_v\0"
  "sa_bmnzi_b\0"
  "sa_bmz_v\0"
  "sa_bmzi_b\0"
  "sa_bneg_b\0"
  "sa_bneg_d\0"
  "sa_bneg_h\0"
  "sa_bneg_w\0"
  "sa_bnegi_b\0"
  "sa_bnegi_d\0"
  "sa_bnegi_h\0"
  "sa_bnegi_w\0"
  "sa_bnz_b\0"
  "sa_bnz_d\0"
  "sa_bnz_h\0"
  "sa_bnz_v\0"
  "sa_bnz_w\0"
  "sa_bsel_v\0"
  "sa_bseli_b\0"
  "sa_bset_b\0"
  "sa_bset_d\0"
  "sa_bset_h\0"
  "sa_bset_w\0"
  "sa_bseti_b\0"
  "sa_bseti_d\0"
  "sa_bseti_h\0"
  "sa_bseti_w\0"
  "sa_bz_b\0"
  "sa_bz_d\0"
  "sa_bz_h\0"
  "sa_bz_v\0"
  "sa_bz_w\0"
  "sa_ceq_b\0"
  "sa_ceq_d\0"
  "sa_ceq_h\0"
  "sa_ceq_w\0"
  "sa_ceqi_b\0"
  "sa_ceqi_d\0"
  "sa_ceqi_h\0"
  "sa_ceqi_w\0"
  "sa_cfcmsa\0"
  "sa_cle_s_b\0"
  "sa_cle_s_d\0"
  "sa_cle_s_h\0"
  "sa_cle_s_w\0"
  "sa_cle_u_b\0"
  "sa_cle_u_d\0"
  "sa_cle_u_h\0"
  "sa_cle_u_w\0"
  "sa_clei_s_b\0"
  "sa_clei_s_d\0"
  "sa_clei_s_h\0"
  "sa_clei_s_w\0"
  "sa_clei_u_b\0"
  "sa_clei_u_d\0"
  "sa_clei_u_h\0"
  "sa_clei_u_w\0"
  "sa_clt_s_b\0"
  "sa_clt_s_d\0"
  "sa_clt_s_h\0"
  "sa_clt_s_w\0"
  "sa_clt_u_b\0"
  "sa_clt_u_d\0"
  "sa_clt_u_h\0"
  "sa_clt_u_w\0"
  "sa_clti_s_b\0"
  "sa_clti_s_d\0"
  "sa_clti_s_h\0"
  "sa_clti_s_w\0"
  "sa_clti_u_b\0"
  "sa_clti_u_d\0"
  "sa_clti_u_h\0"
  "sa_clti_u_w\0"
  "sa_copy_s_b\0"
  "sa_copy_s_d\0"
  "sa_copy_s_h\0"
  "sa_copy_s_w\0"
  "sa_copy_u_b\0"
  "sa_copy_u_d\0"
  "sa_copy_u_h\0"
  "sa_copy_u_w\0"
  "sa_ctcmsa\0"
  "sa_div_s_b\0"
  "sa_div_s_d\0"
  "sa_div_s_h\0"
  "sa_div_s_w\0"
  "sa_div_u_b\0"
  "sa_div_u_d\0"
  "sa_div_u_h\0"
  "sa_div_u_w\0"
  "sa_dotp_s_d\0"
  "sa_dotp_s_h\0"
  "sa_dotp_s_w\0"
  "sa_dotp_u_d\0"
  "sa_dotp_u_h\0"
  "sa_dotp_u_w\0"
  "sa_dpadd_s_d\0"
  "sa_dpadd_s_h\0"
  "sa_dpadd_s_w\0"
  "sa_dpadd_u_d\0"
  "sa_dpadd_u_h\0"
  "sa_dpadd_u_w\0"
  "sa_dpsub_s_d\0"
  "sa_dpsub_s_h\0"
  "sa_dpsub_s_w\0"
  "sa_dpsub_u_d\0"
  "sa_dpsub_u_h\0"
  "sa_dpsub_u_w\0"
  "sa_fadd_d\0"
  "sa_fadd_w\0"
  "sa_fcaf_d\0"
  "sa_fcaf_w\0"
  "sa_fceq_d\0"
  "sa_fceq_w\0"
  "sa_fclass_d\0"
  "sa_fclass_w\0"
  "sa_fcle_d\0"
  "sa_fcle_w\0"
  "sa_fclt_d\0"
  "sa_fclt_w\0"
  "sa_fcne_d\0"
  "sa_fcne_w\0"
  "sa_fcor_d\0"
  "sa_fcor_w\0"
  "sa_fcueq_d\0"
  "sa_fcueq_w\0"
  "sa_fcule_d\0"
  "sa_fcule_w\0"
  "sa_fcult_d\0"
  "sa_fcult_w\0"
  "sa_fcun_d\0"
  "sa_fcun_w\0"
  "sa_fcune_d\0"
  "sa_fcune_w\0"
  "sa_fdiv_d\0"
  "sa_fdiv_w\0"
  "sa_fexdo_h\0"
  "sa_fexdo_w\0"
  "sa_fexp2_d\0"
  "sa_fexp2_w\0"
  "sa_fexupl_d\0"
  "sa_fexupl_w\0"
  "sa_fexupr_d\0"
  "sa_fexupr_w\0"
  "sa_ffint_s_d\0"
  "sa_ffint_s_w\0"
  "sa_ffint_u_d\0"
  "sa_ffint_u_w\0"
  "sa_ffql_d\0"
  "sa_ffql_w\0"
  "sa_ffqr_d\0"
  "sa_ffqr_w\0"
  "sa_fill_b\0"
  "sa_fill_d\0"
  "sa_fill_h\0"
  "sa_fill_w\0"
  "sa_flog2_d\0"
  "sa_flog2_w\0"
  "sa_fmadd_d\0"
  "sa_fmadd_w\0"
  "sa_fmax_a_d\0"
  "sa_fmax_a_w\0"
  "sa_fmax_d\0"
  "sa_fmax_w\0"
  "sa_fmin_a_d\0"
  "sa_fmin_a_w\0"
  "sa_fmin_d\0"
  "sa_fmin_w\0"
  "sa_fmsub_d\0"
  "sa_fmsub_w\0"
  "sa_fmul_d\0"
  "sa_fmul_w\0"
  "sa_frcp_d\0"
  "sa_frcp_w\0"
  "sa_frint_d\0"
  "sa_frint_w\0"
  "sa_frsqrt_d\0"
  "sa_frsqrt_w\0"
  "sa_fsaf_d\0"
  "sa_fsaf_w\0"
  "sa_fseq_d\0"
  "sa_fseq_w\0"
  "sa_fsle_d\0"
  "sa_fsle_w\0"
  "sa_fslt_d\0"
  "sa_fslt_w\0"
  "sa_fsne_d\0"
  "sa_fsne_w\0"
  "sa_fsor_d\0"
  "sa_fsor_w\0"
  "sa_fsqrt_d\0"
  "sa_fsqrt_w\0"
  "sa_fsub_d\0"
  "sa_fsub_w\0"
  "sa_fsueq_d\0"
  "sa_fsueq_w\0"
  "sa_fsule_d\0"
  "sa_fsule_w\0"
  "sa_fsult_d\0"
  "sa_fsult_w\0"
  "sa_fsun_d\0"
  "sa_fsun_w\0"
  "sa_fsune_d\0"
  "sa_fsune_w\0"
  "sa_ftint_s_d\0"
  "sa_ftint_s_w\0"
  "sa_ftint_u_d\0"
  "sa_ftint_u_w\0"
  "sa_ftq_h\0"
  "sa_ftq_w\0"
  "sa_ftrunc_s_d\0"
  "sa_ftrunc_s_w\0"
  "sa_ftrunc_u_d\0"
  "sa_ftrunc_u_w\0"
  "sa_hadd_s_d\0"
  "sa_hadd_s_h\0"
  "sa_hadd_s_w\0"
  "sa_hadd_u_d\0"
  "sa_hadd_u_h\0"
  "sa_hadd_u_w\0"
  "sa_hsub_s_d\0"
  "sa_hsub_s_h\0"
  "sa_hsub_s_w\0"
  "sa_hsub_u_d\0"
  "sa_hsub_u_h\0"
  "sa_hsub_u_w\0"
  "sa_ilvev_b\0"
  "sa_ilvev_d\0"
  "sa_ilvev_h\0"
  "sa_ilvev_w\0"
  "sa_ilvl_b\0"
  "sa_ilvl_d\0"
  "sa_ilvl_h\0"
  "sa_ilvl_w\0"
  "sa_ilvod_b\0"
  "sa_ilvod_d\0"
  "sa_ilvod_h\0"
  "sa_ilvod_w\0"
  "sa_ilvr_b\0"
  "sa_ilvr_d\0"
  "sa_ilvr_h\0"
  "sa_ilvr_w\0"
  "sa_insert_b\0"
  "sa_insert_d\0"
  "sa_insert_h\0"
  "sa_insert_w\0"
  "sa_insve_b\0"
  "sa_insve_d\0"
  "sa_insve_h\0"
  "sa_insve_w\0"
  "sa_ld_b\0"
  "sa_ld_d\0"
  "sa_ld_h\0"
  "sa_ld_w\0"
  "sa_ldi_b\0"
  "sa_ldi_d\0"
  "sa_ldi_h\0"
  "sa_ldi_w\0"
  "sa_ldr_d\0"
  "sa_ldr_w\0"
  "sa_madd_q_h\0"
  "sa_madd_q_w\0"
  "sa_maddr_q_h\0"
  "sa_maddr_q_w\0"
  "sa_maddv_b\0"
  "sa_maddv_d\0"
  "sa_maddv_h\0"
  "sa_maddv_w\0"
  "sa_max_a_b\0"
  "sa_max_a_d\0"
  "sa_max_a_h\0"
  "sa_max_a_w\0"
  "sa_max_s_b\0"
  "sa_max_s_d\0"
  "sa_max_s_h\0"
  "sa_max_s_w\0"
  "sa_max_u_b\0"
  "sa_max_u_d\0"
  "sa_max_u_h\0"
  "sa_max_u_w\0"
  "sa_maxi_s_b\0"
  "sa_maxi_s_d\0"
  "sa_maxi_s_h\0"
  "sa_maxi_s_w\0"
  "sa_maxi_u_b\0"
  "sa_maxi_u_d\0"
  "sa_maxi_u_h\0"
  "sa_maxi_u_w\0"
  "sa_min_a_b\0"
  "sa_min_a_d\0"
  "sa_min_a_h\0"
  "sa_min_a_w\0"
  "sa_min_s_b\0"
  "sa_min_s_d\0"
  "sa_min_s_h\0"
  "sa_min_s_w\0"
  "sa_min_u_b\0"
  "sa_min_u_d\0"
  "sa_min_u_h\0"
  "sa_min_u_w\0"
  "sa_mini_s_b\0"
  "sa_mini_s_d\0"
  "sa_mini_s_h\0"
  "sa_mini_s_w\0"
  "sa_mini_u_b\0"
  "sa_mini_u_d\0"
  "sa_mini_u_h\0"
  "sa_mini_u_w\0"
  "sa_mod_s_b\0"
  "sa_mod_s_d\0"
  "sa_mod_s_h\0"
  "sa_mod_s_w\0"
  "sa_mod_u_b\0"
  "sa_mod_u_d\0"
  "sa_mod_u_h\0"
  "sa_mod_u_w\0"
  "sa_move_v\0"
  "sa_msub_q_h\0"
  "sa_msub_q_w\0"
  "sa_msubr_q_h\0"
  "sa_msubr_q_w\0"
  "sa_msubv_b\0"
  "sa_msubv_d\0"
  "sa_msubv_h\0"
  "sa_msubv_w\0"
  "sa_mul_q_h\0"
  "sa_mul_q_w\0"
  "sa_mulr_q_h\0"
  "sa_mulr_q_w\0"
  "sa_mulv_b\0"
  "sa_mulv_d\0"
  "sa_mulv_h\0"
  "sa_mulv_w\0"
  "sa_nloc_b\0"
  "sa_nloc_d\0"
  "sa_nloc_h\0"
  "sa_nloc_w\0"
  "sa_nlzc_b\0"
  "sa_nlzc_d\0"
  "sa_nlzc_h\0"
  "sa_nlzc_w\0"
  "sa_nor_v\0"
  "sa_nori_b\0"
  "sa_or_v\0"
  "sa_ori_b\0"
  "sa_pckev_b\0"
  "sa_pckev_d\0"
  "sa_pckev_h\0"
  "sa_pckev_w\0"
  "sa_pckod_b\0"
  "sa_pckod_d\0"
  "sa_pckod_h\0"
  "sa_pckod_w\0"
  "sa_pcnt_b\0"
  "sa_pcnt_d\0"
  "sa_pcnt_h\0"
  "sa_pcnt_w\0"
  "sa_sat_s_b\0"
  "sa_sat_s_d\0"
  "sa_sat_s_h\0"
  "sa_sat_s_w\0"
  "sa_sat_u_b\0"
  "sa_sat_u_d\0"
  "sa_sat_u_h\0"
  "sa_sat_u_w\0"
  "sa_shf_b\0"
  "sa_shf_h\0"
  "sa_shf_w\0"
  "sa_sld_b\0"
  "sa_sld_d\0"
  "sa_sld_h\0"
  "sa_sld_w\0"
  "sa_sldi_b\0"
  "sa_sldi_d\0"
  "sa_sldi_h\0"
  "sa_sldi_w\0"
  "sa_sll_b\0"
  "sa_sll_d\0"
  "sa_sll_h\0"
  "sa_sll_w\0"
  "sa_slli_b\0"
  "sa_slli_d\0"
  "sa_slli_h\0"
  "sa_slli_w\0"
  "sa_splat_b\0"
  "sa_splat_d\0"
  "sa_splat_h\0"
  "sa_splat_w\0"
  "sa_splati_b\0"
  "sa_splati_d\0"
  "sa_splati_h\0"
  "sa_splati_w\0"
  "sa_sra_b\0"
  "sa_sra_d\0"
  "sa_sra_h\0"
  "sa_sra_w\0"
  "sa_srai_b\0"
  "sa_srai_d\0"
  "sa_srai_h\0"
  "sa_srai_w\0"
  "sa_srar_b\0"
  "sa_srar_d\0"
  "sa_srar_h\0"
  "sa_srar_w\0"
  "sa_srari_b\0"
  "sa_srari_d\0"
  "sa_srari_h\0"
  "sa_srari_w\0"
  "sa_srl_b\0"
  "sa_srl_d\0"
  "sa_srl_h\0"
  "sa_srl_w\0"
  "sa_srli_b\0"
  "sa_srli_d\0"
  "sa_srli_h\0"
  "sa_srli_w\0"
  "sa_srlr_b\0"
  "sa_srlr_d\0"
  "sa_srlr_h\0"
  "sa_srlr_w\0"
  "sa_srlri_b\0"
  "sa_srlri_d\0"
  "sa_srlri_h\0"
  "sa_srlri_w\0"
  "sa_st_b\0"
  "sa_st_d\0"
  "sa_st_h\0"
  "sa_st_w\0"
  "sa_str_d\0"
  "sa_str_w\0"
  "sa_subs_s_b\0"
  "sa_subs_s_d\0"
  "sa_subs_s_h\0"
  "sa_subs_s_w\0"
  "sa_subs_u_b\0"
  "sa_subs_u_d\0"
  "sa_subs_u_h\0"
  "sa_subs_u_w\0"
  "sa_subsus_u_b\0"
  "sa_subsus_u_d\0"
  "sa_subsus_u_h\0"
  "sa_subsus_u_w\0"
  "sa_subsuu_s_b\0"
  "sa_subsuu_s_d\0"
  "sa_subsuu_s_h\0"
  "sa_subsuu_s_w\0"
  "sa_subv_b\0"
  "sa_subv_d\0"
  "sa_subv_h\0"
  "sa_subv_w\0"
  "sa_subvi_b\0"
  "sa_subvi_d\0"
  "sa_subvi_h\0"
  "sa_subvi_w\0"
  "sa_vshf_b\0"
  "sa_vshf_d\0"
  "sa_vshf_h\0"
  "sa_vshf_w\0"
  "sa_xor_v\0"
  "sa_xori_b\0"
  "activemask\0"
  "add_rm_d\0"
  "add_rm_f\0"
  "add_rm_ftz_f\0"
  "add_rm_ftz_sat_f\0"
  "add_rm_sat_f\0"
  "add_rn_d\0"
  "add_rn_f\0"
  "add_rn_ftz_f\0"
  "add_rn_ftz_sat_f\0"
  "add_rn_sat_f\0"
  "add_rp_d\0"
  "add_rp_f\0"
  "add_rp_ftz_f\0"
  "add_rp_ftz_sat_f\0"
  "add_rp_sat_f\0"
  "add_rz_d\0"
  "add_rz_f\0"
  "add_rz_ftz_f\0"
  "add_rz_ftz_sat_f\0"
  "add_rz_sat_f\0"
  "bar_warp_sync\0"
  "bf16x2_to_ue8m0x2_rp\0"
  "bf16x2_to_ue8m0x2_rp_satfinite\0"
  "bf16x2_to_ue8m0x2_rz\0"
  "bf16x2_to_ue8m0x2_rz_satfinite\0"
  "bf2h_rn\0"
  "bf2h_rn_ftz\0"
  "ceil_d\0"
  "ceil_f\0"
  "ceil_ftz_f\0"
  "cos_approx_f\0"
  "cos_approx_ftz_f\0"
  "cp_async_commit_group\0"
  "cp_async_mbarrier_arrive\0"
  "cp_async_mbarrier_arrive_noinc\0"
  "cp_async_mbarrier_arrive_noinc_shared\0"
  "cp_async_mbarrier_arrive_shared\0"
  "cp_async_wait_all\0"
  "cp_async_wait_group\0"
  "d2f_rm\0"
  "d2f_rm_ftz\0"
  "d2f_rn\0"
  "d2f_rn_ftz\0"
  "d2f_rp\0"
  "d2f_rp_ftz\0"
  "d2f_rz\0"
  "d2f_rz_ftz\0"
  "d2i_hi\0"
  "d2i_lo\0"
  "d2i_rm\0"
  "d2i_rn\0"
  "d2i_rp\0"
  "d2i_rz\0"
  "d2ll_rm\0"
  "d2ll_rn\0"
  "d2ll_rp\0"
  "d2ll_rz\0"
  "d2ui_rm\0"
  "d2ui_rn\0"
  "d2ui_rp\0"
  "d2ui_rz\0"
  "d2ull_rm\0"
  "d2ull_rn\0"
  "d2ull_rp\0"
  "d2ull_rz\0"
  "div_approx_f\0"
  "div_approx_ftz_f\0"
  "div_full\0"
  "div_full_ftz\0"
  "div_rm_d\0"
  "div_rm_f\0"
  "div_rm_ftz_f\0"
  "div_rn_d\0"
  "div_rn_f\0"
  "div_rn_ftz_f\0"
  "div_rp_d\0"
  "div_rp_f\0"
  "div_rp_ftz_f\0"
  "div_rz_d\0"
  "div_rz_f\0"
  "div_rz_ftz_f\0"
  "e2m1x2_to_f16x2_rn\0"
  "e2m1x2_to_f16x2_rn_relu\0"
  "e2m3x2_to_f16x2_rn\0"
  "e2m3x2_to_f16x2_rn_relu\0"
  "e3m2x2_to_f16x2_rn\0"
  "e3m2x2_to_f16x2_rn_relu\0"
  "e4m3x2_to_f16x2_rn\0"
  "e4m3x2_to_f16x2_rn_relu\0"
  "e5m2x2_to_f16x2_rn\0"
  "e5m2x2_to_f16x2_rn_relu\0"
  "exit\0"
  "f16x2_to_e4m3x2_rn\0"
  "f16x2_to_e4m3x2_rn_relu\0"
  "f16x2_to_e5m2x2_rn\0"
  "f16x2_to_e5m2x2_rn_relu\0"
  "f2bf16_rn\0"
  "f2bf16_rn_relu\0"
  "f2bf16_rn_relu_satfinite\0"
  "f2bf16_rn_satfinite\0"
  "f2bf16_rz\0"
  "f2bf16_rz_relu\0"
  "f2bf16_rz_relu_satfinite\0"
  "f2bf16_rz_satfinite\0"
  "f2f16_rn\0"
  "f2f16_rn_relu\0"
  "f2f16_rn_relu_satfinite\0"
  "f2f16_rn_satfinite\0"
  "f2f16_rz\0"
  "f2f16_rz_relu\0"
  "f2f16_rz_relu_satfinite\0"
  "f2f16_rz_satfinite\0"
  "f2h_rn\0"
  "f2h_rn_ftz\0"
  "f2i_rm\0"
  "f2i_rm_ftz\0"
  "f2i_rn\0"
  "f2i_rn_ftz\0"
  "f2i_rp\0"
  "f2i_rp_ftz\0"
  "f2i_rz\0"
  "f2i_rz_ftz\0"
  "f2ll_rm\0"
  "f2ll_rm_ftz\0"
  "f2ll_rn\0"
  "f2ll_rn_ftz\0"
  "f2ll_rp\0"
  "f2ll_rp_ftz\0"
  "f2ll_rz\0"
  "f2ll_rz_ftz\0"
  "f2tf32_rn\0"
  "f2tf32_rn_relu\0"
  "f2tf32_rn_relu_satfinite\0"
  "f2tf32_rn_satfinite\0"
  "f2tf32_rna\0"
  "f2tf32_rna_satfinite\0"
  "f2tf32_rz\0"
  "f2tf32_rz_relu\0"
  "f2tf32_rz_relu_satfinite\0"
  "f2tf32_rz_satfinite\0"
  "f2ui_rm\0"
  "f2ui_rm_ftz\0"
  "f2ui_rn\0"
  "f2ui_rn_ftz\0"
  "f2ui_rp\0"
  "f2ui_rp_ftz\0"
  "f2ui_rz\0"
  "f2ui_rz_ftz\0"
  "f2ull_rm\0"
  "f2ull_rm_ftz\0"
  "f2ull_rn\0"
  "f2ull_rn_ftz\0"
  "f2ull_rp\0"
  "f2ull_rp_ftz\0"
  "f2ull_rz\0"
  "f2ull_rz_ftz\0"
  "f32x4_to_e2m1x4_rs_relu_satfinite\0"
  "f32x4_to_e2m1x4_rs_satfinite\0"
  "f32x4_to_e2m3x4_rs_relu_satfinite\0"
  "f32x4_to_e2m3x4_rs_satfinite\0"
  "f32x4_to_e3m2x4_rs_relu_satfinite\0"
  "f32x4_to_e3m2x4_rs_satfinite\0"
  "f32x4_to_e4m3x4_rs_relu_satfinite\0"
  "f32x4_to_e4m3x4_rs_satfinite\0"
  "f32x4_to_e5m2x4_rs_relu_satfinite\0"
  "f32x4_to_e5m2x4_rs_satfinite\0"
  "ff2bf16x2_rn\0"
  "ff2bf16x2_rn_relu\0"
  "ff2bf16x2_rn_relu_satfinite\0"
  "ff2bf16x2_rn_satfinite\0"
  "ff2bf16x2_rs\0"
  "ff2bf16x2_rs_relu\0"
  "ff2bf16x2_rs_relu_satfinite\0"
  "ff2bf16x2_rs_satfinite\0"
  "ff2bf16x2_rz\0"
  "ff2bf16x2_rz_relu\0"
  "ff2bf16x2_rz_relu_satfinite\0"
  "ff2bf16x2_rz_satfinite\0"
  "ff2f16x2_rn\0"
  "ff2f16x2_rn_relu\0"
  "ff2f16x2_rn_relu_satfinite\0"
  "ff2f16x2_rn_satfinite\0"
  "ff2f16x2_rs\0"
  "ff2f16x2_rs_relu\0"
  "ff2f16x2_rs_relu_satfinite\0"
  "ff2f16x2_rs_satfinite\0"
  "ff2f16x2_rz\0"
  "ff2f16x2_rz_relu\0"
  "ff2f16x2_rz_relu_satfinite\0"
  "ff2f16x2_rz_satfinite\0"
  "ff_to_e2m1x2_rn_relu_satfinite\0"
  "ff_to_e2m1x2_rn_satfinite\0"
  "ff_to_e2m3x2_rn_relu_satfinite\0"
  "ff_to_e2m3x2_rn_satfinite\0"
  "ff_to_e3m2x2_rn_relu_satfinite\0"
  "ff_to_e3m2x2_rn_satfinite\0"
  "ff_to_e4m3x2_rn\0"
  "ff_to_e4m3x2_rn_relu\0"
  "ff_to_e5m2x2_rn\0"
  "ff_to_e5m2x2_rn_relu\0"
  "ff_to_ue8m0x2_rp\0"
  "ff_to_ue8m0x2_rp_satfinite\0"
  "ff_to_ue8m0x2_rz\0"
  "ff_to_ue8m0x2_rz_satfinite\0"
  "floor_d\0"
  "floor_f\0"
  "floor_ftz_f\0"
  "fma_rm_d\0"
  "fma_rm_f\0"
  "fma_rm_ftz_f\0"
  "fma_rm_ftz_sat_f\0"
  "fma_rm_sat_f\0"
  "fma_rn_bf16\0"
  "fma_rn_bf16x2\0"
  "fma_rn_d\0"
  "fma_rn_f\0"
  "fma_rn_ftz_bf16\0"
  "fma_rn_ftz_bf16x2\0"
  "fma_rn_ftz_f\0"
  "fma_rn_ftz_relu_bf16\0"
  "fma_rn_ftz_relu_bf16x2\0"
  "fma_rn_ftz_sat_bf16\0"
  "fma_rn_ftz_sat_bf16x2\0"
  "fma_rn_ftz_sat_f\0"
  "fma_rn_relu_bf16\0"
  "fma_rn_relu_bf16x2\0"
  "fma_rn_sat_bf16\0"
  "fma_rn_sat_bf16x2\0"
  "fma_rn_sat_f\0"
  "fma_rp_d\0"
  "fma_rp_f\0"
  "fma_rp_ftz_f\0"
  "fma_rp_ftz_sat_f\0"
  "fma_rp_sat_f\0"
  "fma_rz_d\0"
  "fma_rz_f\0"
  "fma_rz_ftz_f\0"
  "fma_rz_ftz_sat_f\0"
  "fma_rz_sat_f\0"
  "fmax_bf16\0"
  "fmax_bf16x2\0"
  "fmax_d\0"
  "fmax_f\0"
  "fmax_ftz_bf16\0"
  "fmax_ftz_bf16x2\0"
  "fmax_ftz_f\0"
  "fmax_ftz_nan_bf16\0"
  "fmax_ftz_nan_bf16x2\0"
  "fmax_ftz_nan_f\0"
  "fmax_ftz_nan_xorsign_abs_bf16\0"
  "fmax_ftz_nan_xorsign_abs_bf16x2\0"
  "fmax_ftz_nan_xorsign_abs_f\0"
  "fmax_ftz_xorsign_abs_bf16\0"
  "fmax_ftz_xorsign_abs_bf16x2\0"
  "fmax_ftz_xorsign_abs_f\0"
  "fmax_nan_bf16\0"
  "fmax_nan_bf16x2\0"
  "fmax_nan_f\0"
  "fmax_nan_xorsign_abs_bf16\0"
  "fmax_nan_xorsign_abs_bf16x2\0"
  "fmax_nan_xorsign_abs_f\0"
  "fmax_xorsign_abs_bf16\0"
  "fmax_xorsign_abs_bf16x2\0"
  "fmax_xorsign_abs_f\0"
  "fmin_bf16\0"
  "fmin_bf16x2\0"
  "fmin_d\0"
  "fmin_f\0"
  "fmin_ftz_bf16\0"
  "fmin_ftz_bf16x2\0"
  "fmin_ftz_f\0"
  "fmin_ftz_nan_bf16\0"
  "fmin_ftz_nan_bf16x2\0"
  "fmin_ftz_nan_f\0"
  "fmin_ftz_nan_xorsign_abs_bf16\0"
  "fmin_ftz_nan_xorsign_abs_bf16x2\0"
  "fmin_ftz_nan_xorsign_abs_f\0"
  "fmin_ftz_xorsign_abs_bf16\0"
  "fmin_ftz_xorsign_abs_bf16x2\0"
  "fmin_ftz_xorsign_abs_f\0"
  "fmin_nan_bf16\0"
  "fmin_nan_bf16x2\0"
  "fmin_nan_f\0"
  "fmin_nan_xorsign_abs_bf16\0"
  "fmin_nan_xorsign_abs_bf16x2\0"
  "fmin_nan_xorsign_abs_f\0"
  "fmin_xorsign_abs_bf16\0"
  "fmin_xorsign_abs_bf16x2\0"
  "fmin_xorsign_abs_f\0"
  "fns\0"
  "i2d_rm\0"
  "i2d_rn\0"
  "i2d_rp\0"
  "i2d_rz\0"
  "i2f_rm\0"
  "i2f_rn\0"
  "i2f_rp\0"
  "i2f_rz\0"
  "isspacep_const\0"
  "isspacep_global\0"
  "isspacep_local\0"
  "isspacep_shared\0"
  "isspacep_shared_cluster\0"
  "istypep_sampler\0"
  "istypep_surface\0"
  "istypep_texture\0"
  "lg2_approx_d\0"
  "lg2_approx_f\0"
  "lg2_approx_ftz_f\0"
  "ll2d_rm\0"
  "ll2d_rn\0"
  "ll2d_rp\0"
  "ll2d_rz\0"
  "ll2f_rm\0"
  "ll2f_rn\0"
  "ll2f_rp\0"
  "ll2f_rz\0"
  "lohi_i2d\0"
  "match_any_sync_i32\0"
  "match_any_sync_i64\0"
  "mbarrier_arrive\0"
  "mbarrier_arrive_drop\0"
  "mbarrier_arrive_drop_noComplete\0"
  "mbarrier_arrive_drop_noComplete_shared\0"
  "mbarrier_arrive_drop_shared\0"
  "mbarrier_arrive_noComplete\0"
  "mbarrier_arrive_noComplete_shared\0"
  "mbarrier_arrive_shared\0"
  "mbarrier_init\0"
  "mbarrier_init_shared\0"
  "mbarrier_inval\0"
  "mbarrier_inval_shared\0"
  "mbarrier_pending_count\0"
  "mbarrier_test_wait\0"
  "mbarrier_test_wait_shared\0"
  "membar_cta\0"
  "membar_gl\0"
  "membar_sys\0"
  "mul24_i\0"
  "mul24_ui\0"
  "mul_rm_d\0"
  "mul_rm_f\0"
  "mul_rm_ftz_f\0"
  "mul_rn_d\0"
  "mul_rn_f\0"
  "mul_rn_ftz_f\0"
  "mul_rp_d\0"
  "mul_rp_f\0"
  "mul_rp_ftz_f\0"
  "mul_rz_d\0"
  "mul_rz_f\0"
  "mul_rz_ftz_f\0"
  "mulhi_i\0"
  "mulhi_ll\0"
  "mulhi_s\0"
  "mulhi_ui\0"
  "mulhi_ull\0"
  "mulhi_us\0"
  "nanosleep\0"
  "neg_bf16\0"
  "neg_bf16x2\0"
  "pm_event_mask\0"
  "prmt\0"
  "rcp_approx_ftz_d\0"
  "rcp_approx_ftz_f\0"
  "rcp_rm_d\0"
  "rcp_rm_f\0"
  "rcp_rm_ftz_f\0"
  "rcp_rn_d\0"
  "rcp_rn_f\0"
  "rcp_rn_ftz_f\0"
  "rcp_rp_d\0"
  "rcp_rp_f\0"
  "rcp_rp_ftz_f\0"
  "rcp_rz_d\0"
  "rcp_rz_f\0"
  "rcp_rz_ftz_f\0"
  "read_ptx_sreg_clock\0"
  "read_ptx_sreg_clock64\0"
  "read_ptx_sreg_ctaid_w\0"
  "read_ptx_sreg_ctaid_x\0"
  "read_ptx_sreg_ctaid_y\0"
  "read_ptx_sreg_ctaid_z\0"
  "read_ptx_sreg_envreg0\0"
  "read_ptx_sreg_envreg1\0"
  "read_ptx_sreg_envreg10\0"
  "read_ptx_sreg_envreg11\0"
  "read_ptx_sreg_envreg12\0"
  "read_ptx_sreg_envreg13\0"
  "read_ptx_sreg_envreg14\0"
  "read_ptx_sreg_envreg15\0"
  "read_ptx_sreg_envreg16\0"
  "read_ptx_sreg_envreg17\0"
  "read_ptx_sreg_envreg18\0"
  "read_ptx_sreg_envreg19\0"
  "read_ptx_sreg_envreg2\0"
  "read_ptx_sreg_envreg20\0"
  "read_ptx_sreg_envreg21\0"
  "read_ptx_sreg_envreg22\0"
  "read_ptx_sreg_envreg23\0"
  "read_ptx_sreg_envreg24\0"
  "read_ptx_sreg_envreg25\0"
  "read_ptx_sreg_envreg26\0"
  "read_ptx_sreg_envreg27\0"
  "read_ptx_sreg_envreg28\0"
  "read_ptx_sreg_envreg29\0"
  "read_ptx_sreg_envreg3\0"
  "read_ptx_sreg_envreg30\0"
  "read_ptx_sreg_envreg31\0"
  "read_ptx_sreg_envreg4\0"
  "read_ptx_sreg_envreg5\0"
  "read_ptx_sreg_envreg6\0"
  "read_ptx_sreg_envreg7\0"
  "read_ptx_sreg_envreg8\0"
  "read_ptx_sreg_envreg9\0"
  "read_ptx_sreg_globaltimer\0"
  "read_ptx_sreg_globaltimer_lo\0"
  "read_ptx_sreg_gridid\0"
  "read_ptx_sreg_laneid\0"
  "read_ptx_sreg_lanemask_eq\0"
  "read_ptx_sreg_lanemask_ge\0"
  "read_ptx_sreg_lanemask_gt\0"
  "read_ptx_sreg_lanemask_le\0"
  "read_ptx_sreg_lanemask_lt\0"
  "read_ptx_sreg_nctaid_w\0"
  "read_ptx_sreg_nctaid_x\0"
  "read_ptx_sreg_nctaid_y\0"
  "read_ptx_sreg_nctaid_z\0"
  "read_ptx_sreg_nsmid\0"
  "read_ptx_sreg_ntid_w\0"
  "read_ptx_sreg_ntid_x\0"
  "read_ptx_sreg_ntid_y\0"
  "read_ptx_sreg_ntid_z\0"
  "read_ptx_sreg_nwarpid\0"
  "read_ptx_sreg_pm0\0"
  "read_ptx_sreg_pm1\0"
  "read_ptx_sreg_pm2\0"
  "read_ptx_sreg_pm3\0"
  "read_ptx_sreg_smid\0"
  "read_ptx_sreg_tid_w\0"
  "read_ptx_sreg_tid_x\0"
  "read_ptx_sreg_tid_y\0"
  "read_ptx_sreg_tid_z\0"
  "read_ptx_sreg_warpid\0"
  "read_ptx_sreg_warpsize\0"
  "redux_sync_add\0"
  "redux_sync_and\0"
  "redux_sync_fmax\0"
  "redux_sync_fmax_NaN\0"
  "redux_sync_fmax_abs\0"
  "redux_sync_fmax_abs_NaN\0"
  "redux_sync_fmin\0"
  "redux_sync_fmin_NaN\0"
  "redux_sync_fmin_abs\0"
  "redux_sync_fmin_abs_NaN\0"
  "redux_sync_max\0"
  "redux_sync_min\0"
  "redux_sync_or\0"
  "redux_sync_umax\0"
  "redux_sync_umin\0"
  "redux_sync_xor\0"
  "reflect\0"
  "round_d\0"
  "round_f\0"
  "round_ftz_f\0"
  "rsqrt_approx_d\0"
  "rsqrt_approx_f\0"
  "rsqrt_approx_ftz_d\0"
  "rsqrt_approx_ftz_f\0"
  "sad_i\0"
  "sad_ll\0"
  "sad_s\0"
  "sad_ui\0"
  "sad_ull\0"
  "sad_us\0"
  "saturate_d\0"
  "saturate_f\0"
  "saturate_ftz_f\0"
  "shfl_bfly_f32\0"
  "shfl_bfly_i32\0"
  "shfl_down_f32\0"
  "shfl_down_i32\0"
  "shfl_idx_f32\0"
  "shfl_idx_i32\0"
  "shfl_sync_bfly_f32\0"
  "shfl_sync_bfly_i32\0"
  "shfl_sync_down_f32\0"
  "shfl_sync_down_i32\0"
  "shfl_sync_idx_f32\0"
  "shfl_sync_idx_i32\0"
  "shfl_sync_up_f32\0"
  "shfl_sync_up_i32\0"
  "shfl_up_f32\0"
  "shfl_up_i32\0"
  "sin_approx_f\0"
  "sin_approx_ftz_f\0"
  "sqrt_approx_f\0"
  "sqrt_approx_ftz_f\0"
  "sqrt_f\0"
  "sqrt_rm_d\0"
  "sqrt_rm_f\0"
  "sqrt_rm_ftz_f\0"
  "sqrt_rn_d\0"
  "sqrt_rn_f\0"
  "sqrt_rn_ftz_f\0"
  "sqrt_rp_d\0"
  "sqrt_rp_f\0"
  "sqrt_rp_ftz_f\0"
  "sqrt_rz_d\0"
  "sqrt_rz_f\0"
  "sqrt_rz_ftz_f\0"
  "suq_array_size\0"
  "suq_channel_data_type\0"
  "suq_channel_order\0"
  "suq_depth\0"
  "suq_height\0"
  "suq_width\0"
  "sust_b_1d_array_i16_clamp\0"
  "sust_b_1d_array_i16_trap\0"
  "sust_b_1d_array_i16_zero\0"
  "sust_b_1d_array_i32_clamp\0"
  "sust_b_1d_array_i32_trap\0"
  "sust_b_1d_array_i32_zero\0"
  "sust_b_1d_array_i64_clamp\0"
  "sust_b_1d_array_i64_trap\0"
  "sust_b_1d_array_i64_zero\0"
  "sust_b_1d_array_i8_clamp\0"
  "sust_b_1d_array_i8_trap\0"
  "sust_b_1d_array_i8_zero\0"
  "sust_b_1d_array_v2i16_clamp\0"
  "sust_b_1d_array_v2i16_trap\0"
  "sust_b_1d_array_v2i16_zero\0"
  "sust_b_1d_array_v2i32_clamp\0"
  "sust_b_1d_array_v2i32_trap\0"
  "sust_b_1d_array_v2i32_zero\0"
  "sust_b_1d_array_v2i64_clamp\0"
  "sust_b_1d_array_v2i64_trap\0"
  "sust_b_1d_array_v2i64_zero\0"
  "sust_b_1d_array_v2i8_clamp\0"
  "sust_b_1d_array_v2i8_trap\0"
  "sust_b_1d_array_v2i8_zero\0"
  "sust_b_1d_array_v4i16_clamp\0"
  "sust_b_1d_array_v4i16_trap\0"
  "sust_b_1d_array_v4i16_zero\0"
  "sust_b_1d_array_v4i32_clamp\0"
  "sust_b_1d_array_v4i32_trap\0"
  "sust_b_1d_array_v4i32_zero\0"
  "sust_b_1d_array_v4i8_clamp\0"
  "sust_b_1d_array_v4i8_trap\0"
  "sust_b_1d_array_v4i8_zero\0"
  "sust_b_1d_i16_clamp\0"
  "sust_b_1d_i16_trap\0"
  "sust_b_1d_i16_zero\0"
  "sust_b_1d_i32_clamp\0"
  "sust_b_1d_i32_trap\0"
  "sust_b_1d_i32_zero\0"
  "sust_b_1d_i64_clamp\0"
  "sust_b_1d_i64_trap\0"
  "sust_b_1d_i64_zero\0"
  "sust_b_1d_i8_clamp\0"
  "sust_b_1d_i8_trap\0"
  "sust_b_1d_i8_zero\0"
  "sust_b_1d_v2i16_clamp\0"
  "sust_b_1d_v2i16_trap\0"
  "sust_b_1d_v2i16_zero\0"
  "sust_b_1d_v2i32_clamp\0"
  "sust_b_1d_v2i32_trap\0"
  "sust_b_1d_v2i32_zero\0"
  "sust_b_1d_v2i64_clamp\0"
  "sust_b_1d_v2i64_trap\0"
  "sust_b_1d_v2i64_zero\0"
  "sust_b_1d_v2i8_clamp\0"
  "sust_b_1d_v2i8_trap\0"
  "sust_b_1d_v2i8_zero\0"
  "sust_b_1d_v4i16_clamp\0"
  "sust_b_1d_v4i16_trap\0"
  "sust_b_1d_v4i16_zero\0"
  "sust_b_1d_v4i32_clamp\0"
  "sust_b_1d_v4i32_trap\0"
  "sust_b_1d_v4i32_zero\0"
  "sust_b_1d_v4i8_clamp\0"
  "sust_b_1d_v4i8_trap\0"
  "sust_b_1d_v4i8_zero\0"
  "sust_b_2d_array_i16_clamp\0"
  "sust_b_2d_array_i16_trap\0"
  "sust_b_2d_array_i16_zero\0"
  "sust_b_2d_array_i32_clamp\0"
  "sust_b_2d_array_i32_trap\0"
  "sust_b_2d_array_i32_zero\0"
  "sust_b_2d_array_i64_clamp\0"
  "sust_b_2d_array_i64_trap\0"
  "sust_b_2d_array_i64_zero\0"
  "sust_b_2d_array_i8_clamp\0"
  "sust_b_2d_array_i8_trap\0"
  "sust_b_2d_array_i8_zero\0"
  "sust_b_2d_array_v2i16_clamp\0"
  "sust_b_2d_array_v2i16_trap\0"
  "sust_b_2d_array_v2i16_zero\0"
  "sust_b_2d_array_v2i32_clamp\0"
  "sust_b_2d_array_v2i32_trap\0"
  "sust_b_2d_array_v2i32_zero\0"
  "sust_b_2d_array_v2i64_clamp\0"
  "sust_b_2d_array_v2i64_trap\0"
  "sust_b_2d_array_v2i64_zero\0"
  "sust_b_2d_array_v2i8_clamp\0"
  "sust_b_2d_array_v2i8_trap\0"
  "sust_b_2d_array_v2i8_zero\0"
  "sust_b_2d_array_v4i16_clamp\0"
  "sust_b_2d_array_v4i16_trap\0"
  "sust_b_2d_array_v4i16_zero\0"
  "sust_b_2d_array_v4i32_clamp\0"
  "sust_b_2d_array_v4i32_trap\0"
  "sust_b_2d_array_v4i32_zero\0"
  "sust_b_2d_array_v4i8_clamp\0"
  "sust_b_2d_array_v4i8_trap\0"
  "sust_b_2d_array_v4i8_zero\0"
  "sust_b_2d_i16_clamp\0"
  "sust_b_2d_i16_trap\0"
  "sust_b_2d_i16_zero\0"
  "sust_b_2d_i32_clamp\0"
  "sust_b_2d_i32_trap\0"
  "sust_b_2d_i32_zero\0"
  "sust_b_2d_i64_clamp\0"
  "sust_b_2d_i64_trap\0"
  "sust_b_2d_i64_zero\0"
  "sust_b_2d_i8_clamp\0"
  "sust_b_2d_i8_trap\0"
  "sust_b_2d_i8_zero\0"
  "sust_b_2d_v2i16_clamp\0"
  "sust_b_2d_v2i16_trap\0"
  "sust_b_2d_v2i16_zero\0"
  "sust_b_2d_v2i32_clamp\0"
  "sust_b_2d_v2i32_trap\0"
  "sust_b_2d_v2i32_zero\0"
  "sust_b_2d_v2i64_clamp\0"
  "sust_b_2d_v2i64_trap\0"
  "sust_b_2d_v2i64_zero\0"
  "sust_b_2d_v2i8_clamp\0"
  "sust_b_2d_v2i8_trap\0"
  "sust_b_2d_v2i8_zero\0"
  "sust_b_2d_v4i16_clamp\0"
  "sust_b_2d_v4i16_trap\0"
  "sust_b_2d_v4i16_zero\0"
  "sust_b_2d_v4i32_clamp\0"
  "sust_b_2d_v4i32_trap\0"
  "sust_b_2d_v4i32_zero\0"
  "sust_b_2d_v4i8_clamp\0"
  "sust_b_2d_v4i8_trap\0"
  "sust_b_2d_v4i8_zero\0"
  "sust_b_3d_i16_clamp\0"
  "sust_b_3d_i16_trap\0"
  "sust_b_3d_i16_zero\0"
  "sust_b_3d_i32_clamp\0"
  "sust_b_3d_i32_trap\0"
  "sust_b_3d_i32_zero\0"
  "sust_b_3d_i64_clamp\0"
  "sust_b_3d_i64_trap\0"
  "sust_b_3d_i64_zero\0"
  "sust_b_3d_i8_clamp\0"
  "sust_b_3d_i8_trap\0"
  "sust_b_3d_i8_zero\0"
  "sust_b_3d_v2i16_clamp\0"
  "sust_b_3d_v2i16_trap\0"
  "sust_b_3d_v2i16_zero\0"
  "sust_b_3d_v2i32_clamp\0"
  "sust_b_3d_v2i32_trap\0"
  "sust_b_3d_v2i32_zero\0"
  "sust_b_3d_v2i64_clamp\0"
  "sust_b_3d_v2i64_trap\0"
  "sust_b_3d_v2i64_zero\0"
  "sust_b_3d_v2i8_clamp\0"
  "sust_b_3d_v2i8_trap\0"
  "sust_b_3d_v2i8_zero\0"
  "sust_b_3d_v4i16_clamp\0"
  "sust_b_3d_v4i16_trap\0"
  "sust_b_3d_v4i16_zero\0"
  "sust_b_3d_v4i32_clamp\0"
  "sust_b_3d_v4i32_trap\0"
  "sust_b_3d_v4i32_zero\0"
  "sust_b_3d_v4i8_clamp\0"
  "sust_b_3d_v4i8_trap\0"
  "sust_b_3d_v4i8_zero\0"
  "sust_p_1d_array_i16_trap\0"
  "sust_p_1d_array_i32_trap\0"
  "sust_p_1d_array_i8_trap\0"
  "sust_p_1d_array_v2i16_trap\0"
  "sust_p_1d_array_v2i32_trap\0"
  "sust_p_1d_array_v2i8_trap\0"
  "sust_p_1d_array_v4i16_trap\0"
  "sust_p_1d_array_v4i32_trap\0"
  "sust_p_1d_array_v4i8_trap\0"
  "sust_p_1d_i16_trap\0"
  "sust_p_1d_i32_trap\0"
  "sust_p_1d_i8_trap\0"
  "sust_p_1d_v2i16_trap\0"
  "sust_p_1d_v2i32_trap\0"
  "sust_p_1d_v2i8_trap\0"
  "sust_p_1d_v4i16_trap\0"
  "sust_p_1d_v4i32_trap\0"
  "sust_p_1d_v4i8_trap\0"
  "sust_p_2d_array_i16_trap\0"
  "sust_p_2d_array_i32_trap\0"
  "sust_p_2d_array_i8_trap\0"
  "sust_p_2d_array_v2i16_trap\0"
  "sust_p_2d_array_v2i32_trap\0"
  "sust_p_2d_array_v2i8_trap\0"
  "sust_p_2d_array_v4i16_trap\0"
  "sust_p_2d_array_v4i32_trap\0"
  "sust_p_2d_array_v4i8_trap\0"
  "sust_p_2d_i16_trap\0"
  "sust_p_2d_i32_trap\0"
  "sust_p_2d_i8_trap\0"
  "sust_p_2d_v2i16_trap\0"
  "sust_p_2d_v2i32_trap\0"
  "sust_p_2d_v2i8_trap\0"
  "sust_p_2d_v4i16_trap\0"
  "sust_p_2d_v4i32_trap\0"
  "sust_p_2d_v4i8_trap\0"
  "sust_p_3d_i16_trap\0"
  "sust_p_3d_i32_trap\0"
  "sust_p_3d_i8_trap\0"
  "sust_p_3d_v2i16_trap\0"
  "sust_p_3d_v2i32_trap\0"
  "sust_p_3d_v2i8_trap\0"
  "sust_p_3d_v4i16_trap\0"
  "sust_p_3d_v4i32_trap\0"
  "sust_p_3d_v4i8_trap\0"
  "trunc_d\0"
  "trunc_f\0"
  "trunc_ftz_f\0"
  "txq_array_size\0"
  "txq_channel_data_type\0"
  "txq_channel_order\0"
  "txq_depth\0"
  "txq_height\0"
  "txq_num_mipmap_levels\0"
  "txq_num_samples\0"
  "txq_width\0"
  "ue8m0x2_to_bf16x2\0"
  "ui2d_rm\0"
  "ui2d_rn\0"
  "ui2d_rp\0"
  "ui2d_rz\0"
  "ui2f_rm\0"
  "ui2f_rn\0"
  "ui2f_rp\0"
  "ui2f_rz\0"
  "ull2d_rm\0"
  "ull2d_rn\0"
  "ull2d_rp\0"
  "ull2d_rz\0"
  "ull2f_rm\0"
  "ull2f_rn\0"
  "ull2f_rp\0"
  "ull2f_rz\0"
  "vote_all\0"
  "vote_all_sync\0"
  "vote_any\0"
  "vote_any_sync\0"
  "vote_ballot\0"
  "vote_ballot_sync\0"
  "vote_uni\0"
  "vote_uni_sync\0"
  "addf128_round_to_odd\0"
  "addg6s\0"
  "altivec_crypto_vcipher\0"
  "altivec_crypto_vcipherlast\0"
  "altivec_crypto_vncipher\0"
  "altivec_crypto_vncipherlast\0"
  "altivec_crypto_vpermxor\0"
  "altivec_crypto_vpermxor_be\0"
  "altivec_crypto_vpmsumb\0"
  "altivec_crypto_vpmsumd\0"
  "altivec_crypto_vpmsumh\0"
  "altivec_crypto_vpmsumw\0"
  "altivec_crypto_vsbox\0"
  "altivec_crypto_vshasigmad\0"
  "altivec_crypto_vshasigmaw\0"
  "altivec_dss\0"
  "altivec_dssall\0"
  "altivec_dst\0"
  "altivec_dstst\0"
  "altivec_dststt\0"
  "altivec_dstt\0"
  "altivec_mfvscr\0"
  "altivec_mtvscr\0"
  "altivec_mtvsrbm\0"
  "altivec_mtvsrdm\0"
  "altivec_mtvsrhm\0"
  "altivec_mtvsrqm\0"
  "altivec_mtvsrwm\0"
  "altivec_vabsdub\0"
  "altivec_vabsduh\0"
  "altivec_vabsduw\0"
  "altivec_vaddcuq\0"
  "altivec_vaddcuw\0"
  "altivec_vaddecuq\0"
  "altivec_vaddeuqm\0"
  "altivec_vaddsbs\0"
  "altivec_vaddshs\0"
  "altivec_vaddsws\0"
  "altivec_vaddubs\0"
  "altivec_vadduhs\0"
  "altivec_vadduws\0"
  "altivec_vavgsb\0"
  "altivec_vavgsh\0"
  "altivec_vavgsw\0"
  "altivec_vavgub\0"
  "altivec_vavguh\0"
  "altivec_vavguw\0"
  "altivec_vbpermd\0"
  "altivec_vbpermq\0"
  "altivec_vcfsx\0"
  "altivec_vcfuged\0"
  "altivec_vcfux\0"
  "altivec_vclrlb\0"
  "altivec_vclrrb\0"
  "altivec_vclzdm\0"
  "altivec_vclzlsbb\0"
  "altivec_vcmpbfp\0"
  "altivec_vcmpbfp_p\0"
  "altivec_vcmpeqfp\0"
  "altivec_vcmpeqfp_p\0"
  "altivec_vcmpequb\0"
  "altivec_vcmpequb_p\0"
  "altivec_vcmpequd\0"
  "altivec_vcmpequd_p\0"
  "altivec_vcmpequh\0"
  "altivec_vcmpequh_p\0"
  "altivec_vcmpequq\0"
  "altivec_vcmpequq_p\0"
  "altivec_vcmpequw\0"
  "altivec_vcmpequw_p\0"
  "altivec_vcmpgefp\0"
  "altivec_vcmpgefp_p\0"
  "altivec_vcmpgtfp\0"
  "altivec_vcmpgtfp_p\0"
  "altivec_vcmpgtsb\0"
  "altivec_vcmpgtsb_p\0"
  "altivec_vcmpgtsd\0"
  "altivec_vcmpgtsd_p\0"
  "altivec_vcmpgtsh\0"
  "altivec_vcmpgtsh_p\0"
  "altivec_vcmpgtsq\0"
  "altivec_vcmpgtsq_p\0"
  "altivec_vcmpgtsw\0"
  "altivec_vcmpgtsw_p\0"
  "altivec_vcmpgtub\0"
  "altivec_vcmpgtub_p\0"
  "altivec_vcmpgtud\0"
  "altivec_vcmpgtud_p\0"
  "altivec_vcmpgtuh\0"
  "altivec_vcmpgtuh_p\0"
  "altivec_vcmpgtuq\0"
  "altivec_vcmpgtuq_p\0"
  "altivec_vcmpgtuw\0"
  "altivec_vcmpgtuw_p\0"
  "altivec_vcmpneb\0"
  "altivec_vcmpneb_p\0"
  "altivec_vcmpneh\0"
  "altivec_vcmpneh_p\0"
  "altivec_vcmpnew\0"
  "altivec_vcmpnew_p\0"
  "altivec_vcmpnezb\0"
  "altivec_vcmpnezb_p\0"
  "altivec_vcmpnezh\0"
  "altivec_vcmpnezh_p\0"
  "altivec_vcmpnezw\0"
  "altivec_vcmpnezw_p\0"
  "altivec_vcntmbb\0"
  "altivec_vcntmbd\0"
  "altivec_vcntmbh\0"
  "altivec_vcntmbw\0"
  "altivec_vctsxs\0"
  "altivec_vctuxs\0"
  "altivec_vctzdm\0"
  "altivec_vctzlsbb\0"
  "altivec_vdivesd\0"
  "altivec_vdivesq\0"
  "altivec_vdivesw\0"
  "altivec_vdiveud\0"
  "altivec_vdiveuq\0"
  "altivec_vdiveuw\0"
  "altivec_vexpandbm\0"
  "altivec_vexpanddm\0"
  "altivec_vexpandhm\0"
  "altivec_vexpandqm\0"
  "altivec_vexpandwm\0"
  "altivec_vexptefp\0"
  "altivec_vextddvlx\0"
  "altivec_vextddvrx\0"
  "altivec_vextdubvlx\0"
  "altivec_vextdubvrx\0"
  "altivec_vextduhvlx\0"
  "altivec_vextduhvrx\0"
  "altivec_vextduwvlx\0"
  "altivec_vextduwvrx\0"
  "altivec_vextractbm\0"
  "altivec_vextractdm\0"
  "altivec_vextracthm\0"
  "altivec_vextractqm\0"
  "altivec_vextractwm\0"
  "altivec_vextsb2d\0"
  "altivec_vextsb2w\0"
  "altivec_vextsd2q\0"
  "altivec_vextsh2d\0"
  "altivec_vextsh2w\0"
  "altivec_vextsw2d\0"
  "altivec_vgbbd\0"
  "altivec_vgnb\0"
  "altivec_vinsblx\0"
  "altivec_vinsbrx\0"
  "altivec_vinsbvlx\0"
  "altivec_vinsbvrx\0"
  "altivec_vinsdlx\0"
  "altivec_vinsdrx\0"
  "altivec_vinshlx\0"
  "altivec_vinshrx\0"
  "altivec_vinshvlx\0"
  "altivec_vinshvrx\0"
  "altivec_vinswlx\0"
  "altivec_vinswrx\0"
  "altivec_vinswvlx\0"
  "altivec_vinswvrx\0"
  "altivec_vlogefp\0"
  "altivec_vmaddfp\0"
  "altivec_vmaxfp\0"
  "altivec_vmaxsb\0"
  "altivec_vmaxsd\0"
  "altivec_vmaxsh\0"
  "altivec_vmaxsw\0"
  "altivec_vmaxub\0"
  "altivec_vmaxud\0"
  "altivec_vmaxuh\0"
  "altivec_vmaxuw\0"
  "altivec_vmhaddshs\0"
  "altivec_vmhraddshs\0"
  "altivec_vminfp\0"
  "altivec_vminsb\0"
  "altivec_vminsd\0"
  "altivec_vminsh\0"
  "altivec_vminsw\0"
  "altivec_vminub\0"
  "altivec_vminud\0"
  "altivec_vminuh\0"
  "altivec_vminuw\0"
  "altivec_vmladduhm\0"
  "altivec_vmsumcud\0"
  "altivec_vmsummbm\0"
  "altivec_vmsumshm\0"
  "altivec_vmsumshs\0"
  "altivec_vmsumubm\0"
  "altivec_vmsumudm\0"
  "altivec_vmsumuhm\0"
  "altivec_vmsumuhs\0"
  "altivec_vmulesb\0"
  "altivec_vmulesd\0"
  "altivec_vmulesh\0"
  "altivec_vmulesw\0"
  "altivec_vmuleub\0"
  "altivec_vmuleud\0"
  "altivec_vmuleuh\0"
  "altivec_vmuleuw\0"
  "altivec_vmulhsd\0"
  "altivec_vmulhsw\0"
  "altivec_vmulhud\0"
  "altivec_vmulhuw\0"
  "altivec_vmulosb\0"
  "altivec_vmulosd\0"
  "altivec_vmulosh\0"
  "altivec_vmulosw\0"
  "altivec_vmuloub\0"
  "altivec_vmuloud\0"
  "altivec_vmulouh\0"
  "altivec_vmulouw\0"
  "altivec_vnmsubfp\0"
  "altivec_vpdepd\0"
  "altivec_vperm_4si\0"
  "altivec_vpextd\0"
  "altivec_vpkpx\0"
  "altivec_vpksdss\0"
  "altivec_vpksdus\0"
  "altivec_vpkshss\0"
  "altivec_vpkshus\0"
  "altivec_vpkswss\0"
  "altivec_vpkswus\0"
  "altivec_vpkudus\0"
  "altivec_vpkuhus\0"
  "altivec_vpkuwus\0"
  "altivec_vprtybd\0"
  "altivec_vprtybq\0"
  "altivec_vprtybw\0"
  "altivec_vrefp\0"
  "altivec_vrfim\0"
  "altivec_vrfin\0"
  "altivec_vrfip\0"
  "altivec_vrfiz\0"
  "altivec_vrlb\0"
  "altivec_vrld\0"
  "altivec_vrldmi\0"
  "altivec_vrldnm\0"
  "altivec_vrlh\0"
  "altivec_vrlqmi\0"
  "altivec_vrlqnm\0"
  "altivec_vrlw\0"
  "altivec_vrlwmi\0"
  "altivec_vrlwnm\0"
  "altivec_vrsqrtefp\0"
  "altivec_vsel_4si\0"
  "altivec_vsl\0"
  "altivec_vslb\0"
  "altivec_vsldbi\0"
  "altivec_vslh\0"
  "altivec_vslo\0"
  "altivec_vslv\0"
  "altivec_vslw\0"
  "altivec_vsr\0"
  "altivec_vsrab\0"
  "altivec_vsrah\0"
  "altivec_vsraw\0"
  "altivec_vsrb\0"
  "altivec_vsrdbi\0"
  "altivec_vsrh\0"
  "altivec_vsro\0"
  "altivec_vsrv\0"
  "altivec_vsrw\0"
  "altivec_vstribl\0"
  "altivec_vstribl_p\0"
  "altivec_vstribr\0"
  "altivec_vstribr_p\0"
  "altivec_vstrihl\0"
  "altivec_vstrihl_p\0"
  "altivec_vstrihr\0"
  "altivec_vstrihr_p\0"
  "altivec_vsubcuq\0"
  "altivec_vsubcuw\0"
  "altivec_vsubecuq\0"
  "altivec_vsubeuqm\0"
  "altivec_vsubsbs\0"
  "altivec_vsubshs\0"
  "altivec_vsubsws\0"
  "altivec_vsububs\0"
  "altivec_vsubuhs\0"
  "altivec_vsubuws\0"
  "altivec_vsum2sws\0"
  "altivec_vsum4sbs\0"
  "altivec_vsum4shs\0"
  "altivec_vsum4ubs\0"
  "altivec_vsumsws\0"
  "altivec_vupkhpx\0"
  "altivec_vupkhsb\0"
  "altivec_vupkhsh\0"
  "altivec_vupkhsw\0"
  "altivec_vupklpx\0"
  "altivec_vupklsb\0"
  "altivec_vupklsh\0"
  "altivec_vupklsw\0"
  "amo_ldat\0"
  "amo_lwat\0"
  "bpermd\0"
  "cbcdtd\0"
  "cdtbcd\0"
  "cfuged\0"
  "cntlzdm\0"
  "cnttzdm\0"
  "darn\0"
  "darn_32\0"
  "darn_raw\0"
  "dcbf\0"
  "divde\0"
  "divdeu\0"
  "divf128_round_to_odd\0"
  "divwe\0"
  "divweu\0"
  "fmaf128_round_to_odd\0"
  "get_texasr\0"
  "get_texasru\0"
  "get_tfhar\0"
  "get_tfiar\0"
  "mulf128_round_to_odd\0"
  "pack_longdouble\0"
  "pdepd\0"
  "pextd\0"
  "ppc_addex\0"
  "ppc_addg6s\0"
  "ppc_bcdadd\0"
  "ppc_bcdadd_p\0"
  "ppc_bcdcopysign\0"
  "ppc_bcdsetsign\0"
  "ppc_bcdsub\0"
  "ppc_bcdsub_p\0"
  "ppc_cbcdtd\0"
  "ppc_cdtbcd\0"
  "ppc_cmpeqb\0"
  "ppc_cmprb\0"
  "ppc_compare_exp_eq\0"
  "ppc_compare_exp_gt\0"
  "ppc_compare_exp_lt\0"
  "ppc_compare_exp_uo\0"
  "ppc_dcbfl\0"
  "ppc_dcbflp\0"
  "ppc_dcbst\0"
  "ppc_dcbt\0"
  "ppc_dcbtst\0"
  "ppc_dcbtstt\0"
  "ppc_dcbtt\0"
  "ppc_dcbz\0"
  "ppc_eieio\0"
  "ppc_extract_exp\0"
  "ppc_extract_sig\0"
  "ppc_fcfid\0"
  "ppc_fcfud\0"
  "ppc_fctid\0"
  "ppc_fctidz\0"
  "ppc_fctiw\0"
  "ppc_fctiwz\0"
  "ppc_fctudz\0"
  "ppc_fctuwz\0"
  "ppc_fence\0"
  "ppc_fmsub\0"
  "ppc_fmsubs\0"
  "ppc_fnabs\0"
  "ppc_fnabss\0"
  "ppc_fnmadd\0"
  "ppc_fnmadds\0"
  "ppc_fre\0"
  "ppc_fres\0"
  "ppc_frsqrte\0"
  "ppc_frsqrtes\0"
  "ppc_fsel\0"
  "ppc_fsels\0"
  "ppc_icbt\0"
  "ppc_insert_exp\0"
  "ppc_iospace_eieio\0"
  "ppc_iospace_lwsync\0"
  "ppc_iospace_sync\0"
  "ppc_isync\0"
  "ppc_load4r\0"
  "ppc_load8r\0"
  "ppc_lwsync\0"
  "ppc_maddhd\0"
  "ppc_maddhdu\0"
  "ppc_maddld\0"
  "ppc_mffsl\0"
  "ppc_mfmsr\0"
  "ppc_mftbu\0"
  "ppc_mtfsb0\0"
  "ppc_mtfsb1\0"
  "ppc_mtfsfi\0"
  "ppc_mtmsr\0"
  "ppc_mulhd\0"
  "ppc_mulhdu\0"
  "ppc_mulhw\0"
  "ppc_mulhwu\0"
  "ppc_national2packed\0"
  "ppc_packed2national\0"
  "ppc_packed2zoned\0"
  "ppc_rlwimi\0"
  "ppc_rlwnm\0"
  "ppc_setb\0"
  "ppc_stbcx\0"
  "ppc_stdcx\0"
  "ppc_stfiw\0"
  "ppc_store2r\0"
  "ppc_store4r\0"
  "ppc_store8r\0"
  "ppc_stwcx\0"
  "ppc_sync\0"
  "ppc_tdw\0"
  "ppc_trap\0"
  "ppc_trapd\0"
  "ppc_tw\0"
  "ppc_zoned2packed\0"
  "readflm\0"
  "set_texasr\0"
  "set_texasru\0"
  "set_tfhar\0"
  "set_tfiar\0"
  "setflm\0"
  "setrnd\0"
  "sqrtf128_round_to_odd\0"
  "subf128_round_to_odd\0"
  "tabort\0"
  "tabortdc\0"
  "tabortdci\0"
  "tabortwc\0"
  "tabortwci\0"
  "tbegin\0"
  "tcheck\0"
  "tend\0"
  "tendall\0"
  "trechkpt\0"
  "treclaim\0"
  "tresume\0"
  "truncf128_round_to_odd\0"
  "tsr\0"
  "tsuspend\0"
  "ttest\0"
  "unpack_longdouble\0"
  "vsx_scalar_extract_expq\0"
  "vsx_scalar_insert_exp_qp\0"
  "vsx_xsmaxdp\0"
  "vsx_xsmindp\0"
  "vsx_xvcmpeqdp\0"
  "vsx_xvcmpeqdp_p\0"
  "vsx_xvcmpeqsp\0"
  "vsx_xvcmpeqsp_p\0"
  "vsx_xvcmpgedp\0"
  "vsx_xvcmpgedp_p\0"
  "vsx_xvcmpgesp\0"
  "vsx_xvcmpgesp_p\0"
  "vsx_xvcmpgtdp\0"
  "vsx_xvcmpgtdp_p\0"
  "vsx_xvcmpgtsp\0"
  "vsx_xvcmpgtsp_p\0"
  "vsx_xvcvbf16spn\0"
  "vsx_xvcvdpsp\0"
  "vsx_xvcvdpsxws\0"
  "vsx_xvcvdpuxws\0"
  "vsx_xvcvhpsp\0"
  "vsx_xvcvspbf16\0"
  "vsx_xvcvspdp\0"
  "vsx_xvcvsphp\0"
  "vsx_xvcvspsxds\0"
  "vsx_xvcvspuxds\0"
  "vsx_xvcvsxdsp\0"
  "vsx_xvcvsxwdp\0"
  "vsx_xvcvuxdsp\0"
  "vsx_xvcvuxwdp\0"
  "vsx_xvdivdp\0"
  "vsx_xvdivsp\0"
  "vsx_xviexpdp\0"
  "vsx_xviexpsp\0"
  "vsx_xvmaxdp\0"
  "vsx_xvmaxsp\0"
  "vsx_xvmindp\0"
  "vsx_xvminsp\0"
  "vsx_xvredp\0"
  "vsx_xvresp\0"
  "vsx_xvrlw\0"
  "vsx_xvrsqrtedp\0"
  "vsx_xvrsqrtesp\0"
  "vsx_xvtdivdp\0"
  "vsx_xvtdivsp\0"
  "vsx_xvtlsbb\0"
  "vsx_xvtsqrtdp\0"
  "vsx_xvtsqrtsp\0"
  "vsx_xvtstdcdp\0"
  "vsx_xvtstdcsp\0"
  "vsx_xvxexpdp\0"
  "vsx_xvxexpsp\0"
  "vsx_xvxsigdp\0"
  "vsx_xvxsigsp\0"
  "vsx_xxblendvb\0"
  "vsx_xxblendvd\0"
  "vsx_xxblendvh\0"
  "vsx_xxblendvw\0"
  "vsx_xxeval\0"
  "vsx_xxextractuw\0"
  "vsx_xxgenpcvbm\0"
  "vsx_xxgenpcvdm\0"
  "vsx_xxgenpcvhm\0"
  "vsx_xxgenpcvwm\0"
  "vsx_xxinsertw\0"
  "vsx_xxleqv\0"
  "vsx_xxpermx\0"
  "group_barrier\0"
  "rat_store_typed\0"
  "read_global_size_x\0"
  "read_global_size_y\0"
  "read_global_size_z\0"
  "read_ngroups_x\0"
  "read_ngroups_y\0"
  "read_ngroups_z\0"
  "read_tgid_x\0"
  "read_tgid_y\0"
  "read_tgid_z\0"
  "read_tidig_x\0"
  "read_tidig_y\0"
  "read_tidig_z\0"
  "aes32dsi\0"
  "aes32dsmi\0"
  "aes32esi\0"
  "aes32esmi\0"
  "aes64ds\0"
  "aes64dsm\0"
  "aes64es\0"
  "aes64esm\0"
  "aes64im\0"
  "aes64ks1i\0"
  "aes64ks2\0"
  "mips_ehb\0"
  "mips_ihb\0"
  "mips_pause\0"
  "sha512sig0\0"
  "sha512sig0h\0"
  "sha512sig0l\0"
  "sha512sig1\0"
  "sha512sig1h\0"
  "sha512sig1l\0"
  "sha512sum0\0"
  "sha512sum0r\0"
  "sha512sum1\0"
  "sha512sum1r\0"
  "s390_bdepg\0"
  "s390_bextg\0"
  "s390_efpc\0"
  "s390_lcbb\0"
  "s390_sfpc\0"
  "s390_vaccb\0"
  "s390_vacccq\0"
  "s390_vaccf\0"
  "s390_vaccg\0"
  "s390_vacch\0"
  "s390_vaccq\0"
  "s390_vacq\0"
  "s390_vaq\0"
  "s390_vavgb\0"
  "s390_vavgf\0"
  "s390_vavgg\0"
  "s390_vavgh\0"
  "s390_vavglb\0"
  "s390_vavglf\0"
  "s390_vavglg\0"
  "s390_vavglh\0"
  "s390_vavglq\0"
  "s390_vavgq\0"
  "s390_vbperm\0"
  "s390_vcfn\0"
  "s390_vcksm\0"
  "s390_vclfnhs\0"
  "s390_vclfnls\0"
  "s390_vcnf\0"
  "s390_vcrnfs\0"
  "s390_verimb\0"
  "s390_verimf\0"
  "s390_verimg\0"
  "s390_verimh\0"
  "s390_veval\0"
  "s390_vfaeb\0"
  "s390_vfaef\0"
  "s390_vfaeh\0"
  "s390_vfaezb\0"
  "s390_vfaezf\0"
  "s390_vfaezh\0"
  "s390_vfeeb\0"
  "s390_vfeef\0"
  "s390_vfeeh\0"
  "s390_vfeezb\0"
  "s390_vfeezf\0"
  "s390_vfeezh\0"
  "s390_vfeneb\0"
  "s390_vfenef\0"
  "s390_vfeneh\0"
  "s390_vfenezb\0"
  "s390_vfenezf\0"
  "s390_vfenezh\0"
  "s390_vgemb\0"
  "s390_vgemf\0"
  "s390_vgemg\0"
  "s390_vgemh\0"
  "s390_vgemq\0"
  "s390_vgfmab\0"
  "s390_vgfmaf\0"
  "s390_vgfmag\0"
  "s390_vgfmah\0"
  "s390_vgfmb\0"
  "s390_vgfmf\0"
  "s390_vgfmg\0"
  "s390_vgfmh\0"
  "s390_vistrb\0"
  "s390_vistrf\0"
  "s390_vistrh\0"
  "s390_vlbb\0"
  "s390_vll\0"
  "s390_vlrlr\0"
  "s390_vmaeb\0"
  "s390_vmaef\0"
  "s390_vmaeg\0"
  "s390_vmaeh\0"
  "s390_vmahb\0"
  "s390_vmahf\0"
  "s390_vmahg\0"
  "s390_vmahh\0"
  "s390_vmahq\0"
  "s390_vmaleb\0"
  "s390_vmalef\0"
  "s390_vmaleg\0"
  "s390_vmaleh\0"
  "s390_vmalhb\0"
  "s390_vmalhf\0"
  "s390_vmalhg\0"
  "s390_vmalhh\0"
  "s390_vmalhq\0"
  "s390_vmalob\0"
  "s390_vmalof\0"
  "s390_vmalog\0"
  "s390_vmaloh\0"
  "s390_vmaob\0"
  "s390_vmaof\0"
  "s390_vmaog\0"
  "s390_vmaoh\0"
  "s390_vmeb\0"
  "s390_vmef\0"
  "s390_vmeg\0"
  "s390_vmeh\0"
  "s390_vmhb\0"
  "s390_vmhf\0"
  "s390_vmhg\0"
  "s390_vmhh\0"
  "s390_vmhq\0"
  "s390_vmleb\0"
  "s390_vmlef\0"
  "s390_vmleg\0"
  "s390_vmleh\0"
  "s390_vmlhb\0"
  "s390_vmlhf\0"
  "s390_vmlhg\0"
  "s390_vmlhh\0"
  "s390_vmlhq\0"
  "s390_vmlob\0"
  "s390_vmlof\0"
  "s390_vmlog\0"
  "s390_vmloh\0"
  "s390_vmob\0"
  "s390_vmof\0"
  "s390_vmog\0"
  "s390_vmoh\0"
  "s390_vmslg\0"
  "s390_vpdi\0"
  "s390_vperm\0"
  "s390_vpklsf\0"
  "s390_vpklsg\0"
  "s390_vpklsh\0"
  "s390_vpksf\0"
  "s390_vpksg\0"
  "s390_vpksh\0"
  "s390_vsbcbiq\0"
  "s390_vsbiq\0"
  "s390_vscbib\0"
  "s390_vscbif\0"
  "s390_vscbig\0"
  "s390_vscbih\0"
  "s390_vscbiq\0"
  "s390_vsl\0"
  "s390_vslb\0"
  "s390_vsld\0"
  "s390_vsldb\0"
  "s390_vsq\0"
  "s390_vsra\0"
  "s390_vsrab\0"
  "s390_vsrd\0"
  "s390_vsrl\0"
  "s390_vsrlb\0"
  "s390_vstl\0"
  "s390_vstrcb\0"
  "s390_vstrcf\0"
  "s390_vstrch\0"
  "s390_vstrczb\0"
  "s390_vstrczf\0"
  "s390_vstrczh\0"
  "s390_vstrlr\0"
  "s390_vsumb\0"
  "s390_vsumgf\0"
  "s390_vsumgh\0"
  "s390_vsumh\0"
  "s390_vsumqf\0"
  "s390_vsumqg\0"
  "s390_vtm\0"
  "s390_vuphb\0"
  "s390_vuphf\0"
  "s390_vuphg\0"
  "s390_vuphh\0"
  "s390_vuplb\0"
  "s390_vuplf\0"
  "s390_vuplg\0"
  "s390_vuplhb\0"
  "s390_vuplhf\0"
  "s390_vuplhg\0"
  "s390_vuplhh\0"
  "s390_vuplhw\0"
  "s390_vupllb\0"
  "s390_vupllf\0"
  "s390_vupllg\0"
  "s390_vupllh\0"
  "tx_assist\0"
  "tx_nesting_depth\0"
  "num_subgroups\0"
  "subgroup_ballot\0"
  "subgroup_id\0"
  "subgroup_local_invocation_id\0"
  "subgroup_max_size\0"
  "subgroup_size\0"
  "andm_MMM\0"
  "andm_mmm\0"
  "eqvm_MMM\0"
  "eqvm_mmm\0"
  "extract_vm512l\0"
  "extract_vm512u\0"
  "fencec_s\0"
  "fencei\0"
  "fencem_s\0"
  "fidcr_sss\0"
  "insert_vm512l\0"
  "insert_vm512u\0"
  "lcr_sss\0"
  "lsv_vvss\0"
  "lvm_MMss\0"
  "lvm_mmss\0"
  "lvsd_svs\0"
  "lvsl_svs\0"
  "lvss_svs\0"
  "lzvm_sml\0"
  "negm_MM\0"
  "negm_mm\0"
  "nndm_MMM\0"
  "nndm_mmm\0"
  "orm_MMM\0"
  "orm_mmm\0"
  "pack_f32a\0"
  "pack_f32p\0"
  "pcvm_sml\0"
  "pfchv_ssl\0"
  "pfchvnc_ssl\0"
  "pvadds_vsvMvl\0"
  "pvadds_vsvl\0"
  "pvadds_vsvvl\0"
  "pvadds_vvvMvl\0"
  "pvadds_vvvl\0"
  "pvadds_vvvvl\0"
  "pvaddu_vsvMvl\0"
  "pvaddu_vsvl\0"
  "pvaddu_vsvvl\0"
  "pvaddu_vvvMvl\0"
  "pvaddu_vvvl\0"
  "pvaddu_vvvvl\0"
  "pvand_vsvMvl\0"
  "pvand_vsvl\0"
  "pvand_vsvvl\0"
  "pvand_vvvMvl\0"
  "pvand_vvvl\0"
  "pvand_vvvvl\0"
  "pvbrd_vsMvl\0"
  "pvbrd_vsl\0"
  "pvbrd_vsvl\0"
  "pvbrv_vvMvl\0"
  "pvbrv_vvl\0"
  "pvbrv_vvvl\0"
  "pvbrvlo_vvl\0"
  "pvbrvlo_vvmvl\0"
  "pvbrvlo_vvvl\0"
  "pvbrvup_vvl\0"
  "pvbrvup_vvmvl\0"
  "pvbrvup_vvvl\0"
  "pvcmps_vsvMvl\0"
  "pvcmps_vsvl\0"
  "pvcmps_vsvvl\0"
  "pvcmps_vvvMvl\0"
  "pvcmps_vvvl\0"
  "pvcmps_vvvvl\0"
  "pvcmpu_vsvMvl\0"
  "pvcmpu_vsvl\0"
  "pvcmpu_vsvvl\0"
  "pvcmpu_vvvMvl\0"
  "pvcmpu_vvvl\0"
  "pvcmpu_vvvvl\0"
  "pvcvtsw_vvl\0"
  "pvcvtsw_vvvl\0"
  "pvcvtws_vvMvl\0"
  "pvcvtws_vvl\0"
  "pvcvtws_vvvl\0"
  "pvcvtwsrz_vvMvl\0"
  "pvcvtwsrz_vvl\0"
  "pvcvtwsrz_vvvl\0"
  "pveqv_vsvMvl\0"
  "pveqv_vsvl\0"
  "pveqv_vsvvl\0"
  "pveqv_vvvMvl\0"
  "pveqv_vvvl\0"
  "pveqv_vvvvl\0"
  "pvfadd_vsvMvl\0"
  "pvfadd_vsvl\0"
  "pvfadd_vsvvl\0"
  "pvfadd_vvvMvl\0"
  "pvfadd_vvvl\0"
  "pvfadd_vvvvl\0"
  "pvfcmp_vsvMvl\0"
  "pvfcmp_vsvl\0"
  "pvfcmp_vsvvl\0"
  "pvfcmp_vvvMvl\0"
  "pvfcmp_vvvl\0"
  "pvfcmp_vvvvl\0"
  "pvfmad_vsvvMvl\0"
  "pvfmad_vsvvl\0"
  "pvfmad_vsvvvl\0"
  "pvfmad_vvsvMvl\0"
  "pvfmad_vvsvl\0"
  "pvfmad_vvsvvl\0"
  "pvfmad_vvvvMvl\0"
  "pvfmad_vvvvl\0"
  "pvfmad_vvvvvl\0"
  "pvfmax_vsvMvl\0"
  "pvfmax_vsvl\0"
  "pvfmax_vsvvl\0"
  "pvfmax_vvvMvl\0"
  "pvfmax_vvvl\0"
  "pvfmax_vvvvl\0"
  "pvfmin_vsvMvl\0"
  "pvfmin_vsvl\0"
  "pvfmin_vsvvl\0"
  "pvfmin_vvvMvl\0"
  "pvfmin_vvvl\0"
  "pvfmin_vvvvl\0"
  "pvfmkaf_Ml\0"
  "pvfmkat_Ml\0"
  "pvfmkseq_MvMl\0"
  "pvfmkseq_Mvl\0"
  "pvfmkseqnan_MvMl\0"
  "pvfmkseqnan_Mvl\0"
  "pvfmksge_MvMl\0"
  "pvfmksge_Mvl\0"
  "pvfmksgenan_MvMl\0"
  "pvfmksgenan_Mvl\0"
  "pvfmksgt_MvMl\0"
  "pvfmksgt_Mvl\0"
  "pvfmksgtnan_MvMl\0"
  "pvfmksgtnan_Mvl\0"
  "pvfmksle_MvMl\0"
  "pvfmksle_Mvl\0"
  "pvfmkslenan_MvMl\0"
  "pvfmkslenan_Mvl\0"
  "pvfmksloeq_mvl\0"
  "pvfmksloeq_mvml\0"
  "pvfmksloeqnan_mvl\0"
  "pvfmksloeqnan_mvml\0"
  "pvfmksloge_mvl\0"
  "pvfmksloge_mvml\0"
  "pvfmkslogenan_mvl\0"
  "pvfmkslogenan_mvml\0"
  "pvfmkslogt_mvl\0"
  "pvfmkslogt_mvml\0"
  "pvfmkslogtnan_mvl\0"
  "pvfmkslogtnan_mvml\0"
  "pvfmkslole_mvl\0"
  "pvfmkslole_mvml\0"
  "pvfmkslolenan_mvl\0"
  "pvfmkslolenan_mvml\0"
  "pvfmkslolt_mvl\0"
  "pvfmkslolt_mvml\0"
  "pvfmksloltnan_mvl\0"
  "pvfmksloltnan_mvml\0"
  "pvfmkslonan_mvl\0"
  "pvfmkslonan_mvml\0"
  "pvfmkslone_mvl\0"
  "pvfmkslone_mvml\0"
  "pvfmkslonenan_mvl\0"
  "pvfmkslonenan_mvml\0"
  "pvfmkslonum_mvl\0"
  "pvfmkslonum_mvml\0"
  "pvfmkslt_MvMl\0"
  "pvfmkslt_Mvl\0"
  "pvfmksltnan_MvMl\0"
  "pvfmksltnan_Mvl\0"
  "pvfmksnan_MvMl\0"
  "pvfmksnan_Mvl\0"
  "pvfmksne_MvMl\0"
  "pvfmksne_Mvl\0"
  "pvfmksnenan_MvMl\0"
  "pvfmksnenan_Mvl\0"
  "pvfmksnum_MvMl\0"
  "pvfmksnum_Mvl\0"
  "pvfmksupeq_mvl\0"
  "pvfmksupeq_mvml\0"
  "pvfmksupeqnan_mvl\0"
  "pvfmksupeqnan_mvml\0"
  "pvfmksupge_mvl\0"
  "pvfmksupge_mvml\0"
  "pvfmksupgenan_mvl\0"
  "pvfmksupgenan_mvml\0"
  "pvfmksupgt_mvl\0"
  "pvfmksupgt_mvml\0"
  "pvfmksupgtnan_mvl\0"
  "pvfmksupgtnan_mvml\0"
  "pvfmksuple_mvl\0"
  "pvfmksuple_mvml\0"
  "pvfmksuplenan_mvl\0"
  "pvfmksuplenan_mvml\0"
  "pvfmksuplt_mvl\0"
  "pvfmksuplt_mvml\0"
  "pvfmksupltnan_mvl\0"
  "pvfmksupltnan_mvml\0"
  "pvfmksupnan_mvl\0"
  "pvfmksupnan_mvml\0"
  "pvfmksupne_mvl\0"
  "pvfmksupne_mvml\0"
  "pvfmksupnenan_mvl\0"
  "pvfmksupnenan_mvml\0"
  "pvfmksupnum_mvl\0"
  "pvfmksupnum_mvml\0"
  "pvfmkweq_MvMl\0"
  "pvfmkweq_Mvl\0"
  "pvfmkweqnan_MvMl\0"
  "pvfmkweqnan_Mvl\0"
  "pvfmkwge_MvMl\0"
  "pvfmkwge_Mvl\0"
  "pvfmkwgenan_MvMl\0"
  "pvfmkwgenan_Mvl\0"
  "pvfmkwgt_MvMl\0"
  "pvfmkwgt_Mvl\0"
  "pvfmkwgtnan_MvMl\0"
  "pvfmkwgtnan_Mvl\0"
  "pvfmkwle_MvMl\0"
  "pvfmkwle_Mvl\0"
  "pvfmkwlenan_MvMl\0"
  "pvfmkwlenan_Mvl\0"
  "pvfmkwloeq_mvl\0"
  "pvfmkwloeq_mvml\0"
  "pvfmkwloeqnan_mvl\0"
  "pvfmkwloeqnan_mvml\0"
  "pvfmkwloge_mvl\0"
  "pvfmkwloge_mvml\0"
  "pvfmkwlogenan_mvl\0"
  "pvfmkwlogenan_mvml\0"
  "pvfmkwlogt_mvl\0"
  "pvfmkwlogt_mvml\0"
  "pvfmkwlogtnan_mvl\0"
  "pvfmkwlogtnan_mvml\0"
  "pvfmkwlole_mvl\0"
  "pvfmkwlole_mvml\0"
  "pvfmkwlolenan_mvl\0"
  "pvfmkwlolenan_mvml\0"
  "pvfmkwlolt_mvl\0"
  "pvfmkwlolt_mvml\0"
  "pvfmkwloltnan_mvl\0"
  "pvfmkwloltnan_mvml\0"
  "pvfmkwlonan_mvl\0"
  "pvfmkwlonan_mvml\0"
  "pvfmkwlone_mvl\0"
  "pvfmkwlone_mvml\0"
  "pvfmkwlonenan_mvl\0"
  "pvfmkwlonenan_mvml\0"
  "pvfmkwlonum_mvl\0"
  "pvfmkwlonum_mvml\0"
  "pvfmkwlt_MvMl\0"
  "pvfmkwlt_Mvl\0"
  "pvfmkwltnan_MvMl\0"
  "pvfmkwltnan_Mvl\0"
  "pvfmkwnan_MvMl\0"
  "pvfmkwnan_Mvl\0"
  "pvfmkwne_MvMl\0"
  "pvfmkwne_Mvl\0"
  "pvfmkwnenan_MvMl\0"
  "pvfmkwnenan_Mvl\0"
  "pvfmkwnum_MvMl\0"
  "pvfmkwnum_Mvl\0"
  "pvfmkwupeq_mvl\0"
  "pvfmkwupeq_mvml\0"
  "pvfmkwupeqnan_mvl\0"
  "pvfmkwupeqnan_mvml\0"
  "pvfmkwupge_mvl\0"
  "pvfmkwupge_mvml\0"
  "pvfmkwupgenan_mvl\0"
  "pvfmkwupgenan_mvml\0"
  "pvfmkwupgt_mvl\0"
  "pvfmkwupgt_mvml\0"
  "pvfmkwupgtnan_mvl\0"
  "pvfmkwupgtnan_mvml\0"
  "pvfmkwuple_mvl\0"
  "pvfmkwuple_mvml\0"
  "pvfmkwuplenan_mvl\0"
  "pvfmkwuplenan_mvml\0"
  "pvfmkwuplt_mvl\0"
  "pvfmkwuplt_mvml\0"
  "pvfmkwupltnan_mvl\0"
  "pvfmkwupltnan_mvml\0"
  "pvfmkwupnan_mvl\0"
  "pvfmkwupnan_mvml\0"
  "pvfmkwupne_mvl\0"
  "pvfmkwupne_mvml\0"
  "pvfmkwupnenan_mvl\0"
  "pvfmkwupnenan_mvml\0"
  "pvfmkwupnum_mvl\0"
  "pvfmkwupnum_mvml\0"
  "pvfmsb_vsvvMvl\0"
  "pvfmsb_vsvvl\0"
  "pvfmsb_vsvvvl\0"
  "pvfmsb_vvsvMvl\0"
  "pvfmsb_vvsvl\0"
  "pvfmsb_vvsvvl\0"
  "pvfmsb_vvvvMvl\0"
  "pvfmsb_vvvvl\0"
  "pvfmsb_vvvvvl\0"
  "pvfmul_vsvMvl\0"
  "pvfmul_vsvl\0"
  "pvfmul_vsvvl\0"
  "pvfmul_vvvMvl\0"
  "pvfmul_vvvl\0"
  "pvfmul_vvvvl\0"
  "pvfnmad_vsvvMvl\0"
  "pvfnmad_vsvvl\0"
  "pvfnmad_vsvvvl\0"
  "pvfnmad_vvsvMvl\0"
  "pvfnmad_vvsvl\0"
  "pvfnmad_vvsvvl\0"
  "pvfnmad_vvvvMvl\0"
  "pvfnmad_vvvvl\0"
  "pvfnmad_vvvvvl\0"
  "pvfnmsb_vsvvMvl\0"
  "pvfnmsb_vsvvl\0"
  "pvfnmsb_vsvvvl\0"
  "pvfnmsb_vvsvMvl\0"
  "pvfnmsb_vvsvl\0"
  "pvfnmsb_vvsvvl\0"
  "pvfnmsb_vvvvMvl\0"
  "pvfnmsb_vvvvl\0"
  "pvfnmsb_vvvvvl\0"
  "pvfsub_vsvMvl\0"
  "pvfsub_vsvl\0"
  "pvfsub_vsvvl\0"
  "pvfsub_vvvMvl\0"
  "pvfsub_vvvl\0"
  "pvfsub_vvvvl\0"
  "pvldz_vvMvl\0"
  "pvldz_vvl\0"
  "pvldz_vvvl\0"
  "pvldzlo_vvl\0"
  "pvldzlo_vvmvl\0"
  "pvldzlo_vvvl\0"
  "pvldzup_vvl\0"
  "pvldzup_vvmvl\0"
  "pvldzup_vvvl\0"
  "pvmaxs_vsvMvl\0"
  "pvmaxs_vsvl\0"
  "pvmaxs_vsvvl\0"
  "pvmaxs_vvvMvl\0"
  "pvmaxs_vvvl\0"
  "pvmaxs_vvvvl\0"
  "pvmins_vsvMvl\0"
  "pvmins_vsvl\0"
  "pvmins_vsvvl\0"
  "pvmins_vvvMvl\0"
  "pvmins_vvvl\0"
  "pvmins_vvvvl\0"
  "pvor_vsvMvl\0"
  "pvor_vsvl\0"
  "pvor_vsvvl\0"
  "pvor_vvvMvl\0"
  "pvor_vvvl\0"
  "pvor_vvvvl\0"
  "pvpcnt_vvMvl\0"
  "pvpcnt_vvl\0"
  "pvpcnt_vvvl\0"
  "pvpcntlo_vvl\0"
  "pvpcntlo_vvmvl\0"
  "pvpcntlo_vvvl\0"
  "pvpcntup_vvl\0"
  "pvpcntup_vvmvl\0"
  "pvpcntup_vvvl\0"
  "pvrcp_vvl\0"
  "pvrcp_vvvl\0"
  "pvrsqrt_vvl\0"
  "pvrsqrt_vvvl\0"
  "pvrsqrtnex_vvl\0"
  "pvrsqrtnex_vvvl\0"
  "pvseq_vl\0"
  "pvseq_vvl\0"
  "pvseqlo_vl\0"
  "pvseqlo_vvl\0"
  "pvsequp_vl\0"
  "pvsequp_vvl\0"
  "pvsla_vvsMvl\0"
  "pvsla_vvsl\0"
  "pvsla_vvsvl\0"
  "pvsla_vvvMvl\0"
  "pvsla_vvvl\0"
  "pvsla_vvvvl\0"
  "pvsll_vvsMvl\0"
  "pvsll_vvsl\0"
  "pvsll_vvsvl\0"
  "pvsll_vvvMvl\0"
  "pvsll_vvvl\0"
  "pvsll_vvvvl\0"
  "pvsra_vvsMvl\0"
  "pvsra_vvsl\0"
  "pvsra_vvsvl\0"
  "pvsra_vvvMvl\0"
  "pvsra_vvvl\0"
  "pvsra_vvvvl\0"
  "pvsrl_vvsMvl\0"
  "pvsrl_vvsl\0"
  "pvsrl_vvsvl\0"
  "pvsrl_vvvMvl\0"
  "pvsrl_vvvl\0"
  "pvsrl_vvvvl\0"
  "pvsubs_vsvMvl\0"
  "pvsubs_vsvl\0"
  "pvsubs_vsvvl\0"
  "pvsubs_vvvMvl\0"
  "pvsubs_vvvl\0"
  "pvsubs_vvvvl\0"
  "pvsubu_vsvMvl\0"
  "pvsubu_vsvl\0"
  "pvsubu_vsvvl\0"
  "pvsubu_vvvMvl\0"
  "pvsubu_vvvl\0"
  "pvsubu_vvvvl\0"
  "pvxor_vsvMvl\0"
  "pvxor_vsvl\0"
  "pvxor_vsvvl\0"
  "pvxor_vvvMvl\0"
  "pvxor_vvvl\0"
  "pvxor_vvvvl\0"
  "scr_sss\0"
  "svm_sMs\0"
  "svm_sms\0"
  "svob\0"
  "tovm_sml\0"
  "tscr_ssss\0"
  "vaddsl_vsvl\0"
  "vaddsl_vsvmvl\0"
  "vaddsl_vsvvl\0"
  "vaddsl_vvvl\0"
  "vaddsl_vvvmvl\0"
  "vaddsl_vvvvl\0"
  "vaddswsx_vsvl\0"
  "vaddswsx_vsvmvl\0"
  "vaddswsx_vsvvl\0"
  "vaddswsx_vvvl\0"
  "vaddswsx_vvvmvl\0"
  "vaddswsx_vvvvl\0"
  "vaddswzx_vsvl\0"
  "vaddswzx_vsvmvl\0"
  "vaddswzx_vsvvl\0"
  "vaddswzx_vvvl\0"
  "vaddswzx_vvvmvl\0"
  "vaddswzx_vvvvl\0"
  "vaddul_vsvl\0"
  "vaddul_vsvmvl\0"
  "vaddul_vsvvl\0"
  "vaddul_vvvl\0"
  "vaddul_vvvmvl\0"
  "vaddul_vvvvl\0"
  "vadduw_vsvl\0"
  "vadduw_vsvmvl\0"
  "vadduw_vsvvl\0"
  "vadduw_vvvl\0"
  "vadduw_vvvmvl\0"
  "vadduw_vvvvl\0"
  "vand_vsvl\0"
  "vand_vsvmvl\0"
  "vand_vsvvl\0"
  "vand_vvvl\0"
  "vand_vvvmvl\0"
  "vand_vvvvl\0"
  "vbrdd_vsl\0"
  "vbrdd_vsmvl\0"
  "vbrdd_vsvl\0"
  "vbrdl_vsl\0"
  "vbrdl_vsmvl\0"
  "vbrdl_vsvl\0"
  "vbrds_vsl\0"
  "vbrds_vsmvl\0"
  "vbrds_vsvl\0"
  "vbrdw_vsl\0"
  "vbrdw_vsmvl\0"
  "vbrdw_vsvl\0"
  "vbrv_vvl\0"
  "vbrv_vvmvl\0"
  "vbrv_vvvl\0"
  "vcmpsl_vsvl\0"
  "vcmpsl_vsvmvl\0"
  "vcmpsl_vsvvl\0"
  "vcmpsl_vvvl\0"
  "vcmpsl_vvvmvl\0"
  "vcmpsl_vvvvl\0"
  "vcmpswsx_vsvl\0"
  "vcmpswsx_vsvmvl\0"
  "vcmpswsx_vsvvl\0"
  "vcmpswsx_vvvl\0"
  "vcmpswsx_vvvmvl\0"
  "vcmpswsx_vvvvl\0"
  "vcmpswzx_vsvl\0"
  "vcmpswzx_vsvmvl\0"
  "vcmpswzx_vsvvl\0"
  "vcmpswzx_vvvl\0"
  "vcmpswzx_vvvmvl\0"
  "vcmpswzx_vvvvl\0"
  "vcmpul_vsvl\0"
  "vcmpul_vsvmvl\0"
  "vcmpul_vsvvl\0"
  "vcmpul_vvvl\0"
  "vcmpul_vvvmvl\0"
  "vcmpul_vvvvl\0"
  "vcmpuw_vsvl\0"
  "vcmpuw_vsvmvl\0"
  "vcmpuw_vsvvl\0"
  "vcmpuw_vvvl\0"
  "vcmpuw_vvvmvl\0"
  "vcmpuw_vvvvl\0"
  "vcp_vvmvl\0"
  "vcvtdl_vvl\0"
  "vcvtdl_vvvl\0"
  "vcvtds_vvl\0"
  "vcvtds_vvvl\0"
  "vcvtdw_vvl\0"
  "vcvtdw_vvvl\0"
  "vcvtld_vvl\0"
  "vcvtld_vvmvl\0"
  "vcvtld_vvvl\0"
  "vcvtldrz_vvl\0"
  "vcvtldrz_vvmvl\0"
  "vcvtldrz_vvvl\0"
  "vcvtsd_vvl\0"
  "vcvtsd_vvvl\0"
  "vcvtsw_vvl\0"
  "vcvtsw_vvvl\0"
  "vcvtwdsx_vvl\0"
  "vcvtwdsx_vvmvl\0"
  "vcvtwdsx_vvvl\0"
  "vcvtwdsxrz_vvl\0"
  "vcvtwdsxrz_vvmvl\0"
  "vcvtwdsxrz_vvvl\0"
  "vcvtwdzx_vvl\0"
  "vcvtwdzx_vvmvl\0"
  "vcvtwdzx_vvvl\0"
  "vcvtwdzxrz_vvl\0"
  "vcvtwdzxrz_vvmvl\0"
  "vcvtwdzxrz_vvvl\0"
  "vcvtwssx_vvl\0"
  "vcvtwssx_vvmvl\0"
  "vcvtwssx_vvvl\0"
  "vcvtwssxrz_vvl\0"
  "vcvtwssxrz_vvmvl\0"
  "vcvtwssxrz_vvvl\0"
  "vcvtwszx_vvl\0"
  "vcvtwszx_vvmvl\0"
  "vcvtwszx_vvvl\0"
  "vcvtwszxrz_vvl\0"
  "vcvtwszxrz_vvmvl\0"
  "vcvtwszxrz_vvvl\0"
  "vdivsl_vsvl\0"
  "vdivsl_vsvmvl\0"
  "vdivsl_vsvvl\0"
  "vdivsl_vvsl\0"
  "vdivsl_vvsmvl\0"
  "vdivsl_vvsvl\0"
  "vdivsl_vvvl\0"
  "vdivsl_vvvmvl\0"
  "vdivsl_vvvvl\0"
  "vdivswsx_vsvl\0"
  "vdivswsx_vsvmvl\0"
  "vdivswsx_vsvvl\0"
  "vdivswsx_vvsl\0"
  "vdivswsx_vvsmvl\0"
  "vdivswsx_vvsvl\0"
  "vdivswsx_vvvl\0"
  "vdivswsx_vvvmvl\0"
  "vdivswsx_vvvvl\0"
  "vdivswzx_vsvl\0"
  "vdivswzx_vsvmvl\0"
  "vdivswzx_vsvvl\0"
  "vdivswzx_vvsl\0"
  "vdivswzx_vvsmvl\0"
  "vdivswzx_vvsvl\0"
  "vdivswzx_vvvl\0"
  "vdivswzx_vvvmvl\0"
  "vdivswzx_vvvvl\0"
  "vdivul_vsvl\0"
  "vdivul_vsvmvl\0"
  "vdivul_vsvvl\0"
  "vdivul_vvsl\0"
  "vdivul_vvsmvl\0"
  "vdivul_vvsvl\0"
  "vdivul_vvvl\0"
  "vdivul_vvvmvl\0"
  "vdivul_vvvvl\0"
  "vdivuw_vsvl\0"
  "vdivuw_vsvmvl\0"
  "vdivuw_vsvvl\0"
  "vdivuw_vvsl\0"
  "vdivuw_vvsmvl\0"
  "vdivuw_vvsvl\0"
  "vdivuw_vvvl\0"
  "vdivuw_vvvmvl\0"
  "vdivuw_vvvvl\0"
  "veqv_vsvl\0"
  "veqv_vsvmvl\0"
  "veqv_vsvvl\0"
  "veqv_vvvl\0"
  "veqv_vvvmvl\0"
  "veqv_vvvvl\0"
  "vex_vvmvl\0"
  "vfaddd_vsvl\0"
  "vfaddd_vsvmvl\0"
  "vfaddd_vsvvl\0"
  "vfaddd_vvvl\0"
  "vfaddd_vvvmvl\0"
  "vfaddd_vvvvl\0"
  "vfadds_vsvl\0"
  "vfadds_vsvmvl\0"
  "vfadds_vsvvl\0"
  "vfadds_vvvl\0"
  "vfadds_vvvmvl\0"
  "vfadds_vvvvl\0"
  "vfcmpd_vsvl\0"
  "vfcmpd_vsvmvl\0"
  "vfcmpd_vsvvl\0"
  "vfcmpd_vvvl\0"
  "vfcmpd_vvvmvl\0"
  "vfcmpd_vvvvl\0"
  "vfcmps_vsvl\0"
  "vfcmps_vsvmvl\0"
  "vfcmps_vsvvl\0"
  "vfcmps_vvvl\0"
  "vfcmps_vvvmvl\0"
  "vfcmps_vvvvl\0"
  "vfdivd_vsvl\0"
  "vfdivd_vsvmvl\0"
  "vfdivd_vsvvl\0"
  "vfdivd_vvvl\0"
  "vfdivd_vvvmvl\0"
  "vfdivd_vvvvl\0"
  "vfdivs_vsvl\0"
  "vfdivs_vsvmvl\0"
  "vfdivs_vsvvl\0"
  "vfdivs_vvvl\0"
  "vfdivs_vvvmvl\0"
  "vfdivs_vvvvl\0"
  "vfmadd_vsvvl\0"
  "vfmadd_vsvvmvl\0"
  "vfmadd_vsvvvl\0"
  "vfmadd_vvsvl\0"
  "vfmadd_vvsvmvl\0"
  "vfmadd_vvsvvl\0"
  "vfmadd_vvvvl\0"
  "vfmadd_vvvvmvl\0"
  "vfmadd_vvvvvl\0"
  "vfmads_vsvvl\0"
  "vfmads_vsvvmvl\0"
  "vfmads_vsvvvl\0"
  "vfmads_vvsvl\0"
  "vfmads_vvsvmvl\0"
  "vfmads_vvsvvl\0"
  "vfmads_vvvvl\0"
  "vfmads_vvvvmvl\0"
  "vfmads_vvvvvl\0"
  "vfmaxd_vsvl\0"
  "vfmaxd_vsvmvl\0"
  "vfmaxd_vsvvl\0"
  "vfmaxd_vvvl\0"
  "vfmaxd_vvvmvl\0"
  "vfmaxd_vvvvl\0"
  "vfmaxs_vsvl\0"
  "vfmaxs_vsvmvl\0"
  "vfmaxs_vsvvl\0"
  "vfmaxs_vvvl\0"
  "vfmaxs_vvvmvl\0"
  "vfmaxs_vvvvl\0"
  "vfmind_vsvl\0"
  "vfmind_vsvmvl\0"
  "vfmind_vsvvl\0"
  "vfmind_vvvl\0"
  "vfmind_vvvmvl\0"
  "vfmind_vvvvl\0"
  "vfmins_vsvl\0"
  "vfmins_vsvmvl\0"
  "vfmins_vsvvl\0"
  "vfmins_vvvl\0"
  "vfmins_vvvmvl\0"
  "vfmins_vvvvl\0"
  "vfmkdeq_mvl\0"
  "vfmkdeq_mvml\0"
  "vfmkdeqnan_mvl\0"
  "vfmkdeqnan_mvml\0"
  "vfmkdge_mvl\0"
  "vfmkdge_mvml\0"
  "vfmkdgenan_mvl\0"
  "vfmkdgenan_mvml\0"
  "vfmkdgt_mvl\0"
  "vfmkdgt_mvml\0"
  "vfmkdgtnan_mvl\0"
  "vfmkdgtnan_mvml\0"
  "vfmkdle_mvl\0"
  "vfmkdle_mvml\0"
  "vfmkdlenan_mvl\0"
  "vfmkdlenan_mvml\0"
  "vfmkdlt_mvl\0"
  "vfmkdlt_mvml\0"
  "vfmkdltnan_mvl\0"
  "vfmkdltnan_mvml\0"
  "vfmkdnan_mvl\0"
  "vfmkdnan_mvml\0"
  "vfmkdne_mvl\0"
  "vfmkdne_mvml\0"
  "vfmkdnenan_mvl\0"
  "vfmkdnenan_mvml\0"
  "vfmkdnum_mvl\0"
  "vfmkdnum_mvml\0"
  "vfmklaf_ml\0"
  "vfmklat_ml\0"
  "vfmkleq_mvl\0"
  "vfmkleq_mvml\0"
  "vfmkleqnan_mvl\0"
  "vfmkleqnan_mvml\0"
  "vfmklge_mvl\0"
  "vfmklge_mvml\0"
  "vfmklgenan_mvl\0"
  "vfmklgenan_mvml\0"
  "vfmklgt_mvl\0"
  "vfmklgt_mvml\0"
  "vfmklgtnan_mvl\0"
  "vfmklgtnan_mvml\0"
  "vfmklle_mvl\0"
  "vfmklle_mvml\0"
  "vfmkllenan_mvl\0"
  "vfmkllenan_mvml\0"
  "vfmkllt_mvl\0"
  "vfmkllt_mvml\0"
  "vfmklltnan_mvl\0"
  "vfmklltnan_mvml\0"
  "vfmklnan_mvl\0"
  "vfmklnan_mvml\0"
  "vfmklne_mvl\0"
  "vfmklne_mvml\0"
  "vfmklnenan_mvl\0"
  "vfmklnenan_mvml\0"
  "vfmklnum_mvl\0"
  "vfmklnum_mvml\0"
  "vfmkseq_mvl\0"
  "vfmkseq_mvml\0"
  "vfmkseqnan_mvl\0"
  "vfmkseqnan_mvml\0"
  "vfmksge_mvl\0"
  "vfmksge_mvml\0"
  "vfmksgenan_mvl\0"
  "vfmksgenan_mvml\0"
  "vfmksgt_mvl\0"
  "vfmksgt_mvml\0"
  "vfmksgtnan_mvl\0"
  "vfmksgtnan_mvml\0"
  "vfmksle_mvl\0"
  "vfmksle_mvml\0"
  "vfmkslenan_mvl\0"
  "vfmkslenan_mvml\0"
  "vfmkslt_mvl\0"
  "vfmkslt_mvml\0"
  "vfmksltnan_mvl\0"
  "vfmksltnan_mvml\0"
  "vfmksnan_mvl\0"
  "vfmksnan_mvml\0"
  "vfmksne_mvl\0"
  "vfmksne_mvml\0"
  "vfmksnenan_mvl\0"
  "vfmksnenan_mvml\0"
  "vfmksnum_mvl\0"
  "vfmksnum_mvml\0"
  "vfmkweq_mvl\0"
  "vfmkweq_mvml\0"
  "vfmkweqnan_mvl\0"
  "vfmkweqnan_mvml\0"
  "vfmkwge_mvl\0"
  "vfmkwge_mvml\0"
  "vfmkwgenan_mvl\0"
  "vfmkwgenan_mvml\0"
  "vfmkwgt_mvl\0"
  "vfmkwgt_mvml\0"
  "vfmkwgtnan_mvl\0"
  "vfmkwgtnan_mvml\0"
  "vfmkwle_mvl\0"
  "vfmkwle_mvml\0"
  "vfmkwlenan_mvl\0"
  "vfmkwlenan_mvml\0"
  "vfmkwlt_mvl\0"
  "vfmkwlt_mvml\0"
  "vfmkwltnan_mvl\0"
  "vfmkwltnan_mvml\0"
  "vfmkwnan_mvl\0"
  "vfmkwnan_mvml\0"
  "vfmkwne_mvl\0"
  "vfmkwne_mvml\0"
  "vfmkwnenan_mvl\0"
  "vfmkwnenan_mvml\0"
  "vfmkwnum_mvl\0"
  "vfmkwnum_mvml\0"
  "vfmsbd_vsvvl\0"
  "vfmsbd_vsvvmvl\0"
  "vfmsbd_vsvvvl\0"
  "vfmsbd_vvsvl\0"
  "vfmsbd_vvsvmvl\0"
  "vfmsbd_vvsvvl\0"
  "vfmsbd_vvvvl\0"
  "vfmsbd_vvvvmvl\0"
  "vfmsbd_vvvvvl\0"
  "vfmsbs_vsvvl\0"
  "vfmsbs_vsvvmvl\0"
  "vfmsbs_vsvvvl\0"
  "vfmsbs_vvsvl\0"
  "vfmsbs_vvsvmvl\0"
  "vfmsbs_vvsvvl\0"
  "vfmsbs_vvvvl\0"
  "vfmsbs_vvvvmvl\0"
  "vfmsbs_vvvvvl\0"
  "vfmuld_vsvl\0"
  "vfmuld_vsvmvl\0"
  "vfmuld_vsvvl\0"
  "vfmuld_vvvl\0"
  "vfmuld_vvvmvl\0"
  "vfmuld_vvvvl\0"
  "vfmuls_vsvl\0"
  "vfmuls_vsvmvl\0"
  "vfmuls_vsvvl\0"
  "vfmuls_vvvl\0"
  "vfmuls_vvvmvl\0"
  "vfmuls_vvvvl\0"
  "vfnmadd_vsvvl\0"
  "vfnmadd_vsvvmvl\0"
  "vfnmadd_vsvvvl\0"
  "vfnmadd_vvsvl\0"
  "vfnmadd_vvsvmvl\0"
  "vfnmadd_vvsvvl\0"
  "vfnmadd_vvvvl\0"
  "vfnmadd_vvvvmvl\0"
  "vfnmadd_vvvvvl\0"
  "vfnmads_vsvvl\0"
  "vfnmads_vsvvmvl\0"
  "vfnmads_vsvvvl\0"
  "vfnmads_vvsvl\0"
  "vfnmads_vvsvmvl\0"
  "vfnmads_vvsvvl\0"
  "vfnmads_vvvvl\0"
  "vfnmads_vvvvmvl\0"
  "vfnmads_vvvvvl\0"
  "vfnmsbd_vsvvl\0"
  "vfnmsbd_vsvvmvl\0"
  "vfnmsbd_vsvvvl\0"
  "vfnmsbd_vvsvl\0"
  "vfnmsbd_vvsvmvl\0"
  "vfnmsbd_vvsvvl\0"
  "vfnmsbd_vvvvl\0"
  "vfnmsbd_vvvvmvl\0"
  "vfnmsbd_vvvvvl\0"
  "vfnmsbs_vsvvl\0"
  "vfnmsbs_vsvvmvl\0"
  "vfnmsbs_vsvvvl\0"
  "vfnmsbs_vvsvl\0"
  "vfnmsbs_vvsvmvl\0"
  "vfnmsbs_vvsvvl\0"
  "vfnmsbs_vvvvl\0"
  "vfnmsbs_vvvvmvl\0"
  "vfnmsbs_vvvvvl\0"
  "vfrmaxdfst_vvl\0"
  "vfrmaxdfst_vvvl\0"
  "vfrmaxdlst_vvl\0"
  "vfrmaxdlst_vvvl\0"
  "vfrmaxsfst_vvl\0"
  "vfrmaxsfst_vvvl\0"
  "vfrmaxslst_vvl\0"
  "vfrmaxslst_vvvl\0"
  "vfrmindfst_vvl\0"
  "vfrmindfst_vvvl\0"
  "vfrmindlst_vvl\0"
  "vfrmindlst_vvvl\0"
  "vfrminsfst_vvl\0"
  "vfrminsfst_vvvl\0"
  "vfrminslst_vvl\0"
  "vfrminslst_vvvl\0"
  "vfsqrtd_vvl\0"
  "vfsqrtd_vvvl\0"
  "vfsqrts_vvl\0"
  "vfsqrts_vvvl\0"
  "vfsubd_vsvl\0"
  "vfsubd_vsvmvl\0"
  "vfsubd_vsvvl\0"
  "vfsubd_vvvl\0"
  "vfsubd_vvvmvl\0"
  "vfsubd_vvvvl\0"
  "vfsubs_vsvl\0"
  "vfsubs_vsvmvl\0"
  "vfsubs_vsvvl\0"
  "vfsubs_vvvl\0"
  "vfsubs_vvvmvl\0"
  "vfsubs_vvvvl\0"
  "vfsumd_vvl\0"
  "vfsumd_vvml\0"
  "vfsums_vvl\0"
  "vfsums_vvml\0"
  "vgt_vvssl\0"
  "vgt_vvssml\0"
  "vgt_vvssmvl\0"
  "vgt_vvssvl\0"
  "vgtlsx_vvssl\0"
  "vgtlsx_vvssml\0"
  "vgtlsx_vvssmvl\0"
  "vgtlsx_vvssvl\0"
  "vgtlsxnc_vvssl\0"
  "vgtlsxnc_vvssml\0"
  "vgtlsxnc_vvssmvl\0"
  "vgtlsxnc_vvssvl\0"
  "vgtlzx_vvssl\0"
  "vgtlzx_vvssml\0"
  "vgtlzx_vvssmvl\0"
  "vgtlzx_vvssvl\0"
  "vgtlzxnc_vvssl\0"
  "vgtlzxnc_vvssml\0"
  "vgtlzxnc_vvssmvl\0"
  "vgtlzxnc_vvssvl\0"
  "vgtnc_vvssl\0"
  "vgtnc_vvssml\0"
  "vgtnc_vvssmvl\0"
  "vgtnc_vvssvl\0"
  "vgtu_vvssl\0"
  "vgtu_vvssml\0"
  "vgtu_vvssmvl\0"
  "vgtu_vvssvl\0"
  "vgtunc_vvssl\0"
  "vgtunc_vvssml\0"
  "vgtunc_vvssmvl\0"
  "vgtunc_vvssvl\0"
  "vld2d_vssl\0"
  "vld2d_vssvl\0"
  "vld2dnc_vssl\0"
  "vld2dnc_vssvl\0"
  "vld_vssl\0"
  "vld_vssvl\0"
  "vldl2dsx_vssl\0"
  "vldl2dsx_vssvl\0"
  "vldl2dsxnc_vssl\0"
  "vldl2dsxnc_vssvl\0"
  "vldl2dzx_vssl\0"
  "vldl2dzx_vssvl\0"
  "vldl2dzxnc_vssl\0"
  "vldl2dzxnc_vssvl\0"
  "vldlsx_vssl\0"
  "vldlsx_vssvl\0"
  "vldlsxnc_vssl\0"
  "vldlsxnc_vssvl\0"
  "vldlzx_vssl\0"
  "vldlzx_vssvl\0"
  "vldlzxnc_vssl\0"
  "vldlzxnc_vssvl\0"
  "vldnc_vssl\0"
  "vldnc_vssvl\0"
  "vldu2d_vssl\0"
  "vldu2d_vssvl\0"
  "vldu2dnc_vssl\0"
  "vldu2dnc_vssvl\0"
  "vldu_vssl\0"
  "vldu_vssvl\0"
  "vldunc_vssl\0"
  "vldunc_vssvl\0"
  "vldz_vvl\0"
  "vldz_vvmvl\0"
  "vldz_vvvl\0"
  "vmaxsl_vsvl\0"
  "vmaxsl_vsvmvl\0"
  "vmaxsl_vsvvl\0"
  "vmaxsl_vvvl\0"
  "vmaxsl_vvvmvl\0"
  "vmaxsl_vvvvl\0"
  "vmaxswsx_vsvl\0"
  "vmaxswsx_vsvmvl\0"
  "vmaxswsx_vsvvl\0"
  "vmaxswsx_vvvl\0"
  "vmaxswsx_vvvmvl\0"
  "vmaxswsx_vvvvl\0"
  "vmaxswzx_vsvl\0"
  "vmaxswzx_vsvmvl\0"
  "vmaxswzx_vsvvl\0"
  "vmaxswzx_vvvl\0"
  "vmaxswzx_vvvmvl\0"
  "vmaxswzx_vvvvl\0"
  "vminsl_vsvl\0"
  "vminsl_vsvmvl\0"
  "vminsl_vsvvl\0"
  "vminsl_vvvl\0"
  "vminsl_vvvmvl\0"
  "vminsl_vvvvl\0"
  "vminswsx_vsvl\0"
  "vminswsx_vsvmvl\0"
  "vminswsx_vsvvl\0"
  "vminswsx_vvvl\0"
  "vminswsx_vvvmvl\0"
  "vminswsx_vvvvl\0"
  "vminswzx_vsvl\0"
  "vminswzx_vsvmvl\0"
  "vminswzx_vsvvl\0"
  "vminswzx_vvvl\0"
  "vminswzx_vvvmvl\0"
  "vminswzx_vvvvl\0"
  "vmrg_vsvml\0"
  "vmrg_vsvmvl\0"
  "vmrg_vvvml\0"
  "vmrg_vvvmvl\0"
  "vmrgw_vsvMl\0"
  "vmrgw_vsvMvl\0"
  "vmrgw_vvvMl\0"
  "vmrgw_vvvMvl\0"
  "vmulsl_vsvl\0"
  "vmulsl_vsvmvl\0"
  "vmulsl_vsvvl\0"
  "vmulsl_vvvl\0"
  "vmulsl_vvvmvl\0"
  "vmulsl_vvvvl\0"
  "vmulslw_vsvl\0"
  "vmulslw_vsvvl\0"
  "vmulslw_vvvl\0"
  "vmulslw_vvvvl\0"
  "vmulswsx_vsvl\0"
  "vmulswsx_vsvmvl\0"
  "vmulswsx_vsvvl\0"
  "vmulswsx_vvvl\0"
  "vmulswsx_vvvmvl\0"
  "vmulswsx_vvvvl\0"
  "vmulswzx_vsvl\0"
  "vmulswzx_vsvmvl\0"
  "vmulswzx_vsvvl\0"
  "vmulswzx_vvvl\0"
  "vmulswzx_vvvmvl\0"
  "vmulswzx_vvvvl\0"
  "vmulul_vsvl\0"
  "vmulul_vsvmvl\0"
  "vmulul_vsvvl\0"
  "vmulul_vvvl\0"
  "vmulul_vvvmvl\0"
  "vmulul_vvvvl\0"
  "vmuluw_vsvl\0"
  "vmuluw_vsvmvl\0"
  "vmuluw_vsvvl\0"
  "vmuluw_vvvl\0"
  "vmuluw_vvvmvl\0"
  "vmuluw_vvvvl\0"
  "vmv_vsvl\0"
  "vmv_vsvmvl\0"
  "vmv_vsvvl\0"
  "vor_vsvl\0"
  "vor_vsvmvl\0"
  "vor_vsvvl\0"
  "vor_vvvl\0"
  "vor_vvvmvl\0"
  "vor_vvvvl\0"
  "vpcnt_vvl\0"
  "vpcnt_vvmvl\0"
  "vpcnt_vvvl\0"
  "vrand_vvl\0"
  "vrand_vvml\0"
  "vrcpd_vvl\0"
  "vrcpd_vvvl\0"
  "vrcps_vvl\0"
  "vrcps_vvvl\0"
  "vrmaxslfst_vvl\0"
  "vrmaxslfst_vvvl\0"
  "vrmaxsllst_vvl\0"
  "vrmaxsllst_vvvl\0"
  "vrmaxswfstsx_vvl\0"
  "vrmaxswfstsx_vvvl\0"
  "vrmaxswfstzx_vvl\0"
  "vrmaxswfstzx_vvvl\0"
  "vrmaxswlstsx_vvl\0"
  "vrmaxswlstsx_vvvl\0"
  "vrmaxswlstzx_vvl\0"
  "vrmaxswlstzx_vvvl\0"
  "vrminslfst_vvl\0"
  "vrminslfst_vvvl\0"
  "vrminsllst_vvl\0"
  "vrminsllst_vvvl\0"
  "vrminswfstsx_vvl\0"
  "vrminswfstsx_vvvl\0"
  "vrminswfstzx_vvl\0"
  "vrminswfstzx_vvvl\0"
  "vrminswlstsx_vvl\0"
  "vrminswlstsx_vvvl\0"
  "vrminswlstzx_vvl\0"
  "vrminswlstzx_vvvl\0"
  "vror_vvl\0"
  "vror_vvml\0"
  "vrsqrtd_vvl\0"
  "vrsqrtd_vvvl\0"
  "vrsqrtdnex_vvl\0"
  "vrsqrtdnex_vvvl\0"
  "vrsqrts_vvl\0"
  "vrsqrts_vvvl\0"
  "vrsqrtsnex_vvl\0"
  "vrsqrtsnex_vvvl\0"
  "vrxor_vvl\0"
  "vrxor_vvml\0"
  "vsc_vvssl\0"
  "vsc_vvssml\0"
  "vscl_vvssl\0"
  "vscl_vvssml\0"
  "vsclnc_vvssl\0"
  "vsclnc_vvssml\0"
  "vsclncot_vvssl\0"
  "vsclncot_vvssml\0"
  "vsclot_vvssl\0"
  "vsclot_vvssml\0"
  "vscnc_vvssl\0"
  "vscnc_vvssml\0"
  "vscncot_vvssl\0"
  "vscncot_vvssml\0"
  "vscot_vvssl\0"
  "vscot_vvssml\0"
  "vscu_vvssl\0"
  "vscu_vvssml\0"
  "vscunc_vvssl\0"
  "vscunc_vvssml\0"
  "vscuncot_vvssl\0"
  "vscuncot_vvssml\0"
  "vscuot_vvssl\0"
  "vscuot_vvssml\0"
  "vseq_vl\0"
  "vseq_vvl\0"
  "vsfa_vvssl\0"
  "vsfa_vvssmvl\0"
  "vsfa_vvssvl\0"
  "vshf_vvvsl\0"
  "vshf_vvvsvl\0"
  "vslal_vvsl\0"
  "vslal_vvsmvl\0"
  "vslal_vvsvl\0"
  "vslal_vvvl\0"
  "vslal_vvvmvl\0"
  "vslal_vvvvl\0"
  "vslawsx_vvsl\0"
  "vslawsx_vvsmvl\0"
  "vslawsx_vvsvl\0"
  "vslawsx_vvvl\0"
  "vslawsx_vvvmvl\0"
  "vslawsx_vvvvl\0"
  "vslawzx_vvsl\0"
  "vslawzx_vvsmvl\0"
  "vslawzx_vvsvl\0"
  "vslawzx_vvvl\0"
  "vslawzx_vvvmvl\0"
  "vslawzx_vvvvl\0"
  "vsll_vvsl\0"
  "vsll_vvsmvl\0"
  "vsll_vvsvl\0"
  "vsll_vvvl\0"
  "vsll_vvvmvl\0"
  "vsll_vvvvl\0"
  "vsral_vvsl\0"
  "vsral_vvsmvl\0"
  "vsral_vvsvl\0"
  "vsral_vvvl\0"
  "vsral_vvvmvl\0"
  "vsral_vvvvl\0"
  "vsrawsx_vvsl\0"
  "vsrawsx_vvsmvl\0"
  "vsrawsx_vvsvl\0"
  "vsrawsx_vvvl\0"
  "vsrawsx_vvvmvl\0"
  "vsrawsx_vvvvl\0"
  "vsrawzx_vvsl\0"
  "vsrawzx_vvsmvl\0"
  "vsrawzx_vvsvl\0"
  "vsrawzx_vvvl\0"
  "vsrawzx_vvvmvl\0"
  "vsrawzx_vvvvl\0"
  "vsrl_vvsl\0"
  "vsrl_vvsmvl\0"
  "vsrl_vvsvl\0"
  "vsrl_vvvl\0"
  "vsrl_vvvmvl\0"
  "vsrl_vvvvl\0"
  "vst2d_vssl\0"
  "vst2d_vssml\0"
  "vst2dnc_vssl\0"
  "vst2dnc_vssml\0"
  "vst2dncot_vssl\0"
  "vst2dncot_vssml\0"
  "vst2dot_vssl\0"
  "vst2dot_vssml\0"
  "vst_vssl\0"
  "vst_vssml\0"
  "vstl2d_vssl\0"
  "vstl2d_vssml\0"
  "vstl2dnc_vssl\0"
  "vstl2dnc_vssml\0"
  "vstl2dncot_vssl\0"
  "vstl2dncot_vssml\0"
  "vstl2dot_vssl\0"
  "vstl2dot_vssml\0"
  "vstl_vssl\0"
  "vstl_vssml\0"
  "vstlnc_vssl\0"
  "vstlnc_vssml\0"
  "vstlncot_vssl\0"
  "vstlncot_vssml\0"
  "vstlot_vssl\0"
  "vstlot_vssml\0"
  "vstnc_vssl\0"
  "vstnc_vssml\0"
  "vstncot_vssl\0"
  "vstncot_vssml\0"
  "vstot_vssl\0"
  "vstot_vssml\0"
  "vstu2d_vssl\0"
  "vstu2d_vssml\0"
  "vstu2dnc_vssl\0"
  "vstu2dnc_vssml\0"
  "vstu2dncot_vssl\0"
  "vstu2dncot_vssml\0"
  "vstu2dot_vssl\0"
  "vstu2dot_vssml\0"
  "vstu_vssl\0"
  "vstu_vssml\0"
  "vstunc_vssl\0"
  "vstunc_vssml\0"
  "vstuncot_vssl\0"
  "vstuncot_vssml\0"
  "vstuot_vssl\0"
  "vstuot_vssml\0"
  "vsubsl_vsvl\0"
  "vsubsl_vsvmvl\0"
  "vsubsl_vsvvl\0"
  "vsubsl_vvvl\0"
  "vsubsl_vvvmvl\0"
  "vsubsl_vvvvl\0"
  "vsubswsx_vsvl\0"
  "vsubswsx_vsvmvl\0"
  "vsubswsx_vsvvl\0"
  "vsubswsx_vvvl\0"
  "vsubswsx_vvvmvl\0"
  "vsubswsx_vvvvl\0"
  "vsubswzx_vsvl\0"
  "vsubswzx_vsvmvl\0"
  "vsubswzx_vsvvl\0"
  "vsubswzx_vvvl\0"
  "vsubswzx_vvvmvl\0"
  "vsubswzx_vvvvl\0"
  "vsubul_vsvl\0"
  "vsubul_vsvmvl\0"
  "vsubul_vsvvl\0"
  "vsubul_vvvl\0"
  "vsubul_vvvmvl\0"
  "vsubul_vvvvl\0"
  "vsubuw_vsvl\0"
  "vsubuw_vsvmvl\0"
  "vsubuw_vsvvl\0"
  "vsubuw_vvvl\0"
  "vsubuw_vvvmvl\0"
  "vsubuw_vvvvl\0"
  "vsuml_vvl\0"
  "vsuml_vvml\0"
  "vsumwsx_vvl\0"
  "vsumwsx_vvml\0"
  "vsumwzx_vvl\0"
  "vsumwzx_vvml\0"
  "vxor_vsvl\0"
  "vxor_vsvmvl\0"
  "vxor_vsvvl\0"
  "vxor_vvvl\0"
  "vxor_vvvmvl\0"
  "vxor_vvvvl\0"
  "xorm_MMM\0"
  "xorm_mmm\0"
  "aadd32\0"
  "aadd64\0"
  "aand32\0"
  "aand64\0"
  "addpd512\0"
  "addph512\0"
  "addps512\0"
  "addsd_round_mask\0"
  "addsh_round_mask\0"
  "addss_round_mask\0"
  "addsubpd\0"
  "addsubpd256\0"
  "addsubps\0"
  "addsubps256\0"
  "aesdec128\0"
  "aesdec256\0"
  "aesdec512\0"
  "aesdeclast128\0"
  "aesdeclast256\0"
  "aesdeclast512\0"
  "aesenc128\0"
  "aesenc256\0"
  "aesenc512\0"
  "aesenclast128\0"
  "aesenclast256\0"
  "aesenclast512\0"
  "aesimc128\0"
  "aeskeygenassist128\0"
  "aor32\0"
  "aor64\0"
  "axor32\0"
  "axor64\0"
  "bextr_u32\0"
  "bextr_u64\0"
  "bextri_u32\0"
  "bextri_u64\0"
  "blendvpd\0"
  "blendvpd256\0"
  "blendvps\0"
  "blendvps256\0"
  "broadcastmb128\0"
  "broadcastmb256\0"
  "broadcastmb512\0"
  "broadcastmw128\0"
  "broadcastmw256\0"
  "broadcastmw512\0"
  "bzhi_di\0"
  "bzhi_si\0"
  "cldemote\0"
  "clflush\0"
  "clflushopt\0"
  "clrssbsy\0"
  "clui\0"
  "clwb\0"
  "clzero\0"
  "cmpccxadd32\0"
  "cmpccxadd64\0"
  "cmpsd\0"
  "cmpsd_mask\0"
  "cmpsh_mask\0"
  "cmpss\0"
  "cmpss_mask\0"
  "comieq\0"
  "comige\0"
  "comigt\0"
  "comile\0"
  "comilt\0"
  "comineq\0"
  "comisdeq\0"
  "comisdge\0"
  "comisdgt\0"
  "comisdle\0"
  "comisdlt\0"
  "comisdneq\0"
  "crc32di\0"
  "crc32hi\0"
  "crc32qi\0"
  "crc32si\0"
  "cvtne2ps2bf16_128\0"
  "cvtne2ps2bf16_256\0"
  "cvtne2ps2bf16_512\0"
  "cvtneps2bf16_256\0"
  "cvtneps2bf16_512\0"
  "cvtpd2dq\0"
  "cvtpd2dq128_mask\0"
  "cvtpd2dq256\0"
  "cvtpd2dq512_mask\0"
  "cvtpd2ps\0"
  "cvtpd2ps256\0"
  "cvtpd2ps512_mask\0"
  "cvtpd2ps_mask\0"
  "cvtpd2qq128_mask\0"
  "cvtpd2qq256_mask\0"
  "cvtpd2qq512_mask\0"
  "cvtpd2udq128_mask\0"
  "cvtpd2udq256_mask\0"
  "cvtpd2udq512_mask\0"
  "cvtpd2uqq128_mask\0"
  "cvtpd2uqq256_mask\0"
  "cvtpd2uqq512_mask\0"
  "cvtps2dq\0"
  "cvtps2dq128_mask\0"
  "cvtps2dq256\0"
  "cvtps2dq256_mask\0"
  "cvtps2dq512_mask\0"
  "cvtps2pd512_mask\0"
  "cvtps2qq128_mask\0"
  "cvtps2qq256_mask\0"
  "cvtps2qq512_mask\0"
  "cvtps2udq128_mask\0"
  "cvtps2udq256_mask\0"
  "cvtps2udq512_mask\0"
  "cvtps2uqq128_mask\0"
  "cvtps2uqq256_mask\0"
  "cvtps2uqq512_mask\0"
  "cvtqq2ps128_mask\0"
  "cvtsd2si\0"
  "cvtsd2si64\0"
  "cvtsd2ss\0"
  "cvtsd2ss_round_mask\0"
  "cvtsi2sd64\0"
  "cvtsi2ss32\0"
  "cvtsi2ss64\0"
  "cvtss2sd_round_mask\0"
  "cvtss2si\0"
  "cvtss2si64\0"
  "cvttpd2dq\0"
  "cvttpd2dq128_mask\0"
  "cvttpd2dq256\0"
  "cvttpd2dq512_mask\0"
  "cvttpd2qq128_mask\0"
  "cvttpd2qq256_mask\0"
  "cvttpd2qq512_mask\0"
  "cvttpd2udq128_mask\0"
  "cvttpd2udq256_mask\0"
  "cvttpd2udq512_mask\0"
  "cvttpd2uqq128_mask\0"
  "cvttpd2uqq256_mask\0"
  "cvttpd2uqq512_mask\0"
  "cvttps2dq\0"
  "cvttps2dq256\0"
  "cvttps2dq512_mask\0"
  "cvttps2qq128_mask\0"
  "cvttps2qq256_mask\0"
  "cvttps2qq512_mask\0"
  "cvttps2udq128_mask\0"
  "cvttps2udq256_mask\0"
  "cvttps2udq512_mask\0"
  "cvttps2uqq128_mask\0"
  "cvttps2uqq256_mask\0"
  "cvttps2uqq512_mask\0"
  "cvttsd2si\0"
  "cvttsd2si64\0"
  "cvttss2si\0"
  "cvttss2si64\0"
  "cvtuqq2ps128_mask\0"
  "cvtusi2sd64\0"
  "cvtusi2ss32\0"
  "cvtusi2ss64\0"
  "dbpsadbw128\0"
  "dbpsadbw256\0"
  "dbpsadbw512\0"
  "directstore_u32\0"
  "directstore_u64\0"
  "divpd512\0"
  "divph512\0"
  "divps512\0"
  "divsd_round_mask\0"
  "divsh_round_mask\0"
  "divss_round_mask\0"
  "dpbf16ps_128\0"
  "dpbf16ps_256\0"
  "dpbf16ps_512\0"
  "dppd\0"
  "dpps\0"
  "dpps256\0"
  "emms\0"
  "enqcmd\0"
  "enqcmds\0"
  "extrq\0"
  "extrqi\0"
  "fixupimmpd128_mask\0"
  "fixupimmpd128_maskz\0"
  "fixupimmpd256_mask\0"
  "fixupimmpd256_maskz\0"
  "fixupimmpd512_mask\0"
  "fixupimmpd512_maskz\0"
  "fixupimmps128_mask\0"
  "fixupimmps128_maskz\0"
  "fixupimmps256_mask\0"
  "fixupimmps256_maskz\0"
  "fixupimmps512_mask\0"
  "fixupimmps512_maskz\0"
  "fixupimmsd_mask\0"
  "fixupimmsd_maskz\0"
  "fixupimmss_mask\0"
  "fixupimmss_maskz\0"
  "fpclasssd_mask\0"
  "fpclasssh_mask\0"
  "fpclassss_mask\0"
  "fxrstor\0"
  "fxrstor64\0"
  "fxsave\0"
  "fxsave64\0"
  "gatherd_d\0"
  "gatherd_d256\0"
  "gatherd_pd\0"
  "gatherd_pd256\0"
  "gatherd_ps\0"
  "gatherd_ps256\0"
  "gatherd_q\0"
  "gatherd_q256\0"
  "gatherq_d\0"
  "gatherq_d256\0"
  "gatherq_pd\0"
  "gatherq_pd256\0"
  "gatherq_ps\0"
  "gatherq_ps256\0"
  "gatherq_q\0"
  "gatherq_q256\0"
  "getexppd128_mask\0"
  "getexppd256_mask\0"
  "getexppd512_mask\0"
  "getexpph128_mask\0"
  "getexpph256_mask\0"
  "getexpph512_mask\0"
  "getexpps128_mask\0"
  "getexpps256_mask\0"
  "getexpps512_mask\0"
  "getexpsd128_round_mask\0"
  "getexpsh128_round_mask\0"
  "getexpss128_round_mask\0"
  "getmantpd128_mask\0"
  "getmantpd256_mask\0"
  "getmantpd512_mask\0"
  "getmantph128_mask\0"
  "getmantph256_mask\0"
  "getmantph512_mask\0"
  "getmantps128_mask\0"
  "getmantps256_mask\0"
  "getmantps512_mask\0"
  "getmantsd_round_mask\0"
  "getmantsh_round_mask\0"
  "getmantss_round_mask\0"
  "haddpd\0"
  "haddpd256\0"
  "haddps\0"
  "haddps256\0"
  "hsubpd\0"
  "hsubpd256\0"
  "hsubps\0"
  "hsubps256\0"
  "incsspd\0"
  "incsspq\0"
  "insertps128\0"
  "insertq\0"
  "insertqi\0"
  "invpcid\0"
  "lddqu\0"
  "lddqu256\0"
  "lfence\0"
  "llwpcb\0"
  "loadiwkey\0"
  "lwpins32\0"
  "lwpins64\0"
  "lwpval32\0"
  "lwpval64\0"
  "maskloadd\0"
  "maskloadd256\0"
  "maskloadpd\0"
  "maskloadpd256\0"
  "maskloadps\0"
  "maskloadps256\0"
  "maskloadq\0"
  "maskloadq256\0"
  "maskmovdqu\0"
  "maskstored\0"
  "maskstored256\0"
  "maskstorepd\0"
  "maskstorepd256\0"
  "maskstoreps\0"
  "maskstoreps256\0"
  "maskstoreq\0"
  "maskstoreq256\0"
  "maxpd\0"
  "maxpd256\0"
  "maxpd512\0"
  "maxph128\0"
  "maxph256\0"
  "maxph512\0"
  "maxps\0"
  "maxps256\0"
  "maxps512\0"
  "maxsd\0"
  "maxsd_round_mask\0"
  "maxsh_round_mask\0"
  "maxss\0"
  "maxss_round_mask\0"
  "mfence\0"
  "minpd\0"
  "minpd256\0"
  "minpd512\0"
  "minph128\0"
  "minph256\0"
  "minph512\0"
  "minps\0"
  "minps256\0"
  "minps512\0"
  "minsd\0"
  "minsd_round_mask\0"
  "minsh_round_mask\0"
  "minss\0"
  "minss_round_mask\0"
  "monitor\0"
  "monitorx\0"
  "movdir64b\0"
  "movmskpd\0"
  "movmskpd256\0"
  "movmskps\0"
  "movmskps256\0"
  "movrsdi\0"
  "movrshi\0"
  "movrsqi\0"
  "movrssi\0"
  "mpsadbw128\0"
  "mpsadbw256\0"
  "mpsadbw512\0"
  "mulpd512\0"
  "mulph512\0"
  "mulps512\0"
  "mulsd_round_mask\0"
  "mulsh_round_mask\0"
  "mulss_round_mask\0"
  "mwait\0"
  "mwaitx\0"
  "packssdw128\0"
  "packssdw256\0"
  "packssdw512\0"
  "packsswb128\0"
  "packsswb256\0"
  "packsswb512\0"
  "packusdw128\0"
  "packusdw256\0"
  "packusdw512\0"
  "packuswb128\0"
  "packuswb256\0"
  "packuswb512\0"
  "pause\0"
  "pavgb128\0"
  "pavgb256\0"
  "pavgb512\0"
  "pavgw128\0"
  "pavgw256\0"
  "pavgw512\0"
  "pblendvb128\0"
  "pblendvb256\0"
  "pclmulqdq128\0"
  "pclmulqdq256\0"
  "pclmulqdq512\0"
  "pcmpestri128\0"
  "pcmpestria128\0"
  "pcmpestric128\0"
  "pcmpestrio128\0"
  "pcmpestris128\0"
  "pcmpestriz128\0"
  "pcmpestrm128\0"
  "pcmpistri128\0"
  "pcmpistria128\0"
  "pcmpistric128\0"
  "pcmpistrio128\0"
  "pcmpistris128\0"
  "pcmpistriz128\0"
  "pcmpistrm128\0"
  "pdep_di\0"
  "pdep_si\0"
  "permvardf256\0"
  "permvardf512\0"
  "permvardi256\0"
  "permvardi512\0"
  "permvarhi128\0"
  "permvarhi256\0"
  "permvarhi512\0"
  "permvarqi128\0"
  "permvarqi256\0"
  "permvarqi512\0"
  "permvarsf256\0"
  "permvarsf512\0"
  "permvarsi256\0"
  "permvarsi512\0"
  "pext_di\0"
  "pext_si\0"
  "phaddd128\0"
  "phaddd256\0"
  "phaddsw128\0"
  "phaddsw256\0"
  "phaddw128\0"
  "phaddw256\0"
  "phminposuw128\0"
  "phsubd128\0"
  "phsubd256\0"
  "phsubsw128\0"
  "phsubsw256\0"
  "phsubw128\0"
  "phsubw256\0"
  "pmaddubsw128\0"
  "pmaddubsw256\0"
  "pmaddubsw512\0"
  "pmaddwd128\0"
  "pmaddwd256\0"
  "pmaddwd512\0"
  "pmovdb128_mask\0"
  "pmovdb128mem_mask\0"
  "pmovdb256_mask\0"
  "pmovdb256mem_mask\0"
  "pmovdb512mem_mask\0"
  "pmovdw128_mask\0"
  "pmovdw128mem_mask\0"
  "pmovdw256_mask\0"
  "pmovdw256mem_mask\0"
  "pmovdw512mem_mask\0"
  "pmovmskb128\0"
  "pmovmskb256\0"
  "pmovqb128_mask\0"
  "pmovqb128mem_mask\0"
  "pmovqb256_mask\0"
  "pmovqb256mem_mask\0"
  "pmovqb512_mask\0"
  "pmovqb512mem_mask\0"
  "pmovqd128_mask\0"
  "pmovqd128mem_mask\0"
  "pmovqd256mem_mask\0"
  "pmovqd512mem_mask\0"
  "pmovqw128_mask\0"
  "pmovqw128mem_mask\0"
  "pmovqw256_mask\0"
  "pmovqw256mem_mask\0"
  "pmovqw512mem_mask\0"
  "pmovsdb128_mask\0"
  "pmovsdb128mem_mask\0"
  "pmovsdb256_mask\0"
  "pmovsdb256mem_mask\0"
  "pmovsdb512_mask\0"
  "pmovsdb512mem_mask\0"
  "pmovsdw128_mask\0"
  "pmovsdw128mem_mask\0"
  "pmovsdw256_mask\0"
  "pmovsdw256mem_mask\0"
  "pmovsdw512_mask\0"
  "pmovsdw512mem_mask\0"
  "pmovsqb128_mask\0"
  "pmovsqb128mem_mask\0"
  "pmovsqb256_mask\0"
  "pmovsqb256mem_mask\0"
  "pmovsqb512_mask\0"
  "pmovsqb512mem_mask\0"
  "pmovsqd128_mask\0"
  "pmovsqd128mem_mask\0"
  "pmovsqd256_mask\0"
  "pmovsqd256mem_mask\0"
  "pmovsqd512_mask\0"
  "pmovsqd512mem_mask\0"
  "pmovsqw128_mask\0"
  "pmovsqw128mem_mask\0"
  "pmovsqw256_mask\0"
  "pmovsqw256mem_mask\0"
  "pmovsqw512_mask\0"
  "pmovsqw512mem_mask\0"
  "pmovswb128_mask\0"
  "pmovswb128mem_mask\0"
  "pmovswb256_mask\0"
  "pmovswb256mem_mask\0"
  "pmovswb512_mask\0"
  "pmovswb512mem_mask\0"
  "pmovusdb128_mask\0"
  "pmovusdb128mem_mask\0"
  "pmovusdb256_mask\0"
  "pmovusdb256mem_mask\0"
  "pmovusdb512_mask\0"
  "pmovusdb512mem_mask\0"
  "pmovusdw128_mask\0"
  "pmovusdw128mem_mask\0"
  "pmovusdw256_mask\0"
  "pmovusdw256mem_mask\0"
  "pmovusdw512_mask\0"
  "pmovusdw512mem_mask\0"
  "pmovusqb128_mask\0"
  "pmovusqb128mem_mask\0"
  "pmovusqb256_mask\0"
  "pmovusqb256mem_mask\0"
  "pmovusqb512_mask\0"
  "pmovusqb512mem_mask\0"
  "pmovusqd128_mask\0"
  "pmovusqd128mem_mask\0"
  "pmovusqd256_mask\0"
  "pmovusqd256mem_mask\0"
  "pmovusqd512_mask\0"
  "pmovusqd512mem_mask\0"
  "pmovusqw128_mask\0"
  "pmovusqw128mem_mask\0"
  "pmovusqw256_mask\0"
  "pmovusqw256mem_mask\0"
  "pmovusqw512_mask\0"
  "pmovusqw512mem_mask\0"
  "pmovuswb128_mask\0"
  "pmovuswb128mem_mask\0"
  "pmovuswb256_mask\0"
  "pmovuswb256mem_mask\0"
  "pmovuswb512_mask\0"
  "pmovuswb512mem_mask\0"
  "pmovwb128_mask\0"
  "pmovwb128mem_mask\0"
  "pmovwb256mem_mask\0"
  "pmovwb512mem_mask\0"
  "pmulhrsw128\0"
  "pmulhrsw256\0"
  "pmulhrsw512\0"
  "pmulhuw128\0"
  "pmulhuw256\0"
  "pmulhuw512\0"
  "pmulhw128\0"
  "pmulhw256\0"
  "pmulhw512\0"
  "prefetchrs\0"
  "psadbw128\0"
  "psadbw256\0"
  "psadbw512\0"
  "pshufb128\0"
  "pshufb256\0"
  "pshufb512\0"
  "psignb128\0"
  "psignb256\0"
  "psignd128\0"
  "psignd256\0"
  "psignw128\0"
  "psignw256\0"
  "pslld128\0"
  "pslld256\0"
  "pslld512\0"
  "pslldi128\0"
  "pslldi256\0"
  "pslldi512\0"
  "psllq128\0"
  "psllq256\0"
  "psllq512\0"
  "psllqi128\0"
  "psllqi256\0"
  "psllqi512\0"
  "psllv16hi\0"
  "psllv16si\0"
  "psllv2di\0"
  "psllv32hi\0"
  "psllv4di\0"
  "psllv4si\0"
  "psllv8di\0"
  "psllv8hi\0"
  "psllv8si\0"
  "psllw128\0"
  "psllw256\0"
  "psllw512\0"
  "psllwi128\0"
  "psllwi256\0"
  "psllwi512\0"
  "psrad128\0"
  "psrad256\0"
  "psrad512\0"
  "psradi128\0"
  "psradi256\0"
  "psradi512\0"
  "psraq128\0"
  "psraq256\0"
  "psraq512\0"
  "psraqi128\0"
  "psraqi256\0"
  "psraqi512\0"
  "psrav16hi\0"
  "psrav16si\0"
  "psrav32hi\0"
  "psrav4si\0"
  "psrav8di\0"
  "psrav8hi\0"
  "psrav8si\0"
  "psravq128\0"
  "psravq256\0"
  "psraw128\0"
  "psraw256\0"
  "psraw512\0"
  "psrawi128\0"
  "psrawi256\0"
  "psrawi512\0"
  "psrld128\0"
  "psrld256\0"
  "psrld512\0"
  "psrldi128\0"
  "psrldi256\0"
  "psrldi512\0"
  "psrlq128\0"
  "psrlq256\0"
  "psrlq512\0"
  "psrlqi128\0"
  "psrlqi256\0"
  "psrlqi512\0"
  "psrlv16hi\0"
  "psrlv16si\0"
  "psrlv2di\0"
  "psrlv32hi\0"
  "psrlv4di\0"
  "psrlv4si\0"
  "psrlv8di\0"
  "psrlv8hi\0"
  "psrlv8si\0"
  "psrlw128\0"
  "psrlw256\0"
  "psrlw512\0"
  "psrlwi128\0"
  "psrlwi256\0"
  "psrlwi512\0"
  "pternlogd128\0"
  "pternlogd256\0"
  "pternlogd512\0"
  "pternlogq128\0"
  "pternlogq256\0"
  "pternlogq512\0"
  "ptestc128\0"
  "ptestc256\0"
  "ptestnzc128\0"
  "ptestnzc256\0"
  "ptestz128\0"
  "ptestz256\0"
  "ptwrite32\0"
  "ptwrite64\0"
  "rangepd128_mask\0"
  "rangepd256_mask\0"
  "rangepd512_mask\0"
  "rangeps128_mask\0"
  "rangeps256_mask\0"
  "rangeps512_mask\0"
  "rangesd128_round_mask\0"
  "rangess128_round_mask\0"
  "rcp14pd128_mask\0"
  "rcp14pd256_mask\0"
  "rcp14pd512_mask\0"
  "rcp14ps128_mask\0"
  "rcp14ps256_mask\0"
  "rcp14ps512_mask\0"
  "rcp14sd_mask\0"
  "rcp14ss_mask\0"
  "rcpph128_mask\0"
  "rcpph256_mask\0"
  "rcpph512_mask\0"
  "rcpps\0"
  "rcpps256\0"
  "rcpsh_mask\0"
  "rcpss\0"
  "rdfsbase32\0"
  "rdfsbase64\0"
  "rdgsbase32\0"
  "rdgsbase64\0"
  "rdpid\0"
  "rdpkru\0"
  "rdpmc\0"
  "rdpru\0"
  "rdsspd\0"
  "rdsspq\0"
  "rdtsc\0"
  "readeflags_u32\0"
  "readeflags_u64\0"
  "reducepd128_mask\0"
  "reducepd256_mask\0"
  "reducepd512_mask\0"
  "reduceph128_mask\0"
  "reduceph256_mask\0"
  "reduceph512_mask\0"
  "reduceps128_mask\0"
  "reduceps256_mask\0"
  "reduceps512_mask\0"
  "reducesd_mask\0"
  "reducesh_mask\0"
  "reducess_mask\0"
  "rndscalepd_128_mask\0"
  "rndscalepd_256_mask\0"
  "rndscalepd_mask\0"
  "rndscaleph_128_mask\0"
  "rndscaleph_256_mask\0"
  "rndscaleph_mask\0"
  "rndscaleps_128_mask\0"
  "rndscaleps_256_mask\0"
  "rndscaleps_mask\0"
  "rndscalesd_round_mask\0"
  "rndscalesh_round_mask\0"
  "rndscaless_round_mask\0"
  "rsqrt14pd128_mask\0"
  "rsqrt14pd256_mask\0"
  "rsqrt14pd512_mask\0"
  "rsqrt14ps128_mask\0"
  "rsqrt14ps256_mask\0"
  "rsqrt14ps512_mask\0"
  "rsqrt14sd_mask\0"
  "rsqrt14ss_mask\0"
  "rsqrtph128_mask\0"
  "rsqrtph256_mask\0"
  "rsqrtph512_mask\0"
  "rsqrtps\0"
  "rsqrtps256\0"
  "rsqrtsh_mask\0"
  "rsqrtss\0"
  "rstorssp\0"
  "saveprevssp\0"
  "scalefpd128_mask\0"
  "scalefpd256_mask\0"
  "scalefpd512_mask\0"
  "scalefph128_mask\0"
  "scalefph256_mask\0"
  "scalefph512_mask\0"
  "scalefps128_mask\0"
  "scalefps256_mask\0"
  "scalefps512_mask\0"
  "scalefsd_round_mask\0"
  "scalefsh_round_mask\0"
  "scalefss_round_mask\0"
  "senduipi\0"
  "serialize\0"
  "setssbsy\0"
  "sfence\0"
  "sha1msg1\0"
  "sha1msg2\0"
  "sha1nexte\0"
  "sha1rnds4\0"
  "sha256msg1\0"
  "sha256msg2\0"
  "sha256rnds2\0"
  "slwpcb\0"
  "stui\0"
  "subpd512\0"
  "subph512\0"
  "subps512\0"
  "subsd_round_mask\0"
  "subsh_round_mask\0"
  "subss_round_mask\0"
  "tcmmimfp16ps\0"
  "tcmmimfp16ps_internal\0"
  "tcmmrlfp16ps\0"
  "tcmmrlfp16ps_internal\0"
  "tcvtrowd2ps\0"
  "tcvtrowd2ps_internal\0"
  "tcvtrowd2psi\0"
  "tcvtrowps2bf16h\0"
  "tcvtrowps2bf16h_internal\0"
  "tcvtrowps2bf16hi\0"
  "tcvtrowps2bf16l\0"
  "tcvtrowps2bf16l_internal\0"
  "tcvtrowps2bf16li\0"
  "tcvtrowps2phh\0"
  "tcvtrowps2phh_internal\0"
  "tcvtrowps2phhi\0"
  "tcvtrowps2phl\0"
  "tcvtrowps2phl_internal\0"
  "tcvtrowps2phli\0"
  "tdpbf16ps\0"
  "tdpbf16ps_internal\0"
  "tdpbf8ps\0"
  "tdpbf8ps_internal\0"
  "tdpbhf8ps\0"
  "tdpbhf8ps_internal\0"
  "tdpbssd\0"
  "tdpbssd_internal\0"
  "tdpbsud\0"
  "tdpbsud_internal\0"
  "tdpbusd\0"
  "tdpbusd_internal\0"
  "tdpbuud\0"
  "tdpbuud_internal\0"
  "tdpfp16ps\0"
  "tdpfp16ps_internal\0"
  "tdphbf8ps\0"
  "tdphbf8ps_internal\0"
  "tdphf8ps\0"
  "tdphf8ps_internal\0"
  "testui\0"
  "tile_loadconfig\0"
  "tile_loadconfig_internal\0"
  "tile_storeconfig\0"
  "tileloadd64\0"
  "tileloadd64_internal\0"
  "tileloaddrs64\0"
  "tileloaddrs64_internal\0"
  "tileloaddrst164\0"
  "tileloaddrst164_internal\0"
  "tileloaddt164\0"
  "tileloaddt164_internal\0"
  "tilemovrow\0"
  "tilemovrow_internal\0"
  "tilemovrowi\0"
  "tilerelease\0"
  "tilestored64\0"
  "tilestored64_internal\0"
  "tilezero\0"
  "tilezero_internal\0"
  "tmmultf32ps\0"
  "tmmultf32ps_internal\0"
  "tpause\0"
  "ucomieq\0"
  "ucomige\0"
  "ucomigt\0"
  "ucomile\0"
  "ucomilt\0"
  "ucomineq\0"
  "ucomisdeq\0"
  "ucomisdge\0"
  "ucomisdgt\0"
  "ucomisdle\0"
  "ucomisdlt\0"
  "ucomisdneq\0"
  "umonitor\0"
  "umwait\0"
  "urdmsr\0"
  "uwrmsr\0"
  "vaddbf16128\0"
  "vaddbf16256\0"
  "vaddbf16512\0"
  "vaddpd256_round\0"
  "vaddph256_round\0"
  "vaddps256_round\0"
  "vbcstnebf162ps128\0"
  "vbcstnebf162ps256\0"
  "vbcstnesh2ps128\0"
  "vbcstnesh2ps256\0"
  "vcomisbf16eq\0"
  "vcomisbf16ge\0"
  "vcomisbf16gt\0"
  "vcomisbf16le\0"
  "vcomisbf16lt\0"
  "vcomisbf16neq\0"
  "vcomisd\0"
  "vcomish\0"
  "vcomiss\0"
  "vcvt2ph2bf8_128\0"
  "vcvt2ph2bf8_256\0"
  "vcvt2ph2bf8_512\0"
  "vcvt2ph2bf8s_128\0"
  "vcvt2ph2bf8s_256\0"
  "vcvt2ph2bf8s_512\0"
  "vcvt2ph2hf8_128\0"
  "vcvt2ph2hf8_256\0"
  "vcvt2ph2hf8_512\0"
  "vcvt2ph2hf8s_128\0"
  "vcvt2ph2hf8s_256\0"
  "vcvt2ph2hf8s_512\0"
  "vcvt2ps2phx128_mask\0"
  "vcvt2ps2phx256_mask\0"
  "vcvt2ps2phx512_mask\0"
  "vcvtbf162ibs128\0"
  "vcvtbf162ibs256\0"
  "vcvtbf162ibs512\0"
  "vcvtbf162iubs128\0"
  "vcvtbf162iubs256\0"
  "vcvtbf162iubs512\0"
  "vcvtbiasph2bf8_128_mask\0"
  "vcvtbiasph2bf8_256_mask\0"
  "vcvtbiasph2bf8_512_mask\0"
  "vcvtbiasph2bf8s_128_mask\0"
  "vcvtbiasph2bf8s_256_mask\0"
  "vcvtbiasph2bf8s_512_mask\0"
  "vcvtbiasph2hf8_128_mask\0"
  "vcvtbiasph2hf8_256_mask\0"
  "vcvtbiasph2hf8_512_mask\0"
  "vcvtbiasph2hf8s_128_mask\0"
  "vcvtbiasph2hf8s_256_mask\0"
  "vcvtbiasph2hf8s_512_mask\0"
  "vcvtdq2ph128_mask\0"
  "vcvthf8_2ph128_mask\0"
  "vcvthf8_2ph256_mask\0"
  "vcvthf8_2ph512_mask\0"
  "vcvtneebf162ps128\0"
  "vcvtneebf162ps256\0"
  "vcvtneeph2ps128\0"
  "vcvtneeph2ps256\0"
  "vcvtneobf162ps128\0"
  "vcvtneobf162ps256\0"
  "vcvtneoph2ps128\0"
  "vcvtneoph2ps256\0"
  "vcvtneps2bf16128\0"
  "vcvtneps2bf16256\0"
  "vcvtpd2ph128_mask\0"
  "vcvtpd2ph256_mask\0"
  "vcvtpd2ph512_mask\0"
  "vcvtph2bf8_128_mask\0"
  "vcvtph2bf8_256_mask\0"
  "vcvtph2bf8_512_mask\0"
  "vcvtph2bf8s_128_mask\0"
  "vcvtph2bf8s_256_mask\0"
  "vcvtph2bf8s_512_mask\0"
  "vcvtph2dq128_mask\0"
  "vcvtph2dq256_mask\0"
  "vcvtph2dq512_mask\0"
  "vcvtph2hf8_128_mask\0"
  "vcvtph2hf8_256_mask\0"
  "vcvtph2hf8_512_mask\0"
  "vcvtph2hf8s_128_mask\0"
  "vcvtph2hf8s_256_mask\0"
  "vcvtph2hf8s_512_mask\0"
  "vcvtph2ibs128_mask\0"
  "vcvtph2ibs256_mask\0"
  "vcvtph2ibs512_mask\0"
  "vcvtph2iubs128_mask\0"
  "vcvtph2iubs256_mask\0"
  "vcvtph2iubs512_mask\0"
  "vcvtph2pd128_mask\0"
  "vcvtph2pd256_mask\0"
  "vcvtph2pd512_mask\0"
  "vcvtph2psx128_mask\0"
  "vcvtph2psx256_mask\0"
  "vcvtph2psx512_mask\0"
  "vcvtph2qq128_mask\0"
  "vcvtph2qq256_mask\0"
  "vcvtph2qq512_mask\0"
  "vcvtph2udq128_mask\0"
  "vcvtph2udq256_mask\0"
  "vcvtph2udq512_mask\0"
  "vcvtph2uqq128_mask\0"
  "vcvtph2uqq256_mask\0"
  "vcvtph2uqq512_mask\0"
  "vcvtph2uw128_mask\0"
  "vcvtph2uw256_mask\0"
  "vcvtph2uw512_mask\0"
  "vcvtph2w128_mask\0"
  "vcvtph2w256_mask\0"
  "vcvtph2w512_mask\0"
  "vcvtps2ibs128_mask\0"
  "vcvtps2ibs256_mask\0"
  "vcvtps2ibs512_mask\0"
  "vcvtps2iubs128_mask\0"
  "vcvtps2iubs256_mask\0"
  "vcvtps2iubs512_mask\0"
  "vcvtps2ph\0"
  "vcvtps2ph256\0"
  "vcvtps2ph256_mask\0"
  "vcvtps2ph512_mask\0"
  "vcvtps2ph_mask\0"
  "vcvtps2phx128_mask\0"
  "vcvtps2phx256_mask\0"
  "vcvtps2phx512_mask\0"
  "vcvtqq2ph128_mask\0"
  "vcvtqq2ph256_mask\0"
  "vcvtsd2sh_round_mask\0"
  "vcvtsd2si32\0"
  "vcvtsd2si64\0"
  "vcvtsd2usi32\0"
  "vcvtsd2usi64\0"
  "vcvtsh2sd_round_mask\0"
  "vcvtsh2si32\0"
  "vcvtsh2si64\0"
  "vcvtsh2ss_round_mask\0"
  "vcvtsh2usi32\0"
  "vcvtsh2usi64\0"
  "vcvtsi2sh\0"
  "vcvtsi642sh\0"
  "vcvtss2sh_round_mask\0"
  "vcvtss2si32\0"
  "vcvtss2si64\0"
  "vcvtss2usi32\0"
  "vcvtss2usi64\0"
  "vcvttbf162ibs128\0"
  "vcvttbf162ibs256\0"
  "vcvttbf162ibs512\0"
  "vcvttbf162iubs128\0"
  "vcvttbf162iubs256\0"
  "vcvttbf162iubs512\0"
  "vcvttpd2dqs128_mask\0"
  "vcvttpd2dqs256_mask\0"
  "vcvttpd2dqs512_round_mask\0"
  "vcvttpd2qqs128_mask\0"
  "vcvttpd2qqs256_mask\0"
  "vcvttpd2qqs512_round_mask\0"
  "vcvttpd2udqs128_mask\0"
  "vcvttpd2udqs256_mask\0"
  "vcvttpd2udqs512_round_mask\0"
  "vcvttpd2uqqs128_mask\0"
  "vcvttpd2uqqs256_mask\0"
  "vcvttpd2uqqs512_round_mask\0"
  "vcvttph2dq128_mask\0"
  "vcvttph2dq256_mask\0"
  "vcvttph2dq512_mask\0"
  "vcvttph2ibs128_mask\0"
  "vcvttph2ibs256_mask\0"
  "vcvttph2ibs512_mask\0"
  "vcvttph2iubs128_mask\0"
  "vcvttph2iubs256_mask\0"
  "vcvttph2iubs512_mask\0"
  "vcvttph2qq128_mask\0"
  "vcvttph2qq256_mask\0"
  "vcvttph2qq512_mask\0"
  "vcvttph2udq128_mask\0"
  "vcvttph2udq256_mask\0"
  "vcvttph2udq512_mask\0"
  "vcvttph2uqq128_mask\0"
  "vcvttph2uqq256_mask\0"
  "vcvttph2uqq512_mask\0"
  "vcvttph2uw128_mask\0"
  "vcvttph2uw256_mask\0"
  "vcvttph2uw512_mask\0"
  "vcvttph2w128_mask\0"
  "vcvttph2w256_mask\0"
  "vcvttph2w512_mask\0"
  "vcvttps2dqs128_mask\0"
  "vcvttps2dqs256_mask\0"
  "vcvttps2dqs512_round_mask\0"
  "vcvttps2ibs128_mask\0"
  "vcvttps2ibs256_mask\0"
  "vcvttps2ibs512_mask\0"
  "vcvttps2iubs128_mask\0"
  "vcvttps2iubs256_mask\0"
  "vcvttps2iubs512_mask\0"
  "vcvttps2qqs128_mask\0"
  "vcvttps2qqs256_mask\0"
  "vcvttps2qqs512_round_mask\0"
  "vcvttps2udqs128_mask\0"
  "vcvttps2udqs256_mask\0"
  "vcvttps2udqs512_round_mask\0"
  "vcvttps2uqqs128_mask\0"
  "vcvttps2uqqs256_mask\0"
  "vcvttps2uqqs512_round_mask\0"
  "vcvttsd2si32\0"
  "vcvttsd2si64\0"
  "vcvttsd2sis32\0"
  "vcvttsd2sis64\0"
  "vcvttsd2usi32\0"
  "vcvttsd2usi64\0"
  "vcvttsd2usis32\0"
  "vcvttsd2usis64\0"
  "vcvttsh2si32\0"
  "vcvttsh2si64\0"
  "vcvttsh2usi32\0"
  "vcvttsh2usi64\0"
  "vcvttss2si32\0"
  "vcvttss2si64\0"
  "vcvttss2sis32\0"
  "vcvttss2sis64\0"
  "vcvttss2usi32\0"
  "vcvttss2usi64\0"
  "vcvttss2usis32\0"
  "vcvttss2usis64\0"
  "vcvtudq2ph128_mask\0"
  "vcvtuqq2ph128_mask\0"
  "vcvtuqq2ph256_mask\0"
  "vcvtusi2sh\0"
  "vcvtusi642sh\0"
  "vdivbf16128\0"
  "vdivbf16256\0"
  "vdivbf16512\0"
  "vdpphps128\0"
  "vdpphps256\0"
  "vdpphps512\0"
  "vfcmaddcph128_mask\0"
  "vfcmaddcph128_maskz\0"
  "vfcmaddcph256_mask\0"
  "vfcmaddcph256_maskz\0"
  "vfcmaddcph512_mask3\0"
  "vfcmaddcph512_maskz\0"
  "vfcmaddcsh_mask\0"
  "vfcmaddcsh_maskz\0"
  "vfcmulcph128_mask\0"
  "vfcmulcph256_mask\0"
  "vfcmulcph512_mask\0"
  "vfcmulcsh_mask\0"
  "vfmadd132bf16128\0"
  "vfmadd132bf16256\0"
  "vfmadd132bf16512\0"
  "vfmadd213bf16128\0"
  "vfmadd213bf16256\0"
  "vfmadd231bf16128\0"
  "vfmadd231bf16256\0"
  "vfmadd231bf16512\0"
  "vfmaddcph128_mask\0"
  "vfmaddcph128_maskz\0"
  "vfmaddcph256_mask\0"
  "vfmaddcph256_maskz\0"
  "vfmaddcph512_mask3\0"
  "vfmaddcph512_maskz\0"
  "vfmaddcsh_mask\0"
  "vfmaddcsh_maskz\0"
  "vfmaddsubpd\0"
  "vfmaddsubpd256\0"
  "vfmaddsubph\0"
  "vfmaddsubph256\0"
  "vfmaddsubps\0"
  "vfmaddsubps256\0"
  "vfmsub132bf16128\0"
  "vfmsub132bf16256\0"
  "vfmsub132bf16512\0"
  "vfmsub213bf16128\0"
  "vfmsub213bf16256\0"
  "vfmsub213bf16512\0"
  "vfmsub231bf16128\0"
  "vfmsub231bf16256\0"
  "vfmsub231bf16512\0"
  "vfmulcph128_mask\0"
  "vfmulcph256_mask\0"
  "vfmulcph512_mask\0"
  "vfmulcsh_mask\0"
  "vfnmadd132bf16128\0"
  "vfnmadd132bf16256\0"
  "vfnmadd132bf16512\0"
  "vfnmadd213bf16128\0"
  "vfnmadd213bf16256\0"
  "vfnmadd213bf16512\0"
  "vfnmadd231bf16128\0"
  "vfnmadd231bf16256\0"
  "vfnmadd231bf16512\0"
  "vfnmsub132bf16128\0"
  "vfnmsub132bf16256\0"
  "vfnmsub132bf16512\0"
  "vfnmsub213bf16128\0"
  "vfnmsub213bf16256\0"
  "vfnmsub213bf16512\0"
  "vfnmsub231bf16128\0"
  "vfnmsub231bf16256\0"
  "vfnmsub231bf16512\0"
  "vfrczpd\0"
  "vfrczpd256\0"
  "vfrczps\0"
  "vfrczps256\0"
  "vfrczsd\0"
  "vfrczss\0"
  "vgetexpbf16128_mask\0"
  "vgetexpbf16256_mask\0"
  "vgetexpbf16512_mask\0"
  "vgetmantbf16128_mask\0"
  "vgetmantbf16256_mask\0"
  "vgetmantbf16512_mask\0"
  "vgf2p8affineinvqb_v16qi\0"
  "vgf2p8affineinvqb_v32qi\0"
  "vgf2p8affineinvqb_v64qi\0"
  "vgf2p8affineqb_v16qi\0"
  "vgf2p8affineqb_v32qi\0"
  "vgf2p8affineqb_v64qi\0"
  "vgf2p8mulb_v16qi\0"
  "vgf2p8mulb_v32qi\0"
  "vgf2p8mulb_v64qi\0"
  "vmaxbf16128\0"
  "vmaxbf16256\0"
  "vmaxbf16512\0"
  "vminbf16128\0"
  "vminbf16256\0"
  "vminbf16512\0"
  "vminmaxbf16128\0"
  "vminmaxbf16256\0"
  "vminmaxbf16512\0"
  "vminmaxpd128\0"
  "vminmaxpd128_mask\0"
  "vminmaxpd256\0"
  "vminmaxpd256_mask\0"
  "vminmaxpd512_round_mask\0"
  "vminmaxph128\0"
  "vminmaxph128_mask\0"
  "vminmaxph256\0"
  "vminmaxph256_mask\0"
  "vminmaxph512_round_mask\0"
  "vminmaxps128\0"
  "vminmaxps128_mask\0"
  "vminmaxps256\0"
  "vminmaxps256_mask\0"
  "vminmaxps512_round_mask\0"
  "vminmaxsd_round_mask\0"
  "vminmaxsh_round_mask\0"
  "vminmaxss_round_mask\0"
  "vmovrsb128\0"
  "vmovrsb256\0"
  "vmovrsb512\0"
  "vmovrsd128\0"
  "vmovrsd256\0"
  "vmovrsd512\0"
  "vmovrsq128\0"
  "vmovrsq256\0"
  "vmovrsq512\0"
  "vmovrsw128\0"
  "vmovrsw256\0"
  "vmovrsw512\0"
  "vmulbf16128\0"
  "vmulbf16256\0"
  "vmulbf16512\0"
  "vpconflictdi_128\0"
  "vpconflictdi_256\0"
  "vpconflictdi_512\0"
  "vpconflictsi_128\0"
  "vpconflictsi_256\0"
  "vpconflictsi_512\0"
  "vpdpbssd128\0"
  "vpdpbssd256\0"
  "vpdpbssd512\0"
  "vpdpbssds128\0"
  "vpdpbssds256\0"
  "vpdpbssds512\0"
  "vpdpbsud128\0"
  "vpdpbsud256\0"
  "vpdpbsud512\0"
  "vpdpbsuds128\0"
  "vpdpbsuds256\0"
  "vpdpbsuds512\0"
  "vpdpbusd128\0"
  "vpdpbusd256\0"
  "vpdpbusd512\0"
  "vpdpbusds128\0"
  "vpdpbusds256\0"
  "vpdpbusds512\0"
  "vpdpbuud128\0"
  "vpdpbuud256\0"
  "vpdpbuud512\0"
  "vpdpbuuds128\0"
  "vpdpbuuds256\0"
  "vpdpbuuds512\0"
  "vpdpwssd128\0"
  "vpdpwssd256\0"
  "vpdpwssd512\0"
  "vpdpwssds128\0"
  "vpdpwssds256\0"
  "vpdpwssds512\0"
  "vpdpwsud128\0"
  "vpdpwsud256\0"
  "vpdpwsud512\0"
  "vpdpwsuds128\0"
  "vpdpwsuds256\0"
  "vpdpwsuds512\0"
  "vpdpwusd128\0"
  "vpdpwusd256\0"
  "vpdpwusd512\0"
  "vpdpwusds128\0"
  "vpdpwusds256\0"
  "vpdpwusds512\0"
  "vpdpwuud128\0"
  "vpdpwuud256\0"
  "vpdpwuud512\0"
  "vpdpwuuds128\0"
  "vpdpwuuds256\0"
  "vpdpwuuds512\0"
  "vpermi2vard128\0"
  "vpermi2vard256\0"
  "vpermi2vard512\0"
  "vpermi2varhi128\0"
  "vpermi2varhi256\0"
  "vpermi2varhi512\0"
  "vpermi2varpd128\0"
  "vpermi2varpd256\0"
  "vpermi2varpd512\0"
  "vpermi2varps128\0"
  "vpermi2varps256\0"
  "vpermi2varps512\0"
  "vpermi2varq128\0"
  "vpermi2varq256\0"
  "vpermi2varq512\0"
  "vpermi2varqi128\0"
  "vpermi2varqi256\0"
  "vpermi2varqi512\0"
  "vpermil2pd\0"
  "vpermil2pd256\0"
  "vpermil2ps\0"
  "vpermil2ps256\0"
  "vpermilvarpd\0"
  "vpermilvarpd256\0"
  "vpermilvarpd512\0"
  "vpermilvarps\0"
  "vpermilvarps256\0"
  "vpermilvarps512\0"
  "vphaddbd\0"
  "vphaddbq\0"
  "vphaddbw\0"
  "vphadddq\0"
  "vphaddubd\0"
  "vphaddubq\0"
  "vphaddubw\0"
  "vphaddudq\0"
  "vphadduwd\0"
  "vphadduwq\0"
  "vphaddwd\0"
  "vphaddwq\0"
  "vphsubbw\0"
  "vphsubdq\0"
  "vphsubwd\0"
  "vpmacsdd\0"
  "vpmacsdqh\0"
  "vpmacsdql\0"
  "vpmacssdd\0"
  "vpmacssdqh\0"
  "vpmacssdql\0"
  "vpmacsswd\0"
  "vpmacssww\0"
  "vpmacswd\0"
  "vpmacsww\0"
  "vpmadcsswd\0"
  "vpmadcswd\0"
  "vpmadd52huq128\0"
  "vpmadd52huq256\0"
  "vpmadd52huq512\0"
  "vpmadd52luq128\0"
  "vpmadd52luq256\0"
  "vpmadd52luq512\0"
  "vpmultishiftqb128\0"
  "vpmultishiftqb256\0"
  "vpmultishiftqb512\0"
  "vpperm\0"
  "vpshab\0"
  "vpshad\0"
  "vpshaq\0"
  "vpshaw\0"
  "vpshlb\0"
  "vpshld\0"
  "vpshlq\0"
  "vpshlw\0"
  "vrcpbf16128_mask\0"
  "vrcpbf16256_mask\0"
  "vrcpbf16512_mask\0"
  "vreducebf16128_mask\0"
  "vreducebf16256_mask\0"
  "vreducebf16512_mask\0"
  "vrndscalebf16_128_mask\0"
  "vrndscalebf16_256_mask\0"
  "vrndscalebf16_mask\0"
  "vrsqrtbf16128_mask\0"
  "vrsqrtbf16256_mask\0"
  "vrsqrtbf16512_mask\0"
  "vscalefbf16128_mask\0"
  "vscalefbf16256_mask\0"
  "vscalefbf16512_mask\0"
  "vsha512msg1\0"
  "vsha512msg2\0"
  "vsha512rnds2\0"
  "vsm3msg1\0"
  "vsm3msg2\0"
  "vsm3rnds2\0"
  "vsm4key4128\0"
  "vsm4key4256\0"
  "vsm4key4512\0"
  "vsm4rnds4128\0"
  "vsm4rnds4256\0"
  "vsm4rnds4512\0"
  "vsubbf16128\0"
  "vsubbf16256\0"
  "vsubbf16512\0"
  "vtestcpd\0"
  "vtestcpd256\0"
  "vtestcps\0"
  "vtestcps256\0"
  "vtestnzcpd\0"
  "vtestnzcpd256\0"
  "vtestnzcps\0"
  "vtestnzcps256\0"
  "vtestzpd\0"
  "vtestzpd256\0"
  "vtestzps\0"
  "vtestzps256\0"
  "vzeroall\0"
  "vzeroupper\0"
  "wbinvd\0"
  "wbnoinvd\0"
  "wrfsbase32\0"
  "wrfsbase64\0"
  "wrgsbase32\0"
  "wrgsbase64\0"
  "writeeflags_u32\0"
  "writeeflags_u64\0"
  "wrpkru\0"
  "wrssd\0"
  "wrssq\0"
  "wrussd\0"
  "wrussq\0"
  "xabort\0"
  "xbegin\0"
  "xend\0"
  "xresldtrk\0"
  "xsusldtrk\0"
  "xtest\0"
  "bitrev\0"
  "getid\0"
  "getps\0"
  "setps\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinNames = BuiltinNamesStorage;

  struct BuiltinEntry {
    ID IntrinsicID;
    unsigned StrTabOffset;
    const char *getName() const { return BuiltinNames[StrTabOffset].data(); }
    bool operator<(StringRef RHS) const {
      return strncmp(getName(), RHS.data(), RHS.size()) < 0;
    }
  };

  // Target independent builtins.
  static constexpr BuiltinEntry Names[] = {
    {adjust_trampoline, 1}, // __builtin_adjust_trampoline
    {allow_runtime_check, 19}, // __builtin_allow_runtime_check
    {debugtrap, 39}, // __builtin_debugtrap
    {init_trampoline, 49}, // __builtin_init_trampoline
    {objectsize, 65}, // __builtin_object_size
    {stackrestore, 77}, // __builtin_stack_restore
    {stacksave, 91}, // __builtin_stack_save
    {thread_pointer, 102}, // __builtin_thread_pointer
    {trap, 117}, // __builtin_trap
    {eh_unwind_init, 122}, // __builtin_unwind_init
  }; // Names

  // Builtins for aarch64.
  static constexpr BuiltinEntry aarch64Names[] = {
    {aarch64_chkfeat, 134}, // __builtin_arm_chkfeat
    {aarch64_dmb, 146}, // __builtin_arm_dmb
    {aarch64_dsb, 154}, // __builtin_arm_dsb
    {aarch64_gcspopm, 162}, // __builtin_arm_gcspopm
    {aarch64_gcsss, 174}, // __builtin_arm_gcsss
    {aarch64_sme_in_streaming_mode, 184}, // __builtin_arm_in_streaming_mode
    {aarch64_isb, 206}, // __builtin_arm_isb
    {aarch64_prefetch, 214}, // __builtin_arm_prefetch
    {aarch64_range_prefetch, 227}, // __builtin_arm_range_prefetch
    {aarch64_sve_aesd, 246}, // __builtin_sve_svaesd_u8
    {aarch64_sve_aese, 260}, // __builtin_sve_svaese_u8
    {aarch64_sve_aesimc, 274}, // __builtin_sve_svaesimc_u8
    {aarch64_sve_aesmc, 290}, // __builtin_sve_svaesmc_u8
    {aarch64_sve_rax1, 305}, // __builtin_sve_svrax1_u64
    {aarch64_sve_rdffr, 320}, // __builtin_sve_svrdffr
    {aarch64_sve_rdffr_z, 332}, // __builtin_sve_svrdffr_z
    {aarch64_sve_setffr, 346}, // __builtin_sve_svsetffr
    {aarch64_sve_sm4e, 359}, // __builtin_sve_svsm4e_u32
    {aarch64_sve_sm4ekey, 374}, // __builtin_sve_svsm4ekey_u32
    {aarch64_sve_wrffr, 392}, // __builtin_sve_svwrffr
  }; // aarch64Names

  // Builtins for amdgcn.
  static constexpr BuiltinEntry amdgcnNames[] = {
    {amdgcn_add_max_i32, 404}, // __builtin_amdgcn_add_max_i32
    {amdgcn_add_max_u32, 416}, // __builtin_amdgcn_add_max_u32
    {amdgcn_add_min_i32, 428}, // __builtin_amdgcn_add_min_i32
    {amdgcn_add_min_u32, 440}, // __builtin_amdgcn_add_min_u32
    {amdgcn_alignbyte, 452}, // __builtin_amdgcn_alignbyte
    {amdgcn_ashr_pk_i8_i32, 462}, // __builtin_amdgcn_ashr_pk_i8_i32
    {amdgcn_ashr_pk_u8_i32, 477}, // __builtin_amdgcn_ashr_pk_u8_i32
    {amdgcn_buffer_wbinvl1, 492}, // __builtin_amdgcn_buffer_wbinvl1
    {amdgcn_buffer_wbinvl1_sc, 507}, // __builtin_amdgcn_buffer_wbinvl1_sc
    {amdgcn_buffer_wbinvl1_vol, 525}, // __builtin_amdgcn_buffer_wbinvl1_vol
    {amdgcn_cluster_id_x, 544}, // __builtin_amdgcn_cluster_id_x
    {amdgcn_cluster_id_y, 557}, // __builtin_amdgcn_cluster_id_y
    {amdgcn_cluster_id_z, 570}, // __builtin_amdgcn_cluster_id_z
    {amdgcn_cluster_load_async_to_lds_b128, 583}, // __builtin_amdgcn_cluster_load_async_to_lds_b128
    {amdgcn_cluster_load_async_to_lds_b32, 614}, // __builtin_amdgcn_cluster_load_async_to_lds_b32
    {amdgcn_cluster_load_async_to_lds_b64, 644}, // __builtin_amdgcn_cluster_load_async_to_lds_b64
    {amdgcn_cluster_load_async_to_lds_b8, 674}, // __builtin_amdgcn_cluster_load_async_to_lds_b8
    {amdgcn_cluster_workgroup_flat_id, 703}, // __builtin_amdgcn_cluster_workgroup_flat_id
    {amdgcn_cluster_workgroup_id_x, 729}, // __builtin_amdgcn_cluster_workgroup_id_x
    {amdgcn_cluster_workgroup_id_y, 752}, // __builtin_amdgcn_cluster_workgroup_id_y
    {amdgcn_cluster_workgroup_id_z, 775}, // __builtin_amdgcn_cluster_workgroup_id_z
    {amdgcn_cluster_workgroup_max_flat_id, 798}, // __builtin_amdgcn_cluster_workgroup_max_flat_id
    {amdgcn_cluster_workgroup_max_id_x, 828}, // __builtin_amdgcn_cluster_workgroup_max_id_x
    {amdgcn_cluster_workgroup_max_id_y, 855}, // __builtin_amdgcn_cluster_workgroup_max_id_y
    {amdgcn_cluster_workgroup_max_id_z, 882}, // __builtin_amdgcn_cluster_workgroup_max_id_z
    {amdgcn_cubeid, 909}, // __builtin_amdgcn_cubeid
    {amdgcn_cubema, 916}, // __builtin_amdgcn_cubema
    {amdgcn_cubesc, 923}, // __builtin_amdgcn_cubesc
    {amdgcn_cubetc, 930}, // __builtin_amdgcn_cubetc
    {amdgcn_cvt_f16_bf8, 937}, // __builtin_amdgcn_cvt_f16_bf8
    {amdgcn_cvt_f16_fp8, 949}, // __builtin_amdgcn_cvt_f16_fp8
    {amdgcn_cvt_f32_bf8, 961}, // __builtin_amdgcn_cvt_f32_bf8
    {amdgcn_cvt_f32_fp8, 973}, // __builtin_amdgcn_cvt_f32_fp8
    {amdgcn_cvt_f32_fp8_e5m3, 985}, // __builtin_amdgcn_cvt_f32_fp8_e5m3
    {amdgcn_cvt_off_f32_i4, 1002}, // __builtin_amdgcn_cvt_off_f32_i4
    {amdgcn_cvt_pk_bf8_f16, 1017}, // __builtin_amdgcn_cvt_pk_bf8_f16
    {amdgcn_cvt_pk_bf8_f32, 1032}, // __builtin_amdgcn_cvt_pk_bf8_f32
    {amdgcn_cvt_pk_f16_bf8, 1047}, // __builtin_amdgcn_cvt_pk_f16_bf8
    {amdgcn_cvt_pk_f16_fp8, 1062}, // __builtin_amdgcn_cvt_pk_f16_fp8
    {amdgcn_cvt_pk_f32_bf8, 1077}, // __builtin_amdgcn_cvt_pk_f32_bf8
    {amdgcn_cvt_pk_f32_fp8, 1092}, // __builtin_amdgcn_cvt_pk_f32_fp8
    {amdgcn_cvt_pk_fp8_f16, 1107}, // __builtin_amdgcn_cvt_pk_fp8_f16
    {amdgcn_cvt_pk_fp8_f32, 1122}, // __builtin_amdgcn_cvt_pk_fp8_f32
    {amdgcn_cvt_pk_fp8_f32_e5m3, 1137}, // __builtin_amdgcn_cvt_pk_fp8_f32_e5m3
    {amdgcn_cvt_pk_i16, 1157}, // __builtin_amdgcn_cvt_pk_i16
    {amdgcn_cvt_pk_u16, 1168}, // __builtin_amdgcn_cvt_pk_u16
    {amdgcn_cvt_pk_u8_f32, 1179}, // __builtin_amdgcn_cvt_pk_u8_f32
    {amdgcn_cvt_pknorm_i16, 1193}, // __builtin_amdgcn_cvt_pknorm_i16
    {amdgcn_cvt_pknorm_u16, 1208}, // __builtin_amdgcn_cvt_pknorm_u16
    {amdgcn_cvt_pkrtz, 1223}, // __builtin_amdgcn_cvt_pkrtz
    {amdgcn_cvt_scale_pk16_bf16_bf6, 1233}, // __builtin_amdgcn_cvt_scale_pk16_bf16_bf6
    {amdgcn_cvt_scale_pk16_bf16_fp6, 1257}, // __builtin_amdgcn_cvt_scale_pk16_bf16_fp6
    {amdgcn_cvt_scale_pk16_f16_bf6, 1281}, // __builtin_amdgcn_cvt_scale_pk16_f16_bf6
    {amdgcn_cvt_scale_pk16_f16_fp6, 1304}, // __builtin_amdgcn_cvt_scale_pk16_f16_fp6
    {amdgcn_cvt_scale_pk16_f32_bf6, 1327}, // __builtin_amdgcn_cvt_scale_pk16_f32_bf6
    {amdgcn_cvt_scale_pk16_f32_fp6, 1350}, // __builtin_amdgcn_cvt_scale_pk16_f32_fp6
    {amdgcn_cvt_scale_pk8_bf16_bf8, 1373}, // __builtin_amdgcn_cvt_scale_pk8_bf16_bf8
    {amdgcn_cvt_scale_pk8_bf16_fp4, 1396}, // __builtin_amdgcn_cvt_scale_pk8_bf16_fp4
    {amdgcn_cvt_scale_pk8_bf16_fp8, 1419}, // __builtin_amdgcn_cvt_scale_pk8_bf16_fp8
    {amdgcn_cvt_scale_pk8_f16_bf8, 1442}, // __builtin_amdgcn_cvt_scale_pk8_f16_bf8
    {amdgcn_cvt_scale_pk8_f16_fp4, 1464}, // __builtin_amdgcn_cvt_scale_pk8_f16_fp4
    {amdgcn_cvt_scale_pk8_f16_fp8, 1486}, // __builtin_amdgcn_cvt_scale_pk8_f16_fp8
    {amdgcn_cvt_scale_pk8_f32_bf8, 1508}, // __builtin_amdgcn_cvt_scale_pk8_f32_bf8
    {amdgcn_cvt_scale_pk8_f32_fp4, 1530}, // __builtin_amdgcn_cvt_scale_pk8_f32_fp4
    {amdgcn_cvt_scale_pk8_f32_fp8, 1552}, // __builtin_amdgcn_cvt_scale_pk8_f32_fp8
    {amdgcn_cvt_scalef32_2xpk16_bf6_f32, 1574}, // __builtin_amdgcn_cvt_scalef32_2xpk16_bf6_f32
    {amdgcn_cvt_scalef32_2xpk16_fp6_f32, 1602}, // __builtin_amdgcn_cvt_scalef32_2xpk16_fp6_f32
    {amdgcn_cvt_scalef32_f16_bf8, 1630}, // __builtin_amdgcn_cvt_scalef32_f16_bf8
    {amdgcn_cvt_scalef32_f16_fp8, 1651}, // __builtin_amdgcn_cvt_scalef32_f16_fp8
    {amdgcn_cvt_scalef32_f32_bf8, 1672}, // __builtin_amdgcn_cvt_scalef32_f32_bf8
    {amdgcn_cvt_scalef32_f32_fp8, 1693}, // __builtin_amdgcn_cvt_scalef32_f32_fp8
    {amdgcn_cvt_scalef32_pk16_bf6_bf16, 1714}, // __builtin_amdgcn_cvt_scalef32_pk16_bf6_bf16
    {amdgcn_cvt_scalef32_pk16_bf6_f16, 1741}, // __builtin_amdgcn_cvt_scalef32_pk16_bf6_f16
    {amdgcn_cvt_scalef32_pk16_bf6_f32, 1767}, // __builtin_amdgcn_cvt_scalef32_pk16_bf6_f32
    {amdgcn_cvt_scalef32_pk16_fp6_bf16, 1793}, // __builtin_amdgcn_cvt_scalef32_pk16_fp6_bf16
    {amdgcn_cvt_scalef32_pk16_fp6_f16, 1820}, // __builtin_amdgcn_cvt_scalef32_pk16_fp6_f16
    {amdgcn_cvt_scalef32_pk16_fp6_f32, 1846}, // __builtin_amdgcn_cvt_scalef32_pk16_fp6_f32
    {amdgcn_cvt_scalef32_pk32_bf16_bf6, 1872}, // __builtin_amdgcn_cvt_scalef32_pk32_bf16_bf6
    {amdgcn_cvt_scalef32_pk32_bf16_fp6, 1899}, // __builtin_amdgcn_cvt_scalef32_pk32_bf16_fp6
    {amdgcn_cvt_scalef32_pk32_bf6_bf16, 1926}, // __builtin_amdgcn_cvt_scalef32_pk32_bf6_bf16
    {amdgcn_cvt_scalef32_pk32_bf6_f16, 1953}, // __builtin_amdgcn_cvt_scalef32_pk32_bf6_f16
    {amdgcn_cvt_scalef32_pk32_f16_bf6, 1979}, // __builtin_amdgcn_cvt_scalef32_pk32_f16_bf6
    {amdgcn_cvt_scalef32_pk32_f16_fp6, 2005}, // __builtin_amdgcn_cvt_scalef32_pk32_f16_fp6
    {amdgcn_cvt_scalef32_pk32_f32_bf6, 2031}, // __builtin_amdgcn_cvt_scalef32_pk32_f32_bf6
    {amdgcn_cvt_scalef32_pk32_f32_fp6, 2057}, // __builtin_amdgcn_cvt_scalef32_pk32_f32_fp6
    {amdgcn_cvt_scalef32_pk32_fp6_bf16, 2083}, // __builtin_amdgcn_cvt_scalef32_pk32_fp6_bf16
    {amdgcn_cvt_scalef32_pk32_fp6_f16, 2110}, // __builtin_amdgcn_cvt_scalef32_pk32_fp6_f16
    {amdgcn_cvt_scalef32_pk8_bf8_bf16, 2136}, // __builtin_amdgcn_cvt_scalef32_pk8_bf8_bf16
    {amdgcn_cvt_scalef32_pk8_bf8_f16, 2162}, // __builtin_amdgcn_cvt_scalef32_pk8_bf8_f16
    {amdgcn_cvt_scalef32_pk8_bf8_f32, 2187}, // __builtin_amdgcn_cvt_scalef32_pk8_bf8_f32
    {amdgcn_cvt_scalef32_pk8_fp4_bf16, 2212}, // __builtin_amdgcn_cvt_scalef32_pk8_fp4_bf16
    {amdgcn_cvt_scalef32_pk8_fp4_f16, 2238}, // __builtin_amdgcn_cvt_scalef32_pk8_fp4_f16
    {amdgcn_cvt_scalef32_pk8_fp4_f32, 2263}, // __builtin_amdgcn_cvt_scalef32_pk8_fp4_f32
    {amdgcn_cvt_scalef32_pk8_fp8_bf16, 2288}, // __builtin_amdgcn_cvt_scalef32_pk8_fp8_bf16
    {amdgcn_cvt_scalef32_pk8_fp8_f16, 2314}, // __builtin_amdgcn_cvt_scalef32_pk8_fp8_f16
    {amdgcn_cvt_scalef32_pk8_fp8_f32, 2339}, // __builtin_amdgcn_cvt_scalef32_pk8_fp8_f32
    {amdgcn_cvt_scalef32_pk_bf16_bf8, 2364}, // __builtin_amdgcn_cvt_scalef32_pk_bf16_bf8
    {amdgcn_cvt_scalef32_pk_bf16_fp4, 2389}, // __builtin_amdgcn_cvt_scalef32_pk_bf16_fp4
    {amdgcn_cvt_scalef32_pk_bf16_fp8, 2414}, // __builtin_amdgcn_cvt_scalef32_pk_bf16_fp8
    {amdgcn_cvt_scalef32_pk_bf8_bf16, 2439}, // __builtin_amdgcn_cvt_scalef32_pk_bf8_bf16
    {amdgcn_cvt_scalef32_pk_bf8_f16, 2464}, // __builtin_amdgcn_cvt_scalef32_pk_bf8_f16
    {amdgcn_cvt_scalef32_pk_bf8_f32, 2488}, // __builtin_amdgcn_cvt_scalef32_pk_bf8_f32
    {amdgcn_cvt_scalef32_pk_f16_bf8, 2512}, // __builtin_amdgcn_cvt_scalef32_pk_f16_bf8
    {amdgcn_cvt_scalef32_pk_f16_fp4, 2536}, // __builtin_amdgcn_cvt_scalef32_pk_f16_fp4
    {amdgcn_cvt_scalef32_pk_f16_fp8, 2560}, // __builtin_amdgcn_cvt_scalef32_pk_f16_fp8
    {amdgcn_cvt_scalef32_pk_f32_bf8, 2584}, // __builtin_amdgcn_cvt_scalef32_pk_f32_bf8
    {amdgcn_cvt_scalef32_pk_f32_fp4, 2608}, // __builtin_amdgcn_cvt_scalef32_pk_f32_fp4
    {amdgcn_cvt_scalef32_pk_f32_fp8, 2632}, // __builtin_amdgcn_cvt_scalef32_pk_f32_fp8
    {amdgcn_cvt_scalef32_pk_fp4_bf16, 2656}, // __builtin_amdgcn_cvt_scalef32_pk_fp4_bf16
    {amdgcn_cvt_scalef32_pk_fp4_f16, 2681}, // __builtin_amdgcn_cvt_scalef32_pk_fp4_f16
    {amdgcn_cvt_scalef32_pk_fp4_f32, 2705}, // __builtin_amdgcn_cvt_scalef32_pk_fp4_f32
    {amdgcn_cvt_scalef32_pk_fp8_bf16, 2729}, // __builtin_amdgcn_cvt_scalef32_pk_fp8_bf16
    {amdgcn_cvt_scalef32_pk_fp8_f16, 2754}, // __builtin_amdgcn_cvt_scalef32_pk_fp8_f16
    {amdgcn_cvt_scalef32_pk_fp8_f32, 2778}, // __builtin_amdgcn_cvt_scalef32_pk_fp8_f32
    {amdgcn_cvt_scalef32_sr_bf8_bf16, 2802}, // __builtin_amdgcn_cvt_scalef32_sr_bf8_bf16
    {amdgcn_cvt_scalef32_sr_bf8_f16, 2827}, // __builtin_amdgcn_cvt_scalef32_sr_bf8_f16
    {amdgcn_cvt_scalef32_sr_bf8_f32, 2851}, // __builtin_amdgcn_cvt_scalef32_sr_bf8_f32
    {amdgcn_cvt_scalef32_sr_fp8_bf16, 2875}, // __builtin_amdgcn_cvt_scalef32_sr_fp8_bf16
    {amdgcn_cvt_scalef32_sr_fp8_f16, 2900}, // __builtin_amdgcn_cvt_scalef32_sr_fp8_f16
    {amdgcn_cvt_scalef32_sr_fp8_f32, 2924}, // __builtin_amdgcn_cvt_scalef32_sr_fp8_f32
    {amdgcn_cvt_scalef32_sr_pk16_bf6_bf16, 2948}, // __builtin_amdgcn_cvt_scalef32_sr_pk16_bf6_bf16
    {amdgcn_cvt_scalef32_sr_pk16_bf6_f16, 2978}, // __builtin_amdgcn_cvt_scalef32_sr_pk16_bf6_f16
    {amdgcn_cvt_scalef32_sr_pk16_bf6_f32, 3007}, // __builtin_amdgcn_cvt_scalef32_sr_pk16_bf6_f32
    {amdgcn_cvt_scalef32_sr_pk16_fp6_bf16, 3036}, // __builtin_amdgcn_cvt_scalef32_sr_pk16_fp6_bf16
    {amdgcn_cvt_scalef32_sr_pk16_fp6_f16, 3066}, // __builtin_amdgcn_cvt_scalef32_sr_pk16_fp6_f16
    {amdgcn_cvt_scalef32_sr_pk16_fp6_f32, 3095}, // __builtin_amdgcn_cvt_scalef32_sr_pk16_fp6_f32
    {amdgcn_cvt_scalef32_sr_pk32_bf6_bf16, 3124}, // __builtin_amdgcn_cvt_scalef32_sr_pk32_bf6_bf16
    {amdgcn_cvt_scalef32_sr_pk32_bf6_f16, 3154}, // __builtin_amdgcn_cvt_scalef32_sr_pk32_bf6_f16
    {amdgcn_cvt_scalef32_sr_pk32_bf6_f32, 3183}, // __builtin_amdgcn_cvt_scalef32_sr_pk32_bf6_f32
    {amdgcn_cvt_scalef32_sr_pk32_fp6_bf16, 3212}, // __builtin_amdgcn_cvt_scalef32_sr_pk32_fp6_bf16
    {amdgcn_cvt_scalef32_sr_pk32_fp6_f16, 3242}, // __builtin_amdgcn_cvt_scalef32_sr_pk32_fp6_f16
    {amdgcn_cvt_scalef32_sr_pk32_fp6_f32, 3271}, // __builtin_amdgcn_cvt_scalef32_sr_pk32_fp6_f32
    {amdgcn_cvt_scalef32_sr_pk8_bf8_bf16, 3300}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_bf8_bf16
    {amdgcn_cvt_scalef32_sr_pk8_bf8_f16, 3329}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_bf8_f16
    {amdgcn_cvt_scalef32_sr_pk8_bf8_f32, 3357}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_bf8_f32
    {amdgcn_cvt_scalef32_sr_pk8_fp4_bf16, 3385}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_fp4_bf16
    {amdgcn_cvt_scalef32_sr_pk8_fp4_f16, 3414}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_fp4_f16
    {amdgcn_cvt_scalef32_sr_pk8_fp4_f32, 3442}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_fp4_f32
    {amdgcn_cvt_scalef32_sr_pk8_fp8_bf16, 3470}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_fp8_bf16
    {amdgcn_cvt_scalef32_sr_pk8_fp8_f16, 3499}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_fp8_f16
    {amdgcn_cvt_scalef32_sr_pk8_fp8_f32, 3527}, // __builtin_amdgcn_cvt_scalef32_sr_pk8_fp8_f32
    {amdgcn_cvt_scalef32_sr_pk_fp4_bf16, 3555}, // __builtin_amdgcn_cvt_scalef32_sr_pk_fp4_bf16
    {amdgcn_cvt_scalef32_sr_pk_fp4_f16, 3583}, // __builtin_amdgcn_cvt_scalef32_sr_pk_fp4_f16
    {amdgcn_cvt_scalef32_sr_pk_fp4_f32, 3610}, // __builtin_amdgcn_cvt_scalef32_sr_pk_fp4_f32
    {amdgcn_cvt_sr_bf16_f32, 3637}, // __builtin_amdgcn_cvt_sr_bf16_f32
    {amdgcn_cvt_sr_bf8_f16, 3653}, // __builtin_amdgcn_cvt_sr_bf8_f16
    {amdgcn_cvt_sr_bf8_f32, 3668}, // __builtin_amdgcn_cvt_sr_bf8_f32
    {amdgcn_cvt_sr_f16_f32, 3683}, // __builtin_amdgcn_cvt_sr_f16_f32
    {amdgcn_cvt_sr_fp8_f16, 3698}, // __builtin_amdgcn_cvt_sr_fp8_f16
    {amdgcn_cvt_sr_fp8_f32, 3713}, // __builtin_amdgcn_cvt_sr_fp8_f32
    {amdgcn_cvt_sr_fp8_f32_e5m3, 3728}, // __builtin_amdgcn_cvt_sr_fp8_f32_e5m3
    {amdgcn_cvt_sr_pk_bf16_f32, 3748}, // __builtin_amdgcn_cvt_sr_pk_bf16_f32
    {amdgcn_cvt_sr_pk_f16_f32, 3767}, // __builtin_amdgcn_cvt_sr_pk_f16_f32
    {amdgcn_dispatch_id, 3785}, // __builtin_amdgcn_dispatch_id
    {amdgcn_dot4_f32_bf8_bf8, 3797}, // __builtin_amdgcn_dot4_f32_bf8_bf8
    {amdgcn_dot4_f32_bf8_fp8, 3814}, // __builtin_amdgcn_dot4_f32_bf8_fp8
    {amdgcn_dot4_f32_fp8_bf8, 3831}, // __builtin_amdgcn_dot4_f32_fp8_bf8
    {amdgcn_dot4_f32_fp8_fp8, 3848}, // __builtin_amdgcn_dot4_f32_fp8_fp8
    {amdgcn_ds_add_gs_reg_rtn, 3865}, // __builtin_amdgcn_ds_add_gs_reg_rtn
    {amdgcn_ds_atomic_async_barrier_arrive_b64, 3883}, // __builtin_amdgcn_ds_atomic_async_barrier_arrive_b64
    {amdgcn_ds_atomic_barrier_arrive_rtn_b64, 3918}, // __builtin_amdgcn_ds_atomic_barrier_arrive_rtn_b64
    {amdgcn_ds_bpermute, 3951}, // __builtin_amdgcn_ds_bpermute
    {amdgcn_ds_bpermute_fi_b32, 3963}, // __builtin_amdgcn_ds_bpermute_fi_b32
    {amdgcn_ds_gws_barrier, 3982}, // __builtin_amdgcn_ds_gws_barrier
    {amdgcn_ds_gws_init, 3997}, // __builtin_amdgcn_ds_gws_init
    {amdgcn_ds_gws_sema_br, 4009}, // __builtin_amdgcn_ds_gws_sema_br
    {amdgcn_ds_gws_sema_p, 4024}, // __builtin_amdgcn_ds_gws_sema_p
    {amdgcn_ds_gws_sema_release_all, 4038}, // __builtin_amdgcn_ds_gws_sema_release_all
    {amdgcn_ds_gws_sema_v, 4062}, // __builtin_amdgcn_ds_gws_sema_v
    {amdgcn_ds_permute, 4076}, // __builtin_amdgcn_ds_permute
    {amdgcn_ds_sub_gs_reg_rtn, 4087}, // __builtin_amdgcn_ds_sub_gs_reg_rtn
    {amdgcn_ds_swizzle, 4105}, // __builtin_amdgcn_ds_swizzle
    {amdgcn_endpgm, 4116}, // __builtin_amdgcn_endpgm
    {amdgcn_fdot2, 4123}, // __builtin_amdgcn_fdot2
    {amdgcn_fdot2_bf16_bf16, 4129}, // __builtin_amdgcn_fdot2_bf16_bf16
    {amdgcn_fdot2_f16_f16, 4145}, // __builtin_amdgcn_fdot2_f16_f16
    {amdgcn_fdot2_f32_bf16, 4159}, // __builtin_amdgcn_fdot2_f32_bf16
    {amdgcn_fdot2c_f32_bf16, 4174}, // __builtin_amdgcn_fdot2c_f32_bf16
    {amdgcn_flat_prefetch, 4190}, // __builtin_amdgcn_flat_prefetch
    {amdgcn_fmul_legacy, 4204}, // __builtin_amdgcn_fmul_legacy
    {amdgcn_global_load_async_to_lds_b128, 4216}, // __builtin_amdgcn_global_load_async_to_lds_b128
    {amdgcn_global_load_async_to_lds_b32, 4246}, // __builtin_amdgcn_global_load_async_to_lds_b32
    {amdgcn_global_load_async_to_lds_b64, 4275}, // __builtin_amdgcn_global_load_async_to_lds_b64
    {amdgcn_global_load_async_to_lds_b8, 4304}, // __builtin_amdgcn_global_load_async_to_lds_b8
    {amdgcn_global_load_lds, 4332}, // __builtin_amdgcn_global_load_lds
    {amdgcn_global_prefetch, 4348}, // __builtin_amdgcn_global_prefetch
    {amdgcn_global_store_async_from_lds_b128, 4364}, // __builtin_amdgcn_global_store_async_from_lds_b128
    {amdgcn_global_store_async_from_lds_b32, 4397}, // __builtin_amdgcn_global_store_async_from_lds_b32
    {amdgcn_global_store_async_from_lds_b64, 4429}, // __builtin_amdgcn_global_store_async_from_lds_b64
    {amdgcn_global_store_async_from_lds_b8, 4461}, // __builtin_amdgcn_global_store_async_from_lds_b8
    {amdgcn_groupstaticsize, 4492}, // __builtin_amdgcn_groupstaticsize
    {amdgcn_iglp_opt, 4508}, // __builtin_amdgcn_iglp_opt
    {amdgcn_implicit_buffer_ptr, 4517}, // __builtin_amdgcn_implicit_buffer_ptr
    {amdgcn_implicitarg_ptr, 4537}, // __builtin_amdgcn_implicitarg_ptr
    {amdgcn_interp_mov, 4553}, // __builtin_amdgcn_interp_mov
    {amdgcn_interp_p1, 4564}, // __builtin_amdgcn_interp_p1
    {amdgcn_interp_p1_f16, 4574}, // __builtin_amdgcn_interp_p1_f16
    {amdgcn_interp_p2, 4588}, // __builtin_amdgcn_interp_p2
    {amdgcn_interp_p2_f16, 4598}, // __builtin_amdgcn_interp_p2_f16
    {amdgcn_is_private, 4612}, // __builtin_amdgcn_is_private
    {amdgcn_is_shared, 4623}, // __builtin_amdgcn_is_shared
    {amdgcn_kernarg_segment_ptr, 4633}, // __builtin_amdgcn_kernarg_segment_ptr
    {amdgcn_lerp, 4653}, // __builtin_amdgcn_lerp
    {amdgcn_mbcnt_hi, 4658}, // __builtin_amdgcn_mbcnt_hi
    {amdgcn_mbcnt_lo, 4667}, // __builtin_amdgcn_mbcnt_lo
    {amdgcn_mfma_f32_16x16x16bf16_1k, 4676}, // __builtin_amdgcn_mfma_f32_16x16x16bf16_1k
    {amdgcn_mfma_f32_16x16x16f16, 4701}, // __builtin_amdgcn_mfma_f32_16x16x16f16
    {amdgcn_mfma_f32_16x16x1f32, 4722}, // __builtin_amdgcn_mfma_f32_16x16x1f32
    {amdgcn_mfma_f32_16x16x2bf16, 4742}, // __builtin_amdgcn_mfma_f32_16x16x2bf16
    {amdgcn_mfma_f32_16x16x32_bf16, 4763}, // __builtin_amdgcn_mfma_f32_16x16x32_bf16
    {amdgcn_mfma_f32_16x16x32_bf8_bf8, 4786}, // __builtin_amdgcn_mfma_f32_16x16x32_bf8_bf8
    {amdgcn_mfma_f32_16x16x32_bf8_fp8, 4812}, // __builtin_amdgcn_mfma_f32_16x16x32_bf8_fp8
    {amdgcn_mfma_f32_16x16x32_f16, 4838}, // __builtin_amdgcn_mfma_f32_16x16x32_f16
    {amdgcn_mfma_f32_16x16x32_fp8_bf8, 4860}, // __builtin_amdgcn_mfma_f32_16x16x32_fp8_bf8
    {amdgcn_mfma_f32_16x16x32_fp8_fp8, 4886}, // __builtin_amdgcn_mfma_f32_16x16x32_fp8_fp8
    {amdgcn_mfma_f32_16x16x4bf16_1k, 4912}, // __builtin_amdgcn_mfma_f32_16x16x4bf16_1k
    {amdgcn_mfma_f32_16x16x4f16, 4936}, // __builtin_amdgcn_mfma_f32_16x16x4f16
    {amdgcn_mfma_f32_16x16x4f32, 4956}, // __builtin_amdgcn_mfma_f32_16x16x4f32
    {amdgcn_mfma_f32_16x16x8_xf32, 4976}, // __builtin_amdgcn_mfma_f32_16x16x8_xf32
    {amdgcn_mfma_f32_16x16x8bf16, 4998}, // __builtin_amdgcn_mfma_f32_16x16x8bf16
    {amdgcn_mfma_f32_32x32x16_bf16, 5019}, // __builtin_amdgcn_mfma_f32_32x32x16_bf16
    {amdgcn_mfma_f32_32x32x16_bf8_bf8, 5042}, // __builtin_amdgcn_mfma_f32_32x32x16_bf8_bf8
    {amdgcn_mfma_f32_32x32x16_bf8_fp8, 5068}, // __builtin_amdgcn_mfma_f32_32x32x16_bf8_fp8
    {amdgcn_mfma_f32_32x32x16_f16, 5094}, // __builtin_amdgcn_mfma_f32_32x32x16_f16
    {amdgcn_mfma_f32_32x32x16_fp8_bf8, 5116}, // __builtin_amdgcn_mfma_f32_32x32x16_fp8_bf8
    {amdgcn_mfma_f32_32x32x16_fp8_fp8, 5142}, // __builtin_amdgcn_mfma_f32_32x32x16_fp8_fp8
    {amdgcn_mfma_f32_32x32x1f32, 5168}, // __builtin_amdgcn_mfma_f32_32x32x1f32
    {amdgcn_mfma_f32_32x32x2bf16, 5188}, // __builtin_amdgcn_mfma_f32_32x32x2bf16
    {amdgcn_mfma_f32_32x32x2f32, 5209}, // __builtin_amdgcn_mfma_f32_32x32x2f32
    {amdgcn_mfma_f32_32x32x4_xf32, 5229}, // __builtin_amdgcn_mfma_f32_32x32x4_xf32
    {amdgcn_mfma_f32_32x32x4bf16, 5251}, // __builtin_amdgcn_mfma_f32_32x32x4bf16
    {amdgcn_mfma_f32_32x32x4bf16_1k, 5272}, // __builtin_amdgcn_mfma_f32_32x32x4bf16_1k
    {amdgcn_mfma_f32_32x32x4f16, 5296}, // __builtin_amdgcn_mfma_f32_32x32x4f16
    {amdgcn_mfma_f32_32x32x8bf16_1k, 5316}, // __builtin_amdgcn_mfma_f32_32x32x8bf16_1k
    {amdgcn_mfma_f32_32x32x8f16, 5340}, // __builtin_amdgcn_mfma_f32_32x32x8f16
    {amdgcn_mfma_f32_4x4x1f32, 5360}, // __builtin_amdgcn_mfma_f32_4x4x1f32
    {amdgcn_mfma_f32_4x4x2bf16, 5378}, // __builtin_amdgcn_mfma_f32_4x4x2bf16
    {amdgcn_mfma_f32_4x4x4bf16_1k, 5397}, // __builtin_amdgcn_mfma_f32_4x4x4bf16_1k
    {amdgcn_mfma_f32_4x4x4f16, 5419}, // __builtin_amdgcn_mfma_f32_4x4x4f16
    {amdgcn_mfma_f64_16x16x4f64, 5437}, // __builtin_amdgcn_mfma_f64_16x16x4f64
    {amdgcn_mfma_f64_4x4x4f64, 5457}, // __builtin_amdgcn_mfma_f64_4x4x4f64
    {amdgcn_mfma_i32_16x16x16i8, 5475}, // __builtin_amdgcn_mfma_i32_16x16x16i8
    {amdgcn_mfma_i32_16x16x32_i8, 5495}, // __builtin_amdgcn_mfma_i32_16x16x32_i8
    {amdgcn_mfma_i32_16x16x4i8, 5516}, // __builtin_amdgcn_mfma_i32_16x16x4i8
    {amdgcn_mfma_i32_16x16x64_i8, 5535}, // __builtin_amdgcn_mfma_i32_16x16x64_i8
    {amdgcn_mfma_i32_32x32x16_i8, 5556}, // __builtin_amdgcn_mfma_i32_32x32x16_i8
    {amdgcn_mfma_i32_32x32x32_i8, 5577}, // __builtin_amdgcn_mfma_i32_32x32x32_i8
    {amdgcn_mfma_i32_32x32x4i8, 5598}, // __builtin_amdgcn_mfma_i32_32x32x4i8
    {amdgcn_mfma_i32_32x32x8i8, 5617}, // __builtin_amdgcn_mfma_i32_32x32x8i8
    {amdgcn_mfma_i32_4x4x4i8, 5636}, // __builtin_amdgcn_mfma_i32_4x4x4i8
    {amdgcn_mqsad_pk_u16_u8, 5653}, // __builtin_amdgcn_mqsad_pk_u16_u8
    {amdgcn_mqsad_u32_u8, 5669}, // __builtin_amdgcn_mqsad_u32_u8
    {amdgcn_msad_u8, 5682}, // __builtin_amdgcn_msad_u8
    {amdgcn_perm, 5690}, // __builtin_amdgcn_perm
    {amdgcn_perm_pk16_b4_u4, 5695}, // __builtin_amdgcn_perm_pk16_b4_u4
    {amdgcn_perm_pk16_b6_u4, 5711}, // __builtin_amdgcn_perm_pk16_b6_u4
    {amdgcn_perm_pk16_b8_u4, 5727}, // __builtin_amdgcn_perm_pk16_b8_u4
    {amdgcn_permlane16_var, 5743}, // __builtin_amdgcn_permlane16_var
    {amdgcn_permlane_bcast, 5758}, // __builtin_amdgcn_permlane_bcast
    {amdgcn_permlane_down, 5773}, // __builtin_amdgcn_permlane_down
    {amdgcn_permlane_idx_gen, 5787}, // __builtin_amdgcn_permlane_idx_gen
    {amdgcn_permlane_up, 5804}, // __builtin_amdgcn_permlane_up
    {amdgcn_permlane_xor, 5816}, // __builtin_amdgcn_permlane_xor
    {amdgcn_permlanex16_var, 5829}, // __builtin_amdgcn_permlanex16_var
    {amdgcn_pk_add_max_i16, 5845}, // __builtin_amdgcn_pk_add_max_i16
    {amdgcn_pk_add_max_u16, 5860}, // __builtin_amdgcn_pk_add_max_u16
    {amdgcn_pk_add_min_i16, 5875}, // __builtin_amdgcn_pk_add_min_i16
    {amdgcn_pk_add_min_u16, 5890}, // __builtin_amdgcn_pk_add_min_u16
    {amdgcn_prng_b32, 5905}, // __builtin_amdgcn_prng_b32
    {amdgcn_qsad_pk_u16_u8, 5914}, // __builtin_amdgcn_qsad_pk_u16_u8
    {amdgcn_queue_ptr, 5929}, // __builtin_amdgcn_queue_ptr
    {amdgcn_raw_ptr_buffer_load_lds, 5939}, // __builtin_amdgcn_raw_ptr_buffer_load_lds
    {amdgcn_rcp_legacy, 5963}, // __builtin_amdgcn_rcp_legacy
    {amdgcn_rsq_legacy, 5974}, // __builtin_amdgcn_rsq_legacy
    {amdgcn_s_barrier, 5985}, // __builtin_amdgcn_s_barrier
    {amdgcn_s_barrier_init, 5995}, // __builtin_amdgcn_s_barrier_init
    {amdgcn_s_barrier_join, 6010}, // __builtin_amdgcn_s_barrier_join
    {amdgcn_s_barrier_leave, 6025}, // __builtin_amdgcn_s_barrier_leave
    {amdgcn_s_barrier_signal, 6041}, // __builtin_amdgcn_s_barrier_signal
    {amdgcn_s_barrier_signal_isfirst, 6058}, // __builtin_amdgcn_s_barrier_signal_isfirst
    {amdgcn_s_barrier_signal_var, 6083}, // __builtin_amdgcn_s_barrier_signal_var
    {amdgcn_s_barrier_wait, 6104}, // __builtin_amdgcn_s_barrier_wait
    {amdgcn_s_buffer_prefetch_data, 6119}, // __builtin_amdgcn_s_buffer_prefetch_data
    {amdgcn_s_cluster_barrier, 6142}, // __builtin_amdgcn_s_cluster_barrier
    {amdgcn_s_dcache_inv, 6160}, // __builtin_amdgcn_s_dcache_inv
    {amdgcn_s_dcache_inv_vol, 6173}, // __builtin_amdgcn_s_dcache_inv_vol
    {amdgcn_s_dcache_wb, 6190}, // __builtin_amdgcn_s_dcache_wb
    {amdgcn_s_dcache_wb_vol, 6202}, // __builtin_amdgcn_s_dcache_wb_vol
    {amdgcn_s_decperflevel, 6218}, // __builtin_amdgcn_s_decperflevel
    {amdgcn_s_get_barrier_state, 6233}, // __builtin_amdgcn_s_get_barrier_state
    {amdgcn_s_get_named_barrier_state, 6253}, // __builtin_amdgcn_s_get_named_barrier_state
    {amdgcn_s_get_waveid_in_workgroup, 6279}, // __builtin_amdgcn_s_get_waveid_in_workgroup
    {amdgcn_s_getpc, 6305}, // __builtin_amdgcn_s_getpc
    {amdgcn_s_getreg, 6313}, // __builtin_amdgcn_s_getreg
    {amdgcn_s_incperflevel, 6322}, // __builtin_amdgcn_s_incperflevel
    {amdgcn_s_memrealtime, 6337}, // __builtin_amdgcn_s_memrealtime
    {amdgcn_s_memtime, 6351}, // __builtin_amdgcn_s_memtime
    {amdgcn_s_monitor_sleep, 6361}, // __builtin_amdgcn_s_monitor_sleep
    {amdgcn_s_sendmsg, 6377}, // __builtin_amdgcn_s_sendmsg
    {amdgcn_s_sendmsghalt, 6387}, // __builtin_amdgcn_s_sendmsghalt
    {amdgcn_s_setprio, 6401}, // __builtin_amdgcn_s_setprio
    {amdgcn_s_setprio_inc_wg, 6411}, // __builtin_amdgcn_s_setprio_inc_wg
    {amdgcn_s_setreg, 6428}, // __builtin_amdgcn_s_setreg
    {amdgcn_s_sleep, 6437}, // __builtin_amdgcn_s_sleep
    {amdgcn_s_sleep_var, 6445}, // __builtin_amdgcn_s_sleep_var
    {amdgcn_s_ttracedata, 6457}, // __builtin_amdgcn_s_ttracedata
    {amdgcn_s_ttracedata_imm, 6470}, // __builtin_amdgcn_s_ttracedata_imm
    {amdgcn_s_wait_asynccnt, 6487}, // __builtin_amdgcn_s_wait_asynccnt
    {amdgcn_s_wait_event_export_ready, 6503}, // __builtin_amdgcn_s_wait_event_export_ready
    {amdgcn_s_wait_tensorcnt, 6529}, // __builtin_amdgcn_s_wait_tensorcnt
    {amdgcn_s_waitcnt, 6546}, // __builtin_amdgcn_s_waitcnt
    {amdgcn_s_wakeup_barrier, 6556}, // __builtin_amdgcn_s_wakeup_barrier
    {amdgcn_sad_hi_u8, 6573}, // __builtin_amdgcn_sad_hi_u8
    {amdgcn_sad_u16, 6583}, // __builtin_amdgcn_sad_u16
    {amdgcn_sad_u8, 6591}, // __builtin_amdgcn_sad_u8
    {amdgcn_sat_pk4_i4_i8, 6598}, // __builtin_amdgcn_sat_pk4_i4_i8
    {amdgcn_sat_pk4_u4_u8, 6612}, // __builtin_amdgcn_sat_pk4_u4_u8
    {amdgcn_sched_barrier, 6626}, // __builtin_amdgcn_sched_barrier
    {amdgcn_sched_group_barrier, 6640}, // __builtin_amdgcn_sched_group_barrier
    {amdgcn_sdot2, 6660}, // __builtin_amdgcn_sdot2
    {amdgcn_sdot4, 6666}, // __builtin_amdgcn_sdot4
    {amdgcn_sdot8, 6672}, // __builtin_amdgcn_sdot8
    {amdgcn_smfmac_f32_16x16x128_bf8_bf8, 6678}, // __builtin_amdgcn_smfmac_f32_16x16x128_bf8_bf8
    {amdgcn_smfmac_f32_16x16x128_bf8_fp8, 6707}, // __builtin_amdgcn_smfmac_f32_16x16x128_bf8_fp8
    {amdgcn_smfmac_f32_16x16x128_fp8_bf8, 6736}, // __builtin_amdgcn_smfmac_f32_16x16x128_fp8_bf8
    {amdgcn_smfmac_f32_16x16x128_fp8_fp8, 6765}, // __builtin_amdgcn_smfmac_f32_16x16x128_fp8_fp8
    {amdgcn_smfmac_f32_16x16x32_bf16, 6794}, // __builtin_amdgcn_smfmac_f32_16x16x32_bf16
    {amdgcn_smfmac_f32_16x16x32_f16, 6819}, // __builtin_amdgcn_smfmac_f32_16x16x32_f16
    {amdgcn_smfmac_f32_16x16x64_bf16, 6843}, // __builtin_amdgcn_smfmac_f32_16x16x64_bf16
    {amdgcn_smfmac_f32_16x16x64_bf8_bf8, 6868}, // __builtin_amdgcn_smfmac_f32_16x16x64_bf8_bf8
    {amdgcn_smfmac_f32_16x16x64_bf8_fp8, 6896}, // __builtin_amdgcn_smfmac_f32_16x16x64_bf8_fp8
    {amdgcn_smfmac_f32_16x16x64_f16, 6924}, // __builtin_amdgcn_smfmac_f32_16x16x64_f16
    {amdgcn_smfmac_f32_16x16x64_fp8_bf8, 6948}, // __builtin_amdgcn_smfmac_f32_16x16x64_fp8_bf8
    {amdgcn_smfmac_f32_16x16x64_fp8_fp8, 6976}, // __builtin_amdgcn_smfmac_f32_16x16x64_fp8_fp8
    {amdgcn_smfmac_f32_32x32x16_bf16, 7004}, // __builtin_amdgcn_smfmac_f32_32x32x16_bf16
    {amdgcn_smfmac_f32_32x32x16_f16, 7029}, // __builtin_amdgcn_smfmac_f32_32x32x16_f16
    {amdgcn_smfmac_f32_32x32x32_bf16, 7053}, // __builtin_amdgcn_smfmac_f32_32x32x32_bf16
    {amdgcn_smfmac_f32_32x32x32_bf8_bf8, 7078}, // __builtin_amdgcn_smfmac_f32_32x32x32_bf8_bf8
    {amdgcn_smfmac_f32_32x32x32_bf8_fp8, 7106}, // __builtin_amdgcn_smfmac_f32_32x32x32_bf8_fp8
    {amdgcn_smfmac_f32_32x32x32_f16, 7134}, // __builtin_amdgcn_smfmac_f32_32x32x32_f16
    {amdgcn_smfmac_f32_32x32x32_fp8_bf8, 7158}, // __builtin_amdgcn_smfmac_f32_32x32x32_fp8_bf8
    {amdgcn_smfmac_f32_32x32x32_fp8_fp8, 7186}, // __builtin_amdgcn_smfmac_f32_32x32x32_fp8_fp8
    {amdgcn_smfmac_f32_32x32x64_bf8_bf8, 7214}, // __builtin_amdgcn_smfmac_f32_32x32x64_bf8_bf8
    {amdgcn_smfmac_f32_32x32x64_bf8_fp8, 7242}, // __builtin_amdgcn_smfmac_f32_32x32x64_bf8_fp8
    {amdgcn_smfmac_f32_32x32x64_fp8_bf8, 7270}, // __builtin_amdgcn_smfmac_f32_32x32x64_fp8_bf8
    {amdgcn_smfmac_f32_32x32x64_fp8_fp8, 7298}, // __builtin_amdgcn_smfmac_f32_32x32x64_fp8_fp8
    {amdgcn_smfmac_i32_16x16x128_i8, 7326}, // __builtin_amdgcn_smfmac_i32_16x16x128_i8
    {amdgcn_smfmac_i32_16x16x64_i8, 7350}, // __builtin_amdgcn_smfmac_i32_16x16x64_i8
    {amdgcn_smfmac_i32_32x32x32_i8, 7373}, // __builtin_amdgcn_smfmac_i32_32x32x32_i8
    {amdgcn_smfmac_i32_32x32x64_i8, 7396}, // __builtin_amdgcn_smfmac_i32_32x32x64_i8
    {amdgcn_struct_ptr_buffer_load_lds, 7419}, // __builtin_amdgcn_struct_ptr_buffer_load_lds
    {amdgcn_sudot4, 7446}, // __builtin_amdgcn_sudot4
    {amdgcn_sudot8, 7453}, // __builtin_amdgcn_sudot8
    {amdgcn_tensor_load_to_lds, 7460}, // __builtin_amdgcn_tensor_load_to_lds
    {amdgcn_tensor_load_to_lds_d2, 7479}, // __builtin_amdgcn_tensor_load_to_lds_d2
    {amdgcn_tensor_store_from_lds, 7501}, // __builtin_amdgcn_tensor_store_from_lds
    {amdgcn_tensor_store_from_lds_d2, 7523}, // __builtin_amdgcn_tensor_store_from_lds_d2
    {amdgcn_udot2, 7548}, // __builtin_amdgcn_udot2
    {amdgcn_udot4, 7554}, // __builtin_amdgcn_udot4
    {amdgcn_udot8, 7560}, // __builtin_amdgcn_udot8
    {amdgcn_wave_barrier, 7566}, // __builtin_amdgcn_wave_barrier
    {amdgcn_wavefrontsize, 7579}, // __builtin_amdgcn_wavefrontsize
    {amdgcn_workgroup_id_x, 7593}, // __builtin_amdgcn_workgroup_id_x
    {amdgcn_workgroup_id_y, 7608}, // __builtin_amdgcn_workgroup_id_y
    {amdgcn_workgroup_id_z, 7623}, // __builtin_amdgcn_workgroup_id_z
    {amdgcn_workitem_id_x, 7638}, // __builtin_amdgcn_workitem_id_x
    {amdgcn_workitem_id_y, 7652}, // __builtin_amdgcn_workitem_id_y
    {amdgcn_workitem_id_z, 7666}, // __builtin_amdgcn_workitem_id_z
  }; // amdgcnNames

  // Builtins for arm.
  static constexpr BuiltinEntry armNames[] = {
    {arm_cdp, 7680}, // __builtin_arm_cdp
    {arm_cdp2, 7684}, // __builtin_arm_cdp2
    {arm_cmse_tt, 7689}, // __builtin_arm_cmse_TT
    {arm_cmse_tta, 7697}, // __builtin_arm_cmse_TTA
    {arm_cmse_ttat, 7706}, // __builtin_arm_cmse_TTAT
    {arm_cmse_ttt, 7716}, // __builtin_arm_cmse_TTT
    {arm_dmb, 7725}, // __builtin_arm_dmb
    {arm_dsb, 7729}, // __builtin_arm_dsb
    {arm_get_fpscr, 7733}, // __builtin_arm_get_fpscr
    {arm_isb, 7743}, // __builtin_arm_isb
    {arm_ldc, 7747}, // __builtin_arm_ldc
    {arm_ldc2, 7751}, // __builtin_arm_ldc2
    {arm_ldc2l, 7756}, // __builtin_arm_ldc2l
    {arm_ldcl, 7762}, // __builtin_arm_ldcl
    {arm_mcr, 7767}, // __builtin_arm_mcr
    {arm_mcr2, 7771}, // __builtin_arm_mcr2
    {arm_mrc, 7776}, // __builtin_arm_mrc
    {arm_mrc2, 7780}, // __builtin_arm_mrc2
    {arm_qadd, 7785}, // __builtin_arm_qadd
    {arm_qadd16, 7790}, // __builtin_arm_qadd16
    {arm_qadd8, 7797}, // __builtin_arm_qadd8
    {arm_qasx, 7803}, // __builtin_arm_qasx
    {arm_qsax, 7808}, // __builtin_arm_qsax
    {arm_qsub, 7813}, // __builtin_arm_qsub
    {arm_qsub16, 7818}, // __builtin_arm_qsub16
    {arm_qsub8, 7825}, // __builtin_arm_qsub8
    {arm_sadd16, 7831}, // __builtin_arm_sadd16
    {arm_sadd8, 7838}, // __builtin_arm_sadd8
    {arm_sasx, 7844}, // __builtin_arm_sasx
    {arm_sel, 7849}, // __builtin_arm_sel
    {arm_set_fpscr, 7853}, // __builtin_arm_set_fpscr
    {arm_shadd16, 7863}, // __builtin_arm_shadd16
    {arm_shadd8, 7871}, // __builtin_arm_shadd8
    {arm_shasx, 7878}, // __builtin_arm_shasx
    {arm_shsax, 7884}, // __builtin_arm_shsax
    {arm_shsub16, 7890}, // __builtin_arm_shsub16
    {arm_shsub8, 7898}, // __builtin_arm_shsub8
    {arm_smlabb, 7905}, // __builtin_arm_smlabb
    {arm_smlabt, 7912}, // __builtin_arm_smlabt
    {arm_smlad, 7919}, // __builtin_arm_smlad
    {arm_smladx, 7925}, // __builtin_arm_smladx
    {arm_smlald, 7932}, // __builtin_arm_smlald
    {arm_smlaldx, 7939}, // __builtin_arm_smlaldx
    {arm_smlatb, 7947}, // __builtin_arm_smlatb
    {arm_smlatt, 7954}, // __builtin_arm_smlatt
    {arm_smlawb, 7961}, // __builtin_arm_smlawb
    {arm_smlawt, 7968}, // __builtin_arm_smlawt
    {arm_smlsd, 7975}, // __builtin_arm_smlsd
    {arm_smlsdx, 7981}, // __builtin_arm_smlsdx
    {arm_smlsld, 7988}, // __builtin_arm_smlsld
    {arm_smlsldx, 7995}, // __builtin_arm_smlsldx
    {arm_smuad, 8003}, // __builtin_arm_smuad
    {arm_smuadx, 8009}, // __builtin_arm_smuadx
    {arm_smulbb, 8016}, // __builtin_arm_smulbb
    {arm_smulbt, 8023}, // __builtin_arm_smulbt
    {arm_smultb, 8030}, // __builtin_arm_smultb
    {arm_smultt, 8037}, // __builtin_arm_smultt
    {arm_smulwb, 8044}, // __builtin_arm_smulwb
    {arm_smulwt, 8051}, // __builtin_arm_smulwt
    {arm_smusd, 8058}, // __builtin_arm_smusd
    {arm_smusdx, 8064}, // __builtin_arm_smusdx
    {arm_ssat, 8071}, // __builtin_arm_ssat
    {arm_ssat16, 8076}, // __builtin_arm_ssat16
    {arm_ssax, 8083}, // __builtin_arm_ssax
    {arm_ssub16, 8088}, // __builtin_arm_ssub16
    {arm_ssub8, 8095}, // __builtin_arm_ssub8
    {arm_stc, 8101}, // __builtin_arm_stc
    {arm_stc2, 8105}, // __builtin_arm_stc2
    {arm_stc2l, 8110}, // __builtin_arm_stc2l
    {arm_stcl, 8116}, // __builtin_arm_stcl
    {arm_sxtab16, 8121}, // __builtin_arm_sxtab16
    {arm_sxtb16, 8129}, // __builtin_arm_sxtb16
    {arm_uadd16, 8136}, // __builtin_arm_uadd16
    {arm_uadd8, 8143}, // __builtin_arm_uadd8
    {arm_uasx, 8149}, // __builtin_arm_uasx
    {arm_uhadd16, 8154}, // __builtin_arm_uhadd16
    {arm_uhadd8, 8162}, // __builtin_arm_uhadd8
    {arm_uhasx, 8169}, // __builtin_arm_uhasx
    {arm_uhsax, 8175}, // __builtin_arm_uhsax
    {arm_uhsub16, 8181}, // __builtin_arm_uhsub16
    {arm_uhsub8, 8189}, // __builtin_arm_uhsub8
    {arm_uqadd16, 8196}, // __builtin_arm_uqadd16
    {arm_uqadd8, 8204}, // __builtin_arm_uqadd8
    {arm_uqasx, 8211}, // __builtin_arm_uqasx
    {arm_uqsax, 8217}, // __builtin_arm_uqsax
    {arm_uqsub16, 8223}, // __builtin_arm_uqsub16
    {arm_uqsub8, 8231}, // __builtin_arm_uqsub8
    {arm_usad8, 8238}, // __builtin_arm_usad8
    {arm_usada8, 8244}, // __builtin_arm_usada8
    {arm_usat, 8251}, // __builtin_arm_usat
    {arm_usat16, 8256}, // __builtin_arm_usat16
    {arm_usax, 8263}, // __builtin_arm_usax
    {arm_usub16, 8268}, // __builtin_arm_usub16
    {arm_usub8, 8275}, // __builtin_arm_usub8
    {arm_uxtab16, 8281}, // __builtin_arm_uxtab16
    {arm_uxtb16, 8289}, // __builtin_arm_uxtb16
  }; // armNames

  // Builtins for bpf.
  static constexpr BuiltinEntry bpfNames[] = {
    {bpf_btf_type_id, 8296}, // __builtin_bpf_btf_type_id
    {bpf_compare, 8308}, // __builtin_bpf_compare
    {bpf_getelementptr_and_load, 8316}, // __builtin_bpf_getelementptr_and_load
    {bpf_getelementptr_and_store, 8339}, // __builtin_bpf_getelementptr_and_store
    {bpf_load_byte, 8363}, // __builtin_bpf_load_byte
    {bpf_load_half, 8373}, // __builtin_bpf_load_half
    {bpf_load_word, 8383}, // __builtin_bpf_load_word
    {bpf_passthrough, 8393}, // __builtin_bpf_passthrough
    {bpf_preserve_enum_value, 8405}, // __builtin_bpf_preserve_enum_value
    {bpf_preserve_field_info, 8425}, // __builtin_bpf_preserve_field_info
    {bpf_preserve_type_info, 8445}, // __builtin_bpf_preserve_type_info
    {bpf_pseudo, 8464}, // __builtin_bpf_pseudo
  }; // bpfNames

  // Builtins for hexagon.
  static constexpr BuiltinEntry hexagonNames[] = {
    {hexagon_A2_abs, 8471}, // __builtin_HEXAGON_A2_abs
    {hexagon_A2_absp, 8486}, // __builtin_HEXAGON_A2_absp
    {hexagon_A2_abssat, 8502}, // __builtin_HEXAGON_A2_abssat
    {hexagon_A2_add, 8520}, // __builtin_HEXAGON_A2_add
    {hexagon_A2_addh_h16_hh, 8535}, // __builtin_HEXAGON_A2_addh_h16_hh
    {hexagon_A2_addh_h16_hl, 8558}, // __builtin_HEXAGON_A2_addh_h16_hl
    {hexagon_A2_addh_h16_lh, 8581}, // __builtin_HEXAGON_A2_addh_h16_lh
    {hexagon_A2_addh_h16_ll, 8604}, // __builtin_HEXAGON_A2_addh_h16_ll
    {hexagon_A2_addh_h16_sat_hh, 8627}, // __builtin_HEXAGON_A2_addh_h16_sat_hh
    {hexagon_A2_addh_h16_sat_hl, 8654}, // __builtin_HEXAGON_A2_addh_h16_sat_hl
    {hexagon_A2_addh_h16_sat_lh, 8681}, // __builtin_HEXAGON_A2_addh_h16_sat_lh
    {hexagon_A2_addh_h16_sat_ll, 8708}, // __builtin_HEXAGON_A2_addh_h16_sat_ll
    {hexagon_A2_addh_l16_hl, 8735}, // __builtin_HEXAGON_A2_addh_l16_hl
    {hexagon_A2_addh_l16_ll, 8758}, // __builtin_HEXAGON_A2_addh_l16_ll
    {hexagon_A2_addh_l16_sat_hl, 8781}, // __builtin_HEXAGON_A2_addh_l16_sat_hl
    {hexagon_A2_addh_l16_sat_ll, 8808}, // __builtin_HEXAGON_A2_addh_l16_sat_ll
    {hexagon_A2_addi, 8835}, // __builtin_HEXAGON_A2_addi
    {hexagon_A2_addp, 8851}, // __builtin_HEXAGON_A2_addp
    {hexagon_A2_addpsat, 8867}, // __builtin_HEXAGON_A2_addpsat
    {hexagon_A2_addsat, 8886}, // __builtin_HEXAGON_A2_addsat
    {hexagon_A2_addsp, 8904}, // __builtin_HEXAGON_A2_addsp
    {hexagon_A2_and, 8921}, // __builtin_HEXAGON_A2_and
    {hexagon_A2_andir, 8936}, // __builtin_HEXAGON_A2_andir
    {hexagon_A2_andp, 8953}, // __builtin_HEXAGON_A2_andp
    {hexagon_A2_aslh, 8969}, // __builtin_HEXAGON_A2_aslh
    {hexagon_A2_asrh, 8985}, // __builtin_HEXAGON_A2_asrh
    {hexagon_A2_combine_hh, 9001}, // __builtin_HEXAGON_A2_combine_hh
    {hexagon_A2_combine_hl, 9023}, // __builtin_HEXAGON_A2_combine_hl
    {hexagon_A2_combine_lh, 9045}, // __builtin_HEXAGON_A2_combine_lh
    {hexagon_A2_combine_ll, 9067}, // __builtin_HEXAGON_A2_combine_ll
    {hexagon_A2_combineii, 9089}, // __builtin_HEXAGON_A2_combineii
    {hexagon_A2_combinew, 9110}, // __builtin_HEXAGON_A2_combinew
    {hexagon_A2_max, 9130}, // __builtin_HEXAGON_A2_max
    {hexagon_A2_maxp, 9145}, // __builtin_HEXAGON_A2_maxp
    {hexagon_A2_maxu, 9161}, // __builtin_HEXAGON_A2_maxu
    {hexagon_A2_maxup, 9177}, // __builtin_HEXAGON_A2_maxup
    {hexagon_A2_min, 9194}, // __builtin_HEXAGON_A2_min
    {hexagon_A2_minp, 9209}, // __builtin_HEXAGON_A2_minp
    {hexagon_A2_minu, 9225}, // __builtin_HEXAGON_A2_minu
    {hexagon_A2_minup, 9241}, // __builtin_HEXAGON_A2_minup
    {hexagon_A2_neg, 9258}, // __builtin_HEXAGON_A2_neg
    {hexagon_A2_negp, 9273}, // __builtin_HEXAGON_A2_negp
    {hexagon_A2_negsat, 9289}, // __builtin_HEXAGON_A2_negsat
    {hexagon_A2_not, 9307}, // __builtin_HEXAGON_A2_not
    {hexagon_A2_notp, 9322}, // __builtin_HEXAGON_A2_notp
    {hexagon_A2_or, 9338}, // __builtin_HEXAGON_A2_or
    {hexagon_A2_orir, 9352}, // __builtin_HEXAGON_A2_orir
    {hexagon_A2_orp, 9368}, // __builtin_HEXAGON_A2_orp
    {hexagon_A2_roundsat, 9383}, // __builtin_HEXAGON_A2_roundsat
    {hexagon_A2_sat, 9403}, // __builtin_HEXAGON_A2_sat
    {hexagon_A2_satb, 9418}, // __builtin_HEXAGON_A2_satb
    {hexagon_A2_sath, 9434}, // __builtin_HEXAGON_A2_sath
    {hexagon_A2_satub, 9450}, // __builtin_HEXAGON_A2_satub
    {hexagon_A2_satuh, 9467}, // __builtin_HEXAGON_A2_satuh
    {hexagon_A2_sub, 9484}, // __builtin_HEXAGON_A2_sub
    {hexagon_A2_subh_h16_hh, 9499}, // __builtin_HEXAGON_A2_subh_h16_hh
    {hexagon_A2_subh_h16_hl, 9522}, // __builtin_HEXAGON_A2_subh_h16_hl
    {hexagon_A2_subh_h16_lh, 9545}, // __builtin_HEXAGON_A2_subh_h16_lh
    {hexagon_A2_subh_h16_ll, 9568}, // __builtin_HEXAGON_A2_subh_h16_ll
    {hexagon_A2_subh_h16_sat_hh, 9591}, // __builtin_HEXAGON_A2_subh_h16_sat_hh
    {hexagon_A2_subh_h16_sat_hl, 9618}, // __builtin_HEXAGON_A2_subh_h16_sat_hl
    {hexagon_A2_subh_h16_sat_lh, 9645}, // __builtin_HEXAGON_A2_subh_h16_sat_lh
    {hexagon_A2_subh_h16_sat_ll, 9672}, // __builtin_HEXAGON_A2_subh_h16_sat_ll
    {hexagon_A2_subh_l16_hl, 9699}, // __builtin_HEXAGON_A2_subh_l16_hl
    {hexagon_A2_subh_l16_ll, 9722}, // __builtin_HEXAGON_A2_subh_l16_ll
    {hexagon_A2_subh_l16_sat_hl, 9745}, // __builtin_HEXAGON_A2_subh_l16_sat_hl
    {hexagon_A2_subh_l16_sat_ll, 9772}, // __builtin_HEXAGON_A2_subh_l16_sat_ll
    {hexagon_A2_subp, 9799}, // __builtin_HEXAGON_A2_subp
    {hexagon_A2_subri, 9815}, // __builtin_HEXAGON_A2_subri
    {hexagon_A2_subsat, 9832}, // __builtin_HEXAGON_A2_subsat
    {hexagon_A2_svaddh, 9850}, // __builtin_HEXAGON_A2_svaddh
    {hexagon_A2_svaddhs, 9868}, // __builtin_HEXAGON_A2_svaddhs
    {hexagon_A2_svadduhs, 9887}, // __builtin_HEXAGON_A2_svadduhs
    {hexagon_A2_svavgh, 9907}, // __builtin_HEXAGON_A2_svavgh
    {hexagon_A2_svavghs, 9925}, // __builtin_HEXAGON_A2_svavghs
    {hexagon_A2_svnavgh, 9944}, // __builtin_HEXAGON_A2_svnavgh
    {hexagon_A2_svsubh, 9963}, // __builtin_HEXAGON_A2_svsubh
    {hexagon_A2_svsubhs, 9981}, // __builtin_HEXAGON_A2_svsubhs
    {hexagon_A2_svsubuhs, 10000}, // __builtin_HEXAGON_A2_svsubuhs
    {hexagon_A2_swiz, 10020}, // __builtin_HEXAGON_A2_swiz
    {hexagon_A2_sxtb, 10036}, // __builtin_HEXAGON_A2_sxtb
    {hexagon_A2_sxth, 10052}, // __builtin_HEXAGON_A2_sxth
    {hexagon_A2_sxtw, 10068}, // __builtin_HEXAGON_A2_sxtw
    {hexagon_A2_tfr, 10084}, // __builtin_HEXAGON_A2_tfr
    {hexagon_A2_tfrih, 10099}, // __builtin_HEXAGON_A2_tfrih
    {hexagon_A2_tfril, 10116}, // __builtin_HEXAGON_A2_tfril
    {hexagon_A2_tfrp, 10133}, // __builtin_HEXAGON_A2_tfrp
    {hexagon_A2_tfrpi, 10149}, // __builtin_HEXAGON_A2_tfrpi
    {hexagon_A2_tfrsi, 10166}, // __builtin_HEXAGON_A2_tfrsi
    {hexagon_A2_vabsh, 10183}, // __builtin_HEXAGON_A2_vabsh
    {hexagon_A2_vabshsat, 10200}, // __builtin_HEXAGON_A2_vabshsat
    {hexagon_A2_vabsw, 10220}, // __builtin_HEXAGON_A2_vabsw
    {hexagon_A2_vabswsat, 10237}, // __builtin_HEXAGON_A2_vabswsat
    {hexagon_A2_vaddb_map, 10257}, // __builtin_HEXAGON_A2_vaddb_map
    {hexagon_A2_vaddh, 10278}, // __builtin_HEXAGON_A2_vaddh
    {hexagon_A2_vaddhs, 10295}, // __builtin_HEXAGON_A2_vaddhs
    {hexagon_A2_vaddub, 10313}, // __builtin_HEXAGON_A2_vaddub
    {hexagon_A2_vaddubs, 10331}, // __builtin_HEXAGON_A2_vaddubs
    {hexagon_A2_vadduhs, 10350}, // __builtin_HEXAGON_A2_vadduhs
    {hexagon_A2_vaddw, 10369}, // __builtin_HEXAGON_A2_vaddw
    {hexagon_A2_vaddws, 10386}, // __builtin_HEXAGON_A2_vaddws
    {hexagon_A2_vavgh, 10404}, // __builtin_HEXAGON_A2_vavgh
    {hexagon_A2_vavghcr, 10421}, // __builtin_HEXAGON_A2_vavghcr
    {hexagon_A2_vavghr, 10440}, // __builtin_HEXAGON_A2_vavghr
    {hexagon_A2_vavgub, 10458}, // __builtin_HEXAGON_A2_vavgub
    {hexagon_A2_vavgubr, 10476}, // __builtin_HEXAGON_A2_vavgubr
    {hexagon_A2_vavguh, 10495}, // __builtin_HEXAGON_A2_vavguh
    {hexagon_A2_vavguhr, 10513}, // __builtin_HEXAGON_A2_vavguhr
    {hexagon_A2_vavguw, 10532}, // __builtin_HEXAGON_A2_vavguw
    {hexagon_A2_vavguwr, 10550}, // __builtin_HEXAGON_A2_vavguwr
    {hexagon_A2_vavgw, 10569}, // __builtin_HEXAGON_A2_vavgw
    {hexagon_A2_vavgwcr, 10586}, // __builtin_HEXAGON_A2_vavgwcr
    {hexagon_A2_vavgwr, 10605}, // __builtin_HEXAGON_A2_vavgwr
    {hexagon_A2_vcmpbeq, 10623}, // __builtin_HEXAGON_A2_vcmpbeq
    {hexagon_A2_vcmpbgtu, 10642}, // __builtin_HEXAGON_A2_vcmpbgtu
    {hexagon_A2_vcmpheq, 10662}, // __builtin_HEXAGON_A2_vcmpheq
    {hexagon_A2_vcmphgt, 10681}, // __builtin_HEXAGON_A2_vcmphgt
    {hexagon_A2_vcmphgtu, 10700}, // __builtin_HEXAGON_A2_vcmphgtu
    {hexagon_A2_vcmpweq, 10720}, // __builtin_HEXAGON_A2_vcmpweq
    {hexagon_A2_vcmpwgt, 10739}, // __builtin_HEXAGON_A2_vcmpwgt
    {hexagon_A2_vcmpwgtu, 10758}, // __builtin_HEXAGON_A2_vcmpwgtu
    {hexagon_A2_vconj, 10778}, // __builtin_HEXAGON_A2_vconj
    {hexagon_A2_vmaxb, 10795}, // __builtin_HEXAGON_A2_vmaxb
    {hexagon_A2_vmaxh, 10812}, // __builtin_HEXAGON_A2_vmaxh
    {hexagon_A2_vmaxub, 10829}, // __builtin_HEXAGON_A2_vmaxub
    {hexagon_A2_vmaxuh, 10847}, // __builtin_HEXAGON_A2_vmaxuh
    {hexagon_A2_vmaxuw, 10865}, // __builtin_HEXAGON_A2_vmaxuw
    {hexagon_A2_vmaxw, 10883}, // __builtin_HEXAGON_A2_vmaxw
    {hexagon_A2_vminb, 10900}, // __builtin_HEXAGON_A2_vminb
    {hexagon_A2_vminh, 10917}, // __builtin_HEXAGON_A2_vminh
    {hexagon_A2_vminub, 10934}, // __builtin_HEXAGON_A2_vminub
    {hexagon_A2_vminuh, 10952}, // __builtin_HEXAGON_A2_vminuh
    {hexagon_A2_vminuw, 10970}, // __builtin_HEXAGON_A2_vminuw
    {hexagon_A2_vminw, 10988}, // __builtin_HEXAGON_A2_vminw
    {hexagon_A2_vnavgh, 11005}, // __builtin_HEXAGON_A2_vnavgh
    {hexagon_A2_vnavghcr, 11023}, // __builtin_HEXAGON_A2_vnavghcr
    {hexagon_A2_vnavghr, 11043}, // __builtin_HEXAGON_A2_vnavghr
    {hexagon_A2_vnavgw, 11062}, // __builtin_HEXAGON_A2_vnavgw
    {hexagon_A2_vnavgwcr, 11080}, // __builtin_HEXAGON_A2_vnavgwcr
    {hexagon_A2_vnavgwr, 11100}, // __builtin_HEXAGON_A2_vnavgwr
    {hexagon_A2_vraddub, 11119}, // __builtin_HEXAGON_A2_vraddub
    {hexagon_A2_vraddub_acc, 11138}, // __builtin_HEXAGON_A2_vraddub_acc
    {hexagon_A2_vrsadub, 11161}, // __builtin_HEXAGON_A2_vrsadub
    {hexagon_A2_vrsadub_acc, 11180}, // __builtin_HEXAGON_A2_vrsadub_acc
    {hexagon_A2_vsubb_map, 11203}, // __builtin_HEXAGON_A2_vsubb_map
    {hexagon_A2_vsubh, 11224}, // __builtin_HEXAGON_A2_vsubh
    {hexagon_A2_vsubhs, 11241}, // __builtin_HEXAGON_A2_vsubhs
    {hexagon_A2_vsubub, 11259}, // __builtin_HEXAGON_A2_vsubub
    {hexagon_A2_vsububs, 11277}, // __builtin_HEXAGON_A2_vsububs
    {hexagon_A2_vsubuhs, 11296}, // __builtin_HEXAGON_A2_vsubuhs
    {hexagon_A2_vsubw, 11315}, // __builtin_HEXAGON_A2_vsubw
    {hexagon_A2_vsubws, 11332}, // __builtin_HEXAGON_A2_vsubws
    {hexagon_A2_xor, 11350}, // __builtin_HEXAGON_A2_xor
    {hexagon_A2_xorp, 11365}, // __builtin_HEXAGON_A2_xorp
    {hexagon_A2_zxtb, 11381}, // __builtin_HEXAGON_A2_zxtb
    {hexagon_A2_zxth, 11397}, // __builtin_HEXAGON_A2_zxth
    {hexagon_A4_andn, 11413}, // __builtin_HEXAGON_A4_andn
    {hexagon_A4_andnp, 11429}, // __builtin_HEXAGON_A4_andnp
    {hexagon_A4_bitsplit, 11446}, // __builtin_HEXAGON_A4_bitsplit
    {hexagon_A4_bitspliti, 11466}, // __builtin_HEXAGON_A4_bitspliti
    {hexagon_A4_boundscheck, 11487}, // __builtin_HEXAGON_A4_boundscheck
    {hexagon_A4_cmpbeq, 11510}, // __builtin_HEXAGON_A4_cmpbeq
    {hexagon_A4_cmpbeqi, 11528}, // __builtin_HEXAGON_A4_cmpbeqi
    {hexagon_A4_cmpbgt, 11547}, // __builtin_HEXAGON_A4_cmpbgt
    {hexagon_A4_cmpbgti, 11565}, // __builtin_HEXAGON_A4_cmpbgti
    {hexagon_A4_cmpbgtu, 11584}, // __builtin_HEXAGON_A4_cmpbgtu
    {hexagon_A4_cmpbgtui, 11603}, // __builtin_HEXAGON_A4_cmpbgtui
    {hexagon_A4_cmpheq, 11623}, // __builtin_HEXAGON_A4_cmpheq
    {hexagon_A4_cmpheqi, 11641}, // __builtin_HEXAGON_A4_cmpheqi
    {hexagon_A4_cmphgt, 11660}, // __builtin_HEXAGON_A4_cmphgt
    {hexagon_A4_cmphgti, 11678}, // __builtin_HEXAGON_A4_cmphgti
    {hexagon_A4_cmphgtu, 11697}, // __builtin_HEXAGON_A4_cmphgtu
    {hexagon_A4_cmphgtui, 11716}, // __builtin_HEXAGON_A4_cmphgtui
    {hexagon_A4_combineir, 11736}, // __builtin_HEXAGON_A4_combineir
    {hexagon_A4_combineri, 11757}, // __builtin_HEXAGON_A4_combineri
    {hexagon_A4_cround_ri, 11778}, // __builtin_HEXAGON_A4_cround_ri
    {hexagon_A4_cround_rr, 11799}, // __builtin_HEXAGON_A4_cround_rr
    {hexagon_A4_modwrapu, 11820}, // __builtin_HEXAGON_A4_modwrapu
    {hexagon_A4_orn, 11840}, // __builtin_HEXAGON_A4_orn
    {hexagon_A4_ornp, 11855}, // __builtin_HEXAGON_A4_ornp
    {hexagon_A4_rcmpeq, 11871}, // __builtin_HEXAGON_A4_rcmpeq
    {hexagon_A4_rcmpeqi, 11889}, // __builtin_HEXAGON_A4_rcmpeqi
    {hexagon_A4_rcmpneq, 11908}, // __builtin_HEXAGON_A4_rcmpneq
    {hexagon_A4_rcmpneqi, 11927}, // __builtin_HEXAGON_A4_rcmpneqi
    {hexagon_A4_round_ri, 11947}, // __builtin_HEXAGON_A4_round_ri
    {hexagon_A4_round_ri_sat, 11967}, // __builtin_HEXAGON_A4_round_ri_sat
    {hexagon_A4_round_rr, 11991}, // __builtin_HEXAGON_A4_round_rr
    {hexagon_A4_round_rr_sat, 12011}, // __builtin_HEXAGON_A4_round_rr_sat
    {hexagon_A4_tlbmatch, 12035}, // __builtin_HEXAGON_A4_tlbmatch
    {hexagon_A4_vcmpbeq_any, 12055}, // __builtin_HEXAGON_A4_vcmpbeq_any
    {hexagon_A4_vcmpbeqi, 12078}, // __builtin_HEXAGON_A4_vcmpbeqi
    {hexagon_A4_vcmpbgt, 12098}, // __builtin_HEXAGON_A4_vcmpbgt
    {hexagon_A4_vcmpbgti, 12117}, // __builtin_HEXAGON_A4_vcmpbgti
    {hexagon_A4_vcmpbgtui, 12137}, // __builtin_HEXAGON_A4_vcmpbgtui
    {hexagon_A4_vcmpheqi, 12158}, // __builtin_HEXAGON_A4_vcmpheqi
    {hexagon_A4_vcmphgti, 12178}, // __builtin_HEXAGON_A4_vcmphgti
    {hexagon_A4_vcmphgtui, 12198}, // __builtin_HEXAGON_A4_vcmphgtui
    {hexagon_A4_vcmpweqi, 12219}, // __builtin_HEXAGON_A4_vcmpweqi
    {hexagon_A4_vcmpwgti, 12239}, // __builtin_HEXAGON_A4_vcmpwgti
    {hexagon_A4_vcmpwgtui, 12259}, // __builtin_HEXAGON_A4_vcmpwgtui
    {hexagon_A4_vrmaxh, 12280}, // __builtin_HEXAGON_A4_vrmaxh
    {hexagon_A4_vrmaxuh, 12298}, // __builtin_HEXAGON_A4_vrmaxuh
    {hexagon_A4_vrmaxuw, 12317}, // __builtin_HEXAGON_A4_vrmaxuw
    {hexagon_A4_vrmaxw, 12336}, // __builtin_HEXAGON_A4_vrmaxw
    {hexagon_A4_vrminh, 12354}, // __builtin_HEXAGON_A4_vrminh
    {hexagon_A4_vrminuh, 12372}, // __builtin_HEXAGON_A4_vrminuh
    {hexagon_A4_vrminuw, 12391}, // __builtin_HEXAGON_A4_vrminuw
    {hexagon_A4_vrminw, 12410}, // __builtin_HEXAGON_A4_vrminw
    {hexagon_A5_vaddhubs, 12428}, // __builtin_HEXAGON_A5_vaddhubs
    {hexagon_A6_vcmpbeq_notany, 12448}, // __builtin_HEXAGON_A6_vcmpbeq_notany
    {hexagon_A7_clip, 12474}, // __builtin_HEXAGON_A7_clip
    {hexagon_A7_croundd_ri, 12490}, // __builtin_HEXAGON_A7_croundd_ri
    {hexagon_A7_croundd_rr, 12512}, // __builtin_HEXAGON_A7_croundd_rr
    {hexagon_A7_vclip, 12534}, // __builtin_HEXAGON_A7_vclip
    {hexagon_C2_all8, 12551}, // __builtin_HEXAGON_C2_all8
    {hexagon_C2_and, 12567}, // __builtin_HEXAGON_C2_and
    {hexagon_C2_andn, 12582}, // __builtin_HEXAGON_C2_andn
    {hexagon_C2_any8, 12598}, // __builtin_HEXAGON_C2_any8
    {hexagon_C2_bitsclr, 12614}, // __builtin_HEXAGON_C2_bitsclr
    {hexagon_C2_bitsclri, 12633}, // __builtin_HEXAGON_C2_bitsclri
    {hexagon_C2_bitsset, 12653}, // __builtin_HEXAGON_C2_bitsset
    {hexagon_C2_cmpeq, 12672}, // __builtin_HEXAGON_C2_cmpeq
    {hexagon_C2_cmpeqi, 12689}, // __builtin_HEXAGON_C2_cmpeqi
    {hexagon_C2_cmpeqp, 12707}, // __builtin_HEXAGON_C2_cmpeqp
    {hexagon_C2_cmpgei, 12725}, // __builtin_HEXAGON_C2_cmpgei
    {hexagon_C2_cmpgeui, 12743}, // __builtin_HEXAGON_C2_cmpgeui
    {hexagon_C2_cmpgt, 12762}, // __builtin_HEXAGON_C2_cmpgt
    {hexagon_C2_cmpgti, 12779}, // __builtin_HEXAGON_C2_cmpgti
    {hexagon_C2_cmpgtp, 12797}, // __builtin_HEXAGON_C2_cmpgtp
    {hexagon_C2_cmpgtu, 12815}, // __builtin_HEXAGON_C2_cmpgtu
    {hexagon_C2_cmpgtui, 12833}, // __builtin_HEXAGON_C2_cmpgtui
    {hexagon_C2_cmpgtup, 12852}, // __builtin_HEXAGON_C2_cmpgtup
    {hexagon_C2_cmplt, 12871}, // __builtin_HEXAGON_C2_cmplt
    {hexagon_C2_cmpltu, 12888}, // __builtin_HEXAGON_C2_cmpltu
    {hexagon_C2_mask, 12906}, // __builtin_HEXAGON_C2_mask
    {hexagon_C2_mux, 12922}, // __builtin_HEXAGON_C2_mux
    {hexagon_C2_muxii, 12937}, // __builtin_HEXAGON_C2_muxii
    {hexagon_C2_muxir, 12954}, // __builtin_HEXAGON_C2_muxir
    {hexagon_C2_muxri, 12971}, // __builtin_HEXAGON_C2_muxri
    {hexagon_C2_not, 12988}, // __builtin_HEXAGON_C2_not
    {hexagon_C2_or, 13003}, // __builtin_HEXAGON_C2_or
    {hexagon_C2_orn, 13017}, // __builtin_HEXAGON_C2_orn
    {hexagon_C2_pxfer_map, 13032}, // __builtin_HEXAGON_C2_pxfer_map
    {hexagon_C2_tfrpr, 13053}, // __builtin_HEXAGON_C2_tfrpr
    {hexagon_C2_tfrrp, 13070}, // __builtin_HEXAGON_C2_tfrrp
    {hexagon_C2_vitpack, 13087}, // __builtin_HEXAGON_C2_vitpack
    {hexagon_C2_vmux, 13106}, // __builtin_HEXAGON_C2_vmux
    {hexagon_C2_xor, 13122}, // __builtin_HEXAGON_C2_xor
    {hexagon_C4_and_and, 13137}, // __builtin_HEXAGON_C4_and_and
    {hexagon_C4_and_andn, 13156}, // __builtin_HEXAGON_C4_and_andn
    {hexagon_C4_and_or, 13176}, // __builtin_HEXAGON_C4_and_or
    {hexagon_C4_and_orn, 13194}, // __builtin_HEXAGON_C4_and_orn
    {hexagon_C4_cmplte, 13213}, // __builtin_HEXAGON_C4_cmplte
    {hexagon_C4_cmpltei, 13231}, // __builtin_HEXAGON_C4_cmpltei
    {hexagon_C4_cmplteu, 13250}, // __builtin_HEXAGON_C4_cmplteu
    {hexagon_C4_cmplteui, 13269}, // __builtin_HEXAGON_C4_cmplteui
    {hexagon_C4_cmpneq, 13289}, // __builtin_HEXAGON_C4_cmpneq
    {hexagon_C4_cmpneqi, 13307}, // __builtin_HEXAGON_C4_cmpneqi
    {hexagon_C4_fastcorner9, 13326}, // __builtin_HEXAGON_C4_fastcorner9
    {hexagon_C4_fastcorner9_not, 13349}, // __builtin_HEXAGON_C4_fastcorner9_not
    {hexagon_C4_nbitsclr, 13376}, // __builtin_HEXAGON_C4_nbitsclr
    {hexagon_C4_nbitsclri, 13396}, // __builtin_HEXAGON_C4_nbitsclri
    {hexagon_C4_nbitsset, 13417}, // __builtin_HEXAGON_C4_nbitsset
    {hexagon_C4_or_and, 13437}, // __builtin_HEXAGON_C4_or_and
    {hexagon_C4_or_andn, 13455}, // __builtin_HEXAGON_C4_or_andn
    {hexagon_C4_or_or, 13474}, // __builtin_HEXAGON_C4_or_or
    {hexagon_C4_or_orn, 13491}, // __builtin_HEXAGON_C4_or_orn
    {hexagon_F2_conv_d2df, 13509}, // __builtin_HEXAGON_F2_conv_d2df
    {hexagon_F2_conv_d2sf, 13530}, // __builtin_HEXAGON_F2_conv_d2sf
    {hexagon_F2_conv_df2d, 13551}, // __builtin_HEXAGON_F2_conv_df2d
    {hexagon_F2_conv_df2d_chop, 13572}, // __builtin_HEXAGON_F2_conv_df2d_chop
    {hexagon_F2_conv_df2sf, 13598}, // __builtin_HEXAGON_F2_conv_df2sf
    {hexagon_F2_conv_df2ud, 13620}, // __builtin_HEXAGON_F2_conv_df2ud
    {hexagon_F2_conv_df2ud_chop, 13642}, // __builtin_HEXAGON_F2_conv_df2ud_chop
    {hexagon_F2_conv_df2uw, 13669}, // __builtin_HEXAGON_F2_conv_df2uw
    {hexagon_F2_conv_df2uw_chop, 13691}, // __builtin_HEXAGON_F2_conv_df2uw_chop
    {hexagon_F2_conv_df2w, 13718}, // __builtin_HEXAGON_F2_conv_df2w
    {hexagon_F2_conv_df2w_chop, 13739}, // __builtin_HEXAGON_F2_conv_df2w_chop
    {hexagon_F2_conv_sf2d, 13765}, // __builtin_HEXAGON_F2_conv_sf2d
    {hexagon_F2_conv_sf2d_chop, 13786}, // __builtin_HEXAGON_F2_conv_sf2d_chop
    {hexagon_F2_conv_sf2df, 13812}, // __builtin_HEXAGON_F2_conv_sf2df
    {hexagon_F2_conv_sf2ud, 13834}, // __builtin_HEXAGON_F2_conv_sf2ud
    {hexagon_F2_conv_sf2ud_chop, 13856}, // __builtin_HEXAGON_F2_conv_sf2ud_chop
    {hexagon_F2_conv_sf2uw, 13883}, // __builtin_HEXAGON_F2_conv_sf2uw
    {hexagon_F2_conv_sf2uw_chop, 13905}, // __builtin_HEXAGON_F2_conv_sf2uw_chop
    {hexagon_F2_conv_sf2w, 13932}, // __builtin_HEXAGON_F2_conv_sf2w
    {hexagon_F2_conv_sf2w_chop, 13953}, // __builtin_HEXAGON_F2_conv_sf2w_chop
    {hexagon_F2_conv_ud2df, 13979}, // __builtin_HEXAGON_F2_conv_ud2df
    {hexagon_F2_conv_ud2sf, 14001}, // __builtin_HEXAGON_F2_conv_ud2sf
    {hexagon_F2_conv_uw2df, 14023}, // __builtin_HEXAGON_F2_conv_uw2df
    {hexagon_F2_conv_uw2sf, 14045}, // __builtin_HEXAGON_F2_conv_uw2sf
    {hexagon_F2_conv_w2df, 14067}, // __builtin_HEXAGON_F2_conv_w2df
    {hexagon_F2_conv_w2sf, 14088}, // __builtin_HEXAGON_F2_conv_w2sf
    {hexagon_F2_dfadd, 14109}, // __builtin_HEXAGON_F2_dfadd
    {hexagon_F2_dfclass, 14126}, // __builtin_HEXAGON_F2_dfclass
    {hexagon_F2_dfcmpeq, 14145}, // __builtin_HEXAGON_F2_dfcmpeq
    {hexagon_F2_dfcmpge, 14164}, // __builtin_HEXAGON_F2_dfcmpge
    {hexagon_F2_dfcmpgt, 14183}, // __builtin_HEXAGON_F2_dfcmpgt
    {hexagon_F2_dfcmpuo, 14202}, // __builtin_HEXAGON_F2_dfcmpuo
    {hexagon_F2_dfimm_n, 14221}, // __builtin_HEXAGON_F2_dfimm_n
    {hexagon_F2_dfimm_p, 14240}, // __builtin_HEXAGON_F2_dfimm_p
    {hexagon_F2_dfmax, 14259}, // __builtin_HEXAGON_F2_dfmax
    {hexagon_F2_dfmin, 14276}, // __builtin_HEXAGON_F2_dfmin
    {hexagon_F2_dfmpyfix, 14293}, // __builtin_HEXAGON_F2_dfmpyfix
    {hexagon_F2_dfmpyhh, 14313}, // __builtin_HEXAGON_F2_dfmpyhh
    {hexagon_F2_dfmpylh, 14332}, // __builtin_HEXAGON_F2_dfmpylh
    {hexagon_F2_dfmpyll, 14351}, // __builtin_HEXAGON_F2_dfmpyll
    {hexagon_F2_dfsub, 14370}, // __builtin_HEXAGON_F2_dfsub
    {hexagon_F2_sfadd, 14387}, // __builtin_HEXAGON_F2_sfadd
    {hexagon_F2_sfclass, 14404}, // __builtin_HEXAGON_F2_sfclass
    {hexagon_F2_sfcmpeq, 14423}, // __builtin_HEXAGON_F2_sfcmpeq
    {hexagon_F2_sfcmpge, 14442}, // __builtin_HEXAGON_F2_sfcmpge
    {hexagon_F2_sfcmpgt, 14461}, // __builtin_HEXAGON_F2_sfcmpgt
    {hexagon_F2_sfcmpuo, 14480}, // __builtin_HEXAGON_F2_sfcmpuo
    {hexagon_F2_sffixupd, 14499}, // __builtin_HEXAGON_F2_sffixupd
    {hexagon_F2_sffixupn, 14519}, // __builtin_HEXAGON_F2_sffixupn
    {hexagon_F2_sffixupr, 14539}, // __builtin_HEXAGON_F2_sffixupr
    {hexagon_F2_sffma, 14559}, // __builtin_HEXAGON_F2_sffma
    {hexagon_F2_sffma_lib, 14576}, // __builtin_HEXAGON_F2_sffma_lib
    {hexagon_F2_sffma_sc, 14597}, // __builtin_HEXAGON_F2_sffma_sc
    {hexagon_F2_sffms, 14617}, // __builtin_HEXAGON_F2_sffms
    {hexagon_F2_sffms_lib, 14634}, // __builtin_HEXAGON_F2_sffms_lib
    {hexagon_F2_sfimm_n, 14655}, // __builtin_HEXAGON_F2_sfimm_n
    {hexagon_F2_sfimm_p, 14674}, // __builtin_HEXAGON_F2_sfimm_p
    {hexagon_F2_sfmax, 14693}, // __builtin_HEXAGON_F2_sfmax
    {hexagon_F2_sfmin, 14710}, // __builtin_HEXAGON_F2_sfmin
    {hexagon_F2_sfmpy, 14727}, // __builtin_HEXAGON_F2_sfmpy
    {hexagon_F2_sfsub, 14744}, // __builtin_HEXAGON_F2_sfsub
    {hexagon_L2_loadw_locked, 14761}, // __builtin_HEXAGON_L2_loadw_locked
    {hexagon_M2_acci, 14785}, // __builtin_HEXAGON_M2_acci
    {hexagon_M2_accii, 14801}, // __builtin_HEXAGON_M2_accii
    {hexagon_M2_cmaci_s0, 14818}, // __builtin_HEXAGON_M2_cmaci_s0
    {hexagon_M2_cmacr_s0, 14838}, // __builtin_HEXAGON_M2_cmacr_s0
    {hexagon_M2_cmacs_s0, 14858}, // __builtin_HEXAGON_M2_cmacs_s0
    {hexagon_M2_cmacs_s1, 14878}, // __builtin_HEXAGON_M2_cmacs_s1
    {hexagon_M2_cmacsc_s0, 14898}, // __builtin_HEXAGON_M2_cmacsc_s0
    {hexagon_M2_cmacsc_s1, 14919}, // __builtin_HEXAGON_M2_cmacsc_s1
    {hexagon_M2_cmpyi_s0, 14940}, // __builtin_HEXAGON_M2_cmpyi_s0
    {hexagon_M2_cmpyr_s0, 14960}, // __builtin_HEXAGON_M2_cmpyr_s0
    {hexagon_M2_cmpyrs_s0, 14980}, // __builtin_HEXAGON_M2_cmpyrs_s0
    {hexagon_M2_cmpyrs_s1, 15001}, // __builtin_HEXAGON_M2_cmpyrs_s1
    {hexagon_M2_cmpyrsc_s0, 15022}, // __builtin_HEXAGON_M2_cmpyrsc_s0
    {hexagon_M2_cmpyrsc_s1, 15044}, // __builtin_HEXAGON_M2_cmpyrsc_s1
    {hexagon_M2_cmpys_s0, 15066}, // __builtin_HEXAGON_M2_cmpys_s0
    {hexagon_M2_cmpys_s1, 15086}, // __builtin_HEXAGON_M2_cmpys_s1
    {hexagon_M2_cmpysc_s0, 15106}, // __builtin_HEXAGON_M2_cmpysc_s0
    {hexagon_M2_cmpysc_s1, 15127}, // __builtin_HEXAGON_M2_cmpysc_s1
    {hexagon_M2_cnacs_s0, 15148}, // __builtin_HEXAGON_M2_cnacs_s0
    {hexagon_M2_cnacs_s1, 15168}, // __builtin_HEXAGON_M2_cnacs_s1
    {hexagon_M2_cnacsc_s0, 15188}, // __builtin_HEXAGON_M2_cnacsc_s0
    {hexagon_M2_cnacsc_s1, 15209}, // __builtin_HEXAGON_M2_cnacsc_s1
    {hexagon_M2_dpmpyss_acc_s0, 15230}, // __builtin_HEXAGON_M2_dpmpyss_acc_s0
    {hexagon_M2_dpmpyss_nac_s0, 15256}, // __builtin_HEXAGON_M2_dpmpyss_nac_s0
    {hexagon_M2_dpmpyss_rnd_s0, 15282}, // __builtin_HEXAGON_M2_dpmpyss_rnd_s0
    {hexagon_M2_dpmpyss_s0, 15308}, // __builtin_HEXAGON_M2_dpmpyss_s0
    {hexagon_M2_dpmpyuu_acc_s0, 15330}, // __builtin_HEXAGON_M2_dpmpyuu_acc_s0
    {hexagon_M2_dpmpyuu_nac_s0, 15356}, // __builtin_HEXAGON_M2_dpmpyuu_nac_s0
    {hexagon_M2_dpmpyuu_s0, 15382}, // __builtin_HEXAGON_M2_dpmpyuu_s0
    {hexagon_M2_hmmpyh_rs1, 15404}, // __builtin_HEXAGON_M2_hmmpyh_rs1
    {hexagon_M2_hmmpyh_s1, 15426}, // __builtin_HEXAGON_M2_hmmpyh_s1
    {hexagon_M2_hmmpyl_rs1, 15447}, // __builtin_HEXAGON_M2_hmmpyl_rs1
    {hexagon_M2_hmmpyl_s1, 15469}, // __builtin_HEXAGON_M2_hmmpyl_s1
    {hexagon_M2_maci, 15490}, // __builtin_HEXAGON_M2_maci
    {hexagon_M2_macsin, 15506}, // __builtin_HEXAGON_M2_macsin
    {hexagon_M2_macsip, 15524}, // __builtin_HEXAGON_M2_macsip
    {hexagon_M2_mmachs_rs0, 15542}, // __builtin_HEXAGON_M2_mmachs_rs0
    {hexagon_M2_mmachs_rs1, 15564}, // __builtin_HEXAGON_M2_mmachs_rs1
    {hexagon_M2_mmachs_s0, 15586}, // __builtin_HEXAGON_M2_mmachs_s0
    {hexagon_M2_mmachs_s1, 15607}, // __builtin_HEXAGON_M2_mmachs_s1
    {hexagon_M2_mmacls_rs0, 15628}, // __builtin_HEXAGON_M2_mmacls_rs0
    {hexagon_M2_mmacls_rs1, 15650}, // __builtin_HEXAGON_M2_mmacls_rs1
    {hexagon_M2_mmacls_s0, 15672}, // __builtin_HEXAGON_M2_mmacls_s0
    {hexagon_M2_mmacls_s1, 15693}, // __builtin_HEXAGON_M2_mmacls_s1
    {hexagon_M2_mmacuhs_rs0, 15714}, // __builtin_HEXAGON_M2_mmacuhs_rs0
    {hexagon_M2_mmacuhs_rs1, 15737}, // __builtin_HEXAGON_M2_mmacuhs_rs1
    {hexagon_M2_mmacuhs_s0, 15760}, // __builtin_HEXAGON_M2_mmacuhs_s0
    {hexagon_M2_mmacuhs_s1, 15782}, // __builtin_HEXAGON_M2_mmacuhs_s1
    {hexagon_M2_mmaculs_rs0, 15804}, // __builtin_HEXAGON_M2_mmaculs_rs0
    {hexagon_M2_mmaculs_rs1, 15827}, // __builtin_HEXAGON_M2_mmaculs_rs1
    {hexagon_M2_mmaculs_s0, 15850}, // __builtin_HEXAGON_M2_mmaculs_s0
    {hexagon_M2_mmaculs_s1, 15872}, // __builtin_HEXAGON_M2_mmaculs_s1
    {hexagon_M2_mmpyh_rs0, 15894}, // __builtin_HEXAGON_M2_mmpyh_rs0
    {hexagon_M2_mmpyh_rs1, 15915}, // __builtin_HEXAGON_M2_mmpyh_rs1
    {hexagon_M2_mmpyh_s0, 15936}, // __builtin_HEXAGON_M2_mmpyh_s0
    {hexagon_M2_mmpyh_s1, 15956}, // __builtin_HEXAGON_M2_mmpyh_s1
    {hexagon_M2_mmpyl_rs0, 15976}, // __builtin_HEXAGON_M2_mmpyl_rs0
    {hexagon_M2_mmpyl_rs1, 15997}, // __builtin_HEXAGON_M2_mmpyl_rs1
    {hexagon_M2_mmpyl_s0, 16018}, // __builtin_HEXAGON_M2_mmpyl_s0
    {hexagon_M2_mmpyl_s1, 16038}, // __builtin_HEXAGON_M2_mmpyl_s1
    {hexagon_M2_mmpyuh_rs0, 16058}, // __builtin_HEXAGON_M2_mmpyuh_rs0
    {hexagon_M2_mmpyuh_rs1, 16080}, // __builtin_HEXAGON_M2_mmpyuh_rs1
    {hexagon_M2_mmpyuh_s0, 16102}, // __builtin_HEXAGON_M2_mmpyuh_s0
    {hexagon_M2_mmpyuh_s1, 16123}, // __builtin_HEXAGON_M2_mmpyuh_s1
    {hexagon_M2_mmpyul_rs0, 16144}, // __builtin_HEXAGON_M2_mmpyul_rs0
    {hexagon_M2_mmpyul_rs1, 16166}, // __builtin_HEXAGON_M2_mmpyul_rs1
    {hexagon_M2_mmpyul_s0, 16188}, // __builtin_HEXAGON_M2_mmpyul_s0
    {hexagon_M2_mmpyul_s1, 16209}, // __builtin_HEXAGON_M2_mmpyul_s1
    {hexagon_M2_mnaci, 16230}, // __builtin_HEXAGON_M2_mnaci
    {hexagon_M2_mpy_acc_hh_s0, 16247}, // __builtin_HEXAGON_M2_mpy_acc_hh_s0
    {hexagon_M2_mpy_acc_hh_s1, 16272}, // __builtin_HEXAGON_M2_mpy_acc_hh_s1
    {hexagon_M2_mpy_acc_hl_s0, 16297}, // __builtin_HEXAGON_M2_mpy_acc_hl_s0
    {hexagon_M2_mpy_acc_hl_s1, 16322}, // __builtin_HEXAGON_M2_mpy_acc_hl_s1
    {hexagon_M2_mpy_acc_lh_s0, 16347}, // __builtin_HEXAGON_M2_mpy_acc_lh_s0
    {hexagon_M2_mpy_acc_lh_s1, 16372}, // __builtin_HEXAGON_M2_mpy_acc_lh_s1
    {hexagon_M2_mpy_acc_ll_s0, 16397}, // __builtin_HEXAGON_M2_mpy_acc_ll_s0
    {hexagon_M2_mpy_acc_ll_s1, 16422}, // __builtin_HEXAGON_M2_mpy_acc_ll_s1
    {hexagon_M2_mpy_acc_sat_hh_s0, 16447}, // __builtin_HEXAGON_M2_mpy_acc_sat_hh_s0
    {hexagon_M2_mpy_acc_sat_hh_s1, 16476}, // __builtin_HEXAGON_M2_mpy_acc_sat_hh_s1
    {hexagon_M2_mpy_acc_sat_hl_s0, 16505}, // __builtin_HEXAGON_M2_mpy_acc_sat_hl_s0
    {hexagon_M2_mpy_acc_sat_hl_s1, 16534}, // __builtin_HEXAGON_M2_mpy_acc_sat_hl_s1
    {hexagon_M2_mpy_acc_sat_lh_s0, 16563}, // __builtin_HEXAGON_M2_mpy_acc_sat_lh_s0
    {hexagon_M2_mpy_acc_sat_lh_s1, 16592}, // __builtin_HEXAGON_M2_mpy_acc_sat_lh_s1
    {hexagon_M2_mpy_acc_sat_ll_s0, 16621}, // __builtin_HEXAGON_M2_mpy_acc_sat_ll_s0
    {hexagon_M2_mpy_acc_sat_ll_s1, 16650}, // __builtin_HEXAGON_M2_mpy_acc_sat_ll_s1
    {hexagon_M2_mpy_hh_s0, 16679}, // __builtin_HEXAGON_M2_mpy_hh_s0
    {hexagon_M2_mpy_hh_s1, 16700}, // __builtin_HEXAGON_M2_mpy_hh_s1
    {hexagon_M2_mpy_hl_s0, 16721}, // __builtin_HEXAGON_M2_mpy_hl_s0
    {hexagon_M2_mpy_hl_s1, 16742}, // __builtin_HEXAGON_M2_mpy_hl_s1
    {hexagon_M2_mpy_lh_s0, 16763}, // __builtin_HEXAGON_M2_mpy_lh_s0
    {hexagon_M2_mpy_lh_s1, 16784}, // __builtin_HEXAGON_M2_mpy_lh_s1
    {hexagon_M2_mpy_ll_s0, 16805}, // __builtin_HEXAGON_M2_mpy_ll_s0
    {hexagon_M2_mpy_ll_s1, 16826}, // __builtin_HEXAGON_M2_mpy_ll_s1
    {hexagon_M2_mpy_nac_hh_s0, 16847}, // __builtin_HEXAGON_M2_mpy_nac_hh_s0
    {hexagon_M2_mpy_nac_hh_s1, 16872}, // __builtin_HEXAGON_M2_mpy_nac_hh_s1
    {hexagon_M2_mpy_nac_hl_s0, 16897}, // __builtin_HEXAGON_M2_mpy_nac_hl_s0
    {hexagon_M2_mpy_nac_hl_s1, 16922}, // __builtin_HEXAGON_M2_mpy_nac_hl_s1
    {hexagon_M2_mpy_nac_lh_s0, 16947}, // __builtin_HEXAGON_M2_mpy_nac_lh_s0
    {hexagon_M2_mpy_nac_lh_s1, 16972}, // __builtin_HEXAGON_M2_mpy_nac_lh_s1
    {hexagon_M2_mpy_nac_ll_s0, 16997}, // __builtin_HEXAGON_M2_mpy_nac_ll_s0
    {hexagon_M2_mpy_nac_ll_s1, 17022}, // __builtin_HEXAGON_M2_mpy_nac_ll_s1
    {hexagon_M2_mpy_nac_sat_hh_s0, 17047}, // __builtin_HEXAGON_M2_mpy_nac_sat_hh_s0
    {hexagon_M2_mpy_nac_sat_hh_s1, 17076}, // __builtin_HEXAGON_M2_mpy_nac_sat_hh_s1
    {hexagon_M2_mpy_nac_sat_hl_s0, 17105}, // __builtin_HEXAGON_M2_mpy_nac_sat_hl_s0
    {hexagon_M2_mpy_nac_sat_hl_s1, 17134}, // __builtin_HEXAGON_M2_mpy_nac_sat_hl_s1
    {hexagon_M2_mpy_nac_sat_lh_s0, 17163}, // __builtin_HEXAGON_M2_mpy_nac_sat_lh_s0
    {hexagon_M2_mpy_nac_sat_lh_s1, 17192}, // __builtin_HEXAGON_M2_mpy_nac_sat_lh_s1
    {hexagon_M2_mpy_nac_sat_ll_s0, 17221}, // __builtin_HEXAGON_M2_mpy_nac_sat_ll_s0
    {hexagon_M2_mpy_nac_sat_ll_s1, 17250}, // __builtin_HEXAGON_M2_mpy_nac_sat_ll_s1
    {hexagon_M2_mpy_rnd_hh_s0, 17279}, // __builtin_HEXAGON_M2_mpy_rnd_hh_s0
    {hexagon_M2_mpy_rnd_hh_s1, 17304}, // __builtin_HEXAGON_M2_mpy_rnd_hh_s1
    {hexagon_M2_mpy_rnd_hl_s0, 17329}, // __builtin_HEXAGON_M2_mpy_rnd_hl_s0
    {hexagon_M2_mpy_rnd_hl_s1, 17354}, // __builtin_HEXAGON_M2_mpy_rnd_hl_s1
    {hexagon_M2_mpy_rnd_lh_s0, 17379}, // __builtin_HEXAGON_M2_mpy_rnd_lh_s0
    {hexagon_M2_mpy_rnd_lh_s1, 17404}, // __builtin_HEXAGON_M2_mpy_rnd_lh_s1
    {hexagon_M2_mpy_rnd_ll_s0, 17429}, // __builtin_HEXAGON_M2_mpy_rnd_ll_s0
    {hexagon_M2_mpy_rnd_ll_s1, 17454}, // __builtin_HEXAGON_M2_mpy_rnd_ll_s1
    {hexagon_M2_mpy_sat_hh_s0, 17479}, // __builtin_HEXAGON_M2_mpy_sat_hh_s0
    {hexagon_M2_mpy_sat_hh_s1, 17504}, // __builtin_HEXAGON_M2_mpy_sat_hh_s1
    {hexagon_M2_mpy_sat_hl_s0, 17529}, // __builtin_HEXAGON_M2_mpy_sat_hl_s0
    {hexagon_M2_mpy_sat_hl_s1, 17554}, // __builtin_HEXAGON_M2_mpy_sat_hl_s1
    {hexagon_M2_mpy_sat_lh_s0, 17579}, // __builtin_HEXAGON_M2_mpy_sat_lh_s0
    {hexagon_M2_mpy_sat_lh_s1, 17604}, // __builtin_HEXAGON_M2_mpy_sat_lh_s1
    {hexagon_M2_mpy_sat_ll_s0, 17629}, // __builtin_HEXAGON_M2_mpy_sat_ll_s0
    {hexagon_M2_mpy_sat_ll_s1, 17654}, // __builtin_HEXAGON_M2_mpy_sat_ll_s1
    {hexagon_M2_mpy_sat_rnd_hh_s0, 17679}, // __builtin_HEXAGON_M2_mpy_sat_rnd_hh_s0
    {hexagon_M2_mpy_sat_rnd_hh_s1, 17708}, // __builtin_HEXAGON_M2_mpy_sat_rnd_hh_s1
    {hexagon_M2_mpy_sat_rnd_hl_s0, 17737}, // __builtin_HEXAGON_M2_mpy_sat_rnd_hl_s0
    {hexagon_M2_mpy_sat_rnd_hl_s1, 17766}, // __builtin_HEXAGON_M2_mpy_sat_rnd_hl_s1
    {hexagon_M2_mpy_sat_rnd_lh_s0, 17795}, // __builtin_HEXAGON_M2_mpy_sat_rnd_lh_s0
    {hexagon_M2_mpy_sat_rnd_lh_s1, 17824}, // __builtin_HEXAGON_M2_mpy_sat_rnd_lh_s1
    {hexagon_M2_mpy_sat_rnd_ll_s0, 17853}, // __builtin_HEXAGON_M2_mpy_sat_rnd_ll_s0
    {hexagon_M2_mpy_sat_rnd_ll_s1, 17882}, // __builtin_HEXAGON_M2_mpy_sat_rnd_ll_s1
    {hexagon_M2_mpy_up, 17911}, // __builtin_HEXAGON_M2_mpy_up
    {hexagon_M2_mpy_up_s1, 17929}, // __builtin_HEXAGON_M2_mpy_up_s1
    {hexagon_M2_mpy_up_s1_sat, 17950}, // __builtin_HEXAGON_M2_mpy_up_s1_sat
    {hexagon_M2_mpyd_acc_hh_s0, 17975}, // __builtin_HEXAGON_M2_mpyd_acc_hh_s0
    {hexagon_M2_mpyd_acc_hh_s1, 18001}, // __builtin_HEXAGON_M2_mpyd_acc_hh_s1
    {hexagon_M2_mpyd_acc_hl_s0, 18027}, // __builtin_HEXAGON_M2_mpyd_acc_hl_s0
    {hexagon_M2_mpyd_acc_hl_s1, 18053}, // __builtin_HEXAGON_M2_mpyd_acc_hl_s1
    {hexagon_M2_mpyd_acc_lh_s0, 18079}, // __builtin_HEXAGON_M2_mpyd_acc_lh_s0
    {hexagon_M2_mpyd_acc_lh_s1, 18105}, // __builtin_HEXAGON_M2_mpyd_acc_lh_s1
    {hexagon_M2_mpyd_acc_ll_s0, 18131}, // __builtin_HEXAGON_M2_mpyd_acc_ll_s0
    {hexagon_M2_mpyd_acc_ll_s1, 18157}, // __builtin_HEXAGON_M2_mpyd_acc_ll_s1
    {hexagon_M2_mpyd_hh_s0, 18183}, // __builtin_HEXAGON_M2_mpyd_hh_s0
    {hexagon_M2_mpyd_hh_s1, 18205}, // __builtin_HEXAGON_M2_mpyd_hh_s1
    {hexagon_M2_mpyd_hl_s0, 18227}, // __builtin_HEXAGON_M2_mpyd_hl_s0
    {hexagon_M2_mpyd_hl_s1, 18249}, // __builtin_HEXAGON_M2_mpyd_hl_s1
    {hexagon_M2_mpyd_lh_s0, 18271}, // __builtin_HEXAGON_M2_mpyd_lh_s0
    {hexagon_M2_mpyd_lh_s1, 18293}, // __builtin_HEXAGON_M2_mpyd_lh_s1
    {hexagon_M2_mpyd_ll_s0, 18315}, // __builtin_HEXAGON_M2_mpyd_ll_s0
    {hexagon_M2_mpyd_ll_s1, 18337}, // __builtin_HEXAGON_M2_mpyd_ll_s1
    {hexagon_M2_mpyd_nac_hh_s0, 18359}, // __builtin_HEXAGON_M2_mpyd_nac_hh_s0
    {hexagon_M2_mpyd_nac_hh_s1, 18385}, // __builtin_HEXAGON_M2_mpyd_nac_hh_s1
    {hexagon_M2_mpyd_nac_hl_s0, 18411}, // __builtin_HEXAGON_M2_mpyd_nac_hl_s0
    {hexagon_M2_mpyd_nac_hl_s1, 18437}, // __builtin_HEXAGON_M2_mpyd_nac_hl_s1
    {hexagon_M2_mpyd_nac_lh_s0, 18463}, // __builtin_HEXAGON_M2_mpyd_nac_lh_s0
    {hexagon_M2_mpyd_nac_lh_s1, 18489}, // __builtin_HEXAGON_M2_mpyd_nac_lh_s1
    {hexagon_M2_mpyd_nac_ll_s0, 18515}, // __builtin_HEXAGON_M2_mpyd_nac_ll_s0
    {hexagon_M2_mpyd_nac_ll_s1, 18541}, // __builtin_HEXAGON_M2_mpyd_nac_ll_s1
    {hexagon_M2_mpyd_rnd_hh_s0, 18567}, // __builtin_HEXAGON_M2_mpyd_rnd_hh_s0
    {hexagon_M2_mpyd_rnd_hh_s1, 18593}, // __builtin_HEXAGON_M2_mpyd_rnd_hh_s1
    {hexagon_M2_mpyd_rnd_hl_s0, 18619}, // __builtin_HEXAGON_M2_mpyd_rnd_hl_s0
    {hexagon_M2_mpyd_rnd_hl_s1, 18645}, // __builtin_HEXAGON_M2_mpyd_rnd_hl_s1
    {hexagon_M2_mpyd_rnd_lh_s0, 18671}, // __builtin_HEXAGON_M2_mpyd_rnd_lh_s0
    {hexagon_M2_mpyd_rnd_lh_s1, 18697}, // __builtin_HEXAGON_M2_mpyd_rnd_lh_s1
    {hexagon_M2_mpyd_rnd_ll_s0, 18723}, // __builtin_HEXAGON_M2_mpyd_rnd_ll_s0
    {hexagon_M2_mpyd_rnd_ll_s1, 18749}, // __builtin_HEXAGON_M2_mpyd_rnd_ll_s1
    {hexagon_M2_mpyi, 18775}, // __builtin_HEXAGON_M2_mpyi
    {hexagon_M2_mpysmi, 18791}, // __builtin_HEXAGON_M2_mpysmi
    {hexagon_M2_mpysu_up, 18809}, // __builtin_HEXAGON_M2_mpysu_up
    {hexagon_M2_mpyu_acc_hh_s0, 18829}, // __builtin_HEXAGON_M2_mpyu_acc_hh_s0
    {hexagon_M2_mpyu_acc_hh_s1, 18855}, // __builtin_HEXAGON_M2_mpyu_acc_hh_s1
    {hexagon_M2_mpyu_acc_hl_s0, 18881}, // __builtin_HEXAGON_M2_mpyu_acc_hl_s0
    {hexagon_M2_mpyu_acc_hl_s1, 18907}, // __builtin_HEXAGON_M2_mpyu_acc_hl_s1
    {hexagon_M2_mpyu_acc_lh_s0, 18933}, // __builtin_HEXAGON_M2_mpyu_acc_lh_s0
    {hexagon_M2_mpyu_acc_lh_s1, 18959}, // __builtin_HEXAGON_M2_mpyu_acc_lh_s1
    {hexagon_M2_mpyu_acc_ll_s0, 18985}, // __builtin_HEXAGON_M2_mpyu_acc_ll_s0
    {hexagon_M2_mpyu_acc_ll_s1, 19011}, // __builtin_HEXAGON_M2_mpyu_acc_ll_s1
    {hexagon_M2_mpyu_hh_s0, 19037}, // __builtin_HEXAGON_M2_mpyu_hh_s0
    {hexagon_M2_mpyu_hh_s1, 19059}, // __builtin_HEXAGON_M2_mpyu_hh_s1
    {hexagon_M2_mpyu_hl_s0, 19081}, // __builtin_HEXAGON_M2_mpyu_hl_s0
    {hexagon_M2_mpyu_hl_s1, 19103}, // __builtin_HEXAGON_M2_mpyu_hl_s1
    {hexagon_M2_mpyu_lh_s0, 19125}, // __builtin_HEXAGON_M2_mpyu_lh_s0
    {hexagon_M2_mpyu_lh_s1, 19147}, // __builtin_HEXAGON_M2_mpyu_lh_s1
    {hexagon_M2_mpyu_ll_s0, 19169}, // __builtin_HEXAGON_M2_mpyu_ll_s0
    {hexagon_M2_mpyu_ll_s1, 19191}, // __builtin_HEXAGON_M2_mpyu_ll_s1
    {hexagon_M2_mpyu_nac_hh_s0, 19213}, // __builtin_HEXAGON_M2_mpyu_nac_hh_s0
    {hexagon_M2_mpyu_nac_hh_s1, 19239}, // __builtin_HEXAGON_M2_mpyu_nac_hh_s1
    {hexagon_M2_mpyu_nac_hl_s0, 19265}, // __builtin_HEXAGON_M2_mpyu_nac_hl_s0
    {hexagon_M2_mpyu_nac_hl_s1, 19291}, // __builtin_HEXAGON_M2_mpyu_nac_hl_s1
    {hexagon_M2_mpyu_nac_lh_s0, 19317}, // __builtin_HEXAGON_M2_mpyu_nac_lh_s0
    {hexagon_M2_mpyu_nac_lh_s1, 19343}, // __builtin_HEXAGON_M2_mpyu_nac_lh_s1
    {hexagon_M2_mpyu_nac_ll_s0, 19369}, // __builtin_HEXAGON_M2_mpyu_nac_ll_s0
    {hexagon_M2_mpyu_nac_ll_s1, 19395}, // __builtin_HEXAGON_M2_mpyu_nac_ll_s1
    {hexagon_M2_mpyu_up, 19421}, // __builtin_HEXAGON_M2_mpyu_up
    {hexagon_M2_mpyud_acc_hh_s0, 19440}, // __builtin_HEXAGON_M2_mpyud_acc_hh_s0
    {hexagon_M2_mpyud_acc_hh_s1, 19467}, // __builtin_HEXAGON_M2_mpyud_acc_hh_s1
    {hexagon_M2_mpyud_acc_hl_s0, 19494}, // __builtin_HEXAGON_M2_mpyud_acc_hl_s0
    {hexagon_M2_mpyud_acc_hl_s1, 19521}, // __builtin_HEXAGON_M2_mpyud_acc_hl_s1
    {hexagon_M2_mpyud_acc_lh_s0, 19548}, // __builtin_HEXAGON_M2_mpyud_acc_lh_s0
    {hexagon_M2_mpyud_acc_lh_s1, 19575}, // __builtin_HEXAGON_M2_mpyud_acc_lh_s1
    {hexagon_M2_mpyud_acc_ll_s0, 19602}, // __builtin_HEXAGON_M2_mpyud_acc_ll_s0
    {hexagon_M2_mpyud_acc_ll_s1, 19629}, // __builtin_HEXAGON_M2_mpyud_acc_ll_s1
    {hexagon_M2_mpyud_hh_s0, 19656}, // __builtin_HEXAGON_M2_mpyud_hh_s0
    {hexagon_M2_mpyud_hh_s1, 19679}, // __builtin_HEXAGON_M2_mpyud_hh_s1
    {hexagon_M2_mpyud_hl_s0, 19702}, // __builtin_HEXAGON_M2_mpyud_hl_s0
    {hexagon_M2_mpyud_hl_s1, 19725}, // __builtin_HEXAGON_M2_mpyud_hl_s1
    {hexagon_M2_mpyud_lh_s0, 19748}, // __builtin_HEXAGON_M2_mpyud_lh_s0
    {hexagon_M2_mpyud_lh_s1, 19771}, // __builtin_HEXAGON_M2_mpyud_lh_s1
    {hexagon_M2_mpyud_ll_s0, 19794}, // __builtin_HEXAGON_M2_mpyud_ll_s0
    {hexagon_M2_mpyud_ll_s1, 19817}, // __builtin_HEXAGON_M2_mpyud_ll_s1
    {hexagon_M2_mpyud_nac_hh_s0, 19840}, // __builtin_HEXAGON_M2_mpyud_nac_hh_s0
    {hexagon_M2_mpyud_nac_hh_s1, 19867}, // __builtin_HEXAGON_M2_mpyud_nac_hh_s1
    {hexagon_M2_mpyud_nac_hl_s0, 19894}, // __builtin_HEXAGON_M2_mpyud_nac_hl_s0
    {hexagon_M2_mpyud_nac_hl_s1, 19921}, // __builtin_HEXAGON_M2_mpyud_nac_hl_s1
    {hexagon_M2_mpyud_nac_lh_s0, 19948}, // __builtin_HEXAGON_M2_mpyud_nac_lh_s0
    {hexagon_M2_mpyud_nac_lh_s1, 19975}, // __builtin_HEXAGON_M2_mpyud_nac_lh_s1
    {hexagon_M2_mpyud_nac_ll_s0, 20002}, // __builtin_HEXAGON_M2_mpyud_nac_ll_s0
    {hexagon_M2_mpyud_nac_ll_s1, 20029}, // __builtin_HEXAGON_M2_mpyud_nac_ll_s1
    {hexagon_M2_mpyui, 20056}, // __builtin_HEXAGON_M2_mpyui
    {hexagon_M2_nacci, 20073}, // __builtin_HEXAGON_M2_nacci
    {hexagon_M2_naccii, 20090}, // __builtin_HEXAGON_M2_naccii
    {hexagon_M2_subacc, 20108}, // __builtin_HEXAGON_M2_subacc
    {hexagon_M2_vabsdiffh, 20126}, // __builtin_HEXAGON_M2_vabsdiffh
    {hexagon_M2_vabsdiffw, 20147}, // __builtin_HEXAGON_M2_vabsdiffw
    {hexagon_M2_vcmac_s0_sat_i, 20168}, // __builtin_HEXAGON_M2_vcmac_s0_sat_i
    {hexagon_M2_vcmac_s0_sat_r, 20194}, // __builtin_HEXAGON_M2_vcmac_s0_sat_r
    {hexagon_M2_vcmpy_s0_sat_i, 20220}, // __builtin_HEXAGON_M2_vcmpy_s0_sat_i
    {hexagon_M2_vcmpy_s0_sat_r, 20246}, // __builtin_HEXAGON_M2_vcmpy_s0_sat_r
    {hexagon_M2_vcmpy_s1_sat_i, 20272}, // __builtin_HEXAGON_M2_vcmpy_s1_sat_i
    {hexagon_M2_vcmpy_s1_sat_r, 20298}, // __builtin_HEXAGON_M2_vcmpy_s1_sat_r
    {hexagon_M2_vdmacs_s0, 20324}, // __builtin_HEXAGON_M2_vdmacs_s0
    {hexagon_M2_vdmacs_s1, 20345}, // __builtin_HEXAGON_M2_vdmacs_s1
    {hexagon_M2_vdmpyrs_s0, 20366}, // __builtin_HEXAGON_M2_vdmpyrs_s0
    {hexagon_M2_vdmpyrs_s1, 20388}, // __builtin_HEXAGON_M2_vdmpyrs_s1
    {hexagon_M2_vdmpys_s0, 20410}, // __builtin_HEXAGON_M2_vdmpys_s0
    {hexagon_M2_vdmpys_s1, 20431}, // __builtin_HEXAGON_M2_vdmpys_s1
    {hexagon_M2_vmac2, 20452}, // __builtin_HEXAGON_M2_vmac2
    {hexagon_M2_vmac2es, 20469}, // __builtin_HEXAGON_M2_vmac2es
    {hexagon_M2_vmac2es_s0, 20488}, // __builtin_HEXAGON_M2_vmac2es_s0
    {hexagon_M2_vmac2es_s1, 20510}, // __builtin_HEXAGON_M2_vmac2es_s1
    {hexagon_M2_vmac2s_s0, 20532}, // __builtin_HEXAGON_M2_vmac2s_s0
    {hexagon_M2_vmac2s_s1, 20553}, // __builtin_HEXAGON_M2_vmac2s_s1
    {hexagon_M2_vmac2su_s0, 20574}, // __builtin_HEXAGON_M2_vmac2su_s0
    {hexagon_M2_vmac2su_s1, 20596}, // __builtin_HEXAGON_M2_vmac2su_s1
    {hexagon_M2_vmpy2es_s0, 20618}, // __builtin_HEXAGON_M2_vmpy2es_s0
    {hexagon_M2_vmpy2es_s1, 20640}, // __builtin_HEXAGON_M2_vmpy2es_s1
    {hexagon_M2_vmpy2s_s0, 20662}, // __builtin_HEXAGON_M2_vmpy2s_s0
    {hexagon_M2_vmpy2s_s0pack, 20683}, // __builtin_HEXAGON_M2_vmpy2s_s0pack
    {hexagon_M2_vmpy2s_s1, 20708}, // __builtin_HEXAGON_M2_vmpy2s_s1
    {hexagon_M2_vmpy2s_s1pack, 20729}, // __builtin_HEXAGON_M2_vmpy2s_s1pack
    {hexagon_M2_vmpy2su_s0, 20754}, // __builtin_HEXAGON_M2_vmpy2su_s0
    {hexagon_M2_vmpy2su_s1, 20776}, // __builtin_HEXAGON_M2_vmpy2su_s1
    {hexagon_M2_vraddh, 20798}, // __builtin_HEXAGON_M2_vraddh
    {hexagon_M2_vradduh, 20816}, // __builtin_HEXAGON_M2_vradduh
    {hexagon_M2_vrcmaci_s0, 20835}, // __builtin_HEXAGON_M2_vrcmaci_s0
    {hexagon_M2_vrcmaci_s0c, 20857}, // __builtin_HEXAGON_M2_vrcmaci_s0c
    {hexagon_M2_vrcmacr_s0, 20880}, // __builtin_HEXAGON_M2_vrcmacr_s0
    {hexagon_M2_vrcmacr_s0c, 20902}, // __builtin_HEXAGON_M2_vrcmacr_s0c
    {hexagon_M2_vrcmpyi_s0, 20925}, // __builtin_HEXAGON_M2_vrcmpyi_s0
    {hexagon_M2_vrcmpyi_s0c, 20947}, // __builtin_HEXAGON_M2_vrcmpyi_s0c
    {hexagon_M2_vrcmpyr_s0, 20970}, // __builtin_HEXAGON_M2_vrcmpyr_s0
    {hexagon_M2_vrcmpyr_s0c, 20992}, // __builtin_HEXAGON_M2_vrcmpyr_s0c
    {hexagon_M2_vrcmpys_acc_s1, 21015}, // __builtin_HEXAGON_M2_vrcmpys_acc_s1
    {hexagon_M2_vrcmpys_s1, 21041}, // __builtin_HEXAGON_M2_vrcmpys_s1
    {hexagon_M2_vrcmpys_s1rp, 21063}, // __builtin_HEXAGON_M2_vrcmpys_s1rp
    {hexagon_M2_vrmac_s0, 21087}, // __builtin_HEXAGON_M2_vrmac_s0
    {hexagon_M2_vrmpy_s0, 21107}, // __builtin_HEXAGON_M2_vrmpy_s0
    {hexagon_M2_xor_xacc, 21127}, // __builtin_HEXAGON_M2_xor_xacc
    {hexagon_M4_and_and, 21147}, // __builtin_HEXAGON_M4_and_and
    {hexagon_M4_and_andn, 21166}, // __builtin_HEXAGON_M4_and_andn
    {hexagon_M4_and_or, 21186}, // __builtin_HEXAGON_M4_and_or
    {hexagon_M4_and_xor, 21204}, // __builtin_HEXAGON_M4_and_xor
    {hexagon_M4_cmpyi_wh, 21223}, // __builtin_HEXAGON_M4_cmpyi_wh
    {hexagon_M4_cmpyi_whc, 21243}, // __builtin_HEXAGON_M4_cmpyi_whc
    {hexagon_M4_cmpyr_wh, 21264}, // __builtin_HEXAGON_M4_cmpyr_wh
    {hexagon_M4_cmpyr_whc, 21284}, // __builtin_HEXAGON_M4_cmpyr_whc
    {hexagon_M4_mac_up_s1_sat, 21305}, // __builtin_HEXAGON_M4_mac_up_s1_sat
    {hexagon_M4_mpyri_addi, 21330}, // __builtin_HEXAGON_M4_mpyri_addi
    {hexagon_M4_mpyri_addr, 21352}, // __builtin_HEXAGON_M4_mpyri_addr
    {hexagon_M4_mpyri_addr_u2, 21374}, // __builtin_HEXAGON_M4_mpyri_addr_u2
    {hexagon_M4_mpyrr_addi, 21399}, // __builtin_HEXAGON_M4_mpyrr_addi
    {hexagon_M4_mpyrr_addr, 21421}, // __builtin_HEXAGON_M4_mpyrr_addr
    {hexagon_M4_nac_up_s1_sat, 21443}, // __builtin_HEXAGON_M4_nac_up_s1_sat
    {hexagon_M4_or_and, 21468}, // __builtin_HEXAGON_M4_or_and
    {hexagon_M4_or_andn, 21486}, // __builtin_HEXAGON_M4_or_andn
    {hexagon_M4_or_or, 21505}, // __builtin_HEXAGON_M4_or_or
    {hexagon_M4_or_xor, 21522}, // __builtin_HEXAGON_M4_or_xor
    {hexagon_M4_pmpyw, 21540}, // __builtin_HEXAGON_M4_pmpyw
    {hexagon_M4_pmpyw_acc, 21557}, // __builtin_HEXAGON_M4_pmpyw_acc
    {hexagon_M4_vpmpyh, 21578}, // __builtin_HEXAGON_M4_vpmpyh
    {hexagon_M4_vpmpyh_acc, 21596}, // __builtin_HEXAGON_M4_vpmpyh_acc
    {hexagon_M4_vrmpyeh_acc_s0, 21618}, // __builtin_HEXAGON_M4_vrmpyeh_acc_s0
    {hexagon_M4_vrmpyeh_acc_s1, 21644}, // __builtin_HEXAGON_M4_vrmpyeh_acc_s1
    {hexagon_M4_vrmpyeh_s0, 21670}, // __builtin_HEXAGON_M4_vrmpyeh_s0
    {hexagon_M4_vrmpyeh_s1, 21692}, // __builtin_HEXAGON_M4_vrmpyeh_s1
    {hexagon_M4_vrmpyoh_acc_s0, 21714}, // __builtin_HEXAGON_M4_vrmpyoh_acc_s0
    {hexagon_M4_vrmpyoh_acc_s1, 21740}, // __builtin_HEXAGON_M4_vrmpyoh_acc_s1
    {hexagon_M4_vrmpyoh_s0, 21766}, // __builtin_HEXAGON_M4_vrmpyoh_s0
    {hexagon_M4_vrmpyoh_s1, 21788}, // __builtin_HEXAGON_M4_vrmpyoh_s1
    {hexagon_M4_xor_and, 21810}, // __builtin_HEXAGON_M4_xor_and
    {hexagon_M4_xor_andn, 21829}, // __builtin_HEXAGON_M4_xor_andn
    {hexagon_M4_xor_or, 21849}, // __builtin_HEXAGON_M4_xor_or
    {hexagon_M4_xor_xacc, 21867}, // __builtin_HEXAGON_M4_xor_xacc
    {hexagon_M5_vdmacbsu, 21887}, // __builtin_HEXAGON_M5_vdmacbsu
    {hexagon_M5_vdmpybsu, 21907}, // __builtin_HEXAGON_M5_vdmpybsu
    {hexagon_M5_vmacbsu, 21927}, // __builtin_HEXAGON_M5_vmacbsu
    {hexagon_M5_vmacbuu, 21946}, // __builtin_HEXAGON_M5_vmacbuu
    {hexagon_M5_vmpybsu, 21965}, // __builtin_HEXAGON_M5_vmpybsu
    {hexagon_M5_vmpybuu, 21984}, // __builtin_HEXAGON_M5_vmpybuu
    {hexagon_M5_vrmacbsu, 22003}, // __builtin_HEXAGON_M5_vrmacbsu
    {hexagon_M5_vrmacbuu, 22023}, // __builtin_HEXAGON_M5_vrmacbuu
    {hexagon_M5_vrmpybsu, 22043}, // __builtin_HEXAGON_M5_vrmpybsu
    {hexagon_M5_vrmpybuu, 22063}, // __builtin_HEXAGON_M5_vrmpybuu
    {hexagon_M6_vabsdiffb, 22083}, // __builtin_HEXAGON_M6_vabsdiffb
    {hexagon_M6_vabsdiffub, 22104}, // __builtin_HEXAGON_M6_vabsdiffub
    {hexagon_M7_dcmpyiw, 22126}, // __builtin_HEXAGON_M7_dcmpyiw
    {hexagon_M7_dcmpyiw_acc, 22145}, // __builtin_HEXAGON_M7_dcmpyiw_acc
    {hexagon_M7_dcmpyiwc, 22168}, // __builtin_HEXAGON_M7_dcmpyiwc
    {hexagon_M7_dcmpyiwc_acc, 22188}, // __builtin_HEXAGON_M7_dcmpyiwc_acc
    {hexagon_M7_dcmpyrw, 22212}, // __builtin_HEXAGON_M7_dcmpyrw
    {hexagon_M7_dcmpyrw_acc, 22231}, // __builtin_HEXAGON_M7_dcmpyrw_acc
    {hexagon_M7_dcmpyrwc, 22254}, // __builtin_HEXAGON_M7_dcmpyrwc
    {hexagon_M7_dcmpyrwc_acc, 22274}, // __builtin_HEXAGON_M7_dcmpyrwc_acc
    {hexagon_M7_vdmpy, 22298}, // __builtin_HEXAGON_M7_vdmpy
    {hexagon_M7_vdmpy_acc, 22315}, // __builtin_HEXAGON_M7_vdmpy_acc
    {hexagon_M7_wcmpyiw, 22336}, // __builtin_HEXAGON_M7_wcmpyiw
    {hexagon_M7_wcmpyiw_rnd, 22355}, // __builtin_HEXAGON_M7_wcmpyiw_rnd
    {hexagon_M7_wcmpyiwc, 22378}, // __builtin_HEXAGON_M7_wcmpyiwc
    {hexagon_M7_wcmpyiwc_rnd, 22398}, // __builtin_HEXAGON_M7_wcmpyiwc_rnd
    {hexagon_M7_wcmpyrw, 22422}, // __builtin_HEXAGON_M7_wcmpyrw
    {hexagon_M7_wcmpyrw_rnd, 22441}, // __builtin_HEXAGON_M7_wcmpyrw_rnd
    {hexagon_M7_wcmpyrwc, 22464}, // __builtin_HEXAGON_M7_wcmpyrwc
    {hexagon_M7_wcmpyrwc_rnd, 22484}, // __builtin_HEXAGON_M7_wcmpyrwc_rnd
    {hexagon_S2_addasl_rrri, 22508}, // __builtin_HEXAGON_S2_addasl_rrri
    {hexagon_S2_asl_i_p, 22531}, // __builtin_HEXAGON_S2_asl_i_p
    {hexagon_S2_asl_i_p_acc, 22550}, // __builtin_HEXAGON_S2_asl_i_p_acc
    {hexagon_S2_asl_i_p_and, 22573}, // __builtin_HEXAGON_S2_asl_i_p_and
    {hexagon_S2_asl_i_p_nac, 22596}, // __builtin_HEXAGON_S2_asl_i_p_nac
    {hexagon_S2_asl_i_p_or, 22619}, // __builtin_HEXAGON_S2_asl_i_p_or
    {hexagon_S2_asl_i_p_xacc, 22641}, // __builtin_HEXAGON_S2_asl_i_p_xacc
    {hexagon_S2_asl_i_r, 22665}, // __builtin_HEXAGON_S2_asl_i_r
    {hexagon_S2_asl_i_r_acc, 22684}, // __builtin_HEXAGON_S2_asl_i_r_acc
    {hexagon_S2_asl_i_r_and, 22707}, // __builtin_HEXAGON_S2_asl_i_r_and
    {hexagon_S2_asl_i_r_nac, 22730}, // __builtin_HEXAGON_S2_asl_i_r_nac
    {hexagon_S2_asl_i_r_or, 22753}, // __builtin_HEXAGON_S2_asl_i_r_or
    {hexagon_S2_asl_i_r_sat, 22775}, // __builtin_HEXAGON_S2_asl_i_r_sat
    {hexagon_S2_asl_i_r_xacc, 22798}, // __builtin_HEXAGON_S2_asl_i_r_xacc
    {hexagon_S2_asl_i_vh, 22822}, // __builtin_HEXAGON_S2_asl_i_vh
    {hexagon_S2_asl_i_vw, 22842}, // __builtin_HEXAGON_S2_asl_i_vw
    {hexagon_S2_asl_r_p, 22862}, // __builtin_HEXAGON_S2_asl_r_p
    {hexagon_S2_asl_r_p_acc, 22881}, // __builtin_HEXAGON_S2_asl_r_p_acc
    {hexagon_S2_asl_r_p_and, 22904}, // __builtin_HEXAGON_S2_asl_r_p_and
    {hexagon_S2_asl_r_p_nac, 22927}, // __builtin_HEXAGON_S2_asl_r_p_nac
    {hexagon_S2_asl_r_p_or, 22950}, // __builtin_HEXAGON_S2_asl_r_p_or
    {hexagon_S2_asl_r_p_xor, 22972}, // __builtin_HEXAGON_S2_asl_r_p_xor
    {hexagon_S2_asl_r_r, 22995}, // __builtin_HEXAGON_S2_asl_r_r
    {hexagon_S2_asl_r_r_acc, 23014}, // __builtin_HEXAGON_S2_asl_r_r_acc
    {hexagon_S2_asl_r_r_and, 23037}, // __builtin_HEXAGON_S2_asl_r_r_and
    {hexagon_S2_asl_r_r_nac, 23060}, // __builtin_HEXAGON_S2_asl_r_r_nac
    {hexagon_S2_asl_r_r_or, 23083}, // __builtin_HEXAGON_S2_asl_r_r_or
    {hexagon_S2_asl_r_r_sat, 23105}, // __builtin_HEXAGON_S2_asl_r_r_sat
    {hexagon_S2_asl_r_vh, 23128}, // __builtin_HEXAGON_S2_asl_r_vh
    {hexagon_S2_asl_r_vw, 23148}, // __builtin_HEXAGON_S2_asl_r_vw
    {hexagon_S2_asr_i_p, 23168}, // __builtin_HEXAGON_S2_asr_i_p
    {hexagon_S2_asr_i_p_acc, 23187}, // __builtin_HEXAGON_S2_asr_i_p_acc
    {hexagon_S2_asr_i_p_and, 23210}, // __builtin_HEXAGON_S2_asr_i_p_and
    {hexagon_S2_asr_i_p_nac, 23233}, // __builtin_HEXAGON_S2_asr_i_p_nac
    {hexagon_S2_asr_i_p_or, 23256}, // __builtin_HEXAGON_S2_asr_i_p_or
    {hexagon_S2_asr_i_p_rnd, 23278}, // __builtin_HEXAGON_S2_asr_i_p_rnd
    {hexagon_S2_asr_i_p_rnd_goodsyntax, 23301}, // __builtin_HEXAGON_S2_asr_i_p_rnd_goodsyntax
    {hexagon_S2_asr_i_r, 23335}, // __builtin_HEXAGON_S2_asr_i_r
    {hexagon_S2_asr_i_r_acc, 23354}, // __builtin_HEXAGON_S2_asr_i_r_acc
    {hexagon_S2_asr_i_r_and, 23377}, // __builtin_HEXAGON_S2_asr_i_r_and
    {hexagon_S2_asr_i_r_nac, 23400}, // __builtin_HEXAGON_S2_asr_i_r_nac
    {hexagon_S2_asr_i_r_or, 23423}, // __builtin_HEXAGON_S2_asr_i_r_or
    {hexagon_S2_asr_i_r_rnd, 23445}, // __builtin_HEXAGON_S2_asr_i_r_rnd
    {hexagon_S2_asr_i_r_rnd_goodsyntax, 23468}, // __builtin_HEXAGON_S2_asr_i_r_rnd_goodsyntax
    {hexagon_S2_asr_i_svw_trun, 23502}, // __builtin_HEXAGON_S2_asr_i_svw_trun
    {hexagon_S2_asr_i_vh, 23528}, // __builtin_HEXAGON_S2_asr_i_vh
    {hexagon_S2_asr_i_vw, 23548}, // __builtin_HEXAGON_S2_asr_i_vw
    {hexagon_S2_asr_r_p, 23568}, // __builtin_HEXAGON_S2_asr_r_p
    {hexagon_S2_asr_r_p_acc, 23587}, // __builtin_HEXAGON_S2_asr_r_p_acc
    {hexagon_S2_asr_r_p_and, 23610}, // __builtin_HEXAGON_S2_asr_r_p_and
    {hexagon_S2_asr_r_p_nac, 23633}, // __builtin_HEXAGON_S2_asr_r_p_nac
    {hexagon_S2_asr_r_p_or, 23656}, // __builtin_HEXAGON_S2_asr_r_p_or
    {hexagon_S2_asr_r_p_xor, 23678}, // __builtin_HEXAGON_S2_asr_r_p_xor
    {hexagon_S2_asr_r_r, 23701}, // __builtin_HEXAGON_S2_asr_r_r
    {hexagon_S2_asr_r_r_acc, 23720}, // __builtin_HEXAGON_S2_asr_r_r_acc
    {hexagon_S2_asr_r_r_and, 23743}, // __builtin_HEXAGON_S2_asr_r_r_and
    {hexagon_S2_asr_r_r_nac, 23766}, // __builtin_HEXAGON_S2_asr_r_r_nac
    {hexagon_S2_asr_r_r_or, 23789}, // __builtin_HEXAGON_S2_asr_r_r_or
    {hexagon_S2_asr_r_r_sat, 23811}, // __builtin_HEXAGON_S2_asr_r_r_sat
    {hexagon_S2_asr_r_svw_trun, 23834}, // __builtin_HEXAGON_S2_asr_r_svw_trun
    {hexagon_S2_asr_r_vh, 23860}, // __builtin_HEXAGON_S2_asr_r_vh
    {hexagon_S2_asr_r_vw, 23880}, // __builtin_HEXAGON_S2_asr_r_vw
    {hexagon_S2_brev, 23900}, // __builtin_HEXAGON_S2_brev
    {hexagon_S2_brevp, 23916}, // __builtin_HEXAGON_S2_brevp
    {hexagon_S2_cl0, 23933}, // __builtin_HEXAGON_S2_cl0
    {hexagon_S2_cl0p, 23948}, // __builtin_HEXAGON_S2_cl0p
    {hexagon_S2_cl1, 23964}, // __builtin_HEXAGON_S2_cl1
    {hexagon_S2_cl1p, 23979}, // __builtin_HEXAGON_S2_cl1p
    {hexagon_S2_clb, 23995}, // __builtin_HEXAGON_S2_clb
    {hexagon_S2_clbnorm, 24010}, // __builtin_HEXAGON_S2_clbnorm
    {hexagon_S2_clbp, 24029}, // __builtin_HEXAGON_S2_clbp
    {hexagon_S2_clrbit_i, 24045}, // __builtin_HEXAGON_S2_clrbit_i
    {hexagon_S2_clrbit_r, 24065}, // __builtin_HEXAGON_S2_clrbit_r
    {hexagon_S2_ct0, 24085}, // __builtin_HEXAGON_S2_ct0
    {hexagon_S2_ct0p, 24100}, // __builtin_HEXAGON_S2_ct0p
    {hexagon_S2_ct1, 24116}, // __builtin_HEXAGON_S2_ct1
    {hexagon_S2_ct1p, 24131}, // __builtin_HEXAGON_S2_ct1p
    {hexagon_S2_deinterleave, 24147}, // __builtin_HEXAGON_S2_deinterleave
    {hexagon_S2_extractu, 24171}, // __builtin_HEXAGON_S2_extractu
    {hexagon_S2_extractu_rp, 24191}, // __builtin_HEXAGON_S2_extractu_rp
    {hexagon_S2_extractup, 24214}, // __builtin_HEXAGON_S2_extractup
    {hexagon_S2_extractup_rp, 24235}, // __builtin_HEXAGON_S2_extractup_rp
    {hexagon_S2_insert, 24259}, // __builtin_HEXAGON_S2_insert
    {hexagon_S2_insert_rp, 24277}, // __builtin_HEXAGON_S2_insert_rp
    {hexagon_S2_insertp, 24298}, // __builtin_HEXAGON_S2_insertp
    {hexagon_S2_insertp_rp, 24317}, // __builtin_HEXAGON_S2_insertp_rp
    {hexagon_S2_interleave, 24339}, // __builtin_HEXAGON_S2_interleave
    {hexagon_S2_lfsp, 24361}, // __builtin_HEXAGON_S2_lfsp
    {hexagon_S2_lsl_r_p, 24377}, // __builtin_HEXAGON_S2_lsl_r_p
    {hexagon_S2_lsl_r_p_acc, 24396}, // __builtin_HEXAGON_S2_lsl_r_p_acc
    {hexagon_S2_lsl_r_p_and, 24419}, // __builtin_HEXAGON_S2_lsl_r_p_and
    {hexagon_S2_lsl_r_p_nac, 24442}, // __builtin_HEXAGON_S2_lsl_r_p_nac
    {hexagon_S2_lsl_r_p_or, 24465}, // __builtin_HEXAGON_S2_lsl_r_p_or
    {hexagon_S2_lsl_r_p_xor, 24487}, // __builtin_HEXAGON_S2_lsl_r_p_xor
    {hexagon_S2_lsl_r_r, 24510}, // __builtin_HEXAGON_S2_lsl_r_r
    {hexagon_S2_lsl_r_r_acc, 24529}, // __builtin_HEXAGON_S2_lsl_r_r_acc
    {hexagon_S2_lsl_r_r_and, 24552}, // __builtin_HEXAGON_S2_lsl_r_r_and
    {hexagon_S2_lsl_r_r_nac, 24575}, // __builtin_HEXAGON_S2_lsl_r_r_nac
    {hexagon_S2_lsl_r_r_or, 24598}, // __builtin_HEXAGON_S2_lsl_r_r_or
    {hexagon_S2_lsl_r_vh, 24620}, // __builtin_HEXAGON_S2_lsl_r_vh
    {hexagon_S2_lsl_r_vw, 24640}, // __builtin_HEXAGON_S2_lsl_r_vw
    {hexagon_S2_lsr_i_p, 24660}, // __builtin_HEXAGON_S2_lsr_i_p
    {hexagon_S2_lsr_i_p_acc, 24679}, // __builtin_HEXAGON_S2_lsr_i_p_acc
    {hexagon_S2_lsr_i_p_and, 24702}, // __builtin_HEXAGON_S2_lsr_i_p_and
    {hexagon_S2_lsr_i_p_nac, 24725}, // __builtin_HEXAGON_S2_lsr_i_p_nac
    {hexagon_S2_lsr_i_p_or, 24748}, // __builtin_HEXAGON_S2_lsr_i_p_or
    {hexagon_S2_lsr_i_p_xacc, 24770}, // __builtin_HEXAGON_S2_lsr_i_p_xacc
    {hexagon_S2_lsr_i_r, 24794}, // __builtin_HEXAGON_S2_lsr_i_r
    {hexagon_S2_lsr_i_r_acc, 24813}, // __builtin_HEXAGON_S2_lsr_i_r_acc
    {hexagon_S2_lsr_i_r_and, 24836}, // __builtin_HEXAGON_S2_lsr_i_r_and
    {hexagon_S2_lsr_i_r_nac, 24859}, // __builtin_HEXAGON_S2_lsr_i_r_nac
    {hexagon_S2_lsr_i_r_or, 24882}, // __builtin_HEXAGON_S2_lsr_i_r_or
    {hexagon_S2_lsr_i_r_xacc, 24904}, // __builtin_HEXAGON_S2_lsr_i_r_xacc
    {hexagon_S2_lsr_i_vh, 24928}, // __builtin_HEXAGON_S2_lsr_i_vh
    {hexagon_S2_lsr_i_vw, 24948}, // __builtin_HEXAGON_S2_lsr_i_vw
    {hexagon_S2_lsr_r_p, 24968}, // __builtin_HEXAGON_S2_lsr_r_p
    {hexagon_S2_lsr_r_p_acc, 24987}, // __builtin_HEXAGON_S2_lsr_r_p_acc
    {hexagon_S2_lsr_r_p_and, 25010}, // __builtin_HEXAGON_S2_lsr_r_p_and
    {hexagon_S2_lsr_r_p_nac, 25033}, // __builtin_HEXAGON_S2_lsr_r_p_nac
    {hexagon_S2_lsr_r_p_or, 25056}, // __builtin_HEXAGON_S2_lsr_r_p_or
    {hexagon_S2_lsr_r_p_xor, 25078}, // __builtin_HEXAGON_S2_lsr_r_p_xor
    {hexagon_S2_lsr_r_r, 25101}, // __builtin_HEXAGON_S2_lsr_r_r
    {hexagon_S2_lsr_r_r_acc, 25120}, // __builtin_HEXAGON_S2_lsr_r_r_acc
    {hexagon_S2_lsr_r_r_and, 25143}, // __builtin_HEXAGON_S2_lsr_r_r_and
    {hexagon_S2_lsr_r_r_nac, 25166}, // __builtin_HEXAGON_S2_lsr_r_r_nac
    {hexagon_S2_lsr_r_r_or, 25189}, // __builtin_HEXAGON_S2_lsr_r_r_or
    {hexagon_S2_lsr_r_vh, 25211}, // __builtin_HEXAGON_S2_lsr_r_vh
    {hexagon_S2_lsr_r_vw, 25231}, // __builtin_HEXAGON_S2_lsr_r_vw
    {hexagon_S2_mask, 25251}, // __builtin_HEXAGON_S2_mask
    {hexagon_S2_packhl, 25267}, // __builtin_HEXAGON_S2_packhl
    {hexagon_S2_parityp, 25285}, // __builtin_HEXAGON_S2_parityp
    {hexagon_S2_setbit_i, 25304}, // __builtin_HEXAGON_S2_setbit_i
    {hexagon_S2_setbit_r, 25324}, // __builtin_HEXAGON_S2_setbit_r
    {hexagon_S2_shuffeb, 25344}, // __builtin_HEXAGON_S2_shuffeb
    {hexagon_S2_shuffeh, 25363}, // __builtin_HEXAGON_S2_shuffeh
    {hexagon_S2_shuffob, 25382}, // __builtin_HEXAGON_S2_shuffob
    {hexagon_S2_shuffoh, 25401}, // __builtin_HEXAGON_S2_shuffoh
    {hexagon_S2_storew_locked, 25420}, // __builtin_HEXAGON_S2_storew_locked
    {hexagon_S2_svsathb, 25445}, // __builtin_HEXAGON_S2_svsathb
    {hexagon_S2_svsathub, 25464}, // __builtin_HEXAGON_S2_svsathub
    {hexagon_S2_tableidxb_goodsyntax, 25484}, // __builtin_HEXAGON_S2_tableidxb_goodsyntax
    {hexagon_S2_tableidxd_goodsyntax, 25516}, // __builtin_HEXAGON_S2_tableidxd_goodsyntax
    {hexagon_S2_tableidxh_goodsyntax, 25548}, // __builtin_HEXAGON_S2_tableidxh_goodsyntax
    {hexagon_S2_tableidxw_goodsyntax, 25580}, // __builtin_HEXAGON_S2_tableidxw_goodsyntax
    {hexagon_S2_togglebit_i, 25612}, // __builtin_HEXAGON_S2_togglebit_i
    {hexagon_S2_togglebit_r, 25635}, // __builtin_HEXAGON_S2_togglebit_r
    {hexagon_S2_tstbit_i, 25658}, // __builtin_HEXAGON_S2_tstbit_i
    {hexagon_S2_tstbit_r, 25678}, // __builtin_HEXAGON_S2_tstbit_r
    {hexagon_S2_valignib, 25698}, // __builtin_HEXAGON_S2_valignib
    {hexagon_S2_valignrb, 25718}, // __builtin_HEXAGON_S2_valignrb
    {hexagon_S2_vcnegh, 25738}, // __builtin_HEXAGON_S2_vcnegh
    {hexagon_S2_vcrotate, 25756}, // __builtin_HEXAGON_S2_vcrotate
    {hexagon_S2_vrcnegh, 25776}, // __builtin_HEXAGON_S2_vrcnegh
    {hexagon_S2_vrndpackwh, 25795}, // __builtin_HEXAGON_S2_vrndpackwh
    {hexagon_S2_vrndpackwhs, 25817}, // __builtin_HEXAGON_S2_vrndpackwhs
    {hexagon_S2_vsathb, 25840}, // __builtin_HEXAGON_S2_vsathb
    {hexagon_S2_vsathb_nopack, 25858}, // __builtin_HEXAGON_S2_vsathb_nopack
    {hexagon_S2_vsathub, 25883}, // __builtin_HEXAGON_S2_vsathub
    {hexagon_S2_vsathub_nopack, 25902}, // __builtin_HEXAGON_S2_vsathub_nopack
    {hexagon_S2_vsatwh, 25928}, // __builtin_HEXAGON_S2_vsatwh
    {hexagon_S2_vsatwh_nopack, 25946}, // __builtin_HEXAGON_S2_vsatwh_nopack
    {hexagon_S2_vsatwuh, 25971}, // __builtin_HEXAGON_S2_vsatwuh
    {hexagon_S2_vsatwuh_nopack, 25990}, // __builtin_HEXAGON_S2_vsatwuh_nopack
    {hexagon_S2_vsplatrb, 26016}, // __builtin_HEXAGON_S2_vsplatrb
    {hexagon_S2_vsplatrh, 26036}, // __builtin_HEXAGON_S2_vsplatrh
    {hexagon_S2_vspliceib, 26056}, // __builtin_HEXAGON_S2_vspliceib
    {hexagon_S2_vsplicerb, 26077}, // __builtin_HEXAGON_S2_vsplicerb
    {hexagon_S2_vsxtbh, 26098}, // __builtin_HEXAGON_S2_vsxtbh
    {hexagon_S2_vsxthw, 26116}, // __builtin_HEXAGON_S2_vsxthw
    {hexagon_S2_vtrunehb, 26134}, // __builtin_HEXAGON_S2_vtrunehb
    {hexagon_S2_vtrunewh, 26154}, // __builtin_HEXAGON_S2_vtrunewh
    {hexagon_S2_vtrunohb, 26174}, // __builtin_HEXAGON_S2_vtrunohb
    {hexagon_S2_vtrunowh, 26194}, // __builtin_HEXAGON_S2_vtrunowh
    {hexagon_S2_vzxtbh, 26214}, // __builtin_HEXAGON_S2_vzxtbh
    {hexagon_S2_vzxthw, 26232}, // __builtin_HEXAGON_S2_vzxthw
    {hexagon_S4_addaddi, 26250}, // __builtin_HEXAGON_S4_addaddi
    {hexagon_S4_addi_asl_ri, 26269}, // __builtin_HEXAGON_S4_addi_asl_ri
    {hexagon_S4_addi_lsr_ri, 26292}, // __builtin_HEXAGON_S4_addi_lsr_ri
    {hexagon_S4_andi_asl_ri, 26315}, // __builtin_HEXAGON_S4_andi_asl_ri
    {hexagon_S4_andi_lsr_ri, 26338}, // __builtin_HEXAGON_S4_andi_lsr_ri
    {hexagon_S4_clbaddi, 26361}, // __builtin_HEXAGON_S4_clbaddi
    {hexagon_S4_clbpaddi, 26380}, // __builtin_HEXAGON_S4_clbpaddi
    {hexagon_S4_clbpnorm, 26400}, // __builtin_HEXAGON_S4_clbpnorm
    {hexagon_S4_extract, 26420}, // __builtin_HEXAGON_S4_extract
    {hexagon_S4_extract_rp, 26439}, // __builtin_HEXAGON_S4_extract_rp
    {hexagon_S4_extractp, 26461}, // __builtin_HEXAGON_S4_extractp
    {hexagon_S4_extractp_rp, 26481}, // __builtin_HEXAGON_S4_extractp_rp
    {hexagon_S4_lsli, 26504}, // __builtin_HEXAGON_S4_lsli
    {hexagon_S4_ntstbit_i, 26520}, // __builtin_HEXAGON_S4_ntstbit_i
    {hexagon_S4_ntstbit_r, 26541}, // __builtin_HEXAGON_S4_ntstbit_r
    {hexagon_S4_or_andi, 26562}, // __builtin_HEXAGON_S4_or_andi
    {hexagon_S4_or_andix, 26581}, // __builtin_HEXAGON_S4_or_andix
    {hexagon_S4_or_ori, 26601}, // __builtin_HEXAGON_S4_or_ori
    {hexagon_S4_ori_asl_ri, 26619}, // __builtin_HEXAGON_S4_ori_asl_ri
    {hexagon_S4_ori_lsr_ri, 26641}, // __builtin_HEXAGON_S4_ori_lsr_ri
    {hexagon_S4_parity, 26663}, // __builtin_HEXAGON_S4_parity
    {hexagon_S4_stored_locked, 26681}, // __builtin_HEXAGON_S4_stored_locked
    {hexagon_S4_subaddi, 26706}, // __builtin_HEXAGON_S4_subaddi
    {hexagon_S4_subi_asl_ri, 26725}, // __builtin_HEXAGON_S4_subi_asl_ri
    {hexagon_S4_subi_lsr_ri, 26748}, // __builtin_HEXAGON_S4_subi_lsr_ri
    {hexagon_S4_vrcrotate, 26771}, // __builtin_HEXAGON_S4_vrcrotate
    {hexagon_S4_vrcrotate_acc, 26792}, // __builtin_HEXAGON_S4_vrcrotate_acc
    {hexagon_S4_vxaddsubh, 26817}, // __builtin_HEXAGON_S4_vxaddsubh
    {hexagon_S4_vxaddsubhr, 26838}, // __builtin_HEXAGON_S4_vxaddsubhr
    {hexagon_S4_vxaddsubw, 26860}, // __builtin_HEXAGON_S4_vxaddsubw
    {hexagon_S4_vxsubaddh, 26881}, // __builtin_HEXAGON_S4_vxsubaddh
    {hexagon_S4_vxsubaddhr, 26902}, // __builtin_HEXAGON_S4_vxsubaddhr
    {hexagon_S4_vxsubaddw, 26924}, // __builtin_HEXAGON_S4_vxsubaddw
    {hexagon_S5_asrhub_rnd_sat_goodsyntax, 26945}, // __builtin_HEXAGON_S5_asrhub_rnd_sat_goodsyntax
    {hexagon_S5_asrhub_sat, 26982}, // __builtin_HEXAGON_S5_asrhub_sat
    {hexagon_S5_popcountp, 27004}, // __builtin_HEXAGON_S5_popcountp
    {hexagon_S5_vasrhrnd_goodsyntax, 27025}, // __builtin_HEXAGON_S5_vasrhrnd_goodsyntax
    {hexagon_S6_rol_i_p, 27056}, // __builtin_HEXAGON_S6_rol_i_p
    {hexagon_S6_rol_i_p_acc, 27075}, // __builtin_HEXAGON_S6_rol_i_p_acc
    {hexagon_S6_rol_i_p_and, 27098}, // __builtin_HEXAGON_S6_rol_i_p_and
    {hexagon_S6_rol_i_p_nac, 27121}, // __builtin_HEXAGON_S6_rol_i_p_nac
    {hexagon_S6_rol_i_p_or, 27144}, // __builtin_HEXAGON_S6_rol_i_p_or
    {hexagon_S6_rol_i_p_xacc, 27166}, // __builtin_HEXAGON_S6_rol_i_p_xacc
    {hexagon_S6_rol_i_r, 27190}, // __builtin_HEXAGON_S6_rol_i_r
    {hexagon_S6_rol_i_r_acc, 27209}, // __builtin_HEXAGON_S6_rol_i_r_acc
    {hexagon_S6_rol_i_r_and, 27232}, // __builtin_HEXAGON_S6_rol_i_r_and
    {hexagon_S6_rol_i_r_nac, 27255}, // __builtin_HEXAGON_S6_rol_i_r_nac
    {hexagon_S6_rol_i_r_or, 27278}, // __builtin_HEXAGON_S6_rol_i_r_or
    {hexagon_S6_rol_i_r_xacc, 27300}, // __builtin_HEXAGON_S6_rol_i_r_xacc
    {hexagon_S6_vsplatrbp, 27324}, // __builtin_HEXAGON_S6_vsplatrbp
    {hexagon_S6_vtrunehb_ppp, 27345}, // __builtin_HEXAGON_S6_vtrunehb_ppp
    {hexagon_S6_vtrunohb_ppp, 27369}, // __builtin_HEXAGON_S6_vtrunohb_ppp
    {hexagon_V6_extractw, 27393}, // __builtin_HEXAGON_V6_extractw
    {hexagon_V6_extractw_128B, 27413}, // __builtin_HEXAGON_V6_extractw_128B
    {hexagon_V6_get_qfext, 27438}, // __builtin_HEXAGON_V6_get_qfext
    {hexagon_V6_get_qfext_128B, 27459}, // __builtin_HEXAGON_V6_get_qfext_128B
    {hexagon_V6_get_qfext_oracc, 27485}, // __builtin_HEXAGON_V6_get_qfext_oracc
    {hexagon_V6_get_qfext_oracc_128B, 27512}, // __builtin_HEXAGON_V6_get_qfext_oracc_128B
    {hexagon_V6_hi, 27544}, // __builtin_HEXAGON_V6_hi
    {hexagon_V6_hi_128B, 27558}, // __builtin_HEXAGON_V6_hi_128B
    {hexagon_V6_lo, 27577}, // __builtin_HEXAGON_V6_lo
    {hexagon_V6_lo_128B, 27591}, // __builtin_HEXAGON_V6_lo_128B
    {hexagon_V6_lvsplatb, 27610}, // __builtin_HEXAGON_V6_lvsplatb
    {hexagon_V6_lvsplatb_128B, 27630}, // __builtin_HEXAGON_V6_lvsplatb_128B
    {hexagon_V6_lvsplath, 27655}, // __builtin_HEXAGON_V6_lvsplath
    {hexagon_V6_lvsplath_128B, 27675}, // __builtin_HEXAGON_V6_lvsplath_128B
    {hexagon_V6_lvsplatw, 27700}, // __builtin_HEXAGON_V6_lvsplatw
    {hexagon_V6_lvsplatw_128B, 27720}, // __builtin_HEXAGON_V6_lvsplatw_128B
    {hexagon_V6_pred_and, 27745}, // __builtin_HEXAGON_V6_pred_and
    {hexagon_V6_pred_and_128B, 27765}, // __builtin_HEXAGON_V6_pred_and_128B
    {hexagon_V6_pred_and_n, 27790}, // __builtin_HEXAGON_V6_pred_and_n
    {hexagon_V6_pred_and_n_128B, 27812}, // __builtin_HEXAGON_V6_pred_and_n_128B
    {hexagon_V6_pred_not, 27839}, // __builtin_HEXAGON_V6_pred_not
    {hexagon_V6_pred_not_128B, 27859}, // __builtin_HEXAGON_V6_pred_not_128B
    {hexagon_V6_pred_or, 27884}, // __builtin_HEXAGON_V6_pred_or
    {hexagon_V6_pred_or_128B, 27903}, // __builtin_HEXAGON_V6_pred_or_128B
    {hexagon_V6_pred_or_n, 27927}, // __builtin_HEXAGON_V6_pred_or_n
    {hexagon_V6_pred_or_n_128B, 27948}, // __builtin_HEXAGON_V6_pred_or_n_128B
    {hexagon_V6_pred_scalar2, 27974}, // __builtin_HEXAGON_V6_pred_scalar2
    {hexagon_V6_pred_scalar2_128B, 27998}, // __builtin_HEXAGON_V6_pred_scalar2_128B
    {hexagon_V6_pred_scalar2v2, 28027}, // __builtin_HEXAGON_V6_pred_scalar2v2
    {hexagon_V6_pred_scalar2v2_128B, 28053}, // __builtin_HEXAGON_V6_pred_scalar2v2_128B
    {hexagon_V6_pred_xor, 28084}, // __builtin_HEXAGON_V6_pred_xor
    {hexagon_V6_pred_xor_128B, 28104}, // __builtin_HEXAGON_V6_pred_xor_128B
    {hexagon_V6_set_qfext, 28129}, // __builtin_HEXAGON_V6_set_qfext
    {hexagon_V6_set_qfext_128B, 28150}, // __builtin_HEXAGON_V6_set_qfext_128B
    {hexagon_V6_shuffeqh, 28176}, // __builtin_HEXAGON_V6_shuffeqh
    {hexagon_V6_shuffeqh_128B, 28196}, // __builtin_HEXAGON_V6_shuffeqh_128B
    {hexagon_V6_shuffeqw, 28221}, // __builtin_HEXAGON_V6_shuffeqw
    {hexagon_V6_shuffeqw_128B, 28241}, // __builtin_HEXAGON_V6_shuffeqw_128B
    {hexagon_V6_v6mpyhubs10, 28266}, // __builtin_HEXAGON_V6_v6mpyhubs10
    {hexagon_V6_v6mpyhubs10_128B, 28289}, // __builtin_HEXAGON_V6_v6mpyhubs10_128B
    {hexagon_V6_v6mpyhubs10_vxx, 28317}, // __builtin_HEXAGON_V6_v6mpyhubs10_vxx
    {hexagon_V6_v6mpyhubs10_vxx_128B, 28344}, // __builtin_HEXAGON_V6_v6mpyhubs10_vxx_128B
    {hexagon_V6_v6mpyvubs10, 28376}, // __builtin_HEXAGON_V6_v6mpyvubs10
    {hexagon_V6_v6mpyvubs10_128B, 28399}, // __builtin_HEXAGON_V6_v6mpyvubs10_128B
    {hexagon_V6_v6mpyvubs10_vxx, 28427}, // __builtin_HEXAGON_V6_v6mpyvubs10_vxx
    {hexagon_V6_v6mpyvubs10_vxx_128B, 28454}, // __builtin_HEXAGON_V6_v6mpyvubs10_vxx_128B
    {hexagon_V6_vS32b_nqpred_ai, 28486}, // __builtin_HEXAGON_V6_vS32b_nqpred_ai
    {hexagon_V6_vS32b_nqpred_ai_128B, 28513}, // __builtin_HEXAGON_V6_vS32b_nqpred_ai_128B
    {hexagon_V6_vS32b_nt_nqpred_ai, 28545}, // __builtin_HEXAGON_V6_vS32b_nt_nqpred_ai
    {hexagon_V6_vS32b_nt_nqpred_ai_128B, 28575}, // __builtin_HEXAGON_V6_vS32b_nt_nqpred_ai_128B
    {hexagon_V6_vS32b_nt_qpred_ai, 28610}, // __builtin_HEXAGON_V6_vS32b_nt_qpred_ai
    {hexagon_V6_vS32b_nt_qpred_ai_128B, 28639}, // __builtin_HEXAGON_V6_vS32b_nt_qpred_ai_128B
    {hexagon_V6_vS32b_qpred_ai, 28673}, // __builtin_HEXAGON_V6_vS32b_qpred_ai
    {hexagon_V6_vS32b_qpred_ai_128B, 28699}, // __builtin_HEXAGON_V6_vS32b_qpred_ai_128B
    {hexagon_V6_vabs_f8, 28730}, // __builtin_HEXAGON_V6_vabs_f8
    {hexagon_V6_vabs_f8_128B, 28749}, // __builtin_HEXAGON_V6_vabs_f8_128B
    {hexagon_V6_vabs_hf, 28773}, // __builtin_HEXAGON_V6_vabs_hf
    {hexagon_V6_vabs_hf_128B, 28792}, // __builtin_HEXAGON_V6_vabs_hf_128B
    {hexagon_V6_vabs_qf16_hf, 28816}, // __builtin_HEXAGON_V6_vabs_qf16_hf
    {hexagon_V6_vabs_qf16_hf_128B, 28840}, // __builtin_HEXAGON_V6_vabs_qf16_hf_128B
    {hexagon_V6_vabs_qf16_qf16, 28869}, // __builtin_HEXAGON_V6_vabs_qf16_qf16
    {hexagon_V6_vabs_qf16_qf16_128B, 28895}, // __builtin_HEXAGON_V6_vabs_qf16_qf16_128B
    {hexagon_V6_vabs_qf32_qf32, 28926}, // __builtin_HEXAGON_V6_vabs_qf32_qf32
    {hexagon_V6_vabs_qf32_qf32_128B, 28952}, // __builtin_HEXAGON_V6_vabs_qf32_qf32_128B
    {hexagon_V6_vabs_qf32_sf, 28983}, // __builtin_HEXAGON_V6_vabs_qf32_sf
    {hexagon_V6_vabs_qf32_sf_128B, 29007}, // __builtin_HEXAGON_V6_vabs_qf32_sf_128B
    {hexagon_V6_vabs_sf, 29036}, // __builtin_HEXAGON_V6_vabs_sf
    {hexagon_V6_vabs_sf_128B, 29055}, // __builtin_HEXAGON_V6_vabs_sf_128B
    {hexagon_V6_vabsb, 29079}, // __builtin_HEXAGON_V6_vabsb
    {hexagon_V6_vabsb_128B, 29096}, // __builtin_HEXAGON_V6_vabsb_128B
    {hexagon_V6_vabsb_sat, 29118}, // __builtin_HEXAGON_V6_vabsb_sat
    {hexagon_V6_vabsb_sat_128B, 29139}, // __builtin_HEXAGON_V6_vabsb_sat_128B
    {hexagon_V6_vabsdiffh, 29165}, // __builtin_HEXAGON_V6_vabsdiffh
    {hexagon_V6_vabsdiffh_128B, 29186}, // __builtin_HEXAGON_V6_vabsdiffh_128B
    {hexagon_V6_vabsdiffub, 29212}, // __builtin_HEXAGON_V6_vabsdiffub
    {hexagon_V6_vabsdiffub_128B, 29234}, // __builtin_HEXAGON_V6_vabsdiffub_128B
    {hexagon_V6_vabsdiffuh, 29261}, // __builtin_HEXAGON_V6_vabsdiffuh
    {hexagon_V6_vabsdiffuh_128B, 29283}, // __builtin_HEXAGON_V6_vabsdiffuh_128B
    {hexagon_V6_vabsdiffw, 29310}, // __builtin_HEXAGON_V6_vabsdiffw
    {hexagon_V6_vabsdiffw_128B, 29331}, // __builtin_HEXAGON_V6_vabsdiffw_128B
    {hexagon_V6_vabsh, 29357}, // __builtin_HEXAGON_V6_vabsh
    {hexagon_V6_vabsh_128B, 29374}, // __builtin_HEXAGON_V6_vabsh_128B
    {hexagon_V6_vabsh_sat, 29396}, // __builtin_HEXAGON_V6_vabsh_sat
    {hexagon_V6_vabsh_sat_128B, 29417}, // __builtin_HEXAGON_V6_vabsh_sat_128B
    {hexagon_V6_vabsw, 29443}, // __builtin_HEXAGON_V6_vabsw
    {hexagon_V6_vabsw_128B, 29460}, // __builtin_HEXAGON_V6_vabsw_128B
    {hexagon_V6_vabsw_sat, 29482}, // __builtin_HEXAGON_V6_vabsw_sat
    {hexagon_V6_vabsw_sat_128B, 29503}, // __builtin_HEXAGON_V6_vabsw_sat_128B
    {hexagon_V6_vadd_hf, 29529}, // __builtin_HEXAGON_V6_vadd_hf
    {hexagon_V6_vadd_hf_128B, 29548}, // __builtin_HEXAGON_V6_vadd_hf_128B
    {hexagon_V6_vadd_hf_f8, 29572}, // __builtin_HEXAGON_V6_vadd_hf_f8
    {hexagon_V6_vadd_hf_f8_128B, 29594}, // __builtin_HEXAGON_V6_vadd_hf_f8_128B
    {hexagon_V6_vadd_hf_hf, 29621}, // __builtin_HEXAGON_V6_vadd_hf_hf
    {hexagon_V6_vadd_hf_hf_128B, 29643}, // __builtin_HEXAGON_V6_vadd_hf_hf_128B
    {hexagon_V6_vadd_qf16, 29670}, // __builtin_HEXAGON_V6_vadd_qf16
    {hexagon_V6_vadd_qf16_128B, 29691}, // __builtin_HEXAGON_V6_vadd_qf16_128B
    {hexagon_V6_vadd_qf16_mix, 29717}, // __builtin_HEXAGON_V6_vadd_qf16_mix
    {hexagon_V6_vadd_qf16_mix_128B, 29742}, // __builtin_HEXAGON_V6_vadd_qf16_mix_128B
    {hexagon_V6_vadd_qf32, 29772}, // __builtin_HEXAGON_V6_vadd_qf32
    {hexagon_V6_vadd_qf32_128B, 29793}, // __builtin_HEXAGON_V6_vadd_qf32_128B
    {hexagon_V6_vadd_qf32_mix, 29819}, // __builtin_HEXAGON_V6_vadd_qf32_mix
    {hexagon_V6_vadd_qf32_mix_128B, 29844}, // __builtin_HEXAGON_V6_vadd_qf32_mix_128B
    {hexagon_V6_vadd_sf, 29874}, // __builtin_HEXAGON_V6_vadd_sf
    {hexagon_V6_vadd_sf_128B, 29893}, // __builtin_HEXAGON_V6_vadd_sf_128B
    {hexagon_V6_vadd_sf_bf, 29917}, // __builtin_HEXAGON_V6_vadd_sf_bf
    {hexagon_V6_vadd_sf_bf_128B, 29939}, // __builtin_HEXAGON_V6_vadd_sf_bf_128B
    {hexagon_V6_vadd_sf_hf, 29966}, // __builtin_HEXAGON_V6_vadd_sf_hf
    {hexagon_V6_vadd_sf_hf_128B, 29988}, // __builtin_HEXAGON_V6_vadd_sf_hf_128B
    {hexagon_V6_vadd_sf_sf, 30015}, // __builtin_HEXAGON_V6_vadd_sf_sf
    {hexagon_V6_vadd_sf_sf_128B, 30037}, // __builtin_HEXAGON_V6_vadd_sf_sf_128B
    {hexagon_V6_vaddb, 30064}, // __builtin_HEXAGON_V6_vaddb
    {hexagon_V6_vaddb_128B, 30081}, // __builtin_HEXAGON_V6_vaddb_128B
    {hexagon_V6_vaddb_dv, 30103}, // __builtin_HEXAGON_V6_vaddb_dv
    {hexagon_V6_vaddb_dv_128B, 30123}, // __builtin_HEXAGON_V6_vaddb_dv_128B
    {hexagon_V6_vaddbnq, 30148}, // __builtin_HEXAGON_V6_vaddbnq
    {hexagon_V6_vaddbnq_128B, 30167}, // __builtin_HEXAGON_V6_vaddbnq_128B
    {hexagon_V6_vaddbq, 30191}, // __builtin_HEXAGON_V6_vaddbq
    {hexagon_V6_vaddbq_128B, 30209}, // __builtin_HEXAGON_V6_vaddbq_128B
    {hexagon_V6_vaddbsat, 30232}, // __builtin_HEXAGON_V6_vaddbsat
    {hexagon_V6_vaddbsat_128B, 30252}, // __builtin_HEXAGON_V6_vaddbsat_128B
    {hexagon_V6_vaddbsat_dv, 30277}, // __builtin_HEXAGON_V6_vaddbsat_dv
    {hexagon_V6_vaddbsat_dv_128B, 30300}, // __builtin_HEXAGON_V6_vaddbsat_dv_128B
    {hexagon_V6_vaddcarrysat, 30328}, // __builtin_HEXAGON_V6_vaddcarrysat
    {hexagon_V6_vaddcarrysat_128B, 30352}, // __builtin_HEXAGON_V6_vaddcarrysat_128B
    {hexagon_V6_vaddclbh, 30381}, // __builtin_HEXAGON_V6_vaddclbh
    {hexagon_V6_vaddclbh_128B, 30401}, // __builtin_HEXAGON_V6_vaddclbh_128B
    {hexagon_V6_vaddclbw, 30426}, // __builtin_HEXAGON_V6_vaddclbw
    {hexagon_V6_vaddclbw_128B, 30446}, // __builtin_HEXAGON_V6_vaddclbw_128B
    {hexagon_V6_vaddh, 30471}, // __builtin_HEXAGON_V6_vaddh
    {hexagon_V6_vaddh_128B, 30488}, // __builtin_HEXAGON_V6_vaddh_128B
    {hexagon_V6_vaddh_dv, 30510}, // __builtin_HEXAGON_V6_vaddh_dv
    {hexagon_V6_vaddh_dv_128B, 30530}, // __builtin_HEXAGON_V6_vaddh_dv_128B
    {hexagon_V6_vaddhnq, 30555}, // __builtin_HEXAGON_V6_vaddhnq
    {hexagon_V6_vaddhnq_128B, 30574}, // __builtin_HEXAGON_V6_vaddhnq_128B
    {hexagon_V6_vaddhq, 30598}, // __builtin_HEXAGON_V6_vaddhq
    {hexagon_V6_vaddhq_128B, 30616}, // __builtin_HEXAGON_V6_vaddhq_128B
    {hexagon_V6_vaddhsat, 30639}, // __builtin_HEXAGON_V6_vaddhsat
    {hexagon_V6_vaddhsat_128B, 30659}, // __builtin_HEXAGON_V6_vaddhsat_128B
    {hexagon_V6_vaddhsat_dv, 30684}, // __builtin_HEXAGON_V6_vaddhsat_dv
    {hexagon_V6_vaddhsat_dv_128B, 30707}, // __builtin_HEXAGON_V6_vaddhsat_dv_128B
    {hexagon_V6_vaddhw, 30735}, // __builtin_HEXAGON_V6_vaddhw
    {hexagon_V6_vaddhw_128B, 30753}, // __builtin_HEXAGON_V6_vaddhw_128B
    {hexagon_V6_vaddhw_acc, 30776}, // __builtin_HEXAGON_V6_vaddhw_acc
    {hexagon_V6_vaddhw_acc_128B, 30798}, // __builtin_HEXAGON_V6_vaddhw_acc_128B
    {hexagon_V6_vaddubh, 30825}, // __builtin_HEXAGON_V6_vaddubh
    {hexagon_V6_vaddubh_128B, 30844}, // __builtin_HEXAGON_V6_vaddubh_128B
    {hexagon_V6_vaddubh_acc, 30868}, // __builtin_HEXAGON_V6_vaddubh_acc
    {hexagon_V6_vaddubh_acc_128B, 30891}, // __builtin_HEXAGON_V6_vaddubh_acc_128B
    {hexagon_V6_vaddubsat, 30919}, // __builtin_HEXAGON_V6_vaddubsat
    {hexagon_V6_vaddubsat_128B, 30940}, // __builtin_HEXAGON_V6_vaddubsat_128B
    {hexagon_V6_vaddubsat_dv, 30966}, // __builtin_HEXAGON_V6_vaddubsat_dv
    {hexagon_V6_vaddubsat_dv_128B, 30990}, // __builtin_HEXAGON_V6_vaddubsat_dv_128B
    {hexagon_V6_vaddububb_sat, 31019}, // __builtin_HEXAGON_V6_vaddububb_sat
    {hexagon_V6_vaddububb_sat_128B, 31044}, // __builtin_HEXAGON_V6_vaddububb_sat_128B
    {hexagon_V6_vadduhsat, 31074}, // __builtin_HEXAGON_V6_vadduhsat
    {hexagon_V6_vadduhsat_128B, 31095}, // __builtin_HEXAGON_V6_vadduhsat_128B
    {hexagon_V6_vadduhsat_dv, 31121}, // __builtin_HEXAGON_V6_vadduhsat_dv
    {hexagon_V6_vadduhsat_dv_128B, 31145}, // __builtin_HEXAGON_V6_vadduhsat_dv_128B
    {hexagon_V6_vadduhw, 31174}, // __builtin_HEXAGON_V6_vadduhw
    {hexagon_V6_vadduhw_128B, 31193}, // __builtin_HEXAGON_V6_vadduhw_128B
    {hexagon_V6_vadduhw_acc, 31217}, // __builtin_HEXAGON_V6_vadduhw_acc
    {hexagon_V6_vadduhw_acc_128B, 31240}, // __builtin_HEXAGON_V6_vadduhw_acc_128B
    {hexagon_V6_vadduwsat, 31268}, // __builtin_HEXAGON_V6_vadduwsat
    {hexagon_V6_vadduwsat_128B, 31289}, // __builtin_HEXAGON_V6_vadduwsat_128B
    {hexagon_V6_vadduwsat_dv, 31315}, // __builtin_HEXAGON_V6_vadduwsat_dv
    {hexagon_V6_vadduwsat_dv_128B, 31339}, // __builtin_HEXAGON_V6_vadduwsat_dv_128B
    {hexagon_V6_vaddw, 31368}, // __builtin_HEXAGON_V6_vaddw
    {hexagon_V6_vaddw_128B, 31385}, // __builtin_HEXAGON_V6_vaddw_128B
    {hexagon_V6_vaddw_dv, 31407}, // __builtin_HEXAGON_V6_vaddw_dv
    {hexagon_V6_vaddw_dv_128B, 31427}, // __builtin_HEXAGON_V6_vaddw_dv_128B
    {hexagon_V6_vaddwnq, 31452}, // __builtin_HEXAGON_V6_vaddwnq
    {hexagon_V6_vaddwnq_128B, 31471}, // __builtin_HEXAGON_V6_vaddwnq_128B
    {hexagon_V6_vaddwq, 31495}, // __builtin_HEXAGON_V6_vaddwq
    {hexagon_V6_vaddwq_128B, 31513}, // __builtin_HEXAGON_V6_vaddwq_128B
    {hexagon_V6_vaddwsat, 31536}, // __builtin_HEXAGON_V6_vaddwsat
    {hexagon_V6_vaddwsat_128B, 31556}, // __builtin_HEXAGON_V6_vaddwsat_128B
    {hexagon_V6_vaddwsat_dv, 31581}, // __builtin_HEXAGON_V6_vaddwsat_dv
    {hexagon_V6_vaddwsat_dv_128B, 31604}, // __builtin_HEXAGON_V6_vaddwsat_dv_128B
    {hexagon_V6_valign4, 31632}, // __builtin_HEXAGON_V6_valign4
    {hexagon_V6_valign4_128B, 31651}, // __builtin_HEXAGON_V6_valign4_128B
    {hexagon_V6_valignb, 31675}, // __builtin_HEXAGON_V6_valignb
    {hexagon_V6_valignb_128B, 31694}, // __builtin_HEXAGON_V6_valignb_128B
    {hexagon_V6_valignbi, 31718}, // __builtin_HEXAGON_V6_valignbi
    {hexagon_V6_valignbi_128B, 31738}, // __builtin_HEXAGON_V6_valignbi_128B
    {hexagon_V6_vand, 31763}, // __builtin_HEXAGON_V6_vand
    {hexagon_V6_vand_128B, 31779}, // __builtin_HEXAGON_V6_vand_128B
    {hexagon_V6_vandnqrt, 31800}, // __builtin_HEXAGON_V6_vandnqrt
    {hexagon_V6_vandnqrt_128B, 31820}, // __builtin_HEXAGON_V6_vandnqrt_128B
    {hexagon_V6_vandnqrt_acc, 31845}, // __builtin_HEXAGON_V6_vandnqrt_acc
    {hexagon_V6_vandnqrt_acc_128B, 31869}, // __builtin_HEXAGON_V6_vandnqrt_acc_128B
    {hexagon_V6_vandqrt, 31898}, // __builtin_HEXAGON_V6_vandqrt
    {hexagon_V6_vandqrt_128B, 31917}, // __builtin_HEXAGON_V6_vandqrt_128B
    {hexagon_V6_vandqrt_acc, 31941}, // __builtin_HEXAGON_V6_vandqrt_acc
    {hexagon_V6_vandqrt_acc_128B, 31964}, // __builtin_HEXAGON_V6_vandqrt_acc_128B
    {hexagon_V6_vandvnqv, 31992}, // __builtin_HEXAGON_V6_vandvnqv
    {hexagon_V6_vandvnqv_128B, 32012}, // __builtin_HEXAGON_V6_vandvnqv_128B
    {hexagon_V6_vandvqv, 32037}, // __builtin_HEXAGON_V6_vandvqv
    {hexagon_V6_vandvqv_128B, 32056}, // __builtin_HEXAGON_V6_vandvqv_128B
    {hexagon_V6_vandvrt, 32080}, // __builtin_HEXAGON_V6_vandvrt
    {hexagon_V6_vandvrt_128B, 32099}, // __builtin_HEXAGON_V6_vandvrt_128B
    {hexagon_V6_vandvrt_acc, 32123}, // __builtin_HEXAGON_V6_vandvrt_acc
    {hexagon_V6_vandvrt_acc_128B, 32146}, // __builtin_HEXAGON_V6_vandvrt_acc_128B
    {hexagon_V6_vaslh, 32174}, // __builtin_HEXAGON_V6_vaslh
    {hexagon_V6_vaslh_128B, 32191}, // __builtin_HEXAGON_V6_vaslh_128B
    {hexagon_V6_vaslh_acc, 32213}, // __builtin_HEXAGON_V6_vaslh_acc
    {hexagon_V6_vaslh_acc_128B, 32234}, // __builtin_HEXAGON_V6_vaslh_acc_128B
    {hexagon_V6_vaslhv, 32260}, // __builtin_HEXAGON_V6_vaslhv
    {hexagon_V6_vaslhv_128B, 32278}, // __builtin_HEXAGON_V6_vaslhv_128B
    {hexagon_V6_vaslw, 32301}, // __builtin_HEXAGON_V6_vaslw
    {hexagon_V6_vaslw_128B, 32318}, // __builtin_HEXAGON_V6_vaslw_128B
    {hexagon_V6_vaslw_acc, 32340}, // __builtin_HEXAGON_V6_vaslw_acc
    {hexagon_V6_vaslw_acc_128B, 32361}, // __builtin_HEXAGON_V6_vaslw_acc_128B
    {hexagon_V6_vaslwv, 32387}, // __builtin_HEXAGON_V6_vaslwv
    {hexagon_V6_vaslwv_128B, 32405}, // __builtin_HEXAGON_V6_vaslwv_128B
    {hexagon_V6_vasr_into, 32428}, // __builtin_HEXAGON_V6_vasr_into
    {hexagon_V6_vasr_into_128B, 32449}, // __builtin_HEXAGON_V6_vasr_into_128B
    {hexagon_V6_vasrh, 32475}, // __builtin_HEXAGON_V6_vasrh
    {hexagon_V6_vasrh_128B, 32492}, // __builtin_HEXAGON_V6_vasrh_128B
    {hexagon_V6_vasrh_acc, 32514}, // __builtin_HEXAGON_V6_vasrh_acc
    {hexagon_V6_vasrh_acc_128B, 32535}, // __builtin_HEXAGON_V6_vasrh_acc_128B
    {hexagon_V6_vasrhbrndsat, 32561}, // __builtin_HEXAGON_V6_vasrhbrndsat
    {hexagon_V6_vasrhbrndsat_128B, 32585}, // __builtin_HEXAGON_V6_vasrhbrndsat_128B
    {hexagon_V6_vasrhbsat, 32614}, // __builtin_HEXAGON_V6_vasrhbsat
    {hexagon_V6_vasrhbsat_128B, 32635}, // __builtin_HEXAGON_V6_vasrhbsat_128B
    {hexagon_V6_vasrhubrndsat, 32661}, // __builtin_HEXAGON_V6_vasrhubrndsat
    {hexagon_V6_vasrhubrndsat_128B, 32686}, // __builtin_HEXAGON_V6_vasrhubrndsat_128B
    {hexagon_V6_vasrhubsat, 32716}, // __builtin_HEXAGON_V6_vasrhubsat
    {hexagon_V6_vasrhubsat_128B, 32738}, // __builtin_HEXAGON_V6_vasrhubsat_128B
    {hexagon_V6_vasrhv, 32765}, // __builtin_HEXAGON_V6_vasrhv
    {hexagon_V6_vasrhv_128B, 32783}, // __builtin_HEXAGON_V6_vasrhv_128B
    {hexagon_V6_vasruhubrndsat, 32806}, // __builtin_HEXAGON_V6_vasruhubrndsat
    {hexagon_V6_vasruhubrndsat_128B, 32832}, // __builtin_HEXAGON_V6_vasruhubrndsat_128B
    {hexagon_V6_vasruhubsat, 32863}, // __builtin_HEXAGON_V6_vasruhubsat
    {hexagon_V6_vasruhubsat_128B, 32886}, // __builtin_HEXAGON_V6_vasruhubsat_128B
    {hexagon_V6_vasruwuhrndsat, 32914}, // __builtin_HEXAGON_V6_vasruwuhrndsat
    {hexagon_V6_vasruwuhrndsat_128B, 32940}, // __builtin_HEXAGON_V6_vasruwuhrndsat_128B
    {hexagon_V6_vasruwuhsat, 32971}, // __builtin_HEXAGON_V6_vasruwuhsat
    {hexagon_V6_vasruwuhsat_128B, 32994}, // __builtin_HEXAGON_V6_vasruwuhsat_128B
    {hexagon_V6_vasrvuhubrndsat, 33022}, // __builtin_HEXAGON_V6_vasrvuhubrndsat
    {hexagon_V6_vasrvuhubrndsat_128B, 33049}, // __builtin_HEXAGON_V6_vasrvuhubrndsat_128B
    {hexagon_V6_vasrvuhubsat, 33081}, // __builtin_HEXAGON_V6_vasrvuhubsat
    {hexagon_V6_vasrvuhubsat_128B, 33105}, // __builtin_HEXAGON_V6_vasrvuhubsat_128B
    {hexagon_V6_vasrvwuhrndsat, 33134}, // __builtin_HEXAGON_V6_vasrvwuhrndsat
    {hexagon_V6_vasrvwuhrndsat_128B, 33160}, // __builtin_HEXAGON_V6_vasrvwuhrndsat_128B
    {hexagon_V6_vasrvwuhsat, 33191}, // __builtin_HEXAGON_V6_vasrvwuhsat
    {hexagon_V6_vasrvwuhsat_128B, 33214}, // __builtin_HEXAGON_V6_vasrvwuhsat_128B
    {hexagon_V6_vasrw, 33242}, // __builtin_HEXAGON_V6_vasrw
    {hexagon_V6_vasrw_128B, 33259}, // __builtin_HEXAGON_V6_vasrw_128B
    {hexagon_V6_vasrw_acc, 33281}, // __builtin_HEXAGON_V6_vasrw_acc
    {hexagon_V6_vasrw_acc_128B, 33302}, // __builtin_HEXAGON_V6_vasrw_acc_128B
    {hexagon_V6_vasrwh, 33328}, // __builtin_HEXAGON_V6_vasrwh
    {hexagon_V6_vasrwh_128B, 33346}, // __builtin_HEXAGON_V6_vasrwh_128B
    {hexagon_V6_vasrwhrndsat, 33369}, // __builtin_HEXAGON_V6_vasrwhrndsat
    {hexagon_V6_vasrwhrndsat_128B, 33393}, // __builtin_HEXAGON_V6_vasrwhrndsat_128B
    {hexagon_V6_vasrwhsat, 33422}, // __builtin_HEXAGON_V6_vasrwhsat
    {hexagon_V6_vasrwhsat_128B, 33443}, // __builtin_HEXAGON_V6_vasrwhsat_128B
    {hexagon_V6_vasrwuhrndsat, 33469}, // __builtin_HEXAGON_V6_vasrwuhrndsat
    {hexagon_V6_vasrwuhrndsat_128B, 33494}, // __builtin_HEXAGON_V6_vasrwuhrndsat_128B
    {hexagon_V6_vasrwuhsat, 33524}, // __builtin_HEXAGON_V6_vasrwuhsat
    {hexagon_V6_vasrwuhsat_128B, 33546}, // __builtin_HEXAGON_V6_vasrwuhsat_128B
    {hexagon_V6_vasrwv, 33573}, // __builtin_HEXAGON_V6_vasrwv
    {hexagon_V6_vasrwv_128B, 33591}, // __builtin_HEXAGON_V6_vasrwv_128B
    {hexagon_V6_vassign, 33614}, // __builtin_HEXAGON_V6_vassign
    {hexagon_V6_vassign_128B, 33633}, // __builtin_HEXAGON_V6_vassign_128B
    {hexagon_V6_vassign_fp, 33657}, // __builtin_HEXAGON_V6_vassign_fp
    {hexagon_V6_vassign_fp_128B, 33679}, // __builtin_HEXAGON_V6_vassign_fp_128B
    {hexagon_V6_vassignp, 33706}, // __builtin_HEXAGON_V6_vassignp
    {hexagon_V6_vassignp_128B, 33726}, // __builtin_HEXAGON_V6_vassignp_128B
    {hexagon_V6_vavgb, 33751}, // __builtin_HEXAGON_V6_vavgb
    {hexagon_V6_vavgb_128B, 33768}, // __builtin_HEXAGON_V6_vavgb_128B
    {hexagon_V6_vavgbrnd, 33790}, // __builtin_HEXAGON_V6_vavgbrnd
    {hexagon_V6_vavgbrnd_128B, 33810}, // __builtin_HEXAGON_V6_vavgbrnd_128B
    {hexagon_V6_vavgh, 33835}, // __builtin_HEXAGON_V6_vavgh
    {hexagon_V6_vavgh_128B, 33852}, // __builtin_HEXAGON_V6_vavgh_128B
    {hexagon_V6_vavghrnd, 33874}, // __builtin_HEXAGON_V6_vavghrnd
    {hexagon_V6_vavghrnd_128B, 33894}, // __builtin_HEXAGON_V6_vavghrnd_128B
    {hexagon_V6_vavgub, 33919}, // __builtin_HEXAGON_V6_vavgub
    {hexagon_V6_vavgub_128B, 33937}, // __builtin_HEXAGON_V6_vavgub_128B
    {hexagon_V6_vavgubrnd, 33960}, // __builtin_HEXAGON_V6_vavgubrnd
    {hexagon_V6_vavgubrnd_128B, 33981}, // __builtin_HEXAGON_V6_vavgubrnd_128B
    {hexagon_V6_vavguh, 34007}, // __builtin_HEXAGON_V6_vavguh
    {hexagon_V6_vavguh_128B, 34025}, // __builtin_HEXAGON_V6_vavguh_128B
    {hexagon_V6_vavguhrnd, 34048}, // __builtin_HEXAGON_V6_vavguhrnd
    {hexagon_V6_vavguhrnd_128B, 34069}, // __builtin_HEXAGON_V6_vavguhrnd_128B
    {hexagon_V6_vavguw, 34095}, // __builtin_HEXAGON_V6_vavguw
    {hexagon_V6_vavguw_128B, 34113}, // __builtin_HEXAGON_V6_vavguw_128B
    {hexagon_V6_vavguwrnd, 34136}, // __builtin_HEXAGON_V6_vavguwrnd
    {hexagon_V6_vavguwrnd_128B, 34157}, // __builtin_HEXAGON_V6_vavguwrnd_128B
    {hexagon_V6_vavgw, 34183}, // __builtin_HEXAGON_V6_vavgw
    {hexagon_V6_vavgw_128B, 34200}, // __builtin_HEXAGON_V6_vavgw_128B
    {hexagon_V6_vavgwrnd, 34222}, // __builtin_HEXAGON_V6_vavgwrnd
    {hexagon_V6_vavgwrnd_128B, 34242}, // __builtin_HEXAGON_V6_vavgwrnd_128B
    {hexagon_V6_vcl0h, 34267}, // __builtin_HEXAGON_V6_vcl0h
    {hexagon_V6_vcl0h_128B, 34284}, // __builtin_HEXAGON_V6_vcl0h_128B
    {hexagon_V6_vcl0w, 34306}, // __builtin_HEXAGON_V6_vcl0w
    {hexagon_V6_vcl0w_128B, 34323}, // __builtin_HEXAGON_V6_vcl0w_128B
    {hexagon_V6_vcombine, 34345}, // __builtin_HEXAGON_V6_vcombine
    {hexagon_V6_vcombine_128B, 34365}, // __builtin_HEXAGON_V6_vcombine_128B
    {hexagon_V6_vconv_bf_qf32, 34390}, // __builtin_HEXAGON_V6_vconv_bf_qf32
    {hexagon_V6_vconv_bf_qf32_128B, 34415}, // __builtin_HEXAGON_V6_vconv_bf_qf32_128B
    {hexagon_V6_vconv_f8_qf16, 34445}, // __builtin_HEXAGON_V6_vconv_f8_qf16
    {hexagon_V6_vconv_f8_qf16_128B, 34470}, // __builtin_HEXAGON_V6_vconv_f8_qf16_128B
    {hexagon_V6_vconv_h_hf, 34500}, // __builtin_HEXAGON_V6_vconv_h_hf
    {hexagon_V6_vconv_h_hf_128B, 34522}, // __builtin_HEXAGON_V6_vconv_h_hf_128B
    {hexagon_V6_vconv_h_hf_rnd, 34549}, // __builtin_HEXAGON_V6_vconv_h_hf_rnd
    {hexagon_V6_vconv_h_hf_rnd_128B, 34575}, // __builtin_HEXAGON_V6_vconv_h_hf_rnd_128B
    {hexagon_V6_vconv_hf_h, 34606}, // __builtin_HEXAGON_V6_vconv_hf_h
    {hexagon_V6_vconv_hf_h_128B, 34628}, // __builtin_HEXAGON_V6_vconv_hf_h_128B
    {hexagon_V6_vconv_hf_qf16, 34655}, // __builtin_HEXAGON_V6_vconv_hf_qf16
    {hexagon_V6_vconv_hf_qf16_128B, 34680}, // __builtin_HEXAGON_V6_vconv_hf_qf16_128B
    {hexagon_V6_vconv_hf_qf32, 34710}, // __builtin_HEXAGON_V6_vconv_hf_qf32
    {hexagon_V6_vconv_hf_qf32_128B, 34735}, // __builtin_HEXAGON_V6_vconv_hf_qf32_128B
    {hexagon_V6_vconv_qf16_f8, 34765}, // __builtin_HEXAGON_V6_vconv_qf16_f8
    {hexagon_V6_vconv_qf16_f8_128B, 34790}, // __builtin_HEXAGON_V6_vconv_qf16_f8_128B
    {hexagon_V6_vconv_qf16_hf, 34820}, // __builtin_HEXAGON_V6_vconv_qf16_hf
    {hexagon_V6_vconv_qf16_hf_128B, 34845}, // __builtin_HEXAGON_V6_vconv_qf16_hf_128B
    {hexagon_V6_vconv_qf16_qf16, 34875}, // __builtin_HEXAGON_V6_vconv_qf16_qf16
    {hexagon_V6_vconv_qf16_qf16_128B, 34902}, // __builtin_HEXAGON_V6_vconv_qf16_qf16_128B
    {hexagon_V6_vconv_qf32_qf32, 34934}, // __builtin_HEXAGON_V6_vconv_qf32_qf32
    {hexagon_V6_vconv_qf32_qf32_128B, 34961}, // __builtin_HEXAGON_V6_vconv_qf32_qf32_128B
    {hexagon_V6_vconv_qf32_sf, 34993}, // __builtin_HEXAGON_V6_vconv_qf32_sf
    {hexagon_V6_vconv_qf32_sf_128B, 35018}, // __builtin_HEXAGON_V6_vconv_qf32_sf_128B
    {hexagon_V6_vconv_sf_qf32, 35048}, // __builtin_HEXAGON_V6_vconv_sf_qf32
    {hexagon_V6_vconv_sf_qf32_128B, 35073}, // __builtin_HEXAGON_V6_vconv_sf_qf32_128B
    {hexagon_V6_vconv_sf_w, 35103}, // __builtin_HEXAGON_V6_vconv_sf_w
    {hexagon_V6_vconv_sf_w_128B, 35125}, // __builtin_HEXAGON_V6_vconv_sf_w_128B
    {hexagon_V6_vconv_w_sf, 35152}, // __builtin_HEXAGON_V6_vconv_w_sf
    {hexagon_V6_vconv_w_sf_128B, 35174}, // __builtin_HEXAGON_V6_vconv_w_sf_128B
    {hexagon_V6_vcvt2_b_hf, 35201}, // __builtin_HEXAGON_V6_vcvt2_b_hf
    {hexagon_V6_vcvt2_b_hf_128B, 35223}, // __builtin_HEXAGON_V6_vcvt2_b_hf_128B
    {hexagon_V6_vcvt2_hf_b, 35250}, // __builtin_HEXAGON_V6_vcvt2_hf_b
    {hexagon_V6_vcvt2_hf_b_128B, 35272}, // __builtin_HEXAGON_V6_vcvt2_hf_b_128B
    {hexagon_V6_vcvt2_hf_ub, 35299}, // __builtin_HEXAGON_V6_vcvt2_hf_ub
    {hexagon_V6_vcvt2_hf_ub_128B, 35322}, // __builtin_HEXAGON_V6_vcvt2_hf_ub_128B
    {hexagon_V6_vcvt2_ub_hf, 35350}, // __builtin_HEXAGON_V6_vcvt2_ub_hf
    {hexagon_V6_vcvt2_ub_hf_128B, 35373}, // __builtin_HEXAGON_V6_vcvt2_ub_hf_128B
    {hexagon_V6_vcvt_b_hf, 35401}, // __builtin_HEXAGON_V6_vcvt_b_hf
    {hexagon_V6_vcvt_b_hf_128B, 35422}, // __builtin_HEXAGON_V6_vcvt_b_hf_128B
    {hexagon_V6_vcvt_bf_sf, 35448}, // __builtin_HEXAGON_V6_vcvt_bf_sf
    {hexagon_V6_vcvt_bf_sf_128B, 35470}, // __builtin_HEXAGON_V6_vcvt_bf_sf_128B
    {hexagon_V6_vcvt_f8_hf, 35497}, // __builtin_HEXAGON_V6_vcvt_f8_hf
    {hexagon_V6_vcvt_f8_hf_128B, 35519}, // __builtin_HEXAGON_V6_vcvt_f8_hf_128B
    {hexagon_V6_vcvt_h_hf, 35546}, // __builtin_HEXAGON_V6_vcvt_h_hf
    {hexagon_V6_vcvt_h_hf_128B, 35567}, // __builtin_HEXAGON_V6_vcvt_h_hf_128B
    {hexagon_V6_vcvt_hf_b, 35593}, // __builtin_HEXAGON_V6_vcvt_hf_b
    {hexagon_V6_vcvt_hf_b_128B, 35614}, // __builtin_HEXAGON_V6_vcvt_hf_b_128B
    {hexagon_V6_vcvt_hf_f8, 35640}, // __builtin_HEXAGON_V6_vcvt_hf_f8
    {hexagon_V6_vcvt_hf_f8_128B, 35662}, // __builtin_HEXAGON_V6_vcvt_hf_f8_128B
    {hexagon_V6_vcvt_hf_h, 35689}, // __builtin_HEXAGON_V6_vcvt_hf_h
    {hexagon_V6_vcvt_hf_h_128B, 35710}, // __builtin_HEXAGON_V6_vcvt_hf_h_128B
    {hexagon_V6_vcvt_hf_sf, 35736}, // __builtin_HEXAGON_V6_vcvt_hf_sf
    {hexagon_V6_vcvt_hf_sf_128B, 35758}, // __builtin_HEXAGON_V6_vcvt_hf_sf_128B
    {hexagon_V6_vcvt_hf_ub, 35785}, // __builtin_HEXAGON_V6_vcvt_hf_ub
    {hexagon_V6_vcvt_hf_ub_128B, 35807}, // __builtin_HEXAGON_V6_vcvt_hf_ub_128B
    {hexagon_V6_vcvt_hf_uh, 35834}, // __builtin_HEXAGON_V6_vcvt_hf_uh
    {hexagon_V6_vcvt_hf_uh_128B, 35856}, // __builtin_HEXAGON_V6_vcvt_hf_uh_128B
    {hexagon_V6_vcvt_sf_hf, 35883}, // __builtin_HEXAGON_V6_vcvt_sf_hf
    {hexagon_V6_vcvt_sf_hf_128B, 35905}, // __builtin_HEXAGON_V6_vcvt_sf_hf_128B
    {hexagon_V6_vcvt_ub_hf, 35932}, // __builtin_HEXAGON_V6_vcvt_ub_hf
    {hexagon_V6_vcvt_ub_hf_128B, 35954}, // __builtin_HEXAGON_V6_vcvt_ub_hf_128B
    {hexagon_V6_vcvt_uh_hf, 35981}, // __builtin_HEXAGON_V6_vcvt_uh_hf
    {hexagon_V6_vcvt_uh_hf_128B, 36003}, // __builtin_HEXAGON_V6_vcvt_uh_hf_128B
    {hexagon_V6_vd0, 36030}, // __builtin_HEXAGON_V6_vd0
    {hexagon_V6_vd0_128B, 36045}, // __builtin_HEXAGON_V6_vd0_128B
    {hexagon_V6_vdd0, 36065}, // __builtin_HEXAGON_V6_vdd0
    {hexagon_V6_vdd0_128B, 36081}, // __builtin_HEXAGON_V6_vdd0_128B
    {hexagon_V6_vdealb, 36102}, // __builtin_HEXAGON_V6_vdealb
    {hexagon_V6_vdealb4w, 36120}, // __builtin_HEXAGON_V6_vdealb4w
    {hexagon_V6_vdealb4w_128B, 36140}, // __builtin_HEXAGON_V6_vdealb4w_128B
    {hexagon_V6_vdealb_128B, 36165}, // __builtin_HEXAGON_V6_vdealb_128B
    {hexagon_V6_vdealh, 36188}, // __builtin_HEXAGON_V6_vdealh
    {hexagon_V6_vdealh_128B, 36206}, // __builtin_HEXAGON_V6_vdealh_128B
    {hexagon_V6_vdealvdd, 36229}, // __builtin_HEXAGON_V6_vdealvdd
    {hexagon_V6_vdealvdd_128B, 36249}, // __builtin_HEXAGON_V6_vdealvdd_128B
    {hexagon_V6_vdelta, 36274}, // __builtin_HEXAGON_V6_vdelta
    {hexagon_V6_vdelta_128B, 36292}, // __builtin_HEXAGON_V6_vdelta_128B
    {hexagon_V6_vdmpy_sf_hf, 36315}, // __builtin_HEXAGON_V6_vdmpy_sf_hf
    {hexagon_V6_vdmpy_sf_hf_128B, 36338}, // __builtin_HEXAGON_V6_vdmpy_sf_hf_128B
    {hexagon_V6_vdmpy_sf_hf_acc, 36366}, // __builtin_HEXAGON_V6_vdmpy_sf_hf_acc
    {hexagon_V6_vdmpy_sf_hf_acc_128B, 36393}, // __builtin_HEXAGON_V6_vdmpy_sf_hf_acc_128B
    {hexagon_V6_vdmpybus, 36425}, // __builtin_HEXAGON_V6_vdmpybus
    {hexagon_V6_vdmpybus_128B, 36445}, // __builtin_HEXAGON_V6_vdmpybus_128B
    {hexagon_V6_vdmpybus_acc, 36470}, // __builtin_HEXAGON_V6_vdmpybus_acc
    {hexagon_V6_vdmpybus_acc_128B, 36494}, // __builtin_HEXAGON_V6_vdmpybus_acc_128B
    {hexagon_V6_vdmpybus_dv, 36523}, // __builtin_HEXAGON_V6_vdmpybus_dv
    {hexagon_V6_vdmpybus_dv_128B, 36546}, // __builtin_HEXAGON_V6_vdmpybus_dv_128B
    {hexagon_V6_vdmpybus_dv_acc, 36574}, // __builtin_HEXAGON_V6_vdmpybus_dv_acc
    {hexagon_V6_vdmpybus_dv_acc_128B, 36601}, // __builtin_HEXAGON_V6_vdmpybus_dv_acc_128B
    {hexagon_V6_vdmpyhb, 36633}, // __builtin_HEXAGON_V6_vdmpyhb
    {hexagon_V6_vdmpyhb_128B, 36652}, // __builtin_HEXAGON_V6_vdmpyhb_128B
    {hexagon_V6_vdmpyhb_acc, 36676}, // __builtin_HEXAGON_V6_vdmpyhb_acc
    {hexagon_V6_vdmpyhb_acc_128B, 36699}, // __builtin_HEXAGON_V6_vdmpyhb_acc_128B
    {hexagon_V6_vdmpyhb_dv, 36727}, // __builtin_HEXAGON_V6_vdmpyhb_dv
    {hexagon_V6_vdmpyhb_dv_128B, 36749}, // __builtin_HEXAGON_V6_vdmpyhb_dv_128B
    {hexagon_V6_vdmpyhb_dv_acc, 36776}, // __builtin_HEXAGON_V6_vdmpyhb_dv_acc
    {hexagon_V6_vdmpyhb_dv_acc_128B, 36802}, // __builtin_HEXAGON_V6_vdmpyhb_dv_acc_128B
    {hexagon_V6_vdmpyhisat, 36833}, // __builtin_HEXAGON_V6_vdmpyhisat
    {hexagon_V6_vdmpyhisat_128B, 36855}, // __builtin_HEXAGON_V6_vdmpyhisat_128B
    {hexagon_V6_vdmpyhisat_acc, 36882}, // __builtin_HEXAGON_V6_vdmpyhisat_acc
    {hexagon_V6_vdmpyhisat_acc_128B, 36908}, // __builtin_HEXAGON_V6_vdmpyhisat_acc_128B
    {hexagon_V6_vdmpyhsat, 36939}, // __builtin_HEXAGON_V6_vdmpyhsat
    {hexagon_V6_vdmpyhsat_128B, 36960}, // __builtin_HEXAGON_V6_vdmpyhsat_128B
    {hexagon_V6_vdmpyhsat_acc, 36986}, // __builtin_HEXAGON_V6_vdmpyhsat_acc
    {hexagon_V6_vdmpyhsat_acc_128B, 37011}, // __builtin_HEXAGON_V6_vdmpyhsat_acc_128B
    {hexagon_V6_vdmpyhsuisat, 37041}, // __builtin_HEXAGON_V6_vdmpyhsuisat
    {hexagon_V6_vdmpyhsuisat_128B, 37065}, // __builtin_HEXAGON_V6_vdmpyhsuisat_128B
    {hexagon_V6_vdmpyhsuisat_acc, 37094}, // __builtin_HEXAGON_V6_vdmpyhsuisat_acc
    {hexagon_V6_vdmpyhsuisat_acc_128B, 37122}, // __builtin_HEXAGON_V6_vdmpyhsuisat_acc_128B
    {hexagon_V6_vdmpyhsusat, 37155}, // __builtin_HEXAGON_V6_vdmpyhsusat
    {hexagon_V6_vdmpyhsusat_128B, 37178}, // __builtin_HEXAGON_V6_vdmpyhsusat_128B
    {hexagon_V6_vdmpyhsusat_acc, 37206}, // __builtin_HEXAGON_V6_vdmpyhsusat_acc
    {hexagon_V6_vdmpyhsusat_acc_128B, 37233}, // __builtin_HEXAGON_V6_vdmpyhsusat_acc_128B
    {hexagon_V6_vdmpyhvsat, 37265}, // __builtin_HEXAGON_V6_vdmpyhvsat
    {hexagon_V6_vdmpyhvsat_128B, 37287}, // __builtin_HEXAGON_V6_vdmpyhvsat_128B
    {hexagon_V6_vdmpyhvsat_acc, 37314}, // __builtin_HEXAGON_V6_vdmpyhvsat_acc
    {hexagon_V6_vdmpyhvsat_acc_128B, 37340}, // __builtin_HEXAGON_V6_vdmpyhvsat_acc_128B
    {hexagon_V6_vdsaduh, 37371}, // __builtin_HEXAGON_V6_vdsaduh
    {hexagon_V6_vdsaduh_128B, 37390}, // __builtin_HEXAGON_V6_vdsaduh_128B
    {hexagon_V6_vdsaduh_acc, 37414}, // __builtin_HEXAGON_V6_vdsaduh_acc
    {hexagon_V6_vdsaduh_acc_128B, 37437}, // __builtin_HEXAGON_V6_vdsaduh_acc_128B
    {hexagon_V6_veqb, 37465}, // __builtin_HEXAGON_V6_veqb
    {hexagon_V6_veqb_128B, 37481}, // __builtin_HEXAGON_V6_veqb_128B
    {hexagon_V6_veqb_and, 37502}, // __builtin_HEXAGON_V6_veqb_and
    {hexagon_V6_veqb_and_128B, 37522}, // __builtin_HEXAGON_V6_veqb_and_128B
    {hexagon_V6_veqb_or, 37547}, // __builtin_HEXAGON_V6_veqb_or
    {hexagon_V6_veqb_or_128B, 37566}, // __builtin_HEXAGON_V6_veqb_or_128B
    {hexagon_V6_veqb_xor, 37590}, // __builtin_HEXAGON_V6_veqb_xor
    {hexagon_V6_veqb_xor_128B, 37610}, // __builtin_HEXAGON_V6_veqb_xor_128B
    {hexagon_V6_veqh, 37635}, // __builtin_HEXAGON_V6_veqh
    {hexagon_V6_veqh_128B, 37651}, // __builtin_HEXAGON_V6_veqh_128B
    {hexagon_V6_veqh_and, 37672}, // __builtin_HEXAGON_V6_veqh_and
    {hexagon_V6_veqh_and_128B, 37692}, // __builtin_HEXAGON_V6_veqh_and_128B
    {hexagon_V6_veqh_or, 37717}, // __builtin_HEXAGON_V6_veqh_or
    {hexagon_V6_veqh_or_128B, 37736}, // __builtin_HEXAGON_V6_veqh_or_128B
    {hexagon_V6_veqh_xor, 37760}, // __builtin_HEXAGON_V6_veqh_xor
    {hexagon_V6_veqh_xor_128B, 37780}, // __builtin_HEXAGON_V6_veqh_xor_128B
    {hexagon_V6_veqhf, 37805}, // __builtin_HEXAGON_V6_veqhf
    {hexagon_V6_veqhf_128B, 37822}, // __builtin_HEXAGON_V6_veqhf_128B
    {hexagon_V6_veqhf_and, 37844}, // __builtin_HEXAGON_V6_veqhf_and
    {hexagon_V6_veqhf_and_128B, 37865}, // __builtin_HEXAGON_V6_veqhf_and_128B
    {hexagon_V6_veqhf_or, 37891}, // __builtin_HEXAGON_V6_veqhf_or
    {hexagon_V6_veqhf_or_128B, 37911}, // __builtin_HEXAGON_V6_veqhf_or_128B
    {hexagon_V6_veqhf_xor, 37936}, // __builtin_HEXAGON_V6_veqhf_xor
    {hexagon_V6_veqhf_xor_128B, 37957}, // __builtin_HEXAGON_V6_veqhf_xor_128B
    {hexagon_V6_veqsf, 37983}, // __builtin_HEXAGON_V6_veqsf
    {hexagon_V6_veqsf_128B, 38000}, // __builtin_HEXAGON_V6_veqsf_128B
    {hexagon_V6_veqsf_and, 38022}, // __builtin_HEXAGON_V6_veqsf_and
    {hexagon_V6_veqsf_and_128B, 38043}, // __builtin_HEXAGON_V6_veqsf_and_128B
    {hexagon_V6_veqsf_or, 38069}, // __builtin_HEXAGON_V6_veqsf_or
    {hexagon_V6_veqsf_or_128B, 38089}, // __builtin_HEXAGON_V6_veqsf_or_128B
    {hexagon_V6_veqsf_xor, 38114}, // __builtin_HEXAGON_V6_veqsf_xor
    {hexagon_V6_veqsf_xor_128B, 38135}, // __builtin_HEXAGON_V6_veqsf_xor_128B
    {hexagon_V6_veqw, 38161}, // __builtin_HEXAGON_V6_veqw
    {hexagon_V6_veqw_128B, 38177}, // __builtin_HEXAGON_V6_veqw_128B
    {hexagon_V6_veqw_and, 38198}, // __builtin_HEXAGON_V6_veqw_and
    {hexagon_V6_veqw_and_128B, 38218}, // __builtin_HEXAGON_V6_veqw_and_128B
    {hexagon_V6_veqw_or, 38243}, // __builtin_HEXAGON_V6_veqw_or
    {hexagon_V6_veqw_or_128B, 38262}, // __builtin_HEXAGON_V6_veqw_or_128B
    {hexagon_V6_veqw_xor, 38286}, // __builtin_HEXAGON_V6_veqw_xor
    {hexagon_V6_veqw_xor_128B, 38306}, // __builtin_HEXAGON_V6_veqw_xor_128B
    {hexagon_V6_vfmax_f8, 38331}, // __builtin_HEXAGON_V6_vfmax_f8
    {hexagon_V6_vfmax_f8_128B, 38351}, // __builtin_HEXAGON_V6_vfmax_f8_128B
    {hexagon_V6_vfmax_hf, 38376}, // __builtin_HEXAGON_V6_vfmax_hf
    {hexagon_V6_vfmax_hf_128B, 38396}, // __builtin_HEXAGON_V6_vfmax_hf_128B
    {hexagon_V6_vfmax_sf, 38421}, // __builtin_HEXAGON_V6_vfmax_sf
    {hexagon_V6_vfmax_sf_128B, 38441}, // __builtin_HEXAGON_V6_vfmax_sf_128B
    {hexagon_V6_vfmin_f8, 38466}, // __builtin_HEXAGON_V6_vfmin_f8
    {hexagon_V6_vfmin_f8_128B, 38486}, // __builtin_HEXAGON_V6_vfmin_f8_128B
    {hexagon_V6_vfmin_hf, 38511}, // __builtin_HEXAGON_V6_vfmin_hf
    {hexagon_V6_vfmin_hf_128B, 38531}, // __builtin_HEXAGON_V6_vfmin_hf_128B
    {hexagon_V6_vfmin_sf, 38556}, // __builtin_HEXAGON_V6_vfmin_sf
    {hexagon_V6_vfmin_sf_128B, 38576}, // __builtin_HEXAGON_V6_vfmin_sf_128B
    {hexagon_V6_vfneg_f8, 38601}, // __builtin_HEXAGON_V6_vfneg_f8
    {hexagon_V6_vfneg_f8_128B, 38621}, // __builtin_HEXAGON_V6_vfneg_f8_128B
    {hexagon_V6_vfneg_hf, 38646}, // __builtin_HEXAGON_V6_vfneg_hf
    {hexagon_V6_vfneg_hf_128B, 38666}, // __builtin_HEXAGON_V6_vfneg_hf_128B
    {hexagon_V6_vfneg_sf, 38691}, // __builtin_HEXAGON_V6_vfneg_sf
    {hexagon_V6_vfneg_sf_128B, 38711}, // __builtin_HEXAGON_V6_vfneg_sf_128B
    {hexagon_V6_vgather_vscattermh, 38736}, // __builtin_HEXAGON_V6_vgather_vscattermh
    {hexagon_V6_vgather_vscattermh_128B, 38766}, // __builtin_HEXAGON_V6_vgather_vscattermh_128B
    {hexagon_V6_vgathermh, 38801}, // __builtin_HEXAGON_V6_vgathermh
    {hexagon_V6_vgathermh_128B, 38822}, // __builtin_HEXAGON_V6_vgathermh_128B
    {hexagon_V6_vgathermhq, 38848}, // __builtin_HEXAGON_V6_vgathermhq
    {hexagon_V6_vgathermhq_128B, 38870}, // __builtin_HEXAGON_V6_vgathermhq_128B
    {hexagon_V6_vgathermhw, 38897}, // __builtin_HEXAGON_V6_vgathermhw
    {hexagon_V6_vgathermhw_128B, 38919}, // __builtin_HEXAGON_V6_vgathermhw_128B
    {hexagon_V6_vgathermhwq, 38946}, // __builtin_HEXAGON_V6_vgathermhwq
    {hexagon_V6_vgathermhwq_128B, 38969}, // __builtin_HEXAGON_V6_vgathermhwq_128B
    {hexagon_V6_vgathermw, 38997}, // __builtin_HEXAGON_V6_vgathermw
    {hexagon_V6_vgathermw_128B, 39018}, // __builtin_HEXAGON_V6_vgathermw_128B
    {hexagon_V6_vgathermwq, 39044}, // __builtin_HEXAGON_V6_vgathermwq
    {hexagon_V6_vgathermwq_128B, 39066}, // __builtin_HEXAGON_V6_vgathermwq_128B
    {hexagon_V6_vgtb, 39093}, // __builtin_HEXAGON_V6_vgtb
    {hexagon_V6_vgtb_128B, 39109}, // __builtin_HEXAGON_V6_vgtb_128B
    {hexagon_V6_vgtb_and, 39130}, // __builtin_HEXAGON_V6_vgtb_and
    {hexagon_V6_vgtb_and_128B, 39150}, // __builtin_HEXAGON_V6_vgtb_and_128B
    {hexagon_V6_vgtb_or, 39175}, // __builtin_HEXAGON_V6_vgtb_or
    {hexagon_V6_vgtb_or_128B, 39194}, // __builtin_HEXAGON_V6_vgtb_or_128B
    {hexagon_V6_vgtb_xor, 39218}, // __builtin_HEXAGON_V6_vgtb_xor
    {hexagon_V6_vgtb_xor_128B, 39238}, // __builtin_HEXAGON_V6_vgtb_xor_128B
    {hexagon_V6_vgtbf, 39263}, // __builtin_HEXAGON_V6_vgtbf
    {hexagon_V6_vgtbf_128B, 39280}, // __builtin_HEXAGON_V6_vgtbf_128B
    {hexagon_V6_vgtbf_and, 39302}, // __builtin_HEXAGON_V6_vgtbf_and
    {hexagon_V6_vgtbf_and_128B, 39323}, // __builtin_HEXAGON_V6_vgtbf_and_128B
    {hexagon_V6_vgtbf_or, 39349}, // __builtin_HEXAGON_V6_vgtbf_or
    {hexagon_V6_vgtbf_or_128B, 39369}, // __builtin_HEXAGON_V6_vgtbf_or_128B
    {hexagon_V6_vgtbf_xor, 39394}, // __builtin_HEXAGON_V6_vgtbf_xor
    {hexagon_V6_vgtbf_xor_128B, 39415}, // __builtin_HEXAGON_V6_vgtbf_xor_128B
    {hexagon_V6_vgth, 39441}, // __builtin_HEXAGON_V6_vgth
    {hexagon_V6_vgth_128B, 39457}, // __builtin_HEXAGON_V6_vgth_128B
    {hexagon_V6_vgth_and, 39478}, // __builtin_HEXAGON_V6_vgth_and
    {hexagon_V6_vgth_and_128B, 39498}, // __builtin_HEXAGON_V6_vgth_and_128B
    {hexagon_V6_vgth_or, 39523}, // __builtin_HEXAGON_V6_vgth_or
    {hexagon_V6_vgth_or_128B, 39542}, // __builtin_HEXAGON_V6_vgth_or_128B
    {hexagon_V6_vgth_xor, 39566}, // __builtin_HEXAGON_V6_vgth_xor
    {hexagon_V6_vgth_xor_128B, 39586}, // __builtin_HEXAGON_V6_vgth_xor_128B
    {hexagon_V6_vgthf, 39611}, // __builtin_HEXAGON_V6_vgthf
    {hexagon_V6_vgthf_128B, 39628}, // __builtin_HEXAGON_V6_vgthf_128B
    {hexagon_V6_vgthf_and, 39650}, // __builtin_HEXAGON_V6_vgthf_and
    {hexagon_V6_vgthf_and_128B, 39671}, // __builtin_HEXAGON_V6_vgthf_and_128B
    {hexagon_V6_vgthf_or, 39697}, // __builtin_HEXAGON_V6_vgthf_or
    {hexagon_V6_vgthf_or_128B, 39717}, // __builtin_HEXAGON_V6_vgthf_or_128B
    {hexagon_V6_vgthf_xor, 39742}, // __builtin_HEXAGON_V6_vgthf_xor
    {hexagon_V6_vgthf_xor_128B, 39763}, // __builtin_HEXAGON_V6_vgthf_xor_128B
    {hexagon_V6_vgtsf, 39789}, // __builtin_HEXAGON_V6_vgtsf
    {hexagon_V6_vgtsf_128B, 39806}, // __builtin_HEXAGON_V6_vgtsf_128B
    {hexagon_V6_vgtsf_and, 39828}, // __builtin_HEXAGON_V6_vgtsf_and
    {hexagon_V6_vgtsf_and_128B, 39849}, // __builtin_HEXAGON_V6_vgtsf_and_128B
    {hexagon_V6_vgtsf_or, 39875}, // __builtin_HEXAGON_V6_vgtsf_or
    {hexagon_V6_vgtsf_or_128B, 39895}, // __builtin_HEXAGON_V6_vgtsf_or_128B
    {hexagon_V6_vgtsf_xor, 39920}, // __builtin_HEXAGON_V6_vgtsf_xor
    {hexagon_V6_vgtsf_xor_128B, 39941}, // __builtin_HEXAGON_V6_vgtsf_xor_128B
    {hexagon_V6_vgtub, 39967}, // __builtin_HEXAGON_V6_vgtub
    {hexagon_V6_vgtub_128B, 39984}, // __builtin_HEXAGON_V6_vgtub_128B
    {hexagon_V6_vgtub_and, 40006}, // __builtin_HEXAGON_V6_vgtub_and
    {hexagon_V6_vgtub_and_128B, 40027}, // __builtin_HEXAGON_V6_vgtub_and_128B
    {hexagon_V6_vgtub_or, 40053}, // __builtin_HEXAGON_V6_vgtub_or
    {hexagon_V6_vgtub_or_128B, 40073}, // __builtin_HEXAGON_V6_vgtub_or_128B
    {hexagon_V6_vgtub_xor, 40098}, // __builtin_HEXAGON_V6_vgtub_xor
    {hexagon_V6_vgtub_xor_128B, 40119}, // __builtin_HEXAGON_V6_vgtub_xor_128B
    {hexagon_V6_vgtuh, 40145}, // __builtin_HEXAGON_V6_vgtuh
    {hexagon_V6_vgtuh_128B, 40162}, // __builtin_HEXAGON_V6_vgtuh_128B
    {hexagon_V6_vgtuh_and, 40184}, // __builtin_HEXAGON_V6_vgtuh_and
    {hexagon_V6_vgtuh_and_128B, 40205}, // __builtin_HEXAGON_V6_vgtuh_and_128B
    {hexagon_V6_vgtuh_or, 40231}, // __builtin_HEXAGON_V6_vgtuh_or
    {hexagon_V6_vgtuh_or_128B, 40251}, // __builtin_HEXAGON_V6_vgtuh_or_128B
    {hexagon_V6_vgtuh_xor, 40276}, // __builtin_HEXAGON_V6_vgtuh_xor
    {hexagon_V6_vgtuh_xor_128B, 40297}, // __builtin_HEXAGON_V6_vgtuh_xor_128B
    {hexagon_V6_vgtuw, 40323}, // __builtin_HEXAGON_V6_vgtuw
    {hexagon_V6_vgtuw_128B, 40340}, // __builtin_HEXAGON_V6_vgtuw_128B
    {hexagon_V6_vgtuw_and, 40362}, // __builtin_HEXAGON_V6_vgtuw_and
    {hexagon_V6_vgtuw_and_128B, 40383}, // __builtin_HEXAGON_V6_vgtuw_and_128B
    {hexagon_V6_vgtuw_or, 40409}, // __builtin_HEXAGON_V6_vgtuw_or
    {hexagon_V6_vgtuw_or_128B, 40429}, // __builtin_HEXAGON_V6_vgtuw_or_128B
    {hexagon_V6_vgtuw_xor, 40454}, // __builtin_HEXAGON_V6_vgtuw_xor
    {hexagon_V6_vgtuw_xor_128B, 40475}, // __builtin_HEXAGON_V6_vgtuw_xor_128B
    {hexagon_V6_vgtw, 40501}, // __builtin_HEXAGON_V6_vgtw
    {hexagon_V6_vgtw_128B, 40517}, // __builtin_HEXAGON_V6_vgtw_128B
    {hexagon_V6_vgtw_and, 40538}, // __builtin_HEXAGON_V6_vgtw_and
    {hexagon_V6_vgtw_and_128B, 40558}, // __builtin_HEXAGON_V6_vgtw_and_128B
    {hexagon_V6_vgtw_or, 40583}, // __builtin_HEXAGON_V6_vgtw_or
    {hexagon_V6_vgtw_or_128B, 40602}, // __builtin_HEXAGON_V6_vgtw_or_128B
    {hexagon_V6_vgtw_xor, 40626}, // __builtin_HEXAGON_V6_vgtw_xor
    {hexagon_V6_vgtw_xor_128B, 40646}, // __builtin_HEXAGON_V6_vgtw_xor_128B
    {hexagon_V6_vilog2_hf, 40671}, // __builtin_HEXAGON_V6_vilog2_hf
    {hexagon_V6_vilog2_hf_128B, 40692}, // __builtin_HEXAGON_V6_vilog2_hf_128B
    {hexagon_V6_vilog2_qf16, 40718}, // __builtin_HEXAGON_V6_vilog2_qf16
    {hexagon_V6_vilog2_qf16_128B, 40741}, // __builtin_HEXAGON_V6_vilog2_qf16_128B
    {hexagon_V6_vilog2_qf32, 40769}, // __builtin_HEXAGON_V6_vilog2_qf32
    {hexagon_V6_vilog2_qf32_128B, 40792}, // __builtin_HEXAGON_V6_vilog2_qf32_128B
    {hexagon_V6_vilog2_sf, 40820}, // __builtin_HEXAGON_V6_vilog2_sf
    {hexagon_V6_vilog2_sf_128B, 40841}, // __builtin_HEXAGON_V6_vilog2_sf_128B
    {hexagon_V6_vinsertwr, 40867}, // __builtin_HEXAGON_V6_vinsertwr
    {hexagon_V6_vinsertwr_128B, 40888}, // __builtin_HEXAGON_V6_vinsertwr_128B
    {hexagon_V6_vlalignb, 40914}, // __builtin_HEXAGON_V6_vlalignb
    {hexagon_V6_vlalignb_128B, 40934}, // __builtin_HEXAGON_V6_vlalignb_128B
    {hexagon_V6_vlalignbi, 40959}, // __builtin_HEXAGON_V6_vlalignbi
    {hexagon_V6_vlalignbi_128B, 40980}, // __builtin_HEXAGON_V6_vlalignbi_128B
    {hexagon_V6_vlsrb, 41006}, // __builtin_HEXAGON_V6_vlsrb
    {hexagon_V6_vlsrb_128B, 41023}, // __builtin_HEXAGON_V6_vlsrb_128B
    {hexagon_V6_vlsrh, 41045}, // __builtin_HEXAGON_V6_vlsrh
    {hexagon_V6_vlsrh_128B, 41062}, // __builtin_HEXAGON_V6_vlsrh_128B
    {hexagon_V6_vlsrhv, 41084}, // __builtin_HEXAGON_V6_vlsrhv
    {hexagon_V6_vlsrhv_128B, 41102}, // __builtin_HEXAGON_V6_vlsrhv_128B
    {hexagon_V6_vlsrw, 41125}, // __builtin_HEXAGON_V6_vlsrw
    {hexagon_V6_vlsrw_128B, 41142}, // __builtin_HEXAGON_V6_vlsrw_128B
    {hexagon_V6_vlsrwv, 41164}, // __builtin_HEXAGON_V6_vlsrwv
    {hexagon_V6_vlsrwv_128B, 41182}, // __builtin_HEXAGON_V6_vlsrwv_128B
    {hexagon_V6_vlut4, 41205}, // __builtin_HEXAGON_V6_vlut4
    {hexagon_V6_vlut4_128B, 41222}, // __builtin_HEXAGON_V6_vlut4_128B
    {hexagon_V6_vlutvvb, 41244}, // __builtin_HEXAGON_V6_vlutvvb
    {hexagon_V6_vlutvvb_128B, 41263}, // __builtin_HEXAGON_V6_vlutvvb_128B
    {hexagon_V6_vlutvvb_nm, 41287}, // __builtin_HEXAGON_V6_vlutvvb_nm
    {hexagon_V6_vlutvvb_nm_128B, 41309}, // __builtin_HEXAGON_V6_vlutvvb_nm_128B
    {hexagon_V6_vlutvvb_oracc, 41336}, // __builtin_HEXAGON_V6_vlutvvb_oracc
    {hexagon_V6_vlutvvb_oracc_128B, 41361}, // __builtin_HEXAGON_V6_vlutvvb_oracc_128B
    {hexagon_V6_vlutvvb_oracci, 41391}, // __builtin_HEXAGON_V6_vlutvvb_oracci
    {hexagon_V6_vlutvvb_oracci_128B, 41417}, // __builtin_HEXAGON_V6_vlutvvb_oracci_128B
    {hexagon_V6_vlutvvbi, 41448}, // __builtin_HEXAGON_V6_vlutvvbi
    {hexagon_V6_vlutvvbi_128B, 41468}, // __builtin_HEXAGON_V6_vlutvvbi_128B
    {hexagon_V6_vlutvwh, 41493}, // __builtin_HEXAGON_V6_vlutvwh
    {hexagon_V6_vlutvwh_128B, 41512}, // __builtin_HEXAGON_V6_vlutvwh_128B
    {hexagon_V6_vlutvwh_nm, 41536}, // __builtin_HEXAGON_V6_vlutvwh_nm
    {hexagon_V6_vlutvwh_nm_128B, 41558}, // __builtin_HEXAGON_V6_vlutvwh_nm_128B
    {hexagon_V6_vlutvwh_oracc, 41585}, // __builtin_HEXAGON_V6_vlutvwh_oracc
    {hexagon_V6_vlutvwh_oracc_128B, 41610}, // __builtin_HEXAGON_V6_vlutvwh_oracc_128B
    {hexagon_V6_vlutvwh_oracci, 41640}, // __builtin_HEXAGON_V6_vlutvwh_oracci
    {hexagon_V6_vlutvwh_oracci_128B, 41666}, // __builtin_HEXAGON_V6_vlutvwh_oracci_128B
    {hexagon_V6_vlutvwhi, 41697}, // __builtin_HEXAGON_V6_vlutvwhi
    {hexagon_V6_vlutvwhi_128B, 41717}, // __builtin_HEXAGON_V6_vlutvwhi_128B
    {hexagon_V6_vmax_bf, 41742}, // __builtin_HEXAGON_V6_vmax_bf
    {hexagon_V6_vmax_bf_128B, 41761}, // __builtin_HEXAGON_V6_vmax_bf_128B
    {hexagon_V6_vmax_hf, 41785}, // __builtin_HEXAGON_V6_vmax_hf
    {hexagon_V6_vmax_hf_128B, 41804}, // __builtin_HEXAGON_V6_vmax_hf_128B
    {hexagon_V6_vmax_sf, 41828}, // __builtin_HEXAGON_V6_vmax_sf
    {hexagon_V6_vmax_sf_128B, 41847}, // __builtin_HEXAGON_V6_vmax_sf_128B
    {hexagon_V6_vmaxb, 41871}, // __builtin_HEXAGON_V6_vmaxb
    {hexagon_V6_vmaxb_128B, 41888}, // __builtin_HEXAGON_V6_vmaxb_128B
    {hexagon_V6_vmaxh, 41910}, // __builtin_HEXAGON_V6_vmaxh
    {hexagon_V6_vmaxh_128B, 41927}, // __builtin_HEXAGON_V6_vmaxh_128B
    {hexagon_V6_vmaxub, 41949}, // __builtin_HEXAGON_V6_vmaxub
    {hexagon_V6_vmaxub_128B, 41967}, // __builtin_HEXAGON_V6_vmaxub_128B
    {hexagon_V6_vmaxuh, 41990}, // __builtin_HEXAGON_V6_vmaxuh
    {hexagon_V6_vmaxuh_128B, 42008}, // __builtin_HEXAGON_V6_vmaxuh_128B
    {hexagon_V6_vmaxw, 42031}, // __builtin_HEXAGON_V6_vmaxw
    {hexagon_V6_vmaxw_128B, 42048}, // __builtin_HEXAGON_V6_vmaxw_128B
    {hexagon_V6_vmerge_qf, 42070}, // __builtin_HEXAGON_V6_vmerge_qf
    {hexagon_V6_vmerge_qf_128B, 42091}, // __builtin_HEXAGON_V6_vmerge_qf_128B
    {hexagon_V6_vmin_bf, 42117}, // __builtin_HEXAGON_V6_vmin_bf
    {hexagon_V6_vmin_bf_128B, 42136}, // __builtin_HEXAGON_V6_vmin_bf_128B
    {hexagon_V6_vmin_hf, 42160}, // __builtin_HEXAGON_V6_vmin_hf
    {hexagon_V6_vmin_hf_128B, 42179}, // __builtin_HEXAGON_V6_vmin_hf_128B
    {hexagon_V6_vmin_sf, 42203}, // __builtin_HEXAGON_V6_vmin_sf
    {hexagon_V6_vmin_sf_128B, 42222}, // __builtin_HEXAGON_V6_vmin_sf_128B
    {hexagon_V6_vminb, 42246}, // __builtin_HEXAGON_V6_vminb
    {hexagon_V6_vminb_128B, 42263}, // __builtin_HEXAGON_V6_vminb_128B
    {hexagon_V6_vminh, 42285}, // __builtin_HEXAGON_V6_vminh
    {hexagon_V6_vminh_128B, 42302}, // __builtin_HEXAGON_V6_vminh_128B
    {hexagon_V6_vminub, 42324}, // __builtin_HEXAGON_V6_vminub
    {hexagon_V6_vminub_128B, 42342}, // __builtin_HEXAGON_V6_vminub_128B
    {hexagon_V6_vminuh, 42365}, // __builtin_HEXAGON_V6_vminuh
    {hexagon_V6_vminuh_128B, 42383}, // __builtin_HEXAGON_V6_vminuh_128B
    {hexagon_V6_vminw, 42406}, // __builtin_HEXAGON_V6_vminw
    {hexagon_V6_vminw_128B, 42423}, // __builtin_HEXAGON_V6_vminw_128B
    {hexagon_V6_vmpabus, 42445}, // __builtin_HEXAGON_V6_vmpabus
    {hexagon_V6_vmpabus_128B, 42464}, // __builtin_HEXAGON_V6_vmpabus_128B
    {hexagon_V6_vmpabus_acc, 42488}, // __builtin_HEXAGON_V6_vmpabus_acc
    {hexagon_V6_vmpabus_acc_128B, 42511}, // __builtin_HEXAGON_V6_vmpabus_acc_128B
    {hexagon_V6_vmpabusv, 42539}, // __builtin_HEXAGON_V6_vmpabusv
    {hexagon_V6_vmpabusv_128B, 42559}, // __builtin_HEXAGON_V6_vmpabusv_128B
    {hexagon_V6_vmpabuu, 42584}, // __builtin_HEXAGON_V6_vmpabuu
    {hexagon_V6_vmpabuu_128B, 42603}, // __builtin_HEXAGON_V6_vmpabuu_128B
    {hexagon_V6_vmpabuu_acc, 42627}, // __builtin_HEXAGON_V6_vmpabuu_acc
    {hexagon_V6_vmpabuu_acc_128B, 42650}, // __builtin_HEXAGON_V6_vmpabuu_acc_128B
    {hexagon_V6_vmpabuuv, 42678}, // __builtin_HEXAGON_V6_vmpabuuv
    {hexagon_V6_vmpabuuv_128B, 42698}, // __builtin_HEXAGON_V6_vmpabuuv_128B
    {hexagon_V6_vmpahb, 42723}, // __builtin_HEXAGON_V6_vmpahb
    {hexagon_V6_vmpahb_128B, 42741}, // __builtin_HEXAGON_V6_vmpahb_128B
    {hexagon_V6_vmpahb_acc, 42764}, // __builtin_HEXAGON_V6_vmpahb_acc
    {hexagon_V6_vmpahb_acc_128B, 42786}, // __builtin_HEXAGON_V6_vmpahb_acc_128B
    {hexagon_V6_vmpahhsat, 42813}, // __builtin_HEXAGON_V6_vmpahhsat
    {hexagon_V6_vmpahhsat_128B, 42834}, // __builtin_HEXAGON_V6_vmpahhsat_128B
    {hexagon_V6_vmpauhb, 42860}, // __builtin_HEXAGON_V6_vmpauhb
    {hexagon_V6_vmpauhb_128B, 42879}, // __builtin_HEXAGON_V6_vmpauhb_128B
    {hexagon_V6_vmpauhb_acc, 42903}, // __builtin_HEXAGON_V6_vmpauhb_acc
    {hexagon_V6_vmpauhb_acc_128B, 42926}, // __builtin_HEXAGON_V6_vmpauhb_acc_128B
    {hexagon_V6_vmpauhuhsat, 42954}, // __builtin_HEXAGON_V6_vmpauhuhsat
    {hexagon_V6_vmpauhuhsat_128B, 42977}, // __builtin_HEXAGON_V6_vmpauhuhsat_128B
    {hexagon_V6_vmpsuhuhsat, 43005}, // __builtin_HEXAGON_V6_vmpsuhuhsat
    {hexagon_V6_vmpsuhuhsat_128B, 43028}, // __builtin_HEXAGON_V6_vmpsuhuhsat_128B
    {hexagon_V6_vmpy_hf_f8, 43056}, // __builtin_HEXAGON_V6_vmpy_hf_f8
    {hexagon_V6_vmpy_hf_f8_128B, 43078}, // __builtin_HEXAGON_V6_vmpy_hf_f8_128B
    {hexagon_V6_vmpy_hf_f8_acc, 43105}, // __builtin_HEXAGON_V6_vmpy_hf_f8_acc
    {hexagon_V6_vmpy_hf_f8_acc_128B, 43131}, // __builtin_HEXAGON_V6_vmpy_hf_f8_acc_128B
    {hexagon_V6_vmpy_hf_hf, 43162}, // __builtin_HEXAGON_V6_vmpy_hf_hf
    {hexagon_V6_vmpy_hf_hf_128B, 43184}, // __builtin_HEXAGON_V6_vmpy_hf_hf_128B
    {hexagon_V6_vmpy_hf_hf_acc, 43211}, // __builtin_HEXAGON_V6_vmpy_hf_hf_acc
    {hexagon_V6_vmpy_hf_hf_acc_128B, 43237}, // __builtin_HEXAGON_V6_vmpy_hf_hf_acc_128B
    {hexagon_V6_vmpy_qf16, 43268}, // __builtin_HEXAGON_V6_vmpy_qf16
    {hexagon_V6_vmpy_qf16_128B, 43289}, // __builtin_HEXAGON_V6_vmpy_qf16_128B
    {hexagon_V6_vmpy_qf16_hf, 43315}, // __builtin_HEXAGON_V6_vmpy_qf16_hf
    {hexagon_V6_vmpy_qf16_hf_128B, 43339}, // __builtin_HEXAGON_V6_vmpy_qf16_hf_128B
    {hexagon_V6_vmpy_qf16_mix_hf, 43368}, // __builtin_HEXAGON_V6_vmpy_qf16_mix_hf
    {hexagon_V6_vmpy_qf16_mix_hf_128B, 43396}, // __builtin_HEXAGON_V6_vmpy_qf16_mix_hf_128B
    {hexagon_V6_vmpy_qf32, 43429}, // __builtin_HEXAGON_V6_vmpy_qf32
    {hexagon_V6_vmpy_qf32_128B, 43450}, // __builtin_HEXAGON_V6_vmpy_qf32_128B
    {hexagon_V6_vmpy_qf32_hf, 43476}, // __builtin_HEXAGON_V6_vmpy_qf32_hf
    {hexagon_V6_vmpy_qf32_hf_128B, 43500}, // __builtin_HEXAGON_V6_vmpy_qf32_hf_128B
    {hexagon_V6_vmpy_qf32_mix_hf, 43529}, // __builtin_HEXAGON_V6_vmpy_qf32_mix_hf
    {hexagon_V6_vmpy_qf32_mix_hf_128B, 43557}, // __builtin_HEXAGON_V6_vmpy_qf32_mix_hf_128B
    {hexagon_V6_vmpy_qf32_qf16, 43590}, // __builtin_HEXAGON_V6_vmpy_qf32_qf16
    {hexagon_V6_vmpy_qf32_qf16_128B, 43616}, // __builtin_HEXAGON_V6_vmpy_qf32_qf16_128B
    {hexagon_V6_vmpy_qf32_sf, 43647}, // __builtin_HEXAGON_V6_vmpy_qf32_sf
    {hexagon_V6_vmpy_qf32_sf_128B, 43671}, // __builtin_HEXAGON_V6_vmpy_qf32_sf_128B
    {hexagon_V6_vmpy_rt_hf, 43700}, // __builtin_HEXAGON_V6_vmpy_rt_hf
    {hexagon_V6_vmpy_rt_hf_128B, 43722}, // __builtin_HEXAGON_V6_vmpy_rt_hf_128B
    {hexagon_V6_vmpy_rt_qf16, 43749}, // __builtin_HEXAGON_V6_vmpy_rt_qf16
    {hexagon_V6_vmpy_rt_qf16_128B, 43773}, // __builtin_HEXAGON_V6_vmpy_rt_qf16_128B
    {hexagon_V6_vmpy_rt_sf, 43802}, // __builtin_HEXAGON_V6_vmpy_rt_sf
    {hexagon_V6_vmpy_rt_sf_128B, 43824}, // __builtin_HEXAGON_V6_vmpy_rt_sf_128B
    {hexagon_V6_vmpy_sf_bf, 43851}, // __builtin_HEXAGON_V6_vmpy_sf_bf
    {hexagon_V6_vmpy_sf_bf_128B, 43873}, // __builtin_HEXAGON_V6_vmpy_sf_bf_128B
    {hexagon_V6_vmpy_sf_bf_acc, 43900}, // __builtin_HEXAGON_V6_vmpy_sf_bf_acc
    {hexagon_V6_vmpy_sf_bf_acc_128B, 43926}, // __builtin_HEXAGON_V6_vmpy_sf_bf_acc_128B
    {hexagon_V6_vmpy_sf_hf, 43957}, // __builtin_HEXAGON_V6_vmpy_sf_hf
    {hexagon_V6_vmpy_sf_hf_128B, 43979}, // __builtin_HEXAGON_V6_vmpy_sf_hf_128B
    {hexagon_V6_vmpy_sf_hf_acc, 44006}, // __builtin_HEXAGON_V6_vmpy_sf_hf_acc
    {hexagon_V6_vmpy_sf_hf_acc_128B, 44032}, // __builtin_HEXAGON_V6_vmpy_sf_hf_acc_128B
    {hexagon_V6_vmpy_sf_sf, 44063}, // __builtin_HEXAGON_V6_vmpy_sf_sf
    {hexagon_V6_vmpy_sf_sf_128B, 44085}, // __builtin_HEXAGON_V6_vmpy_sf_sf_128B
    {hexagon_V6_vmpybus, 44112}, // __builtin_HEXAGON_V6_vmpybus
    {hexagon_V6_vmpybus_128B, 44131}, // __builtin_HEXAGON_V6_vmpybus_128B
    {hexagon_V6_vmpybus_acc, 44155}, // __builtin_HEXAGON_V6_vmpybus_acc
    {hexagon_V6_vmpybus_acc_128B, 44178}, // __builtin_HEXAGON_V6_vmpybus_acc_128B
    {hexagon_V6_vmpybusv, 44206}, // __builtin_HEXAGON_V6_vmpybusv
    {hexagon_V6_vmpybusv_128B, 44226}, // __builtin_HEXAGON_V6_vmpybusv_128B
    {hexagon_V6_vmpybusv_acc, 44251}, // __builtin_HEXAGON_V6_vmpybusv_acc
    {hexagon_V6_vmpybusv_acc_128B, 44275}, // __builtin_HEXAGON_V6_vmpybusv_acc_128B
    {hexagon_V6_vmpybv, 44304}, // __builtin_HEXAGON_V6_vmpybv
    {hexagon_V6_vmpybv_128B, 44322}, // __builtin_HEXAGON_V6_vmpybv_128B
    {hexagon_V6_vmpybv_acc, 44345}, // __builtin_HEXAGON_V6_vmpybv_acc
    {hexagon_V6_vmpybv_acc_128B, 44367}, // __builtin_HEXAGON_V6_vmpybv_acc_128B
    {hexagon_V6_vmpyewuh, 44394}, // __builtin_HEXAGON_V6_vmpyewuh
    {hexagon_V6_vmpyewuh_128B, 44414}, // __builtin_HEXAGON_V6_vmpyewuh_128B
    {hexagon_V6_vmpyewuh_64, 44439}, // __builtin_HEXAGON_V6_vmpyewuh_64
    {hexagon_V6_vmpyewuh_64_128B, 44462}, // __builtin_HEXAGON_V6_vmpyewuh_64_128B
    {hexagon_V6_vmpyh, 44490}, // __builtin_HEXAGON_V6_vmpyh
    {hexagon_V6_vmpyh_128B, 44507}, // __builtin_HEXAGON_V6_vmpyh_128B
    {hexagon_V6_vmpyh_acc, 44529}, // __builtin_HEXAGON_V6_vmpyh_acc
    {hexagon_V6_vmpyh_acc_128B, 44550}, // __builtin_HEXAGON_V6_vmpyh_acc_128B
    {hexagon_V6_vmpyhsat_acc, 44576}, // __builtin_HEXAGON_V6_vmpyhsat_acc
    {hexagon_V6_vmpyhsat_acc_128B, 44600}, // __builtin_HEXAGON_V6_vmpyhsat_acc_128B
    {hexagon_V6_vmpyhsrs, 44629}, // __builtin_HEXAGON_V6_vmpyhsrs
    {hexagon_V6_vmpyhsrs_128B, 44649}, // __builtin_HEXAGON_V6_vmpyhsrs_128B
    {hexagon_V6_vmpyhss, 44674}, // __builtin_HEXAGON_V6_vmpyhss
    {hexagon_V6_vmpyhss_128B, 44693}, // __builtin_HEXAGON_V6_vmpyhss_128B
    {hexagon_V6_vmpyhus, 44717}, // __builtin_HEXAGON_V6_vmpyhus
    {hexagon_V6_vmpyhus_128B, 44736}, // __builtin_HEXAGON_V6_vmpyhus_128B
    {hexagon_V6_vmpyhus_acc, 44760}, // __builtin_HEXAGON_V6_vmpyhus_acc
    {hexagon_V6_vmpyhus_acc_128B, 44783}, // __builtin_HEXAGON_V6_vmpyhus_acc_128B
    {hexagon_V6_vmpyhv, 44811}, // __builtin_HEXAGON_V6_vmpyhv
    {hexagon_V6_vmpyhv_128B, 44829}, // __builtin_HEXAGON_V6_vmpyhv_128B
    {hexagon_V6_vmpyhv_acc, 44852}, // __builtin_HEXAGON_V6_vmpyhv_acc
    {hexagon_V6_vmpyhv_acc_128B, 44874}, // __builtin_HEXAGON_V6_vmpyhv_acc_128B
    {hexagon_V6_vmpyhvsrs, 44901}, // __builtin_HEXAGON_V6_vmpyhvsrs
    {hexagon_V6_vmpyhvsrs_128B, 44922}, // __builtin_HEXAGON_V6_vmpyhvsrs_128B
    {hexagon_V6_vmpyieoh, 44948}, // __builtin_HEXAGON_V6_vmpyieoh
    {hexagon_V6_vmpyieoh_128B, 44968}, // __builtin_HEXAGON_V6_vmpyieoh_128B
    {hexagon_V6_vmpyiewh_acc, 44993}, // __builtin_HEXAGON_V6_vmpyiewh_acc
    {hexagon_V6_vmpyiewh_acc_128B, 45017}, // __builtin_HEXAGON_V6_vmpyiewh_acc_128B
    {hexagon_V6_vmpyiewuh, 45046}, // __builtin_HEXAGON_V6_vmpyiewuh
    {hexagon_V6_vmpyiewuh_128B, 45067}, // __builtin_HEXAGON_V6_vmpyiewuh_128B
    {hexagon_V6_vmpyiewuh_acc, 45093}, // __builtin_HEXAGON_V6_vmpyiewuh_acc
    {hexagon_V6_vmpyiewuh_acc_128B, 45118}, // __builtin_HEXAGON_V6_vmpyiewuh_acc_128B
    {hexagon_V6_vmpyih, 45148}, // __builtin_HEXAGON_V6_vmpyih
    {hexagon_V6_vmpyih_128B, 45166}, // __builtin_HEXAGON_V6_vmpyih_128B
    {hexagon_V6_vmpyih_acc, 45189}, // __builtin_HEXAGON_V6_vmpyih_acc
    {hexagon_V6_vmpyih_acc_128B, 45211}, // __builtin_HEXAGON_V6_vmpyih_acc_128B
    {hexagon_V6_vmpyihb, 45238}, // __builtin_HEXAGON_V6_vmpyihb
    {hexagon_V6_vmpyihb_128B, 45257}, // __builtin_HEXAGON_V6_vmpyihb_128B
    {hexagon_V6_vmpyihb_acc, 45281}, // __builtin_HEXAGON_V6_vmpyihb_acc
    {hexagon_V6_vmpyihb_acc_128B, 45304}, // __builtin_HEXAGON_V6_vmpyihb_acc_128B
    {hexagon_V6_vmpyiowh, 45332}, // __builtin_HEXAGON_V6_vmpyiowh
    {hexagon_V6_vmpyiowh_128B, 45352}, // __builtin_HEXAGON_V6_vmpyiowh_128B
    {hexagon_V6_vmpyiwb, 45377}, // __builtin_HEXAGON_V6_vmpyiwb
    {hexagon_V6_vmpyiwb_128B, 45396}, // __builtin_HEXAGON_V6_vmpyiwb_128B
    {hexagon_V6_vmpyiwb_acc, 45420}, // __builtin_HEXAGON_V6_vmpyiwb_acc
    {hexagon_V6_vmpyiwb_acc_128B, 45443}, // __builtin_HEXAGON_V6_vmpyiwb_acc_128B
    {hexagon_V6_vmpyiwh, 45471}, // __builtin_HEXAGON_V6_vmpyiwh
    {hexagon_V6_vmpyiwh_128B, 45490}, // __builtin_HEXAGON_V6_vmpyiwh_128B
    {hexagon_V6_vmpyiwh_acc, 45514}, // __builtin_HEXAGON_V6_vmpyiwh_acc
    {hexagon_V6_vmpyiwh_acc_128B, 45537}, // __builtin_HEXAGON_V6_vmpyiwh_acc_128B
    {hexagon_V6_vmpyiwub, 45565}, // __builtin_HEXAGON_V6_vmpyiwub
    {hexagon_V6_vmpyiwub_128B, 45585}, // __builtin_HEXAGON_V6_vmpyiwub_128B
    {hexagon_V6_vmpyiwub_acc, 45610}, // __builtin_HEXAGON_V6_vmpyiwub_acc
    {hexagon_V6_vmpyiwub_acc_128B, 45634}, // __builtin_HEXAGON_V6_vmpyiwub_acc_128B
    {hexagon_V6_vmpyowh, 45663}, // __builtin_HEXAGON_V6_vmpyowh
    {hexagon_V6_vmpyowh_128B, 45682}, // __builtin_HEXAGON_V6_vmpyowh_128B
    {hexagon_V6_vmpyowh_64_acc, 45706}, // __builtin_HEXAGON_V6_vmpyowh_64_acc
    {hexagon_V6_vmpyowh_64_acc_128B, 45732}, // __builtin_HEXAGON_V6_vmpyowh_64_acc_128B
    {hexagon_V6_vmpyowh_rnd, 45763}, // __builtin_HEXAGON_V6_vmpyowh_rnd
    {hexagon_V6_vmpyowh_rnd_128B, 45786}, // __builtin_HEXAGON_V6_vmpyowh_rnd_128B
    {hexagon_V6_vmpyowh_rnd_sacc, 45814}, // __builtin_HEXAGON_V6_vmpyowh_rnd_sacc
    {hexagon_V6_vmpyowh_rnd_sacc_128B, 45842}, // __builtin_HEXAGON_V6_vmpyowh_rnd_sacc_128B
    {hexagon_V6_vmpyowh_sacc, 45875}, // __builtin_HEXAGON_V6_vmpyowh_sacc
    {hexagon_V6_vmpyowh_sacc_128B, 45899}, // __builtin_HEXAGON_V6_vmpyowh_sacc_128B
    {hexagon_V6_vmpyub, 45928}, // __builtin_HEXAGON_V6_vmpyub
    {hexagon_V6_vmpyub_128B, 45946}, // __builtin_HEXAGON_V6_vmpyub_128B
    {hexagon_V6_vmpyub_acc, 45969}, // __builtin_HEXAGON_V6_vmpyub_acc
    {hexagon_V6_vmpyub_acc_128B, 45991}, // __builtin_HEXAGON_V6_vmpyub_acc_128B
    {hexagon_V6_vmpyubv, 46018}, // __builtin_HEXAGON_V6_vmpyubv
    {hexagon_V6_vmpyubv_128B, 46037}, // __builtin_HEXAGON_V6_vmpyubv_128B
    {hexagon_V6_vmpyubv_acc, 46061}, // __builtin_HEXAGON_V6_vmpyubv_acc
    {hexagon_V6_vmpyubv_acc_128B, 46084}, // __builtin_HEXAGON_V6_vmpyubv_acc_128B
    {hexagon_V6_vmpyuh, 46112}, // __builtin_HEXAGON_V6_vmpyuh
    {hexagon_V6_vmpyuh_128B, 46130}, // __builtin_HEXAGON_V6_vmpyuh_128B
    {hexagon_V6_vmpyuh_acc, 46153}, // __builtin_HEXAGON_V6_vmpyuh_acc
    {hexagon_V6_vmpyuh_acc_128B, 46175}, // __builtin_HEXAGON_V6_vmpyuh_acc_128B
    {hexagon_V6_vmpyuhe, 46202}, // __builtin_HEXAGON_V6_vmpyuhe
    {hexagon_V6_vmpyuhe_128B, 46221}, // __builtin_HEXAGON_V6_vmpyuhe_128B
    {hexagon_V6_vmpyuhe_acc, 46245}, // __builtin_HEXAGON_V6_vmpyuhe_acc
    {hexagon_V6_vmpyuhe_acc_128B, 46268}, // __builtin_HEXAGON_V6_vmpyuhe_acc_128B
    {hexagon_V6_vmpyuhv, 46296}, // __builtin_HEXAGON_V6_vmpyuhv
    {hexagon_V6_vmpyuhv_128B, 46315}, // __builtin_HEXAGON_V6_vmpyuhv_128B
    {hexagon_V6_vmpyuhv_acc, 46339}, // __builtin_HEXAGON_V6_vmpyuhv_acc
    {hexagon_V6_vmpyuhv_acc_128B, 46362}, // __builtin_HEXAGON_V6_vmpyuhv_acc_128B
    {hexagon_V6_vmpyuhvs, 46390}, // __builtin_HEXAGON_V6_vmpyuhvs
    {hexagon_V6_vmpyuhvs_128B, 46410}, // __builtin_HEXAGON_V6_vmpyuhvs_128B
    {hexagon_V6_vmux, 46435}, // __builtin_HEXAGON_V6_vmux
    {hexagon_V6_vmux_128B, 46451}, // __builtin_HEXAGON_V6_vmux_128B
    {hexagon_V6_vnavgb, 46472}, // __builtin_HEXAGON_V6_vnavgb
    {hexagon_V6_vnavgb_128B, 46490}, // __builtin_HEXAGON_V6_vnavgb_128B
    {hexagon_V6_vnavgh, 46513}, // __builtin_HEXAGON_V6_vnavgh
    {hexagon_V6_vnavgh_128B, 46531}, // __builtin_HEXAGON_V6_vnavgh_128B
    {hexagon_V6_vnavgub, 46554}, // __builtin_HEXAGON_V6_vnavgub
    {hexagon_V6_vnavgub_128B, 46573}, // __builtin_HEXAGON_V6_vnavgub_128B
    {hexagon_V6_vnavgw, 46597}, // __builtin_HEXAGON_V6_vnavgw
    {hexagon_V6_vnavgw_128B, 46615}, // __builtin_HEXAGON_V6_vnavgw_128B
    {hexagon_V6_vneg_qf16_hf, 46638}, // __builtin_HEXAGON_V6_vneg_qf16_hf
    {hexagon_V6_vneg_qf16_hf_128B, 46662}, // __builtin_HEXAGON_V6_vneg_qf16_hf_128B
    {hexagon_V6_vneg_qf16_qf16, 46691}, // __builtin_HEXAGON_V6_vneg_qf16_qf16
    {hexagon_V6_vneg_qf16_qf16_128B, 46717}, // __builtin_HEXAGON_V6_vneg_qf16_qf16_128B
    {hexagon_V6_vneg_qf32_qf32, 46748}, // __builtin_HEXAGON_V6_vneg_qf32_qf32
    {hexagon_V6_vneg_qf32_qf32_128B, 46774}, // __builtin_HEXAGON_V6_vneg_qf32_qf32_128B
    {hexagon_V6_vneg_qf32_sf, 46805}, // __builtin_HEXAGON_V6_vneg_qf32_sf
    {hexagon_V6_vneg_qf32_sf_128B, 46829}, // __builtin_HEXAGON_V6_vneg_qf32_sf_128B
    {hexagon_V6_vnormamth, 46858}, // __builtin_HEXAGON_V6_vnormamth
    {hexagon_V6_vnormamth_128B, 46879}, // __builtin_HEXAGON_V6_vnormamth_128B
    {hexagon_V6_vnormamtw, 46905}, // __builtin_HEXAGON_V6_vnormamtw
    {hexagon_V6_vnormamtw_128B, 46926}, // __builtin_HEXAGON_V6_vnormamtw_128B
    {hexagon_V6_vnot, 46952}, // __builtin_HEXAGON_V6_vnot
    {hexagon_V6_vnot_128B, 46968}, // __builtin_HEXAGON_V6_vnot_128B
    {hexagon_V6_vor, 46989}, // __builtin_HEXAGON_V6_vor
    {hexagon_V6_vor_128B, 47004}, // __builtin_HEXAGON_V6_vor_128B
    {hexagon_V6_vpackeb, 47024}, // __builtin_HEXAGON_V6_vpackeb
    {hexagon_V6_vpackeb_128B, 47043}, // __builtin_HEXAGON_V6_vpackeb_128B
    {hexagon_V6_vpackeh, 47067}, // __builtin_HEXAGON_V6_vpackeh
    {hexagon_V6_vpackeh_128B, 47086}, // __builtin_HEXAGON_V6_vpackeh_128B
    {hexagon_V6_vpackhb_sat, 47110}, // __builtin_HEXAGON_V6_vpackhb_sat
    {hexagon_V6_vpackhb_sat_128B, 47133}, // __builtin_HEXAGON_V6_vpackhb_sat_128B
    {hexagon_V6_vpackhub_sat, 47161}, // __builtin_HEXAGON_V6_vpackhub_sat
    {hexagon_V6_vpackhub_sat_128B, 47185}, // __builtin_HEXAGON_V6_vpackhub_sat_128B
    {hexagon_V6_vpackob, 47214}, // __builtin_HEXAGON_V6_vpackob
    {hexagon_V6_vpackob_128B, 47233}, // __builtin_HEXAGON_V6_vpackob_128B
    {hexagon_V6_vpackoh, 47257}, // __builtin_HEXAGON_V6_vpackoh
    {hexagon_V6_vpackoh_128B, 47276}, // __builtin_HEXAGON_V6_vpackoh_128B
    {hexagon_V6_vpackwh_sat, 47300}, // __builtin_HEXAGON_V6_vpackwh_sat
    {hexagon_V6_vpackwh_sat_128B, 47323}, // __builtin_HEXAGON_V6_vpackwh_sat_128B
    {hexagon_V6_vpackwuh_sat, 47351}, // __builtin_HEXAGON_V6_vpackwuh_sat
    {hexagon_V6_vpackwuh_sat_128B, 47375}, // __builtin_HEXAGON_V6_vpackwuh_sat_128B
    {hexagon_V6_vpopcounth, 47404}, // __builtin_HEXAGON_V6_vpopcounth
    {hexagon_V6_vpopcounth_128B, 47426}, // __builtin_HEXAGON_V6_vpopcounth_128B
    {hexagon_V6_vprefixqb, 47453}, // __builtin_HEXAGON_V6_vprefixqb
    {hexagon_V6_vprefixqb_128B, 47474}, // __builtin_HEXAGON_V6_vprefixqb_128B
    {hexagon_V6_vprefixqh, 47500}, // __builtin_HEXAGON_V6_vprefixqh
    {hexagon_V6_vprefixqh_128B, 47521}, // __builtin_HEXAGON_V6_vprefixqh_128B
    {hexagon_V6_vprefixqw, 47547}, // __builtin_HEXAGON_V6_vprefixqw
    {hexagon_V6_vprefixqw_128B, 47568}, // __builtin_HEXAGON_V6_vprefixqw_128B
    {hexagon_V6_vrdelta, 47594}, // __builtin_HEXAGON_V6_vrdelta
    {hexagon_V6_vrdelta_128B, 47613}, // __builtin_HEXAGON_V6_vrdelta_128B
    {hexagon_V6_vrmpybub_rtt, 47637}, // __builtin_HEXAGON_V6_vrmpybub_rtt
    {hexagon_V6_vrmpybub_rtt_128B, 47661}, // __builtin_HEXAGON_V6_vrmpybub_rtt_128B
    {hexagon_V6_vrmpybub_rtt_acc, 47690}, // __builtin_HEXAGON_V6_vrmpybub_rtt_acc
    {hexagon_V6_vrmpybub_rtt_acc_128B, 47718}, // __builtin_HEXAGON_V6_vrmpybub_rtt_acc_128B
    {hexagon_V6_vrmpybus, 47751}, // __builtin_HEXAGON_V6_vrmpybus
    {hexagon_V6_vrmpybus_128B, 47771}, // __builtin_HEXAGON_V6_vrmpybus_128B
    {hexagon_V6_vrmpybus_acc, 47796}, // __builtin_HEXAGON_V6_vrmpybus_acc
    {hexagon_V6_vrmpybus_acc_128B, 47820}, // __builtin_HEXAGON_V6_vrmpybus_acc_128B
    {hexagon_V6_vrmpybusi, 47849}, // __builtin_HEXAGON_V6_vrmpybusi
    {hexagon_V6_vrmpybusi_128B, 47870}, // __builtin_HEXAGON_V6_vrmpybusi_128B
    {hexagon_V6_vrmpybusi_acc, 47896}, // __builtin_HEXAGON_V6_vrmpybusi_acc
    {hexagon_V6_vrmpybusi_acc_128B, 47921}, // __builtin_HEXAGON_V6_vrmpybusi_acc_128B
    {hexagon_V6_vrmpybusv, 47951}, // __builtin_HEXAGON_V6_vrmpybusv
    {hexagon_V6_vrmpybusv_128B, 47972}, // __builtin_HEXAGON_V6_vrmpybusv_128B
    {hexagon_V6_vrmpybusv_acc, 47998}, // __builtin_HEXAGON_V6_vrmpybusv_acc
    {hexagon_V6_vrmpybusv_acc_128B, 48023}, // __builtin_HEXAGON_V6_vrmpybusv_acc_128B
    {hexagon_V6_vrmpybv, 48053}, // __builtin_HEXAGON_V6_vrmpybv
    {hexagon_V6_vrmpybv_128B, 48072}, // __builtin_HEXAGON_V6_vrmpybv_128B
    {hexagon_V6_vrmpybv_acc, 48096}, // __builtin_HEXAGON_V6_vrmpybv_acc
    {hexagon_V6_vrmpybv_acc_128B, 48119}, // __builtin_HEXAGON_V6_vrmpybv_acc_128B
    {hexagon_V6_vrmpyub, 48147}, // __builtin_HEXAGON_V6_vrmpyub
    {hexagon_V6_vrmpyub_128B, 48166}, // __builtin_HEXAGON_V6_vrmpyub_128B
    {hexagon_V6_vrmpyub_acc, 48190}, // __builtin_HEXAGON_V6_vrmpyub_acc
    {hexagon_V6_vrmpyub_acc_128B, 48213}, // __builtin_HEXAGON_V6_vrmpyub_acc_128B
    {hexagon_V6_vrmpyub_rtt, 48241}, // __builtin_HEXAGON_V6_vrmpyub_rtt
    {hexagon_V6_vrmpyub_rtt_128B, 48264}, // __builtin_HEXAGON_V6_vrmpyub_rtt_128B
    {hexagon_V6_vrmpyub_rtt_acc, 48292}, // __builtin_HEXAGON_V6_vrmpyub_rtt_acc
    {hexagon_V6_vrmpyub_rtt_acc_128B, 48319}, // __builtin_HEXAGON_V6_vrmpyub_rtt_acc_128B
    {hexagon_V6_vrmpyubi, 48351}, // __builtin_HEXAGON_V6_vrmpyubi
    {hexagon_V6_vrmpyubi_128B, 48371}, // __builtin_HEXAGON_V6_vrmpyubi_128B
    {hexagon_V6_vrmpyubi_acc, 48396}, // __builtin_HEXAGON_V6_vrmpyubi_acc
    {hexagon_V6_vrmpyubi_acc_128B, 48420}, // __builtin_HEXAGON_V6_vrmpyubi_acc_128B
    {hexagon_V6_vrmpyubv, 48449}, // __builtin_HEXAGON_V6_vrmpyubv
    {hexagon_V6_vrmpyubv_128B, 48469}, // __builtin_HEXAGON_V6_vrmpyubv_128B
    {hexagon_V6_vrmpyubv_acc, 48494}, // __builtin_HEXAGON_V6_vrmpyubv_acc
    {hexagon_V6_vrmpyubv_acc_128B, 48518}, // __builtin_HEXAGON_V6_vrmpyubv_acc_128B
    {hexagon_V6_vror, 48547}, // __builtin_HEXAGON_V6_vror
    {hexagon_V6_vror_128B, 48563}, // __builtin_HEXAGON_V6_vror_128B
    {hexagon_V6_vrotr, 48584}, // __builtin_HEXAGON_V6_vrotr
    {hexagon_V6_vrotr_128B, 48601}, // __builtin_HEXAGON_V6_vrotr_128B
    {hexagon_V6_vroundhb, 48623}, // __builtin_HEXAGON_V6_vroundhb
    {hexagon_V6_vroundhb_128B, 48643}, // __builtin_HEXAGON_V6_vroundhb_128B
    {hexagon_V6_vroundhub, 48668}, // __builtin_HEXAGON_V6_vroundhub
    {hexagon_V6_vroundhub_128B, 48689}, // __builtin_HEXAGON_V6_vroundhub_128B
    {hexagon_V6_vrounduhub, 48715}, // __builtin_HEXAGON_V6_vrounduhub
    {hexagon_V6_vrounduhub_128B, 48737}, // __builtin_HEXAGON_V6_vrounduhub_128B
    {hexagon_V6_vrounduwuh, 48764}, // __builtin_HEXAGON_V6_vrounduwuh
    {hexagon_V6_vrounduwuh_128B, 48786}, // __builtin_HEXAGON_V6_vrounduwuh_128B
    {hexagon_V6_vroundwh, 48813}, // __builtin_HEXAGON_V6_vroundwh
    {hexagon_V6_vroundwh_128B, 48833}, // __builtin_HEXAGON_V6_vroundwh_128B
    {hexagon_V6_vroundwuh, 48858}, // __builtin_HEXAGON_V6_vroundwuh
    {hexagon_V6_vroundwuh_128B, 48879}, // __builtin_HEXAGON_V6_vroundwuh_128B
    {hexagon_V6_vrsadubi, 48905}, // __builtin_HEXAGON_V6_vrsadubi
    {hexagon_V6_vrsadubi_128B, 48925}, // __builtin_HEXAGON_V6_vrsadubi_128B
    {hexagon_V6_vrsadubi_acc, 48950}, // __builtin_HEXAGON_V6_vrsadubi_acc
    {hexagon_V6_vrsadubi_acc_128B, 48974}, // __builtin_HEXAGON_V6_vrsadubi_acc_128B
    {hexagon_V6_vsatdw, 49003}, // __builtin_HEXAGON_V6_vsatdw
    {hexagon_V6_vsatdw_128B, 49021}, // __builtin_HEXAGON_V6_vsatdw_128B
    {hexagon_V6_vsathub, 49044}, // __builtin_HEXAGON_V6_vsathub
    {hexagon_V6_vsathub_128B, 49063}, // __builtin_HEXAGON_V6_vsathub_128B
    {hexagon_V6_vsatuwuh, 49087}, // __builtin_HEXAGON_V6_vsatuwuh
    {hexagon_V6_vsatuwuh_128B, 49107}, // __builtin_HEXAGON_V6_vsatuwuh_128B
    {hexagon_V6_vsatwh, 49132}, // __builtin_HEXAGON_V6_vsatwh
    {hexagon_V6_vsatwh_128B, 49150}, // __builtin_HEXAGON_V6_vsatwh_128B
    {hexagon_V6_vsb, 49173}, // __builtin_HEXAGON_V6_vsb
    {hexagon_V6_vsb_128B, 49188}, // __builtin_HEXAGON_V6_vsb_128B
    {hexagon_V6_vscattermh, 49208}, // __builtin_HEXAGON_V6_vscattermh
    {hexagon_V6_vscattermh_128B, 49230}, // __builtin_HEXAGON_V6_vscattermh_128B
    {hexagon_V6_vscattermh_add, 49257}, // __builtin_HEXAGON_V6_vscattermh_add
    {hexagon_V6_vscattermh_add_128B, 49283}, // __builtin_HEXAGON_V6_vscattermh_add_128B
    {hexagon_V6_vscattermhq, 49314}, // __builtin_HEXAGON_V6_vscattermhq
    {hexagon_V6_vscattermhq_128B, 49337}, // __builtin_HEXAGON_V6_vscattermhq_128B
    {hexagon_V6_vscattermhw, 49365}, // __builtin_HEXAGON_V6_vscattermhw
    {hexagon_V6_vscattermhw_128B, 49388}, // __builtin_HEXAGON_V6_vscattermhw_128B
    {hexagon_V6_vscattermhw_add, 49416}, // __builtin_HEXAGON_V6_vscattermhw_add
    {hexagon_V6_vscattermhw_add_128B, 49443}, // __builtin_HEXAGON_V6_vscattermhw_add_128B
    {hexagon_V6_vscattermhwq, 49475}, // __builtin_HEXAGON_V6_vscattermhwq
    {hexagon_V6_vscattermhwq_128B, 49499}, // __builtin_HEXAGON_V6_vscattermhwq_128B
    {hexagon_V6_vscattermw, 49528}, // __builtin_HEXAGON_V6_vscattermw
    {hexagon_V6_vscattermw_128B, 49550}, // __builtin_HEXAGON_V6_vscattermw_128B
    {hexagon_V6_vscattermw_add, 49577}, // __builtin_HEXAGON_V6_vscattermw_add
    {hexagon_V6_vscattermw_add_128B, 49603}, // __builtin_HEXAGON_V6_vscattermw_add_128B
    {hexagon_V6_vscattermwq, 49634}, // __builtin_HEXAGON_V6_vscattermwq
    {hexagon_V6_vscattermwq_128B, 49657}, // __builtin_HEXAGON_V6_vscattermwq_128B
    {hexagon_V6_vsh, 49685}, // __builtin_HEXAGON_V6_vsh
    {hexagon_V6_vsh_128B, 49700}, // __builtin_HEXAGON_V6_vsh_128B
    {hexagon_V6_vshufeh, 49720}, // __builtin_HEXAGON_V6_vshufeh
    {hexagon_V6_vshufeh_128B, 49739}, // __builtin_HEXAGON_V6_vshufeh_128B
    {hexagon_V6_vshuffb, 49763}, // __builtin_HEXAGON_V6_vshuffb
    {hexagon_V6_vshuffb_128B, 49782}, // __builtin_HEXAGON_V6_vshuffb_128B
    {hexagon_V6_vshuffeb, 49806}, // __builtin_HEXAGON_V6_vshuffeb
    {hexagon_V6_vshuffeb_128B, 49826}, // __builtin_HEXAGON_V6_vshuffeb_128B
    {hexagon_V6_vshuffh, 49851}, // __builtin_HEXAGON_V6_vshuffh
    {hexagon_V6_vshuffh_128B, 49870}, // __builtin_HEXAGON_V6_vshuffh_128B
    {hexagon_V6_vshuffob, 49894}, // __builtin_HEXAGON_V6_vshuffob
    {hexagon_V6_vshuffob_128B, 49914}, // __builtin_HEXAGON_V6_vshuffob_128B
    {hexagon_V6_vshuffvdd, 49939}, // __builtin_HEXAGON_V6_vshuffvdd
    {hexagon_V6_vshuffvdd_128B, 49960}, // __builtin_HEXAGON_V6_vshuffvdd_128B
    {hexagon_V6_vshufoeb, 49986}, // __builtin_HEXAGON_V6_vshufoeb
    {hexagon_V6_vshufoeb_128B, 50006}, // __builtin_HEXAGON_V6_vshufoeb_128B
    {hexagon_V6_vshufoeh, 50031}, // __builtin_HEXAGON_V6_vshufoeh
    {hexagon_V6_vshufoeh_128B, 50051}, // __builtin_HEXAGON_V6_vshufoeh_128B
    {hexagon_V6_vshufoh, 50076}, // __builtin_HEXAGON_V6_vshufoh
    {hexagon_V6_vshufoh_128B, 50095}, // __builtin_HEXAGON_V6_vshufoh_128B
    {hexagon_V6_vsub_hf, 50119}, // __builtin_HEXAGON_V6_vsub_hf
    {hexagon_V6_vsub_hf_128B, 50138}, // __builtin_HEXAGON_V6_vsub_hf_128B
    {hexagon_V6_vsub_hf_f8, 50162}, // __builtin_HEXAGON_V6_vsub_hf_f8
    {hexagon_V6_vsub_hf_f8_128B, 50184}, // __builtin_HEXAGON_V6_vsub_hf_f8_128B
    {hexagon_V6_vsub_hf_hf, 50211}, // __builtin_HEXAGON_V6_vsub_hf_hf
    {hexagon_V6_vsub_hf_hf_128B, 50233}, // __builtin_HEXAGON_V6_vsub_hf_hf_128B
    {hexagon_V6_vsub_hf_mix, 50260}, // __builtin_HEXAGON_V6_vsub_hf_mix
    {hexagon_V6_vsub_hf_mix_128B, 50283}, // __builtin_HEXAGON_V6_vsub_hf_mix_128B
    {hexagon_V6_vsub_qf16, 50311}, // __builtin_HEXAGON_V6_vsub_qf16
    {hexagon_V6_vsub_qf16_128B, 50332}, // __builtin_HEXAGON_V6_vsub_qf16_128B
    {hexagon_V6_vsub_qf16_mix, 50358}, // __builtin_HEXAGON_V6_vsub_qf16_mix
    {hexagon_V6_vsub_qf16_mix_128B, 50383}, // __builtin_HEXAGON_V6_vsub_qf16_mix_128B
    {hexagon_V6_vsub_qf32, 50413}, // __builtin_HEXAGON_V6_vsub_qf32
    {hexagon_V6_vsub_qf32_128B, 50434}, // __builtin_HEXAGON_V6_vsub_qf32_128B
    {hexagon_V6_vsub_qf32_mix, 50460}, // __builtin_HEXAGON_V6_vsub_qf32_mix
    {hexagon_V6_vsub_qf32_mix_128B, 50485}, // __builtin_HEXAGON_V6_vsub_qf32_mix_128B
    {hexagon_V6_vsub_sf, 50515}, // __builtin_HEXAGON_V6_vsub_sf
    {hexagon_V6_vsub_sf_128B, 50534}, // __builtin_HEXAGON_V6_vsub_sf_128B
    {hexagon_V6_vsub_sf_bf, 50558}, // __builtin_HEXAGON_V6_vsub_sf_bf
    {hexagon_V6_vsub_sf_bf_128B, 50580}, // __builtin_HEXAGON_V6_vsub_sf_bf_128B
    {hexagon_V6_vsub_sf_hf, 50607}, // __builtin_HEXAGON_V6_vsub_sf_hf
    {hexagon_V6_vsub_sf_hf_128B, 50629}, // __builtin_HEXAGON_V6_vsub_sf_hf_128B
    {hexagon_V6_vsub_sf_mix, 50656}, // __builtin_HEXAGON_V6_vsub_sf_mix
    {hexagon_V6_vsub_sf_mix_128B, 50679}, // __builtin_HEXAGON_V6_vsub_sf_mix_128B
    {hexagon_V6_vsub_sf_sf, 50707}, // __builtin_HEXAGON_V6_vsub_sf_sf
    {hexagon_V6_vsub_sf_sf_128B, 50729}, // __builtin_HEXAGON_V6_vsub_sf_sf_128B
    {hexagon_V6_vsubb, 50756}, // __builtin_HEXAGON_V6_vsubb
    {hexagon_V6_vsubb_128B, 50773}, // __builtin_HEXAGON_V6_vsubb_128B
    {hexagon_V6_vsubb_dv, 50795}, // __builtin_HEXAGON_V6_vsubb_dv
    {hexagon_V6_vsubb_dv_128B, 50815}, // __builtin_HEXAGON_V6_vsubb_dv_128B
    {hexagon_V6_vsubbnq, 50840}, // __builtin_HEXAGON_V6_vsubbnq
    {hexagon_V6_vsubbnq_128B, 50859}, // __builtin_HEXAGON_V6_vsubbnq_128B
    {hexagon_V6_vsubbq, 50883}, // __builtin_HEXAGON_V6_vsubbq
    {hexagon_V6_vsubbq_128B, 50901}, // __builtin_HEXAGON_V6_vsubbq_128B
    {hexagon_V6_vsubbsat, 50924}, // __builtin_HEXAGON_V6_vsubbsat
    {hexagon_V6_vsubbsat_128B, 50944}, // __builtin_HEXAGON_V6_vsubbsat_128B
    {hexagon_V6_vsubbsat_dv, 50969}, // __builtin_HEXAGON_V6_vsubbsat_dv
    {hexagon_V6_vsubbsat_dv_128B, 50992}, // __builtin_HEXAGON_V6_vsubbsat_dv_128B
    {hexagon_V6_vsubh, 51020}, // __builtin_HEXAGON_V6_vsubh
    {hexagon_V6_vsubh_128B, 51037}, // __builtin_HEXAGON_V6_vsubh_128B
    {hexagon_V6_vsubh_dv, 51059}, // __builtin_HEXAGON_V6_vsubh_dv
    {hexagon_V6_vsubh_dv_128B, 51079}, // __builtin_HEXAGON_V6_vsubh_dv_128B
    {hexagon_V6_vsubhnq, 51104}, // __builtin_HEXAGON_V6_vsubhnq
    {hexagon_V6_vsubhnq_128B, 51123}, // __builtin_HEXAGON_V6_vsubhnq_128B
    {hexagon_V6_vsubhq, 51147}, // __builtin_HEXAGON_V6_vsubhq
    {hexagon_V6_vsubhq_128B, 51165}, // __builtin_HEXAGON_V6_vsubhq_128B
    {hexagon_V6_vsubhsat, 51188}, // __builtin_HEXAGON_V6_vsubhsat
    {hexagon_V6_vsubhsat_128B, 51208}, // __builtin_HEXAGON_V6_vsubhsat_128B
    {hexagon_V6_vsubhsat_dv, 51233}, // __builtin_HEXAGON_V6_vsubhsat_dv
    {hexagon_V6_vsubhsat_dv_128B, 51256}, // __builtin_HEXAGON_V6_vsubhsat_dv_128B
    {hexagon_V6_vsubhw, 51284}, // __builtin_HEXAGON_V6_vsubhw
    {hexagon_V6_vsubhw_128B, 51302}, // __builtin_HEXAGON_V6_vsubhw_128B
    {hexagon_V6_vsububh, 51325}, // __builtin_HEXAGON_V6_vsububh
    {hexagon_V6_vsububh_128B, 51344}, // __builtin_HEXAGON_V6_vsububh_128B
    {hexagon_V6_vsububsat, 51368}, // __builtin_HEXAGON_V6_vsububsat
    {hexagon_V6_vsububsat_128B, 51389}, // __builtin_HEXAGON_V6_vsububsat_128B
    {hexagon_V6_vsububsat_dv, 51415}, // __builtin_HEXAGON_V6_vsububsat_dv
    {hexagon_V6_vsububsat_dv_128B, 51439}, // __builtin_HEXAGON_V6_vsububsat_dv_128B
    {hexagon_V6_vsubububb_sat, 51468}, // __builtin_HEXAGON_V6_vsubububb_sat
    {hexagon_V6_vsubububb_sat_128B, 51493}, // __builtin_HEXAGON_V6_vsubububb_sat_128B
    {hexagon_V6_vsubuhsat, 51523}, // __builtin_HEXAGON_V6_vsubuhsat
    {hexagon_V6_vsubuhsat_128B, 51544}, // __builtin_HEXAGON_V6_vsubuhsat_128B
    {hexagon_V6_vsubuhsat_dv, 51570}, // __builtin_HEXAGON_V6_vsubuhsat_dv
    {hexagon_V6_vsubuhsat_dv_128B, 51594}, // __builtin_HEXAGON_V6_vsubuhsat_dv_128B
    {hexagon_V6_vsubuhw, 51623}, // __builtin_HEXAGON_V6_vsubuhw
    {hexagon_V6_vsubuhw_128B, 51642}, // __builtin_HEXAGON_V6_vsubuhw_128B
    {hexagon_V6_vsubuwsat, 51666}, // __builtin_HEXAGON_V6_vsubuwsat
    {hexagon_V6_vsubuwsat_128B, 51687}, // __builtin_HEXAGON_V6_vsubuwsat_128B
    {hexagon_V6_vsubuwsat_dv, 51713}, // __builtin_HEXAGON_V6_vsubuwsat_dv
    {hexagon_V6_vsubuwsat_dv_128B, 51737}, // __builtin_HEXAGON_V6_vsubuwsat_dv_128B
    {hexagon_V6_vsubw, 51766}, // __builtin_HEXAGON_V6_vsubw
    {hexagon_V6_vsubw_128B, 51783}, // __builtin_HEXAGON_V6_vsubw_128B
    {hexagon_V6_vsubw_dv, 51805}, // __builtin_HEXAGON_V6_vsubw_dv
    {hexagon_V6_vsubw_dv_128B, 51825}, // __builtin_HEXAGON_V6_vsubw_dv_128B
    {hexagon_V6_vsubwnq, 51850}, // __builtin_HEXAGON_V6_vsubwnq
    {hexagon_V6_vsubwnq_128B, 51869}, // __builtin_HEXAGON_V6_vsubwnq_128B
    {hexagon_V6_vsubwq, 51893}, // __builtin_HEXAGON_V6_vsubwq
    {hexagon_V6_vsubwq_128B, 51911}, // __builtin_HEXAGON_V6_vsubwq_128B
    {hexagon_V6_vsubwsat, 51934}, // __builtin_HEXAGON_V6_vsubwsat
    {hexagon_V6_vsubwsat_128B, 51954}, // __builtin_HEXAGON_V6_vsubwsat_128B
    {hexagon_V6_vsubwsat_dv, 51979}, // __builtin_HEXAGON_V6_vsubwsat_dv
    {hexagon_V6_vsubwsat_dv_128B, 52002}, // __builtin_HEXAGON_V6_vsubwsat_dv_128B
    {hexagon_V6_vswap, 52030}, // __builtin_HEXAGON_V6_vswap
    {hexagon_V6_vswap_128B, 52047}, // __builtin_HEXAGON_V6_vswap_128B
    {hexagon_V6_vtmpyb, 52069}, // __builtin_HEXAGON_V6_vtmpyb
    {hexagon_V6_vtmpyb_128B, 52087}, // __builtin_HEXAGON_V6_vtmpyb_128B
    {hexagon_V6_vtmpyb_acc, 52110}, // __builtin_HEXAGON_V6_vtmpyb_acc
    {hexagon_V6_vtmpyb_acc_128B, 52132}, // __builtin_HEXAGON_V6_vtmpyb_acc_128B
    {hexagon_V6_vtmpybus, 52159}, // __builtin_HEXAGON_V6_vtmpybus
    {hexagon_V6_vtmpybus_128B, 52179}, // __builtin_HEXAGON_V6_vtmpybus_128B
    {hexagon_V6_vtmpybus_acc, 52204}, // __builtin_HEXAGON_V6_vtmpybus_acc
    {hexagon_V6_vtmpybus_acc_128B, 52228}, // __builtin_HEXAGON_V6_vtmpybus_acc_128B
    {hexagon_V6_vtmpyhb, 52257}, // __builtin_HEXAGON_V6_vtmpyhb
    {hexagon_V6_vtmpyhb_128B, 52276}, // __builtin_HEXAGON_V6_vtmpyhb_128B
    {hexagon_V6_vtmpyhb_acc, 52300}, // __builtin_HEXAGON_V6_vtmpyhb_acc
    {hexagon_V6_vtmpyhb_acc_128B, 52323}, // __builtin_HEXAGON_V6_vtmpyhb_acc_128B
    {hexagon_V6_vunpackb, 52351}, // __builtin_HEXAGON_V6_vunpackb
    {hexagon_V6_vunpackb_128B, 52371}, // __builtin_HEXAGON_V6_vunpackb_128B
    {hexagon_V6_vunpackh, 52396}, // __builtin_HEXAGON_V6_vunpackh
    {hexagon_V6_vunpackh_128B, 52416}, // __builtin_HEXAGON_V6_vunpackh_128B
    {hexagon_V6_vunpackob, 52441}, // __builtin_HEXAGON_V6_vunpackob
    {hexagon_V6_vunpackob_128B, 52462}, // __builtin_HEXAGON_V6_vunpackob_128B
    {hexagon_V6_vunpackoh, 52488}, // __builtin_HEXAGON_V6_vunpackoh
    {hexagon_V6_vunpackoh_128B, 52509}, // __builtin_HEXAGON_V6_vunpackoh_128B
    {hexagon_V6_vunpackub, 52535}, // __builtin_HEXAGON_V6_vunpackub
    {hexagon_V6_vunpackub_128B, 52556}, // __builtin_HEXAGON_V6_vunpackub_128B
    {hexagon_V6_vunpackuh, 52582}, // __builtin_HEXAGON_V6_vunpackuh
    {hexagon_V6_vunpackuh_128B, 52603}, // __builtin_HEXAGON_V6_vunpackuh_128B
    {hexagon_V6_vxor, 52629}, // __builtin_HEXAGON_V6_vxor
    {hexagon_V6_vxor_128B, 52645}, // __builtin_HEXAGON_V6_vxor_128B
    {hexagon_V6_vzb, 52666}, // __builtin_HEXAGON_V6_vzb
    {hexagon_V6_vzb_128B, 52681}, // __builtin_HEXAGON_V6_vzb_128B
    {hexagon_V6_vzh, 52701}, // __builtin_HEXAGON_V6_vzh
    {hexagon_V6_vzh_128B, 52716}, // __builtin_HEXAGON_V6_vzh_128B
    {hexagon_Y2_dccleana, 52736}, // __builtin_HEXAGON_Y2_dccleana
    {hexagon_Y2_dccleaninva, 52756}, // __builtin_HEXAGON_Y2_dccleaninva
    {hexagon_Y2_dcfetch, 52779}, // __builtin_HEXAGON_Y2_dcfetch
    {hexagon_Y2_dcinva, 52798}, // __builtin_HEXAGON_Y2_dcinva
    {hexagon_Y2_dczeroa, 52816}, // __builtin_HEXAGON_Y2_dczeroa
    {hexagon_Y4_l2fetch, 52835}, // __builtin_HEXAGON_Y4_l2fetch
    {hexagon_Y5_l2fetch, 52854}, // __builtin_HEXAGON_Y5_l2fetch
    {hexagon_Y6_dmlink, 52873}, // __builtin_HEXAGON_Y6_dmlink
    {hexagon_Y6_dmpause, 52891}, // __builtin_HEXAGON_Y6_dmpause
    {hexagon_Y6_dmpoll, 52910}, // __builtin_HEXAGON_Y6_dmpoll
    {hexagon_Y6_dmresume, 52928}, // __builtin_HEXAGON_Y6_dmresume
    {hexagon_Y6_dmstart, 52948}, // __builtin_HEXAGON_Y6_dmstart
    {hexagon_Y6_dmwait, 52967}, // __builtin_HEXAGON_Y6_dmwait
    {hexagon_prefetch, 52985}, // __builtin_HEXAGON_prefetch
    {hexagon_L4_loadd_locked, 53002}, // __builtin__HEXAGON_L4_loadd_locked
    {hexagon_S2_storerb_pbr, 53027}, // __builtin_brev_stb
    {hexagon_S2_storerd_pbr, 53036}, // __builtin_brev_std
    {hexagon_S2_storerh_pbr, 53045}, // __builtin_brev_sth
    {hexagon_S2_storerf_pbr, 53054}, // __builtin_brev_sthhi
    {hexagon_S2_storeri_pbr, 53065}, // __builtin_brev_stw
    {hexagon_circ_ldb, 53074}, // __builtin_circ_ldb
    {hexagon_circ_ldd, 53083}, // __builtin_circ_ldd
    {hexagon_circ_ldh, 53092}, // __builtin_circ_ldh
    {hexagon_circ_ldub, 53101}, // __builtin_circ_ldub
    {hexagon_circ_lduh, 53111}, // __builtin_circ_lduh
    {hexagon_circ_ldw, 53121}, // __builtin_circ_ldw
    {hexagon_circ_stb, 53130}, // __builtin_circ_stb
    {hexagon_circ_std, 53139}, // __builtin_circ_std
    {hexagon_circ_sth, 53148}, // __builtin_circ_sth
    {hexagon_circ_sthhi, 53157}, // __builtin_circ_sthhi
    {hexagon_circ_stw, 53168}, // __builtin_circ_stw
    {hexagon_vmemcpy, 53177}, // __builtin_hexagon_vmemcpy
    {hexagon_vmemset, 53193}, // __builtin_hexagon_vmemset
  }; // hexagonNames

  // Builtins for loongarch.
  static constexpr BuiltinEntry loongarchNames[] = {
    {loongarch_lasx_cast_128, 53209}, // __builtin_lasx_cast_128
    {loongarch_lasx_cast_128_d, 53222}, // __builtin_lasx_cast_128_d
    {loongarch_lasx_cast_128_s, 53237}, // __builtin_lasx_cast_128_s
    {loongarch_lasx_concat_128, 53252}, // __builtin_lasx_concat_128
    {loongarch_lasx_concat_128_d, 53267}, // __builtin_lasx_concat_128_d
    {loongarch_lasx_concat_128_s, 53284}, // __builtin_lasx_concat_128_s
    {loongarch_lasx_extract_128_hi, 53301}, // __builtin_lasx_extract_128_hi
    {loongarch_lasx_extract_128_hi_d, 53320}, // __builtin_lasx_extract_128_hi_d
    {loongarch_lasx_extract_128_hi_s, 53341}, // __builtin_lasx_extract_128_hi_s
    {loongarch_lasx_extract_128_lo, 53362}, // __builtin_lasx_extract_128_lo
    {loongarch_lasx_extract_128_lo_d, 53381}, // __builtin_lasx_extract_128_lo_d
    {loongarch_lasx_extract_128_lo_s, 53402}, // __builtin_lasx_extract_128_lo_s
    {loongarch_lasx_insert_128_hi, 53423}, // __builtin_lasx_insert_128_hi
    {loongarch_lasx_insert_128_hi_d, 53441}, // __builtin_lasx_insert_128_hi_d
    {loongarch_lasx_insert_128_hi_s, 53461}, // __builtin_lasx_insert_128_hi_s
    {loongarch_lasx_insert_128_lo, 53481}, // __builtin_lasx_insert_128_lo
    {loongarch_lasx_insert_128_lo_d, 53499}, // __builtin_lasx_insert_128_lo_d
    {loongarch_lasx_insert_128_lo_s, 53519}, // __builtin_lasx_insert_128_lo_s
    {loongarch_lasx_vext2xv_d_b, 53539}, // __builtin_lasx_vext2xv_d_b
    {loongarch_lasx_vext2xv_d_h, 53555}, // __builtin_lasx_vext2xv_d_h
    {loongarch_lasx_vext2xv_d_w, 53571}, // __builtin_lasx_vext2xv_d_w
    {loongarch_lasx_vext2xv_du_bu, 53587}, // __builtin_lasx_vext2xv_du_bu
    {loongarch_lasx_vext2xv_du_hu, 53605}, // __builtin_lasx_vext2xv_du_hu
    {loongarch_lasx_vext2xv_du_wu, 53623}, // __builtin_lasx_vext2xv_du_wu
    {loongarch_lasx_vext2xv_h_b, 53641}, // __builtin_lasx_vext2xv_h_b
    {loongarch_lasx_vext2xv_hu_bu, 53657}, // __builtin_lasx_vext2xv_hu_bu
    {loongarch_lasx_vext2xv_w_b, 53675}, // __builtin_lasx_vext2xv_w_b
    {loongarch_lasx_vext2xv_w_h, 53691}, // __builtin_lasx_vext2xv_w_h
    {loongarch_lasx_vext2xv_wu_bu, 53707}, // __builtin_lasx_vext2xv_wu_bu
    {loongarch_lasx_vext2xv_wu_hu, 53725}, // __builtin_lasx_vext2xv_wu_hu
    {loongarch_lasx_xbnz_b, 53743}, // __builtin_lasx_xbnz_b
    {loongarch_lasx_xbnz_d, 53754}, // __builtin_lasx_xbnz_d
    {loongarch_lasx_xbnz_h, 53765}, // __builtin_lasx_xbnz_h
    {loongarch_lasx_xbnz_v, 53776}, // __builtin_lasx_xbnz_v
    {loongarch_lasx_xbnz_w, 53787}, // __builtin_lasx_xbnz_w
    {loongarch_lasx_xbz_b, 53798}, // __builtin_lasx_xbz_b
    {loongarch_lasx_xbz_d, 53808}, // __builtin_lasx_xbz_d
    {loongarch_lasx_xbz_h, 53818}, // __builtin_lasx_xbz_h
    {loongarch_lasx_xbz_v, 53828}, // __builtin_lasx_xbz_v
    {loongarch_lasx_xbz_w, 53838}, // __builtin_lasx_xbz_w
    {loongarch_lasx_xvabsd_b, 53848}, // __builtin_lasx_xvabsd_b
    {loongarch_lasx_xvabsd_bu, 53861}, // __builtin_lasx_xvabsd_bu
    {loongarch_lasx_xvabsd_d, 53875}, // __builtin_lasx_xvabsd_d
    {loongarch_lasx_xvabsd_du, 53888}, // __builtin_lasx_xvabsd_du
    {loongarch_lasx_xvabsd_h, 53902}, // __builtin_lasx_xvabsd_h
    {loongarch_lasx_xvabsd_hu, 53915}, // __builtin_lasx_xvabsd_hu
    {loongarch_lasx_xvabsd_w, 53929}, // __builtin_lasx_xvabsd_w
    {loongarch_lasx_xvabsd_wu, 53942}, // __builtin_lasx_xvabsd_wu
    {loongarch_lasx_xvadd_b, 53956}, // __builtin_lasx_xvadd_b
    {loongarch_lasx_xvadd_d, 53968}, // __builtin_lasx_xvadd_d
    {loongarch_lasx_xvadd_h, 53980}, // __builtin_lasx_xvadd_h
    {loongarch_lasx_xvadd_q, 53992}, // __builtin_lasx_xvadd_q
    {loongarch_lasx_xvadd_w, 54004}, // __builtin_lasx_xvadd_w
    {loongarch_lasx_xvadda_b, 54016}, // __builtin_lasx_xvadda_b
    {loongarch_lasx_xvadda_d, 54029}, // __builtin_lasx_xvadda_d
    {loongarch_lasx_xvadda_h, 54042}, // __builtin_lasx_xvadda_h
    {loongarch_lasx_xvadda_w, 54055}, // __builtin_lasx_xvadda_w
    {loongarch_lasx_xvaddi_bu, 54068}, // __builtin_lasx_xvaddi_bu
    {loongarch_lasx_xvaddi_du, 54082}, // __builtin_lasx_xvaddi_du
    {loongarch_lasx_xvaddi_hu, 54096}, // __builtin_lasx_xvaddi_hu
    {loongarch_lasx_xvaddi_wu, 54110}, // __builtin_lasx_xvaddi_wu
    {loongarch_lasx_xvaddwev_d_w, 54124}, // __builtin_lasx_xvaddwev_d_w
    {loongarch_lasx_xvaddwev_d_wu, 54141}, // __builtin_lasx_xvaddwev_d_wu
    {loongarch_lasx_xvaddwev_d_wu_w, 54159}, // __builtin_lasx_xvaddwev_d_wu_w
    {loongarch_lasx_xvaddwev_h_b, 54179}, // __builtin_lasx_xvaddwev_h_b
    {loongarch_lasx_xvaddwev_h_bu, 54196}, // __builtin_lasx_xvaddwev_h_bu
    {loongarch_lasx_xvaddwev_h_bu_b, 54214}, // __builtin_lasx_xvaddwev_h_bu_b
    {loongarch_lasx_xvaddwev_q_d, 54234}, // __builtin_lasx_xvaddwev_q_d
    {loongarch_lasx_xvaddwev_q_du, 54251}, // __builtin_lasx_xvaddwev_q_du
    {loongarch_lasx_xvaddwev_q_du_d, 54269}, // __builtin_lasx_xvaddwev_q_du_d
    {loongarch_lasx_xvaddwev_w_h, 54289}, // __builtin_lasx_xvaddwev_w_h
    {loongarch_lasx_xvaddwev_w_hu, 54306}, // __builtin_lasx_xvaddwev_w_hu
    {loongarch_lasx_xvaddwev_w_hu_h, 54324}, // __builtin_lasx_xvaddwev_w_hu_h
    {loongarch_lasx_xvaddwod_d_w, 54344}, // __builtin_lasx_xvaddwod_d_w
    {loongarch_lasx_xvaddwod_d_wu, 54361}, // __builtin_lasx_xvaddwod_d_wu
    {loongarch_lasx_xvaddwod_d_wu_w, 54379}, // __builtin_lasx_xvaddwod_d_wu_w
    {loongarch_lasx_xvaddwod_h_b, 54399}, // __builtin_lasx_xvaddwod_h_b
    {loongarch_lasx_xvaddwod_h_bu, 54416}, // __builtin_lasx_xvaddwod_h_bu
    {loongarch_lasx_xvaddwod_h_bu_b, 54434}, // __builtin_lasx_xvaddwod_h_bu_b
    {loongarch_lasx_xvaddwod_q_d, 54454}, // __builtin_lasx_xvaddwod_q_d
    {loongarch_lasx_xvaddwod_q_du, 54471}, // __builtin_lasx_xvaddwod_q_du
    {loongarch_lasx_xvaddwod_q_du_d, 54489}, // __builtin_lasx_xvaddwod_q_du_d
    {loongarch_lasx_xvaddwod_w_h, 54509}, // __builtin_lasx_xvaddwod_w_h
    {loongarch_lasx_xvaddwod_w_hu, 54526}, // __builtin_lasx_xvaddwod_w_hu
    {loongarch_lasx_xvaddwod_w_hu_h, 54544}, // __builtin_lasx_xvaddwod_w_hu_h
    {loongarch_lasx_xvand_v, 54564}, // __builtin_lasx_xvand_v
    {loongarch_lasx_xvandi_b, 54576}, // __builtin_lasx_xvandi_b
    {loongarch_lasx_xvandn_v, 54589}, // __builtin_lasx_xvandn_v
    {loongarch_lasx_xvavg_b, 54602}, // __builtin_lasx_xvavg_b
    {loongarch_lasx_xvavg_bu, 54614}, // __builtin_lasx_xvavg_bu
    {loongarch_lasx_xvavg_d, 54627}, // __builtin_lasx_xvavg_d
    {loongarch_lasx_xvavg_du, 54639}, // __builtin_lasx_xvavg_du
    {loongarch_lasx_xvavg_h, 54652}, // __builtin_lasx_xvavg_h
    {loongarch_lasx_xvavg_hu, 54664}, // __builtin_lasx_xvavg_hu
    {loongarch_lasx_xvavg_w, 54677}, // __builtin_lasx_xvavg_w
    {loongarch_lasx_xvavg_wu, 54689}, // __builtin_lasx_xvavg_wu
    {loongarch_lasx_xvavgr_b, 54702}, // __builtin_lasx_xvavgr_b
    {loongarch_lasx_xvavgr_bu, 54715}, // __builtin_lasx_xvavgr_bu
    {loongarch_lasx_xvavgr_d, 54729}, // __builtin_lasx_xvavgr_d
    {loongarch_lasx_xvavgr_du, 54742}, // __builtin_lasx_xvavgr_du
    {loongarch_lasx_xvavgr_h, 54756}, // __builtin_lasx_xvavgr_h
    {loongarch_lasx_xvavgr_hu, 54769}, // __builtin_lasx_xvavgr_hu
    {loongarch_lasx_xvavgr_w, 54783}, // __builtin_lasx_xvavgr_w
    {loongarch_lasx_xvavgr_wu, 54796}, // __builtin_lasx_xvavgr_wu
    {loongarch_lasx_xvbitclr_b, 54810}, // __builtin_lasx_xvbitclr_b
    {loongarch_lasx_xvbitclr_d, 54825}, // __builtin_lasx_xvbitclr_d
    {loongarch_lasx_xvbitclr_h, 54840}, // __builtin_lasx_xvbitclr_h
    {loongarch_lasx_xvbitclr_w, 54855}, // __builtin_lasx_xvbitclr_w
    {loongarch_lasx_xvbitclri_b, 54870}, // __builtin_lasx_xvbitclri_b
    {loongarch_lasx_xvbitclri_d, 54886}, // __builtin_lasx_xvbitclri_d
    {loongarch_lasx_xvbitclri_h, 54902}, // __builtin_lasx_xvbitclri_h
    {loongarch_lasx_xvbitclri_w, 54918}, // __builtin_lasx_xvbitclri_w
    {loongarch_lasx_xvbitrev_b, 54934}, // __builtin_lasx_xvbitrev_b
    {loongarch_lasx_xvbitrev_d, 54949}, // __builtin_lasx_xvbitrev_d
    {loongarch_lasx_xvbitrev_h, 54964}, // __builtin_lasx_xvbitrev_h
    {loongarch_lasx_xvbitrev_w, 54979}, // __builtin_lasx_xvbitrev_w
    {loongarch_lasx_xvbitrevi_b, 54994}, // __builtin_lasx_xvbitrevi_b
    {loongarch_lasx_xvbitrevi_d, 55010}, // __builtin_lasx_xvbitrevi_d
    {loongarch_lasx_xvbitrevi_h, 55026}, // __builtin_lasx_xvbitrevi_h
    {loongarch_lasx_xvbitrevi_w, 55042}, // __builtin_lasx_xvbitrevi_w
    {loongarch_lasx_xvbitsel_v, 55058}, // __builtin_lasx_xvbitsel_v
    {loongarch_lasx_xvbitseli_b, 55073}, // __builtin_lasx_xvbitseli_b
    {loongarch_lasx_xvbitset_b, 55089}, // __builtin_lasx_xvbitset_b
    {loongarch_lasx_xvbitset_d, 55104}, // __builtin_lasx_xvbitset_d
    {loongarch_lasx_xvbitset_h, 55119}, // __builtin_lasx_xvbitset_h
    {loongarch_lasx_xvbitset_w, 55134}, // __builtin_lasx_xvbitset_w
    {loongarch_lasx_xvbitseti_b, 55149}, // __builtin_lasx_xvbitseti_b
    {loongarch_lasx_xvbitseti_d, 55165}, // __builtin_lasx_xvbitseti_d
    {loongarch_lasx_xvbitseti_h, 55181}, // __builtin_lasx_xvbitseti_h
    {loongarch_lasx_xvbitseti_w, 55197}, // __builtin_lasx_xvbitseti_w
    {loongarch_lasx_xvbsll_v, 55213}, // __builtin_lasx_xvbsll_v
    {loongarch_lasx_xvbsrl_v, 55226}, // __builtin_lasx_xvbsrl_v
    {loongarch_lasx_xvclo_b, 55239}, // __builtin_lasx_xvclo_b
    {loongarch_lasx_xvclo_d, 55251}, // __builtin_lasx_xvclo_d
    {loongarch_lasx_xvclo_h, 55263}, // __builtin_lasx_xvclo_h
    {loongarch_lasx_xvclo_w, 55275}, // __builtin_lasx_xvclo_w
    {loongarch_lasx_xvclz_b, 55287}, // __builtin_lasx_xvclz_b
    {loongarch_lasx_xvclz_d, 55299}, // __builtin_lasx_xvclz_d
    {loongarch_lasx_xvclz_h, 55311}, // __builtin_lasx_xvclz_h
    {loongarch_lasx_xvclz_w, 55323}, // __builtin_lasx_xvclz_w
    {loongarch_lasx_xvdiv_b, 55335}, // __builtin_lasx_xvdiv_b
    {loongarch_lasx_xvdiv_bu, 55347}, // __builtin_lasx_xvdiv_bu
    {loongarch_lasx_xvdiv_d, 55360}, // __builtin_lasx_xvdiv_d
    {loongarch_lasx_xvdiv_du, 55372}, // __builtin_lasx_xvdiv_du
    {loongarch_lasx_xvdiv_h, 55385}, // __builtin_lasx_xvdiv_h
    {loongarch_lasx_xvdiv_hu, 55397}, // __builtin_lasx_xvdiv_hu
    {loongarch_lasx_xvdiv_w, 55410}, // __builtin_lasx_xvdiv_w
    {loongarch_lasx_xvdiv_wu, 55422}, // __builtin_lasx_xvdiv_wu
    {loongarch_lasx_xvexth_d_w, 55435}, // __builtin_lasx_xvexth_d_w
    {loongarch_lasx_xvexth_du_wu, 55450}, // __builtin_lasx_xvexth_du_wu
    {loongarch_lasx_xvexth_h_b, 55467}, // __builtin_lasx_xvexth_h_b
    {loongarch_lasx_xvexth_hu_bu, 55482}, // __builtin_lasx_xvexth_hu_bu
    {loongarch_lasx_xvexth_q_d, 55499}, // __builtin_lasx_xvexth_q_d
    {loongarch_lasx_xvexth_qu_du, 55514}, // __builtin_lasx_xvexth_qu_du
    {loongarch_lasx_xvexth_w_h, 55531}, // __builtin_lasx_xvexth_w_h
    {loongarch_lasx_xvexth_wu_hu, 55546}, // __builtin_lasx_xvexth_wu_hu
    {loongarch_lasx_xvextl_q_d, 55563}, // __builtin_lasx_xvextl_q_d
    {loongarch_lasx_xvextl_qu_du, 55578}, // __builtin_lasx_xvextl_qu_du
    {loongarch_lasx_xvextrins_b, 55595}, // __builtin_lasx_xvextrins_b
    {loongarch_lasx_xvextrins_d, 55611}, // __builtin_lasx_xvextrins_d
    {loongarch_lasx_xvextrins_h, 55627}, // __builtin_lasx_xvextrins_h
    {loongarch_lasx_xvextrins_w, 55643}, // __builtin_lasx_xvextrins_w
    {loongarch_lasx_xvfadd_d, 55659}, // __builtin_lasx_xvfadd_d
    {loongarch_lasx_xvfadd_s, 55672}, // __builtin_lasx_xvfadd_s
    {loongarch_lasx_xvfclass_d, 55685}, // __builtin_lasx_xvfclass_d
    {loongarch_lasx_xvfclass_s, 55700}, // __builtin_lasx_xvfclass_s
    {loongarch_lasx_xvfcmp_caf_d, 55715}, // __builtin_lasx_xvfcmp_caf_d
    {loongarch_lasx_xvfcmp_caf_s, 55732}, // __builtin_lasx_xvfcmp_caf_s
    {loongarch_lasx_xvfcmp_ceq_d, 55749}, // __builtin_lasx_xvfcmp_ceq_d
    {loongarch_lasx_xvfcmp_ceq_s, 55766}, // __builtin_lasx_xvfcmp_ceq_s
    {loongarch_lasx_xvfcmp_cle_d, 55783}, // __builtin_lasx_xvfcmp_cle_d
    {loongarch_lasx_xvfcmp_cle_s, 55800}, // __builtin_lasx_xvfcmp_cle_s
    {loongarch_lasx_xvfcmp_clt_d, 55817}, // __builtin_lasx_xvfcmp_clt_d
    {loongarch_lasx_xvfcmp_clt_s, 55834}, // __builtin_lasx_xvfcmp_clt_s
    {loongarch_lasx_xvfcmp_cne_d, 55851}, // __builtin_lasx_xvfcmp_cne_d
    {loongarch_lasx_xvfcmp_cne_s, 55868}, // __builtin_lasx_xvfcmp_cne_s
    {loongarch_lasx_xvfcmp_cor_d, 55885}, // __builtin_lasx_xvfcmp_cor_d
    {loongarch_lasx_xvfcmp_cor_s, 55902}, // __builtin_lasx_xvfcmp_cor_s
    {loongarch_lasx_xvfcmp_cueq_d, 55919}, // __builtin_lasx_xvfcmp_cueq_d
    {loongarch_lasx_xvfcmp_cueq_s, 55937}, // __builtin_lasx_xvfcmp_cueq_s
    {loongarch_lasx_xvfcmp_cule_d, 55955}, // __builtin_lasx_xvfcmp_cule_d
    {loongarch_lasx_xvfcmp_cule_s, 55973}, // __builtin_lasx_xvfcmp_cule_s
    {loongarch_lasx_xvfcmp_cult_d, 55991}, // __builtin_lasx_xvfcmp_cult_d
    {loongarch_lasx_xvfcmp_cult_s, 56009}, // __builtin_lasx_xvfcmp_cult_s
    {loongarch_lasx_xvfcmp_cun_d, 56027}, // __builtin_lasx_xvfcmp_cun_d
    {loongarch_lasx_xvfcmp_cun_s, 56044}, // __builtin_lasx_xvfcmp_cun_s
    {loongarch_lasx_xvfcmp_cune_d, 56061}, // __builtin_lasx_xvfcmp_cune_d
    {loongarch_lasx_xvfcmp_cune_s, 56079}, // __builtin_lasx_xvfcmp_cune_s
    {loongarch_lasx_xvfcmp_saf_d, 56097}, // __builtin_lasx_xvfcmp_saf_d
    {loongarch_lasx_xvfcmp_saf_s, 56114}, // __builtin_lasx_xvfcmp_saf_s
    {loongarch_lasx_xvfcmp_seq_d, 56131}, // __builtin_lasx_xvfcmp_seq_d
    {loongarch_lasx_xvfcmp_seq_s, 56148}, // __builtin_lasx_xvfcmp_seq_s
    {loongarch_lasx_xvfcmp_sle_d, 56165}, // __builtin_lasx_xvfcmp_sle_d
    {loongarch_lasx_xvfcmp_sle_s, 56182}, // __builtin_lasx_xvfcmp_sle_s
    {loongarch_lasx_xvfcmp_slt_d, 56199}, // __builtin_lasx_xvfcmp_slt_d
    {loongarch_lasx_xvfcmp_slt_s, 56216}, // __builtin_lasx_xvfcmp_slt_s
    {loongarch_lasx_xvfcmp_sne_d, 56233}, // __builtin_lasx_xvfcmp_sne_d
    {loongarch_lasx_xvfcmp_sne_s, 56250}, // __builtin_lasx_xvfcmp_sne_s
    {loongarch_lasx_xvfcmp_sor_d, 56267}, // __builtin_lasx_xvfcmp_sor_d
    {loongarch_lasx_xvfcmp_sor_s, 56284}, // __builtin_lasx_xvfcmp_sor_s
    {loongarch_lasx_xvfcmp_sueq_d, 56301}, // __builtin_lasx_xvfcmp_sueq_d
    {loongarch_lasx_xvfcmp_sueq_s, 56319}, // __builtin_lasx_xvfcmp_sueq_s
    {loongarch_lasx_xvfcmp_sule_d, 56337}, // __builtin_lasx_xvfcmp_sule_d
    {loongarch_lasx_xvfcmp_sule_s, 56355}, // __builtin_lasx_xvfcmp_sule_s
    {loongarch_lasx_xvfcmp_sult_d, 56373}, // __builtin_lasx_xvfcmp_sult_d
    {loongarch_lasx_xvfcmp_sult_s, 56391}, // __builtin_lasx_xvfcmp_sult_s
    {loongarch_lasx_xvfcmp_sun_d, 56409}, // __builtin_lasx_xvfcmp_sun_d
    {loongarch_lasx_xvfcmp_sun_s, 56426}, // __builtin_lasx_xvfcmp_sun_s
    {loongarch_lasx_xvfcmp_sune_d, 56443}, // __builtin_lasx_xvfcmp_sune_d
    {loongarch_lasx_xvfcmp_sune_s, 56461}, // __builtin_lasx_xvfcmp_sune_s
    {loongarch_lasx_xvfcvt_h_s, 56479}, // __builtin_lasx_xvfcvt_h_s
    {loongarch_lasx_xvfcvt_s_d, 56494}, // __builtin_lasx_xvfcvt_s_d
    {loongarch_lasx_xvfcvth_d_s, 56509}, // __builtin_lasx_xvfcvth_d_s
    {loongarch_lasx_xvfcvth_s_h, 56525}, // __builtin_lasx_xvfcvth_s_h
    {loongarch_lasx_xvfcvtl_d_s, 56541}, // __builtin_lasx_xvfcvtl_d_s
    {loongarch_lasx_xvfcvtl_s_h, 56557}, // __builtin_lasx_xvfcvtl_s_h
    {loongarch_lasx_xvfdiv_d, 56573}, // __builtin_lasx_xvfdiv_d
    {loongarch_lasx_xvfdiv_s, 56586}, // __builtin_lasx_xvfdiv_s
    {loongarch_lasx_xvffint_d_l, 56599}, // __builtin_lasx_xvffint_d_l
    {loongarch_lasx_xvffint_d_lu, 56615}, // __builtin_lasx_xvffint_d_lu
    {loongarch_lasx_xvffint_s_l, 56632}, // __builtin_lasx_xvffint_s_l
    {loongarch_lasx_xvffint_s_w, 56648}, // __builtin_lasx_xvffint_s_w
    {loongarch_lasx_xvffint_s_wu, 56664}, // __builtin_lasx_xvffint_s_wu
    {loongarch_lasx_xvffinth_d_w, 56681}, // __builtin_lasx_xvffinth_d_w
    {loongarch_lasx_xvffintl_d_w, 56698}, // __builtin_lasx_xvffintl_d_w
    {loongarch_lasx_xvflogb_d, 56715}, // __builtin_lasx_xvflogb_d
    {loongarch_lasx_xvflogb_s, 56729}, // __builtin_lasx_xvflogb_s
    {loongarch_lasx_xvfmadd_d, 56743}, // __builtin_lasx_xvfmadd_d
    {loongarch_lasx_xvfmadd_s, 56757}, // __builtin_lasx_xvfmadd_s
    {loongarch_lasx_xvfmax_d, 56771}, // __builtin_lasx_xvfmax_d
    {loongarch_lasx_xvfmax_s, 56784}, // __builtin_lasx_xvfmax_s
    {loongarch_lasx_xvfmaxa_d, 56797}, // __builtin_lasx_xvfmaxa_d
    {loongarch_lasx_xvfmaxa_s, 56811}, // __builtin_lasx_xvfmaxa_s
    {loongarch_lasx_xvfmin_d, 56825}, // __builtin_lasx_xvfmin_d
    {loongarch_lasx_xvfmin_s, 56838}, // __builtin_lasx_xvfmin_s
    {loongarch_lasx_xvfmina_d, 56851}, // __builtin_lasx_xvfmina_d
    {loongarch_lasx_xvfmina_s, 56865}, // __builtin_lasx_xvfmina_s
    {loongarch_lasx_xvfmsub_d, 56879}, // __builtin_lasx_xvfmsub_d
    {loongarch_lasx_xvfmsub_s, 56893}, // __builtin_lasx_xvfmsub_s
    {loongarch_lasx_xvfmul_d, 56907}, // __builtin_lasx_xvfmul_d
    {loongarch_lasx_xvfmul_s, 56920}, // __builtin_lasx_xvfmul_s
    {loongarch_lasx_xvfnmadd_d, 56933}, // __builtin_lasx_xvfnmadd_d
    {loongarch_lasx_xvfnmadd_s, 56948}, // __builtin_lasx_xvfnmadd_s
    {loongarch_lasx_xvfnmsub_d, 56963}, // __builtin_lasx_xvfnmsub_d
    {loongarch_lasx_xvfnmsub_s, 56978}, // __builtin_lasx_xvfnmsub_s
    {loongarch_lasx_xvfrecip_d, 56993}, // __builtin_lasx_xvfrecip_d
    {loongarch_lasx_xvfrecip_s, 57008}, // __builtin_lasx_xvfrecip_s
    {loongarch_lasx_xvfrecipe_d, 57023}, // __builtin_lasx_xvfrecipe_d
    {loongarch_lasx_xvfrecipe_s, 57039}, // __builtin_lasx_xvfrecipe_s
    {loongarch_lasx_xvfrint_d, 57055}, // __builtin_lasx_xvfrint_d
    {loongarch_lasx_xvfrint_s, 57069}, // __builtin_lasx_xvfrint_s
    {loongarch_lasx_xvfrintrm_d, 57083}, // __builtin_lasx_xvfrintrm_d
    {loongarch_lasx_xvfrintrm_s, 57099}, // __builtin_lasx_xvfrintrm_s
    {loongarch_lasx_xvfrintrne_d, 57115}, // __builtin_lasx_xvfrintrne_d
    {loongarch_lasx_xvfrintrne_s, 57132}, // __builtin_lasx_xvfrintrne_s
    {loongarch_lasx_xvfrintrp_d, 57149}, // __builtin_lasx_xvfrintrp_d
    {loongarch_lasx_xvfrintrp_s, 57165}, // __builtin_lasx_xvfrintrp_s
    {loongarch_lasx_xvfrintrz_d, 57181}, // __builtin_lasx_xvfrintrz_d
    {loongarch_lasx_xvfrintrz_s, 57197}, // __builtin_lasx_xvfrintrz_s
    {loongarch_lasx_xvfrsqrt_d, 57213}, // __builtin_lasx_xvfrsqrt_d
    {loongarch_lasx_xvfrsqrt_s, 57228}, // __builtin_lasx_xvfrsqrt_s
    {loongarch_lasx_xvfrsqrte_d, 57243}, // __builtin_lasx_xvfrsqrte_d
    {loongarch_lasx_xvfrsqrte_s, 57259}, // __builtin_lasx_xvfrsqrte_s
    {loongarch_lasx_xvfrstp_b, 57275}, // __builtin_lasx_xvfrstp_b
    {loongarch_lasx_xvfrstp_h, 57289}, // __builtin_lasx_xvfrstp_h
    {loongarch_lasx_xvfrstpi_b, 57303}, // __builtin_lasx_xvfrstpi_b
    {loongarch_lasx_xvfrstpi_h, 57318}, // __builtin_lasx_xvfrstpi_h
    {loongarch_lasx_xvfsqrt_d, 57333}, // __builtin_lasx_xvfsqrt_d
    {loongarch_lasx_xvfsqrt_s, 57347}, // __builtin_lasx_xvfsqrt_s
    {loongarch_lasx_xvfsub_d, 57361}, // __builtin_lasx_xvfsub_d
    {loongarch_lasx_xvfsub_s, 57374}, // __builtin_lasx_xvfsub_s
    {loongarch_lasx_xvftint_l_d, 57387}, // __builtin_lasx_xvftint_l_d
    {loongarch_lasx_xvftint_lu_d, 57403}, // __builtin_lasx_xvftint_lu_d
    {loongarch_lasx_xvftint_w_d, 57420}, // __builtin_lasx_xvftint_w_d
    {loongarch_lasx_xvftint_w_s, 57436}, // __builtin_lasx_xvftint_w_s
    {loongarch_lasx_xvftint_wu_s, 57452}, // __builtin_lasx_xvftint_wu_s
    {loongarch_lasx_xvftinth_l_s, 57469}, // __builtin_lasx_xvftinth_l_s
    {loongarch_lasx_xvftintl_l_s, 57486}, // __builtin_lasx_xvftintl_l_s
    {loongarch_lasx_xvftintrm_l_d, 57503}, // __builtin_lasx_xvftintrm_l_d
    {loongarch_lasx_xvftintrm_w_d, 57521}, // __builtin_lasx_xvftintrm_w_d
    {loongarch_lasx_xvftintrm_w_s, 57539}, // __builtin_lasx_xvftintrm_w_s
    {loongarch_lasx_xvftintrmh_l_s, 57557}, // __builtin_lasx_xvftintrmh_l_s
    {loongarch_lasx_xvftintrml_l_s, 57576}, // __builtin_lasx_xvftintrml_l_s
    {loongarch_lasx_xvftintrne_l_d, 57595}, // __builtin_lasx_xvftintrne_l_d
    {loongarch_lasx_xvftintrne_w_d, 57614}, // __builtin_lasx_xvftintrne_w_d
    {loongarch_lasx_xvftintrne_w_s, 57633}, // __builtin_lasx_xvftintrne_w_s
    {loongarch_lasx_xvftintrneh_l_s, 57652}, // __builtin_lasx_xvftintrneh_l_s
    {loongarch_lasx_xvftintrnel_l_s, 57672}, // __builtin_lasx_xvftintrnel_l_s
    {loongarch_lasx_xvftintrp_l_d, 57692}, // __builtin_lasx_xvftintrp_l_d
    {loongarch_lasx_xvftintrp_w_d, 57710}, // __builtin_lasx_xvftintrp_w_d
    {loongarch_lasx_xvftintrp_w_s, 57728}, // __builtin_lasx_xvftintrp_w_s
    {loongarch_lasx_xvftintrph_l_s, 57746}, // __builtin_lasx_xvftintrph_l_s
    {loongarch_lasx_xvftintrpl_l_s, 57765}, // __builtin_lasx_xvftintrpl_l_s
    {loongarch_lasx_xvftintrz_l_d, 57784}, // __builtin_lasx_xvftintrz_l_d
    {loongarch_lasx_xvftintrz_lu_d, 57802}, // __builtin_lasx_xvftintrz_lu_d
    {loongarch_lasx_xvftintrz_w_d, 57821}, // __builtin_lasx_xvftintrz_w_d
    {loongarch_lasx_xvftintrz_w_s, 57839}, // __builtin_lasx_xvftintrz_w_s
    {loongarch_lasx_xvftintrz_wu_s, 57857}, // __builtin_lasx_xvftintrz_wu_s
    {loongarch_lasx_xvftintrzh_l_s, 57876}, // __builtin_lasx_xvftintrzh_l_s
    {loongarch_lasx_xvftintrzl_l_s, 57895}, // __builtin_lasx_xvftintrzl_l_s
    {loongarch_lasx_xvhaddw_d_w, 57914}, // __builtin_lasx_xvhaddw_d_w
    {loongarch_lasx_xvhaddw_du_wu, 57930}, // __builtin_lasx_xvhaddw_du_wu
    {loongarch_lasx_xvhaddw_h_b, 57948}, // __builtin_lasx_xvhaddw_h_b
    {loongarch_lasx_xvhaddw_hu_bu, 57964}, // __builtin_lasx_xvhaddw_hu_bu
    {loongarch_lasx_xvhaddw_q_d, 57982}, // __builtin_lasx_xvhaddw_q_d
    {loongarch_lasx_xvhaddw_qu_du, 57998}, // __builtin_lasx_xvhaddw_qu_du
    {loongarch_lasx_xvhaddw_w_h, 58016}, // __builtin_lasx_xvhaddw_w_h
    {loongarch_lasx_xvhaddw_wu_hu, 58032}, // __builtin_lasx_xvhaddw_wu_hu
    {loongarch_lasx_xvhsubw_d_w, 58050}, // __builtin_lasx_xvhsubw_d_w
    {loongarch_lasx_xvhsubw_du_wu, 58066}, // __builtin_lasx_xvhsubw_du_wu
    {loongarch_lasx_xvhsubw_h_b, 58084}, // __builtin_lasx_xvhsubw_h_b
    {loongarch_lasx_xvhsubw_hu_bu, 58100}, // __builtin_lasx_xvhsubw_hu_bu
    {loongarch_lasx_xvhsubw_q_d, 58118}, // __builtin_lasx_xvhsubw_q_d
    {loongarch_lasx_xvhsubw_qu_du, 58134}, // __builtin_lasx_xvhsubw_qu_du
    {loongarch_lasx_xvhsubw_w_h, 58152}, // __builtin_lasx_xvhsubw_w_h
    {loongarch_lasx_xvhsubw_wu_hu, 58168}, // __builtin_lasx_xvhsubw_wu_hu
    {loongarch_lasx_xvilvh_b, 58186}, // __builtin_lasx_xvilvh_b
    {loongarch_lasx_xvilvh_d, 58199}, // __builtin_lasx_xvilvh_d
    {loongarch_lasx_xvilvh_h, 58212}, // __builtin_lasx_xvilvh_h
    {loongarch_lasx_xvilvh_w, 58225}, // __builtin_lasx_xvilvh_w
    {loongarch_lasx_xvilvl_b, 58238}, // __builtin_lasx_xvilvl_b
    {loongarch_lasx_xvilvl_d, 58251}, // __builtin_lasx_xvilvl_d
    {loongarch_lasx_xvilvl_h, 58264}, // __builtin_lasx_xvilvl_h
    {loongarch_lasx_xvilvl_w, 58277}, // __builtin_lasx_xvilvl_w
    {loongarch_lasx_xvinsgr2vr_d, 58290}, // __builtin_lasx_xvinsgr2vr_d
    {loongarch_lasx_xvinsgr2vr_w, 58307}, // __builtin_lasx_xvinsgr2vr_w
    {loongarch_lasx_xvinsve0_d, 58324}, // __builtin_lasx_xvinsve0_d
    {loongarch_lasx_xvinsve0_w, 58339}, // __builtin_lasx_xvinsve0_w
    {loongarch_lasx_xvld, 58354}, // __builtin_lasx_xvld
    {loongarch_lasx_xvldi, 58363}, // __builtin_lasx_xvldi
    {loongarch_lasx_xvldrepl_b, 58373}, // __builtin_lasx_xvldrepl_b
    {loongarch_lasx_xvldrepl_d, 58388}, // __builtin_lasx_xvldrepl_d
    {loongarch_lasx_xvldrepl_h, 58403}, // __builtin_lasx_xvldrepl_h
    {loongarch_lasx_xvldrepl_w, 58418}, // __builtin_lasx_xvldrepl_w
    {loongarch_lasx_xvldx, 58433}, // __builtin_lasx_xvldx
    {loongarch_lasx_xvmadd_b, 58443}, // __builtin_lasx_xvmadd_b
    {loongarch_lasx_xvmadd_d, 58456}, // __builtin_lasx_xvmadd_d
    {loongarch_lasx_xvmadd_h, 58469}, // __builtin_lasx_xvmadd_h
    {loongarch_lasx_xvmadd_w, 58482}, // __builtin_lasx_xvmadd_w
    {loongarch_lasx_xvmaddwev_d_w, 58495}, // __builtin_lasx_xvmaddwev_d_w
    {loongarch_lasx_xvmaddwev_d_wu, 58513}, // __builtin_lasx_xvmaddwev_d_wu
    {loongarch_lasx_xvmaddwev_d_wu_w, 58532}, // __builtin_lasx_xvmaddwev_d_wu_w
    {loongarch_lasx_xvmaddwev_h_b, 58553}, // __builtin_lasx_xvmaddwev_h_b
    {loongarch_lasx_xvmaddwev_h_bu, 58571}, // __builtin_lasx_xvmaddwev_h_bu
    {loongarch_lasx_xvmaddwev_h_bu_b, 58590}, // __builtin_lasx_xvmaddwev_h_bu_b
    {loongarch_lasx_xvmaddwev_q_d, 58611}, // __builtin_lasx_xvmaddwev_q_d
    {loongarch_lasx_xvmaddwev_q_du, 58629}, // __builtin_lasx_xvmaddwev_q_du
    {loongarch_lasx_xvmaddwev_q_du_d, 58648}, // __builtin_lasx_xvmaddwev_q_du_d
    {loongarch_lasx_xvmaddwev_w_h, 58669}, // __builtin_lasx_xvmaddwev_w_h
    {loongarch_lasx_xvmaddwev_w_hu, 58687}, // __builtin_lasx_xvmaddwev_w_hu
    {loongarch_lasx_xvmaddwev_w_hu_h, 58706}, // __builtin_lasx_xvmaddwev_w_hu_h
    {loongarch_lasx_xvmaddwod_d_w, 58727}, // __builtin_lasx_xvmaddwod_d_w
    {loongarch_lasx_xvmaddwod_d_wu, 58745}, // __builtin_lasx_xvmaddwod_d_wu
    {loongarch_lasx_xvmaddwod_d_wu_w, 58764}, // __builtin_lasx_xvmaddwod_d_wu_w
    {loongarch_lasx_xvmaddwod_h_b, 58785}, // __builtin_lasx_xvmaddwod_h_b
    {loongarch_lasx_xvmaddwod_h_bu, 58803}, // __builtin_lasx_xvmaddwod_h_bu
    {loongarch_lasx_xvmaddwod_h_bu_b, 58822}, // __builtin_lasx_xvmaddwod_h_bu_b
    {loongarch_lasx_xvmaddwod_q_d, 58843}, // __builtin_lasx_xvmaddwod_q_d
    {loongarch_lasx_xvmaddwod_q_du, 58861}, // __builtin_lasx_xvmaddwod_q_du
    {loongarch_lasx_xvmaddwod_q_du_d, 58880}, // __builtin_lasx_xvmaddwod_q_du_d
    {loongarch_lasx_xvmaddwod_w_h, 58901}, // __builtin_lasx_xvmaddwod_w_h
    {loongarch_lasx_xvmaddwod_w_hu, 58919}, // __builtin_lasx_xvmaddwod_w_hu
    {loongarch_lasx_xvmaddwod_w_hu_h, 58938}, // __builtin_lasx_xvmaddwod_w_hu_h
    {loongarch_lasx_xvmax_b, 58959}, // __builtin_lasx_xvmax_b
    {loongarch_lasx_xvmax_bu, 58971}, // __builtin_lasx_xvmax_bu
    {loongarch_lasx_xvmax_d, 58984}, // __builtin_lasx_xvmax_d
    {loongarch_lasx_xvmax_du, 58996}, // __builtin_lasx_xvmax_du
    {loongarch_lasx_xvmax_h, 59009}, // __builtin_lasx_xvmax_h
    {loongarch_lasx_xvmax_hu, 59021}, // __builtin_lasx_xvmax_hu
    {loongarch_lasx_xvmax_w, 59034}, // __builtin_lasx_xvmax_w
    {loongarch_lasx_xvmax_wu, 59046}, // __builtin_lasx_xvmax_wu
    {loongarch_lasx_xvmaxi_b, 59059}, // __builtin_lasx_xvmaxi_b
    {loongarch_lasx_xvmaxi_bu, 59072}, // __builtin_lasx_xvmaxi_bu
    {loongarch_lasx_xvmaxi_d, 59086}, // __builtin_lasx_xvmaxi_d
    {loongarch_lasx_xvmaxi_du, 59099}, // __builtin_lasx_xvmaxi_du
    {loongarch_lasx_xvmaxi_h, 59113}, // __builtin_lasx_xvmaxi_h
    {loongarch_lasx_xvmaxi_hu, 59126}, // __builtin_lasx_xvmaxi_hu
    {loongarch_lasx_xvmaxi_w, 59140}, // __builtin_lasx_xvmaxi_w
    {loongarch_lasx_xvmaxi_wu, 59153}, // __builtin_lasx_xvmaxi_wu
    {loongarch_lasx_xvmin_b, 59167}, // __builtin_lasx_xvmin_b
    {loongarch_lasx_xvmin_bu, 59179}, // __builtin_lasx_xvmin_bu
    {loongarch_lasx_xvmin_d, 59192}, // __builtin_lasx_xvmin_d
    {loongarch_lasx_xvmin_du, 59204}, // __builtin_lasx_xvmin_du
    {loongarch_lasx_xvmin_h, 59217}, // __builtin_lasx_xvmin_h
    {loongarch_lasx_xvmin_hu, 59229}, // __builtin_lasx_xvmin_hu
    {loongarch_lasx_xvmin_w, 59242}, // __builtin_lasx_xvmin_w
    {loongarch_lasx_xvmin_wu, 59254}, // __builtin_lasx_xvmin_wu
    {loongarch_lasx_xvmini_b, 59267}, // __builtin_lasx_xvmini_b
    {loongarch_lasx_xvmini_bu, 59280}, // __builtin_lasx_xvmini_bu
    {loongarch_lasx_xvmini_d, 59294}, // __builtin_lasx_xvmini_d
    {loongarch_lasx_xvmini_du, 59307}, // __builtin_lasx_xvmini_du
    {loongarch_lasx_xvmini_h, 59321}, // __builtin_lasx_xvmini_h
    {loongarch_lasx_xvmini_hu, 59334}, // __builtin_lasx_xvmini_hu
    {loongarch_lasx_xvmini_w, 59348}, // __builtin_lasx_xvmini_w
    {loongarch_lasx_xvmini_wu, 59361}, // __builtin_lasx_xvmini_wu
    {loongarch_lasx_xvmod_b, 59375}, // __builtin_lasx_xvmod_b
    {loongarch_lasx_xvmod_bu, 59387}, // __builtin_lasx_xvmod_bu
    {loongarch_lasx_xvmod_d, 59400}, // __builtin_lasx_xvmod_d
    {loongarch_lasx_xvmod_du, 59412}, // __builtin_lasx_xvmod_du
    {loongarch_lasx_xvmod_h, 59425}, // __builtin_lasx_xvmod_h
    {loongarch_lasx_xvmod_hu, 59437}, // __builtin_lasx_xvmod_hu
    {loongarch_lasx_xvmod_w, 59450}, // __builtin_lasx_xvmod_w
    {loongarch_lasx_xvmod_wu, 59462}, // __builtin_lasx_xvmod_wu
    {loongarch_lasx_xvmskgez_b, 59475}, // __builtin_lasx_xvmskgez_b
    {loongarch_lasx_xvmskltz_b, 59490}, // __builtin_lasx_xvmskltz_b
    {loongarch_lasx_xvmskltz_d, 59505}, // __builtin_lasx_xvmskltz_d
    {loongarch_lasx_xvmskltz_h, 59520}, // __builtin_lasx_xvmskltz_h
    {loongarch_lasx_xvmskltz_w, 59535}, // __builtin_lasx_xvmskltz_w
    {loongarch_lasx_xvmsknz_b, 59550}, // __builtin_lasx_xvmsknz_b
    {loongarch_lasx_xvmsub_b, 59564}, // __builtin_lasx_xvmsub_b
    {loongarch_lasx_xvmsub_d, 59577}, // __builtin_lasx_xvmsub_d
    {loongarch_lasx_xvmsub_h, 59590}, // __builtin_lasx_xvmsub_h
    {loongarch_lasx_xvmsub_w, 59603}, // __builtin_lasx_xvmsub_w
    {loongarch_lasx_xvmuh_b, 59616}, // __builtin_lasx_xvmuh_b
    {loongarch_lasx_xvmuh_bu, 59628}, // __builtin_lasx_xvmuh_bu
    {loongarch_lasx_xvmuh_d, 59641}, // __builtin_lasx_xvmuh_d
    {loongarch_lasx_xvmuh_du, 59653}, // __builtin_lasx_xvmuh_du
    {loongarch_lasx_xvmuh_h, 59666}, // __builtin_lasx_xvmuh_h
    {loongarch_lasx_xvmuh_hu, 59678}, // __builtin_lasx_xvmuh_hu
    {loongarch_lasx_xvmuh_w, 59691}, // __builtin_lasx_xvmuh_w
    {loongarch_lasx_xvmuh_wu, 59703}, // __builtin_lasx_xvmuh_wu
    {loongarch_lasx_xvmul_b, 59716}, // __builtin_lasx_xvmul_b
    {loongarch_lasx_xvmul_d, 59728}, // __builtin_lasx_xvmul_d
    {loongarch_lasx_xvmul_h, 59740}, // __builtin_lasx_xvmul_h
    {loongarch_lasx_xvmul_w, 59752}, // __builtin_lasx_xvmul_w
    {loongarch_lasx_xvmulwev_d_w, 59764}, // __builtin_lasx_xvmulwev_d_w
    {loongarch_lasx_xvmulwev_d_wu, 59781}, // __builtin_lasx_xvmulwev_d_wu
    {loongarch_lasx_xvmulwev_d_wu_w, 59799}, // __builtin_lasx_xvmulwev_d_wu_w
    {loongarch_lasx_xvmulwev_h_b, 59819}, // __builtin_lasx_xvmulwev_h_b
    {loongarch_lasx_xvmulwev_h_bu, 59836}, // __builtin_lasx_xvmulwev_h_bu
    {loongarch_lasx_xvmulwev_h_bu_b, 59854}, // __builtin_lasx_xvmulwev_h_bu_b
    {loongarch_lasx_xvmulwev_q_d, 59874}, // __builtin_lasx_xvmulwev_q_d
    {loongarch_lasx_xvmulwev_q_du, 59891}, // __builtin_lasx_xvmulwev_q_du
    {loongarch_lasx_xvmulwev_q_du_d, 59909}, // __builtin_lasx_xvmulwev_q_du_d
    {loongarch_lasx_xvmulwev_w_h, 59929}, // __builtin_lasx_xvmulwev_w_h
    {loongarch_lasx_xvmulwev_w_hu, 59946}, // __builtin_lasx_xvmulwev_w_hu
    {loongarch_lasx_xvmulwev_w_hu_h, 59964}, // __builtin_lasx_xvmulwev_w_hu_h
    {loongarch_lasx_xvmulwod_d_w, 59984}, // __builtin_lasx_xvmulwod_d_w
    {loongarch_lasx_xvmulwod_d_wu, 60001}, // __builtin_lasx_xvmulwod_d_wu
    {loongarch_lasx_xvmulwod_d_wu_w, 60019}, // __builtin_lasx_xvmulwod_d_wu_w
    {loongarch_lasx_xvmulwod_h_b, 60039}, // __builtin_lasx_xvmulwod_h_b
    {loongarch_lasx_xvmulwod_h_bu, 60056}, // __builtin_lasx_xvmulwod_h_bu
    {loongarch_lasx_xvmulwod_h_bu_b, 60074}, // __builtin_lasx_xvmulwod_h_bu_b
    {loongarch_lasx_xvmulwod_q_d, 60094}, // __builtin_lasx_xvmulwod_q_d
    {loongarch_lasx_xvmulwod_q_du, 60111}, // __builtin_lasx_xvmulwod_q_du
    {loongarch_lasx_xvmulwod_q_du_d, 60129}, // __builtin_lasx_xvmulwod_q_du_d
    {loongarch_lasx_xvmulwod_w_h, 60149}, // __builtin_lasx_xvmulwod_w_h
    {loongarch_lasx_xvmulwod_w_hu, 60166}, // __builtin_lasx_xvmulwod_w_hu
    {loongarch_lasx_xvmulwod_w_hu_h, 60184}, // __builtin_lasx_xvmulwod_w_hu_h
    {loongarch_lasx_xvneg_b, 60204}, // __builtin_lasx_xvneg_b
    {loongarch_lasx_xvneg_d, 60216}, // __builtin_lasx_xvneg_d
    {loongarch_lasx_xvneg_h, 60228}, // __builtin_lasx_xvneg_h
    {loongarch_lasx_xvneg_w, 60240}, // __builtin_lasx_xvneg_w
    {loongarch_lasx_xvnor_v, 60252}, // __builtin_lasx_xvnor_v
    {loongarch_lasx_xvnori_b, 60264}, // __builtin_lasx_xvnori_b
    {loongarch_lasx_xvor_v, 60277}, // __builtin_lasx_xvor_v
    {loongarch_lasx_xvori_b, 60288}, // __builtin_lasx_xvori_b
    {loongarch_lasx_xvorn_v, 60300}, // __builtin_lasx_xvorn_v
    {loongarch_lasx_xvpackev_b, 60312}, // __builtin_lasx_xvpackev_b
    {loongarch_lasx_xvpackev_d, 60327}, // __builtin_lasx_xvpackev_d
    {loongarch_lasx_xvpackev_h, 60342}, // __builtin_lasx_xvpackev_h
    {loongarch_lasx_xvpackev_w, 60357}, // __builtin_lasx_xvpackev_w
    {loongarch_lasx_xvpackod_b, 60372}, // __builtin_lasx_xvpackod_b
    {loongarch_lasx_xvpackod_d, 60387}, // __builtin_lasx_xvpackod_d
    {loongarch_lasx_xvpackod_h, 60402}, // __builtin_lasx_xvpackod_h
    {loongarch_lasx_xvpackod_w, 60417}, // __builtin_lasx_xvpackod_w
    {loongarch_lasx_xvpcnt_b, 60432}, // __builtin_lasx_xvpcnt_b
    {loongarch_lasx_xvpcnt_d, 60445}, // __builtin_lasx_xvpcnt_d
    {loongarch_lasx_xvpcnt_h, 60458}, // __builtin_lasx_xvpcnt_h
    {loongarch_lasx_xvpcnt_w, 60471}, // __builtin_lasx_xvpcnt_w
    {loongarch_lasx_xvperm_w, 60484}, // __builtin_lasx_xvperm_w
    {loongarch_lasx_xvpermi_d, 60497}, // __builtin_lasx_xvpermi_d
    {loongarch_lasx_xvpermi_q, 60511}, // __builtin_lasx_xvpermi_q
    {loongarch_lasx_xvpermi_w, 60525}, // __builtin_lasx_xvpermi_w
    {loongarch_lasx_xvpickev_b, 60539}, // __builtin_lasx_xvpickev_b
    {loongarch_lasx_xvpickev_d, 60554}, // __builtin_lasx_xvpickev_d
    {loongarch_lasx_xvpickev_h, 60569}, // __builtin_lasx_xvpickev_h
    {loongarch_lasx_xvpickev_w, 60584}, // __builtin_lasx_xvpickev_w
    {loongarch_lasx_xvpickod_b, 60599}, // __builtin_lasx_xvpickod_b
    {loongarch_lasx_xvpickod_d, 60614}, // __builtin_lasx_xvpickod_d
    {loongarch_lasx_xvpickod_h, 60629}, // __builtin_lasx_xvpickod_h
    {loongarch_lasx_xvpickod_w, 60644}, // __builtin_lasx_xvpickod_w
    {loongarch_lasx_xvpickve2gr_d, 60659}, // __builtin_lasx_xvpickve2gr_d
    {loongarch_lasx_xvpickve2gr_du, 60677}, // __builtin_lasx_xvpickve2gr_du
    {loongarch_lasx_xvpickve2gr_w, 60696}, // __builtin_lasx_xvpickve2gr_w
    {loongarch_lasx_xvpickve2gr_wu, 60714}, // __builtin_lasx_xvpickve2gr_wu
    {loongarch_lasx_xvpickve_d, 60733}, // __builtin_lasx_xvpickve_d
    {loongarch_lasx_xvpickve_d_f, 60748}, // __builtin_lasx_xvpickve_d_f
    {loongarch_lasx_xvpickve_w, 60765}, // __builtin_lasx_xvpickve_w
    {loongarch_lasx_xvpickve_w_f, 60780}, // __builtin_lasx_xvpickve_w_f
    {loongarch_lasx_xvrepl128vei_b, 60797}, // __builtin_lasx_xvrepl128vei_b
    {loongarch_lasx_xvrepl128vei_d, 60816}, // __builtin_lasx_xvrepl128vei_d
    {loongarch_lasx_xvrepl128vei_h, 60835}, // __builtin_lasx_xvrepl128vei_h
    {loongarch_lasx_xvrepl128vei_w, 60854}, // __builtin_lasx_xvrepl128vei_w
    {loongarch_lasx_xvreplgr2vr_b, 60873}, // __builtin_lasx_xvreplgr2vr_b
    {loongarch_lasx_xvreplgr2vr_d, 60891}, // __builtin_lasx_xvreplgr2vr_d
    {loongarch_lasx_xvreplgr2vr_h, 60909}, // __builtin_lasx_xvreplgr2vr_h
    {loongarch_lasx_xvreplgr2vr_w, 60927}, // __builtin_lasx_xvreplgr2vr_w
    {loongarch_lasx_xvrepli_b, 60945}, // __builtin_lasx_xvrepli_b
    {loongarch_lasx_xvrepli_d, 60959}, // __builtin_lasx_xvrepli_d
    {loongarch_lasx_xvrepli_h, 60973}, // __builtin_lasx_xvrepli_h
    {loongarch_lasx_xvrepli_w, 60987}, // __builtin_lasx_xvrepli_w
    {loongarch_lasx_xvreplve0_b, 61001}, // __builtin_lasx_xvreplve0_b
    {loongarch_lasx_xvreplve0_d, 61017}, // __builtin_lasx_xvreplve0_d
    {loongarch_lasx_xvreplve0_h, 61033}, // __builtin_lasx_xvreplve0_h
    {loongarch_lasx_xvreplve0_q, 61049}, // __builtin_lasx_xvreplve0_q
    {loongarch_lasx_xvreplve0_w, 61065}, // __builtin_lasx_xvreplve0_w
    {loongarch_lasx_xvreplve_b, 61081}, // __builtin_lasx_xvreplve_b
    {loongarch_lasx_xvreplve_d, 61096}, // __builtin_lasx_xvreplve_d
    {loongarch_lasx_xvreplve_h, 61111}, // __builtin_lasx_xvreplve_h
    {loongarch_lasx_xvreplve_w, 61126}, // __builtin_lasx_xvreplve_w
    {loongarch_lasx_xvrotr_b, 61141}, // __builtin_lasx_xvrotr_b
    {loongarch_lasx_xvrotr_d, 61154}, // __builtin_lasx_xvrotr_d
    {loongarch_lasx_xvrotr_h, 61167}, // __builtin_lasx_xvrotr_h
    {loongarch_lasx_xvrotr_w, 61180}, // __builtin_lasx_xvrotr_w
    {loongarch_lasx_xvrotri_b, 61193}, // __builtin_lasx_xvrotri_b
    {loongarch_lasx_xvrotri_d, 61207}, // __builtin_lasx_xvrotri_d
    {loongarch_lasx_xvrotri_h, 61221}, // __builtin_lasx_xvrotri_h
    {loongarch_lasx_xvrotri_w, 61235}, // __builtin_lasx_xvrotri_w
    {loongarch_lasx_xvsadd_b, 61249}, // __builtin_lasx_xvsadd_b
    {loongarch_lasx_xvsadd_bu, 61262}, // __builtin_lasx_xvsadd_bu
    {loongarch_lasx_xvsadd_d, 61276}, // __builtin_lasx_xvsadd_d
    {loongarch_lasx_xvsadd_du, 61289}, // __builtin_lasx_xvsadd_du
    {loongarch_lasx_xvsadd_h, 61303}, // __builtin_lasx_xvsadd_h
    {loongarch_lasx_xvsadd_hu, 61316}, // __builtin_lasx_xvsadd_hu
    {loongarch_lasx_xvsadd_w, 61330}, // __builtin_lasx_xvsadd_w
    {loongarch_lasx_xvsadd_wu, 61343}, // __builtin_lasx_xvsadd_wu
    {loongarch_lasx_xvsat_b, 61357}, // __builtin_lasx_xvsat_b
    {loongarch_lasx_xvsat_bu, 61369}, // __builtin_lasx_xvsat_bu
    {loongarch_lasx_xvsat_d, 61382}, // __builtin_lasx_xvsat_d
    {loongarch_lasx_xvsat_du, 61394}, // __builtin_lasx_xvsat_du
    {loongarch_lasx_xvsat_h, 61407}, // __builtin_lasx_xvsat_h
    {loongarch_lasx_xvsat_hu, 61419}, // __builtin_lasx_xvsat_hu
    {loongarch_lasx_xvsat_w, 61432}, // __builtin_lasx_xvsat_w
    {loongarch_lasx_xvsat_wu, 61444}, // __builtin_lasx_xvsat_wu
    {loongarch_lasx_xvseq_b, 61457}, // __builtin_lasx_xvseq_b
    {loongarch_lasx_xvseq_d, 61469}, // __builtin_lasx_xvseq_d
    {loongarch_lasx_xvseq_h, 61481}, // __builtin_lasx_xvseq_h
    {loongarch_lasx_xvseq_w, 61493}, // __builtin_lasx_xvseq_w
    {loongarch_lasx_xvseqi_b, 61505}, // __builtin_lasx_xvseqi_b
    {loongarch_lasx_xvseqi_d, 61518}, // __builtin_lasx_xvseqi_d
    {loongarch_lasx_xvseqi_h, 61531}, // __builtin_lasx_xvseqi_h
    {loongarch_lasx_xvseqi_w, 61544}, // __builtin_lasx_xvseqi_w
    {loongarch_lasx_xvshuf4i_b, 61557}, // __builtin_lasx_xvshuf4i_b
    {loongarch_lasx_xvshuf4i_d, 61572}, // __builtin_lasx_xvshuf4i_d
    {loongarch_lasx_xvshuf4i_h, 61587}, // __builtin_lasx_xvshuf4i_h
    {loongarch_lasx_xvshuf4i_w, 61602}, // __builtin_lasx_xvshuf4i_w
    {loongarch_lasx_xvshuf_b, 61617}, // __builtin_lasx_xvshuf_b
    {loongarch_lasx_xvshuf_d, 61630}, // __builtin_lasx_xvshuf_d
    {loongarch_lasx_xvshuf_h, 61643}, // __builtin_lasx_xvshuf_h
    {loongarch_lasx_xvshuf_w, 61656}, // __builtin_lasx_xvshuf_w
    {loongarch_lasx_xvsigncov_b, 61669}, // __builtin_lasx_xvsigncov_b
    {loongarch_lasx_xvsigncov_d, 61685}, // __builtin_lasx_xvsigncov_d
    {loongarch_lasx_xvsigncov_h, 61701}, // __builtin_lasx_xvsigncov_h
    {loongarch_lasx_xvsigncov_w, 61717}, // __builtin_lasx_xvsigncov_w
    {loongarch_lasx_xvsle_b, 61733}, // __builtin_lasx_xvsle_b
    {loongarch_lasx_xvsle_bu, 61745}, // __builtin_lasx_xvsle_bu
    {loongarch_lasx_xvsle_d, 61758}, // __builtin_lasx_xvsle_d
    {loongarch_lasx_xvsle_du, 61770}, // __builtin_lasx_xvsle_du
    {loongarch_lasx_xvsle_h, 61783}, // __builtin_lasx_xvsle_h
    {loongarch_lasx_xvsle_hu, 61795}, // __builtin_lasx_xvsle_hu
    {loongarch_lasx_xvsle_w, 61808}, // __builtin_lasx_xvsle_w
    {loongarch_lasx_xvsle_wu, 61820}, // __builtin_lasx_xvsle_wu
    {loongarch_lasx_xvslei_b, 61833}, // __builtin_lasx_xvslei_b
    {loongarch_lasx_xvslei_bu, 61846}, // __builtin_lasx_xvslei_bu
    {loongarch_lasx_xvslei_d, 61860}, // __builtin_lasx_xvslei_d
    {loongarch_lasx_xvslei_du, 61873}, // __builtin_lasx_xvslei_du
    {loongarch_lasx_xvslei_h, 61887}, // __builtin_lasx_xvslei_h
    {loongarch_lasx_xvslei_hu, 61900}, // __builtin_lasx_xvslei_hu
    {loongarch_lasx_xvslei_w, 61914}, // __builtin_lasx_xvslei_w
    {loongarch_lasx_xvslei_wu, 61927}, // __builtin_lasx_xvslei_wu
    {loongarch_lasx_xvsll_b, 61941}, // __builtin_lasx_xvsll_b
    {loongarch_lasx_xvsll_d, 61953}, // __builtin_lasx_xvsll_d
    {loongarch_lasx_xvsll_h, 61965}, // __builtin_lasx_xvsll_h
    {loongarch_lasx_xvsll_w, 61977}, // __builtin_lasx_xvsll_w
    {loongarch_lasx_xvslli_b, 61989}, // __builtin_lasx_xvslli_b
    {loongarch_lasx_xvslli_d, 62002}, // __builtin_lasx_xvslli_d
    {loongarch_lasx_xvslli_h, 62015}, // __builtin_lasx_xvslli_h
    {loongarch_lasx_xvslli_w, 62028}, // __builtin_lasx_xvslli_w
    {loongarch_lasx_xvsllwil_d_w, 62041}, // __builtin_lasx_xvsllwil_d_w
    {loongarch_lasx_xvsllwil_du_wu, 62058}, // __builtin_lasx_xvsllwil_du_wu
    {loongarch_lasx_xvsllwil_h_b, 62077}, // __builtin_lasx_xvsllwil_h_b
    {loongarch_lasx_xvsllwil_hu_bu, 62094}, // __builtin_lasx_xvsllwil_hu_bu
    {loongarch_lasx_xvsllwil_w_h, 62113}, // __builtin_lasx_xvsllwil_w_h
    {loongarch_lasx_xvsllwil_wu_hu, 62130}, // __builtin_lasx_xvsllwil_wu_hu
    {loongarch_lasx_xvslt_b, 62149}, // __builtin_lasx_xvslt_b
    {loongarch_lasx_xvslt_bu, 62161}, // __builtin_lasx_xvslt_bu
    {loongarch_lasx_xvslt_d, 62174}, // __builtin_lasx_xvslt_d
    {loongarch_lasx_xvslt_du, 62186}, // __builtin_lasx_xvslt_du
    {loongarch_lasx_xvslt_h, 62199}, // __builtin_lasx_xvslt_h
    {loongarch_lasx_xvslt_hu, 62211}, // __builtin_lasx_xvslt_hu
    {loongarch_lasx_xvslt_w, 62224}, // __builtin_lasx_xvslt_w
    {loongarch_lasx_xvslt_wu, 62236}, // __builtin_lasx_xvslt_wu
    {loongarch_lasx_xvslti_b, 62249}, // __builtin_lasx_xvslti_b
    {loongarch_lasx_xvslti_bu, 62262}, // __builtin_lasx_xvslti_bu
    {loongarch_lasx_xvslti_d, 62276}, // __builtin_lasx_xvslti_d
    {loongarch_lasx_xvslti_du, 62289}, // __builtin_lasx_xvslti_du
    {loongarch_lasx_xvslti_h, 62303}, // __builtin_lasx_xvslti_h
    {loongarch_lasx_xvslti_hu, 62316}, // __builtin_lasx_xvslti_hu
    {loongarch_lasx_xvslti_w, 62330}, // __builtin_lasx_xvslti_w
    {loongarch_lasx_xvslti_wu, 62343}, // __builtin_lasx_xvslti_wu
    {loongarch_lasx_xvsra_b, 62357}, // __builtin_lasx_xvsra_b
    {loongarch_lasx_xvsra_d, 62369}, // __builtin_lasx_xvsra_d
    {loongarch_lasx_xvsra_h, 62381}, // __builtin_lasx_xvsra_h
    {loongarch_lasx_xvsra_w, 62393}, // __builtin_lasx_xvsra_w
    {loongarch_lasx_xvsrai_b, 62405}, // __builtin_lasx_xvsrai_b
    {loongarch_lasx_xvsrai_d, 62418}, // __builtin_lasx_xvsrai_d
    {loongarch_lasx_xvsrai_h, 62431}, // __builtin_lasx_xvsrai_h
    {loongarch_lasx_xvsrai_w, 62444}, // __builtin_lasx_xvsrai_w
    {loongarch_lasx_xvsran_b_h, 62457}, // __builtin_lasx_xvsran_b_h
    {loongarch_lasx_xvsran_h_w, 62472}, // __builtin_lasx_xvsran_h_w
    {loongarch_lasx_xvsran_w_d, 62487}, // __builtin_lasx_xvsran_w_d
    {loongarch_lasx_xvsrani_b_h, 62502}, // __builtin_lasx_xvsrani_b_h
    {loongarch_lasx_xvsrani_d_q, 62518}, // __builtin_lasx_xvsrani_d_q
    {loongarch_lasx_xvsrani_h_w, 62534}, // __builtin_lasx_xvsrani_h_w
    {loongarch_lasx_xvsrani_w_d, 62550}, // __builtin_lasx_xvsrani_w_d
    {loongarch_lasx_xvsrar_b, 62566}, // __builtin_lasx_xvsrar_b
    {loongarch_lasx_xvsrar_d, 62579}, // __builtin_lasx_xvsrar_d
    {loongarch_lasx_xvsrar_h, 62592}, // __builtin_lasx_xvsrar_h
    {loongarch_lasx_xvsrar_w, 62605}, // __builtin_lasx_xvsrar_w
    {loongarch_lasx_xvsrari_b, 62618}, // __builtin_lasx_xvsrari_b
    {loongarch_lasx_xvsrari_d, 62632}, // __builtin_lasx_xvsrari_d
    {loongarch_lasx_xvsrari_h, 62646}, // __builtin_lasx_xvsrari_h
    {loongarch_lasx_xvsrari_w, 62660}, // __builtin_lasx_xvsrari_w
    {loongarch_lasx_xvsrarn_b_h, 62674}, // __builtin_lasx_xvsrarn_b_h
    {loongarch_lasx_xvsrarn_h_w, 62690}, // __builtin_lasx_xvsrarn_h_w
    {loongarch_lasx_xvsrarn_w_d, 62706}, // __builtin_lasx_xvsrarn_w_d
    {loongarch_lasx_xvsrarni_b_h, 62722}, // __builtin_lasx_xvsrarni_b_h
    {loongarch_lasx_xvsrarni_d_q, 62739}, // __builtin_lasx_xvsrarni_d_q
    {loongarch_lasx_xvsrarni_h_w, 62756}, // __builtin_lasx_xvsrarni_h_w
    {loongarch_lasx_xvsrarni_w_d, 62773}, // __builtin_lasx_xvsrarni_w_d
    {loongarch_lasx_xvsrl_b, 62790}, // __builtin_lasx_xvsrl_b
    {loongarch_lasx_xvsrl_d, 62802}, // __builtin_lasx_xvsrl_d
    {loongarch_lasx_xvsrl_h, 62814}, // __builtin_lasx_xvsrl_h
    {loongarch_lasx_xvsrl_w, 62826}, // __builtin_lasx_xvsrl_w
    {loongarch_lasx_xvsrli_b, 62838}, // __builtin_lasx_xvsrli_b
    {loongarch_lasx_xvsrli_d, 62851}, // __builtin_lasx_xvsrli_d
    {loongarch_lasx_xvsrli_h, 62864}, // __builtin_lasx_xvsrli_h
    {loongarch_lasx_xvsrli_w, 62877}, // __builtin_lasx_xvsrli_w
    {loongarch_lasx_xvsrln_b_h, 62890}, // __builtin_lasx_xvsrln_b_h
    {loongarch_lasx_xvsrln_h_w, 62905}, // __builtin_lasx_xvsrln_h_w
    {loongarch_lasx_xvsrln_w_d, 62920}, // __builtin_lasx_xvsrln_w_d
    {loongarch_lasx_xvsrlni_b_h, 62935}, // __builtin_lasx_xvsrlni_b_h
    {loongarch_lasx_xvsrlni_d_q, 62951}, // __builtin_lasx_xvsrlni_d_q
    {loongarch_lasx_xvsrlni_h_w, 62967}, // __builtin_lasx_xvsrlni_h_w
    {loongarch_lasx_xvsrlni_w_d, 62983}, // __builtin_lasx_xvsrlni_w_d
    {loongarch_lasx_xvsrlr_b, 62999}, // __builtin_lasx_xvsrlr_b
    {loongarch_lasx_xvsrlr_d, 63012}, // __builtin_lasx_xvsrlr_d
    {loongarch_lasx_xvsrlr_h, 63025}, // __builtin_lasx_xvsrlr_h
    {loongarch_lasx_xvsrlr_w, 63038}, // __builtin_lasx_xvsrlr_w
    {loongarch_lasx_xvsrlri_b, 63051}, // __builtin_lasx_xvsrlri_b
    {loongarch_lasx_xvsrlri_d, 63065}, // __builtin_lasx_xvsrlri_d
    {loongarch_lasx_xvsrlri_h, 63079}, // __builtin_lasx_xvsrlri_h
    {loongarch_lasx_xvsrlri_w, 63093}, // __builtin_lasx_xvsrlri_w
    {loongarch_lasx_xvsrlrn_b_h, 63107}, // __builtin_lasx_xvsrlrn_b_h
    {loongarch_lasx_xvsrlrn_h_w, 63123}, // __builtin_lasx_xvsrlrn_h_w
    {loongarch_lasx_xvsrlrn_w_d, 63139}, // __builtin_lasx_xvsrlrn_w_d
    {loongarch_lasx_xvsrlrni_b_h, 63155}, // __builtin_lasx_xvsrlrni_b_h
    {loongarch_lasx_xvsrlrni_d_q, 63172}, // __builtin_lasx_xvsrlrni_d_q
    {loongarch_lasx_xvsrlrni_h_w, 63189}, // __builtin_lasx_xvsrlrni_h_w
    {loongarch_lasx_xvsrlrni_w_d, 63206}, // __builtin_lasx_xvsrlrni_w_d
    {loongarch_lasx_xvssran_b_h, 63223}, // __builtin_lasx_xvssran_b_h
    {loongarch_lasx_xvssran_bu_h, 63239}, // __builtin_lasx_xvssran_bu_h
    {loongarch_lasx_xvssran_h_w, 63256}, // __builtin_lasx_xvssran_h_w
    {loongarch_lasx_xvssran_hu_w, 63272}, // __builtin_lasx_xvssran_hu_w
    {loongarch_lasx_xvssran_w_d, 63289}, // __builtin_lasx_xvssran_w_d
    {loongarch_lasx_xvssran_wu_d, 63305}, // __builtin_lasx_xvssran_wu_d
    {loongarch_lasx_xvssrani_b_h, 63322}, // __builtin_lasx_xvssrani_b_h
    {loongarch_lasx_xvssrani_bu_h, 63339}, // __builtin_lasx_xvssrani_bu_h
    {loongarch_lasx_xvssrani_d_q, 63357}, // __builtin_lasx_xvssrani_d_q
    {loongarch_lasx_xvssrani_du_q, 63374}, // __builtin_lasx_xvssrani_du_q
    {loongarch_lasx_xvssrani_h_w, 63392}, // __builtin_lasx_xvssrani_h_w
    {loongarch_lasx_xvssrani_hu_w, 63409}, // __builtin_lasx_xvssrani_hu_w
    {loongarch_lasx_xvssrani_w_d, 63427}, // __builtin_lasx_xvssrani_w_d
    {loongarch_lasx_xvssrani_wu_d, 63444}, // __builtin_lasx_xvssrani_wu_d
    {loongarch_lasx_xvssrarn_b_h, 63462}, // __builtin_lasx_xvssrarn_b_h
    {loongarch_lasx_xvssrarn_bu_h, 63479}, // __builtin_lasx_xvssrarn_bu_h
    {loongarch_lasx_xvssrarn_h_w, 63497}, // __builtin_lasx_xvssrarn_h_w
    {loongarch_lasx_xvssrarn_hu_w, 63514}, // __builtin_lasx_xvssrarn_hu_w
    {loongarch_lasx_xvssrarn_w_d, 63532}, // __builtin_lasx_xvssrarn_w_d
    {loongarch_lasx_xvssrarn_wu_d, 63549}, // __builtin_lasx_xvssrarn_wu_d
    {loongarch_lasx_xvssrarni_b_h, 63567}, // __builtin_lasx_xvssrarni_b_h
    {loongarch_lasx_xvssrarni_bu_h, 63585}, // __builtin_lasx_xvssrarni_bu_h
    {loongarch_lasx_xvssrarni_d_q, 63604}, // __builtin_lasx_xvssrarni_d_q
    {loongarch_lasx_xvssrarni_du_q, 63622}, // __builtin_lasx_xvssrarni_du_q
    {loongarch_lasx_xvssrarni_h_w, 63641}, // __builtin_lasx_xvssrarni_h_w
    {loongarch_lasx_xvssrarni_hu_w, 63659}, // __builtin_lasx_xvssrarni_hu_w
    {loongarch_lasx_xvssrarni_w_d, 63678}, // __builtin_lasx_xvssrarni_w_d
    {loongarch_lasx_xvssrarni_wu_d, 63696}, // __builtin_lasx_xvssrarni_wu_d
    {loongarch_lasx_xvssrln_b_h, 63715}, // __builtin_lasx_xvssrln_b_h
    {loongarch_lasx_xvssrln_bu_h, 63731}, // __builtin_lasx_xvssrln_bu_h
    {loongarch_lasx_xvssrln_h_w, 63748}, // __builtin_lasx_xvssrln_h_w
    {loongarch_lasx_xvssrln_hu_w, 63764}, // __builtin_lasx_xvssrln_hu_w
    {loongarch_lasx_xvssrln_w_d, 63781}, // __builtin_lasx_xvssrln_w_d
    {loongarch_lasx_xvssrln_wu_d, 63797}, // __builtin_lasx_xvssrln_wu_d
    {loongarch_lasx_xvssrlni_b_h, 63814}, // __builtin_lasx_xvssrlni_b_h
    {loongarch_lasx_xvssrlni_bu_h, 63831}, // __builtin_lasx_xvssrlni_bu_h
    {loongarch_lasx_xvssrlni_d_q, 63849}, // __builtin_lasx_xvssrlni_d_q
    {loongarch_lasx_xvssrlni_du_q, 63866}, // __builtin_lasx_xvssrlni_du_q
    {loongarch_lasx_xvssrlni_h_w, 63884}, // __builtin_lasx_xvssrlni_h_w
    {loongarch_lasx_xvssrlni_hu_w, 63901}, // __builtin_lasx_xvssrlni_hu_w
    {loongarch_lasx_xvssrlni_w_d, 63919}, // __builtin_lasx_xvssrlni_w_d
    {loongarch_lasx_xvssrlni_wu_d, 63936}, // __builtin_lasx_xvssrlni_wu_d
    {loongarch_lasx_xvssrlrn_b_h, 63954}, // __builtin_lasx_xvssrlrn_b_h
    {loongarch_lasx_xvssrlrn_bu_h, 63971}, // __builtin_lasx_xvssrlrn_bu_h
    {loongarch_lasx_xvssrlrn_h_w, 63989}, // __builtin_lasx_xvssrlrn_h_w
    {loongarch_lasx_xvssrlrn_hu_w, 64006}, // __builtin_lasx_xvssrlrn_hu_w
    {loongarch_lasx_xvssrlrn_w_d, 64024}, // __builtin_lasx_xvssrlrn_w_d
    {loongarch_lasx_xvssrlrn_wu_d, 64041}, // __builtin_lasx_xvssrlrn_wu_d
    {loongarch_lasx_xvssrlrni_b_h, 64059}, // __builtin_lasx_xvssrlrni_b_h
    {loongarch_lasx_xvssrlrni_bu_h, 64077}, // __builtin_lasx_xvssrlrni_bu_h
    {loongarch_lasx_xvssrlrni_d_q, 64096}, // __builtin_lasx_xvssrlrni_d_q
    {loongarch_lasx_xvssrlrni_du_q, 64114}, // __builtin_lasx_xvssrlrni_du_q
    {loongarch_lasx_xvssrlrni_h_w, 64133}, // __builtin_lasx_xvssrlrni_h_w
    {loongarch_lasx_xvssrlrni_hu_w, 64151}, // __builtin_lasx_xvssrlrni_hu_w
    {loongarch_lasx_xvssrlrni_w_d, 64170}, // __builtin_lasx_xvssrlrni_w_d
    {loongarch_lasx_xvssrlrni_wu_d, 64188}, // __builtin_lasx_xvssrlrni_wu_d
    {loongarch_lasx_xvssub_b, 64207}, // __builtin_lasx_xvssub_b
    {loongarch_lasx_xvssub_bu, 64220}, // __builtin_lasx_xvssub_bu
    {loongarch_lasx_xvssub_d, 64234}, // __builtin_lasx_xvssub_d
    {loongarch_lasx_xvssub_du, 64247}, // __builtin_lasx_xvssub_du
    {loongarch_lasx_xvssub_h, 64261}, // __builtin_lasx_xvssub_h
    {loongarch_lasx_xvssub_hu, 64274}, // __builtin_lasx_xvssub_hu
    {loongarch_lasx_xvssub_w, 64288}, // __builtin_lasx_xvssub_w
    {loongarch_lasx_xvssub_wu, 64301}, // __builtin_lasx_xvssub_wu
    {loongarch_lasx_xvst, 64315}, // __builtin_lasx_xvst
    {loongarch_lasx_xvstelm_b, 64324}, // __builtin_lasx_xvstelm_b
    {loongarch_lasx_xvstelm_d, 64338}, // __builtin_lasx_xvstelm_d
    {loongarch_lasx_xvstelm_h, 64352}, // __builtin_lasx_xvstelm_h
    {loongarch_lasx_xvstelm_w, 64366}, // __builtin_lasx_xvstelm_w
    {loongarch_lasx_xvstx, 64380}, // __builtin_lasx_xvstx
    {loongarch_lasx_xvsub_b, 64390}, // __builtin_lasx_xvsub_b
    {loongarch_lasx_xvsub_d, 64402}, // __builtin_lasx_xvsub_d
    {loongarch_lasx_xvsub_h, 64414}, // __builtin_lasx_xvsub_h
    {loongarch_lasx_xvsub_q, 64426}, // __builtin_lasx_xvsub_q
    {loongarch_lasx_xvsub_w, 64438}, // __builtin_lasx_xvsub_w
    {loongarch_lasx_xvsubi_bu, 64450}, // __builtin_lasx_xvsubi_bu
    {loongarch_lasx_xvsubi_du, 64464}, // __builtin_lasx_xvsubi_du
    {loongarch_lasx_xvsubi_hu, 64478}, // __builtin_lasx_xvsubi_hu
    {loongarch_lasx_xvsubi_wu, 64492}, // __builtin_lasx_xvsubi_wu
    {loongarch_lasx_xvsubwev_d_w, 64506}, // __builtin_lasx_xvsubwev_d_w
    {loongarch_lasx_xvsubwev_d_wu, 64523}, // __builtin_lasx_xvsubwev_d_wu
    {loongarch_lasx_xvsubwev_h_b, 64541}, // __builtin_lasx_xvsubwev_h_b
    {loongarch_lasx_xvsubwev_h_bu, 64558}, // __builtin_lasx_xvsubwev_h_bu
    {loongarch_lasx_xvsubwev_q_d, 64576}, // __builtin_lasx_xvsubwev_q_d
    {loongarch_lasx_xvsubwev_q_du, 64593}, // __builtin_lasx_xvsubwev_q_du
    {loongarch_lasx_xvsubwev_w_h, 64611}, // __builtin_lasx_xvsubwev_w_h
    {loongarch_lasx_xvsubwev_w_hu, 64628}, // __builtin_lasx_xvsubwev_w_hu
    {loongarch_lasx_xvsubwod_d_w, 64646}, // __builtin_lasx_xvsubwod_d_w
    {loongarch_lasx_xvsubwod_d_wu, 64663}, // __builtin_lasx_xvsubwod_d_wu
    {loongarch_lasx_xvsubwod_h_b, 64681}, // __builtin_lasx_xvsubwod_h_b
    {loongarch_lasx_xvsubwod_h_bu, 64698}, // __builtin_lasx_xvsubwod_h_bu
    {loongarch_lasx_xvsubwod_q_d, 64716}, // __builtin_lasx_xvsubwod_q_d
    {loongarch_lasx_xvsubwod_q_du, 64733}, // __builtin_lasx_xvsubwod_q_du
    {loongarch_lasx_xvsubwod_w_h, 64751}, // __builtin_lasx_xvsubwod_w_h
    {loongarch_lasx_xvsubwod_w_hu, 64768}, // __builtin_lasx_xvsubwod_w_hu
    {loongarch_lasx_xvxor_v, 64786}, // __builtin_lasx_xvxor_v
    {loongarch_lasx_xvxori_b, 64798}, // __builtin_lasx_xvxori_b
    {loongarch_asrtgt_d, 64811}, // __builtin_loongarch_asrtgt_d
    {loongarch_asrtle_d, 64829}, // __builtin_loongarch_asrtle_d
    {loongarch_break, 64847}, // __builtin_loongarch_break
    {loongarch_cacop_d, 64862}, // __builtin_loongarch_cacop_d
    {loongarch_cacop_w, 64879}, // __builtin_loongarch_cacop_w
    {loongarch_cpucfg, 64896}, // __builtin_loongarch_cpucfg
    {loongarch_crc_w_b_w, 64912}, // __builtin_loongarch_crc_w_b_w
    {loongarch_crc_w_d_w, 64931}, // __builtin_loongarch_crc_w_d_w
    {loongarch_crc_w_h_w, 64950}, // __builtin_loongarch_crc_w_h_w
    {loongarch_crc_w_w_w, 64969}, // __builtin_loongarch_crc_w_w_w
    {loongarch_crcc_w_b_w, 64988}, // __builtin_loongarch_crcc_w_b_w
    {loongarch_crcc_w_d_w, 65008}, // __builtin_loongarch_crcc_w_d_w
    {loongarch_crcc_w_h_w, 65028}, // __builtin_loongarch_crcc_w_h_w
    {loongarch_crcc_w_w_w, 65048}, // __builtin_loongarch_crcc_w_w_w
    {loongarch_csrrd_d, 65068}, // __builtin_loongarch_csrrd_d
    {loongarch_csrrd_w, 65085}, // __builtin_loongarch_csrrd_w
    {loongarch_csrwr_d, 65102}, // __builtin_loongarch_csrwr_d
    {loongarch_csrwr_w, 65119}, // __builtin_loongarch_csrwr_w
    {loongarch_csrxchg_d, 65136}, // __builtin_loongarch_csrxchg_d
    {loongarch_csrxchg_w, 65155}, // __builtin_loongarch_csrxchg_w
    {loongarch_dbar, 65174}, // __builtin_loongarch_dbar
    {loongarch_frecipe_d, 65188}, // __builtin_loongarch_frecipe_d
    {loongarch_frecipe_s, 65207}, // __builtin_loongarch_frecipe_s
    {loongarch_frsqrte_d, 65226}, // __builtin_loongarch_frsqrte_d
    {loongarch_frsqrte_s, 65245}, // __builtin_loongarch_frsqrte_s
    {loongarch_ibar, 65264}, // __builtin_loongarch_ibar
    {loongarch_iocsrrd_b, 65278}, // __builtin_loongarch_iocsrrd_b
    {loongarch_iocsrrd_d, 65297}, // __builtin_loongarch_iocsrrd_d
    {loongarch_iocsrrd_h, 65316}, // __builtin_loongarch_iocsrrd_h
    {loongarch_iocsrrd_w, 65335}, // __builtin_loongarch_iocsrrd_w
    {loongarch_iocsrwr_b, 65354}, // __builtin_loongarch_iocsrwr_b
    {loongarch_iocsrwr_d, 65373}, // __builtin_loongarch_iocsrwr_d
    {loongarch_iocsrwr_h, 65392}, // __builtin_loongarch_iocsrwr_h
    {loongarch_iocsrwr_w, 65411}, // __builtin_loongarch_iocsrwr_w
    {loongarch_lddir_d, 65430}, // __builtin_loongarch_lddir_d
    {loongarch_ldpte_d, 65447}, // __builtin_loongarch_ldpte_d
    {loongarch_movfcsr2gr, 65464}, // __builtin_loongarch_movfcsr2gr
    {loongarch_movgr2fcsr, 65484}, // __builtin_loongarch_movgr2fcsr
    {loongarch_syscall, 65504}, // __builtin_loongarch_syscall
    {loongarch_lsx_bnz_b, 65521}, // __builtin_lsx_bnz_b
    {loongarch_lsx_bnz_d, 65530}, // __builtin_lsx_bnz_d
    {loongarch_lsx_bnz_h, 65539}, // __builtin_lsx_bnz_h
    {loongarch_lsx_bnz_v, 65548}, // __builtin_lsx_bnz_v
    {loongarch_lsx_bnz_w, 65557}, // __builtin_lsx_bnz_w
    {loongarch_lsx_bz_b, 65566}, // __builtin_lsx_bz_b
    {loongarch_lsx_bz_d, 65574}, // __builtin_lsx_bz_d
    {loongarch_lsx_bz_h, 65582}, // __builtin_lsx_bz_h
    {loongarch_lsx_bz_v, 65590}, // __builtin_lsx_bz_v
    {loongarch_lsx_bz_w, 65598}, // __builtin_lsx_bz_w
    {loongarch_lsx_vabsd_b, 65606}, // __builtin_lsx_vabsd_b
    {loongarch_lsx_vabsd_bu, 65617}, // __builtin_lsx_vabsd_bu
    {loongarch_lsx_vabsd_d, 65629}, // __builtin_lsx_vabsd_d
    {loongarch_lsx_vabsd_du, 65640}, // __builtin_lsx_vabsd_du
    {loongarch_lsx_vabsd_h, 65652}, // __builtin_lsx_vabsd_h
    {loongarch_lsx_vabsd_hu, 65663}, // __builtin_lsx_vabsd_hu
    {loongarch_lsx_vabsd_w, 65675}, // __builtin_lsx_vabsd_w
    {loongarch_lsx_vabsd_wu, 65686}, // __builtin_lsx_vabsd_wu
    {loongarch_lsx_vadd_b, 65698}, // __builtin_lsx_vadd_b
    {loongarch_lsx_vadd_d, 65708}, // __builtin_lsx_vadd_d
    {loongarch_lsx_vadd_h, 65718}, // __builtin_lsx_vadd_h
    {loongarch_lsx_vadd_q, 65728}, // __builtin_lsx_vadd_q
    {loongarch_lsx_vadd_w, 65738}, // __builtin_lsx_vadd_w
    {loongarch_lsx_vadda_b, 65748}, // __builtin_lsx_vadda_b
    {loongarch_lsx_vadda_d, 65759}, // __builtin_lsx_vadda_d
    {loongarch_lsx_vadda_h, 65770}, // __builtin_lsx_vadda_h
    {loongarch_lsx_vadda_w, 65781}, // __builtin_lsx_vadda_w
    {loongarch_lsx_vaddi_bu, 65792}, // __builtin_lsx_vaddi_bu
    {loongarch_lsx_vaddi_du, 65804}, // __builtin_lsx_vaddi_du
    {loongarch_lsx_vaddi_hu, 65816}, // __builtin_lsx_vaddi_hu
    {loongarch_lsx_vaddi_wu, 65828}, // __builtin_lsx_vaddi_wu
    {loongarch_lsx_vaddwev_d_w, 65840}, // __builtin_lsx_vaddwev_d_w
    {loongarch_lsx_vaddwev_d_wu, 65855}, // __builtin_lsx_vaddwev_d_wu
    {loongarch_lsx_vaddwev_d_wu_w, 65871}, // __builtin_lsx_vaddwev_d_wu_w
    {loongarch_lsx_vaddwev_h_b, 65889}, // __builtin_lsx_vaddwev_h_b
    {loongarch_lsx_vaddwev_h_bu, 65904}, // __builtin_lsx_vaddwev_h_bu
    {loongarch_lsx_vaddwev_h_bu_b, 65920}, // __builtin_lsx_vaddwev_h_bu_b
    {loongarch_lsx_vaddwev_q_d, 65938}, // __builtin_lsx_vaddwev_q_d
    {loongarch_lsx_vaddwev_q_du, 65953}, // __builtin_lsx_vaddwev_q_du
    {loongarch_lsx_vaddwev_q_du_d, 65969}, // __builtin_lsx_vaddwev_q_du_d
    {loongarch_lsx_vaddwev_w_h, 65987}, // __builtin_lsx_vaddwev_w_h
    {loongarch_lsx_vaddwev_w_hu, 66002}, // __builtin_lsx_vaddwev_w_hu
    {loongarch_lsx_vaddwev_w_hu_h, 66018}, // __builtin_lsx_vaddwev_w_hu_h
    {loongarch_lsx_vaddwod_d_w, 66036}, // __builtin_lsx_vaddwod_d_w
    {loongarch_lsx_vaddwod_d_wu, 66051}, // __builtin_lsx_vaddwod_d_wu
    {loongarch_lsx_vaddwod_d_wu_w, 66067}, // __builtin_lsx_vaddwod_d_wu_w
    {loongarch_lsx_vaddwod_h_b, 66085}, // __builtin_lsx_vaddwod_h_b
    {loongarch_lsx_vaddwod_h_bu, 66100}, // __builtin_lsx_vaddwod_h_bu
    {loongarch_lsx_vaddwod_h_bu_b, 66116}, // __builtin_lsx_vaddwod_h_bu_b
    {loongarch_lsx_vaddwod_q_d, 66134}, // __builtin_lsx_vaddwod_q_d
    {loongarch_lsx_vaddwod_q_du, 66149}, // __builtin_lsx_vaddwod_q_du
    {loongarch_lsx_vaddwod_q_du_d, 66165}, // __builtin_lsx_vaddwod_q_du_d
    {loongarch_lsx_vaddwod_w_h, 66183}, // __builtin_lsx_vaddwod_w_h
    {loongarch_lsx_vaddwod_w_hu, 66198}, // __builtin_lsx_vaddwod_w_hu
    {loongarch_lsx_vaddwod_w_hu_h, 66214}, // __builtin_lsx_vaddwod_w_hu_h
    {loongarch_lsx_vand_v, 66232}, // __builtin_lsx_vand_v
    {loongarch_lsx_vandi_b, 66242}, // __builtin_lsx_vandi_b
    {loongarch_lsx_vandn_v, 66253}, // __builtin_lsx_vandn_v
    {loongarch_lsx_vavg_b, 66264}, // __builtin_lsx_vavg_b
    {loongarch_lsx_vavg_bu, 66274}, // __builtin_lsx_vavg_bu
    {loongarch_lsx_vavg_d, 66285}, // __builtin_lsx_vavg_d
    {loongarch_lsx_vavg_du, 66295}, // __builtin_lsx_vavg_du
    {loongarch_lsx_vavg_h, 66306}, // __builtin_lsx_vavg_h
    {loongarch_lsx_vavg_hu, 66316}, // __builtin_lsx_vavg_hu
    {loongarch_lsx_vavg_w, 66327}, // __builtin_lsx_vavg_w
    {loongarch_lsx_vavg_wu, 66337}, // __builtin_lsx_vavg_wu
    {loongarch_lsx_vavgr_b, 66348}, // __builtin_lsx_vavgr_b
    {loongarch_lsx_vavgr_bu, 66359}, // __builtin_lsx_vavgr_bu
    {loongarch_lsx_vavgr_d, 66371}, // __builtin_lsx_vavgr_d
    {loongarch_lsx_vavgr_du, 66382}, // __builtin_lsx_vavgr_du
    {loongarch_lsx_vavgr_h, 66394}, // __builtin_lsx_vavgr_h
    {loongarch_lsx_vavgr_hu, 66405}, // __builtin_lsx_vavgr_hu
    {loongarch_lsx_vavgr_w, 66417}, // __builtin_lsx_vavgr_w
    {loongarch_lsx_vavgr_wu, 66428}, // __builtin_lsx_vavgr_wu
    {loongarch_lsx_vbitclr_b, 66440}, // __builtin_lsx_vbitclr_b
    {loongarch_lsx_vbitclr_d, 66453}, // __builtin_lsx_vbitclr_d
    {loongarch_lsx_vbitclr_h, 66466}, // __builtin_lsx_vbitclr_h
    {loongarch_lsx_vbitclr_w, 66479}, // __builtin_lsx_vbitclr_w
    {loongarch_lsx_vbitclri_b, 66492}, // __builtin_lsx_vbitclri_b
    {loongarch_lsx_vbitclri_d, 66506}, // __builtin_lsx_vbitclri_d
    {loongarch_lsx_vbitclri_h, 66520}, // __builtin_lsx_vbitclri_h
    {loongarch_lsx_vbitclri_w, 66534}, // __builtin_lsx_vbitclri_w
    {loongarch_lsx_vbitrev_b, 66548}, // __builtin_lsx_vbitrev_b
    {loongarch_lsx_vbitrev_d, 66561}, // __builtin_lsx_vbitrev_d
    {loongarch_lsx_vbitrev_h, 66574}, // __builtin_lsx_vbitrev_h
    {loongarch_lsx_vbitrev_w, 66587}, // __builtin_lsx_vbitrev_w
    {loongarch_lsx_vbitrevi_b, 66600}, // __builtin_lsx_vbitrevi_b
    {loongarch_lsx_vbitrevi_d, 66614}, // __builtin_lsx_vbitrevi_d
    {loongarch_lsx_vbitrevi_h, 66628}, // __builtin_lsx_vbitrevi_h
    {loongarch_lsx_vbitrevi_w, 66642}, // __builtin_lsx_vbitrevi_w
    {loongarch_lsx_vbitsel_v, 66656}, // __builtin_lsx_vbitsel_v
    {loongarch_lsx_vbitseli_b, 66669}, // __builtin_lsx_vbitseli_b
    {loongarch_lsx_vbitset_b, 66683}, // __builtin_lsx_vbitset_b
    {loongarch_lsx_vbitset_d, 66696}, // __builtin_lsx_vbitset_d
    {loongarch_lsx_vbitset_h, 66709}, // __builtin_lsx_vbitset_h
    {loongarch_lsx_vbitset_w, 66722}, // __builtin_lsx_vbitset_w
    {loongarch_lsx_vbitseti_b, 66735}, // __builtin_lsx_vbitseti_b
    {loongarch_lsx_vbitseti_d, 66749}, // __builtin_lsx_vbitseti_d
    {loongarch_lsx_vbitseti_h, 66763}, // __builtin_lsx_vbitseti_h
    {loongarch_lsx_vbitseti_w, 66777}, // __builtin_lsx_vbitseti_w
    {loongarch_lsx_vbsll_v, 66791}, // __builtin_lsx_vbsll_v
    {loongarch_lsx_vbsrl_v, 66802}, // __builtin_lsx_vbsrl_v
    {loongarch_lsx_vclo_b, 66813}, // __builtin_lsx_vclo_b
    {loongarch_lsx_vclo_d, 66823}, // __builtin_lsx_vclo_d
    {loongarch_lsx_vclo_h, 66833}, // __builtin_lsx_vclo_h
    {loongarch_lsx_vclo_w, 66843}, // __builtin_lsx_vclo_w
    {loongarch_lsx_vclz_b, 66853}, // __builtin_lsx_vclz_b
    {loongarch_lsx_vclz_d, 66863}, // __builtin_lsx_vclz_d
    {loongarch_lsx_vclz_h, 66873}, // __builtin_lsx_vclz_h
    {loongarch_lsx_vclz_w, 66883}, // __builtin_lsx_vclz_w
    {loongarch_lsx_vdiv_b, 66893}, // __builtin_lsx_vdiv_b
    {loongarch_lsx_vdiv_bu, 66903}, // __builtin_lsx_vdiv_bu
    {loongarch_lsx_vdiv_d, 66914}, // __builtin_lsx_vdiv_d
    {loongarch_lsx_vdiv_du, 66924}, // __builtin_lsx_vdiv_du
    {loongarch_lsx_vdiv_h, 66935}, // __builtin_lsx_vdiv_h
    {loongarch_lsx_vdiv_hu, 66945}, // __builtin_lsx_vdiv_hu
    {loongarch_lsx_vdiv_w, 66956}, // __builtin_lsx_vdiv_w
    {loongarch_lsx_vdiv_wu, 66966}, // __builtin_lsx_vdiv_wu
    {loongarch_lsx_vexth_d_w, 66977}, // __builtin_lsx_vexth_d_w
    {loongarch_lsx_vexth_du_wu, 66990}, // __builtin_lsx_vexth_du_wu
    {loongarch_lsx_vexth_h_b, 67005}, // __builtin_lsx_vexth_h_b
    {loongarch_lsx_vexth_hu_bu, 67018}, // __builtin_lsx_vexth_hu_bu
    {loongarch_lsx_vexth_q_d, 67033}, // __builtin_lsx_vexth_q_d
    {loongarch_lsx_vexth_qu_du, 67046}, // __builtin_lsx_vexth_qu_du
    {loongarch_lsx_vexth_w_h, 67061}, // __builtin_lsx_vexth_w_h
    {loongarch_lsx_vexth_wu_hu, 67074}, // __builtin_lsx_vexth_wu_hu
    {loongarch_lsx_vextl_q_d, 67089}, // __builtin_lsx_vextl_q_d
    {loongarch_lsx_vextl_qu_du, 67102}, // __builtin_lsx_vextl_qu_du
    {loongarch_lsx_vextrins_b, 67117}, // __builtin_lsx_vextrins_b
    {loongarch_lsx_vextrins_d, 67131}, // __builtin_lsx_vextrins_d
    {loongarch_lsx_vextrins_h, 67145}, // __builtin_lsx_vextrins_h
    {loongarch_lsx_vextrins_w, 67159}, // __builtin_lsx_vextrins_w
    {loongarch_lsx_vfadd_d, 67173}, // __builtin_lsx_vfadd_d
    {loongarch_lsx_vfadd_s, 67184}, // __builtin_lsx_vfadd_s
    {loongarch_lsx_vfclass_d, 67195}, // __builtin_lsx_vfclass_d
    {loongarch_lsx_vfclass_s, 67208}, // __builtin_lsx_vfclass_s
    {loongarch_lsx_vfcmp_caf_d, 67221}, // __builtin_lsx_vfcmp_caf_d
    {loongarch_lsx_vfcmp_caf_s, 67236}, // __builtin_lsx_vfcmp_caf_s
    {loongarch_lsx_vfcmp_ceq_d, 67251}, // __builtin_lsx_vfcmp_ceq_d
    {loongarch_lsx_vfcmp_ceq_s, 67266}, // __builtin_lsx_vfcmp_ceq_s
    {loongarch_lsx_vfcmp_cle_d, 67281}, // __builtin_lsx_vfcmp_cle_d
    {loongarch_lsx_vfcmp_cle_s, 67296}, // __builtin_lsx_vfcmp_cle_s
    {loongarch_lsx_vfcmp_clt_d, 67311}, // __builtin_lsx_vfcmp_clt_d
    {loongarch_lsx_vfcmp_clt_s, 67326}, // __builtin_lsx_vfcmp_clt_s
    {loongarch_lsx_vfcmp_cne_d, 67341}, // __builtin_lsx_vfcmp_cne_d
    {loongarch_lsx_vfcmp_cne_s, 67356}, // __builtin_lsx_vfcmp_cne_s
    {loongarch_lsx_vfcmp_cor_d, 67371}, // __builtin_lsx_vfcmp_cor_d
    {loongarch_lsx_vfcmp_cor_s, 67386}, // __builtin_lsx_vfcmp_cor_s
    {loongarch_lsx_vfcmp_cueq_d, 67401}, // __builtin_lsx_vfcmp_cueq_d
    {loongarch_lsx_vfcmp_cueq_s, 67417}, // __builtin_lsx_vfcmp_cueq_s
    {loongarch_lsx_vfcmp_cule_d, 67433}, // __builtin_lsx_vfcmp_cule_d
    {loongarch_lsx_vfcmp_cule_s, 67449}, // __builtin_lsx_vfcmp_cule_s
    {loongarch_lsx_vfcmp_cult_d, 67465}, // __builtin_lsx_vfcmp_cult_d
    {loongarch_lsx_vfcmp_cult_s, 67481}, // __builtin_lsx_vfcmp_cult_s
    {loongarch_lsx_vfcmp_cun_d, 67497}, // __builtin_lsx_vfcmp_cun_d
    {loongarch_lsx_vfcmp_cun_s, 67512}, // __builtin_lsx_vfcmp_cun_s
    {loongarch_lsx_vfcmp_cune_d, 67527}, // __builtin_lsx_vfcmp_cune_d
    {loongarch_lsx_vfcmp_cune_s, 67543}, // __builtin_lsx_vfcmp_cune_s
    {loongarch_lsx_vfcmp_saf_d, 67559}, // __builtin_lsx_vfcmp_saf_d
    {loongarch_lsx_vfcmp_saf_s, 67574}, // __builtin_lsx_vfcmp_saf_s
    {loongarch_lsx_vfcmp_seq_d, 67589}, // __builtin_lsx_vfcmp_seq_d
    {loongarch_lsx_vfcmp_seq_s, 67604}, // __builtin_lsx_vfcmp_seq_s
    {loongarch_lsx_vfcmp_sle_d, 67619}, // __builtin_lsx_vfcmp_sle_d
    {loongarch_lsx_vfcmp_sle_s, 67634}, // __builtin_lsx_vfcmp_sle_s
    {loongarch_lsx_vfcmp_slt_d, 67649}, // __builtin_lsx_vfcmp_slt_d
    {loongarch_lsx_vfcmp_slt_s, 67664}, // __builtin_lsx_vfcmp_slt_s
    {loongarch_lsx_vfcmp_sne_d, 67679}, // __builtin_lsx_vfcmp_sne_d
    {loongarch_lsx_vfcmp_sne_s, 67694}, // __builtin_lsx_vfcmp_sne_s
    {loongarch_lsx_vfcmp_sor_d, 67709}, // __builtin_lsx_vfcmp_sor_d
    {loongarch_lsx_vfcmp_sor_s, 67724}, // __builtin_lsx_vfcmp_sor_s
    {loongarch_lsx_vfcmp_sueq_d, 67739}, // __builtin_lsx_vfcmp_sueq_d
    {loongarch_lsx_vfcmp_sueq_s, 67755}, // __builtin_lsx_vfcmp_sueq_s
    {loongarch_lsx_vfcmp_sule_d, 67771}, // __builtin_lsx_vfcmp_sule_d
    {loongarch_lsx_vfcmp_sule_s, 67787}, // __builtin_lsx_vfcmp_sule_s
    {loongarch_lsx_vfcmp_sult_d, 67803}, // __builtin_lsx_vfcmp_sult_d
    {loongarch_lsx_vfcmp_sult_s, 67819}, // __builtin_lsx_vfcmp_sult_s
    {loongarch_lsx_vfcmp_sun_d, 67835}, // __builtin_lsx_vfcmp_sun_d
    {loongarch_lsx_vfcmp_sun_s, 67850}, // __builtin_lsx_vfcmp_sun_s
    {loongarch_lsx_vfcmp_sune_d, 67865}, // __builtin_lsx_vfcmp_sune_d
    {loongarch_lsx_vfcmp_sune_s, 67881}, // __builtin_lsx_vfcmp_sune_s
    {loongarch_lsx_vfcvt_h_s, 67897}, // __builtin_lsx_vfcvt_h_s
    {loongarch_lsx_vfcvt_s_d, 67910}, // __builtin_lsx_vfcvt_s_d
    {loongarch_lsx_vfcvth_d_s, 67923}, // __builtin_lsx_vfcvth_d_s
    {loongarch_lsx_vfcvth_s_h, 67937}, // __builtin_lsx_vfcvth_s_h
    {loongarch_lsx_vfcvtl_d_s, 67951}, // __builtin_lsx_vfcvtl_d_s
    {loongarch_lsx_vfcvtl_s_h, 67965}, // __builtin_lsx_vfcvtl_s_h
    {loongarch_lsx_vfdiv_d, 67979}, // __builtin_lsx_vfdiv_d
    {loongarch_lsx_vfdiv_s, 67990}, // __builtin_lsx_vfdiv_s
    {loongarch_lsx_vffint_d_l, 68001}, // __builtin_lsx_vffint_d_l
    {loongarch_lsx_vffint_d_lu, 68015}, // __builtin_lsx_vffint_d_lu
    {loongarch_lsx_vffint_s_l, 68030}, // __builtin_lsx_vffint_s_l
    {loongarch_lsx_vffint_s_w, 68044}, // __builtin_lsx_vffint_s_w
    {loongarch_lsx_vffint_s_wu, 68058}, // __builtin_lsx_vffint_s_wu
    {loongarch_lsx_vffinth_d_w, 68073}, // __builtin_lsx_vffinth_d_w
    {loongarch_lsx_vffintl_d_w, 68088}, // __builtin_lsx_vffintl_d_w
    {loongarch_lsx_vflogb_d, 68103}, // __builtin_lsx_vflogb_d
    {loongarch_lsx_vflogb_s, 68115}, // __builtin_lsx_vflogb_s
    {loongarch_lsx_vfmadd_d, 68127}, // __builtin_lsx_vfmadd_d
    {loongarch_lsx_vfmadd_s, 68139}, // __builtin_lsx_vfmadd_s
    {loongarch_lsx_vfmax_d, 68151}, // __builtin_lsx_vfmax_d
    {loongarch_lsx_vfmax_s, 68162}, // __builtin_lsx_vfmax_s
    {loongarch_lsx_vfmaxa_d, 68173}, // __builtin_lsx_vfmaxa_d
    {loongarch_lsx_vfmaxa_s, 68185}, // __builtin_lsx_vfmaxa_s
    {loongarch_lsx_vfmin_d, 68197}, // __builtin_lsx_vfmin_d
    {loongarch_lsx_vfmin_s, 68208}, // __builtin_lsx_vfmin_s
    {loongarch_lsx_vfmina_d, 68219}, // __builtin_lsx_vfmina_d
    {loongarch_lsx_vfmina_s, 68231}, // __builtin_lsx_vfmina_s
    {loongarch_lsx_vfmsub_d, 68243}, // __builtin_lsx_vfmsub_d
    {loongarch_lsx_vfmsub_s, 68255}, // __builtin_lsx_vfmsub_s
    {loongarch_lsx_vfmul_d, 68267}, // __builtin_lsx_vfmul_d
    {loongarch_lsx_vfmul_s, 68278}, // __builtin_lsx_vfmul_s
    {loongarch_lsx_vfnmadd_d, 68289}, // __builtin_lsx_vfnmadd_d
    {loongarch_lsx_vfnmadd_s, 68302}, // __builtin_lsx_vfnmadd_s
    {loongarch_lsx_vfnmsub_d, 68315}, // __builtin_lsx_vfnmsub_d
    {loongarch_lsx_vfnmsub_s, 68328}, // __builtin_lsx_vfnmsub_s
    {loongarch_lsx_vfrecip_d, 68341}, // __builtin_lsx_vfrecip_d
    {loongarch_lsx_vfrecip_s, 68354}, // __builtin_lsx_vfrecip_s
    {loongarch_lsx_vfrecipe_d, 68367}, // __builtin_lsx_vfrecipe_d
    {loongarch_lsx_vfrecipe_s, 68381}, // __builtin_lsx_vfrecipe_s
    {loongarch_lsx_vfrint_d, 68395}, // __builtin_lsx_vfrint_d
    {loongarch_lsx_vfrint_s, 68407}, // __builtin_lsx_vfrint_s
    {loongarch_lsx_vfrintrm_d, 68419}, // __builtin_lsx_vfrintrm_d
    {loongarch_lsx_vfrintrm_s, 68433}, // __builtin_lsx_vfrintrm_s
    {loongarch_lsx_vfrintrne_d, 68447}, // __builtin_lsx_vfrintrne_d
    {loongarch_lsx_vfrintrne_s, 68462}, // __builtin_lsx_vfrintrne_s
    {loongarch_lsx_vfrintrp_d, 68477}, // __builtin_lsx_vfrintrp_d
    {loongarch_lsx_vfrintrp_s, 68491}, // __builtin_lsx_vfrintrp_s
    {loongarch_lsx_vfrintrz_d, 68505}, // __builtin_lsx_vfrintrz_d
    {loongarch_lsx_vfrintrz_s, 68519}, // __builtin_lsx_vfrintrz_s
    {loongarch_lsx_vfrsqrt_d, 68533}, // __builtin_lsx_vfrsqrt_d
    {loongarch_lsx_vfrsqrt_s, 68546}, // __builtin_lsx_vfrsqrt_s
    {loongarch_lsx_vfrsqrte_d, 68559}, // __builtin_lsx_vfrsqrte_d
    {loongarch_lsx_vfrsqrte_s, 68573}, // __builtin_lsx_vfrsqrte_s
    {loongarch_lsx_vfrstp_b, 68587}, // __builtin_lsx_vfrstp_b
    {loongarch_lsx_vfrstp_h, 68599}, // __builtin_lsx_vfrstp_h
    {loongarch_lsx_vfrstpi_b, 68611}, // __builtin_lsx_vfrstpi_b
    {loongarch_lsx_vfrstpi_h, 68624}, // __builtin_lsx_vfrstpi_h
    {loongarch_lsx_vfsqrt_d, 68637}, // __builtin_lsx_vfsqrt_d
    {loongarch_lsx_vfsqrt_s, 68649}, // __builtin_lsx_vfsqrt_s
    {loongarch_lsx_vfsub_d, 68661}, // __builtin_lsx_vfsub_d
    {loongarch_lsx_vfsub_s, 68672}, // __builtin_lsx_vfsub_s
    {loongarch_lsx_vftint_l_d, 68683}, // __builtin_lsx_vftint_l_d
    {loongarch_lsx_vftint_lu_d, 68697}, // __builtin_lsx_vftint_lu_d
    {loongarch_lsx_vftint_w_d, 68712}, // __builtin_lsx_vftint_w_d
    {loongarch_lsx_vftint_w_s, 68726}, // __builtin_lsx_vftint_w_s
    {loongarch_lsx_vftint_wu_s, 68740}, // __builtin_lsx_vftint_wu_s
    {loongarch_lsx_vftinth_l_s, 68755}, // __builtin_lsx_vftinth_l_s
    {loongarch_lsx_vftintl_l_s, 68770}, // __builtin_lsx_vftintl_l_s
    {loongarch_lsx_vftintrm_l_d, 68785}, // __builtin_lsx_vftintrm_l_d
    {loongarch_lsx_vftintrm_w_d, 68801}, // __builtin_lsx_vftintrm_w_d
    {loongarch_lsx_vftintrm_w_s, 68817}, // __builtin_lsx_vftintrm_w_s
    {loongarch_lsx_vftintrmh_l_s, 68833}, // __builtin_lsx_vftintrmh_l_s
    {loongarch_lsx_vftintrml_l_s, 68850}, // __builtin_lsx_vftintrml_l_s
    {loongarch_lsx_vftintrne_l_d, 68867}, // __builtin_lsx_vftintrne_l_d
    {loongarch_lsx_vftintrne_w_d, 68884}, // __builtin_lsx_vftintrne_w_d
    {loongarch_lsx_vftintrne_w_s, 68901}, // __builtin_lsx_vftintrne_w_s
    {loongarch_lsx_vftintrneh_l_s, 68918}, // __builtin_lsx_vftintrneh_l_s
    {loongarch_lsx_vftintrnel_l_s, 68936}, // __builtin_lsx_vftintrnel_l_s
    {loongarch_lsx_vftintrp_l_d, 68954}, // __builtin_lsx_vftintrp_l_d
    {loongarch_lsx_vftintrp_w_d, 68970}, // __builtin_lsx_vftintrp_w_d
    {loongarch_lsx_vftintrp_w_s, 68986}, // __builtin_lsx_vftintrp_w_s
    {loongarch_lsx_vftintrph_l_s, 69002}, // __builtin_lsx_vftintrph_l_s
    {loongarch_lsx_vftintrpl_l_s, 69019}, // __builtin_lsx_vftintrpl_l_s
    {loongarch_lsx_vftintrz_l_d, 69036}, // __builtin_lsx_vftintrz_l_d
    {loongarch_lsx_vftintrz_lu_d, 69052}, // __builtin_lsx_vftintrz_lu_d
    {loongarch_lsx_vftintrz_w_d, 69069}, // __builtin_lsx_vftintrz_w_d
    {loongarch_lsx_vftintrz_w_s, 69085}, // __builtin_lsx_vftintrz_w_s
    {loongarch_lsx_vftintrz_wu_s, 69101}, // __builtin_lsx_vftintrz_wu_s
    {loongarch_lsx_vftintrzh_l_s, 69118}, // __builtin_lsx_vftintrzh_l_s
    {loongarch_lsx_vftintrzl_l_s, 69135}, // __builtin_lsx_vftintrzl_l_s
    {loongarch_lsx_vhaddw_d_w, 69152}, // __builtin_lsx_vhaddw_d_w
    {loongarch_lsx_vhaddw_du_wu, 69166}, // __builtin_lsx_vhaddw_du_wu
    {loongarch_lsx_vhaddw_h_b, 69182}, // __builtin_lsx_vhaddw_h_b
    {loongarch_lsx_vhaddw_hu_bu, 69196}, // __builtin_lsx_vhaddw_hu_bu
    {loongarch_lsx_vhaddw_q_d, 69212}, // __builtin_lsx_vhaddw_q_d
    {loongarch_lsx_vhaddw_qu_du, 69226}, // __builtin_lsx_vhaddw_qu_du
    {loongarch_lsx_vhaddw_w_h, 69242}, // __builtin_lsx_vhaddw_w_h
    {loongarch_lsx_vhaddw_wu_hu, 69256}, // __builtin_lsx_vhaddw_wu_hu
    {loongarch_lsx_vhsubw_d_w, 69272}, // __builtin_lsx_vhsubw_d_w
    {loongarch_lsx_vhsubw_du_wu, 69286}, // __builtin_lsx_vhsubw_du_wu
    {loongarch_lsx_vhsubw_h_b, 69302}, // __builtin_lsx_vhsubw_h_b
    {loongarch_lsx_vhsubw_hu_bu, 69316}, // __builtin_lsx_vhsubw_hu_bu
    {loongarch_lsx_vhsubw_q_d, 69332}, // __builtin_lsx_vhsubw_q_d
    {loongarch_lsx_vhsubw_qu_du, 69346}, // __builtin_lsx_vhsubw_qu_du
    {loongarch_lsx_vhsubw_w_h, 69362}, // __builtin_lsx_vhsubw_w_h
    {loongarch_lsx_vhsubw_wu_hu, 69376}, // __builtin_lsx_vhsubw_wu_hu
    {loongarch_lsx_vilvh_b, 69392}, // __builtin_lsx_vilvh_b
    {loongarch_lsx_vilvh_d, 69403}, // __builtin_lsx_vilvh_d
    {loongarch_lsx_vilvh_h, 69414}, // __builtin_lsx_vilvh_h
    {loongarch_lsx_vilvh_w, 69425}, // __builtin_lsx_vilvh_w
    {loongarch_lsx_vilvl_b, 69436}, // __builtin_lsx_vilvl_b
    {loongarch_lsx_vilvl_d, 69447}, // __builtin_lsx_vilvl_d
    {loongarch_lsx_vilvl_h, 69458}, // __builtin_lsx_vilvl_h
    {loongarch_lsx_vilvl_w, 69469}, // __builtin_lsx_vilvl_w
    {loongarch_lsx_vinsgr2vr_b, 69480}, // __builtin_lsx_vinsgr2vr_b
    {loongarch_lsx_vinsgr2vr_d, 69495}, // __builtin_lsx_vinsgr2vr_d
    {loongarch_lsx_vinsgr2vr_h, 69510}, // __builtin_lsx_vinsgr2vr_h
    {loongarch_lsx_vinsgr2vr_w, 69525}, // __builtin_lsx_vinsgr2vr_w
    {loongarch_lsx_vld, 69540}, // __builtin_lsx_vld
    {loongarch_lsx_vldi, 69547}, // __builtin_lsx_vldi
    {loongarch_lsx_vldrepl_b, 69555}, // __builtin_lsx_vldrepl_b
    {loongarch_lsx_vldrepl_d, 69568}, // __builtin_lsx_vldrepl_d
    {loongarch_lsx_vldrepl_h, 69581}, // __builtin_lsx_vldrepl_h
    {loongarch_lsx_vldrepl_w, 69594}, // __builtin_lsx_vldrepl_w
    {loongarch_lsx_vldx, 69607}, // __builtin_lsx_vldx
    {loongarch_lsx_vmadd_b, 69615}, // __builtin_lsx_vmadd_b
    {loongarch_lsx_vmadd_d, 69626}, // __builtin_lsx_vmadd_d
    {loongarch_lsx_vmadd_h, 69637}, // __builtin_lsx_vmadd_h
    {loongarch_lsx_vmadd_w, 69648}, // __builtin_lsx_vmadd_w
    {loongarch_lsx_vmaddwev_d_w, 69659}, // __builtin_lsx_vmaddwev_d_w
    {loongarch_lsx_vmaddwev_d_wu, 69675}, // __builtin_lsx_vmaddwev_d_wu
    {loongarch_lsx_vmaddwev_d_wu_w, 69692}, // __builtin_lsx_vmaddwev_d_wu_w
    {loongarch_lsx_vmaddwev_h_b, 69711}, // __builtin_lsx_vmaddwev_h_b
    {loongarch_lsx_vmaddwev_h_bu, 69727}, // __builtin_lsx_vmaddwev_h_bu
    {loongarch_lsx_vmaddwev_h_bu_b, 69744}, // __builtin_lsx_vmaddwev_h_bu_b
    {loongarch_lsx_vmaddwev_q_d, 69763}, // __builtin_lsx_vmaddwev_q_d
    {loongarch_lsx_vmaddwev_q_du, 69779}, // __builtin_lsx_vmaddwev_q_du
    {loongarch_lsx_vmaddwev_q_du_d, 69796}, // __builtin_lsx_vmaddwev_q_du_d
    {loongarch_lsx_vmaddwev_w_h, 69815}, // __builtin_lsx_vmaddwev_w_h
    {loongarch_lsx_vmaddwev_w_hu, 69831}, // __builtin_lsx_vmaddwev_w_hu
    {loongarch_lsx_vmaddwev_w_hu_h, 69848}, // __builtin_lsx_vmaddwev_w_hu_h
    {loongarch_lsx_vmaddwod_d_w, 69867}, // __builtin_lsx_vmaddwod_d_w
    {loongarch_lsx_vmaddwod_d_wu, 69883}, // __builtin_lsx_vmaddwod_d_wu
    {loongarch_lsx_vmaddwod_d_wu_w, 69900}, // __builtin_lsx_vmaddwod_d_wu_w
    {loongarch_lsx_vmaddwod_h_b, 69919}, // __builtin_lsx_vmaddwod_h_b
    {loongarch_lsx_vmaddwod_h_bu, 69935}, // __builtin_lsx_vmaddwod_h_bu
    {loongarch_lsx_vmaddwod_h_bu_b, 69952}, // __builtin_lsx_vmaddwod_h_bu_b
    {loongarch_lsx_vmaddwod_q_d, 69971}, // __builtin_lsx_vmaddwod_q_d
    {loongarch_lsx_vmaddwod_q_du, 69987}, // __builtin_lsx_vmaddwod_q_du
    {loongarch_lsx_vmaddwod_q_du_d, 70004}, // __builtin_lsx_vmaddwod_q_du_d
    {loongarch_lsx_vmaddwod_w_h, 70023}, // __builtin_lsx_vmaddwod_w_h
    {loongarch_lsx_vmaddwod_w_hu, 70039}, // __builtin_lsx_vmaddwod_w_hu
    {loongarch_lsx_vmaddwod_w_hu_h, 70056}, // __builtin_lsx_vmaddwod_w_hu_h
    {loongarch_lsx_vmax_b, 70075}, // __builtin_lsx_vmax_b
    {loongarch_lsx_vmax_bu, 70085}, // __builtin_lsx_vmax_bu
    {loongarch_lsx_vmax_d, 70096}, // __builtin_lsx_vmax_d
    {loongarch_lsx_vmax_du, 70106}, // __builtin_lsx_vmax_du
    {loongarch_lsx_vmax_h, 70117}, // __builtin_lsx_vmax_h
    {loongarch_lsx_vmax_hu, 70127}, // __builtin_lsx_vmax_hu
    {loongarch_lsx_vmax_w, 70138}, // __builtin_lsx_vmax_w
    {loongarch_lsx_vmax_wu, 70148}, // __builtin_lsx_vmax_wu
    {loongarch_lsx_vmaxi_b, 70159}, // __builtin_lsx_vmaxi_b
    {loongarch_lsx_vmaxi_bu, 70170}, // __builtin_lsx_vmaxi_bu
    {loongarch_lsx_vmaxi_d, 70182}, // __builtin_lsx_vmaxi_d
    {loongarch_lsx_vmaxi_du, 70193}, // __builtin_lsx_vmaxi_du
    {loongarch_lsx_vmaxi_h, 70205}, // __builtin_lsx_vmaxi_h
    {loongarch_lsx_vmaxi_hu, 70216}, // __builtin_lsx_vmaxi_hu
    {loongarch_lsx_vmaxi_w, 70228}, // __builtin_lsx_vmaxi_w
    {loongarch_lsx_vmaxi_wu, 70239}, // __builtin_lsx_vmaxi_wu
    {loongarch_lsx_vmin_b, 70251}, // __builtin_lsx_vmin_b
    {loongarch_lsx_vmin_bu, 70261}, // __builtin_lsx_vmin_bu
    {loongarch_lsx_vmin_d, 70272}, // __builtin_lsx_vmin_d
    {loongarch_lsx_vmin_du, 70282}, // __builtin_lsx_vmin_du
    {loongarch_lsx_vmin_h, 70293}, // __builtin_lsx_vmin_h
    {loongarch_lsx_vmin_hu, 70303}, // __builtin_lsx_vmin_hu
    {loongarch_lsx_vmin_w, 70314}, // __builtin_lsx_vmin_w
    {loongarch_lsx_vmin_wu, 70324}, // __builtin_lsx_vmin_wu
    {loongarch_lsx_vmini_b, 70335}, // __builtin_lsx_vmini_b
    {loongarch_lsx_vmini_bu, 70346}, // __builtin_lsx_vmini_bu
    {loongarch_lsx_vmini_d, 70358}, // __builtin_lsx_vmini_d
    {loongarch_lsx_vmini_du, 70369}, // __builtin_lsx_vmini_du
    {loongarch_lsx_vmini_h, 70381}, // __builtin_lsx_vmini_h
    {loongarch_lsx_vmini_hu, 70392}, // __builtin_lsx_vmini_hu
    {loongarch_lsx_vmini_w, 70404}, // __builtin_lsx_vmini_w
    {loongarch_lsx_vmini_wu, 70415}, // __builtin_lsx_vmini_wu
    {loongarch_lsx_vmod_b, 70427}, // __builtin_lsx_vmod_b
    {loongarch_lsx_vmod_bu, 70437}, // __builtin_lsx_vmod_bu
    {loongarch_lsx_vmod_d, 70448}, // __builtin_lsx_vmod_d
    {loongarch_lsx_vmod_du, 70458}, // __builtin_lsx_vmod_du
    {loongarch_lsx_vmod_h, 70469}, // __builtin_lsx_vmod_h
    {loongarch_lsx_vmod_hu, 70479}, // __builtin_lsx_vmod_hu
    {loongarch_lsx_vmod_w, 70490}, // __builtin_lsx_vmod_w
    {loongarch_lsx_vmod_wu, 70500}, // __builtin_lsx_vmod_wu
    {loongarch_lsx_vmskgez_b, 70511}, // __builtin_lsx_vmskgez_b
    {loongarch_lsx_vmskltz_b, 70524}, // __builtin_lsx_vmskltz_b
    {loongarch_lsx_vmskltz_d, 70537}, // __builtin_lsx_vmskltz_d
    {loongarch_lsx_vmskltz_h, 70550}, // __builtin_lsx_vmskltz_h
    {loongarch_lsx_vmskltz_w, 70563}, // __builtin_lsx_vmskltz_w
    {loongarch_lsx_vmsknz_b, 70576}, // __builtin_lsx_vmsknz_b
    {loongarch_lsx_vmsub_b, 70588}, // __builtin_lsx_vmsub_b
    {loongarch_lsx_vmsub_d, 70599}, // __builtin_lsx_vmsub_d
    {loongarch_lsx_vmsub_h, 70610}, // __builtin_lsx_vmsub_h
    {loongarch_lsx_vmsub_w, 70621}, // __builtin_lsx_vmsub_w
    {loongarch_lsx_vmuh_b, 70632}, // __builtin_lsx_vmuh_b
    {loongarch_lsx_vmuh_bu, 70642}, // __builtin_lsx_vmuh_bu
    {loongarch_lsx_vmuh_d, 70653}, // __builtin_lsx_vmuh_d
    {loongarch_lsx_vmuh_du, 70663}, // __builtin_lsx_vmuh_du
    {loongarch_lsx_vmuh_h, 70674}, // __builtin_lsx_vmuh_h
    {loongarch_lsx_vmuh_hu, 70684}, // __builtin_lsx_vmuh_hu
    {loongarch_lsx_vmuh_w, 70695}, // __builtin_lsx_vmuh_w
    {loongarch_lsx_vmuh_wu, 70705}, // __builtin_lsx_vmuh_wu
    {loongarch_lsx_vmul_b, 70716}, // __builtin_lsx_vmul_b
    {loongarch_lsx_vmul_d, 70726}, // __builtin_lsx_vmul_d
    {loongarch_lsx_vmul_h, 70736}, // __builtin_lsx_vmul_h
    {loongarch_lsx_vmul_w, 70746}, // __builtin_lsx_vmul_w
    {loongarch_lsx_vmulwev_d_w, 70756}, // __builtin_lsx_vmulwev_d_w
    {loongarch_lsx_vmulwev_d_wu, 70771}, // __builtin_lsx_vmulwev_d_wu
    {loongarch_lsx_vmulwev_d_wu_w, 70787}, // __builtin_lsx_vmulwev_d_wu_w
    {loongarch_lsx_vmulwev_h_b, 70805}, // __builtin_lsx_vmulwev_h_b
    {loongarch_lsx_vmulwev_h_bu, 70820}, // __builtin_lsx_vmulwev_h_bu
    {loongarch_lsx_vmulwev_h_bu_b, 70836}, // __builtin_lsx_vmulwev_h_bu_b
    {loongarch_lsx_vmulwev_q_d, 70854}, // __builtin_lsx_vmulwev_q_d
    {loongarch_lsx_vmulwev_q_du, 70869}, // __builtin_lsx_vmulwev_q_du
    {loongarch_lsx_vmulwev_q_du_d, 70885}, // __builtin_lsx_vmulwev_q_du_d
    {loongarch_lsx_vmulwev_w_h, 70903}, // __builtin_lsx_vmulwev_w_h
    {loongarch_lsx_vmulwev_w_hu, 70918}, // __builtin_lsx_vmulwev_w_hu
    {loongarch_lsx_vmulwev_w_hu_h, 70934}, // __builtin_lsx_vmulwev_w_hu_h
    {loongarch_lsx_vmulwod_d_w, 70952}, // __builtin_lsx_vmulwod_d_w
    {loongarch_lsx_vmulwod_d_wu, 70967}, // __builtin_lsx_vmulwod_d_wu
    {loongarch_lsx_vmulwod_d_wu_w, 70983}, // __builtin_lsx_vmulwod_d_wu_w
    {loongarch_lsx_vmulwod_h_b, 71001}, // __builtin_lsx_vmulwod_h_b
    {loongarch_lsx_vmulwod_h_bu, 71016}, // __builtin_lsx_vmulwod_h_bu
    {loongarch_lsx_vmulwod_h_bu_b, 71032}, // __builtin_lsx_vmulwod_h_bu_b
    {loongarch_lsx_vmulwod_q_d, 71050}, // __builtin_lsx_vmulwod_q_d
    {loongarch_lsx_vmulwod_q_du, 71065}, // __builtin_lsx_vmulwod_q_du
    {loongarch_lsx_vmulwod_q_du_d, 71081}, // __builtin_lsx_vmulwod_q_du_d
    {loongarch_lsx_vmulwod_w_h, 71099}, // __builtin_lsx_vmulwod_w_h
    {loongarch_lsx_vmulwod_w_hu, 71114}, // __builtin_lsx_vmulwod_w_hu
    {loongarch_lsx_vmulwod_w_hu_h, 71130}, // __builtin_lsx_vmulwod_w_hu_h
    {loongarch_lsx_vneg_b, 71148}, // __builtin_lsx_vneg_b
    {loongarch_lsx_vneg_d, 71158}, // __builtin_lsx_vneg_d
    {loongarch_lsx_vneg_h, 71168}, // __builtin_lsx_vneg_h
    {loongarch_lsx_vneg_w, 71178}, // __builtin_lsx_vneg_w
    {loongarch_lsx_vnor_v, 71188}, // __builtin_lsx_vnor_v
    {loongarch_lsx_vnori_b, 71198}, // __builtin_lsx_vnori_b
    {loongarch_lsx_vor_v, 71209}, // __builtin_lsx_vor_v
    {loongarch_lsx_vori_b, 71218}, // __builtin_lsx_vori_b
    {loongarch_lsx_vorn_v, 71228}, // __builtin_lsx_vorn_v
    {loongarch_lsx_vpackev_b, 71238}, // __builtin_lsx_vpackev_b
    {loongarch_lsx_vpackev_d, 71251}, // __builtin_lsx_vpackev_d
    {loongarch_lsx_vpackev_h, 71264}, // __builtin_lsx_vpackev_h
    {loongarch_lsx_vpackev_w, 71277}, // __builtin_lsx_vpackev_w
    {loongarch_lsx_vpackod_b, 71290}, // __builtin_lsx_vpackod_b
    {loongarch_lsx_vpackod_d, 71303}, // __builtin_lsx_vpackod_d
    {loongarch_lsx_vpackod_h, 71316}, // __builtin_lsx_vpackod_h
    {loongarch_lsx_vpackod_w, 71329}, // __builtin_lsx_vpackod_w
    {loongarch_lsx_vpcnt_b, 71342}, // __builtin_lsx_vpcnt_b
    {loongarch_lsx_vpcnt_d, 71353}, // __builtin_lsx_vpcnt_d
    {loongarch_lsx_vpcnt_h, 71364}, // __builtin_lsx_vpcnt_h
    {loongarch_lsx_vpcnt_w, 71375}, // __builtin_lsx_vpcnt_w
    {loongarch_lsx_vpermi_w, 71386}, // __builtin_lsx_vpermi_w
    {loongarch_lsx_vpickev_b, 71398}, // __builtin_lsx_vpickev_b
    {loongarch_lsx_vpickev_d, 71411}, // __builtin_lsx_vpickev_d
    {loongarch_lsx_vpickev_h, 71424}, // __builtin_lsx_vpickev_h
    {loongarch_lsx_vpickev_w, 71437}, // __builtin_lsx_vpickev_w
    {loongarch_lsx_vpickod_b, 71450}, // __builtin_lsx_vpickod_b
    {loongarch_lsx_vpickod_d, 71463}, // __builtin_lsx_vpickod_d
    {loongarch_lsx_vpickod_h, 71476}, // __builtin_lsx_vpickod_h
    {loongarch_lsx_vpickod_w, 71489}, // __builtin_lsx_vpickod_w
    {loongarch_lsx_vpickve2gr_b, 71502}, // __builtin_lsx_vpickve2gr_b
    {loongarch_lsx_vpickve2gr_bu, 71518}, // __builtin_lsx_vpickve2gr_bu
    {loongarch_lsx_vpickve2gr_d, 71535}, // __builtin_lsx_vpickve2gr_d
    {loongarch_lsx_vpickve2gr_du, 71551}, // __builtin_lsx_vpickve2gr_du
    {loongarch_lsx_vpickve2gr_h, 71568}, // __builtin_lsx_vpickve2gr_h
    {loongarch_lsx_vpickve2gr_hu, 71584}, // __builtin_lsx_vpickve2gr_hu
    {loongarch_lsx_vpickve2gr_w, 71601}, // __builtin_lsx_vpickve2gr_w
    {loongarch_lsx_vpickve2gr_wu, 71617}, // __builtin_lsx_vpickve2gr_wu
    {loongarch_lsx_vreplgr2vr_b, 71634}, // __builtin_lsx_vreplgr2vr_b
    {loongarch_lsx_vreplgr2vr_d, 71650}, // __builtin_lsx_vreplgr2vr_d
    {loongarch_lsx_vreplgr2vr_h, 71666}, // __builtin_lsx_vreplgr2vr_h
    {loongarch_lsx_vreplgr2vr_w, 71682}, // __builtin_lsx_vreplgr2vr_w
    {loongarch_lsx_vrepli_b, 71698}, // __builtin_lsx_vrepli_b
    {loongarch_lsx_vrepli_d, 71710}, // __builtin_lsx_vrepli_d
    {loongarch_lsx_vrepli_h, 71722}, // __builtin_lsx_vrepli_h
    {loongarch_lsx_vrepli_w, 71734}, // __builtin_lsx_vrepli_w
    {loongarch_lsx_vreplve_b, 71746}, // __builtin_lsx_vreplve_b
    {loongarch_lsx_vreplve_d, 71759}, // __builtin_lsx_vreplve_d
    {loongarch_lsx_vreplve_h, 71772}, // __builtin_lsx_vreplve_h
    {loongarch_lsx_vreplve_w, 71785}, // __builtin_lsx_vreplve_w
    {loongarch_lsx_vreplvei_b, 71798}, // __builtin_lsx_vreplvei_b
    {loongarch_lsx_vreplvei_d, 71812}, // __builtin_lsx_vreplvei_d
    {loongarch_lsx_vreplvei_h, 71826}, // __builtin_lsx_vreplvei_h
    {loongarch_lsx_vreplvei_w, 71840}, // __builtin_lsx_vreplvei_w
    {loongarch_lsx_vrotr_b, 71854}, // __builtin_lsx_vrotr_b
    {loongarch_lsx_vrotr_d, 71865}, // __builtin_lsx_vrotr_d
    {loongarch_lsx_vrotr_h, 71876}, // __builtin_lsx_vrotr_h
    {loongarch_lsx_vrotr_w, 71887}, // __builtin_lsx_vrotr_w
    {loongarch_lsx_vrotri_b, 71898}, // __builtin_lsx_vrotri_b
    {loongarch_lsx_vrotri_d, 71910}, // __builtin_lsx_vrotri_d
    {loongarch_lsx_vrotri_h, 71922}, // __builtin_lsx_vrotri_h
    {loongarch_lsx_vrotri_w, 71934}, // __builtin_lsx_vrotri_w
    {loongarch_lsx_vsadd_b, 71946}, // __builtin_lsx_vsadd_b
    {loongarch_lsx_vsadd_bu, 71957}, // __builtin_lsx_vsadd_bu
    {loongarch_lsx_vsadd_d, 71969}, // __builtin_lsx_vsadd_d
    {loongarch_lsx_vsadd_du, 71980}, // __builtin_lsx_vsadd_du
    {loongarch_lsx_vsadd_h, 71992}, // __builtin_lsx_vsadd_h
    {loongarch_lsx_vsadd_hu, 72003}, // __builtin_lsx_vsadd_hu
    {loongarch_lsx_vsadd_w, 72015}, // __builtin_lsx_vsadd_w
    {loongarch_lsx_vsadd_wu, 72026}, // __builtin_lsx_vsadd_wu
    {loongarch_lsx_vsat_b, 72038}, // __builtin_lsx_vsat_b
    {loongarch_lsx_vsat_bu, 72048}, // __builtin_lsx_vsat_bu
    {loongarch_lsx_vsat_d, 72059}, // __builtin_lsx_vsat_d
    {loongarch_lsx_vsat_du, 72069}, // __builtin_lsx_vsat_du
    {loongarch_lsx_vsat_h, 72080}, // __builtin_lsx_vsat_h
    {loongarch_lsx_vsat_hu, 72090}, // __builtin_lsx_vsat_hu
    {loongarch_lsx_vsat_w, 72101}, // __builtin_lsx_vsat_w
    {loongarch_lsx_vsat_wu, 72111}, // __builtin_lsx_vsat_wu
    {loongarch_lsx_vseq_b, 72122}, // __builtin_lsx_vseq_b
    {loongarch_lsx_vseq_d, 72132}, // __builtin_lsx_vseq_d
    {loongarch_lsx_vseq_h, 72142}, // __builtin_lsx_vseq_h
    {loongarch_lsx_vseq_w, 72152}, // __builtin_lsx_vseq_w
    {loongarch_lsx_vseqi_b, 72162}, // __builtin_lsx_vseqi_b
    {loongarch_lsx_vseqi_d, 72173}, // __builtin_lsx_vseqi_d
    {loongarch_lsx_vseqi_h, 72184}, // __builtin_lsx_vseqi_h
    {loongarch_lsx_vseqi_w, 72195}, // __builtin_lsx_vseqi_w
    {loongarch_lsx_vshuf4i_b, 72206}, // __builtin_lsx_vshuf4i_b
    {loongarch_lsx_vshuf4i_d, 72219}, // __builtin_lsx_vshuf4i_d
    {loongarch_lsx_vshuf4i_h, 72232}, // __builtin_lsx_vshuf4i_h
    {loongarch_lsx_vshuf4i_w, 72245}, // __builtin_lsx_vshuf4i_w
    {loongarch_lsx_vshuf_b, 72258}, // __builtin_lsx_vshuf_b
    {loongarch_lsx_vshuf_d, 72269}, // __builtin_lsx_vshuf_d
    {loongarch_lsx_vshuf_h, 72280}, // __builtin_lsx_vshuf_h
    {loongarch_lsx_vshuf_w, 72291}, // __builtin_lsx_vshuf_w
    {loongarch_lsx_vsigncov_b, 72302}, // __builtin_lsx_vsigncov_b
    {loongarch_lsx_vsigncov_d, 72316}, // __builtin_lsx_vsigncov_d
    {loongarch_lsx_vsigncov_h, 72330}, // __builtin_lsx_vsigncov_h
    {loongarch_lsx_vsigncov_w, 72344}, // __builtin_lsx_vsigncov_w
    {loongarch_lsx_vsle_b, 72358}, // __builtin_lsx_vsle_b
    {loongarch_lsx_vsle_bu, 72368}, // __builtin_lsx_vsle_bu
    {loongarch_lsx_vsle_d, 72379}, // __builtin_lsx_vsle_d
    {loongarch_lsx_vsle_du, 72389}, // __builtin_lsx_vsle_du
    {loongarch_lsx_vsle_h, 72400}, // __builtin_lsx_vsle_h
    {loongarch_lsx_vsle_hu, 72410}, // __builtin_lsx_vsle_hu
    {loongarch_lsx_vsle_w, 72421}, // __builtin_lsx_vsle_w
    {loongarch_lsx_vsle_wu, 72431}, // __builtin_lsx_vsle_wu
    {loongarch_lsx_vslei_b, 72442}, // __builtin_lsx_vslei_b
    {loongarch_lsx_vslei_bu, 72453}, // __builtin_lsx_vslei_bu
    {loongarch_lsx_vslei_d, 72465}, // __builtin_lsx_vslei_d
    {loongarch_lsx_vslei_du, 72476}, // __builtin_lsx_vslei_du
    {loongarch_lsx_vslei_h, 72488}, // __builtin_lsx_vslei_h
    {loongarch_lsx_vslei_hu, 72499}, // __builtin_lsx_vslei_hu
    {loongarch_lsx_vslei_w, 72511}, // __builtin_lsx_vslei_w
    {loongarch_lsx_vslei_wu, 72522}, // __builtin_lsx_vslei_wu
    {loongarch_lsx_vsll_b, 72534}, // __builtin_lsx_vsll_b
    {loongarch_lsx_vsll_d, 72544}, // __builtin_lsx_vsll_d
    {loongarch_lsx_vsll_h, 72554}, // __builtin_lsx_vsll_h
    {loongarch_lsx_vsll_w, 72564}, // __builtin_lsx_vsll_w
    {loongarch_lsx_vslli_b, 72574}, // __builtin_lsx_vslli_b
    {loongarch_lsx_vslli_d, 72585}, // __builtin_lsx_vslli_d
    {loongarch_lsx_vslli_h, 72596}, // __builtin_lsx_vslli_h
    {loongarch_lsx_vslli_w, 72607}, // __builtin_lsx_vslli_w
    {loongarch_lsx_vsllwil_d_w, 72618}, // __builtin_lsx_vsllwil_d_w
    {loongarch_lsx_vsllwil_du_wu, 72633}, // __builtin_lsx_vsllwil_du_wu
    {loongarch_lsx_vsllwil_h_b, 72650}, // __builtin_lsx_vsllwil_h_b
    {loongarch_lsx_vsllwil_hu_bu, 72665}, // __builtin_lsx_vsllwil_hu_bu
    {loongarch_lsx_vsllwil_w_h, 72682}, // __builtin_lsx_vsllwil_w_h
    {loongarch_lsx_vsllwil_wu_hu, 72697}, // __builtin_lsx_vsllwil_wu_hu
    {loongarch_lsx_vslt_b, 72714}, // __builtin_lsx_vslt_b
    {loongarch_lsx_vslt_bu, 72724}, // __builtin_lsx_vslt_bu
    {loongarch_lsx_vslt_d, 72735}, // __builtin_lsx_vslt_d
    {loongarch_lsx_vslt_du, 72745}, // __builtin_lsx_vslt_du
    {loongarch_lsx_vslt_h, 72756}, // __builtin_lsx_vslt_h
    {loongarch_lsx_vslt_hu, 72766}, // __builtin_lsx_vslt_hu
    {loongarch_lsx_vslt_w, 72777}, // __builtin_lsx_vslt_w
    {loongarch_lsx_vslt_wu, 72787}, // __builtin_lsx_vslt_wu
    {loongarch_lsx_vslti_b, 72798}, // __builtin_lsx_vslti_b
    {loongarch_lsx_vslti_bu, 72809}, // __builtin_lsx_vslti_bu
    {loongarch_lsx_vslti_d, 72821}, // __builtin_lsx_vslti_d
    {loongarch_lsx_vslti_du, 72832}, // __builtin_lsx_vslti_du
    {loongarch_lsx_vslti_h, 72844}, // __builtin_lsx_vslti_h
    {loongarch_lsx_vslti_hu, 72855}, // __builtin_lsx_vslti_hu
    {loongarch_lsx_vslti_w, 72867}, // __builtin_lsx_vslti_w
    {loongarch_lsx_vslti_wu, 72878}, // __builtin_lsx_vslti_wu
    {loongarch_lsx_vsra_b, 72890}, // __builtin_lsx_vsra_b
    {loongarch_lsx_vsra_d, 72900}, // __builtin_lsx_vsra_d
    {loongarch_lsx_vsra_h, 72910}, // __builtin_lsx_vsra_h
    {loongarch_lsx_vsra_w, 72920}, // __builtin_lsx_vsra_w
    {loongarch_lsx_vsrai_b, 72930}, // __builtin_lsx_vsrai_b
    {loongarch_lsx_vsrai_d, 72941}, // __builtin_lsx_vsrai_d
    {loongarch_lsx_vsrai_h, 72952}, // __builtin_lsx_vsrai_h
    {loongarch_lsx_vsrai_w, 72963}, // __builtin_lsx_vsrai_w
    {loongarch_lsx_vsran_b_h, 72974}, // __builtin_lsx_vsran_b_h
    {loongarch_lsx_vsran_h_w, 72987}, // __builtin_lsx_vsran_h_w
    {loongarch_lsx_vsran_w_d, 73000}, // __builtin_lsx_vsran_w_d
    {loongarch_lsx_vsrani_b_h, 73013}, // __builtin_lsx_vsrani_b_h
    {loongarch_lsx_vsrani_d_q, 73027}, // __builtin_lsx_vsrani_d_q
    {loongarch_lsx_vsrani_h_w, 73041}, // __builtin_lsx_vsrani_h_w
    {loongarch_lsx_vsrani_w_d, 73055}, // __builtin_lsx_vsrani_w_d
    {loongarch_lsx_vsrar_b, 73069}, // __builtin_lsx_vsrar_b
    {loongarch_lsx_vsrar_d, 73080}, // __builtin_lsx_vsrar_d
    {loongarch_lsx_vsrar_h, 73091}, // __builtin_lsx_vsrar_h
    {loongarch_lsx_vsrar_w, 73102}, // __builtin_lsx_vsrar_w
    {loongarch_lsx_vsrari_b, 73113}, // __builtin_lsx_vsrari_b
    {loongarch_lsx_vsrari_d, 73125}, // __builtin_lsx_vsrari_d
    {loongarch_lsx_vsrari_h, 73137}, // __builtin_lsx_vsrari_h
    {loongarch_lsx_vsrari_w, 73149}, // __builtin_lsx_vsrari_w
    {loongarch_lsx_vsrarn_b_h, 73161}, // __builtin_lsx_vsrarn_b_h
    {loongarch_lsx_vsrarn_h_w, 73175}, // __builtin_lsx_vsrarn_h_w
    {loongarch_lsx_vsrarn_w_d, 73189}, // __builtin_lsx_vsrarn_w_d
    {loongarch_lsx_vsrarni_b_h, 73203}, // __builtin_lsx_vsrarni_b_h
    {loongarch_lsx_vsrarni_d_q, 73218}, // __builtin_lsx_vsrarni_d_q
    {loongarch_lsx_vsrarni_h_w, 73233}, // __builtin_lsx_vsrarni_h_w
    {loongarch_lsx_vsrarni_w_d, 73248}, // __builtin_lsx_vsrarni_w_d
    {loongarch_lsx_vsrl_b, 73263}, // __builtin_lsx_vsrl_b
    {loongarch_lsx_vsrl_d, 73273}, // __builtin_lsx_vsrl_d
    {loongarch_lsx_vsrl_h, 73283}, // __builtin_lsx_vsrl_h
    {loongarch_lsx_vsrl_w, 73293}, // __builtin_lsx_vsrl_w
    {loongarch_lsx_vsrli_b, 73303}, // __builtin_lsx_vsrli_b
    {loongarch_lsx_vsrli_d, 73314}, // __builtin_lsx_vsrli_d
    {loongarch_lsx_vsrli_h, 73325}, // __builtin_lsx_vsrli_h
    {loongarch_lsx_vsrli_w, 73336}, // __builtin_lsx_vsrli_w
    {loongarch_lsx_vsrln_b_h, 73347}, // __builtin_lsx_vsrln_b_h
    {loongarch_lsx_vsrln_h_w, 73360}, // __builtin_lsx_vsrln_h_w
    {loongarch_lsx_vsrln_w_d, 73373}, // __builtin_lsx_vsrln_w_d
    {loongarch_lsx_vsrlni_b_h, 73386}, // __builtin_lsx_vsrlni_b_h
    {loongarch_lsx_vsrlni_d_q, 73400}, // __builtin_lsx_vsrlni_d_q
    {loongarch_lsx_vsrlni_h_w, 73414}, // __builtin_lsx_vsrlni_h_w
    {loongarch_lsx_vsrlni_w_d, 73428}, // __builtin_lsx_vsrlni_w_d
    {loongarch_lsx_vsrlr_b, 73442}, // __builtin_lsx_vsrlr_b
    {loongarch_lsx_vsrlr_d, 73453}, // __builtin_lsx_vsrlr_d
    {loongarch_lsx_vsrlr_h, 73464}, // __builtin_lsx_vsrlr_h
    {loongarch_lsx_vsrlr_w, 73475}, // __builtin_lsx_vsrlr_w
    {loongarch_lsx_vsrlri_b, 73486}, // __builtin_lsx_vsrlri_b
    {loongarch_lsx_vsrlri_d, 73498}, // __builtin_lsx_vsrlri_d
    {loongarch_lsx_vsrlri_h, 73510}, // __builtin_lsx_vsrlri_h
    {loongarch_lsx_vsrlri_w, 73522}, // __builtin_lsx_vsrlri_w
    {loongarch_lsx_vsrlrn_b_h, 73534}, // __builtin_lsx_vsrlrn_b_h
    {loongarch_lsx_vsrlrn_h_w, 73548}, // __builtin_lsx_vsrlrn_h_w
    {loongarch_lsx_vsrlrn_w_d, 73562}, // __builtin_lsx_vsrlrn_w_d
    {loongarch_lsx_vsrlrni_b_h, 73576}, // __builtin_lsx_vsrlrni_b_h
    {loongarch_lsx_vsrlrni_d_q, 73591}, // __builtin_lsx_vsrlrni_d_q
    {loongarch_lsx_vsrlrni_h_w, 73606}, // __builtin_lsx_vsrlrni_h_w
    {loongarch_lsx_vsrlrni_w_d, 73621}, // __builtin_lsx_vsrlrni_w_d
    {loongarch_lsx_vssran_b_h, 73636}, // __builtin_lsx_vssran_b_h
    {loongarch_lsx_vssran_bu_h, 73650}, // __builtin_lsx_vssran_bu_h
    {loongarch_lsx_vssran_h_w, 73665}, // __builtin_lsx_vssran_h_w
    {loongarch_lsx_vssran_hu_w, 73679}, // __builtin_lsx_vssran_hu_w
    {loongarch_lsx_vssran_w_d, 73694}, // __builtin_lsx_vssran_w_d
    {loongarch_lsx_vssran_wu_d, 73708}, // __builtin_lsx_vssran_wu_d
    {loongarch_lsx_vssrani_b_h, 73723}, // __builtin_lsx_vssrani_b_h
    {loongarch_lsx_vssrani_bu_h, 73738}, // __builtin_lsx_vssrani_bu_h
    {loongarch_lsx_vssrani_d_q, 73754}, // __builtin_lsx_vssrani_d_q
    {loongarch_lsx_vssrani_du_q, 73769}, // __builtin_lsx_vssrani_du_q
    {loongarch_lsx_vssrani_h_w, 73785}, // __builtin_lsx_vssrani_h_w
    {loongarch_lsx_vssrani_hu_w, 73800}, // __builtin_lsx_vssrani_hu_w
    {loongarch_lsx_vssrani_w_d, 73816}, // __builtin_lsx_vssrani_w_d
    {loongarch_lsx_vssrani_wu_d, 73831}, // __builtin_lsx_vssrani_wu_d
    {loongarch_lsx_vssrarn_b_h, 73847}, // __builtin_lsx_vssrarn_b_h
    {loongarch_lsx_vssrarn_bu_h, 73862}, // __builtin_lsx_vssrarn_bu_h
    {loongarch_lsx_vssrarn_h_w, 73878}, // __builtin_lsx_vssrarn_h_w
    {loongarch_lsx_vssrarn_hu_w, 73893}, // __builtin_lsx_vssrarn_hu_w
    {loongarch_lsx_vssrarn_w_d, 73909}, // __builtin_lsx_vssrarn_w_d
    {loongarch_lsx_vssrarn_wu_d, 73924}, // __builtin_lsx_vssrarn_wu_d
    {loongarch_lsx_vssrarni_b_h, 73940}, // __builtin_lsx_vssrarni_b_h
    {loongarch_lsx_vssrarni_bu_h, 73956}, // __builtin_lsx_vssrarni_bu_h
    {loongarch_lsx_vssrarni_d_q, 73973}, // __builtin_lsx_vssrarni_d_q
    {loongarch_lsx_vssrarni_du_q, 73989}, // __builtin_lsx_vssrarni_du_q
    {loongarch_lsx_vssrarni_h_w, 74006}, // __builtin_lsx_vssrarni_h_w
    {loongarch_lsx_vssrarni_hu_w, 74022}, // __builtin_lsx_vssrarni_hu_w
    {loongarch_lsx_vssrarni_w_d, 74039}, // __builtin_lsx_vssrarni_w_d
    {loongarch_lsx_vssrarni_wu_d, 74055}, // __builtin_lsx_vssrarni_wu_d
    {loongarch_lsx_vssrln_b_h, 74072}, // __builtin_lsx_vssrln_b_h
    {loongarch_lsx_vssrln_bu_h, 74086}, // __builtin_lsx_vssrln_bu_h
    {loongarch_lsx_vssrln_h_w, 74101}, // __builtin_lsx_vssrln_h_w
    {loongarch_lsx_vssrln_hu_w, 74115}, // __builtin_lsx_vssrln_hu_w
    {loongarch_lsx_vssrln_w_d, 74130}, // __builtin_lsx_vssrln_w_d
    {loongarch_lsx_vssrln_wu_d, 74144}, // __builtin_lsx_vssrln_wu_d
    {loongarch_lsx_vssrlni_b_h, 74159}, // __builtin_lsx_vssrlni_b_h
    {loongarch_lsx_vssrlni_bu_h, 74174}, // __builtin_lsx_vssrlni_bu_h
    {loongarch_lsx_vssrlni_d_q, 74190}, // __builtin_lsx_vssrlni_d_q
    {loongarch_lsx_vssrlni_du_q, 74205}, // __builtin_lsx_vssrlni_du_q
    {loongarch_lsx_vssrlni_h_w, 74221}, // __builtin_lsx_vssrlni_h_w
    {loongarch_lsx_vssrlni_hu_w, 74236}, // __builtin_lsx_vssrlni_hu_w
    {loongarch_lsx_vssrlni_w_d, 74252}, // __builtin_lsx_vssrlni_w_d
    {loongarch_lsx_vssrlni_wu_d, 74267}, // __builtin_lsx_vssrlni_wu_d
    {loongarch_lsx_vssrlrn_b_h, 74283}, // __builtin_lsx_vssrlrn_b_h
    {loongarch_lsx_vssrlrn_bu_h, 74298}, // __builtin_lsx_vssrlrn_bu_h
    {loongarch_lsx_vssrlrn_h_w, 74314}, // __builtin_lsx_vssrlrn_h_w
    {loongarch_lsx_vssrlrn_hu_w, 74329}, // __builtin_lsx_vssrlrn_hu_w
    {loongarch_lsx_vssrlrn_w_d, 74345}, // __builtin_lsx_vssrlrn_w_d
    {loongarch_lsx_vssrlrn_wu_d, 74360}, // __builtin_lsx_vssrlrn_wu_d
    {loongarch_lsx_vssrlrni_b_h, 74376}, // __builtin_lsx_vssrlrni_b_h
    {loongarch_lsx_vssrlrni_bu_h, 74392}, // __builtin_lsx_vssrlrni_bu_h
    {loongarch_lsx_vssrlrni_d_q, 74409}, // __builtin_lsx_vssrlrni_d_q
    {loongarch_lsx_vssrlrni_du_q, 74425}, // __builtin_lsx_vssrlrni_du_q
    {loongarch_lsx_vssrlrni_h_w, 74442}, // __builtin_lsx_vssrlrni_h_w
    {loongarch_lsx_vssrlrni_hu_w, 74458}, // __builtin_lsx_vssrlrni_hu_w
    {loongarch_lsx_vssrlrni_w_d, 74475}, // __builtin_lsx_vssrlrni_w_d
    {loongarch_lsx_vssrlrni_wu_d, 74491}, // __builtin_lsx_vssrlrni_wu_d
    {loongarch_lsx_vssub_b, 74508}, // __builtin_lsx_vssub_b
    {loongarch_lsx_vssub_bu, 74519}, // __builtin_lsx_vssub_bu
    {loongarch_lsx_vssub_d, 74531}, // __builtin_lsx_vssub_d
    {loongarch_lsx_vssub_du, 74542}, // __builtin_lsx_vssub_du
    {loongarch_lsx_vssub_h, 74554}, // __builtin_lsx_vssub_h
    {loongarch_lsx_vssub_hu, 74565}, // __builtin_lsx_vssub_hu
    {loongarch_lsx_vssub_w, 74577}, // __builtin_lsx_vssub_w
    {loongarch_lsx_vssub_wu, 74588}, // __builtin_lsx_vssub_wu
    {loongarch_lsx_vst, 74600}, // __builtin_lsx_vst
    {loongarch_lsx_vstelm_b, 74607}, // __builtin_lsx_vstelm_b
    {loongarch_lsx_vstelm_d, 74619}, // __builtin_lsx_vstelm_d
    {loongarch_lsx_vstelm_h, 74631}, // __builtin_lsx_vstelm_h
    {loongarch_lsx_vstelm_w, 74643}, // __builtin_lsx_vstelm_w
    {loongarch_lsx_vstx, 74655}, // __builtin_lsx_vstx
    {loongarch_lsx_vsub_b, 74663}, // __builtin_lsx_vsub_b
    {loongarch_lsx_vsub_d, 74673}, // __builtin_lsx_vsub_d
    {loongarch_lsx_vsub_h, 74683}, // __builtin_lsx_vsub_h
    {loongarch_lsx_vsub_q, 74693}, // __builtin_lsx_vsub_q
    {loongarch_lsx_vsub_w, 74703}, // __builtin_lsx_vsub_w
    {loongarch_lsx_vsubi_bu, 74713}, // __builtin_lsx_vsubi_bu
    {loongarch_lsx_vsubi_du, 74725}, // __builtin_lsx_vsubi_du
    {loongarch_lsx_vsubi_hu, 74737}, // __builtin_lsx_vsubi_hu
    {loongarch_lsx_vsubi_wu, 74749}, // __builtin_lsx_vsubi_wu
    {loongarch_lsx_vsubwev_d_w, 74761}, // __builtin_lsx_vsubwev_d_w
    {loongarch_lsx_vsubwev_d_wu, 74776}, // __builtin_lsx_vsubwev_d_wu
    {loongarch_lsx_vsubwev_h_b, 74792}, // __builtin_lsx_vsubwev_h_b
    {loongarch_lsx_vsubwev_h_bu, 74807}, // __builtin_lsx_vsubwev_h_bu
    {loongarch_lsx_vsubwev_q_d, 74823}, // __builtin_lsx_vsubwev_q_d
    {loongarch_lsx_vsubwev_q_du, 74838}, // __builtin_lsx_vsubwev_q_du
    {loongarch_lsx_vsubwev_w_h, 74854}, // __builtin_lsx_vsubwev_w_h
    {loongarch_lsx_vsubwev_w_hu, 74869}, // __builtin_lsx_vsubwev_w_hu
    {loongarch_lsx_vsubwod_d_w, 74885}, // __builtin_lsx_vsubwod_d_w
    {loongarch_lsx_vsubwod_d_wu, 74900}, // __builtin_lsx_vsubwod_d_wu
    {loongarch_lsx_vsubwod_h_b, 74916}, // __builtin_lsx_vsubwod_h_b
    {loongarch_lsx_vsubwod_h_bu, 74931}, // __builtin_lsx_vsubwod_h_bu
    {loongarch_lsx_vsubwod_q_d, 74947}, // __builtin_lsx_vsubwod_q_d
    {loongarch_lsx_vsubwod_q_du, 74962}, // __builtin_lsx_vsubwod_q_du
    {loongarch_lsx_vsubwod_w_h, 74978}, // __builtin_lsx_vsubwod_w_h
    {loongarch_lsx_vsubwod_w_hu, 74993}, // __builtin_lsx_vsubwod_w_hu
    {loongarch_lsx_vxor_v, 75009}, // __builtin_lsx_vxor_v
    {loongarch_lsx_vxori_b, 75019}, // __builtin_lsx_vxori_b
  }; // loongarchNames

  // Builtins for mips.
  static constexpr BuiltinEntry mipsNames[] = {
    {mips_absq_s_ph, 75030}, // __builtin_mips_absq_s_ph
    {mips_absq_s_qb, 75044}, // __builtin_mips_absq_s_qb
    {mips_absq_s_w, 75058}, // __builtin_mips_absq_s_w
    {mips_addq_ph, 75071}, // __builtin_mips_addq_ph
    {mips_addq_s_ph, 75083}, // __builtin_mips_addq_s_ph
    {mips_addq_s_w, 75097}, // __builtin_mips_addq_s_w
    {mips_addqh_ph, 75110}, // __builtin_mips_addqh_ph
    {mips_addqh_r_ph, 75123}, // __builtin_mips_addqh_r_ph
    {mips_addqh_r_w, 75138}, // __builtin_mips_addqh_r_w
    {mips_addqh_w, 75152}, // __builtin_mips_addqh_w
    {mips_addsc, 75164}, // __builtin_mips_addsc
    {mips_addu_ph, 75174}, // __builtin_mips_addu_ph
    {mips_addu_qb, 75186}, // __builtin_mips_addu_qb
    {mips_addu_s_ph, 75198}, // __builtin_mips_addu_s_ph
    {mips_addu_s_qb, 75212}, // __builtin_mips_addu_s_qb
    {mips_adduh_qb, 75226}, // __builtin_mips_adduh_qb
    {mips_adduh_r_qb, 75239}, // __builtin_mips_adduh_r_qb
    {mips_addwc, 75254}, // __builtin_mips_addwc
    {mips_append, 75264}, // __builtin_mips_append
    {mips_balign, 75275}, // __builtin_mips_balign
    {mips_bitrev, 75286}, // __builtin_mips_bitrev
    {mips_bposge32, 75297}, // __builtin_mips_bposge32
    {mips_cmp_eq_ph, 75310}, // __builtin_mips_cmp_eq_ph
    {mips_cmp_le_ph, 75324}, // __builtin_mips_cmp_le_ph
    {mips_cmp_lt_ph, 75338}, // __builtin_mips_cmp_lt_ph
    {mips_cmpgdu_eq_qb, 75352}, // __builtin_mips_cmpgdu_eq_qb
    {mips_cmpgdu_le_qb, 75369}, // __builtin_mips_cmpgdu_le_qb
    {mips_cmpgdu_lt_qb, 75386}, // __builtin_mips_cmpgdu_lt_qb
    {mips_cmpgu_eq_qb, 75403}, // __builtin_mips_cmpgu_eq_qb
    {mips_cmpgu_le_qb, 75419}, // __builtin_mips_cmpgu_le_qb
    {mips_cmpgu_lt_qb, 75435}, // __builtin_mips_cmpgu_lt_qb
    {mips_cmpu_eq_qb, 75451}, // __builtin_mips_cmpu_eq_qb
    {mips_cmpu_le_qb, 75466}, // __builtin_mips_cmpu_le_qb
    {mips_cmpu_lt_qb, 75481}, // __builtin_mips_cmpu_lt_qb
    {mips_dlsa, 75496}, // __builtin_mips_dlsa
    {mips_dpa_w_ph, 75505}, // __builtin_mips_dpa_w_ph
    {mips_dpaq_s_w_ph, 75518}, // __builtin_mips_dpaq_s_w_ph
    {mips_dpaq_sa_l_w, 75534}, // __builtin_mips_dpaq_sa_l_w
    {mips_dpaqx_s_w_ph, 75550}, // __builtin_mips_dpaqx_s_w_ph
    {mips_dpaqx_sa_w_ph, 75567}, // __builtin_mips_dpaqx_sa_w_ph
    {mips_dpau_h_qbl, 75585}, // __builtin_mips_dpau_h_qbl
    {mips_dpau_h_qbr, 75600}, // __builtin_mips_dpau_h_qbr
    {mips_dpax_w_ph, 75615}, // __builtin_mips_dpax_w_ph
    {mips_dps_w_ph, 75629}, // __builtin_mips_dps_w_ph
    {mips_dpsq_s_w_ph, 75642}, // __builtin_mips_dpsq_s_w_ph
    {mips_dpsq_sa_l_w, 75658}, // __builtin_mips_dpsq_sa_l_w
    {mips_dpsqx_s_w_ph, 75674}, // __builtin_mips_dpsqx_s_w_ph
    {mips_dpsqx_sa_w_ph, 75691}, // __builtin_mips_dpsqx_sa_w_ph
    {mips_dpsu_h_qbl, 75709}, // __builtin_mips_dpsu_h_qbl
    {mips_dpsu_h_qbr, 75724}, // __builtin_mips_dpsu_h_qbr
    {mips_dpsx_w_ph, 75739}, // __builtin_mips_dpsx_w_ph
    {mips_extp, 75753}, // __builtin_mips_extp
    {mips_extpdp, 75762}, // __builtin_mips_extpdp
    {mips_extr_r_w, 75773}, // __builtin_mips_extr_r_w
    {mips_extr_rs_w, 75786}, // __builtin_mips_extr_rs_w
    {mips_extr_s_h, 75800}, // __builtin_mips_extr_s_h
    {mips_extr_w, 75813}, // __builtin_mips_extr_w
    {mips_insv, 75824}, // __builtin_mips_insv
    {mips_lbux, 75833}, // __builtin_mips_lbux
    {mips_lhx, 75842}, // __builtin_mips_lhx
    {mips_lsa, 75850}, // __builtin_mips_lsa
    {mips_lwx, 75858}, // __builtin_mips_lwx
    {mips_madd, 75866}, // __builtin_mips_madd
    {mips_maddu, 75875}, // __builtin_mips_maddu
    {mips_maq_s_w_phl, 75885}, // __builtin_mips_maq_s_w_phl
    {mips_maq_s_w_phr, 75901}, // __builtin_mips_maq_s_w_phr
    {mips_maq_sa_w_phl, 75917}, // __builtin_mips_maq_sa_w_phl
    {mips_maq_sa_w_phr, 75934}, // __builtin_mips_maq_sa_w_phr
    {mips_modsub, 75951}, // __builtin_mips_modsub
    {mips_msub, 75962}, // __builtin_mips_msub
    {mips_msubu, 75971}, // __builtin_mips_msubu
    {mips_mthlip, 75981}, // __builtin_mips_mthlip
    {mips_mul_ph, 75992}, // __builtin_mips_mul_ph
    {mips_mul_s_ph, 76003}, // __builtin_mips_mul_s_ph
    {mips_muleq_s_w_phl, 76016}, // __builtin_mips_muleq_s_w_phl
    {mips_muleq_s_w_phr, 76034}, // __builtin_mips_muleq_s_w_phr
    {mips_muleu_s_ph_qbl, 76052}, // __builtin_mips_muleu_s_ph_qbl
    {mips_muleu_s_ph_qbr, 76071}, // __builtin_mips_muleu_s_ph_qbr
    {mips_mulq_rs_ph, 76090}, // __builtin_mips_mulq_rs_ph
    {mips_mulq_rs_w, 76105}, // __builtin_mips_mulq_rs_w
    {mips_mulq_s_ph, 76119}, // __builtin_mips_mulq_s_ph
    {mips_mulq_s_w, 76133}, // __builtin_mips_mulq_s_w
    {mips_mulsa_w_ph, 76146}, // __builtin_mips_mulsa_w_ph
    {mips_mulsaq_s_w_ph, 76161}, // __builtin_mips_mulsaq_s_w_ph
    {mips_mult, 76179}, // __builtin_mips_mult
    {mips_multu, 76188}, // __builtin_mips_multu
    {mips_packrl_ph, 76198}, // __builtin_mips_packrl_ph
    {mips_pick_ph, 76212}, // __builtin_mips_pick_ph
    {mips_pick_qb, 76224}, // __builtin_mips_pick_qb
    {mips_preceq_w_phl, 76236}, // __builtin_mips_preceq_w_phl
    {mips_preceq_w_phr, 76253}, // __builtin_mips_preceq_w_phr
    {mips_precequ_ph_qbl, 76270}, // __builtin_mips_precequ_ph_qbl
    {mips_precequ_ph_qbla, 76289}, // __builtin_mips_precequ_ph_qbla
    {mips_precequ_ph_qbr, 76309}, // __builtin_mips_precequ_ph_qbr
    {mips_precequ_ph_qbra, 76328}, // __builtin_mips_precequ_ph_qbra
    {mips_preceu_ph_qbl, 76348}, // __builtin_mips_preceu_ph_qbl
    {mips_preceu_ph_qbla, 76366}, // __builtin_mips_preceu_ph_qbla
    {mips_preceu_ph_qbr, 76385}, // __builtin_mips_preceu_ph_qbr
    {mips_preceu_ph_qbra, 76403}, // __builtin_mips_preceu_ph_qbra
    {mips_precr_qb_ph, 76422}, // __builtin_mips_precr_qb_ph
    {mips_precr_sra_ph_w, 76438}, // __builtin_mips_precr_sra_ph_w
    {mips_precr_sra_r_ph_w, 76457}, // __builtin_mips_precr_sra_r_ph_w
    {mips_precrq_ph_w, 76478}, // __builtin_mips_precrq_ph_w
    {mips_precrq_qb_ph, 76494}, // __builtin_mips_precrq_qb_ph
    {mips_precrq_rs_ph_w, 76511}, // __builtin_mips_precrq_rs_ph_w
    {mips_precrqu_s_qb_ph, 76530}, // __builtin_mips_precrqu_s_qb_ph
    {mips_prepend, 76550}, // __builtin_mips_prepend
    {mips_raddu_w_qb, 76562}, // __builtin_mips_raddu_w_qb
    {mips_rddsp, 76577}, // __builtin_mips_rddsp
    {mips_repl_ph, 76587}, // __builtin_mips_repl_ph
    {mips_repl_qb, 76599}, // __builtin_mips_repl_qb
    {mips_shilo, 76611}, // __builtin_mips_shilo
    {mips_shll_ph, 76621}, // __builtin_mips_shll_ph
    {mips_shll_qb, 76633}, // __builtin_mips_shll_qb
    {mips_shll_s_ph, 76645}, // __builtin_mips_shll_s_ph
    {mips_shll_s_w, 76659}, // __builtin_mips_shll_s_w
    {mips_shra_ph, 76672}, // __builtin_mips_shra_ph
    {mips_shra_qb, 76684}, // __builtin_mips_shra_qb
    {mips_shra_r_ph, 76696}, // __builtin_mips_shra_r_ph
    {mips_shra_r_qb, 76710}, // __builtin_mips_shra_r_qb
    {mips_shra_r_w, 76724}, // __builtin_mips_shra_r_w
    {mips_shrl_ph, 76737}, // __builtin_mips_shrl_ph
    {mips_shrl_qb, 76749}, // __builtin_mips_shrl_qb
    {mips_subq_ph, 76761}, // __builtin_mips_subq_ph
    {mips_subq_s_ph, 76773}, // __builtin_mips_subq_s_ph
    {mips_subq_s_w, 76787}, // __builtin_mips_subq_s_w
    {mips_subqh_ph, 76800}, // __builtin_mips_subqh_ph
    {mips_subqh_r_ph, 76813}, // __builtin_mips_subqh_r_ph
    {mips_subqh_r_w, 76828}, // __builtin_mips_subqh_r_w
    {mips_subqh_w, 76842}, // __builtin_mips_subqh_w
    {mips_subu_ph, 76854}, // __builtin_mips_subu_ph
    {mips_subu_qb, 76866}, // __builtin_mips_subu_qb
    {mips_subu_s_ph, 76878}, // __builtin_mips_subu_s_ph
    {mips_subu_s_qb, 76892}, // __builtin_mips_subu_s_qb
    {mips_subuh_qb, 76906}, // __builtin_mips_subuh_qb
    {mips_subuh_r_qb, 76919}, // __builtin_mips_subuh_r_qb
    {mips_wrdsp, 76934}, // __builtin_mips_wrdsp
    {mips_add_a_b, 76944}, // __builtin_msa_add_a_b
    {mips_add_a_d, 76955}, // __builtin_msa_add_a_d
    {mips_add_a_h, 76966}, // __builtin_msa_add_a_h
    {mips_add_a_w, 76977}, // __builtin_msa_add_a_w
    {mips_adds_a_b, 76988}, // __builtin_msa_adds_a_b
    {mips_adds_a_d, 77000}, // __builtin_msa_adds_a_d
    {mips_adds_a_h, 77012}, // __builtin_msa_adds_a_h
    {mips_adds_a_w, 77024}, // __builtin_msa_adds_a_w
    {mips_adds_s_b, 77036}, // __builtin_msa_adds_s_b
    {mips_adds_s_d, 77048}, // __builtin_msa_adds_s_d
    {mips_adds_s_h, 77060}, // __builtin_msa_adds_s_h
    {mips_adds_s_w, 77072}, // __builtin_msa_adds_s_w
    {mips_adds_u_b, 77084}, // __builtin_msa_adds_u_b
    {mips_adds_u_d, 77096}, // __builtin_msa_adds_u_d
    {mips_adds_u_h, 77108}, // __builtin_msa_adds_u_h
    {mips_adds_u_w, 77120}, // __builtin_msa_adds_u_w
    {mips_addv_b, 77132}, // __builtin_msa_addv_b
    {mips_addv_d, 77142}, // __builtin_msa_addv_d
    {mips_addv_h, 77152}, // __builtin_msa_addv_h
    {mips_addv_w, 77162}, // __builtin_msa_addv_w
    {mips_addvi_b, 77172}, // __builtin_msa_addvi_b
    {mips_addvi_d, 77183}, // __builtin_msa_addvi_d
    {mips_addvi_h, 77194}, // __builtin_msa_addvi_h
    {mips_addvi_w, 77205}, // __builtin_msa_addvi_w
    {mips_and_v, 77216}, // __builtin_msa_and_v
    {mips_andi_b, 77225}, // __builtin_msa_andi_b
    {mips_asub_s_b, 77235}, // __builtin_msa_asub_s_b
    {mips_asub_s_d, 77247}, // __builtin_msa_asub_s_d
    {mips_asub_s_h, 77259}, // __builtin_msa_asub_s_h
    {mips_asub_s_w, 77271}, // __builtin_msa_asub_s_w
    {mips_asub_u_b, 77283}, // __builtin_msa_asub_u_b
    {mips_asub_u_d, 77295}, // __builtin_msa_asub_u_d
    {mips_asub_u_h, 77307}, // __builtin_msa_asub_u_h
    {mips_asub_u_w, 77319}, // __builtin_msa_asub_u_w
    {mips_ave_s_b, 77331}, // __builtin_msa_ave_s_b
    {mips_ave_s_d, 77342}, // __builtin_msa_ave_s_d
    {mips_ave_s_h, 77353}, // __builtin_msa_ave_s_h
    {mips_ave_s_w, 77364}, // __builtin_msa_ave_s_w
    {mips_ave_u_b, 77375}, // __builtin_msa_ave_u_b
    {mips_ave_u_d, 77386}, // __builtin_msa_ave_u_d
    {mips_ave_u_h, 77397}, // __builtin_msa_ave_u_h
    {mips_ave_u_w, 77408}, // __builtin_msa_ave_u_w
    {mips_aver_s_b, 77419}, // __builtin_msa_aver_s_b
    {mips_aver_s_d, 77431}, // __builtin_msa_aver_s_d
    {mips_aver_s_h, 77443}, // __builtin_msa_aver_s_h
    {mips_aver_s_w, 77455}, // __builtin_msa_aver_s_w
    {mips_aver_u_b, 77467}, // __builtin_msa_aver_u_b
    {mips_aver_u_d, 77479}, // __builtin_msa_aver_u_d
    {mips_aver_u_h, 77491}, // __builtin_msa_aver_u_h
    {mips_aver_u_w, 77503}, // __builtin_msa_aver_u_w
    {mips_bclr_b, 77515}, // __builtin_msa_bclr_b
    {mips_bclr_d, 77525}, // __builtin_msa_bclr_d
    {mips_bclr_h, 77535}, // __builtin_msa_bclr_h
    {mips_bclr_w, 77545}, // __builtin_msa_bclr_w
    {mips_bclri_b, 77555}, // __builtin_msa_bclri_b
    {mips_bclri_d, 77566}, // __builtin_msa_bclri_d
    {mips_bclri_h, 77577}, // __builtin_msa_bclri_h
    {mips_bclri_w, 77588}, // __builtin_msa_bclri_w
    {mips_binsl_b, 77599}, // __builtin_msa_binsl_b
    {mips_binsl_d, 77610}, // __builtin_msa_binsl_d
    {mips_binsl_h, 77621}, // __builtin_msa_binsl_h
    {mips_binsl_w, 77632}, // __builtin_msa_binsl_w
    {mips_binsli_b, 77643}, // __builtin_msa_binsli_b
    {mips_binsli_d, 77655}, // __builtin_msa_binsli_d
    {mips_binsli_h, 77667}, // __builtin_msa_binsli_h
    {mips_binsli_w, 77679}, // __builtin_msa_binsli_w
    {mips_binsr_b, 77691}, // __builtin_msa_binsr_b
    {mips_binsr_d, 77702}, // __builtin_msa_binsr_d
    {mips_binsr_h, 77713}, // __builtin_msa_binsr_h
    {mips_binsr_w, 77724}, // __builtin_msa_binsr_w
    {mips_binsri_b, 77735}, // __builtin_msa_binsri_b
    {mips_binsri_d, 77747}, // __builtin_msa_binsri_d
    {mips_binsri_h, 77759}, // __builtin_msa_binsri_h
    {mips_binsri_w, 77771}, // __builtin_msa_binsri_w
    {mips_bmnz_v, 77783}, // __builtin_msa_bmnz_v
    {mips_bmnzi_b, 77793}, // __builtin_msa_bmnzi_b
    {mips_bmz_v, 77804}, // __builtin_msa_bmz_v
    {mips_bmzi_b, 77813}, // __builtin_msa_bmzi_b
    {mips_bneg_b, 77823}, // __builtin_msa_bneg_b
    {mips_bneg_d, 77833}, // __builtin_msa_bneg_d
    {mips_bneg_h, 77843}, // __builtin_msa_bneg_h
    {mips_bneg_w, 77853}, // __builtin_msa_bneg_w
    {mips_bnegi_b, 77863}, // __builtin_msa_bnegi_b
    {mips_bnegi_d, 77874}, // __builtin_msa_bnegi_d
    {mips_bnegi_h, 77885}, // __builtin_msa_bnegi_h
    {mips_bnegi_w, 77896}, // __builtin_msa_bnegi_w
    {mips_bnz_b, 77907}, // __builtin_msa_bnz_b
    {mips_bnz_d, 77916}, // __builtin_msa_bnz_d
    {mips_bnz_h, 77925}, // __builtin_msa_bnz_h
    {mips_bnz_v, 77934}, // __builtin_msa_bnz_v
    {mips_bnz_w, 77943}, // __builtin_msa_bnz_w
    {mips_bsel_v, 77952}, // __builtin_msa_bsel_v
    {mips_bseli_b, 77962}, // __builtin_msa_bseli_b
    {mips_bset_b, 77973}, // __builtin_msa_bset_b
    {mips_bset_d, 77983}, // __builtin_msa_bset_d
    {mips_bset_h, 77993}, // __builtin_msa_bset_h
    {mips_bset_w, 78003}, // __builtin_msa_bset_w
    {mips_bseti_b, 78013}, // __builtin_msa_bseti_b
    {mips_bseti_d, 78024}, // __builtin_msa_bseti_d
    {mips_bseti_h, 78035}, // __builtin_msa_bseti_h
    {mips_bseti_w, 78046}, // __builtin_msa_bseti_w
    {mips_bz_b, 78057}, // __builtin_msa_bz_b
    {mips_bz_d, 78065}, // __builtin_msa_bz_d
    {mips_bz_h, 78073}, // __builtin_msa_bz_h
    {mips_bz_v, 78081}, // __builtin_msa_bz_v
    {mips_bz_w, 78089}, // __builtin_msa_bz_w
    {mips_ceq_b, 78097}, // __builtin_msa_ceq_b
    {mips_ceq_d, 78106}, // __builtin_msa_ceq_d
    {mips_ceq_h, 78115}, // __builtin_msa_ceq_h
    {mips_ceq_w, 78124}, // __builtin_msa_ceq_w
    {mips_ceqi_b, 78133}, // __builtin_msa_ceqi_b
    {mips_ceqi_d, 78143}, // __builtin_msa_ceqi_d
    {mips_ceqi_h, 78153}, // __builtin_msa_ceqi_h
    {mips_ceqi_w, 78163}, // __builtin_msa_ceqi_w
    {mips_cfcmsa, 78173}, // __builtin_msa_cfcmsa
    {mips_cle_s_b, 78183}, // __builtin_msa_cle_s_b
    {mips_cle_s_d, 78194}, // __builtin_msa_cle_s_d
    {mips_cle_s_h, 78205}, // __builtin_msa_cle_s_h
    {mips_cle_s_w, 78216}, // __builtin_msa_cle_s_w
    {mips_cle_u_b, 78227}, // __builtin_msa_cle_u_b
    {mips_cle_u_d, 78238}, // __builtin_msa_cle_u_d
    {mips_cle_u_h, 78249}, // __builtin_msa_cle_u_h
    {mips_cle_u_w, 78260}, // __builtin_msa_cle_u_w
    {mips_clei_s_b, 78271}, // __builtin_msa_clei_s_b
    {mips_clei_s_d, 78283}, // __builtin_msa_clei_s_d
    {mips_clei_s_h, 78295}, // __builtin_msa_clei_s_h
    {mips_clei_s_w, 78307}, // __builtin_msa_clei_s_w
    {mips_clei_u_b, 78319}, // __builtin_msa_clei_u_b
    {mips_clei_u_d, 78331}, // __builtin_msa_clei_u_d
    {mips_clei_u_h, 78343}, // __builtin_msa_clei_u_h
    {mips_clei_u_w, 78355}, // __builtin_msa_clei_u_w
    {mips_clt_s_b, 78367}, // __builtin_msa_clt_s_b
    {mips_clt_s_d, 78378}, // __builtin_msa_clt_s_d
    {mips_clt_s_h, 78389}, // __builtin_msa_clt_s_h
    {mips_clt_s_w, 78400}, // __builtin_msa_clt_s_w
    {mips_clt_u_b, 78411}, // __builtin_msa_clt_u_b
    {mips_clt_u_d, 78422}, // __builtin_msa_clt_u_d
    {mips_clt_u_h, 78433}, // __builtin_msa_clt_u_h
    {mips_clt_u_w, 78444}, // __builtin_msa_clt_u_w
    {mips_clti_s_b, 78455}, // __builtin_msa_clti_s_b
    {mips_clti_s_d, 78467}, // __builtin_msa_clti_s_d
    {mips_clti_s_h, 78479}, // __builtin_msa_clti_s_h
    {mips_clti_s_w, 78491}, // __builtin_msa_clti_s_w
    {mips_clti_u_b, 78503}, // __builtin_msa_clti_u_b
    {mips_clti_u_d, 78515}, // __builtin_msa_clti_u_d
    {mips_clti_u_h, 78527}, // __builtin_msa_clti_u_h
    {mips_clti_u_w, 78539}, // __builtin_msa_clti_u_w
    {mips_copy_s_b, 78551}, // __builtin_msa_copy_s_b
    {mips_copy_s_d, 78563}, // __builtin_msa_copy_s_d
    {mips_copy_s_h, 78575}, // __builtin_msa_copy_s_h
    {mips_copy_s_w, 78587}, // __builtin_msa_copy_s_w
    {mips_copy_u_b, 78599}, // __builtin_msa_copy_u_b
    {mips_copy_u_d, 78611}, // __builtin_msa_copy_u_d
    {mips_copy_u_h, 78623}, // __builtin_msa_copy_u_h
    {mips_copy_u_w, 78635}, // __builtin_msa_copy_u_w
    {mips_ctcmsa, 78647}, // __builtin_msa_ctcmsa
    {mips_div_s_b, 78657}, // __builtin_msa_div_s_b
    {mips_div_s_d, 78668}, // __builtin_msa_div_s_d
    {mips_div_s_h, 78679}, // __builtin_msa_div_s_h
    {mips_div_s_w, 78690}, // __builtin_msa_div_s_w
    {mips_div_u_b, 78701}, // __builtin_msa_div_u_b
    {mips_div_u_d, 78712}, // __builtin_msa_div_u_d
    {mips_div_u_h, 78723}, // __builtin_msa_div_u_h
    {mips_div_u_w, 78734}, // __builtin_msa_div_u_w
    {mips_dotp_s_d, 78745}, // __builtin_msa_dotp_s_d
    {mips_dotp_s_h, 78757}, // __builtin_msa_dotp_s_h
    {mips_dotp_s_w, 78769}, // __builtin_msa_dotp_s_w
    {mips_dotp_u_d, 78781}, // __builtin_msa_dotp_u_d
    {mips_dotp_u_h, 78793}, // __builtin_msa_dotp_u_h
    {mips_dotp_u_w, 78805}, // __builtin_msa_dotp_u_w
    {mips_dpadd_s_d, 78817}, // __builtin_msa_dpadd_s_d
    {mips_dpadd_s_h, 78830}, // __builtin_msa_dpadd_s_h
    {mips_dpadd_s_w, 78843}, // __builtin_msa_dpadd_s_w
    {mips_dpadd_u_d, 78856}, // __builtin_msa_dpadd_u_d
    {mips_dpadd_u_h, 78869}, // __builtin_msa_dpadd_u_h
    {mips_dpadd_u_w, 78882}, // __builtin_msa_dpadd_u_w
    {mips_dpsub_s_d, 78895}, // __builtin_msa_dpsub_s_d
    {mips_dpsub_s_h, 78908}, // __builtin_msa_dpsub_s_h
    {mips_dpsub_s_w, 78921}, // __builtin_msa_dpsub_s_w
    {mips_dpsub_u_d, 78934}, // __builtin_msa_dpsub_u_d
    {mips_dpsub_u_h, 78947}, // __builtin_msa_dpsub_u_h
    {mips_dpsub_u_w, 78960}, // __builtin_msa_dpsub_u_w
    {mips_fadd_d, 78973}, // __builtin_msa_fadd_d
    {mips_fadd_w, 78983}, // __builtin_msa_fadd_w
    {mips_fcaf_d, 78993}, // __builtin_msa_fcaf_d
    {mips_fcaf_w, 79003}, // __builtin_msa_fcaf_w
    {mips_fceq_d, 79013}, // __builtin_msa_fceq_d
    {mips_fceq_w, 79023}, // __builtin_msa_fceq_w
    {mips_fclass_d, 79033}, // __builtin_msa_fclass_d
    {mips_fclass_w, 79045}, // __builtin_msa_fclass_w
    {mips_fcle_d, 79057}, // __builtin_msa_fcle_d
    {mips_fcle_w, 79067}, // __builtin_msa_fcle_w
    {mips_fclt_d, 79077}, // __builtin_msa_fclt_d
    {mips_fclt_w, 79087}, // __builtin_msa_fclt_w
    {mips_fcne_d, 79097}, // __builtin_msa_fcne_d
    {mips_fcne_w, 79107}, // __builtin_msa_fcne_w
    {mips_fcor_d, 79117}, // __builtin_msa_fcor_d
    {mips_fcor_w, 79127}, // __builtin_msa_fcor_w
    {mips_fcueq_d, 79137}, // __builtin_msa_fcueq_d
    {mips_fcueq_w, 79148}, // __builtin_msa_fcueq_w
    {mips_fcule_d, 79159}, // __builtin_msa_fcule_d
    {mips_fcule_w, 79170}, // __builtin_msa_fcule_w
    {mips_fcult_d, 79181}, // __builtin_msa_fcult_d
    {mips_fcult_w, 79192}, // __builtin_msa_fcult_w
    {mips_fcun_d, 79203}, // __builtin_msa_fcun_d
    {mips_fcun_w, 79213}, // __builtin_msa_fcun_w
    {mips_fcune_d, 79223}, // __builtin_msa_fcune_d
    {mips_fcune_w, 79234}, // __builtin_msa_fcune_w
    {mips_fdiv_d, 79245}, // __builtin_msa_fdiv_d
    {mips_fdiv_w, 79255}, // __builtin_msa_fdiv_w
    {mips_fexdo_h, 79265}, // __builtin_msa_fexdo_h
    {mips_fexdo_w, 79276}, // __builtin_msa_fexdo_w
    {mips_fexp2_d, 79287}, // __builtin_msa_fexp2_d
    {mips_fexp2_w, 79298}, // __builtin_msa_fexp2_w
    {mips_fexupl_d, 79309}, // __builtin_msa_fexupl_d
    {mips_fexupl_w, 79321}, // __builtin_msa_fexupl_w
    {mips_fexupr_d, 79333}, // __builtin_msa_fexupr_d
    {mips_fexupr_w, 79345}, // __builtin_msa_fexupr_w
    {mips_ffint_s_d, 79357}, // __builtin_msa_ffint_s_d
    {mips_ffint_s_w, 79370}, // __builtin_msa_ffint_s_w
    {mips_ffint_u_d, 79383}, // __builtin_msa_ffint_u_d
    {mips_ffint_u_w, 79396}, // __builtin_msa_ffint_u_w
    {mips_ffql_d, 79409}, // __builtin_msa_ffql_d
    {mips_ffql_w, 79419}, // __builtin_msa_ffql_w
    {mips_ffqr_d, 79429}, // __builtin_msa_ffqr_d
    {mips_ffqr_w, 79439}, // __builtin_msa_ffqr_w
    {mips_fill_b, 79449}, // __builtin_msa_fill_b
    {mips_fill_d, 79459}, // __builtin_msa_fill_d
    {mips_fill_h, 79469}, // __builtin_msa_fill_h
    {mips_fill_w, 79479}, // __builtin_msa_fill_w
    {mips_flog2_d, 79489}, // __builtin_msa_flog2_d
    {mips_flog2_w, 79500}, // __builtin_msa_flog2_w
    {mips_fmadd_d, 79511}, // __builtin_msa_fmadd_d
    {mips_fmadd_w, 79522}, // __builtin_msa_fmadd_w
    {mips_fmax_a_d, 79533}, // __builtin_msa_fmax_a_d
    {mips_fmax_a_w, 79545}, // __builtin_msa_fmax_a_w
    {mips_fmax_d, 79557}, // __builtin_msa_fmax_d
    {mips_fmax_w, 79567}, // __builtin_msa_fmax_w
    {mips_fmin_a_d, 79577}, // __builtin_msa_fmin_a_d
    {mips_fmin_a_w, 79589}, // __builtin_msa_fmin_a_w
    {mips_fmin_d, 79601}, // __builtin_msa_fmin_d
    {mips_fmin_w, 79611}, // __builtin_msa_fmin_w
    {mips_fmsub_d, 79621}, // __builtin_msa_fmsub_d
    {mips_fmsub_w, 79632}, // __builtin_msa_fmsub_w
    {mips_fmul_d, 79643}, // __builtin_msa_fmul_d
    {mips_fmul_w, 79653}, // __builtin_msa_fmul_w
    {mips_frcp_d, 79663}, // __builtin_msa_frcp_d
    {mips_frcp_w, 79673}, // __builtin_msa_frcp_w
    {mips_frint_d, 79683}, // __builtin_msa_frint_d
    {mips_frint_w, 79694}, // __builtin_msa_frint_w
    {mips_frsqrt_d, 79705}, // __builtin_msa_frsqrt_d
    {mips_frsqrt_w, 79717}, // __builtin_msa_frsqrt_w
    {mips_fsaf_d, 79729}, // __builtin_msa_fsaf_d
    {mips_fsaf_w, 79739}, // __builtin_msa_fsaf_w
    {mips_fseq_d, 79749}, // __builtin_msa_fseq_d
    {mips_fseq_w, 79759}, // __builtin_msa_fseq_w
    {mips_fsle_d, 79769}, // __builtin_msa_fsle_d
    {mips_fsle_w, 79779}, // __builtin_msa_fsle_w
    {mips_fslt_d, 79789}, // __builtin_msa_fslt_d
    {mips_fslt_w, 79799}, // __builtin_msa_fslt_w
    {mips_fsne_d, 79809}, // __builtin_msa_fsne_d
    {mips_fsne_w, 79819}, // __builtin_msa_fsne_w
    {mips_fsor_d, 79829}, // __builtin_msa_fsor_d
    {mips_fsor_w, 79839}, // __builtin_msa_fsor_w
    {mips_fsqrt_d, 79849}, // __builtin_msa_fsqrt_d
    {mips_fsqrt_w, 79860}, // __builtin_msa_fsqrt_w
    {mips_fsub_d, 79871}, // __builtin_msa_fsub_d
    {mips_fsub_w, 79881}, // __builtin_msa_fsub_w
    {mips_fsueq_d, 79891}, // __builtin_msa_fsueq_d
    {mips_fsueq_w, 79902}, // __builtin_msa_fsueq_w
    {mips_fsule_d, 79913}, // __builtin_msa_fsule_d
    {mips_fsule_w, 79924}, // __builtin_msa_fsule_w
    {mips_fsult_d, 79935}, // __builtin_msa_fsult_d
    {mips_fsult_w, 79946}, // __builtin_msa_fsult_w
    {mips_fsun_d, 79957}, // __builtin_msa_fsun_d
    {mips_fsun_w, 79967}, // __builtin_msa_fsun_w
    {mips_fsune_d, 79977}, // __builtin_msa_fsune_d
    {mips_fsune_w, 79988}, // __builtin_msa_fsune_w
    {mips_ftint_s_d, 79999}, // __builtin_msa_ftint_s_d
    {mips_ftint_s_w, 80012}, // __builtin_msa_ftint_s_w
    {mips_ftint_u_d, 80025}, // __builtin_msa_ftint_u_d
    {mips_ftint_u_w, 80038}, // __builtin_msa_ftint_u_w
    {mips_ftq_h, 80051}, // __builtin_msa_ftq_h
    {mips_ftq_w, 80060}, // __builtin_msa_ftq_w
    {mips_ftrunc_s_d, 80069}, // __builtin_msa_ftrunc_s_d
    {mips_ftrunc_s_w, 80083}, // __builtin_msa_ftrunc_s_w
    {mips_ftrunc_u_d, 80097}, // __builtin_msa_ftrunc_u_d
    {mips_ftrunc_u_w, 80111}, // __builtin_msa_ftrunc_u_w
    {mips_hadd_s_d, 80125}, // __builtin_msa_hadd_s_d
    {mips_hadd_s_h, 80137}, // __builtin_msa_hadd_s_h
    {mips_hadd_s_w, 80149}, // __builtin_msa_hadd_s_w
    {mips_hadd_u_d, 80161}, // __builtin_msa_hadd_u_d
    {mips_hadd_u_h, 80173}, // __builtin_msa_hadd_u_h
    {mips_hadd_u_w, 80185}, // __builtin_msa_hadd_u_w
    {mips_hsub_s_d, 80197}, // __builtin_msa_hsub_s_d
    {mips_hsub_s_h, 80209}, // __builtin_msa_hsub_s_h
    {mips_hsub_s_w, 80221}, // __builtin_msa_hsub_s_w
    {mips_hsub_u_d, 80233}, // __builtin_msa_hsub_u_d
    {mips_hsub_u_h, 80245}, // __builtin_msa_hsub_u_h
    {mips_hsub_u_w, 80257}, // __builtin_msa_hsub_u_w
    {mips_ilvev_b, 80269}, // __builtin_msa_ilvev_b
    {mips_ilvev_d, 80280}, // __builtin_msa_ilvev_d
    {mips_ilvev_h, 80291}, // __builtin_msa_ilvev_h
    {mips_ilvev_w, 80302}, // __builtin_msa_ilvev_w
    {mips_ilvl_b, 80313}, // __builtin_msa_ilvl_b
    {mips_ilvl_d, 80323}, // __builtin_msa_ilvl_d
    {mips_ilvl_h, 80333}, // __builtin_msa_ilvl_h
    {mips_ilvl_w, 80343}, // __builtin_msa_ilvl_w
    {mips_ilvod_b, 80353}, // __builtin_msa_ilvod_b
    {mips_ilvod_d, 80364}, // __builtin_msa_ilvod_d
    {mips_ilvod_h, 80375}, // __builtin_msa_ilvod_h
    {mips_ilvod_w, 80386}, // __builtin_msa_ilvod_w
    {mips_ilvr_b, 80397}, // __builtin_msa_ilvr_b
    {mips_ilvr_d, 80407}, // __builtin_msa_ilvr_d
    {mips_ilvr_h, 80417}, // __builtin_msa_ilvr_h
    {mips_ilvr_w, 80427}, // __builtin_msa_ilvr_w
    {mips_insert_b, 80437}, // __builtin_msa_insert_b
    {mips_insert_d, 80449}, // __builtin_msa_insert_d
    {mips_insert_h, 80461}, // __builtin_msa_insert_h
    {mips_insert_w, 80473}, // __builtin_msa_insert_w
    {mips_insve_b, 80485}, // __builtin_msa_insve_b
    {mips_insve_d, 80496}, // __builtin_msa_insve_d
    {mips_insve_h, 80507}, // __builtin_msa_insve_h
    {mips_insve_w, 80518}, // __builtin_msa_insve_w
    {mips_ld_b, 80529}, // __builtin_msa_ld_b
    {mips_ld_d, 80537}, // __builtin_msa_ld_d
    {mips_ld_h, 80545}, // __builtin_msa_ld_h
    {mips_ld_w, 80553}, // __builtin_msa_ld_w
    {mips_ldi_b, 80561}, // __builtin_msa_ldi_b
    {mips_ldi_d, 80570}, // __builtin_msa_ldi_d
    {mips_ldi_h, 80579}, // __builtin_msa_ldi_h
    {mips_ldi_w, 80588}, // __builtin_msa_ldi_w
    {mips_ldr_d, 80597}, // __builtin_msa_ldr_d
    {mips_ldr_w, 80606}, // __builtin_msa_ldr_w
    {mips_madd_q_h, 80615}, // __builtin_msa_madd_q_h
    {mips_madd_q_w, 80627}, // __builtin_msa_madd_q_w
    {mips_maddr_q_h, 80639}, // __builtin_msa_maddr_q_h
    {mips_maddr_q_w, 80652}, // __builtin_msa_maddr_q_w
    {mips_maddv_b, 80665}, // __builtin_msa_maddv_b
    {mips_maddv_d, 80676}, // __builtin_msa_maddv_d
    {mips_maddv_h, 80687}, // __builtin_msa_maddv_h
    {mips_maddv_w, 80698}, // __builtin_msa_maddv_w
    {mips_max_a_b, 80709}, // __builtin_msa_max_a_b
    {mips_max_a_d, 80720}, // __builtin_msa_max_a_d
    {mips_max_a_h, 80731}, // __builtin_msa_max_a_h
    {mips_max_a_w, 80742}, // __builtin_msa_max_a_w
    {mips_max_s_b, 80753}, // __builtin_msa_max_s_b
    {mips_max_s_d, 80764}, // __builtin_msa_max_s_d
    {mips_max_s_h, 80775}, // __builtin_msa_max_s_h
    {mips_max_s_w, 80786}, // __builtin_msa_max_s_w
    {mips_max_u_b, 80797}, // __builtin_msa_max_u_b
    {mips_max_u_d, 80808}, // __builtin_msa_max_u_d
    {mips_max_u_h, 80819}, // __builtin_msa_max_u_h
    {mips_max_u_w, 80830}, // __builtin_msa_max_u_w
    {mips_maxi_s_b, 80841}, // __builtin_msa_maxi_s_b
    {mips_maxi_s_d, 80853}, // __builtin_msa_maxi_s_d
    {mips_maxi_s_h, 80865}, // __builtin_msa_maxi_s_h
    {mips_maxi_s_w, 80877}, // __builtin_msa_maxi_s_w
    {mips_maxi_u_b, 80889}, // __builtin_msa_maxi_u_b
    {mips_maxi_u_d, 80901}, // __builtin_msa_maxi_u_d
    {mips_maxi_u_h, 80913}, // __builtin_msa_maxi_u_h
    {mips_maxi_u_w, 80925}, // __builtin_msa_maxi_u_w
    {mips_min_a_b, 80937}, // __builtin_msa_min_a_b
    {mips_min_a_d, 80948}, // __builtin_msa_min_a_d
    {mips_min_a_h, 80959}, // __builtin_msa_min_a_h
    {mips_min_a_w, 80970}, // __builtin_msa_min_a_w
    {mips_min_s_b, 80981}, // __builtin_msa_min_s_b
    {mips_min_s_d, 80992}, // __builtin_msa_min_s_d
    {mips_min_s_h, 81003}, // __builtin_msa_min_s_h
    {mips_min_s_w, 81014}, // __builtin_msa_min_s_w
    {mips_min_u_b, 81025}, // __builtin_msa_min_u_b
    {mips_min_u_d, 81036}, // __builtin_msa_min_u_d
    {mips_min_u_h, 81047}, // __builtin_msa_min_u_h
    {mips_min_u_w, 81058}, // __builtin_msa_min_u_w
    {mips_mini_s_b, 81069}, // __builtin_msa_mini_s_b
    {mips_mini_s_d, 81081}, // __builtin_msa_mini_s_d
    {mips_mini_s_h, 81093}, // __builtin_msa_mini_s_h
    {mips_mini_s_w, 81105}, // __builtin_msa_mini_s_w
    {mips_mini_u_b, 81117}, // __builtin_msa_mini_u_b
    {mips_mini_u_d, 81129}, // __builtin_msa_mini_u_d
    {mips_mini_u_h, 81141}, // __builtin_msa_mini_u_h
    {mips_mini_u_w, 81153}, // __builtin_msa_mini_u_w
    {mips_mod_s_b, 81165}, // __builtin_msa_mod_s_b
    {mips_mod_s_d, 81176}, // __builtin_msa_mod_s_d
    {mips_mod_s_h, 81187}, // __builtin_msa_mod_s_h
    {mips_mod_s_w, 81198}, // __builtin_msa_mod_s_w
    {mips_mod_u_b, 81209}, // __builtin_msa_mod_u_b
    {mips_mod_u_d, 81220}, // __builtin_msa_mod_u_d
    {mips_mod_u_h, 81231}, // __builtin_msa_mod_u_h
    {mips_mod_u_w, 81242}, // __builtin_msa_mod_u_w
    {mips_move_v, 81253}, // __builtin_msa_move_v
    {mips_msub_q_h, 81263}, // __builtin_msa_msub_q_h
    {mips_msub_q_w, 81275}, // __builtin_msa_msub_q_w
    {mips_msubr_q_h, 81287}, // __builtin_msa_msubr_q_h
    {mips_msubr_q_w, 81300}, // __builtin_msa_msubr_q_w
    {mips_msubv_b, 81313}, // __builtin_msa_msubv_b
    {mips_msubv_d, 81324}, // __builtin_msa_msubv_d
    {mips_msubv_h, 81335}, // __builtin_msa_msubv_h
    {mips_msubv_w, 81346}, // __builtin_msa_msubv_w
    {mips_mul_q_h, 81357}, // __builtin_msa_mul_q_h
    {mips_mul_q_w, 81368}, // __builtin_msa_mul_q_w
    {mips_mulr_q_h, 81379}, // __builtin_msa_mulr_q_h
    {mips_mulr_q_w, 81391}, // __builtin_msa_mulr_q_w
    {mips_mulv_b, 81403}, // __builtin_msa_mulv_b
    {mips_mulv_d, 81413}, // __builtin_msa_mulv_d
    {mips_mulv_h, 81423}, // __builtin_msa_mulv_h
    {mips_mulv_w, 81433}, // __builtin_msa_mulv_w
    {mips_nloc_b, 81443}, // __builtin_msa_nloc_b
    {mips_nloc_d, 81453}, // __builtin_msa_nloc_d
    {mips_nloc_h, 81463}, // __builtin_msa_nloc_h
    {mips_nloc_w, 81473}, // __builtin_msa_nloc_w
    {mips_nlzc_b, 81483}, // __builtin_msa_nlzc_b
    {mips_nlzc_d, 81493}, // __builtin_msa_nlzc_d
    {mips_nlzc_h, 81503}, // __builtin_msa_nlzc_h
    {mips_nlzc_w, 81513}, // __builtin_msa_nlzc_w
    {mips_nor_v, 81523}, // __builtin_msa_nor_v
    {mips_nori_b, 81532}, // __builtin_msa_nori_b
    {mips_or_v, 81542}, // __builtin_msa_or_v
    {mips_ori_b, 81550}, // __builtin_msa_ori_b
    {mips_pckev_b, 81559}, // __builtin_msa_pckev_b
    {mips_pckev_d, 81570}, // __builtin_msa_pckev_d
    {mips_pckev_h, 81581}, // __builtin_msa_pckev_h
    {mips_pckev_w, 81592}, // __builtin_msa_pckev_w
    {mips_pckod_b, 81603}, // __builtin_msa_pckod_b
    {mips_pckod_d, 81614}, // __builtin_msa_pckod_d
    {mips_pckod_h, 81625}, // __builtin_msa_pckod_h
    {mips_pckod_w, 81636}, // __builtin_msa_pckod_w
    {mips_pcnt_b, 81647}, // __builtin_msa_pcnt_b
    {mips_pcnt_d, 81657}, // __builtin_msa_pcnt_d
    {mips_pcnt_h, 81667}, // __builtin_msa_pcnt_h
    {mips_pcnt_w, 81677}, // __builtin_msa_pcnt_w
    {mips_sat_s_b, 81687}, // __builtin_msa_sat_s_b
    {mips_sat_s_d, 81698}, // __builtin_msa_sat_s_d
    {mips_sat_s_h, 81709}, // __builtin_msa_sat_s_h
    {mips_sat_s_w, 81720}, // __builtin_msa_sat_s_w
    {mips_sat_u_b, 81731}, // __builtin_msa_sat_u_b
    {mips_sat_u_d, 81742}, // __builtin_msa_sat_u_d
    {mips_sat_u_h, 81753}, // __builtin_msa_sat_u_h
    {mips_sat_u_w, 81764}, // __builtin_msa_sat_u_w
    {mips_shf_b, 81775}, // __builtin_msa_shf_b
    {mips_shf_h, 81784}, // __builtin_msa_shf_h
    {mips_shf_w, 81793}, // __builtin_msa_shf_w
    {mips_sld_b, 81802}, // __builtin_msa_sld_b
    {mips_sld_d, 81811}, // __builtin_msa_sld_d
    {mips_sld_h, 81820}, // __builtin_msa_sld_h
    {mips_sld_w, 81829}, // __builtin_msa_sld_w
    {mips_sldi_b, 81838}, // __builtin_msa_sldi_b
    {mips_sldi_d, 81848}, // __builtin_msa_sldi_d
    {mips_sldi_h, 81858}, // __builtin_msa_sldi_h
    {mips_sldi_w, 81868}, // __builtin_msa_sldi_w
    {mips_sll_b, 81878}, // __builtin_msa_sll_b
    {mips_sll_d, 81887}, // __builtin_msa_sll_d
    {mips_sll_h, 81896}, // __builtin_msa_sll_h
    {mips_sll_w, 81905}, // __builtin_msa_sll_w
    {mips_slli_b, 81914}, // __builtin_msa_slli_b
    {mips_slli_d, 81924}, // __builtin_msa_slli_d
    {mips_slli_h, 81934}, // __builtin_msa_slli_h
    {mips_slli_w, 81944}, // __builtin_msa_slli_w
    {mips_splat_b, 81954}, // __builtin_msa_splat_b
    {mips_splat_d, 81965}, // __builtin_msa_splat_d
    {mips_splat_h, 81976}, // __builtin_msa_splat_h
    {mips_splat_w, 81987}, // __builtin_msa_splat_w
    {mips_splati_b, 81998}, // __builtin_msa_splati_b
    {mips_splati_d, 82010}, // __builtin_msa_splati_d
    {mips_splati_h, 82022}, // __builtin_msa_splati_h
    {mips_splati_w, 82034}, // __builtin_msa_splati_w
    {mips_sra_b, 82046}, // __builtin_msa_sra_b
    {mips_sra_d, 82055}, // __builtin_msa_sra_d
    {mips_sra_h, 82064}, // __builtin_msa_sra_h
    {mips_sra_w, 82073}, // __builtin_msa_sra_w
    {mips_srai_b, 82082}, // __builtin_msa_srai_b
    {mips_srai_d, 82092}, // __builtin_msa_srai_d
    {mips_srai_h, 82102}, // __builtin_msa_srai_h
    {mips_srai_w, 82112}, // __builtin_msa_srai_w
    {mips_srar_b, 82122}, // __builtin_msa_srar_b
    {mips_srar_d, 82132}, // __builtin_msa_srar_d
    {mips_srar_h, 82142}, // __builtin_msa_srar_h
    {mips_srar_w, 82152}, // __builtin_msa_srar_w
    {mips_srari_b, 82162}, // __builtin_msa_srari_b
    {mips_srari_d, 82173}, // __builtin_msa_srari_d
    {mips_srari_h, 82184}, // __builtin_msa_srari_h
    {mips_srari_w, 82195}, // __builtin_msa_srari_w
    {mips_srl_b, 82206}, // __builtin_msa_srl_b
    {mips_srl_d, 82215}, // __builtin_msa_srl_d
    {mips_srl_h, 82224}, // __builtin_msa_srl_h
    {mips_srl_w, 82233}, // __builtin_msa_srl_w
    {mips_srli_b, 82242}, // __builtin_msa_srli_b
    {mips_srli_d, 82252}, // __builtin_msa_srli_d
    {mips_srli_h, 82262}, // __builtin_msa_srli_h
    {mips_srli_w, 82272}, // __builtin_msa_srli_w
    {mips_srlr_b, 82282}, // __builtin_msa_srlr_b
    {mips_srlr_d, 82292}, // __builtin_msa_srlr_d
    {mips_srlr_h, 82302}, // __builtin_msa_srlr_h
    {mips_srlr_w, 82312}, // __builtin_msa_srlr_w
    {mips_srlri_b, 82322}, // __builtin_msa_srlri_b
    {mips_srlri_d, 82333}, // __builtin_msa_srlri_d
    {mips_srlri_h, 82344}, // __builtin_msa_srlri_h
    {mips_srlri_w, 82355}, // __builtin_msa_srlri_w
    {mips_st_b, 82366}, // __builtin_msa_st_b
    {mips_st_d, 82374}, // __builtin_msa_st_d
    {mips_st_h, 82382}, // __builtin_msa_st_h
    {mips_st_w, 82390}, // __builtin_msa_st_w
    {mips_str_d, 82398}, // __builtin_msa_str_d
    {mips_str_w, 82407}, // __builtin_msa_str_w
    {mips_subs_s_b, 82416}, // __builtin_msa_subs_s_b
    {mips_subs_s_d, 82428}, // __builtin_msa_subs_s_d
    {mips_subs_s_h, 82440}, // __builtin_msa_subs_s_h
    {mips_subs_s_w, 82452}, // __builtin_msa_subs_s_w
    {mips_subs_u_b, 82464}, // __builtin_msa_subs_u_b
    {mips_subs_u_d, 82476}, // __builtin_msa_subs_u_d
    {mips_subs_u_h, 82488}, // __builtin_msa_subs_u_h
    {mips_subs_u_w, 82500}, // __builtin_msa_subs_u_w
    {mips_subsus_u_b, 82512}, // __builtin_msa_subsus_u_b
    {mips_subsus_u_d, 82526}, // __builtin_msa_subsus_u_d
    {mips_subsus_u_h, 82540}, // __builtin_msa_subsus_u_h
    {mips_subsus_u_w, 82554}, // __builtin_msa_subsus_u_w
    {mips_subsuu_s_b, 82568}, // __builtin_msa_subsuu_s_b
    {mips_subsuu_s_d, 82582}, // __builtin_msa_subsuu_s_d
    {mips_subsuu_s_h, 82596}, // __builtin_msa_subsuu_s_h
    {mips_subsuu_s_w, 82610}, // __builtin_msa_subsuu_s_w
    {mips_subv_b, 82624}, // __builtin_msa_subv_b
    {mips_subv_d, 82634}, // __builtin_msa_subv_d
    {mips_subv_h, 82644}, // __builtin_msa_subv_h
    {mips_subv_w, 82654}, // __builtin_msa_subv_w
    {mips_subvi_b, 82664}, // __builtin_msa_subvi_b
    {mips_subvi_d, 82675}, // __builtin_msa_subvi_d
    {mips_subvi_h, 82686}, // __builtin_msa_subvi_h
    {mips_subvi_w, 82697}, // __builtin_msa_subvi_w
    {mips_vshf_b, 82708}, // __builtin_msa_vshf_b
    {mips_vshf_d, 82718}, // __builtin_msa_vshf_d
    {mips_vshf_h, 82728}, // __builtin_msa_vshf_h
    {mips_vshf_w, 82738}, // __builtin_msa_vshf_w
    {mips_xor_v, 82748}, // __builtin_msa_xor_v
    {mips_xori_b, 82757}, // __builtin_msa_xori_b
  }; // mipsNames

  // Builtins for nvvm.
  static constexpr BuiltinEntry nvvmNames[] = {
    {nvvm_activemask, 82767}, // __nvvm_activemask
    {nvvm_add_rm_d, 82778}, // __nvvm_add_rm_d
    {nvvm_add_rm_f, 82787}, // __nvvm_add_rm_f
    {nvvm_add_rm_ftz_f, 82796}, // __nvvm_add_rm_ftz_f
    {nvvm_add_rm_ftz_sat_f, 82809}, // __nvvm_add_rm_ftz_sat_f
    {nvvm_add_rm_sat_f, 82826}, // __nvvm_add_rm_sat_f
    {nvvm_add_rn_d, 82839}, // __nvvm_add_rn_d
    {nvvm_add_rn_f, 82848}, // __nvvm_add_rn_f
    {nvvm_add_rn_ftz_f, 82857}, // __nvvm_add_rn_ftz_f
    {nvvm_add_rn_ftz_sat_f, 82870}, // __nvvm_add_rn_ftz_sat_f
    {nvvm_add_rn_sat_f, 82887}, // __nvvm_add_rn_sat_f
    {nvvm_add_rp_d, 82900}, // __nvvm_add_rp_d
    {nvvm_add_rp_f, 82909}, // __nvvm_add_rp_f
    {nvvm_add_rp_ftz_f, 82918}, // __nvvm_add_rp_ftz_f
    {nvvm_add_rp_ftz_sat_f, 82931}, // __nvvm_add_rp_ftz_sat_f
    {nvvm_add_rp_sat_f, 82948}, // __nvvm_add_rp_sat_f
    {nvvm_add_rz_d, 82961}, // __nvvm_add_rz_d
    {nvvm_add_rz_f, 82970}, // __nvvm_add_rz_f
    {nvvm_add_rz_ftz_f, 82979}, // __nvvm_add_rz_ftz_f
    {nvvm_add_rz_ftz_sat_f, 82992}, // __nvvm_add_rz_ftz_sat_f
    {nvvm_add_rz_sat_f, 83009}, // __nvvm_add_rz_sat_f
    {nvvm_bar_warp_sync, 83022}, // __nvvm_bar_warp_sync
    {nvvm_bf16x2_to_ue8m0x2_rp, 83036}, // __nvvm_bf16x2_to_ue8m0x2_rp
    {nvvm_bf16x2_to_ue8m0x2_rp_satfinite, 83057}, // __nvvm_bf16x2_to_ue8m0x2_rp_satfinite
    {nvvm_bf16x2_to_ue8m0x2_rz, 83088}, // __nvvm_bf16x2_to_ue8m0x2_rz
    {nvvm_bf16x2_to_ue8m0x2_rz_satfinite, 83109}, // __nvvm_bf16x2_to_ue8m0x2_rz_satfinite
    {nvvm_bf2h_rn, 83140}, // __nvvm_bf2h_rn
    {nvvm_bf2h_rn_ftz, 83148}, // __nvvm_bf2h_rn_ftz
    {nvvm_ceil_d, 83160}, // __nvvm_ceil_d
    {nvvm_ceil_f, 83167}, // __nvvm_ceil_f
    {nvvm_ceil_ftz_f, 83174}, // __nvvm_ceil_ftz_f
    {nvvm_cos_approx_f, 83185}, // __nvvm_cos_approx_f
    {nvvm_cos_approx_ftz_f, 83198}, // __nvvm_cos_approx_ftz_f
    {nvvm_cp_async_commit_group, 83215}, // __nvvm_cp_async_commit_group
    {nvvm_cp_async_mbarrier_arrive, 83237}, // __nvvm_cp_async_mbarrier_arrive
    {nvvm_cp_async_mbarrier_arrive_noinc, 83262}, // __nvvm_cp_async_mbarrier_arrive_noinc
    {nvvm_cp_async_mbarrier_arrive_noinc_shared, 83293}, // __nvvm_cp_async_mbarrier_arrive_noinc_shared
    {nvvm_cp_async_mbarrier_arrive_shared, 83331}, // __nvvm_cp_async_mbarrier_arrive_shared
    {nvvm_cp_async_wait_all, 83363}, // __nvvm_cp_async_wait_all
    {nvvm_cp_async_wait_group, 83381}, // __nvvm_cp_async_wait_group
    {nvvm_d2f_rm, 83401}, // __nvvm_d2f_rm
    {nvvm_d2f_rm_ftz, 83408}, // __nvvm_d2f_rm_ftz
    {nvvm_d2f_rn, 83419}, // __nvvm_d2f_rn
    {nvvm_d2f_rn_ftz, 83426}, // __nvvm_d2f_rn_ftz
    {nvvm_d2f_rp, 83437}, // __nvvm_d2f_rp
    {nvvm_d2f_rp_ftz, 83444}, // __nvvm_d2f_rp_ftz
    {nvvm_d2f_rz, 83455}, // __nvvm_d2f_rz
    {nvvm_d2f_rz_ftz, 83462}, // __nvvm_d2f_rz_ftz
    {nvvm_d2i_hi, 83473}, // __nvvm_d2i_hi
    {nvvm_d2i_lo, 83480}, // __nvvm_d2i_lo
    {nvvm_d2i_rm, 83487}, // __nvvm_d2i_rm
    {nvvm_d2i_rn, 83494}, // __nvvm_d2i_rn
    {nvvm_d2i_rp, 83501}, // __nvvm_d2i_rp
    {nvvm_d2i_rz, 83508}, // __nvvm_d2i_rz
    {nvvm_d2ll_rm, 83515}, // __nvvm_d2ll_rm
    {nvvm_d2ll_rn, 83523}, // __nvvm_d2ll_rn
    {nvvm_d2ll_rp, 83531}, // __nvvm_d2ll_rp
    {nvvm_d2ll_rz, 83539}, // __nvvm_d2ll_rz
    {nvvm_d2ui_rm, 83547}, // __nvvm_d2ui_rm
    {nvvm_d2ui_rn, 83555}, // __nvvm_d2ui_rn
    {nvvm_d2ui_rp, 83563}, // __nvvm_d2ui_rp
    {nvvm_d2ui_rz, 83571}, // __nvvm_d2ui_rz
    {nvvm_d2ull_rm, 83579}, // __nvvm_d2ull_rm
    {nvvm_d2ull_rn, 83588}, // __nvvm_d2ull_rn
    {nvvm_d2ull_rp, 83597}, // __nvvm_d2ull_rp
    {nvvm_d2ull_rz, 83606}, // __nvvm_d2ull_rz
    {nvvm_div_approx_f, 83615}, // __nvvm_div_approx_f
    {nvvm_div_approx_ftz_f, 83628}, // __nvvm_div_approx_ftz_f
    {nvvm_div_full, 83645}, // __nvvm_div_full
    {nvvm_div_full_ftz, 83654}, // __nvvm_div_full_ftz
    {nvvm_div_rm_d, 83667}, // __nvvm_div_rm_d
    {nvvm_div_rm_f, 83676}, // __nvvm_div_rm_f
    {nvvm_div_rm_ftz_f, 83685}, // __nvvm_div_rm_ftz_f
    {nvvm_div_rn_d, 83698}, // __nvvm_div_rn_d
    {nvvm_div_rn_f, 83707}, // __nvvm_div_rn_f
    {nvvm_div_rn_ftz_f, 83716}, // __nvvm_div_rn_ftz_f
    {nvvm_div_rp_d, 83729}, // __nvvm_div_rp_d
    {nvvm_div_rp_f, 83738}, // __nvvm_div_rp_f
    {nvvm_div_rp_ftz_f, 83747}, // __nvvm_div_rp_ftz_f
    {nvvm_div_rz_d, 83760}, // __nvvm_div_rz_d
    {nvvm_div_rz_f, 83769}, // __nvvm_div_rz_f
    {nvvm_div_rz_ftz_f, 83778}, // __nvvm_div_rz_ftz_f
    {nvvm_e2m1x2_to_f16x2_rn, 83791}, // __nvvm_e2m1x2_to_f16x2_rn
    {nvvm_e2m1x2_to_f16x2_rn_relu, 83810}, // __nvvm_e2m1x2_to_f16x2_rn_relu
    {nvvm_e2m3x2_to_f16x2_rn, 83834}, // __nvvm_e2m3x2_to_f16x2_rn
    {nvvm_e2m3x2_to_f16x2_rn_relu, 83853}, // __nvvm_e2m3x2_to_f16x2_rn_relu
    {nvvm_e3m2x2_to_f16x2_rn, 83877}, // __nvvm_e3m2x2_to_f16x2_rn
    {nvvm_e3m2x2_to_f16x2_rn_relu, 83896}, // __nvvm_e3m2x2_to_f16x2_rn_relu
    {nvvm_e4m3x2_to_f16x2_rn, 83920}, // __nvvm_e4m3x2_to_f16x2_rn
    {nvvm_e4m3x2_to_f16x2_rn_relu, 83939}, // __nvvm_e4m3x2_to_f16x2_rn_relu
    {nvvm_e5m2x2_to_f16x2_rn, 83963}, // __nvvm_e5m2x2_to_f16x2_rn
    {nvvm_e5m2x2_to_f16x2_rn_relu, 83982}, // __nvvm_e5m2x2_to_f16x2_rn_relu
    {nvvm_exit, 84006}, // __nvvm_exit
    {nvvm_f16x2_to_e4m3x2_rn, 84011}, // __nvvm_f16x2_to_e4m3x2_rn
    {nvvm_f16x2_to_e4m3x2_rn_relu, 84030}, // __nvvm_f16x2_to_e4m3x2_rn_relu
    {nvvm_f16x2_to_e5m2x2_rn, 84054}, // __nvvm_f16x2_to_e5m2x2_rn
    {nvvm_f16x2_to_e5m2x2_rn_relu, 84073}, // __nvvm_f16x2_to_e5m2x2_rn_relu
    {nvvm_f2bf16_rn, 84097}, // __nvvm_f2bf16_rn
    {nvvm_f2bf16_rn_relu, 84107}, // __nvvm_f2bf16_rn_relu
    {nvvm_f2bf16_rn_relu_satfinite, 84122}, // __nvvm_f2bf16_rn_relu_satfinite
    {nvvm_f2bf16_rn_satfinite, 84147}, // __nvvm_f2bf16_rn_satfinite
    {nvvm_f2bf16_rz, 84167}, // __nvvm_f2bf16_rz
    {nvvm_f2bf16_rz_relu, 84177}, // __nvvm_f2bf16_rz_relu
    {nvvm_f2bf16_rz_relu_satfinite, 84192}, // __nvvm_f2bf16_rz_relu_satfinite
    {nvvm_f2bf16_rz_satfinite, 84217}, // __nvvm_f2bf16_rz_satfinite
    {nvvm_f2f16_rn, 84237}, // __nvvm_f2f16_rn
    {nvvm_f2f16_rn_relu, 84246}, // __nvvm_f2f16_rn_relu
    {nvvm_f2f16_rn_relu_satfinite, 84260}, // __nvvm_f2f16_rn_relu_satfinite
    {nvvm_f2f16_rn_satfinite, 84284}, // __nvvm_f2f16_rn_satfinite
    {nvvm_f2f16_rz, 84303}, // __nvvm_f2f16_rz
    {nvvm_f2f16_rz_relu, 84312}, // __nvvm_f2f16_rz_relu
    {nvvm_f2f16_rz_relu_satfinite, 84326}, // __nvvm_f2f16_rz_relu_satfinite
    {nvvm_f2f16_rz_satfinite, 84350}, // __nvvm_f2f16_rz_satfinite
    {nvvm_f2h_rn, 84369}, // __nvvm_f2h_rn
    {nvvm_f2h_rn_ftz, 84376}, // __nvvm_f2h_rn_ftz
    {nvvm_f2i_rm, 84387}, // __nvvm_f2i_rm
    {nvvm_f2i_rm_ftz, 84394}, // __nvvm_f2i_rm_ftz
    {nvvm_f2i_rn, 84405}, // __nvvm_f2i_rn
    {nvvm_f2i_rn_ftz, 84412}, // __nvvm_f2i_rn_ftz
    {nvvm_f2i_rp, 84423}, // __nvvm_f2i_rp
    {nvvm_f2i_rp_ftz, 84430}, // __nvvm_f2i_rp_ftz
    {nvvm_f2i_rz, 84441}, // __nvvm_f2i_rz
    {nvvm_f2i_rz_ftz, 84448}, // __nvvm_f2i_rz_ftz
    {nvvm_f2ll_rm, 84459}, // __nvvm_f2ll_rm
    {nvvm_f2ll_rm_ftz, 84467}, // __nvvm_f2ll_rm_ftz
    {nvvm_f2ll_rn, 84479}, // __nvvm_f2ll_rn
    {nvvm_f2ll_rn_ftz, 84487}, // __nvvm_f2ll_rn_ftz
    {nvvm_f2ll_rp, 84499}, // __nvvm_f2ll_rp
    {nvvm_f2ll_rp_ftz, 84507}, // __nvvm_f2ll_rp_ftz
    {nvvm_f2ll_rz, 84519}, // __nvvm_f2ll_rz
    {nvvm_f2ll_rz_ftz, 84527}, // __nvvm_f2ll_rz_ftz
    {nvvm_f2tf32_rn, 84539}, // __nvvm_f2tf32_rn
    {nvvm_f2tf32_rn_relu, 84549}, // __nvvm_f2tf32_rn_relu
    {nvvm_f2tf32_rn_relu_satfinite, 84564}, // __nvvm_f2tf32_rn_relu_satfinite
    {nvvm_f2tf32_rn_satfinite, 84589}, // __nvvm_f2tf32_rn_satfinite
    {nvvm_f2tf32_rna, 84609}, // __nvvm_f2tf32_rna
    {nvvm_f2tf32_rna_satfinite, 84620}, // __nvvm_f2tf32_rna_satfinite
    {nvvm_f2tf32_rz, 84641}, // __nvvm_f2tf32_rz
    {nvvm_f2tf32_rz_relu, 84651}, // __nvvm_f2tf32_rz_relu
    {nvvm_f2tf32_rz_relu_satfinite, 84666}, // __nvvm_f2tf32_rz_relu_satfinite
    {nvvm_f2tf32_rz_satfinite, 84691}, // __nvvm_f2tf32_rz_satfinite
    {nvvm_f2ui_rm, 84711}, // __nvvm_f2ui_rm
    {nvvm_f2ui_rm_ftz, 84719}, // __nvvm_f2ui_rm_ftz
    {nvvm_f2ui_rn, 84731}, // __nvvm_f2ui_rn
    {nvvm_f2ui_rn_ftz, 84739}, // __nvvm_f2ui_rn_ftz
    {nvvm_f2ui_rp, 84751}, // __nvvm_f2ui_rp
    {nvvm_f2ui_rp_ftz, 84759}, // __nvvm_f2ui_rp_ftz
    {nvvm_f2ui_rz, 84771}, // __nvvm_f2ui_rz
    {nvvm_f2ui_rz_ftz, 84779}, // __nvvm_f2ui_rz_ftz
    {nvvm_f2ull_rm, 84791}, // __nvvm_f2ull_rm
    {nvvm_f2ull_rm_ftz, 84800}, // __nvvm_f2ull_rm_ftz
    {nvvm_f2ull_rn, 84813}, // __nvvm_f2ull_rn
    {nvvm_f2ull_rn_ftz, 84822}, // __nvvm_f2ull_rn_ftz
    {nvvm_f2ull_rp, 84835}, // __nvvm_f2ull_rp
    {nvvm_f2ull_rp_ftz, 84844}, // __nvvm_f2ull_rp_ftz
    {nvvm_f2ull_rz, 84857}, // __nvvm_f2ull_rz
    {nvvm_f2ull_rz_ftz, 84866}, // __nvvm_f2ull_rz_ftz
    {nvvm_f32x4_to_e2m1x4_rs_relu_satfinite, 84879}, // __nvvm_f32x4_to_e2m1x4_rs_relu_satfinite
    {nvvm_f32x4_to_e2m1x4_rs_satfinite, 84913}, // __nvvm_f32x4_to_e2m1x4_rs_satfinite
    {nvvm_f32x4_to_e2m3x4_rs_relu_satfinite, 84942}, // __nvvm_f32x4_to_e2m3x4_rs_relu_satfinite
    {nvvm_f32x4_to_e2m3x4_rs_satfinite, 84976}, // __nvvm_f32x4_to_e2m3x4_rs_satfinite
    {nvvm_f32x4_to_e3m2x4_rs_relu_satfinite, 85005}, // __nvvm_f32x4_to_e3m2x4_rs_relu_satfinite
    {nvvm_f32x4_to_e3m2x4_rs_satfinite, 85039}, // __nvvm_f32x4_to_e3m2x4_rs_satfinite
    {nvvm_f32x4_to_e4m3x4_rs_relu_satfinite, 85068}, // __nvvm_f32x4_to_e4m3x4_rs_relu_satfinite
    {nvvm_f32x4_to_e4m3x4_rs_satfinite, 85102}, // __nvvm_f32x4_to_e4m3x4_rs_satfinite
    {nvvm_f32x4_to_e5m2x4_rs_relu_satfinite, 85131}, // __nvvm_f32x4_to_e5m2x4_rs_relu_satfinite
    {nvvm_f32x4_to_e5m2x4_rs_satfinite, 85165}, // __nvvm_f32x4_to_e5m2x4_rs_satfinite
    {nvvm_ff2bf16x2_rn, 85194}, // __nvvm_ff2bf16x2_rn
    {nvvm_ff2bf16x2_rn_relu, 85207}, // __nvvm_ff2bf16x2_rn_relu
    {nvvm_ff2bf16x2_rn_relu_satfinite, 85225}, // __nvvm_ff2bf16x2_rn_relu_satfinite
    {nvvm_ff2bf16x2_rn_satfinite, 85253}, // __nvvm_ff2bf16x2_rn_satfinite
    {nvvm_ff2bf16x2_rs, 85276}, // __nvvm_ff2bf16x2_rs
    {nvvm_ff2bf16x2_rs_relu, 85289}, // __nvvm_ff2bf16x2_rs_relu
    {nvvm_ff2bf16x2_rs_relu_satfinite, 85307}, // __nvvm_ff2bf16x2_rs_relu_satfinite
    {nvvm_ff2bf16x2_rs_satfinite, 85335}, // __nvvm_ff2bf16x2_rs_satfinite
    {nvvm_ff2bf16x2_rz, 85358}, // __nvvm_ff2bf16x2_rz
    {nvvm_ff2bf16x2_rz_relu, 85371}, // __nvvm_ff2bf16x2_rz_relu
    {nvvm_ff2bf16x2_rz_relu_satfinite, 85389}, // __nvvm_ff2bf16x2_rz_relu_satfinite
    {nvvm_ff2bf16x2_rz_satfinite, 85417}, // __nvvm_ff2bf16x2_rz_satfinite
    {nvvm_ff2f16x2_rn, 85440}, // __nvvm_ff2f16x2_rn
    {nvvm_ff2f16x2_rn_relu, 85452}, // __nvvm_ff2f16x2_rn_relu
    {nvvm_ff2f16x2_rn_relu_satfinite, 85469}, // __nvvm_ff2f16x2_rn_relu_satfinite
    {nvvm_ff2f16x2_rn_satfinite, 85496}, // __nvvm_ff2f16x2_rn_satfinite
    {nvvm_ff2f16x2_rs, 85518}, // __nvvm_ff2f16x2_rs
    {nvvm_ff2f16x2_rs_relu, 85530}, // __nvvm_ff2f16x2_rs_relu
    {nvvm_ff2f16x2_rs_relu_satfinite, 85547}, // __nvvm_ff2f16x2_rs_relu_satfinite
    {nvvm_ff2f16x2_rs_satfinite, 85574}, // __nvvm_ff2f16x2_rs_satfinite
    {nvvm_ff2f16x2_rz, 85596}, // __nvvm_ff2f16x2_rz
    {nvvm_ff2f16x2_rz_relu, 85608}, // __nvvm_ff2f16x2_rz_relu
    {nvvm_ff2f16x2_rz_relu_satfinite, 85625}, // __nvvm_ff2f16x2_rz_relu_satfinite
    {nvvm_ff2f16x2_rz_satfinite, 85652}, // __nvvm_ff2f16x2_rz_satfinite
    {nvvm_ff_to_e2m1x2_rn_relu_satfinite, 85674}, // __nvvm_ff_to_e2m1x2_rn_relu_satfinite
    {nvvm_ff_to_e2m1x2_rn_satfinite, 85705}, // __nvvm_ff_to_e2m1x2_rn_satfinite
    {nvvm_ff_to_e2m3x2_rn_relu_satfinite, 85731}, // __nvvm_ff_to_e2m3x2_rn_relu_satfinite
    {nvvm_ff_to_e2m3x2_rn_satfinite, 85762}, // __nvvm_ff_to_e2m3x2_rn_satfinite
    {nvvm_ff_to_e3m2x2_rn_relu_satfinite, 85788}, // __nvvm_ff_to_e3m2x2_rn_relu_satfinite
    {nvvm_ff_to_e3m2x2_rn_satfinite, 85819}, // __nvvm_ff_to_e3m2x2_rn_satfinite
    {nvvm_ff_to_e4m3x2_rn, 85845}, // __nvvm_ff_to_e4m3x2_rn
    {nvvm_ff_to_e4m3x2_rn_relu, 85861}, // __nvvm_ff_to_e4m3x2_rn_relu
    {nvvm_ff_to_e5m2x2_rn, 85882}, // __nvvm_ff_to_e5m2x2_rn
    {nvvm_ff_to_e5m2x2_rn_relu, 85898}, // __nvvm_ff_to_e5m2x2_rn_relu
    {nvvm_ff_to_ue8m0x2_rp, 85919}, // __nvvm_ff_to_ue8m0x2_rp
    {nvvm_ff_to_ue8m0x2_rp_satfinite, 85936}, // __nvvm_ff_to_ue8m0x2_rp_satfinite
    {nvvm_ff_to_ue8m0x2_rz, 85963}, // __nvvm_ff_to_ue8m0x2_rz
    {nvvm_ff_to_ue8m0x2_rz_satfinite, 85980}, // __nvvm_ff_to_ue8m0x2_rz_satfinite
    {nvvm_floor_d, 86007}, // __nvvm_floor_d
    {nvvm_floor_f, 86015}, // __nvvm_floor_f
    {nvvm_floor_ftz_f, 86023}, // __nvvm_floor_ftz_f
    {nvvm_fma_rm_d, 86035}, // __nvvm_fma_rm_d
    {nvvm_fma_rm_f, 86044}, // __nvvm_fma_rm_f
    {nvvm_fma_rm_ftz_f, 86053}, // __nvvm_fma_rm_ftz_f
    {nvvm_fma_rm_ftz_sat_f, 86066}, // __nvvm_fma_rm_ftz_sat_f
    {nvvm_fma_rm_sat_f, 86083}, // __nvvm_fma_rm_sat_f
    {nvvm_fma_rn_bf16, 86096}, // __nvvm_fma_rn_bf16
    {nvvm_fma_rn_bf16x2, 86108}, // __nvvm_fma_rn_bf16x2
    {nvvm_fma_rn_d, 86122}, // __nvvm_fma_rn_d
    {nvvm_fma_rn_f, 86131}, // __nvvm_fma_rn_f
    {nvvm_fma_rn_ftz_bf16, 86140}, // __nvvm_fma_rn_ftz_bf16
    {nvvm_fma_rn_ftz_bf16x2, 86156}, // __nvvm_fma_rn_ftz_bf16x2
    {nvvm_fma_rn_ftz_f, 86174}, // __nvvm_fma_rn_ftz_f
    {nvvm_fma_rn_ftz_relu_bf16, 86187}, // __nvvm_fma_rn_ftz_relu_bf16
    {nvvm_fma_rn_ftz_relu_bf16x2, 86208}, // __nvvm_fma_rn_ftz_relu_bf16x2
    {nvvm_fma_rn_ftz_sat_bf16, 86231}, // __nvvm_fma_rn_ftz_sat_bf16
    {nvvm_fma_rn_ftz_sat_bf16x2, 86251}, // __nvvm_fma_rn_ftz_sat_bf16x2
    {nvvm_fma_rn_ftz_sat_f, 86273}, // __nvvm_fma_rn_ftz_sat_f
    {nvvm_fma_rn_relu_bf16, 86290}, // __nvvm_fma_rn_relu_bf16
    {nvvm_fma_rn_relu_bf16x2, 86307}, // __nvvm_fma_rn_relu_bf16x2
    {nvvm_fma_rn_sat_bf16, 86326}, // __nvvm_fma_rn_sat_bf16
    {nvvm_fma_rn_sat_bf16x2, 86342}, // __nvvm_fma_rn_sat_bf16x2
    {nvvm_fma_rn_sat_f, 86360}, // __nvvm_fma_rn_sat_f
    {nvvm_fma_rp_d, 86373}, // __nvvm_fma_rp_d
    {nvvm_fma_rp_f, 86382}, // __nvvm_fma_rp_f
    {nvvm_fma_rp_ftz_f, 86391}, // __nvvm_fma_rp_ftz_f
    {nvvm_fma_rp_ftz_sat_f, 86404}, // __nvvm_fma_rp_ftz_sat_f
    {nvvm_fma_rp_sat_f, 86421}, // __nvvm_fma_rp_sat_f
    {nvvm_fma_rz_d, 86434}, // __nvvm_fma_rz_d
    {nvvm_fma_rz_f, 86443}, // __nvvm_fma_rz_f
    {nvvm_fma_rz_ftz_f, 86452}, // __nvvm_fma_rz_ftz_f
    {nvvm_fma_rz_ftz_sat_f, 86465}, // __nvvm_fma_rz_ftz_sat_f
    {nvvm_fma_rz_sat_f, 86482}, // __nvvm_fma_rz_sat_f
    {nvvm_fmax_bf16, 86495}, // __nvvm_fmax_bf16
    {nvvm_fmax_bf16x2, 86505}, // __nvvm_fmax_bf16x2
    {nvvm_fmax_d, 86517}, // __nvvm_fmax_d
    {nvvm_fmax_f, 86524}, // __nvvm_fmax_f
    {nvvm_fmax_ftz_bf16, 86531}, // __nvvm_fmax_ftz_bf16
    {nvvm_fmax_ftz_bf16x2, 86545}, // __nvvm_fmax_ftz_bf16x2
    {nvvm_fmax_ftz_f, 86561}, // __nvvm_fmax_ftz_f
    {nvvm_fmax_ftz_nan_bf16, 86572}, // __nvvm_fmax_ftz_nan_bf16
    {nvvm_fmax_ftz_nan_bf16x2, 86590}, // __nvvm_fmax_ftz_nan_bf16x2
    {nvvm_fmax_ftz_nan_f, 86610}, // __nvvm_fmax_ftz_nan_f
    {nvvm_fmax_ftz_nan_xorsign_abs_bf16, 86625}, // __nvvm_fmax_ftz_nan_xorsign_abs_bf16
    {nvvm_fmax_ftz_nan_xorsign_abs_bf16x2, 86655}, // __nvvm_fmax_ftz_nan_xorsign_abs_bf16x2
    {nvvm_fmax_ftz_nan_xorsign_abs_f, 86687}, // __nvvm_fmax_ftz_nan_xorsign_abs_f
    {nvvm_fmax_ftz_xorsign_abs_bf16, 86714}, // __nvvm_fmax_ftz_xorsign_abs_bf16
    {nvvm_fmax_ftz_xorsign_abs_bf16x2, 86740}, // __nvvm_fmax_ftz_xorsign_abs_bf16x2
    {nvvm_fmax_ftz_xorsign_abs_f, 86768}, // __nvvm_fmax_ftz_xorsign_abs_f
    {nvvm_fmax_nan_bf16, 86791}, // __nvvm_fmax_nan_bf16
    {nvvm_fmax_nan_bf16x2, 86805}, // __nvvm_fmax_nan_bf16x2
    {nvvm_fmax_nan_f, 86821}, // __nvvm_fmax_nan_f
    {nvvm_fmax_nan_xorsign_abs_bf16, 86832}, // __nvvm_fmax_nan_xorsign_abs_bf16
    {nvvm_fmax_nan_xorsign_abs_bf16x2, 86858}, // __nvvm_fmax_nan_xorsign_abs_bf16x2
    {nvvm_fmax_nan_xorsign_abs_f, 86886}, // __nvvm_fmax_nan_xorsign_abs_f
    {nvvm_fmax_xorsign_abs_bf16, 86909}, // __nvvm_fmax_xorsign_abs_bf16
    {nvvm_fmax_xorsign_abs_bf16x2, 86931}, // __nvvm_fmax_xorsign_abs_bf16x2
    {nvvm_fmax_xorsign_abs_f, 86955}, // __nvvm_fmax_xorsign_abs_f
    {nvvm_fmin_bf16, 86974}, // __nvvm_fmin_bf16
    {nvvm_fmin_bf16x2, 86984}, // __nvvm_fmin_bf16x2
    {nvvm_fmin_d, 86996}, // __nvvm_fmin_d
    {nvvm_fmin_f, 87003}, // __nvvm_fmin_f
    {nvvm_fmin_ftz_bf16, 87010}, // __nvvm_fmin_ftz_bf16
    {nvvm_fmin_ftz_bf16x2, 87024}, // __nvvm_fmin_ftz_bf16x2
    {nvvm_fmin_ftz_f, 87040}, // __nvvm_fmin_ftz_f
    {nvvm_fmin_ftz_nan_bf16, 87051}, // __nvvm_fmin_ftz_nan_bf16
    {nvvm_fmin_ftz_nan_bf16x2, 87069}, // __nvvm_fmin_ftz_nan_bf16x2
    {nvvm_fmin_ftz_nan_f, 87089}, // __nvvm_fmin_ftz_nan_f
    {nvvm_fmin_ftz_nan_xorsign_abs_bf16, 87104}, // __nvvm_fmin_ftz_nan_xorsign_abs_bf16
    {nvvm_fmin_ftz_nan_xorsign_abs_bf16x2, 87134}, // __nvvm_fmin_ftz_nan_xorsign_abs_bf16x2
    {nvvm_fmin_ftz_nan_xorsign_abs_f, 87166}, // __nvvm_fmin_ftz_nan_xorsign_abs_f
    {nvvm_fmin_ftz_xorsign_abs_bf16, 87193}, // __nvvm_fmin_ftz_xorsign_abs_bf16
    {nvvm_fmin_ftz_xorsign_abs_bf16x2, 87219}, // __nvvm_fmin_ftz_xorsign_abs_bf16x2
    {nvvm_fmin_ftz_xorsign_abs_f, 87247}, // __nvvm_fmin_ftz_xorsign_abs_f
    {nvvm_fmin_nan_bf16, 87270}, // __nvvm_fmin_nan_bf16
    {nvvm_fmin_nan_bf16x2, 87284}, // __nvvm_fmin_nan_bf16x2
    {nvvm_fmin_nan_f, 87300}, // __nvvm_fmin_nan_f
    {nvvm_fmin_nan_xorsign_abs_bf16, 87311}, // __nvvm_fmin_nan_xorsign_abs_bf16
    {nvvm_fmin_nan_xorsign_abs_bf16x2, 87337}, // __nvvm_fmin_nan_xorsign_abs_bf16x2
    {nvvm_fmin_nan_xorsign_abs_f, 87365}, // __nvvm_fmin_nan_xorsign_abs_f
    {nvvm_fmin_xorsign_abs_bf16, 87388}, // __nvvm_fmin_xorsign_abs_bf16
    {nvvm_fmin_xorsign_abs_bf16x2, 87410}, // __nvvm_fmin_xorsign_abs_bf16x2
    {nvvm_fmin_xorsign_abs_f, 87434}, // __nvvm_fmin_xorsign_abs_f
    {nvvm_fns, 87453}, // __nvvm_fns
    {nvvm_i2d_rm, 87457}, // __nvvm_i2d_rm
    {nvvm_i2d_rn, 87464}, // __nvvm_i2d_rn
    {nvvm_i2d_rp, 87471}, // __nvvm_i2d_rp
    {nvvm_i2d_rz, 87478}, // __nvvm_i2d_rz
    {nvvm_i2f_rm, 87485}, // __nvvm_i2f_rm
    {nvvm_i2f_rn, 87492}, // __nvvm_i2f_rn
    {nvvm_i2f_rp, 87499}, // __nvvm_i2f_rp
    {nvvm_i2f_rz, 87506}, // __nvvm_i2f_rz
    {nvvm_isspacep_const, 87513}, // __nvvm_isspacep_const
    {nvvm_isspacep_global, 87528}, // __nvvm_isspacep_global
    {nvvm_isspacep_local, 87544}, // __nvvm_isspacep_local
    {nvvm_isspacep_shared, 87559}, // __nvvm_isspacep_shared
    {nvvm_isspacep_shared_cluster, 87575}, // __nvvm_isspacep_shared_cluster
    {nvvm_istypep_sampler, 87599}, // __nvvm_istypep_sampler
    {nvvm_istypep_surface, 87615}, // __nvvm_istypep_surface
    {nvvm_istypep_texture, 87631}, // __nvvm_istypep_texture
    {nvvm_lg2_approx_d, 87647}, // __nvvm_lg2_approx_d
    {nvvm_lg2_approx_f, 87660}, // __nvvm_lg2_approx_f
    {nvvm_lg2_approx_ftz_f, 87673}, // __nvvm_lg2_approx_ftz_f
    {nvvm_ll2d_rm, 87690}, // __nvvm_ll2d_rm
    {nvvm_ll2d_rn, 87698}, // __nvvm_ll2d_rn
    {nvvm_ll2d_rp, 87706}, // __nvvm_ll2d_rp
    {nvvm_ll2d_rz, 87714}, // __nvvm_ll2d_rz
    {nvvm_ll2f_rm, 87722}, // __nvvm_ll2f_rm
    {nvvm_ll2f_rn, 87730}, // __nvvm_ll2f_rn
    {nvvm_ll2f_rp, 87738}, // __nvvm_ll2f_rp
    {nvvm_ll2f_rz, 87746}, // __nvvm_ll2f_rz
    {nvvm_lohi_i2d, 87754}, // __nvvm_lohi_i2d
    {nvvm_match_any_sync_i32, 87763}, // __nvvm_match_any_sync_i32
    {nvvm_match_any_sync_i64, 87782}, // __nvvm_match_any_sync_i64
    {nvvm_mbarrier_arrive, 87801}, // __nvvm_mbarrier_arrive
    {nvvm_mbarrier_arrive_drop, 87817}, // __nvvm_mbarrier_arrive_drop
    {nvvm_mbarrier_arrive_drop_noComplete, 87838}, // __nvvm_mbarrier_arrive_drop_noComplete
    {nvvm_mbarrier_arrive_drop_noComplete_shared, 87870}, // __nvvm_mbarrier_arrive_drop_noComplete_shared
    {nvvm_mbarrier_arrive_drop_shared, 87909}, // __nvvm_mbarrier_arrive_drop_shared
    {nvvm_mbarrier_arrive_noComplete, 87937}, // __nvvm_mbarrier_arrive_noComplete
    {nvvm_mbarrier_arrive_noComplete_shared, 87964}, // __nvvm_mbarrier_arrive_noComplete_shared
    {nvvm_mbarrier_arrive_shared, 87998}, // __nvvm_mbarrier_arrive_shared
    {nvvm_mbarrier_init, 88021}, // __nvvm_mbarrier_init
    {nvvm_mbarrier_init_shared, 88035}, // __nvvm_mbarrier_init_shared
    {nvvm_mbarrier_inval, 88056}, // __nvvm_mbarrier_inval
    {nvvm_mbarrier_inval_shared, 88071}, // __nvvm_mbarrier_inval_shared
    {nvvm_mbarrier_pending_count, 88093}, // __nvvm_mbarrier_pending_count
    {nvvm_mbarrier_test_wait, 88116}, // __nvvm_mbarrier_test_wait
    {nvvm_mbarrier_test_wait_shared, 88135}, // __nvvm_mbarrier_test_wait_shared
    {nvvm_membar_cta, 88161}, // __nvvm_membar_cta
    {nvvm_membar_gl, 88172}, // __nvvm_membar_gl
    {nvvm_membar_sys, 88182}, // __nvvm_membar_sys
    {nvvm_mul24_i, 88193}, // __nvvm_mul24_i
    {nvvm_mul24_ui, 88201}, // __nvvm_mul24_ui
    {nvvm_mul_rm_d, 88210}, // __nvvm_mul_rm_d
    {nvvm_mul_rm_f, 88219}, // __nvvm_mul_rm_f
    {nvvm_mul_rm_ftz_f, 88228}, // __nvvm_mul_rm_ftz_f
    {nvvm_mul_rn_d, 88241}, // __nvvm_mul_rn_d
    {nvvm_mul_rn_f, 88250}, // __nvvm_mul_rn_f
    {nvvm_mul_rn_ftz_f, 88259}, // __nvvm_mul_rn_ftz_f
    {nvvm_mul_rp_d, 88272}, // __nvvm_mul_rp_d
    {nvvm_mul_rp_f, 88281}, // __nvvm_mul_rp_f
    {nvvm_mul_rp_ftz_f, 88290}, // __nvvm_mul_rp_ftz_f
    {nvvm_mul_rz_d, 88303}, // __nvvm_mul_rz_d
    {nvvm_mul_rz_f, 88312}, // __nvvm_mul_rz_f
    {nvvm_mul_rz_ftz_f, 88321}, // __nvvm_mul_rz_ftz_f
    {nvvm_mulhi_i, 88334}, // __nvvm_mulhi_i
    {nvvm_mulhi_ll, 88342}, // __nvvm_mulhi_ll
    {nvvm_mulhi_s, 88351}, // __nvvm_mulhi_s
    {nvvm_mulhi_ui, 88359}, // __nvvm_mulhi_ui
    {nvvm_mulhi_ull, 88368}, // __nvvm_mulhi_ull
    {nvvm_mulhi_us, 88378}, // __nvvm_mulhi_us
    {nvvm_nanosleep, 88387}, // __nvvm_nanosleep
    {nvvm_neg_bf16, 88397}, // __nvvm_neg_bf16
    {nvvm_neg_bf16x2, 88406}, // __nvvm_neg_bf16x2
    {nvvm_pm_event_mask, 88417}, // __nvvm_pm_event_mask
    {nvvm_prmt, 88431}, // __nvvm_prmt
    {nvvm_rcp_approx_ftz_d, 88436}, // __nvvm_rcp_approx_ftz_d
    {nvvm_rcp_approx_ftz_f, 88453}, // __nvvm_rcp_approx_ftz_f
    {nvvm_rcp_rm_d, 88470}, // __nvvm_rcp_rm_d
    {nvvm_rcp_rm_f, 88479}, // __nvvm_rcp_rm_f
    {nvvm_rcp_rm_ftz_f, 88488}, // __nvvm_rcp_rm_ftz_f
    {nvvm_rcp_rn_d, 88501}, // __nvvm_rcp_rn_d
    {nvvm_rcp_rn_f, 88510}, // __nvvm_rcp_rn_f
    {nvvm_rcp_rn_ftz_f, 88519}, // __nvvm_rcp_rn_ftz_f
    {nvvm_rcp_rp_d, 88532}, // __nvvm_rcp_rp_d
    {nvvm_rcp_rp_f, 88541}, // __nvvm_rcp_rp_f
    {nvvm_rcp_rp_ftz_f, 88550}, // __nvvm_rcp_rp_ftz_f
    {nvvm_rcp_rz_d, 88563}, // __nvvm_rcp_rz_d
    {nvvm_rcp_rz_f, 88572}, // __nvvm_rcp_rz_f
    {nvvm_rcp_rz_ftz_f, 88581}, // __nvvm_rcp_rz_ftz_f
    {nvvm_read_ptx_sreg_clock, 88594}, // __nvvm_read_ptx_sreg_clock
    {nvvm_read_ptx_sreg_clock64, 88614}, // __nvvm_read_ptx_sreg_clock64
    {nvvm_read_ptx_sreg_ctaid_w, 88636}, // __nvvm_read_ptx_sreg_ctaid_w
    {nvvm_read_ptx_sreg_ctaid_x, 88658}, // __nvvm_read_ptx_sreg_ctaid_x
    {nvvm_read_ptx_sreg_ctaid_y, 88680}, // __nvvm_read_ptx_sreg_ctaid_y
    {nvvm_read_ptx_sreg_ctaid_z, 88702}, // __nvvm_read_ptx_sreg_ctaid_z
    {nvvm_read_ptx_sreg_envreg0, 88724}, // __nvvm_read_ptx_sreg_envreg0
    {nvvm_read_ptx_sreg_envreg1, 88746}, // __nvvm_read_ptx_sreg_envreg1
    {nvvm_read_ptx_sreg_envreg10, 88768}, // __nvvm_read_ptx_sreg_envreg10
    {nvvm_read_ptx_sreg_envreg11, 88791}, // __nvvm_read_ptx_sreg_envreg11
    {nvvm_read_ptx_sreg_envreg12, 88814}, // __nvvm_read_ptx_sreg_envreg12
    {nvvm_read_ptx_sreg_envreg13, 88837}, // __nvvm_read_ptx_sreg_envreg13
    {nvvm_read_ptx_sreg_envreg14, 88860}, // __nvvm_read_ptx_sreg_envreg14
    {nvvm_read_ptx_sreg_envreg15, 88883}, // __nvvm_read_ptx_sreg_envreg15
    {nvvm_read_ptx_sreg_envreg16, 88906}, // __nvvm_read_ptx_sreg_envreg16
    {nvvm_read_ptx_sreg_envreg17, 88929}, // __nvvm_read_ptx_sreg_envreg17
    {nvvm_read_ptx_sreg_envreg18, 88952}, // __nvvm_read_ptx_sreg_envreg18
    {nvvm_read_ptx_sreg_envreg19, 88975}, // __nvvm_read_ptx_sreg_envreg19
    {nvvm_read_ptx_sreg_envreg2, 88998}, // __nvvm_read_ptx_sreg_envreg2
    {nvvm_read_ptx_sreg_envreg20, 89020}, // __nvvm_read_ptx_sreg_envreg20
    {nvvm_read_ptx_sreg_envreg21, 89043}, // __nvvm_read_ptx_sreg_envreg21
    {nvvm_read_ptx_sreg_envreg22, 89066}, // __nvvm_read_ptx_sreg_envreg22
    {nvvm_read_ptx_sreg_envreg23, 89089}, // __nvvm_read_ptx_sreg_envreg23
    {nvvm_read_ptx_sreg_envreg24, 89112}, // __nvvm_read_ptx_sreg_envreg24
    {nvvm_read_ptx_sreg_envreg25, 89135}, // __nvvm_read_ptx_sreg_envreg25
    {nvvm_read_ptx_sreg_envreg26, 89158}, // __nvvm_read_ptx_sreg_envreg26
    {nvvm_read_ptx_sreg_envreg27, 89181}, // __nvvm_read_ptx_sreg_envreg27
    {nvvm_read_ptx_sreg_envreg28, 89204}, // __nvvm_read_ptx_sreg_envreg28
    {nvvm_read_ptx_sreg_envreg29, 89227}, // __nvvm_read_ptx_sreg_envreg29
    {nvvm_read_ptx_sreg_envreg3, 89250}, // __nvvm_read_ptx_sreg_envreg3
    {nvvm_read_ptx_sreg_envreg30, 89272}, // __nvvm_read_ptx_sreg_envreg30
    {nvvm_read_ptx_sreg_envreg31, 89295}, // __nvvm_read_ptx_sreg_envreg31
    {nvvm_read_ptx_sreg_envreg4, 89318}, // __nvvm_read_ptx_sreg_envreg4
    {nvvm_read_ptx_sreg_envreg5, 89340}, // __nvvm_read_ptx_sreg_envreg5
    {nvvm_read_ptx_sreg_envreg6, 89362}, // __nvvm_read_ptx_sreg_envreg6
    {nvvm_read_ptx_sreg_envreg7, 89384}, // __nvvm_read_ptx_sreg_envreg7
    {nvvm_read_ptx_sreg_envreg8, 89406}, // __nvvm_read_ptx_sreg_envreg8
    {nvvm_read_ptx_sreg_envreg9, 89428}, // __nvvm_read_ptx_sreg_envreg9
    {nvvm_read_ptx_sreg_globaltimer, 89450}, // __nvvm_read_ptx_sreg_globaltimer
    {nvvm_read_ptx_sreg_globaltimer_lo, 89476}, // __nvvm_read_ptx_sreg_globaltimer_lo
    {nvvm_read_ptx_sreg_gridid, 89505}, // __nvvm_read_ptx_sreg_gridid
    {nvvm_read_ptx_sreg_laneid, 89526}, // __nvvm_read_ptx_sreg_laneid
    {nvvm_read_ptx_sreg_lanemask_eq, 89547}, // __nvvm_read_ptx_sreg_lanemask_eq
    {nvvm_read_ptx_sreg_lanemask_ge, 89573}, // __nvvm_read_ptx_sreg_lanemask_ge
    {nvvm_read_ptx_sreg_lanemask_gt, 89599}, // __nvvm_read_ptx_sreg_lanemask_gt
    {nvvm_read_ptx_sreg_lanemask_le, 89625}, // __nvvm_read_ptx_sreg_lanemask_le
    {nvvm_read_ptx_sreg_lanemask_lt, 89651}, // __nvvm_read_ptx_sreg_lanemask_lt
    {nvvm_read_ptx_sreg_nctaid_w, 89677}, // __nvvm_read_ptx_sreg_nctaid_w
    {nvvm_read_ptx_sreg_nctaid_x, 89700}, // __nvvm_read_ptx_sreg_nctaid_x
    {nvvm_read_ptx_sreg_nctaid_y, 89723}, // __nvvm_read_ptx_sreg_nctaid_y
    {nvvm_read_ptx_sreg_nctaid_z, 89746}, // __nvvm_read_ptx_sreg_nctaid_z
    {nvvm_read_ptx_sreg_nsmid, 89769}, // __nvvm_read_ptx_sreg_nsmid
    {nvvm_read_ptx_sreg_ntid_w, 89789}, // __nvvm_read_ptx_sreg_ntid_w
    {nvvm_read_ptx_sreg_ntid_x, 89810}, // __nvvm_read_ptx_sreg_ntid_x
    {nvvm_read_ptx_sreg_ntid_y, 89831}, // __nvvm_read_ptx_sreg_ntid_y
    {nvvm_read_ptx_sreg_ntid_z, 89852}, // __nvvm_read_ptx_sreg_ntid_z
    {nvvm_read_ptx_sreg_nwarpid, 89873}, // __nvvm_read_ptx_sreg_nwarpid
    {nvvm_read_ptx_sreg_pm0, 89895}, // __nvvm_read_ptx_sreg_pm0
    {nvvm_read_ptx_sreg_pm1, 89913}, // __nvvm_read_ptx_sreg_pm1
    {nvvm_read_ptx_sreg_pm2, 89931}, // __nvvm_read_ptx_sreg_pm2
    {nvvm_read_ptx_sreg_pm3, 89949}, // __nvvm_read_ptx_sreg_pm3
    {nvvm_read_ptx_sreg_smid, 89967}, // __nvvm_read_ptx_sreg_smid
    {nvvm_read_ptx_sreg_tid_w, 89986}, // __nvvm_read_ptx_sreg_tid_w
    {nvvm_read_ptx_sreg_tid_x, 90006}, // __nvvm_read_ptx_sreg_tid_x
    {nvvm_read_ptx_sreg_tid_y, 90026}, // __nvvm_read_ptx_sreg_tid_y
    {nvvm_read_ptx_sreg_tid_z, 90046}, // __nvvm_read_ptx_sreg_tid_z
    {nvvm_read_ptx_sreg_warpid, 90066}, // __nvvm_read_ptx_sreg_warpid
    {nvvm_read_ptx_sreg_warpsize, 90087}, // __nvvm_read_ptx_sreg_warpsize
    {nvvm_redux_sync_add, 90110}, // __nvvm_redux_sync_add
    {nvvm_redux_sync_and, 90125}, // __nvvm_redux_sync_and
    {nvvm_redux_sync_fmax, 90140}, // __nvvm_redux_sync_fmax
    {nvvm_redux_sync_fmax_NaN, 90156}, // __nvvm_redux_sync_fmax_NaN
    {nvvm_redux_sync_fmax_abs, 90176}, // __nvvm_redux_sync_fmax_abs
    {nvvm_redux_sync_fmax_abs_NaN, 90196}, // __nvvm_redux_sync_fmax_abs_NaN
    {nvvm_redux_sync_fmin, 90220}, // __nvvm_redux_sync_fmin
    {nvvm_redux_sync_fmin_NaN, 90236}, // __nvvm_redux_sync_fmin_NaN
    {nvvm_redux_sync_fmin_abs, 90256}, // __nvvm_redux_sync_fmin_abs
    {nvvm_redux_sync_fmin_abs_NaN, 90276}, // __nvvm_redux_sync_fmin_abs_NaN
    {nvvm_redux_sync_max, 90300}, // __nvvm_redux_sync_max
    {nvvm_redux_sync_min, 90315}, // __nvvm_redux_sync_min
    {nvvm_redux_sync_or, 90330}, // __nvvm_redux_sync_or
    {nvvm_redux_sync_umax, 90344}, // __nvvm_redux_sync_umax
    {nvvm_redux_sync_umin, 90360}, // __nvvm_redux_sync_umin
    {nvvm_redux_sync_xor, 90376}, // __nvvm_redux_sync_xor
    {nvvm_reflect, 90391}, // __nvvm_reflect
    {nvvm_round_d, 90399}, // __nvvm_round_d
    {nvvm_round_f, 90407}, // __nvvm_round_f
    {nvvm_round_ftz_f, 90415}, // __nvvm_round_ftz_f
    {nvvm_rsqrt_approx_d, 90427}, // __nvvm_rsqrt_approx_d
    {nvvm_rsqrt_approx_f, 90442}, // __nvvm_rsqrt_approx_f
    {nvvm_rsqrt_approx_ftz_d, 90457}, // __nvvm_rsqrt_approx_ftz_d
    {nvvm_rsqrt_approx_ftz_f, 90476}, // __nvvm_rsqrt_approx_ftz_f
    {nvvm_sad_i, 90495}, // __nvvm_sad_i
    {nvvm_sad_ll, 90501}, // __nvvm_sad_ll
    {nvvm_sad_s, 90508}, // __nvvm_sad_s
    {nvvm_sad_ui, 90514}, // __nvvm_sad_ui
    {nvvm_sad_ull, 90521}, // __nvvm_sad_ull
    {nvvm_sad_us, 90529}, // __nvvm_sad_us
    {nvvm_saturate_d, 90536}, // __nvvm_saturate_d
    {nvvm_saturate_f, 90547}, // __nvvm_saturate_f
    {nvvm_saturate_ftz_f, 90558}, // __nvvm_saturate_ftz_f
    {nvvm_shfl_bfly_f32, 90573}, // __nvvm_shfl_bfly_f32
    {nvvm_shfl_bfly_i32, 90587}, // __nvvm_shfl_bfly_i32
    {nvvm_shfl_down_f32, 90601}, // __nvvm_shfl_down_f32
    {nvvm_shfl_down_i32, 90615}, // __nvvm_shfl_down_i32
    {nvvm_shfl_idx_f32, 90629}, // __nvvm_shfl_idx_f32
    {nvvm_shfl_idx_i32, 90642}, // __nvvm_shfl_idx_i32
    {nvvm_shfl_sync_bfly_f32, 90655}, // __nvvm_shfl_sync_bfly_f32
    {nvvm_shfl_sync_bfly_i32, 90674}, // __nvvm_shfl_sync_bfly_i32
    {nvvm_shfl_sync_down_f32, 90693}, // __nvvm_shfl_sync_down_f32
    {nvvm_shfl_sync_down_i32, 90712}, // __nvvm_shfl_sync_down_i32
    {nvvm_shfl_sync_idx_f32, 90731}, // __nvvm_shfl_sync_idx_f32
    {nvvm_shfl_sync_idx_i32, 90749}, // __nvvm_shfl_sync_idx_i32
    {nvvm_shfl_sync_up_f32, 90767}, // __nvvm_shfl_sync_up_f32
    {nvvm_shfl_sync_up_i32, 90784}, // __nvvm_shfl_sync_up_i32
    {nvvm_shfl_up_f32, 90801}, // __nvvm_shfl_up_f32
    {nvvm_shfl_up_i32, 90813}, // __nvvm_shfl_up_i32
    {nvvm_sin_approx_f, 90825}, // __nvvm_sin_approx_f
    {nvvm_sin_approx_ftz_f, 90838}, // __nvvm_sin_approx_ftz_f
    {nvvm_sqrt_approx_f, 90855}, // __nvvm_sqrt_approx_f
    {nvvm_sqrt_approx_ftz_f, 90869}, // __nvvm_sqrt_approx_ftz_f
    {nvvm_sqrt_f, 90887}, // __nvvm_sqrt_f
    {nvvm_sqrt_rm_d, 90894}, // __nvvm_sqrt_rm_d
    {nvvm_sqrt_rm_f, 90904}, // __nvvm_sqrt_rm_f
    {nvvm_sqrt_rm_ftz_f, 90914}, // __nvvm_sqrt_rm_ftz_f
    {nvvm_sqrt_rn_d, 90928}, // __nvvm_sqrt_rn_d
    {nvvm_sqrt_rn_f, 90938}, // __nvvm_sqrt_rn_f
    {nvvm_sqrt_rn_ftz_f, 90948}, // __nvvm_sqrt_rn_ftz_f
    {nvvm_sqrt_rp_d, 90962}, // __nvvm_sqrt_rp_d
    {nvvm_sqrt_rp_f, 90972}, // __nvvm_sqrt_rp_f
    {nvvm_sqrt_rp_ftz_f, 90982}, // __nvvm_sqrt_rp_ftz_f
    {nvvm_sqrt_rz_d, 90996}, // __nvvm_sqrt_rz_d
    {nvvm_sqrt_rz_f, 91006}, // __nvvm_sqrt_rz_f
    {nvvm_sqrt_rz_ftz_f, 91016}, // __nvvm_sqrt_rz_ftz_f
    {nvvm_suq_array_size, 91030}, // __nvvm_suq_array_size
    {nvvm_suq_channel_data_type, 91045}, // __nvvm_suq_channel_data_type
    {nvvm_suq_channel_order, 91067}, // __nvvm_suq_channel_order
    {nvvm_suq_depth, 91085}, // __nvvm_suq_depth
    {nvvm_suq_height, 91095}, // __nvvm_suq_height
    {nvvm_suq_width, 91106}, // __nvvm_suq_width
    {nvvm_sust_b_1d_array_i16_clamp, 91116}, // __nvvm_sust_b_1d_array_i16_clamp
    {nvvm_sust_b_1d_array_i16_trap, 91142}, // __nvvm_sust_b_1d_array_i16_trap
    {nvvm_sust_b_1d_array_i16_zero, 91167}, // __nvvm_sust_b_1d_array_i16_zero
    {nvvm_sust_b_1d_array_i32_clamp, 91192}, // __nvvm_sust_b_1d_array_i32_clamp
    {nvvm_sust_b_1d_array_i32_trap, 91218}, // __nvvm_sust_b_1d_array_i32_trap
    {nvvm_sust_b_1d_array_i32_zero, 91243}, // __nvvm_sust_b_1d_array_i32_zero
    {nvvm_sust_b_1d_array_i64_clamp, 91268}, // __nvvm_sust_b_1d_array_i64_clamp
    {nvvm_sust_b_1d_array_i64_trap, 91294}, // __nvvm_sust_b_1d_array_i64_trap
    {nvvm_sust_b_1d_array_i64_zero, 91319}, // __nvvm_sust_b_1d_array_i64_zero
    {nvvm_sust_b_1d_array_i8_clamp, 91344}, // __nvvm_sust_b_1d_array_i8_clamp
    {nvvm_sust_b_1d_array_i8_trap, 91369}, // __nvvm_sust_b_1d_array_i8_trap
    {nvvm_sust_b_1d_array_i8_zero, 91393}, // __nvvm_sust_b_1d_array_i8_zero
    {nvvm_sust_b_1d_array_v2i16_clamp, 91417}, // __nvvm_sust_b_1d_array_v2i16_clamp
    {nvvm_sust_b_1d_array_v2i16_trap, 91445}, // __nvvm_sust_b_1d_array_v2i16_trap
    {nvvm_sust_b_1d_array_v2i16_zero, 91472}, // __nvvm_sust_b_1d_array_v2i16_zero
    {nvvm_sust_b_1d_array_v2i32_clamp, 91499}, // __nvvm_sust_b_1d_array_v2i32_clamp
    {nvvm_sust_b_1d_array_v2i32_trap, 91527}, // __nvvm_sust_b_1d_array_v2i32_trap
    {nvvm_sust_b_1d_array_v2i32_zero, 91554}, // __nvvm_sust_b_1d_array_v2i32_zero
    {nvvm_sust_b_1d_array_v2i64_clamp, 91581}, // __nvvm_sust_b_1d_array_v2i64_clamp
    {nvvm_sust_b_1d_array_v2i64_trap, 91609}, // __nvvm_sust_b_1d_array_v2i64_trap
    {nvvm_sust_b_1d_array_v2i64_zero, 91636}, // __nvvm_sust_b_1d_array_v2i64_zero
    {nvvm_sust_b_1d_array_v2i8_clamp, 91663}, // __nvvm_sust_b_1d_array_v2i8_clamp
    {nvvm_sust_b_1d_array_v2i8_trap, 91690}, // __nvvm_sust_b_1d_array_v2i8_trap
    {nvvm_sust_b_1d_array_v2i8_zero, 91716}, // __nvvm_sust_b_1d_array_v2i8_zero
    {nvvm_sust_b_1d_array_v4i16_clamp, 91742}, // __nvvm_sust_b_1d_array_v4i16_clamp
    {nvvm_sust_b_1d_array_v4i16_trap, 91770}, // __nvvm_sust_b_1d_array_v4i16_trap
    {nvvm_sust_b_1d_array_v4i16_zero, 91797}, // __nvvm_sust_b_1d_array_v4i16_zero
    {nvvm_sust_b_1d_array_v4i32_clamp, 91824}, // __nvvm_sust_b_1d_array_v4i32_clamp
    {nvvm_sust_b_1d_array_v4i32_trap, 91852}, // __nvvm_sust_b_1d_array_v4i32_trap
    {nvvm_sust_b_1d_array_v4i32_zero, 91879}, // __nvvm_sust_b_1d_array_v4i32_zero
    {nvvm_sust_b_1d_array_v4i8_clamp, 91906}, // __nvvm_sust_b_1d_array_v4i8_clamp
    {nvvm_sust_b_1d_array_v4i8_trap, 91933}, // __nvvm_sust_b_1d_array_v4i8_trap
    {nvvm_sust_b_1d_array_v4i8_zero, 91959}, // __nvvm_sust_b_1d_array_v4i8_zero
    {nvvm_sust_b_1d_i16_clamp, 91985}, // __nvvm_sust_b_1d_i16_clamp
    {nvvm_sust_b_1d_i16_trap, 92005}, // __nvvm_sust_b_1d_i16_trap
    {nvvm_sust_b_1d_i16_zero, 92024}, // __nvvm_sust_b_1d_i16_zero
    {nvvm_sust_b_1d_i32_clamp, 92043}, // __nvvm_sust_b_1d_i32_clamp
    {nvvm_sust_b_1d_i32_trap, 92063}, // __nvvm_sust_b_1d_i32_trap
    {nvvm_sust_b_1d_i32_zero, 92082}, // __nvvm_sust_b_1d_i32_zero
    {nvvm_sust_b_1d_i64_clamp, 92101}, // __nvvm_sust_b_1d_i64_clamp
    {nvvm_sust_b_1d_i64_trap, 92121}, // __nvvm_sust_b_1d_i64_trap
    {nvvm_sust_b_1d_i64_zero, 92140}, // __nvvm_sust_b_1d_i64_zero
    {nvvm_sust_b_1d_i8_clamp, 92159}, // __nvvm_sust_b_1d_i8_clamp
    {nvvm_sust_b_1d_i8_trap, 92178}, // __nvvm_sust_b_1d_i8_trap
    {nvvm_sust_b_1d_i8_zero, 92196}, // __nvvm_sust_b_1d_i8_zero
    {nvvm_sust_b_1d_v2i16_clamp, 92214}, // __nvvm_sust_b_1d_v2i16_clamp
    {nvvm_sust_b_1d_v2i16_trap, 92236}, // __nvvm_sust_b_1d_v2i16_trap
    {nvvm_sust_b_1d_v2i16_zero, 92257}, // __nvvm_sust_b_1d_v2i16_zero
    {nvvm_sust_b_1d_v2i32_clamp, 92278}, // __nvvm_sust_b_1d_v2i32_clamp
    {nvvm_sust_b_1d_v2i32_trap, 92300}, // __nvvm_sust_b_1d_v2i32_trap
    {nvvm_sust_b_1d_v2i32_zero, 92321}, // __nvvm_sust_b_1d_v2i32_zero
    {nvvm_sust_b_1d_v2i64_clamp, 92342}, // __nvvm_sust_b_1d_v2i64_clamp
    {nvvm_sust_b_1d_v2i64_trap, 92364}, // __nvvm_sust_b_1d_v2i64_trap
    {nvvm_sust_b_1d_v2i64_zero, 92385}, // __nvvm_sust_b_1d_v2i64_zero
    {nvvm_sust_b_1d_v2i8_clamp, 92406}, // __nvvm_sust_b_1d_v2i8_clamp
    {nvvm_sust_b_1d_v2i8_trap, 92427}, // __nvvm_sust_b_1d_v2i8_trap
    {nvvm_sust_b_1d_v2i8_zero, 92447}, // __nvvm_sust_b_1d_v2i8_zero
    {nvvm_sust_b_1d_v4i16_clamp, 92467}, // __nvvm_sust_b_1d_v4i16_clamp
    {nvvm_sust_b_1d_v4i16_trap, 92489}, // __nvvm_sust_b_1d_v4i16_trap
    {nvvm_sust_b_1d_v4i16_zero, 92510}, // __nvvm_sust_b_1d_v4i16_zero
    {nvvm_sust_b_1d_v4i32_clamp, 92531}, // __nvvm_sust_b_1d_v4i32_clamp
    {nvvm_sust_b_1d_v4i32_trap, 92553}, // __nvvm_sust_b_1d_v4i32_trap
    {nvvm_sust_b_1d_v4i32_zero, 92574}, // __nvvm_sust_b_1d_v4i32_zero
    {nvvm_sust_b_1d_v4i8_clamp, 92595}, // __nvvm_sust_b_1d_v4i8_clamp
    {nvvm_sust_b_1d_v4i8_trap, 92616}, // __nvvm_sust_b_1d_v4i8_trap
    {nvvm_sust_b_1d_v4i8_zero, 92636}, // __nvvm_sust_b_1d_v4i8_zero
    {nvvm_sust_b_2d_array_i16_clamp, 92656}, // __nvvm_sust_b_2d_array_i16_clamp
    {nvvm_sust_b_2d_array_i16_trap, 92682}, // __nvvm_sust_b_2d_array_i16_trap
    {nvvm_sust_b_2d_array_i16_zero, 92707}, // __nvvm_sust_b_2d_array_i16_zero
    {nvvm_sust_b_2d_array_i32_clamp, 92732}, // __nvvm_sust_b_2d_array_i32_clamp
    {nvvm_sust_b_2d_array_i32_trap, 92758}, // __nvvm_sust_b_2d_array_i32_trap
    {nvvm_sust_b_2d_array_i32_zero, 92783}, // __nvvm_sust_b_2d_array_i32_zero
    {nvvm_sust_b_2d_array_i64_clamp, 92808}, // __nvvm_sust_b_2d_array_i64_clamp
    {nvvm_sust_b_2d_array_i64_trap, 92834}, // __nvvm_sust_b_2d_array_i64_trap
    {nvvm_sust_b_2d_array_i64_zero, 92859}, // __nvvm_sust_b_2d_array_i64_zero
    {nvvm_sust_b_2d_array_i8_clamp, 92884}, // __nvvm_sust_b_2d_array_i8_clamp
    {nvvm_sust_b_2d_array_i8_trap, 92909}, // __nvvm_sust_b_2d_array_i8_trap
    {nvvm_sust_b_2d_array_i8_zero, 92933}, // __nvvm_sust_b_2d_array_i8_zero
    {nvvm_sust_b_2d_array_v2i16_clamp, 92957}, // __nvvm_sust_b_2d_array_v2i16_clamp
    {nvvm_sust_b_2d_array_v2i16_trap, 92985}, // __nvvm_sust_b_2d_array_v2i16_trap
    {nvvm_sust_b_2d_array_v2i16_zero, 93012}, // __nvvm_sust_b_2d_array_v2i16_zero
    {nvvm_sust_b_2d_array_v2i32_clamp, 93039}, // __nvvm_sust_b_2d_array_v2i32_clamp
    {nvvm_sust_b_2d_array_v2i32_trap, 93067}, // __nvvm_sust_b_2d_array_v2i32_trap
    {nvvm_sust_b_2d_array_v2i32_zero, 93094}, // __nvvm_sust_b_2d_array_v2i32_zero
    {nvvm_sust_b_2d_array_v2i64_clamp, 93121}, // __nvvm_sust_b_2d_array_v2i64_clamp
    {nvvm_sust_b_2d_array_v2i64_trap, 93149}, // __nvvm_sust_b_2d_array_v2i64_trap
    {nvvm_sust_b_2d_array_v2i64_zero, 93176}, // __nvvm_sust_b_2d_array_v2i64_zero
    {nvvm_sust_b_2d_array_v2i8_clamp, 93203}, // __nvvm_sust_b_2d_array_v2i8_clamp
    {nvvm_sust_b_2d_array_v2i8_trap, 93230}, // __nvvm_sust_b_2d_array_v2i8_trap
    {nvvm_sust_b_2d_array_v2i8_zero, 93256}, // __nvvm_sust_b_2d_array_v2i8_zero
    {nvvm_sust_b_2d_array_v4i16_clamp, 93282}, // __nvvm_sust_b_2d_array_v4i16_clamp
    {nvvm_sust_b_2d_array_v4i16_trap, 93310}, // __nvvm_sust_b_2d_array_v4i16_trap
    {nvvm_sust_b_2d_array_v4i16_zero, 93337}, // __nvvm_sust_b_2d_array_v4i16_zero
    {nvvm_sust_b_2d_array_v4i32_clamp, 93364}, // __nvvm_sust_b_2d_array_v4i32_clamp
    {nvvm_sust_b_2d_array_v4i32_trap, 93392}, // __nvvm_sust_b_2d_array_v4i32_trap
    {nvvm_sust_b_2d_array_v4i32_zero, 93419}, // __nvvm_sust_b_2d_array_v4i32_zero
    {nvvm_sust_b_2d_array_v4i8_clamp, 93446}, // __nvvm_sust_b_2d_array_v4i8_clamp
    {nvvm_sust_b_2d_array_v4i8_trap, 93473}, // __nvvm_sust_b_2d_array_v4i8_trap
    {nvvm_sust_b_2d_array_v4i8_zero, 93499}, // __nvvm_sust_b_2d_array_v4i8_zero
    {nvvm_sust_b_2d_i16_clamp, 93525}, // __nvvm_sust_b_2d_i16_clamp
    {nvvm_sust_b_2d_i16_trap, 93545}, // __nvvm_sust_b_2d_i16_trap
    {nvvm_sust_b_2d_i16_zero, 93564}, // __nvvm_sust_b_2d_i16_zero
    {nvvm_sust_b_2d_i32_clamp, 93583}, // __nvvm_sust_b_2d_i32_clamp
    {nvvm_sust_b_2d_i32_trap, 93603}, // __nvvm_sust_b_2d_i32_trap
    {nvvm_sust_b_2d_i32_zero, 93622}, // __nvvm_sust_b_2d_i32_zero
    {nvvm_sust_b_2d_i64_clamp, 93641}, // __nvvm_sust_b_2d_i64_clamp
    {nvvm_sust_b_2d_i64_trap, 93661}, // __nvvm_sust_b_2d_i64_trap
    {nvvm_sust_b_2d_i64_zero, 93680}, // __nvvm_sust_b_2d_i64_zero
    {nvvm_sust_b_2d_i8_clamp, 93699}, // __nvvm_sust_b_2d_i8_clamp
    {nvvm_sust_b_2d_i8_trap, 93718}, // __nvvm_sust_b_2d_i8_trap
    {nvvm_sust_b_2d_i8_zero, 93736}, // __nvvm_sust_b_2d_i8_zero
    {nvvm_sust_b_2d_v2i16_clamp, 93754}, // __nvvm_sust_b_2d_v2i16_clamp
    {nvvm_sust_b_2d_v2i16_trap, 93776}, // __nvvm_sust_b_2d_v2i16_trap
    {nvvm_sust_b_2d_v2i16_zero, 93797}, // __nvvm_sust_b_2d_v2i16_zero
    {nvvm_sust_b_2d_v2i32_clamp, 93818}, // __nvvm_sust_b_2d_v2i32_clamp
    {nvvm_sust_b_2d_v2i32_trap, 93840}, // __nvvm_sust_b_2d_v2i32_trap
    {nvvm_sust_b_2d_v2i32_zero, 93861}, // __nvvm_sust_b_2d_v2i32_zero
    {nvvm_sust_b_2d_v2i64_clamp, 93882}, // __nvvm_sust_b_2d_v2i64_clamp
    {nvvm_sust_b_2d_v2i64_trap, 93904}, // __nvvm_sust_b_2d_v2i64_trap
    {nvvm_sust_b_2d_v2i64_zero, 93925}, // __nvvm_sust_b_2d_v2i64_zero
    {nvvm_sust_b_2d_v2i8_clamp, 93946}, // __nvvm_sust_b_2d_v2i8_clamp
    {nvvm_sust_b_2d_v2i8_trap, 93967}, // __nvvm_sust_b_2d_v2i8_trap
    {nvvm_sust_b_2d_v2i8_zero, 93987}, // __nvvm_sust_b_2d_v2i8_zero
    {nvvm_sust_b_2d_v4i16_clamp, 94007}, // __nvvm_sust_b_2d_v4i16_clamp
    {nvvm_sust_b_2d_v4i16_trap, 94029}, // __nvvm_sust_b_2d_v4i16_trap
    {nvvm_sust_b_2d_v4i16_zero, 94050}, // __nvvm_sust_b_2d_v4i16_zero
    {nvvm_sust_b_2d_v4i32_clamp, 94071}, // __nvvm_sust_b_2d_v4i32_clamp
    {nvvm_sust_b_2d_v4i32_trap, 94093}, // __nvvm_sust_b_2d_v4i32_trap
    {nvvm_sust_b_2d_v4i32_zero, 94114}, // __nvvm_sust_b_2d_v4i32_zero
    {nvvm_sust_b_2d_v4i8_clamp, 94135}, // __nvvm_sust_b_2d_v4i8_clamp
    {nvvm_sust_b_2d_v4i8_trap, 94156}, // __nvvm_sust_b_2d_v4i8_trap
    {nvvm_sust_b_2d_v4i8_zero, 94176}, // __nvvm_sust_b_2d_v4i8_zero
    {nvvm_sust_b_3d_i16_clamp, 94196}, // __nvvm_sust_b_3d_i16_clamp
    {nvvm_sust_b_3d_i16_trap, 94216}, // __nvvm_sust_b_3d_i16_trap
    {nvvm_sust_b_3d_i16_zero, 94235}, // __nvvm_sust_b_3d_i16_zero
    {nvvm_sust_b_3d_i32_clamp, 94254}, // __nvvm_sust_b_3d_i32_clamp
    {nvvm_sust_b_3d_i32_trap, 94274}, // __nvvm_sust_b_3d_i32_trap
    {nvvm_sust_b_3d_i32_zero, 94293}, // __nvvm_sust_b_3d_i32_zero
    {nvvm_sust_b_3d_i64_clamp, 94312}, // __nvvm_sust_b_3d_i64_clamp
    {nvvm_sust_b_3d_i64_trap, 94332}, // __nvvm_sust_b_3d_i64_trap
    {nvvm_sust_b_3d_i64_zero, 94351}, // __nvvm_sust_b_3d_i64_zero
    {nvvm_sust_b_3d_i8_clamp, 94370}, // __nvvm_sust_b_3d_i8_clamp
    {nvvm_sust_b_3d_i8_trap, 94389}, // __nvvm_sust_b_3d_i8_trap
    {nvvm_sust_b_3d_i8_zero, 94407}, // __nvvm_sust_b_3d_i8_zero
    {nvvm_sust_b_3d_v2i16_clamp, 94425}, // __nvvm_sust_b_3d_v2i16_clamp
    {nvvm_sust_b_3d_v2i16_trap, 94447}, // __nvvm_sust_b_3d_v2i16_trap
    {nvvm_sust_b_3d_v2i16_zero, 94468}, // __nvvm_sust_b_3d_v2i16_zero
    {nvvm_sust_b_3d_v2i32_clamp, 94489}, // __nvvm_sust_b_3d_v2i32_clamp
    {nvvm_sust_b_3d_v2i32_trap, 94511}, // __nvvm_sust_b_3d_v2i32_trap
    {nvvm_sust_b_3d_v2i32_zero, 94532}, // __nvvm_sust_b_3d_v2i32_zero
    {nvvm_sust_b_3d_v2i64_clamp, 94553}, // __nvvm_sust_b_3d_v2i64_clamp
    {nvvm_sust_b_3d_v2i64_trap, 94575}, // __nvvm_sust_b_3d_v2i64_trap
    {nvvm_sust_b_3d_v2i64_zero, 94596}, // __nvvm_sust_b_3d_v2i64_zero
    {nvvm_sust_b_3d_v2i8_clamp, 94617}, // __nvvm_sust_b_3d_v2i8_clamp
    {nvvm_sust_b_3d_v2i8_trap, 94638}, // __nvvm_sust_b_3d_v2i8_trap
    {nvvm_sust_b_3d_v2i8_zero, 94658}, // __nvvm_sust_b_3d_v2i8_zero
    {nvvm_sust_b_3d_v4i16_clamp, 94678}, // __nvvm_sust_b_3d_v4i16_clamp
    {nvvm_sust_b_3d_v4i16_trap, 94700}, // __nvvm_sust_b_3d_v4i16_trap
    {nvvm_sust_b_3d_v4i16_zero, 94721}, // __nvvm_sust_b_3d_v4i16_zero
    {nvvm_sust_b_3d_v4i32_clamp, 94742}, // __nvvm_sust_b_3d_v4i32_clamp
    {nvvm_sust_b_3d_v4i32_trap, 94764}, // __nvvm_sust_b_3d_v4i32_trap
    {nvvm_sust_b_3d_v4i32_zero, 94785}, // __nvvm_sust_b_3d_v4i32_zero
    {nvvm_sust_b_3d_v4i8_clamp, 94806}, // __nvvm_sust_b_3d_v4i8_clamp
    {nvvm_sust_b_3d_v4i8_trap, 94827}, // __nvvm_sust_b_3d_v4i8_trap
    {nvvm_sust_b_3d_v4i8_zero, 94847}, // __nvvm_sust_b_3d_v4i8_zero
    {nvvm_sust_p_1d_array_i16_trap, 94867}, // __nvvm_sust_p_1d_array_i16_trap
    {nvvm_sust_p_1d_array_i32_trap, 94892}, // __nvvm_sust_p_1d_array_i32_trap
    {nvvm_sust_p_1d_array_i8_trap, 94917}, // __nvvm_sust_p_1d_array_i8_trap
    {nvvm_sust_p_1d_array_v2i16_trap, 94941}, // __nvvm_sust_p_1d_array_v2i16_trap
    {nvvm_sust_p_1d_array_v2i32_trap, 94968}, // __nvvm_sust_p_1d_array_v2i32_trap
    {nvvm_sust_p_1d_array_v2i8_trap, 94995}, // __nvvm_sust_p_1d_array_v2i8_trap
    {nvvm_sust_p_1d_array_v4i16_trap, 95021}, // __nvvm_sust_p_1d_array_v4i16_trap
    {nvvm_sust_p_1d_array_v4i32_trap, 95048}, // __nvvm_sust_p_1d_array_v4i32_trap
    {nvvm_sust_p_1d_array_v4i8_trap, 95075}, // __nvvm_sust_p_1d_array_v4i8_trap
    {nvvm_sust_p_1d_i16_trap, 95101}, // __nvvm_sust_p_1d_i16_trap
    {nvvm_sust_p_1d_i32_trap, 95120}, // __nvvm_sust_p_1d_i32_trap
    {nvvm_sust_p_1d_i8_trap, 95139}, // __nvvm_sust_p_1d_i8_trap
    {nvvm_sust_p_1d_v2i16_trap, 95157}, // __nvvm_sust_p_1d_v2i16_trap
    {nvvm_sust_p_1d_v2i32_trap, 95178}, // __nvvm_sust_p_1d_v2i32_trap
    {nvvm_sust_p_1d_v2i8_trap, 95199}, // __nvvm_sust_p_1d_v2i8_trap
    {nvvm_sust_p_1d_v4i16_trap, 95219}, // __nvvm_sust_p_1d_v4i16_trap
    {nvvm_sust_p_1d_v4i32_trap, 95240}, // __nvvm_sust_p_1d_v4i32_trap
    {nvvm_sust_p_1d_v4i8_trap, 95261}, // __nvvm_sust_p_1d_v4i8_trap
    {nvvm_sust_p_2d_array_i16_trap, 95281}, // __nvvm_sust_p_2d_array_i16_trap
    {nvvm_sust_p_2d_array_i32_trap, 95306}, // __nvvm_sust_p_2d_array_i32_trap
    {nvvm_sust_p_2d_array_i8_trap, 95331}, // __nvvm_sust_p_2d_array_i8_trap
    {nvvm_sust_p_2d_array_v2i16_trap, 95355}, // __nvvm_sust_p_2d_array_v2i16_trap
    {nvvm_sust_p_2d_array_v2i32_trap, 95382}, // __nvvm_sust_p_2d_array_v2i32_trap
    {nvvm_sust_p_2d_array_v2i8_trap, 95409}, // __nvvm_sust_p_2d_array_v2i8_trap
    {nvvm_sust_p_2d_array_v4i16_trap, 95435}, // __nvvm_sust_p_2d_array_v4i16_trap
    {nvvm_sust_p_2d_array_v4i32_trap, 95462}, // __nvvm_sust_p_2d_array_v4i32_trap
    {nvvm_sust_p_2d_array_v4i8_trap, 95489}, // __nvvm_sust_p_2d_array_v4i8_trap
    {nvvm_sust_p_2d_i16_trap, 95515}, // __nvvm_sust_p_2d_i16_trap
    {nvvm_sust_p_2d_i32_trap, 95534}, // __nvvm_sust_p_2d_i32_trap
    {nvvm_sust_p_2d_i8_trap, 95553}, // __nvvm_sust_p_2d_i8_trap
    {nvvm_sust_p_2d_v2i16_trap, 95571}, // __nvvm_sust_p_2d_v2i16_trap
    {nvvm_sust_p_2d_v2i32_trap, 95592}, // __nvvm_sust_p_2d_v2i32_trap
    {nvvm_sust_p_2d_v2i8_trap, 95613}, // __nvvm_sust_p_2d_v2i8_trap
    {nvvm_sust_p_2d_v4i16_trap, 95633}, // __nvvm_sust_p_2d_v4i16_trap
    {nvvm_sust_p_2d_v4i32_trap, 95654}, // __nvvm_sust_p_2d_v4i32_trap
    {nvvm_sust_p_2d_v4i8_trap, 95675}, // __nvvm_sust_p_2d_v4i8_trap
    {nvvm_sust_p_3d_i16_trap, 95695}, // __nvvm_sust_p_3d_i16_trap
    {nvvm_sust_p_3d_i32_trap, 95714}, // __nvvm_sust_p_3d_i32_trap
    {nvvm_sust_p_3d_i8_trap, 95733}, // __nvvm_sust_p_3d_i8_trap
    {nvvm_sust_p_3d_v2i16_trap, 95751}, // __nvvm_sust_p_3d_v2i16_trap
    {nvvm_sust_p_3d_v2i32_trap, 95772}, // __nvvm_sust_p_3d_v2i32_trap
    {nvvm_sust_p_3d_v2i8_trap, 95793}, // __nvvm_sust_p_3d_v2i8_trap
    {nvvm_sust_p_3d_v4i16_trap, 95813}, // __nvvm_sust_p_3d_v4i16_trap
    {nvvm_sust_p_3d_v4i32_trap, 95834}, // __nvvm_sust_p_3d_v4i32_trap
    {nvvm_sust_p_3d_v4i8_trap, 95855}, // __nvvm_sust_p_3d_v4i8_trap
    {nvvm_trunc_d, 95875}, // __nvvm_trunc_d
    {nvvm_trunc_f, 95883}, // __nvvm_trunc_f
    {nvvm_trunc_ftz_f, 95891}, // __nvvm_trunc_ftz_f
    {nvvm_txq_array_size, 95903}, // __nvvm_txq_array_size
    {nvvm_txq_channel_data_type, 95918}, // __nvvm_txq_channel_data_type
    {nvvm_txq_channel_order, 95940}, // __nvvm_txq_channel_order
    {nvvm_txq_depth, 95958}, // __nvvm_txq_depth
    {nvvm_txq_height, 95968}, // __nvvm_txq_height
    {nvvm_txq_num_mipmap_levels, 95979}, // __nvvm_txq_num_mipmap_levels
    {nvvm_txq_num_samples, 96001}, // __nvvm_txq_num_samples
    {nvvm_txq_width, 96017}, // __nvvm_txq_width
    {nvvm_ue8m0x2_to_bf16x2, 96027}, // __nvvm_ue8m0x2_to_bf16x2
    {nvvm_ui2d_rm, 96045}, // __nvvm_ui2d_rm
    {nvvm_ui2d_rn, 96053}, // __nvvm_ui2d_rn
    {nvvm_ui2d_rp, 96061}, // __nvvm_ui2d_rp
    {nvvm_ui2d_rz, 96069}, // __nvvm_ui2d_rz
    {nvvm_ui2f_rm, 96077}, // __nvvm_ui2f_rm
    {nvvm_ui2f_rn, 96085}, // __nvvm_ui2f_rn
    {nvvm_ui2f_rp, 96093}, // __nvvm_ui2f_rp
    {nvvm_ui2f_rz, 96101}, // __nvvm_ui2f_rz
    {nvvm_ull2d_rm, 96109}, // __nvvm_ull2d_rm
    {nvvm_ull2d_rn, 96118}, // __nvvm_ull2d_rn
    {nvvm_ull2d_rp, 96127}, // __nvvm_ull2d_rp
    {nvvm_ull2d_rz, 96136}, // __nvvm_ull2d_rz
    {nvvm_ull2f_rm, 96145}, // __nvvm_ull2f_rm
    {nvvm_ull2f_rn, 96154}, // __nvvm_ull2f_rn
    {nvvm_ull2f_rp, 96163}, // __nvvm_ull2f_rp
    {nvvm_ull2f_rz, 96172}, // __nvvm_ull2f_rz
    {nvvm_vote_all, 96181}, // __nvvm_vote_all
    {nvvm_vote_all_sync, 96190}, // __nvvm_vote_all_sync
    {nvvm_vote_any, 96204}, // __nvvm_vote_any
    {nvvm_vote_any_sync, 96213}, // __nvvm_vote_any_sync
    {nvvm_vote_ballot, 96227}, // __nvvm_vote_ballot
    {nvvm_vote_ballot_sync, 96239}, // __nvvm_vote_ballot_sync
    {nvvm_vote_uni, 96256}, // __nvvm_vote_uni
    {nvvm_vote_uni_sync, 96265}, // __nvvm_vote_uni_sync
  }; // nvvmNames

  // Builtins for ppc.
  static constexpr BuiltinEntry ppcNames[] = {
    {ppc_addf128_round_to_odd, 96279}, // __builtin_addf128_round_to_odd
    {ppc_addg6s, 96300}, // __builtin_addg6s
    {ppc_altivec_crypto_vcipher, 96307}, // __builtin_altivec_crypto_vcipher
    {ppc_altivec_crypto_vcipherlast, 96330}, // __builtin_altivec_crypto_vcipherlast
    {ppc_altivec_crypto_vncipher, 96357}, // __builtin_altivec_crypto_vncipher
    {ppc_altivec_crypto_vncipherlast, 96381}, // __builtin_altivec_crypto_vncipherlast
    {ppc_altivec_crypto_vpermxor, 96409}, // __builtin_altivec_crypto_vpermxor
    {ppc_altivec_crypto_vpermxor_be, 96433}, // __builtin_altivec_crypto_vpermxor_be
    {ppc_altivec_crypto_vpmsumb, 96460}, // __builtin_altivec_crypto_vpmsumb
    {ppc_altivec_crypto_vpmsumd, 96483}, // __builtin_altivec_crypto_vpmsumd
    {ppc_altivec_crypto_vpmsumh, 96506}, // __builtin_altivec_crypto_vpmsumh
    {ppc_altivec_crypto_vpmsumw, 96529}, // __builtin_altivec_crypto_vpmsumw
    {ppc_altivec_crypto_vsbox, 96552}, // __builtin_altivec_crypto_vsbox
    {ppc_altivec_crypto_vshasigmad, 96573}, // __builtin_altivec_crypto_vshasigmad
    {ppc_altivec_crypto_vshasigmaw, 96599}, // __builtin_altivec_crypto_vshasigmaw
    {ppc_altivec_dss, 96625}, // __builtin_altivec_dss
    {ppc_altivec_dssall, 96637}, // __builtin_altivec_dssall
    {ppc_altivec_dst, 96652}, // __builtin_altivec_dst
    {ppc_altivec_dstst, 96664}, // __builtin_altivec_dstst
    {ppc_altivec_dststt, 96678}, // __builtin_altivec_dststt
    {ppc_altivec_dstt, 96693}, // __builtin_altivec_dstt
    {ppc_altivec_mfvscr, 96706}, // __builtin_altivec_mfvscr
    {ppc_altivec_mtvscr, 96721}, // __builtin_altivec_mtvscr
    {ppc_altivec_mtvsrbm, 96736}, // __builtin_altivec_mtvsrbm
    {ppc_altivec_mtvsrdm, 96752}, // __builtin_altivec_mtvsrdm
    {ppc_altivec_mtvsrhm, 96768}, // __builtin_altivec_mtvsrhm
    {ppc_altivec_mtvsrqm, 96784}, // __builtin_altivec_mtvsrqm
    {ppc_altivec_mtvsrwm, 96800}, // __builtin_altivec_mtvsrwm
    {ppc_altivec_vabsdub, 96816}, // __builtin_altivec_vabsdub
    {ppc_altivec_vabsduh, 96832}, // __builtin_altivec_vabsduh
    {ppc_altivec_vabsduw, 96848}, // __builtin_altivec_vabsduw
    {ppc_altivec_vaddcuq, 96864}, // __builtin_altivec_vaddcuq
    {ppc_altivec_vaddcuw, 96880}, // __builtin_altivec_vaddcuw
    {ppc_altivec_vaddecuq, 96896}, // __builtin_altivec_vaddecuq
    {ppc_altivec_vaddeuqm, 96913}, // __builtin_altivec_vaddeuqm
    {ppc_altivec_vaddsbs, 96930}, // __builtin_altivec_vaddsbs
    {ppc_altivec_vaddshs, 96946}, // __builtin_altivec_vaddshs
    {ppc_altivec_vaddsws, 96962}, // __builtin_altivec_vaddsws
    {ppc_altivec_vaddubs, 96978}, // __builtin_altivec_vaddubs
    {ppc_altivec_vadduhs, 96994}, // __builtin_altivec_vadduhs
    {ppc_altivec_vadduws, 97010}, // __builtin_altivec_vadduws
    {ppc_altivec_vavgsb, 97026}, // __builtin_altivec_vavgsb
    {ppc_altivec_vavgsh, 97041}, // __builtin_altivec_vavgsh
    {ppc_altivec_vavgsw, 97056}, // __builtin_altivec_vavgsw
    {ppc_altivec_vavgub, 97071}, // __builtin_altivec_vavgub
    {ppc_altivec_vavguh, 97086}, // __builtin_altivec_vavguh
    {ppc_altivec_vavguw, 97101}, // __builtin_altivec_vavguw
    {ppc_altivec_vbpermd, 97116}, // __builtin_altivec_vbpermd
    {ppc_altivec_vbpermq, 97132}, // __builtin_altivec_vbpermq
    {ppc_altivec_vcfsx, 97148}, // __builtin_altivec_vcfsx
    {ppc_altivec_vcfuged, 97162}, // __builtin_altivec_vcfuged
    {ppc_altivec_vcfux, 97178}, // __builtin_altivec_vcfux
    {ppc_altivec_vclrlb, 97192}, // __builtin_altivec_vclrlb
    {ppc_altivec_vclrrb, 97207}, // __builtin_altivec_vclrrb
    {ppc_altivec_vclzdm, 97222}, // __builtin_altivec_vclzdm
    {ppc_altivec_vclzlsbb, 97237}, // __builtin_altivec_vclzlsbb
    {ppc_altivec_vcmpbfp, 97254}, // __builtin_altivec_vcmpbfp
    {ppc_altivec_vcmpbfp_p, 97270}, // __builtin_altivec_vcmpbfp_p
    {ppc_altivec_vcmpeqfp, 97288}, // __builtin_altivec_vcmpeqfp
    {ppc_altivec_vcmpeqfp_p, 97305}, // __builtin_altivec_vcmpeqfp_p
    {ppc_altivec_vcmpequb, 97324}, // __builtin_altivec_vcmpequb
    {ppc_altivec_vcmpequb_p, 97341}, // __builtin_altivec_vcmpequb_p
    {ppc_altivec_vcmpequd, 97360}, // __builtin_altivec_vcmpequd
    {ppc_altivec_vcmpequd_p, 97377}, // __builtin_altivec_vcmpequd_p
    {ppc_altivec_vcmpequh, 97396}, // __builtin_altivec_vcmpequh
    {ppc_altivec_vcmpequh_p, 97413}, // __builtin_altivec_vcmpequh_p
    {ppc_altivec_vcmpequq, 97432}, // __builtin_altivec_vcmpequq
    {ppc_altivec_vcmpequq_p, 97449}, // __builtin_altivec_vcmpequq_p
    {ppc_altivec_vcmpequw, 97468}, // __builtin_altivec_vcmpequw
    {ppc_altivec_vcmpequw_p, 97485}, // __builtin_altivec_vcmpequw_p
    {ppc_altivec_vcmpgefp, 97504}, // __builtin_altivec_vcmpgefp
    {ppc_altivec_vcmpgefp_p, 97521}, // __builtin_altivec_vcmpgefp_p
    {ppc_altivec_vcmpgtfp, 97540}, // __builtin_altivec_vcmpgtfp
    {ppc_altivec_vcmpgtfp_p, 97557}, // __builtin_altivec_vcmpgtfp_p
    {ppc_altivec_vcmpgtsb, 97576}, // __builtin_altivec_vcmpgtsb
    {ppc_altivec_vcmpgtsb_p, 97593}, // __builtin_altivec_vcmpgtsb_p
    {ppc_altivec_vcmpgtsd, 97612}, // __builtin_altivec_vcmpgtsd
    {ppc_altivec_vcmpgtsd_p, 97629}, // __builtin_altivec_vcmpgtsd_p
    {ppc_altivec_vcmpgtsh, 97648}, // __builtin_altivec_vcmpgtsh
    {ppc_altivec_vcmpgtsh_p, 97665}, // __builtin_altivec_vcmpgtsh_p
    {ppc_altivec_vcmpgtsq, 97684}, // __builtin_altivec_vcmpgtsq
    {ppc_altivec_vcmpgtsq_p, 97701}, // __builtin_altivec_vcmpgtsq_p
    {ppc_altivec_vcmpgtsw, 97720}, // __builtin_altivec_vcmpgtsw
    {ppc_altivec_vcmpgtsw_p, 97737}, // __builtin_altivec_vcmpgtsw_p
    {ppc_altivec_vcmpgtub, 97756}, // __builtin_altivec_vcmpgtub
    {ppc_altivec_vcmpgtub_p, 97773}, // __builtin_altivec_vcmpgtub_p
    {ppc_altivec_vcmpgtud, 97792}, // __builtin_altivec_vcmpgtud
    {ppc_altivec_vcmpgtud_p, 97809}, // __builtin_altivec_vcmpgtud_p
    {ppc_altivec_vcmpgtuh, 97828}, // __builtin_altivec_vcmpgtuh
    {ppc_altivec_vcmpgtuh_p, 97845}, // __builtin_altivec_vcmpgtuh_p
    {ppc_altivec_vcmpgtuq, 97864}, // __builtin_altivec_vcmpgtuq
    {ppc_altivec_vcmpgtuq_p, 97881}, // __builtin_altivec_vcmpgtuq_p
    {ppc_altivec_vcmpgtuw, 97900}, // __builtin_altivec_vcmpgtuw
    {ppc_altivec_vcmpgtuw_p, 97917}, // __builtin_altivec_vcmpgtuw_p
    {ppc_altivec_vcmpneb, 97936}, // __builtin_altivec_vcmpneb
    {ppc_altivec_vcmpneb_p, 97952}, // __builtin_altivec_vcmpneb_p
    {ppc_altivec_vcmpneh, 97970}, // __builtin_altivec_vcmpneh
    {ppc_altivec_vcmpneh_p, 97986}, // __builtin_altivec_vcmpneh_p
    {ppc_altivec_vcmpnew, 98004}, // __builtin_altivec_vcmpnew
    {ppc_altivec_vcmpnew_p, 98020}, // __builtin_altivec_vcmpnew_p
    {ppc_altivec_vcmpnezb, 98038}, // __builtin_altivec_vcmpnezb
    {ppc_altivec_vcmpnezb_p, 98055}, // __builtin_altivec_vcmpnezb_p
    {ppc_altivec_vcmpnezh, 98074}, // __builtin_altivec_vcmpnezh
    {ppc_altivec_vcmpnezh_p, 98091}, // __builtin_altivec_vcmpnezh_p
    {ppc_altivec_vcmpnezw, 98110}, // __builtin_altivec_vcmpnezw
    {ppc_altivec_vcmpnezw_p, 98127}, // __builtin_altivec_vcmpnezw_p
    {ppc_altivec_vcntmbb, 98146}, // __builtin_altivec_vcntmbb
    {ppc_altivec_vcntmbd, 98162}, // __builtin_altivec_vcntmbd
    {ppc_altivec_vcntmbh, 98178}, // __builtin_altivec_vcntmbh
    {ppc_altivec_vcntmbw, 98194}, // __builtin_altivec_vcntmbw
    {ppc_altivec_vctsxs, 98210}, // __builtin_altivec_vctsxs
    {ppc_altivec_vctuxs, 98225}, // __builtin_altivec_vctuxs
    {ppc_altivec_vctzdm, 98240}, // __builtin_altivec_vctzdm
    {ppc_altivec_vctzlsbb, 98255}, // __builtin_altivec_vctzlsbb
    {ppc_altivec_vdivesd, 98272}, // __builtin_altivec_vdivesd
    {ppc_altivec_vdivesq, 98288}, // __builtin_altivec_vdivesq
    {ppc_altivec_vdivesw, 98304}, // __builtin_altivec_vdivesw
    {ppc_altivec_vdiveud, 98320}, // __builtin_altivec_vdiveud
    {ppc_altivec_vdiveuq, 98336}, // __builtin_altivec_vdiveuq
    {ppc_altivec_vdiveuw, 98352}, // __builtin_altivec_vdiveuw
    {ppc_altivec_vexpandbm, 98368}, // __builtin_altivec_vexpandbm
    {ppc_altivec_vexpanddm, 98386}, // __builtin_altivec_vexpanddm
    {ppc_altivec_vexpandhm, 98404}, // __builtin_altivec_vexpandhm
    {ppc_altivec_vexpandqm, 98422}, // __builtin_altivec_vexpandqm
    {ppc_altivec_vexpandwm, 98440}, // __builtin_altivec_vexpandwm
    {ppc_altivec_vexptefp, 98458}, // __builtin_altivec_vexptefp
    {ppc_altivec_vextddvlx, 98475}, // __builtin_altivec_vextddvlx
    {ppc_altivec_vextddvrx, 98493}, // __builtin_altivec_vextddvrx
    {ppc_altivec_vextdubvlx, 98511}, // __builtin_altivec_vextdubvlx
    {ppc_altivec_vextdubvrx, 98530}, // __builtin_altivec_vextdubvrx
    {ppc_altivec_vextduhvlx, 98549}, // __builtin_altivec_vextduhvlx
    {ppc_altivec_vextduhvrx, 98568}, // __builtin_altivec_vextduhvrx
    {ppc_altivec_vextduwvlx, 98587}, // __builtin_altivec_vextduwvlx
    {ppc_altivec_vextduwvrx, 98606}, // __builtin_altivec_vextduwvrx
    {ppc_altivec_vextractbm, 98625}, // __builtin_altivec_vextractbm
    {ppc_altivec_vextractdm, 98644}, // __builtin_altivec_vextractdm
    {ppc_altivec_vextracthm, 98663}, // __builtin_altivec_vextracthm
    {ppc_altivec_vextractqm, 98682}, // __builtin_altivec_vextractqm
    {ppc_altivec_vextractwm, 98701}, // __builtin_altivec_vextractwm
    {ppc_altivec_vextsb2d, 98720}, // __builtin_altivec_vextsb2d
    {ppc_altivec_vextsb2w, 98737}, // __builtin_altivec_vextsb2w
    {ppc_altivec_vextsd2q, 98754}, // __builtin_altivec_vextsd2q
    {ppc_altivec_vextsh2d, 98771}, // __builtin_altivec_vextsh2d
    {ppc_altivec_vextsh2w, 98788}, // __builtin_altivec_vextsh2w
    {ppc_altivec_vextsw2d, 98805}, // __builtin_altivec_vextsw2d
    {ppc_altivec_vgbbd, 98822}, // __builtin_altivec_vgbbd
    {ppc_altivec_vgnb, 98836}, // __builtin_altivec_vgnb
    {ppc_altivec_vinsblx, 98849}, // __builtin_altivec_vinsblx
    {ppc_altivec_vinsbrx, 98865}, // __builtin_altivec_vinsbrx
    {ppc_altivec_vinsbvlx, 98881}, // __builtin_altivec_vinsbvlx
    {ppc_altivec_vinsbvrx, 98898}, // __builtin_altivec_vinsbvrx
    {ppc_altivec_vinsdlx, 98915}, // __builtin_altivec_vinsdlx
    {ppc_altivec_vinsdrx, 98931}, // __builtin_altivec_vinsdrx
    {ppc_altivec_vinshlx, 98947}, // __builtin_altivec_vinshlx
    {ppc_altivec_vinshrx, 98963}, // __builtin_altivec_vinshrx
    {ppc_altivec_vinshvlx, 98979}, // __builtin_altivec_vinshvlx
    {ppc_altivec_vinshvrx, 98996}, // __builtin_altivec_vinshvrx
    {ppc_altivec_vinswlx, 99013}, // __builtin_altivec_vinswlx
    {ppc_altivec_vinswrx, 99029}, // __builtin_altivec_vinswrx
    {ppc_altivec_vinswvlx, 99045}, // __builtin_altivec_vinswvlx
    {ppc_altivec_vinswvrx, 99062}, // __builtin_altivec_vinswvrx
    {ppc_altivec_vlogefp, 99079}, // __builtin_altivec_vlogefp
    {ppc_altivec_vmaddfp, 99095}, // __builtin_altivec_vmaddfp
    {ppc_altivec_vmaxfp, 99111}, // __builtin_altivec_vmaxfp
    {ppc_altivec_vmaxsb, 99126}, // __builtin_altivec_vmaxsb
    {ppc_altivec_vmaxsd, 99141}, // __builtin_altivec_vmaxsd
    {ppc_altivec_vmaxsh, 99156}, // __builtin_altivec_vmaxsh
    {ppc_altivec_vmaxsw, 99171}, // __builtin_altivec_vmaxsw
    {ppc_altivec_vmaxub, 99186}, // __builtin_altivec_vmaxub
    {ppc_altivec_vmaxud, 99201}, // __builtin_altivec_vmaxud
    {ppc_altivec_vmaxuh, 99216}, // __builtin_altivec_vmaxuh
    {ppc_altivec_vmaxuw, 99231}, // __builtin_altivec_vmaxuw
    {ppc_altivec_vmhaddshs, 99246}, // __builtin_altivec_vmhaddshs
    {ppc_altivec_vmhraddshs, 99264}, // __builtin_altivec_vmhraddshs
    {ppc_altivec_vminfp, 99283}, // __builtin_altivec_vminfp
    {ppc_altivec_vminsb, 99298}, // __builtin_altivec_vminsb
    {ppc_altivec_vminsd, 99313}, // __builtin_altivec_vminsd
    {ppc_altivec_vminsh, 99328}, // __builtin_altivec_vminsh
    {ppc_altivec_vminsw, 99343}, // __builtin_altivec_vminsw
    {ppc_altivec_vminub, 99358}, // __builtin_altivec_vminub
    {ppc_altivec_vminud, 99373}, // __builtin_altivec_vminud
    {ppc_altivec_vminuh, 99388}, // __builtin_altivec_vminuh
    {ppc_altivec_vminuw, 99403}, // __builtin_altivec_vminuw
    {ppc_altivec_vmladduhm, 99418}, // __builtin_altivec_vmladduhm
    {ppc_altivec_vmsumcud, 99436}, // __builtin_altivec_vmsumcud
    {ppc_altivec_vmsummbm, 99453}, // __builtin_altivec_vmsummbm
    {ppc_altivec_vmsumshm, 99470}, // __builtin_altivec_vmsumshm
    {ppc_altivec_vmsumshs, 99487}, // __builtin_altivec_vmsumshs
    {ppc_altivec_vmsumubm, 99504}, // __builtin_altivec_vmsumubm
    {ppc_altivec_vmsumudm, 99521}, // __builtin_altivec_vmsumudm
    {ppc_altivec_vmsumuhm, 99538}, // __builtin_altivec_vmsumuhm
    {ppc_altivec_vmsumuhs, 99555}, // __builtin_altivec_vmsumuhs
    {ppc_altivec_vmulesb, 99572}, // __builtin_altivec_vmulesb
    {ppc_altivec_vmulesd, 99588}, // __builtin_altivec_vmulesd
    {ppc_altivec_vmulesh, 99604}, // __builtin_altivec_vmulesh
    {ppc_altivec_vmulesw, 99620}, // __builtin_altivec_vmulesw
    {ppc_altivec_vmuleub, 99636}, // __builtin_altivec_vmuleub
    {ppc_altivec_vmuleud, 99652}, // __builtin_altivec_vmuleud
    {ppc_altivec_vmuleuh, 99668}, // __builtin_altivec_vmuleuh
    {ppc_altivec_vmuleuw, 99684}, // __builtin_altivec_vmuleuw
    {ppc_altivec_vmulhsd, 99700}, // __builtin_altivec_vmulhsd
    {ppc_altivec_vmulhsw, 99716}, // __builtin_altivec_vmulhsw
    {ppc_altivec_vmulhud, 99732}, // __builtin_altivec_vmulhud
    {ppc_altivec_vmulhuw, 99748}, // __builtin_altivec_vmulhuw
    {ppc_altivec_vmulosb, 99764}, // __builtin_altivec_vmulosb
    {ppc_altivec_vmulosd, 99780}, // __builtin_altivec_vmulosd
    {ppc_altivec_vmulosh, 99796}, // __builtin_altivec_vmulosh
    {ppc_altivec_vmulosw, 99812}, // __builtin_altivec_vmulosw
    {ppc_altivec_vmuloub, 99828}, // __builtin_altivec_vmuloub
    {ppc_altivec_vmuloud, 99844}, // __builtin_altivec_vmuloud
    {ppc_altivec_vmulouh, 99860}, // __builtin_altivec_vmulouh
    {ppc_altivec_vmulouw, 99876}, // __builtin_altivec_vmulouw
    {ppc_altivec_vnmsubfp, 99892}, // __builtin_altivec_vnmsubfp
    {ppc_altivec_vpdepd, 99909}, // __builtin_altivec_vpdepd
    {ppc_altivec_vperm, 99924}, // __builtin_altivec_vperm_4si
    {ppc_altivec_vpextd, 99942}, // __builtin_altivec_vpextd
    {ppc_altivec_vpkpx, 99957}, // __builtin_altivec_vpkpx
    {ppc_altivec_vpksdss, 99971}, // __builtin_altivec_vpksdss
    {ppc_altivec_vpksdus, 99987}, // __builtin_altivec_vpksdus
    {ppc_altivec_vpkshss, 100003}, // __builtin_altivec_vpkshss
    {ppc_altivec_vpkshus, 100019}, // __builtin_altivec_vpkshus
    {ppc_altivec_vpkswss, 100035}, // __builtin_altivec_vpkswss
    {ppc_altivec_vpkswus, 100051}, // __builtin_altivec_vpkswus
    {ppc_altivec_vpkudus, 100067}, // __builtin_altivec_vpkudus
    {ppc_altivec_vpkuhus, 100083}, // __builtin_altivec_vpkuhus
    {ppc_altivec_vpkuwus, 100099}, // __builtin_altivec_vpkuwus
    {ppc_altivec_vprtybd, 100115}, // __builtin_altivec_vprtybd
    {ppc_altivec_vprtybq, 100131}, // __builtin_altivec_vprtybq
    {ppc_altivec_vprtybw, 100147}, // __builtin_altivec_vprtybw
    {ppc_altivec_vrefp, 100163}, // __builtin_altivec_vrefp
    {ppc_altivec_vrfim, 100177}, // __builtin_altivec_vrfim
    {ppc_altivec_vrfin, 100191}, // __builtin_altivec_vrfin
    {ppc_altivec_vrfip, 100205}, // __builtin_altivec_vrfip
    {ppc_altivec_vrfiz, 100219}, // __builtin_altivec_vrfiz
    {ppc_altivec_vrlb, 100233}, // __builtin_altivec_vrlb
    {ppc_altivec_vrld, 100246}, // __builtin_altivec_vrld
    {ppc_altivec_vrldmi, 100259}, // __builtin_altivec_vrldmi
    {ppc_altivec_vrldnm, 100274}, // __builtin_altivec_vrldnm
    {ppc_altivec_vrlh, 100289}, // __builtin_altivec_vrlh
    {ppc_altivec_vrlqmi, 100302}, // __builtin_altivec_vrlqmi
    {ppc_altivec_vrlqnm, 100317}, // __builtin_altivec_vrlqnm
    {ppc_altivec_vrlw, 100332}, // __builtin_altivec_vrlw
    {ppc_altivec_vrlwmi, 100345}, // __builtin_altivec_vrlwmi
    {ppc_altivec_vrlwnm, 100360}, // __builtin_altivec_vrlwnm
    {ppc_altivec_vrsqrtefp, 100375}, // __builtin_altivec_vrsqrtefp
    {ppc_altivec_vsel, 100393}, // __builtin_altivec_vsel_4si
    {ppc_altivec_vsl, 100410}, // __builtin_altivec_vsl
    {ppc_altivec_vslb, 100422}, // __builtin_altivec_vslb
    {ppc_altivec_vsldbi, 100435}, // __builtin_altivec_vsldbi
    {ppc_altivec_vslh, 100450}, // __builtin_altivec_vslh
    {ppc_altivec_vslo, 100463}, // __builtin_altivec_vslo
    {ppc_altivec_vslv, 100476}, // __builtin_altivec_vslv
    {ppc_altivec_vslw, 100489}, // __builtin_altivec_vslw
    {ppc_altivec_vsr, 100502}, // __builtin_altivec_vsr
    {ppc_altivec_vsrab, 100514}, // __builtin_altivec_vsrab
    {ppc_altivec_vsrah, 100528}, // __builtin_altivec_vsrah
    {ppc_altivec_vsraw, 100542}, // __builtin_altivec_vsraw
    {ppc_altivec_vsrb, 100556}, // __builtin_altivec_vsrb
    {ppc_altivec_vsrdbi, 100569}, // __builtin_altivec_vsrdbi
    {ppc_altivec_vsrh, 100584}, // __builtin_altivec_vsrh
    {ppc_altivec_vsro, 100597}, // __builtin_altivec_vsro
    {ppc_altivec_vsrv, 100610}, // __builtin_altivec_vsrv
    {ppc_altivec_vsrw, 100623}, // __builtin_altivec_vsrw
    {ppc_altivec_vstribl, 100636}, // __builtin_altivec_vstribl
    {ppc_altivec_vstribl_p, 100652}, // __builtin_altivec_vstribl_p
    {ppc_altivec_vstribr, 100670}, // __builtin_altivec_vstribr
    {ppc_altivec_vstribr_p, 100686}, // __builtin_altivec_vstribr_p
    {ppc_altivec_vstrihl, 100704}, // __builtin_altivec_vstrihl
    {ppc_altivec_vstrihl_p, 100720}, // __builtin_altivec_vstrihl_p
    {ppc_altivec_vstrihr, 100738}, // __builtin_altivec_vstrihr
    {ppc_altivec_vstrihr_p, 100754}, // __builtin_altivec_vstrihr_p
    {ppc_altivec_vsubcuq, 100772}, // __builtin_altivec_vsubcuq
    {ppc_altivec_vsubcuw, 100788}, // __builtin_altivec_vsubcuw
    {ppc_altivec_vsubecuq, 100804}, // __builtin_altivec_vsubecuq
    {ppc_altivec_vsubeuqm, 100821}, // __builtin_altivec_vsubeuqm
    {ppc_altivec_vsubsbs, 100838}, // __builtin_altivec_vsubsbs
    {ppc_altivec_vsubshs, 100854}, // __builtin_altivec_vsubshs
    {ppc_altivec_vsubsws, 100870}, // __builtin_altivec_vsubsws
    {ppc_altivec_vsububs, 100886}, // __builtin_altivec_vsububs
    {ppc_altivec_vsubuhs, 100902}, // __builtin_altivec_vsubuhs
    {ppc_altivec_vsubuws, 100918}, // __builtin_altivec_vsubuws
    {ppc_altivec_vsum2sws, 100934}, // __builtin_altivec_vsum2sws
    {ppc_altivec_vsum4sbs, 100951}, // __builtin_altivec_vsum4sbs
    {ppc_altivec_vsum4shs, 100968}, // __builtin_altivec_vsum4shs
    {ppc_altivec_vsum4ubs, 100985}, // __builtin_altivec_vsum4ubs
    {ppc_altivec_vsumsws, 101002}, // __builtin_altivec_vsumsws
    {ppc_altivec_vupkhpx, 101018}, // __builtin_altivec_vupkhpx
    {ppc_altivec_vupkhsb, 101034}, // __builtin_altivec_vupkhsb
    {ppc_altivec_vupkhsh, 101050}, // __builtin_altivec_vupkhsh
    {ppc_altivec_vupkhsw, 101066}, // __builtin_altivec_vupkhsw
    {ppc_altivec_vupklpx, 101082}, // __builtin_altivec_vupklpx
    {ppc_altivec_vupklsb, 101098}, // __builtin_altivec_vupklsb
    {ppc_altivec_vupklsh, 101114}, // __builtin_altivec_vupklsh
    {ppc_altivec_vupklsw, 101130}, // __builtin_altivec_vupklsw
    {ppc_amo_ldat, 101146}, // __builtin_amo_ldat
    {ppc_amo_lwat, 101155}, // __builtin_amo_lwat
    {ppc_bpermd, 101164}, // __builtin_bpermd
    {ppc_cbcdtd, 101171}, // __builtin_cbcdtd
    {ppc_cdtbcd, 101178}, // __builtin_cdtbcd
    {ppc_cfuged, 101185}, // __builtin_cfuged
    {ppc_cntlzdm, 101192}, // __builtin_cntlzdm
    {ppc_cnttzdm, 101200}, // __builtin_cnttzdm
    {ppc_darn, 101208}, // __builtin_darn
    {ppc_darn32, 101213}, // __builtin_darn_32
    {ppc_darnraw, 101221}, // __builtin_darn_raw
    {ppc_dcbf, 101230}, // __builtin_dcbf
    {ppc_divde, 101235}, // __builtin_divde
    {ppc_divdeu, 101241}, // __builtin_divdeu
    {ppc_divf128_round_to_odd, 101248}, // __builtin_divf128_round_to_odd
    {ppc_divwe, 101269}, // __builtin_divwe
    {ppc_divweu, 101275}, // __builtin_divweu
    {ppc_fmaf128_round_to_odd, 101282}, // __builtin_fmaf128_round_to_odd
    {ppc_get_texasr, 101303}, // __builtin_get_texasr
    {ppc_get_texasru, 101314}, // __builtin_get_texasru
    {ppc_get_tfhar, 101326}, // __builtin_get_tfhar
    {ppc_get_tfiar, 101336}, // __builtin_get_tfiar
    {ppc_mulf128_round_to_odd, 101346}, // __builtin_mulf128_round_to_odd
    {ppc_pack_longdouble, 101367}, // __builtin_pack_longdouble
    {ppc_pdepd, 101383}, // __builtin_pdepd
    {ppc_pextd, 101389}, // __builtin_pextd
    {ppc_addex, 101395}, // __builtin_ppc_addex
    {ppc_addg6sd, 101405}, // __builtin_ppc_addg6s
    {ppc_bcdadd, 101416}, // __builtin_ppc_bcdadd
    {ppc_bcdadd_p, 101427}, // __builtin_ppc_bcdadd_p
    {ppc_bcdcopysign, 101440}, // __builtin_ppc_bcdcopysign
    {ppc_bcdsetsign, 101456}, // __builtin_ppc_bcdsetsign
    {ppc_bcdsub, 101471}, // __builtin_ppc_bcdsub
    {ppc_bcdsub_p, 101482}, // __builtin_ppc_bcdsub_p
    {ppc_cbcdtdd, 101495}, // __builtin_ppc_cbcdtd
    {ppc_cdtbcdd, 101506}, // __builtin_ppc_cdtbcd
    {ppc_cmpeqb, 101517}, // __builtin_ppc_cmpeqb
    {ppc_cmprb, 101528}, // __builtin_ppc_cmprb
    {ppc_compare_exp_eq, 101538}, // __builtin_ppc_compare_exp_eq
    {ppc_compare_exp_gt, 101557}, // __builtin_ppc_compare_exp_gt
    {ppc_compare_exp_lt, 101576}, // __builtin_ppc_compare_exp_lt
    {ppc_compare_exp_uo, 101595}, // __builtin_ppc_compare_exp_uo
    {ppc_dcbfl, 101614}, // __builtin_ppc_dcbfl
    {ppc_dcbflp, 101624}, // __builtin_ppc_dcbflp
    {ppc_dcbst, 101635}, // __builtin_ppc_dcbst
    {ppc_dcbt, 101645}, // __builtin_ppc_dcbt
    {ppc_dcbtst, 101654}, // __builtin_ppc_dcbtst
    {ppc_dcbtstt, 101665}, // __builtin_ppc_dcbtstt
    {ppc_dcbtt, 101677}, // __builtin_ppc_dcbtt
    {ppc_dcbz, 101687}, // __builtin_ppc_dcbz
    {ppc_eieio, 101696}, // __builtin_ppc_eieio
    {ppc_extract_exp, 101706}, // __builtin_ppc_extract_exp
    {ppc_extract_sig, 101722}, // __builtin_ppc_extract_sig
    {ppc_fcfid, 101738}, // __builtin_ppc_fcfid
    {ppc_fcfud, 101748}, // __builtin_ppc_fcfud
    {ppc_fctid, 101758}, // __builtin_ppc_fctid
    {ppc_fctidz, 101768}, // __builtin_ppc_fctidz
    {ppc_fctiw, 101779}, // __builtin_ppc_fctiw
    {ppc_fctiwz, 101789}, // __builtin_ppc_fctiwz
    {ppc_fctudz, 101800}, // __builtin_ppc_fctudz
    {ppc_fctuwz, 101811}, // __builtin_ppc_fctuwz
    {ppc_fence, 101822}, // __builtin_ppc_fence
    {ppc_fmsub, 101832}, // __builtin_ppc_fmsub
    {ppc_fmsubs, 101842}, // __builtin_ppc_fmsubs
    {ppc_fnabs, 101853}, // __builtin_ppc_fnabs
    {ppc_fnabss, 101863}, // __builtin_ppc_fnabss
    {ppc_fnmadd, 101874}, // __builtin_ppc_fnmadd
    {ppc_fnmadds, 101885}, // __builtin_ppc_fnmadds
    {ppc_fre, 101897}, // __builtin_ppc_fre
    {ppc_fres, 101905}, // __builtin_ppc_fres
    {ppc_frsqrte, 101914}, // __builtin_ppc_frsqrte
    {ppc_frsqrtes, 101926}, // __builtin_ppc_frsqrtes
    {ppc_fsel, 101939}, // __builtin_ppc_fsel
    {ppc_fsels, 101948}, // __builtin_ppc_fsels
    {ppc_icbt, 101958}, // __builtin_ppc_icbt
    {ppc_insert_exp, 101967}, // __builtin_ppc_insert_exp
    {ppc_iospace_eieio, 101982}, // __builtin_ppc_iospace_eieio
    {ppc_iospace_lwsync, 102000}, // __builtin_ppc_iospace_lwsync
    {ppc_iospace_sync, 102019}, // __builtin_ppc_iospace_sync
    {ppc_isync, 102036}, // __builtin_ppc_isync
    {ppc_load4r, 102046}, // __builtin_ppc_load4r
    {ppc_load8r, 102057}, // __builtin_ppc_load8r
    {ppc_lwsync, 102068}, // __builtin_ppc_lwsync
    {ppc_maddhd, 102079}, // __builtin_ppc_maddhd
    {ppc_maddhdu, 102090}, // __builtin_ppc_maddhdu
    {ppc_maddld, 102102}, // __builtin_ppc_maddld
    {ppc_mffsl, 102113}, // __builtin_ppc_mffsl
    {ppc_mfmsr, 102123}, // __builtin_ppc_mfmsr
    {ppc_mftbu, 102133}, // __builtin_ppc_mftbu
    {ppc_mtfsb0, 102143}, // __builtin_ppc_mtfsb0
    {ppc_mtfsb1, 102154}, // __builtin_ppc_mtfsb1
    {ppc_mtfsfi, 102165}, // __builtin_ppc_mtfsfi
    {ppc_mtmsr, 102176}, // __builtin_ppc_mtmsr
    {ppc_mulhd, 102186}, // __builtin_ppc_mulhd
    {ppc_mulhdu, 102196}, // __builtin_ppc_mulhdu
    {ppc_mulhw, 102207}, // __builtin_ppc_mulhw
    {ppc_mulhwu, 102217}, // __builtin_ppc_mulhwu
    {ppc_national2packed, 102228}, // __builtin_ppc_national2packed
    {ppc_packed2national, 102248}, // __builtin_ppc_packed2national
    {ppc_packed2zoned, 102268}, // __builtin_ppc_packed2zoned
    {ppc_rlwimi, 102285}, // __builtin_ppc_rlwimi
    {ppc_rlwnm, 102296}, // __builtin_ppc_rlwnm
    {ppc_setb, 102306}, // __builtin_ppc_setb
    {ppc_stbcx, 102315}, // __builtin_ppc_stbcx
    {ppc_stdcx, 102325}, // __builtin_ppc_stdcx
    {ppc_stfiw, 102335}, // __builtin_ppc_stfiw
    {ppc_store2r, 102345}, // __builtin_ppc_store2r
    {ppc_store4r, 102357}, // __builtin_ppc_store4r
    {ppc_store8r, 102369}, // __builtin_ppc_store8r
    {ppc_stwcx, 102381}, // __builtin_ppc_stwcx
    {ppc_sync, 102391}, // __builtin_ppc_sync
    {ppc_tdw, 102400}, // __builtin_ppc_tdw
    {ppc_trap, 102408}, // __builtin_ppc_trap
    {ppc_trapd, 102417}, // __builtin_ppc_trapd
    {ppc_tw, 102427}, // __builtin_ppc_tw
    {ppc_zoned2packed, 102434}, // __builtin_ppc_zoned2packed
    {ppc_readflm, 102451}, // __builtin_readflm
    {ppc_set_texasr, 102459}, // __builtin_set_texasr
    {ppc_set_texasru, 102470}, // __builtin_set_texasru
    {ppc_set_tfhar, 102482}, // __builtin_set_tfhar
    {ppc_set_tfiar, 102492}, // __builtin_set_tfiar
    {ppc_setflm, 102502}, // __builtin_setflm
    {ppc_setrnd, 102509}, // __builtin_setrnd
    {ppc_sqrtf128_round_to_odd, 102516}, // __builtin_sqrtf128_round_to_odd
    {ppc_subf128_round_to_odd, 102538}, // __builtin_subf128_round_to_odd
    {ppc_tabort, 102559}, // __builtin_tabort
    {ppc_tabortdc, 102566}, // __builtin_tabortdc
    {ppc_tabortdci, 102575}, // __builtin_tabortdci
    {ppc_tabortwc, 102585}, // __builtin_tabortwc
    {ppc_tabortwci, 102594}, // __builtin_tabortwci
    {ppc_tbegin, 102604}, // __builtin_tbegin
    {ppc_tcheck, 102611}, // __builtin_tcheck
    {ppc_tend, 102618}, // __builtin_tend
    {ppc_tendall, 102623}, // __builtin_tendall
    {ppc_trechkpt, 102631}, // __builtin_trechkpt
    {ppc_treclaim, 102640}, // __builtin_treclaim
    {ppc_tresume, 102649}, // __builtin_tresume
    {ppc_truncf128_round_to_odd, 102657}, // __builtin_truncf128_round_to_odd
    {ppc_tsr, 102680}, // __builtin_tsr
    {ppc_tsuspend, 102684}, // __builtin_tsuspend
    {ppc_ttest, 102693}, // __builtin_ttest
    {ppc_unpack_longdouble, 102699}, // __builtin_unpack_longdouble
    {ppc_scalar_extract_expq, 102717}, // __builtin_vsx_scalar_extract_expq
    {ppc_scalar_insert_exp_qp, 102741}, // __builtin_vsx_scalar_insert_exp_qp
    {ppc_vsx_xsmaxdp, 102766}, // __builtin_vsx_xsmaxdp
    {ppc_vsx_xsmindp, 102778}, // __builtin_vsx_xsmindp
    {ppc_vsx_xvcmpeqdp, 102790}, // __builtin_vsx_xvcmpeqdp
    {ppc_vsx_xvcmpeqdp_p, 102804}, // __builtin_vsx_xvcmpeqdp_p
    {ppc_vsx_xvcmpeqsp, 102820}, // __builtin_vsx_xvcmpeqsp
    {ppc_vsx_xvcmpeqsp_p, 102834}, // __builtin_vsx_xvcmpeqsp_p
    {ppc_vsx_xvcmpgedp, 102850}, // __builtin_vsx_xvcmpgedp
    {ppc_vsx_xvcmpgedp_p, 102864}, // __builtin_vsx_xvcmpgedp_p
    {ppc_vsx_xvcmpgesp, 102880}, // __builtin_vsx_xvcmpgesp
    {ppc_vsx_xvcmpgesp_p, 102894}, // __builtin_vsx_xvcmpgesp_p
    {ppc_vsx_xvcmpgtdp, 102910}, // __builtin_vsx_xvcmpgtdp
    {ppc_vsx_xvcmpgtdp_p, 102924}, // __builtin_vsx_xvcmpgtdp_p
    {ppc_vsx_xvcmpgtsp, 102940}, // __builtin_vsx_xvcmpgtsp
    {ppc_vsx_xvcmpgtsp_p, 102954}, // __builtin_vsx_xvcmpgtsp_p
    {ppc_vsx_xvcvbf16spn, 102970}, // __builtin_vsx_xvcvbf16spn
    {ppc_vsx_xvcvdpsp, 102986}, // __builtin_vsx_xvcvdpsp
    {ppc_vsx_xvcvdpsxws, 102999}, // __builtin_vsx_xvcvdpsxws
    {ppc_vsx_xvcvdpuxws, 103014}, // __builtin_vsx_xvcvdpuxws
    {ppc_vsx_xvcvhpsp, 103029}, // __builtin_vsx_xvcvhpsp
    {ppc_vsx_xvcvspbf16, 103042}, // __builtin_vsx_xvcvspbf16
    {ppc_vsx_xvcvspdp, 103057}, // __builtin_vsx_xvcvspdp
    {ppc_vsx_xvcvsphp, 103070}, // __builtin_vsx_xvcvsphp
    {ppc_vsx_xvcvspsxds, 103083}, // __builtin_vsx_xvcvspsxds
    {ppc_vsx_xvcvspuxds, 103098}, // __builtin_vsx_xvcvspuxds
    {ppc_vsx_xvcvsxdsp, 103113}, // __builtin_vsx_xvcvsxdsp
    {ppc_vsx_xvcvsxwdp, 103127}, // __builtin_vsx_xvcvsxwdp
    {ppc_vsx_xvcvuxdsp, 103141}, // __builtin_vsx_xvcvuxdsp
    {ppc_vsx_xvcvuxwdp, 103155}, // __builtin_vsx_xvcvuxwdp
    {ppc_vsx_xvdivdp, 103169}, // __builtin_vsx_xvdivdp
    {ppc_vsx_xvdivsp, 103181}, // __builtin_vsx_xvdivsp
    {ppc_vsx_xviexpdp, 103193}, // __builtin_vsx_xviexpdp
    {ppc_vsx_xviexpsp, 103206}, // __builtin_vsx_xviexpsp
    {ppc_vsx_xvmaxdp, 103219}, // __builtin_vsx_xvmaxdp
    {ppc_vsx_xvmaxsp, 103231}, // __builtin_vsx_xvmaxsp
    {ppc_vsx_xvmindp, 103243}, // __builtin_vsx_xvmindp
    {ppc_vsx_xvminsp, 103255}, // __builtin_vsx_xvminsp
    {ppc_vsx_xvredp, 103267}, // __builtin_vsx_xvredp
    {ppc_vsx_xvresp, 103278}, // __builtin_vsx_xvresp
    {ppc_vsx_xvrlw, 103289}, // __builtin_vsx_xvrlw
    {ppc_vsx_xvrsqrtedp, 103299}, // __builtin_vsx_xvrsqrtedp
    {ppc_vsx_xvrsqrtesp, 103314}, // __builtin_vsx_xvrsqrtesp
    {ppc_vsx_xvtdivdp, 103329}, // __builtin_vsx_xvtdivdp
    {ppc_vsx_xvtdivsp, 103342}, // __builtin_vsx_xvtdivsp
    {ppc_vsx_xvtlsbb, 103355}, // __builtin_vsx_xvtlsbb
    {ppc_vsx_xvtsqrtdp, 103367}, // __builtin_vsx_xvtsqrtdp
    {ppc_vsx_xvtsqrtsp, 103381}, // __builtin_vsx_xvtsqrtsp
    {ppc_vsx_xvtstdcdp, 103395}, // __builtin_vsx_xvtstdcdp
    {ppc_vsx_xvtstdcsp, 103409}, // __builtin_vsx_xvtstdcsp
    {ppc_vsx_xvxexpdp, 103423}, // __builtin_vsx_xvxexpdp
    {ppc_vsx_xvxexpsp, 103436}, // __builtin_vsx_xvxexpsp
    {ppc_vsx_xvxsigdp, 103449}, // __builtin_vsx_xvxsigdp
    {ppc_vsx_xvxsigsp, 103462}, // __builtin_vsx_xvxsigsp
    {ppc_vsx_xxblendvb, 103475}, // __builtin_vsx_xxblendvb
    {ppc_vsx_xxblendvd, 103489}, // __builtin_vsx_xxblendvd
    {ppc_vsx_xxblendvh, 103503}, // __builtin_vsx_xxblendvh
    {ppc_vsx_xxblendvw, 103517}, // __builtin_vsx_xxblendvw
    {ppc_vsx_xxeval, 103531}, // __builtin_vsx_xxeval
    {ppc_vsx_xxextractuw, 103542}, // __builtin_vsx_xxextractuw
    {ppc_vsx_xxgenpcvbm, 103558}, // __builtin_vsx_xxgenpcvbm
    {ppc_vsx_xxgenpcvdm, 103573}, // __builtin_vsx_xxgenpcvdm
    {ppc_vsx_xxgenpcvhm, 103588}, // __builtin_vsx_xxgenpcvhm
    {ppc_vsx_xxgenpcvwm, 103603}, // __builtin_vsx_xxgenpcvwm
    {ppc_vsx_xxinsertw, 103618}, // __builtin_vsx_xxinsertw
    {ppc_vsx_xxleqv, 103632}, // __builtin_vsx_xxleqv
    {ppc_vsx_xxpermx, 103643}, // __builtin_vsx_xxpermx
  }; // ppcNames

  // Builtins for r600.
  static constexpr BuiltinEntry r600Names[] = {
    {r600_group_barrier, 103655}, // __builtin_r600_group_barrier
    {r600_implicitarg_ptr, 4537}, // __builtin_r600_implicitarg_ptr
    {r600_rat_store_typed, 103669}, // __builtin_r600_rat_store_typed
    {r600_read_global_size_x, 103685}, // __builtin_r600_read_global_size_x
    {r600_read_global_size_y, 103704}, // __builtin_r600_read_global_size_y
    {r600_read_global_size_z, 103723}, // __builtin_r600_read_global_size_z
    {r600_read_ngroups_x, 103742}, // __builtin_r600_read_ngroups_x
    {r600_read_ngroups_y, 103757}, // __builtin_r600_read_ngroups_y
    {r600_read_ngroups_z, 103772}, // __builtin_r600_read_ngroups_z
    {r600_read_tgid_x, 103787}, // __builtin_r600_read_tgid_x
    {r600_read_tgid_y, 103799}, // __builtin_r600_read_tgid_y
    {r600_read_tgid_z, 103811}, // __builtin_r600_read_tgid_z
    {r600_read_tidig_x, 103823}, // __builtin_r600_read_tidig_x
    {r600_read_tidig_y, 103836}, // __builtin_r600_read_tidig_y
    {r600_read_tidig_z, 103849}, // __builtin_r600_read_tidig_z
  }; // r600Names

  // Builtins for riscv.
  static constexpr BuiltinEntry riscvNames[] = {
    {riscv_aes32dsi, 103862}, // __builtin_riscv_aes32dsi
    {riscv_aes32dsmi, 103871}, // __builtin_riscv_aes32dsmi
    {riscv_aes32esi, 103881}, // __builtin_riscv_aes32esi
    {riscv_aes32esmi, 103890}, // __builtin_riscv_aes32esmi
    {riscv_aes64ds, 103900}, // __builtin_riscv_aes64ds
    {riscv_aes64dsm, 103908}, // __builtin_riscv_aes64dsm
    {riscv_aes64es, 103917}, // __builtin_riscv_aes64es
    {riscv_aes64esm, 103925}, // __builtin_riscv_aes64esm
    {riscv_aes64im, 103934}, // __builtin_riscv_aes64im
    {riscv_aes64ks1i, 103942}, // __builtin_riscv_aes64ks1i
    {riscv_aes64ks2, 103952}, // __builtin_riscv_aes64ks2
    {riscv_mips_ehb, 103961}, // __builtin_riscv_mips_ehb
    {riscv_mips_ihb, 103970}, // __builtin_riscv_mips_ihb
    {riscv_mips_pause, 103979}, // __builtin_riscv_mips_pause
    {riscv_sha512sig0, 103990}, // __builtin_riscv_sha512sig0
    {riscv_sha512sig0h, 104001}, // __builtin_riscv_sha512sig0h
    {riscv_sha512sig0l, 104013}, // __builtin_riscv_sha512sig0l
    {riscv_sha512sig1, 104025}, // __builtin_riscv_sha512sig1
    {riscv_sha512sig1h, 104036}, // __builtin_riscv_sha512sig1h
    {riscv_sha512sig1l, 104048}, // __builtin_riscv_sha512sig1l
    {riscv_sha512sum0, 104060}, // __builtin_riscv_sha512sum0
    {riscv_sha512sum0r, 104071}, // __builtin_riscv_sha512sum0r
    {riscv_sha512sum1, 104083}, // __builtin_riscv_sha512sum1
    {riscv_sha512sum1r, 104094}, // __builtin_riscv_sha512sum1r
  }; // riscvNames

  // Builtins for s390.
  static constexpr BuiltinEntry s390Names[] = {
    {s390_bdepg, 104106}, // __builtin_s390_bdepg
    {s390_bextg, 104117}, // __builtin_s390_bextg
    {s390_efpc, 104128}, // __builtin_s390_efpc
    {s390_lcbb, 104138}, // __builtin_s390_lcbb
    {s390_sfpc, 104148}, // __builtin_s390_sfpc
    {s390_vaccb, 104158}, // __builtin_s390_vaccb
    {s390_vacccq, 104169}, // __builtin_s390_vacccq
    {s390_vaccf, 104181}, // __builtin_s390_vaccf
    {s390_vaccg, 104192}, // __builtin_s390_vaccg
    {s390_vacch, 104203}, // __builtin_s390_vacch
    {s390_vaccq, 104214}, // __builtin_s390_vaccq
    {s390_vacq, 104225}, // __builtin_s390_vacq
    {s390_vaq, 104235}, // __builtin_s390_vaq
    {s390_vavgb, 104244}, // __builtin_s390_vavgb
    {s390_vavgf, 104255}, // __builtin_s390_vavgf
    {s390_vavgg, 104266}, // __builtin_s390_vavgg
    {s390_vavgh, 104277}, // __builtin_s390_vavgh
    {s390_vavglb, 104288}, // __builtin_s390_vavglb
    {s390_vavglf, 104300}, // __builtin_s390_vavglf
    {s390_vavglg, 104312}, // __builtin_s390_vavglg
    {s390_vavglh, 104324}, // __builtin_s390_vavglh
    {s390_vavglq, 104336}, // __builtin_s390_vavglq
    {s390_vavgq, 104348}, // __builtin_s390_vavgq
    {s390_vbperm, 104359}, // __builtin_s390_vbperm
    {s390_vcfn, 104371}, // __builtin_s390_vcfn
    {s390_vcksm, 104381}, // __builtin_s390_vcksm
    {s390_vclfnhs, 104392}, // __builtin_s390_vclfnhs
    {s390_vclfnls, 104405}, // __builtin_s390_vclfnls
    {s390_vcnf, 104418}, // __builtin_s390_vcnf
    {s390_vcrnfs, 104428}, // __builtin_s390_vcrnfs
    {s390_verimb, 104440}, // __builtin_s390_verimb
    {s390_verimf, 104452}, // __builtin_s390_verimf
    {s390_verimg, 104464}, // __builtin_s390_verimg
    {s390_verimh, 104476}, // __builtin_s390_verimh
    {s390_veval, 104488}, // __builtin_s390_veval
    {s390_vfaeb, 104499}, // __builtin_s390_vfaeb
    {s390_vfaef, 104510}, // __builtin_s390_vfaef
    {s390_vfaeh, 104521}, // __builtin_s390_vfaeh
    {s390_vfaezb, 104532}, // __builtin_s390_vfaezb
    {s390_vfaezf, 104544}, // __builtin_s390_vfaezf
    {s390_vfaezh, 104556}, // __builtin_s390_vfaezh
    {s390_vfeeb, 104568}, // __builtin_s390_vfeeb
    {s390_vfeef, 104579}, // __builtin_s390_vfeef
    {s390_vfeeh, 104590}, // __builtin_s390_vfeeh
    {s390_vfeezb, 104601}, // __builtin_s390_vfeezb
    {s390_vfeezf, 104613}, // __builtin_s390_vfeezf
    {s390_vfeezh, 104625}, // __builtin_s390_vfeezh
    {s390_vfeneb, 104637}, // __builtin_s390_vfeneb
    {s390_vfenef, 104649}, // __builtin_s390_vfenef
    {s390_vfeneh, 104661}, // __builtin_s390_vfeneh
    {s390_vfenezb, 104673}, // __builtin_s390_vfenezb
    {s390_vfenezf, 104686}, // __builtin_s390_vfenezf
    {s390_vfenezh, 104699}, // __builtin_s390_vfenezh
    {s390_vgemb, 104712}, // __builtin_s390_vgemb
    {s390_vgemf, 104723}, // __builtin_s390_vgemf
    {s390_vgemg, 104734}, // __builtin_s390_vgemg
    {s390_vgemh, 104745}, // __builtin_s390_vgemh
    {s390_vgemq, 104756}, // __builtin_s390_vgemq
    {s390_vgfmab, 104767}, // __builtin_s390_vgfmab
    {s390_vgfmaf, 104779}, // __builtin_s390_vgfmaf
    {s390_vgfmag, 104791}, // __builtin_s390_vgfmag
    {s390_vgfmah, 104803}, // __builtin_s390_vgfmah
    {s390_vgfmb, 104815}, // __builtin_s390_vgfmb
    {s390_vgfmf, 104826}, // __builtin_s390_vgfmf
    {s390_vgfmg, 104837}, // __builtin_s390_vgfmg
    {s390_vgfmh, 104848}, // __builtin_s390_vgfmh
    {s390_vistrb, 104859}, // __builtin_s390_vistrb
    {s390_vistrf, 104871}, // __builtin_s390_vistrf
    {s390_vistrh, 104883}, // __builtin_s390_vistrh
    {s390_vlbb, 104895}, // __builtin_s390_vlbb
    {s390_vll, 104905}, // __builtin_s390_vll
    {s390_vlrl, 104914}, // __builtin_s390_vlrlr
    {s390_vmaeb, 104925}, // __builtin_s390_vmaeb
    {s390_vmaef, 104936}, // __builtin_s390_vmaef
    {s390_vmaeg, 104947}, // __builtin_s390_vmaeg
    {s390_vmaeh, 104958}, // __builtin_s390_vmaeh
    {s390_vmahb, 104969}, // __builtin_s390_vmahb
    {s390_vmahf, 104980}, // __builtin_s390_vmahf
    {s390_vmahg, 104991}, // __builtin_s390_vmahg
    {s390_vmahh, 105002}, // __builtin_s390_vmahh
    {s390_vmahq, 105013}, // __builtin_s390_vmahq
    {s390_vmaleb, 105024}, // __builtin_s390_vmaleb
    {s390_vmalef, 105036}, // __builtin_s390_vmalef
    {s390_vmaleg, 105048}, // __builtin_s390_vmaleg
    {s390_vmaleh, 105060}, // __builtin_s390_vmaleh
    {s390_vmalhb, 105072}, // __builtin_s390_vmalhb
    {s390_vmalhf, 105084}, // __builtin_s390_vmalhf
    {s390_vmalhg, 105096}, // __builtin_s390_vmalhg
    {s390_vmalhh, 105108}, // __builtin_s390_vmalhh
    {s390_vmalhq, 105120}, // __builtin_s390_vmalhq
    {s390_vmalob, 105132}, // __builtin_s390_vmalob
    {s390_vmalof, 105144}, // __builtin_s390_vmalof
    {s390_vmalog, 105156}, // __builtin_s390_vmalog
    {s390_vmaloh, 105168}, // __builtin_s390_vmaloh
    {s390_vmaob, 105180}, // __builtin_s390_vmaob
    {s390_vmaof, 105191}, // __builtin_s390_vmaof
    {s390_vmaog, 105202}, // __builtin_s390_vmaog
    {s390_vmaoh, 105213}, // __builtin_s390_vmaoh
    {s390_vmeb, 105224}, // __builtin_s390_vmeb
    {s390_vmef, 105234}, // __builtin_s390_vmef
    {s390_vmeg, 105244}, // __builtin_s390_vmeg
    {s390_vmeh, 105254}, // __builtin_s390_vmeh
    {s390_vmhb, 105264}, // __builtin_s390_vmhb
    {s390_vmhf, 105274}, // __builtin_s390_vmhf
    {s390_vmhg, 105284}, // __builtin_s390_vmhg
    {s390_vmhh, 105294}, // __builtin_s390_vmhh
    {s390_vmhq, 105304}, // __builtin_s390_vmhq
    {s390_vmleb, 105314}, // __builtin_s390_vmleb
    {s390_vmlef, 105325}, // __builtin_s390_vmlef
    {s390_vmleg, 105336}, // __builtin_s390_vmleg
    {s390_vmleh, 105347}, // __builtin_s390_vmleh
    {s390_vmlhb, 105358}, // __builtin_s390_vmlhb
    {s390_vmlhf, 105369}, // __builtin_s390_vmlhf
    {s390_vmlhg, 105380}, // __builtin_s390_vmlhg
    {s390_vmlhh, 105391}, // __builtin_s390_vmlhh
    {s390_vmlhq, 105402}, // __builtin_s390_vmlhq
    {s390_vmlob, 105413}, // __builtin_s390_vmlob
    {s390_vmlof, 105424}, // __builtin_s390_vmlof
    {s390_vmlog, 105435}, // __builtin_s390_vmlog
    {s390_vmloh, 105446}, // __builtin_s390_vmloh
    {s390_vmob, 105457}, // __builtin_s390_vmob
    {s390_vmof, 105467}, // __builtin_s390_vmof
    {s390_vmog, 105477}, // __builtin_s390_vmog
    {s390_vmoh, 105487}, // __builtin_s390_vmoh
    {s390_vmslg, 105497}, // __builtin_s390_vmslg
    {s390_vpdi, 105508}, // __builtin_s390_vpdi
    {s390_vperm, 105518}, // __builtin_s390_vperm
    {s390_vpklsf, 105529}, // __builtin_s390_vpklsf
    {s390_vpklsg, 105541}, // __builtin_s390_vpklsg
    {s390_vpklsh, 105553}, // __builtin_s390_vpklsh
    {s390_vpksf, 105565}, // __builtin_s390_vpksf
    {s390_vpksg, 105576}, // __builtin_s390_vpksg
    {s390_vpksh, 105587}, // __builtin_s390_vpksh
    {s390_vsbcbiq, 105598}, // __builtin_s390_vsbcbiq
    {s390_vsbiq, 105611}, // __builtin_s390_vsbiq
    {s390_vscbib, 105622}, // __builtin_s390_vscbib
    {s390_vscbif, 105634}, // __builtin_s390_vscbif
    {s390_vscbig, 105646}, // __builtin_s390_vscbig
    {s390_vscbih, 105658}, // __builtin_s390_vscbih
    {s390_vscbiq, 105670}, // __builtin_s390_vscbiq
    {s390_vsl, 105682}, // __builtin_s390_vsl
    {s390_vslb, 105691}, // __builtin_s390_vslb
    {s390_vsld, 105701}, // __builtin_s390_vsld
    {s390_vsldb, 105711}, // __builtin_s390_vsldb
    {s390_vsq, 105722}, // __builtin_s390_vsq
    {s390_vsra, 105731}, // __builtin_s390_vsra
    {s390_vsrab, 105741}, // __builtin_s390_vsrab
    {s390_vsrd, 105752}, // __builtin_s390_vsrd
    {s390_vsrl, 105762}, // __builtin_s390_vsrl
    {s390_vsrlb, 105772}, // __builtin_s390_vsrlb
    {s390_vstl, 105783}, // __builtin_s390_vstl
    {s390_vstrcb, 105793}, // __builtin_s390_vstrcb
    {s390_vstrcf, 105805}, // __builtin_s390_vstrcf
    {s390_vstrch, 105817}, // __builtin_s390_vstrch
    {s390_vstrczb, 105829}, // __builtin_s390_vstrczb
    {s390_vstrczf, 105842}, // __builtin_s390_vstrczf
    {s390_vstrczh, 105855}, // __builtin_s390_vstrczh
    {s390_vstrl, 105868}, // __builtin_s390_vstrlr
    {s390_vsumb, 105880}, // __builtin_s390_vsumb
    {s390_vsumgf, 105891}, // __builtin_s390_vsumgf
    {s390_vsumgh, 105903}, // __builtin_s390_vsumgh
    {s390_vsumh, 105915}, // __builtin_s390_vsumh
    {s390_vsumqf, 105926}, // __builtin_s390_vsumqf
    {s390_vsumqg, 105938}, // __builtin_s390_vsumqg
    {s390_vtm, 105950}, // __builtin_s390_vtm
    {s390_vuphb, 105959}, // __builtin_s390_vuphb
    {s390_vuphf, 105970}, // __builtin_s390_vuphf
    {s390_vuphg, 105981}, // __builtin_s390_vuphg
    {s390_vuphh, 105992}, // __builtin_s390_vuphh
    {s390_vuplb, 106003}, // __builtin_s390_vuplb
    {s390_vuplf, 106014}, // __builtin_s390_vuplf
    {s390_vuplg, 106025}, // __builtin_s390_vuplg
    {s390_vuplhb, 106036}, // __builtin_s390_vuplhb
    {s390_vuplhf, 106048}, // __builtin_s390_vuplhf
    {s390_vuplhg, 106060}, // __builtin_s390_vuplhg
    {s390_vuplhh, 106072}, // __builtin_s390_vuplhh
    {s390_vuplhw, 106084}, // __builtin_s390_vuplhw
    {s390_vupllb, 106096}, // __builtin_s390_vupllb
    {s390_vupllf, 106108}, // __builtin_s390_vupllf
    {s390_vupllg, 106120}, // __builtin_s390_vupllg
    {s390_vupllh, 106132}, // __builtin_s390_vupllh
    {s390_tend, 102618}, // __builtin_tend
    {s390_ppa_txassist, 106144}, // __builtin_tx_assist
    {s390_etnd, 106154}, // __builtin_tx_nesting_depth
  }; // s390Names

  // Builtins for spv.
  static constexpr BuiltinEntry spvNames[] = {
    {spv_group_memory_barrier_with_group_sync, 103655}, // __builtin_spirv_group_barrier
    {spv_num_subgroups, 106171}, // __builtin_spirv_num_subgroups
    {spv_wave_ballot, 106185}, // __builtin_spirv_subgroup_ballot
    {spv_subgroup_id, 106201}, // __builtin_spirv_subgroup_id
    {spv_subgroup_local_invocation_id, 106213}, // __builtin_spirv_subgroup_local_invocation_id
    {spv_subgroup_max_size, 106242}, // __builtin_spirv_subgroup_max_size
    {spv_subgroup_size, 106260}, // __builtin_spirv_subgroup_size
  }; // spvNames

  // Builtins for ve.
  static constexpr BuiltinEntry veNames[] = {
    {ve_vl_andm_MMM, 106274}, // __builtin_ve_vl_andm_MMM
    {ve_vl_andm_mmm, 106283}, // __builtin_ve_vl_andm_mmm
    {ve_vl_eqvm_MMM, 106292}, // __builtin_ve_vl_eqvm_MMM
    {ve_vl_eqvm_mmm, 106301}, // __builtin_ve_vl_eqvm_mmm
    {ve_vl_extract_vm512l, 106310}, // __builtin_ve_vl_extract_vm512l
    {ve_vl_extract_vm512u, 106325}, // __builtin_ve_vl_extract_vm512u
    {ve_vl_fencec_s, 106340}, // __builtin_ve_vl_fencec_s
    {ve_vl_fencei, 106349}, // __builtin_ve_vl_fencei
    {ve_vl_fencem_s, 106356}, // __builtin_ve_vl_fencem_s
    {ve_vl_fidcr_sss, 106365}, // __builtin_ve_vl_fidcr_sss
    {ve_vl_insert_vm512l, 106375}, // __builtin_ve_vl_insert_vm512l
    {ve_vl_insert_vm512u, 106389}, // __builtin_ve_vl_insert_vm512u
    {ve_vl_lcr_sss, 106403}, // __builtin_ve_vl_lcr_sss
    {ve_vl_lsv_vvss, 106411}, // __builtin_ve_vl_lsv_vvss
    {ve_vl_lvm_MMss, 106420}, // __builtin_ve_vl_lvm_MMss
    {ve_vl_lvm_mmss, 106429}, // __builtin_ve_vl_lvm_mmss
    {ve_vl_lvsd_svs, 106438}, // __builtin_ve_vl_lvsd_svs
    {ve_vl_lvsl_svs, 106447}, // __builtin_ve_vl_lvsl_svs
    {ve_vl_lvss_svs, 106456}, // __builtin_ve_vl_lvss_svs
    {ve_vl_lzvm_sml, 106465}, // __builtin_ve_vl_lzvm_sml
    {ve_vl_negm_MM, 106474}, // __builtin_ve_vl_negm_MM
    {ve_vl_negm_mm, 106482}, // __builtin_ve_vl_negm_mm
    {ve_vl_nndm_MMM, 106490}, // __builtin_ve_vl_nndm_MMM
    {ve_vl_nndm_mmm, 106499}, // __builtin_ve_vl_nndm_mmm
    {ve_vl_orm_MMM, 106508}, // __builtin_ve_vl_orm_MMM
    {ve_vl_orm_mmm, 106516}, // __builtin_ve_vl_orm_mmm
    {ve_vl_pack_f32a, 106524}, // __builtin_ve_vl_pack_f32a
    {ve_vl_pack_f32p, 106534}, // __builtin_ve_vl_pack_f32p
    {ve_vl_pcvm_sml, 106544}, // __builtin_ve_vl_pcvm_sml
    {ve_vl_pfchv_ssl, 106553}, // __builtin_ve_vl_pfchv_ssl
    {ve_vl_pfchvnc_ssl, 106563}, // __builtin_ve_vl_pfchvnc_ssl
    {ve_vl_pvadds_vsvMvl, 106575}, // __builtin_ve_vl_pvadds_vsvMvl
    {ve_vl_pvadds_vsvl, 106589}, // __builtin_ve_vl_pvadds_vsvl
    {ve_vl_pvadds_vsvvl, 106601}, // __builtin_ve_vl_pvadds_vsvvl
    {ve_vl_pvadds_vvvMvl, 106614}, // __builtin_ve_vl_pvadds_vvvMvl
    {ve_vl_pvadds_vvvl, 106628}, // __builtin_ve_vl_pvadds_vvvl
    {ve_vl_pvadds_vvvvl, 106640}, // __builtin_ve_vl_pvadds_vvvvl
    {ve_vl_pvaddu_vsvMvl, 106653}, // __builtin_ve_vl_pvaddu_vsvMvl
    {ve_vl_pvaddu_vsvl, 106667}, // __builtin_ve_vl_pvaddu_vsvl
    {ve_vl_pvaddu_vsvvl, 106679}, // __builtin_ve_vl_pvaddu_vsvvl
    {ve_vl_pvaddu_vvvMvl, 106692}, // __builtin_ve_vl_pvaddu_vvvMvl
    {ve_vl_pvaddu_vvvl, 106706}, // __builtin_ve_vl_pvaddu_vvvl
    {ve_vl_pvaddu_vvvvl, 106718}, // __builtin_ve_vl_pvaddu_vvvvl
    {ve_vl_pvand_vsvMvl, 106731}, // __builtin_ve_vl_pvand_vsvMvl
    {ve_vl_pvand_vsvl, 106744}, // __builtin_ve_vl_pvand_vsvl
    {ve_vl_pvand_vsvvl, 106755}, // __builtin_ve_vl_pvand_vsvvl
    {ve_vl_pvand_vvvMvl, 106767}, // __builtin_ve_vl_pvand_vvvMvl
    {ve_vl_pvand_vvvl, 106780}, // __builtin_ve_vl_pvand_vvvl
    {ve_vl_pvand_vvvvl, 106791}, // __builtin_ve_vl_pvand_vvvvl
    {ve_vl_pvbrd_vsMvl, 106803}, // __builtin_ve_vl_pvbrd_vsMvl
    {ve_vl_pvbrd_vsl, 106815}, // __builtin_ve_vl_pvbrd_vsl
    {ve_vl_pvbrd_vsvl, 106825}, // __builtin_ve_vl_pvbrd_vsvl
    {ve_vl_pvbrv_vvMvl, 106836}, // __builtin_ve_vl_pvbrv_vvMvl
    {ve_vl_pvbrv_vvl, 106848}, // __builtin_ve_vl_pvbrv_vvl
    {ve_vl_pvbrv_vvvl, 106858}, // __builtin_ve_vl_pvbrv_vvvl
    {ve_vl_pvbrvlo_vvl, 106869}, // __builtin_ve_vl_pvbrvlo_vvl
    {ve_vl_pvbrvlo_vvmvl, 106881}, // __builtin_ve_vl_pvbrvlo_vvmvl
    {ve_vl_pvbrvlo_vvvl, 106895}, // __builtin_ve_vl_pvbrvlo_vvvl
    {ve_vl_pvbrvup_vvl, 106908}, // __builtin_ve_vl_pvbrvup_vvl
    {ve_vl_pvbrvup_vvmvl, 106920}, // __builtin_ve_vl_pvbrvup_vvmvl
    {ve_vl_pvbrvup_vvvl, 106934}, // __builtin_ve_vl_pvbrvup_vvvl
    {ve_vl_pvcmps_vsvMvl, 106947}, // __builtin_ve_vl_pvcmps_vsvMvl
    {ve_vl_pvcmps_vsvl, 106961}, // __builtin_ve_vl_pvcmps_vsvl
    {ve_vl_pvcmps_vsvvl, 106973}, // __builtin_ve_vl_pvcmps_vsvvl
    {ve_vl_pvcmps_vvvMvl, 106986}, // __builtin_ve_vl_pvcmps_vvvMvl
    {ve_vl_pvcmps_vvvl, 107000}, // __builtin_ve_vl_pvcmps_vvvl
    {ve_vl_pvcmps_vvvvl, 107012}, // __builtin_ve_vl_pvcmps_vvvvl
    {ve_vl_pvcmpu_vsvMvl, 107025}, // __builtin_ve_vl_pvcmpu_vsvMvl
    {ve_vl_pvcmpu_vsvl, 107039}, // __builtin_ve_vl_pvcmpu_vsvl
    {ve_vl_pvcmpu_vsvvl, 107051}, // __builtin_ve_vl_pvcmpu_vsvvl
    {ve_vl_pvcmpu_vvvMvl, 107064}, // __builtin_ve_vl_pvcmpu_vvvMvl
    {ve_vl_pvcmpu_vvvl, 107078}, // __builtin_ve_vl_pvcmpu_vvvl
    {ve_vl_pvcmpu_vvvvl, 107090}, // __builtin_ve_vl_pvcmpu_vvvvl
    {ve_vl_pvcvtsw_vvl, 107103}, // __builtin_ve_vl_pvcvtsw_vvl
    {ve_vl_pvcvtsw_vvvl, 107115}, // __builtin_ve_vl_pvcvtsw_vvvl
    {ve_vl_pvcvtws_vvMvl, 107128}, // __builtin_ve_vl_pvcvtws_vvMvl
    {ve_vl_pvcvtws_vvl, 107142}, // __builtin_ve_vl_pvcvtws_vvl
    {ve_vl_pvcvtws_vvvl, 107154}, // __builtin_ve_vl_pvcvtws_vvvl
    {ve_vl_pvcvtwsrz_vvMvl, 107167}, // __builtin_ve_vl_pvcvtwsrz_vvMvl
    {ve_vl_pvcvtwsrz_vvl, 107183}, // __builtin_ve_vl_pvcvtwsrz_vvl
    {ve_vl_pvcvtwsrz_vvvl, 107197}, // __builtin_ve_vl_pvcvtwsrz_vvvl
    {ve_vl_pveqv_vsvMvl, 107212}, // __builtin_ve_vl_pveqv_vsvMvl
    {ve_vl_pveqv_vsvl, 107225}, // __builtin_ve_vl_pveqv_vsvl
    {ve_vl_pveqv_vsvvl, 107236}, // __builtin_ve_vl_pveqv_vsvvl
    {ve_vl_pveqv_vvvMvl, 107248}, // __builtin_ve_vl_pveqv_vvvMvl
    {ve_vl_pveqv_vvvl, 107261}, // __builtin_ve_vl_pveqv_vvvl
    {ve_vl_pveqv_vvvvl, 107272}, // __builtin_ve_vl_pveqv_vvvvl
    {ve_vl_pvfadd_vsvMvl, 107284}, // __builtin_ve_vl_pvfadd_vsvMvl
    {ve_vl_pvfadd_vsvl, 107298}, // __builtin_ve_vl_pvfadd_vsvl
    {ve_vl_pvfadd_vsvvl, 107310}, // __builtin_ve_vl_pvfadd_vsvvl
    {ve_vl_pvfadd_vvvMvl, 107323}, // __builtin_ve_vl_pvfadd_vvvMvl
    {ve_vl_pvfadd_vvvl, 107337}, // __builtin_ve_vl_pvfadd_vvvl
    {ve_vl_pvfadd_vvvvl, 107349}, // __builtin_ve_vl_pvfadd_vvvvl
    {ve_vl_pvfcmp_vsvMvl, 107362}, // __builtin_ve_vl_pvfcmp_vsvMvl
    {ve_vl_pvfcmp_vsvl, 107376}, // __builtin_ve_vl_pvfcmp_vsvl
    {ve_vl_pvfcmp_vsvvl, 107388}, // __builtin_ve_vl_pvfcmp_vsvvl
    {ve_vl_pvfcmp_vvvMvl, 107401}, // __builtin_ve_vl_pvfcmp_vvvMvl
    {ve_vl_pvfcmp_vvvl, 107415}, // __builtin_ve_vl_pvfcmp_vvvl
    {ve_vl_pvfcmp_vvvvl, 107427}, // __builtin_ve_vl_pvfcmp_vvvvl
    {ve_vl_pvfmad_vsvvMvl, 107440}, // __builtin_ve_vl_pvfmad_vsvvMvl
    {ve_vl_pvfmad_vsvvl, 107455}, // __builtin_ve_vl_pvfmad_vsvvl
    {ve_vl_pvfmad_vsvvvl, 107468}, // __builtin_ve_vl_pvfmad_vsvvvl
    {ve_vl_pvfmad_vvsvMvl, 107482}, // __builtin_ve_vl_pvfmad_vvsvMvl
    {ve_vl_pvfmad_vvsvl, 107497}, // __builtin_ve_vl_pvfmad_vvsvl
    {ve_vl_pvfmad_vvsvvl, 107510}, // __builtin_ve_vl_pvfmad_vvsvvl
    {ve_vl_pvfmad_vvvvMvl, 107524}, // __builtin_ve_vl_pvfmad_vvvvMvl
    {ve_vl_pvfmad_vvvvl, 107539}, // __builtin_ve_vl_pvfmad_vvvvl
    {ve_vl_pvfmad_vvvvvl, 107552}, // __builtin_ve_vl_pvfmad_vvvvvl
    {ve_vl_pvfmax_vsvMvl, 107566}, // __builtin_ve_vl_pvfmax_vsvMvl
    {ve_vl_pvfmax_vsvl, 107580}, // __builtin_ve_vl_pvfmax_vsvl
    {ve_vl_pvfmax_vsvvl, 107592}, // __builtin_ve_vl_pvfmax_vsvvl
    {ve_vl_pvfmax_vvvMvl, 107605}, // __builtin_ve_vl_pvfmax_vvvMvl
    {ve_vl_pvfmax_vvvl, 107619}, // __builtin_ve_vl_pvfmax_vvvl
    {ve_vl_pvfmax_vvvvl, 107631}, // __builtin_ve_vl_pvfmax_vvvvl
    {ve_vl_pvfmin_vsvMvl, 107644}, // __builtin_ve_vl_pvfmin_vsvMvl
    {ve_vl_pvfmin_vsvl, 107658}, // __builtin_ve_vl_pvfmin_vsvl
    {ve_vl_pvfmin_vsvvl, 107670}, // __builtin_ve_vl_pvfmin_vsvvl
    {ve_vl_pvfmin_vvvMvl, 107683}, // __builtin_ve_vl_pvfmin_vvvMvl
    {ve_vl_pvfmin_vvvl, 107697}, // __builtin_ve_vl_pvfmin_vvvl
    {ve_vl_pvfmin_vvvvl, 107709}, // __builtin_ve_vl_pvfmin_vvvvl
    {ve_vl_pvfmkaf_Ml, 107722}, // __builtin_ve_vl_pvfmkaf_Ml
    {ve_vl_pvfmkat_Ml, 107733}, // __builtin_ve_vl_pvfmkat_Ml
    {ve_vl_pvfmkseq_MvMl, 107744}, // __builtin_ve_vl_pvfmkseq_MvMl
    {ve_vl_pvfmkseq_Mvl, 107758}, // __builtin_ve_vl_pvfmkseq_Mvl
    {ve_vl_pvfmkseqnan_MvMl, 107771}, // __builtin_ve_vl_pvfmkseqnan_MvMl
    {ve_vl_pvfmkseqnan_Mvl, 107788}, // __builtin_ve_vl_pvfmkseqnan_Mvl
    {ve_vl_pvfmksge_MvMl, 107804}, // __builtin_ve_vl_pvfmksge_MvMl
    {ve_vl_pvfmksge_Mvl, 107818}, // __builtin_ve_vl_pvfmksge_Mvl
    {ve_vl_pvfmksgenan_MvMl, 107831}, // __builtin_ve_vl_pvfmksgenan_MvMl
    {ve_vl_pvfmksgenan_Mvl, 107848}, // __builtin_ve_vl_pvfmksgenan_Mvl
    {ve_vl_pvfmksgt_MvMl, 107864}, // __builtin_ve_vl_pvfmksgt_MvMl
    {ve_vl_pvfmksgt_Mvl, 107878}, // __builtin_ve_vl_pvfmksgt_Mvl
    {ve_vl_pvfmksgtnan_MvMl, 107891}, // __builtin_ve_vl_pvfmksgtnan_MvMl
    {ve_vl_pvfmksgtnan_Mvl, 107908}, // __builtin_ve_vl_pvfmksgtnan_Mvl
    {ve_vl_pvfmksle_MvMl, 107924}, // __builtin_ve_vl_pvfmksle_MvMl
    {ve_vl_pvfmksle_Mvl, 107938}, // __builtin_ve_vl_pvfmksle_Mvl
    {ve_vl_pvfmkslenan_MvMl, 107951}, // __builtin_ve_vl_pvfmkslenan_MvMl
    {ve_vl_pvfmkslenan_Mvl, 107968}, // __builtin_ve_vl_pvfmkslenan_Mvl
    {ve_vl_pvfmksloeq_mvl, 107984}, // __builtin_ve_vl_pvfmksloeq_mvl
    {ve_vl_pvfmksloeq_mvml, 107999}, // __builtin_ve_vl_pvfmksloeq_mvml
    {ve_vl_pvfmksloeqnan_mvl, 108015}, // __builtin_ve_vl_pvfmksloeqnan_mvl
    {ve_vl_pvfmksloeqnan_mvml, 108033}, // __builtin_ve_vl_pvfmksloeqnan_mvml
    {ve_vl_pvfmksloge_mvl, 108052}, // __builtin_ve_vl_pvfmksloge_mvl
    {ve_vl_pvfmksloge_mvml, 108067}, // __builtin_ve_vl_pvfmksloge_mvml
    {ve_vl_pvfmkslogenan_mvl, 108083}, // __builtin_ve_vl_pvfmkslogenan_mvl
    {ve_vl_pvfmkslogenan_mvml, 108101}, // __builtin_ve_vl_pvfmkslogenan_mvml
    {ve_vl_pvfmkslogt_mvl, 108120}, // __builtin_ve_vl_pvfmkslogt_mvl
    {ve_vl_pvfmkslogt_mvml, 108135}, // __builtin_ve_vl_pvfmkslogt_mvml
    {ve_vl_pvfmkslogtnan_mvl, 108151}, // __builtin_ve_vl_pvfmkslogtnan_mvl
    {ve_vl_pvfmkslogtnan_mvml, 108169}, // __builtin_ve_vl_pvfmkslogtnan_mvml
    {ve_vl_pvfmkslole_mvl, 108188}, // __builtin_ve_vl_pvfmkslole_mvl
    {ve_vl_pvfmkslole_mvml, 108203}, // __builtin_ve_vl_pvfmkslole_mvml
    {ve_vl_pvfmkslolenan_mvl, 108219}, // __builtin_ve_vl_pvfmkslolenan_mvl
    {ve_vl_pvfmkslolenan_mvml, 108237}, // __builtin_ve_vl_pvfmkslolenan_mvml
    {ve_vl_pvfmkslolt_mvl, 108256}, // __builtin_ve_vl_pvfmkslolt_mvl
    {ve_vl_pvfmkslolt_mvml, 108271}, // __builtin_ve_vl_pvfmkslolt_mvml
    {ve_vl_pvfmksloltnan_mvl, 108287}, // __builtin_ve_vl_pvfmksloltnan_mvl
    {ve_vl_pvfmksloltnan_mvml, 108305}, // __builtin_ve_vl_pvfmksloltnan_mvml
    {ve_vl_pvfmkslonan_mvl, 108324}, // __builtin_ve_vl_pvfmkslonan_mvl
    {ve_vl_pvfmkslonan_mvml, 108340}, // __builtin_ve_vl_pvfmkslonan_mvml
    {ve_vl_pvfmkslone_mvl, 108357}, // __builtin_ve_vl_pvfmkslone_mvl
    {ve_vl_pvfmkslone_mvml, 108372}, // __builtin_ve_vl_pvfmkslone_mvml
    {ve_vl_pvfmkslonenan_mvl, 108388}, // __builtin_ve_vl_pvfmkslonenan_mvl
    {ve_vl_pvfmkslonenan_mvml, 108406}, // __builtin_ve_vl_pvfmkslonenan_mvml
    {ve_vl_pvfmkslonum_mvl, 108425}, // __builtin_ve_vl_pvfmkslonum_mvl
    {ve_vl_pvfmkslonum_mvml, 108441}, // __builtin_ve_vl_pvfmkslonum_mvml
    {ve_vl_pvfmkslt_MvMl, 108458}, // __builtin_ve_vl_pvfmkslt_MvMl
    {ve_vl_pvfmkslt_Mvl, 108472}, // __builtin_ve_vl_pvfmkslt_Mvl
    {ve_vl_pvfmksltnan_MvMl, 108485}, // __builtin_ve_vl_pvfmksltnan_MvMl
    {ve_vl_pvfmksltnan_Mvl, 108502}, // __builtin_ve_vl_pvfmksltnan_Mvl
    {ve_vl_pvfmksnan_MvMl, 108518}, // __builtin_ve_vl_pvfmksnan_MvMl
    {ve_vl_pvfmksnan_Mvl, 108533}, // __builtin_ve_vl_pvfmksnan_Mvl
    {ve_vl_pvfmksne_MvMl, 108547}, // __builtin_ve_vl_pvfmksne_MvMl
    {ve_vl_pvfmksne_Mvl, 108561}, // __builtin_ve_vl_pvfmksne_Mvl
    {ve_vl_pvfmksnenan_MvMl, 108574}, // __builtin_ve_vl_pvfmksnenan_MvMl
    {ve_vl_pvfmksnenan_Mvl, 108591}, // __builtin_ve_vl_pvfmksnenan_Mvl
    {ve_vl_pvfmksnum_MvMl, 108607}, // __builtin_ve_vl_pvfmksnum_MvMl
    {ve_vl_pvfmksnum_Mvl, 108622}, // __builtin_ve_vl_pvfmksnum_Mvl
    {ve_vl_pvfmksupeq_mvl, 108636}, // __builtin_ve_vl_pvfmksupeq_mvl
    {ve_vl_pvfmksupeq_mvml, 108651}, // __builtin_ve_vl_pvfmksupeq_mvml
    {ve_vl_pvfmksupeqnan_mvl, 108667}, // __builtin_ve_vl_pvfmksupeqnan_mvl
    {ve_vl_pvfmksupeqnan_mvml, 108685}, // __builtin_ve_vl_pvfmksupeqnan_mvml
    {ve_vl_pvfmksupge_mvl, 108704}, // __builtin_ve_vl_pvfmksupge_mvl
    {ve_vl_pvfmksupge_mvml, 108719}, // __builtin_ve_vl_pvfmksupge_mvml
    {ve_vl_pvfmksupgenan_mvl, 108735}, // __builtin_ve_vl_pvfmksupgenan_mvl
    {ve_vl_pvfmksupgenan_mvml, 108753}, // __builtin_ve_vl_pvfmksupgenan_mvml
    {ve_vl_pvfmksupgt_mvl, 108772}, // __builtin_ve_vl_pvfmksupgt_mvl
    {ve_vl_pvfmksupgt_mvml, 108787}, // __builtin_ve_vl_pvfmksupgt_mvml
    {ve_vl_pvfmksupgtnan_mvl, 108803}, // __builtin_ve_vl_pvfmksupgtnan_mvl
    {ve_vl_pvfmksupgtnan_mvml, 108821}, // __builtin_ve_vl_pvfmksupgtnan_mvml
    {ve_vl_pvfmksuple_mvl, 108840}, // __builtin_ve_vl_pvfmksuple_mvl
    {ve_vl_pvfmksuple_mvml, 108855}, // __builtin_ve_vl_pvfmksuple_mvml
    {ve_vl_pvfmksuplenan_mvl, 108871}, // __builtin_ve_vl_pvfmksuplenan_mvl
    {ve_vl_pvfmksuplenan_mvml, 108889}, // __builtin_ve_vl_pvfmksuplenan_mvml
    {ve_vl_pvfmksuplt_mvl, 108908}, // __builtin_ve_vl_pvfmksuplt_mvl
    {ve_vl_pvfmksuplt_mvml, 108923}, // __builtin_ve_vl_pvfmksuplt_mvml
    {ve_vl_pvfmksupltnan_mvl, 108939}, // __builtin_ve_vl_pvfmksupltnan_mvl
    {ve_vl_pvfmksupltnan_mvml, 108957}, // __builtin_ve_vl_pvfmksupltnan_mvml
    {ve_vl_pvfmksupnan_mvl, 108976}, // __builtin_ve_vl_pvfmksupnan_mvl
    {ve_vl_pvfmksupnan_mvml, 108992}, // __builtin_ve_vl_pvfmksupnan_mvml
    {ve_vl_pvfmksupne_mvl, 109009}, // __builtin_ve_vl_pvfmksupne_mvl
    {ve_vl_pvfmksupne_mvml, 109024}, // __builtin_ve_vl_pvfmksupne_mvml
    {ve_vl_pvfmksupnenan_mvl, 109040}, // __builtin_ve_vl_pvfmksupnenan_mvl
    {ve_vl_pvfmksupnenan_mvml, 109058}, // __builtin_ve_vl_pvfmksupnenan_mvml
    {ve_vl_pvfmksupnum_mvl, 109077}, // __builtin_ve_vl_pvfmksupnum_mvl
    {ve_vl_pvfmksupnum_mvml, 109093}, // __builtin_ve_vl_pvfmksupnum_mvml
    {ve_vl_pvfmkweq_MvMl, 109110}, // __builtin_ve_vl_pvfmkweq_MvMl
    {ve_vl_pvfmkweq_Mvl, 109124}, // __builtin_ve_vl_pvfmkweq_Mvl
    {ve_vl_pvfmkweqnan_MvMl, 109137}, // __builtin_ve_vl_pvfmkweqnan_MvMl
    {ve_vl_pvfmkweqnan_Mvl, 109154}, // __builtin_ve_vl_pvfmkweqnan_Mvl
    {ve_vl_pvfmkwge_MvMl, 109170}, // __builtin_ve_vl_pvfmkwge_MvMl
    {ve_vl_pvfmkwge_Mvl, 109184}, // __builtin_ve_vl_pvfmkwge_Mvl
    {ve_vl_pvfmkwgenan_MvMl, 109197}, // __builtin_ve_vl_pvfmkwgenan_MvMl
    {ve_vl_pvfmkwgenan_Mvl, 109214}, // __builtin_ve_vl_pvfmkwgenan_Mvl
    {ve_vl_pvfmkwgt_MvMl, 109230}, // __builtin_ve_vl_pvfmkwgt_MvMl
    {ve_vl_pvfmkwgt_Mvl, 109244}, // __builtin_ve_vl_pvfmkwgt_Mvl
    {ve_vl_pvfmkwgtnan_MvMl, 109257}, // __builtin_ve_vl_pvfmkwgtnan_MvMl
    {ve_vl_pvfmkwgtnan_Mvl, 109274}, // __builtin_ve_vl_pvfmkwgtnan_Mvl
    {ve_vl_pvfmkwle_MvMl, 109290}, // __builtin_ve_vl_pvfmkwle_MvMl
    {ve_vl_pvfmkwle_Mvl, 109304}, // __builtin_ve_vl_pvfmkwle_Mvl
    {ve_vl_pvfmkwlenan_MvMl, 109317}, // __builtin_ve_vl_pvfmkwlenan_MvMl
    {ve_vl_pvfmkwlenan_Mvl, 109334}, // __builtin_ve_vl_pvfmkwlenan_Mvl
    {ve_vl_pvfmkwloeq_mvl, 109350}, // __builtin_ve_vl_pvfmkwloeq_mvl
    {ve_vl_pvfmkwloeq_mvml, 109365}, // __builtin_ve_vl_pvfmkwloeq_mvml
    {ve_vl_pvfmkwloeqnan_mvl, 109381}, // __builtin_ve_vl_pvfmkwloeqnan_mvl
    {ve_vl_pvfmkwloeqnan_mvml, 109399}, // __builtin_ve_vl_pvfmkwloeqnan_mvml
    {ve_vl_pvfmkwloge_mvl, 109418}, // __builtin_ve_vl_pvfmkwloge_mvl
    {ve_vl_pvfmkwloge_mvml, 109433}, // __builtin_ve_vl_pvfmkwloge_mvml
    {ve_vl_pvfmkwlogenan_mvl, 109449}, // __builtin_ve_vl_pvfmkwlogenan_mvl
    {ve_vl_pvfmkwlogenan_mvml, 109467}, // __builtin_ve_vl_pvfmkwlogenan_mvml
    {ve_vl_pvfmkwlogt_mvl, 109486}, // __builtin_ve_vl_pvfmkwlogt_mvl
    {ve_vl_pvfmkwlogt_mvml, 109501}, // __builtin_ve_vl_pvfmkwlogt_mvml
    {ve_vl_pvfmkwlogtnan_mvl, 109517}, // __builtin_ve_vl_pvfmkwlogtnan_mvl
    {ve_vl_pvfmkwlogtnan_mvml, 109535}, // __builtin_ve_vl_pvfmkwlogtnan_mvml
    {ve_vl_pvfmkwlole_mvl, 109554}, // __builtin_ve_vl_pvfmkwlole_mvl
    {ve_vl_pvfmkwlole_mvml, 109569}, // __builtin_ve_vl_pvfmkwlole_mvml
    {ve_vl_pvfmkwlolenan_mvl, 109585}, // __builtin_ve_vl_pvfmkwlolenan_mvl
    {ve_vl_pvfmkwlolenan_mvml, 109603}, // __builtin_ve_vl_pvfmkwlolenan_mvml
    {ve_vl_pvfmkwlolt_mvl, 109622}, // __builtin_ve_vl_pvfmkwlolt_mvl
    {ve_vl_pvfmkwlolt_mvml, 109637}, // __builtin_ve_vl_pvfmkwlolt_mvml
    {ve_vl_pvfmkwloltnan_mvl, 109653}, // __builtin_ve_vl_pvfmkwloltnan_mvl
    {ve_vl_pvfmkwloltnan_mvml, 109671}, // __builtin_ve_vl_pvfmkwloltnan_mvml
    {ve_vl_pvfmkwlonan_mvl, 109690}, // __builtin_ve_vl_pvfmkwlonan_mvl
    {ve_vl_pvfmkwlonan_mvml, 109706}, // __builtin_ve_vl_pvfmkwlonan_mvml
    {ve_vl_pvfmkwlone_mvl, 109723}, // __builtin_ve_vl_pvfmkwlone_mvl
    {ve_vl_pvfmkwlone_mvml, 109738}, // __builtin_ve_vl_pvfmkwlone_mvml
    {ve_vl_pvfmkwlonenan_mvl, 109754}, // __builtin_ve_vl_pvfmkwlonenan_mvl
    {ve_vl_pvfmkwlonenan_mvml, 109772}, // __builtin_ve_vl_pvfmkwlonenan_mvml
    {ve_vl_pvfmkwlonum_mvl, 109791}, // __builtin_ve_vl_pvfmkwlonum_mvl
    {ve_vl_pvfmkwlonum_mvml, 109807}, // __builtin_ve_vl_pvfmkwlonum_mvml
    {ve_vl_pvfmkwlt_MvMl, 109824}, // __builtin_ve_vl_pvfmkwlt_MvMl
    {ve_vl_pvfmkwlt_Mvl, 109838}, // __builtin_ve_vl_pvfmkwlt_Mvl
    {ve_vl_pvfmkwltnan_MvMl, 109851}, // __builtin_ve_vl_pvfmkwltnan_MvMl
    {ve_vl_pvfmkwltnan_Mvl, 109868}, // __builtin_ve_vl_pvfmkwltnan_Mvl
    {ve_vl_pvfmkwnan_MvMl, 109884}, // __builtin_ve_vl_pvfmkwnan_MvMl
    {ve_vl_pvfmkwnan_Mvl, 109899}, // __builtin_ve_vl_pvfmkwnan_Mvl
    {ve_vl_pvfmkwne_MvMl, 109913}, // __builtin_ve_vl_pvfmkwne_MvMl
    {ve_vl_pvfmkwne_Mvl, 109927}, // __builtin_ve_vl_pvfmkwne_Mvl
    {ve_vl_pvfmkwnenan_MvMl, 109940}, // __builtin_ve_vl_pvfmkwnenan_MvMl
    {ve_vl_pvfmkwnenan_Mvl, 109957}, // __builtin_ve_vl_pvfmkwnenan_Mvl
    {ve_vl_pvfmkwnum_MvMl, 109973}, // __builtin_ve_vl_pvfmkwnum_MvMl
    {ve_vl_pvfmkwnum_Mvl, 109988}, // __builtin_ve_vl_pvfmkwnum_Mvl
    {ve_vl_pvfmkwupeq_mvl, 110002}, // __builtin_ve_vl_pvfmkwupeq_mvl
    {ve_vl_pvfmkwupeq_mvml, 110017}, // __builtin_ve_vl_pvfmkwupeq_mvml
    {ve_vl_pvfmkwupeqnan_mvl, 110033}, // __builtin_ve_vl_pvfmkwupeqnan_mvl
    {ve_vl_pvfmkwupeqnan_mvml, 110051}, // __builtin_ve_vl_pvfmkwupeqnan_mvml
    {ve_vl_pvfmkwupge_mvl, 110070}, // __builtin_ve_vl_pvfmkwupge_mvl
    {ve_vl_pvfmkwupge_mvml, 110085}, // __builtin_ve_vl_pvfmkwupge_mvml
    {ve_vl_pvfmkwupgenan_mvl, 110101}, // __builtin_ve_vl_pvfmkwupgenan_mvl
    {ve_vl_pvfmkwupgenan_mvml, 110119}, // __builtin_ve_vl_pvfmkwupgenan_mvml
    {ve_vl_pvfmkwupgt_mvl, 110138}, // __builtin_ve_vl_pvfmkwupgt_mvl
    {ve_vl_pvfmkwupgt_mvml, 110153}, // __builtin_ve_vl_pvfmkwupgt_mvml
    {ve_vl_pvfmkwupgtnan_mvl, 110169}, // __builtin_ve_vl_pvfmkwupgtnan_mvl
    {ve_vl_pvfmkwupgtnan_mvml, 110187}, // __builtin_ve_vl_pvfmkwupgtnan_mvml
    {ve_vl_pvfmkwuple_mvl, 110206}, // __builtin_ve_vl_pvfmkwuple_mvl
    {ve_vl_pvfmkwuple_mvml, 110221}, // __builtin_ve_vl_pvfmkwuple_mvml
    {ve_vl_pvfmkwuplenan_mvl, 110237}, // __builtin_ve_vl_pvfmkwuplenan_mvl
    {ve_vl_pvfmkwuplenan_mvml, 110255}, // __builtin_ve_vl_pvfmkwuplenan_mvml
    {ve_vl_pvfmkwuplt_mvl, 110274}, // __builtin_ve_vl_pvfmkwuplt_mvl
    {ve_vl_pvfmkwuplt_mvml, 110289}, // __builtin_ve_vl_pvfmkwuplt_mvml
    {ve_vl_pvfmkwupltnan_mvl, 110305}, // __builtin_ve_vl_pvfmkwupltnan_mvl
    {ve_vl_pvfmkwupltnan_mvml, 110323}, // __builtin_ve_vl_pvfmkwupltnan_mvml
    {ve_vl_pvfmkwupnan_mvl, 110342}, // __builtin_ve_vl_pvfmkwupnan_mvl
    {ve_vl_pvfmkwupnan_mvml, 110358}, // __builtin_ve_vl_pvfmkwupnan_mvml
    {ve_vl_pvfmkwupne_mvl, 110375}, // __builtin_ve_vl_pvfmkwupne_mvl
    {ve_vl_pvfmkwupne_mvml, 110390}, // __builtin_ve_vl_pvfmkwupne_mvml
    {ve_vl_pvfmkwupnenan_mvl, 110406}, // __builtin_ve_vl_pvfmkwupnenan_mvl
    {ve_vl_pvfmkwupnenan_mvml, 110424}, // __builtin_ve_vl_pvfmkwupnenan_mvml
    {ve_vl_pvfmkwupnum_mvl, 110443}, // __builtin_ve_vl_pvfmkwupnum_mvl
    {ve_vl_pvfmkwupnum_mvml, 110459}, // __builtin_ve_vl_pvfmkwupnum_mvml
    {ve_vl_pvfmsb_vsvvMvl, 110476}, // __builtin_ve_vl_pvfmsb_vsvvMvl
    {ve_vl_pvfmsb_vsvvl, 110491}, // __builtin_ve_vl_pvfmsb_vsvvl
    {ve_vl_pvfmsb_vsvvvl, 110504}, // __builtin_ve_vl_pvfmsb_vsvvvl
    {ve_vl_pvfmsb_vvsvMvl, 110518}, // __builtin_ve_vl_pvfmsb_vvsvMvl
    {ve_vl_pvfmsb_vvsvl, 110533}, // __builtin_ve_vl_pvfmsb_vvsvl
    {ve_vl_pvfmsb_vvsvvl, 110546}, // __builtin_ve_vl_pvfmsb_vvsvvl
    {ve_vl_pvfmsb_vvvvMvl, 110560}, // __builtin_ve_vl_pvfmsb_vvvvMvl
    {ve_vl_pvfmsb_vvvvl, 110575}, // __builtin_ve_vl_pvfmsb_vvvvl
    {ve_vl_pvfmsb_vvvvvl, 110588}, // __builtin_ve_vl_pvfmsb_vvvvvl
    {ve_vl_pvfmul_vsvMvl, 110602}, // __builtin_ve_vl_pvfmul_vsvMvl
    {ve_vl_pvfmul_vsvl, 110616}, // __builtin_ve_vl_pvfmul_vsvl
    {ve_vl_pvfmul_vsvvl, 110628}, // __builtin_ve_vl_pvfmul_vsvvl
    {ve_vl_pvfmul_vvvMvl, 110641}, // __builtin_ve_vl_pvfmul_vvvMvl
    {ve_vl_pvfmul_vvvl, 110655}, // __builtin_ve_vl_pvfmul_vvvl
    {ve_vl_pvfmul_vvvvl, 110667}, // __builtin_ve_vl_pvfmul_vvvvl
    {ve_vl_pvfnmad_vsvvMvl, 110680}, // __builtin_ve_vl_pvfnmad_vsvvMvl
    {ve_vl_pvfnmad_vsvvl, 110696}, // __builtin_ve_vl_pvfnmad_vsvvl
    {ve_vl_pvfnmad_vsvvvl, 110710}, // __builtin_ve_vl_pvfnmad_vsvvvl
    {ve_vl_pvfnmad_vvsvMvl, 110725}, // __builtin_ve_vl_pvfnmad_vvsvMvl
    {ve_vl_pvfnmad_vvsvl, 110741}, // __builtin_ve_vl_pvfnmad_vvsvl
    {ve_vl_pvfnmad_vvsvvl, 110755}, // __builtin_ve_vl_pvfnmad_vvsvvl
    {ve_vl_pvfnmad_vvvvMvl, 110770}, // __builtin_ve_vl_pvfnmad_vvvvMvl
    {ve_vl_pvfnmad_vvvvl, 110786}, // __builtin_ve_vl_pvfnmad_vvvvl
    {ve_vl_pvfnmad_vvvvvl, 110800}, // __builtin_ve_vl_pvfnmad_vvvvvl
    {ve_vl_pvfnmsb_vsvvMvl, 110815}, // __builtin_ve_vl_pvfnmsb_vsvvMvl
    {ve_vl_pvfnmsb_vsvvl, 110831}, // __builtin_ve_vl_pvfnmsb_vsvvl
    {ve_vl_pvfnmsb_vsvvvl, 110845}, // __builtin_ve_vl_pvfnmsb_vsvvvl
    {ve_vl_pvfnmsb_vvsvMvl, 110860}, // __builtin_ve_vl_pvfnmsb_vvsvMvl
    {ve_vl_pvfnmsb_vvsvl, 110876}, // __builtin_ve_vl_pvfnmsb_vvsvl
    {ve_vl_pvfnmsb_vvsvvl, 110890}, // __builtin_ve_vl_pvfnmsb_vvsvvl
    {ve_vl_pvfnmsb_vvvvMvl, 110905}, // __builtin_ve_vl_pvfnmsb_vvvvMvl
    {ve_vl_pvfnmsb_vvvvl, 110921}, // __builtin_ve_vl_pvfnmsb_vvvvl
    {ve_vl_pvfnmsb_vvvvvl, 110935}, // __builtin_ve_vl_pvfnmsb_vvvvvl
    {ve_vl_pvfsub_vsvMvl, 110950}, // __builtin_ve_vl_pvfsub_vsvMvl
    {ve_vl_pvfsub_vsvl, 110964}, // __builtin_ve_vl_pvfsub_vsvl
    {ve_vl_pvfsub_vsvvl, 110976}, // __builtin_ve_vl_pvfsub_vsvvl
    {ve_vl_pvfsub_vvvMvl, 110989}, // __builtin_ve_vl_pvfsub_vvvMvl
    {ve_vl_pvfsub_vvvl, 111003}, // __builtin_ve_vl_pvfsub_vvvl
    {ve_vl_pvfsub_vvvvl, 111015}, // __builtin_ve_vl_pvfsub_vvvvl
    {ve_vl_pvldz_vvMvl, 111028}, // __builtin_ve_vl_pvldz_vvMvl
    {ve_vl_pvldz_vvl, 111040}, // __builtin_ve_vl_pvldz_vvl
    {ve_vl_pvldz_vvvl, 111050}, // __builtin_ve_vl_pvldz_vvvl
    {ve_vl_pvldzlo_vvl, 111061}, // __builtin_ve_vl_pvldzlo_vvl
    {ve_vl_pvldzlo_vvmvl, 111073}, // __builtin_ve_vl_pvldzlo_vvmvl
    {ve_vl_pvldzlo_vvvl, 111087}, // __builtin_ve_vl_pvldzlo_vvvl
    {ve_vl_pvldzup_vvl, 111100}, // __builtin_ve_vl_pvldzup_vvl
    {ve_vl_pvldzup_vvmvl, 111112}, // __builtin_ve_vl_pvldzup_vvmvl
    {ve_vl_pvldzup_vvvl, 111126}, // __builtin_ve_vl_pvldzup_vvvl
    {ve_vl_pvmaxs_vsvMvl, 111139}, // __builtin_ve_vl_pvmaxs_vsvMvl
    {ve_vl_pvmaxs_vsvl, 111153}, // __builtin_ve_vl_pvmaxs_vsvl
    {ve_vl_pvmaxs_vsvvl, 111165}, // __builtin_ve_vl_pvmaxs_vsvvl
    {ve_vl_pvmaxs_vvvMvl, 111178}, // __builtin_ve_vl_pvmaxs_vvvMvl
    {ve_vl_pvmaxs_vvvl, 111192}, // __builtin_ve_vl_pvmaxs_vvvl
    {ve_vl_pvmaxs_vvvvl, 111204}, // __builtin_ve_vl_pvmaxs_vvvvl
    {ve_vl_pvmins_vsvMvl, 111217}, // __builtin_ve_vl_pvmins_vsvMvl
    {ve_vl_pvmins_vsvl, 111231}, // __builtin_ve_vl_pvmins_vsvl
    {ve_vl_pvmins_vsvvl, 111243}, // __builtin_ve_vl_pvmins_vsvvl
    {ve_vl_pvmins_vvvMvl, 111256}, // __builtin_ve_vl_pvmins_vvvMvl
    {ve_vl_pvmins_vvvl, 111270}, // __builtin_ve_vl_pvmins_vvvl
    {ve_vl_pvmins_vvvvl, 111282}, // __builtin_ve_vl_pvmins_vvvvl
    {ve_vl_pvor_vsvMvl, 111295}, // __builtin_ve_vl_pvor_vsvMvl
    {ve_vl_pvor_vsvl, 111307}, // __builtin_ve_vl_pvor_vsvl
    {ve_vl_pvor_vsvvl, 111317}, // __builtin_ve_vl_pvor_vsvvl
    {ve_vl_pvor_vvvMvl, 111328}, // __builtin_ve_vl_pvor_vvvMvl
    {ve_vl_pvor_vvvl, 111340}, // __builtin_ve_vl_pvor_vvvl
    {ve_vl_pvor_vvvvl, 111350}, // __builtin_ve_vl_pvor_vvvvl
    {ve_vl_pvpcnt_vvMvl, 111361}, // __builtin_ve_vl_pvpcnt_vvMvl
    {ve_vl_pvpcnt_vvl, 111374}, // __builtin_ve_vl_pvpcnt_vvl
    {ve_vl_pvpcnt_vvvl, 111385}, // __builtin_ve_vl_pvpcnt_vvvl
    {ve_vl_pvpcntlo_vvl, 111397}, // __builtin_ve_vl_pvpcntlo_vvl
    {ve_vl_pvpcntlo_vvmvl, 111410}, // __builtin_ve_vl_pvpcntlo_vvmvl
    {ve_vl_pvpcntlo_vvvl, 111425}, // __builtin_ve_vl_pvpcntlo_vvvl
    {ve_vl_pvpcntup_vvl, 111439}, // __builtin_ve_vl_pvpcntup_vvl
    {ve_vl_pvpcntup_vvmvl, 111452}, // __builtin_ve_vl_pvpcntup_vvmvl
    {ve_vl_pvpcntup_vvvl, 111467}, // __builtin_ve_vl_pvpcntup_vvvl
    {ve_vl_pvrcp_vvl, 111481}, // __builtin_ve_vl_pvrcp_vvl
    {ve_vl_pvrcp_vvvl, 111491}, // __builtin_ve_vl_pvrcp_vvvl
    {ve_vl_pvrsqrt_vvl, 111502}, // __builtin_ve_vl_pvrsqrt_vvl
    {ve_vl_pvrsqrt_vvvl, 111514}, // __builtin_ve_vl_pvrsqrt_vvvl
    {ve_vl_pvrsqrtnex_vvl, 111527}, // __builtin_ve_vl_pvrsqrtnex_vvl
    {ve_vl_pvrsqrtnex_vvvl, 111542}, // __builtin_ve_vl_pvrsqrtnex_vvvl
    {ve_vl_pvseq_vl, 111558}, // __builtin_ve_vl_pvseq_vl
    {ve_vl_pvseq_vvl, 111567}, // __builtin_ve_vl_pvseq_vvl
    {ve_vl_pvseqlo_vl, 111577}, // __builtin_ve_vl_pvseqlo_vl
    {ve_vl_pvseqlo_vvl, 111588}, // __builtin_ve_vl_pvseqlo_vvl
    {ve_vl_pvsequp_vl, 111600}, // __builtin_ve_vl_pvsequp_vl
    {ve_vl_pvsequp_vvl, 111611}, // __builtin_ve_vl_pvsequp_vvl
    {ve_vl_pvsla_vvsMvl, 111623}, // __builtin_ve_vl_pvsla_vvsMvl
    {ve_vl_pvsla_vvsl, 111636}, // __builtin_ve_vl_pvsla_vvsl
    {ve_vl_pvsla_vvsvl, 111647}, // __builtin_ve_vl_pvsla_vvsvl
    {ve_vl_pvsla_vvvMvl, 111659}, // __builtin_ve_vl_pvsla_vvvMvl
    {ve_vl_pvsla_vvvl, 111672}, // __builtin_ve_vl_pvsla_vvvl
    {ve_vl_pvsla_vvvvl, 111683}, // __builtin_ve_vl_pvsla_vvvvl
    {ve_vl_pvsll_vvsMvl, 111695}, // __builtin_ve_vl_pvsll_vvsMvl
    {ve_vl_pvsll_vvsl, 111708}, // __builtin_ve_vl_pvsll_vvsl
    {ve_vl_pvsll_vvsvl, 111719}, // __builtin_ve_vl_pvsll_vvsvl
    {ve_vl_pvsll_vvvMvl, 111731}, // __builtin_ve_vl_pvsll_vvvMvl
    {ve_vl_pvsll_vvvl, 111744}, // __builtin_ve_vl_pvsll_vvvl
    {ve_vl_pvsll_vvvvl, 111755}, // __builtin_ve_vl_pvsll_vvvvl
    {ve_vl_pvsra_vvsMvl, 111767}, // __builtin_ve_vl_pvsra_vvsMvl
    {ve_vl_pvsra_vvsl, 111780}, // __builtin_ve_vl_pvsra_vvsl
    {ve_vl_pvsra_vvsvl, 111791}, // __builtin_ve_vl_pvsra_vvsvl
    {ve_vl_pvsra_vvvMvl, 111803}, // __builtin_ve_vl_pvsra_vvvMvl
    {ve_vl_pvsra_vvvl, 111816}, // __builtin_ve_vl_pvsra_vvvl
    {ve_vl_pvsra_vvvvl, 111827}, // __builtin_ve_vl_pvsra_vvvvl
    {ve_vl_pvsrl_vvsMvl, 111839}, // __builtin_ve_vl_pvsrl_vvsMvl
    {ve_vl_pvsrl_vvsl, 111852}, // __builtin_ve_vl_pvsrl_vvsl
    {ve_vl_pvsrl_vvsvl, 111863}, // __builtin_ve_vl_pvsrl_vvsvl
    {ve_vl_pvsrl_vvvMvl, 111875}, // __builtin_ve_vl_pvsrl_vvvMvl
    {ve_vl_pvsrl_vvvl, 111888}, // __builtin_ve_vl_pvsrl_vvvl
    {ve_vl_pvsrl_vvvvl, 111899}, // __builtin_ve_vl_pvsrl_vvvvl
    {ve_vl_pvsubs_vsvMvl, 111911}, // __builtin_ve_vl_pvsubs_vsvMvl
    {ve_vl_pvsubs_vsvl, 111925}, // __builtin_ve_vl_pvsubs_vsvl
    {ve_vl_pvsubs_vsvvl, 111937}, // __builtin_ve_vl_pvsubs_vsvvl
    {ve_vl_pvsubs_vvvMvl, 111950}, // __builtin_ve_vl_pvsubs_vvvMvl
    {ve_vl_pvsubs_vvvl, 111964}, // __builtin_ve_vl_pvsubs_vvvl
    {ve_vl_pvsubs_vvvvl, 111976}, // __builtin_ve_vl_pvsubs_vvvvl
    {ve_vl_pvsubu_vsvMvl, 111989}, // __builtin_ve_vl_pvsubu_vsvMvl
    {ve_vl_pvsubu_vsvl, 112003}, // __builtin_ve_vl_pvsubu_vsvl
    {ve_vl_pvsubu_vsvvl, 112015}, // __builtin_ve_vl_pvsubu_vsvvl
    {ve_vl_pvsubu_vvvMvl, 112028}, // __builtin_ve_vl_pvsubu_vvvMvl
    {ve_vl_pvsubu_vvvl, 112042}, // __builtin_ve_vl_pvsubu_vvvl
    {ve_vl_pvsubu_vvvvl, 112054}, // __builtin_ve_vl_pvsubu_vvvvl
    {ve_vl_pvxor_vsvMvl, 112067}, // __builtin_ve_vl_pvxor_vsvMvl
    {ve_vl_pvxor_vsvl, 112080}, // __builtin_ve_vl_pvxor_vsvl
    {ve_vl_pvxor_vsvvl, 112091}, // __builtin_ve_vl_pvxor_vsvvl
    {ve_vl_pvxor_vvvMvl, 112103}, // __builtin_ve_vl_pvxor_vvvMvl
    {ve_vl_pvxor_vvvl, 112116}, // __builtin_ve_vl_pvxor_vvvl
    {ve_vl_pvxor_vvvvl, 112127}, // __builtin_ve_vl_pvxor_vvvvl
    {ve_vl_scr_sss, 112139}, // __builtin_ve_vl_scr_sss
    {ve_vl_svm_sMs, 112147}, // __builtin_ve_vl_svm_sMs
    {ve_vl_svm_sms, 112155}, // __builtin_ve_vl_svm_sms
    {ve_vl_svob, 112163}, // __builtin_ve_vl_svob
    {ve_vl_tovm_sml, 112168}, // __builtin_ve_vl_tovm_sml
    {ve_vl_tscr_ssss, 112177}, // __builtin_ve_vl_tscr_ssss
    {ve_vl_vaddsl_vsvl, 112187}, // __builtin_ve_vl_vaddsl_vsvl
    {ve_vl_vaddsl_vsvmvl, 112199}, // __builtin_ve_vl_vaddsl_vsvmvl
    {ve_vl_vaddsl_vsvvl, 112213}, // __builtin_ve_vl_vaddsl_vsvvl
    {ve_vl_vaddsl_vvvl, 112226}, // __builtin_ve_vl_vaddsl_vvvl
    {ve_vl_vaddsl_vvvmvl, 112238}, // __builtin_ve_vl_vaddsl_vvvmvl
    {ve_vl_vaddsl_vvvvl, 112252}, // __builtin_ve_vl_vaddsl_vvvvl
    {ve_vl_vaddswsx_vsvl, 112265}, // __builtin_ve_vl_vaddswsx_vsvl
    {ve_vl_vaddswsx_vsvmvl, 112279}, // __builtin_ve_vl_vaddswsx_vsvmvl
    {ve_vl_vaddswsx_vsvvl, 112295}, // __builtin_ve_vl_vaddswsx_vsvvl
    {ve_vl_vaddswsx_vvvl, 112310}, // __builtin_ve_vl_vaddswsx_vvvl
    {ve_vl_vaddswsx_vvvmvl, 112324}, // __builtin_ve_vl_vaddswsx_vvvmvl
    {ve_vl_vaddswsx_vvvvl, 112340}, // __builtin_ve_vl_vaddswsx_vvvvl
    {ve_vl_vaddswzx_vsvl, 112355}, // __builtin_ve_vl_vaddswzx_vsvl
    {ve_vl_vaddswzx_vsvmvl, 112369}, // __builtin_ve_vl_vaddswzx_vsvmvl
    {ve_vl_vaddswzx_vsvvl, 112385}, // __builtin_ve_vl_vaddswzx_vsvvl
    {ve_vl_vaddswzx_vvvl, 112400}, // __builtin_ve_vl_vaddswzx_vvvl
    {ve_vl_vaddswzx_vvvmvl, 112414}, // __builtin_ve_vl_vaddswzx_vvvmvl
    {ve_vl_vaddswzx_vvvvl, 112430}, // __builtin_ve_vl_vaddswzx_vvvvl
    {ve_vl_vaddul_vsvl, 112445}, // __builtin_ve_vl_vaddul_vsvl
    {ve_vl_vaddul_vsvmvl, 112457}, // __builtin_ve_vl_vaddul_vsvmvl
    {ve_vl_vaddul_vsvvl, 112471}, // __builtin_ve_vl_vaddul_vsvvl
    {ve_vl_vaddul_vvvl, 112484}, // __builtin_ve_vl_vaddul_vvvl
    {ve_vl_vaddul_vvvmvl, 112496}, // __builtin_ve_vl_vaddul_vvvmvl
    {ve_vl_vaddul_vvvvl, 112510}, // __builtin_ve_vl_vaddul_vvvvl
    {ve_vl_vadduw_vsvl, 112523}, // __builtin_ve_vl_vadduw_vsvl
    {ve_vl_vadduw_vsvmvl, 112535}, // __builtin_ve_vl_vadduw_vsvmvl
    {ve_vl_vadduw_vsvvl, 112549}, // __builtin_ve_vl_vadduw_vsvvl
    {ve_vl_vadduw_vvvl, 112562}, // __builtin_ve_vl_vadduw_vvvl
    {ve_vl_vadduw_vvvmvl, 112574}, // __builtin_ve_vl_vadduw_vvvmvl
    {ve_vl_vadduw_vvvvl, 112588}, // __builtin_ve_vl_vadduw_vvvvl
    {ve_vl_vand_vsvl, 112601}, // __builtin_ve_vl_vand_vsvl
    {ve_vl_vand_vsvmvl, 112611}, // __builtin_ve_vl_vand_vsvmvl
    {ve_vl_vand_vsvvl, 112623}, // __builtin_ve_vl_vand_vsvvl
    {ve_vl_vand_vvvl, 112634}, // __builtin_ve_vl_vand_vvvl
    {ve_vl_vand_vvvmvl, 112644}, // __builtin_ve_vl_vand_vvvmvl
    {ve_vl_vand_vvvvl, 112656}, // __builtin_ve_vl_vand_vvvvl
    {ve_vl_vbrdd_vsl, 112667}, // __builtin_ve_vl_vbrdd_vsl
    {ve_vl_vbrdd_vsmvl, 112677}, // __builtin_ve_vl_vbrdd_vsmvl
    {ve_vl_vbrdd_vsvl, 112689}, // __builtin_ve_vl_vbrdd_vsvl
    {ve_vl_vbrdl_vsl, 112700}, // __builtin_ve_vl_vbrdl_vsl
    {ve_vl_vbrdl_vsmvl, 112710}, // __builtin_ve_vl_vbrdl_vsmvl
    {ve_vl_vbrdl_vsvl, 112722}, // __builtin_ve_vl_vbrdl_vsvl
    {ve_vl_vbrds_vsl, 112733}, // __builtin_ve_vl_vbrds_vsl
    {ve_vl_vbrds_vsmvl, 112743}, // __builtin_ve_vl_vbrds_vsmvl
    {ve_vl_vbrds_vsvl, 112755}, // __builtin_ve_vl_vbrds_vsvl
    {ve_vl_vbrdw_vsl, 112766}, // __builtin_ve_vl_vbrdw_vsl
    {ve_vl_vbrdw_vsmvl, 112776}, // __builtin_ve_vl_vbrdw_vsmvl
    {ve_vl_vbrdw_vsvl, 112788}, // __builtin_ve_vl_vbrdw_vsvl
    {ve_vl_vbrv_vvl, 112799}, // __builtin_ve_vl_vbrv_vvl
    {ve_vl_vbrv_vvmvl, 112808}, // __builtin_ve_vl_vbrv_vvmvl
    {ve_vl_vbrv_vvvl, 112819}, // __builtin_ve_vl_vbrv_vvvl
    {ve_vl_vcmpsl_vsvl, 112829}, // __builtin_ve_vl_vcmpsl_vsvl
    {ve_vl_vcmpsl_vsvmvl, 112841}, // __builtin_ve_vl_vcmpsl_vsvmvl
    {ve_vl_vcmpsl_vsvvl, 112855}, // __builtin_ve_vl_vcmpsl_vsvvl
    {ve_vl_vcmpsl_vvvl, 112868}, // __builtin_ve_vl_vcmpsl_vvvl
    {ve_vl_vcmpsl_vvvmvl, 112880}, // __builtin_ve_vl_vcmpsl_vvvmvl
    {ve_vl_vcmpsl_vvvvl, 112894}, // __builtin_ve_vl_vcmpsl_vvvvl
    {ve_vl_vcmpswsx_vsvl, 112907}, // __builtin_ve_vl_vcmpswsx_vsvl
    {ve_vl_vcmpswsx_vsvmvl, 112921}, // __builtin_ve_vl_vcmpswsx_vsvmvl
    {ve_vl_vcmpswsx_vsvvl, 112937}, // __builtin_ve_vl_vcmpswsx_vsvvl
    {ve_vl_vcmpswsx_vvvl, 112952}, // __builtin_ve_vl_vcmpswsx_vvvl
    {ve_vl_vcmpswsx_vvvmvl, 112966}, // __builtin_ve_vl_vcmpswsx_vvvmvl
    {ve_vl_vcmpswsx_vvvvl, 112982}, // __builtin_ve_vl_vcmpswsx_vvvvl
    {ve_vl_vcmpswzx_vsvl, 112997}, // __builtin_ve_vl_vcmpswzx_vsvl
    {ve_vl_vcmpswzx_vsvmvl, 113011}, // __builtin_ve_vl_vcmpswzx_vsvmvl
    {ve_vl_vcmpswzx_vsvvl, 113027}, // __builtin_ve_vl_vcmpswzx_vsvvl
    {ve_vl_vcmpswzx_vvvl, 113042}, // __builtin_ve_vl_vcmpswzx_vvvl
    {ve_vl_vcmpswzx_vvvmvl, 113056}, // __builtin_ve_vl_vcmpswzx_vvvmvl
    {ve_vl_vcmpswzx_vvvvl, 113072}, // __builtin_ve_vl_vcmpswzx_vvvvl
    {ve_vl_vcmpul_vsvl, 113087}, // __builtin_ve_vl_vcmpul_vsvl
    {ve_vl_vcmpul_vsvmvl, 113099}, // __builtin_ve_vl_vcmpul_vsvmvl
    {ve_vl_vcmpul_vsvvl, 113113}, // __builtin_ve_vl_vcmpul_vsvvl
    {ve_vl_vcmpul_vvvl, 113126}, // __builtin_ve_vl_vcmpul_vvvl
    {ve_vl_vcmpul_vvvmvl, 113138}, // __builtin_ve_vl_vcmpul_vvvmvl
    {ve_vl_vcmpul_vvvvl, 113152}, // __builtin_ve_vl_vcmpul_vvvvl
    {ve_vl_vcmpuw_vsvl, 113165}, // __builtin_ve_vl_vcmpuw_vsvl
    {ve_vl_vcmpuw_vsvmvl, 113177}, // __builtin_ve_vl_vcmpuw_vsvmvl
    {ve_vl_vcmpuw_vsvvl, 113191}, // __builtin_ve_vl_vcmpuw_vsvvl
    {ve_vl_vcmpuw_vvvl, 113204}, // __builtin_ve_vl_vcmpuw_vvvl
    {ve_vl_vcmpuw_vvvmvl, 113216}, // __builtin_ve_vl_vcmpuw_vvvmvl
    {ve_vl_vcmpuw_vvvvl, 113230}, // __builtin_ve_vl_vcmpuw_vvvvl
    {ve_vl_vcp_vvmvl, 113243}, // __builtin_ve_vl_vcp_vvmvl
    {ve_vl_vcvtdl_vvl, 113253}, // __builtin_ve_vl_vcvtdl_vvl
    {ve_vl_vcvtdl_vvvl, 113264}, // __builtin_ve_vl_vcvtdl_vvvl
    {ve_vl_vcvtds_vvl, 113276}, // __builtin_ve_vl_vcvtds_vvl
    {ve_vl_vcvtds_vvvl, 113287}, // __builtin_ve_vl_vcvtds_vvvl
    {ve_vl_vcvtdw_vvl, 113299}, // __builtin_ve_vl_vcvtdw_vvl
    {ve_vl_vcvtdw_vvvl, 113310}, // __builtin_ve_vl_vcvtdw_vvvl
    {ve_vl_vcvtld_vvl, 113322}, // __builtin_ve_vl_vcvtld_vvl
    {ve_vl_vcvtld_vvmvl, 113333}, // __builtin_ve_vl_vcvtld_vvmvl
    {ve_vl_vcvtld_vvvl, 113346}, // __builtin_ve_vl_vcvtld_vvvl
    {ve_vl_vcvtldrz_vvl, 113358}, // __builtin_ve_vl_vcvtldrz_vvl
    {ve_vl_vcvtldrz_vvmvl, 113371}, // __builtin_ve_vl_vcvtldrz_vvmvl
    {ve_vl_vcvtldrz_vvvl, 113386}, // __builtin_ve_vl_vcvtldrz_vvvl
    {ve_vl_vcvtsd_vvl, 113400}, // __builtin_ve_vl_vcvtsd_vvl
    {ve_vl_vcvtsd_vvvl, 113411}, // __builtin_ve_vl_vcvtsd_vvvl
    {ve_vl_vcvtsw_vvl, 113423}, // __builtin_ve_vl_vcvtsw_vvl
    {ve_vl_vcvtsw_vvvl, 113434}, // __builtin_ve_vl_vcvtsw_vvvl
    {ve_vl_vcvtwdsx_vvl, 113446}, // __builtin_ve_vl_vcvtwdsx_vvl
    {ve_vl_vcvtwdsx_vvmvl, 113459}, // __builtin_ve_vl_vcvtwdsx_vvmvl
    {ve_vl_vcvtwdsx_vvvl, 113474}, // __builtin_ve_vl_vcvtwdsx_vvvl
    {ve_vl_vcvtwdsxrz_vvl, 113488}, // __builtin_ve_vl_vcvtwdsxrz_vvl
    {ve_vl_vcvtwdsxrz_vvmvl, 113503}, // __builtin_ve_vl_vcvtwdsxrz_vvmvl
    {ve_vl_vcvtwdsxrz_vvvl, 113520}, // __builtin_ve_vl_vcvtwdsxrz_vvvl
    {ve_vl_vcvtwdzx_vvl, 113536}, // __builtin_ve_vl_vcvtwdzx_vvl
    {ve_vl_vcvtwdzx_vvmvl, 113549}, // __builtin_ve_vl_vcvtwdzx_vvmvl
    {ve_vl_vcvtwdzx_vvvl, 113564}, // __builtin_ve_vl_vcvtwdzx_vvvl
    {ve_vl_vcvtwdzxrz_vvl, 113578}, // __builtin_ve_vl_vcvtwdzxrz_vvl
    {ve_vl_vcvtwdzxrz_vvmvl, 113593}, // __builtin_ve_vl_vcvtwdzxrz_vvmvl
    {ve_vl_vcvtwdzxrz_vvvl, 113610}, // __builtin_ve_vl_vcvtwdzxrz_vvvl
    {ve_vl_vcvtwssx_vvl, 113626}, // __builtin_ve_vl_vcvtwssx_vvl
    {ve_vl_vcvtwssx_vvmvl, 113639}, // __builtin_ve_vl_vcvtwssx_vvmvl
    {ve_vl_vcvtwssx_vvvl, 113654}, // __builtin_ve_vl_vcvtwssx_vvvl
    {ve_vl_vcvtwssxrz_vvl, 113668}, // __builtin_ve_vl_vcvtwssxrz_vvl
    {ve_vl_vcvtwssxrz_vvmvl, 113683}, // __builtin_ve_vl_vcvtwssxrz_vvmvl
    {ve_vl_vcvtwssxrz_vvvl, 113700}, // __builtin_ve_vl_vcvtwssxrz_vvvl
    {ve_vl_vcvtwszx_vvl, 113716}, // __builtin_ve_vl_vcvtwszx_vvl
    {ve_vl_vcvtwszx_vvmvl, 113729}, // __builtin_ve_vl_vcvtwszx_vvmvl
    {ve_vl_vcvtwszx_vvvl, 113744}, // __builtin_ve_vl_vcvtwszx_vvvl
    {ve_vl_vcvtwszxrz_vvl, 113758}, // __builtin_ve_vl_vcvtwszxrz_vvl
    {ve_vl_vcvtwszxrz_vvmvl, 113773}, // __builtin_ve_vl_vcvtwszxrz_vvmvl
    {ve_vl_vcvtwszxrz_vvvl, 113790}, // __builtin_ve_vl_vcvtwszxrz_vvvl
    {ve_vl_vdivsl_vsvl, 113806}, // __builtin_ve_vl_vdivsl_vsvl
    {ve_vl_vdivsl_vsvmvl, 113818}, // __builtin_ve_vl_vdivsl_vsvmvl
    {ve_vl_vdivsl_vsvvl, 113832}, // __builtin_ve_vl_vdivsl_vsvvl
    {ve_vl_vdivsl_vvsl, 113845}, // __builtin_ve_vl_vdivsl_vvsl
    {ve_vl_vdivsl_vvsmvl, 113857}, // __builtin_ve_vl_vdivsl_vvsmvl
    {ve_vl_vdivsl_vvsvl, 113871}, // __builtin_ve_vl_vdivsl_vvsvl
    {ve_vl_vdivsl_vvvl, 113884}, // __builtin_ve_vl_vdivsl_vvvl
    {ve_vl_vdivsl_vvvmvl, 113896}, // __builtin_ve_vl_vdivsl_vvvmvl
    {ve_vl_vdivsl_vvvvl, 113910}, // __builtin_ve_vl_vdivsl_vvvvl
    {ve_vl_vdivswsx_vsvl, 113923}, // __builtin_ve_vl_vdivswsx_vsvl
    {ve_vl_vdivswsx_vsvmvl, 113937}, // __builtin_ve_vl_vdivswsx_vsvmvl
    {ve_vl_vdivswsx_vsvvl, 113953}, // __builtin_ve_vl_vdivswsx_vsvvl
    {ve_vl_vdivswsx_vvsl, 113968}, // __builtin_ve_vl_vdivswsx_vvsl
    {ve_vl_vdivswsx_vvsmvl, 113982}, // __builtin_ve_vl_vdivswsx_vvsmvl
    {ve_vl_vdivswsx_vvsvl, 113998}, // __builtin_ve_vl_vdivswsx_vvsvl
    {ve_vl_vdivswsx_vvvl, 114013}, // __builtin_ve_vl_vdivswsx_vvvl
    {ve_vl_vdivswsx_vvvmvl, 114027}, // __builtin_ve_vl_vdivswsx_vvvmvl
    {ve_vl_vdivswsx_vvvvl, 114043}, // __builtin_ve_vl_vdivswsx_vvvvl
    {ve_vl_vdivswzx_vsvl, 114058}, // __builtin_ve_vl_vdivswzx_vsvl
    {ve_vl_vdivswzx_vsvmvl, 114072}, // __builtin_ve_vl_vdivswzx_vsvmvl
    {ve_vl_vdivswzx_vsvvl, 114088}, // __builtin_ve_vl_vdivswzx_vsvvl
    {ve_vl_vdivswzx_vvsl, 114103}, // __builtin_ve_vl_vdivswzx_vvsl
    {ve_vl_vdivswzx_vvsmvl, 114117}, // __builtin_ve_vl_vdivswzx_vvsmvl
    {ve_vl_vdivswzx_vvsvl, 114133}, // __builtin_ve_vl_vdivswzx_vvsvl
    {ve_vl_vdivswzx_vvvl, 114148}, // __builtin_ve_vl_vdivswzx_vvvl
    {ve_vl_vdivswzx_vvvmvl, 114162}, // __builtin_ve_vl_vdivswzx_vvvmvl
    {ve_vl_vdivswzx_vvvvl, 114178}, // __builtin_ve_vl_vdivswzx_vvvvl
    {ve_vl_vdivul_vsvl, 114193}, // __builtin_ve_vl_vdivul_vsvl
    {ve_vl_vdivul_vsvmvl, 114205}, // __builtin_ve_vl_vdivul_vsvmvl
    {ve_vl_vdivul_vsvvl, 114219}, // __builtin_ve_vl_vdivul_vsvvl
    {ve_vl_vdivul_vvsl, 114232}, // __builtin_ve_vl_vdivul_vvsl
    {ve_vl_vdivul_vvsmvl, 114244}, // __builtin_ve_vl_vdivul_vvsmvl
    {ve_vl_vdivul_vvsvl, 114258}, // __builtin_ve_vl_vdivul_vvsvl
    {ve_vl_vdivul_vvvl, 114271}, // __builtin_ve_vl_vdivul_vvvl
    {ve_vl_vdivul_vvvmvl, 114283}, // __builtin_ve_vl_vdivul_vvvmvl
    {ve_vl_vdivul_vvvvl, 114297}, // __builtin_ve_vl_vdivul_vvvvl
    {ve_vl_vdivuw_vsvl, 114310}, // __builtin_ve_vl_vdivuw_vsvl
    {ve_vl_vdivuw_vsvmvl, 114322}, // __builtin_ve_vl_vdivuw_vsvmvl
    {ve_vl_vdivuw_vsvvl, 114336}, // __builtin_ve_vl_vdivuw_vsvvl
    {ve_vl_vdivuw_vvsl, 114349}, // __builtin_ve_vl_vdivuw_vvsl
    {ve_vl_vdivuw_vvsmvl, 114361}, // __builtin_ve_vl_vdivuw_vvsmvl
    {ve_vl_vdivuw_vvsvl, 114375}, // __builtin_ve_vl_vdivuw_vvsvl
    {ve_vl_vdivuw_vvvl, 114388}, // __builtin_ve_vl_vdivuw_vvvl
    {ve_vl_vdivuw_vvvmvl, 114400}, // __builtin_ve_vl_vdivuw_vvvmvl
    {ve_vl_vdivuw_vvvvl, 114414}, // __builtin_ve_vl_vdivuw_vvvvl
    {ve_vl_veqv_vsvl, 114427}, // __builtin_ve_vl_veqv_vsvl
    {ve_vl_veqv_vsvmvl, 114437}, // __builtin_ve_vl_veqv_vsvmvl
    {ve_vl_veqv_vsvvl, 114449}, // __builtin_ve_vl_veqv_vsvvl
    {ve_vl_veqv_vvvl, 114460}, // __builtin_ve_vl_veqv_vvvl
    {ve_vl_veqv_vvvmvl, 114470}, // __builtin_ve_vl_veqv_vvvmvl
    {ve_vl_veqv_vvvvl, 114482}, // __builtin_ve_vl_veqv_vvvvl
    {ve_vl_vex_vvmvl, 114493}, // __builtin_ve_vl_vex_vvmvl
    {ve_vl_vfaddd_vsvl, 114503}, // __builtin_ve_vl_vfaddd_vsvl
    {ve_vl_vfaddd_vsvmvl, 114515}, // __builtin_ve_vl_vfaddd_vsvmvl
    {ve_vl_vfaddd_vsvvl, 114529}, // __builtin_ve_vl_vfaddd_vsvvl
    {ve_vl_vfaddd_vvvl, 114542}, // __builtin_ve_vl_vfaddd_vvvl
    {ve_vl_vfaddd_vvvmvl, 114554}, // __builtin_ve_vl_vfaddd_vvvmvl
    {ve_vl_vfaddd_vvvvl, 114568}, // __builtin_ve_vl_vfaddd_vvvvl
    {ve_vl_vfadds_vsvl, 114581}, // __builtin_ve_vl_vfadds_vsvl
    {ve_vl_vfadds_vsvmvl, 114593}, // __builtin_ve_vl_vfadds_vsvmvl
    {ve_vl_vfadds_vsvvl, 114607}, // __builtin_ve_vl_vfadds_vsvvl
    {ve_vl_vfadds_vvvl, 114620}, // __builtin_ve_vl_vfadds_vvvl
    {ve_vl_vfadds_vvvmvl, 114632}, // __builtin_ve_vl_vfadds_vvvmvl
    {ve_vl_vfadds_vvvvl, 114646}, // __builtin_ve_vl_vfadds_vvvvl
    {ve_vl_vfcmpd_vsvl, 114659}, // __builtin_ve_vl_vfcmpd_vsvl
    {ve_vl_vfcmpd_vsvmvl, 114671}, // __builtin_ve_vl_vfcmpd_vsvmvl
    {ve_vl_vfcmpd_vsvvl, 114685}, // __builtin_ve_vl_vfcmpd_vsvvl
    {ve_vl_vfcmpd_vvvl, 114698}, // __builtin_ve_vl_vfcmpd_vvvl
    {ve_vl_vfcmpd_vvvmvl, 114710}, // __builtin_ve_vl_vfcmpd_vvvmvl
    {ve_vl_vfcmpd_vvvvl, 114724}, // __builtin_ve_vl_vfcmpd_vvvvl
    {ve_vl_vfcmps_vsvl, 114737}, // __builtin_ve_vl_vfcmps_vsvl
    {ve_vl_vfcmps_vsvmvl, 114749}, // __builtin_ve_vl_vfcmps_vsvmvl
    {ve_vl_vfcmps_vsvvl, 114763}, // __builtin_ve_vl_vfcmps_vsvvl
    {ve_vl_vfcmps_vvvl, 114776}, // __builtin_ve_vl_vfcmps_vvvl
    {ve_vl_vfcmps_vvvmvl, 114788}, // __builtin_ve_vl_vfcmps_vvvmvl
    {ve_vl_vfcmps_vvvvl, 114802}, // __builtin_ve_vl_vfcmps_vvvvl
    {ve_vl_vfdivd_vsvl, 114815}, // __builtin_ve_vl_vfdivd_vsvl
    {ve_vl_vfdivd_vsvmvl, 114827}, // __builtin_ve_vl_vfdivd_vsvmvl
    {ve_vl_vfdivd_vsvvl, 114841}, // __builtin_ve_vl_vfdivd_vsvvl
    {ve_vl_vfdivd_vvvl, 114854}, // __builtin_ve_vl_vfdivd_vvvl
    {ve_vl_vfdivd_vvvmvl, 114866}, // __builtin_ve_vl_vfdivd_vvvmvl
    {ve_vl_vfdivd_vvvvl, 114880}, // __builtin_ve_vl_vfdivd_vvvvl
    {ve_vl_vfdivs_vsvl, 114893}, // __builtin_ve_vl_vfdivs_vsvl
    {ve_vl_vfdivs_vsvmvl, 114905}, // __builtin_ve_vl_vfdivs_vsvmvl
    {ve_vl_vfdivs_vsvvl, 114919}, // __builtin_ve_vl_vfdivs_vsvvl
    {ve_vl_vfdivs_vvvl, 114932}, // __builtin_ve_vl_vfdivs_vvvl
    {ve_vl_vfdivs_vvvmvl, 114944}, // __builtin_ve_vl_vfdivs_vvvmvl
    {ve_vl_vfdivs_vvvvl, 114958}, // __builtin_ve_vl_vfdivs_vvvvl
    {ve_vl_vfmadd_vsvvl, 114971}, // __builtin_ve_vl_vfmadd_vsvvl
    {ve_vl_vfmadd_vsvvmvl, 114984}, // __builtin_ve_vl_vfmadd_vsvvmvl
    {ve_vl_vfmadd_vsvvvl, 114999}, // __builtin_ve_vl_vfmadd_vsvvvl
    {ve_vl_vfmadd_vvsvl, 115013}, // __builtin_ve_vl_vfmadd_vvsvl
    {ve_vl_vfmadd_vvsvmvl, 115026}, // __builtin_ve_vl_vfmadd_vvsvmvl
    {ve_vl_vfmadd_vvsvvl, 115041}, // __builtin_ve_vl_vfmadd_vvsvvl
    {ve_vl_vfmadd_vvvvl, 115055}, // __builtin_ve_vl_vfmadd_vvvvl
    {ve_vl_vfmadd_vvvvmvl, 115068}, // __builtin_ve_vl_vfmadd_vvvvmvl
    {ve_vl_vfmadd_vvvvvl, 115083}, // __builtin_ve_vl_vfmadd_vvvvvl
    {ve_vl_vfmads_vsvvl, 115097}, // __builtin_ve_vl_vfmads_vsvvl
    {ve_vl_vfmads_vsvvmvl, 115110}, // __builtin_ve_vl_vfmads_vsvvmvl
    {ve_vl_vfmads_vsvvvl, 115125}, // __builtin_ve_vl_vfmads_vsvvvl
    {ve_vl_vfmads_vvsvl, 115139}, // __builtin_ve_vl_vfmads_vvsvl
    {ve_vl_vfmads_vvsvmvl, 115152}, // __builtin_ve_vl_vfmads_vvsvmvl
    {ve_vl_vfmads_vvsvvl, 115167}, // __builtin_ve_vl_vfmads_vvsvvl
    {ve_vl_vfmads_vvvvl, 115181}, // __builtin_ve_vl_vfmads_vvvvl
    {ve_vl_vfmads_vvvvmvl, 115194}, // __builtin_ve_vl_vfmads_vvvvmvl
    {ve_vl_vfmads_vvvvvl, 115209}, // __builtin_ve_vl_vfmads_vvvvvl
    {ve_vl_vfmaxd_vsvl, 115223}, // __builtin_ve_vl_vfmaxd_vsvl
    {ve_vl_vfmaxd_vsvmvl, 115235}, // __builtin_ve_vl_vfmaxd_vsvmvl
    {ve_vl_vfmaxd_vsvvl, 115249}, // __builtin_ve_vl_vfmaxd_vsvvl
    {ve_vl_vfmaxd_vvvl, 115262}, // __builtin_ve_vl_vfmaxd_vvvl
    {ve_vl_vfmaxd_vvvmvl, 115274}, // __builtin_ve_vl_vfmaxd_vvvmvl
    {ve_vl_vfmaxd_vvvvl, 115288}, // __builtin_ve_vl_vfmaxd_vvvvl
    {ve_vl_vfmaxs_vsvl, 115301}, // __builtin_ve_vl_vfmaxs_vsvl
    {ve_vl_vfmaxs_vsvmvl, 115313}, // __builtin_ve_vl_vfmaxs_vsvmvl
    {ve_vl_vfmaxs_vsvvl, 115327}, // __builtin_ve_vl_vfmaxs_vsvvl
    {ve_vl_vfmaxs_vvvl, 115340}, // __builtin_ve_vl_vfmaxs_vvvl
    {ve_vl_vfmaxs_vvvmvl, 115352}, // __builtin_ve_vl_vfmaxs_vvvmvl
    {ve_vl_vfmaxs_vvvvl, 115366}, // __builtin_ve_vl_vfmaxs_vvvvl
    {ve_vl_vfmind_vsvl, 115379}, // __builtin_ve_vl_vfmind_vsvl
    {ve_vl_vfmind_vsvmvl, 115391}, // __builtin_ve_vl_vfmind_vsvmvl
    {ve_vl_vfmind_vsvvl, 115405}, // __builtin_ve_vl_vfmind_vsvvl
    {ve_vl_vfmind_vvvl, 115418}, // __builtin_ve_vl_vfmind_vvvl
    {ve_vl_vfmind_vvvmvl, 115430}, // __builtin_ve_vl_vfmind_vvvmvl
    {ve_vl_vfmind_vvvvl, 115444}, // __builtin_ve_vl_vfmind_vvvvl
    {ve_vl_vfmins_vsvl, 115457}, // __builtin_ve_vl_vfmins_vsvl
    {ve_vl_vfmins_vsvmvl, 115469}, // __builtin_ve_vl_vfmins_vsvmvl
    {ve_vl_vfmins_vsvvl, 115483}, // __builtin_ve_vl_vfmins_vsvvl
    {ve_vl_vfmins_vvvl, 115496}, // __builtin_ve_vl_vfmins_vvvl
    {ve_vl_vfmins_vvvmvl, 115508}, // __builtin_ve_vl_vfmins_vvvmvl
    {ve_vl_vfmins_vvvvl, 115522}, // __builtin_ve_vl_vfmins_vvvvl
    {ve_vl_vfmkdeq_mvl, 115535}, // __builtin_ve_vl_vfmkdeq_mvl
    {ve_vl_vfmkdeq_mvml, 115547}, // __builtin_ve_vl_vfmkdeq_mvml
    {ve_vl_vfmkdeqnan_mvl, 115560}, // __builtin_ve_vl_vfmkdeqnan_mvl
    {ve_vl_vfmkdeqnan_mvml, 115575}, // __builtin_ve_vl_vfmkdeqnan_mvml
    {ve_vl_vfmkdge_mvl, 115591}, // __builtin_ve_vl_vfmkdge_mvl
    {ve_vl_vfmkdge_mvml, 115603}, // __builtin_ve_vl_vfmkdge_mvml
    {ve_vl_vfmkdgenan_mvl, 115616}, // __builtin_ve_vl_vfmkdgenan_mvl
    {ve_vl_vfmkdgenan_mvml, 115631}, // __builtin_ve_vl_vfmkdgenan_mvml
    {ve_vl_vfmkdgt_mvl, 115647}, // __builtin_ve_vl_vfmkdgt_mvl
    {ve_vl_vfmkdgt_mvml, 115659}, // __builtin_ve_vl_vfmkdgt_mvml
    {ve_vl_vfmkdgtnan_mvl, 115672}, // __builtin_ve_vl_vfmkdgtnan_mvl
    {ve_vl_vfmkdgtnan_mvml, 115687}, // __builtin_ve_vl_vfmkdgtnan_mvml
    {ve_vl_vfmkdle_mvl, 115703}, // __builtin_ve_vl_vfmkdle_mvl
    {ve_vl_vfmkdle_mvml, 115715}, // __builtin_ve_vl_vfmkdle_mvml
    {ve_vl_vfmkdlenan_mvl, 115728}, // __builtin_ve_vl_vfmkdlenan_mvl
    {ve_vl_vfmkdlenan_mvml, 115743}, // __builtin_ve_vl_vfmkdlenan_mvml
    {ve_vl_vfmkdlt_mvl, 115759}, // __builtin_ve_vl_vfmkdlt_mvl
    {ve_vl_vfmkdlt_mvml, 115771}, // __builtin_ve_vl_vfmkdlt_mvml
    {ve_vl_vfmkdltnan_mvl, 115784}, // __builtin_ve_vl_vfmkdltnan_mvl
    {ve_vl_vfmkdltnan_mvml, 115799}, // __builtin_ve_vl_vfmkdltnan_mvml
    {ve_vl_vfmkdnan_mvl, 115815}, // __builtin_ve_vl_vfmkdnan_mvl
    {ve_vl_vfmkdnan_mvml, 115828}, // __builtin_ve_vl_vfmkdnan_mvml
    {ve_vl_vfmkdne_mvl, 115842}, // __builtin_ve_vl_vfmkdne_mvl
    {ve_vl_vfmkdne_mvml, 115854}, // __builtin_ve_vl_vfmkdne_mvml
    {ve_vl_vfmkdnenan_mvl, 115867}, // __builtin_ve_vl_vfmkdnenan_mvl
    {ve_vl_vfmkdnenan_mvml, 115882}, // __builtin_ve_vl_vfmkdnenan_mvml
    {ve_vl_vfmkdnum_mvl, 115898}, // __builtin_ve_vl_vfmkdnum_mvl
    {ve_vl_vfmkdnum_mvml, 115911}, // __builtin_ve_vl_vfmkdnum_mvml
    {ve_vl_vfmklaf_ml, 115925}, // __builtin_ve_vl_vfmklaf_ml
    {ve_vl_vfmklat_ml, 115936}, // __builtin_ve_vl_vfmklat_ml
    {ve_vl_vfmkleq_mvl, 115947}, // __builtin_ve_vl_vfmkleq_mvl
    {ve_vl_vfmkleq_mvml, 115959}, // __builtin_ve_vl_vfmkleq_mvml
    {ve_vl_vfmkleqnan_mvl, 115972}, // __builtin_ve_vl_vfmkleqnan_mvl
    {ve_vl_vfmkleqnan_mvml, 115987}, // __builtin_ve_vl_vfmkleqnan_mvml
    {ve_vl_vfmklge_mvl, 116003}, // __builtin_ve_vl_vfmklge_mvl
    {ve_vl_vfmklge_mvml, 116015}, // __builtin_ve_vl_vfmklge_mvml
    {ve_vl_vfmklgenan_mvl, 116028}, // __builtin_ve_vl_vfmklgenan_mvl
    {ve_vl_vfmklgenan_mvml, 116043}, // __builtin_ve_vl_vfmklgenan_mvml
    {ve_vl_vfmklgt_mvl, 116059}, // __builtin_ve_vl_vfmklgt_mvl
    {ve_vl_vfmklgt_mvml, 116071}, // __builtin_ve_vl_vfmklgt_mvml
    {ve_vl_vfmklgtnan_mvl, 116084}, // __builtin_ve_vl_vfmklgtnan_mvl
    {ve_vl_vfmklgtnan_mvml, 116099}, // __builtin_ve_vl_vfmklgtnan_mvml
    {ve_vl_vfmklle_mvl, 116115}, // __builtin_ve_vl_vfmklle_mvl
    {ve_vl_vfmklle_mvml, 116127}, // __builtin_ve_vl_vfmklle_mvml
    {ve_vl_vfmkllenan_mvl, 116140}, // __builtin_ve_vl_vfmkllenan_mvl
    {ve_vl_vfmkllenan_mvml, 116155}, // __builtin_ve_vl_vfmkllenan_mvml
    {ve_vl_vfmkllt_mvl, 116171}, // __builtin_ve_vl_vfmkllt_mvl
    {ve_vl_vfmkllt_mvml, 116183}, // __builtin_ve_vl_vfmkllt_mvml
    {ve_vl_vfmklltnan_mvl, 116196}, // __builtin_ve_vl_vfmklltnan_mvl
    {ve_vl_vfmklltnan_mvml, 116211}, // __builtin_ve_vl_vfmklltnan_mvml
    {ve_vl_vfmklnan_mvl, 116227}, // __builtin_ve_vl_vfmklnan_mvl
    {ve_vl_vfmklnan_mvml, 116240}, // __builtin_ve_vl_vfmklnan_mvml
    {ve_vl_vfmklne_mvl, 116254}, // __builtin_ve_vl_vfmklne_mvl
    {ve_vl_vfmklne_mvml, 116266}, // __builtin_ve_vl_vfmklne_mvml
    {ve_vl_vfmklnenan_mvl, 116279}, // __builtin_ve_vl_vfmklnenan_mvl
    {ve_vl_vfmklnenan_mvml, 116294}, // __builtin_ve_vl_vfmklnenan_mvml
    {ve_vl_vfmklnum_mvl, 116310}, // __builtin_ve_vl_vfmklnum_mvl
    {ve_vl_vfmklnum_mvml, 116323}, // __builtin_ve_vl_vfmklnum_mvml
    {ve_vl_vfmkseq_mvl, 116337}, // __builtin_ve_vl_vfmkseq_mvl
    {ve_vl_vfmkseq_mvml, 116349}, // __builtin_ve_vl_vfmkseq_mvml
    {ve_vl_vfmkseqnan_mvl, 116362}, // __builtin_ve_vl_vfmkseqnan_mvl
    {ve_vl_vfmkseqnan_mvml, 116377}, // __builtin_ve_vl_vfmkseqnan_mvml
    {ve_vl_vfmksge_mvl, 116393}, // __builtin_ve_vl_vfmksge_mvl
    {ve_vl_vfmksge_mvml, 116405}, // __builtin_ve_vl_vfmksge_mvml
    {ve_vl_vfmksgenan_mvl, 116418}, // __builtin_ve_vl_vfmksgenan_mvl
    {ve_vl_vfmksgenan_mvml, 116433}, // __builtin_ve_vl_vfmksgenan_mvml
    {ve_vl_vfmksgt_mvl, 116449}, // __builtin_ve_vl_vfmksgt_mvl
    {ve_vl_vfmksgt_mvml, 116461}, // __builtin_ve_vl_vfmksgt_mvml
    {ve_vl_vfmksgtnan_mvl, 116474}, // __builtin_ve_vl_vfmksgtnan_mvl
    {ve_vl_vfmksgtnan_mvml, 116489}, // __builtin_ve_vl_vfmksgtnan_mvml
    {ve_vl_vfmksle_mvl, 116505}, // __builtin_ve_vl_vfmksle_mvl
    {ve_vl_vfmksle_mvml, 116517}, // __builtin_ve_vl_vfmksle_mvml
    {ve_vl_vfmkslenan_mvl, 116530}, // __builtin_ve_vl_vfmkslenan_mvl
    {ve_vl_vfmkslenan_mvml, 116545}, // __builtin_ve_vl_vfmkslenan_mvml
    {ve_vl_vfmkslt_mvl, 116561}, // __builtin_ve_vl_vfmkslt_mvl
    {ve_vl_vfmkslt_mvml, 116573}, // __builtin_ve_vl_vfmkslt_mvml
    {ve_vl_vfmksltnan_mvl, 116586}, // __builtin_ve_vl_vfmksltnan_mvl
    {ve_vl_vfmksltnan_mvml, 116601}, // __builtin_ve_vl_vfmksltnan_mvml
    {ve_vl_vfmksnan_mvl, 116617}, // __builtin_ve_vl_vfmksnan_mvl
    {ve_vl_vfmksnan_mvml, 116630}, // __builtin_ve_vl_vfmksnan_mvml
    {ve_vl_vfmksne_mvl, 116644}, // __builtin_ve_vl_vfmksne_mvl
    {ve_vl_vfmksne_mvml, 116656}, // __builtin_ve_vl_vfmksne_mvml
    {ve_vl_vfmksnenan_mvl, 116669}, // __builtin_ve_vl_vfmksnenan_mvl
    {ve_vl_vfmksnenan_mvml, 116684}, // __builtin_ve_vl_vfmksnenan_mvml
    {ve_vl_vfmksnum_mvl, 116700}, // __builtin_ve_vl_vfmksnum_mvl
    {ve_vl_vfmksnum_mvml, 116713}, // __builtin_ve_vl_vfmksnum_mvml
    {ve_vl_vfmkweq_mvl, 116727}, // __builtin_ve_vl_vfmkweq_mvl
    {ve_vl_vfmkweq_mvml, 116739}, // __builtin_ve_vl_vfmkweq_mvml
    {ve_vl_vfmkweqnan_mvl, 116752}, // __builtin_ve_vl_vfmkweqnan_mvl
    {ve_vl_vfmkweqnan_mvml, 116767}, // __builtin_ve_vl_vfmkweqnan_mvml
    {ve_vl_vfmkwge_mvl, 116783}, // __builtin_ve_vl_vfmkwge_mvl
    {ve_vl_vfmkwge_mvml, 116795}, // __builtin_ve_vl_vfmkwge_mvml
    {ve_vl_vfmkwgenan_mvl, 116808}, // __builtin_ve_vl_vfmkwgenan_mvl
    {ve_vl_vfmkwgenan_mvml, 116823}, // __builtin_ve_vl_vfmkwgenan_mvml
    {ve_vl_vfmkwgt_mvl, 116839}, // __builtin_ve_vl_vfmkwgt_mvl
    {ve_vl_vfmkwgt_mvml, 116851}, // __builtin_ve_vl_vfmkwgt_mvml
    {ve_vl_vfmkwgtnan_mvl, 116864}, // __builtin_ve_vl_vfmkwgtnan_mvl
    {ve_vl_vfmkwgtnan_mvml, 116879}, // __builtin_ve_vl_vfmkwgtnan_mvml
    {ve_vl_vfmkwle_mvl, 116895}, // __builtin_ve_vl_vfmkwle_mvl
    {ve_vl_vfmkwle_mvml, 116907}, // __builtin_ve_vl_vfmkwle_mvml
    {ve_vl_vfmkwlenan_mvl, 116920}, // __builtin_ve_vl_vfmkwlenan_mvl
    {ve_vl_vfmkwlenan_mvml, 116935}, // __builtin_ve_vl_vfmkwlenan_mvml
    {ve_vl_vfmkwlt_mvl, 116951}, // __builtin_ve_vl_vfmkwlt_mvl
    {ve_vl_vfmkwlt_mvml, 116963}, // __builtin_ve_vl_vfmkwlt_mvml
    {ve_vl_vfmkwltnan_mvl, 116976}, // __builtin_ve_vl_vfmkwltnan_mvl
    {ve_vl_vfmkwltnan_mvml, 116991}, // __builtin_ve_vl_vfmkwltnan_mvml
    {ve_vl_vfmkwnan_mvl, 117007}, // __builtin_ve_vl_vfmkwnan_mvl
    {ve_vl_vfmkwnan_mvml, 117020}, // __builtin_ve_vl_vfmkwnan_mvml
    {ve_vl_vfmkwne_mvl, 117034}, // __builtin_ve_vl_vfmkwne_mvl
    {ve_vl_vfmkwne_mvml, 117046}, // __builtin_ve_vl_vfmkwne_mvml
    {ve_vl_vfmkwnenan_mvl, 117059}, // __builtin_ve_vl_vfmkwnenan_mvl
    {ve_vl_vfmkwnenan_mvml, 117074}, // __builtin_ve_vl_vfmkwnenan_mvml
    {ve_vl_vfmkwnum_mvl, 117090}, // __builtin_ve_vl_vfmkwnum_mvl
    {ve_vl_vfmkwnum_mvml, 117103}, // __builtin_ve_vl_vfmkwnum_mvml
    {ve_vl_vfmsbd_vsvvl, 117117}, // __builtin_ve_vl_vfmsbd_vsvvl
    {ve_vl_vfmsbd_vsvvmvl, 117130}, // __builtin_ve_vl_vfmsbd_vsvvmvl
    {ve_vl_vfmsbd_vsvvvl, 117145}, // __builtin_ve_vl_vfmsbd_vsvvvl
    {ve_vl_vfmsbd_vvsvl, 117159}, // __builtin_ve_vl_vfmsbd_vvsvl
    {ve_vl_vfmsbd_vvsvmvl, 117172}, // __builtin_ve_vl_vfmsbd_vvsvmvl
    {ve_vl_vfmsbd_vvsvvl, 117187}, // __builtin_ve_vl_vfmsbd_vvsvvl
    {ve_vl_vfmsbd_vvvvl, 117201}, // __builtin_ve_vl_vfmsbd_vvvvl
    {ve_vl_vfmsbd_vvvvmvl, 117214}, // __builtin_ve_vl_vfmsbd_vvvvmvl
    {ve_vl_vfmsbd_vvvvvl, 117229}, // __builtin_ve_vl_vfmsbd_vvvvvl
    {ve_vl_vfmsbs_vsvvl, 117243}, // __builtin_ve_vl_vfmsbs_vsvvl
    {ve_vl_vfmsbs_vsvvmvl, 117256}, // __builtin_ve_vl_vfmsbs_vsvvmvl
    {ve_vl_vfmsbs_vsvvvl, 117271}, // __builtin_ve_vl_vfmsbs_vsvvvl
    {ve_vl_vfmsbs_vvsvl, 117285}, // __builtin_ve_vl_vfmsbs_vvsvl
    {ve_vl_vfmsbs_vvsvmvl, 117298}, // __builtin_ve_vl_vfmsbs_vvsvmvl
    {ve_vl_vfmsbs_vvsvvl, 117313}, // __builtin_ve_vl_vfmsbs_vvsvvl
    {ve_vl_vfmsbs_vvvvl, 117327}, // __builtin_ve_vl_vfmsbs_vvvvl
    {ve_vl_vfmsbs_vvvvmvl, 117340}, // __builtin_ve_vl_vfmsbs_vvvvmvl
    {ve_vl_vfmsbs_vvvvvl, 117355}, // __builtin_ve_vl_vfmsbs_vvvvvl
    {ve_vl_vfmuld_vsvl, 117369}, // __builtin_ve_vl_vfmuld_vsvl
    {ve_vl_vfmuld_vsvmvl, 117381}, // __builtin_ve_vl_vfmuld_vsvmvl
    {ve_vl_vfmuld_vsvvl, 117395}, // __builtin_ve_vl_vfmuld_vsvvl
    {ve_vl_vfmuld_vvvl, 117408}, // __builtin_ve_vl_vfmuld_vvvl
    {ve_vl_vfmuld_vvvmvl, 117420}, // __builtin_ve_vl_vfmuld_vvvmvl
    {ve_vl_vfmuld_vvvvl, 117434}, // __builtin_ve_vl_vfmuld_vvvvl
    {ve_vl_vfmuls_vsvl, 117447}, // __builtin_ve_vl_vfmuls_vsvl
    {ve_vl_vfmuls_vsvmvl, 117459}, // __builtin_ve_vl_vfmuls_vsvmvl
    {ve_vl_vfmuls_vsvvl, 117473}, // __builtin_ve_vl_vfmuls_vsvvl
    {ve_vl_vfmuls_vvvl, 117486}, // __builtin_ve_vl_vfmuls_vvvl
    {ve_vl_vfmuls_vvvmvl, 117498}, // __builtin_ve_vl_vfmuls_vvvmvl
    {ve_vl_vfmuls_vvvvl, 117512}, // __builtin_ve_vl_vfmuls_vvvvl
    {ve_vl_vfnmadd_vsvvl, 117525}, // __builtin_ve_vl_vfnmadd_vsvvl
    {ve_vl_vfnmadd_vsvvmvl, 117539}, // __builtin_ve_vl_vfnmadd_vsvvmvl
    {ve_vl_vfnmadd_vsvvvl, 117555}, // __builtin_ve_vl_vfnmadd_vsvvvl
    {ve_vl_vfnmadd_vvsvl, 117570}, // __builtin_ve_vl_vfnmadd_vvsvl
    {ve_vl_vfnmadd_vvsvmvl, 117584}, // __builtin_ve_vl_vfnmadd_vvsvmvl
    {ve_vl_vfnmadd_vvsvvl, 117600}, // __builtin_ve_vl_vfnmadd_vvsvvl
    {ve_vl_vfnmadd_vvvvl, 117615}, // __builtin_ve_vl_vfnmadd_vvvvl
    {ve_vl_vfnmadd_vvvvmvl, 117629}, // __builtin_ve_vl_vfnmadd_vvvvmvl
    {ve_vl_vfnmadd_vvvvvl, 117645}, // __builtin_ve_vl_vfnmadd_vvvvvl
    {ve_vl_vfnmads_vsvvl, 117660}, // __builtin_ve_vl_vfnmads_vsvvl
    {ve_vl_vfnmads_vsvvmvl, 117674}, // __builtin_ve_vl_vfnmads_vsvvmvl
    {ve_vl_vfnmads_vsvvvl, 117690}, // __builtin_ve_vl_vfnmads_vsvvvl
    {ve_vl_vfnmads_vvsvl, 117705}, // __builtin_ve_vl_vfnmads_vvsvl
    {ve_vl_vfnmads_vvsvmvl, 117719}, // __builtin_ve_vl_vfnmads_vvsvmvl
    {ve_vl_vfnmads_vvsvvl, 117735}, // __builtin_ve_vl_vfnmads_vvsvvl
    {ve_vl_vfnmads_vvvvl, 117750}, // __builtin_ve_vl_vfnmads_vvvvl
    {ve_vl_vfnmads_vvvvmvl, 117764}, // __builtin_ve_vl_vfnmads_vvvvmvl
    {ve_vl_vfnmads_vvvvvl, 117780}, // __builtin_ve_vl_vfnmads_vvvvvl
    {ve_vl_vfnmsbd_vsvvl, 117795}, // __builtin_ve_vl_vfnmsbd_vsvvl
    {ve_vl_vfnmsbd_vsvvmvl, 117809}, // __builtin_ve_vl_vfnmsbd_vsvvmvl
    {ve_vl_vfnmsbd_vsvvvl, 117825}, // __builtin_ve_vl_vfnmsbd_vsvvvl
    {ve_vl_vfnmsbd_vvsvl, 117840}, // __builtin_ve_vl_vfnmsbd_vvsvl
    {ve_vl_vfnmsbd_vvsvmvl, 117854}, // __builtin_ve_vl_vfnmsbd_vvsvmvl
    {ve_vl_vfnmsbd_vvsvvl, 117870}, // __builtin_ve_vl_vfnmsbd_vvsvvl
    {ve_vl_vfnmsbd_vvvvl, 117885}, // __builtin_ve_vl_vfnmsbd_vvvvl
    {ve_vl_vfnmsbd_vvvvmvl, 117899}, // __builtin_ve_vl_vfnmsbd_vvvvmvl
    {ve_vl_vfnmsbd_vvvvvl, 117915}, // __builtin_ve_vl_vfnmsbd_vvvvvl
    {ve_vl_vfnmsbs_vsvvl, 117930}, // __builtin_ve_vl_vfnmsbs_vsvvl
    {ve_vl_vfnmsbs_vsvvmvl, 117944}, // __builtin_ve_vl_vfnmsbs_vsvvmvl
    {ve_vl_vfnmsbs_vsvvvl, 117960}, // __builtin_ve_vl_vfnmsbs_vsvvvl
    {ve_vl_vfnmsbs_vvsvl, 117975}, // __builtin_ve_vl_vfnmsbs_vvsvl
    {ve_vl_vfnmsbs_vvsvmvl, 117989}, // __builtin_ve_vl_vfnmsbs_vvsvmvl
    {ve_vl_vfnmsbs_vvsvvl, 118005}, // __builtin_ve_vl_vfnmsbs_vvsvvl
    {ve_vl_vfnmsbs_vvvvl, 118020}, // __builtin_ve_vl_vfnmsbs_vvvvl
    {ve_vl_vfnmsbs_vvvvmvl, 118034}, // __builtin_ve_vl_vfnmsbs_vvvvmvl
    {ve_vl_vfnmsbs_vvvvvl, 118050}, // __builtin_ve_vl_vfnmsbs_vvvvvl
    {ve_vl_vfrmaxdfst_vvl, 118065}, // __builtin_ve_vl_vfrmaxdfst_vvl
    {ve_vl_vfrmaxdfst_vvvl, 118080}, // __builtin_ve_vl_vfrmaxdfst_vvvl
    {ve_vl_vfrmaxdlst_vvl, 118096}, // __builtin_ve_vl_vfrmaxdlst_vvl
    {ve_vl_vfrmaxdlst_vvvl, 118111}, // __builtin_ve_vl_vfrmaxdlst_vvvl
    {ve_vl_vfrmaxsfst_vvl, 118127}, // __builtin_ve_vl_vfrmaxsfst_vvl
    {ve_vl_vfrmaxsfst_vvvl, 118142}, // __builtin_ve_vl_vfrmaxsfst_vvvl
    {ve_vl_vfrmaxslst_vvl, 118158}, // __builtin_ve_vl_vfrmaxslst_vvl
    {ve_vl_vfrmaxslst_vvvl, 118173}, // __builtin_ve_vl_vfrmaxslst_vvvl
    {ve_vl_vfrmindfst_vvl, 118189}, // __builtin_ve_vl_vfrmindfst_vvl
    {ve_vl_vfrmindfst_vvvl, 118204}, // __builtin_ve_vl_vfrmindfst_vvvl
    {ve_vl_vfrmindlst_vvl, 118220}, // __builtin_ve_vl_vfrmindlst_vvl
    {ve_vl_vfrmindlst_vvvl, 118235}, // __builtin_ve_vl_vfrmindlst_vvvl
    {ve_vl_vfrminsfst_vvl, 118251}, // __builtin_ve_vl_vfrminsfst_vvl
    {ve_vl_vfrminsfst_vvvl, 118266}, // __builtin_ve_vl_vfrminsfst_vvvl
    {ve_vl_vfrminslst_vvl, 118282}, // __builtin_ve_vl_vfrminslst_vvl
    {ve_vl_vfrminslst_vvvl, 118297}, // __builtin_ve_vl_vfrminslst_vvvl
    {ve_vl_vfsqrtd_vvl, 118313}, // __builtin_ve_vl_vfsqrtd_vvl
    {ve_vl_vfsqrtd_vvvl, 118325}, // __builtin_ve_vl_vfsqrtd_vvvl
    {ve_vl_vfsqrts_vvl, 118338}, // __builtin_ve_vl_vfsqrts_vvl
    {ve_vl_vfsqrts_vvvl, 118350}, // __builtin_ve_vl_vfsqrts_vvvl
    {ve_vl_vfsubd_vsvl, 118363}, // __builtin_ve_vl_vfsubd_vsvl
    {ve_vl_vfsubd_vsvmvl, 118375}, // __builtin_ve_vl_vfsubd_vsvmvl
    {ve_vl_vfsubd_vsvvl, 118389}, // __builtin_ve_vl_vfsubd_vsvvl
    {ve_vl_vfsubd_vvvl, 118402}, // __builtin_ve_vl_vfsubd_vvvl
    {ve_vl_vfsubd_vvvmvl, 118414}, // __builtin_ve_vl_vfsubd_vvvmvl
    {ve_vl_vfsubd_vvvvl, 118428}, // __builtin_ve_vl_vfsubd_vvvvl
    {ve_vl_vfsubs_vsvl, 118441}, // __builtin_ve_vl_vfsubs_vsvl
    {ve_vl_vfsubs_vsvmvl, 118453}, // __builtin_ve_vl_vfsubs_vsvmvl
    {ve_vl_vfsubs_vsvvl, 118467}, // __builtin_ve_vl_vfsubs_vsvvl
    {ve_vl_vfsubs_vvvl, 118480}, // __builtin_ve_vl_vfsubs_vvvl
    {ve_vl_vfsubs_vvvmvl, 118492}, // __builtin_ve_vl_vfsubs_vvvmvl
    {ve_vl_vfsubs_vvvvl, 118506}, // __builtin_ve_vl_vfsubs_vvvvl
    {ve_vl_vfsumd_vvl, 118519}, // __builtin_ve_vl_vfsumd_vvl
    {ve_vl_vfsumd_vvml, 118530}, // __builtin_ve_vl_vfsumd_vvml
    {ve_vl_vfsums_vvl, 118542}, // __builtin_ve_vl_vfsums_vvl
    {ve_vl_vfsums_vvml, 118553}, // __builtin_ve_vl_vfsums_vvml
    {ve_vl_vgt_vvssl, 118565}, // __builtin_ve_vl_vgt_vvssl
    {ve_vl_vgt_vvssml, 118575}, // __builtin_ve_vl_vgt_vvssml
    {ve_vl_vgt_vvssmvl, 118586}, // __builtin_ve_vl_vgt_vvssmvl
    {ve_vl_vgt_vvssvl, 118598}, // __builtin_ve_vl_vgt_vvssvl
    {ve_vl_vgtlsx_vvssl, 118609}, // __builtin_ve_vl_vgtlsx_vvssl
    {ve_vl_vgtlsx_vvssml, 118622}, // __builtin_ve_vl_vgtlsx_vvssml
    {ve_vl_vgtlsx_vvssmvl, 118636}, // __builtin_ve_vl_vgtlsx_vvssmvl
    {ve_vl_vgtlsx_vvssvl, 118651}, // __builtin_ve_vl_vgtlsx_vvssvl
    {ve_vl_vgtlsxnc_vvssl, 118665}, // __builtin_ve_vl_vgtlsxnc_vvssl
    {ve_vl_vgtlsxnc_vvssml, 118680}, // __builtin_ve_vl_vgtlsxnc_vvssml
    {ve_vl_vgtlsxnc_vvssmvl, 118696}, // __builtin_ve_vl_vgtlsxnc_vvssmvl
    {ve_vl_vgtlsxnc_vvssvl, 118713}, // __builtin_ve_vl_vgtlsxnc_vvssvl
    {ve_vl_vgtlzx_vvssl, 118729}, // __builtin_ve_vl_vgtlzx_vvssl
    {ve_vl_vgtlzx_vvssml, 118742}, // __builtin_ve_vl_vgtlzx_vvssml
    {ve_vl_vgtlzx_vvssmvl, 118756}, // __builtin_ve_vl_vgtlzx_vvssmvl
    {ve_vl_vgtlzx_vvssvl, 118771}, // __builtin_ve_vl_vgtlzx_vvssvl
    {ve_vl_vgtlzxnc_vvssl, 118785}, // __builtin_ve_vl_vgtlzxnc_vvssl
    {ve_vl_vgtlzxnc_vvssml, 118800}, // __builtin_ve_vl_vgtlzxnc_vvssml
    {ve_vl_vgtlzxnc_vvssmvl, 118816}, // __builtin_ve_vl_vgtlzxnc_vvssmvl
    {ve_vl_vgtlzxnc_vvssvl, 118833}, // __builtin_ve_vl_vgtlzxnc_vvssvl
    {ve_vl_vgtnc_vvssl, 118849}, // __builtin_ve_vl_vgtnc_vvssl
    {ve_vl_vgtnc_vvssml, 118861}, // __builtin_ve_vl_vgtnc_vvssml
    {ve_vl_vgtnc_vvssmvl, 118874}, // __builtin_ve_vl_vgtnc_vvssmvl
    {ve_vl_vgtnc_vvssvl, 118888}, // __builtin_ve_vl_vgtnc_vvssvl
    {ve_vl_vgtu_vvssl, 118901}, // __builtin_ve_vl_vgtu_vvssl
    {ve_vl_vgtu_vvssml, 118912}, // __builtin_ve_vl_vgtu_vvssml
    {ve_vl_vgtu_vvssmvl, 118924}, // __builtin_ve_vl_vgtu_vvssmvl
    {ve_vl_vgtu_vvssvl, 118937}, // __builtin_ve_vl_vgtu_vvssvl
    {ve_vl_vgtunc_vvssl, 118949}, // __builtin_ve_vl_vgtunc_vvssl
    {ve_vl_vgtunc_vvssml, 118962}, // __builtin_ve_vl_vgtunc_vvssml
    {ve_vl_vgtunc_vvssmvl, 118976}, // __builtin_ve_vl_vgtunc_vvssmvl
    {ve_vl_vgtunc_vvssvl, 118991}, // __builtin_ve_vl_vgtunc_vvssvl
    {ve_vl_vld2d_vssl, 119005}, // __builtin_ve_vl_vld2d_vssl
    {ve_vl_vld2d_vssvl, 119016}, // __builtin_ve_vl_vld2d_vssvl
    {ve_vl_vld2dnc_vssl, 119028}, // __builtin_ve_vl_vld2dnc_vssl
    {ve_vl_vld2dnc_vssvl, 119041}, // __builtin_ve_vl_vld2dnc_vssvl
    {ve_vl_vld_vssl, 119055}, // __builtin_ve_vl_vld_vssl
    {ve_vl_vld_vssvl, 119064}, // __builtin_ve_vl_vld_vssvl
    {ve_vl_vldl2dsx_vssl, 119074}, // __builtin_ve_vl_vldl2dsx_vssl
    {ve_vl_vldl2dsx_vssvl, 119088}, // __builtin_ve_vl_vldl2dsx_vssvl
    {ve_vl_vldl2dsxnc_vssl, 119103}, // __builtin_ve_vl_vldl2dsxnc_vssl
    {ve_vl_vldl2dsxnc_vssvl, 119119}, // __builtin_ve_vl_vldl2dsxnc_vssvl
    {ve_vl_vldl2dzx_vssl, 119136}, // __builtin_ve_vl_vldl2dzx_vssl
    {ve_vl_vldl2dzx_vssvl, 119150}, // __builtin_ve_vl_vldl2dzx_vssvl
    {ve_vl_vldl2dzxnc_vssl, 119165}, // __builtin_ve_vl_vldl2dzxnc_vssl
    {ve_vl_vldl2dzxnc_vssvl, 119181}, // __builtin_ve_vl_vldl2dzxnc_vssvl
    {ve_vl_vldlsx_vssl, 119198}, // __builtin_ve_vl_vldlsx_vssl
    {ve_vl_vldlsx_vssvl, 119210}, // __builtin_ve_vl_vldlsx_vssvl
    {ve_vl_vldlsxnc_vssl, 119223}, // __builtin_ve_vl_vldlsxnc_vssl
    {ve_vl_vldlsxnc_vssvl, 119237}, // __builtin_ve_vl_vldlsxnc_vssvl
    {ve_vl_vldlzx_vssl, 119252}, // __builtin_ve_vl_vldlzx_vssl
    {ve_vl_vldlzx_vssvl, 119264}, // __builtin_ve_vl_vldlzx_vssvl
    {ve_vl_vldlzxnc_vssl, 119277}, // __builtin_ve_vl_vldlzxnc_vssl
    {ve_vl_vldlzxnc_vssvl, 119291}, // __builtin_ve_vl_vldlzxnc_vssvl
    {ve_vl_vldnc_vssl, 119306}, // __builtin_ve_vl_vldnc_vssl
    {ve_vl_vldnc_vssvl, 119317}, // __builtin_ve_vl_vldnc_vssvl
    {ve_vl_vldu2d_vssl, 119329}, // __builtin_ve_vl_vldu2d_vssl
    {ve_vl_vldu2d_vssvl, 119341}, // __builtin_ve_vl_vldu2d_vssvl
    {ve_vl_vldu2dnc_vssl, 119354}, // __builtin_ve_vl_vldu2dnc_vssl
    {ve_vl_vldu2dnc_vssvl, 119368}, // __builtin_ve_vl_vldu2dnc_vssvl
    {ve_vl_vldu_vssl, 119383}, // __builtin_ve_vl_vldu_vssl
    {ve_vl_vldu_vssvl, 119393}, // __builtin_ve_vl_vldu_vssvl
    {ve_vl_vldunc_vssl, 119404}, // __builtin_ve_vl_vldunc_vssl
    {ve_vl_vldunc_vssvl, 119416}, // __builtin_ve_vl_vldunc_vssvl
    {ve_vl_vldz_vvl, 119429}, // __builtin_ve_vl_vldz_vvl
    {ve_vl_vldz_vvmvl, 119438}, // __builtin_ve_vl_vldz_vvmvl
    {ve_vl_vldz_vvvl, 119449}, // __builtin_ve_vl_vldz_vvvl
    {ve_vl_vmaxsl_vsvl, 119459}, // __builtin_ve_vl_vmaxsl_vsvl
    {ve_vl_vmaxsl_vsvmvl, 119471}, // __builtin_ve_vl_vmaxsl_vsvmvl
    {ve_vl_vmaxsl_vsvvl, 119485}, // __builtin_ve_vl_vmaxsl_vsvvl
    {ve_vl_vmaxsl_vvvl, 119498}, // __builtin_ve_vl_vmaxsl_vvvl
    {ve_vl_vmaxsl_vvvmvl, 119510}, // __builtin_ve_vl_vmaxsl_vvvmvl
    {ve_vl_vmaxsl_vvvvl, 119524}, // __builtin_ve_vl_vmaxsl_vvvvl
    {ve_vl_vmaxswsx_vsvl, 119537}, // __builtin_ve_vl_vmaxswsx_vsvl
    {ve_vl_vmaxswsx_vsvmvl, 119551}, // __builtin_ve_vl_vmaxswsx_vsvmvl
    {ve_vl_vmaxswsx_vsvvl, 119567}, // __builtin_ve_vl_vmaxswsx_vsvvl
    {ve_vl_vmaxswsx_vvvl, 119582}, // __builtin_ve_vl_vmaxswsx_vvvl
    {ve_vl_vmaxswsx_vvvmvl, 119596}, // __builtin_ve_vl_vmaxswsx_vvvmvl
    {ve_vl_vmaxswsx_vvvvl, 119612}, // __builtin_ve_vl_vmaxswsx_vvvvl
    {ve_vl_vmaxswzx_vsvl, 119627}, // __builtin_ve_vl_vmaxswzx_vsvl
    {ve_vl_vmaxswzx_vsvmvl, 119641}, // __builtin_ve_vl_vmaxswzx_vsvmvl
    {ve_vl_vmaxswzx_vsvvl, 119657}, // __builtin_ve_vl_vmaxswzx_vsvvl
    {ve_vl_vmaxswzx_vvvl, 119672}, // __builtin_ve_vl_vmaxswzx_vvvl
    {ve_vl_vmaxswzx_vvvmvl, 119686}, // __builtin_ve_vl_vmaxswzx_vvvmvl
    {ve_vl_vmaxswzx_vvvvl, 119702}, // __builtin_ve_vl_vmaxswzx_vvvvl
    {ve_vl_vminsl_vsvl, 119717}, // __builtin_ve_vl_vminsl_vsvl
    {ve_vl_vminsl_vsvmvl, 119729}, // __builtin_ve_vl_vminsl_vsvmvl
    {ve_vl_vminsl_vsvvl, 119743}, // __builtin_ve_vl_vminsl_vsvvl
    {ve_vl_vminsl_vvvl, 119756}, // __builtin_ve_vl_vminsl_vvvl
    {ve_vl_vminsl_vvvmvl, 119768}, // __builtin_ve_vl_vminsl_vvvmvl
    {ve_vl_vminsl_vvvvl, 119782}, // __builtin_ve_vl_vminsl_vvvvl
    {ve_vl_vminswsx_vsvl, 119795}, // __builtin_ve_vl_vminswsx_vsvl
    {ve_vl_vminswsx_vsvmvl, 119809}, // __builtin_ve_vl_vminswsx_vsvmvl
    {ve_vl_vminswsx_vsvvl, 119825}, // __builtin_ve_vl_vminswsx_vsvvl
    {ve_vl_vminswsx_vvvl, 119840}, // __builtin_ve_vl_vminswsx_vvvl
    {ve_vl_vminswsx_vvvmvl, 119854}, // __builtin_ve_vl_vminswsx_vvvmvl
    {ve_vl_vminswsx_vvvvl, 119870}, // __builtin_ve_vl_vminswsx_vvvvl
    {ve_vl_vminswzx_vsvl, 119885}, // __builtin_ve_vl_vminswzx_vsvl
    {ve_vl_vminswzx_vsvmvl, 119899}, // __builtin_ve_vl_vminswzx_vsvmvl
    {ve_vl_vminswzx_vsvvl, 119915}, // __builtin_ve_vl_vminswzx_vsvvl
    {ve_vl_vminswzx_vvvl, 119930}, // __builtin_ve_vl_vminswzx_vvvl
    {ve_vl_vminswzx_vvvmvl, 119944}, // __builtin_ve_vl_vminswzx_vvvmvl
    {ve_vl_vminswzx_vvvvl, 119960}, // __builtin_ve_vl_vminswzx_vvvvl
    {ve_vl_vmrg_vsvml, 119975}, // __builtin_ve_vl_vmrg_vsvml
    {ve_vl_vmrg_vsvmvl, 119986}, // __builtin_ve_vl_vmrg_vsvmvl
    {ve_vl_vmrg_vvvml, 119998}, // __builtin_ve_vl_vmrg_vvvml
    {ve_vl_vmrg_vvvmvl, 120009}, // __builtin_ve_vl_vmrg_vvvmvl
    {ve_vl_vmrgw_vsvMl, 120021}, // __builtin_ve_vl_vmrgw_vsvMl
    {ve_vl_vmrgw_vsvMvl, 120033}, // __builtin_ve_vl_vmrgw_vsvMvl
    {ve_vl_vmrgw_vvvMl, 120046}, // __builtin_ve_vl_vmrgw_vvvMl
    {ve_vl_vmrgw_vvvMvl, 120058}, // __builtin_ve_vl_vmrgw_vvvMvl
    {ve_vl_vmulsl_vsvl, 120071}, // __builtin_ve_vl_vmulsl_vsvl
    {ve_vl_vmulsl_vsvmvl, 120083}, // __builtin_ve_vl_vmulsl_vsvmvl
    {ve_vl_vmulsl_vsvvl, 120097}, // __builtin_ve_vl_vmulsl_vsvvl
    {ve_vl_vmulsl_vvvl, 120110}, // __builtin_ve_vl_vmulsl_vvvl
    {ve_vl_vmulsl_vvvmvl, 120122}, // __builtin_ve_vl_vmulsl_vvvmvl
    {ve_vl_vmulsl_vvvvl, 120136}, // __builtin_ve_vl_vmulsl_vvvvl
    {ve_vl_vmulslw_vsvl, 120149}, // __builtin_ve_vl_vmulslw_vsvl
    {ve_vl_vmulslw_vsvvl, 120162}, // __builtin_ve_vl_vmulslw_vsvvl
    {ve_vl_vmulslw_vvvl, 120176}, // __builtin_ve_vl_vmulslw_vvvl
    {ve_vl_vmulslw_vvvvl, 120189}, // __builtin_ve_vl_vmulslw_vvvvl
    {ve_vl_vmulswsx_vsvl, 120203}, // __builtin_ve_vl_vmulswsx_vsvl
    {ve_vl_vmulswsx_vsvmvl, 120217}, // __builtin_ve_vl_vmulswsx_vsvmvl
    {ve_vl_vmulswsx_vsvvl, 120233}, // __builtin_ve_vl_vmulswsx_vsvvl
    {ve_vl_vmulswsx_vvvl, 120248}, // __builtin_ve_vl_vmulswsx_vvvl
    {ve_vl_vmulswsx_vvvmvl, 120262}, // __builtin_ve_vl_vmulswsx_vvvmvl
    {ve_vl_vmulswsx_vvvvl, 120278}, // __builtin_ve_vl_vmulswsx_vvvvl
    {ve_vl_vmulswzx_vsvl, 120293}, // __builtin_ve_vl_vmulswzx_vsvl
    {ve_vl_vmulswzx_vsvmvl, 120307}, // __builtin_ve_vl_vmulswzx_vsvmvl
    {ve_vl_vmulswzx_vsvvl, 120323}, // __builtin_ve_vl_vmulswzx_vsvvl
    {ve_vl_vmulswzx_vvvl, 120338}, // __builtin_ve_vl_vmulswzx_vvvl
    {ve_vl_vmulswzx_vvvmvl, 120352}, // __builtin_ve_vl_vmulswzx_vvvmvl
    {ve_vl_vmulswzx_vvvvl, 120368}, // __builtin_ve_vl_vmulswzx_vvvvl
    {ve_vl_vmulul_vsvl, 120383}, // __builtin_ve_vl_vmulul_vsvl
    {ve_vl_vmulul_vsvmvl, 120395}, // __builtin_ve_vl_vmulul_vsvmvl
    {ve_vl_vmulul_vsvvl, 120409}, // __builtin_ve_vl_vmulul_vsvvl
    {ve_vl_vmulul_vvvl, 120422}, // __builtin_ve_vl_vmulul_vvvl
    {ve_vl_vmulul_vvvmvl, 120434}, // __builtin_ve_vl_vmulul_vvvmvl
    {ve_vl_vmulul_vvvvl, 120448}, // __builtin_ve_vl_vmulul_vvvvl
    {ve_vl_vmuluw_vsvl, 120461}, // __builtin_ve_vl_vmuluw_vsvl
    {ve_vl_vmuluw_vsvmvl, 120473}, // __builtin_ve_vl_vmuluw_vsvmvl
    {ve_vl_vmuluw_vsvvl, 120487}, // __builtin_ve_vl_vmuluw_vsvvl
    {ve_vl_vmuluw_vvvl, 120500}, // __builtin_ve_vl_vmuluw_vvvl
    {ve_vl_vmuluw_vvvmvl, 120512}, // __builtin_ve_vl_vmuluw_vvvmvl
    {ve_vl_vmuluw_vvvvl, 120526}, // __builtin_ve_vl_vmuluw_vvvvl
    {ve_vl_vmv_vsvl, 120539}, // __builtin_ve_vl_vmv_vsvl
    {ve_vl_vmv_vsvmvl, 120548}, // __builtin_ve_vl_vmv_vsvmvl
    {ve_vl_vmv_vsvvl, 120559}, // __builtin_ve_vl_vmv_vsvvl
    {ve_vl_vor_vsvl, 120569}, // __builtin_ve_vl_vor_vsvl
    {ve_vl_vor_vsvmvl, 120578}, // __builtin_ve_vl_vor_vsvmvl
    {ve_vl_vor_vsvvl, 120589}, // __builtin_ve_vl_vor_vsvvl
    {ve_vl_vor_vvvl, 120599}, // __builtin_ve_vl_vor_vvvl
    {ve_vl_vor_vvvmvl, 120608}, // __builtin_ve_vl_vor_vvvmvl
    {ve_vl_vor_vvvvl, 120619}, // __builtin_ve_vl_vor_vvvvl
    {ve_vl_vpcnt_vvl, 120629}, // __builtin_ve_vl_vpcnt_vvl
    {ve_vl_vpcnt_vvmvl, 120639}, // __builtin_ve_vl_vpcnt_vvmvl
    {ve_vl_vpcnt_vvvl, 120651}, // __builtin_ve_vl_vpcnt_vvvl
    {ve_vl_vrand_vvl, 120662}, // __builtin_ve_vl_vrand_vvl
    {ve_vl_vrand_vvml, 120672}, // __builtin_ve_vl_vrand_vvml
    {ve_vl_vrcpd_vvl, 120683}, // __builtin_ve_vl_vrcpd_vvl
    {ve_vl_vrcpd_vvvl, 120693}, // __builtin_ve_vl_vrcpd_vvvl
    {ve_vl_vrcps_vvl, 120704}, // __builtin_ve_vl_vrcps_vvl
    {ve_vl_vrcps_vvvl, 120714}, // __builtin_ve_vl_vrcps_vvvl
    {ve_vl_vrmaxslfst_vvl, 120725}, // __builtin_ve_vl_vrmaxslfst_vvl
    {ve_vl_vrmaxslfst_vvvl, 120740}, // __builtin_ve_vl_vrmaxslfst_vvvl
    {ve_vl_vrmaxsllst_vvl, 120756}, // __builtin_ve_vl_vrmaxsllst_vvl
    {ve_vl_vrmaxsllst_vvvl, 120771}, // __builtin_ve_vl_vrmaxsllst_vvvl
    {ve_vl_vrmaxswfstsx_vvl, 120787}, // __builtin_ve_vl_vrmaxswfstsx_vvl
    {ve_vl_vrmaxswfstsx_vvvl, 120804}, // __builtin_ve_vl_vrmaxswfstsx_vvvl
    {ve_vl_vrmaxswfstzx_vvl, 120822}, // __builtin_ve_vl_vrmaxswfstzx_vvl
    {ve_vl_vrmaxswfstzx_vvvl, 120839}, // __builtin_ve_vl_vrmaxswfstzx_vvvl
    {ve_vl_vrmaxswlstsx_vvl, 120857}, // __builtin_ve_vl_vrmaxswlstsx_vvl
    {ve_vl_vrmaxswlstsx_vvvl, 120874}, // __builtin_ve_vl_vrmaxswlstsx_vvvl
    {ve_vl_vrmaxswlstzx_vvl, 120892}, // __builtin_ve_vl_vrmaxswlstzx_vvl
    {ve_vl_vrmaxswlstzx_vvvl, 120909}, // __builtin_ve_vl_vrmaxswlstzx_vvvl
    {ve_vl_vrminslfst_vvl, 120927}, // __builtin_ve_vl_vrminslfst_vvl
    {ve_vl_vrminslfst_vvvl, 120942}, // __builtin_ve_vl_vrminslfst_vvvl
    {ve_vl_vrminsllst_vvl, 120958}, // __builtin_ve_vl_vrminsllst_vvl
    {ve_vl_vrminsllst_vvvl, 120973}, // __builtin_ve_vl_vrminsllst_vvvl
    {ve_vl_vrminswfstsx_vvl, 120989}, // __builtin_ve_vl_vrminswfstsx_vvl
    {ve_vl_vrminswfstsx_vvvl, 121006}, // __builtin_ve_vl_vrminswfstsx_vvvl
    {ve_vl_vrminswfstzx_vvl, 121024}, // __builtin_ve_vl_vrminswfstzx_vvl
    {ve_vl_vrminswfstzx_vvvl, 121041}, // __builtin_ve_vl_vrminswfstzx_vvvl
    {ve_vl_vrminswlstsx_vvl, 121059}, // __builtin_ve_vl_vrminswlstsx_vvl
    {ve_vl_vrminswlstsx_vvvl, 121076}, // __builtin_ve_vl_vrminswlstsx_vvvl
    {ve_vl_vrminswlstzx_vvl, 121094}, // __builtin_ve_vl_vrminswlstzx_vvl
    {ve_vl_vrminswlstzx_vvvl, 121111}, // __builtin_ve_vl_vrminswlstzx_vvvl
    {ve_vl_vror_vvl, 121129}, // __builtin_ve_vl_vror_vvl
    {ve_vl_vror_vvml, 121138}, // __builtin_ve_vl_vror_vvml
    {ve_vl_vrsqrtd_vvl, 121148}, // __builtin_ve_vl_vrsqrtd_vvl
    {ve_vl_vrsqrtd_vvvl, 121160}, // __builtin_ve_vl_vrsqrtd_vvvl
    {ve_vl_vrsqrtdnex_vvl, 121173}, // __builtin_ve_vl_vrsqrtdnex_vvl
    {ve_vl_vrsqrtdnex_vvvl, 121188}, // __builtin_ve_vl_vrsqrtdnex_vvvl
    {ve_vl_vrsqrts_vvl, 121204}, // __builtin_ve_vl_vrsqrts_vvl
    {ve_vl_vrsqrts_vvvl, 121216}, // __builtin_ve_vl_vrsqrts_vvvl
    {ve_vl_vrsqrtsnex_vvl, 121229}, // __builtin_ve_vl_vrsqrtsnex_vvl
    {ve_vl_vrsqrtsnex_vvvl, 121244}, // __builtin_ve_vl_vrsqrtsnex_vvvl
    {ve_vl_vrxor_vvl, 121260}, // __builtin_ve_vl_vrxor_vvl
    {ve_vl_vrxor_vvml, 121270}, // __builtin_ve_vl_vrxor_vvml
    {ve_vl_vsc_vvssl, 121281}, // __builtin_ve_vl_vsc_vvssl
    {ve_vl_vsc_vvssml, 121291}, // __builtin_ve_vl_vsc_vvssml
    {ve_vl_vscl_vvssl, 121302}, // __builtin_ve_vl_vscl_vvssl
    {ve_vl_vscl_vvssml, 121313}, // __builtin_ve_vl_vscl_vvssml
    {ve_vl_vsclnc_vvssl, 121325}, // __builtin_ve_vl_vsclnc_vvssl
    {ve_vl_vsclnc_vvssml, 121338}, // __builtin_ve_vl_vsclnc_vvssml
    {ve_vl_vsclncot_vvssl, 121352}, // __builtin_ve_vl_vsclncot_vvssl
    {ve_vl_vsclncot_vvssml, 121367}, // __builtin_ve_vl_vsclncot_vvssml
    {ve_vl_vsclot_vvssl, 121383}, // __builtin_ve_vl_vsclot_vvssl
    {ve_vl_vsclot_vvssml, 121396}, // __builtin_ve_vl_vsclot_vvssml
    {ve_vl_vscnc_vvssl, 121410}, // __builtin_ve_vl_vscnc_vvssl
    {ve_vl_vscnc_vvssml, 121422}, // __builtin_ve_vl_vscnc_vvssml
    {ve_vl_vscncot_vvssl, 121435}, // __builtin_ve_vl_vscncot_vvssl
    {ve_vl_vscncot_vvssml, 121449}, // __builtin_ve_vl_vscncot_vvssml
    {ve_vl_vscot_vvssl, 121464}, // __builtin_ve_vl_vscot_vvssl
    {ve_vl_vscot_vvssml, 121476}, // __builtin_ve_vl_vscot_vvssml
    {ve_vl_vscu_vvssl, 121489}, // __builtin_ve_vl_vscu_vvssl
    {ve_vl_vscu_vvssml, 121500}, // __builtin_ve_vl_vscu_vvssml
    {ve_vl_vscunc_vvssl, 121512}, // __builtin_ve_vl_vscunc_vvssl
    {ve_vl_vscunc_vvssml, 121525}, // __builtin_ve_vl_vscunc_vvssml
    {ve_vl_vscuncot_vvssl, 121539}, // __builtin_ve_vl_vscuncot_vvssl
    {ve_vl_vscuncot_vvssml, 121554}, // __builtin_ve_vl_vscuncot_vvssml
    {ve_vl_vscuot_vvssl, 121570}, // __builtin_ve_vl_vscuot_vvssl
    {ve_vl_vscuot_vvssml, 121583}, // __builtin_ve_vl_vscuot_vvssml
    {ve_vl_vseq_vl, 121597}, // __builtin_ve_vl_vseq_vl
    {ve_vl_vseq_vvl, 121605}, // __builtin_ve_vl_vseq_vvl
    {ve_vl_vsfa_vvssl, 121614}, // __builtin_ve_vl_vsfa_vvssl
    {ve_vl_vsfa_vvssmvl, 121625}, // __builtin_ve_vl_vsfa_vvssmvl
    {ve_vl_vsfa_vvssvl, 121638}, // __builtin_ve_vl_vsfa_vvssvl
    {ve_vl_vshf_vvvsl, 121650}, // __builtin_ve_vl_vshf_vvvsl
    {ve_vl_vshf_vvvsvl, 121661}, // __builtin_ve_vl_vshf_vvvsvl
    {ve_vl_vslal_vvsl, 121673}, // __builtin_ve_vl_vslal_vvsl
    {ve_vl_vslal_vvsmvl, 121684}, // __builtin_ve_vl_vslal_vvsmvl
    {ve_vl_vslal_vvsvl, 121697}, // __builtin_ve_vl_vslal_vvsvl
    {ve_vl_vslal_vvvl, 121709}, // __builtin_ve_vl_vslal_vvvl
    {ve_vl_vslal_vvvmvl, 121720}, // __builtin_ve_vl_vslal_vvvmvl
    {ve_vl_vslal_vvvvl, 121733}, // __builtin_ve_vl_vslal_vvvvl
    {ve_vl_vslawsx_vvsl, 121745}, // __builtin_ve_vl_vslawsx_vvsl
    {ve_vl_vslawsx_vvsmvl, 121758}, // __builtin_ve_vl_vslawsx_vvsmvl
    {ve_vl_vslawsx_vvsvl, 121773}, // __builtin_ve_vl_vslawsx_vvsvl
    {ve_vl_vslawsx_vvvl, 121787}, // __builtin_ve_vl_vslawsx_vvvl
    {ve_vl_vslawsx_vvvmvl, 121800}, // __builtin_ve_vl_vslawsx_vvvmvl
    {ve_vl_vslawsx_vvvvl, 121815}, // __builtin_ve_vl_vslawsx_vvvvl
    {ve_vl_vslawzx_vvsl, 121829}, // __builtin_ve_vl_vslawzx_vvsl
    {ve_vl_vslawzx_vvsmvl, 121842}, // __builtin_ve_vl_vslawzx_vvsmvl
    {ve_vl_vslawzx_vvsvl, 121857}, // __builtin_ve_vl_vslawzx_vvsvl
    {ve_vl_vslawzx_vvvl, 121871}, // __builtin_ve_vl_vslawzx_vvvl
    {ve_vl_vslawzx_vvvmvl, 121884}, // __builtin_ve_vl_vslawzx_vvvmvl
    {ve_vl_vslawzx_vvvvl, 121899}, // __builtin_ve_vl_vslawzx_vvvvl
    {ve_vl_vsll_vvsl, 121913}, // __builtin_ve_vl_vsll_vvsl
    {ve_vl_vsll_vvsmvl, 121923}, // __builtin_ve_vl_vsll_vvsmvl
    {ve_vl_vsll_vvsvl, 121935}, // __builtin_ve_vl_vsll_vvsvl
    {ve_vl_vsll_vvvl, 121946}, // __builtin_ve_vl_vsll_vvvl
    {ve_vl_vsll_vvvmvl, 121956}, // __builtin_ve_vl_vsll_vvvmvl
    {ve_vl_vsll_vvvvl, 121968}, // __builtin_ve_vl_vsll_vvvvl
    {ve_vl_vsral_vvsl, 121979}, // __builtin_ve_vl_vsral_vvsl
    {ve_vl_vsral_vvsmvl, 121990}, // __builtin_ve_vl_vsral_vvsmvl
    {ve_vl_vsral_vvsvl, 122003}, // __builtin_ve_vl_vsral_vvsvl
    {ve_vl_vsral_vvvl, 122015}, // __builtin_ve_vl_vsral_vvvl
    {ve_vl_vsral_vvvmvl, 122026}, // __builtin_ve_vl_vsral_vvvmvl
    {ve_vl_vsral_vvvvl, 122039}, // __builtin_ve_vl_vsral_vvvvl
    {ve_vl_vsrawsx_vvsl, 122051}, // __builtin_ve_vl_vsrawsx_vvsl
    {ve_vl_vsrawsx_vvsmvl, 122064}, // __builtin_ve_vl_vsrawsx_vvsmvl
    {ve_vl_vsrawsx_vvsvl, 122079}, // __builtin_ve_vl_vsrawsx_vvsvl
    {ve_vl_vsrawsx_vvvl, 122093}, // __builtin_ve_vl_vsrawsx_vvvl
    {ve_vl_vsrawsx_vvvmvl, 122106}, // __builtin_ve_vl_vsrawsx_vvvmvl
    {ve_vl_vsrawsx_vvvvl, 122121}, // __builtin_ve_vl_vsrawsx_vvvvl
    {ve_vl_vsrawzx_vvsl, 122135}, // __builtin_ve_vl_vsrawzx_vvsl
    {ve_vl_vsrawzx_vvsmvl, 122148}, // __builtin_ve_vl_vsrawzx_vvsmvl
    {ve_vl_vsrawzx_vvsvl, 122163}, // __builtin_ve_vl_vsrawzx_vvsvl
    {ve_vl_vsrawzx_vvvl, 122177}, // __builtin_ve_vl_vsrawzx_vvvl
    {ve_vl_vsrawzx_vvvmvl, 122190}, // __builtin_ve_vl_vsrawzx_vvvmvl
    {ve_vl_vsrawzx_vvvvl, 122205}, // __builtin_ve_vl_vsrawzx_vvvvl
    {ve_vl_vsrl_vvsl, 122219}, // __builtin_ve_vl_vsrl_vvsl
    {ve_vl_vsrl_vvsmvl, 122229}, // __builtin_ve_vl_vsrl_vvsmvl
    {ve_vl_vsrl_vvsvl, 122241}, // __builtin_ve_vl_vsrl_vvsvl
    {ve_vl_vsrl_vvvl, 122252}, // __builtin_ve_vl_vsrl_vvvl
    {ve_vl_vsrl_vvvmvl, 122262}, // __builtin_ve_vl_vsrl_vvvmvl
    {ve_vl_vsrl_vvvvl, 122274}, // __builtin_ve_vl_vsrl_vvvvl
    {ve_vl_vst2d_vssl, 122285}, // __builtin_ve_vl_vst2d_vssl
    {ve_vl_vst2d_vssml, 122296}, // __builtin_ve_vl_vst2d_vssml
    {ve_vl_vst2dnc_vssl, 122308}, // __builtin_ve_vl_vst2dnc_vssl
    {ve_vl_vst2dnc_vssml, 122321}, // __builtin_ve_vl_vst2dnc_vssml
    {ve_vl_vst2dncot_vssl, 122335}, // __builtin_ve_vl_vst2dncot_vssl
    {ve_vl_vst2dncot_vssml, 122350}, // __builtin_ve_vl_vst2dncot_vssml
    {ve_vl_vst2dot_vssl, 122366}, // __builtin_ve_vl_vst2dot_vssl
    {ve_vl_vst2dot_vssml, 122379}, // __builtin_ve_vl_vst2dot_vssml
    {ve_vl_vst_vssl, 122393}, // __builtin_ve_vl_vst_vssl
    {ve_vl_vst_vssml, 122402}, // __builtin_ve_vl_vst_vssml
    {ve_vl_vstl2d_vssl, 122412}, // __builtin_ve_vl_vstl2d_vssl
    {ve_vl_vstl2d_vssml, 122424}, // __builtin_ve_vl_vstl2d_vssml
    {ve_vl_vstl2dnc_vssl, 122437}, // __builtin_ve_vl_vstl2dnc_vssl
    {ve_vl_vstl2dnc_vssml, 122451}, // __builtin_ve_vl_vstl2dnc_vssml
    {ve_vl_vstl2dncot_vssl, 122466}, // __builtin_ve_vl_vstl2dncot_vssl
    {ve_vl_vstl2dncot_vssml, 122482}, // __builtin_ve_vl_vstl2dncot_vssml
    {ve_vl_vstl2dot_vssl, 122499}, // __builtin_ve_vl_vstl2dot_vssl
    {ve_vl_vstl2dot_vssml, 122513}, // __builtin_ve_vl_vstl2dot_vssml
    {ve_vl_vstl_vssl, 122528}, // __builtin_ve_vl_vstl_vssl
    {ve_vl_vstl_vssml, 122538}, // __builtin_ve_vl_vstl_vssml
    {ve_vl_vstlnc_vssl, 122549}, // __builtin_ve_vl_vstlnc_vssl
    {ve_vl_vstlnc_vssml, 122561}, // __builtin_ve_vl_vstlnc_vssml
    {ve_vl_vstlncot_vssl, 122574}, // __builtin_ve_vl_vstlncot_vssl
    {ve_vl_vstlncot_vssml, 122588}, // __builtin_ve_vl_vstlncot_vssml
    {ve_vl_vstlot_vssl, 122603}, // __builtin_ve_vl_vstlot_vssl
    {ve_vl_vstlot_vssml, 122615}, // __builtin_ve_vl_vstlot_vssml
    {ve_vl_vstnc_vssl, 122628}, // __builtin_ve_vl_vstnc_vssl
    {ve_vl_vstnc_vssml, 122639}, // __builtin_ve_vl_vstnc_vssml
    {ve_vl_vstncot_vssl, 122651}, // __builtin_ve_vl_vstncot_vssl
    {ve_vl_vstncot_vssml, 122664}, // __builtin_ve_vl_vstncot_vssml
    {ve_vl_vstot_vssl, 122678}, // __builtin_ve_vl_vstot_vssl
    {ve_vl_vstot_vssml, 122689}, // __builtin_ve_vl_vstot_vssml
    {ve_vl_vstu2d_vssl, 122701}, // __builtin_ve_vl_vstu2d_vssl
    {ve_vl_vstu2d_vssml, 122713}, // __builtin_ve_vl_vstu2d_vssml
    {ve_vl_vstu2dnc_vssl, 122726}, // __builtin_ve_vl_vstu2dnc_vssl
    {ve_vl_vstu2dnc_vssml, 122740}, // __builtin_ve_vl_vstu2dnc_vssml
    {ve_vl_vstu2dncot_vssl, 122755}, // __builtin_ve_vl_vstu2dncot_vssl
    {ve_vl_vstu2dncot_vssml, 122771}, // __builtin_ve_vl_vstu2dncot_vssml
    {ve_vl_vstu2dot_vssl, 122788}, // __builtin_ve_vl_vstu2dot_vssl
    {ve_vl_vstu2dot_vssml, 122802}, // __builtin_ve_vl_vstu2dot_vssml
    {ve_vl_vstu_vssl, 122817}, // __builtin_ve_vl_vstu_vssl
    {ve_vl_vstu_vssml, 122827}, // __builtin_ve_vl_vstu_vssml
    {ve_vl_vstunc_vssl, 122838}, // __builtin_ve_vl_vstunc_vssl
    {ve_vl_vstunc_vssml, 122850}, // __builtin_ve_vl_vstunc_vssml
    {ve_vl_vstuncot_vssl, 122863}, // __builtin_ve_vl_vstuncot_vssl
    {ve_vl_vstuncot_vssml, 122877}, // __builtin_ve_vl_vstuncot_vssml
    {ve_vl_vstuot_vssl, 122892}, // __builtin_ve_vl_vstuot_vssl
    {ve_vl_vstuot_vssml, 122904}, // __builtin_ve_vl_vstuot_vssml
    {ve_vl_vsubsl_vsvl, 122917}, // __builtin_ve_vl_vsubsl_vsvl
    {ve_vl_vsubsl_vsvmvl, 122929}, // __builtin_ve_vl_vsubsl_vsvmvl
    {ve_vl_vsubsl_vsvvl, 122943}, // __builtin_ve_vl_vsubsl_vsvvl
    {ve_vl_vsubsl_vvvl, 122956}, // __builtin_ve_vl_vsubsl_vvvl
    {ve_vl_vsubsl_vvvmvl, 122968}, // __builtin_ve_vl_vsubsl_vvvmvl
    {ve_vl_vsubsl_vvvvl, 122982}, // __builtin_ve_vl_vsubsl_vvvvl
    {ve_vl_vsubswsx_vsvl, 122995}, // __builtin_ve_vl_vsubswsx_vsvl
    {ve_vl_vsubswsx_vsvmvl, 123009}, // __builtin_ve_vl_vsubswsx_vsvmvl
    {ve_vl_vsubswsx_vsvvl, 123025}, // __builtin_ve_vl_vsubswsx_vsvvl
    {ve_vl_vsubswsx_vvvl, 123040}, // __builtin_ve_vl_vsubswsx_vvvl
    {ve_vl_vsubswsx_vvvmvl, 123054}, // __builtin_ve_vl_vsubswsx_vvvmvl
    {ve_vl_vsubswsx_vvvvl, 123070}, // __builtin_ve_vl_vsubswsx_vvvvl
    {ve_vl_vsubswzx_vsvl, 123085}, // __builtin_ve_vl_vsubswzx_vsvl
    {ve_vl_vsubswzx_vsvmvl, 123099}, // __builtin_ve_vl_vsubswzx_vsvmvl
    {ve_vl_vsubswzx_vsvvl, 123115}, // __builtin_ve_vl_vsubswzx_vsvvl
    {ve_vl_vsubswzx_vvvl, 123130}, // __builtin_ve_vl_vsubswzx_vvvl
    {ve_vl_vsubswzx_vvvmvl, 123144}, // __builtin_ve_vl_vsubswzx_vvvmvl
    {ve_vl_vsubswzx_vvvvl, 123160}, // __builtin_ve_vl_vsubswzx_vvvvl
    {ve_vl_vsubul_vsvl, 123175}, // __builtin_ve_vl_vsubul_vsvl
    {ve_vl_vsubul_vsvmvl, 123187}, // __builtin_ve_vl_vsubul_vsvmvl
    {ve_vl_vsubul_vsvvl, 123201}, // __builtin_ve_vl_vsubul_vsvvl
    {ve_vl_vsubul_vvvl, 123214}, // __builtin_ve_vl_vsubul_vvvl
    {ve_vl_vsubul_vvvmvl, 123226}, // __builtin_ve_vl_vsubul_vvvmvl
    {ve_vl_vsubul_vvvvl, 123240}, // __builtin_ve_vl_vsubul_vvvvl
    {ve_vl_vsubuw_vsvl, 123253}, // __builtin_ve_vl_vsubuw_vsvl
    {ve_vl_vsubuw_vsvmvl, 123265}, // __builtin_ve_vl_vsubuw_vsvmvl
    {ve_vl_vsubuw_vsvvl, 123279}, // __builtin_ve_vl_vsubuw_vsvvl
    {ve_vl_vsubuw_vvvl, 123292}, // __builtin_ve_vl_vsubuw_vvvl
    {ve_vl_vsubuw_vvvmvl, 123304}, // __builtin_ve_vl_vsubuw_vvvmvl
    {ve_vl_vsubuw_vvvvl, 123318}, // __builtin_ve_vl_vsubuw_vvvvl
    {ve_vl_vsuml_vvl, 123331}, // __builtin_ve_vl_vsuml_vvl
    {ve_vl_vsuml_vvml, 123341}, // __builtin_ve_vl_vsuml_vvml
    {ve_vl_vsumwsx_vvl, 123352}, // __builtin_ve_vl_vsumwsx_vvl
    {ve_vl_vsumwsx_vvml, 123364}, // __builtin_ve_vl_vsumwsx_vvml
    {ve_vl_vsumwzx_vvl, 123377}, // __builtin_ve_vl_vsumwzx_vvl
    {ve_vl_vsumwzx_vvml, 123389}, // __builtin_ve_vl_vsumwzx_vvml
    {ve_vl_vxor_vsvl, 123402}, // __builtin_ve_vl_vxor_vsvl
    {ve_vl_vxor_vsvmvl, 123412}, // __builtin_ve_vl_vxor_vsvmvl
    {ve_vl_vxor_vsvvl, 123424}, // __builtin_ve_vl_vxor_vsvvl
    {ve_vl_vxor_vvvl, 123435}, // __builtin_ve_vl_vxor_vvvl
    {ve_vl_vxor_vvvmvl, 123445}, // __builtin_ve_vl_vxor_vvvmvl
    {ve_vl_vxor_vvvvl, 123457}, // __builtin_ve_vl_vxor_vvvvl
    {ve_vl_xorm_MMM, 123468}, // __builtin_ve_vl_xorm_MMM
    {ve_vl_xorm_mmm, 123477}, // __builtin_ve_vl_xorm_mmm
  }; // veNames

  // Builtins for x86.
  static constexpr BuiltinEntry x86Names[] = {
    {x86_aadd32, 123486}, // __builtin_ia32_aadd32
    {x86_aadd64, 123493}, // __builtin_ia32_aadd64
    {x86_aand32, 123500}, // __builtin_ia32_aand32
    {x86_aand64, 123507}, // __builtin_ia32_aand64
    {x86_avx512_add_pd_512, 123514}, // __builtin_ia32_addpd512
    {x86_avx512fp16_add_ph_512, 123523}, // __builtin_ia32_addph512
    {x86_avx512_add_ps_512, 123532}, // __builtin_ia32_addps512
    {x86_avx512_mask_add_sd_round, 123541}, // __builtin_ia32_addsd_round_mask
    {x86_avx512fp16_mask_add_sh_round, 123558}, // __builtin_ia32_addsh_round_mask
    {x86_avx512_mask_add_ss_round, 123575}, // __builtin_ia32_addss_round_mask
    {x86_sse3_addsub_pd, 123592}, // __builtin_ia32_addsubpd
    {x86_avx_addsub_pd_256, 123601}, // __builtin_ia32_addsubpd256
    {x86_sse3_addsub_ps, 123613}, // __builtin_ia32_addsubps
    {x86_avx_addsub_ps_256, 123622}, // __builtin_ia32_addsubps256
    {x86_aesni_aesdec, 123634}, // __builtin_ia32_aesdec128
    {x86_aesni_aesdec_256, 123644}, // __builtin_ia32_aesdec256
    {x86_aesni_aesdec_512, 123654}, // __builtin_ia32_aesdec512
    {x86_aesni_aesdeclast, 123664}, // __builtin_ia32_aesdeclast128
    {x86_aesni_aesdeclast_256, 123678}, // __builtin_ia32_aesdeclast256
    {x86_aesni_aesdeclast_512, 123692}, // __builtin_ia32_aesdeclast512
    {x86_aesni_aesenc, 123706}, // __builtin_ia32_aesenc128
    {x86_aesni_aesenc_256, 123716}, // __builtin_ia32_aesenc256
    {x86_aesni_aesenc_512, 123726}, // __builtin_ia32_aesenc512
    {x86_aesni_aesenclast, 123736}, // __builtin_ia32_aesenclast128
    {x86_aesni_aesenclast_256, 123750}, // __builtin_ia32_aesenclast256
    {x86_aesni_aesenclast_512, 123764}, // __builtin_ia32_aesenclast512
    {x86_aesni_aesimc, 123778}, // __builtin_ia32_aesimc128
    {x86_aesni_aeskeygenassist, 123788}, // __builtin_ia32_aeskeygenassist128
    {x86_aor32, 123807}, // __builtin_ia32_aor32
    {x86_aor64, 123813}, // __builtin_ia32_aor64
    {x86_axor32, 123819}, // __builtin_ia32_axor32
    {x86_axor64, 123826}, // __builtin_ia32_axor64
    {x86_bmi_bextr_32, 123833}, // __builtin_ia32_bextr_u32
    {x86_bmi_bextr_64, 123843}, // __builtin_ia32_bextr_u64
    {x86_tbm_bextri_u32, 123853}, // __builtin_ia32_bextri_u32
    {x86_tbm_bextri_u64, 123864}, // __builtin_ia32_bextri_u64
    {x86_sse41_blendvpd, 123875}, // __builtin_ia32_blendvpd
    {x86_avx_blendv_pd_256, 123884}, // __builtin_ia32_blendvpd256
    {x86_sse41_blendvps, 123896}, // __builtin_ia32_blendvps
    {x86_avx_blendv_ps_256, 123905}, // __builtin_ia32_blendvps256
    {x86_avx512_broadcastmb_128, 123917}, // __builtin_ia32_broadcastmb128
    {x86_avx512_broadcastmb_256, 123932}, // __builtin_ia32_broadcastmb256
    {x86_avx512_broadcastmb_512, 123947}, // __builtin_ia32_broadcastmb512
    {x86_avx512_broadcastmw_128, 123962}, // __builtin_ia32_broadcastmw128
    {x86_avx512_broadcastmw_256, 123977}, // __builtin_ia32_broadcastmw256
    {x86_avx512_broadcastmw_512, 123992}, // __builtin_ia32_broadcastmw512
    {x86_bmi_bzhi_64, 124007}, // __builtin_ia32_bzhi_di
    {x86_bmi_bzhi_32, 124015}, // __builtin_ia32_bzhi_si
    {x86_cldemote, 124023}, // __builtin_ia32_cldemote
    {x86_sse2_clflush, 124032}, // __builtin_ia32_clflush
    {x86_clflushopt, 124040}, // __builtin_ia32_clflushopt
    {x86_clrssbsy, 124051}, // __builtin_ia32_clrssbsy
    {x86_clui, 124060}, // __builtin_ia32_clui
    {x86_clwb, 124065}, // __builtin_ia32_clwb
    {x86_clzero, 124070}, // __builtin_ia32_clzero
    {x86_cmpccxadd32, 124077}, // __builtin_ia32_cmpccxadd32
    {x86_cmpccxadd64, 124089}, // __builtin_ia32_cmpccxadd64
    {x86_sse2_cmp_sd, 124101}, // __builtin_ia32_cmpsd
    {x86_avx512_mask_cmp_sd, 124107}, // __builtin_ia32_cmpsd_mask
    {x86_avx512fp16_mask_cmp_sh, 124118}, // __builtin_ia32_cmpsh_mask
    {x86_sse_cmp_ss, 124129}, // __builtin_ia32_cmpss
    {x86_avx512_mask_cmp_ss, 124135}, // __builtin_ia32_cmpss_mask
    {x86_sse_comieq_ss, 124146}, // __builtin_ia32_comieq
    {x86_sse_comige_ss, 124153}, // __builtin_ia32_comige
    {x86_sse_comigt_ss, 124160}, // __builtin_ia32_comigt
    {x86_sse_comile_ss, 124167}, // __builtin_ia32_comile
    {x86_sse_comilt_ss, 124174}, // __builtin_ia32_comilt
    {x86_sse_comineq_ss, 124181}, // __builtin_ia32_comineq
    {x86_sse2_comieq_sd, 124189}, // __builtin_ia32_comisdeq
    {x86_sse2_comige_sd, 124198}, // __builtin_ia32_comisdge
    {x86_sse2_comigt_sd, 124207}, // __builtin_ia32_comisdgt
    {x86_sse2_comile_sd, 124216}, // __builtin_ia32_comisdle
    {x86_sse2_comilt_sd, 124225}, // __builtin_ia32_comisdlt
    {x86_sse2_comineq_sd, 124234}, // __builtin_ia32_comisdneq
    {x86_sse42_crc32_64_64, 124244}, // __builtin_ia32_crc32di
    {x86_sse42_crc32_32_16, 124252}, // __builtin_ia32_crc32hi
    {x86_sse42_crc32_32_8, 124260}, // __builtin_ia32_crc32qi
    {x86_sse42_crc32_32_32, 124268}, // __builtin_ia32_crc32si
    {x86_avx512bf16_cvtne2ps2bf16_128, 124276}, // __builtin_ia32_cvtne2ps2bf16_128
    {x86_avx512bf16_cvtne2ps2bf16_256, 124294}, // __builtin_ia32_cvtne2ps2bf16_256
    {x86_avx512bf16_cvtne2ps2bf16_512, 124312}, // __builtin_ia32_cvtne2ps2bf16_512
    {x86_avx512bf16_cvtneps2bf16_256, 124330}, // __builtin_ia32_cvtneps2bf16_256
    {x86_avx512bf16_cvtneps2bf16_512, 124347}, // __builtin_ia32_cvtneps2bf16_512
    {x86_sse2_cvtpd2dq, 124364}, // __builtin_ia32_cvtpd2dq
    {x86_avx512_mask_cvtpd2dq_128, 124373}, // __builtin_ia32_cvtpd2dq128_mask
    {x86_avx_cvt_pd2dq_256, 124390}, // __builtin_ia32_cvtpd2dq256
    {x86_avx512_mask_cvtpd2dq_512, 124402}, // __builtin_ia32_cvtpd2dq512_mask
    {x86_sse2_cvtpd2ps, 124419}, // __builtin_ia32_cvtpd2ps
    {x86_avx_cvt_pd2_ps_256, 124428}, // __builtin_ia32_cvtpd2ps256
    {x86_avx512_mask_cvtpd2ps_512, 124440}, // __builtin_ia32_cvtpd2ps512_mask
    {x86_avx512_mask_cvtpd2ps, 124457}, // __builtin_ia32_cvtpd2ps_mask
    {x86_avx512_mask_cvtpd2qq_128, 124471}, // __builtin_ia32_cvtpd2qq128_mask
    {x86_avx512_mask_cvtpd2qq_256, 124488}, // __builtin_ia32_cvtpd2qq256_mask
    {x86_avx512_mask_cvtpd2qq_512, 124505}, // __builtin_ia32_cvtpd2qq512_mask
    {x86_avx512_mask_cvtpd2udq_128, 124522}, // __builtin_ia32_cvtpd2udq128_mask
    {x86_avx512_mask_cvtpd2udq_256, 124540}, // __builtin_ia32_cvtpd2udq256_mask
    {x86_avx512_mask_cvtpd2udq_512, 124558}, // __builtin_ia32_cvtpd2udq512_mask
    {x86_avx512_mask_cvtpd2uqq_128, 124576}, // __builtin_ia32_cvtpd2uqq128_mask
    {x86_avx512_mask_cvtpd2uqq_256, 124594}, // __builtin_ia32_cvtpd2uqq256_mask
    {x86_avx512_mask_cvtpd2uqq_512, 124612}, // __builtin_ia32_cvtpd2uqq512_mask
    {x86_sse2_cvtps2dq, 124630}, // __builtin_ia32_cvtps2dq
    {x86_avx512_mask_cvtps2dq_128, 124639}, // __builtin_ia32_cvtps2dq128_mask
    {x86_avx_cvt_ps2dq_256, 124656}, // __builtin_ia32_cvtps2dq256
    {x86_avx512_mask_cvtps2dq_256, 124668}, // __builtin_ia32_cvtps2dq256_mask
    {x86_avx512_mask_cvtps2dq_512, 124685}, // __builtin_ia32_cvtps2dq512_mask
    {x86_avx512_mask_cvtps2pd_512, 124702}, // __builtin_ia32_cvtps2pd512_mask
    {x86_avx512_mask_cvtps2qq_128, 124719}, // __builtin_ia32_cvtps2qq128_mask
    {x86_avx512_mask_cvtps2qq_256, 124736}, // __builtin_ia32_cvtps2qq256_mask
    {x86_avx512_mask_cvtps2qq_512, 124753}, // __builtin_ia32_cvtps2qq512_mask
    {x86_avx512_mask_cvtps2udq_128, 124770}, // __builtin_ia32_cvtps2udq128_mask
    {x86_avx512_mask_cvtps2udq_256, 124788}, // __builtin_ia32_cvtps2udq256_mask
    {x86_avx512_mask_cvtps2udq_512, 124806}, // __builtin_ia32_cvtps2udq512_mask
    {x86_avx512_mask_cvtps2uqq_128, 124824}, // __builtin_ia32_cvtps2uqq128_mask
    {x86_avx512_mask_cvtps2uqq_256, 124842}, // __builtin_ia32_cvtps2uqq256_mask
    {x86_avx512_mask_cvtps2uqq_512, 124860}, // __builtin_ia32_cvtps2uqq512_mask
    {x86_avx512_mask_cvtqq2ps_128, 124878}, // __builtin_ia32_cvtqq2ps128_mask
    {x86_sse2_cvtsd2si, 124895}, // __builtin_ia32_cvtsd2si
    {x86_sse2_cvtsd2si64, 124904}, // __builtin_ia32_cvtsd2si64
    {x86_sse2_cvtsd2ss, 124915}, // __builtin_ia32_cvtsd2ss
    {x86_avx512_mask_cvtsd2ss_round, 124924}, // __builtin_ia32_cvtsd2ss_round_mask
    {x86_avx512_cvtsi2sd64, 124944}, // __builtin_ia32_cvtsi2sd64
    {x86_avx512_cvtsi2ss32, 124955}, // __builtin_ia32_cvtsi2ss32
    {x86_avx512_cvtsi2ss64, 124966}, // __builtin_ia32_cvtsi2ss64
    {x86_avx512_mask_cvtss2sd_round, 124977}, // __builtin_ia32_cvtss2sd_round_mask
    {x86_sse_cvtss2si, 124997}, // __builtin_ia32_cvtss2si
    {x86_sse_cvtss2si64, 125006}, // __builtin_ia32_cvtss2si64
    {x86_sse2_cvttpd2dq, 125017}, // __builtin_ia32_cvttpd2dq
    {x86_avx512_mask_cvttpd2dq_128, 125027}, // __builtin_ia32_cvttpd2dq128_mask
    {x86_avx_cvtt_pd2dq_256, 125045}, // __builtin_ia32_cvttpd2dq256
    {x86_avx512_mask_cvttpd2dq_512, 125058}, // __builtin_ia32_cvttpd2dq512_mask
    {x86_avx512_mask_cvttpd2qq_128, 125076}, // __builtin_ia32_cvttpd2qq128_mask
    {x86_avx512_mask_cvttpd2qq_256, 125094}, // __builtin_ia32_cvttpd2qq256_mask
    {x86_avx512_mask_cvttpd2qq_512, 125112}, // __builtin_ia32_cvttpd2qq512_mask
    {x86_avx512_mask_cvttpd2udq_128, 125130}, // __builtin_ia32_cvttpd2udq128_mask
    {x86_avx512_mask_cvttpd2udq_256, 125149}, // __builtin_ia32_cvttpd2udq256_mask
    {x86_avx512_mask_cvttpd2udq_512, 125168}, // __builtin_ia32_cvttpd2udq512_mask
    {x86_avx512_mask_cvttpd2uqq_128, 125187}, // __builtin_ia32_cvttpd2uqq128_mask
    {x86_avx512_mask_cvttpd2uqq_256, 125206}, // __builtin_ia32_cvttpd2uqq256_mask
    {x86_avx512_mask_cvttpd2uqq_512, 125225}, // __builtin_ia32_cvttpd2uqq512_mask
    {x86_sse2_cvttps2dq, 125244}, // __builtin_ia32_cvttps2dq
    {x86_avx_cvtt_ps2dq_256, 125254}, // __builtin_ia32_cvttps2dq256
    {x86_avx512_mask_cvttps2dq_512, 125267}, // __builtin_ia32_cvttps2dq512_mask
    {x86_avx512_mask_cvttps2qq_128, 125285}, // __builtin_ia32_cvttps2qq128_mask
    {x86_avx512_mask_cvttps2qq_256, 125303}, // __builtin_ia32_cvttps2qq256_mask
    {x86_avx512_mask_cvttps2qq_512, 125321}, // __builtin_ia32_cvttps2qq512_mask
    {x86_avx512_mask_cvttps2udq_128, 125339}, // __builtin_ia32_cvttps2udq128_mask
    {x86_avx512_mask_cvttps2udq_256, 125358}, // __builtin_ia32_cvttps2udq256_mask
    {x86_avx512_mask_cvttps2udq_512, 125377}, // __builtin_ia32_cvttps2udq512_mask
    {x86_avx512_mask_cvttps2uqq_128, 125396}, // __builtin_ia32_cvttps2uqq128_mask
    {x86_avx512_mask_cvttps2uqq_256, 125415}, // __builtin_ia32_cvttps2uqq256_mask
    {x86_avx512_mask_cvttps2uqq_512, 125434}, // __builtin_ia32_cvttps2uqq512_mask
    {x86_sse2_cvttsd2si, 125453}, // __builtin_ia32_cvttsd2si
    {x86_sse2_cvttsd2si64, 125463}, // __builtin_ia32_cvttsd2si64
    {x86_sse_cvttss2si, 125475}, // __builtin_ia32_cvttss2si
    {x86_sse_cvttss2si64, 125485}, // __builtin_ia32_cvttss2si64
    {x86_avx512_mask_cvtuqq2ps_128, 125497}, // __builtin_ia32_cvtuqq2ps128_mask
    {x86_avx512_cvtusi642sd, 125515}, // __builtin_ia32_cvtusi2sd64
    {x86_avx512_cvtusi2ss, 125527}, // __builtin_ia32_cvtusi2ss32
    {x86_avx512_cvtusi642ss, 125539}, // __builtin_ia32_cvtusi2ss64
    {x86_avx512_dbpsadbw_128, 125551}, // __builtin_ia32_dbpsadbw128
    {x86_avx512_dbpsadbw_256, 125563}, // __builtin_ia32_dbpsadbw256
    {x86_avx512_dbpsadbw_512, 125575}, // __builtin_ia32_dbpsadbw512
    {x86_directstore32, 125587}, // __builtin_ia32_directstore_u32
    {x86_directstore64, 125603}, // __builtin_ia32_directstore_u64
    {x86_avx512_div_pd_512, 125619}, // __builtin_ia32_divpd512
    {x86_avx512fp16_div_ph_512, 125628}, // __builtin_ia32_divph512
    {x86_avx512_div_ps_512, 125637}, // __builtin_ia32_divps512
    {x86_avx512_mask_div_sd_round, 125646}, // __builtin_ia32_divsd_round_mask
    {x86_avx512fp16_mask_div_sh_round, 125663}, // __builtin_ia32_divsh_round_mask
    {x86_avx512_mask_div_ss_round, 125680}, // __builtin_ia32_divss_round_mask
    {x86_avx512bf16_dpbf16ps_128, 125697}, // __builtin_ia32_dpbf16ps_128
    {x86_avx512bf16_dpbf16ps_256, 125710}, // __builtin_ia32_dpbf16ps_256
    {x86_avx512bf16_dpbf16ps_512, 125723}, // __builtin_ia32_dpbf16ps_512
    {x86_sse41_dppd, 125736}, // __builtin_ia32_dppd
    {x86_sse41_dpps, 125741}, // __builtin_ia32_dpps
    {x86_avx_dp_ps_256, 125746}, // __builtin_ia32_dpps256
    {x86_mmx_emms, 125754}, // __builtin_ia32_emms
    {x86_enqcmd, 125759}, // __builtin_ia32_enqcmd
    {x86_enqcmds, 125766}, // __builtin_ia32_enqcmds
    {x86_sse4a_extrq, 125774}, // __builtin_ia32_extrq
    {x86_sse4a_extrqi, 125780}, // __builtin_ia32_extrqi
    {x86_avx512_mask_fixupimm_pd_128, 125787}, // __builtin_ia32_fixupimmpd128_mask
    {x86_avx512_maskz_fixupimm_pd_128, 125806}, // __builtin_ia32_fixupimmpd128_maskz
    {x86_avx512_mask_fixupimm_pd_256, 125826}, // __builtin_ia32_fixupimmpd256_mask
    {x86_avx512_maskz_fixupimm_pd_256, 125845}, // __builtin_ia32_fixupimmpd256_maskz
    {x86_avx512_mask_fixupimm_pd_512, 125865}, // __builtin_ia32_fixupimmpd512_mask
    {x86_avx512_maskz_fixupimm_pd_512, 125884}, // __builtin_ia32_fixupimmpd512_maskz
    {x86_avx512_mask_fixupimm_ps_128, 125904}, // __builtin_ia32_fixupimmps128_mask
    {x86_avx512_maskz_fixupimm_ps_128, 125923}, // __builtin_ia32_fixupimmps128_maskz
    {x86_avx512_mask_fixupimm_ps_256, 125943}, // __builtin_ia32_fixupimmps256_mask
    {x86_avx512_maskz_fixupimm_ps_256, 125962}, // __builtin_ia32_fixupimmps256_maskz
    {x86_avx512_mask_fixupimm_ps_512, 125982}, // __builtin_ia32_fixupimmps512_mask
    {x86_avx512_maskz_fixupimm_ps_512, 126001}, // __builtin_ia32_fixupimmps512_maskz
    {x86_avx512_mask_fixupimm_sd, 126021}, // __builtin_ia32_fixupimmsd_mask
    {x86_avx512_maskz_fixupimm_sd, 126037}, // __builtin_ia32_fixupimmsd_maskz
    {x86_avx512_mask_fixupimm_ss, 126054}, // __builtin_ia32_fixupimmss_mask
    {x86_avx512_maskz_fixupimm_ss, 126070}, // __builtin_ia32_fixupimmss_maskz
    {x86_avx512_mask_fpclass_sd, 126087}, // __builtin_ia32_fpclasssd_mask
    {x86_avx512fp16_mask_fpclass_sh, 126102}, // __builtin_ia32_fpclasssh_mask
    {x86_avx512_mask_fpclass_ss, 126117}, // __builtin_ia32_fpclassss_mask
    {x86_fxrstor, 126132}, // __builtin_ia32_fxrstor
    {x86_fxrstor64, 126140}, // __builtin_ia32_fxrstor64
    {x86_fxsave, 126150}, // __builtin_ia32_fxsave
    {x86_fxsave64, 126157}, // __builtin_ia32_fxsave64
    {x86_avx2_gather_d_d, 126166}, // __builtin_ia32_gatherd_d
    {x86_avx2_gather_d_d_256, 126176}, // __builtin_ia32_gatherd_d256
    {x86_avx2_gather_d_pd, 126189}, // __builtin_ia32_gatherd_pd
    {x86_avx2_gather_d_pd_256, 126200}, // __builtin_ia32_gatherd_pd256
    {x86_avx2_gather_d_ps, 126214}, // __builtin_ia32_gatherd_ps
    {x86_avx2_gather_d_ps_256, 126225}, // __builtin_ia32_gatherd_ps256
    {x86_avx2_gather_d_q, 126239}, // __builtin_ia32_gatherd_q
    {x86_avx2_gather_d_q_256, 126249}, // __builtin_ia32_gatherd_q256
    {x86_avx2_gather_q_d, 126262}, // __builtin_ia32_gatherq_d
    {x86_avx2_gather_q_d_256, 126272}, // __builtin_ia32_gatherq_d256
    {x86_avx2_gather_q_pd, 126285}, // __builtin_ia32_gatherq_pd
    {x86_avx2_gather_q_pd_256, 126296}, // __builtin_ia32_gatherq_pd256
    {x86_avx2_gather_q_ps, 126310}, // __builtin_ia32_gatherq_ps
    {x86_avx2_gather_q_ps_256, 126321}, // __builtin_ia32_gatherq_ps256
    {x86_avx2_gather_q_q, 126335}, // __builtin_ia32_gatherq_q
    {x86_avx2_gather_q_q_256, 126345}, // __builtin_ia32_gatherq_q256
    {x86_avx512_mask_getexp_pd_128, 126358}, // __builtin_ia32_getexppd128_mask
    {x86_avx512_mask_getexp_pd_256, 126375}, // __builtin_ia32_getexppd256_mask
    {x86_avx512_mask_getexp_pd_512, 126392}, // __builtin_ia32_getexppd512_mask
    {x86_avx512fp16_mask_getexp_ph_128, 126409}, // __builtin_ia32_getexpph128_mask
    {x86_avx512fp16_mask_getexp_ph_256, 126426}, // __builtin_ia32_getexpph256_mask
    {x86_avx512fp16_mask_getexp_ph_512, 126443}, // __builtin_ia32_getexpph512_mask
    {x86_avx512_mask_getexp_ps_128, 126460}, // __builtin_ia32_getexpps128_mask
    {x86_avx512_mask_getexp_ps_256, 126477}, // __builtin_ia32_getexpps256_mask
    {x86_avx512_mask_getexp_ps_512, 126494}, // __builtin_ia32_getexpps512_mask
    {x86_avx512_mask_getexp_sd, 126511}, // __builtin_ia32_getexpsd128_round_mask
    {x86_avx512fp16_mask_getexp_sh, 126534}, // __builtin_ia32_getexpsh128_round_mask
    {x86_avx512_mask_getexp_ss, 126557}, // __builtin_ia32_getexpss128_round_mask
    {x86_avx512_mask_getmant_pd_128, 126580}, // __builtin_ia32_getmantpd128_mask
    {x86_avx512_mask_getmant_pd_256, 126598}, // __builtin_ia32_getmantpd256_mask
    {x86_avx512_mask_getmant_pd_512, 126616}, // __builtin_ia32_getmantpd512_mask
    {x86_avx512fp16_mask_getmant_ph_128, 126634}, // __builtin_ia32_getmantph128_mask
    {x86_avx512fp16_mask_getmant_ph_256, 126652}, // __builtin_ia32_getmantph256_mask
    {x86_avx512fp16_mask_getmant_ph_512, 126670}, // __builtin_ia32_getmantph512_mask
    {x86_avx512_mask_getmant_ps_128, 126688}, // __builtin_ia32_getmantps128_mask
    {x86_avx512_mask_getmant_ps_256, 126706}, // __builtin_ia32_getmantps256_mask
    {x86_avx512_mask_getmant_ps_512, 126724}, // __builtin_ia32_getmantps512_mask
    {x86_avx512_mask_getmant_sd, 126742}, // __builtin_ia32_getmantsd_round_mask
    {x86_avx512fp16_mask_getmant_sh, 126763}, // __builtin_ia32_getmantsh_round_mask
    {x86_avx512_mask_getmant_ss, 126784}, // __builtin_ia32_getmantss_round_mask
    {x86_sse3_hadd_pd, 126805}, // __builtin_ia32_haddpd
    {x86_avx_hadd_pd_256, 126812}, // __builtin_ia32_haddpd256
    {x86_sse3_hadd_ps, 126822}, // __builtin_ia32_haddps
    {x86_avx_hadd_ps_256, 126829}, // __builtin_ia32_haddps256
    {x86_sse3_hsub_pd, 126839}, // __builtin_ia32_hsubpd
    {x86_avx_hsub_pd_256, 126846}, // __builtin_ia32_hsubpd256
    {x86_sse3_hsub_ps, 126856}, // __builtin_ia32_hsubps
    {x86_avx_hsub_ps_256, 126863}, // __builtin_ia32_hsubps256
    {x86_incsspd, 126873}, // __builtin_ia32_incsspd
    {x86_incsspq, 126881}, // __builtin_ia32_incsspq
    {x86_sse41_insertps, 126889}, // __builtin_ia32_insertps128
    {x86_sse4a_insertq, 126901}, // __builtin_ia32_insertq
    {x86_sse4a_insertqi, 126909}, // __builtin_ia32_insertqi
    {x86_invpcid, 126918}, // __builtin_ia32_invpcid
    {x86_sse3_ldu_dq, 126926}, // __builtin_ia32_lddqu
    {x86_avx_ldu_dq_256, 126932}, // __builtin_ia32_lddqu256
    {x86_sse2_lfence, 126941}, // __builtin_ia32_lfence
    {x86_llwpcb, 126948}, // __builtin_ia32_llwpcb
    {x86_loadiwkey, 126955}, // __builtin_ia32_loadiwkey
    {x86_lwpins32, 126965}, // __builtin_ia32_lwpins32
    {x86_lwpins64, 126974}, // __builtin_ia32_lwpins64
    {x86_lwpval32, 126983}, // __builtin_ia32_lwpval32
    {x86_lwpval64, 126992}, // __builtin_ia32_lwpval64
    {x86_avx2_maskload_d, 127001}, // __builtin_ia32_maskloadd
    {x86_avx2_maskload_d_256, 127011}, // __builtin_ia32_maskloadd256
    {x86_avx_maskload_pd, 127024}, // __builtin_ia32_maskloadpd
    {x86_avx_maskload_pd_256, 127035}, // __builtin_ia32_maskloadpd256
    {x86_avx_maskload_ps, 127049}, // __builtin_ia32_maskloadps
    {x86_avx_maskload_ps_256, 127060}, // __builtin_ia32_maskloadps256
    {x86_avx2_maskload_q, 127074}, // __builtin_ia32_maskloadq
    {x86_avx2_maskload_q_256, 127084}, // __builtin_ia32_maskloadq256
    {x86_sse2_maskmov_dqu, 127097}, // __builtin_ia32_maskmovdqu
    {x86_avx2_maskstore_d, 127108}, // __builtin_ia32_maskstored
    {x86_avx2_maskstore_d_256, 127119}, // __builtin_ia32_maskstored256
    {x86_avx_maskstore_pd, 127133}, // __builtin_ia32_maskstorepd
    {x86_avx_maskstore_pd_256, 127145}, // __builtin_ia32_maskstorepd256
    {x86_avx_maskstore_ps, 127160}, // __builtin_ia32_maskstoreps
    {x86_avx_maskstore_ps_256, 127172}, // __builtin_ia32_maskstoreps256
    {x86_avx2_maskstore_q, 127187}, // __builtin_ia32_maskstoreq
    {x86_avx2_maskstore_q_256, 127198}, // __builtin_ia32_maskstoreq256
    {x86_sse2_max_pd, 127212}, // __builtin_ia32_maxpd
    {x86_avx_max_pd_256, 127218}, // __builtin_ia32_maxpd256
    {x86_avx512_max_pd_512, 127227}, // __builtin_ia32_maxpd512
    {x86_avx512fp16_max_ph_128, 127236}, // __builtin_ia32_maxph128
    {x86_avx512fp16_max_ph_256, 127245}, // __builtin_ia32_maxph256
    {x86_avx512fp16_max_ph_512, 127254}, // __builtin_ia32_maxph512
    {x86_sse_max_ps, 127263}, // __builtin_ia32_maxps
    {x86_avx_max_ps_256, 127269}, // __builtin_ia32_maxps256
    {x86_avx512_max_ps_512, 127278}, // __builtin_ia32_maxps512
    {x86_sse2_max_sd, 127287}, // __builtin_ia32_maxsd
    {x86_avx512_mask_max_sd_round, 127293}, // __builtin_ia32_maxsd_round_mask
    {x86_avx512fp16_mask_max_sh_round, 127310}, // __builtin_ia32_maxsh_round_mask
    {x86_sse_max_ss, 127327}, // __builtin_ia32_maxss
    {x86_avx512_mask_max_ss_round, 127333}, // __builtin_ia32_maxss_round_mask
    {x86_sse2_mfence, 127350}, // __builtin_ia32_mfence
    {x86_sse2_min_pd, 127357}, // __builtin_ia32_minpd
    {x86_avx_min_pd_256, 127363}, // __builtin_ia32_minpd256
    {x86_avx512_min_pd_512, 127372}, // __builtin_ia32_minpd512
    {x86_avx512fp16_min_ph_128, 127381}, // __builtin_ia32_minph128
    {x86_avx512fp16_min_ph_256, 127390}, // __builtin_ia32_minph256
    {x86_avx512fp16_min_ph_512, 127399}, // __builtin_ia32_minph512
    {x86_sse_min_ps, 127408}, // __builtin_ia32_minps
    {x86_avx_min_ps_256, 127414}, // __builtin_ia32_minps256
    {x86_avx512_min_ps_512, 127423}, // __builtin_ia32_minps512
    {x86_sse2_min_sd, 127432}, // __builtin_ia32_minsd
    {x86_avx512_mask_min_sd_round, 127438}, // __builtin_ia32_minsd_round_mask
    {x86_avx512fp16_mask_min_sh_round, 127455}, // __builtin_ia32_minsh_round_mask
    {x86_sse_min_ss, 127472}, // __builtin_ia32_minss
    {x86_avx512_mask_min_ss_round, 127478}, // __builtin_ia32_minss_round_mask
    {x86_sse3_monitor, 127495}, // __builtin_ia32_monitor
    {x86_monitorx, 127503}, // __builtin_ia32_monitorx
    {x86_movdir64b, 127512}, // __builtin_ia32_movdir64b
    {x86_sse2_movmsk_pd, 127522}, // __builtin_ia32_movmskpd
    {x86_avx_movmsk_pd_256, 127531}, // __builtin_ia32_movmskpd256
    {x86_sse_movmsk_ps, 127543}, // __builtin_ia32_movmskps
    {x86_avx_movmsk_ps_256, 127552}, // __builtin_ia32_movmskps256
    {x86_movrsdi, 127564}, // __builtin_ia32_movrsdi
    {x86_movrshi, 127572}, // __builtin_ia32_movrshi
    {x86_movrsqi, 127580}, // __builtin_ia32_movrsqi
    {x86_movrssi, 127588}, // __builtin_ia32_movrssi
    {x86_sse41_mpsadbw, 127596}, // __builtin_ia32_mpsadbw128
    {x86_avx2_mpsadbw, 127607}, // __builtin_ia32_mpsadbw256
    {x86_avx10_vmpsadbw_512, 127618}, // __builtin_ia32_mpsadbw512
    {x86_avx512_mul_pd_512, 127629}, // __builtin_ia32_mulpd512
    {x86_avx512fp16_mul_ph_512, 127638}, // __builtin_ia32_mulph512
    {x86_avx512_mul_ps_512, 127647}, // __builtin_ia32_mulps512
    {x86_avx512_mask_mul_sd_round, 127656}, // __builtin_ia32_mulsd_round_mask
    {x86_avx512fp16_mask_mul_sh_round, 127673}, // __builtin_ia32_mulsh_round_mask
    {x86_avx512_mask_mul_ss_round, 127690}, // __builtin_ia32_mulss_round_mask
    {x86_sse3_mwait, 127707}, // __builtin_ia32_mwait
    {x86_mwaitx, 127713}, // __builtin_ia32_mwaitx
    {x86_sse2_packssdw_128, 127720}, // __builtin_ia32_packssdw128
    {x86_avx2_packssdw, 127732}, // __builtin_ia32_packssdw256
    {x86_avx512_packssdw_512, 127744}, // __builtin_ia32_packssdw512
    {x86_sse2_packsswb_128, 127756}, // __builtin_ia32_packsswb128
    {x86_avx2_packsswb, 127768}, // __builtin_ia32_packsswb256
    {x86_avx512_packsswb_512, 127780}, // __builtin_ia32_packsswb512
    {x86_sse41_packusdw, 127792}, // __builtin_ia32_packusdw128
    {x86_avx2_packusdw, 127804}, // __builtin_ia32_packusdw256
    {x86_avx512_packusdw_512, 127816}, // __builtin_ia32_packusdw512
    {x86_sse2_packuswb_128, 127828}, // __builtin_ia32_packuswb128
    {x86_avx2_packuswb, 127840}, // __builtin_ia32_packuswb256
    {x86_avx512_packuswb_512, 127852}, // __builtin_ia32_packuswb512
    {x86_sse2_pause, 127864}, // __builtin_ia32_pause
    {x86_sse2_pavg_b, 127870}, // __builtin_ia32_pavgb128
    {x86_avx2_pavg_b, 127879}, // __builtin_ia32_pavgb256
    {x86_avx512_pavg_b_512, 127888}, // __builtin_ia32_pavgb512
    {x86_sse2_pavg_w, 127897}, // __builtin_ia32_pavgw128
    {x86_avx2_pavg_w, 127906}, // __builtin_ia32_pavgw256
    {x86_avx512_pavg_w_512, 127915}, // __builtin_ia32_pavgw512
    {x86_sse41_pblendvb, 127924}, // __builtin_ia32_pblendvb128
    {x86_avx2_pblendvb, 127936}, // __builtin_ia32_pblendvb256
    {x86_pclmulqdq, 127948}, // __builtin_ia32_pclmulqdq128
    {x86_pclmulqdq_256, 127961}, // __builtin_ia32_pclmulqdq256
    {x86_pclmulqdq_512, 127974}, // __builtin_ia32_pclmulqdq512
    {x86_sse42_pcmpestri128, 127987}, // __builtin_ia32_pcmpestri128
    {x86_sse42_pcmpestria128, 128000}, // __builtin_ia32_pcmpestria128
    {x86_sse42_pcmpestric128, 128014}, // __builtin_ia32_pcmpestric128
    {x86_sse42_pcmpestrio128, 128028}, // __builtin_ia32_pcmpestrio128
    {x86_sse42_pcmpestris128, 128042}, // __builtin_ia32_pcmpestris128
    {x86_sse42_pcmpestriz128, 128056}, // __builtin_ia32_pcmpestriz128
    {x86_sse42_pcmpestrm128, 128070}, // __builtin_ia32_pcmpestrm128
    {x86_sse42_pcmpistri128, 128083}, // __builtin_ia32_pcmpistri128
    {x86_sse42_pcmpistria128, 128096}, // __builtin_ia32_pcmpistria128
    {x86_sse42_pcmpistric128, 128110}, // __builtin_ia32_pcmpistric128
    {x86_sse42_pcmpistrio128, 128124}, // __builtin_ia32_pcmpistrio128
    {x86_sse42_pcmpistris128, 128138}, // __builtin_ia32_pcmpistris128
    {x86_sse42_pcmpistriz128, 128152}, // __builtin_ia32_pcmpistriz128
    {x86_sse42_pcmpistrm128, 128166}, // __builtin_ia32_pcmpistrm128
    {x86_bmi_pdep_64, 128179}, // __builtin_ia32_pdep_di
    {x86_bmi_pdep_32, 128187}, // __builtin_ia32_pdep_si
    {x86_avx512_permvar_df_256, 128195}, // __builtin_ia32_permvardf256
    {x86_avx512_permvar_df_512, 128208}, // __builtin_ia32_permvardf512
    {x86_avx512_permvar_di_256, 128221}, // __builtin_ia32_permvardi256
    {x86_avx512_permvar_di_512, 128234}, // __builtin_ia32_permvardi512
    {x86_avx512_permvar_hi_128, 128247}, // __builtin_ia32_permvarhi128
    {x86_avx512_permvar_hi_256, 128260}, // __builtin_ia32_permvarhi256
    {x86_avx512_permvar_hi_512, 128273}, // __builtin_ia32_permvarhi512
    {x86_avx512_permvar_qi_128, 128286}, // __builtin_ia32_permvarqi128
    {x86_avx512_permvar_qi_256, 128299}, // __builtin_ia32_permvarqi256
    {x86_avx512_permvar_qi_512, 128312}, // __builtin_ia32_permvarqi512
    {x86_avx2_permps, 128325}, // __builtin_ia32_permvarsf256
    {x86_avx512_permvar_sf_512, 128338}, // __builtin_ia32_permvarsf512
    {x86_avx2_permd, 128351}, // __builtin_ia32_permvarsi256
    {x86_avx512_permvar_si_512, 128364}, // __builtin_ia32_permvarsi512
    {x86_bmi_pext_64, 128377}, // __builtin_ia32_pext_di
    {x86_bmi_pext_32, 128385}, // __builtin_ia32_pext_si
    {x86_ssse3_phadd_d_128, 128393}, // __builtin_ia32_phaddd128
    {x86_avx2_phadd_d, 128403}, // __builtin_ia32_phaddd256
    {x86_ssse3_phadd_sw_128, 128413}, // __builtin_ia32_phaddsw128
    {x86_avx2_phadd_sw, 128424}, // __builtin_ia32_phaddsw256
    {x86_ssse3_phadd_w_128, 128435}, // __builtin_ia32_phaddw128
    {x86_avx2_phadd_w, 128445}, // __builtin_ia32_phaddw256
    {x86_sse41_phminposuw, 128455}, // __builtin_ia32_phminposuw128
    {x86_ssse3_phsub_d_128, 128469}, // __builtin_ia32_phsubd128
    {x86_avx2_phsub_d, 128479}, // __builtin_ia32_phsubd256
    {x86_ssse3_phsub_sw_128, 128489}, // __builtin_ia32_phsubsw128
    {x86_avx2_phsub_sw, 128500}, // __builtin_ia32_phsubsw256
    {x86_ssse3_phsub_w_128, 128511}, // __builtin_ia32_phsubw128
    {x86_avx2_phsub_w, 128521}, // __builtin_ia32_phsubw256
    {x86_ssse3_pmadd_ub_sw_128, 128531}, // __builtin_ia32_pmaddubsw128
    {x86_avx2_pmadd_ub_sw, 128544}, // __builtin_ia32_pmaddubsw256
    {x86_avx512_pmaddubs_w_512, 128557}, // __builtin_ia32_pmaddubsw512
    {x86_sse2_pmadd_wd, 128570}, // __builtin_ia32_pmaddwd128
    {x86_avx2_pmadd_wd, 128581}, // __builtin_ia32_pmaddwd256
    {x86_avx512_pmaddw_d_512, 128592}, // __builtin_ia32_pmaddwd512
    {x86_avx512_mask_pmov_db_128, 128603}, // __builtin_ia32_pmovdb128_mask
    {x86_avx512_mask_pmov_db_mem_128, 128618}, // __builtin_ia32_pmovdb128mem_mask
    {x86_avx512_mask_pmov_db_256, 128636}, // __builtin_ia32_pmovdb256_mask
    {x86_avx512_mask_pmov_db_mem_256, 128651}, // __builtin_ia32_pmovdb256mem_mask
    {x86_avx512_mask_pmov_db_mem_512, 128669}, // __builtin_ia32_pmovdb512mem_mask
    {x86_avx512_mask_pmov_dw_128, 128687}, // __builtin_ia32_pmovdw128_mask
    {x86_avx512_mask_pmov_dw_mem_128, 128702}, // __builtin_ia32_pmovdw128mem_mask
    {x86_avx512_mask_pmov_dw_256, 128720}, // __builtin_ia32_pmovdw256_mask
    {x86_avx512_mask_pmov_dw_mem_256, 128735}, // __builtin_ia32_pmovdw256mem_mask
    {x86_avx512_mask_pmov_dw_mem_512, 128753}, // __builtin_ia32_pmovdw512mem_mask
    {x86_sse2_pmovmskb_128, 128771}, // __builtin_ia32_pmovmskb128
    {x86_avx2_pmovmskb, 128783}, // __builtin_ia32_pmovmskb256
    {x86_avx512_mask_pmov_qb_128, 128795}, // __builtin_ia32_pmovqb128_mask
    {x86_avx512_mask_pmov_qb_mem_128, 128810}, // __builtin_ia32_pmovqb128mem_mask
    {x86_avx512_mask_pmov_qb_256, 128828}, // __builtin_ia32_pmovqb256_mask
    {x86_avx512_mask_pmov_qb_mem_256, 128843}, // __builtin_ia32_pmovqb256mem_mask
    {x86_avx512_mask_pmov_qb_512, 128861}, // __builtin_ia32_pmovqb512_mask
    {x86_avx512_mask_pmov_qb_mem_512, 128876}, // __builtin_ia32_pmovqb512mem_mask
    {x86_avx512_mask_pmov_qd_128, 128894}, // __builtin_ia32_pmovqd128_mask
    {x86_avx512_mask_pmov_qd_mem_128, 128909}, // __builtin_ia32_pmovqd128mem_mask
    {x86_avx512_mask_pmov_qd_mem_256, 128927}, // __builtin_ia32_pmovqd256mem_mask
    {x86_avx512_mask_pmov_qd_mem_512, 128945}, // __builtin_ia32_pmovqd512mem_mask
    {x86_avx512_mask_pmov_qw_128, 128963}, // __builtin_ia32_pmovqw128_mask
    {x86_avx512_mask_pmov_qw_mem_128, 128978}, // __builtin_ia32_pmovqw128mem_mask
    {x86_avx512_mask_pmov_qw_256, 128996}, // __builtin_ia32_pmovqw256_mask
    {x86_avx512_mask_pmov_qw_mem_256, 129011}, // __builtin_ia32_pmovqw256mem_mask
    {x86_avx512_mask_pmov_qw_mem_512, 129029}, // __builtin_ia32_pmovqw512mem_mask
    {x86_avx512_mask_pmovs_db_128, 129047}, // __builtin_ia32_pmovsdb128_mask
    {x86_avx512_mask_pmovs_db_mem_128, 129063}, // __builtin_ia32_pmovsdb128mem_mask
    {x86_avx512_mask_pmovs_db_256, 129082}, // __builtin_ia32_pmovsdb256_mask
    {x86_avx512_mask_pmovs_db_mem_256, 129098}, // __builtin_ia32_pmovsdb256mem_mask
    {x86_avx512_mask_pmovs_db_512, 129117}, // __builtin_ia32_pmovsdb512_mask
    {x86_avx512_mask_pmovs_db_mem_512, 129133}, // __builtin_ia32_pmovsdb512mem_mask
    {x86_avx512_mask_pmovs_dw_128, 129152}, // __builtin_ia32_pmovsdw128_mask
    {x86_avx512_mask_pmovs_dw_mem_128, 129168}, // __builtin_ia32_pmovsdw128mem_mask
    {x86_avx512_mask_pmovs_dw_256, 129187}, // __builtin_ia32_pmovsdw256_mask
    {x86_avx512_mask_pmovs_dw_mem_256, 129203}, // __builtin_ia32_pmovsdw256mem_mask
    {x86_avx512_mask_pmovs_dw_512, 129222}, // __builtin_ia32_pmovsdw512_mask
    {x86_avx512_mask_pmovs_dw_mem_512, 129238}, // __builtin_ia32_pmovsdw512mem_mask
    {x86_avx512_mask_pmovs_qb_128, 129257}, // __builtin_ia32_pmovsqb128_mask
    {x86_avx512_mask_pmovs_qb_mem_128, 129273}, // __builtin_ia32_pmovsqb128mem_mask
    {x86_avx512_mask_pmovs_qb_256, 129292}, // __builtin_ia32_pmovsqb256_mask
    {x86_avx512_mask_pmovs_qb_mem_256, 129308}, // __builtin_ia32_pmovsqb256mem_mask
    {x86_avx512_mask_pmovs_qb_512, 129327}, // __builtin_ia32_pmovsqb512_mask
    {x86_avx512_mask_pmovs_qb_mem_512, 129343}, // __builtin_ia32_pmovsqb512mem_mask
    {x86_avx512_mask_pmovs_qd_128, 129362}, // __builtin_ia32_pmovsqd128_mask
    {x86_avx512_mask_pmovs_qd_mem_128, 129378}, // __builtin_ia32_pmovsqd128mem_mask
    {x86_avx512_mask_pmovs_qd_256, 129397}, // __builtin_ia32_pmovsqd256_mask
    {x86_avx512_mask_pmovs_qd_mem_256, 129413}, // __builtin_ia32_pmovsqd256mem_mask
    {x86_avx512_mask_pmovs_qd_512, 129432}, // __builtin_ia32_pmovsqd512_mask
    {x86_avx512_mask_pmovs_qd_mem_512, 129448}, // __builtin_ia32_pmovsqd512mem_mask
    {x86_avx512_mask_pmovs_qw_128, 129467}, // __builtin_ia32_pmovsqw128_mask
    {x86_avx512_mask_pmovs_qw_mem_128, 129483}, // __builtin_ia32_pmovsqw128mem_mask
    {x86_avx512_mask_pmovs_qw_256, 129502}, // __builtin_ia32_pmovsqw256_mask
    {x86_avx512_mask_pmovs_qw_mem_256, 129518}, // __builtin_ia32_pmovsqw256mem_mask
    {x86_avx512_mask_pmovs_qw_512, 129537}, // __builtin_ia32_pmovsqw512_mask
    {x86_avx512_mask_pmovs_qw_mem_512, 129553}, // __builtin_ia32_pmovsqw512mem_mask
    {x86_avx512_mask_pmovs_wb_128, 129572}, // __builtin_ia32_pmovswb128_mask
    {x86_avx512_mask_pmovs_wb_mem_128, 129588}, // __builtin_ia32_pmovswb128mem_mask
    {x86_avx512_mask_pmovs_wb_256, 129607}, // __builtin_ia32_pmovswb256_mask
    {x86_avx512_mask_pmovs_wb_mem_256, 129623}, // __builtin_ia32_pmovswb256mem_mask
    {x86_avx512_mask_pmovs_wb_512, 129642}, // __builtin_ia32_pmovswb512_mask
    {x86_avx512_mask_pmovs_wb_mem_512, 129658}, // __builtin_ia32_pmovswb512mem_mask
    {x86_avx512_mask_pmovus_db_128, 129677}, // __builtin_ia32_pmovusdb128_mask
    {x86_avx512_mask_pmovus_db_mem_128, 129694}, // __builtin_ia32_pmovusdb128mem_mask
    {x86_avx512_mask_pmovus_db_256, 129714}, // __builtin_ia32_pmovusdb256_mask
    {x86_avx512_mask_pmovus_db_mem_256, 129731}, // __builtin_ia32_pmovusdb256mem_mask
    {x86_avx512_mask_pmovus_db_512, 129751}, // __builtin_ia32_pmovusdb512_mask
    {x86_avx512_mask_pmovus_db_mem_512, 129768}, // __builtin_ia32_pmovusdb512mem_mask
    {x86_avx512_mask_pmovus_dw_128, 129788}, // __builtin_ia32_pmovusdw128_mask
    {x86_avx512_mask_pmovus_dw_mem_128, 129805}, // __builtin_ia32_pmovusdw128mem_mask
    {x86_avx512_mask_pmovus_dw_256, 129825}, // __builtin_ia32_pmovusdw256_mask
    {x86_avx512_mask_pmovus_dw_mem_256, 129842}, // __builtin_ia32_pmovusdw256mem_mask
    {x86_avx512_mask_pmovus_dw_512, 129862}, // __builtin_ia32_pmovusdw512_mask
    {x86_avx512_mask_pmovus_dw_mem_512, 129879}, // __builtin_ia32_pmovusdw512mem_mask
    {x86_avx512_mask_pmovus_qb_128, 129899}, // __builtin_ia32_pmovusqb128_mask
    {x86_avx512_mask_pmovus_qb_mem_128, 129916}, // __builtin_ia32_pmovusqb128mem_mask
    {x86_avx512_mask_pmovus_qb_256, 129936}, // __builtin_ia32_pmovusqb256_mask
    {x86_avx512_mask_pmovus_qb_mem_256, 129953}, // __builtin_ia32_pmovusqb256mem_mask
    {x86_avx512_mask_pmovus_qb_512, 129973}, // __builtin_ia32_pmovusqb512_mask
    {x86_avx512_mask_pmovus_qb_mem_512, 129990}, // __builtin_ia32_pmovusqb512mem_mask
    {x86_avx512_mask_pmovus_qd_128, 130010}, // __builtin_ia32_pmovusqd128_mask
    {x86_avx512_mask_pmovus_qd_mem_128, 130027}, // __builtin_ia32_pmovusqd128mem_mask
    {x86_avx512_mask_pmovus_qd_256, 130047}, // __builtin_ia32_pmovusqd256_mask
    {x86_avx512_mask_pmovus_qd_mem_256, 130064}, // __builtin_ia32_pmovusqd256mem_mask
    {x86_avx512_mask_pmovus_qd_512, 130084}, // __builtin_ia32_pmovusqd512_mask
    {x86_avx512_mask_pmovus_qd_mem_512, 130101}, // __builtin_ia32_pmovusqd512mem_mask
    {x86_avx512_mask_pmovus_qw_128, 130121}, // __builtin_ia32_pmovusqw128_mask
    {x86_avx512_mask_pmovus_qw_mem_128, 130138}, // __builtin_ia32_pmovusqw128mem_mask
    {x86_avx512_mask_pmovus_qw_256, 130158}, // __builtin_ia32_pmovusqw256_mask
    {x86_avx512_mask_pmovus_qw_mem_256, 130175}, // __builtin_ia32_pmovusqw256mem_mask
    {x86_avx512_mask_pmovus_qw_512, 130195}, // __builtin_ia32_pmovusqw512_mask
    {x86_avx512_mask_pmovus_qw_mem_512, 130212}, // __builtin_ia32_pmovusqw512mem_mask
    {x86_avx512_mask_pmovus_wb_128, 130232}, // __builtin_ia32_pmovuswb128_mask
    {x86_avx512_mask_pmovus_wb_mem_128, 130249}, // __builtin_ia32_pmovuswb128mem_mask
    {x86_avx512_mask_pmovus_wb_256, 130269}, // __builtin_ia32_pmovuswb256_mask
    {x86_avx512_mask_pmovus_wb_mem_256, 130286}, // __builtin_ia32_pmovuswb256mem_mask
    {x86_avx512_mask_pmovus_wb_512, 130306}, // __builtin_ia32_pmovuswb512_mask
    {x86_avx512_mask_pmovus_wb_mem_512, 130323}, // __builtin_ia32_pmovuswb512mem_mask
    {x86_avx512_mask_pmov_wb_128, 130343}, // __builtin_ia32_pmovwb128_mask
    {x86_avx512_mask_pmov_wb_mem_128, 130358}, // __builtin_ia32_pmovwb128mem_mask
    {x86_avx512_mask_pmov_wb_mem_256, 130376}, // __builtin_ia32_pmovwb256mem_mask
    {x86_avx512_mask_pmov_wb_mem_512, 130394}, // __builtin_ia32_pmovwb512mem_mask
    {x86_ssse3_pmul_hr_sw_128, 130412}, // __builtin_ia32_pmulhrsw128
    {x86_avx2_pmul_hr_sw, 130424}, // __builtin_ia32_pmulhrsw256
    {x86_avx512_pmul_hr_sw_512, 130436}, // __builtin_ia32_pmulhrsw512
    {x86_sse2_pmulhu_w, 130448}, // __builtin_ia32_pmulhuw128
    {x86_avx2_pmulhu_w, 130459}, // __builtin_ia32_pmulhuw256
    {x86_avx512_pmulhu_w_512, 130470}, // __builtin_ia32_pmulhuw512
    {x86_sse2_pmulh_w, 130481}, // __builtin_ia32_pmulhw128
    {x86_avx2_pmulh_w, 130491}, // __builtin_ia32_pmulhw256
    {x86_avx512_pmulh_w_512, 130501}, // __builtin_ia32_pmulhw512
    {x86_prefetchrs, 130511}, // __builtin_ia32_prefetchrs
    {x86_sse2_psad_bw, 130522}, // __builtin_ia32_psadbw128
    {x86_avx2_psad_bw, 130532}, // __builtin_ia32_psadbw256
    {x86_avx512_psad_bw_512, 130542}, // __builtin_ia32_psadbw512
    {x86_ssse3_pshuf_b_128, 130552}, // __builtin_ia32_pshufb128
    {x86_avx2_pshuf_b, 130562}, // __builtin_ia32_pshufb256
    {x86_avx512_pshuf_b_512, 130572}, // __builtin_ia32_pshufb512
    {x86_ssse3_psign_b_128, 130582}, // __builtin_ia32_psignb128
    {x86_avx2_psign_b, 130592}, // __builtin_ia32_psignb256
    {x86_ssse3_psign_d_128, 130602}, // __builtin_ia32_psignd128
    {x86_avx2_psign_d, 130612}, // __builtin_ia32_psignd256
    {x86_ssse3_psign_w_128, 130622}, // __builtin_ia32_psignw128
    {x86_avx2_psign_w, 130632}, // __builtin_ia32_psignw256
    {x86_sse2_psll_d, 130642}, // __builtin_ia32_pslld128
    {x86_avx2_psll_d, 130651}, // __builtin_ia32_pslld256
    {x86_avx512_psll_d_512, 130660}, // __builtin_ia32_pslld512
    {x86_sse2_pslli_d, 130669}, // __builtin_ia32_pslldi128
    {x86_avx2_pslli_d, 130679}, // __builtin_ia32_pslldi256
    {x86_avx512_pslli_d_512, 130689}, // __builtin_ia32_pslldi512
    {x86_sse2_psll_q, 130699}, // __builtin_ia32_psllq128
    {x86_avx2_psll_q, 130708}, // __builtin_ia32_psllq256
    {x86_avx512_psll_q_512, 130717}, // __builtin_ia32_psllq512
    {x86_sse2_pslli_q, 130726}, // __builtin_ia32_psllqi128
    {x86_avx2_pslli_q, 130736}, // __builtin_ia32_psllqi256
    {x86_avx512_pslli_q_512, 130746}, // __builtin_ia32_psllqi512
    {x86_avx512_psllv_w_256, 130756}, // __builtin_ia32_psllv16hi
    {x86_avx512_psllv_d_512, 130766}, // __builtin_ia32_psllv16si
    {x86_avx2_psllv_q, 130776}, // __builtin_ia32_psllv2di
    {x86_avx512_psllv_w_512, 130785}, // __builtin_ia32_psllv32hi
    {x86_avx2_psllv_q_256, 130795}, // __builtin_ia32_psllv4di
    {x86_avx2_psllv_d, 130804}, // __builtin_ia32_psllv4si
    {x86_avx512_psllv_q_512, 130813}, // __builtin_ia32_psllv8di
    {x86_avx512_psllv_w_128, 130822}, // __builtin_ia32_psllv8hi
    {x86_avx2_psllv_d_256, 130831}, // __builtin_ia32_psllv8si
    {x86_sse2_psll_w, 130840}, // __builtin_ia32_psllw128
    {x86_avx2_psll_w, 130849}, // __builtin_ia32_psllw256
    {x86_avx512_psll_w_512, 130858}, // __builtin_ia32_psllw512
    {x86_sse2_pslli_w, 130867}, // __builtin_ia32_psllwi128
    {x86_avx2_pslli_w, 130877}, // __builtin_ia32_psllwi256
    {x86_avx512_pslli_w_512, 130887}, // __builtin_ia32_psllwi512
    {x86_sse2_psra_d, 130897}, // __builtin_ia32_psrad128
    {x86_avx2_psra_d, 130906}, // __builtin_ia32_psrad256
    {x86_avx512_psra_d_512, 130915}, // __builtin_ia32_psrad512
    {x86_sse2_psrai_d, 130924}, // __builtin_ia32_psradi128
    {x86_avx2_psrai_d, 130934}, // __builtin_ia32_psradi256
    {x86_avx512_psrai_d_512, 130944}, // __builtin_ia32_psradi512
    {x86_avx512_psra_q_128, 130954}, // __builtin_ia32_psraq128
    {x86_avx512_psra_q_256, 130963}, // __builtin_ia32_psraq256
    {x86_avx512_psra_q_512, 130972}, // __builtin_ia32_psraq512
    {x86_avx512_psrai_q_128, 130981}, // __builtin_ia32_psraqi128
    {x86_avx512_psrai_q_256, 130991}, // __builtin_ia32_psraqi256
    {x86_avx512_psrai_q_512, 131001}, // __builtin_ia32_psraqi512
    {x86_avx512_psrav_w_256, 131011}, // __builtin_ia32_psrav16hi
    {x86_avx512_psrav_d_512, 131021}, // __builtin_ia32_psrav16si
    {x86_avx512_psrav_w_512, 131031}, // __builtin_ia32_psrav32hi
    {x86_avx2_psrav_d, 131041}, // __builtin_ia32_psrav4si
    {x86_avx512_psrav_q_512, 131050}, // __builtin_ia32_psrav8di
    {x86_avx512_psrav_w_128, 131059}, // __builtin_ia32_psrav8hi
    {x86_avx2_psrav_d_256, 131068}, // __builtin_ia32_psrav8si
    {x86_avx512_psrav_q_128, 131077}, // __builtin_ia32_psravq128
    {x86_avx512_psrav_q_256, 131087}, // __builtin_ia32_psravq256
    {x86_sse2_psra_w, 131097}, // __builtin_ia32_psraw128
    {x86_avx2_psra_w, 131106}, // __builtin_ia32_psraw256
    {x86_avx512_psra_w_512, 131115}, // __builtin_ia32_psraw512
    {x86_sse2_psrai_w, 131124}, // __builtin_ia32_psrawi128
    {x86_avx2_psrai_w, 131134}, // __builtin_ia32_psrawi256
    {x86_avx512_psrai_w_512, 131144}, // __builtin_ia32_psrawi512
    {x86_sse2_psrl_d, 131154}, // __builtin_ia32_psrld128
    {x86_avx2_psrl_d, 131163}, // __builtin_ia32_psrld256
    {x86_avx512_psrl_d_512, 131172}, // __builtin_ia32_psrld512
    {x86_sse2_psrli_d, 131181}, // __builtin_ia32_psrldi128
    {x86_avx2_psrli_d, 131191}, // __builtin_ia32_psrldi256
    {x86_avx512_psrli_d_512, 131201}, // __builtin_ia32_psrldi512
    {x86_sse2_psrl_q, 131211}, // __builtin_ia32_psrlq128
    {x86_avx2_psrl_q, 131220}, // __builtin_ia32_psrlq256
    {x86_avx512_psrl_q_512, 131229}, // __builtin_ia32_psrlq512
    {x86_sse2_psrli_q, 131238}, // __builtin_ia32_psrlqi128
    {x86_avx2_psrli_q, 131248}, // __builtin_ia32_psrlqi256
    {x86_avx512_psrli_q_512, 131258}, // __builtin_ia32_psrlqi512
    {x86_avx512_psrlv_w_256, 131268}, // __builtin_ia32_psrlv16hi
    {x86_avx512_psrlv_d_512, 131278}, // __builtin_ia32_psrlv16si
    {x86_avx2_psrlv_q, 131288}, // __builtin_ia32_psrlv2di
    {x86_avx512_psrlv_w_512, 131297}, // __builtin_ia32_psrlv32hi
    {x86_avx2_psrlv_q_256, 131307}, // __builtin_ia32_psrlv4di
    {x86_avx2_psrlv_d, 131316}, // __builtin_ia32_psrlv4si
    {x86_avx512_psrlv_q_512, 131325}, // __builtin_ia32_psrlv8di
    {x86_avx512_psrlv_w_128, 131334}, // __builtin_ia32_psrlv8hi
    {x86_avx2_psrlv_d_256, 131343}, // __builtin_ia32_psrlv8si
    {x86_sse2_psrl_w, 131352}, // __builtin_ia32_psrlw128
    {x86_avx2_psrl_w, 131361}, // __builtin_ia32_psrlw256
    {x86_avx512_psrl_w_512, 131370}, // __builtin_ia32_psrlw512
    {x86_sse2_psrli_w, 131379}, // __builtin_ia32_psrlwi128
    {x86_avx2_psrli_w, 131389}, // __builtin_ia32_psrlwi256
    {x86_avx512_psrli_w_512, 131399}, // __builtin_ia32_psrlwi512
    {x86_avx512_pternlog_d_128, 131409}, // __builtin_ia32_pternlogd128
    {x86_avx512_pternlog_d_256, 131422}, // __builtin_ia32_pternlogd256
    {x86_avx512_pternlog_d_512, 131435}, // __builtin_ia32_pternlogd512
    {x86_avx512_pternlog_q_128, 131448}, // __builtin_ia32_pternlogq128
    {x86_avx512_pternlog_q_256, 131461}, // __builtin_ia32_pternlogq256
    {x86_avx512_pternlog_q_512, 131474}, // __builtin_ia32_pternlogq512
    {x86_sse41_ptestc, 131487}, // __builtin_ia32_ptestc128
    {x86_avx_ptestc_256, 131497}, // __builtin_ia32_ptestc256
    {x86_sse41_ptestnzc, 131507}, // __builtin_ia32_ptestnzc128
    {x86_avx_ptestnzc_256, 131519}, // __builtin_ia32_ptestnzc256
    {x86_sse41_ptestz, 131531}, // __builtin_ia32_ptestz128
    {x86_avx_ptestz_256, 131541}, // __builtin_ia32_ptestz256
    {x86_ptwrite32, 131551}, // __builtin_ia32_ptwrite32
    {x86_ptwrite64, 131561}, // __builtin_ia32_ptwrite64
    {x86_avx512_mask_range_pd_128, 131571}, // __builtin_ia32_rangepd128_mask
    {x86_avx512_mask_range_pd_256, 131587}, // __builtin_ia32_rangepd256_mask
    {x86_avx512_mask_range_pd_512, 131603}, // __builtin_ia32_rangepd512_mask
    {x86_avx512_mask_range_ps_128, 131619}, // __builtin_ia32_rangeps128_mask
    {x86_avx512_mask_range_ps_256, 131635}, // __builtin_ia32_rangeps256_mask
    {x86_avx512_mask_range_ps_512, 131651}, // __builtin_ia32_rangeps512_mask
    {x86_avx512_mask_range_sd, 131667}, // __builtin_ia32_rangesd128_round_mask
    {x86_avx512_mask_range_ss, 131689}, // __builtin_ia32_rangess128_round_mask
    {x86_avx512_rcp14_pd_128, 131711}, // __builtin_ia32_rcp14pd128_mask
    {x86_avx512_rcp14_pd_256, 131727}, // __builtin_ia32_rcp14pd256_mask
    {x86_avx512_rcp14_pd_512, 131743}, // __builtin_ia32_rcp14pd512_mask
    {x86_avx512_rcp14_ps_128, 131759}, // __builtin_ia32_rcp14ps128_mask
    {x86_avx512_rcp14_ps_256, 131775}, // __builtin_ia32_rcp14ps256_mask
    {x86_avx512_rcp14_ps_512, 131791}, // __builtin_ia32_rcp14ps512_mask
    {x86_avx512_rcp14_sd, 131807}, // __builtin_ia32_rcp14sd_mask
    {x86_avx512_rcp14_ss, 131820}, // __builtin_ia32_rcp14ss_mask
    {x86_avx512fp16_mask_rcp_ph_128, 131833}, // __builtin_ia32_rcpph128_mask
    {x86_avx512fp16_mask_rcp_ph_256, 131847}, // __builtin_ia32_rcpph256_mask
    {x86_avx512fp16_mask_rcp_ph_512, 131861}, // __builtin_ia32_rcpph512_mask
    {x86_sse_rcp_ps, 131875}, // __builtin_ia32_rcpps
    {x86_avx_rcp_ps_256, 131881}, // __builtin_ia32_rcpps256
    {x86_avx512fp16_mask_rcp_sh, 131890}, // __builtin_ia32_rcpsh_mask
    {x86_sse_rcp_ss, 131901}, // __builtin_ia32_rcpss
    {x86_rdfsbase_32, 131907}, // __builtin_ia32_rdfsbase32
    {x86_rdfsbase_64, 131918}, // __builtin_ia32_rdfsbase64
    {x86_rdgsbase_32, 131929}, // __builtin_ia32_rdgsbase32
    {x86_rdgsbase_64, 131940}, // __builtin_ia32_rdgsbase64
    {x86_rdpid, 131951}, // __builtin_ia32_rdpid
    {x86_rdpkru, 131957}, // __builtin_ia32_rdpkru
    {x86_rdpmc, 131964}, // __builtin_ia32_rdpmc
    {x86_rdpru, 131970}, // __builtin_ia32_rdpru
    {x86_rdsspd, 131976}, // __builtin_ia32_rdsspd
    {x86_rdsspq, 131983}, // __builtin_ia32_rdsspq
    {x86_rdtsc, 131990}, // __builtin_ia32_rdtsc
    {x86_flags_read_u32, 131996}, // __builtin_ia32_readeflags_u32
    {x86_flags_read_u64, 132011}, // __builtin_ia32_readeflags_u64
    {x86_avx512_mask_reduce_pd_128, 132026}, // __builtin_ia32_reducepd128_mask
    {x86_avx512_mask_reduce_pd_256, 132043}, // __builtin_ia32_reducepd256_mask
    {x86_avx512_mask_reduce_pd_512, 132060}, // __builtin_ia32_reducepd512_mask
    {x86_avx512fp16_mask_reduce_ph_128, 132077}, // __builtin_ia32_reduceph128_mask
    {x86_avx512fp16_mask_reduce_ph_256, 132094}, // __builtin_ia32_reduceph256_mask
    {x86_avx512fp16_mask_reduce_ph_512, 132111}, // __builtin_ia32_reduceph512_mask
    {x86_avx512_mask_reduce_ps_128, 132128}, // __builtin_ia32_reduceps128_mask
    {x86_avx512_mask_reduce_ps_256, 132145}, // __builtin_ia32_reduceps256_mask
    {x86_avx512_mask_reduce_ps_512, 132162}, // __builtin_ia32_reduceps512_mask
    {x86_avx512_mask_reduce_sd, 132179}, // __builtin_ia32_reducesd_mask
    {x86_avx512fp16_mask_reduce_sh, 132193}, // __builtin_ia32_reducesh_mask
    {x86_avx512_mask_reduce_ss, 132207}, // __builtin_ia32_reducess_mask
    {x86_avx512_mask_rndscale_pd_128, 132221}, // __builtin_ia32_rndscalepd_128_mask
    {x86_avx512_mask_rndscale_pd_256, 132241}, // __builtin_ia32_rndscalepd_256_mask
    {x86_avx512_mask_rndscale_pd_512, 132261}, // __builtin_ia32_rndscalepd_mask
    {x86_avx512fp16_mask_rndscale_ph_128, 132277}, // __builtin_ia32_rndscaleph_128_mask
    {x86_avx512fp16_mask_rndscale_ph_256, 132297}, // __builtin_ia32_rndscaleph_256_mask
    {x86_avx512fp16_mask_rndscale_ph_512, 132317}, // __builtin_ia32_rndscaleph_mask
    {x86_avx512_mask_rndscale_ps_128, 132333}, // __builtin_ia32_rndscaleps_128_mask
    {x86_avx512_mask_rndscale_ps_256, 132353}, // __builtin_ia32_rndscaleps_256_mask
    {x86_avx512_mask_rndscale_ps_512, 132373}, // __builtin_ia32_rndscaleps_mask
    {x86_avx512_mask_rndscale_sd, 132389}, // __builtin_ia32_rndscalesd_round_mask
    {x86_avx512fp16_mask_rndscale_sh, 132411}, // __builtin_ia32_rndscalesh_round_mask
    {x86_avx512_mask_rndscale_ss, 132433}, // __builtin_ia32_rndscaless_round_mask
    {x86_avx512_rsqrt14_pd_128, 132455}, // __builtin_ia32_rsqrt14pd128_mask
    {x86_avx512_rsqrt14_pd_256, 132473}, // __builtin_ia32_rsqrt14pd256_mask
    {x86_avx512_rsqrt14_pd_512, 132491}, // __builtin_ia32_rsqrt14pd512_mask
    {x86_avx512_rsqrt14_ps_128, 132509}, // __builtin_ia32_rsqrt14ps128_mask
    {x86_avx512_rsqrt14_ps_256, 132527}, // __builtin_ia32_rsqrt14ps256_mask
    {x86_avx512_rsqrt14_ps_512, 132545}, // __builtin_ia32_rsqrt14ps512_mask
    {x86_avx512_rsqrt14_sd, 132563}, // __builtin_ia32_rsqrt14sd_mask
    {x86_avx512_rsqrt14_ss, 132578}, // __builtin_ia32_rsqrt14ss_mask
    {x86_avx512fp16_mask_rsqrt_ph_128, 132593}, // __builtin_ia32_rsqrtph128_mask
    {x86_avx512fp16_mask_rsqrt_ph_256, 132609}, // __builtin_ia32_rsqrtph256_mask
    {x86_avx512fp16_mask_rsqrt_ph_512, 132625}, // __builtin_ia32_rsqrtph512_mask
    {x86_sse_rsqrt_ps, 132641}, // __builtin_ia32_rsqrtps
    {x86_avx_rsqrt_ps_256, 132649}, // __builtin_ia32_rsqrtps256
    {x86_avx512fp16_mask_rsqrt_sh, 132660}, // __builtin_ia32_rsqrtsh_mask
    {x86_sse_rsqrt_ss, 132673}, // __builtin_ia32_rsqrtss
    {x86_rstorssp, 132681}, // __builtin_ia32_rstorssp
    {x86_saveprevssp, 132690}, // __builtin_ia32_saveprevssp
    {x86_avx512_mask_scalef_pd_128, 132702}, // __builtin_ia32_scalefpd128_mask
    {x86_avx512_mask_scalef_pd_256, 132719}, // __builtin_ia32_scalefpd256_mask
    {x86_avx512_mask_scalef_pd_512, 132736}, // __builtin_ia32_scalefpd512_mask
    {x86_avx512fp16_mask_scalef_ph_128, 132753}, // __builtin_ia32_scalefph128_mask
    {x86_avx512fp16_mask_scalef_ph_256, 132770}, // __builtin_ia32_scalefph256_mask
    {x86_avx512fp16_mask_scalef_ph_512, 132787}, // __builtin_ia32_scalefph512_mask
    {x86_avx512_mask_scalef_ps_128, 132804}, // __builtin_ia32_scalefps128_mask
    {x86_avx512_mask_scalef_ps_256, 132821}, // __builtin_ia32_scalefps256_mask
    {x86_avx512_mask_scalef_ps_512, 132838}, // __builtin_ia32_scalefps512_mask
    {x86_avx512_mask_scalef_sd, 132855}, // __builtin_ia32_scalefsd_round_mask
    {x86_avx512fp16_mask_scalef_sh, 132875}, // __builtin_ia32_scalefsh_round_mask
    {x86_avx512_mask_scalef_ss, 132895}, // __builtin_ia32_scalefss_round_mask
    {x86_senduipi, 132915}, // __builtin_ia32_senduipi
    {x86_serialize, 132924}, // __builtin_ia32_serialize
    {x86_setssbsy, 132934}, // __builtin_ia32_setssbsy
    {x86_sse_sfence, 132943}, // __builtin_ia32_sfence
    {x86_sha1msg1, 132950}, // __builtin_ia32_sha1msg1
    {x86_sha1msg2, 132959}, // __builtin_ia32_sha1msg2
    {x86_sha1nexte, 132968}, // __builtin_ia32_sha1nexte
    {x86_sha1rnds4, 132978}, // __builtin_ia32_sha1rnds4
    {x86_sha256msg1, 132988}, // __builtin_ia32_sha256msg1
    {x86_sha256msg2, 132999}, // __builtin_ia32_sha256msg2
    {x86_sha256rnds2, 133010}, // __builtin_ia32_sha256rnds2
    {x86_slwpcb, 133022}, // __builtin_ia32_slwpcb
    {x86_stui, 133029}, // __builtin_ia32_stui
    {x86_avx512_sub_pd_512, 133034}, // __builtin_ia32_subpd512
    {x86_avx512fp16_sub_ph_512, 133043}, // __builtin_ia32_subph512
    {x86_avx512_sub_ps_512, 133052}, // __builtin_ia32_subps512
    {x86_avx512_mask_sub_sd_round, 133061}, // __builtin_ia32_subsd_round_mask
    {x86_avx512fp16_mask_sub_sh_round, 133078}, // __builtin_ia32_subsh_round_mask
    {x86_avx512_mask_sub_ss_round, 133095}, // __builtin_ia32_subss_round_mask
    {x86_tcmmimfp16ps, 133112}, // __builtin_ia32_tcmmimfp16ps
    {x86_tcmmimfp16ps_internal, 133125}, // __builtin_ia32_tcmmimfp16ps_internal
    {x86_tcmmrlfp16ps, 133147}, // __builtin_ia32_tcmmrlfp16ps
    {x86_tcmmrlfp16ps_internal, 133160}, // __builtin_ia32_tcmmrlfp16ps_internal
    {x86_tcvtrowd2ps, 133182}, // __builtin_ia32_tcvtrowd2ps
    {x86_tcvtrowd2ps_internal, 133194}, // __builtin_ia32_tcvtrowd2ps_internal
    {x86_tcvtrowd2psi, 133215}, // __builtin_ia32_tcvtrowd2psi
    {x86_tcvtrowps2bf16h, 133228}, // __builtin_ia32_tcvtrowps2bf16h
    {x86_tcvtrowps2bf16h_internal, 133244}, // __builtin_ia32_tcvtrowps2bf16h_internal
    {x86_tcvtrowps2bf16hi, 133269}, // __builtin_ia32_tcvtrowps2bf16hi
    {x86_tcvtrowps2bf16l, 133286}, // __builtin_ia32_tcvtrowps2bf16l
    {x86_tcvtrowps2bf16l_internal, 133302}, // __builtin_ia32_tcvtrowps2bf16l_internal
    {x86_tcvtrowps2bf16li, 133327}, // __builtin_ia32_tcvtrowps2bf16li
    {x86_tcvtrowps2phh, 133344}, // __builtin_ia32_tcvtrowps2phh
    {x86_tcvtrowps2phh_internal, 133358}, // __builtin_ia32_tcvtrowps2phh_internal
    {x86_tcvtrowps2phhi, 133381}, // __builtin_ia32_tcvtrowps2phhi
    {x86_tcvtrowps2phl, 133396}, // __builtin_ia32_tcvtrowps2phl
    {x86_tcvtrowps2phl_internal, 133410}, // __builtin_ia32_tcvtrowps2phl_internal
    {x86_tcvtrowps2phli, 133433}, // __builtin_ia32_tcvtrowps2phli
    {x86_tdpbf16ps, 133448}, // __builtin_ia32_tdpbf16ps
    {x86_tdpbf16ps_internal, 133458}, // __builtin_ia32_tdpbf16ps_internal
    {x86_tdpbf8ps, 133477}, // __builtin_ia32_tdpbf8ps
    {x86_tdpbf8ps_internal, 133486}, // __builtin_ia32_tdpbf8ps_internal
    {x86_tdpbhf8ps, 133504}, // __builtin_ia32_tdpbhf8ps
    {x86_tdpbhf8ps_internal, 133514}, // __builtin_ia32_tdpbhf8ps_internal
    {x86_tdpbssd, 133533}, // __builtin_ia32_tdpbssd
    {x86_tdpbssd_internal, 133541}, // __builtin_ia32_tdpbssd_internal
    {x86_tdpbsud, 133558}, // __builtin_ia32_tdpbsud
    {x86_tdpbsud_internal, 133566}, // __builtin_ia32_tdpbsud_internal
    {x86_tdpbusd, 133583}, // __builtin_ia32_tdpbusd
    {x86_tdpbusd_internal, 133591}, // __builtin_ia32_tdpbusd_internal
    {x86_tdpbuud, 133608}, // __builtin_ia32_tdpbuud
    {x86_tdpbuud_internal, 133616}, // __builtin_ia32_tdpbuud_internal
    {x86_tdpfp16ps, 133633}, // __builtin_ia32_tdpfp16ps
    {x86_tdpfp16ps_internal, 133643}, // __builtin_ia32_tdpfp16ps_internal
    {x86_tdphbf8ps, 133662}, // __builtin_ia32_tdphbf8ps
    {x86_tdphbf8ps_internal, 133672}, // __builtin_ia32_tdphbf8ps_internal
    {x86_tdphf8ps, 133691}, // __builtin_ia32_tdphf8ps
    {x86_tdphf8ps_internal, 133700}, // __builtin_ia32_tdphf8ps_internal
    {x86_testui, 133718}, // __builtin_ia32_testui
    {x86_ldtilecfg, 133725}, // __builtin_ia32_tile_loadconfig
    {x86_ldtilecfg_internal, 133741}, // __builtin_ia32_tile_loadconfig_internal
    {x86_sttilecfg, 133766}, // __builtin_ia32_tile_storeconfig
    {x86_tileloadd64, 133783}, // __builtin_ia32_tileloadd64
    {x86_tileloadd64_internal, 133795}, // __builtin_ia32_tileloadd64_internal
    {x86_tileloaddrs64, 133816}, // __builtin_ia32_tileloaddrs64
    {x86_tileloaddrs64_internal, 133830}, // __builtin_ia32_tileloaddrs64_internal
    {x86_tileloaddrst164, 133853}, // __builtin_ia32_tileloaddrst164
    {x86_tileloaddrst164_internal, 133869}, // __builtin_ia32_tileloaddrst164_internal
    {x86_tileloaddt164, 133894}, // __builtin_ia32_tileloaddt164
    {x86_tileloaddt164_internal, 133908}, // __builtin_ia32_tileloaddt164_internal
    {x86_tilemovrow, 133931}, // __builtin_ia32_tilemovrow
    {x86_tilemovrow_internal, 133942}, // __builtin_ia32_tilemovrow_internal
    {x86_tilemovrowi, 133962}, // __builtin_ia32_tilemovrowi
    {x86_tilerelease, 133974}, // __builtin_ia32_tilerelease
    {x86_tilestored64, 133986}, // __builtin_ia32_tilestored64
    {x86_tilestored64_internal, 133999}, // __builtin_ia32_tilestored64_internal
    {x86_tilezero, 134021}, // __builtin_ia32_tilezero
    {x86_tilezero_internal, 134030}, // __builtin_ia32_tilezero_internal
    {x86_tmmultf32ps, 134048}, // __builtin_ia32_tmmultf32ps
    {x86_tmmultf32ps_internal, 134060}, // __builtin_ia32_tmmultf32ps_internal
    {x86_tpause, 134081}, // __builtin_ia32_tpause
    {x86_sse_ucomieq_ss, 134088}, // __builtin_ia32_ucomieq
    {x86_sse_ucomige_ss, 134096}, // __builtin_ia32_ucomige
    {x86_sse_ucomigt_ss, 134104}, // __builtin_ia32_ucomigt
    {x86_sse_ucomile_ss, 134112}, // __builtin_ia32_ucomile
    {x86_sse_ucomilt_ss, 134120}, // __builtin_ia32_ucomilt
    {x86_sse_ucomineq_ss, 134128}, // __builtin_ia32_ucomineq
    {x86_sse2_ucomieq_sd, 134137}, // __builtin_ia32_ucomisdeq
    {x86_sse2_ucomige_sd, 134147}, // __builtin_ia32_ucomisdge
    {x86_sse2_ucomigt_sd, 134157}, // __builtin_ia32_ucomisdgt
    {x86_sse2_ucomile_sd, 134167}, // __builtin_ia32_ucomisdle
    {x86_sse2_ucomilt_sd, 134177}, // __builtin_ia32_ucomisdlt
    {x86_sse2_ucomineq_sd, 134187}, // __builtin_ia32_ucomisdneq
    {x86_umonitor, 134198}, // __builtin_ia32_umonitor
    {x86_umwait, 134207}, // __builtin_ia32_umwait
    {x86_urdmsr, 134214}, // __builtin_ia32_urdmsr
    {x86_uwrmsr, 134221}, // __builtin_ia32_uwrmsr
    {x86_avx10_vaddbf16128, 134228}, // __builtin_ia32_vaddbf16128
    {x86_avx10_vaddbf16256, 134240}, // __builtin_ia32_vaddbf16256
    {x86_avx10_vaddbf16512, 134252}, // __builtin_ia32_vaddbf16512
    {x86_avx10_vaddpd256, 134264}, // __builtin_ia32_vaddpd256_round
    {x86_avx10_vaddph256, 134280}, // __builtin_ia32_vaddph256_round
    {x86_avx10_vaddps256, 134296}, // __builtin_ia32_vaddps256_round
    {x86_vbcstnebf162ps128, 134312}, // __builtin_ia32_vbcstnebf162ps128
    {x86_vbcstnebf162ps256, 134330}, // __builtin_ia32_vbcstnebf162ps256
    {x86_vbcstnesh2ps128, 134348}, // __builtin_ia32_vbcstnesh2ps128
    {x86_vbcstnesh2ps256, 134364}, // __builtin_ia32_vbcstnesh2ps256
    {x86_avx10_vcomisbf16eq, 134380}, // __builtin_ia32_vcomisbf16eq
    {x86_avx10_vcomisbf16ge, 134393}, // __builtin_ia32_vcomisbf16ge
    {x86_avx10_vcomisbf16gt, 134406}, // __builtin_ia32_vcomisbf16gt
    {x86_avx10_vcomisbf16le, 134419}, // __builtin_ia32_vcomisbf16le
    {x86_avx10_vcomisbf16lt, 134432}, // __builtin_ia32_vcomisbf16lt
    {x86_avx10_vcomisbf16neq, 134445}, // __builtin_ia32_vcomisbf16neq
    {x86_avx512_vcomi_sd, 134459}, // __builtin_ia32_vcomisd
    {x86_avx512fp16_vcomi_sh, 134467}, // __builtin_ia32_vcomish
    {x86_avx512_vcomi_ss, 134475}, // __builtin_ia32_vcomiss
    {x86_avx10_vcvt2ph2bf8128, 134483}, // __builtin_ia32_vcvt2ph2bf8_128
    {x86_avx10_vcvt2ph2bf8256, 134499}, // __builtin_ia32_vcvt2ph2bf8_256
    {x86_avx10_vcvt2ph2bf8512, 134515}, // __builtin_ia32_vcvt2ph2bf8_512
    {x86_avx10_vcvt2ph2bf8s128, 134531}, // __builtin_ia32_vcvt2ph2bf8s_128
    {x86_avx10_vcvt2ph2bf8s256, 134548}, // __builtin_ia32_vcvt2ph2bf8s_256
    {x86_avx10_vcvt2ph2bf8s512, 134565}, // __builtin_ia32_vcvt2ph2bf8s_512
    {x86_avx10_vcvt2ph2hf8128, 134582}, // __builtin_ia32_vcvt2ph2hf8_128
    {x86_avx10_vcvt2ph2hf8256, 134598}, // __builtin_ia32_vcvt2ph2hf8_256
    {x86_avx10_vcvt2ph2hf8512, 134614}, // __builtin_ia32_vcvt2ph2hf8_512
    {x86_avx10_vcvt2ph2hf8s128, 134630}, // __builtin_ia32_vcvt2ph2hf8s_128
    {x86_avx10_vcvt2ph2hf8s256, 134647}, // __builtin_ia32_vcvt2ph2hf8s_256
    {x86_avx10_vcvt2ph2hf8s512, 134664}, // __builtin_ia32_vcvt2ph2hf8s_512
    {x86_avx10_mask_vcvt2ps2phx_128, 134681}, // __builtin_ia32_vcvt2ps2phx128_mask
    {x86_avx10_mask_vcvt2ps2phx_256, 134701}, // __builtin_ia32_vcvt2ps2phx256_mask
    {x86_avx10_mask_vcvt2ps2phx_512, 134721}, // __builtin_ia32_vcvt2ps2phx512_mask
    {x86_avx10_vcvtbf162ibs128, 134741}, // __builtin_ia32_vcvtbf162ibs128
    {x86_avx10_vcvtbf162ibs256, 134757}, // __builtin_ia32_vcvtbf162ibs256
    {x86_avx10_vcvtbf162ibs512, 134773}, // __builtin_ia32_vcvtbf162ibs512
    {x86_avx10_vcvtbf162iubs128, 134789}, // __builtin_ia32_vcvtbf162iubs128
    {x86_avx10_vcvtbf162iubs256, 134806}, // __builtin_ia32_vcvtbf162iubs256
    {x86_avx10_vcvtbf162iubs512, 134823}, // __builtin_ia32_vcvtbf162iubs512
    {x86_avx10_mask_vcvtbiasph2bf8128, 134840}, // __builtin_ia32_vcvtbiasph2bf8_128_mask
    {x86_avx10_mask_vcvtbiasph2bf8256, 134864}, // __builtin_ia32_vcvtbiasph2bf8_256_mask
    {x86_avx10_mask_vcvtbiasph2bf8512, 134888}, // __builtin_ia32_vcvtbiasph2bf8_512_mask
    {x86_avx10_mask_vcvtbiasph2bf8s128, 134912}, // __builtin_ia32_vcvtbiasph2bf8s_128_mask
    {x86_avx10_mask_vcvtbiasph2bf8s256, 134937}, // __builtin_ia32_vcvtbiasph2bf8s_256_mask
    {x86_avx10_mask_vcvtbiasph2bf8s512, 134962}, // __builtin_ia32_vcvtbiasph2bf8s_512_mask
    {x86_avx10_mask_vcvtbiasph2hf8128, 134987}, // __builtin_ia32_vcvtbiasph2hf8_128_mask
    {x86_avx10_mask_vcvtbiasph2hf8256, 135011}, // __builtin_ia32_vcvtbiasph2hf8_256_mask
    {x86_avx10_mask_vcvtbiasph2hf8512, 135035}, // __builtin_ia32_vcvtbiasph2hf8_512_mask
    {x86_avx10_mask_vcvtbiasph2hf8s128, 135059}, // __builtin_ia32_vcvtbiasph2hf8s_128_mask
    {x86_avx10_mask_vcvtbiasph2hf8s256, 135084}, // __builtin_ia32_vcvtbiasph2hf8s_256_mask
    {x86_avx10_mask_vcvtbiasph2hf8s512, 135109}, // __builtin_ia32_vcvtbiasph2hf8s_512_mask
    {x86_avx512fp16_mask_vcvtdq2ph_128, 135134}, // __builtin_ia32_vcvtdq2ph128_mask
    {x86_avx10_mask_vcvthf82ph128, 135152}, // __builtin_ia32_vcvthf8_2ph128_mask
    {x86_avx10_mask_vcvthf82ph256, 135172}, // __builtin_ia32_vcvthf8_2ph256_mask
    {x86_avx10_mask_vcvthf82ph512, 135192}, // __builtin_ia32_vcvthf8_2ph512_mask
    {x86_vcvtneebf162ps128, 135212}, // __builtin_ia32_vcvtneebf162ps128
    {x86_vcvtneebf162ps256, 135230}, // __builtin_ia32_vcvtneebf162ps256
    {x86_vcvtneeph2ps128, 135248}, // __builtin_ia32_vcvtneeph2ps128
    {x86_vcvtneeph2ps256, 135264}, // __builtin_ia32_vcvtneeph2ps256
    {x86_vcvtneobf162ps128, 135280}, // __builtin_ia32_vcvtneobf162ps128
    {x86_vcvtneobf162ps256, 135298}, // __builtin_ia32_vcvtneobf162ps256
    {x86_vcvtneoph2ps128, 135316}, // __builtin_ia32_vcvtneoph2ps128
    {x86_vcvtneoph2ps256, 135332}, // __builtin_ia32_vcvtneoph2ps256
    {x86_vcvtneps2bf16128, 135348}, // __builtin_ia32_vcvtneps2bf16128
    {x86_vcvtneps2bf16256, 135365}, // __builtin_ia32_vcvtneps2bf16256
    {x86_avx512fp16_mask_vcvtpd2ph_128, 135382}, // __builtin_ia32_vcvtpd2ph128_mask
    {x86_avx512fp16_mask_vcvtpd2ph_256, 135400}, // __builtin_ia32_vcvtpd2ph256_mask
    {x86_avx512fp16_mask_vcvtpd2ph_512, 135418}, // __builtin_ia32_vcvtpd2ph512_mask
    {x86_avx10_mask_vcvtph2bf8128, 135436}, // __builtin_ia32_vcvtph2bf8_128_mask
    {x86_avx10_mask_vcvtph2bf8256, 135456}, // __builtin_ia32_vcvtph2bf8_256_mask
    {x86_avx10_mask_vcvtph2bf8512, 135476}, // __builtin_ia32_vcvtph2bf8_512_mask
    {x86_avx10_mask_vcvtph2bf8s128, 135496}, // __builtin_ia32_vcvtph2bf8s_128_mask
    {x86_avx10_mask_vcvtph2bf8s256, 135517}, // __builtin_ia32_vcvtph2bf8s_256_mask
    {x86_avx10_mask_vcvtph2bf8s512, 135538}, // __builtin_ia32_vcvtph2bf8s_512_mask
    {x86_avx512fp16_mask_vcvtph2dq_128, 135559}, // __builtin_ia32_vcvtph2dq128_mask
    {x86_avx512fp16_mask_vcvtph2dq_256, 135577}, // __builtin_ia32_vcvtph2dq256_mask
    {x86_avx512fp16_mask_vcvtph2dq_512, 135595}, // __builtin_ia32_vcvtph2dq512_mask
    {x86_avx10_mask_vcvtph2hf8128, 135613}, // __builtin_ia32_vcvtph2hf8_128_mask
    {x86_avx10_mask_vcvtph2hf8256, 135633}, // __builtin_ia32_vcvtph2hf8_256_mask
    {x86_avx10_mask_vcvtph2hf8512, 135653}, // __builtin_ia32_vcvtph2hf8_512_mask
    {x86_avx10_mask_vcvtph2hf8s128, 135673}, // __builtin_ia32_vcvtph2hf8s_128_mask
    {x86_avx10_mask_vcvtph2hf8s256, 135694}, // __builtin_ia32_vcvtph2hf8s_256_mask
    {x86_avx10_mask_vcvtph2hf8s512, 135715}, // __builtin_ia32_vcvtph2hf8s_512_mask
    {x86_avx10_mask_vcvtph2ibs128, 135736}, // __builtin_ia32_vcvtph2ibs128_mask
    {x86_avx10_mask_vcvtph2ibs256, 135755}, // __builtin_ia32_vcvtph2ibs256_mask
    {x86_avx10_mask_vcvtph2ibs512, 135774}, // __builtin_ia32_vcvtph2ibs512_mask
    {x86_avx10_mask_vcvtph2iubs128, 135793}, // __builtin_ia32_vcvtph2iubs128_mask
    {x86_avx10_mask_vcvtph2iubs256, 135813}, // __builtin_ia32_vcvtph2iubs256_mask
    {x86_avx10_mask_vcvtph2iubs512, 135833}, // __builtin_ia32_vcvtph2iubs512_mask
    {x86_avx512fp16_mask_vcvtph2pd_128, 135853}, // __builtin_ia32_vcvtph2pd128_mask
    {x86_avx512fp16_mask_vcvtph2pd_256, 135871}, // __builtin_ia32_vcvtph2pd256_mask
    {x86_avx512fp16_mask_vcvtph2pd_512, 135889}, // __builtin_ia32_vcvtph2pd512_mask
    {x86_avx512fp16_mask_vcvtph2psx_128, 135907}, // __builtin_ia32_vcvtph2psx128_mask
    {x86_avx512fp16_mask_vcvtph2psx_256, 135926}, // __builtin_ia32_vcvtph2psx256_mask
    {x86_avx512fp16_mask_vcvtph2psx_512, 135945}, // __builtin_ia32_vcvtph2psx512_mask
    {x86_avx512fp16_mask_vcvtph2qq_128, 135964}, // __builtin_ia32_vcvtph2qq128_mask
    {x86_avx512fp16_mask_vcvtph2qq_256, 135982}, // __builtin_ia32_vcvtph2qq256_mask
    {x86_avx512fp16_mask_vcvtph2qq_512, 136000}, // __builtin_ia32_vcvtph2qq512_mask
    {x86_avx512fp16_mask_vcvtph2udq_128, 136018}, // __builtin_ia32_vcvtph2udq128_mask
    {x86_avx512fp16_mask_vcvtph2udq_256, 136037}, // __builtin_ia32_vcvtph2udq256_mask
    {x86_avx512fp16_mask_vcvtph2udq_512, 136056}, // __builtin_ia32_vcvtph2udq512_mask
    {x86_avx512fp16_mask_vcvtph2uqq_128, 136075}, // __builtin_ia32_vcvtph2uqq128_mask
    {x86_avx512fp16_mask_vcvtph2uqq_256, 136094}, // __builtin_ia32_vcvtph2uqq256_mask
    {x86_avx512fp16_mask_vcvtph2uqq_512, 136113}, // __builtin_ia32_vcvtph2uqq512_mask
    {x86_avx512fp16_mask_vcvtph2uw_128, 136132}, // __builtin_ia32_vcvtph2uw128_mask
    {x86_avx512fp16_mask_vcvtph2uw_256, 136150}, // __builtin_ia32_vcvtph2uw256_mask
    {x86_avx512fp16_mask_vcvtph2uw_512, 136168}, // __builtin_ia32_vcvtph2uw512_mask
    {x86_avx512fp16_mask_vcvtph2w_128, 136186}, // __builtin_ia32_vcvtph2w128_mask
    {x86_avx512fp16_mask_vcvtph2w_256, 136203}, // __builtin_ia32_vcvtph2w256_mask
    {x86_avx512fp16_mask_vcvtph2w_512, 136220}, // __builtin_ia32_vcvtph2w512_mask
    {x86_avx10_mask_vcvtps2ibs128, 136237}, // __builtin_ia32_vcvtps2ibs128_mask
    {x86_avx10_mask_vcvtps2ibs256, 136256}, // __builtin_ia32_vcvtps2ibs256_mask
    {x86_avx10_mask_vcvtps2ibs512, 136275}, // __builtin_ia32_vcvtps2ibs512_mask
    {x86_avx10_mask_vcvtps2iubs128, 136294}, // __builtin_ia32_vcvtps2iubs128_mask
    {x86_avx10_mask_vcvtps2iubs256, 136314}, // __builtin_ia32_vcvtps2iubs256_mask
    {x86_avx10_mask_vcvtps2iubs512, 136334}, // __builtin_ia32_vcvtps2iubs512_mask
    {x86_vcvtps2ph_128, 136354}, // __builtin_ia32_vcvtps2ph
    {x86_vcvtps2ph_256, 136364}, // __builtin_ia32_vcvtps2ph256
    {x86_avx512_mask_vcvtps2ph_256, 136377}, // __builtin_ia32_vcvtps2ph256_mask
    {x86_avx512_mask_vcvtps2ph_512, 136395}, // __builtin_ia32_vcvtps2ph512_mask
    {x86_avx512_mask_vcvtps2ph_128, 136413}, // __builtin_ia32_vcvtps2ph_mask
    {x86_avx512fp16_mask_vcvtps2phx_128, 136428}, // __builtin_ia32_vcvtps2phx128_mask
    {x86_avx512fp16_mask_vcvtps2phx_256, 136447}, // __builtin_ia32_vcvtps2phx256_mask
    {x86_avx512fp16_mask_vcvtps2phx_512, 136466}, // __builtin_ia32_vcvtps2phx512_mask
    {x86_avx512fp16_mask_vcvtqq2ph_128, 136485}, // __builtin_ia32_vcvtqq2ph128_mask
    {x86_avx512fp16_mask_vcvtqq2ph_256, 136503}, // __builtin_ia32_vcvtqq2ph256_mask
    {x86_avx512fp16_mask_vcvtsd2sh_round, 136521}, // __builtin_ia32_vcvtsd2sh_round_mask
    {x86_avx512_vcvtsd2si32, 136542}, // __builtin_ia32_vcvtsd2si32
    {x86_avx512_vcvtsd2si64, 136554}, // __builtin_ia32_vcvtsd2si64
    {x86_avx512_vcvtsd2usi32, 136566}, // __builtin_ia32_vcvtsd2usi32
    {x86_avx512_vcvtsd2usi64, 136579}, // __builtin_ia32_vcvtsd2usi64
    {x86_avx512fp16_mask_vcvtsh2sd_round, 136592}, // __builtin_ia32_vcvtsh2sd_round_mask
    {x86_avx512fp16_vcvtsh2si32, 136613}, // __builtin_ia32_vcvtsh2si32
    {x86_avx512fp16_vcvtsh2si64, 136625}, // __builtin_ia32_vcvtsh2si64
    {x86_avx512fp16_mask_vcvtsh2ss_round, 136637}, // __builtin_ia32_vcvtsh2ss_round_mask
    {x86_avx512fp16_vcvtsh2usi32, 136658}, // __builtin_ia32_vcvtsh2usi32
    {x86_avx512fp16_vcvtsh2usi64, 136671}, // __builtin_ia32_vcvtsh2usi64
    {x86_avx512fp16_vcvtsi2sh, 136684}, // __builtin_ia32_vcvtsi2sh
    {x86_avx512fp16_vcvtsi642sh, 136694}, // __builtin_ia32_vcvtsi642sh
    {x86_avx512fp16_mask_vcvtss2sh_round, 136706}, // __builtin_ia32_vcvtss2sh_round_mask
    {x86_avx512_vcvtss2si32, 136727}, // __builtin_ia32_vcvtss2si32
    {x86_avx512_vcvtss2si64, 136739}, // __builtin_ia32_vcvtss2si64
    {x86_avx512_vcvtss2usi32, 136751}, // __builtin_ia32_vcvtss2usi32
    {x86_avx512_vcvtss2usi64, 136764}, // __builtin_ia32_vcvtss2usi64
    {x86_avx10_vcvttbf162ibs128, 136777}, // __builtin_ia32_vcvttbf162ibs128
    {x86_avx10_vcvttbf162ibs256, 136794}, // __builtin_ia32_vcvttbf162ibs256
    {x86_avx10_vcvttbf162ibs512, 136811}, // __builtin_ia32_vcvttbf162ibs512
    {x86_avx10_vcvttbf162iubs128, 136828}, // __builtin_ia32_vcvttbf162iubs128
    {x86_avx10_vcvttbf162iubs256, 136846}, // __builtin_ia32_vcvttbf162iubs256
    {x86_avx10_vcvttbf162iubs512, 136864}, // __builtin_ia32_vcvttbf162iubs512
    {x86_avx10_mask_vcvttpd2dqs_128, 136882}, // __builtin_ia32_vcvttpd2dqs128_mask
    {x86_avx10_mask_vcvttpd2dqs_256, 136902}, // __builtin_ia32_vcvttpd2dqs256_mask
    {x86_avx10_mask_vcvttpd2dqs_round_512, 136922}, // __builtin_ia32_vcvttpd2dqs512_round_mask
    {x86_avx10_mask_vcvttpd2qqs_128, 136948}, // __builtin_ia32_vcvttpd2qqs128_mask
    {x86_avx10_mask_vcvttpd2qqs_256, 136968}, // __builtin_ia32_vcvttpd2qqs256_mask
    {x86_avx10_mask_vcvttpd2qqs_round_512, 136988}, // __builtin_ia32_vcvttpd2qqs512_round_mask
    {x86_avx10_mask_vcvttpd2udqs_128, 137014}, // __builtin_ia32_vcvttpd2udqs128_mask
    {x86_avx10_mask_vcvttpd2udqs_256, 137035}, // __builtin_ia32_vcvttpd2udqs256_mask
    {x86_avx10_mask_vcvttpd2udqs_round_512, 137056}, // __builtin_ia32_vcvttpd2udqs512_round_mask
    {x86_avx10_mask_vcvttpd2uqqs_128, 137083}, // __builtin_ia32_vcvttpd2uqqs128_mask
    {x86_avx10_mask_vcvttpd2uqqs_256, 137104}, // __builtin_ia32_vcvttpd2uqqs256_mask
    {x86_avx10_mask_vcvttpd2uqqs_round_512, 137125}, // __builtin_ia32_vcvttpd2uqqs512_round_mask
    {x86_avx512fp16_mask_vcvttph2dq_128, 137152}, // __builtin_ia32_vcvttph2dq128_mask
    {x86_avx512fp16_mask_vcvttph2dq_256, 137171}, // __builtin_ia32_vcvttph2dq256_mask
    {x86_avx512fp16_mask_vcvttph2dq_512, 137190}, // __builtin_ia32_vcvttph2dq512_mask
    {x86_avx10_mask_vcvttph2ibs128, 137209}, // __builtin_ia32_vcvttph2ibs128_mask
    {x86_avx10_mask_vcvttph2ibs256, 137229}, // __builtin_ia32_vcvttph2ibs256_mask
    {x86_avx10_mask_vcvttph2ibs512, 137249}, // __builtin_ia32_vcvttph2ibs512_mask
    {x86_avx10_mask_vcvttph2iubs128, 137269}, // __builtin_ia32_vcvttph2iubs128_mask
    {x86_avx10_mask_vcvttph2iubs256, 137290}, // __builtin_ia32_vcvttph2iubs256_mask
    {x86_avx10_mask_vcvttph2iubs512, 137311}, // __builtin_ia32_vcvttph2iubs512_mask
    {x86_avx512fp16_mask_vcvttph2qq_128, 137332}, // __builtin_ia32_vcvttph2qq128_mask
    {x86_avx512fp16_mask_vcvttph2qq_256, 137351}, // __builtin_ia32_vcvttph2qq256_mask
    {x86_avx512fp16_mask_vcvttph2qq_512, 137370}, // __builtin_ia32_vcvttph2qq512_mask
    {x86_avx512fp16_mask_vcvttph2udq_128, 137389}, // __builtin_ia32_vcvttph2udq128_mask
    {x86_avx512fp16_mask_vcvttph2udq_256, 137409}, // __builtin_ia32_vcvttph2udq256_mask
    {x86_avx512fp16_mask_vcvttph2udq_512, 137429}, // __builtin_ia32_vcvttph2udq512_mask
    {x86_avx512fp16_mask_vcvttph2uqq_128, 137449}, // __builtin_ia32_vcvttph2uqq128_mask
    {x86_avx512fp16_mask_vcvttph2uqq_256, 137469}, // __builtin_ia32_vcvttph2uqq256_mask
    {x86_avx512fp16_mask_vcvttph2uqq_512, 137489}, // __builtin_ia32_vcvttph2uqq512_mask
    {x86_avx512fp16_mask_vcvttph2uw_128, 137509}, // __builtin_ia32_vcvttph2uw128_mask
    {x86_avx512fp16_mask_vcvttph2uw_256, 137528}, // __builtin_ia32_vcvttph2uw256_mask
    {x86_avx512fp16_mask_vcvttph2uw_512, 137547}, // __builtin_ia32_vcvttph2uw512_mask
    {x86_avx512fp16_mask_vcvttph2w_128, 137566}, // __builtin_ia32_vcvttph2w128_mask
    {x86_avx512fp16_mask_vcvttph2w_256, 137584}, // __builtin_ia32_vcvttph2w256_mask
    {x86_avx512fp16_mask_vcvttph2w_512, 137602}, // __builtin_ia32_vcvttph2w512_mask
    {x86_avx10_mask_vcvttps2dqs_128, 137620}, // __builtin_ia32_vcvttps2dqs128_mask
    {x86_avx10_mask_vcvttps2dqs_256, 137640}, // __builtin_ia32_vcvttps2dqs256_mask
    {x86_avx10_mask_vcvttps2dqs_round_512, 137660}, // __builtin_ia32_vcvttps2dqs512_round_mask
    {x86_avx10_mask_vcvttps2ibs128, 137686}, // __builtin_ia32_vcvttps2ibs128_mask
    {x86_avx10_mask_vcvttps2ibs256, 137706}, // __builtin_ia32_vcvttps2ibs256_mask
    {x86_avx10_mask_vcvttps2ibs512, 137726}, // __builtin_ia32_vcvttps2ibs512_mask
    {x86_avx10_mask_vcvttps2iubs128, 137746}, // __builtin_ia32_vcvttps2iubs128_mask
    {x86_avx10_mask_vcvttps2iubs256, 137767}, // __builtin_ia32_vcvttps2iubs256_mask
    {x86_avx10_mask_vcvttps2iubs512, 137788}, // __builtin_ia32_vcvttps2iubs512_mask
    {x86_avx10_mask_vcvttps2qqs_128, 137809}, // __builtin_ia32_vcvttps2qqs128_mask
    {x86_avx10_mask_vcvttps2qqs_256, 137829}, // __builtin_ia32_vcvttps2qqs256_mask
    {x86_avx10_mask_vcvttps2qqs_round_512, 137849}, // __builtin_ia32_vcvttps2qqs512_round_mask
    {x86_avx10_mask_vcvttps2udqs_128, 137875}, // __builtin_ia32_vcvttps2udqs128_mask
    {x86_avx10_mask_vcvttps2udqs_256, 137896}, // __builtin_ia32_vcvttps2udqs256_mask
    {x86_avx10_mask_vcvttps2udqs_round_512, 137917}, // __builtin_ia32_vcvttps2udqs512_round_mask
    {x86_avx10_mask_vcvttps2uqqs_128, 137944}, // __builtin_ia32_vcvttps2uqqs128_mask
    {x86_avx10_mask_vcvttps2uqqs_256, 137965}, // __builtin_ia32_vcvttps2uqqs256_mask
    {x86_avx10_mask_vcvttps2uqqs_round_512, 137986}, // __builtin_ia32_vcvttps2uqqs512_round_mask
    {x86_avx512_cvttsd2si, 138013}, // __builtin_ia32_vcvttsd2si32
    {x86_avx512_cvttsd2si64, 138026}, // __builtin_ia32_vcvttsd2si64
    {x86_avx10_vcvttsd2sis, 138039}, // __builtin_ia32_vcvttsd2sis32
    {x86_avx10_vcvttsd2sis64, 138053}, // __builtin_ia32_vcvttsd2sis64
    {x86_avx512_cvttsd2usi, 138067}, // __builtin_ia32_vcvttsd2usi32
    {x86_avx512_cvttsd2usi64, 138081}, // __builtin_ia32_vcvttsd2usi64
    {x86_avx10_vcvttsd2usis, 138095}, // __builtin_ia32_vcvttsd2usis32
    {x86_avx10_vcvttsd2usis64, 138110}, // __builtin_ia32_vcvttsd2usis64
    {x86_avx512fp16_vcvttsh2si32, 138125}, // __builtin_ia32_vcvttsh2si32
    {x86_avx512fp16_vcvttsh2si64, 138138}, // __builtin_ia32_vcvttsh2si64
    {x86_avx512fp16_vcvttsh2usi32, 138151}, // __builtin_ia32_vcvttsh2usi32
    {x86_avx512fp16_vcvttsh2usi64, 138165}, // __builtin_ia32_vcvttsh2usi64
    {x86_avx512_cvttss2si, 138179}, // __builtin_ia32_vcvttss2si32
    {x86_avx512_cvttss2si64, 138192}, // __builtin_ia32_vcvttss2si64
    {x86_avx10_vcvttss2sis, 138205}, // __builtin_ia32_vcvttss2sis32
    {x86_avx10_vcvttss2sis64, 138219}, // __builtin_ia32_vcvttss2sis64
    {x86_avx512_cvttss2usi, 138233}, // __builtin_ia32_vcvttss2usi32
    {x86_avx512_cvttss2usi64, 138247}, // __builtin_ia32_vcvttss2usi64
    {x86_avx10_vcvttss2usis, 138261}, // __builtin_ia32_vcvttss2usis32
    {x86_avx10_vcvttss2usis64, 138276}, // __builtin_ia32_vcvttss2usis64
    {x86_avx512fp16_mask_vcvtudq2ph_128, 138291}, // __builtin_ia32_vcvtudq2ph128_mask
    {x86_avx512fp16_mask_vcvtuqq2ph_128, 138310}, // __builtin_ia32_vcvtuqq2ph128_mask
    {x86_avx512fp16_mask_vcvtuqq2ph_256, 138329}, // __builtin_ia32_vcvtuqq2ph256_mask
    {x86_avx512fp16_vcvtusi2sh, 138348}, // __builtin_ia32_vcvtusi2sh
    {x86_avx512fp16_vcvtusi642sh, 138359}, // __builtin_ia32_vcvtusi642sh
    {x86_avx10_vdivbf16128, 138372}, // __builtin_ia32_vdivbf16128
    {x86_avx10_vdivbf16256, 138384}, // __builtin_ia32_vdivbf16256
    {x86_avx10_vdivbf16512, 138396}, // __builtin_ia32_vdivbf16512
    {x86_avx10_vdpphps_128, 138408}, // __builtin_ia32_vdpphps128
    {x86_avx10_vdpphps_256, 138419}, // __builtin_ia32_vdpphps256
    {x86_avx10_vdpphps_512, 138430}, // __builtin_ia32_vdpphps512
    {x86_avx512fp16_mask_vfcmadd_cph_128, 138441}, // __builtin_ia32_vfcmaddcph128_mask
    {x86_avx512fp16_maskz_vfcmadd_cph_128, 138460}, // __builtin_ia32_vfcmaddcph128_maskz
    {x86_avx512fp16_mask_vfcmadd_cph_256, 138480}, // __builtin_ia32_vfcmaddcph256_mask
    {x86_avx512fp16_maskz_vfcmadd_cph_256, 138499}, // __builtin_ia32_vfcmaddcph256_maskz
    {x86_avx512fp16_mask_vfcmadd_cph_512, 138519}, // __builtin_ia32_vfcmaddcph512_mask3
    {x86_avx512fp16_maskz_vfcmadd_cph_512, 138539}, // __builtin_ia32_vfcmaddcph512_maskz
    {x86_avx512fp16_mask_vfcmadd_csh, 138559}, // __builtin_ia32_vfcmaddcsh_mask
    {x86_avx512fp16_maskz_vfcmadd_csh, 138575}, // __builtin_ia32_vfcmaddcsh_maskz
    {x86_avx512fp16_mask_vfcmul_cph_128, 138592}, // __builtin_ia32_vfcmulcph128_mask
    {x86_avx512fp16_mask_vfcmul_cph_256, 138610}, // __builtin_ia32_vfcmulcph256_mask
    {x86_avx512fp16_mask_vfcmul_cph_512, 138628}, // __builtin_ia32_vfcmulcph512_mask
    {x86_avx512fp16_mask_vfcmul_csh, 138646}, // __builtin_ia32_vfcmulcsh_mask
    {x86_avx10_vfmadd132bf16128, 138661}, // __builtin_ia32_vfmadd132bf16128
    {x86_avx10_vfmadd132bf16256, 138678}, // __builtin_ia32_vfmadd132bf16256
    {x86_avx10_vfmadd132bf16512, 138695}, // __builtin_ia32_vfmadd132bf16512
    {x86_avx10_vfmadd213bf16128, 138712}, // __builtin_ia32_vfmadd213bf16128
    {x86_avx10_vfmadd213bf16256, 138729}, // __builtin_ia32_vfmadd213bf16256
    {x86_avx10_vfmadd231bf16128, 138746}, // __builtin_ia32_vfmadd231bf16128
    {x86_avx10_vfmadd231bf16256, 138763}, // __builtin_ia32_vfmadd231bf16256
    {x86_avx10_vfmadd231bf16512, 138780}, // __builtin_ia32_vfmadd231bf16512
    {x86_avx512fp16_mask_vfmadd_cph_128, 138797}, // __builtin_ia32_vfmaddcph128_mask
    {x86_avx512fp16_maskz_vfmadd_cph_128, 138815}, // __builtin_ia32_vfmaddcph128_maskz
    {x86_avx512fp16_mask_vfmadd_cph_256, 138834}, // __builtin_ia32_vfmaddcph256_mask
    {x86_avx512fp16_maskz_vfmadd_cph_256, 138852}, // __builtin_ia32_vfmaddcph256_maskz
    {x86_avx512fp16_mask_vfmadd_cph_512, 138871}, // __builtin_ia32_vfmaddcph512_mask3
    {x86_avx512fp16_maskz_vfmadd_cph_512, 138890}, // __builtin_ia32_vfmaddcph512_maskz
    {x86_avx512fp16_mask_vfmadd_csh, 138909}, // __builtin_ia32_vfmaddcsh_mask
    {x86_avx512fp16_maskz_vfmadd_csh, 138924}, // __builtin_ia32_vfmaddcsh_maskz
    {x86_fma_vfmaddsub_pd, 138940}, // __builtin_ia32_vfmaddsubpd
    {x86_fma_vfmaddsub_pd_256, 138952}, // __builtin_ia32_vfmaddsubpd256
    {x86_avx512fp16_vfmaddsub_ph_128, 138967}, // __builtin_ia32_vfmaddsubph
    {x86_avx512fp16_vfmaddsub_ph_256, 138979}, // __builtin_ia32_vfmaddsubph256
    {x86_fma_vfmaddsub_ps, 138994}, // __builtin_ia32_vfmaddsubps
    {x86_fma_vfmaddsub_ps_256, 139006}, // __builtin_ia32_vfmaddsubps256
    {x86_avx10_vfmsub132bf16128, 139021}, // __builtin_ia32_vfmsub132bf16128
    {x86_avx10_vfmsub132bf16256, 139038}, // __builtin_ia32_vfmsub132bf16256
    {x86_avx10_vfmsub132bf16512, 139055}, // __builtin_ia32_vfmsub132bf16512
    {x86_avx10_vfmsub213bf16128, 139072}, // __builtin_ia32_vfmsub213bf16128
    {x86_avx10_vfmsub213bf16256, 139089}, // __builtin_ia32_vfmsub213bf16256
    {x86_avx10_vfmsub213bf16512, 139106}, // __builtin_ia32_vfmsub213bf16512
    {x86_avx10_vfmsub231bf16128, 139123}, // __builtin_ia32_vfmsub231bf16128
    {x86_avx10_vfmsub231bf16256, 139140}, // __builtin_ia32_vfmsub231bf16256
    {x86_avx10_vfmsub231bf16512, 139157}, // __builtin_ia32_vfmsub231bf16512
    {x86_avx512fp16_mask_vfmul_cph_128, 139174}, // __builtin_ia32_vfmulcph128_mask
    {x86_avx512fp16_mask_vfmul_cph_256, 139191}, // __builtin_ia32_vfmulcph256_mask
    {x86_avx512fp16_mask_vfmul_cph_512, 139208}, // __builtin_ia32_vfmulcph512_mask
    {x86_avx512fp16_mask_vfmul_csh, 139225}, // __builtin_ia32_vfmulcsh_mask
    {x86_avx10_vfnmadd132bf16128, 139239}, // __builtin_ia32_vfnmadd132bf16128
    {x86_avx10_vfnmadd132bf16256, 139257}, // __builtin_ia32_vfnmadd132bf16256
    {x86_avx10_vfnmadd132bf16512, 139275}, // __builtin_ia32_vfnmadd132bf16512
    {x86_avx10_vfnmadd213bf16128, 139293}, // __builtin_ia32_vfnmadd213bf16128
    {x86_avx10_vfnmadd213bf16256, 139311}, // __builtin_ia32_vfnmadd213bf16256
    {x86_avx10_vfnmadd213bf16512, 139329}, // __builtin_ia32_vfnmadd213bf16512
    {x86_avx10_vfnmadd231bf16128, 139347}, // __builtin_ia32_vfnmadd231bf16128
    {x86_avx10_vfnmadd231bf16256, 139365}, // __builtin_ia32_vfnmadd231bf16256
    {x86_avx10_vfnmadd231bf16512, 139383}, // __builtin_ia32_vfnmadd231bf16512
    {x86_avx10_vfnmsub132bf16128, 139401}, // __builtin_ia32_vfnmsub132bf16128
    {x86_avx10_vfnmsub132bf16256, 139419}, // __builtin_ia32_vfnmsub132bf16256
    {x86_avx10_vfnmsub132bf16512, 139437}, // __builtin_ia32_vfnmsub132bf16512
    {x86_avx10_vfnmsub213bf16128, 139455}, // __builtin_ia32_vfnmsub213bf16128
    {x86_avx10_vfnmsub213bf16256, 139473}, // __builtin_ia32_vfnmsub213bf16256
    {x86_avx10_vfnmsub213bf16512, 139491}, // __builtin_ia32_vfnmsub213bf16512
    {x86_avx10_vfnmsub231bf16128, 139509}, // __builtin_ia32_vfnmsub231bf16128
    {x86_avx10_vfnmsub231bf16256, 139527}, // __builtin_ia32_vfnmsub231bf16256
    {x86_avx10_vfnmsub231bf16512, 139545}, // __builtin_ia32_vfnmsub231bf16512
    {x86_xop_vfrcz_pd, 139563}, // __builtin_ia32_vfrczpd
    {x86_xop_vfrcz_pd_256, 139571}, // __builtin_ia32_vfrczpd256
    {x86_xop_vfrcz_ps, 139582}, // __builtin_ia32_vfrczps
    {x86_xop_vfrcz_ps_256, 139590}, // __builtin_ia32_vfrczps256
    {x86_xop_vfrcz_sd, 139601}, // __builtin_ia32_vfrczsd
    {x86_xop_vfrcz_ss, 139609}, // __builtin_ia32_vfrczss
    {x86_avx10_mask_getexp_bf16_128, 139617}, // __builtin_ia32_vgetexpbf16128_mask
    {x86_avx10_mask_getexp_bf16_256, 139637}, // __builtin_ia32_vgetexpbf16256_mask
    {x86_avx10_mask_getexp_bf16_512, 139657}, // __builtin_ia32_vgetexpbf16512_mask
    {x86_avx10_mask_getmant_bf16_128, 139677}, // __builtin_ia32_vgetmantbf16128_mask
    {x86_avx10_mask_getmant_bf16_256, 139698}, // __builtin_ia32_vgetmantbf16256_mask
    {x86_avx10_mask_getmant_bf16_512, 139719}, // __builtin_ia32_vgetmantbf16512_mask
    {x86_vgf2p8affineinvqb_128, 139740}, // __builtin_ia32_vgf2p8affineinvqb_v16qi
    {x86_vgf2p8affineinvqb_256, 139764}, // __builtin_ia32_vgf2p8affineinvqb_v32qi
    {x86_vgf2p8affineinvqb_512, 139788}, // __builtin_ia32_vgf2p8affineinvqb_v64qi
    {x86_vgf2p8affineqb_128, 139812}, // __builtin_ia32_vgf2p8affineqb_v16qi
    {x86_vgf2p8affineqb_256, 139833}, // __builtin_ia32_vgf2p8affineqb_v32qi
    {x86_vgf2p8affineqb_512, 139854}, // __builtin_ia32_vgf2p8affineqb_v64qi
    {x86_vgf2p8mulb_128, 139875}, // __builtin_ia32_vgf2p8mulb_v16qi
    {x86_vgf2p8mulb_256, 139892}, // __builtin_ia32_vgf2p8mulb_v32qi
    {x86_vgf2p8mulb_512, 139909}, // __builtin_ia32_vgf2p8mulb_v64qi
    {x86_avx10_vmaxbf16128, 139926}, // __builtin_ia32_vmaxbf16128
    {x86_avx10_vmaxbf16256, 139938}, // __builtin_ia32_vmaxbf16256
    {x86_avx10_vmaxbf16512, 139950}, // __builtin_ia32_vmaxbf16512
    {x86_avx10_vminbf16128, 139962}, // __builtin_ia32_vminbf16128
    {x86_avx10_vminbf16256, 139974}, // __builtin_ia32_vminbf16256
    {x86_avx10_vminbf16512, 139986}, // __builtin_ia32_vminbf16512
    {x86_avx10_vminmaxbf16128, 139998}, // __builtin_ia32_vminmaxbf16128
    {x86_avx10_vminmaxbf16256, 140013}, // __builtin_ia32_vminmaxbf16256
    {x86_avx10_vminmaxbf16512, 140028}, // __builtin_ia32_vminmaxbf16512
    {x86_avx10_vminmaxpd128, 140043}, // __builtin_ia32_vminmaxpd128
    {x86_avx10_mask_vminmaxpd128, 140056}, // __builtin_ia32_vminmaxpd128_mask
    {x86_avx10_vminmaxpd256, 140074}, // __builtin_ia32_vminmaxpd256
    {x86_avx10_mask_vminmaxpd256, 140087}, // __builtin_ia32_vminmaxpd256_mask
    {x86_avx10_mask_vminmaxpd_round, 140105}, // __builtin_ia32_vminmaxpd512_round_mask
    {x86_avx10_vminmaxph128, 140129}, // __builtin_ia32_vminmaxph128
    {x86_avx10_mask_vminmaxph128, 140142}, // __builtin_ia32_vminmaxph128_mask
    {x86_avx10_vminmaxph256, 140160}, // __builtin_ia32_vminmaxph256
    {x86_avx10_mask_vminmaxph256, 140173}, // __builtin_ia32_vminmaxph256_mask
    {x86_avx10_mask_vminmaxph_round, 140191}, // __builtin_ia32_vminmaxph512_round_mask
    {x86_avx10_vminmaxps128, 140215}, // __builtin_ia32_vminmaxps128
    {x86_avx10_mask_vminmaxps128, 140228}, // __builtin_ia32_vminmaxps128_mask
    {x86_avx10_vminmaxps256, 140246}, // __builtin_ia32_vminmaxps256
    {x86_avx10_mask_vminmaxps256, 140259}, // __builtin_ia32_vminmaxps256_mask
    {x86_avx10_mask_vminmaxps_round, 140277}, // __builtin_ia32_vminmaxps512_round_mask
    {x86_avx10_mask_vminmaxsd_round, 140301}, // __builtin_ia32_vminmaxsd_round_mask
    {x86_avx10_mask_vminmaxsh_round, 140322}, // __builtin_ia32_vminmaxsh_round_mask
    {x86_avx10_mask_vminmaxss_round, 140343}, // __builtin_ia32_vminmaxss_round_mask
    {x86_avx10_vmovrsb128, 140364}, // __builtin_ia32_vmovrsb128
    {x86_avx10_vmovrsb256, 140375}, // __builtin_ia32_vmovrsb256
    {x86_avx10_vmovrsb512, 140386}, // __builtin_ia32_vmovrsb512
    {x86_avx10_vmovrsd128, 140397}, // __builtin_ia32_vmovrsd128
    {x86_avx10_vmovrsd256, 140408}, // __builtin_ia32_vmovrsd256
    {x86_avx10_vmovrsd512, 140419}, // __builtin_ia32_vmovrsd512
    {x86_avx10_vmovrsq128, 140430}, // __builtin_ia32_vmovrsq128
    {x86_avx10_vmovrsq256, 140441}, // __builtin_ia32_vmovrsq256
    {x86_avx10_vmovrsq512, 140452}, // __builtin_ia32_vmovrsq512
    {x86_avx10_vmovrsw128, 140463}, // __builtin_ia32_vmovrsw128
    {x86_avx10_vmovrsw256, 140474}, // __builtin_ia32_vmovrsw256
    {x86_avx10_vmovrsw512, 140485}, // __builtin_ia32_vmovrsw512
    {x86_avx10_vmulbf16128, 140496}, // __builtin_ia32_vmulbf16128
    {x86_avx10_vmulbf16256, 140508}, // __builtin_ia32_vmulbf16256
    {x86_avx10_vmulbf16512, 140520}, // __builtin_ia32_vmulbf16512
    {x86_avx512_conflict_q_128, 140532}, // __builtin_ia32_vpconflictdi_128
    {x86_avx512_conflict_q_256, 140549}, // __builtin_ia32_vpconflictdi_256
    {x86_avx512_conflict_q_512, 140566}, // __builtin_ia32_vpconflictdi_512
    {x86_avx512_conflict_d_128, 140583}, // __builtin_ia32_vpconflictsi_128
    {x86_avx512_conflict_d_256, 140600}, // __builtin_ia32_vpconflictsi_256
    {x86_avx512_conflict_d_512, 140617}, // __builtin_ia32_vpconflictsi_512
    {x86_avx2_vpdpbssd_128, 140634}, // __builtin_ia32_vpdpbssd128
    {x86_avx2_vpdpbssd_256, 140646}, // __builtin_ia32_vpdpbssd256
    {x86_avx10_vpdpbssd_512, 140658}, // __builtin_ia32_vpdpbssd512
    {x86_avx2_vpdpbssds_128, 140670}, // __builtin_ia32_vpdpbssds128
    {x86_avx2_vpdpbssds_256, 140683}, // __builtin_ia32_vpdpbssds256
    {x86_avx10_vpdpbssds_512, 140696}, // __builtin_ia32_vpdpbssds512
    {x86_avx2_vpdpbsud_128, 140709}, // __builtin_ia32_vpdpbsud128
    {x86_avx2_vpdpbsud_256, 140721}, // __builtin_ia32_vpdpbsud256
    {x86_avx10_vpdpbsud_512, 140733}, // __builtin_ia32_vpdpbsud512
    {x86_avx2_vpdpbsuds_128, 140745}, // __builtin_ia32_vpdpbsuds128
    {x86_avx2_vpdpbsuds_256, 140758}, // __builtin_ia32_vpdpbsuds256
    {x86_avx10_vpdpbsuds_512, 140771}, // __builtin_ia32_vpdpbsuds512
    {x86_avx512_vpdpbusd_128, 140784}, // __builtin_ia32_vpdpbusd128
    {x86_avx512_vpdpbusd_256, 140796}, // __builtin_ia32_vpdpbusd256
    {x86_avx512_vpdpbusd_512, 140808}, // __builtin_ia32_vpdpbusd512
    {x86_avx512_vpdpbusds_128, 140820}, // __builtin_ia32_vpdpbusds128
    {x86_avx512_vpdpbusds_256, 140833}, // __builtin_ia32_vpdpbusds256
    {x86_avx512_vpdpbusds_512, 140846}, // __builtin_ia32_vpdpbusds512
    {x86_avx2_vpdpbuud_128, 140859}, // __builtin_ia32_vpdpbuud128
    {x86_avx2_vpdpbuud_256, 140871}, // __builtin_ia32_vpdpbuud256
    {x86_avx10_vpdpbuud_512, 140883}, // __builtin_ia32_vpdpbuud512
    {x86_avx2_vpdpbuuds_128, 140895}, // __builtin_ia32_vpdpbuuds128
    {x86_avx2_vpdpbuuds_256, 140908}, // __builtin_ia32_vpdpbuuds256
    {x86_avx10_vpdpbuuds_512, 140921}, // __builtin_ia32_vpdpbuuds512
    {x86_avx512_vpdpwssd_128, 140934}, // __builtin_ia32_vpdpwssd128
    {x86_avx512_vpdpwssd_256, 140946}, // __builtin_ia32_vpdpwssd256
    {x86_avx512_vpdpwssd_512, 140958}, // __builtin_ia32_vpdpwssd512
    {x86_avx512_vpdpwssds_128, 140970}, // __builtin_ia32_vpdpwssds128
    {x86_avx512_vpdpwssds_256, 140983}, // __builtin_ia32_vpdpwssds256
    {x86_avx512_vpdpwssds_512, 140996}, // __builtin_ia32_vpdpwssds512
    {x86_avx2_vpdpwsud_128, 141009}, // __builtin_ia32_vpdpwsud128
    {x86_avx2_vpdpwsud_256, 141021}, // __builtin_ia32_vpdpwsud256
    {x86_avx10_vpdpwsud_512, 141033}, // __builtin_ia32_vpdpwsud512
    {x86_avx2_vpdpwsuds_128, 141045}, // __builtin_ia32_vpdpwsuds128
    {x86_avx2_vpdpwsuds_256, 141058}, // __builtin_ia32_vpdpwsuds256
    {x86_avx10_vpdpwsuds_512, 141071}, // __builtin_ia32_vpdpwsuds512
    {x86_avx2_vpdpwusd_128, 141084}, // __builtin_ia32_vpdpwusd128
    {x86_avx2_vpdpwusd_256, 141096}, // __builtin_ia32_vpdpwusd256
    {x86_avx10_vpdpwusd_512, 141108}, // __builtin_ia32_vpdpwusd512
    {x86_avx2_vpdpwusds_128, 141120}, // __builtin_ia32_vpdpwusds128
    {x86_avx2_vpdpwusds_256, 141133}, // __builtin_ia32_vpdpwusds256
    {x86_avx10_vpdpwusds_512, 141146}, // __builtin_ia32_vpdpwusds512
    {x86_avx2_vpdpwuud_128, 141159}, // __builtin_ia32_vpdpwuud128
    {x86_avx2_vpdpwuud_256, 141171}, // __builtin_ia32_vpdpwuud256
    {x86_avx10_vpdpwuud_512, 141183}, // __builtin_ia32_vpdpwuud512
    {x86_avx2_vpdpwuuds_128, 141195}, // __builtin_ia32_vpdpwuuds128
    {x86_avx2_vpdpwuuds_256, 141208}, // __builtin_ia32_vpdpwuuds256
    {x86_avx10_vpdpwuuds_512, 141221}, // __builtin_ia32_vpdpwuuds512
    {x86_avx512_vpermi2var_d_128, 141234}, // __builtin_ia32_vpermi2vard128
    {x86_avx512_vpermi2var_d_256, 141249}, // __builtin_ia32_vpermi2vard256
    {x86_avx512_vpermi2var_d_512, 141264}, // __builtin_ia32_vpermi2vard512
    {x86_avx512_vpermi2var_hi_128, 141279}, // __builtin_ia32_vpermi2varhi128
    {x86_avx512_vpermi2var_hi_256, 141295}, // __builtin_ia32_vpermi2varhi256
    {x86_avx512_vpermi2var_hi_512, 141311}, // __builtin_ia32_vpermi2varhi512
    {x86_avx512_vpermi2var_pd_128, 141327}, // __builtin_ia32_vpermi2varpd128
    {x86_avx512_vpermi2var_pd_256, 141343}, // __builtin_ia32_vpermi2varpd256
    {x86_avx512_vpermi2var_pd_512, 141359}, // __builtin_ia32_vpermi2varpd512
    {x86_avx512_vpermi2var_ps_128, 141375}, // __builtin_ia32_vpermi2varps128
    {x86_avx512_vpermi2var_ps_256, 141391}, // __builtin_ia32_vpermi2varps256
    {x86_avx512_vpermi2var_ps_512, 141407}, // __builtin_ia32_vpermi2varps512
    {x86_avx512_vpermi2var_q_128, 141423}, // __builtin_ia32_vpermi2varq128
    {x86_avx512_vpermi2var_q_256, 141438}, // __builtin_ia32_vpermi2varq256
    {x86_avx512_vpermi2var_q_512, 141453}, // __builtin_ia32_vpermi2varq512
    {x86_avx512_vpermi2var_qi_128, 141468}, // __builtin_ia32_vpermi2varqi128
    {x86_avx512_vpermi2var_qi_256, 141484}, // __builtin_ia32_vpermi2varqi256
    {x86_avx512_vpermi2var_qi_512, 141500}, // __builtin_ia32_vpermi2varqi512
    {x86_xop_vpermil2pd, 141516}, // __builtin_ia32_vpermil2pd
    {x86_xop_vpermil2pd_256, 141527}, // __builtin_ia32_vpermil2pd256
    {x86_xop_vpermil2ps, 141541}, // __builtin_ia32_vpermil2ps
    {x86_xop_vpermil2ps_256, 141552}, // __builtin_ia32_vpermil2ps256
    {x86_avx_vpermilvar_pd, 141566}, // __builtin_ia32_vpermilvarpd
    {x86_avx_vpermilvar_pd_256, 141579}, // __builtin_ia32_vpermilvarpd256
    {x86_avx512_vpermilvar_pd_512, 141595}, // __builtin_ia32_vpermilvarpd512
    {x86_avx_vpermilvar_ps, 141611}, // __builtin_ia32_vpermilvarps
    {x86_avx_vpermilvar_ps_256, 141624}, // __builtin_ia32_vpermilvarps256
    {x86_avx512_vpermilvar_ps_512, 141640}, // __builtin_ia32_vpermilvarps512
    {x86_xop_vphaddbd, 141656}, // __builtin_ia32_vphaddbd
    {x86_xop_vphaddbq, 141665}, // __builtin_ia32_vphaddbq
    {x86_xop_vphaddbw, 141674}, // __builtin_ia32_vphaddbw
    {x86_xop_vphadddq, 141683}, // __builtin_ia32_vphadddq
    {x86_xop_vphaddubd, 141692}, // __builtin_ia32_vphaddubd
    {x86_xop_vphaddubq, 141702}, // __builtin_ia32_vphaddubq
    {x86_xop_vphaddubw, 141712}, // __builtin_ia32_vphaddubw
    {x86_xop_vphaddudq, 141722}, // __builtin_ia32_vphaddudq
    {x86_xop_vphadduwd, 141732}, // __builtin_ia32_vphadduwd
    {x86_xop_vphadduwq, 141742}, // __builtin_ia32_vphadduwq
    {x86_xop_vphaddwd, 141752}, // __builtin_ia32_vphaddwd
    {x86_xop_vphaddwq, 141761}, // __builtin_ia32_vphaddwq
    {x86_xop_vphsubbw, 141770}, // __builtin_ia32_vphsubbw
    {x86_xop_vphsubdq, 141779}, // __builtin_ia32_vphsubdq
    {x86_xop_vphsubwd, 141788}, // __builtin_ia32_vphsubwd
    {x86_xop_vpmacsdd, 141797}, // __builtin_ia32_vpmacsdd
    {x86_xop_vpmacsdqh, 141806}, // __builtin_ia32_vpmacsdqh
    {x86_xop_vpmacsdql, 141816}, // __builtin_ia32_vpmacsdql
    {x86_xop_vpmacssdd, 141826}, // __builtin_ia32_vpmacssdd
    {x86_xop_vpmacssdqh, 141836}, // __builtin_ia32_vpmacssdqh
    {x86_xop_vpmacssdql, 141847}, // __builtin_ia32_vpmacssdql
    {x86_xop_vpmacsswd, 141858}, // __builtin_ia32_vpmacsswd
    {x86_xop_vpmacssww, 141868}, // __builtin_ia32_vpmacssww
    {x86_xop_vpmacswd, 141878}, // __builtin_ia32_vpmacswd
    {x86_xop_vpmacsww, 141887}, // __builtin_ia32_vpmacsww
    {x86_xop_vpmadcsswd, 141896}, // __builtin_ia32_vpmadcsswd
    {x86_xop_vpmadcswd, 141907}, // __builtin_ia32_vpmadcswd
    {x86_avx512_vpmadd52h_uq_128, 141917}, // __builtin_ia32_vpmadd52huq128
    {x86_avx512_vpmadd52h_uq_256, 141932}, // __builtin_ia32_vpmadd52huq256
    {x86_avx512_vpmadd52h_uq_512, 141947}, // __builtin_ia32_vpmadd52huq512
    {x86_avx512_vpmadd52l_uq_128, 141962}, // __builtin_ia32_vpmadd52luq128
    {x86_avx512_vpmadd52l_uq_256, 141977}, // __builtin_ia32_vpmadd52luq256
    {x86_avx512_vpmadd52l_uq_512, 141992}, // __builtin_ia32_vpmadd52luq512
    {x86_avx512_pmultishift_qb_128, 142007}, // __builtin_ia32_vpmultishiftqb128
    {x86_avx512_pmultishift_qb_256, 142025}, // __builtin_ia32_vpmultishiftqb256
    {x86_avx512_pmultishift_qb_512, 142043}, // __builtin_ia32_vpmultishiftqb512
    {x86_xop_vpperm, 142061}, // __builtin_ia32_vpperm
    {x86_xop_vpshab, 142068}, // __builtin_ia32_vpshab
    {x86_xop_vpshad, 142075}, // __builtin_ia32_vpshad
    {x86_xop_vpshaq, 142082}, // __builtin_ia32_vpshaq
    {x86_xop_vpshaw, 142089}, // __builtin_ia32_vpshaw
    {x86_xop_vpshlb, 142096}, // __builtin_ia32_vpshlb
    {x86_xop_vpshld, 142103}, // __builtin_ia32_vpshld
    {x86_xop_vpshlq, 142110}, // __builtin_ia32_vpshlq
    {x86_xop_vpshlw, 142117}, // __builtin_ia32_vpshlw
    {x86_avx10_mask_rcp_bf16_128, 142124}, // __builtin_ia32_vrcpbf16128_mask
    {x86_avx10_mask_rcp_bf16_256, 142141}, // __builtin_ia32_vrcpbf16256_mask
    {x86_avx10_mask_rcp_bf16_512, 142158}, // __builtin_ia32_vrcpbf16512_mask
    {x86_avx10_mask_reduce_bf16_128, 142175}, // __builtin_ia32_vreducebf16128_mask
    {x86_avx10_mask_reduce_bf16_256, 142195}, // __builtin_ia32_vreducebf16256_mask
    {x86_avx10_mask_reduce_bf16_512, 142215}, // __builtin_ia32_vreducebf16512_mask
    {x86_avx10_mask_rndscale_bf16_128, 142235}, // __builtin_ia32_vrndscalebf16_128_mask
    {x86_avx10_mask_rndscale_bf16_256, 142258}, // __builtin_ia32_vrndscalebf16_256_mask
    {x86_avx10_mask_rndscale_bf16_512, 142281}, // __builtin_ia32_vrndscalebf16_mask
    {x86_avx10_mask_rsqrt_bf16_128, 142300}, // __builtin_ia32_vrsqrtbf16128_mask
    {x86_avx10_mask_rsqrt_bf16_256, 142319}, // __builtin_ia32_vrsqrtbf16256_mask
    {x86_avx10_mask_rsqrt_bf16_512, 142338}, // __builtin_ia32_vrsqrtbf16512_mask
    {x86_avx10_mask_scalef_bf16_128, 142357}, // __builtin_ia32_vscalefbf16128_mask
    {x86_avx10_mask_scalef_bf16_256, 142377}, // __builtin_ia32_vscalefbf16256_mask
    {x86_avx10_mask_scalef_bf16_512, 142397}, // __builtin_ia32_vscalefbf16512_mask
    {x86_vsha512msg1, 142417}, // __builtin_ia32_vsha512msg1
    {x86_vsha512msg2, 142429}, // __builtin_ia32_vsha512msg2
    {x86_vsha512rnds2, 142441}, // __builtin_ia32_vsha512rnds2
    {x86_vsm3msg1, 142454}, // __builtin_ia32_vsm3msg1
    {x86_vsm3msg2, 142463}, // __builtin_ia32_vsm3msg2
    {x86_vsm3rnds2, 142472}, // __builtin_ia32_vsm3rnds2
    {x86_vsm4key4128, 142482}, // __builtin_ia32_vsm4key4128
    {x86_vsm4key4256, 142494}, // __builtin_ia32_vsm4key4256
    {x86_vsm4key4512, 142506}, // __builtin_ia32_vsm4key4512
    {x86_vsm4rnds4128, 142518}, // __builtin_ia32_vsm4rnds4128
    {x86_vsm4rnds4256, 142531}, // __builtin_ia32_vsm4rnds4256
    {x86_vsm4rnds4512, 142544}, // __builtin_ia32_vsm4rnds4512
    {x86_avx10_vsubbf16128, 142557}, // __builtin_ia32_vsubbf16128
    {x86_avx10_vsubbf16256, 142569}, // __builtin_ia32_vsubbf16256
    {x86_avx10_vsubbf16512, 142581}, // __builtin_ia32_vsubbf16512
    {x86_avx_vtestc_pd, 142593}, // __builtin_ia32_vtestcpd
    {x86_avx_vtestc_pd_256, 142602}, // __builtin_ia32_vtestcpd256
    {x86_avx_vtestc_ps, 142614}, // __builtin_ia32_vtestcps
    {x86_avx_vtestc_ps_256, 142623}, // __builtin_ia32_vtestcps256
    {x86_avx_vtestnzc_pd, 142635}, // __builtin_ia32_vtestnzcpd
    {x86_avx_vtestnzc_pd_256, 142646}, // __builtin_ia32_vtestnzcpd256
    {x86_avx_vtestnzc_ps, 142660}, // __builtin_ia32_vtestnzcps
    {x86_avx_vtestnzc_ps_256, 142671}, // __builtin_ia32_vtestnzcps256
    {x86_avx_vtestz_pd, 142685}, // __builtin_ia32_vtestzpd
    {x86_avx_vtestz_pd_256, 142694}, // __builtin_ia32_vtestzpd256
    {x86_avx_vtestz_ps, 142706}, // __builtin_ia32_vtestzps
    {x86_avx_vtestz_ps_256, 142715}, // __builtin_ia32_vtestzps256
    {x86_avx_vzeroall, 142727}, // __builtin_ia32_vzeroall
    {x86_avx_vzeroupper, 142736}, // __builtin_ia32_vzeroupper
    {x86_wbinvd, 142747}, // __builtin_ia32_wbinvd
    {x86_wbnoinvd, 142754}, // __builtin_ia32_wbnoinvd
    {x86_wrfsbase_32, 142763}, // __builtin_ia32_wrfsbase32
    {x86_wrfsbase_64, 142774}, // __builtin_ia32_wrfsbase64
    {x86_wrgsbase_32, 142785}, // __builtin_ia32_wrgsbase32
    {x86_wrgsbase_64, 142796}, // __builtin_ia32_wrgsbase64
    {x86_flags_write_u32, 142807}, // __builtin_ia32_writeeflags_u32
    {x86_flags_write_u64, 142823}, // __builtin_ia32_writeeflags_u64
    {x86_wrpkru, 142839}, // __builtin_ia32_wrpkru
    {x86_wrssd, 142846}, // __builtin_ia32_wrssd
    {x86_wrssq, 142852}, // __builtin_ia32_wrssq
    {x86_wrussd, 142858}, // __builtin_ia32_wrussd
    {x86_wrussq, 142865}, // __builtin_ia32_wrussq
    {x86_xabort, 142872}, // __builtin_ia32_xabort
    {x86_xbegin, 142879}, // __builtin_ia32_xbegin
    {x86_xend, 142886}, // __builtin_ia32_xend
    {x86_xresldtrk, 142891}, // __builtin_ia32_xresldtrk
    {x86_xsusldtrk, 142901}, // __builtin_ia32_xsusldtrk
    {x86_xtest, 142911}, // __builtin_ia32_xtest
  }; // x86Names

  // Builtins for xcore.
  static constexpr BuiltinEntry xcoreNames[] = {
    {xcore_bitrev, 142917}, // __builtin_bitrev
    {xcore_getid, 142924}, // __builtin_getid
    {xcore_getps, 142930}, // __builtin_getps
    {xcore_setps, 142936}, // __builtin_setps
  }; // xcoreNames


  struct TargetEntry {
    StringLiteral TargetPrefix;
    ArrayRef<BuiltinEntry> Names;
    StringLiteral CommonPrefix;
    bool operator<(StringRef RHS) const {
      return TargetPrefix < RHS;
    };
  };
  static constexpr TargetEntry TargetTable[] = {
    {"aarch64", aarch64Names, "__builtin_"},
    {"amdgcn", amdgcnNames, "__builtin_amdgcn_"},
    {"arm", armNames, "__builtin_arm_"},
    {"bpf", bpfNames, "__builtin_bpf_"},
    {"hexagon", hexagonNames, "__builtin_"},
    {"loongarch", loongarchNames, "__builtin_l"},
    {"mips", mipsNames, "__builtin_m"},
    {"nvvm", nvvmNames, "__nvvm_"},
    {"ppc", ppcNames, "__builtin_"},
    {"r600", r600Names, "__builtin_r600_"},
    {"riscv", riscvNames, "__builtin_riscv_"},
    {"s390", s390Names, "__builtin_"},
    {"spv", spvNames, "__builtin_spirv_"},
    {"ve", veNames, "__builtin_ve_vl_"},
    {"x86", x86Names, "__builtin_ia32_"},
    {"xcore", xcoreNames, "__builtin_"},
  };

  // Check if it's a target independent builtin.
  // Copy the builtin name so we can use it in consume_front without clobbering
  // if for the lookup in the target specific table.
  StringRef Suffix = BuiltinName;
  if (Suffix.consume_front("__builtin_")) {
    auto II = lower_bound(Names, Suffix);
    if (II != std::end(Names) && II->getName() == Suffix)
      return II->IntrinsicID;
  }

  auto TI = lower_bound(TargetTable, TargetPrefix);
  if (TI == std::end(TargetTable) || TI->TargetPrefix != TargetPrefix)
    return not_intrinsic;
  // This is the last use of BuiltinName, so no need to copy before using it in
  // consume_front.
  if (!BuiltinName.consume_front(TI->CommonPrefix))
    return not_intrinsic;
  auto II = lower_bound(TI->Names, BuiltinName);
  if (II == std::end(TI->Names) || II->getName() != BuiltinName)
    return not_intrinsic;
  return II->IntrinsicID;
}
#endif // GET_LLVM_INTRINSIC_FOR_CLANG_BUILTIN


// Get the LLVM intrinsic that corresponds to a builtin. This is used by the
// C front-end. The builtin name is passed in as BuiltinName, and a target
// prefix (e.g. 'ppc') is passed in as TargetPrefix.
#ifdef GET_LLVM_INTRINSIC_FOR_MS_BUILTIN
Intrinsic::ID
Intrinsic::getIntrinsicForMSBuiltin(StringRef TargetPrefix, 
                                      StringRef BuiltinName) {
  using namespace Intrinsic;

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Woverlength-strings"
#endif
static constexpr char BuiltinNamesStorage[] =
  "\0"
  "dmb\0"
  "dsb\0"
  "isb\0"
  "MoveFromCoprocessor\0"
  "MoveFromCoprocessor2\0"
  "_dmb\0"
  "_dsb\0"
  "_isb\0"
  ;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

static constexpr llvm::StringTable
BuiltinNames = BuiltinNamesStorage;

  struct BuiltinEntry {
    ID IntrinsicID;
    unsigned StrTabOffset;
    const char *getName() const { return BuiltinNames[StrTabOffset].data(); }
    bool operator<(StringRef RHS) const {
      return strncmp(getName(), RHS.data(), RHS.size()) < 0;
    }
  };

  // Builtins for aarch64.
  static constexpr BuiltinEntry aarch64Names[] = {
    {aarch64_dmb, 1}, // __dmb
    {aarch64_dsb, 5}, // __dsb
    {aarch64_isb, 9}, // __isb
  }; // aarch64Names

  // Builtins for arm.
  static constexpr BuiltinEntry armNames[] = {
    {arm_mrc, 13}, // _MoveFromCoprocessor
    {arm_mrc2, 33}, // _MoveFromCoprocessor2
    {arm_dmb, 54}, // __dmb
    {arm_dsb, 59}, // __dsb
    {arm_isb, 64}, // __isb
  }; // armNames


  struct TargetEntry {
    StringLiteral TargetPrefix;
    ArrayRef<BuiltinEntry> Names;
    StringLiteral CommonPrefix;
    bool operator<(StringRef RHS) const {
      return TargetPrefix < RHS;
    };
  };
  static constexpr TargetEntry TargetTable[] = {
    {"aarch64", aarch64Names, "__"},
    {"arm", armNames, "_"},
  };

  auto TI = lower_bound(TargetTable, TargetPrefix);
  if (TI == std::end(TargetTable) || TI->TargetPrefix != TargetPrefix)
    return not_intrinsic;
  // This is the last use of BuiltinName, so no need to copy before using it in
  // consume_front.
  if (!BuiltinName.consume_front(TI->CommonPrefix))
    return not_intrinsic;
  auto II = lower_bound(TI->Names, BuiltinName);
  if (II == std::end(TI->Names) || II->getName() != BuiltinName)
    return not_intrinsic;
  return II->IntrinsicID;
}
#endif // GET_LLVM_INTRINSIC_FOR_MS_BUILTIN

